!     path:      $Source: /storm/rc1/cvsroot/rc/rrtmg_lw/src/rrtmg_lw_k_g.f90,v $
!     author:    $Author: mike $
!     revision:  $Revision: 1.4 $
!     created:   $Date: 2009/05/22 21:04:30 $
!
!  --------------------------------------------------------------------------
! |                                                                          |
! |  Copyright 2002-2009, Atmospheric & Environmental Research, Inc. (AER).  |
! |  This software may be used, copied, or redistributed as long as it is    |
! |  not sold and this copyright notice is reproduced on each copy made.     |
! |  This model is provided as is without any express or implied warranties. |
! |                       (http://www.rtweb.aer.com/)                        |
! |                                                                          |
!  --------------------------------------------------------------------------

! **************************************************************************
!      subroutine lw_kgbnn
! **************************************************************************
!  RRTM Longwave Radiative Transfer Model
!  Atmospheric and Environmental Research, Inc., Cambridge, MA
!
!  Original version:   E. J. Mlawer, et al.
!  Revision for GCMs:  Michael J. Iacono; October, 2002
!  Revision for F90 formatting:  Michael J. Iacono; June 2006
!
!  This file contains 16 subroutines that include the 
!  absorption coefficients and other data for each of the 16 longwave
!  spectral bands used in RRTM.  Here, the data are defined for 16
!  g-points, or sub-intervals, per band.  These data are combined and
!  weighted using a mapping procedure in routine RRTMG_LW_INIT to reduce
!  the total number of g-points from 256 to 140 for use in the GCM.
! **************************************************************************
      subroutine lw_kgb01
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg01, only : fracrefao, fracrefbo, kao, kbo, kao_mn2, kbo_mn2, &
                            selfrefo, forrefo
      use rrlw_vsn, only: hvrkg, hnamkg

      implicit none
      save

      hvrkg = '$Revision: 1.4 $'

! Planck fraction mapping level: P = 212.7250 mbar, T = 223.06 K
      fracrefao(:) = (/ &
        2.1227E-01_rb,1.8897E-01_rb,1.3934E-01_rb,1.1557E-01_rb,9.5282E-02_rb,8.3359E-02_rb, &
        6.5333E-02_rb,5.2016E-02_rb,3.4272E-02_rb,4.0257E-03_rb,3.1857E-03_rb,2.6014E-03_rb, &
        1.9141E-03_rb,1.2612E-03_rb,5.3169E-04_rb,7.6476E-05_rb/)

! Planck fraction mapping level: P = 212.7250 mbar, T = 223.06 K
! These Planck fractions were calculated using lower atmosphere
! parameters.
      fracrefbo(:) = (/ &
        2.1227E-01_rb,1.8897E-01_rb,1.3934E-01_rb,1.1557E-01_rb,9.5282E-02_rb,8.3359E-02_rb, &
        6.5333E-02_rb,5.2016E-02_rb,3.4272E-02_rb,4.0257E-03_rb,3.1857E-03_rb,2.6014E-03_rb, &
        1.9141E-03_rb,1.2612E-03_rb,5.3169E-04_rb,7.6476E-05_rb/)

!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels > ~100mb and temperatures.  The first
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the corresponding TREF for this  pressure level, 
!     JT = 2 refers to the temperatureTREF-15, JT = 1 is for TREF-30, 
!     JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  The second 
!     index, JP, runs from 1 to 13 and refers to the corresponding 
!     pressure level in PREF (e.g. JP = 1 is for a pressure of 1053.63 mb).  
!     The third index, IG, goes from 1 to 16, and tells us which 
!     g-interval the absorption coefficients are for.

      kao(:, 1, 1) = (/ &
     &1.1936e-01_rb,1.2127e-01_rb,1.2290e-01_rb,1.2324e-01_rb,1.2302e-01_rb/)
      kao(:, 2, 1) = (/ &
     &9.3666e-02_rb,9.6336e-02_rb,9.8457e-02_rb,9.9155e-02_rb,9.9330e-02_rb/)
      kao(:, 3, 1) = (/ &
     &7.3480e-02_rb,7.5571e-02_rb,7.7998e-02_rb,7.9619e-02_rb,8.0000e-02_rb/)
      kao(:, 4, 1) = (/ &
     &5.8417e-02_rb,6.0062e-02_rb,6.2150e-02_rb,6.4024e-02_rb,6.4943e-02_rb/)
      kao(:, 5, 1) = (/ &
     &4.6742e-02_rb,4.8171e-02_rb,4.9840e-02_rb,5.1580e-02_rb,5.2974e-02_rb/)
      kao(:, 6, 1) = (/ &
     &3.7093e-02_rb,3.8712e-02_rb,4.0031e-02_rb,4.1473e-02_rb,4.2868e-02_rb/)
      kao(:, 7, 1) = (/ &
     &2.9515e-02_rb,3.1114e-02_rb,3.2344e-02_rb,3.3496e-02_rb,3.4715e-02_rb/)
      kao(:, 8, 1) = (/ &
     &2.3548e-02_rb,2.5012e-02_rb,2.6176e-02_rb,2.7253e-02_rb,2.8263e-02_rb/)
      kao(:, 9, 1) = (/ &
     &1.9136e-02_rb,2.0420e-02_rb,2.1607e-02_rb,2.2607e-02_rb,2.3502e-02_rb/)
      kao(:,10, 1) = (/ &
     &1.6947e-02_rb,1.8986e-02_rb,2.0617e-02_rb,2.1896e-02_rb,2.2928e-02_rb/)
      kao(:,11, 1) = (/ &
     &1.4724e-02_rb,1.6490e-02_rb,1.8468e-02_rb,1.9875e-02_rb,2.1095e-02_rb/)
      kao(:,12, 1) = (/ &
     &1.2332e-02_rb,1.3992e-02_rb,1.5549e-02_rb,1.7137e-02_rb,1.8190e-02_rb/)
      kao(:,13, 1) = (/ &
     &1.0144e-02_rb,1.1539e-02_rb,1.2861e-02_rb,1.4225e-02_rb,1.5224e-02_rb/)
      kao(:, 1, 2) = (/ &
     &3.5138e-01_rb,3.5502e-01_rb,3.5774e-01_rb,3.5894e-01_rb,3.5740e-01_rb/)
      kao(:, 2, 2) = (/ &
     &2.8740e-01_rb,2.8961e-01_rb,2.9119e-01_rb,2.9268e-01_rb,2.9128e-01_rb/)
      kao(:, 3, 2) = (/ &
     &2.3142e-01_rb,2.3495e-01_rb,2.3595e-01_rb,2.3731e-01_rb,2.3828e-01_rb/)
      kao(:, 4, 2) = (/ &
     &1.8684e-01_rb,1.9090e-01_rb,1.9387e-01_rb,1.9355e-01_rb,1.9518e-01_rb/)
      kao(:, 5, 2) = (/ &
     &1.5108e-01_rb,1.5525e-01_rb,1.5832e-01_rb,1.5904e-01_rb,1.5962e-01_rb/)
      kao(:, 6, 2) = (/ &
     &1.2219e-01_rb,1.2585e-01_rb,1.2901e-01_rb,1.3090e-01_rb,1.3158e-01_rb/)
      kao(:, 7, 2) = (/ &
     &9.8743e-02_rb,1.0185e-01_rb,1.0506e-01_rb,1.0763e-01_rb,1.0820e-01_rb/)
      kao(:, 8, 2) = (/ &
     &7.9966e-02_rb,8.2922e-02_rb,8.5859e-02_rb,8.8462e-02_rb,8.9282e-02_rb/)
      kao(:, 9, 2) = (/ &
     &6.5726e-02_rb,6.8954e-02_rb,7.1786e-02_rb,7.3678e-02_rb,7.5233e-02_rb/)
      kao(:,10, 2) = (/ &
     &6.0011e-02_rb,6.2230e-02_rb,6.5254e-02_rb,6.7031e-02_rb,6.9052e-02_rb/)
      kao(:,11, 2) = (/ &
     &5.5743e-02_rb,5.7099e-02_rb,5.8802e-02_rb,6.0875e-02_rb,6.2690e-02_rb/)
      kao(:,12, 2) = (/ &
     &4.9598e-02_rb,5.1234e-02_rb,5.2659e-02_rb,5.3951e-02_rb,5.5431e-02_rb/)
      kao(:,13, 2) = (/ &
     &4.1938e-02_rb,4.3540e-02_rb,4.4990e-02_rb,4.6074e-02_rb,4.6999e-02_rb/)
      kao(:, 1, 3) = (/ &
     &8.4590e-01_rb,8.4151e-01_rb,8.3682e-01_rb,8.3729e-01_rb,8.3944e-01_rb/)
      kao(:, 2, 3) = (/ &
     &6.8809e-01_rb,6.8696e-01_rb,6.8512e-01_rb,6.8470e-01_rb,6.8864e-01_rb/)
      kao(:, 3, 3) = (/ &
     &5.6065e-01_rb,5.5891e-01_rb,5.5871e-01_rb,5.5784e-01_rb,5.6029e-01_rb/)
      kao(:, 4, 3) = (/ &
     &4.6097e-01_rb,4.6021e-01_rb,4.5874e-01_rb,4.6110e-01_rb,4.6088e-01_rb/)
      kao(:, 5, 3) = (/ &
     &3.8076e-01_rb,3.8128e-01_rb,3.8007e-01_rb,3.8126e-01_rb,3.8214e-01_rb/)
      kao(:, 6, 3) = (/ &
     &3.1396e-01_rb,3.1576e-01_rb,3.1554e-01_rb,3.1478e-01_rb,3.1602e-01_rb/)
      kao(:, 7, 3) = (/ &
     &2.5841e-01_rb,2.6039e-01_rb,2.6160e-01_rb,2.6034e-01_rb,2.6147e-01_rb/)
      kao(:, 8, 3) = (/ &
     &2.1354e-01_rb,2.1540e-01_rb,2.1664e-01_rb,2.1675e-01_rb,2.1689e-01_rb/)
      kao(:, 9, 3) = (/ &
     &1.8409e-01_rb,1.8340e-01_rb,1.8423e-01_rb,1.8535e-01_rb,1.8550e-01_rb/)
      kao(:,10, 3) = (/ &
     &1.7687e-01_rb,1.8149e-01_rb,1.8138e-01_rb,1.8043e-01_rb,1.8050e-01_rb/)
      kao(:,11, 3) = (/ &
     &1.6660e-01_rb,1.6980e-01_rb,1.7005e-01_rb,1.7156e-01_rb,1.6983e-01_rb/)
      kao(:,12, 3) = (/ &
     &1.4699e-01_rb,1.5037e-01_rb,1.5032e-01_rb,1.5012e-01_rb,1.5041e-01_rb/)
      kao(:,13, 3) = (/ &
     &1.2497e-01_rb,1.2733e-01_rb,1.2732e-01_rb,1.2710e-01_rb,1.2722e-01_rb/)
      kao(:, 1, 4) = (/ &
     &1.8528e+00_rb,1.8358e+00_rb,1.8100e+00_rb,1.7728e+00_rb,1.7348e+00_rb/)
      kao(:, 2, 4) = (/ &
     &1.5191e+00_rb,1.5067e+00_rb,1.4864e+00_rb,1.4586e+00_rb,1.4273e+00_rb/)
      kao(:, 3, 4) = (/ &
     &1.2421e+00_rb,1.2332e+00_rb,1.2184e+00_rb,1.1999e+00_rb,1.1763e+00_rb/)
      kao(:, 4, 4) = (/ &
     &1.0235e+00_rb,1.0199e+00_rb,1.0105e+00_rb,9.9649e-01_rb,9.8141e-01_rb/)
      kao(:, 5, 4) = (/ &
     &8.4444e-01_rb,8.4579e-01_rb,8.4104e-01_rb,8.3201e-01_rb,8.2148e-01_rb/)
      kao(:, 6, 4) = (/ &
     &6.9623e-01_rb,7.0066e-01_rb,6.9879e-01_rb,6.9507e-01_rb,6.8745e-01_rb/)
      kao(:, 7, 4) = (/ &
     &5.7271e-01_rb,5.7848e-01_rb,5.7971e-01_rb,5.7773e-01_rb,5.7376e-01_rb/)
      kao(:, 8, 4) = (/ &
     &4.7082e-01_rb,4.7730e-01_rb,4.8061e-01_rb,4.7996e-01_rb,4.7783e-01_rb/)
      kao(:, 9, 4) = (/ &
     &3.8997e-01_rb,3.9821e-01_rb,4.0249e-01_rb,4.0315e-01_rb,4.0145e-01_rb/)
      kao(:,10, 4) = (/ &
     &3.5012e-01_rb,3.4335e-01_rb,3.3833e-01_rb,3.3654e-01_rb,3.3448e-01_rb/)
      kao(:,11, 4) = (/ &
     &3.1060e-01_rb,3.1084e-01_rb,3.0440e-01_rb,2.9201e-01_rb,2.8482e-01_rb/)
      kao(:,12, 4) = (/ &
     &2.7883e-01_rb,2.7190e-01_rb,2.6805e-01_rb,2.5917e-01_rb,2.4860e-01_rb/)
      kao(:,13, 4) = (/ &
     &2.3244e-01_rb,2.2666e-01_rb,2.2365e-01_rb,2.1647e-01_rb,2.0828e-01_rb/)
      kao(:, 1, 5) = (/ &
     &3.3225e+00_rb,3.2784e+00_rb,3.2423e+00_rb,3.2074e+00_rb,3.1704e+00_rb/)
      kao(:, 2, 5) = (/ &
     &2.7581e+00_rb,2.7256e+00_rb,2.6962e+00_rb,2.6686e+00_rb,2.6375e+00_rb/)
      kao(:, 3, 5) = (/ &
     &2.2798e+00_rb,2.2570e+00_rb,2.2330e+00_rb,2.2089e+00_rb,2.1829e+00_rb/)
      kao(:, 4, 5) = (/ &
     &1.8979e+00_rb,1.8796e+00_rb,1.8601e+00_rb,1.8393e+00_rb,1.8181e+00_rb/)
      kao(:, 5, 5) = (/ &
     &1.5861e+00_rb,1.5695e+00_rb,1.5550e+00_rb,1.5372e+00_rb,1.5203e+00_rb/)
      kao(:, 6, 5) = (/ &
     &1.3252e+00_rb,1.3124e+00_rb,1.3013e+00_rb,1.2872e+00_rb,1.2740e+00_rb/)
      kao(:, 7, 5) = (/ &
     &1.1027e+00_rb,1.0966e+00_rb,1.0870e+00_rb,1.0776e+00_rb,1.0667e+00_rb/)
      kao(:, 8, 5) = (/ &
     &9.1563e-01_rb,9.1410e-01_rb,9.0764e-01_rb,8.9982e-01_rb,8.9251e-01_rb/)
      kao(:, 9, 5) = (/ &
     &7.6595e-01_rb,7.6784e-01_rb,7.6494e-01_rb,7.5888e-01_rb,7.5288e-01_rb/)
      kao(:,10, 5) = (/ &
     &6.3388e-01_rb,6.4169e-01_rb,6.4724e-01_rb,6.4562e-01_rb,6.3971e-01_rb/)
      kao(:,11, 5) = (/ &
     &5.2794e-01_rb,5.2411e-01_rb,5.2455e-01_rb,5.2936e-01_rb,5.2927e-01_rb/)
      kao(:,12, 5) = (/ &
     &4.3721e-01_rb,4.3826e-01_rb,4.3490e-01_rb,4.3598e-01_rb,4.3794e-01_rb/)
      kao(:,13, 5) = (/ &
     &3.6693e-01_rb,3.6670e-01_rb,3.6374e-01_rb,3.6513e-01_rb,3.6694e-01_rb/)
      kao(:, 1, 6) = (/ &
     &6.6283e+00_rb,6.5357e+00_rb,6.4205e+00_rb,6.3046e+00_rb,6.1937e+00_rb/)
      kao(:, 2, 6) = (/ &
     &5.5921e+00_rb,5.5122e+00_rb,5.4227e+00_rb,5.3355e+00_rb,5.2548e+00_rb/)
      kao(:, 3, 6) = (/ &
     &4.6918e+00_rb,4.6295e+00_rb,4.5667e+00_rb,4.5016e+00_rb,4.4390e+00_rb/)
      kao(:, 4, 6) = (/ &
     &3.9477e+00_rb,3.9046e+00_rb,3.8607e+00_rb,3.8148e+00_rb,3.7628e+00_rb/)
      kao(:, 5, 6) = (/ &
     &3.3255e+00_rb,3.2963e+00_rb,3.2647e+00_rb,3.2299e+00_rb,3.1875e+00_rb/)
      kao(:, 6, 6) = (/ &
     &2.7972e+00_rb,2.7784e+00_rb,2.7561e+00_rb,2.7282e+00_rb,2.6941e+00_rb/)
      kao(:, 7, 6) = (/ &
     &2.3458e+00_rb,2.3342e+00_rb,2.3185e+00_rb,2.2969e+00_rb,2.2704e+00_rb/)
      kao(:, 8, 6) = (/ &
     &1.9572e+00_rb,1.9527e+00_rb,1.9423e+00_rb,1.9266e+00_rb,1.9056e+00_rb/)
      kao(:, 9, 6) = (/ &
     &1.6385e+00_rb,1.6357e+00_rb,1.6281e+00_rb,1.6171e+00_rb,1.6000e+00_rb/)
      kao(:,10, 6) = (/ &
     &1.4144e+00_rb,1.4143e+00_rb,1.4078e+00_rb,1.4009e+00_rb,1.3895e+00_rb/)
      kao(:,11, 6) = (/ &
     &1.1820e+00_rb,1.1847e+00_rb,1.1842e+00_rb,1.1773e+00_rb,1.1681e+00_rb/)
      kao(:,12, 6) = (/ &
     &9.7725e-01_rb,9.8091e-01_rb,9.8204e-01_rb,9.7703e-01_rb,9.7159e-01_rb/)
      kao(:,13, 6) = (/ &
     &8.1815e-01_rb,8.2185e-01_rb,8.2056e-01_rb,8.1693e-01_rb,8.1119e-01_rb/)
      kao(:, 1, 7) = (/ &
     &1.4313e+01_rb,1.4082e+01_rb,1.3850e+01_rb,1.3623e+01_rb,1.3382e+01_rb/)
      kao(:, 2, 7) = (/ &
     &1.2515e+01_rb,1.2314e+01_rb,1.2108e+01_rb,1.1882e+01_rb,1.1633e+01_rb/)
      kao(:, 3, 7) = (/ &
     &1.0802e+01_rb,1.0631e+01_rb,1.0439e+01_rb,1.0228e+01_rb,1.0005e+01_rb/)
      kao(:, 4, 7) = (/ &
     &9.2958e+00_rb,9.1469e+00_rb,8.9753e+00_rb,8.7884e+00_rb,8.6020e+00_rb/)
      kao(:, 5, 7) = (/ &
     &7.9646e+00_rb,7.8400e+00_rb,7.6929e+00_rb,7.5385e+00_rb,7.3901e+00_rb/)
      kao(:, 6, 7) = (/ &
     &6.7986e+00_rb,6.6983e+00_rb,6.5761e+00_rb,6.4518e+00_rb,6.3346e+00_rb/)
      kao(:, 7, 7) = (/ &
     &5.7703e+00_rb,5.6909e+00_rb,5.5962e+00_rb,5.4989e+00_rb,5.4062e+00_rb/)
      kao(:, 8, 7) = (/ &
     &4.8763e+00_rb,4.8159e+00_rb,4.7441e+00_rb,4.6692e+00_rb,4.5978e+00_rb/)
      kao(:, 9, 7) = (/ &
     &4.0868e+00_rb,4.0475e+00_rb,3.9918e+00_rb,3.9346e+00_rb,3.8784e+00_rb/)
      kao(:,10, 7) = (/ &
     &3.5589e+00_rb,3.5215e+00_rb,3.4698e+00_rb,3.4096e+00_rb,3.3531e+00_rb/)
      kao(:,11, 7) = (/ &
     &3.0702e+00_rb,3.0357e+00_rb,2.9927e+00_rb,2.9456e+00_rb,2.8950e+00_rb/)
      kao(:,12, 7) = (/ &
     &2.6022e+00_rb,2.5722e+00_rb,2.5392e+00_rb,2.5002e+00_rb,2.4548e+00_rb/)
      kao(:,13, 7) = (/ &
     &2.1727e+00_rb,2.1507e+00_rb,2.1220e+00_rb,2.0892e+00_rb,2.0541e+00_rb/)
      kao(:, 1, 8) = (/ &
     &3.4174e+01_rb,3.3741e+01_rb,3.3273e+01_rb,3.2762e+01_rb,3.2231e+01_rb/)
      kao(:, 2, 8) = (/ &
     &3.1501e+01_rb,3.1077e+01_rb,3.0621e+01_rb,3.0167e+01_rb,2.9717e+01_rb/)
      kao(:, 3, 8) = (/ &
     &2.8624e+01_rb,2.8247e+01_rb,2.7846e+01_rb,2.7455e+01_rb,2.7037e+01_rb/)
      kao(:, 4, 8) = (/ &
     &2.5860e+01_rb,2.5530e+01_rb,2.5179e+01_rb,2.4813e+01_rb,2.4424e+01_rb/)
      kao(:, 5, 8) = (/ &
     &2.3133e+01_rb,2.2868e+01_rb,2.2552e+01_rb,2.2222e+01_rb,2.1874e+01_rb/)
      kao(:, 6, 8) = (/ &
     &2.0417e+01_rb,2.0203e+01_rb,1.9954e+01_rb,1.9685e+01_rb,1.9370e+01_rb/)
      kao(:, 7, 8) = (/ &
     &1.7808e+01_rb,1.7656e+01_rb,1.7468e+01_rb,1.7234e+01_rb,1.6957e+01_rb/)
      kao(:, 8, 8) = (/ &
     &1.5349e+01_rb,1.5257e+01_rb,1.5109e+01_rb,1.4918e+01_rb,1.4683e+01_rb/)
      kao(:, 9, 8) = (/ &
     &1.3085e+01_rb,1.3031e+01_rb,1.2930e+01_rb,1.2779e+01_rb,1.2590e+01_rb/)
      kao(:,10, 8) = (/ &
     &1.0971e+01_rb,1.0917e+01_rb,1.0852e+01_rb,1.0748e+01_rb,1.0603e+01_rb/)
      kao(:,11, 8) = (/ &
     &9.5673e+00_rb,9.4985e+00_rb,9.4337e+00_rb,9.3153e+00_rb,9.1800e+00_rb/)
      kao(:,12, 8) = (/ &
     &8.2885e+00_rb,8.2471e+00_rb,8.1591e+00_rb,8.0541e+00_rb,7.9348e+00_rb/)
      kao(:,13, 8) = (/ &
     &7.1017e+00_rb,7.0462e+00_rb,6.9629e+00_rb,6.8569e+00_rb,6.7518e+00_rb/)
      kao(:, 1, 9) = (/ &
     &9.9761e+01_rb,9.8762e+01_rb,9.7605e+01_rb,9.6340e+01_rb,9.4995e+01_rb/)
      kao(:, 2, 9) = (/ &
     &9.9948e+01_rb,9.8866e+01_rb,9.7644e+01_rb,9.6363e+01_rb,9.4985e+01_rb/)
      kao(:, 3, 9) = (/ &
     &9.9007e+01_rb,9.8007e+01_rb,9.6838e+01_rb,9.5570e+01_rb,9.4251e+01_rb/)
      kao(:, 4, 9) = (/ &
     &9.6662e+01_rb,9.5791e+01_rb,9.4804e+01_rb,9.3746e+01_rb,9.2558e+01_rb/)
      kao(:, 5, 9) = (/ &
     &9.3093e+01_rb,9.2470e+01_rb,9.1704e+01_rb,9.0779e+01_rb,8.9739e+01_rb/)
      kao(:, 6, 9) = (/ &
     &8.8508e+01_rb,8.8114e+01_rb,8.7530e+01_rb,8.6816e+01_rb,8.5948e+01_rb/)
      kao(:, 7, 9) = (/ &
     &8.2974e+01_rb,8.2771e+01_rb,8.2394e+01_rb,8.1879e+01_rb,8.1118e+01_rb/)
      kao(:, 8, 9) = (/ &
     &7.6684e+01_rb,7.6645e+01_rb,7.6457e+01_rb,7.6020e+01_rb,7.5326e+01_rb/)
      kao(:, 9, 9) = (/ &
     &6.9973e+01_rb,7.0055e+01_rb,6.9978e+01_rb,6.9596e+01_rb,6.8941e+01_rb/)
      kao(:,10, 9) = (/ &
     &6.2676e+01_rb,6.2928e+01_rb,6.2907e+01_rb,6.2549e+01_rb,6.1993e+01_rb/)
      kao(:,11, 9) = (/ &
     &5.5263e+01_rb,5.5480e+01_rb,5.5306e+01_rb,5.4944e+01_rb,5.4442e+01_rb/)
      kao(:,12, 9) = (/ &
     &4.8636e+01_rb,4.8624e+01_rb,4.8458e+01_rb,4.8148e+01_rb,4.7712e+01_rb/)
      kao(:,13, 9) = (/ &
     &4.2167e+01_rb,4.2169e+01_rb,4.2037e+01_rb,4.1794e+01_rb,4.1453e+01_rb/)
      kao(:, 1,10) = (/ &
     &2.2059e+02_rb,2.1997e+02_rb,2.1921e+02_rb,2.1843e+02_rb,2.1743e+02_rb/)
      kao(:, 2,10) = (/ &
     &2.3102e+02_rb,2.3132e+02_rb,2.3151e+02_rb,2.3081e+02_rb,2.3008e+02_rb/)
      kao(:, 3,10) = (/ &
     &2.4111e+02_rb,2.4197e+02_rb,2.4166e+02_rb,2.4016e+02_rb,2.3836e+02_rb/)
      kao(:, 4,10) = (/ &
     &2.5158e+02_rb,2.5229e+02_rb,2.5082e+02_rb,2.4795e+02_rb,2.4522e+02_rb/)
      kao(:, 5,10) = (/ &
     &2.5765e+02_rb,2.5772e+02_rb,2.5582e+02_rb,2.5268e+02_rb,2.4923e+02_rb/)
      kao(:, 6,10) = (/ &
     &2.5856e+02_rb,2.5749e+02_rb,2.5502e+02_rb,2.5166e+02_rb,2.4826e+02_rb/)
      kao(:, 7,10) = (/ &
     &2.5368e+02_rb,2.5237e+02_rb,2.4999e+02_rb,2.4711e+02_rb,2.4470e+02_rb/)
      kao(:, 8,10) = (/ &
     &2.4476e+02_rb,2.4415e+02_rb,2.4213e+02_rb,2.4060e+02_rb,2.3968e+02_rb/)
      kao(:, 9,10) = (/ &
     &2.3178e+02_rb,2.3169e+02_rb,2.3048e+02_rb,2.3066e+02_rb,2.3132e+02_rb/)
      kao(:,10,10) = (/ &
     &2.1736e+02_rb,2.1726e+02_rb,2.1730e+02_rb,2.1880e+02_rb,2.1934e+02_rb/)
      kao(:,11,10) = (/ &
     &2.0114e+02_rb,2.0172e+02_rb,2.0351e+02_rb,2.0457e+02_rb,2.0387e+02_rb/)
      kao(:,12,10) = (/ &
     &1.8488e+02_rb,1.8650e+02_rb,1.8715e+02_rb,1.8619e+02_rb,1.8458e+02_rb/)
      kao(:,13,10) = (/ &
     &1.7142e+02_rb,1.7281e+02_rb,1.7236e+02_rb,1.7114e+02_rb,1.6934e+02_rb/)
      kao(:, 1,11) = (/ &
     &2.9459e+02_rb,2.9267e+02_rb,2.9151e+02_rb,2.8931e+02_rb,2.8695e+02_rb/)
      kao(:, 2,11) = (/ &
     &3.1654e+02_rb,3.1722e+02_rb,3.1773e+02_rb,3.1519e+02_rb,3.1185e+02_rb/)
      kao(:, 3,11) = (/ &
     &3.3345e+02_rb,3.3361e+02_rb,3.3487e+02_rb,3.3474e+02_rb,3.3330e+02_rb/)
      kao(:, 4,11) = (/ &
     &3.4222e+02_rb,3.4143e+02_rb,3.4230e+02_rb,3.4254e+02_rb,3.4132e+02_rb/)
      kao(:, 5,11) = (/ &
     &3.5206e+02_rb,3.5057e+02_rb,3.4997e+02_rb,3.4857e+02_rb,3.4588e+02_rb/)
      kao(:, 6,11) = (/ &
     &3.6028e+02_rb,3.6070e+02_rb,3.6119e+02_rb,3.5790e+02_rb,3.5262e+02_rb/)
      kao(:, 7,11) = (/ &
     &3.6620e+02_rb,3.6652e+02_rb,3.6631e+02_rb,3.6276e+02_rb,3.5799e+02_rb/)
      kao(:, 8,11) = (/ &
     &3.6855e+02_rb,3.6845e+02_rb,3.6741e+02_rb,3.6316e+02_rb,3.5815e+02_rb/)
      kao(:, 9,11) = (/ &
     &3.6490e+02_rb,3.6506e+02_rb,3.6353e+02_rb,3.5922e+02_rb,3.5455e+02_rb/)
      kao(:,10,11) = (/ &
     &3.5317e+02_rb,3.5433e+02_rb,3.5339e+02_rb,3.5013e+02_rb,3.4719e+02_rb/)
      kao(:,11,11) = (/ &
     &3.3639e+02_rb,3.3748e+02_rb,3.3591e+02_rb,3.3444e+02_rb,3.3466e+02_rb/)
      kao(:,12,11) = (/ &
     &3.1388e+02_rb,3.1454e+02_rb,3.1501e+02_rb,3.1656e+02_rb,3.1666e+02_rb/)
      kao(:,13,11) = (/ &
     &2.9044e+02_rb,2.9107e+02_rb,2.9274e+02_rb,2.9324e+02_rb,2.9124e+02_rb/)
      kao(:, 1,12) = (/ &
     &3.8243e+02_rb,3.8335e+02_rb,3.8560e+02_rb,3.8967e+02_rb,3.9390e+02_rb/)
      kao(:, 2,12) = (/ &
     &4.2476e+02_rb,4.2018e+02_rb,4.1643e+02_rb,4.1764e+02_rb,4.1841e+02_rb/)
      kao(:, 3,12) = (/ &
     &4.5902e+02_rb,4.5366e+02_rb,4.4789e+02_rb,4.4616e+02_rb,4.4366e+02_rb/)
      kao(:, 4,12) = (/ &
     &4.8576e+02_rb,4.8192e+02_rb,4.7838e+02_rb,4.7712e+02_rb,4.7436e+02_rb/)
      kao(:, 5,12) = (/ &
     &5.0548e+02_rb,5.0130e+02_rb,4.9886e+02_rb,4.9965e+02_rb,4.9992e+02_rb/)
      kao(:, 6,12) = (/ &
     &5.2714e+02_rb,5.1902e+02_rb,5.1379e+02_rb,5.1543e+02_rb,5.1780e+02_rb/)
      kao(:, 7,12) = (/ &
     &5.5149e+02_rb,5.4136e+02_rb,5.3395e+02_rb,5.3343e+02_rb,5.3337e+02_rb/)
      kao(:, 8,12) = (/ &
     &5.7260e+02_rb,5.6332e+02_rb,5.5702e+02_rb,5.5596e+02_rb,5.5242e+02_rb/)
      kao(:, 9,12) = (/ &
     &5.8640e+02_rb,5.7843e+02_rb,5.7443e+02_rb,5.7374e+02_rb,5.6970e+02_rb/)
      kao(:,10,12) = (/ &
     &5.9437e+02_rb,5.8760e+02_rb,5.8375e+02_rb,5.8166e+02_rb,5.7639e+02_rb/)
      kao(:,11,12) = (/ &
     &5.8998e+02_rb,5.8468e+02_rb,5.8336e+02_rb,5.7943e+02_rb,5.7161e+02_rb/)
      kao(:,12,12) = (/ &
     &5.7429e+02_rb,5.7360e+02_rb,5.7128e+02_rb,5.6564e+02_rb,5.6011e+02_rb/)
      kao(:,13,12) = (/ &
     &5.5035e+02_rb,5.5083e+02_rb,5.4893e+02_rb,5.4549e+02_rb,5.4357e+02_rb/)
      kao(:, 1,13) = (/ &
     &4.9343e+02_rb,4.9586e+02_rb,4.9750e+02_rb,4.9873e+02_rb,4.9898e+02_rb/)
      kao(:, 2,13) = (/ &
     &5.4485e+02_rb,5.4781e+02_rb,5.5082e+02_rb,5.5168e+02_rb,5.5417e+02_rb/)
      kao(:, 3,13) = (/ &
     &6.1399e+02_rb,6.1257e+02_rb,6.1216e+02_rb,6.1051e+02_rb,6.1193e+02_rb/)
      kao(:, 4,13) = (/ &
     &6.9330e+02_rb,6.8682e+02_rb,6.8040e+02_rb,6.7417e+02_rb,6.7247e+02_rb/)
      kao(:, 5,13) = (/ &
     &7.6881e+02_rb,7.6265e+02_rb,7.5375e+02_rb,7.4386e+02_rb,7.3721e+02_rb/)
      kao(:, 6,13) = (/ &
     &8.3052e+02_rb,8.2987e+02_rb,8.2363e+02_rb,8.1336e+02_rb,8.0445e+02_rb/)
      kao(:, 7,13) = (/ &
     &8.8082e+02_rb,8.8344e+02_rb,8.8117e+02_rb,8.7395e+02_rb,8.6653e+02_rb/)
      kao(:, 8,13) = (/ &
     &9.2485e+02_rb,9.2603e+02_rb,9.2355e+02_rb,9.1727e+02_rb,9.1414e+02_rb/)
      kao(:, 9,13) = (/ &
     &9.7276e+02_rb,9.7146e+02_rb,9.6566e+02_rb,9.5635e+02_rb,9.5212e+02_rb/)
      kao(:,10,13) = (/ &
     &1.0175e+03_rb,1.0163e+03_rb,1.0069e+03_rb,9.9610e+02_rb,9.9075e+02_rb/)
      kao(:,11,13) = (/ &
     &1.0564e+03_rb,1.0547e+03_rb,1.0423e+03_rb,1.0316e+03_rb,1.0272e+03_rb/)
      kao(:,12,13) = (/ &
     &1.0856e+03_rb,1.0807e+03_rb,1.0716e+03_rb,1.0657e+03_rb,1.0617e+03_rb/)
      kao(:,13,13) = (/ &
     &1.1006e+03_rb,1.0930e+03_rb,1.0868e+03_rb,1.0845e+03_rb,1.0783e+03_rb/)
      kao(:, 1,14) = (/ &
     &7.0211e+02_rb,7.0175e+02_rb,7.0131e+02_rb,6.9909e+02_rb,6.9776e+02_rb/)
      kao(:, 2,14) = (/ &
     &7.9113e+02_rb,7.9461e+02_rb,7.9834e+02_rb,7.9969e+02_rb,7.9813e+02_rb/)
      kao(:, 3,14) = (/ &
     &8.8896e+02_rb,8.9686e+02_rb,9.0679e+02_rb,9.1320e+02_rb,9.1562e+02_rb/)
      kao(:, 4,14) = (/ &
     &9.9457e+02_rb,1.0048e+03_rb,1.0163e+03_rb,1.0256e+03_rb,1.0309e+03_rb/)
      kao(:, 5,14) = (/ &
     &1.1171e+03_rb,1.1226e+03_rb,1.1342e+03_rb,1.1464e+03_rb,1.1541e+03_rb/)
      kao(:, 6,14) = (/ &
     &1.2650e+03_rb,1.2607e+03_rb,1.2659e+03_rb,1.2764e+03_rb,1.2854e+03_rb/)
      kao(:, 7,14) = (/ &
     &1.4362e+03_rb,1.4239e+03_rb,1.4228e+03_rb,1.4266e+03_rb,1.4315e+03_rb/)
      kao(:, 8,14) = (/ &
     &1.6263e+03_rb,1.6088e+03_rb,1.5994e+03_rb,1.5975e+03_rb,1.5958e+03_rb/)
      kao(:, 9,14) = (/ &
     &1.8039e+03_rb,1.7901e+03_rb,1.7817e+03_rb,1.7757e+03_rb,1.7698e+03_rb/)
      kao(:,10,14) = (/ &
     &1.9623e+03_rb,1.9495e+03_rb,1.9467e+03_rb,1.9462e+03_rb,1.9411e+03_rb/)
      kao(:,11,14) = (/ &
     &2.1010e+03_rb,2.0894e+03_rb,2.0912e+03_rb,2.0963e+03_rb,2.0914e+03_rb/)
      kao(:,12,14) = (/ &
     &2.2319e+03_rb,2.2209e+03_rb,2.2201e+03_rb,2.2217e+03_rb,2.2137e+03_rb/)
      kao(:,13,14) = (/ &
     &2.3516e+03_rb,2.3425e+03_rb,2.3393e+03_rb,2.3344e+03_rb,2.3274e+03_rb/)
      kao(:, 1,15) = (/ &
     &1.0409e+03_rb,1.0602e+03_rb,1.0710e+03_rb,1.0853e+03_rb,1.1018e+03_rb/)
      kao(:, 2,15) = (/ &
     &1.2151e+03_rb,1.2303e+03_rb,1.2347e+03_rb,1.2478e+03_rb,1.2680e+03_rb/)
      kao(:, 3,15) = (/ &
     &1.4193e+03_rb,1.4320e+03_rb,1.4382e+03_rb,1.4463e+03_rb,1.4605e+03_rb/)
      kao(:, 4,15) = (/ &
     &1.6341e+03_rb,1.6472e+03_rb,1.6601e+03_rb,1.6700e+03_rb,1.6846e+03_rb/)
      kao(:, 5,15) = (/ &
     &1.8786e+03_rb,1.8904e+03_rb,1.9028e+03_rb,1.9104e+03_rb,1.9250e+03_rb/)
      kao(:, 6,15) = (/ &
     &2.1492e+03_rb,2.1663e+03_rb,2.1776e+03_rb,2.1833e+03_rb,2.1940e+03_rb/)
      kao(:, 7,15) = (/ &
     &2.4310e+03_rb,2.4567e+03_rb,2.4727e+03_rb,2.4804e+03_rb,2.4891e+03_rb/)
      kao(:, 8,15) = (/ &
     &2.7261e+03_rb,2.7552e+03_rb,2.7803e+03_rb,2.7970e+03_rb,2.8141e+03_rb/)
      kao(:, 9,15) = (/ &
     &3.0867e+03_rb,3.0994e+03_rb,3.1170e+03_rb,3.1402e+03_rb,3.1679e+03_rb/)
      kao(:,10,15) = (/ &
     &3.5269e+03_rb,3.5204e+03_rb,3.5225e+03_rb,3.5345e+03_rb,3.5601e+03_rb/)
      kao(:,11,15) = (/ &
     &4.0271e+03_rb,4.0096e+03_rb,4.0024e+03_rb,4.0052e+03_rb,4.0246e+03_rb/)
      kao(:,12,15) = (/ &
     &4.5798e+03_rb,4.5585e+03_rb,4.5503e+03_rb,4.5442e+03_rb,4.5532e+03_rb/)
      kao(:,13,15) = (/ &
     &5.1911e+03_rb,5.1717e+03_rb,5.1553e+03_rb,5.1395e+03_rb,5.1397e+03_rb/)
      kao(:, 1,16) = (/ &
     &1.2071e+03_rb,1.2414e+03_rb,1.3062e+03_rb,1.3645e+03_rb,1.4123e+03_rb/)
      kao(:, 2,16) = (/ &
     &1.4670e+03_rb,1.4886e+03_rb,1.5547e+03_rb,1.6269e+03_rb,1.6875e+03_rb/)
      kao(:, 3,16) = (/ &
     &1.7847e+03_rb,1.7994e+03_rb,1.8394e+03_rb,1.9097e+03_rb,1.9863e+03_rb/)
      kao(:, 4,16) = (/ &
     &2.1409e+03_rb,2.1717e+03_rb,2.1953e+03_rb,2.2438e+03_rb,2.3059e+03_rb/)
      kao(:, 5,16) = (/ &
     &2.5417e+03_rb,2.5918e+03_rb,2.6196e+03_rb,2.6591e+03_rb,2.7124e+03_rb/)
      kao(:, 6,16) = (/ &
     &3.0242e+03_rb,3.0688e+03_rb,3.1113e+03_rb,3.1534e+03_rb,3.2074e+03_rb/)
      kao(:, 7,16) = (/ &
     &3.6491e+03_rb,3.6695e+03_rb,3.6939e+03_rb,3.7229e+03_rb,3.7906e+03_rb/)
      kao(:, 8,16) = (/ &
     &4.4223e+03_rb,4.4232e+03_rb,4.4297e+03_rb,4.4268e+03_rb,4.4608e+03_rb/)
      kao(:, 9,16) = (/ &
     &5.3502e+03_rb,5.3400e+03_rb,5.3338e+03_rb,5.3140e+03_rb,5.2816e+03_rb/)
      kao(:,10,16) = (/ &
     &6.4333e+03_rb,6.4257e+03_rb,6.4093e+03_rb,6.3744e+03_rb,6.3162e+03_rb/)
      kao(:,11,16) = (/ &
     &7.6805e+03_rb,7.6778e+03_rb,7.6466e+03_rb,7.5865e+03_rb,7.5106e+03_rb/)
      kao(:,12,16) = (/ &
     &9.1073e+03_rb,9.0972e+03_rb,9.0428e+03_rb,8.9445e+03_rb,8.8997e+03_rb/)
      kao(:,13,16) = (/ &
     &1.0699e+04_rb,1.0668e+04_rb,1.0580e+04_rb,1.0493e+04_rb,1.0473e+04_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &1.0144e-02_rb,1.1539e-02_rb,1.2861e-02_rb,1.4225e-02_rb,1.5224e-02_rb/)
      kbo(:,14, 1) = (/ &
     &8.4327e-03_rb,9.5989e-03_rb,1.0734e-02_rb,1.1890e-02_rb,1.2872e-02_rb/)
      kbo(:,15, 1) = (/ &
     &6.9967e-03_rb,7.9676e-03_rb,8.9559e-03_rb,9.9119e-03_rb,1.0533e-02_rb/)
      kbo(:,16, 1) = (/ &
     &5.7988e-03_rb,6.6068e-03_rb,7.4553e-03_rb,8.2402e-03_rb,8.5310e-03_rb/)
      kbo(:,17, 1) = (/ &
     &4.8055e-03_rb,5.4781e-03_rb,6.1884e-03_rb,6.8148e-03_rb,6.9563e-03_rb/)
      kbo(:,18, 1) = (/ &
     &3.9877e-03_rb,4.5423e-03_rb,5.1325e-03_rb,5.5098e-03_rb,5.6946e-03_rb/)
      kbo(:,19, 1) = (/ &
     &3.3110e-03_rb,3.7880e-03_rb,4.2806e-03_rb,4.4996e-03_rb,4.6784e-03_rb/)
      kbo(:,20, 1) = (/ &
     &2.7631e-03_rb,3.1756e-03_rb,3.5803e-03_rb,3.7100e-03_rb,3.8598e-03_rb/)
      kbo(:,21, 1) = (/ &
     &2.3113e-03_rb,2.6584e-03_rb,2.9347e-03_rb,3.0481e-03_rb,3.1824e-03_rb/)
      kbo(:,22, 1) = (/ &
     &1.9510e-03_rb,2.2389e-03_rb,2.4076e-03_rb,2.5106e-03_rb,2.6293e-03_rb/)
      kbo(:,23, 1) = (/ &
     &1.6490e-03_rb,1.8833e-03_rb,1.9839e-03_rb,2.0707e-03_rb,2.1757e-03_rb/)
      kbo(:,24, 1) = (/ &
     &1.3921e-03_rb,1.5822e-03_rb,1.6416e-03_rb,1.7136e-03_rb,1.8019e-03_rb/)
      kbo(:,25, 1) = (/ &
     &1.1754e-03_rb,1.2986e-03_rb,1.3546e-03_rb,1.4184e-03_rb,1.4951e-03_rb/)
      kbo(:,26, 1) = (/ &
     &9.9384e-04_rb,1.0692e-03_rb,1.1181e-03_rb,1.1767e-03_rb,1.2423e-03_rb/)
      kbo(:,27, 1) = (/ &
     &8.3932e-04_rb,8.8446e-04_rb,9.2496e-04_rb,9.7555e-04_rb,1.0286e-03_rb/)
      kbo(:,28, 1) = (/ &
     &7.0655e-04_rb,7.3418e-04_rb,7.6647e-04_rb,8.0853e-04_rb,8.5091e-04_rb/)
      kbo(:,29, 1) = (/ &
     &5.8016e-04_rb,6.0558e-04_rb,6.3439e-04_rb,6.6940e-04_rb,7.0187e-04_rb/)
      kbo(:,30, 1) = (/ &
     &4.7757e-04_rb,4.9924e-04_rb,5.2442e-04_rb,5.5397e-04_rb,5.7829e-04_rb/)
      kbo(:,31, 1) = (/ &
     &3.9420e-04_rb,4.1123e-04_rb,4.3298e-04_rb,4.5563e-04_rb,4.7437e-04_rb/)
      kbo(:,32, 1) = (/ &
     &3.2571e-04_rb,3.3960e-04_rb,3.5704e-04_rb,3.7387e-04_rb,3.8849e-04_rb/)
      kbo(:,33, 1) = (/ &
     &2.6741e-04_rb,2.8027e-04_rb,2.9349e-04_rb,3.0636e-04_rb,3.1790e-04_rb/)
      kbo(:,34, 1) = (/ &
     &2.2029e-04_rb,2.3073e-04_rb,2.4120e-04_rb,2.5116e-04_rb,2.5977e-04_rb/)
      kbo(:,35, 1) = (/ &
     &1.8092e-04_rb,1.8913e-04_rb,1.9723e-04_rb,2.0490e-04_rb,2.1120e-04_rb/)
      kbo(:,36, 1) = (/ &
     &1.4775e-04_rb,1.5426e-04_rb,1.6038e-04_rb,1.6643e-04_rb,1.7137e-04_rb/)
      kbo(:,37, 1) = (/ &
     &1.2005e-04_rb,1.2534e-04_rb,1.3001e-04_rb,1.3495e-04_rb,1.3888e-04_rb/)
      kbo(:,38, 1) = (/ &
     &9.7560e-05_rb,1.0177e-04_rb,1.0540e-04_rb,1.0924e-04_rb,1.1256e-04_rb/)
      kbo(:,39, 1) = (/ &
     &7.9344e-05_rb,8.2615e-05_rb,8.5368e-05_rb,8.8405e-05_rb,9.1186e-05_rb/)
      kbo(:,40, 1) = (/ &
     &6.4575e-05_rb,6.7008e-05_rb,6.9369e-05_rb,7.1740e-05_rb,7.4003e-05_rb/)
      kbo(:,41, 1) = (/ &
     &5.2697e-05_rb,5.4334e-05_rb,5.6409e-05_rb,5.8228e-05_rb,6.0014e-05_rb/)
      kbo(:,42, 1) = (/ &
     &4.3089e-05_rb,4.4120e-05_rb,4.5815e-05_rb,4.7218e-05_rb,4.8658e-05_rb/)
      kbo(:,43, 1) = (/ &
     &3.4429e-05_rb,3.5847e-05_rb,3.7149e-05_rb,3.8380e-05_rb,3.9479e-05_rb/)
      kbo(:,44, 1) = (/ &
     &2.7169e-05_rb,2.9300e-05_rb,3.0128e-05_rb,3.1175e-05_rb,3.2021e-05_rb/)
      kbo(:,45, 1) = (/ &
     &2.1411e-05_rb,2.3782e-05_rb,2.4360e-05_rb,2.5287e-05_rb,2.6025e-05_rb/)
      kbo(:,46, 1) = (/ &
     &1.6754e-05_rb,1.8795e-05_rb,1.9931e-05_rb,2.0532e-05_rb,2.1145e-05_rb/)
      kbo(:,47, 1) = (/ &
     &1.3067e-05_rb,1.4757e-05_rb,1.6247e-05_rb,1.6575e-05_rb,1.7184e-05_rb/)
      kbo(:,48, 1) = (/ &
     &1.0193e-05_rb,1.1527e-05_rb,1.2841e-05_rb,1.3599e-05_rb,1.3946e-05_rb/)
      kbo(:,49, 1) = (/ &
     &7.9417e-06_rb,8.9971e-06_rb,1.0112e-05_rb,1.1045e-05_rb,1.1285e-05_rb/)
      kbo(:,50, 1) = (/ &
     &6.1950e-06_rb,7.0678e-06_rb,7.9571e-06_rb,8.7859e-06_rb,9.3116e-06_rb/)
      kbo(:,51, 1) = (/ &
     &4.8121e-06_rb,5.5540e-06_rb,6.2634e-06_rb,6.9968e-06_rb,7.5750e-06_rb/)
      kbo(:,52, 1) = (/ &
     &3.7732e-06_rb,4.3400e-06_rb,4.9368e-06_rb,5.5255e-06_rb,6.0507e-06_rb/)
      kbo(:,53, 1) = (/ &
     &2.9588e-06_rb,3.3855e-06_rb,3.8934e-06_rb,4.3558e-06_rb,4.8377e-06_rb/)
      kbo(:,54, 1) = (/ &
     &2.2637e-06_rb,2.6550e-06_rb,3.0640e-06_rb,3.4670e-06_rb,3.8645e-06_rb/)
      kbo(:,55, 1) = (/ &
     &1.7394e-06_rb,2.1318e-06_rb,2.4092e-06_rb,2.7718e-06_rb,3.0969e-06_rb/)
      kbo(:,56, 1) = (/ &
     &1.3379e-06_rb,1.6962e-06_rb,1.9182e-06_rb,2.2019e-06_rb,2.5057e-06_rb/)
      kbo(:,57, 1) = (/ &
     &1.0894e-06_rb,1.3086e-06_rb,1.5250e-06_rb,1.7825e-06_rb,2.0078e-06_rb/)
      kbo(:,58, 1) = (/ &
     &8.2505e-07_rb,1.0180e-06_rb,1.2548e-06_rb,1.4584e-06_rb,1.7398e-06_rb/)
      kbo(:,59, 1) = (/ &
     &7.1802e-07_rb,8.6784e-07_rb,1.0974e-06_rb,1.3504e-06_rb,1.4314e-06_rb/)
      kbo(:,13, 2) = (/ &
     &4.1938e-02_rb,4.3540e-02_rb,4.4990e-02_rb,4.6074e-02_rb,4.6999e-02_rb/)
      kbo(:,14, 2) = (/ &
     &3.5928e-02_rb,3.7447e-02_rb,3.8935e-02_rb,3.9816e-02_rb,4.0666e-02_rb/)
      kbo(:,15, 2) = (/ &
     &3.0539e-02_rb,3.1836e-02_rb,3.3086e-02_rb,3.4096e-02_rb,3.5203e-02_rb/)
      kbo(:,16, 2) = (/ &
     &2.5542e-02_rb,2.6607e-02_rb,2.7654e-02_rb,2.8622e-02_rb,2.9849e-02_rb/)
      kbo(:,17, 2) = (/ &
     &2.1059e-02_rb,2.2159e-02_rb,2.3097e-02_rb,2.3970e-02_rb,2.5021e-02_rb/)
      kbo(:,18, 2) = (/ &
     &1.7420e-02_rb,1.8344e-02_rb,1.9223e-02_rb,2.0092e-02_rb,2.0887e-02_rb/)
      kbo(:,19, 2) = (/ &
     &1.4527e-02_rb,1.5294e-02_rb,1.6071e-02_rb,1.6950e-02_rb,1.7660e-02_rb/)
      kbo(:,20, 2) = (/ &
     &1.2057e-02_rb,1.2740e-02_rb,1.3375e-02_rb,1.4212e-02_rb,1.4836e-02_rb/)
      kbo(:,21, 2) = (/ &
     &1.0024e-02_rb,1.0630e-02_rb,1.1202e-02_rb,1.1918e-02_rb,1.2460e-02_rb/)
      kbo(:,22, 2) = (/ &
     &8.3586e-03_rb,8.8721e-03_rb,9.4319e-03_rb,1.0016e-02_rb,1.0451e-02_rb/)
      kbo(:,23, 2) = (/ &
     &7.0023e-03_rb,7.4292e-03_rb,7.9425e-03_rb,8.4001e-03_rb,8.6668e-03_rb/)
      kbo(:,24, 2) = (/ &
     &5.8805e-03_rb,6.2414e-03_rb,6.6855e-03_rb,7.0380e-03_rb,7.1881e-03_rb/)
      kbo(:,25, 2) = (/ &
     &4.9300e-03_rb,5.2659e-03_rb,5.6130e-03_rb,5.8139e-03_rb,5.9744e-03_rb/)
      kbo(:,26, 2) = (/ &
     &4.1346e-03_rb,4.4268e-03_rb,4.6994e-03_rb,4.8189e-03_rb,4.9700e-03_rb/)
      kbo(:,27, 2) = (/ &
     &3.4478e-03_rb,3.6949e-03_rb,3.8760e-03_rb,3.9800e-03_rb,4.1083e-03_rb/)
      kbo(:,28, 2) = (/ &
     &2.8662e-03_rb,3.0720e-03_rb,3.1855e-03_rb,3.2844e-03_rb,3.3841e-03_rb/)
      kbo(:,29, 2) = (/ &
     &2.3700e-03_rb,2.5289e-03_rb,2.6049e-03_rb,2.6846e-03_rb,2.7555e-03_rb/)
      kbo(:,30, 2) = (/ &
     &1.9529e-03_rb,2.0734e-03_rb,2.1236e-03_rb,2.1766e-03_rb,2.2279e-03_rb/)
      kbo(:,31, 2) = (/ &
     &1.5951e-03_rb,1.6761e-03_rb,1.7103e-03_rb,1.7458e-03_rb,1.7856e-03_rb/)
      kbo(:,32, 2) = (/ &
     &1.2965e-03_rb,1.3546e-03_rb,1.3767e-03_rb,1.4016e-03_rb,1.4345e-03_rb/)
      kbo(:,33, 2) = (/ &
     &1.0497e-03_rb,1.0916e-03_rb,1.1061e-03_rb,1.1259e-03_rb,1.1490e-03_rb/)
      kbo(:,34, 2) = (/ &
     &8.5263e-04_rb,8.8439e-04_rb,8.9299e-04_rb,9.0901e-04_rb,9.2683e-04_rb/)
      kbo(:,35, 2) = (/ &
     &6.8961e-04_rb,7.1452e-04_rb,7.2125e-04_rb,7.3408e-04_rb,7.4718e-04_rb/)
      kbo(:,36, 2) = (/ &
     &5.5538e-04_rb,5.7454e-04_rb,5.8468e-04_rb,5.9295e-04_rb,6.0312e-04_rb/)
      kbo(:,37, 2) = (/ &
     &4.4687e-04_rb,4.6228e-04_rb,4.7485e-04_rb,4.8052e-04_rb,4.8826e-04_rb/)
      kbo(:,38, 2) = (/ &
     &3.5911e-04_rb,3.7176e-04_rb,3.8278e-04_rb,3.8998e-04_rb,3.9487e-04_rb/)
      kbo(:,39, 2) = (/ &
     &2.8825e-04_rb,2.9868e-04_rb,3.0761e-04_rb,3.1500e-04_rb,3.2065e-04_rb/)
      kbo(:,40, 2) = (/ &
     &2.3221e-04_rb,2.4117e-04_rb,2.4916e-04_rb,2.5539e-04_rb,2.6149e-04_rb/)
      kbo(:,41, 2) = (/ &
     &1.8711e-04_rb,1.9520e-04_rb,2.0184e-04_rb,2.0732e-04_rb,2.1208e-04_rb/)
      kbo(:,42, 2) = (/ &
     &1.5044e-04_rb,1.5806e-04_rb,1.6349e-04_rb,1.6813e-04_rb,1.7186e-04_rb/)
      kbo(:,43, 2) = (/ &
     &1.2147e-04_rb,1.2754e-04_rb,1.3202e-04_rb,1.3603e-04_rb,1.3922e-04_rb/)
      kbo(:,44, 2) = (/ &
     &9.7631e-05_rb,1.0254e-04_rb,1.0664e-04_rb,1.1000e-04_rb,1.1278e-04_rb/)
      kbo(:,45, 2) = (/ &
     &7.8551e-05_rb,8.2386e-05_rb,8.6252e-05_rb,8.8821e-05_rb,9.1113e-05_rb/)
      kbo(:,46, 2) = (/ &
     &6.3082e-05_rb,6.6217e-05_rb,6.9264e-05_rb,7.1597e-05_rb,7.3596e-05_rb/)
      kbo(:,47, 2) = (/ &
     &5.0668e-05_rb,5.3148e-05_rb,5.5648e-05_rb,5.8023e-05_rb,5.9460e-05_rb/)
      kbo(:,48, 2) = (/ &
     &4.0723e-05_rb,4.2652e-05_rb,4.4714e-05_rb,4.6562e-05_rb,4.8110e-05_rb/)
      kbo(:,49, 2) = (/ &
     &3.2577e-05_rb,3.4241e-05_rb,3.5845e-05_rb,3.7418e-05_rb,3.8704e-05_rb/)
      kbo(:,50, 2) = (/ &
     &2.6548e-05_rb,2.7645e-05_rb,2.8887e-05_rb,3.0243e-05_rb,3.1285e-05_rb/)
      kbo(:,51, 2) = (/ &
     &2.1632e-05_rb,2.2305e-05_rb,2.3402e-05_rb,2.4420e-05_rb,2.5424e-05_rb/)
      kbo(:,52, 2) = (/ &
     &1.7494e-05_rb,1.8158e-05_rb,1.8946e-05_rb,1.9746e-05_rb,2.0626e-05_rb/)
      kbo(:,53, 2) = (/ &
     &1.4060e-05_rb,1.4924e-05_rb,1.5296e-05_rb,1.6047e-05_rb,1.6661e-05_rb/)
      kbo(:,54, 2) = (/ &
     &1.1502e-05_rb,1.2189e-05_rb,1.2591e-05_rb,1.3072e-05_rb,1.3585e-05_rb/)
      kbo(:,55, 2) = (/ &
     &9.4731e-06_rb,9.9000e-06_rb,1.0499e-05_rb,1.0730e-05_rb,1.1201e-05_rb/)
      kbo(:,56, 2) = (/ &
     &7.7842e-06_rb,8.0609e-06_rb,8.6328e-06_rb,8.9073e-06_rb,9.2258e-06_rb/)
      kbo(:,57, 2) = (/ &
     &6.1271e-06_rb,6.6999e-06_rb,7.0368e-06_rb,7.4466e-06_rb,7.6395e-06_rb/)
      kbo(:,58, 2) = (/ &
     &4.7873e-06_rb,5.5615e-06_rb,5.7470e-06_rb,6.1637e-06_rb,6.3705e-06_rb/)
      kbo(:,59, 2) = (/ &
     &3.8649e-06_rb,4.5966e-06_rb,4.7965e-06_rb,5.1490e-06_rb,5.3880e-06_rb/)
      kbo(:,13, 3) = (/ &
     &1.2497e-01_rb,1.2733e-01_rb,1.2732e-01_rb,1.2710e-01_rb,1.2722e-01_rb/)
      kbo(:,14, 3) = (/ &
     &1.0770e-01_rb,1.1044e-01_rb,1.0994e-01_rb,1.0972e-01_rb,1.0842e-01_rb/)
      kbo(:,15, 3) = (/ &
     &9.2313e-02_rb,9.4782e-02_rb,9.3928e-02_rb,9.2439e-02_rb,9.0946e-02_rb/)
      kbo(:,16, 3) = (/ &
     &7.7737e-02_rb,7.9134e-02_rb,7.7340e-02_rb,7.6811e-02_rb,7.6114e-02_rb/)
      kbo(:,17, 3) = (/ &
     &6.5446e-02_rb,6.4598e-02_rb,6.3955e-02_rb,6.3795e-02_rb,6.3695e-02_rb/)
      kbo(:,18, 3) = (/ &
     &5.3197e-02_rb,5.2989e-02_rb,5.2755e-02_rb,5.2824e-02_rb,5.2780e-02_rb/)
      kbo(:,19, 3) = (/ &
     &4.3741e-02_rb,4.3850e-02_rb,4.3971e-02_rb,4.3934e-02_rb,4.3215e-02_rb/)
      kbo(:,20, 3) = (/ &
     &3.6203e-02_rb,3.6356e-02_rb,3.6481e-02_rb,3.6039e-02_rb,3.5482e-02_rb/)
      kbo(:,21, 3) = (/ &
     &2.9983e-02_rb,3.0109e-02_rb,3.0136e-02_rb,2.9600e-02_rb,2.9205e-02_rb/)
      kbo(:,22, 3) = (/ &
     &2.4798e-02_rb,2.4802e-02_rb,2.4628e-02_rb,2.4239e-02_rb,2.4061e-02_rb/)
      kbo(:,23, 3) = (/ &
     &2.0477e-02_rb,2.0405e-02_rb,2.0149e-02_rb,1.9990e-02_rb,1.9997e-02_rb/)
      kbo(:,24, 3) = (/ &
     &1.6953e-02_rb,1.6782e-02_rb,1.6632e-02_rb,1.6618e-02_rb,1.6673e-02_rb/)
      kbo(:,25, 3) = (/ &
     &1.3995e-02_rb,1.3802e-02_rb,1.3748e-02_rb,1.3853e-02_rb,1.3849e-02_rb/)
      kbo(:,26, 3) = (/ &
     &1.1507e-02_rb,1.1388e-02_rb,1.1372e-02_rb,1.1472e-02_rb,1.1479e-02_rb/)
      kbo(:,27, 3) = (/ &
     &9.4100e-03_rb,9.3552e-03_rb,9.3927e-03_rb,9.4480e-03_rb,9.4683e-03_rb/)
      kbo(:,28, 3) = (/ &
     &7.6709e-03_rb,7.6555e-03_rb,7.7364e-03_rb,7.7513e-03_rb,7.7691e-03_rb/)
      kbo(:,29, 3) = (/ &
     &6.2318e-03_rb,6.2179e-03_rb,6.2783e-03_rb,6.2823e-03_rb,6.2984e-03_rb/)
      kbo(:,30, 3) = (/ &
     &5.0369e-03_rb,5.0223e-03_rb,5.0690e-03_rb,5.0675e-03_rb,5.0877e-03_rb/)
      kbo(:,31, 3) = (/ &
     &4.0270e-03_rb,4.0248e-03_rb,4.0530e-03_rb,4.0550e-03_rb,4.0668e-03_rb/)
      kbo(:,32, 3) = (/ &
     &3.2266e-03_rb,3.2233e-03_rb,3.2444e-03_rb,3.2484e-03_rb,3.2485e-03_rb/)
      kbo(:,33, 3) = (/ &
     &2.5826e-03_rb,2.5793e-03_rb,2.5980e-03_rb,2.5954e-03_rb,2.5860e-03_rb/)
      kbo(:,34, 3) = (/ &
     &2.0751e-03_rb,2.0785e-03_rb,2.0913e-03_rb,2.0819e-03_rb,2.0719e-03_rb/)
      kbo(:,35, 3) = (/ &
     &1.6732e-03_rb,1.6746e-03_rb,1.6840e-03_rb,1.6712e-03_rb,1.6630e-03_rb/)
      kbo(:,36, 3) = (/ &
     &1.3540e-03_rb,1.3534e-03_rb,1.3539e-03_rb,1.3458e-03_rb,1.3380e-03_rb/)
      kbo(:,37, 3) = (/ &
     &1.1025e-03_rb,1.0986e-03_rb,1.0965e-03_rb,1.0906e-03_rb,1.0838e-03_rb/)
      kbo(:,38, 3) = (/ &
     &8.9827e-04_rb,8.9144e-04_rb,8.8894e-04_rb,8.8260e-04_rb,8.7601e-04_rb/)
      kbo(:,39, 3) = (/ &
     &7.3039e-04_rb,7.2351e-04_rb,7.1956e-04_rb,7.1446e-04_rb,7.0735e-04_rb/)
      kbo(:,40, 3) = (/ &
     &6.0257e-04_rb,5.9335e-04_rb,5.8899e-04_rb,5.8497e-04_rb,5.7809e-04_rb/)
      kbo(:,41, 3) = (/ &
     &4.9748e-04_rb,4.8829e-04_rb,4.8330e-04_rb,4.7897e-04_rb,4.7418e-04_rb/)
      kbo(:,42, 3) = (/ &
     &4.0962e-04_rb,4.0192e-04_rb,3.9631e-04_rb,3.9272e-04_rb,3.8922e-04_rb/)
      kbo(:,43, 3) = (/ &
     &3.3511e-04_rb,3.3335e-04_rb,3.2739e-04_rb,3.2309e-04_rb,3.2009e-04_rb/)
      kbo(:,44, 3) = (/ &
     &2.7494e-04_rb,2.7440e-04_rb,2.7166e-04_rb,2.6681e-04_rb,2.6291e-04_rb/)
      kbo(:,45, 3) = (/ &
     &2.2409e-04_rb,2.2465e-04_rb,2.2546e-04_rb,2.2120e-04_rb,2.1688e-04_rb/)
      kbo(:,46, 3) = (/ &
     &1.8344e-04_rb,1.8445e-04_rb,1.8370e-04_rb,1.8435e-04_rb,1.8060e-04_rb/)
      kbo(:,47, 3) = (/ &
     &1.5220e-04_rb,1.5094e-04_rb,1.5093e-04_rb,1.5107e-04_rb,1.5073e-04_rb/)
      kbo(:,48, 3) = (/ &
     &1.2685e-04_rb,1.2406e-04_rb,1.2418e-04_rb,1.2347e-04_rb,1.2407e-04_rb/)
      kbo(:,49, 3) = (/ &
     &1.0283e-04_rb,1.0360e-04_rb,1.0186e-04_rb,1.0181e-04_rb,1.0123e-04_rb/)
      kbo(:,50, 3) = (/ &
     &8.3231e-05_rb,8.5593e-05_rb,8.4616e-05_rb,8.3839e-05_rb,8.3224e-05_rb/)
      kbo(:,51, 3) = (/ &
     &6.6892e-05_rb,7.0153e-05_rb,7.0699e-05_rb,6.9691e-05_rb,6.9167e-05_rb/)
      kbo(:,52, 3) = (/ &
     &5.4090e-05_rb,5.6975e-05_rb,5.8223e-05_rb,5.8345e-05_rb,5.7364e-05_rb/)
      kbo(:,53, 3) = (/ &
     &4.3363e-05_rb,4.5702e-05_rb,4.7865e-05_rb,4.7887e-05_rb,4.7945e-05_rb/)
      kbo(:,54, 3) = (/ &
     &3.4722e-05_rb,3.7579e-05_rb,3.9122e-05_rb,3.9883e-05_rb,3.9754e-05_rb/)
      kbo(:,55, 3) = (/ &
     &2.7984e-05_rb,3.0530e-05_rb,3.1930e-05_rb,3.3227e-05_rb,3.3134e-05_rb/)
      kbo(:,56, 3) = (/ &
     &2.2565e-05_rb,2.4737e-05_rb,2.6495e-05_rb,2.7386e-05_rb,2.7845e-05_rb/)
      kbo(:,57, 3) = (/ &
     &1.8624e-05_rb,1.9996e-05_rb,2.1604e-05_rb,2.2454e-05_rb,2.3200e-05_rb/)
      kbo(:,58, 3) = (/ &
     &1.5567e-05_rb,1.6215e-05_rb,1.7630e-05_rb,1.8795e-05_rb,1.9192e-05_rb/)
      kbo(:,59, 3) = (/ &
     &1.3180e-05_rb,1.3591e-05_rb,1.4748e-05_rb,1.5698e-05_rb,1.6153e-05_rb/)
      kbo(:,13, 4) = (/ &
     &2.3244e-01_rb,2.2666e-01_rb,2.2365e-01_rb,2.1647e-01_rb,2.0828e-01_rb/)
      kbo(:,14, 4) = (/ &
     &1.9358e-01_rb,1.8830e-01_rb,1.8743e-01_rb,1.8019e-01_rb,1.7506e-01_rb/)
      kbo(:,15, 4) = (/ &
     &1.6047e-01_rb,1.5782e-01_rb,1.5488e-01_rb,1.5178e-01_rb,1.4896e-01_rb/)
      kbo(:,16, 4) = (/ &
     &1.3221e-01_rb,1.3030e-01_rb,1.2930e-01_rb,1.2731e-01_rb,1.2502e-01_rb/)
      kbo(:,17, 4) = (/ &
     &1.0863e-01_rb,1.0862e-01_rb,1.0815e-01_rb,1.0706e-01_rb,1.0464e-01_rb/)
      kbo(:,18, 4) = (/ &
     &9.0031e-02_rb,9.0210e-02_rb,9.0179e-02_rb,8.8891e-02_rb,8.7072e-02_rb/)
      kbo(:,19, 4) = (/ &
     &7.4933e-02_rb,7.5517e-02_rb,7.5432e-02_rb,7.4716e-02_rb,7.4259e-02_rb/)
      kbo(:,20, 4) = (/ &
     &6.2292e-02_rb,6.2900e-02_rb,6.2796e-02_rb,6.2799e-02_rb,6.2655e-02_rb/)
      kbo(:,21, 4) = (/ &
     &5.1753e-02_rb,5.2259e-02_rb,5.2338e-02_rb,5.2648e-02_rb,5.2692e-02_rb/)
      kbo(:,22, 4) = (/ &
     &4.2879e-02_rb,4.3337e-02_rb,4.3671e-02_rb,4.3911e-02_rb,4.3964e-02_rb/)
      kbo(:,23, 4) = (/ &
     &3.5511e-02_rb,3.5957e-02_rb,3.6400e-02_rb,3.6560e-02_rb,3.6588e-02_rb/)
      kbo(:,24, 4) = (/ &
     &2.9490e-02_rb,3.0002e-02_rb,3.0363e-02_rb,3.0503e-02_rb,3.0607e-02_rb/)
      kbo(:,25, 4) = (/ &
     &2.4491e-02_rb,2.5028e-02_rb,2.5370e-02_rb,2.5485e-02_rb,2.5616e-02_rb/)
      kbo(:,26, 4) = (/ &
     &2.0405e-02_rb,2.0858e-02_rb,2.1166e-02_rb,2.1267e-02_rb,2.1358e-02_rb/)
      kbo(:,27, 4) = (/ &
     &1.6966e-02_rb,1.7305e-02_rb,1.7522e-02_rb,1.7603e-02_rb,1.7664e-02_rb/)
      kbo(:,28, 4) = (/ &
     &1.4041e-02_rb,1.4283e-02_rb,1.4398e-02_rb,1.4482e-02_rb,1.4540e-02_rb/)
      kbo(:,29, 4) = (/ &
     &1.1473e-02_rb,1.1643e-02_rb,1.1712e-02_rb,1.1784e-02_rb,1.1808e-02_rb/)
      kbo(:,30, 4) = (/ &
     &9.3128e-03_rb,9.4393e-03_rb,9.4816e-03_rb,9.5274e-03_rb,9.5277e-03_rb/)
      kbo(:,31, 4) = (/ &
     &7.5018e-03_rb,7.5697e-03_rb,7.5938e-03_rb,7.6234e-03_rb,7.6068e-03_rb/)
      kbo(:,32, 4) = (/ &
     &6.0276e-03_rb,6.0587e-03_rb,6.0760e-03_rb,6.0793e-03_rb,6.0890e-03_rb/)
      kbo(:,33, 4) = (/ &
     &4.8317e-03_rb,4.8465e-03_rb,4.8480e-03_rb,4.8590e-03_rb,4.8679e-03_rb/)
      kbo(:,34, 4) = (/ &
     &3.8930e-03_rb,3.8936e-03_rb,3.8908e-03_rb,3.9081e-03_rb,3.9148e-03_rb/)
      kbo(:,35, 4) = (/ &
     &3.1327e-03_rb,3.1329e-03_rb,3.1313e-03_rb,3.1437e-03_rb,3.1485e-03_rb/)
      kbo(:,36, 4) = (/ &
     &2.5288e-03_rb,2.5255e-03_rb,2.5228e-03_rb,2.5303e-03_rb,2.5349e-03_rb/)
      kbo(:,37, 4) = (/ &
     &2.0481e-03_rb,2.0483e-03_rb,2.0398e-03_rb,2.0467e-03_rb,2.0499e-03_rb/)
      kbo(:,38, 4) = (/ &
     &1.6558e-03_rb,1.6596e-03_rb,1.6492e-03_rb,1.6534e-03_rb,1.6569e-03_rb/)
      kbo(:,39, 4) = (/ &
     &1.3374e-03_rb,1.3400e-03_rb,1.3351e-03_rb,1.3347e-03_rb,1.3383e-03_rb/)
      kbo(:,40, 4) = (/ &
     &1.0845e-03_rb,1.0921e-03_rb,1.0887e-03_rb,1.0875e-03_rb,1.0897e-03_rb/)
      kbo(:,41, 4) = (/ &
     &8.8154e-04_rb,8.8864e-04_rb,8.8779e-04_rb,8.8674e-04_rb,8.8749e-04_rb/)
      kbo(:,42, 4) = (/ &
     &7.1843e-04_rb,7.2263e-04_rb,7.2341e-04_rb,7.2250e-04_rb,7.2207e-04_rb/)
      kbo(:,43, 4) = (/ &
     &5.8799e-04_rb,5.8635e-04_rb,5.8966e-04_rb,5.8857e-04_rb,5.8778e-04_rb/)
      kbo(:,44, 4) = (/ &
     &4.7968e-04_rb,4.7905e-04_rb,4.7843e-04_rb,4.7895e-04_rb,4.7900e-04_rb/)
      kbo(:,45, 4) = (/ &
     &3.9220e-04_rb,3.9274e-04_rb,3.8844e-04_rb,3.8841e-04_rb,3.8937e-04_rb/)
      kbo(:,46, 4) = (/ &
     &3.2002e-04_rb,3.2007e-04_rb,3.1928e-04_rb,3.1499e-04_rb,3.1489e-04_rb/)
      kbo(:,47, 4) = (/ &
     &2.6159e-04_rb,2.6272e-04_rb,2.6214e-04_rb,2.5941e-04_rb,2.5552e-04_rb/)
      kbo(:,48, 4) = (/ &
     &2.1397e-04_rb,2.1500e-04_rb,2.1426e-04_rb,2.1370e-04_rb,2.0956e-04_rb/)
      kbo(:,49, 4) = (/ &
     &1.7795e-04_rb,1.7554e-04_rb,1.7551e-04_rb,1.7452e-04_rb,1.7328e-04_rb/)
      kbo(:,50, 4) = (/ &
     &1.4762e-04_rb,1.4610e-04_rb,1.4428e-04_rb,1.4351e-04_rb,1.4273e-04_rb/)
      kbo(:,51, 4) = (/ &
     &1.2354e-04_rb,1.2144e-04_rb,1.1938e-04_rb,1.1805e-04_rb,1.1688e-04_rb/)
      kbo(:,52, 4) = (/ &
     &1.0353e-04_rb,1.0111e-04_rb,9.9958e-05_rb,9.7469e-05_rb,9.6574e-05_rb/)
      kbo(:,53, 4) = (/ &
     &8.8311e-05_rb,8.4522e-05_rb,8.3164e-05_rb,8.1847e-05_rb,7.9623e-05_rb/)
      kbo(:,54, 4) = (/ &
     &7.5002e-05_rb,7.1277e-05_rb,6.9429e-05_rb,6.8546e-05_rb,6.7044e-05_rb/)
      kbo(:,55, 4) = (/ &
     &6.4688e-05_rb,6.1558e-05_rb,5.8597e-05_rb,5.7310e-05_rb,5.6741e-05_rb/)
      kbo(:,56, 4) = (/ &
     &5.4545e-05_rb,5.2835e-05_rb,4.9722e-05_rb,4.8222e-05_rb,4.7437e-05_rb/)
      kbo(:,57, 4) = (/ &
     &4.5567e-05_rb,4.5770e-05_rb,4.3541e-05_rb,4.1141e-05_rb,3.9867e-05_rb/)
      kbo(:,58, 4) = (/ &
     &3.8423e-05_rb,3.8314e-05_rb,3.7767e-05_rb,3.5438e-05_rb,3.3984e-05_rb/)
      kbo(:,59, 4) = (/ &
     &3.2695e-05_rb,3.2545e-05_rb,3.2199e-05_rb,3.0741e-05_rb,2.9151e-05_rb/)
      kbo(:,13, 5) = (/ &
     &3.6693e-01_rb,3.6670e-01_rb,3.6374e-01_rb,3.6513e-01_rb,3.6694e-01_rb/)
      kbo(:,14, 5) = (/ &
     &3.1193e-01_rb,3.1009e-01_rb,3.0785e-01_rb,3.1108e-01_rb,3.1338e-01_rb/)
      kbo(:,15, 5) = (/ &
     &2.6635e-01_rb,2.6398e-01_rb,2.6603e-01_rb,2.6792e-01_rb,2.6974e-01_rb/)
      kbo(:,16, 5) = (/ &
     &2.2560e-01_rb,2.2536e-01_rb,2.2722e-01_rb,2.2790e-01_rb,2.2934e-01_rb/)
      kbo(:,17, 5) = (/ &
     &1.9102e-01_rb,1.9205e-01_rb,1.9235e-01_rb,1.9256e-01_rb,1.9454e-01_rb/)
      kbo(:,18, 5) = (/ &
     &1.6220e-01_rb,1.6268e-01_rb,1.6247e-01_rb,1.6354e-01_rb,1.6555e-01_rb/)
      kbo(:,19, 5) = (/ &
     &1.3796e-01_rb,1.3803e-01_rb,1.3823e-01_rb,1.3977e-01_rb,1.4117e-01_rb/)
      kbo(:,20, 5) = (/ &
     &1.1636e-01_rb,1.1631e-01_rb,1.1719e-01_rb,1.1821e-01_rb,1.1915e-01_rb/)
      kbo(:,21, 5) = (/ &
     &9.7502e-02_rb,9.7703e-02_rb,9.8615e-02_rb,9.9561e-02_rb,1.0025e-01_rb/)
      kbo(:,22, 5) = (/ &
     &8.1562e-02_rb,8.1981e-02_rb,8.2659e-02_rb,8.3450e-02_rb,8.3918e-02_rb/)
      kbo(:,23, 5) = (/ &
     &6.8080e-02_rb,6.8683e-02_rb,6.9366e-02_rb,6.9894e-02_rb,7.0282e-02_rb/)
      kbo(:,24, 5) = (/ &
     &5.7119e-02_rb,5.7682e-02_rb,5.8192e-02_rb,5.8547e-02_rb,5.9029e-02_rb/)
      kbo(:,25, 5) = (/ &
     &4.7811e-02_rb,4.8293e-02_rb,4.8606e-02_rb,4.8996e-02_rb,4.9473e-02_rb/)
      kbo(:,26, 5) = (/ &
     &3.9890e-02_rb,4.0316e-02_rb,4.0576e-02_rb,4.1016e-02_rb,4.1455e-02_rb/)
      kbo(:,27, 5) = (/ &
     &3.3117e-02_rb,3.3470e-02_rb,3.3770e-02_rb,3.4189e-02_rb,3.4527e-02_rb/)
      kbo(:,28, 5) = (/ &
     &2.7406e-02_rb,2.7694e-02_rb,2.7971e-02_rb,2.8332e-02_rb,2.8599e-02_rb/)
      kbo(:,29, 5) = (/ &
     &2.2401e-02_rb,2.2630e-02_rb,2.2890e-02_rb,2.3150e-02_rb,2.3396e-02_rb/)
      kbo(:,30, 5) = (/ &
     &1.8227e-02_rb,1.8399e-02_rb,1.8623e-02_rb,1.8850e-02_rb,1.9028e-02_rb/)
      kbo(:,31, 5) = (/ &
     &1.4683e-02_rb,1.4830e-02_rb,1.5011e-02_rb,1.5187e-02_rb,1.5326e-02_rb/)
      kbo(:,32, 5) = (/ &
     &1.1826e-02_rb,1.1971e-02_rb,1.2101e-02_rb,1.2245e-02_rb,1.2313e-02_rb/)
      kbo(:,33, 5) = (/ &
     &9.5307e-03_rb,9.6419e-03_rb,9.7421e-03_rb,9.8284e-03_rb,9.8826e-03_rb/)
      kbo(:,34, 5) = (/ &
     &7.7176e-03_rb,7.8031e-03_rb,7.8838e-03_rb,7.9405e-03_rb,7.9679e-03_rb/)
      kbo(:,35, 5) = (/ &
     &6.2512e-03_rb,6.3124e-03_rb,6.3712e-03_rb,6.4126e-03_rb,6.4324e-03_rb/)
      kbo(:,36, 5) = (/ &
     &5.0560e-03_rb,5.1076e-03_rb,5.1576e-03_rb,5.1918e-03_rb,5.2018e-03_rb/)
      kbo(:,37, 5) = (/ &
     &4.0965e-03_rb,4.1427e-03_rb,4.1869e-03_rb,4.2164e-03_rb,4.2222e-03_rb/)
      kbo(:,38, 5) = (/ &
     &3.3169e-03_rb,3.3541e-03_rb,3.3940e-03_rb,3.4174e-03_rb,3.4236e-03_rb/)
      kbo(:,39, 5) = (/ &
     &2.6806e-03_rb,2.7157e-03_rb,2.7440e-03_rb,2.7658e-03_rb,2.7739e-03_rb/)
      kbo(:,40, 5) = (/ &
     &2.1766e-03_rb,2.2105e-03_rb,2.2356e-03_rb,2.2533e-03_rb,2.2654e-03_rb/)
      kbo(:,41, 5) = (/ &
     &1.7690e-03_rb,1.7997e-03_rb,1.8217e-03_rb,1.8373e-03_rb,1.8492e-03_rb/)
      kbo(:,42, 5) = (/ &
     &1.4370e-03_rb,1.4625e-03_rb,1.4833e-03_rb,1.4970e-03_rb,1.5072e-03_rb/)
      kbo(:,43, 5) = (/ &
     &1.1671e-03_rb,1.1863e-03_rb,1.2051e-03_rb,1.2193e-03_rb,1.2276e-03_rb/)
      kbo(:,44, 5) = (/ &
     &9.4820e-04_rb,9.6187e-04_rb,9.7830e-04_rb,9.9143e-04_rb,9.9963e-04_rb/)
      kbo(:,45, 5) = (/ &
     &7.6916e-04_rb,7.7831e-04_rb,7.9153e-04_rb,8.0495e-04_rb,8.1281e-04_rb/)
      kbo(:,46, 5) = (/ &
     &6.2348e-04_rb,6.3114e-04_rb,6.4009e-04_rb,6.5199e-04_rb,6.6002e-04_rb/)
      kbo(:,47, 5) = (/ &
     &5.0390e-04_rb,5.1138e-04_rb,5.1807e-04_rb,5.2655e-04_rb,5.3606e-04_rb/)
      kbo(:,48, 5) = (/ &
     &4.0606e-04_rb,4.1379e-04_rb,4.1952e-04_rb,4.2537e-04_rb,4.3314e-04_rb/)
      kbo(:,49, 5) = (/ &
     &3.2659e-04_rb,3.3285e-04_rb,3.3902e-04_rb,3.4315e-04_rb,3.4880e-04_rb/)
      kbo(:,50, 5) = (/ &
     &2.6466e-04_rb,2.6781e-04_rb,2.7393e-04_rb,2.7826e-04_rb,2.8216e-04_rb/)
      kbo(:,51, 5) = (/ &
     &2.1507e-04_rb,2.1721e-04_rb,2.2109e-04_rb,2.2570e-04_rb,2.2911e-04_rb/)
      kbo(:,52, 5) = (/ &
     &1.7470e-04_rb,1.7639e-04_rb,1.7813e-04_rb,1.8197e-04_rb,1.8512e-04_rb/)
      kbo(:,53, 5) = (/ &
     &1.4115e-04_rb,1.4348e-04_rb,1.4425e-04_rb,1.4633e-04_rb,1.4939e-04_rb/)
      kbo(:,54, 5) = (/ &
     &1.1575e-04_rb,1.1704e-04_rb,1.1812e-04_rb,1.1890e-04_rb,1.2106e-04_rb/)
      kbo(:,55, 5) = (/ &
     &9.4726e-05_rb,9.5455e-05_rb,9.6825e-05_rb,9.7465e-05_rb,9.8412e-05_rb/)
      kbo(:,56, 5) = (/ &
     &7.9719e-05_rb,7.8475e-05_rb,7.9571e-05_rb,8.0057e-05_rb,8.0565e-05_rb/)
      kbo(:,57, 5) = (/ &
     &6.7373e-05_rb,6.4174e-05_rb,6.4600e-05_rb,6.5492e-05_rb,6.6130e-05_rb/)
      kbo(:,58, 5) = (/ &
     &5.8248e-05_rb,5.4990e-05_rb,5.3012e-05_rb,5.3610e-05_rb,5.4283e-05_rb/)
      kbo(:,59, 5) = (/ &
     &5.0210e-05_rb,4.6885e-05_rb,4.4678e-05_rb,4.4690e-05_rb,4.5330e-05_rb/)
      kbo(:,13, 6) = (/ &
     &8.1815e-01_rb,8.2185e-01_rb,8.2056e-01_rb,8.1693e-01_rb,8.1119e-01_rb/)
      kbo(:,14, 6) = (/ &
     &6.9938e-01_rb,7.0403e-01_rb,7.0310e-01_rb,6.9893e-01_rb,6.9382e-01_rb/)
      kbo(:,15, 6) = (/ &
     &6.0306e-01_rb,6.0562e-01_rb,6.0433e-01_rb,6.0124e-01_rb,5.9539e-01_rb/)
      kbo(:,16, 6) = (/ &
     &5.1717e-01_rb,5.1741e-01_rb,5.1602e-01_rb,5.1303e-01_rb,5.0799e-01_rb/)
      kbo(:,17, 6) = (/ &
     &4.3954e-01_rb,4.3996e-01_rb,4.3917e-01_rb,4.3585e-01_rb,4.3144e-01_rb/)
      kbo(:,18, 6) = (/ &
     &3.7098e-01_rb,3.7164e-01_rb,3.7046e-01_rb,3.6778e-01_rb,3.6412e-01_rb/)
      kbo(:,19, 6) = (/ &
     &3.1410e-01_rb,3.1452e-01_rb,3.1318e-01_rb,3.1054e-01_rb,3.0829e-01_rb/)
      kbo(:,20, 6) = (/ &
     &2.6425e-01_rb,2.6434e-01_rb,2.6287e-01_rb,2.6129e-01_rb,2.6054e-01_rb/)
      kbo(:,21, 6) = (/ &
     &2.2166e-01_rb,2.2161e-01_rb,2.2078e-01_rb,2.2017e-01_rb,2.1929e-01_rb/)
      kbo(:,22, 6) = (/ &
     &1.8499e-01_rb,1.8512e-01_rb,1.8480e-01_rb,1.8474e-01_rb,1.8395e-01_rb/)
      kbo(:,23, 6) = (/ &
     &1.5462e-01_rb,1.5460e-01_rb,1.5448e-01_rb,1.5440e-01_rb,1.5366e-01_rb/)
      kbo(:,24, 6) = (/ &
     &1.2929e-01_rb,1.2940e-01_rb,1.2953e-01_rb,1.2932e-01_rb,1.2872e-01_rb/)
      kbo(:,25, 6) = (/ &
     &1.0802e-01_rb,1.0819e-01_rb,1.0834e-01_rb,1.0817e-01_rb,1.0761e-01_rb/)
      kbo(:,26, 6) = (/ &
     &9.0060e-02_rb,9.0350e-02_rb,9.0334e-02_rb,9.0150e-02_rb,8.9781e-02_rb/)
      kbo(:,27, 6) = (/ &
     &7.4680e-02_rb,7.4863e-02_rb,7.4908e-02_rb,7.4831e-02_rb,7.4608e-02_rb/)
      kbo(:,28, 6) = (/ &
     &6.1687e-02_rb,6.1865e-02_rb,6.1970e-02_rb,6.1917e-02_rb,6.1874e-02_rb/)
      kbo(:,29, 6) = (/ &
     &5.0513e-02_rb,5.0701e-02_rb,5.0801e-02_rb,5.0773e-02_rb,5.0734e-02_rb/)
      kbo(:,30, 6) = (/ &
     &4.1201e-02_rb,4.1379e-02_rb,4.1449e-02_rb,4.1422e-02_rb,4.1408e-02_rb/)
      kbo(:,31, 6) = (/ &
     &3.3339e-02_rb,3.3480e-02_rb,3.3517e-02_rb,3.3503e-02_rb,3.3496e-02_rb/)
      kbo(:,32, 6) = (/ &
     &2.6941e-02_rb,2.7025e-02_rb,2.7045e-02_rb,2.7032e-02_rb,2.7041e-02_rb/)
      kbo(:,33, 6) = (/ &
     &2.1746e-02_rb,2.1784e-02_rb,2.1791e-02_rb,2.1788e-02_rb,2.1780e-02_rb/)
      kbo(:,34, 6) = (/ &
     &1.7618e-02_rb,1.7652e-02_rb,1.7653e-02_rb,1.7651e-02_rb,1.7645e-02_rb/)
      kbo(:,35, 6) = (/ &
     &1.4278e-02_rb,1.4314e-02_rb,1.4305e-02_rb,1.4306e-02_rb,1.4294e-02_rb/)
      kbo(:,36, 6) = (/ &
     &1.1580e-02_rb,1.1612e-02_rb,1.1606e-02_rb,1.1608e-02_rb,1.1605e-02_rb/)
      kbo(:,37, 6) = (/ &
     &9.4219e-03_rb,9.4537e-03_rb,9.4624e-03_rb,9.4565e-03_rb,9.4577e-03_rb/)
      kbo(:,38, 6) = (/ &
     &7.6536e-03_rb,7.6870e-03_rb,7.7033e-03_rb,7.6954e-03_rb,7.6994e-03_rb/)
      kbo(:,39, 6) = (/ &
     &6.2054e-03_rb,6.2357e-03_rb,6.2573e-03_rb,6.2573e-03_rb,6.2600e-03_rb/)
      kbo(:,40, 6) = (/ &
     &5.0787e-03_rb,5.1000e-03_rb,5.1231e-03_rb,5.1304e-03_rb,5.1270e-03_rb/)
      kbo(:,41, 6) = (/ &
     &4.1553e-03_rb,4.1734e-03_rb,4.1950e-03_rb,4.2100e-03_rb,4.2073e-03_rb/)
      kbo(:,42, 6) = (/ &
     &3.3951e-03_rb,3.4115e-03_rb,3.4321e-03_rb,3.4456e-03_rb,3.4459e-03_rb/)
      kbo(:,43, 6) = (/ &
     &2.7739e-03_rb,2.7892e-03_rb,2.8063e-03_rb,2.8198e-03_rb,2.8248e-03_rb/)
      kbo(:,44, 6) = (/ &
     &2.2638e-03_rb,2.2784e-03_rb,2.2918e-03_rb,2.3046e-03_rb,2.3113e-03_rb/)
      kbo(:,45, 6) = (/ &
     &1.8451e-03_rb,1.8580e-03_rb,1.8678e-03_rb,1.8806e-03_rb,1.8860e-03_rb/)
      kbo(:,46, 6) = (/ &
     &1.5013e-03_rb,1.5134e-03_rb,1.5237e-03_rb,1.5316e-03_rb,1.5391e-03_rb/)
      kbo(:,47, 6) = (/ &
     &1.2205e-03_rb,1.2340e-03_rb,1.2429e-03_rb,1.2503e-03_rb,1.2568e-03_rb/)
      kbo(:,48, 6) = (/ &
     &9.8959e-04_rb,1.0036e-03_rb,1.0115e-03_rb,1.0182e-03_rb,1.0235e-03_rb/)
      kbo(:,49, 6) = (/ &
     &8.0128e-04_rb,8.1327e-04_rb,8.2211e-04_rb,8.2807e-04_rb,8.3181e-04_rb/)
      kbo(:,50, 6) = (/ &
     &6.5077e-04_rb,6.6049e-04_rb,6.6942e-04_rb,6.7493e-04_rb,6.7909e-04_rb/)
      kbo(:,51, 6) = (/ &
     &5.2860e-04_rb,5.3765e-04_rb,5.4562e-04_rb,5.5099e-04_rb,5.5519e-04_rb/)
      kbo(:,52, 6) = (/ &
     &4.2741e-04_rb,4.3678e-04_rb,4.4379e-04_rb,4.4944e-04_rb,4.5321e-04_rb/)
      kbo(:,53, 6) = (/ &
     &3.4420e-04_rb,3.5368e-04_rb,3.5979e-04_rb,3.6513e-04_rb,3.6858e-04_rb/)
      kbo(:,54, 6) = (/ &
     &2.7911e-04_rb,2.8752e-04_rb,2.9393e-04_rb,2.9865e-04_rb,3.0238e-04_rb/)
      kbo(:,55, 6) = (/ &
     &2.2659e-04_rb,2.3415e-04_rb,2.4084e-04_rb,2.4534e-04_rb,2.4896e-04_rb/)
      kbo(:,56, 6) = (/ &
     &1.8317e-04_rb,1.9078e-04_rb,1.9656e-04_rb,2.0135e-04_rb,2.0466e-04_rb/)
      kbo(:,57, 6) = (/ &
     &1.4828e-04_rb,1.5527e-04_rb,1.6035e-04_rb,1.6503e-04_rb,1.6825e-04_rb/)
      kbo(:,58, 6) = (/ &
     &1.1818e-04_rb,1.2542e-04_rb,1.3082e-04_rb,1.3476e-04_rb,1.3831e-04_rb/)
      kbo(:,59, 6) = (/ &
     &9.6673e-05_rb,1.0368e-04_rb,1.0861e-04_rb,1.1211e-04_rb,1.1553e-04_rb/)
      kbo(:,13, 7) = (/ &
     &2.1727e+00_rb,2.1507e+00_rb,2.1220e+00_rb,2.0892e+00_rb,2.0541e+00_rb/)
      kbo(:,14, 7) = (/ &
     &1.7980e+00_rb,1.7753e+00_rb,1.7493e+00_rb,1.7241e+00_rb,1.6941e+00_rb/)
      kbo(:,15, 7) = (/ &
     &1.5327e+00_rb,1.5148e+00_rb,1.4951e+00_rb,1.4705e+00_rb,1.4480e+00_rb/)
      kbo(:,16, 7) = (/ &
     &1.3007e+00_rb,1.2909e+00_rb,1.2720e+00_rb,1.2527e+00_rb,1.2359e+00_rb/)
      kbo(:,17, 7) = (/ &
     &1.1117e+00_rb,1.1002e+00_rb,1.0859e+00_rb,1.0728e+00_rb,1.0608e+00_rb/)
      kbo(:,18, 7) = (/ &
     &9.4895e-01_rb,9.3669e-01_rb,9.2658e-01_rb,9.1630e-01_rb,9.0461e-01_rb/)
      kbo(:,19, 7) = (/ &
     &8.0771e-01_rb,7.9960e-01_rb,7.9052e-01_rb,7.8134e-01_rb,7.7103e-01_rb/)
      kbo(:,20, 7) = (/ &
     &6.8397e-01_rb,6.7733e-01_rb,6.6931e-01_rb,6.6104e-01_rb,6.5276e-01_rb/)
      kbo(:,21, 7) = (/ &
     &5.7680e-01_rb,5.7078e-01_rb,5.6416e-01_rb,5.5706e-01_rb,5.4996e-01_rb/)
      kbo(:,22, 7) = (/ &
     &4.8402e-01_rb,4.7901e-01_rb,4.7292e-01_rb,4.6701e-01_rb,4.6224e-01_rb/)
      kbo(:,23, 7) = (/ &
     &4.0435e-01_rb,4.0004e-01_rb,3.9588e-01_rb,3.9219e-01_rb,3.8835e-01_rb/)
      kbo(:,24, 7) = (/ &
     &3.3766e-01_rb,3.3478e-01_rb,3.3178e-01_rb,3.2887e-01_rb,3.2604e-01_rb/)
      kbo(:,25, 7) = (/ &
     &2.8141e-01_rb,2.7941e-01_rb,2.7716e-01_rb,2.7469e-01_rb,2.7266e-01_rb/)
      kbo(:,26, 7) = (/ &
     &2.3429e-01_rb,2.3260e-01_rb,2.3087e-01_rb,2.2897e-01_rb,2.2751e-01_rb/)
      kbo(:,27, 7) = (/ &
     &1.9420e-01_rb,1.9275e-01_rb,1.9122e-01_rb,1.8991e-01_rb,1.8923e-01_rb/)
      kbo(:,28, 7) = (/ &
     &1.6015e-01_rb,1.5905e-01_rb,1.5800e-01_rb,1.5718e-01_rb,1.5696e-01_rb/)
      kbo(:,29, 7) = (/ &
     &1.3094e-01_rb,1.3014e-01_rb,1.2944e-01_rb,1.2906e-01_rb,1.2899e-01_rb/)
      kbo(:,30, 7) = (/ &
     &1.0666e-01_rb,1.0604e-01_rb,1.0563e-01_rb,1.0549e-01_rb,1.0548e-01_rb/)
      kbo(:,31, 7) = (/ &
     &8.6275e-02_rb,8.5845e-02_rb,8.5617e-02_rb,8.5505e-02_rb,8.5535e-02_rb/)
      kbo(:,32, 7) = (/ &
     &6.9798e-02_rb,6.9528e-02_rb,6.9393e-02_rb,6.9336e-02_rb,6.9374e-02_rb/)
      kbo(:,33, 7) = (/ &
     &5.6418e-02_rb,5.6253e-02_rb,5.6185e-02_rb,5.6126e-02_rb,5.6165e-02_rb/)
      kbo(:,34, 7) = (/ &
     &4.5866e-02_rb,4.5727e-02_rb,4.5682e-02_rb,4.5616e-02_rb,4.5707e-02_rb/)
      kbo(:,35, 7) = (/ &
     &3.7274e-02_rb,3.7180e-02_rb,3.7143e-02_rb,3.7120e-02_rb,3.7200e-02_rb/)
      kbo(:,36, 7) = (/ &
     &3.0350e-02_rb,3.0274e-02_rb,3.0257e-02_rb,3.0235e-02_rb,3.0299e-02_rb/)
      kbo(:,37, 7) = (/ &
     &2.4821e-02_rb,2.4753e-02_rb,2.4742e-02_rb,2.4741e-02_rb,2.4790e-02_rb/)
      kbo(:,38, 7) = (/ &
     &2.0268e-02_rb,2.0223e-02_rb,2.0196e-02_rb,2.0200e-02_rb,2.0236e-02_rb/)
      kbo(:,39, 7) = (/ &
     &1.6521e-02_rb,1.6487e-02_rb,1.6459e-02_rb,1.6462e-02_rb,1.6499e-02_rb/)
      kbo(:,40, 7) = (/ &
     &1.3551e-02_rb,1.3542e-02_rb,1.3531e-02_rb,1.3538e-02_rb,1.3571e-02_rb/)
      kbo(:,41, 7) = (/ &
     &1.1139e-02_rb,1.1135e-02_rb,1.1130e-02_rb,1.1141e-02_rb,1.1171e-02_rb/)
      kbo(:,42, 7) = (/ &
     &9.1492e-03_rb,9.1515e-03_rb,9.1518e-03_rb,9.1649e-03_rb,9.1900e-03_rb/)
      kbo(:,43, 7) = (/ &
     &7.5083e-03_rb,7.5191e-03_rb,7.5258e-03_rb,7.5334e-03_rb,7.5552e-03_rb/)
      kbo(:,44, 7) = (/ &
     &6.1594e-03_rb,6.1707e-03_rb,6.1785e-03_rb,6.1865e-03_rb,6.2090e-03_rb/)
      kbo(:,45, 7) = (/ &
     &5.0436e-03_rb,5.0526e-03_rb,5.0656e-03_rb,5.0711e-03_rb,5.0867e-03_rb/)
      kbo(:,46, 7) = (/ &
     &4.1291e-03_rb,4.1408e-03_rb,4.1483e-03_rb,4.1539e-03_rb,4.1670e-03_rb/)
      kbo(:,47, 7) = (/ &
     &3.3808e-03_rb,3.3903e-03_rb,3.4014e-03_rb,3.4059e-03_rb,3.4179e-03_rb/)
      kbo(:,48, 7) = (/ &
     &2.7630e-03_rb,2.7730e-03_rb,2.7794e-03_rb,2.7876e-03_rb,2.7961e-03_rb/)
      kbo(:,49, 7) = (/ &
     &2.2529e-03_rb,2.2614e-03_rb,2.2681e-03_rb,2.2743e-03_rb,2.2825e-03_rb/)
      kbo(:,50, 7) = (/ &
     &1.8409e-03_rb,1.8507e-03_rb,1.8571e-03_rb,1.8629e-03_rb,1.8694e-03_rb/)
      kbo(:,51, 7) = (/ &
     &1.5059e-03_rb,1.5160e-03_rb,1.5224e-03_rb,1.5287e-03_rb,1.5343e-03_rb/)
      kbo(:,52, 7) = (/ &
     &1.2303e-03_rb,1.2392e-03_rb,1.2461e-03_rb,1.2522e-03_rb,1.2570e-03_rb/)
      kbo(:,53, 7) = (/ &
     &1.0020e-03_rb,1.0115e-03_rb,1.0178e-03_rb,1.0227e-03_rb,1.0281e-03_rb/)
      kbo(:,54, 7) = (/ &
     &8.2130e-04_rb,8.3060e-04_rb,8.3778e-04_rb,8.4304e-04_rb,8.4741e-04_rb/)
      kbo(:,55, 7) = (/ &
     &6.7498e-04_rb,6.8670e-04_rb,6.9292e-04_rb,6.9895e-04_rb,7.0313e-04_rb/)
      kbo(:,56, 7) = (/ &
     &5.5427e-04_rb,5.6524e-04_rb,5.7318e-04_rb,5.7892e-04_rb,5.8355e-04_rb/)
      kbo(:,57, 7) = (/ &
     &4.5437e-04_rb,4.6585e-04_rb,4.7333e-04_rb,4.7899e-04_rb,4.8346e-04_rb/)
      kbo(:,58, 7) = (/ &
     &3.7314e-04_rb,3.8385e-04_rb,3.9164e-04_rb,3.9732e-04_rb,4.0183e-04_rb/)
      kbo(:,59, 7) = (/ &
     &3.1164e-04_rb,3.2163e-04_rb,3.2929e-04_rb,3.3498e-04_rb,3.3967e-04_rb/)
      kbo(:,13, 8) = (/ &
     &7.1017e+00_rb,7.0462e+00_rb,6.9629e+00_rb,6.8569e+00_rb,6.7518e+00_rb/)
      kbo(:,14, 8) = (/ &
     &5.9884e+00_rb,5.9470e+00_rb,5.8924e+00_rb,5.8216e+00_rb,5.7342e+00_rb/)
      kbo(:,15, 8) = (/ &
     &4.9849e+00_rb,4.9563e+00_rb,4.9071e+00_rb,4.8502e+00_rb,4.7771e+00_rb/)
      kbo(:,16, 8) = (/ &
     &4.1453e+00_rb,4.1136e+00_rb,4.0790e+00_rb,4.0345e+00_rb,3.9753e+00_rb/)
      kbo(:,17, 8) = (/ &
     &3.5269e+00_rb,3.5022e+00_rb,3.4717e+00_rb,3.4323e+00_rb,3.3830e+00_rb/)
      kbo(:,18, 8) = (/ &
     &3.0258e+00_rb,3.0139e+00_rb,2.9887e+00_rb,2.9518e+00_rb,2.9096e+00_rb/)
      kbo(:,19, 8) = (/ &
     &2.6100e+00_rb,2.6057e+00_rb,2.5839e+00_rb,2.5531e+00_rb,2.5181e+00_rb/)
      kbo(:,20, 8) = (/ &
     &2.2532e+00_rb,2.2444e+00_rb,2.2242e+00_rb,2.2015e+00_rb,2.1737e+00_rb/)
      kbo(:,21, 8) = (/ &
     &1.9361e+00_rb,1.9257e+00_rb,1.9080e+00_rb,1.8885e+00_rb,1.8663e+00_rb/)
      kbo(:,22, 8) = (/ &
     &1.6493e+00_rb,1.6394e+00_rb,1.6256e+00_rb,1.6084e+00_rb,1.5911e+00_rb/)
      kbo(:,23, 8) = (/ &
     &1.3971e+00_rb,1.3901e+00_rb,1.3774e+00_rb,1.3628e+00_rb,1.3482e+00_rb/)
      kbo(:,24, 8) = (/ &
     &1.1828e+00_rb,1.1757e+00_rb,1.1653e+00_rb,1.1541e+00_rb,1.1420e+00_rb/)
      kbo(:,25, 8) = (/ &
     &9.9841e-01_rb,9.9173e-01_rb,9.8314e-01_rb,9.7419e-01_rb,9.6526e-01_rb/)
      kbo(:,26, 8) = (/ &
     &8.3815e-01_rb,8.3237e-01_rb,8.2631e-01_rb,8.2072e-01_rb,8.1439e-01_rb/)
      kbo(:,27, 8) = (/ &
     &6.9885e-01_rb,6.9534e-01_rb,6.9172e-01_rb,6.8787e-01_rb,6.8202e-01_rb/)
      kbo(:,28, 8) = (/ &
     &5.8056e-01_rb,5.7858e-01_rb,5.7609e-01_rb,5.7252e-01_rb,5.6730e-01_rb/)
      kbo(:,29, 8) = (/ &
     &4.7795e-01_rb,4.7646e-01_rb,4.7447e-01_rb,4.7127e-01_rb,4.6716e-01_rb/)
      kbo(:,30, 8) = (/ &
     &3.9180e-01_rb,3.9040e-01_rb,3.8874e-01_rb,3.8567e-01_rb,3.8256e-01_rb/)
      kbo(:,31, 8) = (/ &
     &3.1850e-01_rb,3.1735e-01_rb,3.1544e-01_rb,3.1324e-01_rb,3.1109e-01_rb/)
      kbo(:,32, 8) = (/ &
     &2.5851e-01_rb,2.5736e-01_rb,2.5590e-01_rb,2.5439e-01_rb,2.5264e-01_rb/)
      kbo(:,33, 8) = (/ &
     &2.0938e-01_rb,2.0841e-01_rb,2.0741e-01_rb,2.0615e-01_rb,2.0501e-01_rb/)
      kbo(:,34, 8) = (/ &
     &1.7037e-01_rb,1.6964e-01_rb,1.6891e-01_rb,1.6812e-01_rb,1.6728e-01_rb/)
      kbo(:,35, 8) = (/ &
     &1.3875e-01_rb,1.3827e-01_rb,1.3776e-01_rb,1.3724e-01_rb,1.3661e-01_rb/)
      kbo(:,36, 8) = (/ &
     &1.1324e-01_rb,1.1288e-01_rb,1.1260e-01_rb,1.1226e-01_rb,1.1182e-01_rb/)
      kbo(:,37, 8) = (/ &
     &9.2766e-02_rb,9.2595e-02_rb,9.2422e-02_rb,9.2186e-02_rb,9.1910e-02_rb/)
      kbo(:,38, 8) = (/ &
     &7.5939e-02_rb,7.5826e-02_rb,7.5761e-02_rb,7.5637e-02_rb,7.5518e-02_rb/)
      kbo(:,39, 8) = (/ &
     &6.2076e-02_rb,6.2014e-02_rb,6.2019e-02_rb,6.2000e-02_rb,6.1957e-02_rb/)
      kbo(:,40, 8) = (/ &
     &5.1067e-02_rb,5.1119e-02_rb,5.1147e-02_rb,5.1217e-02_rb,5.1232e-02_rb/)
      kbo(:,41, 8) = (/ &
     &4.2004e-02_rb,4.2110e-02_rb,4.2194e-02_rb,4.2288e-02_rb,4.2348e-02_rb/)
      kbo(:,42, 8) = (/ &
     &3.4565e-02_rb,3.4686e-02_rb,3.4772e-02_rb,3.4891e-02_rb,3.5001e-02_rb/)
      kbo(:,43, 8) = (/ &
     &2.8439e-02_rb,2.8558e-02_rb,2.8651e-02_rb,2.8781e-02_rb,2.8908e-02_rb/)
      kbo(:,44, 8) = (/ &
     &2.3381e-02_rb,2.3495e-02_rb,2.3607e-02_rb,2.3736e-02_rb,2.3863e-02_rb/)
      kbo(:,45, 8) = (/ &
     &1.9178e-02_rb,1.9302e-02_rb,1.9411e-02_rb,1.9527e-02_rb,1.9645e-02_rb/)
      kbo(:,46, 8) = (/ &
     &1.5718e-02_rb,1.5824e-02_rb,1.5935e-02_rb,1.6052e-02_rb,1.6161e-02_rb/)
      kbo(:,47, 8) = (/ &
     &1.2888e-02_rb,1.2994e-02_rb,1.3091e-02_rb,1.3201e-02_rb,1.3310e-02_rb/)
      kbo(:,48, 8) = (/ &
     &1.0534e-02_rb,1.0644e-02_rb,1.0741e-02_rb,1.0828e-02_rb,1.0922e-02_rb/)
      kbo(:,49, 8) = (/ &
     &8.5773e-03_rb,8.6840e-03_rb,8.7716e-03_rb,8.8561e-03_rb,8.9344e-03_rb/)
      kbo(:,50, 8) = (/ &
     &7.0118e-03_rb,7.1121e-03_rb,7.1954e-03_rb,7.2655e-03_rb,7.3372e-03_rb/)
      kbo(:,51, 8) = (/ &
     &5.7344e-03_rb,5.8326e-03_rb,5.9098e-03_rb,5.9770e-03_rb,6.0423e-03_rb/)
      kbo(:,52, 8) = (/ &
     &4.6784e-03_rb,4.7703e-03_rb,4.8415e-03_rb,4.9016e-03_rb,4.9646e-03_rb/)
      kbo(:,53, 8) = (/ &
     &3.8084e-03_rb,3.8892e-03_rb,3.9565e-03_rb,4.0107e-03_rb,4.0624e-03_rb/)
      kbo(:,54, 8) = (/ &
     &3.1302e-03_rb,3.1988e-03_rb,3.2604e-03_rb,3.3140e-03_rb,3.3627e-03_rb/)
      kbo(:,55, 8) = (/ &
     &2.5815e-03_rb,2.6417e-03_rb,2.7045e-03_rb,2.7548e-03_rb,2.8045e-03_rb/)
      kbo(:,56, 8) = (/ &
     &2.1299e-03_rb,2.1858e-03_rb,2.2412e-03_rb,2.2906e-03_rb,2.3394e-03_rb/)
      kbo(:,57, 8) = (/ &
     &1.7549e-03_rb,1.8058e-03_rb,1.8577e-03_rb,1.9050e-03_rb,1.9496e-03_rb/)
      kbo(:,58, 8) = (/ &
     &1.4508e-03_rb,1.4946e-03_rb,1.5437e-03_rb,1.5867e-03_rb,1.6288e-03_rb/)
      kbo(:,59, 8) = (/ &
     &1.2226e-03_rb,1.2669e-03_rb,1.3143e-03_rb,1.3547e-03_rb,1.3969e-03_rb/)
      kbo(:,13, 9) = (/ &
     &4.2167e+01_rb,4.2169e+01_rb,4.2037e+01_rb,4.1794e+01_rb,4.1453e+01_rb/)
      kbo(:,14, 9) = (/ &
     &3.6302e+01_rb,3.6287e+01_rb,3.6134e+01_rb,3.5895e+01_rb,3.5553e+01_rb/)
      kbo(:,15, 9) = (/ &
     &3.1018e+01_rb,3.0974e+01_rb,3.0831e+01_rb,3.0595e+01_rb,3.0284e+01_rb/)
      kbo(:,16, 9) = (/ &
     &2.6381e+01_rb,2.6327e+01_rb,2.6171e+01_rb,2.5940e+01_rb,2.5630e+01_rb/)
      kbo(:,17, 9) = (/ &
     &2.2159e+01_rb,2.2069e+01_rb,2.1921e+01_rb,2.1709e+01_rb,2.1443e+01_rb/)
      kbo(:,18, 9) = (/ &
     &1.8546e+01_rb,1.8422e+01_rb,1.8249e+01_rb,1.8049e+01_rb,1.7832e+01_rb/)
      kbo(:,19, 9) = (/ &
     &1.5581e+01_rb,1.5418e+01_rb,1.5257e+01_rb,1.5071e+01_rb,1.4880e+01_rb/)
      kbo(:,20, 9) = (/ &
     &1.3153e+01_rb,1.2997e+01_rb,1.2837e+01_rb,1.2663e+01_rb,1.2482e+01_rb/)
      kbo(:,21, 9) = (/ &
     &1.1154e+01_rb,1.1003e+01_rb,1.0876e+01_rb,1.0723e+01_rb,1.0559e+01_rb/)
      kbo(:,22, 9) = (/ &
     &9.6248e+00_rb,9.4985e+00_rb,9.3603e+00_rb,9.2175e+00_rb,9.0746e+00_rb/)
      kbo(:,23, 9) = (/ &
     &8.3452e+00_rb,8.2318e+00_rb,8.1134e+00_rb,7.9935e+00_rb,7.8653e+00_rb/)
      kbo(:,24, 9) = (/ &
     &7.2560e+00_rb,7.1534e+00_rb,7.0580e+00_rb,6.9498e+00_rb,6.8440e+00_rb/)
      kbo(:,25, 9) = (/ &
     &6.3013e+00_rb,6.2279e+00_rb,6.1411e+00_rb,6.0460e+00_rb,5.9566e+00_rb/)
      kbo(:,26, 9) = (/ &
     &5.4473e+00_rb,5.3837e+00_rb,5.3066e+00_rb,5.2353e+00_rb,5.1609e+00_rb/)
      kbo(:,27, 9) = (/ &
     &4.6665e+00_rb,4.6055e+00_rb,4.5477e+00_rb,4.4876e+00_rb,4.4277e+00_rb/)
      kbo(:,28, 9) = (/ &
     &3.9616e+00_rb,3.9145e+00_rb,3.8668e+00_rb,3.8180e+00_rb,3.7694e+00_rb/)
      kbo(:,29, 9) = (/ &
     &3.3226e+00_rb,3.2862e+00_rb,3.2469e+00_rb,3.2091e+00_rb,3.1718e+00_rb/)
      kbo(:,30, 9) = (/ &
     &2.7659e+00_rb,2.7390e+00_rb,2.7094e+00_rb,2.6815e+00_rb,2.6543e+00_rb/)
      kbo(:,31, 9) = (/ &
     &2.2826e+00_rb,2.2617e+00_rb,2.2409e+00_rb,2.2203e+00_rb,2.1979e+00_rb/)
      kbo(:,32, 9) = (/ &
     &1.8802e+00_rb,1.8648e+00_rb,1.8493e+00_rb,1.8340e+00_rb,1.8168e+00_rb/)
      kbo(:,33, 9) = (/ &
     &1.5466e+00_rb,1.5353e+00_rb,1.5232e+00_rb,1.5114e+00_rb,1.4999e+00_rb/)
      kbo(:,34, 9) = (/ &
     &1.2752e+00_rb,1.2674e+00_rb,1.2588e+00_rb,1.2512e+00_rb,1.2426e+00_rb/)
      kbo(:,35, 9) = (/ &
     &1.0520e+00_rb,1.0466e+00_rb,1.0412e+00_rb,1.0356e+00_rb,1.0296e+00_rb/)
      kbo(:,36, 9) = (/ &
     &8.6897e-01_rb,8.6546e-01_rb,8.6220e-01_rb,8.5818e-01_rb,8.5421e-01_rb/)
      kbo(:,37, 9) = (/ &
     &7.1978e-01_rb,7.1767e-01_rb,7.1572e-01_rb,7.1373e-01_rb,7.1176e-01_rb/)
      kbo(:,38, 9) = (/ &
     &5.9482e-01_rb,5.9396e-01_rb,5.9353e-01_rb,5.9295e-01_rb,5.9232e-01_rb/)
      kbo(:,39, 9) = (/ &
     &4.9071e-01_rb,4.9116e-01_rb,4.9162e-01_rb,4.9173e-01_rb,4.9201e-01_rb/)
      kbo(:,40, 9) = (/ &
     &4.0707e-01_rb,4.0846e-01_rb,4.0948e-01_rb,4.1018e-01_rb,4.1089e-01_rb/)
      kbo(:,41, 9) = (/ &
     &3.3813e-01_rb,3.3959e-01_rb,3.4089e-01_rb,3.4218e-01_rb,3.4329e-01_rb/)
      kbo(:,42, 9) = (/ &
     &2.8061e-01_rb,2.8231e-01_rb,2.8363e-01_rb,2.8501e-01_rb,2.8630e-01_rb/)
      kbo(:,43, 9) = (/ &
     &2.3284e-01_rb,2.3448e-01_rb,2.3606e-01_rb,2.3737e-01_rb,2.3893e-01_rb/)
      kbo(:,44, 9) = (/ &
     &1.9306e-01_rb,1.9456e-01_rb,1.9627e-01_rb,1.9759e-01_rb,1.9913e-01_rb/)
      kbo(:,45, 9) = (/ &
     &1.5984e-01_rb,1.6139e-01_rb,1.6282e-01_rb,1.6435e-01_rb,1.6569e-01_rb/)
      kbo(:,46, 9) = (/ &
     &1.3218e-01_rb,1.3367e-01_rb,1.3504e-01_rb,1.3656e-01_rb,1.3780e-01_rb/)
      kbo(:,47, 9) = (/ &
     &1.0925e-01_rb,1.1079e-01_rb,1.1209e-01_rb,1.1337e-01_rb,1.1463e-01_rb/)
      kbo(:,48, 9) = (/ &
     &9.0249e-02_rb,9.1522e-02_rb,9.2774e-02_rb,9.3960e-02_rb,9.5177e-02_rb/)
      kbo(:,49, 9) = (/ &
     &7.4377e-02_rb,7.5400e-02_rb,7.6569e-02_rb,7.7633e-02_rb,7.8742e-02_rb/)
      kbo(:,50, 9) = (/ &
     &6.1418e-02_rb,6.2415e-02_rb,6.3422e-02_rb,6.4413e-02_rb,6.5389e-02_rb/)
      kbo(:,51, 9) = (/ &
     &5.0729e-02_rb,5.1644e-02_rb,5.2564e-02_rb,5.3444e-02_rb,5.4353e-02_rb/)
      kbo(:,52, 9) = (/ &
     &4.1908e-02_rb,4.2660e-02_rb,4.3502e-02_rb,4.4323e-02_rb,4.5095e-02_rb/)
      kbo(:,53, 9) = (/ &
     &3.4462e-02_rb,3.5223e-02_rb,3.5939e-02_rb,3.6640e-02_rb,3.7360e-02_rb/)
      kbo(:,54, 9) = (/ &
     &2.8588e-02_rb,2.9269e-02_rb,2.9929e-02_rb,3.0560e-02_rb,3.1160e-02_rb/)
      kbo(:,55, 9) = (/ &
     &2.3807e-02_rb,2.4406e-02_rb,2.4997e-02_rb,2.5573e-02_rb,2.6196e-02_rb/)
      kbo(:,56, 9) = (/ &
     &1.9841e-02_rb,2.0395e-02_rb,2.0945e-02_rb,2.1491e-02_rb,2.2006e-02_rb/)
      kbo(:,57, 9) = (/ &
     &1.6519e-02_rb,1.7032e-02_rb,1.7546e-02_rb,1.8054e-02_rb,1.8550e-02_rb/)
      kbo(:,58, 9) = (/ &
     &1.3793e-02_rb,1.4292e-02_rb,1.4753e-02_rb,1.5213e-02_rb,1.5695e-02_rb/)
      kbo(:,59, 9) = (/ &
     &1.1809e-02_rb,1.2269e-02_rb,1.2713e-02_rb,1.3163e-02_rb,1.3635e-02_rb/)
      kbo(:,13,10) = (/ &
     &1.7142e+02_rb,1.7281e+02_rb,1.7236e+02_rb,1.7114e+02_rb,1.6934e+02_rb/)
      kbo(:,14,10) = (/ &
     &1.5574e+02_rb,1.5556e+02_rb,1.5469e+02_rb,1.5334e+02_rb,1.5193e+02_rb/)
      kbo(:,15,10) = (/ &
     &1.3728e+02_rb,1.3704e+02_rb,1.3637e+02_rb,1.3560e+02_rb,1.3459e+02_rb/)
      kbo(:,16,10) = (/ &
     &1.1953e+02_rb,1.1932e+02_rb,1.1895e+02_rb,1.1818e+02_rb,1.1715e+02_rb/)
      kbo(:,17,10) = (/ &
     &1.0359e+02_rb,1.0375e+02_rb,1.0320e+02_rb,1.0240e+02_rb,1.0131e+02_rb/)
      kbo(:,18,10) = (/ &
     &8.8878e+01_rb,8.9020e+01_rb,8.8706e+01_rb,8.7973e+01_rb,8.6746e+01_rb/)
      kbo(:,19,10) = (/ &
     &7.4020e+01_rb,7.4305e+01_rb,7.4019e+01_rb,7.3466e+01_rb,7.2562e+01_rb/)
      kbo(:,20,10) = (/ &
     &6.0721e+01_rb,6.1015e+01_rb,6.0921e+01_rb,6.0532e+01_rb,5.9983e+01_rb/)
      kbo(:,21,10) = (/ &
     &5.0465e+01_rb,5.0665e+01_rb,5.0241e+01_rb,4.9849e+01_rb,4.9385e+01_rb/)
      kbo(:,22,10) = (/ &
     &4.0819e+01_rb,4.0731e+01_rb,4.0529e+01_rb,4.0271e+01_rb,3.9749e+01_rb/)
      kbo(:,23,10) = (/ &
     &3.4205e+01_rb,3.4202e+01_rb,3.4020e+01_rb,3.3582e+01_rb,3.3186e+01_rb/)
      kbo(:,24,10) = (/ &
     &2.9307e+01_rb,2.9168e+01_rb,2.8849e+01_rb,2.8552e+01_rb,2.8160e+01_rb/)
      kbo(:,25,10) = (/ &
     &2.5427e+01_rb,2.5125e+01_rb,2.4814e+01_rb,2.4511e+01_rb,2.4195e+01_rb/)
      kbo(:,26,10) = (/ &
     &2.2227e+01_rb,2.1953e+01_rb,2.1723e+01_rb,2.1344e+01_rb,2.1061e+01_rb/)
      kbo(:,27,10) = (/ &
     &1.9562e+01_rb,1.9387e+01_rb,1.9087e+01_rb,1.8779e+01_rb,1.8567e+01_rb/)
      kbo(:,28,10) = (/ &
     &1.7147e+01_rb,1.6935e+01_rb,1.6668e+01_rb,1.6479e+01_rb,1.6263e+01_rb/)
      kbo(:,29,10) = (/ &
     &1.4781e+01_rb,1.4573e+01_rb,1.4404e+01_rb,1.4242e+01_rb,1.4093e+01_rb/)
      kbo(:,30,10) = (/ &
     &1.2615e+01_rb,1.2471e+01_rb,1.2325e+01_rb,1.2193e+01_rb,1.2047e+01_rb/)
      kbo(:,31,10) = (/ &
     &1.0637e+01_rb,1.0540e+01_rb,1.0405e+01_rb,1.0303e+01_rb,1.0219e+01_rb/)
      kbo(:,32,10) = (/ &
     &8.9113e+00_rb,8.8364e+00_rb,8.7502e+00_rb,8.6801e+00_rb,8.6141e+00_rb/)
      kbo(:,33,10) = (/ &
     &7.4220e+00_rb,7.3714e+00_rb,7.3296e+00_rb,7.2916e+00_rb,7.2335e+00_rb/)
      kbo(:,34,10) = (/ &
     &6.2166e+00_rb,6.1844e+00_rb,6.1565e+00_rb,6.1097e+00_rb,6.0839e+00_rb/)
      kbo(:,35,10) = (/ &
     &5.1978e+00_rb,5.1804e+00_rb,5.1597e+00_rb,5.1357e+00_rb,5.1182e+00_rb/)
      kbo(:,36,10) = (/ &
     &4.3470e+00_rb,4.3385e+00_rb,4.3233e+00_rb,4.3177e+00_rb,4.3104e+00_rb/)
      kbo(:,37,10) = (/ &
     &3.6398e+00_rb,3.6424e+00_rb,3.6345e+00_rb,3.6386e+00_rb,3.6334e+00_rb/)
      kbo(:,38,10) = (/ &
     &3.0434e+00_rb,3.0512e+00_rb,3.0559e+00_rb,3.0639e+00_rb,3.0639e+00_rb/)
      kbo(:,39,10) = (/ &
     &2.5458e+00_rb,2.5538e+00_rb,2.5569e+00_rb,2.5728e+00_rb,2.5769e+00_rb/)
      kbo(:,40,10) = (/ &
     &2.1317e+00_rb,2.1438e+00_rb,2.1539e+00_rb,2.1706e+00_rb,2.1835e+00_rb/)
      kbo(:,41,10) = (/ &
     &1.7848e+00_rb,1.8052e+00_rb,1.8158e+00_rb,1.8336e+00_rb,1.8470e+00_rb/)
      kbo(:,42,10) = (/ &
     &1.4932e+00_rb,1.5091e+00_rb,1.5289e+00_rb,1.5466e+00_rb,1.5621e+00_rb/)
      kbo(:,43,10) = (/ &
     &1.2466e+00_rb,1.2672e+00_rb,1.2843e+00_rb,1.3063e+00_rb,1.3216e+00_rb/)
      kbo(:,44,10) = (/ &
     &1.0395e+00_rb,1.0592e+00_rb,1.0758e+00_rb,1.0990e+00_rb,1.1149e+00_rb/)
      kbo(:,45,10) = (/ &
     &8.6570e-01_rb,8.8331e-01_rb,9.0051e-01_rb,9.1956e-01_rb,9.3855e-01_rb/)
      kbo(:,46,10) = (/ &
     &7.2006e-01_rb,7.3467e-01_rb,7.5104e-01_rb,7.6845e-01_rb,7.8787e-01_rb/)
      kbo(:,47,10) = (/ &
     &5.9818e-01_rb,6.0970e-01_rb,6.2444e-01_rb,6.4248e-01_rb,6.5898e-01_rb/)
      kbo(:,48,10) = (/ &
     &4.9441e-01_rb,5.0661e-01_rb,5.1925e-01_rb,5.3379e-01_rb,5.5029e-01_rb/)
      kbo(:,49,10) = (/ &
     &4.0883e-01_rb,4.1946e-01_rb,4.3019e-01_rb,4.4310e-01_rb,4.5870e-01_rb/)
      kbo(:,50,10) = (/ &
     &3.3724e-01_rb,3.4698e-01_rb,3.5791e-01_rb,3.6811e-01_rb,3.8262e-01_rb/)
      kbo(:,51,10) = (/ &
     &2.7937e-01_rb,2.8820e-01_rb,2.9748e-01_rb,3.0671e-01_rb,3.1956e-01_rb/)
      kbo(:,52,10) = (/ &
     &2.3100e-01_rb,2.3806e-01_rb,2.4610e-01_rb,2.5504e-01_rb,2.6621e-01_rb/)
      kbo(:,53,10) = (/ &
     &1.9108e-01_rb,1.9654e-01_rb,2.0325e-01_rb,2.1194e-01_rb,2.2154e-01_rb/)
      kbo(:,54,10) = (/ &
     &1.5899e-01_rb,1.6347e-01_rb,1.6947e-01_rb,1.7655e-01_rb,1.8610e-01_rb/)
      kbo(:,55,10) = (/ &
     &1.3326e-01_rb,1.3696e-01_rb,1.4221e-01_rb,1.4834e-01_rb,1.5647e-01_rb/)
      kbo(:,56,10) = (/ &
     &1.1049e-01_rb,1.1469e-01_rb,1.1935e-01_rb,1.2428e-01_rb,1.3176e-01_rb/)
      kbo(:,57,10) = (/ &
     &9.2761e-02_rb,9.6003e-02_rb,9.9705e-02_rb,1.0515e-01_rb,1.1098e-01_rb/)
      kbo(:,58,10) = (/ &
     &7.7951e-02_rb,8.0603e-02_rb,8.4255e-02_rb,8.8708e-02_rb,9.3824e-02_rb/)
      kbo(:,59,10) = (/ &
     &6.6831e-02_rb,6.9853e-02_rb,7.3196e-02_rb,7.7714e-02_rb,8.2447e-02_rb/)
      kbo(:,13,11) = (/ &
     &2.9044e+02_rb,2.9107e+02_rb,2.9274e+02_rb,2.9324e+02_rb,2.9124e+02_rb/)
      kbo(:,14,11) = (/ &
     &2.7118e+02_rb,2.7257e+02_rb,2.7251e+02_rb,2.7007e+02_rb,2.6710e+02_rb/)
      kbo(:,15,11) = (/ &
     &2.5062e+02_rb,2.5131e+02_rb,2.4974e+02_rb,2.4698e+02_rb,2.4378e+02_rb/)
      kbo(:,16,11) = (/ &
     &2.2673e+02_rb,2.2541e+02_rb,2.2341e+02_rb,2.2106e+02_rb,2.1873e+02_rb/)
      kbo(:,17,11) = (/ &
     &1.9976e+02_rb,1.9861e+02_rb,1.9716e+02_rb,1.9539e+02_rb,1.9362e+02_rb/)
      kbo(:,18,11) = (/ &
     &1.7313e+02_rb,1.7242e+02_rb,1.7179e+02_rb,1.7065e+02_rb,1.6917e+02_rb/)
      kbo(:,19,11) = (/ &
     &1.5014e+02_rb,1.4982e+02_rb,1.4894e+02_rb,1.4750e+02_rb,1.4584e+02_rb/)
      kbo(:,20,11) = (/ &
     &1.2916e+02_rb,1.2859e+02_rb,1.2771e+02_rb,1.2637e+02_rb,1.2459e+02_rb/)
      kbo(:,21,11) = (/ &
     &1.0806e+02_rb,1.0775e+02_rb,1.0718e+02_rb,1.0610e+02_rb,1.0475e+02_rb/)
      kbo(:,22,11) = (/ &
     &8.9165e+01_rb,8.9073e+01_rb,8.8652e+01_rb,8.7815e+01_rb,8.6979e+01_rb/)
      kbo(:,23,11) = (/ &
     &7.2556e+01_rb,7.2172e+01_rb,7.1575e+01_rb,7.1058e+01_rb,7.0305e+01_rb/)
      kbo(:,24,11) = (/ &
     &5.8830e+01_rb,5.8306e+01_rb,5.7758e+01_rb,5.7080e+01_rb,5.6291e+01_rb/)
      kbo(:,25,11) = (/ &
     &4.9821e+01_rb,4.9499e+01_rb,4.8965e+01_rb,4.8414e+01_rb,4.7810e+01_rb/)
      kbo(:,26,11) = (/ &
     &4.2622e+01_rb,4.2267e+01_rb,4.1876e+01_rb,4.1470e+01_rb,4.0806e+01_rb/)
      kbo(:,27,11) = (/ &
     &3.6706e+01_rb,3.6324e+01_rb,3.5968e+01_rb,3.5540e+01_rb,3.5058e+01_rb/)
      kbo(:,28,11) = (/ &
     &3.1978e+01_rb,3.1566e+01_rb,3.1237e+01_rb,3.0772e+01_rb,3.0454e+01_rb/)
      kbo(:,29,11) = (/ &
     &2.7828e+01_rb,2.7575e+01_rb,2.7171e+01_rb,2.6860e+01_rb,2.6508e+01_rb/)
      kbo(:,30,11) = (/ &
     &2.4147e+01_rb,2.3835e+01_rb,2.3571e+01_rb,2.3332e+01_rb,2.3114e+01_rb/)
      kbo(:,31,11) = (/ &
     &2.0703e+01_rb,2.0408e+01_rb,2.0254e+01_rb,2.0067e+01_rb,1.9883e+01_rb/)
      kbo(:,32,11) = (/ &
     &1.7639e+01_rb,1.7476e+01_rb,1.7324e+01_rb,1.7159e+01_rb,1.7076e+01_rb/)
      kbo(:,33,11) = (/ &
     &1.4975e+01_rb,1.4865e+01_rb,1.4744e+01_rb,1.4668e+01_rb,1.4624e+01_rb/)
      kbo(:,34,11) = (/ &
     &1.2711e+01_rb,1.2631e+01_rb,1.2589e+01_rb,1.2550e+01_rb,1.2508e+01_rb/)
      kbo(:,35,11) = (/ &
     &1.0751e+01_rb,1.0723e+01_rb,1.0704e+01_rb,1.0702e+01_rb,1.0681e+01_rb/)
      kbo(:,36,11) = (/ &
     &9.0814e+00_rb,9.0752e+00_rb,9.0908e+00_rb,9.1081e+00_rb,9.1217e+00_rb/)
      kbo(:,37,11) = (/ &
     &7.6820e+00_rb,7.7040e+00_rb,7.7379e+00_rb,7.7515e+00_rb,7.7864e+00_rb/)
      kbo(:,38,11) = (/ &
     &6.4923e+00_rb,6.5176e+00_rb,6.5490e+00_rb,6.5779e+00_rb,6.6281e+00_rb/)
      kbo(:,39,11) = (/ &
     &5.4567e+00_rb,5.4975e+00_rb,5.5468e+00_rb,5.5843e+00_rb,5.6396e+00_rb/)
      kbo(:,40,11) = (/ &
     &4.6201e+00_rb,4.6597e+00_rb,4.7074e+00_rb,4.7531e+00_rb,4.8054e+00_rb/)
      kbo(:,41,11) = (/ &
     &3.8974e+00_rb,3.9399e+00_rb,3.9967e+00_rb,4.0418e+00_rb,4.1011e+00_rb/)
      kbo(:,42,11) = (/ &
     &3.2907e+00_rb,3.3361e+00_rb,3.3839e+00_rb,3.4382e+00_rb,3.4997e+00_rb/)
      kbo(:,43,11) = (/ &
     &2.7732e+00_rb,2.8146e+00_rb,2.8677e+00_rb,2.9201e+00_rb,2.9866e+00_rb/)
      kbo(:,44,11) = (/ &
     &2.3284e+00_rb,2.3728e+00_rb,2.4251e+00_rb,2.4785e+00_rb,2.5440e+00_rb/)
      kbo(:,45,11) = (/ &
     &1.9528e+00_rb,1.9951e+00_rb,2.0498e+00_rb,2.1003e+00_rb,2.1644e+00_rb/)
      kbo(:,46,11) = (/ &
     &1.6318e+00_rb,1.6755e+00_rb,1.7227e+00_rb,1.7737e+00_rb,1.8354e+00_rb/)
      kbo(:,47,11) = (/ &
     &1.3599e+00_rb,1.4032e+00_rb,1.4486e+00_rb,1.4978e+00_rb,1.5556e+00_rb/)
      kbo(:,48,11) = (/ &
     &1.1311e+00_rb,1.1685e+00_rb,1.2136e+00_rb,1.2615e+00_rb,1.3144e+00_rb/)
      kbo(:,49,11) = (/ &
     &9.3555e-01_rb,9.7341e-01_rb,1.0126e+00_rb,1.0587e+00_rb,1.1063e+00_rb/)
      kbo(:,50,11) = (/ &
     &7.7693e-01_rb,8.0966e-01_rb,8.4684e-01_rb,8.9020e-01_rb,9.3317e-01_rb/)
      kbo(:,51,11) = (/ &
     &6.4536e-01_rb,6.7488e-01_rb,7.0604e-01_rb,7.4875e-01_rb,7.8723e-01_rb/)
      kbo(:,52,11) = (/ &
     &5.3457e-01_rb,5.5971e-01_rb,5.9092e-01_rb,6.2609e-01_rb,6.6312e-01_rb/)
      kbo(:,53,11) = (/ &
     &4.4222e-01_rb,4.6435e-01_rb,4.9056e-01_rb,5.2208e-01_rb,5.5614e-01_rb/)
      kbo(:,54,11) = (/ &
     &3.6743e-01_rb,3.8638e-01_rb,4.1127e-01_rb,4.3844e-01_rb,4.6892e-01_rb/)
      kbo(:,55,11) = (/ &
     &3.0658e-01_rb,3.2360e-01_rb,3.4391e-01_rb,3.6924e-01_rb,3.9670e-01_rb/)
      kbo(:,56,11) = (/ &
     &2.5551e-01_rb,2.6954e-01_rb,2.8819e-01_rb,3.1080e-01_rb,3.3508e-01_rb/)
      kbo(:,57,11) = (/ &
     &2.1322e-01_rb,2.2632e-01_rb,2.4139e-01_rb,2.5957e-01_rb,2.8310e-01_rb/)
      kbo(:,58,11) = (/ &
     &1.7898e-01_rb,1.8948e-01_rb,2.0228e-01_rb,2.1870e-01_rb,2.4000e-01_rb/)
      kbo(:,59,11) = (/ &
     &1.5351e-01_rb,1.6320e-01_rb,1.7439e-01_rb,1.8953e-01_rb,2.0982e-01_rb/)
      kbo(:,13,12) = (/ &
     &5.5035e+02_rb,5.5083e+02_rb,5.4893e+02_rb,5.4549e+02_rb,5.4357e+02_rb/)
      kbo(:,14,12) = (/ &
     &5.1666e+02_rb,5.1747e+02_rb,5.1731e+02_rb,5.1865e+02_rb,5.1926e+02_rb/)
      kbo(:,15,12) = (/ &
     &4.8259e+02_rb,4.8322e+02_rb,4.8500e+02_rb,4.8625e+02_rb,4.8440e+02_rb/)
      kbo(:,16,12) = (/ &
     &4.4814e+02_rb,4.5080e+02_rb,4.5253e+02_rb,4.5080e+02_rb,4.4664e+02_rb/)
      kbo(:,17,12) = (/ &
     &4.1611e+02_rb,4.1780e+02_rb,4.1594e+02_rb,4.1174e+02_rb,4.0672e+02_rb/)
      kbo(:,18,12) = (/ &
     &3.8097e+02_rb,3.8017e+02_rb,3.7694e+02_rb,3.7269e+02_rb,3.6847e+02_rb/)
      kbo(:,19,12) = (/ &
     &3.3900e+02_rb,3.3735e+02_rb,3.3499e+02_rb,3.3205e+02_rb,3.2864e+02_rb/)
      kbo(:,20,12) = (/ &
     &2.9644e+02_rb,2.9567e+02_rb,2.9387e+02_rb,2.9171e+02_rb,2.8914e+02_rb/)
      kbo(:,21,12) = (/ &
     &2.5711e+02_rb,2.5659e+02_rb,2.5545e+02_rb,2.5349e+02_rb,2.5090e+02_rb/)
      kbo(:,22,12) = (/ &
     &2.2227e+02_rb,2.2144e+02_rb,2.1973e+02_rb,2.1742e+02_rb,2.1472e+02_rb/)
      kbo(:,23,12) = (/ &
     &1.8874e+02_rb,1.8786e+02_rb,1.8655e+02_rb,1.8434e+02_rb,1.8190e+02_rb/)
      kbo(:,24,12) = (/ &
     &1.5708e+02_rb,1.5682e+02_rb,1.5564e+02_rb,1.5414e+02_rb,1.5250e+02_rb/)
      kbo(:,25,12) = (/ &
     &1.2691e+02_rb,1.2649e+02_rb,1.2582e+02_rb,1.2476e+02_rb,1.2335e+02_rb/)
      kbo(:,26,12) = (/ &
     &1.0261e+02_rb,1.0208e+02_rb,1.0124e+02_rb,1.0027e+02_rb,9.9263e+01_rb/)
      kbo(:,27,12) = (/ &
     &8.5769e+01_rb,8.4989e+01_rb,8.4207e+01_rb,8.3351e+01_rb,8.2249e+01_rb/)
      kbo(:,28,12) = (/ &
     &7.3314e+01_rb,7.2801e+01_rb,7.2005e+01_rb,7.1293e+01_rb,7.0277e+01_rb/)
      kbo(:,29,12) = (/ &
     &6.2526e+01_rb,6.2052e+01_rb,6.1497e+01_rb,6.0774e+01_rb,6.0060e+01_rb/)
      kbo(:,30,12) = (/ &
     &5.3636e+01_rb,5.3199e+01_rb,5.2692e+01_rb,5.1970e+01_rb,5.1392e+01_rb/)
      kbo(:,31,12) = (/ &
     &4.6032e+01_rb,4.5690e+01_rb,4.5122e+01_rb,4.4615e+01_rb,4.4345e+01_rb/)
      kbo(:,32,12) = (/ &
     &3.9736e+01_rb,3.9262e+01_rb,3.8850e+01_rb,3.8552e+01_rb,3.8247e+01_rb/)
      kbo(:,33,12) = (/ &
     &3.4154e+01_rb,3.3807e+01_rb,3.3534e+01_rb,3.3264e+01_rb,3.3008e+01_rb/)
      kbo(:,34,12) = (/ &
     &2.9451e+01_rb,2.9219e+01_rb,2.9024e+01_rb,2.8833e+01_rb,2.8685e+01_rb/)
      kbo(:,35,12) = (/ &
     &2.5346e+01_rb,2.5246e+01_rb,2.5119e+01_rb,2.4977e+01_rb,2.4976e+01_rb/)
      kbo(:,36,12) = (/ &
     &2.1817e+01_rb,2.1779e+01_rb,2.1703e+01_rb,2.1681e+01_rb,2.1750e+01_rb/)
      kbo(:,37,12) = (/ &
     &1.8813e+01_rb,1.8767e+01_rb,1.8799e+01_rb,1.8847e+01_rb,1.8941e+01_rb/)
      kbo(:,38,12) = (/ &
     &1.6125e+01_rb,1.6141e+01_rb,1.6208e+01_rb,1.6337e+01_rb,1.6455e+01_rb/)
      kbo(:,39,12) = (/ &
     &1.3770e+01_rb,1.3861e+01_rb,1.3953e+01_rb,1.4099e+01_rb,1.4238e+01_rb/)
      kbo(:,40,12) = (/ &
     &1.1787e+01_rb,1.1887e+01_rb,1.2028e+01_rb,1.2178e+01_rb,1.2352e+01_rb/)
      kbo(:,41,12) = (/ &
     &1.0073e+01_rb,1.0213e+01_rb,1.0351e+01_rb,1.0547e+01_rb,1.0724e+01_rb/)
      kbo(:,42,12) = (/ &
     &8.5925e+00_rb,8.7331e+00_rb,8.9300e+00_rb,9.0948e+00_rb,9.2833e+00_rb/)
      kbo(:,43,12) = (/ &
     &7.3118e+00_rb,7.4895e+00_rb,7.6577e+00_rb,7.8465e+00_rb,8.0457e+00_rb/)
      kbo(:,44,12) = (/ &
     &6.2222e+00_rb,6.3886e+00_rb,6.5774e+00_rb,6.7646e+00_rb,6.9710e+00_rb/)
      kbo(:,45,12) = (/ &
     &5.2923e+00_rb,5.4528e+00_rb,5.6365e+00_rb,5.8147e+00_rb,6.0122e+00_rb/)
      kbo(:,46,12) = (/ &
     &4.4844e+00_rb,4.6394e+00_rb,4.8198e+00_rb,4.9986e+00_rb,5.1861e+00_rb/)
      kbo(:,47,12) = (/ &
     &3.7915e+00_rb,3.9458e+00_rb,4.1119e+00_rb,4.2905e+00_rb,4.4702e+00_rb/)
      kbo(:,48,12) = (/ &
     &3.2031e+00_rb,3.3449e+00_rb,3.4984e+00_rb,3.6554e+00_rb,3.8276e+00_rb/)
      kbo(:,49,12) = (/ &
     &2.6829e+00_rb,2.8156e+00_rb,2.9666e+00_rb,3.1113e+00_rb,3.2647e+00_rb/)
      kbo(:,50,12) = (/ &
     &2.2570e+00_rb,2.3778e+00_rb,2.5052e+00_rb,2.6542e+00_rb,2.7988e+00_rb/)
      kbo(:,51,12) = (/ &
     &1.8957e+00_rb,2.0048e+00_rb,2.1275e+00_rb,2.2538e+00_rb,2.3944e+00_rb/)
      kbo(:,52,12) = (/ &
     &1.5859e+00_rb,1.6936e+00_rb,1.7982e+00_rb,1.9164e+00_rb,2.0433e+00_rb/)
      kbo(:,53,12) = (/ &
     &1.3268e+00_rb,1.4138e+00_rb,1.5132e+00_rb,1.6183e+00_rb,1.7346e+00_rb/)
      kbo(:,54,12) = (/ &
     &1.1132e+00_rb,1.1981e+00_rb,1.2847e+00_rb,1.3880e+00_rb,1.4926e+00_rb/)
      kbo(:,55,12) = (/ &
     &9.3399e-01_rb,1.0137e+00_rb,1.0949e+00_rb,1.1891e+00_rb,1.2944e+00_rb/)
      kbo(:,56,12) = (/ &
     &7.8822e-01_rb,8.5806e-01_rb,9.3275e-01_rb,1.0165e+00_rb,1.1194e+00_rb/)
      kbo(:,57,12) = (/ &
     &6.5846e-01_rb,7.2237e-01_rb,7.9487e-01_rb,8.7625e-01_rb,9.6708e-01_rb/)
      kbo(:,58,12) = (/ &
     &5.5082e-01_rb,6.0932e-01_rb,6.7677e-01_rb,7.5228e-01_rb,8.3907e-01_rb/)
      kbo(:,59,12) = (/ &
     &4.7097e-01_rb,5.2693e-01_rb,5.8955e-01_rb,6.6574e-01_rb,7.5245e-01_rb/)
      kbo(:,13,13) = (/ &
     &1.1006e+03_rb,1.0930e+03_rb,1.0868e+03_rb,1.0845e+03_rb,1.0783e+03_rb/)
      kbo(:,14,13) = (/ &
     &1.0959e+03_rb,1.0919e+03_rb,1.0910e+03_rb,1.0859e+03_rb,1.0743e+03_rb/)
      kbo(:,15,13) = (/ &
     &1.0739e+03_rb,1.0736e+03_rb,1.0715e+03_rb,1.0637e+03_rb,1.0554e+03_rb/)
      kbo(:,16,13) = (/ &
     &1.0348e+03_rb,1.0368e+03_rb,1.0336e+03_rb,1.0297e+03_rb,1.0251e+03_rb/)
      kbo(:,17,13) = (/ &
     &9.8104e+02_rb,9.8318e+02_rb,9.8416e+02_rb,9.8388e+02_rb,9.8177e+02_rb/)
      kbo(:,18,13) = (/ &
     &9.1939e+02_rb,9.2259e+02_rb,9.2492e+02_rb,9.2597e+02_rb,9.2235e+02_rb/)
      kbo(:,19,13) = (/ &
     &8.5744e+02_rb,8.6138e+02_rb,8.6320e+02_rb,8.6092e+02_rb,8.5380e+02_rb/)
      kbo(:,20,13) = (/ &
     &7.9222e+02_rb,7.9480e+02_rb,7.9309e+02_rb,7.8688e+02_rb,7.7879e+02_rb/)
      kbo(:,21,13) = (/ &
     &7.2352e+02_rb,7.2196e+02_rb,7.1706e+02_rb,7.1061e+02_rb,7.0329e+02_rb/)
      kbo(:,22,13) = (/ &
     &6.4947e+02_rb,6.4590e+02_rb,6.4130e+02_rb,6.3579e+02_rb,6.2875e+02_rb/)
      kbo(:,23,13) = (/ &
     &5.7230e+02_rb,5.6958e+02_rb,5.6554e+02_rb,5.6079e+02_rb,5.5572e+02_rb/)
      kbo(:,24,13) = (/ &
     &4.9907e+02_rb,4.9687e+02_rb,4.9379e+02_rb,4.8982e+02_rb,4.8472e+02_rb/)
      kbo(:,25,13) = (/ &
     &4.3090e+02_rb,4.2914e+02_rb,4.2610e+02_rb,4.2208e+02_rb,4.1721e+02_rb/)
      kbo(:,26,13) = (/ &
     &3.6834e+02_rb,3.6617e+02_rb,3.6313e+02_rb,3.5932e+02_rb,3.5509e+02_rb/)
      kbo(:,27,13) = (/ &
     &3.0815e+02_rb,3.0692e+02_rb,3.0435e+02_rb,3.0103e+02_rb,2.9773e+02_rb/)
      kbo(:,28,13) = (/ &
     &2.5402e+02_rb,2.5274e+02_rb,2.5083e+02_rb,2.4862e+02_rb,2.4624e+02_rb/)
      kbo(:,29,13) = (/ &
     &2.0985e+02_rb,2.0878e+02_rb,2.0729e+02_rb,2.0573e+02_rb,2.0375e+02_rb/)
      kbo(:,30,13) = (/ &
     &1.7400e+02_rb,1.7284e+02_rb,1.7160e+02_rb,1.7041e+02_rb,1.6891e+02_rb/)
      kbo(:,31,13) = (/ &
     &1.4590e+02_rb,1.4515e+02_rb,1.4409e+02_rb,1.4281e+02_rb,1.4174e+02_rb/)
      kbo(:,32,13) = (/ &
     &1.2352e+02_rb,1.2281e+02_rb,1.2185e+02_rb,1.2095e+02_rb,1.2000e+02_rb/)
      kbo(:,33,13) = (/ &
     &1.0504e+02_rb,1.0445e+02_rb,1.0374e+02_rb,1.0318e+02_rb,1.0254e+02_rb/)
      kbo(:,34,13) = (/ &
     &8.9914e+01_rb,8.9264e+01_rb,8.8774e+01_rb,8.8232e+01_rb,8.7968e+01_rb/)
      kbo(:,35,13) = (/ &
     &7.7137e+01_rb,7.6673e+01_rb,7.6351e+01_rb,7.5989e+01_rb,7.5812e+01_rb/)
      kbo(:,36,13) = (/ &
     &6.6630e+01_rb,6.6370e+01_rb,6.6000e+01_rb,6.5870e+01_rb,6.5834e+01_rb/)
      kbo(:,37,13) = (/ &
     &5.7835e+01_rb,5.7711e+01_rb,5.7500e+01_rb,5.7478e+01_rb,5.7625e+01_rb/)
      kbo(:,38,13) = (/ &
     &5.0326e+01_rb,5.0239e+01_rb,5.0196e+01_rb,5.0287e+01_rb,5.0433e+01_rb/)
      kbo(:,39,13) = (/ &
     &4.3783e+01_rb,4.3749e+01_rb,4.3859e+01_rb,4.3984e+01_rb,4.4262e+01_rb/)
      kbo(:,40,13) = (/ &
     &3.8311e+01_rb,3.8366e+01_rb,3.8494e+01_rb,3.8770e+01_rb,3.9082e+01_rb/)
      kbo(:,41,13) = (/ &
     &3.3505e+01_rb,3.3696e+01_rb,3.3809e+01_rb,3.4139e+01_rb,3.4534e+01_rb/)
      kbo(:,42,13) = (/ &
     &2.9251e+01_rb,2.9507e+01_rb,2.9744e+01_rb,3.0106e+01_rb,3.0543e+01_rb/)
      kbo(:,43,13) = (/ &
     &2.5517e+01_rb,2.5852e+01_rb,2.6152e+01_rb,2.6566e+01_rb,2.7067e+01_rb/)
      kbo(:,44,13) = (/ &
     &2.2216e+01_rb,2.2648e+01_rb,2.3006e+01_rb,2.3424e+01_rb,2.3903e+01_rb/)
      kbo(:,45,13) = (/ &
     &1.9313e+01_rb,1.9753e+01_rb,2.0163e+01_rb,2.0601e+01_rb,2.1122e+01_rb/)
      kbo(:,46,13) = (/ &
     &1.6765e+01_rb,1.7207e+01_rb,1.7660e+01_rb,1.8173e+01_rb,1.8710e+01_rb/)
      kbo(:,47,13) = (/ &
     &1.4512e+01_rb,1.4984e+01_rb,1.5402e+01_rb,1.5957e+01_rb,1.6479e+01_rb/)
      kbo(:,48,13) = (/ &
     &1.2486e+01_rb,1.2967e+01_rb,1.3459e+01_rb,1.3983e+01_rb,1.4524e+01_rb/)
      kbo(:,49,13) = (/ &
     &1.0733e+01_rb,1.1206e+01_rb,1.1665e+01_rb,1.2207e+01_rb,1.2773e+01_rb/)
      kbo(:,50,13) = (/ &
     &9.2157e+00_rb,9.6749e+00_rb,1.0151e+01_rb,1.0704e+01_rb,1.1247e+01_rb/)
      kbo(:,51,13) = (/ &
     &7.9591e+00_rb,8.3907e+00_rb,8.8535e+00_rb,9.3811e+00_rb,9.9247e+00_rb/)
      kbo(:,52,13) = (/ &
     &6.8068e+00_rb,7.2371e+00_rb,7.6934e+00_rb,8.2063e+00_rb,8.7553e+00_rb/)
      kbo(:,53,13) = (/ &
     &5.7967e+00_rb,6.2085e+00_rb,6.6421e+00_rb,7.1263e+00_rb,7.6815e+00_rb/)
      kbo(:,54,13) = (/ &
     &4.9653e+00_rb,5.3639e+00_rb,5.8069e+00_rb,6.2889e+00_rb,6.8002e+00_rb/)
      kbo(:,55,13) = (/ &
     &4.2889e+00_rb,4.6915e+00_rb,5.1242e+00_rb,5.6024e+00_rb,6.0981e+00_rb/)
      kbo(:,56,13) = (/ &
     &3.6900e+00_rb,4.0833e+00_rb,4.5086e+00_rb,4.9809e+00_rb,5.4831e+00_rb/)
      kbo(:,57,13) = (/ &
     &3.1898e+00_rb,3.5703e+00_rb,3.9801e+00_rb,4.4470e+00_rb,4.9431e+00_rb/)
      kbo(:,58,13) = (/ &
     &2.7316e+00_rb,3.0989e+00_rb,3.5187e+00_rb,3.9710e+00_rb,4.4675e+00_rb/)
      kbo(:,59,13) = (/ &
     &2.4661e+00_rb,2.8142e+00_rb,3.2518e+00_rb,3.7206e+00_rb,4.2604e+00_rb/)
      kbo(:,13,14) = (/ &
     &2.3516e+03_rb,2.3425e+03_rb,2.3393e+03_rb,2.3344e+03_rb,2.3274e+03_rb/)
      kbo(:,14,14) = (/ &
     &2.4560e+03_rb,2.4460e+03_rb,2.4416e+03_rb,2.4361e+03_rb,2.4328e+03_rb/)
      kbo(:,15,14) = (/ &
     &2.5504e+03_rb,2.5434e+03_rb,2.5388e+03_rb,2.5327e+03_rb,2.5248e+03_rb/)
      kbo(:,16,14) = (/ &
     &2.6346e+03_rb,2.6312e+03_rb,2.6203e+03_rb,2.6088e+03_rb,2.5971e+03_rb/)
      kbo(:,17,14) = (/ &
     &2.7008e+03_rb,2.6935e+03_rb,2.6794e+03_rb,2.6636e+03_rb,2.6477e+03_rb/)
      kbo(:,18,14) = (/ &
     &2.7305e+03_rb,2.7239e+03_rb,2.7111e+03_rb,2.6981e+03_rb,2.6813e+03_rb/)
      kbo(:,19,14) = (/ &
     &2.7180e+03_rb,2.7110e+03_rb,2.7049e+03_rb,2.6968e+03_rb,2.6814e+03_rb/)
      kbo(:,20,14) = (/ &
     &2.6672e+03_rb,2.6665e+03_rb,2.6675e+03_rb,2.6606e+03_rb,2.6429e+03_rb/)
      kbo(:,21,14) = (/ &
     &2.5815e+03_rb,2.5893e+03_rb,2.5905e+03_rb,2.5819e+03_rb,2.5661e+03_rb/)
      kbo(:,22,14) = (/ &
     &2.4673e+03_rb,2.4773e+03_rb,2.4773e+03_rb,2.4699e+03_rb,2.4562e+03_rb/)
      kbo(:,23,14) = (/ &
     &2.3326e+03_rb,2.3404e+03_rb,2.3409e+03_rb,2.3341e+03_rb,2.3220e+03_rb/)
      kbo(:,24,14) = (/ &
     &2.1770e+03_rb,2.1831e+03_rb,2.1839e+03_rb,2.1779e+03_rb,2.1650e+03_rb/)
      kbo(:,25,14) = (/ &
     &2.0113e+03_rb,2.0166e+03_rb,2.0154e+03_rb,2.0063e+03_rb,1.9910e+03_rb/)
      kbo(:,26,14) = (/ &
     &1.8365e+03_rb,1.8415e+03_rb,1.8363e+03_rb,1.8253e+03_rb,1.8117e+03_rb/)
      kbo(:,27,14) = (/ &
     &1.6595e+03_rb,1.6581e+03_rb,1.6515e+03_rb,1.6428e+03_rb,1.6322e+03_rb/)
      kbo(:,28,14) = (/ &
     &1.4809e+03_rb,1.4771e+03_rb,1.4724e+03_rb,1.4658e+03_rb,1.4569e+03_rb/)
      kbo(:,29,14) = (/ &
     &1.3089e+03_rb,1.3068e+03_rb,1.3044e+03_rb,1.2991e+03_rb,1.2930e+03_rb/)
      kbo(:,30,14) = (/ &
     &1.1460e+03_rb,1.1478e+03_rb,1.1472e+03_rb,1.1450e+03_rb,1.1415e+03_rb/)
      kbo(:,31,14) = (/ &
     &9.9693e+02_rb,1.0002e+03_rb,1.0008e+03_rb,1.0017e+03_rb,1.0008e+03_rb/)
      kbo(:,32,14) = (/ &
     &8.6070e+02_rb,8.6544e+02_rb,8.6833e+02_rb,8.7044e+02_rb,8.7331e+02_rb/)
      kbo(:,33,14) = (/ &
     &7.4260e+02_rb,7.4642e+02_rb,7.5058e+02_rb,7.5528e+02_rb,7.5991e+02_rb/)
      kbo(:,34,14) = (/ &
     &6.3829e+02_rb,6.4348e+02_rb,6.4940e+02_rb,6.5551e+02_rb,6.6078e+02_rb/)
      kbo(:,35,14) = (/ &
     &5.4846e+02_rb,5.5540e+02_rb,5.6285e+02_rb,5.6932e+02_rb,5.7603e+02_rb/)
      kbo(:,36,14) = (/ &
     &4.7198e+02_rb,4.7993e+02_rb,4.8784e+02_rb,4.9478e+02_rb,5.0307e+02_rb/)
      kbo(:,37,14) = (/ &
     &4.0498e+02_rb,4.1318e+02_rb,4.2147e+02_rb,4.2948e+02_rb,4.3820e+02_rb/)
      kbo(:,38,14) = (/ &
     &3.4831e+02_rb,3.5680e+02_rb,3.6471e+02_rb,3.7305e+02_rb,3.8282e+02_rb/)
      kbo(:,39,14) = (/ &
     &3.0154e+02_rb,3.0929e+02_rb,3.1734e+02_rb,3.2632e+02_rb,3.3596e+02_rb/)
      kbo(:,40,14) = (/ &
     &2.6084e+02_rb,2.6836e+02_rb,2.7578e+02_rb,2.8455e+02_rb,2.9457e+02_rb/)
      kbo(:,41,14) = (/ &
     &2.2737e+02_rb,2.3379e+02_rb,2.4131e+02_rb,2.4917e+02_rb,2.5928e+02_rb/)
      kbo(:,42,14) = (/ &
     &1.9807e+02_rb,2.0493e+02_rb,2.1238e+02_rb,2.2005e+02_rb,2.2904e+02_rb/)
      kbo(:,43,14) = (/ &
     &1.7261e+02_rb,1.7936e+02_rb,1.8667e+02_rb,1.9473e+02_rb,2.0355e+02_rb/)
      kbo(:,44,14) = (/ &
     &1.5104e+02_rb,1.5759e+02_rb,1.6444e+02_rb,1.7229e+02_rb,1.8143e+02_rb/)
      kbo(:,45,14) = (/ &
     &1.3320e+02_rb,1.3908e+02_rb,1.4536e+02_rb,1.5288e+02_rb,1.6210e+02_rb/)
      kbo(:,46,14) = (/ &
     &1.1788e+02_rb,1.2339e+02_rb,1.2957e+02_rb,1.3630e+02_rb,1.4477e+02_rb/)
      kbo(:,47,14) = (/ &
     &1.0441e+02_rb,1.0994e+02_rb,1.1542e+02_rb,1.2186e+02_rb,1.2998e+02_rb/)
      kbo(:,48,14) = (/ &
     &9.2989e+01_rb,9.8393e+01_rb,1.0367e+02_rb,1.0970e+02_rb,1.1684e+02_rb/)
      kbo(:,49,14) = (/ &
     &8.3245e+01_rb,8.8109e+01_rb,9.3736e+01_rb,9.8882e+01_rb,1.0570e+02_rb/)
      kbo(:,50,14) = (/ &
     &7.4420e+01_rb,7.9603e+01_rb,8.4592e+01_rb,9.0187e+01_rb,9.6226e+01_rb/)
      kbo(:,51,14) = (/ &
     &6.6553e+01_rb,7.2000e+01_rb,7.7144e+01_rb,8.2193e+01_rb,8.8503e+01_rb/)
      kbo(:,52,14) = (/ &
     &5.9519e+01_rb,6.4793e+01_rb,7.0321e+01_rb,7.5806e+01_rb,8.1580e+01_rb/)
      kbo(:,53,14) = (/ &
     &5.3233e+01_rb,5.8402e+01_rb,6.4217e+01_rb,6.9900e+01_rb,7.5580e+01_rb/)
      kbo(:,54,14) = (/ &
     &4.7589e+01_rb,5.2923e+01_rb,5.8451e+01_rb,6.4196e+01_rb,7.0322e+01_rb/)
      kbo(:,55,14) = (/ &
     &4.2742e+01_rb,4.7820e+01_rb,5.3445e+01_rb,5.9447e+01_rb,6.5405e+01_rb/)
      kbo(:,56,14) = (/ &
     &3.8175e+01_rb,4.3483e+01_rb,4.9069e+01_rb,5.5046e+01_rb,6.1266e+01_rb/)
      kbo(:,57,14) = (/ &
     &3.4510e+01_rb,3.9326e+01_rb,4.4806e+01_rb,5.1166e+01_rb,5.7169e+01_rb/)
      kbo(:,58,14) = (/ &
     &3.1419e+01_rb,3.6120e+01_rb,4.1281e+01_rb,4.7436e+01_rb,5.4030e+01_rb/)
      kbo(:,59,14) = (/ &
     &3.0087e+01_rb,3.4624e+01_rb,3.9514e+01_rb,4.5129e+01_rb,5.1873e+01_rb/)
      kbo(:,13,15) = (/ &
     &5.1911e+03_rb,5.1717e+03_rb,5.1553e+03_rb,5.1395e+03_rb,5.1397e+03_rb/)
      kbo(:,14,15) = (/ &
     &5.8547e+03_rb,5.8285e+03_rb,5.7963e+03_rb,5.7766e+03_rb,5.7715e+03_rb/)
      kbo(:,15,15) = (/ &
     &6.5298e+03_rb,6.4858e+03_rb,6.4506e+03_rb,6.4312e+03_rb,6.4243e+03_rb/)
      kbo(:,16,15) = (/ &
     &7.2117e+03_rb,7.1510e+03_rb,7.1189e+03_rb,7.1000e+03_rb,7.0870e+03_rb/)
      kbo(:,17,15) = (/ &
     &7.8953e+03_rb,7.8291e+03_rb,7.7890e+03_rb,7.7670e+03_rb,7.7489e+03_rb/)
      kbo(:,18,15) = (/ &
     &8.5855e+03_rb,8.5208e+03_rb,8.4668e+03_rb,8.4206e+03_rb,8.3980e+03_rb/)
      kbo(:,19,15) = (/ &
     &9.2674e+03_rb,9.2049e+03_rb,9.1326e+03_rb,9.0736e+03_rb,9.0525e+03_rb/)
      kbo(:,20,15) = (/ &
     &9.9108e+03_rb,9.8356e+03_rb,9.7636e+03_rb,9.7141e+03_rb,9.6905e+03_rb/)
      kbo(:,21,15) = (/ &
     &1.0486e+04_rb,1.0415e+04_rb,1.0355e+04_rb,1.0317e+04_rb,1.0287e+04_rb/)
      kbo(:,22,15) = (/ &
     &1.0985e+04_rb,1.0920e+04_rb,1.0875e+04_rb,1.0845e+04_rb,1.0804e+04_rb/)
      kbo(:,23,15) = (/ &
     &1.1398e+04_rb,1.1342e+04_rb,1.1308e+04_rb,1.1276e+04_rb,1.1234e+04_rb/)
      kbo(:,24,15) = (/ &
     &1.1709e+04_rb,1.1672e+04_rb,1.1645e+04_rb,1.1614e+04_rb,1.1576e+04_rb/)
      kbo(:,25,15) = (/ &
     &1.1914e+04_rb,1.1896e+04_rb,1.1878e+04_rb,1.1862e+04_rb,1.1837e+04_rb/)
      kbo(:,26,15) = (/ &
     &1.2040e+04_rb,1.2031e+04_rb,1.2029e+04_rb,1.2020e+04_rb,1.1998e+04_rb/)
      kbo(:,27,15) = (/ &
     &1.2071e+04_rb,1.2081e+04_rb,1.2089e+04_rb,1.2085e+04_rb,1.2069e+04_rb/)
      kbo(:,28,15) = (/ &
     &1.2021e+04_rb,1.2058e+04_rb,1.2068e+04_rb,1.2068e+04_rb,1.2073e+04_rb/)
      kbo(:,29,15) = (/ &
     &1.1886e+04_rb,1.1940e+04_rb,1.1970e+04_rb,1.1995e+04_rb,1.2017e+04_rb/)
      kbo(:,30,15) = (/ &
     &1.1676e+04_rb,1.1749e+04_rb,1.1808e+04_rb,1.1861e+04_rb,1.1908e+04_rb/)
      kbo(:,31,15) = (/ &
     &1.1416e+04_rb,1.1515e+04_rb,1.1609e+04_rb,1.1694e+04_rb,1.1764e+04_rb/)
      kbo(:,32,15) = (/ &
     &1.1108e+04_rb,1.1250e+04_rb,1.1385e+04_rb,1.1503e+04_rb,1.1598e+04_rb/)
      kbo(:,33,15) = (/ &
     &1.0773e+04_rb,1.0963e+04_rb,1.1141e+04_rb,1.1294e+04_rb,1.1425e+04_rb/)
      kbo(:,34,15) = (/ &
     &1.0438e+04_rb,1.0673e+04_rb,1.0885e+04_rb,1.1077e+04_rb,1.1249e+04_rb/)
      kbo(:,35,15) = (/ &
     &1.0089e+04_rb,1.0368e+04_rb,1.0621e+04_rb,1.0852e+04_rb,1.1064e+04_rb/)
      kbo(:,36,15) = (/ &
     &9.7201e+03_rb,1.0041e+04_rb,1.0341e+04_rb,1.0614e+04_rb,1.0861e+04_rb/)
      kbo(:,37,15) = (/ &
     &9.3163e+03_rb,9.6805e+03_rb,1.0024e+04_rb,1.0342e+04_rb,1.0632e+04_rb/)
      kbo(:,38,15) = (/ &
     &8.9272e+03_rb,9.3307e+03_rb,9.7169e+03_rb,1.0074e+04_rb,1.0402e+04_rb/)
      kbo(:,39,15) = (/ &
     &8.5526e+03_rb,8.9993e+03_rb,9.4206e+03_rb,9.8154e+03_rb,1.0180e+04_rb/)
      kbo(:,40,15) = (/ &
     &8.1494e+03_rb,8.6355e+03_rb,9.0962e+03_rb,9.5285e+03_rb,9.9305e+03_rb/)
      kbo(:,41,15) = (/ &
     &7.7529e+03_rb,8.2788e+03_rb,8.7766e+03_rb,9.2446e+03_rb,9.6773e+03_rb/)
      kbo(:,42,15) = (/ &
     &7.3760e+03_rb,7.9339e+03_rb,8.4681e+03_rb,8.9681e+03_rb,9.4340e+03_rb/)
      kbo(:,43,15) = (/ &
     &6.9819e+03_rb,7.5721e+03_rb,8.1409e+03_rb,8.6746e+03_rb,9.1691e+03_rb/)
      kbo(:,44,15) = (/ &
     &6.5827e+03_rb,7.2081e+03_rb,7.8084e+03_rb,8.3740e+03_rb,8.8982e+03_rb/)
      kbo(:,45,15) = (/ &
     &6.1901e+03_rb,6.8485e+03_rb,7.4795e+03_rb,8.0786e+03_rb,8.6303e+03_rb/)
      kbo(:,46,15) = (/ &
     &5.8024e+03_rb,6.4801e+03_rb,7.1413e+03_rb,7.7702e+03_rb,8.3548e+03_rb/)
      kbo(:,47,15) = (/ &
     &5.3938e+03_rb,6.0870e+03_rb,6.7778e+03_rb,7.4341e+03_rb,8.0495e+03_rb/)
      kbo(:,48,15) = (/ &
     &5.0092e+03_rb,5.7052e+03_rb,6.4111e+03_rb,7.0962e+03_rb,7.7409e+03_rb/)
      kbo(:,49,15) = (/ &
     &4.6045e+03_rb,5.3446e+03_rb,6.0469e+03_rb,6.7562e+03_rb,7.4302e+03_rb/)
      kbo(:,50,15) = (/ &
     &4.2414e+03_rb,4.9940e+03_rb,5.7177e+03_rb,6.4318e+03_rb,7.1308e+03_rb/)
      kbo(:,51,15) = (/ &
     &3.9058e+03_rb,4.6435e+03_rb,5.4144e+03_rb,6.1320e+03_rb,6.8340e+03_rb/)
      kbo(:,52,15) = (/ &
     &3.5828e+03_rb,4.3242e+03_rb,5.0835e+03_rb,5.8308e+03_rb,6.5455e+03_rb/)
      kbo(:,53,15) = (/ &
     &3.2733e+03_rb,3.9950e+03_rb,4.7590e+03_rb,5.5300e+03_rb,6.2613e+03_rb/)
      kbo(:,54,15) = (/ &
     &2.9973e+03_rb,3.7033e+03_rb,4.4798e+03_rb,5.2468e+03_rb,6.0054e+03_rb/)
      kbo(:,55,15) = (/ &
     &2.7347e+03_rb,3.4529e+03_rb,4.2020e+03_rb,4.9932e+03_rb,5.7360e+03_rb/)
      kbo(:,56,15) = (/ &
     &2.4953e+03_rb,3.1956e+03_rb,3.9366e+03_rb,4.7176e+03_rb,5.4818e+03_rb/)
      kbo(:,57,15) = (/ &
     &2.2617e+03_rb,2.9413e+03_rb,3.6881e+03_rb,4.4499e+03_rb,5.2349e+03_rb/)
      kbo(:,58,15) = (/ &
     &2.0370e+03_rb,2.7148e+03_rb,3.4460e+03_rb,4.2224e+03_rb,4.9906e+03_rb/)
      kbo(:,59,15) = (/ &
     &1.9478e+03_rb,2.6130e+03_rb,3.3529e+03_rb,4.1205e+03_rb,4.8990e+03_rb/)
      kbo(:,13,16) = (/ &
     &1.0699e+04_rb,1.0668e+04_rb,1.0580e+04_rb,1.0493e+04_rb,1.0473e+04_rb/)
      kbo(:,14,16) = (/ &
     &1.2469e+04_rb,1.2443e+04_rb,1.2338e+04_rb,1.2279e+04_rb,1.2260e+04_rb/)
      kbo(:,15,16) = (/ &
     &1.4513e+04_rb,1.4466e+04_rb,1.4361e+04_rb,1.4360e+04_rb,1.4376e+04_rb/)
      kbo(:,16,16) = (/ &
     &1.6769e+04_rb,1.6706e+04_rb,1.6727e+04_rb,1.6775e+04_rb,1.6795e+04_rb/)
      kbo(:,17,16) = (/ &
     &1.9233e+04_rb,1.9285e+04_rb,1.9387e+04_rb,1.9445e+04_rb,1.9443e+04_rb/)
      kbo(:,18,16) = (/ &
     &2.2082e+04_rb,2.2141e+04_rb,2.2296e+04_rb,2.2362e+04_rb,2.2373e+04_rb/)
      kbo(:,19,16) = (/ &
     &2.5392e+04_rb,2.5419e+04_rb,2.5533e+04_rb,2.5623e+04_rb,2.5622e+04_rb/)
      kbo(:,20,16) = (/ &
     &2.9227e+04_rb,2.9228e+04_rb,2.9232e+04_rb,2.9272e+04_rb,2.9286e+04_rb/)
      kbo(:,21,16) = (/ &
     &3.3689e+04_rb,3.3556e+04_rb,3.3511e+04_rb,3.3436e+04_rb,3.3399e+04_rb/)
      kbo(:,22,16) = (/ &
     &3.8640e+04_rb,3.8494e+04_rb,3.8304e+04_rb,3.8118e+04_rb,3.8006e+04_rb/)
      kbo(:,23,16) = (/ &
     &4.4135e+04_rb,4.3894e+04_rb,4.3600e+04_rb,4.3191e+04_rb,4.3063e+04_rb/)
      kbo(:,24,16) = (/ &
     &5.0120e+04_rb,4.9723e+04_rb,4.9276e+04_rb,4.8847e+04_rb,4.8550e+04_rb/)
      kbo(:,25,16) = (/ &
     &5.6400e+04_rb,5.5824e+04_rb,5.5260e+04_rb,5.4753e+04_rb,5.4318e+04_rb/)
      kbo(:,26,16) = (/ &
     &6.2742e+04_rb,6.2101e+04_rb,6.1449e+04_rb,6.0838e+04_rb,6.0246e+04_rb/)
      kbo(:,27,16) = (/ &
     &6.9256e+04_rb,6.8498e+04_rb,6.7755e+04_rb,6.7035e+04_rb,6.6255e+04_rb/)
      kbo(:,28,16) = (/ &
     &7.5845e+04_rb,7.4890e+04_rb,7.4038e+04_rb,7.3125e+04_rb,7.2119e+04_rb/)
      kbo(:,29,16) = (/ &
     &8.2416e+04_rb,8.1292e+04_rb,8.0172e+04_rb,7.8988e+04_rb,7.7733e+04_rb/)
      kbo(:,30,16) = (/ &
     &8.8923e+04_rb,8.7524e+04_rb,8.6114e+04_rb,8.4633e+04_rb,8.3083e+04_rb/)
      kbo(:,31,16) = (/ &
     &9.5128e+04_rb,9.3474e+04_rb,9.1409e+04_rb,8.9609e+04_rb,8.8121e+04_rb/)
      kbo(:,32,16) = (/ &
     &1.0109e+05_rb,9.9056e+04_rb,9.6959e+04_rb,9.4838e+04_rb,9.2728e+04_rb/)
      kbo(:,33,16) = (/ &
     &1.0668e+05_rb,1.0426e+05_rb,1.0177e+05_rb,9.9309e+04_rb,9.6864e+04_rb/)
      kbo(:,34,16) = (/ &
     &1.1181e+05_rb,1.0901e+05_rb,1.0619e+05_rb,1.0339e+05_rb,1.0062e+05_rb/)
      kbo(:,35,16) = (/ &
     &1.1669e+05_rb,1.1352e+05_rb,1.1036e+05_rb,1.0724e+05_rb,1.0414e+05_rb/)
      kbo(:,36,16) = (/ &
     &1.2143e+05_rb,1.1741e+05_rb,1.1444e+05_rb,1.1099e+05_rb,1.0759e+05_rb/)
      kbo(:,37,16) = (/ &
     &1.2629e+05_rb,1.2248e+05_rb,1.1868e+05_rb,1.1492e+05_rb,1.1120e+05_rb/)
      kbo(:,38,16) = (/ &
     &1.3077e+05_rb,1.2669e+05_rb,1.2260e+05_rb,1.1857e+05_rb,1.1458e+05_rb/)
      kbo(:,39,16) = (/ &
     &1.3413e+05_rb,1.3013e+05_rb,1.2620e+05_rb,1.2191e+05_rb,1.1768e+05_rb/)
      kbo(:,40,16) = (/ &
     &1.3905e+05_rb,1.3448e+05_rb,1.2993e+05_rb,1.2540e+05_rb,1.2095e+05_rb/)
      kbo(:,41,16) = (/ &
     &1.4302e+05_rb,1.3822e+05_rb,1.3346e+05_rb,1.2873e+05_rb,1.2407e+05_rb/)
      kbo(:,42,16) = (/ &
     &1.4675e+05_rb,1.4173e+05_rb,1.3673e+05_rb,1.3182e+05_rb,1.2698e+05_rb/)
      kbo(:,43,16) = (/ &
     &1.5047e+05_rb,1.4528e+05_rb,1.4010e+05_rb,1.3500e+05_rb,1.2941e+05_rb/)
      kbo(:,44,16) = (/ &
     &1.5413e+05_rb,1.4878e+05_rb,1.4344e+05_rb,1.3816e+05_rb,1.3300e+05_rb/)
      kbo(:,45,16) = (/ &
     &1.5764e+05_rb,1.5214e+05_rb,1.4665e+05_rb,1.4055e+05_rb,1.3586e+05_rb/)
      kbo(:,46,16) = (/ &
     &1.6104e+05_rb,1.5546e+05_rb,1.4981e+05_rb,1.4422e+05_rb,1.3874e+05_rb/)
      kbo(:,47,16) = (/ &
     &1.6453e+05_rb,1.5891e+05_rb,1.5314e+05_rb,1.4742e+05_rb,1.4180e+05_rb/)
      kbo(:,48,16) = (/ &
     &1.6775e+05_rb,1.6219e+05_rb,1.5640e+05_rb,1.5054e+05_rb,1.4481e+05_rb/)
      kbo(:,49,16) = (/ &
     &1.7104e+05_rb,1.6528e+05_rb,1.5957e+05_rb,1.5363e+05_rb,1.4773e+05_rb/)
      kbo(:,50,16) = (/ &
     &1.7401e+05_rb,1.6818e+05_rb,1.6240e+05_rb,1.5648e+05_rb,1.5051e+05_rb/)
      kbo(:,51,16) = (/ &
     &1.7667e+05_rb,1.7104e+05_rb,1.6499e+05_rb,1.5912e+05_rb,1.5318e+05_rb/)
      kbo(:,52,16) = (/ &
     &1.7922e+05_rb,1.7356e+05_rb,1.6772e+05_rb,1.6169e+05_rb,1.5572e+05_rb/)
      kbo(:,53,16) = (/ &
     &1.8153e+05_rb,1.7622e+05_rb,1.7035e+05_rb,1.6422e+05_rb,1.5819e+05_rb/)
      kbo(:,54,16) = (/ &
     &1.8363e+05_rb,1.7848e+05_rb,1.7259e+05_rb,1.6657e+05_rb,1.6042e+05_rb/)
      kbo(:,55,16) = (/ &
     &1.8551e+05_rb,1.8043e+05_rb,1.7479e+05_rb,1.6867e+05_rb,1.6267e+05_rb/)
      kbo(:,56,16) = (/ &
     &1.8723e+05_rb,1.8233e+05_rb,1.7687e+05_rb,1.7089e+05_rb,1.6480e+05_rb/)
      kbo(:,57,16) = (/ &
     &1.8886e+05_rb,1.8420e+05_rb,1.7883e+05_rb,1.7299e+05_rb,1.6681e+05_rb/)
      kbo(:,58,16) = (/ &
     &1.9037e+05_rb,1.8581e+05_rb,1.8067e+05_rb,1.7477e+05_rb,1.6881e+05_rb/)
      kbo(:,59,16) = (/ &
     &1.9100e+05_rb,1.8659e+05_rb,1.8132e+05_rb,1.7562e+05_rb,1.6956e+05_rb/)

      kao_mn2(:, 1) = (/ &
     & 5.12042e-08_rb, 5.51239e-08_rb, 5.93436e-08_rb, 6.38863e-08_rb, 6.87767e-08_rb, &
     & 7.40415e-08_rb, 7.97093e-08_rb, 8.58110e-08_rb, 9.23797e-08_rb, 9.94513e-08_rb, &
     & 1.07064e-07_rb, 1.15260e-07_rb, 1.24083e-07_rb, 1.33581e-07_rb, 1.43807e-07_rb, &
     & 1.54815e-07_rb, 1.66666e-07_rb, 1.79424e-07_rb, 1.93159e-07_rb/)
      kao_mn2(:, 2) = (/ &
     & 2.30938e-07_rb, 2.41696e-07_rb, 2.52955e-07_rb, 2.64738e-07_rb, 2.77071e-07_rb, &
     & 2.89978e-07_rb, 3.03486e-07_rb, 3.17623e-07_rb, 3.32419e-07_rb, 3.47904e-07_rb, &
     & 3.64111e-07_rb, 3.81072e-07_rb, 3.98824e-07_rb, 4.17402e-07_rb, 4.36846e-07_rb, &
     & 4.57196e-07_rb, 4.78494e-07_rb, 5.00784e-07_rb, 5.24112e-07_rb/)
      kao_mn2(:, 3) = (/ &
     & 6.70458e-07_rb, 7.04274e-07_rb, 7.39795e-07_rb, 7.77109e-07_rb, 8.16304e-07_rb, &
     & 8.57476e-07_rb, 9.00724e-07_rb, 9.46154e-07_rb, 9.93876e-07_rb, 1.04400e-06_rb, &
     & 1.09666e-06_rb, 1.15197e-06_rb, 1.21008e-06_rb, 1.27111e-06_rb, 1.33522e-06_rb, &
     & 1.40256e-06_rb, 1.47331e-06_rb, 1.54761e-06_rb, 1.62567e-06_rb/)
      kao_mn2(:, 4) = (/ &
     & 1.84182e-06_rb, 1.89203e-06_rb, 1.94360e-06_rb, 1.99658e-06_rb, 2.05101e-06_rb, &
     & 2.10692e-06_rb, 2.16435e-06_rb, 2.22335e-06_rb, 2.28396e-06_rb, 2.34622e-06_rb, &
     & 2.41017e-06_rb, 2.47587e-06_rb, 2.54337e-06_rb, 2.61270e-06_rb, 2.68392e-06_rb, &
     & 2.75708e-06_rb, 2.83224e-06_rb, 2.90944e-06_rb, 2.98875e-06_rb/)
      kao_mn2(:, 5) = (/ &
     & 3.41996e-06_rb, 3.32758e-06_rb, 3.23770e-06_rb, 3.15024e-06_rb, 3.06515e-06_rb, &
     & 2.98235e-06_rb, 2.90180e-06_rb, 2.82341e-06_rb, 2.74715e-06_rb, 2.67294e-06_rb, &
     & 2.60074e-06_rb, 2.53049e-06_rb, 2.46214e-06_rb, 2.39563e-06_rb, 2.33092e-06_rb, &
     & 2.26796e-06_rb, 2.20670e-06_rb, 2.14709e-06_rb, 2.08910e-06_rb/)
      kao_mn2(:, 6) = (/ &
     & 3.38746e-06_rb, 3.25966e-06_rb, 3.13669e-06_rb, 3.01836e-06_rb, 2.90449e-06_rb, &
     & 2.79491e-06_rb, 2.68947e-06_rb, 2.58801e-06_rb, 2.49037e-06_rb, 2.39642e-06_rb, &
     & 2.30601e-06_rb, 2.21902e-06_rb, 2.13530e-06_rb, 2.05475e-06_rb, 1.97723e-06_rb, &
     & 1.90264e-06_rb, 1.83086e-06_rb, 1.76179e-06_rb, 1.69532e-06_rb/)
      kao_mn2(:, 7) = (/ &
     & 3.17530e-06_rb, 3.07196e-06_rb, 2.97199e-06_rb, 2.87527e-06_rb, 2.78170e-06_rb, &
     & 2.69118e-06_rb, 2.60360e-06_rb, 2.51887e-06_rb, 2.43690e-06_rb, 2.35759e-06_rb, &
     & 2.28087e-06_rb, 2.20664e-06_rb, 2.13483e-06_rb, 2.06536e-06_rb, 1.99814e-06_rb, &
     & 1.93312e-06_rb, 1.87021e-06_rb, 1.80934e-06_rb, 1.75046e-06_rb/)
      kao_mn2(:, 8) = (/ &
     & 2.84701e-06_rb, 2.77007e-06_rb, 2.69521e-06_rb, 2.62237e-06_rb, 2.55150e-06_rb, &
     & 2.48254e-06_rb, 2.41545e-06_rb, 2.35017e-06_rb, 2.28666e-06_rb, 2.22486e-06_rb, &
     & 2.16473e-06_rb, 2.10623e-06_rb, 2.04930e-06_rb, 1.99392e-06_rb, 1.94003e-06_rb, &
     & 1.88760e-06_rb, 1.83659e-06_rb, 1.78695e-06_rb, 1.73866e-06_rb/)
      kao_mn2(:, 9) = (/ &
     & 2.79917e-06_rb, 2.73207e-06_rb, 2.66658e-06_rb, 2.60266e-06_rb, 2.54027e-06_rb, &
     & 2.47937e-06_rb, 2.41994e-06_rb, 2.36192e-06_rb, 2.30530e-06_rb, 2.25004e-06_rb, &
     & 2.19610e-06_rb, 2.14346e-06_rb, 2.09208e-06_rb, 2.04193e-06_rb, 1.99298e-06_rb, &
     & 1.94520e-06_rb, 1.89857e-06_rb, 1.85306e-06_rb, 1.80864e-06_rb/)
      kao_mn2(:,10) = (/ &
     & 2.74910e-06_rb, 2.64462e-06_rb, 2.54412e-06_rb, 2.44743e-06_rb, 2.35442e-06_rb, &
     & 2.26495e-06_rb, 2.17887e-06_rb, 2.09606e-06_rb, 2.01641e-06_rb, 1.93978e-06_rb, &
     & 1.86606e-06_rb, 1.79514e-06_rb, 1.72692e-06_rb, 1.66129e-06_rb, 1.59815e-06_rb, &
     & 1.53742e-06_rb, 1.47899e-06_rb, 1.42278e-06_rb, 1.36871e-06_rb/)
      kao_mn2(:,11) = (/ &
     & 2.63952e-06_rb, 2.60263e-06_rb, 2.56626e-06_rb, 2.53039e-06_rb, 2.49503e-06_rb, &
     & 2.46016e-06_rb, 2.42578e-06_rb, 2.39188e-06_rb, 2.35845e-06_rb, 2.32549e-06_rb, &
     & 2.29299e-06_rb, 2.26094e-06_rb, 2.22934e-06_rb, 2.19819e-06_rb, 2.16747e-06_rb, &
     & 2.13717e-06_rb, 2.10731e-06_rb, 2.07786e-06_rb, 2.04882e-06_rb/)
      kao_mn2(:,12) = (/ &
     & 2.94106e-06_rb, 2.82819e-06_rb, 2.71966e-06_rb, 2.61528e-06_rb, 2.51492e-06_rb, &
     & 2.41841e-06_rb, 2.32560e-06_rb, 2.23635e-06_rb, 2.15053e-06_rb, 2.06800e-06_rb, &
     & 1.98863e-06_rb, 1.91232e-06_rb, 1.83893e-06_rb, 1.76836e-06_rb, 1.70049e-06_rb, &
     & 1.63524e-06_rb, 1.57248e-06_rb, 1.51214e-06_rb, 1.45411e-06_rb/)
      kao_mn2(:,13) = (/ &
     & 2.94607e-06_rb, 2.87369e-06_rb, 2.80309e-06_rb, 2.73422e-06_rb, 2.66705e-06_rb, &
     & 2.60152e-06_rb, 2.53760e-06_rb, 2.47526e-06_rb, 2.41445e-06_rb, 2.35513e-06_rb, &
     & 2.29726e-06_rb, 2.24082e-06_rb, 2.18577e-06_rb, 2.13207e-06_rb, 2.07969e-06_rb, &
     & 2.02859e-06_rb, 1.97875e-06_rb, 1.93014e-06_rb, 1.88272e-06_rb/)
      kao_mn2(:,14) = (/ &
     & 2.58051e-06_rb, 2.48749e-06_rb, 2.39782e-06_rb, 2.31139e-06_rb, 2.22807e-06_rb, &
     & 2.14775e-06_rb, 2.07033e-06_rb, 1.99570e-06_rb, 1.92376e-06_rb, 1.85441e-06_rb, &
     & 1.78756e-06_rb, 1.72313e-06_rb, 1.66101e-06_rb, 1.60114e-06_rb, 1.54342e-06_rb, &
     & 1.48778e-06_rb, 1.43415e-06_rb, 1.38245e-06_rb, 1.33262e-06_rb/)
      kao_mn2(:,15) = (/ &
     & 3.03447e-06_rb, 2.88559e-06_rb, 2.74401e-06_rb, 2.60938e-06_rb, 2.48135e-06_rb, &
     & 2.35961e-06_rb, 2.24384e-06_rb, 2.13375e-06_rb, 2.02906e-06_rb, 1.92951e-06_rb, &
     & 1.83484e-06_rb, 1.74481e-06_rb, 1.65921e-06_rb, 1.57780e-06_rb, 1.50039e-06_rb, &
     & 1.42677e-06_rb, 1.35677e-06_rb, 1.29020e-06_rb, 1.22690e-06_rb/)
      kao_mn2(:,16) = (/ &
     & 1.48655e-06_rb, 1.48283e-06_rb, 1.47913e-06_rb, 1.47543e-06_rb, 1.47174e-06_rb, &
     & 1.46806e-06_rb, 1.46439e-06_rb, 1.46072e-06_rb, 1.45707e-06_rb, 1.45343e-06_rb, &
     & 1.44979e-06_rb, 1.44617e-06_rb, 1.44255e-06_rb, 1.43894e-06_rb, 1.43534e-06_rb, &
     & 1.43176e-06_rb, 1.42817e-06_rb, 1.42460e-06_rb, 1.42104e-06_rb/)
      kbo_mn2(:, 1) = (/ &
     & 5.12042e-08_rb, 5.51239e-08_rb, 5.93436e-08_rb, 6.38863e-08_rb, 6.87767e-08_rb, &
     & 7.40415e-08_rb, 7.97093e-08_rb, 8.58110e-08_rb, 9.23797e-08_rb, 9.94513e-08_rb, &
     & 1.07064e-07_rb, 1.15260e-07_rb, 1.24083e-07_rb, 1.33581e-07_rb, 1.43807e-07_rb, &
     & 1.54815e-07_rb, 1.66666e-07_rb, 1.79424e-07_rb, 1.93159e-07_rb/)
      kbo_mn2(:, 2) = (/ &
     & 2.30938e-07_rb, 2.41696e-07_rb, 2.52955e-07_rb, 2.64738e-07_rb, 2.77071e-07_rb, &
     & 2.89978e-07_rb, 3.03486e-07_rb, 3.17623e-07_rb, 3.32419e-07_rb, 3.47904e-07_rb, &
     & 3.64111e-07_rb, 3.81072e-07_rb, 3.98824e-07_rb, 4.17402e-07_rb, 4.36846e-07_rb, &
     & 4.57196e-07_rb, 4.78494e-07_rb, 5.00784e-07_rb, 5.24112e-07_rb/)
      kbo_mn2(:, 3) = (/ &
     & 6.70458e-07_rb, 7.04274e-07_rb, 7.39795e-07_rb, 7.77109e-07_rb, 8.16304e-07_rb, &
     & 8.57476e-07_rb, 9.00724e-07_rb, 9.46154e-07_rb, 9.93876e-07_rb, 1.04400e-06_rb, &
     & 1.09666e-06_rb, 1.15197e-06_rb, 1.21008e-06_rb, 1.27111e-06_rb, 1.33522e-06_rb, &
     & 1.40256e-06_rb, 1.47331e-06_rb, 1.54761e-06_rb, 1.62567e-06_rb/)
      kbo_mn2(:, 4) = (/ &
     & 1.84182e-06_rb, 1.89203e-06_rb, 1.94360e-06_rb, 1.99658e-06_rb, 2.05101e-06_rb, &
     & 2.10692e-06_rb, 2.16435e-06_rb, 2.22335e-06_rb, 2.28396e-06_rb, 2.34622e-06_rb, &
     & 2.41017e-06_rb, 2.47587e-06_rb, 2.54337e-06_rb, 2.61270e-06_rb, 2.68392e-06_rb, &
     & 2.75708e-06_rb, 2.83224e-06_rb, 2.90944e-06_rb, 2.98875e-06_rb/)
      kbo_mn2(:, 5) = (/ &
     & 3.41996e-06_rb, 3.32758e-06_rb, 3.23770e-06_rb, 3.15024e-06_rb, 3.06515e-06_rb, &
     & 2.98235e-06_rb, 2.90180e-06_rb, 2.82341e-06_rb, 2.74715e-06_rb, 2.67294e-06_rb, &
     & 2.60074e-06_rb, 2.53049e-06_rb, 2.46214e-06_rb, 2.39563e-06_rb, 2.33092e-06_rb, &
     & 2.26796e-06_rb, 2.20670e-06_rb, 2.14709e-06_rb, 2.08910e-06_rb/)
      kbo_mn2(:, 6) = (/ &
     & 3.38746e-06_rb, 3.25966e-06_rb, 3.13669e-06_rb, 3.01836e-06_rb, 2.90449e-06_rb, &
     & 2.79491e-06_rb, 2.68947e-06_rb, 2.58801e-06_rb, 2.49037e-06_rb, 2.39642e-06_rb, &
     & 2.30601e-06_rb, 2.21902e-06_rb, 2.13530e-06_rb, 2.05475e-06_rb, 1.97723e-06_rb, &
     & 1.90264e-06_rb, 1.83086e-06_rb, 1.76179e-06_rb, 1.69532e-06_rb/)
      kbo_mn2(:, 7) = (/ &
     & 3.17530e-06_rb, 3.07196e-06_rb, 2.97199e-06_rb, 2.87527e-06_rb, 2.78170e-06_rb, &
     & 2.69118e-06_rb, 2.60360e-06_rb, 2.51887e-06_rb, 2.43690e-06_rb, 2.35759e-06_rb, &
     & 2.28087e-06_rb, 2.20664e-06_rb, 2.13483e-06_rb, 2.06536e-06_rb, 1.99814e-06_rb, &
     & 1.93312e-06_rb, 1.87021e-06_rb, 1.80934e-06_rb, 1.75046e-06_rb/)
      kbo_mn2(:, 8) = (/ &
     & 2.84701e-06_rb, 2.77007e-06_rb, 2.69521e-06_rb, 2.62237e-06_rb, 2.55150e-06_rb, &
     & 2.48254e-06_rb, 2.41545e-06_rb, 2.35017e-06_rb, 2.28666e-06_rb, 2.22486e-06_rb, &
     & 2.16473e-06_rb, 2.10623e-06_rb, 2.04930e-06_rb, 1.99392e-06_rb, 1.94003e-06_rb, &
     & 1.88760e-06_rb, 1.83659e-06_rb, 1.78695e-06_rb, 1.73866e-06_rb/)
      kbo_mn2(:, 9) = (/ &
     & 2.79917e-06_rb, 2.73207e-06_rb, 2.66658e-06_rb, 2.60266e-06_rb, 2.54027e-06_rb, &
     & 2.47937e-06_rb, 2.41994e-06_rb, 2.36192e-06_rb, 2.30530e-06_rb, 2.25004e-06_rb, &
     & 2.19610e-06_rb, 2.14346e-06_rb, 2.09208e-06_rb, 2.04193e-06_rb, 1.99298e-06_rb, &
     & 1.94520e-06_rb, 1.89857e-06_rb, 1.85306e-06_rb, 1.80864e-06_rb/)
      kbo_mn2(:,10) = (/ &
     & 2.74910e-06_rb, 2.64462e-06_rb, 2.54412e-06_rb, 2.44743e-06_rb, 2.35442e-06_rb, &
     & 2.26495e-06_rb, 2.17887e-06_rb, 2.09606e-06_rb, 2.01641e-06_rb, 1.93978e-06_rb, &
     & 1.86606e-06_rb, 1.79514e-06_rb, 1.72692e-06_rb, 1.66129e-06_rb, 1.59815e-06_rb, &
     & 1.53742e-06_rb, 1.47899e-06_rb, 1.42278e-06_rb, 1.36871e-06_rb/)
      kbo_mn2(:,11) = (/ &
     & 2.63952e-06_rb, 2.60263e-06_rb, 2.56626e-06_rb, 2.53039e-06_rb, 2.49503e-06_rb, &
     & 2.46016e-06_rb, 2.42578e-06_rb, 2.39188e-06_rb, 2.35845e-06_rb, 2.32549e-06_rb, &
     & 2.29299e-06_rb, 2.26094e-06_rb, 2.22934e-06_rb, 2.19819e-06_rb, 2.16747e-06_rb, &
     & 2.13717e-06_rb, 2.10731e-06_rb, 2.07786e-06_rb, 2.04882e-06_rb/)
      kbo_mn2(:,12) = (/ &
     & 2.94106e-06_rb, 2.82819e-06_rb, 2.71966e-06_rb, 2.61528e-06_rb, 2.51492e-06_rb, &
     & 2.41841e-06_rb, 2.32560e-06_rb, 2.23635e-06_rb, 2.15053e-06_rb, 2.06800e-06_rb, &
     & 1.98863e-06_rb, 1.91232e-06_rb, 1.83893e-06_rb, 1.76836e-06_rb, 1.70049e-06_rb, &
     & 1.63524e-06_rb, 1.57248e-06_rb, 1.51214e-06_rb, 1.45411e-06_rb/)
      kbo_mn2(:,13) = (/ &
     & 2.94607e-06_rb, 2.87369e-06_rb, 2.80309e-06_rb, 2.73422e-06_rb, 2.66705e-06_rb, &
     & 2.60152e-06_rb, 2.53760e-06_rb, 2.47526e-06_rb, 2.41445e-06_rb, 2.35513e-06_rb, &
     & 2.29726e-06_rb, 2.24082e-06_rb, 2.18577e-06_rb, 2.13207e-06_rb, 2.07969e-06_rb, &
     & 2.02859e-06_rb, 1.97875e-06_rb, 1.93014e-06_rb, 1.88272e-06_rb/)
      kbo_mn2(:,14) = (/ &
     & 2.58051e-06_rb, 2.48749e-06_rb, 2.39782e-06_rb, 2.31139e-06_rb, 2.22807e-06_rb, &
     & 2.14775e-06_rb, 2.07033e-06_rb, 1.99570e-06_rb, 1.92376e-06_rb, 1.85441e-06_rb, &
     & 1.78756e-06_rb, 1.72313e-06_rb, 1.66101e-06_rb, 1.60114e-06_rb, 1.54342e-06_rb, &
     & 1.48778e-06_rb, 1.43415e-06_rb, 1.38245e-06_rb, 1.33262e-06_rb/)
      kbo_mn2(:,15) = (/ &
     & 3.03447e-06_rb, 2.88559e-06_rb, 2.74401e-06_rb, 2.60938e-06_rb, 2.48135e-06_rb, &
     & 2.35961e-06_rb, 2.24384e-06_rb, 2.13375e-06_rb, 2.02906e-06_rb, 1.92951e-06_rb, &
     & 1.83484e-06_rb, 1.74481e-06_rb, 1.65921e-06_rb, 1.57780e-06_rb, 1.50039e-06_rb, &
     & 1.42677e-06_rb, 1.35677e-06_rb, 1.29020e-06_rb, 1.22690e-06_rb/)
      kbo_mn2(:,16) = (/ &
     & 1.48655e-06_rb, 1.48283e-06_rb, 1.47913e-06_rb, 1.47543e-06_rb, 1.47174e-06_rb, &
     & 1.46806e-06_rb, 1.46439e-06_rb, 1.46072e-06_rb, 1.45707e-06_rb, 1.45343e-06_rb, &
     & 1.44979e-06_rb, 1.44617e-06_rb, 1.44255e-06_rb, 1.43894e-06_rb, 1.43534e-06_rb, &
     & 1.43176e-06_rb, 1.42817e-06_rb, 1.42460e-06_rb, 1.42104e-06_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &3.6742e-02_rb,1.0664e-01_rb,2.6132e-01_rb,2.7906e-01_rb,2.8151e-01_rb,2.7465e-01_rb, &
     &2.8530e-01_rb,2.9123e-01_rb,3.0697e-01_rb,3.1801e-01_rb,3.2444e-01_rb,2.7746e-01_rb, &
     &3.1994e-01_rb,2.9750e-01_rb,2.1226e-01_rb,1.2847e-01_rb/)
      forrefo(2,:) = (/ &
     &4.0450e-02_rb,1.1085e-01_rb,2.9205e-01_rb,3.1934e-01_rb,3.1739e-01_rb,3.1450e-01_rb, &
     &3.2797e-01_rb,3.2223e-01_rb,3.3099e-01_rb,3.4800e-01_rb,3.4046e-01_rb,3.5700e-01_rb, &
     &3.8264e-01_rb,3.6679e-01_rb,3.3481e-01_rb,3.2113e-01_rb/)
      forrefo(3,:) = (/ &
     &4.6952e-02_rb,1.1999e-01_rb,3.1473e-01_rb,3.7015e-01_rb,3.6913e-01_rb,3.6352e-01_rb, &
     &3.7754e-01_rb,3.7402e-01_rb,3.7113e-01_rb,3.7720e-01_rb,3.8365e-01_rb,4.0876e-01_rb, &
     &4.2968e-01_rb,4.4186e-01_rb,4.3468e-01_rb,4.7083e-01_rb/)
      forrefo(4,:) = (/ &
     &7.0645e-02_rb,1.6618e-01_rb,2.8516e-01_rb,3.1819e-01_rb,3.0131e-01_rb,2.9552e-01_rb, &
     &2.8972e-01_rb,2.9348e-01_rb,2.8668e-01_rb,2.8483e-01_rb,2.8130e-01_rb,2.7757e-01_rb, &
     &2.9735e-01_rb,3.1684e-01_rb,3.0681e-01_rb,3.6778e-01_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 2.16803e+00_rb, 1.98236e+00_rb, 1.81260e+00_rb, 1.65737e+00_rb, 1.51544e+00_rb, &
     & 1.38567e+00_rb, 1.26700e+00_rb, 1.15850e+00_rb, 1.05929e+00_rb, 9.68576e-01_rb/)
      selfrefo(:, 2) = (/ &
     & 3.70149e+00_rb, 3.43145e+00_rb, 3.18110e+00_rb, 2.94902e+00_rb, 2.73387e+00_rb, &
     & 2.53441e+00_rb, 2.34951e+00_rb, 2.17810e+00_rb, 2.01919e+00_rb, 1.87188e+00_rb/)
      selfrefo(:, 3) = (/ &
     & 6.17433e+00_rb, 5.62207e+00_rb, 5.11920e+00_rb, 4.66131e+00_rb, 4.24438e+00_rb, &
     & 3.86474e+00_rb, 3.51906e+00_rb, 3.20430e+00_rb, 2.91769e+00_rb, 2.65672e+00_rb/)
      selfrefo(:, 4) = (/ &
     & 6.56459e+00_rb, 5.94787e+00_rb, 5.38910e+00_rb, 4.88282e+00_rb, 4.42410e+00_rb, &
     & 4.00848e+00_rb, 3.63190e+00_rb, 3.29070e+00_rb, 2.98155e+00_rb, 2.70145e+00_rb/)
      selfrefo(:, 5) = (/ &
     & 6.49581e+00_rb, 5.91114e+00_rb, 5.37910e+00_rb, 4.89494e+00_rb, 4.45436e+00_rb, &
     & 4.05344e+00_rb, 3.68860e+00_rb, 3.35660e+00_rb, 3.05448e+00_rb, 2.77956e+00_rb/)
      selfrefo(:, 6) = (/ &
     & 6.50189e+00_rb, 5.89381e+00_rb, 5.34260e+00_rb, 4.84294e+00_rb, 4.39001e+00_rb, &
     & 3.97944e+00_rb, 3.60727e+00_rb, 3.26990e+00_rb, 2.96409e+00_rb, 2.68687e+00_rb/)
      selfrefo(:, 7) = (/ &
     & 6.64768e+00_rb, 6.01719e+00_rb, 5.44650e+00_rb, 4.92993e+00_rb, 4.46236e+00_rb, &
     & 4.03914e+00_rb, 3.65605e+00_rb, 3.30930e+00_rb, 2.99543e+00_rb, 2.71134e+00_rb/)
      selfrefo(:, 8) = (/ &
     & 6.43744e+00_rb, 5.87166e+00_rb, 5.35560e+00_rb, 4.88490e+00_rb, 4.45557e+00_rb, &
     & 4.06397e+00_rb, 3.70679e+00_rb, 3.38100e+00_rb, 3.08384e+00_rb, 2.81281e+00_rb/)
      selfrefo(:, 9) = (/ &
     & 6.55466e+00_rb, 5.99777e+00_rb, 5.48820e+00_rb, 5.02192e+00_rb, 4.59525e+00_rb, &
     & 4.20484e+00_rb, 3.84759e+00_rb, 3.52070e+00_rb, 3.22158e+00_rb, 2.94787e+00_rb/)
      selfrefo(:,10) = (/ &
     & 6.84510e+00_rb, 6.26933e+00_rb, 5.74200e+00_rb, 5.25902e+00_rb, 4.81667e+00_rb, &
     & 4.41152e+00_rb, 4.04046e+00_rb, 3.70060e+00_rb, 3.38933e+00_rb, 3.10424e+00_rb/)
      selfrefo(:,11) = (/ &
     & 6.83128e+00_rb, 6.25536e+00_rb, 5.72800e+00_rb, 5.24510e+00_rb, 4.80291e+00_rb, &
     & 4.39799e+00_rb, 4.02722e+00_rb, 3.68770e+00_rb, 3.37681e+00_rb, 3.09212e+00_rb/)
      selfrefo(:,12) = (/ &
     & 7.35969e+00_rb, 6.61719e+00_rb, 5.94960e+00_rb, 5.34936e+00_rb, 4.80968e+00_rb, &
     & 4.32445e+00_rb, 3.88817e+00_rb, 3.49590e+00_rb, 3.14321e+00_rb, 2.82610e+00_rb/)
      selfrefo(:,13) = (/ &
     & 7.50064e+00_rb, 6.80749e+00_rb, 6.17840e+00_rb, 5.60744e+00_rb, 5.08925e+00_rb, &
     & 4.61894e+00_rb, 4.19210e+00_rb, 3.80470e+00_rb, 3.45310e+00_rb, 3.13399e+00_rb/)
      selfrefo(:,14) = (/ &
     & 7.40801e+00_rb, 6.71328e+00_rb, 6.08370e+00_rb, 5.51316e+00_rb, 4.99613e+00_rb, &
     & 4.52759e+00_rb, 4.10298e+00_rb, 3.71820e+00_rb, 3.36950e+00_rb, 3.05351e+00_rb/)
      selfrefo(:,15) = (/ &
     & 7.51895e+00_rb, 6.68846e+00_rb, 5.94970e+00_rb, 5.29254e+00_rb, 4.70796e+00_rb, &
     & 4.18795e+00_rb, 3.72538e+00_rb, 3.31390e+00_rb, 2.94787e+00_rb, 2.62227e+00_rb/)
      selfrefo(:,16) = (/ &
     & 7.84774e+00_rb, 6.80673e+00_rb, 5.90380e+00_rb, 5.12065e+00_rb, 4.44138e+00_rb, &
     & 3.85223e+00_rb, 3.34122e+00_rb, 2.89800e+00_rb, 2.51357e+00_rb, 2.18014e+00_rb/)

      end subroutine lw_kgb01

! **************************************************************************
      subroutine lw_kgb02
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg02, only : fracrefao, fracrefbo, kao, kbo, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level: P = 1053.630 mbar, T = 294.2 K
      fracrefao(:) = (/ &
        1.6388e-01_rb, 1.5241e-01_rb, 1.4290e-01_rb, 1.2864e-01_rb, &
        1.1615e-01_rb, 1.0047e-01_rb, 8.0013e-02_rb, 6.0445e-02_rb, &
        4.0530e-02_rb, 4.3879e-03_rb, 3.5726e-03_rb, 2.7669e-03_rb, &
        2.0078e-03_rb, 1.2864e-03_rb, 4.7630e-04_rb, 6.9109e-05_rb/)

! Planck fraction mapping level: P = 3.206e-2 mb, T = 197.92 K
      fracrefbo(:) = (/ &
        1.4697e-01_rb, 1.4826e-01_rb, 1.4278e-01_rb, 1.3320e-01_rb, &
        1.1965e-01_rb, 1.0297e-01_rb, 8.4170e-02_rb, 6.3282e-02_rb, &
        4.2868e-02_rb, 4.6644e-03_rb, 3.8619e-03_rb, 3.0533e-03_rb, &
        2.2359e-03_rb, 1.4226e-03_rb, 5.3642e-04_rb, 7.6316e-05_rb/)

!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels > ~100mb and temperatures.  The first
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the corresponding TREF for this  pressure level, 
!     JT = 2 refers to the temperatureTREF-15, JT = 1 is for TREF-30, 
!     JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  The second 
!     index, JP, runs from 1 to 13 and refers to the corresponding 
!     pressure level in PREF (e.g. JP = 1 is for a pressure of 1053.63 mb).  
!     The third index, IG, goes from 1 to 16, and tells us which 
!     g-interval the absorption coefficients are for.

      kao(:, 1, 1) = (/ &
     &4.9444e-03_rb,5.9459e-03_rb,7.0909e-03_rb,8.2082e-03_rb,9.2071e-03_rb/)
      kao(:, 2, 1) = (/ &
     &3.6253e-03_rb,4.3860e-03_rb,5.2541e-03_rb,6.0419e-03_rb,6.8835e-03_rb/)
      kao(:, 3, 1) = (/ &
     &2.5338e-03_rb,3.0985e-03_rb,3.7400e-03_rb,4.2996e-03_rb,4.9675e-03_rb/)
      kao(:, 4, 1) = (/ &
     &1.7755e-03_rb,2.1973e-03_rb,2.6664e-03_rb,3.1111e-03_rb,3.6091e-03_rb/)
      kao(:, 5, 1) = (/ &
     &1.2565e-03_rb,1.5789e-03_rb,1.9196e-03_rb,2.3096e-03_rb,2.6519e-03_rb/)
      kao(:, 6, 1) = (/ &
     &8.8624e-04_rb,1.1175e-03_rb,1.3818e-03_rb,1.6934e-03_rb,1.9635e-03_rb/)
      kao(:, 7, 1) = (/ &
     &6.2843e-04_rb,7.9253e-04_rb,1.0012e-03_rb,1.2222e-03_rb,1.4734e-03_rb/)
      kao(:, 8, 1) = (/ &
     &4.5023e-04_rb,5.6483e-04_rb,7.1568e-04_rb,8.9026e-04_rb,1.0929e-03_rb/)
      kao(:, 9, 1) = (/ &
     &3.2520e-04_rb,4.0407e-04_rb,5.1209e-04_rb,6.4949e-04_rb,7.9581e-04_rb/)
      kao(:,10, 1) = (/ &
     &2.4126e-04_rb,2.9581e-04_rb,3.7404e-04_rb,4.7585e-04_rb,5.9085e-04_rb/)
      kao(:,11, 1) = (/ &
     &1.9501e-04_rb,2.4152e-04_rb,3.0649e-04_rb,3.9104e-04_rb,4.8881e-04_rb/)
      kao(:,12, 1) = (/ &
     &1.5881e-04_rb,1.9763e-04_rb,2.5177e-04_rb,3.2441e-04_rb,4.0632e-04_rb/)
      kao(:,13, 1) = (/ &
     &1.3003e-04_rb,1.6216e-04_rb,2.0768e-04_rb,2.6864e-04_rb,3.3497e-04_rb/)
      kao(:, 1, 2) = (/ &
     &1.0027e-02_rb,1.1791e-02_rb,1.3932e-02_rb,1.6666e-02_rb,2.0126e-02_rb/)
      kao(:, 2, 2) = (/ &
     &7.4056e-03_rb,8.8285e-03_rb,1.0460e-02_rb,1.2669e-02_rb,1.5282e-02_rb/)
      kao(:, 3, 2) = (/ &
     &5.2661e-03_rb,6.2928e-03_rb,7.5540e-03_rb,9.2387e-03_rb,1.1182e-02_rb/)
      kao(:, 4, 2) = (/ &
     &3.7830e-03_rb,4.5144e-03_rb,5.4622e-03_rb,6.6869e-03_rb,8.1435e-03_rb/)
      kao(:, 5, 2) = (/ &
     &2.7480e-03_rb,3.3042e-03_rb,3.9726e-03_rb,4.8258e-03_rb,5.9510e-03_rb/)
      kao(:, 6, 2) = (/ &
     &2.0063e-03_rb,2.3984e-03_rb,2.8713e-03_rb,3.4864e-03_rb,4.3135e-03_rb/)
      kao(:, 7, 2) = (/ &
     &1.4846e-03_rb,1.7453e-03_rb,2.0981e-03_rb,2.5444e-03_rb,3.1056e-03_rb/)
      kao(:, 8, 2) = (/ &
     &1.1210e-03_rb,1.2879e-03_rb,1.5449e-03_rb,1.8524e-03_rb,2.2562e-03_rb/)
      kao(:, 9, 2) = (/ &
     &8.5574e-04_rb,9.6423e-04_rb,1.1336e-03_rb,1.3659e-03_rb,1.6594e-03_rb/)
      kao(:,10, 2) = (/ &
     &6.6848e-04_rb,7.4016e-04_rb,8.5479e-04_rb,1.0274e-03_rb,1.2343e-03_rb/)
      kao(:,11, 2) = (/ &
     &5.5022e-04_rb,6.0473e-04_rb,6.9963e-04_rb,8.4106e-04_rb,1.0097e-03_rb/)
      kao(:,12, 2) = (/ &
     &4.4972e-04_rb,4.9436e-04_rb,5.7239e-04_rb,6.8738e-04_rb,8.3038e-04_rb/)
      kao(:,13, 2) = (/ &
     &3.6745e-04_rb,4.0460e-04_rb,4.6828e-04_rb,5.6427e-04_rb,6.8387e-04_rb/)
      kao(:, 1, 3) = (/ &
     &2.0475e-02_rb,2.4886e-02_rb,2.8056e-02_rb,3.1687e-02_rb,3.6011e-02_rb/)
      kao(:, 2, 3) = (/ &
     &1.5293e-02_rb,1.8649e-02_rb,2.1197e-02_rb,2.4106e-02_rb,2.7485e-02_rb/)
      kao(:, 3, 3) = (/ &
     &1.0918e-02_rb,1.3480e-02_rb,1.5496e-02_rb,1.7769e-02_rb,2.0398e-02_rb/)
      kao(:, 4, 3) = (/ &
     &7.7395e-03_rb,9.6625e-03_rb,1.1379e-02_rb,1.3130e-02_rb,1.5170e-02_rb/)
      kao(:, 5, 3) = (/ &
     &5.4995e-03_rb,6.9063e-03_rb,8.5233e-03_rb,9.8125e-03_rb,1.1368e-02_rb/)
      kao(:, 6, 3) = (/ &
     &3.9002e-03_rb,4.9307e-03_rb,6.2037e-03_rb,7.3100e-03_rb,8.4689e-03_rb/)
      kao(:, 7, 3) = (/ &
     &2.7369e-03_rb,3.5078e-03_rb,4.4352e-03_rb,5.5098e-03_rb,6.3463e-03_rb/)
      kao(:, 8, 3) = (/ &
     &1.9294e-03_rb,2.5028e-03_rb,3.1751e-03_rb,4.0255e-03_rb,4.7593e-03_rb/)
      kao(:, 9, 3) = (/ &
     &1.2806e-03_rb,1.7659e-03_rb,2.2786e-03_rb,2.8933e-03_rb,3.6112e-03_rb/)
      kao(:,10, 3) = (/ &
     &8.6429e-04_rb,1.2871e-03_rb,1.6661e-03_rb,2.1207e-03_rb,2.6956e-03_rb/)
      kao(:,11, 3) = (/ &
     &7.0989e-04_rb,1.0555e-03_rb,1.3691e-03_rb,1.7450e-03_rb,2.2184e-03_rb/)
      kao(:,12, 3) = (/ &
     &5.8630e-04_rb,8.6615e-04_rb,1.1265e-03_rb,1.4357e-03_rb,1.8221e-03_rb/)
      kao(:,13, 3) = (/ &
     &4.8372e-04_rb,7.1031e-04_rb,9.2618e-04_rb,1.1787e-03_rb,1.4998e-03_rb/)
      kao(:, 1, 4) = (/ &
     &3.4150e-02_rb,4.0367e-02_rb,4.9280e-02_rb,5.9100e-02_rb,6.9499e-02_rb/)
      kao(:, 2, 4) = (/ &
     &2.5504e-02_rb,3.0447e-02_rb,3.7321e-02_rb,4.4944e-02_rb,5.3298e-02_rb/)
      kao(:, 3, 4) = (/ &
     &1.8371e-02_rb,2.2085e-02_rb,2.7226e-02_rb,3.2994e-02_rb,3.9403e-02_rb/)
      kao(:, 4, 4) = (/ &
     &1.3235e-02_rb,1.6046e-02_rb,1.9813e-02_rb,2.4156e-02_rb,2.9082e-02_rb/)
      kao(:, 5, 4) = (/ &
     &9.5998e-03_rb,1.1709e-02_rb,1.4338e-02_rb,1.7707e-02_rb,2.1505e-02_rb/)
      kao(:, 6, 4) = (/ &
     &6.9022e-03_rb,8.5630e-03_rb,1.0395e-02_rb,1.2869e-02_rb,1.5787e-02_rb/)
      kao(:, 7, 4) = (/ &
     &5.0622e-03_rb,6.1861e-03_rb,7.5642e-03_rb,9.2835e-03_rb,1.1530e-02_rb/)
      kao(:, 8, 4) = (/ &
     &3.7734e-03_rb,4.4956e-03_rb,5.5749e-03_rb,6.7721e-03_rb,8.4253e-03_rb/)
      kao(:, 9, 4) = (/ &
     &2.9619e-03_rb,3.3315e-03_rb,4.0621e-03_rb,4.9694e-03_rb,6.1117e-03_rb/)
      kao(:,10, 4) = (/ &
     &2.2204e-03_rb,2.5123e-03_rb,3.0176e-03_rb,3.7321e-03_rb,4.5545e-03_rb/)
      kao(:,11, 4) = (/ &
     &1.8288e-03_rb,2.0602e-03_rb,2.4838e-03_rb,3.0685e-03_rb,3.7508e-03_rb/)
      kao(:,12, 4) = (/ &
     &1.5051e-03_rb,1.6916e-03_rb,2.0466e-03_rb,2.5239e-03_rb,3.0905e-03_rb/)
      kao(:,13, 4) = (/ &
     &1.2373e-03_rb,1.3907e-03_rb,1.6848e-03_rb,2.0764e-03_rb,2.5455e-03_rb/)
      kao(:, 1, 5) = (/ &
     &7.4128e-02_rb,9.0328e-02_rb,1.0845e-01_rb,1.2842e-01_rb,1.4993e-01_rb/)
      kao(:, 2, 5) = (/ &
     &5.6421e-02_rb,6.8932e-02_rb,8.3294e-02_rb,9.9111e-02_rb,1.1617e-01_rb/)
      kao(:, 3, 5) = (/ &
     &4.0916e-02_rb,5.0385e-02_rb,6.1257e-02_rb,7.3474e-02_rb,8.6732e-02_rb/)
      kao(:, 4, 5) = (/ &
     &2.9430e-02_rb,3.6556e-02_rb,4.4742e-02_rb,5.4199e-02_rb,6.4580e-02_rb/)
      kao(:, 5, 5) = (/ &
     &2.1212e-02_rb,2.6602e-02_rb,3.2780e-02_rb,4.0084e-02_rb,4.8236e-02_rb/)
      kao(:, 6, 5) = (/ &
     &1.5151e-02_rb,1.9129e-02_rb,2.3897e-02_rb,2.9436e-02_rb,3.5798e-02_rb/)
      kao(:, 7, 5) = (/ &
     &1.0718e-02_rb,1.3802e-02_rb,1.7389e-02_rb,2.1505e-02_rb,2.6440e-02_rb/)
      kao(:, 8, 5) = (/ &
     &7.4917e-03_rb,9.9125e-03_rb,1.2578e-02_rb,1.5770e-02_rb,1.9483e-02_rb/)
      kao(:, 9, 5) = (/ &
     &5.1168e-03_rb,7.0368e-03_rb,9.1089e-03_rb,1.1527e-02_rb,1.4293e-02_rb/)
      kao(:,10, 5) = (/ &
     &3.6983e-03_rb,5.0627e-03_rb,6.6991e-03_rb,8.5275e-03_rb,1.0696e-02_rb/)
      kao(:,11, 5) = (/ &
     &3.0484e-03_rb,4.1916e-03_rb,5.5346e-03_rb,7.0513e-03_rb,8.8498e-03_rb/)
      kao(:,12, 5) = (/ &
     &2.5174e-03_rb,3.4714e-03_rb,4.5718e-03_rb,5.8341e-03_rb,7.3195e-03_rb/)
      kao(:,13, 5) = (/ &
     &2.0770e-03_rb,2.8684e-03_rb,3.7718e-03_rb,4.8192e-03_rb,6.0412e-03_rb/)
      kao(:, 1, 6) = (/ &
     &2.0613e-01_rb,2.5157e-01_rb,3.0178e-01_rb,3.5683e-01_rb,4.1542e-01_rb/)
      kao(:, 2, 6) = (/ &
     &1.5819e-01_rb,1.9432e-01_rb,2.3452e-01_rb,2.7858e-01_rb,3.2587e-01_rb/)
      kao(:, 3, 6) = (/ &
     &1.1531e-01_rb,1.4306e-01_rb,1.7435e-01_rb,2.0876e-01_rb,2.4632e-01_rb/)
      kao(:, 4, 6) = (/ &
     &8.3284e-02_rb,1.0444e-01_rb,1.2864e-01_rb,1.5560e-01_rb,1.8535e-01_rb/)
      kao(:, 5, 6) = (/ &
     &6.0166e-02_rb,7.6315e-02_rb,9.4937e-02_rb,1.1604e-01_rb,1.3958e-01_rb/)
      kao(:, 6, 6) = (/ &
     &4.3003e-02_rb,5.5230e-02_rb,6.9447e-02_rb,8.5781e-02_rb,1.0424e-01_rb/)
      kao(:, 7, 6) = (/ &
     &3.0490e-02_rb,3.9769e-02_rb,5.0618e-02_rb,6.3200e-02_rb,7.7597e-02_rb/)
      kao(:, 8, 6) = (/ &
     &2.1570e-02_rb,2.8573e-02_rb,3.6803e-02_rb,4.6437e-02_rb,5.7626e-02_rb/)
      kao(:, 9, 6) = (/ &
     &1.5155e-02_rb,2.0450e-02_rb,2.6673e-02_rb,3.4004e-02_rb,4.2620e-02_rb/)
      kao(:,10, 6) = (/ &
     &1.0852e-02_rb,1.4893e-02_rb,1.9668e-02_rb,2.5324e-02_rb,3.1992e-02_rb/)
      kao(:,11, 6) = (/ &
     &9.0511e-03_rb,1.2422e-02_rb,1.6374e-02_rb,2.1084e-02_rb,2.6643e-02_rb/)
      kao(:,12, 6) = (/ &
     &7.5332e-03_rb,1.0338e-02_rb,1.3623e-02_rb,1.7531e-02_rb,2.2174e-02_rb/)
      kao(:,13, 6) = (/ &
     &6.2568e-03_rb,8.5754e-03_rb,1.1313e-02_rb,1.4562e-02_rb,1.8439e-02_rb/)
      kao(:, 1, 7) = (/ &
     &6.4924e-01_rb,7.7307e-01_rb,9.0386e-01_rb,1.0388e+00_rb,1.1773e+00_rb/)
      kao(:, 2, 7) = (/ &
     &5.0577e-01_rb,6.0786e-01_rb,7.1639e-01_rb,8.2855e-01_rb,9.4493e-01_rb/)
      kao(:, 3, 7) = (/ &
     &3.7384e-01_rb,4.5638e-01_rb,5.4361e-01_rb,6.3563e-01_rb,7.3108e-01_rb/)
      kao(:, 4, 7) = (/ &
     &2.7157e-01_rb,3.3769e-01_rb,4.0798e-01_rb,4.8298e-01_rb,5.6158e-01_rb/)
      kao(:, 5, 7) = (/ &
     &1.9648e-01_rb,2.4838e-01_rb,3.0521e-01_rb,3.6590e-01_rb,4.3005e-01_rb/)
      kao(:, 6, 7) = (/ &
     &1.3966e-01_rb,1.7987e-01_rb,2.2524e-01_rb,2.7405e-01_rb,3.2593e-01_rb/)
      kao(:, 7, 7) = (/ &
     &9.8353e-02_rb,1.2917e-01_rb,1.6445e-01_rb,2.0365e-01_rb,2.4531e-01_rb/)
      kao(:, 8, 7) = (/ &
     &6.8795e-02_rb,9.2244e-02_rb,1.1951e-01_rb,1.5053e-01_rb,1.8392e-01_rb/)
      kao(:, 9, 7) = (/ &
     &4.7720e-02_rb,6.5374e-02_rb,8.6294e-02_rb,1.1044e-01_rb,1.3721e-01_rb/)
      kao(:,10, 7) = (/ &
     &3.3881e-02_rb,4.7211e-02_rb,6.3377e-02_rb,8.2294e-02_rb,1.0374e-01_rb/)
      kao(:,11, 7) = (/ &
     &2.8304e-02_rb,3.9386e-02_rb,5.2911e-02_rb,6.8671e-02_rb,8.6477e-02_rb/)
      kao(:,12, 7) = (/ &
     &2.3643e-02_rb,3.2849e-02_rb,4.4121e-02_rb,5.7246e-02_rb,7.2052e-02_rb/)
      kao(:,13, 7) = (/ &
     &1.9678e-02_rb,2.7339e-02_rb,3.6693e-02_rb,4.7583e-02_rb,5.9838e-02_rb/)
      kao(:, 1, 8) = (/ &
     &1.5852e+00_rb,1.8937e+00_rb,2.2233e+00_rb,2.5759e+00_rb,2.9447e+00_rb/)
      kao(:, 2, 8) = (/ &
     &1.3020e+00_rb,1.5608e+00_rb,1.8408e+00_rb,2.1415e+00_rb,2.4614e+00_rb/)
      kao(:, 3, 8) = (/ &
     &1.0126e+00_rb,1.2239e+00_rb,1.4553e+00_rb,1.7040e+00_rb,1.9702e+00_rb/)
      kao(:, 4, 8) = (/ &
     &7.6856e-01_rb,9.4025e-01_rb,1.1316e+00_rb,1.3386e+00_rb,1.5605e+00_rb/)
      kao(:, 5, 8) = (/ &
     &5.7616e-01_rb,7.1359e-01_rb,8.6832e-01_rb,1.0388e+00_rb,1.2228e+00_rb/)
      kao(:, 6, 8) = (/ &
     &4.2350e-01_rb,5.3268e-01_rb,6.5605e-01_rb,7.9334e-01_rb,9.4223e-01_rb/)
      kao(:, 7, 8) = (/ &
     &3.0686e-01_rb,3.9225e-01_rb,4.9010e-01_rb,5.9944e-01_rb,7.1967e-01_rb/)
      kao(:, 8, 8) = (/ &
     &2.2070e-01_rb,2.8688e-01_rb,3.6348e-01_rb,4.5007e-01_rb,5.4597e-01_rb/)
      kao(:, 9, 8) = (/ &
     &1.5750e-01_rb,2.0832e-01_rb,2.6774e-01_rb,3.3584e-01_rb,4.1169e-01_rb/)
      kao(:,10, 8) = (/ &
     &1.1458e-01_rb,1.5398e-01_rb,2.0047e-01_rb,2.5420e-01_rb,3.1452e-01_rb/)
      kao(:,11, 8) = (/ &
     &9.6352e-02_rb,1.2945e-01_rb,1.6850e-01_rb,2.1343e-01_rb,2.6418e-01_rb/)
      kao(:,12, 8) = (/ &
     &8.0859e-02_rb,1.0869e-01_rb,1.4142e-01_rb,1.7905e-01_rb,2.2159e-01_rb/)
      kao(:,13, 8) = (/ &
     &6.7685e-02_rb,9.1028e-02_rb,1.1838e-01_rb,1.4990e-01_rb,1.8559e-01_rb/)
      kao(:, 1, 9) = (/ &
     &5.2550e+00_rb,6.3269e+00_rb,7.5188e+00_rb,8.8046e+00_rb,1.0194e+01_rb/)
      kao(:, 2, 9) = (/ &
     &4.6603e+00_rb,5.6500e+00_rb,6.7441e+00_rb,7.9369e+00_rb,9.2112e+00_rb/)
      kao(:, 3, 9) = (/ &
     &3.9124e+00_rb,4.7902e+00_rb,5.7591e+00_rb,6.8243e+00_rb,7.9677e+00_rb/)
      kao(:, 4, 9) = (/ &
     &3.2096e+00_rb,3.9597e+00_rb,4.8029e+00_rb,5.7377e+00_rb,6.7626e+00_rb/)
      kao(:, 5, 9) = (/ &
     &2.5970e+00_rb,3.2435e+00_rb,3.9726e+00_rb,4.7826e+00_rb,5.6814e+00_rb/)
      kao(:, 6, 9) = (/ &
     &2.0409e+00_rb,2.5854e+00_rb,3.2065e+00_rb,3.9078e+00_rb,4.6887e+00_rb/)
      kao(:, 7, 9) = (/ &
     &1.5708e+00_rb,2.0187e+00_rb,2.5381e+00_rb,3.1327e+00_rb,3.8016e+00_rb/)
      kao(:, 8, 9) = (/ &
     &1.1892e+00_rb,1.5546e+00_rb,1.9836e+00_rb,2.4782e+00_rb,3.0361e+00_rb/)
      kao(:, 9, 9) = (/ &
     &8.8412e-01_rb,1.1765e+00_rb,1.5263e+00_rb,1.9325e+00_rb,2.3921e+00_rb/)
      kao(:,10, 9) = (/ &
     &6.6160e-01_rb,8.9598e-01_rb,1.1789e+00_rb,1.5110e+00_rb,1.8897e+00_rb/)
      kao(:,11, 9) = (/ &
     &5.6952e-01_rb,7.7339e-01_rb,1.0185e+00_rb,1.3041e+00_rb,1.6292e+00_rb/)
      kao(:,12, 9) = (/ &
     &4.8820e-01_rb,6.6319e-01_rb,8.7290e-01_rb,1.1167e+00_rb,1.3956e+00_rb/)
      kao(:,13, 9) = (/ &
     &4.1555e-01_rb,5.6459e-01_rb,7.4208e-01_rb,9.4959e-01_rb,1.1894e+00_rb/)
      kao(:, 1,10) = (/ &
     &1.2977e+01_rb,1.5399e+01_rb,1.8084e+01_rb,2.1107e+01_rb,2.4279e+01_rb/)
      kao(:, 2,10) = (/ &
     &1.2242e+01_rb,1.4713e+01_rb,1.7530e+01_rb,2.0660e+01_rb,2.4161e+01_rb/)
      kao(:, 3,10) = (/ &
     &1.0793e+01_rb,1.3120e+01_rb,1.5954e+01_rb,1.9118e+01_rb,2.2561e+01_rb/)
      kao(:, 4,10) = (/ &
     &9.2115e+00_rb,1.1467e+01_rb,1.4090e+01_rb,1.6870e+01_rb,1.9865e+01_rb/)
      kao(:, 5,10) = (/ &
     &7.7076e+00_rb,9.6964e+00_rb,1.2018e+01_rb,1.4581e+01_rb,1.7283e+01_rb/)
      kao(:, 6,10) = (/ &
     &6.3938e+00_rb,8.1632e+00_rb,1.0170e+01_rb,1.2328e+01_rb,1.4702e+01_rb/)
      kao(:, 7,10) = (/ &
     &5.2643e+00_rb,6.8431e+00_rb,8.5879e+00_rb,1.0497e+01_rb,1.2599e+01_rb/)
      kao(:, 8,10) = (/ &
     &4.2306e+00_rb,5.5592e+00_rb,7.0542e+00_rb,8.7515e+00_rb,1.0687e+01_rb/)
      kao(:, 9,10) = (/ &
     &3.3029e+00_rb,4.4196e+00_rb,5.6740e+00_rb,7.1182e+00_rb,8.8343e+00_rb/)
      kao(:,10,10) = (/ &
     &2.6002e+00_rb,3.5171e+00_rb,4.6049e+00_rb,5.8654e+00_rb,7.3550e+00_rb/)
      kao(:,11,10) = (/ &
     &2.3411e+00_rb,3.1582e+00_rb,4.1576e+00_rb,5.3440e+00_rb,6.7154e+00_rb/)
      kao(:,12,10) = (/ &
     &2.0671e+00_rb,2.8097e+00_rb,3.7158e+00_rb,4.7938e+00_rb,5.9967e+00_rb/)
      kao(:,13,10) = (/ &
     &1.8130e+00_rb,2.4724e+00_rb,3.2908e+00_rb,4.2191e+00_rb,5.2351e+00_rb/)
      kao(:, 1,11) = (/ &
     &1.7369e+01_rb,2.0982e+01_rb,2.4652e+01_rb,2.8692e+01_rb,3.3179e+01_rb/)
      kao(:, 2,11) = (/ &
     &1.6823e+01_rb,2.0457e+01_rb,2.4236e+01_rb,2.8171e+01_rb,3.2333e+01_rb/)
      kao(:, 3,11) = (/ &
     &1.5375e+01_rb,1.8905e+01_rb,2.2457e+01_rb,2.6420e+01_rb,3.0766e+01_rb/)
      kao(:, 4,11) = (/ &
     &1.3375e+01_rb,1.6618e+01_rb,2.0128e+01_rb,2.4207e+01_rb,2.8664e+01_rb/)
      kao(:, 5,11) = (/ &
     &1.1552e+01_rb,1.4477e+01_rb,1.7747e+01_rb,2.1616e+01_rb,2.5966e+01_rb/)
      kao(:, 6,11) = (/ &
     &9.7662e+00_rb,1.2339e+01_rb,1.5425e+01_rb,1.9082e+01_rb,2.3024e+01_rb/)
      kao(:, 7,11) = (/ &
     &8.0435e+00_rb,1.0356e+01_rb,1.3162e+01_rb,1.6422e+01_rb,1.9976e+01_rb/)
      kao(:, 8,11) = (/ &
     &6.6475e+00_rb,8.6800e+00_rb,1.1208e+01_rb,1.4127e+01_rb,1.7194e+01_rb/)
      kao(:, 9,11) = (/ &
     &5.4286e+00_rb,7.1938e+00_rb,9.4274e+00_rb,1.1995e+01_rb,1.4788e+01_rb/)
      kao(:,10,11) = (/ &
     &4.4519e+00_rb,5.9777e+00_rb,7.8591e+00_rb,1.0067e+01_rb,1.2572e+01_rb/)
      kao(:,11,11) = (/ &
     &4.1097e+00_rb,5.5519e+00_rb,7.2615e+00_rb,9.2610e+00_rb,1.1615e+01_rb/)
      kao(:,12,11) = (/ &
     &3.7565e+00_rb,5.0648e+00_rb,6.6341e+00_rb,8.4916e+00_rb,1.0710e+01_rb/)
      kao(:,13,11) = (/ &
     &3.3842e+00_rb,4.5598e+00_rb,5.9909e+00_rb,7.7387e+00_rb,9.7720e+00_rb/)
      kao(:, 1,12) = (/ &
     &2.2275e+01_rb,2.7471e+01_rb,3.3492e+01_rb,4.0225e+01_rb,4.7465e+01_rb/)
      kao(:, 2,12) = (/ &
     &2.2476e+01_rb,2.7446e+01_rb,3.3072e+01_rb,3.9336e+01_rb,4.6205e+01_rb/)
      kao(:, 3,12) = (/ &
     &2.1150e+01_rb,2.6276e+01_rb,3.1885e+01_rb,3.7595e+01_rb,4.3863e+01_rb/)
      kao(:, 4,12) = (/ &
     &1.9455e+01_rb,2.4367e+01_rb,2.9875e+01_rb,3.5713e+01_rb,4.1802e+01_rb/)
      kao(:, 5,12) = (/ &
     &1.7406e+01_rb,2.2244e+01_rb,2.7648e+01_rb,3.3299e+01_rb,3.9489e+01_rb/)
      kao(:, 6,12) = (/ &
     &1.5271e+01_rb,1.9641e+01_rb,2.4614e+01_rb,3.0133e+01_rb,3.6348e+01_rb/)
      kao(:, 7,12) = (/ &
     &1.3343e+01_rb,1.7102e+01_rb,2.1495e+01_rb,2.6582e+01_rb,3.2562e+01_rb/)
      kao(:, 8,12) = (/ &
     &1.1399e+01_rb,1.4767e+01_rb,1.8657e+01_rb,2.3239e+01_rb,2.8870e+01_rb/)
      kao(:, 9,12) = (/ &
     &9.4462e+00_rb,1.2437e+01_rb,1.5942e+01_rb,2.0083e+01_rb,2.5175e+01_rb/)
      kao(:,10,12) = (/ &
     &7.9561e+00_rb,1.0576e+01_rb,1.3751e+01_rb,1.7625e+01_rb,2.2224e+01_rb/)
      kao(:,11,12) = (/ &
     &7.6703e+00_rb,1.0214e+01_rb,1.3362e+01_rb,1.7172e+01_rb,2.1565e+01_rb/)
      kao(:,12,12) = (/ &
     &7.3002e+00_rb,9.7613e+00_rb,1.2808e+01_rb,1.6448e+01_rb,2.0508e+01_rb/)
      kao(:,13,12) = (/ &
     &6.8227e+00_rb,9.1662e+00_rb,1.2005e+01_rb,1.5291e+01_rb,1.9104e+01_rb/)
      kao(:, 1,13) = (/ &
     &2.8154e+01_rb,3.5750e+01_rb,4.4791e+01_rb,5.4873e+01_rb,6.6140e+01_rb/)
      kao(:, 2,13) = (/ &
     &2.8745e+01_rb,3.6229e+01_rb,4.5340e+01_rb,5.5950e+01_rb,6.7860e+01_rb/)
      kao(:, 3,13) = (/ &
     &2.8731e+01_rb,3.5740e+01_rb,4.4190e+01_rb,5.4581e+01_rb,6.6390e+01_rb/)
      kao(:, 4,13) = (/ &
     &2.7586e+01_rb,3.5028e+01_rb,4.3053e+01_rb,5.2513e+01_rb,6.3590e+01_rb/)
      kao(:, 5,13) = (/ &
     &2.6083e+01_rb,3.3437e+01_rb,4.1632e+01_rb,5.0781e+01_rb,6.1113e+01_rb/)
      kao(:, 6,13) = (/ &
     &2.3829e+01_rb,3.1109e+01_rb,3.9228e+01_rb,4.8371e+01_rb,5.8278e+01_rb/)
      kao(:, 7,13) = (/ &
     &2.1209e+01_rb,2.8372e+01_rb,3.6376e+01_rb,4.5363e+01_rb,5.5372e+01_rb/)
      kao(:, 8,13) = (/ &
     &1.8790e+01_rb,2.5354e+01_rb,3.3043e+01_rb,4.1793e+01_rb,5.1443e+01_rb/)
      kao(:, 9,13) = (/ &
     &1.6798e+01_rb,2.2642e+01_rb,2.9578e+01_rb,3.7822e+01_rb,4.7058e+01_rb/)
      kao(:,10,13) = (/ &
     &1.4972e+01_rb,2.0558e+01_rb,2.6854e+01_rb,3.4253e+01_rb,4.2864e+01_rb/)
      kao(:,11,13) = (/ &
     &1.5310e+01_rb,2.0781e+01_rb,2.6920e+01_rb,3.4183e+01_rb,4.2625e+01_rb/)
      kao(:,12,13) = (/ &
     &1.5266e+01_rb,2.0520e+01_rb,2.6628e+01_rb,3.3712e+01_rb,4.2182e+01_rb/)
      kao(:,13,13) = (/ &
     &1.4957e+01_rb,1.9939e+01_rb,2.5957e+01_rb,3.3111e+01_rb,4.1486e+01_rb/)
      kao(:, 1,14) = (/ &
     &4.6330e+01_rb,6.0030e+01_rb,7.5480e+01_rb,9.2536e+01_rb,1.1059e+02_rb/)
      kao(:, 2,14) = (/ &
     &4.4855e+01_rb,5.8960e+01_rb,7.5059e+01_rb,9.2958e+01_rb,1.1209e+02_rb/)
      kao(:, 3,14) = (/ &
     &4.1447e+01_rb,5.4787e+01_rb,7.1113e+01_rb,8.9291e+01_rb,1.0911e+02_rb/)
      kao(:, 4,14) = (/ &
     &3.9104e+01_rb,5.0668e+01_rb,6.5934e+01_rb,8.3732e+01_rb,1.0396e+02_rb/)
      kao(:, 5,14) = (/ &
     &3.8023e+01_rb,4.8275e+01_rb,6.1835e+01_rb,7.8920e+01_rb,9.8684e+01_rb/)
      kao(:, 6,14) = (/ &
     &3.6671e+01_rb,4.6928e+01_rb,5.9373e+01_rb,7.4910e+01_rb,9.4120e+01_rb/)
      kao(:, 7,14) = (/ &
     &3.4713e+01_rb,4.5254e+01_rb,5.8164e+01_rb,7.3270e+01_rb,9.1067e+01_rb/)
      kao(:, 8,14) = (/ &
     &3.2317e+01_rb,4.2931e+01_rb,5.6350e+01_rb,7.1845e+01_rb,8.9533e+01_rb/)
      kao(:, 9,14) = (/ &
     &2.9466e+01_rb,3.9947e+01_rb,5.3450e+01_rb,6.9454e+01_rb,8.7436e+01_rb/)
      kao(:,10,14) = (/ &
     &2.7304e+01_rb,3.7491e+01_rb,5.1151e+01_rb,6.7458e+01_rb,8.6217e+01_rb/)
      kao(:,11,14) = (/ &
     &2.9371e+01_rb,4.0579e+01_rb,5.5614e+01_rb,7.3135e+01_rb,9.3406e+01_rb/)
      kao(:,12,14) = (/ &
     &3.1557e+01_rb,4.3832e+01_rb,5.9668e+01_rb,7.8288e+01_rb,9.9282e+01_rb/)
      kao(:,13,14) = (/ &
     &3.3702e+01_rb,4.7002e+01_rb,6.3302e+01_rb,8.2327e+01_rb,1.0363e+02_rb/)
      kao(:, 1,15) = (/ &
     &7.1097e+01_rb,9.1809e+01_rb,1.1513e+02_rb,1.4077e+02_rb,1.6931e+02_rb/)
      kao(:, 2,15) = (/ &
     &7.6481e+01_rb,9.9988e+01_rb,1.2655e+02_rb,1.5641e+02_rb,1.9001e+02_rb/)
      kao(:, 3,15) = (/ &
     &7.6089e+01_rb,1.0186e+02_rb,1.3121e+02_rb,1.6509e+02_rb,2.0316e+02_rb/)
      kao(:, 4,15) = (/ &
     &7.1689e+01_rb,9.8693e+01_rb,1.3061e+02_rb,1.6745e+02_rb,2.0924e+02_rb/)
      kao(:, 5,15) = (/ &
     &6.5396e+01_rb,9.2850e+01_rb,1.2644e+02_rb,1.6569e+02_rb,2.1030e+02_rb/)
      kao(:, 6,15) = (/ &
     &5.9839e+01_rb,8.4120e+01_rb,1.1791e+02_rb,1.5835e+02_rb,2.0497e+02_rb/)
      kao(:, 7,15) = (/ &
     &5.7504e+01_rb,7.7966e+01_rb,1.0776e+02_rb,1.4729e+02_rb,1.9476e+02_rb/)
      kao(:, 8,15) = (/ &
     &5.7004e+01_rb,7.5268e+01_rb,1.0086e+02_rb,1.3678e+02_rb,1.8339e+02_rb/)
      kao(:, 9,15) = (/ &
     &5.5577e+01_rb,7.3906e+01_rb,9.7044e+01_rb,1.2904e+02_rb,1.7226e+02_rb/)
      kao(:,10,15) = (/ &
     &5.5416e+01_rb,7.4211e+01_rb,9.6440e+01_rb,1.2687e+02_rb,1.6750e+02_rb/)
      kao(:,11,15) = (/ &
     &6.2860e+01_rb,8.4332e+01_rb,1.0913e+02_rb,1.4265e+02_rb,1.8694e+02_rb/)
      kao(:,12,15) = (/ &
     &7.1048e+01_rb,9.5286e+01_rb,1.2362e+02_rb,1.6046e+02_rb,2.0975e+02_rb/)
      kao(:,13,15) = (/ &
     &7.9451e+01_rb,1.0631e+02_rb,1.3951e+02_rb,1.8076e+02_rb,2.3534e+02_rb/)
      kao(:, 1,16) = (/ &
     &7.6064e+01_rb,1.0164e+02_rb,1.3126e+02_rb,1.6469e+02_rb,2.0152e+02_rb/)
      kao(:, 2,16) = (/ &
     &8.3803e+01_rb,1.1323e+02_rb,1.4816e+02_rb,1.8796e+02_rb,2.3219e+02_rb/)
      kao(:, 3,16) = (/ &
     &8.7402e+01_rb,1.1813e+02_rb,1.5778e+02_rb,2.0377e+02_rb,2.5553e+02_rb/)
      kao(:, 4,16) = (/ &
     &8.7865e+01_rb,1.1968e+02_rb,1.6142e+02_rb,2.1263e+02_rb,2.7144e+02_rb/)
      kao(:, 5,16) = (/ &
     &8.6499e+01_rb,1.2062e+02_rb,1.6367e+02_rb,2.1790e+02_rb,2.8301e+02_rb/)
      kao(:, 6,16) = (/ &
     &8.2494e+01_rb,1.1893e+02_rb,1.6400e+02_rb,2.2086e+02_rb,2.8964e+02_rb/)
      kao(:, 7,16) = (/ &
     &7.7078e+01_rb,1.1526e+02_rb,1.6302e+02_rb,2.2306e+02_rb,2.9645e+02_rb/)
      kao(:, 8,16) = (/ &
     &7.0841e+01_rb,1.1011e+02_rb,1.6059e+02_rb,2.2458e+02_rb,3.0342e+02_rb/)
      kao(:, 9,16) = (/ &
     &7.4013e+01_rb,1.0313e+02_rb,1.5566e+02_rb,2.2324e+02_rb,3.0785e+02_rb/)
      kao(:,10,16) = (/ &
     &8.3043e+01_rb,1.0119e+02_rb,1.5304e+02_rb,2.2535e+02_rb,3.1705e+02_rb/)
      kao(:,11,16) = (/ &
     &1.0007e+02_rb,1.2237e+02_rb,1.8107e+02_rb,2.6762e+02_rb,3.7672e+02_rb/)
      kao(:,12,16) = (/ &
     &1.1998e+02_rb,1.4801e+02_rb,2.1370e+02_rb,3.1623e+02_rb,4.4496e+02_rb/)
      kao(:,13,16) = (/ &
     &1.4282e+02_rb,1.7830e+02_rb,2.5018e+02_rb,3.7037e+02_rb,5.2070e+02_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &1.3000e-04_rb,1.6216e-04_rb,2.0768e-04_rb,2.6864e-04_rb,3.3497e-04_rb/)
      kbo(:,14, 1) = (/ &
     &1.0850e-04_rb,1.3602e-04_rb,1.7553e-04_rb,2.2686e-04_rb,2.8162e-04_rb/)
      kbo(:,15, 1) = (/ &
     &9.0864e-05_rb,1.1476e-04_rb,1.4832e-04_rb,1.9081e-04_rb,2.3698e-04_rb/)
      kbo(:,16, 1) = (/ &
     &7.6202e-05_rb,9.6897e-05_rb,1.2521e-04_rb,1.6032e-04_rb,1.9925e-04_rb/)
      kbo(:,17, 1) = (/ &
     &6.3984e-05_rb,8.1671e-05_rb,1.0565e-04_rb,1.3448e-04_rb,1.6696e-04_rb/)
      kbo(:,18, 1) = (/ &
     &5.3749e-05_rb,6.8759e-05_rb,8.9036e-05_rb,1.1274e-04_rb,1.3824e-04_rb/)
      kbo(:,19, 1) = (/ &
     &4.5160e-05_rb,5.7874e-05_rb,7.4954e-05_rb,9.4487e-05_rb,1.1485e-04_rb/)
      kbo(:,20, 1) = (/ &
     &3.8050e-05_rb,4.8934e-05_rb,6.3376e-05_rb,7.9393e-05_rb,9.5565e-05_rb/)
      kbo(:,21, 1) = (/ &
     &3.2121e-05_rb,4.1368e-05_rb,5.3585e-05_rb,6.6749e-05_rb,7.9704e-05_rb/)
      kbo(:,22, 1) = (/ &
     &2.7597e-05_rb,3.5606e-05_rb,4.6066e-05_rb,5.6919e-05_rb,6.7252e-05_rb/)
      kbo(:,23, 1) = (/ &
     &2.3703e-05_rb,3.0672e-05_rb,3.9251e-05_rb,4.8622e-05_rb,5.6783e-05_rb/)
      kbo(:,24, 1) = (/ &
     &2.0444e-05_rb,2.6471e-05_rb,3.3535e-05_rb,4.1441e-05_rb,4.8070e-05_rb/)
      kbo(:,25, 1) = (/ &
     &1.7691e-05_rb,2.2899e-05_rb,2.8737e-05_rb,3.4776e-05_rb,4.0820e-05_rb/)
      kbo(:,26, 1) = (/ &
     &1.5391e-05_rb,1.9900e-05_rb,2.4708e-05_rb,2.9438e-05_rb,3.4884e-05_rb/)
      kbo(:,27, 1) = (/ &
     &1.3401e-05_rb,1.7231e-05_rb,2.1300e-05_rb,2.5041e-05_rb,2.9843e-05_rb/)
      kbo(:,28, 1) = (/ &
     &1.1671e-05_rb,1.4827e-05_rb,1.8383e-05_rb,2.1307e-05_rb,2.5549e-05_rb/)
      kbo(:,29, 1) = (/ &
     &1.0183e-05_rb,1.2798e-05_rb,1.5535e-05_rb,1.8197e-05_rb,2.1838e-05_rb/)
      kbo(:,30, 1) = (/ &
     &8.8844e-06_rb,1.1043e-05_rb,1.3172e-05_rb,1.5593e-05_rb,1.8678e-05_rb/)
      kbo(:,31, 1) = (/ &
     &7.7313e-06_rb,9.5518e-06_rb,1.1237e-05_rb,1.3382e-05_rb,1.5999e-05_rb/)
      kbo(:,32, 1) = (/ &
     &6.6794e-06_rb,8.2781e-06_rb,9.5924e-06_rb,1.1496e-05_rb,1.3717e-05_rb/)
      kbo(:,33, 1) = (/ &
     &5.7815e-06_rb,7.0042e-06_rb,8.2098e-06_rb,9.8489e-06_rb,1.1766e-05_rb/)
      kbo(:,34, 1) = (/ &
     &4.9641e-06_rb,5.9235e-06_rb,7.0090e-06_rb,8.3895e-06_rb,1.0037e-05_rb/)
      kbo(:,35, 1) = (/ &
     &4.1780e-06_rb,4.9520e-06_rb,5.8797e-06_rb,7.0288e-06_rb,8.4123e-06_rb/)
      kbo(:,36, 1) = (/ &
     &3.4370e-06_rb,4.0690e-06_rb,4.8335e-06_rb,5.7766e-06_rb,6.9140e-06_rb/)
      kbo(:,37, 1) = (/ &
     &2.7366e-06_rb,3.2617e-06_rb,3.8617e-06_rb,4.6224e-06_rb,5.5308e-06_rb/)
      kbo(:,38, 1) = (/ &
     &2.1763e-06_rb,2.6155e-06_rb,3.0811e-06_rb,3.6939e-06_rb,4.4152e-06_rb/)
      kbo(:,39, 1) = (/ &
     &1.7313e-06_rb,2.1017e-06_rb,2.4606e-06_rb,2.9519e-06_rb,3.5266e-06_rb/)
      kbo(:,40, 1) = (/ &
     &1.3469e-06_rb,1.6693e-06_rb,1.9344e-06_rb,2.3182e-06_rb,2.7667e-06_rb/)
      kbo(:,41, 1) = (/ &
     &1.0461e-06_rb,1.2927e-06_rb,1.5201e-06_rb,1.8107e-06_rb,2.1653e-06_rb/)
      kbo(:,42, 1) = (/ &
     &8.0658e-07_rb,1.0022e-06_rb,1.1943e-06_rb,1.4143e-06_rb,1.6945e-06_rb/)
      kbo(:,43, 1) = (/ &
     &6.1022e-07_rb,7.6847e-07_rb,9.3637e-07_rb,1.0945e-06_rb,1.3137e-06_rb/)
      kbo(:,44, 1) = (/ &
     &4.5840e-07_rb,5.8613e-07_rb,7.2558e-07_rb,8.4731e-07_rb,1.0130e-06_rb/)
      kbo(:,45, 1) = (/ &
     &3.4424e-07_rb,4.4504e-07_rb,5.5242e-07_rb,6.5691e-07_rb,7.7925e-07_rb/)
      kbo(:,46, 1) = (/ &
     &2.5697e-07_rb,3.3261e-07_rb,4.1921e-07_rb,5.1218e-07_rb,5.9773e-07_rb/)
      kbo(:,47, 1) = (/ &
     &1.8987e-07_rb,2.4567e-07_rb,3.1544e-07_rb,3.9053e-07_rb,4.5787e-07_rb/)
      kbo(:,48, 1) = (/ &
     &1.4058e-07_rb,1.8138e-07_rb,2.3483e-07_rb,2.9330e-07_rb,3.5197e-07_rb/)
      kbo(:,49, 1) = (/ &
     &1.0462e-07_rb,1.3385e-07_rb,1.7347e-07_rb,2.2053e-07_rb,2.7396e-07_rb/)
      kbo(:,50, 1) = (/ &
     &7.9205e-08_rb,9.9611e-08_rb,1.2890e-07_rb,1.6695e-07_rb,2.0670e-07_rb/)
      kbo(:,51, 1) = (/ &
     &6.0628e-08_rb,7.4396e-08_rb,9.6049e-08_rb,1.2454e-07_rb,1.5638e-07_rb/)
      kbo(:,52, 1) = (/ &
     &4.5002e-08_rb,5.5943e-08_rb,7.1532e-08_rb,9.2844e-08_rb,1.1852e-07_rb/)
      kbo(:,53, 1) = (/ &
     &3.2555e-08_rb,4.2490e-08_rb,5.3383e-08_rb,6.9149e-08_rb,8.9727e-08_rb/)
      kbo(:,54, 1) = (/ &
     &2.3696e-08_rb,3.2653e-08_rb,4.0140e-08_rb,5.1914e-08_rb,6.7441e-08_rb/)
      kbo(:,55, 1) = (/ &
     &1.7247e-08_rb,2.4546e-08_rb,3.0469e-08_rb,3.9087e-08_rb,5.0840e-08_rb/)
      kbo(:,56, 1) = (/ &
     &1.2515e-08_rb,1.8025e-08_rb,2.3296e-08_rb,2.9456e-08_rb,3.8300e-08_rb/)
      kbo(:,57, 1) = (/ &
     &9.0211e-09_rb,1.3183e-08_rb,1.7891e-08_rb,2.2220e-08_rb,2.8857e-08_rb/)
      kbo(:,58, 1) = (/ &
     &6.4935e-09_rb,9.6472e-09_rb,1.3706e-08_rb,1.6858e-08_rb,2.1803e-08_rb/)
      kbo(:,59, 1) = (/ &
     &5.0413e-09_rb,7.5477e-09_rb,1.0800e-08_rb,1.3392e-08_rb,1.7290e-08_rb/)
      kbo(:,13, 2) = (/ &
     &3.6782e-04_rb,4.0460e-04_rb,4.6828e-04_rb,5.6427e-04_rb,6.8387e-04_rb/)
      kbo(:,14, 2) = (/ &
     &3.0206e-04_rb,3.3534e-04_rb,3.8904e-04_rb,4.6908e-04_rb,5.7354e-04_rb/)
      kbo(:,15, 2) = (/ &
     &2.4903e-04_rb,2.7718e-04_rb,3.2574e-04_rb,3.9271e-04_rb,4.8202e-04_rb/)
      kbo(:,16, 2) = (/ &
     &2.0570e-04_rb,2.3043e-04_rb,2.7323e-04_rb,3.2940e-04_rb,4.0539e-04_rb/)
      kbo(:,17, 2) = (/ &
     &1.6973e-04_rb,1.9226e-04_rb,2.2908e-04_rb,2.7627e-04_rb,3.4057e-04_rb/)
      kbo(:,18, 2) = (/ &
     &1.4059e-04_rb,1.6018e-04_rb,1.9182e-04_rb,2.3171e-04_rb,2.8730e-04_rb/)
      kbo(:,19, 2) = (/ &
     &1.1617e-04_rb,1.3358e-04_rb,1.6042e-04_rb,1.9424e-04_rb,2.4177e-04_rb/)
      kbo(:,20, 2) = (/ &
     &9.6546e-05_rb,1.1187e-04_rb,1.3461e-04_rb,1.6319e-04_rb,2.0430e-04_rb/)
      kbo(:,21, 2) = (/ &
     &8.0465e-05_rb,9.3512e-05_rb,1.1260e-04_rb,1.3712e-04_rb,1.7240e-04_rb/)
      kbo(:,22, 2) = (/ &
     &6.7521e-05_rb,7.9100e-05_rb,9.4853e-05_rb,1.1694e-04_rb,1.4756e-04_rb/)
      kbo(:,23, 2) = (/ &
     &5.6632e-05_rb,6.7211e-05_rb,8.0875e-05_rb,9.9624e-05_rb,1.2638e-04_rb/)
      kbo(:,24, 2) = (/ &
     &4.7866e-05_rb,5.7310e-05_rb,6.9154e-05_rb,8.5301e-05_rb,1.0831e-04_rb/)
      kbo(:,25, 2) = (/ &
     &4.0652e-05_rb,4.8933e-05_rb,5.9257e-05_rb,7.3835e-05_rb,9.2951e-05_rb/)
      kbo(:,26, 2) = (/ &
     &3.4655e-05_rb,4.1694e-05_rb,5.0935e-05_rb,6.3979e-05_rb,8.0042e-05_rb/)
      kbo(:,27, 2) = (/ &
     &2.9633e-05_rb,3.5562e-05_rb,4.3853e-05_rb,5.5348e-05_rb,6.8920e-05_rb/)
      kbo(:,28, 2) = (/ &
     &2.5432e-05_rb,3.0614e-05_rb,3.7665e-05_rb,4.7871e-05_rb,5.9328e-05_rb/)
      kbo(:,29, 2) = (/ &
     &2.1848e-05_rb,2.6432e-05_rb,3.2844e-05_rb,4.1383e-05_rb,5.1047e-05_rb/)
      kbo(:,30, 2) = (/ &
     &1.8680e-05_rb,2.2782e-05_rb,2.8578e-05_rb,3.5766e-05_rb,4.3772e-05_rb/)
      kbo(:,31, 2) = (/ &
     &1.5971e-05_rb,1.9690e-05_rb,2.4825e-05_rb,3.0907e-05_rb,3.7564e-05_rb/)
      kbo(:,32, 2) = (/ &
     &1.3804e-05_rb,1.6977e-05_rb,2.1562e-05_rb,2.6712e-05_rb,3.2275e-05_rb/)
      kbo(:,33, 2) = (/ &
     &1.1945e-05_rb,1.4856e-05_rb,1.8696e-05_rb,2.3037e-05_rb,2.7659e-05_rb/)
      kbo(:,34, 2) = (/ &
     &1.0252e-05_rb,1.2851e-05_rb,1.6075e-05_rb,1.9670e-05_rb,2.3561e-05_rb/)
      kbo(:,35, 2) = (/ &
     &8.6288e-06_rb,1.0847e-05_rb,1.3532e-05_rb,1.6502e-05_rb,1.9733e-05_rb/)
      kbo(:,36, 2) = (/ &
     &7.0970e-06_rb,8.9246e-06_rb,1.1130e-05_rb,1.3564e-05_rb,1.6217e-05_rb/)
      kbo(:,37, 2) = (/ &
     &5.6546e-06_rb,7.0912e-06_rb,8.8667e-06_rb,1.0842e-05_rb,1.2980e-05_rb/)
      kbo(:,38, 2) = (/ &
     &4.4964e-06_rb,5.6180e-06_rb,7.0484e-06_rb,8.6481e-06_rb,1.0377e-05_rb/)
      kbo(:,39, 2) = (/ &
     &3.5794e-06_rb,4.4460e-06_rb,5.6006e-06_rb,6.9055e-06_rb,8.2935e-06_rb/)
      kbo(:,40, 2) = (/ &
     &2.7849e-06_rb,3.4244e-06_rb,4.3497e-06_rb,5.3888e-06_rb,6.5103e-06_rb/)
      kbo(:,41, 2) = (/ &
     &2.1634e-06_rb,2.6661e-06_rb,3.3623e-06_rb,4.1856e-06_rb,5.0849e-06_rb/)
      kbo(:,42, 2) = (/ &
     &1.6951e-06_rb,2.0688e-06_rb,2.5958e-06_rb,3.2479e-06_rb,3.9733e-06_rb/)
      kbo(:,43, 2) = (/ &
     &1.3156e-06_rb,1.5889e-06_rb,1.9703e-06_rb,2.4872e-06_rb,3.0727e-06_rb/)
      kbo(:,44, 2) = (/ &
     &1.0083e-06_rb,1.2108e-06_rb,1.4918e-06_rb,1.8898e-06_rb,2.3484e-06_rb/)
      kbo(:,45, 2) = (/ &
     &7.7597e-07_rb,9.3178e-07_rb,1.1396e-06_rb,1.4323e-06_rb,1.7919e-06_rb/)
      kbo(:,46, 2) = (/ &
     &5.9640e-07_rb,7.1743e-07_rb,8.6664e-07_rb,1.0739e-06_rb,1.3585e-06_rb/)
      kbo(:,47, 2) = (/ &
     &4.5875e-07_rb,5.4303e-07_rb,6.5202e-07_rb,8.0415e-07_rb,1.0165e-06_rb/)
      kbo(:,48, 2) = (/ &
     &3.5634e-07_rb,4.1385e-07_rb,4.9968e-07_rb,6.0505e-07_rb,7.5779e-07_rb/)
      kbo(:,49, 2) = (/ &
     &2.7966e-07_rb,3.1607e-07_rb,3.7849e-07_rb,4.5539e-07_rb,5.6073e-07_rb/)
      kbo(:,50, 2) = (/ &
     &2.2335e-07_rb,2.4619e-07_rb,2.8832e-07_rb,3.4563e-07_rb,4.2520e-07_rb/)
      kbo(:,51, 2) = (/ &
     &1.6810e-07_rb,1.9184e-07_rb,2.2126e-07_rb,2.6681e-07_rb,3.2272e-07_rb/)
      kbo(:,52, 2) = (/ &
     &1.2477e-07_rb,1.5183e-07_rb,1.7051e-07_rb,2.0325e-07_rb,2.4465e-07_rb/)
      kbo(:,53, 2) = (/ &
     &9.1697e-08_rb,1.2163e-07_rb,1.3293e-07_rb,1.5546e-07_rb,1.8654e-07_rb/)
      kbo(:,54, 2) = (/ &
     &6.8066e-08_rb,9.0724e-08_rb,1.0426e-07_rb,1.1988e-07_rb,1.4459e-07_rb/)
      kbo(:,55, 2) = (/ &
     &5.0853e-08_rb,6.8116e-08_rb,8.2949e-08_rb,9.3086e-08_rb,1.1106e-07_rb/)
      kbo(:,56, 2) = (/ &
     &3.8097e-08_rb,5.0708e-08_rb,6.6601e-08_rb,7.3097e-08_rb,8.5626e-08_rb/)
      kbo(:,57, 2) = (/ &
     &2.8575e-08_rb,3.7778e-08_rb,5.0543e-08_rb,5.7226e-08_rb,6.6193e-08_rb/)
      kbo(:,58, 2) = (/ &
     &2.1274e-08_rb,2.8319e-08_rb,3.8061e-08_rb,4.5485e-08_rb,5.1336e-08_rb/)
      kbo(:,59, 2) = (/ &
     &1.6785e-08_rb,2.2389e-08_rb,3.0047e-08_rb,3.6850e-08_rb,4.1308e-08_rb/)
      kbo(:,13, 3) = (/ &
     &4.8329e-04_rb,7.1031e-04_rb,9.2618e-04_rb,1.1787e-03_rb,1.4998e-03_rb/)
      kbo(:,14, 3) = (/ &
     &4.1573e-04_rb,5.9621e-04_rb,7.7781e-04_rb,9.9253e-04_rb,1.2561e-03_rb/)
      kbo(:,15, 3) = (/ &
     &3.5745e-04_rb,5.0356e-04_rb,6.5391e-04_rb,8.3999e-04_rb,1.0559e-03_rb/)
      kbo(:,16, 3) = (/ &
     &3.0758e-04_rb,4.2512e-04_rb,5.5063e-04_rb,7.1032e-04_rb,8.8405e-04_rb/)
      kbo(:,17, 3) = (/ &
     &2.6428e-04_rb,3.5939e-04_rb,4.6476e-04_rb,5.9990e-04_rb,7.3444e-04_rb/)
      kbo(:,18, 3) = (/ &
     &2.2620e-04_rb,3.0371e-04_rb,3.9242e-04_rb,5.0569e-04_rb,6.1057e-04_rb/)
      kbo(:,19, 3) = (/ &
     &1.9349e-04_rb,2.5703e-04_rb,3.3045e-04_rb,4.2572e-04_rb,5.0885e-04_rb/)
      kbo(:,20, 3) = (/ &
     &1.6601e-04_rb,2.1679e-04_rb,2.7938e-04_rb,3.5877e-04_rb,4.2456e-04_rb/)
      kbo(:,21, 3) = (/ &
     &1.3973e-04_rb,1.8356e-04_rb,2.3660e-04_rb,3.0163e-04_rb,3.5438e-04_rb/)
      kbo(:,22, 3) = (/ &
     &1.2024e-04_rb,1.5775e-04_rb,2.0424e-04_rb,2.5718e-04_rb,2.9870e-04_rb/)
      kbo(:,23, 3) = (/ &
     &1.0421e-04_rb,1.3581e-04_rb,1.7576e-04_rb,2.1785e-04_rb,2.5256e-04_rb/)
      kbo(:,24, 3) = (/ &
     &9.0642e-05_rb,1.1736e-04_rb,1.5142e-04_rb,1.8334e-04_rb,2.1432e-04_rb/)
      kbo(:,25, 3) = (/ &
     &7.8653e-05_rb,1.0139e-04_rb,1.3046e-04_rb,1.5520e-04_rb,1.8200e-04_rb/)
      kbo(:,26, 3) = (/ &
     &6.8347e-05_rb,8.8330e-05_rb,1.1221e-04_rb,1.3139e-04_rb,1.5501e-04_rb/)
      kbo(:,27, 3) = (/ &
     &5.9438e-05_rb,7.7110e-05_rb,9.6484e-05_rb,1.1163e-04_rb,1.3198e-04_rb/)
      kbo(:,28, 3) = (/ &
     &5.1865e-05_rb,6.7062e-05_rb,8.1622e-05_rb,9.5189e-05_rb,1.1250e-04_rb/)
      kbo(:,29, 3) = (/ &
     &4.5229e-05_rb,5.8248e-05_rb,6.9371e-05_rb,8.1305e-05_rb,9.5987e-05_rb/)
      kbo(:,30, 3) = (/ &
     &3.9548e-05_rb,5.0236e-05_rb,5.8927e-05_rb,6.9361e-05_rb,8.1985e-05_rb/)
      kbo(:,31, 3) = (/ &
     &3.4672e-05_rb,4.3357e-05_rb,5.0119e-05_rb,5.9230e-05_rb,7.0087e-05_rb/)
      kbo(:,32, 3) = (/ &
     &3.0291e-05_rb,3.6761e-05_rb,4.2859e-05_rb,5.0621e-05_rb,5.9826e-05_rb/)
      kbo(:,33, 3) = (/ &
     &2.6343e-05_rb,3.1291e-05_rb,3.6685e-05_rb,4.3251e-05_rb,5.1136e-05_rb/)
      kbo(:,34, 3) = (/ &
     &2.2619e-05_rb,2.6479e-05_rb,3.1160e-05_rb,3.6817e-05_rb,4.3454e-05_rb/)
      kbo(:,35, 3) = (/ &
     &1.9044e-05_rb,2.2094e-05_rb,2.6069e-05_rb,3.0814e-05_rb,3.6332e-05_rb/)
      kbo(:,36, 3) = (/ &
     &1.5660e-05_rb,1.8150e-05_rb,2.1420e-05_rb,2.5318e-05_rb,2.9845e-05_rb/)
      kbo(:,37, 3) = (/ &
     &1.2481e-05_rb,1.4567e-05_rb,1.7160e-05_rb,2.0273e-05_rb,2.3924e-05_rb/)
      kbo(:,38, 3) = (/ &
     &9.9229e-06_rb,1.1707e-05_rb,1.3728e-05_rb,1.6208e-05_rb,1.9147e-05_rb/)
      kbo(:,39, 3) = (/ &
     &7.8980e-06_rb,9.3824e-06_rb,1.0994e-05_rb,1.2956e-05_rb,1.5319e-05_rb/)
      kbo(:,40, 3) = (/ &
     &6.1214e-06_rb,7.4148e-06_rb,8.6411e-06_rb,1.0203e-05_rb,1.2052e-05_rb/)
      kbo(:,41, 3) = (/ &
     &4.7079e-06_rb,5.8792e-06_rb,6.7814e-06_rb,8.0075e-06_rb,9.4649e-06_rb/)
      kbo(:,42, 3) = (/ &
     &3.6048e-06_rb,4.5698e-06_rb,5.3449e-06_rb,6.2854e-06_rb,7.4263e-06_rb/)
      kbo(:,43, 3) = (/ &
     &2.7216e-06_rb,3.5076e-06_rb,4.1783e-06_rb,4.8932e-06_rb,5.7661e-06_rb/)
      kbo(:,44, 3) = (/ &
     &2.0468e-06_rb,2.6504e-06_rb,3.2648e-06_rb,3.7820e-06_rb,4.4701e-06_rb/)
      kbo(:,45, 3) = (/ &
     &1.5361e-06_rb,1.9893e-06_rb,2.5184e-06_rb,2.9393e-06_rb,3.4602e-06_rb/)
      kbo(:,46, 3) = (/ &
     &1.1503e-06_rb,1.4832e-06_rb,1.9137e-06_rb,2.2823e-06_rb,2.6719e-06_rb/)
      kbo(:,47, 3) = (/ &
     &8.3777e-07_rb,1.0971e-06_rb,1.4217e-06_rb,1.7739e-06_rb,2.0443e-06_rb/)
      kbo(:,48, 3) = (/ &
     &6.1407e-07_rb,8.0972e-07_rb,1.0453e-06_rb,1.3363e-06_rb,1.5776e-06_rb/)
      kbo(:,49, 3) = (/ &
     &4.3890e-07_rb,5.9676e-07_rb,7.7449e-07_rb,1.0018e-06_rb,1.2151e-06_rb/)
      kbo(:,50, 3) = (/ &
     &3.0232e-07_rb,4.3632e-07_rb,5.7501e-07_rb,7.4600e-07_rb,9.3964e-07_rb/)
      kbo(:,51, 3) = (/ &
     &2.1820e-07_rb,3.2558e-07_rb,4.2933e-07_rb,5.5411e-07_rb,7.1236e-07_rb/)
      kbo(:,52, 3) = (/ &
     &1.6142e-07_rb,2.3096e-07_rb,3.1815e-07_rb,4.1428e-07_rb,5.3670e-07_rb/)
      kbo(:,53, 3) = (/ &
     &1.2239e-07_rb,1.5927e-07_rb,2.3415e-07_rb,3.0848e-07_rb,4.0010e-07_rb/)
      kbo(:,54, 3) = (/ &
     &9.4287e-08_rb,1.1772e-07_rb,1.7609e-07_rb,2.3226e-07_rb,2.9972e-07_rb/)
      kbo(:,55, 3) = (/ &
     &7.3559e-08_rb,8.8158e-08_rb,1.2627e-07_rb,1.7401e-07_rb,2.2659e-07_rb/)
      kbo(:,56, 3) = (/ &
     &5.7474e-08_rb,6.7419e-08_rb,8.9064e-08_rb,1.2949e-07_rb,1.7067e-07_rb/)
      kbo(:,57, 3) = (/ &
     &4.5317e-08_rb,5.2043e-08_rb,6.5424e-08_rb,9.7761e-08_rb,1.2882e-07_rb/)
      kbo(:,58, 3) = (/ &
     &3.4988e-08_rb,4.0617e-08_rb,4.8970e-08_rb,7.1601e-08_rb,9.7431e-08_rb/)
      kbo(:,59, 3) = (/ &
     &2.8131e-08_rb,3.2653e-08_rb,3.8986e-08_rb,5.5678e-08_rb,7.6933e-08_rb/)
      kbo(:,13, 4) = (/ &
     &1.2375e-03_rb,1.3907e-03_rb,1.6848e-03_rb,2.0764e-03_rb,2.5455e-03_rb/)
      kbo(:,14, 4) = (/ &
     &1.0352e-03_rb,1.1626e-03_rb,1.4153e-03_rb,1.7410e-03_rb,2.1410e-03_rb/)
      kbo(:,15, 4) = (/ &
     &8.6916e-04_rb,9.7450e-04_rb,1.1918e-03_rb,1.4617e-03_rb,1.8090e-03_rb/)
      kbo(:,16, 4) = (/ &
     &7.1556e-04_rb,8.1812e-04_rb,1.0040e-03_rb,1.2311e-03_rb,1.5286e-03_rb/)
      kbo(:,17, 4) = (/ &
     &5.8965e-04_rb,6.8521e-04_rb,8.4372e-04_rb,1.0375e-03_rb,1.2976e-03_rb/)
      kbo(:,18, 4) = (/ &
     &4.8726e-04_rb,5.7342e-04_rb,7.0979e-04_rb,8.7350e-04_rb,1.0996e-03_rb/)
      kbo(:,19, 4) = (/ &
     &4.0346e-04_rb,4.7923e-04_rb,5.9512e-04_rb,7.3474e-04_rb,9.2909e-04_rb/)
      kbo(:,20, 4) = (/ &
     &3.3252e-04_rb,4.0340e-04_rb,5.0129e-04_rb,6.2120e-04_rb,7.8738e-04_rb/)
      kbo(:,21, 4) = (/ &
     &2.7824e-04_rb,3.3959e-04_rb,4.2206e-04_rb,5.2456e-04_rb,6.6651e-04_rb/)
      kbo(:,22, 4) = (/ &
     &2.3686e-04_rb,2.9111e-04_rb,3.6067e-04_rb,4.4836e-04_rb,5.7225e-04_rb/)
      kbo(:,23, 4) = (/ &
     &2.0203e-04_rb,2.4985e-04_rb,3.0880e-04_rb,3.8545e-04_rb,4.9113e-04_rb/)
      kbo(:,24, 4) = (/ &
     &1.7215e-04_rb,2.1409e-04_rb,2.6491e-04_rb,3.3340e-04_rb,4.2129e-04_rb/)
      kbo(:,25, 4) = (/ &
     &1.4744e-04_rb,1.8341e-04_rb,2.2761e-04_rb,2.8826e-04_rb,3.6138e-04_rb/)
      kbo(:,26, 4) = (/ &
     &1.2712e-04_rb,1.5795e-04_rb,1.9671e-04_rb,2.5046e-04_rb,3.1109e-04_rb/)
      kbo(:,27, 4) = (/ &
     &1.1012e-04_rb,1.3626e-04_rb,1.6969e-04_rb,2.1697e-04_rb,2.6764e-04_rb/)
      kbo(:,28, 4) = (/ &
     &9.5268e-05_rb,1.1775e-04_rb,1.4811e-04_rb,1.8739e-04_rb,2.3002e-04_rb/)
      kbo(:,29, 4) = (/ &
     &8.1990e-05_rb,1.0183e-04_rb,1.2887e-04_rb,1.6155e-04_rb,1.9779e-04_rb/)
      kbo(:,30, 4) = (/ &
     &7.0812e-05_rb,8.8197e-05_rb,1.1209e-04_rb,1.3932e-04_rb,1.6990e-04_rb/)
      kbo(:,31, 4) = (/ &
     &6.1297e-05_rb,7.6242e-05_rb,9.7371e-05_rb,1.2004e-04_rb,1.4578e-04_rb/)
      kbo(:,32, 4) = (/ &
     &5.3068e-05_rb,6.6743e-05_rb,8.4358e-05_rb,1.0341e-04_rb,1.2514e-04_rb/)
      kbo(:,33, 4) = (/ &
     &4.6021e-05_rb,5.8210e-05_rb,7.2823e-05_rb,8.9095e-05_rb,1.0732e-04_rb/)
      kbo(:,34, 4) = (/ &
     &3.9611e-05_rb,5.0325e-05_rb,6.2484e-05_rb,7.6131e-05_rb,9.1418e-05_rb/)
      kbo(:,35, 4) = (/ &
     &3.3314e-05_rb,4.2493e-05_rb,5.2519e-05_rb,6.3851e-05_rb,7.6543e-05_rb/)
      kbo(:,36, 4) = (/ &
     &2.7397e-05_rb,3.4956e-05_rb,4.3170e-05_rb,5.2466e-05_rb,6.2879e-05_rb/)
      kbo(:,37, 4) = (/ &
     &2.1842e-05_rb,2.7769e-05_rb,3.4424e-05_rb,4.1912e-05_rb,5.0314e-05_rb/)
      kbo(:,38, 4) = (/ &
     &1.7352e-05_rb,2.1971e-05_rb,2.7389e-05_rb,3.3429e-05_rb,4.0195e-05_rb/)
      kbo(:,39, 4) = (/ &
     &1.3770e-05_rb,1.7414e-05_rb,2.1778e-05_rb,2.6648e-05_rb,3.2100e-05_rb/)
      kbo(:,40, 4) = (/ &
     &1.0696e-05_rb,1.3450e-05_rb,1.6968e-05_rb,2.0795e-05_rb,2.5164e-05_rb/)
      kbo(:,41, 4) = (/ &
     &8.2926e-06_rb,1.0301e-05_rb,1.3150e-05_rb,1.6196e-05_rb,1.9663e-05_rb/)
      kbo(:,42, 4) = (/ &
     &6.4321e-06_rb,7.9930e-06_rb,1.0153e-05_rb,1.2599e-05_rb,1.5345e-05_rb/)
      kbo(:,43, 4) = (/ &
     &4.9294e-06_rb,6.1147e-06_rb,7.7222e-06_rb,9.6711e-06_rb,1.1840e-05_rb/)
      kbo(:,44, 4) = (/ &
     &3.7796e-06_rb,4.6518e-06_rb,5.8136e-06_rb,7.3808e-06_rb,9.0677e-06_rb/)
      kbo(:,45, 4) = (/ &
     &2.8649e-06_rb,3.5446e-06_rb,4.4045e-06_rb,5.6003e-06_rb,6.9419e-06_rb/)
      kbo(:,46, 4) = (/ &
     &2.1636e-06_rb,2.6897e-06_rb,3.3348e-06_rb,4.2109e-06_rb,5.2785e-06_rb/)
      kbo(:,47, 4) = (/ &
     &1.6423e-06_rb,2.0325e-06_rb,2.5019e-06_rb,3.1092e-06_rb,3.9697e-06_rb/)
      kbo(:,48, 4) = (/ &
     &1.2438e-06_rb,1.5176e-06_rb,1.8805e-06_rb,2.3386e-06_rb,2.9598e-06_rb/)
      kbo(:,49, 4) = (/ &
     &9.6637e-07_rb,1.1421e-06_rb,1.4184e-06_rb,1.7520e-06_rb,2.2027e-06_rb/)
      kbo(:,50, 4) = (/ &
     &7.6681e-07_rb,8.7015e-07_rb,1.0718e-06_rb,1.3233e-06_rb,1.6436e-06_rb/)
      kbo(:,51, 4) = (/ &
     &5.7638e-07_rb,6.6674e-07_rb,8.0765e-07_rb,1.0019e-06_rb,1.2448e-06_rb/)
      kbo(:,52, 4) = (/ &
     &4.3579e-07_rb,5.2641e-07_rb,6.1480e-07_rb,7.6360e-07_rb,9.4101e-07_rb/)
      kbo(:,53, 4) = (/ &
     &3.2630e-07_rb,4.1175e-07_rb,4.6866e-07_rb,5.7670e-07_rb,7.1273e-07_rb/)
      kbo(:,54, 4) = (/ &
     &2.4508e-07_rb,3.1226e-07_rb,3.6234e-07_rb,4.3785e-07_rb,5.4334e-07_rb/)
      kbo(:,55, 4) = (/ &
     &1.8867e-07_rb,2.3849e-07_rb,2.8812e-07_rb,3.3656e-07_rb,4.1795e-07_rb/)
      kbo(:,56, 4) = (/ &
     &1.3888e-07_rb,1.8052e-07_rb,2.2770e-07_rb,2.5896e-07_rb,3.1922e-07_rb/)
      kbo(:,57, 4) = (/ &
     &9.9211e-08_rb,1.3628e-07_rb,1.7309e-07_rb,2.0040e-07_rb,2.4315e-07_rb/)
      kbo(:,58, 4) = (/ &
     &7.2311e-08_rb,1.0440e-07_rb,1.3249e-07_rb,1.5839e-07_rb,1.8707e-07_rb/)
      kbo(:,59, 4) = (/ &
     &5.6449e-08_rb,8.3969e-08_rb,1.0579e-07_rb,1.2870e-07_rb,1.4987e-07_rb/)
      kbo(:,13, 5) = (/ &
     &2.0768e-03_rb,2.8684e-03_rb,3.7718e-03_rb,4.8192e-03_rb,6.0412e-03_rb/)
      kbo(:,14, 5) = (/ &
     &1.7564e-03_rb,2.4332e-03_rb,3.1824e-03_rb,4.0625e-03_rb,5.0889e-03_rb/)
      kbo(:,15, 5) = (/ &
     &1.4908e-03_rb,2.0712e-03_rb,2.6934e-03_rb,3.4353e-03_rb,4.2909e-03_rb/)
      kbo(:,16, 5) = (/ &
     &1.2837e-03_rb,1.7624e-03_rb,2.2850e-03_rb,2.9120e-03_rb,3.6366e-03_rb/)
      kbo(:,17, 5) = (/ &
     &1.1007e-03_rb,1.4970e-03_rb,1.9372e-03_rb,2.4633e-03_rb,3.0868e-03_rb/)
      kbo(:,18, 5) = (/ &
     &9.3998e-04_rb,1.2697e-03_rb,1.6378e-03_rb,2.0802e-03_rb,2.6082e-03_rb/)
      kbo(:,19, 5) = (/ &
     &8.0100e-04_rb,1.0742e-03_rb,1.3849e-03_rb,1.7569e-03_rb,2.2030e-03_rb/)
      kbo(:,20, 5) = (/ &
     &6.8663e-04_rb,9.1195e-04_rb,1.1757e-03_rb,1.4883e-03_rb,1.8647e-03_rb/)
      kbo(:,21, 5) = (/ &
     &5.8705e-04_rb,7.7399e-04_rb,9.9752e-04_rb,1.2607e-03_rb,1.5771e-03_rb/)
      kbo(:,22, 5) = (/ &
     &5.1050e-04_rb,6.6826e-04_rb,8.5880e-04_rb,1.0835e-03_rb,1.3514e-03_rb/)
      kbo(:,23, 5) = (/ &
     &4.4216e-04_rb,5.7700e-04_rb,7.3906e-04_rb,9.3102e-04_rb,1.1572e-03_rb/)
      kbo(:,24, 5) = (/ &
     &3.8397e-04_rb,4.9926e-04_rb,6.3682e-04_rb,8.0099e-04_rb,9.9155e-04_rb/)
      kbo(:,25, 5) = (/ &
     &3.3452e-04_rb,4.3352e-04_rb,5.4972e-04_rb,6.8946e-04_rb,8.5204e-04_rb/)
      kbo(:,26, 5) = (/ &
     &2.9253e-04_rb,3.7705e-04_rb,4.7637e-04_rb,5.9546e-04_rb,7.3410e-04_rb/)
      kbo(:,27, 5) = (/ &
     &2.5516e-04_rb,3.2740e-04_rb,4.1271e-04_rb,5.1423e-04_rb,6.3183e-04_rb/)
      kbo(:,28, 5) = (/ &
     &2.2216e-04_rb,2.8389e-04_rb,3.5748e-04_rb,4.4367e-04_rb,5.4344e-04_rb/)
      kbo(:,29, 5) = (/ &
     &1.9425e-04_rb,2.4642e-04_rb,3.0966e-04_rb,3.8285e-04_rb,4.6743e-04_rb/)
      kbo(:,30, 5) = (/ &
     &1.6916e-04_rb,2.1405e-04_rb,2.6782e-04_rb,3.3008e-04_rb,4.0176e-04_rb/)
      kbo(:,31, 5) = (/ &
     &1.4722e-04_rb,1.8596e-04_rb,2.3162e-04_rb,2.8455e-04_rb,3.4535e-04_rb/)
      kbo(:,32, 5) = (/ &
     &1.2822e-04_rb,1.6146e-04_rb,2.0021e-04_rb,2.4528e-04_rb,2.9677e-04_rb/)
      kbo(:,33, 5) = (/ &
     &1.1158e-04_rb,1.4011e-04_rb,1.7313e-04_rb,2.1133e-04_rb,2.5496e-04_rb/)
      kbo(:,34, 5) = (/ &
     &9.6356e-05_rb,1.2055e-04_rb,1.4854e-04_rb,1.8079e-04_rb,2.1758e-04_rb/)
      kbo(:,35, 5) = (/ &
     &8.1343e-05_rb,1.0151e-04_rb,1.2490e-04_rb,1.5178e-04_rb,1.8239e-04_rb/)
      kbo(:,36, 5) = (/ &
     &6.6965e-05_rb,8.3520e-05_rb,1.0275e-04_rb,1.2482e-04_rb,1.4993e-04_rb/)
      kbo(:,37, 5) = (/ &
     &5.3206e-05_rb,6.6545e-05_rb,8.1990e-05_rb,9.9764e-05_rb,1.2001e-04_rb/)
      kbo(:,38, 5) = (/ &
     &4.2208e-05_rb,5.2902e-05_rb,6.5299e-05_rb,7.9577e-05_rb,9.5895e-05_rb/)
      kbo(:,39, 5) = (/ &
     &3.3472e-05_rb,4.2041e-05_rb,5.1984e-05_rb,6.3462e-05_rb,7.6593e-05_rb/)
      kbo(:,40, 5) = (/ &
     &2.5925e-05_rb,3.2660e-05_rb,4.0509e-05_rb,4.9627e-05_rb,6.0037e-05_rb/)
      kbo(:,41, 5) = (/ &
     &2.0003e-05_rb,2.5267e-05_rb,3.1469e-05_rb,3.8672e-05_rb,4.6923e-05_rb/)
      kbo(:,42, 5) = (/ &
     &1.5411e-05_rb,1.9519e-05_rb,2.4420e-05_rb,3.0101e-05_rb,3.6637e-05_rb/)
      kbo(:,43, 5) = (/ &
     &1.1687e-05_rb,1.4867e-05_rb,1.8698e-05_rb,2.3140e-05_rb,2.8269e-05_rb/)
      kbo(:,44, 5) = (/ &
     &8.7679e-06_rb,1.1261e-05_rb,1.4209e-05_rb,1.7673e-05_rb,2.1693e-05_rb/)
      kbo(:,45, 5) = (/ &
     &6.5931e-06_rb,8.5087e-06_rb,1.0776e-05_rb,1.3478e-05_rb,1.6614e-05_rb/)
      kbo(:,46, 5) = (/ &
     &4.9080e-06_rb,6.3755e-06_rb,8.1160e-06_rb,1.0213e-05_rb,1.2647e-05_rb/)
      kbo(:,47, 5) = (/ &
     &3.5919e-06_rb,4.7038e-06_rb,6.0534e-06_rb,7.6485e-06_rb,9.5281e-06_rb/)
      kbo(:,48, 5) = (/ &
     &2.6062e-06_rb,3.4739e-06_rb,4.4987e-06_rb,5.7085e-06_rb,7.1618e-06_rb/)
      kbo(:,49, 5) = (/ &
     &1.8621e-06_rb,2.5496e-06_rb,3.3246e-06_rb,4.2578e-06_rb,5.3684e-06_rb/)
      kbo(:,50, 5) = (/ &
     &1.3311e-06_rb,1.8748e-06_rb,2.4717e-06_rb,3.1893e-06_rb,4.0386e-06_rb/)
      kbo(:,51, 5) = (/ &
     &9.8374e-07_rb,1.3665e-06_rb,1.8393e-06_rb,2.3887e-06_rb,3.0388e-06_rb/)
      kbo(:,52, 5) = (/ &
     &7.2293e-07_rb,9.8525e-07_rb,1.3616e-06_rb,1.7795e-06_rb,2.2858e-06_rb/)
      kbo(:,53, 5) = (/ &
     &5.3539e-07_rb,7.1322e-07_rb,1.0025e-06_rb,1.3275e-06_rb,1.7157e-06_rb/)
      kbo(:,54, 5) = (/ &
     &3.9532e-07_rb,5.3074e-07_rb,7.3585e-07_rb,9.9485e-07_rb,1.2941e-06_rb/)
      kbo(:,55, 5) = (/ &
     &2.9574e-07_rb,3.9534e-07_rb,5.3898e-07_rb,7.4534e-07_rb,9.7510e-07_rb/)
      kbo(:,56, 5) = (/ &
     &2.2158e-07_rb,2.9575e-07_rb,3.9553e-07_rb,5.5662e-07_rb,7.3612e-07_rb/)
      kbo(:,57, 5) = (/ &
     &1.7387e-07_rb,2.1999e-07_rb,2.9597e-07_rb,4.1152e-07_rb,5.5434e-07_rb/)
      kbo(:,58, 5) = (/ &
     &1.3813e-07_rb,1.6569e-07_rb,2.2197e-07_rb,3.0386e-07_rb,4.1774e-07_rb/)
      kbo(:,59, 5) = (/ &
     &1.1153e-07_rb,1.3014e-07_rb,1.7513e-07_rb,2.3832e-07_rb,3.3096e-07_rb/)
      kbo(:,13, 6) = (/ &
     &6.2563e-03_rb,8.5754e-03_rb,1.1313e-02_rb,1.4562e-02_rb,1.8439e-02_rb/)
      kbo(:,14, 6) = (/ &
     &5.3407e-03_rb,7.2874e-03_rb,9.6031e-03_rb,1.2344e-02_rb,1.5616e-02_rb/)
      kbo(:,15, 6) = (/ &
     &4.5624e-03_rb,6.2026e-03_rb,8.1616e-03_rb,1.0480e-02_rb,1.3236e-02_rb/)
      kbo(:,16, 6) = (/ &
     &3.8957e-03_rb,5.2793e-03_rb,6.9259e-03_rb,8.8846e-03_rb,1.1202e-02_rb/)
      kbo(:,17, 6) = (/ &
     &3.3209e-03_rb,4.4815e-03_rb,5.8698e-03_rb,7.5204e-03_rb,9.4486e-03_rb/)
      kbo(:,18, 6) = (/ &
     &2.8290e-03_rb,3.8089e-03_rb,4.9773e-03_rb,6.3684e-03_rb,7.9890e-03_rb/)
      kbo(:,19, 6) = (/ &
     &2.4112e-03_rb,3.2462e-03_rb,4.2316e-03_rb,5.4060e-03_rb,6.7720e-03_rb/)
      kbo(:,20, 6) = (/ &
     &2.0691e-03_rb,2.7791e-03_rb,3.6122e-03_rb,4.6028e-03_rb,5.7715e-03_rb/)
      kbo(:,21, 6) = (/ &
     &1.7739e-03_rb,2.3724e-03_rb,3.0729e-03_rb,3.9123e-03_rb,4.9081e-03_rb/)
      kbo(:,22, 6) = (/ &
     &1.5468e-03_rb,2.0535e-03_rb,2.6512e-03_rb,3.3770e-03_rb,4.2260e-03_rb/)
      kbo(:,23, 6) = (/ &
     &1.3475e-03_rb,1.7768e-03_rb,2.2959e-03_rb,2.9141e-03_rb,3.6373e-03_rb/)
      kbo(:,24, 6) = (/ &
     &1.1752e-03_rb,1.5439e-03_rb,1.9895e-03_rb,2.5170e-03_rb,3.1328e-03_rb/)
      kbo(:,25, 6) = (/ &
     &1.0263e-03_rb,1.3444e-03_rb,1.7264e-03_rb,2.1761e-03_rb,2.6990e-03_rb/)
      kbo(:,26, 6) = (/ &
     &9.0166e-04_rb,1.1758e-03_rb,1.5031e-03_rb,1.8871e-03_rb,2.3306e-03_rb/)
      kbo(:,27, 6) = (/ &
     &7.9141e-04_rb,1.0269e-03_rb,1.3074e-03_rb,1.6348e-03_rb,2.0092e-03_rb/)
      kbo(:,28, 6) = (/ &
     &6.9371e-04_rb,8.9592e-04_rb,1.1355e-03_rb,1.4137e-03_rb,1.7327e-03_rb/)
      kbo(:,29, 6) = (/ &
     &6.0795e-04_rb,7.8166e-04_rb,9.8572e-04_rb,1.2233e-03_rb,1.4938e-03_rb/)
      kbo(:,30, 6) = (/ &
     &5.3230e-04_rb,6.8106e-04_rb,8.5522e-04_rb,1.0576e-03_rb,1.2859e-03_rb/)
      kbo(:,31, 6) = (/ &
     &4.6592e-04_rb,5.9303e-04_rb,7.4221e-04_rb,9.1346e-04_rb,1.1061e-03_rb/)
      kbo(:,32, 6) = (/ &
     &4.0751e-04_rb,5.1643e-04_rb,6.4366e-04_rb,7.8848e-04_rb,9.5095e-04_rb/)
      kbo(:,33, 6) = (/ &
     &3.5610e-04_rb,4.4934e-04_rb,5.5753e-04_rb,6.8015e-04_rb,8.1668e-04_rb/)
      kbo(:,34, 6) = (/ &
     &3.0849e-04_rb,3.8773e-04_rb,4.7909e-04_rb,5.8242e-04_rb,6.9697e-04_rb/)
      kbo(:,35, 6) = (/ &
     &2.6100e-04_rb,3.2729e-04_rb,4.0344e-04_rb,4.8961e-04_rb,5.8449e-04_rb/)
      kbo(:,36, 6) = (/ &
     &2.1525e-04_rb,2.6972e-04_rb,3.3236e-04_rb,4.0320e-04_rb,4.8106e-04_rb/)
      kbo(:,37, 6) = (/ &
     &1.7127e-04_rb,2.1505e-04_rb,2.6562e-04_rb,3.2288e-04_rb,3.8605e-04_rb/)
      kbo(:,38, 6) = (/ &
     &1.3592e-04_rb,1.7107e-04_rb,2.1184e-04_rb,2.5803e-04_rb,3.0917e-04_rb/)
      kbo(:,39, 6) = (/ &
     &1.0785e-04_rb,1.3607e-04_rb,1.6885e-04_rb,2.0614e-04_rb,2.4755e-04_rb/)
      kbo(:,40, 6) = (/ &
     &8.3445e-05_rb,1.0573e-04_rb,1.3174e-04_rb,1.6145e-04_rb,1.9465e-04_rb/)
      kbo(:,41, 6) = (/ &
     &6.4274e-05_rb,8.1836e-05_rb,1.0238e-04_rb,1.2600e-04_rb,1.5256e-04_rb/)
      kbo(:,42, 6) = (/ &
     &4.9425e-05_rb,6.3242e-05_rb,7.9461e-05_rb,9.8197e-05_rb,1.1940e-04_rb/)
      kbo(:,43, 6) = (/ &
     &3.7393e-05_rb,4.8142e-05_rb,6.0818e-05_rb,7.5575e-05_rb,9.2337e-05_rb/)
      kbo(:,44, 6) = (/ &
     &2.8031e-05_rb,3.6364e-05_rb,4.6223e-05_rb,5.7754e-05_rb,7.0960e-05_rb/)
      kbo(:,45, 6) = (/ &
     &2.0952e-05_rb,2.7396e-05_rb,3.5050e-05_rb,4.4048e-05_rb,5.4413e-05_rb/)
      kbo(:,46, 6) = (/ &
     &1.5504e-05_rb,2.0469e-05_rb,2.6380e-05_rb,3.3353e-05_rb,4.1465e-05_rb/)
      kbo(:,47, 6) = (/ &
     &1.1285e-05_rb,1.5071e-05_rb,1.9596e-05_rb,2.4961e-05_rb,3.1243e-05_rb/)
      kbo(:,48, 6) = (/ &
     &8.1713e-06_rb,1.1048e-05_rb,1.4507e-05_rb,1.8626e-05_rb,2.3478e-05_rb/)
      kbo(:,49, 6) = (/ &
     &5.8892e-06_rb,8.0594e-06_rb,1.0700e-05_rb,1.3854e-05_rb,1.7591e-05_rb/)
      kbo(:,50, 6) = (/ &
     &4.2527e-06_rb,5.8989e-06_rb,7.9160e-06_rb,1.0341e-05_rb,1.3223e-05_rb/)
      kbo(:,51, 6) = (/ &
     &3.0672e-06_rb,4.3146e-06_rb,5.8537e-06_rb,7.7179e-06_rb,9.9420e-06_rb/)
      kbo(:,52, 6) = (/ &
     &2.1919e-06_rb,3.1415e-06_rb,4.3130e-06_rb,5.7435e-06_rb,7.4576e-06_rb/)
      kbo(:,53, 6) = (/ &
     &1.5489e-06_rb,2.2740e-06_rb,3.1638e-06_rb,4.2575e-06_rb,5.5767e-06_rb/)
      kbo(:,54, 6) = (/ &
     &1.1055e-06_rb,1.6569e-06_rb,2.3353e-06_rb,3.1738e-06_rb,4.1929e-06_rb/)
      kbo(:,55, 6) = (/ &
     &7.7959e-07_rb,1.2025e-06_rb,1.7241e-06_rb,2.3677e-06_rb,3.1567e-06_rb/)
      kbo(:,56, 6) = (/ &
     &5.5393e-07_rb,8.6666e-07_rb,1.2673e-06_rb,1.7619e-06_rb,2.3695e-06_rb/)
      kbo(:,57, 6) = (/ &
     &3.8362e-07_rb,6.2174e-07_rb,9.3002e-07_rb,1.3068e-06_rb,1.7740e-06_rb/)
      kbo(:,58, 6) = (/ &
     &2.6381e-07_rb,4.4330e-07_rb,6.7991e-07_rb,9.7066e-07_rb,1.3310e-06_rb/)
      kbo(:,59, 6) = (/ &
     &2.0120e-07_rb,3.4415e-07_rb,5.3237e-07_rb,7.6501e-07_rb,1.0533e-06_rb/)
      kbo(:,13, 7) = (/ &
     &1.9678e-02_rb,2.7339e-02_rb,3.6693e-02_rb,4.7583e-02_rb,5.9838e-02_rb/)
      kbo(:,14, 7) = (/ &
     &1.6855e-02_rb,2.3332e-02_rb,3.1200e-02_rb,4.0354e-02_rb,5.0549e-02_rb/)
      kbo(:,15, 7) = (/ &
     &1.4447e-02_rb,1.9926e-02_rb,2.6539e-02_rb,3.4203e-02_rb,4.2741e-02_rb/)
      kbo(:,16, 7) = (/ &
     &1.2375e-02_rb,1.7007e-02_rb,2.2570e-02_rb,2.9011e-02_rb,3.6155e-02_rb/)
      kbo(:,17, 7) = (/ &
     &1.0572e-02_rb,1.4480e-02_rb,1.9148e-02_rb,2.4557e-02_rb,3.0543e-02_rb/)
      kbo(:,18, 7) = (/ &
     &9.0187e-03_rb,1.2308e-02_rb,1.6230e-02_rb,2.0762e-02_rb,2.5754e-02_rb/)
      kbo(:,19, 7) = (/ &
     &7.6960e-03_rb,1.0462e-02_rb,1.3757e-02_rb,1.7553e-02_rb,2.1712e-02_rb/)
      kbo(:,20, 7) = (/ &
     &6.5947e-03_rb,8.9295e-03_rb,1.1707e-02_rb,1.4880e-02_rb,1.8332e-02_rb/)
      kbo(:,21, 7) = (/ &
     &5.6518e-03_rb,7.6362e-03_rb,9.9897e-03_rb,1.2646e-02_rb,1.5515e-02_rb/)
      kbo(:,22, 7) = (/ &
     &4.9551e-03_rb,6.6562e-03_rb,8.6611e-03_rb,1.0883e-02_rb,1.3285e-02_rb/)
      kbo(:,23, 7) = (/ &
     &4.3463e-03_rb,5.8019e-03_rb,7.4874e-03_rb,9.3424e-03_rb,1.1393e-02_rb/)
      kbo(:,24, 7) = (/ &
     &3.8088e-03_rb,5.0439e-03_rb,6.4617e-03_rb,8.0406e-03_rb,9.7631e-03_rb/)
      kbo(:,25, 7) = (/ &
     &3.3364e-03_rb,4.3855e-03_rb,5.5891e-03_rb,6.9199e-03_rb,8.3730e-03_rb/)
      kbo(:,26, 7) = (/ &
     &2.9302e-03_rb,3.8269e-03_rb,4.8484e-03_rb,5.9724e-03_rb,7.1930e-03_rb/)
      kbo(:,27, 7) = (/ &
     &2.5694e-03_rb,3.3414e-03_rb,4.2035e-03_rb,5.1445e-03_rb,6.1710e-03_rb/)
      kbo(:,28, 7) = (/ &
     &2.2557e-03_rb,2.9117e-03_rb,3.6362e-03_rb,4.4284e-03_rb,5.2859e-03_rb/)
      kbo(:,29, 7) = (/ &
     &1.9791e-03_rb,2.5337e-03_rb,3.1431e-03_rb,3.8083e-03_rb,4.5227e-03_rb/)
      kbo(:,30, 7) = (/ &
     &1.7348e-03_rb,2.2004e-03_rb,2.7128e-03_rb,3.2695e-03_rb,3.8671e-03_rb/)
      kbo(:,31, 7) = (/ &
     &1.5168e-03_rb,1.9090e-03_rb,2.3389e-03_rb,2.8042e-03_rb,3.3063e-03_rb/)
      kbo(:,32, 7) = (/ &
     &1.3245e-03_rb,1.6546e-03_rb,2.0148e-03_rb,2.4067e-03_rb,2.8256e-03_rb/)
      kbo(:,33, 7) = (/ &
     &1.1547e-03_rb,1.4322e-03_rb,1.7355e-03_rb,2.0639e-03_rb,2.4127e-03_rb/)
      kbo(:,34, 7) = (/ &
     &9.9732e-04_rb,1.2299e-03_rb,1.4851e-03_rb,1.7585e-03_rb,2.0486e-03_rb/)
      kbo(:,35, 7) = (/ &
     &8.4240e-04_rb,1.0365e-03_rb,1.2482e-03_rb,1.4745e-03_rb,1.7149e-03_rb/)
      kbo(:,36, 7) = (/ &
     &6.9459e-04_rb,8.5479e-04_rb,1.0290e-03_rb,1.2151e-03_rb,1.4131e-03_rb/)
      kbo(:,37, 7) = (/ &
     &5.5418e-04_rb,6.8431e-04_rb,8.2605e-04_rb,9.7772e-04_rb,1.1393e-03_rb/)
      kbo(:,38, 7) = (/ &
     &4.4103e-04_rb,5.4666e-04_rb,6.6164e-04_rb,7.8531e-04_rb,9.1717e-04_rb/)
      kbo(:,39, 7) = (/ &
     &3.5078e-04_rb,4.3647e-04_rb,5.2986e-04_rb,6.3056e-04_rb,7.3809e-04_rb/)
      kbo(:,40, 7) = (/ &
     &2.7223e-04_rb,3.4099e-04_rb,4.1623e-04_rb,4.9763e-04_rb,5.8479e-04_rb/)
      kbo(:,41, 7) = (/ &
     &2.1016e-04_rb,2.6529e-04_rb,3.2578e-04_rb,3.9146e-04_rb,4.6190e-04_rb/)
      kbo(:,42, 7) = (/ &
     &1.6186e-04_rb,2.0599e-04_rb,2.5454e-04_rb,3.0749e-04_rb,3.6441e-04_rb/)
      kbo(:,43, 7) = (/ &
     &1.2244e-04_rb,1.5755e-04_rb,1.9631e-04_rb,2.3871e-04_rb,2.8458e-04_rb/)
      kbo(:,44, 7) = (/ &
     &9.1656e-05_rb,1.1940e-04_rb,1.5021e-04_rb,1.8408e-04_rb,2.2084e-04_rb/)
      kbo(:,45, 7) = (/ &
     &6.8343e-05_rb,9.0130e-05_rb,1.1460e-04_rb,1.4155e-04_rb,1.7102e-04_rb/)
      kbo(:,46, 7) = (/ &
     &5.0438e-05_rb,6.7336e-05_rb,8.6692e-05_rb,1.0813e-04_rb,1.3160e-04_rb/)
      kbo(:,47, 7) = (/ &
     &3.6553e-05_rb,4.9462e-05_rb,6.4648e-05_rb,8.1563e-05_rb,1.0018e-04_rb/)
      kbo(:,48, 7) = (/ &
     &2.6343e-05_rb,3.6142e-05_rb,4.7933e-05_rb,6.1264e-05_rb,7.5986e-05_rb/)
      kbo(:,49, 7) = (/ &
     &1.8866e-05_rb,2.6263e-05_rb,3.5311e-05_rb,4.5778e-05_rb,5.7425e-05_rb/)
      kbo(:,50, 7) = (/ &
     &1.3557e-05_rb,1.9139e-05_rb,2.6064e-05_rb,3.4268e-05_rb,4.3473e-05_rb/)
      kbo(:,51, 7) = (/ &
     &9.7297e-06_rb,1.3926e-05_rb,1.9216e-05_rb,2.5597e-05_rb,3.2867e-05_rb/)
      kbo(:,52, 7) = (/ &
     &6.9563e-06_rb,1.0085e-05_rb,1.4103e-05_rb,1.9022e-05_rb,2.4754e-05_rb/)
      kbo(:,53, 7) = (/ &
     &4.9502e-06_rb,7.2644e-06_rb,1.0302e-05_rb,1.4070e-05_rb,1.8558e-05_rb/)
      kbo(:,54, 7) = (/ &
     &3.5499e-06_rb,5.2696e-06_rb,7.5685e-06_rb,1.0462e-05_rb,1.3965e-05_rb/)
      kbo(:,55, 7) = (/ &
     &2.5517e-06_rb,3.8332e-06_rb,5.5671e-06_rb,7.7870e-06_rb,1.0502e-05_rb/)
      kbo(:,56, 7) = (/ &
     &1.8250e-06_rb,2.7802e-06_rb,4.0807e-06_rb,5.7760e-06_rb,7.8765e-06_rb/)
      kbo(:,57, 7) = (/ &
     &1.3020e-06_rb,2.0096e-06_rb,2.9783e-06_rb,4.2688e-06_rb,5.8872e-06_rb/)
      kbo(:,58, 7) = (/ &
     &9.3216e-07_rb,1.4580e-06_rb,2.1842e-06_rb,3.1634e-06_rb,4.4097e-06_rb/)
      kbo(:,59, 7) = (/ &
     &7.2634e-07_rb,1.1405e-06_rb,1.7167e-06_rb,2.4950e-06_rb,3.4919e-06_rb/)
      kbo(:,13, 8) = (/ &
     &6.7703e-02_rb,9.1028e-02_rb,1.1838e-01_rb,1.4990e-01_rb,1.8559e-01_rb/)
      kbo(:,14, 8) = (/ &
     &5.8222e-02_rb,7.8007e-02_rb,1.0118e-01_rb,1.2784e-01_rb,1.5811e-01_rb/)
      kbo(:,15, 8) = (/ &
     &5.0069e-02_rb,6.6838e-02_rb,8.6469e-02_rb,1.0901e-01_rb,1.3468e-01_rb/)
      kbo(:,16, 8) = (/ &
     &4.3038e-02_rb,5.7247e-02_rb,7.3838e-02_rb,9.2876e-02_rb,1.1462e-01_rb/)
      kbo(:,17, 8) = (/ &
     &3.6905e-02_rb,4.8922e-02_rb,6.2921e-02_rb,7.8980e-02_rb,9.7249e-02_rb/)
      kbo(:,18, 8) = (/ &
     &3.1539e-02_rb,4.1670e-02_rb,5.3450e-02_rb,6.6951e-02_rb,8.2335e-02_rb/)
      kbo(:,19, 8) = (/ &
     &2.6840e-02_rb,3.5366e-02_rb,4.5264e-02_rb,5.6569e-02_rb,6.9496e-02_rb/)
      kbo(:,20, 8) = (/ &
     &2.2918e-02_rb,3.0090e-02_rb,3.8404e-02_rb,4.7928e-02_rb,5.8831e-02_rb/)
      kbo(:,21, 8) = (/ &
     &1.9556e-02_rb,2.5586e-02_rb,3.2570e-02_rb,4.0587e-02_rb,4.9753e-02_rb/)
      kbo(:,22, 8) = (/ &
     &1.6988e-02_rb,2.2111e-02_rb,2.8028e-02_rb,3.4852e-02_rb,4.2609e-02_rb/)
      kbo(:,23, 8) = (/ &
     &1.4811e-02_rb,1.9201e-02_rb,2.4269e-02_rb,3.0112e-02_rb,3.6663e-02_rb/)
      kbo(:,24, 8) = (/ &
     &1.2965e-02_rb,1.6745e-02_rb,2.1118e-02_rb,2.6093e-02_rb,3.1689e-02_rb/)
      kbo(:,25, 8) = (/ &
     &1.1393e-02_rb,1.4643e-02_rb,1.8383e-02_rb,2.2644e-02_rb,2.7400e-02_rb/)
      kbo(:,26, 8) = (/ &
     &1.0038e-02_rb,1.2830e-02_rb,1.6015e-02_rb,1.9632e-02_rb,2.3704e-02_rb/)
      kbo(:,27, 8) = (/ &
     &8.8123e-03_rb,1.1178e-02_rb,1.3890e-02_rb,1.6992e-02_rb,2.0492e-02_rb/)
      kbo(:,28, 8) = (/ &
     &7.6974e-03_rb,9.7127e-03_rb,1.2040e-02_rb,1.4716e-02_rb,1.7696e-02_rb/)
      kbo(:,29, 8) = (/ &
     &6.7206e-03_rb,8.4382e-03_rb,1.0458e-02_rb,1.2740e-02_rb,1.5282e-02_rb/)
      kbo(:,30, 8) = (/ &
     &5.8552e-03_rb,7.3524e-03_rb,9.0759e-03_rb,1.1022e-02_rb,1.3175e-02_rb/)
      kbo(:,31, 8) = (/ &
     &5.1145e-03_rb,6.3996e-03_rb,7.8758e-03_rb,9.5283e-03_rb,1.1351e-02_rb/)
      kbo(:,32, 8) = (/ &
     &4.4686e-03_rb,5.5753e-03_rb,6.8326e-03_rb,8.2331e-03_rb,9.7798e-03_rb/)
      kbo(:,33, 8) = (/ &
     &3.9075e-03_rb,4.8544e-03_rb,5.9240e-03_rb,7.1154e-03_rb,8.4216e-03_rb/)
      kbo(:,34, 8) = (/ &
     &3.3885e-03_rb,4.1943e-03_rb,5.1017e-03_rb,6.1096e-03_rb,7.2102e-03_rb/)
      kbo(:,35, 8) = (/ &
     &2.8755e-03_rb,3.5515e-03_rb,4.3142e-03_rb,5.1540e-03_rb,6.0867e-03_rb/)
      kbo(:,36, 8) = (/ &
     &2.3819e-03_rb,2.9429e-03_rb,3.5744e-03_rb,4.2743e-03_rb,5.0503e-03_rb/)
      kbo(:,37, 8) = (/ &
     &1.9080e-03_rb,2.3651e-03_rb,2.8794e-03_rb,3.4539e-03_rb,4.0906e-03_rb/)
      kbo(:,38, 8) = (/ &
     &1.5252e-03_rb,1.8961e-03_rb,2.3154e-03_rb,2.7875e-03_rb,3.3076e-03_rb/)
      kbo(:,39, 8) = (/ &
     &1.2186e-03_rb,1.5204e-03_rb,1.8625e-03_rb,2.2483e-03_rb,2.6746e-03_rb/)
      kbo(:,40, 8) = (/ &
     &9.5047e-04_rb,1.1919e-03_rb,1.4679e-03_rb,1.7800e-03_rb,2.1255e-03_rb/)
      kbo(:,41, 8) = (/ &
     &7.3797e-04_rb,9.3055e-04_rb,1.1523e-03_rb,1.4042e-03_rb,1.6837e-03_rb/)
      kbo(:,42, 8) = (/ &
     &5.7195e-04_rb,7.2531e-04_rb,9.0341e-04_rb,1.1062e-03_rb,1.3320e-03_rb/)
      kbo(:,43, 8) = (/ &
     &4.3632e-04_rb,5.5716e-04_rb,6.9866e-04_rb,8.6074e-04_rb,1.0421e-03_rb/)
      kbo(:,44, 8) = (/ &
     &3.2967e-04_rb,4.2439e-04_rb,5.3607e-04_rb,6.6494e-04_rb,8.0995e-04_rb/)
      kbo(:,45, 8) = (/ &
     &2.4830e-04_rb,3.2230e-04_rb,4.1025e-04_rb,5.1229e-04_rb,6.2794e-04_rb/)
      kbo(:,46, 8) = (/ &
     &1.8517e-04_rb,2.4270e-04_rb,3.1143e-04_rb,3.9189e-04_rb,4.8363e-04_rb/)
      kbo(:,47, 8) = (/ &
     &1.3585e-04_rb,1.8008e-04_rb,2.3317e-04_rb,2.9603e-04_rb,3.6826e-04_rb/)
      kbo(:,48, 8) = (/ &
     &9.9135e-05_rb,1.3301e-04_rb,1.7388e-04_rb,2.2278e-04_rb,2.7947e-04_rb/)
      kbo(:,49, 8) = (/ &
     &7.1904e-05_rb,9.7733e-05_rb,1.2918e-04_rb,1.6699e-04_rb,2.1129e-04_rb/)
      kbo(:,50, 8) = (/ &
     &5.2307e-05_rb,7.2019e-05_rb,9.6216e-05_rb,1.2554e-04_rb,1.6020e-04_rb/)
      kbo(:,51, 8) = (/ &
     &3.7969e-05_rb,5.2997e-05_rb,7.1620e-05_rb,9.4315e-05_rb,1.2143e-04_rb/)
      kbo(:,52, 8) = (/ &
     &2.7413e-05_rb,3.8825e-05_rb,5.3093e-05_rb,7.0665e-05_rb,9.1720e-05_rb/)
      kbo(:,53, 8) = (/ &
     &1.9654e-05_rb,2.8289e-05_rb,3.9179e-05_rb,5.2708e-05_rb,6.9024e-05_rb/)
      kbo(:,54, 8) = (/ &
     &1.4176e-05_rb,2.0734e-05_rb,2.9077e-05_rb,3.9534e-05_rb,5.2234e-05_rb/)
      kbo(:,55, 8) = (/ &
     &1.0223e-05_rb,1.5194e-05_rb,2.1592e-05_rb,2.9664e-05_rb,3.9585e-05_rb/)
      kbo(:,56, 8) = (/ &
     &7.3305e-06_rb,1.1089e-05_rb,1.5976e-05_rb,2.2196e-05_rb,2.9906e-05_rb/)
      kbo(:,57, 8) = (/ &
     &5.2231e-06_rb,8.0475e-06_rb,1.1769e-05_rb,1.6548e-05_rb,2.2524e-05_rb/)
      kbo(:,58, 8) = (/ &
     &3.7230e-06_rb,5.8489e-06_rb,8.6848e-06_rb,1.2363e-05_rb,1.6999e-05_rb/)
      kbo(:,59, 8) = (/ &
     &2.8893e-06_rb,4.5778e-06_rb,6.8447e-06_rb,9.7955e-06_rb,1.3534e-05_rb/)
      kbo(:,13, 9) = (/ &
     &4.1576e-01_rb,5.6459e-01_rb,7.4208e-01_rb,9.4959e-01_rb,1.1894e+00_rb/)
      kbo(:,14, 9) = (/ &
     &3.6259e-01_rb,4.8996e-01_rb,6.4236e-01_rb,8.2178e-01_rb,1.0273e+00_rb/)
      kbo(:,15, 9) = (/ &
     &3.1572e-01_rb,4.2505e-01_rb,5.5650e-01_rb,7.1044e-01_rb,8.8534e-01_rb/)
      kbo(:,16, 9) = (/ &
     &2.7461e-01_rb,3.6869e-01_rb,4.8148e-01_rb,6.1276e-01_rb,7.6136e-01_rb/)
      kbo(:,17, 9) = (/ &
     &2.3842e-01_rb,3.1920e-01_rb,4.1566e-01_rb,5.2724e-01_rb,6.5339e-01_rb/)
      kbo(:,18, 9) = (/ &
     &2.0643e-01_rb,2.7580e-01_rb,3.5808e-01_rb,4.5284e-01_rb,5.5957e-01_rb/)
      kbo(:,19, 9) = (/ &
     &1.7844e-01_rb,2.3768e-01_rb,3.0746e-01_rb,3.8778e-01_rb,4.7826e-01_rb/)
      kbo(:,20, 9) = (/ &
     &1.5457e-01_rb,2.0515e-01_rb,2.6455e-01_rb,3.3275e-01_rb,4.0945e-01_rb/)
      kbo(:,21, 9) = (/ &
     &1.3372e-01_rb,1.7673e-01_rb,2.2718e-01_rb,2.8500e-01_rb,3.5014e-01_rb/)
      kbo(:,22, 9) = (/ &
     &1.1755e-01_rb,1.5455e-01_rb,1.9772e-01_rb,2.4719e-01_rb,3.0287e-01_rb/)
      kbo(:,23, 9) = (/ &
     &1.0319e-01_rb,1.3488e-01_rb,1.7188e-01_rb,2.1417e-01_rb,2.6161e-01_rb/)
      kbo(:,24, 9) = (/ &
     &9.0543e-02_rb,1.1778e-01_rb,1.4943e-01_rb,1.8553e-01_rb,2.2596e-01_rb/)
      kbo(:,25, 9) = (/ &
     &7.9491e-02_rb,1.0285e-01_rb,1.2998e-01_rb,1.6082e-01_rb,1.9540e-01_rb/)
      kbo(:,26, 9) = (/ &
     &7.0023e-02_rb,9.0152e-02_rb,1.1342e-01_rb,1.3988e-01_rb,1.6957e-01_rb/)
      kbo(:,27, 9) = (/ &
     &6.1675e-02_rb,7.9028e-02_rb,9.9049e-02_rb,1.2176e-01_rb,1.4715e-01_rb/)
      kbo(:,28, 9) = (/ &
     &5.4332e-02_rb,6.9290e-02_rb,8.6538e-02_rb,1.0602e-01_rb,1.2768e-01_rb/)
      kbo(:,29, 9) = (/ &
     &4.8016e-02_rb,6.0968e-02_rb,7.5734e-02_rb,9.2386e-02_rb,1.1090e-01_rb/)
      kbo(:,30, 9) = (/ &
     &4.2494e-02_rb,5.3680e-02_rb,6.6413e-02_rb,8.0729e-02_rb,9.6539e-02_rb/)
      kbo(:,31, 9) = (/ &
     &3.7548e-02_rb,4.7237e-02_rb,5.8255e-02_rb,7.0556e-02_rb,8.4102e-02_rb/)
      kbo(:,32, 9) = (/ &
     &3.3189e-02_rb,4.1551e-02_rb,5.1044e-02_rb,6.1646e-02_rb,7.3306e-02_rb/)
      kbo(:,33, 9) = (/ &
     &2.9274e-02_rb,3.6514e-02_rb,4.4704e-02_rb,5.3824e-02_rb,6.3833e-02_rb/)
      kbo(:,34, 9) = (/ &
     &2.5627e-02_rb,3.1864e-02_rb,3.8903e-02_rb,4.6747e-02_rb,5.5596e-02_rb/)
      kbo(:,35, 9) = (/ &
     &2.1952e-02_rb,2.7259e-02_rb,3.3247e-02_rb,4.0073e-02_rb,4.7679e-02_rb/)
      kbo(:,36, 9) = (/ &
     &1.8347e-02_rb,2.2808e-02_rb,2.7904e-02_rb,3.3738e-02_rb,4.0178e-02_rb/)
      kbo(:,37, 9) = (/ &
     &1.4833e-02_rb,1.8504e-02_rb,2.2807e-02_rb,2.7664e-02_rb,3.3111e-02_rb/)
      kbo(:,38, 9) = (/ &
     &1.1957e-02_rb,1.5000e-02_rb,1.8592e-02_rb,2.2638e-02_rb,2.7241e-02_rb/)
      kbo(:,39, 9) = (/ &
     &9.6263e-03_rb,1.2169e-02_rb,1.5133e-02_rb,1.8540e-02_rb,2.2409e-02_rb/)
      kbo(:,40, 9) = (/ &
     &7.5755e-03_rb,9.6595e-03_rb,1.2095e-02_rb,1.4919e-02_rb,1.8126e-02_rb/)
      kbo(:,41, 9) = (/ &
     &5.9308e-03_rb,7.6313e-03_rb,9.6291e-03_rb,1.1965e-02_rb,1.4606e-02_rb/)
      kbo(:,42, 9) = (/ &
     &4.6365e-03_rb,6.0158e-03_rb,7.6539e-03_rb,9.5769e-03_rb,1.1776e-02_rb/)
      kbo(:,43, 9) = (/ &
     &3.5646e-03_rb,4.6692e-03_rb,5.9951e-03_rb,7.5584e-03_rb,9.3611e-03_rb/)
      kbo(:,44, 9) = (/ &
     &2.7083e-03_rb,3.5831e-03_rb,4.6545e-03_rb,5.9178e-03_rb,7.3883e-03_rb/)
      kbo(:,45, 9) = (/ &
     &2.0498e-03_rb,2.7405e-03_rb,3.5962e-03_rb,4.6175e-03_rb,5.8123e-03_rb/)
      kbo(:,46, 9) = (/ &
     &1.5335e-03_rb,2.0746e-03_rb,2.7520e-03_rb,3.5687e-03_rb,4.5390e-03_rb/)
      kbo(:,47, 9) = (/ &
     &1.1263e-03_rb,1.5445e-03_rb,2.0740e-03_rb,2.7200e-03_rb,3.4958e-03_rb/)
      kbo(:,48, 9) = (/ &
     &8.2184e-04_rb,1.1429e-03_rb,1.5538e-03_rb,2.0625e-03_rb,2.6781e-03_rb/)
      kbo(:,49, 9) = (/ &
     &5.9594e-04_rb,8.3975e-04_rb,1.1570e-03_rb,1.5542e-03_rb,2.0406e-03_rb/)
      kbo(:,50, 9) = (/ &
     &4.3385e-04_rb,6.1951e-04_rb,8.6528e-04_rb,1.1761e-03_rb,1.5615e-03_rb/)
      kbo(:,51, 9) = (/ &
     &3.1545e-04_rb,4.5652e-04_rb,6.4640e-04_rb,8.8983e-04_rb,1.1948e-03_rb/)
      kbo(:,52, 9) = (/ &
     &2.2788e-04_rb,3.3444e-04_rb,4.7984e-04_rb,6.6976e-04_rb,9.1050e-04_rb/)
      kbo(:,53, 9) = (/ &
     &1.6338e-04_rb,2.4338e-04_rb,3.5401e-04_rb,5.0057e-04_rb,6.8907e-04_rb/)
      kbo(:,54, 9) = (/ &
     &1.1823e-04_rb,1.7879e-04_rb,2.6356e-04_rb,3.7767e-04_rb,5.2615e-04_rb/)
      kbo(:,55, 9) = (/ &
     &8.5773e-05_rb,1.3160e-04_rb,1.9666e-04_rb,2.8549e-04_rb,4.0286e-04_rb/)
      kbo(:,56, 9) = (/ &
     &6.1815e-05_rb,9.6294e-05_rb,1.4602e-04_rb,2.1495e-04_rb,3.0724e-04_rb/)
      kbo(:,57, 9) = (/ &
     &4.4282e-05_rb,7.0071e-05_rb,1.0785e-04_rb,1.6097e-04_rb,2.3304e-04_rb/)
      kbo(:,58, 9) = (/ &
     &3.1779e-05_rb,5.1099e-05_rb,7.9795e-05_rb,1.2084e-04_rb,1.7730e-04_rb/)
      kbo(:,59, 9) = (/ &
     &2.4962e-05_rb,4.0511e-05_rb,6.3843e-05_rb,9.7519e-05_rb,1.4426e-04_rb/)
      kbo(:,13,10) = (/ &
     &1.8139e+00_rb,2.4724e+00_rb,3.2908e+00_rb,4.2191e+00_rb,5.2351e+00_rb/)
      kbo(:,14,10) = (/ &
     &1.6219e+00_rb,2.2181e+00_rb,2.9298e+00_rb,3.7301e+00_rb,4.6329e+00_rb/)
      kbo(:,15,10) = (/ &
     &1.4445e+00_rb,1.9645e+00_rb,2.5717e+00_rb,3.2739e+00_rb,4.0834e+00_rb/)
      kbo(:,16,10) = (/ &
     &1.2756e+00_rb,1.7196e+00_rb,2.2454e+00_rb,2.8649e+00_rb,3.5710e+00_rb/)
      kbo(:,17,10) = (/ &
     &1.1121e+00_rb,1.4934e+00_rb,1.9517e+00_rb,2.4879e+00_rb,3.0883e+00_rb/)
      kbo(:,18,10) = (/ &
     &9.6871e-01_rb,1.2953e+00_rb,1.6900e+00_rb,2.1475e+00_rb,2.6630e+00_rb/)
      kbo(:,19,10) = (/ &
     &8.4238e-01_rb,1.1243e+00_rb,1.4633e+00_rb,1.8537e+00_rb,2.2906e+00_rb/)
      kbo(:,20,10) = (/ &
     &7.3459e-01_rb,9.7958e-01_rb,1.2711e+00_rb,1.6026e+00_rb,1.9721e+00_rb/)
      kbo(:,21,10) = (/ &
     &6.4101e-01_rb,8.5278e-01_rb,1.0998e+00_rb,1.3825e+00_rb,1.6970e+00_rb/)
      kbo(:,22,10) = (/ &
     &5.6942e-01_rb,7.5220e-01_rb,9.6596e-01_rb,1.2079e+00_rb,1.4762e+00_rb/)
      kbo(:,23,10) = (/ &
     &5.0503e-01_rb,6.6296e-01_rb,8.4599e-01_rb,1.0531e+00_rb,1.2851e+00_rb/)
      kbo(:,24,10) = (/ &
     &4.4682e-01_rb,5.8292e-01_rb,7.4005e-01_rb,9.1863e-01_rb,1.1187e+00_rb/)
      kbo(:,25,10) = (/ &
     &3.9593e-01_rb,5.1361e-01_rb,6.4951e-01_rb,8.0398e-01_rb,9.7427e-01_rb/)
      kbo(:,26,10) = (/ &
     &3.5212e-01_rb,4.5450e-01_rb,5.7266e-01_rb,7.0571e-01_rb,8.5181e-01_rb/)
      kbo(:,27,10) = (/ &
     &3.1284e-01_rb,4.0173e-01_rb,5.0408e-01_rb,6.1838e-01_rb,7.4572e-01_rb/)
      kbo(:,28,10) = (/ &
     &2.7721e-01_rb,3.5447e-01_rb,4.4231e-01_rb,5.4145e-01_rb,6.5404e-01_rb/)
      kbo(:,29,10) = (/ &
     &2.4517e-01_rb,3.1175e-01_rb,3.8832e-01_rb,4.7588e-01_rb,5.7437e-01_rb/)
      kbo(:,30,10) = (/ &
     &2.1578e-01_rb,2.7344e-01_rb,3.4039e-01_rb,4.1657e-01_rb,5.0271e-01_rb/)
      kbo(:,31,10) = (/ &
     &1.9067e-01_rb,2.4099e-01_rb,2.9914e-01_rb,3.6552e-01_rb,4.4046e-01_rb/)
      kbo(:,32,10) = (/ &
     &1.6874e-01_rb,2.1279e-01_rb,2.6352e-01_rb,3.2154e-01_rb,3.8678e-01_rb/)
      kbo(:,33,10) = (/ &
     &1.4979e-01_rb,1.8828e-01_rb,2.3277e-01_rb,2.8345e-01_rb,3.4103e-01_rb/)
      kbo(:,34,10) = (/ &
     &1.3170e-01_rb,1.6528e-01_rb,2.0425e-01_rb,2.4841e-01_rb,2.9642e-01_rb/)
      kbo(:,35,10) = (/ &
     &1.1324e-01_rb,1.4235e-01_rb,1.7590e-01_rb,2.1287e-01_rb,2.5375e-01_rb/)
      kbo(:,36,10) = (/ &
     &9.5231e-02_rb,1.1973e-01_rb,1.4778e-01_rb,1.7871e-01_rb,2.1338e-01_rb/)
      kbo(:,37,10) = (/ &
     &7.7445e-02_rb,9.7552e-02_rb,1.1991e-01_rb,1.4578e-01_rb,1.7559e-01_rb/)
      kbo(:,38,10) = (/ &
     &6.2913e-02_rb,7.9354e-02_rb,9.7396e-02_rb,1.1872e-01_rb,1.4502e-01_rb/)
      kbo(:,39,10) = (/ &
     &5.1069e-02_rb,6.4406e-02_rb,7.9431e-02_rb,9.7751e-02_rb,1.1994e-01_rb/)
      kbo(:,40,10) = (/ &
     &4.0487e-02_rb,5.1192e-02_rb,6.3494e-02_rb,7.8782e-02_rb,9.6672e-02_rb/)
      kbo(:,41,10) = (/ &
     &3.1933e-02_rb,4.0574e-02_rb,5.0815e-02_rb,6.3454e-02_rb,7.7908e-02_rb/)
      kbo(:,42,10) = (/ &
     &2.5104e-02_rb,3.2117e-02_rb,4.0690e-02_rb,5.1132e-02_rb,6.3360e-02_rb/)
      kbo(:,43,10) = (/ &
     &1.9359e-02_rb,2.4985e-02_rb,3.2134e-02_rb,4.0672e-02_rb,5.0944e-02_rb/)
      kbo(:,44,10) = (/ &
     &1.4822e-02_rb,1.9377e-02_rb,2.5087e-02_rb,3.2083e-02_rb,4.0842e-02_rb/)
      kbo(:,45,10) = (/ &
     &1.1292e-02_rb,1.4912e-02_rb,1.9592e-02_rb,2.5310e-02_rb,3.2495e-02_rb/)
      kbo(:,46,10) = (/ &
     &8.5215e-03_rb,1.1374e-02_rb,1.5134e-02_rb,1.9829e-02_rb,2.5701e-02_rb/)
      kbo(:,47,10) = (/ &
     &6.3304e-03_rb,8.5539e-03_rb,1.1519e-02_rb,1.5304e-02_rb,2.0043e-02_rb/)
      kbo(:,48,10) = (/ &
     &4.6596e-03_rb,6.3842e-03_rb,8.7184e-03_rb,1.1727e-02_rb,1.5575e-02_rb/)
      kbo(:,49,10) = (/ &
     &3.3978e-03_rb,4.7268e-03_rb,6.5354e-03_rb,8.9194e-03_rb,1.1986e-02_rb/)
      kbo(:,50,10) = (/ &
     &2.4889e-03_rb,3.5204e-03_rb,4.9247e-03_rb,6.8240e-03_rb,9.2921e-03_rb/)
      kbo(:,51,10) = (/ &
     &1.8182e-03_rb,2.6209e-03_rb,3.7132e-03_rb,5.2128e-03_rb,7.2010e-03_rb/)
      kbo(:,52,10) = (/ &
     &1.3186e-03_rb,1.9376e-03_rb,2.7823e-03_rb,3.9639e-03_rb,5.5462e-03_rb/)
      kbo(:,53,10) = (/ &
     &9.4728e-04_rb,1.4218e-03_rb,2.0704e-03_rb,2.9919e-03_rb,4.2442e-03_rb/)
      kbo(:,54,10) = (/ &
     &6.9069e-04_rb,1.0561e-03_rb,1.5627e-03_rb,2.2955e-03_rb,3.3049e-03_rb/)
      kbo(:,55,10) = (/ &
     &5.0377e-04_rb,7.8848e-04_rb,1.1876e-03_rb,1.7754e-03_rb,2.5917e-03_rb/)
      kbo(:,56,10) = (/ &
     &3.6627e-04_rb,5.8682e-04_rb,8.9763e-04_rb,1.3649e-03_rb,2.0311e-03_rb/)
      kbo(:,57,10) = (/ &
     &2.6429e-04_rb,4.3296e-04_rb,6.7428e-04_rb,1.0407e-03_rb,1.5759e-03_rb/)
      kbo(:,58,10) = (/ &
     &1.9120e-04_rb,3.1987e-04_rb,5.0621e-04_rb,7.9436e-04_rb,1.2215e-03_rb/)
      kbo(:,59,10) = (/ &
     &1.5379e-04_rb,2.5891e-04_rb,4.1558e-04_rb,6.6123e-04_rb,1.0327e-03_rb/)
      kbo(:,13,11) = (/ &
     &3.3861e+00_rb,4.5598e+00_rb,5.9909e+00_rb,7.7387e+00_rb,9.7720e+00_rb/)
      kbo(:,14,11) = (/ &
     &3.0835e+00_rb,4.1638e+00_rb,5.4907e+00_rb,7.0868e+00_rb,8.8764e+00_rb/)
      kbo(:,15,11) = (/ &
     &2.7946e+00_rb,3.7805e+00_rb,4.9918e+00_rb,6.3780e+00_rb,7.9256e+00_rb/)
      kbo(:,16,11) = (/ &
     &2.5106e+00_rb,3.4021e+00_rb,4.4602e+00_rb,5.6582e+00_rb,7.0321e+00_rb/)
      kbo(:,17,11) = (/ &
     &2.2366e+00_rb,3.0111e+00_rb,3.9217e+00_rb,4.9755e+00_rb,6.2040e+00_rb/)
      kbo(:,18,11) = (/ &
     &1.9672e+00_rb,2.6349e+00_rb,3.4220e+00_rb,4.3497e+00_rb,5.4155e+00_rb/)
      kbo(:,19,11) = (/ &
     &1.7131e+00_rb,2.2896e+00_rb,2.9754e+00_rb,3.7775e+00_rb,4.6830e+00_rb/)
      kbo(:,20,11) = (/ &
     &1.4996e+00_rb,1.9951e+00_rb,2.5864e+00_rb,3.2729e+00_rb,4.0503e+00_rb/)
      kbo(:,21,11) = (/ &
     &1.3113e+00_rb,1.7407e+00_rb,2.2502e+00_rb,2.8371e+00_rb,3.4967e+00_rb/)
      kbo(:,22,11) = (/ &
     &1.1682e+00_rb,1.5452e+00_rb,1.9872e+00_rb,2.4911e+00_rb,3.0547e+00_rb/)
      kbo(:,23,11) = (/ &
     &1.0421e+00_rb,1.3707e+00_rb,1.7516e+00_rb,2.1846e+00_rb,2.6659e+00_rb/)
      kbo(:,24,11) = (/ &
     &9.2979e-01_rb,1.2143e+00_rb,1.5435e+00_rb,1.9148e+00_rb,2.3275e+00_rb/)
      kbo(:,25,11) = (/ &
     &8.2875e-01_rb,1.0753e+00_rb,1.3588e+00_rb,1.6784e+00_rb,2.0353e+00_rb/)
      kbo(:,26,11) = (/ &
     &7.3934e-01_rb,9.5318e-01_rb,1.1984e+00_rb,1.4754e+00_rb,1.7853e+00_rb/)
      kbo(:,27,11) = (/ &
     &6.5883e-01_rb,8.4429e-01_rb,1.0568e+00_rb,1.2987e+00_rb,1.5650e+00_rb/)
      kbo(:,28,11) = (/ &
     &5.8647e-01_rb,7.4845e-01_rb,9.3430e-01_rb,1.1424e+00_rb,1.3714e+00_rb/)
      kbo(:,29,11) = (/ &
     &5.2333e-01_rb,6.6481e-01_rb,8.2647e-01_rb,1.0068e+00_rb,1.2071e+00_rb/)
      kbo(:,30,11) = (/ &
     &4.6646e-01_rb,5.9039e-01_rb,7.3092e-01_rb,8.8855e-01_rb,1.0662e+00_rb/)
      kbo(:,31,11) = (/ &
     &4.1617e-01_rb,5.2426e-01_rb,6.4699e-01_rb,7.8730e-01_rb,9.4645e-01_rb/)
      kbo(:,32,11) = (/ &
     &3.7143e-01_rb,4.6600e-01_rb,5.7506e-01_rb,6.9984e-01_rb,8.4024e-01_rb/)
      kbo(:,33,11) = (/ &
     &3.3164e-01_rb,4.1539e-01_rb,5.1227e-01_rb,6.2282e-01_rb,7.4734e-01_rb/)
      kbo(:,34,11) = (/ &
     &2.9391e-01_rb,3.6839e-01_rb,4.5381e-01_rb,5.5129e-01_rb,6.6077e-01_rb/)
      kbo(:,35,11) = (/ &
     &2.5539e-01_rb,3.2000e-01_rb,3.9467e-01_rb,4.7981e-01_rb,5.7557e-01_rb/)
      kbo(:,36,11) = (/ &
     &2.1634e-01_rb,2.7190e-01_rb,3.3604e-01_rb,4.0953e-01_rb,4.9314e-01_rb/)
      kbo(:,37,11) = (/ &
     &1.7643e-01_rb,2.2324e-01_rb,2.7784e-01_rb,3.4019e-01_rb,4.0996e-01_rb/)
      kbo(:,38,11) = (/ &
     &1.4359e-01_rb,1.8284e-01_rb,2.2947e-01_rb,2.8247e-01_rb,3.3992e-01_rb/)
      kbo(:,39,11) = (/ &
     &1.1703e-01_rb,1.4999e-01_rb,1.8946e-01_rb,2.3346e-01_rb,2.8235e-01_rb/)
      kbo(:,40,11) = (/ &
     &9.2508e-02_rb,1.1962e-01_rb,1.5225e-01_rb,1.8852e-01_rb,2.3037e-01_rb/)
      kbo(:,41,11) = (/ &
     &7.2812e-02_rb,9.4920e-02_rb,1.2149e-01_rb,1.5155e-01_rb,1.8718e-01_rb/)
      kbo(:,42,11) = (/ &
     &5.7212e-02_rb,7.5261e-02_rb,9.6717e-02_rb,1.2163e-01_rb,1.5109e-01_rb/)
      kbo(:,43,11) = (/ &
     &4.4118e-02_rb,5.8641e-02_rb,7.5812e-02_rb,9.6186e-02_rb,1.2035e-01_rb/)
      kbo(:,44,11) = (/ &
     &3.3705e-02_rb,4.5203e-02_rb,5.8991e-02_rb,7.5535e-02_rb,9.5312e-02_rb/)
      kbo(:,45,11) = (/ &
     &2.5795e-02_rb,3.4826e-02_rb,4.5764e-02_rb,5.9102e-02_rb,7.5973e-02_rb/)
      kbo(:,46,11) = (/ &
     &1.9500e-02_rb,2.6666e-02_rb,3.5264e-02_rb,4.5887e-02_rb,6.0050e-02_rb/)
      kbo(:,47,11) = (/ &
     &1.4478e-02_rb,2.0069e-02_rb,2.6758e-02_rb,3.5119e-02_rb,4.6750e-02_rb/)
      kbo(:,48,11) = (/ &
     &1.0673e-02_rb,1.5022e-02_rb,2.0274e-02_rb,2.6837e-02_rb,3.6213e-02_rb/)
      kbo(:,49,11) = (/ &
     &7.8044e-03_rb,1.1183e-02_rb,1.5301e-02_rb,2.0457e-02_rb,2.8037e-02_rb/)
      kbo(:,50,11) = (/ &
     &5.7286e-03_rb,8.3541e-03_rb,1.1590e-02_rb,1.5692e-02_rb,2.1700e-02_rb/)
      kbo(:,51,11) = (/ &
     &4.2038e-03_rb,6.2320e-03_rb,8.7867e-03_rb,1.2044e-02_rb,1.6737e-02_rb/)
      kbo(:,52,11) = (/ &
     &3.0623e-03_rb,4.6247e-03_rb,6.6271e-03_rb,9.2102e-03_rb,1.2933e-02_rb/)
      kbo(:,53,11) = (/ &
     &2.2163e-03_rb,3.3976e-03_rb,4.9642e-03_rb,7.0028e-03_rb,9.9664e-03_rb/)
      kbo(:,54,11) = (/ &
     &1.6196e-03_rb,2.5242e-03_rb,3.7641e-03_rb,5.3709e-03_rb,7.7537e-03_rb/)
      kbo(:,55,11) = (/ &
     &1.1913e-03_rb,1.8900e-03_rb,2.8670e-03_rb,4.1465e-03_rb,6.0719e-03_rb/)
      kbo(:,56,11) = (/ &
     &8.6854e-04_rb,1.4072e-03_rb,2.1797e-03_rb,3.2039e-03_rb,4.7482e-03_rb/)
      kbo(:,57,11) = (/ &
     &6.2940e-04_rb,1.0423e-03_rb,1.6521e-03_rb,2.4738e-03_rb,3.7243e-03_rb/)
      kbo(:,58,11) = (/ &
     &4.5766e-04_rb,7.7541e-04_rb,1.2578e-03_rb,1.9222e-03_rb,2.9509e-03_rb/)
      kbo(:,59,11) = (/ &
     &3.7158e-04_rb,6.4512e-04_rb,1.0666e-03_rb,1.6673e-03_rb,2.6085e-03_rb/)
      kbo(:,13,12) = (/ &
     &6.8265e+00_rb,9.1662e+00_rb,1.2005e+01_rb,1.5291e+01_rb,1.9104e+01_rb/)
      kbo(:,14,12) = (/ &
     &6.4505e+00_rb,8.6663e+00_rb,1.1302e+01_rb,1.4372e+01_rb,1.8022e+01_rb/)
      kbo(:,15,12) = (/ &
     &6.0333e+00_rb,8.0721e+00_rb,1.0531e+01_rb,1.3476e+01_rb,1.6920e+01_rb/)
      kbo(:,16,12) = (/ &
     &5.5683e+00_rb,7.4562e+00_rb,9.7508e+00_rb,1.2490e+01_rb,1.5606e+01_rb/)
      kbo(:,17,12) = (/ &
     &5.0713e+00_rb,6.8025e+00_rb,8.9065e+00_rb,1.1361e+01_rb,1.4093e+01_rb/)
      kbo(:,18,12) = (/ &
     &4.5774e+00_rb,6.1450e+00_rb,8.0133e+00_rb,1.0144e+01_rb,1.2548e+01_rb/)
      kbo(:,19,12) = (/ &
     &4.0864e+00_rb,5.4694e+00_rb,7.0931e+00_rb,8.9659e+00_rb,1.1117e+01_rb/)
      kbo(:,20,12) = (/ &
     &3.6294e+00_rb,4.8248e+00_rb,6.2421e+00_rb,7.9048e+00_rb,9.7882e+00_rb/)
      kbo(:,21,12) = (/ &
     &3.1876e+00_rb,4.2266e+00_rb,5.4689e+00_rb,6.9148e+00_rb,8.5415e+00_rb/)
      kbo(:,22,12) = (/ &
     &2.8446e+00_rb,3.7580e+00_rb,4.8472e+00_rb,6.1036e+00_rb,7.5044e+00_rb/)
      kbo(:,23,12) = (/ &
     &2.5375e+00_rb,3.3389e+00_rb,4.2884e+00_rb,5.3709e+00_rb,6.5794e+00_rb/)
      kbo(:,24,12) = (/ &
     &2.2687e+00_rb,2.9742e+00_rb,3.7979e+00_rb,4.7327e+00_rb,5.7691e+00_rb/)
      kbo(:,25,12) = (/ &
     &2.0354e+00_rb,2.6539e+00_rb,3.3704e+00_rb,4.1780e+00_rb,5.0680e+00_rb/)
      kbo(:,26,12) = (/ &
     &1.8347e+00_rb,2.3795e+00_rb,3.0035e+00_rb,3.7009e+00_rb,4.4678e+00_rb/)
      kbo(:,27,12) = (/ &
     &1.6539e+00_rb,2.1288e+00_rb,2.6709e+00_rb,3.2754e+00_rb,3.9447e+00_rb/)
      kbo(:,28,12) = (/ &
     &1.4848e+00_rb,1.8995e+00_rb,2.3700e+00_rb,2.8962e+00_rb,3.4805e+00_rb/)
      kbo(:,29,12) = (/ &
     &1.3323e+00_rb,1.6945e+00_rb,2.1064e+00_rb,2.5664e+00_rb,3.0755e+00_rb/)
      kbo(:,30,12) = (/ &
     &1.1951e+00_rb,1.5124e+00_rb,1.8729e+00_rb,2.2771e+00_rb,2.7207e+00_rb/)
      kbo(:,31,12) = (/ &
     &1.0723e+00_rb,1.3520e+00_rb,1.6707e+00_rb,2.0245e+00_rb,2.4134e+00_rb/)
      kbo(:,32,12) = (/ &
     &9.6436e-01_rb,1.2123e+00_rb,1.4924e+00_rb,1.8027e+00_rb,2.1488e+00_rb/)
      kbo(:,33,12) = (/ &
     &8.6875e-01_rb,1.0883e+00_rb,1.3350e+00_rb,1.6111e+00_rb,1.9241e+00_rb/)
      kbo(:,34,12) = (/ &
     &7.7744e-01_rb,9.7074e-01_rb,1.1897e+00_rb,1.4382e+00_rb,1.7216e+00_rb/)
      kbo(:,35,12) = (/ &
     &6.8108e-01_rb,8.5081e-01_rb,1.0447e+00_rb,1.2673e+00_rb,1.5231e+00_rb/)
      kbo(:,36,12) = (/ &
     &5.8251e-01_rb,7.3036e-01_rb,9.0117e-01_rb,1.0985e+00_rb,1.3262e+00_rb/)
      kbo(:,37,12) = (/ &
     &4.8147e-01_rb,6.0751e-01_rb,7.5487e-01_rb,9.2714e-01_rb,1.1257e+00_rb/)
      kbo(:,38,12) = (/ &
     &3.9749e-01_rb,5.0467e-01_rb,6.3154e-01_rb,7.8124e-01_rb,9.5438e-01_rb/)
      kbo(:,39,12) = (/ &
     &3.2829e-01_rb,4.1995e-01_rb,5.2901e-01_rb,6.5861e-01_rb,8.0986e-01_rb/)
      kbo(:,40,12) = (/ &
     &2.6399e-01_rb,3.4098e-01_rb,4.3333e-01_rb,5.4338e-01_rb,6.7333e-01_rb/)
      kbo(:,41,12) = (/ &
     &2.1121e-01_rb,2.7554e-01_rb,3.5340e-01_rb,4.4699e-01_rb,5.5819e-01_rb/)
      kbo(:,42,12) = (/ &
     &1.6875e-01_rb,2.2242e-01_rb,2.8790e-01_rb,3.6718e-01_rb,4.6248e-01_rb/)
      kbo(:,43,12) = (/ &
     &1.3242e-01_rb,1.7649e-01_rb,2.3091e-01_rb,2.9729e-01_rb,3.7802e-01_rb/)
      kbo(:,44,12) = (/ &
     &1.0268e-01_rb,1.3874e-01_rb,1.8350e-01_rb,2.3873e-01_rb,3.0615e-01_rb/)
      kbo(:,45,12) = (/ &
     &7.9157e-02_rb,1.0857e-01_rb,1.4543e-01_rb,1.9132e-01_rb,2.4703e-01_rb/)
      kbo(:,46,12) = (/ &
     &6.0349e-02_rb,8.3945e-02_rb,1.1412e-01_rb,1.5200e-01_rb,1.9783e-01_rb/)
      kbo(:,47,12) = (/ &
     &4.5026e-02_rb,6.3638e-02_rb,8.7982e-02_rb,1.1888e-01_rb,1.5630e-01_rb/)
      kbo(:,48,12) = (/ &
     &3.3400e-02_rb,4.7982e-02_rb,6.7403e-02_rb,9.2494e-02_rb,1.2316e-01_rb/)
      kbo(:,49,12) = (/ &
     &2.4633e-02_rb,3.5960e-02_rb,5.1349e-02_rb,7.1624e-02_rb,9.6517e-02_rb/)
      kbo(:,50,12) = (/ &
     &1.8199e-02_rb,2.7006e-02_rb,3.9181e-02_rb,5.5479e-02_rb,7.5913e-02_rb/)
      kbo(:,51,12) = (/ &
     &1.3397e-02_rb,2.0251e-02_rb,2.9827e-02_rb,4.2894e-02_rb,5.9697e-02_rb/)
      kbo(:,52,12) = (/ &
     &9.8047e-03_rb,1.5092e-02_rb,2.2613e-02_rb,3.3016e-02_rb,4.6660e-02_rb/)
      kbo(:,53,12) = (/ &
     &7.1233e-03_rb,1.1184e-02_rb,1.7048e-02_rb,2.5317e-02_rb,3.6316e-02_rb/)
      kbo(:,54,12) = (/ &
     &5.1904e-03_rb,8.3108e-03_rb,1.2872e-02_rb,1.9462e-02_rb,2.8342e-02_rb/)
      kbo(:,55,12) = (/ &
     &3.7706e-03_rb,6.1523e-03_rb,9.6915e-03_rb,1.4909e-02_rb,2.2083e-02_rb/)
      kbo(:,56,12) = (/ &
     &2.7214e-03_rb,4.5313e-03_rb,7.2581e-03_rb,1.1344e-02_rb,1.7112e-02_rb/)
      kbo(:,57,12) = (/ &
     &1.9679e-03_rb,3.3305e-03_rb,5.4024e-03_rb,8.5896e-03_rb,1.3181e-02_rb/)
      kbo(:,58,12) = (/ &
     &1.4312e-03_rb,2.4830e-03_rb,4.0992e-03_rb,6.5707e-03_rb,1.0148e-02_rb/)
      kbo(:,59,12) = (/ &
     &1.1735e-03_rb,2.0715e-03_rb,3.4672e-03_rb,5.6187e-03_rb,8.6747e-03_rb/)
      kbo(:,13,13) = (/ &
     &1.4965e+01_rb,1.9939e+01_rb,2.5957e+01_rb,3.3111e+01_rb,4.1486e+01_rb/)
      kbo(:,14,13) = (/ &
     &1.4891e+01_rb,1.9752e+01_rb,2.5714e+01_rb,3.2877e+01_rb,4.1201e+01_rb/)
      kbo(:,15,13) = (/ &
     &1.4632e+01_rb,1.9464e+01_rb,2.5349e+01_rb,3.2327e+01_rb,4.0239e+01_rb/)
      kbo(:,16,13) = (/ &
     &1.4224e+01_rb,1.8920e+01_rb,2.4612e+01_rb,3.1240e+01_rb,3.8764e+01_rb/)
      kbo(:,17,13) = (/ &
     &1.3596e+01_rb,1.8071e+01_rb,2.3399e+01_rb,2.9603e+01_rb,3.6827e+01_rb/)
      kbo(:,18,13) = (/ &
     &1.2765e+01_rb,1.6956e+01_rb,2.1940e+01_rb,2.7801e+01_rb,3.4621e+01_rb/)
      kbo(:,19,13) = (/ &
     &1.1845e+01_rb,1.5708e+01_rb,2.0355e+01_rb,2.5828e+01_rb,3.2043e+01_rb/)
      kbo(:,20,13) = (/ &
     &1.0881e+01_rb,1.4480e+01_rb,1.8761e+01_rb,2.3735e+01_rb,2.9355e+01_rb/)
      kbo(:,21,13) = (/ &
     &9.9388e+00_rb,1.3181e+01_rb,1.7060e+01_rb,2.1504e+01_rb,2.6520e+01_rb/)
      kbo(:,22,13) = (/ &
     &9.1395e+00_rb,1.2102e+01_rb,1.5540e+01_rb,1.9513e+01_rb,2.4061e+01_rb/)
      kbo(:,23,13) = (/ &
     &8.3446e+00_rb,1.0972e+01_rb,1.4055e+01_rb,1.7623e+01_rb,2.1727e+01_rb/)
      kbo(:,24,13) = (/ &
     &7.5591e+00_rb,9.8924e+00_rb,1.2651e+01_rb,1.5869e+01_rb,1.9520e+01_rb/)
      kbo(:,25,13) = (/ &
     &6.8143e+00_rb,8.8970e+00_rb,1.1359e+01_rb,1.4218e+01_rb,1.7424e+01_rb/)
      kbo(:,26,13) = (/ &
     &6.1698e+00_rb,8.0210e+00_rb,1.0220e+01_rb,1.2729e+01_rb,1.5532e+01_rb/)
      kbo(:,27,13) = (/ &
     &5.5728e+00_rb,7.2306e+00_rb,9.1639e+00_rb,1.1369e+01_rb,1.3833e+01_rb/)
      kbo(:,28,13) = (/ &
     &5.0450e+00_rb,6.5146e+00_rb,8.2221e+00_rb,1.0165e+01_rb,1.2320e+01_rb/)
      kbo(:,29,13) = (/ &
     &4.5820e+00_rb,5.8856e+00_rb,7.3962e+00_rb,9.1032e+00_rb,1.0999e+01_rb/)
      kbo(:,30,13) = (/ &
     &4.1619e+00_rb,5.3258e+00_rb,6.6581e+00_rb,8.1640e+00_rb,9.8348e+00_rb/)
      kbo(:,31,13) = (/ &
     &3.7986e+00_rb,4.8336e+00_rb,6.0169e+00_rb,7.3498e+00_rb,8.8372e+00_rb/)
      kbo(:,32,13) = (/ &
     &3.4747e+00_rb,4.3988e+00_rb,5.4556e+00_rb,6.6497e+00_rb,7.9860e+00_rb/)
      kbo(:,33,13) = (/ &
     &3.1806e+00_rb,4.0134e+00_rb,4.9644e+00_rb,6.0424e+00_rb,7.2445e+00_rb/)
      kbo(:,34,13) = (/ &
     &2.8901e+00_rb,3.6383e+00_rb,4.4979e+00_rb,5.4687e+00_rb,6.5591e+00_rb/)
      kbo(:,35,13) = (/ &
     &2.5701e+00_rb,3.2361e+00_rb,4.0067e+00_rb,4.8794e+00_rb,5.8576e+00_rb/)
      kbo(:,36,13) = (/ &
     &2.2323e+00_rb,2.8199e+00_rb,3.4998e+00_rb,4.2791e+00_rb,5.1575e+00_rb/)
      kbo(:,37,13) = (/ &
     &1.8736e+00_rb,2.3827e+00_rb,2.9738e+00_rb,3.6564e+00_rb,4.4360e+00_rb/)
      kbo(:,38,13) = (/ &
     &1.5728e+00_rb,2.0148e+00_rb,2.5311e+00_rb,3.1303e+00_rb,3.8226e+00_rb/)
      kbo(:,39,13) = (/ &
     &1.3237e+00_rb,1.7072e+00_rb,2.1603e+00_rb,2.6914e+00_rb,3.3072e+00_rb/)
      kbo(:,40,13) = (/ &
     &1.0847e+00_rb,1.4132e+00_rb,1.8059e+00_rb,2.2702e+00_rb,2.8143e+00_rb/)
      kbo(:,41,13) = (/ &
     &8.8581e-01_rb,1.1668e+00_rb,1.5064e+00_rb,1.9133e+00_rb,2.3935e+00_rb/)
      kbo(:,42,13) = (/ &
     &7.2374e-01_rb,9.6367e-01_rb,1.2577e+00_rb,1.6144e+00_rb,2.0403e+00_rb/)
      kbo(:,43,13) = (/ &
     &5.8123e-01_rb,7.8367e-01_rb,1.0353e+00_rb,1.3440e+00_rb,1.7177e+00_rb/)
      kbo(:,44,13) = (/ &
     &4.6198e-01_rb,6.3195e-01_rb,8.4539e-01_rb,1.1117e+00_rb,1.4376e+00_rb/)
      kbo(:,45,13) = (/ &
     &3.6584e-01_rb,5.0822e-01_rb,6.8932e-01_rb,9.1864e-01_rb,1.2028e+00_rb/)
      kbo(:,46,13) = (/ &
     &2.8644e-01_rb,4.0442e-01_rb,5.5738e-01_rb,7.5306e-01_rb,9.9957e-01_rb/)
      kbo(:,47,13) = (/ &
     &2.1956e-01_rb,3.1585e-01_rb,4.4303e-01_rb,6.0814e-01_rb,8.1897e-01_rb/)
      kbo(:,48,13) = (/ &
     &1.6674e-01_rb,2.4495e-01_rb,3.5019e-01_rb,4.8911e-01_rb,6.6858e-01_rb/)
      kbo(:,49,13) = (/ &
     &1.2553e-01_rb,1.8856e-01_rb,2.7515e-01_rb,3.9103e-01_rb,5.4421e-01_rb/)
      kbo(:,50,13) = (/ &
     &9.4665e-02_rb,1.4544e-01_rb,2.1677e-01_rb,3.1374e-01_rb,4.4454e-01_rb/)
      kbo(:,51,13) = (/ &
     &7.1191e-02_rb,1.1194e-01_rb,1.7036e-01_rb,2.5168e-01_rb,3.6277e-01_rb/)
      kbo(:,52,13) = (/ &
     &5.3052e-02_rb,8.5501e-02_rb,1.3304e-01_rb,2.0087e-01_rb,2.9517e-01_rb/)
      kbo(:,53,13) = (/ &
     &3.9132e-02_rb,6.4715e-02_rb,1.0314e-01_rb,1.5924e-01_rb,2.3881e-01_rb/)
      kbo(:,54,13) = (/ &
     &2.9043e-02_rb,4.9273e-02_rb,8.0552e-02_rb,1.2707e-01_rb,1.9473e-01_rb/)
      kbo(:,55,13) = (/ &
     &2.1525e-02_rb,3.7498e-02_rb,6.2875e-02_rb,1.0150e-01_rb,1.5889e-01_rb/)
      kbo(:,56,13) = (/ &
     &1.5833e-02_rb,2.8322e-02_rb,4.8765e-02_rb,8.0651e-02_rb,1.2905e-01_rb/)
      kbo(:,57,13) = (/ &
     &1.1526e-02_rb,2.1205e-02_rb,3.7510e-02_rb,6.3651e-02_rb,1.0431e-01_rb/)
      kbo(:,58,13) = (/ &
     &8.3779e-03_rb,1.5855e-02_rb,2.8776e-02_rb,5.0260e-02_rb,8.4604e-02_rb/)
      kbo(:,59,13) = (/ &
     &6.7851e-03_rb,1.3119e-02_rb,2.4293e-02_rb,4.3321e-02_rb,7.4531e-02_rb/)
      kbo(:,13,14) = (/ &
     &3.3721e+01_rb,4.7002e+01_rb,6.3302e+01_rb,8.2327e+01_rb,1.0363e+02_rb/)
      kbo(:,14,14) = (/ &
     &3.6911e+01_rb,5.0874e+01_rb,6.7511e+01_rb,8.6774e+01_rb,1.0872e+02_rb/)
      kbo(:,15,14) = (/ &
     &3.9962e+01_rb,5.4067e+01_rb,7.0780e+01_rb,9.0289e+01_rb,1.1312e+02_rb/)
      kbo(:,16,14) = (/ &
     &4.2338e+01_rb,5.6683e+01_rb,7.3332e+01_rb,9.3095e+01_rb,1.1626e+02_rb/)
      kbo(:,17,14) = (/ &
     &4.3916e+01_rb,5.8217e+01_rb,7.5023e+01_rb,9.4925e+01_rb,1.1793e+02_rb/)
      kbo(:,18,14) = (/ &
     &4.4576e+01_rb,5.8845e+01_rb,7.5702e+01_rb,9.5514e+01_rb,1.1830e+02_rb/)
      kbo(:,19,14) = (/ &
     &4.4328e+01_rb,5.8466e+01_rb,7.5196e+01_rb,9.4726e+01_rb,1.1748e+02_rb/)
      kbo(:,20,14) = (/ &
     &4.3798e+01_rb,5.7631e+01_rb,7.4169e+01_rb,9.3518e+01_rb,1.1584e+02_rb/)
      kbo(:,21,14) = (/ &
     &4.2847e+01_rb,5.6429e+01_rb,7.2627e+01_rb,9.1567e+01_rb,1.1316e+02_rb/)
      kbo(:,22,14) = (/ &
     &4.2232e+01_rb,5.5705e+01_rb,7.1694e+01_rb,9.0118e+01_rb,1.1101e+02_rb/)
      kbo(:,23,14) = (/ &
     &4.1301e+01_rb,5.4371e+01_rb,6.9797e+01_rb,8.7676e+01_rb,1.0772e+02_rb/)
      kbo(:,24,14) = (/ &
     &4.0036e+01_rb,5.2522e+01_rb,6.7301e+01_rb,8.4384e+01_rb,1.0387e+02_rb/)
      kbo(:,25,14) = (/ &
     &3.8535e+01_rb,5.0451e+01_rb,6.4572e+01_rb,8.0891e+01_rb,9.9814e+01_rb/)
      kbo(:,26,14) = (/ &
     &3.6990e+01_rb,4.8476e+01_rb,6.1896e+01_rb,7.7604e+01_rb,9.5632e+01_rb/)
      kbo(:,27,14) = (/ &
     &3.5439e+01_rb,4.6349e+01_rb,5.9201e+01_rb,7.4166e+01_rb,9.1237e+01_rb/)
      kbo(:,28,14) = (/ &
     &3.3799e+01_rb,4.4185e+01_rb,5.6512e+01_rb,7.0702e+01_rb,8.6988e+01_rb/)
      kbo(:,29,14) = (/ &
     &3.2225e+01_rb,4.2139e+01_rb,5.3908e+01_rb,6.7503e+01_rb,8.3109e+01_rb/)
      kbo(:,30,14) = (/ &
     &3.0748e+01_rb,4.0218e+01_rb,5.1459e+01_rb,6.4621e+01_rb,7.9612e+01_rb/)
      kbo(:,31,14) = (/ &
     &2.9425e+01_rb,3.8499e+01_rb,4.9352e+01_rb,6.2118e+01_rb,7.6745e+01_rb/)
      kbo(:,32,14) = (/ &
     &2.8242e+01_rb,3.7039e+01_rb,4.7648e+01_rb,6.0092e+01_rb,7.4536e+01_rb/)
      kbo(:,33,14) = (/ &
     &2.7263e+01_rb,3.5883e+01_rb,4.6290e+01_rb,5.8616e+01_rb,7.3025e+01_rb/)
      kbo(:,34,14) = (/ &
     &2.6245e+01_rb,3.4728e+01_rb,4.4963e+01_rb,5.7211e+01_rb,7.1614e+01_rb/)
      kbo(:,35,14) = (/ &
     &2.4792e+01_rb,3.3010e+01_rb,4.3016e+01_rb,5.5082e+01_rb,6.9332e+01_rb/)
      kbo(:,36,14) = (/ &
     &2.2854e+01_rb,3.0681e+01_rb,4.0335e+01_rb,5.2076e+01_rb,6.5971e+01_rb/)
      kbo(:,37,14) = (/ &
     &2.0215e+01_rb,2.7477e+01_rb,3.6561e+01_rb,4.7698e+01_rb,6.0990e+01_rb/)
      kbo(:,38,14) = (/ &
     &1.7901e+01_rb,2.4657e+01_rb,3.3200e+01_rb,4.3777e+01_rb,5.6518e+01_rb/)
      kbo(:,39,14) = (/ &
     &1.5916e+01_rb,2.2229e+01_rb,3.0285e+01_rb,4.0352e+01_rb,5.2594e+01_rb/)
      kbo(:,40,14) = (/ &
     &1.3687e+01_rb,1.9453e+01_rb,2.6865e+01_rb,3.6273e+01_rb,4.7801e+01_rb/)
      kbo(:,41,14) = (/ &
     &1.1706e+01_rb,1.6957e+01_rb,2.3775e+01_rb,3.2529e+01_rb,4.3345e+01_rb/)
      kbo(:,42,14) = (/ &
     &1.0015e+01_rb,1.4777e+01_rb,2.1056e+01_rb,2.9181e+01_rb,3.9336e+01_rb/)
      kbo(:,43,14) = (/ &
     &8.3602e+00_rb,1.2595e+01_rb,1.8289e+01_rb,2.5700e+01_rb,3.5139e+01_rb/)
      kbo(:,44,14) = (/ &
     &6.8842e+00_rb,1.0588e+01_rb,1.5702e+01_rb,2.2421e+01_rb,3.1090e+01_rb/)
      kbo(:,45,14) = (/ &
     &5.6505e+00_rb,8.8697e+00_rb,1.3428e+01_rb,1.9525e+01_rb,2.7445e+01_rb/)
      kbo(:,46,14) = (/ &
     &4.5669e+00_rb,7.3282e+00_rb,1.1331e+01_rb,1.6810e+01_rb,2.3987e+01_rb/)
      kbo(:,47,14) = (/ &
     &3.5910e+00_rb,5.9011e+00_rb,9.3338e+00_rb,1.4162e+01_rb,2.0590e+01_rb/)
      kbo(:,48,14) = (/ &
     &2.8046e+00_rb,4.7156e+00_rb,7.6334e+00_rb,1.1848e+01_rb,1.7577e+01_rb/)
      kbo(:,49,14) = (/ &
     &2.1748e+00_rb,3.7383e+00_rb,6.1955e+00_rb,9.8356e+00_rb,1.4909e+01_rb/)
      kbo(:,50,14) = (/ &
     &1.6968e+00_rb,2.9824e+00_rb,5.0552e+00_rb,8.1970e+00_rb,1.2692e+01_rb/)
      kbo(:,51,14) = (/ &
     &1.3228e+00_rb,2.3793e+00_rb,4.1196e+00_rb,6.8244e+00_rb,1.0784e+01_rb/)
      kbo(:,52,14) = (/ &
     &1.0245e+00_rb,1.8865e+00_rb,3.3355e+00_rb,5.6478e+00_rb,9.1028e+00_rb/)
      kbo(:,53,14) = (/ &
     &7.8728e-01_rb,1.4840e+00_rb,2.6816e+00_rb,4.6391e+00_rb,7.6341e+00_rb/)
      kbo(:,54,14) = (/ &
     &6.1018e-01_rb,1.1815e+00_rb,2.1812e+00_rb,3.8474e+00_rb,6.4584e+00_rb/)
      kbo(:,55,14) = (/ &
     &4.7246e-01_rb,9.4354e-01_rb,1.7799e+00_rb,3.1974e+00_rb,5.4714e+00_rb/)
      kbo(:,56,14) = (/ &
     &3.6268e-01_rb,7.4897e-01_rb,1.4435e+00_rb,2.6438e+00_rb,4.6110e+00_rb/)
      kbo(:,57,14) = (/ &
     &2.7617e-01_rb,5.8818e-01_rb,1.1638e+00_rb,2.1763e+00_rb,3.8632e+00_rb/)
      kbo(:,58,14) = (/ &
     &2.1026e-01_rb,4.6281e-01_rb,9.4334e-01_rb,1.8002e+00_rb,3.2506e+00_rb/)
      kbo(:,59,14) = (/ &
     &1.8510e-01_rb,4.1578e-01_rb,8.6198e-01_rb,1.6609e+00_rb,3.0231e+00_rb/)
      kbo(:,13,15) = (/ &
     &7.9497e+01_rb,1.0631e+02_rb,1.3951e+02_rb,1.8076e+02_rb,2.3534e+02_rb/)
      kbo(:,14,15) = (/ &
     &9.0008e+01_rb,1.2102e+02_rb,1.6070e+02_rb,2.0844e+02_rb,2.6891e+02_rb/)
      kbo(:,15,15) = (/ &
     &1.0142e+02_rb,1.3773e+02_rb,1.8424e+02_rb,2.3942e+02_rb,3.0596e+02_rb/)
      kbo(:,16,15) = (/ &
     &1.1444e+02_rb,1.5595e+02_rb,2.0951e+02_rb,2.7261e+02_rb,3.4599e+02_rb/)
      kbo(:,17,15) = (/ &
     &1.2871e+02_rb,1.7599e+02_rb,2.3611e+02_rb,3.0737e+02_rb,3.8849e+02_rb/)
      kbo(:,18,15) = (/ &
     &1.4444e+02_rb,1.9729e+02_rb,2.6334e+02_rb,3.4204e+02_rb,4.3103e+02_rb/)
      kbo(:,19,15) = (/ &
     &1.6121e+02_rb,2.1913e+02_rb,2.9095e+02_rb,3.7632e+02_rb,4.7170e+02_rb/)
      kbo(:,20,15) = (/ &
     &1.7880e+02_rb,2.4196e+02_rb,3.1956e+02_rb,4.1027e+02_rb,5.1312e+02_rb/)
      kbo(:,21,15) = (/ &
     &1.9582e+02_rb,2.6409e+02_rb,3.4675e+02_rb,4.4276e+02_rb,5.5264e+02_rb/)
      kbo(:,22,15) = (/ &
     &2.1631e+02_rb,2.9018e+02_rb,3.7825e+02_rb,4.8067e+02_rb,5.9651e+02_rb/)
      kbo(:,23,15) = (/ &
     &2.3693e+02_rb,3.1638e+02_rb,4.0986e+02_rb,5.1726e+02_rb,6.3877e+02_rb/)
      kbo(:,24,15) = (/ &
     &2.5815e+02_rb,3.4252e+02_rb,4.4099e+02_rb,5.5343e+02_rb,6.7985e+02_rb/)
      kbo(:,25,15) = (/ &
     &2.7940e+02_rb,3.6830e+02_rb,4.7141e+02_rb,5.8880e+02_rb,7.2036e+02_rb/)
      kbo(:,26,15) = (/ &
     &3.0147e+02_rb,3.9459e+02_rb,5.0246e+02_rb,6.2482e+02_rb,7.6276e+02_rb/)
      kbo(:,27,15) = (/ &
     &3.2318e+02_rb,4.2065e+02_rb,5.3313e+02_rb,6.6087e+02_rb,8.0529e+02_rb/)
      kbo(:,28,15) = (/ &
     &3.4508e+02_rb,4.4657e+02_rb,5.6379e+02_rb,6.9725e+02_rb,8.4792e+02_rb/)
      kbo(:,29,15) = (/ &
     &3.6777e+02_rb,4.7381e+02_rb,5.9626e+02_rb,7.3554e+02_rb,8.9251e+02_rb/)
      kbo(:,30,15) = (/ &
     &3.9103e+02_rb,5.0185e+02_rb,6.2997e+02_rb,7.7485e+02_rb,9.3819e+02_rb/)
      kbo(:,31,15) = (/ &
     &4.1560e+02_rb,5.3166e+02_rb,6.6523e+02_rb,8.1615e+02_rb,9.8559e+02_rb/)
      kbo(:,32,15) = (/ &
     &4.4194e+02_rb,5.6341e+02_rb,7.0244e+02_rb,8.5957e+02_rb,1.0345e+03_rb/)
      kbo(:,33,15) = (/ &
     &4.7004e+02_rb,5.9690e+02_rb,7.4161e+02_rb,9.0465e+02_rb,1.0849e+03_rb/)
      kbo(:,34,15) = (/ &
     &4.9560e+02_rb,6.2720e+02_rb,7.7702e+02_rb,9.4526e+02_rb,1.1301e+03_rb/)
      kbo(:,35,15) = (/ &
     &5.1020e+02_rb,6.4498e+02_rb,7.9798e+02_rb,9.6921e+02_rb,1.1571e+03_rb/)
      kbo(:,36,15) = (/ &
     &5.1153e+02_rb,6.4732e+02_rb,8.0146e+02_rb,9.7374e+02_rb,1.1629e+03_rb/)
      kbo(:,37,15) = (/ &
     &4.9411e+02_rb,6.2784e+02_rb,7.8019e+02_rb,9.5084e+02_rb,1.1384e+03_rb/)
      kbo(:,38,15) = (/ &
     &4.7607e+02_rb,6.0738e+02_rb,7.5785e+02_rb,9.2634e+02_rb,1.1122e+03_rb/)
      kbo(:,39,15) = (/ &
     &4.5860e+02_rb,5.8729e+02_rb,7.3571e+02_rb,9.0212e+02_rb,1.0835e+03_rb/)
      kbo(:,40,15) = (/ &
     &4.2990e+02_rb,5.5385e+02_rb,6.9803e+02_rb,8.6039e+02_rb,1.0406e+03_rb/)
      kbo(:,41,15) = (/ &
     &4.0087e+02_rb,5.1992e+02_rb,6.5921e+02_rb,8.1732e+02_rb,9.9347e+02_rb/)
      kbo(:,42,15) = (/ &
     &3.7298e+02_rb,4.8724e+02_rb,6.2146e+02_rb,7.7537e+02_rb,9.4699e+02_rb/)
      kbo(:,43,15) = (/ &
     &3.4053e+02_rb,4.4909e+02_rb,5.7712e+02_rb,7.2520e+02_rb,8.9154e+02_rb/)
      kbo(:,44,15) = (/ &
     &3.0753e+02_rb,4.1020e+02_rb,5.3144e+02_rb,6.7302e+02_rb,8.3358e+02_rb/)
      kbo(:,45,15) = (/ &
     &2.7654e+02_rb,3.7323e+02_rb,4.8808e+02_rb,6.2285e+02_rb,7.7741e+02_rb/)
      kbo(:,46,15) = (/ &
     &2.4531e+02_rb,3.3622e+02_rb,4.4437e+02_rb,5.7189e+02_rb,7.1969e+02_rb/)
      kbo(:,47,15) = (/ &
     &2.1302e+02_rb,2.9761e+02_rb,3.9853e+02_rb,5.1809e+02_rb,6.5783e+02_rb/)
      kbo(:,48,15) = (/ &
     &1.8355e+02_rb,2.6147e+02_rb,3.5562e+02_rb,4.6754e+02_rb,5.9909e+02_rb/)
      kbo(:,49,15) = (/ &
     &1.5701e+02_rb,2.2791e+02_rb,3.1569e+02_rb,4.2021e+02_rb,5.4365e+02_rb/)
      kbo(:,50,15) = (/ &
     &1.3468e+02_rb,1.9893e+02_rb,2.8071e+02_rb,3.7858e+02_rb,4.9472e+02_rb/)
      kbo(:,51,15) = (/ &
     &1.1526e+02_rb,1.7331e+02_rb,2.4880e+02_rb,3.4068e+02_rb,4.5000e+02_rb/)
      kbo(:,52,15) = (/ &
     &9.7991e+01_rb,1.5012e+02_rb,2.1915e+02_rb,3.0532e+02_rb,4.0798e+02_rb/)
      kbo(:,53,15) = (/ &
     &8.2622e+01_rb,1.2914e+02_rb,1.9179e+02_rb,2.7194e+02_rb,3.6825e+02_rb/)
      kbo(:,54,15) = (/ &
     &7.0111e+01_rb,1.1182e+02_rb,1.6882e+02_rb,2.4316e+02_rb,3.3408e+02_rb/)
      kbo(:,55,15) = (/ &
     &5.9451e+01_rb,9.6898e+01_rb,1.4869e+02_rb,2.1737e+02_rb,3.0323e+02_rb/)
      kbo(:,56,15) = (/ &
     &4.9982e+01_rb,8.3446e+01_rb,1.3033e+02_rb,1.9338e+02_rb,2.7397e+02_rb/)
      kbo(:,57,15) = (/ &
     &4.1621e+01_rb,7.1384e+01_rb,1.1363e+02_rb,1.7127e+02_rb,2.4633e+02_rb/)
      kbo(:,58,15) = (/ &
     &3.4650e+01_rb,6.1162e+01_rb,9.9313e+01_rb,1.5200e+02_rb,2.2168e+02_rb/)
      kbo(:,59,15) = (/ &
     &3.2067e+01_rb,5.7347e+01_rb,9.3961e+01_rb,1.4472e+02_rb,2.1224e+02_rb/)
      kbo(:,13,16) = (/ &
     &1.4292e+02_rb,1.7830e+02_rb,2.5018e+02_rb,3.7037e+02_rb,5.2070e+02_rb/)
      kbo(:,14,16) = (/ &
     &1.7229e+02_rb,2.1935e+02_rb,3.0306e+02_rb,4.4388e+02_rb,6.2020e+02_rb/)
      kbo(:,15,16) = (/ &
     &2.0873e+02_rb,2.6972e+02_rb,3.6756e+02_rb,5.2712e+02_rb,7.3393e+02_rb/)
      kbo(:,16,16) = (/ &
     &2.5439e+02_rb,3.3072e+02_rb,4.4570e+02_rb,6.2226e+02_rb,8.6517e+02_rb/)
      kbo(:,17,16) = (/ &
     &3.0902e+02_rb,4.0145e+02_rb,5.3763e+02_rb,7.3415e+02_rb,1.0082e+03_rb/)
      kbo(:,18,16) = (/ &
     &3.7239e+02_rb,4.8343e+02_rb,6.4354e+02_rb,8.6401e+02_rb,1.1697e+03_rb/)
      kbo(:,19,16) = (/ &
     &4.4344e+02_rb,5.7744e+02_rb,7.6245e+02_rb,1.0126e+03_rb,1.3544e+03_rb/)
      kbo(:,20,16) = (/ &
     &5.2591e+02_rb,6.8473e+02_rb,8.9843e+02_rb,1.1879e+03_rb,1.5662e+03_rb/)
      kbo(:,21,16) = (/ &
     &6.1975e+02_rb,8.0389e+02_rb,1.0527e+03_rb,1.3850e+03_rb,1.8066e+03_rb/)
      kbo(:,22,16) = (/ &
     &7.3890e+02_rb,9.5515e+02_rb,1.2474e+03_rb,1.6302e+03_rb,2.1130e+03_rb/)
      kbo(:,23,16) = (/ &
     &8.7503e+02_rb,1.1289e+03_rb,1.4705e+03_rb,1.9102e+03_rb,2.4554e+03_rb/)
      kbo(:,24,16) = (/ &
     &1.0277e+03_rb,1.3283e+03_rb,1.7241e+03_rb,2.2247e+03_rb,2.8319e+03_rb/)
      kbo(:,25,16) = (/ &
     &1.2034e+03_rb,1.5543e+03_rb,2.0073e+03_rb,2.5693e+03_rb,3.2356e+03_rb/)
      kbo(:,26,16) = (/ &
     &1.4042e+03_rb,1.8088e+03_rb,2.3218e+03_rb,2.9229e+03_rb,3.6714e+03_rb/)
      kbo(:,27,16) = (/ &
     &1.6234e+03_rb,2.0833e+03_rb,2.6544e+03_rb,3.3382e+03_rb,4.1196e+03_rb/)
      kbo(:,28,16) = (/ &
     &1.8563e+03_rb,2.3730e+03_rb,3.0002e+03_rb,3.7388e+03_rb,4.5730e+03_rb/)
      kbo(:,29,16) = (/ &
     &2.1060e+03_rb,2.6609e+03_rb,3.3576e+03_rb,4.1487e+03_rb,5.0295e+03_rb/)
      kbo(:,30,16) = (/ &
     &2.3669e+03_rb,2.9882e+03_rb,3.7190e+03_rb,4.5573e+03_rb,5.4823e+03_rb/)
      kbo(:,31,16) = (/ &
     &2.6382e+03_rb,3.3079e+03_rb,4.0851e+03_rb,4.9669e+03_rb,5.9326e+03_rb/)
      kbo(:,32,16) = (/ &
     &2.9162e+03_rb,3.6311e+03_rb,4.4535e+03_rb,5.3188e+03_rb,6.3774e+03_rb/)
      kbo(:,33,16) = (/ &
     &3.1977e+03_rb,3.9546e+03_rb,4.8182e+03_rb,5.7741e+03_rb,6.8136e+03_rb/)
      kbo(:,34,16) = (/ &
     &3.4540e+03_rb,4.2467e+03_rb,5.1445e+03_rb,6.1305e+03_rb,7.1986e+03_rb/)
      kbo(:,35,16) = (/ &
     &3.6329e+03_rb,4.4502e+03_rb,5.3698e+03_rb,6.3759e+03_rb,7.4635e+03_rb/)
      kbo(:,36,16) = (/ &
     &3.7190e+03_rb,4.5470e+03_rb,5.4751e+03_rb,6.4918e+03_rb,7.5883e+03_rb/)
      kbo(:,37,16) = (/ &
     &3.6766e+03_rb,4.4979e+03_rb,5.4194e+03_rb,6.4326e+03_rb,7.5253e+03_rb/)
      kbo(:,38,16) = (/ &
     &3.6175e+03_rb,4.4311e+03_rb,5.3444e+03_rb,6.3514e+03_rb,7.4384e+03_rb/)
      kbo(:,39,16) = (/ &
     &3.5511e+03_rb,4.3577e+03_rb,5.2603e+03_rb,6.2593e+03_rb,7.3011e+03_rb/)
      kbo(:,40,16) = (/ &
     &3.4038e+03_rb,4.1899e+03_rb,5.0730e+03_rb,6.0546e+03_rb,7.1202e+03_rb/)
      kbo(:,41,16) = (/ &
     &3.2449e+03_rb,4.0090e+03_rb,4.8718e+03_rb,5.8306e+03_rb,6.8794e+03_rb/)
      kbo(:,42,16) = (/ &
     &3.0878e+03_rb,3.8270e+03_rb,4.6679e+03_rb,5.6043e+03_rb,6.6342e+03_rb/)
      kbo(:,43,16) = (/ &
     &2.8974e+03_rb,3.6038e+03_rb,4.4171e+03_rb,5.3250e+03_rb,6.3295e+03_rb/)
      kbo(:,44,16) = (/ &
     &2.6963e+03_rb,3.3670e+03_rb,4.1477e+03_rb,5.0263e+03_rb,5.9993e+03_rb/)
      kbo(:,45,16) = (/ &
     &2.5060e+03_rb,3.1396e+03_rb,3.8862e+03_rb,4.7337e+03_rb,5.6766e+03_rb/)
      kbo(:,46,16) = (/ &
     &2.3164e+03_rb,2.9079e+03_rb,3.6158e+03_rb,4.4299e+03_rb,5.3398e+03_rb/)
      kbo(:,47,16) = (/ &
     &2.1174e+03_rb,2.6621e+03_rb,3.3261e+03_rb,4.0998e+03_rb,4.9727e+03_rb/)
      kbo(:,48,16) = (/ &
     &1.9329e+03_rb,2.4347e+03_rb,3.0518e+03_rb,3.7834e+03_rb,4.6186e+03_rb/)
      kbo(:,49,16) = (/ &
     &1.7595e+03_rb,2.2251e+03_rb,2.7943e+03_rb,3.4811e+03_rb,4.2776e+03_rb/)
      kbo(:,50,16) = (/ &
     &1.6082e+03_rb,2.0431e+03_rb,2.5690e+03_rb,3.2137e+03_rb,3.9707e+03_rb/)
      kbo(:,51,16) = (/ &
     &1.4710e+03_rb,1.8773e+03_rb,2.3673e+03_rb,2.9684e+03_rb,3.6854e+03_rb/)
      kbo(:,52,16) = (/ &
     &1.3426e+03_rb,1.7222e+03_rb,2.1800e+03_rb,2.7371e+03_rb,3.4141e+03_rb/)
      kbo(:,53,16) = (/ &
     &1.2224e+03_rb,1.5765e+03_rb,2.0045e+03_rb,2.5219e+03_rb,3.1562e+03_rb/)
      kbo(:,54,16) = (/ &
     &1.1196e+03_rb,1.4516e+03_rb,1.8541e+03_rb,2.3387e+03_rb,2.9325e+03_rb/)
      kbo(:,55,16) = (/ &
     &1.0267e+03_rb,1.3387e+03_rb,1.7175e+03_rb,2.1745e+03_rb,2.7302e+03_rb/)
      kbo(:,56,16) = (/ &
     &9.3987e+02_rb,1.2326e+03_rb,1.5894e+03_rb,2.0202e+03_rb,2.5405e+03_rb/)
      kbo(:,57,16) = (/ &
     &8.5792e+02_rb,1.1333e+03_rb,1.4474e+03_rb,1.8742e+03_rb,2.3631e+03_rb/)
      kbo(:,58,16) = (/ &
     &7.8443e+02_rb,1.0443e+03_rb,1.3605e+03_rb,1.7435e+03_rb,2.2055e+03_rb/)
      kbo(:,59,16) = (/ &
     &7.5656e+02_rb,1.0103e+03_rb,1.3189e+03_rb,1.6936e+03_rb,2.1455e+03_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &2.8549e-03_rb,4.8281e-03_rb,6.2570e-03_rb,8.2731e-03_rb,7.9056e-03_rb,7.7840e-03_rb, &
     &1.0115e-02_rb,9.6599e-03_rb,1.0153e-02_rb,1.0921e-02_rb,1.2408e-02_rb,1.3496e-02_rb, &
     &1.5059e-02_rb,1.4636e-02_rb,1.6483e-02_rb,1.2394e-02_rb/)
      forrefo(2,:) = (/ &
     &3.0036e-03_rb,5.1093e-03_rb,5.7317e-03_rb,9.2246e-03_rb,8.9829e-03_rb,8.6477e-03_rb, &
     &1.1448e-02_rb,1.0391e-02_rb,1.0211e-02_rb,1.2921e-02_rb,1.2726e-02_rb,1.2426e-02_rb, &
     &1.4609e-02_rb,1.5783e-02_rb,1.6617e-02_rb,1.6858e-02_rb/)
      forrefo(3,:) = (/ &
     &3.0771e-03_rb,5.1206e-03_rb,5.8426e-03_rb,9.5727e-03_rb,1.0338e-02_rb,9.3737e-03_rb, &
     &1.2805e-02_rb,1.1272e-02_rb,1.1353e-02_rb,1.1837e-02_rb,1.1550e-02_rb,1.3020e-02_rb, &
     &1.3536e-02_rb,1.6226e-02_rb,1.6039e-02_rb,2.2578e-02_rb/)
      forrefo(4,:) = (/ &
     &3.3072e-03_rb,5.0240e-03_rb,6.8474e-03_rb,8.2736e-03_rb,8.6151e-03_rb,8.6762e-03_rb, &
     &1.1476e-02_rb,1.0246e-02_rb,1.0819e-02_rb,1.0640e-02_rb,1.0545e-02_rb,1.0533e-02_rb, &
     &1.0496e-02_rb,1.0142e-02_rb,9.7979e-03_rb,1.5255e-02_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 7.25695e-01_rb, 6.53591e-01_rb, 5.88650e-01_rb, 5.30162e-01_rb, 4.77485e-01_rb, &
     & 4.30042e-01_rb, 3.87313e-01_rb, 3.48830e-01_rb, 3.14170e-01_rb, 2.82954e-01_rb/)
      selfrefo(:, 2) = (/ &
     & 9.61996e-01_rb, 8.77853e-01_rb, 8.01070e-01_rb, 7.31003e-01_rb, 6.67064e-01_rb, &
     & 6.08718e-01_rb, 5.55476e-01_rb, 5.06890e-01_rb, 4.62554e-01_rb, 4.22096e-01_rb/)
      selfrefo(:, 3) = (/ &
     & 9.72584e-01_rb, 9.02658e-01_rb, 8.37760e-01_rb, 7.77527e-01_rb, 7.21626e-01_rb, &
     & 6.69743e-01_rb, 6.21591e-01_rb, 5.76900e-01_rb, 5.35423e-01_rb, 4.96927e-01_rb/)
      selfrefo(:, 4) = (/ &
     & 1.24790e+00_rb, 1.14353e+00_rb, 1.04790e+00_rb, 9.60263e-01_rb, 8.79956e-01_rb, &
     & 8.06364e-01_rb, 7.38927e-01_rb, 6.77130e-01_rb, 6.20501e-01_rb, 5.68608e-01_rb/)
      selfrefo(:, 5) = (/ &
     & 1.23574e+00_rb, 1.12928e+00_rb, 1.03200e+00_rb, 9.43096e-01_rb, 8.61851e-01_rb, &
     & 7.87605e-01_rb, 7.19755e-01_rb, 6.57750e-01_rb, 6.01087e-01_rb, 5.49305e-01_rb/)
      selfrefo(:, 6) = (/ &
     & 1.20921e+00_rb, 1.10660e+00_rb, 1.01270e+00_rb, 9.26766e-01_rb, 8.48124e-01_rb, &
     & 7.76155e-01_rb, 7.10293e-01_rb, 6.50020e-01_rb, 5.94861e-01_rb, 5.44384e-01_rb/)
      selfrefo(:, 7) = (/ &
     & 1.38112e+00_rb, 1.26727e+00_rb, 1.16280e+00_rb, 1.06694e+00_rb, 9.78990e-01_rb, &
     & 8.98287e-01_rb, 8.24236e-01_rb, 7.56290e-01_rb, 6.93945e-01_rb, 6.36739e-01_rb/)
      selfrefo(:, 8) = (/ &
     & 1.30321e+00_rb, 1.20127e+00_rb, 1.10730e+00_rb, 1.02068e+00_rb, 9.40840e-01_rb, &
     & 8.67243e-01_rb, 7.99403e-01_rb, 7.36870e-01_rb, 6.79229e-01_rb, 6.26096e-01_rb/)
      selfrefo(:, 9) = (/ &
     & 1.26713e+00_rb, 1.17927e+00_rb, 1.09750e+00_rb, 1.02140e+00_rb, 9.50575e-01_rb, &
     & 8.84662e-01_rb, 8.23319e-01_rb, 7.66230e-01_rb, 7.13099e-01_rb, 6.63653e-01_rb/)
      selfrefo(:,10) = (/ &
     & 1.49824e+00_rb, 1.37053e+00_rb, 1.25370e+00_rb, 1.14683e+00_rb, 1.04908e+00_rb, &
     & 9.59651e-01_rb, 8.77849e-01_rb, 8.03020e-01_rb, 7.34569e-01_rb, 6.71954e-01_rb/)
      selfrefo(:,11) = (/ &
     & 1.44786e+00_rb, 1.34594e+00_rb, 1.25120e+00_rb, 1.16313e+00_rb, 1.08125e+00_rb, &
     & 1.00514e+00_rb, 9.34392e-01_rb, 8.68620e-01_rb, 8.07477e-01_rb, 7.50639e-01_rb/)
      selfrefo(:,12) = (/ &
     & 1.38460e+00_rb, 1.30437e+00_rb, 1.22880e+00_rb, 1.15760e+00_rb, 1.09053e+00_rb, &
     & 1.02735e+00_rb, 9.67825e-01_rb, 9.11750e-01_rb, 8.58924e-01_rb, 8.09159e-01_rb/)
      selfrefo(:,13) = (/ &
     & 1.51953e+00_rb, 1.42822e+00_rb, 1.34240e+00_rb, 1.26173e+00_rb, 1.18592e+00_rb, &
     & 1.11465e+00_rb, 1.04768e+00_rb, 9.84720e-01_rb, 9.25548e-01_rb, 8.69932e-01_rb/)
      selfrefo(:,14) = (/ &
     & 1.62608e+00_rb, 1.51021e+00_rb, 1.40260e+00_rb, 1.30266e+00_rb, 1.20983e+00_rb, &
     & 1.12363e+00_rb, 1.04356e+00_rb, 9.69200e-01_rb, 9.00138e-01_rb, 8.35998e-01_rb/)
      selfrefo(:,15) = (/ &
     & 1.65383e+00_rb, 1.54808e+00_rb, 1.44910e+00_rb, 1.35644e+00_rb, 1.26971e+00_rb, &
     & 1.18853e+00_rb, 1.11254e+00_rb, 1.04140e+00_rb, 9.74813e-01_rb, 9.12484e-01_rb/)
      selfrefo(:,16) = (/ &
     & 1.78105e+00_rb, 1.61421e+00_rb, 1.46300e+00_rb, 1.32595e+00_rb, 1.20174e+00_rb, &
     & 1.08917e+00_rb, 9.87141e-01_rb, 8.94670e-01_rb, 8.10861e-01_rb, 7.34904e-01_rb/)

      end subroutine lw_kgb02

! **************************************************************************
      subroutine lw_kgb03
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg03, only : fracrefao, fracrefbo, kao, kbo, kao_mn2o, &
                            kbo_mn2o, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level: P=212.7250 mbar, T = 223.06 K
      fracrefao(:, 1) = (/ &
     &   1.6251e-01_rb,1.5572e-01_rb,1.4557e-01_rb,1.3208e-01_rb,1.1582e-01_rb,9.6895e-02_rb, &
     &   7.8720e-02_rb,5.8462e-02_rb,3.9631e-02_rb,4.3001e-03_rb,3.5555e-03_rb,2.8101e-03_rb, &
     &   2.0547e-03_rb,1.3109e-03_rb,4.9403e-04_rb,6.9515e-05_rb/)
      fracrefao(:, 2) = (/ &
     &   1.6006e-01_rb,1.5576e-01_rb,1.4609e-01_rb,1.3276e-01_rb,1.1594e-01_rb,9.7336e-02_rb, &
     &   7.9035e-02_rb,5.8696e-02_rb,3.9723e-02_rb,4.3001e-03_rb,3.5555e-03_rb,2.8101e-03_rb, &
     &   2.0547e-03_rb,1.3109e-03_rb,4.9403e-04_rb,6.9515e-05_rb/)
      fracrefao(:, 3) = (/ &
     &   1.5952e-01_rb,1.5566e-01_rb,1.4590e-01_rb,1.3294e-01_rb,1.1599e-01_rb,9.7511e-02_rb, &
     &   7.9127e-02_rb,5.8888e-02_rb,3.9874e-02_rb,4.3001e-03_rb,3.5555e-03_rb,2.8102e-03_rb, &
     &   2.0547e-03_rb,1.3109e-03_rb,4.9403e-04_rb,6.9515e-05_rb/)
      fracrefao(:, 4) = (/ &
     &   1.5907e-01_rb,1.5541e-01_rb,1.4585e-01_rb,1.3316e-01_rb,1.1596e-01_rb,9.7647e-02_rb, &
     &   7.9243e-02_rb,5.9024e-02_rb,4.0028e-02_rb,4.3112e-03_rb,3.5555e-03_rb,2.8102e-03_rb, &
     &   2.0547e-03_rb,1.3109e-03_rb,4.9403e-04_rb,6.9515e-05_rb/)
      fracrefao(:, 5) = (/ &
     &   1.5862e-01_rb,1.5517e-01_rb,1.4588e-01_rb,1.3328e-01_rb,1.1585e-01_rb,9.7840e-02_rb, &
     &   7.9364e-02_rb,5.9174e-02_rb,4.0160e-02_rb,4.3403e-03_rb,3.5900e-03_rb,2.8102e-03_rb, &
     &   2.0547e-03_rb,1.3109e-03_rb,4.9403e-04_rb,6.9515e-05_rb/)
      fracrefao(:, 6) = (/ &
     &   1.5830e-01_rb,1.5490e-01_rb,1.4582e-01_rb,1.3331e-01_rb,1.1567e-01_rb,9.8079e-02_rb, &
     &   7.9510e-02_rb,5.9369e-02_rb,4.0326e-02_rb,4.3343e-03_rb,3.5908e-03_rb,2.8527e-03_rb, &
     &   2.0655e-03_rb,1.3109e-03_rb,4.9403e-04_rb,6.9515e-05_rb/)
      fracrefao(:, 7) = (/ &
     &   1.5789e-01_rb,1.5435e-01_rb,1.4595e-01_rb,1.3304e-01_rb,1.1566e-01_rb,9.8426e-02_rb, &
     &   7.9704e-02_rb,5.9618e-02_rb,4.0520e-02_rb,4.3812e-03_rb,3.6147e-03_rb,2.8395e-03_rb, &
     &   2.1301e-03_rb,1.3145e-03_rb,4.9403e-04_rb,6.9515e-05_rb/)
      fracrefao(:, 8) = (/ &
     &   1.5704e-01_rb,1.5398e-01_rb,1.4564e-01_rb,1.3222e-01_rb,1.1586e-01_rb,9.9230e-02_rb, &
     &   8.0011e-02_rb,6.0149e-02_rb,4.0790e-02_rb,4.4253e-03_rb,3.6534e-03_rb,2.9191e-03_rb, &
     &   2.1373e-03_rb,1.3558e-03_rb,5.1631e-04_rb,7.8794e-05_rb/)
      fracrefao(:, 9) = (/ &
     &   1.5270e-01_rb,1.5126e-01_rb,1.4264e-01_rb,1.3106e-01_rb,1.1740e-01_rb,1.0137e-01_rb, &
     &   8.3057e-02_rb,6.2282e-02_rb,4.2301e-02_rb,4.6486e-03_rb,3.8159e-03_rb,3.0472e-03_rb, &
     &   2.2870e-03_rb,1.4818e-03_rb,5.6773e-04_rb,7.8794e-05_rb/)

! Planck fraction mapping level: p = 95.8 mbar, t = 215.7 k
      fracrefbo(:, 1) = (/ &
     &   1.6413e-01_rb,1.5665e-01_rb,1.4606e-01_rb,1.3184e-01_rb,1.1517e-01_rb,9.6243e-02_rb, &
     &   7.7982e-02_rb,5.8165e-02_rb,3.9311e-02_rb,4.2586e-03_rb,3.5189e-03_rb,2.7793e-03_rb, &
     &   2.0376e-03_rb,1.2938e-03_rb,4.8853e-04_rb,6.8745e-05_rb/)
      fracrefbo(:, 2) = (/ &
     &   1.6254e-01_rb,1.5674e-01_rb,1.4652e-01_rb,1.3221e-01_rb,1.1535e-01_rb,9.6439e-02_rb, &
     &   7.8155e-02_rb,5.8254e-02_rb,3.9343e-02_rb,4.2586e-03_rb,3.5189e-03_rb,2.7793e-03_rb, &
     &   2.0376e-03_rb,1.2938e-03_rb,4.8853e-04_rb,6.8745e-05_rb/)
      fracrefbo(:, 3) = (/ &
     &   1.6177e-01_rb,1.5664e-01_rb,1.4669e-01_rb,1.3242e-01_rb,1.1541e-01_rb,9.6536e-02_rb, &
     &   7.8257e-02_rb,5.8387e-02_rb,3.9431e-02_rb,4.2587e-03_rb,3.5189e-03_rb,2.7793e-03_rb, &
     &   2.0376e-03_rb,1.2938e-03_rb,4.8853e-04_rb,6.8745e-05_rb/)
      fracrefbo(:, 4) = (/ &
     &   1.6077e-01_rb,1.5679e-01_rb,1.4648e-01_rb,1.3273e-01_rb,1.1546e-01_rb,9.6779e-02_rb, &
     &   7.8371e-02_rb,5.8546e-02_rb,3.9611e-02_rb,4.2772e-03_rb,3.5190e-03_rb,2.7793e-03_rb, &
     &   2.0376e-03_rb,1.2938e-03_rb,4.8853e-04_rb,6.8745e-05_rb/)
      fracrefbo(:, 5) = (/ &
     &   1.6067e-01_rb,1.5608e-01_rb,1.4247e-01_rb,1.2881e-01_rb,1.1449e-01_rb,9.8802e-02_rb, &
     &   8.0828e-02_rb,6.0977e-02_rb,4.1494e-02_rb,4.5116e-03_rb,3.7290e-03_rb,2.9460e-03_rb, &
     &   2.1948e-03_rb,1.3778e-03_rb,5.4552e-04_rb,7.9969e-05_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &3.1886e-07_rb,1.8969e-04_rb,4.1239e-04_rb,6.4667e-04_rb,8.5292e-04_rb,1.1836e-03_rb, &
     &1.4006e-03_rb,1.5411e-03_rb,8.3293e-04_rb/)
      kao(:, 2, 1, 1) = (/ &
     &4.3208e-07_rb,2.2286e-04_rb,4.4627e-04_rb,7.2330e-04_rb,9.3694e-04_rb,1.2337e-03_rb, &
     &1.5482e-03_rb,1.7773e-03_rb,1.0038e-03_rb/)
      kao(:, 3, 1, 1) = (/ &
     &5.8834e-07_rb,2.6143e-04_rb,5.0441e-04_rb,7.7424e-04_rb,1.0403e-03_rb,1.3331e-03_rb, &
     &1.7908e-03_rb,2.0845e-03_rb,1.1619e-03_rb/)
      kao(:, 4, 1, 1) = (/ &
     &7.9926e-07_rb,3.0532e-04_rb,5.7655e-04_rb,8.4667e-04_rb,1.1189e-03_rb,1.4849e-03_rb, &
     &1.7793e-03_rb,2.3392e-03_rb,1.2758e-03_rb/)
      kao(:, 5, 1, 1) = (/ &
     &1.0726e-06_rb,3.5559e-04_rb,6.6472e-04_rb,9.5586e-04_rb,1.2287e-03_rb,1.5629e-03_rb, &
     &1.9684e-03_rb,2.6521e-03_rb,1.4781e-03_rb/)
      kao(:, 1, 2, 1) = (/ &
     &3.7301e-07_rb,1.5101e-04_rb,3.0613e-04_rb,5.2444e-04_rb,6.6785e-04_rb,9.0347e-04_rb, &
     &1.1047e-03_rb,1.2717e-03_rb,6.0920e-04_rb/)
      kao(:, 2, 2, 1) = (/ &
     &5.0464e-07_rb,1.7900e-04_rb,3.4606e-04_rb,5.4112e-04_rb,7.7706e-04_rb,9.3108e-04_rb, &
     &1.2909e-03_rb,1.4671e-03_rb,7.3637e-04_rb/)
      kao(:, 3, 2, 1) = (/ &
     &6.9109e-07_rb,2.1210e-04_rb,4.0308e-04_rb,5.9670e-04_rb,8.3572e-04_rb,1.0539e-03_rb, &
     &1.2895e-03_rb,1.6281e-03_rb,8.3917e-04_rb/)
      kao(:, 4, 2, 1) = (/ &
     &9.4449e-07_rb,2.4973e-04_rb,4.6862e-04_rb,6.7662e-04_rb,9.0294e-04_rb,1.1388e-03_rb, &
     &1.4230e-03_rb,1.8780e-03_rb,9.5236e-04_rb/)
      kao(:, 5, 2, 1) = (/ &
     &1.2785e-06_rb,2.9283e-04_rb,5.4407e-04_rb,7.7522e-04_rb,1.0053e-03_rb,1.2254e-03_rb, &
     &1.5592e-03_rb,1.9031e-03_rb,1.1164e-03_rb/)
      kao(:, 1, 3, 1) = (/ &
     &5.5921e-07_rb,1.1924e-04_rb,2.2509e-04_rb,3.4318e-04_rb,5.2308e-04_rb,6.5616e-04_rb, &
     &8.5230e-04_rb,1.0069e-03_rb,4.1268e-04_rb/)
      kao(:, 2, 3, 1) = (/ &
     &7.5055e-07_rb,1.4483e-04_rb,2.6805e-04_rb,3.9345e-04_rb,5.4245e-04_rb,7.6863e-04_rb, &
     &8.6931e-04_rb,1.1266e-03_rb,5.0829e-04_rb/)
      kao(:, 3, 3, 1) = (/ &
     &1.0282e-06_rb,1.7418e-04_rb,3.1823e-04_rb,4.6052e-04_rb,6.0646e-04_rb,8.0699e-04_rb, &
     &9.9010e-04_rb,1.2822e-03_rb,5.8353e-04_rb/)
      kao(:, 4, 3, 1) = (/ &
     &1.4165e-06_rb,2.0776e-04_rb,3.7640e-04_rb,5.3896e-04_rb,6.9736e-04_rb,8.7108e-04_rb, &
     &1.0948e-03_rb,1.3035e-03_rb,6.8429e-04_rb/)
      kao(:, 5, 3, 1) = (/ &
     &1.9344e-06_rb,2.4744e-04_rb,4.4263e-04_rb,6.2983e-04_rb,8.0258e-04_rb,9.7503e-04_rb, &
     &1.1651e-03_rb,1.4497e-03_rb,8.1461e-04_rb/)
      kao(:, 1, 4, 1) = (/ &
     &8.5732e-07_rb,9.6467e-05_rb,1.7424e-04_rb,2.5100e-04_rb,3.3065e-04_rb,4.8724e-04_rb, &
     &6.0299e-04_rb,7.8582e-04_rb,2.9808e-04_rb/)
      kao(:, 2, 4, 1) = (/ &
     &1.1308e-06_rb,1.1826e-04_rb,2.1303e-04_rb,3.0261e-04_rb,3.9186e-04_rb,5.0849e-04_rb, &
     &7.0322e-04_rb,8.2275e-04_rb,3.5462e-04_rb/)
      kao(:, 3, 4, 1) = (/ &
     &1.5409e-06_rb,1.4413e-04_rb,2.5818e-04_rb,3.6263e-04_rb,4.6469e-04_rb,5.7592e-04_rb, &
     &7.3867e-04_rb,8.7496e-04_rb,4.2760e-04_rb/)
      kao(:, 4, 4, 1) = (/ &
     &2.1300e-06_rb,1.7563e-04_rb,3.1026e-04_rb,4.3228e-04_rb,5.4859e-04_rb,6.6681e-04_rb, &
     &7.9581e-04_rb,9.9932e-04_rb,5.0892e-04_rb/)
      kao(:, 5, 4, 1) = (/ &
     &2.9291e-06_rb,2.1272e-04_rb,3.6864e-04_rb,5.1165e-04_rb,6.4677e-04_rb,7.7619e-04_rb, &
     &8.9814e-04_rb,1.0897e-03_rb,6.0064e-04_rb/)
      kao(:, 1, 5, 1) = (/ &
     &1.2415e-06_rb,7.8277e-05_rb,1.3778e-04_rb,1.9284e-04_rb,2.4740e-04_rb,3.0422e-04_rb, &
     &4.5046e-04_rb,5.5097e-04_rb,2.1557e-04_rb/)
      kao(:, 2, 5, 1) = (/ &
     &1.5909e-06_rb,9.7521e-05_rb,1.7125e-04_rb,2.3852e-04_rb,3.0221e-04_rb,3.6563e-04_rb, &
     &4.6525e-04_rb,6.1320e-04_rb,2.5987e-04_rb/)
      kao(:, 3, 5, 1) = (/ &
     &2.1423e-06_rb,1.2163e-04_rb,2.0881e-04_rb,2.9129e-04_rb,3.6625e-04_rb,4.3908e-04_rb, &
     &5.2524e-04_rb,7.1422e-04_rb,3.1853e-04_rb/)
      kao(:, 4, 5, 1) = (/ &
     &2.9538e-06_rb,1.5072e-04_rb,2.5416e-04_rb,3.5255e-04_rb,4.4063e-04_rb,5.2156e-04_rb, &
     &6.0639e-04_rb,7.4820e-04_rb,3.8686e-04_rb/)
      kao(:, 5, 5, 1) = (/ &
     &4.0808e-06_rb,1.8492e-04_rb,3.0793e-04_rb,4.2155e-04_rb,5.2630e-04_rb,6.1892e-04_rb, &
     &7.0820e-04_rb,8.0049e-04_rb,4.6257e-04_rb/)
      kao(:, 1, 6, 1) = (/ &
     &1.7276e-06_rb,6.3066e-05_rb,1.0985e-04_rb,1.5043e-04_rb,1.8792e-04_rb,2.2547e-04_rb, &
     &2.7278e-04_rb,4.0029e-04_rb,1.5625e-04_rb/)
      kao(:, 2, 6, 1) = (/ &
     &2.1189e-06_rb,8.0476e-05_rb,1.3664e-04_rb,1.8841e-04_rb,2.3492e-04_rb,2.7888e-04_rb, &
     &3.2236e-04_rb,4.6057e-04_rb,1.9653e-04_rb/)
      kao(:, 3, 6, 1) = (/ &
     &2.7917e-06_rb,1.0200e-04_rb,1.6954e-04_rb,2.3141e-04_rb,2.8961e-04_rb,3.4143e-04_rb, &
     &3.9023e-04_rb,4.8352e-04_rb,2.3947e-04_rb/)
      kao(:, 4, 6, 1) = (/ &
     &3.8168e-06_rb,1.2811e-04_rb,2.1054e-04_rb,2.8370e-04_rb,3.5217e-04_rb,4.1385e-04_rb, &
     &4.6780e-04_rb,5.3253e-04_rb,2.9421e-04_rb/)
      kao(:, 5, 6, 1) = (/ &
     &5.2799e-06_rb,1.5893e-04_rb,2.5910e-04_rb,3.4517e-04_rb,4.2434e-04_rb,4.9842e-04_rb, &
     &5.5728e-04_rb,6.1195e-04_rb,3.4978e-04_rb/)
      kao(:, 1, 7, 1) = (/ &
     &2.6141e-06_rb,5.1662e-05_rb,8.7249e-05_rb,1.2099e-04_rb,1.4820e-04_rb,1.7386e-04_rb, &
     &1.9973e-04_rb,2.8823e-04_rb,1.2462e-04_rb/)
      kao(:, 2, 7, 1) = (/ &
     &3.0451e-06_rb,6.6903e-05_rb,1.1099e-04_rb,1.5079e-04_rb,1.8779e-04_rb,2.1746e-04_rb, &
     &2.4532e-04_rb,2.8791e-04_rb,1.5686e-04_rb/)
      kao(:, 3, 7, 1) = (/ &
     &3.8821e-06_rb,8.6234e-05_rb,1.4155e-04_rb,1.8817e-04_rb,2.3079e-04_rb,2.7142e-04_rb, &
     &3.0362e-04_rb,3.3424e-04_rb,1.9309e-04_rb/)
      kao(:, 4, 7, 1) = (/ &
     &5.2168e-06_rb,1.0990e-04_rb,1.7879e-04_rb,2.3488e-04_rb,2.8585e-04_rb,3.3138e-04_rb, &
     &3.7038e-04_rb,4.0029e-04_rb,2.2841e-04_rb/)
      kao(:, 5, 7, 1) = (/ &
     &7.1879e-06_rb,1.3891e-04_rb,2.2275e-04_rb,2.9114e-04_rb,3.5068e-04_rb,4.0298e-04_rb, &
     &4.4929e-04_rb,4.7758e-04_rb,2.7333e-04_rb/)
      kao(:, 1, 8, 1) = (/ &
     &4.8895e-06_rb,4.6642e-05_rb,7.3353e-05_rb,9.8230e-05_rb,1.2186e-04_rb,1.4040e-04_rb, &
     &1.5472e-04_rb,1.7158e-04_rb,1.2116e-04_rb/)
      kao(:, 2, 8, 1) = (/ &
     &5.4032e-06_rb,6.0479e-05_rb,9.5191e-05_rb,1.2606e-04_rb,1.5260e-04_rb,1.7774e-04_rb, &
     &1.9641e-04_rb,2.0965e-04_rb,1.4264e-04_rb/)
      kao(:, 3, 8, 1) = (/ &
     &6.5997e-06_rb,7.8772e-05_rb,1.2353e-04_rb,1.6212e-04_rb,1.9427e-04_rb,2.2313e-04_rb, &
     &2.4636e-04_rb,2.5986e-04_rb,1.7188e-04_rb/)
      kao(:, 4, 8, 1) = (/ &
     &8.6548e-06_rb,1.0205e-04_rb,1.5903e-04_rb,2.0720e-04_rb,2.4581e-04_rb,2.7759e-04_rb, &
     &3.0314e-04_rb,3.2037e-04_rb,2.0203e-04_rb/)
      kao(:, 5, 8, 1) = (/ &
     &1.1792e-05_rb,1.3168e-04_rb,2.0236e-04_rb,2.6066e-04_rb,3.0759e-04_rb,3.4497e-04_rb, &
     &3.7389e-04_rb,3.8968e-04_rb,2.4008e-04_rb/)
      kao(:, 1, 9, 1) = (/ &
     &1.7686e-05_rb,6.2925e-05_rb,8.5198e-05_rb,1.0213e-04_rb,1.1543e-04_rb,1.2780e-04_rb, &
     &1.3994e-04_rb,1.4164e-04_rb,1.0306e-04_rb/)
      kao(:, 2, 9, 1) = (/ &
     &1.8629e-05_rb,7.9145e-05_rb,1.0839e-04_rb,1.3177e-04_rb,1.5002e-04_rb,1.6481e-04_rb, &
     &1.7578e-04_rb,1.8331e-04_rb,1.3692e-04_rb/)
      kao(:, 3, 9, 1) = (/ &
     &2.1655e-05_rb,1.0259e-04_rb,1.4047e-04_rb,1.7139e-04_rb,1.9624e-04_rb,2.1451e-04_rb, &
     &2.2698e-04_rb,2.2863e-04_rb,1.7350e-04_rb/)
      kao(:, 4, 9, 1) = (/ &
     &2.7461e-05_rb,1.3575e-04_rb,1.8409e-04_rb,2.2366e-04_rb,2.5484e-04_rb,2.7815e-04_rb, &
     &2.9152e-04_rb,2.8719e-04_rb,2.1425e-04_rb/)
      kao(:, 5, 9, 1) = (/ &
     &3.6727e-05_rb,1.8189e-04_rb,2.4258e-04_rb,2.9022e-04_rb,3.2921e-04_rb,3.5617e-04_rb, &
     &3.6983e-04_rb,3.5850e-04_rb,2.5866e-04_rb/)
      kao(:, 1,10, 1) = (/ &
     &7.2663e-05_rb,1.2805e-04_rb,1.4610e-04_rb,1.5523e-04_rb,1.5931e-04_rb,1.5768e-04_rb, &
     &1.4950e-04_rb,1.3593e-04_rb,1.1207e-04_rb/)
      kao(:, 2,10, 1) = (/ &
     &7.4019e-05_rb,1.5108e-04_rb,1.7625e-04_rb,1.9103e-04_rb,1.9729e-04_rb,1.9817e-04_rb, &
     &1.9144e-04_rb,1.7408e-04_rb,1.5499e-04_rb/)
      kao(:, 3,10, 1) = (/ &
     &8.2403e-05_rb,1.8863e-04_rb,2.2414e-04_rb,2.4536e-04_rb,2.5373e-04_rb,2.5549e-04_rb, &
     &2.4939e-04_rb,2.2811e-04_rb,2.0551e-04_rb/)
      kao(:, 4,10, 1) = (/ &
     &1.0093e-04_rb,2.4723e-04_rb,2.9626e-04_rb,3.2296e-04_rb,3.3348e-04_rb,3.3522e-04_rb, &
     &3.2677e-04_rb,2.9826e-04_rb,2.6379e-04_rb/)
      kao(:, 5,10, 1) = (/ &
     &1.3200e-04_rb,3.3367e-04_rb,4.0011e-04_rb,4.3332e-04_rb,4.4493e-04_rb,4.4261e-04_rb, &
     &4.2611e-04_rb,3.8581e-04_rb,3.2912e-04_rb/)
      kao(:, 1,11, 1) = (/ &
     &1.0393e-04_rb,1.5840e-04_rb,1.7203e-04_rb,1.7648e-04_rb,1.7454e-04_rb,1.6667e-04_rb, &
     &1.5308e-04_rb,1.2939e-04_rb,1.0993e-04_rb/)
      kao(:, 2,11, 1) = (/ &
     &1.0623e-04_rb,1.8473e-04_rb,2.0510e-04_rb,2.1424e-04_rb,2.1529e-04_rb,2.0732e-04_rb, &
     &1.9332e-04_rb,1.6735e-04_rb,1.5034e-04_rb/)
      kao(:, 3,11, 1) = (/ &
     &1.1880e-04_rb,2.2821e-04_rb,2.5935e-04_rb,2.7369e-04_rb,2.7717e-04_rb,2.6782e-04_rb, &
     &2.5002e-04_rb,2.1993e-04_rb,1.9822e-04_rb/)
      kao(:, 4,11, 1) = (/ &
     &1.4561e-04_rb,2.9665e-04_rb,3.4318e-04_rb,3.6287e-04_rb,3.6642e-04_rb,3.5386e-04_rb, &
     &3.2944e-04_rb,2.8811e-04_rb,2.5319e-04_rb/)
      kao(:, 5,11, 1) = (/ &
     &1.9054e-04_rb,3.9809e-04_rb,4.6486e-04_rb,4.9198e-04_rb,4.9373e-04_rb,4.7462e-04_rb, &
     &4.3676e-04_rb,3.7664e-04_rb,3.1222e-04_rb/)
      kao(:, 1,12, 1) = (/ &
     &1.1070e-04_rb,1.5948e-04_rb,1.6935e-04_rb,1.7123e-04_rb,1.6742e-04_rb,1.5636e-04_rb, &
     &1.4108e-04_rb,1.1641e-04_rb,9.3609e-05_rb/)
      kao(:, 2,12, 1) = (/ &
     &1.1350e-04_rb,1.8499e-04_rb,2.0156e-04_rb,2.0690e-04_rb,2.0458e-04_rb,1.9450e-04_rb, &
     &1.7678e-04_rb,1.4985e-04_rb,1.2839e-04_rb/)
      kao(:, 3,12, 1) = (/ &
     &1.2733e-04_rb,2.2735e-04_rb,2.5402e-04_rb,2.6442e-04_rb,2.6348e-04_rb,2.5189e-04_rb, &
     &2.2992e-04_rb,1.9655e-04_rb,1.6928e-04_rb/)
      kao(:, 4,12, 1) = (/ &
     &1.5627e-04_rb,2.9456e-04_rb,3.3549e-04_rb,3.5142e-04_rb,3.5055e-04_rb,3.3475e-04_rb, &
     &3.0473e-04_rb,2.5892e-04_rb,2.1653e-04_rb/)
      kao(:, 5,12, 1) = (/ &
     &2.0441e-04_rb,3.9462e-04_rb,4.5461e-04_rb,4.7637e-04_rb,4.7456e-04_rb,4.5071e-04_rb, &
     &4.0805e-04_rb,3.4047e-04_rb,2.6779e-04_rb/)
      kao(:, 1,13, 1) = (/ &
     &9.4796e-05_rb,1.3566e-04_rb,1.4361e-04_rb,1.4479e-04_rb,1.4118e-04_rb,1.3147e-04_rb, &
     &1.1819e-04_rb,9.7170e-05_rb,7.6303e-05_rb/)
      kao(:, 2,13, 1) = (/ &
     &9.7387e-05_rb,1.5742e-04_rb,1.7115e-04_rb,1.7535e-04_rb,1.7300e-04_rb,1.6383e-04_rb, &
     &1.4824e-04_rb,1.2513e-04_rb,1.0469e-04_rb/)
      kao(:, 3,13, 1) = (/ &
     &1.0948e-04_rb,1.9354e-04_rb,2.1601e-04_rb,2.2450e-04_rb,2.2321e-04_rb,2.1287e-04_rb, &
     &1.9350e-04_rb,1.6433e-04_rb,1.3819e-04_rb/)
      kao(:, 4,13, 1) = (/ &
     &1.3445e-04_rb,2.5083e-04_rb,2.8549e-04_rb,2.9847e-04_rb,2.9725e-04_rb,2.8343e-04_rb, &
     &2.5763e-04_rb,2.1720e-04_rb,1.7601e-04_rb/)
      kao(:, 5,13, 1) = (/ &
     &1.7573e-04_rb,3.3657e-04_rb,3.8710e-04_rb,4.0560e-04_rb,4.0357e-04_rb,3.8257e-04_rb, &
     &3.4653e-04_rb,2.8685e-04_rb,2.1855e-04_rb/)
      kao(:, 1, 1, 2) = (/ &
     &1.3073e-06_rb,4.2059e-04_rb,6.7561e-04_rb,8.6609e-04_rb,1.3520e-03_rb,1.6085e-03_rb, &
     &2.0908e-03_rb,3.5262e-03_rb,1.7846e-03_rb/)
      kao(:, 2, 1, 2) = (/ &
     &1.8197e-06_rb,5.0768e-04_rb,8.4110e-04_rb,1.0238e-03_rb,1.3765e-03_rb,1.9239e-03_rb, &
     &2.2997e-03_rb,3.5756e-03_rb,2.0767e-03_rb/)
      kao(:, 3, 1, 2) = (/ &
     &2.5084e-06_rb,6.1290e-04_rb,1.0202e-03_rb,1.2693e-03_rb,1.5516e-03_rb,2.0078e-03_rb, &
     &2.5461e-03_rb,3.4922e-03_rb,2.2668e-03_rb/)
      kao(:, 4, 1, 2) = (/ &
     &3.4224e-06_rb,7.3392e-04_rb,1.2104e-03_rb,1.5455e-03_rb,1.8866e-03_rb,2.1966e-03_rb, &
     &2.9624e-03_rb,3.8708e-03_rb,2.5909e-03_rb/)
      kao(:, 5, 1, 2) = (/ &
     &4.5990e-06_rb,8.7030e-04_rb,1.4253e-03_rb,1.8395e-03_rb,2.2537e-03_rb,2.6314e-03_rb, &
     &3.0779e-03_rb,4.2442e-03_rb,2.9663e-03_rb/)
      kao(:, 1, 2, 2) = (/ &
     &1.5178e-06_rb,3.4269e-04_rb,6.0748e-04_rb,7.1986e-04_rb,9.3674e-04_rb,1.3121e-03_rb, &
     &1.6102e-03_rb,2.4069e-03_rb,1.3247e-03_rb/)
      kao(:, 2, 2, 2) = (/ &
     &2.1299e-06_rb,4.2524e-04_rb,7.3703e-04_rb,9.3776e-04_rb,1.0398e-03_rb,1.4367e-03_rb, &
     &1.7353e-03_rb,2.4061e-03_rb,1.5243e-03_rb/)
      kao(:, 3, 2, 2) = (/ &
     &2.9612e-06_rb,5.1753e-04_rb,8.8626e-04_rb,1.1500e-03_rb,1.2896e-03_rb,1.5263e-03_rb, &
     &2.0876e-03_rb,2.6389e-03_rb,1.6966e-03_rb/)
      kao(:, 4, 2, 2) = (/ &
     &4.0897e-06_rb,6.2596e-04_rb,1.0597e-03_rb,1.3843e-03_rb,1.5850e-03_rb,1.8098e-03_rb, &
     &2.1268e-03_rb,2.8656e-03_rb,1.9584e-03_rb/)
      kao(:, 5, 2, 2) = (/ &
     &5.5687e-06_rb,7.4555e-04_rb,1.2600e-03_rb,1.6347e-03_rb,1.9070e-03_rb,2.1914e-03_rb, &
     &2.4166e-03_rb,3.3007e-03_rb,2.2669e-03_rb/)
      kao(:, 1, 3, 2) = (/ &
     &2.2234e-06_rb,2.7444e-04_rb,5.0085e-04_rb,6.9534e-04_rb,7.7629e-04_rb,8.8854e-04_rb, &
     &1.1917e-03_rb,1.5373e-03_rb,9.1595e-04_rb/)
      kao(:, 2, 3, 2) = (/ &
     &3.1317e-06_rb,3.4263e-04_rb,6.2257e-04_rb,8.4941e-04_rb,1.0127e-03_rb,1.0143e-03_rb, &
     &1.3362e-03_rb,1.6583e-03_rb,1.0393e-03_rb/)
      kao(:, 3, 3, 2) = (/ &
     &4.4080e-06_rb,4.2422e-04_rb,7.6347e-04_rb,1.0279e-03_rb,1.2387e-03_rb,1.2967e-03_rb, &
     &1.3932e-03_rb,1.8408e-03_rb,1.2152e-03_rb/)
      kao(:, 4, 3, 2) = (/ &
     &6.1670e-06_rb,5.2130e-04_rb,9.2425e-04_rb,1.2412e-03_rb,1.4849e-03_rb,1.6022e-03_rb, &
     &1.6415e-03_rb,2.0935e-03_rb,1.4201e-03_rb/)
      kao(:, 5, 3, 2) = (/ &
     &8.5264e-06_rb,6.3507e-04_rb,1.1072e-03_rb,1.4843e-03_rb,1.7747e-03_rb,1.9362e-03_rb, &
     &2.0233e-03_rb,2.1572e-03_rb,1.6619e-03_rb/)
      kao(:, 1, 4, 2) = (/ &
     &3.2835e-06_rb,2.2111e-04_rb,3.9199e-04_rb,5.4782e-04_rb,7.1479e-04_rb,7.6154e-04_rb, &
     &8.2345e-04_rb,1.0888e-03_rb,6.1749e-04_rb/)
      kao(:, 2, 4, 2) = (/ &
     &4.6177e-06_rb,2.8239e-04_rb,4.9392e-04_rb,6.9299e-04_rb,8.7089e-04_rb,1.0035e-03_rb, &
     &9.4113e-04_rb,1.2776e-03_rb,7.2212e-04_rb/)
      kao(:, 3, 4, 2) = (/ &
     &6.5470e-06_rb,3.5777e-04_rb,6.1550e-04_rb,8.6066e-04_rb,1.0686e-03_rb,1.2181e-03_rb, &
     &1.2204e-03_rb,1.3196e-03_rb,8.5856e-04_rb/)
      kao(:, 4, 4, 2) = (/ &
     &9.2896e-06_rb,4.4826e-04_rb,7.6106e-04_rb,1.0547e-03_rb,1.2989e-03_rb,1.4671e-03_rb, &
     &1.5260e-03_rb,1.4238e-03_rb,1.0237e-03_rb/)
      kao(:, 5, 4, 2) = (/ &
     &1.3071e-05_rb,5.5733e-04_rb,9.3625e-04_rb,1.2821e-03_rb,1.5559e-03_rb,1.7516e-03_rb, &
     &1.8364e-03_rb,1.7115e-03_rb,1.2165e-03_rb/)
      kao(:, 1, 5, 2) = (/ &
     &4.5002e-06_rb,1.7869e-04_rb,3.1112e-04_rb,4.2906e-04_rb,5.3570e-04_rb,6.6321e-04_rb, &
     &6.6606e-04_rb,8.0088e-04_rb,4.3845e-04_rb/)
      kao(:, 2, 5, 2) = (/ &
     &6.2709e-06_rb,2.3526e-04_rb,3.9946e-04_rb,5.4822e-04_rb,6.8446e-04_rb,8.1650e-04_rb, &
     &8.9841e-04_rb,8.3661e-04_rb,5.3250e-04_rb/)
      kao(:, 3, 5, 2) = (/ &
     &8.9215e-06_rb,3.0515e-04_rb,5.1007e-04_rb,6.9053e-04_rb,8.5739e-04_rb,1.0092e-03_rb, &
     &1.0997e-03_rb,9.5118e-04_rb,6.4123e-04_rb/)
      kao(:, 4, 5, 2) = (/ &
     &1.2814e-05_rb,3.9188e-04_rb,6.4418e-04_rb,8.6120e-04_rb,1.0618e-03_rb,1.2430e-03_rb, &
     &1.3327e-03_rb,1.2360e-03_rb,7.6830e-04_rb/)
      kao(:, 5, 5, 2) = (/ &
     &1.8376e-05_rb,4.9865e-04_rb,8.0644e-04_rb,1.0670e-03_rb,1.3038e-03_rb,1.5043e-03_rb, &
     &1.6049e-03_rb,1.5432e-03_rb,9.1314e-04_rb/)
      kao(:, 1, 6, 2) = (/ &
     &5.7892e-06_rb,1.4663e-04_rb,2.4065e-04_rb,3.3064e-04_rb,4.1135e-04_rb,4.8169e-04_rb, &
     &5.8379e-04_rb,5.7710e-04_rb,3.1814e-04_rb/)
      kao(:, 2, 6, 2) = (/ &
     &7.9690e-06_rb,1.9590e-04_rb,3.2041e-04_rb,4.3222e-04_rb,5.3334e-04_rb,6.2056e-04_rb, &
     &7.1751e-04_rb,6.6758e-04_rb,3.9067e-04_rb/)
      kao(:, 3, 6, 2) = (/ &
     &1.1323e-05_rb,2.6004e-04_rb,4.1914e-04_rb,5.5855e-04_rb,6.8140e-04_rb,7.8989e-04_rb, &
     &8.8852e-04_rb,8.8775e-04_rb,4.8310e-04_rb/)
      kao(:, 4, 6, 2) = (/ &
     &1.6358e-05_rb,3.4053e-04_rb,5.4044e-04_rb,7.1305e-04_rb,8.6088e-04_rb,9.8765e-04_rb, &
     &1.0974e-03_rb,1.1000e-03_rb,5.7948e-04_rb/)
      kao(:, 5, 6, 2) = (/ &
     &2.3775e-05_rb,4.4089e-04_rb,6.9082e-04_rb,9.0148e-04_rb,1.0783e-03_rb,1.2236e-03_rb, &
     &1.3409e-03_rb,1.3317e-03_rb,6.9627e-04_rb/)
      kao(:, 1, 7, 2) = (/ &
     &8.0091e-06_rb,1.2605e-04_rb,1.9716e-04_rb,2.5635e-04_rb,3.1697e-04_rb,3.6727e-04_rb, &
     &4.1064e-04_rb,4.3234e-04_rb,2.4150e-04_rb/)
      kao(:, 2, 7, 2) = (/ &
     &1.0776e-05_rb,1.7034e-04_rb,2.6589e-04_rb,3.4715e-04_rb,4.2062e-04_rb,4.8583e-04_rb, &
     &5.3757e-04_rb,5.9551e-04_rb,2.9725e-04_rb/)
      kao(:, 3, 7, 2) = (/ &
     &1.5189e-05_rb,2.2959e-04_rb,3.5476e-04_rb,4.5964e-04_rb,5.5426e-04_rb,6.3049e-04_rb, &
     &6.9328e-04_rb,7.3902e-04_rb,3.6802e-04_rb/)
      kao(:, 4, 7, 2) = (/ &
     &2.2007e-05_rb,3.0657e-04_rb,4.6692e-04_rb,5.9983e-04_rb,7.1322e-04_rb,8.0913e-04_rb, &
     &8.7898e-04_rb,9.1398e-04_rb,4.5358e-04_rb/)
      kao(:, 5, 7, 2) = (/ &
     &3.2377e-05_rb,4.0363e-04_rb,6.0728e-04_rb,7.7481e-04_rb,9.1139e-04_rb,1.0238e-03_rb, &
     &1.0988e-03_rb,1.1244e-03_rb,5.4283e-04_rb/)
      kao(:, 1, 8, 2) = (/ &
     &1.3646e-05_rb,1.1718e-04_rb,1.7739e-04_rb,2.2327e-04_rb,2.5768e-04_rb,2.9227e-04_rb, &
     &3.2168e-04_rb,3.3945e-04_rb,2.0435e-04_rb/)
      kao(:, 2, 8, 2) = (/ &
     &1.7797e-05_rb,1.5982e-04_rb,2.4160e-04_rb,3.0351e-04_rb,3.5441e-04_rb,3.9644e-04_rb, &
     &4.3035e-04_rb,4.4547e-04_rb,2.6027e-04_rb/)
      kao(:, 3, 8, 2) = (/ &
     &2.4726e-05_rb,2.1816e-04_rb,3.2650e-04_rb,4.0893e-04_rb,4.7479e-04_rb,5.2784e-04_rb, &
     &5.6990e-04_rb,5.8414e-04_rb,3.2342e-04_rb/)
      kao(:, 4, 8, 2) = (/ &
     &3.5739e-05_rb,2.9719e-04_rb,4.3967e-04_rb,5.4459e-04_rb,6.2961e-04_rb,6.9435e-04_rb, &
     &7.4016e-04_rb,7.4703e-04_rb,3.9508e-04_rb/)
      kao(:, 5, 8, 2) = (/ &
     &5.2993e-05_rb,4.0333e-04_rb,5.8500e-04_rb,7.1845e-04_rb,8.2505e-04_rb,9.0108e-04_rb, &
     &9.4665e-04_rb,9.4587e-04_rb,4.7280e-04_rb/)
      kao(:, 1, 9, 2) = (/ &
     &4.5250e-05_rb,1.4686e-04_rb,2.0230e-04_rb,2.4828e-04_rb,2.8036e-04_rb,2.9801e-04_rb, &
     &2.9921e-04_rb,2.8815e-04_rb,2.4577e-04_rb/)
      kao(:, 2, 9, 2) = (/ &
     &5.6592e-05_rb,2.0110e-04_rb,2.8181e-04_rb,3.4405e-04_rb,3.8167e-04_rb,4.0863e-04_rb, &
     &4.1514e-04_rb,3.9115e-04_rb,3.2441e-04_rb/)
      kao(:, 3, 9, 2) = (/ &
     &7.7017e-05_rb,2.8027e-04_rb,3.9316e-04_rb,4.7187e-04_rb,5.1925e-04_rb,5.5669e-04_rb, &
     &5.6308e-04_rb,5.3177e-04_rb,4.1003e-04_rb/)
      kao(:, 4, 9, 2) = (/ &
     &1.1025e-04_rb,3.9471e-04_rb,5.4612e-04_rb,6.4824e-04_rb,7.1055e-04_rb,7.5245e-04_rb, &
     &7.5557e-04_rb,7.0714e-04_rb,5.0547e-04_rb/)
      kao(:, 5, 9, 2) = (/ &
     &1.6377e-04_rb,5.5266e-04_rb,7.5260e-04_rb,8.8463e-04_rb,9.6217e-04_rb,1.0084e-03_rb, &
     &1.0010e-03_rb,9.2831e-04_rb,5.9992e-04_rb/)
      kao(:, 1,10, 2) = (/ &
     &1.7408e-04_rb,2.8709e-04_rb,3.2527e-04_rb,3.4835e-04_rb,3.5894e-04_rb,3.5919e-04_rb, &
     &3.5263e-04_rb,3.1484e-04_rb,2.7571e-04_rb/)
      kao(:, 2,10, 2) = (/ &
     &2.0961e-04_rb,3.6816e-04_rb,4.3143e-04_rb,4.6881e-04_rb,4.9659e-04_rb,5.0147e-04_rb, &
     &4.9272e-04_rb,4.3339e-04_rb,3.8859e-04_rb/)
      kao(:, 3,10, 2) = (/ &
     &2.7776e-04_rb,5.0192e-04_rb,5.9603e-04_rb,6.5181e-04_rb,6.9357e-04_rb,7.0312e-04_rb, &
     &6.7864e-04_rb,5.9253e-04_rb,5.1467e-04_rb/)
      kao(:, 4,10, 2) = (/ &
     &3.9424e-04_rb,7.1243e-04_rb,8.4498e-04_rb,9.2473e-04_rb,9.7484e-04_rb,9.8216e-04_rb, &
     &9.3644e-04_rb,8.0724e-04_rb,6.5632e-04_rb/)
      kao(:, 5,10, 2) = (/ &
     &5.8403e-04_rb,1.0226e-03_rb,1.2011e-03_rb,1.3089e-03_rb,1.3689e-03_rb,1.3686e-03_rb, &
     &1.2847e-03_rb,1.0916e-03_rb,8.0745e-04_rb/)
      kao(:, 1,11, 2) = (/ &
     &2.5093e-04_rb,3.5665e-04_rb,3.8701e-04_rb,3.9631e-04_rb,3.9372e-04_rb,3.8144e-04_rb, &
     &3.5373e-04_rb,3.1423e-04_rb,3.0104e-04_rb/)
      kao(:, 2,11, 2) = (/ &
     &3.0561e-04_rb,4.5397e-04_rb,5.0521e-04_rb,5.3017e-04_rb,5.3366e-04_rb,5.2816e-04_rb, &
     &4.9620e-04_rb,4.3115e-04_rb,4.1409e-04_rb/)
      kao(:, 3,11, 2) = (/ &
     &4.0935e-04_rb,6.2197e-04_rb,6.9995e-04_rb,7.3845e-04_rb,7.4646e-04_rb,7.4054e-04_rb, &
     &7.0035e-04_rb,5.8985e-04_rb,5.4448e-04_rb/)
      kao(:, 4,11, 2) = (/ &
     &5.8584e-04_rb,8.9030e-04_rb,1.0016e-03_rb,1.0495e-03_rb,1.0645e-03_rb,1.0487e-03_rb, &
     &9.8184e-04_rb,8.1405e-04_rb,6.7706e-04_rb/)
      kao(:, 5,11, 2) = (/ &
     &8.7589e-04_rb,1.3050e-03_rb,1.4464e-03_rb,1.5053e-03_rb,1.5175e-03_rb,1.4779e-03_rb, &
     &1.3691e-03_rb,1.1154e-03_rb,8.3668e-04_rb/)
      kao(:, 1,12, 2) = (/ &
     &2.6981e-04_rb,3.6165e-04_rb,3.8393e-04_rb,3.8621e-04_rb,3.7773e-04_rb,3.5863e-04_rb, &
     &3.2723e-04_rb,2.8136e-04_rb,2.7937e-04_rb/)
      kao(:, 2,12, 2) = (/ &
     &3.3189e-04_rb,4.6172e-04_rb,5.0224e-04_rb,5.1546e-04_rb,5.1134e-04_rb,4.9419e-04_rb, &
     &4.6078e-04_rb,3.9503e-04_rb,3.8165e-04_rb/)
      kao(:, 3,12, 2) = (/ &
     &4.4945e-04_rb,6.3614e-04_rb,7.0015e-04_rb,7.2309e-04_rb,7.1840e-04_rb,6.9882e-04_rb, &
     &6.5062e-04_rb,5.4697e-04_rb,5.0283e-04_rb/)
      kao(:, 4,12, 2) = (/ &
     &6.4949e-04_rb,9.2459e-04_rb,1.0096e-03_rb,1.0404e-03_rb,1.0301e-03_rb,9.9704e-04_rb, &
     &9.2064e-04_rb,7.5726e-04_rb,6.3715e-04_rb/)
      kao(:, 5,12, 2) = (/ &
     &9.7948e-04_rb,1.3695e-03_rb,1.4755e-03_rb,1.5098e-03_rb,1.4823e-03_rb,1.4173e-03_rb, &
     &1.2918e-03_rb,1.0472e-03_rb,7.8120e-04_rb/)
      kao(:, 1,13, 2) = (/ &
     &2.3347e-04_rb,3.0984e-04_rb,3.2740e-04_rb,3.2836e-04_rb,3.1964e-04_rb,3.0307e-04_rb, &
     &2.7562e-04_rb,2.3564e-04_rb,2.3674e-04_rb/)
      kao(:, 2,13, 2) = (/ &
     &2.9027e-04_rb,3.9857e-04_rb,4.3170e-04_rb,4.4144e-04_rb,4.3482e-04_rb,4.1943e-04_rb, &
     &3.8943e-04_rb,3.3194e-04_rb,3.2365e-04_rb/)
      kao(:, 3,13, 2) = (/ &
     &3.9524e-04_rb,5.5472e-04_rb,6.0676e-04_rb,6.2319e-04_rb,6.1597e-04_rb,5.9710e-04_rb, &
     &5.5278e-04_rb,4.6213e-04_rb,4.2552e-04_rb/)
      kao(:, 4,13, 2) = (/ &
     &5.7486e-04_rb,8.1393e-04_rb,8.8216e-04_rb,9.0454e-04_rb,8.9206e-04_rb,8.5566e-04_rb, &
     &7.8499e-04_rb,6.4422e-04_rb,5.3476e-04_rb/)
      kao(:, 5,13, 2) = (/ &
     &8.7246e-04_rb,1.2122e-03_rb,1.2989e-03_rb,1.3221e-03_rb,1.2955e-03_rb,1.2260e-03_rb, &
     &1.1071e-03_rb,8.9174e-04_rb,6.5849e-04_rb/)
      kao(:, 1, 1, 3) = (/ &
     &5.2546e-06_rb,7.1189e-04_rb,1.0882e-03_rb,1.5407e-03_rb,1.6694e-03_rb,2.0484e-03_rb, &
     &2.8523e-03_rb,4.1696e-03_rb,2.5067e-03_rb/)
      kao(:, 2, 1, 3) = (/ &
     &7.7092e-06_rb,8.7649e-04_rb,1.3426e-03_rb,1.8360e-03_rb,2.2345e-03_rb,2.3018e-03_rb, &
     &2.9500e-03_rb,4.8247e-03_rb,2.8678e-03_rb/)
      kao(:, 3, 1, 3) = (/ &
     &1.1096e-05_rb,1.0569e-03_rb,1.6133e-03_rb,2.2145e-03_rb,2.7396e-03_rb,3.0159e-03_rb, &
     &3.2179e-03_rb,5.1113e-03_rb,3.6024e-03_rb/)
      kao(:, 4, 1, 3) = (/ &
     &1.5736e-05_rb,1.2601e-03_rb,1.9380e-03_rb,2.6510e-03_rb,3.2424e-03_rb,3.7893e-03_rb, &
     &3.9992e-03_rb,5.0794e-03_rb,4.4455e-03_rb/)
      kao(:, 5, 1, 3) = (/ &
     &2.1768e-05_rb,1.4881e-03_rb,2.3013e-03_rb,3.1272e-03_rb,3.8505e-03_rb,4.4934e-03_rb, &
     &5.1336e-03_rb,5.8796e-03_rb,5.3642e-03_rb/)
      kao(:, 1, 2, 3) = (/ &
     &6.4189e-06_rb,6.4436e-04_rb,9.4389e-04_rb,1.2239e-03_rb,1.5862e-03_rb,1.5943e-03_rb, &
     &2.1761e-03_rb,2.9768e-03_rb,1.8319e-03_rb/)
      kao(:, 2, 2, 3) = (/ &
     &9.5139e-06_rb,7.9863e-04_rb,1.1748e-03_rb,1.5104e-03_rb,1.9094e-03_rb,2.0876e-03_rb, &
     &2.2172e-03_rb,3.3690e-03_rb,2.2079e-03_rb/)
      kao(:, 3, 2, 3) = (/ &
     &1.3920e-05_rb,9.7738e-04_rb,1.4337e-03_rb,1.8422e-03_rb,2.2904e-03_rb,2.6664e-03_rb, &
     &2.6655e-03_rb,3.3322e-03_rb,2.7973e-03_rb/)
      kao(:, 4, 2, 3) = (/ &
     &1.9815e-05_rb,1.1745e-03_rb,1.7255e-03_rb,2.2150e-03_rb,2.7638e-03_rb,3.1950e-03_rb, &
     &3.5144e-03_rb,3.6601e-03_rb,3.4291e-03_rb/)
      kao(:, 5, 2, 3) = (/ &
     &2.7567e-05_rb,1.3937e-03_rb,2.0553e-03_rb,2.6513e-03_rb,3.2736e-03_rb,3.7952e-03_rb, &
     &4.2837e-03_rb,4.4922e-03_rb,4.1160e-03_rb/)
      kao(:, 1, 3, 3) = (/ &
     &9.6319e-06_rb,5.9789e-04_rb,9.0789e-04_rb,1.1077e-03_rb,1.2525e-03_rb,1.5120e-03_rb, &
     &1.4959e-03_rb,2.1276e-03_rb,1.3456e-03_rb/)
      kao(:, 2, 3, 3) = (/ &
     &1.4531e-05_rb,7.5640e-04_rb,1.1412e-03_rb,1.3989e-03_rb,1.5703e-03_rb,1.7944e-03_rb, &
     &1.9386e-03_rb,2.1696e-03_rb,1.7109e-03_rb/)
      kao(:, 3, 3, 3) = (/ &
     &2.1457e-05_rb,9.3610e-04_rb,1.4115e-03_rb,1.7272e-03_rb,1.9369e-03_rb,2.2003e-03_rb, &
     &2.5012e-03_rb,2.3167e-03_rb,2.1265e-03_rb/)
      kao(:, 4, 3, 3) = (/ &
     &3.1112e-05_rb,1.1388e-03_rb,1.7200e-03_rb,2.0901e-03_rb,2.3590e-03_rb,2.6864e-03_rb, &
     &2.9768e-03_rb,2.9114e-03_rb,2.5844e-03_rb/)
      kao(:, 5, 3, 3) = (/ &
     &4.3735e-05_rb,1.3632e-03_rb,2.0663e-03_rb,2.5031e-03_rb,2.8359e-03_rb,3.2124e-03_rb, &
     &3.5643e-03_rb,3.8138e-03_rb,3.0766e-03_rb/)
      kao(:, 1, 4, 3) = (/ &
     &1.4332e-05_rb,5.0481e-04_rb,8.3885e-04_rb,1.0386e-03_rb,1.1528e-03_rb,1.2153e-03_rb, &
     &1.3236e-03_rb,1.3236e-03_rb,9.9177e-04_rb/)
      kao(:, 2, 4, 3) = (/ &
     &2.1745e-05_rb,6.5797e-04_rb,1.0771e-03_rb,1.3282e-03_rb,1.4860e-03_rb,1.5492e-03_rb, &
     &1.6188e-03_rb,1.5468e-03_rb,1.2757e-03_rb/)
      kao(:, 3, 4, 3) = (/ &
     &3.2667e-05_rb,8.3614e-04_rb,1.3479e-03_rb,1.6673e-03_rb,1.8582e-03_rb,1.9416e-03_rb, &
     &2.0036e-03_rb,2.1024e-03_rb,1.5754e-03_rb/)
      kao(:, 4, 4, 3) = (/ &
     &4.8093e-05_rb,1.0472e-03_rb,1.6541e-03_rb,2.0462e-03_rb,2.2910e-03_rb,2.3927e-03_rb, &
     &2.4569e-03_rb,2.6010e-03_rb,1.9210e-03_rb/)
      kao(:, 5, 4, 3) = (/ &
     &6.8610e-05_rb,1.2933e-03_rb,1.9995e-03_rb,2.4752e-03_rb,2.7858e-03_rb,2.9107e-03_rb, &
     &2.9847e-03_rb,3.0912e-03_rb,2.3242e-03_rb/)
      kao(:, 1, 5, 3) = (/ &
     &1.9357e-05_rb,4.3188e-04_rb,6.9760e-04_rb,9.1881e-04_rb,1.0444e-03_rb,1.0861e-03_rb, &
     &1.0808e-03_rb,1.0528e-03_rb,7.6202e-04_rb/)
      kao(:, 2, 5, 3) = (/ &
     &2.9833e-05_rb,5.6548e-04_rb,9.1440e-04_rb,1.2005e-03_rb,1.3604e-03_rb,1.4256e-03_rb, &
     &1.3945e-03_rb,1.3872e-03_rb,9.4421e-04_rb/)
      kao(:, 3, 5, 3) = (/ &
     &4.5578e-05_rb,7.2730e-04_rb,1.1762e-03_rb,1.5269e-03_rb,1.7329e-03_rb,1.8168e-03_rb, &
     &1.7751e-03_rb,1.6935e-03_rb,1.1652e-03_rb/)
      kao(:, 4, 5, 3) = (/ &
     &6.7915e-05_rb,9.2227e-04_rb,1.4839e-03_rb,1.8964e-03_rb,2.1540e-03_rb,2.2580e-03_rb, &
     &2.2284e-03_rb,2.0768e-03_rb,1.4187e-03_rb/)
      kao(:, 5, 5, 3) = (/ &
     &9.8232e-05_rb,1.1614e-03_rb,1.8417e-03_rb,2.3162e-03_rb,2.6333e-03_rb,2.7678e-03_rb, &
     &2.7278e-03_rb,2.5343e-03_rb,1.7135e-03_rb/)
      kao(:, 1, 6, 3) = (/ &
     &2.4087e-05_rb,3.6023e-04_rb,5.7620e-04_rb,7.4775e-04_rb,8.9007e-04_rb,9.5429e-04_rb, &
     &9.1764e-04_rb,8.4636e-04_rb,5.7959e-04_rb/)
      kao(:, 2, 6, 3) = (/ &
     &3.7139e-05_rb,4.8232e-04_rb,7.6564e-04_rb,9.9341e-04_rb,1.1781e-03_rb,1.2665e-03_rb, &
     &1.2330e-03_rb,1.0918e-03_rb,7.2688e-04_rb/)
      kao(:, 3, 6, 3) = (/ &
     &5.7836e-05_rb,6.3457e-04_rb,9.9482e-04_rb,1.2902e-03_rb,1.5235e-03_rb,1.6284e-03_rb, &
     &1.5983e-03_rb,1.4005e-03_rb,8.9110e-04_rb/)
      kao(:, 4, 6, 3) = (/ &
     &8.7827e-05_rb,8.3029e-04_rb,1.2776e-03_rb,1.6365e-03_rb,1.9214e-03_rb,2.0538e-03_rb, &
     &2.0181e-03_rb,1.7675e-03_rb,1.0912e-03_rb/)
      kao(:, 5, 6, 3) = (/ &
     &1.2951e-04_rb,1.0725e-03_rb,1.6206e-03_rb,2.0554e-03_rb,2.3768e-03_rb,2.5369e-03_rb, &
     &2.5004e-03_rb,2.2047e-03_rb,1.3222e-03_rb/)
      kao(:, 1, 7, 3) = (/ &
     &3.1610e-05_rb,3.0736e-04_rb,4.8099e-04_rb,6.1875e-04_rb,7.2425e-04_rb,8.0714e-04_rb, &
     &8.2765e-04_rb,7.1397e-04_rb,4.2627e-04_rb/)
      kao(:, 2, 7, 3) = (/ &
     &4.8734e-05_rb,4.2352e-04_rb,6.5473e-04_rb,8.3909e-04_rb,9.8036e-04_rb,1.0919e-03_rb, &
     &1.1195e-03_rb,9.5621e-04_rb,5.5009e-04_rb/)
      kao(:, 3, 7, 3) = (/ &
     &7.5777e-05_rb,5.7189e-04_rb,8.6982e-04_rb,1.1085e-03_rb,1.2941e-03_rb,1.4332e-03_rb, &
     &1.4595e-03_rb,1.2666e-03_rb,6.8930e-04_rb/)
      kao(:, 4, 7, 3) = (/ &
     &1.1758e-04_rb,7.6635e-04_rb,1.1428e-03_rb,1.4431e-03_rb,1.6723e-03_rb,1.8359e-03_rb, &
     &1.8555e-03_rb,1.6214e-03_rb,8.4503e-04_rb/)
      kao(:, 5, 7, 3) = (/ &
     &1.7719e-04_rb,1.0176e-03_rb,1.4892e-03_rb,1.8529e-03_rb,2.1284e-03_rb,2.3074e-03_rb, &
     &2.3192e-03_rb,2.0277e-03_rb,1.0269e-03_rb/)
      kao(:, 1, 8, 3) = (/ &
     &5.0016e-05_rb,3.0350e-04_rb,4.3130e-04_rb,5.4117e-04_rb,6.2740e-04_rb,6.8603e-04_rb, &
     &7.1531e-04_rb,6.7206e-04_rb,3.4534e-04_rb/)
      kao(:, 2, 8, 3) = (/ &
     &7.6305e-05_rb,4.2110e-04_rb,6.0202e-04_rb,7.5262e-04_rb,8.6967e-04_rb,9.4998e-04_rb, &
     &9.8539e-04_rb,9.2682e-04_rb,4.4615e-04_rb/)
      kao(:, 3, 8, 3) = (/ &
     &1.1911e-04_rb,5.8304e-04_rb,8.2209e-04_rb,1.0213e-03_rb,1.1738e-03_rb,1.2774e-03_rb, &
     &1.3191e-03_rb,1.2307e-03_rb,5.5948e-04_rb/)
      kao(:, 4, 8, 3) = (/ &
     &1.8649e-04_rb,7.9923e-04_rb,1.1102e-03_rb,1.3602e-03_rb,1.5530e-03_rb,1.6805e-03_rb, &
     &1.7288e-03_rb,1.5932e-03_rb,6.8951e-04_rb/)
      kao(:, 5, 8, 3) = (/ &
     &2.8664e-04_rb,1.0891e-03_rb,1.4894e-03_rb,1.7956e-03_rb,2.0238e-03_rb,2.1795e-03_rb, &
     &2.2144e-03_rb,2.0105e-03_rb,8.3262e-04_rb/)
      kao(:, 1, 9, 3) = (/ &
     &1.5293e-04_rb,4.6711e-04_rb,5.9429e-04_rb,6.7143e-04_rb,7.0270e-04_rb,7.2578e-04_rb, &
     &7.2031e-04_rb,6.7023e-04_rb,4.6857e-04_rb/)
      kao(:, 2, 9, 3) = (/ &
     &2.2743e-04_rb,6.5435e-04_rb,8.3260e-04_rb,9.2221e-04_rb,9.8476e-04_rb,1.0204e-03_rb, &
     &1.0235e-03_rb,9.4955e-04_rb,5.6708e-04_rb/)
      kao(:, 3, 9, 3) = (/ &
     &3.5601e-04_rb,9.4855e-04_rb,1.1602e-03_rb,1.2875e-03_rb,1.3640e-03_rb,1.4067e-03_rb, &
     &1.4070e-03_rb,1.3032e-03_rb,6.4714e-04_rb/)
      kao(:, 4, 9, 3) = (/ &
     &5.5911e-04_rb,1.3606e-03_rb,1.6272e-03_rb,1.7782e-03_rb,1.8671e-03_rb,1.9159e-03_rb, &
     &1.8899e-03_rb,1.7374e-03_rb,7.3858e-04_rb/)
      kao(:, 5, 9, 3) = (/ &
     &8.7081e-04_rb,1.9356e-03_rb,2.2669e-03_rb,2.4429e-03_rb,2.5498e-03_rb,2.5834e-03_rb, &
     &2.5150e-03_rb,2.2763e-03_rb,8.7139e-04_rb/)
      kao(:, 1,10, 3) = (/ &
     &5.5025e-04_rb,9.1638e-04_rb,1.0524e-03_rb,1.1151e-03_rb,1.1173e-03_rb,1.0672e-03_rb, &
     &9.7649e-04_rb,7.7345e-04_rb,6.5002e-04_rb/)
      kao(:, 2,10, 3) = (/ &
     &7.9126e-04_rb,1.3002e-03_rb,1.4860e-03_rb,1.5598e-03_rb,1.5441e-03_rb,1.5015e-03_rb, &
     &1.3399e-03_rb,1.0909e-03_rb,8.3499e-04_rb/)
      kao(:, 3,10, 3) = (/ &
     &1.2237e-03_rb,1.9321e-03_rb,2.1587e-03_rb,2.2526e-03_rb,2.2235e-03_rb,2.1040e-03_rb, &
     &1.8744e-03_rb,1.5238e-03_rb,1.0298e-03_rb/)
      kao(:, 4,10, 3) = (/ &
     &1.9295e-03_rb,2.9068e-03_rb,3.1939e-03_rb,3.2884e-03_rb,3.1828e-03_rb,2.9739e-03_rb, &
     &2.6100e-03_rb,2.0967e-03_rb,1.2389e-03_rb/)
      kao(:, 5,10, 3) = (/ &
     &3.0420e-03_rb,4.3560e-03_rb,4.6911e-03_rb,4.7252e-03_rb,4.5246e-03_rb,4.1666e-03_rb, &
     &3.6292e-03_rb,2.8608e-03_rb,1.4677e-03_rb/)
      kao(:, 1,11, 3) = (/ &
     &8.0124e-04_rb,1.1600e-03_rb,1.2577e-03_rb,1.2913e-03_rb,1.2758e-03_rb,1.1948e-03_rb, &
     &1.0607e-03_rb,8.2464e-04_rb,6.8579e-04_rb/)
      kao(:, 2,11, 3) = (/ &
     &1.1673e-03_rb,1.6642e-03_rb,1.8003e-03_rb,1.8395e-03_rb,1.7987e-03_rb,1.6759e-03_rb, &
     &1.4977e-03_rb,1.1597e-03_rb,8.7714e-04_rb/)
      kao(:, 3,11, 3) = (/ &
     &1.8208e-03_rb,2.5072e-03_rb,2.6736e-03_rb,2.6935e-03_rb,2.6255e-03_rb,2.4196e-03_rb, &
     &2.1072e-03_rb,1.6350e-03_rb,1.1070e-03_rb/)
      kao(:, 4,11, 3) = (/ &
     &2.8872e-03_rb,3.8298e-03_rb,4.0028e-03_rb,4.0090e-03_rb,3.8400e-03_rb,3.4828e-03_rb, &
     &2.9827e-03_rb,2.2825e-03_rb,1.3306e-03_rb/)
      kao(:, 5,11, 3) = (/ &
     &4.5758e-03_rb,5.8214e-03_rb,5.9954e-03_rb,5.8806e-03_rb,5.5457e-03_rb,4.9671e-03_rb, &
     &4.1979e-03_rb,3.1537e-03_rb,1.5896e-03_rb/)
      kao(:, 1,12, 3) = (/ &
     &8.6957e-04_rb,1.1918e-03_rb,1.2597e-03_rb,1.2688e-03_rb,1.2375e-03_rb,1.1513e-03_rb, &
     &1.0110e-03_rb,7.8620e-04_rb,6.6607e-04_rb/)
      kao(:, 2,12, 3) = (/ &
     &1.2841e-03_rb,1.7262e-03_rb,1.8227e-03_rb,1.8310e-03_rb,1.7702e-03_rb,1.6364e-03_rb, &
     &1.4379e-03_rb,1.0990e-03_rb,8.6314e-04_rb/)
      kao(:, 3,12, 3) = (/ &
     &2.0183e-03_rb,2.6229e-03_rb,2.7286e-03_rb,2.7135e-03_rb,2.6112e-03_rb,2.3987e-03_rb, &
     &2.0472e-03_rb,1.5496e-03_rb,1.0681e-03_rb/)
      kao(:, 4,12, 3) = (/ &
     &3.2212e-03_rb,4.0504e-03_rb,4.1472e-03_rb,4.0702e-03_rb,3.8583e-03_rb,3.4748e-03_rb, &
     &2.9262e-03_rb,2.1914e-03_rb,1.2601e-03_rb/)
      kao(:, 5,12, 3) = (/ &
     &5.1511e-03_rb,6.2196e-03_rb,6.2591e-03_rb,6.0402e-03_rb,5.6302e-03_rb,4.9946e-03_rb, &
     &4.1641e-03_rb,3.0581e-03_rb,1.4932e-03_rb/)
      kao(:, 1,13, 3) = (/ &
     &7.5570e-04_rb,1.0245e-03_rb,1.0772e-03_rb,1.0860e-03_rb,1.0524e-03_rb,9.7557e-04_rb, &
     &8.5610e-04_rb,6.6220e-04_rb,5.7960e-04_rb/)
      kao(:, 2,13, 3) = (/ &
     &1.1289e-03_rb,1.4995e-03_rb,1.5736e-03_rb,1.5761e-03_rb,1.5191e-03_rb,1.4055e-03_rb, &
     &1.2213e-03_rb,9.3390e-04_rb,7.5295e-04_rb/)
      kao(:, 3,13, 3) = (/ &
     &1.7891e-03_rb,2.2942e-03_rb,2.3709e-03_rb,2.3513e-03_rb,2.2646e-03_rb,2.0616e-03_rb, &
     &1.7545e-03_rb,1.3283e-03_rb,9.2119e-04_rb/)
      kao(:, 4,13, 3) = (/ &
     &2.8872e-03_rb,3.5675e-03_rb,3.6372e-03_rb,3.5628e-03_rb,3.3407e-03_rb,3.0031e-03_rb, &
     &2.5207e-03_rb,1.8863e-03_rb,1.0888e-03_rb/)
      kao(:, 5,13, 3) = (/ &
     &4.6509e-03_rb,5.5327e-03_rb,5.4994e-03_rb,5.2862e-03_rb,4.8993e-03_rb,4.3534e-03_rb, &
     &3.6092e-03_rb,2.6447e-03_rb,1.2715e-03_rb/)
      kao(:, 1, 1, 4) = (/ &
     &2.0197e-05_rb,1.1667e-03_rb,1.8703e-03_rb,2.3597e-03_rb,2.8442e-03_rb,3.3004e-03_rb, &
     &3.9209e-03_rb,5.1050e-03_rb,4.4032e-03_rb/)
      kao(:, 2, 1, 4) = (/ &
     &2.9878e-05_rb,1.4283e-03_rb,2.2934e-03_rb,2.9621e-03_rb,3.4763e-03_rb,4.1665e-03_rb, &
     &4.8948e-03_rb,5.8620e-03_rb,5.4661e-03_rb/)
      kao(:, 3, 1, 4) = (/ &
     &4.2867e-05_rb,1.7152e-03_rb,2.7742e-03_rb,3.5744e-03_rb,4.2361e-03_rb,5.0272e-03_rb, &
     &5.9642e-03_rb,7.9378e-03_rb,6.6044e-03_rb/)
      kao(:, 4, 1, 4) = (/ &
     &5.9869e-05_rb,2.0335e-03_rb,3.2827e-03_rb,4.2442e-03_rb,5.1046e-03_rb,5.9179e-03_rb, &
     &7.1401e-03_rb,9.6601e-03_rb,7.8490e-03_rb/)
      kao(:, 5, 1, 4) = (/ &
     &8.1517e-05_rb,2.3834e-03_rb,3.8440e-03_rb,4.9959e-03_rb,6.0214e-03_rb,7.0692e-03_rb, &
     &8.4265e-03_rb,1.1443e-02_rb,9.2888e-03_rb/)
      kao(:, 1, 2, 4) = (/ &
     &2.5366e-05_rb,1.0671e-03_rb,1.6131e-03_rb,2.1123e-03_rb,2.3685e-03_rb,2.7185e-03_rb, &
     &3.0418e-03_rb,3.4879e-03_rb,3.3143e-03_rb/)
      kao(:, 2, 2, 4) = (/ &
     &3.7671e-05_rb,1.3088e-03_rb,2.0014e-03_rb,2.6106e-03_rb,3.0312e-03_rb,3.3873e-03_rb, &
     &3.9017e-03_rb,4.5215e-03_rb,4.1143e-03_rb/)
      kao(:, 3, 2, 4) = (/ &
     &5.4541e-05_rb,1.5795e-03_rb,2.4306e-03_rb,3.1726e-03_rb,3.7137e-03_rb,4.1208e-03_rb, &
     &4.7737e-03_rb,5.7376e-03_rb,4.9901e-03_rb/)
      kao(:, 4, 2, 4) = (/ &
     &7.7056e-05_rb,1.8857e-03_rb,2.9009e-03_rb,3.7836e-03_rb,4.4364e-03_rb,5.0235e-03_rb, &
     &5.6427e-03_rb,6.9365e-03_rb,5.9848e-03_rb/)
      kao(:, 5, 2, 4) = (/ &
     &1.0566e-04_rb,2.2344e-03_rb,3.4137e-03_rb,4.4552e-03_rb,5.2671e-03_rb,5.9853e-03_rb, &
     &6.7249e-03_rb,8.2397e-03_rb,7.1296e-03_rb/)
      kao(:, 1, 3, 4) = (/ &
     &3.9558e-05_rb,1.0742e-03_rb,1.5248e-03_rb,1.8501e-03_rb,2.1830e-03_rb,2.3261e-03_rb, &
     &2.4391e-03_rb,2.5890e-03_rb,2.3829e-03_rb/)
      kao(:, 2, 3, 4) = (/ &
     &5.9594e-05_rb,1.3476e-03_rb,1.8940e-03_rb,2.3182e-03_rb,2.7486e-03_rb,3.0410e-03_rb, &
     &3.0839e-03_rb,3.3287e-03_rb,2.9662e-03_rb/)
      kao(:, 3, 3, 4) = (/ &
     &8.7495e-05_rb,1.6651e-03_rb,2.3108e-03_rb,2.8515e-03_rb,3.3798e-03_rb,3.7359e-03_rb, &
     &3.8327e-03_rb,4.1905e-03_rb,3.5949e-03_rb/)
      kao(:, 4, 3, 4) = (/ &
     &1.2450e-04_rb,2.0293e-03_rb,2.7849e-03_rb,3.4502e-03_rb,4.0760e-03_rb,4.5002e-03_rb, &
     &4.7432e-03_rb,5.0708e-03_rb,4.3551e-03_rb/)
      kao(:, 5, 3, 4) = (/ &
     &1.7405e-04_rb,2.4316e-03_rb,3.3182e-03_rb,4.1142e-03_rb,4.8387e-03_rb,5.3673e-03_rb, &
     &5.6716e-03_rb,6.0036e-03_rb,5.2554e-03_rb/)
      kao(:, 1, 4, 4) = (/ &
     &6.0993e-05_rb,1.0865e-03_rb,1.5064e-03_rb,1.7956e-03_rb,1.9492e-03_rb,2.0758e-03_rb, &
     &2.1376e-03_rb,1.9974e-03_rb,1.7194e-03_rb/)
      kao(:, 2, 4, 4) = (/ &
     &9.3739e-05_rb,1.3938e-03_rb,1.9090e-03_rb,2.2655e-03_rb,2.4714e-03_rb,2.6468e-03_rb, &
     &2.7998e-03_rb,2.5993e-03_rb,2.1766e-03_rb/)
      kao(:, 3, 4, 4) = (/ &
     &1.3999e-04_rb,1.7512e-03_rb,2.3888e-03_rb,2.8018e-03_rb,3.0687e-03_rb,3.3097e-03_rb, &
     &3.4825e-03_rb,3.2297e-03_rb,2.6912e-03_rb/)
      kao(:, 4, 4, 4) = (/ &
     &2.0284e-04_rb,2.1576e-03_rb,2.9507e-03_rb,3.4179e-03_rb,3.7408e-03_rb,4.0628e-03_rb, &
     &4.2589e-03_rb,4.0554e-03_rb,3.2822e-03_rb/)
      kao(:, 5, 4, 4) = (/ &
     &2.8653e-04_rb,2.6101e-03_rb,3.5917e-03_rb,4.1189e-03_rb,4.5031e-03_rb,4.8876e-03_rb, &
     &5.1085e-03_rb,4.9728e-03_rb,3.9394e-03_rb/)
      kao(:, 1, 5, 4) = (/ &
     &8.6213e-05_rb,1.0235e-03_rb,1.4463e-03_rb,1.6932e-03_rb,1.8586e-03_rb,1.8899e-03_rb, &
     &1.8325e-03_rb,1.6931e-03_rb,1.2167e-03_rb/)
      kao(:, 2, 5, 4) = (/ &
     &1.3497e-04_rb,1.3639e-03_rb,1.8867e-03_rb,2.1813e-03_rb,2.3731e-03_rb,2.4352e-03_rb, &
     &2.3757e-03_rb,2.2169e-03_rb,1.5707e-03_rb/)
      kao(:, 3, 5, 4) = (/ &
     &2.0469e-04_rb,1.7684e-03_rb,2.3970e-03_rb,2.7679e-03_rb,2.9816e-03_rb,3.0549e-03_rb, &
     &3.0098e-03_rb,2.8798e-03_rb,1.9802e-03_rb/)
      kao(:, 4, 5, 4) = (/ &
     &3.0118e-04_rb,2.2412e-03_rb,2.9952e-03_rb,3.4514e-03_rb,3.7055e-03_rb,3.7675e-03_rb, &
     &3.7358e-03_rb,3.5681e-03_rb,2.4524e-03_rb/)
      kao(:, 5, 5, 4) = (/ &
     &4.3178e-04_rb,2.7692e-03_rb,3.6699e-03_rb,4.2507e-03_rb,4.5341e-03_rb,4.5904e-03_rb, &
     &4.5606e-03_rb,4.3497e-03_rb,2.9904e-03_rb/)
      kao(:, 1, 6, 4) = (/ &
     &1.1224e-04_rb,9.6248e-04_rb,1.3486e-03_rb,1.5651e-03_rb,1.6739e-03_rb,1.7217e-03_rb, &
     &1.6200e-03_rb,1.3812e-03_rb,9.3024e-04_rb/)
      kao(:, 2, 6, 4) = (/ &
     &1.7788e-04_rb,1.2979e-03_rb,1.7963e-03_rb,2.0621e-03_rb,2.2013e-03_rb,2.2377e-03_rb, &
     &2.1278e-03_rb,1.8809e-03_rb,1.2042e-03_rb/)
      kao(:, 3, 6, 4) = (/ &
     &2.7340e-04_rb,1.7148e-03_rb,2.3323e-03_rb,2.6619e-03_rb,2.8238e-03_rb,2.8631e-03_rb, &
     &2.7284e-03_rb,2.4183e-03_rb,1.5217e-03_rb/)
      kao(:, 4, 6, 4) = (/ &
     &4.0959e-04_rb,2.2331e-03_rb,2.9678e-03_rb,3.3799e-03_rb,3.5736e-03_rb,3.5928e-03_rb, &
     &3.4271e-03_rb,3.0552e-03_rb,1.8843e-03_rb/)
      kao(:, 5, 6, 4) = (/ &
     &5.9620e-04_rb,2.8328e-03_rb,3.7008e-03_rb,4.1972e-03_rb,4.4330e-03_rb,4.4525e-03_rb, &
     &4.2185e-03_rb,3.7711e-03_rb,2.2988e-03_rb/)
      kao(:, 1, 7, 4) = (/ &
     &1.5344e-04_rb,9.2316e-04_rb,1.2773e-03_rb,1.5000e-03_rb,1.6102e-03_rb,1.6127e-03_rb, &
     &1.5115e-03_rb,1.2377e-03_rb,7.0764e-04_rb/)
      kao(:, 2, 7, 4) = (/ &
     &2.4398e-04_rb,1.2809e-03_rb,1.7477e-03_rb,2.0347e-03_rb,2.1522e-03_rb,2.1311e-03_rb, &
     &1.9950e-03_rb,1.6669e-03_rb,9.1774e-04_rb/)
      kao(:, 3, 7, 4) = (/ &
     &3.8058e-04_rb,1.7477e-03_rb,2.3440e-03_rb,2.6829e-03_rb,2.8039e-03_rb,2.7555e-03_rb, &
     &2.5876e-03_rb,2.1763e-03_rb,1.1586e-03_rb/)
      kao(:, 4, 7, 4) = (/ &
     &5.7739e-04_rb,2.3349e-03_rb,3.0698e-03_rb,3.4470e-03_rb,3.5790e-03_rb,3.5331e-03_rb, &
     &3.3054e-03_rb,2.7805e-03_rb,1.4458e-03_rb/)
      kao(:, 5, 7, 4) = (/ &
     &8.5208e-04_rb,3.0383e-03_rb,3.9002e-03_rb,4.3347e-03_rb,4.5004e-03_rb,4.4415e-03_rb, &
     &4.1236e-03_rb,3.4788e-03_rb,1.7825e-03_rb/)
      kao(:, 1, 8, 4) = (/ &
     &2.5035e-04_rb,9.8089e-04_rb,1.3026e-03_rb,1.5173e-03_rb,1.6357e-03_rb,1.6535e-03_rb, &
     &1.5386e-03_rb,1.2613e-03_rb,5.4812e-04_rb/)
      kao(:, 2, 8, 4) = (/ &
     &3.9983e-04_rb,1.4074e-03_rb,1.8431e-03_rb,2.1183e-03_rb,2.2695e-03_rb,2.2610e-03_rb, &
     &2.0892e-03_rb,1.6966e-03_rb,7.0487e-04_rb/)
      kao(:, 3, 8, 4) = (/ &
     &6.2524e-04_rb,1.9883e-03_rb,2.5535e-03_rb,2.8971e-03_rb,3.0599e-03_rb,3.0259e-03_rb, &
     &2.7534e-03_rb,2.2128e-03_rb,8.9347e-04_rb/)
      kao(:, 4, 8, 4) = (/ &
     &9.6475e-04_rb,2.7489e-03_rb,3.4725e-03_rb,3.8856e-03_rb,4.0355e-03_rb,3.9357e-03_rb, &
     &3.5484e-03_rb,2.8450e-03_rb,1.1141e-03_rb/)
      kao(:, 5, 8, 4) = (/ &
     &1.4492e-03_rb,3.7234e-03_rb,4.6023e-03_rb,5.0761e-03_rb,5.1898e-03_rb,5.0041e-03_rb, &
     &4.4962e-03_rb,3.6082e-03_rb,1.3876e-03_rb/)
      kao(:, 1, 9, 4) = (/ &
     &7.7717e-04_rb,1.6076e-03_rb,1.9327e-03_rb,2.0950e-03_rb,2.1727e-03_rb,2.1507e-03_rb, &
     &2.0239e-03_rb,1.6619e-03_rb,6.5283e-04_rb/)
      kao(:, 2, 9, 4) = (/ &
     &1.2257e-03_rb,2.4216e-03_rb,2.8205e-03_rb,3.0706e-03_rb,3.1558e-03_rb,3.0893e-03_rb, &
     &2.8639e-03_rb,2.3215e-03_rb,8.1326e-04_rb/)
      kao(:, 3, 9, 4) = (/ &
     &1.9258e-03_rb,3.5788e-03_rb,4.1331e-03_rb,4.4121e-03_rb,4.4825e-03_rb,4.3456e-03_rb, &
     &3.9751e-03_rb,3.1546e-03_rb,1.0072e-03_rb/)
      kao(:, 4, 9, 4) = (/ &
     &3.0058e-03_rb,5.1816e-03_rb,5.8926e-03_rb,6.2133e-03_rb,6.2513e-03_rb,5.9943e-03_rb, &
     &5.4160e-03_rb,4.1735e-03_rb,1.2133e-03_rb/)
      kao(:, 5, 9, 4) = (/ &
     &4.5860e-03_rb,7.3561e-03_rb,8.1694e-03_rb,8.5315e-03_rb,8.5009e-03_rb,8.0696e-03_rb, &
     &7.1597e-03_rb,5.3914e-03_rb,1.4524e-03_rb/)
      kao(:, 1,10, 4) = (/ &
     &2.7632e-03_rb,3.7578e-03_rb,3.8929e-03_rb,3.8648e-03_rb,3.7150e-03_rb,3.4386e-03_rb, &
     &2.9824e-03_rb,2.3042e-03_rb,1.1595e-03_rb/)
      kao(:, 2,10, 4) = (/ &
     &4.3618e-03_rb,5.8439e-03_rb,6.0237e-03_rb,5.9173e-03_rb,5.6198e-03_rb,5.0739e-03_rb, &
     &4.3966e-03_rb,3.3846e-03_rb,1.3310e-03_rb/)
      kao(:, 3,10, 4) = (/ &
     &6.8076e-03_rb,8.8616e-03_rb,9.0867e-03_rb,8.8297e-03_rb,8.2980e-03_rb,7.5120e-03_rb, &
     &6.4366e-03_rb,4.8357e-03_rb,1.4427e-03_rb/)
      kao(:, 4,10, 4) = (/ &
     &1.0727e-02_rb,1.3247e-02_rb,1.3386e-02_rb,1.2895e-02_rb,1.2079e-02_rb,1.0841e-02_rb, &
     &9.1826e-03_rb,6.8074e-03_rb,1.5623e-03_rb/)
      kao(:, 5,10, 4) = (/ &
     &1.6578e-02_rb,1.9641e-02_rb,1.9517e-02_rb,1.8627e-02_rb,1.7184e-02_rb,1.5227e-02_rb, &
     &1.2710e-02_rb,9.3140e-03_rb,1.8266e-03_rb/)
      kao(:, 1,11, 4) = (/ &
     &4.3397e-03_rb,5.2633e-03_rb,5.2314e-03_rb,5.0017e-03_rb,4.6314e-03_rb,4.1308e-03_rb, &
     &3.4621e-03_rb,2.5597e-03_rb,1.2582e-03_rb/)
      kao(:, 2,11, 4) = (/ &
     &6.6934e-03_rb,8.1477e-03_rb,8.0949e-03_rb,7.7188e-03_rb,7.1188e-03_rb,6.2977e-03_rb, &
     &5.1713e-03_rb,3.7779e-03_rb,1.4574e-03_rb/)
      kao(:, 3,11, 4) = (/ &
     &1.0432e-02_rb,1.2428e-02_rb,1.2196e-02_rb,1.1560e-02_rb,1.0566e-02_rb,9.2464e-03_rb, &
     &7.6156e-03_rb,5.5141e-03_rb,1.6582e-03_rb/)
      kao(:, 4,11, 4) = (/ &
     &1.6491e-02_rb,1.8759e-02_rb,1.8258e-02_rb,1.7109e-02_rb,1.5472e-02_rb,1.3463e-02_rb, &
     &1.0963e-02_rb,7.7973e-03_rb,1.8335e-03_rb/)
      kao(:, 5,11, 4) = (/ &
     &2.5430e-02_rb,2.8078e-02_rb,2.7002e-02_rb,2.5032e-02_rb,2.2392e-02_rb,1.9261e-02_rb, &
     &1.5474e-02_rb,1.0784e-02_rb,2.0075e-03_rb/)
      kao(:, 1,12, 4) = (/ &
     &4.9273e-03_rb,5.7931e-03_rb,5.6746e-03_rb,5.3611e-03_rb,4.8847e-03_rb,4.2709e-03_rb, &
     &3.4855e-03_rb,2.4889e-03_rb,1.1526e-03_rb/)
      kao(:, 2,12, 4) = (/ &
     &7.6884e-03_rb,8.9132e-03_rb,8.6777e-03_rb,8.1782e-03_rb,7.4327e-03_rb,6.4648e-03_rb, &
     &5.2372e-03_rb,3.6981e-03_rb,1.3373e-03_rb/)
      kao(:, 3,12, 4) = (/ &
     &1.2107e-02_rb,1.3711e-02_rb,1.3177e-02_rb,1.2250e-02_rb,1.1036e-02_rb,9.5160e-03_rb, &
     &7.6801e-03_rb,5.4155e-03_rb,1.5194e-03_rb/)
      kao(:, 4,12, 4) = (/ &
     &1.8869e-02_rb,2.0811e-02_rb,1.9833e-02_rb,1.8325e-02_rb,1.6361e-02_rb,1.3983e-02_rb, &
     &1.1160e-02_rb,7.6837e-03_rb,1.7468e-03_rb/)
      kao(:, 5,12, 4) = (/ &
     &2.9063e-02_rb,3.1131e-02_rb,2.9461e-02_rb,2.6993e-02_rb,2.3838e-02_rb,2.0192e-02_rb, &
     &1.5924e-02_rb,1.0737e-02_rb,1.9884e-03_rb/)
      kao(:, 1,13, 4) = (/ &
     &4.3869e-03_rb,5.1798e-03_rb,5.0989e-03_rb,4.8169e-03_rb,4.3794e-03_rb,3.7839e-03_rb, &
     &3.0521e-03_rb,2.1587e-03_rb,9.4784e-04_rb/)
      kao(:, 2,13, 4) = (/ &
     &6.9481e-03_rb,8.0525e-03_rb,7.8458e-03_rb,7.3314e-03_rb,6.6078e-03_rb,5.7100e-03_rb, &
     &4.5663e-03_rb,3.2136e-03_rb,1.0961e-03_rb/)
      kao(:, 3,13, 4) = (/ &
     &1.1021e-02_rb,1.2458e-02_rb,1.1977e-02_rb,1.1021e-02_rb,9.8258e-03_rb,8.4083e-03_rb, &
     &6.7419e-03_rb,4.6947e-03_rb,1.2238e-03_rb/)
      kao(:, 4,13, 4) = (/ &
     &1.7310e-02_rb,1.8923e-02_rb,1.8001e-02_rb,1.6509e-02_rb,1.4657e-02_rb,1.2419e-02_rb, &
     &9.8548e-03_rb,6.6916e-03_rb,1.4326e-03_rb/)
      kao(:, 5,13, 4) = (/ &
     &2.6619e-02_rb,2.8296e-02_rb,2.6623e-02_rb,2.4187e-02_rb,2.1294e-02_rb,1.7939e-02_rb, &
     &1.4103e-02_rb,9.4141e-03_rb,1.6712e-03_rb/)
      kao(:, 1, 1, 5) = (/ &
     &1.5033e-04_rb,2.0446e-03_rb,3.1681e-03_rb,4.0928e-03_rb,4.9645e-03_rb,5.8752e-03_rb, &
     &6.8891e-03_rb,9.7460e-03_rb,8.2375e-03_rb/)
      kao(:, 2, 1, 5) = (/ &
     &2.0389e-04_rb,2.4966e-03_rb,3.8465e-03_rb,4.9929e-03_rb,6.1245e-03_rb,7.2940e-03_rb, &
     &8.7776e-03_rb,1.1499e-02_rb,1.0182e-02_rb/)
      kao(:, 3, 1, 5) = (/ &
     &2.6769e-04_rb,3.0079e-03_rb,4.6076e-03_rb,5.9938e-03_rb,7.3703e-03_rb,8.8250e-03_rb, &
     &1.0916e-02_rb,1.3468e-02_rb,1.2268e-02_rb/)
      kao(:, 4, 1, 5) = (/ &
     &3.4024e-04_rb,3.5831e-03_rb,5.4751e-03_rb,7.1124e-03_rb,8.7423e-03_rb,1.0600e-02_rb, &
     &1.2985e-02_rb,1.6757e-02_rb,1.4652e-02_rb/)
      kao(:, 5, 1, 5) = (/ &
     &4.2020e-04_rb,4.2180e-03_rb,6.4294e-03_rb,8.3453e-03_rb,1.0248e-02_rb,1.2449e-02_rb, &
     &1.5350e-02_rb,1.9955e-02_rb,1.7214e-02_rb/)
      kao(:, 1, 2, 5) = (/ &
     &1.8399e-04_rb,1.8419e-03_rb,2.8498e-03_rb,3.5822e-03_rb,4.2442e-03_rb,4.8814e-03_rb, &
     &5.3141e-03_rb,6.9582e-03_rb,6.1890e-03_rb/)
      kao(:, 2, 2, 5) = (/ &
     &2.5292e-04_rb,2.2771e-03_rb,3.4958e-03_rb,4.3938e-03_rb,5.2283e-03_rb,6.0330e-03_rb, &
     &6.8353e-03_rb,8.0230e-03_rb,7.6976e-03_rb/)
      kao(:, 3, 2, 5) = (/ &
     &3.3540e-04_rb,2.7711e-03_rb,4.2257e-03_rb,5.3109e-03_rb,6.3352e-03_rb,7.3450e-03_rb, &
     &8.3567e-03_rb,1.0142e-02_rb,9.3716e-03_rb/)
      kao(:, 4, 2, 5) = (/ &
     &4.2928e-04_rb,3.3267e-03_rb,5.0595e-03_rb,6.3597e-03_rb,7.5719e-03_rb,8.7753e-03_rb, &
     &1.0137e-02_rb,1.2487e-02_rb,1.1226e-02_rb/)
      kao(:, 5, 2, 5) = (/ &
     &5.3326e-04_rb,3.9397e-03_rb,5.9807e-03_rb,7.5174e-03_rb,8.9383e-03_rb,1.0365e-02_rb, &
     &1.1992e-02_rb,1.4789e-02_rb,1.3251e-02_rb/)
      kao(:, 1, 3, 5) = (/ &
     &2.8126e-04_rb,1.8765e-03_rb,2.6588e-03_rb,3.3485e-03_rb,3.8221e-03_rb,4.1636e-03_rb, &
     &4.4682e-03_rb,4.4846e-03_rb,4.3919e-03_rb/)
      kao(:, 2, 3, 5) = (/ &
     &3.9449e-04_rb,2.3306e-03_rb,3.3231e-03_rb,4.1650e-03_rb,4.7241e-03_rb,5.1684e-03_rb, &
     &5.5884e-03_rb,5.9167e-03_rb,5.5074e-03_rb/)
      kao(:, 3, 3, 5) = (/ &
     &5.3269e-04_rb,2.8500e-03_rb,4.0835e-03_rb,5.0834e-03_rb,5.7677e-03_rb,6.3235e-03_rb, &
     &6.8829e-03_rb,7.4745e-03_rb,6.8347e-03_rb/)
      kao(:, 4, 3, 5) = (/ &
     &6.9304e-04_rb,3.4418e-03_rb,4.9465e-03_rb,6.1263e-03_rb,6.9668e-03_rb,7.6366e-03_rb, &
     &8.3133e-03_rb,9.1010e-03_rb,8.2831e-03_rb/)
      kao(:, 5, 3, 5) = (/ &
     &8.7133e-04_rb,4.1178e-03_rb,5.9088e-03_rb,7.3026e-03_rb,8.3125e-03_rb,9.0996e-03_rb, &
     &9.9113e-03_rb,1.0888e-02_rb,9.8539e-03_rb/)
      kao(:, 1, 4, 5) = (/ &
     &4.2942e-04_rb,2.1209e-03_rb,2.6611e-03_rb,3.0954e-03_rb,3.5092e-03_rb,3.7795e-03_rb, &
     &3.7536e-03_rb,3.6803e-03_rb,3.1833e-03_rb/)
      kao(:, 2, 4, 5) = (/ &
     &6.1278e-04_rb,2.6801e-03_rb,3.3648e-03_rb,3.9108e-03_rb,4.4289e-03_rb,4.7305e-03_rb, &
     &4.7677e-03_rb,4.7187e-03_rb,4.0112e-03_rb/)
      kao(:, 3, 4, 5) = (/ &
     &8.4431e-04_rb,3.3316e-03_rb,4.1648e-03_rb,4.8597e-03_rb,5.4829e-03_rb,5.8484e-03_rb, &
     &5.9135e-03_rb,5.9451e-03_rb,4.9891e-03_rb/)
      kao(:, 4, 4, 5) = (/ &
     &1.1175e-03_rb,4.0506e-03_rb,5.0747e-03_rb,5.9647e-03_rb,6.6966e-03_rb,7.1102e-03_rb, &
     &7.2127e-03_rb,7.2668e-03_rb,6.0794e-03_rb/)
      kao(:, 5, 4, 5) = (/ &
     &1.4334e-03_rb,4.8948e-03_rb,6.1056e-03_rb,7.2069e-03_rb,8.0737e-03_rb,8.5555e-03_rb, &
     &8.6934e-03_rb,8.7560e-03_rb,7.1896e-03_rb/)
      kao(:, 1, 5, 5) = (/ &
     &6.0926e-04_rb,2.3529e-03_rb,2.8355e-03_rb,3.0797e-03_rb,3.2149e-03_rb,3.3400e-03_rb, &
     &3.3910e-03_rb,2.9801e-03_rb,2.2681e-03_rb/)
      kao(:, 2, 5, 5) = (/ &
     &8.8131e-04_rb,3.0196e-03_rb,3.6353e-03_rb,3.9718e-03_rb,4.1316e-03_rb,4.2694e-03_rb, &
     &4.3037e-03_rb,3.9012e-03_rb,2.9392e-03_rb/)
      kao(:, 3, 5, 5) = (/ &
     &1.2303e-03_rb,3.8066e-03_rb,4.5787e-03_rb,4.9631e-03_rb,5.1972e-03_rb,5.3645e-03_rb, &
     &5.3804e-03_rb,4.9263e-03_rb,3.6983e-03_rb/)
      kao(:, 4, 5, 5) = (/ &
     &1.6619e-03_rb,4.7363e-03_rb,5.6662e-03_rb,6.1253e-03_rb,6.4225e-03_rb,6.6560e-03_rb, &
     &6.6051e-03_rb,6.0844e-03_rb,4.5482e-03_rb/)
      kao(:, 5, 5, 5) = (/ &
     &2.1681e-03_rb,5.8273e-03_rb,6.8808e-03_rb,7.4389e-03_rb,7.8269e-03_rb,8.1204e-03_rb, &
     &8.0166e-03_rb,7.4003e-03_rb,5.4950e-03_rb/)
      kao(:, 1, 6, 5) = (/ &
     &8.3319e-04_rb,2.4541e-03_rb,2.9309e-03_rb,3.1239e-03_rb,3.1232e-03_rb,3.0304e-03_rb, &
     &2.8953e-03_rb,2.6232e-03_rb,1.6854e-03_rb/)
      kao(:, 2, 6, 5) = (/ &
     &1.2038e-03_rb,3.2544e-03_rb,3.8397e-03_rb,4.0740e-03_rb,4.0835e-03_rb,3.9879e-03_rb, &
     &3.7753e-03_rb,3.4082e-03_rb,2.1468e-03_rb/)
      kao(:, 3, 6, 5) = (/ &
     &1.6852e-03_rb,4.1990e-03_rb,4.9217e-03_rb,5.1945e-03_rb,5.2325e-03_rb,5.0738e-03_rb, &
     &4.7918e-03_rb,4.2939e-03_rb,2.6825e-03_rb/)
      kao(:, 4, 6, 5) = (/ &
     &2.3055e-03_rb,5.2874e-03_rb,6.1886e-03_rb,6.5148e-03_rb,6.5533e-03_rb,6.3614e-03_rb, &
     &5.9949e-03_rb,5.3440e-03_rb,3.2990e-03_rb/)
      kao(:, 5, 6, 5) = (/ &
     &3.0550e-03_rb,6.5592e-03_rb,7.6795e-03_rb,8.0591e-03_rb,8.0426e-03_rb,7.8256e-03_rb, &
     &7.4185e-03_rb,6.5412e-03_rb,4.0170e-03_rb/)
      kao(:, 1, 7, 5) = (/ &
     &1.2030e-03_rb,2.7895e-03_rb,3.1932e-03_rb,3.3085e-03_rb,3.2497e-03_rb,3.0373e-03_rb, &
     &2.6860e-03_rb,2.2491e-03_rb,1.3507e-03_rb/)
      kao(:, 2, 7, 5) = (/ &
     &1.7561e-03_rb,3.7366e-03_rb,4.1989e-03_rb,4.3780e-03_rb,4.3275e-03_rb,4.0454e-03_rb, &
     &3.5823e-03_rb,2.9962e-03_rb,1.7424e-03_rb/)
      kao(:, 3, 7, 5) = (/ &
     &2.4695e-03_rb,4.8641e-03_rb,5.4890e-03_rb,5.7046e-03_rb,5.6266e-03_rb,5.2456e-03_rb, &
     &4.6755e-03_rb,3.8694e-03_rb,2.1970e-03_rb/)
      kao(:, 4, 7, 5) = (/ &
     &3.4036e-03_rb,6.2572e-03_rb,7.0247e-03_rb,7.3182e-03_rb,7.1220e-03_rb,6.6595e-03_rb, &
     &5.9470e-03_rb,4.8905e-03_rb,2.7089e-03_rb/)
      kao(:, 5, 7, 5) = (/ &
     &4.5461e-03_rb,7.9331e-03_rb,8.8637e-03_rb,9.1326e-03_rb,8.9102e-03_rb,8.3275e-03_rb, &
     &7.4120e-03_rb,6.0989e-03_rb,3.2778e-03_rb/)
      kao(:, 1, 8, 5) = (/ &
     &2.0449e-03_rb,3.7053e-03_rb,4.0860e-03_rb,4.1499e-03_rb,3.9922e-03_rb,3.6227e-03_rb, &
     &3.0421e-03_rb,2.2282e-03_rb,1.0954e-03_rb/)
      kao(:, 2, 8, 5) = (/ &
     &3.0260e-03_rb,5.1232e-03_rb,5.5625e-03_rb,5.5873e-03_rb,5.3321e-03_rb,4.8458e-03_rb, &
     &4.0703e-03_rb,3.0319e-03_rb,1.4218e-03_rb/)
      kao(:, 3, 8, 5) = (/ &
     &4.3499e-03_rb,6.8685e-03_rb,7.3484e-03_rb,7.3183e-03_rb,6.9565e-03_rb,6.3086e-03_rb, &
     &5.3805e-03_rb,3.9991e-03_rb,1.7827e-03_rb/)
      kao(:, 4, 8, 5) = (/ &
     &6.0988e-03_rb,9.0048e-03_rb,9.4728e-03_rb,9.3798e-03_rb,8.9487e-03_rb,8.1669e-03_rb, &
     &6.9162e-03_rb,5.1727e-03_rb,2.2090e-03_rb/)
      kao(:, 5, 8, 5) = (/ &
     &8.2301e-03_rb,1.1592e-02_rb,1.2058e-02_rb,1.1936e-02_rb,1.1386e-02_rb,1.0345e-02_rb, &
     &8.7069e-03_rb,6.5415e-03_rb,2.6879e-03_rb/)
      kao(:, 1, 9, 5) = (/ &
     &6.4341e-03_rb,8.4932e-03_rb,8.6319e-03_rb,8.3108e-03_rb,7.7018e-03_rb,6.7763e-03_rb, &
     &5.5115e-03_rb,3.7767e-03_rb,9.5475e-04_rb/)
      kao(:, 2, 9, 5) = (/ &
     &9.6759e-03_rb,1.2213e-02_rb,1.2266e-02_rb,1.1725e-02_rb,1.0741e-02_rb,9.3855e-03_rb, &
     &7.5654e-03_rb,5.1517e-03_rb,1.1933e-03_rb/)
      kao(:, 3, 9, 5) = (/ &
     &1.4274e-02_rb,1.7116e-02_rb,1.6970e-02_rb,1.6030e-02_rb,1.4545e-02_rb,1.2624e-02_rb, &
     &1.0092e-02_rb,6.8273e-03_rb,1.4956e-03_rb/)
      kao(:, 4, 9, 5) = (/ &
     &2.0348e-02_rb,2.3456e-02_rb,2.2925e-02_rb,2.1415e-02_rb,1.9260e-02_rb,1.6502e-02_rb, &
     &1.3098e-02_rb,8.8597e-03_rb,1.8913e-03_rb/)
      kao(:, 5, 9, 5) = (/ &
     &2.7947e-02_rb,3.1392e-02_rb,3.0350e-02_rb,2.8042e-02_rb,2.4920e-02_rb,2.1171e-02_rb, &
     &1.6720e-02_rb,1.1275e-02_rb,2.3159e-03_rb/)
      kao(:, 1,10, 5) = (/ &
     &2.3218e-02_rb,2.4547e-02_rb,2.3456e-02_rb,2.1452e-02_rb,1.8855e-02_rb,1.5703e-02_rb, &
     &1.2047e-02_rb,7.7884e-03_rb,1.4115e-03_rb/)
      kao(:, 2,10, 5) = (/ &
     &3.5352e-02_rb,3.6479e-02_rb,3.4471e-02_rb,3.1299e-02_rb,2.7239e-02_rb,2.2668e-02_rb, &
     &1.7318e-02_rb,1.0990e-02_rb,1.7428e-03_rb/)
      kao(:, 3,10, 5) = (/ &
     &5.3041e-02_rb,5.3290e-02_rb,4.9753e-02_rb,4.4634e-02_rb,3.8593e-02_rb,3.1830e-02_rb, &
     &2.4004e-02_rb,1.5032e-02_rb,2.1711e-03_rb/)
      kao(:, 4,10, 5) = (/ &
     &7.6819e-02_rb,7.5905e-02_rb,7.0118e-02_rb,6.2386e-02_rb,5.3448e-02_rb,4.3595e-02_rb, &
     &3.2572e-02_rb,2.0010e-02_rb,2.6405e-03_rb/)
      kao(:, 5,10, 5) = (/ &
     &1.0741e-01_rb,1.0446e-01_rb,9.5780e-02_rb,8.4676e-02_rb,7.2117e-02_rb,5.8406e-02_rb, &
     &4.3338e-02_rb,2.6148e-02_rb,3.0152e-03_rb/)
      kao(:, 1,11, 5) = (/ &
     &3.5267e-02_rb,3.5419e-02_rb,3.3018e-02_rb,2.9652e-02_rb,2.5661e-02_rb,2.1081e-02_rb, &
     &1.5812e-02_rb,9.8203e-03_rb,1.5039e-03_rb/)
      kao(:, 2,11, 5) = (/ &
     &5.4088e-02_rb,5.3137e-02_rb,4.9102e-02_rb,4.3719e-02_rb,3.7498e-02_rb,3.0622e-02_rb, &
     &2.2914e-02_rb,1.4036e-02_rb,1.8588e-03_rb/)
      kao(:, 3,11, 5) = (/ &
     &8.1309e-02_rb,7.8515e-02_rb,7.1873e-02_rb,6.3455e-02_rb,5.4008e-02_rb,4.3709e-02_rb, &
     &3.2277e-02_rb,1.9424e-02_rb,2.2498e-03_rb/)
      kao(:, 4,11, 5) = (/ &
     &1.1800e-01_rb,1.1244e-01_rb,1.0217e-01_rb,8.9567e-02_rb,7.5664e-02_rb,6.0559e-02_rb, &
     &4.4387e-02_rb,2.6353e-02_rb,2.7248e-03_rb/)
      kao(:, 5,11, 5) = (/ &
     &1.6472e-01_rb,1.5538e-01_rb,1.4035e-01_rb,1.2265e-01_rb,1.0315e-01_rb,8.2131e-02_rb, &
     &5.9771e-02_rb,3.4979e-02_rb,3.2176e-03_rb/)
      kao(:, 1,12, 5) = (/ &
     &3.9647e-02_rb,3.8955e-02_rb,3.5866e-02_rb,3.1882e-02_rb,2.7372e-02_rb,2.2283e-02_rb, &
     &1.6581e-02_rb,1.0137e-02_rb,1.4428e-03_rb/)
      kao(:, 2,12, 5) = (/ &
     &6.1147e-02_rb,5.9154e-02_rb,5.3991e-02_rb,4.7633e-02_rb,4.0515e-02_rb,3.2821e-02_rb, &
     &2.4231e-02_rb,1.4611e-02_rb,1.7795e-03_rb/)
      kao(:, 3,12, 5) = (/ &
     &9.1962e-02_rb,8.7537e-02_rb,7.9486e-02_rb,6.9739e-02_rb,5.8957e-02_rb,4.7222e-02_rb, &
     &3.4488e-02_rb,2.0425e-02_rb,2.1548e-03_rb/)
      kao(:, 4,12, 5) = (/ &
     &1.3375e-01_rb,1.2593e-01_rb,1.1362e-01_rb,9.8983e-02_rb,8.2938e-02_rb,6.5961e-02_rb, &
     &4.7849e-02_rb,2.7977e-02_rb,2.5823e-03_rb/)
      kao(:, 5,12, 5) = (/ &
     &1.8619e-01_rb,1.7389e-01_rb,1.5614e-01_rb,1.3563e-01_rb,1.1356e-01_rb,8.9900e-02_rb, &
     &6.4743e-02_rb,3.7437e-02_rb,3.0416e-03_rb/)
      kao(:, 1,13, 5) = (/ &
     &3.5348e-02_rb,3.4594e-02_rb,3.1651e-02_rb,2.8032e-02_rb,2.3972e-02_rb,1.9476e-02_rb, &
     &1.4520e-02_rb,8.8483e-03_rb,1.2073e-03_rb/)
      kao(:, 2,13, 5) = (/ &
     &5.4657e-02_rb,5.2717e-02_rb,4.7880e-02_rb,4.2169e-02_rb,3.5919e-02_rb,2.9028e-02_rb, &
     &2.1424e-02_rb,1.2832e-02_rb,1.4997e-03_rb/)
      kao(:, 3,13, 5) = (/ &
     &8.2593e-02_rb,7.8668e-02_rb,7.1038e-02_rb,6.2226e-02_rb,5.2555e-02_rb,4.1980e-02_rb, &
     &3.0595e-02_rb,1.8057e-02_rb,1.8484e-03_rb/)
      kao(:, 4,13, 5) = (/ &
     &1.1968e-01_rb,1.1269e-01_rb,1.0134e-01_rb,8.8277e-02_rb,7.4125e-02_rb,5.9019e-02_rb, &
     &4.2726e-02_rb,2.4879e-02_rb,2.2113e-03_rb/)
      kao(:, 5,13, 5) = (/ &
     &1.6694e-01_rb,1.5606e-01_rb,1.3993e-01_rb,1.2146e-01_rb,1.0158e-01_rb,8.0368e-02_rb, &
     &5.7742e-02_rb,3.3361e-02_rb,2.6209e-03_rb/)
      kao(:, 1, 1, 6) = (/ &
     &5.8710e-04_rb,3.7447e-03_rb,5.6598e-03_rb,7.4215e-03_rb,9.2308e-03_rb,1.1302e-02_rb, &
     &1.3745e-02_rb,1.7130e-02_rb,1.6376e-02_rb/)
      kao(:, 2, 1, 6) = (/ &
     &7.1095e-04_rb,4.5295e-03_rb,6.8956e-03_rb,9.1329e-03_rb,1.1441e-02_rb,1.4069e-02_rb, &
     &1.7260e-02_rb,2.1984e-02_rb,2.0159e-02_rb/)
      kao(:, 3, 1, 6) = (/ &
     &8.4824e-04_rb,5.3994e-03_rb,8.2567e-03_rb,1.1010e-02_rb,1.3909e-02_rb,1.7204e-02_rb, &
     &2.1091e-02_rb,2.7132e-02_rb,2.4524e-02_rb/)
      kao(:, 4, 1, 6) = (/ &
     &9.9962e-04_rb,6.3675e-03_rb,9.7929e-03_rb,1.3116e-02_rb,1.6664e-02_rb,2.0670e-02_rb, &
     &2.5520e-02_rb,3.3009e-02_rb,2.9504e-02_rb/)
      kao(:, 5, 1, 6) = (/ &
     &1.1677e-03_rb,7.4449e-03_rb,1.1517e-02_rb,1.5476e-02_rb,1.9740e-02_rb,2.4530e-02_rb, &
     &3.0349e-02_rb,3.9385e-02_rb,3.5028e-02_rb/)
      kao(:, 1, 2, 6) = (/ &
     &7.3151e-04_rb,3.6448e-03_rb,5.2016e-03_rb,6.6421e-03_rb,7.9400e-03_rb,9.1018e-03_rb, &
     &1.0516e-02_rb,1.2549e-02_rb,1.2514e-02_rb/)
      kao(:, 2, 2, 6) = (/ &
     &8.8990e-04_rb,4.4216e-03_rb,6.3659e-03_rb,8.1663e-03_rb,9.8302e-03_rb,1.1392e-02_rb, &
     &1.3321e-02_rb,1.6283e-02_rb,1.5469e-02_rb/)
      kao(:, 3, 2, 6) = (/ &
     &1.0714e-03_rb,5.2883e-03_rb,7.6564e-03_rb,9.8457e-03_rb,1.1909e-02_rb,1.3940e-02_rb, &
     &1.6405e-02_rb,2.0166e-02_rb,1.8847e-02_rb/)
      kao(:, 4, 2, 6) = (/ &
     &1.2782e-03_rb,6.2637e-03_rb,9.0995e-03_rb,1.1738e-02_rb,1.4242e-02_rb,1.6761e-02_rb, &
     &1.9871e-02_rb,2.4604e-02_rb,2.2753e-02_rb/)
      kao(:, 5, 2, 6) = (/ &
     &1.5101e-03_rb,7.3491e-03_rb,1.0728e-02_rb,1.3865e-02_rb,1.6850e-02_rb,1.9892e-02_rb, &
     &2.3687e-02_rb,2.9581e-02_rb,2.7171e-02_rb/)
      kao(:, 1, 3, 6) = (/ &
     &1.1614e-03_rb,3.9286e-03_rb,5.2992e-03_rb,6.2920e-03_rb,7.1803e-03_rb,7.9571e-03_rb, &
     &8.5455e-03_rb,9.0349e-03_rb,9.0567e-03_rb/)
      kao(:, 2, 3, 6) = (/ &
     &1.4526e-03_rb,4.8338e-03_rb,6.5076e-03_rb,7.7704e-03_rb,8.9197e-03_rb,9.9629e-03_rb, &
     &1.0799e-02_rb,1.1625e-02_rb,1.1298e-02_rb/)
      kao(:, 3, 3, 6) = (/ &
     &1.7725e-03_rb,5.8343e-03_rb,7.8680e-03_rb,9.4298e-03_rb,1.0866e-02_rb,1.2179e-02_rb, &
     &1.3264e-02_rb,1.4525e-02_rb,1.3847e-02_rb/)
      kao(:, 4, 3, 6) = (/ &
     &2.1368e-03_rb,6.9576e-03_rb,9.3967e-03_rb,1.1298e-02_rb,1.3038e-02_rb,1.4633e-02_rb, &
     &1.6013e-02_rb,1.7774e-02_rb,1.6807e-02_rb/)
      kao(:, 5, 3, 6) = (/ &
     &2.5530e-03_rb,8.2213e-03_rb,1.1119e-02_rb,1.3386e-02_rb,1.5467e-02_rb,1.7396e-02_rb, &
     &1.9105e-02_rb,2.1405e-02_rb,2.0198e-02_rb/)
      kao(:, 1, 4, 6) = (/ &
     &1.8393e-03_rb,4.2719e-03_rb,5.6108e-03_rb,6.4238e-03_rb,6.8867e-03_rb,7.1924e-03_rb, &
     &7.4060e-03_rb,7.2862e-03_rb,6.3156e-03_rb/)
      kao(:, 2, 4, 6) = (/ &
     &2.3510e-03_rb,5.3330e-03_rb,6.9923e-03_rb,7.9801e-03_rb,8.5829e-03_rb,9.0352e-03_rb, &
     &9.3670e-03_rb,9.3790e-03_rb,7.9787e-03_rb/)
      kao(:, 3, 4, 6) = (/ &
     &2.9328e-03_rb,6.5477e-03_rb,8.5524e-03_rb,9.7522e-03_rb,1.0523e-02_rb,1.1105e-02_rb, &
     &1.1571e-02_rb,1.1684e-02_rb,9.9124e-03_rb/)
      kao(:, 4, 4, 6) = (/ &
     &3.6136e-03_rb,7.9708e-03_rb,1.0306e-02_rb,1.1748e-02_rb,1.2708e-02_rb,1.3440e-02_rb, &
     &1.4032e-02_rb,1.4271e-02_rb,1.2172e-02_rb/)
      kao(:, 5, 4, 6) = (/ &
     &4.3442e-03_rb,9.5761e-03_rb,1.2277e-02_rb,1.3997e-02_rb,1.5163e-02_rb,1.6053e-02_rb, &
     &1.6795e-02_rb,1.7163e-02_rb,1.4922e-02_rb/)
      kao(:, 1, 5, 6) = (/ &
     &2.6688e-03_rb,4.8052e-03_rb,5.7832e-03_rb,6.4797e-03_rb,6.8502e-03_rb,6.8218e-03_rb, &
     &6.5242e-03_rb,6.0722e-03_rb,4.6753e-03_rb/)
      kao(:, 2, 5, 6) = (/ &
     &3.4688e-03_rb,6.1297e-03_rb,7.3230e-03_rb,8.1782e-03_rb,8.6322e-03_rb,8.6195e-03_rb, &
     &8.3126e-03_rb,7.8540e-03_rb,5.9385e-03_rb/)
      kao(:, 3, 5, 6) = (/ &
     &4.4058e-03_rb,7.6630e-03_rb,9.1013e-03_rb,1.0147e-02_rb,1.0667e-02_rb,1.0680e-02_rb, &
     &1.0355e-02_rb,9.8543e-03_rb,7.3828e-03_rb/)
      kao(:, 4, 5, 6) = (/ &
     &5.5115e-03_rb,9.4361e-03_rb,1.1163e-02_rb,1.2385e-02_rb,1.2986e-02_rb,1.3010e-02_rb, &
     &1.2662e-02_rb,1.2099e-02_rb,9.0803e-03_rb/)
      kao(:, 5, 5, 6) = (/ &
     &6.7862e-03_rb,1.1482e-02_rb,1.3563e-02_rb,1.4942e-02_rb,1.5588e-02_rb,1.5643e-02_rb, &
     &1.5261e-02_rb,1.4627e-02_rb,1.1026e-02_rb/)
      kao(:, 1, 6, 6) = (/ &
     &3.5724e-03_rb,5.4370e-03_rb,6.0294e-03_rb,6.3914e-03_rb,6.6047e-03_rb,6.5232e-03_rb, &
     &6.0277e-03_rb,5.0661e-03_rb,3.1530e-03_rb/)
      kao(:, 2, 6, 6) = (/ &
     &4.7182e-03_rb,7.1382e-03_rb,7.8365e-03_rb,8.2706e-03_rb,8.5053e-03_rb,8.3495e-03_rb, &
     &7.7282e-03_rb,6.6198e-03_rb,4.1407e-03_rb/)
      kao(:, 3, 6, 6) = (/ &
     &6.1150e-03_rb,9.1981e-03_rb,9.9430e-03_rb,1.0464e-02_rb,1.0671e-02_rb,1.0482e-02_rb, &
     &9.6931e-03_rb,8.3868e-03_rb,5.3331e-03_rb/)
      kao(:, 4, 6, 6) = (/ &
     &7.7447e-03_rb,1.1494e-02_rb,1.2445e-02_rb,1.3024e-02_rb,1.3207e-02_rb,1.2894e-02_rb, &
     &1.1946e-02_rb,1.0395e-02_rb,6.6921e-03_rb/)
      kao(:, 5, 6, 6) = (/ &
     &9.6796e-03_rb,1.4190e-02_rb,1.5330e-02_rb,1.5967e-02_rb,1.6158e-02_rb,1.5661e-02_rb, &
     &1.4492e-02_rb,1.2674e-02_rb,8.2498e-03_rb/)
      kao(:, 1, 7, 6) = (/ &
     &4.9616e-03_rb,6.6121e-03_rb,6.8416e-03_rb,6.8101e-03_rb,6.6768e-03_rb,6.3696e-03_rb, &
     &5.8141e-03_rb,4.6733e-03_rb,2.5227e-03_rb/)
      kao(:, 2, 7, 6) = (/ &
     &6.6954e-03_rb,8.8189e-03_rb,9.1778e-03_rb,9.1281e-03_rb,8.8939e-03_rb,8.4196e-03_rb, &
     &7.6478e-03_rb,6.1227e-03_rb,3.2129e-03_rb/)
      kao(:, 3, 7, 6) = (/ &
     &8.9023e-03_rb,1.1584e-02_rb,1.1984e-02_rb,1.1899e-02_rb,1.1446e-02_rb,1.0830e-02_rb, &
     &9.7223e-03_rb,7.7935e-03_rb,4.0071e-03_rb/)
      kao(:, 4, 7, 6) = (/ &
     &1.1382e-02_rb,1.4883e-02_rb,1.5340e-02_rb,1.5077e-02_rb,1.4572e-02_rb,1.3613e-02_rb, &
     &1.2124e-02_rb,9.7242e-03_rb,4.9238e-03_rb/)
      kao(:, 5, 7, 6) = (/ &
     &1.4371e-02_rb,1.8723e-02_rb,1.9154e-02_rb,1.8910e-02_rb,1.8193e-02_rb,1.6854e-02_rb, &
     &1.4917e-02_rb,1.1939e-02_rb,6.0317e-03_rb/)
      kao(:, 1, 8, 6) = (/ &
     &8.7001e-03_rb,9.7747e-03_rb,9.5071e-03_rb,8.9379e-03_rb,8.2227e-03_rb,7.2962e-03_rb, &
     &6.1868e-03_rb,4.7953e-03_rb,2.0794e-03_rb/)
      kao(:, 2, 8, 6) = (/ &
     &1.1618e-02_rb,1.3178e-02_rb,1.2958e-02_rb,1.2245e-02_rb,1.1180e-02_rb,9.8759e-03_rb, &
     &8.4959e-03_rb,6.4689e-03_rb,2.6798e-03_rb/)
      kao(:, 3, 8, 6) = (/ &
     &1.5351e-02_rb,1.7665e-02_rb,1.7319e-02_rb,1.6267e-02_rb,1.4838e-02_rb,1.3188e-02_rb, &
     &1.1167e-02_rb,8.4425e-03_rb,3.4113e-03_rb/)
      kao(:, 4, 8, 6) = (/ &
     &2.0092e-02_rb,2.2911e-02_rb,2.2431e-02_rb,2.1181e-02_rb,1.9393e-02_rb,1.7119e-02_rb, &
     &1.4369e-02_rb,1.0695e-02_rb,4.2567e-03_rb/)
      kao(:, 5, 8, 6) = (/ &
     &2.5625e-02_rb,2.9136e-02_rb,2.8640e-02_rb,2.6980e-02_rb,2.4612e-02_rb,2.1703e-02_rb, &
     &1.8202e-02_rb,1.3310e-02_rb,5.2101e-03_rb/)
      kao(:, 1, 9, 6) = (/ &
     &2.8411e-02_rb,2.7805e-02_rb,2.5436e-02_rb,2.2677e-02_rb,1.9516e-02_rb,1.5867e-02_rb, &
     &1.1958e-02_rb,7.5969e-03_rb,2.0612e-03_rb/)
      kao(:, 2, 9, 6) = (/ &
     &3.8814e-02_rb,3.7908e-02_rb,3.4867e-02_rb,3.0948e-02_rb,2.6641e-02_rb,2.1668e-02_rb, &
     &1.6345e-02_rb,1.0429e-02_rb,2.5903e-03_rb/)
      kao(:, 3, 9, 6) = (/ &
     &5.1537e-02_rb,5.0347e-02_rb,4.6262e-02_rb,4.1144e-02_rb,3.5324e-02_rb,2.8924e-02_rb, &
     &2.1923e-02_rb,1.4064e-02_rb,3.2116e-03_rb/)
      kao(:, 4, 9, 6) = (/ &
     &6.6909e-02_rb,6.5234e-02_rb,6.0364e-02_rb,5.3943e-02_rb,4.6443e-02_rb,3.8124e-02_rb, &
     &2.8947e-02_rb,1.8549e-02_rb,3.8623e-03_rb/)
      kao(:, 5, 9, 6) = (/ &
     &8.6078e-02_rb,8.4018e-02_rb,7.7843e-02_rb,6.9506e-02_rb,5.9853e-02_rb,4.9159e-02_rb, &
     &3.7146e-02_rb,2.3977e-02_rb,4.6095e-03_rb/)
      kao(:, 1,10, 6) = (/ &
     &1.0479e-01_rb,9.4804e-02_rb,8.4158e-02_rb,7.2419e-02_rb,5.9897e-02_rb,4.7014e-02_rb, &
     &3.3513e-02_rb,1.8915e-02_rb,2.3959e-03_rb/)
      kao(:, 2,10, 6) = (/ &
     &1.4593e-01_rb,1.3214e-01_rb,1.1705e-01_rb,1.0074e-01_rb,8.3547e-02_rb,6.5542e-02_rb, &
     &4.6772e-02_rb,2.6375e-02_rb,2.8756e-03_rb/)
      kao(:, 3,10, 6) = (/ &
     &1.9678e-01_rb,1.7825e-01_rb,1.5773e-01_rb,1.3590e-01_rb,1.1277e-01_rb,8.8462e-02_rb, &
     &6.3144e-02_rb,3.5733e-02_rb,3.4719e-03_rb/)
      kao(:, 4,10, 6) = (/ &
     &2.5901e-01_rb,2.3451e-01_rb,2.0787e-01_rb,1.7931e-01_rb,1.4891e-01_rb,1.1693e-01_rb, &
     &8.3230e-02_rb,4.7067e-02_rb,4.3129e-03_rb/)
      kao(:, 5,10, 6) = (/ &
     &3.3485e-01_rb,3.0286e-01_rb,2.6808e-01_rb,2.3135e-01_rb,1.9236e-01_rb,1.5087e-01_rb, &
     &1.0672e-01_rb,6.0498e-02_rb,5.2412e-03_rb/)
      kao(:, 1,11, 6) = (/ &
     &1.5729e-01_rb,1.4058e-01_rb,1.2399e-01_rb,1.0596e-01_rb,8.6953e-02_rb,6.7449e-02_rb, &
     &4.7388e-02_rb,2.6098e-02_rb,2.4928e-03_rb/)
      kao(:, 2,11, 6) = (/ &
     &2.1957e-01_rb,1.9641e-01_rb,1.7272e-01_rb,1.4763e-01_rb,1.2135e-01_rb,9.4358e-02_rb, &
     &6.6246e-02_rb,3.6579e-02_rb,3.1141e-03_rb/)
      kao(:, 3,11, 6) = (/ &
     &2.9796e-01_rb,2.6647e-01_rb,2.3432e-01_rb,2.0028e-01_rb,1.6476e-01_rb,1.2810e-01_rb, &
     &8.9989e-02_rb,4.9741e-02_rb,3.7347e-03_rb/)
      kao(:, 4,11, 6) = (/ &
     &3.9483e-01_rb,3.5349e-01_rb,3.1058e-01_rb,2.6565e-01_rb,2.1911e-01_rb,1.7032e-01_rb, &
     &1.1939e-01_rb,6.5674e-02_rb,4.4794e-03_rb/)
      kao(:, 5,11, 6) = (/ &
     &5.1293e-01_rb,4.5889e-01_rb,4.0288e-01_rb,3.4402e-01_rb,2.8339e-01_rb,2.1998e-01_rb, &
     &1.5383e-01_rb,8.4493e-02_rb,5.3916e-03_rb/)
      kao(:, 1,12, 6) = (/ &
     &1.7496e-01_rb,1.5620e-01_rb,1.3720e-01_rb,1.1692e-01_rb,9.5664e-02_rb,7.3896e-02_rb, &
     &5.1517e-02_rb,2.8131e-02_rb,2.4011e-03_rb/)
      kao(:, 2,12, 6) = (/ &
     &2.4535e-01_rb,2.1911e-01_rb,1.9222e-01_rb,1.6394e-01_rb,1.3439e-01_rb,1.0379e-01_rb, &
     &7.2409e-02_rb,3.9528e-02_rb,2.9367e-03_rb/)
      kao(:, 3,12, 6) = (/ &
     &3.3489e-01_rb,2.9916e-01_rb,2.6204e-01_rb,2.2337e-01_rb,1.8311e-01_rb,1.4171e-01_rb, &
     &9.8967e-02_rb,5.4062e-02_rb,3.5110e-03_rb/)
      kao(:, 4,12, 6) = (/ &
     &4.4645e-01_rb,3.9881e-01_rb,3.4940e-01_rb,2.9764e-01_rb,2.4438e-01_rb,1.8899e-01_rb, &
     &1.3169e-01_rb,7.1745e-02_rb,4.2744e-03_rb/)
      kao(:, 5,12, 6) = (/ &
     &5.8440e-01_rb,5.2135e-01_rb,4.5608e-01_rb,3.8804e-01_rb,3.1784e-01_rb,2.4555e-01_rb, &
     &1.7082e-01_rb,9.2836e-02_rb,5.0546e-03_rb/)
      kao(:, 1,13, 6) = (/ &
     &1.5548e-01_rb,1.3934e-01_rb,1.2226e-01_rb,1.0396e-01_rb,8.5068e-02_rb,6.5711e-02_rb, &
     &4.5702e-02_rb,2.4873e-02_rb,2.0740e-03_rb/)
      kao(:, 2,13, 6) = (/ &
     &2.1897e-01_rb,1.9631e-01_rb,1.7240e-01_rb,1.4676e-01_rb,1.2003e-01_rb,9.2645e-02_rb, &
     &6.4538e-02_rb,3.5191e-02_rb,2.5124e-03_rb/)
      kao(:, 3,13, 6) = (/ &
     &3.0093e-01_rb,2.6954e-01_rb,2.3618e-01_rb,2.0119e-01_rb,1.6478e-01_rb,1.2740e-01_rb, &
     &8.8848e-02_rb,4.8366e-02_rb,3.0385e-03_rb/)
      kao(:, 4,13, 6) = (/ &
     &4.0480e-01_rb,3.6240e-01_rb,3.1715e-01_rb,2.6983e-01_rb,2.2076e-01_rb,1.7039e-01_rb, &
     &1.1847e-01_rb,6.4388e-02_rb,3.6949e-03_rb/)
      kao(:, 5,13, 6) = (/ &
     &5.3109e-01_rb,4.7451e-01_rb,4.1461e-01_rb,3.5239e-01_rb,2.8790e-01_rb,2.2202e-01_rb, &
     &1.5435e-01_rb,8.3628e-02_rb,4.3087e-03_rb/)
      kao(:, 1, 1, 7) = (/ &
     &1.3848e-03_rb,7.4265e-03_rb,1.1328e-02_rb,1.5765e-02_rb,2.0617e-02_rb,2.5760e-02_rb, &
     &3.1394e-02_rb,3.9174e-02_rb,3.7962e-02_rb/)
      kao(:, 2, 1, 7) = (/ &
     &1.6671e-03_rb,8.9945e-03_rb,1.3803e-02_rb,1.9211e-02_rb,2.5130e-02_rb,3.1434e-02_rb, &
     &3.8541e-02_rb,4.9244e-02_rb,4.6325e-02_rb/)
      kao(:, 3, 1, 7) = (/ &
     &1.9714e-03_rb,1.0738e-02_rb,1.6621e-02_rb,2.3153e-02_rb,3.0242e-02_rb,3.7827e-02_rb, &
     &4.6470e-02_rb,6.0286e-02_rb,5.5806e-02_rb/)
      kao(:, 4, 1, 7) = (/ &
     &2.3028e-03_rb,1.2683e-02_rb,1.9788e-02_rb,2.7634e-02_rb,3.6042e-02_rb,4.5063e-02_rb, &
     &5.5400e-02_rb,7.2329e-02_rb,6.6551e-02_rb/)
      kao(:, 5, 1, 7) = (/ &
     &2.6597e-03_rb,1.4839e-02_rb,2.3341e-02_rb,3.2680e-02_rb,4.2600e-02_rb,5.3211e-02_rb, &
     &6.5429e-02_rb,8.5816e-02_rb,7.8751e-02_rb/)
      kao(:, 1, 2, 7) = (/ &
     &1.7365e-03_rb,7.5104e-03_rb,1.0855e-02_rb,1.3649e-02_rb,1.6794e-02_rb,2.0487e-02_rb, &
     &2.4622e-02_rb,2.9785e-02_rb,2.9296e-02_rb/)
      kao(:, 2, 2, 7) = (/ &
     &2.1191e-03_rb,9.1841e-03_rb,1.3246e-02_rb,1.6722e-02_rb,2.0613e-02_rb,2.5164e-02_rb, &
     &3.0319e-02_rb,3.7310e-02_rb,3.6002e-02_rb/)
      kao(:, 3, 2, 7) = (/ &
     &2.5333e-03_rb,1.1030e-02_rb,1.5921e-02_rb,2.0206e-02_rb,2.4966e-02_rb,3.0432e-02_rb, &
     &3.6701e-02_rb,4.5559e-02_rb,4.3603e-02_rb/)
      kao(:, 4, 2, 7) = (/ &
     &2.9828e-03_rb,1.3051e-02_rb,1.8932e-02_rb,2.4145e-02_rb,2.9915e-02_rb,3.6454e-02_rb, &
     &4.3919e-02_rb,5.4674e-02_rb,5.2233e-02_rb/)
      kao(:, 5, 2, 7) = (/ &
     &3.4709e-03_rb,1.5291e-02_rb,2.2279e-02_rb,2.8573e-02_rb,3.5520e-02_rb,4.3284e-02_rb, &
     &5.2127e-02_rb,6.5008e-02_rb,6.1979e-02_rb/)
      kao(:, 1, 3, 7) = (/ &
     &2.8092e-03_rb,8.4318e-03_rb,1.1352e-02_rb,1.3672e-02_rb,1.5434e-02_rb,1.6834e-02_rb, &
     &1.8680e-02_rb,2.1551e-02_rb,2.1437e-02_rb/)
      kao(:, 2, 3, 7) = (/ &
     &3.4524e-03_rb,1.0436e-02_rb,1.4060e-02_rb,1.6891e-02_rb,1.9071e-02_rb,2.0847e-02_rb, &
     &2.3206e-02_rb,2.6955e-02_rb,2.6565e-02_rb/)
      kao(:, 3, 3, 7) = (/ &
     &4.1827e-03_rb,1.2653e-02_rb,1.7072e-02_rb,2.0477e-02_rb,2.3160e-02_rb,2.5389e-02_rb, &
     &2.8361e-02_rb,3.3016e-02_rb,3.2451e-02_rb/)
      kao(:, 4, 3, 7) = (/ &
     &4.9958e-03_rb,1.5103e-02_rb,2.0380e-02_rb,2.4512e-02_rb,2.7790e-02_rb,3.0621e-02_rb, &
     &3.4270e-02_rb,3.9878e-02_rb,3.9158e-02_rb/)
      kao(:, 5, 3, 7) = (/ &
     &5.8802e-03_rb,1.7776e-02_rb,2.4055e-02_rb,2.9030e-02_rb,3.2999e-02_rb,3.6516e-02_rb, &
     &4.0970e-02_rb,4.7651e-02_rb,4.6784e-02_rb/)
      kao(:, 1, 4, 7) = (/ &
     &4.5589e-03_rb,1.0079e-02_rb,1.2337e-02_rb,1.3994e-02_rb,1.5249e-02_rb,1.6070e-02_rb, &
     &1.6321e-02_rb,1.6191e-02_rb,1.5492e-02_rb/)
      kao(:, 2, 4, 7) = (/ &
     &5.6834e-03_rb,1.2638e-02_rb,1.5464e-02_rb,1.7583e-02_rb,1.9154e-02_rb,2.0166e-02_rb, &
     &2.0456e-02_rb,2.0438e-02_rb,1.9456e-02_rb/)
      kao(:, 3, 4, 7) = (/ &
     &6.9619e-03_rb,1.5492e-02_rb,1.8969e-02_rb,2.1612e-02_rb,2.3568e-02_rb,2.4770e-02_rb, &
     &2.5120e-02_rb,2.5223e-02_rb,2.3997e-02_rb/)
      kao(:, 4, 4, 7) = (/ &
     &8.4006e-03_rb,1.8644e-02_rb,2.2892e-02_rb,2.6108e-02_rb,2.8487e-02_rb,2.9981e-02_rb, &
     &3.0457e-02_rb,3.0724e-02_rb,2.9183e-02_rb/)
      kao(:, 5, 4, 7) = (/ &
     &1.0053e-02_rb,2.2099e-02_rb,2.7239e-02_rb,3.1080e-02_rb,3.3975e-02_rb,3.5831e-02_rb, &
     &3.6496e-02_rb,3.6989e-02_rb,3.5086e-02_rb/)
      kao(:, 1, 5, 7) = (/ &
     &6.7689e-03_rb,1.1903e-02_rb,1.3648e-02_rb,1.4571e-02_rb,1.5091e-02_rb,1.5249e-02_rb, &
     &1.4963e-02_rb,1.3941e-02_rb,1.1072e-02_rb/)
      kao(:, 2, 5, 7) = (/ &
     &8.5889e-03_rb,1.5080e-02_rb,1.7390e-02_rb,1.8530e-02_rb,1.9222e-02_rb,1.9473e-02_rb, &
     &1.9084e-02_rb,1.7764e-02_rb,1.3833e-02_rb/)
      kao(:, 3, 5, 7) = (/ &
     &1.0679e-02_rb,1.8745e-02_rb,2.1599e-02_rb,2.3039e-02_rb,2.3924e-02_rb,2.4268e-02_rb, &
     &2.3807e-02_rb,2.2090e-02_rb,1.7335e-02_rb/)
      kao(:, 4, 5, 7) = (/ &
     &1.3055e-02_rb,2.2828e-02_rb,2.6303e-02_rb,2.8136e-02_rb,2.9230e-02_rb,2.9685e-02_rb, &
     &2.9170e-02_rb,2.7046e-02_rb,2.1372e-02_rb/)
      kao(:, 5, 5, 7) = (/ &
     &1.5751e-02_rb,2.7271e-02_rb,3.1547e-02_rb,3.3833e-02_rb,3.5204e-02_rb,3.5707e-02_rb, &
     &3.5181e-02_rb,3.2654e-02_rb,2.6029e-02_rb/)
      kao(:, 1, 6, 7) = (/ &
     &9.2094e-03_rb,1.3724e-02_rb,1.5036e-02_rb,1.5371e-02_rb,1.5106e-02_rb,1.4473e-02_rb, &
     &1.3532e-02_rb,1.2020e-02_rb,8.1670e-03_rb/)
      kao(:, 2, 6, 7) = (/ &
     &1.1986e-02_rb,1.7634e-02_rb,1.9365e-02_rb,1.9834e-02_rb,1.9516e-02_rb,1.8750e-02_rb, &
     &1.7602e-02_rb,1.5593e-02_rb,1.0511e-02_rb/)
      kao(:, 3, 6, 7) = (/ &
     &1.5180e-02_rb,2.2067e-02_rb,2.4393e-02_rb,2.4963e-02_rb,2.4596e-02_rb,2.3680e-02_rb, &
     &2.2274e-02_rb,1.9750e-02_rb,1.3124e-02_rb/)
      kao(:, 4, 6, 7) = (/ &
     &1.8879e-02_rb,2.7272e-02_rb,3.0068e-02_rb,3.0821e-02_rb,3.0411e-02_rb,2.9355e-02_rb, &
     &2.7616e-02_rb,2.4539e-02_rb,1.6137e-02_rb/)
      kao(:, 5, 6, 7) = (/ &
     &2.3118e-02_rb,3.3049e-02_rb,3.6382e-02_rb,3.7419e-02_rb,3.7019e-02_rb,3.5790e-02_rb, &
     &3.3628e-02_rb,2.9915e-02_rb,1.9032e-02_rb/)
      kao(:, 1, 7, 7) = (/ &
     &1.3180e-02_rb,1.6672e-02_rb,1.7592e-02_rb,1.7458e-02_rb,1.6513e-02_rb,1.5058e-02_rb, &
     &1.3095e-02_rb,1.0650e-02_rb,5.4642e-03_rb/)
      kao(:, 2, 7, 7) = (/ &
     &1.7434e-02_rb,2.2125e-02_rb,2.3174e-02_rb,2.2835e-02_rb,2.1611e-02_rb,1.9729e-02_rb, &
     &1.7181e-02_rb,1.4117e-02_rb,7.1375e-03_rb/)
      kao(:, 3, 7, 7) = (/ &
     &2.2566e-02_rb,2.8274e-02_rb,2.9519e-02_rb,2.9093e-02_rb,2.7592e-02_rb,2.5177e-02_rb, &
     &2.2032e-02_rb,1.8185e-02_rb,9.3084e-03_rb/)
      kao(:, 4, 7, 7) = (/ &
     &2.8653e-02_rb,3.5321e-02_rb,3.6780e-02_rb,3.6287e-02_rb,3.4434e-02_rb,3.1535e-02_rb, &
     &2.7705e-02_rb,2.2900e-02_rb,1.1848e-02_rb/)
      kao(:, 5, 7, 7) = (/ &
     &3.5765e-02_rb,4.3430e-02_rb,4.5100e-02_rb,4.4435e-02_rb,4.2245e-02_rb,3.8834e-02_rb, &
     &3.4253e-02_rb,2.8260e-02_rb,1.4688e-02_rb/)
      kao(:, 1, 8, 7) = (/ &
     &2.1747e-02_rb,2.4454e-02_rb,2.4292e-02_rb,2.3080e-02_rb,2.1111e-02_rb,1.8543e-02_rb, &
     &1.5300e-02_rb,1.1054e-02_rb,4.8558e-03_rb/)
      kao(:, 2, 8, 7) = (/ &
     &3.0389e-02_rb,3.3327e-02_rb,3.2926e-02_rb,3.1134e-02_rb,2.8533e-02_rb,2.5003e-02_rb, &
     &2.0402e-02_rb,1.4724e-02_rb,6.1788e-03_rb/)
      kao(:, 3, 8, 7) = (/ &
     &4.0290e-02_rb,4.3505e-02_rb,4.2950e-02_rb,4.0676e-02_rb,3.7191e-02_rb,3.2422e-02_rb, &
     &2.6427e-02_rb,1.9111e-02_rb,7.6799e-03_rb/)
      kao(:, 4, 8, 7) = (/ &
     &5.2072e-02_rb,5.5809e-02_rb,5.4895e-02_rb,5.1714e-02_rb,4.6978e-02_rb,4.0907e-02_rb, &
     &3.3411e-02_rb,2.4362e-02_rb,9.3208e-03_rb/)
      kao(:, 5, 8, 7) = (/ &
     &6.6160e-02_rb,7.0258e-02_rb,6.8654e-02_rb,6.4320e-02_rb,5.8240e-02_rb,5.0667e-02_rb, &
     &4.1561e-02_rb,3.0531e-02_rb,1.1097e-02_rb/)
      kao(:, 1, 9, 7) = (/ &
     &6.9879e-02_rb,6.6386e-02_rb,6.1416e-02_rb,5.4862e-02_rb,4.7283e-02_rb,3.9112e-02_rb, &
     &2.9846e-02_rb,1.9026e-02_rb,5.0281e-03_rb/)
      kao(:, 2, 9, 7) = (/ &
     &9.7072e-02_rb,9.2199e-02_rb,8.5540e-02_rb,7.6855e-02_rb,6.6492e-02_rb,5.4707e-02_rb, &
     &4.1549e-02_rb,2.6366e-02_rb,6.3749e-03_rb/)
      kao(:, 3, 9, 7) = (/ &
     &1.3274e-01_rb,1.2639e-01_rb,1.1670e-01_rb,1.0412e-01_rb,8.9576e-02_rb,7.3400e-02_rb, &
     &5.5548e-02_rb,3.5017e-02_rb,7.9283e-03_rb/)
      kao(:, 4, 9, 7) = (/ &
     &1.7725e-01_rb,1.6726e-01_rb,1.5276e-01_rb,1.3606e-01_rb,1.1668e-01_rb,9.5385e-02_rb, &
     &7.1895e-02_rb,4.5017e-02_rb,9.7700e-03_rb/)
      kao(:, 5, 9, 7) = (/ &
     &2.2900e-01_rb,2.1469e-01_rb,1.9562e-01_rb,1.7346e-01_rb,1.4892e-01_rb,1.2126e-01_rb, &
     &9.1015e-02_rb,5.6621e-02_rb,1.1851e-02_rb/)
      kao(:, 1,10, 7) = (/ &
     &2.6110e-01_rb,2.3191e-01_rb,2.0431e-01_rb,1.7643e-01_rb,1.4693e-01_rb,1.1545e-01_rb, &
     &8.1908e-02_rb,4.6480e-02_rb,5.5448e-03_rb/)
      kao(:, 2,10, 7) = (/ &
     &3.6304e-01_rb,3.2305e-01_rb,2.8523e-01_rb,2.4612e-01_rb,2.0468e-01_rb,1.6117e-01_rb, &
     &1.1489e-01_rb,6.5350e-02_rb,7.0032e-03_rb/)
      kao(:, 3,10, 7) = (/ &
     &5.0185e-01_rb,4.4688e-01_rb,3.9417e-01_rb,3.3909e-01_rb,2.8117e-01_rb,2.2056e-01_rb, &
     &1.5648e-01_rb,8.9037e-02_rb,8.6105e-03_rb/)
      kao(:, 4,10, 7) = (/ &
     &6.7448e-01_rb,6.0038e-01_rb,5.2817e-01_rb,4.5305e-01_rb,3.7479e-01_rb,2.9367e-01_rb, &
     &2.0933e-01_rb,1.1852e-01_rb,1.0215e-02_rb/)
      kao(:, 5,10, 7) = (/ &
     &8.8770e-01_rb,7.8961e-01_rb,6.9429e-01_rb,5.9500e-01_rb,4.9158e-01_rb,3.8401e-01_rb, &
     &2.7292e-01_rb,1.5364e-01_rb,1.2140e-02_rb/)
      kao(:, 1,11, 7) = (/ &
     &4.0532e-01_rb,3.5824e-01_rb,3.1239e-01_rb,2.6732e-01_rb,2.2024e-01_rb,1.7123e-01_rb, &
     &1.1969e-01_rb,6.6094e-02_rb,5.4214e-03_rb/)
      kao(:, 2,11, 7) = (/ &
     &5.6606e-01_rb,5.0050e-01_rb,4.3693e-01_rb,3.7301e-01_rb,3.0650e-01_rb,2.3759e-01_rb, &
     &1.6618e-01_rb,9.1406e-02_rb,6.5476e-03_rb/)
      kao(:, 3,11, 7) = (/ &
     &7.7109e-01_rb,6.8230e-01_rb,5.9547e-01_rb,5.0803e-01_rb,4.1719e-01_rb,3.2299e-01_rb, &
     &2.2593e-01_rb,1.2450e-01_rb,7.8892e-03_rb/)
      kao(:, 4,11, 7) = (/ &
     &1.0345e+00_rb,9.1439e-01_rb,7.9746e-01_rb,6.7877e-01_rb,5.5627e-01_rb,4.3053e-01_rb, &
     &3.0103e-01_rb,1.6533e-01_rb,9.6823e-03_rb/)
      kao(:, 5,11, 7) = (/ &
     &1.3462e+00_rb,1.1891e+00_rb,1.0362e+00_rb,8.8065e-01_rb,7.2071e-01_rb,5.5791e-01_rb, &
     &3.9044e-01_rb,2.1464e-01_rb,1.1736e-02_rb/)
      kao(:, 1,12, 7) = (/ &
     &4.6420e-01_rb,4.0988e-01_rb,3.5698e-01_rb,3.0422e-01_rb,2.4918e-01_rb,1.9252e-01_rb, &
     &1.3423e-01_rb,7.3160e-02_rb,4.7396e-03_rb/)
      kao(:, 2,12, 7) = (/ &
     &6.5450e-01_rb,5.7811e-01_rb,5.0333e-01_rb,4.2804e-01_rb,3.4996e-01_rb,2.7019e-01_rb, &
     &1.8778e-01_rb,1.0212e-01_rb,5.9506e-03_rb/)
      kao(:, 3,12, 7) = (/ &
     &8.9348e-01_rb,7.8900e-01_rb,6.8666e-01_rb,5.8272e-01_rb,4.7603e-01_rb,3.6677e-01_rb, &
     &2.5473e-01_rb,1.3789e-01_rb,7.3179e-03_rb/)
      kao(:, 4,12, 7) = (/ &
     &1.1756e+00_rb,1.0374e+00_rb,9.0174e-01_rb,7.6533e-01_rb,6.2509e-01_rb,4.8180e-01_rb, &
     &3.3511e-01_rb,1.8185e-01_rb,8.9523e-03_rb/)
      kao(:, 5,12, 7) = (/ &
     &1.5297e+00_rb,1.3494e+00_rb,1.1729e+00_rb,9.9401e-01_rb,8.1052e-01_rb,6.2387e-01_rb, &
     &4.3290e-01_rb,2.3433e-01_rb,1.0845e-02_rb/)
      kao(:, 1,13, 7) = (/ &
     &4.2167e-01_rb,3.7276e-01_rb,3.2553e-01_rb,2.7726e-01_rb,2.2695e-01_rb,1.7522e-01_rb, &
     &1.2184e-01_rb,6.6143e-02_rb,4.0030e-03_rb/)
      kao(:, 2,13, 7) = (/ &
     &6.0140e-01_rb,5.3188e-01_rb,4.6380e-01_rb,3.9365e-01_rb,3.2184e-01_rb,2.4778e-01_rb, &
     &1.7190e-01_rb,9.3023e-02_rb,5.1104e-03_rb/)
      kao(:, 3,13, 7) = (/ &
     &8.1807e-01_rb,7.2295e-01_rb,6.3004e-01_rb,5.3457e-01_rb,4.3643e-01_rb,3.3605e-01_rb, &
     &2.3292e-01_rb,1.2549e-01_rb,6.4347e-03_rb/)
      kao(:, 4,13, 7) = (/ &
     &1.0778e+00_rb,9.5216e-01_rb,8.2890e-01_rb,7.0211e-01_rb,5.7259e-01_rb,4.4078e-01_rb, &
     &3.0511e-01_rb,1.6456e-01_rb,7.8490e-03_rb/)
      kao(:, 5,13, 7) = (/ &
     &1.3868e+00_rb,1.2246e+00_rb,1.0647e+00_rb,9.0150e-01_rb,7.3547e-01_rb,5.6615e-01_rb, &
     &3.9191e-01_rb,2.1135e-01_rb,9.6903e-03_rb/)
      kao(:, 1, 1, 8) = (/ &
     &3.5696e-03_rb,1.6007e-02_rb,2.9119e-02_rb,4.2110e-02_rb,5.4860e-02_rb,6.8076e-02_rb, &
     &8.3917e-02_rb,1.0558e-01_rb,1.0534e-01_rb/)
      kao(:, 2, 1, 8) = (/ &
     &4.2863e-03_rb,1.9722e-02_rb,3.5791e-02_rb,5.1660e-02_rb,6.7170e-02_rb,8.3493e-02_rb, &
     &1.0314e-01_rb,1.2980e-01_rb,1.2936e-01_rb/)
      kao(:, 3, 1, 8) = (/ &
     &5.0567e-03_rb,2.3939e-02_rb,4.3341e-02_rb,6.2437e-02_rb,8.1028e-02_rb,1.0094e-01_rb, &
     &1.2485e-01_rb,1.5697e-01_rb,1.5657e-01_rb/)
      kao(:, 4, 1, 8) = (/ &
     &5.8865e-03_rb,2.8699e-02_rb,5.1840e-02_rb,7.4548e-02_rb,9.6669e-02_rb,1.2066e-01_rb, &
     &1.4947e-01_rb,1.8794e-01_rb,1.8748e-01_rb/)
      kao(:, 5, 1, 8) = (/ &
     &6.7695e-03_rb,3.4031e-02_rb,6.1358e-02_rb,8.8096e-02_rb,1.1420e-01_rb,1.4289e-01_rb, &
     &1.7720e-01_rb,2.2237e-01_rb,2.2240e-01_rb/)
      kao(:, 1, 2, 8) = (/ &
     &4.7118e-03_rb,1.5877e-02_rb,2.5399e-02_rb,3.5918e-02_rb,4.6457e-02_rb,5.6939e-02_rb, &
     &6.8068e-02_rb,8.4536e-02_rb,8.5555e-02_rb/)
      kao(:, 2, 2, 8) = (/ &
     &5.7108e-03_rb,1.9571e-02_rb,3.1487e-02_rb,4.4433e-02_rb,5.7366e-02_rb,7.0154e-02_rb, &
     &8.4052e-02_rb,1.0472e-01_rb,1.0570e-01_rb/)
      kao(:, 3, 2, 8) = (/ &
     &6.7866e-03_rb,2.3743e-02_rb,3.8403e-02_rb,5.4069e-02_rb,6.9657e-02_rb,8.4997e-02_rb, &
     &1.0211e-01_rb,1.2762e-01_rb,1.2838e-01_rb/)
      kao(:, 4, 2, 8) = (/ &
     &7.9436e-03_rb,2.8460e-02_rb,4.6231e-02_rb,6.4990e-02_rb,8.3598e-02_rb,1.0182e-01_rb, &
     &1.2260e-01_rb,1.5356e-01_rb,1.5440e-01_rb/)
      kao(:, 5, 2, 8) = (/ &
     &9.1873e-03_rb,3.3722e-02_rb,5.4999e-02_rb,7.7194e-02_rb,9.9137e-02_rb,1.2068e-01_rb, &
     &1.4584e-01_rb,1.8285e-01_rb,1.8375e-01_rb/)
      kao(:, 1, 3, 8) = (/ &
     &7.9289e-03_rb,1.8898e-02_rb,2.5653e-02_rb,3.1610e-02_rb,3.8347e-02_rb,4.5822e-02_rb, &
     &5.3494e-02_rb,6.2174e-02_rb,6.4789e-02_rb/)
      kao(:, 2, 3, 8) = (/ &
     &9.7250e-03_rb,2.3378e-02_rb,3.1960e-02_rb,3.9650e-02_rb,4.8157e-02_rb,5.7463e-02_rb, &
     &6.6987e-02_rb,7.7905e-02_rb,8.1125e-02_rb/)
      kao(:, 3, 3, 8) = (/ &
     &1.1690e-02_rb,2.8477e-02_rb,3.9050e-02_rb,4.8735e-02_rb,5.9251e-02_rb,7.0590e-02_rb, &
     &8.2118e-02_rb,9.5596e-02_rb,9.9562e-02_rb/)
      kao(:, 4, 3, 8) = (/ &
     &1.3819e-02_rb,3.4230e-02_rb,4.7053e-02_rb,5.8936e-02_rb,7.1761e-02_rb,8.5391e-02_rb, &
     &9.9148e-02_rb,1.1547e-01_rb,1.2059e-01_rb/)
      kao(:, 5, 3, 8) = (/ &
     &1.6126e-02_rb,4.0610e-02_rb,5.5935e-02_rb,7.0350e-02_rb,8.5758e-02_rb,1.0193e-01_rb, &
     &1.1815e-01_rb,1.3794e-01_rb,1.4428e-01_rb/)
      kao(:, 1, 4, 8) = (/ &
     &1.3217e-02_rb,2.3479e-02_rb,2.9539e-02_rb,3.3751e-02_rb,3.6721e-02_rb,3.9161e-02_rb, &
     &4.2339e-02_rb,4.7025e-02_rb,4.7847e-02_rb/)
      kao(:, 2, 4, 8) = (/ &
     &1.6447e-02_rb,2.9417e-02_rb,3.7093e-02_rb,4.2546e-02_rb,4.6456e-02_rb,4.9765e-02_rb, &
     &5.3922e-02_rb,5.9724e-02_rb,6.0703e-02_rb/)
      kao(:, 3, 4, 8) = (/ &
     &2.0055e-02_rb,3.6163e-02_rb,4.5727e-02_rb,5.2541e-02_rb,5.7472e-02_rb,6.1844e-02_rb, &
     &6.7058e-02_rb,7.4090e-02_rb,7.5241e-02_rb/)
      kao(:, 4, 4, 8) = (/ &
     &2.4006e-02_rb,4.3785e-02_rb,5.5469e-02_rb,6.3755e-02_rb,6.9920e-02_rb,7.5527e-02_rb, &
     &8.2000e-02_rb,9.0387e-02_rb,9.1968e-02_rb/)
      kao(:, 5, 4, 8) = (/ &
     &2.8301e-02_rb,5.2295e-02_rb,6.6272e-02_rb,7.6263e-02_rb,8.3883e-02_rb,9.0941e-02_rb, &
     &9.8884e-02_rb,1.0876e-01_rb,1.1102e-01_rb/)
      kao(:, 1, 5, 8) = (/ &
     &1.9932e-02_rb,2.8909e-02_rb,3.3929e-02_rb,3.6963e-02_rb,3.8599e-02_rb,3.9090e-02_rb, &
     &3.8100e-02_rb,3.6647e-02_rb,3.4570e-02_rb/)
      kao(:, 2, 5, 8) = (/ &
     &2.5221e-02_rb,3.6867e-02_rb,4.3184e-02_rb,4.7147e-02_rb,4.9394e-02_rb,5.0060e-02_rb, &
     &4.9016e-02_rb,4.7379e-02_rb,4.4959e-02_rb/)
      kao(:, 3, 5, 8) = (/ &
     &3.1232e-02_rb,4.5824e-02_rb,5.3773e-02_rb,5.8918e-02_rb,6.1826e-02_rb,6.2626e-02_rb, &
     &6.1399e-02_rb,5.9643e-02_rb,5.6358e-02_rb/)
      kao(:, 4, 5, 8) = (/ &
     &3.7911e-02_rb,5.5890e-02_rb,6.5835e-02_rb,7.2317e-02_rb,7.5820e-02_rb,7.6769e-02_rb, &
     &7.5408e-02_rb,7.3611e-02_rb,6.9498e-02_rb/)
      kao(:, 5, 5, 8) = (/ &
     &4.5225e-02_rb,6.7203e-02_rb,7.9322e-02_rb,8.7058e-02_rb,9.1394e-02_rb,9.2728e-02_rb, &
     &9.1290e-02_rb,8.9467e-02_rb,8.4545e-02_rb/)
      kao(:, 1, 6, 8) = (/ &
     &2.7668e-02_rb,3.4842e-02_rb,3.8147e-02_rb,3.9719e-02_rb,4.0056e-02_rb,3.9073e-02_rb, &
     &3.6625e-02_rb,3.2055e-02_rb,2.4026e-02_rb/)
      kao(:, 2, 6, 8) = (/ &
     &3.5525e-02_rb,4.5073e-02_rb,4.9553e-02_rb,5.1630e-02_rb,5.2026e-02_rb,5.0818e-02_rb, &
     &4.7728e-02_rb,4.1935e-02_rb,3.1627e-02_rb/)
      kao(:, 3, 6, 8) = (/ &
     &4.4689e-02_rb,5.6776e-02_rb,6.2560e-02_rb,6.5413e-02_rb,6.6041e-02_rb,6.4568e-02_rb, &
     &6.0721e-02_rb,5.3252e-02_rb,4.0421e-02_rb/)
      kao(:, 4, 6, 8) = (/ &
     &5.5122e-02_rb,7.0187e-02_rb,7.7371e-02_rb,8.1166e-02_rb,8.2060e-02_rb,8.0200e-02_rb, &
     &7.5289e-02_rb,6.6107e-02_rb,5.0711e-02_rb/)
      kao(:, 5, 6, 8) = (/ &
     &6.6674e-02_rb,8.5247e-02_rb,9.4184e-02_rb,9.8741e-02_rb,9.9891e-02_rb,9.7702e-02_rb, &
     &9.1660e-02_rb,8.0784e-02_rb,6.3321e-02_rb/)
      kao(:, 1, 7, 8) = (/ &
     &4.0056e-02_rb,4.4898e-02_rb,4.5971e-02_rb,4.5352e-02_rb,4.3659e-02_rb,4.0747e-02_rb, &
     &3.6554e-02_rb,3.0205e-02_rb,1.8196e-02_rb/)
      kao(:, 2, 7, 8) = (/ &
     &5.2185e-02_rb,5.8633e-02_rb,6.0480e-02_rb,6.0085e-02_rb,5.7874e-02_rb,5.4080e-02_rb, &
     &4.8546e-02_rb,4.0072e-02_rb,2.4060e-02_rb/)
      kao(:, 3, 7, 8) = (/ &
     &6.6434e-02_rb,7.5122e-02_rb,7.7544e-02_rb,7.7216e-02_rb,7.4584e-02_rb,6.9858e-02_rb, &
     &6.2703e-02_rb,5.1788e-02_rb,3.0507e-02_rb/)
      kao(:, 4, 7, 8) = (/ &
     &8.3239e-02_rb,9.4089e-02_rb,9.7312e-02_rb,9.7000e-02_rb,9.3885e-02_rb,8.8076e-02_rb, &
     &7.9042e-02_rb,6.5047e-02_rb,3.6382e-02_rb/)
      kao(:, 5, 7, 8) = (/ &
     &1.0221e-01_rb,1.1561e-01_rb,1.1992e-01_rb,1.1954e-01_rb,1.1572e-01_rb,1.0845e-01_rb, &
     &9.7278e-02_rb,8.0126e-02_rb,4.5426e-02_rb/)
      kao(:, 1, 8, 8) = (/ &
     &6.9306e-02_rb,6.9494e-02_rb,6.7600e-02_rb,6.3267e-02_rb,5.7322e-02_rb,5.0091e-02_rb, &
     &4.1741e-02_rb,3.1433e-02_rb,1.2053e-02_rb/)
      kao(:, 2, 8, 8) = (/ &
     &9.1357e-02_rb,9.2237e-02_rb,8.9335e-02_rb,8.3857e-02_rb,7.6444e-02_rb,6.7451e-02_rb, &
     &5.6474e-02_rb,4.2659e-02_rb,1.6127e-02_rb/)
      kao(:, 3, 8, 8) = (/ &
     &1.1801e-01_rb,1.1955e-01_rb,1.1588e-01_rb,1.0908e-01_rb,9.9881e-02_rb,8.8243e-02_rb, &
     &7.4146e-02_rb,5.6165e-02_rb,2.1600e-02_rb/)
      kao(:, 4, 8, 8) = (/ &
     &1.4950e-01_rb,1.5197e-01_rb,1.4749e-01_rb,1.3894e-01_rb,1.2736e-01_rb,1.1271e-01_rb, &
     &9.4953e-02_rb,7.1880e-02_rb,2.7972e-02_rb/)
      kao(:, 5, 8, 8) = (/ &
     &1.8646e-01_rb,1.8942e-01_rb,1.8392e-01_rb,1.7346e-01_rb,1.5924e-01_rb,1.4113e-01_rb, &
     &1.1856e-01_rb,8.9580e-02_rb,3.5566e-02_rb/)
      kao(:, 1, 9, 8) = (/ &
     &2.2096e-01_rb,1.9931e-01_rb,1.8126e-01_rb,1.6106e-01_rb,1.3834e-01_rb,1.1283e-01_rb, &
     &8.4790e-02_rb,5.3465e-02_rb,1.3124e-02_rb/)
      kao(:, 2, 9, 8) = (/ &
     &3.0111e-01_rb,2.7269e-01_rb,2.4731e-01_rb,2.1828e-01_rb,1.8616e-01_rb,1.5202e-01_rb, &
     &1.1457e-01_rb,7.2434e-02_rb,1.6890e-02_rb/)
      kao(:, 3, 9, 8) = (/ &
     &3.9547e-01_rb,3.5811e-01_rb,3.2380e-01_rb,2.8636e-01_rb,2.4465e-01_rb,1.9968e-01_rb, &
     &1.5075e-01_rb,9.6319e-02_rb,2.1030e-02_rb/)
      kao(:, 4, 9, 8) = (/ &
     &5.0622e-01_rb,4.6008e-01_rb,4.1701e-01_rb,3.6806e-01_rb,3.1477e-01_rb,2.5740e-01_rb, &
     &1.9504e-01_rb,1.2517e-01_rb,2.5567e-02_rb/)
      kao(:, 5, 9, 8) = (/ &
     &6.3970e-01_rb,5.8282e-01_rb,5.2753e-01_rb,4.6589e-01_rb,3.9821e-01_rb,3.2594e-01_rb, &
     &2.4744e-01_rb,1.5892e-01_rb,3.0707e-02_rb/)
      kao(:, 1,10, 8) = (/ &
     &7.9609e-01_rb,6.9877e-01_rb,6.0631e-01_rb,5.1323e-01_rb,4.2176e-01_rb,3.3040e-01_rb, &
     &2.3557e-01_rb,1.3340e-01_rb,1.8364e-02_rb/)
      kao(:, 2,10, 8) = (/ &
     &1.1185e+00_rb,9.8179e-01_rb,8.5163e-01_rb,7.2111e-01_rb,5.9292e-01_rb,4.6368e-01_rb, &
     &3.2777e-01_rb,1.8494e-01_rb,2.2731e-02_rb/)
      kao(:, 3,10, 8) = (/ &
     &1.4977e+00_rb,1.3152e+00_rb,1.1422e+00_rb,9.6927e-01_rb,7.9815e-01_rb,6.2269e-01_rb, &
     &4.4054e-01_rb,2.4719e-01_rb,2.7714e-02_rb/)
      kao(:, 4,10, 8) = (/ &
     &1.9588e+00_rb,1.7209e+00_rb,1.4963e+00_rb,1.2699e+00_rb,1.0440e+00_rb,8.1231e-01_rb, &
     &5.7215e-01_rb,3.2165e-01_rb,3.3541e-02_rb/)
      kao(:, 5,10, 8) = (/ &
     &2.4948e+00_rb,2.1931e+00_rb,1.9066e+00_rb,1.6194e+00_rb,1.3305e+00_rb,1.0363e+00_rb, &
     &7.3187e-01_rb,4.1258e-01_rb,3.9461e-02_rb/)
      kao(:, 1,11, 8) = (/ &
     &1.1786e+00_rb,1.0326e+00_rb,8.9194e-01_rb,7.5040e-01_rb,6.1100e-01_rb,4.7311e-01_rb, &
     &3.3100e-01_rb,1.8184e-01_rb,1.9912e-02_rb/)
      kao(:, 2,11, 8) = (/ &
     &1.6577e+00_rb,1.4527e+00_rb,1.2557e+00_rb,1.0583e+00_rb,8.6330e-01_rb,6.6758e-01_rb, &
     &4.6659e-01_rb,2.5585e-01_rb,2.4574e-02_rb/)
      kao(:, 3,11, 8) = (/ &
     &2.2529e+00_rb,1.9745e+00_rb,1.7078e+00_rb,1.4395e+00_rb,1.1743e+00_rb,9.0668e-01_rb, &
     &6.3142e-01_rb,3.4477e-01_rb,2.9771e-02_rb/)
      kao(:, 4,11, 8) = (/ &
     &2.9660e+00_rb,2.6012e+00_rb,2.2499e+00_rb,1.8975e+00_rb,1.5469e+00_rb,1.1941e+00_rb, &
     &8.2956e-01_rb,4.5247e-01_rb,3.5340e-02_rb/)
      kao(:, 5,11, 8) = (/ &
     &3.8252e+00_rb,3.3560e+00_rb,2.9025e+00_rb,2.4493e+00_rb,1.9962e+00_rb,1.5364e+00_rb, &
     &1.0662e+00_rb,5.8207e-01_rb,4.0740e-02_rb/)
      kao(:, 1,12, 8) = (/ &
     &1.3334e+00_rb,1.1683e+00_rb,1.0080e+00_rb,8.4695e-01_rb,6.8881e-01_rb,5.3043e-01_rb, &
     &3.6715e-01_rb,1.9749e-01_rb,1.8810e-02_rb/)
      kao(:, 2,12, 8) = (/ &
     &1.8510e+00_rb,1.6221e+00_rb,1.4015e+00_rb,1.1795e+00_rb,9.6045e-01_rb,7.4002e-01_rb, &
     &5.1281e-01_rb,2.7717e-01_rb,2.2865e-02_rb/)
      kao(:, 3,12, 8) = (/ &
     &2.5252e+00_rb,2.2140e+00_rb,1.9140e+00_rb,1.6128e+00_rb,1.3137e+00_rb,1.0115e+00_rb, &
     &6.9957e-01_rb,3.7776e-01_rb,2.7421e-02_rb/)
      kao(:, 4,12, 8) = (/ &
     &3.3724e+00_rb,2.9581e+00_rb,2.5564e+00_rb,2.1530e+00_rb,1.7520e+00_rb,1.3471e+00_rb, &
     &9.2947e-01_rb,5.0112e-01_rb,3.2393e-02_rb/)
      kao(:, 5,12, 8) = (/ &
     &4.3792e+00_rb,3.8422e+00_rb,3.3210e+00_rb,2.7983e+00_rb,2.2755e+00_rb,1.7461e+00_rb, &
     &1.2059e+00_rb,6.4752e-01_rb,3.7070e-02_rb/)
      kao(:, 1,13, 8) = (/ &
     &1.2116e+00_rb,1.0627e+00_rb,9.1758e-01_rb,7.7277e-01_rb,6.2996e-01_rb,4.8487e-01_rb, &
     &3.3536e-01_rb,1.7964e-01_rb,1.5468e-02_rb/)
      kao(:, 2,13, 8) = (/ &
     &1.6818e+00_rb,1.4755e+00_rb,1.2746e+00_rb,1.0748e+00_rb,8.7634e-01_rb,6.7377e-01_rb, &
     &4.6518e-01_rb,2.4901e-01_rb,1.8709e-02_rb/)
      kao(:, 3,13, 8) = (/ &
     &2.2861e+00_rb,2.0071e+00_rb,1.7358e+00_rb,1.4644e+00_rb,1.1940e+00_rb,9.1837e-01_rb, &
     &6.3513e-01_rb,3.4226e-01_rb,2.2464e-02_rb/)
      kao(:, 4,13, 8) = (/ &
     &3.0747e+00_rb,2.7003e+00_rb,2.3365e+00_rb,1.9712e+00_rb,1.6050e+00_rb,1.2324e+00_rb, &
     &8.5082e-01_rb,4.5761e-01_rb,2.6279e-02_rb/)
      kao(:, 5,13, 8) = (/ &
     &4.0462e+00_rb,3.5547e+00_rb,3.0752e+00_rb,2.5948e+00_rb,2.1107e+00_rb,1.6188e+00_rb, &
     &1.1159e+00_rb,5.9825e-01_rb,3.0261e-02_rb/)
      kao(:, 1, 1, 9) = (/ &
     &7.9433e-03_rb,4.9696e-02_rb,8.9399e-02_rb,1.2891e-01_rb,1.7031e-01_rb,2.1327e-01_rb, &
     &2.5635e-01_rb,2.9834e-01_rb,3.4005e-01_rb/)
      kao(:, 2, 1, 9) = (/ &
     &9.5773e-03_rb,6.0610e-02_rb,1.0882e-01_rb,1.5730e-01_rb,2.0827e-01_rb,2.6089e-01_rb, &
     &3.1374e-01_rb,3.6559e-01_rb,4.1584e-01_rb/)
      kao(:, 3, 1, 9) = (/ &
     &1.1355e-02_rb,7.2723e-02_rb,1.3048e-01_rb,1.8917e-01_rb,2.5096e-01_rb,3.1435e-01_rb, &
     &3.7828e-01_rb,4.4211e-01_rb,5.0107e-01_rb/)
      kao(:, 4, 1, 9) = (/ &
     &1.3277e-02_rb,8.6204e-02_rb,1.5483e-01_rb,2.2520e-01_rb,2.9933e-01_rb,3.7480e-01_rb, &
     &4.5117e-01_rb,5.2865e-01_rb,5.9762e-01_rb/)
      kao(:, 5, 1, 9) = (/ &
     &1.5326e-02_rb,1.0116e-01_rb,1.8188e-01_rb,2.6559e-01_rb,3.5380e-01_rb,4.4327e-01_rb, &
     &5.3402e-01_rb,6.2664e-01_rb,7.0640e-01_rb/)
      kao(:, 1, 2, 9) = (/ &
     &1.1776e-02_rb,4.7878e-02_rb,8.4972e-02_rb,1.1878e-01_rb,1.5366e-01_rb,1.9061e-01_rb, &
     &2.3033e-01_rb,2.7211e-01_rb,3.0314e-01_rb/)
      kao(:, 2, 2, 9) = (/ &
     &1.4294e-02_rb,5.8956e-02_rb,1.0416e-01_rb,1.4559e-01_rb,1.8871e-01_rb,2.3457e-01_rb, &
     &2.8358e-01_rb,3.3549e-01_rb,3.7290e-01_rb/)
      kao(:, 3, 2, 9) = (/ &
     &1.7066e-02_rb,7.1345e-02_rb,1.2565e-01_rb,1.7589e-01_rb,2.2857e-01_rb,2.8471e-01_rb, &
     &3.4402e-01_rb,4.0658e-01_rb,4.5272e-01_rb/)
      kao(:, 4, 2, 9) = (/ &
     &2.0089e-02_rb,8.5201e-02_rb,1.4972e-01_rb,2.1022e-01_rb,2.7384e-01_rb,3.4170e-01_rb, &
     &4.1272e-01_rb,4.8704e-01_rb,5.4350e-01_rb/)
      kao(:, 5, 2, 9) = (/ &
     &2.3307e-02_rb,1.0059e-01_rb,1.7602e-01_rb,2.4834e-01_rb,3.2442e-01_rb,4.0570e-01_rb, &
     &4.8998e-01_rb,5.7844e-01_rb,6.4523e-01_rb/)
      kao(:, 1, 3, 9) = (/ &
     &2.2348e-02_rb,5.0240e-02_rb,8.0496e-02_rb,1.1072e-01_rb,1.3814e-01_rb,1.6488e-01_rb, &
     &1.9401e-01_rb,2.2881e-01_rb,2.5470e-01_rb/)
      kao(:, 2, 3, 9) = (/ &
     &2.7442e-02_rb,6.2748e-02_rb,1.0008e-01_rb,1.3702e-01_rb,1.7047e-01_rb,2.0379e-01_rb, &
     &2.4015e-01_rb,2.8371e-01_rb,3.1563e-01_rb/)
      kao(:, 3, 3, 9) = (/ &
     &3.3066e-02_rb,7.6711e-02_rb,1.2229e-01_rb,1.6700e-01_rb,2.0740e-01_rb,2.4849e-01_rb, &
     &2.9339e-01_rb,3.4685e-01_rb,3.8604e-01_rb/)
      kao(:, 4, 3, 9) = (/ &
     &3.9269e-02_rb,9.2366e-02_rb,1.4749e-01_rb,2.0103e-01_rb,2.4924e-01_rb,2.9944e-01_rb, &
     &3.5443e-01_rb,4.1958e-01_rb,4.6663e-01_rb/)
      kao(:, 5, 3, 9) = (/ &
     &4.5933e-02_rb,1.0972e-01_rb,1.7562e-01_rb,2.3869e-01_rb,2.9592e-01_rb,3.5671e-01_rb, &
     &4.2354e-01_rb,5.0205e-01_rb,5.5762e-01_rb/)
      kao(:, 1, 4, 9) = (/ &
     &4.2106e-02_rb,6.4369e-02_rb,8.4555e-02_rb,1.0542e-01_rb,1.2690e-01_rb,1.4750e-01_rb, &
     &1.6487e-01_rb,1.8576e-01_rb,2.0651e-01_rb/)
      kao(:, 2, 4, 9) = (/ &
     &5.2576e-02_rb,8.1026e-02_rb,1.0683e-01_rb,1.3280e-01_rb,1.5950e-01_rb,1.8491e-01_rb, &
     &2.0655e-01_rb,2.3307e-01_rb,2.5966e-01_rb/)
      kao(:, 3, 4, 9) = (/ &
     &6.4079e-02_rb,9.9622e-02_rb,1.3194e-01_rb,1.6392e-01_rb,1.9672e-01_rb,2.2769e-01_rb, &
     &2.5451e-01_rb,2.8781e-01_rb,3.2142e-01_rb/)
      kao(:, 4, 4, 9) = (/ &
     &7.6781e-02_rb,1.2036e-01_rb,1.6040e-01_rb,1.9955e-01_rb,2.3909e-01_rb,2.7584e-01_rb, &
     &3.0872e-01_rb,3.5017e-01_rb,3.9149e-01_rb/)
      kao(:, 5, 4, 9) = (/ &
     &9.0605e-02_rb,1.4314e-01_rb,1.9223e-01_rb,2.3943e-01_rb,2.8645e-01_rb,3.2911e-01_rb, &
     &3.6945e-01_rb,4.2071e-01_rb,4.7061e-01_rb/)
      kao(:, 1, 5, 9) = (/ &
     &7.1850e-02_rb,8.9617e-02_rb,1.0358e-01_rb,1.1417e-01_rb,1.2410e-01_rb,1.3445e-01_rb, &
     &1.4500e-01_rb,1.5289e-01_rb,1.6393e-01_rb/)
      kao(:, 2, 5, 9) = (/ &
     &9.1160e-02_rb,1.1411e-01_rb,1.3218e-01_rb,1.4610e-01_rb,1.5890e-01_rb,1.7194e-01_rb, &
     &1.8487e-01_rb,1.9440e-01_rb,2.0899e-01_rb/)
      kao(:, 3, 5, 9) = (/ &
     &1.1266e-01_rb,1.4163e-01_rb,1.6452e-01_rb,1.8216e-01_rb,1.9856e-01_rb,2.1504e-01_rb, &
     &2.3093e-01_rb,2.4255e-01_rb,2.6179e-01_rb/)
      kao(:, 4, 5, 9) = (/ &
     &1.3642e-01_rb,1.7249e-01_rb,2.0078e-01_rb,2.2297e-01_rb,2.4396e-01_rb,2.6433e-01_rb, &
     &2.8347e-01_rb,2.9753e-01_rb,3.2213e-01_rb/)
      kao(:, 5, 5, 9) = (/ &
     &1.6239e-01_rb,2.0660e-01_rb,2.4114e-01_rb,2.6894e-01_rb,2.9484e-01_rb,3.1958e-01_rb, &
     &3.4207e-01_rb,3.5958e-01_rb,3.9095e-01_rb/)
      kao(:, 1, 6, 9) = (/ &
     &1.1140e-01_rb,1.2144e-01_rb,1.3218e-01_rb,1.3718e-01_rb,1.3653e-01_rb,1.3402e-01_rb, &
     &1.3174e-01_rb,1.2946e-01_rb,1.2677e-01_rb/)
      kao(:, 2, 6, 9) = (/ &
     &1.4420e-01_rb,1.5827e-01_rb,1.7119e-01_rb,1.7776e-01_rb,1.7742e-01_rb,1.7454e-01_rb, &
     &1.7134e-01_rb,1.6760e-01_rb,1.6399e-01_rb/)
      kao(:, 3, 6, 9) = (/ &
     &1.8124e-01_rb,2.0002e-01_rb,2.1589e-01_rb,2.2389e-01_rb,2.2423e-01_rb,2.2127e-01_rb, &
     &2.1711e-01_rb,2.1204e-01_rb,2.0793e-01_rb/)
      kao(:, 4, 6, 9) = (/ &
     &2.2248e-01_rb,2.4636e-01_rb,2.6609e-01_rb,2.7594e-01_rb,2.7715e-01_rb,2.7479e-01_rb, &
     &2.7017e-01_rb,2.6283e-01_rb,2.5856e-01_rb/)
      kao(:, 5, 6, 9) = (/ &
     &2.6774e-01_rb,2.9748e-01_rb,3.2183e-01_rb,3.3450e-01_rb,3.3675e-01_rb,3.3499e-01_rb, &
     &3.3006e-01_rb,3.2020e-01_rb,3.1680e-01_rb/)
      kao(:, 1, 7, 9) = (/ &
     &1.7732e-01_rb,1.7534e-01_rb,1.7803e-01_rb,1.7714e-01_rb,1.7022e-01_rb,1.5702e-01_rb, &
     &1.3821e-01_rb,1.1726e-01_rb,9.2624e-02_rb/)
      kao(:, 2, 7, 9) = (/ &
     &2.3477e-01_rb,2.3316e-01_rb,2.3664e-01_rb,2.3403e-01_rb,2.2497e-01_rb,2.0771e-01_rb, &
     &1.8285e-01_rb,1.5521e-01_rb,1.2312e-01_rb/)
      kao(:, 3, 7, 9) = (/ &
     &3.0054e-01_rb,2.9947e-01_rb,3.0371e-01_rb,2.9989e-01_rb,2.8797e-01_rb,2.6596e-01_rb, &
     &2.3476e-01_rb,1.9945e-01_rb,1.5929e-01_rb/)
      kao(:, 4, 7, 9) = (/ &
     &3.7459e-01_rb,3.7495e-01_rb,3.7937e-01_rb,3.7465e-01_rb,3.5915e-01_rb,3.3235e-01_rb, &
     &2.9471e-01_rb,2.5108e-01_rb,2.0370e-01_rb/)
      kao(:, 5, 7, 9) = (/ &
     &4.5653e-01_rb,4.5857e-01_rb,4.6346e-01_rb,4.5781e-01_rb,4.3887e-01_rb,4.0748e-01_rb, &
     &3.6268e-01_rb,3.0976e-01_rb,2.5195e-01_rb/)
      kao(:, 1, 8, 9) = (/ &
     &3.3219e-01_rb,3.0467e-01_rb,2.8618e-01_rb,2.6721e-01_rb,2.4601e-01_rb,2.1811e-01_rb, &
     &1.8145e-01_rb,1.3269e-01_rb,6.6401e-02_rb/)
      kao(:, 2, 8, 9) = (/ &
     &4.4950e-01_rb,4.1374e-01_rb,3.8999e-01_rb,3.6461e-01_rb,3.3388e-01_rb,2.9490e-01_rb, &
     &2.4508e-01_rb,1.7935e-01_rb,9.0026e-02_rb/)
      kao(:, 3, 8, 9) = (/ &
     &5.8717e-01_rb,5.4166e-01_rb,5.1100e-01_rb,4.7736e-01_rb,4.3579e-01_rb,3.8461e-01_rb, &
     &3.1925e-01_rb,2.3358e-01_rb,1.1833e-01_rb/)
      kao(:, 4, 8, 9) = (/ &
     &7.4406e-01_rb,6.8778e-01_rb,6.4996e-01_rb,6.0622e-01_rb,5.5241e-01_rb,4.8728e-01_rb, &
     &4.0419e-01_rb,2.9657e-01_rb,1.5189e-01_rb/)
      kao(:, 5, 8, 9) = (/ &
     &9.1907e-01_rb,8.5125e-01_rb,8.0569e-01_rb,7.5012e-01_rb,6.8369e-01_rb,6.0185e-01_rb, &
     &4.9995e-01_rb,3.6832e-01_rb,1.9101e-01_rb/)
      kao(:, 1, 9, 9) = (/ &
     &1.1485e+00_rb,1.0089e+00_rb,8.7880e-01_rb,7.5762e-01_rb,6.3622e-01_rb,5.1377e-01_rb, &
     &3.9068e-01_rb,2.5323e-01_rb,5.4057e-02_rb/)
      kao(:, 2, 9, 9) = (/ &
     &1.5919e+00_rb,1.3983e+00_rb,1.2193e+00_rb,1.0539e+00_rb,8.8759e-01_rb,7.1827e-01_rb, &
     &5.4603e-01_rb,3.5236e-01_rb,7.3567e-02_rb/)
      kao(:, 3, 9, 9) = (/ &
     &2.1202e+00_rb,1.8629e+00_rb,1.6282e+00_rb,1.4084e+00_rb,1.1875e+00_rb,9.6283e-01_rb, &
     &7.2992e-01_rb,4.6911e-01_rb,9.7269e-02_rb/)
      kao(:, 4, 9, 9) = (/ &
     &2.7354e+00_rb,2.4050e+00_rb,2.1038e+00_rb,1.8214e+00_rb,1.5366e+00_rb,1.2471e+00_rb, &
     &9.4177e-01_rb,6.0391e-01_rb,1.2338e-01_rb/)
      kao(:, 5, 9, 9) = (/ &
     &3.4317e+00_rb,3.0176e+00_rb,2.6421e+00_rb,2.2877e+00_rb,1.9310e+00_rb,1.5677e+00_rb, &
     &1.1814e+00_rb,7.5635e-01_rb,1.5353e-01_rb/)
      kao(:, 1,10, 9) = (/ &
     &4.4436e+00_rb,3.8882e+00_rb,3.3348e+00_rb,2.7888e+00_rb,2.2477e+00_rb,1.7134e+00_rb, &
     &1.1871e+00_rb,6.5782e-01_rb,7.0663e-02_rb/)
      kao(:, 2,10, 9) = (/ &
     &6.2997e+00_rb,5.5124e+00_rb,4.7288e+00_rb,3.9559e+00_rb,3.1897e+00_rb,2.4334e+00_rb, &
     &1.6915e+00_rb,9.3954e-01_rb,8.7418e-02_rb/)
      kao(:, 3,10, 9) = (/ &
     &8.5619e+00_rb,7.4925e+00_rb,6.4279e+00_rb,5.3771e+00_rb,4.3365e+00_rb,3.3136e+00_rb, &
     &2.3060e+00_rb,1.2843e+00_rb,1.0436e-01_rb/)
      kao(:, 4,10, 9) = (/ &
     &1.1219e+01_rb,9.8180e+00_rb,8.4228e+00_rb,7.0486e+00_rb,5.6898e+00_rb,4.3532e+00_rb, &
     &3.0319e+00_rb,1.6916e+00_rb,1.2593e-01_rb/)
      kao(:, 5,10, 9) = (/ &
     &1.4295e+01_rb,1.2510e+01_rb,1.0734e+01_rb,8.9819e+00_rb,7.2530e+00_rb,5.5514e+00_rb, &
     &3.8643e+00_rb,2.1564e+00_rb,1.5004e-01_rb/)
      kao(:, 1,11, 9) = (/ &
     &7.0050e+00_rb,6.1296e+00_rb,5.2549e+00_rb,4.3868e+00_rb,3.5242e+00_rb,2.6669e+00_rb, &
     &1.8245e+00_rb,9.8168e-01_rb,8.5745e-02_rb/)
      kao(:, 2,11, 9) = (/ &
     &9.9554e+00_rb,8.7114e+00_rb,7.4687e+00_rb,6.2351e+00_rb,5.0098e+00_rb,3.7942e+00_rb, &
     &2.5980e+00_rb,1.4001e+00_rb,1.0478e-01_rb/)
      kao(:, 3,11, 9) = (/ &
     &1.3533e+01_rb,1.1842e+01_rb,1.0152e+01_rb,8.4771e+00_rb,6.8123e+00_rb,5.1640e+00_rb, &
     &3.5405e+00_rb,1.9111e+00_rb,1.2573e-01_rb/)
      kao(:, 4,11, 9) = (/ &
     &1.7771e+01_rb,1.5551e+01_rb,1.3333e+01_rb,1.1134e+01_rb,8.9512e+00_rb,6.7876e+00_rb, &
     &4.6577e+00_rb,2.5172e+00_rb,1.4855e-01_rb/)
      kao(:, 5,11, 9) = (/ &
     &2.2680e+01_rb,1.9846e+01_rb,1.7019e+01_rb,1.4214e+01_rb,1.1430e+01_rb,8.6745e+00_rb, &
     &5.9538e+00_rb,3.2175e+00_rb,1.7203e-01_rb/)
      kao(:, 1,12, 9) = (/ &
     &8.1047e+00_rb,7.0920e+00_rb,6.0802e+00_rb,5.0761e+00_rb,4.0767e+00_rb,3.0841e+00_rb, &
     &2.1058e+00_rb,1.1266e+00_rb,9.0852e-02_rb/)
      kao(:, 2,12, 9) = (/ &
     &1.1567e+01_rb,1.0121e+01_rb,8.6775e+00_rb,7.2441e+00_rb,5.8181e+00_rb,4.4027e+00_rb, &
     &3.0080e+00_rb,1.6091e+00_rb,1.1081e-01_rb/)
      kao(:, 3,12, 9) = (/ &
     &1.5756e+01_rb,1.3787e+01_rb,1.1820e+01_rb,9.8684e+00_rb,7.9263e+00_rb,6.0014e+00_rb, &
     &4.1030e+00_rb,2.1969e+00_rb,1.3256e-01_rb/)
      kao(:, 4,12, 9) = (/ &
     &2.0731e+01_rb,1.8141e+01_rb,1.5554e+01_rb,1.2989e+01_rb,1.0436e+01_rb,7.9047e+00_rb, &
     &5.4071e+00_rb,2.8962e+00_rb,1.5690e-01_rb/)
      kao(:, 5,12, 9) = (/ &
     &2.6483e+01_rb,2.3176e+01_rb,1.9872e+01_rb,1.6594e+01_rb,1.3339e+01_rb,1.0111e+01_rb, &
     &6.9150e+00_rb,3.7095e+00_rb,1.8186e-01_rb/)
      kao(:, 1,13, 9) = (/ &
     &7.4699e+00_rb,6.5364e+00_rb,5.6062e+00_rb,4.6835e+00_rb,3.7650e+00_rb,2.8555e+00_rb, &
     &1.9560e+00_rb,1.0515e+00_rb,8.5273e-02_rb/)
      kao(:, 2,13, 9) = (/ &
     &1.0680e+01_rb,9.3454e+00_rb,8.0165e+00_rb,6.6976e+00_rb,5.3854e+00_rb,4.0877e+00_rb, &
     &2.8036e+00_rb,1.5078e+00_rb,1.0411e-01_rb/)
      kao(:, 3,13, 9) = (/ &
     &1.4620e+01_rb,1.2793e+01_rb,1.0973e+01_rb,9.1678e+00_rb,7.3735e+00_rb,5.5972e+00_rb, &
     &3.8382e+00_rb,2.0618e+00_rb,1.2386e-01_rb/)
      kao(:, 4,13, 9) = (/ &
     &1.9258e+01_rb,1.6852e+01_rb,1.4455e+01_rb,1.2079e+01_rb,9.7197e+00_rb,7.3817e+00_rb, &
     &5.0639e+00_rb,2.7225e+00_rb,1.4610e-01_rb/)
      kao(:, 5,13, 9) = (/ &
     &2.4621e+01_rb,2.1546e+01_rb,1.8485e+01_rb,1.5448e+01_rb,1.2432e+01_rb,9.4454e+00_rb, &
     &6.4771e+00_rb,3.4853e+00_rb,1.7007e-01_rb/)
      kao(:, 1, 1,10) = (/ &
     &1.3879e-02_rb,1.0986e-01_rb,1.9192e-01_rb,2.8700e-01_rb,3.8129e-01_rb,4.7527e-01_rb, &
     &5.7067e-01_rb,6.6779e-01_rb,7.6243e-01_rb/)
      kao(:, 2, 1,10) = (/ &
     &1.6954e-02_rb,1.3303e-01_rb,2.3564e-01_rb,3.5200e-01_rb,4.6702e-01_rb,5.8054e-01_rb, &
     &6.9523e-01_rb,8.1372e-01_rb,9.3367e-01_rb/)
      kao(:, 3, 1,10) = (/ &
     &2.0369e-02_rb,1.5906e-01_rb,2.8523e-01_rb,4.2640e-01_rb,5.6627e-01_rb,7.0405e-01_rb, &
     &8.3748e-01_rb,9.6604e-01_rb,1.1321e+00_rb/)
      kao(:, 4, 1,10) = (/ &
     &2.4061e-02_rb,1.8742e-01_rb,3.4065e-01_rb,5.0987e-01_rb,6.7810e-01_rb,8.4492e-01_rb, &
     &1.0078e+00_rb,1.1403e+00_rb,1.3558e+00_rb/)
      kao(:, 5, 1,10) = (/ &
     &2.7911e-02_rb,2.1680e-01_rb,4.0347e-01_rb,6.0365e-01_rb,8.0163e-01_rb,9.9663e-01_rb, &
     &1.1842e+00_rb,1.3430e+00_rb,1.6028e+00_rb/)
      kao(:, 1, 2,10) = (/ &
     &2.2158e-02_rb,1.1276e-01_rb,1.8891e-01_rb,2.7518e-01_rb,3.6524e-01_rb,4.5317e-01_rb, &
     &5.3652e-01_rb,6.1296e-01_rb,7.3020e-01_rb/)
      kao(:, 2, 2,10) = (/ &
     &2.7256e-02_rb,1.3808e-01_rb,2.3100e-01_rb,3.4051e-01_rb,4.5180e-01_rb,5.6015e-01_rb, &
     &6.6160e-01_rb,7.4973e-01_rb,9.0314e-01_rb/)
      kao(:, 3, 2,10) = (/ &
     &3.2742e-02_rb,1.6645e-01_rb,2.7575e-01_rb,4.1007e-01_rb,5.4427e-01_rb,6.7555e-01_rb, &
     &8.0050e-01_rb,9.1019e-01_rb,1.0879e+00_rb/)
      kao(:, 4, 2,10) = (/ &
     &3.8593e-02_rb,1.9807e-01_rb,3.2475e-01_rb,4.8552e-01_rb,6.4531e-01_rb,8.0341e-01_rb, &
     &9.5786e-01_rb,1.0994e+00_rb,1.2898e+00_rb/)
      kao(:, 5, 2,10) = (/ &
     &4.4713e-02_rb,2.3139e-01_rb,3.8401e-01_rb,5.7431e-01_rb,7.6329e-01_rb,9.5026e-01_rb, &
     &1.1317e+00_rb,1.3007e+00_rb,1.5256e+00_rb/)
      kao(:, 1, 3,10) = (/ &
     &4.4889e-02_rb,1.1945e-01_rb,1.9910e-01_rb,2.6592e-01_rb,3.2716e-01_rb,4.0820e-01_rb, &
     &4.8832e-01_rb,5.6450e-01_rb,6.5333e-01_rb/)
      kao(:, 2, 3,10) = (/ &
     &5.6096e-02_rb,1.4847e-01_rb,2.4845e-01_rb,3.3071e-01_rb,4.1171e-01_rb,5.1418e-01_rb, &
     &6.1578e-01_rb,7.1081e-01_rb,8.2224e-01_rb/)
      kao(:, 3, 3,10) = (/ &
     &6.8702e-02_rb,1.8161e-01_rb,3.0124e-01_rb,3.9617e-01_rb,5.0089e-01_rb,6.2554e-01_rb, &
     &7.4933e-01_rb,8.6662e-01_rb,1.0003e+00_rb/)
      kao(:, 4, 3,10) = (/ &
     &8.1949e-02_rb,2.1813e-01_rb,3.5788e-01_rb,4.6675e-01_rb,6.0112e-01_rb,7.4982e-01_rb, &
     &8.9722e-01_rb,1.0369e+00_rb,1.1996e+00_rb/)
      kao(:, 5, 3,10) = (/ &
     &9.5752e-02_rb,2.5792e-01_rb,4.1860e-01_rb,5.4742e-01_rb,7.1669e-01_rb,8.9255e-01_rb, &
     &1.0648e+00_rb,1.2234e+00_rb,1.4293e+00_rb/)
      kao(:, 1, 4,10) = (/ &
     &9.2779e-02_rb,1.3647e-01_rb,2.0374e-01_rb,2.6880e-01_rb,3.2474e-01_rb,3.6630e-01_rb, &
     &4.2915e-01_rb,4.9644e-01_rb,5.7357e-01_rb/)
      kao(:, 2, 4,10) = (/ &
     &1.1604e-01_rb,1.7477e-01_rb,2.5924e-01_rb,3.4288e-01_rb,4.1143e-01_rb,4.5661e-01_rb, &
     &5.3933e-01_rb,6.2792e-01_rb,7.1853e-01_rb/)
      kao(:, 3, 4,10) = (/ &
     &1.4325e-01_rb,2.1810e-01_rb,3.2204e-01_rb,4.2345e-01_rb,5.0270e-01_rb,5.5390e-01_rb, &
     &6.5839e-01_rb,7.6862e-01_rb,8.7526e-01_rb/)
      kao(:, 4, 4,10) = (/ &
     &1.7320e-01_rb,2.6694e-01_rb,3.8938e-01_rb,5.0945e-01_rb,6.0406e-01_rb,6.6936e-01_rb, &
     &8.0058e-01_rb,9.3405e-01_rb,1.0639e+00_rb/)
      kao(:, 5, 4,10) = (/ &
     &2.0528e-01_rb,3.2153e-01_rb,4.6268e-01_rb,6.0392e-01_rb,7.1636e-01_rb,8.0678e-01_rb, &
     &9.6596e-01_rb,1.1248e+00_rb,1.2848e+00_rb/)
      kao(:, 1, 5,10) = (/ &
     &1.7241e-01_rb,1.8815e-01_rb,2.2088e-01_rb,2.6810e-01_rb,3.2220e-01_rb,3.6810e-01_rb, &
     &3.9585e-01_rb,4.3476e-01_rb,4.9559e-01_rb/)
      kao(:, 2, 5,10) = (/ &
     &2.1928e-01_rb,2.4297e-01_rb,2.8762e-01_rb,3.4907e-01_rb,4.1506e-01_rb,4.6919e-01_rb, &
     &4.9756e-01_rb,5.5222e-01_rb,6.2884e-01_rb/)
      kao(:, 3, 5,10) = (/ &
     &2.7265e-01_rb,3.0607e-01_rb,3.6483e-01_rb,4.4370e-01_rb,5.1761e-01_rb,5.7690e-01_rb, &
     &6.0530e-01_rb,6.7866e-01_rb,7.7290e-01_rb/)
      kao(:, 4, 5,10) = (/ &
     &3.3288e-01_rb,3.7610e-01_rb,4.5166e-01_rb,5.4662e-01_rb,6.2951e-01_rb,6.9865e-01_rb, &
     &7.2973e-01_rb,8.2842e-01_rb,9.4575e-01_rb/)
      kao(:, 5, 5,10) = (/ &
     &3.9915e-01_rb,4.5195e-01_rb,5.4707e-01_rb,6.6103e-01_rb,7.5773e-01_rb,8.3891e-01_rb, &
     &8.7097e-01_rb,1.0005e+00_rb,1.1404e+00_rb/)
      kao(:, 1, 6,10) = (/ &
     &2.9778e-01_rb,2.8967e-01_rb,2.9242e-01_rb,2.9076e-01_rb,3.1606e-01_rb,3.5216e-01_rb, &
     &3.7514e-01_rb,3.6360e-01_rb,4.0286e-01_rb/)
      kao(:, 2, 6,10) = (/ &
     &3.8364e-01_rb,3.7347e-01_rb,3.8384e-01_rb,3.8402e-01_rb,4.1811e-01_rb,4.6141e-01_rb, &
     &4.8908e-01_rb,4.6937e-01_rb,5.2349e-01_rb/)
      kao(:, 3, 6,10) = (/ &
     &4.8196e-01_rb,4.7366e-01_rb,4.8805e-01_rb,4.9747e-01_rb,5.3761e-01_rb,5.8138e-01_rb, &
     &6.1239e-01_rb,5.8662e-01_rb,6.5890e-01_rb/)
      kao(:, 4, 6,10) = (/ &
     &5.9004e-01_rb,5.9246e-01_rb,6.0698e-01_rb,6.2535e-01_rb,6.7125e-01_rb,7.1432e-01_rb, &
     &7.4926e-01_rb,7.2627e-01_rb,8.2202e-01_rb/)
      kao(:, 5, 6,10) = (/ &
     &7.1225e-01_rb,7.2336e-01_rb,7.3969e-01_rb,7.6299e-01_rb,8.2248e-01_rb,8.6951e-01_rb, &
     &9.0679e-01_rb,8.7991e-01_rb,1.0003e+00_rb/)
      kao(:, 1, 7,10) = (/ &
     &5.2983e-01_rb,4.8709e-01_rb,4.4813e-01_rb,4.1883e-01_rb,3.8710e-01_rb,3.5910e-01_rb, &
     &3.5198e-01_rb,3.4364e-01_rb,3.2708e-01_rb/)
      kao(:, 2, 7,10) = (/ &
     &6.9820e-01_rb,6.4182e-01_rb,5.9773e-01_rb,5.6199e-01_rb,5.1585e-01_rb,4.8198e-01_rb, &
     &4.7538e-01_rb,4.5593e-01_rb,4.2838e-01_rb/)
      kao(:, 3, 7,10) = (/ &
     &8.9166e-01_rb,8.2011e-01_rb,7.7520e-01_rb,7.2195e-01_rb,6.6977e-01_rb,6.2908e-01_rb, &
     &6.1720e-01_rb,5.8358e-01_rb,5.4876e-01_rb/)
      kao(:, 4, 7,10) = (/ &
     &1.1048e+00_rb,1.0150e+00_rb,9.7493e-01_rb,9.0555e-01_rb,8.5261e-01_rb,7.9496e-01_rb, &
     &7.7495e-01_rb,7.2954e-01_rb,6.9328e-01_rb/)
      kao(:, 5, 7,10) = (/ &
     &1.3386e+00_rb,1.2295e+00_rb,1.1950e+00_rb,1.1151e+00_rb,1.0543e+00_rb,9.8064e-01_rb, &
     &9.5591e-01_rb,8.9484e-01_rb,8.5404e-01_rb/)
      kao(:, 1, 8,10) = (/ &
     &1.0881e+00_rb,9.5225e-01_rb,8.5699e-01_rb,7.4043e-01_rb,6.3719e-01_rb,5.4699e-01_rb, &
     &4.4366e-01_rb,3.3722e-01_rb,2.5365e-01_rb/)
      kao(:, 2, 8,10) = (/ &
     &1.4729e+00_rb,1.2892e+00_rb,1.1603e+00_rb,1.0057e+00_rb,8.7735e-01_rb,7.4624e-01_rb, &
     &6.0685e-01_rb,4.5839e-01_rb,3.4968e-01_rb/)
      kao(:, 3, 8,10) = (/ &
     &1.9193e+00_rb,1.6816e+00_rb,1.5156e+00_rb,1.3265e+00_rb,1.1534e+00_rb,9.7741e-01_rb, &
     &7.9951e-01_rb,6.0809e-01_rb,4.5275e-01_rb/)
      kao(:, 4, 8,10) = (/ &
     &2.4196e+00_rb,2.1220e+00_rb,1.9097e+00_rb,1.6942e+00_rb,1.4693e+00_rb,1.2439e+00_rb, &
     &1.0194e+00_rb,7.8139e-01_rb,5.7536e-01_rb/)
      kao(:, 5, 8,10) = (/ &
     &2.9662e+00_rb,2.6064e+00_rb,2.3390e+00_rb,2.0966e+00_rb,1.8122e+00_rb,1.5517e+00_rb, &
     &1.2698e+00_rb,9.8175e-01_rb,7.1420e-01_rb/)
      kao(:, 1, 9,10) = (/ &
     &4.0693e+00_rb,3.5609e+00_rb,3.0521e+00_rb,2.5470e+00_rb,2.0946e+00_rb,1.6231e+00_rb, &
     &1.1342e+00_rb,6.9341e-01_rb,2.3254e-01_rb/)
      kao(:, 2, 9,10) = (/ &
     &5.6525e+00_rb,4.9459e+00_rb,4.2396e+00_rb,3.5367e+00_rb,2.9143e+00_rb,2.2533e+00_rb, &
     &1.5867e+00_rb,9.6567e-01_rb,2.9706e-01_rb/)
      kao(:, 3, 9,10) = (/ &
     &7.5336e+00_rb,6.5921e+00_rb,5.6502e+00_rb,4.7197e+00_rb,3.8915e+00_rb,3.0030e+00_rb, &
     &2.1420e+00_rb,1.2862e+00_rb,3.6402e-01_rb/)
      kao(:, 4, 9,10) = (/ &
     &9.6696e+00_rb,8.4608e+00_rb,7.2526e+00_rb,6.0675e+00_rb,4.9886e+00_rb,3.8558e+00_rb, &
     &2.7779e+00_rb,1.6618e+00_rb,4.0961e-01_rb/)
      kao(:, 5, 9,10) = (/ &
     &1.2059e+01_rb,1.0552e+01_rb,9.0447e+00_rb,7.5858e+00_rb,6.2136e+00_rb,4.8233e+00_rb, &
     &3.4799e+00_rb,2.0915e+00_rb,3.7531e-01_rb/)
      kao(:, 1,10,10) = (/ &
     &1.7088e+01_rb,1.4952e+01_rb,1.2816e+01_rb,1.0681e+01_rb,8.5444e+00_rb,6.4081e+00_rb, &
     &4.3169e+00_rb,2.2612e+00_rb,1.5325e-01_rb/)
      kao(:, 2,10,10) = (/ &
     &2.4265e+01_rb,2.1232e+01_rb,1.8199e+01_rb,1.5167e+01_rb,1.2132e+01_rb,9.0996e+00_rb, &
     &6.1194e+00_rb,3.2115e+00_rb,2.0907e-01_rb/)
      kao(:, 3,10,10) = (/ &
     &3.2859e+01_rb,2.8752e+01_rb,2.4646e+01_rb,2.0537e+01_rb,1.6430e+01_rb,1.2323e+01_rb, &
     &8.2885e+00_rb,4.3528e+00_rb,2.8093e-01_rb/)
      kao(:, 4,10,10) = (/ &
     &4.2869e+01_rb,3.7512e+01_rb,3.2153e+01_rb,2.6795e+01_rb,2.1435e+01_rb,1.6077e+01_rb, &
     &1.0833e+01_rb,5.6764e+00_rb,3.4638e-01_rb/)
      kao(:, 5,10,10) = (/ &
     &5.4098e+01_rb,4.7336e+01_rb,4.0575e+01_rb,3.3816e+01_rb,2.7052e+01_rb,2.0291e+01_rb, &
     &1.3707e+01_rb,7.1952e+00_rb,4.8935e-01_rb/)
      kao(:, 1,11,10) = (/ &
     &2.9317e+01_rb,2.5652e+01_rb,2.1987e+01_rb,1.8324e+01_rb,1.4658e+01_rb,1.0994e+01_rb, &
     &7.3313e+00_rb,3.7877e+00_rb,2.2554e-01_rb/)
      kao(:, 2,11,10) = (/ &
     &4.1549e+01_rb,3.6355e+01_rb,3.1162e+01_rb,2.5968e+01_rb,2.0775e+01_rb,1.5581e+01_rb, &
     &1.0388e+01_rb,5.3868e+00_rb,2.3983e-01_rb/)
      kao(:, 3,11,10) = (/ &
     &5.6191e+01_rb,4.9168e+01_rb,4.2143e+01_rb,3.5117e+01_rb,2.8096e+01_rb,2.1071e+01_rb, &
     &1.4052e+01_rb,7.2903e+00_rb,2.8347e-01_rb/)
      kao(:, 4,11,10) = (/ &
     &7.3162e+01_rb,6.4016e+01_rb,5.4871e+01_rb,4.5729e+01_rb,3.6583e+01_rb,2.7439e+01_rb, &
     &1.8309e+01_rb,9.4965e+00_rb,3.4363e-01_rb/)
      kao(:, 5,11,10) = (/ &
     &9.2126e+01_rb,8.0611e+01_rb,6.9095e+01_rb,5.7582e+01_rb,4.6068e+01_rb,3.4554e+01_rb, &
     &2.3073e+01_rb,1.1954e+01_rb,4.6470e-01_rb/)
      kao(:, 1,12,10) = (/ &
     &3.6684e+01_rb,3.2099e+01_rb,2.7514e+01_rb,2.2927e+01_rb,1.8342e+01_rb,1.3757e+01_rb, &
     &9.1743e+00_rb,4.7217e+00_rb,2.6068e-01_rb/)
      kao(:, 2,12,10) = (/ &
     &5.1868e+01_rb,4.5387e+01_rb,3.8901e+01_rb,3.2419e+01_rb,2.5934e+01_rb,1.9452e+01_rb, &
     &1.2967e+01_rb,6.6916e+00_rb,2.7916e-01_rb/)
      kao(:, 3,12,10) = (/ &
     &7.0081e+01_rb,6.1322e+01_rb,5.2561e+01_rb,4.3801e+01_rb,3.5043e+01_rb,2.6282e+01_rb, &
     &1.7526e+01_rb,9.0530e+00_rb,3.3145e-01_rb/)
      kao(:, 4,12,10) = (/ &
     &9.0957e+01_rb,7.9591e+01_rb,6.8227e+01_rb,5.6859e+01_rb,4.5488e+01_rb,3.4114e+01_rb, &
     &2.2760e+01_rb,1.1751e+01_rb,3.6053e-01_rb/)
      kao(:, 5,12,10) = (/ &
     &1.1478e+02_rb,1.0044e+02_rb,8.6086e+01_rb,7.1743e+01_rb,5.7394e+01_rb,4.3046e+01_rb, &
     &2.8737e+01_rb,1.4814e+01_rb,4.5144e-01_rb/)
      kao(:, 1,13,10) = (/ &
     &3.5965e+01_rb,3.1472e+01_rb,2.6975e+01_rb,2.2479e+01_rb,1.7984e+01_rb,1.3488e+01_rb, &
     &9.0232e+00_rb,4.6648e+00_rb,2.4705e-01_rb/)
      kao(:, 2,13,10) = (/ &
     &5.0800e+01_rb,4.4451e+01_rb,3.8101e+01_rb,3.1752e+01_rb,2.5403e+01_rb,1.9053e+01_rb, &
     &1.2739e+01_rb,6.5982e+00_rb,2.8941e-01_rb/)
      kao(:, 3,13,10) = (/ &
     &6.8393e+01_rb,5.9846e+01_rb,5.1296e+01_rb,4.2746e+01_rb,3.4199e+01_rb,2.5650e+01_rb, &
     &1.7151e+01_rb,8.8890e+00_rb,3.3776e-01_rb/)
      kao(:, 4,13,10) = (/ &
     &8.8957e+01_rb,7.7837e+01_rb,6.6721e+01_rb,5.5604e+01_rb,4.4485e+01_rb,3.3364e+01_rb, &
     &2.2331e+01_rb,1.1554e+01_rb,3.9400e-01_rb/)
      kao(:, 5,13,10) = (/ &
     &1.1248e+02_rb,9.8422e+01_rb,8.4359e+01_rb,7.0296e+01_rb,5.6240e+01_rb,4.2182e+01_rb, &
     &2.8274e+01_rb,1.4614e+01_rb,4.6294e-01_rb/)
      kao(:, 1, 1,11) = (/ &
     &2.0991e-02_rb,1.4510e-01_rb,2.7530e-01_rb,4.1253e-01_rb,5.4888e-01_rb,6.8317e-01_rb, &
     &8.1040e-01_rb,9.0204e-01_rb,1.0977e+00_rb/)
      kao(:, 2, 1,11) = (/ &
     &2.6377e-02_rb,1.7420e-01_rb,3.3534e-01_rb,5.0284e-01_rb,6.6978e-01_rb,8.3508e-01_rb, &
     &9.9362e-01_rb,1.1152e+00_rb,1.3395e+00_rb/)
      kao(:, 3, 1,11) = (/ &
     &3.2301e-02_rb,2.0470e-01_rb,4.0014e-01_rb,5.9936e-01_rb,7.9730e-01_rb,9.9294e-01_rb, &
     &1.1840e+00_rb,1.3371e+00_rb,1.5943e+00_rb/)
      kao(:, 4, 1,11) = (/ &
     &3.8610e-02_rb,2.3990e-01_rb,4.7445e-01_rb,7.0939e-01_rb,9.4186e-01_rb,1.1687e+00_rb, &
     &1.3842e+00_rb,1.5657e+00_rb,1.8832e+00_rb/)
      kao(:, 5, 1,11) = (/ &
     &4.5285e-02_rb,2.8044e-01_rb,5.5866e-01_rb,8.3454e-01_rb,1.1074e+00_rb,1.3742e+00_rb, &
     &1.6243e+00_rb,1.8087e+00_rb,2.2141e+00_rb/)
      kao(:, 1, 2,11) = (/ &
     &3.0334e-02_rb,1.5806e-01_rb,2.5391e-01_rb,3.8056e-01_rb,5.0695e-01_rb,6.3318e-01_rb, &
     &7.5908e-01_rb,8.7813e-01_rb,1.0138e+00_rb/)
      kao(:, 2, 2,11) = (/ &
     &3.8224e-02_rb,1.9234e-01_rb,3.1173e-01_rb,4.6706e-01_rb,6.2209e-01_rb,7.7707e-01_rb, &
     &9.3334e-01_rb,1.0857e+00_rb,1.2438e+00_rb/)
      kao(:, 3, 2,11) = (/ &
     &4.7042e-02_rb,2.2793e-01_rb,3.7775e-01_rb,5.6573e-01_rb,7.5275e-01_rb,9.3856e-01_rb, &
     &1.1227e+00_rb,1.3034e+00_rb,1.5049e+00_rb/)
      kao(:, 4, 2,11) = (/ &
     &5.6790e-02_rb,2.6663e-01_rb,4.5515e-01_rb,6.8101e-01_rb,9.0489e-01_rb,1.1244e+00_rb, &
     &1.3363e+00_rb,1.5352e+00_rb,1.8090e+00_rb/)
      kao(:, 5, 2,11) = (/ &
     &6.7198e-02_rb,3.0909e-01_rb,5.3855e-01_rb,8.0620e-01_rb,1.0720e+00_rb,1.3339e+00_rb, &
     &1.5872e+00_rb,1.8075e+00_rb,2.1432e+00_rb/)
      kao(:, 1, 3,11) = (/ &
     &5.8386e-02_rb,1.7533e-01_rb,2.7134e-01_rb,3.4280e-01_rb,4.5327e-01_rb,5.6550e-01_rb, &
     &6.7606e-01_rb,7.8244e-01_rb,9.0646e-01_rb/)
      kao(:, 2, 3,11) = (/ &
     &7.4136e-02_rb,2.1912e-01_rb,3.3311e-01_rb,4.2266e-01_rb,5.6203e-01_rb,7.0075e-01_rb, &
     &8.3707e-01_rb,9.6959e-01_rb,1.1236e+00_rb/)
      kao(:, 3, 3,11) = (/ &
     &9.1573e-02_rb,2.6509e-01_rb,4.0096e-01_rb,5.1730e-01_rb,6.8812e-01_rb,8.5747e-01_rb, &
     &1.0237e+00_rb,1.1823e+00_rb,1.3753e+00_rb/)
      kao(:, 4, 3,11) = (/ &
     &1.1115e-01_rb,3.1410e-01_rb,4.7626e-01_rb,6.2447e-01_rb,8.3118e-01_rb,1.0365e+00_rb, &
     &1.2383e+00_rb,1.4280e+00_rb,1.6615e+00_rb/)
      kao(:, 5, 3,11) = (/ &
     &1.3245e-01_rb,3.6725e-01_rb,5.5683e-01_rb,7.4336e-01_rb,9.8988e-01_rb,1.2355e+00_rb, &
     &1.4780e+00_rb,1.7132e+00_rb,1.9788e+00_rb/)
      kao(:, 1, 4,11) = (/ &
     &1.1641e-01_rb,1.9770e-01_rb,2.9439e-01_rb,3.7525e-01_rb,4.3722e-01_rb,5.0094e-01_rb, &
     &6.0009e-01_rb,6.9731e-01_rb,8.0196e-01_rb/)
      kao(:, 2, 4,11) = (/ &
     &1.4966e-01_rb,2.5765e-01_rb,3.7252e-01_rb,4.6834e-01_rb,5.4216e-01_rb,6.3606e-01_rb, &
     &7.6075e-01_rb,8.7975e-01_rb,1.0184e+00_rb/)
      kao(:, 3, 4,11) = (/ &
     &1.8710e-01_rb,3.2245e-01_rb,4.5679e-01_rb,5.7205e-01_rb,6.6194e-01_rb,7.9151e-01_rb, &
     &9.4643e-01_rb,1.0919e+00_rb,1.2671e+00_rb/)
      kao(:, 4, 4,11) = (/ &
     &2.2853e-01_rb,3.9201e-01_rb,5.5041e-01_rb,6.8810e-01_rb,7.8876e-01_rb,9.5787e-01_rb, &
     &1.1458e+00_rb,1.3241e+00_rb,1.5335e+00_rb/)
      kao(:, 5, 4,11) = (/ &
     &2.7304e-01_rb,4.6530e-01_rb,6.5411e-01_rb,8.1513e-01_rb,9.2682e-01_rb,1.1406e+00_rb, &
     &1.3653e+00_rb,1.5825e+00_rb,1.8252e+00_rb/)
      kao(:, 1, 5,11) = (/ &
     &2.2361e-01_rb,2.4318e-01_rb,3.1539e-01_rb,3.9443e-01_rb,4.5716e-01_rb,5.0109e-01_rb, &
     &5.2154e-01_rb,6.0679e-01_rb,6.9571e-01_rb/)
      kao(:, 2, 5,11) = (/ &
     &2.8781e-01_rb,3.2451e-01_rb,4.1536e-01_rb,5.0630e-01_rb,5.8344e-01_rb,6.3855e-01_rb, &
     &6.7297e-01_rb,7.8266e-01_rb,8.9726e-01_rb/)
      kao(:, 3, 5,11) = (/ &
     &3.6159e-01_rb,4.1915e-01_rb,5.2707e-01_rb,6.2976e-01_rb,7.2674e-01_rb,7.9740e-01_rb, &
     &8.4985e-01_rb,9.8938e-01_rb,1.1325e+00_rb/)
      kao(:, 4, 5,11) = (/ &
     &4.4416e-01_rb,5.2452e-01_rb,6.5017e-01_rb,7.6880e-01_rb,8.8658e-01_rb,9.6270e-01_rb, &
     &1.0399e+00_rb,1.2125e+00_rb,1.3846e+00_rb/)
      kao(:, 5, 5,11) = (/ &
     &5.3394e-01_rb,6.3638e-01_rb,7.8407e-01_rb,9.2197e-01_rb,1.0572e+00_rb,1.1328e+00_rb, &
     &1.2555e+00_rb,1.4611e+00_rb,1.6711e+00_rb/)
      kao(:, 1, 6,11) = (/ &
     &3.9154e-01_rb,3.6960e-01_rb,3.6660e-01_rb,4.0826e-01_rb,4.6123e-01_rb,5.0084e-01_rb, &
     &5.2154e-01_rb,5.2728e-01_rb,6.0320e-01_rb/)
      kao(:, 2, 6,11) = (/ &
     &5.1275e-01_rb,4.8556e-01_rb,4.9732e-01_rb,5.4788e-01_rb,6.0699e-01_rb,6.5333e-01_rb, &
     &6.7418e-01_rb,6.8648e-01_rb,7.8580e-01_rb/)
      kao(:, 3, 6,11) = (/ &
     &6.5141e-01_rb,6.2128e-01_rb,6.5130e-01_rb,7.0712e-01_rb,7.7202e-01_rb,8.3303e-01_rb, &
     &8.5314e-01_rb,8.6953e-01_rb,9.9315e-01_rb/)
      kao(:, 4, 6,11) = (/ &
     &8.1112e-01_rb,7.6989e-01_rb,8.2193e-01_rb,8.8492e-01_rb,9.6073e-01_rb,1.0362e+00_rb, &
     &1.0443e+00_rb,1.0666e+00_rb,1.2159e+00_rb/)
      kao(:, 5, 6,11) = (/ &
     &9.8628e-01_rb,9.3596e-01_rb,1.0065e+00_rb,1.0840e+00_rb,1.1687e+00_rb,1.2489e+00_rb, &
     &1.2474e+00_rb,1.2983e+00_rb,1.4796e+00_rb/)
      kao(:, 1, 7,11) = (/ &
     &6.9811e-01_rb,6.2404e-01_rb,5.7431e-01_rb,5.2596e-01_rb,5.0219e-01_rb,5.1011e-01_rb, &
     &5.2289e-01_rb,4.9946e-01_rb,5.0228e-01_rb/)
      kao(:, 2, 7,11) = (/ &
     &9.3257e-01_rb,8.3430e-01_rb,7.6751e-01_rb,7.1360e-01_rb,6.8353e-01_rb,6.8813e-01_rb, &
     &6.9358e-01_rb,6.6018e-01_rb,6.7200e-01_rb/)
      kao(:, 3, 7,11) = (/ &
     &1.1984e+00_rb,1.0755e+00_rb,9.9660e-01_rb,9.4008e-01_rb,8.9760e-01_rb,8.9906e-01_rb, &
     &8.9631e-01_rb,8.4016e-01_rb,8.6247e-01_rb/)
      kao(:, 4, 7,11) = (/ &
     &1.5061e+00_rb,1.3555e+00_rb,1.2704e+00_rb,1.2009e+00_rb,1.1418e+00_rb,1.1469e+00_rb, &
     &1.1302e+00_rb,1.0283e+00_rb,1.0687e+00_rb/)
      kao(:, 5, 7,11) = (/ &
     &1.8551e+00_rb,1.6736e+00_rb,1.5791e+00_rb,1.4850e+00_rb,1.4175e+00_rb,1.4225e+00_rb, &
     &1.3804e+00_rb,1.2333e+00_rb,1.3098e+00_rb/)
      kao(:, 1, 8,11) = (/ &
     &1.4751e+00_rb,1.2907e+00_rb,1.1413e+00_rb,9.9788e-01_rb,8.3765e-01_rb,7.0312e-01_rb, &
     &5.8437e-01_rb,5.1212e-01_rb,4.0848e-01_rb/)
      kao(:, 2, 8,11) = (/ &
     &2.0110e+00_rb,1.7597e+00_rb,1.5595e+00_rb,1.3603e+00_rb,1.1539e+00_rb,9.7291e-01_rb, &
     &7.9698e-01_rb,7.0110e-01_rb,5.5259e-01_rb/)
      kao(:, 3, 8,11) = (/ &
     &2.6286e+00_rb,2.3001e+00_rb,2.0385e+00_rb,1.7785e+00_rb,1.5362e+00_rb,1.2889e+00_rb, &
     &1.0590e+00_rb,9.2250e-01_rb,7.2275e-01_rb/)
      kao(:, 4, 8,11) = (/ &
     &3.3350e+00_rb,2.9182e+00_rb,2.5850e+00_rb,2.2617e+00_rb,1.9780e+00_rb,1.6580e+00_rb, &
     &1.3758e+00_rb,1.1729e+00_rb,9.1532e-01_rb/)
      kao(:, 5, 8,11) = (/ &
     &4.1346e+00_rb,3.6181e+00_rb,3.2106e+00_rb,2.8264e+00_rb,2.4835e+00_rb,2.0714e+00_rb, &
     &1.7385e+00_rb,1.4468e+00_rb,1.1430e+00_rb/)
      kao(:, 1, 9,11) = (/ &
     &5.7681e+00_rb,5.0468e+00_rb,4.3260e+00_rb,3.6050e+00_rb,2.9109e+00_rb,2.2673e+00_rb, &
     &1.5776e+00_rb,9.1302e-01_rb,3.3856e-01_rb/)
      kao(:, 2, 9,11) = (/ &
     &8.0519e+00_rb,7.0463e+00_rb,6.0395e+00_rb,5.0331e+00_rb,4.0586e+00_rb,3.1762e+00_rb, &
     &2.2073e+00_rb,1.2968e+00_rb,3.9705e-01_rb/)
      kao(:, 3, 9,11) = (/ &
     &1.0740e+01_rb,9.3971e+00_rb,8.0549e+00_rb,6.7129e+00_rb,5.4190e+00_rb,4.2355e+00_rb, &
     &2.9531e+00_rb,1.7521e+00_rb,4.0437e-01_rb/)
      kao(:, 4, 9,11) = (/ &
     &1.3840e+01_rb,1.2110e+01_rb,1.0380e+01_rb,8.6502e+00_rb,7.0077e+00_rb,5.4549e+00_rb, &
     &3.8354e+00_rb,2.2704e+00_rb,5.4385e-01_rb/)
      kao(:, 5, 9,11) = (/ &
     &1.7291e+01_rb,1.5129e+01_rb,1.2969e+01_rb,1.0808e+01_rb,8.7913e+00_rb,6.8118e+00_rb, &
     &4.8594e+00_rb,2.8643e+00_rb,9.3844e-01_rb/)
      kao(:, 1,10,11) = (/ &
     &2.5281e+01_rb,2.2121e+01_rb,1.8961e+01_rb,1.5801e+01_rb,1.2641e+01_rb,9.4806e+00_rb, &
     &6.3207e+00_rb,3.2921e+00_rb,2.8225e-01_rb/)
      kao(:, 2,10,11) = (/ &
     &3.6089e+01_rb,3.1577e+01_rb,2.7066e+01_rb,2.2555e+01_rb,1.8044e+01_rb,1.3532e+01_rb, &
     &9.0226e+00_rb,4.7197e+00_rb,3.9375e-01_rb/)
      kao(:, 3,10,11) = (/ &
     &4.8997e+01_rb,4.2873e+01_rb,3.6749e+01_rb,3.0623e+01_rb,2.4498e+01_rb,1.8373e+01_rb, &
     &1.2249e+01_rb,6.4130e+00_rb,6.0950e-01_rb/)
      kao(:, 4,10,11) = (/ &
     &6.4090e+01_rb,5.6079e+01_rb,4.8068e+01_rb,4.0059e+01_rb,3.2048e+01_rb,2.4036e+01_rb, &
     &1.6029e+01_rb,8.3902e+00_rb,8.3229e-01_rb/)
      kao(:, 5,10,11) = (/ &
     &8.0977e+01_rb,7.0856e+01_rb,6.0735e+01_rb,5.0611e+01_rb,4.0489e+01_rb,3.0368e+01_rb, &
     &2.0260e+01_rb,1.0591e+01_rb,9.6456e-01_rb/)
      kao(:, 1,11,11) = (/ &
     &4.5405e+01_rb,3.9730e+01_rb,3.4055e+01_rb,2.8379e+01_rb,2.2704e+01_rb,1.7027e+01_rb, &
     &1.1352e+01_rb,5.7629e+00_rb,2.2978e-01_rb/)
      kao(:, 2,11,11) = (/ &
     &6.4516e+01_rb,5.6450e+01_rb,4.8385e+01_rb,4.0321e+01_rb,3.2258e+01_rb,2.4193e+01_rb, &
     &1.6129e+01_rb,8.1743e+00_rb,3.8450e-01_rb/)
      kao(:, 3,11,11) = (/ &
     &8.7570e+01_rb,7.6629e+01_rb,6.5678e+01_rb,5.4738e+01_rb,4.3785e+01_rb,3.2840e+01_rb, &
     &2.1894e+01_rb,1.1105e+01_rb,5.0528e-01_rb/)
      kao(:, 4,11,11) = (/ &
     &1.1421e+02_rb,9.9924e+01_rb,8.5650e+01_rb,7.1372e+01_rb,5.7099e+01_rb,4.2826e+01_rb, &
     &2.8549e+01_rb,1.4506e+01_rb,6.3580e-01_rb/)
      kao(:, 5,11,11) = (/ &
     &1.4404e+02_rb,1.2604e+02_rb,1.0803e+02_rb,9.0033e+01_rb,7.2026e+01_rb,5.4020e+01_rb, &
     &3.6011e+01_rb,1.8334e+01_rb,7.9612e-01_rb/)
      kao(:, 1,12,11) = (/ &
     &5.9271e+01_rb,5.1859e+01_rb,4.4453e+01_rb,3.7044e+01_rb,2.9635e+01_rb,2.2226e+01_rb, &
     &1.4818e+01_rb,7.4861e+00_rb,2.2024e-01_rb/)
      kao(:, 2,12,11) = (/ &
     &8.4185e+01_rb,7.3665e+01_rb,6.3138e+01_rb,5.2615e+01_rb,4.2094e+01_rb,3.1570e+01_rb, &
     &2.1046e+01_rb,1.0615e+01_rb,3.8802e-01_rb/)
      kao(:, 3,12,11) = (/ &
     &1.1399e+02_rb,9.9749e+01_rb,8.5496e+01_rb,7.1245e+01_rb,5.6997e+01_rb,4.2749e+01_rb, &
     &2.8499e+01_rb,1.4383e+01_rb,4.9342e-01_rb/)
      kao(:, 4,12,11) = (/ &
     &1.4842e+02_rb,1.2987e+02_rb,1.1132e+02_rb,9.2770e+01_rb,7.4216e+01_rb,5.5662e+01_rb, &
     &3.7107e+01_rb,1.8761e+01_rb,6.8194e-01_rb/)
      kao(:, 5,12,11) = (/ &
     &1.8694e+02_rb,1.6356e+02_rb,1.4020e+02_rb,1.1683e+02_rb,9.3474e+01_rb,7.0107e+01_rb, &
     &4.6738e+01_rb,2.3682e+01_rb,7.5051e-01_rb/)
      kao(:, 1,13,11) = (/ &
     &6.1162e+01_rb,5.3516e+01_rb,4.5871e+01_rb,3.8225e+01_rb,3.0581e+01_rb,2.2936e+01_rb, &
     &1.5291e+01_rb,7.7722e+00_rb,3.0501e-01_rb/)
      kao(:, 2,13,11) = (/ &
     &8.6643e+01_rb,7.5815e+01_rb,6.4984e+01_rb,5.4153e+01_rb,4.3324e+01_rb,3.2493e+01_rb, &
     &2.1662e+01_rb,1.1016e+01_rb,3.8069e-01_rb/)
      kao(:, 3,13,11) = (/ &
     &1.1697e+02_rb,1.0235e+02_rb,8.7727e+01_rb,7.3107e+01_rb,5.8486e+01_rb,4.3863e+01_rb, &
     &2.9242e+01_rb,1.4879e+01_rb,4.9510e-01_rb/)
      kao(:, 4,13,11) = (/ &
     &1.5186e+02_rb,1.3288e+02_rb,1.1389e+02_rb,9.4916e+01_rb,7.5936e+01_rb,5.6949e+01_rb, &
     &3.7967e+01_rb,1.9342e+01_rb,5.7912e-01_rb/)
      kao(:, 5,13,11) = (/ &
     &1.9078e+02_rb,1.6694e+02_rb,1.4309e+02_rb,1.1924e+02_rb,9.5394e+01_rb,7.1546e+01_rb, &
     &4.7697e+01_rb,2.4327e+01_rb,6.7000e-01_rb/)
      kao(:, 1, 1,12) = (/ &
     &4.4327e-02_rb,1.9901e-01_rb,3.9567e-01_rb,5.8848e-01_rb,7.7476e-01_rb,9.4853e-01_rb, &
     &1.0934e+00_rb,1.1527e+00_rb,1.5495e+00_rb/)
      kao(:, 2, 1,12) = (/ &
     &5.3905e-02_rb,2.4740e-01_rb,4.9242e-01_rb,7.3367e-01_rb,9.6823e-01_rb,1.1892e+00_rb, &
     &1.3767e+00_rb,1.4534e+00_rb,1.9364e+00_rb/)
      kao(:, 3, 1,12) = (/ &
     &6.4069e-02_rb,2.9762e-01_rb,5.9267e-01_rb,8.8396e-01_rb,1.1680e+00_rb,1.4379e+00_rb, &
     &1.6734e+00_rb,1.7853e+00_rb,2.3359e+00_rb/)
      kao(:, 4, 1,12) = (/ &
     &7.4599e-02_rb,3.4908e-01_rb,6.9548e-01_rb,1.0378e+00_rb,1.3725e+00_rb,1.6921e+00_rb, &
     &1.9734e+00_rb,2.1192e+00_rb,2.7449e+00_rb/)
      kao(:, 5, 1,12) = (/ &
     &8.5274e-02_rb,4.0394e-01_rb,8.0404e-01_rb,1.1983e+00_rb,1.5830e+00_rb,1.9477e+00_rb, &
     &2.2662e+00_rb,2.4521e+00_rb,3.1659e+00_rb/)
      kao(:, 1, 2,12) = (/ &
     &6.1665e-02_rb,2.1077e-01_rb,3.8355e-01_rb,5.7339e-01_rb,7.6055e-01_rb,9.4236e-01_rb, &
     &1.1091e+00_rb,1.2168e+00_rb,1.5210e+00_rb/)
      kao(:, 2, 2,12) = (/ &
     &7.6234e-02_rb,2.5639e-01_rb,4.7826e-01_rb,7.1532e-01_rb,9.5007e-01_rb,1.1793e+00_rb, &
     &1.3941e+00_rb,1.5452e+00_rb,1.9001e+00_rb/)
      kao(:, 3, 2,12) = (/ &
     &9.1892e-02_rb,3.0376e-01_rb,5.7761e-01_rb,8.6445e-01_rb,1.1488e+00_rb,1.4281e+00_rb, &
     &1.6923e+00_rb,1.8917e+00_rb,2.2975e+00_rb/)
      kao(:, 4, 2,12) = (/ &
     &1.0837e-01_rb,3.5273e-01_rb,6.8106e-01_rb,1.0192e+00_rb,1.3548e+00_rb,1.6858e+00_rb, &
     &2.0012e+00_rb,2.2488e+00_rb,2.7094e+00_rb/)
      kao(:, 5, 2,12) = (/ &
     &1.2523e-01_rb,4.0526e-01_rb,7.9735e-01_rb,1.1921e+00_rb,1.5816e+00_rb,1.9625e+00_rb, &
     &2.3213e+00_rb,2.6087e+00_rb,3.1625e+00_rb/)
      kao(:, 1, 3,12) = (/ &
     &1.1120e-01_rb,2.4868e-01_rb,3.6808e-01_rb,5.1840e-01_rb,6.9017e-01_rb,8.6056e-01_rb, &
     &1.0271e+00_rb,1.1758e+00_rb,1.3802e+00_rb/)
      kao(:, 2, 3,12) = (/ &
     &1.4053e-01_rb,3.1536e-01_rb,4.5583e-01_rb,6.5442e-01_rb,8.7173e-01_rb,1.0879e+00_rb, &
     &1.3003e+00_rb,1.4947e+00_rb,1.7433e+00_rb/)
      kao(:, 3, 3,12) = (/ &
     &1.7249e-01_rb,3.8643e-01_rb,5.4830e-01_rb,7.9885e-01_rb,1.0645e+00_rb,1.3292e+00_rb, &
     &1.5905e+00_rb,1.8338e+00_rb,2.1288e+00_rb/)
      kao(:, 4, 3,12) = (/ &
     &2.0676e-01_rb,4.5780e-01_rb,6.4549e-01_rb,9.5437e-01_rb,1.2710e+00_rb,1.5864e+00_rb, &
     &1.8970e+00_rb,2.1896e+00_rb,2.5411e+00_rb/)
      kao(:, 5, 3,12) = (/ &
     &2.4212e-01_rb,5.2871e-01_rb,7.5484e-01_rb,1.1269e+00_rb,1.5000e+00_rb,1.8700e+00_rb, &
     &2.2336e+00_rb,2.5647e+00_rb,2.9987e+00_rb/)
      kao(:, 1, 4,12) = (/ &
     &2.0092e-01_rb,3.0566e-01_rb,4.3321e-01_rb,5.2770e-01_rb,6.0160e-01_rb,7.5023e-01_rb, &
     &8.9870e-01_rb,1.0432e+00_rb,1.2013e+00_rb/)
      kao(:, 2, 4,12) = (/ &
     &2.6071e-01_rb,3.8703e-01_rb,5.5757e-01_rb,6.5930e-01_rb,7.6619e-01_rb,9.5732e-01_rb, &
     &1.1477e+00_rb,1.3364e+00_rb,1.5321e+00_rb/)
      kao(:, 3, 4,12) = (/ &
     &3.2640e-01_rb,4.7994e-01_rb,6.8737e-01_rb,7.9575e-01_rb,9.4617e-01_rb,1.1821e+00_rb, &
     &1.4178e+00_rb,1.6541e+00_rb,1.8914e+00_rb/)
      kao(:, 4, 4,12) = (/ &
     &3.9855e-01_rb,5.8348e-01_rb,8.2078e-01_rb,9.3830e-01_rb,1.1490e+00_rb,1.4352e+00_rb, &
     &1.7207e+00_rb,2.0041e+00_rb,2.2963e+00_rb/)
      kao(:, 5, 4,12) = (/ &
     &4.7570e-01_rb,6.9415e-01_rb,9.5472e-01_rb,1.0922e+00_rb,1.3776e+00_rb,1.7196e+00_rb, &
     &2.0594e+00_rb,2.3902e+00_rb,2.7528e+00_rb/)
      kao(:, 1, 5,12) = (/ &
     &3.3247e-01_rb,3.9594e-01_rb,4.9056e-01_rb,5.8813e-01_rb,6.5607e-01_rb,6.8035e-01_rb, &
     &7.9032e-01_rb,9.1840e-01_rb,1.0557e+00_rb/)
      kao(:, 2, 5,12) = (/ &
     &4.4282e-01_rb,5.1203e-01_rb,6.4486e-01_rb,7.7152e-01_rb,8.4090e-01_rb,8.6628e-01_rb, &
     &1.0179e+00_rb,1.1841e+00_rb,1.3589e+00_rb/)
      kao(:, 3, 5,12) = (/ &
     &5.6912e-01_rb,6.3900e-01_rb,8.2247e-01_rb,9.6727e-01_rb,1.0369e+00_rb,1.0698e+00_rb, &
     &1.2706e+00_rb,1.4785e+00_rb,1.6959e+00_rb/)
      kao(:, 4, 5,12) = (/ &
     &7.0931e-01_rb,7.8069e-01_rb,1.0153e+00_rb,1.1726e+00_rb,1.2423e+00_rb,1.3043e+00_rb, &
     &1.5604e+00_rb,1.8146e+00_rb,2.0829e+00_rb/)
      kao(:, 5, 5,12) = (/ &
     &8.5949e-01_rb,9.3825e-01_rb,1.2105e+00_rb,1.3829e+00_rb,1.4592e+00_rb,1.5748e+00_rb, &
     &1.8870e+00_rb,2.1938e+00_rb,2.5189e+00_rb/)
      kao(:, 1, 6,12) = (/ &
     &5.2509e-01_rb,5.1721e-01_rb,5.7378e-01_rb,6.3384e-01_rb,6.9692e-01_rb,7.3419e-01_rb, &
     &7.1864e-01_rb,7.9317e-01_rb,9.0866e-01_rb/)
      kao(:, 2, 6,12) = (/ &
     &7.0957e-01_rb,6.9625e-01_rb,7.6638e-01_rb,8.5821e-01_rb,9.3314e-01_rb,9.6244e-01_rb, &
     &9.2911e-01_rb,1.0410e+00_rb,1.1915e+00_rb/)
      kao(:, 3, 6,12) = (/ &
     &9.2892e-01_rb,9.0647e-01_rb,9.9421e-01_rb,1.1180e+00_rb,1.1912e+00_rb,1.2074e+00_rb, &
     &1.1606e+00_rb,1.3214e+00_rb,1.5144e+00_rb/)
      kao(:, 4, 6,12) = (/ &
     &1.1749e+00_rb,1.1432e+00_rb,1.2560e+00_rb,1.3977e+00_rb,1.4686e+00_rb,1.4691e+00_rb, &
     &1.4281e+00_rb,1.6462e+00_rb,1.8876e+00_rb/)
      kao(:, 5, 6,12) = (/ &
     &1.4442e+00_rb,1.4016e+00_rb,1.5480e+00_rb,1.6922e+00_rb,1.7621e+00_rb,1.7564e+00_rb, &
     &1.7321e+00_rb,2.0105e+00_rb,2.3048e+00_rb/)
      kao(:, 1, 7,12) = (/ &
     &9.1849e-01_rb,8.0624e-01_rb,7.9050e-01_rb,7.5675e-01_rb,7.6178e-01_rb,7.8133e-01_rb, &
     &7.7608e-01_rb,6.9116e-01_rb,7.7991e-01_rb/)
      kao(:, 2, 7,12) = (/ &
     &1.2413e+00_rb,1.0918e+00_rb,1.0769e+00_rb,1.0420e+00_rb,1.0608e+00_rb,1.0726e+00_rb, &
     &1.0404e+00_rb,9.1318e-01_rb,1.0358e+00_rb/)
      kao(:, 3, 7,12) = (/ &
     &1.6444e+00_rb,1.4500e+00_rb,1.4209e+00_rb,1.3929e+00_rb,1.4200e+00_rb,1.3992e+00_rb, &
     &1.3321e+00_rb,1.1740e+00_rb,1.3388e+00_rb/)
      kao(:, 4, 7,12) = (/ &
     &2.1122e+00_rb,1.8669e+00_rb,1.8040e+00_rb,1.8049e+00_rb,1.8188e+00_rb,1.7570e+00_rb, &
     &1.6501e+00_rb,1.4861e+00_rb,1.6989e+00_rb/)
      kao(:, 5, 7,12) = (/ &
     &2.6250e+00_rb,2.3265e+00_rb,2.2289e+00_rb,2.2673e+00_rb,2.2456e+00_rb,2.1426e+00_rb, &
     &2.0040e+00_rb,1.8373e+00_rb,2.0997e+00_rb/)
      kao(:, 1, 8,12) = (/ &
     &1.9827e+00_rb,1.7349e+00_rb,1.4981e+00_rb,1.3371e+00_rb,1.1453e+00_rb,9.6602e-01_rb, &
     &8.7309e-01_rb,7.7254e-01_rb,6.6571e-01_rb/)
      kao(:, 2, 8,12) = (/ &
     &2.7151e+00_rb,2.3758e+00_rb,2.0507e+00_rb,1.8313e+00_rb,1.5691e+00_rb,1.3551e+00_rb, &
     &1.2325e+00_rb,1.0537e+00_rb,8.9966e-01_rb/)
      kao(:, 3, 8,12) = (/ &
     &3.5997e+00_rb,3.1499e+00_rb,2.7291e+00_rb,2.4373e+00_rb,2.1034e+00_rb,1.8593e+00_rb, &
     &1.6579e+00_rb,1.3773e+00_rb,1.1789e+00_rb/)
      kao(:, 4, 8,12) = (/ &
     &4.6665e+00_rb,4.0832e+00_rb,3.5519e+00_rb,3.1544e+00_rb,2.7468e+00_rb,2.4617e+00_rb, &
     &2.1388e+00_rb,1.7464e+00_rb,1.5079e+00_rb/)
      kao(:, 5, 8,12) = (/ &
     &5.8669e+00_rb,5.1335e+00_rb,4.4792e+00_rb,3.9559e+00_rb,3.4817e+00_rb,3.1241e+00_rb, &
     &2.6649e+00_rb,2.1629e+00_rb,1.8732e+00_rb/)
      kao(:, 1, 9,12) = (/ &
     &8.0387e+00_rb,7.0340e+00_rb,6.0293e+00_rb,5.0245e+00_rb,4.0196e+00_rb,3.0957e+00_rb, &
     &2.1773e+00_rb,1.2357e+00_rb,4.2050e-01_rb/)
      kao(:, 2, 9,12) = (/ &
     &1.1280e+01_rb,9.8705e+00_rb,8.4601e+00_rb,7.0501e+00_rb,5.6402e+00_rb,4.3334e+00_rb, &
     &3.0492e+00_rb,1.7467e+00_rb,4.6837e-01_rb/)
      kao(:, 3, 9,12) = (/ &
     &1.5202e+01_rb,1.3302e+01_rb,1.1401e+01_rb,9.5008e+00_rb,7.6009e+00_rb,5.8446e+00_rb, &
     &4.1033e+00_rb,2.4072e+00_rb,9.0067e-01_rb/)
      kao(:, 4, 9,12) = (/ &
     &1.9804e+01_rb,1.7328e+01_rb,1.4853e+01_rb,1.2377e+01_rb,9.9040e+00_rb,7.6319e+00_rb, &
     &5.3589e+00_rb,3.2206e+00_rb,1.3277e+00_rb/)
      kao(:, 5, 9,12) = (/ &
     &2.5091e+01_rb,2.1955e+01_rb,1.8819e+01_rb,1.5683e+01_rb,1.2554e+01_rb,9.6970e+00_rb, &
     &6.8190e+00_rb,4.1412e+00_rb,1.6570e+00_rb/)
      kao(:, 1,10,12) = (/ &
     &3.6853e+01_rb,3.2247e+01_rb,2.7641e+01_rb,2.3035e+01_rb,1.8427e+01_rb,1.3820e+01_rb, &
     &9.2142e+00_rb,4.7017e+00_rb,5.2096e-01_rb/)
      kao(:, 2,10,12) = (/ &
     &5.2866e+01_rb,4.6255e+01_rb,3.9648e+01_rb,3.3040e+01_rb,2.6431e+01_rb,1.9825e+01_rb, &
     &1.3217e+01_rb,6.7223e+00_rb,8.5705e-01_rb/)
      kao(:, 3,10,12) = (/ &
     &7.2440e+01_rb,6.3382e+01_rb,5.4326e+01_rb,4.5272e+01_rb,3.6218e+01_rb,2.7164e+01_rb, &
     &1.8108e+01_rb,9.2103e+00_rb,9.7984e-01_rb/)
      kao(:, 4,10,12) = (/ &
     &9.5327e+01_rb,8.3414e+01_rb,7.1501e+01_rb,5.9582e+01_rb,4.7667e+01_rb,3.5750e+01_rb, &
     &2.3834e+01_rb,1.2146e+01_rb,1.1774e+00_rb/)
      kao(:, 5,10,12) = (/ &
     &1.2143e+02_rb,1.0625e+02_rb,9.1069e+01_rb,7.5889e+01_rb,6.0713e+01_rb,4.5537e+01_rb, &
     &3.0358e+01_rb,1.5516e+01_rb,1.3416e+00_rb/)
      kao(:, 1,11,12) = (/ &
     &6.9348e+01_rb,6.0680e+01_rb,5.2012e+01_rb,4.3342e+01_rb,3.4673e+01_rb,2.6006e+01_rb, &
     &1.7338e+01_rb,8.6702e+00_rb,3.0466e-01_rb/)
      kao(:, 2,11,12) = (/ &
     &9.9352e+01_rb,8.6930e+01_rb,7.4511e+01_rb,6.2093e+01_rb,4.9676e+01_rb,3.7257e+01_rb, &
     &2.4839e+01_rb,1.2419e+01_rb,6.8510e-01_rb/)
      kao(:, 3,11,12) = (/ &
     &1.3563e+02_rb,1.1868e+02_rb,1.0173e+02_rb,8.4769e+01_rb,6.7818e+01_rb,5.0866e+01_rb, &
     &3.3908e+01_rb,1.6957e+01_rb,1.0467e+00_rb/)
      kao(:, 4,11,12) = (/ &
     &1.7753e+02_rb,1.5535e+02_rb,1.3315e+02_rb,1.1096e+02_rb,8.8765e+01_rb,6.6576e+01_rb, &
     &4.4384e+01_rb,2.2206e+01_rb,1.3289e+00_rb/)
      kao(:, 5,11,12) = (/ &
     &2.2508e+02_rb,1.9694e+02_rb,1.6881e+02_rb,1.4068e+02_rb,1.1255e+02_rb,8.4405e+01_rb, &
     &5.6267e+01_rb,2.8168e+01_rb,1.6371e+00_rb/)
      kao(:, 1,12,12) = (/ &
     &9.5815e+01_rb,8.3836e+01_rb,7.1860e+01_rb,5.9885e+01_rb,4.7907e+01_rb,3.5931e+01_rb, &
     &2.3953e+01_rb,1.1977e+01_rb,2.4035e-01_rb/)
      kao(:, 2,12,12) = (/ &
     &1.3694e+02_rb,1.1982e+02_rb,1.0270e+02_rb,8.5588e+01_rb,6.8468e+01_rb,5.1352e+01_rb, &
     &3.4234e+01_rb,1.7118e+01_rb,5.7126e-01_rb/)
      kao(:, 3,12,12) = (/ &
     &1.8607e+02_rb,1.6282e+02_rb,1.3956e+02_rb,1.1630e+02_rb,9.3038e+01_rb,6.9781e+01_rb, &
     &4.6518e+01_rb,2.3259e+01_rb,8.8813e-01_rb/)
      kao(:, 4,12,12) = (/ &
     &2.4268e+02_rb,2.1235e+02_rb,1.8201e+02_rb,1.5168e+02_rb,1.2134e+02_rb,9.1010e+01_rb, &
     &6.0670e+01_rb,3.0335e+01_rb,1.1237e+00_rb/)
      kao(:, 5,12,12) = (/ &
     &3.0642e+02_rb,2.6813e+02_rb,2.2982e+02_rb,1.9152e+02_rb,1.5322e+02_rb,1.1491e+02_rb, &
     &7.6606e+01_rb,3.8310e+01_rb,1.5308e+00_rb/)
      kao(:, 1,13,12) = (/ &
     &1.0396e+02_rb,9.0973e+01_rb,7.7973e+01_rb,6.4982e+01_rb,5.1984e+01_rb,3.8991e+01_rb, &
     &2.5992e+01_rb,1.3003e+01_rb,2.8896e-01_rb/)
      kao(:, 2,13,12) = (/ &
     &1.4817e+02_rb,1.2965e+02_rb,1.1113e+02_rb,9.2604e+01_rb,7.4089e+01_rb,5.5562e+01_rb, &
     &3.7043e+01_rb,1.8523e+01_rb,6.4071e-01_rb/)
      kao(:, 3,13,12) = (/ &
     &2.0056e+02_rb,1.7549e+02_rb,1.5042e+02_rb,1.2535e+02_rb,1.0028e+02_rb,7.5210e+01_rb, &
     &5.0138e+01_rb,2.5080e+01_rb,7.6073e-01_rb/)
      kao(:, 4,13,12) = (/ &
     &2.6107e+02_rb,2.2842e+02_rb,1.9581e+02_rb,1.6317e+02_rb,1.3054e+02_rb,9.7901e+01_rb, &
     &6.5267e+01_rb,3.2665e+01_rb,1.0550e+00_rb/)
      kao(:, 5,13,12) = (/ &
     &3.2818e+02_rb,2.8717e+02_rb,2.4613e+02_rb,2.0512e+02_rb,1.6409e+02_rb,1.2307e+02_rb, &
     &8.2040e+01_rb,4.1093e+01_rb,1.4069e+00_rb/)
      kao(:, 1, 1,13) = (/ &
     &8.9189e-02_rb,2.7547e-01_rb,5.4755e-01_rb,8.1466e-01_rb,1.0734e+00_rb,1.3167e+00_rb, &
     &1.5262e+00_rb,1.6545e+00_rb,2.1467e+00_rb/)
      kao(:, 2, 1,13) = (/ &
     &1.0564e-01_rb,3.3448e-01_rb,6.6333e-01_rb,9.8354e-01_rb,1.2897e+00_rb,1.5719e+00_rb, &
     &1.8097e+00_rb,1.9474e+00_rb,2.5794e+00_rb/)
      kao(:, 3, 1,13) = (/ &
     &1.2232e-01_rb,4.0251e-01_rb,7.9745e-01_rb,1.1809e+00_rb,1.5463e+00_rb,1.8798e+00_rb, &
     &2.1512e+00_rb,2.2853e+00_rb,3.0925e+00_rb/)
      kao(:, 4, 1,13) = (/ &
     &1.3906e-01_rb,4.8151e-01_rb,9.5370e-01_rb,1.4117e+00_rb,1.8477e+00_rb,2.2440e+00_rb, &
     &2.5627e+00_rb,2.7054e+00_rb,3.6953e+00_rb/)
      kao(:, 5, 1,13) = (/ &
     &1.5537e-01_rb,5.6891e-01_rb,1.1282e+00_rb,1.6723e+00_rb,2.1918e+00_rb,2.6654e+00_rb, &
     &3.0480e+00_rb,3.1989e+00_rb,4.3834e+00_rb/)
      kao(:, 1, 2,13) = (/ &
     &1.2651e-01_rb,2.7511e-01_rb,5.4811e-01_rb,8.1792e-01_rb,1.0822e+00_rb,1.3369e+00_rb, &
     &1.5702e+00_rb,1.7388e+00_rb,2.1643e+00_rb/)
      kao(:, 2, 2,13) = (/ &
     &1.5153e-01_rb,3.4384e-01_rb,6.8417e-01_rb,1.0197e+00_rb,1.3466e+00_rb,1.6574e+00_rb, &
     &1.9318e+00_rb,2.0933e+00_rb,2.6930e+00_rb/)
      kao(:, 3, 2,13) = (/ &
     &1.7716e-01_rb,4.2323e-01_rb,8.4160e-01_rb,1.2530e+00_rb,1.6525e+00_rb,2.0285e+00_rb, &
     &2.3523e+00_rb,2.5052e+00_rb,3.3049e+00_rb/)
      kao(:, 4, 2,13) = (/ &
     &2.0304e-01_rb,5.0914e-01_rb,1.0128e+00_rb,1.5083e+00_rb,1.9894e+00_rb,2.4425e+00_rb, &
     &2.8273e+00_rb,2.9947e+00_rb,3.9785e+00_rb/)
      kao(:, 5, 2,13) = (/ &
     &2.2856e-01_rb,5.9708e-01_rb,1.1882e+00_rb,1.7705e+00_rb,2.3375e+00_rb,2.8753e+00_rb, &
     &3.3417e+00_rb,3.5598e+00_rb,4.6746e+00_rb/)
      kao(:, 1, 3,13) = (/ &
     &2.3155e-01_rb,3.6106e-01_rb,5.1524e-01_rb,7.7074e-01_rb,1.0237e+00_rb,1.2717e+00_rb, &
     &1.5079e+00_rb,1.7067e+00_rb,2.0474e+00_rb/)
      kao(:, 2, 3,13) = (/ &
     &2.8222e-01_rb,4.3122e-01_rb,6.6114e-01_rb,9.8854e-01_rb,1.3120e+00_rb,1.6278e+00_rb, &
     &1.9248e+00_rb,2.1616e+00_rb,2.6238e+00_rb/)
      kao(:, 3, 3,13) = (/ &
     &3.3490e-01_rb,5.0925e-01_rb,8.3049e-01_rb,1.2417e+00_rb,1.6476e+00_rb,2.0430e+00_rb, &
     &2.4133e+00_rb,2.6954e+00_rb,3.2949e+00_rb/)
      kao(:, 4, 3,13) = (/ &
     &3.8898e-01_rb,5.9951e-01_rb,1.0132e+00_rb,1.5151e+00_rb,2.0114e+00_rb,2.4964e+00_rb, &
     &2.9531e+00_rb,3.3043e+00_rb,4.0224e+00_rb/)
      kao(:, 5, 3,13) = (/ &
     &4.4295e-01_rb,6.9228e-01_rb,1.1941e+00_rb,1.7863e+00_rb,2.3719e+00_rb,2.9449e+00_rb, &
     &3.4879e+00_rb,3.9170e+00_rb,4.7434e+00_rb/)
      kao(:, 1, 4,13) = (/ &
     &4.2438e-01_rb,5.0777e-01_rb,6.1519e-01_rb,7.0596e-01_rb,9.3438e-01_rb,1.1645e+00_rb, &
     &1.3892e+00_rb,1.5946e+00_rb,1.8686e+00_rb/)
      kao(:, 2, 4,13) = (/ &
     &5.2911e-01_rb,6.2653e-01_rb,7.5903e-01_rb,9.2591e-01_rb,1.2320e+00_rb,1.5352e+00_rb, &
     &1.8307e+00_rb,2.0955e+00_rb,2.4637e+00_rb/)
      kao(:, 3, 4,13) = (/ &
     &6.3945e-01_rb,7.5086e-01_rb,9.2902e-01_rb,1.1816e+00_rb,1.5728e+00_rb,1.9600e+00_rb, &
     &2.3380e+00_rb,2.6793e+00_rb,3.1453e+00_rb/)
      kao(:, 4, 4,13) = (/ &
     &7.5432e-01_rb,8.7946e-01_rb,1.1073e+00_rb,1.4569e+00_rb,1.9394e+00_rb,2.4183e+00_rb, &
     &2.8875e+00_rb,3.3173e+00_rb,3.8782e+00_rb/)
      kao(:, 5, 4,13) = (/ &
     &8.7134e-01_rb,1.0123e+00_rb,1.2829e+00_rb,1.7403e+00_rb,2.3172e+00_rb,2.8902e+00_rb, &
     &3.4528e+00_rb,3.9744e+00_rb,4.6335e+00_rb/)
      kao(:, 1, 5,13) = (/ &
     &7.1207e-01_rb,6.8711e-01_rb,8.3241e-01_rb,8.6529e-01_rb,8.8550e-01_rb,1.0406e+00_rb, &
     &1.2451e+00_rb,1.4407e+00_rb,1.6670e+00_rb/)
      kao(:, 2, 5,13) = (/ &
     &9.0944e-01_rb,8.8418e-01_rb,1.0384e+00_rb,1.0957e+00_rb,1.1610e+00_rb,1.4029e+00_rb, &
     &1.6788e+00_rb,1.9429e+00_rb,2.2472e+00_rb/)
      kao(:, 3, 5,13) = (/ &
     &1.1217e+00_rb,1.1051e+00_rb,1.2550e+00_rb,1.3677e+00_rb,1.4770e+00_rb,1.8154e+00_rb, &
     &2.1738e+00_rb,2.5182e+00_rb,2.9077e+00_rb/)
      kao(:, 4, 5,13) = (/ &
     &1.3460e+00_rb,1.3429e+00_rb,1.4887e+00_rb,1.6567e+00_rb,1.8206e+00_rb,2.2609e+00_rb, &
     &2.7082e+00_rb,3.1425e+00_rb,3.6199e+00_rb/)
      kao(:, 5, 5,13) = (/ &
     &1.5795e+00_rb,1.5881e+00_rb,1.7517e+00_rb,1.9324e+00_rb,2.1859e+00_rb,2.7284e+00_rb, &
     &3.2694e+00_rb,3.7971e+00_rb,4.3676e+00_rb/)
      kao(:, 1, 6,13) = (/ &
     &1.1023e+00_rb,9.7526e-01_rb,1.0617e+00_rb,1.1008e+00_rb,1.0651e+00_rb,1.0178e+00_rb, &
     &1.0836e+00_rb,1.2590e+00_rb,1.4474e+00_rb/)
      kao(:, 2, 6,13) = (/ &
     &1.4464e+00_rb,1.2819e+00_rb,1.3748e+00_rb,1.4078e+00_rb,1.3857e+00_rb,1.3476e+00_rb, &
     &1.4882e+00_rb,1.7304e+00_rb,1.9875e+00_rb/)
      kao(:, 3, 6,13) = (/ &
     &1.8308e+00_rb,1.6242e+00_rb,1.7155e+00_rb,1.7456e+00_rb,1.7675e+00_rb,1.7237e+00_rb, &
     &1.9554e+00_rb,2.2760e+00_rb,2.6094e+00_rb/)
      kao(:, 4, 6,13) = (/ &
     &2.2453e+00_rb,1.9975e+00_rb,2.0781e+00_rb,2.1295e+00_rb,2.1828e+00_rb,2.1327e+00_rb, &
     &2.4652e+00_rb,2.8713e+00_rb,3.2893e+00_rb/)
      kao(:, 5, 6,13) = (/ &
     &2.6836e+00_rb,2.3986e+00_rb,2.4577e+00_rb,2.5567e+00_rb,2.5877e+00_rb,2.5663e+00_rb, &
     &3.0104e+00_rb,3.5064e+00_rb,4.0161e+00_rb/)
      kao(:, 1, 7,13) = (/ &
     &1.7788e+00_rb,1.5565e+00_rb,1.4296e+00_rb,1.4541e+00_rb,1.3809e+00_rb,1.2482e+00_rb, &
     &1.1009e+00_rb,1.0852e+00_rb,1.2442e+00_rb/)
      kao(:, 2, 7,13) = (/ &
     &2.4100e+00_rb,2.1089e+00_rb,1.9246e+00_rb,1.9363e+00_rb,1.8185e+00_rb,1.6636e+00_rb, &
     &1.4819e+00_rb,1.5124e+00_rb,1.7324e+00_rb/)
      kao(:, 3, 7,13) = (/ &
     &3.1319e+00_rb,2.7405e+00_rb,2.5003e+00_rb,2.4636e+00_rb,2.3111e+00_rb,2.1652e+00_rb, &
     &1.9114e+00_rb,2.0080e+00_rb,2.2982e+00_rb/)
      kao(:, 4, 7,13) = (/ &
     &3.9360e+00_rb,3.4440e+00_rb,3.1598e+00_rb,3.0372e+00_rb,2.8747e+00_rb,2.7334e+00_rb, &
     &2.3814e+00_rb,2.5579e+00_rb,2.9254e+00_rb/)
      kao(:, 5, 7,13) = (/ &
     &4.8010e+00_rb,4.2009e+00_rb,3.8764e+00_rb,3.6526e+00_rb,3.5065e+00_rb,3.3078e+00_rb, &
     &2.8824e+00_rb,3.1625e+00_rb,3.6156e+00_rb/)
      kao(:, 1, 8,13) = (/ &
     &3.4656e+00_rb,3.0324e+00_rb,2.5993e+00_rb,2.2536e+00_rb,2.0768e+00_rb,1.8323e+00_rb, &
     &1.4794e+00_rb,1.1100e+00_rb,1.0719e+00_rb/)
      kao(:, 2, 8,13) = (/ &
     &4.7953e+00_rb,4.1960e+00_rb,3.5964e+00_rb,3.1195e+00_rb,2.8558e+00_rb,2.4960e+00_rb, &
     &2.0234e+00_rb,1.5264e+00_rb,1.5151e+00_rb/)
      kao(:, 3, 8,13) = (/ &
     &6.4119e+00_rb,5.6106e+00_rb,4.8091e+00_rb,4.1637e+00_rb,3.7658e+00_rb,3.2529e+00_rb, &
     &2.6777e+00_rb,1.9921e+00_rb,2.0306e+00_rb/)
      kao(:, 4, 8,13) = (/ &
     &8.2532e+00_rb,7.2211e+00_rb,6.1898e+00_rb,5.3688e+00_rb,4.7948e+00_rb,4.1025e+00_rb, &
     &3.4458e+00_rb,2.5062e+00_rb,2.6143e+00_rb/)
      kao(:, 5, 8,13) = (/ &
     &1.0283e+01_rb,8.9971e+00_rb,7.7118e+00_rb,6.7087e+00_rb,5.9163e+00_rb,5.0630e+00_rb, &
     &4.2877e+00_rb,3.0597e+00_rb,3.2702e+00_rb/)
      kao(:, 1, 9,13) = (/ &
     &1.3039e+01_rb,1.1409e+01_rb,9.7786e+00_rb,8.1486e+00_rb,6.5192e+00_rb,4.9000e+00_rb, &
     &3.5526e+00_rb,2.2083e+00_rb,6.1249e-01_rb/)
      kao(:, 2, 9,13) = (/ &
     &1.8190e+01_rb,1.5916e+01_rb,1.3643e+01_rb,1.1369e+01_rb,9.0953e+00_rb,6.8331e+00_rb, &
     &4.9544e+00_rb,3.0917e+00_rb,1.3174e+00_rb/)
      kao(:, 3, 9,13) = (/ &
     &2.4717e+01_rb,2.1628e+01_rb,1.8538e+01_rb,1.5449e+01_rb,1.2359e+01_rb,9.2996e+00_rb, &
     &6.7414e+00_rb,4.1578e+00_rb,1.7841e+00_rb/)
      kao(:, 4, 9,13) = (/ &
     &3.2573e+01_rb,2.8503e+01_rb,2.4432e+01_rb,2.0359e+01_rb,1.6288e+01_rb,1.2267e+01_rb, &
     &8.8606e+00_rb,5.4011e+00_rb,2.3241e+00_rb/)
      kao(:, 5, 9,13) = (/ &
     &4.1562e+01_rb,3.6368e+01_rb,3.1173e+01_rb,2.5977e+01_rb,2.0781e+01_rb,1.5678e+01_rb, &
     &1.1261e+01_rb,6.8301e+00_rb,2.9541e+00_rb/)
      kao(:, 1,10,13) = (/ &
     &5.7951e+01_rb,5.0708e+01_rb,4.3464e+01_rb,3.6220e+01_rb,2.8976e+01_rb,2.1732e+01_rb, &
     &1.4488e+01_rb,7.2520e+00_rb,5.6538e-01_rb/)
      kao(:, 2,10,13) = (/ &
     &8.1306e+01_rb,7.1141e+01_rb,6.0979e+01_rb,5.0813e+01_rb,4.0652e+01_rb,3.0489e+01_rb, &
     &2.0326e+01_rb,1.0163e+01_rb,6.5761e-01_rb/)
      kao(:, 3,10,13) = (/ &
     &1.1158e+02_rb,9.7625e+01_rb,8.3679e+01_rb,6.9730e+01_rb,5.5786e+01_rb,4.1839e+01_rb, &
     &2.7893e+01_rb,1.3953e+01_rb,1.0530e+00_rb/)
      kao(:, 4,10,13) = (/ &
     &1.4877e+02_rb,1.3018e+02_rb,1.1158e+02_rb,9.2984e+01_rb,7.4388e+01_rb,5.5791e+01_rb, &
     &3.7194e+01_rb,1.8619e+01_rb,1.0469e+00_rb/)
      kao(:, 5,10,13) = (/ &
     &1.9259e+02_rb,1.6851e+02_rb,1.4444e+02_rb,1.2037e+02_rb,9.6298e+01_rb,7.2222e+01_rb, &
     &4.8145e+01_rb,2.4115e+01_rb,9.0336e-01_rb/)
      kao(:, 1,11,13) = (/ &
     &1.0724e+02_rb,9.3834e+01_rb,8.0425e+01_rb,6.7028e+01_rb,5.3621e+01_rb,4.0215e+01_rb, &
     &2.6808e+01_rb,1.3405e+01_rb,7.5947e-01_rb/)
      kao(:, 2,11,13) = (/ &
     &1.5135e+02_rb,1.3244e+02_rb,1.1352e+02_rb,9.4596e+01_rb,7.5677e+01_rb,5.6758e+01_rb, &
     &3.7839e+01_rb,1.8919e+01_rb,7.7568e-01_rb/)
      kao(:, 3,11,13) = (/ &
     &2.0634e+02_rb,1.8056e+02_rb,1.5476e+02_rb,1.2896e+02_rb,1.0317e+02_rb,7.7384e+01_rb, &
     &5.1587e+01_rb,2.5792e+01_rb,1.1446e+00_rb/)
      kao(:, 4,11,13) = (/ &
     &2.7388e+02_rb,2.3963e+02_rb,2.0540e+02_rb,1.7116e+02_rb,1.3692e+02_rb,1.0270e+02_rb, &
     &6.8464e+01_rb,3.4233e+01_rb,1.4967e+00_rb/)
      kao(:, 5,11,13) = (/ &
     &3.5254e+02_rb,3.0844e+02_rb,2.6442e+02_rb,2.2035e+02_rb,1.7628e+02_rb,1.3220e+02_rb, &
     &8.8134e+01_rb,4.4067e+01_rb,1.3951e+00_rb/)
      kao(:, 1,12,13) = (/ &
     &1.4929e+02_rb,1.3063e+02_rb,1.1197e+02_rb,9.3306e+01_rb,7.4648e+01_rb,5.5984e+01_rb, &
     &3.7322e+01_rb,1.8661e+01_rb,7.7179e-01_rb/)
      kao(:, 2,12,13) = (/ &
     &2.1213e+02_rb,1.8562e+02_rb,1.5909e+02_rb,1.3258e+02_rb,1.0606e+02_rb,7.9546e+01_rb, &
     &5.3033e+01_rb,2.6517e+01_rb,8.3102e-01_rb/)
      kao(:, 3,12,13) = (/ &
     &2.8848e+02_rb,2.5242e+02_rb,2.1635e+02_rb,1.8030e+02_rb,1.4425e+02_rb,1.0818e+02_rb, &
     &7.2116e+01_rb,3.6059e+01_rb,1.2769e+00_rb/)
      kao(:, 4,12,13) = (/ &
     &3.7976e+02_rb,3.3226e+02_rb,2.8481e+02_rb,2.3733e+02_rb,1.8986e+02_rb,1.4240e+02_rb, &
     &9.4932e+01_rb,4.7464e+01_rb,1.6812e+00_rb/)
      kao(:, 5,12,13) = (/ &
     &4.8508e+02_rb,4.2444e+02_rb,3.6380e+02_rb,3.0318e+02_rb,2.4253e+02_rb,1.8190e+02_rb, &
     &1.2127e+02_rb,6.0633e+01_rb,1.6837e+00_rb/)
      kao(:, 1,13,13) = (/ &
     &1.6844e+02_rb,1.4739e+02_rb,1.2633e+02_rb,1.0528e+02_rb,8.4219e+01_rb,6.3167e+01_rb, &
     &4.2110e+01_rb,2.1055e+01_rb,6.4746e-01_rb/)
      kao(:, 2,13,13) = (/ &
     &2.3974e+02_rb,2.0978e+02_rb,1.7981e+02_rb,1.4983e+02_rb,1.1987e+02_rb,8.9901e+01_rb, &
     &5.9937e+01_rb,2.9967e+01_rb,7.7977e-01_rb/)
      kao(:, 3,13,13) = (/ &
     &3.2617e+02_rb,2.8540e+02_rb,2.4462e+02_rb,2.0386e+02_rb,1.6309e+02_rb,1.2231e+02_rb, &
     &8.1544e+01_rb,4.0770e+01_rb,1.5118e+00_rb/)
      kao(:, 4,13,13) = (/ &
     &4.2665e+02_rb,3.7332e+02_rb,3.1999e+02_rb,2.6667e+02_rb,2.1333e+02_rb,1.5999e+02_rb, &
     &1.0667e+02_rb,5.3332e+01_rb,1.6652e+00_rb/)
      kao(:, 5,13,13) = (/ &
     &5.4098e+02_rb,4.7336e+02_rb,4.0573e+02_rb,3.3811e+02_rb,2.7048e+02_rb,2.0286e+02_rb, &
     &1.3525e+02_rb,6.7619e+01_rb,1.7274e+00_rb/)
      kao(:, 1, 1,14) = (/ &
     &1.4473e-01_rb,4.5196e-01_rb,8.9698e-01_rb,1.3316e+00_rb,1.7483e+00_rb,2.1329e+00_rb, &
     &2.4492e+00_rb,2.5366e+00_rb,3.4967e+00_rb/)
      kao(:, 2, 1,14) = (/ &
     &1.6846e-01_rb,5.5554e-01_rb,1.1039e+00_rb,1.6417e+00_rb,2.1611e+00_rb,2.6456e+00_rb, &
     &3.0463e+00_rb,3.1695e+00_rb,4.3222e+00_rb/)
      kao(:, 3, 1,14) = (/ &
     &1.9174e-01_rb,6.6749e-01_rb,1.3276e+00_rb,1.9756e+00_rb,2.6027e+00_rb,3.1886e+00_rb, &
     &3.6789e+00_rb,3.8433e+00_rb,5.2055e+00_rb/)
      kao(:, 4, 1,14) = (/ &
     &2.1490e-01_rb,7.8476e-01_rb,1.5609e+00_rb,2.3233e+00_rb,3.0639e+00_rb,3.7563e+00_rb, &
     &4.3396e+00_rb,4.5493e+00_rb,6.1279e+00_rb/)
      kao(:, 5, 1,14) = (/ &
     &2.3691e-01_rb,9.0765e-01_rb,1.8042e+00_rb,2.6834e+00_rb,3.5348e+00_rb,4.3362e+00_rb, &
     &5.0160e+00_rb,5.2774e+00_rb,7.0696e+00_rb/)
      kao(:, 1, 2,14) = (/ &
     &2.1296e-01_rb,4.6215e-01_rb,9.1984e-01_rb,1.3706e+00_rb,1.8091e+00_rb,2.2245e+00_rb, &
     &2.5860e+00_rb,2.7730e+00_rb,3.6182e+00_rb/)
      kao(:, 2, 2,14) = (/ &
     &2.5016e-01_rb,5.6432e-01_rb,1.1237e+00_rb,1.6756e+00_rb,2.2151e+00_rb,2.7294e+00_rb, &
     &3.1864e+00_rb,3.4606e+00_rb,4.4301e+00_rb/)
      kao(:, 3, 2,14) = (/ &
     &2.8692e-01_rb,6.7375e-01_rb,1.3425e+00_rb,2.0033e+00_rb,2.6512e+00_rb,3.2743e+00_rb, &
     &3.8384e+00_rb,4.2145e+00_rb,5.3023e+00_rb/)
      kao(:, 4, 2,14) = (/ &
     &3.2327e-01_rb,7.9645e-01_rb,1.5866e+00_rb,2.3669e+00_rb,3.1323e+00_rb,3.8679e+00_rb, &
     &4.5425e+00_rb,5.0100e+00_rb,6.2645e+00_rb/)
      kao(:, 5, 2,14) = (/ &
     &3.5851e-01_rb,9.3518e-01_rb,1.8621e+00_rb,2.7777e+00_rb,3.6730e+00_rb,4.5297e+00_rb, &
     &5.3029e+00_rb,5.8281e+00_rb,7.3458e+00_rb/)
      kao(:, 1, 3,14) = (/ &
     &4.0569e-01_rb,4.7346e-01_rb,9.0139e-01_rb,1.3483e+00_rb,1.7892e+00_rb,2.2204e+00_rb, &
     &2.6252e+00_rb,2.9358e+00_rb,3.5784e+00_rb/)
      kao(:, 2, 3,14) = (/ &
     &4.8357e-01_rb,5.7585e-01_rb,1.1016e+00_rb,1.6482e+00_rb,2.1888e+00_rb,2.7185e+00_rb, &
     &3.2205e+00_rb,3.6184e+00_rb,4.3777e+00_rb/)
      kao(:, 3, 3,14) = (/ &
     &5.6184e-01_rb,6.8539e-01_rb,1.3182e+00_rb,1.9726e+00_rb,2.6206e+00_rb,3.2555e+00_rb, &
     &3.8616e+00_rb,4.3597e+00_rb,5.2411e+00_rb/)
      kao(:, 4, 3,14) = (/ &
     &6.3937e-01_rb,8.0074e-01_rb,1.5659e+00_rb,2.3420e+00_rb,3.1101e+00_rb,3.8617e+00_rb, &
     &4.5735e+00_rb,5.1595e+00_rb,6.2201e+00_rb/)
      kao(:, 5, 3,14) = (/ &
     &7.1542e-01_rb,9.3290e-01_rb,1.8594e+00_rb,2.7803e+00_rb,3.6907e+00_rb,4.5815e+00_rb, &
     &5.4228e+00_rb,6.0997e+00_rb,7.3811e+00_rb/)
      kao(:, 1, 4,14) = (/ &
     &7.7613e-01_rb,7.2482e-01_rb,8.8213e-01_rb,1.2906e+00_rb,1.7174e+00_rb,2.1406e+00_rb, &
     &2.5536e+00_rb,2.9272e+00_rb,3.4348e+00_rb/)
      kao(:, 2, 4,14) = (/ &
     &9.4054e-01_rb,9.0136e-01_rb,1.0781e+00_rb,1.5850e+00_rb,2.1098e+00_rb,2.6295e+00_rb, &
     &3.1372e+00_rb,3.6020e+00_rb,4.2194e+00_rb/)
      kao(:, 3, 4,14) = (/ &
     &1.1095e+00_rb,1.0843e+00_rb,1.2887e+00_rb,1.9170e+00_rb,2.5512e+00_rb,3.1791e+00_rb, &
     &3.7919e+00_rb,4.3511e+00_rb,5.1021e+00_rb/)
      kao(:, 4, 4,14) = (/ &
     &1.2791e+00_rb,1.2603e+00_rb,1.5382e+00_rb,2.3038e+00_rb,3.0649e+00_rb,3.8173e+00_rb, &
     &4.5513e+00_rb,5.2083e+00_rb,6.1294e+00_rb/)
      kao(:, 5, 4,14) = (/ &
     &1.4465e+00_rb,1.4292e+00_rb,1.8362e+00_rb,2.7499e+00_rb,3.6579e+00_rb,4.5561e+00_rb, &
     &5.4280e+00_rb,6.2050e+00_rb,7.3150e+00_rb/)
      kao(:, 1, 5,14) = (/ &
     &1.3553e+00_rb,1.1859e+00_rb,1.1889e+00_rb,1.2820e+00_rb,1.6126e+00_rb,2.0133e+00_rb, &
     &2.4100e+00_rb,2.7917e+00_rb,3.2251e+00_rb/)
      kao(:, 2, 5,14) = (/ &
     &1.6746e+00_rb,1.4653e+00_rb,1.4722e+00_rb,1.5790e+00_rb,2.0132e+00_rb,2.5133e+00_rb, &
     &3.0071e+00_rb,3.4795e+00_rb,4.0262e+00_rb/)
      kao(:, 3, 5,14) = (/ &
     &2.0071e+00_rb,1.7563e+00_rb,1.7644e+00_rb,1.8926e+00_rb,2.4705e+00_rb,3.0826e+00_rb, &
     &3.6880e+00_rb,4.2630e+00_rb,4.9404e+00_rb/)
      kao(:, 4, 5,14) = (/ &
     &2.3461e+00_rb,2.0539e+00_rb,2.0623e+00_rb,2.2628e+00_rb,3.0107e+00_rb,3.7562e+00_rb, &
     &4.4918e+00_rb,5.1850e+00_rb,6.0203e+00_rb/)
      kao(:, 5, 5,14) = (/ &
     &2.6832e+00_rb,2.3569e+00_rb,2.3578e+00_rb,2.7266e+00_rb,3.6306e+00_rb,4.5292e+00_rb, &
     &5.4146e+00_rb,6.2480e+00_rb,7.2595e+00_rb/)
      kao(:, 1, 6,14) = (/ &
     &2.1798e+00_rb,1.9076e+00_rb,1.6634e+00_rb,1.6610e+00_rb,1.6126e+00_rb,1.8086e+00_rb, &
     &2.1681e+00_rb,2.5212e+00_rb,2.8955e+00_rb/)
      kao(:, 2, 6,14) = (/ &
     &2.7568e+00_rb,2.4124e+00_rb,2.1319e+00_rb,2.0890e+00_rb,2.0379e+00_rb,2.3402e+00_rb, &
     &2.8045e+00_rb,3.2592e+00_rb,3.7464e+00_rb/)
      kao(:, 3, 6,14) = (/ &
     &3.3653e+00_rb,2.9447e+00_rb,2.6494e+00_rb,2.5260e+00_rb,2.4774e+00_rb,2.9464e+00_rb, &
     &3.5300e+00_rb,4.0980e+00_rb,4.7178e+00_rb/)
      kao(:, 4, 6,14) = (/ &
     &3.9980e+00_rb,3.4981e+00_rb,3.2081e+00_rb,2.9759e+00_rb,2.9694e+00_rb,3.6441e+00_rb, &
     &4.3643e+00_rb,5.0633e+00_rb,5.8364e+00_rb/)
      kao(:, 5, 6,14) = (/ &
     &4.6371e+00_rb,4.0569e+00_rb,3.7676e+00_rb,3.4231e+00_rb,3.5724e+00_rb,4.4465e+00_rb, &
     &5.3245e+00_rb,6.1747e+00_rb,7.1186e+00_rb/)
      kao(:, 1, 7,14) = (/ &
     &3.6500e+00_rb,3.1940e+00_rb,2.7377e+00_rb,2.3521e+00_rb,2.2005e+00_rb,1.9226e+00_rb, &
     &1.8951e+00_rb,2.2032e+00_rb,2.5224e+00_rb/)
      kao(:, 2, 7,14) = (/ &
     &4.7380e+00_rb,4.1460e+00_rb,3.5535e+00_rb,3.0746e+00_rb,2.8560e+00_rb,2.4862e+00_rb, &
     &2.5325e+00_rb,2.9498e+00_rb,3.3790e+00_rb/)
      kao(:, 3, 7,14) = (/ &
     &5.9099e+00_rb,5.1713e+00_rb,4.4326e+00_rb,3.9038e+00_rb,3.5341e+00_rb,3.0823e+00_rb, &
     &3.2930e+00_rb,3.8333e+00_rb,4.3949e+00_rb/)
      kao(:, 4, 7,14) = (/ &
     &7.1406e+00_rb,6.2480e+00_rb,5.3561e+00_rb,4.7936e+00_rb,4.2290e+00_rb,3.7296e+00_rb, &
     &4.1867e+00_rb,4.8720e+00_rb,5.5890e+00_rb/)
      kao(:, 5, 7,14) = (/ &
     &8.4067e+00_rb,7.3559e+00_rb,6.3050e+00_rb,5.6953e+00_rb,4.9319e+00_rb,4.4981e+00_rb, &
     &5.2053e+00_rb,6.0551e+00_rb,6.9478e+00_rb/)
      kao(:, 1, 8,14) = (/ &
     &7.2793e+00_rb,6.3695e+00_rb,5.4593e+00_rb,4.5496e+00_rb,3.6668e+00_rb,3.0739e+00_rb, &
     &2.3812e+00_rb,1.8850e+00_rb,2.1414e+00_rb/)
      kao(:, 2, 8,14) = (/ &
     &9.7148e+00_rb,8.5004e+00_rb,7.2858e+00_rb,6.0719e+00_rb,4.9067e+00_rb,4.1050e+00_rb, &
     &3.1504e+00_rb,2.5960e+00_rb,2.9698e+00_rb/)
      kao(:, 3, 8,14) = (/ &
     &1.2408e+01_rb,1.0858e+01_rb,9.3058e+00_rb,7.7549e+00_rb,6.3008e+00_rb,5.2586e+00_rb, &
     &3.9820e+00_rb,3.4865e+00_rb,3.9886e+00_rb/)
      kao(:, 4, 8,14) = (/ &
     &1.5301e+01_rb,1.3388e+01_rb,1.1476e+01_rb,9.5632e+00_rb,7.8364e+00_rb,6.4696e+00_rb, &
     &4.8601e+00_rb,4.5513e+00_rb,5.2081e+00_rb/)
      kao(:, 5, 8,14) = (/ &
     &1.8319e+01_rb,1.6029e+01_rb,1.3740e+01_rb,1.1449e+01_rb,9.4798e+00_rb,7.7150e+00_rb, &
     &5.8215e+00_rb,5.7877e+00_rb,6.6226e+00_rb/)
      kao(:, 1, 9,14) = (/ &
     &2.7093e+01_rb,2.3706e+01_rb,2.0321e+01_rb,1.6934e+01_rb,1.3547e+01_rb,1.0161e+01_rb, &
     &6.7735e+00_rb,3.8947e+00_rb,1.7904e+00_rb/)
      kao(:, 2, 9,14) = (/ &
     &3.7356e+01_rb,3.2687e+01_rb,2.8017e+01_rb,2.3347e+01_rb,1.8679e+01_rb,1.4008e+01_rb, &
     &9.3393e+00_rb,5.3672e+00_rb,2.5563e+00_rb/)
      kao(:, 3, 9,14) = (/ &
     &4.9010e+01_rb,4.2883e+01_rb,3.6757e+01_rb,3.0630e+01_rb,2.4505e+01_rb,1.8379e+01_rb, &
     &1.2257e+01_rb,7.0400e+00_rb,3.5347e+00_rb/)
      kao(:, 4, 9,14) = (/ &
     &6.1735e+01_rb,5.4018e+01_rb,4.6300e+01_rb,3.8583e+01_rb,3.0868e+01_rb,2.3150e+01_rb, &
     &1.5458e+01_rb,8.8071e+00_rb,4.7393e+00_rb/)
      kao(:, 5, 9,14) = (/ &
     &7.5284e+01_rb,6.5873e+01_rb,5.6463e+01_rb,4.7052e+01_rb,3.7640e+01_rb,2.8233e+01_rb, &
     &1.8889e+01_rb,1.0654e+01_rb,6.1635e+00_rb/)
      kao(:, 1,10,14) = (/ &
     &1.1501e+02_rb,1.0064e+02_rb,8.6262e+01_rb,7.1883e+01_rb,5.7506e+01_rb,4.3130e+01_rb, &
     &2.8755e+01_rb,1.4378e+01_rb,5.7116e-01_rb/)
      kao(:, 2,10,14) = (/ &
     &1.6323e+02_rb,1.4283e+02_rb,1.2242e+02_rb,1.0202e+02_rb,8.1614e+01_rb,6.1213e+01_rb, &
     &4.0807e+01_rb,2.0405e+01_rb,4.6944e-01_rb/)
      kao(:, 3,10,14) = (/ &
     &2.1950e+02_rb,1.9206e+02_rb,1.6462e+02_rb,1.3718e+02_rb,1.0975e+02_rb,8.2311e+01_rb, &
     &5.4875e+01_rb,2.7438e+01_rb,1.0245e+00_rb/)
      kao(:, 4,10,14) = (/ &
     &2.8264e+02_rb,2.4730e+02_rb,2.1197e+02_rb,1.7666e+02_rb,1.4132e+02_rb,1.0599e+02_rb, &
     &7.0664e+01_rb,3.5330e+01_rb,2.8560e+00_rb/)
      kao(:, 5,10,14) = (/ &
     &3.5102e+02_rb,3.0716e+02_rb,2.6326e+02_rb,2.1939e+02_rb,1.7551e+02_rb,1.3164e+02_rb, &
     &8.7759e+01_rb,4.3879e+01_rb,5.5928e+00_rb/)
      kao(:, 1,11,14) = (/ &
     &2.0513e+02_rb,1.7948e+02_rb,1.5384e+02_rb,1.2821e+02_rb,1.0256e+02_rb,7.6924e+01_rb, &
     &5.1281e+01_rb,2.5640e+01_rb,9.4489e-01_rb/)
      kao(:, 2,11,14) = (/ &
     &2.8915e+02_rb,2.5300e+02_rb,2.1687e+02_rb,1.8072e+02_rb,1.4458e+02_rb,1.0844e+02_rb, &
     &7.2287e+01_rb,3.6146e+01_rb,1.0101e+00_rb/)
      kao(:, 3,11,14) = (/ &
     &3.8981e+02_rb,3.4108e+02_rb,2.9236e+02_rb,2.4364e+02_rb,1.9490e+02_rb,1.4617e+02_rb, &
     &9.7460e+01_rb,4.8730e+01_rb,5.8913e-01_rb/)
      kao(:, 4,11,14) = (/ &
     &5.0429e+02_rb,4.4124e+02_rb,3.7819e+02_rb,3.1515e+02_rb,2.5212e+02_rb,1.8910e+02_rb, &
     &1.2606e+02_rb,6.3034e+01_rb,1.0227e+00_rb/)
      kao(:, 5,11,14) = (/ &
     &6.2985e+02_rb,5.5113e+02_rb,4.7241e+02_rb,3.9368e+02_rb,3.1495e+02_rb,2.3621e+02_rb, &
     &1.5747e+02_rb,7.8734e+01_rb,3.1704e+00_rb/)
      kao(:, 1,12,14) = (/ &
     &2.7480e+02_rb,2.4043e+02_rb,2.0609e+02_rb,1.7173e+02_rb,1.3740e+02_rb,1.0305e+02_rb, &
     &6.8696e+01_rb,3.4350e+01_rb,1.0331e+00_rb/)
      kao(:, 2,12,14) = (/ &
     &3.8434e+02_rb,3.3629e+02_rb,2.8824e+02_rb,2.4020e+02_rb,1.9217e+02_rb,1.4412e+02_rb, &
     &9.6082e+01_rb,4.8041e+01_rb,1.0697e+00_rb/)
      kao(:, 3,12,14) = (/ &
     &5.1835e+02_rb,4.5357e+02_rb,3.8876e+02_rb,3.2398e+02_rb,2.5917e+02_rb,1.9438e+02_rb, &
     &1.2959e+02_rb,6.4795e+01_rb,6.6152e-01_rb/)
      kao(:, 4,12,14) = (/ &
     &6.7292e+02_rb,5.8878e+02_rb,5.0467e+02_rb,4.2055e+02_rb,3.3645e+02_rb,2.5234e+02_rb, &
     &1.6823e+02_rb,8.4112e+01_rb,1.1390e+00_rb/)
      kao(:, 5,12,14) = (/ &
     &8.4273e+02_rb,7.3739e+02_rb,6.3205e+02_rb,5.2668e+02_rb,4.2135e+02_rb,3.1602e+02_rb, &
     &2.1069e+02_rb,1.0534e+02_rb,2.2409e+00_rb/)
      kao(:, 1,13,14) = (/ &
     &2.9725e+02_rb,2.6009e+02_rb,2.2295e+02_rb,1.8578e+02_rb,1.4862e+02_rb,1.1147e+02_rb, &
     &7.4314e+01_rb,3.7158e+01_rb,8.5883e-01_rb/)
      kao(:, 2,13,14) = (/ &
     &4.1294e+02_rb,3.6134e+02_rb,3.0971e+02_rb,2.5809e+02_rb,2.0649e+02_rb,1.5487e+02_rb, &
     &1.0324e+02_rb,5.1621e+01_rb,7.4403e-01_rb/)
      kao(:, 3,13,14) = (/ &
     &5.5484e+02_rb,4.8548e+02_rb,4.1611e+02_rb,3.4677e+02_rb,2.7741e+02_rb,2.0806e+02_rb, &
     &1.3871e+02_rb,6.9351e+01_rb,6.5779e-01_rb/)
      kao(:, 4,13,14) = (/ &
     &7.2016e+02_rb,6.3017e+02_rb,5.4015e+02_rb,4.5013e+02_rb,3.6010e+02_rb,2.7007e+02_rb, &
     &1.8004e+02_rb,9.0025e+01_rb,1.1133e+00_rb/)
      kao(:, 5,13,14) = (/ &
     &9.0551e+02_rb,7.9235e+02_rb,6.7916e+02_rb,5.6596e+02_rb,4.5276e+02_rb,3.3958e+02_rb, &
     &2.2638e+02_rb,1.1319e+02_rb,2.5474e+00_rb/)
      kao(:, 1, 1,15) = (/ &
     &1.6993e-01_rb,6.2319e-01_rb,1.2302e+00_rb,1.8117e+00_rb,2.3534e+00_rb,2.8224e+00_rb, &
     &3.1427e+00_rb,3.0683e+00_rb,4.7067e+00_rb/)
      kao(:, 2, 1,15) = (/ &
     &1.9652e-01_rb,7.8355e-01_rb,1.5463e+00_rb,2.2775e+00_rb,2.9572e+00_rb,3.5450e+00_rb, &
     &3.9419e+00_rb,3.8320e+00_rb,5.9144e+00_rb/)
      kao(:, 3, 1,15) = (/ &
     &2.2200e-01_rb,9.5617e-01_rb,1.8872e+00_rb,2.7806e+00_rb,3.6114e+00_rb,4.3315e+00_rb, &
     &4.8212e+00_rb,4.6828e+00_rb,7.2228e+00_rb/)
      kao(:, 4, 1,15) = (/ &
     &2.4720e-01_rb,1.1411e+00_rb,2.2528e+00_rb,3.3193e+00_rb,4.3116e+00_rb,5.1713e+00_rb, &
     &5.7552e+00_rb,5.5959e+00_rb,8.6231e+00_rb/)
      kao(:, 5, 1,15) = (/ &
     &2.7067e-01_rb,1.3368e+00_rb,2.6392e+00_rb,3.8885e+00_rb,5.0531e+00_rb,6.0617e+00_rb, &
     &6.7490e+00_rb,6.5681e+00_rb,1.0106e+01_rb/)
      kao(:, 1, 2,15) = (/ &
     &2.5495e-01_rb,6.7714e-01_rb,1.3429e+00_rb,1.9911e+00_rb,2.6102e+00_rb,3.1741e+00_rb, &
     &3.6176e+00_rb,3.6989e+00_rb,5.2204e+00_rb/)
      kao(:, 2, 2,15) = (/ &
     &2.9713e-01_rb,8.6139e-01_rb,1.7085e+00_rb,2.5326e+00_rb,3.3191e+00_rb,4.0358e+00_rb, &
     &4.5977e+00_rb,4.6904e+00_rb,6.6382e+00_rb/)
      kao(:, 3, 2,15) = (/ &
     &3.3791e-01_rb,1.0632e+00_rb,2.1089e+00_rb,3.1273e+00_rb,4.0996e+00_rb,4.9862e+00_rb, &
     &5.6823e+00_rb,5.7948e+00_rb,8.1990e+00_rb/)
      kao(:, 4, 2,15) = (/ &
     &3.7751e-01_rb,1.2778e+00_rb,2.5347e+00_rb,3.7591e+00_rb,4.9277e+00_rb,5.9944e+00_rb, &
     &6.8345e+00_rb,6.9830e+00_rb,9.8554e+00_rb/)
      kao(:, 5, 2,15) = (/ &
     &4.1544e-01_rb,1.5054e+00_rb,2.9855e+00_rb,4.4278e+00_rb,5.8067e+00_rb,7.0647e+00_rb, &
     &8.0557e+00_rb,8.2364e+00_rb,1.1613e+01_rb/)
      kao(:, 1, 3,15) = (/ &
     &4.9608e-01_rb,6.8984e-01_rb,1.3746e+00_rb,2.0502e+00_rb,2.7121e+00_rb,3.3455e+00_rb, &
     &3.9122e+00_rb,4.2490e+00_rb,5.4242e+00_rb/)
      kao(:, 2, 3,15) = (/ &
     &5.8591e-01_rb,8.9422e-01_rb,1.7812e+00_rb,2.6579e+00_rb,3.5151e+00_rb,4.3360e+00_rb, &
     &5.0694e+00_rb,5.5020e+00_rb,7.0303e+00_rb/)
      kao(:, 3, 3,15) = (/ &
     &6.7409e-01_rb,1.1231e+00_rb,2.2377e+00_rb,3.3379e+00_rb,4.4153e+00_rb,5.4452e+00_rb, &
     &6.3666e+00_rb,6.9097e+00_rb,8.8307e+00_rb/)
      kao(:, 4, 3,15) = (/ &
     &7.5926e-01_rb,1.3688e+00_rb,2.7271e+00_rb,4.0692e+00_rb,5.3814e+00_rb,6.6410e+00_rb, &
     &7.7706e+00_rb,8.4452e+00_rb,1.0763e+01_rb/)
      kao(:, 5, 3,15) = (/ &
     &8.4200e-01_rb,1.6284e+00_rb,3.2443e+00_rb,4.8405e+00_rb,6.4040e+00_rb,7.9036e+00_rb, &
     &9.2460e+00_rb,1.0057e+01_rb,1.2808e+01_rb/)
      kao(:, 1, 4,15) = (/ &
     &9.7215e-01_rb,8.5067e-01_rb,1.3532e+00_rb,2.0247e+00_rb,2.6895e+00_rb,3.3419e+00_rb, &
     &3.9624e+00_rb,4.4662e+00_rb,5.3791e+00_rb/)
      kao(:, 2, 4,15) = (/ &
     &1.1632e+00_rb,1.0264e+00_rb,1.7937e+00_rb,2.6847e+00_rb,3.5667e+00_rb,4.4317e+00_rb, &
     &5.2538e+00_rb,5.9212e+00_rb,7.1333e+00_rb/)
      kao(:, 3, 4,15) = (/ &
     &1.3574e+00_rb,1.2217e+00_rb,2.2951e+00_rb,3.4352e+00_rb,4.5625e+00_rb,5.6686e+00_rb, &
     &6.7222e+00_rb,7.5788e+00_rb,9.1250e+00_rb/)
      kao(:, 4, 4,15) = (/ &
     &1.5473e+00_rb,1.4621e+00_rb,2.8487e+00_rb,4.2635e+00_rb,5.6651e+00_rb,7.0387e+00_rb, &
     &8.3491e+00_rb,9.4177e+00_rb,1.1330e+01_rb/)
      kao(:, 5, 4,15) = (/ &
     &1.7305e+00_rb,1.7312e+00_rb,3.4330e+00_rb,5.1366e+00_rb,6.8257e+00_rb,8.4825e+00_rb, &
     &1.0065e+01_rb,1.1356e+01_rb,1.3651e+01_rb/)
      kao(:, 1, 5,15) = (/ &
     &1.7388e+00_rb,1.5215e+00_rb,1.3772e+00_rb,1.9641e+00_rb,2.6140e+00_rb,3.2573e+00_rb, &
     &3.8853e+00_rb,4.4515e+00_rb,5.2280e+00_rb/)
      kao(:, 2, 5,15) = (/ &
     &2.1169e+00_rb,1.8521e+00_rb,1.7962e+00_rb,2.6470e+00_rb,3.5230e+00_rb,4.3915e+00_rb, &
     &5.2396e+00_rb,6.0117e+00_rb,7.0460e+00_rb/)
      kao(:, 3, 5,15) = (/ &
     &2.5041e+00_rb,2.1909e+00_rb,2.3110e+00_rb,3.4553e+00_rb,4.5986e+00_rb,5.7308e+00_rb, &
     &6.8405e+00_rb,7.8473e+00_rb,9.1972e+00_rb/)
      kao(:, 4, 5,15) = (/ &
     &2.8915e+00_rb,2.5301e+00_rb,2.9136e+00_rb,4.3640e+00_rb,5.8101e+00_rb,7.2418e+00_rb, &
     &8.6447e+00_rb,9.9184e+00_rb,1.1620e+01_rb/)
      kao(:, 5, 5,15) = (/ &
     &3.2655e+00_rb,2.8574e+00_rb,3.5660e+00_rb,5.3438e+00_rb,7.1123e+00_rb,8.8675e+00_rb, &
     &1.0583e+01_rb,1.2152e+01_rb,1.4224e+01_rb/)
      kao(:, 1, 6,15) = (/ &
     &2.8657e+00_rb,2.5077e+00_rb,2.1495e+00_rb,1.9426e+00_rb,2.5509e+00_rb,3.1827e+00_rb, &
     &3.8069e+00_rb,4.3973e+00_rb,5.1017e+00_rb/)
      kao(:, 2, 6,15) = (/ &
     &3.5656e+00_rb,3.1200e+00_rb,2.6742e+00_rb,2.5857e+00_rb,3.4175e+00_rb,4.2650e+00_rb, &
     &5.1021e+00_rb,5.8962e+00_rb,6.8350e+00_rb/)
      kao(:, 3, 6,15) = (/ &
     &4.2807e+00_rb,3.7458e+00_rb,3.2105e+00_rb,3.3891e+00_rb,4.4929e+00_rb,5.6082e+00_rb, &
     &6.7100e+00_rb,7.7616e+00_rb,8.9857e+00_rb/)
      kao(:, 4, 6,15) = (/ &
     &5.0170e+00_rb,4.3893e+00_rb,3.7667e+00_rb,4.3378e+00_rb,5.7738e+00_rb,7.2087e+00_rb, &
     &8.6270e+00_rb,9.9837e+00_rb,1.1548e+01_rb/)
      kao(:, 5, 6,15) = (/ &
     &5.7403e+00_rb,5.0226e+00_rb,4.3725e+00_rb,5.3984e+00_rb,7.1936e+00_rb,8.9774e+00_rb, &
     &1.0747e+01_rb,1.2440e+01_rb,1.4387e+01_rb/)
      kao(:, 1, 7,15) = (/ &
     &4.9113e+00_rb,4.2974e+00_rb,3.6835e+00_rb,3.0697e+00_rb,2.6153e+00_rb,3.1091e+00_rb, &
     &3.7251e+00_rb,4.3239e+00_rb,4.9783e+00_rb/)
      kao(:, 2, 7,15) = (/ &
     &6.2562e+00_rb,5.4743e+00_rb,4.6920e+00_rb,3.9102e+00_rb,3.4334e+00_rb,4.1812e+00_rb, &
     &5.0095e+00_rb,5.8168e+00_rb,6.6958e+00_rb/)
      kao(:, 3, 7,15) = (/ &
     &7.6604e+00_rb,6.7031e+00_rb,5.7453e+00_rb,4.7880e+00_rb,4.4844e+00_rb,5.5063e+00_rb, &
     &6.5975e+00_rb,7.6628e+00_rb,8.8178e+00_rb/)
      kao(:, 4, 7,15) = (/ &
     &9.1018e+00_rb,7.9643e+00_rb,6.8268e+00_rb,5.7331e+00_rb,5.7498e+00_rb,7.0854e+00_rb, &
     &8.4900e+00_rb,9.8659e+00_rb,1.1347e+01_rb/)
      kao(:, 5, 7,15) = (/ &
     &1.0558e+01_rb,9.2378e+00_rb,7.9183e+00_rb,6.7553e+00_rb,7.1878e+00_rb,8.8933e+00_rb, &
     &1.0658e+01_rb,1.2385e+01_rb,1.4238e+01_rb/)
      kao(:, 1, 8,15) = (/ &
     &1.0009e+01_rb,8.7571e+00_rb,7.5061e+00_rb,6.2547e+00_rb,5.0042e+00_rb,3.7993e+00_rb, &
     &3.6112e+00_rb,4.2052e+00_rb,4.8194e+00_rb/)
      kao(:, 2, 8,15) = (/ &
     &1.3089e+01_rb,1.1454e+01_rb,9.8171e+00_rb,8.1811e+00_rb,6.5451e+00_rb,5.0064e+00_rb, &
     &4.9059e+00_rb,5.7129e+00_rb,6.5481e+00_rb/)
      kao(:, 3, 8,15) = (/ &
     &1.6387e+01_rb,1.4339e+01_rb,1.2290e+01_rb,1.0242e+01_rb,8.1941e+00_rb,6.3517e+00_rb, &
     &6.5066e+00_rb,7.5778e+00_rb,8.6840e+00_rb/)
      kao(:, 4, 8,15) = (/ &
     &1.9809e+01_rb,1.7333e+01_rb,1.4856e+01_rb,1.2381e+01_rb,9.9047e+00_rb,7.9569e+00_rb, &
     &8.4189e+00_rb,9.8052e+00_rb,1.1235e+01_rb/)
      kao(:, 5, 8,15) = (/ &
     &2.3310e+01_rb,2.0396e+01_rb,1.7482e+01_rb,1.4569e+01_rb,1.1654e+01_rb,9.7998e+00_rb, &
     &1.0613e+01_rb,1.2362e+01_rb,1.4162e+01_rb/)
      kao(:, 1, 9,15) = (/ &
     &3.8130e+01_rb,3.3362e+01_rb,2.8596e+01_rb,2.3830e+01_rb,1.9065e+01_rb,1.4298e+01_rb, &
     &9.5322e+00_rb,4.9254e+00_rb,4.5210e+00_rb/)
      kao(:, 2, 9,15) = (/ &
     &5.1350e+01_rb,4.4933e+01_rb,3.8514e+01_rb,3.2094e+01_rb,2.5677e+01_rb,1.9258e+01_rb, &
     &1.2838e+01_rb,6.6242e+00_rb,6.3225e+00_rb/)
      kao(:, 3, 9,15) = (/ &
     &6.5865e+01_rb,5.7633e+01_rb,4.9398e+01_rb,4.1167e+01_rb,3.2931e+01_rb,2.4701e+01_rb, &
     &1.6465e+01_rb,8.5406e+00_rb,8.4711e+00_rb/)
      kao(:, 4, 9,15) = (/ &
     &8.1194e+01_rb,7.1041e+01_rb,6.0891e+01_rb,5.0738e+01_rb,4.0591e+01_rb,3.0449e+01_rb, &
     &2.0297e+01_rb,1.0902e+01_rb,1.1050e+01_rb/)
      kao(:, 5, 9,15) = (/ &
     &9.6976e+01_rb,8.4851e+01_rb,7.2734e+01_rb,6.0609e+01_rb,4.8485e+01_rb,3.6366e+01_rb, &
     &2.4244e+01_rb,1.3609e+01_rb,1.4022e+01_rb/)
      kao(:, 1,10,15) = (/ &
     &1.6495e+02_rb,1.4434e+02_rb,1.2371e+02_rb,1.0308e+02_rb,8.2478e+01_rb,6.1862e+01_rb, &
     &4.1240e+01_rb,2.0620e+01_rb,1.6305e+00_rb/)
      kao(:, 2,10,15) = (/ &
     &2.2828e+02_rb,1.9973e+02_rb,1.7121e+02_rb,1.4267e+02_rb,1.1414e+02_rb,8.5604e+01_rb, &
     &5.7062e+01_rb,2.8536e+01_rb,5.3393e+00_rb/)
      kao(:, 3,10,15) = (/ &
     &2.9925e+02_rb,2.6184e+02_rb,2.2444e+02_rb,1.8703e+02_rb,1.4963e+02_rb,1.1222e+02_rb, &
     &7.4815e+01_rb,3.7408e+01_rb,8.3308e+00_rb/)
      kao(:, 4,10,15) = (/ &
     &3.7719e+02_rb,3.3003e+02_rb,2.8287e+02_rb,2.3573e+02_rb,1.8859e+02_rb,1.4144e+02_rb, &
     &9.4294e+01_rb,4.7146e+01_rb,1.0953e+01_rb/)
      kao(:, 5,10,15) = (/ &
     &4.5955e+02_rb,4.0214e+02_rb,3.4467e+02_rb,2.8724e+02_rb,2.2979e+02_rb,1.7235e+02_rb, &
     &1.1490e+02_rb,5.7448e+01_rb,1.3975e+01_rb/)
      kao(:, 1,11,15) = (/ &
     &2.9347e+02_rb,2.5677e+02_rb,2.2010e+02_rb,1.8342e+02_rb,1.4673e+02_rb,1.1005e+02_rb, &
     &7.3365e+01_rb,3.6685e+01_rb,2.5030e-01_rb/)
      kao(:, 2,11,15) = (/ &
     &4.0846e+02_rb,3.5738e+02_rb,3.0635e+02_rb,2.5530e+02_rb,2.0423e+02_rb,1.5319e+02_rb, &
     &1.0212e+02_rb,5.1058e+01_rb,2.2261e+00_rb/)
      kao(:, 3,11,15) = (/ &
     &5.3934e+02_rb,4.7191e+02_rb,4.0450e+02_rb,3.3708e+02_rb,2.6969e+02_rb,2.0225e+02_rb, &
     &1.3483e+02_rb,6.7419e+01_rb,7.0745e+00_rb/)
      kao(:, 4,11,15) = (/ &
     &6.8280e+02_rb,5.9745e+02_rb,5.1209e+02_rb,4.2676e+02_rb,3.4141e+02_rb,2.5605e+02_rb, &
     &1.7070e+02_rb,8.5353e+01_rb,1.1909e+01_rb/)
      kao(:, 5,11,15) = (/ &
     &8.3619e+02_rb,7.3167e+02_rb,6.2713e+02_rb,5.2262e+02_rb,4.1810e+02_rb,3.1359e+02_rb, &
     &2.0904e+02_rb,1.0453e+02_rb,1.5372e+01_rb/)
      kao(:, 1,12,15) = (/ &
     &3.9587e+02_rb,3.4637e+02_rb,2.9690e+02_rb,2.4742e+02_rb,1.9794e+02_rb,1.4844e+02_rb, &
     &9.8964e+01_rb,4.9484e+01_rb,3.4642e-05_rb/)
      kao(:, 2,12,15) = (/ &
     &5.5272e+02_rb,4.8359e+02_rb,4.1455e+02_rb,3.4543e+02_rb,2.7636e+02_rb,2.0725e+02_rb, &
     &1.3818e+02_rb,6.9086e+01_rb,1.7207e+00_rb/)
      kao(:, 3,12,15) = (/ &
     &7.3408e+02_rb,6.4226e+02_rb,5.5052e+02_rb,4.5884e+02_rb,3.6704e+02_rb,2.7529e+02_rb, &
     &1.8352e+02_rb,9.1757e+01_rb,6.4555e+00_rb/)
      kao(:, 4,12,15) = (/ &
     &9.3778e+02_rb,8.2044e+02_rb,7.0331e+02_rb,5.8607e+02_rb,4.6883e+02_rb,3.5166e+02_rb, &
     &2.3443e+02_rb,1.1722e+02_rb,1.0884e+01_rb/)
      kao(:, 5,12,15) = (/ &
     &1.1629e+03_rb,1.0175e+03_rb,8.7216e+02_rb,7.2684e+02_rb,5.8142e+02_rb,4.3609e+02_rb, &
     &2.9073e+02_rb,1.4537e+02_rb,1.6871e+01_rb/)
      kao(:, 1,13,15) = (/ &
     &4.3408e+02_rb,3.7980e+02_rb,3.2553e+02_rb,2.7129e+02_rb,2.1704e+02_rb,1.6278e+02_rb, &
     &1.0851e+02_rb,5.4254e+01_rb,3.3387e-01_rb/)
      kao(:, 2,13,15) = (/ &
     &6.1113e+02_rb,5.3482e+02_rb,4.5841e+02_rb,3.8201e+02_rb,3.0560e+02_rb,2.2920e+02_rb, &
     &1.5279e+02_rb,7.6397e+01_rb,3.4060e+00_rb/)
      kao(:, 3,13,15) = (/ &
     &8.2033e+02_rb,7.1780e+02_rb,6.1525e+02_rb,5.1272e+02_rb,4.1017e+02_rb,3.0764e+02_rb, &
     &2.0509e+02_rb,1.0254e+02_rb,7.3848e+00_rb/)
      kao(:, 4,13,15) = (/ &
     &1.0623e+03_rb,9.2952e+02_rb,7.9671e+02_rb,6.6390e+02_rb,5.3111e+02_rb,3.9834e+02_rb, &
     &2.6556e+02_rb,1.3278e+02_rb,1.3310e+01_rb/)
      kao(:, 5,13,15) = (/ &
     &1.3308e+03_rb,1.1645e+03_rb,9.9813e+02_rb,8.3174e+02_rb,6.6542e+02_rb,4.9907e+02_rb, &
     &3.3271e+02_rb,1.6635e+02_rb,1.8499e+01_rb/)
      kao(:, 1, 1,16) = (/ &
     &1.7254e-01_rb,6.6462e-01_rb,1.3115e+00_rb,1.9318e+00_rb,2.5084e+00_rb,3.0076e+00_rb, &
     &3.3449e+00_rb,3.2523e+00_rb,5.0169e+00_rb/)
      kao(:, 2, 1,16) = (/ &
     &1.9945e-01_rb,8.1688e-01_rb,1.6118e+00_rb,2.3741e+00_rb,3.0836e+00_rb,3.6966e+00_rb, &
     &4.1116e+00_rb,3.9980e+00_rb,6.1673e+00_rb/)
      kao(:, 3, 1,16) = (/ &
     &2.2510e-01_rb,1.0070e+00_rb,1.9841e+00_rb,2.9159e+00_rb,3.7748e+00_rb,4.5037e+00_rb, &
     &4.9689e+00_rb,4.7883e+00_rb,7.5495e+00_rb/)
      kao(:, 4, 1,16) = (/ &
     &2.5048e-01_rb,1.2252e+00_rb,2.4140e+00_rb,3.5479e+00_rb,4.5928e+00_rb,5.4806e+00_rb, &
     &6.0471e+00_rb,5.7831e+00_rb,9.1856e+00_rb/)
      kao(:, 5, 1,16) = (/ &
     &2.7413e-01_rb,1.4585e+00_rb,2.8735e+00_rb,4.2233e+00_rb,5.4678e+00_rb,6.4913e+00_rb, &
     &7.1996e+00_rb,6.8835e+00_rb,1.0936e+01_rb/)
      kao(:, 1, 2,16) = (/ &
     &2.5949e-01_rb,7.4073e-01_rb,1.4685e+00_rb,2.1764e+00_rb,2.8509e+00_rb,3.4639e+00_rb, &
     &3.9405e+00_rb,4.0097e+00_rb,5.7017e+00_rb/)
      kao(:, 2, 2,16) = (/ &
     &3.0220e-01_rb,9.2128e-01_rb,1.8172e+00_rb,2.6931e+00_rb,3.5460e+00_rb,4.3080e+00_rb, &
     &4.9007e+00_rb,4.9876e+00_rb,7.0918e+00_rb/)
      kao(:, 3, 2,16) = (/ &
     &3.4341e-01_rb,1.1291e+00_rb,2.2360e+00_rb,3.3088e+00_rb,4.3252e+00_rb,5.2383e+00_rb, &
     &5.9360e+00_rb,6.0318e+00_rb,8.6502e+00_rb/)
      kao(:, 4, 2,16) = (/ &
     &3.8325e-01_rb,1.3874e+00_rb,2.7481e+00_rb,4.0666e+00_rb,5.3159e+00_rb,6.4367e+00_rb, &
     &7.2790e+00_rb,7.3059e+00_rb,1.0632e+01_rb/)
      kao(:, 5, 2,16) = (/ &
     &4.2153e-01_rb,1.6660e+00_rb,3.2992e+00_rb,4.8823e+00_rb,6.3819e+00_rb,7.7290e+00_rb, &
     &8.7410e+00_rb,8.7737e+00_rb,1.2764e+01_rb/)
      kao(:, 1, 3,16) = (/ &
     &5.0644e-01_rb,7.8255e-01_rb,1.5587e+00_rb,2.3239e+00_rb,3.0716e+00_rb,3.7848e+00_rb, &
     &4.4177e+00_rb,4.7740e+00_rb,6.1431e+00_rb/)
      kao(:, 2, 3,16) = (/ &
     &5.9759e-01_rb,9.9169e-01_rb,1.9751e+00_rb,2.9450e+00_rb,3.8926e+00_rb,4.7966e+00_rb, &
     &5.5986e+00_rb,6.0493e+00_rb,7.7852e+00_rb/)
      kao(:, 3, 3,16) = (/ &
     &6.8696e-01_rb,1.2186e+00_rb,2.4265e+00_rb,3.6186e+00_rb,4.7825e+00_rb,5.8630e+00_rb, &
     &6.8786e+00_rb,7.4328e+00_rb,9.5649e+00_rb/)
      kao(:, 4, 3,16) = (/ &
     &7.7291e-01_rb,1.5044e+00_rb,2.9942e+00_rb,4.4622e+00_rb,5.8918e+00_rb,7.2471e+00_rb, &
     &8.4308e+00_rb,9.0321e+00_rb,1.1784e+01_rb/)
      kao(:, 5, 3,16) = (/ &
     &8.5655e-01_rb,1.8308e+00_rb,3.6443e+00_rb,5.4303e+00_rb,7.1696e+00_rb,8.8191e+00_rb, &
     &1.0261e+01_rb,1.0994e+01_rb,1.4339e+01_rb/)
      kao(:, 1, 4,16) = (/ &
     &9.9660e-01_rb,8.7187e-01_rb,1.6058e+00_rb,2.4020e+00_rb,3.1900e+00_rb,3.9596e+00_rb, &
     &4.6884e+00_rb,5.2602e+00_rb,6.3800e+00_rb/)
      kao(:, 2, 4,16) = (/ &
     &1.1909e+00_rb,1.0454e+00_rb,2.0813e+00_rb,3.1135e+00_rb,4.1338e+00_rb,5.1320e+00_rb, &
     &6.0754e+00_rb,6.8176e+00_rb,8.2675e+00_rb/)
      kao(:, 3, 4,16) = (/ &
     &1.3883e+00_rb,1.3053e+00_rb,2.6054e+00_rb,3.8972e+00_rb,5.1746e+00_rb,6.4245e+00_rb, &
     &7.6056e+00_rb,8.5415e+00_rb,1.0349e+01_rb/)
      kao(:, 4, 4,16) = (/ &
     &1.5813e+00_rb,1.6017e+00_rb,3.1965e+00_rb,4.7803e+00_rb,6.3446e+00_rb,7.8728e+00_rb, &
     &9.3118e+00_rb,1.0423e+01_rb,1.2689e+01_rb/)
      kao(:, 5, 4,16) = (/ &
     &1.7666e+00_rb,1.9742e+00_rb,3.9398e+00_rb,5.8918e+00_rb,7.8191e+00_rb,9.6976e+00_rb, &
     &1.1463e+01_rb,1.2804e+01_rb,1.5638e+01_rb/)
      kao(:, 1, 5,16) = (/ &
     &1.7905e+00_rb,1.5668e+00_rb,1.6306e+00_rb,2.4424e+00_rb,3.2499e+00_rb,4.0478e+00_rb, &
     &4.8237e+00_rb,5.5141e+00_rb,6.4998e+00_rb/)
      kao(:, 2, 5,16) = (/ &
     &2.1763e+00_rb,1.9041e+00_rb,2.1658e+00_rb,3.2440e+00_rb,4.3166e+00_rb,5.3771e+00_rb, &
     &6.4071e+00_rb,7.3247e+00_rb,8.6332e+00_rb/)
      kao(:, 3, 5,16) = (/ &
     &2.5702e+00_rb,2.2488e+00_rb,2.7671e+00_rb,4.1446e+00_rb,5.5145e+00_rb,6.8692e+00_rb, &
     &8.1862e+00_rb,9.3584e+00_rb,1.1029e+01_rb/)
      kao(:, 4, 5,16) = (/ &
     &2.9652e+00_rb,2.5941e+00_rb,3.4291e+00_rb,5.1370e+00_rb,6.8346e+00_rb,8.5115e+00_rb, &
     &1.0142e+01_rb,1.1590e+01_rb,1.3669e+01_rb/)
      kao(:, 5, 5,16) = (/ &
     &3.3442e+00_rb,2.9260e+00_rb,4.2225e+00_rb,6.3231e+00_rb,8.4117e+00_rb,1.0473e+01_rb, &
     &1.2473e+01_rb,1.4226e+01_rb,1.6823e+01_rb/)
      kao(:, 1, 6,16) = (/ &
     &2.9659e+00_rb,2.5957e+00_rb,2.2251e+00_rb,2.4192e+00_rb,3.2212e+00_rb,4.0191e+00_rb, &
     &4.8055e+00_rb,5.5432e+00_rb,6.4424e+00_rb/)
      kao(:, 2, 6,16) = (/ &
     &3.6832e+00_rb,3.2223e+00_rb,2.7616e+00_rb,3.3057e+00_rb,4.4026e+00_rb,5.4931e+00_rb, &
     &6.5666e+00_rb,7.5768e+00_rb,8.8051e+00_rb/)
      kao(:, 3, 6,16) = (/ &
     &4.4116e+00_rb,3.8604e+00_rb,3.3096e+00_rb,4.3238e+00_rb,5.7589e+00_rb,7.1852e+00_rb, &
     &8.5905e+00_rb,9.9138e+00_rb,1.1518e+01_rb/)
      kao(:, 4, 6,16) = (/ &
     &5.1628e+00_rb,4.5167e+00_rb,3.8530e+00_rb,5.4598e+00_rb,7.2711e+00_rb,9.0722e+00_rb, &
     &1.0846e+01_rb,1.2515e+01_rb,1.4542e+01_rb/)
      kao(:, 5, 6,16) = (/ &
     &5.8968e+00_rb,5.1602e+00_rb,4.5054e+00_rb,6.7527e+00_rb,8.9935e+00_rb,1.1218e+01_rb, &
     &1.3409e+01_rb,1.5457e+01_rb,1.7987e+01_rb/)
      kao(:, 1, 7,16) = (/ &
     &5.1220e+00_rb,4.4578e+00_rb,3.8412e+00_rb,3.1847e+00_rb,3.1367e+00_rb,3.9170e+00_rb, &
     &4.6915e+00_rb,5.4417e+00_rb,6.2734e+00_rb/)
      kao(:, 2, 7,16) = (/ &
     &6.5146e+00_rb,5.7005e+00_rb,4.8852e+00_rb,4.0720e+00_rb,4.4224e+00_rb,5.5237e+00_rb, &
     &6.6151e+00_rb,7.6744e+00_rb,8.8447e+00_rb/)
      kao(:, 3, 7,16) = (/ &
     &7.9642e+00_rb,6.9679e+00_rb,5.9715e+00_rb,4.9772e+00_rb,5.9352e+00_rb,7.4116e+00_rb, &
     &8.8775e+00_rb,1.0298e+01_rb,1.1870e+01_rb/)
      kao(:, 4, 7,16) = (/ &
     &9.4454e+00_rb,8.2650e+00_rb,7.0847e+00_rb,5.9047e+00_rb,7.6537e+00_rb,9.5584e+00_rb, &
     &1.1448e+01_rb,1.3281e+01_rb,1.5307e+01_rb/)
      kao(:, 5, 7,16) = (/ &
     &1.0942e+01_rb,9.5731e+00_rb,8.2052e+00_rb,7.1985e+00_rb,9.5915e+00_rb,1.1978e+01_rb, &
     &1.4344e+01_rb,1.6634e+01_rb,1.9183e+01_rb/)
      kao(:, 1, 8,16) = (/ &
     &1.0533e+01_rb,9.2146e+00_rb,7.8983e+00_rb,6.5814e+00_rb,5.2661e+00_rb,3.9493e+00_rb, &
     &4.5315e+00_rb,5.2737e+00_rb,6.0497e+00_rb/)
      kao(:, 2, 8,16) = (/ &
     &1.3750e+01_rb,1.2032e+01_rb,1.0313e+01_rb,8.5934e+00_rb,6.8745e+00_rb,5.4846e+00_rb, &
     &6.5763e+00_rb,7.6139e+00_rb,8.7787e+00_rb/)
      kao(:, 3, 8,16) = (/ &
     &1.7175e+01_rb,1.5029e+01_rb,1.2881e+01_rb,1.0735e+01_rb,8.5883e+00_rb,7.5634e+00_rb, &
     &9.0692e+00_rb,1.0555e+01_rb,1.2107e+01_rb/)
      kao(:, 4, 8,16) = (/ &
     &2.0707e+01_rb,1.8120e+01_rb,1.5532e+01_rb,1.2944e+01_rb,1.0356e+01_rb,9.9766e+00_rb, &
     &1.1963e+01_rb,1.3921e+01_rb,1.5970e+01_rb/)
      kao(:, 5, 8,16) = (/ &
     &2.4320e+01_rb,2.1282e+01_rb,1.8238e+01_rb,1.5200e+01_rb,1.2161e+01_rb,1.2723e+01_rb, &
     &1.5254e+01_rb,1.7752e+01_rb,2.0364e+01_rb/)
      kao(:, 1, 9,16) = (/ &
     &4.0363e+01_rb,3.5318e+01_rb,3.0273e+01_rb,2.5227e+01_rb,2.0181e+01_rb,1.5134e+01_rb, &
     &1.0090e+01_rb,5.4212e+00_rb,6.1996e+00_rb/)
      kao(:, 2, 9,16) = (/ &
     &5.4231e+01_rb,4.7444e+01_rb,4.0667e+01_rb,3.3896e+01_rb,2.7114e+01_rb,2.0336e+01_rb, &
     &1.3557e+01_rb,7.4876e+00_rb,8.5638e+00_rb/)
      kao(:, 3, 9,16) = (/ &
     &6.9375e+01_rb,6.0706e+01_rb,5.2030e+01_rb,4.3363e+01_rb,3.4693e+01_rb,2.6019e+01_rb, &
     &1.7346e+01_rb,1.0642e+01_rb,1.2169e+01_rb/)
      kao(:, 4, 9,16) = (/ &
     &8.5364e+01_rb,7.4694e+01_rb,6.4003e+01_rb,5.3349e+01_rb,4.2675e+01_rb,3.2007e+01_rb, &
     &2.1338e+01_rb,1.4379e+01_rb,1.6444e+01_rb/)
      kao(:, 5, 9,16) = (/ &
     &1.0173e+02_rb,8.9011e+01_rb,7.6294e+01_rb,6.3578e+01_rb,5.0858e+01_rb,3.8147e+01_rb, &
     &2.5431e+01_rb,1.8715e+01_rb,2.1406e+01_rb/)
      kao(:, 1,10,16) = (/ &
     &1.7518e+02_rb,1.5326e+02_rb,1.3138e+02_rb,1.0946e+02_rb,8.7589e+01_rb,6.5684e+01_rb, &
     &4.3795e+01_rb,2.1898e+01_rb,6.6096e+00_rb/)
      kao(:, 2,10,16) = (/ &
     &2.4209e+02_rb,2.1181e+02_rb,1.8152e+02_rb,1.5127e+02_rb,1.2102e+02_rb,9.0779e+01_rb, &
     &6.0510e+01_rb,3.0258e+01_rb,8.7477e+00_rb/)
      kao(:, 3,10,16) = (/ &
     &3.1702e+02_rb,2.7739e+02_rb,2.3777e+02_rb,1.9810e+02_rb,1.5849e+02_rb,1.1888e+02_rb, &
     &7.9249e+01_rb,3.9623e+01_rb,1.2377e+01_rb/)
      kao(:, 4,10,16) = (/ &
     &3.9791e+02_rb,3.4817e+02_rb,2.9841e+02_rb,2.4869e+02_rb,1.9896e+02_rb,1.4923e+02_rb, &
     &9.9472e+01_rb,4.9736e+01_rb,1.7102e+01_rb/)
      kao(:, 5,10,16) = (/ &
     &4.8342e+02_rb,4.2295e+02_rb,3.6251e+02_rb,3.0211e+02_rb,2.4169e+02_rb,1.8128e+02_rb, &
     &1.2083e+02_rb,6.0427e+01_rb,2.2752e+01_rb/)
      kao(:, 1,11,16) = (/ &
     &3.1380e+02_rb,2.7453e+02_rb,2.3526e+02_rb,1.9609e+02_rb,1.5686e+02_rb,1.1764e+02_rb, &
     &7.8455e+01_rb,3.9219e+01_rb,4.1702e-05_rb/)
      kao(:, 2,11,16) = (/ &
     &4.3458e+02_rb,3.8032e+02_rb,3.2596e+02_rb,2.7165e+02_rb,2.1736e+02_rb,1.6300e+02_rb, &
     &1.0866e+02_rb,5.4331e+01_rb,1.0603e+01_rb/)
      kao(:, 3,11,16) = (/ &
     &5.7279e+02_rb,5.0121e+02_rb,4.2963e+02_rb,3.5802e+02_rb,2.8639e+02_rb,2.1479e+02_rb, &
     &1.4321e+02_rb,7.1606e+01_rb,1.4737e+01_rb/)
      kao(:, 4,11,16) = (/ &
     &7.2603e+02_rb,6.3199e+02_rb,5.4450e+02_rb,4.5379e+02_rb,3.6299e+02_rb,2.7226e+02_rb, &
     &1.8150e+02_rb,9.0755e+01_rb,2.0418e+01_rb/)
      kao(:, 5,11,16) = (/ &
     &8.9810e+02_rb,7.8584e+02_rb,6.7355e+02_rb,5.6126e+02_rb,4.4901e+02_rb,3.3674e+02_rb, &
     &2.2451e+02_rb,1.1225e+02_rb,2.7345e+01_rb/)
      kao(:, 1,12,16) = (/ &
     &4.2564e+02_rb,3.7252e+02_rb,3.1927e+02_rb,2.6605e+02_rb,2.1287e+02_rb,1.5962e+02_rb, &
     &1.0643e+02_rb,5.3212e+01_rb,3.3974e-05_rb/)
      kao(:, 2,12,16) = (/ &
     &5.9696e+02_rb,5.2232e+02_rb,4.4774e+02_rb,3.7307e+02_rb,2.9847e+02_rb,2.2385e+02_rb, &
     &1.4924e+02_rb,7.4619e+01_rb,1.2841e+01_rb/)
      kao(:, 3,12,16) = (/ &
     &8.0130e+02_rb,7.0109e+02_rb,6.0095e+02_rb,5.0082e+02_rb,4.0064e+02_rb,3.0049e+02_rb, &
     &2.0030e+02_rb,1.0016e+02_rb,1.7604e+01_rb/)
      kao(:, 4,12,16) = (/ &
     &1.0424e+03_rb,9.1209e+02_rb,7.8186e+02_rb,6.5151e+02_rb,5.2121e+02_rb,3.9092e+02_rb, &
     &2.6059e+02_rb,1.3030e+02_rb,2.4408e+01_rb/)
      kao(:, 5,12,16) = (/ &
     &1.3107e+03_rb,1.1467e+03_rb,9.8296e+02_rb,8.1915e+02_rb,6.5538e+02_rb,4.9148e+02_rb, &
     &3.2766e+02_rb,1.6385e+02_rb,3.2853e+01_rb/)
      kao(:, 1,13,16) = (/ &
     &4.8122e+02_rb,4.2105e+02_rb,3.6087e+02_rb,3.0076e+02_rb,2.4060e+02_rb,1.8046e+02_rb, &
     &1.2030e+02_rb,6.0148e+01_rb,7.3993e+00_rb/)
      kao(:, 2,13,16) = (/ &
     &6.9206e+02_rb,6.0557e+02_rb,5.1907e+02_rb,4.3255e+02_rb,3.4603e+02_rb,2.5955e+02_rb, &
     &1.7302e+02_rb,8.6508e+01_rb,1.5501e+01_rb/)
      kao(:, 3,13,16) = (/ &
     &9.4703e+02_rb,8.2867e+02_rb,7.1030e+02_rb,5.9189e+02_rb,4.7354e+02_rb,3.5512e+02_rb, &
     &2.3675e+02_rb,1.1838e+02_rb,2.1029e+01_rb/)
      kao(:, 4,13,16) = (/ &
     &1.2338e+03_rb,1.0854e+03_rb,9.3026e+02_rb,7.7526e+02_rb,6.2018e+02_rb,4.6513e+02_rb, &
     &3.1009e+02_rb,1.5423e+02_rb,2.9122e+01_rb/)
      kao(:, 5,13,16) = (/ &
     &1.5695e+03_rb,1.3734e+03_rb,1.1772e+03_rb,9.8104e+02_rb,7.8484e+02_rb,5.8860e+02_rb, &
     &3.9240e+02_rb,1.9620e+02_rb,3.9289e+01_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:, 1,13, 1) = (/ &
     &9.4796e-05_rb,1.4361e-04_rb,1.4118e-04_rb,1.1819e-04_rb,7.6303e-05_rb/)
      kbo(:, 2,13, 1) = (/ &
     &9.7387e-05_rb,1.7115e-04_rb,1.7300e-04_rb,1.4824e-04_rb,1.0469e-04_rb/)
      kbo(:, 3,13, 1) = (/ &
     &1.0951e-04_rb,2.1604e-04_rb,2.2323e-04_rb,1.9350e-04_rb,1.3818e-04_rb/)
      kbo(:, 4,13, 1) = (/ &
     &1.3449e-04_rb,2.8552e-04_rb,2.9728e-04_rb,2.5764e-04_rb,1.7597e-04_rb/)
      kbo(:, 5,13, 1) = (/ &
     &1.7577e-04_rb,3.8714e-04_rb,4.0359e-04_rb,3.4654e-04_rb,2.1851e-04_rb/)
      kbo(:, 1,14, 1) = (/ &
     &7.9000e-05_rb,1.2113e-04_rb,1.1920e-04_rb,9.9664e-05_rb,6.3646e-05_rb/)
      kbo(:, 2,14, 1) = (/ &
     &8.1870e-05_rb,1.4526e-04_rb,1.4705e-04_rb,1.2557e-04_rb,8.6376e-05_rb/)
      kbo(:, 3,14, 1) = (/ &
     &9.2975e-05_rb,1.8451e-04_rb,1.9063e-04_rb,1.6489e-04_rb,1.1348e-04_rb/)
      kbo(:, 4,14, 1) = (/ &
     &1.1499e-04_rb,2.4478e-04_rb,2.5460e-04_rb,2.2031e-04_rb,1.4451e-04_rb/)
      kbo(:, 5,14, 1) = (/ &
     &1.5077e-04_rb,3.3228e-04_rb,3.4580e-04_rb,2.9716e-04_rb,1.7798e-04_rb/)
      kbo(:, 1,15, 1) = (/ &
     &6.3146e-05_rb,9.9373e-05_rb,9.8373e-05_rb,8.2691e-05_rb,5.2397e-05_rb/)
      kbo(:, 2,15, 1) = (/ &
     &6.6078e-05_rb,1.2006e-04_rb,1.2228e-04_rb,1.0469e-04_rb,7.0411e-05_rb/)
      kbo(:, 3,15, 1) = (/ &
     &7.5821e-05_rb,1.5360e-04_rb,1.5869e-04_rb,1.3814e-04_rb,9.1603e-05_rb/)
      kbo(:, 4,15, 1) = (/ &
     &9.4372e-05_rb,2.0448e-04_rb,2.1279e-04_rb,1.8521e-04_rb,1.1587e-04_rb/)
      kbo(:, 5,15, 1) = (/ &
     &1.2416e-04_rb,2.7753e-04_rb,2.8967e-04_rb,2.5010e-04_rb,1.4409e-04_rb/)
      kbo(:, 1,16, 1) = (/ &
     &4.9335e-05_rb,8.0237e-05_rb,8.0247e-05_rb,6.8036e-05_rb,4.2623e-05_rb/)
      kbo(:, 2,16, 1) = (/ &
     &5.2173e-05_rb,9.7834e-05_rb,1.0022e-04_rb,8.6725e-05_rb,5.7211e-05_rb/)
      kbo(:, 3,16, 1) = (/ &
     &6.0461e-05_rb,1.2605e-04_rb,1.3061e-04_rb,1.1470e-04_rb,7.4324e-05_rb/)
      kbo(:, 4,16, 1) = (/ &
     &7.5694e-05_rb,1.6827e-04_rb,1.7572e-04_rb,1.5404e-04_rb,9.3686e-05_rb/)
      kbo(:, 5,16, 1) = (/ &
     &9.9955e-05_rb,2.2866e-04_rb,2.3956e-04_rb,2.0826e-04_rb,1.1548e-04_rb/)
      kbo(:, 1,17, 1) = (/ &
     &3.8266e-05_rb,6.4449e-05_rb,6.5172e-05_rb,5.5789e-05_rb,3.5130e-05_rb/)
      kbo(:, 2,17, 1) = (/ &
     &4.0895e-05_rb,7.9297e-05_rb,8.1758e-05_rb,7.1548e-05_rb,4.6781e-05_rb/)
      kbo(:, 3,17, 1) = (/ &
     &4.7807e-05_rb,1.0278e-04_rb,1.0697e-04_rb,9.4840e-05_rb,6.0133e-05_rb/)
      kbo(:, 4,17, 1) = (/ &
     &6.0159e-05_rb,1.3765e-04_rb,1.4446e-04_rb,1.2759e-04_rb,7.5139e-05_rb/)
      kbo(:, 5,17, 1) = (/ &
     &7.9747e-05_rb,1.8704e-04_rb,1.9692e-04_rb,1.7259e-04_rb,9.1648e-05_rb/)
      kbo(:, 1,18, 1) = (/ &
     &2.9184e-05_rb,5.1211e-05_rb,5.2362e-05_rb,4.5451e-05_rb,2.8937e-05_rb/)
      kbo(:, 2,18, 1) = (/ &
     &3.1502e-05_rb,6.3605e-05_rb,6.6096e-05_rb,5.8591e-05_rb,3.8202e-05_rb/)
      kbo(:, 3,18, 1) = (/ &
     &3.7129e-05_rb,8.2865e-05_rb,8.6932e-05_rb,7.7931e-05_rb,4.8777e-05_rb/)
      kbo(:, 4,18, 1) = (/ &
     &4.6953e-05_rb,1.1128e-04_rb,1.1763e-04_rb,1.0496e-04_rb,6.0806e-05_rb/)
      kbo(:, 5,18, 1) = (/ &
     &6.2502e-05_rb,1.5129e-04_rb,1.6025e-04_rb,1.4184e-04_rb,7.4536e-05_rb/)
      kbo(:, 1,19, 1) = (/ &
     &2.2672e-05_rb,4.1242e-05_rb,4.2527e-05_rb,3.7342e-05_rb,2.3934e-05_rb/)
      kbo(:, 2,19, 1) = (/ &
     &2.4699e-05_rb,5.1600e-05_rb,5.3953e-05_rb,4.8350e-05_rb,3.1473e-05_rb/)
      kbo(:, 3,19, 1) = (/ &
     &2.9334e-05_rb,6.7547e-05_rb,7.1294e-05_rb,6.4461e-05_rb,4.0025e-05_rb/)
      kbo(:, 4,19, 1) = (/ &
     &3.7286e-05_rb,9.0838e-05_rb,9.6625e-05_rb,8.6862e-05_rb,4.9869e-05_rb/)
      kbo(:, 5,19, 1) = (/ &
     &4.9822e-05_rb,1.2352e-04_rb,1.3152e-04_rb,1.1713e-04_rb,6.1219e-05_rb/)
      kbo(:, 1,20, 1) = (/ &
     &1.8003e-05_rb,3.3802e-05_rb,3.5042e-05_rb,3.1068e-05_rb,2.0074e-05_rb/)
      kbo(:, 2,20, 1) = (/ &
     &1.9811e-05_rb,4.2600e-05_rb,4.4765e-05_rb,4.0426e-05_rb,2.6188e-05_rb/)
      kbo(:, 3,20, 1) = (/ &
     &2.3740e-05_rb,5.5990e-05_rb,5.9400e-05_rb,5.4019e-05_rb,3.3275e-05_rb/)
      kbo(:, 4,20, 1) = (/ &
     &3.0362e-05_rb,7.5435e-05_rb,8.0630e-05_rb,7.2808e-05_rb,4.1380e-05_rb/)
      kbo(:, 5,20, 1) = (/ &
     &4.0743e-05_rb,1.0263e-04_rb,1.0963e-04_rb,9.7998e-05_rb,5.0442e-05_rb/)
      kbo(:, 1,21, 1) = (/ &
     &1.4410e-05_rb,2.7872e-05_rb,2.9027e-05_rb,2.5943e-05_rb,1.6820e-05_rb/)
      kbo(:, 2,21, 1) = (/ &
     &1.6018e-05_rb,3.5348e-05_rb,3.7320e-05_rb,3.3900e-05_rb,2.1893e-05_rb/)
      kbo(:, 3,21, 1) = (/ &
     &1.9364e-05_rb,4.6630e-05_rb,4.9701e-05_rb,4.5376e-05_rb,2.7723e-05_rb/)
      kbo(:, 4,21, 1) = (/ &
     &2.4923e-05_rb,6.2936e-05_rb,6.7505e-05_rb,6.1129e-05_rb,3.4253e-05_rb/)
      kbo(:, 5,21, 1) = (/ &
     &3.3581e-05_rb,8.5657e-05_rb,9.1740e-05_rb,8.2183e-05_rb,4.1502e-05_rb/)
      kbo(:, 1,22, 1) = (/ &
     &1.1606e-05_rb,2.3345e-05_rb,2.4458e-05_rb,2.2044e-05_rb,1.4341e-05_rb/)
      kbo(:, 2,22, 1) = (/ &
     &1.3108e-05_rb,2.9860e-05_rb,3.1703e-05_rb,2.8973e-05_rb,1.8544e-05_rb/)
      kbo(:, 3,22, 1) = (/ &
     &1.6058e-05_rb,3.9598e-05_rb,4.2399e-05_rb,3.8843e-05_rb,2.3272e-05_rb/)
      kbo(:, 4,22, 1) = (/ &
     &2.0871e-05_rb,5.3592e-05_rb,5.7668e-05_rb,5.2303e-05_rb,2.8594e-05_rb/)
      kbo(:, 5,22, 1) = (/ &
     &2.8287e-05_rb,7.2910e-05_rb,7.8274e-05_rb,7.0191e-05_rb,3.4495e-05_rb/)
      kbo(:, 1,23, 1) = (/ &
     &9.3973e-06_rb,1.9636e-05_rb,2.0704e-05_rb,1.8793e-05_rb,1.2204e-05_rb/)
      kbo(:, 2,23, 1) = (/ &
     &1.0787e-05_rb,2.5322e-05_rb,2.7025e-05_rb,2.4831e-05_rb,1.5654e-05_rb/)
      kbo(:, 3,23, 1) = (/ &
     &1.3385e-05_rb,3.3729e-05_rb,3.6290e-05_rb,3.3307e-05_rb,1.9609e-05_rb/)
      kbo(:, 4,23, 1) = (/ &
     &1.7566e-05_rb,4.5745e-05_rb,4.9368e-05_rb,4.4833e-05_rb,2.3893e-05_rb/)
      kbo(:, 5,23, 1) = (/ &
     &2.3931e-05_rb,6.2215e-05_rb,6.6919e-05_rb,6.0045e-05_rb,2.8668e-05_rb/)
      kbo(:, 1,24, 1) = (/ &
     &7.7170e-06_rb,1.6679e-05_rb,1.7685e-05_rb,1.6128e-05_rb,1.0358e-05_rb/)
      kbo(:, 2,24, 1) = (/ &
     &9.0120e-06_rb,2.1659e-05_rb,2.3221e-05_rb,2.1395e-05_rb,1.3206e-05_rb/)
      kbo(:, 3,24, 1) = (/ &
     &1.1319e-05_rb,2.8989e-05_rb,3.1289e-05_rb,2.8717e-05_rb,1.6361e-05_rb/)
      kbo(:, 4,24, 1) = (/ &
     &1.4992e-05_rb,3.9385e-05_rb,4.2555e-05_rb,3.8633e-05_rb,1.9891e-05_rb/)
      kbo(:, 5,24, 1) = (/ &
     &2.0520e-05_rb,5.3479e-05_rb,5.7600e-05_rb,5.1654e-05_rb,2.3746e-05_rb/)
      kbo(:, 1,25, 1) = (/ &
     &6.3838e-06_rb,1.4248e-05_rb,1.5168e-05_rb,1.3906e-05_rb,8.8251e-06_rb/)
      kbo(:, 2,25, 1) = (/ &
     &7.5878e-06_rb,1.8635e-05_rb,2.0050e-05_rb,1.8513e-05_rb,1.1119e-05_rb/)
      kbo(:, 3,25, 1) = (/ &
     &9.6475e-06_rb,2.5052e-05_rb,2.7110e-05_rb,2.4846e-05_rb,1.3734e-05_rb/)
      kbo(:, 4,25, 1) = (/ &
     &1.2889e-05_rb,3.4075e-05_rb,3.6834e-05_rb,3.3385e-05_rb,1.6609e-05_rb/)
      kbo(:, 5,25, 1) = (/ &
     &1.7706e-05_rb,4.6181e-05_rb,4.9751e-05_rb,4.4544e-05_rb,1.9735e-05_rb/)
      kbo(:, 1,26, 1) = (/ &
     &5.3393e-06_rb,1.2288e-05_rb,1.3132e-05_rb,1.2096e-05_rb,7.5264e-06_rb/)
      kbo(:, 2,26, 1) = (/ &
     &6.4609e-06_rb,1.6189e-05_rb,1.7467e-05_rb,1.6131e-05_rb,9.4332e-06_rb/)
      kbo(:, 3,26, 1) = (/ &
     &8.3200e-06_rb,2.1853e-05_rb,2.3681e-05_rb,2.1653e-05_rb,1.1593e-05_rb/)
      kbo(:, 4,26, 1) = (/ &
     &1.1209e-05_rb,2.9737e-05_rb,3.2133e-05_rb,2.9046e-05_rb,1.3941e-05_rb/)
      kbo(:, 5,26, 1) = (/ &
     &1.5443e-05_rb,4.0196e-05_rb,4.3302e-05_rb,3.8639e-05_rb,1.6481e-05_rb/)
      kbo(:, 1,27, 1) = (/ &
     &4.5029e-06_rb,1.0646e-05_rb,1.1429e-05_rb,1.0549e-05_rb,6.4263e-06_rb/)
      kbo(:, 2,27, 1) = (/ &
     &5.5408e-06_rb,1.4124e-05_rb,1.5272e-05_rb,1.4080e-05_rb,8.0128e-06_rb/)
      kbo(:, 3,27, 1) = (/ &
     &7.2235e-06_rb,1.9128e-05_rb,2.0734e-05_rb,1.8902e-05_rb,9.7874e-06_rb/)
      kbo(:, 4,27, 1) = (/ &
     &9.8044e-06_rb,2.6022e-05_rb,2.8102e-05_rb,2.5321e-05_rb,1.1642e-05_rb/)
      kbo(:, 5,27, 1) = (/ &
     &1.3531e-05_rb,3.5058e-05_rb,3.7762e-05_rb,3.3580e-05_rb,1.3712e-05_rb/)
      kbo(:, 1,28, 1) = (/ &
     &3.8267e-06_rb,9.2605e-06_rb,9.9753e-06_rb,9.2087e-06_rb,5.4910e-06_rb/)
      kbo(:, 2,28, 1) = (/ &
     &4.7813e-06_rb,1.2361e-05_rb,1.3381e-05_rb,1.2301e-05_rb,6.7809e-06_rb/)
      kbo(:, 3,28, 1) = (/ &
     &6.3052e-06_rb,1.6784e-05_rb,1.8184e-05_rb,1.6520e-05_rb,8.1872e-06_rb/)
      kbo(:, 4,28, 1) = (/ &
     &8.6092e-06_rb,2.2800e-05_rb,2.4603e-05_rb,2.2078e-05_rb,9.6541e-06_rb/)
      kbo(:, 5,28, 1) = (/ &
     &1.1891e-05_rb,3.0613e-05_rb,3.2964e-05_rb,2.9199e-05_rb,1.1371e-05_rb/)
      kbo(:, 1,29, 1) = (/ &
     &3.2554e-06_rb,8.0646e-06_rb,8.7115e-06_rb,8.0443e-06_rb,4.6597e-06_rb/)
      kbo(:, 2,29, 1) = (/ &
     &4.1267e-06_rb,1.0826e-05_rb,1.1737e-05_rb,1.0754e-05_rb,5.7169e-06_rb/)
      kbo(:, 3,29, 1) = (/ &
     &5.4998e-06_rb,1.4722e-05_rb,1.5944e-05_rb,1.4441e-05_rb,6.8326e-06_rb/)
      kbo(:, 4,29, 1) = (/ &
     &7.5454e-06_rb,1.9957e-05_rb,2.1533e-05_rb,1.9256e-05_rb,8.0674e-06_rb/)
      kbo(:, 5,29, 1) = (/ &
     &1.0421e-05_rb,2.6692e-05_rb,2.8752e-05_rb,2.5378e-05_rb,9.4635e-06_rb/)
      kbo(:, 1,30, 1) = (/ &
     &2.7722e-06_rb,7.0257e-06_rb,7.6070e-06_rb,7.0209e-06_rb,3.9148e-06_rb/)
      kbo(:, 2,30, 1) = (/ &
     &3.5629e-06_rb,9.4753e-06_rb,1.0287e-05_rb,9.3951e-06_rb,4.7641e-06_rb/)
      kbo(:, 3,30, 1) = (/ &
     &4.7932e-06_rb,1.2893e-05_rb,1.3959e-05_rb,1.2600e-05_rb,5.6708e-06_rb/)
      kbo(:, 4,30, 1) = (/ &
     &6.5982e-06_rb,1.7431e-05_rb,1.8810e-05_rb,1.6759e-05_rb,6.6902e-06_rb/)
      kbo(:, 5,30, 1) = (/ &
     &9.1066e-06_rb,2.3231e-05_rb,2.5041e-05_rb,2.2031e-05_rb,7.8394e-06_rb/)
      kbo(:, 1,31, 1) = (/ &
     &2.3506e-06_rb,6.1052e-06_rb,6.6305e-06_rb,6.1196e-06_rb,3.2861e-06_rb/)
      kbo(:, 2,31, 1) = (/ &
     &3.0616e-06_rb,8.2670e-06_rb,8.9874e-06_rb,8.1933e-06_rb,3.9690e-06_rb/)
      kbo(:, 3,31, 1) = (/ &
     &4.1518e-06_rb,1.1245e-05_rb,1.2180e-05_rb,1.0973e-05_rb,4.7077e-06_rb/)
      kbo(:, 4,31, 1) = (/ &
     &5.7278e-06_rb,1.5154e-05_rb,1.6366e-05_rb,1.4549e-05_rb,5.5398e-06_rb/)
      kbo(:, 5,31, 1) = (/ &
     &7.8906e-06_rb,2.0109e-05_rb,2.1708e-05_rb,1.9053e-05_rb,6.4566e-06_rb/)
      kbo(:, 1,32, 1) = (/ &
     &2.0097e-06_rb,5.3323e-06_rb,5.8021e-06_rb,5.3502e-06_rb,2.7241e-06_rb/)
      kbo(:, 2,32, 1) = (/ &
     &2.6508e-06_rb,7.2383e-06_rb,7.8773e-06_rb,7.1648e-06_rb,3.2755e-06_rb/)
      kbo(:, 3,32, 1) = (/ &
     &3.6184e-06_rb,9.8348e-06_rb,1.0656e-05_rb,9.5737e-06_rb,3.8774e-06_rb/)
      kbo(:, 4,32, 1) = (/ &
     &4.9951e-06_rb,1.3205e-05_rb,1.4271e-05_rb,1.2656e-05_rb,4.5194e-06_rb/)
      kbo(:, 5,32, 1) = (/ &
     &6.8654e-06_rb,1.7445e-05_rb,1.8860e-05_rb,1.6511e-05_rb,5.2597e-06_rb/)
      kbo(:, 1,33, 1) = (/ &
     &1.7307e-06_rb,4.6719e-06_rb,5.0926e-06_rb,4.6864e-06_rb,2.2504e-06_rb/)
      kbo(:, 2,33, 1) = (/ &
     &2.3079e-06_rb,6.3540e-06_rb,6.9152e-06_rb,6.2764e-06_rb,2.6926e-06_rb/)
      kbo(:, 3,33, 1) = (/ &
     &3.1665e-06_rb,8.6147e-06_rb,9.3343e-06_rb,8.3614e-06_rb,3.1814e-06_rb/)
      kbo(:, 4,33, 1) = (/ &
     &4.3731e-06_rb,1.1528e-05_rb,1.2464e-05_rb,1.1023e-05_rb,3.7164e-06_rb/)
      kbo(:, 5,33, 1) = (/ &
     &5.9878e-06_rb,1.5151e-05_rb,1.6400e-05_rb,1.4316e-05_rb,4.2974e-06_rb/)
      kbo(:, 1,34, 1) = (/ &
     &1.4981e-06_rb,4.0834e-06_rb,4.4531e-06_rb,4.0843e-06_rb,1.8539e-06_rb/)
      kbo(:, 2,34, 1) = (/ &
     &2.0150e-06_rb,5.5569e-06_rb,6.0419e-06_rb,5.4716e-06_rb,2.2285e-06_rb/)
      kbo(:, 3,34, 1) = (/ &
     &2.7725e-06_rb,7.5151e-06_rb,8.1387e-06_rb,7.2690e-06_rb,2.6334e-06_rb/)
      kbo(:, 4,34, 1) = (/ &
     &3.8256e-06_rb,1.0019e-05_rb,1.0836e-05_rb,9.5549e-06_rb,3.0709e-06_rb/)
      kbo(:, 5,34, 1) = (/ &
     &5.2217e-06_rb,1.3116e-05_rb,1.4207e-05_rb,1.2368e-05_rb,3.5492e-06_rb/)
      kbo(:, 1,35, 1) = (/ &
     &1.2683e-06_rb,3.4717e-06_rb,3.7843e-06_rb,3.4639e-06_rb,1.5033e-06_rb/)
      kbo(:, 2,35, 1) = (/ &
     &1.7130e-06_rb,4.7242e-06_rb,5.1321e-06_rb,4.6386e-06_rb,1.8045e-06_rb/)
      kbo(:, 3,35, 1) = (/ &
     &2.3598e-06_rb,6.3794e-06_rb,6.9043e-06_rb,6.1552e-06_rb,2.1312e-06_rb/)
      kbo(:, 4,35, 1) = (/ &
     &3.2533e-06_rb,8.4861e-06_rb,9.1773e-06_rb,8.0771e-06_rb,2.4797e-06_rb/)
      kbo(:, 5,35, 1) = (/ &
     &4.4317e-06_rb,1.1084e-05_rb,1.2009e-05_rb,1.0436e-05_rb,2.8460e-06_rb/)
      kbo(:, 1,36, 1) = (/ &
     &1.0446e-06_rb,2.8596e-06_rb,3.1142e-06_rb,2.8488e-06_rb,1.1841e-06_rb/)
      kbo(:, 2,36, 1) = (/ &
     &1.4115e-06_rb,3.8911e-06_rb,4.2236e-06_rb,3.8148e-06_rb,1.4188e-06_rb/)
      kbo(:, 3,36, 1) = (/ &
     &1.9447e-06_rb,5.2529e-06_rb,5.6822e-06_rb,5.0612e-06_rb,1.6763e-06_rb/)
      kbo(:, 4,36, 1) = (/ &
     &2.6806e-06_rb,6.9854e-06_rb,7.5506e-06_rb,6.6402e-06_rb,1.9492e-06_rb/)
      kbo(:, 5,36, 1) = (/ &
     &3.6503e-06_rb,9.1210e-06_rb,9.8800e-06_rb,8.5784e-06_rb,2.2478e-06_rb/)
      kbo(:, 1,37, 1) = (/ &
     &8.3141e-07_rb,2.2606e-06_rb,2.4589e-06_rb,2.2521e-06_rb,9.1953e-07_rb/)
      kbo(:, 2,37, 1) = (/ &
     &1.1186e-06_rb,3.0759e-06_rb,3.3374e-06_rb,3.0170e-06_rb,1.1075e-06_rb/)
      kbo(:, 3,37, 1) = (/ &
     &1.5392e-06_rb,4.1590e-06_rb,4.4972e-06_rb,4.0085e-06_rb,1.3109e-06_rb/)
      kbo(:, 4,37, 1) = (/ &
     &2.1232e-06_rb,5.5433e-06_rb,5.9872e-06_rb,5.2697e-06_rb,1.5318e-06_rb/)
      kbo(:, 5,37, 1) = (/ &
     &2.8971e-06_rb,7.2549e-06_rb,7.8520e-06_rb,6.8216e-06_rb,1.7720e-06_rb/)
      kbo(:, 1,38, 1) = (/ &
     &6.6100e-07_rb,1.7834e-06_rb,1.9371e-06_rb,1.7763e-06_rb,7.1225e-07_rb/)
      kbo(:, 2,38, 1) = (/ &
     &8.8499e-07_rb,2.4255e-06_rb,2.6307e-06_rb,2.3793e-06_rb,8.6302e-07_rb/)
      kbo(:, 3,38, 1) = (/ &
     &1.2155e-06_rb,3.2848e-06_rb,3.5503e-06_rb,3.1675e-06_rb,1.0267e-06_rb/)
      kbo(:, 4,38, 1) = (/ &
     &1.6779e-06_rb,4.3885e-06_rb,4.7364e-06_rb,4.1721e-06_rb,1.2041e-06_rb/)
      kbo(:, 5,38, 1) = (/ &
     &2.2938e-06_rb,5.7578e-06_rb,6.2256e-06_rb,5.4129e-06_rb,1.3941e-06_rb/)
      kbo(:, 1,39, 1) = (/ &
     &5.2622e-07_rb,1.4074e-06_rb,1.5268e-06_rb,1.4013e-06_rb,5.4821e-07_rb/)
      kbo(:, 2,39, 1) = (/ &
     &7.0069e-07_rb,1.9130e-06_rb,2.0737e-06_rb,1.8765e-06_rb,6.6832e-07_rb/)
      kbo(:, 3,39, 1) = (/ &
     &9.6029e-07_rb,2.5942e-06_rb,2.8023e-06_rb,2.5024e-06_rb,8.0073e-07_rb/)
      kbo(:, 4,39, 1) = (/ &
     &1.3259e-06_rb,3.4737e-06_rb,3.7464e-06_rb,3.3022e-06_rb,9.3841e-07_rb/)
      kbo(:, 5,39, 1) = (/ &
     &1.8162e-06_rb,4.5691e-06_rb,4.9354e-06_rb,4.2944e-06_rb,1.0896e-06_rb/)
      kbo(:, 1,40, 1) = (/ &
     &4.1271e-07_rb,1.0842e-06_rb,1.1736e-06_rb,1.0777e-06_rb,4.1787e-07_rb/)
      kbo(:, 2,40, 1) = (/ &
     &5.4389e-07_rb,1.4709e-06_rb,1.5926e-06_rb,1.4421e-06_rb,5.1017e-07_rb/)
      kbo(:, 3,40, 1) = (/ &
     &7.4188e-07_rb,1.9982e-06_rb,2.1561e-06_rb,1.9284e-06_rb,6.1575e-07_rb/)
      kbo(:, 4,40, 1) = (/ &
     &1.0242e-06_rb,2.6854e-06_rb,2.8924e-06_rb,2.5529e-06_rb,7.2613e-07_rb/)
      kbo(:, 5,40, 1) = (/ &
     &1.4067e-06_rb,3.5476e-06_rb,3.8249e-06_rb,3.3331e-06_rb,8.4610e-07_rb/)
      kbo(:, 1,41, 1) = (/ &
     &3.2432e-07_rb,8.3392e-07_rb,9.0025e-07_rb,8.2659e-07_rb,3.1699e-07_rb/)
      kbo(:, 2,41, 1) = (/ &
     &4.2215e-07_rb,1.1277e-06_rb,1.2186e-06_rb,1.1049e-06_rb,3.8959e-07_rb/)
      kbo(:, 3,41, 1) = (/ &
     &5.7203e-07_rb,1.5335e-06_rb,1.6530e-06_rb,1.4808e-06_rb,4.7229e-07_rb/)
      kbo(:, 4,41, 1) = (/ &
     &7.8892e-07_rb,2.0682e-06_rb,2.2251e-06_rb,1.9663e-06_rb,5.6019e-07_rb/)
      kbo(:, 5,41, 1) = (/ &
     &1.0862e-06_rb,2.7447e-06_rb,2.9535e-06_rb,2.5776e-06_rb,6.5591e-07_rb/)
      kbo(:, 1,42, 1) = (/ &
     &2.5587e-07_rb,6.4253e-07_rb,6.9167e-07_rb,6.3420e-07_rb,2.3952e-07_rb/)
      kbo(:, 2,42, 1) = (/ &
     &3.2863e-07_rb,8.6497e-07_rb,9.3264e-07_rb,8.4649e-07_rb,2.9775e-07_rb/)
      kbo(:, 3,42, 1) = (/ &
     &4.4164e-07_rb,1.1761e-06_rb,1.2667e-06_rb,1.1361e-06_rb,3.6120e-07_rb/)
      kbo(:, 4,42, 1) = (/ &
     &6.0774e-07_rb,1.5914e-06_rb,1.7100e-06_rb,1.5131e-06_rb,4.3157e-07_rb/)
      kbo(:, 5,42, 1) = (/ &
     &8.3802e-07_rb,2.1211e-06_rb,2.2785e-06_rb,1.9912e-06_rb,5.0721e-07_rb/)
      kbo(:, 1,43, 1) = (/ &
     &2.0233e-07_rb,4.9120e-07_rb,5.2639e-07_rb,4.8089e-07_rb,1.7965e-07_rb/)
      kbo(:, 2,43, 1) = (/ &
     &2.5538e-07_rb,6.5692e-07_rb,7.0547e-07_rb,6.4075e-07_rb,2.2496e-07_rb/)
      kbo(:, 3,43, 1) = (/ &
     &3.3917e-07_rb,8.9179e-07_rb,9.5869e-07_rb,8.5986e-07_rb,2.7469e-07_rb/)
      kbo(:, 4,43, 1) = (/ &
     &4.6446e-07_rb,1.2101e-06_rb,1.2978e-06_rb,1.1498e-06_rb,3.3044e-07_rb/)
      kbo(:, 5,43, 1) = (/ &
     &6.4112e-07_rb,1.6212e-06_rb,1.7373e-06_rb,1.5201e-06_rb,3.9028e-07_rb/)
      kbo(:, 1,44, 1) = (/ &
     &1.6122e-07_rb,3.7551e-07_rb,3.9960e-07_rb,3.6377e-07_rb,1.3351e-07_rb/)
      kbo(:, 2,44, 1) = (/ &
     &1.9938e-07_rb,4.9773e-07_rb,5.3195e-07_rb,4.8258e-07_rb,1.6857e-07_rb/)
      kbo(:, 3,44, 1) = (/ &
     &2.6075e-07_rb,6.7334e-07_rb,7.2162e-07_rb,6.4719e-07_rb,2.0768e-07_rb/)
      kbo(:, 4,44, 1) = (/ &
     &3.5440e-07_rb,9.1528e-07_rb,9.7926e-07_rb,8.6831e-07_rb,2.5128e-07_rb/)
      kbo(:, 5,44, 1) = (/ &
     &4.8882e-07_rb,1.2321e-06_rb,1.3171e-06_rb,1.1536e-06_rb,2.9843e-07_rb/)
      kbo(:, 1,45, 1) = (/ &
     &1.2988e-07_rb,2.8870e-07_rb,3.0532e-07_rb,2.7582e-07_rb,9.8684e-08_rb/)
      kbo(:, 2,45, 1) = (/ &
     &1.5689e-07_rb,3.7846e-07_rb,4.0235e-07_rb,3.6402e-07_rb,1.2561e-07_rb/)
      kbo(:, 3,45, 1) = (/ &
     &2.0165e-07_rb,5.0920e-07_rb,5.4343e-07_rb,4.8733e-07_rb,1.5620e-07_rb/)
      kbo(:, 4,45, 1) = (/ &
     &2.7120e-07_rb,6.9216e-07_rb,7.3881e-07_rb,6.5533e-07_rb,1.9009e-07_rb/)
      kbo(:, 5,45, 1) = (/ &
     &3.7310e-07_rb,9.3568e-07_rb,9.9762e-07_rb,8.7425e-07_rb,2.2785e-07_rb/)
      kbo(:, 1,46, 1) = (/ &
     &1.0564e-07_rb,2.2197e-07_rb,2.3290e-07_rb,2.0875e-07_rb,7.2454e-08_rb/)
      kbo(:, 2,46, 1) = (/ &
     &1.2404e-07_rb,2.8745e-07_rb,3.0379e-07_rb,2.7335e-07_rb,9.2971e-08_rb/)
      kbo(:, 3,46, 1) = (/ &
     &1.5624e-07_rb,3.8369e-07_rb,4.0737e-07_rb,3.6500e-07_rb,1.1671e-07_rb/)
      kbo(:, 4,46, 1) = (/ &
     &2.0727e-07_rb,5.2051e-07_rb,5.5407e-07_rb,4.9156e-07_rb,1.4314e-07_rb/)
      kbo(:, 5,46, 1) = (/ &
     &2.8357e-07_rb,7.0586e-07_rb,7.5047e-07_rb,6.5798e-07_rb,1.7235e-07_rb/)
      kbo(:, 1,47, 1) = (/ &
     &8.6916e-08_rb,1.7066e-07_rb,1.7731e-07_rb,1.5740e-07_rb,5.2956e-08_rb/)
      kbo(:, 2,47, 1) = (/ &
     &9.8603e-08_rb,2.1737e-07_rb,2.2800e-07_rb,2.0365e-07_rb,6.8355e-08_rb/)
      kbo(:, 3,47, 1) = (/ &
     &1.2114e-07_rb,2.8693e-07_rb,3.0257e-07_rb,2.7060e-07_rb,8.6098e-08_rb/)
      kbo(:, 4,47, 1) = (/ &
     &1.5770e-07_rb,3.8735e-07_rb,4.1075e-07_rb,3.6401e-07_rb,1.0681e-07_rb/)
      kbo(:, 5,47, 1) = (/ &
     &2.1366e-07_rb,5.2607e-07_rb,5.5766e-07_rb,4.8899e-07_rb,1.2936e-07_rb/)
      kbo(:, 1,48, 1) = (/ &
     &7.2690e-08_rb,1.3243e-07_rb,1.3635e-07_rb,1.1959e-07_rb,3.9422e-08_rb/)
      kbo(:, 2,48, 1) = (/ &
     &7.9634e-08_rb,1.6556e-07_rb,1.7235e-07_rb,1.5270e-07_rb,4.9882e-08_rb/)
      kbo(:, 3,48, 1) = (/ &
     &9.5002e-08_rb,2.1570e-07_rb,2.2592e-07_rb,2.0102e-07_rb,6.3148e-08_rb/)
      kbo(:, 4,48, 1) = (/ &
     &1.2105e-07_rb,2.8907e-07_rb,3.0482e-07_rb,2.6976e-07_rb,7.9242e-08_rb/)
      kbo(:, 5,48, 1) = (/ &
     &1.6180e-07_rb,3.9224e-07_rb,4.1437e-07_rb,3.6322e-07_rb,9.6936e-08_rb/)
      kbo(:, 1,49, 1) = (/ &
     &6.1774e-08_rb,1.0401e-07_rb,1.0607e-07_rb,9.1394e-08_rb,2.9238e-08_rb/)
      kbo(:, 2,49, 1) = (/ &
     &6.5366e-08_rb,1.2732e-07_rb,1.3151e-07_rb,1.1526e-07_rb,3.6075e-08_rb/)
      kbo(:, 3,49, 1) = (/ &
     &7.5416e-08_rb,1.6321e-07_rb,1.6967e-07_rb,1.4985e-07_rb,4.6286e-08_rb/)
      kbo(:, 4,49, 1) = (/ &
     &9.3863e-08_rb,2.1656e-07_rb,2.2682e-07_rb,2.0018e-07_rb,5.8261e-08_rb/)
      kbo(:, 5,49, 1) = (/ &
     &1.2334e-07_rb,2.9285e-07_rb,3.0807e-07_rb,2.6986e-07_rb,7.1843e-08_rb/)
      kbo(:, 1,50, 1) = (/ &
     &5.3206e-08_rb,8.3103e-08_rb,8.3626e-08_rb,7.1042e-08_rb,2.0418e-08_rb/)
      kbo(:, 2,50, 1) = (/ &
     &5.4599e-08_rb,9.9314e-08_rb,1.0171e-07_rb,8.8074e-08_rb,2.6669e-08_rb/)
      kbo(:, 3,50, 1) = (/ &
     &6.0981e-08_rb,1.2515e-07_rb,1.2917e-07_rb,1.1305e-07_rb,3.4171e-08_rb/)
      kbo(:, 4,50, 1) = (/ &
     &7.3992e-08_rb,1.6416e-07_rb,1.7066e-07_rb,1.4995e-07_rb,4.3100e-08_rb/)
      kbo(:, 5,50, 1) = (/ &
     &9.5401e-08_rb,2.2074e-07_rb,2.3106e-07_rb,2.0200e-07_rb,5.3679e-08_rb/)
      kbo(:, 1,51, 1) = (/ &
     &4.6282e-08_rb,6.7151e-08_rb,6.6436e-08_rb,5.6117e-08_rb,1.4310e-08_rb/)
      kbo(:, 2,51, 1) = (/ &
     &4.6293e-08_rb,7.8464e-08_rb,7.9611e-08_rb,6.7952e-08_rb,2.0313e-08_rb/)
      kbo(:, 3,51, 1) = (/ &
     &5.0084e-08_rb,9.7036e-08_rb,9.9288e-08_rb,8.6023e-08_rb,2.5137e-08_rb/)
      kbo(:, 4,51, 1) = (/ &
     &5.9071e-08_rb,1.2553e-07_rb,1.2948e-07_rb,1.1300e-07_rb,3.2036e-08_rb/)
      kbo(:, 5,51, 1) = (/ &
     &7.4640e-08_rb,1.6747e-07_rb,1.7423e-07_rb,1.5170e-07_rb,4.0217e-08_rb/)
      kbo(:, 1,52, 1) = (/ &
     &4.0629e-08_rb,5.4875e-08_rb,5.3346e-08_rb,4.4940e-08_rb,9.9095e-09_rb/)
      kbo(:, 2,52, 1) = (/ &
     &3.9810e-08_rb,6.2753e-08_rb,6.2779e-08_rb,5.3045e-08_rb,1.4355e-08_rb/)
      kbo(:, 3,52, 1) = (/ &
     &4.1749e-08_rb,7.5935e-08_rb,7.6945e-08_rb,6.5932e-08_rb,1.8547e-08_rb/)
      kbo(:, 4,52, 1) = (/ &
     &4.7734e-08_rb,9.6672e-08_rb,9.8927e-08_rb,8.5559e-08_rb,2.3788e-08_rb/)
      kbo(:, 5,52, 1) = (/ &
     &5.9002e-08_rb,1.2763e-07_rb,1.3186e-07_rb,1.1423e-07_rb,2.9932e-08_rb/)
      kbo(:, 1,53, 1) = (/ &
     &3.5975e-08_rb,4.5399e-08_rb,4.2771e-08_rb,3.6200e-08_rb,6.7635e-09_rb/)
      kbo(:, 2,53, 1) = (/ &
     &3.4672e-08_rb,5.0776e-08_rb,4.9763e-08_rb,4.1778e-08_rb,1.0001e-08_rb/)
      kbo(:, 3,53, 1) = (/ &
     &3.5325e-08_rb,6.0020e-08_rb,6.0156e-08_rb,5.0998e-08_rb,1.3726e-08_rb/)
      kbo(:, 4,53, 1) = (/ &
     &3.9133e-08_rb,7.5038e-08_rb,7.6143e-08_rb,6.5063e-08_rb,1.7480e-08_rb/)
      kbo(:, 5,53, 1) = (/ &
     &4.7140e-08_rb,9.7792e-08_rb,1.0026e-07_rb,8.6276e-08_rb,2.2211e-08_rb/)
      kbo(:, 1,54, 1) = (/ &
     &3.2026e-08_rb,3.8173e-08_rb,3.4913e-08_rb,2.9554e-08_rb,4.7895e-09_rb/)
      kbo(:, 2,54, 1) = (/ &
     &3.0459e-08_rb,4.1712e-08_rb,4.0172e-08_rb,3.3417e-08_rb,7.2202e-09_rb/)
      kbo(:, 3,54, 1) = (/ &
     &3.0344e-08_rb,4.8248e-08_rb,4.7728e-08_rb,3.9999e-08_rb,1.0387e-08_rb/)
      kbo(:, 4,54, 1) = (/ &
     &3.2650e-08_rb,5.9169e-08_rb,5.9453e-08_rb,5.0241e-08_rb,1.3202e-08_rb/)
      kbo(:, 5,54, 1) = (/ &
     &3.8298e-08_rb,7.6040e-08_rb,7.7304e-08_rb,6.5975e-08_rb,1.6815e-08_rb/)
      kbo(:, 1,55, 1) = (/ &
     &2.8677e-08_rb,3.2441e-08_rb,2.8748e-08_rb,2.4151e-08_rb,3.5459e-09_rb/)
      kbo(:, 2,55, 1) = (/ &
     &2.6981e-08_rb,3.4720e-08_rb,3.2391e-08_rb,2.6974e-08_rb,5.3560e-09_rb/)
      kbo(:, 3,55, 1) = (/ &
     &2.6409e-08_rb,3.9355e-08_rb,3.8167e-08_rb,3.1739e-08_rb,7.7448e-09_rb/)
      kbo(:, 4,55, 1) = (/ &
     &2.7649e-08_rb,4.7215e-08_rb,4.6970e-08_rb,3.9144e-08_rb,1.0056e-08_rb/)
      kbo(:, 5,55, 1) = (/ &
     &3.1553e-08_rb,5.9776e-08_rb,6.0219e-08_rb,5.0851e-08_rb,1.2897e-08_rb/)
      kbo(:, 1,56, 1) = (/ &
     &2.5871e-08_rb,2.7896e-08_rb,2.4025e-08_rb,1.9941e-08_rb,2.5789e-09_rb/)
      kbo(:, 2,56, 1) = (/ &
     &2.4111e-08_rb,2.9230e-08_rb,2.6562e-08_rb,2.2012e-08_rb,3.9211e-09_rb/)
      kbo(:, 3,56, 1) = (/ &
     &2.3260e-08_rb,3.2443e-08_rb,3.0891e-08_rb,2.5392e-08_rb,5.7991e-09_rb/)
      kbo(:, 4,56, 1) = (/ &
     &2.3744e-08_rb,3.8105e-08_rb,3.7441e-08_rb,3.0750e-08_rb,7.7370e-09_rb/)
      kbo(:, 5,56, 1) = (/ &
     &2.6357e-08_rb,4.7419e-08_rb,4.7264e-08_rb,3.9435e-08_rb,9.8948e-09_rb/)
      kbo(:, 1,57, 1) = (/ &
     &2.3535e-08_rb,2.4259e-08_rb,2.0309e-08_rb,1.6435e-08_rb,1.8424e-09_rb/)
      kbo(:, 2,57, 1) = (/ &
     &2.1750e-08_rb,2.4981e-08_rb,2.1975e-08_rb,1.7965e-08_rb,2.8485e-09_rb/)
      kbo(:, 3,57, 1) = (/ &
     &2.0716e-08_rb,2.7095e-08_rb,2.5073e-08_rb,2.0518e-08_rb,4.3296e-09_rb/)
      kbo(:, 4,57, 1) = (/ &
     &2.0688e-08_rb,3.1150e-08_rb,2.9998e-08_rb,2.4389e-08_rb,6.0949e-09_rb/)
      kbo(:, 5,57, 1) = (/ &
     &2.2336e-08_rb,3.7957e-08_rb,3.7415e-08_rb,3.0783e-08_rb,7.5649e-09_rb/)
      kbo(:, 1,58, 1) = (/ &
     &1.0425e-08_rb,1.1830e-08_rb,1.1340e-08_rb,1.0982e-08_rb,1.3443e-09_rb/)
      kbo(:, 2,58, 1) = (/ &
     &9.5648e-09_rb,1.2002e-08_rb,1.2045e-08_rb,1.1710e-08_rb,2.0849e-09_rb/)
      kbo(:, 3,58, 1) = (/ &
     &9.0146e-09_rb,1.2747e-08_rb,1.3512e-08_rb,1.3201e-08_rb,3.2654e-09_rb/)
      kbo(:, 4,58, 1) = (/ &
     &8.8493e-09_rb,1.4369e-08_rb,1.5929e-08_rb,1.5466e-08_rb,4.7356e-09_rb/)
      kbo(:, 5,58, 1) = (/ &
     &9.3086e-09_rb,1.7138e-08_rb,1.9573e-08_rb,1.9214e-08_rb,5.8278e-09_rb/)
      kbo(:, 1,59, 1) = (/ &
     &9.0380e-09_rb,1.0080e-08_rb,9.6059e-09_rb,9.1584e-09_rb,1.1081e-09_rb/)
      kbo(:, 2,59, 1) = (/ &
     &8.2691e-09_rb,1.0165e-08_rb,1.0115e-08_rb,9.7014e-09_rb,1.7533e-09_rb/)
      kbo(:, 3,59, 1) = (/ &
     &7.7641e-09_rb,1.0688e-08_rb,1.1200e-08_rb,1.0889e-08_rb,2.7394e-09_rb/)
      kbo(:, 4,59, 1) = (/ &
     &7.5756e-09_rb,1.1927e-08_rb,1.3083e-08_rb,1.2667e-08_rb,3.9313e-09_rb/)
      kbo(:, 5,59, 1) = (/ &
     &7.8886e-09_rb,1.4088e-08_rb,1.5992e-08_rb,1.5688e-08_rb,4.8202e-09_rb/)
      kbo(:, 1,13, 2) = (/ &
     &2.3347e-04_rb,3.2740e-04_rb,3.1964e-04_rb,2.7562e-04_rb,2.3674e-04_rb/)
      kbo(:, 2,13, 2) = (/ &
     &2.9027e-04_rb,4.3170e-04_rb,4.3482e-04_rb,3.8943e-04_rb,3.2365e-04_rb/)
      kbo(:, 3,13, 2) = (/ &
     &3.9534e-04_rb,6.0684e-04_rb,6.1602e-04_rb,5.5281e-04_rb,4.2550e-04_rb/)
      kbo(:, 4,13, 2) = (/ &
     &5.7501e-04_rb,8.8229e-04_rb,8.9214e-04_rb,7.8504e-04_rb,5.3481e-04_rb/)
      kbo(:, 5,13, 2) = (/ &
     &8.7274e-04_rb,1.2992e-03_rb,1.2957e-03_rb,1.1072e-03_rb,6.5845e-04_rb/)
      kbo(:, 1,14, 2) = (/ &
     &2.0019e-04_rb,2.7999e-04_rb,2.7248e-04_rb,2.3561e-04_rb,2.0892e-04_rb/)
      kbo(:, 2,14, 2) = (/ &
     &2.5252e-04_rb,3.7448e-04_rb,3.7546e-04_rb,3.3478e-04_rb,2.8574e-04_rb/)
      kbo(:, 3,14, 2) = (/ &
     &3.4599e-04_rb,5.3246e-04_rb,5.3788e-04_rb,4.7751e-04_rb,3.6978e-04_rb/)
      kbo(:, 4,14, 2) = (/ &
     &5.0873e-04_rb,7.7992e-04_rb,7.8607e-04_rb,6.8145e-04_rb,4.6369e-04_rb/)
      kbo(:, 5,14, 2) = (/ &
     &7.7937e-04_rb,1.1544e-03_rb,1.1464e-03_rb,9.6300e-04_rb,5.7084e-04_rb/)
      kbo(:, 1,15, 2) = (/ &
     &1.6496e-04_rb,2.3313e-04_rb,2.2745e-04_rb,1.9815e-04_rb,1.7191e-04_rb/)
      kbo(:, 2,15, 2) = (/ &
     &2.0995e-04_rb,3.1642e-04_rb,3.1725e-04_rb,2.8352e-04_rb,2.3300e-04_rb/)
      kbo(:, 3,15, 2) = (/ &
     &2.9058e-04_rb,4.5406e-04_rb,4.6090e-04_rb,4.0582e-04_rb,3.0079e-04_rb/)
      kbo(:, 4,15, 2) = (/ &
     &4.3180e-04_rb,6.6786e-04_rb,6.7439e-04_rb,5.7938e-04_rb,3.7802e-04_rb/)
      kbo(:, 5,15, 2) = (/ &
     &6.6674e-04_rb,9.9697e-04_rb,9.8451e-04_rb,8.2076e-04_rb,4.7099e-04_rb/)
      kbo(:, 1,16, 2) = (/ &
     &1.3273e-04_rb,1.9160e-04_rb,1.8863e-04_rb,1.6534e-04_rb,1.4034e-04_rb/)
      kbo(:, 2,16, 2) = (/ &
     &1.7022e-04_rb,2.6274e-04_rb,2.6590e-04_rb,2.3774e-04_rb,1.8655e-04_rb/)
      kbo(:, 3,16, 2) = (/ &
     &2.3798e-04_rb,3.7894e-04_rb,3.8748e-04_rb,3.4032e-04_rb,2.3998e-04_rb/)
      kbo(:, 4,16, 2) = (/ &
     &3.5812e-04_rb,5.6381e-04_rb,5.6807e-04_rb,4.8599e-04_rb,3.0617e-04_rb/)
      kbo(:, 5,16, 2) = (/ &
     &5.5540e-04_rb,8.4478e-04_rb,8.3109e-04_rb,6.8770e-04_rb,3.7973e-04_rb/)
      kbo(:, 1,17, 2) = (/ &
     &1.0577e-04_rb,1.5680e-04_rb,1.5536e-04_rb,1.3786e-04_rb,1.1416e-04_rb/)
      kbo(:, 2,17, 2) = (/ &
     &1.3659e-04_rb,2.1658e-04_rb,2.2147e-04_rb,1.9766e-04_rb,1.5136e-04_rb/)
      kbo(:, 3,17, 2) = (/ &
     &1.9353e-04_rb,3.1591e-04_rb,3.2447e-04_rb,2.8337e-04_rb,1.9661e-04_rb/)
      kbo(:, 4,17, 2) = (/ &
     &2.9393e-04_rb,4.7241e-04_rb,4.7609e-04_rb,4.0460e-04_rb,2.4887e-04_rb/)
      kbo(:, 5,17, 2) = (/ &
     &4.5598e-04_rb,7.0740e-04_rb,6.9589e-04_rb,5.7452e-04_rb,3.0304e-04_rb/)
      kbo(:, 1,18, 2) = (/ &
     &8.2750e-05_rb,1.2686e-04_rb,1.2742e-04_rb,1.1403e-04_rb,9.3961e-05_rb/)
      kbo(:, 2,18, 2) = (/ &
     &1.0793e-04_rb,1.7707e-04_rb,1.8353e-04_rb,1.6336e-04_rb,1.2524e-04_rb/)
      kbo(:, 3,18, 2) = (/ &
     &1.5481e-04_rb,2.6078e-04_rb,2.6950e-04_rb,2.3444e-04_rb,1.6211e-04_rb/)
      kbo(:, 4,18, 2) = (/ &
     &2.3641e-04_rb,3.9024e-04_rb,3.9536e-04_rb,3.3586e-04_rb,2.0255e-04_rb/)
      kbo(:, 5,18, 2) = (/ &
     &3.6699e-04_rb,5.8299e-04_rb,5.7531e-04_rb,4.7694e-04_rb,2.4459e-04_rb/)
      kbo(:, 1,19, 2) = (/ &
     &6.5789e-05_rb,1.0377e-04_rb,1.0573e-04_rb,9.4747e-05_rb,7.8309e-05_rb/)
      kbo(:, 2,19, 2) = (/ &
     &8.6739e-05_rb,1.4667e-04_rb,1.5321e-04_rb,1.3630e-04_rb,1.0400e-04_rb/)
      kbo(:, 3,19, 2) = (/ &
     &1.2570e-04_rb,2.1696e-04_rb,2.2493e-04_rb,1.9585e-04_rb,1.3355e-04_rb/)
      kbo(:, 4,19, 2) = (/ &
     &1.9254e-04_rb,3.2492e-04_rb,3.2972e-04_rb,2.8110e-04_rb,1.6478e-04_rb/)
      kbo(:, 5,19, 2) = (/ &
     &2.9952e-04_rb,4.8402e-04_rb,4.8010e-04_rb,3.9892e-04_rb,1.9831e-04_rb/)
      kbo(:, 1,20, 2) = (/ &
     &5.3588e-05_rb,8.6632e-05_rb,8.9247e-05_rb,8.0067e-05_rb,6.5775e-05_rb/)
      kbo(:, 2,20, 2) = (/ &
     &7.1606e-05_rb,1.2377e-04_rb,1.3011e-04_rb,1.1561e-04_rb,8.7374e-05_rb/)
      kbo(:, 3,20, 2) = (/ &
     &1.0490e-04_rb,1.8400e-04_rb,1.9125e-04_rb,1.6630e-04_rb,1.1025e-04_rb/)
      kbo(:, 4,20, 2) = (/ &
     &1.6107e-04_rb,2.7570e-04_rb,2.7972e-04_rb,2.3864e-04_rb,1.3590e-04_rb/)
      kbo(:, 5,20, 2) = (/ &
     &2.5044e-04_rb,4.0870e-04_rb,4.0643e-04_rb,3.3832e-04_rb,1.6243e-04_rb/)
      kbo(:, 1,21, 2) = (/ &
     &4.3995e-05_rb,7.2776e-05_rb,7.5695e-05_rb,6.8050e-05_rb,5.5495e-05_rb/)
      kbo(:, 2,21, 2) = (/ &
     &5.9611e-05_rb,1.0482e-04_rb,1.1056e-04_rb,9.8202e-05_rb,7.3158e-05_rb/)
      kbo(:, 3,21, 2) = (/ &
     &8.7989e-05_rb,1.5634e-04_rb,1.6268e-04_rb,1.4172e-04_rb,9.0987e-05_rb/)
      kbo(:, 4,21, 2) = (/ &
     &1.3548e-04_rb,2.3394e-04_rb,2.3798e-04_rb,2.0295e-04_rb,1.1189e-04_rb/)
      kbo(:, 5,21, 2) = (/ &
     &2.1032e-04_rb,3.4525e-04_rb,3.4399e-04_rb,2.8683e-04_rb,1.3326e-04_rb/)
      kbo(:, 1,22, 2) = (/ &
     &3.6803e-05_rb,6.2518e-05_rb,6.5746e-05_rb,5.8989e-05_rb,4.7547e-05_rb/)
      kbo(:, 2,22, 2) = (/ &
     &5.0761e-05_rb,9.1003e-05_rb,9.6193e-05_rb,8.5393e-05_rb,6.0785e-05_rb/)
      kbo(:, 3,22, 2) = (/ &
     &7.5680e-05_rb,1.3601e-04_rb,1.4124e-04_rb,1.2316e-04_rb,7.5929e-05_rb/)
      kbo(:, 4,22, 2) = (/ &
     &1.1711e-04_rb,2.0303e-04_rb,2.0625e-04_rb,1.7599e-04_rb,9.2008e-05_rb/)
      kbo(:, 5,22, 2) = (/ &
     &1.8108e-04_rb,2.9768e-04_rb,2.9683e-04_rb,2.4768e-04_rb,1.0998e-04_rb/)
      kbo(:, 1,23, 2) = (/ &
     &3.1003e-05_rb,5.3823e-05_rb,5.7061e-05_rb,5.1271e-05_rb,4.0280e-05_rb/)
      kbo(:, 2,23, 2) = (/ &
     &4.3496e-05_rb,7.9103e-05_rb,8.3635e-05_rb,7.4369e-05_rb,5.0624e-05_rb/)
      kbo(:, 3,23, 2) = (/ &
     &6.5401e-05_rb,1.1830e-04_rb,1.2269e-04_rb,1.0707e-04_rb,6.2127e-05_rb/)
      kbo(:, 4,23, 2) = (/ &
     &1.0147e-04_rb,1.7576e-04_rb,1.7852e-04_rb,1.5248e-04_rb,7.5578e-05_rb/)
      kbo(:, 5,23, 2) = (/ &
     &1.5625e-04_rb,2.5690e-04_rb,2.5634e-04_rb,2.1357e-04_rb,9.0733e-05_rb/)
      kbo(:, 1,24, 2) = (/ &
     &2.6471e-05_rb,4.6863e-05_rb,4.9818e-05_rb,4.4807e-05_rb,3.2886e-05_rb/)
      kbo(:, 2,24, 2) = (/ &
     &3.7731e-05_rb,6.9280e-05_rb,7.2915e-05_rb,6.4892e-05_rb,4.1198e-05_rb/)
      kbo(:, 3,24, 2) = (/ &
     &5.7248e-05_rb,1.0347e-04_rb,1.0698e-04_rb,9.3403e-05_rb,5.0808e-05_rb/)
      kbo(:, 4,24, 2) = (/ &
     &8.8791e-05_rb,1.5319e-04_rb,1.5530e-04_rb,1.3247e-04_rb,6.1812e-05_rb/)
      kbo(:, 5,24, 2) = (/ &
     &1.3624e-04_rb,2.2303e-04_rb,2.2220e-04_rb,1.8474e-04_rb,7.4324e-05_rb/)
      kbo(:, 1,25, 2) = (/ &
     &2.2808e-05_rb,4.1048e-05_rb,4.3589e-05_rb,3.9316e-05_rb,2.6891e-05_rb/)
      kbo(:, 2,25, 2) = (/ &
     &3.2985e-05_rb,6.0936e-05_rb,6.3907e-05_rb,5.6898e-05_rb,3.3888e-05_rb/)
      kbo(:, 3,25, 2) = (/ &
     &5.0481e-05_rb,9.0713e-05_rb,9.3687e-05_rb,8.1693e-05_rb,4.1615e-05_rb/)
      kbo(:, 4,25, 2) = (/ &
     &7.8151e-05_rb,1.3406e-04_rb,1.3563e-04_rb,1.1547e-04_rb,5.0834e-05_rb/)
      kbo(:, 5,25, 2) = (/ &
     &1.1946e-04_rb,1.9452e-04_rb,1.9337e-04_rb,1.6037e-04_rb,6.0346e-05_rb/)
      kbo(:, 1,26, 2) = (/ &
     &1.9934e-05_rb,3.6349e-05_rb,3.8562e-05_rb,3.4809e-05_rb,2.2369e-05_rb/)
      kbo(:, 2,26, 2) = (/ &
     &2.9291e-05_rb,5.4051e-05_rb,5.6667e-05_rb,5.0301e-05_rb,2.7895e-05_rb/)
      kbo(:, 3,26, 2) = (/ &
     &4.5044e-05_rb,8.0419e-05_rb,8.2812e-05_rb,7.2007e-05_rb,3.4611e-05_rb/)
      kbo(:, 4,26, 2) = (/ &
     &6.9643e-05_rb,1.1855e-04_rb,1.1958e-04_rb,1.0144e-04_rb,4.1836e-05_rb/)
      kbo(:, 5,26, 2) = (/ &
     &1.0589e-04_rb,1.7128e-04_rb,1.6969e-04_rb,1.4033e-04_rb,4.9219e-05_rb/)
      kbo(:, 1,27, 2) = (/ &
     &1.7559e-05_rb,3.2313e-05_rb,3.4216e-05_rb,3.0835e-05_rb,1.8255e-05_rb/)
      kbo(:, 2,27, 2) = (/ &
     &2.6182e-05_rb,4.8079e-05_rb,5.0267e-05_rb,4.4498e-05_rb,2.3042e-05_rb/)
      kbo(:, 3,27, 2) = (/ &
     &4.0350e-05_rb,7.1540e-05_rb,7.3359e-05_rb,6.3538e-05_rb,2.8359e-05_rb/)
      kbo(:, 4,27, 2) = (/ &
     &6.2227e-05_rb,1.0500e-04_rb,1.0550e-04_rb,8.9231e-05_rb,3.3804e-05_rb/)
      kbo(:, 5,27, 2) = (/ &
     &9.4129e-05_rb,1.5114e-04_rb,1.4903e-04_rb,1.2285e-04_rb,4.0082e-05_rb/)
      kbo(:, 1,28, 2) = (/ &
     &1.5570e-05_rb,2.8729e-05_rb,3.0391e-05_rb,2.7331e-05_rb,1.4928e-05_rb/)
      kbo(:, 2,28, 2) = (/ &
     &2.3454e-05_rb,4.2868e-05_rb,4.4647e-05_rb,3.9416e-05_rb,1.8685e-05_rb/)
      kbo(:, 3,28, 2) = (/ &
     &3.6218e-05_rb,6.3627e-05_rb,6.4995e-05_rb,5.6106e-05_rb,2.2770e-05_rb/)
      kbo(:, 4,28, 2) = (/ &
     &5.5686e-05_rb,9.3113e-05_rb,9.3100e-05_rb,7.8497e-05_rb,2.7363e-05_rb/)
      kbo(:, 5,28, 2) = (/ &
     &8.3686e-05_rb,1.3339e-04_rb,1.3104e-04_rb,1.0755e-04_rb,3.2541e-05_rb/)
      kbo(:, 1,29, 2) = (/ &
     &1.3843e-05_rb,2.5623e-05_rb,2.7065e-05_rb,2.4295e-05_rb,1.2134e-05_rb/)
      kbo(:, 2,29, 2) = (/ &
     &2.1009e-05_rb,3.8248e-05_rb,3.9732e-05_rb,3.4977e-05_rb,1.5044e-05_rb/)
      kbo(:, 3,29, 2) = (/ &
     &3.2483e-05_rb,5.6676e-05_rb,5.7569e-05_rb,4.9592e-05_rb,1.8408e-05_rb/)
      kbo(:, 4,29, 2) = (/ &
     &4.9712e-05_rb,8.2574e-05_rb,8.2234e-05_rb,6.9025e-05_rb,2.2259e-05_rb/)
      kbo(:, 5,29, 2) = (/ &
     &7.4303e-05_rb,1.1753e-04_rb,1.1526e-04_rb,9.4159e-05_rb,2.6343e-05_rb/)
      kbo(:, 1,30, 2) = (/ &
     &1.2283e-05_rb,2.2826e-05_rb,2.4077e-05_rb,2.1586e-05_rb,9.7017e-06_rb/)
      kbo(:, 2,30, 2) = (/ &
     &1.8796e-05_rb,3.4070e-05_rb,3.5216e-05_rb,3.0990e-05_rb,1.2060e-05_rb/)
      kbo(:, 3,30, 2) = (/ &
     &2.9052e-05_rb,5.0361e-05_rb,5.0950e-05_rb,4.3716e-05_rb,1.4847e-05_rb/)
      kbo(:, 4,30, 2) = (/ &
     &4.4245e-05_rb,7.2929e-05_rb,7.2487e-05_rb,6.0571e-05_rb,1.7868e-05_rb/)
      kbo(:, 5,30, 2) = (/ &
     &6.5686e-05_rb,1.0336e-04_rb,1.0108e-04_rb,8.2235e-05_rb,2.0877e-05_rb/)
      kbo(:, 1,31, 2) = (/ &
     &1.0864e-05_rb,2.0303e-05_rb,2.1341e-05_rb,1.9138e-05_rb,7.8312e-06_rb/)
      kbo(:, 2,31, 2) = (/ &
     &1.6718e-05_rb,3.0263e-05_rb,3.1180e-05_rb,2.7363e-05_rb,9.6909e-06_rb/)
      kbo(:, 3,31, 2) = (/ &
     &2.5784e-05_rb,4.4485e-05_rb,4.4959e-05_rb,3.8436e-05_rb,1.1703e-05_rb/)
      kbo(:, 4,31, 2) = (/ &
     &3.9035e-05_rb,6.4113e-05_rb,6.3593e-05_rb,5.2930e-05_rb,1.4083e-05_rb/)
      kbo(:, 5,31, 2) = (/ &
     &5.7600e-05_rb,9.0427e-05_rb,8.8190e-05_rb,7.1491e-05_rb,1.6696e-05_rb/)
      kbo(:, 1,32, 2) = (/ &
     &9.6861e-06_rb,1.8129e-05_rb,1.9009e-05_rb,1.7005e-05_rb,6.3389e-06_rb/)
      kbo(:, 2,32, 2) = (/ &
     &1.4957e-05_rb,2.6929e-05_rb,2.7692e-05_rb,2.4209e-05_rb,7.7689e-06_rb/)
      kbo(:, 3,32, 2) = (/ &
     &2.2989e-05_rb,3.9434e-05_rb,3.9744e-05_rb,3.3837e-05_rb,9.3738e-06_rb/)
      kbo(:, 4,32, 2) = (/ &
     &3.4606e-05_rb,5.6545e-05_rb,5.5926e-05_rb,4.6356e-05_rb,1.1253e-05_rb/)
      kbo(:, 5,32, 2) = (/ &
     &5.0712e-05_rb,7.9301e-05_rb,7.7059e-05_rb,6.2285e-05_rb,1.3276e-05_rb/)
      kbo(:, 1,33, 2) = (/ &
     &8.6881e-06_rb,1.6215e-05_rb,1.6965e-05_rb,1.5122e-05_rb,5.0839e-06_rb/)
      kbo(:, 2,33, 2) = (/ &
     &1.3426e-05_rb,2.4037e-05_rb,2.4628e-05_rb,2.1434e-05_rb,6.2318e-06_rb/)
      kbo(:, 3,33, 2) = (/ &
     &2.0550e-05_rb,3.5027e-05_rb,3.5160e-05_rb,2.9820e-05_rb,7.5123e-06_rb/)
      kbo(:, 4,33, 2) = (/ &
     &3.0762e-05_rb,4.9971e-05_rb,4.9236e-05_rb,4.0640e-05_rb,8.9286e-06_rb/)
      kbo(:, 5,33, 2) = (/ &
     &4.4719e-05_rb,6.9592e-05_rb,6.7432e-05_rb,5.4322e-05_rb,1.0569e-05_rb/)
      kbo(:, 1,34, 2) = (/ &
     &7.7791e-06_rb,1.4442e-05_rb,1.5046e-05_rb,1.3360e-05_rb,4.1600e-06_rb/)
      kbo(:, 2,34, 2) = (/ &
     &1.2019e-05_rb,2.1346e-05_rb,2.1771e-05_rb,1.8857e-05_rb,5.0353e-06_rb/)
      kbo(:, 3,34, 2) = (/ &
     &1.8324e-05_rb,3.0972e-05_rb,3.0949e-05_rb,2.6117e-05_rb,6.0447e-06_rb/)
      kbo(:, 4,34, 2) = (/ &
     &2.7261e-05_rb,4.3932e-05_rb,4.3157e-05_rb,3.5442e-05_rb,7.1905e-06_rb/)
      kbo(:, 5,34, 2) = (/ &
     &3.9385e-05_rb,6.0888e-05_rb,5.8795e-05_rb,4.7179e-05_rb,8.5133e-06_rb/)
      kbo(:, 1,35, 2) = (/ &
     &6.6949e-06_rb,1.2390e-05_rb,1.2873e-05_rb,1.1410e-05_rb,3.2837e-06_rb/)
      kbo(:, 2,35, 2) = (/ &
     &1.0342e-05_rb,1.8289e-05_rb,1.8599e-05_rb,1.6068e-05_rb,4.0072e-06_rb/)
      kbo(:, 3,35, 2) = (/ &
     &1.5729e-05_rb,2.6447e-05_rb,2.6382e-05_rb,2.2198e-05_rb,4.8111e-06_rb/)
      kbo(:, 4,35, 2) = (/ &
     &2.3326e-05_rb,3.7424e-05_rb,3.6694e-05_rb,3.0052e-05_rb,5.7503e-06_rb/)
      kbo(:, 5,35, 2) = (/ &
     &3.3582e-05_rb,5.1735e-05_rb,4.9852e-05_rb,3.9921e-05_rb,6.8317e-06_rb/)
      kbo(:, 1,36, 2) = (/ &
     &5.5116e-06_rb,1.0195e-05_rb,1.0585e-05_rb,9.3814e-06_rb,2.5556e-06_rb/)
      kbo(:, 2,36, 2) = (/ &
     &8.5137e-06_rb,1.5037e-05_rb,1.5296e-05_rb,1.3209e-05_rb,3.1325e-06_rb/)
      kbo(:, 3,36, 2) = (/ &
     &1.2948e-05_rb,2.1753e-05_rb,2.1694e-05_rb,1.8250e-05_rb,3.7680e-06_rb/)
      kbo(:, 4,36, 2) = (/ &
     &1.9198e-05_rb,3.0787e-05_rb,3.0171e-05_rb,2.4706e-05_rb,4.5193e-06_rb/)
      kbo(:, 5,36, 2) = (/ &
     &2.7624e-05_rb,4.2550e-05_rb,4.0975e-05_rb,3.2811e-05_rb,5.3463e-06_rb/)
      kbo(:, 1,37, 2) = (/ &
     &4.2943e-06_rb,7.9625e-06_rb,8.2825e-06_rb,7.3555e-06_rb,1.9862e-06_rb/)
      kbo(:, 2,37, 2) = (/ &
     &6.6379e-06_rb,1.1772e-05_rb,1.1998e-05_rb,1.0386e-05_rb,2.4388e-06_rb/)
      kbo(:, 3,37, 2) = (/ &
     &1.0124e-05_rb,1.7087e-05_rb,1.7068e-05_rb,1.4395e-05_rb,2.9448e-06_rb/)
      kbo(:, 4,37, 2) = (/ &
     &1.5070e-05_rb,2.4266e-05_rb,2.3812e-05_rb,1.9543e-05_rb,3.5287e-06_rb/)
      kbo(:, 5,37, 2) = (/ &
     &2.1767e-05_rb,3.3653e-05_rb,3.2450e-05_rb,2.6024e-05_rb,4.1879e-06_rb/)
      kbo(:, 1,38, 2) = (/ &
     &3.3357e-06_rb,6.1963e-06_rb,6.4603e-06_rb,5.7499e-06_rb,1.5402e-06_rb/)
      kbo(:, 2,38, 2) = (/ &
     &5.1578e-06_rb,9.1863e-06_rb,9.3819e-06_rb,8.1434e-06_rb,1.8945e-06_rb/)
      kbo(:, 3,38, 2) = (/ &
     &7.8901e-06_rb,1.3380e-05_rb,1.3389e-05_rb,1.1321e-05_rb,2.2920e-06_rb/)
      kbo(:, 4,38, 2) = (/ &
     &1.1791e-05_rb,1.9066e-05_rb,1.8739e-05_rb,1.5418e-05_rb,2.7521e-06_rb/)
      kbo(:, 5,38, 2) = (/ &
     &1.7101e-05_rb,2.6537e-05_rb,2.5629e-05_rb,2.0591e-05_rb,3.2788e-06_rb/)
      kbo(:, 1,39, 2) = (/ &
     &2.5925e-06_rb,4.8221e-06_rb,5.0377e-06_rb,4.4928e-06_rb,1.1896e-06_rb/)
      kbo(:, 2,39, 2) = (/ &
     &4.0080e-06_rb,7.1657e-06_rb,7.3339e-06_rb,6.3822e-06_rb,1.4636e-06_rb/)
      kbo(:, 3,39, 2) = (/ &
     &6.1480e-06_rb,1.0471e-05_rb,1.0498e-05_rb,8.8998e-06_rb,1.7757e-06_rb/)
      kbo(:, 4,39, 2) = (/ &
     &9.2224e-06_rb,1.4975e-05_rb,1.4740e-05_rb,1.2159e-05_rb,2.1407e-06_rb/)
      kbo(:, 5,39, 2) = (/ &
     &1.3430e-05_rb,2.0919e-05_rb,2.0232e-05_rb,1.6286e-05_rb,2.5517e-06_rb/)
      kbo(:, 1,40, 2) = (/ &
     &1.9503e-06_rb,3.6256e-06_rb,3.7983e-06_rb,3.3983e-06_rb,9.0970e-07_rb/)
      kbo(:, 2,40, 2) = (/ &
     &3.0096e-06_rb,5.4072e-06_rb,5.5509e-06_rb,4.8509e-06_rb,1.1295e-06_rb/)
      kbo(:, 3,40, 2) = (/ &
     &4.6341e-06_rb,7.9436e-06_rb,7.9881e-06_rb,6.7974e-06_rb,1.3749e-06_rb/)
      kbo(:, 4,40, 2) = (/ &
     &6.9959e-06_rb,1.1429e-05_rb,1.1278e-05_rb,9.3390e-06_rb,1.6560e-06_rb/)
      kbo(:, 5,40, 2) = (/ &
     &1.0262e-05_rb,1.6061e-05_rb,1.5575e-05_rb,1.2573e-05_rb,1.9765e-06_rb/)
      kbo(:, 1,41, 2) = (/ &
     &1.4645e-06_rb,2.7131e-06_rb,2.8491e-06_rb,2.5571e-06_rb,6.9451e-07_rb/)
      kbo(:, 2,41, 2) = (/ &
     &2.2502e-06_rb,4.0586e-06_rb,4.1799e-06_rb,3.6674e-06_rb,8.6660e-07_rb/)
      kbo(:, 3,41, 2) = (/ &
     &3.4750e-06_rb,5.9931e-06_rb,6.0465e-06_rb,5.1654e-06_rb,1.0602e-06_rb/)
      kbo(:, 4,41, 2) = (/ &
     &5.2792e-06_rb,8.6774e-06_rb,8.5850e-06_rb,7.1396e-06_rb,1.2820e-06_rb/)
      kbo(:, 5,41, 2) = (/ &
     &7.8024e-06_rb,1.2273e-05_rb,1.1934e-05_rb,9.6653e-06_rb,1.5281e-06_rb/)
      kbo(:, 1,42, 2) = (/ &
     &1.1030e-06_rb,2.0298e-06_rb,2.1354e-06_rb,1.9220e-06_rb,5.2769e-07_rb/)
      kbo(:, 2,42, 2) = (/ &
     &1.6830e-06_rb,3.0429e-06_rb,3.1437e-06_rb,2.7680e-06_rb,6.5992e-07_rb/)
      kbo(:, 3,42, 2) = (/ &
     &2.6031e-06_rb,4.5149e-06_rb,4.5688e-06_rb,3.9192e-06_rb,8.1465e-07_rb/)
      kbo(:, 4,42, 2) = (/ &
     &3.9772e-06_rb,6.5768e-06_rb,6.5252e-06_rb,5.4495e-06_rb,9.8750e-07_rb/)
      kbo(:, 5,42, 2) = (/ &
     &5.9212e-06_rb,9.3604e-06_rb,9.1253e-06_rb,7.4182e-06_rb,1.1776e-06_rb/)
      kbo(:, 1,43, 2) = (/ &
     &8.2428e-07_rb,1.4953e-06_rb,1.5737e-06_rb,1.4196e-06_rb,3.9479e-07_rb/)
      kbo(:, 2,43, 2) = (/ &
     &1.2422e-06_rb,2.2430e-06_rb,2.3243e-06_rb,2.0532e-06_rb,4.9846e-07_rb/)
      kbo(:, 3,43, 2) = (/ &
     &1.9204e-06_rb,3.3444e-06_rb,3.3959e-06_rb,2.9263e-06_rb,6.1792e-07_rb/)
      kbo(:, 4,43, 2) = (/ &
     &2.9515e-06_rb,4.9083e-06_rb,4.8847e-06_rb,4.0977e-06_rb,7.5196e-07_rb/)
      kbo(:, 5,43, 2) = (/ &
     &4.4328e-06_rb,7.0409e-06_rb,6.8831e-06_rb,5.6178e-06_rb,9.0060e-07_rb/)
      kbo(:, 1,44, 2) = (/ &
     &6.1791e-07_rb,1.0964e-06_rb,1.1526e-06_rb,1.0396e-06_rb,2.9276e-07_rb/)
      kbo(:, 2,44, 2) = (/ &
     &9.1458e-07_rb,1.6410e-06_rb,1.7044e-06_rb,1.5103e-06_rb,3.7305e-07_rb/)
      kbo(:, 3,44, 2) = (/ &
     &1.4081e-06_rb,2.4567e-06_rb,2.5035e-06_rb,2.1667e-06_rb,4.6414e-07_rb/)
      kbo(:, 4,44, 2) = (/ &
     &2.1742e-06_rb,3.6323e-06_rb,3.6267e-06_rb,3.0560e-06_rb,5.6825e-07_rb/)
      kbo(:, 5,44, 2) = (/ &
     &3.2939e-06_rb,5.2553e-06_rb,5.1516e-06_rb,4.2228e-06_rb,6.8470e-07_rb/)
      kbo(:, 1,45, 2) = (/ &
     &4.6818e-07_rb,8.0720e-07_rb,8.4515e-07_rb,7.6099e-07_rb,2.1643e-07_rb/)
      kbo(:, 2,45, 2) = (/ &
     &6.7728e-07_rb,1.2013e-06_rb,1.2493e-06_rb,1.1096e-06_rb,2.7840e-07_rb/)
      kbo(:, 3,45, 2) = (/ &
     &1.0340e-06_rb,1.8032e-06_rb,1.8431e-06_rb,1.6011e-06_rb,3.4809e-07_rb/)
      kbo(:, 4,45, 2) = (/ &
     &1.6004e-06_rb,2.6834e-06_rb,2.6875e-06_rb,2.2740e-06_rb,4.2767e-07_rb/)
      kbo(:, 5,45, 2) = (/ &
     &2.4431e-06_rb,3.9144e-06_rb,3.8469e-06_rb,3.1675e-06_rb,5.1847e-07_rb/)
      kbo(:, 1,46, 2) = (/ &
     &3.5715e-07_rb,5.9421e-07_rb,6.1734e-07_rb,5.5305e-07_rb,1.5800e-07_rb/)
      kbo(:, 2,46, 2) = (/ &
     &5.0187e-07_rb,8.7381e-07_rb,9.0848e-07_rb,8.0843e-07_rb,2.0563e-07_rb/)
      kbo(:, 3,46, 2) = (/ &
     &7.5515e-07_rb,1.3118e-06_rb,1.3445e-06_rb,1.1721e-06_rb,2.5967e-07_rb/)
      kbo(:, 4,46, 2) = (/ &
     &1.1677e-06_rb,1.9633e-06_rb,1.9726e-06_rb,1.6766e-06_rb,3.2108e-07_rb/)
      kbo(:, 5,46, 2) = (/ &
     &1.7950e-06_rb,2.8885e-06_rb,2.8468e-06_rb,2.3553e-06_rb,3.9011e-07_rb/)
      kbo(:, 1,47, 2) = (/ &
     &2.7398e-07_rb,4.3581e-07_rb,4.4704e-07_rb,3.9668e-07_rb,1.1323e-07_rb/)
      kbo(:, 2,47, 2) = (/ &
     &3.7103e-07_rb,6.2850e-07_rb,6.5163e-07_rb,5.8000e-07_rb,1.4990e-07_rb/)
      kbo(:, 3,47, 2) = (/ &
     &5.4577e-07_rb,9.3956e-07_rb,9.6531e-07_rb,8.4415e-07_rb,1.9262e-07_rb/)
      kbo(:, 4,47, 2) = (/ &
     &8.3895e-07_rb,1.4116e-06_rb,1.4241e-06_rb,1.2169e-06_rb,2.3951e-07_rb/)
      kbo(:, 5,47, 2) = (/ &
     &1.2964e-06_rb,2.0956e-06_rb,2.0729e-06_rb,1.7247e-06_rb,2.9208e-07_rb/)
      kbo(:, 1,48, 2) = (/ &
     &2.1387e-07_rb,3.2471e-07_rb,3.2631e-07_rb,2.8559e-07_rb,7.9995e-08_rb/)
      kbo(:, 2,48, 2) = (/ &
     &2.7873e-07_rb,4.5569e-07_rb,4.6902e-07_rb,4.1584e-07_rb,1.0856e-07_rb/)
      kbo(:, 3,48, 2) = (/ &
     &3.9810e-07_rb,6.7425e-07_rb,6.9287e-07_rb,6.0740e-07_rb,1.4073e-07_rb/)
      kbo(:, 4,48, 2) = (/ &
     &6.0448e-07_rb,1.0143e-06_rb,1.0265e-06_rb,8.8097e-07_rb,1.7712e-07_rb/)
      kbo(:, 5,48, 2) = (/ &
     &9.3542e-07_rb,1.5166e-06_rb,1.5052e-06_rb,1.2587e-06_rb,2.1758e-07_rb/)
      kbo(:, 1,49, 2) = (/ &
     &1.6983e-07_rb,2.4624e-07_rb,2.4047e-07_rb,2.0742e-07_rb,5.5874e-08_rb/)
      kbo(:, 2,49, 2) = (/ &
     &2.1316e-07_rb,3.3392e-07_rb,3.3939e-07_rb,2.9852e-07_rb,7.7373e-08_rb/)
      kbo(:, 3,49, 2) = (/ &
     &2.9400e-07_rb,4.8579e-07_rb,4.9802e-07_rb,4.3681e-07_rb,1.0173e-07_rb/)
      kbo(:, 4,49, 2) = (/ &
     &4.3777e-07_rb,7.2867e-07_rb,7.3912e-07_rb,6.3632e-07_rb,1.2966e-07_rb/)
      kbo(:, 5,49, 2) = (/ &
     &6.7522e-07_rb,1.0953e-06_rb,1.0905e-06_rb,9.1600e-07_rb,1.6042e-07_rb/)
      kbo(:, 1,50, 2) = (/ &
     &1.3748e-07_rb,1.9105e-07_rb,1.8074e-07_rb,1.5294e-07_rb,4.0734e-08_rb/)
      kbo(:, 2,50, 2) = (/ &
     &1.6685e-07_rb,2.5027e-07_rb,2.4971e-07_rb,2.1712e-07_rb,5.5541e-08_rb/)
      kbo(:, 3,50, 2) = (/ &
     &2.2205e-07_rb,3.5580e-07_rb,3.6241e-07_rb,3.1709e-07_rb,7.4088e-08_rb/)
      kbo(:, 4,50, 2) = (/ &
     &3.2265e-07_rb,5.2953e-07_rb,5.3723e-07_rb,4.6323e-07_rb,9.5830e-08_rb/)
      kbo(:, 5,50, 2) = (/ &
     &4.9355e-07_rb,7.9777e-07_rb,7.9617e-07_rb,6.7111e-07_rb,1.2000e-07_rb/)
      kbo(:, 1,51, 2) = (/ &
     &1.1310e-07_rb,1.5095e-07_rb,1.3873e-07_rb,1.1416e-07_rb,2.8930e-08_rb/)
      kbo(:, 2,51, 2) = (/ &
     &1.3297e-07_rb,1.9104e-07_rb,1.8603e-07_rb,1.5944e-07_rb,3.9649e-08_rb/)
      kbo(:, 3,51, 2) = (/ &
     &1.7089e-07_rb,2.6387e-07_rb,2.6609e-07_rb,2.3142e-07_rb,5.4118e-08_rb/)
      kbo(:, 4,51, 2) = (/ &
     &2.4121e-07_rb,3.8782e-07_rb,3.9262e-07_rb,3.3860e-07_rb,7.0920e-08_rb/)
      kbo(:, 5,51, 2) = (/ &
     &3.6391e-07_rb,5.8370e-07_rb,5.8331e-07_rb,4.9304e-07_rb,8.9643e-08_rb/)
      kbo(:, 1,52, 2) = (/ &
     &9.4432e-08_rb,1.2100e-07_rb,1.0830e-07_rb,8.6173e-08_rb,2.0454e-08_rb/)
      kbo(:, 2,52, 2) = (/ &
     &1.0754e-07_rb,1.4816e-07_rb,1.4031e-07_rb,1.1784e-07_rb,2.8884e-08_rb/)
      kbo(:, 3,52, 2) = (/ &
     &1.3365e-07_rb,1.9803e-07_rb,1.9664e-07_rb,1.6925e-07_rb,3.9314e-08_rb/)
      kbo(:, 4,52, 2) = (/ &
     &1.8256e-07_rb,2.8565e-07_rb,2.8767e-07_rb,2.4757e-07_rb,5.1552e-08_rb/)
      kbo(:, 5,52, 2) = (/ &
     &2.7006e-07_rb,4.2782e-07_rb,4.2750e-07_rb,3.6187e-07_rb,6.6239e-08_rb/)
      kbo(:, 1,53, 2) = (/ &
     &7.9939e-08_rb,9.8272e-08_rb,8.6307e-08_rb,6.6012e-08_rb,1.4604e-08_rb/)
      kbo(:, 2,53, 2) = (/ &
     &8.8279e-08_rb,1.1673e-07_rb,1.0758e-07_rb,8.7937e-08_rb,2.0451e-08_rb/)
      kbo(:, 3,53, 2) = (/ &
     &1.0623e-07_rb,1.5078e-07_rb,1.4654e-07_rb,1.2425e-07_rb,2.8208e-08_rb/)
      kbo(:, 4,53, 2) = (/ &
     &1.4011e-07_rb,2.1203e-07_rb,2.1153e-07_rb,1.8129e-07_rb,3.7255e-08_rb/)
      kbo(:, 5,53, 2) = (/ &
     &2.0211e-07_rb,3.1443e-07_rb,3.1357e-07_rb,2.6539e-07_rb,4.8314e-08_rb/)
      kbo(:, 1,54, 2) = (/ &
     &6.8612e-08_rb,8.0858e-08_rb,7.0080e-08_rb,5.1088e-08_rb,1.0952e-08_rb/)
      kbo(:, 2,54, 2) = (/ &
     &7.3786e-08_rb,9.3902e-08_rb,8.4431e-08_rb,6.7059e-08_rb,1.4950e-08_rb/)
      kbo(:, 3,54, 2) = (/ &
     &8.6236e-08_rb,1.1751e-07_rb,1.1148e-07_rb,9.2909e-08_rb,2.0689e-08_rb/)
      kbo(:, 4,54, 2) = (/ &
     &1.1017e-07_rb,1.6065e-07_rb,1.5823e-07_rb,1.3458e-07_rb,2.7837e-08_rb/)
      kbo(:, 5,54, 2) = (/ &
     &1.5462e-07_rb,2.3495e-07_rb,2.3325e-07_rb,1.9698e-07_rb,3.6383e-08_rb/)
      kbo(:, 1,55, 2) = (/ &
     &5.9732e-08_rb,6.7403e-08_rb,5.7777e-08_rb,3.9915e-08_rb,8.5037e-09_rb/)
      kbo(:, 2,55, 2) = (/ &
     &6.2576e-08_rb,7.6690e-08_rb,6.7863e-08_rb,5.1930e-08_rb,1.1059e-08_rb/)
      kbo(:, 3,55, 2) = (/ &
     &7.1126e-08_rb,9.3200e-08_rb,8.6382e-08_rb,7.0395e-08_rb,1.5573e-08_rb/)
      kbo(:, 4,55, 2) = (/ &
     &8.8230e-08_rb,1.2378e-07_rb,1.1984e-07_rb,1.0085e-07_rb,2.1225e-08_rb/)
      kbo(:, 5,55, 2) = (/ &
     &1.2026e-07_rb,1.7765e-07_rb,1.7500e-07_rb,1.4721e-07_rb,2.7783e-08_rb/)
      kbo(:, 1,56, 2) = (/ &
     &5.2687e-08_rb,5.6857e-08_rb,4.7895e-08_rb,3.2001e-08_rb,6.6275e-09_rb/)
      kbo(:, 2,56, 2) = (/ &
     &5.3766e-08_rb,6.3415e-08_rb,5.5171e-08_rb,4.0341e-08_rb,8.2176e-09_rb/)
      kbo(:, 3,56, 2) = (/ &
     &5.9487e-08_rb,7.5070e-08_rb,6.7900e-08_rb,5.3874e-08_rb,1.1605e-08_rb/)
      kbo(:, 4,56, 2) = (/ &
     &7.1736e-08_rb,9.6663e-08_rb,9.1606e-08_rb,7.5983e-08_rb,1.6105e-08_rb/)
      kbo(:, 5,56, 2) = (/ &
     &9.4811e-08_rb,1.3547e-07_rb,1.3204e-07_rb,1.1036e-07_rb,2.1203e-08_rb/)
      kbo(:, 1,57, 2) = (/ &
     &4.7030e-08_rb,4.8568e-08_rb,3.9988e-08_rb,2.6219e-08_rb,4.9408e-09_rb/)
      kbo(:, 2,57, 2) = (/ &
     &4.6811e-08_rb,5.3015e-08_rb,4.5546e-08_rb,3.1769e-08_rb,6.1353e-09_rb/)
      kbo(:, 3,57, 2) = (/ &
     &5.0494e-08_rb,6.1370e-08_rb,5.4487e-08_rb,4.1710e-08_rb,8.6536e-09_rb/)
      kbo(:, 4,57, 2) = (/ &
     &5.9216e-08_rb,7.6592e-08_rb,7.0994e-08_rb,5.7693e-08_rb,1.1916e-08_rb/)
      kbo(:, 5,57, 2) = (/ &
     &7.5908e-08_rb,1.0451e-07_rb,1.0035e-07_rb,8.3097e-08_rb,1.6230e-08_rb/)
      kbo(:, 1,58, 2) = (/ &
     &2.0524e-08_rb,2.3377e-08_rb,2.2103e-08_rb,1.7291e-08_rb,3.6303e-09_rb/)
      kbo(:, 2,58, 2) = (/ &
     &2.0023e-08_rb,2.5008e-08_rb,2.4831e-08_rb,2.0273e-08_rb,4.7070e-09_rb/)
      kbo(:, 3,58, 2) = (/ &
     &2.1076e-08_rb,2.8356e-08_rb,2.8987e-08_rb,2.5840e-08_rb,6.5048e-09_rb/)
      kbo(:, 4,58, 2) = (/ &
     &2.4072e-08_rb,3.4377e-08_rb,3.6577e-08_rb,3.5144e-08_rb,8.9689e-09_rb/)
      kbo(:, 5,58, 2) = (/ &
     &3.0004e-08_rb,4.5626e-08_rb,5.0474e-08_rb,4.9976e-08_rb,1.2428e-08_rb/)
      kbo(:, 1,59, 2) = (/ &
     &1.7700e-08_rb,1.9843e-08_rb,1.8643e-08_rb,1.4520e-08_rb,2.9679e-09_rb/)
      kbo(:, 2,59, 2) = (/ &
     &1.7149e-08_rb,2.1008e-08_rb,2.0747e-08_rb,1.6830e-08_rb,3.9081e-09_rb/)
      kbo(:, 3,59, 2) = (/ &
     &1.7873e-08_rb,2.3612e-08_rb,2.4046e-08_rb,2.1098e-08_rb,5.4593e-09_rb/)
      kbo(:, 4,59, 2) = (/ &
     &2.0201e-08_rb,2.8287e-08_rb,2.9905e-08_rb,2.8473e-08_rb,7.4791e-09_rb/)
      kbo(:, 5,59, 2) = (/ &
     &2.4905e-08_rb,3.7065e-08_rb,4.0703e-08_rb,4.0125e-08_rb,1.0391e-08_rb/)
      kbo(:, 1,13, 3) = (/ &
     &7.5570e-04_rb,1.0772e-03_rb,1.0524e-03_rb,8.5610e-04_rb,5.7960e-04_rb/)
      kbo(:, 2,13, 3) = (/ &
     &1.1289e-03_rb,1.5736e-03_rb,1.5191e-03_rb,1.2213e-03_rb,7.5295e-04_rb/)
      kbo(:, 3,13, 3) = (/ &
     &1.7896e-03_rb,2.3714e-03_rb,2.2650e-03_rb,1.7547e-03_rb,9.2113e-04_rb/)
      kbo(:, 4,13, 3) = (/ &
     &2.8881e-03_rb,3.6381e-03_rb,3.3414e-03_rb,2.5211e-03_rb,1.0886e-03_rb/)
      kbo(:, 5,13, 3) = (/ &
     &4.6526e-03_rb,5.5012e-03_rb,4.9005e-03_rb,3.6099e-03_rb,1.2715e-03_rb/)
      kbo(:, 1,14, 3) = (/ &
     &6.5628e-04_rb,9.2369e-04_rb,9.0249e-04_rb,7.3144e-04_rb,5.2166e-04_rb/)
      kbo(:, 2,14, 3) = (/ &
     &1.0001e-03_rb,1.3681e-03_rb,1.3197e-03_rb,1.0469e-03_rb,6.6148e-04_rb/)
      kbo(:, 3,14, 3) = (/ &
     &1.6037e-03_rb,2.0859e-03_rb,1.9738e-03_rb,1.5151e-03_rb,7.9350e-04_rb/)
      kbo(:, 4,14, 3) = (/ &
     &2.6225e-03_rb,3.2284e-03_rb,2.9281e-03_rb,2.1941e-03_rb,9.2716e-04_rb/)
      kbo(:, 5,14, 3) = (/ &
     &4.2437e-03_rb,4.9002e-03_rb,4.3187e-03_rb,3.1691e-03_rb,1.0609e-03_rb/)
      kbo(:, 1,15, 3) = (/ &
     &5.5021e-04_rb,7.7058e-04_rb,7.4997e-04_rb,6.0721e-04_rb,4.5513e-04_rb/)
      kbo(:, 2,15, 3) = (/ &
     &8.5046e-04_rb,1.1533e-03_rb,1.1173e-03_rb,8.7626e-04_rb,5.6093e-04_rb/)
      kbo(:, 3,15, 3) = (/ &
     &1.3828e-03_rb,1.7898e-03_rb,1.6630e-03_rb,1.2772e-03_rb,6.6129e-04_rb/)
      kbo(:, 4,15, 3) = (/ &
     &2.2885e-03_rb,2.7671e-03_rb,2.4977e-03_rb,1.8674e-03_rb,7.6057e-04_rb/)
      kbo(:, 5,15, 3) = (/ &
     &3.7000e-03_rb,4.2354e-03_rb,3.7118e-03_rb,2.7146e-03_rb,8.7024e-04_rb/)
      kbo(:, 1,16, 3) = (/ &
     &4.5181e-04_rb,6.3192e-04_rb,6.1748e-04_rb,4.9522e-04_rb,3.6566e-04_rb/)
      kbo(:, 2,16, 3) = (/ &
     &7.0837e-04_rb,9.6443e-04_rb,9.2028e-04_rb,7.2449e-04_rb,4.4224e-04_rb/)
      kbo(:, 3,16, 3) = (/ &
     &1.1719e-03_rb,1.5071e-03_rb,1.3916e-03_rb,1.0704e-03_rb,5.1878e-04_rb/)
      kbo(:, 4,16, 3) = (/ &
     &1.9347e-03_rb,2.3410e-03_rb,2.1054e-03_rb,1.5750e-03_rb,5.9381e-04_rb/)
      kbo(:, 5,16, 3) = (/ &
     &3.1267e-03_rb,3.5933e-03_rb,3.1446e-03_rb,2.3012e-03_rb,6.8790e-04_rb/)
      kbo(:, 1,17, 3) = (/ &
     &3.7149e-04_rb,5.2008e-04_rb,5.0762e-04_rb,4.0757e-04_rb,2.8847e-04_rb/)
      kbo(:, 2,17, 3) = (/ &
     &5.8832e-04_rb,8.0612e-04_rb,7.6169e-04_rb,6.0348e-04_rb,3.4723e-04_rb/)
      kbo(:, 3,17, 3) = (/ &
     &9.7995e-04_rb,1.2582e-03_rb,1.1654e-03_rb,8.9803e-04_rb,4.0483e-04_rb/)
      kbo(:, 4,17, 3) = (/ &
     &1.6147e-03_rb,1.9685e-03_rb,1.7733e-03_rb,1.3298e-03_rb,4.7136e-04_rb/)
      kbo(:, 5,17, 3) = (/ &
     &2.6071e-03_rb,3.0247e-03_rb,2.6450e-03_rb,1.9458e-03_rb,5.5921e-04_rb/)
      kbo(:, 1,18, 3) = (/ &
     &3.0061e-04_rb,4.2884e-04_rb,4.1648e-04_rb,3.3754e-04_rb,2.3127e-04_rb/)
      kbo(:, 2,18, 3) = (/ &
     &4.8246e-04_rb,6.6793e-04_rb,6.3270e-04_rb,5.0432e-04_rb,2.7279e-04_rb/)
      kbo(:, 3,18, 3) = (/ &
     &8.0392e-04_rb,1.0486e-03_rb,9.7147e-04_rb,7.5301e-04_rb,3.2007e-04_rb/)
      kbo(:, 4,18, 3) = (/ &
     &1.3260e-03_rb,1.6386e-03_rb,1.4792e-03_rb,1.1164e-03_rb,3.8144e-04_rb/)
      kbo(:, 5,18, 3) = (/ &
     &2.1473e-03_rb,2.5126e-03_rb,2.2067e-03_rb,1.6276e-03_rb,4.4653e-04_rb/)
      kbo(:, 1,19, 3) = (/ &
     &2.4724e-04_rb,3.5910e-04_rb,3.4734e-04_rb,2.8349e-04_rb,1.8511e-04_rb/)
      kbo(:, 2,19, 3) = (/ &
     &4.0125e-04_rb,5.6017e-04_rb,5.3189e-04_rb,4.2493e-04_rb,2.2055e-04_rb/)
      kbo(:, 3,19, 3) = (/ &
     &6.6924e-04_rb,8.8342e-04_rb,8.2067e-04_rb,6.3860e-04_rb,2.6260e-04_rb/)
      kbo(:, 4,19, 3) = (/ &
     &1.1061e-03_rb,1.3792e-03_rb,1.2462e-03_rb,9.4376e-04_rb,3.1226e-04_rb/)
      kbo(:, 5,19, 3) = (/ &
     &1.7964e-03_rb,2.1126e-03_rb,1.8583e-03_rb,1.3721e-03_rb,3.6746e-04_rb/)
      kbo(:, 1,20, 3) = (/ &
     &2.0934e-04_rb,3.0700e-04_rb,2.9722e-04_rb,2.4267e-04_rb,1.4926e-04_rb/)
      kbo(:, 2,20, 3) = (/ &
     &3.4260e-04_rb,4.8187e-04_rb,4.5763e-04_rb,3.6566e-04_rb,1.7991e-04_rb/)
      kbo(:, 3,20, 3) = (/ &
     &5.7163e-04_rb,7.5991e-04_rb,7.0644e-04_rb,5.4990e-04_rb,2.1557e-04_rb/)
      kbo(:, 4,20, 3) = (/ &
     &9.4873e-04_rb,1.1845e-03_rb,1.0719e-03_rb,8.1150e-04_rb,2.5564e-04_rb/)
      kbo(:, 5,20, 3) = (/ &
     &1.5383e-03_rb,1.8175e-03_rb,1.5965e-03_rb,1.1755e-03_rb,3.0407e-04_rb/)
      kbo(:, 1,21, 3) = (/ &
     &1.7843e-04_rb,2.6350e-04_rb,2.5564e-04_rb,2.0870e-04_rb,1.2133e-04_rb/)
      kbo(:, 2,21, 3) = (/ &
     &2.9332e-04_rb,4.1557e-04_rb,3.9544e-04_rb,3.1597e-04_rb,1.4677e-04_rb/)
      kbo(:, 3,21, 3) = (/ &
     &4.9132e-04_rb,6.5537e-04_rb,6.0874e-04_rb,4.7437e-04_rb,1.7728e-04_rb/)
      kbo(:, 4,21, 3) = (/ &
     &8.1854e-04_rb,1.0220e-03_rb,9.2374e-04_rb,6.9881e-04_rb,2.1085e-04_rb/)
      kbo(:, 5,21, 3) = (/ &
     &1.3233e-03_rb,1.5674e-03_rb,1.3737e-03_rb,1.0105e-03_rb,2.5000e-04_rb/)
      kbo(:, 1,22, 3) = (/ &
     &1.5600e-04_rb,2.3165e-04_rb,2.2455e-04_rb,1.8380e-04_rb,9.8965e-05_rb/)
      kbo(:, 2,22, 3) = (/ &
     &2.5879e-04_rb,3.6647e-04_rb,3.4891e-04_rb,2.7934e-04_rb,1.2038e-04_rb/)
      kbo(:, 3,22, 3) = (/ &
     &4.3583e-04_rb,5.7888e-04_rb,5.3753e-04_rb,4.1782e-04_rb,1.4492e-04_rb/)
      kbo(:, 4,22, 3) = (/ &
     &7.2393e-04_rb,9.0328e-04_rb,8.1451e-04_rb,6.1366e-04_rb,1.7404e-04_rb/)
      kbo(:, 5,22, 3) = (/ &
     &1.1641e-03_rb,1.3807e-03_rb,1.2062e-03_rb,8.8532e-04_rb,2.0548e-04_rb/)
      kbo(:, 1,23, 3) = (/ &
     &1.3677e-04_rb,2.0433e-04_rb,1.9788e-04_rb,1.6232e-04_rb,8.0071e-05_rb/)
      kbo(:, 2,23, 3) = (/ &
     &2.2851e-04_rb,3.2392e-04_rb,3.0774e-04_rb,2.4623e-04_rb,9.8059e-05_rb/)
      kbo(:, 3,23, 3) = (/ &
     &3.8639e-04_rb,5.1167e-04_rb,4.7457e-04_rb,3.6749e-04_rb,1.1880e-04_rb/)
      kbo(:, 4,23, 3) = (/ &
     &6.3801e-04_rb,7.9694e-04_rb,7.1675e-04_rb,5.3954e-04_rb,1.4295e-04_rb/)
      kbo(:, 5,23, 3) = (/ &
     &1.0211e-03_rb,1.2096e-03_rb,1.0561e-03_rb,7.7552e-04_rb,1.6841e-04_rb/)
      kbo(:, 1,24, 3) = (/ &
     &1.2065e-04_rb,1.8059e-04_rb,1.7513e-04_rb,1.4381e-04_rb,6.4790e-05_rb/)
      kbo(:, 2,24, 3) = (/ &
     &2.0387e-04_rb,2.8742e-04_rb,2.7331e-04_rb,2.1771e-04_rb,7.9169e-05_rb/)
      kbo(:, 3,24, 3) = (/ &
     &3.4413e-04_rb,4.5518e-04_rb,4.2044e-04_rb,3.2458e-04_rb,9.5995e-05_rb/)
      kbo(:, 4,24, 3) = (/ &
     &5.6569e-04_rb,7.0542e-04_rb,6.3169e-04_rb,4.7542e-04_rb,1.1593e-04_rb/)
      kbo(:, 5,24, 3) = (/ &
     &8.9817e-04_rb,1.0616e-03_rb,9.2695e-04_rb,6.8043e-04_rb,1.3621e-04_rb/)
      kbo(:, 1,25, 3) = (/ &
     &1.0740e-04_rb,1.6092e-04_rb,1.5588e-04_rb,1.2763e-04_rb,5.2835e-05_rb/)
      kbo(:, 2,25, 3) = (/ &
     &1.8277e-04_rb,2.5653e-04_rb,2.4346e-04_rb,1.9325e-04_rb,6.4708e-05_rb/)
      kbo(:, 3,25, 3) = (/ &
     &3.0775e-04_rb,4.0577e-04_rb,3.7301e-04_rb,2.8766e-04_rb,7.8479e-05_rb/)
      kbo(:, 4,25, 3) = (/ &
     &5.0348e-04_rb,6.2434e-04_rb,5.5838e-04_rb,4.1972e-04_rb,9.2999e-05_rb/)
      kbo(:, 5,25, 3) = (/ &
     &7.9236e-04_rb,9.3393e-04_rb,8.1486e-04_rb,5.9712e-04_rb,1.1091e-04_rb/)
      kbo(:, 1,26, 3) = (/ &
     &9.7543e-05_rb,1.4515e-04_rb,1.4079e-04_rb,1.1469e-04_rb,4.3203e-05_rb/)
      kbo(:, 2,26, 3) = (/ &
     &1.6637e-04_rb,2.3211e-04_rb,2.1906e-04_rb,1.7338e-04_rb,5.3501e-05_rb/)
      kbo(:, 3,26, 3) = (/ &
     &2.7898e-04_rb,3.6497e-04_rb,3.3420e-04_rb,2.5717e-04_rb,6.4391e-05_rb/)
      kbo(:, 4,26, 3) = (/ &
     &4.5278e-04_rb,5.5785e-04_rb,4.9753e-04_rb,3.7293e-04_rb,7.6357e-05_rb/)
      kbo(:, 5,26, 3) = (/ &
     &7.0495e-04_rb,8.2859e-04_rb,7.2055e-04_rb,5.2702e-04_rb,9.1483e-05_rb/)
      kbo(:, 1,27, 3) = (/ &
     &8.8905e-05_rb,1.3136e-04_rb,1.2690e-04_rb,1.0312e-04_rb,3.5556e-05_rb/)
      kbo(:, 2,27, 3) = (/ &
     &1.5157e-04_rb,2.0993e-04_rb,1.9708e-04_rb,1.5550e-04_rb,4.3708e-05_rb/)
      kbo(:, 3,27, 3) = (/ &
     &2.5313e-04_rb,3.2779e-04_rb,2.9943e-04_rb,2.2974e-04_rb,5.2449e-05_rb/)
      kbo(:, 4,27, 3) = (/ &
     &4.0707e-04_rb,4.9850e-04_rb,4.4267e-04_rb,3.3107e-04_rb,6.3153e-05_rb/)
      kbo(:, 5,27, 3) = (/ &
     &6.2804e-04_rb,7.3434e-04_rb,6.3748e-04_rb,4.6470e-04_rb,7.4410e-05_rb/)
      kbo(:, 1,28, 3) = (/ &
     &8.1071e-05_rb,1.1893e-04_rb,1.1441e-04_rb,9.2670e-05_rb,2.9048e-05_rb/)
      kbo(:, 2,28, 3) = (/ &
     &1.3818e-04_rb,1.8937e-04_rb,1.7730e-04_rb,1.3932e-04_rb,3.5467e-05_rb/)
      kbo(:, 3,28, 3) = (/ &
     &2.2925e-04_rb,2.9449e-04_rb,2.6762e-04_rb,2.0484e-04_rb,4.2994e-05_rb/)
      kbo(:, 4,28, 3) = (/ &
     &3.6544e-04_rb,4.4427e-04_rb,3.9356e-04_rb,2.9338e-04_rb,5.1335e-05_rb/)
      kbo(:, 5,28, 3) = (/ &
     &5.5873e-04_rb,6.5113e-04_rb,5.6346e-04_rb,4.0933e-04_rb,6.0144e-05_rb/)
      kbo(:, 1,29, 3) = (/ &
     &7.4203e-05_rb,1.0781e-04_rb,1.0341e-04_rb,8.3458e-05_rb,2.3671e-05_rb/)
      kbo(:, 2,29, 3) = (/ &
     &1.2605e-04_rb,1.7113e-04_rb,1.5936e-04_rb,1.2487e-04_rb,2.9121e-05_rb/)
      kbo(:, 3,29, 3) = (/ &
     &2.0703e-04_rb,2.6395e-04_rb,2.3919e-04_rb,1.8248e-04_rb,3.5057e-05_rb/)
      kbo(:, 4,29, 3) = (/ &
     &3.2741e-04_rb,3.9598e-04_rb,3.4952e-04_rb,2.5961e-04_rb,4.1444e-05_rb/)
      kbo(:, 5,29, 3) = (/ &
     &4.9624e-04_rb,5.7661e-04_rb,4.9724e-04_rb,3.6006e-04_rb,4.8751e-05_rb/)
      kbo(:, 1,30, 3) = (/ &
     &6.7677e-05_rb,9.7468e-05_rb,9.3138e-05_rb,7.4923e-05_rb,1.9287e-05_rb/)
      kbo(:, 2,30, 3) = (/ &
     &1.1431e-04_rb,1.5374e-04_rb,1.4284e-04_rb,1.1153e-04_rb,2.3591e-05_rb/)
      kbo(:, 3,30, 3) = (/ &
     &1.8623e-04_rb,2.3583e-04_rb,2.1291e-04_rb,1.6193e-04_rb,2.8158e-05_rb/)
      kbo(:, 4,30, 3) = (/ &
     &2.9180e-04_rb,3.5163e-04_rb,3.0921e-04_rb,2.2892e-04_rb,3.3328e-05_rb/)
      kbo(:, 5,30, 3) = (/ &
     &4.3918e-04_rb,5.0854e-04_rb,4.3751e-04_rb,3.1569e-04_rb,3.9501e-05_rb/)
      kbo(:, 1,31, 3) = (/ &
     &6.1393e-05_rb,8.7694e-05_rb,8.3630e-05_rb,6.7048e-05_rb,1.5382e-05_rb/)
      kbo(:, 2,31, 3) = (/ &
     &1.0290e-04_rb,1.3743e-04_rb,1.2731e-04_rb,9.9241e-05_rb,1.8651e-05_rb/)
      kbo(:, 3,31, 3) = (/ &
     &1.6626e-04_rb,2.0956e-04_rb,1.8851e-04_rb,1.4301e-04_rb,2.2748e-05_rb/)
      kbo(:, 4,31, 3) = (/ &
     &2.5819e-04_rb,3.1016e-04_rb,2.7211e-04_rb,2.0081e-04_rb,2.7039e-05_rb/)
      kbo(:, 5,31, 3) = (/ &
     &3.8535e-04_rb,4.4535e-04_rb,3.8254e-04_rb,2.7522e-04_rb,3.1790e-05_rb/)
      kbo(:, 1,32, 3) = (/ &
     &5.5908e-05_rb,7.9178e-05_rb,7.5227e-05_rb,6.0166e-05_rb,1.2301e-05_rb/)
      kbo(:, 2,32, 3) = (/ &
     &9.3029e-05_rb,1.2336e-04_rb,1.1373e-04_rb,8.8415e-05_rb,1.5236e-05_rb/)
      kbo(:, 3,32, 3) = (/ &
     &1.4900e-04_rb,1.8677e-04_rb,1.6744e-04_rb,1.2652e-04_rb,1.8544e-05_rb/)
      kbo(:, 4,32, 3) = (/ &
     &2.2938e-04_rb,2.7443e-04_rb,2.4002e-04_rb,1.7648e-04_rb,2.2012e-05_rb/)
      kbo(:, 5,32, 3) = (/ &
     &3.3936e-04_rb,3.9112e-04_rb,3.3532e-04_rb,2.4040e-04_rb,2.5885e-05_rb/)
      kbo(:, 1,33, 3) = (/ &
     &5.1027e-05_rb,7.1592e-05_rb,6.7752e-05_rb,5.3984e-05_rb,1.0069e-05_rb/)
      kbo(:, 2,33, 3) = (/ &
     &8.4306e-05_rb,1.1082e-04_rb,1.0178e-04_rb,7.8786e-05_rb,1.2424e-05_rb/)
      kbo(:, 3,33, 3) = (/ &
     &1.3378e-04_rb,1.6666e-04_rb,1.4886e-04_rb,1.1198e-04_rb,1.5017e-05_rb/)
      kbo(:, 4,33, 3) = (/ &
     &2.0407e-04_rb,2.4310e-04_rb,2.1204e-04_rb,1.5519e-04_rb,1.7833e-05_rb/)
      kbo(:, 5,33, 3) = (/ &
     &2.9932e-04_rb,3.4399e-04_rb,2.9425e-04_rb,2.1002e-04_rb,2.0751e-05_rb/)
      kbo(:, 1,34, 3) = (/ &
     &4.6423e-05_rb,6.4385e-05_rb,6.0651e-05_rb,4.8084e-05_rb,8.2313e-06_rb/)
      kbo(:, 2,34, 3) = (/ &
     &7.6075e-05_rb,9.9124e-05_rb,9.0586e-05_rb,6.9780e-05_rb,1.0168e-05_rb/)
      kbo(:, 3,34, 3) = (/ &
     &1.1981e-04_rb,1.4811e-04_rb,1.3176e-04_rb,9.8597e-05_rb,1.2205e-05_rb/)
      kbo(:, 4,34, 3) = (/ &
     &1.8123e-04_rb,2.1472e-04_rb,1.8662e-04_rb,1.3590e-04_rb,1.4492e-05_rb/)
      kbo(:, 5,34, 3) = (/ &
     &2.6376e-04_rb,3.0190e-04_rb,2.5753e-04_rb,1.8301e-04_rb,1.6946e-05_rb/)
      kbo(:, 1,35, 3) = (/ &
     &4.0215e-05_rb,5.5516e-05_rb,5.2178e-05_rb,4.1265e-05_rb,6.6300e-06_rb/)
      kbo(:, 2,35, 3) = (/ &
     &6.5646e-05_rb,8.5207e-05_rb,7.7691e-05_rb,5.9694e-05_rb,8.1351e-06_rb/)
      kbo(:, 3,35, 3) = (/ &
     &1.0298e-04_rb,1.2694e-04_rb,1.1266e-04_rb,8.4059e-05_rb,9.8103e-06_rb/)
      kbo(:, 4,35, 3) = (/ &
     &1.5503e-04_rb,1.8334e-04_rb,1.5905e-04_rb,1.1551e-04_rb,1.1630e-05_rb/)
      kbo(:, 5,35, 3) = (/ &
     &2.2476e-04_rb,2.5697e-04_rb,2.1886e-04_rb,1.5514e-04_rb,1.3591e-05_rb/)
      kbo(:, 1,36, 3) = (/ &
     &3.2994e-05_rb,4.5660e-05_rb,4.2930e-05_rb,3.3969e-05_rb,5.1542e-06_rb/)
      kbo(:, 2,36, 3) = (/ &
     &5.3916e-05_rb,7.0104e-05_rb,6.3946e-05_rb,4.9132e-05_rb,6.3498e-06_rb/)
      kbo(:, 3,36, 3) = (/ &
     &8.4552e-05_rb,1.0443e-04_rb,9.2717e-05_rb,6.9170e-05_rb,7.6665e-06_rb/)
      kbo(:, 4,36, 3) = (/ &
     &1.2731e-04_rb,1.5086e-04_rb,1.3088e-04_rb,9.4999e-05_rb,9.0883e-06_rb/)
      kbo(:, 5,36, 3) = (/ &
     &1.8455e-04_rb,2.1131e-04_rb,1.8000e-04_rb,1.2756e-04_rb,1.0606e-05_rb/)
      kbo(:, 1,37, 3) = (/ &
     &2.5429e-05_rb,3.5502e-05_rb,3.3507e-05_rb,2.6612e-05_rb,3.9872e-06_rb/)
      kbo(:, 2,37, 3) = (/ &
     &4.1789e-05_rb,5.4774e-05_rb,5.0118e-05_rb,3.8644e-05_rb,4.9244e-06_rb/)
      kbo(:, 3,37, 3) = (/ &
     &6.5920e-05_rb,8.1963e-05_rb,7.2974e-05_rb,5.4610e-05_rb,5.9794e-06_rb/)
      kbo(:, 4,37, 3) = (/ &
     &9.9825e-05_rb,1.1892e-04_rb,1.0339e-04_rb,7.5254e-05_rb,7.1021e-06_rb/)
      kbo(:, 5,37, 3) = (/ &
     &1.4543e-04_rb,1.6731e-04_rb,1.4273e-04_rb,1.0135e-04_rb,8.2993e-06_rb/)
      kbo(:, 1,38, 3) = (/ &
     &1.9510e-05_rb,2.7499e-05_rb,2.6048e-05_rb,2.0774e-05_rb,3.0726e-06_rb/)
      kbo(:, 2,38, 3) = (/ &
     &3.2252e-05_rb,4.2623e-05_rb,3.9146e-05_rb,3.0291e-05_rb,3.8102e-06_rb/)
      kbo(:, 3,38, 3) = (/ &
     &5.1187e-05_rb,6.4097e-05_rb,5.7223e-05_rb,4.2970e-05_rb,4.6390e-06_rb/)
      kbo(:, 4,38, 3) = (/ &
     &7.8004e-05_rb,9.3430e-05_rb,8.1421e-05_rb,5.9437e-05_rb,5.5099e-06_rb/)
      kbo(:, 5,38, 3) = (/ &
     &1.1424e-04_rb,1.3207e-04_rb,1.1286e-04_rb,8.0305e-05_rb,6.4580e-06_rb/)
      kbo(:, 1,39, 3) = (/ &
     &1.4955e-05_rb,2.1282e-05_rb,2.0237e-05_rb,1.6208e-05_rb,2.3414e-06_rb/)
      kbo(:, 2,39, 3) = (/ &
     &2.4867e-05_rb,3.3144e-05_rb,3.0547e-05_rb,2.3724e-05_rb,2.9298e-06_rb/)
      kbo(:, 3,39, 3) = (/ &
     &3.9718e-05_rb,5.0092e-05_rb,4.4848e-05_rb,3.3792e-05_rb,3.5597e-06_rb/)
      kbo(:, 4,39, 3) = (/ &
     &6.0902e-05_rb,7.3365e-05_rb,6.4080e-05_rb,4.6913e-05_rb,4.2665e-06_rb/)
      kbo(:, 5,39, 3) = (/ &
     &8.9673e-05_rb,1.0416e-04_rb,8.9163e-05_rb,6.3592e-05_rb,5.0003e-06_rb/)
      kbo(:, 1,40, 3) = (/ &
     &1.0973e-05_rb,1.5834e-05_rb,1.5141e-05_rb,1.2207e-05_rb,1.7695e-06_rb/)
      kbo(:, 2,40, 3) = (/ &
     &1.8417e-05_rb,2.4851e-05_rb,2.3037e-05_rb,1.7992e-05_rb,2.2304e-06_rb/)
      kbo(:, 3,40, 3) = (/ &
     &2.9718e-05_rb,3.7867e-05_rb,3.4065e-05_rb,2.5810e-05_rb,2.7325e-06_rb/)
      kbo(:, 4,40, 3) = (/ &
     &4.6050e-05_rb,5.5927e-05_rb,4.9024e-05_rb,3.6065e-05_rb,3.2836e-06_rb/)
      kbo(:, 5,40, 3) = (/ &
     &6.8462e-05_rb,8.0022e-05_rb,6.8684e-05_rb,4.9179e-05_rb,3.8780e-06_rb/)
      kbo(:, 1,41, 3) = (/ &
     &7.9936e-06_rb,1.1702e-05_rb,1.1250e-05_rb,9.1326e-06_rb,1.3289e-06_rb/)
      kbo(:, 2,41, 3) = (/ &
     &1.3541e-05_rb,1.8510e-05_rb,1.7261e-05_rb,1.3563e-05_rb,1.6876e-06_rb/)
      kbo(:, 3,41, 3) = (/ &
     &2.2085e-05_rb,2.8450e-05_rb,2.5728e-05_rb,1.9606e-05_rb,2.0896e-06_rb/)
      kbo(:, 4,41, 3) = (/ &
     &3.4599e-05_rb,4.2393e-05_rb,3.7310e-05_rb,2.7591e-05_rb,2.5142e-06_rb/)
      kbo(:, 5,41, 3) = (/ &
     &5.1973e-05_rb,6.1166e-05_rb,5.2649e-05_rb,3.7859e-05_rb,2.9985e-06_rb/)
      kbo(:, 1,42, 3) = (/ &
     &5.8131e-06_rb,8.6287e-06_rb,8.3395e-06_rb,6.8180e-06_rb,9.9321e-07_rb/)
      kbo(:, 2,42, 3) = (/ &
     &9.9286e-06_rb,1.3753e-05_rb,1.2903e-05_rb,1.0201e-05_rb,1.2739e-06_rb/)
      kbo(:, 3,42, 3) = (/ &
     &1.6369e-05_rb,2.1324e-05_rb,1.9387e-05_rb,1.4861e-05_rb,1.5862e-06_rb/)
      kbo(:, 4,42, 3) = (/ &
     &2.5917e-05_rb,3.2058e-05_rb,2.8333e-05_rb,2.1064e-05_rb,1.9289e-06_rb/)
      kbo(:, 5,42, 3) = (/ &
     &3.9355e-05_rb,4.6652e-05_rb,4.0275e-05_rb,2.9099e-05_rb,2.3091e-06_rb/)
      kbo(:, 1,43, 3) = (/ &
     &4.1379e-06_rb,6.2276e-06_rb,6.0505e-06_rb,4.9862e-06_rb,7.3153e-07_rb/)
      kbo(:, 2,43, 3) = (/ &
     &7.1228e-06_rb,1.0012e-05_rb,9.4538e-06_rb,7.5288e-06_rb,9.4466e-07_rb/)
      kbo(:, 3,43, 3) = (/ &
     &1.1889e-05_rb,1.5688e-05_rb,1.4350e-05_rb,1.1075e-05_rb,1.1892e-06_rb/)
      kbo(:, 4,43, 3) = (/ &
     &1.9074e-05_rb,2.3844e-05_rb,2.1179e-05_rb,1.5845e-05_rb,1.4637e-06_rb/)
      kbo(:, 5,43, 3) = (/ &
     &2.9351e-05_rb,3.5073e-05_rb,3.0382e-05_rb,2.2068e-05_rb,1.7581e-06_rb/)
      kbo(:, 1,44, 3) = (/ &
     &2.9204e-06_rb,4.4462e-06_rb,4.3423e-06_rb,3.6069e-06_rb,5.3481e-07_rb/)
      kbo(:, 2,44, 3) = (/ &
     &5.0527e-06_rb,7.2110e-06_rb,6.8508e-06_rb,5.4982e-06_rb,6.9481e-07_rb/)
      kbo(:, 3,44, 3) = (/ &
     &8.5402e-06_rb,1.1419e-05_rb,1.0515e-05_rb,8.1724e-06_rb,8.8543e-07_rb/)
      kbo(:, 4,44, 3) = (/ &
     &1.3894e-05_rb,1.7562e-05_rb,1.5682e-05_rb,1.1814e-05_rb,1.0984e-06_rb/)
      kbo(:, 5,44, 3) = (/ &
     &2.1692e-05_rb,2.6150e-05_rb,2.2738e-05_rb,1.6619e-05_rb,1.3258e-06_rb/)
      kbo(:, 1,45, 3) = (/ &
     &2.0654e-06_rb,3.1706e-06_rb,3.1106e-06_rb,2.6027e-06_rb,3.8843e-07_rb/)
      kbo(:, 2,45, 3) = (/ &
     &3.5785e-06_rb,5.1800e-06_rb,4.9498e-06_rb,4.0029e-06_rb,5.0552e-07_rb/)
      kbo(:, 3,45, 3) = (/ &
     &6.1156e-06_rb,8.2899e-06_rb,7.6825e-06_rb,6.0114e-06_rb,6.4899e-07_rb/)
      kbo(:, 4,45, 3) = (/ &
     &1.0090e-05_rb,1.2899e-05_rb,1.1584e-05_rb,8.7849e-06_rb,8.1182e-07_rb/)
      kbo(:, 5,45, 3) = (/ &
     &1.5974e-05_rb,1.9436e-05_rb,1.6972e-05_rb,1.2481e-05_rb,9.9260e-07_rb/)
      kbo(:, 1,46, 3) = (/ &
     &1.4538e-06_rb,2.2381e-06_rb,2.2044e-06_rb,1.8570e-06_rb,2.7780e-07_rb/)
      kbo(:, 2,46, 3) = (/ &
     &2.5054e-06_rb,3.6761e-06_rb,3.5333e-06_rb,2.8803e-06_rb,3.6666e-07_rb/)
      kbo(:, 3,46, 3) = (/ &
     &4.3205e-06_rb,5.9430e-06_rb,5.5451e-06_rb,4.3709e-06_rb,4.7052e-07_rb/)
      kbo(:, 4,46, 3) = (/ &
     &7.2299e-06_rb,9.3616e-06_rb,8.4601e-06_rb,6.4628e-06_rb,5.9539e-07_rb/)
      kbo(:, 5,46, 3) = (/ &
     &1.1621e-05_rb,1.4285e-05_rb,1.2535e-05_rb,9.2836e-06_rb,7.3565e-07_rb/)
      kbo(:, 1,47, 3) = (/ &
     &1.0143e-06_rb,1.5538e-06_rb,1.5325e-06_rb,1.2979e-06_rb,1.9733e-07_rb/)
      kbo(:, 2,47, 3) = (/ &
     &1.7208e-06_rb,2.5538e-06_rb,2.4694e-06_rb,2.0305e-06_rb,2.6304e-07_rb/)
      kbo(:, 3,47, 3) = (/ &
     &2.9828e-06_rb,4.1702e-06_rb,3.9202e-06_rb,3.1167e-06_rb,3.4065e-07_rb/)
      kbo(:, 4,47, 3) = (/ &
     &5.0644e-06_rb,6.6541e-06_rb,6.0593e-06_rb,4.6660e-06_rb,4.3326e-07_rb/)
      kbo(:, 5,47, 3) = (/ &
     &8.2782e-06_rb,1.0300e-05_rb,9.0956e-06_rb,6.7879e-06_rb,5.4017e-07_rb/)
      kbo(:, 1,48, 3) = (/ &
     &7.2030e-07_rb,1.0845e-06_rb,1.0676e-06_rb,9.0611e-07_rb,1.3761e-07_rb/)
      kbo(:, 2,48, 3) = (/ &
     &1.1872e-06_rb,1.7730e-06_rb,1.7228e-06_rb,1.4271e-06_rb,1.8706e-07_rb/)
      kbo(:, 3,48, 3) = (/ &
     &2.0561e-06_rb,2.9161e-06_rb,2.7609e-06_rb,2.2135e-06_rb,2.4544e-07_rb/)
      kbo(:, 4,48, 3) = (/ &
     &3.5327e-06_rb,4.7102e-06_rb,4.3223e-06_rb,3.3540e-06_rb,3.1231e-07_rb/)
      kbo(:, 5,48, 3) = (/ &
     &5.8701e-06_rb,7.3943e-06_rb,6.5729e-06_rb,4.9432e-06_rb,3.9364e-07_rb/)
      kbo(:, 1,49, 3) = (/ &
     &5.2463e-07_rb,7.6406e-07_rb,7.4789e-07_rb,6.3224e-07_rb,9.4269e-08_rb/)
      kbo(:, 2,49, 3) = (/ &
     &8.2733e-07_rb,1.2326e-06_rb,1.2007e-06_rb,1.0005e-06_rb,1.3092e-07_rb/)
      kbo(:, 3,49, 3) = (/ &
     &1.4174e-06_rb,2.0342e-06_rb,1.9376e-06_rb,1.5657e-06_rb,1.7460e-07_rb/)
      kbo(:, 4,49, 3) = (/ &
     &2.4552e-06_rb,3.3221e-06_rb,3.0702e-06_rb,2.4004e-06_rb,2.2553e-07_rb/)
      kbo(:, 5,49, 3) = (/ &
     &4.1423e-06_rb,5.2851e-06_rb,4.7305e-06_rb,3.5840e-06_rb,2.8424e-07_rb/)
      kbo(:, 1,50, 3) = (/ &
     &3.9659e-07_rb,5.5178e-07_rb,5.3440e-07_rb,4.4722e-07_rb,6.5930e-08_rb/)
      kbo(:, 2,50, 3) = (/ &
     &5.9210e-07_rb,8.7140e-07_rb,8.4803e-07_rb,7.0817e-07_rb,9.3562e-08_rb/)
      kbo(:, 3,50, 3) = (/ &
     &9.9396e-07_rb,1.4345e-06_rb,1.3722e-06_rb,1.1160e-06_rb,1.2619e-07_rb/)
      kbo(:, 4,50, 3) = (/ &
     &1.7258e-06_rb,2.3624e-06_rb,2.1973e-06_rb,1.7292e-06_rb,1.6444e-07_rb/)
      kbo(:, 5,50, 3) = (/ &
     &2.9480e-06_rb,3.8052e-06_rb,3.4270e-06_rb,2.6131e-06_rb,2.0855e-07_rb/)
      kbo(:, 1,51, 3) = (/ &
     &3.0780e-07_rb,4.0724e-07_rb,3.8772e-07_rb,3.1909e-07_rb,4.7152e-08_rb/)
      kbo(:, 2,51, 3) = (/ &
     &4.3389e-07_rb,6.2376e-07_rb,6.0442e-07_rb,5.0387e-07_rb,6.6848e-08_rb/)
      kbo(:, 3,51, 3) = (/ &
     &7.0578e-07_rb,1.0183e-06_rb,9.7661e-07_rb,7.9773e-07_rb,9.1673e-08_rb/)
      kbo(:, 4,51, 3) = (/ &
     &1.2202e-06_rb,1.6857e-06_rb,1.5762e-06_rb,1.2481e-06_rb,1.2080e-07_rb/)
      kbo(:, 5,51, 3) = (/ &
     &2.1050e-06_rb,2.7463e-06_rb,2.4878e-06_rb,1.9078e-06_rb,1.5488e-07_rb/)
      kbo(:, 1,52, 3) = (/ &
     &2.4467e-07_rb,3.0672e-07_rb,2.8540e-07_rb,2.2927e-07_rb,3.3766e-08_rb/)
      kbo(:, 2,52, 3) = (/ &
     &3.2577e-07_rb,4.5139e-07_rb,4.3382e-07_rb,3.5906e-07_rb,4.7409e-08_rb/)
      kbo(:, 3,52, 3) = (/ &
     &5.0672e-07_rb,7.2555e-07_rb,6.9595e-07_rb,5.7001e-07_rb,6.5991e-08_rb/)
      kbo(:, 4,52, 3) = (/ &
     &8.6516e-07_rb,1.2018e-06_rb,1.1289e-06_rb,8.9884e-07_rb,8.8646e-08_rb/)
      kbo(:, 5,52, 3) = (/ &
     &1.5011e-06_rb,1.9777e-06_rb,1.8012e-06_rb,1.3890e-06_rb,1.1411e-07_rb/)
      kbo(:, 1,53, 3) = (/ &
     &1.9893e-07_rb,2.3655e-07_rb,2.1398e-07_rb,1.6650e-07_rb,2.3697e-08_rb/)
      kbo(:, 2,53, 3) = (/ &
     &2.5083e-07_rb,3.3159e-07_rb,3.1422e-07_rb,2.5680e-07_rb,3.3792e-08_rb/)
      kbo(:, 3,53, 3) = (/ &
     &3.6925e-07_rb,5.1963e-07_rb,4.9710e-07_rb,4.0705e-07_rb,4.7057e-08_rb/)
      kbo(:, 4,53, 3) = (/ &
     &6.1624e-07_rb,8.5716e-07_rb,8.0760e-07_rb,6.4558e-07_rb,6.4327e-08_rb/)
      kbo(:, 5,53, 3) = (/ &
     &1.0701e-06_rb,1.4207e-06_rb,1.3006e-06_rb,1.0081e-06_rb,8.3703e-08_rb/)
      kbo(:, 1,54, 3) = (/ &
     &1.6569e-07_rb,1.8826e-07_rb,1.6516e-07_rb,1.2463e-07_rb,1.7341e-08_rb/)
      kbo(:, 2,54, 3) = (/ &
     &1.9916e-07_rb,2.5095e-07_rb,2.3323e-07_rb,1.8716e-07_rb,2.4743e-08_rb/)
      kbo(:, 3,54, 3) = (/ &
     &2.7800e-07_rb,3.8083e-07_rb,3.6198e-07_rb,2.9489e-07_rb,3.4593e-08_rb/)
      kbo(:, 4,54, 3) = (/ &
     &4.4953e-07_rb,6.2218e-07_rb,5.8637e-07_rb,4.6947e-07_rb,4.7657e-08_rb/)
      kbo(:, 5,54, 3) = (/ &
     &7.7653e-07_rb,1.0348e-06_rb,9.5106e-07_rb,7.3964e-07_rb,6.2914e-08_rb/)
      kbo(:, 1,55, 3) = (/ &
     &1.4043e-07_rb,1.5348e-07_rb,1.3050e-07_rb,9.5790e-08_rb,1.3021e-08_rb/)
      kbo(:, 2,55, 3) = (/ &
     &1.6195e-07_rb,1.9454e-07_rb,1.7661e-07_rb,1.3842e-07_rb,1.8310e-08_rb/)
      kbo(:, 3,55, 3) = (/ &
     &2.1498e-07_rb,2.8400e-07_rb,2.6714e-07_rb,2.1561e-07_rb,2.5769e-08_rb/)
      kbo(:, 4,55, 3) = (/ &
     &3.3384e-07_rb,4.5663e-07_rb,4.2946e-07_rb,3.4367e-07_rb,3.5731e-08_rb/)
      kbo(:, 5,55, 3) = (/ &
     &5.6980e-07_rb,7.5956e-07_rb,6.9954e-07_rb,5.4544e-07_rb,4.7978e-08_rb/)
      kbo(:, 1,56, 3) = (/ &
     &1.2086e-07_rb,1.2723e-07_rb,1.0539e-07_rb,7.4446e-08_rb,9.2261e-09_rb/)
      kbo(:, 2,56, 3) = (/ &
     &1.3434e-07_rb,1.5407e-07_rb,1.3616e-07_rb,1.0396e-07_rb,1.3447e-08_rb/)
      kbo(:, 3,56, 3) = (/ &
     &1.7000e-07_rb,2.1506e-07_rb,1.9916e-07_rb,1.5861e-07_rb,1.9233e-08_rb/)
      kbo(:, 4,56, 3) = (/ &
     &2.5155e-07_rb,3.3746e-07_rb,3.1592e-07_rb,2.5204e-07_rb,2.6859e-08_rb/)
      kbo(:, 5,56, 3) = (/ &
     &4.2088e-07_rb,5.5907e-07_rb,5.1525e-07_rb,4.0220e-07_rb,3.6549e-08_rb/)
      kbo(:, 1,57, 3) = (/ &
     &1.0553e-07_rb,1.0707e-07_rb,8.7153e-08_rb,5.8944e-08_rb,6.8033e-09_rb/)
      kbo(:, 2,57, 3) = (/ &
     &1.1362e-07_rb,1.2487e-07_rb,1.0708e-07_rb,7.9506e-08_rb,9.8943e-09_rb/)
      kbo(:, 3,57, 3) = (/ &
     &1.3729e-07_rb,1.6577e-07_rb,1.5039e-07_rb,1.1755e-07_rb,1.4310e-08_rb/)
      kbo(:, 4,57, 3) = (/ &
     &1.9303e-07_rb,2.5185e-07_rb,2.3386e-07_rb,1.8533e-07_rb,2.0155e-08_rb/)
      kbo(:, 5,57, 3) = (/ &
     &3.1378e-07_rb,4.1313e-07_rb,3.8025e-07_rb,2.9656e-07_rb,2.7619e-08_rb/)
      kbo(:, 1,58, 3) = (/ &
     &4.5186e-08_rb,5.0890e-08_rb,4.7832e-08_rb,3.7472e-08_rb,5.1696e-09_rb/)
      kbo(:, 2,58, 3) = (/ &
     &4.7401e-08_rb,5.7618e-08_rb,5.6357e-08_rb,4.8955e-08_rb,7.4243e-09_rb/)
      kbo(:, 3,58, 3) = (/ &
     &5.5010e-08_rb,7.2882e-08_rb,7.5797e-08_rb,7.0136e-08_rb,1.0687e-08_rb/)
      kbo(:, 4,58, 3) = (/ &
     &7.3718e-08_rb,1.0653e-07_rb,1.1470e-07_rb,1.0884e-07_rb,1.5198e-08_rb/)
      kbo(:, 5,58, 3) = (/ &
     &1.1547e-07_rb,1.7203e-07_rb,1.8517e-07_rb,1.7422e-07_rb,2.1153e-08_rb/)
      kbo(:, 1,59, 3) = (/ &
     &3.8720e-08_rb,4.3027e-08_rb,4.0190e-08_rb,3.1252e-08_rb,4.1397e-09_rb/)
      kbo(:, 2,59, 3) = (/ &
     &4.0213e-08_rb,4.8103e-08_rb,4.6638e-08_rb,4.0062e-08_rb,6.0380e-09_rb/)
      kbo(:, 3,59, 3) = (/ &
     &4.5994e-08_rb,5.9694e-08_rb,6.1452e-08_rb,5.6505e-08_rb,8.8443e-09_rb/)
      kbo(:, 4,59, 3) = (/ &
     &6.0487e-08_rb,8.5685e-08_rb,9.1743e-08_rb,8.6818e-08_rb,1.2879e-08_rb/)
      kbo(:, 5,59, 3) = (/ &
     &9.3188e-08_rb,1.3722e-07_rb,1.4752e-07_rb,1.3892e-07_rb,1.7999e-08_rb/)
      kbo(:, 1,13, 4) = (/ &
     &4.3869e-03_rb,5.0989e-03_rb,4.3794e-03_rb,3.0521e-03_rb,9.4784e-04_rb/)
      kbo(:, 2,13, 4) = (/ &
     &6.9481e-03_rb,7.8458e-03_rb,6.6078e-03_rb,4.5663e-03_rb,1.0961e-03_rb/)
      kbo(:, 3,13, 4) = (/ &
     &1.1023e-02_rb,1.1980e-02_rb,9.8280e-03_rb,6.7431e-03_rb,1.2240e-03_rb/)
      kbo(:, 4,13, 4) = (/ &
     &1.7316e-02_rb,1.8007e-02_rb,1.4661e-02_rb,9.8568e-03_rb,1.4326e-03_rb/)
      kbo(:, 5,13, 4) = (/ &
     &2.6628e-02_rb,2.6632e-02_rb,2.1301e-02_rb,1.4107e-02_rb,1.6715e-03_rb/)
      kbo(:, 1,14, 4) = (/ &
     &3.8856e-03_rb,4.5615e-03_rb,3.8907e-03_rb,2.6924e-03_rb,7.6763e-04_rb/)
      kbo(:, 2,14, 4) = (/ &
     &6.2253e-03_rb,7.0528e-03_rb,5.8968e-03_rb,4.0418e-03_rb,8.6051e-04_rb/)
      kbo(:, 3,14, 4) = (/ &
     &1.0002e-02_rb,1.0752e-02_rb,8.8582e-03_rb,6.0065e-03_rb,1.0065e-03_rb/)
      kbo(:, 4,14, 4) = (/ &
     &1.5699e-02_rb,1.6229e-02_rb,1.3163e-02_rb,8.7970e-03_rb,1.2227e-03_rb/)
      kbo(:, 5,14, 4) = (/ &
     &2.4269e-02_rb,2.4030e-02_rb,1.9137e-02_rb,1.2536e-02_rb,1.4474e-03_rb/)
      kbo(:, 1,15, 4) = (/ &
     &3.2659e-03_rb,3.8770e-03_rb,3.3101e-03_rb,2.3048e-03_rb,5.7573e-04_rb/)
      kbo(:, 2,15, 4) = (/ &
     &5.3241e-03_rb,6.0372e-03_rb,5.0494e-03_rb,3.4916e-03_rb,6.8321e-04_rb/)
      kbo(:, 3,15, 4) = (/ &
     &8.6002e-03_rb,9.2086e-03_rb,7.6334e-03_rb,5.1974e-03_rb,8.4736e-04_rb/)
      kbo(:, 4,15, 4) = (/ &
     &1.3547e-02_rb,1.3992e-02_rb,1.1347e-02_rb,7.5834e-03_rb,1.0287e-03_rb/)
      kbo(:, 5,15, 4) = (/ &
     &2.1009e-02_rb,2.0762e-02_rb,1.6574e-02_rb,1.0811e-02_rb,1.2066e-03_rb/)
      kbo(:, 1,16, 4) = (/ &
     &2.6697e-03_rb,3.2234e-03_rb,2.7529e-03_rb,1.9451e-03_rb,4.4184e-04_rb/)
      kbo(:, 2,16, 4) = (/ &
     &4.4186e-03_rb,5.0347e-03_rb,4.2523e-03_rb,2.9486e-03_rb,5.5826e-04_rb/)
      kbo(:, 3,16, 4) = (/ &
     &7.1546e-03_rb,7.7280e-03_rb,6.4040e-03_rb,4.4001e-03_rb,6.8881e-04_rb/)
      kbo(:, 4,16, 4) = (/ &
     &1.1406e-02_rb,1.1811e-02_rb,9.5855e-03_rb,6.4241e-03_rb,8.3295e-04_rb/)
      kbo(:, 5,16, 4) = (/ &
     &1.7781e-02_rb,1.7577e-02_rb,1.4001e-02_rb,9.1645e-03_rb,9.7098e-04_rb/)
      kbo(:, 1,17, 4) = (/ &
     &2.1896e-03_rb,2.6538e-03_rb,2.2904e-03_rb,1.6414e-03_rb,3.5480e-04_rb/)
      kbo(:, 2,17, 4) = (/ &
     &3.6444e-03_rb,4.1897e-03_rb,3.5440e-03_rb,2.4805e-03_rb,4.4971e-04_rb/)
      kbo(:, 3,17, 4) = (/ &
     &5.9916e-03_rb,6.4686e-03_rb,5.3743e-03_rb,3.6974e-03_rb,5.5799e-04_rb/)
      kbo(:, 4,17, 4) = (/ &
     &9.5963e-03_rb,9.9128e-03_rb,8.0553e-03_rb,5.4068e-03_rb,6.7241e-04_rb/)
      kbo(:, 5,17, 4) = (/ &
     &1.5025e-02_rb,1.4809e-02_rb,1.1810e-02_rb,7.7160e-03_rb,7.7063e-04_rb/)
      kbo(:, 1,18, 4) = (/ &
     &1.7997e-03_rb,2.1886e-03_rb,1.9082e-03_rb,1.3670e-03_rb,2.9021e-04_rb/)
      kbo(:, 2,18, 4) = (/ &
     &3.0120e-03_rb,3.4746e-03_rb,2.9450e-03_rb,2.0706e-03_rb,3.7282e-04_rb/)
      kbo(:, 3,18, 4) = (/ &
     &4.9798e-03_rb,5.3990e-03_rb,4.4938e-03_rb,3.0816e-03_rb,4.5930e-04_rb/)
      kbo(:, 4,18, 4) = (/ &
     &8.0081e-03_rb,8.2644e-03_rb,6.7223e-03_rb,4.5032e-03_rb,5.4734e-04_rb/)
      kbo(:, 5,18, 4) = (/ &
     &1.2492e-02_rb,1.2373e-02_rb,9.8644e-03_rb,6.4255e-03_rb,6.3020e-04_rb/)
      kbo(:, 1,19, 4) = (/ &
     &1.5064e-03_rb,1.8451e-03_rb,1.6056e-03_rb,1.1522e-03_rb,2.4682e-04_rb/)
      kbo(:, 2,19, 4) = (/ &
     &2.5474e-03_rb,2.9294e-03_rb,2.4915e-03_rb,1.7507e-03_rb,3.1538e-04_rb/)
      kbo(:, 3,19, 4) = (/ &
     &4.2186e-03_rb,4.5798e-03_rb,3.8063e-03_rb,2.5988e-03_rb,3.8674e-04_rb/)
      kbo(:, 4,19, 4) = (/ &
     &6.8019e-03_rb,7.0063e-03_rb,5.6982e-03_rb,3.8032e-03_rb,4.6124e-04_rb/)
      kbo(:, 5,19, 4) = (/ &
     &1.0595e-02_rb,1.0486e-02_rb,8.3528e-03_rb,5.4249e-03_rb,5.3320e-04_rb/)
      kbo(:, 1,20, 4) = (/ &
     &1.3102e-03_rb,1.5983e-03_rb,1.3859e-03_rb,9.9291e-04_rb,2.1228e-04_rb/)
      kbo(:, 2,20, 4) = (/ &
     &2.2269e-03_rb,2.5410e-03_rb,2.1563e-03_rb,1.5072e-03_rb,2.6816e-04_rb/)
      kbo(:, 3,20, 4) = (/ &
     &3.6896e-03_rb,3.9778e-03_rb,3.2977e-03_rb,2.2454e-03_rb,3.2819e-04_rb/)
      kbo(:, 4,20, 4) = (/ &
     &5.9414e-03_rb,6.0960e-03_rb,4.9353e-03_rb,3.2744e-03_rb,3.9225e-04_rb/)
      kbo(:, 5,20, 4) = (/ &
     &9.2578e-03_rb,9.1102e-03_rb,7.2255e-03_rb,4.6712e-03_rb,4.5336e-04_rb/)
      kbo(:, 1,21, 4) = (/ &
     &1.1473e-03_rb,1.3888e-03_rb,1.2016e-03_rb,8.6011e-04_rb,1.8127e-04_rb/)
      kbo(:, 2,21, 4) = (/ &
     &1.9565e-03_rb,2.2181e-03_rb,1.8746e-03_rb,1.3038e-03_rb,2.2830e-04_rb/)
      kbo(:, 3,21, 4) = (/ &
     &3.2520e-03_rb,3.4808e-03_rb,2.8694e-03_rb,1.9461e-03_rb,2.7800e-04_rb/)
      kbo(:, 4,21, 4) = (/ &
     &5.2204e-03_rb,5.3317e-03_rb,4.2972e-03_rb,2.8342e-03_rb,3.3105e-04_rb/)
      kbo(:, 5,21, 4) = (/ &
     &8.1069e-03_rb,7.9508e-03_rb,6.2747e-03_rb,4.0360e-03_rb,3.8585e-04_rb/)
      kbo(:, 1,22, 4) = (/ &
     &1.0382e-03_rb,1.2396e-03_rb,1.0678e-03_rb,7.6273e-04_rb,1.5466e-04_rb/)
      kbo(:, 2,22, 4) = (/ &
     &1.7720e-03_rb,1.9893e-03_rb,1.6695e-03_rb,1.1556e-03_rb,1.9356e-04_rb/)
      kbo(:, 3,22, 4) = (/ &
     &2.9369e-03_rb,3.1200e-03_rb,2.5557e-03_rb,1.7237e-03_rb,2.3624e-04_rb/)
      kbo(:, 4,22, 4) = (/ &
     &4.6992e-03_rb,4.7636e-03_rb,3.8178e-03_rb,2.5063e-03_rb,2.7855e-04_rb/)
      kbo(:, 5,22, 4) = (/ &
     &7.2282e-03_rb,7.0542e-03_rb,5.5487e-03_rb,3.5490e-03_rb,3.2630e-04_rb/)
      kbo(:, 1,23, 4) = (/ &
     &9.4131e-04_rb,1.1095e-03_rb,9.5148e-04_rb,6.7660e-04_rb,1.3244e-04_rb/)
      kbo(:, 2,23, 4) = (/ &
     &1.6035e-03_rb,1.7844e-03_rb,1.4894e-03_rb,1.0262e-03_rb,1.6442e-04_rb/)
      kbo(:, 3,23, 4) = (/ &
     &2.6522e-03_rb,2.7950e-03_rb,2.2775e-03_rb,1.5271e-03_rb,2.0008e-04_rb/)
      kbo(:, 4,23, 4) = (/ &
     &4.2106e-03_rb,4.2402e-03_rb,3.3871e-03_rb,2.2131e-03_rb,2.3445e-04_rb/)
      kbo(:, 5,23, 4) = (/ &
     &6.4196e-03_rb,6.2427e-03_rb,4.8940e-03_rb,3.1130e-03_rb,2.7496e-04_rb/)
      kbo(:, 1,24, 4) = (/ &
     &8.5493e-04_rb,1.0008e-03_rb,8.5274e-04_rb,6.0213e-04_rb,1.1224e-04_rb/)
      kbo(:, 2,24, 4) = (/ &
     &1.4576e-03_rb,1.6070e-03_rb,1.3354e-03_rb,9.1642e-04_rb,1.3965e-04_rb/)
      kbo(:, 3,24, 4) = (/ &
     &2.4005e-03_rb,2.5083e-03_rb,2.0375e-03_rb,1.3606e-03_rb,1.6869e-04_rb/)
      kbo(:, 4,24, 4) = (/ &
     &3.7805e-03_rb,3.7882e-03_rb,3.0168e-03_rb,1.9590e-03_rb,1.9824e-04_rb/)
      kbo(:, 5,24, 4) = (/ &
     &5.7340e-03_rb,5.5429e-03_rb,4.3252e-03_rb,2.7397e-03_rb,2.2899e-04_rb/)
      kbo(:, 1,25, 4) = (/ &
     &7.8083e-04_rb,9.0613e-04_rb,7.6855e-04_rb,5.3955e-04_rb,9.3397e-05_rb/)
      kbo(:, 2,25, 4) = (/ &
     &1.3293e-03_rb,1.4516e-03_rb,1.2021e-03_rb,8.2125e-04_rb,1.1644e-04_rb/)
      kbo(:, 3,25, 4) = (/ &
     &2.1730e-03_rb,2.2546e-03_rb,1.8276e-03_rb,1.2136e-03_rb,1.4039e-04_rb/)
      kbo(:, 4,25, 4) = (/ &
     &3.3994e-03_rb,3.3901e-03_rb,2.6854e-03_rb,1.7374e-03_rb,1.6529e-04_rb/)
      kbo(:, 5,25, 4) = (/ &
     &5.1177e-03_rb,4.9296e-03_rb,3.8308e-03_rb,2.4138e-03_rb,1.9292e-04_rb/)
      kbo(:, 1,26, 4) = (/ &
     &7.2414e-04_rb,8.2998e-04_rb,6.9991e-04_rb,4.8962e-04_rb,7.9259e-05_rb/)
      kbo(:, 2,26, 4) = (/ &
     &1.2259e-03_rb,1.3255e-03_rb,1.0917e-03_rb,7.4177e-04_rb,9.7580e-05_rb/)
      kbo(:, 3,26, 4) = (/ &
     &1.9848e-03_rb,2.0462e-03_rb,1.6503e-03_rb,1.0895e-03_rb,1.1629e-04_rb/)
      kbo(:, 4,26, 4) = (/ &
     &3.0863e-03_rb,3.0554e-03_rb,2.4123e-03_rb,1.5502e-03_rb,1.3732e-04_rb/)
      kbo(:, 5,26, 4) = (/ &
     &4.5999e-03_rb,4.4081e-03_rb,3.4159e-03_rb,2.1402e-03_rb,1.6386e-04_rb/)
      kbo(:, 1,27, 4) = (/ &
     &6.7040e-04_rb,7.6049e-04_rb,6.3931e-04_rb,4.4424e-04_rb,6.5890e-05_rb/)
      kbo(:, 2,27, 4) = (/ &
     &1.1282e-03_rb,1.2096e-03_rb,9.9105e-04_rb,6.6941e-04_rb,8.0876e-05_rb/)
      kbo(:, 3,27, 4) = (/ &
     &1.8147e-03_rb,1.8556e-03_rb,1.4889e-03_rb,9.7714e-04_rb,9.6344e-05_rb/)
      kbo(:, 4,27, 4) = (/ &
     &2.7940e-03_rb,2.7516e-03_rb,2.1627e-03_rb,1.3811e-03_rb,1.1546e-04_rb/)
      kbo(:, 5,27, 4) = (/ &
     &4.1206e-03_rb,3.9340e-03_rb,3.0363e-03_rb,1.8956e-03_rb,1.3791e-04_rb/)
      kbo(:, 1,28, 4) = (/ &
     &6.1963e-04_rb,6.9617e-04_rb,5.8205e-04_rb,4.0239e-04_rb,5.3842e-05_rb/)
      kbo(:, 2,28, 4) = (/ &
     &1.0354e-03_rb,1.1013e-03_rb,8.9815e-04_rb,6.0298e-04_rb,6.6022e-05_rb/)
      kbo(:, 3,28, 4) = (/ &
     &1.6516e-03_rb,1.6797e-03_rb,1.3412e-03_rb,8.7507e-04_rb,8.0123e-05_rb/)
      kbo(:, 4,28, 4) = (/ &
     &2.5197e-03_rb,2.4709e-03_rb,1.9338e-03_rb,1.2274e-03_rb,9.7074e-05_rb/)
      kbo(:, 5,28, 4) = (/ &
     &3.6744e-03_rb,3.4958e-03_rb,2.6900e-03_rb,1.6731e-03_rb,1.1603e-04_rb/)
      kbo(:, 1,29, 4) = (/ &
     &5.7255e-04_rb,6.3719e-04_rb,5.3001e-04_rb,3.6415e-04_rb,4.3783e-05_rb/)
      kbo(:, 2,29, 4) = (/ &
     &9.4899e-04_rb,1.0006e-03_rb,8.1341e-04_rb,5.4243e-04_rb,5.3930e-05_rb/)
      kbo(:, 3,29, 4) = (/ &
     &1.4973e-03_rb,1.5143e-03_rb,1.2048e-03_rb,7.8083e-04_rb,6.6803e-05_rb/)
      kbo(:, 4,29, 4) = (/ &
     &2.2572e-03_rb,2.2044e-03_rb,1.7199e-03_rb,1.0861e-03_rb,8.0985e-05_rb/)
      kbo(:, 5,29, 4) = (/ &
     &3.2527e-03_rb,3.0849e-03_rb,2.3685e-03_rb,1.4686e-03_rb,9.6201e-05_rb/)
      kbo(:, 1,30, 4) = (/ &
     &5.2637e-04_rb,5.8081e-04_rb,4.8010e-04_rb,3.2816e-04_rb,3.5536e-05_rb/)
      kbo(:, 2,30, 4) = (/ &
     &8.6289e-04_rb,9.0357e-04_rb,7.3215e-04_rb,4.8521e-04_rb,4.4824e-05_rb/)
      kbo(:, 3,30, 4) = (/ &
     &1.3456e-03_rb,1.3553e-03_rb,1.0747e-03_rb,6.9223e-04_rb,5.5319e-05_rb/)
      kbo(:, 4,30, 4) = (/ &
     &2.0065e-03_rb,1.9507e-03_rb,1.5172e-03_rb,9.5518e-04_rb,6.6828e-05_rb/)
      kbo(:, 5,30, 4) = (/ &
     &2.8573e-03_rb,2.7008e-03_rb,2.0712e-03_rb,1.2802e-03_rb,7.9313e-05_rb/)
      kbo(:, 1,31, 4) = (/ &
     &4.7945e-04_rb,5.2447e-04_rb,4.3249e-04_rb,2.9383e-04_rb,2.9390e-05_rb/)
      kbo(:, 2,31, 4) = (/ &
     &7.7587e-04_rb,8.0972e-04_rb,6.5328e-04_rb,4.3065e-04_rb,3.6906e-05_rb/)
      kbo(:, 3,31, 4) = (/ &
     &1.1974e-03_rb,1.2004e-03_rb,9.4868e-04_rb,6.0827e-04_rb,4.5016e-05_rb/)
      kbo(:, 4,31, 4) = (/ &
     &1.7617e-03_rb,1.7074e-03_rb,1.3255e-03_rb,8.3172e-04_rb,5.4252e-05_rb/)
      kbo(:, 5,31, 4) = (/ &
     &2.4825e-03_rb,2.3423e-03_rb,1.7933e-03_rb,1.1061e-03_rb,6.4138e-05_rb/)
      kbo(:, 1,32, 4) = (/ &
     &4.3738e-04_rb,4.7509e-04_rb,3.9041e-04_rb,2.6369e-04_rb,2.4019e-05_rb/)
      kbo(:, 2,32, 4) = (/ &
     &6.9937e-04_rb,7.2599e-04_rb,5.8354e-04_rb,3.8245e-04_rb,2.9892e-05_rb/)
      kbo(:, 3,32, 4) = (/ &
     &1.0660e-03_rb,1.0639e-03_rb,8.3776e-04_rb,5.3497e-04_rb,3.6437e-05_rb/)
      kbo(:, 4,32, 4) = (/ &
     &1.5490e-03_rb,1.4965e-03_rb,1.1588e-03_rb,7.2497e-04_rb,4.3698e-05_rb/)
      kbo(:, 5,32, 4) = (/ &
     &2.1609e-03_rb,2.0352e-03_rb,1.5553e-03_rb,9.5666e-04_rb,5.1532e-05_rb/)
      kbo(:, 1,33, 4) = (/ &
     &3.9889e-04_rb,4.2999e-04_rb,3.5197e-04_rb,2.3634e-04_rb,1.9311e-05_rb/)
      kbo(:, 2,33, 4) = (/ &
     &6.2991e-04_rb,6.5021e-04_rb,5.2059e-04_rb,3.3932e-04_rb,2.4117e-05_rb/)
      kbo(:, 3,33, 4) = (/ &
     &9.4755e-04_rb,9.4129e-04_rb,7.3904e-04_rb,4.6991e-04_rb,2.9354e-05_rb/)
      kbo(:, 4,33, 4) = (/ &
     &1.3617e-03_rb,1.3113e-03_rb,1.0129e-03_rb,6.3180e-04_rb,3.5029e-05_rb/)
      kbo(:, 5,33, 4) = (/ &
     &1.8805e-03_rb,1.7688e-03_rb,1.3485e-03_rb,8.2760e-04_rb,4.1309e-05_rb/)
      kbo(:, 1,34, 4) = (/ &
     &3.6214e-04_rb,3.8716e-04_rb,3.1549e-04_rb,2.1059e-04_rb,1.5692e-05_rb/)
      kbo(:, 2,34, 4) = (/ &
     &5.6474e-04_rb,5.7963e-04_rb,4.6204e-04_rb,2.9966e-04_rb,1.9450e-05_rb/)
      kbo(:, 3,34, 4) = (/ &
     &8.4034e-04_rb,8.3067e-04_rb,6.4971e-04_rb,4.1144e-04_rb,2.3693e-05_rb/)
      kbo(:, 4,34, 4) = (/ &
     &1.1957e-03_rb,1.1475e-03_rb,8.8389e-04_rb,5.4962e-04_rb,2.8253e-05_rb/)
      kbo(:, 5,34, 4) = (/ &
     &1.6380e-03_rb,1.5373e-03_rb,1.1695e-03_rb,7.1576e-04_rb,3.2957e-05_rb/)
      kbo(:, 1,35, 4) = (/ &
     &3.1359e-04_rb,3.3446e-04_rb,2.7193e-04_rb,1.8091e-04_rb,1.2437e-05_rb/)
      kbo(:, 2,35, 4) = (/ &
     &4.8583e-04_rb,4.9752e-04_rb,3.9566e-04_rb,2.5593e-04_rb,1.5305e-05_rb/)
      kbo(:, 3,35, 4) = (/ &
     &7.1875e-04_rb,7.0909e-04_rb,5.5358e-04_rb,3.4993e-04_rb,1.8635e-05_rb/)
      kbo(:, 4,35, 4) = (/ &
     &1.0171e-03_rb,9.7513e-04_rb,7.5023e-04_rb,4.6569e-04_rb,2.2205e-05_rb/)
      kbo(:, 5,35, 4) = (/ &
     &1.3872e-03_rb,1.3015e-03_rb,9.8912e-04_rb,6.0449e-04_rb,2.6045e-05_rb/)
      kbo(:, 1,36, 4) = (/ &
     &2.5808e-04_rb,2.7599e-04_rb,2.2440e-04_rb,1.4937e-04_rb,9.6631e-06_rb/)
      kbo(:, 2,36, 4) = (/ &
     &3.9964e-04_rb,4.1003e-04_rb,3.2602e-04_rb,2.1097e-04_rb,1.1834e-05_rb/)
      kbo(:, 3,36, 4) = (/ &
     &5.9092e-04_rb,5.8383e-04_rb,4.5590e-04_rb,2.8824e-04_rb,1.4353e-05_rb/)
      kbo(:, 4,36, 4) = (/ &
     &8.3595e-04_rb,8.0277e-04_rb,6.1763e-04_rb,3.8334e-04_rb,1.7189e-05_rb/)
      kbo(:, 5,36, 4) = (/ &
     &1.1396e-03_rb,1.0707e-03_rb,8.1375e-04_rb,4.9743e-04_rb,2.0326e-05_rb/)
      kbo(:, 1,37, 4) = (/ &
     &2.0121e-04_rb,2.1647e-04_rb,1.7647e-04_rb,1.1785e-04_rb,7.4310e-06_rb/)
      kbo(:, 2,37, 4) = (/ &
     &3.1330e-04_rb,3.2327e-04_rb,2.5768e-04_rb,1.6717e-04_rb,9.1199e-06_rb/)
      kbo(:, 3,37, 4) = (/ &
     &4.6580e-04_rb,4.6262e-04_rb,3.6181e-04_rb,2.2919e-04_rb,1.1050e-05_rb/)
      kbo(:, 4,37, 4) = (/ &
     &6.6212e-04_rb,6.3859e-04_rb,4.9190e-04_rb,3.0576e-04_rb,1.3280e-05_rb/)
      kbo(:, 5,37, 4) = (/ &
     &9.0672e-04_rb,8.5472e-04_rb,6.5027e-04_rb,3.9796e-04_rb,1.5719e-05_rb/)
      kbo(:, 1,38, 4) = (/ &
     &1.5610e-04_rb,1.6913e-04_rb,1.3823e-04_rb,9.2645e-05_rb,5.6716e-06_rb/)
      kbo(:, 2,38, 4) = (/ &
     &2.4467e-04_rb,2.5398e-04_rb,2.0295e-04_rb,1.3206e-04_rb,6.9811e-06_rb/)
      kbo(:, 3,38, 4) = (/ &
     &3.6591e-04_rb,3.6541e-04_rb,2.8633e-04_rb,1.8175e-04_rb,8.4645e-06_rb/)
      kbo(:, 4,38, 4) = (/ &
     &5.2302e-04_rb,5.0671e-04_rb,3.9077e-04_rb,2.4331e-04_rb,1.0160e-05_rb/)
      kbo(:, 5,38, 4) = (/ &
     &7.1953e-04_rb,6.8071e-04_rb,5.1841e-04_rb,3.1771e-04_rb,1.2087e-05_rb/)
      kbo(:, 1,39, 4) = (/ &
     &1.2096e-04_rb,1.3199e-04_rb,1.0815e-04_rb,7.2736e-05_rb,4.3132e-06_rb/)
      kbo(:, 2,39, 4) = (/ &
     &1.9086e-04_rb,1.9937e-04_rb,1.5970e-04_rb,1.0426e-04_rb,5.3181e-06_rb/)
      kbo(:, 3,39, 4) = (/ &
     &2.8720e-04_rb,2.8844e-04_rb,2.2641e-04_rb,1.4405e-04_rb,6.4566e-06_rb/)
      kbo(:, 4,39, 4) = (/ &
     &4.1286e-04_rb,4.0180e-04_rb,3.1020e-04_rb,1.9350e-04_rb,7.7296e-06_rb/)
      kbo(:, 5,39, 4) = (/ &
     &5.7063e-04_rb,5.4191e-04_rb,4.1317e-04_rb,2.5355e-04_rb,9.2083e-06_rb/)
      kbo(:, 1,40, 4) = (/ &
     &9.0120e-05_rb,9.9405e-05_rb,8.1774e-05_rb,5.5348e-05_rb,3.2802e-06_rb/)
      kbo(:, 2,40, 4) = (/ &
     &1.4396e-04_rb,1.5172e-04_rb,1.2199e-04_rb,8.0081e-05_rb,4.0450e-06_rb/)
      kbo(:, 3,40, 4) = (/ &
     &2.1897e-04_rb,2.2172e-04_rb,1.7453e-04_rb,1.1150e-04_rb,4.9391e-06_rb/)
      kbo(:, 4,40, 4) = (/ &
     &3.1818e-04_rb,3.1148e-04_rb,2.4096e-04_rb,1.5075e-04_rb,5.9260e-06_rb/)
      kbo(:, 5,40, 4) = (/ &
     &4.4338e-04_rb,4.2299e-04_rb,3.2304e-04_rb,1.9869e-04_rb,7.0429e-06_rb/)
      kbo(:, 1,41, 4) = (/ &
     &6.6586e-05_rb,7.4344e-05_rb,6.1391e-05_rb,4.1826e-05_rb,2.4789e-06_rb/)
      kbo(:, 2,41, 4) = (/ &
     &1.0786e-04_rb,1.1471e-04_rb,9.2627e-05_rb,6.1134e-05_rb,3.0831e-06_rb/)
      kbo(:, 3,41, 4) = (/ &
     &1.6588e-04_rb,1.6943e-04_rb,1.3383e-04_rb,8.5899e-05_rb,3.7712e-06_rb/)
      kbo(:, 4,41, 4) = (/ &
     &2.4383e-04_rb,2.4037e-04_rb,1.8645e-04_rb,1.1700e-04_rb,4.5478e-06_rb/)
      kbo(:, 5,41, 4) = (/ &
     &3.4300e-04_rb,3.2889e-04_rb,2.5168e-04_rb,1.5514e-04_rb,5.3910e-06_rb/)
      kbo(:, 1,42, 4) = (/ &
     &4.8998e-05_rb,5.5419e-05_rb,4.5947e-05_rb,3.1516e-05_rb,1.8667e-06_rb/)
      kbo(:, 2,42, 4) = (/ &
     &8.0484e-05_rb,8.6481e-05_rb,7.0103e-05_rb,4.6544e-05_rb,2.3354e-06_rb/)
      kbo(:, 3,42, 4) = (/ &
     &1.2531e-04_rb,1.2923e-04_rb,1.0238e-04_rb,6.6044e-05_rb,2.8708e-06_rb/)
      kbo(:, 4,42, 4) = (/ &
     &1.8633e-04_rb,1.8503e-04_rb,1.4397e-04_rb,9.0645e-05_rb,3.4606e-06_rb/)
      kbo(:, 5,42, 4) = (/ &
     &2.6476e-04_rb,2.5534e-04_rb,1.9578e-04_rb,1.2098e-04_rb,4.1084e-06_rb/)
      kbo(:, 1,43, 4) = (/ &
     &3.5242e-05_rb,4.0436e-05_rb,3.3714e-05_rb,2.3281e-05_rb,1.3919e-06_rb/)
      kbo(:, 2,43, 4) = (/ &
     &5.8897e-05_rb,6.4054e-05_rb,5.2168e-05_rb,3.4865e-05_rb,1.7548e-06_rb/)
      kbo(:, 3,43, 4) = (/ &
     &9.3170e-05_rb,9.7046e-05_rb,7.7251e-05_rb,5.0106e-05_rb,2.1668e-06_rb/)
      kbo(:, 4,43, 4) = (/ &
     &1.4048e-04_rb,1.4075e-04_rb,1.0986e-04_rb,6.9448e-05_rb,2.6074e-06_rb/)
      kbo(:, 5,43, 4) = (/ &
     &2.0220e-04_rb,1.9644e-04_rb,1.5089e-04_rb,9.3492e-05_rb,3.1278e-06_rb/)
      kbo(:, 1,44, 4) = (/ &
     &2.5004e-05_rb,2.9138e-05_rb,2.4437e-05_rb,1.6989e-05_rb,1.0250e-06_rb/)
      kbo(:, 2,44, 4) = (/ &
     &4.2585e-05_rb,4.6938e-05_rb,3.8422e-05_rb,2.5865e-05_rb,1.3112e-06_rb/)
      kbo(:, 3,44, 4) = (/ &
     &6.8563e-05_rb,7.2232e-05_rb,5.7761e-05_rb,3.7700e-05_rb,1.6186e-06_rb/)
      kbo(:, 4,44, 4) = (/ &
     &1.0501e-04_rb,1.0630e-04_rb,8.3253e-05_rb,5.2860e-05_rb,1.9651e-06_rb/)
      kbo(:, 5,44, 4) = (/ &
     &1.5343e-04_rb,1.5019e-04_rb,1.1567e-04_rb,7.1872e-05_rb,2.3710e-06_rb/)
      kbo(:, 1,45, 4) = (/ &
     &1.7648e-05_rb,2.0899e-05_rb,1.7624e-05_rb,1.2332e-05_rb,7.4778e-07_rb/)
      kbo(:, 2,45, 4) = (/ &
     &3.0648e-05_rb,3.4228e-05_rb,2.8145e-05_rb,1.9101e-05_rb,9.7150e-07_rb/)
      kbo(:, 3,45, 4) = (/ &
     &5.0244e-05_rb,5.3520e-05_rb,4.3001e-05_rb,2.8243e-05_rb,1.2073e-06_rb/)
      kbo(:, 4,45, 4) = (/ &
     &7.8191e-05_rb,7.9995e-05_rb,6.2888e-05_rb,4.0157e-05_rb,1.4831e-06_rb/)
      kbo(:, 5,45, 4) = (/ &
     &1.1605e-04_rb,1.1449e-04_rb,8.8448e-05_rb,5.5140e-05_rb,1.7858e-06_rb/)
      kbo(:, 1,46, 4) = (/ &
     &1.2241e-05_rb,1.4754e-05_rb,1.2528e-05_rb,8.8151e-06_rb,5.3848e-07_rb/)
      kbo(:, 2,46, 4) = (/ &
     &2.1731e-05_rb,2.4625e-05_rb,2.0352e-05_rb,1.3921e-05_rb,7.0792e-07_rb/)
      kbo(:, 3,46, 4) = (/ &
     &3.6328e-05_rb,3.9139e-05_rb,3.1626e-05_rb,2.0940e-05_rb,8.9555e-07_rb/)
      kbo(:, 4,46, 4) = (/ &
     &5.7531e-05_rb,5.9510e-05_rb,4.7024e-05_rb,3.0210e-05_rb,1.1090e-06_rb/)
      kbo(:, 5,46, 4) = (/ &
     &8.6821e-05_rb,8.6443e-05_rb,6.7046e-05_rb,4.1968e-05_rb,1.3451e-06_rb/)
      kbo(:, 1,47, 4) = (/ &
     &8.2449e-06_rb,1.0146e-05_rb,8.6807e-06_rb,6.1518e-06_rb,3.8332e-07_rb/)
      kbo(:, 2,47, 4) = (/ &
     &1.4999e-05_rb,1.7293e-05_rb,1.4384e-05_rb,9.9184e-06_rb,5.1069e-07_rb/)
      kbo(:, 3,47, 4) = (/ &
     &2.5663e-05_rb,2.8031e-05_rb,2.2776e-05_rb,1.5214e-05_rb,6.5507e-07_rb/)
      kbo(:, 4,47, 4) = (/ &
     &4.1483e-05_rb,4.3442e-05_rb,3.4514e-05_rb,2.2339e-05_rb,8.2140e-07_rb/)
      kbo(:, 5,47, 4) = (/ &
     &6.3807e-05_rb,6.4263e-05_rb,5.0034e-05_rb,3.1504e-05_rb,1.0057e-06_rb/)
      kbo(:, 1,48, 4) = (/ &
     &5.5161e-06_rb,6.9345e-06_rb,5.9711e-06_rb,4.2630e-06_rb,2.7090e-07_rb/)
      kbo(:, 2,48, 4) = (/ &
     &1.0274e-05_rb,1.2067e-05_rb,1.0101e-05_rb,7.0075e-06_rb,3.6468e-07_rb/)
      kbo(:, 3,48, 4) = (/ &
     &1.7999e-05_rb,1.9964e-05_rb,1.6291e-05_rb,1.0982e-05_rb,4.7593e-07_rb/)
      kbo(:, 4,48, 4) = (/ &
     &2.9748e-05_rb,3.1502e-05_rb,2.5173e-05_rb,1.6424e-05_rb,6.0510e-07_rb/)
      kbo(:, 5,48, 4) = (/ &
     &4.6627e-05_rb,4.7519e-05_rb,3.7159e-05_rb,2.3556e-05_rb,7.4697e-07_rb/)
      kbo(:, 1,49, 4) = (/ &
     &3.6712e-06_rb,4.7085e-06_rb,4.0821e-06_rb,2.9344e-06_rb,1.8902e-07_rb/)
      kbo(:, 2,49, 4) = (/ &
     &6.9808e-06_rb,8.3574e-06_rb,7.0495e-06_rb,4.9158e-06_rb,2.5862e-07_rb/)
      kbo(:, 3,49, 4) = (/ &
     &1.2533e-05_rb,1.4129e-05_rb,1.1598e-05_rb,7.8741e-06_rb,3.4164e-07_rb/)
      kbo(:, 4,49, 4) = (/ &
     &2.1199e-05_rb,2.2725e-05_rb,1.8253e-05_rb,1.2008e-05_rb,4.3895e-07_rb/)
      kbo(:, 5,49, 4) = (/ &
     &3.3893e-05_rb,3.4945e-05_rb,2.7461e-05_rb,1.7529e-05_rb,5.4985e-07_rb/)
      kbo(:, 1,50, 4) = (/ &
     &2.4835e-06_rb,3.2299e-06_rb,2.8168e-06_rb,2.0358e-06_rb,1.3342e-07_rb/)
      kbo(:, 2,50, 4) = (/ &
     &4.7895e-06_rb,5.8338e-06_rb,4.9516e-06_rb,3.4721e-06_rb,1.8475e-07_rb/)
      kbo(:, 3,50, 4) = (/ &
     &8.7933e-06_rb,1.0062e-05_rb,8.3077e-06_rb,5.6722e-06_rb,2.4761e-07_rb/)
      kbo(:, 4,50, 4) = (/ &
     &1.5202e-05_rb,1.6503e-05_rb,1.3305e-05_rb,8.8168e-06_rb,3.2240e-07_rb/)
      kbo(:, 5,50, 4) = (/ &
     &2.4800e-05_rb,2.5828e-05_rb,2.0397e-05_rb,1.3093e-05_rb,4.0814e-07_rb/)
      kbo(:, 1,51, 4) = (/ &
     &1.7011e-06_rb,2.2256e-06_rb,1.9504e-06_rb,1.4154e-06_rb,9.4900e-08_rb/)
      kbo(:, 2,51, 4) = (/ &
     &3.2953e-06_rb,4.0790e-06_rb,3.4794e-06_rb,2.4537e-06_rb,1.3273e-07_rb/)
      kbo(:, 3,51, 4) = (/ &
     &6.1746e-06_rb,7.1689e-06_rb,5.9544e-06_rb,4.0832e-06_rb,1.8014e-07_rb/)
      kbo(:, 4,51, 4) = (/ &
     &1.0912e-05_rb,1.1993e-05_rb,9.7046e-06_rb,6.4754e-06_rb,2.3777e-07_rb/)
      kbo(:, 5,51, 4) = (/ &
     &1.8173e-05_rb,1.9097e-05_rb,1.5144e-05_rb,9.7867e-06_rb,3.0334e-07_rb/)
      kbo(:, 1,52, 4) = (/ &
     &1.1779e-06_rb,1.5381e-06_rb,1.3497e-06_rb,9.8158e-07_rb,6.6305e-08_rb/)
      kbo(:, 2,52, 4) = (/ &
     &2.2623e-06_rb,2.8412e-06_rb,2.4342e-06_rb,1.7266e-06_rb,9.4865e-08_rb/)
      kbo(:, 3,52, 4) = (/ &
     &4.3170e-06_rb,5.0854e-06_rb,4.2468e-06_rb,2.9260e-06_rb,1.3026e-07_rb/)
      kbo(:, 4,52, 4) = (/ &
     &7.7969e-06_rb,8.6772e-06_rb,7.0531e-06_rb,4.7333e-06_rb,1.7384e-07_rb/)
      kbo(:, 5,52, 4) = (/ &
     &1.3261e-05_rb,1.4075e-05_rb,1.1203e-05_rb,7.2837e-06_rb,2.2485e-07_rb/)
      kbo(:, 1,53, 4) = (/ &
     &8.3089e-07_rb,1.0687e-06_rb,9.3518e-07_rb,6.7997e-07_rb,4.5831e-08_rb/)
      kbo(:, 2,53, 4) = (/ &
     &1.5556e-06_rb,1.9711e-06_rb,1.6966e-06_rb,1.2083e-06_rb,6.7497e-08_rb/)
      kbo(:, 3,53, 4) = (/ &
     &3.0048e-06_rb,3.5908e-06_rb,3.0131e-06_rb,2.0844e-06_rb,9.3485e-08_rb/)
      kbo(:, 4,53, 4) = (/ &
     &5.5438e-06_rb,6.2468e-06_rb,5.1016e-06_rb,3.4387e-06_rb,1.2594e-07_rb/)
      kbo(:, 5,53, 4) = (/ &
     &9.6328e-06_rb,1.0329e-05_rb,8.2446e-06_rb,5.3978e-06_rb,1.6450e-07_rb/)
      kbo(:, 1,54, 4) = (/ &
     &6.1034e-07_rb,7.6407e-07_rb,6.6371e-07_rb,4.8016e-07_rb,3.1960e-08_rb/)
      kbo(:, 2,54, 4) = (/ &
     &1.0977e-06_rb,1.3930e-06_rb,1.2021e-06_rb,8.5784e-07_rb,4.9012e-08_rb/)
      kbo(:, 3,54, 4) = (/ &
     &2.1270e-06_rb,2.5705e-06_rb,2.1637e-06_rb,1.5024e-06_rb,6.8673e-08_rb/)
      kbo(:, 4,54, 4) = (/ &
     &3.9965e-06_rb,4.5514e-06_rb,3.7320e-06_rb,2.5226e-06_rb,9.3587e-08_rb/)
      kbo(:, 5,54, 4) = (/ &
     &7.0830e-06_rb,7.6654e-06_rb,6.1329e-06_rb,4.0351e-06_rb,1.2382e-07_rb/)
      kbo(:, 1,55, 4) = (/ &
     &4.6385e-07_rb,5.5921e-07_rb,4.8019e-07_rb,3.4337e-07_rb,2.2394e-08_rb/)
      kbo(:, 2,55, 4) = (/ &
     &7.8989e-07_rb,9.9640e-07_rb,8.5986e-07_rb,6.1313e-07_rb,3.5977e-08_rb/)
      kbo(:, 3,55, 4) = (/ &
     &1.5192e-06_rb,1.8516e-06_rb,1.5624e-06_rb,1.0878e-06_rb,5.1500e-08_rb/)
      kbo(:, 4,55, 4) = (/ &
     &2.8980e-06_rb,3.3321e-06_rb,2.7426e-06_rb,1.8581e-06_rb,7.0762e-08_rb/)
      kbo(:, 5,55, 4) = (/ &
     &5.2356e-06_rb,5.7136e-06_rb,4.5826e-06_rb,3.0253e-06_rb,9.4338e-08_rb/)
      kbo(:, 1,56, 4) = (/ &
     &3.6371e-07_rb,4.1822e-07_rb,3.5281e-07_rb,2.4786e-07_rb,1.6072e-08_rb/)
      kbo(:, 2,56, 4) = (/ &
     &5.7786e-07_rb,7.1776e-07_rb,6.1684e-07_rb,4.3889e-07_rb,2.6450e-08_rb/)
      kbo(:, 3,56, 4) = (/ &
     &1.0893e-06_rb,1.3335e-06_rb,1.1271e-06_rb,7.8636e-07_rb,3.8603e-08_rb/)
      kbo(:, 4,56, 4) = (/ &
     &2.1010e-06_rb,2.4368e-06_rb,2.0114e-06_rb,1.3652e-06_rb,5.3303e-08_rb/)
      kbo(:, 5,56, 4) = (/ &
     &3.8675e-06_rb,4.2536e-06_rb,3.4205e-06_rb,2.2622e-06_rb,7.1626e-08_rb/)
      kbo(:, 1,57, 4) = (/ &
     &2.9431e-07_rb,3.2133e-07_rb,2.6457e-07_rb,1.8146e-07_rb,1.1259e-08_rb/)
      kbo(:, 2,57, 4) = (/ &
     &4.3212e-07_rb,5.2280e-07_rb,4.4601e-07_rb,3.1527e-07_rb,1.9122e-08_rb/)
      kbo(:, 3,57, 4) = (/ &
     &7.8679e-07_rb,9.6129e-07_rb,8.1340e-07_rb,5.6766e-07_rb,2.8681e-08_rb/)
      kbo(:, 4,57, 4) = (/ &
     &1.5240e-06_rb,1.7810e-06_rb,1.4725e-06_rb,1.0013e-06_rb,4.0212e-08_rb/)
      kbo(:, 5,57, 4) = (/ &
     &2.8546e-06_rb,3.1617e-06_rb,2.5494e-06_rb,1.6875e-06_rb,5.4480e-08_rb/)
      kbo(:, 1,58, 4) = (/ &
     &1.1920e-07_rb,1.4193e-07_rb,1.3380e-07_rb,1.0782e-07_rb,7.9525e-09_rb/)
      kbo(:, 2,58, 4) = (/ &
     &1.6186e-07_rb,2.1696e-07_rb,2.1477e-07_rb,1.8165e-07_rb,1.3814e-08_rb/)
      kbo(:, 3,58, 4) = (/ &
     &2.8087e-07_rb,3.9076e-07_rb,3.8760e-07_rb,3.2679e-07_rb,2.1394e-08_rb/)
      kbo(:, 4,58, 4) = (/ &
     &5.4191e-07_rb,7.3009e-07_rb,7.0916e-07_rb,5.8413e-07_rb,3.0724e-08_rb/)
      kbo(:, 5,58, 4) = (/ &
     &1.0299e-06_rb,1.3167e-06_rb,1.2488e-06_rb,1.0008e-06_rb,4.1885e-08_rb/)
      kbo(:, 1,59, 4) = (/ &
     &1.0039e-07_rb,1.1714e-07_rb,1.0974e-07_rb,8.7504e-08_rb,6.4674e-09_rb/)
      kbo(:, 2,59, 4) = (/ &
     &1.3241e-07_rb,1.7435e-07_rb,1.7189e-07_rb,1.4496e-07_rb,1.1458e-08_rb/)
      kbo(:, 3,59, 4) = (/ &
     &2.2493e-07_rb,3.1060e-07_rb,3.0841e-07_rb,2.6052e-07_rb,1.7735e-08_rb/)
      kbo(:, 4,59, 4) = (/ &
     &4.3202e-07_rb,5.8172e-07_rb,5.6654e-07_rb,4.6829e-07_rb,2.5818e-08_rb/)
      kbo(:, 5,59, 4) = (/ &
     &8.2523e-07_rb,1.0563e-06_rb,1.0053e-06_rb,8.0852e-07_rb,3.5572e-08_rb/)
      kbo(:, 1,13, 5) = (/ &
     &3.5348e-02_rb,3.1651e-02_rb,2.3972e-02_rb,1.4520e-02_rb,1.2073e-03_rb/)
      kbo(:, 2,13, 5) = (/ &
     &5.4657e-02_rb,4.7880e-02_rb,3.5919e-02_rb,2.1424e-02_rb,1.4997e-03_rb/)
      kbo(:, 3,13, 5) = (/ &
     &8.2602e-02_rb,7.1045e-02_rb,5.2562e-02_rb,3.0599e-02_rb,1.8487e-03_rb/)
      kbo(:, 4,13, 5) = (/ &
     &1.1970e-01_rb,1.0136e-01_rb,7.4139e-02_rb,4.2733e-02_rb,2.2117e-03_rb/)
      kbo(:, 5,13, 5) = (/ &
     &1.6697e-01_rb,1.3996e-01_rb,1.0160e-01_rb,5.7755e-02_rb,2.6210e-03_rb/)
      kbo(:, 1,14, 5) = (/ &
     &3.1413e-02_rb,2.7971e-02_rb,2.1089e-02_rb,1.2782e-02_rb,9.9834e-04_rb/)
      kbo(:, 2,14, 5) = (/ &
     &4.9001e-02_rb,4.2669e-02_rb,3.1890e-02_rb,1.8901e-02_rb,1.2749e-03_rb/)
      kbo(:, 3,14, 5) = (/ &
     &7.3648e-02_rb,6.3169e-02_rb,4.6557e-02_rb,2.7140e-02_rb,1.5545e-03_rb/)
      kbo(:, 4,14, 5) = (/ &
     &1.0633e-01_rb,9.0035e-02_rb,6.5788e-02_rb,3.7842e-02_rb,1.8680e-03_rb/)
      kbo(:, 5,14, 5) = (/ &
     &1.4847e-01_rb,1.2452e-01_rb,9.0180e-02_rb,5.1123e-02_rb,2.2124e-03_rb/)
      kbo(:, 1,15, 5) = (/ &
     &2.6691e-02_rb,2.3767e-02_rb,1.7918e-02_rb,1.0846e-02_rb,8.4376e-04_rb/)
      kbo(:, 2,15, 5) = (/ &
     &4.1563e-02_rb,3.6328e-02_rb,2.7130e-02_rb,1.6082e-02_rb,1.0611e-03_rb/)
      kbo(:, 3,15, 5) = (/ &
     &6.2559e-02_rb,5.3858e-02_rb,3.9632e-02_rb,2.3133e-02_rb,1.2845e-03_rb/)
      kbo(:, 4,15, 5) = (/ &
     &9.0477e-02_rb,7.6844e-02_rb,5.6216e-02_rb,3.2223e-02_rb,1.5617e-03_rb/)
      kbo(:, 5,15, 5) = (/ &
     &1.2655e-01_rb,1.0613e-01_rb,7.6652e-02_rb,4.3561e-02_rb,1.9061e-03_rb/)
      kbo(:, 1,16, 5) = (/ &
     &2.1986e-02_rb,1.9716e-02_rb,1.4938e-02_rb,9.0172e-03_rb,6.9426e-04_rb/)
      kbo(:, 2,16, 5) = (/ &
     &3.4448e-02_rb,3.0201e-02_rb,2.2563e-02_rb,1.3402e-02_rb,8.6528e-04_rb/)
      kbo(:, 3,16, 5) = (/ &
     &5.1943e-02_rb,4.4837e-02_rb,3.3116e-02_rb,1.9274e-02_rb,1.0580e-03_rb/)
      kbo(:, 4,16, 5) = (/ &
     &7.5192e-02_rb,6.4048e-02_rb,4.6767e-02_rb,2.6829e-02_rb,1.3132e-03_rb/)
      kbo(:, 5,16, 5) = (/ &
     &1.0492e-01_rb,8.8109e-02_rb,6.3827e-02_rb,3.6312e-02_rb,1.6006e-03_rb/)
      kbo(:, 1,17, 5) = (/ &
     &1.8067e-02_rb,1.6242e-02_rb,1.2317e-02_rb,7.4587e-03_rb,5.8084e-04_rb/)
      kbo(:, 2,17, 5) = (/ &
     &2.8443e-02_rb,2.5054e-02_rb,1.8720e-02_rb,1.1098e-02_rb,7.2334e-04_rb/)
      kbo(:, 3,17, 5) = (/ &
     &4.2795e-02_rb,3.7062e-02_rb,2.7408e-02_rb,1.5965e-02_rb,9.0099e-04_rb/)
      kbo(:, 4,17, 5) = (/ &
     &6.1919e-02_rb,5.2891e-02_rb,3.8653e-02_rb,2.2221e-02_rb,1.1193e-03_rb/)
      kbo(:, 5,17, 5) = (/ &
     &8.6219e-02_rb,7.2608e-02_rb,5.2687e-02_rb,2.9957e-02_rb,1.3671e-03_rb/)
      kbo(:, 1,18, 5) = (/ &
     &1.4708e-02_rb,1.3327e-02_rb,1.0113e-02_rb,6.1290e-03_rb,4.8316e-04_rb/)
      kbo(:, 2,18, 5) = (/ &
     &2.3136e-02_rb,2.0491e-02_rb,1.5341e-02_rb,9.1147e-03_rb,6.0374e-04_rb/)
      kbo(:, 3,18, 5) = (/ &
     &3.4747e-02_rb,3.0283e-02_rb,2.2393e-02_rb,1.3077e-02_rb,7.6598e-04_rb/)
      kbo(:, 4,18, 5) = (/ &
     &5.0288e-02_rb,4.3057e-02_rb,3.1490e-02_rb,1.8130e-02_rb,9.5069e-04_rb/)
      kbo(:, 5,18, 5) = (/ &
     &6.9756e-02_rb,5.9005e-02_rb,4.2850e-02_rb,2.4412e-02_rb,1.1633e-03_rb/)
      kbo(:, 1,19, 5) = (/ &
     &1.2271e-02_rb,1.1130e-02_rb,8.4511e-03_rb,5.1168e-03_rb,4.1293e-04_rb/)
      kbo(:, 2,19, 5) = (/ &
     &1.9205e-02_rb,1.7036e-02_rb,1.2763e-02_rb,7.5902e-03_rb,5.1993e-04_rb/)
      kbo(:, 3,19, 5) = (/ &
     &2.8789e-02_rb,2.5103e-02_rb,1.8579e-02_rb,1.0864e-02_rb,6.5561e-04_rb/)
      kbo(:, 4,19, 5) = (/ &
     &4.1470e-02_rb,3.5549e-02_rb,2.6049e-02_rb,1.5006e-02_rb,8.1395e-04_rb/)
      kbo(:, 5,19, 5) = (/ &
     &5.7470e-02_rb,4.8743e-02_rb,3.5383e-02_rb,2.0175e-02_rb,9.9598e-04_rb/)
      kbo(:, 1,20, 5) = (/ &
     &1.0604e-02_rb,9.5861e-03_rb,7.2655e-03_rb,4.3823e-03_rb,3.5617e-04_rb/)
      kbo(:, 2,20, 5) = (/ &
     &1.6495e-02_rb,1.4613e-02_rb,1.0934e-02_rb,6.4798e-03_rb,4.4865e-04_rb/)
      kbo(:, 3,20, 5) = (/ &
     &2.4651e-02_rb,2.1430e-02_rb,1.5803e-02_rb,9.2155e-03_rb,5.6360e-04_rb/)
      kbo(:, 4,20, 5) = (/ &
     &3.5214e-02_rb,3.0218e-02_rb,2.2119e-02_rb,1.2722e-02_rb,7.0046e-04_rb/)
      kbo(:, 5,20, 5) = (/ &
     &4.8685e-02_rb,4.1243e-02_rb,2.9948e-02_rb,1.7048e-02_rb,8.5450e-04_rb/)
      kbo(:, 1,21, 5) = (/ &
     &9.2365e-03_rb,8.3349e-03_rb,6.3064e-03_rb,3.7870e-03_rb,3.0556e-04_rb/)
      kbo(:, 2,21, 5) = (/ &
     &1.4270e-02_rb,1.2642e-02_rb,9.4383e-03_rb,5.5694e-03_rb,3.8499e-04_rb/)
      kbo(:, 3,21, 5) = (/ &
     &2.1188e-02_rb,1.8415e-02_rb,1.3582e-02_rb,7.8888e-03_rb,4.8632e-04_rb/)
      kbo(:, 4,21, 5) = (/ &
     &3.0204e-02_rb,2.5895e-02_rb,1.8922e-02_rb,1.0859e-02_rb,6.0144e-04_rb/)
      kbo(:, 5,21, 5) = (/ &
     &4.1738e-02_rb,3.5266e-02_rb,2.5553e-02_rb,1.4491e-02_rb,7.3167e-04_rb/)
      kbo(:, 1,22, 5) = (/ &
     &8.2084e-03_rb,7.4154e-03_rb,5.5986e-03_rb,3.3447e-03_rb,2.6030e-04_rb/)
      kbo(:, 2,22, 5) = (/ &
     &1.2593e-02_rb,1.1132e-02_rb,8.2855e-03_rb,4.8830e-03_rb,3.3150e-04_rb/)
      kbo(:, 3,22, 5) = (/ &
     &1.8558e-02_rb,1.6118e-02_rb,1.1879e-02_rb,6.8818e-03_rb,4.1718e-04_rb/)
      kbo(:, 4,22, 5) = (/ &
     &2.6423e-02_rb,2.2608e-02_rb,1.6490e-02_rb,9.4242e-03_rb,5.1837e-04_rb/)
      kbo(:, 5,22, 5) = (/ &
     &3.6423e-02_rb,3.0712e-02_rb,2.2210e-02_rb,1.2556e-02_rb,6.2757e-04_rb/)
      kbo(:, 1,23, 5) = (/ &
     &7.2962e-03_rb,6.5767e-03_rb,4.9576e-03_rb,2.9518e-03_rb,2.2147e-04_rb/)
      kbo(:, 2,23, 5) = (/ &
     &1.1116e-02_rb,9.7878e-03_rb,7.2822e-03_rb,4.2736e-03_rb,2.8452e-04_rb/)
      kbo(:, 3,23, 5) = (/ &
     &1.6294e-02_rb,1.4142e-02_rb,1.0397e-02_rb,6.0039e-03_rb,3.5768e-04_rb/)
      kbo(:, 4,23, 5) = (/ &
     &2.3164e-02_rb,1.9768e-02_rb,1.4383e-02_rb,8.1899e-03_rb,4.4507e-04_rb/)
      kbo(:, 5,23, 5) = (/ &
     &3.1815e-02_rb,2.6782e-02_rb,1.9341e-02_rb,1.0913e-02_rb,5.3593e-04_rb/)
      kbo(:, 1,24, 5) = (/ &
     &6.5132e-03_rb,5.8530e-03_rb,4.4011e-03_rb,2.6156e-03_rb,1.8866e-04_rb/)
      kbo(:, 2,24, 5) = (/ &
     &9.8979e-03_rb,8.6925e-03_rb,6.4521e-03_rb,3.7670e-03_rb,2.4398e-04_rb/)
      kbo(:, 3,24, 5) = (/ &
     &1.4458e-02_rb,1.2498e-02_rb,9.1615e-03_rb,5.2667e-03_rb,3.0838e-04_rb/)
      kbo(:, 4,24, 5) = (/ &
     &2.0434e-02_rb,1.7400e-02_rb,1.2635e-02_rb,7.1775e-03_rb,3.8038e-04_rb/)
      kbo(:, 5,24, 5) = (/ &
     &2.8011e-02_rb,2.3552e-02_rb,1.6988e-02_rb,9.5542e-03_rb,4.5842e-04_rb/)
      kbo(:, 1,25, 5) = (/ &
     &5.8496e-03_rb,5.2463e-03_rb,3.9347e-03_rb,2.3271e-03_rb,1.6445e-04_rb/)
      kbo(:, 2,25, 5) = (/ &
     &8.8417e-03_rb,7.7604e-03_rb,5.7347e-03_rb,3.3321e-03_rb,2.1081e-04_rb/)
      kbo(:, 3,25, 5) = (/ &
     &1.2868e-02_rb,1.1077e-02_rb,8.0978e-03_rb,4.6433e-03_rb,2.6559e-04_rb/)
      kbo(:, 4,25, 5) = (/ &
     &1.8100e-02_rb,1.5382e-02_rb,1.1156e-02_rb,6.3115e-03_rb,3.2583e-04_rb/)
      kbo(:, 5,25, 5) = (/ &
     &2.4695e-02_rb,2.0722e-02_rb,1.4925e-02_rb,8.3824e-03_rb,3.9105e-04_rb/)
      kbo(:, 1,26, 5) = (/ &
     &5.3224e-03_rb,4.7440e-03_rb,3.5455e-03_rb,2.0856e-03_rb,1.4273e-04_rb/)
      kbo(:, 2,26, 5) = (/ &
     &7.9880e-03_rb,6.9773e-03_rb,5.1401e-03_rb,2.9750e-03_rb,1.8261e-04_rb/)
      kbo(:, 3,26, 5) = (/ &
     &1.1564e-02_rb,9.9215e-03_rb,7.2343e-03_rb,4.1334e-03_rb,2.2903e-04_rb/)
      kbo(:, 4,26, 5) = (/ &
     &1.6157e-02_rb,1.3698e-02_rb,9.9057e-03_rb,5.5934e-03_rb,2.7897e-04_rb/)
      kbo(:, 5,26, 5) = (/ &
     &2.1895e-02_rb,1.8349e-02_rb,1.3199e-02_rb,7.3992e-03_rb,3.3238e-04_rb/)
      kbo(:, 1,27, 5) = (/ &
     &4.8372e-03_rb,4.2909e-03_rb,3.1935e-03_rb,1.8703e-03_rb,1.2222e-04_rb/)
      kbo(:, 2,27, 5) = (/ &
     &7.2219e-03_rb,6.2785e-03_rb,4.6150e-03_rb,2.6594e-03_rb,1.5564e-04_rb/)
      kbo(:, 3,27, 5) = (/ &
     &1.0384e-02_rb,8.8912e-03_rb,6.4664e-03_rb,3.6778e-03_rb,1.9361e-04_rb/)
      kbo(:, 4,27, 5) = (/ &
     &1.4401e-02_rb,1.2183e-02_rb,8.7984e-03_rb,4.9566e-03_rb,2.3445e-04_rb/)
      kbo(:, 5,27, 5) = (/ &
     &1.9400e-02_rb,1.6243e-02_rb,1.1670e-02_rb,6.5316e-03_rb,2.7869e-04_rb/)
      kbo(:, 1,28, 5) = (/ &
     &4.3907e-03_rb,3.8800e-03_rb,2.8785e-03_rb,1.6793e-03_rb,1.0352e-04_rb/)
      kbo(:, 2,28, 5) = (/ &
     &6.5180e-03_rb,5.6500e-03_rb,4.1431e-03_rb,2.3781e-03_rb,1.3090e-04_rb/)
      kbo(:, 3,28, 5) = (/ &
     &9.2995e-03_rb,7.9428e-03_rb,5.7641e-03_rb,3.2684e-03_rb,1.6163e-04_rb/)
      kbo(:, 4,28, 5) = (/ &
     &1.2806e-02_rb,1.0816e-02_rb,7.7992e-03_rb,4.3863e-03_rb,1.9422e-04_rb/)
      kbo(:, 5,28, 5) = (/ &
     &1.7155e-02_rb,1.4361e-02_rb,1.0305e-02_rb,5.7599e-03_rb,2.3020e-04_rb/)
      kbo(:, 1,29, 5) = (/ &
     &3.9730e-03_rb,3.5033e-03_rb,2.5903e-03_rb,1.5043e-03_rb,8.6138e-05_rb/)
      kbo(:, 2,29, 5) = (/ &
     &5.8501e-03_rb,5.0589e-03_rb,3.6981e-03_rb,2.1165e-03_rb,1.0866e-04_rb/)
      kbo(:, 3,29, 5) = (/ &
     &8.2752e-03_rb,7.0540e-03_rb,5.1122e-03_rb,2.8938e-03_rb,1.3206e-04_rb/)
      kbo(:, 4,29, 5) = (/ &
     &1.1323e-02_rb,9.5548e-03_rb,6.8809e-03_rb,3.8639e-03_rb,1.5770e-04_rb/)
      kbo(:, 5,29, 5) = (/ &
     &1.5087e-02_rb,1.2624e-02_rb,9.0498e-03_rb,5.0491e-03_rb,1.8805e-04_rb/)
      kbo(:, 1,30, 5) = (/ &
     &3.5719e-03_rb,3.1399e-03_rb,2.3162e-03_rb,1.3392e-03_rb,7.1373e-05_rb/)
      kbo(:, 2,30, 5) = (/ &
     &5.2107e-03_rb,4.4986e-03_rb,3.2816e-03_rb,1.8730e-03_rb,8.8119e-05_rb/)
      kbo(:, 3,30, 5) = (/ &
     &7.3165e-03_rb,6.2315e-03_rb,4.5078e-03_rb,2.5469e-03_rb,1.0662e-04_rb/)
      kbo(:, 4,30, 5) = (/ &
     &9.9520e-03_rb,8.3892e-03_rb,6.0327e-03_rb,3.3847e-03_rb,1.2803e-04_rb/)
      kbo(:, 5,30, 5) = (/ &
     &1.3166e-02_rb,1.1029e-02_rb,7.9003e-03_rb,4.4030e-03_rb,1.5121e-04_rb/)
      kbo(:, 1,31, 5) = (/ &
     &3.1788e-03_rb,2.7864e-03_rb,2.0507e-03_rb,1.1825e-03_rb,5.7200e-05_rb/)
      kbo(:, 2,31, 5) = (/ &
     &4.5964e-03_rb,3.9617e-03_rb,2.8849e-03_rb,1.6429e-03_rb,7.0401e-05_rb/)
      kbo(:, 3,31, 5) = (/ &
     &6.4014e-03_rb,5.4495e-03_rb,3.9350e-03_rb,2.2208e-03_rb,8.5247e-05_rb/)
      kbo(:, 4,31, 5) = (/ &
     &8.6375e-03_rb,7.2878e-03_rb,5.2393e-03_rb,2.9385e-03_rb,1.0146e-04_rb/)
      kbo(:, 5,31, 5) = (/ &
     &1.1347e-02_rb,9.5229e-03_rb,6.8169e-03_rb,3.7997e-03_rb,1.1891e-04_rb/)
      kbo(:, 1,32, 5) = (/ &
     &2.8359e-03_rb,2.4792e-03_rb,1.8198e-03_rb,1.0462e-03_rb,4.5712e-05_rb/)
      kbo(:, 2,32, 5) = (/ &
     &4.0627e-03_rb,3.4962e-03_rb,2.5410e-03_rb,1.4431e-03_rb,5.6022e-05_rb/)
      kbo(:, 3,32, 5) = (/ &
     &5.6078e-03_rb,4.7711e-03_rb,3.4442e-03_rb,1.9418e-03_rb,6.7267e-05_rb/)
      kbo(:, 4,32, 5) = (/ &
     &7.5047e-03_rb,6.3406e-03_rb,4.5581e-03_rb,2.5548e-03_rb,7.9547e-05_rb/)
      kbo(:, 5,32, 5) = (/ &
     &9.7792e-03_rb,8.2213e-03_rb,5.8868e-03_rb,3.2848e-03_rb,9.2763e-05_rb/)
      kbo(:, 1,33, 5) = (/ &
     &2.5284e-03_rb,2.2057e-03_rb,1.6155e-03_rb,9.2588e-04_rb,3.6267e-05_rb/)
      kbo(:, 2,33, 5) = (/ &
     &3.5886e-03_rb,3.0844e-03_rb,2.2382e-03_rb,1.2695e-03_rb,4.4003e-05_rb/)
      kbo(:, 3,33, 5) = (/ &
     &4.9097e-03_rb,4.1805e-03_rb,3.0157e-03_rb,1.6993e-03_rb,5.2825e-05_rb/)
      kbo(:, 4,33, 5) = (/ &
     &6.5165e-03_rb,5.5167e-03_rb,3.9645e-03_rb,2.2215e-03_rb,6.1889e-05_rb/)
      kbo(:, 5,33, 5) = (/ &
     &8.4209e-03_rb,7.0899e-03_rb,5.0775e-03_rb,2.8354e-03_rb,7.2610e-05_rb/)
      kbo(:, 1,34, 5) = (/ &
     &2.2520e-03_rb,1.9602e-03_rb,1.4328e-03_rb,8.1868e-04_rb,2.9417e-05_rb/)
      kbo(:, 2,34, 5) = (/ &
     &3.1681e-03_rb,2.7215e-03_rb,1.9717e-03_rb,1.1166e-03_rb,3.5490e-05_rb/)
      kbo(:, 3,34, 5) = (/ &
     &4.2977e-03_rb,3.6615e-03_rb,2.6403e-03_rb,1.4869e-03_rb,4.1854e-05_rb/)
      kbo(:, 4,34, 5) = (/ &
     &5.6597e-03_rb,4.7967e-03_rb,3.4462e-03_rb,1.9298e-03_rb,4.9348e-05_rb/)
      kbo(:, 5,34, 5) = (/ &
     &7.2681e-03_rb,6.1253e-03_rb,4.3873e-03_rb,2.4514e-03_rb,5.8186e-05_rb/)
      kbo(:, 1,35, 5) = (/ &
     &1.9366e-03_rb,1.6854e-03_rb,1.2304e-03_rb,7.0199e-04_rb,2.3008e-05_rb/)
      kbo(:, 2,35, 5) = (/ &
     &2.7102e-03_rb,2.3296e-03_rb,1.6872e-03_rb,9.5490e-04_rb,2.7664e-05_rb/)
      kbo(:, 3,35, 5) = (/ &
     &3.6574e-03_rb,3.1206e-03_rb,2.2499e-03_rb,1.2662e-03_rb,3.3089e-05_rb/)
      kbo(:, 4,35, 5) = (/ &
     &4.7963e-03_rb,4.0694e-03_rb,2.9228e-03_rb,1.6371e-03_rb,3.9372e-05_rb/)
      kbo(:, 5,35, 5) = (/ &
     &6.1352e-03_rb,5.1795e-03_rb,3.7111e-03_rb,2.0740e-03_rb,4.6276e-05_rb/)
      kbo(:, 1,36, 5) = (/ &
     &1.6034e-03_rb,1.3976e-03_rb,1.0202e-03_rb,5.8186e-04_rb,1.7663e-05_rb/)
      kbo(:, 2,36, 5) = (/ &
     &2.2421e-03_rb,1.9307e-03_rb,1.3983e-03_rb,7.9118e-04_rb,2.1547e-05_rb/)
      kbo(:, 3,36, 5) = (/ &
     &3.0206e-03_rb,2.5824e-03_rb,1.8623e-03_rb,1.0484e-03_rb,2.5886e-05_rb/)
      kbo(:, 4,36, 5) = (/ &
     &3.9570e-03_rb,3.3630e-03_rb,2.4161e-03_rb,1.3541e-03_rb,3.0872e-05_rb/)
      kbo(:, 5,36, 5) = (/ &
     &5.0582e-03_rb,4.2785e-03_rb,3.0668e-03_rb,1.7150e-03_rb,3.6301e-05_rb/)
      kbo(:, 1,37, 5) = (/ &
     &1.2748e-03_rb,1.1146e-03_rb,8.1407e-04_rb,4.6450e-04_rb,1.3581e-05_rb/)
      kbo(:, 2,37, 5) = (/ &
     &1.7898e-03_rb,1.5447e-03_rb,1.1192e-03_rb,6.3354e-04_rb,1.6729e-05_rb/)
      kbo(:, 3,37, 5) = (/ &
     &2.4187e-03_rb,2.0714e-03_rb,1.4947e-03_rb,8.4187e-04_rb,2.0205e-05_rb/)
      kbo(:, 4,37, 5) = (/ &
     &3.1762e-03_rb,2.7057e-03_rb,1.9449e-03_rb,1.0902e-03_rb,2.4038e-05_rb/)
      kbo(:, 5,37, 5) = (/ &
     &4.0701e-03_rb,3.4504e-03_rb,2.4741e-03_rb,1.3836e-03_rb,2.8365e-05_rb/)
      kbo(:, 1,38, 5) = (/ &
     &1.0102e-03_rb,8.8559e-04_rb,6.4741e-04_rb,3.6947e-04_rb,1.0412e-05_rb/)
      kbo(:, 2,38, 5) = (/ &
     &1.4249e-03_rb,1.2327e-03_rb,8.9319e-04_rb,5.0584e-04_rb,1.2901e-05_rb/)
      kbo(:, 3,38, 5) = (/ &
     &1.9318e-03_rb,1.6579e-03_rb,1.1967e-03_rb,6.7434e-04_rb,1.5672e-05_rb/)
      kbo(:, 4,38, 5) = (/ &
     &2.5434e-03_rb,2.1714e-03_rb,1.5616e-03_rb,8.7562e-04_rb,1.8795e-05_rb/)
      kbo(:, 5,38, 5) = (/ &
     &3.2683e-03_rb,2.7765e-03_rb,1.9923e-03_rb,1.1142e-03_rb,2.2145e-05_rb/)
      kbo(:, 1,39, 5) = (/ &
     &7.9946e-04_rb,7.0298e-04_rb,5.1416e-04_rb,2.9368e-04_rb,7.9720e-06_rb/)
      kbo(:, 2,39, 5) = (/ &
     &1.1333e-03_rb,9.8290e-04_rb,7.1255e-04_rb,4.0353e-04_rb,9.9223e-06_rb/)
      kbo(:, 3,39, 5) = (/ &
     &1.5418e-03_rb,1.3259e-03_rb,9.5739e-04_rb,5.3970e-04_rb,1.2141e-05_rb/)
      kbo(:, 4,39, 5) = (/ &
     &2.0355e-03_rb,1.7418e-03_rb,1.2533e-03_rb,7.0303e-04_rb,1.4591e-05_rb/)
      kbo(:, 5,39, 5) = (/ &
     &2.6233e-03_rb,2.2332e-03_rb,1.6027e-03_rb,8.9673e-04_rb,1.7278e-05_rb/)
      kbo(:, 1,40, 5) = (/ &
     &6.1558e-04_rb,5.4339e-04_rb,3.9799e-04_rb,2.2761e-04_rb,6.0679e-06_rb/)
      kbo(:, 2,40, 5) = (/ &
     &8.8100e-04_rb,7.6638e-04_rb,5.5623e-04_rb,3.1507e-04_rb,7.6104e-06_rb/)
      kbo(:, 3,40, 5) = (/ &
     &1.2072e-03_rb,1.0404e-03_rb,7.5136e-04_rb,4.2373e-04_rb,9.3444e-06_rb/)
      kbo(:, 4,40, 5) = (/ &
     &1.6029e-03_rb,1.3742e-03_rb,9.8944e-04_rb,5.5501e-04_rb,1.1318e-05_rb/)
      kbo(:, 5,40, 5) = (/ &
     &2.0762e-03_rb,1.7710e-03_rb,1.2711e-03_rb,7.1134e-04_rb,1.3497e-05_rb/)
      kbo(:, 1,41, 5) = (/ &
     &4.7150e-04_rb,4.1786e-04_rb,3.0641e-04_rb,1.7559e-04_rb,4.6077e-06_rb/)
      kbo(:, 2,41, 5) = (/ &
     &6.8147e-04_rb,5.9486e-04_rb,4.3209e-04_rb,2.4498e-04_rb,5.8135e-06_rb/)
      kbo(:, 3,41, 5) = (/ &
     &9.4213e-04_rb,8.1372e-04_rb,5.8780e-04_rb,3.3145e-04_rb,7.1715e-06_rb/)
      kbo(:, 4,41, 5) = (/ &
     &1.2582e-03_rb,1.0809e-03_rb,7.7844e-04_rb,4.3685e-04_rb,8.7285e-06_rb/)
      kbo(:, 5,41, 5) = (/ &
     &1.6388e-03_rb,1.4010e-03_rb,1.0055e-03_rb,5.6266e-04_rb,1.0465e-05_rb/)
      kbo(:, 1,42, 5) = (/ &
     &3.6009e-04_rb,3.2039e-04_rb,2.3540e-04_rb,1.3511e-04_rb,3.4818e-06_rb/)
      kbo(:, 2,42, 5) = (/ &
     &5.2591e-04_rb,4.6075e-04_rb,3.3505e-04_rb,1.9005e-04_rb,4.4174e-06_rb/)
      kbo(:, 3,42, 5) = (/ &
     &7.3377e-04_rb,6.3539e-04_rb,4.5927e-04_rb,2.5904e-04_rb,5.4797e-06_rb/)
      kbo(:, 4,42, 5) = (/ &
     &9.8665e-04_rb,8.4900e-04_rb,6.1152e-04_rb,3.4335e-04_rb,6.7219e-06_rb/)
      kbo(:, 5,42, 5) = (/ &
     &1.2919e-03_rb,1.1071e-03_rb,7.9454e-04_rb,4.4455e-04_rb,8.0996e-06_rb/)
      kbo(:, 1,43, 5) = (/ &
     &2.7068e-04_rb,2.4195e-04_rb,1.7816e-04_rb,1.0249e-04_rb,2.5837e-06_rb/)
      kbo(:, 2,43, 5) = (/ &
     &4.0104e-04_rb,3.5254e-04_rb,2.5672e-04_rb,1.4582e-04_rb,3.3146e-06_rb/)
      kbo(:, 3,43, 5) = (/ &
     &5.6618e-04_rb,4.9150e-04_rb,3.5538e-04_rb,2.0045e-04_rb,4.1432e-06_rb/)
      kbo(:, 4,43, 5) = (/ &
     &7.6835e-04_rb,6.6216e-04_rb,4.7675e-04_rb,2.6781e-04_rb,5.1315e-06_rb/)
      kbo(:, 5,43, 5) = (/ &
     &1.0127e-03_rb,8.6923e-04_rb,6.2405e-04_rb,3.4899e-04_rb,6.1983e-06_rb/)
      kbo(:, 1,44, 5) = (/ &
     &2.0139e-04_rb,1.8106e-04_rb,1.3358e-04_rb,7.7094e-05_rb,1.8960e-06_rb/)
      kbo(:, 2,44, 5) = (/ &
     &3.0345e-04_rb,2.6774e-04_rb,1.9525e-04_rb,1.1112e-04_rb,2.4595e-06_rb/)
      kbo(:, 3,44, 5) = (/ &
     &4.3439e-04_rb,3.7822e-04_rb,2.7355e-04_rb,1.5433e-04_rb,3.1158e-06_rb/)
      kbo(:, 4,44, 5) = (/ &
     &5.9600e-04_rb,5.1425e-04_rb,3.7025e-04_rb,2.0795e-04_rb,3.8746e-06_rb/)
      kbo(:, 5,44, 5) = (/ &
     &7.9184e-04_rb,6.8038e-04_rb,4.8839e-04_rb,2.7303e-04_rb,4.7113e-06_rb/)
      kbo(:, 1,45, 5) = (/ &
     &1.4924e-04_rb,1.3479e-04_rb,9.9739e-05_rb,5.7756e-05_rb,1.3827e-06_rb/)
      kbo(:, 2,45, 5) = (/ &
     &2.2880e-04_rb,2.0270e-04_rb,1.4808e-04_rb,8.4395e-05_rb,1.8081e-06_rb/)
      kbo(:, 3,45, 5) = (/ &
     &3.3229e-04_rb,2.9032e-04_rb,2.1015e-04_rb,1.1857e-04_rb,2.3261e-06_rb/)
      kbo(:, 4,45, 5) = (/ &
     &4.6152e-04_rb,3.9893e-04_rb,2.8721e-04_rb,1.6120e-04_rb,2.9080e-06_rb/)
      kbo(:, 5,45, 5) = (/ &
     &6.1857e-04_rb,5.3204e-04_rb,3.8190e-04_rb,2.1341e-04_rb,3.5612e-06_rb/)
      kbo(:, 1,46, 5) = (/ &
     &1.0906e-04_rb,9.9059e-05_rb,7.3477e-05_rb,4.2734e-05_rb,1.0024e-06_rb/)
      kbo(:, 2,46, 5) = (/ &
     &1.7043e-04_rb,1.5165e-04_rb,1.1108e-04_rb,6.3473e-05_rb,1.3227e-06_rb/)
      kbo(:, 3,46, 5) = (/ &
     &2.5193e-04_rb,2.2079e-04_rb,1.6005e-04_rb,9.0376e-05_rb,1.7191e-06_rb/)
      kbo(:, 4,46, 5) = (/ &
     &3.5457e-04_rb,3.0711e-04_rb,2.2121e-04_rb,1.2412e-04_rb,2.1640e-06_rb/)
      kbo(:, 5,46, 5) = (/ &
     &4.8016e-04_rb,4.1350e-04_rb,2.9669e-04_rb,1.6585e-04_rb,2.6681e-06_rb/)
      kbo(:, 1,47, 5) = (/ &
     &7.7791e-05_rb,7.1164e-05_rb,5.2966e-05_rb,3.0950e-05_rb,7.1979e-07_rb/)
      kbo(:, 2,47, 5) = (/ &
     &1.2436e-04_rb,1.1134e-04_rb,8.1758e-05_rb,4.6856e-05_rb,9.5466e-07_rb/)
      kbo(:, 3,47, 5) = (/ &
     &1.8767e-04_rb,1.6518e-04_rb,1.1995e-04_rb,6.7855e-05_rb,1.2553e-06_rb/)
      kbo(:, 4,47, 5) = (/ &
     &2.6877e-04_rb,2.3350e-04_rb,1.6823e-04_rb,9.4438e-05_rb,1.6001e-06_rb/)
      kbo(:, 5,47, 5) = (/ &
     &3.6855e-04_rb,3.1774e-04_rb,2.2797e-04_rb,1.2742e-04_rb,1.9855e-06_rb/)
      kbo(:, 1,48, 5) = (/ &
     &5.5044e-05_rb,5.0693e-05_rb,3.7891e-05_rb,2.2267e-05_rb,5.1420e-07_rb/)
      kbo(:, 2,48, 5) = (/ &
     &9.0215e-05_rb,8.1170e-05_rb,5.9770e-05_rb,3.4413e-05_rb,6.9075e-07_rb/)
      kbo(:, 3,48, 5) = (/ &
     &1.3907e-04_rb,1.2292e-04_rb,8.9447e-05_rb,5.0716e-05_rb,9.0307e-07_rb/)
      kbo(:, 4,48, 5) = (/ &
     &2.0282e-04_rb,1.7688e-04_rb,1.2753e-04_rb,7.1593e-05_rb,1.1665e-06_rb/)
      kbo(:, 5,48, 5) = (/ &
     &2.8252e-04_rb,2.4382e-04_rb,1.7492e-04_rb,9.7702e-05_rb,1.4649e-06_rb/)
      kbo(:, 1,49, 5) = (/ &
     &3.8587e-05_rb,3.5798e-05_rb,2.6873e-05_rb,1.5895e-05_rb,3.6284e-07_rb/)
      kbo(:, 2,49, 5) = (/ &
     &6.4973e-05_rb,5.8784e-05_rb,4.3404e-05_rb,2.5118e-05_rb,4.9340e-07_rb/)
      kbo(:, 3,49, 5) = (/ &
     &1.0247e-04_rb,9.0960e-05_rb,6.6355e-05_rb,3.7720e-05_rb,6.5251e-07_rb/)
      kbo(:, 4,49, 5) = (/ &
     &1.5241e-04_rb,1.3334e-04_rb,9.6283e-05_rb,5.4131e-05_rb,8.4176e-07_rb/)
      kbo(:, 5,49, 5) = (/ &
     &2.1585e-04_rb,1.8661e-04_rb,1.3387e-04_rb,7.4762e-05_rb,1.0688e-06_rb/)
      kbo(:, 1,50, 5) = (/ &
     &2.7204e-05_rb,2.5412e-05_rb,1.9146e-05_rb,1.1397e-05_rb,2.5753e-07_rb/)
      kbo(:, 2,50, 5) = (/ &
     &4.7009e-05_rb,4.2778e-05_rb,3.1689e-05_rb,1.8416e-05_rb,3.5668e-07_rb/)
      kbo(:, 3,50, 5) = (/ &
     &7.5923e-05_rb,6.7680e-05_rb,4.9459e-05_rb,2.8188e-05_rb,4.7630e-07_rb/)
      kbo(:, 4,50, 5) = (/ &
     &1.1516e-04_rb,1.0107e-04_rb,7.3089e-05_rb,4.1129e-05_rb,6.1736e-07_rb/)
      kbo(:, 5,50, 5) = (/ &
     &1.6568e-04_rb,1.4364e-04_rb,1.0307e-04_rb,5.7556e-05_rb,7.8811e-07_rb/)
      kbo(:, 1,51, 5) = (/ &
     &1.9132e-05_rb,1.8022e-05_rb,1.3620e-05_rb,8.1585e-06_rb,1.8251e-07_rb/)
      kbo(:, 2,51, 5) = (/ &
     &3.4015e-05_rb,3.1104e-05_rb,2.3112e-05_rb,1.3493e-05_rb,2.5742e-07_rb/)
      kbo(:, 3,51, 5) = (/ &
     &5.6251e-05_rb,5.0351e-05_rb,3.6864e-05_rb,2.1082e-05_rb,3.4853e-07_rb/)
      kbo(:, 4,51, 5) = (/ &
     &8.7117e-05_rb,7.6686e-05_rb,5.5526e-05_rb,3.1296e-05_rb,4.5614e-07_rb/)
      kbo(:, 5,51, 5) = (/ &
     &1.2742e-04_rb,1.1075e-04_rb,7.9490e-05_rb,4.4386e-05_rb,5.8324e-07_rb/)
      kbo(:, 1,52, 5) = (/ &
     &1.3343e-05_rb,1.2680e-05_rb,9.6205e-06_rb,5.7937e-06_rb,1.2757e-07_rb/)
      kbo(:, 2,52, 5) = (/ &
     &2.4457e-05_rb,2.2489e-05_rb,1.6762e-05_rb,9.8342e-06_rb,1.8427e-07_rb/)
      kbo(:, 3,52, 5) = (/ &
     &4.1490e-05_rb,3.7272e-05_rb,2.7347e-05_rb,1.5697e-05_rb,2.5311e-07_rb/)
      kbo(:, 4,52, 5) = (/ &
     &6.5698e-05_rb,5.8019e-05_rb,4.2067e-05_rb,2.3741e-05_rb,3.3454e-07_rb/)
      kbo(:, 5,52, 5) = (/ &
     &9.7769e-05_rb,8.5199e-05_rb,6.1200e-05_rb,3.4184e-05_rb,4.3074e-07_rb/)
      kbo(:, 1,53, 5) = (/ &
     &9.2286e-06_rb,8.8438e-06_rb,6.7394e-06_rb,4.0796e-06_rb,8.7990e-08_rb/)
      kbo(:, 2,53, 5) = (/ &
     &1.7462e-05_rb,1.6145e-05_rb,1.2071e-05_rb,7.1204e-06_rb,1.3004e-07_rb/)
      kbo(:, 3,53, 5) = (/ &
     &3.0397e-05_rb,2.7436e-05_rb,2.0177e-05_rb,1.1626e-05_rb,1.8173e-07_rb/)
      kbo(:, 4,53, 5) = (/ &
     &4.9319e-05_rb,4.3681e-05_rb,3.1722e-05_rb,1.7943e-05_rb,2.4392e-07_rb/)
      kbo(:, 5,53, 5) = (/ &
     &7.4848e-05_rb,6.5369e-05_rb,4.7021e-05_rb,2.6269e-05_rb,3.1719e-07_rb/)
      kbo(:, 1,54, 5) = (/ &
     &6.4770e-06_rb,6.2515e-06_rb,4.7770e-06_rb,2.9074e-06_rb,6.2611e-08_rb/)
      kbo(:, 2,54, 5) = (/ &
     &1.2608e-05_rb,1.1724e-05_rb,8.7837e-06_rb,5.2044e-06_rb,9.4222e-08_rb/)
      kbo(:, 3,54, 5) = (/ &
     &2.2532e-05_rb,2.0405e-05_rb,1.5043e-05_rb,8.6950e-06_rb,1.3411e-07_rb/)
      kbo(:, 4,54, 5) = (/ &
     &3.7397e-05_rb,3.3212e-05_rb,2.4145e-05_rb,1.3685e-05_rb,1.8245e-07_rb/)
      kbo(:, 5,54, 5) = (/ &
     &5.7878e-05_rb,5.0645e-05_rb,3.6444e-05_rb,2.0382e-05_rb,2.3871e-07_rb/)
      kbo(:, 1,55, 5) = (/ &
     &4.5852e-06_rb,4.4390e-06_rb,3.4016e-06_rb,2.0767e-06_rb,4.4961e-08_rb/)
      kbo(:, 2,55, 5) = (/ &
     &9.1363e-06_rb,8.5468e-06_rb,6.4138e-06_rb,3.8136e-06_rb,6.9204e-08_rb/)
      kbo(:, 3,55, 5) = (/ &
     &1.6770e-05_rb,1.5238e-05_rb,1.1251e-05_rb,6.5245e-06_rb,1.0020e-07_rb/)
      kbo(:, 4,55, 5) = (/ &
     &2.8475e-05_rb,2.5343e-05_rb,1.8439e-05_rb,1.0474e-05_rb,1.3796e-07_rb/)
      kbo(:, 5,55, 5) = (/ &
     &4.4988e-05_rb,3.9423e-05_rb,2.8373e-05_rb,1.5872e-05_rb,1.8258e-07_rb/)
      kbo(:, 1,56, 5) = (/ &
     &3.2543e-06_rb,3.1469e-06_rb,2.4158e-06_rb,1.4787e-06_rb,3.1932e-08_rb/)
      kbo(:, 2,56, 5) = (/ &
     &6.5989e-06_rb,6.2038e-06_rb,4.6696e-06_rb,2.7831e-06_rb,5.0531e-08_rb/)
      kbo(:, 3,56, 5) = (/ &
     &1.2453e-05_rb,1.1352e-05_rb,8.3945e-06_rb,4.8820e-06_rb,7.4538e-08_rb/)
      kbo(:, 4,56, 5) = (/ &
     &2.1655e-05_rb,1.9314e-05_rb,1.4065e-05_rb,8.0028e-06_rb,1.0397e-07_rb/)
      kbo(:, 5,56, 5) = (/ &
     &3.4950e-05_rb,3.0673e-05_rb,2.2085e-05_rb,1.2363e-05_rb,1.3925e-07_rb/)
      kbo(:, 1,57, 5) = (/ &
     &2.3166e-06_rb,2.2390e-06_rb,1.7142e-06_rb,1.0504e-06_rb,2.2682e-08_rb/)
      kbo(:, 2,57, 5) = (/ &
     &4.7540e-06_rb,4.4892e-06_rb,3.3846e-06_rb,2.0245e-06_rb,3.6988e-08_rb/)
      kbo(:, 3,57, 5) = (/ &
     &9.2246e-06_rb,8.4320e-06_rb,6.2438e-06_rb,3.6425e-06_rb,5.5328e-08_rb/)
      kbo(:, 4,57, 5) = (/ &
     &1.6448e-05_rb,1.4704e-05_rb,1.0717e-05_rb,6.1069e-06_rb,7.8210e-08_rb/)
      kbo(:, 5,57, 5) = (/ &
     &2.7140e-05_rb,2.3849e-05_rb,1.7182e-05_rb,9.6233e-06_rb,1.0611e-07_rb/)
      kbo(:, 1,58, 5) = (/ &
     &8.1594e-07_rb,9.0016e-07_rb,8.0661e-07_rb,5.9543e-07_rb,1.6269e-08_rb/)
      kbo(:, 2,58, 5) = (/ &
     &1.6728e-06_rb,1.8193e-06_rb,1.6122e-06_rb,1.1689e-06_rb,2.7161e-08_rb/)
      kbo(:, 3,58, 5) = (/ &
     &3.3325e-06_rb,3.5076e-06_rb,3.0497e-06_rb,2.1581e-06_rb,4.1468e-08_rb/)
      kbo(:, 4,58, 5) = (/ &
     &6.0908e-06_rb,6.2615e-06_rb,5.3621e-06_rb,3.7018e-06_rb,5.9556e-08_rb/)
      kbo(:, 5,58, 5) = (/ &
     &1.0274e-05_rb,1.0375e-05_rb,8.7754e-06_rb,5.9519e-06_rb,8.1351e-08_rb/)
      kbo(:, 1,59, 5) = (/ &
     &6.5199e-07_rb,7.1682e-07_rb,6.4285e-07_rb,4.7533e-07_rb,1.3398e-08_rb/)
      kbo(:, 2,59, 5) = (/ &
     &1.3361e-06_rb,1.4531e-06_rb,1.2913e-06_rb,9.4099e-07_rb,2.2445e-08_rb/)
      kbo(:, 3,59, 5) = (/ &
     &2.6852e-06_rb,2.8341e-06_rb,2.4717e-06_rb,1.7572e-06_rb,3.4660e-08_rb/)
      kbo(:, 4,59, 5) = (/ &
     &4.9575e-06_rb,5.1104e-06_rb,4.3915e-06_rb,3.0485e-06_rb,4.9566e-08_rb/)
      kbo(:, 5,59, 5) = (/ &
     &8.4320e-06_rb,8.5398e-06_rb,7.2464e-06_rb,4.9418e-06_rb,6.8105e-08_rb/)
      kbo(:, 1,13, 6) = (/ &
     &1.5548e-01_rb,1.2226e-01_rb,8.5068e-02_rb,4.5702e-02_rb,2.0740e-03_rb/)
      kbo(:, 2,13, 6) = (/ &
     &2.1897e-01_rb,1.7240e-01_rb,1.2003e-01_rb,6.4538e-02_rb,2.5124e-03_rb/)
      kbo(:, 3,13, 6) = (/ &
     &3.0095e-01_rb,2.3621e-01_rb,1.6479e-01_rb,8.8859e-02_rb,3.0382e-03_rb/)
      kbo(:, 4,13, 6) = (/ &
     &4.0485e-01_rb,3.1719e-01_rb,2.2079e-01_rb,1.1848e-01_rb,3.6959e-03_rb/)
      kbo(:, 5,13, 6) = (/ &
     &5.3118e-01_rb,4.1467e-01_rb,2.8796e-01_rb,1.5438e-01_rb,4.3103e-03_rb/)
      kbo(:, 1,14, 6) = (/ &
     &1.3732e-01_rb,1.0825e-01_rb,7.5397e-02_rb,4.0495e-02_rb,1.7538e-03_rb/)
      kbo(:, 2,14, 6) = (/ &
     &1.9463e-01_rb,1.5344e-01_rb,1.0670e-01_rb,5.7417e-02_rb,2.1725e-03_rb/)
      kbo(:, 3,14, 6) = (/ &
     &2.6896e-01_rb,2.1173e-01_rb,1.4739e-01_rb,7.9254e-02_rb,2.6810e-03_rb/)
      kbo(:, 4,14, 6) = (/ &
     &3.6294e-01_rb,2.8446e-01_rb,1.9761e-01_rb,1.0591e-01_rb,3.1748e-03_rb/)
      kbo(:, 5,14, 6) = (/ &
     &4.7752e-01_rb,3.7323e-01_rb,2.5905e-01_rb,1.3864e-01_rb,3.7684e-03_rb/)
      kbo(:, 1,15, 6) = (/ &
     &1.1609e-01_rb,9.1864e-02_rb,6.4056e-02_rb,3.4496e-02_rb,1.4724e-03_rb/)
      kbo(:, 2,15, 6) = (/ &
     &1.6575e-01_rb,1.3099e-01_rb,9.1234e-02_rb,4.9161e-02_rb,1.8695e-03_rb/)
      kbo(:, 3,15, 6) = (/ &
     &2.2962e-01_rb,1.8109e-01_rb,1.2603e-01_rb,6.7814e-02_rb,2.2935e-03_rb/)
      kbo(:, 4,15, 6) = (/ &
     &3.1077e-01_rb,2.4430e-01_rb,1.6934e-01_rb,9.0957e-02_rb,2.7633e-03_rb/)
      kbo(:, 5,15, 6) = (/ &
     &4.0968e-01_rb,3.2160e-01_rb,2.2302e-01_rb,1.1925e-01_rb,3.2819e-03_rb/)
      kbo(:, 1,16, 6) = (/ &
     &9.6387e-02_rb,7.6493e-02_rb,5.3335e-02_rb,2.8835e-02_rb,1.2273e-03_rb/)
      kbo(:, 2,16, 6) = (/ &
     &1.3769e-01_rb,1.0917e-01_rb,7.6074e-02_rb,4.1117e-02_rb,1.5607e-03_rb/)
      kbo(:, 3,16, 6) = (/ &
     &1.9131e-01_rb,1.5123e-01_rb,1.0523e-01_rb,5.6738e-02_rb,1.9638e-03_rb/)
      kbo(:, 4,16, 6) = (/ &
     &2.5899e-01_rb,2.0399e-01_rb,1.4165e-01_rb,7.6203e-02_rb,2.3618e-03_rb/)
      kbo(:, 5,16, 6) = (/ &
     &3.4214e-01_rb,2.6934e-01_rb,1.8675e-01_rb,1.0014e-01_rb,2.8281e-03_rb/)
      kbo(:, 1,17, 6) = (/ &
     &7.9271e-02_rb,6.3126e-02_rb,4.4100e-02_rb,2.3917e-02_rb,1.0217e-03_rb/)
      kbo(:, 2,17, 6) = (/ &
     &1.1345e-01_rb,9.0182e-02_rb,6.2977e-02_rb,3.4156e-02_rb,1.3299e-03_rb/)
      kbo(:, 3,17, 6) = (/ &
     &1.5797e-01_rb,1.2511e-01_rb,8.7087e-02_rb,4.7059e-02_rb,1.6628e-03_rb/)
      kbo(:, 4,17, 6) = (/ &
     &2.1402e-01_rb,1.6897e-01_rb,1.1752e-01_rb,6.3332e-02_rb,2.0151e-03_rb/)
      kbo(:, 5,17, 6) = (/ &
     &2.8305e-01_rb,2.2332e-01_rb,1.5502e-01_rb,8.3415e-02_rb,2.4638e-03_rb/)
      kbo(:, 1,18, 6) = (/ &
     &6.4211e-02_rb,5.1302e-02_rb,3.5902e-02_rb,1.9554e-02_rb,8.7983e-04_rb/)
      kbo(:, 2,18, 6) = (/ &
     &9.2120e-02_rb,7.3433e-02_rb,5.1355e-02_rb,2.7890e-02_rb,1.1536e-03_rb/)
      kbo(:, 3,18, 6) = (/ &
     &1.2833e-01_rb,1.0186e-01_rb,7.1081e-02_rb,3.8483e-02_rb,1.4378e-03_rb/)
      kbo(:, 4,18, 6) = (/ &
     &1.7390e-01_rb,1.3787e-01_rb,9.6012e-02_rb,5.1930e-02_rb,1.7641e-03_rb/)
      kbo(:, 5,18, 6) = (/ &
     &2.3011e-01_rb,1.8206e-01_rb,1.2668e-01_rb,6.8404e-02_rb,2.1654e-03_rb/)
      kbo(:, 1,19, 6) = (/ &
     &5.2788e-02_rb,4.2299e-02_rb,2.9673e-02_rb,1.6227e-02_rb,7.6310e-04_rb/)
      kbo(:, 2,19, 6) = (/ &
     &7.5889e-02_rb,6.0648e-02_rb,4.2457e-02_rb,2.3108e-02_rb,1.0035e-03_rb/)
      kbo(:, 3,19, 6) = (/ &
     &1.0575e-01_rb,8.4190e-02_rb,5.8852e-02_rb,3.1993e-02_rb,1.2648e-03_rb/)
      kbo(:, 4,19, 6) = (/ &
     &1.4328e-01_rb,1.1394e-01_rb,7.9533e-02_rb,4.3155e-02_rb,1.5714e-03_rb/)
      kbo(:, 5,19, 6) = (/ &
     &1.8940e-01_rb,1.5028e-01_rb,1.0488e-01_rb,5.6817e-02_rb,1.9149e-03_rb/)
      kbo(:, 1,20, 6) = (/ &
     &4.4520e-02_rb,3.5758e-02_rb,2.5128e-02_rb,1.3753e-02_rb,6.6571e-04_rb/)
      kbo(:, 2,20, 6) = (/ &
     &6.4045e-02_rb,5.1271e-02_rb,3.5901e-02_rb,1.9582e-02_rb,8.7391e-04_rb/)
      kbo(:, 3,20, 6) = (/ &
     &8.9082e-02_rb,7.1164e-02_rb,4.9821e-02_rb,2.7122e-02_rb,1.1092e-03_rb/)
      kbo(:, 4,20, 6) = (/ &
     &1.2061e-01_rb,9.6051e-02_rb,6.7127e-02_rb,3.6518e-02_rb,1.3635e-03_rb/)
      kbo(:, 5,20, 6) = (/ &
     &1.5922e-01_rb,1.2658e-01_rb,8.8405e-02_rb,4.7977e-02_rb,1.6664e-03_rb/)
      kbo(:, 1,21, 6) = (/ &
     &3.7777e-02_rb,3.0383e-02_rb,2.1331e-02_rb,1.1694e-02_rb,5.7845e-04_rb/)
      kbo(:, 2,21, 6) = (/ &
     &5.4331e-02_rb,4.3484e-02_rb,3.0481e-02_rb,1.6684e-02_rb,7.5762e-04_rb/)
      kbo(:, 3,21, 6) = (/ &
     &7.5428e-02_rb,6.0294e-02_rb,4.2245e-02_rb,2.3058e-02_rb,9.5396e-04_rb/)
      kbo(:, 4,21, 6) = (/ &
     &1.0181e-01_rb,8.1193e-02_rb,5.6844e-02_rb,3.0978e-02_rb,1.1796e-03_rb/)
      kbo(:, 5,21, 6) = (/ &
     &1.3419e-01_rb,1.0693e-01_rb,7.4825e-02_rb,4.0693e-02_rb,1.4376e-03_rb/)
      kbo(:, 1,22, 6) = (/ &
     &3.2832e-02_rb,2.6387e-02_rb,1.8543e-02_rb,1.0190e-02_rb,5.0764e-04_rb/)
      kbo(:, 2,22, 6) = (/ &
     &4.6987e-02_rb,3.7684e-02_rb,2.6458e-02_rb,1.4480e-02_rb,6.6076e-04_rb/)
      kbo(:, 3,22, 6) = (/ &
     &6.5046e-02_rb,5.2018e-02_rb,3.6448e-02_rb,1.9932e-02_rb,8.2689e-04_rb/)
      kbo(:, 4,22, 6) = (/ &
     &8.7447e-02_rb,6.9814e-02_rb,4.8898e-02_rb,2.6679e-02_rb,1.0180e-03_rb/)
      kbo(:, 5,22, 6) = (/ &
     &1.1504e-01_rb,9.1878e-02_rb,6.4286e-02_rb,3.4972e-02_rb,1.2412e-03_rb/)
      kbo(:, 1,23, 6) = (/ &
     &2.8625e-02_rb,2.3013e-02_rb,1.6180e-02_rb,8.8954e-03_rb,4.4551e-04_rb/)
      kbo(:, 2,23, 6) = (/ &
     &4.0740e-02_rb,3.2715e-02_rb,2.2979e-02_rb,1.2589e-02_rb,5.7081e-04_rb/)
      kbo(:, 3,23, 6) = (/ &
     &5.6140e-02_rb,4.4931e-02_rb,3.1539e-02_rb,1.7244e-02_rb,7.1483e-04_rb/)
      kbo(:, 4,23, 6) = (/ &
     &7.5282e-02_rb,6.0221e-02_rb,4.2218e-02_rb,2.3054e-02_rb,8.7570e-04_rb/)
      kbo(:, 5,23, 6) = (/ &
     &9.9200e-02_rb,7.9197e-02_rb,5.5422e-02_rb,3.0138e-02_rb,1.0690e-03_rb/)
      kbo(:, 1,24, 6) = (/ &
     &2.5225e-02_rb,2.0294e-02_rb,1.4276e-02_rb,7.8427e-03_rb,3.8602e-04_rb/)
      kbo(:, 2,24, 6) = (/ &
     &3.5712e-02_rb,2.8695e-02_rb,2.0149e-02_rb,1.1049e-02_rb,4.9372e-04_rb/)
      kbo(:, 3,24, 6) = (/ &
     &4.9060e-02_rb,3.9310e-02_rb,2.7586e-02_rb,1.5091e-02_rb,6.1539e-04_rb/)
      kbo(:, 4,24, 6) = (/ &
     &6.5785e-02_rb,5.2643e-02_rb,3.6889e-02_rb,2.0124e-02_rb,7.5365e-04_rb/)
      kbo(:, 5,24, 6) = (/ &
     &8.6520e-02_rb,6.9119e-02_rb,4.8335e-02_rb,2.6295e-02_rb,9.2000e-04_rb/)
      kbo(:, 1,25, 6) = (/ &
     &2.2344e-02_rb,1.7976e-02_rb,1.2645e-02_rb,6.9412e-03_rb,3.3195e-04_rb/)
      kbo(:, 2,25, 6) = (/ &
     &3.1531e-02_rb,2.5281e-02_rb,1.7784e-02_rb,9.7415e-03_rb,4.2393e-04_rb/)
      kbo(:, 3,25, 6) = (/ &
     &4.3153e-02_rb,3.4621e-02_rb,2.4312e-02_rb,1.3288e-02_rb,5.2565e-04_rb/)
      kbo(:, 4,25, 6) = (/ &
     &5.7875e-02_rb,4.6285e-02_rb,3.2426e-02_rb,1.7676e-02_rb,6.4832e-04_rb/)
      kbo(:, 5,25, 6) = (/ &
     &7.6134e-02_rb,6.0793e-02_rb,4.2524e-02_rb,2.3109e-02_rb,7.8380e-04_rb/)
      kbo(:, 1,26, 6) = (/ &
     &1.9994e-02_rb,1.6102e-02_rb,1.1323e-02_rb,6.2216e-03_rb,2.8629e-04_rb/)
      kbo(:, 2,26, 6) = (/ &
     &2.8079e-02_rb,2.2581e-02_rb,1.5869e-02_rb,8.6875e-03_rb,3.6314e-04_rb/)
      kbo(:, 3,26, 6) = (/ &
     &3.8417e-02_rb,3.0834e-02_rb,2.1637e-02_rb,1.1805e-02_rb,4.5229e-04_rb/)
      kbo(:, 4,26, 6) = (/ &
     &5.1448e-02_rb,4.1174e-02_rb,2.8844e-02_rb,1.5712e-02_rb,5.5890e-04_rb/)
      kbo(:, 5,26, 6) = (/ &
     &6.7708e-02_rb,5.4151e-02_rb,3.7876e-02_rb,2.0559e-02_rb,6.6983e-04_rb/)
      kbo(:, 1,27, 6) = (/ &
     &1.7896e-02_rb,1.4429e-02_rb,1.0152e-02_rb,5.5772e-03_rb,2.4427e-04_rb/)
      kbo(:, 2,27, 6) = (/ &
     &2.5042e-02_rb,2.0157e-02_rb,1.4173e-02_rb,7.7571e-03_rb,3.0979e-04_rb/)
      kbo(:, 3,27, 6) = (/ &
     &3.4208e-02_rb,2.7475e-02_rb,1.9284e-02_rb,1.0527e-02_rb,3.8671e-04_rb/)
      kbo(:, 4,27, 6) = (/ &
     &4.5837e-02_rb,3.6717e-02_rb,2.5729e-02_rb,1.4005e-02_rb,4.7399e-04_rb/)
      kbo(:, 5,27, 6) = (/ &
     &6.0284e-02_rb,4.8205e-02_rb,3.3722e-02_rb,1.8275e-02_rb,5.6657e-04_rb/)
      kbo(:, 1,28, 6) = (/ &
     &1.6009e-02_rb,1.2912e-02_rb,9.0950e-03_rb,4.9907e-03_rb,2.0749e-04_rb/)
      kbo(:, 2,28, 6) = (/ &
     &2.2344e-02_rb,1.7993e-02_rb,1.2646e-02_rb,6.9273e-03_rb,2.6196e-04_rb/)
      kbo(:, 3,28, 6) = (/ &
     &3.0516e-02_rb,2.4521e-02_rb,1.7216e-02_rb,9.3897e-03_rb,3.2594e-04_rb/)
      kbo(:, 4,28, 6) = (/ &
     &4.0856e-02_rb,3.2755e-02_rb,2.2958e-02_rb,1.2475e-02_rb,3.9748e-04_rb/)
      kbo(:, 5,28, 6) = (/ &
     &5.3627e-02_rb,4.2900e-02_rb,3.0014e-02_rb,1.6247e-02_rb,4.7387e-04_rb/)
      kbo(:, 1,29, 6) = (/ &
     &1.4262e-02_rb,1.1522e-02_rb,8.1115e-03_rb,4.4530e-03_rb,1.7407e-04_rb/)
      kbo(:, 2,29, 6) = (/ &
     &1.9887e-02_rb,1.6027e-02_rb,1.1276e-02_rb,6.1687e-03_rb,2.1854e-04_rb/)
      kbo(:, 3,29, 6) = (/ &
     &2.7141e-02_rb,2.1827e-02_rb,1.5323e-02_rb,8.3518e-03_rb,2.7116e-04_rb/)
      kbo(:, 4,29, 6) = (/ &
     &3.6246e-02_rb,2.9092e-02_rb,2.0386e-02_rb,1.1064e-02_rb,3.2874e-04_rb/)
      kbo(:, 5,29, 6) = (/ &
     &4.7427e-02_rb,3.8019e-02_rb,2.6585e-02_rb,1.4385e-02_rb,3.8972e-04_rb/)
      kbo(:, 1,30, 6) = (/ &
     &1.2676e-02_rb,1.0246e-02_rb,7.2173e-03_rb,3.9637e-03_rb,1.4416e-04_rb/)
      kbo(:, 2,30, 6) = (/ &
     &1.7653e-02_rb,1.4244e-02_rb,1.0017e-02_rb,5.4770e-03_rb,1.8113e-04_rb/)
      kbo(:, 3,30, 6) = (/ &
     &2.4022e-02_rb,1.9341e-02_rb,1.3583e-02_rb,7.3998e-03_rb,2.2282e-04_rb/)
      kbo(:, 4,30, 6) = (/ &
     &3.1996e-02_rb,2.5721e-02_rb,1.8027e-02_rb,9.7785e-03_rb,2.6784e-04_rb/)
      kbo(:, 5,30, 6) = (/ &
     &4.1773e-02_rb,3.3512e-02_rb,2.3450e-02_rb,1.2688e-02_rb,3.1737e-04_rb/)
      kbo(:, 1,31, 6) = (/ &
     &1.1178e-02_rb,9.0514e-03_rb,6.3801e-03_rb,3.5048e-03_rb,1.1770e-04_rb/)
      kbo(:, 2,31, 6) = (/ &
     &1.5523e-02_rb,1.2547e-02_rb,8.8303e-03_rb,4.8276e-03_rb,1.4724e-04_rb/)
      kbo(:, 3,31, 6) = (/ &
     &2.1060e-02_rb,1.6992e-02_rb,1.1943e-02_rb,6.5025e-03_rb,1.7875e-04_rb/)
      kbo(:, 4,31, 6) = (/ &
     &2.7976e-02_rb,2.2531e-02_rb,1.5799e-02_rb,8.5667e-03_rb,2.1450e-04_rb/)
      kbo(:, 5,31, 6) = (/ &
     &3.6405e-02_rb,2.9271e-02_rb,2.0490e-02_rb,1.1088e-02_rb,2.5503e-04_rb/)
      kbo(:, 1,32, 6) = (/ &
     &9.9013e-03_rb,8.0343e-03_rb,5.6674e-03_rb,3.1129e-03_rb,9.6361e-05_rb/)
      kbo(:, 2,32, 6) = (/ &
     &1.3706e-02_rb,1.1101e-02_rb,7.8125e-03_rb,4.2746e-03_rb,1.1902e-04_rb/)
      kbo(:, 3,32, 6) = (/ &
     &1.8546e-02_rb,1.4995e-02_rb,1.0536e-02_rb,5.7328e-03_rb,1.4373e-04_rb/)
      kbo(:, 4,32, 6) = (/ &
     &2.4550e-02_rb,1.9811e-02_rb,1.3886e-02_rb,7.5313e-03_rb,1.7332e-04_rb/)
      kbo(:, 5,32, 6) = (/ &
     &3.1853e-02_rb,2.5667e-02_rb,1.7969e-02_rb,9.7246e-03_rb,2.0625e-04_rb/)
      kbo(:, 1,33, 6) = (/ &
     &8.7980e-03_rb,7.1519e-03_rb,5.0451e-03_rb,2.7712e-03_rb,7.8202e-05_rb/)
      kbo(:, 2,33, 6) = (/ &
     &1.2135e-02_rb,9.8435e-03_rb,6.9325e-03_rb,3.7902e-03_rb,9.6250e-05_rb/)
      kbo(:, 3,33, 6) = (/ &
     &1.6367e-02_rb,1.3251e-02_rb,9.3075e-03_rb,5.0637e-03_rb,1.1723e-04_rb/)
      kbo(:, 4,33, 6) = (/ &
     &2.1594e-02_rb,1.7451e-02_rb,1.2238e-02_rb,6.6390e-03_rb,1.4129e-04_rb/)
      kbo(:, 5,33, 6) = (/ &
     &2.7922e-02_rb,2.2553e-02_rb,1.5798e-02_rb,8.5480e-03_rb,1.6598e-04_rb/)
      kbo(:, 1,34, 6) = (/ &
     &7.8297e-03_rb,6.3727e-03_rb,4.4952e-03_rb,2.4671e-03_rb,6.3879e-05_rb/)
      kbo(:, 2,34, 6) = (/ &
     &1.0770e-02_rb,8.7510e-03_rb,6.1616e-03_rb,3.3644e-03_rb,7.9137e-05_rb/)
      kbo(:, 3,34, 6) = (/ &
     &1.4484e-02_rb,1.1741e-02_rb,8.2482e-03_rb,4.4844e-03_rb,9.6600e-05_rb/)
      kbo(:, 4,34, 6) = (/ &
     &1.9050e-02_rb,1.5427e-02_rb,1.0820e-02_rb,5.8678e-03_rb,1.1495e-04_rb/)
      kbo(:, 5,34, 6) = (/ &
     &2.4524e-02_rb,1.9867e-02_rb,1.3919e-02_rb,7.5292e-03_rb,1.3453e-04_rb/)
      kbo(:, 1,35, 6) = (/ &
     &6.7630e-03_rb,5.5126e-03_rb,3.8884e-03_rb,2.1340e-03_rb,5.1721e-05_rb/)
      kbo(:, 2,35, 6) = (/ &
     &9.2985e-03_rb,7.5662e-03_rb,5.3252e-03_rb,2.9070e-03_rb,6.4032e-05_rb/)
      kbo(:, 3,35, 6) = (/ &
     &1.2489e-02_rb,1.0141e-02_rb,7.1246e-03_rb,3.8749e-03_rb,7.6974e-05_rb/)
      kbo(:, 4,35, 6) = (/ &
     &1.6400e-02_rb,1.3312e-02_rb,9.3389e-03_rb,5.0659e-03_rb,9.1148e-05_rb/)
      kbo(:, 5,35, 6) = (/ &
     &2.1049e-02_rb,1.7098e-02_rb,1.1985e-02_rb,6.4857e-03_rb,1.0683e-04_rb/)
      kbo(:, 1,36, 6) = (/ &
     &5.6496e-03_rb,4.6119e-03_rb,3.2530e-03_rb,1.7852e-03_rb,4.0312e-05_rb/)
      kbo(:, 2,36, 6) = (/ &
     &7.7767e-03_rb,6.3372e-03_rb,4.4630e-03_rb,2.4380e-03_rb,4.9929e-05_rb/)
      kbo(:, 3,36, 6) = (/ &
     &1.0461e-02_rb,8.5144e-03_rb,5.9836e-03_rb,3.2549e-03_rb,6.0118e-05_rb/)
      kbo(:, 4,36, 6) = (/ &
     &1.3740e-02_rb,1.1182e-02_rb,7.8492e-03_rb,4.2598e-03_rb,7.1064e-05_rb/)
      kbo(:, 5,36, 6) = (/ &
     &1.7612e-02_rb,1.4350e-02_rb,1.0064e-02_rb,5.4485e-03_rb,8.3577e-05_rb/)
      kbo(:, 1,37, 6) = (/ &
     &4.5423e-03_rb,3.7134e-03_rb,2.6216e-03_rb,1.4392e-03_rb,3.1256e-05_rb/)
      kbo(:, 2,37, 6) = (/ &
     &6.2879e-03_rb,5.1321e-03_rb,3.6149e-03_rb,1.9739e-03_rb,3.8734e-05_rb/)
      kbo(:, 3,37, 6) = (/ &
     &8.4900e-03_rb,6.9227e-03_rb,4.8667e-03_rb,2.6482e-03_rb,4.6682e-05_rb/)
      kbo(:, 4,37, 6) = (/ &
     &1.1183e-02_rb,9.1172e-03_rb,6.4017e-03_rb,3.4761e-03_rb,5.5592e-05_rb/)
      kbo(:, 5,37, 6) = (/ &
     &1.4352e-02_rb,1.1719e-02_rb,8.2209e-03_rb,4.4549e-03_rb,6.5503e-05_rb/)
      kbo(:, 1,38, 6) = (/ &
     &3.6392e-03_rb,2.9801e-03_rb,2.1050e-03_rb,1.1562e-03_rb,2.4090e-05_rb/)
      kbo(:, 2,38, 6) = (/ &
     &5.0658e-03_rb,4.1416e-03_rb,2.9194e-03_rb,1.5946e-03_rb,2.9868e-05_rb/)
      kbo(:, 3,38, 6) = (/ &
     &6.8711e-03_rb,5.6107e-03_rb,3.9454e-03_rb,2.1481e-03_rb,3.6072e-05_rb/)
      kbo(:, 4,38, 6) = (/ &
     &9.0731e-03_rb,7.4127e-03_rb,5.2066e-03_rb,2.8281e-03_rb,4.3140e-05_rb/)
      kbo(:, 5,38, 6) = (/ &
     &1.1663e-02_rb,9.5418e-03_rb,6.6954e-03_rb,3.6310e-03_rb,5.1139e-05_rb/)
      kbo(:, 1,39, 6) = (/ &
     &2.9118e-03_rb,2.3886e-03_rb,1.6882e-03_rb,9.2755e-04_rb,1.8391e-05_rb/)
      kbo(:, 2,39, 6) = (/ &
     &4.0750e-03_rb,3.3381e-03_rb,2.3540e-03_rb,1.2863e-03_rb,2.2806e-05_rb/)
      kbo(:, 3,39, 6) = (/ &
     &5.5513e-03_rb,4.5420e-03_rb,3.1948e-03_rb,1.7397e-03_rb,2.7682e-05_rb/)
      kbo(:, 4,39, 6) = (/ &
     &7.3492e-03_rb,6.0179e-03_rb,4.2285e-03_rb,2.2979e-03_rb,3.3352e-05_rb/)
      kbo(:, 5,39, 6) = (/ &
     &9.4667e-03_rb,7.7616e-03_rb,5.4485e-03_rb,2.9560e-03_rb,3.9651e-05_rb/)
      kbo(:, 1,40, 6) = (/ &
     &2.2712e-03_rb,1.8658e-03_rb,1.3192e-03_rb,7.2479e-04_rb,1.3995e-05_rb/)
      kbo(:, 2,40, 6) = (/ &
     &3.2040e-03_rb,2.6288e-03_rb,1.8549e-03_rb,1.0140e-03_rb,1.7455e-05_rb/)
      kbo(:, 3,40, 6) = (/ &
     &4.3985e-03_rb,3.6050e-03_rb,2.5372e-03_rb,1.3813e-03_rb,2.1295e-05_rb/)
      kbo(:, 4,40, 6) = (/ &
     &5.8565e-03_rb,4.8013e-03_rb,3.3753e-03_rb,1.8342e-03_rb,2.5756e-05_rb/)
      kbo(:, 5,40, 6) = (/ &
     &7.5776e-03_rb,6.2219e-03_rb,4.3685e-03_rb,2.3699e-03_rb,3.0715e-05_rb/)
      kbo(:, 1,41, 6) = (/ &
     &1.7612e-03_rb,1.4492e-03_rb,1.0250e-03_rb,5.6325e-04_rb,1.0604e-05_rb/)
      kbo(:, 2,41, 6) = (/ &
     &2.5080e-03_rb,2.0608e-03_rb,1.4546e-03_rb,7.9552e-04_rb,1.3310e-05_rb/)
      kbo(:, 3,41, 6) = (/ &
     &3.4681e-03_rb,2.8477e-03_rb,2.0051e-03_rb,1.0923e-03_rb,1.6349e-05_rb/)
      kbo(:, 4,41, 6) = (/ &
     &4.6501e-03_rb,3.8188e-03_rb,2.6845e-03_rb,1.4588e-03_rb,1.9855e-05_rb/)
      kbo(:, 5,41, 6) = (/ &
     &6.0491e-03_rb,4.9741e-03_rb,3.4944e-03_rb,1.8953e-03_rb,2.3793e-05_rb/)
      kbo(:, 1,42, 6) = (/ &
     &1.3619e-03_rb,1.1225e-03_rb,7.9390e-04_rb,4.3654e-04_rb,7.9892e-06_rb/)
      kbo(:, 2,42, 6) = (/ &
     &1.9585e-03_rb,1.6109e-03_rb,1.1375e-03_rb,6.2244e-04_rb,1.0111e-05_rb/)
      kbo(:, 3,42, 6) = (/ &
     &2.7289e-03_rb,2.2437e-03_rb,1.5805e-03_rb,8.6127e-04_rb,1.2500e-05_rb/)
      kbo(:, 4,42, 6) = (/ &
     &3.6852e-03_rb,3.0311e-03_rb,2.1316e-03_rb,1.1583e-03_rb,1.5248e-05_rb/)
      kbo(:, 5,42, 6) = (/ &
     &4.8235e-03_rb,3.9702e-03_rb,2.7892e-03_rb,1.5134e-03_rb,1.8350e-05_rb/)
      kbo(:, 1,43, 6) = (/ &
     &1.0392e-03_rb,8.5695e-04_rb,6.0598e-04_rb,3.3332e-04_rb,5.9552e-06_rb/)
      kbo(:, 2,43, 6) = (/ &
     &1.5101e-03_rb,1.2436e-03_rb,8.7832e-04_rb,4.8071e-04_rb,7.5961e-06_rb/)
      kbo(:, 3,43, 6) = (/ &
     &2.1254e-03_rb,1.7498e-03_rb,1.2325e-03_rb,6.7181e-04_rb,9.4477e-06_rb/)
      kbo(:, 4,43, 6) = (/ &
     &2.8974e-03_rb,2.3860e-03_rb,1.6788e-03_rb,9.1195e-04_rb,1.1585e-05_rb/)
      kbo(:, 5,43, 6) = (/ &
     &3.8244e-03_rb,3.1517e-03_rb,2.2140e-03_rb,1.2005e-03_rb,1.4013e-05_rb/)
      kbo(:, 1,44, 6) = (/ &
     &7.8683e-04_rb,6.4875e-04_rb,4.5878e-04_rb,2.5223e-04_rb,4.3968e-06_rb/)
      kbo(:, 2,44, 6) = (/ &
     &1.1559e-03_rb,9.5305e-04_rb,6.7277e-04_rb,3.6830e-04_rb,5.6394e-06_rb/)
      kbo(:, 3,44, 6) = (/ &
     &1.6454e-03_rb,1.3558e-03_rb,9.5534e-04_rb,5.2055e-04_rb,7.0859e-06_rb/)
      kbo(:, 4,44, 6) = (/ &
     &2.2669e-03_rb,1.8686e-03_rb,1.3148e-03_rb,7.1399e-04_rb,8.7462e-06_rb/)
      kbo(:, 5,44, 6) = (/ &
     &3.0201e-03_rb,2.4917e-03_rb,1.7509e-03_rb,9.4904e-04_rb,1.0639e-05_rb/)
      kbo(:, 1,45, 6) = (/ &
     &5.9365e-04_rb,4.8959e-04_rb,3.4607e-04_rb,1.9020e-04_rb,3.2229e-06_rb/)
      kbo(:, 2,45, 6) = (/ &
     &8.8218e-04_rb,7.2803e-04_rb,5.1390e-04_rb,2.8115e-04_rb,4.1709e-06_rb/)
      kbo(:, 3,45, 6) = (/ &
     &1.2705e-03_rb,1.0482e-03_rb,7.3838e-04_rb,4.0225e-04_rb,5.2818e-06_rb/)
      kbo(:, 4,45, 6) = (/ &
     &1.7692e-03_rb,1.4597e-03_rb,1.0269e-03_rb,5.5768e-04_rb,6.5567e-06_rb/)
      kbo(:, 5,45, 6) = (/ &
     &2.3806e-03_rb,1.9657e-03_rb,1.3812e-03_rb,7.4841e-04_rb,8.0347e-06_rb/)
      kbo(:, 1,46, 6) = (/ &
     &4.4335e-04_rb,3.6560e-04_rb,2.5820e-04_rb,1.4186e-04_rb,2.3394e-06_rb/)
      kbo(:, 2,46, 6) = (/ &
     &6.6652e-04_rb,5.5048e-04_rb,3.8857e-04_rb,2.1254e-04_rb,3.0543e-06_rb/)
      kbo(:, 3,46, 6) = (/ &
     &9.7139e-04_rb,8.0233e-04_rb,5.6521e-04_rb,3.0796e-04_rb,3.9016e-06_rb/)
      kbo(:, 4,46, 6) = (/ &
     &1.3697e-03_rb,1.1313e-03_rb,7.9539e-04_rb,4.3177e-04_rb,4.8930e-06_rb/)
      kbo(:, 5,46, 6) = (/ &
     &1.8630e-03_rb,1.5401e-03_rb,1.0822e-03_rb,5.8603e-04_rb,6.0211e-06_rb/)
      kbo(:, 1,47, 6) = (/ &
     &3.2494e-04_rb,2.6781e-04_rb,1.8906e-04_rb,1.0379e-04_rb,1.6703e-06_rb/)
      kbo(:, 2,47, 6) = (/ &
     &4.9477e-04_rb,4.0876e-04_rb,2.8822e-04_rb,1.5780e-04_rb,2.2235e-06_rb/)
      kbo(:, 3,47, 6) = (/ &
     &7.3190e-04_rb,6.0441e-04_rb,4.2581e-04_rb,2.3196e-04_rb,2.8600e-06_rb/)
      kbo(:, 4,47, 6) = (/ &
     &1.0450e-03_rb,8.6362e-04_rb,6.0711e-04_rb,3.2954e-04_rb,3.6203e-06_rb/)
      kbo(:, 5,47, 6) = (/ &
     &1.4403e-03_rb,1.1920e-03_rb,8.3715e-04_rb,4.5311e-04_rb,4.4864e-06_rb/)
      kbo(:, 1,48, 6) = (/ &
     &2.3680e-04_rb,1.9527e-04_rb,1.3764e-04_rb,7.5497e-05_rb,1.1789e-06_rb/)
      kbo(:, 2,48, 6) = (/ &
     &3.6634e-04_rb,3.0233e-04_rb,2.1313e-04_rb,1.1661e-04_rb,1.5963e-06_rb/)
      kbo(:, 3,48, 6) = (/ &
     &5.4895e-04_rb,4.5372e-04_rb,3.1944e-04_rb,1.7402e-04_rb,2.0888e-06_rb/)
      kbo(:, 4,48, 6) = (/ &
     &7.9543e-04_rb,6.5717e-04_rb,4.6181e-04_rb,2.5062e-04_rb,2.6621e-06_rb/)
      kbo(:, 5,48, 6) = (/ &
     &1.1099e-03_rb,9.1890e-04_rb,6.4520e-04_rb,3.4917e-04_rb,3.3226e-06_rb/)
      kbo(:, 1,49, 6) = (/ &
     &1.7167e-04_rb,1.4156e-04_rb,9.9697e-05_rb,5.4621e-05_rb,8.2177e-07_rb/)
      kbo(:, 2,49, 6) = (/ &
     &2.6978e-04_rb,2.2270e-04_rb,1.5688e-04_rb,8.5772e-05_rb,1.1335e-06_rb/)
      kbo(:, 3,49, 6) = (/ &
     &4.1033e-04_rb,3.3911e-04_rb,2.3859e-04_rb,1.2993e-04_rb,1.5022e-06_rb/)
      kbo(:, 4,49, 6) = (/ &
     &6.0289e-04_rb,4.9870e-04_rb,3.5040e-04_rb,1.9001e-04_rb,1.9421e-06_rb/)
      kbo(:, 5,49, 6) = (/ &
     &8.5268e-04_rb,7.0612e-04_rb,4.9559e-04_rb,2.6808e-04_rb,2.4464e-06_rb/)
      kbo(:, 1,50, 6) = (/ &
     &1.2531e-04_rb,1.0333e-04_rb,7.2754e-05_rb,3.9781e-05_rb,5.8260e-07_rb/)
      kbo(:, 2,50, 6) = (/ &
     &2.0028e-04_rb,1.6520e-04_rb,1.1625e-04_rb,6.3517e-05_rb,8.1312e-07_rb/)
      kbo(:, 3,50, 6) = (/ &
     &3.0881e-04_rb,2.5524e-04_rb,1.7955e-04_rb,9.7765e-05_rb,1.0963e-06_rb/)
      kbo(:, 4,50, 6) = (/ &
     &4.6015e-04_rb,3.8061e-04_rb,2.6731e-04_rb,1.4488e-04_rb,1.4340e-06_rb/)
      kbo(:, 5,50, 6) = (/ &
     &6.5988e-04_rb,5.4636e-04_rb,3.8320e-04_rb,2.0709e-04_rb,1.8202e-06_rb/)
      kbo(:, 1,51, 6) = (/ &
     &9.1633e-05_rb,7.5509e-05_rb,5.3128e-05_rb,2.9020e-05_rb,4.1464e-07_rb/)
      kbo(:, 2,51, 6) = (/ &
     &1.4891e-04_rb,1.2270e-04_rb,8.6279e-05_rb,4.7099e-05_rb,5.8807e-07_rb/)
      kbo(:, 3,51, 6) = (/ &
     &2.3292e-04_rb,1.9249e-04_rb,1.3529e-04_rb,7.3646e-05_rb,8.0371e-07_rb/)
      kbo(:, 4,51, 6) = (/ &
     &3.5233e-04_rb,2.9123e-04_rb,2.0433e-04_rb,1.1072e-04_rb,1.0615e-06_rb/)
      kbo(:, 5,51, 6) = (/ &
     &5.1166e-04_rb,4.2368e-04_rb,2.9697e-04_rb,1.6038e-04_rb,1.3623e-06_rb/)
      kbo(:, 1,52, 6) = (/ &
     &6.6748e-05_rb,5.4951e-05_rb,3.8637e-05_rb,2.1072e-05_rb,2.9423e-07_rb/)
      kbo(:, 2,52, 6) = (/ &
     &1.1034e-04_rb,9.0898e-05_rb,6.3804e-05_rb,3.4773e-05_rb,4.2185e-07_rb/)
      kbo(:, 3,52, 6) = (/ &
     &1.7525e-04_rb,1.4472e-04_rb,1.0161e-04_rb,5.5277e-05_rb,5.8386e-07_rb/)
      kbo(:, 4,52, 6) = (/ &
     &2.6896e-04_rb,2.2246e-04_rb,1.5598e-04_rb,8.4497e-05_rb,7.8029e-07_rb/)
      kbo(:, 5,52, 6) = (/ &
     &3.9638e-04_rb,3.2814e-04_rb,2.2978e-04_rb,1.2397e-04_rb,1.0114e-06_rb/)
      kbo(:, 1,53, 6) = (/ &
     &4.8387e-05_rb,3.9789e-05_rb,2.7956e-05_rb,1.5224e-05_rb,2.0710e-07_rb/)
      kbo(:, 2,53, 6) = (/ &
     &8.1422e-05_rb,6.7055e-05_rb,4.7034e-05_rb,2.5569e-05_rb,3.0032e-07_rb/)
      kbo(:, 3,53, 6) = (/ &
     &1.3143e-04_rb,1.0846e-04_rb,7.6092e-05_rb,4.1325e-05_rb,4.2132e-07_rb/)
      kbo(:, 4,53, 6) = (/ &
     &2.0478e-04_rb,1.6937e-04_rb,1.1872e-04_rb,6.4236e-05_rb,5.6954e-07_rb/)
      kbo(:, 5,53, 6) = (/ &
     &3.0662e-04_rb,2.5374e-04_rb,1.7755e-04_rb,9.5705e-05_rb,7.4673e-07_rb/)
      kbo(:, 1,54, 6) = (/ &
     &3.5548e-05_rb,2.9201e-05_rb,2.0499e-05_rb,1.1142e-05_rb,1.4888e-07_rb/)
      kbo(:, 2,54, 6) = (/ &
     &6.0867e-05_rb,5.0080e-05_rb,3.5102e-05_rb,1.9053e-05_rb,2.1957e-07_rb/)
      kbo(:, 3,54, 6) = (/ &
     &9.9905e-05_rb,8.2294e-05_rb,5.7679e-05_rb,3.1278e-05_rb,3.1167e-07_rb/)
      kbo(:, 4,54, 6) = (/ &
     &1.5784e-04_rb,1.3048e-04_rb,9.1376e-05_rb,4.9396e-05_rb,4.2569e-07_rb/)
      kbo(:, 5,54, 6) = (/ &
     &2.3990e-04_rb,1.9844e-04_rb,1.3879e-04_rb,7.4726e-05_rb,5.6432e-07_rb/)
      kbo(:, 1,55, 6) = (/ &
     &2.6254e-05_rb,2.1551e-05_rb,1.5109e-05_rb,8.2013e-06_rb,1.0776e-07_rb/)
      kbo(:, 2,55, 6) = (/ &
     &4.5756e-05_rb,3.7594e-05_rb,2.6330e-05_rb,1.4266e-05_rb,1.6227e-07_rb/)
      kbo(:, 3,55, 6) = (/ &
     &7.6387e-05_rb,6.2840e-05_rb,4.3983e-05_rb,2.3813e-05_rb,2.3322e-07_rb/)
      kbo(:, 4,55, 6) = (/ &
     &1.2233e-04_rb,1.0105e-04_rb,7.0729e-05_rb,3.8184e-05_rb,3.2242e-07_rb/)
      kbo(:, 5,55, 6) = (/ &
     &1.8866e-04_rb,1.5604e-04_rb,1.0907e-04_rb,5.8660e-05_rb,4.3190e-07_rb/)
      kbo(:, 1,56, 6) = (/ &
     &1.9352e-05_rb,1.5886e-05_rb,1.1124e-05_rb,6.0284e-06_rb,7.7832e-08_rb/)
      kbo(:, 2,56, 6) = (/ &
     &3.4378e-05_rb,2.8197e-05_rb,1.9721e-05_rb,1.0667e-05_rb,1.1948e-07_rb/)
      kbo(:, 3,56, 6) = (/ &
     &5.8349e-05_rb,4.7950e-05_rb,3.3520e-05_rb,1.8111e-05_rb,1.7363e-07_rb/)
      kbo(:, 4,56, 6) = (/ &
     &9.4866e-05_rb,7.8290e-05_rb,5.4723e-05_rb,2.9510e-05_rb,2.4385e-07_rb/)
      kbo(:, 5,56, 6) = (/ &
     &1.4839e-04_rb,1.2273e-04_rb,8.5680e-05_rb,4.6045e-05_rb,3.3022e-07_rb/)
      kbo(:, 1,57, 6) = (/ &
     &1.4255e-05_rb,1.1680e-05_rb,8.1776e-06_rb,4.4235e-06_rb,5.5927e-08_rb/)
      kbo(:, 2,57, 6) = (/ &
     &2.5815e-05_rb,2.1132e-05_rb,1.4763e-05_rb,7.9663e-06_rb,8.7522e-08_rb/)
      kbo(:, 3,57, 6) = (/ &
     &4.4552e-05_rb,3.6579e-05_rb,2.5531e-05_rb,1.3761e-05_rb,1.2925e-07_rb/)
      kbo(:, 4,57, 6) = (/ &
     &7.3589e-05_rb,6.0640e-05_rb,4.2341e-05_rb,2.2797e-05_rb,1.8364e-07_rb/)
      kbo(:, 5,57, 6) = (/ &
     &1.1687e-04_rb,9.6565e-05_rb,6.7349e-05_rb,3.6145e-05_rb,2.5190e-07_rb/)
      kbo(:, 1,58, 6) = (/ &
     &5.1283e-06_rb,4.8164e-06_rb,3.9533e-06_rb,2.5844e-06_rb,4.0411e-08_rb/)
      kbo(:, 2,58, 6) = (/ &
     &9.4663e-06_rb,8.8829e-06_rb,7.2756e-06_rb,4.7397e-06_rb,6.4616e-08_rb/)
      kbo(:, 3,58, 6) = (/ &
     &1.6608e-05_rb,1.5628e-05_rb,1.2791e-05_rb,8.3232e-06_rb,9.6916e-08_rb/)
      kbo(:, 4,58, 6) = (/ &
     &2.7869e-05_rb,2.6331e-05_rb,2.1549e-05_rb,1.4010e-05_rb,1.3884e-07_rb/)
      kbo(:, 5,58, 6) = (/ &
     &4.4916e-05_rb,4.2582e-05_rb,3.4830e-05_rb,2.2586e-05_rb,1.9305e-07_rb/)
      kbo(:, 1,59, 6) = (/ &
     &4.1166e-06_rb,3.8763e-06_rb,3.1932e-06_rb,2.0984e-06_rb,3.3138e-08_rb/)
      kbo(:, 2,59, 6) = (/ &
     &7.6679e-06_rb,7.2183e-06_rb,5.9356e-06_rb,3.8849e-06_rb,5.3542e-08_rb/)
      kbo(:, 3,59, 6) = (/ &
     &1.3578e-05_rb,1.2809e-05_rb,1.0527e-05_rb,6.8844e-06_rb,8.1037e-08_rb/)
      kbo(:, 4,59, 6) = (/ &
     &2.2972e-05_rb,2.1775e-05_rb,1.7891e-05_rb,1.1690e-05_rb,1.1735e-07_rb/)
      kbo(:, 5,59, 6) = (/ &
     &3.7296e-05_rb,3.5490e-05_rb,2.9150e-05_rb,1.9007e-05_rb,1.6270e-07_rb/)
      kbo(:, 1,13, 7) = (/ &
     &4.2167e-01_rb,3.2553e-01_rb,2.2695e-01_rb,1.2184e-01_rb,4.0030e-03_rb/)
      kbo(:, 2,13, 7) = (/ &
     &6.0140e-01_rb,4.6380e-01_rb,3.2184e-01_rb,1.7190e-01_rb,5.1104e-03_rb/)
      kbo(:, 3,13, 7) = (/ &
     &8.1816e-01_rb,6.3015e-01_rb,4.3650e-01_rb,2.3296e-01_rb,6.4365e-03_rb/)
      kbo(:, 4,13, 7) = (/ &
     &1.0780e+00_rb,8.2908e-01_rb,5.7271e-01_rb,3.0519e-01_rb,7.8514e-03_rb/)
      kbo(:, 5,13, 7) = (/ &
     &1.3871e+00_rb,1.0650e+00_rb,7.3566e-01_rb,3.9202e-01_rb,9.6931e-03_rb/)
      kbo(:, 1,14, 7) = (/ &
     &3.8425e-01_rb,2.9801e-01_rb,2.0688e-01_rb,1.1055e-01_rb,3.4668e-03_rb/)
      kbo(:, 2,14, 7) = (/ &
     &5.4386e-01_rb,4.2093e-01_rb,2.9177e-01_rb,1.5544e-01_rb,4.5429e-03_rb/)
      kbo(:, 3,14, 7) = (/ &
     &7.3998e-01_rb,5.7137e-01_rb,3.9497e-01_rb,2.0995e-01_rb,5.6171e-03_rb/)
      kbo(:, 4,14, 7) = (/ &
     &9.7642e-01_rb,7.5246e-01_rb,5.1952e-01_rb,2.7577e-01_rb,6.9867e-03_rb/)
      kbo(:, 5,14, 7) = (/ &
     &1.2610e+00_rb,9.7017e-01_rb,6.6888e-01_rb,3.5427e-01_rb,8.6535e-03_rb/)
      kbo(:, 1,15, 7) = (/ &
     &3.3299e-01_rb,2.5933e-01_rb,1.7991e-01_rb,9.5905e-02_rb,2.9886e-03_rb/)
      kbo(:, 2,15, 7) = (/ &
     &4.6962e-01_rb,3.6448e-01_rb,2.5234e-01_rb,1.3421e-01_rb,3.8718e-03_rb/)
      kbo(:, 3,15, 7) = (/ &
     &6.3987e-01_rb,4.9512e-01_rb,3.4211e-01_rb,1.8150e-01_rb,4.8805e-03_rb/)
      kbo(:, 4,15, 7) = (/ &
     &8.4586e-01_rb,6.5388e-01_rb,4.5155e-01_rb,2.3908e-01_rb,6.1279e-03_rb/)
      kbo(:, 5,15, 7) = (/ &
     &1.0948e+00_rb,8.4428e-01_rb,5.8246e-01_rb,3.0839e-01_rb,7.4849e-03_rb/)
      kbo(:, 1,16, 7) = (/ &
     &2.8051e-01_rb,2.1899e-01_rb,1.5183e-01_rb,8.0896e-02_rb,2.5509e-03_rb/)
      kbo(:, 2,16, 7) = (/ &
     &3.9506e-01_rb,3.0781e-01_rb,2.1302e-01_rb,1.1324e-01_rb,3.3108e-03_rb/)
      kbo(:, 3,16, 7) = (/ &
     &5.3870e-01_rb,4.1855e-01_rb,2.8938e-01_rb,1.5357e-01_rb,4.2013e-03_rb/)
      kbo(:, 4,16, 7) = (/ &
     &7.1526e-01_rb,5.5449e-01_rb,3.8283e-01_rb,2.0269e-01_rb,5.1902e-03_rb/)
      kbo(:, 5,16, 7) = (/ &
     &9.2727e-01_rb,7.1785e-01_rb,4.9548e-01_rb,2.6200e-01_rb,6.4068e-03_rb/)
      kbo(:, 1,17, 7) = (/ &
     &2.3378e-01_rb,1.8318e-01_rb,1.2698e-01_rb,6.7717e-02_rb,2.1927e-03_rb/)
      kbo(:, 2,17, 7) = (/ &
     &3.2948e-01_rb,2.5742e-01_rb,1.7822e-01_rb,9.4896e-02_rb,2.8584e-03_rb/)
      kbo(:, 3,17, 7) = (/ &
     &4.5000e-01_rb,3.5080e-01_rb,2.4241e-01_rb,1.2872e-01_rb,3.6126e-03_rb/)
      kbo(:, 4,17, 7) = (/ &
     &5.9887e-01_rb,4.6579e-01_rb,3.2151e-01_rb,1.7042e-01_rb,4.5208e-03_rb/)
      kbo(:, 5,17, 7) = (/ &
     &7.7711e-01_rb,6.0357e-01_rb,4.1660e-01_rb,2.2035e-01_rb,5.5539e-03_rb/)
      kbo(:, 1,18, 7) = (/ &
     &1.9227e-01_rb,1.5110e-01_rb,1.0483e-01_rb,5.6032e-02_rb,1.9220e-03_rb/)
      kbo(:, 2,18, 7) = (/ &
     &2.7135e-01_rb,2.1238e-01_rb,1.4707e-01_rb,7.8382e-02_rb,2.4923e-03_rb/)
      kbo(:, 3,18, 7) = (/ &
     &3.7049e-01_rb,2.8960e-01_rb,2.0022e-01_rb,1.0642e-01_rb,3.1976e-03_rb/)
      kbo(:, 4,18, 7) = (/ &
     &4.9231e-01_rb,3.8398e-01_rb,2.6515e-01_rb,1.4069e-01_rb,3.9529e-03_rb/)
      kbo(:, 5,18, 7) = (/ &
     &6.4035e-01_rb,4.9892e-01_rb,3.4418e-01_rb,1.8220e-01_rb,4.8074e-03_rb/)
      kbo(:, 1,19, 7) = (/ &
     &1.6097e-01_rb,1.2681e-01_rb,8.7983e-02_rb,4.7067e-02_rb,1.6998e-03_rb/)
      kbo(:, 2,19, 7) = (/ &
     &2.2674e-01_rb,1.7812e-01_rb,1.2337e-01_rb,6.5758e-02_rb,2.2148e-03_rb/)
      kbo(:, 3,19, 7) = (/ &
     &3.0942e-01_rb,2.4239e-01_rb,1.6768e-01_rb,8.9247e-02_rb,2.8171e-03_rb/)
      kbo(:, 4,19, 7) = (/ &
     &4.1121e-01_rb,3.2156e-01_rb,2.2226e-01_rb,1.1805e-01_rb,3.4852e-03_rb/)
      kbo(:, 5,19, 7) = (/ &
     &5.3661e-01_rb,4.1891e-01_rb,2.8897e-01_rb,1.5308e-01_rb,4.2703e-03_rb/)
      kbo(:, 1,20, 7) = (/ &
     &1.3816e-01_rb,1.0899e-01_rb,7.5568e-02_rb,4.0384e-02_rb,1.5084e-03_rb/)
      kbo(:, 2,20, 7) = (/ &
     &1.9390e-01_rb,1.5237e-01_rb,1.0558e-01_rb,5.6347e-02_rb,1.9533e-03_rb/)
      kbo(:, 3,20, 7) = (/ &
     &2.6431e-01_rb,2.0734e-01_rb,1.4349e-01_rb,7.6381e-02_rb,2.4829e-03_rb/)
      kbo(:, 4,20, 7) = (/ &
     &3.5230e-01_rb,2.7579e-01_rb,1.9049e-01_rb,1.0107e-01_rb,3.1175e-03_rb/)
      kbo(:, 5,20, 7) = (/ &
     &4.5959e-01_rb,3.5917e-01_rb,2.4774e-01_rb,1.3132e-01_rb,3.8257e-03_rb/)
      kbo(:, 1,21, 7) = (/ &
     &1.1885e-01_rb,9.3725e-02_rb,6.5041e-02_rb,3.4803e-02_rb,1.3411e-03_rb/)
      kbo(:, 2,21, 7) = (/ &
     &1.6635e-01_rb,1.3102e-01_rb,9.0813e-02_rb,4.8481e-02_rb,1.7301e-03_rb/)
      kbo(:, 3,21, 7) = (/ &
     &2.2707e-01_rb,1.7845e-01_rb,1.2348e-01_rb,6.5731e-02_rb,2.2027e-03_rb/)
      kbo(:, 4,21, 7) = (/ &
     &3.0299e-01_rb,2.3748e-01_rb,1.6403e-01_rb,8.7154e-02_rb,2.7646e-03_rb/)
      kbo(:, 5,21, 7) = (/ &
     &3.9491e-01_rb,3.0909e-01_rb,2.1323e-01_rb,1.1320e-01_rb,3.3663e-03_rb/)
      kbo(:, 1,22, 7) = (/ &
     &1.0371e-01_rb,8.1936e-02_rb,5.6886e-02_rb,3.0452e-02_rb,1.1973e-03_rb/)
      kbo(:, 2,22, 7) = (/ &
     &1.4515e-01_rb,1.1443e-01_rb,7.9345e-02_rb,4.2377e-02_rb,1.5408e-03_rb/)
      kbo(:, 3,22, 7) = (/ &
     &1.9815e-01_rb,1.5584e-01_rb,1.0788e-01_rb,5.7461e-02_rb,1.9609e-03_rb/)
      kbo(:, 4,22, 7) = (/ &
     &2.6352e-01_rb,2.0697e-01_rb,1.4313e-01_rb,7.6128e-02_rb,2.4333e-03_rb/)
      kbo(:, 5,22, 7) = (/ &
     &3.4221e-01_rb,2.6821e-01_rb,1.8537e-01_rb,9.8647e-02_rb,2.9393e-03_rb/)
      kbo(:, 1,23, 7) = (/ &
     &9.0639e-02_rb,7.1734e-02_rb,4.9824e-02_rb,2.6695e-02_rb,1.0611e-03_rb/)
      kbo(:, 2,23, 7) = (/ &
     &1.2684e-01_rb,1.0014e-01_rb,6.9426e-02_rb,3.7107e-02_rb,1.3714e-03_rb/)
      kbo(:, 3,23, 7) = (/ &
     &1.7268e-01_rb,1.3597e-01_rb,9.4123e-02_rb,5.0237e-02_rb,1.7212e-03_rb/)
      kbo(:, 4,23, 7) = (/ &
     &2.2866e-01_rb,1.7972e-01_rb,1.2432e-01_rb,6.6277e-02_rb,2.1161e-03_rb/)
      kbo(:, 5,23, 7) = (/ &
     &2.9584e-01_rb,2.3227e-01_rb,1.6067e-01_rb,8.5547e-02_rb,2.5346e-03_rb/)
      kbo(:, 1,24, 7) = (/ &
     &8.0071e-02_rb,6.3444e-02_rb,4.4064e-02_rb,2.3644e-02_rb,9.3348e-04_rb/)
      kbo(:, 2,24, 7) = (/ &
     &1.1181e-01_rb,8.8354e-02_rb,6.1271e-02_rb,3.2794e-02_rb,1.1882e-03_rb/)
      kbo(:, 3,24, 7) = (/ &
     &1.5149e-01_rb,1.1939e-01_rb,8.2725e-02_rb,4.4223e-02_rb,1.4832e-03_rb/)
      kbo(:, 4,24, 7) = (/ &
     &1.9978e-01_rb,1.5722e-01_rb,1.0888e-01_rb,5.8128e-02_rb,1.8190e-03_rb/)
      kbo(:, 5,24, 7) = (/ &
     &2.5770e-01_rb,2.0260e-01_rb,1.4030e-01_rb,7.4844e-02_rb,2.1747e-03_rb/)
      kbo(:, 1,25, 7) = (/ &
     &7.0951e-02_rb,5.6269e-02_rb,3.9108e-02_rb,2.1016e-02_rb,8.0964e-04_rb/)
      kbo(:, 2,25, 7) = (/ &
     &9.8623e-02_rb,7.8013e-02_rb,5.4088e-02_rb,2.9015e-02_rb,1.0247e-03_rb/)
      kbo(:, 3,25, 7) = (/ &
     &1.3295e-01_rb,1.0488e-01_rb,7.2709e-02_rb,3.8919e-02_rb,1.2776e-03_rb/)
      kbo(:, 4,25, 7) = (/ &
     &1.7470e-01_rb,1.3764e-01_rb,9.5373e-02_rb,5.0958e-02_rb,1.5592e-03_rb/)
      kbo(:, 5,25, 7) = (/ &
     &2.2493e-01_rb,1.7708e-01_rb,1.2270e-01_rb,6.5502e-02_rb,1.8657e-03_rb/)
      kbo(:, 1,26, 7) = (/ &
     &6.3451e-02_rb,5.0352e-02_rb,3.4999e-02_rb,1.8811e-02_rb,7.0696e-04_rb/)
      kbo(:, 2,26, 7) = (/ &
     &8.7646e-02_rb,6.9321e-02_rb,4.8128e-02_rb,2.5820e-02_rb,8.9108e-04_rb/)
      kbo(:, 3,26, 7) = (/ &
     &1.1756e-01_rb,9.2804e-02_rb,6.4336e-02_rb,3.4465e-02_rb,1.1063e-03_rb/)
      kbo(:, 4,26, 7) = (/ &
     &1.5401e-01_rb,1.2150e-01_rb,8.4218e-02_rb,4.5022e-02_rb,1.3420e-03_rb/)
      kbo(:, 5,26, 7) = (/ &
     &1.9847e-01_rb,1.5626e-01_rb,1.0822e-01_rb,5.7833e-02_rb,1.6057e-03_rb/)
      kbo(:, 1,27, 7) = (/ &
     &5.6793e-02_rb,4.5057e-02_rb,3.1339e-02_rb,1.6842e-02_rb,6.0571e-04_rb/)
      kbo(:, 2,27, 7) = (/ &
     &7.8027e-02_rb,6.1732e-02_rb,4.2853e-02_rb,2.2968e-02_rb,7.6180e-04_rb/)
      kbo(:, 3,27, 7) = (/ &
     &1.0421e-01_rb,8.2314e-02_rb,5.7109e-02_rb,3.0579e-02_rb,9.4573e-04_rb/)
      kbo(:, 4,27, 7) = (/ &
     &1.3651e-01_rb,1.0769e-01_rb,7.4645e-02_rb,3.9915e-02_rb,1.1425e-03_rb/)
      kbo(:, 5,27, 7) = (/ &
     &1.7591e-01_rb,1.3862e-01_rb,9.6018e-02_rb,5.1325e-02_rb,1.3608e-03_rb/)
      kbo(:, 1,28, 7) = (/ &
     &5.0832e-02_rb,4.0335e-02_rb,2.8062e-02_rb,1.5080e-02_rb,5.0959e-04_rb/)
      kbo(:, 2,28, 7) = (/ &
     &6.9491e-02_rb,5.4973e-02_rb,3.8187e-02_rb,2.0466e-02_rb,6.4488e-04_rb/)
      kbo(:, 3,28, 7) = (/ &
     &9.2684e-02_rb,7.3230e-02_rb,5.0823e-02_rb,2.7216e-02_rb,7.9606e-04_rb/)
      kbo(:, 4,28, 7) = (/ &
     &1.2155e-01_rb,9.5886e-02_rb,6.6455e-02_rb,3.5548e-02_rb,9.5678e-04_rb/)
      kbo(:, 5,28, 7) = (/ &
     &1.5688e-01_rb,1.2363e-01_rb,8.5583e-02_rb,4.5732e-02_rb,1.1415e-03_rb/)
      kbo(:, 1,29, 7) = (/ &
     &4.5296e-02_rb,3.5922e-02_rb,2.4992e-02_rb,1.3423e-02_rb,4.2743e-04_rb/)
      kbo(:, 2,29, 7) = (/ &
     &6.1694e-02_rb,4.8833e-02_rb,3.3922e-02_rb,1.8191e-02_rb,5.3907e-04_rb/)
      kbo(:, 3,29, 7) = (/ &
     &8.2276e-02_rb,6.5053e-02_rb,4.5148e-02_rb,2.4171e-02_rb,6.5935e-04_rb/)
      kbo(:, 4,29, 7) = (/ &
     &1.0791e-01_rb,8.5197e-02_rb,5.9066e-02_rb,3.1598e-02_rb,7.9358e-04_rb/)
      kbo(:, 5,29, 7) = (/ &
     &1.3964e-01_rb,1.1005e-01_rb,7.6235e-02_rb,4.0713e-02_rb,9.4519e-04_rb/)
      kbo(:, 1,30, 7) = (/ &
     &4.0185e-02_rb,3.1886e-02_rb,2.2195e-02_rb,1.1919e-02_rb,3.5485e-04_rb/)
      kbo(:, 2,30, 7) = (/ &
     &5.4626e-02_rb,4.3278e-02_rb,3.0084e-02_rb,1.6135e-02_rb,4.4303e-04_rb/)
      kbo(:, 3,30, 7) = (/ &
     &7.2953e-02_rb,5.7701e-02_rb,4.0048e-02_rb,2.1442e-02_rb,5.4169e-04_rb/)
      kbo(:, 4,30, 7) = (/ &
     &9.5812e-02_rb,7.5701e-02_rb,5.2497e-02_rb,2.8075e-02_rb,6.5509e-04_rb/)
      kbo(:, 5,30, 7) = (/ &
     &1.2402e-01_rb,9.7862e-02_rb,6.7806e-02_rb,3.6235e-02_rb,7.7850e-04_rb/)
      kbo(:, 1,31, 7) = (/ &
     &3.5348e-02_rb,2.8090e-02_rb,1.9559e-02_rb,1.0513e-02_rb,2.9039e-04_rb/)
      kbo(:, 2,31, 7) = (/ &
     &4.8053e-02_rb,3.8118e-02_rb,2.6496e-02_rb,1.4221e-02_rb,3.5964e-04_rb/)
      kbo(:, 3,31, 7) = (/ &
     &6.4212e-02_rb,5.0845e-02_rb,3.5309e-02_rb,1.8919e-02_rb,4.4117e-04_rb/)
      kbo(:, 4,31, 7) = (/ &
     &8.4429e-02_rb,6.6772e-02_rb,4.6322e-02_rb,2.4787e-02_rb,5.3116e-04_rb/)
      kbo(:, 5,31, 7) = (/ &
     &1.0928e-01_rb,8.6339e-02_rb,5.9901e-02_rb,3.2032e-02_rb,6.3348e-04_rb/)
      kbo(:, 1,32, 7) = (/ &
     &3.1311e-02_rb,2.4896e-02_rb,1.7336e-02_rb,9.3247e-03_rb,2.3608e-04_rb/)
      kbo(:, 2,32, 7) = (/ &
     &4.2575e-02_rb,3.3801e-02_rb,2.3511e-02_rb,1.2616e-02_rb,2.9452e-04_rb/)
      kbo(:, 3,32, 7) = (/ &
     &5.6935e-02_rb,4.5129e-02_rb,3.1359e-02_rb,1.6809e-02_rb,3.6093e-04_rb/)
      kbo(:, 4,32, 7) = (/ &
     &7.4842e-02_rb,5.9256e-02_rb,4.1156e-02_rb,2.2039e-02_rb,4.3435e-04_rb/)
      kbo(:, 5,32, 7) = (/ &
     &9.6933e-02_rb,7.6740e-02_rb,5.3274e-02_rb,2.8476e-02_rb,5.1533e-04_rb/)
      kbo(:, 1,33, 7) = (/ &
     &2.7886e-02_rb,2.2206e-02_rb,1.5465e-02_rb,8.3176e-03_rb,1.9330e-04_rb/)
      kbo(:, 2,33, 7) = (/ &
     &3.7954e-02_rb,3.0161e-02_rb,2.0981e-02_rb,1.1260e-02_rb,2.4048e-04_rb/)
      kbo(:, 3,33, 7) = (/ &
     &5.0729e-02_rb,4.0269e-02_rb,2.7996e-02_rb,1.5008e-02_rb,2.9283e-04_rb/)
      kbo(:, 4,33, 7) = (/ &
     &6.6680e-02_rb,5.2910e-02_rb,3.6768e-02_rb,1.9685e-02_rb,3.5174e-04_rb/)
      kbo(:, 5,33, 7) = (/ &
     &8.6325e-02_rb,6.8491e-02_rb,4.7579e-02_rb,2.5448e-02_rb,4.1781e-04_rb/)
      kbo(:, 1,34, 7) = (/ &
     &2.4971e-02_rb,1.9901e-02_rb,1.3860e-02_rb,7.4508e-03_rb,1.5943e-04_rb/)
      kbo(:, 2,34, 7) = (/ &
     &3.3993e-02_rb,2.7044e-02_rb,1.8817e-02_rb,1.0102e-02_rb,1.9737e-04_rb/)
      kbo(:, 3,34, 7) = (/ &
     &4.5431e-02_rb,3.6116e-02_rb,2.5118e-02_rb,1.3468e-02_rb,2.4097e-04_rb/)
      kbo(:, 4,34, 7) = (/ &
     &5.9751e-02_rb,4.7485e-02_rb,3.3007e-02_rb,1.7673e-02_rb,2.8983e-04_rb/)
      kbo(:, 5,34, 7) = (/ &
     &7.7416e-02_rb,6.1530e-02_rb,4.2751e-02_rb,2.2863e-02_rb,3.4328e-04_rb/)
      kbo(:, 1,35, 7) = (/ &
     &2.1796e-02_rb,1.7387e-02_rb,1.2111e-02_rb,6.5084e-03_rb,1.2835e-04_rb/)
      kbo(:, 2,35, 7) = (/ &
     &2.9698e-02_rb,2.3666e-02_rb,1.6479e-02_rb,8.8433e-03_rb,1.5863e-04_rb/)
      kbo(:, 3,35, 7) = (/ &
     &3.9760e-02_rb,3.1675e-02_rb,2.2035e-02_rb,1.1812e-02_rb,1.9396e-04_rb/)
      kbo(:, 4,35, 7) = (/ &
     &5.2402e-02_rb,4.1735e-02_rb,2.9029e-02_rb,1.5538e-02_rb,2.3292e-04_rb/)
      kbo(:, 5,35, 7) = (/ &
     &6.8053e-02_rb,5.4173e-02_rb,3.7664e-02_rb,2.0141e-02_rb,2.7598e-04_rb/)
      kbo(:, 1,36, 7) = (/ &
     &1.8439e-02_rb,1.4736e-02_rb,1.0269e-02_rb,5.5181e-03_rb,1.0059e-04_rb/)
      kbo(:, 2,36, 7) = (/ &
     &2.5228e-02_rb,2.0135e-02_rb,1.4024e-02_rb,7.5257e-03_rb,1.2421e-04_rb/)
      kbo(:, 3,36, 7) = (/ &
     &3.3891e-02_rb,2.7058e-02_rb,1.8832e-02_rb,1.0097e-02_rb,1.5179e-04_rb/)
      kbo(:, 4,36, 7) = (/ &
     &4.4855e-02_rb,3.5783e-02_rb,2.4899e-02_rb,1.3331e-02_rb,1.8254e-04_rb/)
      kbo(:, 5,36, 7) = (/ &
     &5.8411e-02_rb,4.6611e-02_rb,3.2419e-02_rb,1.7335e-02_rb,2.1654e-04_rb/)
      kbo(:, 1,37, 7) = (/ &
     &1.5025e-02_rb,1.2025e-02_rb,8.3808e-03_rb,4.5043e-03_rb,7.8293e-05_rb/)
      kbo(:, 2,37, 7) = (/ &
     &2.0678e-02_rb,1.6540e-02_rb,1.1522e-02_rb,6.1849e-03_rb,9.7252e-05_rb/)
      kbo(:, 3,37, 7) = (/ &
     &2.7964e-02_rb,2.2364e-02_rb,1.5572e-02_rb,8.3486e-03_rb,1.1926e-04_rb/)
      kbo(:, 4,37, 7) = (/ &
     &3.7225e-02_rb,2.9764e-02_rb,2.0719e-02_rb,1.1092e-02_rb,1.4386e-04_rb/)
      kbo(:, 5,37, 7) = (/ &
     &4.8779e-02_rb,3.9013e-02_rb,2.7143e-02_rb,1.4513e-02_rb,1.7127e-04_rb/)
      kbo(:, 1,38, 7) = (/ &
     &1.2194e-02_rb,9.7772e-03_rb,6.8154e-03_rb,3.6630e-03_rb,6.0721e-05_rb/)
      kbo(:, 2,38, 7) = (/ &
     &1.6892e-02_rb,1.3540e-02_rb,9.4349e-03_rb,5.0655e-03_rb,7.5931e-05_rb/)
      kbo(:, 3,38, 7) = (/ &
     &2.2993e-02_rb,1.8427e-02_rb,1.2834e-02_rb,6.8825e-03_rb,9.3480e-05_rb/)
      kbo(:, 4,38, 7) = (/ &
     &3.0804e-02_rb,2.4686e-02_rb,1.7192e-02_rb,9.2065e-03_rb,1.1312e-04_rb/)
      kbo(:, 5,38, 7) = (/ &
     &4.0626e-02_rb,3.2569e-02_rb,2.2673e-02_rb,1.2127e-02_rb,1.3481e-04_rb/)
      kbo(:, 1,39, 7) = (/ &
     &9.8863e-03_rb,7.9385e-03_rb,5.5336e-03_rb,2.9753e-03_rb,4.6829e-05_rb/)
      kbo(:, 2,39, 7) = (/ &
     &1.3782e-02_rb,1.1068e-02_rb,7.7135e-03_rb,4.1426e-03_rb,5.8903e-05_rb/)
      kbo(:, 3,39, 7) = (/ &
     &1.8884e-02_rb,1.5163e-02_rb,1.0568e-02_rb,5.6692e-03_rb,7.2758e-05_rb/)
      kbo(:, 4,39, 7) = (/ &
     &2.5472e-02_rb,2.0459e-02_rb,1.4252e-02_rb,7.6329e-03_rb,8.8280e-05_rb/)
      kbo(:, 5,39, 7) = (/ &
     &3.3793e-02_rb,2.7159e-02_rb,1.8912e-02_rb,1.0123e-02_rb,1.0549e-04_rb/)
      kbo(:, 1,40, 7) = (/ &
     &7.8183e-03_rb,6.2872e-03_rb,4.3840e-03_rb,2.3583e-03_rb,3.5934e-05_rb/)
      kbo(:, 2,40, 7) = (/ &
     &1.1002e-02_rb,8.8456e-03_rb,6.1665e-03_rb,3.3129e-03_rb,4.5505e-05_rb/)
      kbo(:, 3,40, 7) = (/ &
     &1.5204e-02_rb,1.2224e-02_rb,8.5201e-03_rb,4.5727e-03_rb,5.6603e-05_rb/)
      kbo(:, 4,40, 7) = (/ &
     &2.0667e-02_rb,1.6639e-02_rb,1.1596e-02_rb,6.2148e-03_rb,6.8922e-05_rb/)
      kbo(:, 5,40, 7) = (/ &
     &2.7656e-02_rb,2.2280e-02_rb,1.5521e-02_rb,8.3102e-03_rb,8.2779e-05_rb/)
      kbo(:, 1,41, 7) = (/ &
     &6.1500e-03_rb,4.9524e-03_rb,3.4538e-03_rb,1.8579e-03_rb,2.7494e-05_rb/)
      kbo(:, 2,41, 7) = (/ &
     &8.7326e-03_rb,7.0361e-03_rb,4.9035e-03_rb,2.6349e-03_rb,3.5036e-05_rb/)
      kbo(:, 3,41, 7) = (/ &
     &1.2182e-02_rb,9.8121e-03_rb,6.8364e-03_rb,3.6704e-03_rb,4.3900e-05_rb/)
      kbo(:, 4,41, 7) = (/ &
     &1.6698e-02_rb,1.3470e-02_rb,9.3901e-03_rb,5.0360e-03_rb,5.3809e-05_rb/)
      kbo(:, 5,41, 7) = (/ &
     &2.2541e-02_rb,1.8202e-02_rb,1.2686e-02_rb,6.7978e-03_rb,6.4784e-05_rb/)
      kbo(:, 1,42, 7) = (/ &
     &4.8216e-03_rb,3.8878e-03_rb,2.7121e-03_rb,1.4585e-03_rb,2.0930e-05_rb/)
      kbo(:, 2,42, 7) = (/ &
     &6.9151e-03_rb,5.5793e-03_rb,3.8882e-03_rb,2.0902e-03_rb,2.6862e-05_rb/)
      kbo(:, 3,42, 7) = (/ &
     &9.7317e-03_rb,7.8533e-03_rb,5.4747e-03_rb,2.9388e-03_rb,3.3897e-05_rb/)
      kbo(:, 4,42, 7) = (/ &
     &1.3459e-02_rb,1.0876e-02_rb,7.5844e-03_rb,4.0697e-03_rb,4.1759e-05_rb/)
      kbo(:, 5,42, 7) = (/ &
     &1.8320e-02_rb,1.4838e-02_rb,1.0343e-02_rb,5.5456e-03_rb,5.0540e-05_rb/)
      kbo(:, 1,43, 7) = (/ &
     &3.7271e-03_rb,3.0076e-03_rb,2.0983e-03_rb,1.1283e-03_rb,1.5715e-05_rb/)
      kbo(:, 2,43, 7) = (/ &
     &5.4034e-03_rb,4.3650e-03_rb,3.0427e-03_rb,1.6347e-03_rb,2.0366e-05_rb/)
      kbo(:, 3,43, 7) = (/ &
     &7.6865e-03_rb,6.2120e-03_rb,4.3312e-03_rb,2.3251e-03_rb,2.5938e-05_rb/)
      kbo(:, 4,43, 7) = (/ &
     &1.0735e-02_rb,8.6871e-03_rb,6.0565e-03_rb,3.2504e-03_rb,3.2151e-05_rb/)
      kbo(:, 5,43, 7) = (/ &
     &1.4744e-02_rb,1.1965e-02_rb,8.3459e-03_rb,4.4756e-03_rb,3.9126e-05_rb/)
      kbo(:, 1,44, 7) = (/ &
     &2.8560e-03_rb,2.3067e-03_rb,1.6088e-03_rb,8.6423e-04_rb,1.1691e-05_rb/)
      kbo(:, 2,44, 7) = (/ &
     &4.1914e-03_rb,3.3878e-03_rb,2.3617e-03_rb,1.2680e-03_rb,1.5336e-05_rb/)
      kbo(:, 3,44, 7) = (/ &
     &6.0257e-03_rb,4.8753e-03_rb,3.3991e-03_rb,1.8248e-03_rb,1.9674e-05_rb/)
      kbo(:, 4,44, 7) = (/ &
     &8.5074e-03_rb,6.8953e-03_rb,4.8067e-03_rb,2.5783e-03_rb,2.4601e-05_rb/)
      kbo(:, 5,44, 7) = (/ &
     &1.1798e-02_rb,9.5893e-03_rb,6.6876e-03_rb,3.5866e-03_rb,3.0110e-05_rb/)
      kbo(:, 1,45, 7) = (/ &
     &2.1795e-03_rb,1.7620e-03_rb,1.2288e-03_rb,6.5976e-04_rb,8.6355e-06_rb/)
      kbo(:, 2,45, 7) = (/ &
     &3.2384e-03_rb,2.6190e-03_rb,1.8255e-03_rb,9.8000e-04_rb,1.1470e-05_rb/)
      kbo(:, 3,45, 7) = (/ &
     &4.7095e-03_rb,3.8135e-03_rb,2.6589e-03_rb,1.4269e-03_rb,1.4819e-05_rb/)
      kbo(:, 4,45, 7) = (/ &
     &6.7188e-03_rb,5.4533e-03_rb,3.8021e-03_rb,2.0391e-03_rb,1.8692e-05_rb/)
      kbo(:, 5,45, 7) = (/ &
     &9.4160e-03_rb,7.6627e-03_rb,5.3428e-03_rb,2.8648e-03_rb,2.3071e-05_rb/)
      kbo(:, 1,46, 7) = (/ &
     &1.6442e-03_rb,1.3301e-03_rb,9.2731e-04_rb,4.9724e-04_rb,6.3123e-06_rb/)
      kbo(:, 2,46, 7) = (/ &
     &2.4756e-03_rb,2.0038e-03_rb,1.3961e-03_rb,7.4858e-04_rb,8.4887e-06_rb/)
      kbo(:, 3,46, 7) = (/ &
     &3.6430e-03_rb,2.9527e-03_rb,2.0584e-03_rb,1.1034e-03_rb,1.1065e-05_rb/)
      kbo(:, 4,46, 7) = (/ &
     &5.2551e-03_rb,4.2702e-03_rb,2.9775e-03_rb,1.5960e-03_rb,1.4093e-05_rb/)
      kbo(:, 5,46, 7) = (/ &
     &7.4490e-03_rb,6.0700e-03_rb,4.2323e-03_rb,2.2675e-03_rb,1.7540e-05_rb/)
      kbo(:, 1,47, 7) = (/ &
     &1.2145e-03_rb,9.8337e-04_rb,6.8536e-04_rb,3.6730e-04_rb,4.5517e-06_rb/)
      kbo(:, 2,47, 7) = (/ &
     &1.8591e-03_rb,1.5045e-03_rb,1.0482e-03_rb,5.6130e-04_rb,6.1946e-06_rb/)
      kbo(:, 3,47, 7) = (/ &
     &2.7714e-03_rb,2.2470e-03_rb,1.5654e-03_rb,8.3875e-04_rb,8.1842e-06_rb/)
      kbo(:, 4,47, 7) = (/ &
     &4.0468e-03_rb,3.2903e-03_rb,2.2941e-03_rb,1.2283e-03_rb,1.0514e-05_rb/)
      kbo(:, 5,47, 7) = (/ &
     &5.8053e-03_rb,4.7349e-03_rb,3.3018e-03_rb,1.7685e-03_rb,1.3244e-05_rb/)
      kbo(:, 1,48, 7) = (/ &
     &8.9209e-04_rb,7.2273e-04_rb,5.0348e-04_rb,2.6958e-04_rb,3.2450e-06_rb/)
      kbo(:, 2,48, 7) = (/ &
     &1.3877e-03_rb,1.1243e-03_rb,7.8261e-04_rb,4.1863e-04_rb,4.4867e-06_rb/)
      kbo(:, 3,48, 7) = (/ &
     &2.0991e-03_rb,1.7020e-03_rb,1.1852e-03_rb,6.3416e-04_rb,6.0052e-06_rb/)
      kbo(:, 4,48, 7) = (/ &
     &3.1030e-03_rb,2.5246e-03_rb,1.7595e-03_rb,9.4166e-04_rb,7.8064e-06_rb/)
      kbo(:, 5,48, 7) = (/ &
     &4.5012e-03_rb,3.6752e-03_rb,2.5627e-03_rb,1.3716e-03_rb,9.9188e-06_rb/)
      kbo(:, 1,49, 7) = (/ &
     &6.5052e-04_rb,5.2741e-04_rb,3.6738e-04_rb,1.9665e-04_rb,2.2928e-06_rb/)
      kbo(:, 2,49, 7) = (/ &
     &1.0291e-03_rb,8.3378e-04_rb,5.8035e-04_rb,3.1024e-04_rb,3.2141e-06_rb/)
      kbo(:, 3,49, 7) = (/ &
     &1.5816e-03_rb,1.2832e-03_rb,8.9291e-04_rb,4.7687e-04_rb,4.3608e-06_rb/)
      kbo(:, 4,49, 7) = (/ &
     &2.3687e-03_rb,1.9273e-03_rb,1.3422e-03_rb,7.1737e-04_rb,5.7390e-06_rb/)
      kbo(:, 5,49, 7) = (/ &
     &3.4765e-03_rb,2.8403e-03_rb,1.9797e-03_rb,1.0580e-03_rb,7.3616e-06_rb/)
      kbo(:, 1,50, 7) = (/ &
     &4.7889e-04_rb,3.8798e-04_rb,2.7009e-04_rb,1.4448e-04_rb,1.6356e-06_rb/)
      kbo(:, 2,50, 7) = (/ &
     &7.6956e-04_rb,6.2385e-04_rb,4.3396e-04_rb,2.3170e-04_rb,2.3299e-06_rb/)
      kbo(:, 3,50, 7) = (/ &
     &1.1998e-03_rb,9.7385e-04_rb,6.7714e-04_rb,3.6138e-04_rb,3.2007e-06_rb/)
      kbo(:, 4,50, 7) = (/ &
     &1.8222e-03_rb,1.4833e-03_rb,1.0320e-03_rb,5.5070e-04_rb,4.2632e-06_rb/)
      kbo(:, 5,50, 7) = (/ &
     &2.7064e-03_rb,2.2113e-03_rb,1.5405e-03_rb,8.2234e-04_rb,5.5273e-06_rb/)
      kbo(:, 1,51, 7) = (/ &
     &3.5344e-04_rb,2.8635e-04_rb,1.9912e-04_rb,1.0636e-04_rb,1.1680e-06_rb/)
      kbo(:, 2,51, 7) = (/ &
     &5.7631e-04_rb,4.6738e-04_rb,3.2502e-04_rb,1.7332e-04_rb,1.6902e-06_rb/)
      kbo(:, 3,51, 7) = (/ &
     &9.1142e-04_rb,7.3982e-04_rb,5.1424e-04_rb,2.7410e-04_rb,2.3543e-06_rb/)
      kbo(:, 4,51, 7) = (/ &
     &1.4045e-03_rb,1.1437e-03_rb,7.9544e-04_rb,4.2395e-04_rb,3.1729e-06_rb/)
      kbo(:, 5,51, 7) = (/ &
     &2.1131e-03_rb,1.7268e-03_rb,1.2023e-03_rb,6.4090e-04_rb,4.1620e-06_rb/)
      kbo(:, 1,52, 7) = (/ &
     &2.5933e-04_rb,2.1020e-04_rb,1.4608e-04_rb,7.7951e-05_rb,8.2759e-07_rb/)
      kbo(:, 2,52, 7) = (/ &
     &4.3070e-04_rb,3.4917e-04_rb,2.4257e-04_rb,1.2919e-04_rb,1.2165e-06_rb/)
      kbo(:, 3,52, 7) = (/ &
     &6.9048e-04_rb,5.6079e-04_rb,3.8962e-04_rb,2.0733e-04_rb,1.7208e-06_rb/)
      kbo(:, 4,52, 7) = (/ &
     &1.0785e-03_rb,8.7815e-04_rb,6.1039e-04_rb,3.2491e-04_rb,2.3474e-06_rb/)
      kbo(:, 5,52, 7) = (/ &
     &1.6449e-03_rb,1.3448e-03_rb,9.3585e-04_rb,4.9830e-04_rb,3.1177e-06_rb/)
      kbo(:, 1,53, 7) = (/ &
     &1.8893e-04_rb,1.5324e-04_rb,1.0643e-04_rb,5.6765e-05_rb,5.8012e-07_rb/)
      kbo(:, 2,53, 7) = (/ &
     &3.2030e-04_rb,2.5977e-04_rb,1.8033e-04_rb,9.5988e-05_rb,8.6790e-07_rb/)
      kbo(:, 3,53, 7) = (/ &
     &5.2133e-04_rb,4.2303e-04_rb,2.9359e-04_rb,1.5604e-04_rb,1.2459e-06_rb/)
      kbo(:, 4,53, 7) = (/ &
     &8.2474e-04_rb,6.7149e-04_rb,4.6612e-04_rb,2.4781e-04_rb,1.7256e-06_rb/)
      kbo(:, 5,53, 7) = (/ &
     &1.2742e-03_rb,1.0424e-03_rb,7.2493e-04_rb,3.8558e-04_rb,2.3164e-06_rb/)
      kbo(:, 1,54, 7) = (/ &
     &1.3969e-04_rb,1.1317e-04_rb,7.8561e-05_rb,4.1870e-05_rb,4.1563e-07_rb/)
      kbo(:, 2,54, 7) = (/ &
     &2.4197e-04_rb,1.9607e-04_rb,1.3602e-04_rb,7.2325e-05_rb,6.3273e-07_rb/)
      kbo(:, 3,54, 7) = (/ &
     &4.0015e-04_rb,3.2487e-04_rb,2.2527e-04_rb,1.1951e-04_rb,9.2081e-07_rb/)
      kbo(:, 4,54, 7) = (/ &
     &6.4075e-04_rb,5.2162e-04_rb,3.6218e-04_rb,1.9219e-04_rb,1.2926e-06_rb/)
      kbo(:, 5,54, 7) = (/ &
     &1.0011e-03_rb,8.1978e-04_rb,5.6992e-04_rb,3.0275e-04_rb,1.7539e-06_rb/)
      kbo(:, 1,55, 7) = (/ &
     &1.0370e-04_rb,8.4064e-05_rb,5.8320e-05_rb,3.1035e-05_rb,3.0125e-07_rb/)
      kbo(:, 2,55, 7) = (/ &
     &1.8386e-04_rb,1.4899e-04_rb,1.0330e-04_rb,5.4881e-05_rb,4.6624e-07_rb/)
      kbo(:, 3,55, 7) = (/ &
     &3.0954e-04_rb,2.5123e-04_rb,1.7413e-04_rb,9.2312e-05_rb,6.8850e-07_rb/)
      kbo(:, 4,55, 7) = (/ &
     &5.0203e-04_rb,4.0855e-04_rb,2.8330e-04_rb,1.5019e-04_rb,9.7772e-07_rb/)
      kbo(:, 5,55, 7) = (/ &
     &7.9354e-04_rb,6.4976e-04_rb,4.5113e-04_rb,2.3954e-04_rb,1.3409e-06_rb/)
      kbo(:, 1,56, 7) = (/ &
     &7.6693e-05_rb,6.2162e-05_rb,4.3120e-05_rb,2.2933e-05_rb,2.1684e-07_rb/)
      kbo(:, 2,56, 7) = (/ &
     &1.3934e-04_rb,1.1294e-04_rb,7.8219e-05_rb,4.1520e-05_rb,3.4179e-07_rb/)
      kbo(:, 3,56, 7) = (/ &
     &2.3910e-04_rb,1.9403e-04_rb,1.3441e-04_rb,7.1189e-05_rb,5.1274e-07_rb/)
      kbo(:, 4,56, 7) = (/ &
     &3.9378e-04_rb,3.2054e-04_rb,2.2198e-04_rb,1.1748e-04_rb,7.3840e-07_rb/)
      kbo(:, 5,56, 7) = (/ &
     &6.2957e-04_rb,5.1531e-04_rb,3.5753e-04_rb,1.8952e-04_rb,1.0222e-06_rb/)
      kbo(:, 1,57, 7) = (/ &
     &5.6535e-05_rb,4.5829e-05_rb,3.1754e-05_rb,1.6880e-05_rb,1.5535e-07_rb/)
      kbo(:, 2,57, 7) = (/ &
     &1.0543e-04_rb,8.5393e-05_rb,5.9099e-05_rb,3.1336e-05_rb,2.4941e-07_rb/)
      kbo(:, 3,57, 7) = (/ &
     &1.8447e-04_rb,1.4976e-04_rb,1.0364e-04_rb,5.4850e-05_rb,3.7980e-07_rb/)
      kbo(:, 4,57, 7) = (/ &
     &3.0910e-04_rb,2.5132e-04_rb,1.7397e-04_rb,9.1976e-05_rb,5.5506e-07_rb/)
      kbo(:, 5,57, 7) = (/ &
     &4.9998e-04_rb,4.0888e-04_rb,2.8341e-04_rb,1.4999e-04_rb,7.7934e-07_rb/)
      kbo(:, 1,58, 7) = (/ &
     &2.0338e-05_rb,1.8913e-05_rb,1.5378e-05_rb,9.8853e-06_rb,1.1183e-07_rb/)
      kbo(:, 2,58, 7) = (/ &
     &3.8917e-05_rb,3.6144e-05_rb,2.9335e-05_rb,1.8800e-05_rb,1.8309e-07_rb/)
      kbo(:, 3,58, 7) = (/ &
     &6.9478e-05_rb,6.4684e-05_rb,5.2517e-05_rb,3.3620e-05_rb,2.8333e-07_rb/)
      kbo(:, 4,58, 7) = (/ &
     &1.1843e-04_rb,1.1052e-04_rb,8.9713e-05_rb,5.7328e-05_rb,4.1951e-07_rb/)
      kbo(:, 5,58, 7) = (/ &
     &1.9434e-04_rb,1.8218e-04_rb,1.4806e-04_rb,9.4685e-05_rb,5.9796e-07_rb/)
      kbo(:, 1,59, 7) = (/ &
     &1.6486e-05_rb,1.5394e-05_rb,1.2565e-05_rb,8.1204e-06_rb,9.1178e-08_rb/)
      kbo(:, 2,59, 7) = (/ &
     &3.1937e-05_rb,2.9800e-05_rb,2.4292e-05_rb,1.5665e-05_rb,1.5065e-07_rb/)
      kbo(:, 3,59, 7) = (/ &
     &5.7683e-05_rb,5.3905e-05_rb,4.3942e-05_rb,2.8299e-05_rb,2.3496e-07_rb/)
      kbo(:, 4,59, 7) = (/ &
     &9.9323e-05_rb,9.3088e-05_rb,7.5927e-05_rb,4.8813e-05_rb,3.4922e-07_rb/)
      kbo(:, 5,59, 7) = (/ &
     &1.6451e-04_rb,1.5507e-04_rb,1.2661e-04_rb,8.1439e-05_rb,5.0093e-07_rb/)
      kbo(:, 1,13, 8) = (/ &
     &1.2116e+00_rb,9.1758e-01_rb,6.2996e-01_rb,3.3536e-01_rb,1.5468e-02_rb/)
      kbo(:, 2,13, 8) = (/ &
     &1.6818e+00_rb,1.2746e+00_rb,8.7634e-01_rb,4.6518e-01_rb,1.8709e-02_rb/)
      kbo(:, 3,13, 8) = (/ &
     &2.2865e+00_rb,1.7360e+00_rb,1.1942e+00_rb,6.3525e-01_rb,2.2469e-02_rb/)
      kbo(:, 4,13, 8) = (/ &
     &3.0754e+00_rb,2.3371e+00_rb,1.6054e+00_rb,8.5102e-01_rb,2.6288e-02_rb/)
      kbo(:, 5,13, 8) = (/ &
     &4.0473e+00_rb,3.0761e+00_rb,2.1113e+00_rb,1.1162e+00_rb,3.0276e-02_rb/)
      kbo(:, 1,14, 8) = (/ &
     &1.1078e+00_rb,8.4059e-01_rb,5.7946e-01_rb,3.0612e-01_rb,1.2063e-02_rb/)
      kbo(:, 2,14, 8) = (/ &
     &1.5335e+00_rb,1.1661e+00_rb,8.0499e-01_rb,4.2627e-01_rb,1.4718e-02_rb/)
      kbo(:, 3,14, 8) = (/ &
     &2.0884e+00_rb,1.5886e+00_rb,1.0949e+00_rb,5.8050e-01_rb,1.7857e-02_rb/)
      kbo(:, 4,14, 8) = (/ &
     &2.8120e+00_rb,2.1413e+00_rb,1.4745e+00_rb,7.8098e-01_rb,2.1007e-02_rb/)
      kbo(:, 5,14, 8) = (/ &
     &3.6989e+00_rb,2.8189e+00_rb,1.9396e+00_rb,1.0257e+00_rb,2.4295e-02_rb/)
      kbo(:, 1,15, 8) = (/ &
     &9.7042e-01_rb,7.3947e-01_rb,5.1034e-01_rb,2.6949e-01_rb,9.7938e-03_rb/)
      kbo(:, 2,15, 8) = (/ &
     &1.3436e+00_rb,1.0262e+00_rb,7.0874e-01_rb,3.7517e-01_rb,1.2203e-02_rb/)
      kbo(:, 3,15, 8) = (/ &
     &1.8449e+00_rb,1.4109e+00_rb,9.7248e-01_rb,5.1316e-01_rb,1.4807e-02_rb/)
      kbo(:, 4,15, 8) = (/ &
     &2.4626e+00_rb,1.8840e+00_rb,1.3006e+00_rb,6.9030e-01_rb,1.7518e-02_rb/)
      kbo(:, 5,15, 8) = (/ &
     &3.2241e+00_rb,2.4640e+00_rb,1.6957e+00_rb,8.9766e-01_rb,2.0767e-02_rb/)
      kbo(:, 1,16, 8) = (/ &
     &8.2995e-01_rb,6.3671e-01_rb,4.3954e-01_rb,2.3206e-01_rb,8.0720e-03_rb/)
      kbo(:, 2,16, 8) = (/ &
     &1.1570e+00_rb,8.8835e-01_rb,6.1260e-01_rb,3.2369e-01_rb,1.0091e-02_rb/)
      kbo(:, 3,16, 8) = (/ &
     &1.5781e+00_rb,1.2128e+00_rb,8.3725e-01_rb,4.4354e-01_rb,1.2349e-02_rb/)
      kbo(:, 4,16, 8) = (/ &
     &2.1038e+00_rb,1.6176e+00_rb,1.1171e+00_rb,5.9216e-01_rb,1.4755e-02_rb/)
      kbo(:, 5,16, 8) = (/ &
     &2.7385e+00_rb,2.1023e+00_rb,1.4490e+00_rb,7.6892e-01_rb,1.7667e-02_rb/)
      kbo(:, 1,17, 8) = (/ &
     &7.0692e-01_rb,5.4586e-01_rb,3.7661e-01_rb,1.9889e-01_rb,6.7483e-03_rb/)
      kbo(:, 2,17, 8) = (/ &
     &9.8295e-01_rb,7.5893e-01_rb,5.2307e-01_rb,2.7659e-01_rb,8.4850e-03_rb/)
      kbo(:, 3,17, 8) = (/ &
     &1.3367e+00_rb,1.0327e+00_rb,7.1373e-01_rb,3.7884e-01_rb,1.0478e-02_rb/)
      kbo(:, 4,17, 8) = (/ &
     &1.7756e+00_rb,1.3697e+00_rb,9.4344e-01_rb,4.9980e-01_rb,1.2797e-02_rb/)
      kbo(:, 5,17, 8) = (/ &
     &2.3127e+00_rb,1.7838e+00_rb,1.2303e+00_rb,6.5347e-01_rb,1.5355e-02_rb/)
      kbo(:, 1,18, 8) = (/ &
     &5.9108e-01_rb,4.5836e-01_rb,3.1625e-01_rb,1.6744e-01_rb,5.7287e-03_rb/)
      kbo(:, 2,18, 8) = (/ &
     &8.2052e-01_rb,6.3681e-01_rb,4.3945e-01_rb,2.3289e-01_rb,7.3032e-03_rb/)
      kbo(:, 3,18, 8) = (/ &
     &1.1131e+00_rb,8.6338e-01_rb,5.9577e-01_rb,3.1578e-01_rb,9.0895e-03_rb/)
      kbo(:, 4,18, 8) = (/ &
     &1.4765e+00_rb,1.1444e+00_rb,7.8909e-01_rb,4.1827e-01_rb,1.1179e-02_rb/)
      kbo(:, 5,18, 8) = (/ &
     &1.9250e+00_rb,1.4898e+00_rb,1.0284e+00_rb,5.4576e-01_rb,1.3498e-02_rb/)
      kbo(:, 1,19, 8) = (/ &
     &5.0114e-01_rb,3.9028e-01_rb,2.6968e-01_rb,1.4303e-01_rb,5.0299e-03_rb/)
      kbo(:, 2,19, 8) = (/ &
     &6.9768e-01_rb,5.4308e-01_rb,3.7493e-01_rb,1.9896e-01_rb,6.4752e-03_rb/)
      kbo(:, 3,19, 8) = (/ &
     &9.4555e-01_rb,7.3537e-01_rb,5.0733e-01_rb,2.6864e-01_rb,8.1323e-03_rb/)
      kbo(:, 4,19, 8) = (/ &
     &1.2515e+00_rb,9.7204e-01_rb,6.7090e-01_rb,3.5577e-01_rb,1.0007e-02_rb/)
      kbo(:, 5,19, 8) = (/ &
     &1.6326e+00_rb,1.2674e+00_rb,8.7478e-01_rb,4.6363e-01_rb,1.2144e-02_rb/)
      kbo(:, 1,20, 8) = (/ &
     &4.3618e-01_rb,3.4070e-01_rb,2.3544e-01_rb,1.2514e-01_rb,4.4918e-03_rb/)
      kbo(:, 2,20, 8) = (/ &
     &6.0844e-01_rb,4.7506e-01_rb,3.2779e-01_rb,1.7405e-01_rb,5.8213e-03_rb/)
      kbo(:, 3,20, 8) = (/ &
     &8.2504e-01_rb,6.4315e-01_rb,4.4377e-01_rb,2.3521e-01_rb,7.3449e-03_rb/)
      kbo(:, 4,20, 8) = (/ &
     &1.0904e+00_rb,8.4892e-01_rb,5.8560e-01_rb,3.1033e-01_rb,9.1136e-03_rb/)
      kbo(:, 5,20, 8) = (/ &
     &1.4182e+00_rb,1.1040e+00_rb,7.6212e-01_rb,4.0404e-01_rb,1.1084e-02_rb/)
      kbo(:, 1,21, 8) = (/ &
     &3.8157e-01_rb,2.9889e-01_rb,2.0662e-01_rb,1.0989e-01_rb,4.0172e-03_rb/)
      kbo(:, 2,21, 8) = (/ &
     &5.3249e-01_rb,4.1652e-01_rb,2.8764e-01_rb,1.5288e-01_rb,5.2442e-03_rb/)
      kbo(:, 3,21, 8) = (/ &
     &7.2280e-01_rb,5.6456e-01_rb,3.8951e-01_rb,2.0661e-01_rb,6.6692e-03_rb/)
      kbo(:, 4,21, 8) = (/ &
     &9.5675e-01_rb,7.4634e-01_rb,5.1474e-01_rb,2.7285e-01_rb,8.2754e-03_rb/)
      kbo(:, 5,21, 8) = (/ &
     &1.2430e+00_rb,9.6862e-01_rb,6.6782e-01_rb,3.5364e-01_rb,1.0090e-02_rb/)
      kbo(:, 1,22, 8) = (/ &
     &3.4101e-01_rb,2.6749e-01_rb,1.8491e-01_rb,9.8339e-02_rb,3.6398e-03_rb/)
      kbo(:, 2,22, 8) = (/ &
     &4.7480e-01_rb,3.7184e-01_rb,2.5674e-01_rb,1.3643e-01_rb,4.7506e-03_rb/)
      kbo(:, 3,22, 8) = (/ &
     &6.4273e-01_rb,5.0295e-01_rb,3.4722e-01_rb,1.8435e-01_rb,6.0333e-03_rb/)
      kbo(:, 4,22, 8) = (/ &
     &8.5104e-01_rb,6.6530e-01_rb,4.5905e-01_rb,2.4340e-01_rb,7.4888e-03_rb/)
      kbo(:, 5,22, 8) = (/ &
     &1.1079e+00_rb,8.6526e-01_rb,5.9621e-01_rb,3.1544e-01_rb,9.1269e-03_rb/)
      kbo(:, 1,23, 8) = (/ &
     &3.0421e-01_rb,2.3897e-01_rb,1.6516e-01_rb,8.7923e-02_rb,3.2631e-03_rb/)
      kbo(:, 2,23, 8) = (/ &
     &4.2200e-01_rb,3.3116e-01_rb,2.2888e-01_rb,1.2169e-01_rb,4.2529e-03_rb/)
      kbo(:, 3,23, 8) = (/ &
     &5.7072e-01_rb,4.4747e-01_rb,3.0900e-01_rb,1.6407e-01_rb,5.3912e-03_rb/)
      kbo(:, 4,23, 8) = (/ &
     &7.5672e-01_rb,5.9253e-01_rb,4.0891e-01_rb,2.1688e-01_rb,6.6960e-03_rb/)
      kbo(:, 5,23, 8) = (/ &
     &9.8731e-01_rb,7.7239e-01_rb,5.3220e-01_rb,2.8204e-01_rb,8.2651e-03_rb/)
      kbo(:, 1,24, 8) = (/ &
     &2.7335e-01_rb,2.1493e-01_rb,1.4863e-01_rb,7.9120e-02_rb,2.9206e-03_rb/)
      kbo(:, 2,24, 8) = (/ &
     &3.7818e-01_rb,2.9704e-01_rb,2.0533e-01_rb,1.0922e-01_rb,3.7959e-03_rb/)
      kbo(:, 3,24, 8) = (/ &
     &5.1197e-01_rb,4.0167e-01_rb,2.7730e-01_rb,1.4723e-01_rb,4.8218e-03_rb/)
      kbo(:, 4,24, 8) = (/ &
     &6.8021e-01_rb,5.3323e-01_rb,3.6769e-01_rb,1.9508e-01_rb,6.0367e-03_rb/)
      kbo(:, 5,24, 8) = (/ &
     &8.8740e-01_rb,6.9520e-01_rb,4.7892e-01_rb,2.5373e-01_rb,7.4083e-03_rb/)
      kbo(:, 1,25, 8) = (/ &
     &2.4667e-01_rb,1.9408e-01_rb,1.3424e-01_rb,7.1436e-02_rb,2.6098e-03_rb/)
      kbo(:, 2,25, 8) = (/ &
     &3.4107e-01_rb,2.6809e-01_rb,1.8527e-01_rb,9.8480e-02_rb,3.3820e-03_rb/)
      kbo(:, 3,25, 8) = (/ &
     &4.6237e-01_rb,3.6308e-01_rb,2.5057e-01_rb,1.3298e-01_rb,4.3157e-03_rb/)
      kbo(:, 4,25, 8) = (/ &
     &6.1385e-01_rb,4.8168e-01_rb,3.3224e-01_rb,1.7632e-01_rb,5.3399e-03_rb/)
      kbo(:, 5,25, 8) = (/ &
     &7.9905e-01_rb,6.2682e-01_rb,4.3186e-01_rb,2.2913e-01_rb,6.4913e-03_rb/)
      kbo(:, 1,26, 8) = (/ &
     &2.2404e-01_rb,1.7649e-01_rb,1.2214e-01_rb,6.5022e-02_rb,2.3384e-03_rb/)
      kbo(:, 2,26, 8) = (/ &
     &3.1013e-01_rb,2.4403e-01_rb,1.6870e-01_rb,8.9741e-02_rb,3.0179e-03_rb/)
      kbo(:, 3,26, 8) = (/ &
     &4.2049e-01_rb,3.3066e-01_rb,2.2838e-01_rb,1.2138e-01_rb,3.8039e-03_rb/)
      kbo(:, 4,26, 8) = (/ &
     &5.5790e-01_rb,4.3821e-01_rb,3.0237e-01_rb,1.6056e-01_rb,4.6658e-03_rb/)
      kbo(:, 5,26, 8) = (/ &
     &7.2377e-01_rb,5.6837e-01_rb,3.9196e-01_rb,2.0795e-01_rb,5.6275e-03_rb/)
      kbo(:, 1,27, 8) = (/ &
     &2.0411e-01_rb,1.6088e-01_rb,1.1131e-01_rb,5.9303e-02_rb,2.0568e-03_rb/)
      kbo(:, 2,27, 8) = (/ &
     &2.8256e-01_rb,2.2260e-01_rb,1.5392e-01_rb,8.1961e-02_rb,2.6349e-03_rb/)
      kbo(:, 3,27, 8) = (/ &
     &3.8251e-01_rb,3.0112e-01_rb,2.0805e-01_rb,1.1068e-01_rb,3.2772e-03_rb/)
      kbo(:, 4,27, 8) = (/ &
     &5.0562e-01_rb,3.9785e-01_rb,2.7464e-01_rb,1.4598e-01_rb,4.0077e-03_rb/)
      kbo(:, 5,27, 8) = (/ &
     &6.5445e-01_rb,5.1441e-01_rb,3.5497e-01_rb,1.8851e-01_rb,4.8055e-03_rb/)
      kbo(:, 1,28, 8) = (/ &
     &1.8628e-01_rb,1.4699e-01_rb,1.0171e-01_rb,5.4235e-02_rb,1.7799e-03_rb/)
      kbo(:, 2,28, 8) = (/ &
     &2.5749e-01_rb,2.0322e-01_rb,1.4053e-01_rb,7.4881e-02_rb,2.2515e-03_rb/)
      kbo(:, 3,28, 8) = (/ &
     &3.4728e-01_rb,2.7380e-01_rb,1.8920e-01_rb,1.0075e-01_rb,2.7863e-03_rb/)
      kbo(:, 4,28, 8) = (/ &
     &4.5755e-01_rb,3.6044e-01_rb,2.4894e-01_rb,1.3243e-01_rb,3.3903e-03_rb/)
      kbo(:, 5,28, 8) = (/ &
     &5.9045e-01_rb,4.6494e-01_rb,3.2101e-01_rb,1.7052e-01_rb,4.0404e-03_rb/)
      kbo(:, 1,29, 8) = (/ &
     &1.6897e-01_rb,1.3358e-01_rb,9.2501e-02_rb,4.9422e-02_rb,1.5070e-03_rb/)
      kbo(:, 2,29, 8) = (/ &
     &2.3294e-01_rb,1.8406e-01_rb,1.2733e-01_rb,6.7942e-02_rb,1.8975e-03_rb/)
      kbo(:, 3,29, 8) = (/ &
     &3.1307e-01_rb,2.4704e-01_rb,1.7081e-01_rb,9.1043e-02_rb,2.3375e-03_rb/)
      kbo(:, 4,29, 8) = (/ &
     &4.1126e-01_rb,3.2432e-01_rb,2.2417e-01_rb,1.1931e-01_rb,2.8199e-03_rb/)
      kbo(:, 5,29, 8) = (/ &
     &5.2992e-01_rb,4.1770e-01_rb,2.8860e-01_rb,1.5337e-01_rb,3.3535e-03_rb/)
      kbo(:, 1,30, 8) = (/ &
     &1.5242e-01_rb,1.2071e-01_rb,8.3664e-02_rb,4.4778e-02_rb,1.2669e-03_rb/)
      kbo(:, 2,30, 8) = (/ &
     &2.0927e-01_rb,1.6559e-01_rb,1.1465e-01_rb,6.1290e-02_rb,1.5824e-03_rb/)
      kbo(:, 3,30, 8) = (/ &
     &2.8036e-01_rb,2.2167e-01_rb,1.5340e-01_rb,8.1865e-02_rb,1.9342e-03_rb/)
      kbo(:, 4,30, 8) = (/ &
     &3.6796e-01_rb,2.9065e-01_rb,2.0100e-01_rb,1.0707e-01_rb,2.3143e-03_rb/)
      kbo(:, 5,30, 8) = (/ &
     &4.7407e-01_rb,3.7431e-01_rb,2.5849e-01_rb,1.3745e-01_rb,2.7612e-03_rb/)
      kbo(:, 1,31, 8) = (/ &
     &1.3611e-01_rb,1.0796e-01_rb,7.4865e-02_rb,4.0122e-02_rb,1.0479e-03_rb/)
      kbo(:, 2,31, 8) = (/ &
     &1.8611e-01_rb,1.4751e-01_rb,1.0224e-01_rb,5.4734e-02_rb,1.2997e-03_rb/)
      kbo(:, 3,31, 8) = (/ &
     &2.4882e-01_rb,1.9706e-01_rb,1.3650e-01_rb,7.2954e-02_rb,1.5789e-03_rb/)
      kbo(:, 4,31, 8) = (/ &
     &3.2631e-01_rb,2.5819e-01_rb,1.7872e-01_rb,9.5286e-02_rb,1.8990e-03_rb/)
      kbo(:, 5,31, 8) = (/ &
     &4.2088e-01_rb,3.3264e-01_rb,2.2986e-01_rb,1.2231e-01_rb,2.2523e-03_rb/)
      kbo(:, 1,32, 8) = (/ &
     &1.2210e-01_rb,9.6994e-02_rb,6.7326e-02_rb,3.6123e-02_rb,8.5904e-04_rb/)
      kbo(:, 2,32, 8) = (/ &
     &1.6652e-01_rb,1.3210e-01_rb,9.1670e-02_rb,4.9107e-02_rb,1.0623e-03_rb/)
      kbo(:, 3,32, 8) = (/ &
     &2.2222e-01_rb,1.7626e-01_rb,1.2220e-01_rb,6.5357e-02_rb,1.2931e-03_rb/)
      kbo(:, 4,32, 8) = (/ &
     &2.9160e-01_rb,2.3096e-01_rb,1.5993e-01_rb,8.5342e-02_rb,1.5468e-03_rb/)
      kbo(:, 5,32, 8) = (/ &
     &3.7617e-01_rb,2.9768e-01_rb,2.0589e-01_rb,1.0971e-01_rb,1.8394e-03_rb/)
      kbo(:, 1,33, 8) = (/ &
     &1.0986e-01_rb,8.7382e-02_rb,6.0737e-02_rb,3.2609e-02_rb,7.0177e-04_rb/)
      kbo(:, 2,33, 8) = (/ &
     &1.4961e-01_rb,1.1887e-01_rb,8.2542e-02_rb,4.4260e-02_rb,8.6489e-04_rb/)
      kbo(:, 3,33, 8) = (/ &
     &1.9974e-01_rb,1.5861e-01_rb,1.0995e-01_rb,5.8836e-02_rb,1.0533e-03_rb/)
      kbo(:, 4,33, 8) = (/ &
     &2.6219e-01_rb,2.0793e-01_rb,1.4396e-01_rb,7.6884e-02_rb,1.2624e-03_rb/)
      kbo(:, 5,33, 8) = (/ &
     &3.3855e-01_rb,2.6852e-01_rb,1.8574e-01_rb,9.9060e-02_rb,1.4976e-03_rb/)
      kbo(:, 1,34, 8) = (/ &
     &9.9355e-02_rb,7.9091e-02_rb,5.5001e-02_rb,2.9534e-02_rb,5.8155e-04_rb/)
      kbo(:, 2,34, 8) = (/ &
     &1.3530e-01_rb,1.0764e-01_rb,7.4771e-02_rb,4.0076e-02_rb,7.1503e-04_rb/)
      kbo(:, 3,34, 8) = (/ &
     &1.8087e-01_rb,1.4373e-01_rb,9.9666e-02_rb,5.3339e-02_rb,8.6827e-04_rb/)
      kbo(:, 4,34, 8) = (/ &
     &2.3750e-01_rb,1.8867e-01_rb,1.3063e-01_rb,6.9822e-02_rb,1.0409e-03_rb/)
      kbo(:, 5,34, 8) = (/ &
     &3.0762e-01_rb,2.4432e-01_rb,1.6897e-01_rb,9.0140e-02_rb,1.2357e-03_rb/)
      kbo(:, 1,35, 8) = (/ &
     &8.7519e-02_rb,6.9769e-02_rb,4.8552e-02_rb,2.6085e-02_rb,4.6868e-04_rb/)
      kbo(:, 2,35, 8) = (/ &
     &1.1953e-01_rb,9.5204e-02_rb,6.6129e-02_rb,3.5470e-02_rb,5.7740e-04_rb/)
      kbo(:, 3,35, 8) = (/ &
     &1.6035e-01_rb,1.2756e-01_rb,8.8463e-02_rb,4.7361e-02_rb,7.0118e-04_rb/)
      kbo(:, 4,35, 8) = (/ &
     &2.1123e-01_rb,1.6801e-01_rb,1.1639e-01_rb,6.2235e-02_rb,8.4220e-04_rb/)
      kbo(:, 5,35, 8) = (/ &
     &2.7484e-01_rb,2.1861e-01_rb,1.5121e-01_rb,8.0717e-02_rb,1.0033e-03_rb/)
      kbo(:, 1,36, 8) = (/ &
     &7.4849e-02_rb,5.9737e-02_rb,4.1593e-02_rb,2.2353e-02_rb,3.6660e-04_rb/)
      kbo(:, 2,36, 8) = (/ &
     &1.0278e-01_rb,8.1972e-02_rb,5.6955e-02_rb,3.0544e-02_rb,4.5336e-04_rb/)
      kbo(:, 3,36, 8) = (/ &
     &1.3843e-01_rb,1.1029e-01_rb,7.6519e-02_rb,4.1000e-02_rb,5.5390e-04_rb/)
      kbo(:, 4,36, 8) = (/ &
     &1.8385e-01_rb,1.4649e-01_rb,1.0149e-01_rb,5.4257e-02_rb,6.6682e-04_rb/)
      kbo(:, 5,36, 8) = (/ &
     &2.4059e-01_rb,1.9160e-01_rb,1.3256e-01_rb,7.0808e-02_rb,7.9593e-04_rb/)
      kbo(:, 1,37, 8) = (/ &
     &6.1778e-02_rb,4.9350e-02_rb,3.4377e-02_rb,1.8481e-02_rb,2.8834e-04_rb/)
      kbo(:, 2,37, 8) = (/ &
     &8.5595e-02_rb,6.8343e-02_rb,4.7513e-02_rb,2.5474e-02_rb,3.5693e-04_rb/)
      kbo(:, 3,37, 8) = (/ &
     &1.1627e-01_rb,9.2744e-02_rb,6.4387e-02_rb,3.4481e-02_rb,4.3707e-04_rb/)
      kbo(:, 4,37, 8) = (/ &
     &1.5562e-01_rb,1.2411e-01_rb,8.6025e-02_rb,4.6007e-02_rb,5.2965e-04_rb/)
      kbo(:, 5,37, 8) = (/ &
     &2.0514e-01_rb,1.6357e-01_rb,1.1324e-01_rb,6.0461e-02_rb,6.3315e-04_rb/)
      kbo(:, 1,38, 8) = (/ &
     &5.0881e-02_rb,4.0678e-02_rb,2.8356e-02_rb,1.5247e-02_rb,2.2603e-04_rb/)
      kbo(:, 2,38, 8) = (/ &
     &7.1115e-02_rb,5.6873e-02_rb,3.9548e-02_rb,2.1211e-02_rb,2.8054e-04_rb/)
      kbo(:, 3,38, 8) = (/ &
     &9.7490e-02_rb,7.7896e-02_rb,5.4103e-02_rb,2.8959e-02_rb,3.4399e-04_rb/)
      kbo(:, 4,38, 8) = (/ &
     &1.3157e-01_rb,1.0506e-01_rb,7.2849e-02_rb,3.8945e-02_rb,4.1860e-04_rb/)
      kbo(:, 5,38, 8) = (/ &
     &1.7483e-01_rb,1.3966e-01_rb,9.6758e-02_rb,5.1645e-02_rb,5.0365e-04_rb/)
      kbo(:, 1,39, 8) = (/ &
     &4.1872e-02_rb,3.3548e-02_rb,2.3398e-02_rb,1.2576e-02_rb,1.7614e-04_rb/)
      kbo(:, 2,39, 8) = (/ &
     &5.9094e-02_rb,4.7316e-02_rb,3.2924e-02_rb,1.7662e-02_rb,2.1952e-04_rb/)
      kbo(:, 3,39, 8) = (/ &
     &8.1814e-02_rb,6.5458e-02_rb,4.5468e-02_rb,2.4343e-02_rb,2.7005e-04_rb/)
      kbo(:, 4,39, 8) = (/ &
     &1.1129e-01_rb,8.9026e-02_rb,6.1776e-02_rb,3.3031e-02_rb,3.3030e-04_rb/)
      kbo(:, 5,39, 8) = (/ &
     &1.4922e-01_rb,1.1944e-01_rb,8.2784e-02_rb,4.4132e-02_rb,4.0030e-04_rb/)
      kbo(:, 1,40, 8) = (/ &
     &3.3647e-02_rb,2.6996e-02_rb,1.8830e-02_rb,1.0123e-02_rb,1.3664e-04_rb/)
      kbo(:, 2,40, 8) = (/ &
     &4.8077e-02_rb,3.8535e-02_rb,2.6836e-02_rb,1.4394e-02_rb,1.7115e-04_rb/)
      kbo(:, 3,40, 8) = (/ &
     &6.7356e-02_rb,5.3948e-02_rb,3.7498e-02_rb,2.0063e-02_rb,2.1194e-04_rb/)
      kbo(:, 4,40, 8) = (/ &
     &9.2612e-02_rb,7.4161e-02_rb,5.1473e-02_rb,2.7514e-02_rb,2.6017e-04_rb/)
      kbo(:, 5,40, 8) = (/ &
     &1.2534e-01_rb,1.0040e-01_rb,6.9623e-02_rb,3.7148e-02_rb,3.1694e-04_rb/)
      kbo(:, 1,41, 8) = (/ &
     &2.6919e-02_rb,2.1625e-02_rb,1.5091e-02_rb,8.1073e-03_rb,1.0569e-04_rb/)
      kbo(:, 2,41, 8) = (/ &
     &3.8959e-02_rb,3.1258e-02_rb,2.1769e-02_rb,1.1674e-02_rb,1.3335e-04_rb/)
      kbo(:, 3,41, 8) = (/ &
     &5.5227e-02_rb,4.4304e-02_rb,3.0810e-02_rb,1.6485e-02_rb,1.6620e-04_rb/)
      kbo(:, 4,41, 8) = (/ &
     &7.6819e-02_rb,6.1609e-02_rb,4.2770e-02_rb,2.2843e-02_rb,2.0484e-04_rb/)
      kbo(:, 5,41, 8) = (/ &
     &1.0504e-01_rb,8.4290e-02_rb,5.8439e-02_rb,3.1171e-02_rb,2.5031e-04_rb/)
      kbo(:, 1,42, 8) = (/ &
     &2.1470e-02_rb,1.7281e-02_rb,1.2058e-02_rb,6.4788e-03_rb,8.1238e-05_rb/)
      kbo(:, 2,42, 8) = (/ &
     &3.1494e-02_rb,2.5316e-02_rb,1.7631e-02_rb,9.4523e-03_rb,1.0344e-04_rb/)
      kbo(:, 3,42, 8) = (/ &
     &4.5222e-02_rb,3.6333e-02_rb,2.5262e-02_rb,1.3522e-02_rb,1.2977e-04_rb/)
      kbo(:, 4,42, 8) = (/ &
     &6.3644e-02_rb,5.1128e-02_rb,3.5506e-02_rb,1.8951e-02_rb,1.6105e-04_rb/)
      kbo(:, 5,42, 8) = (/ &
     &8.7974e-02_rb,7.0671e-02_rb,4.9043e-02_rb,2.6143e-02_rb,1.9719e-04_rb/)
      kbo(:, 1,43, 8) = (/ &
     &1.6861e-02_rb,1.3590e-02_rb,9.4853e-03_rb,5.0945e-03_rb,6.1844e-05_rb/)
      kbo(:, 2,43, 8) = (/ &
     &2.5102e-02_rb,2.0210e-02_rb,1.4080e-02_rb,7.5450e-03_rb,7.9573e-05_rb/)
      kbo(:, 3,43, 8) = (/ &
     &3.6570e-02_rb,2.9430e-02_rb,2.0473e-02_rb,1.0954e-02_rb,1.0063e-04_rb/)
      kbo(:, 4,43, 8) = (/ &
     &5.2231e-02_rb,4.1980e-02_rb,2.9163e-02_rb,1.5562e-02_rb,1.2552e-04_rb/)
      kbo(:, 5,43, 8) = (/ &
     &7.3055e-02_rb,5.8741e-02_rb,4.0761e-02_rb,2.1719e-02_rb,1.5491e-04_rb/)
      kbo(:, 1,44, 8) = (/ &
     &1.3112e-02_rb,1.0583e-02_rb,7.3867e-03_rb,3.9656e-03_rb,4.6708e-05_rb/)
      kbo(:, 2,44, 8) = (/ &
     &1.9838e-02_rb,1.5995e-02_rb,1.1144e-02_rb,5.9703e-03_rb,6.0776e-05_rb/)
      kbo(:, 3,44, 8) = (/ &
     &2.9345e-02_rb,2.3649e-02_rb,1.6458e-02_rb,8.7984e-03_rb,7.7487e-05_rb/)
      kbo(:, 4,44, 8) = (/ &
     &4.2542e-02_rb,3.4237e-02_rb,2.3782e-02_rb,1.2696e-02_rb,9.7391e-05_rb/)
      kbo(:, 5,44, 8) = (/ &
     &6.0327e-02_rb,4.8544e-02_rb,3.3677e-02_rb,1.7934e-02_rb,1.2115e-04_rb/)
      kbo(:, 1,45, 8) = (/ &
     &1.0146e-02_rb,8.1985e-03_rb,5.7225e-03_rb,3.0712e-03_rb,3.5068e-05_rb/)
      kbo(:, 2,45, 8) = (/ &
     &1.5607e-02_rb,1.2601e-02_rb,8.7806e-03_rb,4.7019e-03_rb,4.6144e-05_rb/)
      kbo(:, 3,45, 8) = (/ &
     &2.3451e-02_rb,1.8930e-02_rb,1.3177e-02_rb,7.0434e-03_rb,5.9457e-05_rb/)
      kbo(:, 4,45, 8) = (/ &
     &3.4514e-02_rb,2.7825e-02_rb,1.9327e-02_rb,1.0311e-02_rb,7.5434e-05_rb/)
      kbo(:, 5,45, 8) = (/ &
     &4.9675e-02_rb,4.0043e-02_rb,2.7784e-02_rb,1.4786e-02_rb,9.4540e-05_rb/)
      kbo(:, 1,46, 8) = (/ &
     &7.7351e-03_rb,6.2590e-03_rb,4.3712e-03_rb,2.3470e-03_rb,2.6024e-05_rb/)
      kbo(:, 2,46, 8) = (/ &
     &1.2117e-02_rb,9.8019e-03_rb,6.8312e-03_rb,3.6581e-03_rb,3.4737e-05_rb/)
      kbo(:, 3,46, 8) = (/ &
     &1.8521e-02_rb,1.4972e-02_rb,1.0423e-02_rb,5.5707e-03_rb,4.5227e-05_rb/)
      kbo(:, 4,46, 8) = (/ &
     &2.7718e-02_rb,2.2376e-02_rb,1.5549e-02_rb,8.2942e-03_rb,5.8064e-05_rb/)
      kbo(:, 5,46, 8) = (/ &
     &4.0521e-02_rb,3.2720e-02_rb,2.2708e-02_rb,1.2084e-02_rb,7.3480e-05_rb/)
      kbo(:, 1,47, 8) = (/ &
     &5.7592e-03_rb,4.6672e-03_rb,3.2593e-03_rb,1.7501e-03_rb,1.9095e-05_rb/)
      kbo(:, 2,47, 8) = (/ &
     &9.2060e-03_rb,7.4576e-03_rb,5.1997e-03_rb,2.7848e-03_rb,2.5839e-05_rb/)
      kbo(:, 3,47, 8) = (/ &
     &1.4346e-02_rb,1.1614e-02_rb,8.0835e-03_rb,4.3208e-03_rb,3.4203e-05_rb/)
      kbo(:, 4,47, 8) = (/ &
     &2.1845e-02_rb,1.7678e-02_rb,1.2284e-02_rb,6.5493e-03_rb,4.4407e-05_rb/)
      kbo(:, 5,47, 8) = (/ &
     &3.2496e-02_rb,2.6280e-02_rb,1.8234e-02_rb,9.7018e-03_rb,5.6773e-05_rb/)
      kbo(:, 1,48, 8) = (/ &
     &4.2511e-03_rb,3.4498e-03_rb,2.4093e-03_rb,1.2936e-03_rb,1.3859e-05_rb/)
      kbo(:, 2,48, 8) = (/ &
     &6.9347e-03_rb,5.6254e-03_rb,3.9226e-03_rb,2.1019e-03_rb,1.9117e-05_rb/)
      kbo(:, 3,48, 8) = (/ &
     &1.1018e-02_rb,8.9390e-03_rb,6.2242e-03_rb,3.3284e-03_rb,2.5648e-05_rb/)
      kbo(:, 4,48, 8) = (/ &
     &1.7113e-02_rb,1.3870e-02_rb,9.6439e-03_rb,5.1401e-03_rb,3.3736e-05_rb/)
      kbo(:, 5,48, 8) = (/ &
     &2.5919e-02_rb,2.1005e-02_rb,1.4575e-02_rb,7.7539e-03_rb,4.3617e-05_rb/)
      kbo(:, 1,49, 8) = (/ &
     &3.1098e-03_rb,2.5273e-03_rb,1.7645e-03_rb,9.4708e-04_rb,9.9370e-06_rb/)
      kbo(:, 2,49, 8) = (/ &
     &5.1784e-03_rb,4.2068e-03_rb,2.9339e-03_rb,1.5725e-03_rb,1.3987e-05_rb/)
      kbo(:, 3,49, 8) = (/ &
     &8.3947e-03_rb,6.8228e-03_rb,4.7538e-03_rb,2.5418e-03_rb,1.9085e-05_rb/)
      kbo(:, 4,49, 8) = (/ &
     &1.3293e-02_rb,1.0799e-02_rb,7.5113e-03_rb,4.0058e-03_rb,2.5451e-05_rb/)
      kbo(:, 5,49, 8) = (/ &
     &2.0521e-02_rb,1.6668e-02_rb,1.1574e-02_rb,6.1522e-03_rb,3.3363e-05_rb/)
      kbo(:, 1,50, 8) = (/ &
     &2.2918e-03_rb,1.8636e-03_rb,1.3008e-03_rb,6.9789e-04_rb,7.1979e-06_rb/)
      kbo(:, 2,50, 8) = (/ &
     &3.8913e-03_rb,3.1670e-03_rb,2.2089e-03_rb,1.1842e-03_rb,1.0331e-05_rb/)
      kbo(:, 3,50, 8) = (/ &
     &6.4390e-03_rb,5.2441e-03_rb,3.6531e-03_rb,1.9538e-03_rb,1.4344e-05_rb/)
      kbo(:, 4,50, 8) = (/ &
     &1.0397e-02_rb,8.4649e-03_rb,5.8892e-03_rb,3.1423e-03_rb,1.9397e-05_rb/)
      kbo(:, 5,50, 8) = (/ &
     &1.6371e-02_rb,1.3317e-02_rb,9.2504e-03_rb,4.9185e-03_rb,2.5741e-05_rb/)
      kbo(:, 1,51, 8) = (/ &
     &1.6928e-03_rb,1.3767e-03_rb,9.6023e-04_rb,5.1382e-04_rb,5.2246e-06_rb/)
      kbo(:, 2,51, 8) = (/ &
     &2.9272e-03_rb,2.3839e-03_rb,1.6628e-03_rb,8.9103e-04_rb,7.6388e-06_rb/)
      kbo(:, 3,51, 8) = (/ &
     &4.9429e-03_rb,4.0319e-03_rb,2.8087e-03_rb,1.5020e-03_rb,1.0781e-05_rb/)
      kbo(:, 4,51, 8) = (/ &
     &8.1364e-03_rb,6.6389e-03_rb,4.6188e-03_rb,2.4654e-03_rb,1.4827e-05_rb/)
      kbo(:, 5,51, 8) = (/ &
     &1.3068e-02_rb,1.0656e-02_rb,7.4020e-03_rb,3.9372e-03_rb,1.9921e-05_rb/)
      kbo(:, 1,52, 8) = (/ &
     &1.2399e-03_rb,1.0091e-03_rb,7.0357e-04_rb,3.7650e-04_rb,3.7522e-06_rb/)
      kbo(:, 2,52, 8) = (/ &
     &2.1899e-03_rb,1.7829e-03_rb,1.2430e-03_rb,6.6579e-04_rb,5.6100e-06_rb/)
      kbo(:, 3,52, 8) = (/ &
     &3.7665e-03_rb,3.0795e-03_rb,2.1447e-03_rb,1.1464e-03_rb,8.0632e-06_rb/)
      kbo(:, 4,52, 8) = (/ &
     &6.3261e-03_rb,5.1729e-03_rb,3.5994e-03_rb,1.9224e-03_rb,1.1265e-05_rb/)
      kbo(:, 5,52, 8) = (/ &
     &1.0363e-02_rb,8.4710e-03_rb,5.8842e-03_rb,3.1302e-03_rb,1.5370e-05_rb/)
      kbo(:, 1,53, 8) = (/ &
     &9.0216e-04_rb,7.3408e-04_rb,5.1164e-04_rb,2.7340e-04_rb,2.6590e-06_rb/)
      kbo(:, 2,53, 8) = (/ &
     &1.6262e-03_rb,1.3270e-03_rb,9.2425e-04_rb,4.9425e-04_rb,4.0810e-06_rb/)
      kbo(:, 3,53, 8) = (/ &
     &2.8513e-03_rb,2.3327e-03_rb,1.6237e-03_rb,8.6789e-04_rb,5.9668e-06_rb/)
      kbo(:, 4,53, 8) = (/ &
     &4.8843e-03_rb,4.0007e-03_rb,2.7846e-03_rb,1.4866e-03_rb,8.4879e-06_rb/)
      kbo(:, 5,53, 8) = (/ &
     &8.1619e-03_rb,6.6885e-03_rb,4.6478e-03_rb,2.4735e-03_rb,1.1744e-05_rb/)
      kbo(:, 1,54, 8) = (/ &
     &6.6939e-04_rb,5.4497e-04_rb,3.7910e-04_rb,2.0225e-04_rb,1.9239e-06_rb/)
      kbo(:, 2,54, 8) = (/ &
     &1.2278e-03_rb,1.0030e-03_rb,6.9836e-04_rb,3.7314e-04_rb,3.0194e-06_rb/)
      kbo(:, 3,54, 8) = (/ &
     &2.1984e-03_rb,1.7996e-03_rb,1.2511e-03_rb,6.6777e-04_rb,4.5056e-06_rb/)
      kbo(:, 4,54, 8) = (/ &
     &3.8331e-03_rb,3.1457e-03_rb,2.1886e-03_rb,1.1687e-03_rb,6.4964e-06_rb/)
      kbo(:, 5,54, 8) = (/ &
     &6.5432e-03_rb,5.3709e-03_rb,3.7307e-03_rb,1.9859e-03_rb,9.1107e-06_rb/)
      kbo(:, 1,55, 8) = (/ &
     &5.0111e-04_rb,4.0755e-04_rb,2.8321e-04_rb,1.5081e-04_rb,1.4025e-06_rb/)
      kbo(:, 2,55, 8) = (/ &
     &9.3502e-04_rb,7.6365e-04_rb,5.3113e-04_rb,2.8338e-04_rb,2.2498e-06_rb/)
      kbo(:, 3,55, 8) = (/ &
     &1.7077e-03_rb,1.3993e-03_rb,9.7261e-04_rb,5.1891e-04_rb,3.4289e-06_rb/)
      kbo(:, 4,55, 8) = (/ &
     &3.0342e-03_rb,2.4941e-03_rb,1.7338e-03_rb,9.2454e-04_rb,5.0131e-06_rb/)
      kbo(:, 5,55, 8) = (/ &
     &5.2834e-03_rb,4.3447e-03_rb,3.0182e-03_rb,1.6053e-03_rb,7.1210e-06_rb/)
      kbo(:, 1,56, 8) = (/ &
     &3.7616e-04_rb,3.0519e-04_rb,2.1170e-04_rb,1.1247e-04_rb,1.0140e-06_rb/)
      kbo(:, 2,56, 8) = (/ &
     &7.1281e-04_rb,5.8173e-04_rb,4.0420e-04_rb,2.1505e-04_rb,1.6673e-06_rb/)
      kbo(:, 3,56, 8) = (/ &
     &1.3229e-03_rb,1.0846e-03_rb,7.5367e-04_rb,4.0126e-04_rb,2.5921e-06_rb/)
      kbo(:, 4,56, 8) = (/ &
     &2.4034e-03_rb,1.9754e-03_rb,1.3714e-03_rb,7.3027e-04_rb,3.8577e-06_rb/)
      kbo(:, 5,56, 8) = (/ &
     &4.2589e-03_rb,3.5089e-03_rb,2.4364e-03_rb,1.2956e-03_rb,5.5656e-06_rb/)
      kbo(:, 1,57, 8) = (/ &
     &2.8163e-04_rb,2.2851e-04_rb,1.5825e-04_rb,8.3801e-05_rb,7.2920e-07_rb/)
      kbo(:, 2,57, 8) = (/ &
     &5.4316e-04_rb,4.4288e-04_rb,3.0736e-04_rb,1.6324e-04_rb,1.2275e-06_rb/)
      kbo(:, 3,57, 8) = (/ &
     &1.0242e-03_rb,8.3907e-04_rb,5.8250e-04_rb,3.0984e-04_rb,1.9504e-06_rb/)
      kbo(:, 4,57, 8) = (/ &
     &1.8977e-03_rb,1.5630e-03_rb,1.0848e-03_rb,5.7691e-04_rb,2.9526e-06_rb/)
      kbo(:, 5,57, 8) = (/ &
     &3.4290e-03_rb,2.8298e-03_rb,1.9639e-03_rb,1.0437e-03_rb,4.3203e-06_rb/)
      kbo(:, 1,58, 8) = (/ &
     &1.0299e-04_rb,9.5896e-05_rb,7.7844e-05_rb,4.9788e-05_rb,5.2623e-07_rb/)
      kbo(:, 2,58, 8) = (/ &
     &2.0307e-04_rb,1.9013e-04_rb,1.5450e-04_rb,9.9035e-05_rb,9.0616e-07_rb/)
      kbo(:, 3,58, 8) = (/ &
     &3.8839e-04_rb,3.6515e-04_rb,2.9695e-04_rb,1.9078e-04_rb,1.4737e-06_rb/)
      kbo(:, 4,58, 8) = (/ &
     &7.3294e-04_rb,6.9303e-04_rb,5.6426e-04_rb,3.6255e-04_rb,2.2710e-06_rb/)
      kbo(:, 5,58, 8) = (/ &
     &1.3532e-03_rb,1.2806e-03_rb,1.0426e-03_rb,6.7017e-04_rb,3.3698e-06_rb/)
      kbo(:, 1,59, 8) = (/ &
     &8.5450e-05_rb,7.9848e-05_rb,6.5073e-05_rb,4.1875e-05_rb,4.3253e-07_rb/)
      kbo(:, 2,59, 8) = (/ &
     &1.7170e-04_rb,1.6133e-04_rb,1.3170e-04_rb,8.4866e-05_rb,7.5223e-07_rb/)
      kbo(:, 3,59, 8) = (/ &
     &3.3324e-04_rb,3.1480e-04_rb,2.5716e-04_rb,1.6610e-04_rb,1.2329e-06_rb/)
      kbo(:, 4,59, 8) = (/ &
     &6.4000e-04_rb,6.0786e-04_rb,4.9691e-04_rb,3.2112e-04_rb,1.9165e-06_rb/)
      kbo(:, 5,59, 8) = (/ &
     &1.2020e-03_rb,1.1456e-03_rb,9.3584e-04_rb,6.0398e-04_rb,2.8638e-06_rb/)
      kbo(:, 1,13, 9) = (/ &
     &7.4699e+00_rb,5.6062e+00_rb,3.7650e+00_rb,1.9560e+00_rb,8.5273e-02_rb/)
      kbo(:, 2,13, 9) = (/ &
     &1.0680e+01_rb,8.0165e+00_rb,5.3854e+00_rb,2.8036e+00_rb,1.0411e-01_rb/)
      kbo(:, 3,13, 9) = (/ &
     &1.4623e+01_rb,1.0976e+01_rb,7.3752e+00_rb,3.8391e+00_rb,1.2390e-01_rb/)
      kbo(:, 4,13, 9) = (/ &
     &1.9264e+01_rb,1.4459e+01_rb,9.7226e+00_rb,5.0654e+00_rb,1.4616e-01_rb/)
      kbo(:, 5,13, 9) = (/ &
     &2.4629e+01_rb,1.8491e+01_rb,1.2437e+01_rb,6.4792e+00_rb,1.7014e-01_rb/)
      kbo(:, 1,14, 9) = (/ &
     &6.8938e+00_rb,5.1783e+00_rb,3.4875e+00_rb,1.8264e+00_rb,7.6714e-02_rb/)
      kbo(:, 2,14, 9) = (/ &
     &9.8527e+00_rb,7.4006e+00_rb,4.9851e+00_rb,2.6106e+00_rb,9.2077e-02_rb/)
      kbo(:, 3,14, 9) = (/ &
     &1.3464e+01_rb,1.0115e+01_rb,6.8188e+00_rb,3.5707e+00_rb,1.0926e-01_rb/)
      kbo(:, 4,14, 9) = (/ &
     &1.7717e+01_rb,1.3311e+01_rb,8.9749e+00_rb,4.7000e+00_rb,1.2815e-01_rb/)
      kbo(:, 5,14, 9) = (/ &
     &2.2681e+01_rb,1.7041e+01_rb,1.1492e+01_rb,6.0178e+00_rb,1.4885e-01_rb/)
      kbo(:, 1,15, 9) = (/ &
     &6.1047e+00_rb,4.5908e+00_rb,3.1074e+00_rb,1.6392e+00_rb,6.2768e-02_rb/)
      kbo(:, 2,15, 9) = (/ &
     &8.7019e+00_rb,6.5442e+00_rb,4.4301e+00_rb,2.3356e+00_rb,7.5768e-02_rb/)
      kbo(:, 3,15, 9) = (/ &
     &1.1847e+01_rb,8.9091e+00_rb,6.0354e+00_rb,3.1840e+00_rb,9.0152e-02_rb/)
      kbo(:, 4,15, 9) = (/ &
     &1.5627e+01_rb,1.1753e+01_rb,7.9592e+00_rb,4.1918e+00_rb,1.0596e-01_rb/)
      kbo(:, 5,15, 9) = (/ &
     &2.0065e+01_rb,1.5096e+01_rb,1.0229e+01_rb,5.3867e+00_rb,1.2348e-01_rb/)
      kbo(:, 1,16, 9) = (/ &
     &5.2710e+00_rb,3.9706e+00_rb,2.7052e+00_rb,1.4383e+00_rb,5.0640e-02_rb/)
      kbo(:, 2,16, 9) = (/ &
     &7.4843e+00_rb,5.6386e+00_rb,3.8435e+00_rb,2.0428e+00_rb,6.0690e-02_rb/)
      kbo(:, 3,16, 9) = (/ &
     &1.0203e+01_rb,7.6869e+00_rb,5.2380e+00_rb,2.7809e+00_rb,7.2017e-02_rb/)
      kbo(:, 4,16, 9) = (/ &
     &1.3478e+01_rb,1.0155e+01_rb,6.9170e+00_rb,3.6687e+00_rb,8.5243e-02_rb/)
      kbo(:, 5,16, 9) = (/ &
     &1.7365e+01_rb,1.3088e+01_rb,8.9123e+00_rb,4.7182e+00_rb,9.9040e-02_rb/)
      kbo(:, 1,17, 9) = (/ &
     &4.4902e+00_rb,3.3911e+00_rb,2.3244e+00_rb,1.2481e+00_rb,4.0283e-02_rb/)
      kbo(:, 2,17, 9) = (/ &
     &6.3739e+00_rb,4.8152e+00_rb,3.3005e+00_rb,1.7693e+00_rb,4.9025e-02_rb/)
      kbo(:, 3,17, 9) = (/ &
     &8.7028e+00_rb,6.5740e+00_rb,4.5022e+00_rb,2.4070e+00_rb,5.9283e-02_rb/)
      kbo(:, 4,17, 9) = (/ &
     &1.1530e+01_rb,8.7139e+00_rb,5.9683e+00_rb,3.1839e+00_rb,7.0217e-02_rb/)
      kbo(:, 5,17, 9) = (/ &
     &1.4910e+01_rb,1.1266e+01_rb,7.7058e+00_rb,4.0973e+00_rb,8.1092e-02_rb/)
      kbo(:, 1,18, 9) = (/ &
     &3.7512e+00_rb,2.8459e+00_rb,1.9612e+00_rb,1.0622e+00_rb,3.4382e-02_rb/)
      kbo(:, 2,18, 9) = (/ &
     &5.3265e+00_rb,4.0412e+00_rb,2.7835e+00_rb,1.5026e+00_rb,4.2209e-02_rb/)
      kbo(:, 3,18, 9) = (/ &
     &7.2937e+00_rb,5.5341e+00_rb,3.8093e+00_rb,2.0485e+00_rb,5.1165e-02_rb/)
      kbo(:, 4,18, 9) = (/ &
     &9.7044e+00_rb,7.3625e+00_rb,5.0637e+00_rb,2.7136e+00_rb,6.0247e-02_rb/)
      kbo(:, 5,18, 9) = (/ &
     &1.2597e+01_rb,9.5561e+00_rb,6.5653e+00_rb,3.5052e+00_rb,7.0721e-02_rb/)
      kbo(:, 1,19, 9) = (/ &
     &3.1949e+00_rb,2.4358e+00_rb,1.6862e+00_rb,9.1523e-01_rb,2.9755e-02_rb/)
      kbo(:, 2,19, 9) = (/ &
     &4.5382e+00_rb,3.4601e+00_rb,2.3940e+00_rb,1.2945e+00_rb,3.7026e-02_rb/)
      kbo(:, 3,19, 9) = (/ &
     &6.2344e+00_rb,4.7532e+00_rb,3.2851e+00_rb,1.7697e+00_rb,4.5045e-02_rb/)
      kbo(:, 4,19, 9) = (/ &
     &8.3349e+00_rb,6.3521e+00_rb,4.3827e+00_rb,2.3518e+00_rb,5.3532e-02_rb/)
      kbo(:, 5,19, 9) = (/ &
     &1.0858e+01_rb,8.2689e+00_rb,5.6978e+00_rb,3.0490e+00_rb,6.2338e-02_rb/)
      kbo(:, 1,20, 9) = (/ &
     &2.7983e+00_rb,2.1430e+00_rb,1.4907e+00_rb,8.0787e-01_rb,2.6638e-02_rb/)
      kbo(:, 2,20, 9) = (/ &
     &3.9801e+00_rb,3.0473e+00_rb,2.1167e+00_rb,1.1423e+00_rb,3.3344e-02_rb/)
      kbo(:, 3,20, 9) = (/ &
     &5.4854e+00_rb,4.1978e+00_rb,2.9095e+00_rb,1.5650e+00_rb,4.0067e-02_rb/)
      kbo(:, 4,20, 9) = (/ &
     &7.3522e+00_rb,5.6240e+00_rb,3.8910e+00_rb,2.0868e+00_rb,4.7603e-02_rb/)
      kbo(:, 5,20, 9) = (/ &
     &9.6006e+00_rb,7.3375e+00_rb,5.0662e+00_rb,2.7087e+00_rb,5.5504e-02_rb/)
      kbo(:, 1,21, 9) = (/ &
     &2.4723e+00_rb,1.9017e+00_rb,1.3269e+00_rb,7.1717e-01_rb,2.3842e-02_rb/)
      kbo(:, 2,21, 9) = (/ &
     &3.5234e+00_rb,2.7089e+00_rb,1.8854e+00_rb,1.0156e+00_rb,2.9423e-02_rb/)
      kbo(:, 3,21, 9) = (/ &
     &4.8624e+00_rb,3.7359e+00_rb,2.5948e+00_rb,1.3934e+00_rb,3.5538e-02_rb/)
      kbo(:, 4,21, 9) = (/ &
     &6.5245e+00_rb,5.0095e+00_rb,3.4715e+00_rb,1.8586e+00_rb,4.1992e-02_rb/)
      kbo(:, 5,21, 9) = (/ &
     &8.5269e+00_rb,6.5419e+00_rb,4.5264e+00_rb,2.4180e+00_rb,4.9380e-02_rb/)
      kbo(:, 1,22, 9) = (/ &
     &2.2453e+00_rb,1.7353e+00_rb,1.2110e+00_rb,6.5234e-01_rb,2.1295e-02_rb/)
      kbo(:, 2,22, 9) = (/ &
     &3.1933e+00_rb,2.4664e+00_rb,1.7176e+00_rb,9.2252e-01_rb,2.6174e-02_rb/)
      kbo(:, 3,22, 9) = (/ &
     &4.4012e+00_rb,3.3958e+00_rb,2.3595e+00_rb,1.2638e+00_rb,3.1657e-02_rb/)
      kbo(:, 4,22, 9) = (/ &
     &5.8834e+00_rb,4.5350e+00_rb,3.1467e+00_rb,1.6824e+00_rb,3.7644e-02_rb/)
      kbo(:, 5,22, 9) = (/ &
     &7.6501e+00_rb,5.8912e+00_rb,4.0828e+00_rb,2.1806e+00_rb,4.4263e-02_rb/)
      kbo(:, 1,23, 9) = (/ &
     &2.0571e+00_rb,1.5967e+00_rb,1.1115e+00_rb,5.9605e-01_rb,1.8827e-02_rb/)
      kbo(:, 2,23, 9) = (/ &
     &2.9057e+00_rb,2.2535e+00_rb,1.5681e+00_rb,8.3999e-01_rb,2.3337e-02_rb/)
      kbo(:, 3,23, 9) = (/ &
     &3.9800e+00_rb,3.0833e+00_rb,2.1434e+00_rb,1.1464e+00_rb,2.8190e-02_rb/)
      kbo(:, 4,23, 9) = (/ &
     &5.2885e+00_rb,4.0922e+00_rb,2.8421e+00_rb,1.5182e+00_rb,3.3828e-02_rb/)
      kbo(:, 5,23, 9) = (/ &
     &6.8471e+00_rb,5.2918e+00_rb,3.6724e+00_rb,1.9594e+00_rb,3.9956e-02_rb/)
      kbo(:, 1,24, 9) = (/ &
     &1.9070e+00_rb,1.4857e+00_rb,1.0329e+00_rb,5.5181e-01_rb,1.6663e-02_rb/)
      kbo(:, 2,24, 9) = (/ &
     &2.6677e+00_rb,2.0773e+00_rb,1.4448e+00_rb,7.7214e-01_rb,2.0662e-02_rb/)
      kbo(:, 3,24, 9) = (/ &
     &3.6252e+00_rb,2.8187e+00_rb,1.9595e+00_rb,1.0463e+00_rb,2.5179e-02_rb/)
      kbo(:, 4,24, 9) = (/ &
     &4.7921e+00_rb,3.7213e+00_rb,2.5847e+00_rb,1.3784e+00_rb,3.0300e-02_rb/)
      kbo(:, 5,24, 9) = (/ &
     &6.1875e+00_rb,4.7981e+00_rb,3.3299e+00_rb,1.7738e+00_rb,3.5858e-02_rb/)
      kbo(:, 1,25, 9) = (/ &
     &1.7679e+00_rb,1.3823e+00_rb,9.6005e-01_rb,5.1215e-01_rb,1.4756e-02_rb/)
      kbo(:, 2,25, 9) = (/ &
     &2.4506e+00_rb,1.9149e+00_rb,1.3310e+00_rb,7.1009e-01_rb,1.8413e-02_rb/)
      kbo(:, 3,25, 9) = (/ &
     &3.3092e+00_rb,2.5821e+00_rb,1.7930e+00_rb,9.5566e-01_rb,2.2613e-02_rb/)
      kbo(:, 4,25, 9) = (/ &
     &4.3581e+00_rb,3.3956e+00_rb,2.3561e+00_rb,1.2539e+00_rb,2.7407e-02_rb/)
      kbo(:, 5,25, 9) = (/ &
     &5.6148e+00_rb,4.3684e+00_rb,3.0296e+00_rb,1.6108e+00_rb,3.2730e-02_rb/)
      kbo(:, 1,26, 9) = (/ &
     &1.6550e+00_rb,1.2970e+00_rb,8.9910e-01_rb,4.7856e-01_rb,1.3235e-02_rb/)
      kbo(:, 2,26, 9) = (/ &
     &2.2724e+00_rb,1.7804e+00_rb,1.2348e+00_rb,6.5702e-01_rb,1.6632e-02_rb/)
      kbo(:, 3,26, 9) = (/ &
     &3.0501e+00_rb,2.3862e+00_rb,1.6545e+00_rb,8.7945e-01_rb,2.0602e-02_rb/)
      kbo(:, 4,26, 9) = (/ &
     &3.9998e+00_rb,3.1253e+00_rb,2.1658e+00_rb,1.1505e+00_rb,2.5120e-02_rb/)
      kbo(:, 5,26, 9) = (/ &
     &5.1417e+00_rb,4.0125e+00_rb,2.7805e+00_rb,1.4756e+00_rb,3.0188e-02_rb/)
      kbo(:, 1,27, 9) = (/ &
     &1.5517e+00_rb,1.2181e+00_rb,8.4303e-01_rb,4.4762e-01_rb,1.1922e-02_rb/)
      kbo(:, 2,27, 9) = (/ &
     &2.1187e+00_rb,1.6625e+00_rb,1.1508e+00_rb,6.1078e-01_rb,1.5090e-02_rb/)
      kbo(:, 3,27, 9) = (/ &
     &2.8285e+00_rb,2.2174e+00_rb,1.5344e+00_rb,8.1390e-01_rb,1.8733e-02_rb/)
      kbo(:, 4,27, 9) = (/ &
     &3.6946e+00_rb,2.8938e+00_rb,2.0026e+00_rb,1.0611e+00_rb,2.2964e-02_rb/)
      kbo(:, 5,27, 9) = (/ &
     &4.7297e+00_rb,3.7026e+00_rb,2.5639e+00_rb,1.3585e+00_rb,2.7799e-02_rb/)
      kbo(:, 1,28, 9) = (/ &
     &1.4557e+00_rb,1.1450e+00_rb,7.9124e-01_rb,4.1922e-01_rb,1.0671e-02_rb/)
      kbo(:, 2,28, 9) = (/ &
     &1.9807e+00_rb,1.5572e+00_rb,1.0762e+00_rb,5.7024e-01_rb,1.3585e-02_rb/)
      kbo(:, 3,28, 9) = (/ &
     &2.6353e+00_rb,2.0694e+00_rb,1.4297e+00_rb,7.5652e-01_rb,1.7012e-02_rb/)
      kbo(:, 4,28, 9) = (/ &
     &3.4268e+00_rb,2.6900e+00_rb,1.8600e+00_rb,9.8357e-01_rb,2.1005e-02_rb/)
      kbo(:, 5,28, 9) = (/ &
     &4.3754e+00_rb,3.4337e+00_rb,2.3745e+00_rb,1.2550e+00_rb,2.5556e-02_rb/)
      kbo(:, 1,29, 9) = (/ &
     &1.3575e+00_rb,1.0707e+00_rb,7.3899e-01_rb,3.9163e-01_rb,9.5187e-03_rb/)
      kbo(:, 2,29, 9) = (/ &
     &1.8443e+00_rb,1.4534e+00_rb,1.0029e+00_rb,5.3089e-01_rb,1.2207e-02_rb/)
      kbo(:, 3,29, 9) = (/ &
     &2.4443e+00_rb,1.9245e+00_rb,1.3293e+00_rb,7.0249e-01_rb,1.5292e-02_rb/)
      kbo(:, 4,29, 9) = (/ &
     &3.1715e+00_rb,2.4958e+00_rb,1.7237e+00_rb,9.1036e-01_rb,1.8851e-02_rb/)
      kbo(:, 5,29, 9) = (/ &
     &4.0410e+00_rb,3.1790e+00_rb,2.1964e+00_rb,1.1590e+00_rb,2.3121e-02_rb/)
      kbo(:, 1,30, 9) = (/ &
     &1.2615e+00_rb,9.9705e-01_rb,6.8810e-01_rb,3.6451e-01_rb,8.4241e-03_rb/)
      kbo(:, 2,30, 9) = (/ &
     &1.7102e+00_rb,1.3510e+00_rb,9.3203e-01_rb,4.9262e-01_rb,1.0769e-02_rb/)
      kbo(:, 3,30, 9) = (/ &
     &2.2602e+00_rb,1.7845e+00_rb,1.2315e+00_rb,6.5024e-01_rb,1.3569e-02_rb/)
      kbo(:, 4,30, 9) = (/ &
     &2.9275e+00_rb,2.3103e+00_rb,1.5947e+00_rb,8.4085e-01_rb,1.6868e-02_rb/)
      kbo(:, 5,30, 9) = (/ &
     &3.7272e+00_rb,2.9401e+00_rb,2.0301e+00_rb,1.0701e+00_rb,2.0720e-02_rb/)
      kbo(:, 1,31, 9) = (/ &
     &1.1628e+00_rb,9.2130e-01_rb,6.3548e-01_rb,3.3629e-01_rb,7.3212e-03_rb/)
      kbo(:, 2,31, 9) = (/ &
     &1.5710e+00_rb,1.2442e+00_rb,8.5840e-01_rb,4.5341e-01_rb,9.4089e-03_rb/)
      kbo(:, 3,31, 9) = (/ &
     &2.0741e+00_rb,1.6418e+00_rb,1.1326e+00_rb,5.9743e-01_rb,1.1908e-02_rb/)
      kbo(:, 4,31, 9) = (/ &
     &2.6838e+00_rb,2.1236e+00_rb,1.4650e+00_rb,7.7246e-01_rb,1.4841e-02_rb/)
      kbo(:, 5,31, 9) = (/ &
     &3.4158e+00_rb,2.7036e+00_rb,1.8662e+00_rb,9.8360e-01_rb,1.8280e-02_rb/)
      kbo(:, 1,32, 9) = (/ &
     &1.0770e+00_rb,8.5530e-01_rb,5.8998e-01_rb,3.1205e-01_rb,6.3886e-03_rb/)
      kbo(:, 2,32, 9) = (/ &
     &1.4521e+00_rb,1.1529e+00_rb,7.9467e-01_rb,4.1942e-01_rb,8.2479e-03_rb/)
      kbo(:, 3,32, 9) = (/ &
     &1.9141e+00_rb,1.5187e+00_rb,1.0473e+00_rb,5.5229e-01_rb,1.0468e-02_rb/)
      kbo(:, 4,32, 9) = (/ &
     &2.4773e+00_rb,1.9665e+00_rb,1.3565e+00_rb,7.1494e-01_rb,1.3077e-02_rb/)
      kbo(:, 5,32, 9) = (/ &
     &3.1536e+00_rb,2.5044e+00_rb,1.7279e+00_rb,9.1011e-01_rb,1.6196e-02_rb/)
      kbo(:, 1,33, 9) = (/ &
     &1.0004e+00_rb,7.9686e-01_rb,5.4943e-01_rb,2.9036e-01_rb,5.6169e-03_rb/)
      kbo(:, 2,33, 9) = (/ &
     &1.3472e+00_rb,1.0720e+00_rb,7.3887e-01_rb,3.9002e-01_rb,7.2464e-03_rb/)
      kbo(:, 3,33, 9) = (/ &
     &1.7767e+00_rb,1.4145e+00_rb,9.7493e-01_rb,5.1400e-01_rb,9.1682e-03_rb/)
      kbo(:, 4,33, 9) = (/ &
     &2.3006e+00_rb,1.8321e+00_rb,1.2634e+00_rb,6.6568e-01_rb,1.1493e-02_rb/)
      kbo(:, 5,33, 9) = (/ &
     &2.9306e+00_rb,2.3339e+00_rb,1.6102e+00_rb,8.4827e-01_rb,1.4147e-02_rb/)
      kbo(:, 1,34, 9) = (/ &
     &9.3256e-01_rb,7.4416e-01_rb,5.1264e-01_rb,2.7079e-01_rb,4.9465e-03_rb/)
      kbo(:, 2,34, 9) = (/ &
     &1.2561e+00_rb,1.0020e+00_rb,6.9034e-01_rb,3.6434e-01_rb,6.3960e-03_rb/)
      kbo(:, 3,34, 9) = (/ &
     &1.6587e+00_rb,1.3239e+00_rb,9.1206e-01_rb,4.8092e-01_rb,8.1576e-03_rb/)
      kbo(:, 4,34, 9) = (/ &
     &2.1501e+00_rb,1.7167e+00_rb,1.1837e+00_rb,6.2351e-01_rb,1.0194e-02_rb/)
      kbo(:, 5,34, 9) = (/ &
     &2.7413e+00_rb,2.1905e+00_rb,1.5115e+00_rb,7.9635e-01_rb,1.2542e-02_rb/)
      kbo(:, 1,35, 9) = (/ &
     &8.4840e-01_rb,6.7846e-01_rb,4.6725e-01_rb,2.4678e-01_rb,4.2581e-03_rb/)
      kbo(:, 2,35, 9) = (/ &
     &1.1466e+00_rb,9.1713e-01_rb,6.3179e-01_rb,3.3319e-01_rb,5.5317e-03_rb/)
      kbo(:, 3,35, 9) = (/ &
     &1.5185e+00_rb,1.2151e+00_rb,8.3727e-01_rb,4.4133e-01_rb,7.0625e-03_rb/)
      kbo(:, 4,35, 9) = (/ &
     &1.9737e+00_rb,1.5810e+00_rb,1.0899e+00_rb,5.7421e-01_rb,8.8571e-03_rb/)
      kbo(:, 5,35, 9) = (/ &
     &2.5262e+00_rb,2.0251e+00_rb,1.3975e+00_rb,7.3625e-01_rb,1.0897e-02_rb/)
      kbo(:, 1,36, 9) = (/ &
     &7.5165e-01_rb,6.0226e-01_rb,4.1465e-01_rb,2.1892e-01_rb,3.5512e-03_rb/)
      kbo(:, 2,36, 9) = (/ &
     &1.0210e+00_rb,8.1866e-01_rb,5.6371e-01_rb,2.9729e-01_rb,4.6496e-03_rb/)
      kbo(:, 3,36, 9) = (/ &
     &1.3584e+00_rb,1.0902e+00_rb,7.5109e-01_rb,3.9583e-01_rb,5.9580e-03_rb/)
      kbo(:, 4,36, 9) = (/ &
     &1.7752e+00_rb,1.4261e+00_rb,9.8305e-01_rb,5.1817e-01_rb,7.4947e-03_rb/)
      kbo(:, 5,36, 9) = (/ &
     &2.2842e+00_rb,1.8370e+00_rb,1.2679e+00_rb,6.6806e-01_rb,9.2616e-03_rb/)
      kbo(:, 1,37, 9) = (/ &
     &6.4071e-01_rb,5.1457e-01_rb,3.5423e-01_rb,1.8691e-01_rb,2.9273e-03_rb/)
      kbo(:, 2,37, 9) = (/ &
     &8.7819e-01_rb,7.0566e-01_rb,4.8576e-01_rb,2.5612e-01_rb,3.8482e-03_rb/)
      kbo(:, 3,37, 9) = (/ &
     &1.1781e+00_rb,9.4755e-01_rb,6.5255e-01_rb,3.4395e-01_rb,4.9507e-03_rb/)
      kbo(:, 4,37, 9) = (/ &
     &1.5513e+00_rb,1.2498e+00_rb,8.6132e-01_rb,4.5404e-01_rb,6.2679e-03_rb/)
      kbo(:, 5,37, 9) = (/ &
     &2.0117e+00_rb,1.6227e+00_rb,1.1201e+00_rb,5.9042e-01_rb,7.7688e-03_rb/)
      kbo(:, 1,38, 9) = (/ &
     &5.4507e-01_rb,4.3885e-01_rb,3.0197e-01_rb,1.5934e-01_rb,2.3915e-03_rb/)
      kbo(:, 2,38, 9) = (/ &
     &7.5435e-01_rb,6.0772e-01_rb,4.1826e-01_rb,2.2053e-01_rb,3.1638e-03_rb/)
      kbo(:, 3,38, 9) = (/ &
     &1.0211e+00_rb,8.2323e-01_rb,5.6695e-01_rb,2.9883e-01_rb,4.0903e-03_rb/)
      kbo(:, 4,38, 9) = (/ &
     &1.3557e+00_rb,1.0950e+00_rb,7.5476e-01_rb,3.9799e-01_rb,5.2077e-03_rb/)
      kbo(:, 5,38, 9) = (/ &
     &1.7730e+00_rb,1.4345e+00_rb,9.9017e-01_rb,5.2203e-01_rb,6.5004e-03_rb/)
      kbo(:, 1,39, 9) = (/ &
     &4.6353e-01_rb,3.7406e-01_rb,2.5754e-01_rb,1.3588e-01_rb,1.9343e-03_rb/)
      kbo(:, 2,39, 9) = (/ &
     &6.4812e-01_rb,5.2360e-01_rb,3.6039e-01_rb,1.9002e-01_rb,2.5847e-03_rb/)
      kbo(:, 3,39, 9) = (/ &
     &8.8607e-01_rb,7.1665e-01_rb,4.9365e-01_rb,2.6013e-01_rb,3.3630e-03_rb/)
      kbo(:, 4,39, 9) = (/ &
     &1.1877e+00_rb,9.6199e-01_rb,6.6338e-01_rb,3.4986e-01_rb,4.2963e-03_rb/)
      kbo(:, 5,39, 9) = (/ &
     &1.5671e+00_rb,1.2712e+00_rb,8.7761e-01_rb,4.6286e-01_rb,5.3839e-03_rb/)
      kbo(:, 1,40, 9) = (/ &
     &3.8448e-01_rb,3.1107e-01_rb,2.1410e-01_rb,1.1297e-01_rb,1.5568e-03_rb/)
      kbo(:, 2,40, 9) = (/ &
     &5.4501e-01_rb,4.4126e-01_rb,3.0380e-01_rb,1.6009e-01_rb,2.1042e-03_rb/)
      kbo(:, 3,40, 9) = (/ &
     &7.5403e-01_rb,6.1132e-01_rb,4.2098e-01_rb,2.2180e-01_rb,2.7494e-03_rb/)
      kbo(:, 4,40, 9) = (/ &
     &1.0220e+00_rb,8.2995e-01_rb,5.7215e-01_rb,3.0165e-01_rb,3.5443e-03_rb/)
      kbo(:, 5,40, 9) = (/ &
     &1.3627e+00_rb,1.1080e+00_rb,7.6492e-01_rb,4.0348e-01_rb,4.4643e-03_rb/)
      kbo(:, 1,41, 9) = (/ &
     &3.1745e-01_rb,2.5733e-01_rb,1.7713e-01_rb,9.3436e-02_rb,1.2472e-03_rb/)
      kbo(:, 2,41, 9) = (/ &
     &4.5638e-01_rb,3.7042e-01_rb,2.5491e-01_rb,1.3436e-01_rb,1.6993e-03_rb/)
      kbo(:, 3,41, 9) = (/ &
     &6.3982e-01_rb,5.1972e-01_rb,3.5784e-01_rb,1.8851e-01_rb,2.2393e-03_rb/)
      kbo(:, 4,41, 9) = (/ &
     &8.7763e-01_rb,7.1397e-01_rb,4.9232e-01_rb,2.5948e-01_rb,2.9086e-03_rb/)
      kbo(:, 5,41, 9) = (/ &
     &1.1829e+00_rb,9.6420e-01_rb,6.6569e-01_rb,3.5114e-01_rb,3.6948e-03_rb/)
      kbo(:, 1,42, 9) = (/ &
     &2.6148e-01_rb,2.1244e-01_rb,1.4620e-01_rb,7.7106e-02_rb,9.9249e-04_rb/)
      kbo(:, 2,42, 9) = (/ &
     &3.8152e-01_rb,3.1036e-01_rb,2.1367e-01_rb,1.1258e-01_rb,1.3669e-03_rb/)
      kbo(:, 3,42, 9) = (/ &
     &5.4219e-01_rb,4.4152e-01_rb,3.0416e-01_rb,1.6017e-01_rb,1.8223e-03_rb/)
      kbo(:, 4,42, 9) = (/ &
     &7.5310e-01_rb,6.1412e-01_rb,4.2345e-01_rb,2.2321e-01_rb,2.3766e-03_rb/)
      kbo(:, 5,42, 9) = (/ &
     &1.0272e+00_rb,8.3945e-01_rb,5.7960e-01_rb,3.0577e-01_rb,3.0463e-03_rb/)
      kbo(:, 1,43, 9) = (/ &
     &2.1184e-01_rb,1.7233e-01_rb,1.1866e-01_rb,6.2577e-02_rb,7.7589e-04_rb/)
      kbo(:, 2,43, 9) = (/ &
     &3.1451e-01_rb,2.5652e-01_rb,1.7660e-01_rb,9.2987e-02_rb,1.0831e-03_rb/)
      kbo(:, 3,43, 9) = (/ &
     &4.5438e-01_rb,3.7074e-01_rb,2.5543e-01_rb,1.3448e-01_rb,1.4611e-03_rb/)
      kbo(:, 4,43, 9) = (/ &
     &6.4007e-01_rb,5.2328e-01_rb,3.6079e-01_rb,1.9007e-01_rb,1.9178e-03_rb/)
      kbo(:, 5,43, 9) = (/ &
     &8.8479e-01_rb,7.2477e-01_rb,5.0046e-01_rb,2.6398e-01_rb,2.4774e-03_rb/)
      kbo(:, 1,44, 9) = (/ &
     &1.6982e-01_rb,1.3846e-01_rb,9.5299e-02_rb,5.0248e-02_rb,5.9857e-04_rb/)
      kbo(:, 2,44, 9) = (/ &
     &2.5706e-01_rb,2.1005e-01_rb,1.4462e-01_rb,7.6126e-02_rb,8.4785e-04_rb/)
      kbo(:, 3,44, 9) = (/ &
     &3.7786e-01_rb,3.0904e-01_rb,2.1286e-01_rb,1.1204e-01_rb,1.1605e-03_rb/)
      kbo(:, 4,44, 9) = (/ &
     &5.4071e-01_rb,4.4303e-01_rb,3.0555e-01_rb,1.6090e-01_rb,1.5453e-03_rb/)
      kbo(:, 5,44, 9) = (/ &
     &7.5841e-01_rb,6.2258e-01_rb,4.2996e-01_rb,2.2675e-01_rb,1.9959e-03_rb/)
      kbo(:, 1,45, 9) = (/ &
     &1.3545e-01_rb,1.1062e-01_rb,7.6155e-02_rb,4.0151e-02_rb,4.5711e-04_rb/)
      kbo(:, 2,45, 9) = (/ &
     &2.0919e-01_rb,1.7120e-01_rb,1.1790e-01_rb,6.2073e-02_rb,6.5798e-04_rb/)
      kbo(:, 3,45, 9) = (/ &
     &3.1316e-01_rb,2.5663e-01_rb,1.7679e-01_rb,9.2995e-02_rb,9.1164e-04_rb/)
      kbo(:, 4,45, 9) = (/ &
     &4.5567e-01_rb,3.7416e-01_rb,2.5813e-01_rb,1.3590e-01_rb,1.2287e-03_rb/)
      kbo(:, 5,45, 9) = (/ &
     &6.4919e-01_rb,5.3401e-01_rb,3.6882e-01_rb,1.9448e-01_rb,1.6029e-03_rb/)
      kbo(:, 1,46, 9) = (/ &
     &1.0668e-01_rb,8.7165e-02_rb,6.0007e-02_rb,3.1628e-02_rb,3.4444e-04_rb/)
      kbo(:, 2,46, 9) = (/ &
     &1.6793e-01_rb,1.3770e-01_rb,9.4805e-02_rb,4.9929e-02_rb,5.0372e-04_rb/)
      kbo(:, 3,46, 9) = (/ &
     &2.5640e-01_rb,2.1057e-01_rb,1.4511e-01_rb,7.6324e-02_rb,7.0786e-04_rb/)
      kbo(:, 4,46, 9) = (/ &
     &3.8014e-01_rb,3.1276e-01_rb,2.1587e-01_rb,1.1362e-01_rb,9.6784e-04_rb/)
      kbo(:, 5,46, 9) = (/ &
     &5.5078e-01_rb,4.5406e-01_rb,3.1364e-01_rb,1.6534e-01_rb,1.2803e-03_rb/)
      kbo(:, 1,47, 9) = (/ &
     &8.2159e-02_rb,6.7194e-02_rb,4.6255e-02_rb,2.4369e-02_rb,2.5696e-04_rb/)
      kbo(:, 2,47, 9) = (/ &
     &1.3183e-01_rb,1.0829e-01_rb,7.4558e-02_rb,3.9260e-02_rb,3.8064e-04_rb/)
      kbo(:, 3,47, 9) = (/ &
     &2.0579e-01_rb,1.6932e-01_rb,1.1664e-01_rb,6.1365e-02_rb,5.4299e-04_rb/)
      kbo(:, 4,47, 9) = (/ &
     &3.1127e-01_rb,2.5673e-01_rb,1.7713e-01_rb,9.3234e-02_rb,7.5354e-04_rb/)
      kbo(:, 5,47, 9) = (/ &
     &4.5944e-01_rb,3.7947e-01_rb,2.6220e-01_rb,1.3818e-01_rb,1.0123e-03_rb/)
      kbo(:, 1,48, 9) = (/ &
     &6.2805e-02_rb,5.1372e-02_rb,3.5337e-02_rb,1.8627e-02_rb,1.8799e-04_rb/)
      kbo(:, 2,48, 9) = (/ &
     &1.0282e-01_rb,8.4440e-02_rb,5.8150e-02_rb,3.0624e-02_rb,2.8497e-04_rb/)
      kbo(:, 3,48, 9) = (/ &
     &1.6397e-01_rb,1.3516e-01_rb,9.3148e-02_rb,4.8994e-02_rb,4.1296e-04_rb/)
      kbo(:, 4,48, 9) = (/ &
     &2.5353e-01_rb,2.0948e-01_rb,1.4454e-01_rb,7.6030e-02_rb,5.7970e-04_rb/)
      kbo(:, 5,48, 9) = (/ &
     &3.8151e-01_rb,3.1578e-01_rb,2.1823e-01_rb,1.1499e-01_rb,7.9062e-04_rb/)
      kbo(:, 1,49, 9) = (/ &
     &4.7675e-02_rb,3.9033e-02_rb,2.6875e-02_rb,1.4167e-02_rb,1.3591e-04_rb/)
      kbo(:, 2,49, 9) = (/ &
     &7.9683e-02_rb,6.5540e-02_rb,4.5120e-02_rb,2.3756e-02_rb,2.0952e-04_rb/)
      kbo(:, 3,49, 9) = (/ &
     &1.2966e-01_rb,1.0708e-01_rb,7.3804e-02_rb,3.8837e-02_rb,3.0991e-04_rb/)
      kbo(:, 4,49, 9) = (/ &
     &2.0506e-01_rb,1.6980e-01_rb,1.1714e-01_rb,6.1628e-02_rb,4.4095e-04_rb/)
      kbo(:, 5,49, 9) = (/ &
     &3.1515e-01_rb,2.6147e-01_rb,1.8074e-01_rb,9.5226e-02_rb,6.0820e-04_rb/)
      kbo(:, 1,50, 9) = (/ &
     &3.6548e-02_rb,2.9943e-02_rb,2.0610e-02_rb,1.0865e-02_rb,9.9917e-05_rb/)
      kbo(:, 2,50, 9) = (/ &
     &6.2279e-02_rb,5.1209e-02_rb,3.5254e-02_rb,1.8566e-02_rb,1.5653e-04_rb/)
      kbo(:, 3,50, 9) = (/ &
     &1.0318e-01_rb,8.5332e-02_rb,5.8808e-02_rb,3.0962e-02_rb,2.3703e-04_rb/)
      kbo(:, 4,50, 9) = (/ &
     &1.6688e-01_rb,1.3839e-01_rb,9.5487e-02_rb,5.0236e-02_rb,3.4171e-04_rb/)
      kbo(:, 5,50, 9) = (/ &
     &2.6185e-01_rb,2.1767e-01_rb,1.5045e-01_rb,7.9263e-02_rb,4.7617e-04_rb/)
      kbo(:, 1,51, 9) = (/ &
     &2.7952e-02_rb,2.2969e-02_rb,1.5811e-02_rb,8.3417e-03_rb,7.4671e-05_rb/)
      kbo(:, 2,51, 9) = (/ &
     &4.8817e-02_rb,4.0171e-02_rb,2.7641e-02_rb,1.4555e-02_rb,1.1814e-04_rb/)
      kbo(:, 3,51, 9) = (/ &
     &8.2508e-02_rb,6.8219e-02_rb,4.6983e-02_rb,2.4707e-02_rb,1.8096e-04_rb/)
      kbo(:, 4,51, 9) = (/ &
     &1.3588e-01_rb,1.1285e-01_rb,7.7859e-02_rb,4.0970e-02_rb,2.6754e-04_rb/)
      kbo(:, 5,51, 9) = (/ &
     &2.1776e-01_rb,1.8131e-01_rb,1.2532e-01_rb,6.5999e-02_rb,3.7856e-04_rb/)
      kbo(:, 1,52, 9) = (/ &
     &2.1199e-02_rb,1.7459e-02_rb,1.2025e-02_rb,6.3452e-03_rb,5.5107e-05_rb/)
      kbo(:, 2,52, 9) = (/ &
     &3.8088e-02_rb,3.1354e-02_rb,2.1571e-02_rb,1.1353e-02_rb,8.8069e-05_rb/)
      kbo(:, 3,52, 9) = (/ &
     &6.5619e-02_rb,5.4321e-02_rb,3.7416e-02_rb,1.9692e-02_rb,1.3680e-04_rb/)
      kbo(:, 4,52, 9) = (/ &
     &1.1018e-01_rb,9.1593e-02_rb,6.3191e-02_rb,3.3254e-02_rb,2.0641e-04_rb/)
      kbo(:, 5,52, 9) = (/ &
     &1.8039e-01_rb,1.5044e-01_rb,1.0398e-01_rb,5.4762e-02_rb,2.9766e-04_rb/)
      kbo(:, 1,53, 9) = (/ &
     &1.5917e-02_rb,1.3124e-02_rb,9.0469e-03_rb,4.7728e-03_rb,4.0227e-05_rb/)
      kbo(:, 2,53, 9) = (/ &
     &2.9453e-02_rb,2.4295e-02_rb,1.6728e-02_rb,8.8064e-03_rb,6.5428e-05_rb/)
      kbo(:, 3,53, 9) = (/ &
     &5.2020e-02_rb,4.3058e-02_rb,2.9653e-02_rb,1.5599e-02_rb,1.0270e-04_rb/)
      kbo(:, 4,53, 9) = (/ &
     &8.9031e-02_rb,7.3955e-02_rb,5.0990e-02_rb,2.6843e-02_rb,1.5632e-04_rb/)
      kbo(:, 5,53, 9) = (/ &
     &1.4878e-01_rb,1.2427e-01_rb,8.5889e-02_rb,4.5242e-02_rb,2.3095e-04_rb/)
      kbo(:, 1,54, 9) = (/ &
     &1.2118e-02_rb,1.0014e-02_rb,6.9093e-03_rb,3.6479e-03_rb,3.0413e-05_rb/)
      kbo(:, 2,54, 9) = (/ &
     &2.3157e-02_rb,1.9122e-02_rb,1.3172e-02_rb,6.9364e-03_rb,5.0720e-05_rb/)
      kbo(:, 3,54, 9) = (/ &
     &4.1929e-02_rb,3.4711e-02_rb,2.3913e-02_rb,1.2578e-02_rb,8.0499e-05_rb/)
      kbo(:, 4,54, 9) = (/ &
     &7.3182e-02_rb,6.0821e-02_rb,4.1911e-02_rb,2.2054e-02_rb,1.2452e-04_rb/)
      kbo(:, 5,54, 9) = (/ &
     &1.2434e-01_rb,1.0391e-01_rb,7.1807e-02_rb,3.7810e-02_rb,1.8706e-04_rb/)
      kbo(:, 1,55, 9) = (/ &
     &9.2663e-03_rb,7.6785e-03_rb,5.3030e-03_rb,2.7962e-03_rb,2.3331e-05_rb/)
      kbo(:, 2,55, 9) = (/ &
     &1.8303e-02_rb,1.5169e-02_rb,1.0453e-02_rb,5.5039e-03_rb,4.0062e-05_rb/)
      kbo(:, 3,55, 9) = (/ &
     &3.4131e-02_rb,2.8258e-02_rb,1.9454e-02_rb,1.0230e-02_rb,6.4887e-05_rb/)
      kbo(:, 4,55, 9) = (/ &
     &6.0672e-02_rb,5.0398e-02_rb,3.4733e-02_rb,1.8270e-02_rb,1.0169e-04_rb/)
      kbo(:, 5,55, 9) = (/ &
     &1.0468e-01_rb,8.7488e-02_rb,6.0450e-02_rb,3.1810e-02_rb,1.5537e-04_rb/)
      kbo(:, 1,56, 9) = (/ &
     &7.0383e-03_rb,5.8452e-03_rb,4.0381e-03_rb,2.1288e-03_rb,1.7811e-05_rb/)
      kbo(:, 2,56, 9) = (/ &
     &1.4408e-02_rb,1.1966e-02_rb,8.2523e-03_rb,4.3448e-03_rb,3.1488e-05_rb/)
      kbo(:, 3,56, 9) = (/ &
     &2.7695e-02_rb,2.2953e-02_rb,1.5811e-02_rb,8.3080e-03_rb,5.2239e-05_rb/)
      kbo(:, 4,56, 9) = (/ &
     &5.0327e-02_rb,4.1815e-02_rb,2.8827e-02_rb,1.5149e-02_rb,8.2952e-05_rb/)
      kbo(:, 5,56, 9) = (/ &
     &8.8477e-02_rb,7.3786e-02_rb,5.0905e-02_rb,2.6783e-02_rb,1.2900e-04_rb/)
      kbo(:, 1,57, 9) = (/ &
     &5.2984e-03_rb,4.4048e-03_rb,3.0442e-03_rb,1.6062e-03_rb,1.3485e-05_rb/)
      kbo(:, 2,57, 9) = (/ &
     &1.1277e-02_rb,9.3839e-03_rb,6.4745e-03_rb,3.4126e-03_rb,2.4664e-05_rb/)
      kbo(:, 3,57, 9) = (/ &
     &2.2383e-02_rb,1.8598e-02_rb,1.2810e-02_rb,6.7278e-03_rb,4.2073e-05_rb/)
      kbo(:, 4,57, 9) = (/ &
     &4.1793e-02_rb,3.4722e-02_rb,2.3901e-02_rb,1.2561e-02_rb,6.8099e-05_rb/)
      kbo(:, 5,57, 9) = (/ &
     &7.4879e-02_rb,6.2449e-02_rb,4.3075e-02_rb,2.2643e-02_rb,1.0701e-04_rb/)
      kbo(:, 1,58, 9) = (/ &
     &1.9422e-03_rb,1.8562e-03_rb,1.5070e-03_rb,9.6178e-04_rb,1.0242e-05_rb/)
      kbo(:, 2,58, 9) = (/ &
     &4.2963e-03_rb,4.1126e-03_rb,3.3378e-03_rb,2.1278e-03_rb,1.9475e-05_rb/)
      kbo(:, 3,58, 9) = (/ &
     &8.8321e-03_rb,8.4312e-03_rb,6.8267e-03_rb,4.3437e-03_rb,3.4347e-05_rb/)
      kbo(:, 4,58, 9) = (/ &
     &1.7006e-02_rb,1.6220e-02_rb,1.3112e-02_rb,8.3245e-03_rb,5.6453e-05_rb/)
      kbo(:, 5,58, 9) = (/ &
     &3.1001e-02_rb,2.9664e-02_rb,2.4013e-02_rb,1.5274e-02_rb,9.0315e-05_rb/)
      kbo(:, 1,59, 9) = (/ &
     &1.6680e-03_rb,1.5995e-03_rb,1.3049e-03_rb,8.3836e-04_rb,9.2179e-06_rb/)
      kbo(:, 2,59, 9) = (/ &
     &3.7712e-03_rb,3.6258e-03_rb,2.9561e-03_rb,1.8979e-03_rb,1.8017e-05_rb/)
      kbo(:, 3,59, 9) = (/ &
     &7.9144e-03_rb,7.6082e-03_rb,6.1886e-03_rb,3.9647e-03_rb,3.2435e-05_rb/)
      kbo(:, 4,59, 9) = (/ &
     &1.5562e-02_rb,1.4917e-02_rb,1.2127e-02_rb,7.7579e-03_rb,5.4862e-05_rb/)
      kbo(:, 5,59, 9) = (/ &
     &2.8789e-02_rb,2.7607e-02_rb,2.2446e-02_rb,1.4356e-02_rb,8.9155e-05_rb/)
      kbo(:, 1,13,10) = (/ &
     &3.5965e+01_rb,2.6975e+01_rb,1.7984e+01_rb,9.0232e+00_rb,2.4705e-01_rb/)
      kbo(:, 2,13,10) = (/ &
     &5.0800e+01_rb,3.8101e+01_rb,2.5403e+01_rb,1.2739e+01_rb,2.8941e-01_rb/)
      kbo(:, 3,13,10) = (/ &
     &6.8409e+01_rb,5.1309e+01_rb,3.4208e+01_rb,1.7156e+01_rb,3.3771e-01_rb/)
      kbo(:, 4,13,10) = (/ &
     &8.8981e+01_rb,6.6739e+01_rb,4.4497e+01_rb,2.2337e+01_rb,3.9418e-01_rb/)
      kbo(:, 5,13,10) = (/ &
     &1.1252e+02_rb,8.4389e+01_rb,5.6260e+01_rb,2.8284e+01_rb,4.6348e-01_rb/)
      kbo(:, 1,14,10) = (/ &
     &3.4773e+01_rb,2.6079e+01_rb,1.7386e+01_rb,8.7850e+00_rb,2.3514e-01_rb/)
      kbo(:, 2,14,10) = (/ &
     &4.8754e+01_rb,3.6565e+01_rb,2.4376e+01_rb,1.2316e+01_rb,2.9035e-01_rb/)
      kbo(:, 3,14,10) = (/ &
     &6.5535e+01_rb,4.9151e+01_rb,3.2769e+01_rb,1.6566e+01_rb,3.5401e-01_rb/)
      kbo(:, 4,14,10) = (/ &
     &8.5152e+01_rb,6.3871e+01_rb,4.2583e+01_rb,2.1543e+01_rb,4.0952e-01_rb/)
      kbo(:, 5,14,10) = (/ &
     &1.0714e+02_rb,8.0354e+01_rb,5.3570e+01_rb,2.7127e+01_rb,4.6861e-01_rb/)
      kbo(:, 1,15,10) = (/ &
     &3.1708e+01_rb,2.3780e+01_rb,1.5854e+01_rb,8.0780e+00_rb,2.2725e-01_rb/)
      kbo(:, 2,15,10) = (/ &
     &4.4299e+01_rb,3.3227e+01_rb,2.2149e+01_rb,1.1301e+01_rb,2.7237e-01_rb/)
      kbo(:, 3,15,10) = (/ &
     &5.9451e+01_rb,4.4587e+01_rb,2.9727e+01_rb,1.5175e+01_rb,3.1480e-01_rb/)
      kbo(:, 4,15,10) = (/ &
     &7.6914e+01_rb,5.7688e+01_rb,3.8460e+01_rb,1.9633e+01_rb,3.5895e-01_rb/)
      kbo(:, 5,15,10) = (/ &
     &9.6187e+01_rb,7.2137e+01_rb,4.8097e+01_rb,2.4537e+01_rb,4.0136e-01_rb/)
      kbo(:, 1,16,10) = (/ &
     &2.8000e+01_rb,2.1001e+01_rb,1.4008e+01_rb,7.1879e+00_rb,1.8684e-01_rb/)
      kbo(:, 2,16,10) = (/ &
     &3.8983e+01_rb,2.9238e+01_rb,1.9500e+01_rb,1.0007e+01_rb,2.3111e-01_rb/)
      kbo(:, 3,16,10) = (/ &
     &5.2090e+01_rb,3.9071e+01_rb,2.6063e+01_rb,1.3380e+01_rb,2.7168e-01_rb/)
      kbo(:, 4,16,10) = (/ &
     &6.6962e+01_rb,5.0223e+01_rb,3.3517e+01_rb,1.7217e+01_rb,3.1091e-01_rb/)
      kbo(:, 5,16,10) = (/ &
     &8.3570e+01_rb,6.2687e+01_rb,4.1858e+01_rb,2.1531e+01_rb,3.6276e-01_rb/)
      kbo(:, 1,17,10) = (/ &
     &2.4264e+01_rb,1.8198e+01_rb,1.2195e+01_rb,6.2664e+00_rb,1.7021e-01_rb/)
      kbo(:, 2,17,10) = (/ &
     &3.3645e+01_rb,2.5234e+01_rb,1.6913e+01_rb,8.7049e+00_rb,1.9751e-01_rb/)
      kbo(:, 3,17,10) = (/ &
     &4.4643e+01_rb,3.3486e+01_rb,2.2452e+01_rb,1.1586e+01_rb,2.2201e-01_rb/)
      kbo(:, 4,17,10) = (/ &
     &5.7207e+01_rb,4.2902e+01_rb,2.8791e+01_rb,1.4899e+01_rb,2.5554e-01_rb/)
      kbo(:, 5,17,10) = (/ &
     &7.1193e+01_rb,5.3402e+01_rb,3.5856e+01_rb,1.8608e+01_rb,2.9547e-01_rb/)
      kbo(:, 1,18,10) = (/ &
     &2.0477e+01_rb,1.5356e+01_rb,1.0367e+01_rb,5.3622e+00_rb,1.3534e-01_rb/)
      kbo(:, 2,18,10) = (/ &
     &2.8178e+01_rb,2.1132e+01_rb,1.4278e+01_rb,7.4149e+00_rb,1.5634e-01_rb/)
      kbo(:, 3,18,10) = (/ &
     &3.7210e+01_rb,2.7911e+01_rb,1.8862e+01_rb,9.8384e+00_rb,1.8020e-01_rb/)
      kbo(:, 4,18,10) = (/ &
     &4.7546e+01_rb,3.5659e+01_rb,2.4100e+01_rb,1.2600e+01_rb,2.1507e-01_rb/)
      kbo(:, 5,18,10) = (/ &
     &5.8973e+01_rb,4.4229e+01_rb,2.9880e+01_rb,1.5650e+01_rb,2.3467e-01_rb/)
      kbo(:, 1,19,10) = (/ &
     &1.7373e+01_rb,1.3040e+01_rb,8.8390e+00_rb,4.6588e+00_rb,1.1972e-01_rb/)
      kbo(:, 2,19,10) = (/ &
     &2.3793e+01_rb,1.7857e+01_rb,1.2109e+01_rb,6.3963e+00_rb,1.3774e-01_rb/)
      kbo(:, 3,19,10) = (/ &
     &3.1336e+01_rb,2.3520e+01_rb,1.5961e+01_rb,8.4390e+00_rb,1.6091e-01_rb/)
      kbo(:, 4,19,10) = (/ &
     &3.9866e+01_rb,2.9935e+01_rb,2.0331e+01_rb,1.0756e+01_rb,1.7955e-01_rb/)
      kbo(:, 5,19,10) = (/ &
     &4.9495e+01_rb,3.7175e+01_rb,2.5257e+01_rb,1.3339e+01_rb,2.0663e-01_rb/)
      kbo(:, 1,20,10) = (/ &
     &1.5037e+01_rb,1.1322e+01_rb,7.6844e+00_rb,4.1127e+00_rb,1.0485e-01_rb/)
      kbo(:, 2,20,10) = (/ &
     &2.0515e+01_rb,1.5450e+01_rb,1.0499e+01_rb,5.6182e+00_rb,1.2148e-01_rb/)
      kbo(:, 3,20,10) = (/ &
     &2.6886e+01_rb,2.0252e+01_rb,1.3792e+01_rb,7.3741e+00_rb,1.4561e-01_rb/)
      kbo(:, 4,20,10) = (/ &
     &3.4287e+01_rb,2.5832e+01_rb,1.7615e+01_rb,9.3860e+00_rb,1.6294e-01_rb/)
      kbo(:, 5,20,10) = (/ &
     &4.2841e+01_rb,3.2278e+01_rb,2.2023e+01_rb,1.1711e+01_rb,1.8624e-01_rb/)
      kbo(:, 1,21,10) = (/ &
     &1.3152e+01_rb,9.9438e+00_rb,6.7780e+00_rb,3.6471e+00_rb,9.1853e-02_rb/)
      kbo(:, 2,21,10) = (/ &
     &1.7866e+01_rb,1.3511e+01_rb,9.2298e+00_rb,4.9595e+00_rb,1.1141e-01_rb/)
      kbo(:, 3,21,10) = (/ &
     &2.3469e+01_rb,1.7747e+01_rb,1.2135e+01_rb,6.5086e+00_rb,1.2785e-01_rb/)
      kbo(:, 4,21,10) = (/ &
     &3.0120e+01_rb,2.2761e+01_rb,1.5567e+01_rb,8.3305e+00_rb,1.5176e-01_rb/)
      kbo(:, 5,21,10) = (/ &
     &3.7884e+01_rb,2.8609e+01_rb,1.9560e+01_rb,1.0448e+01_rb,1.7309e-01_rb/)
      kbo(:, 1,22,10) = (/ &
     &1.1757e+01_rb,8.9118e+00_rb,6.1382e+00_rb,3.3115e+00_rb,8.1057e-02_rb/)
      kbo(:, 2,22,10) = (/ &
     &1.5980e+01_rb,1.2109e+01_rb,8.3336e+00_rb,4.4832e+00_rb,9.8906e-02_rb/)
      kbo(:, 3,22,10) = (/ &
     &2.1108e+01_rb,1.5991e+01_rb,1.0992e+01_rb,5.8971e+00_rb,1.1586e-01_rb/)
      kbo(:, 4,22,10) = (/ &
     &2.7285e+01_rb,2.0659e+01_rb,1.4174e+01_rb,7.5863e+00_rb,1.3507e-01_rb/)
      kbo(:, 5,22,10) = (/ &
     &3.4676e+01_rb,2.6248e+01_rb,1.7966e+01_rb,9.5810e+00_rb,1.5851e-01_rb/)
      kbo(:, 1,23,10) = (/ &
     &1.0481e+01_rb,7.9685e+00_rb,5.5490e+00_rb,2.9996e+00_rb,7.4309e-02_rb/)
      kbo(:, 2,23,10) = (/ &
     &1.4420e+01_rb,1.0955e+01_rb,7.5878e+00_rb,4.0805e+00_rb,8.6253e-02_rb/)
      kbo(:, 3,23,10) = (/ &
     &1.9219e+01_rb,1.4600e+01_rb,1.0073e+01_rb,5.3977e+00_rb,1.0580e-01_rb/)
      kbo(:, 4,23,10) = (/ &
     &2.5108e+01_rb,1.9069e+01_rb,1.3115e+01_rb,7.0020e+00_rb,1.2265e-01_rb/)
      kbo(:, 5,23,10) = (/ &
     &3.2043e+01_rb,2.4337e+01_rb,1.6704e+01_rb,8.8942e+00_rb,1.4381e-01_rb/)
      kbo(:, 1,24,10) = (/ &
     &9.5276e+00_rb,7.2768e+00_rb,5.0826e+00_rb,2.7465e+00_rb,6.4655e-02_rb/)
      kbo(:, 2,24,10) = (/ &
     &1.3271e+01_rb,1.0121e+01_rb,7.0214e+00_rb,3.7657e+00_rb,7.9291e-02_rb/)
      kbo(:, 3,24,10) = (/ &
     &1.7863e+01_rb,1.3622e+01_rb,9.4201e+00_rb,5.0307e+00_rb,9.5461e-02_rb/)
      kbo(:, 4,24,10) = (/ &
     &2.3325e+01_rb,1.7783e+01_rb,1.2273e+01_rb,6.5420e+00_rb,1.1274e-01_rb/)
      kbo(:, 5,24,10) = (/ &
     &2.9564e+01_rb,2.2536e+01_rb,1.5537e+01_rb,8.2816e+00_rb,1.3418e-01_rb/)
      kbo(:, 1,25,10) = (/ &
     &8.8160e+00_rb,6.7673e+00_rb,4.7220e+00_rb,2.5379e+00_rb,5.8489e-02_rb/)
      kbo(:, 2,25,10) = (/ &
     &1.2357e+01_rb,9.4685e+00_rb,6.5735e+00_rb,3.5130e+00_rb,7.2064e-02_rb/)
      kbo(:, 3,25,10) = (/ &
     &1.6557e+01_rb,1.2678e+01_rb,8.7976e+00_rb,4.6917e+00_rb,8.6281e-02_rb/)
      kbo(:, 4,25,10) = (/ &
     &2.1444e+01_rb,1.6408e+01_rb,1.1376e+01_rb,6.0717e+00_rb,1.0416e-01_rb/)
      kbo(:, 5,25,10) = (/ &
     &2.7072e+01_rb,2.0703e+01_rb,1.4347e+01_rb,7.6551e+00_rb,1.2572e-01_rb/)
      kbo(:, 1,26,10) = (/ &
     &8.2437e+00_rb,6.3645e+00_rb,4.4442e+00_rb,2.3799e+00_rb,5.3046e-02_rb/)
      kbo(:, 2,26,10) = (/ &
     &1.1483e+01_rb,8.8437e+00_rb,6.1605e+00_rb,3.2883e+00_rb,6.5278e-02_rb/)
      kbo(:, 3,26,10) = (/ &
     &1.5261e+01_rb,1.1739e+01_rb,8.1695e+00_rb,4.3593e+00_rb,8.0191e-02_rb/)
      kbo(:, 4,26,10) = (/ &
     &1.9700e+01_rb,1.5135e+01_rb,1.0534e+01_rb,5.6197e+00_rb,9.7601e-02_rb/)
      kbo(:, 5,26,10) = (/ &
     &2.4857e+01_rb,1.9072e+01_rb,1.3248e+01_rb,7.0653e+00_rb,1.1704e-01_rb/)
      kbo(:, 1,27,10) = (/ &
     &7.7081e+00_rb,5.9885e+00_rb,4.1814e+00_rb,2.2306e+00_rb,4.7409e-02_rb/)
      kbo(:, 2,27,10) = (/ &
     &1.0576e+01_rb,8.1915e+00_rb,5.7182e+00_rb,3.0504e+00_rb,5.8512e-02_rb/)
      kbo(:, 3,27,10) = (/ &
     &1.4014e+01_rb,1.0837e+01_rb,7.5629e+00_rb,4.0315e+00_rb,7.3955e-02_rb/)
      kbo(:, 4,27,10) = (/ &
     &1.8113e+01_rb,1.3973e+01_rb,9.7390e+00_rb,5.1847e+00_rb,9.0794e-02_rb/)
      kbo(:, 5,27,10) = (/ &
     &2.2862e+01_rb,1.7593e+01_rb,1.2246e+01_rb,6.5228e+00_rb,1.0871e-01_rb/)
      kbo(:, 1,28,10) = (/ &
     &7.2063e+00_rb,5.6196e+00_rb,3.9203e+00_rb,2.0882e+00_rb,4.3129e-02_rb/)
      kbo(:, 2,28,10) = (/ &
     &9.7817e+00_rb,7.6085e+00_rb,5.3084e+00_rb,2.8196e+00_rb,5.4812e-02_rb/)
      kbo(:, 3,28,10) = (/ &
     &1.2904e+01_rb,1.0029e+01_rb,7.0025e+00_rb,3.7236e+00_rb,6.8395e-02_rb/)
      kbo(:, 4,28,10) = (/ &
     &1.6696e+01_rb,1.2935e+01_rb,9.0135e+00_rb,4.7943e+00_rb,8.2716e-02_rb/)
      kbo(:, 5,28,10) = (/ &
     &2.1096e+01_rb,1.6309e+01_rb,1.1368e+01_rb,6.0506e+00_rb,9.9935e-02_rb/)
      kbo(:, 1,29,10) = (/ &
     &6.7278e+00_rb,5.2636e+00_rb,3.6717e+00_rb,1.9437e+00_rb,3.9266e-02_rb/)
      kbo(:, 2,29,10) = (/ &
     &9.0515e+00_rb,7.0710e+00_rb,4.9355e+00_rb,2.6130e+00_rb,4.9834e-02_rb/)
      kbo(:, 3,29,10) = (/ &
     &1.1919e+01_rb,9.2978e+00_rb,6.4770e+00_rb,3.4412e+00_rb,6.2077e-02_rb/)
      kbo(:, 4,29,10) = (/ &
     &1.5380e+01_rb,1.1978e+01_rb,8.3499e+00_rb,4.4362e+00_rb,7.6798e-02_rb/)
      kbo(:, 5,29,10) = (/ &
     &1.9420e+01_rb,1.5097e+01_rb,1.0536e+01_rb,5.6045e+00_rb,9.1669e-02_rb/)
      kbo(:, 1,30,10) = (/ &
     &6.2882e+00_rb,4.9355e+00_rb,3.4323e+00_rb,1.8115e+00_rb,3.5713e-02_rb/)
      kbo(:, 2,30,10) = (/ &
     &8.4169e+00_rb,6.5906e+00_rb,4.5841e+00_rb,2.4262e+00_rb,4.5752e-02_rb/)
      kbo(:, 3,30,10) = (/ &
     &1.1024e+01_rb,8.6363e+00_rb,6.0145e+00_rb,3.1864e+00_rb,5.7473e-02_rb/)
      kbo(:, 4,30,10) = (/ &
     &1.4192e+01_rb,1.1103e+01_rb,7.7439e+00_rb,4.1120e+00_rb,7.0729e-02_rb/)
      kbo(:, 5,30,10) = (/ &
     &1.7890e+01_rb,1.3978e+01_rb,9.7651e+00_rb,5.1860e+00_rb,8.5534e-02_rb/)
      kbo(:, 1,31,10) = (/ &
     &5.8439e+00_rb,4.6051e+00_rb,3.1957e+00_rb,1.6863e+00_rb,3.2654e-02_rb/)
      kbo(:, 2,31,10) = (/ &
     &7.8033e+00_rb,6.1378e+00_rb,4.2622e+00_rb,2.2515e+00_rb,4.2183e-02_rb/)
      kbo(:, 3,31,10) = (/ &
     &1.0196e+01_rb,8.0235e+00_rb,5.5836e+00_rb,2.9533e+00_rb,5.3011e-02_rb/)
      kbo(:, 4,31,10) = (/ &
     &1.3072e+01_rb,1.0286e+01_rb,7.1778e+00_rb,3.8005e+00_rb,6.4875e-02_rb/)
      kbo(:, 5,31,10) = (/ &
     &1.6463e+01_rb,1.2924e+01_rb,9.0292e+00_rb,4.7849e+00_rb,7.7772e-02_rb/)
      kbo(:, 1,32,10) = (/ &
     &5.4664e+00_rb,4.3291e+00_rb,2.9999e+00_rb,1.5808e+00_rb,2.9923e-02_rb/)
      kbo(:, 2,32,10) = (/ &
     &7.2874e+00_rb,5.7710e+00_rb,4.0068e+00_rb,2.1122e+00_rb,3.8267e-02_rb/)
      kbo(:, 3,32,10) = (/ &
     &9.5041e+00_rb,7.5231e+00_rb,5.2374e+00_rb,2.7661e+00_rb,4.7331e-02_rb/)
      kbo(:, 4,32,10) = (/ &
     &1.2176e+01_rb,9.6156e+00_rb,6.7066e+00_rb,3.5449e+00_rb,5.8625e-02_rb/)
      kbo(:, 5,32,10) = (/ &
     &1.5322e+01_rb,1.2084e+01_rb,8.4483e+00_rb,4.4709e+00_rb,6.9340e-02_rb/)
      kbo(:, 1,33,10) = (/ &
     &5.1512e+00_rb,4.0967e+00_rb,2.8362e+00_rb,1.4933e+00_rb,2.6393e-02_rb/)
      kbo(:, 2,33,10) = (/ &
     &6.8599e+00_rb,5.4552e+00_rb,3.7902e+00_rb,1.9993e+00_rb,3.4040e-02_rb/)
      kbo(:, 3,33,10) = (/ &
     &8.9395e+00_rb,7.1038e+00_rb,4.9489e+00_rb,2.6104e+00_rb,4.2820e-02_rb/)
      kbo(:, 4,33,10) = (/ &
     &1.1449e+01_rb,9.0816e+00_rb,6.3367e+00_rb,3.3421e+00_rb,5.1919e-02_rb/)
      kbo(:, 5,33,10) = (/ &
     &1.4423e+01_rb,1.1426e+01_rb,7.9850e+00_rb,4.2179e+00_rb,6.2402e-02_rb/)
      kbo(:, 1,34,10) = (/ &
     &4.8646e+00_rb,3.8880e+00_rb,2.6932e+00_rb,1.4194e+00_rb,2.3781e-02_rb/)
      kbo(:, 2,34,10) = (/ &
     &6.4672e+00_rb,5.1782e+00_rb,3.6018e+00_rb,1.8962e+00_rb,3.0492e-02_rb/)
      kbo(:, 3,34,10) = (/ &
     &8.4535e+00_rb,6.7497e+00_rb,4.7054e+00_rb,2.4770e+00_rb,3.8339e-02_rb/)
      kbo(:, 4,34,10) = (/ &
     &1.0856e+01_rb,8.6484e+00_rb,6.0350e+00_rb,3.1793e+00_rb,4.6570e-02_rb/)
      kbo(:, 5,34,10) = (/ &
     &1.3723e+01_rb,1.0909e+01_rb,7.6246e+00_rb,4.0214e+00_rb,5.6226e-02_rb/)
      kbo(:, 1,35,10) = (/ &
     &4.4902e+00_rb,3.6079e+00_rb,2.5027e+00_rb,1.3156e+00_rb,2.1238e-02_rb/)
      kbo(:, 2,35,10) = (/ &
     &5.9918e+00_rb,4.8154e+00_rb,3.3497e+00_rb,1.7639e+00_rb,2.7153e-02_rb/)
      kbo(:, 3,35,10) = (/ &
     &7.8693e+00_rb,6.3098e+00_rb,4.3964e+00_rb,2.3124e+00_rb,3.3573e-02_rb/)
      kbo(:, 4,35,10) = (/ &
     &1.0156e+01_rb,8.1193e+00_rb,5.6658e+00_rb,2.9812e+00_rb,4.1298e-02_rb/)
      kbo(:, 5,35,10) = (/ &
     &1.2867e+01_rb,1.0277e+01_rb,7.1860e+00_rb,3.7872e+00_rb,4.9619e-02_rb/)
      kbo(:, 1,36,10) = (/ &
     &4.0317e+00_rb,3.2575e+00_rb,2.2591e+00_rb,1.1864e+00_rb,1.8322e-02_rb/)
      kbo(:, 2,36,10) = (/ &
     &5.4230e+00_rb,4.3764e+00_rb,3.0444e+00_rb,1.6008e+00_rb,2.3437e-02_rb/)
      kbo(:, 3,36,10) = (/ &
     &7.1807e+00_rb,5.7770e+00_rb,4.0236e+00_rb,2.1147e+00_rb,2.8934e-02_rb/)
      kbo(:, 4,36,10) = (/ &
     &9.3214e+00_rb,7.4860e+00_rb,5.2247e+00_rb,2.7453e+00_rb,3.5476e-02_rb/)
      kbo(:, 5,36,10) = (/ &
     &1.1871e+01_rb,9.5152e+00_rb,6.6538e+00_rb,3.5057e+00_rb,4.2476e-02_rb/)
      kbo(:, 1,37,10) = (/ &
     &3.4945e+00_rb,2.8322e+00_rb,1.9642e+00_rb,1.0316e+00_rb,1.5292e-02_rb/)
      kbo(:, 2,37,10) = (/ &
     &4.7474e+00_rb,3.8477e+00_rb,2.6749e+00_rb,1.4057e+00_rb,1.9829e-02_rb/)
      kbo(:, 3,37,10) = (/ &
     &6.3563e+00_rb,5.1351e+00_rb,3.5754e+00_rb,1.8774e+00_rb,2.4925e-02_rb/)
      kbo(:, 4,37,10) = (/ &
     &8.3333e+00_rb,6.7182e+00_rb,4.6872e+00_rb,2.4610e+00_rb,3.0513e-02_rb/)
      kbo(:, 5,37,10) = (/ &
     &1.0705e+01_rb,8.6094e+00_rb,6.0143e+00_rb,3.1664e+00_rb,3.6834e-02_rb/)
      kbo(:, 1,38,10) = (/ &
     &3.0288e+00_rb,2.4619e+00_rb,1.7084e+00_rb,8.9674e-01_rb,1.2715e-02_rb/)
      kbo(:, 2,38,10) = (/ &
     &4.1608e+00_rb,3.3834e+00_rb,2.3515e+00_rb,1.2353e+00_rb,1.6555e-02_rb/)
      kbo(:, 3,38,10) = (/ &
     &5.6346e+00_rb,4.5721e+00_rb,3.1807e+00_rb,1.6688e+00_rb,2.1253e-02_rb/)
      kbo(:, 4,38,10) = (/ &
     &7.4639e+00_rb,6.0370e+00_rb,4.2092e+00_rb,2.2090e+00_rb,2.6215e-02_rb/)
      kbo(:, 5,38,10) = (/ &
     &9.6723e+00_rb,7.8054e+00_rb,5.4498e+00_rb,2.8669e+00_rb,3.1673e-02_rb/)
      kbo(:, 1,39,10) = (/ &
     &2.6346e+00_rb,2.1501e+00_rb,1.4897e+00_rb,7.8179e-01_rb,1.0494e-02_rb/)
      kbo(:, 2,39,10) = (/ &
     &3.6623e+00_rb,2.9868e+00_rb,2.0748e+00_rb,1.0900e+00_rb,1.3801e-02_rb/)
      kbo(:, 3,39,10) = (/ &
     &5.0149e+00_rb,4.0819e+00_rb,2.8376e+00_rb,1.4889e+00_rb,1.7948e-02_rb/)
      kbo(:, 4,39,10) = (/ &
     &6.7074e+00_rb,5.4432e+00_rb,3.7906e+00_rb,1.9883e+00_rb,2.2190e-02_rb/)
      kbo(:, 5,39,10) = (/ &
     &8.7675e+00_rb,7.1019e+00_rb,4.9572e+00_rb,2.6054e+00_rb,2.6961e-02_rb/)
      kbo(:, 1,40,10) = (/ &
     &2.2414e+00_rb,1.8312e+00_rb,1.2669e+00_rb,6.6464e-01_rb,8.6033e-03_rb/)
      kbo(:, 2,40,10) = (/ &
     &3.1461e+00_rb,2.5731e+00_rb,1.7855e+00_rb,9.3826e-01_rb,1.1425e-02_rb/)
      kbo(:, 3,40,10) = (/ &
     &4.3645e+00_rb,3.5639e+00_rb,2.4772e+00_rb,1.3001e+00_rb,1.5150e-02_rb/)
      kbo(:, 4,40,10) = (/ &
     &5.9195e+00_rb,4.8196e+00_rb,3.3561e+00_rb,1.7603e+00_rb,1.8984e-02_rb/)
      kbo(:, 5,40,10) = (/ &
     &7.8344e+00_rb,6.3662e+00_rb,4.4409e+00_rb,2.3324e+00_rb,2.3286e-02_rb/)
      kbo(:, 1,41,10) = (/ &
     &1.9009e+00_rb,1.5581e+00_rb,1.0772e+00_rb,5.6390e-01_rb,7.0145e-03_rb/)
      kbo(:, 2,41,10) = (/ &
     &2.6997e+00_rb,2.2094e+00_rb,1.5334e+00_rb,8.0488e-01_rb,9.5005e-03_rb/)
      kbo(:, 3,41,10) = (/ &
     &3.7885e+00_rb,3.1035e+00_rb,2.1567e+00_rb,1.1320e+00_rb,1.2665e-02_rb/)
      kbo(:, 4,41,10) = (/ &
     &5.2083e+00_rb,4.2587e+00_rb,2.9632e+00_rb,1.5552e+00_rb,1.6096e-02_rb/)
      kbo(:, 5,41,10) = (/ &
     &6.9899e+00_rb,5.6957e+00_rb,3.9715e+00_rb,2.0851e+00_rb,2.0007e-02_rb/)
      kbo(:, 1,42,10) = (/ &
     &1.6073e+00_rb,1.3214e+00_rb,9.1410e-01_rb,4.7869e-01_rb,5.6658e-03_rb/)
      kbo(:, 2,42,10) = (/ &
     &2.3166e+00_rb,1.9016e+00_rb,1.3175e+00_rb,6.9100e-01_rb,7.8082e-03_rb/)
      kbo(:, 3,42,10) = (/ &
     &3.2935e+00_rb,2.7045e+00_rb,1.8773e+00_rb,9.8575e-01_rb,1.0484e-02_rb/)
      kbo(:, 4,42,10) = (/ &
     &4.5881e+00_rb,3.7639e+00_rb,2.6177e+00_rb,1.3737e+00_rb,1.3601e-02_rb/)
      kbo(:, 5,42,10) = (/ &
     &6.2370e+00_rb,5.0967e+00_rb,3.5534e+00_rb,1.8654e+00_rb,1.7039e-02_rb/)
      kbo(:, 1,43,10) = (/ &
     &1.3404e+00_rb,1.1058e+00_rb,7.6461e-01_rb,4.0031e-01_rb,4.4847e-03_rb/)
      kbo(:, 2,43,10) = (/ &
     &1.9669e+00_rb,1.6173e+00_rb,1.1194e+00_rb,5.8704e-01_rb,6.2717e-03_rb/)
      kbo(:, 3,43,10) = (/ &
     &2.8361e+00_rb,2.3337e+00_rb,1.6185e+00_rb,8.4949e-01_rb,8.5308e-03_rb/)
      kbo(:, 4,43,10) = (/ &
     &4.0094e+00_rb,3.2948e+00_rb,2.2900e+00_rb,1.2020e+00_rb,1.1259e-02_rb/)
      kbo(:, 5,43,10) = (/ &
     &5.5262e+00_rb,4.5280e+00_rb,3.1542e+00_rb,1.6556e+00_rb,1.4384e-02_rb/)
      kbo(:, 1,44,10) = (/ &
     &1.1066e+00_rb,9.1547e-01_rb,6.3275e-01_rb,3.3150e-01_rb,3.4897e-03_rb/)
      kbo(:, 2,44,10) = (/ &
     &1.6542e+00_rb,1.3635e+00_rb,9.4399e-01_rb,4.9494e-01_rb,4.9430e-03_rb/)
      kbo(:, 3,44,10) = (/ &
     &2.4266e+00_rb,2.0020e+00_rb,1.3887e+00_rb,7.2847e-01_rb,6.8682e-03_rb/)
      kbo(:, 4,44,10) = (/ &
     &3.4886e+00_rb,2.8708e+00_rb,1.9931e+00_rb,1.0459e+00_rb,9.1381e-03_rb/)
      kbo(:, 5,44,10) = (/ &
     &4.8773e+00_rb,4.0046e+00_rb,2.7873e+00_rb,1.4630e+00_rb,1.1960e-02_rb/)
      kbo(:, 1,45,10) = (/ &
     &9.0739e-01_rb,7.5424e-01_rb,5.2128e-01_rb,2.7273e-01_rb,2.7098e-03_rb/)
      kbo(:, 2,45,10) = (/ &
     &1.3852e+00_rb,1.1467e+00_rb,7.9304e-01_rb,4.1578e-01_rb,3.8548e-03_rb/)
      kbo(:, 3,45,10) = (/ &
     &2.0713e+00_rb,1.7151e+00_rb,1.1892e+00_rb,6.2426e-01_rb,5.4604e-03_rb/)
      kbo(:, 4,45,10) = (/ &
     &3.0326e+00_rb,2.5006e+00_rb,1.7345e+00_rb,9.0988e-01_rb,7.3887e-03_rb/)
      kbo(:, 5,45,10) = (/ &
     &4.3008e+00_rb,3.5387e+00_rb,2.4612e+00_rb,1.2914e+00_rb,9.7321e-03_rb/)
      kbo(:, 1,46,10) = (/ &
     &7.3199e-01_rb,6.1145e-01_rb,4.2240e-01_rb,2.2117e-01_rb,2.0785e-03_rb/)
      kbo(:, 2,46,10) = (/ &
     &1.1438e+00_rb,9.5098e-01_rb,6.5833e-01_rb,3.4471e-01_rb,2.9805e-03_rb/)
      kbo(:, 3,46,10) = (/ &
     &1.7481e+00_rb,1.4525e+00_rb,1.0068e+00_rb,5.2845e-01_rb,4.2717e-03_rb/)
      kbo(:, 4,46,10) = (/ &
     &2.6098e+00_rb,2.1582e+00_rb,1.4963e+00_rb,7.8501e-01_rb,5.8631e-03_rb/)
      kbo(:, 5,46,10) = (/ &
     &3.7663e+00_rb,3.1031e+00_rb,2.1564e+00_rb,1.1312e+00_rb,7.7831e-03_rb/)
      kbo(:, 1,47,10) = (/ &
     &5.7232e-01_rb,4.8032e-01_rb,3.3210e-01_rb,1.7403e-01_rb,1.5461e-03_rb/)
      kbo(:, 2,47,10) = (/ &
     &9.2386e-01_rb,7.7116e-01_rb,5.3364e-01_rb,2.7934e-01_rb,2.3209e-03_rb/)
      kbo(:, 3,47,10) = (/ &
     &1.4429e+00_rb,1.2024e+00_rb,8.3406e-01_rb,4.3757e-01_rb,3.2763e-03_rb/)
      kbo(:, 4,47,10) = (/ &
     &2.2047e+00_rb,1.8273e+00_rb,1.2675e+00_rb,6.6495e-01_rb,4.5628e-03_rb/)
      kbo(:, 5,47,10) = (/ &
     &3.2456e+00_rb,2.6812e+00_rb,1.8614e+00_rb,9.7623e-01_rb,6.1070e-03_rb/)
      kbo(:, 1,48,10) = (/ &
     &4.4270e-01_rb,3.7313e-01_rb,2.5809e-01_rb,1.3535e-01_rb,1.1374e-03_rb/)
      kbo(:, 2,48,10) = (/ &
     &7.3749e-01_rb,6.1935e-01_rb,4.2906e-01_rb,2.2453e-01_rb,1.7379e-03_rb/)
      kbo(:, 3,48,10) = (/ &
     &1.1839e+00_rb,9.8928e-01_rb,6.8613e-01_rb,3.6002e-01_rb,2.5032e-03_rb/)
      kbo(:, 4,48,10) = (/ &
     &1.8512e+00_rb,1.5395e+00_rb,1.0677e+00_rb,5.6075e-01_rb,3.5238e-03_rb/)
      kbo(:, 5,48,10) = (/ &
     &2.7862e+00_rb,2.3065e+00_rb,1.6009e+00_rb,8.3981e-01_rb,4.7398e-03_rb/)
      kbo(:, 1,49,10) = (/ &
     &3.3829e-01_rb,2.8482e-01_rb,1.9699e-01_rb,1.0339e-01_rb,8.1861e-04_rb/)
      kbo(:, 2,49,10) = (/ &
     &5.8200e-01_rb,4.9068e-01_rb,3.4013e-01_rb,1.7816e-01_rb,1.2634e-03_rb/)
      kbo(:, 3,49,10) = (/ &
     &9.6365e-01_rb,8.0857e-01_rb,5.6081e-01_rb,2.9418e-01_rb,1.8795e-03_rb/)
      kbo(:, 4,49,10) = (/ &
     &1.5457e+00_rb,1.2894e+00_rb,8.9458e-01_rb,4.6973e-01_rb,2.7104e-03_rb/)
      kbo(:, 5,49,10) = (/ &
     &2.3812e+00_rb,1.9755e+00_rb,1.3708e+00_rb,7.1925e-01_rb,3.6105e-03_rb/)
      kbo(:, 1,50,10) = (/ &
     &2.5998e-01_rb,2.1962e-01_rb,1.5188e-01_rb,7.9506e-02_rb,6.0363e-04_rb/)
      kbo(:, 2,50,10) = (/ &
     &4.5974e-01_rb,3.8940e-01_rb,2.6982e-01_rb,1.4155e-01_rb,9.5023e-04_rb/)
      kbo(:, 3,50,10) = (/ &
     &7.8578e-01_rb,6.6189e-01_rb,4.5931e-01_rb,2.4091e-01_rb,1.4366e-03_rb/)
      kbo(:, 4,50,10) = (/ &
     &1.2930e+00_rb,1.0829e+00_rb,7.5147e-01_rb,3.9450e-01_rb,2.1250e-03_rb/)
      kbo(:, 5,50,10) = (/ &
     &2.0418e+00_rb,1.6990e+00_rb,1.1790e+00_rb,6.1880e-01_rb,2.8716e-03_rb/)
      kbo(:, 1,51,10) = (/ &
     &2.0132e-01_rb,1.6910e-01_rb,1.1674e-01_rb,6.1107e-02_rb,4.4485e-04_rb/)
      kbo(:, 2,51,10) = (/ &
     &3.6041e-01_rb,3.0656e-01_rb,2.1282e-01_rb,1.1161e-01_rb,7.3157e-04_rb/)
      kbo(:, 3,51,10) = (/ &
     &6.3582e-01_rb,5.3869e-01_rb,3.7423e-01_rb,1.9656e-01_rb,1.1225e-03_rb/)
      kbo(:, 4,51,10) = (/ &
     &1.0802e+00_rb,9.0777e-01_rb,6.3030e-01_rb,3.3074e-01_rb,1.6531e-03_rb/)
      kbo(:, 5,51,10) = (/ &
     &1.7498e+00_rb,1.4602e+00_rb,1.0135e+00_rb,5.3219e-01_rb,2.3172e-03_rb/)
      kbo(:, 1,52,10) = (/ &
     &1.5547e-01_rb,1.3052e-01_rb,9.0061e-02_rb,4.7075e-02_rb,3.3160e-04_rb/)
      kbo(:, 2,52,10) = (/ &
     &2.8347e-01_rb,2.4049e-01_rb,1.6666e-01_rb,8.7433e-02_rb,5.5078e-04_rb/)
      kbo(:, 3,52,10) = (/ &
     &5.1184e-01_rb,4.3501e-01_rb,3.0211e-01_rb,1.5868e-01_rb,8.7008e-04_rb/)
      kbo(:, 4,52,10) = (/ &
     &8.9697e-01_rb,7.5690e-01_rb,5.2561e-01_rb,2.7576e-01_rb,1.2672e-03_rb/)
      kbo(:, 5,52,10) = (/ &
     &1.4927e+00_rb,1.2493e+00_rb,8.6741e-01_rb,4.5551e-01_rb,1.8458e-03_rb/)
      kbo(:, 1,53,10) = (/ &
     &1.2003e-01_rb,1.0051e-01_rb,6.9268e-02_rb,3.6248e-02_rb,2.4457e-04_rb/)
      kbo(:, 2,53,10) = (/ &
     &2.2120e-01_rb,1.8754e-01_rb,1.2989e-01_rb,6.8226e-02_rb,4.0791e-04_rb/)
      kbo(:, 3,53,10) = (/ &
     &4.0694e-01_rb,3.4775e-01_rb,2.4176e-01_rb,1.2712e-01_rb,6.5057e-04_rb/)
      kbo(:, 4,53,10) = (/ &
     &7.3824e-01_rb,6.2670e-01_rb,4.3556e-01_rb,2.2862e-01_rb,9.8738e-04_rb/)
      kbo(:, 5,53,10) = (/ &
     &1.2666e+00_rb,1.0632e+00_rb,7.3852e-01_rb,3.8765e-01_rb,1.4077e-03_rb/)
      kbo(:, 1,54,10) = (/ &
     &9.3960e-02_rb,7.8771e-02_rb,5.4240e-02_rb,2.8401e-02_rb,1.9316e-04_rb/)
      kbo(:, 2,54,10) = (/ &
     &1.7676e-01_rb,1.4984e-01_rb,1.0336e-01_rb,5.4121e-02_rb,3.1568e-04_rb/)
      kbo(:, 3,54,10) = (/ &
     &3.2867e-01_rb,2.7969e-01_rb,1.9447e-01_rb,1.0231e-01_rb,5.2512e-04_rb/)
      kbo(:, 4,54,10) = (/ &
     &6.0965e-01_rb,5.1980e-01_rb,3.6169e-01_rb,1.9003e-01_rb,8.0976e-04_rb/)
      kbo(:, 5,54,10) = (/ &
     &1.0822e+00_rb,9.1223e-01_rb,6.3374e-01_rb,3.3269e-01_rb,1.1750e-03_rb/)
      kbo(:, 1,55,10) = (/ &
     &7.3906e-02_rb,6.2107e-02_rb,4.2864e-02_rb,2.2445e-02_rb,1.5599e-04_rb/)
      kbo(:, 2,55,10) = (/ &
     &1.4369e-01_rb,1.2050e-01_rb,8.3134e-02_rb,4.3567e-02_rb,2.5335e-04_rb/)
      kbo(:, 3,55,10) = (/ &
     &2.6760e-01_rb,2.2827e-01_rb,1.5835e-01_rb,8.3096e-02_rb,4.2909e-04_rb/)
      kbo(:, 4,55,10) = (/ &
     &5.0351e-01_rb,4.3105e-01_rb,3.0006e-01_rb,1.5782e-01_rb,6.9202e-04_rb/)
      kbo(:, 5,55,10) = (/ &
     &9.2458e-01_rb,7.8288e-01_rb,5.4396e-01_rb,2.8582e-01_rb,1.0207e-03_rb/)
      kbo(:, 1,56,10) = (/ &
     &5.7777e-02_rb,4.8496e-02_rb,3.3489e-02_rb,1.7579e-02_rb,1.2442e-04_rb/)
      kbo(:, 2,56,10) = (/ &
     &1.1665e-01_rb,9.7805e-02_rb,6.7433e-02_rb,3.5342e-02_rb,2.0689e-04_rb/)
      kbo(:, 3,56,10) = (/ &
     &2.1880e-01_rb,1.8636e-01_rb,1.2905e-01_rb,6.7804e-02_rb,3.5149e-04_rb/)
      kbo(:, 4,56,10) = (/ &
     &4.1579e-01_rb,3.5633e-01_rb,2.4779e-01_rb,1.3043e-01_rb,5.7437e-04_rb/)
      kbo(:, 5,56,10) = (/ &
     &7.8392e-01_rb,6.6810e-01_rb,4.6498e-01_rb,2.4434e-01_rb,8.7392e-04_rb/)
      kbo(:, 1,57,10) = (/ &
     &4.4962e-02_rb,3.7745e-02_rb,2.6020e-02_rb,1.3669e-02_rb,1.0074e-04_rb/)
      kbo(:, 2,57,10) = (/ &
     &9.3975e-02_rb,7.9039e-02_rb,5.4468e-02_rb,2.8583e-02_rb,1.6888e-04_rb/)
      kbo(:, 3,57,10) = (/ &
     &1.8083e-01_rb,1.5305e-01_rb,1.0598e-01_rb,5.5519e-02_rb,2.8180e-04_rb/)
      kbo(:, 4,57,10) = (/ &
     &3.4541e-01_rb,2.9431e-01_rb,2.0476e-01_rb,1.0780e-01_rb,4.7931e-04_rb/)
      kbo(:, 5,57,10) = (/ &
     &6.6212e-01_rb,5.6589e-01_rb,3.9390e-01_rb,2.0726e-01_rb,7.5979e-04_rb/)
      kbo(:, 1,58,10) = (/ &
     &1.7062e-02_rb,1.6422e-02_rb,1.3301e-02_rb,8.4676e-03_rb,8.2142e-05_rb/)
      kbo(:, 2,58,10) = (/ &
     &3.7044e-02_rb,3.5791e-02_rb,2.8933e-02_rb,1.8344e-02_rb,1.3979e-04_rb/)
      kbo(:, 3,58,10) = (/ &
     &7.3627e-02_rb,7.1176e-02_rb,5.7648e-02_rb,3.6534e-02_rb,2.3072e-04_rb/)
      kbo(:, 4,58,10) = (/ &
     &1.4036e-01_rb,1.3776e-01_rb,1.1219e-01_rb,7.1459e-02_rb,4.0521e-04_rb/)
      kbo(:, 5,58,10) = (/ &
     &2.7375e-01_rb,2.6857e-01_rb,2.1925e-01_rb,1.3965e-01_rb,6.6868e-04_rb/)
      kbo(:, 1,59,10) = (/ &
     &1.5026e-02_rb,1.4620e-02_rb,1.1891e-02_rb,7.6139e-03_rb,7.7772e-05_rb/)
      kbo(:, 2,59,10) = (/ &
     &3.3598e-02_rb,3.2618e-02_rb,2.6475e-02_rb,1.6873e-02_rb,1.3363e-04_rb/)
      kbo(:, 3,59,10) = (/ &
     &6.7778e-02_rb,6.5724e-02_rb,5.3548e-02_rb,3.4085e-02_rb,2.3127e-04_rb/)
      kbo(:, 4,59,10) = (/ &
     &1.2973e-01_rb,1.2749e-01_rb,1.0439e-01_rb,6.6705e-02_rb,3.9949e-04_rb/)
      kbo(:, 5,59,10) = (/ &
     &2.5383e-01_rb,2.5006e-01_rb,2.0520e-01_rb,1.3156e-01_rb,6.6551e-04_rb/)
      kbo(:, 1,13,11) = (/ &
     &6.1162e+01_rb,4.5871e+01_rb,3.0581e+01_rb,1.5291e+01_rb,3.0501e-01_rb/)
      kbo(:, 2,13,11) = (/ &
     &8.6643e+01_rb,6.4984e+01_rb,4.3324e+01_rb,2.1662e+01_rb,3.8069e-01_rb/)
      kbo(:, 3,13,11) = (/ &
     &1.1700e+02_rb,8.7751e+01_rb,5.8502e+01_rb,2.9250e+01_rb,4.9491e-01_rb/)
      kbo(:, 4,13,11) = (/ &
     &1.5190e+02_rb,1.1392e+02_rb,7.5955e+01_rb,3.7977e+01_rb,5.7908e-01_rb/)
      kbo(:, 5,13,11) = (/ &
     &1.9085e+02_rb,1.4314e+02_rb,9.5429e+01_rb,4.7715e+01_rb,6.6979e-01_rb/)
      kbo(:, 1,14,11) = (/ &
     &6.2014e+01_rb,4.6511e+01_rb,3.1007e+01_rb,1.5504e+01_rb,3.5592e-01_rb/)
      kbo(:, 2,14,11) = (/ &
     &8.7238e+01_rb,6.5431e+01_rb,4.3621e+01_rb,2.1811e+01_rb,3.9219e-01_rb/)
      kbo(:, 3,14,11) = (/ &
     &1.1714e+02_rb,8.7851e+01_rb,5.8570e+01_rb,2.9287e+01_rb,4.4292e-01_rb/)
      kbo(:, 4,14,11) = (/ &
     &1.5139e+02_rb,1.1354e+02_rb,7.5699e+01_rb,3.7864e+01_rb,5.2460e-01_rb/)
      kbo(:, 5,14,11) = (/ &
     &1.8982e+02_rb,1.4235e+02_rb,9.4908e+01_rb,4.7484e+01_rb,6.6257e-01_rb/)
      kbo(:, 1,15,11) = (/ &
     &5.9186e+01_rb,4.4388e+01_rb,2.9592e+01_rb,1.4841e+01_rb,3.2021e-01_rb/)
      kbo(:, 2,15,11) = (/ &
     &8.2638e+01_rb,6.1979e+01_rb,4.1319e+01_rb,2.0713e+01_rb,3.7708e-01_rb/)
      kbo(:, 3,15,11) = (/ &
     &1.1028e+02_rb,8.2716e+01_rb,5.5144e+01_rb,2.7652e+01_rb,4.6488e-01_rb/)
      kbo(:, 4,15,11) = (/ &
     &1.4201e+02_rb,1.0651e+02_rb,7.1008e+01_rb,3.5635e+01_rb,5.3643e-01_rb/)
      kbo(:, 5,15,11) = (/ &
     &1.7780e+02_rb,1.3336e+02_rb,8.8908e+01_rb,4.4680e+01_rb,6.1809e-01_rb/)
      kbo(:, 1,16,11) = (/ &
     &5.3877e+01_rb,4.0407e+01_rb,2.6940e+01_rb,1.3617e+01_rb,3.0459e-01_rb/)
      kbo(:, 2,16,11) = (/ &
     &7.4732e+01_rb,5.6052e+01_rb,3.7368e+01_rb,1.8895e+01_rb,3.6381e-01_rb/)
      kbo(:, 3,16,11) = (/ &
     &9.9303e+01_rb,7.4473e+01_rb,4.9649e+01_rb,2.5124e+01_rb,4.2325e-01_rb/)
      kbo(:, 4,16,11) = (/ &
     &1.2768e+02_rb,9.5754e+01_rb,6.3837e+01_rb,3.2325e+01_rb,4.7285e-01_rb/)
      kbo(:, 5,16,11) = (/ &
     &1.5927e+02_rb,1.1945e+02_rb,7.9632e+01_rb,4.0351e+01_rb,5.0976e-01_rb/)
      kbo(:, 1,17,11) = (/ &
     &4.7648e+01_rb,3.5735e+01_rb,2.3823e+01_rb,1.2164e+01_rb,2.5744e-01_rb/)
      kbo(:, 2,17,11) = (/ &
     &6.5724e+01_rb,4.9293e+01_rb,3.2863e+01_rb,1.6792e+01_rb,3.1059e-01_rb/)
      kbo(:, 3,17,11) = (/ &
     &8.7138e+01_rb,6.5349e+01_rb,4.3569e+01_rb,2.2270e+01_rb,3.5847e-01_rb/)
      kbo(:, 4,17,11) = (/ &
     &1.1169e+02_rb,8.3769e+01_rb,5.5845e+01_rb,2.8543e+01_rb,4.0184e-01_rb/)
      kbo(:, 5,17,11) = (/ &
     &1.3846e+02_rb,1.0385e+02_rb,6.9237e+01_rb,3.5386e+01_rb,4.6254e-01_rb/)
      kbo(:, 1,18,11) = (/ &
     &4.0731e+01_rb,3.0550e+01_rb,2.0398e+01_rb,1.0482e+01_rb,2.2929e-01_rb/)
      kbo(:, 2,18,11) = (/ &
     &5.6038e+01_rb,4.2027e+01_rb,2.8058e+01_rb,1.4421e+01_rb,2.6841e-01_rb/)
      kbo(:, 3,18,11) = (/ &
     &7.4130e+01_rb,5.5602e+01_rb,3.7127e+01_rb,1.9095e+01_rb,3.0083e-01_rb/)
      kbo(:, 4,18,11) = (/ &
     &9.4414e+01_rb,7.0811e+01_rb,4.7323e+01_rb,2.4371e+01_rb,3.4142e-01_rb/)
      kbo(:, 5,18,11) = (/ &
     &1.1687e+02_rb,8.7655e+01_rb,5.8604e+01_rb,3.0228e+01_rb,3.9932e-01_rb/)
      kbo(:, 1,19,11) = (/ &
     &3.5043e+01_rb,2.6281e+01_rb,1.7645e+01_rb,9.0880e+00_rb,1.9041e-01_rb/)
      kbo(:, 2,19,11) = (/ &
     &4.8095e+01_rb,3.6072e+01_rb,2.4224e+01_rb,1.2500e+01_rb,2.2005e-01_rb/)
      kbo(:, 3,19,11) = (/ &
     &6.3280e+01_rb,4.7465e+01_rb,3.1884e+01_rb,1.6505e+01_rb,2.5190e-01_rb/)
      kbo(:, 4,19,11) = (/ &
     &8.0459e+01_rb,6.0352e+01_rb,4.0562e+01_rb,2.1050e+01_rb,3.0347e-01_rb/)
      kbo(:, 5,19,11) = (/ &
     &9.9405e+01_rb,7.4555e+01_rb,5.0141e+01_rb,2.6072e+01_rb,3.3814e-01_rb/)
      kbo(:, 1,20,11) = (/ &
     &3.0698e+01_rb,2.3028e+01_rb,1.5559e+01_rb,8.0744e+00_rb,1.6851e-01_rb/)
      kbo(:, 2,20,11) = (/ &
     &4.1938e+01_rb,3.1449e+01_rb,2.1266e+01_rb,1.1075e+01_rb,1.9604e-01_rb/)
      kbo(:, 3,20,11) = (/ &
     &5.4971e+01_rb,4.1226e+01_rb,2.7873e+01_rb,1.4553e+01_rb,2.3334e-01_rb/)
      kbo(:, 4,20,11) = (/ &
     &6.9683e+01_rb,5.2264e+01_rb,3.5322e+01_rb,1.8474e+01_rb,2.8155e-01_rb/)
      kbo(:, 5,20,11) = (/ &
     &8.5796e+01_rb,6.4350e+01_rb,4.3496e+01_rb,2.2781e+01_rb,3.1819e-01_rb/)
      kbo(:, 1,21,11) = (/ &
     &2.6961e+01_rb,2.0239e+01_rb,1.3721e+01_rb,7.2357e+00_rb,1.5633e-01_rb/)
      kbo(:, 2,21,11) = (/ &
     &3.6624e+01_rb,2.7486e+01_rb,1.8642e+01_rb,9.8382e+00_rb,1.8477e-01_rb/)
      kbo(:, 3,21,11) = (/ &
     &4.7842e+01_rb,3.5913e+01_rb,2.4362e+01_rb,1.2852e+01_rb,2.2793e-01_rb/)
      kbo(:, 4,21,11) = (/ &
     &6.0403e+01_rb,4.5355e+01_rb,3.0794e+01_rb,1.6247e+01_rb,2.5875e-01_rb/)
      kbo(:, 5,21,11) = (/ &
     &7.4328e+01_rb,5.5827e+01_rb,3.7921e+01_rb,1.9978e+01_rb,3.0003e-01_rb/)
      kbo(:, 1,22,11) = (/ &
     &2.4106e+01_rb,1.8145e+01_rb,1.2318e+01_rb,6.5788e+00_rb,1.4346e-01_rb/)
      kbo(:, 2,22,11) = (/ &
     &3.2489e+01_rb,2.4455e+01_rb,1.6620e+01_rb,8.8645e+00_rb,1.7673e-01_rb/)
      kbo(:, 3,22,11) = (/ &
     &4.2111e+01_rb,3.1704e+01_rb,2.1585e+01_rb,1.1502e+01_rb,2.1135e-01_rb/)
      kbo(:, 4,22,11) = (/ &
     &5.3091e+01_rb,3.9979e+01_rb,2.7248e+01_rb,1.4479e+01_rb,2.4545e-01_rb/)
      kbo(:, 5,22,11) = (/ &
     &6.5531e+01_rb,4.9344e+01_rb,3.3661e+01_rb,1.7840e+01_rb,2.8184e-01_rb/)
      kbo(:, 1,23,11) = (/ &
     &2.1586e+01_rb,1.6314e+01_rb,1.1115e+01_rb,5.9696e+00_rb,1.3118e-01_rb/)
      kbo(:, 2,23,11) = (/ &
     &2.8872e+01_rb,2.1824e+01_rb,1.4900e+01_rb,7.9884e+00_rb,1.6791e-01_rb/)
      kbo(:, 3,23,11) = (/ &
     &3.7392e+01_rb,2.8256e+01_rb,1.9318e+01_rb,1.0333e+01_rb,1.9239e-01_rb/)
      kbo(:, 4,23,11) = (/ &
     &4.7286e+01_rb,3.5715e+01_rb,2.4417e+01_rb,1.3025e+01_rb,2.2854e-01_rb/)
      kbo(:, 5,23,11) = (/ &
     &5.8587e+01_rb,4.4225e+01_rb,3.0228e+01_rb,1.6103e+01_rb,2.6383e-01_rb/)
      kbo(:, 1,24,11) = (/ &
     &1.9610e+01_rb,1.4859e+01_rb,1.0204e+01_rb,5.4843e+00_rb,1.2305e-01_rb/)
      kbo(:, 2,24,11) = (/ &
     &2.6168e+01_rb,1.9825e+01_rb,1.3617e+01_rb,7.3028e+00_rb,1.4696e-01_rb/)
      kbo(:, 3,24,11) = (/ &
     &3.4009e+01_rb,2.5754e+01_rb,1.7666e+01_rb,9.4475e+00_rb,1.7619e-01_rb/)
      kbo(:, 4,24,11) = (/ &
     &4.3202e+01_rb,3.2697e+01_rb,2.2398e+01_rb,1.1966e+01_rb,2.0711e-01_rb/)
      kbo(:, 5,24,11) = (/ &
     &5.3950e+01_rb,4.0810e+01_rb,2.7914e+01_rb,1.4881e+01_rb,2.4491e-01_rb/)
      kbo(:, 1,25,11) = (/ &
     &1.7913e+01_rb,1.3599e+01_rb,9.4137e+00_rb,5.0616e+00_rb,1.1244e-01_rb/)
      kbo(:, 2,25,11) = (/ &
     &2.4002e+01_rb,1.8218e+01_rb,1.2577e+01_rb,6.7433e+00_rb,1.3574e-01_rb/)
      kbo(:, 3,25,11) = (/ &
     &3.1373e+01_rb,2.3808e+01_rb,1.6390e+01_rb,8.7732e+00_rb,1.6587e-01_rb/)
      kbo(:, 4,25,11) = (/ &
     &4.0212e+01_rb,3.0510e+01_rb,2.0949e+01_rb,1.1182e+01_rb,1.9353e-01_rb/)
      kbo(:, 5,25,11) = (/ &
     &5.0507e+01_rb,3.8314e+01_rb,2.6252e+01_rb,1.3993e+01_rb,2.2689e-01_rb/)
      kbo(:, 1,26,11) = (/ &
     &1.6610e+01_rb,1.2648e+01_rb,8.7899e+00_rb,4.7180e+00_rb,1.0410e-01_rb/)
      kbo(:, 2,26,11) = (/ &
     &2.2395e+01_rb,1.7048e+01_rb,1.1804e+01_rb,6.3257e+00_rb,1.2812e-01_rb/)
      kbo(:, 3,26,11) = (/ &
     &2.9495e+01_rb,2.2453e+01_rb,1.5498e+01_rb,8.2851e+00_rb,1.5321e-01_rb/)
      kbo(:, 4,26,11) = (/ &
     &3.7868e+01_rb,2.8829e+01_rb,1.9854e+01_rb,1.0593e+01_rb,1.8381e-01_rb/)
      kbo(:, 5,26,11) = (/ &
     &4.7321e+01_rb,3.6030e+01_rb,2.4795e+01_rb,1.3247e+01_rb,2.1577e-01_rb/)
      kbo(:, 1,27,11) = (/ &
     &1.5552e+01_rb,1.1893e+01_rb,8.2787e+00_rb,4.4401e+00_rb,9.5478e-02_rb/)
      kbo(:, 2,27,11) = (/ &
     &2.1129e+01_rb,1.6157e+01_rb,1.1211e+01_rb,5.9970e+00_rb,1.1930e-01_rb/)
      kbo(:, 3,27,11) = (/ &
     &2.7798e+01_rb,2.1248e+01_rb,1.4714e+01_rb,7.8588e+00_rb,1.4147e-01_rb/)
      kbo(:, 4,27,11) = (/ &
     &3.5437e+01_rb,2.7084e+01_rb,1.8740e+01_rb,1.0024e+01_rb,1.6966e-01_rb/)
      kbo(:, 5,27,11) = (/ &
     &4.4196e+01_rb,3.3778e+01_rb,2.3350e+01_rb,1.2494e+01_rb,2.0142e-01_rb/)
      kbo(:, 1,28,11) = (/ &
     &1.4683e+01_rb,1.1292e+01_rb,7.8718e+00_rb,4.2106e+00_rb,8.7356e-02_rb/)
      kbo(:, 2,28,11) = (/ &
     &1.9886e+01_rb,1.5281e+01_rb,1.0638e+01_rb,5.6861e+00_rb,1.0714e-01_rb/)
      kbo(:, 3,28,11) = (/ &
     &2.5973e+01_rb,1.9937e+01_rb,1.3871e+01_rb,7.4210e+00_rb,1.3061e-01_rb/)
      kbo(:, 4,28,11) = (/ &
     &3.3032e+01_rb,2.5345e+01_rb,1.7630e+01_rb,9.4399e+00_rb,1.5744e-01_rb/)
      kbo(:, 5,28,11) = (/ &
     &4.1210e+01_rb,3.1582e+01_rb,2.1933e+01_rb,1.1759e+01_rb,1.8456e-01_rb/)
      kbo(:, 1,29,11) = (/ &
     &1.3790e+01_rb,1.0662e+01_rb,7.4451e+00_rb,3.9766e+00_rb,7.8021e-02_rb/)
      kbo(:, 2,29,11) = (/ &
     &1.8507e+01_rb,1.4288e+01_rb,9.9758e+00_rb,5.3363e+00_rb,9.5820e-02_rb/)
      kbo(:, 3,29,11) = (/ &
     &2.4091e+01_rb,1.8575e+01_rb,1.2977e+01_rb,6.9398e+00_rb,1.1898e-01_rb/)
      kbo(:, 4,29,11) = (/ &
     &3.0652e+01_rb,2.3593e+01_rb,1.6474e+01_rb,8.8251e+00_rb,1.4373e-01_rb/)
      kbo(:, 5,29,11) = (/ &
     &3.8268e+01_rb,2.9419e+01_rb,2.0534e+01_rb,1.1026e+01_rb,1.6837e-01_rb/)
      kbo(:, 1,30,11) = (/ &
     &1.2835e+01_rb,9.9842e+00_rb,6.9893e+00_rb,3.7286e+00_rb,6.9765e-02_rb/)
      kbo(:, 2,30,11) = (/ &
     &1.7126e+01_rb,1.3305e+01_rb,9.3238e+00_rb,4.9785e+00_rb,8.7990e-02_rb/)
      kbo(:, 3,30,11) = (/ &
     &2.2319e+01_rb,1.7278e+01_rb,1.2100e+01_rb,6.4728e+00_rb,1.0830e-01_rb/)
      kbo(:, 4,30,11) = (/ &
     &2.8412e+01_rb,2.1956e+01_rb,1.5383e+01_rb,8.2493e+00_rb,1.2854e-01_rb/)
      kbo(:, 5,30,11) = (/ &
     &3.5540e+01_rb,2.7422e+01_rb,1.9221e+01_rb,1.0332e+01_rb,1.5041e-01_rb/)
      kbo(:, 1,31,11) = (/ &
     &1.1866e+01_rb,9.2840e+00_rb,6.5153e+00_rb,3.4693e+00_rb,6.2880e-02_rb/)
      kbo(:, 2,31,11) = (/ &
     &1.5808e+01_rb,1.2342e+01_rb,8.6596e+00_rb,4.6193e+00_rb,7.8616e-02_rb/)
      kbo(:, 3,31,11) = (/ &
     &2.0587e+01_rb,1.6022e+01_rb,1.1250e+01_rb,6.0171e+00_rb,9.4908e-02_rb/)
      kbo(:, 4,31,11) = (/ &
     &2.6251e+01_rb,2.0378e+01_rb,1.4318e+01_rb,7.6818e+00_rb,1.1384e-01_rb/)
      kbo(:, 5,31,11) = (/ &
     &3.2893e+01_rb,2.5487e+01_rb,1.7928e+01_rb,9.6467e+00_rb,1.3241e-01_rb/)
      kbo(:, 1,32,11) = (/ &
     &1.1095e+01_rb,8.7224e+00_rb,6.1226e+00_rb,3.2519e+00_rb,5.6839e-02_rb/)
      kbo(:, 2,32,11) = (/ &
     &1.4751e+01_rb,1.1560e+01_rb,8.1226e+00_rb,4.3325e+00_rb,7.0855e-02_rb/)
      kbo(:, 3,32,11) = (/ &
     &1.9205e+01_rb,1.5015e+01_rb,1.0554e+01_rb,5.6424e+00_rb,8.5721e-02_rb/)
      kbo(:, 4,32,11) = (/ &
     &2.4504e+01_rb,1.9119e+01_rb,1.3461e+01_rb,7.2214e+00_rb,1.0137e-01_rb/)
      kbo(:, 5,32,11) = (/ &
     &3.0723e+01_rb,2.3918e+01_rb,1.6887e+01_rb,9.0922e+00_rb,1.2101e-01_rb/)
      kbo(:, 1,33,11) = (/ &
     &1.0469e+01_rb,8.2653e+00_rb,5.8027e+00_rb,3.0803e+00_rb,5.1389e-02_rb/)
      kbo(:, 2,33,11) = (/ &
     &1.3905e+01_rb,1.0963e+01_rb,7.7038e+00_rb,4.0998e+00_rb,6.3408e-02_rb/)
      kbo(:, 3,33,11) = (/ &
     &1.8080e+01_rb,1.4209e+01_rb,9.9969e+00_rb,5.3421e+00_rb,7.7128e-02_rb/)
      kbo(:, 4,33,11) = (/ &
     &2.3055e+01_rb,1.8085e+01_rb,1.2765e+01_rb,6.8504e+00_rb,9.2141e-02_rb/)
      kbo(:, 5,33,11) = (/ &
     &2.8901e+01_rb,2.2629e+01_rb,1.6031e+01_rb,8.6400e+00_rb,1.0949e-01_rb/)
      kbo(:, 1,34,11) = (/ &
     &9.9524e+00_rb,7.8988e+00_rb,5.5437e+00_rb,2.9382e+00_rb,4.7321e-02_rb/)
      kbo(:, 2,34,11) = (/ &
     &1.3224e+01_rb,1.0461e+01_rb,7.3463e+00_rb,3.9087e+00_rb,5.8418e-02_rb/)
      kbo(:, 3,34,11) = (/ &
     &1.7166e+01_rb,1.3560e+01_rb,9.5537e+00_rb,5.1034e+00_rb,7.0181e-02_rb/)
      kbo(:, 4,34,11) = (/ &
     &2.1887e+01_rb,1.7264e+01_rb,1.2211e+01_rb,6.5526e+00_rb,8.4706e-02_rb/)
      kbo(:, 5,34,11) = (/ &
     &2.7487e+01_rb,2.1622e+01_rb,1.5350e+01_rb,8.2698e+00_rb,9.8588e-02_rb/)
      kbo(:, 1,35,11) = (/ &
     &9.2896e+00_rb,7.3988e+00_rb,5.1836e+00_rb,2.7478e+00_rb,4.1417e-02_rb/)
      kbo(:, 2,35,11) = (/ &
     &1.2346e+01_rb,9.8120e+00_rb,6.8895e+00_rb,3.6621e+00_rb,5.1614e-02_rb/)
      kbo(:, 3,35,11) = (/ &
     &1.6038e+01_rb,1.2737e+01_rb,8.9856e+00_rb,4.7992e+00_rb,6.2173e-02_rb/)
      kbo(:, 4,35,11) = (/ &
     &2.0501e+01_rb,1.6246e+01_rb,1.1507e+01_rb,6.1715e+00_rb,7.4346e-02_rb/)
      kbo(:, 5,35,11) = (/ &
     &2.5865e+01_rb,2.0429e+01_rb,1.4515e+01_rb,7.8139e+00_rb,8.6989e-02_rb/)
      kbo(:, 1,36,11) = (/ &
     &8.4641e+00_rb,6.7680e+00_rb,4.7384e+00_rb,2.5089e+00_rb,3.5370e-02_rb/)
      kbo(:, 2,36,11) = (/ &
     &1.1283e+01_rb,9.0077e+00_rb,6.3272e+00_rb,3.3586e+00_rb,4.3427e-02_rb/)
      kbo(:, 3,36,11) = (/ &
     &1.4709e+01_rb,1.1740e+01_rb,8.2852e+00_rb,4.4192e+00_rb,5.3699e-02_rb/)
      kbo(:, 4,36,11) = (/ &
     &1.8892e+01_rb,1.5035e+01_rb,1.0652e+01_rb,5.7082e+00_rb,6.4287e-02_rb/)
      kbo(:, 5,36,11) = (/ &
     &2.3976e+01_rb,1.9028e+01_rb,1.3526e+01_rb,7.2706e+00_rb,7.5870e-02_rb/)
      kbo(:, 1,37,11) = (/ &
     &7.4506e+00_rb,5.9897e+00_rb,4.1905e+00_rb,2.2153e+00_rb,2.9963e-02_rb/)
      kbo(:, 2,37,11) = (/ &
     &1.0029e+01_rb,8.0389e+00_rb,5.6426e+00_rb,2.9898e+00_rb,3.7220e-02_rb/)
      kbo(:, 3,37,11) = (/ &
     &1.3144e+01_rb,1.0534e+01_rb,7.4309e+00_rb,3.9559e+00_rb,4.6416e-02_rb/)
      kbo(:, 4,37,11) = (/ &
     &1.6999e+01_rb,1.3585e+01_rb,9.6217e+00_rb,5.1490e+00_rb,5.5486e-02_rb/)
      kbo(:, 5,37,11) = (/ &
     &2.1741e+01_rb,1.7327e+01_rb,1.2322e+01_rb,6.6152e+00_rb,6.5816e-02_rb/)
      kbo(:, 1,38,11) = (/ &
     &6.5566e+00_rb,5.3014e+00_rb,3.7057e+00_rb,1.9579e+00_rb,2.5356e-02_rb/)
      kbo(:, 2,38,11) = (/ &
     &8.9151e+00_rb,7.1841e+00_rb,5.0400e+00_rb,2.6656e+00_rb,3.1891e-02_rb/)
      kbo(:, 3,38,11) = (/ &
     &1.1778e+01_rb,9.4689e+00_rb,6.6768e+00_rb,3.5488e+00_rb,3.9709e-02_rb/)
      kbo(:, 4,38,11) = (/ &
     &1.5333e+01_rb,1.2304e+01_rb,8.7122e+00_rb,4.6542e+00_rb,4.7360e-02_rb/)
      kbo(:, 5,38,11) = (/ &
     &1.9762e+01_rb,1.5812e+01_rb,1.1244e+01_rb,6.0319e+00_rb,5.6650e-02_rb/)
      kbo(:, 1,39,11) = (/ &
     &5.7815e+00_rb,4.6987e+00_rb,3.2839e+00_rb,1.7336e+00_rb,2.1361e-02_rb/)
      kbo(:, 2,39,11) = (/ &
     &7.9403e+00_rb,6.4302e+00_rb,4.5109e+00_rb,2.3848e+00_rb,2.6909e-02_rb/)
      kbo(:, 3,39,11) = (/ &
     &1.0589e+01_rb,8.5486e+00_rb,6.0232e+00_rb,3.1966e+00_rb,3.3274e-02_rb/)
      kbo(:, 4,39,11) = (/ &
     &1.3890e+01_rb,1.1188e+01_rb,7.9191e+00_rb,4.2244e+00_rb,4.0390e-02_rb/)
      kbo(:, 5,39,11) = (/ &
     &1.8032e+01_rb,1.4487e+01_rb,1.0297e+01_rb,5.5201e+00_rb,4.8824e-02_rb/)
      kbo(:, 1,40,11) = (/ &
     &4.9663e+00_rb,4.0580e+00_rb,2.8368e+00_rb,1.4961e+00_rb,1.7977e-02_rb/)
      kbo(:, 2,40,11) = (/ &
     &6.9269e+00_rb,5.6391e+00_rb,3.9540e+00_rb,2.0894e+00_rb,2.2949e-02_rb/)
      kbo(:, 3,40,11) = (/ &
     &9.3578e+00_rb,7.5891e+00_rb,5.3452e+00_rb,2.8327e+00_rb,2.8350e-02_rb/)
      kbo(:, 4,40,11) = (/ &
     &1.2407e+01_rb,1.0028e+01_rb,7.0894e+00_rb,3.7744e+00_rb,3.4399e-02_rb/)
      kbo(:, 5,40,11) = (/ &
     &1.6240e+01_rb,1.3091e+01_rb,9.2991e+00_rb,4.9787e+00_rb,4.2027e-02_rb/)
      kbo(:, 1,41,11) = (/ &
     &4.2485e+00_rb,3.4862e+00_rb,2.4352e+00_rb,1.2849e+00_rb,1.5088e-02_rb/)
      kbo(:, 2,41,11) = (/ &
     &6.0166e+00_rb,4.9271e+00_rb,3.4537e+00_rb,1.8243e+00_rb,1.9453e-02_rb/)
      kbo(:, 3,41,11) = (/ &
     &8.2485e+00_rb,6.7221e+00_rb,4.7321e+00_rb,2.5060e+00_rb,2.4198e-02_rb/)
      kbo(:, 4,41,11) = (/ &
     &1.1066e+01_rb,8.9791e+00_rb,6.3459e+00_rb,3.3734e+00_rb,2.9570e-02_rb/)
      kbo(:, 5,41,11) = (/ &
     &1.4636e+01_rb,1.1834e+01_rb,8.3996e+00_rb,4.4886e+00_rb,3.6128e-02_rb/)
      kbo(:, 1,42,11) = (/ &
     &3.6346e+00_rb,2.9948e+00_rb,2.0896e+00_rb,1.1018e+00_rb,1.2599e-02_rb/)
      kbo(:, 2,42,11) = (/ &
     &5.2244e+00_rb,4.2974e+00_rb,3.0116e+00_rb,1.5911e+00_rb,1.6323e-02_rb/)
      kbo(:, 3,42,11) = (/ &
     &7.2643e+00_rb,5.9499e+00_rb,4.1877e+00_rb,2.2172e+00_rb,2.0619e-02_rb/)
      kbo(:, 4,42,11) = (/ &
     &9.8747e+00_rb,8.0447e+00_rb,5.6854e+00_rb,3.0206e+00_rb,2.5282e-02_rb/)
      kbo(:, 5,42,11) = (/ &
     &1.3206e+01_rb,1.0716e+01_rb,7.6001e+00_rb,4.0544e+00_rb,3.0876e-02_rb/)
      kbo(:, 1,43,11) = (/ &
     &3.0660e+00_rb,2.5370e+00_rb,1.7687e+00_rb,9.3216e-01_rb,1.0308e-02_rb/)
      kbo(:, 2,43,11) = (/ &
     &4.4856e+00_rb,3.7056e+00_rb,2.5945e+00_rb,1.3700e+00_rb,1.3601e-02_rb/)
      kbo(:, 3,43,11) = (/ &
     &6.3372e+00_rb,5.2140e+00_rb,3.6676e+00_rb,1.9419e+00_rb,1.7306e-02_rb/)
      kbo(:, 4,43,11) = (/ &
     &8.7409e+00_rb,7.1536e+00_rb,5.0545e+00_rb,2.6839e+00_rb,2.1526e-02_rb/)
      kbo(:, 5,43,11) = (/ &
     &1.1846e+01_rb,9.6432e+00_rb,6.8374e+00_rb,3.6437e+00_rb,2.6238e-02_rb/)
      kbo(:, 1,44,11) = (/ &
     &2.5664e+00_rb,2.1306e+00_rb,1.4853e+00_rb,7.8185e-01_rb,8.3390e-03_rb/)
      kbo(:, 2,44,11) = (/ &
     &3.8265e+00_rb,3.1748e+00_rb,2.2204e+00_rb,1.1717e+00_rb,1.1111e-02_rb/)
      kbo(:, 3,44,11) = (/ &
     &5.5006e+00_rb,4.5418e+00_rb,3.1915e+00_rb,1.6892e+00_rb,1.4312e-02_rb/)
      kbo(:, 4,44,11) = (/ &
     &7.6990e+00_rb,6.3295e+00_rb,4.4707e+00_rb,2.3728e+00_rb,1.8022e-02_rb/)
      kbo(:, 5,44,11) = (/ &
     &1.0590e+01_rb,8.6508e+00_rb,6.1308e+00_rb,3.2656e+00_rb,2.2284e-02_rb/)
      kbo(:, 1,45,11) = (/ &
     &2.1449e+00_rb,1.7852e+00_rb,1.2434e+00_rb,6.5459e-01_rb,6.5374e-03_rb/)
      kbo(:, 2,45,11) = (/ &
     &3.2586e+00_rb,2.7120e+00_rb,1.8962e+00_rb,1.0002e+00_rb,8.9661e-03_rb/)
      kbo(:, 3,45,11) = (/ &
     &4.7677e+00_rb,3.9512e+00_rb,2.7729e+00_rb,1.4668e+00_rb,1.1723e-02_rb/)
      kbo(:, 4,45,11) = (/ &
     &6.7759e+00_rb,5.5907e+00_rb,3.9459e+00_rb,2.0943e+00_rb,1.4893e-02_rb/)
      kbo(:, 5,45,11) = (/ &
     &9.4617e+00_rb,7.7574e+00_rb,5.4954e+00_rb,2.9257e+00_rb,1.8879e-02_rb/)
      kbo(:, 1,46,11) = (/ &
     &1.7693e+00_rb,1.4778e+00_rb,1.0286e+00_rb,5.4110e-01_rb,5.0474e-03_rb/)
      kbo(:, 2,46,11) = (/ &
     &2.7478e+00_rb,2.2930e+00_rb,1.6009e+00_rb,8.4457e-01_rb,7.0726e-03_rb/)
      kbo(:, 3,46,11) = (/ &
     &4.0941e+00_rb,3.4053e+00_rb,2.3875e+00_rb,1.2626e+00_rb,9.5023e-03_rb/)
      kbo(:, 4,46,11) = (/ &
     &5.9189e+00_rb,4.8999e+00_rb,3.4535e+00_rb,1.8324e+00_rb,1.2322e-02_rb/)
      kbo(:, 5,46,11) = (/ &
     &8.3887e+00_rb,6.9026e+00_rb,4.8893e+00_rb,2.6020e+00_rb,1.5617e-02_rb/)
      kbo(:, 1,47,11) = (/ &
     &1.4241e+00_rb,1.1941e+00_rb,8.3002e-01_rb,4.3643e-01_rb,3.7828e-03_rb/)
      kbo(:, 2,47,11) = (/ &
     &2.2698e+00_rb,1.8986e+00_rb,1.3247e+00_rb,6.9822e-01_rb,5.4676e-03_rb/)
      kbo(:, 3,47,11) = (/ &
     &3.4547e+00_rb,2.8836e+00_rb,2.0196e+00_rb,1.0678e+00_rb,7.6213e-03_rb/)
      kbo(:, 4,47,11) = (/ &
     &5.0871e+00_rb,4.2288e+00_rb,2.9766e+00_rb,1.5778e+00_rb,9.9658e-03_rb/)
      kbo(:, 5,47,11) = (/ &
     &7.3321e+00_rb,6.0542e+00_rb,4.2837e+00_rb,2.2790e+00_rb,1.3055e-02_rb/)
      kbo(:, 1,48,11) = (/ &
     &1.1337e+00_rb,9.5606e-01_rb,6.6479e-01_rb,3.4941e-01_rb,2.8319e-03_rb/)
      kbo(:, 2,48,11) = (/ &
     &1.8661e+00_rb,1.5653e+00_rb,1.0902e+00_rb,5.7432e-01_rb,4.1758e-03_rb/)
      kbo(:, 3,48,11) = (/ &
     &2.9045e+00_rb,2.4321e+00_rb,1.7012e+00_rb,8.9870e-01_rb,5.9617e-03_rb/)
      kbo(:, 4,48,11) = (/ &
     &4.3570e+00_rb,3.6357e+00_rb,2.5565e+00_rb,1.3546e+00_rb,7.9840e-03_rb/)
      kbo(:, 5,48,11) = (/ &
     &6.3922e+00_rb,5.2980e+00_rb,3.7432e+00_rb,1.9904e+00_rb,1.0519e-02_rb/)
      kbo(:, 1,49,11) = (/ &
     &8.9211e-01_rb,7.5781e-01_rb,5.2674e-01_rb,2.7699e-01_rb,2.0587e-03_rb/)
      kbo(:, 2,49,11) = (/ &
     &1.5203e+00_rb,1.2812e+00_rb,8.9206e-01_rb,4.6970e-01_rb,3.1077e-03_rb/)
      kbo(:, 3,49,11) = (/ &
     &2.4343e+00_rb,2.0415e+00_rb,1.4265e+00_rb,7.5292e-01_rb,4.5062e-03_rb/)
      kbo(:, 4,49,11) = (/ &
     &3.7205e+00_rb,3.1142e+00_rb,2.1868e+00_rb,1.1585e+00_rb,6.1377e-03_rb/)
      kbo(:, 5,49,11) = (/ &
     &5.5564e+00_rb,4.6243e+00_rb,3.2638e+00_rb,1.7346e+00_rb,8.3879e-03_rb/)
      kbo(:, 1,50,11) = (/ &
     &6.9944e-01_rb,5.9730e-01_rb,4.1544e-01_rb,2.1892e-01_rb,1.5159e-03_rb/)
      kbo(:, 2,50,11) = (/ &
     &1.2386e+00_rb,1.0495e+00_rb,7.3105e-01_rb,3.8493e-01_rb,2.3639e-03_rb/)
      kbo(:, 3,50,11) = (/ &
     &2.0475e+00_rb,1.7233e+00_rb,1.2030e+00_rb,6.3443e-01_rb,3.5129e-03_rb/)
      kbo(:, 4,50,11) = (/ &
     &3.1987e+00_rb,2.6838e+00_rb,1.8828e+00_rb,9.9721e-01_rb,4.9131e-03_rb/)
      kbo(:, 5,50,11) = (/ &
     &4.8613e+00_rb,4.0604e+00_rb,2.8638e+00_rb,1.5216e+00_rb,6.8102e-03_rb/)
      kbo(:, 1,51,11) = (/ &
     &5.4324e-01_rb,4.6725e-01_rb,3.2546e-01_rb,1.7172e-01_rb,1.1297e-03_rb/)
      kbo(:, 2,51,11) = (/ &
     &1.0054e+00_rb,8.5632e-01_rb,5.9623e-01_rb,3.1426e-01_rb,1.7844e-03_rb/)
      kbo(:, 3,51,11) = (/ &
     &1.7180e+00_rb,1.4523e+00_rb,1.0136e+00_rb,5.3456e-01_rb,2.7288e-03_rb/)
      kbo(:, 4,51,11) = (/ &
     &2.7560e+00_rb,2.3178e+00_rb,1.6244e+00_rb,8.6011e-01_rb,3.9147e-03_rb/)
      kbo(:, 5,51,11) = (/ &
     &4.2636e+00_rb,3.5725e+00_rb,2.5176e+00_rb,1.3376e+00_rb,5.4601e-03_rb/)
      kbo(:, 1,52,11) = (/ &
     &4.1659e-01_rb,3.6012e-01_rb,2.5102e-01_rb,1.3270e-01_rb,8.3775e-04_rb/)
      kbo(:, 2,52,11) = (/ &
     &8.0451e-01_rb,6.9074e-01_rb,4.8148e-01_rb,2.5404e-01_rb,1.3424e-03_rb/)
      kbo(:, 3,52,11) = (/ &
     &1.4303e+00_rb,1.2146e+00_rb,8.4783e-01_rb,4.4730e-01_rb,2.0961e-03_rb/)
      kbo(:, 4,52,11) = (/ &
     &2.3661e+00_rb,1.9968e+00_rb,1.3985e+00_rb,7.3984e-01_rb,3.0912e-03_rb/)
      kbo(:, 5,52,11) = (/ &
     &3.7364e+00_rb,3.1393e+00_rb,2.2101e+00_rb,1.1738e+00_rb,4.3230e-03_rb/)
      kbo(:, 1,53,11) = (/ &
     &3.1784e-01_rb,2.7470e-01_rb,1.9112e-01_rb,1.0099e-01_rb,6.0100e-04_rb/)
      kbo(:, 2,53,11) = (/ &
     &6.3733e-01_rb,5.5043e-01_rb,3.8376e-01_rb,2.0263e-01_rb,1.0152e-03_rb/)
      kbo(:, 3,53,11) = (/ &
     &1.1810e+00_rb,1.0077e+00_rb,7.0333e-01_rb,3.7118e-01_rb,1.5799e-03_rb/)
      kbo(:, 4,53,11) = (/ &
     &2.0184e+00_rb,1.7109e+00_rb,1.1985e+00_rb,6.3377e-01_rb,2.3636e-03_rb/)
      kbo(:, 5,53,11) = (/ &
     &3.2693e+00_rb,2.7525e+00_rb,1.9355e+00_rb,1.0278e+00_rb,3.4086e-03_rb/)
      kbo(:, 1,54,11) = (/ &
     &2.4587e-01_rb,2.1298e-01_rb,1.4829e-01_rb,7.8206e-02_rb,4.4874e-04_rb/)
      kbo(:, 2,54,11) = (/ &
     &5.0524e-01_rb,4.3896e-01_rb,3.0674e-01_rb,1.6236e-01_rb,7.8054e-04_rb/)
      kbo(:, 3,54,11) = (/ &
     &9.7870e-01_rb,8.4204e-01_rb,5.8784e-01_rb,3.1052e-01_rb,1.2701e-03_rb/)
      kbo(:, 4,54,11) = (/ &
     &1.7368e+00_rb,1.4787e+00_rb,1.0357e+00_rb,5.4775e-01_rb,1.9340e-03_rb/)
      kbo(:, 5,54,11) = (/ &
     &2.8933e+00_rb,2.4419e+00_rb,1.7163e+00_rb,9.1082e-01_rb,2.8443e-03_rb/)
      kbo(:, 1,55,11) = (/ &
     &1.9418e-01_rb,1.6698e-01_rb,1.1582e-01_rb,6.0924e-02_rb,3.5209e-04_rb/)
      kbo(:, 2,55,11) = (/ &
     &3.9791e-01_rb,3.4892e-01_rb,2.4392e-01_rb,1.2929e-01_rb,6.3188e-04_rb/)
      kbo(:, 3,55,11) = (/ &
     &8.0845e-01_rb,6.9946e-01_rb,4.8912e-01_rb,2.5894e-01_rb,1.0591e-03_rb/)
      kbo(:, 4,55,11) = (/ &
     &1.4970e+00_rb,1.2804e+00_rb,8.9678e-01_rb,4.7424e-01_rb,1.6138e-03_rb/)
      kbo(:, 5,55,11) = (/ &
     &2.5700e+00_rb,2.1774e+00_rb,1.5304e+00_rb,8.1150e-01_rb,2.4277e-03_rb/)
      kbo(:, 1,56,11) = (/ &
     &1.5370e-01_rb,1.3145e-01_rb,9.1321e-02_rb,4.8124e-02_rb,2.7531e-04_rb/)
      kbo(:, 2,56,11) = (/ &
     &3.1616e-01_rb,2.7583e-01_rb,1.9245e-01_rb,1.0171e-01_rb,5.0269e-04_rb/)
      kbo(:, 3,56,11) = (/ &
     &6.6199e-01_rb,5.7649e-01_rb,4.0352e-01_rb,2.1386e-01_rb,8.6775e-04_rb/)
      kbo(:, 4,56,11) = (/ &
     &1.2822e+00_rb,1.1021e+00_rb,7.7256e-01_rb,4.0891e-01_rb,1.3519e-03_rb/)
      kbo(:, 5,56,11) = (/ &
     &2.2783e+00_rb,1.9368e+00_rb,1.3622e+00_rb,7.2221e-01_rb,2.0801e-03_rb/)
      kbo(:, 1,57,11) = (/ &
     &1.2165e-01_rb,1.0433e-01_rb,7.2303e-02_rb,3.8009e-02_rb,2.1938e-04_rb/)
      kbo(:, 2,57,11) = (/ &
     &2.5198e-01_rb,2.1938e-01_rb,1.5311e-01_rb,8.0877e-02_rb,3.9875e-04_rb/)
      kbo(:, 3,57,11) = (/ &
     &5.3606e-01_rb,4.7039e-01_rb,3.2945e-01_rb,1.7508e-01_rb,6.9954e-04_rb/)
      kbo(:, 4,57,11) = (/ &
     &1.0880e+00_rb,9.4273e-01_rb,6.6142e-01_rb,3.5027e-01_rb,1.1454e-03_rb/)
      kbo(:, 5,57,11) = (/ &
     &2.0139e+00_rb,1.7191e+00_rb,1.2097e+00_rb,6.4100e-01_rb,1.7901e-03_rb/)
      kbo(:, 1,58,11) = (/ &
     &4.6919e-02_rb,4.6268e-02_rb,3.7584e-02_rb,2.3911e-02_rb,1.7988e-04_rb/)
      kbo(:, 2,58,11) = (/ &
     &9.9628e-02_rb,9.8832e-02_rb,8.0744e-02_rb,5.1438e-02_rb,3.2482e-04_rb/)
      kbo(:, 3,58,11) = (/ &
     &2.1097e-01_rb,2.1358e-01_rb,1.7570e-01_rb,1.1318e-01_rb,5.8194e-04_rb/)
      kbo(:, 4,58,11) = (/ &
     &4.4837e-01_rb,4.4802e-01_rb,3.6936e-01_rb,2.3727e-01_rb,9.7962e-04_rb/)
      kbo(:, 5,58,11) = (/ &
     &8.6568e-01_rb,8.5230e-01_rb,7.0456e-01_rb,4.5178e-01_rb,1.5789e-03_rb/)
      kbo(:, 1,59,11) = (/ &
     &4.1955e-02_rb,4.1496e-02_rb,3.3924e-02_rb,2.1748e-02_rb,1.7054e-04_rb/)
      kbo(:, 2,59,11) = (/ &
     &9.0096e-02_rb,8.9872e-02_rb,7.3696e-02_rb,4.7378e-02_rb,3.1638e-04_rb/)
      kbo(:, 3,59,11) = (/ &
     &1.9169e-01_rb,1.9390e-01_rb,1.6037e-01_rb,1.0394e-01_rb,5.9374e-04_rb/)
      kbo(:, 4,59,11) = (/ &
     &4.1170e-01_rb,4.1459e-01_rb,3.4386e-01_rb,2.2297e-01_rb,1.0082e-03_rb/)
      kbo(:, 5,59,11) = (/ &
     &8.1839e-01_rb,8.1107e-01_rb,6.7470e-01_rb,4.3624e-01_rb,1.6869e-03_rb/)
      kbo(:, 1,13,12) = (/ &
     &1.0396e+02_rb,7.7973e+01_rb,5.1984e+01_rb,2.5992e+01_rb,2.8896e-01_rb/)
      kbo(:, 2,13,12) = (/ &
     &1.4817e+02_rb,1.1113e+02_rb,7.4089e+01_rb,3.7043e+01_rb,6.4071e-01_rb/)
      kbo(:, 3,13,12) = (/ &
     &2.0061e+02_rb,1.5046e+02_rb,1.0030e+02_rb,5.0152e+01_rb,7.6116e-01_rb/)
      kbo(:, 4,13,12) = (/ &
     &2.6114e+02_rb,1.9586e+02_rb,1.3057e+02_rb,6.5284e+01_rb,1.0553e+00_rb/)
      kbo(:, 5,13,12) = (/ &
     &3.2831e+02_rb,2.4622e+02_rb,1.6415e+02_rb,8.2071e+01_rb,1.4067e+00_rb/)
      kbo(:, 1,14,12) = (/ &
     &1.1135e+02_rb,8.3512e+01_rb,5.5676e+01_rb,2.7839e+01_rb,4.1053e-01_rb/)
      kbo(:, 2,14,12) = (/ &
     &1.5695e+02_rb,1.1771e+02_rb,7.8475e+01_rb,3.9240e+01_rb,6.2372e-01_rb/)
      kbo(:, 3,14,12) = (/ &
     &2.1121e+02_rb,1.5841e+02_rb,1.0560e+02_rb,5.2802e+01_rb,8.0301e-01_rb/)
      kbo(:, 4,14,12) = (/ &
     &2.7313e+02_rb,2.0485e+02_rb,1.3657e+02_rb,6.8283e+01_rb,1.0170e+00_rb/)
      kbo(:, 5,14,12) = (/ &
     &3.4124e+02_rb,2.5592e+02_rb,1.7062e+02_rb,8.5308e+01_rb,1.1064e+00_rb/)
      kbo(:, 1,15,12) = (/ &
     &1.1186e+02_rb,8.3897e+01_rb,5.5935e+01_rb,2.7966e+01_rb,4.8952e-01_rb/)
      kbo(:, 2,15,12) = (/ &
     &1.5638e+02_rb,1.1729e+02_rb,7.8194e+01_rb,3.9097e+01_rb,5.8507e-01_rb/)
      kbo(:, 3,15,12) = (/ &
     &2.0878e+02_rb,1.5659e+02_rb,1.0439e+02_rb,5.2197e+01_rb,6.7687e-01_rb/)
      kbo(:, 4,15,12) = (/ &
     &2.6790e+02_rb,2.0092e+02_rb,1.3395e+02_rb,6.6979e+01_rb,7.8981e-01_rb/)
      kbo(:, 5,15,12) = (/ &
     &3.3305e+02_rb,2.4981e+02_rb,1.6654e+02_rb,8.3272e+01_rb,9.5540e-01_rb/)
      kbo(:, 1,16,12) = (/ &
     &1.0741e+02_rb,8.0552e+01_rb,5.3706e+01_rb,2.6852e+01_rb,4.1874e-01_rb/)
      kbo(:, 2,16,12) = (/ &
     &1.4898e+02_rb,1.1174e+02_rb,7.4489e+01_rb,3.7244e+01_rb,5.0405e-01_rb/)
      kbo(:, 3,16,12) = (/ &
     &1.9721e+02_rb,1.4789e+02_rb,9.8599e+01_rb,4.9301e+01_rb,6.1901e-01_rb/)
      kbo(:, 4,16,12) = (/ &
     &2.5144e+02_rb,1.8859e+02_rb,1.2573e+02_rb,6.2879e+01_rb,7.4947e-01_rb/)
      kbo(:, 5,16,12) = (/ &
     &3.1058e+02_rb,2.3295e+02_rb,1.5529e+02_rb,7.7686e+01_rb,8.7491e-01_rb/)
      kbo(:, 1,17,12) = (/ &
     &9.9838e+01_rb,7.4879e+01_rb,4.9919e+01_rb,2.5012e+01_rb,3.9338e-01_rb/)
      kbo(:, 2,17,12) = (/ &
     &1.3717e+02_rb,1.0287e+02_rb,6.8583e+01_rb,3.4361e+01_rb,4.7893e-01_rb/)
      kbo(:, 3,17,12) = (/ &
     &1.8052e+02_rb,1.3539e+02_rb,9.0259e+01_rb,4.5242e+01_rb,5.7262e-01_rb/)
      kbo(:, 4,17,12) = (/ &
     &2.2880e+02_rb,1.7160e+02_rb,1.1440e+02_rb,5.7395e+01_rb,6.8306e-01_rb/)
      kbo(:, 5,17,12) = (/ &
     &2.8220e+02_rb,2.1165e+02_rb,1.4109e+02_rb,7.0870e+01_rb,7.6022e-01_rb/)
      kbo(:, 1,18,12) = (/ &
     &8.8836e+01_rb,6.6627e+01_rb,4.4421e+01_rb,2.2423e+01_rb,3.7355e-01_rb/)
      kbo(:, 2,18,12) = (/ &
     &1.2136e+02_rb,9.1017e+01_rb,6.0679e+01_rb,3.0651e+01_rb,4.3123e-01_rb/)
      kbo(:, 3,18,12) = (/ &
     &1.5857e+02_rb,1.1893e+02_rb,7.9284e+01_rb,4.0088e+01_rb,5.1039e-01_rb/)
      kbo(:, 4,18,12) = (/ &
     &2.0050e+02_rb,1.5036e+02_rb,1.0025e+02_rb,5.0735e+01_rb,5.8388e-01_rb/)
      kbo(:, 5,18,12) = (/ &
     &2.4631e+02_rb,1.8473e+02_rb,1.2316e+02_rb,6.2388e+01_rb,6.7078e-01_rb/)
      kbo(:, 1,19,12) = (/ &
     &7.8886e+01_rb,5.9167e+01_rb,3.9445e+01_rb,2.0119e+01_rb,3.3866e-01_rb/)
      kbo(:, 2,19,12) = (/ &
     &1.0689e+02_rb,8.0167e+01_rb,5.3444e+01_rb,2.7289e+01_rb,4.0283e-01_rb/)
      kbo(:, 3,19,12) = (/ &
     &1.3921e+02_rb,1.0440e+02_rb,6.9601e+01_rb,3.5553e+01_rb,4.5001e-01_rb/)
      kbo(:, 4,19,12) = (/ &
     &1.7541e+02_rb,1.3156e+02_rb,8.7714e+01_rb,4.4793e+01_rb,5.0843e-01_rb/)
      kbo(:, 5,19,12) = (/ &
     &2.1523e+02_rb,1.6142e+02_rb,1.0764e+02_rb,5.4993e+01_rb,5.9607e-01_rb/)
      kbo(:, 1,20,12) = (/ &
     &7.0328e+01_rb,5.2750e+01_rb,3.5209e+01_rb,1.8081e+01_rb,3.1237e-01_rb/)
      kbo(:, 2,20,12) = (/ &
     &9.4812e+01_rb,7.1110e+01_rb,4.7453e+01_rb,2.4387e+01_rb,3.6655e-01_rb/)
      kbo(:, 3,20,12) = (/ &
     &1.2307e+02_rb,9.2305e+01_rb,6.1607e+01_rb,3.1685e+01_rb,4.1127e-01_rb/)
      kbo(:, 4,20,12) = (/ &
     &1.5482e+02_rb,1.1612e+02_rb,7.7556e+01_rb,3.9922e+01_rb,4.7264e-01_rb/)
      kbo(:, 5,20,12) = (/ &
     &1.8921e+02_rb,1.4190e+02_rb,9.4818e+01_rb,4.8866e+01_rb,5.5288e-01_rb/)
      kbo(:, 1,21,12) = (/ &
     &6.2329e+01_rb,4.6745e+01_rb,3.1314e+01_rb,1.6132e+01_rb,2.8533e-01_rb/)
      kbo(:, 2,21,12) = (/ &
     &8.3734e+01_rb,6.2791e+01_rb,4.2073e+01_rb,2.1717e+01_rb,3.3097e-01_rb/)
      kbo(:, 3,21,12) = (/ &
     &1.0843e+02_rb,8.1321e+01_rb,5.4516e+01_rb,2.8198e+01_rb,3.8205e-01_rb/)
      kbo(:, 4,21,12) = (/ &
     &1.3597e+02_rb,1.0198e+02_rb,6.8398e+01_rb,3.5455e+01_rb,4.6172e-01_rb/)
      kbo(:, 5,21,12) = (/ &
     &1.6566e+02_rb,1.2424e+02_rb,8.3370e+01_rb,4.3320e+01_rb,5.2289e-01_rb/)
      kbo(:, 1,22,12) = (/ &
     &5.5853e+01_rb,4.1896e+01_rb,2.8239e+01_rb,1.4630e+01_rb,2.6479e-01_rb/)
      kbo(:, 2,22,12) = (/ &
     &7.4659e+01_rb,5.5993e+01_rb,3.7758e+01_rb,1.9621e+01_rb,3.0587e-01_rb/)
      kbo(:, 3,22,12) = (/ &
     &9.6287e+01_rb,7.2215e+01_rb,4.8710e+01_rb,2.5388e+01_rb,3.6961e-01_rb/)
      kbo(:, 4,22,12) = (/ &
     &1.2002e+02_rb,9.0023e+01_rb,6.0736e+01_rb,3.1753e+01_rb,4.4273e-01_rb/)
      kbo(:, 5,22,12) = (/ &
     &1.4596e+02_rb,1.0947e+02_rb,7.3878e+01_rb,3.8690e+01_rb,5.0560e-01_rb/)
      kbo(:, 1,23,12) = (/ &
     &4.9987e+01_rb,3.7501e+01_rb,2.5418e+01_rb,1.3325e+01_rb,2.4880e-01_rb/)
      kbo(:, 2,23,12) = (/ &
     &6.6514e+01_rb,4.9903e+01_rb,3.3828e+01_rb,1.7769e+01_rb,2.9089e-01_rb/)
      kbo(:, 3,23,12) = (/ &
     &8.5155e+01_rb,6.3887e+01_rb,4.3325e+01_rb,2.2796e+01_rb,3.6730e-01_rb/)
      kbo(:, 4,23,12) = (/ &
     &1.0588e+02_rb,7.9440e+01_rb,5.3898e+01_rb,2.8390e+01_rb,4.2226e-01_rb/)
      kbo(:, 5,23,12) = (/ &
     &1.2867e+02_rb,9.6554e+01_rb,6.5521e+01_rb,3.4526e+01_rb,4.8786e-01_rb/)
      kbo(:, 1,24,12) = (/ &
     &4.5237e+01_rb,3.3990e+01_rb,2.3082e+01_rb,1.2266e+01_rb,2.3374e-01_rb/)
      kbo(:, 2,24,12) = (/ &
     &5.9731e+01_rb,4.4882e+01_rb,3.0518e+01_rb,1.6222e+01_rb,2.9093e-01_rb/)
      kbo(:, 3,24,12) = (/ &
     &7.6147e+01_rb,5.7227e+01_rb,3.8961e+01_rb,2.0700e+01_rb,3.4607e-01_rb/)
      kbo(:, 4,24,12) = (/ &
     &9.4592e+01_rb,7.1092e+01_rb,4.8449e+01_rb,2.5703e+01_rb,4.0338e-01_rb/)
      kbo(:, 5,24,12) = (/ &
     &1.1508e+02_rb,8.6507e+01_rb,5.8986e+01_rb,3.1241e+01_rb,4.6347e-01_rb/)
      kbo(:, 1,25,12) = (/ &
     &4.1207e+01_rb,3.1054e+01_rb,2.1145e+01_rb,1.1344e+01_rb,2.1959e-01_rb/)
      kbo(:, 2,25,12) = (/ &
     &5.4029e+01_rb,4.0717e+01_rb,2.7774e+01_rb,1.4878e+01_rb,2.7843e-01_rb/)
      kbo(:, 3,25,12) = (/ &
     &6.8735e+01_rb,5.1802e+01_rb,3.5391e+01_rb,1.8919e+01_rb,3.2302e-01_rb/)
      kbo(:, 4,25,12) = (/ &
     &8.5416e+01_rb,6.4368e+01_rb,4.4030e+01_rb,2.3485e+01_rb,3.8140e-01_rb/)
      kbo(:, 5,25,12) = (/ &
     &1.0430e+02_rb,7.8570e+01_rb,5.3767e+01_rb,2.8613e+01_rb,4.3338e-01_rb/)
      kbo(:, 1,26,12) = (/ &
     &3.7870e+01_rb,2.8646e+01_rb,1.9592e+01_rb,1.0560e+01_rb,2.1067e-01_rb/)
      kbo(:, 2,26,12) = (/ &
     &4.9486e+01_rb,3.7425e+01_rb,2.5645e+01_rb,1.3791e+01_rb,2.5277e-01_rb/)
      kbo(:, 3,26,12) = (/ &
     &6.2993e+01_rb,4.7626e+01_rb,3.2666e+01_rb,1.7516e+01_rb,3.0077e-01_rb/)
      kbo(:, 4,26,12) = (/ &
     &7.8659e+01_rb,5.9439e+01_rb,4.0756e+01_rb,2.1814e+01_rb,3.5095e-01_rb/)
      kbo(:, 5,26,12) = (/ &
     &9.6658e+01_rb,7.2997e+01_rb,5.0039e+01_rb,2.6742e+01_rb,4.0727e-01_rb/)
      kbo(:, 1,27,12) = (/ &
     &3.5089e+01_rb,2.6619e+01_rb,1.8325e+01_rb,9.8881e+00_rb,1.9876e-01_rb/)
      kbo(:, 2,27,12) = (/ &
     &4.5899e+01_rb,3.4800e+01_rb,2.3944e+01_rb,1.2890e+01_rb,2.3298e-01_rb/)
      kbo(:, 3,27,12) = (/ &
     &5.8737e+01_rb,4.4506e+01_rb,3.0596e+01_rb,1.6445e+01_rb,2.8191e-01_rb/)
      kbo(:, 4,27,12) = (/ &
     &7.3770e+01_rb,5.5858e+01_rb,3.8369e+01_rb,2.0601e+01_rb,3.2481e-01_rb/)
      kbo(:, 5,27,12) = (/ &
     &9.1107e+01_rb,6.8940e+01_rb,4.7296e+01_rb,2.5356e+01_rb,3.8171e-01_rb/)
      kbo(:, 1,28,12) = (/ &
     &3.2818e+01_rb,2.4943e+01_rb,1.7280e+01_rb,9.3226e+00_rb,1.7817e-01_rb/)
      kbo(:, 2,28,12) = (/ &
     &4.3184e+01_rb,3.2811e+01_rb,2.2668e+01_rb,1.2208e+01_rb,2.1857e-01_rb/)
      kbo(:, 3,28,12) = (/ &
     &5.5616e+01_rb,4.2235e+01_rb,2.9105e+01_rb,1.5656e+01_rb,2.5762e-01_rb/)
      kbo(:, 4,28,12) = (/ &
     &7.0259e+01_rb,5.3319e+01_rb,3.6659e+01_rb,1.9693e+01_rb,3.0371e-01_rb/)
      kbo(:, 5,28,12) = (/ &
     &8.7123e+01_rb,6.6079e+01_rb,4.5396e+01_rb,2.4379e+01_rb,3.5478e-01_rb/)
      kbo(:, 1,29,12) = (/ &
     &3.0859e+01_rb,2.3511e+01_rb,1.6360e+01_rb,8.8280e+00_rb,1.6679e-01_rb/)
      kbo(:, 2,29,12) = (/ &
     &4.0867e+01_rb,3.1128e+01_rb,2.1584e+01_rb,1.1631e+01_rb,2.0451e-01_rb/)
      kbo(:, 3,29,12) = (/ &
     &5.2915e+01_rb,4.0294e+01_rb,2.7845e+01_rb,1.4987e+01_rb,2.3670e-01_rb/)
      kbo(:, 4,29,12) = (/ &
     &6.7028e+01_rb,5.1030e+01_rb,3.5190e+01_rb,1.8944e+01_rb,2.7999e-01_rb/)
      kbo(:, 5,29,12) = (/ &
     &8.3169e+01_rb,6.3273e+01_rb,4.3571e+01_rb,2.3474e+01_rb,3.2963e-01_rb/)
      kbo(:, 1,30,12) = (/ &
     &2.9181e+01_rb,2.2308e+01_rb,1.5570e+01_rb,8.4046e+00_rb,1.5217e-01_rb/)
      kbo(:, 2,30,12) = (/ &
     &3.8810e+01_rb,2.9664e+01_rb,2.0634e+01_rb,1.1131e+01_rb,1.8255e-01_rb/)
      kbo(:, 3,30,12) = (/ &
     &5.0316e+01_rb,3.8456e+01_rb,2.6686e+01_rb,1.4393e+01_rb,2.1489e-01_rb/)
      kbo(:, 4,30,12) = (/ &
     &6.3652e+01_rb,4.8625e+01_rb,3.3675e+01_rb,1.8188e+01_rb,2.5258e-01_rb/)
      kbo(:, 5,30,12) = (/ &
     &7.8956e+01_rb,6.0282e+01_rb,4.1671e+01_rb,2.2569e+01_rb,2.9642e-01_rb/)
      kbo(:, 1,31,12) = (/ &
     &2.7538e+01_rb,2.1159e+01_rb,1.4815e+01_rb,8.0002e+00_rb,1.3523e-01_rb/)
      kbo(:, 2,31,12) = (/ &
     &3.6621e+01_rb,2.8123e+01_rb,1.9646e+01_rb,1.0617e+01_rb,1.6072e-01_rb/)
      kbo(:, 3,31,12) = (/ &
     &4.7347e+01_rb,3.6341e+01_rb,2.5341e+01_rb,1.3718e+01_rb,1.9204e-01_rb/)
      kbo(:, 4,31,12) = (/ &
     &5.9881e+01_rb,4.5936e+01_rb,3.1977e+01_rb,1.7350e+01_rb,2.2658e-01_rb/)
      kbo(:, 5,31,12) = (/ &
     &7.4375e+01_rb,5.6987e+01_rb,3.9602e+01_rb,2.1563e+01_rb,2.6205e-01_rb/)
      kbo(:, 1,32,12) = (/ &
     &2.6147e+01_rb,2.0200e+01_rb,1.4192e+01_rb,7.6720e+00_rb,1.1829e-01_rb/)
      kbo(:, 2,32,12) = (/ &
     &3.4618e+01_rb,2.6728e+01_rb,1.8768e+01_rb,1.0173e+01_rb,1.4448e-01_rb/)
      kbo(:, 3,32,12) = (/ &
     &4.4733e+01_rb,3.4488e+01_rb,2.4181e+01_rb,1.3140e+01_rb,1.7290e-01_rb/)
      kbo(:, 4,32,12) = (/ &
     &5.6644e+01_rb,4.3620e+01_rb,3.0542e+01_rb,1.6653e+01_rb,2.0200e-01_rb/)
      kbo(:, 5,32,12) = (/ &
     &7.0667e+01_rb,5.4331e+01_rb,3.7949e+01_rb,2.0755e+01_rb,2.3023e-01_rb/)
      kbo(:, 1,33,12) = (/ &
     &2.4867e+01_rb,1.9325e+01_rb,1.3642e+01_rb,7.3884e+00_rb,1.0477e-01_rb/)
      kbo(:, 2,33,12) = (/ &
     &3.2857e+01_rb,2.5486e+01_rb,1.7993e+01_rb,9.7748e+00_rb,1.2807e-01_rb/)
      kbo(:, 3,33,12) = (/ &
     &4.2482e+01_rb,3.2899e+01_rb,2.3208e+01_rb,1.2658e+01_rb,1.5122e-01_rb/)
      kbo(:, 4,33,12) = (/ &
     &5.4028e+01_rb,4.1749e+01_rb,2.9378e+01_rb,1.6087e+01_rb,1.7656e-01_rb/)
      kbo(:, 5,33,12) = (/ &
     &6.7699e+01_rb,5.2220e+01_rb,3.6682e+01_rb,2.0157e+01_rb,2.0029e-01_rb/)
      kbo(:, 1,34,12) = (/ &
     &2.3698e+01_rb,1.8505e+01_rb,1.3121e+01_rb,7.1187e+00_rb,9.5888e-02_rb/)
      kbo(:, 2,34,12) = (/ &
     &3.1359e+01_rb,2.4427e+01_rb,1.7335e+01_rb,9.4402e+00_rb,1.1540e-01_rb/)
      kbo(:, 3,34,12) = (/ &
     &4.0701e+01_rb,3.1624e+01_rb,2.2413e+01_rb,1.2266e+01_rb,1.3371e-01_rb/)
      kbo(:, 4,34,12) = (/ &
     &5.2041e+01_rb,4.0338e+01_rb,2.8530e+01_rb,1.5676e+01_rb,1.5490e-01_rb/)
      kbo(:, 5,34,12) = (/ &
     &6.5491e+01_rb,5.0663e+01_rb,3.5768e+01_rb,1.9755e+01_rb,1.8157e-01_rb/)
      kbo(:, 1,35,12) = (/ &
     &2.2108e+01_rb,1.7359e+01_rb,1.2354e+01_rb,6.7085e+00_rb,8.3925e-02_rb/)
      kbo(:, 2,35,12) = (/ &
     &2.9390e+01_rb,2.2999e+01_rb,1.6388e+01_rb,8.9464e+00_rb,9.9962e-02_rb/)
      kbo(:, 3,35,12) = (/ &
     &3.8435e+01_rb,2.9966e+01_rb,2.1324e+01_rb,1.1694e+01_rb,1.1806e-01_rb/)
      kbo(:, 4,35,12) = (/ &
     &4.9418e+01_rb,3.8439e+01_rb,2.7323e+01_rb,1.5063e+01_rb,1.3630e-01_rb/)
      kbo(:, 5,35,12) = (/ &
     &6.2546e+01_rb,4.8535e+01_rb,3.4447e+01_rb,1.9092e+01_rb,1.6585e-01_rb/)
      kbo(:, 1,36,12) = (/ &
     &2.0129e+01_rb,1.5891e+01_rb,1.1346e+01_rb,6.1643e+00_rb,7.2166e-02_rb/)
      kbo(:, 2,36,12) = (/ &
     &2.6967e+01_rb,2.1198e+01_rb,1.5154e+01_rb,8.2858e+00_rb,8.5917e-02_rb/)
      kbo(:, 3,36,12) = (/ &
     &3.5568e+01_rb,2.7847e+01_rb,1.9901e+01_rb,1.0934e+01_rb,1.0193e-01_rb/)
      kbo(:, 4,36,12) = (/ &
     &4.6092e+01_rb,3.5975e+01_rb,2.5697e+01_rb,1.4200e+01_rb,1.2167e-01_rb/)
      kbo(:, 5,36,12) = (/ &
     &5.8785e+01_rb,4.5750e+01_rb,3.2634e+01_rb,1.8141e+01_rb,1.4958e-01_rb/)
      kbo(:, 1,37,12) = (/ &
     &1.7737e+01_rb,1.4074e+01_rb,1.0069e+01_rb,5.4659e+00_rb,6.3045e-02_rb/)
      kbo(:, 2,37,12) = (/ &
     &2.3991e+01_rb,1.8955e+01_rb,1.3586e+01_rb,7.4275e+00_rb,7.4702e-02_rb/)
      kbo(:, 3,37,12) = (/ &
     &3.1985e+01_rb,2.5160e+01_rb,1.8048e+01_rb,9.9281e+00_rb,8.9095e-02_rb/)
      kbo(:, 4,37,12) = (/ &
     &4.1889e+01_rb,3.2824e+01_rb,2.3555e+01_rb,1.3031e+01_rb,1.0656e-01_rb/)
      kbo(:, 5,37,12) = (/ &
     &5.3988e+01_rb,4.2162e+01_rb,3.0205e+01_rb,1.6815e+01_rb,1.3149e-01_rb/)
      kbo(:, 1,38,12) = (/ &
     &1.5671e+01_rb,1.2491e+01_rb,8.9462e+00_rb,4.8489e+00_rb,5.4067e-02_rb/)
      kbo(:, 2,38,12) = (/ &
     &2.1387e+01_rb,1.6974e+01_rb,1.2184e+01_rb,6.6604e+00_rb,6.5509e-02_rb/)
      kbo(:, 3,38,12) = (/ &
     &2.8785e+01_rb,2.2756e+01_rb,1.6373e+01_rb,9.0146e+00_rb,7.7568e-02_rb/)
      kbo(:, 4,38,12) = (/ &
     &3.8117e+01_rb,2.9996e+01_rb,2.1601e+01_rb,1.1967e+01_rb,9.4071e-02_rb/)
      kbo(:, 5,38,12) = (/ &
     &4.9667e+01_rb,3.8921e+01_rb,2.7995e+01_rb,1.5610e+01_rb,1.1469e-01_rb/)
      kbo(:, 1,39,12) = (/ &
     &1.3901e+01_rb,1.1137e+01_rb,7.9846e+00_rb,4.3195e+00_rb,4.6183e-02_rb/)
      kbo(:, 2,39,12) = (/ &
     &1.9150e+01_rb,1.5270e+01_rb,1.0976e+01_rb,5.9905e+00_rb,5.6093e-02_rb/)
      kbo(:, 3,39,12) = (/ &
     &2.6010e+01_rb,2.0653e+01_rb,1.4897e+01_rb,8.2086e+00_rb,6.7656e-02_rb/)
      kbo(:, 4,39,12) = (/ &
     &3.4809e+01_rb,2.7511e+01_rb,1.9872e+01_rb,1.1026e+01_rb,8.1649e-02_rb/)
      kbo(:, 5,39,12) = (/ &
     &4.5848e+01_rb,3.6052e+01_rb,2.6039e+01_rb,1.4537e+01_rb,9.9747e-02_rb/)
      kbo(:, 1,40,12) = (/ &
     &1.2081e+01_rb,9.7243e+00_rb,6.9733e+00_rb,3.7653e+00_rb,3.9581e-02_rb/)
      kbo(:, 2,40,12) = (/ &
     &1.6838e+01_rb,1.3484e+01_rb,9.6953e+00_rb,5.2801e+00_rb,4.8568e-02_rb/)
      kbo(:, 3,40,12) = (/ &
     &2.3104e+01_rb,1.8425e+01_rb,1.3302e+01_rb,7.3218e+00_rb,5.8955e-02_rb/)
      kbo(:, 4,40,12) = (/ &
     &3.1273e+01_rb,2.4818e+01_rb,1.7967e+01_rb,9.9757e+00_rb,7.0859e-02_rb/)
      kbo(:, 5,40,12) = (/ &
     &4.1711e+01_rb,3.2920e+01_rb,2.3848e+01_rb,1.3319e+01_rb,8.6539e-02_rb/)
      kbo(:, 1,41,12) = (/ &
     &1.0485e+01_rb,8.4800e+00_rb,6.0787e+00_rb,3.2769e+00_rb,3.3606e-02_rb/)
      kbo(:, 2,41,12) = (/ &
     &1.4786e+01_rb,1.1894e+01_rb,8.5576e+00_rb,4.6493e+00_rb,4.1457e-02_rb/)
      kbo(:, 3,41,12) = (/ &
     &2.0530e+01_rb,1.6437e+01_rb,1.1871e+01_rb,6.5218e+00_rb,5.0842e-02_rb/)
      kbo(:, 4,41,12) = (/ &
     &2.8093e+01_rb,2.2374e+01_rb,1.6223e+01_rb,9.0068e+00_rb,6.1229e-02_rb/)
      kbo(:, 5,41,12) = (/ &
     &3.7921e+01_rb,3.0039e+01_rb,2.1812e+01_rb,1.2190e+01_rb,7.5282e-02_rb/)
      kbo(:, 1,42,12) = (/ &
     &9.1079e+00_rb,7.3993e+00_rb,5.3023e+00_rb,2.8534e+00_rb,2.8075e-02_rb/)
      kbo(:, 2,42,12) = (/ &
     &1.3005e+01_rb,1.0510e+01_rb,7.5619e+00_rb,4.1019e+00_rb,3.5441e-02_rb/)
      kbo(:, 3,42,12) = (/ &
     &1.8277e+01_rb,1.4697e+01_rb,1.0617e+01_rb,5.8190e+00_rb,4.3089e-02_rb/)
      kbo(:, 4,42,12) = (/ &
     &2.5289e+01_rb,2.0221e+01_rb,1.4670e+01_rb,8.1365e+00_rb,5.2317e-02_rb/)
      kbo(:, 5,42,12) = (/ &
     &3.4535e+01_rb,2.7452e+01_rb,1.9969e+01_rb,1.1164e+01_rb,6.5436e-02_rb/)
      kbo(:, 1,43,12) = (/ &
     &7.8247e+00_rb,6.3834e+00_rb,4.5697e+00_rb,2.4551e+00_rb,2.3117e-02_rb/)
      kbo(:, 2,43,12) = (/ &
     &1.1336e+01_rb,9.1978e+00_rb,6.6143e+00_rb,3.5804e+00_rb,2.9659e-02_rb/)
      kbo(:, 3,43,12) = (/ &
     &1.6137e+01_rb,1.3030e+01_rb,9.4153e+00_rb,5.1449e+00_rb,3.6857e-02_rb/)
      kbo(:, 4,43,12) = (/ &
     &2.2622e+01_rb,1.8150e+01_rb,1.3165e+01_rb,7.2856e+00_rb,4.4557e-02_rb/)
      kbo(:, 5,43,12) = (/ &
     &3.1253e+01_rb,2.4925e+01_rb,1.8144e+01_rb,1.0136e+01_rb,5.5864e-02_rb/)
      kbo(:, 1,44,12) = (/ &
     &6.6782e+00_rb,5.4729e+00_rb,3.9116e+00_rb,2.0984e+00_rb,1.9194e-02_rb/)
      kbo(:, 2,44,12) = (/ &
     &9.8370e+00_rb,8.0127e+00_rb,5.7576e+00_rb,3.1087e+00_rb,2.4741e-02_rb/)
      kbo(:, 3,44,12) = (/ &
     &1.4194e+01_rb,1.1506e+01_rb,8.3119e+00_rb,4.5308e+00_rb,3.1291e-02_rb/)
      kbo(:, 4,44,12) = (/ &
     &2.0172e+01_rb,1.6246e+01_rb,1.1781e+01_rb,6.4997e+00_rb,3.7622e-02_rb/)
      kbo(:, 5,44,12) = (/ &
     &2.8219e+01_rb,2.2570e+01_rb,1.6428e+01_rb,9.1628e+00_rb,4.7831e-02_rb/)
      kbo(:, 1,45,12) = (/ &
     &5.6770e+00_rb,4.6794e+00_rb,3.3425e+00_rb,1.7902e+00_rb,1.5806e-02_rb/)
      kbo(:, 2,45,12) = (/ &
     &8.5322e+00_rb,6.9760e+00_rb,5.0078e+00_rb,2.6991e+00_rb,2.0417e-02_rb/)
      kbo(:, 3,45,12) = (/ &
     &1.2498e+01_rb,1.0162e+01_rb,7.3367e+00_rb,3.9899e+00_rb,2.6093e-02_rb/)
      kbo(:, 4,45,12) = (/ &
     &1.7997e+01_rb,1.4548e+01_rb,1.0552e+01_rb,5.8018e+00_rb,3.2356e-02_rb/)
      kbo(:, 5,45,12) = (/ &
     &2.5510e+01_rb,2.0455e+01_rb,1.4885e+01_rb,8.2823e+00_rb,4.0610e-02_rb/)
      kbo(:, 1,46,12) = (/ &
     &4.7638e+00_rb,3.9496e+00_rb,2.8218e+00_rb,1.5105e+00_rb,1.2604e-02_rb/)
      kbo(:, 2,46,12) = (/ &
     &7.3248e+00_rb,6.0189e+00_rb,4.3157e+00_rb,2.3220e+00_rb,1.6943e-02_rb/)
      kbo(:, 3,46,12) = (/ &
     &1.0931e+01_rb,8.9155e+00_rb,6.4285e+00_rb,3.4858e+00_rb,2.1734e-02_rb/)
      kbo(:, 4,46,12) = (/ &
     &1.5948e+01_rb,1.2933e+01_rb,9.3805e+00_rb,5.1432e+00_rb,2.7407e-02_rb/)
      kbo(:, 5,46,12) = (/ &
     &2.2923e+01_rb,1.8438e+01_rb,1.3407e+01_rb,7.4358e+00_rb,3.4309e-02_rb/)
      kbo(:, 1,47,12) = (/ &
     &3.9080e+00_rb,3.2569e+00_rb,2.3263e+00_rb,1.2446e+00_rb,1.0169e-02_rb/)
      kbo(:, 2,47,12) = (/ &
     &6.1567e+00_rb,5.0883e+00_rb,3.6497e+00_rb,1.9607e+00_rb,1.3733e-02_rb/)
      kbo(:, 3,47,12) = (/ &
     &9.4053e+00_rb,7.6982e+00_rb,5.5432e+00_rb,2.9978e+00_rb,1.8162e-02_rb/)
      kbo(:, 4,47,12) = (/ &
     &1.3941e+01_rb,1.1337e+01_rb,8.2150e+00_rb,4.4897e+00_rb,2.3207e-02_rb/)
      kbo(:, 5,47,12) = (/ &
     &2.0314e+01_rb,1.6396e+01_rb,1.1915e+01_rb,6.5825e+00_rb,2.8974e-02_rb/)
      kbo(:, 1,48,12) = (/ &
     &3.1887e+00_rb,2.6690e+00_rb,1.9049e+00_rb,1.0177e+00_rb,7.9795e-03_rb/)
      kbo(:, 2,48,12) = (/ &
     &5.1425e+00_rb,4.2754e+00_rb,3.0657e+00_rb,1.6476e+00_rb,1.1064e-02_rb/)
      kbo(:, 3,48,12) = (/ &
     &8.0497e+00_rb,6.6225e+00_rb,4.7642e+00_rb,2.5722e+00_rb,1.4743e-02_rb/)
      kbo(:, 4,48,12) = (/ &
     &1.2183e+01_rb,9.9349e+00_rb,7.1838e+00_rb,3.9122e+00_rb,1.9211e-02_rb/)
      kbo(:, 5,48,12) = (/ &
     &1.7984e+01_rb,1.4555e+01_rb,1.0573e+01_rb,5.8220e+00_rb,2.4214e-02_rb/)
      kbo(:, 1,49,12) = (/ &
     &2.5869e+00_rb,2.1741e+00_rb,1.5486e+00_rb,8.2603e-01_rb,6.2330e-03_rb/)
      kbo(:, 2,49,12) = (/ &
     &4.2741e+00_rb,3.5702e+00_rb,2.5591e+00_rb,1.3745e+00_rb,8.6780e-03_rb/)
      kbo(:, 3,49,12) = (/ &
     &6.8485e+00_rb,5.6671e+00_rb,4.0787e+00_rb,2.1997e+00_rb,1.1935e-02_rb/)
      kbo(:, 4,49,12) = (/ &
     &1.0623e+01_rb,8.6880e+00_rb,6.2733e+00_rb,3.4054e+00_rb,1.5780e-02_rb/)
      kbo(:, 5,49,12) = (/ &
     &1.5914e+01_rb,1.2906e+01_rb,9.3651e+00_rb,5.1392e+00_rb,2.0415e-02_rb/)
      kbo(:, 1,50,12) = (/ &
     &2.1098e+00_rb,1.7804e+00_rb,1.2663e+00_rb,6.7426e-01_rb,4.8509e-03_rb/)
      kbo(:, 2,50,12) = (/ &
     &3.5764e+00_rb,2.9999e+00_rb,2.1487e+00_rb,1.1529e+00_rb,6.9665e-03_rb/)
      kbo(:, 3,50,12) = (/ &
     &5.8619e+00_rb,4.8746e+00_rb,3.5094e+00_rb,1.8934e+00_rb,9.7336e-03_rb/)
      kbo(:, 4,50,12) = (/ &
     &9.3128e+00_rb,7.6488e+00_rb,5.5163e+00_rb,2.9881e+00_rb,1.3124e-02_rb/)
      kbo(:, 5,50,12) = (/ &
     &1.4210e+01_rb,1.1543e+01_rb,8.3635e+00_rb,4.5727e+00_rb,1.7352e-02_rb/)
      kbo(:, 1,51,12) = (/ &
     &1.7153e+00_rb,1.4551e+00_rb,1.0341e+00_rb,5.4981e-01_rb,3.8011e-03_rb/)
      kbo(:, 2,51,12) = (/ &
     &2.9969e+00_rb,2.5231e+00_rb,1.8054e+00_rb,9.6765e-01_rb,5.5539e-03_rb/)
      kbo(:, 3,51,12) = (/ &
     &5.0265e+00_rb,4.1972e+00_rb,3.0205e+00_rb,1.6294e+00_rb,8.0318e-03_rb/)
      kbo(:, 4,51,12) = (/ &
     &8.1641e+00_rb,6.7342e+00_rb,4.8593e+00_rb,2.6284e+00_rb,1.0987e-02_rb/)
      kbo(:, 5,51,12) = (/ &
     &1.2731e+01_rb,1.0364e+01_rb,7.4966e+00_rb,4.0842e+00_rb,1.4784e-02_rb/)
      kbo(:, 1,52,12) = (/ &
     &1.3829e+00_rb,1.1793e+00_rb,8.3692e-01_rb,4.4489e-01_rb,2.9111e-03_rb/)
      kbo(:, 2,52,12) = (/ &
     &2.5001e+00_rb,2.1140e+00_rb,1.5110e+00_rb,8.0824e-01_rb,4.4285e-03_rb/)
      kbo(:, 3,52,12) = (/ &
     &4.2993e+00_rb,3.6038e+00_rb,2.5920e+00_rb,1.3972e+00_rb,6.4988e-03_rb/)
      kbo(:, 4,52,12) = (/ &
     &7.1362e+00_rb,5.9093e+00_rb,4.2654e+00_rb,2.3076e+00_rb,9.0777e-03_rb/)
      kbo(:, 5,52,12) = (/ &
     &1.1388e+01_rb,9.2949e+00_rb,6.7113e+00_rb,3.6475e+00_rb,1.2584e-02_rb/)
      kbo(:, 1,53,12) = (/ &
     &1.0996e+00_rb,9.4448e-01_rb,6.6999e-01_rb,3.5618e-01_rb,2.1752e-03_rb/)
      kbo(:, 2,53,12) = (/ &
     &2.0714e+00_rb,1.7598e+00_rb,1.2567e+00_rb,6.7105e-01_rb,3.4164e-03_rb/)
      kbo(:, 3,53,12) = (/ &
     &3.6653e+00_rb,3.0827e+00_rb,2.2148e+00_rb,1.1926e+00_rb,5.1289e-03_rb/)
      kbo(:, 4,53,12) = (/ &
     &6.2207e+00_rb,5.1680e+00_rb,3.7282e+00_rb,2.0166e+00_rb,7.4836e-03_rb/)
      kbo(:, 5,53,12) = (/ &
     &1.0144e+01_rb,8.3054e+00_rb,5.9967e+00_rb,3.2508e+00_rb,1.0390e-02_rb/)
      kbo(:, 1,54,12) = (/ &
     &8.8096e-01_rb,7.6003e-01_rb,5.3828e-01_rb,2.8652e-01_rb,1.7028e-03_rb/)
      kbo(:, 2,54,12) = (/ &
     &1.7342e+00_rb,1.4790e+00_rb,1.0559e+00_rb,5.6372e-01_rb,2.7715e-03_rb/)
      kbo(:, 3,54,12) = (/ &
     &3.1660e+00_rb,2.6700e+00_rb,1.9162e+00_rb,1.0305e+00_rb,4.2409e-03_rb/)
      kbo(:, 4,54,12) = (/ &
     &5.4916e+00_rb,4.5733e+00_rb,3.2967e+00_rb,1.7833e+00_rb,6.3614e-03_rb/)
      kbo(:, 5,54,12) = (/ &
     &9.1316e+00_rb,7.4962e+00_rb,5.4128e+00_rb,2.9325e+00_rb,9.0996e-03_rb/)
      kbo(:, 1,55,12) = (/ &
     &7.0080e-01_rb,6.0941e-01_rb,4.3132e-01_rb,2.2991e-01_rb,1.3497e-03_rb/)
      kbo(:, 2,55,12) = (/ &
     &1.4554e+00_rb,1.2471e+00_rb,8.8937e-01_rb,4.7463e-01_rb,2.3282e-03_rb/)
      kbo(:, 3,55,12) = (/ &
     &2.7458e+00_rb,2.3243e+00_rb,1.6671e+00_rb,8.9493e-01_rb,3.6585e-03_rb/)
      kbo(:, 4,55,12) = (/ &
     &4.8778e+00_rb,4.0704e+00_rb,2.9318e+00_rb,1.5851e+00_rb,5.5673e-03_rb/)
      kbo(:, 5,55,12) = (/ &
     &8.2670e+00_rb,6.7971e+00_rb,4.9063e+00_rb,2.6581e+00_rb,8.0481e-03_rb/)
      kbo(:, 1,56,12) = (/ &
     &5.5050e-01_rb,4.8277e-01_rb,3.4128e-01_rb,1.8181e-01_rb,1.0666e-03_rb/)
      kbo(:, 2,56,12) = (/ &
     &1.2088e+00_rb,1.0426e+00_rb,7.4350e-01_rb,3.9744e-01_rb,1.9421e-03_rb/)
      kbo(:, 3,56,12) = (/ &
     &2.3753e+00_rb,2.0178e+00_rb,1.4463e+00_rb,7.7559e-01_rb,3.1486e-03_rb/)
      kbo(:, 4,56,12) = (/ &
     &4.3336e+00_rb,3.6227e+00_rb,2.6061e+00_rb,1.4075e+00_rb,4.8772e-03_rb/)
      kbo(:, 5,56,12) = (/ &
     &7.4871e+00_rb,6.1648e+00_rb,4.4455e+00_rb,2.4084e+00_rb,7.2633e-03_rb/)
      kbo(:, 1,57,12) = (/ &
     &4.2691e-01_rb,3.7593e-01_rb,2.6606e-01_rb,1.4195e-01_rb,8.3793e-04_rb/)
      kbo(:, 2,57,12) = (/ &
     &9.9567e-01_rb,8.6325e-01_rb,6.1463e-01_rb,3.2859e-01_rb,1.6242e-03_rb/)
      kbo(:, 3,57,12) = (/ &
     &2.0486e+00_rb,1.7449e+00_rb,1.2502e+00_rb,6.7018e-01_rb,2.7399e-03_rb/)
      kbo(:, 4,57,12) = (/ &
     &3.8496e+00_rb,3.2234e+00_rb,2.3159e+00_rb,1.2492e+00_rb,4.2649e-03_rb/)
      kbo(:, 5,57,12) = (/ &
     &6.7867e+00_rb,5.5927e+00_rb,4.0285e+00_rb,2.1823e+00_rb,6.5093e-03_rb/)
      kbo(:, 1,58,12) = (/ &
     &1.6356e-01_rb,1.6482e-01_rb,1.3630e-01_rb,8.7916e-02_rb,6.5877e-04_rb/)
      kbo(:, 2,58,12) = (/ &
     &3.9653e-01_rb,3.9771e-01_rb,3.3219e-01_rb,2.1522e-01_rb,1.3561e-03_rb/)
      kbo(:, 3,58,12) = (/ &
     &8.5978e-01_rb,8.4388e-01_rb,7.0959e-01_rb,4.6009e-01_rb,2.4231e-03_rb/)
      kbo(:, 4,58,12) = (/ &
     &1.6685e+00_rb,1.6071e+00_rb,1.3546e+00_rb,8.8295e-01_rb,3.8944e-03_rb/)
      kbo(:, 5,58,12) = (/ &
     &3.0057e+00_rb,2.8445e+00_rb,2.4022e+00_rb,1.5741e+00_rb,5.9596e-03_rb/)
      kbo(:, 1,59,12) = (/ &
     &1.4551e-01_rb,1.4725e-01_rb,1.2251e-01_rb,7.9561e-02_rb,6.8074e-04_rb/)
      kbo(:, 2,59,12) = (/ &
     &3.6080e-01_rb,3.6451e-01_rb,3.0649e-01_rb,2.0003e-01_rb,1.4324e-03_rb/)
      kbo(:, 3,59,12) = (/ &
     &8.0811e-01_rb,7.9823e-01_rb,6.7489e-01_rb,4.4138e-01_rb,2.5709e-03_rb/)
      kbo(:, 4,59,12) = (/ &
     &1.6058e+00_rb,1.5512e+00_rb,1.3138e+00_rb,8.6244e-01_rb,4.2753e-03_rb/)
      kbo(:, 5,59,12) = (/ &
     &2.9249e+00_rb,2.7730e+00_rb,2.3517e+00_rb,1.5520e+00_rb,6.3367e-03_rb/)
      kbo(:, 1,13,13) = (/ &
     &1.6844e+02_rb,1.2633e+02_rb,8.4219e+01_rb,4.2110e+01_rb,6.4746e-01_rb/)
      kbo(:, 2,13,13) = (/ &
     &2.3974e+02_rb,1.7981e+02_rb,1.1987e+02_rb,5.9937e+01_rb,7.7977e-01_rb/)
      kbo(:, 3,13,13) = (/ &
     &3.2626e+02_rb,2.4469e+02_rb,1.6313e+02_rb,8.1566e+01_rb,1.5118e+00_rb/)
      kbo(:, 4,13,13) = (/ &
     &4.2676e+02_rb,3.2006e+02_rb,2.1338e+02_rb,1.0669e+02_rb,1.6642e+00_rb/)
      kbo(:, 5,13,13) = (/ &
     &5.4119e+02_rb,4.0589e+02_rb,2.7059e+02_rb,1.3530e+02_rb,1.7313e+00_rb/)
      kbo(:, 1,14,13) = (/ &
     &1.9041e+02_rb,1.4281e+02_rb,9.5205e+01_rb,4.7604e+01_rb,4.8593e-01_rb/)
      kbo(:, 2,14,13) = (/ &
     &2.6943e+02_rb,2.0207e+02_rb,1.3472e+02_rb,6.7360e+01_rb,1.0523e+00_rb/)
      kbo(:, 3,14,13) = (/ &
     &3.6326e+02_rb,2.7244e+02_rb,1.8162e+02_rb,9.0813e+01_rb,1.4658e+00_rb/)
      kbo(:, 4,14,13) = (/ &
     &4.7082e+02_rb,3.5312e+02_rb,2.3542e+02_rb,1.1771e+02_rb,1.6120e+00_rb/)
      kbo(:, 5,14,13) = (/ &
     &5.9086e+02_rb,4.4312e+02_rb,2.9542e+02_rb,1.4771e+02_rb,1.8462e+00_rb/)
      kbo(:, 1,15,13) = (/ &
     &2.0331e+02_rb,1.5249e+02_rb,1.0165e+02_rb,5.0828e+01_rb,7.0211e-01_rb/)
      kbo(:, 2,15,13) = (/ &
     &2.8491e+02_rb,2.1369e+02_rb,1.4246e+02_rb,7.1232e+01_rb,1.1193e+00_rb/)
      kbo(:, 3,15,13) = (/ &
     &3.8104e+02_rb,2.8579e+02_rb,1.9054e+02_rb,9.5263e+01_rb,1.3482e+00_rb/)
      kbo(:, 4,15,13) = (/ &
     &4.8986e+02_rb,3.6740e+02_rb,2.4492e+02_rb,1.2246e+02_rb,1.6094e+00_rb/)
      kbo(:, 5,15,13) = (/ &
     &6.0932e+02_rb,4.5699e+02_rb,3.0468e+02_rb,1.5233e+02_rb,1.6798e+00_rb/)
      kbo(:, 1,16,13) = (/ &
     &2.0774e+02_rb,1.5583e+02_rb,1.0388e+02_rb,5.1937e+01_rb,8.3999e-01_rb/)
      kbo(:, 2,16,13) = (/ &
     &2.8883e+02_rb,2.1662e+02_rb,1.4442e+02_rb,7.2209e+01_rb,1.0268e+00_rb/)
      kbo(:, 3,16,13) = (/ &
     &3.8302e+02_rb,2.8729e+02_rb,1.9153e+02_rb,9.5770e+01_rb,1.2054e+00_rb/)
      kbo(:, 4,16,13) = (/ &
     &4.8856e+02_rb,3.6642e+02_rb,2.4427e+02_rb,1.2213e+02_rb,1.2630e+00_rb/)
      kbo(:, 5,16,13) = (/ &
     &6.0471e+02_rb,4.5354e+02_rb,3.0235e+02_rb,1.5118e+02_rb,1.5611e+00_rb/)
      kbo(:, 1,17,13) = (/ &
     &2.0589e+02_rb,1.5442e+02_rb,1.0294e+02_rb,5.1473e+01_rb,7.0898e-01_rb/)
      kbo(:, 2,17,13) = (/ &
     &2.8383e+02_rb,2.1287e+02_rb,1.4191e+02_rb,7.0957e+01_rb,8.7566e-01_rb/)
      kbo(:, 3,17,13) = (/ &
     &3.7354e+02_rb,2.8016e+02_rb,1.8678e+02_rb,9.3389e+01_rb,1.0083e+00_rb/)
      kbo(:, 4,17,13) = (/ &
     &4.7406e+02_rb,3.5553e+02_rb,2.3702e+02_rb,1.1851e+02_rb,1.2409e+00_rb/)
      kbo(:, 5,17,13) = (/ &
     &5.8252e+02_rb,4.3688e+02_rb,2.9126e+02_rb,1.4563e+02_rb,1.5701e+00_rb/)
      kbo(:, 1,18,13) = (/ &
     &1.9631e+02_rb,1.4723e+02_rb,9.8151e+01_rb,4.9076e+01_rb,6.5277e-01_rb/)
      kbo(:, 2,18,13) = (/ &
     &2.6794e+02_rb,2.0095e+02_rb,1.3397e+02_rb,6.6983e+01_rb,7.9253e-01_rb/)
      kbo(:, 3,18,13) = (/ &
     &3.5060e+02_rb,2.6297e+02_rb,1.7531e+02_rb,8.7658e+01_rb,9.7984e-01_rb/)
      kbo(:, 4,18,13) = (/ &
     &4.4214e+02_rb,3.3161e+02_rb,2.2107e+02_rb,1.1054e+02_rb,1.2414e+00_rb/)
      kbo(:, 5,18,13) = (/ &
     &5.4117e+02_rb,4.0586e+02_rb,2.7058e+02_rb,1.3530e+02_rb,1.5487e+00_rb/)
      kbo(:, 1,19,13) = (/ &
     &1.8546e+02_rb,1.3909e+02_rb,9.2728e+01_rb,4.6396e+01_rb,6.1789e-01_rb/)
      kbo(:, 2,19,13) = (/ &
     &2.5191e+02_rb,1.8893e+02_rb,1.2596e+02_rb,6.3020e+01_rb,7.3049e-01_rb/)
      kbo(:, 3,19,13) = (/ &
     &3.2752e+02_rb,2.4563e+02_rb,1.6376e+02_rb,8.1963e+01_rb,9.1418e-01_rb/)
      kbo(:, 4,19,13) = (/ &
     &4.1109e+02_rb,3.0832e+02_rb,2.0555e+02_rb,1.0295e+02_rb,1.1663e+00_rb/)
      kbo(:, 5,19,13) = (/ &
     &5.0069e+02_rb,3.7554e+02_rb,2.5035e+02_rb,1.2551e+02_rb,1.4682e+00_rb/)
      kbo(:, 1,20,13) = (/ &
     &1.7660e+02_rb,1.3245e+02_rb,8.8301e+01_rb,4.4344e+01_rb,5.8624e-01_rb/)
      kbo(:, 2,20,13) = (/ &
     &2.3782e+02_rb,1.7835e+02_rb,1.1891e+02_rb,5.9734e+01_rb,7.0627e-01_rb/)
      kbo(:, 3,20,13) = (/ &
     &3.0729e+02_rb,2.3046e+02_rb,1.5365e+02_rb,7.7248e+01_rb,8.6369e-01_rb/)
      kbo(:, 4,20,13) = (/ &
     &3.8357e+02_rb,2.8768e+02_rb,1.9179e+02_rb,9.6513e+01_rb,1.0255e+00_rb/)
      kbo(:, 5,20,13) = (/ &
     &4.6652e+02_rb,3.4991e+02_rb,2.3327e+02_rb,1.1751e+02_rb,1.3105e+00_rb/)
      kbo(:, 1,21,13) = (/ &
     &1.6552e+02_rb,1.2414e+02_rb,8.2761e+01_rb,4.1834e+01_rb,5.4233e-01_rb/)
      kbo(:, 2,21,13) = (/ &
     &2.2144e+02_rb,1.6608e+02_rb,1.1072e+02_rb,5.6006e+01_rb,6.6551e-01_rb/)
      kbo(:, 3,21,13) = (/ &
     &2.8447e+02_rb,2.1334e+02_rb,1.4223e+02_rb,7.2029e+01_rb,7.8267e-01_rb/)
      kbo(:, 4,21,13) = (/ &
     &3.5426e+02_rb,2.6570e+02_rb,1.7714e+02_rb,8.9791e+01_rb,9.3322e-01_rb/)
      kbo(:, 5,21,13) = (/ &
     &4.3028e+02_rb,3.2271e+02_rb,2.1513e+02_rb,1.0915e+02_rb,1.1469e+00_rb/)
      kbo(:, 1,22,13) = (/ &
     &1.5536e+02_rb,1.1652e+02_rb,7.7681e+01_rb,3.9611e+01_rb,5.1617e-01_rb/)
      kbo(:, 2,22,13) = (/ &
     &2.0596e+02_rb,1.5446e+02_rb,1.0298e+02_rb,5.2572e+01_rb,6.2474e-01_rb/)
      kbo(:, 3,22,13) = (/ &
     &2.6312e+02_rb,1.9733e+02_rb,1.3157e+02_rb,6.7220e+01_rb,7.4033e-01_rb/)
      kbo(:, 4,22,13) = (/ &
     &3.2670e+02_rb,2.4500e+02_rb,1.6337e+02_rb,8.3538e+01_rb,8.8562e-01_rb/)
      kbo(:, 5,22,13) = (/ &
     &3.9539e+02_rb,2.9656e+02_rb,1.9778e+02_rb,1.0126e+02_rb,1.0777e+00_rb/)
      kbo(:, 1,23,13) = (/ &
     &1.4370e+02_rb,1.0779e+02_rb,7.1941e+01_rb,3.6979e+01_rb,5.0881e-01_rb/)
      kbo(:, 2,23,13) = (/ &
     &1.8910e+02_rb,1.4182e+02_rb,9.4675e+01_rb,4.8747e+01_rb,5.9561e-01_rb/)
      kbo(:, 3,23,13) = (/ &
     &2.4065e+02_rb,1.8048e+02_rb,1.2051e+02_rb,6.2134e+01_rb,7.0317e-01_rb/)
      kbo(:, 4,23,13) = (/ &
     &2.9775e+02_rb,2.2332e+02_rb,1.4916e+02_rb,7.6994e+01_rb,8.5956e-01_rb/)
      kbo(:, 5,23,13) = (/ &
     &3.5987e+02_rb,2.6990e+02_rb,1.8034e+02_rb,9.3182e+01_rb,1.0114e+00_rb/)
      kbo(:, 1,24,13) = (/ &
     &1.3242e+02_rb,9.9313e+01_rb,6.6507e+01_rb,3.4403e+01_rb,4.7778e-01_rb/)
      kbo(:, 2,24,13) = (/ &
     &1.7334e+02_rb,1.3001e+02_rb,8.7073e+01_rb,4.5133e+01_rb,5.5652e-01_rb/)
      kbo(:, 3,24,13) = (/ &
     &2.1981e+02_rb,1.6486e+02_rb,1.1045e+02_rb,5.7389e+01_rb,6.8303e-01_rb/)
      kbo(:, 4,24,13) = (/ &
     &2.7150e+02_rb,2.0360e+02_rb,1.3644e+02_rb,7.1035e+01_rb,8.3312e-01_rb/)
      kbo(:, 5,24,13) = (/ &
     &3.2837e+02_rb,2.4630e+02_rb,1.6509e+02_rb,8.6103e+01_rb,9.5638e-01_rb/)
      kbo(:, 1,25,13) = (/ &
     &1.2137e+02_rb,9.1042e+01_rb,6.1224e+01_rb,3.1886e+01_rb,4.5611e-01_rb/)
      kbo(:, 2,25,13) = (/ &
     &1.5823e+02_rb,1.1867e+02_rb,7.9841e+01_rb,4.1707e+01_rb,5.3430e-01_rb/)
      kbo(:, 3,25,13) = (/ &
     &2.0017e+02_rb,1.5012e+02_rb,1.0102e+02_rb,5.2903e+01_rb,6.6826e-01_rb/)
      kbo(:, 4,25,13) = (/ &
     &2.4727e+02_rb,1.8545e+02_rb,1.2481e+02_rb,6.5530e+01_rb,7.9106e-01_rb/)
      kbo(:, 5,25,13) = (/ &
     &2.9868e+02_rb,2.2401e+02_rb,1.5081e+02_rb,7.9363e+01_rb,9.2773e-01_rb/)
      kbo(:, 1,26,13) = (/ &
     &1.1166e+02_rb,8.3750e+01_rb,5.6641e+01_rb,2.9761e+01_rb,4.3741e-01_rb/)
      kbo(:, 2,26,13) = (/ &
     &1.4505e+02_rb,1.0880e+02_rb,7.3586e+01_rb,3.8778e+01_rb,5.3620e-01_rb/)
      kbo(:, 3,26,13) = (/ &
     &1.8336e+02_rb,1.3753e+02_rb,9.3041e+01_rb,4.9168e+01_rb,6.5727e-01_rb/)
      kbo(:, 4,26,13) = (/ &
     &2.2613e+02_rb,1.6961e+02_rb,1.1481e+02_rb,6.0809e+01_rb,7.5877e-01_rb/)
      kbo(:, 5,26,13) = (/ &
     &2.7339e+02_rb,2.0507e+02_rb,1.3883e+02_rb,7.3616e+01_rb,9.0221e-01_rb/)
      kbo(:, 1,27,13) = (/ &
     &1.0286e+02_rb,7.7197e+01_rb,5.2520e+01_rb,2.7881e+01_rb,4.2067e-01_rb/)
      kbo(:, 2,27,13) = (/ &
     &1.3339e+02_rb,1.0010e+02_rb,6.8159e+01_rb,3.6272e+01_rb,5.2994e-01_rb/)
      kbo(:, 3,27,13) = (/ &
     &1.6841e+02_rb,1.2640e+02_rb,8.6087e+01_rb,4.5886e+01_rb,6.2144e-01_rb/)
      kbo(:, 4,27,13) = (/ &
     &2.0796e+02_rb,1.5610e+02_rb,1.0629e+02_rb,5.6702e+01_rb,7.2963e-01_rb/)
      kbo(:, 5,27,13) = (/ &
     &2.5159e+02_rb,1.8888e+02_rb,1.2865e+02_rb,6.8691e+01_rb,8.6563e-01_rb/)
      kbo(:, 1,28,13) = (/ &
     &9.5256e+01_rb,7.1619e+01_rb,4.8936e+01_rb,2.6277e+01_rb,4.0676e-01_rb/)
      kbo(:, 2,28,13) = (/ &
     &1.2337e+02_rb,9.2726e+01_rb,6.3443e+01_rb,3.4072e+01_rb,4.9978e-01_rb/)
      kbo(:, 3,28,13) = (/ &
     &1.5583e+02_rb,1.1712e+02_rb,8.0174e+01_rb,4.3046e+01_rb,5.7218e-01_rb/)
      kbo(:, 4,28,13) = (/ &
     &1.9223e+02_rb,1.4447e+02_rb,9.8926e+01_rb,5.3164e+01_rb,6.9683e-01_rb/)
      kbo(:, 5,28,13) = (/ &
     &2.3315e+02_rb,1.7521e+02_rb,1.1992e+02_rb,6.4418e+01_rb,8.1878e-01_rb/)
      kbo(:, 1,29,13) = (/ &
     &8.8532e+01_rb,6.6695e+01_rb,4.5757e+01_rb,2.4810e+01_rb,3.8492e-01_rb/)
      kbo(:, 2,29,13) = (/ &
     &1.1463e+02_rb,8.6325e+01_rb,5.9300e+01_rb,3.2087e+01_rb,4.4646e-01_rb/)
      kbo(:, 3,29,13) = (/ &
     &1.4438e+02_rb,1.0869e+02_rb,7.4772e+01_rb,4.0468e+01_rb,5.3953e-01_rb/)
      kbo(:, 4,29,13) = (/ &
     &1.7851e+02_rb,1.3434e+02_rb,9.2412e+01_rb,4.9954e+01_rb,6.4107e-01_rb/)
      kbo(:, 5,29,13) = (/ &
     &2.1752e+02_rb,1.6365e+02_rb,1.1246e+02_rb,6.0724e+01_rb,7.6365e-01_rb/)
      kbo(:, 1,30,13) = (/ &
     &8.2710e+01_rb,6.2476e+01_rb,4.3045e+01_rb,2.3511e+01_rb,3.4189e-01_rb/)
      kbo(:, 2,30,13) = (/ &
     &1.0675e+02_rb,8.0586e+01_rb,5.5620e+01_rb,3.0345e+01_rb,4.0578e-01_rb/)
      kbo(:, 3,30,13) = (/ &
     &1.3482e+02_rb,1.0169e+02_rb,7.0226e+01_rb,3.8224e+01_rb,5.0418e-01_rb/)
      kbo(:, 4,30,13) = (/ &
     &1.6752e+02_rb,1.2628e+02_rb,8.7179e+01_rb,4.7334e+01_rb,5.9789e-01_rb/)
      kbo(:, 5,30,13) = (/ &
     &2.0548e+02_rb,1.5477e+02_rb,1.0678e+02_rb,5.7811e+01_rb,7.2304e-01_rb/)
      kbo(:, 1,31,13) = (/ &
     &7.6989e+01_rb,5.8322e+01_rb,4.0414e+01_rb,2.2252e+01_rb,3.0592e-01_rb/)
      kbo(:, 2,31,13) = (/ &
     &9.9596e+01_rb,7.5380e+01_rb,5.2274e+01_rb,2.8707e+01_rb,3.7677e-01_rb/)
      kbo(:, 3,31,13) = (/ &
     &1.2653e+02_rb,9.5656e+01_rb,6.6313e+01_rb,3.6293e+01_rb,4.5864e-01_rb/)
      kbo(:, 4,31,13) = (/ &
     &1.5839e+02_rb,1.1960e+02_rb,8.2835e+01_rb,4.5152e+01_rb,5.5168e-01_rb/)
      kbo(:, 5,31,13) = (/ &
     &1.9525e+02_rb,1.4733e+02_rb,1.0199e+02_rb,5.5418e+01_rb,6.7878e-01_rb/)
      kbo(:, 1,32,13) = (/ &
     &7.2617e+01_rb,5.5198e+01_rb,3.8454e+01_rb,2.1285e+01_rb,2.7831e-01_rb/)
      kbo(:, 2,32,13) = (/ &
     &9.4496e+01_rb,7.1727e+01_rb,4.9927e+01_rb,2.7539e+01_rb,3.4616e-01_rb/)
      kbo(:, 3,32,13) = (/ &
     &1.2095e+02_rb,9.1662e+01_rb,6.3739e+01_rb,3.5032e+01_rb,4.1226e-01_rb/)
      kbo(:, 4,32,13) = (/ &
     &1.5220e+02_rb,1.1521e+02_rb,8.0042e+01_rb,4.3833e+01_rb,5.1531e-01_rb/)
      kbo(:, 5,32,13) = (/ &
     &1.8837e+02_rb,1.4240e+02_rb,9.8836e+01_rb,5.3991e+01_rb,6.4895e-01_rb/)
      kbo(:, 1,33,13) = (/ &
     &6.9586e+01_rb,5.3075e+01_rb,3.7119e+01_rb,2.0620e+01_rb,2.5421e-01_rb/)
      kbo(:, 2,33,13) = (/ &
     &9.1190e+01_rb,6.9432e+01_rb,4.8467e+01_rb,2.6834e+01_rb,3.0964e-01_rb/)
      kbo(:, 3,33,13) = (/ &
     &1.1725e+02_rb,8.9117e+01_rb,6.2134e+01_rb,3.4318e+01_rb,3.8320e-01_rb/)
      kbo(:, 4,33,13) = (/ &
     &1.4810e+02_rb,1.1236e+02_rb,7.8257e+01_rb,4.3108e+01_rb,4.8588e-01_rb/)
      kbo(:, 5,33,13) = (/ &
     &1.8404e+02_rb,1.3940e+02_rb,9.6928e+01_rb,5.3246e+01_rb,6.2021e-01_rb/)
      kbo(:, 1,34,13) = (/ &
     &6.7703e+01_rb,5.1785e+01_rb,3.6334e+01_rb,2.0214e+01_rb,2.2826e-01_rb/)
      kbo(:, 2,34,13) = (/ &
     &8.9165e+01_rb,6.8046e+01_rb,4.7643e+01_rb,2.6485e+01_rb,2.7978e-01_rb/)
      kbo(:, 3,34,13) = (/ &
     &1.1511e+02_rb,8.7692e+01_rb,6.1290e+01_rb,3.4007e+01_rb,3.5723e-01_rb/)
      kbo(:, 4,34,13) = (/ &
     &1.4598e+02_rb,1.1096e+02_rb,7.7422e+01_rb,4.2877e+01_rb,4.6377e-01_rb/)
      kbo(:, 5,34,13) = (/ &
     &1.8238e+02_rb,1.3836e+02_rb,9.6327e+01_rb,5.3168e+01_rb,5.7690e-01_rb/)
      kbo(:, 1,35,13) = (/ &
     &6.4861e+01_rb,4.9746e+01_rb,3.5026e+01_rb,1.9551e+01_rb,2.0156e-01_rb/)
      kbo(:, 2,35,13) = (/ &
     &8.5976e+01_rb,6.5785e+01_rb,4.6175e+01_rb,2.5782e+01_rb,2.5401e-01_rb/)
      kbo(:, 3,35,13) = (/ &
     &1.1165e+02_rb,8.5245e+01_rb,5.9716e+01_rb,3.3273e+01_rb,3.2846e-01_rb/)
      kbo(:, 4,35,13) = (/ &
     &1.4250e+02_rb,1.0853e+02_rb,7.5838e+01_rb,4.2162e+01_rb,4.2675e-01_rb/)
      kbo(:, 5,35,13) = (/ &
     &1.7910e+02_rb,1.3608e+02_rb,9.4891e+01_rb,5.2581e+01_rb,5.2256e-01_rb/)
      kbo(:, 1,36,13) = (/ &
     &6.0764e+01_rb,4.6732e+01_rb,3.3052e+01_rb,1.8514e+01_rb,1.7227e-01_rb/)
      kbo(:, 2,36,13) = (/ &
     &8.1347e+01_rb,6.2416e+01_rb,4.3933e+01_rb,2.4619e+01_rb,2.3170e-01_rb/)
      kbo(:, 3,36,13) = (/ &
     &1.0653e+02_rb,8.1514e+01_rb,5.7203e+01_rb,3.2018e+01_rb,2.9859e-01_rb/)
      kbo(:, 4,36,13) = (/ &
     &1.3711e+02_rb,1.0461e+02_rb,7.3196e+01_rb,4.0838e+01_rb,3.8265e-01_rb/)
      kbo(:, 5,36,13) = (/ &
     &1.7349e+02_rb,1.3202e+02_rb,9.2179e+01_rb,5.1271e+01_rb,4.7154e-01_rb/)
      kbo(:, 1,37,13) = (/ &
     &5.5040e+01_rb,4.2456e+01_rb,3.0166e+01_rb,1.6967e+01_rb,1.4686e-01_rb/)
      kbo(:, 2,37,13) = (/ &
     &7.4751e+01_rb,5.7522e+01_rb,4.0627e+01_rb,2.2828e+01_rb,1.9745e-01_rb/)
      kbo(:, 3,37,13) = (/ &
     &9.9126e+01_rb,7.6001e+01_rb,5.3446e+01_rb,3.0003e+01_rb,2.5830e-01_rb/)
      kbo(:, 4,37,13) = (/ &
     &1.2888e+02_rb,9.8527e+01_rb,6.9056e+01_rb,3.8643e+01_rb,3.3278e-01_rb/)
      kbo(:, 5,37,13) = (/ &
     &1.6453e+02_rb,1.2541e+02_rb,8.7685e+01_rb,4.8918e+01_rb,4.1260e-01_rb/)
      kbo(:, 1,38,13) = (/ &
     &4.9859e+01_rb,3.8586e+01_rb,2.7550e+01_rb,1.5547e+01_rb,1.2396e-01_rb/)
      kbo(:, 2,38,13) = (/ &
     &6.8738e+01_rb,5.3032e+01_rb,3.7612e+01_rb,2.1200e+01_rb,1.6720e-01_rb/)
      kbo(:, 3,38,13) = (/ &
     &9.2373e+01_rb,7.0999e+01_rb,5.0053e+01_rb,2.8170e+01_rb,2.2370e-01_rb/)
      kbo(:, 4,38,13) = (/ &
     &1.2130e+02_rb,9.2918e+01_rb,6.5279e+01_rb,3.6626e+01_rb,2.8704e-01_rb/)
      kbo(:, 5,38,13) = (/ &
     &1.5627e+02_rb,1.1930e+02_rb,8.3547e+01_rb,4.6739e+01_rb,3.6180e-01_rb/)
      kbo(:, 1,39,13) = (/ &
     &4.5302e+01_rb,3.5171e+01_rb,2.5236e+01_rb,1.4286e+01_rb,1.0568e-01_rb/)
      kbo(:, 2,39,13) = (/ &
     &6.3391e+01_rb,4.9041e+01_rb,3.4923e+01_rb,1.9751e+01_rb,1.4249e-01_rb/)
      kbo(:, 3,39,13) = (/ &
     &8.6323e+01_rb,6.6507e+01_rb,4.7040e+01_rb,2.6533e+01_rb,1.9303e-01_rb/)
      kbo(:, 4,39,13) = (/ &
     &1.1457e+02_rb,8.7914e+01_rb,6.1899e+01_rb,3.4824e+01_rb,2.4868e-01_rb/)
      kbo(:, 5,39,13) = (/ &
     &1.4887e+02_rb,1.1383e+02_rb,7.9847e+01_rb,4.4779e+01_rb,3.1416e-01_rb/)
      kbo(:, 1,40,13) = (/ &
     &4.0177e+01_rb,3.1298e+01_rb,2.2565e+01_rb,1.2805e+01_rb,8.8138e-02_rb/)
      kbo(:, 2,40,13) = (/ &
     &5.7199e+01_rb,4.4376e+01_rb,3.1743e+01_rb,1.8006e+01_rb,1.1856e-01_rb/)
      kbo(:, 3,40,13) = (/ &
     &7.9150e+01_rb,6.1135e+01_rb,4.3398e+01_rb,2.4540e+01_rb,1.6359e-01_rb/)
      kbo(:, 4,40,13) = (/ &
     &1.0651e+02_rb,8.1890e+01_rb,5.7786e+01_rb,3.2576e+01_rb,2.1421e-01_rb/)
      kbo(:, 5,40,13) = (/ &
     &1.3985e+02_rb,1.0710e+02_rb,7.5255e+01_rb,4.2282e+01_rb,2.7560e-01_rb/)
      kbo(:, 1,41,13) = (/ &
     &3.5483e+01_rb,2.7745e+01_rb,2.0104e+01_rb,1.1435e+01_rb,7.4578e-02_rb/)
      kbo(:, 2,41,13) = (/ &
     &5.1443e+01_rb,4.0033e+01_rb,2.8766e+01_rb,1.6358e+01_rb,1.0080e-01_rb/)
      kbo(:, 3,41,13) = (/ &
     &7.2353e+01_rb,5.6024e+01_rb,3.9928e+01_rb,2.2644e+01_rb,1.3798e-01_rb/)
      kbo(:, 4,41,13) = (/ &
     &9.8821e+01_rb,7.6124e+01_rb,5.3862e+01_rb,3.0414e+01_rb,1.8395e-01_rb/)
      kbo(:, 5,41,13) = (/ &
     &1.3121e+02_rb,1.0062e+02_rb,7.0817e+01_rb,3.9858e+01_rb,2.3971e-01_rb/)
      kbo(:, 1,42,13) = (/ &
     &3.1312e+01_rb,2.4582e+01_rb,1.7898e+01_rb,1.0203e+01_rb,6.3311e-02_rb/)
      kbo(:, 2,42,13) = (/ &
     &4.6257e+01_rb,3.6106e+01_rb,2.6059e+01_rb,1.4850e+01_rb,8.5987e-02_rb/)
      kbo(:, 3,42,13) = (/ &
     &6.6134e+01_rb,5.1324e+01_rb,3.6711e+01_rb,2.0879e+01_rb,1.1696e-01_rb/)
      kbo(:, 4,42,13) = (/ &
     &9.1649e+01_rb,7.0739e+01_rb,5.0191e+01_rb,2.8396e+01_rb,1.5868e-01_rb/)
      kbo(:, 5,42,13) = (/ &
     &1.2315e+02_rb,9.4550e+01_rb,6.6649e+01_rb,3.7570e+01_rb,2.0886e-01_rb/)
      kbo(:, 1,43,13) = (/ &
     &2.7228e+01_rb,2.1458e+01_rb,1.5694e+01_rb,8.9614e+00_rb,5.3608e-02_rb/)
      kbo(:, 2,43,13) = (/ &
     &4.1053e+01_rb,3.2140e+01_rb,2.3306e+01_rb,1.3302e+01_rb,7.1800e-02_rb/)
      kbo(:, 3,43,13) = (/ &
     &5.9801e+01_rb,4.6516e+01_rb,3.3390e+01_rb,1.9030e+01_rb,9.7105e-02_rb/)
      kbo(:, 4,43,13) = (/ &
     &8.4239e+01_rb,6.5135e+01_rb,4.6335e+01_rb,2.6274e+01_rb,1.3391e-01_rb/)
      kbo(:, 5,43,13) = (/ &
     &1.1479e+02_rb,8.8247e+01_rb,6.2314e+01_rb,3.5145e+01_rb,1.7840e-01_rb/)
      kbo(:, 1,44,13) = (/ &
     &2.3465e+01_rb,1.8562e+01_rb,1.3638e+01_rb,7.7925e+00_rb,4.4874e-02_rb/)
      kbo(:, 2,44,13) = (/ &
     &3.6146e+01_rb,2.8382e+01_rb,2.0672e+01_rb,1.1814e+01_rb,5.9063e-02_rb/)
      kbo(:, 3,44,13) = (/ &
     &5.3714e+01_rb,4.1873e+01_rb,3.0165e+01_rb,1.7212e+01_rb,7.9828e-02_rb/)
      kbo(:, 4,44,13) = (/ &
     &7.6981e+01_rb,5.9614e+01_rb,4.2519e+01_rb,2.4160e+01_rb,1.1248e-01_rb/)
      kbo(:, 5,44,13) = (/ &
     &1.0651e+02_rb,8.1983e+01_rb,5.8013e+01_rb,3.2734e+01_rb,1.4901e-01_rb/)
      kbo(:, 1,45,13) = (/ &
     &2.0186e+01_rb,1.6021e+01_rb,1.1808e+01_rb,6.7511e+00_rb,3.7575e-02_rb/)
      kbo(:, 2,45,13) = (/ &
     &3.1735e+01_rb,2.4996e+01_rb,1.8280e+01_rb,1.0459e+01_rb,4.9524e-02_rb/)
      kbo(:, 3,45,13) = (/ &
     &4.8111e+01_rb,3.7595e+01_rb,2.7182e+01_rb,1.5526e+01_rb,6.7700e-02_rb/)
      kbo(:, 4,45,13) = (/ &
     &7.0180e+01_rb,5.4438e+01_rb,3.8926e+01_rb,2.2155e+01_rb,9.2168e-02_rb/)
      kbo(:, 5,45,13) = (/ &
     &9.8620e+01_rb,7.6015e+01_rb,5.3889e+01_rb,3.0449e+01_rb,1.2546e-01_rb/)
      kbo(:, 1,46,13) = (/ &
     &1.7168e+01_rb,1.3674e+01_rb,1.0102e+01_rb,5.7721e+00_rb,3.1506e-02_rb/)
      kbo(:, 2,46,13) = (/ &
     &2.7551e+01_rb,2.1759e+01_rb,1.5975e+01_rb,9.1472e+00_rb,4.1368e-02_rb/)
      kbo(:, 3,46,13) = (/ &
     &4.2631e+01_rb,3.3389e+01_rb,2.4233e+01_rb,1.3855e+01_rb,5.6581e-02_rb/)
      kbo(:, 4,46,13) = (/ &
     &6.3374e+01_rb,4.9245e+01_rb,3.5311e+01_rb,2.0117e+01_rb,7.4981e-02_rb/)
      kbo(:, 5,46,13) = (/ &
     &9.0545e+01_rb,6.9872e+01_rb,4.9627e+01_rb,2.8091e+01_rb,1.0356e-01_rb/)
      kbo(:, 1,47,13) = (/ &
     &1.4285e+01_rb,1.1420e+01_rb,8.4474e+00_rb,4.8173e+00_rb,2.6647e-02_rb/)
      kbo(:, 2,47,13) = (/ &
     &2.3421e+01_rb,1.8547e+01_rb,1.3658e+01_rb,7.8222e+00_rb,3.5142e-02_rb/)
      kbo(:, 3,47,13) = (/ &
     &3.7014e+01_rb,2.9064e+01_rb,2.1182e+01_rb,1.2119e+01_rb,4.6223e-02_rb/)
      kbo(:, 4,47,13) = (/ &
     &5.6196e+01_rb,4.3754e+01_rb,3.1477e+01_rb,1.7951e+01_rb,6.2561e-02_rb/)
      kbo(:, 5,47,13) = (/ &
     &8.1804e+01_rb,6.3215e+01_rb,4.5000e+01_rb,2.5512e+01_rb,8.4936e-02_rb/)
      kbo(:, 1,48,13) = (/ &
     &1.1839e+01_rb,9.5013e+00_rb,7.0342e+00_rb,3.9991e+00_rb,2.2276e-02_rb/)
      kbo(:, 2,48,13) = (/ &
     &1.9820e+01_rb,1.5737e+01_rb,1.1620e+01_rb,6.6464e+00_rb,2.9148e-02_rb/)
      kbo(:, 3,48,13) = (/ &
     &3.1994e+01_rb,2.5174e+01_rb,1.8417e+01_rb,1.0540e+01_rb,3.8297e-02_rb/)
      kbo(:, 4,48,13) = (/ &
     &4.9572e+01_rb,3.8675e+01_rb,2.7923e+01_rb,1.5936e+01_rb,5.2300e-02_rb/)
      kbo(:, 5,48,13) = (/ &
     &7.3600e+01_rb,5.6966e+01_rb,4.0638e+01_rb,2.3068e+01_rb,6.9650e-02_rb/)
      kbo(:, 1,49,13) = (/ &
     &9.7471e+00_rb,7.8584e+00_rb,5.8267e+00_rb,3.3020e+00_rb,1.8196e-02_rb/)
      kbo(:, 2,49,13) = (/ &
     &1.6704e+01_rb,1.3302e+01_rb,9.8376e+00_rb,5.6165e+00_rb,2.4552e-02_rb/)
      kbo(:, 3,49,13) = (/ &
     &2.7532e+01_rb,2.1706e+01_rb,1.5922e+01_rb,9.1112e+00_rb,3.1859e-02_rb/)
      kbo(:, 4,49,13) = (/ &
     &4.3516e+01_rb,3.4017e+01_rb,2.4645e+01_rb,1.4071e+01_rb,4.2368e-02_rb/)
      kbo(:, 5,49,13) = (/ &
     &6.5913e+01_rb,5.1094e+01_rb,3.6540e+01_rb,2.0762e+01_rb,5.7956e-02_rb/)
      kbo(:, 1,50,13) = (/ &
     &8.0804e+00_rb,6.5442e+00_rb,4.8528e+00_rb,2.7471e+00_rb,1.5480e-02_rb/)
      kbo(:, 2,50,13) = (/ &
     &1.4208e+01_rb,1.1343e+01_rb,8.3916e+00_rb,4.7793e+00_rb,2.1108e-02_rb/)
      kbo(:, 3,50,13) = (/ &
     &2.3865e+01_rb,1.8845e+01_rb,1.3856e+01_rb,7.9173e+00_rb,2.8070e-02_rb/)
      kbo(:, 4,50,13) = (/ &
     &3.8439e+01_rb,3.0084e+01_rb,2.1861e+01_rb,1.2484e+01_rb,3.6716e-02_rb/)
      kbo(:, 5,50,13) = (/ &
     &5.9311e+01_rb,4.6040e+01_rb,3.3007e+01_rb,1.8765e+01_rb,4.7982e-02_rb/)
      kbo(:, 1,51,13) = (/ &
     &6.6974e+00_rb,5.4506e+00_rb,4.0401e+00_rb,2.2835e+00_rb,1.2897e-02_rb/)
      kbo(:, 2,51,13) = (/ &
     &1.2105e+01_rb,9.6894e+00_rb,7.1742e+00_rb,4.0729e+00_rb,1.8234e-02_rb/)
      kbo(:, 3,51,13) = (/ &
     &2.0726e+01_rb,1.6392e+01_rb,1.2076e+01_rb,6.8856e+00_rb,2.4297e-02_rb/)
      kbo(:, 4,51,13) = (/ &
     &3.4010e+01_rb,2.6646e+01_rb,1.9402e+01_rb,1.1077e+01_rb,3.1447e-02_rb/)
      kbo(:, 5,51,13) = (/ &
     &5.3414e+01_rb,4.1508e+01_rb,2.9825e+01_rb,1.6965e+01_rb,4.1659e-02_rb/)
      kbo(:, 1,52,13) = (/ &
     &5.5139e+00_rb,4.5099e+00_rb,3.3452e+00_rb,1.8859e+00_rb,1.0839e-02_rb/)
      kbo(:, 2,52,13) = (/ &
     &1.0265e+01_rb,8.2392e+00_rb,6.1066e+00_rb,3.4594e+00_rb,1.5307e-02_rb/)
      kbo(:, 3,52,13) = (/ &
     &1.7969e+01_rb,1.4231e+01_rb,1.0492e+01_rb,5.9702e+00_rb,2.0782e-02_rb/)
      kbo(:, 4,52,13) = (/ &
     &3.0022e+01_rb,2.3539e+01_rb,1.7173e+01_rb,9.7942e+00_rb,2.7615e-02_rb/)
      kbo(:, 5,52,13) = (/ &
     &4.7976e+01_rb,3.7311e+01_rb,2.6868e+01_rb,1.5284e+01_rb,3.5883e-02_rb/)
      kbo(:, 1,53,13) = (/ &
     &4.5089e+00_rb,3.7050e+00_rb,2.7486e+00_rb,1.5449e+00_rb,8.8988e-03_rb/)
      kbo(:, 2,53,13) = (/ &
     &8.6595e+00_rb,6.9701e+00_rb,5.1646e+00_rb,2.9217e+00_rb,1.2928e-02_rb/)
      kbo(:, 3,53,13) = (/ &
     &1.5539e+01_rb,1.2324e+01_rb,9.0856e+00_rb,5.1569e+00_rb,1.7922e-02_rb/)
      kbo(:, 4,53,13) = (/ &
     &2.6425e+01_rb,2.0735e+01_rb,1.5153e+01_rb,8.6253e+00_rb,2.3907e-02_rb/)
      kbo(:, 5,53,13) = (/ &
     &4.2979e+01_rb,3.3444e+01_rb,2.4119e+01_rb,1.3719e+01_rb,3.1307e-02_rb/)
      kbo(:, 1,54,13) = (/ &
     &3.7369e+00_rb,3.0844e+00_rb,2.2855e+00_rb,1.2806e+00_rb,7.5969e-03_rb/)
      kbo(:, 2,54,13) = (/ &
     &7.3945e+00_rb,5.9700e+00_rb,4.4241e+00_rb,2.4976e+00_rb,1.1170e-02_rb/)
      kbo(:, 3,54,13) = (/ &
     &1.3610e+01_rb,1.0807e+01_rb,7.9714e+00_rb,4.5128e+00_rb,1.6129e-02_rb/)
      kbo(:, 4,54,13) = (/ &
     &2.3560e+01_rb,1.8494e+01_rb,1.3530e+01_rb,7.6837e+00_rb,2.1838e-02_rb/)
      kbo(:, 5,54,13) = (/ &
     &3.8949e+01_rb,3.0311e+01_rb,2.1882e+01_rb,1.2434e+01_rb,2.8239e-02_rb/)
      kbo(:, 1,55,13) = (/ &
     &3.1147e+00_rb,2.5777e+00_rb,1.9050e+00_rb,1.0637e+00_rb,6.5369e-03_rb/)
      kbo(:, 2,55,13) = (/ &
     &6.3439e+00_rb,5.1347e+00_rb,3.8054e+00_rb,2.1454e+00_rb,1.0024e-02_rb/)
      kbo(:, 3,55,13) = (/ &
     &1.1985e+01_rb,9.5229e+00_rb,7.0258e+00_rb,3.9713e+00_rb,1.4330e-02_rb/)
      kbo(:, 4,55,13) = (/ &
     &2.1145e+01_rb,1.6600e+01_rb,1.2148e+01_rb,6.8814e+00_rb,1.9929e-02_rb/)
      kbo(:, 5,55,13) = (/ &
     &3.5505e+01_rb,2.7628e+01_rb,1.9956e+01_rb,1.1323e+01_rb,2.6070e-02_rb/)
      kbo(:, 1,56,13) = (/ &
     &2.5841e+00_rb,2.1412e+00_rb,1.5792e+00_rb,8.7847e-01_rb,5.7272e-03_rb/)
      kbo(:, 2,56,13) = (/ &
     &5.4279e+00_rb,4.4047e+00_rb,3.2621e+00_rb,1.8370e+00_rb,9.0085e-03_rb/)
      kbo(:, 3,56,13) = (/ &
     &1.0541e+01_rb,8.3802e+00_rb,6.1801e+00_rb,3.4870e+00_rb,1.2894e-02_rb/)
      kbo(:, 4,56,13) = (/ &
     &1.8998e+01_rb,1.4914e+01_rb,1.0910e+01_rb,6.1631e+00_rb,1.8216e-02_rb/)
      kbo(:, 5,56,13) = (/ &
     &3.2387e+01_rb,2.5195e+01_rb,1.8200e+01_rb,1.0311e+01_rb,2.5017e-02_rb/)
      kbo(:, 1,57,13) = (/ &
     &2.1273e+00_rb,1.7663e+00_rb,1.2997e+00_rb,7.2182e-01_rb,4.9990e-03_rb/)
      kbo(:, 2,57,13) = (/ &
     &4.6288e+00_rb,3.7653e+00_rb,2.7876e+00_rb,1.5653e+00_rb,7.9939e-03_rb/)
      kbo(:, 3,57,13) = (/ &
     &9.2459e+00_rb,7.3572e+00_rb,5.4258e+00_rb,3.0550e+00_rb,1.1761e-02_rb/)
      kbo(:, 4,57,13) = (/ &
     &1.7067e+01_rb,1.3395e+01_rb,9.7952e+00_rb,5.5222e+00_rb,1.6851e-02_rb/)
      kbo(:, 5,57,13) = (/ &
     &2.9574e+01_rb,2.2993e+01_rb,1.6606e+01_rb,9.3912e+00_rb,2.2852e-02_rb/)
      kbo(:, 1,58,13) = (/ &
     &8.4744e-01_rb,8.1145e-01_rb,7.0194e-01_rb,4.7172e-01_rb,4.4639e-03_rb/)
      kbo(:, 2,58,13) = (/ &
     &1.9303e+00_rb,1.8049e+00_rb,1.5645e+00_rb,1.0595e+00_rb,7.1718e-03_rb/)
      kbo(:, 3,58,13) = (/ &
     &3.9593e+00_rb,3.6185e+00_rb,3.1318e+00_rb,2.1308e+00_rb,1.0759e-02_rb/)
      kbo(:, 4,58,13) = (/ &
     &7.4878e+00_rb,6.7423e+00_rb,5.7807e+00_rb,3.9390e+00_rb,1.5270e-02_rb/)
      kbo(:, 5,58,13) = (/ &
     &1.3201e+01_rb,1.1774e+01_rb,9.9717e+00_rb,6.8098e+00_rb,2.1046e-02_rb/)
      kbo(:, 1,59,13) = (/ &
     &7.8389e-01_rb,7.5361e-01_rb,6.5545e-01_rb,4.4390e-01_rb,4.7731e-03_rb/)
      kbo(:, 2,59,13) = (/ &
     &1.8316e+00_rb,1.7144e+00_rb,1.4906e+00_rb,1.0154e+00_rb,7.3182e-03_rb/)
      kbo(:, 3,59,13) = (/ &
     &3.8012e+00_rb,3.4796e+00_rb,3.0220e+00_rb,2.0697e+00_rb,1.0970e-02_rb/)
      kbo(:, 4,59,13) = (/ &
     &7.2668e+00_rb,6.5534e+00_rb,5.6332e+00_rb,3.8611e+00_rb,1.5317e-02_rb/)
      kbo(:, 5,59,13) = (/ &
     &1.2911e+01_rb,1.1536e+01_rb,9.7915e+00_rb,6.7229e+00_rb,2.2015e-02_rb/)
      kbo(:, 1,13,14) = (/ &
     &2.9725e+02_rb,2.2295e+02_rb,1.4862e+02_rb,7.4314e+01_rb,8.5883e-01_rb/)
      kbo(:, 2,13,14) = (/ &
     &4.1294e+02_rb,3.0971e+02_rb,2.0649e+02_rb,1.0324e+02_rb,7.4403e-01_rb/)
      kbo(:, 3,13,14) = (/ &
     &5.5502e+02_rb,4.1625e+02_rb,2.7751e+02_rb,1.3875e+02_rb,6.6008e-01_rb/)
      kbo(:, 4,13,14) = (/ &
     &7.2029e+02_rb,5.4024e+02_rb,3.6017e+02_rb,1.8008e+02_rb,1.1155e+00_rb/)
      kbo(:, 5,13,14) = (/ &
     &9.0592e+02_rb,6.7946e+02_rb,4.5296e+02_rb,2.2648e+02_rb,2.5465e+00_rb/)
      kbo(:, 1,14,14) = (/ &
     &3.2435e+02_rb,2.4326e+02_rb,1.6217e+02_rb,8.1092e+01_rb,7.8098e-01_rb/)
      kbo(:, 2,14,14) = (/ &
     &4.4892e+02_rb,3.3668e+02_rb,2.2444e+02_rb,1.1223e+02_rb,5.4602e-01_rb/)
      kbo(:, 3,14,14) = (/ &
     &6.0099e+02_rb,4.5075e+02_rb,3.0049e+02_rb,1.5025e+02_rb,1.0325e+00_rb/)
      kbo(:, 4,14,14) = (/ &
     &7.7937e+02_rb,5.8453e+02_rb,3.8969e+02_rb,1.9485e+02_rb,1.5016e+00_rb/)
      kbo(:, 5,14,14) = (/ &
     &9.8034e+02_rb,7.3527e+02_rb,4.9018e+02_rb,2.4510e+02_rb,3.4894e+00_rb/)
      kbo(:, 1,15,14) = (/ &
     &3.4430e+02_rb,2.5823e+02_rb,1.7215e+02_rb,8.6071e+01_rb,4.7442e-01_rb/)
      kbo(:, 2,15,14) = (/ &
     &4.7694e+02_rb,3.5770e+02_rb,2.3846e+02_rb,1.1924e+02_rb,9.8957e-01_rb/)
      kbo(:, 3,15,14) = (/ &
     &6.3659e+02_rb,4.7745e+02_rb,3.1829e+02_rb,1.5915e+02_rb,1.3277e+00_rb/)
      kbo(:, 4,15,14) = (/ &
     &8.2316e+02_rb,6.1735e+02_rb,4.1157e+02_rb,2.0578e+02_rb,2.6545e+00_rb/)
      kbo(:, 5,15,14) = (/ &
     &1.0339e+03_rb,7.7536e+02_rb,5.1695e+02_rb,2.5844e+02_rb,4.8753e+00_rb/)
      kbo(:, 1,16,14) = (/ &
     &3.6224e+02_rb,2.7168e+02_rb,1.8111e+02_rb,9.0555e+01_rb,8.6361e-01_rb/)
      kbo(:, 2,16,14) = (/ &
     &5.0155e+02_rb,3.7615e+02_rb,2.5077e+02_rb,1.2539e+02_rb,1.3098e+00_rb/)
      kbo(:, 3,16,14) = (/ &
     &6.6682e+02_rb,5.0012e+02_rb,3.3343e+02_rb,1.6672e+02_rb,2.0810e+00_rb/)
      kbo(:, 4,16,14) = (/ &
     &8.5980e+02_rb,6.4485e+02_rb,4.2990e+02_rb,2.1496e+02_rb,4.0118e+00_rb/)
      kbo(:, 5,16,14) = (/ &
     &1.0768e+03_rb,8.0756e+02_rb,5.3838e+02_rb,2.6919e+02_rb,6.0151e+00_rb/)
      kbo(:, 1,17,14) = (/ &
     &3.8011e+02_rb,2.8509e+02_rb,1.9007e+02_rb,9.5029e+01_rb,1.2720e+00_rb/)
      kbo(:, 2,17,14) = (/ &
     &5.2601e+02_rb,3.9448e+02_rb,2.6301e+02_rb,1.3150e+02_rb,1.5543e+00_rb/)
      kbo(:, 3,17,14) = (/ &
     &6.9761e+02_rb,5.2320e+02_rb,3.4880e+02_rb,1.7439e+02_rb,2.9841e+00_rb/)
      kbo(:, 4,17,14) = (/ &
     &8.9515e+02_rb,6.7136e+02_rb,4.4757e+02_rb,2.2378e+02_rb,4.6409e+00_rb/)
      kbo(:, 5,17,14) = (/ &
     &1.1181e+03_rb,8.3857e+02_rb,5.5903e+02_rb,2.7954e+02_rb,6.6157e+00_rb/)
      kbo(:, 1,18,14) = (/ &
     &3.9222e+02_rb,2.9415e+02_rb,1.9611e+02_rb,9.8051e+01_rb,1.3802e+00_rb/)
      kbo(:, 2,18,14) = (/ &
     &5.4105e+02_rb,4.0577e+02_rb,2.7053e+02_rb,1.3526e+02_rb,2.0079e+00_rb/)
      kbo(:, 3,18,14) = (/ &
     &7.1441e+02_rb,5.3583e+02_rb,3.5723e+02_rb,1.7861e+02_rb,3.3624e+00_rb/)
      kbo(:, 4,18,14) = (/ &
     &9.1265e+02_rb,6.8450e+02_rb,4.5632e+02_rb,2.2816e+02_rb,4.9650e+00_rb/)
      kbo(:, 5,18,14) = (/ &
     &1.1336e+03_rb,8.5015e+02_rb,5.6679e+02_rb,2.8338e+02_rb,6.8761e+00_rb/)
      kbo(:, 1,19,14) = (/ &
     &4.0839e+02_rb,3.0630e+02_rb,2.0420e+02_rb,1.0209e+02_rb,1.4198e+00_rb/)
      kbo(:, 2,19,14) = (/ &
     &5.5968e+02_rb,4.1977e+02_rb,2.7985e+02_rb,1.3992e+02_rb,2.2241e+00_rb/)
      kbo(:, 3,19,14) = (/ &
     &7.3637e+02_rb,5.5226e+02_rb,3.6818e+02_rb,1.8410e+02_rb,3.5653e+00_rb/)
      kbo(:, 4,19,14) = (/ &
     &9.3504e+02_rb,7.0126e+02_rb,4.6751e+02_rb,2.3375e+02_rb,5.1035e+00_rb/)
      kbo(:, 5,19,14) = (/ &
     &1.1560e+03_rb,8.6697e+02_rb,5.7801e+02_rb,2.8901e+02_rb,6.9044e+00_rb/)
      kbo(:, 1,20,14) = (/ &
     &4.2971e+02_rb,3.2229e+02_rb,2.1485e+02_rb,1.0743e+02_rb,1.5393e+00_rb/)
      kbo(:, 2,20,14) = (/ &
     &5.8587e+02_rb,4.3942e+02_rb,2.9293e+02_rb,1.4647e+02_rb,2.2601e+00_rb/)
      kbo(:, 3,20,14) = (/ &
     &7.6617e+02_rb,5.7464e+02_rb,3.8311e+02_rb,1.9155e+02_rb,3.5353e+00_rb/)
      kbo(:, 4,20,14) = (/ &
     &9.6798e+02_rb,7.2594e+02_rb,4.8398e+02_rb,2.4197e+02_rb,5.0919e+00_rb/)
      kbo(:, 5,20,14) = (/ &
     &1.1898e+03_rb,8.9235e+02_rb,5.9489e+02_rb,2.9744e+02_rb,6.7835e+00_rb/)
      kbo(:, 1,21,14) = (/ &
     &4.4889e+02_rb,3.3666e+02_rb,2.2444e+02_rb,1.1222e+02_rb,1.6427e+00_rb/)
      kbo(:, 2,21,14) = (/ &
     &6.0830e+02_rb,4.5623e+02_rb,3.0417e+02_rb,1.5208e+02_rb,2.2298e+00_rb/)
      kbo(:, 3,21,14) = (/ &
     &7.9080e+02_rb,5.9312e+02_rb,3.9541e+02_rb,1.9769e+02_rb,3.4367e+00_rb/)
      kbo(:, 4,21,14) = (/ &
     &9.9371e+02_rb,7.4527e+02_rb,4.9684e+02_rb,2.4842e+02_rb,4.8609e+00_rb/)
      kbo(:, 5,21,14) = (/ &
     &1.2155e+03_rb,9.1159e+02_rb,6.0771e+02_rb,3.0387e+02_rb,6.5479e+00_rb/)
      kbo(:, 1,22,14) = (/ &
     &4.7127e+02_rb,3.5343e+02_rb,2.3563e+02_rb,1.1782e+02_rb,1.6925e+00_rb/)
      kbo(:, 2,22,14) = (/ &
     &6.3239e+02_rb,4.7428e+02_rb,3.1618e+02_rb,1.5809e+02_rb,2.2427e+00_rb/)
      kbo(:, 3,22,14) = (/ &
     &8.1553e+02_rb,6.1164e+02_rb,4.0775e+02_rb,2.0390e+02_rb,3.3409e+00_rb/)
      kbo(:, 4,22,14) = (/ &
     &1.0189e+03_rb,7.6417e+02_rb,5.0944e+02_rb,2.5480e+02_rb,4.6697e+00_rb/)
      kbo(:, 5,22,14) = (/ &
     &1.2378e+03_rb,9.2835e+02_rb,6.1890e+02_rb,3.0965e+02_rb,6.2760e+00_rb/)
      kbo(:, 1,23,14) = (/ &
     &4.8713e+02_rb,3.6534e+02_rb,2.4356e+02_rb,1.2192e+02_rb,1.6599e+00_rb/)
      kbo(:, 2,23,14) = (/ &
     &6.4775e+02_rb,4.8583e+02_rb,3.2387e+02_rb,1.6212e+02_rb,2.1938e+00_rb/)
      kbo(:, 3,23,14) = (/ &
     &8.3023e+02_rb,6.2268e+02_rb,4.1512e+02_rb,2.0786e+02_rb,3.2003e+00_rb/)
      kbo(:, 4,23,14) = (/ &
     &1.0300e+03_rb,7.7249e+02_rb,5.1499e+02_rb,2.5803e+02_rb,4.4341e+00_rb/)
      kbo(:, 5,23,14) = (/ &
     &1.2450e+03_rb,9.3374e+02_rb,6.2249e+02_rb,3.1208e+02_rb,5.9729e+00_rb/)
      kbo(:, 1,24,14) = (/ &
     &5.0112e+02_rb,3.7583e+02_rb,2.5056e+02_rb,1.2575e+02_rb,1.5724e+00_rb/)
      kbo(:, 2,24,14) = (/ &
     &6.6153e+02_rb,4.9614e+02_rb,3.3076e+02_rb,1.6604e+02_rb,2.1250e+00_rb/)
      kbo(:, 3,24,14) = (/ &
     &8.4162e+02_rb,6.3120e+02_rb,4.2079e+02_rb,2.1132e+02_rb,3.0488e+00_rb/)
      kbo(:, 4,24,14) = (/ &
     &1.0387e+03_rb,7.7903e+02_rb,5.1934e+02_rb,2.6092e+02_rb,4.1964e+00_rb/)
      kbo(:, 5,24,14) = (/ &
     &1.2497e+03_rb,9.3727e+02_rb,6.2485e+02_rb,3.1407e+02_rb,5.6661e+00_rb/)
      kbo(:, 1,25,14) = (/ &
     &5.0957e+02_rb,3.8217e+02_rb,2.5479e+02_rb,1.2836e+02_rb,1.4865e+00_rb/)
      kbo(:, 2,25,14) = (/ &
     &6.6771e+02_rb,5.0078e+02_rb,3.3385e+02_rb,1.6824e+02_rb,2.0184e+00_rb/)
      kbo(:, 3,25,14) = (/ &
     &8.4466e+02_rb,6.3351e+02_rb,4.2233e+02_rb,2.1290e+02_rb,2.8760e+00_rb/)
      kbo(:, 4,25,14) = (/ &
     &1.0379e+03_rb,7.7843e+02_rb,5.1895e+02_rb,2.6171e+02_rb,3.9646e+00_rb/)
      kbo(:, 5,25,14) = (/ &
     &1.2454e+03_rb,9.3411e+02_rb,6.2273e+02_rb,3.1409e+02_rb,5.3432e+00_rb/)
      kbo(:, 1,26,14) = (/ &
     &5.1543e+02_rb,3.8657e+02_rb,2.5772e+02_rb,1.3040e+02_rb,1.4310e+00_rb/)
      kbo(:, 2,26,14) = (/ &
     &6.7055e+02_rb,5.0292e+02_rb,3.3528e+02_rb,1.6969e+02_rb,1.9779e+00_rb/)
      kbo(:, 3,26,14) = (/ &
     &8.4393e+02_rb,6.3294e+02_rb,4.2195e+02_rb,2.1362e+02_rb,2.7140e+00_rb/)
      kbo(:, 4,26,14) = (/ &
     &1.0339e+03_rb,7.7538e+02_rb,5.1691e+02_rb,2.6173e+02_rb,3.7614e+00_rb/)
      kbo(:, 5,26,14) = (/ &
     &1.2377e+03_rb,9.2822e+02_rb,6.1881e+02_rb,3.1339e+02_rb,5.0524e+00_rb/)
      kbo(:, 1,27,14) = (/ &
     &5.1680e+02_rb,3.8760e+02_rb,2.5844e+02_rb,1.3139e+02_rb,1.4644e+00_rb/)
      kbo(:, 2,27,14) = (/ &
     &6.6877e+02_rb,5.0158e+02_rb,3.3442e+02_rb,1.7006e+02_rb,1.9743e+00_rb/)
      kbo(:, 3,27,14) = (/ &
     &8.3912e+02_rb,6.2935e+02_rb,4.1964e+02_rb,2.1343e+02_rb,2.6423e+00_rb/)
      kbo(:, 4,27,14) = (/ &
     &1.0255e+03_rb,7.6907e+02_rb,5.1284e+02_rb,2.6091e+02_rb,3.5939e+00_rb/)
      kbo(:, 5,27,14) = (/ &
     &1.2273e+03_rb,9.2048e+02_rb,6.1378e+02_rb,3.1226e+02_rb,4.8265e+00_rb/)
      kbo(:, 1,28,14) = (/ &
     &5.1471e+02_rb,3.8604e+02_rb,2.5761e+02_rb,1.3161e+02_rb,1.4975e+00_rb/)
      kbo(:, 2,28,14) = (/ &
     &6.6390e+02_rb,4.9793e+02_rb,3.3227e+02_rb,1.6984e+02_rb,1.9755e+00_rb/)
      kbo(:, 3,28,14) = (/ &
     &8.3099e+02_rb,6.2328e+02_rb,4.1590e+02_rb,2.1266e+02_rb,2.6592e+00_rb/)
      kbo(:, 4,28,14) = (/ &
     &1.0159e+03_rb,7.6197e+02_rb,5.0842e+02_rb,2.5994e+02_rb,3.4854e+00_rb/)
      kbo(:, 5,28,14) = (/ &
     &1.2168e+03_rb,9.1262e+02_rb,6.0898e+02_rb,3.1138e+02_rb,4.6837e+00_rb/)
      kbo(:, 1,29,14) = (/ &
     &5.0817e+02_rb,3.8113e+02_rb,2.5474e+02_rb,1.3092e+02_rb,1.4861e+00_rb/)
      kbo(:, 2,29,14) = (/ &
     &6.5385e+02_rb,4.9038e+02_rb,3.2768e+02_rb,1.6851e+02_rb,1.9807e+00_rb/)
      kbo(:, 3,29,14) = (/ &
     &8.1862e+02_rb,6.1397e+02_rb,4.1013e+02_rb,2.1090e+02_rb,2.6083e+00_rb/)
      kbo(:, 4,29,14) = (/ &
     &1.0015e+03_rb,7.5115e+02_rb,5.0169e+02_rb,2.5799e+02_rb,3.4366e+00_rb/)
      kbo(:, 5,29,14) = (/ &
     &1.2004e+03_rb,9.0023e+02_rb,6.0130e+02_rb,3.0911e+02_rb,4.6309e+00_rb/)
      kbo(:, 1,30,14) = (/ &
     &4.9890e+02_rb,3.7416e+02_rb,2.5051e+02_rb,1.2967e+02_rb,1.4792e+00_rb/)
      kbo(:, 2,30,14) = (/ &
     &6.4183e+02_rb,4.8137e+02_rb,3.2213e+02_rb,1.6671e+02_rb,1.9639e+00_rb/)
      kbo(:, 3,30,14) = (/ &
     &8.0457e+02_rb,6.0342e+02_rb,4.0369e+02_rb,2.0889e+02_rb,2.5564e+00_rb/)
      kbo(:, 4,30,14) = (/ &
     &9.8521e+02_rb,7.3890e+02_rb,4.9415e+02_rb,2.5564e+02_rb,3.4162e+00_rb/)
      kbo(:, 5,30,14) = (/ &
     &1.1820e+03_rb,8.8651e+02_rb,5.9274e+02_rb,3.0646e+02_rb,4.6345e+00_rb/)
      kbo(:, 1,31,14) = (/ &
     &4.8682e+02_rb,3.6513e+02_rb,2.4494e+02_rb,1.2775e+02_rb,1.4549e+00_rb/)
      kbo(:, 2,31,14) = (/ &
     &6.2686e+02_rb,4.7014e+02_rb,3.1520e+02_rb,1.6432e+02_rb,1.8973e+00_rb/)
      kbo(:, 3,31,14) = (/ &
     &7.8640e+02_rb,5.8981e+02_rb,3.9524e+02_rb,2.0591e+02_rb,2.5203e+00_rb/)
      kbo(:, 4,31,14) = (/ &
     &9.6406e+02_rb,7.2305e+02_rb,4.8423e+02_rb,2.5217e+02_rb,3.4800e+00_rb/)
      kbo(:, 5,31,14) = (/ &
     &1.1586e+03_rb,8.6894e+02_rb,5.8171e+02_rb,3.0260e+02_rb,4.7311e+00_rb/)
      kbo(:, 1,32,14) = (/ &
     &4.7960e+02_rb,3.5970e+02_rb,2.4176e+02_rb,1.2713e+02_rb,1.4049e+00_rb/)
      kbo(:, 2,32,14) = (/ &
     &6.1788e+02_rb,4.6339e+02_rb,3.1125e+02_rb,1.6347e+02_rb,1.8610e+00_rb/)
      kbo(:, 3,32,14) = (/ &
     &7.7566e+02_rb,5.8175e+02_rb,3.9047e+02_rb,2.0479e+02_rb,2.5736e+00_rb/)
      kbo(:, 4,32,14) = (/ &
     &9.5231e+02_rb,7.1420e+02_rb,4.7904e+02_rb,2.5086e+02_rb,3.5724e+00_rb/)
      kbo(:, 5,32,14) = (/ &
     &1.1464e+03_rb,8.5978e+02_rb,5.7626e+02_rb,3.0128e+02_rb,4.8525e+00_rb/)
      kbo(:, 1,33,14) = (/ &
     &4.7673e+02_rb,3.5758e+02_rb,2.4086e+02_rb,1.2764e+02_rb,1.3592e+00_rb/)
      kbo(:, 2,33,14) = (/ &
     &6.1463e+02_rb,4.6099e+02_rb,3.1024e+02_rb,1.6404e+02_rb,1.8844e+00_rb/)
      kbo(:, 3,33,14) = (/ &
     &7.7253e+02_rb,5.7937e+02_rb,3.8948e+02_rb,2.0544e+02_rb,2.6672e+00_rb/)
      kbo(:, 4,33,14) = (/ &
     &9.4922e+02_rb,7.1189e+02_rb,4.7815e+02_rb,2.5158e+02_rb,3.7054e+00_rb/)
      kbo(:, 5,33,14) = (/ &
     &1.1446e+03_rb,8.5842e+02_rb,5.7603e+02_rb,3.0239e+02_rb,5.0353e+00_rb/)
      kbo(:, 1,34,14) = (/ &
     &4.7829e+02_rb,3.5882e+02_rb,2.4214e+02_rb,1.2917e+02_rb,1.3360e+00_rb/)
      kbo(:, 2,34,14) = (/ &
     &6.1765e+02_rb,4.6331e+02_rb,3.1226e+02_rb,1.6594e+02_rb,1.9203e+00_rb/)
      kbo(:, 3,34,14) = (/ &
     &7.7743e+02_rb,5.8310e+02_rb,3.9256e+02_rb,2.0792e+02_rb,2.7349e+00_rb/)
      kbo(:, 4,34,14) = (/ &
     &9.5686e+02_rb,7.1762e+02_rb,4.8256e+02_rb,2.5475e+02_rb,3.8031e+00_rb/)
      kbo(:, 5,34,14) = (/ &
     &1.1550e+03_rb,8.6629e+02_rb,5.8189e+02_rb,3.0631e+02_rb,5.2021e+00_rb/)
      kbo(:, 1,35,14) = (/ &
     &4.7226e+02_rb,3.5443e+02_rb,2.3957e+02_rb,1.2851e+02_rb,1.3086e+00_rb/)
      kbo(:, 2,35,14) = (/ &
     &6.1214e+02_rb,4.5923e+02_rb,3.0998e+02_rb,1.6546e+02_rb,1.9154e+00_rb/)
      kbo(:, 3,35,14) = (/ &
     &7.7280e+02_rb,5.7976e+02_rb,3.9077e+02_rb,2.0773e+02_rb,2.7357e+00_rb/)
      kbo(:, 4,35,14) = (/ &
     &9.5397e+02_rb,7.1554e+02_rb,4.8165e+02_rb,2.5508e+02_rb,3.8286e+00_rb/)
      kbo(:, 5,35,14) = (/ &
     &1.1540e+03_rb,8.6552e+02_rb,5.8187e+02_rb,3.0708e+02_rb,5.2636e+00_rb/)
      kbo(:, 1,36,14) = (/ &
     &4.5693e+02_rb,3.4301e+02_rb,2.3220e+02_rb,1.2520e+02_rb,1.2533e+00_rb/)
      kbo(:, 2,36,14) = (/ &
     &5.9574e+02_rb,4.4710e+02_rb,3.0221e+02_rb,1.6199e+02_rb,1.8411e+00_rb/)
      kbo(:, 3,36,14) = (/ &
     &7.5609e+02_rb,5.6723e+02_rb,3.8282e+02_rb,2.0421e+02_rb,2.6504e+00_rb/)
      kbo(:, 4,36,14) = (/ &
     &9.3751e+02_rb,7.0328e+02_rb,4.7386e+02_rb,2.5165e+02_rb,3.7407e+00_rb/)
      kbo(:, 5,36,14) = (/ &
     &1.1381e+03_rb,8.5353e+02_rb,5.7430e+02_rb,3.0378e+02_rb,5.1704e+00_rb/)
      kbo(:, 1,37,14) = (/ &
     &4.2922e+02_rb,3.2232e+02_rb,2.1847e+02_rb,1.1834e+02_rb,1.1213e+00_rb/)
      kbo(:, 2,37,14) = (/ &
     &5.6522e+02_rb,4.2422e+02_rb,2.8713e+02_rb,1.5453e+02_rb,1.6760e+00_rb/)
      kbo(:, 3,37,14) = (/ &
     &7.2332e+02_rb,5.4281e+02_rb,3.6673e+02_rb,1.9621e+02_rb,2.4344e+00_rb/)
      kbo(:, 4,37,14) = (/ &
     &9.0338e+02_rb,6.7777e+02_rb,4.5711e+02_rb,2.4334e+02_rb,3.4735e+00_rb/)
      kbo(:, 5,37,14) = (/ &
     &1.1034e+03_rb,8.2764e+02_rb,5.5725e+02_rb,2.9534e+02_rb,4.8510e+00_rb/)
      kbo(:, 1,38,14) = (/ &
     &4.0355e+02_rb,3.0314e+02_rb,2.0578e+02_rb,1.1195e+02_rb,1.0088e+00_rb/)
      kbo(:, 2,38,14) = (/ &
     &5.3660e+02_rb,4.0286e+02_rb,2.7296e+02_rb,1.4742e+02_rb,1.5281e+00_rb/)
      kbo(:, 3,38,14) = (/ &
     &6.9245e+02_rb,5.1973e+02_rb,3.5152e+02_rb,1.8858e+02_rb,2.2407e+00_rb/)
      kbo(:, 4,38,14) = (/ &
     &8.7113e+02_rb,6.5360e+02_rb,4.4117e+02_rb,2.3535e+02_rb,3.2324e+00_rb/)
      kbo(:, 5,38,14) = (/ &
     &1.0703e+03_rb,8.0291e+02_rb,5.4092e+02_rb,2.8714e+02_rb,4.5526e+00_rb/)
      kbo(:, 1,39,14) = (/ &
     &3.8048e+02_rb,2.8595e+02_rb,1.9433e+02_rb,1.0614e+02_rb,9.1242e-01_rb/)
      kbo(:, 2,39,14) = (/ &
     &5.1067e+02_rb,3.8348e+02_rb,2.6010e+02_rb,1.4089e+02_rb,1.4025e+00_rb/)
      kbo(:, 3,39,14) = (/ &
     &6.6451e+02_rb,4.9880e+02_rb,3.3759e+02_rb,1.8159e+02_rb,2.0765e+00_rb/)
      kbo(:, 4,39,14) = (/ &
     &8.4141e+02_rb,6.3140e+02_rb,4.2638e+02_rb,2.2792e+02_rb,3.0229e+00_rb/)
      kbo(:, 5,39,14) = (/ &
     &1.0396e+03_rb,7.7992e+02_rb,5.2571e+02_rb,2.7947e+02_rb,4.2952e+00_rb/)
      kbo(:, 1,40,14) = (/ &
     &3.5133e+02_rb,2.6415e+02_rb,1.7970e+02_rb,9.8495e+01_rb,7.8366e-01_rb/)
      kbo(:, 2,40,14) = (/ &
     &4.7771e+02_rb,3.5880e+02_rb,2.4356e+02_rb,1.3227e+02_rb,1.2276e+00_rb/)
      kbo(:, 3,40,14) = (/ &
     &6.2817e+02_rb,4.7163e+02_rb,3.1937e+02_rb,1.7209e+02_rb,1.8434e+00_rb/)
      kbo(:, 4,40,14) = (/ &
     &8.0277e+02_rb,6.0241e+02_rb,4.0711e+02_rb,2.1788e+02_rb,2.7123e+00_rb/)
      kbo(:, 5,40,14) = (/ &
     &9.9978e+02_rb,7.5011e+02_rb,5.0577e+02_rb,2.6911e+02_rb,3.8911e+00_rb/)
      kbo(:, 1,41,14) = (/ &
     &3.2342e+02_rb,2.4324e+02_rb,1.6567e+02_rb,9.1094e+01_rb,6.6818e-01_rb/)
      kbo(:, 2,41,14) = (/ &
     &4.4573e+02_rb,3.3493e+02_rb,2.2747e+02_rb,1.2382e+02_rb,1.0662e+00_rb/)
      kbo(:, 3,41,14) = (/ &
     &5.9252e+02_rb,4.4488e+02_rb,3.0145e+02_rb,1.6269e+02_rb,1.6310e+00_rb/)
      kbo(:, 4,41,14) = (/ &
     &7.6447e+02_rb,5.7369e+02_rb,3.8787e+02_rb,2.0780e+02_rb,2.4231e+00_rb/)
      kbo(:, 5,41,14) = (/ &
     &9.6002e+02_rb,7.2027e+02_rb,4.8589e+02_rb,2.5869e+02_rb,3.5110e+00_rb/)
      kbo(:, 1,42,14) = (/ &
     &2.9741e+02_rb,2.2375e+02_rb,1.5256e+02_rb,8.4130e+01_rb,5.7099e-01_rb/)
      kbo(:, 2,42,14) = (/ &
     &4.1545e+02_rb,3.1224e+02_rb,2.1217e+02_rb,1.1576e+02_rb,9.2628e-01_rb/)
      kbo(:, 3,42,14) = (/ &
     &5.5858e+02_rb,4.1944e+02_rb,2.8433e+02_rb,1.5367e+02_rb,1.4426e+00_rb/)
      kbo(:, 4,42,14) = (/ &
     &7.2759e+02_rb,5.4609e+02_rb,3.6933e+02_rb,1.9808e+02_rb,2.1657e+00_rb/)
      kbo(:, 5,42,14) = (/ &
     &9.2131e+02_rb,6.9128e+02_rb,4.6648e+02_rb,2.4850e+02_rb,3.1673e+00_rb/)
      kbo(:, 1,43,14) = (/ &
     &2.7012e+02_rb,2.0330e+02_rb,1.3875e+02_rb,7.6704e+01_rb,4.7188e-01_rb/)
      kbo(:, 2,43,14) = (/ &
     &3.8333e+02_rb,2.8816e+02_rb,1.9596e+02_rb,1.0707e+02_rb,7.8390e-01_rb/)
      kbo(:, 3,43,14) = (/ &
     &5.2268e+02_rb,3.9255e+02_rb,2.6617e+02_rb,1.4400e+02_rb,1.2434e+00_rb/)
      kbo(:, 4,43,14) = (/ &
     &6.8847e+02_rb,5.1676e+02_rb,3.4954e+02_rb,1.8752e+02_rb,1.8919e+00_rb/)
      kbo(:, 5,43,14) = (/ &
     &8.8050e+02_rb,6.6068e+02_rb,4.4591e+02_rb,2.3760e+02_rb,2.7970e+00_rb/)
      kbo(:, 1,44,14) = (/ &
     &2.4333e+02_rb,1.8320e+02_rb,1.2513e+02_rb,6.9324e+01_rb,3.8543e-01_rb/)
      kbo(:, 2,44,14) = (/ &
     &3.5160e+02_rb,2.6439e+02_rb,1.7983e+02_rb,9.8391e+01_rb,6.5410e-01_rb/)
      kbo(:, 3,44,14) = (/ &
     &4.8672e+02_rb,3.6559e+02_rb,2.4794e+02_rb,1.3422e+02_rb,1.0574e+00_rb/)
      kbo(:, 4,44,14) = (/ &
     &6.4907e+02_rb,4.8721e+02_rb,3.2959e+02_rb,1.7683e+02_rb,1.6338e+00_rb/)
      kbo(:, 5,44,14) = (/ &
     &8.3935e+02_rb,6.2981e+02_rb,4.2512e+02_rb,2.2646e+02_rb,2.4460e+00_rb/)
      kbo(:, 1,45,14) = (/ &
     &2.1826e+02_rb,1.6439e+02_rb,1.1240e+02_rb,6.2387e+01_rb,3.1362e-01_rb/)
      kbo(:, 2,45,14) = (/ &
     &3.2151e+02_rb,2.4179e+02_rb,1.6452e+02_rb,9.0130e+01_rb,5.4229e-01_rb/)
      kbo(:, 3,45,14) = (/ &
     &4.5202e+02_rb,3.3957e+02_rb,2.3034e+02_rb,1.2476e+02_rb,8.9432e-01_rb/)
      kbo(:, 4,45,14) = (/ &
     &6.1098e+02_rb,4.5864e+02_rb,3.1027e+02_rb,1.6648e+02_rb,1.4094e+00_rb/)
      kbo(:, 5,45,14) = (/ &
     &7.9903e+02_rb,5.9956e+02_rb,4.0465e+02_rb,2.1549e+02_rb,2.1336e+00_rb/)
      kbo(:, 1,46,14) = (/ &
     &1.9334e+02_rb,1.4567e+02_rb,9.9705e+01_rb,5.5445e+01_rb,2.5106e-01_rb/)
      kbo(:, 2,46,14) = (/ &
     &2.9084e+02_rb,2.1878e+02_rb,1.4895e+02_rb,8.1712e+01_rb,4.4167e-01_rb/)
      kbo(:, 3,46,14) = (/ &
     &4.1605e+02_rb,3.1260e+02_rb,2.1206e+02_rb,1.1494e+02_rb,7.4589e-01_rb/)
      kbo(:, 4,46,14) = (/ &
     &5.7087e+02_rb,4.2857e+02_rb,2.8993e+02_rb,1.5559e+02_rb,1.1995e+00_rb/)
      kbo(:, 5,46,14) = (/ &
     &7.5566e+02_rb,5.6703e+02_rb,3.8272e+02_rb,2.0372e+02_rb,1.8420e+00_rb/)
      kbo(:, 1,47,14) = (/ &
     &1.6734e+02_rb,1.2615e+02_rb,8.6467e+01_rb,4.8182e+01_rb,1.9607e-01_rb/)
      kbo(:, 2,47,14) = (/ &
     &2.5798e+02_rb,1.9414e+02_rb,1.3227e+02_rb,7.2690e+01_rb,3.4728e-01_rb/)
      kbo(:, 3,47,14) = (/ &
     &3.7682e+02_rb,2.8319e+02_rb,1.9216e+02_rb,1.0425e+02_rb,6.0496e-01_rb/)
      kbo(:, 4,47,14) = (/ &
     &5.2596e+02_rb,3.9489e+02_rb,2.6718e+02_rb,1.4343e+02_rb,9.9190e-01_rb/)
      kbo(:, 5,47,14) = (/ &
     &7.0637e+02_rb,5.3006e+02_rb,3.5776e+02_rb,1.9039e+02_rb,1.5539e+00_rb/)
      kbo(:, 1,48,14) = (/ &
     &1.4359e+02_rb,1.0831e+02_rb,7.4350e+01_rb,4.1515e+01_rb,1.5112e-01_rb/)
      kbo(:, 2,48,14) = (/ &
     &2.2736e+02_rb,1.7115e+02_rb,1.1670e+02_rb,6.4242e+01_rb,2.7228e-01_rb/)
      kbo(:, 3,48,14) = (/ &
     &3.3955e+02_rb,2.5527e+02_rb,1.7326e+02_rb,9.4064e+01_rb,4.8587e-01_rb/)
      kbo(:, 4,48,14) = (/ &
     &4.8257e+02_rb,3.6236e+02_rb,2.4521e+02_rb,1.3165e+02_rb,8.1598e-01_rb/)
      kbo(:, 5,48,14) = (/ &
     &6.5841e+02_rb,4.9408e+02_rb,3.3351e+02_rb,1.7744e+02_rb,1.3043e+00_rb/)
      kbo(:, 1,49,14) = (/ &
     &1.2205e+02_rb,9.2102e+01_rb,6.3332e+01_rb,3.5432e+01_rb,1.1595e-01_rb/)
      kbo(:, 2,49,14) = (/ &
     &1.9899e+02_rb,1.4985e+02_rb,1.0228e+02_rb,5.6393e+01_rb,2.1177e-01_rb/)
      kbo(:, 3,49,14) = (/ &
     &3.0425e+02_rb,2.2876e+02_rb,1.5534e+02_rb,8.4415e+01_rb,3.8749e-01_rb/)
      kbo(:, 4,49,14) = (/ &
     &4.4104e+02_rb,3.3122e+02_rb,2.2413e+02_rb,1.2036e+02_rb,6.6950e-01_rb/)
      kbo(:, 5,49,14) = (/ &
     &6.1182e+02_rb,4.5914e+02_rb,3.0987e+02_rb,1.6480e+02_rb,1.0872e+00_rb/)
      kbo(:, 1,50,14) = (/ &
     &1.0410e+02_rb,7.8601e+01_rb,5.4134e+01_rb,3.0320e+01_rb,8.9437e-02_rb/)
      kbo(:, 2,50,14) = (/ &
     &1.7476e+02_rb,1.3166e+02_rb,8.9943e+01_rb,4.9647e+01_rb,1.6487e-01_rb/)
      kbo(:, 3,50,14) = (/ &
     &2.7363e+02_rb,2.0578e+02_rb,1.3977e+02_rb,7.5998e+01_rb,3.0880e-01_rb/)
      kbo(:, 4,50,14) = (/ &
     &4.0470e+02_rb,3.0398e+02_rb,2.0569e+02_rb,1.1045e+02_rb,5.4711e-01_rb/)
      kbo(:, 5,50,14) = (/ &
     &5.7059e+02_rb,4.2823e+02_rb,2.8900e+02_rb,1.5363e+02_rb,9.1135e-01_rb/)
      kbo(:, 1,51,14) = (/ &
     &8.8591e+01_rb,6.6918e+01_rb,4.6161e+01_rb,2.5881e+01_rb,7.0825e-02_rb/)
      kbo(:, 2,51,14) = (/ &
     &1.5315e+02_rb,1.1542e+02_rb,7.8914e+01_rb,4.3611e+01_rb,1.2808e-01_rb/)
      kbo(:, 3,51,14) = (/ &
     &2.4591e+02_rb,1.8497e+02_rb,1.2567e+02_rb,6.8391e+01_rb,2.4633e-01_rb/)
      kbo(:, 4,51,14) = (/ &
     &3.7125e+02_rb,2.7889e+02_rb,1.8871e+02_rb,1.0133e+02_rb,4.4644e-01_rb/)
      kbo(:, 5,51,14) = (/ &
     &5.3232e+02_rb,3.9953e+02_rb,2.6959e+02_rb,1.4324e+02_rb,7.6098e-01_rb/)
      kbo(:, 1,52,14) = (/ &
     &7.4832e+01_rb,5.6551e+01_rb,3.9072e+01_rb,2.1921e+01_rb,5.8568e-02_rb/)
      kbo(:, 2,52,14) = (/ &
     &1.3340e+02_rb,1.0056e+02_rb,6.8804e+01_rb,3.8048e+01_rb,9.9737e-02_rb/)
      kbo(:, 3,52,14) = (/ &
     &2.2000e+02_rb,1.6553e+02_rb,1.1251e+02_rb,6.1251e+01_rb,1.9573e-01_rb/)
      kbo(:, 4,52,14) = (/ &
     &3.3944e+02_rb,2.5503e+02_rb,1.7256e+02_rb,9.2621e+01_rb,3.6167e-01_rb/)
      kbo(:, 5,52,14) = (/ &
     &4.9563e+02_rb,3.7201e+02_rb,2.5099e+02_rb,1.3326e+02_rb,6.3209e-01_rb/)
      kbo(:, 1,53,14) = (/ &
     &6.2655e+01_rb,4.7378e+01_rb,3.2786e+01_rb,1.8402e+01_rb,5.0324e-02_rb/)
      kbo(:, 2,53,14) = (/ &
     &1.1533e+02_rb,8.6961e+01_rb,5.9562e+01_rb,3.2947e+01_rb,7.6555e-02_rb/)
      kbo(:, 3,53,14) = (/ &
     &1.9565e+02_rb,1.4725e+02_rb,1.0014e+02_rb,5.4560e+01_rb,1.5441e-01_rb/)
      kbo(:, 4,53,14) = (/ &
     &3.0901e+02_rb,2.3220e+02_rb,1.5711e+02_rb,8.4327e+01_rb,2.9159e-01_rb/)
      kbo(:, 5,53,14) = (/ &
     &4.6024e+02_rb,3.4547e+02_rb,2.3304e+02_rb,1.2363e+02_rb,5.2012e-01_rb/)
      kbo(:, 1,54,14) = (/ &
     &5.3046e+01_rb,4.0133e+01_rb,2.7816e+01_rb,1.5611e+01_rb,4.2542e-02_rb/)
      kbo(:, 2,54,14) = (/ &
     &1.0074e+02_rb,7.5982e+01_rb,5.2075e+01_rb,2.8803e+01_rb,6.4322e-02_rb/)
      kbo(:, 3,54,14) = (/ &
     &1.7557e+02_rb,1.3215e+02_rb,8.9882e+01_rb,4.8976e+01_rb,1.2163e-01_rb/)
      kbo(:, 4,54,14) = (/ &
     &2.8382e+02_rb,2.1329e+02_rb,1.4431e+02_rb,7.7417e+01_rb,2.3542e-01_rb/)
      kbo(:, 5,54,14) = (/ &
     &4.3075e+02_rb,3.2335e+02_rb,2.1806e+02_rb,1.1557e+02_rb,4.2927e-01_rb/)
      kbo(:, 1,55,14) = (/ &
     &4.5010e+01_rb,3.4076e+01_rb,2.3656e+01_rb,1.3271e+01_rb,3.8508e-02_rb/)
      kbo(:, 2,55,14) = (/ &
     &8.8262e+01_rb,6.6580e+01_rb,4.5655e+01_rb,2.5232e+01_rb,5.7123e-02_rb/)
      kbo(:, 3,55,14) = (/ &
     &1.5805e+02_rb,1.1897e+02_rb,8.0930e+01_rb,4.4073e+01_rb,9.5519e-02_rb/)
      kbo(:, 4,55,14) = (/ &
     &2.6161e+02_rb,1.9662e+02_rb,1.3302e+02_rb,7.1301e+01_rb,1.9000e-01_rb/)
      kbo(:, 5,55,14) = (/ &
     &4.0466e+02_rb,3.0378e+02_rb,2.0480e+02_rb,1.0841e+02_rb,3.5396e-01_rb/)
      kbo(:, 1,56,14) = (/ &
     &3.7999e+01_rb,2.8791e+01_rb,2.0014e+01_rb,1.1214e+01_rb,3.4994e-02_rb/)
      kbo(:, 2,56,14) = (/ &
     &7.7059e+01_rb,5.8140e+01_rb,3.9888e+01_rb,2.2021e+01_rb,5.1685e-02_rb/)
      kbo(:, 3,56,14) = (/ &
     &1.4187e+02_rb,1.0680e+02_rb,7.2655e+01_rb,3.9531e+01_rb,7.7408e-02_rb/)
      kbo(:, 4,56,14) = (/ &
     &2.4069e+02_rb,1.8091e+02_rb,1.2237e+02_rb,6.5530e+01_rb,1.5216e-01_rb/)
      kbo(:, 5,56,14) = (/ &
     &3.8007e+02_rb,2.8533e+02_rb,1.9230e+02_rb,1.0164e+02_rb,2.8842e-01_rb/)
      kbo(:, 1,57,14) = (/ &
     &3.1951e+01_rb,2.4226e+01_rb,1.6863e+01_rb,9.4281e+00_rb,3.3086e-02_rb/)
      kbo(:, 2,57,14) = (/ &
     &6.6995e+01_rb,5.0555e+01_rb,3.4699e+01_rb,1.9133e+01_rb,4.5516e-02_rb/)
      kbo(:, 3,57,14) = (/ &
     &1.2705e+02_rb,9.5650e+01_rb,6.5062e+01_rb,3.5359e+01_rb,6.7958e-02_rb/)
      kbo(:, 4,57,14) = (/ &
     &2.2106e+02_rb,1.6616e+02_rb,1.1237e+02_rb,6.0096e+01_rb,1.2025e-01_rb/)
      kbo(:, 5,57,14) = (/ &
     &3.5683e+02_rb,2.6789e+02_rb,1.8049e+02_rb,9.5240e+01_rb,2.3463e-01_rb/)
      kbo(:, 1,58,14) = (/ &
     &1.3120e+01_rb,1.1427e+01_rb,9.3421e+00_rb,6.3041e+00_rb,2.9290e-02_rb/)
      kbo(:, 2,58,14) = (/ &
     &2.8375e+01_rb,2.4586e+01_rb,1.9815e+01_rb,1.3205e+01_rb,4.1783e-02_rb/)
      kbo(:, 3,58,14) = (/ &
     &5.5462e+01_rb,4.7938e+01_rb,3.8264e+01_rb,2.5126e+01_rb,6.2497e-02_rb/)
      kbo(:, 4,58,14) = (/ &
     &9.8962e+01_rb,8.5397e+01_rb,6.7764e+01_rb,4.3784e+01_rb,9.9579e-02_rb/)
      kbo(:, 5,58,14) = (/ &
     &1.6330e+02_rb,1.4075e+02_rb,1.1126e+02_rb,7.0923e+01_rb,1.9106e-01_rb/)
      kbo(:, 1,59,14) = (/ &
     &1.2384e+01_rb,1.0821e+01_rb,8.8760e+00_rb,6.0092e+00_rb,2.9802e-02_rb/)
      kbo(:, 2,59,14) = (/ &
     &2.7135e+01_rb,2.3592e+01_rb,1.9085e+01_rb,1.2767e+01_rb,4.3735e-02_rb/)
      kbo(:, 3,59,14) = (/ &
     &5.3692e+01_rb,4.6568e+01_rb,3.7314e+01_rb,2.4591e+01_rb,6.7061e-02_rb/)
      kbo(:, 4,59,14) = (/ &
     &9.6831e+01_rb,8.3852e+01_rb,6.6810e+01_rb,4.3324e+01_rb,1.0825e-01_rb/)
      kbo(:, 5,59,14) = (/ &
     &1.6129e+02_rb,1.3951e+02_rb,1.1076e+02_rb,7.0881e+01_rb,1.7652e-01_rb/)
      kbo(:, 1,13,15) = (/ &
     &4.3408e+02_rb,3.2553e+02_rb,2.1704e+02_rb,1.0851e+02_rb,3.3387e-01_rb/)
      kbo(:, 2,13,15) = (/ &
     &6.1113e+02_rb,4.5841e+02_rb,3.0560e+02_rb,1.5279e+02_rb,3.4060e+00_rb/)
      kbo(:, 3,13,15) = (/ &
     &8.2062e+02_rb,6.1547e+02_rb,4.1031e+02_rb,2.0516e+02_rb,7.3889e+00_rb/)
      kbo(:, 4,13,15) = (/ &
     &1.0625e+03_rb,7.9684e+02_rb,5.3120e+02_rb,2.6560e+02_rb,1.3324e+01_rb/)
      kbo(:, 5,13,15) = (/ &
     &1.3314e+03_rb,9.9861e+02_rb,6.6575e+02_rb,3.3287e+02_rb,1.8510e+01_rb/)
      kbo(:, 1,14,15) = (/ &
     &4.8927e+02_rb,3.6696e+02_rb,2.4465e+02_rb,1.2233e+02_rb,1.4639e+00_rb/)
      kbo(:, 2,14,15) = (/ &
     &6.9247e+02_rb,5.1938e+02_rb,3.4624e+02_rb,1.7311e+02_rb,5.0161e+00_rb/)
      kbo(:, 3,14,15) = (/ &
     &9.3693e+02_rb,7.0263e+02_rb,4.6839e+02_rb,2.3422e+02_rb,9.2136e+00_rb/)
      kbo(:, 4,14,15) = (/ &
     &1.2198e+03_rb,9.1484e+02_rb,6.0989e+02_rb,3.0495e+02_rb,1.6015e+01_rb/)
      kbo(:, 5,14,15) = (/ &
     &1.5424e+03_rb,1.1568e+03_rb,7.7122e+02_rb,3.8560e+02_rb,2.0607e+01_rb/)
      kbo(:, 1,15,15) = (/ &
     &5.4419e+02_rb,4.0815e+02_rb,2.7209e+02_rb,1.3605e+02_rb,3.3205e+00_rb/)
      kbo(:, 2,15,15) = (/ &
     &7.6785e+02_rb,5.7588e+02_rb,3.8392e+02_rb,1.9198e+02_rb,6.4585e+00_rb/)
      kbo(:, 3,15,15) = (/ &
     &1.0420e+03_rb,7.8154e+02_rb,5.2102e+02_rb,2.6050e+02_rb,1.2593e+01_rb/)
      kbo(:, 4,15,15) = (/ &
     &1.3650e+03_rb,1.0239e+03_rb,6.8254e+02_rb,3.4127e+02_rb,1.7743e+01_rb/)
      kbo(:, 5,15,15) = (/ &
     &1.7328e+03_rb,1.2995e+03_rb,8.6636e+02_rb,4.3319e+02_rb,2.2779e+01_rb/)
      kbo(:, 1,16,15) = (/ &
     &5.9961e+02_rb,4.4968e+02_rb,2.9978e+02_rb,1.4989e+02_rb,4.3949e+00_rb/)
      kbo(:, 2,16,15) = (/ &
     &8.4044e+02_rb,6.3032e+02_rb,4.2019e+02_rb,2.1011e+02_rb,9.1201e+00_rb/)
      kbo(:, 3,16,15) = (/ &
     &1.1438e+03_rb,8.5770e+02_rb,5.7195e+02_rb,2.8595e+02_rb,1.4709e+01_rb/)
      kbo(:, 4,16,15) = (/ &
     &1.4993e+03_rb,1.1245e+03_rb,7.4965e+02_rb,3.7484e+02_rb,1.9517e+01_rb/)
      kbo(:, 5,16,15) = (/ &
     &1.9026e+03_rb,1.4270e+03_rb,9.5133e+02_rb,4.7569e+02_rb,2.4995e+01_rb/)
      kbo(:, 1,17,15) = (/ &
     &6.5740e+02_rb,4.9304e+02_rb,3.2868e+02_rb,1.6434e+02_rb,5.9877e+00_rb/)
      kbo(:, 2,17,15) = (/ &
     &9.1769e+02_rb,6.8825e+02_rb,4.5884e+02_rb,2.2941e+02_rb,1.1589e+01_rb/)
      kbo(:, 3,17,15) = (/ &
     &1.2441e+03_rb,9.3311e+02_rb,6.2203e+02_rb,3.1103e+02_rb,1.6070e+01_rb/)
      kbo(:, 4,17,15) = (/ &
     &1.6286e+03_rb,1.2214e+03_rb,8.1435e+02_rb,4.0716e+02_rb,2.1263e+01_rb/)
      kbo(:, 5,17,15) = (/ &
     &2.0647e+03_rb,1.5485e+03_rb,1.0323e+03_rb,5.1617e+02_rb,2.7194e+01_rb/)
      kbo(:, 1,18,15) = (/ &
     &7.0743e+02_rb,5.3056e+02_rb,3.5371e+02_rb,1.7685e+02_rb,7.5465e+00_rb/)
      kbo(:, 2,18,15) = (/ &
     &9.8473e+02_rb,7.3856e+02_rb,4.9235e+02_rb,2.4618e+02_rb,1.2708e+01_rb/)
      kbo(:, 3,18,15) = (/ &
     &1.3299e+03_rb,9.9741e+02_rb,6.6495e+02_rb,3.3248e+02_rb,1.7433e+01_rb/)
      kbo(:, 4,18,15) = (/ &
     &1.7379e+03_rb,1.3034e+03_rb,8.6887e+02_rb,4.3447e+02_rb,2.2996e+01_rb/)
      kbo(:, 5,18,15) = (/ &
     &2.2009e+03_rb,1.6506e+03_rb,1.1004e+03_rb,5.5020e+02_rb,2.9358e+01_rb/)
      kbo(:, 1,19,15) = (/ &
     &7.7797e+02_rb,5.8347e+02_rb,3.8898e+02_rb,1.9449e+02_rb,8.9058e+00_rb/)
      kbo(:, 2,19,15) = (/ &
     &1.0795e+03_rb,8.0964e+02_rb,5.3974e+02_rb,2.6988e+02_rb,1.3760e+01_rb/)
      kbo(:, 3,19,15) = (/ &
     &1.4503e+03_rb,1.0877e+03_rb,7.2515e+02_rb,3.6258e+02_rb,1.8739e+01_rb/)
      kbo(:, 4,19,15) = (/ &
     &1.8906e+03_rb,1.4179e+03_rb,9.4523e+02_rb,4.7263e+02_rb,2.4641e+01_rb/)
      kbo(:, 5,19,15) = (/ &
     &2.3863e+03_rb,1.7897e+03_rb,1.1933e+03_rb,5.9656e+02_rb,3.1485e+01_rb/)
      kbo(:, 1,20,15) = (/ &
     &8.7874e+02_rb,6.5907e+02_rb,4.3939e+02_rb,2.1969e+02_rb,9.7556e+00_rb/)
      kbo(:, 2,20,15) = (/ &
     &1.2128e+03_rb,9.0962e+02_rb,6.0638e+02_rb,3.0319e+02_rb,1.4896e+01_rb/)
      kbo(:, 3,20,15) = (/ &
     &1.6209e+03_rb,1.2157e+03_rb,8.1051e+02_rb,4.0524e+02_rb,2.0180e+01_rb/)
      kbo(:, 4,20,15) = (/ &
     &2.1009e+03_rb,1.5756e+03_rb,1.0504e+03_rb,5.2525e+02_rb,2.6433e+01_rb/)
      kbo(:, 5,20,15) = (/ &
     &2.6410e+03_rb,1.9808e+03_rb,1.3205e+03_rb,6.6026e+02_rb,3.3724e+01_rb/)
      kbo(:, 1,21,15) = (/ &
     &9.9673e+02_rb,7.4761e+02_rb,4.9839e+02_rb,2.4919e+02_rb,1.0524e+01_rb/)
      kbo(:, 2,21,15) = (/ &
     &1.3673e+03_rb,1.0255e+03_rb,6.8366e+02_rb,3.4184e+02_rb,1.6075e+01_rb/)
      kbo(:, 3,21,15) = (/ &
     &1.8166e+03_rb,1.3625e+03_rb,9.0829e+02_rb,4.5412e+02_rb,2.1690e+01_rb/)
      kbo(:, 4,21,15) = (/ &
     &2.3404e+03_rb,1.7553e+03_rb,1.1702e+03_rb,5.8508e+02_rb,2.8346e+01_rb/)
      kbo(:, 5,21,15) = (/ &
     &2.9258e+03_rb,2.1942e+03_rb,1.4629e+03_rb,7.3140e+02_rb,3.6033e+01_rb/)
      kbo(:, 1,22,15) = (/ &
     &1.1483e+03_rb,8.6125e+02_rb,5.7415e+02_rb,2.8707e+02_rb,1.1768e+01_rb/)
      kbo(:, 2,22,15) = (/ &
     &1.5630e+03_rb,1.1723e+03_rb,7.8152e+02_rb,3.9076e+02_rb,1.7624e+01_rb/)
      kbo(:, 3,22,15) = (/ &
     &2.0593e+03_rb,1.5445e+03_rb,1.0296e+03_rb,5.1484e+02_rb,2.3654e+01_rb/)
      kbo(:, 4,22,15) = (/ &
     &2.6282e+03_rb,1.9711e+03_rb,1.3140e+03_rb,6.5702e+02_rb,3.0777e+01_rb/)
      kbo(:, 5,22,15) = (/ &
     &3.2625e+03_rb,2.4468e+03_rb,1.6313e+03_rb,8.1566e+02_rb,3.8871e+01_rb/)
      kbo(:, 1,23,15) = (/ &
     &1.3167e+03_rb,9.8756e+02_rb,6.5833e+02_rb,3.2919e+02_rb,1.3119e+01_rb/)
      kbo(:, 2,23,15) = (/ &
     &1.7766e+03_rb,1.3324e+03_rb,8.8830e+02_rb,4.4413e+02_rb,1.9267e+01_rb/)
      kbo(:, 3,23,15) = (/ &
     &2.3173e+03_rb,1.7380e+03_rb,1.1586e+03_rb,5.7933e+02_rb,2.5707e+01_rb/)
      kbo(:, 4,23,15) = (/ &
     &2.9343e+03_rb,2.2007e+03_rb,1.4671e+03_rb,7.3355e+02_rb,3.3186e+01_rb/)
      kbo(:, 5,23,15) = (/ &
     &3.6141e+03_rb,2.7105e+03_rb,1.8070e+03_rb,9.0351e+02_rb,4.1634e+01_rb/)
      kbo(:, 1,24,15) = (/ &
     &1.5139e+03_rb,1.1354e+03_rb,7.5695e+02_rb,3.7848e+02_rb,1.4766e+01_rb/)
      kbo(:, 2,24,15) = (/ &
     &2.0213e+03_rb,1.5159e+03_rb,1.0107e+03_rb,5.0532e+02_rb,2.1039e+01_rb/)
      kbo(:, 3,24,15) = (/ &
     &2.6132e+03_rb,1.9598e+03_rb,1.3066e+03_rb,6.5328e+02_rb,2.7799e+01_rb/)
      kbo(:, 4,24,15) = (/ &
     &3.2798e+03_rb,2.4598e+03_rb,1.6399e+03_rb,8.1993e+02_rb,3.5585e+01_rb/)
      kbo(:, 5,24,15) = (/ &
     &4.0096e+03_rb,3.0073e+03_rb,2.0048e+03_rb,1.0024e+03_rb,4.4343e+01_rb/)
      kbo(:, 1,25,15) = (/ &
     &1.7311e+03_rb,1.2983e+03_rb,8.6555e+02_rb,4.3279e+02_rb,1.6410e+01_rb/)
      kbo(:, 2,25,15) = (/ &
     &2.2880e+03_rb,1.7160e+03_rb,1.1440e+03_rb,5.7204e+02_rb,2.2852e+01_rb/)
      kbo(:, 3,25,15) = (/ &
     &2.9293e+03_rb,2.1970e+03_rb,1.4646e+03_rb,7.3234e+02_rb,2.9897e+01_rb/)
      kbo(:, 4,25,15) = (/ &
     &3.6441e+03_rb,2.7331e+03_rb,1.8220e+03_rb,9.1101e+02_rb,3.8003e+01_rb/)
      kbo(:, 5,25,15) = (/ &
     &4.4209e+03_rb,3.3157e+03_rb,2.2104e+03_rb,1.1052e+03_rb,4.7037e+01_rb/)
      kbo(:, 1,26,15) = (/ &
     &1.9753e+03_rb,1.4814e+03_rb,9.8766e+02_rb,4.9382e+02_rb,1.8018e+01_rb/)
      kbo(:, 2,26,15) = (/ &
     &2.5844e+03_rb,1.9383e+03_rb,1.2922e+03_rb,6.4606e+02_rb,2.4551e+01_rb/)
      kbo(:, 3,26,15) = (/ &
     &3.2758e+03_rb,2.4567e+03_rb,1.6379e+03_rb,8.1896e+02_rb,3.2091e+01_rb/)
      kbo(:, 4,26,15) = (/ &
     &4.0392e+03_rb,3.0293e+03_rb,2.0196e+03_rb,1.0098e+03_rb,4.0504e+01_rb/)
      kbo(:, 5,26,15) = (/ &
     &4.8643e+03_rb,3.6484e+03_rb,2.4323e+03_rb,1.2161e+03_rb,4.9853e+01_rb/)
      kbo(:, 1,27,15) = (/ &
     &2.2372e+03_rb,1.6779e+03_rb,1.1187e+03_rb,5.5932e+02_rb,1.9412e+01_rb/)
      kbo(:, 2,27,15) = (/ &
     &2.8964e+03_rb,2.1723e+03_rb,1.4482e+03_rb,7.2410e+02_rb,2.6199e+01_rb/)
      kbo(:, 3,27,15) = (/ &
     &3.6363e+03_rb,2.7272e+03_rb,1.8181e+03_rb,9.0912e+02_rb,3.4164e+01_rb/)
      kbo(:, 4,27,15) = (/ &
     &4.4487e+03_rb,3.3365e+03_rb,2.2244e+03_rb,1.1122e+03_rb,4.3003e+01_rb/)
      kbo(:, 5,27,15) = (/ &
     &5.3193e+03_rb,3.9894e+03_rb,2.6596e+03_rb,1.3298e+03_rb,5.2701e+01_rb/)
      kbo(:, 1,28,15) = (/ &
     &2.5116e+03_rb,1.8837e+03_rb,1.2558e+03_rb,6.2784e+02_rb,2.0876e+01_rb/)
      kbo(:, 2,28,15) = (/ &
     &3.2189e+03_rb,2.4142e+03_rb,1.6096e+03_rb,8.0476e+02_rb,2.7949e+01_rb/)
      kbo(:, 3,28,15) = (/ &
     &4.0085e+03_rb,3.0060e+03_rb,2.0042e+03_rb,1.0021e+03_rb,3.6156e+01_rb/)
      kbo(:, 4,28,15) = (/ &
     &4.8660e+03_rb,3.6495e+03_rb,2.4329e+03_rb,1.2165e+03_rb,4.5448e+01_rb/)
      kbo(:, 5,28,15) = (/ &
     &5.7764e+03_rb,4.3322e+03_rb,2.8883e+03_rb,1.4442e+03_rb,5.5591e+01_rb/)
      kbo(:, 1,29,15) = (/ &
     &2.7788e+03_rb,2.0842e+03_rb,1.3895e+03_rb,6.9477e+02_rb,2.2561e+01_rb/)
      kbo(:, 2,29,15) = (/ &
     &3.5298e+03_rb,2.6472e+03_rb,1.7648e+03_rb,8.8246e+02_rb,2.9936e+01_rb/)
      kbo(:, 3,29,15) = (/ &
     &4.3597e+03_rb,3.2696e+03_rb,2.1798e+03_rb,1.0899e+03_rb,3.8464e+01_rb/)
      kbo(:, 4,29,15) = (/ &
     &5.2526e+03_rb,3.9395e+03_rb,2.6263e+03_rb,1.3131e+03_rb,4.8125e+01_rb/)
      kbo(:, 5,29,15) = (/ &
     &6.1956e+03_rb,4.6467e+03_rb,3.0979e+03_rb,1.5489e+03_rb,5.8615e+01_rb/)
      kbo(:, 1,30,15) = (/ &
     &3.0369e+03_rb,2.2777e+03_rb,1.5185e+03_rb,7.5926e+02_rb,2.4413e+01_rb/)
      kbo(:, 2,30,15) = (/ &
     &3.8244e+03_rb,2.8684e+03_rb,1.9122e+03_rb,9.5609e+02_rb,3.2061e+01_rb/)
      kbo(:, 3,30,15) = (/ &
     &4.6859e+03_rb,3.5141e+03_rb,2.3430e+03_rb,1.1715e+03_rb,4.0933e+01_rb/)
      kbo(:, 4,30,15) = (/ &
     &5.6065e+03_rb,4.2050e+03_rb,2.8032e+03_rb,1.4016e+03_rb,5.0897e+01_rb/)
      kbo(:, 5,30,15) = (/ &
     &6.5717e+03_rb,4.9287e+03_rb,3.2858e+03_rb,1.6429e+03_rb,6.1721e+01_rb/)
      kbo(:, 1,31,15) = (/ &
     &3.2648e+03_rb,2.4486e+03_rb,1.6324e+03_rb,8.1618e+02_rb,2.6426e+01_rb/)
      kbo(:, 2,31,15) = (/ &
     &4.0756e+03_rb,3.0566e+03_rb,2.0378e+03_rb,1.0190e+03_rb,3.4471e+01_rb/)
      kbo(:, 3,31,15) = (/ &
     &4.9563e+03_rb,3.7173e+03_rb,2.4782e+03_rb,1.2392e+03_rb,4.3666e+01_rb/)
      kbo(:, 4,31,15) = (/ &
     &5.8911e+03_rb,4.4181e+03_rb,2.9455e+03_rb,1.4727e+03_rb,5.3750e+01_rb/)
      kbo(:, 5,31,15) = (/ &
     &6.8637e+03_rb,5.1477e+03_rb,3.4319e+03_rb,1.7160e+03_rb,6.4980e+01_rb/)
      kbo(:, 1,32,15) = (/ &
     &3.4972e+03_rb,2.6230e+03_rb,1.7487e+03_rb,8.7434e+02_rb,2.8607e+01_rb/)
      kbo(:, 2,32,15) = (/ &
     &4.3306e+03_rb,3.2480e+03_rb,2.1653e+03_rb,1.0827e+03_rb,3.6943e+01_rb/)
      kbo(:, 3,32,15) = (/ &
     &5.2288e+03_rb,3.9216e+03_rb,2.6144e+03_rb,1.3072e+03_rb,4.6342e+01_rb/)
      kbo(:, 4,32,15) = (/ &
     &6.1756e+03_rb,4.6314e+03_rb,3.0875e+03_rb,1.5439e+03_rb,5.6744e+01_rb/)
      kbo(:, 5,32,15) = (/ &
     &7.1543e+03_rb,5.3657e+03_rb,3.5769e+03_rb,1.7884e+03_rb,6.8387e+01_rb/)
      kbo(:, 1,33,15) = (/ &
     &3.7323e+03_rb,2.7993e+03_rb,1.8662e+03_rb,9.3310e+02_rb,3.0880e+01_rb/)
      kbo(:, 2,33,15) = (/ &
     &4.5855e+03_rb,3.4391e+03_rb,2.2927e+03_rb,1.1463e+03_rb,3.9458e+01_rb/)
      kbo(:, 3,33,15) = (/ &
     &5.4988e+03_rb,4.1239e+03_rb,2.7494e+03_rb,1.3747e+03_rb,4.9066e+01_rb/)
      kbo(:, 4,33,15) = (/ &
     &6.4556e+03_rb,4.8415e+03_rb,3.2279e+03_rb,1.6139e+03_rb,5.9834e+01_rb/)
      kbo(:, 5,33,15) = (/ &
     &7.4364e+03_rb,5.5775e+03_rb,3.7183e+03_rb,1.8591e+03_rb,7.1936e+01_rb/)
      kbo(:, 1,34,15) = (/ &
     &3.9743e+03_rb,2.9808e+03_rb,1.9872e+03_rb,9.9359e+02_rb,3.2850e+01_rb/)
      kbo(:, 2,34,15) = (/ &
     &4.8517e+03_rb,3.6384e+03_rb,2.4257e+03_rb,1.2129e+03_rb,4.1615e+01_rb/)
      kbo(:, 3,34,15) = (/ &
     &5.7848e+03_rb,4.3386e+03_rb,2.8923e+03_rb,1.4462e+03_rb,5.1514e+01_rb/)
      kbo(:, 4,34,15) = (/ &
     &6.7566e+03_rb,5.0673e+03_rb,3.3783e+03_rb,1.6891e+03_rb,6.2632e+01_rb/)
      kbo(:, 5,34,15) = (/ &
     &7.7467e+03_rb,5.8099e+03_rb,3.8734e+03_rb,1.9367e+03_rb,7.5129e+01_rb/)
      kbo(:, 1,35,15) = (/ &
     &4.1392e+03_rb,3.1044e+03_rb,2.0697e+03_rb,1.0348e+03_rb,3.4005e+01_rb/)
      kbo(:, 2,35,15) = (/ &
     &5.0338e+03_rb,3.7756e+03_rb,2.5169e+03_rb,1.2585e+03_rb,4.2889e+01_rb/)
      kbo(:, 3,35,15) = (/ &
     &5.9824e+03_rb,4.4867e+03_rb,2.9910e+03_rb,1.4955e+03_rb,5.2983e+01_rb/)
      kbo(:, 4,35,15) = (/ &
     &6.9652e+03_rb,5.2239e+03_rb,3.4826e+03_rb,1.7414e+03_rb,6.4327e+01_rb/)
      kbo(:, 5,35,15) = (/ &
     &7.9642e+03_rb,5.9733e+03_rb,3.9822e+03_rb,1.9911e+03_rb,7.7087e+01_rb/)
      kbo(:, 1,36,15) = (/ &
     &4.2146e+03_rb,3.1608e+03_rb,2.1072e+03_rb,1.0536e+03_rb,3.4225e+01_rb/)
      kbo(:, 2,36,15) = (/ &
     &5.1200e+03_rb,3.8401e+03_rb,2.5599e+03_rb,1.2800e+03_rb,4.3139e+01_rb/)
      kbo(:, 3,36,15) = (/ &
     &6.0791e+03_rb,4.5594e+03_rb,3.0396e+03_rb,1.5198e+03_rb,5.3291e+01_rb/)
      kbo(:, 4,36,15) = (/ &
     &7.0718e+03_rb,5.3037e+03_rb,3.5357e+03_rb,1.7679e+03_rb,6.4713e+01_rb/)
      kbo(:, 5,36,15) = (/ &
     &8.0803e+03_rb,6.0601e+03_rb,4.0401e+03_rb,2.0201e+03_rb,7.7535e+01_rb/)
      kbo(:, 1,37,15) = (/ &
     &4.1830e+03_rb,3.1372e+03_rb,2.0915e+03_rb,1.0457e+03_rb,3.3171e+01_rb/)
      kbo(:, 2,37,15) = (/ &
     &5.0955e+03_rb,3.8218e+03_rb,2.5477e+03_rb,1.2739e+03_rb,4.1938e+01_rb/)
      kbo(:, 3,37,15) = (/ &
     &6.0651e+03_rb,4.5490e+03_rb,3.0327e+03_rb,1.5163e+03_rb,5.1957e+01_rb/)
      kbo(:, 4,37,15) = (/ &
     &7.0708e+03_rb,5.3029e+03_rb,3.5354e+03_rb,1.7678e+03_rb,6.3244e+01_rb/)
      kbo(:, 5,37,15) = (/ &
     &8.0937e+03_rb,6.0708e+03_rb,4.0470e+03_rb,2.0235e+03_rb,7.5914e+01_rb/)
      kbo(:, 1,38,15) = (/ &
     &4.1367e+03_rb,3.1023e+03_rb,2.0683e+03_rb,1.0342e+03_rb,3.2061e+01_rb/)
      kbo(:, 2,38,15) = (/ &
     &5.0552e+03_rb,3.7913e+03_rb,2.5275e+03_rb,1.2638e+03_rb,4.0668e+01_rb/)
      kbo(:, 3,38,15) = (/ &
     &6.0331e+03_rb,4.5250e+03_rb,3.0168e+03_rb,1.5083e+03_rb,5.0542e+01_rb/)
      kbo(:, 4,38,15) = (/ &
     &7.0503e+03_rb,5.2874e+03_rb,3.5252e+03_rb,1.7626e+03_rb,6.1670e+01_rb/)
      kbo(:, 5,38,15) = (/ &
     &8.0886e+03_rb,6.0662e+03_rb,4.0441e+03_rb,2.0220e+03_rb,7.4175e+01_rb/)
      kbo(:, 1,39,15) = (/ &
     &4.0836e+03_rb,3.0625e+03_rb,2.0418e+03_rb,1.0208e+03_rb,3.0973e+01_rb/)
      kbo(:, 2,39,15) = (/ &
     &5.0068e+03_rb,3.7549e+03_rb,2.5034e+03_rb,1.2516e+03_rb,3.9423e+01_rb/)
      kbo(:, 3,39,15) = (/ &
     &5.9917e+03_rb,4.4934e+03_rb,2.9959e+03_rb,1.4979e+03_rb,4.9140e+01_rb/)
      kbo(:, 4,39,15) = (/ &
     &7.0190e+03_rb,5.2646e+03_rb,3.5096e+03_rb,1.7548e+03_rb,6.0110e+01_rb/)
      kbo(:, 5,39,15) = (/ &
     &8.0703e+03_rb,6.0526e+03_rb,4.0352e+03_rb,2.0176e+03_rb,7.2442e+01_rb/)
      kbo(:, 1,40,15) = (/ &
     &3.9731e+03_rb,2.9800e+03_rb,1.9865e+03_rb,9.9327e+02_rb,2.9119e+01_rb/)
      kbo(:, 2,40,15) = (/ &
     &4.9010e+03_rb,3.6760e+03_rb,2.4505e+03_rb,1.2252e+03_rb,3.7284e+01_rb/)
      kbo(:, 3,40,15) = (/ &
     &5.8967e+03_rb,4.4226e+03_rb,2.9482e+03_rb,1.4742e+03_rb,4.6706e+01_rb/)
      kbo(:, 4,40,15) = (/ &
     &6.9401e+03_rb,5.2049e+03_rb,3.4701e+03_rb,1.7350e+03_rb,5.7383e+01_rb/)
      kbo(:, 5,40,15) = (/ &
     &8.0134e+03_rb,6.0096e+03_rb,4.0066e+03_rb,2.0033e+03_rb,6.9395e+01_rb/)
      kbo(:, 1,41,15) = (/ &
     &3.8503e+03_rb,2.8877e+03_rb,1.9252e+03_rb,9.6258e+02_rb,2.7224e+01_rb/)
      kbo(:, 2,41,15) = (/ &
     &4.7815e+03_rb,3.5862e+03_rb,2.3908e+03_rb,1.1953e+03_rb,3.5101e+01_rb/)
      kbo(:, 3,41,15) = (/ &
     &5.7872e+03_rb,4.3401e+03_rb,2.8935e+03_rb,1.4467e+03_rb,4.4197e+01_rb/)
      kbo(:, 4,41,15) = (/ &
     &6.8458e+03_rb,5.1341e+03_rb,3.4225e+03_rb,1.7115e+03_rb,5.4583e+01_rb/)
      kbo(:, 5,41,15) = (/ &
     &7.9396e+03_rb,5.9547e+03_rb,3.9699e+03_rb,1.9849e+03_rb,6.6259e+01_rb/)
      kbo(:, 1,42,15) = (/ &
     &3.7218e+03_rb,2.7913e+03_rb,1.8581e+03_rb,9.3043e+02_rb,2.5397e+01_rb/)
      kbo(:, 2,42,15) = (/ &
     &4.6552e+03_rb,3.4913e+03_rb,2.3275e+03_rb,1.1638e+03_rb,3.2985e+01_rb/)
      kbo(:, 3,42,15) = (/ &
     &5.6676e+03_rb,4.2515e+03_rb,2.8342e+03_rb,1.4169e+03_rb,4.1777e+01_rb/)
      kbo(:, 4,42,15) = (/ &
     &6.7404e+03_rb,5.0552e+03_rb,3.3706e+03_rb,1.6852e+03_rb,5.1833e+01_rb/)
      kbo(:, 5,42,15) = (/ &
     &7.8548e+03_rb,5.8911e+03_rb,3.9277e+03_rb,1.9637e+03_rb,6.3188e+01_rb/)
      kbo(:, 1,43,15) = (/ &
     &3.5738e+03_rb,2.6801e+03_rb,1.7870e+03_rb,8.9342e+02_rb,2.3264e+01_rb/)
      kbo(:, 2,43,15) = (/ &
     &4.5111e+03_rb,3.3834e+03_rb,2.2559e+03_rb,1.1277e+03_rb,3.0494e+01_rb/)
      kbo(:, 3,43,15) = (/ &
     &5.5364e+03_rb,4.1522e+03_rb,2.7681e+03_rb,1.3841e+03_rb,3.8909e+01_rb/)
      kbo(:, 4,43,15) = (/ &
     &6.6297e+03_rb,4.9723e+03_rb,3.3147e+03_rb,1.6574e+03_rb,4.8575e+01_rb/)
      kbo(:, 5,43,15) = (/ &
     &7.7713e+03_rb,5.8285e+03_rb,3.8856e+03_rb,1.9429e+03_rb,5.9539e+01_rb/)
      kbo(:, 1,44,15) = (/ &
     &3.4144e+03_rb,2.5608e+03_rb,1.7072e+03_rb,8.5364e+02_rb,2.1065e+01_rb/)
      kbo(:, 2,44,15) = (/ &
     &4.3569e+03_rb,3.2674e+03_rb,2.1784e+03_rb,1.0893e+03_rb,2.7929e+01_rb/)
      kbo(:, 3,44,15) = (/ &
     &5.3944e+03_rb,4.0458e+03_rb,2.6971e+03_rb,1.3486e+03_rb,3.5943e+01_rb/)
      kbo(:, 4,44,15) = (/ &
     &6.5105e+03_rb,4.8828e+03_rb,3.2554e+03_rb,1.6276e+03_rb,4.5182e+01_rb/)
      kbo(:, 5,44,15) = (/ &
     &7.6829e+03_rb,5.7622e+03_rb,3.8414e+03_rb,1.9208e+03_rb,5.5715e+01_rb/)
      kbo(:, 1,45,15) = (/ &
     &3.2505e+03_rb,2.4379e+03_rb,1.6254e+03_rb,8.1271e+02_rb,1.8984e+01_rb/)
      kbo(:, 2,45,15) = (/ &
     &4.1974e+03_rb,3.1481e+03_rb,2.0989e+03_rb,1.0493e+03_rb,2.5493e+01_rb/)
      kbo(:, 3,45,15) = (/ &
     &5.2466e+03_rb,3.9350e+03_rb,2.6232e+03_rb,1.3117e+03_rb,3.3113e+01_rb/)
      kbo(:, 4,45,15) = (/ &
     &6.3833e+03_rb,4.7875e+03_rb,3.1918e+03_rb,1.5958e+03_rb,4.1930e+01_rb/)
      kbo(:, 5,45,15) = (/ &
     &7.5868e+03_rb,5.6901e+03_rb,3.7934e+03_rb,1.8967e+03_rb,5.2027e+01_rb/)
      kbo(:, 1,46,15) = (/ &
     &3.0662e+03_rb,2.2996e+03_rb,1.5331e+03_rb,7.6672e+02_rb,1.6905e+01_rb/)
      kbo(:, 2,46,15) = (/ &
     &4.0155e+03_rb,3.0116e+03_rb,2.0079e+03_rb,1.0039e+03_rb,2.3041e+01_rb/)
      kbo(:, 3,46,15) = (/ &
     &5.0741e+03_rb,3.8056e+03_rb,2.5370e+03_rb,1.2685e+03_rb,3.0241e+01_rb/)
      kbo(:, 4,46,15) = (/ &
     &6.2294e+03_rb,4.6721e+03_rb,3.1149e+03_rb,1.5573e+03_rb,3.8622e+01_rb/)
      kbo(:, 5,46,15) = (/ &
     &7.4637e+03_rb,5.5978e+03_rb,3.7320e+03_rb,1.8660e+03_rb,4.8257e+01_rb/)
      kbo(:, 1,47,15) = (/ &
     &2.8472e+03_rb,2.1354e+03_rb,1.4236e+03_rb,7.1208e+02_rb,1.4723e+01_rb/)
      kbo(:, 2,47,15) = (/ &
     &3.7952e+03_rb,2.8464e+03_rb,1.8976e+03_rb,9.4874e+02_rb,2.0451e+01_rb/)
      kbo(:, 3,47,15) = (/ &
     &4.8581e+03_rb,3.6436e+03_rb,2.4290e+03_rb,1.2145e+03_rb,2.7203e+01_rb/)
      kbo(:, 4,47,15) = (/ &
     &6.0303e+03_rb,4.5227e+03_rb,3.0151e+03_rb,1.5075e+03_rb,3.5112e+01_rb/)
      kbo(:, 5,47,15) = (/ &
     &7.2931e+03_rb,5.4698e+03_rb,3.6467e+03_rb,1.8233e+03_rb,4.4239e+01_rb/)
      kbo(:, 1,48,15) = (/ &
     &2.6253e+03_rb,1.9691e+03_rb,1.3127e+03_rb,6.5679e+02_rb,1.2715e+01_rb/)
      kbo(:, 2,48,15) = (/ &
     &3.5692e+03_rb,2.6769e+03_rb,1.7846e+03_rb,8.9231e+02_rb,1.8030e+01_rb/)
      kbo(:, 3,48,15) = (/ &
     &4.6375e+03_rb,3.4782e+03_rb,2.3188e+03_rb,1.1595e+03_rb,2.4364e+01_rb/)
      kbo(:, 4,48,15) = (/ &
     &5.8244e+03_rb,4.3683e+03_rb,2.9120e+03_rb,1.4560e+03_rb,3.1792e+01_rb/)
      kbo(:, 5,48,15) = (/ &
     &7.1141e+03_rb,5.3356e+03_rb,3.5570e+03_rb,1.7785e+03_rb,4.0424e+01_rb/)
      kbo(:, 1,49,15) = (/ &
     &2.4040e+03_rb,1.8030e+03_rb,1.2020e+03_rb,6.0155e+02_rb,1.0861e+01_rb/)
      kbo(:, 2,49,15) = (/ &
     &3.3401e+03_rb,2.5051e+03_rb,1.6700e+03_rb,8.3512e+02_rb,1.5777e+01_rb/)
      kbo(:, 3,49,15) = (/ &
     &4.4127e+03_rb,3.3095e+03_rb,2.2063e+03_rb,1.1031e+03_rb,2.1693e+01_rb/)
      kbo(:, 4,49,15) = (/ &
     &5.6115e+03_rb,4.2087e+03_rb,2.8058e+03_rb,1.4030e+03_rb,2.8659e+01_rb/)
      kbo(:, 5,49,15) = (/ &
     &6.9274e+03_rb,5.1956e+03_rb,3.4636e+03_rb,1.7318e+03_rb,3.6808e+01_rb/)
      kbo(:, 1,50,15) = (/ &
     &2.2030e+03_rb,1.6523e+03_rb,1.1015e+03_rb,5.5137e+02_rb,9.2813e+00_rb/)
      kbo(:, 2,50,15) = (/ &
     &3.1297e+03_rb,2.3473e+03_rb,1.5649e+03_rb,7.8265e+02_rb,1.3818e+01_rb/)
      kbo(:, 3,50,15) = (/ &
     &4.2081e+03_rb,3.1561e+03_rb,2.1041e+03_rb,1.0519e+03_rb,1.9341e+01_rb/)
      kbo(:, 4,50,15) = (/ &
     &5.4214e+03_rb,4.0660e+03_rb,2.7107e+03_rb,1.3553e+03_rb,2.5902e+01_rb/)
      kbo(:, 5,50,15) = (/ &
     &6.7641e+03_rb,5.0731e+03_rb,3.3821e+03_rb,1.6910e+03_rb,3.3602e+01_rb/)
      kbo(:, 1,51,15) = (/ &
     &2.0128e+03_rb,1.5096e+03_rb,1.0064e+03_rb,5.0390e+02_rb,7.8927e+00_rb/)
      kbo(:, 2,51,15) = (/ &
     &2.9276e+03_rb,2.1957e+03_rb,1.4638e+03_rb,7.3206e+02_rb,1.2046e+01_rb/)
      kbo(:, 3,51,15) = (/ &
     &4.0098e+03_rb,3.0073e+03_rb,2.0049e+03_rb,1.0025e+03_rb,1.7204e+01_rb/)
      kbo(:, 4,51,15) = (/ &
     &5.2399e+03_rb,3.9299e+03_rb,2.6200e+03_rb,1.3100e+03_rb,2.3387e+01_rb/)
      kbo(:, 5,51,15) = (/ &
     &6.6101e+03_rb,4.9575e+03_rb,3.3050e+03_rb,1.6525e+03_rb,3.0654e+01_rb/)
      kbo(:, 1,52,15) = (/ &
     &1.8272e+03_rb,1.3704e+03_rb,9.1373e+02_rb,4.5761e+02_rb,6.6428e+00_rb/)
      kbo(:, 2,52,15) = (/ &
     &2.7256e+03_rb,2.0442e+03_rb,1.3627e+03_rb,6.8167e+02_rb,1.0409e+01_rb/)
      kbo(:, 3,52,15) = (/ &
     &3.8098e+03_rb,2.8573e+03_rb,1.9049e+03_rb,9.5240e+02_rb,1.5205e+01_rb/)
      kbo(:, 4,52,15) = (/ &
     &5.0575e+03_rb,3.7931e+03_rb,2.5288e+03_rb,1.2643e+03_rb,2.1008e+01_rb/)
      kbo(:, 5,52,15) = (/ &
     &6.4544e+03_rb,4.8408e+03_rb,3.2272e+03_rb,1.6135e+03_rb,2.7863e+01_rb/)
      kbo(:, 1,53,15) = (/ &
     &1.6455e+03_rb,1.2341e+03_rb,8.2275e+02_rb,4.1218e+02_rb,5.5292e+00_rb/)
      kbo(:, 2,53,15) = (/ &
     &2.5225e+03_rb,1.8919e+03_rb,1.2613e+03_rb,6.3102e+02_rb,8.9199e+00_rb/)
      kbo(:, 3,53,15) = (/ &
     &3.6054e+03_rb,2.7042e+03_rb,1.8028e+03_rb,9.0141e+02_rb,1.3342e+01_rb/)
      kbo(:, 4,53,15) = (/ &
     &4.8703e+03_rb,3.6527e+03_rb,2.4352e+03_rb,1.2176e+03_rb,1.8766e+01_rb/)
      kbo(:, 5,53,15) = (/ &
     &6.2970e+03_rb,4.7228e+03_rb,3.1485e+03_rb,1.5742e+03_rb,2.5231e+01_rb/)
      kbo(:, 1,54,15) = (/ &
     &1.4905e+03_rb,1.1179e+03_rb,7.4526e+02_rb,3.7343e+02_rb,4.6241e+00_rb/)
      kbo(:, 2,54,15) = (/ &
     &2.3494e+03_rb,1.7621e+03_rb,1.1747e+03_rb,5.8778e+02_rb,7.6666e+00_rb/)
      kbo(:, 3,54,15) = (/ &
     &3.4317e+03_rb,2.5738e+03_rb,1.7159e+03_rb,8.5793e+02_rb,1.1744e+01_rb/)
      kbo(:, 4,54,15) = (/ &
     &4.7173e+03_rb,3.5380e+03_rb,2.3587e+03_rb,1.1793e+03_rb,1.6836e+01_rb/)
      kbo(:, 5,54,15) = (/ &
     &6.1810e+03_rb,4.6357e+03_rb,3.0905e+03_rb,1.5452e+03_rb,2.2955e+01_rb/)
      kbo(:, 1,55,15) = (/ &
     &1.3503e+03_rb,1.0127e+03_rb,6.7517e+02_rb,3.3837e+02_rb,3.8496e+00_rb/)
      kbo(:, 2,55,15) = (/ &
     &2.1922e+03_rb,1.6441e+03_rb,1.0961e+03_rb,5.4853e+02_rb,6.5708e+00_rb/)
      kbo(:, 3,55,15) = (/ &
     &3.2739e+03_rb,2.4554e+03_rb,1.6370e+03_rb,8.1853e+02_rb,1.0320e+01_rb/)
      kbo(:, 4,55,15) = (/ &
     &4.5818e+03_rb,3.4364e+03_rb,2.2910e+03_rb,1.1455e+03_rb,1.5094e+01_rb/)
      kbo(:, 5,55,15) = (/ &
     &6.0888e+03_rb,4.5666e+03_rb,3.0444e+03_rb,1.5222e+03_rb,2.0876e+01_rb/)
      kbo(:, 1,56,15) = (/ &
     &1.2164e+03_rb,9.1227e+02_rb,6.0818e+02_rb,3.0487e+02_rb,3.1724e+00_rb/)
      kbo(:, 2,56,15) = (/ &
     &2.0376e+03_rb,1.5282e+03_rb,1.0188e+03_rb,5.0989e+02_rb,5.5890e+00_rb/)
      kbo(:, 3,56,15) = (/ &
     &3.1175e+03_rb,2.3381e+03_rb,1.5588e+03_rb,7.7947e+02_rb,9.0060e+00_rb/)
      kbo(:, 4,56,15) = (/ &
     &4.4491e+03_rb,3.3368e+03_rb,2.2246e+03_rb,1.1123e+03_rb,1.3463e+01_rb/)
      kbo(:, 5,56,15) = (/ &
     &6.0027e+03_rb,4.5020e+03_rb,3.0014e+03_rb,1.5007e+03_rb,1.8912e+01_rb/)
      kbo(:, 1,57,15) = (/ &
     &1.0889e+03_rb,8.1664e+02_rb,5.4443e+02_rb,2.7298e+02_rb,2.5821e+00_rb/)
      kbo(:, 2,57,15) = (/ &
     &1.8871e+03_rb,1.4154e+03_rb,9.4357e+02_rb,4.7231e+02_rb,4.7102e+00_rb/)
      kbo(:, 3,57,15) = (/ &
     &2.9648e+03_rb,2.2236e+03_rb,1.4824e+03_rb,7.4131e+02_rb,7.7911e+00_rb/)
      kbo(:, 4,57,15) = (/ &
     &4.3194e+03_rb,3.2395e+03_rb,2.1597e+03_rb,1.0798e+03_rb,1.1924e+01_rb/)
      kbo(:, 5,57,15) = (/ &
     &5.9252e+03_rb,4.4439e+03_rb,2.9626e+03_rb,1.4813e+03_rb,1.7058e+01_rb/)
      kbo(:, 1,58,15) = (/ &
     &4.7296e+02_rb,4.0724e+02_rb,3.1867e+02_rb,1.9341e+02_rb,2.1052e+00_rb/)
      kbo(:, 2,58,15) = (/ &
     &8.4958e+02_rb,7.3152e+02_rb,5.7242e+02_rb,3.4682e+02_rb,3.9643e+00_rb/)
      kbo(:, 3,58,15) = (/ &
     &1.3719e+03_rb,1.1812e+03_rb,9.2434e+02_rb,5.5948e+02_rb,6.7352e+00_rb/)
      kbo(:, 4,58,15) = (/ &
     &2.0410e+03_rb,1.7574e+03_rb,1.3752e+03_rb,8.3221e+02_rb,1.0550e+01_rb/)
      kbo(:, 5,58,15) = (/ &
     &2.8472e+03_rb,2.4515e+03_rb,1.9184e+03_rb,1.1609e+03_rb,1.5395e+01_rb/)
      kbo(:, 1,59,15) = (/ &
     &4.5680e+02_rb,3.9477e+02_rb,3.1046e+02_rb,1.8976e+02_rb,1.9256e+00_rb/)
      kbo(:, 2,59,15) = (/ &
     &8.3327e+02_rb,7.2012e+02_rb,5.6632e+02_rb,3.4555e+02_rb,3.6779e+00_rb/)
      kbo(:, 3,59,15) = (/ &
     &1.3611e+03_rb,1.1763e+03_rb,9.2510e+02_rb,5.6393e+02_rb,6.3201e+00_rb/)
      kbo(:, 4,59,15) = (/ &
     &2.0429e+03_rb,1.7655e+03_rb,1.3884e+03_rb,8.4623e+02_rb,9.9804e+00_rb/)
      kbo(:, 5,59,15) = (/ &
     &2.8700e+03_rb,2.4803e+03_rb,1.9506e+03_rb,1.1889e+03_rb,1.4739e+01_rb/)
      kbo(:, 1,13,16) = (/ &
     &4.8122e+02_rb,3.6087e+02_rb,2.4060e+02_rb,1.2030e+02_rb,7.3993e+00_rb/)
      kbo(:, 2,13,16) = (/ &
     &6.9206e+02_rb,5.1907e+02_rb,3.4603e+02_rb,1.7302e+02_rb,1.5501e+01_rb/)
      kbo(:, 3,13,16) = (/ &
     &9.4738e+02_rb,7.1056e+02_rb,4.7372e+02_rb,2.3683e+02_rb,2.1040e+01_rb/)
      kbo(:, 4,13,16) = (/ &
     &1.2406e+03_rb,9.3042e+02_rb,6.2029e+02_rb,3.1014e+02_rb,2.9139e+01_rb/)
      kbo(:, 5,13,16) = (/ &
     &1.5703e+03_rb,1.1778e+03_rb,7.8522e+02_rb,3.9259e+02_rb,3.9308e+01_rb/)
      kbo(:, 1,14,16) = (/ &
     &5.7576e+02_rb,4.3182e+02_rb,2.8788e+02_rb,1.4394e+02_rb,1.4009e+01_rb/)
      kbo(:, 2,14,16) = (/ &
     &8.3244e+02_rb,6.2431e+02_rb,4.1619e+02_rb,2.0809e+02_rb,1.9135e+01_rb/)
      kbo(:, 3,14,16) = (/ &
     &1.1392e+03_rb,8.5436e+02_rb,5.6956e+02_rb,2.8480e+02_rb,2.5798e+01_rb/)
      kbo(:, 4,14,16) = (/ &
     &1.4970e+03_rb,1.1226e+03_rb,7.4844e+02_rb,3.7423e+02_rb,3.5612e+01_rb/)
      kbo(:, 5,14,16) = (/ &
     &1.8999e+03_rb,1.4249e+03_rb,9.4985e+02_rb,4.7498e+02_rb,4.7897e+01_rb/)
      kbo(:, 1,15,16) = (/ &
     &6.7686e+02_rb,5.0758e+02_rb,3.3845e+02_rb,1.6922e+02_rb,1.7232e+01_rb/)
      kbo(:, 2,15,16) = (/ &
     &9.7585e+02_rb,7.3193e+02_rb,4.8790e+02_rb,2.4395e+02_rb,2.3521e+01_rb/)
      kbo(:, 3,15,16) = (/ &
     &1.3377e+03_rb,1.0032e+03_rb,6.6881e+02_rb,3.3441e+02_rb,3.1740e+01_rb/)
      kbo(:, 4,15,16) = (/ &
     &1.7575e+03_rb,1.3181e+03_rb,8.7876e+02_rb,4.3935e+02_rb,4.3321e+01_rb/)
      kbo(:, 5,15,16) = (/ &
     &2.2308e+03_rb,1.6730e+03_rb,1.1154e+03_rb,5.5774e+02_rb,5.7999e+01_rb/)
      kbo(:, 1,16,16) = (/ &
     &7.8637e+02_rb,5.8983e+02_rb,3.9321e+02_rb,1.9661e+02_rb,2.1088e+01_rb/)
      kbo(:, 2,16,16) = (/ &
     &1.1333e+03_rb,8.4995e+02_rb,5.6666e+02_rb,2.8332e+02_rb,2.8690e+01_rb/)
      kbo(:, 3,16,16) = (/ &
     &1.5497e+03_rb,1.1623e+03_rb,7.7493e+02_rb,3.8745e+02_rb,3.8949e+01_rb/)
      kbo(:, 4,16,16) = (/ &
     &2.0330e+03_rb,1.5247e+03_rb,1.0164e+03_rb,5.0821e+02_rb,5.2500e+01_rb/)
      kbo(:, 5,16,16) = (/ &
     &2.5789e+03_rb,1.9342e+03_rb,1.2895e+03_rb,6.4478e+02_rb,6.9505e+01_rb/)
      kbo(:, 1,17,16) = (/ &
     &9.1371e+02_rb,6.8529e+02_rb,4.5686e+02_rb,2.2843e+02_rb,2.5518e+01_rb/)
      kbo(:, 2,17,16) = (/ &
     &1.3116e+03_rb,9.8366e+02_rb,6.5574e+02_rb,3.2788e+02_rb,3.4904e+01_rb/)
      kbo(:, 3,17,16) = (/ &
     &1.7880e+03_rb,1.3410e+03_rb,8.9396e+02_rb,4.4698e+02_rb,4.7412e+01_rb/)
      kbo(:, 4,17,16) = (/ &
     &2.3411e+03_rb,1.7558e+03_rb,1.1705e+03_rb,5.8526e+02_rb,6.3218e+01_rb/)
      kbo(:, 5,17,16) = (/ &
     &2.9603e+03_rb,2.2204e+03_rb,1.4801e+03_rb,7.4006e+02_rb,8.2344e+01_rb/)
      kbo(:, 1,18,16) = (/ &
     &1.0453e+03_rb,7.8401e+02_rb,5.2267e+02_rb,2.6132e+02_rb,3.0565e+01_rb/)
      kbo(:, 2,18,16) = (/ &
     &1.4935e+03_rb,1.1202e+03_rb,7.4674e+02_rb,3.7341e+02_rb,4.2204e+01_rb/)
      kbo(:, 3,18,16) = (/ &
     &2.0309e+03_rb,1.5232e+03_rb,1.0154e+03_rb,5.0502e+02_rb,5.7127e+01_rb/)
      kbo(:, 4,18,16) = (/ &
     &2.6479e+03_rb,1.9862e+03_rb,1.3241e+03_rb,6.6202e+02_rb,7.5429e+01_rb/)
      kbo(:, 5,18,16) = (/ &
     &3.3399e+03_rb,2.5049e+03_rb,1.6700e+03_rb,8.3500e+02_rb,9.7021e+01_rb/)
      kbo(:, 1,19,16) = (/ &
     &1.2157e+03_rb,9.1179e+02_rb,6.0790e+02_rb,3.0394e+02_rb,3.6497e+01_rb/)
      kbo(:, 2,19,16) = (/ &
     &1.7308e+03_rb,1.2981e+03_rb,8.6539e+02_rb,4.3270e+02_rb,5.0186e+01_rb/)
      kbo(:, 3,19,16) = (/ &
     &2.3421e+03_rb,1.7565e+03_rb,1.1710e+03_rb,5.8556e+02_rb,6.7960e+01_rb/)
      kbo(:, 4,19,16) = (/ &
     &3.0436e+03_rb,2.2829e+03_rb,1.5219e+03_rb,7.6093e+02_rb,8.8940e+01_rb/)
      kbo(:, 5,19,16) = (/ &
     &3.8257e+03_rb,2.8691e+03_rb,1.9129e+03_rb,9.5637e+02_rb,1.1320e+02_rb/)
      kbo(:, 1,20,16) = (/ &
     &1.4499e+03_rb,1.0875e+03_rb,7.2502e+02_rb,3.6250e+02_rb,4.3451e+01_rb/)
      kbo(:, 2,20,16) = (/ &
     &2.0494e+03_rb,1.5371e+03_rb,1.0247e+03_rb,5.1236e+02_rb,5.9923e+01_rb/)
      kbo(:, 3,20,16) = (/ &
     &2.7595e+03_rb,2.0695e+03_rb,1.3797e+03_rb,6.8984e+02_rb,8.0107e+01_rb/)
      kbo(:, 4,20,16) = (/ &
     &3.5702e+03_rb,2.6775e+03_rb,1.7850e+03_rb,8.9253e+02_rb,1.0388e+02_rb/)
      kbo(:, 5,20,16) = (/ &
     &4.4675e+03_rb,3.3505e+03_rb,2.2337e+03_rb,1.1168e+03_rb,1.3102e+02_rb/)
      kbo(:, 1,21,16) = (/ &
     &1.7354e+03_rb,1.3015e+03_rb,8.6762e+02_rb,4.3383e+02_rb,5.1121e+01_rb/)
      kbo(:, 2,21,16) = (/ &
     &2.4333e+03_rb,1.8248e+03_rb,1.2166e+03_rb,6.0831e+02_rb,7.0122e+01_rb/)
      kbo(:, 3,21,16) = (/ &
     &3.2566e+03_rb,2.4426e+03_rb,1.6283e+03_rb,8.1420e+02_rb,9.3037e+01_rb/)
      kbo(:, 4,21,16) = (/ &
     &4.1875e+03_rb,3.1404e+03_rb,2.0937e+03_rb,1.0468e+03_rb,1.1968e+02_rb/)
      kbo(:, 5,21,16) = (/ &
     &5.2269e+03_rb,3.9201e+03_rb,2.6133e+03_rb,1.3066e+03_rb,1.4963e+02_rb/)
      kbo(:, 1,22,16) = (/ &
     &2.1136e+03_rb,1.5853e+03_rb,1.0569e+03_rb,5.2844e+02_rb,6.0581e+01_rb/)
      kbo(:, 2,22,16) = (/ &
     &2.9316e+03_rb,2.1987e+03_rb,1.4659e+03_rb,7.3292e+02_rb,8.2385e+01_rb/)
      kbo(:, 3,22,16) = (/ &
     &3.8802e+03_rb,2.9101e+03_rb,1.9401e+03_rb,9.7007e+02_rb,1.0823e+02_rb/)
      kbo(:, 4,22,16) = (/ &
     &4.9544e+03_rb,3.7156e+03_rb,2.4770e+03_rb,1.2386e+03_rb,1.3781e+02_rb/)
      kbo(:, 5,22,16) = (/ &
     &6.1512e+03_rb,4.6135e+03_rb,3.0579e+03_rb,1.5378e+03_rb,1.7127e+02_rb/)
      kbo(:, 1,23,16) = (/ &
     &2.5612e+03_rb,1.9208e+03_rb,1.2805e+03_rb,6.4031e+02_rb,7.0795e+01_rb/)
      kbo(:, 2,23,16) = (/ &
     &3.5078e+03_rb,2.6308e+03_rb,1.7538e+03_rb,8.7684e+02_rb,9.5347e+01_rb/)
      kbo(:, 3,23,16) = (/ &
     &4.5972e+03_rb,3.4475e+03_rb,2.2984e+03_rb,1.1493e+03_rb,1.2411e+02_rb/)
      kbo(:, 4,23,16) = (/ &
     &5.8246e+03_rb,4.3680e+03_rb,2.9119e+03_rb,1.4559e+03_rb,1.5734e+02_rb/)
      kbo(:, 5,23,16) = (/ &
     &7.1749e+03_rb,5.3812e+03_rb,3.5874e+03_rb,1.7937e+03_rb,1.9485e+02_rb/)
      kbo(:, 1,24,16) = (/ &
     &3.1040e+03_rb,2.3279e+03_rb,1.5520e+03_rb,7.7603e+02_rb,8.1731e+01_rb/)
      kbo(:, 2,24,16) = (/ &
     &4.2015e+03_rb,3.1511e+03_rb,2.1007e+03_rb,1.0503e+03_rb,1.0915e+02_rb/)
      kbo(:, 3,24,16) = (/ &
     &5.4526e+03_rb,4.0896e+03_rb,2.7263e+03_rb,1.3632e+03_rb,1.4142e+02_rb/)
      kbo(:, 4,24,16) = (/ &
     &6.8486e+03_rb,5.1365e+03_rb,3.4243e+03_rb,1.7120e+03_rb,1.7856e+02_rb/)
      kbo(:, 5,24,16) = (/ &
     &8.3592e+03_rb,6.2694e+03_rb,4.1793e+03_rb,2.0896e+03_rb,2.2042e+02_rb/)
      kbo(:, 1,25,16) = (/ &
     &3.7353e+03_rb,2.8011e+03_rb,1.8675e+03_rb,9.3379e+02_rb,9.3752e+01_rb/)
      kbo(:, 2,25,16) = (/ &
     &4.9915e+03_rb,3.7438e+03_rb,2.4959e+03_rb,1.2479e+03_rb,1.2447e+02_rb/)
      kbo(:, 3,25,16) = (/ &
     &6.4159e+03_rb,4.8120e+03_rb,3.2081e+03_rb,1.6040e+03_rb,1.6041e+02_rb/)
      kbo(:, 4,25,16) = (/ &
     &7.9803e+03_rb,5.9849e+03_rb,3.9899e+03_rb,1.9951e+03_rb,2.0157e+02_rb/)
      kbo(:, 5,25,16) = (/ &
     &9.6532e+03_rb,7.2408e+03_rb,4.8272e+03_rb,2.4135e+03_rb,2.4756e+02_rb/)
      kbo(:, 1,26,16) = (/ &
     &4.4736e+03_rb,3.3551e+03_rb,2.2367e+03_rb,1.1183e+03_rb,1.0731e+02_rb/)
      kbo(:, 2,26,16) = (/ &
     &5.9023e+03_rb,4.4264e+03_rb,2.9511e+03_rb,1.4756e+03_rb,1.4156e+02_rb/)
      kbo(:, 3,26,16) = (/ &
     &7.5046e+03_rb,5.6279e+03_rb,3.7521e+03_rb,1.8761e+03_rb,1.8123e+02_rb/)
      kbo(:, 4,26,16) = (/ &
     &9.2412e+03_rb,6.9304e+03_rb,4.6207e+03_rb,2.3102e+03_rb,2.2639e+02_rb/)
      kbo(:, 5,26,16) = (/ &
     &1.1070e+04_rb,8.3029e+03_rb,5.5351e+03_rb,2.7676e+03_rb,2.7645e+02_rb/)
      kbo(:, 1,27,16) = (/ &
     &5.2658e+03_rb,3.9745e+03_rb,2.6497e+03_rb,1.3248e+03_rb,1.2199e+02_rb/)
      kbo(:, 2,27,16) = (/ &
     &6.9046e+03_rb,5.1781e+03_rb,3.4305e+03_rb,1.7261e+03_rb,1.5967e+02_rb/)
      kbo(:, 3,27,16) = (/ &
     &8.6809e+03_rb,6.5105e+03_rb,4.3403e+03_rb,2.1700e+03_rb,2.0317e+02_rb/)
      kbo(:, 4,27,16) = (/ &
     &1.0514e+04_rb,7.9337e+03_rb,5.2893e+03_rb,2.6448e+03_rb,2.5202e+02_rb/)
      kbo(:, 5,27,16) = (/ &
     &1.2548e+04_rb,9.4115e+03_rb,6.2746e+03_rb,3.1373e+03_rb,3.0568e+02_rb/)
      kbo(:, 1,28,16) = (/ &
     &6.2040e+03_rb,4.6529e+03_rb,3.1020e+03_rb,1.5511e+03_rb,1.3749e+02_rb/)
      kbo(:, 2,28,16) = (/ &
     &7.9799e+03_rb,5.9854e+03_rb,3.9901e+03_rb,1.9952e+03_rb,1.7869e+02_rb/)
      kbo(:, 3,28,16) = (/ &
     &9.9157e+03_rb,7.4385e+03_rb,4.9578e+03_rb,2.4793e+03_rb,2.2573e+02_rb/)
      kbo(:, 4,28,16) = (/ &
     &1.1961e+04_rb,8.9706e+03_rb,5.9799e+03_rb,2.9900e+03_rb,2.7806e+02_rb/)
      kbo(:, 5,28,16) = (/ &
     &1.4060e+04_rb,1.0545e+04_rb,7.0300e+03_rb,3.5151e+03_rb,3.3499e+02_rb/)
      kbo(:, 1,29,16) = (/ &
     &7.1341e+03_rb,5.3506e+03_rb,3.5671e+03_rb,1.7835e+03_rb,1.5398e+02_rb/)
      kbo(:, 2,29,16) = (/ &
     &9.0577e+03_rb,6.7935e+03_rb,4.5290e+03_rb,2.2643e+03_rb,1.9863e+02_rb/)
      kbo(:, 3,29,16) = (/ &
     &1.1122e+04_rb,8.3426e+03_rb,5.5615e+03_rb,2.7806e+03_rb,2.4903e+02_rb/)
      kbo(:, 4,29,16) = (/ &
     &1.3280e+04_rb,9.9609e+03_rb,6.6400e+03_rb,3.3199e+03_rb,3.0450e+02_rb/)
      kbo(:, 5,29,16) = (/ &
     &1.5478e+04_rb,1.1609e+04_rb,7.7394e+03_rb,3.8697e+03_rb,3.6431e+02_rb/)
      kbo(:, 1,30,16) = (/ &
     &8.0566e+03_rb,6.0423e+03_rb,4.0284e+03_rb,2.0142e+03_rb,1.7115e+02_rb/)
      kbo(:, 2,30,16) = (/ &
     &1.0101e+04_rb,7.5759e+03_rb,5.0505e+03_rb,2.5252e+03_rb,2.1901e+02_rb/)
      kbo(:, 3,30,16) = (/ &
     &1.2267e+04_rb,9.1995e+03_rb,6.1334e+03_rb,3.0668e+03_rb,2.7243e+02_rb/)
      kbo(:, 4,30,16) = (/ &
     &1.4511e+04_rb,1.0883e+04_rb,7.2556e+03_rb,3.6279e+03_rb,3.3082e+02_rb/)
      kbo(:, 5,30,16) = (/ &
     &1.6772e+04_rb,1.2579e+04_rb,8.3858e+03_rb,4.1931e+03_rb,3.9317e+02_rb/)
      kbo(:, 1,31,16) = (/ &
     &8.8982e+03_rb,6.6739e+03_rb,4.4493e+03_rb,2.2245e+03_rb,1.8901e+02_rb/)
      kbo(:, 2,31,16) = (/ &
     &1.1025e+04_rb,8.2681e+03_rb,5.5122e+03_rb,2.7560e+03_rb,2.3984e+02_rb/)
      kbo(:, 3,31,16) = (/ &
     &1.3250e+04_rb,9.9387e+03_rb,6.6258e+03_rb,3.3127e+03_rb,2.9609e+02_rb/)
      kbo(:, 4,31,16) = (/ &
     &1.5535e+04_rb,1.1652e+04_rb,7.7678e+03_rb,3.8841e+03_rb,3.5699e+02_rb/)
      kbo(:, 5,31,16) = (/ &
     &1.7813e+04_rb,1.3361e+04_rb,8.9075e+03_rb,4.4535e+03_rb,4.2172e+02_rb/)
      kbo(:, 1,32,16) = (/ &
     &9.7468e+03_rb,7.3100e+03_rb,4.8729e+03_rb,2.4366e+03_rb,2.0736e+02_rb/)
      kbo(:, 2,32,16) = (/ &
     &1.1942e+04_rb,8.9562e+03_rb,5.9708e+03_rb,2.9855e+03_rb,2.6092e+02_rb/)
      kbo(:, 3,32,16) = (/ &
     &1.4217e+04_rb,1.0663e+04_rb,7.1086e+03_rb,3.5542e+03_rb,3.1978e+02_rb/)
      kbo(:, 4,32,16) = (/ &
     &1.6526e+04_rb,1.2394e+04_rb,8.2625e+03_rb,4.1313e+03_rb,3.8300e+02_rb/)
      kbo(:, 5,32,16) = (/ &
     &1.8815e+04_rb,1.4111e+04_rb,9.4077e+03_rb,4.7034e+03_rb,4.4968e+02_rb/)
      kbo(:, 1,33,16) = (/ &
     &1.0585e+04_rb,7.8868e+03_rb,5.2924e+03_rb,2.6461e+03_rb,2.2603e+02_rb/)
      kbo(:, 2,33,16) = (/ &
     &1.2835e+04_rb,9.6280e+03_rb,6.4182e+03_rb,3.2091e+03_rb,2.8215e+02_rb/)
      kbo(:, 3,33,16) = (/ &
     &1.5147e+04_rb,1.1359e+04_rb,7.5723e+03_rb,3.7863e+03_rb,3.3983e+02_rb/)
      kbo(:, 4,33,16) = (/ &
     &1.7461e+04_rb,1.3096e+04_rb,8.7313e+03_rb,4.3654e+03_rb,4.0847e+02_rb/)
      kbo(:, 5,33,16) = (/ &
     &1.9631e+04_rb,1.4815e+04_rb,9.8770e+03_rb,4.9389e+03_rb,4.7201e+02_rb/)
      kbo(:, 1,34,16) = (/ &
     &1.1432e+04_rb,8.5737e+03_rb,5.7158e+03_rb,2.8579e+03_rb,2.4293e+02_rb/)
      kbo(:, 2,34,16) = (/ &
     &1.3656e+04_rb,1.0308e+04_rb,6.8715e+03_rb,3.4360e+03_rb,3.0114e+02_rb/)
      kbo(:, 3,34,16) = (/ &
     &1.6096e+04_rb,1.2073e+04_rb,8.0473e+03_rb,4.0240e+03_rb,3.6423e+02_rb/)
      kbo(:, 4,34,16) = (/ &
     &1.8435e+04_rb,1.3744e+04_rb,9.2181e+03_rb,4.6088e+03_rb,4.3111e+02_rb/)
      kbo(:, 5,34,16) = (/ &
     &2.0743e+04_rb,1.5458e+04_rb,1.0371e+04_rb,5.1857e+03_rb,5.0086e+02_rb/)
      kbo(:, 1,35,16) = (/ &
     &1.2052e+04_rb,9.0391e+03_rb,6.0261e+03_rb,3.0130e+03_rb,2.5425e+02_rb/)
      kbo(:, 2,35,16) = (/ &
     &1.4410e+04_rb,1.0808e+04_rb,7.2046e+03_rb,3.6026e+03_rb,3.1392e+02_rb/)
      kbo(:, 3,35,16) = (/ &
     &1.6796e+04_rb,1.2598e+04_rb,8.3987e+03_rb,4.1994e+03_rb,3.7832e+02_rb/)
      kbo(:, 4,35,16) = (/ &
     &1.9160e+04_rb,1.4370e+04_rb,9.5799e+03_rb,4.7904e+03_rb,4.4634e+02_rb/)
      kbo(:, 5,35,16) = (/ &
     &2.1483e+04_rb,1.6114e+04_rb,1.0742e+04_rb,5.3711e+03_rb,5.1708e+02_rb/)
      kbo(:, 1,36,16) = (/ &
     &1.2411e+04_rb,9.3078e+03_rb,6.2050e+03_rb,3.1026e+03_rb,2.5887e+02_rb/)
      kbo(:, 2,36,16) = (/ &
     &1.4808e+04_rb,1.1106e+04_rb,7.4043e+03_rb,3.7021e+03_rb,3.1927e+02_rb/)
      kbo(:, 3,36,16) = (/ &
     &1.7228e+04_rb,1.2921e+04_rb,8.6130e+03_rb,4.3067e+03_rb,3.8442e+02_rb/)
      kbo(:, 4,36,16) = (/ &
     &1.9618e+04_rb,1.4714e+04_rb,9.8092e+03_rb,4.9047e+03_rb,4.5297e+02_rb/)
      kbo(:, 5,36,16) = (/ &
     &2.1965e+04_rb,1.6474e+04_rb,1.0982e+04_rb,5.4915e+03_rb,5.1909e+02_rb/)
      kbo(:, 1,37,16) = (/ &
     &1.2464e+04_rb,9.3487e+03_rb,6.2318e+03_rb,3.1159e+03_rb,2.5416e+02_rb/)
      kbo(:, 2,37,16) = (/ &
     &1.4910e+04_rb,1.1181e+04_rb,7.4551e+03_rb,3.7272e+03_rb,3.1436e+02_rb/)
      kbo(:, 3,37,16) = (/ &
     &1.7377e+04_rb,1.3033e+04_rb,8.6893e+03_rb,4.3442e+03_rb,3.7942e+02_rb/)
      kbo(:, 4,37,16) = (/ &
     &1.9817e+04_rb,1.4862e+04_rb,9.9088e+03_rb,4.9531e+03_rb,4.4815e+02_rb/)
      kbo(:, 5,37,16) = (/ &
     &2.2212e+04_rb,1.6660e+04_rb,1.1107e+04_rb,5.5531e+03_rb,5.1970e+02_rb/)
      kbo(:, 1,38,16) = (/ &
     &1.2455e+04_rb,9.3421e+03_rb,6.2277e+03_rb,3.1139e+03_rb,2.4843e+02_rb/)
      kbo(:, 2,38,16) = (/ &
     &1.4944e+04_rb,1.1208e+04_rb,7.4715e+03_rb,3.7356e+03_rb,3.0837e+02_rb/)
      kbo(:, 3,38,16) = (/ &
     &1.7458e+04_rb,1.3093e+04_rb,8.7289e+03_rb,4.3646e+03_rb,3.7326e+02_rb/)
      kbo(:, 4,38,16) = (/ &
     &1.9945e+04_rb,1.4959e+04_rb,9.9720e+03_rb,4.9866e+03_rb,4.4198e+02_rb/)
      kbo(:, 5,38,16) = (/ &
     &2.2391e+04_rb,1.6792e+04_rb,1.1195e+04_rb,5.5974e+03_rb,5.1349e+02_rb/)
      kbo(:, 1,39,16) = (/ &
     &1.2406e+04_rb,9.3076e+03_rb,6.2035e+03_rb,3.1024e+03_rb,2.4239e+02_rb/)
      kbo(:, 2,39,16) = (/ &
     &1.4934e+04_rb,1.1200e+04_rb,7.4672e+03_rb,3.7337e+03_rb,3.0193e+02_rb/)
      kbo(:, 3,39,16) = (/ &
     &1.7492e+04_rb,1.3118e+04_rb,8.7456e+03_rb,4.3725e+03_rb,3.6650e+02_rb/)
      kbo(:, 4,39,16) = (/ &
     &2.0028e+04_rb,1.5022e+04_rb,1.0013e+04_rb,5.0072e+03_rb,4.3512e+02_rb/)
      kbo(:, 5,39,16) = (/ &
     &2.2512e+04_rb,1.6885e+04_rb,1.1257e+04_rb,5.6280e+03_rb,5.0671e+02_rb/)
      kbo(:, 1,40,16) = (/ &
     &1.2191e+04_rb,9.1428e+03_rb,6.0955e+03_rb,3.0476e+03_rb,2.3031e+02_rb/)
      kbo(:, 2,40,16) = (/ &
     &1.4768e+04_rb,1.1076e+04_rb,7.3840e+03_rb,3.6920e+03_rb,2.8879e+02_rb/)
      kbo(:, 3,40,16) = (/ &
     &1.7391e+04_rb,1.3044e+04_rb,8.6959e+03_rb,4.3483e+03_rb,3.5253e+02_rb/)
      kbo(:, 4,40,16) = (/ &
     &2.0001e+04_rb,1.5001e+04_rb,1.0001e+04_rb,5.0007e+03_rb,4.2055e+02_rb/)
      kbo(:, 5,40,16) = (/ &
     &2.2559e+04_rb,1.6919e+04_rb,1.1281e+04_rb,5.6404e+03_rb,4.9160e+02_rb/)
      kbo(:, 1,41,16) = (/ &
     &1.1918e+04_rb,8.9381e+03_rb,5.9590e+03_rb,2.9794e+03_rb,2.1766e+02_rb/)
      kbo(:, 2,41,16) = (/ &
     &1.4546e+04_rb,1.0910e+04_rb,7.2239e+03_rb,3.6364e+03_rb,2.7473e+02_rb/)
      kbo(:, 3,41,16) = (/ &
     &1.7237e+04_rb,1.2927e+04_rb,8.6187e+03_rb,4.3092e+03_rb,3.3421e+02_rb/)
      kbo(:, 4,41,16) = (/ &
     &1.9925e+04_rb,1.4942e+04_rb,9.9613e+03_rb,4.9808e+03_rb,4.0468e+02_rb/)
      kbo(:, 5,41,16) = (/ &
     &2.2566e+04_rb,1.6925e+04_rb,1.1282e+04_rb,5.6408e+03_rb,4.7546e+02_rb/)
      kbo(:, 1,42,16) = (/ &
     &1.1611e+04_rb,8.7088e+03_rb,5.8053e+03_rb,2.9027e+03_rb,2.0505e+02_rb/)
      kbo(:, 2,42,16) = (/ &
     &1.4284e+04_rb,1.0639e+04_rb,7.1417e+03_rb,3.5709e+03_rb,2.6066e+02_rb/)
      kbo(:, 3,42,16) = (/ &
     &1.7038e+04_rb,1.2778e+04_rb,8.5196e+03_rb,4.2591e+03_rb,3.2228e+02_rb/)
      kbo(:, 4,42,16) = (/ &
     &1.9799e+04_rb,1.4850e+04_rb,9.8997e+03_rb,4.9502e+03_rb,3.8865e+02_rb/)
      kbo(:, 5,42,16) = (/ &
     &2.2525e+04_rb,1.6894e+04_rb,1.1262e+04_rb,5.6311e+03_rb,4.5874e+02_rb/)
      kbo(:, 1,43,16) = (/ &
     &1.1239e+04_rb,8.4296e+03_rb,5.6199e+03_rb,2.8098e+03_rb,1.8981e+02_rb/)
      kbo(:, 2,43,16) = (/ &
     &1.3966e+04_rb,1.0473e+04_rb,6.9839e+03_rb,3.4914e+03_rb,2.4348e+02_rb/)
      kbo(:, 3,43,16) = (/ &
     &1.6803e+04_rb,1.2602e+04_rb,8.4011e+03_rb,4.2005e+03_rb,3.0350e+02_rb/)
      kbo(:, 4,43,16) = (/ &
     &1.9668e+04_rb,1.4751e+04_rb,9.8339e+03_rb,4.9166e+03_rb,3.6865e+02_rb/)
      kbo(:, 5,43,16) = (/ &
     &2.2506e+04_rb,1.6879e+04_rb,1.1254e+04_rb,5.6261e+03_rb,4.3789e+02_rb/)
      kbo(:, 1,44,16) = (/ &
     &1.0825e+04_rb,8.1186e+03_rb,5.4125e+03_rb,2.7064e+03_rb,1.7391e+02_rb/)
      kbo(:, 2,44,16) = (/ &
     &1.3603e+04_rb,1.0202e+04_rb,6.8014e+03_rb,3.4006e+03_rb,2.2526e+02_rb/)
      kbo(:, 3,44,16) = (/ &
     &1.6530e+04_rb,1.2398e+04_rb,8.2649e+03_rb,4.1327e+03_rb,2.8337e+02_rb/)
      kbo(:, 4,44,16) = (/ &
     &1.9511e+04_rb,1.4634e+04_rb,9.7557e+03_rb,4.8778e+03_rb,3.4703e+02_rb/)
      kbo(:, 5,44,16) = (/ &
     &2.2482e+04_rb,1.6861e+04_rb,1.1241e+04_rb,5.6203e+03_rb,4.1512e+02_rb/)
      kbo(:, 1,45,16) = (/ &
     &1.0398e+04_rb,7.7989e+03_rb,5.1994e+03_rb,2.5997e+03_rb,1.5876e+02_rb/)
      kbo(:, 2,45,16) = (/ &
     &1.3207e+04_rb,9.9052e+03_rb,6.6032e+03_rb,3.3018e+03_rb,2.0761e+02_rb/)
      kbo(:, 3,45,16) = (/ &
     &1.6220e+04_rb,1.2165e+04_rb,8.1095e+03_rb,4.0543e+03_rb,2.6368e+02_rb/)
      kbo(:, 4,45,16) = (/ &
     &1.9318e+04_rb,1.4390e+04_rb,9.6588e+03_rb,4.8297e+03_rb,3.2574e+02_rb/)
      kbo(:, 5,45,16) = (/ &
     &2.2424e+04_rb,1.6818e+04_rb,1.1212e+04_rb,5.6061e+03_rb,3.9259e+02_rb/)
      kbo(:, 1,46,16) = (/ &
     &9.9189e+03_rb,7.4400e+03_rb,4.9600e+03_rb,2.4797e+03_rb,1.4342e+02_rb/)
      kbo(:, 2,46,16) = (/ &
     &1.2731e+04_rb,9.5482e+03_rb,6.3653e+03_rb,3.1826e+03_rb,1.8954e+02_rb/)
      kbo(:, 3,46,16) = (/ &
     &1.5820e+04_rb,1.1865e+04_rb,7.9099e+03_rb,3.9548e+03_rb,2.4324e+02_rb/)
      kbo(:, 4,46,16) = (/ &
     &1.9036e+04_rb,1.4277e+04_rb,9.5173e+03_rb,4.7590e+03_rb,3.0340e+02_rb/)
      kbo(:, 5,46,16) = (/ &
     &2.2283e+04_rb,1.6712e+04_rb,1.1068e+04_rb,5.5708e+03_rb,3.6873e+02_rb/)
      kbo(:, 1,47,16) = (/ &
     &9.3465e+03_rb,7.0098e+03_rb,4.6733e+03_rb,2.3366e+03_rb,1.2741e+02_rb/)
      kbo(:, 2,47,16) = (/ &
     &1.2131e+04_rb,9.0979e+03_rb,6.0653e+03_rb,3.0325e+03_rb,1.7033e+02_rb/)
      kbo(:, 3,47,16) = (/ &
     &1.5279e+04_rb,1.1459e+04_rb,7.6395e+03_rb,3.8199e+03_rb,2.2121e+02_rb/)
      kbo(:, 4,47,16) = (/ &
     &1.8609e+04_rb,1.3956e+04_rb,9.3042e+03_rb,4.6522e+03_rb,2.7888e+02_rb/)
      kbo(:, 5,47,16) = (/ &
     &2.2011e+04_rb,1.6508e+04_rb,1.1006e+04_rb,5.5025e+03_rb,3.4244e+02_rb/)
      kbo(:, 1,48,16) = (/ &
     &8.7925e+03_rb,6.5945e+03_rb,4.3966e+03_rb,2.1981e+03_rb,1.1262e+02_rb/)
      kbo(:, 2,48,16) = (/ &
     &1.1529e+04_rb,8.6468e+03_rb,5.7644e+03_rb,2.8824e+03_rb,1.5232e+02_rb/)
      kbo(:, 3,48,16) = (/ &
     &1.4702e+04_rb,1.1027e+04_rb,7.3513e+03_rb,3.6755e+03_rb,2.0009e+02_rb/)
      kbo(:, 4,48,16) = (/ &
     &1.8139e+04_rb,1.3604e+04_rb,9.0700e+03_rb,4.5350e+03_rb,2.5528e+02_rb/)
      kbo(:, 5,48,16) = (/ &
     &2.1696e+04_rb,1.6272e+04_rb,1.0846e+04_rb,5.4236e+03_rb,3.1666e+02_rb/)
      kbo(:, 1,49,16) = (/ &
     &8.2532e+03_rb,6.1899e+03_rb,4.1262e+03_rb,2.0633e+03_rb,9.9158e+01_rb/)
      kbo(:, 2,49,16) = (/ &
     &1.0925e+04_rb,8.1939e+03_rb,5.4625e+03_rb,2.7312e+03_rb,1.3556e+02_rb/)
      kbo(:, 3,49,16) = (/ &
     &1.4098e+04_rb,1.0574e+04_rb,7.0499e+03_rb,3.5250e+03_rb,1.8015e+02_rb/)
      kbo(:, 4,49,16) = (/ &
     &1.7628e+04_rb,1.3221e+04_rb,8.8139e+03_rb,4.4071e+03_rb,2.3258e+02_rb/)
      kbo(:, 5,49,16) = (/ &
     &2.1334e+04_rb,1.6000e+04_rb,1.0668e+04_rb,5.3338e+03_rb,2.8877e+02_rb/)
      kbo(:, 1,50,16) = (/ &
     &7.7886e+03_rb,5.8414e+03_rb,3.8943e+03_rb,1.9471e+03_rb,8.7562e+01_rb/)
      kbo(:, 2,50,16) = (/ &
     &1.0399e+04_rb,7.7996e+03_rb,5.1995e+03_rb,2.6000e+03_rb,1.2099e+02_rb/)
      kbo(:, 3,50,16) = (/ &
     &1.3554e+04_rb,1.0165e+04_rb,6.7769e+03_rb,3.3882e+03_rb,1.6257e+02_rb/)
      kbo(:, 4,50,16) = (/ &
     &1.7158e+04_rb,1.2869e+04_rb,8.5791e+03_rb,4.2889e+03_rb,2.1215e+02_rb/)
      kbo(:, 5,50,16) = (/ &
     &2.1011e+04_rb,1.5758e+04_rb,1.0505e+04_rb,5.2531e+03_rb,2.6885e+02_rb/)
      kbo(:, 1,51,16) = (/ &
     &7.3653e+03_rb,5.5240e+03_rb,3.6827e+03_rb,1.8412e+03_rb,7.7231e+01_rb/)
      kbo(:, 2,51,16) = (/ &
     &9.9189e+03_rb,7.4392e+03_rb,4.9594e+03_rb,2.4660e+03_rb,1.0798e+02_rb/)
      kbo(:, 3,51,16) = (/ &
     &1.3047e+04_rb,9.7852e+03_rb,6.5235e+03_rb,3.2615e+03_rb,1.4656e+02_rb/)
      kbo(:, 4,51,16) = (/ &
     &1.6702e+04_rb,1.2527e+04_rb,8.3510e+03_rb,4.1752e+03_rb,1.9328e+02_rb/)
      kbo(:, 5,51,16) = (/ &
     &2.0693e+04_rb,1.5520e+04_rb,1.0347e+04_rb,5.1734e+03_rb,2.4760e+02_rb/)
      kbo(:, 1,52,16) = (/ &
     &6.9562e+03_rb,5.2172e+03_rb,3.4778e+03_rb,1.7389e+03_rb,6.7762e+01_rb/)
      kbo(:, 2,52,16) = (/ &
     &9.4563e+03_rb,7.0922e+03_rb,4.7284e+03_rb,2.3642e+03_rb,9.6034e+01_rb/)
      kbo(:, 3,52,16) = (/ &
     &1.2544e+04_rb,9.4081e+03_rb,6.2721e+03_rb,3.1358e+03_rb,1.3163e+02_rb/)
      kbo(:, 4,52,16) = (/ &
     &1.6228e+04_rb,1.2171e+04_rb,8.1139e+03_rb,4.0563e+03_rb,1.7544e+02_rb/)
      kbo(:, 5,52,16) = (/ &
     &2.0214e+04_rb,1.5266e+04_rb,1.0177e+04_rb,5.0897e+03_rb,2.2714e+02_rb/)
      kbo(:, 1,53,16) = (/ &
     &6.5707e+03_rb,4.8896e+03_rb,3.2854e+03_rb,1.6428e+03_rb,5.9086e+01_rb/)
      kbo(:, 2,53,16) = (/ &
     &9.0178e+03_rb,6.7633e+03_rb,4.5088e+03_rb,2.2544e+03_rb,8.4973e+01_rb/)
      kbo(:, 3,53,16) = (/ &
     &1.2063e+04_rb,9.0476e+03_rb,6.0317e+03_rb,3.0159e+03_rb,1.1770e+02_rb/)
      kbo(:, 4,53,16) = (/ &
     &1.5755e+04_rb,1.1816e+04_rb,7.8772e+03_rb,3.9386e+03_rb,1.5858e+02_rb/)
      kbo(:, 5,53,16) = (/ &
     &1.9999e+04_rb,1.4999e+04_rb,9.9995e+03_rb,5.0000e+03_rb,2.0751e+02_rb/)
      kbo(:, 1,54,16) = (/ &
     &6.2721e+03_rb,4.7041e+03_rb,3.1362e+03_rb,1.5681e+03_rb,5.1875e+01_rb/)
      kbo(:, 2,54,16) = (/ &
     &8.6741e+03_rb,6.5056e+03_rb,4.3370e+03_rb,2.1682e+03_rb,7.5628e+01_rb/)
      kbo(:, 3,54,16) = (/ &
     &1.1700e+04_rb,8.7748e+03_rb,5.8499e+03_rb,2.9249e+03_rb,1.0596e+02_rb/)
      kbo(:, 4,54,16) = (/ &
     &1.5404e+04_rb,1.1553e+04_rb,7.7020e+03_rb,3.8510e+03_rb,1.4404e+02_rb/)
      kbo(:, 5,54,16) = (/ &
     &1.9752e+04_rb,1.4814e+04_rb,9.8758e+03_rb,4.9379e+03_rb,1.9030e+02_rb/)
      kbo(:, 1,55,16) = (/ &
     &6.0231e+03_rb,4.5173e+03_rb,3.0116e+03_rb,1.5058e+03_rb,4.5583e+01_rb/)
      kbo(:, 2,55,16) = (/ &
     &8.3823e+03_rb,6.2868e+03_rb,4.1912e+03_rb,2.0956e+03_rb,6.7362e+01_rb/)
      kbo(:, 3,55,16) = (/ &
     &1.1398e+04_rb,8.5488e+03_rb,5.6992e+03_rb,2.8496e+03_rb,9.5542e+01_rb/)
      kbo(:, 4,55,16) = (/ &
     &1.5124e+04_rb,1.1343e+04_rb,7.5617e+03_rb,3.7808e+03_rb,1.3099e+02_rb/)
      kbo(:, 5,55,16) = (/ &
     &1.9568e+04_rb,1.4676e+04_rb,9.7841e+03_rb,4.8920e+03_rb,1.7464e+02_rb/)
      kbo(:, 1,56,16) = (/ &
     &5.7989e+03_rb,4.3492e+03_rb,2.8995e+03_rb,1.4497e+03_rb,3.9828e+01_rb/)
      kbo(:, 2,56,16) = (/ &
     &8.1232e+03_rb,6.0924e+03_rb,4.0616e+03_rb,2.0308e+03_rb,5.9729e+01_rb/)
      kbo(:, 3,56,16) = (/ &
     &1.1139e+04_rb,8.3546e+03_rb,5.5697e+03_rb,2.7849e+03_rb,8.5801e+01_rb/)
      kbo(:, 4,56,16) = (/ &
     &1.4890e+04_rb,1.1167e+04_rb,7.4448e+03_rb,3.7224e+03_rb,1.1874e+02_rb/)
      kbo(:, 5,56,16) = (/ &
     &1.9429e+04_rb,1.4572e+04_rb,9.7148e+03_rb,4.8574e+03_rb,1.5988e+02_rb/)
      kbo(:, 1,57,16) = (/ &
     &5.6021e+03_rb,4.2016e+03_rb,2.8010e+03_rb,1.4005e+03_rb,3.4593e+01_rb/)
      kbo(:, 2,57,16) = (/ &
     &7.9032e+03_rb,5.9274e+03_rb,3.9516e+03_rb,1.9758e+03_rb,5.2766e+01_rb/)
      kbo(:, 3,57,16) = (/ &
     &1.0919e+04_rb,8.1889e+03_rb,5.4593e+03_rb,2.7295e+03_rb,7.6784e+01_rb/)
      kbo(:, 4,57,16) = (/ &
     &1.4709e+04_rb,1.1031e+04_rb,7.3542e+03_rb,3.6771e+03_rb,1.0741e+02_rb/)
      kbo(:, 5,57,16) = (/ &
     &1.9344e+04_rb,1.4508e+04_rb,9.6719e+03_rb,4.8359e+03_rb,1.4582e+02_rb/)
      kbo(:, 1,58,16) = (/ &
     &2.6461e+03_rb,2.2784e+03_rb,1.7829e+03_rb,1.0638e+03_rb,3.0084e+01_rb/)
      kbo(:, 2,58,16) = (/ &
     &3.7547e+03_rb,3.2329e+03_rb,2.5298e+03_rb,1.5309e+03_rb,4.6706e+01_rb/)
      kbo(:, 3,58,16) = (/ &
     &5.2182e+03_rb,4.4930e+03_rb,3.5159e+03_rb,2.1276e+03_rb,6.8855e+01_rb/)
      kbo(:, 4,58,16) = (/ &
     &7.0852e+03_rb,6.1006e+03_rb,4.7738e+03_rb,2.8889e+03_rb,9.7411e+01_rb/)
      kbo(:, 5,58,16) = (/ &
     &9.3867e+03_rb,8.0822e+03_rb,6.3244e+03_rb,3.8273e+03_rb,1.3335e+02_rb/)
      kbo(:, 1,59,16) = (/ &
     &2.6519e+03_rb,2.2918e+03_rb,1.8023e+03_rb,1.0985e+03_rb,2.8399e+01_rb/)
      kbo(:, 2,59,16) = (/ &
     &3.7709e+03_rb,3.2589e+03_rb,2.5629e+03_rb,1.5620e+03_rb,4.4429e+01_rb/)
      kbo(:, 3,59,16) = (/ &
     &5.2534e+03_rb,4.5401e+03_rb,3.5704e+03_rb,2.1761e+03_rb,6.5844e+01_rb/)
      kbo(:, 4,59,16) = (/ &
     &7.1555e+03_rb,6.1839e+03_rb,4.8632e+03_rb,2.9640e+03_rb,9.3608e+01_rb/)
      kbo(:, 5,59,16) = (/ &
     &9.5069e+03_rb,8.2160e+03_rb,6.4613e+03_rb,3.9381e+03_rb,1.2854e+02_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The third index 
!     runs over the g-channel (1 to 16).

      kao_mn2o( 1, :, 1) = (/ &
     & 1.28178e-05_rb, 1.55472e-05_rb, 1.88578e-05_rb, 2.28735e-05_rb, 2.77442e-05_rb, &
     & 3.36520e-05_rb, 4.08179e-05_rb, 4.95098e-05_rb, 6.00525e-05_rb, 7.28401e-05_rb, &
     & 8.83508e-05_rb, 1.07164e-04_rb, 1.29984e-04_rb, 1.57663e-04_rb, 1.91236e-04_rb, &
     & 2.31958e-04_rb, 2.81352e-04_rb, 3.41263e-04_rb, 4.13932e-04_rb/)
      kao_mn2o( 2, :, 1) = (/ &
     & 1.00725e-01_rb, 1.04470e-01_rb, 1.08355e-01_rb, 1.12384e-01_rb, 1.16563e-01_rb, &
     & 1.20898e-01_rb, 1.25394e-01_rb, 1.30057e-01_rb, 1.34893e-01_rb, 1.39909e-01_rb, &
     & 1.45112e-01_rb, 1.50508e-01_rb, 1.56104e-01_rb, 1.61909e-01_rb, 1.67930e-01_rb, &
     & 1.74175e-01_rb, 1.80652e-01_rb, 1.87369e-01_rb, 1.94337e-01_rb/)
      kao_mn2o( 3, :, 1) = (/ &
     & 1.94143e-01_rb, 1.97380e-01_rb, 2.00670e-01_rb, 2.04016e-01_rb, 2.07417e-01_rb, &
     & 2.10875e-01_rb, 2.14390e-01_rb, 2.17964e-01_rb, 2.21598e-01_rb, 2.25292e-01_rb, &
     & 2.29048e-01_rb, 2.32867e-01_rb, 2.36749e-01_rb, 2.40696e-01_rb, 2.44708e-01_rb, &
     & 2.48788e-01_rb, 2.52936e-01_rb, 2.57152e-01_rb, 2.61439e-01_rb/)
      kao_mn2o( 4, :, 1) = (/ &
     & 2.98127e-01_rb, 3.00016e-01_rb, 3.01916e-01_rb, 3.03829e-01_rb, 3.05754e-01_rb, &
     & 3.07691e-01_rb, 3.09640e-01_rb, 3.11601e-01_rb, 3.13575e-01_rb, 3.15562e-01_rb, &
     & 3.17561e-01_rb, 3.19572e-01_rb, 3.21597e-01_rb, 3.23634e-01_rb, 3.25684e-01_rb, &
     & 3.27748e-01_rb, 3.29824e-01_rb, 3.31913e-01_rb, 3.34016e-01_rb/)
      kao_mn2o( 5, :, 1) = (/ &
     & 4.45029e-01_rb, 4.45243e-01_rb, 4.45458e-01_rb, 4.45673e-01_rb, 4.45889e-01_rb, &
     & 4.46104e-01_rb, 4.46319e-01_rb, 4.46535e-01_rb, 4.46750e-01_rb, 4.46966e-01_rb, &
     & 4.47182e-01_rb, 4.47398e-01_rb, 4.47614e-01_rb, 4.47830e-01_rb, 4.48046e-01_rb, &
     & 4.48262e-01_rb, 4.48479e-01_rb, 4.48695e-01_rb, 4.48912e-01_rb/)
      kao_mn2o( 6, :, 1) = (/ &
     & 7.15677e-01_rb, 7.14564e-01_rb, 7.13452e-01_rb, 7.12342e-01_rb, 7.11234e-01_rb, &
     & 7.10127e-01_rb, 7.09022e-01_rb, 7.07919e-01_rb, 7.06818e-01_rb, 7.05718e-01_rb, &
     & 7.04620e-01_rb, 7.03524e-01_rb, 7.02429e-01_rb, 7.01336e-01_rb, 7.00245e-01_rb, &
     & 6.99156e-01_rb, 6.98068e-01_rb, 6.96982e-01_rb, 6.95898e-01_rb/)
      kao_mn2o( 7, :, 1) = (/ &
     & 9.89605e-01_rb, 9.85128e-01_rb, 9.80671e-01_rb, 9.76234e-01_rb, 9.71817e-01_rb, &
     & 9.67421e-01_rb, 9.63044e-01_rb, 9.58687e-01_rb, 9.54350e-01_rb, 9.50032e-01_rb, &
     & 9.45734e-01_rb, 9.41455e-01_rb, 9.37196e-01_rb, 9.32956e-01_rb, 9.28735e-01_rb, &
     & 9.24533e-01_rb, 9.20350e-01_rb, 9.16187e-01_rb, 9.12042e-01_rb/)
      kao_mn2o( 8, :, 1) = (/ &
     & 1.12229e+00_rb, 1.11502e+00_rb, 1.10779e+00_rb, 1.10061e+00_rb, 1.09348e+00_rb, &
     & 1.08639e+00_rb, 1.07935e+00_rb, 1.07235e+00_rb, 1.06540e+00_rb, 1.05850e+00_rb, &
     & 1.05164e+00_rb, 1.04482e+00_rb, 1.03805e+00_rb, 1.03132e+00_rb, 1.02464e+00_rb, &
     & 1.01799e+00_rb, 1.01140e+00_rb, 1.00484e+00_rb, 9.98328e-01_rb/)
      kao_mn2o( 9, :, 1) = (/ &
     & 7.20959e-01_rb, 7.22839e-01_rb, 7.24723e-01_rb, 7.26612e-01_rb, 7.28506e-01_rb, &
     & 7.30405e-01_rb, 7.32309e-01_rb, 7.34218e-01_rb, 7.36132e-01_rb, 7.38051e-01_rb, &
     & 7.39975e-01_rb, 7.41904e-01_rb, 7.43838e-01_rb, 7.45777e-01_rb, 7.47721e-01_rb, &
     & 7.49670e-01_rb, 7.51624e-01_rb, 7.53584e-01_rb, 7.55548e-01_rb/)
      kao_mn2o( 1, :, 2) = (/ &
     & 1.62152e-03_rb, 1.81627e-03_rb, 2.03443e-03_rb, 2.27878e-03_rb, 2.55248e-03_rb, &
     & 2.85906e-03_rb, 3.20245e-03_rb, 3.58710e-03_rb, 4.01794e-03_rb, 4.50053e-03_rb, &
     & 5.04109e-03_rb, 5.64657e-03_rb, 6.32477e-03_rb, 7.08444e-03_rb, 7.93534e-03_rb, &
     & 8.88845e-03_rb, 9.95603e-03_rb, 1.11518e-02_rb, 1.24913e-02_rb/)
      kao_mn2o( 2, :, 2) = (/ &
     & 3.73716e-01_rb, 3.72491e-01_rb, 3.71271e-01_rb, 3.70054e-01_rb, 3.68841e-01_rb, &
     & 3.67633e-01_rb, 3.66428e-01_rb, 3.65227e-01_rb, 3.64031e-01_rb, 3.62838e-01_rb, &
     & 3.61649e-01_rb, 3.60464e-01_rb, 3.59283e-01_rb, 3.58106e-01_rb, 3.56932e-01_rb, &
     & 3.55763e-01_rb, 3.54597e-01_rb, 3.53435e-01_rb, 3.52277e-01_rb/)
      kao_mn2o( 3, :, 2) = (/ &
     & 5.46240e-01_rb, 5.42972e-01_rb, 5.39724e-01_rb, 5.36495e-01_rb, 5.33285e-01_rb, &
     & 5.30095e-01_rb, 5.26923e-01_rb, 5.23771e-01_rb, 5.20637e-01_rb, 5.17523e-01_rb, &
     & 5.14426e-01_rb, 5.11349e-01_rb, 5.08290e-01_rb, 5.05249e-01_rb, 5.02226e-01_rb, &
     & 4.99221e-01_rb, 4.96235e-01_rb, 4.93266e-01_rb, 4.90315e-01_rb/)
      kao_mn2o( 4, :, 2) = (/ &
     & 8.35399e-01_rb, 8.36766e-01_rb, 8.38135e-01_rb, 8.39507e-01_rb, 8.40880e-01_rb, &
     & 8.42256e-01_rb, 8.43635e-01_rb, 8.45015e-01_rb, 8.46398e-01_rb, 8.47783e-01_rb, &
     & 8.49170e-01_rb, 8.50559e-01_rb, 8.51951e-01_rb, 8.53345e-01_rb, 8.54742e-01_rb, &
     & 8.56140e-01_rb, 8.57541e-01_rb, 8.58944e-01_rb, 8.60350e-01_rb/)
      kao_mn2o( 5, :, 2) = (/ &
     & 1.04433e+00_rb, 1.04864e+00_rb, 1.05297e+00_rb, 1.05731e+00_rb, 1.06168e+00_rb, &
     & 1.06606e+00_rb, 1.07046e+00_rb, 1.07488e+00_rb, 1.07932e+00_rb, 1.08377e+00_rb, &
     & 1.08824e+00_rb, 1.09274e+00_rb, 1.09725e+00_rb, 1.10178e+00_rb, 1.10632e+00_rb, &
     & 1.11089e+00_rb, 1.11547e+00_rb, 1.12008e+00_rb, 1.12470e+00_rb/)
      kao_mn2o( 6, :, 2) = (/ &
     & 1.22341e+00_rb, 1.22885e+00_rb, 1.23431e+00_rb, 1.23980e+00_rb, 1.24531e+00_rb, &
     & 1.25084e+00_rb, 1.25640e+00_rb, 1.26199e+00_rb, 1.26760e+00_rb, 1.27323e+00_rb, &
     & 1.27889e+00_rb, 1.28458e+00_rb, 1.29029e+00_rb, 1.29602e+00_rb, 1.30178e+00_rb, &
     & 1.30757e+00_rb, 1.31338e+00_rb, 1.31922e+00_rb, 1.32508e+00_rb/)
      kao_mn2o( 7, :, 2) = (/ &
     & 1.67595e+00_rb, 1.68296e+00_rb, 1.69000e+00_rb, 1.69707e+00_rb, 1.70417e+00_rb, &
     & 1.71130e+00_rb, 1.71846e+00_rb, 1.72565e+00_rb, 1.73287e+00_rb, 1.74012e+00_rb, &
     & 1.74740e+00_rb, 1.75471e+00_rb, 1.76206e+00_rb, 1.76943e+00_rb, 1.77683e+00_rb, &
     & 1.78426e+00_rb, 1.79173e+00_rb, 1.79922e+00_rb, 1.80675e+00_rb/)
      kao_mn2o( 8, :, 2) = (/ &
     & 2.76890e+00_rb, 2.76981e+00_rb, 2.77072e+00_rb, 2.77163e+00_rb, 2.77254e+00_rb, &
     & 2.77345e+00_rb, 2.77436e+00_rb, 2.77527e+00_rb, 2.77618e+00_rb, 2.77709e+00_rb, &
     & 2.77800e+00_rb, 2.77891e+00_rb, 2.77982e+00_rb, 2.78074e+00_rb, 2.78165e+00_rb, &
     & 2.78256e+00_rb, 2.78348e+00_rb, 2.78439e+00_rb, 2.78530e+00_rb/)
      kao_mn2o( 9, :, 2) = (/ &
     & 8.00944e-01_rb, 7.95531e-01_rb, 7.90155e-01_rb, 7.84815e-01_rb, 7.79511e-01_rb, &
     & 7.74243e-01_rb, 7.69011e-01_rb, 7.63813e-01_rb, 7.58652e-01_rb, 7.53525e-01_rb, &
     & 7.48432e-01_rb, 7.43374e-01_rb, 7.38350e-01_rb, 7.33360e-01_rb, 7.28404e-01_rb, &
     & 7.23482e-01_rb, 7.18592e-01_rb, 7.13736e-01_rb, 7.08912e-01_rb/)
      kao_mn2o( 1, :, 3) = (/ &
     & 5.26578e-02_rb, 5.59000e-02_rb, 5.93419e-02_rb, 6.29957e-02_rb, 6.68744e-02_rb, &
     & 7.09920e-02_rb, 7.53631e-02_rb, 8.00034e-02_rb, 8.49294e-02_rb, 9.01586e-02_rb, &
     & 9.57099e-02_rb, 1.01603e-01_rb, 1.07859e-01_rb, 1.14500e-01_rb, 1.21550e-01_rb, &
     & 1.29034e-01_rb, 1.36979e-01_rb, 1.45413e-01_rb, 1.54366e-01_rb/)
      kao_mn2o( 2, :, 3) = (/ &
     & 8.18393e-01_rb, 8.20623e-01_rb, 8.22860e-01_rb, 8.25103e-01_rb, 8.27352e-01_rb, &
     & 8.29608e-01_rb, 8.31869e-01_rb, 8.34137e-01_rb, 8.36410e-01_rb, 8.38690e-01_rb, &
     & 8.40976e-01_rb, 8.43269e-01_rb, 8.45567e-01_rb, 8.47872e-01_rb, 8.50183e-01_rb, &
     & 8.52501e-01_rb, 8.54825e-01_rb, 8.57155e-01_rb, 8.59491e-01_rb/)
      kao_mn2o( 3, :, 3) = (/ &
     & 1.02454e+00_rb, 1.03210e+00_rb, 1.03972e+00_rb, 1.04740e+00_rb, 1.05514e+00_rb, &
     & 1.06293e+00_rb, 1.07077e+00_rb, 1.07868e+00_rb, 1.08665e+00_rb, 1.09467e+00_rb, &
     & 1.10275e+00_rb, 1.11089e+00_rb, 1.11910e+00_rb, 1.12736e+00_rb, 1.13568e+00_rb, &
     & 1.14407e+00_rb, 1.15252e+00_rb, 1.16103e+00_rb, 1.16960e+00_rb/)
      kao_mn2o( 4, :, 3) = (/ &
     & 1.11755e+00_rb, 1.12348e+00_rb, 1.12944e+00_rb, 1.13543e+00_rb, 1.14146e+00_rb, &
     & 1.14752e+00_rb, 1.15360e+00_rb, 1.15972e+00_rb, 1.16588e+00_rb, 1.17206e+00_rb, &
     & 1.17828e+00_rb, 1.18453e+00_rb, 1.19082e+00_rb, 1.19714e+00_rb, 1.20349e+00_rb, &
     & 1.20988e+00_rb, 1.21630e+00_rb, 1.22275e+00_rb, 1.22924e+00_rb/)
      kao_mn2o( 5, :, 3) = (/ &
     & 1.41993e+00_rb, 1.42353e+00_rb, 1.42713e+00_rb, 1.43074e+00_rb, 1.43436e+00_rb, &
     & 1.43799e+00_rb, 1.44163e+00_rb, 1.44528e+00_rb, 1.44894e+00_rb, 1.45261e+00_rb, &
     & 1.45628e+00_rb, 1.45997e+00_rb, 1.46367e+00_rb, 1.46737e+00_rb, 1.47108e+00_rb, &
     & 1.47481e+00_rb, 1.47854e+00_rb, 1.48228e+00_rb, 1.48603e+00_rb/)
      kao_mn2o( 6, :, 3) = (/ &
     & 1.37725e+00_rb, 1.38029e+00_rb, 1.38334e+00_rb, 1.38640e+00_rb, 1.38947e+00_rb, &
     & 1.39254e+00_rb, 1.39562e+00_rb, 1.39870e+00_rb, 1.40179e+00_rb, 1.40489e+00_rb, &
     & 1.40800e+00_rb, 1.41111e+00_rb, 1.41423e+00_rb, 1.41736e+00_rb, 1.42049e+00_rb, &
     & 1.42363e+00_rb, 1.42678e+00_rb, 1.42993e+00_rb, 1.43309e+00_rb/)
      kao_mn2o( 7, :, 3) = (/ &
     & 1.34579e+00_rb, 1.34584e+00_rb, 1.34589e+00_rb, 1.34594e+00_rb, 1.34599e+00_rb, &
     & 1.34603e+00_rb, 1.34608e+00_rb, 1.34613e+00_rb, 1.34618e+00_rb, 1.34623e+00_rb, &
     & 1.34627e+00_rb, 1.34632e+00_rb, 1.34637e+00_rb, 1.34642e+00_rb, 1.34647e+00_rb, &
     & 1.34651e+00_rb, 1.34656e+00_rb, 1.34661e+00_rb, 1.34666e+00_rb/)
      kao_mn2o( 8, :, 3) = (/ &
     & 9.15268e-01_rb, 9.12779e-01_rb, 9.10298e-01_rb, 9.07823e-01_rb, 9.05355e-01_rb, &
     & 9.02893e-01_rb, 9.00438e-01_rb, 8.97990e-01_rb, 8.95549e-01_rb, 8.93114e-01_rb, &
     & 8.90686e-01_rb, 8.88264e-01_rb, 8.85849e-01_rb, 8.83441e-01_rb, 8.81039e-01_rb, &
     & 8.78644e-01_rb, 8.76255e-01_rb, 8.73873e-01_rb, 8.71497e-01_rb/)
      kao_mn2o( 9, :, 3) = (/ &
     & 1.12514e+00_rb, 1.13523e+00_rb, 1.14541e+00_rb, 1.15568e+00_rb, 1.16604e+00_rb, &
     & 1.17649e+00_rb, 1.18704e+00_rb, 1.19768e+00_rb, 1.20841e+00_rb, 1.21925e+00_rb, &
     & 1.23018e+00_rb, 1.24121e+00_rb, 1.25233e+00_rb, 1.26356e+00_rb, 1.27489e+00_rb, &
     & 1.28632e+00_rb, 1.29785e+00_rb, 1.30948e+00_rb, 1.32122e+00_rb/)
      kao_mn2o( 1, :, 4) = (/ &
     & 4.65135e-01_rb, 4.69677e-01_rb, 4.74264e-01_rb, 4.78895e-01_rb, 4.83572e-01_rb, &
     & 4.88294e-01_rb, 4.93063e-01_rb, 4.97878e-01_rb, 5.02740e-01_rb, 5.07649e-01_rb, &
     & 5.12607e-01_rb, 5.17613e-01_rb, 5.22667e-01_rb, 5.27771e-01_rb, 5.32925e-01_rb, &
     & 5.38130e-01_rb, 5.43385e-01_rb, 5.48691e-01_rb, 5.54049e-01_rb/)
      kao_mn2o( 2, :, 4) = (/ &
     & 9.71592e-01_rb, 9.74472e-01_rb, 9.77360e-01_rb, 9.80257e-01_rb, 9.83163e-01_rb, &
     & 9.86077e-01_rb, 9.89000e-01_rb, 9.91931e-01_rb, 9.94871e-01_rb, 9.97820e-01_rb, &
     & 1.00078e+00_rb, 1.00374e+00_rb, 1.00672e+00_rb, 1.00970e+00_rb, 1.01270e+00_rb, &
     & 1.01570e+00_rb, 1.01871e+00_rb, 1.02173e+00_rb, 1.02476e+00_rb/)
      kao_mn2o( 3, :, 4) = (/ &
     & 1.48090e+00_rb, 1.48238e+00_rb, 1.48385e+00_rb, 1.48532e+00_rb, 1.48680e+00_rb, &
     & 1.48828e+00_rb, 1.48976e+00_rb, 1.49124e+00_rb, 1.49272e+00_rb, 1.49420e+00_rb, &
     & 1.49569e+00_rb, 1.49717e+00_rb, 1.49866e+00_rb, 1.50015e+00_rb, 1.50164e+00_rb, &
     & 1.50313e+00_rb, 1.50463e+00_rb, 1.50612e+00_rb, 1.50762e+00_rb/)
      kao_mn2o( 4, :, 4) = (/ &
     & 1.50599e+00_rb, 1.50541e+00_rb, 1.50483e+00_rb, 1.50426e+00_rb, 1.50368e+00_rb, &
     & 1.50310e+00_rb, 1.50252e+00_rb, 1.50195e+00_rb, 1.50137e+00_rb, 1.50079e+00_rb, &
     & 1.50022e+00_rb, 1.49964e+00_rb, 1.49906e+00_rb, 1.49849e+00_rb, 1.49791e+00_rb, &
     & 1.49734e+00_rb, 1.49676e+00_rb, 1.49619e+00_rb, 1.49561e+00_rb/)
      kao_mn2o( 5, :, 4) = (/ &
     & 1.25396e+00_rb, 1.25230e+00_rb, 1.25065e+00_rb, 1.24900e+00_rb, 1.24735e+00_rb, &
     & 1.24570e+00_rb, 1.24405e+00_rb, 1.24241e+00_rb, 1.24077e+00_rb, 1.23913e+00_rb, &
     & 1.23749e+00_rb, 1.23586e+00_rb, 1.23423e+00_rb, 1.23260e+00_rb, 1.23097e+00_rb, &
     & 1.22934e+00_rb, 1.22772e+00_rb, 1.22610e+00_rb, 1.22448e+00_rb/)
      kao_mn2o( 6, :, 4) = (/ &
     & 1.27104e+00_rb, 1.26416e+00_rb, 1.25731e+00_rb, 1.25050e+00_rb, 1.24373e+00_rb, &
     & 1.23700e+00_rb, 1.23030e+00_rb, 1.22364e+00_rb, 1.21701e+00_rb, 1.21043e+00_rb, &
     & 1.20387e+00_rb, 1.19735e+00_rb, 1.19087e+00_rb, 1.18442e+00_rb, 1.17801e+00_rb, &
     & 1.17163e+00_rb, 1.16529e+00_rb, 1.15898e+00_rb, 1.15270e+00_rb/)
      kao_mn2o( 7, :, 4) = (/ &
     & 9.57877e-01_rb, 9.49712e-01_rb, 9.41617e-01_rb, 9.33591e-01_rb, 9.25633e-01_rb, &
     & 9.17743e-01_rb, 9.09920e-01_rb, 9.02164e-01_rb, 8.94473e-01_rb, 8.86849e-01_rb, &
     & 8.79289e-01_rb, 8.71794e-01_rb, 8.64363e-01_rb, 8.56995e-01_rb, 8.49690e-01_rb, &
     & 8.42447e-01_rb, 8.35266e-01_rb, 8.28147e-01_rb, 8.21087e-01_rb/)
      kao_mn2o( 8, :, 4) = (/ &
     & 4.75787e-01_rb, 4.77208e-01_rb, 4.78633e-01_rb, 4.80063e-01_rb, 4.81496e-01_rb, &
     & 4.82934e-01_rb, 4.84377e-01_rb, 4.85823e-01_rb, 4.87274e-01_rb, 4.88730e-01_rb, &
     & 4.90189e-01_rb, 4.91653e-01_rb, 4.93122e-01_rb, 4.94595e-01_rb, 4.96072e-01_rb, &
     & 4.97553e-01_rb, 4.99039e-01_rb, 5.00530e-01_rb, 5.02025e-01_rb/)
      kao_mn2o( 9, :, 4) = (/ &
     & 2.42533e+00_rb, 2.41357e+00_rb, 2.40188e+00_rb, 2.39024e+00_rb, 2.37866e+00_rb, &
     & 2.36713e+00_rb, 2.35566e+00_rb, 2.34425e+00_rb, 2.33289e+00_rb, 2.32158e+00_rb, &
     & 2.31033e+00_rb, 2.29914e+00_rb, 2.28800e+00_rb, 2.27691e+00_rb, 2.26588e+00_rb, &
     & 2.25490e+00_rb, 2.24397e+00_rb, 2.23310e+00_rb, 2.22228e+00_rb/)
      kao_mn2o( 1, :, 5) = (/ &
     & 1.53885e+00_rb, 1.53590e+00_rb, 1.53297e+00_rb, 1.53004e+00_rb, 1.52711e+00_rb, &
     & 1.52419e+00_rb, 1.52128e+00_rb, 1.51837e+00_rb, 1.51547e+00_rb, 1.51257e+00_rb, &
     & 1.50968e+00_rb, 1.50679e+00_rb, 1.50391e+00_rb, 1.50104e+00_rb, 1.49817e+00_rb, &
     & 1.49530e+00_rb, 1.49245e+00_rb, 1.48959e+00_rb, 1.48675e+00_rb/)
      kao_mn2o( 2, :, 5) = (/ &
     & 1.83368e+00_rb, 1.83530e+00_rb, 1.83692e+00_rb, 1.83854e+00_rb, 1.84016e+00_rb, &
     & 1.84178e+00_rb, 1.84340e+00_rb, 1.84503e+00_rb, 1.84665e+00_rb, 1.84828e+00_rb, &
     & 1.84991e+00_rb, 1.85154e+00_rb, 1.85317e+00_rb, 1.85480e+00_rb, 1.85644e+00_rb, &
     & 1.85807e+00_rb, 1.85971e+00_rb, 1.86135e+00_rb, 1.86299e+00_rb/)
      kao_mn2o( 3, :, 5) = (/ &
     & 1.49593e+00_rb, 1.49279e+00_rb, 1.48965e+00_rb, 1.48652e+00_rb, 1.48340e+00_rb, &
     & 1.48028e+00_rb, 1.47717e+00_rb, 1.47406e+00_rb, 1.47096e+00_rb, 1.46787e+00_rb, &
     & 1.46479e+00_rb, 1.46171e+00_rb, 1.45863e+00_rb, 1.45557e+00_rb, 1.45251e+00_rb, &
     & 1.44946e+00_rb, 1.44641e+00_rb, 1.44337e+00_rb, 1.44033e+00_rb/)
      kao_mn2o( 4, :, 5) = (/ &
     & 1.40048e+00_rb, 1.39228e+00_rb, 1.38413e+00_rb, 1.37603e+00_rb, 1.36798e+00_rb, &
     & 1.35997e+00_rb, 1.35201e+00_rb, 1.34410e+00_rb, 1.33623e+00_rb, 1.32841e+00_rb, &
     & 1.32064e+00_rb, 1.31291e+00_rb, 1.30522e+00_rb, 1.29758e+00_rb, 1.28999e+00_rb, &
     & 1.28244e+00_rb, 1.27493e+00_rb, 1.26747e+00_rb, 1.26005e+00_rb/)
      kao_mn2o( 5, :, 5) = (/ &
     & 1.22253e+00_rb, 1.21202e+00_rb, 1.20160e+00_rb, 1.19126e+00_rb, 1.18102e+00_rb, &
     & 1.17087e+00_rb, 1.16080e+00_rb, 1.15082e+00_rb, 1.14092e+00_rb, 1.13111e+00_rb, &
     & 1.12139e+00_rb, 1.11174e+00_rb, 1.10219e+00_rb, 1.09271e+00_rb, 1.08331e+00_rb, &
     & 1.07400e+00_rb, 1.06476e+00_rb, 1.05561e+00_rb, 1.04653e+00_rb/)
      kao_mn2o( 6, :, 5) = (/ &
     & 1.07930e+00_rb, 1.06998e+00_rb, 1.06075e+00_rb, 1.05159e+00_rb, 1.04251e+00_rb, &
     & 1.03352e+00_rb, 1.02459e+00_rb, 1.01575e+00_rb, 1.00698e+00_rb, 9.98291e-01_rb, &
     & 9.89674e-01_rb, 9.81131e-01_rb, 9.72663e-01_rb, 9.64267e-01_rb, 9.55944e-01_rb, &
     & 9.47692e-01_rb, 9.39512e-01_rb, 9.31402e-01_rb, 9.23363e-01_rb/)
      kao_mn2o( 7, :, 5) = (/ &
     & 7.87066e-01_rb, 7.82767e-01_rb, 7.78490e-01_rb, 7.74237e-01_rb, 7.70008e-01_rb, &
     & 7.65801e-01_rb, 7.61617e-01_rb, 7.57457e-01_rb, 7.53319e-01_rb, 7.49203e-01_rb, &
     & 7.45110e-01_rb, 7.41040e-01_rb, 7.36991e-01_rb, 7.32965e-01_rb, 7.28961e-01_rb, &
     & 7.24979e-01_rb, 7.21018e-01_rb, 7.17079e-01_rb, 7.13161e-01_rb/)
      kao_mn2o( 8, :, 5) = (/ &
     & 3.83362e-01_rb, 3.84405e-01_rb, 3.85452e-01_rb, 3.86501e-01_rb, 3.87552e-01_rb, &
     & 3.88607e-01_rb, 3.89665e-01_rb, 3.90725e-01_rb, 3.91788e-01_rb, 3.92855e-01_rb, &
     & 3.93924e-01_rb, 3.94996e-01_rb, 3.96071e-01_rb, 3.97149e-01_rb, 3.98229e-01_rb, &
     & 3.99313e-01_rb, 4.00400e-01_rb, 4.01489e-01_rb, 4.02582e-01_rb/)
      kao_mn2o( 9, :, 5) = (/ &
     & 8.97278e-01_rb, 8.92873e-01_rb, 8.88490e-01_rb, 8.84128e-01_rb, 8.79787e-01_rb, &
     & 8.75468e-01_rb, 8.71170e-01_rb, 8.66893e-01_rb, 8.62637e-01_rb, 8.58402e-01_rb, &
     & 8.54187e-01_rb, 8.49994e-01_rb, 8.45821e-01_rb, 8.41668e-01_rb, 8.37536e-01_rb, &
     & 8.33424e-01_rb, 8.29333e-01_rb, 8.25261e-01_rb, 8.21209e-01_rb/)
      kao_mn2o( 1, :, 6) = (/ &
     & 1.83809e+00_rb, 1.84036e+00_rb, 1.84264e+00_rb, 1.84491e+00_rb, 1.84720e+00_rb, &
     & 1.84948e+00_rb, 1.85177e+00_rb, 1.85406e+00_rb, 1.85635e+00_rb, 1.85864e+00_rb, &
     & 1.86094e+00_rb, 1.86324e+00_rb, 1.86555e+00_rb, 1.86785e+00_rb, 1.87016e+00_rb, &
     & 1.87247e+00_rb, 1.87479e+00_rb, 1.87711e+00_rb, 1.87943e+00_rb/)
      kao_mn2o( 2, :, 6) = (/ &
     & 1.82624e+00_rb, 1.81564e+00_rb, 1.80510e+00_rb, 1.79463e+00_rb, 1.78421e+00_rb, &
     & 1.77386e+00_rb, 1.76356e+00_rb, 1.75333e+00_rb, 1.74315e+00_rb, 1.73304e+00_rb, &
     & 1.72298e+00_rb, 1.71298e+00_rb, 1.70304e+00_rb, 1.69316e+00_rb, 1.68333e+00_rb, &
     & 1.67356e+00_rb, 1.66385e+00_rb, 1.65419e+00_rb, 1.64459e+00_rb/)
      kao_mn2o( 3, :, 6) = (/ &
     & 1.35442e+00_rb, 1.34174e+00_rb, 1.32918e+00_rb, 1.31673e+00_rb, 1.30440e+00_rb, &
     & 1.29219e+00_rb, 1.28010e+00_rb, 1.26811e+00_rb, 1.25624e+00_rb, 1.24448e+00_rb, &
     & 1.23283e+00_rb, 1.22129e+00_rb, 1.20985e+00_rb, 1.19853e+00_rb, 1.18731e+00_rb, &
     & 1.17619e+00_rb, 1.16518e+00_rb, 1.15427e+00_rb, 1.14347e+00_rb/)
      kao_mn2o( 4, :, 6) = (/ &
     & 1.10510e+00_rb, 1.09473e+00_rb, 1.08446e+00_rb, 1.07429e+00_rb, 1.06420e+00_rb, &
     & 1.05422e+00_rb, 1.04433e+00_rb, 1.03453e+00_rb, 1.02482e+00_rb, 1.01520e+00_rb, &
     & 1.00568e+00_rb, 9.96238e-01_rb, 9.86890e-01_rb, 9.77629e-01_rb, 9.68455e-01_rb, &
     & 9.59367e-01_rb, 9.50365e-01_rb, 9.41447e-01_rb, 9.32612e-01_rb/)
      kao_mn2o( 5, :, 6) = (/ &
     & 1.01083e+00_rb, 1.00221e+00_rb, 9.93656e-01_rb, 9.85178e-01_rb, 9.76772e-01_rb, &
     & 9.68437e-01_rb, 9.60174e-01_rb, 9.51981e-01_rb, 9.43859e-01_rb, 9.35805e-01_rb, &
     & 9.27821e-01_rb, 9.19904e-01_rb, 9.12055e-01_rb, 9.04273e-01_rb, 8.96557e-01_rb, &
     & 8.88907e-01_rb, 8.81323e-01_rb, 8.73803e-01_rb, 8.66347e-01_rb/)
      kao_mn2o( 6, :, 6) = (/ &
     & 5.91415e-01_rb, 5.90427e-01_rb, 5.89441e-01_rb, 5.88457e-01_rb, 5.87474e-01_rb, &
     & 5.86493e-01_rb, 5.85514e-01_rb, 5.84536e-01_rb, 5.83559e-01_rb, 5.82585e-01_rb, &
     & 5.81612e-01_rb, 5.80640e-01_rb, 5.79671e-01_rb, 5.78703e-01_rb, 5.77736e-01_rb, &
     & 5.76771e-01_rb, 5.75808e-01_rb, 5.74846e-01_rb, 5.73886e-01_rb/)
      kao_mn2o( 7, :, 6) = (/ &
     & 3.68189e-01_rb, 3.70029e-01_rb, 3.71877e-01_rb, 3.73735e-01_rb, 3.75603e-01_rb, &
     & 3.77479e-01_rb, 3.79365e-01_rb, 3.81260e-01_rb, 3.83165e-01_rb, 3.85079e-01_rb, &
     & 3.87003e-01_rb, 3.88937e-01_rb, 3.90880e-01_rb, 3.92833e-01_rb, 3.94795e-01_rb, &
     & 3.96768e-01_rb, 3.98750e-01_rb, 4.00742e-01_rb, 4.02744e-01_rb/)
      kao_mn2o( 8, :, 6) = (/ &
     & 2.98721e-01_rb, 2.99932e-01_rb, 3.01149e-01_rb, 3.02370e-01_rb, 3.03597e-01_rb, &
     & 3.04828e-01_rb, 3.06064e-01_rb, 3.07306e-01_rb, 3.08552e-01_rb, 3.09804e-01_rb, &
     & 3.11060e-01_rb, 3.12322e-01_rb, 3.13589e-01_rb, 3.14860e-01_rb, 3.16138e-01_rb, &
     & 3.17420e-01_rb, 3.18707e-01_rb, 3.20000e-01_rb, 3.21298e-01_rb/)
      kao_mn2o( 9, :, 6) = (/ &
     & 3.76116e-01_rb, 3.77276e-01_rb, 3.78439e-01_rb, 3.79606e-01_rb, 3.80777e-01_rb, &
     & 3.81951e-01_rb, 3.83129e-01_rb, 3.84310e-01_rb, 3.85495e-01_rb, 3.86684e-01_rb, &
     & 3.87876e-01_rb, 3.89072e-01_rb, 3.90272e-01_rb, 3.91475e-01_rb, 3.92682e-01_rb, &
     & 3.93893e-01_rb, 3.95107e-01_rb, 3.96326e-01_rb, 3.97548e-01_rb/)
      kao_mn2o( 1, :, 7) = (/ &
     & 3.22705e+00_rb, 3.21966e+00_rb, 3.21230e+00_rb, 3.20494e+00_rb, 3.19761e+00_rb, &
     & 3.19029e+00_rb, 3.18299e+00_rb, 3.17571e+00_rb, 3.16844e+00_rb, 3.16119e+00_rb, &
     & 3.15395e+00_rb, 3.14673e+00_rb, 3.13953e+00_rb, 3.13235e+00_rb, 3.12518e+00_rb, &
     & 3.11803e+00_rb, 3.11089e+00_rb, 3.10377e+00_rb, 3.09667e+00_rb/)
      kao_mn2o( 2, :, 7) = (/ &
     & 1.43811e+00_rb, 1.42367e+00_rb, 1.40938e+00_rb, 1.39522e+00_rb, 1.38121e+00_rb, &
     & 1.36735e+00_rb, 1.35362e+00_rb, 1.34002e+00_rb, 1.32657e+00_rb, 1.31325e+00_rb, &
     & 1.30006e+00_rb, 1.28701e+00_rb, 1.27408e+00_rb, 1.26129e+00_rb, 1.24862e+00_rb, &
     & 1.23609e+00_rb, 1.22367e+00_rb, 1.21139e+00_rb, 1.19922e+00_rb/)
      kao_mn2o( 3, :, 7) = (/ &
     & 1.22586e+00_rb, 1.21639e+00_rb, 1.20700e+00_rb, 1.19767e+00_rb, 1.18842e+00_rb, &
     & 1.17924e+00_rb, 1.17014e+00_rb, 1.16110e+00_rb, 1.15213e+00_rb, 1.14323e+00_rb, &
     & 1.13440e+00_rb, 1.12564e+00_rb, 1.11695e+00_rb, 1.10832e+00_rb, 1.09976e+00_rb, &
     & 1.09127e+00_rb, 1.08284e+00_rb, 1.07447e+00_rb, 1.06618e+00_rb/)
      kao_mn2o( 4, :, 7) = (/ &
     & 7.94380e-01_rb, 7.92795e-01_rb, 7.91213e-01_rb, 7.89634e-01_rb, 7.88059e-01_rb, &
     & 7.86486e-01_rb, 7.84917e-01_rb, 7.83351e-01_rb, 7.81787e-01_rb, 7.80227e-01_rb, &
     & 7.78671e-01_rb, 7.77117e-01_rb, 7.75566e-01_rb, 7.74019e-01_rb, 7.72474e-01_rb, &
     & 7.70933e-01_rb, 7.69394e-01_rb, 7.67859e-01_rb, 7.66327e-01_rb/)
      kao_mn2o( 5, :, 7) = (/ &
     & 4.46935e-01_rb, 4.49760e-01_rb, 4.52602e-01_rb, 4.55462e-01_rb, 4.58340e-01_rb, &
     & 4.61237e-01_rb, 4.64152e-01_rb, 4.67085e-01_rb, 4.70037e-01_rb, 4.73007e-01_rb, &
     & 4.75996e-01_rb, 4.79004e-01_rb, 4.82031e-01_rb, 4.85078e-01_rb, 4.88143e-01_rb, &
     & 4.91228e-01_rb, 4.94332e-01_rb, 4.97456e-01_rb, 5.00600e-01_rb/)
      kao_mn2o( 6, :, 7) = (/ &
     & 4.20211e-01_rb, 4.21711e-01_rb, 4.23216e-01_rb, 4.24726e-01_rb, 4.26242e-01_rb, &
     & 4.27763e-01_rb, 4.29290e-01_rb, 4.30822e-01_rb, 4.32359e-01_rb, 4.33902e-01_rb, &
     & 4.35450e-01_rb, 4.37004e-01_rb, 4.38564e-01_rb, 4.40129e-01_rb, 4.41700e-01_rb, &
     & 4.43276e-01_rb, 4.44858e-01_rb, 4.46445e-01_rb, 4.48038e-01_rb/)
      kao_mn2o( 7, :, 7) = (/ &
     & 3.42094e-01_rb, 3.43589e-01_rb, 3.45091e-01_rb, 3.46600e-01_rb, 3.48115e-01_rb, &
     & 3.49637e-01_rb, 3.51165e-01_rb, 3.52700e-01_rb, 3.54242e-01_rb, 3.55791e-01_rb, &
     & 3.57346e-01_rb, 3.58908e-01_rb, 3.60477e-01_rb, 3.62053e-01_rb, 3.63636e-01_rb, &
     & 3.65225e-01_rb, 3.66822e-01_rb, 3.68426e-01_rb, 3.70036e-01_rb/)
      kao_mn2o( 8, :, 7) = (/ &
     & 2.94919e-01_rb, 2.97460e-01_rb, 3.00022e-01_rb, 3.02606e-01_rb, 3.05212e-01_rb, &
     & 3.07841e-01_rb, 3.10492e-01_rb, 3.13167e-01_rb, 3.15864e-01_rb, 3.18584e-01_rb, &
     & 3.21328e-01_rb, 3.24096e-01_rb, 3.26887e-01_rb, 3.29703e-01_rb, 3.32543e-01_rb, &
     & 3.35407e-01_rb, 3.38296e-01_rb, 3.41210e-01_rb, 3.44148e-01_rb/)
      kao_mn2o( 9, :, 7) = (/ &
     & 2.97441e-01_rb, 2.99207e-01_rb, 3.00984e-01_rb, 3.02772e-01_rb, 3.04570e-01_rb, &
     & 3.06379e-01_rb, 3.08199e-01_rb, 3.10029e-01_rb, 3.11870e-01_rb, 3.13723e-01_rb, &
     & 3.15586e-01_rb, 3.17460e-01_rb, 3.19345e-01_rb, 3.21242e-01_rb, 3.23150e-01_rb, &
     & 3.25069e-01_rb, 3.27000e-01_rb, 3.28942e-01_rb, 3.30895e-01_rb/)
      kao_mn2o( 1, :, 8) = (/ &
     & 2.14641e+00_rb, 2.12585e+00_rb, 2.10549e+00_rb, 2.08532e+00_rb, 2.06534e+00_rb, &
     & 2.04556e+00_rb, 2.02596e+00_rb, 2.00656e+00_rb, 1.98733e+00_rb, 1.96830e+00_rb, &
     & 1.94944e+00_rb, 1.93077e+00_rb, 1.91227e+00_rb, 1.89395e+00_rb, 1.87581e+00_rb, &
     & 1.85784e+00_rb, 1.84004e+00_rb, 1.82242e+00_rb, 1.80496e+00_rb/)
      kao_mn2o( 2, :, 8) = (/ &
     & 8.83687e-01_rb, 8.83170e-01_rb, 8.82654e-01_rb, 8.82137e-01_rb, 8.81621e-01_rb, &
     & 8.81106e-01_rb, 8.80590e-01_rb, 8.80075e-01_rb, 8.79560e-01_rb, 8.79046e-01_rb, &
     & 8.78531e-01_rb, 8.78017e-01_rb, 8.77504e-01_rb, 8.76990e-01_rb, 8.76477e-01_rb, &
     & 8.75965e-01_rb, 8.75452e-01_rb, 8.74940e-01_rb, 8.74428e-01_rb/)
      kao_mn2o( 3, :, 8) = (/ &
     & 4.49840e-01_rb, 4.52683e-01_rb, 4.55543e-01_rb, 4.58421e-01_rb, 4.61318e-01_rb, &
     & 4.64233e-01_rb, 4.67166e-01_rb, 4.70118e-01_rb, 4.73088e-01_rb, 4.76078e-01_rb, &
     & 4.79086e-01_rb, 4.82113e-01_rb, 4.85159e-01_rb, 4.88225e-01_rb, 4.91310e-01_rb, &
     & 4.94414e-01_rb, 4.97538e-01_rb, 5.00682e-01_rb, 5.03845e-01_rb/)
      kao_mn2o( 4, :, 8) = (/ &
     & 3.92292e-01_rb, 3.93574e-01_rb, 3.94861e-01_rb, 3.96151e-01_rb, 3.97446e-01_rb, &
     & 3.98746e-01_rb, 4.00049e-01_rb, 4.01357e-01_rb, 4.02669e-01_rb, 4.03985e-01_rb, &
     & 4.05306e-01_rb, 4.06630e-01_rb, 4.07960e-01_rb, 4.09293e-01_rb, 4.10631e-01_rb, &
     & 4.11973e-01_rb, 4.13320e-01_rb, 4.14671e-01_rb, 4.16027e-01_rb/)
      kao_mn2o( 5, :, 8) = (/ &
     & 3.38920e-01_rb, 3.41151e-01_rb, 3.43397e-01_rb, 3.45658e-01_rb, 3.47934e-01_rb, &
     & 3.50225e-01_rb, 3.52531e-01_rb, 3.54852e-01_rb, 3.57189e-01_rb, 3.59541e-01_rb, &
     & 3.61908e-01_rb, 3.64291e-01_rb, 3.66689e-01_rb, 3.69104e-01_rb, 3.71534e-01_rb, &
     & 3.73980e-01_rb, 3.76443e-01_rb, 3.78921e-01_rb, 3.81416e-01_rb/)
      kao_mn2o( 6, :, 8) = (/ &
     & 3.01673e-01_rb, 3.04752e-01_rb, 3.07863e-01_rb, 3.11005e-01_rb, 3.14180e-01_rb, &
     & 3.17387e-01_rb, 3.20626e-01_rb, 3.23899e-01_rb, 3.27205e-01_rb, 3.30545e-01_rb, &
     & 3.33919e-01_rb, 3.37328e-01_rb, 3.40771e-01_rb, 3.44249e-01_rb, 3.47763e-01_rb, &
     & 3.51313e-01_rb, 3.54899e-01_rb, 3.58521e-01_rb, 3.62181e-01_rb/)
      kao_mn2o( 7, :, 8) = (/ &
     & 2.99381e-01_rb, 3.02431e-01_rb, 3.05512e-01_rb, 3.08624e-01_rb, 3.11768e-01_rb, &
     & 3.14945e-01_rb, 3.18153e-01_rb, 3.21394e-01_rb, 3.24668e-01_rb, 3.27976e-01_rb, &
     & 3.31317e-01_rb, 3.34693e-01_rb, 3.38102e-01_rb, 3.41547e-01_rb, 3.45026e-01_rb, &
     & 3.48541e-01_rb, 3.52092e-01_rb, 3.55679e-01_rb, 3.59302e-01_rb/)
      kao_mn2o( 8, :, 8) = (/ &
     & 2.87559e-01_rb, 2.89153e-01_rb, 2.90756e-01_rb, 2.92367e-01_rb, 2.93987e-01_rb, &
     & 2.95617e-01_rb, 2.97255e-01_rb, 2.98902e-01_rb, 3.00559e-01_rb, 3.02225e-01_rb, &
     & 3.03899e-01_rb, 3.05584e-01_rb, 3.07277e-01_rb, 3.08980e-01_rb, 3.10693e-01_rb, &
     & 3.12414e-01_rb, 3.14146e-01_rb, 3.15887e-01_rb, 3.17638e-01_rb/)
      kao_mn2o( 9, :, 8) = (/ &
     & 2.96238e-01_rb, 2.97588e-01_rb, 2.98945e-01_rb, 3.00309e-01_rb, 3.01678e-01_rb, &
     & 3.03054e-01_rb, 3.04436e-01_rb, 3.05824e-01_rb, 3.07219e-01_rb, 3.08620e-01_rb, &
     & 3.10027e-01_rb, 3.11441e-01_rb, 3.12861e-01_rb, 3.14288e-01_rb, 3.15721e-01_rb, &
     & 3.17161e-01_rb, 3.18607e-01_rb, 3.20060e-01_rb, 3.21520e-01_rb/)
      kao_mn2o( 1, :, 9) = (/ &
     & 1.56483e+00_rb, 1.55792e+00_rb, 1.55105e+00_rb, 1.54420e+00_rb, 1.53739e+00_rb, &
     & 1.53060e+00_rb, 1.52384e+00_rb, 1.51712e+00_rb, 1.51042e+00_rb, 1.50376e+00_rb, &
     & 1.49712e+00_rb, 1.49051e+00_rb, 1.48393e+00_rb, 1.47738e+00_rb, 1.47086e+00_rb, &
     & 1.46437e+00_rb, 1.45791e+00_rb, 1.45147e+00_rb, 1.44507e+00_rb/)
      kao_mn2o( 2, :, 9) = (/ &
     & 4.09526e-01_rb, 4.10301e-01_rb, 4.11078e-01_rb, 4.11857e-01_rb, 4.12637e-01_rb, &
     & 4.13418e-01_rb, 4.14201e-01_rb, 4.14986e-01_rb, 4.15771e-01_rb, 4.16559e-01_rb, &
     & 4.17348e-01_rb, 4.18138e-01_rb, 4.18930e-01_rb, 4.19723e-01_rb, 4.20518e-01_rb, &
     & 4.21315e-01_rb, 4.22112e-01_rb, 4.22912e-01_rb, 4.23713e-01_rb/)
      kao_mn2o( 3, :, 9) = (/ &
     & 3.35672e-01_rb, 3.38982e-01_rb, 3.42326e-01_rb, 3.45702e-01_rb, 3.49111e-01_rb, &
     & 3.52554e-01_rb, 3.56031e-01_rb, 3.59543e-01_rb, 3.63089e-01_rb, 3.66670e-01_rb, &
     & 3.70286e-01_rb, 3.73938e-01_rb, 3.77626e-01_rb, 3.81350e-01_rb, 3.85111e-01_rb, &
     & 3.88909e-01_rb, 3.92745e-01_rb, 3.96618e-01_rb, 4.00530e-01_rb/)
      kao_mn2o( 4, :, 9) = (/ &
     & 3.19130e-01_rb, 3.23028e-01_rb, 3.26973e-01_rb, 3.30966e-01_rb, 3.35008e-01_rb, &
     & 3.39100e-01_rb, 3.43241e-01_rb, 3.47433e-01_rb, 3.51676e-01_rb, 3.55971e-01_rb, &
     & 3.60319e-01_rb, 3.64719e-01_rb, 3.69173e-01_rb, 3.73682e-01_rb, 3.78246e-01_rb, &
     & 3.82865e-01_rb, 3.87541e-01_rb, 3.92274e-01_rb, 3.97065e-01_rb/)
      kao_mn2o( 5, :, 9) = (/ &
     & 3.04385e-01_rb, 3.07155e-01_rb, 3.09949e-01_rb, 3.12770e-01_rb, 3.15616e-01_rb, &
     & 3.18488e-01_rb, 3.21386e-01_rb, 3.24310e-01_rb, 3.27261e-01_rb, 3.30239e-01_rb, &
     & 3.33244e-01_rb, 3.36276e-01_rb, 3.39336e-01_rb, 3.42424e-01_rb, 3.45540e-01_rb, &
     & 3.48684e-01_rb, 3.51857e-01_rb, 3.55059e-01_rb, 3.58289e-01_rb/)
      kao_mn2o( 6, :, 9) = (/ &
     & 2.98789e-01_rb, 3.00996e-01_rb, 3.03220e-01_rb, 3.05460e-01_rb, 3.07717e-01_rb, &
     & 3.09990e-01_rb, 3.12281e-01_rb, 3.14588e-01_rb, 3.16912e-01_rb, 3.19253e-01_rb, &
     & 3.21612e-01_rb, 3.23988e-01_rb, 3.26382e-01_rb, 3.28793e-01_rb, 3.31222e-01_rb, &
     & 3.33669e-01_rb, 3.36134e-01_rb, 3.38618e-01_rb, 3.41119e-01_rb/)
      kao_mn2o( 7, :, 9) = (/ &
     & 3.08712e-01_rb, 3.10491e-01_rb, 3.12281e-01_rb, 3.14080e-01_rb, 3.15890e-01_rb, &
     & 3.17710e-01_rb, 3.19541e-01_rb, 3.21382e-01_rb, 3.23234e-01_rb, 3.25097e-01_rb, &
     & 3.26970e-01_rb, 3.28854e-01_rb, 3.30749e-01_rb, 3.32655e-01_rb, 3.34572e-01_rb, &
     & 3.36500e-01_rb, 3.38439e-01_rb, 3.40390e-01_rb, 3.42351e-01_rb/)
      kao_mn2o( 8, :, 9) = (/ &
     & 3.10571e-01_rb, 3.12262e-01_rb, 3.13961e-01_rb, 3.15670e-01_rb, 3.17388e-01_rb, &
     & 3.19115e-01_rb, 3.20852e-01_rb, 3.22598e-01_rb, 3.24354e-01_rb, 3.26120e-01_rb, &
     & 3.27895e-01_rb, 3.29679e-01_rb, 3.31474e-01_rb, 3.33278e-01_rb, 3.35092e-01_rb, &
     & 3.36915e-01_rb, 3.38749e-01_rb, 3.40593e-01_rb, 3.42447e-01_rb/)
      kao_mn2o( 9, :, 9) = (/ &
     & 3.16436e-01_rb, 3.18200e-01_rb, 3.19974e-01_rb, 3.21759e-01_rb, 3.23553e-01_rb, &
     & 3.25357e-01_rb, 3.27172e-01_rb, 3.28996e-01_rb, 3.30831e-01_rb, 3.32675e-01_rb, &
     & 3.34530e-01_rb, 3.36396e-01_rb, 3.38272e-01_rb, 3.40158e-01_rb, 3.42055e-01_rb, &
     & 3.43962e-01_rb, 3.45880e-01_rb, 3.47809e-01_rb, 3.49749e-01_rb/)
      kao_mn2o( 1, :,10) = (/ &
     & 7.68616e-01_rb, 7.63263e-01_rb, 7.57948e-01_rb, 7.52669e-01_rb, 7.47428e-01_rb, &
     & 7.42223e-01_rb, 7.37054e-01_rb, 7.31921e-01_rb, 7.26824e-01_rb, 7.21762e-01_rb, &
     & 7.16736e-01_rb, 7.11744e-01_rb, 7.06788e-01_rb, 7.01866e-01_rb, 6.96978e-01_rb, &
     & 6.92124e-01_rb, 6.87304e-01_rb, 6.82517e-01_rb, 6.77764e-01_rb/)
      kao_mn2o( 2, :,10) = (/ &
     & 4.97271e-01_rb, 5.10054e-01_rb, 5.23165e-01_rb, 5.36614e-01_rb, 5.50408e-01_rb, &
     & 5.64556e-01_rb, 5.79069e-01_rb, 5.93954e-01_rb, 6.09222e-01_rb, 6.24883e-01_rb, &
     & 6.40946e-01_rb, 6.57422e-01_rb, 6.74321e-01_rb, 6.91655e-01_rb, 7.09435e-01_rb, &
     & 7.27671e-01_rb, 7.46377e-01_rb, 7.65563e-01_rb, 7.85242e-01_rb/)
      kao_mn2o( 3, :,10) = (/ &
     & 2.44443e-01_rb, 2.47096e-01_rb, 2.49778e-01_rb, 2.52489e-01_rb, 2.55229e-01_rb, &
     & 2.57999e-01_rb, 2.60799e-01_rb, 2.63630e-01_rb, 2.66491e-01_rb, 2.69383e-01_rb, &
     & 2.72307e-01_rb, 2.75262e-01_rb, 2.78250e-01_rb, 2.81269e-01_rb, 2.84322e-01_rb, &
     & 2.87408e-01_rb, 2.90527e-01_rb, 2.93680e-01_rb, 2.96868e-01_rb/)
      kao_mn2o( 4, :,10) = (/ &
     & 2.01964e-01_rb, 2.02869e-01_rb, 2.03777e-01_rb, 2.04690e-01_rb, 2.05606e-01_rb, &
     & 2.06527e-01_rb, 2.07452e-01_rb, 2.08381e-01_rb, 2.09314e-01_rb, 2.10251e-01_rb, &
     & 2.11193e-01_rb, 2.12139e-01_rb, 2.13089e-01_rb, 2.14043e-01_rb, 2.15002e-01_rb, &
     & 2.15964e-01_rb, 2.16932e-01_rb, 2.17903e-01_rb, 2.18879e-01_rb/)
      kao_mn2o( 5, :,10) = (/ &
     & 2.56972e-01_rb, 2.56837e-01_rb, 2.56702e-01_rb, 2.56567e-01_rb, 2.56432e-01_rb, &
     & 2.56297e-01_rb, 2.56162e-01_rb, 2.56027e-01_rb, 2.55893e-01_rb, 2.55758e-01_rb, &
     & 2.55624e-01_rb, 2.55489e-01_rb, 2.55355e-01_rb, 2.55220e-01_rb, 2.55086e-01_rb, &
     & 2.54952e-01_rb, 2.54818e-01_rb, 2.54684e-01_rb, 2.54550e-01_rb/)
      kao_mn2o( 6, :,10) = (/ &
     & 2.57322e-01_rb, 2.57187e-01_rb, 2.57052e-01_rb, 2.56917e-01_rb, 2.56782e-01_rb, &
     & 2.56647e-01_rb, 2.56512e-01_rb, 2.56377e-01_rb, 2.56243e-01_rb, 2.56108e-01_rb, &
     & 2.55974e-01_rb, 2.55839e-01_rb, 2.55705e-01_rb, 2.55570e-01_rb, 2.55436e-01_rb, &
     & 2.55302e-01_rb, 2.55168e-01_rb, 2.55034e-01_rb, 2.54900e-01_rb/)
      kao_mn2o( 7, :,10) = (/ &
     & 2.56551e-01_rb, 2.56421e-01_rb, 2.56291e-01_rb, 2.56161e-01_rb, 2.56030e-01_rb, &
     & 2.55900e-01_rb, 2.55770e-01_rb, 2.55640e-01_rb, 2.55511e-01_rb, 2.55381e-01_rb, &
     & 2.55251e-01_rb, 2.55121e-01_rb, 2.54992e-01_rb, 2.54862e-01_rb, 2.54733e-01_rb, &
     & 2.54603e-01_rb, 2.54474e-01_rb, 2.54345e-01_rb, 2.54215e-01_rb/)
      kao_mn2o( 8, :,10) = (/ &
     & 2.73629e-01_rb, 2.73460e-01_rb, 2.73291e-01_rb, 2.73122e-01_rb, 2.72953e-01_rb, &
     & 2.72784e-01_rb, 2.72615e-01_rb, 2.72447e-01_rb, 2.72279e-01_rb, 2.72110e-01_rb, &
     & 2.71942e-01_rb, 2.71774e-01_rb, 2.71606e-01_rb, 2.71438e-01_rb, 2.71270e-01_rb, &
     & 2.71102e-01_rb, 2.70935e-01_rb, 2.70767e-01_rb, 2.70600e-01_rb/)
      kao_mn2o( 9, :,10) = (/ &
     & 2.57294e-01_rb, 2.57149e-01_rb, 2.57004e-01_rb, 2.56860e-01_rb, 2.56715e-01_rb, &
     & 2.56570e-01_rb, 2.56426e-01_rb, 2.56282e-01_rb, 2.56137e-01_rb, 2.55993e-01_rb, &
     & 2.55849e-01_rb, 2.55705e-01_rb, 2.55561e-01_rb, 2.55417e-01_rb, 2.55273e-01_rb, &
     & 2.55129e-01_rb, 2.54986e-01_rb, 2.54842e-01_rb, 2.54698e-01_rb/)
      kao_mn2o( 1, :,11) = (/ &
     & 6.91062e-01_rb, 6.84151e-01_rb, 6.77309e-01_rb, 6.70535e-01_rb, 6.63829e-01_rb, &
     & 6.57190e-01_rb, 6.50617e-01_rb, 6.44111e-01_rb, 6.37669e-01_rb, 6.31292e-01_rb, &
     & 6.24978e-01_rb, 6.18728e-01_rb, 6.12540e-01_rb, 6.06414e-01_rb, 6.00349e-01_rb, &
     & 5.94345e-01_rb, 5.88401e-01_rb, 5.82517e-01_rb, 5.76691e-01_rb/)
      kao_mn2o( 2, :,11) = (/ &
     & 1.98698e-01_rb, 2.01182e-01_rb, 2.03698e-01_rb, 2.06244e-01_rb, 2.08823e-01_rb, &
     & 2.11433e-01_rb, 2.14077e-01_rb, 2.16753e-01_rb, 2.19463e-01_rb, 2.22207e-01_rb, &
     & 2.24985e-01_rb, 2.27798e-01_rb, 2.30646e-01_rb, 2.33529e-01_rb, 2.36449e-01_rb, &
     & 2.39405e-01_rb, 2.42398e-01_rb, 2.45429e-01_rb, 2.48497e-01_rb/)
      kao_mn2o( 3, :,11) = (/ &
     & 2.11950e-01_rb, 2.13560e-01_rb, 2.15184e-01_rb, 2.16819e-01_rb, 2.18467e-01_rb, &
     & 2.20127e-01_rb, 2.21800e-01_rb, 2.23486e-01_rb, 2.25185e-01_rb, 2.26896e-01_rb, &
     & 2.28621e-01_rb, 2.30358e-01_rb, 2.32109e-01_rb, 2.33873e-01_rb, 2.35651e-01_rb, &
     & 2.37442e-01_rb, 2.39247e-01_rb, 2.41065e-01_rb, 2.42897e-01_rb/)
      kao_mn2o( 4, :,11) = (/ &
     & 3.14210e-01_rb, 3.13143e-01_rb, 3.12080e-01_rb, 3.11021e-01_rb, 3.09965e-01_rb, &
     & 3.08913e-01_rb, 3.07864e-01_rb, 3.06819e-01_rb, 3.05777e-01_rb, 3.04739e-01_rb, &
     & 3.03705e-01_rb, 3.02674e-01_rb, 3.01646e-01_rb, 3.00622e-01_rb, 2.99602e-01_rb, &
     & 2.98584e-01_rb, 2.97571e-01_rb, 2.96561e-01_rb, 2.95554e-01_rb/)
      kao_mn2o( 5, :,11) = (/ &
     & 3.13536e-01_rb, 3.12459e-01_rb, 3.11386e-01_rb, 3.10316e-01_rb, 3.09250e-01_rb, &
     & 3.08188e-01_rb, 3.07129e-01_rb, 3.06074e-01_rb, 3.05022e-01_rb, 3.03974e-01_rb, &
     & 3.02930e-01_rb, 3.01889e-01_rb, 3.00852e-01_rb, 2.99819e-01_rb, 2.98789e-01_rb, &
     & 2.97762e-01_rb, 2.96739e-01_rb, 2.95720e-01_rb, 2.94704e-01_rb/)
      kao_mn2o( 6, :,11) = (/ &
     & 3.13215e-01_rb, 3.12123e-01_rb, 3.11034e-01_rb, 3.09949e-01_rb, 3.08867e-01_rb, &
     & 3.07790e-01_rb, 3.06716e-01_rb, 3.05646e-01_rb, 3.04579e-01_rb, 3.03517e-01_rb, &
     & 3.02458e-01_rb, 3.01403e-01_rb, 3.00351e-01_rb, 2.99303e-01_rb, 2.98259e-01_rb, &
     & 2.97219e-01_rb, 2.96182e-01_rb, 2.95148e-01_rb, 2.94119e-01_rb/)
      kao_mn2o( 7, :,11) = (/ &
     & 3.14236e-01_rb, 3.13123e-01_rb, 3.12014e-01_rb, 3.10908e-01_rb, 3.09806e-01_rb, &
     & 3.08709e-01_rb, 3.07615e-01_rb, 3.06525e-01_rb, 3.05439e-01_rb, 3.04357e-01_rb, &
     & 3.03278e-01_rb, 3.02204e-01_rb, 3.01133e-01_rb, 3.00066e-01_rb, 2.99003e-01_rb, &
     & 2.97944e-01_rb, 2.96888e-01_rb, 2.95836e-01_rb, 2.94788e-01_rb/)
      kao_mn2o( 8, :,11) = (/ &
     & 2.97453e-01_rb, 2.96420e-01_rb, 2.95391e-01_rb, 2.94366e-01_rb, 2.93344e-01_rb, &
     & 2.92325e-01_rb, 2.91311e-01_rb, 2.90299e-01_rb, 2.89291e-01_rb, 2.88287e-01_rb, &
     & 2.87286e-01_rb, 2.86289e-01_rb, 2.85295e-01_rb, 2.84304e-01_rb, 2.83317e-01_rb, &
     & 2.82334e-01_rb, 2.81354e-01_rb, 2.80377e-01_rb, 2.79404e-01_rb/)
      kao_mn2o( 9, :,11) = (/ &
     & 3.12694e-01_rb, 3.11622e-01_rb, 3.10554e-01_rb, 3.09489e-01_rb, 3.08428e-01_rb, &
     & 3.07370e-01_rb, 3.06316e-01_rb, 3.05266e-01_rb, 3.04220e-01_rb, 3.03177e-01_rb, &
     & 3.02137e-01_rb, 3.01101e-01_rb, 3.00069e-01_rb, 2.99040e-01_rb, 2.98015e-01_rb, &
     & 2.96993e-01_rb, 2.95975e-01_rb, 2.94960e-01_rb, 2.93949e-01_rb/)
      kao_mn2o( 1, :,12) = (/ &
     & 5.30796e-01_rb, 5.50444e-01_rb, 5.70818e-01_rb, 5.91947e-01_rb, 6.13857e-01_rb, &
     & 6.36579e-01_rb, 6.60142e-01_rb, 6.84577e-01_rb, 7.09916e-01_rb, 7.36194e-01_rb, &
     & 7.63444e-01_rb, 7.91702e-01_rb, 8.21007e-01_rb, 8.51396e-01_rb, 8.82910e-01_rb, &
     & 9.15591e-01_rb, 9.49481e-01_rb, 9.84626e-01_rb, 1.02107e+00_rb/)
      kao_mn2o( 2, :,12) = (/ &
     & 1.38469e-01_rb, 1.40959e-01_rb, 1.43493e-01_rb, 1.46073e-01_rb, 1.48699e-01_rb, &
     & 1.51373e-01_rb, 1.54094e-01_rb, 1.56865e-01_rb, 1.59685e-01_rb, 1.62556e-01_rb, &
     & 1.65478e-01_rb, 1.68454e-01_rb, 1.71482e-01_rb, 1.74565e-01_rb, 1.77704e-01_rb, &
     & 1.80899e-01_rb, 1.84151e-01_rb, 1.87462e-01_rb, 1.90833e-01_rb/)
      kao_mn2o( 3, :,12) = (/ &
     & 1.50741e-01_rb, 1.50855e-01_rb, 1.50969e-01_rb, 1.51084e-01_rb, 1.51198e-01_rb, &
     & 1.51313e-01_rb, 1.51427e-01_rb, 1.51542e-01_rb, 1.51657e-01_rb, 1.51772e-01_rb, &
     & 1.51887e-01_rb, 1.52002e-01_rb, 1.52117e-01_rb, 1.52233e-01_rb, 1.52348e-01_rb, &
     & 1.52463e-01_rb, 1.52579e-01_rb, 1.52695e-01_rb, 1.52810e-01_rb/)
      kao_mn2o( 4, :,12) = (/ &
     & 1.80444e-01_rb, 1.79944e-01_rb, 1.79445e-01_rb, 1.78948e-01_rb, 1.78452e-01_rb, &
     & 1.77958e-01_rb, 1.77465e-01_rb, 1.76973e-01_rb, 1.76483e-01_rb, 1.75994e-01_rb, &
     & 1.75506e-01_rb, 1.75020e-01_rb, 1.74535e-01_rb, 1.74051e-01_rb, 1.73569e-01_rb, &
     & 1.73088e-01_rb, 1.72609e-01_rb, 1.72131e-01_rb, 1.71654e-01_rb/)
      kao_mn2o( 5, :,12) = (/ &
     & 1.80595e-01_rb, 1.80033e-01_rb, 1.79474e-01_rb, 1.78916e-01_rb, 1.78359e-01_rb, &
     & 1.77805e-01_rb, 1.77252e-01_rb, 1.76701e-01_rb, 1.76152e-01_rb, 1.75604e-01_rb, &
     & 1.75058e-01_rb, 1.74514e-01_rb, 1.73971e-01_rb, 1.73430e-01_rb, 1.72891e-01_rb, &
     & 1.72354e-01_rb, 1.71818e-01_rb, 1.71284e-01_rb, 1.70751e-01_rb/)
      kao_mn2o( 6, :,12) = (/ &
     & 1.79904e-01_rb, 1.79254e-01_rb, 1.78607e-01_rb, 1.77962e-01_rb, 1.77320e-01_rb, &
     & 1.76680e-01_rb, 1.76042e-01_rb, 1.75406e-01_rb, 1.74773e-01_rb, 1.74142e-01_rb, &
     & 1.73513e-01_rb, 1.72887e-01_rb, 1.72262e-01_rb, 1.71640e-01_rb, 1.71021e-01_rb, &
     & 1.70403e-01_rb, 1.69788e-01_rb, 1.69175e-01_rb, 1.68564e-01_rb/)
      kao_mn2o( 7, :,12) = (/ &
     & 1.78712e-01_rb, 1.77868e-01_rb, 1.77027e-01_rb, 1.76190e-01_rb, 1.75357e-01_rb, &
     & 1.74528e-01_rb, 1.73703e-01_rb, 1.72882e-01_rb, 1.72064e-01_rb, 1.71251e-01_rb, &
     & 1.70441e-01_rb, 1.69636e-01_rb, 1.68834e-01_rb, 1.68036e-01_rb, 1.67241e-01_rb, &
     & 1.66451e-01_rb, 1.65664e-01_rb, 1.64881e-01_rb, 1.64101e-01_rb/)
      kao_mn2o( 8, :,12) = (/ &
     & 1.72346e-01_rb, 1.70873e-01_rb, 1.69413e-01_rb, 1.67965e-01_rb, 1.66530e-01_rb, &
     & 1.65107e-01_rb, 1.63696e-01_rb, 1.62297e-01_rb, 1.60910e-01_rb, 1.59535e-01_rb, &
     & 1.58171e-01_rb, 1.56819e-01_rb, 1.55479e-01_rb, 1.54150e-01_rb, 1.52833e-01_rb, &
     & 1.51527e-01_rb, 1.50232e-01_rb, 1.48948e-01_rb, 1.47675e-01_rb/)
      kao_mn2o( 9, :,12) = (/ &
     & 1.80517e-01_rb, 1.79951e-01_rb, 1.79386e-01_rb, 1.78823e-01_rb, 1.78262e-01_rb, &
     & 1.77702e-01_rb, 1.77144e-01_rb, 1.76588e-01_rb, 1.76034e-01_rb, 1.75481e-01_rb, &
     & 1.74931e-01_rb, 1.74382e-01_rb, 1.73834e-01_rb, 1.73289e-01_rb, 1.72745e-01_rb, &
     & 1.72202e-01_rb, 1.71662e-01_rb, 1.71123e-01_rb, 1.70586e-01_rb/)
      kao_mn2o( 1, :,13) = (/ &
     & 2.41966e-01_rb, 2.50534e-01_rb, 2.59406e-01_rb, 2.68591e-01_rb, 2.78102e-01_rb, &
     & 2.87950e-01_rb, 2.98146e-01_rb, 3.08704e-01_rb, 3.19635e-01_rb, 3.30953e-01_rb, &
     & 3.42672e-01_rb, 3.54806e-01_rb, 3.67370e-01_rb, 3.80379e-01_rb, 3.93848e-01_rb, &
     & 4.07794e-01_rb, 4.22234e-01_rb, 4.37186e-01_rb, 4.52667e-01_rb/)
      kao_mn2o( 2, :,13) = (/ &
     & 1.54385e-01_rb, 1.54015e-01_rb, 1.53646e-01_rb, 1.53279e-01_rb, 1.52912e-01_rb, &
     & 1.52545e-01_rb, 1.52180e-01_rb, 1.51816e-01_rb, 1.51452e-01_rb, 1.51089e-01_rb, &
     & 1.50728e-01_rb, 1.50367e-01_rb, 1.50007e-01_rb, 1.49647e-01_rb, 1.49289e-01_rb, &
     & 1.48932e-01_rb, 1.48575e-01_rb, 1.48219e-01_rb, 1.47864e-01_rb/)
      kao_mn2o( 3, :,13) = (/ &
     & 2.00518e-01_rb, 1.94901e-01_rb, 1.89442e-01_rb, 1.84136e-01_rb, 1.78978e-01_rb, &
     & 1.73965e-01_rb, 1.69092e-01_rb, 1.64356e-01_rb, 1.59752e-01_rb, 1.55278e-01_rb, &
     & 1.50928e-01_rb, 1.46701e-01_rb, 1.42592e-01_rb, 1.38598e-01_rb, 1.34715e-01_rb, &
     & 1.30942e-01_rb, 1.27274e-01_rb, 1.23709e-01_rb, 1.20244e-01_rb/)
      kao_mn2o( 4, :,13) = (/ &
     & 2.03974e-01_rb, 1.98258e-01_rb, 1.92703e-01_rb, 1.87302e-01_rb, 1.82054e-01_rb, &
     & 1.76952e-01_rb, 1.71993e-01_rb, 1.67173e-01_rb, 1.62489e-01_rb, 1.57935e-01_rb, &
     & 1.53509e-01_rb, 1.49207e-01_rb, 1.45026e-01_rb, 1.40962e-01_rb, 1.37012e-01_rb, &
     & 1.33172e-01_rb, 1.29440e-01_rb, 1.25813e-01_rb, 1.22287e-01_rb/)
      kao_mn2o( 5, :,13) = (/ &
     & 2.09410e-01_rb, 2.03543e-01_rb, 1.97841e-01_rb, 1.92298e-01_rb, 1.86911e-01_rb, &
     & 1.81674e-01_rb, 1.76585e-01_rb, 1.71637e-01_rb, 1.66829e-01_rb, 1.62155e-01_rb, &
     & 1.57612e-01_rb, 1.53196e-01_rb, 1.48904e-01_rb, 1.44733e-01_rb, 1.40678e-01_rb, &
     & 1.36736e-01_rb, 1.32906e-01_rb, 1.29182e-01_rb, 1.25563e-01_rb/)
      kao_mn2o( 6, :,13) = (/ &
     & 2.19808e-01_rb, 2.13643e-01_rb, 2.07651e-01_rb, 2.01827e-01_rb, 1.96166e-01_rb, &
     & 1.90664e-01_rb, 1.85317e-01_rb, 1.80119e-01_rb, 1.75067e-01_rb, 1.70157e-01_rb, &
     & 1.65385e-01_rb, 1.60746e-01_rb, 1.56238e-01_rb, 1.51856e-01_rb, 1.47596e-01_rb, &
     & 1.43457e-01_rb, 1.39433e-01_rb, 1.35523e-01_rb, 1.31722e-01_rb/)
      kao_mn2o( 7, :,13) = (/ &
     & 2.47074e-01_rb, 2.40127e-01_rb, 2.33375e-01_rb, 2.26813e-01_rb, 2.20436e-01_rb, &
     & 2.14238e-01_rb, 2.08215e-01_rb, 2.02360e-01_rb, 1.96671e-01_rb, 1.91141e-01_rb, &
     & 1.85767e-01_rb, 1.80544e-01_rb, 1.75467e-01_rb, 1.70534e-01_rb, 1.65739e-01_rb, &
     & 1.61079e-01_rb, 1.56550e-01_rb, 1.52148e-01_rb, 1.47870e-01_rb/)
      kao_mn2o( 8, :,13) = (/ &
     & 2.55282e-01_rb, 2.48105e-01_rb, 2.41130e-01_rb, 2.34350e-01_rb, 2.27762e-01_rb, &
     & 2.21358e-01_rb, 2.15135e-01_rb, 2.09086e-01_rb, 2.03208e-01_rb, 1.97495e-01_rb, &
     & 1.91942e-01_rb, 1.86546e-01_rb, 1.81301e-01_rb, 1.76204e-01_rb, 1.71250e-01_rb, &
     & 1.66435e-01_rb, 1.61756e-01_rb, 1.57208e-01_rb, 1.52788e-01_rb/)
      kao_mn2o( 9, :,13) = (/ &
     & 2.09991e-01_rb, 2.04103e-01_rb, 1.98380e-01_rb, 1.92818e-01_rb, 1.87411e-01_rb, &
     & 1.82156e-01_rb, 1.77048e-01_rb, 1.72084e-01_rb, 1.67259e-01_rb, 1.62569e-01_rb, &
     & 1.58010e-01_rb, 1.53580e-01_rb, 1.49273e-01_rb, 1.45087e-01_rb, 1.41019e-01_rb, &
     & 1.37065e-01_rb, 1.33222e-01_rb, 1.29486e-01_rb, 1.25855e-01_rb/)
      kao_mn2o( 1, :,14) = (/ &
     & 9.08340e-02_rb, 9.50421e-02_rb, 9.94452e-02_rb, 1.04052e-01_rb, 1.08873e-01_rb, &
     & 1.13917e-01_rb, 1.19194e-01_rb, 1.24716e-01_rb, 1.30494e-01_rb, 1.36540e-01_rb, &
     & 1.42865e-01_rb, 1.49484e-01_rb, 1.56409e-01_rb, 1.63655e-01_rb, 1.71237e-01_rb, &
     & 1.79170e-01_rb, 1.87471e-01_rb, 1.96156e-01_rb, 2.05243e-01_rb/)
      kao_mn2o( 2, :,14) = (/ &
     & 3.36945e-02_rb, 3.45144e-02_rb, 3.53542e-02_rb, 3.62144e-02_rb, 3.70956e-02_rb, &
     & 3.79982e-02_rb, 3.89228e-02_rb, 3.98698e-02_rb, 4.08399e-02_rb, 4.18336e-02_rb, &
     & 4.28515e-02_rb, 4.38942e-02_rb, 4.49622e-02_rb, 4.60562e-02_rb, 4.71769e-02_rb, &
     & 4.83248e-02_rb, 4.95006e-02_rb, 5.07051e-02_rb, 5.19388e-02_rb/)
      kao_mn2o( 3, :,14) = (/ &
     & 9.11678e-02_rb, 8.85761e-02_rb, 8.60580e-02_rb, 8.36116e-02_rb, 8.12347e-02_rb, &
     & 7.89253e-02_rb, 7.66817e-02_rb, 7.45017e-02_rb, 7.23838e-02_rb, 7.03261e-02_rb, &
     & 6.83269e-02_rb, 6.63845e-02_rb, 6.44973e-02_rb, 6.26638e-02_rb, 6.08824e-02_rb, &
     & 5.91516e-02_rb, 5.74700e-02_rb, 5.58363e-02_rb, 5.42490e-02_rb/)
      kao_mn2o( 4, :,14) = (/ &
     & 8.43999e-02_rb, 8.20004e-02_rb, 7.96692e-02_rb, 7.74042e-02_rb, 7.52037e-02_rb, &
     & 7.30656e-02_rb, 7.09884e-02_rb, 6.89702e-02_rb, 6.70094e-02_rb, 6.51044e-02_rb, &
     & 6.32535e-02_rb, 6.14552e-02_rb, 5.97081e-02_rb, 5.80106e-02_rb, 5.63614e-02_rb, &
     & 5.47590e-02_rb, 5.32022e-02_rb, 5.16897e-02_rb, 5.02202e-02_rb/)
      kao_mn2o( 5, :,14) = (/ &
     & 7.41279e-02_rb, 7.20196e-02_rb, 6.99712e-02_rb, 6.79811e-02_rb, 6.60476e-02_rb, &
     & 6.41691e-02_rb, 6.23440e-02_rb, 6.05708e-02_rb, 5.88481e-02_rb, 5.71743e-02_rb, &
     & 5.55482e-02_rb, 5.39683e-02_rb, 5.24334e-02_rb, 5.09421e-02_rb, 4.94932e-02_rb, &
     & 4.80855e-02_rb, 4.67179e-02_rb, 4.53891e-02_rb, 4.40982e-02_rb/)
      kao_mn2o( 6, :,14) = (/ &
     & 5.66805e-02_rb, 5.50676e-02_rb, 5.35006e-02_rb, 5.19782e-02_rb, 5.04991e-02_rb, &
     & 4.90621e-02_rb, 4.76659e-02_rb, 4.63096e-02_rb, 4.49918e-02_rb, 4.37115e-02_rb, &
     & 4.24676e-02_rb, 4.12592e-02_rb, 4.00851e-02_rb, 3.89444e-02_rb, 3.78362e-02_rb, &
     & 3.67595e-02_rb, 3.57135e-02_rb, 3.46972e-02_rb, 3.37099e-02_rb/)
      kao_mn2o( 7, :,14) = (/ &
     & 1.23018e-02_rb, 1.19517e-02_rb, 1.16116e-02_rb, 1.12811e-02_rb, 1.09601e-02_rb, &
     & 1.06482e-02_rb, 1.03452e-02_rb, 1.00508e-02_rb, 9.76474e-03_rb, 9.48685e-03_rb, &
     & 9.21687e-03_rb, 8.95458e-03_rb, 8.69974e-03_rb, 8.45216e-03_rb, 8.21163e-03_rb, &
     & 7.97794e-03_rb, 7.75091e-03_rb, 7.53033e-03_rb, 7.31603e-03_rb/)
      kao_mn2o( 8, :,14) = (/ &
     & 3.22403e-07_rb, 3.75986e-07_rb, 4.38475e-07_rb, 5.11349e-07_rb, 5.96335e-07_rb, &
     & 6.95446e-07_rb, 8.11028e-07_rb, 9.45821e-07_rb, 1.10302e-06_rb, 1.28634e-06_rb, &
     & 1.50012e-06_rb, 1.74944e-06_rb, 2.04020e-06_rb, 2.37928e-06_rb, 2.77472e-06_rb, &
     & 3.23587e-06_rb, 3.77367e-06_rb, 4.40085e-06_rb, 5.13227e-06_rb/)
      kao_mn2o( 9, :,14) = (/ &
     & 7.33052e-02_rb, 7.12199e-02_rb, 6.91939e-02_rb, 6.72255e-02_rb, 6.53131e-02_rb, &
     & 6.34551e-02_rb, 6.16500e-02_rb, 5.98963e-02_rb, 5.81924e-02_rb, 5.65370e-02_rb, &
     & 5.49287e-02_rb, 5.33661e-02_rb, 5.18480e-02_rb, 5.03730e-02_rb, 4.89401e-02_rb, &
     & 4.75479e-02_rb, 4.61953e-02_rb, 4.48811e-02_rb, 4.36044e-02_rb/)
      kao_mn2o( 1, :,15) = (/ &
     & 8.80247e-02_rb, 9.01793e-02_rb, 9.23868e-02_rb, 9.46482e-02_rb, 9.69650e-02_rb, &
     & 9.93385e-02_rb, 1.01770e-01_rb, 1.04261e-01_rb, 1.06813e-01_rb, 1.09428e-01_rb, &
     & 1.12107e-01_rb, 1.14851e-01_rb, 1.17662e-01_rb, 1.20542e-01_rb, 1.23493e-01_rb, &
     & 1.26516e-01_rb, 1.29613e-01_rb, 1.32785e-01_rb, 1.36036e-01_rb/)
      kao_mn2o( 2, :,15) = (/ &
     & 3.89107e-07_rb, 4.53768e-07_rb, 5.29173e-07_rb, 6.17109e-07_rb, 7.19658e-07_rb, &
     & 8.39248e-07_rb, 9.78710e-07_rb, 1.14135e-06_rb, 1.33101e-06_rb, 1.55220e-06_rb, &
     & 1.81013e-06_rb, 2.11094e-06_rb, 2.46172e-06_rb, 2.87080e-06_rb, 3.34786e-06_rb, &
     & 3.90420e-06_rb, 4.55298e-06_rb, 5.30958e-06_rb, 6.19190e-06_rb/)
      kao_mn2o( 3, :,15) = (/ &
     & 3.86537e-07_rb, 4.50763e-07_rb, 5.25662e-07_rb, 6.13006e-07_rb, 7.14863e-07_rb, &
     & 8.33644e-07_rb, 9.72162e-07_rb, 1.13370e-06_rb, 1.32207e-06_rb, 1.54175e-06_rb, &
     & 1.79792e-06_rb, 2.09666e-06_rb, 2.44504e-06_rb, 2.85131e-06_rb, 3.32508e-06_rb, &
     & 3.87758e-06_rb, 4.52188e-06_rb, 5.27323e-06_rb, 6.14943e-06_rb/)
      kao_mn2o( 4, :,15) = (/ &
     & 3.81913e-07_rb, 4.45369e-07_rb, 5.19369e-07_rb, 6.05664e-07_rb, 7.06297e-07_rb, &
     & 8.23651e-07_rb, 9.60503e-07_rb, 1.12009e-06_rb, 1.30620e-06_rb, 1.52323e-06_rb, &
     & 1.77632e-06_rb, 2.07147e-06_rb, 2.41565e-06_rb, 2.81701e-06_rb, 3.28507e-06_rb, &
     & 3.83090e-06_rb, 4.46741e-06_rb, 5.20969e-06_rb, 6.07529e-06_rb/)
      kao_mn2o( 5, :,15) = (/ &
     & 3.77265e-07_rb, 4.39951e-07_rb, 5.13053e-07_rb, 5.98303e-07_rb, 6.97717e-07_rb, &
     & 8.13650e-07_rb, 9.48846e-07_rb, 1.10651e-06_rb, 1.29036e-06_rb, 1.50477e-06_rb, &
     & 1.75480e-06_rb, 2.04638e-06_rb, 2.38641e-06_rb, 2.78294e-06_rb, 3.24535e-06_rb, &
     & 3.78460e-06_rb, 4.41345e-06_rb, 5.14679e-06_rb, 6.00198e-06_rb/)
      kao_mn2o( 6, :,15) = (/ &
     & 3.77877e-07_rb, 4.40670e-07_rb, 5.13897e-07_rb, 5.99292e-07_rb, 6.98878e-07_rb, &
     & 8.15012e-07_rb, 9.50444e-07_rb, 1.10838e-06_rb, 1.29256e-06_rb, 1.50735e-06_rb, &
     & 1.75783e-06_rb, 2.04993e-06_rb, 2.39057e-06_rb, 2.78782e-06_rb, 3.25107e-06_rb, &
     & 3.79131e-06_rb, 4.42132e-06_rb, 5.15602e-06_rb, 6.01280e-06_rb/)
      kao_mn2o( 7, :,15) = (/ &
     & 3.80495e-07_rb, 4.43726e-07_rb, 5.17465e-07_rb, 6.03458e-07_rb, 7.03741e-07_rb, &
     & 8.20689e-07_rb, 9.57072e-07_rb, 1.11612e-06_rb, 1.30160e-06_rb, 1.51790e-06_rb, &
     & 1.77014e-06_rb, 2.06430e-06_rb, 2.40735e-06_rb, 2.80741e-06_rb, 3.27394e-06_rb, &
     & 3.81801e-06_rb, 4.45249e-06_rb, 5.19241e-06_rb, 6.05528e-06_rb/)
      kao_mn2o( 8, :,15) = (/ &
     & 3.87881e-07_rb, 4.52329e-07_rb, 5.27486e-07_rb, 6.15129e-07_rb, 7.17335e-07_rb, &
     & 8.36523e-07_rb, 9.75515e-07_rb, 1.13760e-06_rb, 1.32662e-06_rb, 1.54704e-06_rb, &
     & 1.80409e-06_rb, 2.10384e-06_rb, 2.45340e-06_rb, 2.86105e-06_rb, 3.33642e-06_rb, &
     & 3.89078e-06_rb, 4.53725e-06_rb, 5.29112e-06_rb, 6.17026e-06_rb/)
      kao_mn2o( 9, :,15) = (/ &
     & 3.77265e-07_rb, 4.39951e-07_rb, 5.13053e-07_rb, 5.98303e-07_rb, 6.97717e-07_rb, &
     & 8.13650e-07_rb, 9.48846e-07_rb, 1.10651e-06_rb, 1.29036e-06_rb, 1.50477e-06_rb, &
     & 1.75480e-06_rb, 2.04638e-06_rb, 2.38641e-06_rb, 2.78294e-06_rb, 3.24535e-06_rb, &
     & 3.78460e-06_rb, 4.41345e-06_rb, 5.14679e-06_rb, 6.00198e-06_rb/)
      kao_mn2o( 1, :,16) = (/ &
     & 8.84606e-02_rb, 9.05971e-02_rb, 9.27852e-02_rb, 9.50261e-02_rb, 9.73212e-02_rb, &
     & 9.96717e-02_rb, 1.02079e-01_rb, 1.04544e-01_rb, 1.07069e-01_rb, 1.09655e-01_rb, &
     & 1.12304e-01_rb, 1.15016e-01_rb, 1.17794e-01_rb, 1.20639e-01_rb, 1.23553e-01_rb, &
     & 1.26537e-01_rb, 1.29593e-01_rb, 1.32723e-01_rb, 1.35928e-01_rb/)
      kao_mn2o( 2, :,16) = (/ &
     & 8.13898e-07_rb, 9.49130e-07_rb, 1.10683e-06_rb, 1.29073e-06_rb, 1.50519e-06_rb, &
     & 1.75528e-06_rb, 2.04693e-06_rb, 2.38703e-06_rb, 2.78364e-06_rb, 3.24615e-06_rb, &
     & 3.78551e-06_rb, 4.41449e-06_rb, 5.14796e-06_rb, 6.00331e-06_rb, 7.00078e-06_rb, &
     & 8.16398e-06_rb, 9.52045e-06_rb, 1.11023e-05_rb, 1.29470e-05_rb/)
      kao_mn2o( 3, :,16) = (/ &
     & 8.32666e-07_rb, 9.71021e-07_rb, 1.13237e-06_rb, 1.32052e-06_rb, 1.53994e-06_rb, &
     & 1.79581e-06_rb, 2.09420e-06_rb, 2.44217e-06_rb, 2.84796e-06_rb, 3.32117e-06_rb, &
     & 3.87302e-06_rb, 4.51656e-06_rb, 5.26703e-06_rb, 6.14219e-06_rb, 7.16277e-06_rb, &
     & 8.35294e-06_rb, 9.74086e-06_rb, 1.13594e-05_rb, 1.32468e-05_rb/)
      kao_mn2o( 4, :,16) = (/ &
     & 8.70348e-07_rb, 1.01496e-06_rb, 1.18360e-06_rb, 1.38026e-06_rb, 1.60959e-06_rb, &
     & 1.87703e-06_rb, 2.18890e-06_rb, 2.55259e-06_rb, 2.97671e-06_rb, 3.47130e-06_rb, &
     & 4.04807e-06_rb, 4.72067e-06_rb, 5.50502e-06_rb, 6.41970e-06_rb, 7.48635e-06_rb, &
     & 8.73023e-06_rb, 1.01808e-05_rb, 1.18724e-05_rb, 1.38450e-05_rb/)
      kao_mn2o( 5, :,16) = (/ &
     & 9.07957e-07_rb, 1.05882e-06_rb, 1.23475e-06_rb, 1.43991e-06_rb, 1.67916e-06_rb, &
     & 1.95816e-06_rb, 2.28352e-06_rb, 2.66294e-06_rb, 3.10541e-06_rb, 3.62139e-06_rb, &
     & 4.22310e-06_rb, 4.92480e-06_rb, 5.74308e-06_rb, 6.69733e-06_rb, 7.81013e-06_rb, &
     & 9.10784e-06_rb, 1.06212e-05_rb, 1.23859e-05_rb, 1.44439e-05_rb/)
      kao_mn2o( 6, :,16) = (/ &
     & 8.59072e-04_rb, 9.19773e-04_rb, 9.84764e-04_rb, 1.05435e-03_rb, 1.12885e-03_rb, &
     & 1.20861e-03_rb, 1.29401e-03_rb, 1.38544e-03_rb, 1.48334e-03_rb, 1.58815e-03_rb, &
     & 1.70037e-03_rb, 1.82052e-03_rb, 1.94915e-03_rb, 2.08688e-03_rb, 2.23434e-03_rb, &
     & 2.39222e-03_rb, 2.56125e-03_rb, 2.74223e-03_rb, 2.93599e-03_rb/)
      kao_mn2o( 7, :,16) = (/ &
     & 9.17294e-07_rb, 1.06971e-06_rb, 1.24746e-06_rb, 1.45474e-06_rb, 1.69646e-06_rb, &
     & 1.97835e-06_rb, 2.30708e-06_rb, 2.69043e-06_rb, 3.13748e-06_rb, 3.65880e-06_rb, &
     & 4.26676e-06_rb, 4.97574e-06_rb, 5.80251e-06_rb, 6.76667e-06_rb, 7.89104e-06_rb, &
     & 9.20223e-06_rb, 1.07313e-05_rb, 1.25144e-05_rb, 1.45939e-05_rb/)
      kao_mn2o( 8, :,16) = (/ &
     & 9.17341e-07_rb, 1.06977e-06_rb, 1.24752e-06_rb, 1.45480e-06_rb, 1.69653e-06_rb, &
     & 1.97843e-06_rb, 2.30716e-06_rb, 2.69051e-06_rb, 3.13757e-06_rb, 3.65890e-06_rb, &
     & 4.26686e-06_rb, 4.97584e-06_rb, 5.80261e-06_rb, 6.76677e-06_rb, 7.89113e-06_rb, &
     & 9.20231e-06_rb, 1.07314e-05_rb, 1.25145e-05_rb, 1.45939e-05_rb/)
      kao_mn2o( 9, :,16) = (/ &
     & 9.07957e-07_rb, 1.05882e-06_rb, 1.23475e-06_rb, 1.43991e-06_rb, 1.67916e-06_rb, &
     & 1.95816e-06_rb, 2.28352e-06_rb, 2.66294e-06_rb, 3.10541e-06_rb, 3.62139e-06_rb, &
     & 4.22310e-06_rb, 4.92480e-06_rb, 5.74308e-06_rb, 6.69733e-06_rb, 7.81013e-06_rb, &
     & 9.10784e-06_rb, 1.06212e-05_rb, 1.23859e-05_rb, 1.44439e-05_rb/)

!     The array KBO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level above 100~ mb.   The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amounts ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 to 
!     that of gas2.  The second index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The third index 
!     runs over the g-channel (1 to 16).

      kbo_mn2o( 1, :, 1) = (/ &
     & 7.72009e-08_rb, 1.15883e-07_rb, 1.73947e-07_rb, 2.61104e-07_rb, 3.91932e-07_rb, &
     & 5.88311e-07_rb, 8.83088e-07_rb, 1.32556e-06_rb, 1.98975e-06_rb, 2.98672e-06_rb, &
     & 4.48324e-06_rb, 6.72960e-06_rb, 1.01015e-05_rb, 1.51629e-05_rb, 2.27604e-05_rb, &
     & 3.41646e-05_rb, 5.12831e-05_rb, 7.69787e-05_rb, 1.15549e-04_rb/)
      kbo_mn2o( 2, :, 1) = (/ &
     & 1.29932e-05_rb, 1.78207e-05_rb, 2.44419e-05_rb, 3.35232e-05_rb, 4.59786e-05_rb, &
     & 6.30617e-05_rb, 8.64920e-05_rb, 1.18628e-04_rb, 1.62703e-04_rb, 2.23155e-04_rb, &
     & 3.06067e-04_rb, 4.19784e-04_rb, 5.75753e-04_rb, 7.89671e-04_rb, 1.08307e-03_rb, &
     & 1.48548e-03_rb, 2.03740e-03_rb, 2.79439e-03_rb, 3.83262e-03_rb/)
      kbo_mn2o( 3, :, 1) = (/ &
     & 6.44518e-05_rb, 8.10996e-05_rb, 1.02047e-04_rb, 1.28406e-04_rb, 1.61573e-04_rb, &
     & 2.03307e-04_rb, 2.55821e-04_rb, 3.21899e-04_rb, 4.05045e-04_rb, 5.09667e-04_rb, &
     & 6.41313e-04_rb, 8.06964e-04_rb, 1.01540e-03_rb, 1.27768e-03_rb, 1.60770e-03_rb, &
     & 2.02296e-03_rb, 2.54549e-03_rb, 3.20298e-03_rb, 4.03031e-03_rb/)
      kbo_mn2o( 4, :, 1) = (/ &
     & 3.23454e-04_rb, 3.82112e-04_rb, 4.51408e-04_rb, 5.33270e-04_rb, 6.29978e-04_rb, &
     & 7.44223e-04_rb, 8.79187e-04_rb, 1.03863e-03_rb, 1.22698e-03_rb, 1.44949e-03_rb, &
     & 1.71235e-03_rb, 2.02289e-03_rb, 2.38974e-03_rb, 2.82311e-03_rb, 3.33508e-03_rb, &
     & 3.93989e-03_rb, 4.65439e-03_rb, 5.49845e-03_rb, 6.49558e-03_rb/)
      kbo_mn2o( 5, :, 1) = (/ &
     & 1.45978e-04_rb, 1.75646e-04_rb, 2.11344e-04_rb, 2.54296e-04_rb, 3.05978e-04_rb, &
     & 3.68163e-04_rb, 4.42986e-04_rb, 5.33017e-04_rb, 6.41344e-04_rb, 7.71687e-04_rb, &
     & 9.28522e-04_rb, 1.11723e-03_rb, 1.34429e-03_rb, 1.61750e-03_rb, 1.94623e-03_rb, &
     & 2.34177e-03_rb, 2.81770e-03_rb, 3.39035e-03_rb, 4.07939e-03_rb/)
      kbo_mn2o( 1, :, 2) = (/ &
     & 1.94527e-04_rb, 2.38609e-04_rb, 2.92680e-04_rb, 3.59005e-04_rb, 4.40360e-04_rb, &
     & 5.40150e-04_rb, 6.62554e-04_rb, 8.12697e-04_rb, 9.96864e-04_rb, 1.22276e-03_rb, &
     & 1.49986e-03_rb, 1.83974e-03_rb, 2.25665e-03_rb, 2.76803e-03_rb, 3.39530e-03_rb, &
     & 4.16472e-03_rb, 5.10849e-03_rb, 6.26613e-03_rb, 7.68611e-03_rb/)
      kbo_mn2o( 2, :, 2) = (/ &
     & 7.49615e-04_rb, 8.82716e-04_rb, 1.03945e-03_rb, 1.22401e-03_rb, 1.44135e-03_rb, &
     & 1.69727e-03_rb, 1.99863e-03_rb, 2.35351e-03_rb, 2.77139e-03_rb, 3.26347e-03_rb, &
     & 3.84293e-03_rb, 4.52528e-03_rb, 5.32878e-03_rb, 6.27495e-03_rb, 7.38911e-03_rb, &
     & 8.70111e-03_rb, 1.02461e-02_rb, 1.20653e-02_rb, 1.42076e-02_rb/)
      kbo_mn2o( 3, :, 2) = (/ &
     & 1.39162e-03_rb, 1.59859e-03_rb, 1.83634e-03_rb, 2.10946e-03_rb, 2.42319e-03_rb, &
     & 2.78358e-03_rb, 3.19758e-03_rb, 3.67314e-03_rb, 4.21944e-03_rb, 4.84698e-03_rb, &
     & 5.56786e-03_rb, 6.39595e-03_rb, 7.34720e-03_rb, 8.43992e-03_rb, 9.69516e-03_rb, &
     & 1.11371e-02_rb, 1.27935e-02_rb, 1.46962e-02_rb, 1.68819e-02_rb/)
      kbo_mn2o( 4, :, 2) = (/ &
     & 2.42354e-03_rb, 2.73623e-03_rb, 3.08926e-03_rb, 3.48783e-03_rb, 3.93783e-03_rb, &
     & 4.44589e-03_rb, 5.01950e-03_rb, 5.66712e-03_rb, 6.39829e-03_rb, 7.22379e-03_rb, &
     & 8.15581e-03_rb, 9.20807e-03_rb, 1.03961e-02_rb, 1.17374e-02_rb, 1.32517e-02_rb, &
     & 1.49615e-02_rb, 1.68918e-02_rb, 1.90712e-02_rb, 2.15318e-02_rb/)
      kbo_mn2o( 5, :, 2) = (/ &
     & 1.39908e-03_rb, 1.59133e-03_rb, 1.81000e-03_rb, 2.05872e-03_rb, 2.34161e-03_rb, &
     & 2.66338e-03_rb, 3.02937e-03_rb, 3.44564e-03_rb, 3.91912e-03_rb, 4.45766e-03_rb, &
     & 5.07021e-03_rb, 5.76693e-03_rb, 6.55938e-03_rb, 7.46073e-03_rb, 8.48594e-03_rb, &
     & 9.65202e-03_rb, 1.09783e-02_rb, 1.24869e-02_rb, 1.42028e-02_rb/)
      kbo_mn2o( 1, :, 3) = (/ &
     & 8.74797e-03_rb, 9.66828e-03_rb, 1.06854e-02_rb, 1.18095e-02_rb, 1.30519e-02_rb, &
     & 1.44250e-02_rb, 1.59426e-02_rb, 1.76198e-02_rb, 1.94735e-02_rb, 2.15221e-02_rb, &
     & 2.37863e-02_rb, 2.62887e-02_rb, 2.90544e-02_rb, 3.21110e-02_rb, 3.54891e-02_rb, &
     & 3.92227e-02_rb, 4.33491e-02_rb, 4.79095e-02_rb, 5.29497e-02_rb/)
      kbo_mn2o( 2, :, 3) = (/ &
     & 1.43974e-02_rb, 1.56118e-02_rb, 1.69286e-02_rb, 1.83564e-02_rb, 1.99047e-02_rb, &
     & 2.15836e-02_rb, 2.34041e-02_rb, 2.53781e-02_rb, 2.75187e-02_rb, 2.98397e-02_rb, &
     & 3.23566e-02_rb, 3.50857e-02_rb, 3.80451e-02_rb, 4.12540e-02_rb, 4.47336e-02_rb, &
     & 4.85067e-02_rb, 5.25980e-02_rb, 5.70344e-02_rb, 6.18450e-02_rb/)
      kbo_mn2o( 3, :, 3) = (/ &
     & 1.83051e-02_rb, 1.96851e-02_rb, 2.11692e-02_rb, 2.27651e-02_rb, 2.44813e-02_rb, &
     & 2.63270e-02_rb, 2.83117e-02_rb, 3.04461e-02_rb, 3.27414e-02_rb, 3.52097e-02_rb, &
     & 3.78642e-02_rb, 4.07187e-02_rb, 4.37884e-02_rb, 4.70896e-02_rb, 5.06396e-02_rb, &
     & 5.44573e-02_rb, 5.85628e-02_rb, 6.29777e-02_rb, 6.77256e-02_rb/)
      kbo_mn2o( 4, :, 3) = (/ &
     & 2.81258e-02_rb, 2.97806e-02_rb, 3.15328e-02_rb, 3.33880e-02_rb, 3.53524e-02_rb, &
     & 3.74324e-02_rb, 3.96348e-02_rb, 4.19667e-02_rb, 4.44358e-02_rb, 4.70502e-02_rb, &
     & 4.98185e-02_rb, 5.27496e-02_rb, 5.58531e-02_rb, 5.91393e-02_rb, 6.26187e-02_rb, &
     & 6.63030e-02_rb, 7.02039e-02_rb, 7.43344e-02_rb, 7.87079e-02_rb/)
      kbo_mn2o( 5, :, 3) = (/ &
     & 8.82958e-03_rb, 9.53842e-03_rb, 1.03042e-02_rb, 1.11314e-02_rb, 1.20250e-02_rb, &
     & 1.29904e-02_rb, 1.40333e-02_rb, 1.51599e-02_rb, 1.63769e-02_rb, 1.76917e-02_rb, &
     & 1.91120e-02_rb, 2.06463e-02_rb, 2.23038e-02_rb, 2.40944e-02_rb, 2.60287e-02_rb, &
     & 2.81183e-02_rb, 3.03757e-02_rb, 3.28142e-02_rb, 3.54486e-02_rb/)
      kbo_mn2o( 1, :, 4) = (/ &
     & 1.18673e-01_rb, 1.22983e-01_rb, 1.27449e-01_rb, 1.32077e-01_rb, 1.36874e-01_rb, &
     & 1.41845e-01_rb, 1.46996e-01_rb, 1.52334e-01_rb, 1.57866e-01_rb, 1.63599e-01_rb, &
     & 1.69541e-01_rb, 1.75698e-01_rb, 1.82078e-01_rb, 1.88691e-01_rb, 1.95543e-01_rb, &
     & 2.02645e-01_rb, 2.10004e-01_rb, 2.17631e-01_rb, 2.25534e-01_rb/)
      kbo_mn2o( 2, :, 4) = (/ &
     & 1.32161e-01_rb, 1.36550e-01_rb, 1.41084e-01_rb, 1.45769e-01_rb, 1.50610e-01_rb, &
     & 1.55611e-01_rb, 1.60779e-01_rb, 1.66118e-01_rb, 1.71634e-01_rb, 1.77334e-01_rb, &
     & 1.83223e-01_rb, 1.89307e-01_rb, 1.95594e-01_rb, 2.02089e-01_rb, 2.08800e-01_rb, &
     & 2.15734e-01_rb, 2.22898e-01_rb, 2.30300e-01_rb, 2.37947e-01_rb/)
      kbo_mn2o( 3, :, 4) = (/ &
     & 1.44384e-01_rb, 1.48820e-01_rb, 1.53392e-01_rb, 1.58104e-01_rb, 1.62962e-01_rb, &
     & 1.67968e-01_rb, 1.73128e-01_rb, 1.78447e-01_rb, 1.83929e-01_rb, 1.89580e-01_rb, &
     & 1.95404e-01_rb, 2.01407e-01_rb, 2.07594e-01_rb, 2.13972e-01_rb, 2.20546e-01_rb, &
     & 2.27321e-01_rb, 2.34305e-01_rb, 2.41503e-01_rb, 2.48922e-01_rb/)
      kbo_mn2o( 4, :, 4) = (/ &
     & 1.58026e-01_rb, 1.62626e-01_rb, 1.67360e-01_rb, 1.72232e-01_rb, 1.77245e-01_rb, &
     & 1.82405e-01_rb, 1.87714e-01_rb, 1.93179e-01_rb, 1.98802e-01_rb, 2.04589e-01_rb, &
     & 2.10544e-01_rb, 2.16673e-01_rb, 2.22980e-01_rb, 2.29471e-01_rb, 2.36151e-01_rb, &
     & 2.43025e-01_rb, 2.50100e-01_rb, 2.57380e-01_rb, 2.64872e-01_rb/)
      kbo_mn2o( 5, :, 4) = (/ &
     & 4.04885e-02_rb, 4.16064e-02_rb, 4.27551e-02_rb, 4.39355e-02_rb, 4.51485e-02_rb, &
     & 4.63950e-02_rb, 4.76759e-02_rb, 4.89921e-02_rb, 5.03448e-02_rb, 5.17347e-02_rb, &
     & 5.31630e-02_rb, 5.46308e-02_rb, 5.61391e-02_rb, 5.76890e-02_rb, 5.92817e-02_rb, &
     & 6.09184e-02_rb, 6.26003e-02_rb, 6.43286e-02_rb, 6.61047e-02_rb/)
      kbo_mn2o( 1, :, 5) = (/ &
     & 3.97757e-01_rb, 4.01082e-01_rb, 4.04434e-01_rb, 4.07814e-01_rb, 4.11223e-01_rb, &
     & 4.14659e-01_rb, 4.18125e-01_rb, 4.21620e-01_rb, 4.25144e-01_rb, 4.28697e-01_rb, &
     & 4.32280e-01_rb, 4.35893e-01_rb, 4.39536e-01_rb, 4.43209e-01_rb, 4.46913e-01_rb, &
     & 4.50649e-01_rb, 4.54415e-01_rb, 4.58213e-01_rb, 4.62043e-01_rb/)
      kbo_mn2o( 2, :, 5) = (/ &
     & 3.99425e-01_rb, 4.02035e-01_rb, 4.04662e-01_rb, 4.07306e-01_rb, 4.09968e-01_rb, &
     & 4.12647e-01_rb, 4.15343e-01_rb, 4.18057e-01_rb, 4.20789e-01_rb, 4.23539e-01_rb, &
     & 4.26307e-01_rb, 4.29092e-01_rb, 4.31896e-01_rb, 4.34719e-01_rb, 4.37559e-01_rb, &
     & 4.40419e-01_rb, 4.43296e-01_rb, 4.46193e-01_rb, 4.49109e-01_rb/)
      kbo_mn2o( 3, :, 5) = (/ &
     & 4.00527e-01_rb, 4.02848e-01_rb, 4.05182e-01_rb, 4.07530e-01_rb, 4.09892e-01_rb, &
     & 4.12267e-01_rb, 4.14656e-01_rb, 4.17058e-01_rb, 4.19475e-01_rb, 4.21906e-01_rb, &
     & 4.24351e-01_rb, 4.26809e-01_rb, 4.29283e-01_rb, 4.31770e-01_rb, 4.34272e-01_rb, &
     & 4.36788e-01_rb, 4.39319e-01_rb, 4.41865e-01_rb, 4.44426e-01_rb/)
      kbo_mn2o( 4, :, 5) = (/ &
     & 4.11455e-01_rb, 4.13077e-01_rb, 4.14705e-01_rb, 4.16340e-01_rb, 4.17981e-01_rb, &
     & 4.19629e-01_rb, 4.21283e-01_rb, 4.22944e-01_rb, 4.24611e-01_rb, 4.26285e-01_rb, &
     & 4.27966e-01_rb, 4.29653e-01_rb, 4.31346e-01_rb, 4.33047e-01_rb, 4.34754e-01_rb, &
     & 4.36468e-01_rb, 4.38188e-01_rb, 4.39916e-01_rb, 4.41650e-01_rb/)
      kbo_mn2o( 5, :, 5) = (/ &
     & 8.65576e-02_rb, 8.83622e-02_rb, 9.02044e-02_rb, 9.20850e-02_rb, 9.40049e-02_rb, &
     & 9.59647e-02_rb, 9.79655e-02_rb, 1.00008e-01_rb, 1.02093e-01_rb, 1.04221e-01_rb, &
     & 1.06394e-01_rb, 1.08612e-01_rb, 1.10877e-01_rb, 1.13188e-01_rb, 1.15548e-01_rb, &
     & 1.17957e-01_rb, 1.20417e-01_rb, 1.22927e-01_rb, 1.25490e-01_rb/)
      kbo_mn2o( 1, :, 6) = (/ &
     & 6.98675e-01_rb, 7.00999e-01_rb, 7.03331e-01_rb, 7.05671e-01_rb, 7.08019e-01_rb, &
     & 7.10375e-01_rb, 7.12738e-01_rb, 7.15110e-01_rb, 7.17489e-01_rb, 7.19876e-01_rb, &
     & 7.22271e-01_rb, 7.24674e-01_rb, 7.27085e-01_rb, 7.29504e-01_rb, 7.31931e-01_rb, &
     & 7.34366e-01_rb, 7.36809e-01_rb, 7.39261e-01_rb, 7.41720e-01_rb/)
      kbo_mn2o( 2, :, 6) = (/ &
     & 6.98858e-01_rb, 7.01424e-01_rb, 7.03999e-01_rb, 7.06583e-01_rb, 7.09177e-01_rb, &
     & 7.11780e-01_rb, 7.14393e-01_rb, 7.17016e-01_rb, 7.19648e-01_rb, 7.22289e-01_rb, &
     & 7.24941e-01_rb, 7.27602e-01_rb, 7.30273e-01_rb, 7.32954e-01_rb, 7.35644e-01_rb, &
     & 7.38345e-01_rb, 7.41055e-01_rb, 7.43775e-01_rb, 7.46506e-01_rb/)
      kbo_mn2o( 3, :, 6) = (/ &
     & 7.08151e-01_rb, 7.10727e-01_rb, 7.13311e-01_rb, 7.15905e-01_rb, 7.18508e-01_rb, &
     & 7.21121e-01_rb, 7.23743e-01_rb, 7.26375e-01_rb, 7.29017e-01_rb, 7.31668e-01_rb, &
     & 7.34329e-01_rb, 7.36999e-01_rb, 7.39679e-01_rb, 7.42369e-01_rb, 7.45068e-01_rb, &
     & 7.47778e-01_rb, 7.50497e-01_rb, 7.53226e-01_rb, 7.55965e-01_rb/)
      kbo_mn2o( 4, :, 6) = (/ &
     & 7.22269e-01_rb, 7.24981e-01_rb, 7.27704e-01_rb, 7.30437e-01_rb, 7.33180e-01_rb, &
     & 7.35933e-01_rb, 7.38697e-01_rb, 7.41471e-01_rb, 7.44256e-01_rb, 7.47051e-01_rb, &
     & 7.49856e-01_rb, 7.52672e-01_rb, 7.55499e-01_rb, 7.58336e-01_rb, 7.61184e-01_rb, &
     & 7.64043e-01_rb, 7.66912e-01_rb, 7.69792e-01_rb, 7.72683e-01_rb/)
      kbo_mn2o( 5, :, 6) = (/ &
     & 1.75877e-01_rb, 1.78578e-01_rb, 1.81321e-01_rb, 1.84107e-01_rb, 1.86935e-01_rb, &
     & 1.89806e-01_rb, 1.92722e-01_rb, 1.95682e-01_rb, 1.98688e-01_rb, 2.01740e-01_rb, &
     & 2.04839e-01_rb, 2.07986e-01_rb, 2.11181e-01_rb, 2.14425e-01_rb, 2.17719e-01_rb, &
     & 2.21063e-01_rb, 2.24459e-01_rb, 2.27907e-01_rb, 2.31408e-01_rb/)
      kbo_mn2o( 1, :, 7) = (/ &
     & 1.82985e+00_rb, 1.83684e+00_rb, 1.84386e+00_rb, 1.85091e+00_rb, 1.85798e+00_rb, &
     & 1.86508e+00_rb, 1.87221e+00_rb, 1.87937e+00_rb, 1.88655e+00_rb, 1.89376e+00_rb, &
     & 1.90100e+00_rb, 1.90827e+00_rb, 1.91556e+00_rb, 1.92288e+00_rb, 1.93023e+00_rb, &
     & 1.93761e+00_rb, 1.94502e+00_rb, 1.95245e+00_rb, 1.95991e+00_rb/)
      kbo_mn2o( 2, :, 7) = (/ &
     & 1.83229e+00_rb, 1.83943e+00_rb, 1.84659e+00_rb, 1.85379e+00_rb, 1.86100e+00_rb, &
     & 1.86825e+00_rb, 1.87553e+00_rb, 1.88283e+00_rb, 1.89016e+00_rb, 1.89753e+00_rb, &
     & 1.90492e+00_rb, 1.91233e+00_rb, 1.91978e+00_rb, 1.92726e+00_rb, 1.93476e+00_rb, &
     & 1.94230e+00_rb, 1.94986e+00_rb, 1.95746e+00_rb, 1.96508e+00_rb/)
      kbo_mn2o( 3, :, 7) = (/ &
     & 1.84946e+00_rb, 1.85707e+00_rb, 1.86471e+00_rb, 1.87238e+00_rb, 1.88008e+00_rb, &
     & 1.88781e+00_rb, 1.89558e+00_rb, 1.90338e+00_rb, 1.91120e+00_rb, 1.91907e+00_rb, &
     & 1.92696e+00_rb, 1.93489e+00_rb, 1.94285e+00_rb, 1.95084e+00_rb, 1.95886e+00_rb, &
     & 1.96692e+00_rb, 1.97501e+00_rb, 1.98313e+00_rb, 1.99129e+00_rb/)
      kbo_mn2o( 4, :, 7) = (/ &
     & 1.88354e+00_rb, 1.89167e+00_rb, 1.89983e+00_rb, 1.90803e+00_rb, 1.91626e+00_rb, &
     & 1.92453e+00_rb, 1.93283e+00_rb, 1.94117e+00_rb, 1.94955e+00_rb, 1.95796e+00_rb, &
     & 1.96641e+00_rb, 1.97489e+00_rb, 1.98341e+00_rb, 1.99197e+00_rb, 2.00056e+00_rb, &
     & 2.00920e+00_rb, 2.01787e+00_rb, 2.02657e+00_rb, 2.03532e+00_rb/)
      kbo_mn2o( 5, :, 7) = (/ &
     & 3.35154e-01_rb, 3.43258e-01_rb, 3.51557e-01_rb, 3.60058e-01_rb, 3.68764e-01_rb, &
     & 3.77680e-01_rb, 3.86812e-01_rb, 3.96164e-01_rb, 4.05743e-01_rb, 4.15553e-01_rb, &
     & 4.25601e-01_rb, 4.35892e-01_rb, 4.46431e-01_rb, 4.57225e-01_rb, 4.68280e-01_rb, &
     & 4.79603e-01_rb, 4.91199e-01_rb, 5.03075e-01_rb, 5.15239e-01_rb/)
      kbo_mn2o( 1, :, 8) = (/ &
     & 4.46843e+00_rb, 4.49793e+00_rb, 4.52763e+00_rb, 4.55752e+00_rb, 4.58761e+00_rb, &
     & 4.61790e+00_rb, 4.64839e+00_rb, 4.67908e+00_rb, 4.70997e+00_rb, 4.74106e+00_rb, &
     & 4.77236e+00_rb, 4.80387e+00_rb, 4.83559e+00_rb, 4.86751e+00_rb, 4.89965e+00_rb, &
     & 4.93200e+00_rb, 4.96456e+00_rb, 4.99733e+00_rb, 5.03033e+00_rb/)
      kbo_mn2o( 2, :, 8) = (/ &
     & 4.44347e+00_rb, 4.47278e+00_rb, 4.50228e+00_rb, 4.53198e+00_rb, 4.56188e+00_rb, &
     & 4.59197e+00_rb, 4.62227e+00_rb, 4.65276e+00_rb, 4.68345e+00_rb, 4.71435e+00_rb, &
     & 4.74544e+00_rb, 4.77675e+00_rb, 4.80826e+00_rb, 4.83998e+00_rb, 4.87191e+00_rb, &
     & 4.90405e+00_rb, 4.93640e+00_rb, 4.96896e+00_rb, 5.00174e+00_rb/)
      kbo_mn2o( 3, :, 8) = (/ &
     & 4.43138e+00_rb, 4.46017e+00_rb, 4.48916e+00_rb, 4.51834e+00_rb, 4.54770e+00_rb, &
     & 4.57725e+00_rb, 4.60700e+00_rb, 4.63694e+00_rb, 4.66708e+00_rb, 4.69741e+00_rb, &
     & 4.72793e+00_rb, 4.75866e+00_rb, 4.78958e+00_rb, 4.82071e+00_rb, 4.85204e+00_rb, &
     & 4.88357e+00_rb, 4.91531e+00_rb, 4.94725e+00_rb, 4.97941e+00_rb/)
      kbo_mn2o( 4, :, 8) = (/ &
     & 4.47437e+00_rb, 4.50396e+00_rb, 4.53375e+00_rb, 4.56374e+00_rb, 4.59392e+00_rb, &
     & 4.62431e+00_rb, 4.65490e+00_rb, 4.68569e+00_rb, 4.71668e+00_rb, 4.74788e+00_rb, &
     & 4.77928e+00_rb, 4.81089e+00_rb, 4.84271e+00_rb, 4.87474e+00_rb, 4.90698e+00_rb, &
     & 4.93944e+00_rb, 4.97211e+00_rb, 5.00500e+00_rb, 5.03810e+00_rb/)
      kbo_mn2o( 5, :, 8) = (/ &
     & 8.82838e-01_rb, 8.92257e-01_rb, 9.01777e-01_rb, 9.11398e-01_rb, 9.21122e-01_rb, &
     & 9.30950e-01_rb, 9.40883e-01_rb, 9.50921e-01_rb, 9.61067e-01_rb, 9.71321e-01_rb, &
     & 9.81685e-01_rb, 9.92159e-01_rb, 1.00274e+00_rb, 1.01344e+00_rb, 1.02426e+00_rb, &
     & 1.03518e+00_rb, 1.04623e+00_rb, 1.05739e+00_rb, 1.06867e+00_rb/)
      kbo_mn2o( 1, :, 9) = (/ &
     & 7.92826e+00_rb, 7.83168e+00_rb, 7.73628e+00_rb, 7.64204e+00_rb, 7.54895e+00_rb, &
     & 7.45699e+00_rb, 7.36616e+00_rb, 7.27643e+00_rb, 7.18779e+00_rb, 7.10023e+00_rb, &
     & 7.01374e+00_rb, 6.92831e+00_rb, 6.84391e+00_rb, 6.76054e+00_rb, 6.67819e+00_rb, &
     & 6.59684e+00_rb, 6.51648e+00_rb, 6.43710e+00_rb, 6.35869e+00_rb/)
      kbo_mn2o( 2, :, 9) = (/ &
     & 7.89244e+00_rb, 7.79570e+00_rb, 7.70014e+00_rb, 7.60576e+00_rb, 7.51253e+00_rb, &
     & 7.42045e+00_rb, 7.32949e+00_rb, 7.23965e+00_rb, 7.15091e+00_rb, 7.06325e+00_rb, &
     & 6.97668e+00_rb, 6.89116e+00_rb, 6.80669e+00_rb, 6.72326e+00_rb, 6.64085e+00_rb, &
     & 6.55945e+00_rb, 6.47904e+00_rb, 6.39963e+00_rb, 6.32118e+00_rb/)
      kbo_mn2o( 3, :, 9) = (/ &
     & 7.79799e+00_rb, 7.70076e+00_rb, 7.60474e+00_rb, 7.50991e+00_rb, 7.41627e+00_rb, &
     & 7.32379e+00_rb, 7.23247e+00_rb, 7.14229e+00_rb, 7.05323e+00_rb, 6.96528e+00_rb, &
     & 6.87843e+00_rb, 6.79266e+00_rb, 6.70797e+00_rb, 6.62432e+00_rb, 6.54172e+00_rb, &
     & 6.46015e+00_rb, 6.37960e+00_rb, 6.30005e+00_rb, 6.22150e+00_rb/)
      kbo_mn2o( 4, :, 9) = (/ &
     & 7.53750e+00_rb, 7.43715e+00_rb, 7.33814e+00_rb, 7.24045e+00_rb, 7.14406e+00_rb, &
     & 7.04895e+00_rb, 6.95510e+00_rb, 6.86251e+00_rb, 6.77115e+00_rb, 6.68101e+00_rb, &
     & 6.59206e+00_rb, 6.50430e+00_rb, 6.41771e+00_rb, 6.33227e+00_rb, 6.24797e+00_rb, &
     & 6.16479e+00_rb, 6.08272e+00_rb, 6.00174e+00_rb, 5.92184e+00_rb/)
      kbo_mn2o( 5, :, 9) = (/ &
     & 5.06319e+00_rb, 5.08595e+00_rb, 5.10881e+00_rb, 5.13177e+00_rb, 5.15483e+00_rb, &
     & 5.17800e+00_rb, 5.20127e+00_rb, 5.22464e+00_rb, 5.24813e+00_rb, 5.27171e+00_rb, &
     & 5.29540e+00_rb, 5.31920e+00_rb, 5.34311e+00_rb, 5.36712e+00_rb, 5.39124e+00_rb, &
     & 5.41547e+00_rb, 5.43981e+00_rb, 5.46426e+00_rb, 5.48882e+00_rb/)
      kbo_mn2o( 1, :,10) = (/ &
     & 1.05265e+00_rb, 1.03986e+00_rb, 1.02723e+00_rb, 1.01475e+00_rb, 1.00243e+00_rb, &
     & 9.90250e-01_rb, 9.78220e-01_rb, 9.66338e-01_rb, 9.54599e-01_rb, 9.43003e-01_rb, &
     & 9.31548e-01_rb, 9.20232e-01_rb, 9.09054e-01_rb, 8.98011e-01_rb, 8.87102e-01_rb, &
     & 8.76326e-01_rb, 8.65681e-01_rb, 8.55165e-01_rb, 8.44777e-01_rb/)
      kbo_mn2o( 2, :,10) = (/ &
     & 1.05246e+00_rb, 1.03973e+00_rb, 1.02714e+00_rb, 1.01471e+00_rb, 1.00243e+00_rb, &
     & 9.90303e-01_rb, 9.78319e-01_rb, 9.66480e-01_rb, 9.54784e-01_rb, 9.43230e-01_rb, &
     & 9.31815e-01_rb, 9.20539e-01_rb, 9.09399e-01_rb, 8.98394e-01_rb, 8.87522e-01_rb, &
     & 8.76782e-01_rb, 8.66171e-01_rb, 8.55690e-01_rb, 8.45335e-01_rb/)
      kbo_mn2o( 3, :,10) = (/ &
     & 1.05236e+00_rb, 1.03963e+00_rb, 1.02704e+00_rb, 1.01461e+00_rb, 1.00233e+00_rb, &
     & 9.90203e-01_rb, 9.78219e-01_rb, 9.66380e-01_rb, 9.54684e-01_rb, 9.43130e-01_rb, &
     & 9.31715e-01_rb, 9.20439e-01_rb, 9.09300e-01_rb, 8.98295e-01_rb, 8.87423e-01_rb, &
     & 8.76683e-01_rb, 8.66073e-01_rb, 8.55591e-01_rb, 8.45236e-01_rb/)
      kbo_mn2o( 4, :,10) = (/ &
     & 9.30836e-01_rb, 9.21099e-01_rb, 9.11464e-01_rb, 9.01930e-01_rb, 8.92495e-01_rb, &
     & 8.83159e-01_rb, 8.73921e-01_rb, 8.64779e-01_rb, 8.55733e-01_rb, 8.46781e-01_rb, &
     & 8.37923e-01_rb, 8.29158e-01_rb, 8.20485e-01_rb, 8.11902e-01_rb, 8.03409e-01_rb, &
     & 7.95005e-01_rb, 7.86689e-01_rb, 7.78460e-01_rb, 7.70316e-01_rb/)
      kbo_mn2o( 5, :,10) = (/ &
     & 1.65786e+01_rb, 1.66541e+01_rb, 1.67299e+01_rb, 1.68062e+01_rb, 1.68827e+01_rb, &
     & 1.69596e+01_rb, 1.70369e+01_rb, 1.71145e+01_rb, 1.71925e+01_rb, 1.72708e+01_rb, &
     & 1.73495e+01_rb, 1.74285e+01_rb, 1.75079e+01_rb, 1.75877e+01_rb, 1.76678e+01_rb, &
     & 1.77483e+01_rb, 1.78291e+01_rb, 1.79103e+01_rb, 1.79919e+01_rb/)
      kbo_mn2o( 1, :,11) = (/ &
     & 1.74239e-01_rb, 1.77873e-01_rb, 1.81583e-01_rb, 1.85370e-01_rb, 1.89236e-01_rb, &
     & 1.93182e-01_rb, 1.97211e-01_rb, 2.01324e-01_rb, 2.05523e-01_rb, 2.09809e-01_rb, &
     & 2.14185e-01_rb, 2.18652e-01_rb, 2.23212e-01_rb, 2.27867e-01_rb, 2.32620e-01_rb, &
     & 2.37471e-01_rb, 2.42424e-01_rb, 2.47479e-01_rb, 2.52641e-01_rb/)
      kbo_mn2o( 2, :,11) = (/ &
     & 1.74114e-01_rb, 1.77756e-01_rb, 1.81475e-01_rb, 1.85271e-01_rb, 1.89147e-01_rb, &
     & 1.93104e-01_rb, 1.97144e-01_rb, 2.01268e-01_rb, 2.05479e-01_rb, 2.09778e-01_rb, &
     & 2.14166e-01_rb, 2.18647e-01_rb, 2.23221e-01_rb, 2.27890e-01_rb, 2.32658e-01_rb, &
     & 2.37525e-01_rb, 2.42494e-01_rb, 2.47567e-01_rb, 2.52746e-01_rb/)
      kbo_mn2o( 3, :,11) = (/ &
     & 1.74142e-01_rb, 1.77780e-01_rb, 1.81494e-01_rb, 1.85286e-01_rb, 1.89157e-01_rb, &
     & 1.93108e-01_rb, 1.97143e-01_rb, 2.01261e-01_rb, 2.05466e-01_rb, 2.09759e-01_rb, &
     & 2.14141e-01_rb, 2.18615e-01_rb, 2.23182e-01_rb, 2.27845e-01_rb, 2.32605e-01_rb, &
     & 2.37464e-01_rb, 2.42425e-01_rb, 2.47490e-01_rb, 2.52661e-01_rb/)
      kbo_mn2o( 4, :,11) = (/ &
     & 1.74074e-01_rb, 1.77716e-01_rb, 1.81435e-01_rb, 1.85231e-01_rb, 1.89107e-01_rb, &
     & 1.93064e-01_rb, 1.97104e-01_rb, 2.01229e-01_rb, 2.05439e-01_rb, 2.09738e-01_rb, &
     & 2.14127e-01_rb, 2.18607e-01_rb, 2.23181e-01_rb, 2.27852e-01_rb, 2.32619e-01_rb, &
     & 2.37487e-01_rb, 2.42456e-01_rb, 2.47529e-01_rb, 2.52709e-01_rb/)
      kbo_mn2o( 5, :,11) = (/ &
     & 2.31905e+01_rb, 2.33011e+01_rb, 2.34123e+01_rb, 2.35240e+01_rb, 2.36362e+01_rb, &
     & 2.37489e+01_rb, 2.38622e+01_rb, 2.39760e+01_rb, 2.40904e+01_rb, 2.42053e+01_rb, &
     & 2.43208e+01_rb, 2.44368e+01_rb, 2.45533e+01_rb, 2.46705e+01_rb, 2.47881e+01_rb, &
     & 2.49064e+01_rb, 2.50252e+01_rb, 2.51446e+01_rb, 2.52645e+01_rb/)
      kbo_mn2o( 1, :,12) = (/ &
     & 1.60269e-01_rb, 1.62873e-01_rb, 1.65519e-01_rb, 1.68208e-01_rb, 1.70940e-01_rb, &
     & 1.73717e-01_rb, 1.76540e-01_rb, 1.79408e-01_rb, 1.82322e-01_rb, 1.85284e-01_rb, &
     & 1.88295e-01_rb, 1.91354e-01_rb, 1.94462e-01_rb, 1.97622e-01_rb, 2.00832e-01_rb, &
     & 2.04095e-01_rb, 2.07411e-01_rb, 2.10780e-01_rb, 2.14205e-01_rb/)
      kbo_mn2o( 2, :,12) = (/ &
     & 1.60497e-01_rb, 1.63096e-01_rb, 1.65738e-01_rb, 1.68422e-01_rb, 1.71150e-01_rb, &
     & 1.73922e-01_rb, 1.76738e-01_rb, 1.79601e-01_rb, 1.82510e-01_rb, 1.85466e-01_rb, &
     & 1.88469e-01_rb, 1.91522e-01_rb, 1.94624e-01_rb, 1.97776e-01_rb, 2.00979e-01_rb, &
     & 2.04234e-01_rb, 2.07542e-01_rb, 2.10903e-01_rb, 2.14319e-01_rb/)
      kbo_mn2o( 3, :,12) = (/ &
     & 1.60407e-01_rb, 1.63006e-01_rb, 1.65648e-01_rb, 1.68332e-01_rb, 1.71060e-01_rb, &
     & 1.73832e-01_rb, 1.76649e-01_rb, 1.79511e-01_rb, 1.82420e-01_rb, 1.85376e-01_rb, &
     & 1.88380e-01_rb, 1.91433e-01_rb, 1.94535e-01_rb, 1.97687e-01_rb, 2.00891e-01_rb, &
     & 2.04146e-01_rb, 2.07454e-01_rb, 2.10816e-01_rb, 2.14232e-01_rb/)
      kbo_mn2o( 4, :,12) = (/ &
     & 1.60475e-01_rb, 1.63070e-01_rb, 1.65706e-01_rb, 1.68386e-01_rb, 1.71109e-01_rb, &
     & 1.73876e-01_rb, 1.76687e-01_rb, 1.79544e-01_rb, 1.82448e-01_rb, 1.85398e-01_rb, &
     & 1.88396e-01_rb, 1.91442e-01_rb, 1.94538e-01_rb, 1.97684e-01_rb, 2.00881e-01_rb, &
     & 2.04129e-01_rb, 2.07430e-01_rb, 2.10784e-01_rb, 2.14192e-01_rb/)
      kbo_mn2o( 5, :,12) = (/ &
     & 3.25743e+01_rb, 3.27992e+01_rb, 3.30256e+01_rb, 3.32536e+01_rb, 3.34832e+01_rb, &
     & 3.37143e+01_rb, 3.39471e+01_rb, 3.41815e+01_rb, 3.44174e+01_rb, 3.46550e+01_rb, &
     & 3.48943e+01_rb, 3.51352e+01_rb, 3.53778e+01_rb, 3.56220e+01_rb, 3.58679e+01_rb, &
     & 3.61156e+01_rb, 3.63649e+01_rb, 3.66159e+01_rb, 3.68687e+01_rb/)
      kbo_mn2o( 1, :,13) = (/ &
     & 2.01846e-01_rb, 2.03110e-01_rb, 2.04381e-01_rb, 2.05660e-01_rb, 2.06947e-01_rb, &
     & 2.08242e-01_rb, 2.09546e-01_rb, 2.10857e-01_rb, 2.12177e-01_rb, 2.13505e-01_rb, &
     & 2.14841e-01_rb, 2.16186e-01_rb, 2.17539e-01_rb, 2.18900e-01_rb, 2.20270e-01_rb, &
     & 2.21649e-01_rb, 2.23036e-01_rb, 2.24432e-01_rb, 2.25837e-01_rb/)
      kbo_mn2o( 2, :,13) = (/ &
     & 2.01756e-01_rb, 2.03020e-01_rb, 2.04291e-01_rb, 2.05570e-01_rb, 2.06857e-01_rb, &
     & 2.08152e-01_rb, 2.09456e-01_rb, 2.10767e-01_rb, 2.12087e-01_rb, 2.13415e-01_rb, &
     & 2.14751e-01_rb, 2.16096e-01_rb, 2.17449e-01_rb, 2.18810e-01_rb, 2.20181e-01_rb, &
     & 2.21559e-01_rb, 2.22946e-01_rb, 2.24342e-01_rb, 2.25747e-01_rb/)
      kbo_mn2o( 3, :,13) = (/ &
     & 2.01836e-01_rb, 2.03100e-01_rb, 2.04371e-01_rb, 2.05650e-01_rb, 2.06937e-01_rb, &
     & 2.08232e-01_rb, 2.09536e-01_rb, 2.10847e-01_rb, 2.12167e-01_rb, 2.13495e-01_rb, &
     & 2.14831e-01_rb, 2.16176e-01_rb, 2.17529e-01_rb, 2.18890e-01_rb, 2.20260e-01_rb, &
     & 2.21639e-01_rb, 2.23026e-01_rb, 2.24422e-01_rb, 2.25827e-01_rb/)
      kbo_mn2o( 4, :,13) = (/ &
     & 2.01845e-01_rb, 2.03103e-01_rb, 2.04370e-01_rb, 2.05644e-01_rb, 2.06926e-01_rb, &
     & 2.08217e-01_rb, 2.09515e-01_rb, 2.10822e-01_rb, 2.12136e-01_rb, 2.13459e-01_rb, &
     & 2.14790e-01_rb, 2.16129e-01_rb, 2.17477e-01_rb, 2.18833e-01_rb, 2.20198e-01_rb, &
     & 2.21571e-01_rb, 2.22952e-01_rb, 2.24343e-01_rb, 2.25741e-01_rb/)
      kbo_mn2o( 5, :,13) = (/ &
     & 5.34154e+01_rb, 5.31954e+01_rb, 5.29763e+01_rb, 5.27582e+01_rb, 5.25409e+01_rb, &
     & 5.23245e+01_rb, 5.21090e+01_rb, 5.18943e+01_rb, 5.16806e+01_rb, 5.14677e+01_rb, &
     & 5.12558e+01_rb, 5.10446e+01_rb, 5.08344e+01_rb, 5.06250e+01_rb, 5.04165e+01_rb, &
     & 5.02089e+01_rb, 5.00021e+01_rb, 4.97961e+01_rb, 4.95910e+01_rb/)
      kbo_mn2o( 1, :,14) = (/ &
     & 2.87818e-02_rb, 3.17868e-02_rb, 3.51056e-02_rb, 3.87708e-02_rb, 4.28187e-02_rb, &
     & 4.72893e-02_rb, 5.22266e-02_rb, 5.76794e-02_rb, 6.37015e-02_rb, 7.03523e-02_rb, &
     & 7.76976e-02_rb, 8.58097e-02_rb, 9.47688e-02_rb, 1.04663e-01_rb, 1.15591e-01_rb, &
     & 1.27659e-01_rb, 1.40988e-01_rb, 1.55708e-01_rb, 1.71965e-01_rb/)
      kbo_mn2o( 2, :,14) = (/ &
     & 2.88500e-02_rb, 3.18494e-02_rb, 3.51606e-02_rb, 3.88161e-02_rb, 4.28517e-02_rb, &
     & 4.73068e-02_rb, 5.22251e-02_rb, 5.76547e-02_rb, 6.36488e-02_rb, 7.02661e-02_rb, &
     & 7.75714e-02_rb, 8.56362e-02_rb, 9.45395e-02_rb, 1.04368e-01_rb, 1.15219e-01_rb, &
     & 1.27198e-01_rb, 1.40422e-01_rb, 1.55021e-01_rb, 1.71138e-01_rb/)
      kbo_mn2o( 3, :,14) = (/ &
     & 2.88036e-02_rb, 3.18109e-02_rb, 3.51322e-02_rb, 3.88002e-02_rb, 4.28512e-02_rb, &
     & 4.73252e-02_rb, 5.22663e-02_rb, 5.77232e-02_rb, 6.37499e-02_rb, 7.04058e-02_rb, &
     & 7.77567e-02_rb, 8.58751e-02_rb, 9.48410e-02_rb, 1.04743e-01_rb, 1.15679e-01_rb, &
     & 1.27757e-01_rb, 1.41095e-01_rb, 1.55827e-01_rb, 1.72096e-01_rb/)
      kbo_mn2o( 4, :,14) = (/ &
     & 2.87750e-02_rb, 3.17783e-02_rb, 3.50951e-02_rb, 3.87580e-02_rb, 4.28033e-02_rb, &
     & 4.72708e-02_rb, 5.22046e-02_rb, 5.76533e-02_rb, 6.36707e-02_rb, 7.03162e-02_rb, &
     & 7.76553e-02_rb, 8.57604e-02_rb, 9.47114e-02_rb, 1.04597e-01_rb, 1.15514e-01_rb, &
     & 1.27570e-01_rb, 1.40885e-01_rb, 1.55589e-01_rb, 1.71829e-01_rb/)
      kbo_mn2o( 5, :,14) = (/ &
     & 1.11848e+02_rb, 1.09489e+02_rb, 1.07179e+02_rb, 1.04918e+02_rb, 1.02705e+02_rb, &
     & 1.00538e+02_rb, 9.84175e+01_rb, 9.63414e+01_rb, 9.43091e+01_rb, 9.23196e+01_rb, &
     & 9.03722e+01_rb, 8.84658e+01_rb, 8.65996e+01_rb, 8.47728e+01_rb, 8.29845e+01_rb, &
     & 8.12339e+01_rb, 7.95203e+01_rb, 7.78428e+01_rb, 7.62007e+01_rb/)
      kbo_mn2o( 1, :,15) = (/ &
     & 1.52234e-02_rb, 1.69256e-02_rb, 1.88181e-02_rb, 2.09222e-02_rb, 2.32617e-02_rb, &
     & 2.58626e-02_rb, 2.87545e-02_rb, 3.19696e-02_rb, 3.55443e-02_rb, 3.95187e-02_rb, &
     & 4.39374e-02_rb, 4.88503e-02_rb, 5.43124e-02_rb, 6.03854e-02_rb, 6.71373e-02_rb, &
     & 7.46443e-02_rb, 8.29906e-02_rb, 9.22701e-02_rb, 1.02587e-01_rb/)
      kbo_mn2o( 2, :,15) = (/ &
     & 1.52234e-02_rb, 1.69256e-02_rb, 1.88181e-02_rb, 2.09222e-02_rb, 2.32617e-02_rb, &
     & 2.58626e-02_rb, 2.87545e-02_rb, 3.19696e-02_rb, 3.55443e-02_rb, 3.95187e-02_rb, &
     & 4.39374e-02_rb, 4.88503e-02_rb, 5.43124e-02_rb, 6.03854e-02_rb, 6.71373e-02_rb, &
     & 7.46443e-02_rb, 8.29906e-02_rb, 9.22701e-02_rb, 1.02587e-01_rb/)
      kbo_mn2o( 3, :,15) = (/ &
     & 1.52076e-02_rb, 1.69049e-02_rb, 1.87916e-02_rb, 2.08890e-02_rb, 2.32203e-02_rb, &
     & 2.58119e-02_rb, 2.86928e-02_rb, 3.18951e-02_rb, 3.54549e-02_rb, 3.94120e-02_rb, &
     & 4.38107e-02_rb, 4.87003e-02_rb, 5.41357e-02_rb, 6.01777e-02_rb, 6.68940e-02_rb, &
     & 7.43600e-02_rb, 8.26593e-02_rb, 9.18847e-02_rb, 1.02140e-01_rb/)
      kbo_mn2o( 4, :,15) = (/ &
     & 1.52422e-02_rb, 1.69488e-02_rb, 1.88464e-02_rb, 2.09565e-02_rb, 2.33028e-02_rb, &
     & 2.59118e-02_rb, 2.88129e-02_rb, 3.20389e-02_rb, 3.56260e-02_rb, 3.96148e-02_rb, &
     & 4.40501e-02_rb, 4.89821e-02_rb, 5.44662e-02_rb, 6.05643e-02_rb, 6.73452e-02_rb, &
     & 7.48853e-02_rb, 8.32696e-02_rb, 9.25927e-02_rb, 1.02959e-01_rb/)
      kbo_mn2o( 5, :,15) = (/ &
     & 6.94782e+01_rb, 6.73469e+01_rb, 6.52810e+01_rb, 6.32785e+01_rb, 6.13373e+01_rb, &
     & 5.94558e+01_rb, 5.76319e+01_rb, 5.58640e+01_rb, 5.41503e+01_rb, 5.24892e+01_rb, &
     & 5.08791e+01_rb, 4.93183e+01_rb, 4.78055e+01_rb, 4.63390e+01_rb, 4.49175e+01_rb, &
     & 4.35396e+01_rb, 4.22040e+01_rb, 4.09094e+01_rb, 3.96544e+01_rb/)
      kbo_mn2o( 1, :,16) = (/ &
     & 1.08257e-02_rb, 1.15188e-02_rb, 1.22563e-02_rb, 1.30410e-02_rb, 1.38759e-02_rb, &
     & 1.47643e-02_rb, 1.57096e-02_rb, 1.67154e-02_rb, 1.77857e-02_rb, 1.89244e-02_rb, &
     & 2.01360e-02_rb, 2.14252e-02_rb, 2.27970e-02_rb, 2.42565e-02_rb, 2.58096e-02_rb, &
     & 2.74620e-02_rb, 2.92203e-02_rb, 3.10911e-02_rb, 3.30817e-02_rb/)
      kbo_mn2o( 2, :,16) = (/ &
     & 1.08257e-02_rb, 1.15188e-02_rb, 1.22563e-02_rb, 1.30410e-02_rb, 1.38759e-02_rb, &
     & 1.47643e-02_rb, 1.57096e-02_rb, 1.67154e-02_rb, 1.77857e-02_rb, 1.89244e-02_rb, &
     & 2.01360e-02_rb, 2.14252e-02_rb, 2.27970e-02_rb, 2.42565e-02_rb, 2.58096e-02_rb, &
     & 2.74620e-02_rb, 2.92203e-02_rb, 3.10911e-02_rb, 3.30817e-02_rb/)
      kbo_mn2o( 3, :,16) = (/ &
     & 1.08257e-02_rb, 1.15188e-02_rb, 1.22563e-02_rb, 1.30410e-02_rb, 1.38759e-02_rb, &
     & 1.47643e-02_rb, 1.57096e-02_rb, 1.67154e-02_rb, 1.77857e-02_rb, 1.89244e-02_rb, &
     & 2.01360e-02_rb, 2.14252e-02_rb, 2.27970e-02_rb, 2.42565e-02_rb, 2.58096e-02_rb, &
     & 2.74620e-02_rb, 2.92203e-02_rb, 3.10911e-02_rb, 3.30817e-02_rb/)
      kbo_mn2o( 4, :,16) = (/ &
     & 1.08263e-02_rb, 1.15191e-02_rb, 1.22562e-02_rb, 1.30404e-02_rb, 1.38749e-02_rb, &
     & 1.47627e-02_rb, 1.57073e-02_rb, 1.67124e-02_rb, 1.77818e-02_rb, 1.89196e-02_rb, &
     & 2.01302e-02_rb, 2.14183e-02_rb, 2.27888e-02_rb, 2.42470e-02_rb, 2.57986e-02_rb, &
     & 2.74494e-02_rb, 2.92058e-02_rb, 3.10746e-02_rb, 3.30630e-02_rb/)
      kbo_mn2o( 5, :,16) = (/ &
     & 4.53450e-08_rb, 5.75193e-08_rb, 7.29620e-08_rb, 9.25509e-08_rb, 1.17399e-07_rb, &
     & 1.48918e-07_rb, 1.88900e-07_rb, 2.39616e-07_rb, 3.03948e-07_rb, 3.85551e-07_rb, &
     & 4.89064e-07_rb, 6.20369e-07_rb, 7.86925e-07_rb, 9.98199e-07_rb, 1.26619e-06_rb, &
     & 1.60614e-06_rb, 2.03736e-06_rb, 2.58435e-06_rb, 3.27819e-06_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &9.0039e-04_rb,1.1081e-03_rb,1.0732e-03_rb,1.1881e-03_rb,1.2488e-03_rb,1.3170e-03_rb, &
     &1.3317e-03_rb,1.3168e-03_rb,1.3369e-03_rb,1.4228e-03_rb,1.5385e-03_rb,1.7376e-03_rb, &
     &1.7122e-03_rb,1.9002e-03_rb,1.8881e-03_rb,2.1595e-03_rb/)
      forrefo(2,:) = (/ &
     &1.2726e-03_rb,1.3680e-03_rb,1.2494e-03_rb,1.2049e-03_rb,1.2048e-03_rb,1.1256e-03_rb, &
     &1.1170e-03_rb,1.0697e-03_rb,1.1177e-03_rb,1.1883e-03_rb,1.2219e-03_rb,1.2179e-03_rb, &
     &1.5692e-03_rb,1.9103e-03_rb,2.0219e-03_rb,1.6937e-03_rb/)
      forrefo(3,:) = (/ &
     &1.5527e-03_rb,1.6477e-03_rb,1.4973e-03_rb,1.3400e-03_rb,1.0820e-03_rb,9.3315e-04_rb, &
     &8.8132e-04_rb,8.1508e-04_rb,8.3559e-04_rb,7.6492e-04_rb,8.2343e-04_rb,7.1274e-04_rb, &
     &6.6011e-04_rb,6.7179e-04_rb,6.7039e-04_rb,6.7021e-04_rb/)
      forrefo(4,:) = (/ &
     &1.6763e-03_rb,1.6066e-03_rb,1.3927e-03_rb,1.2087e-03_rb,9.8463e-04_rb,8.8414e-04_rb, &
     &8.0976e-04_rb,7.8758e-04_rb,7.7376e-04_rb,7.5785e-04_rb,7.4152e-04_rb,7.3814e-04_rb, &
     &7.4278e-04_rb,7.1745e-04_rb,6.7216e-04_rb,6.4097e-04_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 5.11926e-01_rb, 4.32863e-01_rb, 3.66010e-01_rb, 3.09482e-01_rb, 2.61685e-01_rb, &
     & 2.21269e-01_rb, 1.87096e-01_rb, 1.58200e-01_rb, 1.33767e-01_rb, 1.13108e-01_rb/)
      selfrefo(:, 2) = (/ &
     & 5.02863e-01_rb, 4.35008e-01_rb, 3.76310e-01_rb, 3.25532e-01_rb, 2.81606e-01_rb, &
     & 2.43607e-01_rb, 2.10736e-01_rb, 1.82300e-01_rb, 1.57701e-01_rb, 1.36422e-01_rb/)
      selfrefo(:, 3) = (/ &
     & 4.57628e-01_rb, 3.99663e-01_rb, 3.49040e-01_rb, 3.04829e-01_rb, 2.66218e-01_rb, &
     & 2.32498e-01_rb, 2.03049e-01_rb, 1.77330e-01_rb, 1.54869e-01_rb, 1.35252e-01_rb/)
      selfrefo(:, 4) = (/ &
     & 4.28634e-01_rb, 3.81736e-01_rb, 3.39970e-01_rb, 3.02773e-01_rb, 2.69647e-01_rb, &
     & 2.40144e-01_rb, 2.13870e-01_rb, 1.90470e-01_rb, 1.69630e-01_rb, 1.51071e-01_rb/)
      selfrefo(:, 5) = (/ &
     & 4.21002e-01_rb, 3.77493e-01_rb, 3.38480e-01_rb, 3.03499e-01_rb, 2.72133e-01_rb, &
     & 2.44009e-01_rb, 2.18792e-01_rb, 1.96180e-01_rb, 1.75905e-01_rb, 1.57726e-01_rb/)
      selfrefo(:, 6) = (/ &
     & 3.97517e-01_rb, 3.61167e-01_rb, 3.28140e-01_rb, 2.98133e-01_rb, 2.70871e-01_rb, &
     & 2.46101e-01_rb, 2.23597e-01_rb, 2.03150e-01_rb, 1.84573e-01_rb, 1.67695e-01_rb/)
      selfrefo(:, 7) = (/ &
     & 3.92114e-01_rb, 3.57554e-01_rb, 3.26040e-01_rb, 2.97304e-01_rb, 2.71100e-01_rb, &
     & 2.47206e-01_rb, 2.25418e-01_rb, 2.05550e-01_rb, 1.87433e-01_rb, 1.70913e-01_rb/)
      selfrefo(:, 8) = (/ &
     & 3.79555e-01_rb, 3.47264e-01_rb, 3.17720e-01_rb, 2.90690e-01_rb, 2.65959e-01_rb, &
     & 2.43332e-01_rb, 2.22631e-01_rb, 2.03690e-01_rb, 1.86361e-01_rb, 1.70506e-01_rb/)
      selfrefo(:, 9) = (/ &
     & 3.92644e-01_rb, 3.58048e-01_rb, 3.26500e-01_rb, 2.97732e-01_rb, 2.71498e-01_rb, &
     & 2.47576e-01_rb, 2.25762e-01_rb, 2.05870e-01_rb, 1.87731e-01_rb, 1.71190e-01_rb/)
      selfrefo(:,10) = (/ &
     & 4.06542e-01_rb, 3.71200e-01_rb, 3.38930e-01_rb, 3.09465e-01_rb, 2.82562e-01_rb, &
     & 2.57998e-01_rb, 2.35569e-01_rb, 2.15090e-01_rb, 1.96391e-01_rb, 1.79318e-01_rb/)
      selfrefo(:,11) = (/ &
     & 4.09672e-01_rb, 3.76237e-01_rb, 3.45530e-01_rb, 3.17329e-01_rb, 2.91430e-01_rb, &
     & 2.67645e-01_rb, 2.45801e-01_rb, 2.25740e-01_rb, 2.07316e-01_rb, 1.90396e-01_rb/)
      selfrefo(:,12) = (/ &
     & 3.85140e-01_rb, 3.61989e-01_rb, 3.40230e-01_rb, 3.19779e-01_rb, 3.00557e-01_rb, &
     & 2.82490e-01_rb, 2.65510e-01_rb, 2.49550e-01_rb, 2.34549e-01_rb, 2.20451e-01_rb/)
      selfrefo(:,13) = (/ &
     & 4.87349e-01_rb, 4.42192e-01_rb, 4.01220e-01_rb, 3.64044e-01_rb, 3.30313e-01_rb, &
     & 2.99707e-01_rb, 2.71937e-01_rb, 2.46740e-01_rb, 2.23878e-01_rb, 2.03134e-01_rb/)
      selfrefo(:,14) = (/ &
     & 5.64339e-01_rb, 5.06194e-01_rb, 4.54040e-01_rb, 4.07259e-01_rb, 3.65298e-01_rb, &
     & 3.27661e-01_rb, 2.93901e-01_rb, 2.63620e-01_rb, 2.36459e-01_rb, 2.12096e-01_rb/)
      selfrefo(:,15) = (/ &
     & 5.91123e-01_rb, 5.26420e-01_rb, 4.68800e-01_rb, 4.17486e-01_rb, 3.71790e-01_rb, &
     & 3.31095e-01_rb, 2.94854e-01_rb, 2.62580e-01_rb, 2.33839e-01_rb, 2.08243e-01_rb/)
      selfrefo(:,16) = (/ &
     & 5.04590e-01_rb, 4.65235e-01_rb, 4.28950e-01_rb, 3.95495e-01_rb, 3.64649e-01_rb, &
     & 3.36209e-01_rb, 3.09987e-01_rb, 2.85810e-01_rb, 2.63519e-01_rb, 2.42966e-01_rb/)

      end subroutine lw_kgb03 

! **************************************************************************
      subroutine lw_kgb04
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg04, only : fracrefao, fracrefbo, kao, kbo, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P = 142.5940 mbar, T = 215.70 K
      fracrefao(:, 1) = (/ &
     &   1.5572e-01_rb,1.4925e-01_rb,1.4107e-01_rb,1.3126e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2393e-02_rb,4.2146e-02_rb,4.5907e-03_rb,3.7965e-03_rb,2.9744e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 2) = (/ &
     &   1.5572e-01_rb,1.4925e-01_rb,1.4107e-01_rb,1.3126e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2392e-02_rb,4.2146e-02_rb,4.5906e-03_rb,3.7965e-03_rb,2.9745e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 3) = (/ &
     &   1.5572e-01_rb,1.4925e-01_rb,1.4107e-01_rb,1.3126e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2393e-02_rb,4.2146e-02_rb,4.5907e-03_rb,3.7965e-03_rb,2.9745e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 4) = (/ &
     &   1.5572e-01_rb,1.4925e-01_rb,1.4107e-01_rb,1.3126e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2393e-02_rb,4.2146e-02_rb,4.5907e-03_rb,3.7964e-03_rb,2.9744e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 5) = (/ &
     &   1.5572e-01_rb,1.4925e-01_rb,1.4107e-01_rb,1.3126e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2393e-02_rb,4.2146e-02_rb,4.5907e-03_rb,3.7965e-03_rb,2.9744e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 6) = (/ &
     &   1.5572e-01_rb,1.4925e-01_rb,1.4107e-01_rb,1.3126e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2393e-02_rb,4.2146e-02_rb,4.5907e-03_rb,3.7965e-03_rb,2.9744e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 7) = (/ &
     &   1.5572e-01_rb,1.4926e-01_rb,1.4107e-01_rb,1.3126e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2393e-02_rb,4.2146e-02_rb,4.5908e-03_rb,3.7964e-03_rb,2.9745e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 8) = (/ &
     &   1.5571e-01_rb,1.4926e-01_rb,1.4107e-01_rb,1.3125e-01_rb,1.1791e-01_rb,1.0173e-01_rb, &
     &   8.2949e-02_rb,6.2393e-02_rb,4.2146e-02_rb,4.5907e-03_rb,3.7964e-03_rb,2.9744e-03_rb, &
     &   2.2074e-03_rb,1.4063e-03_rb,5.3012e-04_rb,7.4595e-05_rb/)
      fracrefao(:, 9) = (/ &
     &   1.5952e-01_rb,1.5155e-01_rb,1.4217e-01_rb,1.3077e-01_rb,1.1667e-01_rb,1.0048e-01_rb, &
     &   8.1511e-02_rb,6.1076e-02_rb,4.1111e-02_rb,4.4432e-03_rb,3.6910e-03_rb,2.9076e-03_rb, &
     &   2.1329e-03_rb,1.3566e-03_rb,5.2235e-04_rb,7.9935e-05_rb/)

! Planck fraction mapping level : P = 95.58350 mb, T = 215.70 K
      fracrefbo(:, 1) = (/ &
     &   1.5558e-01_rb,1.4931e-01_rb,1.4104e-01_rb,1.3124e-01_rb,1.1793e-01_rb,1.0160e-01_rb, &
     &   8.3142e-02_rb,6.2403e-02_rb,4.2170e-02_rb,4.5935e-03_rb,3.7976e-03_rb,2.9986e-03_rb, &
     &   2.1890e-03_rb,1.4061e-03_rb,5.3005e-04_rb,7.4587e-05_rb/)
      fracrefbo(:, 2) = (/ &
     &   1.5558e-01_rb,1.4932e-01_rb,1.4104e-01_rb,1.3124e-01_rb,1.1792e-01_rb,1.0159e-01_rb, &
     &   8.3142e-02_rb,6.2403e-02_rb,4.2170e-02_rb,4.5935e-03_rb,3.7976e-03_rb,2.9986e-03_rb, &
     &   2.1890e-03_rb,1.4061e-03_rb,5.3005e-04_rb,7.4587e-05_rb/)
      fracrefbo(:, 3) = (/ &
     &   1.5558e-01_rb,1.4933e-01_rb,1.4103e-01_rb,1.3124e-01_rb,1.1792e-01_rb,1.0159e-01_rb, &
     &   8.3142e-02_rb,6.2403e-02_rb,4.2170e-02_rb,4.5935e-03_rb,3.7976e-03_rb,2.9986e-03_rb, &
     &   2.1890e-03_rb,1.4061e-03_rb,5.3005e-04_rb,7.4587e-05_rb/)
      fracrefbo(:, 4) = (/ &
     &   1.5569e-01_rb,1.4926e-01_rb,1.4102e-01_rb,1.3122e-01_rb,1.1791e-01_rb,1.0159e-01_rb, &
     &   8.3141e-02_rb,6.2403e-02_rb,4.2170e-02_rb,4.5935e-03_rb,3.7976e-03_rb,2.9986e-03_rb, &
     &   2.1890e-03_rb,1.4061e-03_rb,5.3005e-04_rb,7.4587e-05_rb/)
      fracrefbo(:, 5) = (/ &
     &   1.5947e-01_rb,1.5132e-01_rb,1.4195e-01_rb,1.3061e-01_rb,1.1680e-01_rb,1.0054e-01_rb, &
     &   8.1785e-02_rb,6.1212e-02_rb,4.1276e-02_rb,4.4424e-03_rb,3.6628e-03_rb,2.8943e-03_rb, &
     &   2.1134e-03_rb,1.3457e-03_rb,5.1024e-04_rb,7.3998e-05_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &1.0697e-02_rb,1.0151e-02_rb,9.2549e-03_rb,8.2645e-03_rb,7.2061e-03_rb,6.0662e-03_rb, &
     &4.9816e-03_rb,4.8867e-03_rb,2.0269e-04_rb/)
      kao(:, 2, 1, 1) = (/ &
     &1.1677e-02_rb,1.1138e-02_rb,1.0216e-02_rb,9.1909e-03_rb,8.0553e-03_rb,6.8223e-03_rb, &
     &5.5408e-03_rb,4.6854e-03_rb,2.3825e-04_rb/)
      kao(:, 3, 1, 1) = (/ &
     &1.2605e-02_rb,1.2092e-02_rb,1.1179e-02_rb,1.0121e-02_rb,8.9159e-03_rb,7.6027e-03_rb, &
     &6.0392e-03_rb,4.3498e-03_rb,2.8245e-04_rb/)
      kao(:, 4, 1, 1) = (/ &
     &1.3484e-02_rb,1.3046e-02_rb,1.2153e-02_rb,1.1041e-02_rb,9.7806e-03_rb,8.2921e-03_rb, &
     &6.5023e-03_rb,4.3023e-03_rb,3.0318e-04_rb/)
      kao(:, 5, 1, 1) = (/ &
     &1.4330e-02_rb,1.3997e-02_rb,1.3114e-02_rb,1.1943e-02_rb,1.0557e-02_rb,8.8567e-03_rb, &
     &6.8709e-03_rb,4.3577e-03_rb,3.2751e-04_rb/)
      kao(:, 1, 2, 1) = (/ &
     &1.3833e-02_rb,1.2806e-02_rb,1.1465e-02_rb,1.0025e-02_rb,8.5208e-03_rb,6.9589e-03_rb, &
     &5.3336e-03_rb,4.2294e-03_rb,1.4419e-04_rb/)
      kao(:, 2, 2, 1) = (/ &
     &1.5144e-02_rb,1.4068e-02_rb,1.2644e-02_rb,1.1119e-02_rb,9.5204e-03_rb,7.8236e-03_rb, &
     &6.0220e-03_rb,4.4086e-03_rb,1.7355e-04_rb/)
      kao(:, 3, 2, 1) = (/ &
     &1.6397e-02_rb,1.5299e-02_rb,1.3819e-02_rb,1.2230e-02_rb,1.0525e-02_rb,8.6894e-03_rb, &
     &6.7322e-03_rb,4.5883e-03_rb,2.1022e-04_rb/)
      kao(:, 4, 2, 1) = (/ &
     &1.7608e-02_rb,1.6518e-02_rb,1.5010e-02_rb,1.3349e-02_rb,1.1524e-02_rb,9.5665e-03_rb, &
     &7.3586e-03_rb,4.7702e-03_rb,2.1845e-04_rb/)
      kao(:, 5, 2, 1) = (/ &
     &1.8795e-02_rb,1.7739e-02_rb,1.6211e-02_rb,1.4458e-02_rb,1.2504e-02_rb,1.0367e-02_rb, &
     &7.8746e-03_rb,4.9505e-03_rb,2.4452e-04_rb/)
      kao(:, 1, 3, 1) = (/ &
     &2.3349e-02_rb,2.1073e-02_rb,1.8534e-02_rb,1.5886e-02_rb,1.3154e-02_rb,1.0346e-02_rb, &
     &7.4749e-03_rb,4.5440e-03_rb,9.5522e-05_rb/)
      kao(:, 2, 3, 1) = (/ &
     &2.5718e-02_rb,2.3272e-02_rb,2.0503e-02_rb,1.7622e-02_rb,1.4648e-02_rb,1.1596e-02_rb, &
     &8.4531e-03_rb,5.1652e-03_rb,1.1957e-04_rb/)
      kao(:, 3, 3, 1) = (/ &
     &2.8025e-02_rb,2.5421e-02_rb,2.2452e-02_rb,1.9353e-02_rb,1.6163e-02_rb,1.2876e-02_rb, &
     &9.4440e-03_rb,5.8151e-03_rb,1.4638e-04_rb/)
      kao(:, 4, 3, 1) = (/ &
     &3.0273e-02_rb,2.7542e-02_rb,2.4400e-02_rb,2.1110e-02_rb,1.7713e-02_rb,1.4169e-02_rb, &
     &1.0445e-02_rb,6.4078e-03_rb,1.5375e-04_rb/)
      kao(:, 5, 3, 1) = (/ &
     &3.2488e-02_rb,2.9668e-02_rb,2.6366e-02_rb,2.2912e-02_rb,1.9285e-02_rb,1.5463e-02_rb, &
     &1.1411e-02_rb,6.9035e-03_rb,1.7414e-04_rb/)
      kao(:, 1, 4, 1) = (/ &
     &3.9680e-02_rb,3.5246e-02_rb,3.0687e-02_rb,2.5989e-02_rb,2.1199e-02_rb,1.6340e-02_rb, &
     &1.1394e-02_rb,6.3232e-03_rb,6.3142e-05_rb/)
      kao(:, 2, 4, 1) = (/ &
     &4.4067e-02_rb,3.9245e-02_rb,3.4186e-02_rb,2.8983e-02_rb,2.3690e-02_rb,1.8315e-02_rb, &
     &1.2835e-02_rb,7.2156e-03_rb,8.2154e-05_rb/)
      kao(:, 3, 4, 1) = (/ &
     &4.8382e-02_rb,4.3189e-02_rb,3.7634e-02_rb,3.1955e-02_rb,2.6184e-02_rb,2.0317e-02_rb, &
     &1.4329e-02_rb,8.1343e-03_rb,1.0015e-04_rb/)
      kao(:, 4, 4, 1) = (/ &
     &5.2631e-02_rb,4.7051e-02_rb,4.1069e-02_rb,3.4945e-02_rb,2.8716e-02_rb,2.2361e-02_rb, &
     &1.5856e-02_rb,9.0735e-03_rb,1.0763e-04_rb/)
      kao(:, 5, 4, 1) = (/ &
     &5.6812e-02_rb,5.0879e-02_rb,4.4486e-02_rb,3.7949e-02_rb,3.1266e-02_rb,2.4439e-02_rb, &
     &1.7397e-02_rb,9.9865e-03_rb,1.2433e-04_rb/)
      kao(:, 1, 5, 1) = (/ &
     &6.1659e-02_rb,5.4360e-02_rb,4.7000e-02_rb,3.9571e-02_rb,3.2046e-02_rb,2.4431e-02_rb, &
     &1.6731e-02_rb,8.9002e-03_rb,4.4135e-05_rb/)
      kao(:, 2, 5, 1) = (/ &
     &6.9177e-02_rb,6.1077e-02_rb,5.2885e-02_rb,4.4540e-02_rb,3.6083e-02_rb,2.7553e-02_rb, &
     &1.8920e-02_rb,1.0138e-02_rb,5.7278e-05_rb/)
      kao(:, 3, 5, 1) = (/ &
     &7.6561e-02_rb,6.7721e-02_rb,5.8676e-02_rb,4.9436e-02_rb,4.0109e-02_rb,3.0675e-02_rb, &
     &2.1149e-02_rb,1.1429e-02_rb,6.8177e-05_rb/)
      kao(:, 4, 5, 1) = (/ &
     &8.3855e-02_rb,7.4287e-02_rb,6.4398e-02_rb,5.4317e-02_rb,4.4139e-02_rb,3.3855e-02_rb, &
     &2.3434e-02_rb,1.2758e-02_rb,7.5655e-05_rb/)
      kao(:, 5, 5, 1) = (/ &
     &9.1112e-02_rb,8.0836e-02_rb,7.0093e-02_rb,5.9205e-02_rb,4.8195e-02_rb,3.7076e-02_rb, &
     &2.5757e-02_rb,1.4110e-02_rb,8.9921e-05_rb/)
      kao(:, 1, 6, 1) = (/ &
     &8.8882e-02_rb,7.8094e-02_rb,6.7257e-02_rb,5.6376e-02_rb,4.5468e-02_rb,3.4471e-02_rb, &
     &2.3372e-02_rb,1.2141e-02_rb,3.4008e-05_rb/)
      kao(:, 2, 6, 1) = (/ &
     &1.0088e-01_rb,8.8690e-02_rb,7.6454e-02_rb,6.4166e-02_rb,5.1764e-02_rb,3.9256e-02_rb, &
     &2.6651e-02_rb,1.3906e-02_rb,4.1496e-05_rb/)
      kao(:, 3, 6, 1) = (/ &
     &1.1268e-01_rb,9.9170e-02_rb,8.5584e-02_rb,7.1867e-02_rb,5.7997e-02_rb,4.4019e-02_rb, &
     &2.9951e-02_rb,1.5714e-02_rb,4.7587e-05_rb/)
      kao(:, 4, 6, 1) = (/ &
     &1.2443e-01_rb,1.0961e-01_rb,9.4686e-02_rb,7.9524e-02_rb,6.4216e-02_rb,4.8825e-02_rb, &
     &3.3307e-02_rb,1.7586e-02_rb,5.4734e-05_rb/)
      kao(:, 5, 6, 1) = (/ &
     &1.3608e-01_rb,1.2003e-01_rb,1.0372e-01_rb,8.7155e-02_rb,7.0466e-02_rb,5.3660e-02_rb, &
     &3.6716e-02_rb,1.9490e-02_rb,6.4532e-05_rb/)
      kao(:, 1, 7, 1) = (/ &
     &1.3393e-01_rb,1.1743e-01_rb,1.0091e-01_rb,8.4366e-02_rb,6.7796e-02_rb,5.1191e-02_rb, &
     &3.4487e-02_rb,1.7647e-02_rb,2.6258e-05_rb/)
      kao(:, 2, 7, 1) = (/ &
     &1.5385e-01_rb,1.3496e-01_rb,1.1603e-01_rb,9.7081e-02_rb,7.8079e-02_rb,5.8981e-02_rb, &
     &3.9763e-02_rb,2.0387e-02_rb,3.2089e-05_rb/)
      kao(:, 3, 7, 1) = (/ &
     &1.7362e-01_rb,1.5237e-01_rb,1.3107e-01_rb,1.0975e-01_rb,8.8299e-02_rb,6.6730e-02_rb, &
     &4.5023e-02_rb,2.3161e-02_rb,4.0067e-05_rb/)
      kao(:, 4, 7, 1) = (/ &
     &1.9325e-01_rb,1.6971e-01_rb,1.4609e-01_rb,1.2238e-01_rb,9.8509e-02_rb,7.4481e-02_rb, &
     &5.0334e-02_rb,2.5995e-02_rb,4.6630e-05_rb/)
      kao(:, 5, 7, 1) = (/ &
     &2.1303e-01_rb,1.8717e-01_rb,1.6126e-01_rb,1.3511e-01_rb,1.0879e-01_rb,8.2319e-02_rb, &
     &5.5720e-02_rb,2.8900e-02_rb,5.5874e-05_rb/)
      kao(:, 1, 8, 1) = (/ &
     &2.4085e-01_rb,2.1093e-01_rb,1.8102e-01_rb,1.5107e-01_rb,1.2112e-01_rb,9.1137e-02_rb, &
     &6.1112e-02_rb,3.0964e-02_rb,2.6704e-05_rb/)
      kao(:, 2, 8, 1) = (/ &
     &2.8040e-01_rb,2.4563e-01_rb,2.1082e-01_rb,1.7598e-01_rb,1.4114e-01_rb,1.0627e-01_rb, &
     &7.1313e-02_rb,3.6159e-02_rb,3.4463e-05_rb/)
      kao(:, 3, 8, 1) = (/ &
     &3.1986e-01_rb,2.8024e-01_rb,2.4058e-01_rb,2.0091e-01_rb,1.6119e-01_rb,1.2143e-01_rb, &
     &8.1504e-02_rb,4.1377e-02_rb,4.2885e-05_rb/)
      kao(:, 4, 8, 1) = (/ &
     &3.5910e-01_rb,3.1468e-01_rb,2.7026e-01_rb,2.2578e-01_rb,1.8126e-01_rb,1.3658e-01_rb, &
     &9.1734e-02_rb,4.6641e-02_rb,5.1508e-05_rb/)
      kao(:, 5, 8, 1) = (/ &
     &3.9881e-01_rb,3.4958e-01_rb,3.0035e-01_rb,2.5107e-01_rb,2.0163e-01_rb,1.5198e-01_rb, &
     &1.0212e-01_rb,5.2034e-02_rb,5.9872e-05_rb/)
      kao(:, 1, 9, 1) = (/ &
     &8.1251e-01_rb,7.1113e-01_rb,6.0978e-01_rb,5.0834e-01_rb,4.0692e-01_rb,3.0549e-01_rb, &
     &2.0401e-01_rb,1.0247e-01_rb,4.2060e-05_rb/)
      kao(:, 2, 9, 1) = (/ &
     &9.6024e-01_rb,8.4039e-01_rb,7.2060e-01_rb,6.0079e-01_rb,4.8095e-01_rb,3.6106e-01_rb, &
     &2.4120e-01_rb,1.2123e-01_rb,5.7363e-05_rb/)
      kao(:, 3, 9, 1) = (/ &
     &1.1088e+00_rb,9.7056e-01_rb,8.3222e-01_rb,6.9393e-01_rb,5.5555e-01_rb,4.1720e-01_rb, &
     &2.7875e-01_rb,1.4021e-01_rb,7.4113e-05_rb/)
      kao(:, 4, 9, 1) = (/ &
     &1.2568e+00_rb,1.1001e+00_rb,9.4342e-01_rb,7.8665e-01_rb,6.2985e-01_rb,4.7311e-01_rb, &
     &3.1630e-01_rb,1.5915e-01_rb,9.2862e-05_rb/)
      kao(:, 5, 9, 1) = (/ &
     &1.4062e+00_rb,1.2309e+00_rb,1.0557e+00_rb,8.8039e-01_rb,7.0513e-01_rb,5.2977e-01_rb, &
     &3.5432e-01_rb,1.7839e-01_rb,1.0954e-04_rb/)
      kao(:, 1,10, 1) = (/ &
     &3.0868e+00_rb,2.7010e+00_rb,2.3154e+00_rb,1.9296e+00_rb,1.5440e+00_rb,1.1583e+00_rb, &
     &7.7257e-01_rb,3.8682e-01_rb,7.9650e-05_rb/)
      kao(:, 2,10, 1) = (/ &
     &3.6992e+00_rb,3.2371e+00_rb,2.7749e+00_rb,2.3126e+00_rb,1.8504e+00_rb,1.3883e+00_rb, &
     &9.2595e-01_rb,4.6361e-01_rb,1.1315e-04_rb/)
      kao(:, 3,10, 1) = (/ &
     &4.3239e+00_rb,3.7838e+00_rb,3.2436e+00_rb,2.7034e+00_rb,2.1630e+00_rb,1.6228e+00_rb, &
     &1.0824e+00_rb,5.4204e-01_rb,1.4419e-04_rb/)
      kao(:, 4,10, 1) = (/ &
     &4.9483e+00_rb,4.3302e+00_rb,3.7121e+00_rb,3.0937e+00_rb,2.4754e+00_rb,1.8572e+00_rb, &
     &1.2389e+00_rb,6.2062e-01_rb,1.7752e-04_rb/)
      kao(:, 5,10, 1) = (/ &
     &5.5745e+00_rb,4.8780e+00_rb,4.1816e+00_rb,3.4855e+00_rb,2.7890e+00_rb,2.0926e+00_rb, &
     &1.3962e+00_rb,6.9963e-01_rb,2.1449e-04_rb/)
      kao(:, 1,11, 1) = (/ &
     &4.4731e+00_rb,3.9138e+00_rb,3.3549e+00_rb,2.7960e+00_rb,2.2371e+00_rb,1.6781e+00_rb, &
     &1.1191e+00_rb,5.6013e-01_rb,1.1739e-04_rb/)
      kao(:, 2,11, 1) = (/ &
     &5.3673e+00_rb,4.6967e+00_rb,4.0260e+00_rb,3.3552e+00_rb,2.6847e+00_rb,2.0138e+00_rb, &
     &1.3431e+00_rb,6.7224e-01_rb,1.4272e-04_rb/)
      kao(:, 3,11, 1) = (/ &
     &6.2805e+00_rb,5.4954e+00_rb,4.7109e+00_rb,3.9260e+00_rb,3.1412e+00_rb,2.3563e+00_rb, &
     &1.5716e+00_rb,7.8663e-01_rb,1.8731e-04_rb/)
      kao(:, 4,11, 1) = (/ &
     &7.1941e+00_rb,6.2951e+00_rb,5.3963e+00_rb,4.4973e+00_rb,3.5984e+00_rb,2.6993e+00_rb, &
     &1.8005e+00_rb,9.0138e-01_rb,2.2429e-04_rb/)
      kao(:, 5,11, 1) = (/ &
     &8.1084e+00_rb,7.0952e+00_rb,6.0822e+00_rb,5.0692e+00_rb,4.0560e+00_rb,3.0428e+00_rb, &
     &2.0297e+00_rb,1.0163e+00_rb,2.6588e-04_rb/)
      kao(:, 1,12, 1) = (/ &
     &4.8324e+00_rb,4.2285e+00_rb,3.6244e+00_rb,3.0205e+00_rb,2.4167e+00_rb,1.8130e+00_rb, &
     &1.2091e+00_rb,6.0509e-01_rb,1.2228e-04_rb/)
      kao(:, 2,12, 1) = (/ &
     &5.8007e+00_rb,5.0761e+00_rb,4.3512e+00_rb,3.6264e+00_rb,2.9012e+00_rb,2.1763e+00_rb, &
     &1.4515e+00_rb,7.2651e-01_rb,1.6183e-04_rb/)
      kao(:, 3,12, 1) = (/ &
     &6.7955e+00_rb,5.9463e+00_rb,5.0972e+00_rb,4.2481e+00_rb,3.3989e+00_rb,2.5498e+00_rb, &
     &1.7005e+00_rb,8.5109e-01_rb,1.9622e-04_rb/)
      kao(:, 4,12, 1) = (/ &
     &7.7871e+00_rb,6.8141e+00_rb,5.8412e+00_rb,4.8680e+00_rb,3.8949e+00_rb,2.9218e+00_rb, &
     &1.9487e+00_rb,9.7554e-01_rb,2.4527e-04_rb/)
      kao(:, 5,12, 1) = (/ &
     &8.7731e+00_rb,7.6770e+00_rb,6.5805e+00_rb,5.4843e+00_rb,4.3883e+00_rb,3.2921e+00_rb, &
     &2.1959e+00_rb,1.0994e+00_rb,2.9421e-04_rb/)
      kao(:, 1,13, 1) = (/ &
     &4.1936e+00_rb,3.6695e+00_rb,3.1456e+00_rb,2.6217e+00_rb,2.0976e+00_rb,1.5735e+00_rb, &
     &1.0495e+00_rb,5.2527e-01_rb,1.1808e-04_rb/)
      kao(:, 2,13, 1) = (/ &
     &5.0398e+00_rb,4.4101e+00_rb,3.7804e+00_rb,3.1506e+00_rb,2.5209e+00_rb,1.8912e+00_rb, &
     &1.2614e+00_rb,6.3136e-01_rb,1.5519e-04_rb/)
      kao(:, 3,13, 1) = (/ &
     &5.9030e+00_rb,5.1655e+00_rb,4.4278e+00_rb,3.6903e+00_rb,2.9527e+00_rb,2.2152e+00_rb, &
     &1.4774e+00_rb,7.3950e-01_rb,1.9283e-04_rb/)
      kao(:, 4,13, 1) = (/ &
     &6.7598e+00_rb,5.9151e+00_rb,5.0706e+00_rb,4.2260e+00_rb,3.3815e+00_rb,2.5368e+00_rb, &
     &1.6919e+00_rb,8.4706e-01_rb,2.3479e-04_rb/)
      kao(:, 5,13, 1) = (/ &
     &7.6160e+00_rb,6.6645e+00_rb,5.7130e+00_rb,4.7615e+00_rb,3.8100e+00_rb,2.8583e+00_rb, &
     &1.9066e+00_rb,9.5472e-01_rb,2.9034e-04_rb/)
      kao(:, 1, 1, 2) = (/ &
     &2.4501e-02_rb,2.2546e-02_rb,2.0206e-02_rb,1.7580e-02_rb,1.4733e-02_rb,1.1757e-02_rb, &
     &8.5871e-03_rb,4.9715e-03_rb,4.1358e-04_rb/)
      kao(:, 2, 1, 2) = (/ &
     &2.5392e-02_rb,2.3504e-02_rb,2.1061e-02_rb,1.8282e-02_rb,1.5340e-02_rb,1.2238e-02_rb, &
     &8.8921e-03_rb,5.1114e-03_rb,3.8966e-04_rb/)
      kao(:, 3, 1, 2) = (/ &
     &2.6027e-02_rb,2.4221e-02_rb,2.1673e-02_rb,1.8830e-02_rb,1.5807e-02_rb,1.2525e-02_rb, &
     &9.1668e-03_rb,5.5997e-03_rb,3.7711e-04_rb/)
      kao(:, 4, 1, 2) = (/ &
     &2.6673e-02_rb,2.4919e-02_rb,2.2282e-02_rb,1.9349e-02_rb,1.6139e-02_rb,1.2835e-02_rb, &
     &9.4247e-03_rb,5.9143e-03_rb,4.3807e-04_rb/)
      kao(:, 5, 1, 2) = (/ &
     &2.7193e-02_rb,2.5447e-02_rb,2.2733e-02_rb,1.9712e-02_rb,1.6469e-02_rb,1.3190e-02_rb, &
     &9.7078e-03_rb,6.2746e-03_rb,5.1500e-04_rb/)
      kao(:, 1, 2, 2) = (/ &
     &3.1935e-02_rb,2.8971e-02_rb,2.5689e-02_rb,2.2139e-02_rb,1.8357e-02_rb,1.4398e-02_rb, &
     &1.0314e-02_rb,5.7860e-03_rb,2.7704e-04_rb/)
      kao(:, 2, 2, 2) = (/ &
     &3.3322e-02_rb,3.0388e-02_rb,2.6935e-02_rb,2.3180e-02_rb,1.9196e-02_rb,1.5076e-02_rb, &
     &1.0778e-02_rb,6.0195e-03_rb,2.7933e-04_rb/)
      kao(:, 3, 2, 2) = (/ &
     &3.4372e-02_rb,3.1484e-02_rb,2.7888e-02_rb,2.3966e-02_rb,1.9860e-02_rb,1.5585e-02_rb, &
     &1.1079e-02_rb,6.3448e-03_rb,2.7118e-04_rb/)
      kao(:, 4, 2, 2) = (/ &
     &3.5247e-02_rb,3.2382e-02_rb,2.8654e-02_rb,2.4627e-02_rb,2.0404e-02_rb,1.5949e-02_rb, &
     &1.1392e-02_rb,6.6539e-03_rb,3.3360e-04_rb/)
      kao(:, 5, 2, 2) = (/ &
     &3.6022e-02_rb,3.3132e-02_rb,2.9304e-02_rb,2.5200e-02_rb,2.0861e-02_rb,1.6331e-02_rb, &
     &1.1785e-02_rb,6.9927e-03_rb,3.9477e-04_rb/)
      kao(:, 1, 3, 2) = (/ &
     &5.5981e-02_rb,4.9895e-02_rb,4.3680e-02_rb,3.7190e-02_rb,3.0485e-02_rb,2.3562e-02_rb, &
     &1.6425e-02_rb,9.0449e-03_rb,1.7015e-04_rb/)
      kao(:, 2, 3, 2) = (/ &
     &5.8799e-02_rb,5.2631e-02_rb,4.6091e-02_rb,3.9226e-02_rb,3.2137e-02_rb,2.4836e-02_rb, &
     &1.7301e-02_rb,9.5109e-03_rb,1.8305e-04_rb/)
      kao(:, 3, 3, 2) = (/ &
     &6.0996e-02_rb,5.4813e-02_rb,4.8006e-02_rb,4.0820e-02_rb,3.3414e-02_rb,2.5787e-02_rb, &
     &1.7985e-02_rb,9.8187e-03_rb,1.8905e-04_rb/)
      kao(:, 4, 3, 2) = (/ &
     &6.2707e-02_rb,5.6481e-02_rb,4.9388e-02_rb,4.1983e-02_rb,3.4344e-02_rb,2.6526e-02_rb, &
     &1.8461e-02_rb,1.0117e-02_rb,2.4096e-04_rb/)
      kao(:, 5, 3, 2) = (/ &
     &6.4195e-02_rb,5.7890e-02_rb,5.0593e-02_rb,4.2977e-02_rb,3.5166e-02_rb,2.7170e-02_rb, &
     &1.8910e-02_rb,1.0511e-02_rb,2.9172e-04_rb/)
      kao(:, 1, 4, 2) = (/ &
     &1.0046e-01_rb,8.8696e-02_rb,7.6950e-02_rb,6.5010e-02_rb,5.2854e-02_rb,4.0445e-02_rb, &
     &2.7753e-02_rb,1.4715e-02_rb,1.1527e-04_rb/)
      kao(:, 2, 4, 2) = (/ &
     &1.0584e-01_rb,9.3644e-02_rb,8.1343e-02_rb,6.8725e-02_rb,5.5817e-02_rb,4.2663e-02_rb, &
     &2.9258e-02_rb,1.5519e-02_rb,1.2102e-04_rb/)
      kao(:, 3, 4, 2) = (/ &
     &1.1041e-01_rb,9.7926e-02_rb,8.5123e-02_rb,7.1882e-02_rb,5.8341e-02_rb,4.4558e-02_rb, &
     &3.0554e-02_rb,1.6224e-02_rb,1.3450e-04_rb/)
      kao(:, 4, 4, 2) = (/ &
     &1.1403e-01_rb,1.0136e-01_rb,8.8078e-02_rb,7.4319e-02_rb,6.0266e-02_rb,4.6028e-02_rb, &
     &3.1549e-02_rb,1.6730e-02_rb,1.7483e-04_rb/)
      kao(:, 5, 4, 2) = (/ &
     &1.1675e-01_rb,1.0391e-01_rb,9.0226e-02_rb,7.6062e-02_rb,6.1699e-02_rb,4.7119e-02_rb, &
     &3.2315e-02_rb,1.7160e-02_rb,2.1415e-04_rb/)
      kao(:, 1, 5, 2) = (/ &
     &1.6440e-01_rb,1.4448e-01_rb,1.2464e-01_rb,1.0475e-01_rb,8.4706e-02_rb,6.4458e-02_rb, &
     &4.3883e-02_rb,2.2824e-02_rb,7.5213e-05_rb/)
      kao(:, 2, 5, 2) = (/ &
     &1.7418e-01_rb,1.5330e-01_rb,1.3243e-01_rb,1.1140e-01_rb,9.0109e-02_rb,6.8519e-02_rb, &
     &4.6576e-02_rb,2.4202e-02_rb,8.0510e-05_rb/)
      kao(:, 3, 5, 2) = (/ &
     &1.8254e-01_rb,1.6088e-01_rb,1.3912e-01_rb,1.1702e-01_rb,9.4586e-02_rb,7.1837e-02_rb, &
     &4.8767e-02_rb,2.5350e-02_rb,9.6196e-05_rb/)
      kao(:, 4, 5, 2) = (/ &
     &1.8937e-01_rb,1.6713e-01_rb,1.4454e-01_rb,1.2155e-01_rb,9.8195e-02_rb,7.4514e-02_rb, &
     &5.0569e-02_rb,2.6308e-02_rb,1.2696e-04_rb/)
      kao(:, 5, 5, 2) = (/ &
     &1.9452e-01_rb,1.7183e-01_rb,1.4864e-01_rb,1.2495e-01_rb,1.0086e-01_rb,7.6520e-02_rb, &
     &5.1964e-02_rb,2.7040e-02_rb,1.5611e-04_rb/)
      kao(:, 1, 6, 2) = (/ &
     &2.4921e-01_rb,2.1859e-01_rb,1.8798e-01_rb,1.5740e-01_rb,1.2679e-01_rb,9.6030e-02_rb, &
     &6.5011e-02_rb,3.3513e-02_rb,4.7816e-05_rb/)
      kao(:, 2, 6, 2) = (/ &
     &2.6657e-01_rb,2.3398e-01_rb,2.0147e-01_rb,1.6890e-01_rb,1.3619e-01_rb,1.0322e-01_rb, &
     &6.9866e-02_rb,3.5915e-02_rb,5.5771e-05_rb/)
      kao(:, 3, 6, 2) = (/ &
     &2.8078e-01_rb,2.4667e-01_rb,2.1258e-01_rb,1.7831e-01_rb,1.4376e-01_rb,1.0887e-01_rb, &
     &7.3553e-02_rb,3.7752e-02_rb,7.2324e-05_rb/)
      kao(:, 4, 6, 2) = (/ &
     &2.9281e-01_rb,2.5752e-01_rb,2.2202e-01_rb,1.8622e-01_rb,1.5008e-01_rb,1.1356e-01_rb, &
     &7.6693e-02_rb,3.9339e-02_rb,9.1831e-05_rb/)
      kao(:, 5, 6, 2) = (/ &
     &3.0221e-01_rb,2.6598e-01_rb,2.2929e-01_rb,1.9231e-01_rb,1.5490e-01_rb,1.1717e-01_rb, &
     &7.9072e-02_rb,4.0614e-02_rb,1.1670e-04_rb/)
      kao(:, 1, 7, 2) = (/ &
     &3.9471e-01_rb,3.4576e-01_rb,2.9687e-01_rb,2.4797e-01_rb,1.9909e-01_rb,1.5016e-01_rb, &
     &1.0112e-01_rb,5.1719e-02_rb,4.5280e-05_rb/)
      kao(:, 2, 7, 2) = (/ &
     &4.2660e-01_rb,3.7391e-01_rb,3.2118e-01_rb,2.6853e-01_rb,2.1577e-01_rb,1.6297e-01_rb, &
     &1.0986e-01_rb,5.6145e-02_rb,5.4459e-05_rb/)
      kao(:, 3, 7, 2) = (/ &
     &4.5301e-01_rb,3.9723e-01_rb,3.4149e-01_rb,2.8568e-01_rb,2.2975e-01_rb,1.7359e-01_rb, &
     &1.1695e-01_rb,5.9583e-02_rb,6.4819e-05_rb/)
      kao(:, 4, 7, 2) = (/ &
     &4.7467e-01_rb,4.1649e-01_rb,3.5820e-01_rb,2.9973e-01_rb,2.4109e-01_rb,1.8208e-01_rb, &
     &1.2251e-01_rb,6.2380e-02_rb,7.8660e-05_rb/)
      kao(:, 5, 7, 2) = (/ &
     &4.9199e-01_rb,4.3192e-01_rb,3.7144e-01_rb,3.1090e-01_rb,2.5000e-01_rb,1.8865e-01_rb, &
     &1.2686e-01_rb,6.4563e-02_rb,9.4184e-05_rb/)
      kao(:, 1, 8, 2) = (/ &
     &7.4598e-01_rb,6.5300e-01_rb,5.6009e-01_rb,4.6718e-01_rb,3.7433e-01_rb,2.8142e-01_rb, &
     &1.8848e-01_rb,9.5403e-02_rb,4.9176e-05_rb/)
      kao(:, 2, 8, 2) = (/ &
     &8.1434e-01_rb,7.1301e-01_rb,6.1173e-01_rb,5.1044e-01_rb,4.0917e-01_rb,3.0788e-01_rb, &
     &2.0653e-01_rb,1.0477e-01_rb,6.1468e-05_rb/)
      kao(:, 3, 8, 2) = (/ &
     &8.7268e-01_rb,7.6440e-01_rb,6.5604e-01_rb,5.4759e-01_rb,4.3920e-01_rb,3.3065e-01_rb, &
     &2.2200e-01_rb,1.1269e-01_rb,7.5902e-05_rb/)
      kao(:, 4, 8, 2) = (/ &
     &9.2025e-01_rb,8.0619e-01_rb,6.9208e-01_rb,5.7794e-01_rb,4.6363e-01_rb,3.4928e-01_rb, &
     &2.3455e-01_rb,1.1876e-01_rb,9.2466e-05_rb/)
      kao(:, 5, 8, 2) = (/ &
     &9.5892e-01_rb,8.4025e-01_rb,7.2136e-01_rb,6.0245e-01_rb,4.8344e-01_rb,3.6420e-01_rb, &
     &2.4431e-01_rb,1.2353e-01_rb,1.1186e-04_rb/)
      kao(:, 1, 9, 2) = (/ &
     &2.6457e+00_rb,2.3152e+00_rb,1.9846e+00_rb,1.6541e+00_rb,1.3236e+00_rb,9.9317e-01_rb, &
     &6.6276e-01_rb,3.3240e-01_rb,8.9892e-05_rb/)
      kao(:, 2, 9, 2) = (/ &
     &2.9186e+00_rb,2.5541e+00_rb,2.1896e+00_rb,1.8252e+00_rb,1.4607e+00_rb,1.0963e+00_rb, &
     &7.3190e-01_rb,3.6751e-01_rb,1.0914e-04_rb/)
      kao(:, 3, 9, 2) = (/ &
     &3.1587e+00_rb,2.7643e+00_rb,2.3700e+00_rb,1.9759e+00_rb,1.5816e+00_rb,1.1873e+00_rb, &
     &7.9301e-01_rb,3.9869e-01_rb,1.2624e-04_rb/)
      kao(:, 4, 9, 2) = (/ &
     &3.3556e+00_rb,2.9370e+00_rb,2.5183e+00_rb,2.0996e+00_rb,1.6808e+00_rb,1.2622e+00_rb, &
     &8.4339e-01_rb,4.2436e-01_rb,1.5596e-04_rb/)
      kao(:, 5, 9, 2) = (/ &
     &3.5177e+00_rb,3.0788e+00_rb,2.6401e+00_rb,2.2013e+00_rb,1.7625e+00_rb,1.3234e+00_rb, &
     &8.8445e-01_rb,4.4528e-01_rb,1.9369e-04_rb/)
      kao(:, 1,10, 2) = (/ &
     &1.0501e+01_rb,9.1884e+00_rb,7.8763e+00_rb,6.5637e+00_rb,5.2509e+00_rb,3.9387e+00_rb, &
     &2.6260e+00_rb,1.3137e+00_rb,1.7951e-04_rb/)
      kao(:, 2,10, 2) = (/ &
     &1.1694e+01_rb,1.0233e+01_rb,8.7714e+00_rb,7.3095e+00_rb,5.8477e+00_rb,4.3863e+00_rb, &
     &2.9249e+00_rb,1.4636e+00_rb,2.1880e-04_rb/)
      kao(:, 3,10, 2) = (/ &
     &1.2771e+01_rb,1.1175e+01_rb,9.5787e+00_rb,7.9832e+00_rb,6.3867e+00_rb,4.7910e+00_rb, &
     &3.1952e+00_rb,1.5993e+00_rb,2.5504e-04_rb/)
      kao(:, 4,10, 2) = (/ &
     &1.3661e+01_rb,1.1954e+01_rb,1.0247e+01_rb,8.5403e+00_rb,6.8331e+00_rb,5.1260e+00_rb, &
     &3.4191e+00_rb,1.7121e+00_rb,3.1556e-04_rb/)
      kao(:, 5,10, 2) = (/ &
     &1.4403e+01_rb,1.2603e+01_rb,1.0804e+01_rb,9.0041e+00_rb,7.2046e+00_rb,5.4047e+00_rb, &
     &3.6050e+00_rb,1.8055e+00_rb,3.5095e-04_rb/)
      kao(:, 1,11, 2) = (/ &
     &1.5305e+01_rb,1.3391e+01_rb,1.1479e+01_rb,9.5654e+00_rb,7.6527e+00_rb,5.7399e+00_rb, &
     &3.8269e+00_rb,1.9139e+00_rb,2.5263e-04_rb/)
      kao(:, 2,11, 2) = (/ &
     &1.7053e+01_rb,1.4921e+01_rb,1.2790e+01_rb,1.0658e+01_rb,8.5272e+00_rb,6.3954e+00_rb, &
     &4.2646e+00_rb,2.1334e+00_rb,3.2671e-04_rb/)
      kao(:, 3,11, 2) = (/ &
     &1.8597e+01_rb,1.6273e+01_rb,1.3949e+01_rb,1.1625e+01_rb,9.3007e+00_rb,6.9759e+00_rb, &
     &4.6520e+00_rb,2.3277e+00_rb,3.9618e-04_rb/)
      kao(:, 4,11, 2) = (/ &
     &1.9894e+01_rb,1.7409e+01_rb,1.4922e+01_rb,1.2436e+01_rb,9.9497e+00_rb,7.4632e+00_rb, &
     &4.9771e+00_rb,2.4907e+00_rb,4.6029e-04_rb/)
      kao(:, 5,11, 2) = (/ &
     &2.0957e+01_rb,1.8337e+01_rb,1.5718e+01_rb,1.3099e+01_rb,1.0480e+01_rb,7.8611e+00_rb, &
     &5.2427e+00_rb,2.6238e+00_rb,5.1000e-04_rb/)
      kao(:, 1,12, 2) = (/ &
     &1.6595e+01_rb,1.4521e+01_rb,1.2446e+01_rb,1.0372e+01_rb,8.2975e+00_rb,6.2234e+00_rb, &
     &4.1492e+00_rb,2.0749e+00_rb,2.8721e-04_rb/)
      kao(:, 2,12, 2) = (/ &
     &1.8487e+01_rb,1.6178e+01_rb,1.3866e+01_rb,1.1556e+01_rb,9.2444e+00_rb,6.9339e+00_rb, &
     &4.6231e+00_rb,2.3126e+00_rb,3.5210e-04_rb/)
      kao(:, 3,12, 2) = (/ &
     &2.0157e+01_rb,1.7638e+01_rb,1.5119e+01_rb,1.2600e+01_rb,1.0080e+01_rb,7.5607e+00_rb, &
     &5.0414e+00_rb,2.5223e+00_rb,4.4248e-04_rb/)
      kao(:, 4,12, 2) = (/ &
     &2.1568e+01_rb,1.8872e+01_rb,1.6176e+01_rb,1.3480e+01_rb,1.0785e+01_rb,8.0897e+00_rb, &
     &5.3945e+00_rb,2.6993e+00_rb,5.0244e-04_rb/)
      kao(:, 5,12, 2) = (/ &
     &2.2671e+01_rb,1.9838e+01_rb,1.7004e+01_rb,1.4170e+01_rb,1.1337e+01_rb,8.5042e+00_rb, &
     &5.6711e+00_rb,2.8381e+00_rb,5.5567e-04_rb/)
      kao(:, 1,13, 2) = (/ &
     &1.4451e+01_rb,1.2644e+01_rb,1.0838e+01_rb,9.0321e+00_rb,7.2259e+00_rb,5.4197e+00_rb, &
     &3.6133e+00_rb,1.8070e+00_rb,2.7932e-04_rb/)
      kao(:, 2,13, 2) = (/ &
     &1.6097e+01_rb,1.4085e+01_rb,1.2073e+01_rb,1.0061e+01_rb,8.0489e+00_rb,6.0372e+00_rb, &
     &4.0253e+00_rb,2.0137e+00_rb,3.2799e-04_rb/)
      kao(:, 3,13, 2) = (/ &
     &1.7564e+01_rb,1.5369e+01_rb,1.3173e+01_rb,1.0978e+01_rb,8.7826e+00_rb,6.5876e+00_rb, &
     &4.3925e+00_rb,2.1975e+00_rb,4.0780e-04_rb/)
      kao(:, 4,13, 2) = (/ &
     &1.8771e+01_rb,1.6426e+01_rb,1.4079e+01_rb,1.1733e+01_rb,9.3868e+00_rb,7.0409e+00_rb, &
     &4.6954e+00_rb,2.3495e+00_rb,5.0230e-04_rb/)
      kao(:, 5,13, 2) = (/ &
     &1.9694e+01_rb,1.7232e+01_rb,1.4771e+01_rb,1.2310e+01_rb,9.8489e+00_rb,7.3875e+00_rb, &
     &4.9264e+00_rb,2.4656e+00_rb,5.5102e-04_rb/)
      kao(:, 1, 1, 3) = (/ &
     &3.7564e-02_rb,3.3492e-02_rb,2.9385e-02_rb,2.5064e-02_rb,2.0679e-02_rb,1.6159e-02_rb, &
     &1.1502e-02_rb,6.5367e-03_rb,3.9794e-04_rb/)
      kao(:, 2, 1, 3) = (/ &
     &3.6916e-02_rb,3.3151e-02_rb,2.9160e-02_rb,2.5018e-02_rb,2.0751e-02_rb,1.6389e-02_rb, &
     &1.1901e-02_rb,7.2678e-03_rb,5.4686e-04_rb/)
      kao(:, 3, 1, 3) = (/ &
     &3.6399e-02_rb,3.2903e-02_rb,2.9031e-02_rb,2.5012e-02_rb,2.0900e-02_rb,1.6718e-02_rb, &
     &1.2389e-02_rb,8.0801e-03_rb,7.4220e-04_rb/)
      kao(:, 4, 1, 3) = (/ &
     &3.6126e-02_rb,3.2782e-02_rb,2.9047e-02_rb,2.5198e-02_rb,2.1279e-02_rb,1.7152e-02_rb, &
     &1.3045e-02_rb,8.9482e-03_rb,8.9478e-04_rb/)
      kao(:, 5, 1, 3) = (/ &
     &3.5996e-02_rb,3.2833e-02_rb,2.9270e-02_rb,2.5593e-02_rb,2.1712e-02_rb,1.7723e-02_rb, &
     &1.3812e-02_rb,9.7243e-03_rb,1.0975e-03_rb/)
      kao(:, 1, 2, 3) = (/ &
     &4.9545e-02_rb,4.3840e-02_rb,3.8273e-02_rb,3.2472e-02_rb,2.6558e-02_rb,2.0510e-02_rb, &
     &1.4332e-02_rb,7.7992e-03_rb,3.0769e-04_rb/)
      kao(:, 2, 2, 3) = (/ &
     &4.8709e-02_rb,4.3359e-02_rb,3.7969e-02_rb,3.2304e-02_rb,2.6513e-02_rb,2.0603e-02_rb, &
     &1.4626e-02_rb,8.4051e-03_rb,4.1476e-04_rb/)
      kao(:, 3, 2, 3) = (/ &
     &4.8165e-02_rb,4.3120e-02_rb,3.7795e-02_rb,3.2248e-02_rb,2.6570e-02_rb,2.0838e-02_rb, &
     &1.4977e-02_rb,8.9685e-03_rb,5.6832e-04_rb/)
      kao(:, 4, 2, 3) = (/ &
     &4.7962e-02_rb,4.3077e-02_rb,3.7844e-02_rb,3.2393e-02_rb,2.6873e-02_rb,2.1238e-02_rb, &
     &1.5407e-02_rb,9.6431e-03_rb,6.7745e-04_rb/)
      kao(:, 5, 2, 3) = (/ &
     &4.8025e-02_rb,4.3378e-02_rb,3.8202e-02_rb,3.2847e-02_rb,2.7374e-02_rb,2.1742e-02_rb, &
     &1.5999e-02_rb,1.0391e-02_rb,8.3539e-04_rb/)
      kao(:, 1, 3, 3) = (/ &
     &8.8120e-02_rb,7.7386e-02_rb,6.6977e-02_rb,5.6500e-02_rb,4.5838e-02_rb,3.5027e-02_rb, &
     &2.3995e-02_rb,1.2704e-02_rb,2.2676e-04_rb/)
      kao(:, 2, 3, 3) = (/ &
     &8.6799e-02_rb,7.6419e-02_rb,6.6419e-02_rb,5.6119e-02_rb,4.5623e-02_rb,3.4916e-02_rb, &
     &2.4025e-02_rb,1.3004e-02_rb,3.0168e-04_rb/)
      kao(:, 3, 3, 3) = (/ &
     &8.6001e-02_rb,7.6005e-02_rb,6.6184e-02_rb,5.6008e-02_rb,4.5591e-02_rb,3.4990e-02_rb, &
     &2.4263e-02_rb,1.3424e-02_rb,3.9676e-04_rb/)
      kao(:, 4, 3, 3) = (/ &
     &8.5691e-02_rb,7.5994e-02_rb,6.6270e-02_rb,5.6158e-02_rb,4.5788e-02_rb,3.5296e-02_rb, &
     &2.4707e-02_rb,1.3830e-02_rb,4.9186e-04_rb/)
      kao(:, 5, 3, 3) = (/ &
     &8.6172e-02_rb,7.6716e-02_rb,6.6965e-02_rb,5.6820e-02_rb,4.6489e-02_rb,3.5977e-02_rb, &
     &2.5332e-02_rb,1.4387e-02_rb,6.1173e-04_rb/)
      kao(:, 1, 4, 3) = (/ &
     &1.6100e-01_rb,1.4106e-01_rb,1.2117e-01_rb,1.0160e-01_rb,8.1953e-02_rb,6.2189e-02_rb, &
     &4.2184e-02_rb,2.1860e-02_rb,1.5920e-04_rb/)
      kao(:, 2, 4, 3) = (/ &
     &1.5854e-01_rb,1.3897e-01_rb,1.1977e-01_rb,1.0067e-01_rb,8.1408e-02_rb,6.1866e-02_rb, &
     &4.2083e-02_rb,2.1896e-02_rb,2.1901e-04_rb/)
      kao(:, 3, 4, 3) = (/ &
     &1.5713e-01_rb,1.3793e-01_rb,1.1924e-01_rb,1.0046e-01_rb,8.1310e-02_rb,6.1926e-02_rb, &
     &4.2176e-02_rb,2.2134e-02_rb,2.8281e-04_rb/)
      kao(:, 4, 4, 3) = (/ &
     &1.5680e-01_rb,1.3795e-01_rb,1.1956e-01_rb,1.0071e-01_rb,8.1588e-02_rb,6.2136e-02_rb, &
     &4.2469e-02_rb,2.2574e-02_rb,3.5509e-04_rb/)
      kao(:, 5, 4, 3) = (/ &
     &1.5807e-01_rb,1.3940e-01_rb,1.2093e-01_rb,1.0191e-01_rb,8.2580e-02_rb,6.3036e-02_rb, &
     &4.3252e-02_rb,2.3169e-02_rb,4.4437e-04_rb/)
      kao(:, 1, 5, 3) = (/ &
     &2.7295e-01_rb,2.3896e-01_rb,2.0497e-01_rb,1.7107e-01_rb,1.3742e-01_rb,1.0372e-01_rb, &
     &6.9897e-02_rb,3.5756e-02_rb,1.1345e-04_rb/)
      kao(:, 2, 5, 3) = (/ &
     &2.6909e-01_rb,2.3567e-01_rb,2.0228e-01_rb,1.6922e-01_rb,1.3618e-01_rb,1.0303e-01_rb, &
     &6.9598e-02_rb,3.5741e-02_rb,1.5887e-04_rb/)
      kao(:, 3, 5, 3) = (/ &
     &2.6654e-01_rb,2.3348e-01_rb,2.0077e-01_rb,1.6833e-01_rb,1.3583e-01_rb,1.0288e-01_rb, &
     &6.9627e-02_rb,3.5834e-02_rb,2.0432e-04_rb/)
      kao(:, 4, 5, 3) = (/ &
     &2.6633e-01_rb,2.3346e-01_rb,2.0125e-01_rb,1.6903e-01_rb,1.3632e-01_rb,1.0334e-01_rb, &
     &6.9968e-02_rb,3.6152e-02_rb,2.5784e-04_rb/)
      kao(:, 5, 5, 3) = (/ &
     &2.6868e-01_rb,2.3592e-01_rb,2.0366e-01_rb,1.7109e-01_rb,1.3803e-01_rb,1.0467e-01_rb, &
     &7.0932e-02_rb,3.6877e-02_rb,3.2436e-04_rb/)
      kao(:, 1, 6, 3) = (/ &
     &4.3357e-01_rb,3.7951e-01_rb,3.2542e-01_rb,2.7129e-01_rb,2.1728e-01_rb,1.6346e-01_rb, &
     &1.0964e-01_rb,5.5662e-02_rb,8.3325e-05_rb/)
      kao(:, 2, 6, 3) = (/ &
     &4.2794e-01_rb,3.7459e-01_rb,3.2123e-01_rb,2.6798e-01_rb,2.1494e-01_rb,1.6197e-01_rb, &
     &1.0894e-01_rb,5.5488e-02_rb,1.1433e-04_rb/)
      kao(:, 3, 6, 3) = (/ &
     &4.2362e-01_rb,3.7090e-01_rb,3.1822e-01_rb,2.6585e-01_rb,2.1359e-01_rb,1.6133e-01_rb, &
     &1.0870e-01_rb,5.5444e-02_rb,1.4294e-04_rb/)
      kao(:, 4, 6, 3) = (/ &
     &4.2356e-01_rb,3.7092e-01_rb,3.1860e-01_rb,2.6656e-01_rb,2.1457e-01_rb,1.6217e-01_rb, &
     &1.0928e-01_rb,5.5858e-02_rb,1.8487e-04_rb/)
      kao(:, 5, 6, 3) = (/ &
     &4.2732e-01_rb,3.7444e-01_rb,3.2213e-01_rb,2.6992e-01_rb,2.1742e-01_rb,1.6433e-01_rb, &
     &1.1084e-01_rb,5.6756e-02_rb,2.3367e-04_rb/)
      kao(:, 1, 7, 3) = (/ &
     &7.2201e-01_rb,6.3185e-01_rb,5.4168e-01_rb,4.5148e-01_rb,3.6131e-01_rb,2.7123e-01_rb, &
     &1.8131e-01_rb,9.1347e-02_rb,6.1051e-05_rb/)
      kao(:, 2, 7, 3) = (/ &
     &7.1295e-01_rb,6.2391e-01_rb,5.3487e-01_rb,4.4585e-01_rb,3.5694e-01_rb,2.6824e-01_rb, &
     &1.7961e-01_rb,9.0876e-02_rb,8.2662e-05_rb/)
      kao(:, 3, 7, 3) = (/ &
     &7.0704e-01_rb,6.1882e-01_rb,5.3061e-01_rb,4.4252e-01_rb,3.5463e-01_rb,2.6681e-01_rb, &
     &1.7908e-01_rb,9.0926e-02_rb,1.0796e-04_rb/)
      kao(:, 4, 7, 3) = (/ &
     &7.0657e-01_rb,6.1841e-01_rb,5.3039e-01_rb,4.4278e-01_rb,3.5513e-01_rb,2.6769e-01_rb, &
     &1.7995e-01_rb,9.1305e-02_rb,1.3920e-04_rb/)
      kao(:, 5, 7, 3) = (/ &
     &7.1310e-01_rb,6.2430e-01_rb,5.3600e-01_rb,4.4782e-01_rb,3.5976e-01_rb,2.7158e-01_rb, &
     &1.8261e-01_rb,9.2814e-02_rb,1.7572e-04_rb/)
      kao(:, 1, 8, 3) = (/ &
     &1.4344e+00_rb,1.2551e+00_rb,1.0759e+00_rb,8.9667e-01_rb,7.1735e-01_rb,5.3821e-01_rb, &
     &3.5899e-01_rb,1.8011e-01_rb,7.5748e-05_rb/)
      kao(:, 2, 8, 3) = (/ &
     &1.4191e+00_rb,1.2419e+00_rb,1.0645e+00_rb,8.8713e-01_rb,7.0979e-01_rb,5.3268e-01_rb, &
     &3.5557e-01_rb,1.7867e-01_rb,9.9921e-05_rb/)
      kao(:, 3, 8, 3) = (/ &
     &1.4086e+00_rb,1.2327e+00_rb,1.0566e+00_rb,8.8065e-01_rb,7.0478e-01_rb,5.2920e-01_rb, &
     &3.5364e-01_rb,1.7813e-01_rb,1.2236e-04_rb/)
      kao(:, 4, 8, 3) = (/ &
     &1.4087e+00_rb,1.2327e+00_rb,1.0568e+00_rb,8.8106e-01_rb,7.0553e-01_rb,5.2998e-01_rb, &
     &3.5460e-01_rb,1.7919e-01_rb,1.5414e-04_rb/)
      kao(:, 5, 8, 3) = (/ &
     &1.4217e+00_rb,1.2444e+00_rb,1.0672e+00_rb,8.9025e-01_rb,7.1327e-01_rb,5.3627e-01_rb, &
     &3.5965e-01_rb,1.8202e-01_rb,1.9115e-04_rb/)
      kao(:, 1, 9, 3) = (/ &
     &5.3656e+00_rb,4.6949e+00_rb,4.0244e+00_rb,3.3536e+00_rb,2.6829e+00_rb,2.0124e+00_rb, &
     &1.3418e+00_rb,6.7121e-01_rb,1.2938e-04_rb/)
      kao(:, 2, 9, 3) = (/ &
     &5.3249e+00_rb,4.6593e+00_rb,3.9938e+00_rb,3.3284e+00_rb,2.6628e+00_rb,1.9972e+00_rb, &
     &1.3317e+00_rb,6.6623e-01_rb,1.6248e-04_rb/)
      kao(:, 3, 9, 3) = (/ &
     &5.2842e+00_rb,4.6238e+00_rb,3.9633e+00_rb,3.3029e+00_rb,2.6426e+00_rb,1.9822e+00_rb, &
     &1.3219e+00_rb,6.6174e-01_rb,2.1505e-04_rb/)
      kao(:, 4, 9, 3) = (/ &
     &5.2909e+00_rb,4.6297e+00_rb,3.9685e+00_rb,3.3074e+00_rb,2.6461e+00_rb,1.9850e+00_rb, &
     &1.3243e+00_rb,6.6344e-01_rb,2.5323e-04_rb/)
      kao(:, 5, 9, 3) = (/ &
     &5.3328e+00_rb,4.6665e+00_rb,4.0003e+00_rb,3.3342e+00_rb,2.6682e+00_rb,2.0021e+00_rb, &
     &1.3361e+00_rb,6.6998e-01_rb,2.8724e-04_rb/)
      kao(:, 1,10, 3) = (/ &
     &2.2338e+01_rb,1.9545e+01_rb,1.6753e+01_rb,1.3961e+01_rb,1.1170e+01_rb,8.3767e+00_rb, &
     &5.5849e+00_rb,2.7925e+00_rb,2.7123e-04_rb/)
      kao(:, 2,10, 3) = (/ &
     &2.2247e+01_rb,1.9467e+01_rb,1.6686e+01_rb,1.3905e+01_rb,1.1124e+01_rb,8.3431e+00_rb, &
     &5.5623e+00_rb,2.7817e+00_rb,3.3076e-04_rb/)
      kao(:, 3,10, 3) = (/ &
     &2.2079e+01_rb,1.9319e+01_rb,1.6560e+01_rb,1.3800e+01_rb,1.1040e+01_rb,8.2801e+00_rb, &
     &5.5205e+00_rb,2.7607e+00_rb,4.2235e-04_rb/)
      kao(:, 4,10, 3) = (/ &
     &2.2133e+01_rb,1.9366e+01_rb,1.6600e+01_rb,1.3833e+01_rb,1.1067e+01_rb,8.3005e+00_rb, &
     &5.5343e+00_rb,2.7680e+00_rb,4.8628e-04_rb/)
      kao(:, 5,10, 3) = (/ &
     &2.2337e+01_rb,1.9545e+01_rb,1.6754e+01_rb,1.3962e+01_rb,1.1171e+01_rb,8.3788e+00_rb, &
     &5.5873e+00_rb,2.7956e+00_rb,6.0636e-04_rb/)
      kao(:, 1,11, 3) = (/ &
     &3.2372e+01_rb,2.8324e+01_rb,2.4279e+01_rb,2.0232e+01_rb,1.6186e+01_rb,1.2140e+01_rb, &
     &8.0929e+00_rb,4.0469e+00_rb,3.8659e-04_rb/)
      kao(:, 2,11, 3) = (/ &
     &3.2225e+01_rb,2.8197e+01_rb,2.4169e+01_rb,2.0141e+01_rb,1.6113e+01_rb,1.2085e+01_rb, &
     &8.0568e+00_rb,4.0284e+00_rb,4.6837e-04_rb/)
      kao(:, 3,11, 3) = (/ &
     &3.2089e+01_rb,2.8078e+01_rb,2.4068e+01_rb,2.0057e+01_rb,1.6046e+01_rb,1.2035e+01_rb, &
     &8.0232e+00_rb,4.0121e+00_rb,5.6676e-04_rb/)
      kao(:, 4,11, 3) = (/ &
     &3.2281e+01_rb,2.8246e+01_rb,2.4212e+01_rb,2.0176e+01_rb,1.6141e+01_rb,1.2106e+01_rb, &
     &8.0716e+00_rb,4.0370e+00_rb,7.3031e-04_rb/)
      kao(:, 5,11, 3) = (/ &
     &3.2736e+01_rb,2.8646e+01_rb,2.4553e+01_rb,2.0462e+01_rb,1.6370e+01_rb,1.2278e+01_rb, &
     &8.1864e+00_rb,4.0949e+00_rb,8.3516e-04_rb/)
      kao(:, 1,12, 3) = (/ &
     &3.4882e+01_rb,3.0522e+01_rb,2.6162e+01_rb,2.1802e+01_rb,1.7441e+01_rb,1.3081e+01_rb, &
     &8.7208e+00_rb,4.3607e+00_rb,4.1853e-04_rb/)
      kao(:, 2,12, 3) = (/ &
     &3.4744e+01_rb,3.0400e+01_rb,2.6058e+01_rb,2.1715e+01_rb,1.7372e+01_rb,1.3029e+01_rb, &
     &8.6862e+00_rb,4.3435e+00_rb,5.1635e-04_rb/)
      kao(:, 3,12, 3) = (/ &
     &3.4694e+01_rb,3.0356e+01_rb,2.6020e+01_rb,2.1683e+01_rb,1.7347e+01_rb,1.3010e+01_rb, &
     &8.6737e+00_rb,4.3374e+00_rb,5.8258e-04_rb/)
      kao(:, 4,12, 3) = (/ &
     &3.4999e+01_rb,3.0625e+01_rb,2.6249e+01_rb,2.1876e+01_rb,1.7501e+01_rb,1.3126e+01_rb, &
     &8.7513e+00_rb,4.3768e+00_rb,7.2545e-04_rb/)
      kao(:, 5,12, 3) = (/ &
     &3.5658e+01_rb,3.1201e+01_rb,2.6744e+01_rb,2.2288e+01_rb,1.7831e+01_rb,1.3374e+01_rb, &
     &8.9165e+00_rb,4.4593e+00_rb,9.3950e-04_rb/)
      kao(:, 1,13, 3) = (/ &
     &3.0168e+01_rb,2.6399e+01_rb,2.2627e+01_rb,1.8855e+01_rb,1.5085e+01_rb,1.1314e+01_rb, &
     &7.5425e+00_rb,3.7715e+00_rb,4.1524e-04_rb/)
      kao(:, 2,13, 3) = (/ &
     &3.0082e+01_rb,2.6321e+01_rb,2.2562e+01_rb,1.8800e+01_rb,1.5040e+01_rb,1.1280e+01_rb, &
     &7.5203e+00_rb,3.7604e+00_rb,5.1897e-04_rb/)
      kao(:, 3,13, 3) = (/ &
     &3.0098e+01_rb,2.6336e+01_rb,2.2574e+01_rb,1.8812e+01_rb,1.5049e+01_rb,1.1287e+01_rb, &
     &7.5253e+00_rb,3.7634e+00_rb,6.3490e-04_rb/)
      kao(:, 4,13, 3) = (/ &
     &3.0488e+01_rb,2.6677e+01_rb,2.2867e+01_rb,1.9056e+01_rb,1.5245e+01_rb,1.1434e+01_rb, &
     &7.6233e+00_rb,3.8122e+00_rb,7.3304e-04_rb/)
      kao(:, 5,13, 3) = (/ &
     &3.1139e+01_rb,2.7247e+01_rb,2.3355e+01_rb,1.9463e+01_rb,1.5570e+01_rb,1.1678e+01_rb, &
     &7.7859e+00_rb,3.8940e+00_rb,8.6573e-04_rb/)
      kao(:, 1, 1, 4) = (/ &
     &5.2057e-02_rb,4.6191e-02_rb,4.0791e-02_rb,3.5264e-02_rb,2.9386e-02_rb,2.3311e-02_rb, &
     &1.6933e-02_rb,9.9741e-03_rb,9.4742e-04_rb/)
      kao(:, 2, 1, 4) = (/ &
     &5.1799e-02_rb,4.6370e-02_rb,4.1424e-02_rb,3.5942e-02_rb,3.0151e-02_rb,2.4090e-02_rb, &
     &1.7827e-02_rb,1.1138e-02_rb,1.1643e-03_rb/)
      kao(:, 3, 1, 4) = (/ &
     &5.1774e-02_rb,4.6875e-02_rb,4.2240e-02_rb,3.6825e-02_rb,3.1105e-02_rb,2.5101e-02_rb, &
     &1.8967e-02_rb,1.2189e-02_rb,1.4467e-03_rb/)
      kao(:, 4, 1, 4) = (/ &
     &5.2049e-02_rb,4.7822e-02_rb,4.3286e-02_rb,3.7962e-02_rb,3.2336e-02_rb,2.6505e-02_rb, &
     &2.0163e-02_rb,1.3176e-02_rb,1.8457e-03_rb/)
      kao(:, 5, 1, 4) = (/ &
     &5.2435e-02_rb,4.8877e-02_rb,4.4449e-02_rb,3.9288e-02_rb,3.3894e-02_rb,2.7971e-02_rb, &
     &2.1357e-02_rb,1.4142e-02_rb,2.2765e-03_rb/)
      kao(:, 1, 2, 4) = (/ &
     &6.9283e-02_rb,6.1124e-02_rb,5.3312e-02_rb,4.5637e-02_rb,3.7639e-02_rb,2.9444e-02_rb, &
     &2.0926e-02_rb,1.1987e-02_rb,6.7708e-04_rb/)
      kao(:, 2, 2, 4) = (/ &
     &6.9230e-02_rb,6.1365e-02_rb,5.4052e-02_rb,4.6528e-02_rb,3.8569e-02_rb,3.0382e-02_rb, &
     &2.1758e-02_rb,1.2938e-02_rb,8.4618e-04_rb/)
      kao(:, 3, 2, 4) = (/ &
     &6.9474e-02_rb,6.2065e-02_rb,5.5235e-02_rb,4.7695e-02_rb,3.9803e-02_rb,3.1502e-02_rb, &
     &2.2924e-02_rb,1.3989e-02_rb,1.0726e-03_rb/)
      kao(:, 4, 2, 4) = (/ &
     &7.0041e-02_rb,6.3262e-02_rb,5.6631e-02_rb,4.9140e-02_rb,4.1246e-02_rb,3.2954e-02_rb, &
     &2.4393e-02_rb,1.5061e-02_rb,1.3926e-03_rb/)
      kao(:, 5, 2, 4) = (/ &
     &7.0798e-02_rb,6.4648e-02_rb,5.8174e-02_rb,5.0781e-02_rb,4.2976e-02_rb,3.4744e-02_rb, &
     &2.5864e-02_rb,1.6092e-02_rb,1.7223e-03_rb/)
      kao(:, 1, 3, 4) = (/ &
     &1.2468e-01_rb,1.0948e-01_rb,9.4277e-02_rb,7.9471e-02_rb,6.4703e-02_rb,4.9713e-02_rb, &
     &3.4483e-02_rb,1.8729e-02_rb,4.6220e-04_rb/)
      kao(:, 2, 3, 4) = (/ &
     &1.2473e-01_rb,1.0972e-01_rb,9.4939e-02_rb,8.0505e-02_rb,6.5924e-02_rb,5.0895e-02_rb, &
     &3.5571e-02_rb,1.9546e-02_rb,5.8785e-04_rb/)
      kao(:, 3, 3, 4) = (/ &
     &1.2557e-01_rb,1.1074e-01_rb,9.6443e-02_rb,8.2330e-02_rb,6.7617e-02_rb,5.2547e-02_rb, &
     &3.6938e-02_rb,2.0666e-02_rb,7.7467e-04_rb/)
      kao(:, 4, 3, 4) = (/ &
     &1.2675e-01_rb,1.1231e-01_rb,9.8604e-02_rb,8.4502e-02_rb,6.9700e-02_rb,5.4472e-02_rb, &
     &3.8553e-02_rb,2.2075e-02_rb,9.9922e-04_rb/)
      kao(:, 5, 3, 4) = (/ &
     &1.2846e-01_rb,1.1449e-01_rb,1.0119e-01_rb,8.7016e-02_rb,7.2084e-02_rb,5.6669e-02_rb, &
     &4.0555e-02_rb,2.3518e-02_rb,1.2453e-03_rb/)
      kao(:, 1, 4, 4) = (/ &
     &2.3029e-01_rb,2.0185e-01_rb,1.7335e-01_rb,1.4485e-01_rb,1.1669e-01_rb,8.8611e-02_rb, &
     &6.0409e-02_rb,3.1699e-02_rb,3.0371e-04_rb/)
      kao(:, 2, 4, 4) = (/ &
     &2.2988e-01_rb,2.0158e-01_rb,1.7329e-01_rb,1.4525e-01_rb,1.1751e-01_rb,8.9738e-02_rb, &
     &6.1449e-02_rb,3.2569e-02_rb,4.0236e-04_rb/)
      kao(:, 3, 4, 4) = (/ &
     &2.3229e-01_rb,2.0393e-01_rb,1.7566e-01_rb,1.4780e-01_rb,1.2020e-01_rb,9.2062e-02_rb, &
     &6.3397e-02_rb,3.3957e-02_rb,5.4454e-04_rb/)
      kao(:, 4, 4, 4) = (/ &
     &2.3507e-01_rb,2.0665e-01_rb,1.7868e-01_rb,1.5121e-01_rb,1.2350e-01_rb,9.4964e-02_rb, &
     &6.5854e-02_rb,3.5604e-02_rb,7.0998e-04_rb/)
      kao(:, 5, 4, 4) = (/ &
     &2.3825e-01_rb,2.0990e-01_rb,1.8240e-01_rb,1.5515e-01_rb,1.2704e-01_rb,9.8075e-02_rb, &
     &6.8384e-02_rb,3.7494e-02_rb,8.9536e-04_rb/)
      kao(:, 1, 5, 4) = (/ &
     &3.8945e-01_rb,3.4103e-01_rb,2.9259e-01_rb,2.4415e-01_rb,1.9571e-01_rb,1.4754e-01_rb, &
     &9.9565e-02_rb,5.1352e-02_rb,2.0409e-04_rb/)
      kao(:, 2, 5, 4) = (/ &
     &3.8981e-01_rb,3.4147e-01_rb,2.9305e-01_rb,2.4466e-01_rb,1.9659e-01_rb,1.4875e-01_rb, &
     &1.0088e-01_rb,5.2387e-02_rb,2.7339e-04_rb/)
      kao(:, 3, 5, 4) = (/ &
     &3.9497e-01_rb,3.4612e-01_rb,2.9724e-01_rb,2.4861e-01_rb,2.0028e-01_rb,1.5226e-01_rb, &
     &1.0369e-01_rb,5.4177e-02_rb,3.7984e-04_rb/)
      kao(:, 4, 5, 4) = (/ &
     &4.0149e-01_rb,3.5204e-01_rb,3.0270e-01_rb,2.5382e-01_rb,2.0546e-01_rb,1.5684e-01_rb, &
     &1.0724e-01_rb,5.6458e-02_rb,5.0160e-04_rb/)
      kao(:, 5, 5, 4) = (/ &
     &4.0773e-01_rb,3.5786e-01_rb,3.0839e-01_rb,2.5956e-01_rb,2.1090e-01_rb,1.6141e-01_rb, &
     &1.1086e-01_rb,5.8838e-02_rb,6.4384e-04_rb/)
      kao(:, 1, 6, 4) = (/ &
     &6.2109e-01_rb,5.4370e-01_rb,4.6619e-01_rb,3.8877e-01_rb,3.1132e-01_rb,2.3383e-01_rb, &
     &1.5671e-01_rb,7.9602e-02_rb,1.3341e-04_rb/)
      kao(:, 2, 6, 4) = (/ &
     &6.2314e-01_rb,5.4556e-01_rb,4.6793e-01_rb,3.9037e-01_rb,3.1270e-01_rb,2.3534e-01_rb, &
     &1.5827e-01_rb,8.0992e-02_rb,1.8152e-04_rb/)
      kao(:, 3, 6, 4) = (/ &
     &6.2957e-01_rb,5.5136e-01_rb,4.7308e-01_rb,3.9475e-01_rb,3.1676e-01_rb,2.3906e-01_rb, &
     &1.6147e-01_rb,8.3225e-02_rb,2.5970e-04_rb/)
      kao(:, 4, 6, 4) = (/ &
     &6.4091e-01_rb,5.6147e-01_rb,4.8196e-01_rb,4.0265e-01_rb,3.2370e-01_rb,2.4515e-01_rb, &
     &1.6638e-01_rb,8.6255e-02_rb,3.5139e-04_rb/)
      kao(:, 5, 6, 4) = (/ &
     &6.5388e-01_rb,5.7307e-01_rb,4.9223e-01_rb,4.1191e-01_rb,3.3205e-01_rb,2.5237e-01_rb, &
     &1.7171e-01_rb,8.9448e-02_rb,4.5531e-04_rb/)
      kao(:, 1, 7, 4) = (/ &
     &1.0457e+00_rb,9.1519e-01_rb,7.8460e-01_rb,6.5401e-01_rb,5.2349e-01_rb,3.9292e-01_rb, &
     &2.6230e-01_rb,1.3220e-01_rb,1.0032e-04_rb/)
      kao(:, 2, 7, 4) = (/ &
     &1.0500e+00_rb,9.1906e-01_rb,7.8807e-01_rb,6.5699e-01_rb,5.2599e-01_rb,3.9492e-01_rb, &
     &2.6410e-01_rb,1.3358e-01_rb,1.3899e-04_rb/)
      kao(:, 3, 7, 4) = (/ &
     &1.0598e+00_rb,9.2774e-01_rb,7.9565e-01_rb,6.6349e-01_rb,5.3130e-01_rb,3.9936e-01_rb, &
     &2.6782e-01_rb,1.3620e-01_rb,1.8444e-04_rb/)
      kao(:, 4, 7, 4) = (/ &
     &1.0743e+00_rb,9.4059e-01_rb,8.0685e-01_rb,6.7311e-01_rb,5.3945e-01_rb,4.0621e-01_rb, &
     &2.7331e-01_rb,1.3995e-01_rb,2.4657e-04_rb/)
      kao(:, 5, 7, 4) = (/ &
     &1.0996e+00_rb,9.6289e-01_rb,8.2608e-01_rb,6.8953e-01_rb,5.5342e-01_rb,4.1738e-01_rb, &
     &2.8182e-01_rb,1.4485e-01_rb,3.2431e-04_rb/)
      kao(:, 1, 8, 4) = (/ &
     &2.1071e+00_rb,1.8438e+00_rb,1.5804e+00_rb,1.3173e+00_rb,1.0540e+00_rb,7.9071e-01_rb, &
     &5.2749e-01_rb,2.6416e-01_rb,1.1085e-04_rb/)
      kao(:, 2, 8, 4) = (/ &
     &2.1142e+00_rb,1.8501e+00_rb,1.5861e+00_rb,1.3219e+00_rb,1.0578e+00_rb,7.9379e-01_rb, &
     &5.2971e-01_rb,2.6585e-01_rb,1.4810e-04_rb/)
      kao(:, 3, 8, 4) = (/ &
     &2.1318e+00_rb,1.8656e+00_rb,1.5995e+00_rb,1.3333e+00_rb,1.0671e+00_rb,8.0094e-01_rb, &
     &5.3475e-01_rb,2.6918e-01_rb,1.9501e-04_rb/)
      kao(:, 4, 8, 4) = (/ &
     &2.1555e+00_rb,1.8865e+00_rb,1.6175e+00_rb,1.3486e+00_rb,1.0795e+00_rb,8.1059e-01_rb, &
     &5.4205e-01_rb,2.7386e-01_rb,2.4224e-04_rb/)
      kao(:, 5, 8, 4) = (/ &
     &2.2062e+00_rb,1.9311e+00_rb,1.6560e+00_rb,1.3807e+00_rb,1.1056e+00_rb,8.3087e-01_rb, &
     &5.5626e-01_rb,2.8214e-01_rb,2.8558e-04_rb/)
      kao(:, 1, 9, 4) = (/ &
     &7.9972e+00_rb,6.9979e+00_rb,5.9979e+00_rb,4.9986e+00_rb,3.9987e+00_rb,2.9994e+00_rb, &
     &1.9999e+00_rb,1.0002e+00_rb,1.8668e-04_rb/)
      kao(:, 2, 9, 4) = (/ &
     &8.0128e+00_rb,7.0114e+00_rb,6.0097e+00_rb,5.0085e+00_rb,4.0070e+00_rb,3.0056e+00_rb, &
     &2.0042e+00_rb,1.0027e+00_rb,2.5150e-04_rb/)
      kao(:, 3, 9, 4) = (/ &
     &8.0805e+00_rb,7.0702e+00_rb,6.0607e+00_rb,5.0509e+00_rb,4.0412e+00_rb,3.0319e+00_rb, &
     &2.0219e+00_rb,1.0119e+00_rb,3.0396e-04_rb/)
      kao(:, 4, 9, 4) = (/ &
     &8.1743e+00_rb,7.1533e+00_rb,6.1315e+00_rb,5.1108e+00_rb,4.0892e+00_rb,3.0678e+00_rb, &
     &2.0462e+00_rb,1.0247e+00_rb,3.5321e-04_rb/)
      kao(:, 5, 9, 4) = (/ &
     &8.3409e+00_rb,7.2988e+00_rb,6.2567e+00_rb,5.2146e+00_rb,4.1728e+00_rb,3.1306e+00_rb, &
     &2.0885e+00_rb,1.0471e+00_rb,4.4198e-04_rb/)
      kao(:, 1,10, 4) = (/ &
     &3.3771e+01_rb,2.9548e+01_rb,2.5328e+01_rb,2.1105e+01_rb,1.6886e+01_rb,1.2665e+01_rb, &
     &8.4427e+00_rb,4.2216e+00_rb,4.3200e-04_rb/)
      kao(:, 2,10, 4) = (/ &
     &3.3810e+01_rb,2.9587e+01_rb,2.5358e+01_rb,2.1131e+01_rb,1.6905e+01_rb,1.2680e+01_rb, &
     &8.4534e+00_rb,4.2274e+00_rb,5.4393e-04_rb/)
      kao(:, 3,10, 4) = (/ &
     &3.4144e+01_rb,2.9876e+01_rb,2.5609e+01_rb,2.1340e+01_rb,1.7073e+01_rb,1.2805e+01_rb, &
     &8.5375e+00_rb,4.2697e+00_rb,6.4726e-04_rb/)
      kao(:, 4,10, 4) = (/ &
     &3.4577e+01_rb,3.0255e+01_rb,2.5934e+01_rb,2.1613e+01_rb,1.7291e+01_rb,1.2969e+01_rb, &
     &8.6471e+00_rb,4.3251e+00_rb,7.9967e-04_rb/)
      kao(:, 5,10, 4) = (/ &
     &3.5279e+01_rb,3.0869e+01_rb,2.6459e+01_rb,2.2050e+01_rb,1.7642e+01_rb,1.3232e+01_rb, &
     &8.8233e+00_rb,4.4139e+00_rb,8.8613e-04_rb/)
      kao(:, 1,11, 4) = (/ &
     &4.9379e+01_rb,4.3210e+01_rb,3.7035e+01_rb,3.0865e+01_rb,2.4692e+01_rb,1.8519e+01_rb, &
     &1.2346e+01_rb,6.1734e+00_rb,5.9531e-04_rb/)
      kao(:, 2,11, 4) = (/ &
     &4.9664e+01_rb,4.3455e+01_rb,3.7247e+01_rb,3.1040e+01_rb,2.4833e+01_rb,1.8625e+01_rb, &
     &1.2417e+01_rb,6.2092e+00_rb,7.3911e-04_rb/)
      kao(:, 3,11, 4) = (/ &
     &5.0351e+01_rb,4.4056e+01_rb,3.7762e+01_rb,3.1470e+01_rb,2.5175e+01_rb,1.8882e+01_rb, &
     &1.2589e+01_rb,6.2954e+00_rb,9.0535e-04_rb/)
      kao(:, 4,11, 4) = (/ &
     &5.1310e+01_rb,4.4899e+01_rb,3.8484e+01_rb,3.2071e+01_rb,2.5657e+01_rb,1.9244e+01_rb, &
     &1.2831e+01_rb,6.4171e+00_rb,1.0300e-03_rb/)
      kao(:, 5,11, 4) = (/ &
     &5.2691e+01_rb,4.6105e+01_rb,3.9519e+01_rb,3.2933e+01_rb,2.6348e+01_rb,1.9763e+01_rb, &
     &1.3176e+01_rb,6.5918e+00_rb,1.3056e-03_rb/)
      kao(:, 1,12, 4) = (/ &
     &5.3682e+01_rb,4.6971e+01_rb,4.0260e+01_rb,3.3552e+01_rb,2.6842e+01_rb,2.0131e+01_rb, &
     &1.3420e+01_rb,6.7107e+00_rb,5.9618e-04_rb/)
      kao(:, 2,12, 4) = (/ &
     &5.4209e+01_rb,4.7433e+01_rb,4.0657e+01_rb,3.3879e+01_rb,2.7104e+01_rb,2.0329e+01_rb, &
     &1.3553e+01_rb,6.7772e+00_rb,7.6649e-04_rb/)
      kao(:, 3,12, 4) = (/ &
     &5.5191e+01_rb,4.8293e+01_rb,4.1395e+01_rb,3.4495e+01_rb,2.7597e+01_rb,2.0697e+01_rb, &
     &1.3799e+01_rb,6.9009e+00_rb,9.6570e-04_rb/)
      kao(:, 4,12, 4) = (/ &
     &5.6606e+01_rb,4.9530e+01_rb,4.2455e+01_rb,3.5382e+01_rb,2.8306e+01_rb,2.1230e+01_rb, &
     &1.4154e+01_rb,7.0794e+00_rb,1.1503e-03_rb/)
      kao(:, 5,12, 4) = (/ &
     &5.8604e+01_rb,5.1279e+01_rb,4.3956e+01_rb,3.6631e+01_rb,2.9306e+01_rb,2.1982e+01_rb, &
     &1.4657e+01_rb,7.3321e+00_rb,1.3037e-03_rb/)
      kao(:, 1,13, 4) = (/ &
     &4.6897e+01_rb,4.1036e+01_rb,3.5173e+01_rb,2.9311e+01_rb,2.3449e+01_rb,1.7586e+01_rb, &
     &1.1724e+01_rb,5.8626e+00_rb,5.3404e-04_rb/)
      kao(:, 2,13, 4) = (/ &
     &4.7510e+01_rb,4.1569e+01_rb,3.5631e+01_rb,2.9695e+01_rb,2.3756e+01_rb,1.7817e+01_rb, &
     &1.1878e+01_rb,5.9396e+00_rb,7.0585e-04_rb/)
      kao(:, 3,13, 4) = (/ &
     &4.8599e+01_rb,4.2525e+01_rb,3.6450e+01_rb,3.0375e+01_rb,2.4299e+01_rb,1.8226e+01_rb, &
     &1.2152e+01_rb,6.0774e+00_rb,8.6599e-04_rb/)
      kao(:, 4,13, 4) = (/ &
     &5.0122e+01_rb,4.3856e+01_rb,3.7591e+01_rb,3.1328e+01_rb,2.5063e+01_rb,1.8798e+01_rb, &
     &1.2534e+01_rb,6.2697e+00_rb,1.0053e-03_rb/)
      kao(:, 5,13, 4) = (/ &
     &5.2215e+01_rb,4.5689e+01_rb,3.9164e+01_rb,3.2638e+01_rb,2.6111e+01_rb,1.9586e+01_rb, &
     &1.3060e+01_rb,6.5334e+00_rb,1.2313e-03_rb/)
      kao(:, 1, 1, 5) = (/ &
     &8.2901e-02_rb,7.3924e-02_rb,6.4657e-02_rb,5.5597e-02_rb,4.6472e-02_rb,3.6796e-02_rb, &
     &2.6709e-02_rb,1.5948e-02_rb,1.7802e-03_rb/)
      kao(:, 2, 1, 5) = (/ &
     &8.3370e-02_rb,7.4618e-02_rb,6.5698e-02_rb,5.7019e-02_rb,4.7812e-02_rb,3.7978e-02_rb, &
     &2.7855e-02_rb,1.7146e-02_rb,2.3935e-03_rb/)
      kao(:, 3, 1, 5) = (/ &
     &8.4235e-02_rb,7.5730e-02_rb,6.7257e-02_rb,5.8667e-02_rb,4.9229e-02_rb,3.9298e-02_rb, &
     &2.8966e-02_rb,1.8334e-02_rb,3.0382e-03_rb/)
      kao(:, 4, 1, 5) = (/ &
     &8.5176e-02_rb,7.7019e-02_rb,6.8972e-02_rb,6.0287e-02_rb,5.0680e-02_rb,4.0612e-02_rb, &
     &3.0271e-02_rb,1.9679e-02_rb,3.7478e-03_rb/)
      kao(:, 5, 1, 5) = (/ &
     &8.6230e-02_rb,7.8450e-02_rb,7.0705e-02_rb,6.1832e-02_rb,5.2066e-02_rb,4.2065e-02_rb, &
     &3.1638e-02_rb,2.1127e-02_rb,4.5344e-03_rb/)
      kao(:, 1, 2, 5) = (/ &
     &1.1319e-01_rb,1.0032e-01_rb,8.7141e-02_rb,7.4086e-02_rb,6.1196e-02_rb,4.7903e-02_rb, &
     &3.4053e-02_rb,1.9479e-02_rb,1.3324e-03_rb/)
      kao(:, 2, 2, 5) = (/ &
     &1.1444e-01_rb,1.0174e-01_rb,8.8783e-02_rb,7.6019e-02_rb,6.3135e-02_rb,4.9508e-02_rb, &
     &3.5403e-02_rb,2.0488e-02_rb,1.7827e-03_rb/)
      kao(:, 3, 2, 5) = (/ &
     &1.1608e-01_rb,1.0349e-01_rb,9.0802e-02_rb,7.8322e-02_rb,6.5118e-02_rb,5.1198e-02_rb, &
     &3.6801e-02_rb,2.1623e-02_rb,2.2789e-03_rb/)
      kao(:, 4, 2, 5) = (/ &
     &1.1786e-01_rb,1.0545e-01_rb,9.3147e-02_rb,8.0587e-02_rb,6.7042e-02_rb,5.2943e-02_rb, &
     &3.8201e-02_rb,2.2798e-02_rb,2.8272e-03_rb/)
      kao(:, 5, 2, 5) = (/ &
     &1.1969e-01_rb,1.0757e-01_rb,9.5567e-02_rb,8.2763e-02_rb,6.8961e-02_rb,5.4626e-02_rb, &
     &3.9785e-02_rb,2.4020e-02_rb,3.4454e-03_rb/)
      kao(:, 1, 3, 5) = (/ &
     &2.0690e-01_rb,1.8215e-01_rb,1.5725e-01_rb,1.3219e-01_rb,1.0725e-01_rb,8.2467e-02_rb, &
     &5.7330e-02_rb,3.1338e-02_rb,9.1573e-04_rb/)
      kao(:, 2, 3, 5) = (/ &
     &2.1013e-01_rb,1.8537e-01_rb,1.6026e-01_rb,1.3517e-01_rb,1.1017e-01_rb,8.5255e-02_rb, &
     &5.9448e-02_rb,3.2689e-02_rb,1.2507e-03_rb/)
      kao(:, 3, 3, 5) = (/ &
     &2.1385e-01_rb,1.8904e-01_rb,1.6379e-01_rb,1.3865e-01_rb,1.1374e-01_rb,8.8133e-02_rb, &
     &6.1604e-02_rb,3.4070e-02_rb,1.6182e-03_rb/)
      kao(:, 4, 3, 5) = (/ &
     &2.1772e-01_rb,1.9276e-01_rb,1.6753e-01_rb,1.4253e-01_rb,1.1723e-01_rb,9.0813e-02_rb, &
     &6.3766e-02_rb,3.5513e-02_rb,2.0311e-03_rb/)
      kao(:, 5, 3, 5) = (/ &
     &2.2143e-01_rb,1.9638e-01_rb,1.7123e-01_rb,1.4633e-01_rb,1.2052e-01_rb,9.3556e-02_rb, &
     &6.5944e-02_rb,3.7115e-02_rb,2.5088e-03_rb/)
      kao(:, 1, 4, 5) = (/ &
     &3.8507e-01_rb,3.3780e-01_rb,2.9048e-01_rb,2.4314e-01_rb,1.9570e-01_rb,1.4827e-01_rb, &
     &1.0100e-01_rb,5.3296e-02_rb,6.2147e-04_rb/)
      kao(:, 2, 4, 5) = (/ &
     &3.9146e-01_rb,3.4379e-01_rb,2.9605e-01_rb,2.4819e-01_rb,2.0018e-01_rb,1.5228e-01_rb, &
     &1.0449e-01_rb,5.5540e-02_rb,8.6088e-04_rb/)
      kao(:, 3, 4, 5) = (/ &
     &3.9930e-01_rb,3.5111e-01_rb,3.0278e-01_rb,2.5411e-01_rb,2.0550e-01_rb,1.5718e-01_rb, &
     &1.0825e-01_rb,5.7768e-02_rb,1.1308e-03_rb/)
      kao(:, 4, 4, 5) = (/ &
     &4.0750e-01_rb,3.5883e-01_rb,3.0957e-01_rb,2.6026e-01_rb,2.1112e-01_rb,1.6215e-01_rb, &
     &1.1175e-01_rb,5.9905e-02_rb,1.4435e-03_rb/)
      kao(:, 5, 4, 5) = (/ &
     &4.1540e-01_rb,3.6624e-01_rb,3.1618e-01_rb,2.6641e-01_rb,2.1709e-01_rb,1.6699e-01_rb, &
     &1.1521e-01_rb,6.2079e-02_rb,1.8081e-03_rb/)
      kao(:, 1, 5, 5) = (/ &
     &6.6039e-01_rb,5.7849e-01_rb,4.9659e-01_rb,4.1465e-01_rb,3.3268e-01_rb,2.5068e-01_rb, &
     &1.6878e-01_rb,8.6906e-02_rb,4.1392e-04_rb/)
      kao(:, 2, 5, 5) = (/ &
     &6.7138e-01_rb,5.8843e-01_rb,5.0538e-01_rb,4.2238e-01_rb,3.3940e-01_rb,2.5633e-01_rb, &
     &1.7334e-01_rb,9.0010e-02_rb,5.9039e-04_rb/)
      kao(:, 3, 5, 5) = (/ &
     &6.8434e-01_rb,6.0014e-01_rb,5.1596e-01_rb,4.3167e-01_rb,3.4721e-01_rb,2.6273e-01_rb, &
     &1.7850e-01_rb,9.3277e-02_rb,7.8856e-04_rb/)
      kao(:, 4, 5, 5) = (/ &
     &6.9935e-01_rb,6.1381e-01_rb,5.2821e-01_rb,4.4224e-01_rb,3.5606e-01_rb,2.7005e-01_rb, &
     &1.8435e-01_rb,9.6630e-02_rb,1.0256e-03_rb/)
      kao(:, 5, 5, 5) = (/ &
     &7.1524e-01_rb,6.2827e-01_rb,5.4104e-01_rb,4.5319e-01_rb,3.6550e-01_rb,2.7828e-01_rb, &
     &1.9025e-01_rb,9.9988e-02_rb,1.3023e-03_rb/)
      kao(:, 1, 6, 5) = (/ &
     &1.0605e+00_rb,9.2835e-01_rb,7.9638e-01_rb,6.6416e-01_rb,5.3204e-01_rb,3.9988e-01_rb, &
     &2.6772e-01_rb,1.3599e-01_rb,2.6665e-04_rb/)
      kao(:, 2, 6, 5) = (/ &
     &1.0793e+00_rb,9.4508e-01_rb,8.1097e-01_rb,6.7669e-01_rb,5.4239e-01_rb,4.0816e-01_rb, &
     &2.7396e-01_rb,1.4011e-01_rb,3.9578e-04_rb/)
      kao(:, 3, 6, 5) = (/ &
     &1.0996e+00_rb,9.6312e-01_rb,8.2674e-01_rb,6.9026e-01_rb,5.5385e-01_rb,4.1734e-01_rb, &
     &2.8093e-01_rb,1.4437e-01_rb,5.4281e-04_rb/)
      kao(:, 4, 6, 5) = (/ &
     &1.1270e+00_rb,9.8758e-01_rb,8.4814e-01_rb,7.0877e-01_rb,5.6910e-01_rb,4.2937e-01_rb, &
     &2.8968e-01_rb,1.4948e-01_rb,7.1692e-04_rb/)
      kao(:, 5, 6, 5) = (/ &
     &1.1549e+00_rb,1.0125e+00_rb,8.7022e-01_rb,7.2764e-01_rb,5.8478e-01_rb,4.4170e-01_rb, &
     &2.9907e-01_rb,1.5477e-01_rb,9.2735e-04_rb/)
      kao(:, 1, 7, 5) = (/ &
     &1.7902e+00_rb,1.5666e+00_rb,1.3432e+00_rb,1.1198e+00_rb,8.9630e-01_rb,6.7291e-01_rb, &
     &4.4939e-01_rb,2.2594e-01_rb,1.7788e-04_rb/)
      kao(:, 2, 7, 5) = (/ &
     &1.8253e+00_rb,1.5977e+00_rb,1.3701e+00_rb,1.1424e+00_rb,9.1476e-01_rb,6.8709e-01_rb, &
     &4.5936e-01_rb,2.3207e-01_rb,2.6007e-04_rb/)
      kao(:, 3, 7, 5) = (/ &
     &1.8661e+00_rb,1.6335e+00_rb,1.4010e+00_rb,1.1685e+00_rb,9.3601e-01_rb,7.0359e-01_rb, &
     &4.7118e-01_rb,2.3911e-01_rb,3.6579e-04_rb/)
      kao(:, 4, 7, 5) = (/ &
     &1.9131e+00_rb,1.6751e+00_rb,1.4368e+00_rb,1.1989e+00_rb,9.6095e-01_rb,7.2299e-01_rb, &
     &4.8503e-01_rb,2.4706e-01_rb,4.9009e-04_rb/)
      kao(:, 5, 7, 5) = (/ &
     &1.9618e+00_rb,1.7182e+00_rb,1.4745e+00_rb,1.2310e+00_rb,9.8739e-01_rb,7.4364e-01_rb, &
     &4.9951e-01_rb,2.5543e-01_rb,6.4417e-04_rb/)
      kao(:, 1, 8, 5) = (/ &
     &3.6222e+00_rb,3.1697e+00_rb,2.7175e+00_rb,2.2647e+00_rb,1.8122e+00_rb,1.3595e+00_rb, &
     &9.0703e-01_rb,4.5436e-01_rb,1.7700e-04_rb/)
      kao(:, 2, 8, 5) = (/ &
     &3.6989e+00_rb,3.2367e+00_rb,2.7747e+00_rb,2.3129e+00_rb,1.8509e+00_rb,1.3888e+00_rb, &
     &9.2689e-01_rb,4.6487e-01_rb,2.2396e-04_rb/)
      kao(:, 3, 8, 5) = (/ &
     &3.7900e+00_rb,3.3168e+00_rb,2.8434e+00_rb,2.3702e+00_rb,1.8972e+00_rb,1.4240e+00_rb, &
     &9.5085e-01_rb,4.7797e-01_rb,2.7417e-04_rb/)
      kao(:, 4, 8, 5) = (/ &
     &3.8908e+00_rb,3.4054e+00_rb,2.9198e+00_rb,2.4343e+00_rb,1.9488e+00_rb,1.4634e+00_rb, &
     &9.7805e-01_rb,4.9296e-01_rb,3.4875e-04_rb/)
      kao(:, 5, 8, 5) = (/ &
     &3.9856e+00_rb,3.4886e+00_rb,2.9916e+00_rb,2.4947e+00_rb,1.9978e+00_rb,1.5012e+00_rb, &
     &1.0044e+00_rb,5.0758e-01_rb,4.5934e-04_rb/)
      kao(:, 1, 9, 5) = (/ &
     &1.3794e+01_rb,1.2070e+01_rb,1.0346e+01_rb,8.6224e+00_rb,6.8983e+00_rb,5.1738e+00_rb, &
     &3.4497e+00_rb,1.7256e+00_rb,2.3317e-04_rb/)
      kao(:, 2, 9, 5) = (/ &
     &1.4093e+01_rb,1.2332e+01_rb,1.0571e+01_rb,8.8090e+00_rb,7.0478e+00_rb,5.2864e+00_rb, &
     &3.5250e+00_rb,1.7636e+00_rb,2.9550e-04_rb/)
      kao(:, 3, 9, 5) = (/ &
     &1.4464e+01_rb,1.2658e+01_rb,1.0848e+01_rb,9.0415e+00_rb,7.2337e+00_rb,5.4264e+00_rb, &
     &3.6184e+00_rb,1.8107e+00_rb,3.8639e-04_rb/)
      kao(:, 4, 9, 5) = (/ &
     &1.4855e+01_rb,1.2999e+01_rb,1.1143e+01_rb,9.2862e+00_rb,7.4306e+00_rb,5.5737e+00_rb, &
     &3.7173e+00_rb,1.8614e+00_rb,5.0786e-04_rb/)
      kao(:, 5, 9, 5) = (/ &
     &1.5210e+01_rb,1.3311e+01_rb,1.1410e+01_rb,9.5097e+00_rb,7.6091e+00_rb,5.7087e+00_rb, &
     &3.8087e+00_rb,1.9087e+00_rb,5.8982e-04_rb/)
      kao(:, 1,10, 5) = (/ &
     &5.8496e+01_rb,5.1180e+01_rb,4.3867e+01_rb,3.6556e+01_rb,2.9247e+01_rb,2.1935e+01_rb, &
     &1.4623e+01_rb,7.3123e+00_rb,5.6403e-04_rb/)
      kao(:, 2,10, 5) = (/ &
     &5.9786e+01_rb,5.2317e+01_rb,4.4840e+01_rb,3.7369e+01_rb,2.9895e+01_rb,2.2423e+01_rb, &
     &1.4947e+01_rb,7.4749e+00_rb,6.9685e-04_rb/)
      kao(:, 3,10, 5) = (/ &
     &6.1366e+01_rb,5.3692e+01_rb,4.6022e+01_rb,3.8353e+01_rb,3.0682e+01_rb,2.3012e+01_rb, &
     &1.5344e+01_rb,7.6729e+00_rb,8.6661e-04_rb/)
      kao(:, 4,10, 5) = (/ &
     &6.3002e+01_rb,5.5128e+01_rb,4.7254e+01_rb,3.9377e+01_rb,3.1505e+01_rb,2.3630e+01_rb, &
     &1.5753e+01_rb,7.8780e+00_rb,1.0403e-03_rb/)
      kao(:, 5,10, 5) = (/ &
     &6.4493e+01_rb,5.6436e+01_rb,4.8374e+01_rb,4.0310e+01_rb,3.2253e+01_rb,2.4189e+01_rb, &
     &1.6127e+01_rb,8.0668e+00_rb,1.2361e-03_rb/)
      kao(:, 1,11, 5) = (/ &
     &8.7182e+01_rb,7.6287e+01_rb,6.5394e+01_rb,5.4493e+01_rb,4.3594e+01_rb,3.2696e+01_rb, &
     &2.1798e+01_rb,1.0900e+01_rb,7.4362e-04_rb/)
      kao(:, 2,11, 5) = (/ &
     &8.9336e+01_rb,7.8172e+01_rb,6.7002e+01_rb,5.5836e+01_rb,4.4668e+01_rb,3.3502e+01_rb, &
     &2.2335e+01_rb,1.1168e+01_rb,9.0702e-04_rb/)
      kao(:, 3,11, 5) = (/ &
     &9.1581e+01_rb,8.0131e+01_rb,6.8688e+01_rb,5.7237e+01_rb,4.5792e+01_rb,3.4345e+01_rb, &
     &2.2896e+01_rb,1.1449e+01_rb,1.0809e-03_rb/)
      kao(:, 4,11, 5) = (/ &
     &9.3839e+01_rb,8.2107e+01_rb,7.0383e+01_rb,5.8658e+01_rb,4.6921e+01_rb,3.5193e+01_rb, &
     &2.3462e+01_rb,1.1732e+01_rb,1.3235e-03_rb/)
      kao(:, 5,11, 5) = (/ &
     &9.5958e+01_rb,8.3967e+01_rb,7.1970e+01_rb,5.9973e+01_rb,4.7982e+01_rb,3.5987e+01_rb, &
     &2.3994e+01_rb,1.1998e+01_rb,1.5458e-03_rb/)
      kao(:, 1,12, 5) = (/ &
     &9.6680e+01_rb,8.4597e+01_rb,7.2516e+01_rb,6.0430e+01_rb,4.8347e+01_rb,3.6257e+01_rb, &
     &2.4172e+01_rb,1.2087e+01_rb,8.8792e-04_rb/)
      kao(:, 2,12, 5) = (/ &
     &9.9039e+01_rb,8.6662e+01_rb,7.4284e+01_rb,6.1903e+01_rb,4.9522e+01_rb,3.7144e+01_rb, &
     &2.4762e+01_rb,1.2382e+01_rb,1.0348e-03_rb/)
      kao(:, 3,12, 5) = (/ &
     &1.0152e+02_rb,8.8834e+01_rb,7.6150e+01_rb,6.3450e+01_rb,5.0765e+01_rb,3.8071e+01_rb, &
     &2.5383e+01_rb,1.2692e+01_rb,1.2734e-03_rb/)
      kao(:, 4,12, 5) = (/ &
     &1.0389e+02_rb,9.0902e+01_rb,7.7912e+01_rb,6.4935e+01_rb,5.1946e+01_rb,3.8960e+01_rb, &
     &2.5973e+01_rb,1.2989e+01_rb,1.5461e-03_rb/)
      kao(:, 5,12, 5) = (/ &
     &1.0614e+02_rb,9.2872e+01_rb,7.9616e+01_rb,6.6343e+01_rb,5.3076e+01_rb,3.9807e+01_rb, &
     &2.6538e+01_rb,1.3271e+01_rb,1.7863e-03_rb/)
      kao(:, 1,13, 5) = (/ &
     &8.5924e+01_rb,7.5190e+01_rb,6.4446e+01_rb,5.3705e+01_rb,4.2964e+01_rb,3.2222e+01_rb, &
     &2.1482e+01_rb,1.0742e+01_rb,9.3650e-04_rb/)
      kao(:, 2,13, 5) = (/ &
     &8.8011e+01_rb,7.7009e+01_rb,6.6010e+01_rb,5.5012e+01_rb,4.4009e+01_rb,3.3008e+01_rb, &
     &2.2004e+01_rb,1.1003e+01_rb,1.0453e-03_rb/)
      kao(:, 3,13, 5) = (/ &
     &9.0165e+01_rb,7.8893e+01_rb,6.7623e+01_rb,5.6350e+01_rb,4.5080e+01_rb,3.3811e+01_rb, &
     &2.2542e+01_rb,1.1273e+01_rb,1.2632e-03_rb/)
      kao(:, 4,13, 5) = (/ &
     &9.2294e+01_rb,8.0759e+01_rb,6.9218e+01_rb,5.7683e+01_rb,4.6146e+01_rb,3.4609e+01_rb, &
     &2.3075e+01_rb,1.1539e+01_rb,1.5901e-03_rb/)
      kao(:, 5,13, 5) = (/ &
     &9.4515e+01_rb,8.2699e+01_rb,7.0891e+01_rb,5.9075e+01_rb,4.7260e+01_rb,3.5445e+01_rb, &
     &2.3631e+01_rb,1.1818e+01_rb,1.9283e-03_rb/)
      kao(:, 1, 1, 6) = (/ &
     &1.3664e-01_rb,1.2142e-01_rb,1.0604e-01_rb,9.0673e-02_rb,7.5307e-02_rb,5.9951e-02_rb, &
     &4.4049e-02_rb,2.7037e-02_rb,3.4200e-03_rb/)
      kao(:, 2, 1, 6) = (/ &
     &1.3479e-01_rb,1.2041e-01_rb,1.0581e-01_rb,9.1114e-02_rb,7.6669e-02_rb,6.1851e-02_rb, &
     &4.6009e-02_rb,2.9151e-02_rb,4.4244e-03_rb/)
      kao(:, 3, 1, 6) = (/ &
     &1.3313e-01_rb,1.1956e-01_rb,1.0561e-01_rb,9.1887e-02_rb,7.8237e-02_rb,6.3771e-02_rb, &
     &4.8109e-02_rb,3.1604e-02_rb,5.5644e-03_rb/)
      kao(:, 4, 1, 6) = (/ &
     &1.3195e-01_rb,1.1904e-01_rb,1.0583e-01_rb,9.3212e-02_rb,7.9889e-02_rb,6.5652e-02_rb, &
     &5.0318e-02_rb,3.4334e-02_rb,6.8097e-03_rb/)
      kao(:, 5, 1, 6) = (/ &
     &1.3100e-01_rb,1.1871e-01_rb,1.0645e-01_rb,9.4491e-02_rb,8.1510e-02_rb,6.7455e-02_rb, &
     &5.2648e-02_rb,3.7142e-02_rb,8.1798e-03_rb/)
      kao(:, 1, 2, 6) = (/ &
     &1.8948e-01_rb,1.6745e-01_rb,1.4512e-01_rb,1.2300e-01_rb,1.0098e-01_rb,7.9021e-02_rb, &
     &5.6595e-02_rb,3.3068e-02_rb,2.5064e-03_rb/)
      kao(:, 2, 2, 6) = (/ &
     &1.8719e-01_rb,1.6582e-01_rb,1.4439e-01_rb,1.2310e-01_rb,1.0186e-01_rb,8.0667e-02_rb, &
     &5.8428e-02_rb,3.4918e-02_rb,3.2805e-03_rb/)
      kao(:, 3, 2, 6) = (/ &
     &1.8550e-01_rb,1.6475e-01_rb,1.4416e-01_rb,1.2351e-01_rb,1.0330e-01_rb,8.2465e-02_rb, &
     &6.0314e-02_rb,3.6962e-02_rb,4.1499e-03_rb/)
      kao(:, 4, 2, 6) = (/ &
     &1.8451e-01_rb,1.6460e-01_rb,1.4454e-01_rb,1.2487e-01_rb,1.0515e-01_rb,8.4419e-02_rb, &
     &6.2353e-02_rb,3.9320e-02_rb,5.1265e-03_rb/)
      kao(:, 5, 2, 6) = (/ &
     &1.8423e-01_rb,1.6501e-01_rb,1.4553e-01_rb,1.2672e-01_rb,1.0715e-01_rb,8.6338e-02_rb, &
     &6.4353e-02_rb,4.1893e-02_rb,6.2102e-03_rb/)
      kao(:, 1, 3, 6) = (/ &
     &3.5249e-01_rb,3.0976e-01_rb,2.6683e-01_rb,2.2378e-01_rb,1.8098e-01_rb,1.3846e-01_rb, &
     &9.6219e-02_rb,5.2862e-02_rb,1.7476e-03_rb/)
      kao(:, 2, 3, 6) = (/ &
     &3.4945e-01_rb,3.0729e-01_rb,2.6503e-01_rb,2.2304e-01_rb,1.8135e-01_rb,1.3971e-01_rb, &
     &9.8147e-02_rb,5.4704e-02_rb,2.3082e-03_rb/)
      kao(:, 3, 3, 6) = (/ &
     &3.4786e-01_rb,3.0616e-01_rb,2.6475e-01_rb,2.2367e-01_rb,1.8257e-01_rb,1.4178e-01_rb, &
     &1.0043e-01_rb,5.6633e-02_rb,2.9556e-03_rb/)
      kao(:, 4, 3, 6) = (/ &
     &3.4805e-01_rb,3.0690e-01_rb,2.6623e-01_rb,2.2552e-01_rb,1.8508e-01_rb,1.4480e-01_rb, &
     &1.0306e-01_rb,5.8846e-02_rb,3.6970e-03_rb/)
      kao(:, 5, 3, 6) = (/ &
     &3.5013e-01_rb,3.0948e-01_rb,2.6918e-01_rb,2.2850e-01_rb,1.8865e-01_rb,1.4799e-01_rb, &
     &1.0568e-01_rb,6.1131e-02_rb,4.5281e-03_rb/)
      kao(:, 1, 4, 6) = (/ &
     &6.6623e-01_rb,5.8386e-01_rb,5.0153e-01_rb,4.1899e-01_rb,3.3640e-01_rb,2.5407e-01_rb, &
     &1.7225e-01_rb,9.1058e-02_rb,1.1958e-03_rb/)
      kao(:, 2, 4, 6) = (/ &
     &6.6340e-01_rb,5.8167e-01_rb,4.9974e-01_rb,4.1776e-01_rb,3.3619e-01_rb,2.5503e-01_rb, &
     &1.7423e-01_rb,9.3289e-02_rb,1.6080e-03_rb/)
      kao(:, 3, 4, 6) = (/ &
     &6.6480e-01_rb,5.8306e-01_rb,5.0121e-01_rb,4.1990e-01_rb,3.3893e-01_rb,2.5818e-01_rb, &
     &1.7768e-01_rb,9.5987e-02_rb,2.0929e-03_rb/)
      kao(:, 4, 4, 6) = (/ &
     &6.7003e-01_rb,5.8789e-01_rb,5.0627e-01_rb,4.2518e-01_rb,3.4417e-01_rb,2.6306e-01_rb, &
     &1.8230e-01_rb,9.9134e-02_rb,2.6527e-03_rb/)
      kao(:, 5, 4, 6) = (/ &
     &6.7815e-01_rb,5.9566e-01_rb,5.1416e-01_rb,4.3281e-01_rb,3.5098e-01_rb,2.6952e-01_rb, &
     &1.8764e-01_rb,1.0249e-01_rb,3.2896e-03_rb/)
      kao(:, 1, 5, 6) = (/ &
     &1.1550e+00_rb,1.0112e+00_rb,8.6772e-01_rb,7.2399e-01_rb,5.8018e-01_rb,4.3620e-01_rb, &
     &2.9244e-01_rb,1.4976e-01_rb,8.1069e-04_rb/)
      kao(:, 2, 5, 6) = (/ &
     &1.1564e+00_rb,1.0128e+00_rb,8.6891e-01_rb,7.2534e-01_rb,5.8147e-01_rb,4.3781e-01_rb, &
     &2.9477e-01_rb,1.5282e-01_rb,1.1157e-03_rb/)
      kao(:, 3, 5, 6) = (/ &
     &1.1669e+00_rb,1.0223e+00_rb,8.7752e-01_rb,7.3265e-01_rb,5.8799e-01_rb,4.4399e-01_rb, &
     &3.0054e-01_rb,1.5735e-01_rb,1.4779e-03_rb/)
      kao(:, 4, 5, 6) = (/ &
     &1.1830e+00_rb,1.0368e+00_rb,8.9030e-01_rb,7.4407e-01_rb,5.9861e-01_rb,4.5354e-01_rb, &
     &3.0826e-01_rb,1.6275e-01_rb,1.9001e-03_rb/)
      kao(:, 5, 5, 6) = (/ &
     &1.2016e+00_rb,1.0534e+00_rb,9.0529e-01_rb,7.5813e-01_rb,6.1140e-01_rb,4.6432e-01_rb, &
     &3.1746e-01_rb,1.6866e-01_rb,2.3885e-03_rb/)
      kao(:, 1, 6, 6) = (/ &
     &1.8728e+00_rb,1.6394e+00_rb,1.4057e+00_rb,1.1724e+00_rb,9.3873e-01_rb,7.0502e-01_rb, &
     &4.7119e-01_rb,2.3746e-01_rb,5.3401e-04_rb/)
      kao(:, 2, 6, 6) = (/ &
     &1.8855e+00_rb,1.6506e+00_rb,1.4159e+00_rb,1.1808e+00_rb,9.4575e-01_rb,7.1049e-01_rb, &
     &4.7534e-01_rb,2.4122e-01_rb,7.5510e-04_rb/)
      kao(:, 3, 6, 6) = (/ &
     &1.9127e+00_rb,1.6747e+00_rb,1.4367e+00_rb,1.1985e+00_rb,9.6017e-01_rb,7.2202e-01_rb, &
     &4.8438e-01_rb,2.4807e-01_rb,1.0228e-03_rb/)
      kao(:, 4, 6, 6) = (/ &
     &1.9458e+00_rb,1.7041e+00_rb,1.4622e+00_rb,1.2202e+00_rb,9.7826e-01_rb,7.3694e-01_rb, &
     &4.9644e-01_rb,2.5634e-01_rb,1.3413e-03_rb/)
      kao(:, 5, 6, 6) = (/ &
     &1.9819e+00_rb,1.7360e+00_rb,1.4899e+00_rb,1.2440e+00_rb,9.9883e-01_rb,7.5442e-01_rb, &
     &5.0981e-01_rb,2.6524e-01_rb,1.7130e-03_rb/)
      kao(:, 1, 7, 6) = (/ &
     &3.1956e+00_rb,2.7964e+00_rb,2.3973e+00_rb,1.9983e+00_rb,1.5994e+00_rb,1.2005e+00_rb, &
     &8.0140e-01_rb,4.0204e-01_rb,3.4739e-04_rb/)
      kao(:, 2, 7, 6) = (/ &
     &3.2319e+00_rb,2.8287e+00_rb,2.4254e+00_rb,2.0219e+00_rb,1.6186e+00_rb,1.2150e+00_rb, &
     &8.1138e-01_rb,4.0767e-01_rb,4.9767e-04_rb/)
      kao(:, 3, 7, 6) = (/ &
     &3.2870e+00_rb,2.8773e+00_rb,2.4673e+00_rb,2.0573e+00_rb,1.6471e+00_rb,1.2367e+00_rb, &
     &8.2648e-01_rb,4.1699e-01_rb,6.8883e-04_rb/)
      kao(:, 4, 7, 6) = (/ &
     &3.3536e+00_rb,2.9357e+00_rb,2.5178e+00_rb,2.0997e+00_rb,1.6816e+00_rb,1.2633e+00_rb, &
     &8.4550e-01_rb,4.2915e-01_rb,9.2075e-04_rb/)
      kao(:, 5, 7, 6) = (/ &
     &3.4247e+00_rb,2.9983e+00_rb,2.5718e+00_rb,2.1451e+00_rb,1.7185e+00_rb,1.2924e+00_rb, &
     &8.6745e-01_rb,4.4266e-01_rb,1.1994e-03_rb/)
      kao(:, 1, 8, 6) = (/ &
     &6.5522e+00_rb,5.7333e+00_rb,4.9154e+00_rb,4.0958e+00_rb,3.2773e+00_rb,2.4586e+00_rb, &
     &1.6399e+00_rb,8.2128e-01_rb,2.5000e-04_rb/)
      kao(:, 2, 8, 6) = (/ &
     &6.6534e+00_rb,5.8224e+00_rb,4.9909e+00_rb,4.1598e+00_rb,3.3284e+00_rb,2.4975e+00_rb, &
     &1.6667e+00_rb,8.3486e-01_rb,3.5933e-04_rb/)
      kao(:, 3, 8, 6) = (/ &
     &6.7871e+00_rb,5.9396e+00_rb,5.0919e+00_rb,4.2443e+00_rb,3.3968e+00_rb,2.5492e+00_rb, &
     &1.7012e+00_rb,8.5283e-01_rb,4.9387e-04_rb/)
      kao(:, 4, 8, 6) = (/ &
     &6.9317e+00_rb,6.0662e+00_rb,5.2009e+00_rb,4.3357e+00_rb,3.4705e+00_rb,2.6048e+00_rb, &
     &1.7389e+00_rb,8.7334e-01_rb,6.4756e-04_rb/)
      kao(:, 5, 8, 6) = (/ &
     &7.0935e+00_rb,6.2081e+00_rb,5.3230e+00_rb,4.4380e+00_rb,3.5524e+00_rb,2.6671e+00_rb, &
     &1.7815e+00_rb,8.9762e-01_rb,8.2195e-04_rb/)
      kao(:, 1, 9, 6) = (/ &
     &2.5444e+01_rb,2.2265e+01_rb,1.9084e+01_rb,1.5904e+01_rb,1.2723e+01_rb,9.5428e+00_rb, &
     &6.3622e+00_rb,3.1820e+00_rb,4.0178e-04_rb/)
      kao(:, 2, 9, 6) = (/ &
     &2.5907e+01_rb,2.2668e+01_rb,1.9432e+01_rb,1.6194e+01_rb,1.2956e+01_rb,9.7177e+00_rb, &
     &6.4794e+00_rb,3.2416e+00_rb,4.9820e-04_rb/)
      kao(:, 3, 9, 6) = (/ &
     &2.6491e+01_rb,2.3181e+01_rb,1.9870e+01_rb,1.6559e+01_rb,1.3249e+01_rb,9.9392e+00_rb, &
     &6.6274e+00_rb,3.3163e+00_rb,5.8675e-04_rb/)
      kao(:, 4, 9, 6) = (/ &
     &2.7100e+01_rb,2.3714e+01_rb,2.0326e+01_rb,1.6941e+01_rb,1.3555e+01_rb,1.0168e+01_rb, &
     &6.7813e+00_rb,3.3943e+00_rb,7.0841e-04_rb/)
      kao(:, 5, 9, 6) = (/ &
     &2.7765e+01_rb,2.4297e+01_rb,2.0827e+01_rb,1.7358e+01_rb,1.3889e+01_rb,1.0420e+01_rb, &
     &6.9499e+00_rb,3.4794e+00_rb,8.5424e-04_rb/)
      kao(:, 1,10, 6) = (/ &
     &1.1023e+02_rb,9.6432e+01_rb,8.2664e+01_rb,6.8898e+01_rb,5.5114e+01_rb,4.1339e+01_rb, &
     &2.7555e+01_rb,1.3778e+01_rb,8.1537e-04_rb/)
      kao(:, 2,10, 6) = (/ &
     &1.1247e+02_rb,9.8427e+01_rb,8.4346e+01_rb,7.0300e+01_rb,5.6236e+01_rb,4.2180e+01_rb, &
     &2.8118e+01_rb,1.4061e+01_rb,9.9310e-04_rb/)
      kao(:, 3,10, 6) = (/ &
     &1.1514e+02_rb,1.0076e+02_rb,8.6358e+01_rb,7.1971e+01_rb,5.7576e+01_rb,4.3183e+01_rb, &
     &2.8789e+01_rb,1.4396e+01_rb,1.1831e-03_rb/)
      kao(:, 4,10, 6) = (/ &
     &1.1802e+02_rb,1.0327e+02_rb,8.8510e+01_rb,7.3765e+01_rb,5.9016e+01_rb,4.4263e+01_rb, &
     &2.9511e+01_rb,1.4759e+01_rb,1.3868e-03_rb/)
      kao(:, 5,10, 6) = (/ &
     &1.2130e+02_rb,1.0614e+02_rb,9.0978e+01_rb,7.5815e+01_rb,6.0657e+01_rb,4.5493e+01_rb, &
     &3.0333e+01_rb,1.5172e+01_rb,1.7070e-03_rb/)
      kao(:, 1,11, 6) = (/ &
     &1.6690e+02_rb,1.4603e+02_rb,1.2516e+02_rb,1.0431e+02_rb,8.3442e+01_rb,6.2583e+01_rb, &
     &4.1722e+01_rb,2.0862e+01_rb,1.5727e-03_rb/)
      kao(:, 2,11, 6) = (/ &
     &1.7086e+02_rb,1.4949e+02_rb,1.2814e+02_rb,1.0678e+02_rb,8.5424e+01_rb,6.4067e+01_rb, &
     &4.2718e+01_rb,2.1360e+01_rb,1.6234e-03_rb/)
      kao(:, 3,11, 6) = (/ &
     &1.7519e+02_rb,1.5329e+02_rb,1.3139e+02_rb,1.0949e+02_rb,8.7589e+01_rb,6.5698e+01_rb, &
     &4.3799e+01_rb,2.1903e+01_rb,1.7884e-03_rb/)
      kao(:, 4,11, 6) = (/ &
     &1.8016e+02_rb,1.5762e+02_rb,1.3510e+02_rb,1.1259e+02_rb,9.0076e+01_rb,6.7557e+01_rb, &
     &4.5039e+01_rb,2.2524e+01_rb,2.1215e-03_rb/)
      kao(:, 5,11, 6) = (/ &
     &1.8518e+02_rb,1.6203e+02_rb,1.3888e+02_rb,1.1573e+02_rb,9.2591e+01_rb,6.9449e+01_rb, &
     &4.6302e+01_rb,2.3157e+01_rb,2.4813e-03_rb/)
      kao(:, 1,12, 6) = (/ &
     &1.8707e+02_rb,1.6368e+02_rb,1.4030e+02_rb,1.1692e+02_rb,9.3541e+01_rb,7.0153e+01_rb, &
     &4.6766e+01_rb,2.3384e+01_rb,1.9770e-03_rb/)
      kao(:, 2,12, 6) = (/ &
     &1.9183e+02_rb,1.6788e+02_rb,1.4389e+02_rb,1.1990e+02_rb,9.5932e+01_rb,7.1949e+01_rb, &
     &4.7966e+01_rb,2.3985e+01_rb,2.1043e-03_rb/)
      kao(:, 3,12, 6) = (/ &
     &1.9710e+02_rb,1.7246e+02_rb,1.4784e+02_rb,1.2320e+02_rb,9.8552e+01_rb,7.3917e+01_rb, &
     &4.9278e+01_rb,2.4644e+01_rb,2.2289e-03_rb/)
      kao(:, 4,12, 6) = (/ &
     &2.0265e+02_rb,1.7731e+02_rb,1.5198e+02_rb,1.2666e+02_rb,1.0133e+02_rb,7.5997e+01_rb, &
     &5.0665e+01_rb,2.5338e+01_rb,2.5070e-03_rb/)
      kao(:, 5,12, 6) = (/ &
     &2.0830e+02_rb,1.8227e+02_rb,1.5623e+02_rb,1.3019e+02_rb,1.0416e+02_rb,7.8120e+01_rb, &
     &5.2086e+01_rb,2.6047e+01_rb,2.9426e-03_rb/)
      kao(:, 1,13, 6) = (/ &
     &1.6768e+02_rb,1.4672e+02_rb,1.2576e+02_rb,1.0480e+02_rb,8.3836e+01_rb,6.2874e+01_rb, &
     &4.1918e+01_rb,2.0960e+01_rb,1.9900e-03_rb/)
      kao(:, 2,13, 6) = (/ &
     &1.7225e+02_rb,1.5072e+02_rb,1.2919e+02_rb,1.0767e+02_rb,8.6132e+01_rb,6.4601e+01_rb, &
     &4.3064e+01_rb,2.1534e+01_rb,2.2115e-03_rb/)
      kao(:, 3,13, 6) = (/ &
     &1.7710e+02_rb,1.5497e+02_rb,1.3283e+02_rb,1.1069e+02_rb,8.8549e+01_rb,6.6413e+01_rb, &
     &4.4281e+01_rb,2.2144e+01_rb,2.4087e-03_rb/)
      kao(:, 4,13, 6) = (/ &
     &1.8201e+02_rb,1.5926e+02_rb,1.3651e+02_rb,1.1376e+02_rb,9.1007e+01_rb,6.8257e+01_rb, &
     &4.5512e+01_rb,2.2758e+01_rb,2.6872e-03_rb/)
      kao(:, 5,13, 6) = (/ &
     &1.8703e+02_rb,1.6366e+02_rb,1.4029e+02_rb,1.1690e+02_rb,9.3529e+01_rb,7.0144e+01_rb, &
     &4.6769e+01_rb,2.3390e+01_rb,2.9430e-03_rb/)
      kao(:, 1, 1, 7) = (/ &
     &2.6412e-01_rb,2.3209e-01_rb,2.0136e-01_rb,1.7041e-01_rb,1.3987e-01_rb,1.0997e-01_rb, &
     &7.9977e-02_rb,4.8294e-02_rb,7.1406e-03_rb/)
      kao(:, 2, 1, 7) = (/ &
     &2.6114e-01_rb,2.3041e-01_rb,2.0084e-01_rb,1.7141e-01_rb,1.4253e-01_rb,1.1370e-01_rb, &
     &8.4426e-02_rb,5.2292e-02_rb,9.2339e-03_rb/)
      kao(:, 3, 1, 7) = (/ &
     &2.5746e-01_rb,2.2846e-01_rb,2.0043e-01_rb,1.7294e-01_rb,1.4558e-01_rb,1.1774e-01_rb, &
     &8.8686e-02_rb,5.6161e-02_rb,1.1607e-02_rb/)
      kao(:, 4, 1, 7) = (/ &
     &2.5329e-01_rb,2.2641e-01_rb,2.0046e-01_rb,1.7465e-01_rb,1.4868e-01_rb,1.2202e-01_rb, &
     &9.2933e-02_rb,6.0129e-02_rb,1.4258e-02_rb/)
      kao(:, 5, 1, 7) = (/ &
     &2.4914e-01_rb,2.2449e-01_rb,2.0080e-01_rb,1.7670e-01_rb,1.5234e-01_rb,1.2632e-01_rb, &
     &9.7469e-02_rb,6.4803e-02_rb,1.7193e-02_rb/)
      kao(:, 1, 2, 7) = (/ &
     &3.8226e-01_rb,3.3530e-01_rb,2.8943e-01_rb,2.4384e-01_rb,1.9786e-01_rb,1.5259e-01_rb, &
     &1.0817e-01_rb,6.3161e-02_rb,5.4372e-03_rb/)
      kao(:, 2, 2, 7) = (/ &
     &3.7769e-01_rb,3.3226e-01_rb,2.8798e-01_rb,2.4326e-01_rb,1.9923e-01_rb,1.5589e-01_rb, &
     &1.1262e-01_rb,6.7329e-02_rb,7.0870e-03_rb/)
      kao(:, 3, 2, 7) = (/ &
     &3.7182e-01_rb,3.2842e-01_rb,2.8559e-01_rb,2.4304e-01_rb,2.0123e-01_rb,1.5952e-01_rb, &
     &1.1734e-01_rb,7.1516e-02_rb,8.9693e-03_rb/)
      kao(:, 4, 2, 7) = (/ &
     &3.6545e-01_rb,3.2421e-01_rb,2.8324e-01_rb,2.4318e-01_rb,2.0343e-01_rb,1.6337e-01_rb, &
     &1.2179e-01_rb,7.5704e-02_rb,1.1062e-02_rb/)
      kao(:, 5, 2, 7) = (/ &
     &3.5934e-01_rb,3.2018e-01_rb,2.8184e-01_rb,2.4404e-01_rb,2.0628e-01_rb,1.6781e-01_rb, &
     &1.2654e-01_rb,8.0391e-02_rb,1.3382e-02_rb/)
      kao(:, 1, 3, 7) = (/ &
     &7.3419e-01_rb,6.4302e-01_rb,5.5249e-01_rb,4.6249e-01_rb,3.7279e-01_rb,2.8244e-01_rb, &
     &1.9254e-01_rb,1.0461e-01_rb,3.8320e-03_rb/)
      kao(:, 2, 3, 7) = (/ &
     &7.2438e-01_rb,6.3531e-01_rb,5.4697e-01_rb,4.5910e-01_rb,3.7055e-01_rb,2.8234e-01_rb, &
     &1.9553e-01_rb,1.0953e-01_rb,5.0607e-03_rb/)
      kao(:, 3, 3, 7) = (/ &
     &7.1308e-01_rb,6.2644e-01_rb,5.4060e-01_rb,4.5437e-01_rb,3.6826e-01_rb,2.8333e-01_rb, &
     &1.9916e-01_rb,1.1441e-01_rb,6.4784e-03_rb/)
      kao(:, 4, 3, 7) = (/ &
     &7.0097e-01_rb,6.1696e-01_rb,5.3353e-01_rb,4.4963e-01_rb,3.6700e-01_rb,2.8530e-01_rb, &
     &2.0351e-01_rb,1.1914e-01_rb,8.0908e-03_rb/)
      kao(:, 5, 3, 7) = (/ &
     &6.8962e-01_rb,6.0838e-01_rb,5.2688e-01_rb,4.4648e-01_rb,3.6708e-01_rb,2.8814e-01_rb, &
     &2.0859e-01_rb,1.2387e-01_rb,9.8988e-03_rb/)
      kao(:, 1, 4, 7) = (/ &
     &1.4238e+00_rb,1.2465e+00_rb,1.0693e+00_rb,8.9242e-01_rb,7.1591e-01_rb,5.3976e-01_rb, &
     &3.6322e-01_rb,1.8668e-01_rb,2.6351e-03_rb/)
      kao(:, 2, 4, 7) = (/ &
     &1.4051e+00_rb,1.2306e+00_rb,1.0566e+00_rb,8.8285e-01_rb,7.0951e-01_rb,5.3582e-01_rb, &
     &3.6165e-01_rb,1.8980e-01_rb,3.5337e-03_rb/)
      kao(:, 3, 4, 7) = (/ &
     &1.3826e+00_rb,1.2117e+00_rb,1.0413e+00_rb,8.7137e-01_rb,7.0120e-01_rb,5.3024e-01_rb, &
     &3.6085e-01_rb,1.9367e-01_rb,4.5912e-03_rb/)
      kao(:, 4, 4, 7) = (/ &
     &1.3609e+00_rb,1.1937e+00_rb,1.0271e+00_rb,8.6049e-01_rb,6.9280e-01_rb,5.2650e-01_rb, &
     &3.6231e-01_rb,1.9848e-01_rb,5.8142e-03_rb/)
      kao(:, 5, 4, 7) = (/ &
     &1.3409e+00_rb,1.1773e+00_rb,1.0140e+00_rb,8.4932e-01_rb,6.8606e-01_rb,5.2486e-01_rb, &
     &3.6482e-01_rb,2.0369e-01_rb,7.2048e-03_rb/)
      kao(:, 1, 5, 7) = (/ &
     &2.5206e+00_rb,2.2062e+00_rb,1.8917e+00_rb,1.5773e+00_rb,1.2631e+00_rb,9.4923e-01_rb, &
     &6.3588e-01_rb,3.2194e-01_rb,1.8039e-03_rb/)
      kao(:, 2, 5, 7) = (/ &
     &2.4864e+00_rb,2.1765e+00_rb,1.8669e+00_rb,1.5575e+00_rb,1.2483e+00_rb,9.3953e-01_rb, &
     &6.3051e-01_rb,3.2080e-01_rb,2.4598e-03_rb/)
      kao(:, 3, 5, 7) = (/ &
     &2.4502e+00_rb,2.1455e+00_rb,1.8410e+00_rb,1.5370e+00_rb,1.2333e+00_rb,9.2942e-01_rb, &
     &6.2403e-01_rb,3.2160e-01_rb,3.2471e-03_rb/)
      kao(:, 4, 5, 7) = (/ &
     &2.4176e+00_rb,2.1177e+00_rb,1.8181e+00_rb,1.5189e+00_rb,1.2193e+00_rb,9.1845e-01_rb, &
     &6.1937e-01_rb,3.2415e-01_rb,4.1727e-03_rb/)
      kao(:, 5, 5, 7) = (/ &
     &2.3943e+00_rb,2.0977e+00_rb,1.8015e+00_rb,1.5055e+00_rb,1.2082e+00_rb,9.1210e-01_rb, &
     &6.1892e-01_rb,3.2856e-01_rb,5.2383e-03_rb/)
      kao(:, 1, 6, 7) = (/ &
     &4.1535e+00_rb,3.6349e+00_rb,3.1163e+00_rb,2.5976e+00_rb,2.0789e+00_rb,1.5604e+00_rb, &
     &1.0424e+00_rb,5.2499e-01_rb,1.2119e-03_rb/)
      kao(:, 2, 6, 7) = (/ &
     &4.0999e+00_rb,3.5882e+00_rb,3.0765e+00_rb,2.5650e+00_rb,2.0536e+00_rb,1.5426e+00_rb, &
     &1.0319e+00_rb,5.2076e-01_rb,1.6878e-03_rb/)
      kao(:, 3, 6, 7) = (/ &
     &4.0501e+00_rb,3.5452e+00_rb,3.0400e+00_rb,2.5354e+00_rb,2.0310e+00_rb,1.5269e+00_rb, &
     &1.0226e+00_rb,5.1681e-01_rb,2.2703e-03_rb/)
      kao(:, 4, 6, 7) = (/ &
     &4.0172e+00_rb,3.5170e+00_rb,3.0163e+00_rb,2.5163e+00_rb,2.0166e+00_rb,1.5168e+00_rb, &
     &1.0153e+00_rb,5.1695e-01_rb,2.9634e-03_rb/)
      kao(:, 5, 6, 7) = (/ &
     &4.0047e+00_rb,3.5059e+00_rb,3.0081e+00_rb,2.5099e+00_rb,2.0120e+00_rb,1.5128e+00_rb, &
     &1.0149e+00_rb,5.2194e-01_rb,3.7748e-03_rb/)
      kao(:, 1, 7, 7) = (/ &
     &7.1454e+00_rb,6.2526e+00_rb,5.3597e+00_rb,4.4669e+00_rb,3.5744e+00_rb,2.6816e+00_rb, &
     &1.7891e+00_rb,8.9724e-01_rb,7.5005e-04_rb/)
      kao(:, 2, 7, 7) = (/ &
     &7.0662e+00_rb,6.1833e+00_rb,5.3006e+00_rb,4.4177e+00_rb,3.5356e+00_rb,2.6534e+00_rb, &
     &1.7715e+00_rb,8.9007e-01_rb,1.0984e-03_rb/)
      kao(:, 3, 7, 7) = (/ &
     &7.0091e+00_rb,6.1337e+00_rb,5.2586e+00_rb,4.3836e+00_rb,3.5085e+00_rb,2.6339e+00_rb, &
     &1.7596e+00_rb,8.8470e-01_rb,1.5427e-03_rb/)
      kao(:, 4, 7, 7) = (/ &
     &6.9967e+00_rb,6.1229e+00_rb,5.2499e+00_rb,4.3767e+00_rb,3.5037e+00_rb,2.6314e+00_rb, &
     &1.7589e+00_rb,8.8464e-01_rb,2.0814e-03_rb/)
      kao(:, 5, 7, 7) = (/ &
     &7.0262e+00_rb,6.1495e+00_rb,5.2733e+00_rb,4.3969e+00_rb,3.5208e+00_rb,2.6451e+00_rb, &
     &1.7672e+00_rb,8.9234e-01_rb,2.7067e-03_rb/)
      kao(:, 1, 8, 7) = (/ &
     &1.4691e+01_rb,1.2856e+01_rb,1.1019e+01_rb,9.1832e+00_rb,7.3472e+00_rb,5.5112e+00_rb, &
     &3.6752e+00_rb,1.8392e+00_rb,5.0466e-04_rb/)
      kao(:, 2, 8, 7) = (/ &
     &1.4561e+01_rb,1.2741e+01_rb,1.0922e+01_rb,9.1018e+00_rb,7.2824e+00_rb,5.4631e+00_rb, &
     &3.6434e+00_rb,1.8247e+00_rb,7.0108e-04_rb/)
      kao(:, 3, 8, 7) = (/ &
     &1.4517e+01_rb,1.2702e+01_rb,1.0889e+01_rb,9.0751e+00_rb,7.2609e+00_rb,5.4474e+00_rb, &
     &3.6338e+00_rb,1.8215e+00_rb,9.5134e-04_rb/)
      kao(:, 4, 8, 7) = (/ &
     &1.4592e+01_rb,1.2768e+01_rb,1.0945e+01_rb,9.1223e+00_rb,7.2994e+00_rb,5.4774e+00_rb, &
     &3.6552e+00_rb,1.8334e+00_rb,1.2923e-03_rb/)
      kao(:, 5, 8, 7) = (/ &
     &1.4780e+01_rb,1.2933e+01_rb,1.1086e+01_rb,9.2409e+00_rb,7.3952e+00_rb,5.5498e+00_rb, &
     &3.7048e+00_rb,1.8578e+00_rb,1.7359e-03_rb/)
      kao(:, 1, 9, 7) = (/ &
     &5.6885e+01_rb,4.9776e+01_rb,4.2665e+01_rb,3.5554e+01_rb,2.8444e+01_rb,2.1332e+01_rb, &
     &1.4224e+01_rb,7.1126e+00_rb,6.2609e-04_rb/)
      kao(:, 2, 9, 7) = (/ &
     &5.6580e+01_rb,4.9509e+01_rb,4.2436e+01_rb,3.5363e+01_rb,2.8291e+01_rb,2.1219e+01_rb, &
     &1.4147e+01_rb,7.0751e+00_rb,8.0177e-04_rb/)
      kao(:, 3, 9, 7) = (/ &
     &5.6732e+01_rb,4.9637e+01_rb,4.2548e+01_rb,3.5458e+01_rb,2.8366e+01_rb,2.1276e+01_rb, &
     &1.4184e+01_rb,7.0949e+00_rb,1.0220e-03_rb/)
      kao(:, 4, 9, 7) = (/ &
     &5.7468e+01_rb,5.0286e+01_rb,4.3101e+01_rb,3.5920e+01_rb,2.8736e+01_rb,2.1553e+01_rb, &
     &1.4371e+01_rb,7.1892e+00_rb,1.2387e-03_rb/)
      kao(:, 5, 9, 7) = (/ &
     &5.8635e+01_rb,5.1306e+01_rb,4.3977e+01_rb,3.6648e+01_rb,2.9321e+01_rb,2.1992e+01_rb, &
     &1.4665e+01_rb,7.3378e+00_rb,1.5501e-03_rb/)
      kao(:, 1,10, 7) = (/ &
     &2.4487e+02_rb,2.1424e+02_rb,1.8364e+02_rb,1.5304e+02_rb,1.2243e+02_rb,9.1825e+01_rb, &
     &6.1214e+01_rb,3.0606e+01_rb,1.7354e-03_rb/)
      kao(:, 2,10, 7) = (/ &
     &2.4490e+02_rb,2.1431e+02_rb,1.8369e+02_rb,1.5309e+02_rb,1.2247e+02_rb,9.1851e+01_rb, &
     &6.1230e+01_rb,3.0619e+01_rb,1.7748e-03_rb/)
      kao(:, 3,10, 7) = (/ &
     &2.4755e+02_rb,2.1661e+02_rb,1.8568e+02_rb,1.5472e+02_rb,1.2378e+02_rb,9.2834e+01_rb, &
     &6.1889e+01_rb,3.0945e+01_rb,1.9607e-03_rb/)
      kao(:, 4,10, 7) = (/ &
     &2.5293e+02_rb,2.2132e+02_rb,1.8970e+02_rb,1.5807e+02_rb,1.2647e+02_rb,9.4852e+01_rb, &
     &6.3238e+01_rb,3.1620e+01_rb,2.3523e-03_rb/)
      kao(:, 5,10, 7) = (/ &
     &2.5940e+02_rb,2.2699e+02_rb,1.9456e+02_rb,1.6213e+02_rb,1.2971e+02_rb,9.7282e+01_rb, &
     &6.4856e+01_rb,3.2431e+01_rb,2.7354e-03_rb/)
      kao(:, 1,11, 7) = (/ &
     &3.6899e+02_rb,3.2288e+02_rb,2.7677e+02_rb,2.3063e+02_rb,1.8452e+02_rb,1.3837e+02_rb, &
     &9.2255e+01_rb,4.6125e+01_rb,2.1265e-03_rb/)
      kao(:, 2,11, 7) = (/ &
     &3.7335e+02_rb,3.2670e+02_rb,2.8002e+02_rb,2.3334e+02_rb,1.8669e+02_rb,1.4002e+02_rb, &
     &9.3341e+01_rb,4.6674e+01_rb,3.2080e-03_rb/)
      kao(:, 3,11, 7) = (/ &
     &3.8233e+02_rb,3.3453e+02_rb,2.8674e+02_rb,2.3896e+02_rb,1.9116e+02_rb,1.4337e+02_rb, &
     &9.5577e+01_rb,4.7794e+01_rb,3.4910e-03_rb/)
      kao(:, 4,11, 7) = (/ &
     &3.9299e+02_rb,3.4383e+02_rb,2.9472e+02_rb,2.4561e+02_rb,1.9648e+02_rb,1.4736e+02_rb, &
     &9.8242e+01_rb,4.9120e+01_rb,3.8208e-03_rb/)
      kao(:, 5,11, 7) = (/ &
     &4.0483e+02_rb,3.5423e+02_rb,3.0363e+02_rb,2.5301e+02_rb,2.0242e+02_rb,1.5182e+02_rb, &
     &1.0121e+02_rb,5.0609e+01_rb,4.2123e-03_rb/)
      kao(:, 1,12, 7) = (/ &
     &4.1937e+02_rb,3.6695e+02_rb,3.1452e+02_rb,2.6211e+02_rb,2.0969e+02_rb,1.5726e+02_rb, &
     &1.0485e+02_rb,5.2423e+01_rb,2.2784e-03_rb/)
      kao(:, 2,12, 7) = (/ &
     &4.2956e+02_rb,3.7586e+02_rb,3.2216e+02_rb,2.6848e+02_rb,2.1478e+02_rb,1.6108e+02_rb, &
     &1.0739e+02_rb,5.3697e+01_rb,3.5938e-03_rb/)
      kao(:, 3,12, 7) = (/ &
     &4.4222e+02_rb,3.8693e+02_rb,3.3164e+02_rb,2.7638e+02_rb,2.2109e+02_rb,1.6582e+02_rb, &
     &1.1055e+02_rb,5.5277e+01_rb,4.4037e-03_rb/)
      kao(:, 4,12, 7) = (/ &
     &4.5624e+02_rb,3.9921e+02_rb,3.4220e+02_rb,2.8517e+02_rb,2.2812e+02_rb,1.7109e+02_rb, &
     &1.1406e+02_rb,5.7035e+01_rb,4.5266e-03_rb/)
      kao(:, 5,12, 7) = (/ &
     &4.7013e+02_rb,4.1136e+02_rb,3.5259e+02_rb,2.9384e+02_rb,2.3506e+02_rb,1.7630e+02_rb, &
     &1.1754e+02_rb,5.8765e+01_rb,5.1545e-03_rb/)
      kao(:, 1,13, 7) = (/ &
     &3.8573e+02_rb,3.3753e+02_rb,2.8932e+02_rb,2.4110e+02_rb,1.9287e+02_rb,1.4465e+02_rb, &
     &9.6435e+01_rb,4.8220e+01_rb,2.4127e-03_rb/)
      kao(:, 2,13, 7) = (/ &
     &3.9713e+02_rb,3.4748e+02_rb,2.9784e+02_rb,2.4821e+02_rb,1.9856e+02_rb,1.4893e+02_rb, &
     &9.9282e+01_rb,4.9641e+01_rb,3.8706e-03_rb/)
      kao(:, 3,13, 7) = (/ &
     &4.1029e+02_rb,3.5900e+02_rb,3.0771e+02_rb,2.5644e+02_rb,2.0514e+02_rb,1.5386e+02_rb, &
     &1.0257e+02_rb,5.1288e+01_rb,4.7695e-03_rb/)
      kao(:, 4,13, 7) = (/ &
     &4.2344e+02_rb,3.7050e+02_rb,3.1756e+02_rb,2.6463e+02_rb,2.1170e+02_rb,1.5878e+02_rb, &
     &1.0586e+02_rb,5.2932e+01_rb,4.9706e-03_rb/)
      kao(:, 5,13, 7) = (/ &
     &4.3615e+02_rb,3.8165e+02_rb,3.2713e+02_rb,2.7260e+02_rb,2.1808e+02_rb,1.6356e+02_rb, &
     &1.0904e+02_rb,5.4523e+01_rb,5.6617e-03_rb/)
      kao(:, 1, 1, 8) = (/ &
     &5.2022e-01_rb,4.5635e-01_rb,3.9252e-01_rb,3.3010e-01_rb,2.6823e-01_rb,2.0567e-01_rb, &
     &1.4452e-01_rb,8.4865e-02_rb,1.8396e-02_rb/)
      kao(:, 2, 1, 8) = (/ &
     &5.2177e-01_rb,4.5789e-01_rb,3.9474e-01_rb,3.3342e-01_rb,2.7114e-01_rb,2.1031e-01_rb, &
     &1.5104e-01_rb,9.2174e-02_rb,2.3774e-02_rb/)
      kao(:, 3, 1, 8) = (/ &
     &5.2246e-01_rb,4.5863e-01_rb,3.9708e-01_rb,3.3604e-01_rb,2.7541e-01_rb,2.1692e-01_rb, &
     &1.5876e-01_rb,9.9961e-02_rb,3.0156e-02_rb/)
      kao(:, 4, 1, 8) = (/ &
     &5.2222e-01_rb,4.5904e-01_rb,3.9936e-01_rb,3.3918e-01_rb,2.8145e-01_rb,2.2396e-01_rb, &
     &1.6789e-01_rb,1.0815e-01_rb,3.7634e-02_rb/)
      kao(:, 5, 1, 8) = (/ &
     &5.2130e-01_rb,4.5967e-01_rb,4.0135e-01_rb,3.4393e-01_rb,2.8807e-01_rb,2.3225e-01_rb, &
     &1.7760e-01_rb,1.1669e-01_rb,4.6345e-02_rb/)
      kao(:, 1, 2, 8) = (/ &
     &8.1852e-01_rb,7.1745e-01_rb,6.1633e-01_rb,5.1564e-01_rb,4.1643e-01_rb,3.1732e-01_rb, &
     &2.1786e-01_rb,1.2067e-01_rb,1.4675e-02_rb/)
      kao(:, 2, 2, 8) = (/ &
     &8.1998e-01_rb,7.1886e-01_rb,6.1787e-01_rb,5.1883e-01_rb,4.2047e-01_rb,3.2108e-01_rb, &
     &2.2369e-01_rb,1.2867e-01_rb,1.9122e-02_rb/)
      kao(:, 3, 2, 8) = (/ &
     &8.2161e-01_rb,7.2051e-01_rb,6.2063e-01_rb,5.2305e-01_rb,4.2413e-01_rb,3.2660e-01_rb, &
     &2.3145e-01_rb,1.3782e-01_rb,2.4473e-02_rb/)
      kao(:, 4, 2, 8) = (/ &
     &8.2149e-01_rb,7.2071e-01_rb,6.2301e-01_rb,5.2587e-01_rb,4.2865e-01_rb,3.3407e-01_rb, &
     &2.4032e-01_rb,1.4761e-01_rb,3.0869e-02_rb/)
      kao(:, 5, 2, 8) = (/ &
     &8.1957e-01_rb,7.1986e-01_rb,6.2436e-01_rb,5.2838e-01_rb,4.3462e-01_rb,3.4210e-01_rb, &
     &2.5070e-01_rb,1.5726e-01_rb,3.8311e-02_rb/)
      kao(:, 1, 3, 8) = (/ &
     &1.7050e+00_rb,1.4932e+00_rb,1.2814e+00_rb,1.0695e+00_rb,8.5775e-01_rb,6.4751e-01_rb, &
     &4.3833e-01_rb,2.2806e-01_rb,1.0910e-02_rb/)
      kao(:, 2, 3, 8) = (/ &
     &1.7093e+00_rb,1.4971e+00_rb,1.2848e+00_rb,1.0726e+00_rb,8.6216e-01_rb,6.5288e-01_rb, &
     &4.4261e-01_rb,2.3435e-01_rb,1.4370e-02_rb/)
      kao(:, 3, 3, 8) = (/ &
     &1.7123e+00_rb,1.5000e+00_rb,1.2874e+00_rb,1.0764e+00_rb,8.6711e-01_rb,6.5794e-01_rb, &
     &4.4842e-01_rb,2.4360e-01_rb,1.8590e-02_rb/)
      kao(:, 4, 3, 8) = (/ &
     &1.7121e+00_rb,1.4998e+00_rb,1.2880e+00_rb,1.0792e+00_rb,8.7145e-01_rb,6.6223e-01_rb, &
     &4.5614e-01_rb,2.5394e-01_rb,2.3699e-02_rb/)
      kao(:, 5, 3, 8) = (/ &
     &1.7072e+00_rb,1.4957e+00_rb,1.2869e+00_rb,1.0807e+00_rb,8.7401e-01_rb,6.6818e-01_rb, &
     &4.6582e-01_rb,2.6619e-01_rb,2.9551e-02_rb/)
      kao(:, 1, 4, 8) = (/ &
     &3.5671e+00_rb,3.1226e+00_rb,2.6780e+00_rb,2.2334e+00_rb,1.7887e+00_rb,1.3439e+00_rb, &
     &9.0030e-01_rb,4.5879e-01_rb,7.8646e-03_rb/)
      kao(:, 2, 4, 8) = (/ &
     &3.5784e+00_rb,3.1324e+00_rb,2.6867e+00_rb,2.2405e+00_rb,1.7945e+00_rb,1.3495e+00_rb, &
     &9.0658e-01_rb,4.6395e-01_rb,1.0548e-02_rb/)
      kao(:, 3, 4, 8) = (/ &
     &3.5832e+00_rb,3.1369e+00_rb,2.6902e+00_rb,2.2437e+00_rb,1.7981e+00_rb,1.3549e+00_rb, &
     &9.1305e-01_rb,4.7036e-01_rb,1.3834e-02_rb/)
      kao(:, 4, 4, 8) = (/ &
     &3.5802e+00_rb,3.1343e+00_rb,2.6882e+00_rb,2.2430e+00_rb,1.8006e+00_rb,1.3596e+00_rb, &
     &9.1785e-01_rb,4.7927e-01_rb,1.7787e-02_rb/)
      kao(:, 5, 4, 8) = (/ &
     &3.5714e+00_rb,3.1267e+00_rb,2.6827e+00_rb,2.2417e+00_rb,1.8022e+00_rb,1.3635e+00_rb, &
     &9.2392e-01_rb,4.9076e-01_rb,2.2306e-02_rb/)
      kao(:, 1, 5, 8) = (/ &
     &6.7418e+00_rb,5.9003e+00_rb,5.0587e+00_rb,4.2171e+00_rb,3.3754e+00_rb,2.5338e+00_rb, &
     &1.6921e+00_rb,8.5173e-01_rb,5.5269e-03_rb/)
      kao(:, 2, 5, 8) = (/ &
     &6.7656e+00_rb,5.9212e+00_rb,5.0765e+00_rb,4.2319e+00_rb,3.3873e+00_rb,2.5428e+00_rb, &
     &1.6990e+00_rb,8.5857e-01_rb,7.5732e-03_rb/)
      kao(:, 3, 5, 8) = (/ &
     &6.7746e+00_rb,5.9289e+00_rb,5.0835e+00_rb,4.2380e+00_rb,3.3924e+00_rb,2.5474e+00_rb, &
     &1.7057e+00_rb,8.6548e-01_rb,1.0137e-02_rb/)
      kao(:, 4, 5, 8) = (/ &
     &6.7641e+00_rb,5.9200e+00_rb,5.0759e+00_rb,4.2320e+00_rb,3.3891e+00_rb,2.5487e+00_rb, &
     &1.7099e+00_rb,8.7100e-01_rb,1.3219e-02_rb/)
      kao(:, 5, 5, 8) = (/ &
     &6.7413e+00_rb,5.9008e+00_rb,5.0606e+00_rb,4.2207e+00_rb,3.3833e+00_rb,2.5475e+00_rb, &
     &1.7128e+00_rb,8.7813e-01_rb,1.6736e-02_rb/)
      kao(:, 1, 6, 8) = (/ &
     &1.1755e+01_rb,1.0287e+01_rb,8.8182e+00_rb,7.3494e+00_rb,5.8812e+00_rb,4.4127e+00_rb, &
     &2.9442e+00_rb,1.4757e+00_rb,3.7670e-03_rb/)
      kao(:, 2, 6, 8) = (/ &
     &1.1790e+01_rb,1.0318e+01_rb,8.8444e+00_rb,7.3718e+00_rb,5.8990e+00_rb,4.4261e+00_rb, &
     &2.9535e+00_rb,1.4824e+00_rb,5.2756e-03_rb/)
      kao(:, 3, 6, 8) = (/ &
     &1.1802e+01_rb,1.0328e+01_rb,8.8538e+00_rb,7.3796e+00_rb,5.9055e+00_rb,4.4312e+00_rb, &
     &2.9580e+00_rb,1.4892e+00_rb,7.2031e-03_rb/)
      kao(:, 4, 6, 8) = (/ &
     &1.1771e+01_rb,1.0301e+01_rb,8.8310e+00_rb,7.3612e+00_rb,5.8917e+00_rb,4.4224e+00_rb, &
     &2.9568e+00_rb,1.4927e+00_rb,9.5398e-03_rb/)
      kao(:, 5, 6, 8) = (/ &
     &1.1717e+01_rb,1.0254e+01_rb,8.7912e+00_rb,7.3292e+00_rb,5.8672e+00_rb,4.4080e+00_rb, &
     &2.9504e+00_rb,1.4942e+00_rb,1.2264e-02_rb/)
      kao(:, 1, 7, 8) = (/ &
     &2.1201e+01_rb,1.8552e+01_rb,1.5902e+01_rb,1.3253e+01_rb,1.0604e+01_rb,7.9546e+00_rb, &
     &5.3049e+00_rb,2.6554e+00_rb,2.5163e-03_rb/)
      kao(:, 2, 7, 8) = (/ &
     &2.1258e+01_rb,1.8602e+01_rb,1.5945e+01_rb,1.3288e+01_rb,1.0632e+01_rb,7.9754e+00_rb, &
     &5.3196e+00_rb,2.6631e+00_rb,3.6166e-03_rb/)
      kao(:, 3, 7, 8) = (/ &
     &2.1271e+01_rb,1.8614e+01_rb,1.5957e+01_rb,1.3298e+01_rb,1.0640e+01_rb,7.9825e+00_rb, &
     &5.3244e+00_rb,2.6683e+00_rb,5.0339e-03_rb/)
      kao(:, 4, 7, 8) = (/ &
     &2.1213e+01_rb,1.8564e+01_rb,1.5913e+01_rb,1.3262e+01_rb,1.0612e+01_rb,7.9619e+00_rb, &
     &5.3122e+00_rb,2.6672e+00_rb,6.7768e-03_rb/)
      kao(:, 5, 7, 8) = (/ &
     &2.1089e+01_rb,1.8455e+01_rb,1.5820e+01_rb,1.3186e+01_rb,1.0552e+01_rb,7.9178e+00_rb, &
     &5.2873e+00_rb,2.6593e+00_rb,8.8413e-03_rb/)
      kao(:, 1, 8, 8) = (/ &
     &4.5180e+01_rb,3.9535e+01_rb,3.3887e+01_rb,2.8239e+01_rb,2.2591e+01_rb,1.6944e+01_rb, &
     &1.1298e+01_rb,5.6517e+00_rb,1.3715e-03_rb/)
      kao(:, 2, 8, 8) = (/ &
     &4.5290e+01_rb,3.9627e+01_rb,3.3969e+01_rb,2.8308e+01_rb,2.2647e+01_rb,1.6987e+01_rb, &
     &1.1327e+01_rb,5.6660e+00_rb,2.0845e-03_rb/)
      kao(:, 3, 8, 8) = (/ &
     &4.5261e+01_rb,3.9605e+01_rb,3.3948e+01_rb,2.8290e+01_rb,2.2633e+01_rb,1.6977e+01_rb, &
     &1.1320e+01_rb,5.6637e+00_rb,3.2057e-03_rb/)
      kao(:, 4, 8, 8) = (/ &
     &4.5126e+01_rb,3.9486e+01_rb,3.3848e+01_rb,2.8205e+01_rb,2.2567e+01_rb,1.6928e+01_rb, &
     &1.1288e+01_rb,5.6493e+00_rb,4.5853e-03_rb/)
      kao(:, 5, 8, 8) = (/ &
     &4.4865e+01_rb,3.9259e+01_rb,3.3651e+01_rb,2.8046e+01_rb,2.2438e+01_rb,1.6832e+01_rb, &
     &1.1226e+01_rb,5.6233e+00_rb,6.2243e-03_rb/)
      kao(:, 1, 9, 8) = (/ &
     &1.7982e+02_rb,1.5734e+02_rb,1.3487e+02_rb,1.1239e+02_rb,8.9912e+01_rb,6.7434e+01_rb, &
     &4.4957e+01_rb,2.2481e+01_rb,1.3106e-03_rb/)
      kao(:, 2, 9, 8) = (/ &
     &1.7996e+02_rb,1.5746e+02_rb,1.3497e+02_rb,1.1247e+02_rb,8.9976e+01_rb,6.7488e+01_rb, &
     &4.4992e+01_rb,2.2498e+01_rb,1.6404e-03_rb/)
      kao(:, 3, 9, 8) = (/ &
     &1.7965e+02_rb,1.5719e+02_rb,1.3473e+02_rb,1.1228e+02_rb,8.9821e+01_rb,6.7370e+01_rb, &
     &4.4914e+01_rb,2.2461e+01_rb,2.0268e-03_rb/)
      kao(:, 4, 9, 8) = (/ &
     &1.7888e+02_rb,1.5653e+02_rb,1.3416e+02_rb,1.1181e+02_rb,8.9443e+01_rb,6.7088e+01_rb, &
     &4.4727e+01_rb,2.2367e+01_rb,2.6824e-03_rb/)
      kao(:, 5, 9, 8) = (/ &
     &1.7801e+02_rb,1.5576e+02_rb,1.3351e+02_rb,1.1126e+02_rb,8.9014e+01_rb,6.6762e+01_rb, &
     &4.4512e+01_rb,2.2261e+01_rb,3.2961e-03_rb/)
      kao(:, 1,10, 8) = (/ &
     &7.9186e+02_rb,6.9285e+02_rb,5.9387e+02_rb,4.9485e+02_rb,3.9587e+02_rb,2.9692e+02_rb, &
     &1.9795e+02_rb,9.8980e+01_rb,2.6434e-03_rb/)
      kao(:, 2,10, 8) = (/ &
     &7.9156e+02_rb,6.9260e+02_rb,5.9370e+02_rb,4.9476e+02_rb,3.9580e+02_rb,2.9685e+02_rb, &
     &1.9789e+02_rb,9.8956e+01_rb,3.6689e-03_rb/)
      kao(:, 3,10, 8) = (/ &
     &7.8897e+02_rb,6.9037e+02_rb,5.9173e+02_rb,4.9309e+02_rb,3.9450e+02_rb,2.9589e+02_rb, &
     &1.9725e+02_rb,9.8635e+01_rb,4.1214e-03_rb/)
      kao(:, 4,10, 8) = (/ &
     &7.8518e+02_rb,6.8703e+02_rb,5.8889e+02_rb,4.9073e+02_rb,3.9258e+02_rb,2.9444e+02_rb, &
     &1.9631e+02_rb,9.8149e+01_rb,4.6533e-03_rb/)
      kao(:, 5,10, 8) = (/ &
     &7.8271e+02_rb,6.8490e+02_rb,5.8707e+02_rb,4.8922e+02_rb,3.9137e+02_rb,2.9354e+02_rb, &
     &1.9568e+02_rb,9.7851e+01_rb,5.3114e-03_rb/)
      kao(:, 1,11, 8) = (/ &
     &1.2023e+03_rb,1.0521e+03_rb,9.0173e+02_rb,7.5144e+02_rb,6.0116e+02_rb,4.5091e+02_rb, &
     &3.0058e+02_rb,1.5029e+02_rb,2.8585e-03_rb/)
      kao(:, 2,11, 8) = (/ &
     &1.1987e+03_rb,1.0489e+03_rb,8.9906e+02_rb,7.4919e+02_rb,5.9938e+02_rb,4.4951e+02_rb, &
     &2.9969e+02_rb,1.4984e+02_rb,4.2381e-03_rb/)
      kao(:, 3,11, 8) = (/ &
     &1.1938e+03_rb,1.0446e+03_rb,8.9539e+02_rb,7.4611e+02_rb,5.9694e+02_rb,4.4768e+02_rb, &
     &2.9847e+02_rb,1.4923e+02_rb,6.1776e-03_rb/)
      kao(:, 4,11, 8) = (/ &
     &1.1919e+03_rb,1.0429e+03_rb,8.9396e+02_rb,7.4498e+02_rb,5.9597e+02_rb,4.4698e+02_rb, &
     &2.9797e+02_rb,1.4900e+02_rb,6.8124e-03_rb/)
      kao(:, 5,11, 8) = (/ &
     &1.1943e+03_rb,1.0450e+03_rb,8.9567e+02_rb,7.4646e+02_rb,5.9716e+02_rb,4.4786e+02_rb, &
     &2.9857e+02_rb,1.4929e+02_rb,7.2330e-03_rb/)
      kao(:, 1,12, 8) = (/ &
     &1.3468e+03_rb,1.1784e+03_rb,1.0101e+03_rb,8.4170e+02_rb,6.7334e+02_rb,5.0503e+02_rb, &
     &3.3668e+02_rb,1.6834e+02_rb,3.2180e-03_rb/)
      kao(:, 2,12, 8) = (/ &
     &1.3417e+03_rb,1.1740e+03_rb,1.0062e+03_rb,8.3855e+02_rb,6.7082e+02_rb,5.0314e+02_rb, &
     &3.3541e+02_rb,1.6772e+02_rb,4.6846e-03_rb/)
      kao(:, 3,12, 8) = (/ &
     &1.3409e+03_rb,1.1734e+03_rb,1.0058e+03_rb,8.3814e+02_rb,6.7048e+02_rb,5.0287e+02_rb, &
     &3.3524e+02_rb,1.6763e+02_rb,6.5606e-03_rb/)
      kao(:, 4,12, 8) = (/ &
     &1.3471e+03_rb,1.1787e+03_rb,1.0103e+03_rb,8.4192e+02_rb,6.7354e+02_rb,5.0515e+02_rb, &
     &3.3677e+02_rb,1.6839e+02_rb,8.6936e-03_rb/)
      kao(:, 5,12, 8) = (/ &
     &1.3615e+03_rb,1.1913e+03_rb,1.0211e+03_rb,8.5093e+02_rb,6.8072e+02_rb,5.1056e+02_rb, &
     &3.4035e+02_rb,1.7018e+02_rb,9.2918e-03_rb/)
      kao(:, 1,13, 8) = (/ &
     &1.2096e+03_rb,1.0584e+03_rb,9.0717e+02_rb,7.5599e+02_rb,6.0478e+02_rb,4.5359e+02_rb, &
     &3.0240e+02_rb,1.5119e+02_rb,3.1447e-03_rb/)
      kao(:, 2,13, 8) = (/ &
     &1.2095e+03_rb,1.0583e+03_rb,9.0711e+02_rb,7.5597e+02_rb,6.0474e+02_rb,4.5355e+02_rb, &
     &3.0238e+02_rb,1.5119e+02_rb,4.9163e-03_rb/)
      kao(:, 3,13, 8) = (/ &
     &1.2161e+03_rb,1.0641e+03_rb,9.1208e+02_rb,7.6006e+02_rb,6.0804e+02_rb,4.5603e+02_rb, &
     &3.0401e+02_rb,1.5201e+02_rb,6.8865e-03_rb/)
      kao(:, 4,13, 8) = (/ &
     &1.2328e+03_rb,1.0787e+03_rb,9.2458e+02_rb,7.7050e+02_rb,6.1637e+02_rb,4.6230e+02_rb, &
     &3.0819e+02_rb,1.5410e+02_rb,9.4613e-03_rb/)
      kao(:, 5,13, 8) = (/ &
     &1.2601e+03_rb,1.1026e+03_rb,9.4505e+02_rb,7.8755e+02_rb,6.3004e+02_rb,4.7254e+02_rb, &
     &3.1501e+02_rb,1.5751e+02_rb,1.0283e-02_rb/)
      kao(:, 1, 1, 9) = (/ &
     &1.0370e+00_rb,9.0857e-01_rb,7.8023e-01_rb,6.5192e-01_rb,5.2457e-01_rb,4.0025e-01_rb, &
     &2.7828e-01_rb,1.5804e-01_rb,5.8897e-02_rb/)
      kao(:, 2, 1, 9) = (/ &
     &1.0492e+00_rb,9.1962e-01_rb,7.9014e-01_rb,6.6101e-01_rb,5.3535e-01_rb,4.1219e-01_rb, &
     &2.9064e-01_rb,1.7381e-01_rb,7.6414e-02_rb/)
      kao(:, 3, 1, 9) = (/ &
     &1.0617e+00_rb,9.3102e-01_rb,8.0032e-01_rb,6.7200e-01_rb,5.4761e-01_rb,4.2620e-01_rb, &
     &3.0674e-01_rb,1.9538e-01_rb,9.6701e-02_rb/)
      kao(:, 4, 1, 9) = (/ &
     &1.0715e+00_rb,9.3980e-01_rb,8.0850e-01_rb,6.8264e-01_rb,5.6043e-01_rb,4.4205e-01_rb, &
     &3.2592e-01_rb,2.2229e-01_rb,1.1943e-01_rb/)
      kao(:, 5, 1, 9) = (/ &
     &1.0801e+00_rb,9.4751e-01_rb,8.1733e-01_rb,6.9368e-01_rb,5.7613e-01_rb,4.6079e-01_rb, &
     &3.5014e-01_rb,2.5319e-01_rb,1.4487e-01_rb/)
      kao(:, 1, 2, 9) = (/ &
     &1.7442e+00_rb,1.5272e+00_rb,1.3102e+00_rb,1.0933e+00_rb,8.7649e-01_rb,6.6091e-01_rb, &
     &4.4902e-01_rb,2.3891e-01_rb,5.2362e-02_rb/)
      kao(:, 2, 2, 9) = (/ &
     &1.7617e+00_rb,1.5427e+00_rb,1.3238e+00_rb,1.1049e+00_rb,8.8687e-01_rb,6.7276e-01_rb, &
     &4.6180e-01_rb,2.5188e-01_rb,6.8691e-02_rb/)
      kao(:, 3, 2, 9) = (/ &
     &1.7768e+00_rb,1.5563e+00_rb,1.3359e+00_rb,1.1155e+00_rb,8.9933e-01_rb,6.8666e-01_rb, &
     &4.7568e-01_rb,2.6925e-01_rb,8.7599e-02_rb/)
      kao(:, 4, 2, 9) = (/ &
     &1.7902e+00_rb,1.5683e+00_rb,1.3467e+00_rb,1.1272e+00_rb,9.1292e-01_rb,7.0212e-01_rb, &
     &4.9313e-01_rb,2.9218e-01_rb,1.0892e-01_rb/)
      kao(:, 5, 2, 9) = (/ &
     &1.8007e+00_rb,1.5781e+00_rb,1.3558e+00_rb,1.1390e+00_rb,9.2668e-01_rb,7.1895e-01_rb, &
     &5.1345e-01_rb,3.2009e-01_rb,1.3290e-01_rb/)
      kao(:, 1, 3, 9) = (/ &
     &3.9769e+00_rb,3.4805e+00_rb,2.9839e+00_rb,2.4875e+00_rb,1.9912e+00_rb,1.4948e+00_rb, &
     &9.9892e-01_rb,5.0928e-01_rb,4.2777e-02_rb/)
      kao(:, 2, 3, 9) = (/ &
     &4.0084e+00_rb,3.5082e+00_rb,3.0084e+00_rb,2.5082e+00_rb,2.0082e+00_rb,1.5084e+00_rb, &
     &1.0123e+00_rb,5.2189e-01_rb,5.7041e-02_rb/)
      kao(:, 3, 3, 9) = (/ &
     &4.0345e+00_rb,3.5316e+00_rb,3.0286e+00_rb,2.5259e+00_rb,2.0232e+00_rb,1.5223e+00_rb, &
     &1.0275e+00_rb,5.3508e-01_rb,7.3780e-02_rb/)
      kao(:, 4, 3, 9) = (/ &
     &4.0588e+00_rb,3.5535e+00_rb,3.0482e+00_rb,2.5430e+00_rb,2.0380e+00_rb,1.5389e+00_rb, &
     &1.0447e+00_rb,5.5284e-01_rb,9.2939e-02_rb/)
      kao(:, 5, 3, 9) = (/ &
     &4.0766e+00_rb,3.5694e+00_rb,3.0624e+00_rb,2.5553e+00_rb,2.0517e+00_rb,1.5545e+00_rb, &
     &1.0614e+00_rb,5.7409e-01_rb,1.1487e-01_rb/)
      kao(:, 1, 4, 9) = (/ &
     &9.2347e+00_rb,8.0808e+00_rb,6.9267e+00_rb,5.7730e+00_rb,4.6191e+00_rb,3.4653e+00_rb, &
     &2.3115e+00_rb,1.1586e+00_rb,3.3639e-02_rb/)
      kao(:, 2, 4, 9) = (/ &
     &9.3231e+00_rb,8.1587e+00_rb,6.9942e+00_rb,5.8295e+00_rb,4.6650e+00_rb,3.5006e+00_rb, &
     &2.3364e+00_rb,1.1757e+00_rb,4.5584e-02_rb/)
      kao(:, 3, 4, 9) = (/ &
     &9.3883e+00_rb,8.2160e+00_rb,7.0435e+00_rb,5.8714e+00_rb,4.6992e+00_rb,3.5271e+00_rb, &
     &2.3555e+00_rb,1.1930e+00_rb,5.9759e-02_rb/)
      kao(:, 4, 4, 9) = (/ &
     &9.4383e+00_rb,8.2599e+00_rb,7.0821e+00_rb,5.9042e+00_rb,4.7261e+00_rb,3.5482e+00_rb, &
     &2.3742e+00_rb,1.2110e+00_rb,7.6294e-02_rb/)
      kao(:, 5, 4, 9) = (/ &
     &9.4750e+00_rb,8.2926e+00_rb,7.1106e+00_rb,5.9283e+00_rb,4.7463e+00_rb,3.5648e+00_rb, &
     &2.3926e+00_rb,1.2283e+00_rb,9.5542e-02_rb/)
      kao(:, 1, 5, 9) = (/ &
     &1.9585e+01_rb,1.7137e+01_rb,1.4689e+01_rb,1.2242e+01_rb,9.7941e+00_rb,7.3465e+00_rb, &
     &4.8988e+00_rb,2.4515e+00_rb,2.6102e-02_rb/)
      kao(:, 2, 5, 9) = (/ &
     &1.9780e+01_rb,1.7309e+01_rb,1.4837e+01_rb,1.2365e+01_rb,9.8932e+00_rb,7.4216e+00_rb, &
     &4.9500e+00_rb,2.4789e+00_rb,3.5859e-02_rb/)
      kao(:, 3, 5, 9) = (/ &
     &1.9935e+01_rb,1.7444e+01_rb,1.4954e+01_rb,1.2463e+01_rb,9.9721e+00_rb,7.4814e+00_rb, &
     &4.9911e+00_rb,2.5017e+00_rb,4.7575e-02_rb/)
      kao(:, 4, 5, 9) = (/ &
     &2.0051e+01_rb,1.7546e+01_rb,1.5041e+01_rb,1.2537e+01_rb,1.0032e+01_rb,7.5273e+00_rb, &
     &5.0227e+00_rb,2.5239e+00_rb,6.1484e-02_rb/)
      kao(:, 5, 5, 9) = (/ &
     &2.0135e+01_rb,1.7620e+01_rb,1.5106e+01_rb,1.2591e+01_rb,1.0076e+01_rb,7.5609e+00_rb, &
     &5.0463e+00_rb,2.5450e+00_rb,7.7952e-02_rb/)
      kao(:, 1, 6, 9) = (/ &
     &3.8631e+01_rb,3.3801e+01_rb,2.8974e+01_rb,2.4144e+01_rb,1.9316e+01_rb,1.4488e+01_rb, &
     &9.6597e+00_rb,4.8315e+00_rb,1.9604e-02_rb/)
      kao(:, 2, 6, 9) = (/ &
     &3.9022e+01_rb,3.4145e+01_rb,2.9268e+01_rb,2.4390e+01_rb,1.9513e+01_rb,1.4636e+01_rb, &
     &9.7595e+00_rb,4.8831e+00_rb,2.7435e-02_rb/)
      kao(:, 3, 6, 9) = (/ &
     &3.9348e+01_rb,3.4431e+01_rb,2.9514e+01_rb,2.4596e+01_rb,1.9679e+01_rb,1.4761e+01_rb, &
     &9.8442e+00_rb,4.9267e+00_rb,3.6949e-02_rb/)
      kao(:, 4, 6, 9) = (/ &
     &3.9594e+01_rb,3.4646e+01_rb,2.9698e+01_rb,2.4751e+01_rb,1.9803e+01_rb,1.4855e+01_rb, &
     &9.9079e+00_rb,4.9603e+00_rb,4.8442e-02_rb/)
      kao(:, 5, 6, 9) = (/ &
     &3.9770e+01_rb,3.4801e+01_rb,2.9832e+01_rb,2.4862e+01_rb,1.9893e+01_rb,1.4924e+01_rb, &
     &9.9544e+00_rb,4.9871e+00_rb,6.2169e-02_rb/)
      kao(:, 1, 7, 9) = (/ &
     &7.9230e+01_rb,6.9326e+01_rb,5.9424e+01_rb,4.9520e+01_rb,3.9617e+01_rb,2.9713e+01_rb, &
     &1.9810e+01_rb,9.9063e+00_rb,1.4333e-02_rb/)
      kao(:, 2, 7, 9) = (/ &
     &8.0053e+01_rb,7.0047e+01_rb,6.0041e+01_rb,5.0034e+01_rb,4.0029e+01_rb,3.0023e+01_rb, &
     &2.0018e+01_rb,1.0011e+01_rb,2.0474e-02_rb/)
      kao(:, 3, 7, 9) = (/ &
     &8.0747e+01_rb,7.0654e+01_rb,6.0562e+01_rb,5.0470e+01_rb,4.0378e+01_rb,3.0286e+01_rb, &
     &2.0193e+01_rb,1.0101e+01_rb,2.8098e-02_rb/)
      kao(:, 4, 7, 9) = (/ &
     &8.1303e+01_rb,7.1142e+01_rb,6.0980e+01_rb,5.0819e+01_rb,4.0658e+01_rb,3.0496e+01_rb, &
     &2.0334e+01_rb,1.0173e+01_rb,3.7439e-02_rb/)
      kao(:, 5, 7, 9) = (/ &
     &8.1680e+01_rb,7.1472e+01_rb,6.1265e+01_rb,5.1057e+01_rb,4.0847e+01_rb,3.0638e+01_rb, &
     &2.0431e+01_rb,1.0223e+01_rb,4.8718e-02_rb/)
      kao(:, 1, 8, 9) = (/ &
     &1.9140e+02_rb,1.6747e+02_rb,1.4355e+02_rb,1.1962e+02_rb,9.5701e+01_rb,7.1775e+01_rb, &
     &4.7850e+01_rb,2.3927e+01_rb,1.0137e-02_rb/)
      kao(:, 2, 8, 9) = (/ &
     &1.9353e+02_rb,1.6934e+02_rb,1.4515e+02_rb,1.2096e+02_rb,9.6769e+01_rb,7.2577e+01_rb, &
     &4.8385e+01_rb,2.4195e+01_rb,1.4960e-02_rb/)
      kao(:, 3, 8, 9) = (/ &
     &1.9528e+02_rb,1.7086e+02_rb,1.4646e+02_rb,1.2205e+02_rb,9.7642e+01_rb,7.3231e+01_rb, &
     &4.8824e+01_rb,2.4416e+01_rb,2.0883e-02_rb/)
      kao(:, 4, 8, 9) = (/ &
     &1.9672e+02_rb,1.7213e+02_rb,1.4754e+02_rb,1.2295e+02_rb,9.8361e+01_rb,7.3773e+01_rb, &
     &4.9186e+01_rb,2.4598e+01_rb,2.8267e-02_rb/)
      kao(:, 5, 8, 9) = (/ &
     &1.9763e+02_rb,1.7293e+02_rb,1.4824e+02_rb,1.2352e+02_rb,9.8820e+01_rb,7.4120e+01_rb, &
     &4.9417e+01_rb,2.4716e+01_rb,3.7324e-02_rb/)
      kao(:, 1, 9, 9) = (/ &
     &8.5696e+02_rb,7.4986e+02_rb,6.4272e+02_rb,5.3561e+02_rb,4.2849e+02_rb,3.2137e+02_rb, &
     &2.1425e+02_rb,1.0712e+02_rb,4.9710e-03_rb/)
      kao(:, 2, 9, 9) = (/ &
     &8.6661e+02_rb,7.5827e+02_rb,6.4996e+02_rb,5.4164e+02_rb,4.3331e+02_rb,3.2498e+02_rb, &
     &2.1666e+02_rb,1.0833e+02_rb,7.7324e-03_rb/)
      kao(:, 3, 9, 9) = (/ &
     &8.7472e+02_rb,7.6537e+02_rb,6.5602e+02_rb,5.4669e+02_rb,4.3735e+02_rb,3.2801e+02_rb, &
     &2.1868e+02_rb,1.0935e+02_rb,1.2302e-02_rb/)
      kao(:, 4, 9, 9) = (/ &
     &8.8128e+02_rb,7.7113e+02_rb,6.6095e+02_rb,5.5081e+02_rb,4.4064e+02_rb,3.3048e+02_rb, &
     &2.2033e+02_rb,1.1017e+02_rb,1.8205e-02_rb/)
      kao(:, 5, 9, 9) = (/ &
     &8.8594e+02_rb,7.7518e+02_rb,6.6448e+02_rb,5.5372e+02_rb,4.4298e+02_rb,3.3223e+02_rb, &
     &2.2150e+02_rb,1.1076e+02_rb,2.5853e-02_rb/)
      kao(:, 1,10, 9) = (/ &
     &4.2110e+03_rb,3.6847e+03_rb,3.1582e+03_rb,2.6319e+03_rb,2.1055e+03_rb,1.5792e+03_rb, &
     &1.0527e+03_rb,5.2637e+02_rb,5.0235e-03_rb/)
      kao(:, 2,10, 9) = (/ &
     &4.2578e+03_rb,3.7256e+03_rb,3.1934e+03_rb,2.6613e+03_rb,2.1289e+03_rb,1.5967e+03_rb, &
     &1.0645e+03_rb,5.3224e+02_rb,7.3952e-03_rb/)
      kao(:, 3,10, 9) = (/ &
     &4.2976e+03_rb,3.7604e+03_rb,3.2232e+03_rb,2.6860e+03_rb,2.1488e+03_rb,1.6116e+03_rb, &
     &1.0744e+03_rb,5.3720e+02_rb,1.0255e-02_rb/)
      kao(:, 4,10, 9) = (/ &
     &4.3279e+03_rb,3.7869e+03_rb,3.2460e+03_rb,2.7049e+03_rb,2.1640e+03_rb,1.6230e+03_rb, &
     &1.0820e+03_rb,5.4099e+02_rb,1.2154e-02_rb/)
      kao(:, 5,10, 9) = (/ &
     &4.3496e+03_rb,3.8060e+03_rb,3.2622e+03_rb,2.7186e+03_rb,2.1748e+03_rb,1.6311e+03_rb, &
     &1.0874e+03_rb,5.4372e+02_rb,1.4719e-02_rb/)
      kao(:, 1,11, 9) = (/ &
     &7.1173e+03_rb,6.2277e+03_rb,5.3380e+03_rb,4.4482e+03_rb,3.5586e+03_rb,2.6690e+03_rb, &
     &1.7793e+03_rb,8.8967e+02_rb,4.7950e-03_rb/)
      kao(:, 2,11, 9) = (/ &
     &7.1866e+03_rb,6.2881e+03_rb,5.3900e+03_rb,4.4917e+03_rb,3.5933e+03_rb,2.6950e+03_rb, &
     &1.7967e+03_rb,8.9836e+02_rb,1.0381e-02_rb/)
      kao(:, 3,11, 9) = (/ &
     &7.2420e+03_rb,6.3363e+03_rb,5.4313e+03_rb,4.5262e+03_rb,3.6209e+03_rb,2.7156e+03_rb, &
     &1.8104e+03_rb,9.0520e+02_rb,1.1192e-02_rb/)
      kao(:, 4,11, 9) = (/ &
     &7.2818e+03_rb,6.3715e+03_rb,5.4613e+03_rb,4.5511e+03_rb,3.6409e+03_rb,2.7307e+03_rb, &
     &1.8205e+03_rb,9.1025e+02_rb,1.5145e-02_rb/)
      kao(:, 5,11, 9) = (/ &
     &7.3078e+03_rb,6.3944e+03_rb,5.4808e+03_rb,4.5675e+03_rb,3.6540e+03_rb,2.7404e+03_rb, &
     &1.8270e+03_rb,9.1351e+02_rb,1.9391e-02_rb/)
      kao(:, 1,12, 9) = (/ &
     &8.7768e+03_rb,7.6797e+03_rb,6.5826e+03_rb,5.4853e+03_rb,4.3884e+03_rb,3.2912e+03_rb, &
     &2.1942e+03_rb,1.0970e+03_rb,4.5625e-03_rb/)
      kao(:, 2,12, 9) = (/ &
     &8.8507e+03_rb,7.7442e+03_rb,6.6378e+03_rb,5.5316e+03_rb,4.4251e+03_rb,3.3189e+03_rb, &
     &2.2126e+03_rb,1.1063e+03_rb,8.7877e-03_rb/)
      kao(:, 3,12, 9) = (/ &
     &8.9020e+03_rb,7.7891e+03_rb,6.6765e+03_rb,5.5638e+03_rb,4.4512e+03_rb,3.3383e+03_rb, &
     &2.2256e+03_rb,1.1128e+03_rb,1.5209e-02_rb/)
      kao(:, 4,12, 9) = (/ &
     &8.9305e+03_rb,7.8144e+03_rb,6.6981e+03_rb,5.5817e+03_rb,4.4653e+03_rb,3.3489e+03_rb, &
     &2.2326e+03_rb,1.1163e+03_rb,1.7795e-02_rb/)
      kao(:, 5,12, 9) = (/ &
     &8.9442e+03_rb,7.8262e+03_rb,6.7079e+03_rb,5.5901e+03_rb,4.4720e+03_rb,3.3539e+03_rb, &
     &2.2360e+03_rb,1.1180e+03_rb,2.1474e-02_rb/)
      kao(:, 1,13, 9) = (/ &
     &8.5284e+03_rb,7.4622e+03_rb,6.3962e+03_rb,5.3302e+03_rb,4.2643e+03_rb,3.1981e+03_rb, &
     &2.1321e+03_rb,1.0660e+03_rb,6.4304e-03_rb/)
      kao(:, 2,13, 9) = (/ &
     &8.5795e+03_rb,7.5072e+03_rb,6.4348e+03_rb,5.3623e+03_rb,4.2900e+03_rb,3.2173e+03_rb, &
     &2.1449e+03_rb,1.0725e+03_rb,8.8022e-03_rb/)
      kao(:, 3,13, 9) = (/ &
     &8.6101e+03_rb,7.5338e+03_rb,6.4575e+03_rb,5.3813e+03_rb,4.3050e+03_rb,3.2288e+03_rb, &
     &2.1525e+03_rb,1.0763e+03_rb,1.7025e-02_rb/)
      kao(:, 4,13, 9) = (/ &
     &8.6267e+03_rb,7.5481e+03_rb,6.4701e+03_rb,5.3916e+03_rb,4.3132e+03_rb,3.2351e+03_rb, &
     &2.1567e+03_rb,1.0783e+03_rb,2.0223e-02_rb/)
      kao(:, 5,13, 9) = (/ &
     &8.6229e+03_rb,7.5451e+03_rb,6.4669e+03_rb,5.3892e+03_rb,4.3114e+03_rb,3.2335e+03_rb, &
     &2.1558e+03_rb,1.0779e+03_rb,2.3841e-02_rb/)
      kao(:, 1, 1,10) = (/ &
     &3.0277e+00_rb,2.6495e+00_rb,2.2716e+00_rb,1.8933e+00_rb,1.5153e+00_rb,1.1372e+00_rb, &
     &7.5951e-01_rb,4.2431e-01_rb,1.1653e-01_rb/)
      kao(:, 2, 1,10) = (/ &
     &3.0877e+00_rb,2.7021e+00_rb,2.3167e+00_rb,1.9311e+00_rb,1.5454e+00_rb,1.1599e+00_rb, &
     &7.9480e-01_rb,4.6158e-01_rb,1.5133e-01_rb/)
      kao(:, 3, 1,10) = (/ &
     &3.1443e+00_rb,2.7514e+00_rb,2.3589e+00_rb,1.9665e+00_rb,1.5738e+00_rb,1.1839e+00_rb, &
     &8.4936e-01_rb,4.8883e-01_rb,1.8963e-01_rb/)
      kao(:, 4, 1,10) = (/ &
     &3.1902e+00_rb,2.7921e+00_rb,2.3935e+00_rb,1.9952e+00_rb,1.5971e+00_rb,1.2268e+00_rb, &
     &8.9927e-01_rb,5.1426e-01_rb,2.3885e-01_rb/)
      kao(:, 5, 1,10) = (/ &
     &3.2311e+00_rb,2.8275e+00_rb,2.4242e+00_rb,2.0206e+00_rb,1.6182e+00_rb,1.2846e+00_rb, &
     &9.3468e-01_rb,5.5470e-01_rb,2.9337e-01_rb/)
      kao(:, 1, 2,10) = (/ &
     &4.5362e+00_rb,3.9696e+00_rb,3.4027e+00_rb,2.8358e+00_rb,2.2692e+00_rb,1.7025e+00_rb, &
     &1.1358e+00_rb,5.9446e-01_rb,1.1450e-01_rb/)
      kao(:, 2, 2,10) = (/ &
     &4.6490e+00_rb,4.0681e+00_rb,3.4874e+00_rb,2.9068e+00_rb,2.3258e+00_rb,1.7450e+00_rb, &
     &1.1656e+00_rb,6.4847e-01_rb,1.4911e-01_rb/)
      kao(:, 3, 2,10) = (/ &
     &4.7559e+00_rb,4.1617e+00_rb,3.5676e+00_rb,2.9735e+00_rb,2.3793e+00_rb,1.7851e+00_rb, &
     &1.2174e+00_rb,6.9018e-01_rb,1.9066e-01_rb/)
      kao(:, 4, 2,10) = (/ &
     &4.8499e+00_rb,4.2438e+00_rb,3.6378e+00_rb,3.0322e+00_rb,2.4263e+00_rb,1.8213e+00_rb, &
     &1.2864e+00_rb,7.2155e-01_rb,2.3838e-01_rb/)
      kao(:, 5, 2,10) = (/ &
     &4.9283e+00_rb,4.3130e+00_rb,3.6972e+00_rb,3.0815e+00_rb,2.4657e+00_rb,1.8737e+00_rb, &
     &1.3480e+00_rb,7.6180e-01_rb,2.9224e-01_rb/)
      kao(:, 1, 3,10) = (/ &
     &9.1512e+00_rb,8.0077e+00_rb,6.8647e+00_rb,5.7202e+00_rb,4.5764e+00_rb,3.4330e+00_rb, &
     &2.2893e+00_rb,1.1456e+00_rb,1.0238e-01_rb/)
      kao(:, 2, 3,10) = (/ &
     &9.4295e+00_rb,8.2512e+00_rb,7.0726e+00_rb,5.8941e+00_rb,4.7160e+00_rb,3.5376e+00_rb, &
     &2.3591e+00_rb,1.1894e+00_rb,1.3683e-01_rb/)
      kao(:, 3, 3,10) = (/ &
     &9.6929e+00_rb,8.4812e+00_rb,7.2704e+00_rb,6.0594e+00_rb,4.8479e+00_rb,3.6364e+00_rb, &
     &2.4250e+00_rb,1.2581e+00_rb,1.7750e-01_rb/)
      kao(:, 4, 3,10) = (/ &
     &9.9289e+00_rb,8.6885e+00_rb,7.4477e+00_rb,6.2066e+00_rb,4.9652e+00_rb,3.7251e+00_rb, &
     &2.4842e+00_rb,1.3376e+00_rb,2.2412e-01_rb/)
      kao(:, 5, 3,10) = (/ &
     &1.0116e+01_rb,8.8511e+00_rb,7.5880e+00_rb,6.3233e+00_rb,5.0592e+00_rb,3.7951e+00_rb, &
     &2.5511e+00_rb,1.3941e+00_rb,2.7545e-01_rb/)
      kao(:, 1, 4,10) = (/ &
     &1.9425e+01_rb,1.6997e+01_rb,1.4568e+01_rb,1.2141e+01_rb,9.7127e+00_rb,7.2856e+00_rb, &
     &4.8569e+00_rb,2.4292e+00_rb,8.5208e-02_rb/)
      kao(:, 2, 4,10) = (/ &
     &1.9954e+01_rb,1.7460e+01_rb,1.4965e+01_rb,1.2471e+01_rb,9.9767e+00_rb,7.4831e+00_rb, &
     &4.9893e+00_rb,2.4954e+00_rb,1.1708e-01_rb/)
      kao(:, 3, 4,10) = (/ &
     &2.0423e+01_rb,1.7870e+01_rb,1.5318e+01_rb,1.2764e+01_rb,1.0212e+01_rb,7.6596e+00_rb, &
     &5.1068e+00_rb,2.5544e+00_rb,1.5673e-01_rb/)
      kao(:, 4, 4,10) = (/ &
     &2.0864e+01_rb,1.8257e+01_rb,1.5649e+01_rb,1.3042e+01_rb,1.0433e+01_rb,7.8250e+00_rb, &
     &5.2173e+00_rb,2.6098e+00_rb,2.0293e-01_rb/)
      kao(:, 5, 4,10) = (/ &
     &2.1248e+01_rb,1.8592e+01_rb,1.5938e+01_rb,1.3281e+01_rb,1.0624e+01_rb,7.9692e+00_rb, &
     &5.3131e+00_rb,2.6931e+00_rb,2.5421e-01_rb/)
      kao(:, 1, 5,10) = (/ &
     &4.0147e+01_rb,3.5130e+01_rb,3.0111e+01_rb,2.5093e+01_rb,2.0073e+01_rb,1.5056e+01_rb, &
     &1.0038e+01_rb,5.0189e+00_rb,6.8359e-02_rb/)
      kao(:, 2, 5,10) = (/ &
     &4.0924e+01_rb,3.5810e+01_rb,3.0692e+01_rb,2.5576e+01_rb,2.0461e+01_rb,1.5346e+01_rb, &
     &1.0232e+01_rb,5.1160e+00_rb,9.6926e-02_rb/)
      kao(:, 3, 5,10) = (/ &
     &4.1536e+01_rb,3.6343e+01_rb,3.1153e+01_rb,2.5962e+01_rb,2.0769e+01_rb,1.5577e+01_rb, &
     &1.0385e+01_rb,5.1928e+00_rb,1.3304e-01_rb/)
      kao(:, 4, 5,10) = (/ &
     &4.2166e+01_rb,3.6896e+01_rb,3.1624e+01_rb,2.6353e+01_rb,2.1083e+01_rb,1.5813e+01_rb, &
     &1.0542e+01_rb,5.2715e+00_rb,1.7579e-01_rb/)
      kao(:, 5, 5,10) = (/ &
     &4.2771e+01_rb,3.7425e+01_rb,3.2079e+01_rb,2.6732e+01_rb,2.1385e+01_rb,1.6040e+01_rb, &
     &1.0693e+01_rb,5.3473e+00_rb,2.2393e-01_rb/)
      kao(:, 1, 6,10) = (/ &
     &8.0027e+01_rb,7.0022e+01_rb,6.0023e+01_rb,5.0019e+01_rb,4.0013e+01_rb,3.0011e+01_rb, &
     &2.0007e+01_rb,1.0004e+01_rb,5.3924e-02_rb/)
      kao(:, 2, 6,10) = (/ &
     &8.1173e+01_rb,7.1024e+01_rb,6.0878e+01_rb,5.0731e+01_rb,4.0584e+01_rb,3.0439e+01_rb, &
     &2.0293e+01_rb,1.0147e+01_rb,7.8165e-02_rb/)
      kao(:, 3, 6,10) = (/ &
     &8.2280e+01_rb,7.1996e+01_rb,6.1710e+01_rb,5.1425e+01_rb,4.1141e+01_rb,3.0855e+01_rb, &
     &2.0570e+01_rb,1.0285e+01_rb,1.0934e-01_rb/)
      kao(:, 4, 6,10) = (/ &
     &8.3115e+01_rb,7.2722e+01_rb,6.2336e+01_rb,5.1947e+01_rb,4.1558e+01_rb,3.1167e+01_rb, &
     &2.0778e+01_rb,1.0390e+01_rb,1.4595e-01_rb/)
      kao(:, 5, 6,10) = (/ &
     &8.3819e+01_rb,7.3349e+01_rb,6.2867e+01_rb,5.2391e+01_rb,4.1913e+01_rb,3.1434e+01_rb, &
     &2.0957e+01_rb,1.0479e+01_rb,1.8806e-01_rb/)
      kao(:, 1, 7,10) = (/ &
     &1.6996e+02_rb,1.4871e+02_rb,1.2747e+02_rb,1.0622e+02_rb,8.4977e+01_rb,6.3734e+01_rb, &
     &4.2490e+01_rb,2.1246e+01_rb,4.2588e-02_rb/)
      kao(:, 2, 7,10) = (/ &
     &1.7243e+02_rb,1.5088e+02_rb,1.2932e+02_rb,1.0776e+02_rb,8.6211e+01_rb,6.4660e+01_rb, &
     &4.3106e+01_rb,2.1553e+01_rb,6.2851e-02_rb/)
      kao(:, 3, 7,10) = (/ &
     &1.7457e+02_rb,1.5274e+02_rb,1.3092e+02_rb,1.0910e+02_rb,8.7283e+01_rb,6.5459e+01_rb, &
     &4.3641e+01_rb,2.1821e+01_rb,8.8704e-02_rb/)
      kao(:, 4, 7,10) = (/ &
     &1.7636e+02_rb,1.5432e+02_rb,1.3227e+02_rb,1.1023e+02_rb,8.8183e+01_rb,6.6138e+01_rb, &
     &4.4090e+01_rb,2.2046e+01_rb,1.1966e-01_rb/)
      kao(:, 5, 7,10) = (/ &
     &1.7764e+02_rb,1.5544e+02_rb,1.3323e+02_rb,1.1102e+02_rb,8.8816e+01_rb,6.6613e+01_rb, &
     &4.4410e+01_rb,2.2206e+01_rb,1.5600e-01_rb/)
      kao(:, 1, 8,10) = (/ &
     &4.3531e+02_rb,3.8090e+02_rb,3.2647e+02_rb,2.7207e+02_rb,2.1766e+02_rb,1.6325e+02_rb, &
     &1.0883e+02_rb,5.4412e+01_rb,3.2634e-02_rb/)
      kao(:, 2, 8,10) = (/ &
     &4.4098e+02_rb,3.8585e+02_rb,3.3074e+02_rb,2.7562e+02_rb,2.2050e+02_rb,1.6536e+02_rb, &
     &1.1025e+02_rb,5.5123e+01_rb,4.9788e-02_rb/)
      kao(:, 3, 8,10) = (/ &
     &4.4650e+02_rb,3.9069e+02_rb,3.3490e+02_rb,2.7906e+02_rb,2.2325e+02_rb,1.6744e+02_rb, &
     &1.1162e+02_rb,5.5816e+01_rb,7.2094e-02_rb/)
      kao(:, 4, 8,10) = (/ &
     &4.5108e+02_rb,3.9470e+02_rb,3.3831e+02_rb,2.8194e+02_rb,2.2555e+02_rb,1.6915e+02_rb, &
     &1.1277e+02_rb,5.6387e+01_rb,9.8901e-02_rb/)
      kao(:, 5, 8,10) = (/ &
     &4.5442e+02_rb,3.9761e+02_rb,3.4081e+02_rb,2.8400e+02_rb,2.2721e+02_rb,1.7040e+02_rb, &
     &1.1360e+02_rb,5.6798e+01_rb,1.3038e-01_rb/)
      kao(:, 1, 9,10) = (/ &
     &2.1147e+03_rb,1.8504e+03_rb,1.5860e+03_rb,1.3217e+03_rb,1.0574e+03_rb,7.9304e+02_rb, &
     &5.2867e+02_rb,2.6435e+02_rb,1.9424e-02_rb/)
      kao(:, 2, 9,10) = (/ &
     &2.1442e+03_rb,1.8762e+03_rb,1.6082e+03_rb,1.3401e+03_rb,1.0721e+03_rb,8.0408e+02_rb, &
     &5.3608e+02_rb,2.6803e+02_rb,3.6865e-02_rb/)
      kao(:, 3, 9,10) = (/ &
     &2.1685e+03_rb,1.8976e+03_rb,1.6264e+03_rb,1.3553e+03_rb,1.0842e+03_rb,8.1317e+02_rb, &
     &5.4213e+02_rb,2.7107e+02_rb,5.5684e-02_rb/)
      kao(:, 4, 9,10) = (/ &
     &2.1886e+03_rb,1.9151e+03_rb,1.6415e+03_rb,1.3680e+03_rb,1.0943e+03_rb,8.2077e+02_rb, &
     &5.4720e+02_rb,2.7360e+02_rb,7.8135e-02_rb/)
      kao(:, 5, 9,10) = (/ &
     &2.2035e+03_rb,1.9281e+03_rb,1.6526e+03_rb,1.3772e+03_rb,1.1018e+03_rb,8.2634e+02_rb, &
     &5.5093e+02_rb,2.7546e+02_rb,1.0558e-01_rb/)
      kao(:, 1,10,10) = (/ &
     &1.1340e+04_rb,9.9225e+03_rb,8.5053e+03_rb,7.0882e+03_rb,5.6700e+03_rb,4.2524e+03_rb, &
     &2.8350e+03_rb,1.4175e+03_rb,1.1591e-02_rb/)
      kao(:, 2,10,10) = (/ &
     &1.1483e+04_rb,1.0048e+04_rb,8.6127e+03_rb,7.1771e+03_rb,5.7419e+03_rb,4.3066e+03_rb, &
     &2.8709e+03_rb,1.4354e+03_rb,1.2588e-02_rb/)
      kao(:, 3,10,10) = (/ &
     &1.1610e+04_rb,1.0159e+04_rb,8.7071e+03_rb,7.2559e+03_rb,5.8048e+03_rb,4.3532e+03_rb, &
     &2.9023e+03_rb,1.4511e+03_rb,2.5538e-02_rb/)
      kao(:, 4,10,10) = (/ &
     &1.1702e+04_rb,1.0239e+04_rb,8.7766e+03_rb,7.3135e+03_rb,5.8511e+03_rb,4.3884e+03_rb, &
     &2.9255e+03_rb,1.4628e+03_rb,4.5275e-02_rb/)
      kao(:, 5,10,10) = (/ &
     &1.1737e+04_rb,1.0270e+04_rb,8.8031e+03_rb,7.3358e+03_rb,5.8688e+03_rb,4.4014e+03_rb, &
     &2.9344e+03_rb,1.4670e+03_rb,5.8192e-02_rb/)
      kao(:, 1,11,10) = (/ &
     &2.0903e+04_rb,1.8291e+04_rb,1.5678e+04_rb,1.3065e+04_rb,1.0451e+04_rb,7.8389e+03_rb, &
     &5.2261e+03_rb,2.6129e+03_rb,7.1886e-03_rb/)
      kao(:, 2,11,10) = (/ &
     &2.1095e+04_rb,1.8459e+04_rb,1.5821e+04_rb,1.3184e+04_rb,1.0548e+04_rb,7.9109e+03_rb, &
     &5.2736e+03_rb,2.6368e+03_rb,6.0460e-03_rb/)
      kao(:, 3,11,10) = (/ &
     &2.1210e+04_rb,1.8559e+04_rb,1.5908e+04_rb,1.3256e+04_rb,1.0605e+04_rb,7.9536e+03_rb, &
     &5.3027e+03_rb,2.6511e+03_rb,2.6262e-02_rb/)
      kao(:, 4,11,10) = (/ &
     &2.1260e+04_rb,1.8603e+04_rb,1.5945e+04_rb,1.3287e+04_rb,1.0630e+04_rb,7.9726e+03_rb, &
     &5.3149e+03_rb,2.6575e+03_rb,3.1824e-02_rb/)
      kao(:, 5,11,10) = (/ &
     &2.1221e+04_rb,1.8569e+04_rb,1.5915e+04_rb,1.3263e+04_rb,1.0611e+04_rb,7.9580e+03_rb, &
     &5.3053e+03_rb,2.6525e+03_rb,4.2831e-02_rb/)
      kao(:, 1,12,10) = (/ &
     &2.8005e+04_rb,2.4506e+04_rb,2.1003e+04_rb,1.7504e+04_rb,1.4003e+04_rb,1.0502e+04_rb, &
     &7.0018e+03_rb,3.5007e+03_rb,1.9282e-02_rb/)
      kao(:, 2,12,10) = (/ &
     &2.8178e+04_rb,2.4656e+04_rb,2.1135e+04_rb,1.7611e+04_rb,1.4090e+04_rb,1.0567e+04_rb, &
     &7.0445e+03_rb,3.5221e+03_rb,8.6382e-03_rb/)
      kao(:, 3,12,10) = (/ &
     &2.8205e+04_rb,2.4680e+04_rb,2.1153e+04_rb,1.7628e+04_rb,1.4103e+04_rb,1.0577e+04_rb, &
     &7.0512e+03_rb,3.5256e+03_rb,2.9773e-02_rb/)
      kao(:, 4,12,10) = (/ &
     &2.8170e+04_rb,2.4648e+04_rb,2.1126e+04_rb,1.7606e+04_rb,1.4084e+04_rb,1.0563e+04_rb, &
     &7.0421e+03_rb,3.5211e+03_rb,3.9720e-02_rb/)
      kao(:, 5,12,10) = (/ &
     &2.8047e+04_rb,2.4540e+04_rb,2.1034e+04_rb,1.7529e+04_rb,1.4023e+04_rb,1.0517e+04_rb, &
     &7.0115e+03_rb,3.5058e+03_rb,4.1794e-02_rb/)
      kao(:, 1,13,10) = (/ &
     &2.9533e+04_rb,2.5841e+04_rb,2.2150e+04_rb,1.8457e+04_rb,1.4766e+04_rb,1.1075e+04_rb, &
     &7.3833e+03_rb,3.6917e+03_rb,1.5098e-03_rb/)
      kao(:, 2,13,10) = (/ &
     &2.9558e+04_rb,2.5865e+04_rb,2.2169e+04_rb,1.8475e+04_rb,1.4779e+04_rb,1.1085e+04_rb, &
     &7.3900e+03_rb,3.6949e+03_rb,1.2533e-02_rb/)
      kao(:, 3,13,10) = (/ &
     &2.9540e+04_rb,2.5844e+04_rb,2.2155e+04_rb,1.8463e+04_rb,1.4769e+04_rb,1.1077e+04_rb, &
     &7.3851e+03_rb,3.6924e+03_rb,1.7410e-02_rb/)
      kao(:, 4,13,10) = (/ &
     &2.9338e+04_rb,2.5670e+04_rb,2.2004e+04_rb,1.8336e+04_rb,1.4669e+04_rb,1.1002e+04_rb, &
     &7.3346e+03_rb,3.6671e+03_rb,4.5009e-02_rb/)
      kao(:, 5,13,10) = (/ &
     &2.9073e+04_rb,2.5440e+04_rb,2.1806e+04_rb,1.8171e+04_rb,1.4537e+04_rb,1.0903e+04_rb, &
     &7.2684e+03_rb,3.6344e+03_rb,5.0725e-02_rb/)
      kao(:, 1, 1,11) = (/ &
     &5.0284e+00_rb,4.3997e+00_rb,3.7713e+00_rb,3.1433e+00_rb,2.5148e+00_rb,1.8867e+00_rb, &
     &1.2585e+00_rb,6.3034e-01_rb,1.3324e-01_rb/)
      kao(:, 2, 1,11) = (/ &
     &5.0428e+00_rb,4.4122e+00_rb,3.7822e+00_rb,3.1521e+00_rb,2.5223e+00_rb,1.8921e+00_rb, &
     &1.2620e+00_rb,6.6040e-01_rb,1.7160e-01_rb/)
      kao(:, 3, 1,11) = (/ &
     &5.0450e+00_rb,4.4148e+00_rb,3.7841e+00_rb,3.1540e+00_rb,2.5234e+00_rb,1.8930e+00_rb, &
     &1.2629e+00_rb,7.0553e-01_rb,2.2033e-01_rb/)
      kao(:, 4, 1,11) = (/ &
     &5.0314e+00_rb,4.4027e+00_rb,3.7741e+00_rb,3.1454e+00_rb,2.5166e+00_rb,1.8879e+00_rb, &
     &1.2828e+00_rb,7.2981e-01_rb,2.7819e-01_rb/)
      kao(:, 5, 1,11) = (/ &
     &5.0061e+00_rb,4.3808e+00_rb,3.7550e+00_rb,3.1298e+00_rb,2.5041e+00_rb,1.8785e+00_rb, &
     &1.3311e+00_rb,7.4514e-01_rb,3.4945e-01_rb/)
      kao(:, 1, 2,11) = (/ &
     &7.7109e+00_rb,6.7474e+00_rb,5.7839e+00_rb,4.8201e+00_rb,3.8564e+00_rb,2.8926e+00_rb, &
     &1.9289e+00_rb,9.6516e-01_rb,1.3538e-01_rb/)
      kao(:, 2, 2,11) = (/ &
     &7.7553e+00_rb,6.7857e+00_rb,5.8167e+00_rb,4.8473e+00_rb,3.8784e+00_rb,2.9091e+00_rb, &
     &1.9400e+00_rb,9.7434e-01_rb,1.7899e-01_rb/)
      kao(:, 3, 2,11) = (/ &
     &7.7777e+00_rb,6.8060e+00_rb,5.8332e+00_rb,4.8617e+00_rb,3.8897e+00_rb,2.9178e+00_rb, &
     &1.9457e+00_rb,1.0175e+00_rb,2.2662e-01_rb/)
      kao(:, 4, 2,11) = (/ &
     &7.7756e+00_rb,6.8039e+00_rb,5.8323e+00_rb,4.8602e+00_rb,3.8885e+00_rb,2.9169e+00_rb, &
     &1.9452e+00_rb,1.0634e+00_rb,2.8531e-01_rb/)
      kao(:, 5, 2,11) = (/ &
     &7.7495e+00_rb,6.7817e+00_rb,5.8131e+00_rb,4.8446e+00_rb,3.8758e+00_rb,2.9075e+00_rb, &
     &1.9643e+00_rb,1.0863e+00_rb,3.5634e-01_rb/)
      kao(:, 1, 3,11) = (/ &
     &1.5932e+01_rb,1.3941e+01_rb,1.1949e+01_rb,9.9578e+00_rb,7.9671e+00_rb,5.9754e+00_rb, &
     &3.9840e+00_rb,1.9926e+00_rb,1.2714e-01_rb/)
      kao(:, 2, 3,11) = (/ &
     &1.6076e+01_rb,1.4067e+01_rb,1.2058e+01_rb,1.0048e+01_rb,8.0387e+00_rb,6.0296e+00_rb, &
     &4.0199e+00_rb,2.0105e+00_rb,1.7056e-01_rb/)
      kao(:, 3, 3,11) = (/ &
     &1.6164e+01_rb,1.4145e+01_rb,1.2124e+01_rb,1.0103e+01_rb,8.0829e+00_rb,6.0627e+00_rb, &
     &4.0424e+00_rb,2.0217e+00_rb,2.2135e-01_rb/)
      kao(:, 4, 3,11) = (/ &
     &1.6211e+01_rb,1.4185e+01_rb,1.2159e+01_rb,1.0132e+01_rb,8.1062e+00_rb,6.0800e+00_rb, &
     &4.0538e+00_rb,2.0361e+00_rb,2.7967e-01_rb/)
      kao(:, 5, 3,11) = (/ &
     &1.6213e+01_rb,1.4187e+01_rb,1.2160e+01_rb,1.0134e+01_rb,8.1068e+00_rb,6.0810e+00_rb, &
     &4.0544e+00_rb,2.0887e+00_rb,3.5085e-01_rb/)
      kao(:, 1, 4,11) = (/ &
     &3.3505e+01_rb,2.9316e+01_rb,2.5128e+01_rb,2.0940e+01_rb,1.6753e+01_rb,1.2564e+01_rb, &
     &8.3770e+00_rb,4.1888e+00_rb,1.1343e-01_rb/)
      kao(:, 2, 4,11) = (/ &
     &3.3957e+01_rb,2.9712e+01_rb,2.5469e+01_rb,2.1224e+01_rb,1.6979e+01_rb,1.2735e+01_rb, &
     &8.4899e+00_rb,4.2456e+00_rb,1.5601e-01_rb/)
      kao(:, 3, 4,11) = (/ &
     &3.4273e+01_rb,2.9992e+01_rb,2.5707e+01_rb,2.1421e+01_rb,1.7138e+01_rb,1.2854e+01_rb, &
     &8.5693e+00_rb,4.2852e+00_rb,2.0536e-01_rb/)
      kao(:, 4, 4,11) = (/ &
     &3.4537e+01_rb,3.0216e+01_rb,2.5901e+01_rb,2.1585e+01_rb,1.7269e+01_rb,1.2951e+01_rb, &
     &8.6346e+00_rb,4.3178e+00_rb,2.6462e-01_rb/)
      kao(:, 5, 4,11) = (/ &
     &3.4713e+01_rb,3.0370e+01_rb,2.6032e+01_rb,2.1696e+01_rb,1.7356e+01_rb,1.3018e+01_rb, &
     &8.6791e+00_rb,4.3400e+00_rb,3.3230e-01_rb/)
      kao(:, 1, 5,11) = (/ &
     &6.4813e+01_rb,5.6712e+01_rb,4.8608e+01_rb,4.0507e+01_rb,3.2408e+01_rb,2.4306e+01_rb, &
     &1.6204e+01_rb,8.1017e+00_rb,9.6356e-02_rb/)
      kao(:, 2, 5,11) = (/ &
     &6.6164e+01_rb,5.7895e+01_rb,4.9623e+01_rb,4.1352e+01_rb,3.3084e+01_rb,2.4813e+01_rb, &
     &1.6542e+01_rb,8.2707e+00_rb,1.3652e-01_rb/)
      kao(:, 3, 5,11) = (/ &
     &6.7228e+01_rb,5.8826e+01_rb,5.0425e+01_rb,4.2018e+01_rb,3.3613e+01_rb,2.5211e+01_rb, &
     &1.6808e+01_rb,8.4044e+00_rb,1.8472e-01_rb/)
      kao(:, 4, 5,11) = (/ &
     &6.8035e+01_rb,5.9536e+01_rb,5.1024e+01_rb,4.2523e+01_rb,3.4019e+01_rb,2.5513e+01_rb, &
     &1.7010e+01_rb,8.5064e+00_rb,2.4242e-01_rb/)
      kao(:, 5, 5,11) = (/ &
     &6.8554e+01_rb,5.9987e+01_rb,5.1420e+01_rb,4.2849e+01_rb,3.4277e+01_rb,2.5710e+01_rb, &
     &1.7140e+01_rb,8.5702e+00_rb,3.0919e-01_rb/)
      kao(:, 1, 6,11) = (/ &
     &1.1801e+02_rb,1.0326e+02_rb,8.8506e+01_rb,7.3757e+01_rb,5.9002e+01_rb,4.4254e+01_rb, &
     &2.9503e+01_rb,1.4751e+01_rb,7.7344e-02_rb/)
      kao(:, 2, 6,11) = (/ &
     &1.2108e+02_rb,1.0594e+02_rb,9.0808e+01_rb,7.5672e+01_rb,6.0541e+01_rb,4.5406e+01_rb, &
     &3.0271e+01_rb,1.5134e+01_rb,1.1263e-01_rb/)
      kao(:, 3, 6,11) = (/ &
     &1.2354e+02_rb,1.0810e+02_rb,9.2652e+01_rb,7.7213e+01_rb,6.1769e+01_rb,4.6330e+01_rb, &
     &3.0886e+01_rb,1.5443e+01_rb,1.5682e-01_rb/)
      kao(:, 4, 6,11) = (/ &
     &1.2526e+02_rb,1.0960e+02_rb,9.3943e+01_rb,7.8284e+01_rb,6.2629e+01_rb,4.6975e+01_rb, &
     &3.1316e+01_rb,1.5657e+01_rb,2.1207e-01_rb/)
      kao(:, 5, 6,11) = (/ &
     &1.2664e+02_rb,1.1081e+02_rb,9.4976e+01_rb,7.9149e+01_rb,6.3321e+01_rb,4.7490e+01_rb, &
     &3.1658e+01_rb,1.5831e+01_rb,2.7724e-01_rb/)
      kao(:, 1, 7,11) = (/ &
     &2.3249e+02_rb,2.0341e+02_rb,1.7435e+02_rb,1.4529e+02_rb,1.1624e+02_rb,8.7178e+01_rb, &
     &5.8117e+01_rb,2.9059e+01_rb,6.0308e-02_rb/)
      kao(:, 2, 7,11) = (/ &
     &2.3796e+02_rb,2.0820e+02_rb,1.7846e+02_rb,1.4871e+02_rb,1.1897e+02_rb,8.9231e+01_rb, &
     &5.9488e+01_rb,2.9745e+01_rb,9.0208e-02_rb/)
      kao(:, 3, 7,11) = (/ &
     &2.4226e+02_rb,2.1197e+02_rb,1.8170e+02_rb,1.5141e+02_rb,1.2113e+02_rb,9.0843e+01_rb, &
     &6.0568e+01_rb,3.0281e+01_rb,1.2927e-01_rb/)
      kao(:, 4, 7,11) = (/ &
     &2.4544e+02_rb,2.1477e+02_rb,1.8409e+02_rb,1.5341e+02_rb,1.2272e+02_rb,9.2044e+01_rb, &
     &6.1361e+01_rb,3.0682e+01_rb,1.7878e-01_rb/)
      kao(:, 5, 7,11) = (/ &
     &2.4764e+02_rb,2.1668e+02_rb,1.8573e+02_rb,1.5477e+02_rb,1.2381e+02_rb,9.2866e+01_rb, &
     &6.1907e+01_rb,3.0956e+01_rb,2.3824e-01_rb/)
      kao(:, 1, 8,11) = (/ &
     &5.7428e+02_rb,5.0249e+02_rb,4.3070e+02_rb,3.5893e+02_rb,2.8714e+02_rb,2.1536e+02_rb, &
     &1.4357e+02_rb,7.1786e+01_rb,4.7227e-02_rb/)
      kao(:, 2, 8,11) = (/ &
     &5.8499e+02_rb,5.1190e+02_rb,4.3878e+02_rb,3.6564e+02_rb,2.9252e+02_rb,2.1939e+02_rb, &
     &1.4626e+02_rb,7.3130e+01_rb,7.2171e-02_rb/)
      kao(:, 3, 8,11) = (/ &
     &5.9291e+02_rb,5.1879e+02_rb,4.4470e+02_rb,3.7058e+02_rb,2.9645e+02_rb,2.2235e+02_rb, &
     &1.4822e+02_rb,7.4114e+01_rb,1.0536e-01_rb/)
      kao(:, 4, 8,11) = (/ &
     &5.9816e+02_rb,5.2341e+02_rb,4.4862e+02_rb,3.7386e+02_rb,2.9908e+02_rb,2.2431e+02_rb, &
     &1.4954e+02_rb,7.4773e+01_rb,1.4833e-01_rb/)
      kao(:, 5, 8,11) = (/ &
     &6.0144e+02_rb,5.2627e+02_rb,4.5109e+02_rb,3.7590e+02_rb,3.0070e+02_rb,2.2554e+02_rb, &
     &1.5036e+02_rb,7.5181e+01_rb,1.9981e-01_rb/)
      kao(:, 1, 9,11) = (/ &
     &2.7514e+03_rb,2.4073e+03_rb,2.0635e+03_rb,1.7196e+03_rb,1.3757e+03_rb,1.0318e+03_rb, &
     &6.8788e+02_rb,3.4393e+02_rb,3.3107e-02_rb/)
      kao(:, 2, 9,11) = (/ &
     &2.7973e+03_rb,2.4476e+03_rb,2.0979e+03_rb,1.7482e+03_rb,1.3986e+03_rb,1.0490e+03_rb, &
     &6.9932e+02_rb,3.4966e+02_rb,5.6998e-02_rb/)
      kao(:, 3, 9,11) = (/ &
     &2.8265e+03_rb,2.4731e+03_rb,2.1197e+03_rb,1.7664e+03_rb,1.4131e+03_rb,1.0599e+03_rb, &
     &7.0662e+02_rb,3.5330e+02_rb,8.5901e-02_rb/)
      kao(:, 4, 9,11) = (/ &
     &2.8388e+03_rb,2.4841e+03_rb,2.1291e+03_rb,1.7743e+03_rb,1.4194e+03_rb,1.0646e+03_rb, &
     &7.0974e+02_rb,3.5484e+02_rb,1.2233e-01_rb/)
      kao(:, 5, 9,11) = (/ &
     &2.8373e+03_rb,2.4828e+03_rb,2.1280e+03_rb,1.7734e+03_rb,1.4187e+03_rb,1.0640e+03_rb, &
     &7.0933e+02_rb,3.5467e+02_rb,1.6642e-01_rb/)
      kao(:, 1,10,11) = (/ &
     &1.4796e+04_rb,1.2947e+04_rb,1.1097e+04_rb,9.2476e+03_rb,7.3982e+03_rb,5.5489e+03_rb, &
     &3.6992e+03_rb,1.8496e+03_rb,1.7335e-02_rb/)
      kao(:, 2,10,11) = (/ &
     &1.4975e+04_rb,1.3103e+04_rb,1.1231e+04_rb,9.3585e+03_rb,7.4871e+03_rb,5.6155e+03_rb, &
     &3.7435e+03_rb,1.8717e+03_rb,3.1226e-02_rb/)
      kao(:, 3,10,11) = (/ &
     &1.5074e+04_rb,1.3189e+04_rb,1.1305e+04_rb,9.4210e+03_rb,7.5370e+03_rb,5.6528e+03_rb, &
     &3.7685e+03_rb,1.8842e+03_rb,4.3228e-02_rb/)
      kao(:, 4,10,11) = (/ &
     &1.5109e+04_rb,1.3220e+04_rb,1.1331e+04_rb,9.4424e+03_rb,7.5542e+03_rb,5.6659e+03_rb, &
     &3.7773e+03_rb,1.8886e+03_rb,5.6685e-02_rb/)
      kao(:, 5,10,11) = (/ &
     &1.5082e+04_rb,1.3196e+04_rb,1.1311e+04_rb,9.4257e+03_rb,7.5405e+03_rb,5.6555e+03_rb, &
     &3.7704e+03_rb,1.8852e+03_rb,8.6311e-02_rb/)
      kao(:, 1,11,11) = (/ &
     &2.7889e+04_rb,2.4402e+04_rb,2.0916e+04_rb,1.7430e+04_rb,1.3944e+04_rb,1.0458e+04_rb, &
     &6.9720e+03_rb,3.4860e+03_rb,1.1432e-02_rb/)
      kao(:, 2,11,11) = (/ &
     &2.8116e+04_rb,2.4602e+04_rb,2.1087e+04_rb,1.7573e+04_rb,1.4058e+04_rb,1.0543e+04_rb, &
     &7.0292e+03_rb,3.5144e+03_rb,1.1075e-02_rb/)
      kao(:, 3,11,11) = (/ &
     &2.8224e+04_rb,2.4695e+04_rb,2.1166e+04_rb,1.7640e+04_rb,1.4112e+04_rb,1.0583e+04_rb, &
     &7.0560e+03_rb,3.5276e+03_rb,4.6547e-02_rb/)
      kao(:, 4,11,11) = (/ &
     &2.8217e+04_rb,2.4690e+04_rb,2.1162e+04_rb,1.7636e+04_rb,1.4108e+04_rb,1.0581e+04_rb, &
     &7.0540e+03_rb,3.5272e+03_rb,6.2488e-02_rb/)
      kao(:, 5,11,11) = (/ &
     &2.8126e+04_rb,2.4609e+04_rb,2.1093e+04_rb,1.7578e+04_rb,1.4063e+04_rb,1.0547e+04_rb, &
     &7.0311e+03_rb,3.5155e+03_rb,7.5051e-02_rb/)
      kao(:, 1,12,11) = (/ &
     &3.8808e+04_rb,3.3958e+04_rb,2.9106e+04_rb,2.4255e+04_rb,1.9403e+04_rb,1.4552e+04_rb, &
     &9.7020e+03_rb,4.8510e+03_rb,3.0879e-04_rb/)
      kao(:, 2,12,11) = (/ &
     &3.8964e+04_rb,3.4093e+04_rb,2.9222e+04_rb,2.4351e+04_rb,1.9482e+04_rb,1.4612e+04_rb, &
     &9.7411e+03_rb,4.8703e+03_rb,1.3840e-02_rb/)
      kao(:, 3,12,11) = (/ &
     &3.8935e+04_rb,3.4067e+04_rb,2.9198e+04_rb,2.4333e+04_rb,1.9466e+04_rb,1.4600e+04_rb, &
     &9.7333e+03_rb,4.8667e+03_rb,2.1306e-02_rb/)
      kao(:, 4,12,11) = (/ &
     &3.8831e+04_rb,3.3979e+04_rb,2.9123e+04_rb,2.4270e+04_rb,1.9417e+04_rb,1.4562e+04_rb, &
     &9.7077e+03_rb,4.8538e+03_rb,6.5973e-02_rb/)
      kao(:, 5,12,11) = (/ &
     &3.8549e+04_rb,3.3728e+04_rb,2.8909e+04_rb,2.4091e+04_rb,1.9273e+04_rb,1.4455e+04_rb, &
     &9.6358e+03_rb,4.8182e+03_rb,7.9959e-02_rb/)
      kao(:, 1,13,11) = (/ &
     &4.2486e+04_rb,3.7175e+04_rb,3.1864e+04_rb,2.6552e+04_rb,2.1242e+04_rb,1.5932e+04_rb, &
     &1.0621e+04_rb,5.3106e+03_rb,2.4053e-04_rb/)
      kao(:, 2,13,11) = (/ &
     &4.2536e+04_rb,3.7221e+04_rb,3.1903e+04_rb,2.6585e+04_rb,2.1268e+04_rb,1.5951e+04_rb, &
     &1.0634e+04_rb,5.3169e+03_rb,2.3604e-02_rb/)
      kao(:, 3,13,11) = (/ &
     &4.2342e+04_rb,3.7047e+04_rb,3.1757e+04_rb,2.6467e+04_rb,2.1172e+04_rb,1.5879e+04_rb, &
     &1.0585e+04_rb,5.2925e+03_rb,1.5018e-02_rb/)
      kao(:, 4,13,11) = (/ &
     &4.2048e+04_rb,3.6792e+04_rb,3.1537e+04_rb,2.6281e+04_rb,2.1024e+04_rb,1.5767e+04_rb, &
     &1.0512e+04_rb,5.2560e+03_rb,5.3807e-02_rb/)
      kao(:, 5,13,11) = (/ &
     &4.1662e+04_rb,3.6457e+04_rb,3.1248e+04_rb,2.6040e+04_rb,2.0833e+04_rb,1.5623e+04_rb, &
     &1.0416e+04_rb,5.2082e+03_rb,6.8318e-02_rb/)
      kao(:, 1, 1,12) = (/ &
     &7.2849e+00_rb,6.3745e+00_rb,5.4643e+00_rb,4.5537e+00_rb,3.6431e+00_rb,2.7326e+00_rb, &
     &1.8223e+00_rb,9.1190e-01_rb,1.6091e-01_rb/)
      kao(:, 2, 1,12) = (/ &
     &7.1530e+00_rb,6.2588e+00_rb,5.3648e+00_rb,4.4709e+00_rb,3.5770e+00_rb,2.6830e+00_rb, &
     &1.7891e+00_rb,8.9548e-01_rb,2.2421e-01_rb/)
      kao(:, 3, 1,12) = (/ &
     &7.0158e+00_rb,6.1389e+00_rb,5.2624e+00_rb,4.3853e+00_rb,3.5089e+00_rb,2.6317e+00_rb, &
     &1.7551e+00_rb,9.0204e-01_rb,3.0513e-01_rb/)
      kao(:, 4, 1,12) = (/ &
     &6.8752e+00_rb,6.0158e+00_rb,5.1568e+00_rb,4.2975e+00_rb,3.4386e+00_rb,2.5792e+00_rb, &
     &1.7200e+00_rb,9.2056e-01_rb,4.0130e-01_rb/)
      kao(:, 5, 1,12) = (/ &
     &6.7312e+00_rb,5.8897e+00_rb,5.0486e+00_rb,4.2075e+00_rb,3.3663e+00_rb,2.5251e+00_rb, &
     &1.6886e+00_rb,9.1312e-01_rb,5.1163e-01_rb/)
      kao(:, 1, 2,12) = (/ &
     &1.1380e+01_rb,9.9571e+00_rb,8.5349e+00_rb,7.1126e+00_rb,5.6896e+00_rb,4.2680e+00_rb, &
     &2.8456e+00_rb,1.4234e+00_rb,1.6023e-01_rb/)
      kao(:, 2, 2,12) = (/ &
     &1.1190e+01_rb,9.7913e+00_rb,8.3926e+00_rb,6.9944e+00_rb,5.5956e+00_rb,4.1971e+00_rb, &
     &2.7983e+00_rb,1.3998e+00_rb,2.1092e-01_rb/)
      kao(:, 3, 2,12) = (/ &
     &1.0994e+01_rb,9.6196e+00_rb,8.2451e+00_rb,6.8714e+00_rb,5.4970e+00_rb,4.1233e+00_rb, &
     &2.7491e+00_rb,1.3753e+00_rb,2.8249e-01_rb/)
      kao(:, 4, 2,12) = (/ &
     &1.0789e+01_rb,9.4404e+00_rb,8.0917e+00_rb,6.7431e+00_rb,5.3950e+00_rb,4.0464e+00_rb, &
     &2.6980e+00_rb,1.3854e+00_rb,3.7167e-01_rb/)
      kao(:, 5, 2,12) = (/ &
     &1.0574e+01_rb,9.2528e+00_rb,7.9306e+00_rb,6.6095e+00_rb,5.2877e+00_rb,3.9662e+00_rb, &
     &2.6446e+00_rb,1.3862e+00_rb,4.7628e-01_rb/)
      kao(:, 1, 3,12) = (/ &
     &2.4019e+01_rb,2.1015e+01_rb,1.8012e+01_rb,1.5011e+01_rb,1.2010e+01_rb,9.0070e+00_rb, &
     &6.0051e+00_rb,3.0029e+00_rb,1.5458e-01_rb/)
      kao(:, 2, 3,12) = (/ &
     &2.3668e+01_rb,2.0712e+01_rb,1.7752e+01_rb,1.4794e+01_rb,1.1835e+01_rb,8.8764e+00_rb, &
     &5.9181e+00_rb,2.9594e+00_rb,2.0548e-01_rb/)
      kao(:, 3, 3,12) = (/ &
     &2.3299e+01_rb,2.0389e+01_rb,1.7475e+01_rb,1.4563e+01_rb,1.1650e+01_rb,8.7378e+00_rb, &
     &5.8256e+00_rb,2.9134e+00_rb,2.6510e-01_rb/)
      kao(:, 4, 3,12) = (/ &
     &2.2895e+01_rb,2.0033e+01_rb,1.7172e+01_rb,1.4310e+01_rb,1.1449e+01_rb,8.5868e+00_rb, &
     &5.7250e+00_rb,2.8629e+00_rb,3.4340e-01_rb/)
      kao(:, 5, 3,12) = (/ &
     &2.2457e+01_rb,1.9650e+01_rb,1.6845e+01_rb,1.4036e+01_rb,1.1229e+01_rb,8.4225e+00_rb, &
     &5.6152e+00_rb,2.8081e+00_rb,4.3834e-01_rb/)
      kao(:, 1, 4,12) = (/ &
     &5.1943e+01_rb,4.5448e+01_rb,3.8958e+01_rb,3.2464e+01_rb,2.5972e+01_rb,1.9479e+01_rb, &
     &1.2986e+01_rb,6.4930e+00_rb,1.4248e-01_rb/)
      kao(:, 2, 4,12) = (/ &
     &5.1352e+01_rb,4.4935e+01_rb,3.8514e+01_rb,3.2094e+01_rb,2.5675e+01_rb,1.9257e+01_rb, &
     &1.2838e+01_rb,6.4195e+00_rb,1.9502e-01_rb/)
      kao(:, 3, 4,12) = (/ &
     &5.0623e+01_rb,4.4294e+01_rb,3.7964e+01_rb,3.1639e+01_rb,2.5309e+01_rb,1.8983e+01_rb, &
     &1.2655e+01_rb,6.3280e+00_rb,2.5898e-01_rb/)
      kao(:, 4, 4,12) = (/ &
     &4.9799e+01_rb,4.3576e+01_rb,3.7352e+01_rb,3.1124e+01_rb,2.4901e+01_rb,1.8676e+01_rb, &
     &1.2451e+01_rb,6.2257e+00_rb,3.3035e-01_rb/)
      kao(:, 5, 4,12) = (/ &
     &4.8898e+01_rb,4.2791e+01_rb,3.6677e+01_rb,3.0566e+01_rb,2.4452e+01_rb,1.8340e+01_rb, &
     &1.2226e+01_rb,6.1134e+00_rb,4.1835e-01_rb/)
      kao(:, 1, 5,12) = (/ &
     &1.0373e+02_rb,9.0759e+01_rb,7.7796e+01_rb,6.4826e+01_rb,5.1857e+01_rb,3.8894e+01_rb, &
     &2.5931e+01_rb,1.2966e+01_rb,1.2887e-01_rb/)
      kao(:, 2, 5,12) = (/ &
     &1.0276e+02_rb,8.9908e+01_rb,7.7065e+01_rb,6.4222e+01_rb,5.1382e+01_rb,3.8535e+01_rb, &
     &2.5690e+01_rb,1.2845e+01_rb,1.8027e-01_rb/)
      kao(:, 3, 5,12) = (/ &
     &1.0148e+02_rb,8.8794e+01_rb,7.6113e+01_rb,6.3420e+01_rb,5.0736e+01_rb,3.8055e+01_rb, &
     &2.5371e+01_rb,1.2686e+01_rb,2.4350e-01_rb/)
      kao(:, 4, 5,12) = (/ &
     &9.9981e+01_rb,8.7486e+01_rb,7.4989e+01_rb,6.2492e+01_rb,4.9992e+01_rb,3.7496e+01_rb, &
     &2.4996e+01_rb,1.2498e+01_rb,3.1895e-01_rb/)
      kao(:, 5, 5,12) = (/ &
     &9.8403e+01_rb,8.6098e+01_rb,7.3801e+01_rb,6.1498e+01_rb,4.9199e+01_rb,3.6902e+01_rb, &
     &2.4598e+01_rb,1.2301e+01_rb,4.0491e-01_rb/)
      kao(:, 1, 6,12) = (/ &
     &1.9449e+02_rb,1.7021e+02_rb,1.4589e+02_rb,1.2157e+02_rb,9.7258e+01_rb,7.2943e+01_rb, &
     &4.8628e+01_rb,2.4314e+01_rb,1.1014e-01_rb/)
      kao(:, 2, 6,12) = (/ &
     &1.9309e+02_rb,1.6897e+02_rb,1.4483e+02_rb,1.2070e+02_rb,9.6559e+01_rb,7.2419e+01_rb, &
     &4.8279e+01_rb,2.4139e+01_rb,1.6004e-01_rb/)
      kao(:, 3, 6,12) = (/ &
     &1.9140e+02_rb,1.6747e+02_rb,1.4356e+02_rb,1.1963e+02_rb,9.5709e+01_rb,7.1777e+01_rb, &
     &4.7853e+01_rb,2.3926e+01_rb,2.2190e-01_rb/)
      kao(:, 4, 6,12) = (/ &
     &1.8899e+02_rb,1.6537e+02_rb,1.4174e+02_rb,1.1810e+02_rb,9.4493e+01_rb,7.0867e+01_rb, &
     &4.7246e+01_rb,2.3622e+01_rb,2.9571e-01_rb/)
      kao(:, 5, 6,12) = (/ &
     &1.8566e+02_rb,1.6245e+02_rb,1.3925e+02_rb,1.1603e+02_rb,9.2834e+01_rb,6.9623e+01_rb, &
     &4.6415e+01_rb,2.3209e+01_rb,3.8274e-01_rb/)
      kao(:, 1, 7,12) = (/ &
     &3.8417e+02_rb,3.3613e+02_rb,2.8811e+02_rb,2.4009e+02_rb,1.9205e+02_rb,1.4405e+02_rb, &
     &9.6037e+01_rb,4.8023e+01_rb,9.0335e-02_rb/)
      kao(:, 2, 7,12) = (/ &
     &3.8253e+02_rb,3.3473e+02_rb,2.8690e+02_rb,2.3908e+02_rb,1.9128e+02_rb,1.4345e+02_rb, &
     &9.5632e+01_rb,4.7816e+01_rb,1.3557e-01_rb/)
      kao(:, 3, 7,12) = (/ &
     &3.7949e+02_rb,3.3204e+02_rb,2.8460e+02_rb,2.3719e+02_rb,1.8975e+02_rb,1.4230e+02_rb, &
     &9.4872e+01_rb,4.7438e+01_rb,1.9476e-01_rb/)
      kao(:, 4, 7,12) = (/ &
     &3.7465e+02_rb,3.2783e+02_rb,2.8099e+02_rb,2.3416e+02_rb,1.8732e+02_rb,1.4050e+02_rb, &
     &9.3670e+01_rb,4.6831e+01_rb,2.6694e-01_rb/)
      kao(:, 5, 7,12) = (/ &
     &3.6931e+02_rb,3.2313e+02_rb,2.7696e+02_rb,2.3080e+02_rb,1.8464e+02_rb,1.3849e+02_rb, &
     &9.2317e+01_rb,4.6160e+01_rb,3.5213e-01_rb/)
      kao(:, 1, 8,12) = (/ &
     &9.0810e+02_rb,7.9455e+02_rb,6.8107e+02_rb,5.6755e+02_rb,4.5403e+02_rb,3.4053e+02_rb, &
     &2.2701e+02_rb,1.1352e+02_rb,7.1564e-02_rb/)
      kao(:, 2, 8,12) = (/ &
     &9.0605e+02_rb,7.9275e+02_rb,6.7956e+02_rb,5.6628e+02_rb,4.5302e+02_rb,3.3978e+02_rb, &
     &2.2652e+02_rb,1.1325e+02_rb,1.1086e-01_rb/)
      kao(:, 3, 8,12) = (/ &
     &9.0093e+02_rb,7.8832e+02_rb,6.7569e+02_rb,5.6306e+02_rb,4.5046e+02_rb,3.3783e+02_rb, &
     &2.2521e+02_rb,1.1262e+02_rb,1.6463e-01_rb/)
      kao(:, 4, 8,12) = (/ &
     &8.9363e+02_rb,7.8197e+02_rb,6.7027e+02_rb,5.5850e+02_rb,4.4681e+02_rb,3.3513e+02_rb, &
     &2.2342e+02_rb,1.1170e+02_rb,2.3145e-01_rb/)
      kao(:, 5, 8,12) = (/ &
     &8.8174e+02_rb,7.7148e+02_rb,6.6129e+02_rb,5.5110e+02_rb,4.4085e+02_rb,3.3064e+02_rb, &
     &2.2045e+02_rb,1.1021e+02_rb,3.1429e-01_rb/)
      kao(:, 1, 9,12) = (/ &
     &4.0575e+03_rb,3.5502e+03_rb,3.0431e+03_rb,2.5358e+03_rb,2.0286e+03_rb,1.5215e+03_rb, &
     &1.0143e+03_rb,5.0720e+02_rb,5.5603e-02_rb/)
      kao(:, 2, 9,12) = (/ &
     &4.0575e+03_rb,3.5503e+03_rb,3.0431e+03_rb,2.5359e+03_rb,2.0286e+03_rb,1.5216e+03_rb, &
     &1.0143e+03_rb,5.0715e+02_rb,8.8381e-02_rb/)
      kao(:, 3, 9,12) = (/ &
     &4.0490e+03_rb,3.5425e+03_rb,3.0365e+03_rb,2.5303e+03_rb,2.0243e+03_rb,1.5183e+03_rb, &
     &1.0122e+03_rb,5.0611e+02_rb,1.3462e-01_rb/)
      kao(:, 4, 9,12) = (/ &
     &4.0163e+03_rb,3.5141e+03_rb,3.0122e+03_rb,2.5100e+03_rb,2.0082e+03_rb,1.5061e+03_rb, &
     &1.0041e+03_rb,5.0202e+02_rb,1.9418e-01_rb/)
      kao(:, 5, 9,12) = (/ &
     &3.9738e+03_rb,3.4768e+03_rb,2.9801e+03_rb,2.4835e+03_rb,1.9868e+03_rb,1.4900e+03_rb, &
     &9.9338e+02_rb,4.9668e+02_rb,2.7001e-01_rb/)
      kao(:, 1,10,12) = (/ &
     &2.0782e+04_rb,1.8184e+04_rb,1.5586e+04_rb,1.2989e+04_rb,1.0391e+04_rb,7.7931e+03_rb, &
     &5.1956e+03_rb,2.5977e+03_rb,1.5076e-02_rb/)
      kao(:, 2,10,12) = (/ &
     &2.0847e+04_rb,1.8242e+04_rb,1.5636e+04_rb,1.3030e+04_rb,1.0424e+04_rb,7.8180e+03_rb, &
     &5.2120e+03_rb,2.6061e+03_rb,3.9283e-02_rb/)
      kao(:, 3,10,12) = (/ &
     &2.0776e+04_rb,1.8179e+04_rb,1.5582e+04_rb,1.2985e+04_rb,1.0387e+04_rb,7.7905e+03_rb, &
     &5.1937e+03_rb,2.5968e+03_rb,5.3678e-02_rb/)
      kao(:, 4,10,12) = (/ &
     &2.0635e+04_rb,1.8055e+04_rb,1.5476e+04_rb,1.2897e+04_rb,1.0317e+04_rb,7.7379e+03_rb, &
     &5.1584e+03_rb,2.5793e+03_rb,8.4869e-02_rb/)
      kao(:, 5,10,12) = (/ &
     &2.0468e+04_rb,1.7910e+04_rb,1.5352e+04_rb,1.2793e+04_rb,1.0235e+04_rb,7.6761e+03_rb, &
     &5.1174e+03_rb,2.5585e+03_rb,1.7840e-01_rb/)
      kao(:, 1,11,12) = (/ &
     &3.8086e+04_rb,3.3327e+04_rb,2.8563e+04_rb,2.3804e+04_rb,1.9042e+04_rb,1.4282e+04_rb, &
     &9.5217e+03_rb,4.7607e+03_rb,3.5535e-02_rb/)
      kao(:, 2,11,12) = (/ &
     &3.8167e+04_rb,3.3395e+04_rb,2.8624e+04_rb,2.3853e+04_rb,1.9083e+04_rb,1.4312e+04_rb, &
     &9.5414e+03_rb,4.7707e+03_rb,2.0336e-02_rb/)
      kao(:, 3,11,12) = (/ &
     &3.8095e+04_rb,3.3333e+04_rb,2.8571e+04_rb,2.3809e+04_rb,1.9048e+04_rb,1.4285e+04_rb, &
     &9.5235e+03_rb,4.7617e+03_rb,7.3058e-02_rb/)
      kao(:, 4,11,12) = (/ &
     &3.7840e+04_rb,3.3110e+04_rb,2.8380e+04_rb,2.3651e+04_rb,1.8921e+04_rb,1.4191e+04_rb, &
     &9.4603e+03_rb,4.7302e+03_rb,9.2342e-02_rb/)
      kao(:, 5,11,12) = (/ &
     &3.7466e+04_rb,3.2782e+04_rb,2.8098e+04_rb,2.3416e+04_rb,1.8732e+04_rb,1.4049e+04_rb, &
     &9.3659e+03_rb,4.6831e+03_rb,1.1387e-01_rb/)
      kao(:, 1,12,12) = (/ &
     &5.3068e+04_rb,4.6434e+04_rb,3.9801e+04_rb,3.3167e+04_rb,2.6533e+04_rb,1.9901e+04_rb, &
     &1.3266e+04_rb,6.6334e+03_rb,5.3631e-05_rb/)
      kao(:, 2,12,12) = (/ &
     &5.2912e+04_rb,4.6296e+04_rb,3.9684e+04_rb,3.3067e+04_rb,2.6455e+04_rb,1.9841e+04_rb, &
     &1.3228e+04_rb,6.6136e+03_rb,3.3045e-02_rb/)
      kao(:, 3,12,12) = (/ &
     &5.2742e+04_rb,4.6150e+04_rb,3.9556e+04_rb,3.2963e+04_rb,2.6372e+04_rb,1.9779e+04_rb, &
     &1.3186e+04_rb,6.5925e+03_rb,2.4038e-02_rb/)
      kao(:, 4,12,12) = (/ &
     &5.2329e+04_rb,4.5788e+04_rb,3.9248e+04_rb,3.2707e+04_rb,2.6165e+04_rb,1.9623e+04_rb, &
     &1.3082e+04_rb,6.5410e+03_rb,8.8033e-02_rb/)
      kao(:, 5,12,12) = (/ &
     &5.1702e+04_rb,4.5237e+04_rb,3.8772e+04_rb,3.2311e+04_rb,2.5848e+04_rb,1.9386e+04_rb, &
     &1.2925e+04_rb,6.4623e+03_rb,1.0528e-01_rb/)
      kao(:, 1,13,12) = (/ &
     &5.9181e+04_rb,5.1784e+04_rb,4.4385e+04_rb,3.6986e+04_rb,2.9590e+04_rb,2.2192e+04_rb, &
     &1.4795e+04_rb,7.3976e+03_rb,6.3472e-05_rb/)
      kao(:, 2,13,12) = (/ &
     &5.9042e+04_rb,5.1661e+04_rb,4.4282e+04_rb,3.6901e+04_rb,2.9521e+04_rb,2.2141e+04_rb, &
     &1.4760e+04_rb,7.3802e+03_rb,8.2093e-03_rb/)
      kao(:, 3,13,12) = (/ &
     &5.8661e+04_rb,5.1328e+04_rb,4.3997e+04_rb,3.6663e+04_rb,2.9331e+04_rb,2.1998e+04_rb, &
     &1.4665e+04_rb,7.3324e+03_rb,2.9843e-02_rb/)
      kao(:, 4,13,12) = (/ &
     &5.8145e+04_rb,5.0874e+04_rb,4.3608e+04_rb,3.6342e+04_rb,2.9072e+04_rb,2.1804e+04_rb, &
     &1.4536e+04_rb,7.2678e+03_rb,5.7821e-02_rb/)
      kao(:, 5,13,12) = (/ &
     &5.7447e+04_rb,5.0266e+04_rb,4.3086e+04_rb,3.5905e+04_rb,2.8722e+04_rb,2.1542e+04_rb, &
     &1.4361e+04_rb,7.1808e+03_rb,1.1510e-01_rb/)
      kao(:, 1, 1,13) = (/ &
     &9.2441e+00_rb,8.0893e+00_rb,6.9334e+00_rb,5.7782e+00_rb,4.6228e+00_rb,3.4672e+00_rb, &
     &2.3119e+00_rb,1.1567e+00_rb,2.2855e-01_rb/)
      kao(:, 2, 1,13) = (/ &
     &8.8623e+00_rb,7.7552e+00_rb,6.6473e+00_rb,5.5397e+00_rb,4.4320e+00_rb,3.3245e+00_rb, &
     &2.2168e+00_rb,1.1091e+00_rb,3.2335e-01_rb/)
      kao(:, 3, 1,13) = (/ &
     &8.4833e+00_rb,7.4233e+00_rb,6.3625e+00_rb,5.3026e+00_rb,4.2423e+00_rb,3.1825e+00_rb, &
     &2.1219e+00_rb,1.0618e+00_rb,4.4367e-01_rb/)
      kao(:, 4, 1,13) = (/ &
     &8.1215e+00_rb,7.1065e+00_rb,6.0917e+00_rb,5.0761e+00_rb,4.0614e+00_rb,3.0468e+00_rb, &
     &2.0316e+00_rb,1.0641e+00_rb,5.9167e-01_rb/)
      kao(:, 5, 1,13) = (/ &
     &7.7672e+00_rb,6.7964e+00_rb,5.8258e+00_rb,4.8545e+00_rb,3.8845e+00_rb,2.9137e+00_rb, &
     &1.9430e+00_rb,1.0446e+00_rb,7.6901e-01_rb/)
      kao(:, 1, 2,13) = (/ &
     &1.4699e+01_rb,1.2863e+01_rb,1.1024e+01_rb,9.1876e+00_rb,7.3499e+00_rb,5.5129e+00_rb, &
     &3.6758e+00_rb,1.8383e+00_rb,2.1901e-01_rb/)
      kao(:, 2, 2,13) = (/ &
     &1.4081e+01_rb,1.2321e+01_rb,1.0561e+01_rb,8.8012e+00_rb,7.0414e+00_rb,5.2809e+00_rb, &
     &3.5211e+00_rb,1.7613e+00_rb,3.1665e-01_rb/)
      kao(:, 3, 2,13) = (/ &
     &1.3470e+01_rb,1.1786e+01_rb,1.0102e+01_rb,8.4178e+00_rb,6.7345e+00_rb,5.0512e+00_rb, &
     &3.3678e+00_rb,1.6847e+00_rb,4.4141e-01_rb/)
      kao(:, 4, 2,13) = (/ &
     &1.2871e+01_rb,1.1263e+01_rb,9.6542e+00_rb,8.0452e+00_rb,6.4365e+00_rb,4.8276e+00_rb, &
     &3.2187e+00_rb,1.6111e+00_rb,5.9558e-01_rb/)
      kao(:, 5, 2,13) = (/ &
     &1.2298e+01_rb,1.0761e+01_rb,9.2242e+00_rb,7.6869e+00_rb,6.1502e+00_rb,4.6127e+00_rb, &
     &3.0756e+00_rb,1.6035e+00_rb,7.7652e-01_rb/)
      kao(:, 1, 3,13) = (/ &
     &3.1779e+01_rb,2.7807e+01_rb,2.3834e+01_rb,1.9863e+01_rb,1.5890e+01_rb,1.1918e+01_rb, &
     &7.9454e+00_rb,3.9733e+00_rb,1.9638e-01_rb/)
      kao(:, 2, 3,13) = (/ &
     &3.0431e+01_rb,2.6628e+01_rb,2.2823e+01_rb,1.9020e+01_rb,1.5217e+01_rb,1.1412e+01_rb, &
     &7.6086e+00_rb,3.8049e+00_rb,2.7736e-01_rb/)
      kao(:, 3, 3,13) = (/ &
     &2.9103e+01_rb,2.5465e+01_rb,2.1826e+01_rb,1.8188e+01_rb,1.4551e+01_rb,1.0914e+01_rb, &
     &7.2766e+00_rb,3.6387e+00_rb,3.9551e-01_rb/)
      kao(:, 4, 3,13) = (/ &
     &2.7810e+01_rb,2.4334e+01_rb,2.0858e+01_rb,1.7381e+01_rb,1.3906e+01_rb,1.0429e+01_rb, &
     &6.9532e+00_rb,3.4769e+00_rb,5.4452e-01_rb/)
      kao(:, 5, 3,13) = (/ &
     &2.6585e+01_rb,2.3262e+01_rb,1.9938e+01_rb,1.6616e+01_rb,1.3293e+01_rb,9.9693e+00_rb, &
     &6.6465e+00_rb,3.3243e+00_rb,7.1925e-01_rb/)
      kao(:, 1, 4,13) = (/ &
     &7.0542e+01_rb,6.1725e+01_rb,5.2909e+01_rb,4.4087e+01_rb,3.5272e+01_rb,2.6453e+01_rb, &
     &1.7635e+01_rb,8.8184e+00_rb,1.8199e-01_rb/)
      kao(:, 2, 4,13) = (/ &
     &6.7511e+01_rb,5.9069e+01_rb,5.0634e+01_rb,4.2192e+01_rb,3.3755e+01_rb,2.5317e+01_rb, &
     &1.6878e+01_rb,8.4389e+00_rb,2.4599e-01_rb/)
      kao(:, 3, 4,13) = (/ &
     &6.4633e+01_rb,5.6554e+01_rb,4.8477e+01_rb,4.0397e+01_rb,3.2316e+01_rb,2.4237e+01_rb, &
     &1.6158e+01_rb,8.0799e+00_rb,3.3970e-01_rb/)
      kao(:, 4, 4,13) = (/ &
     &6.1828e+01_rb,5.4103e+01_rb,4.6370e+01_rb,3.8643e+01_rb,3.0915e+01_rb,2.3186e+01_rb, &
     &1.5458e+01_rb,7.7299e+00_rb,4.6987e-01_rb/)
      kao(:, 5, 4,13) = (/ &
     &5.9112e+01_rb,5.1730e+01_rb,4.4337e+01_rb,3.6946e+01_rb,2.9558e+01_rb,2.2169e+01_rb, &
     &1.4779e+01_rb,7.3903e+00_rb,6.3062e-01_rb/)
      kao(:, 1, 5,13) = (/ &
     &1.4439e+02_rb,1.2634e+02_rb,1.0830e+02_rb,9.0248e+01_rb,7.2199e+01_rb,5.4151e+01_rb, &
     &3.6099e+01_rb,1.8050e+01_rb,1.6579e-01_rb/)
      kao(:, 2, 5,13) = (/ &
     &1.3838e+02_rb,1.2109e+02_rb,1.0378e+02_rb,8.6480e+01_rb,6.9180e+01_rb,5.1890e+01_rb, &
     &3.4593e+01_rb,1.7299e+01_rb,2.2988e-01_rb/)
      kao(:, 3, 5,13) = (/ &
     &1.3259e+02_rb,1.1602e+02_rb,9.9432e+01_rb,8.2860e+01_rb,6.6285e+01_rb,4.9718e+01_rb, &
     &3.3146e+01_rb,1.6573e+01_rb,3.1063e-01_rb/)
      kao(:, 4, 5,13) = (/ &
     &1.2697e+02_rb,1.1110e+02_rb,9.5219e+01_rb,7.9357e+01_rb,6.3482e+01_rb,4.7609e+01_rb, &
     &3.1740e+01_rb,1.5871e+01_rb,4.1510e-01_rb/)
      kao(:, 5, 5,13) = (/ &
     &1.2152e+02_rb,1.0632e+02_rb,9.1129e+01_rb,7.5942e+01_rb,6.0752e+01_rb,4.5568e+01_rb, &
     &3.0378e+01_rb,1.5190e+01_rb,5.5503e-01_rb/)
      kao(:, 1, 6,13) = (/ &
     &2.7815e+02_rb,2.4339e+02_rb,2.0862e+02_rb,1.7384e+02_rb,1.3908e+02_rb,1.0430e+02_rb, &
     &6.9541e+01_rb,3.4769e+01_rb,1.4932e-01_rb/)
      kao(:, 2, 6,13) = (/ &
     &2.6703e+02_rb,2.3365e+02_rb,2.0028e+02_rb,1.6690e+02_rb,1.3351e+02_rb,1.0014e+02_rb, &
     &6.6764e+01_rb,3.3381e+01_rb,2.1038e-01_rb/)
      kao(:, 3, 6,13) = (/ &
     &2.5610e+02_rb,2.2409e+02_rb,1.9207e+02_rb,1.6007e+02_rb,1.2805e+02_rb,9.6039e+01_rb, &
     &6.4025e+01_rb,3.2016e+01_rb,2.9193e-01_rb/)
      kao(:, 4, 6,13) = (/ &
     &2.4602e+02_rb,2.1527e+02_rb,1.8451e+02_rb,1.5376e+02_rb,1.2300e+02_rb,9.2253e+01_rb, &
     &6.1503e+01_rb,3.0752e+01_rb,3.8735e-01_rb/)
      kao(:, 5, 6,13) = (/ &
     &2.3698e+02_rb,2.0737e+02_rb,1.7775e+02_rb,1.4811e+02_rb,1.1849e+02_rb,8.8866e+01_rb, &
     &5.9248e+01_rb,2.9622e+01_rb,5.0567e-01_rb/)
      kao(:, 1, 7,13) = (/ &
     &5.6531e+02_rb,4.9465e+02_rb,4.2396e+02_rb,3.5331e+02_rb,2.8267e+02_rb,2.1198e+02_rb, &
     &1.4132e+02_rb,7.0665e+01_rb,1.3236e-01_rb/)
      kao(:, 2, 7,13) = (/ &
     &5.4477e+02_rb,4.7663e+02_rb,4.0854e+02_rb,3.4045e+02_rb,2.7236e+02_rb,2.0426e+02_rb, &
     &1.3618e+02_rb,6.8089e+01_rb,1.9224e-01_rb/)
      kao(:, 3, 7,13) = (/ &
     &5.2553e+02_rb,4.5986e+02_rb,3.9416e+02_rb,3.2848e+02_rb,2.6277e+02_rb,1.9706e+02_rb, &
     &1.3138e+02_rb,6.5693e+01_rb,2.7007e-01_rb/)
      kao(:, 4, 7,13) = (/ &
     &5.0736e+02_rb,4.4395e+02_rb,3.8050e+02_rb,3.1709e+02_rb,2.5367e+02_rb,1.9026e+02_rb, &
     &1.2684e+02_rb,6.3421e+01_rb,3.6853e-01_rb/)
      kao(:, 5, 7,13) = (/ &
     &4.8966e+02_rb,4.2846e+02_rb,3.6727e+02_rb,3.0605e+02_rb,2.4484e+02_rb,1.8364e+02_rb, &
     &1.2242e+02_rb,6.1208e+01_rb,4.8516e-01_rb/)
      kao(:, 1, 8,13) = (/ &
     &1.3865e+03_rb,1.2132e+03_rb,1.0399e+03_rb,8.6656e+02_rb,6.9323e+02_rb,5.1995e+02_rb, &
     &3.4663e+02_rb,1.7332e+02_rb,1.1342e-01_rb/)
      kao(:, 2, 8,13) = (/ &
     &1.3418e+03_rb,1.1740e+03_rb,1.0063e+03_rb,8.3858e+02_rb,6.7084e+02_rb,5.0312e+02_rb, &
     &3.3544e+02_rb,1.6771e+02_rb,1.7153e-01_rb/)
      kao(:, 3, 8,13) = (/ &
     &1.2982e+03_rb,1.1360e+03_rb,9.7362e+02_rb,8.1138e+02_rb,6.4912e+02_rb,4.8683e+02_rb, &
     &3.2456e+02_rb,1.6229e+02_rb,2.4648e-01_rb/)
      kao(:, 4, 8,13) = (/ &
     &1.2554e+03_rb,1.0986e+03_rb,9.4155e+02_rb,7.8470e+02_rb,6.2776e+02_rb,4.7081e+02_rb, &
     &3.1388e+02_rb,1.5694e+02_rb,3.4456e-01_rb/)
      kao(:, 5, 8,13) = (/ &
     &1.2173e+03_rb,1.0650e+03_rb,9.1289e+02_rb,7.6069e+02_rb,6.0860e+02_rb,4.5645e+02_rb, &
     &3.0428e+02_rb,1.5215e+02_rb,4.6320e-01_rb/)
      kao(:, 1, 9,13) = (/ &
     &6.4287e+03_rb,5.6254e+03_rb,4.8220e+03_rb,4.0184e+03_rb,3.2146e+03_rb,2.4108e+03_rb, &
     &1.6072e+03_rb,8.0366e+02_rb,9.3435e-02_rb/)
      kao(:, 2, 9,13) = (/ &
     &6.2433e+03_rb,5.4627e+03_rb,4.6824e+03_rb,3.9018e+03_rb,3.1215e+03_rb,2.3411e+03_rb, &
     &1.5608e+03_rb,7.8032e+02_rb,1.4681e-01_rb/)
      kao(:, 3, 9,13) = (/ &
     &6.0612e+03_rb,5.3034e+03_rb,4.5455e+03_rb,3.7883e+03_rb,3.0305e+03_rb,2.2728e+03_rb, &
     &1.5153e+03_rb,7.5756e+02_rb,2.1819e-01_rb/)
      kao(:, 4, 9,13) = (/ &
     &5.8977e+03_rb,5.1604e+03_rb,4.4231e+03_rb,3.6861e+03_rb,2.9487e+03_rb,2.2117e+03_rb, &
     &1.4744e+03_rb,7.3720e+02_rb,3.1394e-01_rb/)
      kao(:, 5, 9,13) = (/ &
     &5.7293e+03_rb,5.0131e+03_rb,4.2969e+03_rb,3.5807e+03_rb,2.8646e+03_rb,2.1484e+03_rb, &
     &1.4323e+03_rb,7.1617e+02_rb,4.3083e-01_rb/)
      kao(:, 1,10,13) = (/ &
     &3.3200e+04_rb,2.9050e+04_rb,2.4899e+04_rb,2.0749e+04_rb,1.6600e+04_rb,1.2450e+04_rb, &
     &8.2998e+03_rb,4.1501e+03_rb,1.3035e-02_rb/)
      kao(:, 2,10,13) = (/ &
     &3.2379e+04_rb,2.8331e+04_rb,2.4286e+04_rb,2.0236e+04_rb,1.6189e+04_rb,1.2142e+04_rb, &
     &8.0946e+03_rb,4.0474e+03_rb,6.4014e-02_rb/)
      kao(:, 3,10,13) = (/ &
     &3.1566e+04_rb,2.7621e+04_rb,2.3674e+04_rb,1.9730e+04_rb,1.5784e+04_rb,1.1837e+04_rb, &
     &7.8919e+03_rb,3.9461e+03_rb,1.0897e-01_rb/)
      kao(:, 4,10,13) = (/ &
     &3.0728e+04_rb,2.6888e+04_rb,2.3048e+04_rb,1.9205e+04_rb,1.5364e+04_rb,1.1523e+04_rb, &
     &7.6820e+03_rb,3.8409e+03_rb,2.5342e-01_rb/)
      kao(:, 5,10,13) = (/ &
     &2.9824e+04_rb,2.6098e+04_rb,2.2369e+04_rb,1.8641e+04_rb,1.4913e+04_rb,1.1185e+04_rb, &
     &7.4563e+03_rb,3.7282e+03_rb,3.9865e-01_rb/)
      kao(:, 1,11,13) = (/ &
     &5.8997e+04_rb,5.1626e+04_rb,4.4246e+04_rb,3.6876e+04_rb,2.9498e+04_rb,2.2124e+04_rb, &
     &1.4750e+04_rb,7.3749e+03_rb,3.4853e-05_rb/)
      kao(:, 2,11,13) = (/ &
     &5.7601e+04_rb,5.0403e+04_rb,4.3203e+04_rb,3.6002e+04_rb,2.8802e+04_rb,2.1601e+04_rb, &
     &1.4400e+04_rb,7.2004e+03_rb,3.9149e-02_rb/)
      kao(:, 3,11,13) = (/ &
     &5.6184e+04_rb,4.9162e+04_rb,4.2136e+04_rb,3.5117e+04_rb,2.8092e+04_rb,2.1068e+04_rb, &
     &1.4046e+04_rb,7.0226e+03_rb,4.1745e-02_rb/)
      kao(:, 4,11,13) = (/ &
     &5.4639e+04_rb,4.7807e+04_rb,4.0979e+04_rb,3.4149e+04_rb,2.7319e+04_rb,2.0489e+04_rb, &
     &1.3659e+04_rb,6.8303e+03_rb,1.4397e-01_rb/)
      kao(:, 5,11,13) = (/ &
     &5.3017e+04_rb,4.6389e+04_rb,3.9763e+04_rb,3.3137e+04_rb,2.6509e+04_rb,1.9881e+04_rb, &
     &1.3255e+04_rb,6.6269e+03_rb,2.2772e-01_rb/)
      kao(:, 1,12,13) = (/ &
     &7.7971e+04_rb,6.8226e+04_rb,5.8475e+04_rb,4.8732e+04_rb,3.8985e+04_rb,2.9238e+04_rb, &
     &1.9491e+04_rb,9.7458e+03_rb,2.6211e-05_rb/)
      kao(:, 2,12,13) = (/ &
     &7.6358e+04_rb,6.6812e+04_rb,5.7265e+04_rb,4.7721e+04_rb,3.8180e+04_rb,2.8631e+04_rb, &
     &1.9088e+04_rb,9.5449e+03_rb,1.6658e-02_rb/)
      kao(:, 3,12,13) = (/ &
     &7.4598e+04_rb,6.5272e+04_rb,5.5949e+04_rb,4.6623e+04_rb,3.7300e+04_rb,2.7976e+04_rb, &
     &1.8650e+04_rb,9.3250e+03_rb,6.1472e-02_rb/)
      kao(:, 4,12,13) = (/ &
     &7.2683e+04_rb,6.3600e+04_rb,5.4513e+04_rb,4.5429e+04_rb,3.6343e+04_rb,2.7256e+04_rb, &
     &1.8171e+04_rb,9.0852e+03_rb,8.7020e-02_rb/)
      kao(:, 5,12,13) = (/ &
     &7.0749e+04_rb,6.1901e+04_rb,5.3059e+04_rb,4.4221e+04_rb,3.5375e+04_rb,2.6529e+04_rb, &
     &1.7687e+04_rb,8.8438e+03_rb,2.2135e-01_rb/)
      kao(:, 1,13,13) = (/ &
     &8.4389e+04_rb,7.3841e+04_rb,6.3292e+04_rb,5.2741e+04_rb,4.2193e+04_rb,3.1646e+04_rb, &
     &2.1097e+04_rb,1.0548e+04_rb,3.5977e-05_rb/)
      kao(:, 2,13,13) = (/ &
     &8.2810e+04_rb,7.2462e+04_rb,6.2107e+04_rb,5.1755e+04_rb,4.1404e+04_rb,3.1055e+04_rb, &
     &2.0703e+04_rb,1.0351e+04_rb,2.8201e-02_rb/)
      kao(:, 3,13,13) = (/ &
     &8.1175e+04_rb,7.1028e+04_rb,6.0878e+04_rb,5.0733e+04_rb,4.0585e+04_rb,3.0440e+04_rb, &
     &2.0293e+04_rb,1.0147e+04_rb,3.8140e-02_rb/)
      kao(:, 4,13,13) = (/ &
     &7.9409e+04_rb,6.9476e+04_rb,5.9554e+04_rb,4.9626e+04_rb,3.9700e+04_rb,2.9777e+04_rb, &
     &1.9850e+04_rb,9.9254e+03_rb,1.0573e-01_rb/)
      kao(:, 5,13,13) = (/ &
     &7.7670e+04_rb,6.7959e+04_rb,5.8251e+04_rb,4.8545e+04_rb,3.8836e+04_rb,2.9127e+04_rb, &
     &1.9417e+04_rb,9.7081e+03_rb,2.2322e-01_rb/)
      kao(:, 1, 1,14) = (/ &
     &1.0586e+01_rb,9.2639e+00_rb,7.9401e+00_rb,6.6170e+00_rb,5.2941e+00_rb,3.9707e+00_rb, &
     &2.6478e+00_rb,1.3250e+00_rb,4.3918e-01_rb/)
      kao(:, 2, 1,14) = (/ &
     &1.0067e+01_rb,8.8090e+00_rb,7.5510e+00_rb,6.2924e+00_rb,5.0343e+00_rb,3.7761e+00_rb, &
     &2.5179e+00_rb,1.2597e+00_rb,5.9709e-01_rb/)
      kao(:, 3, 1,14) = (/ &
     &9.5510e+00_rb,8.3580e+00_rb,7.1635e+00_rb,5.9699e+00_rb,4.7763e+00_rb,3.5829e+00_rb, &
     &2.3891e+00_rb,1.1957e+00_rb,7.8623e-01_rb/)
      kao(:, 4, 1,14) = (/ &
     &9.0745e+00_rb,7.9402e+00_rb,6.8068e+00_rb,5.6724e+00_rb,4.5382e+00_rb,3.4039e+00_rb, &
     &2.2698e+00_rb,1.1544e+00_rb,1.0064e+00_rb/)
      kao(:, 5, 1,14) = (/ &
     &8.6119e+00_rb,7.5354e+00_rb,6.4586e+00_rb,5.3823e+00_rb,4.3069e+00_rb,3.2305e+00_rb, &
     &2.1544e+00_rb,1.1513e+00_rb,1.2568e+00_rb/)
      kao(:, 1, 2,14) = (/ &
     &1.7127e+01_rb,1.4988e+01_rb,1.2847e+01_rb,1.0706e+01_rb,8.5644e+00_rb,6.4237e+00_rb, &
     &4.2830e+00_rb,2.1424e+00_rb,4.0087e-01_rb/)
      kao(:, 2, 2,14) = (/ &
     &1.6266e+01_rb,1.4233e+01_rb,1.2199e+01_rb,1.0167e+01_rb,8.1334e+00_rb,6.1003e+00_rb, &
     &4.0675e+00_rb,2.0344e+00_rb,5.5811e-01_rb/)
      kao(:, 3, 2,14) = (/ &
     &1.5417e+01_rb,1.3492e+01_rb,1.1564e+01_rb,9.6364e+00_rb,7.7096e+00_rb,5.7829e+00_rb, &
     &3.8557e+00_rb,1.9285e+00_rb,7.4958e-01_rb/)
      kao(:, 4, 2,14) = (/ &
     &1.4610e+01_rb,1.2785e+01_rb,1.0958e+01_rb,9.1323e+00_rb,7.3057e+00_rb,5.4803e+00_rb, &
     &3.6538e+00_rb,1.8278e+00_rb,9.7668e-01_rb/)
      kao(:, 5, 2,14) = (/ &
     &1.3849e+01_rb,1.2118e+01_rb,1.0388e+01_rb,8.6567e+00_rb,6.9253e+00_rb,5.1944e+00_rb, &
     &3.4637e+00_rb,1.7621e+00_rb,1.2463e+00_rb/)
      kao(:, 1, 3,14) = (/ &
     &3.7731e+01_rb,3.3014e+01_rb,2.8298e+01_rb,2.3584e+01_rb,1.8865e+01_rb,1.4149e+01_rb, &
     &9.4337e+00_rb,4.7180e+00_rb,3.4082e-01_rb/)
      kao(:, 2, 3,14) = (/ &
     &3.5785e+01_rb,3.1314e+01_rb,2.6842e+01_rb,2.2368e+01_rb,1.7895e+01_rb,1.3421e+01_rb, &
     &8.9480e+00_rb,4.4747e+00_rb,4.8919e-01_rb/)
      kao(:, 3, 3,14) = (/ &
     &3.3896e+01_rb,2.9659e+01_rb,2.5426e+01_rb,2.1185e+01_rb,1.6948e+01_rb,1.2712e+01_rb, &
     &8.4759e+00_rb,4.2386e+00_rb,6.7406e-01_rb/)
      kao(:, 4, 3,14) = (/ &
     &3.2057e+01_rb,2.8051e+01_rb,2.4047e+01_rb,2.0037e+01_rb,1.6031e+01_rb,1.2024e+01_rb, &
     &8.0158e+00_rb,4.0086e+00_rb,8.9895e-01_rb/)
      kao(:, 5, 3,14) = (/ &
     &3.0352e+01_rb,2.6560e+01_rb,2.2767e+01_rb,1.8972e+01_rb,1.5176e+01_rb,1.1383e+01_rb, &
     &7.5899e+00_rb,3.7954e+00_rb,1.1745e+00_rb/)
      kao(:, 1, 4,14) = (/ &
     &8.5512e+01_rb,7.4816e+01_rb,6.4130e+01_rb,5.3439e+01_rb,4.2750e+01_rb,3.2066e+01_rb, &
     &2.1376e+01_rb,1.0689e+01_rb,2.7979e-01_rb/)
      kao(:, 2, 4,14) = (/ &
     &8.0825e+01_rb,7.0719e+01_rb,6.0615e+01_rb,5.0514e+01_rb,4.0410e+01_rb,3.0306e+01_rb, &
     &2.0206e+01_rb,1.0104e+01_rb,4.1456e-01_rb/)
      kao(:, 3, 4,14) = (/ &
     &7.6512e+01_rb,6.6951e+01_rb,5.7383e+01_rb,4.7821e+01_rb,3.8259e+01_rb,2.8694e+01_rb, &
     &1.9130e+01_rb,9.5655e+00_rb,5.8875e-01_rb/)
      kao(:, 4, 4,14) = (/ &
     &7.2339e+01_rb,6.3299e+01_rb,5.4254e+01_rb,4.5213e+01_rb,3.6173e+01_rb,2.7127e+01_rb, &
     &1.8087e+01_rb,9.0435e+00_rb,8.0998e-01_rb/)
      kao(:, 5, 4,14) = (/ &
     &6.8331e+01_rb,5.9800e+01_rb,5.1253e+01_rb,4.2714e+01_rb,3.4172e+01_rb,2.5627e+01_rb, &
     &1.7086e+01_rb,8.5443e+00_rb,1.0852e+00_rb/)
      kao(:, 1, 5,14) = (/ &
     &1.7862e+02_rb,1.5630e+02_rb,1.3397e+02_rb,1.1164e+02_rb,8.9317e+01_rb,6.6992e+01_rb, &
     &4.4659e+01_rb,2.2329e+01_rb,2.4222e-01_rb/)
      kao(:, 2, 5,14) = (/ &
     &1.6866e+02_rb,1.4756e+02_rb,1.2649e+02_rb,1.0541e+02_rb,8.4332e+01_rb,6.3244e+01_rb, &
     &4.2162e+01_rb,2.1082e+01_rb,3.4856e-01_rb/)
      kao(:, 3, 5,14) = (/ &
     &1.5941e+02_rb,1.3948e+02_rb,1.1955e+02_rb,9.9631e+01_rb,7.9709e+01_rb,5.9780e+01_rb, &
     &3.9852e+01_rb,1.9927e+01_rb,5.0734e-01_rb/)
      kao(:, 4, 5,14) = (/ &
     &1.5067e+02_rb,1.3184e+02_rb,1.1302e+02_rb,9.4183e+01_rb,7.5341e+01_rb,5.6507e+01_rb, &
     &3.7674e+01_rb,1.8837e+01_rb,7.2180e-01_rb/)
      kao(:, 5, 5,14) = (/ &
     &1.4230e+02_rb,1.2451e+02_rb,1.0671e+02_rb,8.8935e+01_rb,7.1153e+01_rb,5.3363e+01_rb, &
     &3.5577e+01_rb,1.7789e+01_rb,9.9362e-01_rb/)
      kao(:, 1, 6,14) = (/ &
     &3.5096e+02_rb,3.0703e+02_rb,2.6318e+02_rb,2.1933e+02_rb,1.7546e+02_rb,1.3159e+02_rb, &
     &8.7733e+01_rb,4.3865e+01_rb,2.1734e-01_rb/)
      kao(:, 2, 6,14) = (/ &
     &3.3138e+02_rb,2.8996e+02_rb,2.4854e+02_rb,2.0711e+02_rb,1.6569e+02_rb,1.2426e+02_rb, &
     &8.2845e+01_rb,4.1423e+01_rb,3.0507e-01_rb/)
      kao(:, 3, 6,14) = (/ &
     &3.1254e+02_rb,2.7346e+02_rb,2.3441e+02_rb,1.9532e+02_rb,1.5627e+02_rb,1.1720e+02_rb, &
     &7.8130e+01_rb,3.9066e+01_rb,4.3270e-01_rb/)
      kao(:, 4, 6,14) = (/ &
     &2.9529e+02_rb,2.5841e+02_rb,2.2148e+02_rb,1.8456e+02_rb,1.4764e+02_rb,1.1074e+02_rb, &
     &7.3826e+01_rb,3.6914e+01_rb,6.2701e-01_rb/)
      kao(:, 5, 6,14) = (/ &
     &2.7880e+02_rb,2.4396e+02_rb,2.0911e+02_rb,1.7424e+02_rb,1.3940e+02_rb,1.0455e+02_rb, &
     &6.9706e+01_rb,3.4852e+01_rb,8.8890e-01_rb/)
      kao(:, 1, 7,14) = (/ &
     &7.2513e+02_rb,6.3446e+02_rb,5.4381e+02_rb,4.5320e+02_rb,3.6257e+02_rb,2.7191e+02_rb, &
     &1.8128e+02_rb,9.0650e+01_rb,1.9831e-01_rb/)
      kao(:, 2, 7,14) = (/ &
     &6.8456e+02_rb,5.9902e+02_rb,5.1343e+02_rb,4.2785e+02_rb,3.4230e+02_rb,2.5672e+02_rb, &
     &1.7115e+02_rb,8.5572e+01_rb,2.7818e-01_rb/)
      kao(:, 3, 7,14) = (/ &
     &6.4547e+02_rb,5.6482e+02_rb,4.8412e+02_rb,4.0342e+02_rb,3.2273e+02_rb,2.4204e+02_rb, &
     &1.6137e+02_rb,8.0688e+01_rb,3.8833e-01_rb/)
      kao(:, 4, 7,14) = (/ &
     &6.0978e+02_rb,5.3357e+02_rb,4.5735e+02_rb,3.8114e+02_rb,3.0489e+02_rb,2.2868e+02_rb, &
     &1.5246e+02_rb,7.6224e+01_rb,5.4866e-01_rb/)
      kao(:, 5, 7,14) = (/ &
     &5.7729e+02_rb,5.0509e+02_rb,4.3294e+02_rb,3.6078e+02_rb,2.8862e+02_rb,2.1646e+02_rb, &
     &1.4432e+02_rb,7.2163e+01_rb,7.7837e-01_rb/)
      kao(:, 1, 8,14) = (/ &
     &1.7967e+03_rb,1.5721e+03_rb,1.3476e+03_rb,1.1230e+03_rb,8.9837e+02_rb,6.7379e+02_rb, &
     &4.4917e+02_rb,2.2458e+02_rb,1.7788e-01_rb/)
      kao(:, 2, 8,14) = (/ &
     &1.7006e+03_rb,1.4881e+03_rb,1.2754e+03_rb,1.0629e+03_rb,8.5028e+02_rb,6.3778e+02_rb, &
     &4.2515e+02_rb,2.1258e+02_rb,2.5421e-01_rb/)
      kao(:, 3, 8,14) = (/ &
     &1.6106e+03_rb,1.4094e+03_rb,1.2079e+03_rb,1.0068e+03_rb,8.0532e+02_rb,6.0404e+02_rb, &
     &4.0268e+02_rb,2.0135e+02_rb,3.5995e-01_rb/)
      kao(:, 4, 8,14) = (/ &
     &1.5246e+03_rb,1.3340e+03_rb,1.1435e+03_rb,9.5278e+02_rb,7.6226e+02_rb,5.7172e+02_rb, &
     &3.8114e+02_rb,1.9058e+02_rb,4.9908e-01_rb/)
      kao(:, 5, 8,14) = (/ &
     &1.4465e+03_rb,1.2656e+03_rb,1.0849e+03_rb,9.0407e+02_rb,7.2320e+02_rb,5.4242e+02_rb, &
     &3.6158e+02_rb,1.8081e+02_rb,6.9586e-01_rb/)
      kao(:, 1, 9,14) = (/ &
     &8.4840e+03_rb,7.4233e+03_rb,6.3628e+03_rb,5.3023e+03_rb,4.2423e+03_rb,3.1813e+03_rb, &
     &2.1210e+03_rb,1.0605e+03_rb,1.5591e-01_rb/)
      kao(:, 2, 9,14) = (/ &
     &8.0479e+03_rb,7.0418e+03_rb,6.0359e+03_rb,5.0301e+03_rb,4.0241e+03_rb,3.0180e+03_rb, &
     &2.0121e+03_rb,1.0060e+03_rb,2.2879e-01_rb/)
      kao(:, 3, 9,14) = (/ &
     &7.6325e+03_rb,6.6786e+03_rb,5.7241e+03_rb,4.7707e+03_rb,3.8161e+03_rb,2.8622e+03_rb, &
     &1.9081e+03_rb,9.5403e+02_rb,3.3113e-01_rb/)
      kao(:, 4, 9,14) = (/ &
     &7.2431e+03_rb,6.3376e+03_rb,5.4325e+03_rb,4.5268e+03_rb,3.6217e+03_rb,2.7162e+03_rb, &
     &1.8109e+03_rb,9.0539e+02_rb,4.6545e-01_rb/)
      kao(:, 5, 9,14) = (/ &
     &6.8956e+03_rb,6.0340e+03_rb,5.1720e+03_rb,4.3099e+03_rb,3.4482e+03_rb,2.5859e+03_rb, &
     &1.7240e+03_rb,8.6204e+02_rb,6.4091e-01_rb/)
      kao(:, 1,10,14) = (/ &
     &4.4703e+04_rb,3.9114e+04_rb,3.3523e+04_rb,2.7938e+04_rb,2.2351e+04_rb,1.6764e+04_rb, &
     &1.1175e+04_rb,5.5874e+03_rb,4.1455e-02_rb/)
      kao(:, 2,10,14) = (/ &
     &4.2474e+04_rb,3.7167e+04_rb,3.1854e+04_rb,2.6544e+04_rb,2.1238e+04_rb,1.5927e+04_rb, &
     &1.0619e+04_rb,5.3089e+03_rb,6.6750e-02_rb/)
      kao(:, 3,10,14) = (/ &
     &4.0460e+04_rb,3.5405e+04_rb,3.0347e+04_rb,2.5289e+04_rb,2.0232e+04_rb,1.5174e+04_rb, &
     &1.0116e+04_rb,5.0578e+03_rb,2.7442e-01_rb/)
      kao(:, 4,10,14) = (/ &
     &3.8625e+04_rb,3.3797e+04_rb,2.8970e+04_rb,2.4141e+04_rb,1.9313e+04_rb,1.4484e+04_rb, &
     &9.6562e+03_rb,4.8282e+03_rb,4.4714e-01_rb/)
      kao(:, 5,10,14) = (/ &
     &3.6998e+04_rb,3.2375e+04_rb,2.7750e+04_rb,2.3124e+04_rb,1.8498e+04_rb,1.3874e+04_rb, &
     &9.2495e+03_rb,4.6247e+03_rb,6.1795e-01_rb/)
      kao(:, 1,11,14) = (/ &
     &8.0689e+04_rb,7.0605e+04_rb,6.0514e+04_rb,5.0431e+04_rb,4.0345e+04_rb,3.0258e+04_rb, &
     &2.0175e+04_rb,1.0088e+04_rb,2.7556e-05_rb/)
      kao(:, 2,11,14) = (/ &
     &7.7103e+04_rb,6.7463e+04_rb,5.7832e+04_rb,4.8186e+04_rb,3.8550e+04_rb,2.8913e+04_rb, &
     &1.9275e+04_rb,9.6376e+03_rb,4.0930e-02_rb/)
      kao(:, 3,11,14) = (/ &
     &7.3820e+04_rb,6.4596e+04_rb,5.5369e+04_rb,4.6139e+04_rb,3.6913e+04_rb,2.7687e+04_rb, &
     &1.8457e+04_rb,9.2272e+03_rb,8.1894e-02_rb/)
      kao(:, 4,11,14) = (/ &
     &7.1047e+04_rb,6.2162e+04_rb,5.3281e+04_rb,4.4402e+04_rb,3.5522e+04_rb,2.6643e+04_rb, &
     &1.7761e+04_rb,8.8802e+03_rb,2.3203e-01_rb/)
      kao(:, 5,11,14) = (/ &
     &6.8729e+04_rb,6.0139e+04_rb,5.1549e+04_rb,4.2956e+04_rb,3.4366e+04_rb,2.5774e+04_rb, &
     &1.7184e+04_rb,8.5912e+03_rb,5.4821e-01_rb/)
      kao(:, 1,12,14) = (/ &
     &1.0931e+05_rb,9.5652e+04_rb,8.1990e+04_rb,6.8319e+04_rb,5.4655e+04_rb,4.0991e+04_rb, &
     &2.7328e+04_rb,1.3665e+04_rb,2.8846e-05_rb/)
      kao(:, 2,12,14) = (/ &
     &1.0521e+05_rb,9.2062e+04_rb,7.8903e+04_rb,6.5759e+04_rb,5.2606e+04_rb,3.9453e+04_rb, &
     &2.6301e+04_rb,1.3151e+04_rb,4.7358e-02_rb/)
      kao(:, 3,12,14) = (/ &
     &1.0163e+05_rb,8.8927e+04_rb,7.6219e+04_rb,6.3520e+04_rb,5.0816e+04_rb,3.8110e+04_rb, &
     &2.5405e+04_rb,1.2704e+04_rb,2.4382e-02_rb/)
      kao(:, 4,12,14) = (/ &
     &9.8710e+04_rb,8.6365e+04_rb,7.4034e+04_rb,6.1691e+04_rb,4.9353e+04_rb,3.7016e+04_rb, &
     &2.4677e+04_rb,1.2338e+04_rb,1.4381e-01_rb/)
      kao(:, 5,12,14) = (/ &
     &9.6361e+04_rb,8.4318e+04_rb,7.2271e+04_rb,6.0231e+04_rb,4.8183e+04_rb,3.6135e+04_rb, &
     &2.4090e+04_rb,1.2044e+04_rb,3.4843e-01_rb/)
      kao(:, 1,13,14) = (/ &
     &1.2026e+05_rb,1.0522e+05_rb,9.0197e+04_rb,7.5158e+04_rb,6.0121e+04_rb,4.5095e+04_rb, &
     &3.0062e+04_rb,1.5030e+04_rb,2.3562e-05_rb/)
      kao(:, 2,13,14) = (/ &
     &1.1659e+05_rb,1.0201e+05_rb,8.7436e+04_rb,7.2864e+04_rb,5.8293e+04_rb,4.3717e+04_rb, &
     &2.9144e+04_rb,1.4573e+04_rb,1.6351e-02_rb/)
      kao(:, 3,13,14) = (/ &
     &1.1367e+05_rb,9.9464e+04_rb,8.5255e+04_rb,7.1046e+04_rb,5.6835e+04_rb,4.2627e+04_rb, &
     &2.8420e+04_rb,1.4209e+04_rb,2.9522e-02_rb/)
      kao(:, 4,13,14) = (/ &
     &1.1129e+05_rb,9.7380e+04_rb,8.3472e+04_rb,6.9552e+04_rb,5.5648e+04_rb,4.1732e+04_rb, &
     &2.7823e+04_rb,1.3911e+04_rb,7.5810e-02_rb/)
      kao(:, 5,13,14) = (/ &
     &1.0918e+05_rb,9.5534e+04_rb,8.1880e+04_rb,6.8241e+04_rb,5.4595e+04_rb,4.0946e+04_rb, &
     &2.7297e+04_rb,1.3648e+04_rb,2.4333e-01_rb/)
      kao(:, 1, 1,15) = (/ &
     &1.0978e+01_rb,9.6088e+00_rb,8.2350e+00_rb,6.8632e+00_rb,5.4910e+00_rb,4.1182e+00_rb, &
     &2.7459e+00_rb,1.3741e+00_rb,7.5097e-01_rb/)
      kao(:, 2, 1,15) = (/ &
     &1.0420e+01_rb,9.1192e+00_rb,7.8159e+00_rb,6.5136e+00_rb,5.2114e+00_rb,3.9092e+00_rb, &
     &2.6067e+00_rb,1.3044e+00_rb,1.0341e+00_rb/)
      kao(:, 3, 1,15) = (/ &
     &9.8688e+00_rb,8.6360e+00_rb,7.4026e+00_rb,6.1689e+00_rb,4.9358e+00_rb,3.7018e+00_rb, &
     &2.4689e+00_rb,1.2355e+00_rb,1.3726e+00_rb/)
      kao(:, 4, 1,15) = (/ &
     &9.3623e+00_rb,8.1910e+00_rb,7.0229e+00_rb,5.8526e+00_rb,4.6825e+00_rb,3.5124e+00_rb, &
     &2.3423e+00_rb,1.1838e+00_rb,1.7645e+00_rb/)
      kao(:, 5, 1,15) = (/ &
     &8.8716e+00_rb,7.7621e+00_rb,6.6540e+00_rb,5.5450e+00_rb,4.4367e+00_rb,3.3283e+00_rb, &
     &2.2197e+00_rb,1.4532e+00_rb,2.2082e+00_rb/)
      kao(:, 1, 2,15) = (/ &
     &1.7872e+01_rb,1.5635e+01_rb,1.3402e+01_rb,1.1169e+01_rb,8.9360e+00_rb,6.7019e+00_rb, &
     &4.4692e+00_rb,2.2349e+00_rb,7.8713e-01_rb/)
      kao(:, 2, 2,15) = (/ &
     &1.6936e+01_rb,1.4818e+01_rb,1.2703e+01_rb,1.0586e+01_rb,8.4684e+00_rb,6.3517e+00_rb, &
     &4.2352e+00_rb,2.1186e+00_rb,1.1015e+00_rb/)
      kao(:, 3, 2,15) = (/ &
     &1.6019e+01_rb,1.4019e+01_rb,1.2014e+01_rb,1.0011e+01_rb,8.0107e+00_rb,6.0080e+00_rb, &
     &4.0069e+00_rb,2.0039e+00_rb,1.4812e+00_rb/)
      kao(:, 4, 2,15) = (/ &
     &1.5152e+01_rb,1.3257e+01_rb,1.1364e+01_rb,9.4698e+00_rb,7.5770e+00_rb,5.6829e+00_rb, &
     &3.7895e+00_rb,1.8958e+00_rb,1.9253e+00_rb/)
      kao(:, 5, 2,15) = (/ &
     &1.4341e+01_rb,1.2547e+01_rb,1.0755e+01_rb,8.9625e+00_rb,7.1702e+00_rb,5.3783e+00_rb, &
     &3.5866e+00_rb,1.8251e+00_rb,2.4314e+00_rb/)
      kao(:, 1, 3,15) = (/ &
     &3.9644e+01_rb,3.4688e+01_rb,2.9736e+01_rb,2.4782e+01_rb,1.9827e+01_rb,1.4870e+01_rb, &
     &9.9132e+00_rb,4.9569e+00_rb,7.5098e-01_rb/)
      kao(:, 2, 3,15) = (/ &
     &3.7522e+01_rb,3.2831e+01_rb,2.8147e+01_rb,2.3453e+01_rb,1.8762e+01_rb,1.4073e+01_rb, &
     &9.3827e+00_rb,4.6920e+00_rb,1.0782e+00_rb/)
      kao(:, 3, 3,15) = (/ &
     &3.5460e+01_rb,3.1027e+01_rb,2.6599e+01_rb,2.2162e+01_rb,1.7731e+01_rb,1.3300e+01_rb, &
     &8.8666e+00_rb,4.4344e+00_rb,1.4809e+00_rb/)
      kao(:, 4, 3,15) = (/ &
     &3.3460e+01_rb,2.9273e+01_rb,2.5092e+01_rb,2.0911e+01_rb,1.6733e+01_rb,1.2547e+01_rb, &
     &8.3663e+00_rb,4.1837e+00_rb,1.9598e+00_rb/)
      kao(:, 5, 3,15) = (/ &
     &3.1621e+01_rb,2.7670e+01_rb,2.3718e+01_rb,1.9764e+01_rb,1.5811e+01_rb,1.1859e+01_rb, &
     &7.9074e+00_rb,3.9548e+00_rb,2.5132e+00_rb/)
      kao(:, 1, 4,15) = (/ &
     &9.0592e+01_rb,7.9274e+01_rb,6.7952e+01_rb,5.6623e+01_rb,4.5292e+01_rb,3.3974e+01_rb, &
     &2.2650e+01_rb,1.1327e+01_rb,6.7358e-01_rb/)
      kao(:, 2, 4,15) = (/ &
     &8.5357e+01_rb,7.4682e+01_rb,6.4009e+01_rb,5.3347e+01_rb,4.2676e+01_rb,3.2015e+01_rb, &
     &2.1340e+01_rb,1.0671e+01_rb,9.9693e-01_rb/)
      kao(:, 3, 4,15) = (/ &
     &8.0608e+01_rb,7.0531e+01_rb,6.0449e+01_rb,5.0384e+01_rb,4.0308e+01_rb,3.0230e+01_rb, &
     &2.0154e+01_rb,1.0079e+01_rb,1.4044e+00_rb/)
      kao(:, 4, 4,15) = (/ &
     &7.6021e+01_rb,6.6519e+01_rb,5.7017e+01_rb,4.7513e+01_rb,3.8009e+01_rb,2.8510e+01_rb, &
     &1.9006e+01_rb,9.5047e+00_rb,1.8983e+00_rb/)
      kao(:, 5, 4,15) = (/ &
     &7.1646e+01_rb,6.2695e+01_rb,5.3735e+01_rb,4.4783e+01_rb,3.5822e+01_rb,2.6870e+01_rb, &
     &1.7917e+01_rb,8.9586e+00_rb,2.4776e+00_rb/)
      kao(:, 1, 5,15) = (/ &
     &1.9065e+02_rb,1.6683e+02_rb,1.4299e+02_rb,1.1917e+02_rb,9.5320e+01_rb,7.1489e+01_rb, &
     &4.7665e+01_rb,2.3834e+01_rb,5.8170e-01_rb/)
      kao(:, 2, 5,15) = (/ &
     &1.7924e+02_rb,1.5681e+02_rb,1.3443e+02_rb,1.1201e+02_rb,8.9613e+01_rb,6.7209e+01_rb, &
     &4.4814e+01_rb,2.2406e+01_rb,8.9003e-01_rb/)
      kao(:, 3, 5,15) = (/ &
     &1.6878e+02_rb,1.4769e+02_rb,1.2660e+02_rb,1.0549e+02_rb,8.4396e+01_rb,6.3295e+01_rb, &
     &4.2198e+01_rb,2.1098e+01_rb,1.2889e+00_rb/)
      kao(:, 4, 5,15) = (/ &
     &1.5901e+02_rb,1.3914e+02_rb,1.1927e+02_rb,9.9403e+01_rb,7.9524e+01_rb,5.9641e+01_rb, &
     &3.9759e+01_rb,1.9881e+01_rb,1.7818e+00_rb/)
      kao(:, 5, 5,15) = (/ &
     &1.4967e+02_rb,1.3098e+02_rb,1.1227e+02_rb,9.3560e+01_rb,7.4846e+01_rb,5.6123e+01_rb, &
     &3.7423e+01_rb,1.8714e+01_rb,2.3703e+00_rb/)
      kao(:, 1, 6,15) = (/ &
     &3.7689e+02_rb,3.2969e+02_rb,2.8264e+02_rb,2.3553e+02_rb,1.8841e+02_rb,1.4134e+02_rb, &
     &9.4214e+01_rb,4.7104e+01_rb,4.7634e-01_rb/)
      kao(:, 2, 6,15) = (/ &
     &3.5386e+02_rb,3.0960e+02_rb,2.6540e+02_rb,2.2119e+02_rb,1.7694e+02_rb,1.3270e+02_rb, &
     &8.8456e+01_rb,4.4232e+01_rb,7.5759e-01_rb/)
      kao(:, 3, 6,15) = (/ &
     &3.3202e+02_rb,2.9052e+02_rb,2.4901e+02_rb,2.0752e+02_rb,1.6601e+02_rb,1.2451e+02_rb, &
     &8.3003e+01_rb,4.1506e+01_rb,1.1321e+00_rb/)
      kao(:, 4, 6,15) = (/ &
     &3.1280e+02_rb,2.7368e+02_rb,2.3460e+02_rb,1.9550e+02_rb,1.5640e+02_rb,1.1729e+02_rb, &
     &7.8200e+01_rb,3.9102e+01_rb,1.6067e+00_rb/)
      kao(:, 5, 6,15) = (/ &
     &2.9457e+02_rb,2.5772e+02_rb,2.2093e+02_rb,1.8412e+02_rb,1.4729e+02_rb,1.1048e+02_rb, &
     &7.3641e+01_rb,3.6825e+01_rb,2.1838e+00_rb/)
      kao(:, 1, 7,15) = (/ &
     &7.8539e+02_rb,6.8719e+02_rb,5.8900e+02_rb,4.9081e+02_rb,3.9268e+02_rb,2.9449e+02_rb, &
     &1.9633e+02_rb,9.8167e+01_rb,3.8693e-01_rb/)
      kao(:, 2, 7,15) = (/ &
     &7.3697e+02_rb,6.4481e+02_rb,5.5276e+02_rb,4.6063e+02_rb,3.6848e+02_rb,2.7636e+02_rb, &
     &1.8426e+02_rb,9.2135e+01_rb,6.2265e-01_rb/)
      kao(:, 3, 7,15) = (/ &
     &6.9166e+02_rb,6.0516e+02_rb,5.1875e+02_rb,4.3231e+02_rb,3.4580e+02_rb,2.5935e+02_rb, &
     &1.7293e+02_rb,8.6458e+01_rb,9.6225e-01_rb/)
      kao(:, 4, 7,15) = (/ &
     &6.5012e+02_rb,5.6880e+02_rb,4.8758e+02_rb,4.0634e+02_rb,3.2506e+02_rb,2.4380e+02_rb, &
     &1.6251e+02_rb,8.1260e+01_rb,1.4063e+00_rb/)
      kao(:, 5, 7,15) = (/ &
     &6.1251e+02_rb,5.3595e+02_rb,4.5937e+02_rb,3.8280e+02_rb,3.0627e+02_rb,2.2970e+02_rb, &
     &1.5314e+02_rb,7.6572e+01_rb,1.9665e+00_rb/)
      kao(:, 1, 8,15) = (/ &
     &1.9678e+03_rb,1.7217e+03_rb,1.4759e+03_rb,1.2298e+03_rb,9.8389e+02_rb,7.3793e+02_rb, &
     &4.9200e+02_rb,2.4597e+02_rb,3.3326e-01_rb/)
      kao(:, 2, 8,15) = (/ &
     &1.8454e+03_rb,1.6147e+03_rb,1.3840e+03_rb,1.1533e+03_rb,9.2274e+02_rb,6.9199e+02_rb, &
     &4.6133e+02_rb,2.3066e+02_rb,5.2281e-01_rb/)
      kao(:, 3, 8,15) = (/ &
     &1.7316e+03_rb,1.5151e+03_rb,1.2986e+03_rb,1.0821e+03_rb,8.6576e+02_rb,6.4926e+02_rb, &
     &4.3287e+02_rb,2.1643e+02_rb,8.0537e-01_rb/)
      kao(:, 4, 8,15) = (/ &
     &1.6274e+03_rb,1.4239e+03_rb,1.2204e+03_rb,1.0169e+03_rb,8.1360e+02_rb,6.1025e+02_rb, &
     &4.0681e+02_rb,2.0342e+02_rb,1.2167e+00_rb/)
      kao(:, 5, 8,15) = (/ &
     &1.5364e+03_rb,1.3443e+03_rb,1.1523e+03_rb,9.6013e+02_rb,7.6820e+02_rb,5.7614e+02_rb, &
     &3.8407e+02_rb,1.9205e+02_rb,1.7524e+00_rb/)
      kao(:, 1, 9,15) = (/ &
     &9.3073e+03_rb,8.1544e+03_rb,6.9894e+03_rb,5.8248e+03_rb,4.6604e+03_rb,3.4950e+03_rb, &
     &2.3299e+03_rb,1.1651e+03_rb,2.9954e-01_rb/)
      kao(:, 2, 9,15) = (/ &
     &8.7551e+03_rb,7.6608e+03_rb,6.5662e+03_rb,5.4720e+03_rb,4.3776e+03_rb,3.2831e+03_rb, &
     &2.1886e+03_rb,1.0944e+03_rb,4.5642e-01_rb/)
      kao(:, 3, 9,15) = (/ &
     &8.2482e+03_rb,7.2167e+03_rb,6.1860e+03_rb,5.1552e+03_rb,4.1239e+03_rb,3.0929e+03_rb, &
     &2.0618e+03_rb,1.0309e+03_rb,6.9066e-01_rb/)
      kao(:, 4, 9,15) = (/ &
     &7.8082e+03_rb,6.8324e+03_rb,5.8568e+03_rb,4.8805e+03_rb,3.9039e+03_rb,2.9280e+03_rb, &
     &1.9520e+03_rb,9.7612e+02_rb,1.0400e+00_rb/)
      kao(:, 5, 9,15) = (/ &
     &7.4317e+03_rb,6.5021e+03_rb,5.5734e+03_rb,4.6446e+03_rb,3.7157e+03_rb,2.7869e+03_rb, &
     &1.8578e+03_rb,9.2892e+02_rb,1.5452e+00_rb/)
      kao(:, 1,10,15) = (/ &
     &4.9432e+04_rb,4.3258e+04_rb,3.7074e+04_rb,3.0897e+04_rb,2.4721e+04_rb,1.8537e+04_rb, &
     &1.2358e+04_rb,6.1790e+03_rb,1.8223e-01_rb/)
      kao(:, 2,10,15) = (/ &
     &4.6742e+04_rb,4.0903e+04_rb,3.5058e+04_rb,2.9212e+04_rb,2.3371e+04_rb,1.7528e+04_rb, &
     &1.1686e+04_rb,5.8428e+03_rb,4.2300e-01_rb/)
      kao(:, 3,10,15) = (/ &
     &4.4467e+04_rb,3.8905e+04_rb,3.3347e+04_rb,2.7787e+04_rb,2.2232e+04_rb,1.6675e+04_rb, &
     &1.1116e+04_rb,5.5580e+03_rb,6.2837e-01_rb/)
      kao(:, 4,10,15) = (/ &
     &4.2571e+04_rb,3.7248e+04_rb,3.1929e+04_rb,2.6604e+04_rb,2.1284e+04_rb,1.5965e+04_rb, &
     &1.0643e+04_rb,5.3212e+03_rb,9.2886e-01_rb/)
      kao(:, 5,10,15) = (/ &
     &4.1045e+04_rb,3.5912e+04_rb,3.0783e+04_rb,2.5652e+04_rb,2.0522e+04_rb,1.5392e+04_rb, &
     &1.0261e+04_rb,5.1304e+03_rb,1.3856e+00_rb/)
      kao(:, 1,11,15) = (/ &
     &9.0769e+04_rb,7.9427e+04_rb,6.8076e+04_rb,5.6731e+04_rb,4.5387e+04_rb,3.4039e+04_rb, &
     &2.2692e+04_rb,1.1348e+04_rb,6.3351e-06_rb/)
      kao(:, 2,11,15) = (/ &
     &8.7118e+04_rb,7.6228e+04_rb,6.5339e+04_rb,5.4453e+04_rb,4.3558e+04_rb,3.2668e+04_rb, &
     &2.1780e+04_rb,1.0890e+04_rb,1.2897e-01_rb/)
      kao(:, 3,11,15) = (/ &
     &8.4111e+04_rb,7.3597e+04_rb,6.3086e+04_rb,5.2568e+04_rb,4.2058e+04_rb,3.1544e+04_rb, &
     &2.1027e+04_rb,1.0514e+04_rb,3.6063e-01_rb/)
      kao(:, 4,11,15) = (/ &
     &8.1812e+04_rb,7.1584e+04_rb,6.1356e+04_rb,5.1132e+04_rb,4.0904e+04_rb,3.0678e+04_rb, &
     &2.0453e+04_rb,1.0225e+04_rb,8.1674e-01_rb/)
      kao(:, 5,11,15) = (/ &
     &7.9895e+04_rb,6.9910e+04_rb,5.9923e+04_rb,4.9935e+04_rb,3.9950e+04_rb,2.9961e+04_rb, &
     &1.9975e+04_rb,9.9870e+03_rb,1.4894e+00_rb/)
      kao(:, 1,12,15) = (/ &
     &1.2814e+05_rb,1.1213e+05_rb,9.6104e+04_rb,8.0085e+04_rb,6.4069e+04_rb,4.8051e+04_rb, &
     &3.2036e+04_rb,1.6017e+04_rb,4.6990e-06_rb/)
      kao(:, 2,12,15) = (/ &
     &1.2486e+05_rb,1.0925e+05_rb,9.3637e+04_rb,7.8034e+04_rb,6.2430e+04_rb,4.6820e+04_rb, &
     &3.1210e+04_rb,1.5606e+04_rb,1.4051e-05_rb/)
      kao(:, 3,12,15) = (/ &
     &1.2228e+05_rb,1.0700e+05_rb,9.1710e+04_rb,7.6429e+04_rb,6.1136e+04_rb,4.5856e+04_rb, &
     &3.0569e+04_rb,1.5285e+04_rb,1.5080e-01_rb/)
      kao(:, 4,12,15) = (/ &
     &1.2022e+05_rb,1.0520e+05_rb,9.0163e+04_rb,7.5135e+04_rb,6.0115e+04_rb,4.5084e+04_rb, &
     &3.0055e+04_rb,1.5027e+04_rb,4.4115e-01_rb/)
      kao(:, 5,12,15) = (/ &
     &1.1876e+05_rb,1.0392e+05_rb,8.9071e+04_rb,7.4229e+04_rb,5.9383e+04_rb,4.4534e+04_rb, &
     &2.9691e+04_rb,1.4845e+04_rb,1.2230e+00_rb/)
      kao(:, 1,13,15) = (/ &
     &1.4959e+05_rb,1.3089e+05_rb,1.1219e+05_rb,9.3492e+04_rb,7.4791e+04_rb,5.6100e+04_rb, &
     &3.7396e+04_rb,1.8698e+04_rb,1.6068e-05_rb/)
      kao(:, 2,13,15) = (/ &
     &1.4739e+05_rb,1.2897e+05_rb,1.1053e+05_rb,9.2121e+04_rb,7.3693e+04_rb,5.5270e+04_rb, &
     &3.6845e+04_rb,1.8424e+04_rb,2.4216e-05_rb/)
      kao(:, 3,13,15) = (/ &
     &1.4569e+05_rb,1.2748e+05_rb,1.0927e+05_rb,9.1057e+04_rb,7.2849e+04_rb,5.4630e+04_rb, &
     &3.6423e+04_rb,1.8211e+04_rb,2.3267e-01_rb/)
      kao(:, 4,13,15) = (/ &
     &1.4449e+05_rb,1.2643e+05_rb,1.0837e+05_rb,9.0297e+04_rb,7.2240e+04_rb,5.4184e+04_rb, &
     &3.6121e+04_rb,1.8059e+04_rb,3.2297e-01_rb/)
      kao(:, 5,13,15) = (/ &
     &1.4334e+05_rb,1.2542e+05_rb,1.0750e+05_rb,8.9583e+04_rb,7.1673e+04_rb,5.3754e+04_rb, &
     &3.5834e+04_rb,1.7916e+04_rb,9.5703e-01_rb/)
      kao(:, 1, 1,16) = (/ &
     &1.1012e+01_rb,9.6385e+00_rb,8.2597e+00_rb,6.8841e+00_rb,5.5077e+00_rb,4.1310e+00_rb, &
     &2.7545e+00_rb,1.3780e+00_rb,8.0170e-01_rb/)
      kao(:, 2, 1,16) = (/ &
     &1.0449e+01_rb,9.1438e+00_rb,7.8382e+00_rb,6.5335e+00_rb,5.2262e+00_rb,3.9202e+00_rb, &
     &2.6149e+00_rb,1.3083e+00_rb,1.1089e+00_rb/)
      kao(:, 3, 1,16) = (/ &
     &9.8963e+00_rb,8.6604e+00_rb,7.4233e+00_rb,6.1853e+00_rb,4.9489e+00_rb,3.7122e+00_rb, &
     &2.4757e+00_rb,1.2390e+00_rb,1.4762e+00_rb/)
      kao(:, 4, 1,16) = (/ &
     &9.3879e+00_rb,8.2127e+00_rb,7.0407e+00_rb,5.8670e+00_rb,4.6947e+00_rb,3.5210e+00_rb, &
     &2.3483e+00_rb,1.2086e+00_rb,1.9022e+00_rb/)
      kao(:, 5, 1,16) = (/ &
     &8.8935e+00_rb,7.7813e+00_rb,6.6705e+00_rb,5.5589e+00_rb,4.4480e+00_rb,3.3366e+00_rb, &
     &2.2250e+00_rb,1.5141e+00_rb,2.3892e+00_rb/)
      kao(:, 1, 2,16) = (/ &
     &1.7937e+01_rb,1.5695e+01_rb,1.3452e+01_rb,1.1211e+01_rb,8.9664e+00_rb,6.7267e+00_rb, &
     &4.4861e+00_rb,2.2435e+00_rb,8.6848e-01_rb/)
      kao(:, 2, 2,16) = (/ &
     &1.6997e+01_rb,1.4871e+01_rb,1.2749e+01_rb,1.0624e+01_rb,8.4984e+00_rb,6.3745e+00_rb, &
     &4.2497e+00_rb,2.1262e+00_rb,1.2205e+00_rb/)
      kao(:, 3, 2,16) = (/ &
     &1.6073e+01_rb,1.4067e+01_rb,1.2054e+01_rb,1.0046e+01_rb,8.0383e+00_rb,6.0294e+00_rb, &
     &4.0202e+00_rb,2.0108e+00_rb,1.6477e+00_rb/)
      kao(:, 4, 2,16) = (/ &
     &1.5204e+01_rb,1.3304e+01_rb,1.1402e+01_rb,9.4999e+00_rb,7.6031e+00_rb,5.7017e+00_rb, &
     &3.8022e+00_rb,1.9021e+00_rb,2.1539e+00_rb/)
      kao(:, 5, 2,16) = (/ &
     &1.4385e+01_rb,1.2589e+01_rb,1.0791e+01_rb,8.9907e+00_rb,7.1928e+00_rb,5.3962e+00_rb, &
     &3.5994e+00_rb,1.9006e+00_rb,2.7317e+00_rb/)
      kao(:, 1, 3,16) = (/ &
     &3.9826e+01_rb,3.4837e+01_rb,2.9874e+01_rb,2.4893e+01_rb,1.9916e+01_rb,1.4937e+01_rb, &
     &9.9605e+00_rb,4.9801e+00_rb,8.6605e-01_rb/)
      kao(:, 2, 3,16) = (/ &
     &3.7687e+01_rb,3.2981e+01_rb,2.8272e+01_rb,2.3555e+01_rb,1.8843e+01_rb,1.4136e+01_rb, &
     &9.4254e+00_rb,4.7126e+00_rb,1.2506e+00_rb/)
      kao(:, 3, 3,16) = (/ &
     &3.5608e+01_rb,3.1162e+01_rb,2.6711e+01_rb,2.2256e+01_rb,1.7812e+01_rb,1.3358e+01_rb, &
     &8.9032e+00_rb,4.4535e+00_rb,1.7305e+00_rb/)
      kao(:, 4, 3,16) = (/ &
     &3.3599e+01_rb,2.9402e+01_rb,2.5206e+01_rb,2.1000e+01_rb,1.6804e+01_rb,1.2601e+01_rb, &
     &8.4019e+00_rb,4.2020e+00_rb,2.3040e+00_rb/)
      kao(:, 5, 3,16) = (/ &
     &3.1757e+01_rb,2.7783e+01_rb,2.3809e+01_rb,1.9848e+01_rb,1.5875e+01_rb,1.1908e+01_rb, &
     &7.9407e+00_rb,3.9713e+00_rb,2.9736e+00_rb/)
      kao(:, 1, 4,16) = (/ &
     &9.1010e+01_rb,7.9617e+01_rb,6.8261e+01_rb,5.6883e+01_rb,4.5501e+01_rb,3.4135e+01_rb, &
     &2.2756e+01_rb,1.1381e+01_rb,8.2098e-01_rb/)
      kao(:, 2, 4,16) = (/ &
     &8.5710e+01_rb,7.5003e+01_rb,6.4275e+01_rb,5.3587e+01_rb,4.2849e+01_rb,3.2139e+01_rb, &
     &2.1425e+01_rb,1.0717e+01_rb,1.2260e+00_rb/)
      kao(:, 3, 4,16) = (/ &
     &8.0903e+01_rb,7.0806e+01_rb,6.0673e+01_rb,5.0561e+01_rb,4.0455e+01_rb,3.0343e+01_rb, &
     &2.0221e+01_rb,1.0019e+01_rb,1.7427e+00_rb/)
      kao(:, 4, 4,16) = (/ &
     &7.6250e+01_rb,6.6729e+01_rb,5.7190e+01_rb,4.7658e+01_rb,3.8119e+01_rb,2.8596e+01_rb, &
     &1.9064e+01_rb,9.5357e+00_rb,2.3756e+00_rb/)
      kao(:, 5, 4,16) = (/ &
     &7.1845e+01_rb,6.2892e+01_rb,5.3881e+01_rb,4.4904e+01_rb,3.5931e+01_rb,2.6946e+01_rb, &
     &1.7968e+01_rb,8.9829e+00_rb,3.1270e+00_rb/)
      kao(:, 1, 5,16) = (/ &
     &1.9171e+02_rb,1.6775e+02_rb,1.4380e+02_rb,1.1981e+02_rb,9.5875e+01_rb,7.1871e+01_rb, &
     &4.7928e+01_rb,2.3964e+01_rb,7.5940e-01_rb/)
      kao(:, 2, 5,16) = (/ &
     &1.8016e+02_rb,1.5760e+02_rb,1.3509e+02_rb,1.1260e+02_rb,9.0071e+01_rb,6.7551e+01_rb, &
     &4.5046e+01_rb,2.2517e+01_rb,1.1757e+00_rb/)
      kao(:, 3, 5,16) = (/ &
     &1.6962e+02_rb,1.4839e+02_rb,1.2718e+02_rb,1.0601e+02_rb,8.4813e+01_rb,6.3608e+01_rb, &
     &4.2403e+01_rb,2.1198e+01_rb,1.7223e+00_rb/)
      kao(:, 4, 5,16) = (/ &
     &1.5982e+02_rb,1.3980e+02_rb,1.1984e+02_rb,9.9881e+01_rb,7.9889e+01_rb,5.9924e+01_rb, &
     &3.9950e+01_rb,1.9975e+01_rb,2.4076e+00_rb/)
      kao(:, 5, 5,16) = (/ &
     &1.5041e+02_rb,1.3158e+02_rb,1.1281e+02_rb,9.3993e+01_rb,7.5196e+01_rb,5.6401e+01_rb, &
     &3.7598e+01_rb,1.8805e+01_rb,3.2368e+00_rb/)
      kao(:, 1, 6,16) = (/ &
     &3.8096e+02_rb,3.3337e+02_rb,2.8576e+02_rb,2.3807e+02_rb,1.9047e+02_rb,1.4289e+02_rb, &
     &9.5250e+01_rb,4.7621e+01_rb,6.7593e-01_rb/)
      kao(:, 2, 6,16) = (/ &
     &3.5801e+02_rb,3.1325e+02_rb,2.6851e+02_rb,2.2376e+02_rb,1.7905e+02_rb,1.3426e+02_rb, &
     &8.9499e+01_rb,4.4742e+01_rb,1.0913e+00_rb/)
      kao(:, 3, 6,16) = (/ &
     &3.3608e+02_rb,2.9405e+02_rb,2.5208e+02_rb,2.1010e+02_rb,1.6803e+02_rb,1.2602e+02_rb, &
     &8.4040e+01_rb,4.2005e+01_rb,1.6542e+00_rb/)
      kao(:, 4, 6,16) = (/ &
     &3.1667e+02_rb,2.7704e+02_rb,2.3752e+02_rb,1.9794e+02_rb,1.5835e+02_rb,1.1877e+02_rb, &
     &7.9157e+01_rb,3.9584e+01_rb,2.3801e+00_rb/)
      kao(:, 5, 6,16) = (/ &
     &2.9810e+02_rb,2.6093e+02_rb,2.2364e+02_rb,1.8637e+02_rb,1.4910e+02_rb,1.1183e+02_rb, &
     &7.4550e+01_rb,3.7278e+01_rb,3.2789e+00_rb/)
      kao(:, 1, 7,16) = (/ &
     &7.9749e+02_rb,6.9796e+02_rb,5.9825e+02_rb,4.9855e+02_rb,3.9886e+02_rb,2.9907e+02_rb, &
     &1.9939e+02_rb,9.9689e+01_rb,5.8719e-01_rb/)
      kao(:, 2, 7,16) = (/ &
     &7.4948e+02_rb,6.5592e+02_rb,5.6211e+02_rb,4.6855e+02_rb,3.7478e+02_rb,2.8116e+02_rb, &
     &1.8738e+02_rb,9.3692e+01_rb,9.9296e-01_rb/)
      kao(:, 3, 7,16) = (/ &
     &7.0403e+02_rb,6.1598e+02_rb,5.2804e+02_rb,4.4008e+02_rb,3.5206e+02_rb,2.6406e+02_rb, &
     &1.7605e+02_rb,8.8000e+01_rb,1.5629e+00_rb/)
      kao(:, 4, 7,16) = (/ &
     &6.6205e+02_rb,5.7919e+02_rb,4.9648e+02_rb,4.1378e+02_rb,3.3107e+02_rb,2.4827e+02_rb, &
     &1.6552e+02_rb,8.2766e+01_rb,2.3202e+00_rb/)
      kao(:, 5, 7,16) = (/ &
     &6.2380e+02_rb,5.4580e+02_rb,4.6792e+02_rb,3.8985e+02_rb,3.1191e+02_rb,2.3394e+02_rb, &
     &1.5598e+02_rb,7.7978e+01_rb,3.2803e+00_rb/)
      kao(:, 1, 8,16) = (/ &
     &2.0029e+03_rb,1.7526e+03_rb,1.5027e+03_rb,1.2518e+03_rb,1.0017e+03_rb,7.5137e+02_rb, &
     &5.0076e+02_rb,2.5041e+02_rb,5.0039e-01_rb/)
      kao(:, 2, 8,16) = (/ &
     &1.8839e+03_rb,1.6487e+03_rb,1.4132e+03_rb,1.1775e+03_rb,9.4211e+02_rb,7.0656e+02_rb, &
     &4.7103e+02_rb,2.3550e+02_rb,8.8968e-01_rb/)
      kao(:, 3, 8,16) = (/ &
     &1.7724e+03_rb,1.5508e+03_rb,1.3295e+03_rb,1.1080e+03_rb,8.8638e+02_rb,6.6461e+02_rb, &
     &4.4304e+02_rb,2.2162e+02_rb,1.4586e+00_rb/)
      kao(:, 4, 8,16) = (/ &
     &1.6681e+03_rb,1.4594e+03_rb,1.2510e+03_rb,1.0422e+03_rb,8.3394e+02_rb,6.2548e+02_rb, &
     &4.1698e+02_rb,2.0850e+02_rb,2.2386e+00_rb/)
      kao(:, 5, 8,16) = (/ &
     &1.5734e+03_rb,1.3768e+03_rb,1.1803e+03_rb,9.8337e+02_rb,7.8678e+02_rb,5.9008e+02_rb, &
     &3.9340e+02_rb,1.9668e+02_rb,3.2542e+00_rb/)
      kao(:, 1, 9,16) = (/ &
     &9.5422e+03_rb,8.3482e+03_rb,7.1572e+03_rb,5.9648e+03_rb,4.7724e+03_rb,3.5785e+03_rb, &
     &2.3856e+03_rb,1.1929e+03_rb,4.1531e-01_rb/)
      kao(:, 2, 9,16) = (/ &
     &8.9979e+03_rb,7.8727e+03_rb,6.7481e+03_rb,5.6240e+03_rb,4.4988e+03_rb,3.3743e+03_rb, &
     &2.2493e+03_rb,1.1247e+03_rb,7.8014e-01_rb/)
      kao(:, 3, 9,16) = (/ &
     &8.4879e+03_rb,7.4273e+03_rb,6.3662e+03_rb,5.3047e+03_rb,4.2442e+03_rb,3.1830e+03_rb, &
     &2.1217e+03_rb,1.0610e+03_rb,1.3368e+00_rb/)
      kao(:, 4, 9,16) = (/ &
     &8.0145e+03_rb,7.0116e+03_rb,6.0099e+03_rb,5.0094e+03_rb,4.0072e+03_rb,3.0052e+03_rb, &
     &2.0036e+03_rb,1.0018e+03_rb,2.1276e+00_rb/)
      kao(:, 5, 9,16) = (/ &
     &7.6138e+03_rb,6.6618e+03_rb,5.7100e+03_rb,4.7582e+03_rb,3.8067e+03_rb,2.8551e+03_rb, &
     &1.9031e+03_rb,9.5177e+02_rb,3.1866e+00_rb/)
      kao(:, 1,10,16) = (/ &
     &5.0991e+04_rb,4.4616e+04_rb,3.8235e+04_rb,3.1865e+04_rb,2.5493e+04_rb,1.9119e+04_rb, &
     &1.2748e+04_rb,6.3734e+03_rb,3.5386e-01_rb/)
      kao(:, 2,10,16) = (/ &
     &4.8246e+04_rb,4.2216e+04_rb,3.6189e+04_rb,3.0155e+04_rb,2.4122e+04_rb,1.8092e+04_rb, &
     &1.2062e+04_rb,6.0316e+03_rb,7.0256e-01_rb/)
      kao(:, 3,10,16) = (/ &
     &4.5828e+04_rb,4.0101e+04_rb,3.4373e+04_rb,2.8642e+04_rb,2.2916e+04_rb,1.7187e+04_rb, &
     &1.1458e+04_rb,5.7294e+03_rb,1.2547e+00_rb/)
      kao(:, 4,10,16) = (/ &
     &4.3978e+04_rb,3.8481e+04_rb,3.2983e+04_rb,2.7486e+04_rb,2.1988e+04_rb,1.6492e+04_rb, &
     &1.0995e+04_rb,5.4969e+03_rb,2.0651e+00_rb/)
      kao(:, 5,10,16) = (/ &
     &4.2519e+04_rb,3.7205e+04_rb,3.1889e+04_rb,2.6577e+04_rb,2.1264e+04_rb,1.5948e+04_rb, &
     &1.0631e+04_rb,5.3154e+03_rb,3.1779e+00_rb/)
      kao(:, 1,11,16) = (/ &
     &9.4155e+04_rb,8.2392e+04_rb,7.0618e+04_rb,5.8856e+04_rb,4.7081e+04_rb,3.5308e+04_rb, &
     &2.3542e+04_rb,1.1770e+04_rb,6.1346e-06_rb/)
      kao(:, 2,11,16) = (/ &
     &9.0590e+04_rb,7.9258e+04_rb,6.7935e+04_rb,5.6620e+04_rb,4.5292e+04_rb,3.3967e+04_rb, &
     &2.2646e+04_rb,1.1323e+04_rb,3.3281e-01_rb/)
      kao(:, 3,11,16) = (/ &
     &8.8119e+04_rb,7.7104e+04_rb,6.6094e+04_rb,5.5078e+04_rb,4.4061e+04_rb,3.3053e+04_rb, &
     &2.2031e+04_rb,1.1015e+04_rb,1.5023e+00_rb/)
      kao(:, 4,11,16) = (/ &
     &8.6175e+04_rb,7.5401e+04_rb,6.4627e+04_rb,5.3860e+04_rb,4.3092e+04_rb,3.2317e+04_rb, &
     &2.1545e+04_rb,1.0771e+04_rb,2.4696e+00_rb/)
      kao(:, 5,11,16) = (/ &
     &8.4570e+04_rb,7.4002e+04_rb,6.3432e+04_rb,5.2859e+04_rb,4.2288e+04_rb,3.1714e+04_rb, &
     &2.1142e+04_rb,1.0572e+04_rb,3.7979e+00_rb/)
      kao(:, 1,12,16) = (/ &
     &1.3547e+05_rb,1.1853e+05_rb,1.0160e+05_rb,8.4660e+04_rb,6.7729e+04_rb,5.0794e+04_rb, &
     &3.3867e+04_rb,1.6929e+04_rb,5.0265e-06_rb/)
      kao(:, 2,12,16) = (/ &
     &1.3284e+05_rb,1.1623e+05_rb,9.9623e+04_rb,8.3023e+04_rb,6.6416e+04_rb,4.9810e+04_rb, &
     &3.3206e+04_rb,1.6603e+04_rb,9.0705e-06_rb/)
      kao(:, 3,12,16) = (/ &
     &1.3111e+05_rb,1.1471e+05_rb,9.8324e+04_rb,8.1946e+04_rb,6.5544e+04_rb,4.9165e+04_rb, &
     &3.2777e+04_rb,1.6388e+04_rb,1.7901e+00_rb/)
      kao(:, 4,12,16) = (/ &
     &1.2947e+05_rb,1.1329e+05_rb,9.7103e+04_rb,8.0920e+04_rb,6.4734e+04_rb,4.8090e+04_rb, &
     &3.2370e+04_rb,1.6184e+04_rb,2.9393e+00_rb/)
      kao(:, 5,12,16) = (/ &
     &1.2773e+05_rb,1.1176e+05_rb,9.5799e+04_rb,7.9830e+04_rb,6.3866e+04_rb,4.7901e+04_rb, &
     &3.1932e+04_rb,1.5966e+04_rb,4.5138e+00_rb/)
      kao(:, 1,13,16) = (/ &
     &1.6206e+05_rb,1.4180e+05_rb,1.2155e+05_rb,1.0128e+05_rb,8.1025e+04_rb,6.0775e+04_rb, &
     &4.0514e+04_rb,2.0257e+04_rb,4.1164e-06_rb/)
      kao(:, 2,13,16) = (/ &
     &1.6103e+05_rb,1.4091e+05_rb,1.2078e+05_rb,1.0065e+05_rb,8.0525e+04_rb,6.0389e+04_rb, &
     &4.0260e+04_rb,2.0131e+04_rb,7.4285e-06_rb/)
      kao(:, 3,13,16) = (/ &
     &1.6003e+05_rb,1.4003e+05_rb,1.2001e+05_rb,1.0001e+05_rb,8.0009e+04_rb,6.0009e+04_rb, &
     &4.0005e+04_rb,2.0004e+04_rb,8.7132e-01_rb/)
      kao(:, 4,13,16) = (/ &
     &1.5863e+05_rb,1.3882e+05_rb,1.1899e+05_rb,9.9157e+04_rb,7.9319e+04_rb,5.9492e+04_rb, &
     &3.9661e+04_rb,1.9830e+04_rb,3.4650e+00_rb/)
      kao(:, 5,13,16) = (/ &
     &1.5756e+05_rb,1.3786e+05_rb,1.1816e+05_rb,9.8476e+04_rb,7.8780e+04_rb,5.9085e+04_rb, &
     &3.9388e+04_rb,1.9695e+04_rb,5.3123e+00_rb/) 

!     The array KBO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels  < ~100mb, temperatures, and ratios
!     of H2O to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index, JT, which
!     runs from 1 to 5, corresponds to different temperatures.  More 
!     specifically, JT = 3 means that the data are for the corresponding 
!     reference temperature TREF for this  pressure level, JT = 2 refers 
!     to the TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and
!     JT = 5 is for TREF+30.  The third index, JP, runs from 13 to 59 and
!     refers to the corresponding pressure level in PREF (e.g. JP = 13 is
!     for a pressure of 95.5835 mb).  The fourth index, IG, goes from 1 to
!     16, and tells us which g-interval the absorption coefficients are for.

      kbo(:, 1,13, 1) = (/ &
     &1.9283e+01_rb,1.4533e+01_rb,9.7765e+00_rb,4.9986e+00_rb,1.2562e-02_rb/)
      kbo(:, 2,13, 1) = (/ &
     &2.3187e+01_rb,1.7461e+01_rb,1.1734e+01_rb,5.9860e+00_rb,1.4622e-02_rb/)
      kbo(:, 3,13, 1) = (/ &
     &2.7169e+01_rb,2.0447e+01_rb,1.3724e+01_rb,6.9897e+00_rb,1.6755e-02_rb/)
      kbo(:, 4,13, 1) = (/ &
     &3.1114e+01_rb,2.3405e+01_rb,1.5695e+01_rb,7.9803e+00_rb,1.8932e-02_rb/)
      kbo(:, 5,13, 1) = (/ &
     &3.5056e+01_rb,2.6360e+01_rb,1.7664e+01_rb,8.9659e+00_rb,2.1071e-02_rb/)
      kbo(:, 1,14, 1) = (/ &
     &1.0180e+01_rb,7.7052e+00_rb,5.2175e+00_rb,2.7059e+00_rb,1.0725e-02_rb/)
      kbo(:, 2,14, 1) = (/ &
     &1.2216e+01_rb,9.2332e+00_rb,6.2420e+00_rb,3.2235e+00_rb,1.2472e-02_rb/)
      kbo(:, 3,14, 1) = (/ &
     &1.4258e+01_rb,1.0764e+01_rb,7.2673e+00_rb,3.7434e+00_rb,1.4271e-02_rb/)
      kbo(:, 4,14, 1) = (/ &
     &1.6297e+01_rb,1.2292e+01_rb,8.2867e+00_rb,4.2597e+00_rb,1.6035e-02_rb/)
      kbo(:, 5,14, 1) = (/ &
     &1.8337e+01_rb,1.3822e+01_rb,9.3061e+00_rb,4.7742e+00_rb,1.7777e-02_rb/)
      kbo(:, 1,15, 1) = (/ &
     &5.4843e+00_rb,4.1814e+00_rb,2.8583e+00_rb,1.5122e+00_rb,9.1712e-03_rb/)
      kbo(:, 2,15, 1) = (/ &
     &6.5509e+00_rb,4.9839e+00_rb,3.3981e+00_rb,1.7878e+00_rb,1.0654e-02_rb/)
      kbo(:, 3,15, 1) = (/ &
     &7.6172e+00_rb,5.7836e+00_rb,3.9371e+00_rb,2.0620e+00_rb,1.2109e-02_rb/)
      kbo(:, 4,15, 1) = (/ &
     &8.6919e+00_rb,6.5886e+00_rb,4.4776e+00_rb,2.3372e+00_rb,1.3585e-02_rb/)
      kbo(:, 5,15, 1) = (/ &
     &9.7770e+00_rb,7.4027e+00_rb,5.0231e+00_rb,2.6136e+00_rb,1.5008e-02_rb/)
      kbo(:, 1,16, 1) = (/ &
     &3.5143e+00_rb,2.7004e+00_rb,1.8632e+00_rb,1.0005e+00_rb,7.8549e-03_rb/)
      kbo(:, 2,16, 1) = (/ &
     &4.1782e+00_rb,3.2010e+00_rb,2.2008e+00_rb,1.1754e+00_rb,9.0596e-03_rb/)
      kbo(:, 3,16, 1) = (/ &
     &4.8461e+00_rb,3.7036e+00_rb,2.5395e+00_rb,1.3499e+00_rb,1.0291e-02_rb/)
      kbo(:, 4,16, 1) = (/ &
     &5.5241e+00_rb,4.2125e+00_rb,2.8833e+00_rb,1.5266e+00_rb,1.1500e-02_rb/)
      kbo(:, 5,16, 1) = (/ &
     &6.2095e+00_rb,4.7270e+00_rb,3.2297e+00_rb,1.7035e+00_rb,1.2669e-02_rb/)
      kbo(:, 1,17, 1) = (/ &
     &2.3317e+00_rb,1.8082e+00_rb,1.2606e+00_rb,6.8451e-01_rb,6.6638e-03_rb/)
      kbo(:, 2,17, 1) = (/ &
     &2.7625e+00_rb,2.1339e+00_rb,1.4816e+00_rb,7.9994e-01_rb,7.6839e-03_rb/)
      kbo(:, 3,17, 1) = (/ &
     &3.1980e+00_rb,2.4632e+00_rb,1.7043e+00_rb,9.1671e-01_rb,8.7134e-03_rb/)
      kbo(:, 4,17, 1) = (/ &
     &3.6437e+00_rb,2.7995e+00_rb,1.9319e+00_rb,1.0349e+00_rb,9.7011e-03_rb/)
      kbo(:, 5,17, 1) = (/ &
     &4.0944e+00_rb,3.1388e+00_rb,2.1608e+00_rb,1.1535e+00_rb,1.0621e-02_rb/)
      kbo(:, 1,18, 1) = (/ &
     &1.5862e+00_rb,1.2436e+00_rb,8.7489e-01_rb,4.7986e-01_rb,5.6356e-03_rb/)
      kbo(:, 2,18, 1) = (/ &
     &1.8744e+00_rb,1.4623e+00_rb,1.0242e+00_rb,5.5874e-01_rb,6.4918e-03_rb/)
      kbo(:, 3,18, 1) = (/ &
     &2.1671e+00_rb,1.6845e+00_rb,1.1763e+00_rb,6.3905e-01_rb,7.3340e-03_rb/)
      kbo(:, 4,18, 1) = (/ &
     &2.4666e+00_rb,1.9117e+00_rb,1.3311e+00_rb,7.2008e-01_rb,8.1312e-03_rb/)
      kbo(:, 5,18, 1) = (/ &
     &2.7713e+00_rb,2.1418e+00_rb,1.4869e+00_rb,8.0122e-01_rb,8.8525e-03_rb/)
      kbo(:, 1,19, 1) = (/ &
     &1.0610e+00_rb,8.4360e-01_rb,5.9805e-01_rb,3.3172e-01_rb,4.7490e-03_rb/)
      kbo(:, 2,19, 1) = (/ &
     &1.2512e+00_rb,9.8882e-01_rb,6.9811e-01_rb,3.8521e-01_rb,5.4538e-03_rb/)
      kbo(:, 3,19, 1) = (/ &
     &1.4451e+00_rb,1.1370e+00_rb,8.0056e-01_rb,4.3959e-01_rb,6.1291e-03_rb/)
      kbo(:, 4,19, 1) = (/ &
     &1.6437e+00_rb,1.2883e+00_rb,9.0437e-01_rb,4.9444e-01_rb,6.7635e-03_rb/)
      kbo(:, 5,19, 1) = (/ &
     &1.8467e+00_rb,1.4425e+00_rb,1.0099e+00_rb,5.4929e-01_rb,7.3353e-03_rb/)
      kbo(:, 1,20, 1) = (/ &
     &7.7133e-01_rb,6.2032e-01_rb,4.4197e-01_rb,2.4707e-01_rb,4.0032e-03_rb/)
      kbo(:, 2,20, 1) = (/ &
     &9.0743e-01_rb,7.2503e-01_rb,5.1466e-01_rb,2.8622e-01_rb,4.5715e-03_rb/)
      kbo(:, 3,20, 1) = (/ &
     &1.0470e+00_rb,8.3267e-01_rb,5.8918e-01_rb,3.2603e-01_rb,5.1125e-03_rb/)
      kbo(:, 4,20, 1) = (/ &
     &1.1900e+00_rb,9.4226e-01_rb,6.6481e-01_rb,3.6603e-01_rb,5.6145e-03_rb/)
      kbo(:, 5,20, 1) = (/ &
     &1.3362e+00_rb,1.0539e+00_rb,7.4130e-01_rb,4.0582e-01_rb,6.0699e-03_rb/)
      kbo(:, 1,21, 1) = (/ &
     &5.7866e-01_rb,4.6986e-01_rb,3.3621e-01_rb,1.8917e-01_rb,3.3621e-03_rb/)
      kbo(:, 2,21, 1) = (/ &
     &6.7950e-01_rb,5.4811e-01_rb,3.9072e-01_rb,2.1877e-01_rb,3.8183e-03_rb/)
      kbo(:, 3,21, 1) = (/ &
     &7.8326e-01_rb,6.2869e-01_rb,4.4658e-01_rb,2.4873e-01_rb,4.2512e-03_rb/)
      kbo(:, 4,21, 1) = (/ &
     &8.8933e-01_rb,7.1063e-01_rb,5.0334e-01_rb,2.7872e-01_rb,4.6532e-03_rb/)
      kbo(:, 5,21, 1) = (/ &
     &9.9790e-01_rb,7.9397e-01_rb,5.6013e-01_rb,3.0841e-01_rb,5.0189e-03_rb/)
      kbo(:, 1,22, 1) = (/ &
     &4.5035e-01_rb,3.6838e-01_rb,2.6449e-01_rb,1.4961e-01_rb,2.8371e-03_rb/)
      kbo(:, 2,22, 1) = (/ &
     &5.2710e-01_rb,4.2854e-01_rb,3.0641e-01_rb,1.7248e-01_rb,3.2033e-03_rb/)
      kbo(:, 3,22, 1) = (/ &
     &6.0621e-01_rb,4.9027e-01_rb,3.4931e-01_rb,1.9546e-01_rb,3.5507e-03_rb/)
      kbo(:, 4,22, 1) = (/ &
     &6.8687e-01_rb,5.5302e-01_rb,3.9268e-01_rb,2.1836e-01_rb,3.8721e-03_rb/)
      kbo(:, 5,22, 1) = (/ &
     &7.6986e-01_rb,6.1676e-01_rb,4.3619e-01_rb,2.4105e-01_rb,4.1644e-03_rb/)
      kbo(:, 1,23, 1) = (/ &
     &3.5288e-01_rb,2.9059e-01_rb,2.0929e-01_rb,1.1896e-01_rb,2.3878e-03_rb/)
      kbo(:, 2,23, 1) = (/ &
     &4.1165e-01_rb,3.3703e-01_rb,2.4169e-01_rb,1.3668e-01_rb,2.6830e-03_rb/)
      kbo(:, 3,23, 1) = (/ &
     &4.7210e-01_rb,3.8447e-01_rb,2.7478e-01_rb,1.5435e-01_rb,2.9616e-03_rb/)
      kbo(:, 4,23, 1) = (/ &
     &5.3412e-01_rb,4.3294e-01_rb,3.0818e-01_rb,1.7203e-01_rb,3.2196e-03_rb/)
      kbo(:, 5,23, 1) = (/ &
     &5.9758e-01_rb,4.8162e-01_rb,3.4147e-01_rb,1.8925e-01_rb,3.4517e-03_rb/)
      kbo(:, 1,24, 1) = (/ &
     &2.7554e-01_rb,2.2847e-01_rb,1.6509e-01_rb,9.4305e-02_rb,2.0058e-03_rb/)
      kbo(:, 2,24, 1) = (/ &
     &3.2038e-01_rb,2.6411e-01_rb,1.9003e-01_rb,1.0793e-01_rb,2.2440e-03_rb/)
      kbo(:, 3,24, 1) = (/ &
     &3.6644e-01_rb,3.0049e-01_rb,2.1543e-01_rb,1.2148e-01_rb,2.4688e-03_rb/)
      kbo(:, 4,24, 1) = (/ &
     &4.1394e-01_rb,3.3761e-01_rb,2.4100e-01_rb,1.3500e-01_rb,2.6754e-03_rb/)
      kbo(:, 5,24, 1) = (/ &
     &4.6205e-01_rb,3.7459e-01_rb,2.6626e-01_rb,1.4798e-01_rb,2.8597e-03_rb/)
      kbo(:, 1,25, 1) = (/ &
     &2.1831e-01_rb,1.8211e-01_rb,1.3197e-01_rb,7.5672e-02_rb,1.6848e-03_rb/)
      kbo(:, 2,25, 1) = (/ &
     &2.5300e-01_rb,2.0977e-01_rb,1.5139e-01_rb,8.6229e-02_rb,1.8770e-03_rb/)
      kbo(:, 3,25, 1) = (/ &
     &2.8861e-01_rb,2.3809e-01_rb,1.7109e-01_rb,9.6778e-02_rb,2.0567e-03_rb/)
      kbo(:, 4,25, 1) = (/ &
     &3.2559e-01_rb,2.6686e-01_rb,1.9096e-01_rb,1.0719e-01_rb,2.2230e-03_rb/)
      kbo(:, 5,25, 1) = (/ &
     &3.6217e-01_rb,2.9511e-01_rb,2.1013e-01_rb,1.1707e-01_rb,2.3678e-03_rb/)
      kbo(:, 1,26, 1) = (/ &
     &1.7639e-01_rb,1.4786e-01_rb,1.0735e-01_rb,6.1680e-02_rb,1.4167e-03_rb/)
      kbo(:, 2,26, 1) = (/ &
     &2.0373e-01_rb,1.6972e-01_rb,1.2271e-01_rb,7.0008e-02_rb,1.5712e-03_rb/)
      kbo(:, 3,26, 1) = (/ &
     &2.3189e-01_rb,1.9218e-01_rb,1.3830e-01_rb,7.8351e-02_rb,1.7155e-03_rb/)
      kbo(:, 4,26, 1) = (/ &
     &2.6096e-01_rb,2.1478e-01_rb,1.5388e-01_rb,8.6437e-02_rb,1.8482e-03_rb/)
      kbo(:, 5,26, 1) = (/ &
     &2.8921e-01_rb,2.3658e-01_rb,1.6864e-01_rb,9.4016e-02_rb,1.9628e-03_rb/)
      kbo(:, 1,27, 1) = (/ &
     &1.4816e-01_rb,1.2446e-01_rb,9.0381e-02_rb,5.1884e-02_rb,1.1905e-03_rb/)
      kbo(:, 2,27, 1) = (/ &
     &1.7052e-01_rb,1.4243e-01_rb,1.0297e-01_rb,5.8708e-02_rb,1.3153e-03_rb/)
      kbo(:, 3,27, 1) = (/ &
     &1.9369e-01_rb,1.6085e-01_rb,1.1575e-01_rb,6.5493e-02_rb,1.4309e-03_rb/)
      kbo(:, 4,27, 1) = (/ &
     &2.1729e-01_rb,1.7923e-01_rb,1.2834e-01_rb,7.1951e-02_rb,1.5370e-03_rb/)
      kbo(:, 5,27, 1) = (/ &
     &2.3994e-01_rb,1.9663e-01_rb,1.4015e-01_rb,7.7968e-02_rb,1.6265e-03_rb/)
      kbo(:, 1,28, 1) = (/ &
     &1.2786e-01_rb,1.0743e-01_rb,7.7947e-02_rb,4.4624e-02_rb,1.0001e-03_rb/)
      kbo(:, 2,28, 1) = (/ &
     &1.4668e-01_rb,1.2262e-01_rb,8.8533e-02_rb,5.0350e-02_rb,1.1007e-03_rb/)
      kbo(:, 3,28, 1) = (/ &
     &1.6624e-01_rb,1.3809e-01_rb,9.9259e-02_rb,5.5949e-02_rb,1.1936e-03_rb/)
      kbo(:, 4,28, 1) = (/ &
     &1.8582e-01_rb,1.5336e-01_rb,1.0964e-01_rb,6.1251e-02_rb,1.2774e-03_rb/)
      kbo(:, 5,28, 1) = (/ &
     &2.0447e-01_rb,1.6760e-01_rb,1.1929e-01_rb,6.6127e-02_rb,1.3481e-03_rb/)
      kbo(:, 1,29, 1) = (/ &
     &1.1723e-01_rb,9.8203e-02_rb,7.1006e-02_rb,4.0413e-02_rb,8.4082e-04_rb/)
      kbo(:, 2,29, 1) = (/ &
     &1.3410e-01_rb,1.1181e-01_rb,8.0444e-02_rb,4.5462e-02_rb,9.2201e-04_rb/)
      kbo(:, 3,29, 1) = (/ &
     &1.5159e-01_rb,1.2556e-01_rb,8.9943e-02_rb,5.0342e-02_rb,9.9652e-04_rb/)
      kbo(:, 4,29, 1) = (/ &
     &1.6878e-01_rb,1.3892e-01_rb,9.8991e-02_rb,5.4945e-02_rb,1.0624e-03_rb/)
      kbo(:, 5,29, 1) = (/ &
     &1.8506e-01_rb,1.5133e-01_rb,1.0733e-01_rb,5.9130e-02_rb,1.1186e-03_rb/)
      kbo(:, 1,30, 1) = (/ &
     &1.1017e-01_rb,9.1915e-02_rb,6.6158e-02_rb,3.7385e-02_rb,7.0716e-04_rb/)
      kbo(:, 2,30, 1) = (/ &
     &1.2571e-01_rb,1.0438e-01_rb,7.4791e-02_rb,4.1937e-02_rb,7.7222e-04_rb/)
      kbo(:, 3,30, 1) = (/ &
     &1.4165e-01_rb,1.1688e-01_rb,8.3361e-02_rb,4.6295e-02_rb,8.3205e-04_rb/)
      kbo(:, 4,30, 1) = (/ &
     &1.5709e-01_rb,1.2882e-01_rb,9.1412e-02_rb,5.0358e-02_rb,8.8422e-04_rb/)
      kbo(:, 5,30, 1) = (/ &
     &1.7161e-01_rb,1.3989e-01_rb,9.8798e-02_rb,5.4060e-02_rb,9.2829e-04_rb/)
      kbo(:, 1,31, 1) = (/ &
     &1.0894e-01_rb,9.0338e-02_rb,6.4636e-02_rb,3.6175e-02_rb,5.9489e-04_rb/)
      kbo(:, 2,31, 1) = (/ &
     &1.2399e-01_rb,1.0230e-01_rb,7.2900e-02_rb,4.0465e-02_rb,6.4753e-04_rb/)
      kbo(:, 3,31, 1) = (/ &
     &1.3920e-01_rb,1.1419e-01_rb,8.0980e-02_rb,4.4552e-02_rb,6.9507e-04_rb/)
      kbo(:, 4,31, 1) = (/ &
     &1.5373e-01_rb,1.2537e-01_rb,8.8479e-02_rb,4.8317e-02_rb,7.3661e-04_rb/)
      kbo(:, 5,31, 1) = (/ &
     &1.6735e-01_rb,1.3577e-01_rb,9.5380e-02_rb,5.1770e-02_rb,7.7104e-04_rb/)
      kbo(:, 1,32, 1) = (/ &
     &1.0900e-01_rb,8.9827e-02_rb,6.3903e-02_rb,3.5410e-02_rb,5.0043e-04_rb/)
      kbo(:, 2,32, 1) = (/ &
     &1.2373e-01_rb,1.0144e-01_rb,7.1880e-02_rb,3.9506e-02_rb,5.4269e-04_rb/)
      kbo(:, 3,32, 1) = (/ &
     &1.3829e-01_rb,1.1278e-01_rb,7.9528e-02_rb,4.3367e-02_rb,5.8080e-04_rb/)
      kbo(:, 4,32, 1) = (/ &
     &1.5213e-01_rb,1.2340e-01_rb,8.6611e-02_rb,4.6908e-02_rb,6.1297e-04_rb/)
      kbo(:, 5,32, 1) = (/ &
     &1.6513e-01_rb,1.3334e-01_rb,9.3188e-02_rb,5.0191e-02_rb,6.4067e-04_rb/)
      kbo(:, 1,33, 1) = (/ &
     &1.1105e-01_rb,9.0919e-02_rb,6.4301e-02_rb,3.5277e-02_rb,4.2055e-04_rb/)
      kbo(:, 2,33, 1) = (/ &
     &1.2562e-01_rb,1.0235e-01_rb,7.2096e-02_rb,3.9245e-02_rb,4.5474e-04_rb/)
      kbo(:, 3,33, 1) = (/ &
     &1.3981e-01_rb,1.1334e-01_rb,7.9473e-02_rb,4.2949e-02_rb,4.8461e-04_rb/)
      kbo(:, 4,33, 1) = (/ &
     &1.5326e-01_rb,1.2363e-01_rb,8.6303e-02_rb,4.6360e-02_rb,5.1014e-04_rb/)
      kbo(:, 5,33, 1) = (/ &
     &1.6587e-01_rb,1.3331e-01_rb,9.2704e-02_rb,4.9561e-02_rb,5.3252e-04_rb/)
      kbo(:, 1,34, 1) = (/ &
     &1.0980e-01_rb,8.9454e-02_rb,6.2967e-02_rb,3.4264e-02_rb,3.5179e-04_rb/)
      kbo(:, 2,34, 1) = (/ &
     &1.2374e-01_rb,1.0037e-01_rb,7.0363e-02_rb,3.8004e-02_rb,3.7955e-04_rb/)
      kbo(:, 3,34, 1) = (/ &
     &1.3725e-01_rb,1.1080e-01_rb,7.7340e-02_rb,4.1494e-02_rb,4.0292e-04_rb/)
      kbo(:, 4,34, 1) = (/ &
     &1.5005e-01_rb,1.2058e-01_rb,8.3803e-02_rb,4.4722e-02_rb,4.2388e-04_rb/)
      kbo(:, 5,34, 1) = (/ &
     &1.6190e-01_rb,1.2971e-01_rb,8.9844e-02_rb,4.7765e-02_rb,4.4175e-04_rb/)
      kbo(:, 1,35, 1) = (/ &
     &1.0733e-01_rb,8.7091e-02_rb,6.1056e-02_rb,3.2994e-02_rb,2.9230e-04_rb/)
      kbo(:, 2,35, 1) = (/ &
     &1.2074e-01_rb,9.7561e-02_rb,6.8117e-02_rb,3.6550e-02_rb,3.1434e-04_rb/)
      kbo(:, 3,35, 1) = (/ &
     &1.3369e-01_rb,1.0755e-01_rb,7.4768e-02_rb,3.9873e-02_rb,3.3363e-04_rb/)
      kbo(:, 4,35, 1) = (/ &
     &1.4588e-01_rb,1.1689e-01_rb,8.0950e-02_rb,4.2958e-02_rb,3.5062e-04_rb/)
      kbo(:, 5,35, 1) = (/ &
     &1.5707e-01_rb,1.2552e-01_rb,8.6668e-02_rb,4.5847e-02_rb,3.6479e-04_rb/)
      kbo(:, 1,36, 1) = (/ &
     &1.0289e-01_rb,8.3241e-02_rb,5.8159e-02_rb,3.1253e-02_rb,2.4069e-04_rb/)
      kbo(:, 2,36, 1) = (/ &
     &1.1571e-01_rb,9.3253e-02_rb,6.4895e-02_rb,3.4641e-02_rb,2.5868e-04_rb/)
      kbo(:, 3,36, 1) = (/ &
     &1.2815e-01_rb,1.0284e-01_rb,7.1277e-02_rb,3.7824e-02_rb,2.7449e-04_rb/)
      kbo(:, 4,36, 1) = (/ &
     &1.3974e-01_rb,1.1174e-01_rb,7.7168e-02_rb,4.0766e-02_rb,2.8849e-04_rb/)
      kbo(:, 5,36, 1) = (/ &
     &1.5033e-01_rb,1.1990e-01_rb,8.2576e-02_rb,4.3496e-02_rb,2.9983e-04_rb/)
      kbo(:, 1,37, 1) = (/ &
     &9.4340e-02_rb,7.6226e-02_rb,5.3140e-02_rb,2.8456e-02_rb,1.9559e-04_rb/)
      kbo(:, 2,37, 1) = (/ &
     &1.0634e-01_rb,8.5612e-02_rb,5.9452e-02_rb,3.1630e-02_rb,2.1059e-04_rb/)
      kbo(:, 3,37, 1) = (/ &
     &1.1803e-01_rb,9.4641e-02_rb,6.5469e-02_rb,3.4631e-02_rb,2.2381e-04_rb/)
      kbo(:, 4,37, 1) = (/ &
     &1.2893e-01_rb,1.0302e-01_rb,7.1019e-02_rb,3.7400e-02_rb,2.3549e-04_rb/)
      kbo(:, 5,37, 1) = (/ &
     &1.3882e-01_rb,1.1066e-01_rb,7.6074e-02_rb,3.9944e-02_rb,2.4505e-04_rb/)
      kbo(:, 1,38, 1) = (/ &
     &8.7666e-02_rb,7.0724e-02_rb,4.9189e-02_rb,2.6244e-02_rb,1.5881e-04_rb/)
      kbo(:, 2,38, 1) = (/ &
     &9.9075e-02_rb,7.9666e-02_rb,5.5208e-02_rb,2.9270e-02_rb,1.7135e-04_rb/)
      kbo(:, 3,38, 1) = (/ &
     &1.1022e-01_rb,8.8294e-02_rb,6.0961e-02_rb,3.2140e-02_rb,1.8248e-04_rb/)
      kbo(:, 4,38, 1) = (/ &
     &1.2063e-01_rb,9.6310e-02_rb,6.6270e-02_rb,3.4786e-02_rb,1.9228e-04_rb/)
      kbo(:, 5,38, 1) = (/ &
     &1.3002e-01_rb,1.0355e-01_rb,7.1058e-02_rb,3.7188e-02_rb,2.0028e-04_rb/)
      kbo(:, 1,39, 1) = (/ &
     &8.3319e-02_rb,6.7083e-02_rb,4.6540e-02_rb,2.4732e-02_rb,1.2903e-04_rb/)
      kbo(:, 2,39, 1) = (/ &
     &9.4409e-02_rb,7.5788e-02_rb,5.2401e-02_rb,2.7679e-02_rb,1.3958e-04_rb/)
      kbo(:, 3,39, 1) = (/ &
     &1.0530e-01_rb,8.4225e-02_rb,5.8027e-02_rb,3.0486e-02_rb,1.4894e-04_rb/)
      kbo(:, 4,39, 1) = (/ &
     &1.1544e-01_rb,9.2061e-02_rb,6.3226e-02_rb,3.3071e-02_rb,1.5713e-04_rb/)
      kbo(:, 5,39, 1) = (/ &
     &1.2457e-01_rb,9.9102e-02_rb,6.7879e-02_rb,3.5400e-02_rb,1.6372e-04_rb/)
      kbo(:, 1,40, 1) = (/ &
     &7.3382e-02_rb,5.9091e-02_rb,4.0949e-02_rb,2.1720e-02_rb,1.0369e-04_rb/)
      kbo(:, 2,40, 1) = (/ &
     &8.3492e-02_rb,6.7054e-02_rb,4.6317e-02_rb,2.4422e-02_rb,1.1258e-04_rb/)
      kbo(:, 3,40, 1) = (/ &
     &9.3502e-02_rb,7.4845e-02_rb,5.1514e-02_rb,2.7021e-02_rb,1.2049e-04_rb/)
      kbo(:, 4,40, 1) = (/ &
     &1.0289e-01_rb,8.2115e-02_rb,5.6339e-02_rb,2.9419e-02_rb,1.2744e-04_rb/)
      kbo(:, 5,40, 1) = (/ &
     &1.1132e-01_rb,8.8604e-02_rb,6.0629e-02_rb,3.1559e-02_rb,1.3317e-04_rb/)
      kbo(:, 1,41, 1) = (/ &
     &6.4289e-02_rb,5.1783e-02_rb,3.5851e-02_rb,1.8983e-02_rb,8.3205e-05_rb/)
      kbo(:, 2,41, 1) = (/ &
     &7.3486e-02_rb,5.9047e-02_rb,4.0756e-02_rb,2.1456e-02_rb,9.0601e-05_rb/)
      kbo(:, 3,41, 1) = (/ &
     &8.2657e-02_rb,6.6211e-02_rb,4.5539e-02_rb,2.3850e-02_rb,9.7309e-05_rb/)
      kbo(:, 4,41, 1) = (/ &
     &9.1325e-02_rb,7.2956e-02_rb,5.0013e-02_rb,2.6075e-02_rb,1.0320e-04_rb/)
      kbo(:, 5,41, 1) = (/ &
     &9.9149e-02_rb,7.8969e-02_rb,5.3991e-02_rb,2.8057e-02_rb,1.0825e-04_rb/)
      kbo(:, 1,42, 1) = (/ &
     &5.6899e-02_rb,4.5812e-02_rb,3.1686e-02_rb,1.6747e-02_rb,6.6719e-05_rb/)
      kbo(:, 2,42, 1) = (/ &
     &6.5316e-02_rb,5.2494e-02_rb,3.6198e-02_rb,1.9026e-02_rb,7.2862e-05_rb/)
      kbo(:, 3,42, 1) = (/ &
     &7.3779e-02_rb,5.9136e-02_rb,4.0639e-02_rb,2.1249e-02_rb,7.8557e-05_rb/)
      kbo(:, 4,42, 1) = (/ &
     &8.1877e-02_rb,6.5448e-02_rb,4.4826e-02_rb,2.3334e-02_rb,8.3575e-05_rb/)
      kbo(:, 5,42, 1) = (/ &
     &8.9207e-02_rb,7.1094e-02_rb,4.8562e-02_rb,2.5195e-02_rb,8.7912e-05_rb/)
      kbo(:, 1,43, 1) = (/ &
     &5.0334e-02_rb,4.0495e-02_rb,2.7979e-02_rb,1.4764e-02_rb,5.3104e-05_rb/)
      kbo(:, 2,43, 1) = (/ &
     &5.8063e-02_rb,4.6679e-02_rb,3.2159e-02_rb,1.6880e-02_rb,5.8298e-05_rb/)
      kbo(:, 3,43, 1) = (/ &
     &6.5961e-02_rb,5.2905e-02_rb,3.6327e-02_rb,1.8968e-02_rb,6.3123e-05_rb/)
      kbo(:, 4,43, 1) = (/ &
     &7.3585e-02_rb,5.8862e-02_rb,4.0291e-02_rb,2.0945e-02_rb,6.7425e-05_rb/)
      kbo(:, 5,43, 1) = (/ &
     &8.0560e-02_rb,6.4257e-02_rb,4.3859e-02_rb,2.2720e-02_rb,7.1177e-05_rb/)
      kbo(:, 1,44, 1) = (/ &
     &4.4899e-02_rb,3.6074e-02_rb,2.4891e-02_rb,1.3115e-02_rb,4.2029e-05_rb/)
      kbo(:, 2,44, 1) = (/ &
     &5.2065e-02_rb,4.1850e-02_rb,2.8809e-02_rb,1.5097e-02_rb,4.6479e-05_rb/)
      kbo(:, 3,44, 1) = (/ &
     &5.9485e-02_rb,4.7740e-02_rb,3.2758e-02_rb,1.7077e-02_rb,5.0599e-05_rb/)
      kbo(:, 4,44, 1) = (/ &
     &6.6781e-02_rb,5.3454e-02_rb,3.6566e-02_rb,1.8982e-02_rb,5.4263e-05_rb/)
      kbo(:, 5,44, 1) = (/ &
     &7.3522e-02_rb,5.8690e-02_rb,4.0031e-02_rb,2.0702e-02_rb,5.7530e-05_rb/)
      kbo(:, 1,45, 1) = (/ &
     &4.0792e-02_rb,3.2725e-02_rb,2.2543e-02_rb,1.1850e-02_rb,3.3244e-05_rb/)
      kbo(:, 2,45, 1) = (/ &
     &4.7552e-02_rb,3.8195e-02_rb,2.6267e-02_rb,1.3739e-02_rb,3.7011e-05_rb/)
      kbo(:, 3,45, 1) = (/ &
     &5.4656e-02_rb,4.3858e-02_rb,3.0076e-02_rb,1.5653e-02_rb,4.0475e-05_rb/)
      kbo(:, 4,45, 1) = (/ &
     &6.1725e-02_rb,4.9413e-02_rb,3.3782e-02_rb,1.7505e-02_rb,4.3662e-05_rb/)
      kbo(:, 5,45, 1) = (/ &
     &6.8362e-02_rb,5.4599e-02_rb,3.7216e-02_rb,1.9215e-02_rb,4.6426e-05_rb/)
      kbo(:, 1,46, 1) = (/ &
     &3.7210e-02_rb,2.9813e-02_rb,2.0505e-02_rb,1.0753e-02_rb,2.6192e-05_rb/)
      kbo(:, 2,46, 1) = (/ &
     &4.3666e-02_rb,3.5023e-02_rb,2.4062e-02_rb,1.2564e-02_rb,2.9338e-05_rb/)
      kbo(:, 3,46, 1) = (/ &
     &5.0484e-02_rb,4.0487e-02_rb,2.7746e-02_rb,1.4418e-02_rb,3.2269e-05_rb/)
      kbo(:, 4,46, 1) = (/ &
     &5.7389e-02_rb,4.5948e-02_rb,3.1393e-02_rb,1.6243e-02_rb,3.5047e-05_rb/)
      kbo(:, 5,46, 1) = (/ &
     &6.3983e-02_rb,5.1116e-02_rb,3.4828e-02_rb,1.7955e-02_rb,3.7412e-05_rb/)
      kbo(:, 1,47, 1) = (/ &
     &3.3307e-02_rb,2.6647e-02_rb,1.8307e-02_rb,9.5851e-03_rb,2.0446e-05_rb/)
      kbo(:, 2,47, 1) = (/ &
     &3.9395e-02_rb,3.1552e-02_rb,2.1654e-02_rb,1.1293e-02_rb,2.3124e-05_rb/)
      kbo(:, 3,47, 1) = (/ &
     &4.5832e-02_rb,3.6752e-02_rb,2.5172e-02_rb,1.3065e-02_rb,2.5601e-05_rb/)
      kbo(:, 4,47, 1) = (/ &
     &5.2491e-02_rb,4.2052e-02_rb,2.8722e-02_rb,1.4843e-02_rb,2.7940e-05_rb/)
      kbo(:, 5,47, 1) = (/ &
     &5.8958e-02_rb,4.7137e-02_rb,3.2108e-02_rb,1.6536e-02_rb,3.0007e-05_rb/)
      kbo(:, 1,48, 1) = (/ &
     &3.0577e-02_rb,2.4411e-02_rb,1.6745e-02_rb,8.7495e-03_rb,1.5922e-05_rb/)
      kbo(:, 2,48, 1) = (/ &
     &3.6464e-02_rb,2.9147e-02_rb,1.9975e-02_rb,1.0397e-02_rb,1.8148e-05_rb/)
      kbo(:, 3,48, 1) = (/ &
     &4.2691e-02_rb,3.4197e-02_rb,2.3406e-02_rb,1.2132e-02_rb,2.0290e-05_rb/)
      kbo(:, 4,48, 1) = (/ &
     &4.9243e-02_rb,3.9438e-02_rb,2.6932e-02_rb,1.3903e-02_rb,2.2244e-05_rb/)
      kbo(:, 5,48, 1) = (/ &
     &5.5751e-02_rb,4.4571e-02_rb,3.0355e-02_rb,1.5610e-02_rb,2.4044e-05_rb/)
      kbo(:, 1,49, 1) = (/ &
     &2.9139e-02_rb,2.3202e-02_rb,1.5887e-02_rb,8.2788e-03_rb,1.2379e-05_rb/)
      kbo(:, 2,49, 1) = (/ &
     &3.5049e-02_rb,2.7954e-02_rb,1.9124e-02_rb,9.9302e-03_rb,1.4214e-05_rb/)
      kbo(:, 3,49, 1) = (/ &
     &4.1342e-02_rb,3.3034e-02_rb,2.2588e-02_rb,1.1686e-02_rb,1.6044e-05_rb/)
      kbo(:, 4,49, 1) = (/ &
     &4.8011e-02_rb,3.8402e-02_rb,2.6208e-02_rb,1.3508e-02_rb,1.7733e-05_rb/)
      kbo(:, 5,49, 1) = (/ &
     &5.4790e-02_rb,4.3771e-02_rb,2.9799e-02_rb,1.5302e-02_rb,1.9282e-05_rb/)
      kbo(:, 1,50, 1) = (/ &
     &2.6464e-02_rb,2.1057e-02_rb,1.4400e-02_rb,7.4918e-03_rb,9.6160e-06_rb/)
      kbo(:, 2,50, 1) = (/ &
     &3.2138e-02_rb,2.5589e-02_rb,1.7489e-02_rb,9.0674e-03_rb,1.1117e-05_rb/)
      kbo(:, 3,50, 1) = (/ &
     &3.8201e-02_rb,3.0472e-02_rb,2.0816e-02_rb,1.0758e-02_rb,1.2650e-05_rb/)
      kbo(:, 4,50, 1) = (/ &
     &4.4627e-02_rb,3.5684e-02_rb,2.4344e-02_rb,1.2533e-02_rb,1.4094e-05_rb/)
      kbo(:, 5,50, 1) = (/ &
     &5.1304e-02_rb,4.1009e-02_rb,2.7917e-02_rb,1.4322e-02_rb,1.5452e-05_rb/)
      kbo(:, 1,51, 1) = (/ &
     &2.3471e-02_rb,1.8670e-02_rb,1.2760e-02_rb,6.6303e-03_rb,7.4600e-06_rb/)
      kbo(:, 2,51, 1) = (/ &
     &2.8809e-02_rb,2.2911e-02_rb,1.5642e-02_rb,8.0988e-03_rb,8.6870e-06_rb/)
      kbo(:, 3,51, 1) = (/ &
     &3.4515e-02_rb,2.7502e-02_rb,1.8771e-02_rb,9.6885e-03_rb,9.9502e-06_rb/)
      kbo(:, 4,51, 1) = (/ &
     &4.0564e-02_rb,3.2426e-02_rb,2.2116e-02_rb,1.1378e-02_rb,1.1172e-05_rb/)
      kbo(:, 5,51, 1) = (/ &
     &4.6952e-02_rb,3.7546e-02_rb,2.5560e-02_rb,1.3107e-02_rb,1.2317e-05_rb/)
      kbo(:, 1,52, 1) = (/ &
     &2.1330e-02_rb,1.6945e-02_rb,1.1569e-02_rb,6.0043e-03_rb,5.7740e-06_rb/)
      kbo(:, 2,52, 1) = (/ &
     &2.6486e-02_rb,2.1033e-02_rb,1.4344e-02_rb,7.4158e-03_rb,6.7739e-06_rb/)
      kbo(:, 3,52, 1) = (/ &
     &3.1984e-02_rb,2.5459e-02_rb,1.7358e-02_rb,8.9483e-03_rb,7.8163e-06_rb/)
      kbo(:, 4,52, 1) = (/ &
     &3.7859e-02_rb,3.0220e-02_rb,2.0602e-02_rb,1.0589e-02_rb,8.8379e-06_rb/)
      kbo(:, 5,52, 1) = (/ &
     &4.4095e-02_rb,3.5255e-02_rb,2.3994e-02_rb,1.2295e-02_rb,9.8102e-06_rb/)
      kbo(:, 1,53, 1) = (/ &
     &2.0104e-02_rb,1.5937e-02_rb,1.0867e-02_rb,5.6295e-03_rb,4.4532e-06_rb/)
      kbo(:, 2,53, 1) = (/ &
     &2.5267e-02_rb,2.0039e-02_rb,1.3647e-02_rb,7.0429e-03_rb,5.2727e-06_rb/)
      kbo(:, 3,53, 1) = (/ &
     &3.0800e-02_rb,2.4464e-02_rb,1.6661e-02_rb,8.5763e-03_rb,6.1352e-06_rb/)
      kbo(:, 4,53, 1) = (/ &
     &3.6739e-02_rb,2.9255e-02_rb,1.9925e-02_rb,1.0232e-02_rb,6.9843e-06_rb/)
      kbo(:, 5,53, 1) = (/ &
     &4.3045e-02_rb,3.4379e-02_rb,2.3392e-02_rb,1.1975e-02_rb,7.8095e-06_rb/)
      kbo(:, 1,54, 1) = (/ &
     &1.6441e-02_rb,1.3030e-02_rb,8.8847e-03_rb,4.6022e-03_rb,3.4230e-06_rb/)
      kbo(:, 2,54, 1) = (/ &
     &2.0888e-02_rb,1.6582e-02_rb,1.1293e-02_rb,5.8278e-03_rb,4.0956e-06_rb/)
      kbo(:, 3,54, 1) = (/ &
     &2.5721e-02_rb,2.0433e-02_rb,1.3910e-02_rb,7.1575e-03_rb,4.7876e-06_rb/)
      kbo(:, 4,54, 1) = (/ &
     &3.0904e-02_rb,2.4618e-02_rb,1.6759e-02_rb,8.6032e-03_rb,5.4982e-06_rb/)
      kbo(:, 5,54, 1) = (/ &
     &3.6412e-02_rb,2.9120e-02_rb,1.9814e-02_rb,1.0143e-02_rb,6.1750e-06_rb/)
      kbo(:, 1,55, 1) = (/ &
     &1.2258e-02_rb,9.7268e-03_rb,6.6366e-03_rb,3.4428e-03_rb,2.6213e-06_rb/)
      kbo(:, 2,55, 1) = (/ &
     &1.5748e-02_rb,1.2527e-02_rb,8.5365e-03_rb,4.4115e-03_rb,3.1717e-06_rb/)
      kbo(:, 3,55, 1) = (/ &
     &1.9594e-02_rb,1.5595e-02_rb,1.0619e-02_rb,5.4666e-03_rb,3.7237e-06_rb/)
      kbo(:, 4,55, 1) = (/ &
     &2.3710e-02_rb,1.8931e-02_rb,1.2889e-02_rb,6.6186e-03_rb,4.3031e-06_rb/)
      kbo(:, 5,55, 1) = (/ &
     &2.8112e-02_rb,2.2537e-02_rb,1.5336e-02_rb,7.8537e-03_rb,4.8648e-06_rb/)
      kbo(:, 1,56, 1) = (/ &
     &9.1055e-03_rb,7.2363e-03_rb,4.9401e-03_rb,2.5664e-03_rb,1.9997e-06_rb/)
      kbo(:, 2,56, 1) = (/ &
     &1.1834e-02_rb,9.4303e-03_rb,6.4310e-03_rb,3.3278e-03_rb,2.4479e-06_rb/)
      kbo(:, 3,56, 1) = (/ &
     &1.4882e-02_rb,1.1870e-02_rb,8.0848e-03_rb,4.1651e-03_rb,2.8963e-06_rb/)
      kbo(:, 4,56, 1) = (/ &
     &1.8155e-02_rb,1.4526e-02_rb,9.8917e-03_rb,5.0809e-03_rb,3.3616e-06_rb/)
      kbo(:, 5,56, 1) = (/ &
     &2.1671e-02_rb,1.7405e-02_rb,1.1845e-02_rb,6.0691e-03_rb,3.8293e-06_rb/)
      kbo(:, 1,57, 1) = (/ &
     &6.7286e-03_rb,5.3591e-03_rb,3.6618e-03_rb,1.9048e-03_rb,1.5179e-06_rb/)
      kbo(:, 2,57, 1) = (/ &
     &8.8620e-03_rb,7.0733e-03_rb,4.8264e-03_rb,2.5013e-03_rb,1.8807e-06_rb/)
      kbo(:, 3,57, 1) = (/ &
     &1.1265e-02_rb,9.0088e-03_rb,6.1382e-03_rb,3.1650e-03_rb,2.2483e-06_rb/)
      kbo(:, 4,57, 1) = (/ &
     &1.3873e-02_rb,1.1116e-02_rb,7.5709e-03_rb,3.8911e-03_rb,2.6184e-06_rb/)
      kbo(:, 5,57, 1) = (/ &
     &1.6676e-02_rb,1.3415e-02_rb,9.1296e-03_rb,4.6803e-03_rb,3.0020e-06_rb/)
      kbo(:, 1,58, 1) = (/ &
     &1.3112e-03_rb,1.2811e-03_rb,1.1274e-03_rb,8.2340e-04_rb,1.1519e-06_rb/)
      kbo(:, 2,58, 1) = (/ &
     &1.7518e-03_rb,1.7126e-03_rb,1.5045e-03_rb,1.0941e-03_rb,1.4467e-06_rb/)
      kbo(:, 3,58, 1) = (/ &
     &2.2498e-03_rb,2.2053e-03_rb,1.9345e-03_rb,1.3994e-03_rb,1.7445e-06_rb/)
      kbo(:, 4,58, 1) = (/ &
     &2.7980e-03_rb,2.7467e-03_rb,2.4064e-03_rb,1.7337e-03_rb,2.0427e-06_rb/)
      kbo(:, 5,58, 1) = (/ &
     &3.3868e-03_rb,3.3394e-03_rb,2.9224e-03_rb,2.0998e-03_rb,2.3542e-06_rb/)
      kbo(:, 1,59, 1) = (/ &
     &1.2094e-03_rb,1.1351e-03_rb,9.3340e-04_rb,6.0964e-04_rb,9.0654e-07_rb/)
      kbo(:, 2,59, 1) = (/ &
     &1.6257e-03_rb,1.5254e-03_rb,1.2509e-03_rb,8.1260e-04_rb,1.1440e-06_rb/)
      kbo(:, 3,59, 1) = (/ &
     &2.0981e-03_rb,1.9741e-03_rb,1.6151e-03_rb,1.0426e-03_rb,1.3852e-06_rb/)
      kbo(:, 4,59, 1) = (/ &
     &2.6203e-03_rb,2.4691e-03_rb,2.0150e-03_rb,1.2943e-03_rb,1.6270e-06_rb/)
      kbo(:, 5,59, 1) = (/ &
     &3.1814e-03_rb,3.0122e-03_rb,2.4540e-03_rb,1.5709e-03_rb,1.8687e-06_rb/)
      kbo(:, 1,13, 2) = (/ &
     &6.6529e+01_rb,4.9985e+01_rb,3.3446e+01_rb,1.6923e+01_rb,3.5914e-02_rb/)
      kbo(:, 2,13, 2) = (/ &
     &7.4095e+01_rb,5.5661e+01_rb,3.7226e+01_rb,1.8804e+01_rb,3.9958e-02_rb/)
      kbo(:, 3,13, 2) = (/ &
     &8.0840e+01_rb,6.0720e+01_rb,4.0598e+01_rb,2.0484e+01_rb,4.3881e-02_rb/)
      kbo(:, 4,13, 2) = (/ &
     &8.6396e+01_rb,6.4888e+01_rb,4.3378e+01_rb,2.1867e+01_rb,4.7563e-02_rb/)
      kbo(:, 5,13, 2) = (/ &
     &9.0642e+01_rb,6.8073e+01_rb,4.5503e+01_rb,2.2931e+01_rb,5.1188e-02_rb/)
      kbo(:, 1,14, 2) = (/ &
     &3.4987e+01_rb,2.6327e+01_rb,1.7678e+01_rb,9.0444e+00_rb,3.1025e-02_rb/)
      kbo(:, 2,14, 2) = (/ &
     &3.8934e+01_rb,2.9288e+01_rb,1.9648e+01_rb,1.0023e+01_rb,3.4459e-02_rb/)
      kbo(:, 3,14, 2) = (/ &
     &4.2399e+01_rb,3.1888e+01_rb,2.1376e+01_rb,1.0881e+01_rb,3.7786e-02_rb/)
      kbo(:, 4,14, 2) = (/ &
     &4.5141e+01_rb,3.3946e+01_rb,2.2749e+01_rb,1.1559e+01_rb,4.1037e-02_rb/)
      kbo(:, 5,14, 2) = (/ &
     &4.7193e+01_rb,3.5485e+01_rb,2.3775e+01_rb,1.2071e+01_rb,4.3970e-02_rb/)
      kbo(:, 1,15, 2) = (/ &
     &1.8797e+01_rb,1.4186e+01_rb,9.5905e+00_rb,4.9916e+00_rb,2.6741e-02_rb/)
      kbo(:, 2,15, 2) = (/ &
     &2.0897e+01_rb,1.5761e+01_rb,1.0636e+01_rb,5.5075e+00_rb,2.9658e-02_rb/)
      kbo(:, 3,15, 2) = (/ &
     &2.2675e+01_rb,1.7093e+01_rb,1.1518e+01_rb,5.9424e+00_rb,3.2475e-02_rb/)
      kbo(:, 4,15, 2) = (/ &
     &2.4040e+01_rb,1.8121e+01_rb,1.2200e+01_rb,6.2773e+00_rb,3.5159e-02_rb/)
      kbo(:, 5,15, 2) = (/ &
     &2.5038e+01_rb,1.8870e+01_rb,1.2700e+01_rb,6.5270e+00_rb,3.7635e-02_rb/)
      kbo(:, 1,16, 2) = (/ &
     &1.2062e+01_rb,9.1374e+00_rb,6.2260e+00_rb,3.2778e+00_rb,2.3010e-02_rb/)
      kbo(:, 2,16, 2) = (/ &
     &1.3368e+01_rb,1.0114e+01_rb,6.8731e+00_rb,3.5955e+00_rb,2.5515e-02_rb/)
      kbo(:, 3,16, 2) = (/ &
     &1.4432e+01_rb,1.0911e+01_rb,7.4005e+00_rb,3.8523e+00_rb,2.7873e-02_rb/)
      kbo(:, 4,16, 2) = (/ &
     &1.5210e+01_rb,1.1498e+01_rb,7.7878e+00_rb,4.0427e+00_rb,3.0096e-02_rb/)
      kbo(:, 5,16, 2) = (/ &
     &1.5795e+01_rb,1.1938e+01_rb,8.0810e+00_rb,4.1897e+00_rb,3.2123e-02_rb/)
      kbo(:, 1,17, 2) = (/ &
     &8.0343e+00_rb,6.1202e+00_rb,4.2057e+00_rb,2.2416e+00_rb,1.9722e-02_rb/)
      kbo(:, 2,17, 2) = (/ &
     &8.8550e+00_rb,6.7335e+00_rb,4.6112e+00_rb,2.4399e+00_rb,2.1833e-02_rb/)
      kbo(:, 3,17, 2) = (/ &
     &9.5028e+00_rb,7.2176e+00_rb,4.9290e+00_rb,2.5918e+00_rb,2.3771e-02_rb/)
      kbo(:, 4,17, 2) = (/ &
     &9.9681e+00_rb,7.5665e+00_rb,5.1592e+00_rb,2.7054e+00_rb,2.5587e-02_rb/)
      kbo(:, 5,17, 2) = (/ &
     &1.0341e+01_rb,7.8464e+00_rb,5.3464e+00_rb,2.7998e+00_rb,2.7262e-02_rb/)
      kbo(:, 1,18, 2) = (/ &
     &5.4784e+00_rb,4.2063e+00_rb,2.9118e+00_rb,1.5707e+00_rb,1.6843e-02_rb/)
      kbo(:, 2,18, 2) = (/ &
     &6.0003e+00_rb,4.5949e+00_rb,3.1671e+00_rb,1.6950e+00_rb,1.8568e-02_rb/)
      kbo(:, 3,18, 2) = (/ &
     &6.4044e+00_rb,4.8955e+00_rb,3.3617e+00_rb,1.7866e+00_rb,2.0170e-02_rb/)
      kbo(:, 4,18, 2) = (/ &
     &6.7000e+00_rb,5.1163e+00_rb,3.5087e+00_rb,1.8600e+00_rb,2.1658e-02_rb/)
      kbo(:, 5,18, 2) = (/ &
     &6.9467e+00_rb,5.3017e+00_rb,3.6339e+00_rb,1.9245e+00_rb,2.2992e-02_rb/)
      kbo(:, 1,19, 2) = (/ &
     &3.6607e+00_rb,2.8397e+00_rb,1.9832e+00_rb,1.0810e+00_rb,1.4294e-02_rb/)
      kbo(:, 2,19, 2) = (/ &
     &3.9888e+00_rb,3.0825e+00_rb,2.1422e+00_rb,1.1564e+00_rb,1.5700e-02_rb/)
      kbo(:, 3,19, 2) = (/ &
     &4.2403e+00_rb,3.2678e+00_rb,2.2604e+00_rb,1.2129e+00_rb,1.7026e-02_rb/)
      kbo(:, 4,19, 2) = (/ &
     &4.4304e+00_rb,3.4105e+00_rb,2.3562e+00_rb,1.2611e+00_rb,1.8223e-02_rb/)
      kbo(:, 5,19, 2) = (/ &
     &4.5892e+00_rb,3.5305e+00_rb,2.4376e+00_rb,1.3052e+00_rb,1.9250e-02_rb/)
      kbo(:, 1,20, 2) = (/ &
     &2.6484e+00_rb,2.0725e+00_rb,1.4576e+00_rb,7.9887e-01_rb,1.2095e-02_rb/)
      kbo(:, 2,20, 2) = (/ &
     &2.8723e+00_rb,2.2377e+00_rb,1.5638e+00_rb,8.4850e-01_rb,1.3258e-02_rb/)
      kbo(:, 3,20, 2) = (/ &
     &3.0432e+00_rb,2.3620e+00_rb,1.6433e+00_rb,8.8749e-01_rb,1.4323e-02_rb/)
      kbo(:, 4,20, 2) = (/ &
     &3.1766e+00_rb,2.4628e+00_rb,1.7113e+00_rb,9.2233e-01_rb,1.5251e-02_rb/)
      kbo(:, 5,20, 2) = (/ &
     &3.2896e+00_rb,2.5490e+00_rb,1.7710e+00_rb,9.5551e-01_rb,1.6040e-02_rb/)
      kbo(:, 1,21, 2) = (/ &
     &1.9737e+00_rb,1.5589e+00_rb,1.1005e+00_rb,6.0564e-01_rb,1.0202e-02_rb/)
      kbo(:, 2,21, 2) = (/ &
     &2.1324e+00_rb,1.6743e+00_rb,1.1736e+00_rb,6.4004e-01_rb,1.1146e-02_rb/)
      kbo(:, 3,21, 2) = (/ &
     &2.2543e+00_rb,1.7630e+00_rb,1.2313e+00_rb,6.6865e-01_rb,1.1981e-02_rb/)
      kbo(:, 4,21, 2) = (/ &
     &2.3508e+00_rb,1.8360e+00_rb,1.2809e+00_rb,6.9497e-01_rb,1.2686e-02_rb/)
      kbo(:, 5,21, 2) = (/ &
     &2.4312e+00_rb,1.8986e+00_rb,1.3258e+00_rb,7.2010e-01_rb,1.3291e-02_rb/)
      kbo(:, 1,22, 2) = (/ &
     &1.5156e+00_rb,1.2064e+00_rb,8.5276e-01_rb,4.7037e-01_rb,8.6256e-03_rb/)
      kbo(:, 2,22, 2) = (/ &
     &1.6299e+00_rb,1.2880e+00_rb,9.0439e-01_rb,4.9507e-01_rb,9.3659e-03_rb/)
      kbo(:, 3,22, 2) = (/ &
     &1.7167e+00_rb,1.3521e+00_rb,9.4701e-01_rb,5.1663e-01_rb,1.0004e-02_rb/)
      kbo(:, 4,22, 2) = (/ &
     &1.7876e+00_rb,1.4059e+00_rb,9.8421e-01_rb,5.3690e-01_rb,1.0541e-02_rb/)
      kbo(:, 5,22, 2) = (/ &
     &1.8482e+00_rb,1.4540e+00_rb,1.0190e+00_rb,5.5667e-01_rb,1.1012e-02_rb/)
      kbo(:, 1,23, 2) = (/ &
     &1.1701e+00_rb,9.3735e-01_rb,6.6344e-01_rb,3.6672e-01_rb,7.2585e-03_rb/)
      kbo(:, 2,23, 2) = (/ &
     &1.2526e+00_rb,9.9560e-01_rb,7.0066e-01_rb,3.8497e-01_rb,7.8298e-03_rb/)
      kbo(:, 3,23, 2) = (/ &
     &1.3157e+00_rb,1.0429e+00_rb,7.3247e-01_rb,4.0155e-01_rb,8.3206e-03_rb/)
      kbo(:, 4,23, 2) = (/ &
     &1.3685e+00_rb,1.0835e+00_rb,7.6114e-01_rb,4.1717e-01_rb,8.7383e-03_rb/)
      kbo(:, 5,23, 2) = (/ &
     &1.4146e+00_rb,1.1207e+00_rb,7.8818e-01_rb,4.3301e-01_rb,9.1036e-03_rb/)
      kbo(:, 1,24, 2) = (/ &
     &8.9949e-01_rb,7.2462e-01_rb,5.1357e-01_rb,2.8470e-01_rb,6.0789e-03_rb/)
      kbo(:, 2,24, 2) = (/ &
     &9.5867e-01_rb,7.6639e-01_rb,5.4063e-01_rb,2.9829e-01_rb,6.5218e-03_rb/)
      kbo(:, 3,24, 2) = (/ &
     &1.0042e+00_rb,8.0114e-01_rb,5.6436e-01_rb,3.1100e-01_rb,6.9029e-03_rb/)
      kbo(:, 4,24, 2) = (/ &
     &1.0431e+00_rb,8.3179e-01_rb,5.8619e-01_rb,3.2299e-01_rb,7.2295e-03_rb/)
      kbo(:, 5,24, 2) = (/ &
     &1.0790e+00_rb,8.6138e-01_rb,6.0791e-01_rb,3.3588e-01_rb,7.5120e-03_rb/)
      kbo(:, 1,25, 2) = (/ &
     &7.0112e-01_rb,5.6715e-01_rb,4.0233e-01_rb,2.2369e-01_rb,5.0763e-03_rb/)
      kbo(:, 2,25, 2) = (/ &
     &7.4350e-01_rb,5.9764e-01_rb,4.2250e-01_rb,2.3404e-01_rb,5.4222e-03_rb/)
      kbo(:, 3,25, 2) = (/ &
     &7.7710e-01_rb,6.2349e-01_rb,4.4048e-01_rb,2.4369e-01_rb,5.7195e-03_rb/)
      kbo(:, 4,25, 2) = (/ &
     &8.0619e-01_rb,6.4737e-01_rb,4.5753e-01_rb,2.5334e-01_rb,5.9744e-03_rb/)
      kbo(:, 5,25, 2) = (/ &
     &8.3531e-01_rb,6.7147e-01_rb,4.7545e-01_rb,2.6391e-01_rb,6.1951e-03_rb/)
      kbo(:, 1,26, 2) = (/ &
     &5.5605e-01_rb,4.5118e-01_rb,3.2039e-01_rb,1.7855e-01_rb,4.2346e-03_rb/)
      kbo(:, 2,26, 2) = (/ &
     &5.8683e-01_rb,4.7392e-01_rb,3.3566e-01_rb,1.8647e-01_rb,4.5042e-03_rb/)
      kbo(:, 3,26, 2) = (/ &
     &6.1229e-01_rb,4.9380e-01_rb,3.4972e-01_rb,1.9404e-01_rb,4.7382e-03_rb/)
      kbo(:, 4,26, 2) = (/ &
     &6.3488e-01_rb,5.1281e-01_rb,3.6343e-01_rb,2.0206e-01_rb,4.9373e-03_rb/)
      kbo(:, 5,26, 2) = (/ &
     &6.5886e-01_rb,5.3298e-01_rb,3.7847e-01_rb,2.1092e-01_rb,5.1112e-03_rb/)
      kbo(:, 1,27, 2) = (/ &
     &4.5766e-01_rb,3.7194e-01_rb,2.6418e-01_rb,1.4728e-01_rb,3.5266e-03_rb/)
      kbo(:, 2,27, 2) = (/ &
     &4.8116e-01_rb,3.8971e-01_rb,2.7627e-01_rb,1.5359e-01_rb,3.7388e-03_rb/)
      kbo(:, 3,27, 2) = (/ &
     &5.0141e-01_rb,4.0592e-01_rb,2.8779e-01_rb,1.5989e-01_rb,3.9220e-03_rb/)
      kbo(:, 4,27, 2) = (/ &
     &5.1998e-01_rb,4.2171e-01_rb,2.9940e-01_rb,1.6680e-01_rb,4.0803e-03_rb/)
      kbo(:, 5,27, 2) = (/ &
     &5.4049e-01_rb,4.3926e-01_rb,3.1241e-01_rb,1.7445e-01_rb,4.2171e-03_rb/)
      kbo(:, 1,28, 2) = (/ &
     &3.8656e-01_rb,3.1440e-01_rb,2.2322e-01_rb,1.2431e-01_rb,2.9341e-03_rb/)
      kbo(:, 2,28, 2) = (/ &
     &4.0539e-01_rb,3.2888e-01_rb,2.3321e-01_rb,1.2952e-01_rb,3.1008e-03_rb/)
      kbo(:, 3,28, 2) = (/ &
     &4.2208e-01_rb,3.4256e-01_rb,2.4291e-01_rb,1.3505e-01_rb,3.2456e-03_rb/)
      kbo(:, 4,28, 2) = (/ &
     &4.3788e-01_rb,3.5618e-01_rb,2.5311e-01_rb,1.4108e-01_rb,3.3707e-03_rb/)
      kbo(:, 5,28, 2) = (/ &
     &4.5599e-01_rb,3.7197e-01_rb,2.6475e-01_rb,1.4784e-01_rb,3.4778e-03_rb/)
      kbo(:, 1,29, 2) = (/ &
     &3.4663e-01_rb,2.8147e-01_rb,1.9935e-01_rb,1.1053e-01_rb,2.4407e-03_rb/)
      kbo(:, 2,29, 2) = (/ &
     &3.6288e-01_rb,2.9416e-01_rb,2.0825e-01_rb,1.1525e-01_rb,2.5726e-03_rb/)
      kbo(:, 3,29, 2) = (/ &
     &3.7738e-01_rb,3.0636e-01_rb,2.1696e-01_rb,1.2034e-01_rb,2.6876e-03_rb/)
      kbo(:, 4,29, 2) = (/ &
     &3.9208e-01_rb,3.1911e-01_rb,2.2648e-01_rb,1.2589e-01_rb,2.7881e-03_rb/)
      kbo(:, 5,29, 2) = (/ &
     &4.0886e-01_rb,3.3389e-01_rb,2.3740e-01_rb,1.3220e-01_rb,2.8698e-03_rb/)
      kbo(:, 1,30, 2) = (/ &
     &3.1859e-01_rb,2.5807e-01_rb,1.8226e-01_rb,1.0052e-01_rb,2.0286e-03_rb/)
      kbo(:, 2,30, 2) = (/ &
     &3.3300e-01_rb,2.6960e-01_rb,1.9038e-01_rb,1.0493e-01_rb,2.1345e-03_rb/)
      kbo(:, 3,30, 2) = (/ &
     &3.4608e-01_rb,2.8079e-01_rb,1.9849e-01_rb,1.0968e-01_rb,2.2264e-03_rb/)
      kbo(:, 4,30, 2) = (/ &
     &3.6002e-01_rb,2.9303e-01_rb,2.0760e-01_rb,1.1495e-01_rb,2.3052e-03_rb/)
      kbo(:, 5,30, 2) = (/ &
     &3.7616e-01_rb,3.0718e-01_rb,2.1802e-01_rb,1.2086e-01_rb,2.3703e-03_rb/)
      kbo(:, 1,31, 2) = (/ &
     &3.0804e-01_rb,2.4854e-01_rb,1.7488e-01_rb,9.5771e-02_rb,1.6879e-03_rb/)
      kbo(:, 2,31, 2) = (/ &
     &3.2148e-01_rb,2.5954e-01_rb,1.8261e-01_rb,1.0007e-01_rb,1.7724e-03_rb/)
      kbo(:, 3,31, 2) = (/ &
     &3.3418e-01_rb,2.7047e-01_rb,1.9064e-01_rb,1.0472e-01_rb,1.8461e-03_rb/)
      kbo(:, 4,31, 2) = (/ &
     &3.4816e-01_rb,2.8282e-01_rb,1.9976e-01_rb,1.0990e-01_rb,1.9082e-03_rb/)
      kbo(:, 5,31, 2) = (/ &
     &3.6451e-01_rb,2.9706e-01_rb,2.1019e-01_rb,1.1572e-01_rb,1.9578e-03_rb/)
      kbo(:, 1,32, 2) = (/ &
     &3.0134e-01_rb,2.4222e-01_rb,1.6982e-01_rb,9.2391e-02_rb,1.4045e-03_rb/)
      kbo(:, 2,32, 2) = (/ &
     &3.1413e-01_rb,2.5288e-01_rb,1.7732e-01_rb,9.6600e-02_rb,1.4719e-03_rb/)
      kbo(:, 3,32, 2) = (/ &
     &3.2677e-01_rb,2.6386e-01_rb,1.8545e-01_rb,1.0123e-01_rb,1.5309e-03_rb/)
      kbo(:, 4,32, 2) = (/ &
     &3.4104e-01_rb,2.7646e-01_rb,1.9466e-01_rb,1.0637e-01_rb,1.5796e-03_rb/)
      kbo(:, 5,32, 2) = (/ &
     &3.5782e-01_rb,2.9105e-01_rb,2.0529e-01_rb,1.1222e-01_rb,1.6199e-03_rb/)
      kbo(:, 1,33, 2) = (/ &
     &3.0007e-01_rb,2.4036e-01_rb,1.6789e-01_rb,9.0749e-02_rb,1.1692e-03_rb/)
      kbo(:, 2,33, 2) = (/ &
     &3.1257e-01_rb,2.5093e-01_rb,1.7540e-01_rb,9.4970e-02_rb,1.2231e-03_rb/)
      kbo(:, 3,33, 2) = (/ &
     &3.2551e-01_rb,2.6224e-01_rb,1.8373e-01_rb,9.9635e-02_rb,1.2703e-03_rb/)
      kbo(:, 4,33, 2) = (/ &
     &3.4043e-01_rb,2.7535e-01_rb,1.9325e-01_rb,1.0486e-01_rb,1.3103e-03_rb/)
      kbo(:, 5,33, 2) = (/ &
     &3.5814e-01_rb,2.9066e-01_rb,2.0432e-01_rb,1.1086e-01_rb,1.3413e-03_rb/)
      kbo(:, 1,34, 2) = (/ &
     &2.9092e-01_rb,2.3257e-01_rb,1.6200e-01_rb,8.7114e-02_rb,9.7104e-04_rb/)
      kbo(:, 2,34, 2) = (/ &
     &3.0301e-01_rb,2.4286e-01_rb,1.6939e-01_rb,9.1256e-02_rb,1.0146e-03_rb/)
      kbo(:, 3,34, 2) = (/ &
     &3.1600e-01_rb,2.5431e-01_rb,1.7773e-01_rb,9.5863e-02_rb,1.0532e-03_rb/)
      kbo(:, 4,34, 2) = (/ &
     &3.3116e-01_rb,2.6756e-01_rb,1.8732e-01_rb,1.0106e-01_rb,1.0840e-03_rb/)
      kbo(:, 5,34, 2) = (/ &
     &3.4944e-01_rb,2.8336e-01_rb,1.9862e-01_rb,1.0710e-01_rb,1.1076e-03_rb/)
      kbo(:, 1,35, 2) = (/ &
     &2.8133e-01_rb,2.2448e-01_rb,1.5598e-01_rb,8.3470e-02_rb,8.0326e-04_rb/)
      kbo(:, 2,35, 2) = (/ &
     &2.9315e-01_rb,2.3461e-01_rb,1.6327e-01_rb,8.7518e-02_rb,8.3948e-04_rb/)
      kbo(:, 3,35, 2) = (/ &
     &3.0606e-01_rb,2.4602e-01_rb,1.7152e-01_rb,9.2018e-02_rb,8.6953e-04_rb/)
      kbo(:, 4,35, 2) = (/ &
     &3.2135e-01_rb,2.5937e-01_rb,1.8112e-01_rb,9.7174e-02_rb,8.9330e-04_rb/)
      kbo(:, 5,35, 2) = (/ &
     &3.3996e-01_rb,2.7548e-01_rb,1.9256e-01_rb,1.0324e-01_rb,9.1326e-04_rb/)
      kbo(:, 1,36, 2) = (/ &
     &2.6927e-01_rb,2.1450e-01_rb,1.4871e-01_rb,7.9228e-02_rb,6.6044e-04_rb/)
      kbo(:, 2,36, 2) = (/ &
     &2.8072e-01_rb,2.2434e-01_rb,1.5577e-01_rb,8.3114e-02_rb,6.9019e-04_rb/)
      kbo(:, 3,36, 2) = (/ &
     &2.9329e-01_rb,2.3546e-01_rb,1.6379e-01_rb,8.7445e-02_rb,7.1471e-04_rb/)
      kbo(:, 4,36, 2) = (/ &
     &3.0843e-01_rb,2.4872e-01_rb,1.7326e-01_rb,9.2498e-02_rb,7.3506e-04_rb/)
      kbo(:, 5,36, 2) = (/ &
     &3.2685e-01_rb,2.6470e-01_rb,1.8459e-01_rb,9.8476e-02_rb,7.5121e-04_rb/)
      kbo(:, 1,37, 2) = (/ &
     &2.4965e-01_rb,1.9865e-01_rb,1.3749e-01_rb,7.2983e-02_rb,5.3914e-04_rb/)
      kbo(:, 2,37, 2) = (/ &
     &2.6042e-01_rb,2.0789e-01_rb,1.4407e-01_rb,7.6575e-02_rb,5.6372e-04_rb/)
      kbo(:, 3,37, 2) = (/ &
     &2.7217e-01_rb,2.1833e-01_rb,1.5157e-01_rb,8.0593e-02_rb,5.8448e-04_rb/)
      kbo(:, 4,37, 2) = (/ &
     &2.8633e-01_rb,2.3076e-01_rb,1.6043e-01_rb,8.5310e-02_rb,6.0216e-04_rb/)
      kbo(:, 5,37, 2) = (/ &
     &3.0356e-01_rb,2.4582e-01_rb,1.7110e-01_rb,9.0929e-02_rb,6.1615e-04_rb/)
      kbo(:, 1,38, 2) = (/ &
     &2.3485e-01_rb,1.8661e-01_rb,1.2891e-01_rb,6.8178e-02_rb,4.3970e-04_rb/)
      kbo(:, 2,38, 2) = (/ &
     &2.4515e-01_rb,1.9543e-01_rb,1.3515e-01_rb,7.1546e-02_rb,4.6052e-04_rb/)
      kbo(:, 3,38, 2) = (/ &
     &2.5629e-01_rb,2.0533e-01_rb,1.4225e-01_rb,7.5330e-02_rb,4.7811e-04_rb/)
      kbo(:, 4,38, 2) = (/ &
     &2.6971e-01_rb,2.1715e-01_rb,1.5065e-01_rb,7.9788e-02_rb,4.9330e-04_rb/)
      kbo(:, 5,38, 2) = (/ &
     &2.8605e-01_rb,2.3151e-01_rb,1.6083e-01_rb,8.5133e-02_rb,5.0547e-04_rb/)
      kbo(:, 1,39, 2) = (/ &
     &2.2602e-01_rb,1.7927e-01_rb,1.2358e-01_rb,6.5103e-02_rb,3.5893e-04_rb/)
      kbo(:, 2,39, 2) = (/ &
     &2.3611e-01_rb,1.8789e-01_rb,1.2965e-01_rb,6.8347e-02_rb,3.7626e-04_rb/)
      kbo(:, 3,39, 2) = (/ &
     &2.4693e-01_rb,1.9751e-01_rb,1.3653e-01_rb,7.1994e-02_rb,3.9112e-04_rb/)
      kbo(:, 4,39, 2) = (/ &
     &2.5998e-01_rb,2.0903e-01_rb,1.4470e-01_rb,7.6316e-02_rb,4.0431e-04_rb/)
      kbo(:, 5,39, 2) = (/ &
     &2.7577e-01_rb,2.2297e-01_rb,1.5458e-01_rb,8.1500e-02_rb,4.1520e-04_rb/)
      kbo(:, 1,40, 2) = (/ &
     &2.0347e-01_rb,1.6128e-01_rb,1.1107e-01_rb,5.8373e-02_rb,2.9088e-04_rb/)
      kbo(:, 2,40, 2) = (/ &
     &2.1275e-01_rb,1.6921e-01_rb,1.1659e-01_rb,6.1296e-02_rb,3.0550e-04_rb/)
      kbo(:, 3,40, 2) = (/ &
     &2.2248e-01_rb,1.7787e-01_rb,1.2279e-01_rb,6.4562e-02_rb,3.1830e-04_rb/)
      kbo(:, 4,40, 2) = (/ &
     &2.3419e-01_rb,1.8824e-01_rb,1.3013e-01_rb,6.8434e-02_rb,3.2975e-04_rb/)
      kbo(:, 5,40, 2) = (/ &
     &2.4824e-01_rb,2.0081e-01_rb,1.3903e-01_rb,7.3098e-02_rb,3.3913e-04_rb/)
      kbo(:, 1,41, 2) = (/ &
     &1.8258e-01_rb,1.4470e-01_rb,9.9535e-02_rb,5.2202e-02_rb,2.3535e-04_rb/)
      kbo(:, 2,41, 2) = (/ &
     &1.9113e-01_rb,1.5197e-01_rb,1.0458e-01_rb,5.4844e-02_rb,2.4773e-04_rb/)
      kbo(:, 3,41, 2) = (/ &
     &1.9995e-01_rb,1.5977e-01_rb,1.1015e-01_rb,5.7766e-02_rb,2.5866e-04_rb/)
      kbo(:, 4,41, 2) = (/ &
     &2.1035e-01_rb,1.6906e-01_rb,1.1672e-01_rb,6.1215e-02_rb,2.6866e-04_rb/)
      kbo(:, 5,41, 2) = (/ &
     &2.2281e-01_rb,1.8031e-01_rb,1.2469e-01_rb,6.5393e-02_rb,2.7675e-04_rb/)
      kbo(:, 1,42, 2) = (/ &
     &1.6555e-01_rb,1.3118e-01_rb,9.0125e-02_rb,4.7172e-02_rb,1.9035e-04_rb/)
      kbo(:, 2,42, 2) = (/ &
     &1.7351e-01_rb,1.3791e-01_rb,9.4797e-02_rb,4.9586e-02_rb,2.0086e-04_rb/)
      kbo(:, 3,42, 2) = (/ &
     &1.8167e-01_rb,1.4504e-01_rb,9.9843e-02_rb,5.2229e-02_rb,2.1016e-04_rb/)
      kbo(:, 4,42, 2) = (/ &
     &1.9101e-01_rb,1.5346e-01_rb,1.0581e-01_rb,5.5333e-02_rb,2.1875e-04_rb/)
      kbo(:, 5,42, 2) = (/ &
     &2.0220e-01_rb,1.6362e-01_rb,1.1299e-01_rb,5.9103e-02_rb,2.2587e-04_rb/)
      kbo(:, 1,43, 2) = (/ &
     &1.5093e-01_rb,1.1961e-01_rb,8.2075e-02_rb,4.2874e-02_rb,1.5326e-04_rb/)
      kbo(:, 2,43, 2) = (/ &
     &1.5853e-01_rb,1.2589e-01_rb,8.6428e-02_rb,4.5110e-02_rb,1.6227e-04_rb/)
      kbo(:, 3,43, 2) = (/ &
     &1.6609e-01_rb,1.3244e-01_rb,9.1034e-02_rb,4.7504e-02_rb,1.7029e-04_rb/)
      kbo(:, 4,43, 2) = (/ &
     &1.7452e-01_rb,1.4008e-01_rb,9.6428e-02_rb,5.0296e-02_rb,1.7773e-04_rb/)
      kbo(:, 5,43, 2) = (/ &
     &1.8456e-01_rb,1.4921e-01_rb,1.0290e-01_rb,5.3684e-02_rb,1.8398e-04_rb/)
      kbo(:, 1,44, 2) = (/ &
     &1.3921e-01_rb,1.1030e-01_rb,7.5620e-02_rb,3.9420e-02_rb,1.2316e-04_rb/)
      kbo(:, 2,44, 2) = (/ &
     &1.4661e-01_rb,1.1628e-01_rb,7.9723e-02_rb,4.1521e-02_rb,1.3084e-04_rb/)
      kbo(:, 3,44, 2) = (/ &
     &1.5377e-01_rb,1.2242e-01_rb,8.4004e-02_rb,4.3727e-02_rb,1.3764e-04_rb/)
      kbo(:, 4,44, 2) = (/ &
     &1.6148e-01_rb,1.2941e-01_rb,8.8933e-02_rb,4.6269e-02_rb,1.4416e-04_rb/)
      kbo(:, 5,44, 2) = (/ &
     &1.7060e-01_rb,1.3772e-01_rb,9.4815e-02_rb,4.9343e-02_rb,1.4964e-04_rb/)
      kbo(:, 1,45, 2) = (/ &
     &1.3086e-01_rb,1.0364e-01_rb,7.0980e-02_rb,3.6935e-02_rb,9.8771e-05_rb/)
      kbo(:, 2,45, 2) = (/ &
     &1.3832e-01_rb,1.0957e-01_rb,7.5010e-02_rb,3.8986e-02_rb,1.0533e-04_rb/)
      kbo(:, 3,45, 2) = (/ &
     &1.4520e-01_rb,1.1543e-01_rb,7.9072e-02_rb,4.1056e-02_rb,1.1127e-04_rb/)
      kbo(:, 4,45, 2) = (/ &
     &1.5249e-01_rb,1.2195e-01_rb,8.3674e-02_rb,4.3425e-02_rb,1.1690e-04_rb/)
      kbo(:, 5,45, 2) = (/ &
     &1.6094e-01_rb,1.2967e-01_rb,8.9124e-02_rb,4.6256e-02_rb,1.2176e-04_rb/)
      kbo(:, 1,46, 2) = (/ &
     &1.2395e-01_rb,9.8133e-02_rb,6.7135e-02_rb,3.4876e-02_rb,7.8972e-05_rb/)
      kbo(:, 2,46, 2) = (/ &
     &1.3154e-01_rb,1.0413e-01_rb,7.1186e-02_rb,3.6923e-02_rb,8.4580e-05_rb/)
      kbo(:, 3,46, 2) = (/ &
     &1.3835e-01_rb,1.0981e-01_rb,7.5111e-02_rb,3.8913e-02_rb,8.9707e-05_rb/)
      kbo(:, 4,46, 2) = (/ &
     &1.4539e-01_rb,1.1598e-01_rb,7.9454e-02_rb,4.1140e-02_rb,9.4570e-05_rb/)
      kbo(:, 5,46, 2) = (/ &
     &1.5328e-01_rb,1.2322e-01_rb,8.4552e-02_rb,4.3768e-02_rb,9.8952e-05_rb/)
      kbo(:, 1,47, 2) = (/ &
     &1.1561e-01_rb,9.1621e-02_rb,6.2646e-02_rb,3.2504e-02_rb,6.2836e-05_rb/)
      kbo(:, 2,47, 2) = (/ &
     &1.2359e-01_rb,9.7799e-02_rb,6.6803e-02_rb,3.4586e-02_rb,6.7612e-05_rb/)
      kbo(:, 3,47, 2) = (/ &
     &1.3036e-01_rb,1.0330e-01_rb,7.0567e-02_rb,3.6497e-02_rb,7.2040e-05_rb/)
      kbo(:, 4,47, 2) = (/ &
     &1.3710e-01_rb,1.0912e-01_rb,7.4632e-02_rb,3.8567e-02_rb,7.6196e-05_rb/)
      kbo(:, 5,47, 2) = (/ &
     &1.4440e-01_rb,1.1580e-01_rb,7.9334e-02_rb,4.0981e-02_rb,8.0080e-05_rb/)
      kbo(:, 1,48, 2) = (/ &
     &1.1035e-01_rb,8.7581e-02_rb,5.9868e-02_rb,3.1032e-02_rb,4.9884e-05_rb/)
      kbo(:, 2,48, 2) = (/ &
     &1.1928e-01_rb,9.4310e-02_rb,6.4353e-02_rb,3.3267e-02_rb,5.4019e-05_rb/)
      kbo(:, 3,48, 2) = (/ &
     &1.2628e-01_rb,9.9898e-02_rb,6.8156e-02_rb,3.5187e-02_rb,5.7777e-05_rb/)
      kbo(:, 4,48, 2) = (/ &
     &1.3294e-01_rb,1.0557e-01_rb,7.2080e-02_rb,3.7168e-02_rb,6.1393e-05_rb/)
      kbo(:, 5,48, 2) = (/ &
     &1.3996e-01_rb,1.1191e-01_rb,7.6553e-02_rb,3.9465e-02_rb,6.4815e-05_rb/)
      kbo(:, 1,49, 2) = (/ &
     &1.0914e-01_rb,8.6684e-02_rb,5.9236e-02_rb,3.0679e-02_rb,3.9521e-05_rb/)
      kbo(:, 2,49, 2) = (/ &
     &1.1962e-01_rb,9.4485e-02_rb,6.4406e-02_rb,3.3233e-02_rb,4.3077e-05_rb/)
      kbo(:, 3,49, 2) = (/ &
     &1.2732e-01_rb,1.0058e-01_rb,6.8523e-02_rb,3.5308e-02_rb,4.6318e-05_rb/)
      kbo(:, 4,49, 2) = (/ &
     &1.3429e-01_rb,1.0636e-01_rb,7.2504e-02_rb,3.7320e-02_rb,4.9451e-05_rb/)
      kbo(:, 5,49, 2) = (/ &
     &1.4147e-01_rb,1.1268e-01_rb,7.6951e-02_rb,3.9591e-02_rb,5.2405e-05_rb/)
      kbo(:, 1,50, 2) = (/ &
     &1.0267e-01_rb,8.1582e-02_rb,5.5735e-02_rb,2.8850e-02_rb,3.1268e-05_rb/)
      kbo(:, 2,50, 2) = (/ &
     &1.1402e-01_rb,9.0165e-02_rb,6.1437e-02_rb,3.1671e-02_rb,3.4319e-05_rb/)
      kbo(:, 3,50, 2) = (/ &
     &1.2234e-01_rb,9.6602e-02_rb,6.5766e-02_rb,3.3840e-02_rb,3.7076e-05_rb/)
      kbo(:, 4,50, 2) = (/ &
     &1.2939e-01_rb,1.0230e-01_rb,6.9668e-02_rb,3.5813e-02_rb,3.9737e-05_rb/)
      kbo(:, 5,50, 2) = (/ &
     &1.3639e-01_rb,1.0837e-01_rb,7.3903e-02_rb,3.7967e-02_rb,4.2232e-05_rb/)
      kbo(:, 1,51, 2) = (/ &
     &9.4284e-02_rb,7.4953e-02_rb,5.1186e-02_rb,2.6482e-02_rb,2.4659e-05_rb/)
      kbo(:, 2,51, 2) = (/ &
     &1.0593e-01_rb,8.3948e-02_rb,5.7209e-02_rb,2.9486e-02_rb,2.7298e-05_rb/)
      kbo(:, 3,51, 2) = (/ &
     &1.1492e-01_rb,9.0727e-02_rb,6.1733e-02_rb,3.1740e-02_rb,2.9657e-05_rb/)
      kbo(:, 4,51, 2) = (/ &
     &1.2197e-01_rb,9.6371e-02_rb,6.5573e-02_rb,3.3673e-02_rb,3.1882e-05_rb/)
      kbo(:, 5,51, 2) = (/ &
     &1.2870e-01_rb,1.0212e-01_rb,6.9569e-02_rb,3.5694e-02_rb,3.4030e-05_rb/)
      kbo(:, 1,52, 2) = (/ &
     &8.8743e-02_rb,7.0542e-02_rb,4.8148e-02_rb,2.4885e-02_rb,1.9415e-05_rb/)
      kbo(:, 2,52, 2) = (/ &
     &1.0078e-01_rb,7.9976e-02_rb,5.4503e-02_rb,2.8085e-02_rb,2.1689e-05_rb/)
      kbo(:, 3,52, 2) = (/ &
     &1.1078e-01_rb,8.7454e-02_rb,5.9476e-02_rb,3.0551e-02_rb,2.3703e-05_rb/)
      kbo(:, 4,52, 2) = (/ &
     &1.1817e-01_rb,9.3325e-02_rb,6.3448e-02_rb,3.2545e-02_rb,2.5596e-05_rb/)
      kbo(:, 5,52, 2) = (/ &
     &1.2490e-01_rb,9.8940e-02_rb,6.7340e-02_rb,3.4514e-02_rb,2.7422e-05_rb/)
      kbo(:, 1,53, 2) = (/ &
     &8.6662e-02_rb,6.8864e-02_rb,4.6962e-02_rb,2.4236e-02_rb,1.5263e-05_rb/)
      kbo(:, 2,53, 2) = (/ &
     &9.9515e-02_rb,7.8952e-02_rb,5.3787e-02_rb,2.7692e-02_rb,1.7200e-05_rb/)
      kbo(:, 3,53, 2) = (/ &
     &1.1085e-01_rb,8.7554e-02_rb,5.9520e-02_rb,3.0542e-02_rb,1.8920e-05_rb/)
      kbo(:, 4,53, 2) = (/ &
     &1.1918e-01_rb,9.4022e-02_rb,6.3885e-02_rb,3.2726e-02_rb,2.0582e-05_rb/)
      kbo(:, 5,53, 2) = (/ &
     &1.2628e-01_rb,9.9818e-02_rb,6.7862e-02_rb,3.4747e-02_rb,2.2128e-05_rb/)
      kbo(:, 1,54, 2) = (/ &
     &7.3195e-02_rb,5.8286e-02_rb,3.9755e-02_rb,2.0520e-02_rb,1.1967e-05_rb/)
      kbo(:, 2,54, 2) = (/ &
     &8.4972e-02_rb,6.7544e-02_rb,4.6023e-02_rb,2.3702e-02_rb,1.3585e-05_rb/)
      kbo(:, 3,54, 2) = (/ &
     &9.5709e-02_rb,7.5855e-02_rb,5.1595e-02_rb,2.6489e-02_rb,1.5051e-05_rb/)
      kbo(:, 4,54, 2) = (/ &
     &1.0394e-01_rb,8.2096e-02_rb,5.5779e-02_rb,2.8575e-02_rb,1.6406e-05_rb/)
      kbo(:, 5,54, 2) = (/ &
     &1.1051e-01_rb,8.7391e-02_rb,5.9394e-02_rb,3.0393e-02_rb,1.7745e-05_rb/)
      kbo(:, 1,55, 2) = (/ &
     &5.6277e-02_rb,4.4947e-02_rb,3.0672e-02_rb,1.5850e-02_rb,9.3585e-06_rb/)
      kbo(:, 2,55, 2) = (/ &
     &6.6102e-02_rb,5.2729e-02_rb,3.5949e-02_rb,1.8531e-02_rb,1.0684e-05_rb/)
      kbo(:, 3,55, 2) = (/ &
     &7.5193e-02_rb,5.9877e-02_rb,4.0765e-02_rb,2.0959e-02_rb,1.1938e-05_rb/)
      kbo(:, 4,55, 2) = (/ &
     &8.2620e-02_rb,6.5458e-02_rb,4.4488e-02_rb,2.2798e-02_rb,1.3050e-05_rb/)
      kbo(:, 5,55, 2) = (/ &
     &8.8216e-02_rb,6.9936e-02_rb,4.7530e-02_rb,2.4316e-02_rb,1.4154e-05_rb/)
      kbo(:, 1,56, 2) = (/ &
     &4.3039e-02_rb,3.4453e-02_rb,2.3524e-02_rb,1.2170e-02_rb,7.2880e-06_rb/)
      kbo(:, 2,56, 2) = (/ &
     &5.1258e-02_rb,4.1038e-02_rb,2.7994e-02_rb,1.4446e-02_rb,8.3961e-06_rb/)
      kbo(:, 3,56, 2) = (/ &
     &5.8906e-02_rb,4.7098e-02_rb,3.2089e-02_rb,1.6519e-02_rb,9.4488e-06_rb/)
      kbo(:, 4,56, 2) = (/ &
     &6.5506e-02_rb,5.2094e-02_rb,3.5421e-02_rb,1.8161e-02_rb,1.0382e-05_rb/)
      kbo(:, 5,56, 2) = (/ &
     &7.0360e-02_rb,5.5940e-02_rb,3.8018e-02_rb,1.9445e-02_rb,1.1248e-05_rb/)
      kbo(:, 1,57, 2) = (/ &
     &3.2674e-02_rb,2.6237e-02_rb,1.7922e-02_rb,9.2805e-03_rb,5.6715e-06_rb/)
      kbo(:, 2,57, 2) = (/ &
     &3.9609e-02_rb,3.1835e-02_rb,2.1730e-02_rb,1.1223e-02_rb,6.5904e-06_rb/)
      kbo(:, 3,57, 2) = (/ &
     &4.6006e-02_rb,3.6922e-02_rb,2.5172e-02_rb,1.2973e-02_rb,7.4542e-06_rb/)
      kbo(:, 4,57, 2) = (/ &
     &5.1740e-02_rb,4.1352e-02_rb,2.8137e-02_rb,1.4436e-02_rb,8.2537e-06_rb/)
      kbo(:, 5,57, 2) = (/ &
     &5.6054e-02_rb,4.4686e-02_rb,3.0382e-02_rb,1.5541e-02_rb,8.9675e-06_rb/)
      kbo(:, 1,58, 2) = (/ &
     &6.5163e-03_rb,6.4215e-03_rb,5.6470e-03_rb,4.1032e-03_rb,4.4127e-06_rb/)
      kbo(:, 2,58, 2) = (/ &
     &8.0617e-03_rb,7.9508e-03_rb,6.9853e-03_rb,5.0619e-03_rb,5.1550e-06_rb/)
      kbo(:, 3,58, 2) = (/ &
     &9.4685e-03_rb,9.3297e-03_rb,8.1872e-03_rb,5.9187e-03_rb,5.8809e-06_rb/)
      kbo(:, 4,58, 2) = (/ &
     &1.0753e-02_rb,1.0568e-02_rb,9.2585e-03_rb,6.6629e-03_rb,6.5571e-06_rb/)
      kbo(:, 5,58, 2) = (/ &
     &1.1773e-02_rb,1.1514e-02_rb,1.0073e-02_rb,7.2198e-03_rb,7.1478e-06_rb/)
      kbo(:, 1,59, 2) = (/ &
     &6.0678e-03_rb,5.7484e-03_rb,4.7181e-03_rb,3.0625e-03_rb,3.5292e-06_rb/)
      kbo(:, 2,59, 2) = (/ &
     &7.5775e-03_rb,7.1843e-03_rb,5.8871e-03_rb,3.8076e-03_rb,4.1244e-06_rb/)
      kbo(:, 3,59, 2) = (/ &
     &8.9504e-03_rb,8.4793e-03_rb,6.9385e-03_rb,4.4706e-03_rb,4.7056e-06_rb/)
      kbo(:, 4,59, 2) = (/ &
     &1.0202e-02_rb,9.6400e-03_rb,7.8732e-03_rb,5.0416e-03_rb,5.2546e-06_rb/)
      kbo(:, 5,59, 2) = (/ &
     &1.1221e-02_rb,1.0539e-02_rb,8.5853e-03_rb,5.4711e-03_rb,5.7591e-06_rb/)
      kbo(:, 1,13, 3) = (/ &
     &1.3886e+02_rb,1.0424e+02_rb,6.9618e+01_rb,3.4998e+01_rb,6.8908e-02_rb/)
      kbo(:, 2,13, 3) = (/ &
     &1.3847e+02_rb,1.0394e+02_rb,6.9423e+01_rb,3.4905e+01_rb,7.3584e-02_rb/)
      kbo(:, 3,13, 3) = (/ &
     &1.3854e+02_rb,1.0400e+02_rb,6.9461e+01_rb,3.4924e+01_rb,7.8137e-02_rb/)
      kbo(:, 4,13, 3) = (/ &
     &1.4034e+02_rb,1.0534e+02_rb,7.0354e+01_rb,3.5368e+01_rb,8.2835e-02_rb/)
      kbo(:, 5,13, 3) = (/ &
     &1.4333e+02_rb,1.0759e+02_rb,7.1853e+01_rb,3.6116e+01_rb,8.7500e-02_rb/)
      kbo(:, 1,14, 3) = (/ &
     &7.1765e+01_rb,5.3918e+01_rb,3.6073e+01_rb,1.8228e+01_rb,5.9867e-02_rb/)
      kbo(:, 2,14, 3) = (/ &
     &7.1567e+01_rb,5.3771e+01_rb,3.5977e+01_rb,1.8183e+01_rb,6.3947e-02_rb/)
      kbo(:, 3,14, 3) = (/ &
     &7.1902e+01_rb,5.4021e+01_rb,3.6141e+01_rb,1.8266e+01_rb,6.8160e-02_rb/)
      kbo(:, 4,14, 3) = (/ &
     &7.3101e+01_rb,5.4919e+01_rb,3.6739e+01_rb,1.8569e+01_rb,7.2138e-02_rb/)
      kbo(:, 5,14, 3) = (/ &
     &7.4813e+01_rb,5.6202e+01_rb,3.7593e+01_rb,1.8992e+01_rb,7.6299e-02_rb/)
      kbo(:, 1,15, 3) = (/ &
     &3.7762e+01_rb,2.8418e+01_rb,1.9073e+01_rb,9.7323e+00_rb,5.1937e-02_rb/)
      kbo(:, 2,15, 3) = (/ &
     &3.7725e+01_rb,2.8390e+01_rb,1.9057e+01_rb,9.7303e+00_rb,5.5591e-02_rb/)
      kbo(:, 3,15, 3) = (/ &
     &3.8048e+01_rb,2.8634e+01_rb,1.9221e+01_rb,9.8224e+00_rb,5.9263e-02_rb/)
      kbo(:, 4,15, 3) = (/ &
     &3.8784e+01_rb,2.9182e+01_rb,1.9585e+01_rb,1.0010e+01_rb,6.2883e-02_rb/)
      kbo(:, 5,15, 3) = (/ &
     &3.9804e+01_rb,2.9945e+01_rb,2.0089e+01_rb,1.0260e+01_rb,6.6326e-02_rb/)
      kbo(:, 1,16, 3) = (/ &
     &2.3667e+01_rb,1.7845e+01_rb,1.2024e+01_rb,6.2150e+00_rb,4.4961e-02_rb/)
      kbo(:, 2,16, 3) = (/ &
     &2.3710e+01_rb,1.7881e+01_rb,1.2053e+01_rb,6.2413e+00_rb,4.8227e-02_rb/)
      kbo(:, 3,16, 3) = (/ &
     &2.3983e+01_rb,1.8085e+01_rb,1.2193e+01_rb,6.3263e+00_rb,5.1418e-02_rb/)
      kbo(:, 4,16, 3) = (/ &
     &2.4541e+01_rb,1.8499e+01_rb,1.2468e+01_rb,6.4708e+00_rb,5.4587e-02_rb/)
      kbo(:, 5,16, 3) = (/ &
     &2.5247e+01_rb,1.9026e+01_rb,1.2815e+01_rb,6.6457e+00_rb,5.7578e-02_rb/)
      kbo(:, 1,17, 3) = (/ &
     &1.5411e+01_rb,1.1654e+01_rb,7.9029e+00_rb,4.1290e+00_rb,3.8761e-02_rb/)
      kbo(:, 2,17, 3) = (/ &
     &1.5483e+01_rb,1.1712e+01_rb,7.9484e+00_rb,4.1656e+00_rb,4.1564e-02_rb/)
      kbo(:, 3,17, 3) = (/ &
     &1.5715e+01_rb,1.1885e+01_rb,8.0713e+00_rb,4.2451e+00_rb,4.4369e-02_rb/)
      kbo(:, 4,17, 3) = (/ &
     &1.6109e+01_rb,1.2177e+01_rb,8.2663e+00_rb,4.3534e+00_rb,4.7100e-02_rb/)
      kbo(:, 5,17, 3) = (/ &
     &1.6611e+01_rb,1.2551e+01_rb,8.5125e+00_rb,4.4825e+00_rb,4.9594e-02_rb/)
      kbo(:, 1,18, 3) = (/ &
     &1.0325e+01_rb,7.8390e+00_rb,5.3612e+00_rb,2.8294e+00_rb,3.3186e-02_rb/)
      kbo(:, 2,18, 3) = (/ &
     &1.0398e+01_rb,7.8984e+00_rb,5.4111e+00_rb,2.8675e+00_rb,3.5625e-02_rb/)
      kbo(:, 3,18, 3) = (/ &
     &1.0571e+01_rb,8.0297e+00_rb,5.5102e+00_rb,2.9332e+00_rb,3.8042e-02_rb/)
      kbo(:, 4,18, 3) = (/ &
     &1.0854e+01_rb,8.2395e+00_rb,5.6528e+00_rb,3.0140e+00_rb,4.0327e-02_rb/)
      kbo(:, 5,18, 3) = (/ &
     &1.1221e+01_rb,8.5111e+00_rb,5.8333e+00_rb,3.1124e+00_rb,4.2414e-02_rb/)
      kbo(:, 1,19, 3) = (/ &
     &6.8208e+00_rb,5.2153e+00_rb,3.5957e+00_rb,1.9227e+00_rb,2.8261e-02_rb/)
      kbo(:, 2,19, 3) = (/ &
     &6.8751e+00_rb,5.2621e+00_rb,3.6379e+00_rb,1.9560e+00_rb,3.0383e-02_rb/)
      kbo(:, 3,19, 3) = (/ &
     &7.0006e+00_rb,5.3588e+00_rb,3.7145e+00_rb,2.0054e+00_rb,3.2394e-02_rb/)
      kbo(:, 4,19, 3) = (/ &
     &7.2016e+00_rb,5.5069e+00_rb,3.8180e+00_rb,2.0649e+00_rb,3.4293e-02_rb/)
      kbo(:, 5,19, 3) = (/ &
     &7.4662e+00_rb,5.7018e+00_rb,3.9518e+00_rb,2.1403e+00_rb,3.6047e-02_rb/)
      kbo(:, 1,20, 3) = (/ &
     &4.8794e+00_rb,3.7616e+00_rb,2.6099e+00_rb,1.4082e+00_rb,2.4010e-02_rb/)
      kbo(:, 2,20, 3) = (/ &
     &4.9222e+00_rb,3.8008e+00_rb,2.6471e+00_rb,1.4364e+00_rb,2.5795e-02_rb/)
      kbo(:, 3,20, 3) = (/ &
     &5.0241e+00_rb,3.8811e+00_rb,2.7106e+00_rb,1.4758e+00_rb,2.7487e-02_rb/)
      kbo(:, 4,20, 3) = (/ &
     &5.1822e+00_rb,3.9983e+00_rb,2.7942e+00_rb,1.5252e+00_rb,2.9078e-02_rb/)
      kbo(:, 5,20, 3) = (/ &
     &5.3938e+00_rb,4.1543e+00_rb,2.9036e+00_rb,1.5872e+00_rb,3.0457e-02_rb/)
      kbo(:, 1,21, 3) = (/ &
     &3.5976e+00_rb,2.7957e+00_rb,1.9525e+00_rb,1.0602e+00_rb,2.0310e-02_rb/)
      kbo(:, 2,21, 3) = (/ &
     &3.6356e+00_rb,2.8321e+00_rb,1.9867e+00_rb,1.0846e+00_rb,2.1803e-02_rb/)
      kbo(:, 3,21, 3) = (/ &
     &3.7222e+00_rb,2.9011e+00_rb,2.0401e+00_rb,1.1175e+00_rb,2.3227e-02_rb/)
      kbo(:, 4,21, 3) = (/ &
     &3.8497e+00_rb,2.9972e+00_rb,2.1098e+00_rb,1.1593e+00_rb,2.4505e-02_rb/)
      kbo(:, 5,21, 3) = (/ &
     &4.0214e+00_rb,3.1255e+00_rb,2.2004e+00_rb,1.2111e+00_rb,2.5550e-02_rb/)
      kbo(:, 1,22, 3) = (/ &
     &2.7203e+00_rb,2.1314e+00_rb,1.4964e+00_rb,8.1782e-01_rb,1.7205e-02_rb/)
      kbo(:, 2,22, 3) = (/ &
     &2.7584e+00_rb,2.1682e+00_rb,1.5287e+00_rb,8.3963e-01_rb,1.8458e-02_rb/)
      kbo(:, 3,22, 3) = (/ &
     &2.8354e+00_rb,2.2296e+00_rb,1.5749e+00_rb,8.6850e-01_rb,1.9598e-02_rb/)
      kbo(:, 4,22, 3) = (/ &
     &2.9415e+00_rb,2.3111e+00_rb,1.6346e+00_rb,9.0426e-01_rb,2.0559e-02_rb/)
      kbo(:, 5,22, 3) = (/ &
     &3.0867e+00_rb,2.4212e+00_rb,1.7134e+00_rb,9.4960e-01_rb,2.1311e-02_rb/)
      kbo(:, 1,23, 3) = (/ &
     &2.0695e+00_rb,1.6369e+00_rb,1.1532e+00_rb,6.3485e-01_rb,1.4544e-02_rb/)
      kbo(:, 2,23, 3) = (/ &
     &2.1080e+00_rb,1.6730e+00_rb,1.1833e+00_rb,6.5423e-01_rb,1.5557e-02_rb/)
      kbo(:, 3,23, 3) = (/ &
     &2.1745e+00_rb,1.7262e+00_rb,1.2233e+00_rb,6.7947e-01_rb,1.6419e-02_rb/)
      kbo(:, 4,23, 3) = (/ &
     &2.2647e+00_rb,1.7970e+00_rb,1.2752e+00_rb,7.1089e-01_rb,1.7120e-02_rb/)
      kbo(:, 5,23, 3) = (/ &
     &2.3823e+00_rb,1.8879e+00_rb,1.3412e+00_rb,7.4903e-01_rb,1.7674e-02_rb/)
      kbo(:, 1,24, 3) = (/ &
     &1.5697e+00_rb,1.2531e+00_rb,8.8637e-01_rb,4.9139e-01_rb,1.2257e-02_rb/)
      kbo(:, 2,24, 3) = (/ &
     &1.6055e+00_rb,1.2864e+00_rb,9.1332e-01_rb,5.0866e-01_rb,1.3031e-02_rb/)
      kbo(:, 3,24, 3) = (/ &
     &1.6631e+00_rb,1.3329e+00_rb,9.4834e-01_rb,5.3086e-01_rb,1.3670e-02_rb/)
      kbo(:, 4,24, 3) = (/ &
     &1.7374e+00_rb,1.3923e+00_rb,9.9245e-01_rb,5.5764e-01_rb,1.4194e-02_rb/)
      kbo(:, 5,24, 3) = (/ &
     &1.8327e+00_rb,1.4671e+00_rb,1.0467e+00_rb,5.8890e-01_rb,1.4611e-02_rb/)
      kbo(:, 1,25, 3) = (/ &
     &1.2086e+00_rb,9.7309e-01_rb,6.9155e-01_rb,3.8568e-01_rb,1.0281e-02_rb/)
      kbo(:, 2,25, 3) = (/ &
     &1.2420e+00_rb,1.0035e+00_rb,7.1528e-01_rb,4.0109e-01_rb,1.0857e-02_rb/)
      kbo(:, 3,25, 3) = (/ &
     &1.2906e+00_rb,1.0433e+00_rb,7.4566e-01_rb,4.2042e-01_rb,1.1339e-02_rb/)
      kbo(:, 4,25, 3) = (/ &
     &1.3533e+00_rb,1.0941e+00_rb,7.8339e-01_rb,4.4308e-01_rb,1.1737e-02_rb/)
      kbo(:, 5,25, 3) = (/ &
     &1.4315e+00_rb,1.1563e+00_rb,8.2868e-01_rb,4.6855e-01_rb,1.2054e-02_rb/)
      kbo(:, 1,26, 3) = (/ &
     &9.4867e-01_rb,7.6987e-01_rb,5.4929e-01_rb,3.0788e-01_rb,8.5827e-03_rb/)
      kbo(:, 2,26, 3) = (/ &
     &9.7957e-01_rb,7.9721e-01_rb,5.7050e-01_rb,3.2189e-01_rb,9.0225e-03_rb/)
      kbo(:, 3,26, 3) = (/ &
     &1.0209e+00_rb,8.3198e-01_rb,5.9699e-01_rb,3.3862e-01_rb,9.3908e-03_rb/)
      kbo(:, 4,26, 3) = (/ &
     &1.0742e+00_rb,8.7561e-01_rb,6.2951e-01_rb,3.5776e-01_rb,9.6952e-03_rb/)
      kbo(:, 5,26, 3) = (/ &
     &1.1410e+00_rb,9.2907e-01_rb,6.6837e-01_rb,3.7907e-01_rb,9.9345e-03_rb/)
      kbo(:, 1,27, 3) = (/ &
     &7.7558e-01_rb,6.3324e-01_rb,4.5279e-01_rb,2.5468e-01_rb,7.1414e-03_rb/)
      kbo(:, 2,27, 3) = (/ &
     &8.0374e-01_rb,6.5793e-01_rb,4.7219e-01_rb,2.6746e-01_rb,7.4832e-03_rb/)
      kbo(:, 3,27, 3) = (/ &
     &8.4002e-01_rb,6.8874e-01_rb,4.9567e-01_rb,2.8203e-01_rb,7.7674e-03_rb/)
      kbo(:, 4,27, 3) = (/ &
     &8.8798e-01_rb,7.2827e-01_rb,5.2483e-01_rb,2.9866e-01_rb,7.9993e-03_rb/)
      kbo(:, 5,27, 3) = (/ &
     &9.4670e-01_rb,7.7585e-01_rb,5.5890e-01_rb,3.1722e-01_rb,8.1810e-03_rb/)
      kbo(:, 1,28, 3) = (/ &
     &6.5247e-01_rb,5.3505e-01_rb,3.8309e-01_rb,2.1606e-01_rb,5.9313e-03_rb/)
      kbo(:, 2,28, 3) = (/ &
     &6.7805e-01_rb,5.5765e-01_rb,4.0088e-01_rb,2.2763e-01_rb,6.1964e-03_rb/)
      kbo(:, 3,28, 3) = (/ &
     &7.1138e-01_rb,5.8602e-01_rb,4.2234e-01_rb,2.4051e-01_rb,6.4170e-03_rb/)
      kbo(:, 4,28, 3) = (/ &
     &7.5565e-01_rb,6.2271e-01_rb,4.4913e-01_rb,2.5531e-01_rb,6.5956e-03_rb/)
      kbo(:, 5,28, 3) = (/ &
     &8.0810e-01_rb,6.6536e-01_rb,4.7931e-01_rb,2.7186e-01_rb,6.7356e-03_rb/)
      kbo(:, 1,29, 3) = (/ &
     &5.8359e-01_rb,4.7922e-01_rb,3.4299e-01_rb,1.9330e-01_rb,4.9222e-03_rb/)
      kbo(:, 2,29, 3) = (/ &
     &6.0836e-01_rb,5.0105e-01_rb,3.5992e-01_rb,2.0402e-01_rb,5.1279e-03_rb/)
      kbo(:, 3,29, 3) = (/ &
     &6.4169e-01_rb,5.2929e-01_rb,3.8096e-01_rb,2.1613e-01_rb,5.2989e-03_rb/)
      kbo(:, 4,29, 3) = (/ &
     &6.8418e-01_rb,5.6479e-01_rb,4.0659e-01_rb,2.3010e-01_rb,5.4376e-03_rb/)
      kbo(:, 5,29, 3) = (/ &
     &7.3388e-01_rb,6.0487e-01_rb,4.3487e-01_rb,2.4554e-01_rb,5.5459e-03_rb/)
      kbo(:, 1,30, 3) = (/ &
     &5.3559e-01_rb,4.3987e-01_rb,3.1446e-01_rb,1.7681e-01_rb,4.0812e-03_rb/)
      kbo(:, 2,30, 3) = (/ &
     &5.6092e-01_rb,4.6195e-01_rb,3.3123e-01_rb,1.8699e-01_rb,4.2410e-03_rb/)
      kbo(:, 3,30, 3) = (/ &
     &5.9467e-01_rb,4.9061e-01_rb,3.5234e-01_rb,1.9880e-01_rb,4.3756e-03_rb/)
      kbo(:, 4,30, 3) = (/ &
     &6.3620e-01_rb,5.2529e-01_rb,3.7692e-01_rb,2.1218e-01_rb,4.4820e-03_rb/)
      kbo(:, 5,30, 3) = (/ &
     &6.8416e-01_rb,5.6390e-01_rb,4.0420e-01_rb,2.2692e-01_rb,4.5637e-03_rb/)
      kbo(:, 1,31, 3) = (/ &
     &5.1816e-01_rb,4.2464e-01_rb,3.0261e-01_rb,1.6922e-01_rb,3.3825e-03_rb/)
      kbo(:, 2,31, 3) = (/ &
     &5.4536e-01_rb,4.4814e-01_rb,3.2013e-01_rb,1.7945e-01_rb,3.5084e-03_rb/)
      kbo(:, 3,31, 3) = (/ &
     &5.8079e-01_rb,4.7822e-01_rb,3.4203e-01_rb,1.9145e-01_rb,3.6133e-03_rb/)
      kbo(:, 4,31, 3) = (/ &
     &6.2360e-01_rb,5.1355e-01_rb,3.6685e-01_rb,2.0495e-01_rb,3.6964e-03_rb/)
      kbo(:, 5,31, 3) = (/ &
     &6.7136e-01_rb,5.5242e-01_rb,3.9433e-01_rb,2.1968e-01_rb,3.7632e-03_rb/)
      kbo(:, 1,32, 3) = (/ &
     &5.0836e-01_rb,4.1565e-01_rb,2.9513e-01_rb,1.6401e-01_rb,2.8023e-03_rb/)
      kbo(:, 2,32, 3) = (/ &
     &5.3755e-01_rb,4.4081e-01_rb,3.1373e-01_rb,1.7458e-01_rb,2.9031e-03_rb/)
      kbo(:, 3,32, 3) = (/ &
     &5.7518e-01_rb,4.7260e-01_rb,3.3647e-01_rb,1.8685e-01_rb,2.9841e-03_rb/)
      kbo(:, 4,32, 3) = (/ &
     &6.1917e-01_rb,5.0863e-01_rb,3.6188e-01_rb,2.0060e-01_rb,3.0523e-03_rb/)
      kbo(:, 5,32, 3) = (/ &
     &6.6737e-01_rb,5.4799e-01_rb,3.8954e-01_rb,2.1543e-01_rb,3.1032e-03_rb/)
      kbo(:, 1,33, 3) = (/ &
     &5.0907e-01_rb,4.1513e-01_rb,2.9365e-01_rb,1.6205e-01_rb,2.3233e-03_rb/)
      kbo(:, 2,33, 3) = (/ &
     &5.4095e-01_rb,4.4243e-01_rb,3.1363e-01_rb,1.7315e-01_rb,2.4017e-03_rb/)
      kbo(:, 3,33, 3) = (/ &
     &5.8099e-01_rb,4.7617e-01_rb,3.3742e-01_rb,1.8595e-01_rb,2.4676e-03_rb/)
      kbo(:, 4,33, 3) = (/ &
     &6.2648e-01_rb,5.1328e-01_rb,3.6375e-01_rb,2.0009e-01_rb,2.5196e-03_rb/)
      kbo(:, 5,33, 3) = (/ &
     &6.7646e-01_rb,5.5394e-01_rb,3.9218e-01_rb,2.1533e-01_rb,2.5609e-03_rb/)
      kbo(:, 1,34, 3) = (/ &
     &4.9704e-01_rb,4.0480e-01_rb,2.8555e-01_rb,1.5673e-01_rb,1.9225e-03_rb/)
      kbo(:, 2,34, 3) = (/ &
     &5.3069e-01_rb,4.3364e-01_rb,3.0643e-01_rb,1.6807e-01_rb,1.9847e-03_rb/)
      kbo(:, 3,34, 3) = (/ &
     &5.7151e-01_rb,4.6776e-01_rb,3.3046e-01_rb,1.8101e-01_rb,2.0370e-03_rb/)
      kbo(:, 4,34, 3) = (/ &
     &6.1711e-01_rb,5.0510e-01_rb,3.5686e-01_rb,1.9508e-01_rb,2.0801e-03_rb/)
      kbo(:, 5,34, 3) = (/ &
     &6.6746e-01_rb,5.4585e-01_rb,3.8550e-01_rb,2.1045e-01_rb,2.1144e-03_rb/)
      kbo(:, 1,35, 3) = (/ &
     &4.8386e-01_rb,3.9362e-01_rb,2.7689e-01_rb,1.5116e-01_rb,1.5847e-03_rb/)
      kbo(:, 2,35, 3) = (/ &
     &5.1832e-01_rb,4.2323e-01_rb,2.9822e-01_rb,1.6263e-01_rb,1.6362e-03_rb/)
      kbo(:, 3,35, 3) = (/ &
     &5.5929e-01_rb,4.5721e-01_rb,3.2219e-01_rb,1.7550e-01_rb,1.6795e-03_rb/)
      kbo(:, 4,35, 3) = (/ &
     &6.0482e-01_rb,4.9463e-01_rb,3.4852e-01_rb,1.8949e-01_rb,1.7156e-03_rb/)
      kbo(:, 5,35, 3) = (/ &
     &6.5512e-01_rb,5.3534e-01_rb,3.7729e-01_rb,2.0487e-01_rb,1.7410e-03_rb/)
      kbo(:, 1,36, 3) = (/ &
     &4.6553e-01_rb,3.7836e-01_rb,2.6548e-01_rb,1.4421e-01_rb,1.3024e-03_rb/)
      kbo(:, 2,36, 3) = (/ &
     &4.9953e-01_rb,4.0773e-01_rb,2.8657e-01_rb,1.5552e-01_rb,1.3449e-03_rb/)
      kbo(:, 3,36, 3) = (/ &
     &5.3980e-01_rb,4.4109e-01_rb,3.1015e-01_rb,1.6814e-01_rb,1.3813e-03_rb/)
      kbo(:, 4,36, 3) = (/ &
     &5.8461e-01_rb,4.7793e-01_rb,3.3603e-01_rb,1.8187e-01_rb,1.4105e-03_rb/)
      kbo(:, 5,36, 3) = (/ &
     &6.3401e-01_rb,5.1804e-01_rb,3.6447e-01_rb,1.9702e-01_rb,1.4329e-03_rb/)
      kbo(:, 1,37, 3) = (/ &
     &4.3247e-01_rb,3.5136e-01_rb,2.4603e-01_rb,1.3313e-01_rb,1.0644e-03_rb/)
      kbo(:, 2,37, 3) = (/ &
     &4.6409e-01_rb,3.7893e-01_rb,2.6588e-01_rb,1.4378e-01_rb,1.1011e-03_rb/)
      kbo(:, 3,37, 3) = (/ &
     &5.0193e-01_rb,4.1040e-01_rb,2.8814e-01_rb,1.5568e-01_rb,1.1313e-03_rb/)
      kbo(:, 4,37, 3) = (/ &
     &5.4423e-01_rb,4.4538e-01_rb,3.1275e-01_rb,1.6870e-01_rb,1.1563e-03_rb/)
      kbo(:, 5,37, 3) = (/ &
     &5.9125e-01_rb,4.8359e-01_rb,3.3982e-01_rb,1.8310e-01_rb,1.1762e-03_rb/)
      kbo(:, 1,38, 3) = (/ &
     &4.0752e-01_rb,3.3084e-01_rb,2.3116e-01_rb,1.2457e-01_rb,8.6985e-04_rb/)
      kbo(:, 2,38, 3) = (/ &
     &4.3734e-01_rb,3.5701e-01_rb,2.5006e-01_rb,1.3472e-01_rb,9.0011e-04_rb/)
      kbo(:, 3,38, 3) = (/ &
     &4.7342e-01_rb,3.8717e-01_rb,2.7137e-01_rb,1.4611e-01_rb,9.2699e-04_rb/)
      kbo(:, 4,38, 3) = (/ &
     &5.1398e-01_rb,4.2080e-01_rb,2.9507e-01_rb,1.5862e-01_rb,9.4795e-04_rb/)
      kbo(:, 5,38, 3) = (/ &
     &5.5926e-01_rb,4.5769e-01_rb,3.2123e-01_rb,1.7252e-01_rb,9.6539e-04_rb/)
      kbo(:, 1,39, 3) = (/ &
     &3.9286e-01_rb,3.1853e-01_rb,2.2204e-01_rb,1.1910e-01_rb,7.1027e-04_rb/)
      kbo(:, 2,39, 3) = (/ &
     &4.2168e-01_rb,3.4392e-01_rb,2.4040e-01_rb,1.2899e-01_rb,7.3606e-04_rb/)
      kbo(:, 3,39, 3) = (/ &
     &4.5685e-01_rb,3.7344e-01_rb,2.6129e-01_rb,1.4014e-01_rb,7.5956e-04_rb/)
      kbo(:, 4,39, 3) = (/ &
     &4.9667e-01_rb,4.0652e-01_rb,2.8462e-01_rb,1.5245e-01_rb,7.7770e-04_rb/)
      kbo(:, 5,39, 3) = (/ &
     &5.4133e-01_rb,4.4296e-01_rb,3.1045e-01_rb,1.6619e-01_rb,7.9334e-04_rb/)
      kbo(:, 1,40, 3) = (/ &
     &3.5355e-01_rb,2.8669e-01_rb,1.9956e-01_rb,1.0673e-01_rb,5.7721e-04_rb/)
      kbo(:, 2,40, 3) = (/ &
     &3.7907e-01_rb,3.0944e-01_rb,2.1608e-01_rb,1.1569e-01_rb,5.9933e-04_rb/)
      kbo(:, 3,40, 3) = (/ &
     &4.1075e-01_rb,3.3635e-01_rb,2.3514e-01_rb,1.2587e-01_rb,6.1943e-04_rb/)
      kbo(:, 4,40, 3) = (/ &
     &4.4707e-01_rb,3.6668e-01_rb,2.5658e-01_rb,1.3717e-01_rb,6.3526e-04_rb/)
      kbo(:, 5,40, 3) = (/ &
     &4.8803e-01_rb,4.0025e-01_rb,2.8039e-01_rb,1.4981e-01_rb,6.4905e-04_rb/)
      kbo(:, 1,41, 3) = (/ &
     &3.1722e-01_rb,2.5721e-01_rb,1.7878e-01_rb,9.5352e-02_rb,4.6830e-04_rb/)
      kbo(:, 2,41, 3) = (/ &
     &3.3984e-01_rb,2.7762e-01_rb,1.9366e-01_rb,1.0349e-01_rb,4.8763e-04_rb/)
      kbo(:, 3,41, 3) = (/ &
     &3.6801e-01_rb,3.0195e-01_rb,2.1092e-01_rb,1.1268e-01_rb,5.0461e-04_rb/)
      kbo(:, 4,41, 3) = (/ &
     &4.0101e-01_rb,3.2964e-01_rb,2.3053e-01_rb,1.2303e-01_rb,5.1866e-04_rb/)
      kbo(:, 5,41, 3) = (/ &
     &4.3854e-01_rb,3.6057e-01_rb,2.5249e-01_rb,1.3467e-01_rb,5.3060e-04_rb/)
      kbo(:, 1,42, 3) = (/ &
     &2.8780e-01_rb,2.3320e-01_rb,1.6182e-01_rb,8.6050e-02_rb,3.7987e-04_rb/)
      kbo(:, 2,42, 3) = (/ &
     &3.0806e-01_rb,2.5171e-01_rb,1.7535e-01_rb,9.3494e-02_rb,3.9649e-04_rb/)
      kbo(:, 3,42, 3) = (/ &
     &3.3326e-01_rb,2.7383e-01_rb,1.9111e-01_rb,1.0188e-01_rb,4.1103e-04_rb/)
      kbo(:, 4,42, 3) = (/ &
     &3.6352e-01_rb,2.9931e-01_rb,2.0921e-01_rb,1.1145e-01_rb,4.2329e-04_rb/)
      kbo(:, 5,42, 3) = (/ &
     &3.9814e-01_rb,3.2805e-01_rb,2.2962e-01_rb,1.2223e-01_rb,4.3386e-04_rb/)
      kbo(:, 1,43, 3) = (/ &
     &2.6265e-01_rb,2.1245e-01_rb,1.4714e-01_rb,7.7969e-02_rb,3.0714e-04_rb/)
      kbo(:, 2,43, 3) = (/ &
     &2.8068e-01_rb,2.2918e-01_rb,1.5940e-01_rb,8.4750e-02_rb,3.2151e-04_rb/)
      kbo(:, 3,43, 3) = (/ &
     &3.0314e-01_rb,2.4932e-01_rb,1.7384e-01_rb,9.2466e-02_rb,3.3415e-04_rb/)
      kbo(:, 4,43, 3) = (/ &
     &3.3080e-01_rb,2.7269e-01_rb,1.9045e-01_rb,1.0127e-01_rb,3.4470e-04_rb/)
      kbo(:, 5,43, 3) = (/ &
     &3.6272e-01_rb,2.9942e-01_rb,2.0942e-01_rb,1.1129e-01_rb,3.5407e-04_rb/)
      kbo(:, 1,44, 3) = (/ &
     &2.4281e-01_rb,1.9594e-01_rb,1.3540e-01_rb,7.1467e-02_rb,2.4789e-04_rb/)
      kbo(:, 2,44, 3) = (/ &
     &2.5884e-01_rb,2.1103e-01_rb,1.4648e-01_rb,7.7626e-02_rb,2.6014e-04_rb/)
      kbo(:, 3,44, 3) = (/ &
     &2.7897e-01_rb,2.2939e-01_rb,1.5978e-01_rb,8.4794e-02_rb,2.7117e-04_rb/)
      kbo(:, 4,44, 3) = (/ &
     &3.0448e-01_rb,2.5115e-01_rb,1.7522e-01_rb,9.2989e-02_rb,2.8049e-04_rb/)
      kbo(:, 5,44, 3) = (/ &
     &3.3408e-01_rb,2.7612e-01_rb,1.9297e-01_rb,1.0236e-01_rb,2.8861e-04_rb/)
      kbo(:, 1,45, 3) = (/ &
     &2.2924e-01_rb,1.8447e-01_rb,1.2716e-01_rb,6.6832e-02_rb,2.0004e-04_rb/)
      kbo(:, 2,45, 3) = (/ &
     &2.4373e-01_rb,1.9820e-01_rb,1.3728e-01_rb,7.2484e-02_rb,2.1065e-04_rb/)
      kbo(:, 3,45, 3) = (/ &
     &2.6228e-01_rb,2.1536e-01_rb,1.4978e-01_rb,7.9297e-02_rb,2.1996e-04_rb/)
      kbo(:, 4,45, 3) = (/ &
     &2.8578e-01_rb,2.3573e-01_rb,1.6427e-01_rb,8.6976e-02_rb,2.2795e-04_rb/)
      kbo(:, 5,45, 3) = (/ &
     &3.1387e-01_rb,2.5954e-01_rb,1.8123e-01_rb,9.5940e-02_rb,2.3501e-04_rb/)
      kbo(:, 1,46, 3) = (/ &
     &2.1864e-01_rb,1.7537e-01_rb,1.2059e-01_rb,6.3114e-02_rb,1.6106e-04_rb/)
      kbo(:, 2,46, 3) = (/ &
     &2.3164e-01_rb,1.8773e-01_rb,1.2971e-01_rb,6.8234e-02_rb,1.7008e-04_rb/)
      kbo(:, 3,46, 3) = (/ &
     &2.4878e-01_rb,2.0380e-01_rb,1.4148e-01_rb,7.4690e-02_rb,1.7817e-04_rb/)
      kbo(:, 4,46, 3) = (/ &
     &2.7041e-01_rb,2.2298e-01_rb,1.5518e-01_rb,8.1986e-02_rb,1.8514e-04_rb/)
      kbo(:, 5,46, 3) = (/ &
     &2.9717e-01_rb,2.4573e-01_rb,1.7143e-01_rb,9.0585e-02_rb,1.9112e-04_rb/)
      kbo(:, 1,47, 3) = (/ &
     &2.0666e-01_rb,1.6515e-01_rb,1.1329e-01_rb,5.9056e-02_rb,1.2901e-04_rb/)
      kbo(:, 2,47, 3) = (/ &
     &2.1779e-01_rb,1.7589e-01_rb,1.2122e-01_rb,6.3515e-02_rb,1.3680e-04_rb/)
      kbo(:, 3,47, 3) = (/ &
     &2.3320e-01_rb,1.9053e-01_rb,1.3198e-01_rb,6.9466e-02_rb,1.4384e-04_rb/)
      kbo(:, 4,47, 3) = (/ &
     &2.5274e-01_rb,2.0828e-01_rb,1.4478e-01_rb,7.6351e-02_rb,1.4999e-04_rb/)
      kbo(:, 5,47, 3) = (/ &
     &2.7761e-01_rb,2.2954e-01_rb,1.5995e-01_rb,8.4385e-02_rb,1.5514e-04_rb/)
      kbo(:, 1,48, 3) = (/ &
     &2.0159e-01_rb,1.6032e-01_rb,1.0970e-01_rb,5.6945e-02_rb,1.0330e-04_rb/)
      kbo(:, 2,48, 3) = (/ &
     &2.1094e-01_rb,1.6966e-01_rb,1.1661e-01_rb,6.0834e-02_rb,1.0997e-04_rb/)
      kbo(:, 3,48, 3) = (/ &
     &2.2505e-01_rb,1.8313e-01_rb,1.2656e-01_rb,6.6365e-02_rb,1.1606e-04_rb/)
      kbo(:, 4,48, 3) = (/ &
     &2.4321e-01_rb,1.9993e-01_rb,1.3877e-01_rb,7.3027e-02_rb,1.2143e-04_rb/)
      kbo(:, 5,48, 3) = (/ &
     &2.6669e-01_rb,2.2023e-01_rb,1.5323e-01_rb,8.0709e-02_rb,1.2582e-04_rb/)
      kbo(:, 1,49, 3) = (/ &
     &2.0550e-01_rb,1.6255e-01_rb,1.1090e-01_rb,5.7322e-02_rb,8.2624e-05_rb/)
      kbo(:, 2,49, 3) = (/ &
     &2.1330e-01_rb,1.7068e-01_rb,1.1698e-01_rb,6.0751e-02_rb,8.8378e-05_rb/)
      kbo(:, 3,49, 3) = (/ &
     &2.2651e-01_rb,1.8330e-01_rb,1.2630e-01_rb,6.5972e-02_rb,9.3615e-05_rb/)
      kbo(:, 4,49, 3) = (/ &
     &2.4409e-01_rb,1.9975e-01_rb,1.3835e-01_rb,7.2591e-02_rb,9.8246e-05_rb/)
      kbo(:, 5,49, 3) = (/ &
     &2.6685e-01_rb,2.1980e-01_rb,1.5269e-01_rb,8.0258e-02_rb,1.0201e-04_rb/)
      kbo(:, 1,50, 3) = (/ &
     &2.0001e-01_rb,1.5763e-01_rb,1.0735e-01_rb,5.5313e-02_rb,6.6052e-05_rb/)
      kbo(:, 2,50, 3) = (/ &
     &2.0622e-01_rb,1.6433e-01_rb,1.1239e-01_rb,5.8166e-02_rb,7.0964e-05_rb/)
      kbo(:, 3,50, 3) = (/ &
     &2.1782e-01_rb,1.7560e-01_rb,1.2070e-01_rb,6.2821e-02_rb,7.5460e-05_rb/)
      kbo(:, 4,50, 3) = (/ &
     &2.3393e-01_rb,1.9090e-01_rb,1.3196e-01_rb,6.9056e-02_rb,7.9402e-05_rb/)
      kbo(:, 5,50, 3) = (/ &
     &2.5520e-01_rb,2.0998e-01_rb,1.4571e-01_rb,7.6464e-02_rb,8.2692e-05_rb/)
      kbo(:, 1,51, 3) = (/ &
     &1.9065e-01_rb,1.4985e-01_rb,1.0188e-01_rb,5.2372e-02_rb,5.2796e-05_rb/)
      kbo(:, 2,51, 3) = (/ &
     &1.9564e-01_rb,1.5535e-01_rb,1.0604e-01_rb,5.4717e-02_rb,5.6909e-05_rb/)
      kbo(:, 3,51, 3) = (/ &
     &2.0531e-01_rb,1.6500e-01_rb,1.1319e-01_rb,5.8726e-02_rb,6.0736e-05_rb/)
      kbo(:, 4,51, 3) = (/ &
     &2.1974e-01_rb,1.7886e-01_rb,1.2342e-01_rb,6.4424e-02_rb,6.4172e-05_rb/)
      kbo(:, 5,51, 3) = (/ &
     &2.3909e-01_rb,1.9652e-01_rb,1.3622e-01_rb,7.1393e-02_rb,6.6971e-05_rb/)
      kbo(:, 1,52, 3) = (/ &
     &1.8687e-01_rb,1.4652e-01_rb,9.9463e-02_rb,5.1012e-02_rb,4.2174e-05_rb/)
      kbo(:, 2,52, 3) = (/ &
     &1.9141e-01_rb,1.5137e-01_rb,1.0312e-01_rb,5.3053e-02_rb,4.5594e-05_rb/)
      kbo(:, 3,52, 3) = (/ &
     &1.9940e-01_rb,1.5962e-01_rb,1.0927e-01_rb,5.6516e-02_rb,4.8848e-05_rb/)
      kbo(:, 4,52, 3) = (/ &
     &2.1252e-01_rb,1.7228e-01_rb,1.1865e-01_rb,6.1768e-02_rb,5.1771e-05_rb/)
      kbo(:, 5,52, 3) = (/ &
     &2.3051e-01_rb,1.8896e-01_rb,1.3079e-01_rb,6.8431e-02_rb,5.4234e-05_rb/)
      kbo(:, 1,53, 3) = (/ &
     &1.9049e-01_rb,1.4907e-01_rb,1.0107e-01_rb,5.1727e-02_rb,3.3675e-05_rb/)
      kbo(:, 2,53, 3) = (/ &
     &1.9537e-01_rb,1.5387e-01_rb,1.0462e-01_rb,5.3675e-02_rb,3.6498e-05_rb/)
      kbo(:, 3,53, 3) = (/ &
     &2.0221e-01_rb,1.6109e-01_rb,1.1002e-01_rb,5.6715e-02_rb,3.9286e-05_rb/)
      kbo(:, 4,53, 3) = (/ &
     &2.1438e-01_rb,1.7292e-01_rb,1.1878e-01_rb,6.1645e-02_rb,4.1745e-05_rb/)
      kbo(:, 5,53, 3) = (/ &
     &2.3164e-01_rb,1.8913e-01_rb,1.3065e-01_rb,6.8198e-02_rb,4.3903e-05_rb/)
      kbo(:, 1,54, 3) = (/ &
     &1.6750e-01_rb,1.3117e-01_rb,8.8914e-02_rb,4.5482e-02_rb,2.6834e-05_rb/)
      kbo(:, 2,54, 3) = (/ &
     &1.7260e-01_rb,1.3577e-01_rb,9.2225e-02_rb,4.7257e-02_rb,2.9152e-05_rb/)
      kbo(:, 3,54, 3) = (/ &
     &1.7790e-01_rb,1.4144e-01_rb,9.6473e-02_rb,4.9632e-02_rb,3.1487e-05_rb/)
      kbo(:, 4,54, 3) = (/ &
     &1.8750e-01_rb,1.5107e-01_rb,1.0362e-01_rb,5.3663e-02_rb,3.3610e-05_rb/)
      kbo(:, 5,54, 3) = (/ &
     &2.0185e-01_rb,1.6484e-01_rb,1.1376e-01_rb,5.9292e-02_rb,3.5416e-05_rb/)
      kbo(:, 1,55, 3) = (/ &
     &1.3399e-01_rb,1.0522e-01_rb,7.1343e-02_rb,3.6503e-02_rb,2.1364e-05_rb/)
      kbo(:, 2,55, 3) = (/ &
     &1.3914e-01_rb,1.0951e-01_rb,7.4357e-02_rb,3.8080e-02_rb,2.3275e-05_rb/)
      kbo(:, 3,55, 3) = (/ &
     &1.4309e-01_rb,1.1374e-01_rb,7.7519e-02_rb,3.9837e-02_rb,2.5155e-05_rb/)
      kbo(:, 4,55, 3) = (/ &
     &1.4990e-01_rb,1.2087e-01_rb,8.2841e-02_rb,4.2848e-02_rb,2.6955e-05_rb/)
      kbo(:, 5,55, 3) = (/ &
     &1.6076e-01_rb,1.3155e-01_rb,9.0755e-02_rb,4.7259e-02_rb,2.8512e-05_rb/)
      kbo(:, 1,56, 3) = (/ &
     &1.0674e-01_rb,8.4150e-02_rb,5.7088e-02_rb,2.9230e-02_rb,1.7018e-05_rb/)
      kbo(:, 2,56, 3) = (/ &
     &1.1202e-01_rb,8.8284e-02_rb,5.9924e-02_rb,3.0675e-02_rb,1.8552e-05_rb/)
      kbo(:, 3,56, 3) = (/ &
     &1.1522e-01_rb,9.1603e-02_rb,6.2380e-02_rb,3.2028e-02_rb,2.0079e-05_rb/)
      kbo(:, 4,56, 3) = (/ &
     &1.2002e-01_rb,9.6801e-02_rb,6.6273e-02_rb,3.4240e-02_rb,2.1582e-05_rb/)
      kbo(:, 5,56, 3) = (/ &
     &1.2813e-01_rb,1.0499e-01_rb,7.2382e-02_rb,3.7660e-02_rb,2.2972e-05_rb/)
      kbo(:, 1,57, 3) = (/ &
     &8.4657e-02_rb,6.6996e-02_rb,4.5488e-02_rb,2.3320e-02_rb,1.3524e-05_rb/)
      kbo(:, 2,57, 3) = (/ &
     &8.9961e-02_rb,7.1035e-02_rb,4.8219e-02_rb,2.4686e-02_rb,1.4787e-05_rb/)
      kbo(:, 3,57, 3) = (/ &
     &9.2828e-02_rb,7.3808e-02_rb,5.0230e-02_rb,2.5772e-02_rb,1.6030e-05_rb/)
      kbo(:, 4,57, 3) = (/ &
     &9.6271e-02_rb,7.7620e-02_rb,5.3068e-02_rb,2.7385e-02_rb,1.7279e-05_rb/)
      kbo(:, 5,57, 3) = (/ &
     &1.0221e-01_rb,8.3835e-02_rb,5.7725e-02_rb,3.0004e-02_rb,1.8442e-05_rb/)
      kbo(:, 1,58, 3) = (/ &
     &1.7656e-02_rb,1.7150e-02_rb,1.4988e-02_rb,1.0779e-02_rb,1.0747e-05_rb/)
      kbo(:, 2,58, 3) = (/ &
     &1.9015e-02_rb,1.8411e-02_rb,1.6077e-02_rb,1.1536e-02_rb,1.1805e-05_rb/)
      kbo(:, 3,58, 3) = (/ &
     &1.9736e-02_rb,1.9199e-02_rb,1.6794e-02_rb,1.2067e-02_rb,1.2808e-05_rb/)
      kbo(:, 4,58, 3) = (/ &
     &2.0419e-02_rb,2.0127e-02_rb,1.7678e-02_rb,1.2766e-02_rb,1.3807e-05_rb/)
      kbo(:, 5,58, 3) = (/ &
     &2.1547e-02_rb,2.1640e-02_rb,1.9143e-02_rb,1.3925e-02_rb,1.4766e-05_rb/)
      kbo(:, 1,59, 3) = (/ &
     &1.6768e-02_rb,1.5635e-02_rb,1.2738e-02_rb,8.1686e-03_rb,8.6706e-06_rb/)
      kbo(:, 2,59, 3) = (/ &
     &1.8151e-02_rb,1.6868e-02_rb,1.3724e-02_rb,8.7749e-03_rb,9.5360e-06_rb/)
      kbo(:, 3,59, 3) = (/ &
     &1.8900e-02_rb,1.7641e-02_rb,1.4372e-02_rb,9.2015e-03_rb,1.0356e-05_rb/)
      kbo(:, 4,59, 3) = (/ &
     &1.9564e-02_rb,1.8529e-02_rb,1.5159e-02_rb,9.7488e-03_rb,1.1149e-05_rb/)
      kbo(:, 5,59, 3) = (/ &
     &2.0620e-02_rb,1.9962e-02_rb,1.6458e-02_rb,1.0647e-02_rb,1.1905e-05_rb/)
      kbo(:, 1,13, 4) = (/ &
     &2.1586e+02_rb,1.6199e+02_rb,1.0811e+02_rb,5.4245e+01_rb,1.2105e-01_rb/)
      kbo(:, 2,13, 4) = (/ &
     &2.1869e+02_rb,1.6410e+02_rb,1.0952e+02_rb,5.4940e+01_rb,1.2618e-01_rb/)
      kbo(:, 3,13, 4) = (/ &
     &2.2370e+02_rb,1.6786e+02_rb,1.1202e+02_rb,5.6190e+01_rb,1.3118e-01_rb/)
      kbo(:, 4,13, 4) = (/ &
     &2.3070e+02_rb,1.7312e+02_rb,1.1553e+02_rb,5.7947e+01_rb,1.3630e-01_rb/)
      kbo(:, 5,13, 4) = (/ &
     &2.4034e+02_rb,1.8034e+02_rb,1.2035e+02_rb,6.0352e+01_rb,1.4080e-01_rb/)
      kbo(:, 1,14, 4) = (/ &
     &1.1263e+02_rb,8.4567e+01_rb,5.6502e+01_rb,2.8444e+01_rb,1.0638e-01_rb/)
      kbo(:, 2,14, 4) = (/ &
     &1.1465e+02_rb,8.6072e+01_rb,5.7500e+01_rb,2.8941e+01_rb,1.1108e-01_rb/)
      kbo(:, 3,14, 4) = (/ &
     &1.1796e+02_rb,8.8560e+01_rb,5.9161e+01_rb,2.9768e+01_rb,1.1550e-01_rb/)
      kbo(:, 4,14, 4) = (/ &
     &1.2248e+02_rb,9.1950e+01_rb,6.1421e+01_rb,3.0896e+01_rb,1.2021e-01_rb/)
      kbo(:, 5,14, 4) = (/ &
     &1.2816e+02_rb,9.6211e+01_rb,6.4260e+01_rb,3.2312e+01_rb,1.2485e-01_rb/)
      kbo(:, 1,15, 4) = (/ &
     &5.9854e+01_rb,4.4984e+01_rb,3.0119e+01_rb,1.5274e+01_rb,9.2887e-02_rb/)
      kbo(:, 2,15, 4) = (/ &
     &6.1271e+01_rb,4.6045e+01_rb,3.0823e+01_rb,1.5619e+01_rb,9.7129e-02_rb/)
      kbo(:, 3,15, 4) = (/ &
     &6.3412e+01_rb,4.7652e+01_rb,3.1894e+01_rb,1.6144e+01_rb,1.0153e-01_rb/)
      kbo(:, 4,15, 4) = (/ &
     &6.6234e+01_rb,4.9767e+01_rb,3.3302e+01_rb,1.6842e+01_rb,1.0585e-01_rb/)
      kbo(:, 5,15, 4) = (/ &
     &6.9392e+01_rb,5.2133e+01_rb,3.4876e+01_rb,1.7625e+01_rb,1.1025e-01_rb/)
      kbo(:, 1,16, 4) = (/ &
     &3.7898e+01_rb,2.8521e+01_rb,1.9151e+01_rb,9.8246e+00_rb,8.0720e-02_rb/)
      kbo(:, 2,16, 4) = (/ &
     &3.9024e+01_rb,2.9361e+01_rb,1.9706e+01_rb,1.0088e+01_rb,8.4821e-02_rb/)
      kbo(:, 3,16, 4) = (/ &
     &4.0630e+01_rb,3.0565e+01_rb,2.0505e+01_rb,1.0472e+01_rb,8.9056e-02_rb/)
      kbo(:, 4,16, 4) = (/ &
     &4.2559e+01_rb,3.2011e+01_rb,2.1467e+01_rb,1.0943e+01_rb,9.3186e-02_rb/)
      kbo(:, 5,16, 4) = (/ &
     &4.4564e+01_rb,3.3513e+01_rb,2.2465e+01_rb,1.1432e+01_rb,9.7171e-02_rb/)
      kbo(:, 1,17, 4) = (/ &
     &2.4925e+01_rb,1.8792e+01_rb,1.2674e+01_rb,6.6287e+00_rb,6.9974e-02_rb/)
      kbo(:, 2,17, 4) = (/ &
     &2.5814e+01_rb,1.9454e+01_rb,1.3108e+01_rb,6.8325e+00_rb,7.3907e-02_rb/)
      kbo(:, 3,17, 4) = (/ &
     &2.6980e+01_rb,2.0330e+01_rb,1.3687e+01_rb,7.1100e+00_rb,7.7821e-02_rb/)
      kbo(:, 4,17, 4) = (/ &
     &2.8287e+01_rb,2.1307e+01_rb,1.4334e+01_rb,7.4176e+00_rb,8.1565e-02_rb/)
      kbo(:, 5,17, 4) = (/ &
     &2.9635e+01_rb,2.2318e+01_rb,1.5005e+01_rb,7.7402e+00_rb,8.5222e-02_rb/)
      kbo(:, 1,18, 4) = (/ &
     &1.6833e+01_rb,1.2726e+01_rb,8.6533e+00_rb,4.6132e+00_rb,6.0568e-02_rb/)
      kbo(:, 2,18, 4) = (/ &
     &1.7515e+01_rb,1.3234e+01_rb,8.9835e+00_rb,4.7781e+00_rb,6.4137e-02_rb/)
      kbo(:, 3,18, 4) = (/ &
     &1.8359e+01_rb,1.3866e+01_rb,9.3947e+00_rb,4.9795e+00_rb,6.7564e-02_rb/)
      kbo(:, 4,18, 4) = (/ &
     &1.9269e+01_rb,1.4545e+01_rb,9.8365e+00_rb,5.1909e+00_rb,7.0941e-02_rb/)
      kbo(:, 5,18, 4) = (/ &
     &2.0186e+01_rb,1.5233e+01_rb,1.0291e+01_rb,5.4115e+00_rb,7.4160e-02_rb/)
      kbo(:, 1,19, 4) = (/ &
     &1.1200e+01_rb,8.5062e+00_rb,5.8670e+00_rb,3.1832e+00_rb,5.2174e-02_rb/)
      kbo(:, 2,19, 4) = (/ &
     &1.1703e+01_rb,8.8805e+00_rb,6.1092e+00_rb,3.3121e+00_rb,5.5280e-02_rb/)
      kbo(:, 3,19, 4) = (/ &
     &1.2289e+01_rb,9.3153e+00_rb,6.3897e+00_rb,3.4520e+00_rb,5.8298e-02_rb/)
      kbo(:, 4,19, 4) = (/ &
     &1.2879e+01_rb,9.7549e+00_rb,6.6727e+00_rb,3.5951e+00_rb,6.1230e-02_rb/)
      kbo(:, 5,19, 4) = (/ &
     &1.3509e+01_rb,1.0228e+01_rb,6.9797e+00_rb,3.7461e+00_rb,6.3987e-02_rb/)
      kbo(:, 1,20, 4) = (/ &
     &8.1017e+00_rb,6.1940e+00_rb,4.3223e+00_rb,2.3773e+00_rb,4.4725e-02_rb/)
      kbo(:, 2,20, 4) = (/ &
     &8.4816e+00_rb,6.4735e+00_rb,4.5091e+00_rb,2.4775e+00_rb,4.7408e-02_rb/)
      kbo(:, 3,20, 4) = (/ &
     &8.8970e+00_rb,6.7793e+00_rb,4.7079e+00_rb,2.5791e+00_rb,5.0027e-02_rb/)
      kbo(:, 4,20, 4) = (/ &
     &9.3252e+00_rb,7.0958e+00_rb,4.9137e+00_rb,2.6856e+00_rb,5.2539e-02_rb/)
      kbo(:, 5,20, 4) = (/ &
     &9.8057e+00_rb,7.4546e+00_rb,5.1456e+00_rb,2.8027e+00_rb,5.4910e-02_rb/)
      kbo(:, 1,21, 4) = (/ &
     &6.0501e+00_rb,4.6688e+00_rb,3.2884e+00_rb,1.8254e+00_rb,3.8136e-02_rb/)
      kbo(:, 2,21, 4) = (/ &
     &6.3357e+00_rb,4.8779e+00_rb,3.4312e+00_rb,1.9011e+00_rb,4.0441e-02_rb/)
      kbo(:, 3,21, 4) = (/ &
     &6.6449e+00_rb,5.1029e+00_rb,3.5792e+00_rb,1.9785e+00_rb,4.2660e-02_rb/)
      kbo(:, 4,21, 4) = (/ &
     &6.9762e+00_rb,5.3455e+00_rb,3.7404e+00_rb,2.0617e+00_rb,4.4823e-02_rb/)
      kbo(:, 5,21, 4) = (/ &
     &7.3528e+00_rb,5.6235e+00_rb,3.9228e+00_rb,2.1569e+00_rb,4.6872e-02_rb/)
      kbo(:, 1,22, 4) = (/ &
     &4.6404e+00_rb,3.6182e+00_rb,2.5692e+00_rb,1.4322e+00_rb,3.2495e-02_rb/)
      kbo(:, 2,22, 4) = (/ &
     &4.8647e+00_rb,3.7820e+00_rb,2.6806e+00_rb,1.4906e+00_rb,3.4441e-02_rb/)
      kbo(:, 3,22, 4) = (/ &
     &5.1076e+00_rb,3.9577e+00_rb,2.7989e+00_rb,1.5529e+00_rb,3.6349e-02_rb/)
      kbo(:, 4,22, 4) = (/ &
     &5.3767e+00_rb,4.1547e+00_rb,2.9316e+00_rb,1.6226e+00_rb,3.8200e-02_rb/)
      kbo(:, 5,22, 4) = (/ &
     &5.6763e+00_rb,4.3747e+00_rb,3.0789e+00_rb,1.7012e+00_rb,3.9814e-02_rb/)
      kbo(:, 1,23, 4) = (/ &
     &3.5795e+00_rb,2.8211e+00_rb,2.0153e+00_rb,1.1263e+00_rb,2.7571e-02_rb/)
      kbo(:, 2,23, 4) = (/ &
     &3.7577e+00_rb,2.9513e+00_rb,2.1035e+00_rb,1.1732e+00_rb,2.9222e-02_rb/)
      kbo(:, 3,23, 4) = (/ &
     &3.9530e+00_rb,3.0936e+00_rb,2.2016e+00_rb,1.2254e+00_rb,3.0874e-02_rb/)
      kbo(:, 4,23, 4) = (/ &
     &4.1701e+00_rb,3.2524e+00_rb,2.3097e+00_rb,1.2838e+00_rb,3.2339e-02_rb/)
      kbo(:, 5,23, 4) = (/ &
     &4.4121e+00_rb,3.4311e+00_rb,2.4318e+00_rb,1.3490e+00_rb,3.3498e-02_rb/)
      kbo(:, 1,24, 4) = (/ &
     &2.7499e+00_rb,2.1929e+00_rb,1.5716e+00_rb,8.8208e-01_rb,2.3328e-02_rb/)
      kbo(:, 2,24, 4) = (/ &
     &2.8912e+00_rb,2.2961e+00_rb,1.6422e+00_rb,9.2052e-01_rb,2.4747e-02_rb/)
      kbo(:, 3,24, 4) = (/ &
     &3.0489e+00_rb,2.4127e+00_rb,1.7231e+00_rb,9.6426e-01_rb,2.6067e-02_rb/)
      kbo(:, 4,24, 4) = (/ &
     &3.2231e+00_rb,2.5412e+00_rb,1.8120e+00_rb,1.0128e+00_rb,2.7124e-02_rb/)
      kbo(:, 5,24, 4) = (/ &
     &3.4149e+00_rb,2.6844e+00_rb,1.9115e+00_rb,1.0663e+00_rb,2.7906e-02_rb/)
      kbo(:, 1,25, 4) = (/ &
     &2.1432e+00_rb,1.7277e+00_rb,1.2393e+00_rb,6.9912e-01_rb,1.9707e-02_rb/)
      kbo(:, 2,25, 4) = (/ &
     &2.2580e+00_rb,1.8124e+00_rb,1.2983e+00_rb,7.3186e-01_rb,2.0862e-02_rb/)
      kbo(:, 3,25, 4) = (/ &
     &2.3862e+00_rb,1.9090e+00_rb,1.3657e+00_rb,7.6856e-01_rb,2.1834e-02_rb/)
      kbo(:, 4,25, 4) = (/ &
     &2.5268e+00_rb,2.0149e+00_rb,1.4403e+00_rb,8.0968e-01_rb,2.2567e-02_rb/)
      kbo(:, 5,25, 4) = (/ &
     &2.6782e+00_rb,2.1290e+00_rb,1.5196e+00_rb,8.5344e-01_rb,2.3120e-02_rb/)
      kbo(:, 1,26, 4) = (/ &
     &1.7021e+00_rb,1.3844e+00_rb,9.9410e-01_rb,5.6358e-01_rb,1.6610e-02_rb/)
      kbo(:, 2,26, 4) = (/ &
     &1.7967e+00_rb,1.4558e+00_rb,1.0446e+00_rb,5.9163e-01_rb,1.7478e-02_rb/)
      kbo(:, 3,26, 4) = (/ &
     &1.9029e+00_rb,1.5366e+00_rb,1.1015e+00_rb,6.2314e-01_rb,1.8168e-02_rb/)
      kbo(:, 4,26, 4) = (/ &
     &2.0167e+00_rb,1.6240e+00_rb,1.1633e+00_rb,6.5758e-01_rb,1.8695e-02_rb/)
      kbo(:, 5,26, 4) = (/ &
     &2.1386e+00_rb,1.7169e+00_rb,1.2284e+00_rb,6.9400e-01_rb,1.9096e-02_rb/)
      kbo(:, 1,27, 4) = (/ &
     &1.4057e+00_rb,1.1500e+00_rb,8.2663e-01_rb,4.7009e-01_rb,1.3925e-02_rb/)
      kbo(:, 2,27, 4) = (/ &
     &1.4879e+00_rb,1.2132e+00_rb,8.7128e-01_rb,4.9476e-01_rb,1.4552e-02_rb/)
      kbo(:, 3,27, 4) = (/ &
     &1.5769e+00_rb,1.2821e+00_rb,9.2036e-01_rb,5.2229e-01_rb,1.5059e-02_rb/)
      kbo(:, 4,27, 4) = (/ &
     &1.6734e+00_rb,1.3576e+00_rb,9.7388e-01_rb,5.5224e-01_rb,1.5448e-02_rb/)
      kbo(:, 5,27, 4) = (/ &
     &1.7761e+00_rb,1.4364e+00_rb,1.0297e+00_rb,5.8351e-01_rb,1.5741e-02_rb/)
      kbo(:, 1,28, 4) = (/ &
     &1.1944e+00_rb,9.8092e-01_rb,7.0555e-01_rb,4.0148e-01_rb,1.1607e-02_rb/)
      kbo(:, 2,28, 4) = (/ &
     &1.2669e+00_rb,1.0377e+00_rb,7.4577e-01_rb,4.2386e-01_rb,1.2073e-02_rb/)
      kbo(:, 3,28, 4) = (/ &
     &1.3434e+00_rb,1.0982e+00_rb,7.8916e-01_rb,4.4819e-01_rb,1.2454e-02_rb/)
      kbo(:, 4,28, 4) = (/ &
     &1.4276e+00_rb,1.1642e+00_rb,8.3605e-01_rb,4.7463e-01_rb,1.2747e-02_rb/)
      kbo(:, 5,28, 4) = (/ &
     &1.5174e+00_rb,1.2340e+00_rb,8.8572e-01_rb,5.0242e-01_rb,1.2962e-02_rb/)
      kbo(:, 1,29, 4) = (/ &
     &1.0796e+00_rb,8.8738e-01_rb,6.3729e-01_rb,3.6140e-01_rb,9.6469e-03_rb/)
      kbo(:, 2,29, 4) = (/ &
     &1.1462e+00_rb,9.4012e-01_rb,6.7481e-01_rb,3.8241e-01_rb,1.0006e-02_rb/)
      kbo(:, 3,29, 4) = (/ &
     &1.2166e+00_rb,9.9676e-01_rb,7.1549e-01_rb,4.0534e-01_rb,1.0291e-02_rb/)
      kbo(:, 4,29, 4) = (/ &
     &1.2950e+00_rb,1.0580e+00_rb,7.5913e-01_rb,4.2989e-01_rb,1.0512e-02_rb/)
      kbo(:, 5,29, 4) = (/ &
     &1.3796e+00_rb,1.1246e+00_rb,8.0670e-01_rb,4.5638e-01_rb,1.0678e-02_rb/)
      kbo(:, 1,30, 4) = (/ &
     &1.0009e+00_rb,8.2254e-01_rb,5.8934e-01_rb,3.3271e-01_rb,8.0058e-03_rb/)
      kbo(:, 2,30, 4) = (/ &
     &1.0631e+00_rb,8.7256e-01_rb,6.2533e-01_rb,3.5305e-01_rb,8.2800e-03_rb/)
      kbo(:, 3,30, 4) = (/ &
     &1.1308e+00_rb,9.2710e-01_rb,6.6420e-01_rb,3.7481e-01_rb,8.4963e-03_rb/)
      kbo(:, 4,30, 4) = (/ &
     &1.2063e+00_rb,9.8617e-01_rb,7.0646e-01_rb,3.9833e-01_rb,8.6678e-03_rb/)
      kbo(:, 5,30, 4) = (/ &
     &1.2872e+00_rb,1.0507e+00_rb,7.5301e-01_rb,4.2433e-01_rb,8.7939e-03_rb/)
      kbo(:, 1,31, 4) = (/ &
     &9.7660e-01_rb,8.0084e-01_rb,5.7177e-01_rb,3.2070e-01_rb,6.6382e-03_rb/)
      kbo(:, 2,31, 4) = (/ &
     &1.0374e+00_rb,8.5014e-01_rb,6.0734e-01_rb,3.4069e-01_rb,6.8456e-03_rb/)
      kbo(:, 3,31, 4) = (/ &
     &1.1075e+00_rb,9.0581e-01_rb,6.4681e-01_rb,3.6261e-01_rb,7.0127e-03_rb/)
      kbo(:, 4,31, 4) = (/ &
     &1.1834e+00_rb,9.6644e-01_rb,6.9039e-01_rb,3.8675e-01_rb,7.1469e-03_rb/)
      kbo(:, 5,31, 4) = (/ &
     &1.2660e+00_rb,1.0326e+00_rb,7.3825e-01_rb,4.1334e-01_rb,7.2367e-03_rb/)
      kbo(:, 1,32, 4) = (/ &
     &9.6422e-01_rb,7.8878e-01_rb,5.6132e-01_rb,3.1280e-01_rb,5.4958e-03_rb/)
      kbo(:, 2,32, 4) = (/ &
     &1.0268e+00_rb,8.3964e-01_rb,5.9767e-01_rb,3.3295e-01_rb,5.6574e-03_rb/)
      kbo(:, 3,32, 4) = (/ &
     &1.0993e+00_rb,8.9729e-01_rb,6.3870e-01_rb,3.5556e-01_rb,5.7908e-03_rb/)
      kbo(:, 4,32, 4) = (/ &
     &1.1769e+00_rb,9.5960e-01_rb,6.8365e-01_rb,3.8056e-01_rb,5.8892e-03_rb/)
      kbo(:, 5,32, 4) = (/ &
     &1.2618e+00_rb,1.0283e+00_rb,7.3351e-01_rb,4.0808e-01_rb,5.9563e-03_rb/)
      kbo(:, 1,33, 4) = (/ &
     &9.7044e-01_rb,7.9175e-01_rb,5.6148e-01_rb,3.1071e-01_rb,4.5479e-03_rb/)
      kbo(:, 2,33, 4) = (/ &
     &1.0377e+00_rb,8.4609e-01_rb,5.9989e-01_rb,3.3169e-01_rb,4.6770e-03_rb/)
      kbo(:, 3,33, 4) = (/ &
     &1.1127e+00_rb,9.0612e-01_rb,6.4309e-01_rb,3.5556e-01_rb,4.7785e-03_rb/)
      kbo(:, 4,33, 4) = (/ &
     &1.1941e+00_rb,9.7205e-01_rb,6.9062e-01_rb,3.8191e-01_rb,4.8530e-03_rb/)
      kbo(:, 5,33, 4) = (/ &
     &1.2827e+00_rb,1.0440e+00_rb,7.4287e-01_rb,4.1072e-01_rb,4.9022e-03_rb/)
      kbo(:, 1,34, 4) = (/ &
     &9.5216e-01_rb,7.7596e-01_rb,5.4886e-01_rb,3.0190e-01_rb,3.7602e-03_rb/)
      kbo(:, 2,34, 4) = (/ &
     &1.0214e+00_rb,8.3168e-01_rb,5.8836e-01_rb,3.2354e-01_rb,3.8612e-03_rb/)
      kbo(:, 3,34, 4) = (/ &
     &1.0976e+00_rb,8.9321e-01_rb,6.3261e-01_rb,3.4797e-01_rb,3.9398e-03_rb/)
      kbo(:, 4,34, 4) = (/ &
     &1.1802e+00_rb,9.6059e-01_rb,6.8153e-01_rb,3.7517e-01_rb,3.9961e-03_rb/)
      kbo(:, 5,34, 4) = (/ &
     &1.2700e+00_rb,1.0340e+00_rb,7.3462e-01_rb,4.0436e-01_rb,4.0321e-03_rb/)
      kbo(:, 1,35, 4) = (/ &
     &9.3109e-01_rb,7.5807e-01_rb,5.3485e-01_rb,2.9258e-01_rb,3.0990e-03_rb/)
      kbo(:, 2,35, 4) = (/ &
     &1.0014e+00_rb,8.1463e-01_rb,5.7510e-01_rb,3.1470e-01_rb,3.1794e-03_rb/)
      kbo(:, 3,35, 4) = (/ &
     &1.0779e+00_rb,8.7703e-01_rb,6.2017e-01_rb,3.3965e-01_rb,3.2414e-03_rb/)
      kbo(:, 4,35, 4) = (/ &
     &1.1611e+00_rb,9.4542e-01_rb,6.6993e-01_rb,3.6723e-01_rb,3.2866e-03_rb/)
      kbo(:, 5,35, 4) = (/ &
     &1.2524e+00_rb,1.0199e+00_rb,7.2388e-01_rb,3.9699e-01_rb,3.3190e-03_rb/)
      kbo(:, 1,36, 4) = (/ &
     &8.9907e-01_rb,7.3170e-01_rb,5.1516e-01_rb,2.8053e-01_rb,2.5454e-03_rb/)
      kbo(:, 2,36, 4) = (/ &
     &9.6904e-01_rb,7.8811e-01_rb,5.5541e-01_rb,3.0260e-01_rb,2.6116e-03_rb/)
      kbo(:, 3,36, 4) = (/ &
     &1.0454e+00_rb,8.5074e-01_rb,6.0077e-01_rb,3.2780e-01_rb,2.6639e-03_rb/)
      kbo(:, 4,36, 4) = (/ &
     &1.1275e+00_rb,9.1876e-01_rb,6.5041e-01_rb,3.5533e-01_rb,2.7017e-03_rb/)
      kbo(:, 5,36, 4) = (/ &
     &1.2188e+00_rb,9.9337e-01_rb,7.0446e-01_rb,3.8526e-01_rb,2.7298e-03_rb/)
      kbo(:, 1,37, 4) = (/ &
     &8.3748e-01_rb,6.8208e-01_rb,4.7958e-01_rb,2.6026e-01_rb,2.0819e-03_rb/)
      kbo(:, 2,37, 4) = (/ &
     &9.0437e-01_rb,7.3619e-01_rb,5.1822e-01_rb,2.8144e-01_rb,2.1383e-03_rb/)
      kbo(:, 3,37, 4) = (/ &
     &9.7724e-01_rb,7.9645e-01_rb,5.6196e-01_rb,3.0574e-01_rb,2.1835e-03_rb/)
      kbo(:, 4,37, 4) = (/ &
     &1.0560e+00_rb,8.6198e-01_rb,6.0986e-01_rb,3.3246e-01_rb,2.2167e-03_rb/)
      kbo(:, 5,37, 4) = (/ &
     &1.1435e+00_rb,9.3399e-01_rb,6.6218e-01_rb,3.6148e-01_rb,2.2423e-03_rb/)
      kbo(:, 1,38, 4) = (/ &
     &7.9164e-01_rb,6.4489e-01_rb,4.5275e-01_rb,2.4481e-01_rb,1.7033e-03_rb/)
      kbo(:, 2,38, 4) = (/ &
     &8.5631e-01_rb,6.9735e-01_rb,4.9025e-01_rb,2.6537e-01_rb,1.7520e-03_rb/)
      kbo(:, 3,38, 4) = (/ &
     &9.2681e-01_rb,7.5605e-01_rb,5.3294e-01_rb,2.8911e-01_rb,1.7893e-03_rb/)
      kbo(:, 4,38, 4) = (/ &
     &1.0032e+00_rb,8.1986e-01_rb,5.7959e-01_rb,3.1521e-01_rb,1.8197e-03_rb/)
      kbo(:, 5,38, 4) = (/ &
     &1.0885e+00_rb,8.9047e-01_rb,6.3102e-01_rb,3.4379e-01_rb,1.8432e-03_rb/)
      kbo(:, 1,39, 4) = (/ &
     &7.6561e-01_rb,6.2337e-01_rb,4.3685e-01_rb,2.3531e-01_rb,1.3951e-03_rb/)
      kbo(:, 2,39, 4) = (/ &
     &8.2959e-01_rb,6.7544e-01_rb,4.7411e-01_rb,2.5572e-01_rb,1.4362e-03_rb/)
      kbo(:, 3,39, 4) = (/ &
     &8.9955e-01_rb,7.3386e-01_rb,5.1664e-01_rb,2.7941e-01_rb,1.4681e-03_rb/)
      kbo(:, 4,39, 4) = (/ &
     &9.7533e-01_rb,7.9745e-01_rb,5.6319e-01_rb,3.0547e-01_rb,1.4952e-03_rb/)
      kbo(:, 5,39, 4) = (/ &
     &1.0606e+00_rb,8.6829e-01_rb,6.1487e-01_rb,3.3424e-01_rb,1.5163e-03_rb/)
      kbo(:, 1,40, 4) = (/ &
     &6.9042e-01_rb,5.6314e-01_rb,3.9439e-01_rb,2.1202e-01_rb,1.1367e-03_rb/)
      kbo(:, 2,40, 4) = (/ &
     &7.4888e-01_rb,6.1108e-01_rb,4.2878e-01_rb,2.3085e-01_rb,1.1721e-03_rb/)
      kbo(:, 3,40, 4) = (/ &
     &8.1362e-01_rb,6.6539e-01_rb,4.6832e-01_rb,2.5287e-01_rb,1.1998e-03_rb/)
      kbo(:, 4,40, 4) = (/ &
     &8.8373e-01_rb,7.2479e-01_rb,5.1188e-01_rb,2.7730e-01_rb,1.2242e-03_rb/)
      kbo(:, 5,40, 4) = (/ &
     &9.6254e-01_rb,7.9070e-01_rb,5.6015e-01_rb,3.0430e-01_rb,1.2434e-03_rb/)
      kbo(:, 1,41, 4) = (/ &
     &6.2032e-01_rb,5.0688e-01_rb,3.5482e-01_rb,1.9040e-01_rb,9.2546e-04_rb/)
      kbo(:, 2,41, 4) = (/ &
     &6.7393e-01_rb,5.5118e-01_rb,3.8660e-01_rb,2.0779e-01_rb,9.5549e-04_rb/)
      kbo(:, 3,41, 4) = (/ &
     &7.3329e-01_rb,6.0114e-01_rb,4.2302e-01_rb,2.2810e-01_rb,9.8005e-04_rb/)
      kbo(:, 4,41, 4) = (/ &
     &7.9788e-01_rb,6.5636e-01_rb,4.6356e-01_rb,2.5084e-01_rb,1.0016e-03_rb/)
      kbo(:, 5,41, 4) = (/ &
     &8.7082e-01_rb,7.1789e-01_rb,5.0871e-01_rb,2.7615e-01_rb,1.0188e-03_rb/)
      kbo(:, 1,42, 4) = (/ &
     &5.6327e-01_rb,4.6093e-01_rb,3.2241e-01_rb,1.7266e-01_rb,7.5328e-04_rb/)
      kbo(:, 2,42, 4) = (/ &
     &6.1295e-01_rb,5.0232e-01_rb,3.5212e-01_rb,1.8890e-01_rb,7.7877e-04_rb/)
      kbo(:, 3,42, 4) = (/ &
     &6.6777e-01_rb,5.4861e-01_rb,3.8590e-01_rb,2.0776e-01_rb,8.0043e-04_rb/)
      kbo(:, 4,42, 4) = (/ &
     &7.2797e-01_rb,6.0039e-01_rb,4.2397e-01_rb,2.2912e-01_rb,8.1936e-04_rb/)
      kbo(:, 5,42, 4) = (/ &
     &7.9621e-01_rb,6.5839e-01_rb,4.6656e-01_rb,2.5303e-01_rb,8.3483e-04_rb/)
      kbo(:, 1,43, 4) = (/ &
     &5.1362e-01_rb,4.2068e-01_rb,2.9391e-01_rb,1.5704e-01_rb,6.1131e-04_rb/)
      kbo(:, 2,43, 4) = (/ &
     &5.5956e-01_rb,4.5932e-01_rb,3.2171e-01_rb,1.7223e-01_rb,6.3359e-04_rb/)
      kbo(:, 3,43, 4) = (/ &
     &6.1053e-01_rb,5.0241e-01_rb,3.5315e-01_rb,1.8977e-01_rb,6.5237e-04_rb/)
      kbo(:, 4,43, 4) = (/ &
     &6.6664e-01_rb,5.5084e-01_rb,3.8881e-01_rb,2.0981e-01_rb,6.6921e-04_rb/)
      kbo(:, 5,43, 4) = (/ &
     &7.3047e-01_rb,6.0544e-01_rb,4.2894e-01_rb,2.3232e-01_rb,6.8300e-04_rb/)
      kbo(:, 1,44, 4) = (/ &
     &4.7369e-01_rb,3.8803e-01_rb,2.7069e-01_rb,1.4426e-01_rb,4.9540e-04_rb/)
      kbo(:, 2,44, 4) = (/ &
     &5.1629e-01_rb,4.2426e-01_rb,2.9688e-01_rb,1.5856e-01_rb,5.1475e-04_rb/)
      kbo(:, 3,44, 4) = (/ &
     &5.6430e-01_rb,4.6484e-01_rb,3.2640e-01_rb,1.7499e-01_rb,5.3131e-04_rb/)
      kbo(:, 4,44, 4) = (/ &
     &6.1717e-01_rb,5.1049e-01_rb,3.6006e-01_rb,1.9393e-01_rb,5.4642e-04_rb/)
      kbo(:, 5,44, 4) = (/ &
     &6.7746e-01_rb,5.6241e-01_rb,3.9820e-01_rb,2.1537e-01_rb,5.5870e-04_rb/)
      kbo(:, 1,45, 4) = (/ &
     &4.4552e-01_rb,3.6469e-01_rb,2.5398e-01_rb,1.3497e-01_rb,4.0119e-04_rb/)
      kbo(:, 2,45, 4) = (/ &
     &4.8593e-01_rb,3.9940e-01_rb,2.7911e-01_rb,1.4871e-01_rb,4.1803e-04_rb/)
      kbo(:, 3,45, 4) = (/ &
     &5.3210e-01_rb,4.3837e-01_rb,3.0742e-01_rb,1.6436e-01_rb,4.3281e-04_rb/)
      kbo(:, 4,45, 4) = (/ &
     &5.8266e-01_rb,4.8202e-01_rb,3.3961e-01_rb,1.8254e-01_rb,4.4586e-04_rb/)
      kbo(:, 5,45, 4) = (/ &
     &6.4061e-01_rb,5.3215e-01_rb,3.7643e-01_rb,2.0327e-01_rb,4.5687e-04_rb/)
      kbo(:, 1,46, 4) = (/ &
     &4.2249e-01_rb,3.4536e-01_rb,2.4007e-01_rb,1.2719e-01_rb,3.2444e-04_rb/)
      kbo(:, 2,46, 4) = (/ &
     &4.6085e-01_rb,3.7858e-01_rb,2.6416e-01_rb,1.4037e-01_rb,3.3911e-04_rb/)
      kbo(:, 3,46, 4) = (/ &
     &5.0558e-01_rb,4.1641e-01_rb,2.9158e-01_rb,1.5550e-01_rb,3.5210e-04_rb/)
      kbo(:, 4,46, 4) = (/ &
     &5.5446e-01_rb,4.5852e-01_rb,3.2261e-01_rb,1.7296e-01_rb,3.6323e-04_rb/)
      kbo(:, 5,46, 4) = (/ &
     &6.1034e-01_rb,5.0695e-01_rb,3.5817e-01_rb,1.9305e-01_rb,3.7330e-04_rb/)
      kbo(:, 1,47, 4) = (/ &
     &3.9554e-01_rb,3.2291e-01_rb,2.2401e-01_rb,1.1833e-01_rb,2.6160e-04_rb/)
      kbo(:, 2,47, 4) = (/ &
     &4.3146e-01_rb,3.5412e-01_rb,2.4668e-01_rb,1.3078e-01_rb,2.7433e-04_rb/)
      kbo(:, 3,47, 4) = (/ &
     &4.7369e-01_rb,3.9022e-01_rb,2.7284e-01_rb,1.4515e-01_rb,2.8554e-04_rb/)
      kbo(:, 4,47, 4) = (/ &
     &5.2055e-01_rb,4.3044e-01_rb,3.0232e-01_rb,1.6167e-01_rb,2.9527e-04_rb/)
      kbo(:, 5,47, 4) = (/ &
     &5.7336e-01_rb,4.7635e-01_rb,3.3620e-01_rb,1.8084e-01_rb,3.0428e-04_rb/)
      kbo(:, 1,48, 4) = (/ &
     &3.8090e-01_rb,3.1036e-01_rb,2.1482e-01_rb,1.1310e-01_rb,2.1085e-04_rb/)
      kbo(:, 2,48, 4) = (/ &
     &4.1533e-01_rb,3.4019e-01_rb,2.3651e-01_rb,1.2501e-01_rb,2.2180e-04_rb/)
      kbo(:, 3,48, 4) = (/ &
     &4.5614e-01_rb,3.7537e-01_rb,2.6197e-01_rb,1.3898e-01_rb,2.3143e-04_rb/)
      kbo(:, 4,48, 4) = (/ &
     &5.0246e-01_rb,4.1492e-01_rb,2.9081e-01_rb,1.5505e-01_rb,2.3998e-04_rb/)
      kbo(:, 5,48, 4) = (/ &
     &5.5365e-01_rb,4.5945e-01_rb,3.2372e-01_rb,1.7366e-01_rb,2.4799e-04_rb/)
      kbo(:, 1,49, 4) = (/ &
     &3.8223e-01_rb,3.1052e-01_rb,2.1444e-01_rb,1.1246e-01_rb,1.6987e-04_rb/)
      kbo(:, 2,49, 4) = (/ &
     &4.1659e-01_rb,3.4001e-01_rb,2.3576e-01_rb,1.2419e-01_rb,1.7919e-04_rb/)
      kbo(:, 3,49, 4) = (/ &
     &4.5734e-01_rb,3.7525e-01_rb,2.6133e-01_rb,1.3821e-01_rb,1.8743e-04_rb/)
      kbo(:, 4,49, 4) = (/ &
     &5.0487e-01_rb,4.1574e-01_rb,2.9072e-01_rb,1.5448e-01_rb,1.9508e-04_rb/)
      kbo(:, 5,49, 4) = (/ &
     &5.5695e-01_rb,4.6079e-01_rb,3.2385e-01_rb,1.7317e-01_rb,2.0224e-04_rb/)
      kbo(:, 1,50, 4) = (/ &
     &3.6631e-01_rb,2.9712e-01_rb,2.0481e-01_rb,1.0712e-01_rb,1.3672e-04_rb/)
      kbo(:, 2,50, 4) = (/ &
     &3.9879e-01_rb,3.2486e-01_rb,2.2483e-01_rb,1.1811e-01_rb,1.4458e-04_rb/)
      kbo(:, 3,50, 4) = (/ &
     &4.3761e-01_rb,3.5858e-01_rb,2.4934e-01_rb,1.3160e-01_rb,1.5166e-04_rb/)
      kbo(:, 4,50, 4) = (/ &
     &4.8398e-01_rb,3.9821e-01_rb,2.7799e-01_rb,1.4737e-01_rb,1.5830e-04_rb/)
      kbo(:, 5,50, 4) = (/ &
     &5.3499e-01_rb,4.4212e-01_rb,3.1022e-01_rb,1.6549e-01_rb,1.6465e-04_rb/)
      kbo(:, 1,51, 4) = (/ &
     &3.4406e-01_rb,2.7861e-01_rb,1.9179e-01_rb,1.0006e-01_rb,1.0998e-04_rb/)
      kbo(:, 2,51, 4) = (/ &
     &3.7372e-01_rb,3.0420e-01_rb,2.1018e-01_rb,1.1015e-01_rb,1.1652e-04_rb/)
      kbo(:, 3,51, 4) = (/ &
     &4.0998e-01_rb,3.3570e-01_rb,2.3310e-01_rb,1.2281e-01_rb,1.2263e-04_rb/)
      kbo(:, 4,51, 4) = (/ &
     &4.5397e-01_rb,3.7347e-01_rb,2.6041e-01_rb,1.3780e-01_rb,1.2829e-04_rb/)
      kbo(:, 5,51, 4) = (/ &
     &5.0293e-01_rb,4.1560e-01_rb,2.9124e-01_rb,1.5503e-01_rb,1.3386e-04_rb/)
      kbo(:, 1,52, 4) = (/ &
     &3.3305e-01_rb,2.6887e-01_rb,1.8470e-01_rb,9.6116e-02_rb,8.8414e-05_rb/)
      kbo(:, 2,52, 4) = (/ &
     &3.6030e-01_rb,2.9285e-01_rb,2.0201e-01_rb,1.0559e-01_rb,9.3941e-05_rb/)
      kbo(:, 3,52, 4) = (/ &
     &3.9501e-01_rb,3.2288e-01_rb,2.2382e-01_rb,1.1765e-01_rb,9.9142e-05_rb/)
      kbo(:, 4,52, 4) = (/ &
     &4.3753e-01_rb,3.5954e-01_rb,2.5031e-01_rb,1.3217e-01_rb,1.0404e-04_rb/)
      kbo(:, 5,52, 4) = (/ &
     &4.8592e-01_rb,4.0113e-01_rb,2.8062e-01_rb,1.4902e-01_rb,1.0887e-04_rb/)
      kbo(:, 1,53, 4) = (/ &
     &3.3673e-01_rb,2.7062e-01_rb,1.8541e-01_rb,9.6126e-02_rb,7.1043e-05_rb/)
      kbo(:, 2,53, 4) = (/ &
     &3.6211e-01_rb,2.9355e-01_rb,2.0209e-01_rb,1.0535e-01_rb,7.5715e-05_rb/)
      kbo(:, 3,53, 4) = (/ &
     &3.9659e-01_rb,3.2308e-01_rb,2.2347e-01_rb,1.1715e-01_rb,8.0134e-05_rb/)
      kbo(:, 4,53, 4) = (/ &
     &4.3895e-01_rb,3.5976e-01_rb,2.5000e-01_rb,1.3170e-01_rb,8.4354e-05_rb/)
      kbo(:, 5,53, 4) = (/ &
     &4.8875e-01_rb,4.0230e-01_rb,2.8086e-01_rb,1.4875e-01_rb,8.8431e-05_rb/)
      kbo(:, 1,54, 4) = (/ &
     &2.9565e-01_rb,2.3718e-01_rb,1.6225e-01_rb,8.3929e-02_rb,5.7062e-05_rb/)
      kbo(:, 2,54, 4) = (/ &
     &3.1566e-01_rb,2.5610e-01_rb,1.7621e-01_rb,9.1726e-02_rb,6.0930e-05_rb/)
      kbo(:, 3,54, 4) = (/ &
     &3.4501e-01_rb,2.8145e-01_rb,1.9453e-01_rb,1.0184e-01_rb,6.4630e-05_rb/)
      kbo(:, 4,54, 4) = (/ &
     &3.8166e-01_rb,3.1342e-01_rb,2.1772e-01_rb,1.1458e-01_rb,6.8122e-05_rb/)
      kbo(:, 5,54, 4) = (/ &
     &4.2580e-01_rb,3.5139e-01_rb,2.4524e-01_rb,1.2978e-01_rb,7.1556e-05_rb/)
      kbo(:, 1,55, 4) = (/ &
     &2.3763e-01_rb,1.9054e-01_rb,1.3023e-01_rb,6.7270e-02_rb,4.5734e-05_rb/)
      kbo(:, 2,55, 4) = (/ &
     &2.5160e-01_rb,2.0457e-01_rb,1.4076e-01_rb,7.3233e-02_rb,4.8936e-05_rb/)
      kbo(:, 3,55, 4) = (/ &
     &2.7408e-01_rb,2.2452e-01_rb,1.5520e-01_rb,8.1193e-02_rb,5.2040e-05_rb/)
      kbo(:, 4,55, 4) = (/ &
     &3.0302e-01_rb,2.4999e-01_rb,1.7374e-01_rb,9.1409e-02_rb,5.4914e-05_rb/)
      kbo(:, 5,55, 4) = (/ &
     &3.3851e-01_rb,2.8105e-01_rb,1.9629e-01_rb,1.0386e-01_rb,5.7717e-05_rb/)
      kbo(:, 1,56, 4) = (/ &
     &1.9160e-01_rb,1.5340e-01_rb,1.0472e-01_rb,5.4007e-02_rb,3.6625e-05_rb/)
      kbo(:, 2,56, 4) = (/ &
     &2.0095e-01_rb,1.6357e-01_rb,1.1247e-01_rb,5.8467e-02_rb,3.9320e-05_rb/)
      kbo(:, 3,56, 4) = (/ &
     &2.1781e-01_rb,1.7908e-01_rb,1.2379e-01_rb,6.4708e-02_rb,4.1872e-05_rb/)
      kbo(:, 4,56, 4) = (/ &
     &2.4047e-01_rb,1.9929e-01_rb,1.3856e-01_rb,7.2864e-02_rb,4.4297e-05_rb/)
      kbo(:, 5,56, 4) = (/ &
     &2.6886e-01_rb,2.2463e-01_rb,1.5695e-01_rb,8.3003e-02_rb,4.6496e-05_rb/)
      kbo(:, 1,57, 4) = (/ &
     &1.5488e-01_rb,1.2382e-01_rb,8.4407e-02_rb,4.3459e-02_rb,2.9307e-05_rb/)
      kbo(:, 2,57, 4) = (/ &
     &1.6082e-01_rb,1.3093e-01_rb,8.9925e-02_rb,4.6681e-02_rb,3.1559e-05_rb/)
      kbo(:, 3,57, 4) = (/ &
     &1.7312e-01_rb,1.4284e-01_rb,9.8719e-02_rb,5.1558e-02_rb,3.3681e-05_rb/)
      kbo(:, 4,57, 4) = (/ &
     &1.9084e-01_rb,1.5883e-01_rb,1.1043e-01_rb,5.8031e-02_rb,3.5680e-05_rb/)
      kbo(:, 5,57, 4) = (/ &
     &2.1336e-01_rb,1.7931e-01_rb,1.2534e-01_rb,6.6236e-02_rb,3.7524e-05_rb/)
      kbo(:, 1,58, 4) = (/ &
     &3.3087e-02_rb,3.2322e-02_rb,2.8310e-02_rb,2.0395e-02_rb,2.3455e-05_rb/)
      kbo(:, 2,58, 4) = (/ &
     &3.4053e-02_rb,3.3907e-02_rb,2.9921e-02_rb,2.1729e-02_rb,2.5336e-05_rb/)
      kbo(:, 3,58, 4) = (/ &
     &3.6376e-02_rb,3.6839e-02_rb,3.2736e-02_rb,2.3935e-02_rb,2.7095e-05_rb/)
      kbo(:, 4,58, 4) = (/ &
     &4.0009e-02_rb,4.0914e-02_rb,3.6586e-02_rb,2.6918e-02_rb,2.8737e-05_rb/)
      kbo(:, 5,58, 4) = (/ &
     &4.4706e-02_rb,4.6236e-02_rb,4.1585e-02_rb,3.0778e-02_rb,3.0324e-05_rb/)
      kbo(:, 1,59, 4) = (/ &
     &3.1774e-02_rb,2.9779e-02_rb,2.4302e-02_rb,1.5595e-02_rb,1.9010e-05_rb/)
      kbo(:, 2,59, 4) = (/ &
     &3.2644e-02_rb,3.1267e-02_rb,2.5718e-02_rb,1.6636e-02_rb,2.0536e-05_rb/)
      kbo(:, 3,59, 4) = (/ &
     &3.4800e-02_rb,3.4041e-02_rb,2.8207e-02_rb,1.8368e-02_rb,2.1957e-05_rb/)
      kbo(:, 4,59, 4) = (/ &
     &3.8295e-02_rb,3.7946e-02_rb,3.1647e-02_rb,2.0746e-02_rb,2.3288e-05_rb/)
      kbo(:, 5,59, 4) = (/ &
     &4.2857e-02_rb,4.3069e-02_rb,3.6135e-02_rb,2.3842e-02_rb,2.4528e-05_rb/)
      kbo(:, 1,13, 5) = (/ &
     &3.9549e+02_rb,2.9672e+02_rb,1.9794e+02_rb,9.9160e+01_rb,2.1101e-01_rb/)
      kbo(:, 2,13, 5) = (/ &
     &4.0510e+02_rb,3.0393e+02_rb,2.0275e+02_rb,1.0157e+02_rb,2.1805e-01_rb/)
      kbo(:, 3,13, 5) = (/ &
     &4.1502e+02_rb,3.1136e+02_rb,2.0770e+02_rb,1.0405e+02_rb,2.2406e-01_rb/)
      kbo(:, 4,13, 5) = (/ &
     &4.2481e+02_rb,3.1871e+02_rb,2.1261e+02_rb,1.0651e+02_rb,2.2934e-01_rb/)
      kbo(:, 5,13, 5) = (/ &
     &4.3504e+02_rb,3.2638e+02_rb,2.1772e+02_rb,1.0907e+02_rb,2.3453e-01_rb/)
      kbo(:, 1,14, 5) = (/ &
     &2.0979e+02_rb,1.5743e+02_rb,1.0508e+02_rb,5.2736e+01_rb,1.8842e-01_rb/)
      kbo(:, 2,14, 5) = (/ &
     &2.1502e+02_rb,1.6136e+02_rb,1.0770e+02_rb,5.4046e+01_rb,1.9486e-01_rb/)
      kbo(:, 3,14, 5) = (/ &
     &2.2030e+02_rb,1.6532e+02_rb,1.1034e+02_rb,5.5365e+01_rb,2.0050e-01_rb/)
      kbo(:, 4,14, 5) = (/ &
     &2.2600e+02_rb,1.6960e+02_rb,1.1320e+02_rb,5.6802e+01_rb,2.0562e-01_rb/)
      kbo(:, 5,14, 5) = (/ &
     &2.3272e+02_rb,1.7464e+02_rb,1.1657e+02_rb,5.8488e+01_rb,2.1024e-01_rb/)
      kbo(:, 1,15, 5) = (/ &
     &1.1299e+02_rb,8.4842e+01_rb,5.6692e+01_rb,2.8544e+01_rb,1.6706e-01_rb/)
      kbo(:, 2,15, 5) = (/ &
     &1.1594e+02_rb,8.7050e+01_rb,5.8163e+01_rb,2.9281e+01_rb,1.7285e-01_rb/)
      kbo(:, 3,15, 5) = (/ &
     &1.1912e+02_rb,8.9439e+01_rb,5.9757e+01_rb,3.0081e+01_rb,1.7816e-01_rb/)
      kbo(:, 4,15, 5) = (/ &
     &1.2274e+02_rb,9.2159e+01_rb,6.1575e+01_rb,3.0997e+01_rb,1.8306e-01_rb/)
      kbo(:, 5,15, 5) = (/ &
     &1.2711e+02_rb,9.5434e+01_rb,6.3760e+01_rb,3.2089e+01_rb,1.8793e-01_rb/)
      kbo(:, 1,16, 5) = (/ &
     &7.2342e+01_rb,5.4353e+01_rb,3.6365e+01_rb,1.8388e+01_rb,1.4721e-01_rb/)
      kbo(:, 2,16, 5) = (/ &
     &7.4310e+01_rb,5.5827e+01_rb,3.7348e+01_rb,1.8883e+01_rb,1.5241e-01_rb/)
      kbo(:, 3,16, 5) = (/ &
     &7.6651e+01_rb,5.7588e+01_rb,3.8528e+01_rb,1.9478e+01_rb,1.5739e-01_rb/)
      kbo(:, 4,16, 5) = (/ &
     &7.9417e+01_rb,5.9665e+01_rb,3.9914e+01_rb,2.0174e+01_rb,1.6237e-01_rb/)
      kbo(:, 5,16, 5) = (/ &
     &8.2509e+01_rb,6.1984e+01_rb,4.1462e+01_rb,2.0947e+01_rb,1.6729e-01_rb/)
      kbo(:, 1,17, 5) = (/ &
     &4.7912e+01_rb,3.6032e+01_rb,2.4156e+01_rb,1.2312e+01_rb,1.2867e-01_rb/)
      kbo(:, 2,17, 5) = (/ &
     &4.9396e+01_rb,3.7145e+01_rb,2.4898e+01_rb,1.2683e+01_rb,1.3345e-01_rb/)
      kbo(:, 3,17, 5) = (/ &
     &5.1211e+01_rb,3.8509e+01_rb,2.5812e+01_rb,1.3139e+01_rb,1.3833e-01_rb/)
      kbo(:, 4,17, 5) = (/ &
     &5.3243e+01_rb,4.0036e+01_rb,2.6830e+01_rb,1.3649e+01_rb,1.4345e-01_rb/)
      kbo(:, 5,17, 5) = (/ &
     &5.5359e+01_rb,4.1621e+01_rb,2.7887e+01_rb,1.4174e+01_rb,1.4839e-01_rb/)
      kbo(:, 1,18, 5) = (/ &
     &3.2517e+01_rb,2.4486e+01_rb,1.6464e+01_rb,8.5255e+00_rb,1.1175e-01_rb/)
      kbo(:, 2,18, 5) = (/ &
     &3.3702e+01_rb,2.5376e+01_rb,1.7056e+01_rb,8.8105e+00_rb,1.1647e-01_rb/)
      kbo(:, 3,18, 5) = (/ &
     &3.5055e+01_rb,2.6392e+01_rb,1.7736e+01_rb,9.1463e+00_rb,1.2136e-01_rb/)
      kbo(:, 4,18, 5) = (/ &
     &3.6466e+01_rb,2.7452e+01_rb,1.8446e+01_rb,9.5008e+00_rb,1.2627e-01_rb/)
      kbo(:, 5,18, 5) = (/ &
     &3.7955e+01_rb,2.8569e+01_rb,1.9189e+01_rb,9.8619e+00_rb,1.3101e-01_rb/)
      kbo(:, 1,19, 5) = (/ &
     &2.1727e+01_rb,1.6393e+01_rb,1.1086e+01_rb,5.8761e+00_rb,9.6819e-02_rb/)
      kbo(:, 2,19, 5) = (/ &
     &2.2617e+01_rb,1.7062e+01_rb,1.1530e+01_rb,6.0896e+00_rb,1.0144e-01_rb/)
      kbo(:, 3,19, 5) = (/ &
     &2.3543e+01_rb,1.7760e+01_rb,1.1998e+01_rb,6.3260e+00_rb,1.0605e-01_rb/)
      kbo(:, 4,19, 5) = (/ &
     &2.4512e+01_rb,1.8488e+01_rb,1.2483e+01_rb,6.5653e+00_rb,1.1055e-01_rb/)
      kbo(:, 5,19, 5) = (/ &
     &2.5548e+01_rb,1.9265e+01_rb,1.2998e+01_rb,6.8170e+00_rb,1.1478e-01_rb/)
      kbo(:, 1,20, 5) = (/ &
     &1.5756e+01_rb,1.1916e+01_rb,8.1383e+00_rb,4.3914e+00_rb,8.3853e-02_rb/)
      kbo(:, 2,20, 5) = (/ &
     &1.6431e+01_rb,1.2425e+01_rb,8.4724e+00_rb,4.5624e+00_rb,8.8103e-02_rb/)
      kbo(:, 3,20, 5) = (/ &
     &1.7121e+01_rb,1.2945e+01_rb,8.8179e+00_rb,4.7428e+00_rb,9.2252e-02_rb/)
      kbo(:, 4,20, 5) = (/ &
     &1.7848e+01_rb,1.3492e+01_rb,9.1779e+00_rb,4.9264e+00_rb,9.6228e-02_rb/)
      kbo(:, 5,20, 5) = (/ &
     &1.8606e+01_rb,1.4060e+01_rb,9.5528e+00_rb,5.1134e+00_rb,9.9993e-02_rb/)
      kbo(:, 1,21, 5) = (/ &
     &1.1787e+01_rb,8.9459e+00_rb,6.1836e+00_rb,3.3861e+00_rb,7.2353e-02_rb/)
      kbo(:, 2,21, 5) = (/ &
     &1.2303e+01_rb,9.3354e+00_rb,6.4401e+00_rb,3.5254e+00_rb,7.6084e-02_rb/)
      kbo(:, 3,21, 5) = (/ &
     &1.2826e+01_rb,9.7292e+00_rb,6.7034e+00_rb,3.6681e+00_rb,7.9750e-02_rb/)
      kbo(:, 4,21, 5) = (/ &
     &1.3367e+01_rb,1.0136e+01_rb,6.9714e+00_rb,3.8111e+00_rb,8.3233e-02_rb/)
      kbo(:, 5,21, 5) = (/ &
     &1.3957e+01_rb,1.0578e+01_rb,7.2613e+00_rb,3.9589e+00_rb,8.6478e-02_rb/)
      kbo(:, 1,22, 5) = (/ &
     &9.0718e+00_rb,6.9242e+00_rb,4.8361e+00_rb,2.6781e+00_rb,6.2307e-02_rb/)
      kbo(:, 2,22, 5) = (/ &
     &9.4616e+00_rb,7.2180e+00_rb,5.0370e+00_rb,2.7912e+00_rb,6.5577e-02_rb/)
      kbo(:, 3,22, 5) = (/ &
     &9.8598e+00_rb,7.5169e+00_rb,5.2399e+00_rb,2.9065e+00_rb,6.8755e-02_rb/)
      kbo(:, 4,22, 5) = (/ &
     &1.0285e+01_rb,7.8355e+00_rb,5.4531e+00_rb,3.0213e+00_rb,7.1732e-02_rb/)
      kbo(:, 5,22, 5) = (/ &
     &1.0774e+01_rb,8.1994e+00_rb,5.6924e+00_rb,3.1454e+00_rb,7.4544e-02_rb/)
      kbo(:, 1,23, 5) = (/ &
     &7.0162e+00_rb,5.3998e+00_rb,3.8098e+00_rb,2.1262e+00_rb,5.3397e-02_rb/)
      kbo(:, 2,23, 5) = (/ &
     &7.3118e+00_rb,5.6231e+00_rb,3.9665e+00_rb,2.2161e+00_rb,5.6221e-02_rb/)
      kbo(:, 3,23, 5) = (/ &
     &7.6257e+00_rb,5.8582e+00_rb,4.1301e+00_rb,2.3090e+00_rb,5.8902e-02_rb/)
      kbo(:, 4,23, 5) = (/ &
     &7.9740e+00_rb,6.1181e+00_rb,4.3060e+00_rb,2.4033e+00_rb,6.1507e-02_rb/)
      kbo(:, 5,23, 5) = (/ &
     &8.3852e+00_rb,6.4219e+00_rb,4.5083e+00_rb,2.5100e+00_rb,6.3983e-02_rb/)
      kbo(:, 1,24, 5) = (/ &
     &5.3945e+00_rb,4.1959e+00_rb,2.9887e+00_rb,1.6764e+00_rb,4.5562e-02_rb/)
      kbo(:, 2,24, 5) = (/ &
     &5.6263e+00_rb,4.3723e+00_rb,3.1153e+00_rb,1.7491e+00_rb,4.7953e-02_rb/)
      kbo(:, 3,24, 5) = (/ &
     &5.8803e+00_rb,4.5632e+00_rb,3.2505e+00_rb,1.8236e+00_rb,5.0272e-02_rb/)
      kbo(:, 4,24, 5) = (/ &
     &6.1740e+00_rb,4.7820e+00_rb,3.4008e+00_rb,1.9048e+00_rb,5.2561e-02_rb/)
      kbo(:, 5,24, 5) = (/ &
     &6.5136e+00_rb,5.0303e+00_rb,3.5679e+00_rb,1.9952e+00_rb,5.4580e-02_rb/)
      kbo(:, 1,25, 5) = (/ &
     &4.2050e+00_rb,3.3091e+00_rb,2.3724e+00_rb,1.3359e+00_rb,3.8734e-02_rb/)
      kbo(:, 2,25, 5) = (/ &
     &4.3935e+00_rb,3.4537e+00_rb,2.4783e+00_rb,1.3955e+00_rb,4.0785e-02_rb/)
      kbo(:, 3,25, 5) = (/ &
     &4.6090e+00_rb,3.6169e+00_rb,2.5944e+00_rb,1.4600e+00_rb,4.2817e-02_rb/)
      kbo(:, 4,25, 5) = (/ &
     &4.8571e+00_rb,3.8012e+00_rb,2.7227e+00_rb,1.5304e+00_rb,4.4658e-02_rb/)
      kbo(:, 5,25, 5) = (/ &
     &5.1379e+00_rb,4.0071e+00_rb,2.8658e+00_rb,1.6091e+00_rb,4.6106e-02_rb/)
      kbo(:, 1,26, 5) = (/ &
     &3.3393e+00_rb,2.6590e+00_rb,1.9126e+00_rb,1.0812e+00_rb,3.2872e-02_rb/)
      kbo(:, 2,26, 5) = (/ &
     &3.4998e+00_rb,2.7840e+00_rb,2.0042e+00_rb,1.1322e+00_rb,3.4655e-02_rb/)
      kbo(:, 3,26, 5) = (/ &
     &3.6839e+00_rb,2.9244e+00_rb,2.1044e+00_rb,1.1892e+00_rb,3.6289e-02_rb/)
      kbo(:, 4,26, 5) = (/ &
     &3.8971e+00_rb,3.0834e+00_rb,2.2165e+00_rb,1.2514e+00_rb,3.7614e-02_rb/)
      kbo(:, 5,26, 5) = (/ &
     &4.1266e+00_rb,3.2542e+00_rb,2.3379e+00_rb,1.3199e+00_rb,3.8535e-02_rb/)
      kbo(:, 1,27, 5) = (/ &
     &2.7587e+00_rb,2.2186e+00_rb,1.5971e+00_rb,9.0467e-01_rb,2.7863e-02_rb/)
      kbo(:, 2,27, 5) = (/ &
     &2.9012e+00_rb,2.3317e+00_rb,1.6793e+00_rb,9.5148e-01_rb,2.9310e-02_rb/)
      kbo(:, 3,27, 5) = (/ &
     &3.0653e+00_rb,2.4577e+00_rb,1.7694e+00_rb,1.0026e+00_rb,3.0500e-02_rb/)
      kbo(:, 4,27, 5) = (/ &
     &3.2491e+00_rb,2.5968e+00_rb,1.8696e+00_rb,1.0593e+00_rb,3.1389e-02_rb/)
      kbo(:, 5,27, 5) = (/ &
     &3.4443e+00_rb,2.7432e+00_rb,1.9752e+00_rb,1.1204e+00_rb,3.1994e-02_rb/)
      kbo(:, 1,28, 5) = (/ &
     &2.3461e+00_rb,1.9019e+00_rb,1.3684e+00_rb,7.7601e-01_rb,2.3534e-02_rb/)
      kbo(:, 2,28, 5) = (/ &
     &2.4760e+00_rb,2.0059e+00_rb,1.4439e+00_rb,8.1944e-01_rb,2.4611e-02_rb/)
      kbo(:, 3,28, 5) = (/ &
     &2.6235e+00_rb,2.1209e+00_rb,1.5268e+00_rb,8.6687e-01_rb,2.5434e-02_rb/)
      kbo(:, 4,28, 5) = (/ &
     &2.7836e+00_rb,2.2441e+00_rb,1.6160e+00_rb,9.1849e-01_rb,2.6044e-02_rb/)
      kbo(:, 5,28, 5) = (/ &
     &2.9562e+00_rb,2.3753e+00_rb,1.7120e+00_rb,9.7389e-01_rb,2.6449e-02_rb/)
      kbo(:, 1,29, 5) = (/ &
     &2.1229e+00_rb,1.7281e+00_rb,1.2411e+00_rb,7.0258e-01_rb,1.9767e-02_rb/)
      kbo(:, 2,29, 5) = (/ &
     &2.2483e+00_rb,1.8286e+00_rb,1.3142e+00_rb,7.4438e-01_rb,2.0526e-02_rb/)
      kbo(:, 3,29, 5) = (/ &
     &2.3867e+00_rb,1.9376e+00_rb,1.3932e+00_rb,7.8990e-01_rb,2.1121e-02_rb/)
      kbo(:, 4,29, 5) = (/ &
     &2.5370e+00_rb,2.0544e+00_rb,1.4782e+00_rb,8.3936e-01_rb,2.1540e-02_rb/)
      kbo(:, 5,29, 5) = (/ &
     &2.7005e+00_rb,2.1808e+00_rb,1.5709e+00_rb,8.9251e-01_rb,2.1826e-02_rb/)
      kbo(:, 1,30, 5) = (/ &
     &1.9731e+00_rb,1.6098e+00_rb,1.1534e+00_rb,6.5097e-01_rb,1.6498e-02_rb/)
      kbo(:, 2,30, 5) = (/ &
     &2.0958e+00_rb,1.7088e+00_rb,1.2257e+00_rb,6.9208e-01_rb,1.7054e-02_rb/)
      kbo(:, 3,30, 5) = (/ &
     &2.2285e+00_rb,1.8139e+00_rb,1.3022e+00_rb,7.3642e-01_rb,1.7482e-02_rb/)
      kbo(:, 4,30, 5) = (/ &
     &2.3729e+00_rb,1.9280e+00_rb,1.3861e+00_rb,7.8518e-01_rb,1.7790e-02_rb/)
      kbo(:, 5,30, 5) = (/ &
     &2.5353e+00_rb,2.0546e+00_rb,1.4785e+00_rb,8.3767e-01_rb,1.7989e-02_rb/)
      kbo(:, 1,31, 5) = (/ &
     &1.9340e+00_rb,1.5772e+00_rb,1.1264e+00_rb,6.3182e-01_rb,1.3731e-02_rb/)
      kbo(:, 2,31, 5) = (/ &
     &2.0574e+00_rb,1.6773e+00_rb,1.1995e+00_rb,6.7358e-01_rb,1.4152e-02_rb/)
      kbo(:, 3,31, 5) = (/ &
     &2.1914e+00_rb,1.7848e+00_rb,1.2781e+00_rb,7.1915e-01_rb,1.4466e-02_rb/)
      kbo(:, 4,31, 5) = (/ &
     &2.3406e+00_rb,1.9025e+00_rb,1.3648e+00_rb,7.6925e-01_rb,1.4687e-02_rb/)
      kbo(:, 5,31, 5) = (/ &
     &2.5107e+00_rb,2.0354e+00_rb,1.4612e+00_rb,8.2397e-01_rb,1.4843e-02_rb/)
      kbo(:, 1,32, 5) = (/ &
     &1.9204e+00_rb,1.5654e+00_rb,1.1148e+00_rb,6.2147e-01_rb,1.1411e-02_rb/)
      kbo(:, 2,32, 5) = (/ &
     &2.0455e+00_rb,1.6672e+00_rb,1.1895e+00_rb,6.6431e-01_rb,1.1723e-02_rb/)
      kbo(:, 3,32, 5) = (/ &
     &2.1839e+00_rb,1.7785e+00_rb,1.2711e+00_rb,7.1180e-01_rb,1.1953e-02_rb/)
      kbo(:, 4,32, 5) = (/ &
     &2.3424e+00_rb,1.9038e+00_rb,1.3629e+00_rb,7.6419e-01_rb,1.2125e-02_rb/)
      kbo(:, 5,32, 5) = (/ &
     &2.5216e+00_rb,2.0435e+00_rb,1.4643e+00_rb,8.2232e-01_rb,1.2240e-02_rb/)
      kbo(:, 1,33, 5) = (/ &
     &1.9423e+00_rb,1.5823e+00_rb,1.1236e+00_rb,6.2280e-01_rb,9.4667e-03_rb/)
      kbo(:, 2,33, 5) = (/ &
     &2.0728e+00_rb,1.6884e+00_rb,1.2018e+00_rb,6.6778e-01_rb,9.6991e-03_rb/)
      kbo(:, 3,33, 5) = (/ &
     &2.2222e+00_rb,1.8079e+00_rb,1.2892e+00_rb,7.1818e-01_rb,9.8807e-03_rb/)
      kbo(:, 4,33, 5) = (/ &
     &2.3928e+00_rb,1.9430e+00_rb,1.3880e+00_rb,7.7444e-01_rb,1.0013e-02_rb/)
      kbo(:, 5,33, 5) = (/ &
     &2.5873e+00_rb,2.0935e+00_rb,1.4975e+00_rb,8.3717e-01_rb,1.0093e-02_rb/)
      kbo(:, 1,34, 5) = (/ &
     &1.9127e+00_rb,1.5588e+00_rb,1.1053e+00_rb,6.1048e-01_rb,7.8309e-03_rb/)
      kbo(:, 2,34, 5) = (/ &
     &2.0473e+00_rb,1.6687e+00_rb,1.1862e+00_rb,6.5698e-01_rb,8.0168e-03_rb/)
      kbo(:, 3,34, 5) = (/ &
     &2.2050e+00_rb,1.7947e+00_rb,1.2785e+00_rb,7.0977e-01_rb,8.1571e-03_rb/)
      kbo(:, 4,34, 5) = (/ &
     &2.3844e+00_rb,1.9366e+00_rb,1.3818e+00_rb,7.6872e-01_rb,8.2588e-03_rb/)
      kbo(:, 5,34, 5) = (/ &
     &2.5878e+00_rb,2.0945e+00_rb,1.4977e+00_rb,8.3524e-01_rb,8.3102e-03_rb/)
      kbo(:, 1,35, 5) = (/ &
     &1.8762e+00_rb,1.5298e+00_rb,1.0833e+00_rb,5.9624e-01_rb,6.4645e-03_rb/)
      kbo(:, 2,35, 5) = (/ &
     &2.0161e+00_rb,1.6440e+00_rb,1.1673e+00_rb,6.4448e-01_rb,6.6144e-03_rb/)
      kbo(:, 3,35, 5) = (/ &
     &2.1796e+00_rb,1.7750e+00_rb,1.2632e+00_rb,6.9909e-01_rb,6.7276e-03_rb/)
      kbo(:, 4,35, 5) = (/ &
     &2.3668e+00_rb,1.9227e+00_rb,1.3710e+00_rb,7.6090e-01_rb,6.8022e-03_rb/)
      kbo(:, 5,35, 5) = (/ &
     &2.5757e+00_rb,2.0863e+00_rb,1.4915e+00_rb,8.3029e-01_rb,6.8468e-03_rb/)
      kbo(:, 1,36, 5) = (/ &
     &1.8177e+00_rb,1.4833e+00_rb,1.0492e+00_rb,5.7567e-01_rb,5.3254e-03_rb/)
      kbo(:, 2,36, 5) = (/ &
     &1.9605e+00_rb,1.6001e+00_rb,1.1349e+00_rb,6.2477e-01_rb,5.4464e-03_rb/)
      kbo(:, 3,36, 5) = (/ &
     &2.1269e+00_rb,1.7343e+00_rb,1.2332e+00_rb,6.8062e-01_rb,5.5407e-03_rb/)
      kbo(:, 4,36, 5) = (/ &
     &2.3182e+00_rb,1.8849e+00_rb,1.3435e+00_rb,7.4416e-01_rb,5.6019e-03_rb/)
      kbo(:, 5,36, 5) = (/ &
     &2.5291e+00_rb,2.0513e+00_rb,1.4663e+00_rb,8.1509e-01_rb,5.6408e-03_rb/)
      kbo(:, 1,37, 5) = (/ &
     &1.6997e+00_rb,1.3894e+00_rb,9.8190e-01_rb,5.3748e-01_rb,4.3666e-03_rb/)
      kbo(:, 2,37, 5) = (/ &
     &1.8378e+00_rb,1.5030e+00_rb,1.0654e+00_rb,5.8540e-01_rb,4.4691e-03_rb/)
      kbo(:, 3,37, 5) = (/ &
     &1.9994e+00_rb,1.6341e+00_rb,1.1617e+00_rb,6.4019e-01_rb,4.5481e-03_rb/)
      kbo(:, 4,37, 5) = (/ &
     &2.1860e+00_rb,1.7826e+00_rb,1.2707e+00_rb,7.0283e-01_rb,4.6056e-03_rb/)
      kbo(:, 5,37, 5) = (/ &
     &2.3914e+00_rb,1.9456e+00_rb,1.3911e+00_rb,7.7270e-01_rb,4.6416e-03_rb/)
      kbo(:, 1,38, 5) = (/ &
     &1.6123e+00_rb,1.3193e+00_rb,9.3139e-01_rb,5.0849e-01_rb,3.5784e-03_rb/)
      kbo(:, 2,38, 5) = (/ &
     &1.7479e+00_rb,1.4316e+00_rb,1.0139e+00_rb,5.5580e-01_rb,3.6670e-03_rb/)
      kbo(:, 3,38, 5) = (/ &
     &1.9074e+00_rb,1.5615e+00_rb,1.1092e+00_rb,6.1015e-01_rb,3.7371e-03_rb/)
      kbo(:, 4,38, 5) = (/ &
     &2.0911e+00_rb,1.7089e+00_rb,1.2176e+00_rb,6.7256e-01_rb,3.7869e-03_rb/)
      kbo(:, 5,38, 5) = (/ &
     &2.2948e+00_rb,1.8712e+00_rb,1.3379e+00_rb,7.4236e-01_rb,3.8180e-03_rb/)
      kbo(:, 1,39, 5) = (/ &
     &1.5653e+00_rb,1.2809e+00_rb,9.0306e-01_rb,4.9152e-01_rb,2.9339e-03_rb/)
      kbo(:, 2,39, 5) = (/ &
     &1.7011e+00_rb,1.3939e+00_rb,9.8605e-01_rb,5.3903e-01_rb,3.0110e-03_rb/)
      kbo(:, 3,39, 5) = (/ &
     &1.8625e+00_rb,1.5257e+00_rb,1.0827e+00_rb,5.9416e-01_rb,3.0720e-03_rb/)
      kbo(:, 4,39, 5) = (/ &
     &2.0480e+00_rb,1.6749e+00_rb,1.1927e+00_rb,6.5763e-01_rb,3.1176e-03_rb/)
      kbo(:, 5,39, 5) = (/ &
     &2.2545e+00_rb,1.8403e+00_rb,1.3150e+00_rb,7.2854e-01_rb,3.1424e-03_rb/)
      kbo(:, 1,40, 5) = (/ &
     &1.4161e+00_rb,1.1618e+00_rb,8.1883e-01_rb,4.4506e-01_rb,2.3961e-03_rb/)
      kbo(:, 2,40, 5) = (/ &
     &1.5422e+00_rb,1.2675e+00_rb,8.9661e-01_rb,4.8961e-01_rb,2.4634e-03_rb/)
      kbo(:, 3,40, 5) = (/ &
     &1.6929e+00_rb,1.3914e+00_rb,9.8762e-01_rb,5.4178e-01_rb,2.5182e-03_rb/)
      kbo(:, 4,40, 5) = (/ &
     &1.8671e+00_rb,1.5327e+00_rb,1.0918e+00_rb,6.0196e-01_rb,2.5595e-03_rb/)
      kbo(:, 5,40, 5) = (/ &
     &2.0610e+00_rb,1.6900e+00_rb,1.2084e+00_rb,6.6961e-01_rb,2.5831e-03_rb/)
      kbo(:, 1,41, 5) = (/ &
     &1.2765e+00_rb,1.0500e+00_rb,7.3977e-01_rb,4.0158e-01_rb,1.9558e-03_rb/)
      kbo(:, 2,41, 5) = (/ &
     &1.3934e+00_rb,1.1486e+00_rb,8.1252e-01_rb,4.4320e-01_rb,2.0159e-03_rb/)
      kbo(:, 3,41, 5) = (/ &
     &1.5329e+00_rb,1.2644e+00_rb,8.9774e-01_rb,4.9220e-01_rb,2.0644e-03_rb/)
      kbo(:, 4,41, 5) = (/ &
     &1.6957e+00_rb,1.3977e+00_rb,9.9605e-01_rb,5.4916e-01_rb,2.0998e-03_rb/)
      kbo(:, 5,41, 5) = (/ &
     &1.8778e+00_rb,1.5466e+00_rb,1.1067e+00_rb,6.1346e-01_rb,2.1224e-03_rb/)
      kbo(:, 1,42, 5) = (/ &
     &1.1627e+00_rb,9.5841e-01_rb,6.7494e-01_rb,3.6583e-01_rb,1.5975e-03_rb/)
      kbo(:, 2,42, 5) = (/ &
     &1.2723e+00_rb,1.0513e+00_rb,7.4339e-01_rb,4.0496e-01_rb,1.6498e-03_rb/)
      kbo(:, 3,42, 5) = (/ &
     &1.4029e+00_rb,1.1604e+00_rb,8.2397e-01_rb,4.5136e-01_rb,1.6921e-03_rb/)
      kbo(:, 4,42, 5) = (/ &
     &1.5564e+00_rb,1.2873e+00_rb,9.1770e-01_rb,5.0574e-01_rb,1.7229e-03_rb/)
      kbo(:, 5,42, 5) = (/ &
     &1.7292e+00_rb,1.4295e+00_rb,1.0235e+00_rb,5.6732e-01_rb,1.7439e-03_rb/)
      kbo(:, 1,43, 5) = (/ &
     &1.0630e+00_rb,8.7767e-01_rb,6.1763e-01_rb,3.3410e-01_rb,1.3020e-03_rb/)
      kbo(:, 2,43, 5) = (/ &
     &1.1654e+00_rb,9.6474e-01_rb,6.8178e-01_rb,3.7078e-01_rb,1.3472e-03_rb/)
      kbo(:, 3,43, 5) = (/ &
     &1.2879e+00_rb,1.0676e+00_rb,7.5775e-01_rb,4.1456e-01_rb,1.3843e-03_rb/)
      kbo(:, 4,43, 5) = (/ &
     &1.4323e+00_rb,1.1883e+00_rb,8.4702e-01_rb,4.6628e-01_rb,1.4122e-03_rb/)
      kbo(:, 5,43, 5) = (/ &
     &1.5961e+00_rb,1.3241e+00_rb,9.4813e-01_rb,5.2525e-01_rb,1.4316e-03_rb/)
      kbo(:, 1,44, 5) = (/ &
     &9.8265e-01_rb,8.1189e-01_rb,5.7078e-01_rb,3.0805e-01_rb,1.0595e-03_rb/)
      kbo(:, 2,44, 5) = (/ &
     &1.0790e+00_rb,8.9402e-01_rb,6.3105e-01_rb,3.4248e-01_rb,1.0984e-03_rb/)
      kbo(:, 3,44, 5) = (/ &
     &1.1944e+00_rb,9.9169e-01_rb,7.0312e-01_rb,3.8396e-01_rb,1.1318e-03_rb/)
      kbo(:, 4,44, 5) = (/ &
     &1.3318e+00_rb,1.1077e+00_rb,7.8899e-01_rb,4.3356e-01_rb,1.1566e-03_rb/)
      kbo(:, 5,44, 5) = (/ &
     &1.4887e+00_rb,1.2383e+00_rb,8.8644e-01_rb,4.9053e-01_rb,1.1752e-03_rb/)
      kbo(:, 1,45, 5) = (/ &
     &9.2676e-01_rb,7.6543e-01_rb,5.3731e-01_rb,2.8917e-01_rb,8.6207e-04_rb/)
      kbo(:, 2,45, 5) = (/ &
     &1.0188e+00_rb,8.4420e-01_rb,5.9503e-01_rb,3.2211e-01_rb,8.9589e-04_rb/)
      kbo(:, 3,45, 5) = (/ &
     &1.1295e+00_rb,9.3840e-01_rb,6.6449e-01_rb,3.6207e-01_rb,9.2491e-04_rb/)
      kbo(:, 4,45, 5) = (/ &
     &1.2624e+00_rb,1.0512e+00_rb,7.4797e-01_rb,4.1011e-01_rb,9.4724e-04_rb/)
      kbo(:, 5,45, 5) = (/ &
     &1.4159e+00_rb,1.1794e+00_rb,8.4359e-01_rb,4.6598e-01_rb,9.6431e-04_rb/)
      kbo(:, 1,46, 5) = (/ &
     &8.8115e-01_rb,7.2712e-01_rb,5.0946e-01_rb,2.7333e-01_rb,7.0044e-04_rb/)
      kbo(:, 2,46, 5) = (/ &
     &9.6873e-01_rb,8.0252e-01_rb,5.6477e-01_rb,3.0486e-01_rb,7.2940e-04_rb/)
      kbo(:, 3,46, 5) = (/ &
     &1.0759e+00_rb,8.9393e-01_rb,6.3210e-01_rb,3.4344e-01_rb,7.5456e-04_rb/)
      kbo(:, 4,46, 5) = (/ &
     &1.2051e+00_rb,1.0038e+00_rb,7.1323e-01_rb,3.9017e-01_rb,7.7543e-04_rb/)
      kbo(:, 5,46, 5) = (/ &
     &1.3559e+00_rb,1.1306e+00_rb,8.0761e-01_rb,4.4513e-01_rb,7.9123e-04_rb/)
      kbo(:, 1,47, 5) = (/ &
     &8.2708e-01_rb,6.8179e-01_rb,4.7670e-01_rb,2.5496e-01_rb,5.6723e-04_rb/)
      kbo(:, 2,47, 5) = (/ &
     &9.0841e-01_rb,7.5254e-01_rb,5.2877e-01_rb,2.8459e-01_rb,5.9237e-04_rb/)
      kbo(:, 3,47, 5) = (/ &
     &1.0102e+00_rb,8.3938e-01_rb,5.9264e-01_rb,3.2108e-01_rb,6.1461e-04_rb/)
      kbo(:, 4,47, 5) = (/ &
     &1.1336e+00_rb,9.4433e-01_rb,6.7005e-01_rb,3.6564e-01_rb,6.3367e-04_rb/)
      kbo(:, 5,47, 5) = (/ &
     &1.2789e+00_rb,1.0675e+00_rb,7.6147e-01_rb,4.1873e-01_rb,6.4826e-04_rb/)
      kbo(:, 1,48, 5) = (/ &
     &7.9882e-01_rb,6.5670e-01_rb,4.5801e-01_rb,2.4404e-01_rb,4.5910e-04_rb/)
      kbo(:, 2,48, 5) = (/ &
     &8.7602e-01_rb,7.2469e-01_rb,5.0814e-01_rb,2.7259e-01_rb,4.8082e-04_rb/)
      kbo(:, 3,48, 5) = (/ &
     &9.7485e-01_rb,8.0906e-01_rb,5.6998e-01_rb,3.0783e-01_rb,5.0059e-04_rb/)
      kbo(:, 4,48, 5) = (/ &
     &1.0951e+00_rb,9.1136e-01_rb,6.4533e-01_rb,3.5105e-01_rb,5.1769e-04_rb/)
      kbo(:, 5,48, 5) = (/ &
     &1.2389e+00_rb,1.0335e+00_rb,7.3563e-01_rb,4.0331e-01_rb,5.3087e-04_rb/)
      kbo(:, 1,49, 5) = (/ &
     &8.0402e-01_rb,6.5823e-01_rb,4.5761e-01_rb,2.4276e-01_rb,3.7137e-04_rb/)
      kbo(:, 2,49, 5) = (/ &
     &8.8031e-01_rb,7.2577e-01_rb,5.0751e-01_rb,2.7123e-01_rb,3.9017e-04_rb/)
      kbo(:, 3,49, 5) = (/ &
     &9.7946e-01_rb,8.1065e-01_rb,5.6941e-01_rb,3.0628e-01_rb,4.0777e-04_rb/)
      kbo(:, 4,49, 5) = (/ &
     &1.1011e+00_rb,9.1410e-01_rb,6.4521e-01_rb,3.4961e-01_rb,4.2272e-04_rb/)
      kbo(:, 5,49, 5) = (/ &
     &1.2484e+00_rb,1.0389e+00_rb,7.3730e-01_rb,4.0277e-01_rb,4.3479e-04_rb/)
      kbo(:, 1,50, 5) = (/ &
     &7.7254e-01_rb,6.3104e-01_rb,4.3765e-01_rb,2.3137e-01_rb,3.0017e-04_rb/)
      kbo(:, 2,50, 5) = (/ &
     &8.4478e-01_rb,6.9541e-01_rb,4.8529e-01_rb,2.5862e-01_rb,3.1617e-04_rb/)
      kbo(:, 3,50, 5) = (/ &
     &9.3925e-01_rb,7.7679e-01_rb,5.4465e-01_rb,2.9213e-01_rb,3.3160e-04_rb/)
      kbo(:, 4,50, 5) = (/ &
     &1.0567e+00_rb,8.7678e-01_rb,6.1788e-01_rb,3.3395e-01_rb,3.4487e-04_rb/)
      kbo(:, 5,50, 5) = (/ &
     &1.2008e+00_rb,9.9924e-01_rb,7.0798e-01_rb,3.8577e-01_rb,3.5577e-04_rb/)
      kbo(:, 1,51, 5) = (/ &
     &7.2669e-01_rb,5.9284e-01_rb,4.1030e-01_rb,2.1627e-01_rb,2.4246e-04_rb/)
      kbo(:, 2,51, 5) = (/ &
     &7.9410e-01_rb,6.5300e-01_rb,4.5497e-01_rb,2.4184e-01_rb,2.5602e-04_rb/)
      kbo(:, 3,51, 5) = (/ &
     &8.8167e-01_rb,7.2916e-01_rb,5.1056e-01_rb,2.7327e-01_rb,2.6933e-04_rb/)
      kbo(:, 4,51, 5) = (/ &
     &9.9265e-01_rb,8.2410e-01_rb,5.8011e-01_rb,3.1286e-01_rb,2.8099e-04_rb/)
      kbo(:, 5,51, 5) = (/ &
     &1.1303e+00_rb,9.4117e-01_rb,6.6603e-01_rb,3.6222e-01_rb,2.9075e-04_rb/)
      kbo(:, 1,52, 5) = (/ &
     &7.0293e-01_rb,5.7224e-01_rb,3.9526e-01_rb,2.0761e-01_rb,1.9566e-04_rb/)
      kbo(:, 2,52, 5) = (/ &
     &7.6814e-01_rb,6.3021e-01_rb,4.3806e-01_rb,2.3215e-01_rb,2.0717e-04_rb/)
      kbo(:, 3,52, 5) = (/ &
     &8.5099e-01_rb,7.0282e-01_rb,4.9133e-01_rb,2.6231e-01_rb,2.1874e-04_rb/)
      kbo(:, 4,52, 5) = (/ &
     &9.5857e-01_rb,7.9517e-01_rb,5.5876e-01_rb,3.0061e-01_rb,2.2893e-04_rb/)
      kbo(:, 5,52, 5) = (/ &
     &1.0928e+00_rb,9.0923e-01_rb,6.4230e-01_rb,3.4850e-01_rb,2.3751e-04_rb/)
      kbo(:, 1,53, 5) = (/ &
     &7.0799e-01_rb,5.7451e-01_rb,3.9589e-01_rb,2.0725e-01_rb,1.5775e-04_rb/)
      kbo(:, 2,53, 5) = (/ &
     &7.7421e-01_rb,6.3284e-01_rb,4.3870e-01_rb,2.3160e-01_rb,1.6771e-04_rb/)
      kbo(:, 3,53, 5) = (/ &
     &8.5551e-01_rb,7.0456e-01_rb,4.9136e-01_rb,2.6152e-01_rb,1.7761e-04_rb/)
      kbo(:, 4,53, 5) = (/ &
     &9.6362e-01_rb,7.9737e-01_rb,5.5891e-01_rb,2.9974e-01_rb,1.8647e-04_rb/)
      kbo(:, 5,53, 5) = (/ &
     &1.0993e+00_rb,9.1246e-01_rb,6.4296e-01_rb,3.4771e-01_rb,1.9425e-04_rb/)
      kbo(:, 1,54, 5) = (/ &
     &6.1781e-01_rb,5.0174e-01_rb,3.4551e-01_rb,1.8062e-01_rb,1.2698e-04_rb/)
      kbo(:, 2,54, 5) = (/ &
     &6.7617e-01_rb,5.5332e-01_rb,3.8324e-01_rb,2.0204e-01_rb,1.3548e-04_rb/)
      kbo(:, 3,54, 5) = (/ &
     &7.4609e-01_rb,6.1568e-01_rb,4.2927e-01_rb,2.2831e-01_rb,1.4360e-04_rb/)
      kbo(:, 4,54, 5) = (/ &
     &8.3975e-01_rb,6.9703e-01_rb,4.8868e-01_rb,2.6191e-01_rb,1.5134e-04_rb/)
      kbo(:, 5,54, 5) = (/ &
     &9.5851e-01_rb,7.9875e-01_rb,5.6310e-01_rb,3.0440e-01_rb,1.5803e-04_rb/)
      kbo(:, 1,55, 5) = (/ &
     &4.9218e-01_rb,4.0097e-01_rb,2.7614e-01_rb,1.4430e-01_rb,1.0197e-04_rb/)
      kbo(:, 2,55, 5) = (/ &
     &5.3911e-01_rb,4.4293e-01_rb,3.0686e-01_rb,1.6168e-01_rb,1.0930e-04_rb/)
      kbo(:, 3,55, 5) = (/ &
     &5.9446e-01_rb,4.9291e-01_rb,3.4401e-01_rb,1.8302e-01_rb,1.1592e-04_rb/)
      kbo(:, 4,55, 5) = (/ &
     &6.6816e-01_rb,5.5826e-01_rb,3.9205e-01_rb,2.1030e-01_rb,1.2249e-04_rb/)
      kbo(:, 5,55, 5) = (/ &
     &7.6300e-01_rb,6.4076e-01_rb,4.5273e-01_rb,2.4510e-01_rb,1.2815e-04_rb/)
      kbo(:, 1,56, 5) = (/ &
     &3.9213e-01_rb,3.2040e-01_rb,2.2062e-01_rb,1.1519e-01_rb,8.1886e-05_rb/)
      kbo(:, 2,56, 5) = (/ &
     &4.2970e-01_rb,3.5433e-01_rb,2.4555e-01_rb,1.2928e-01_rb,8.8029e-05_rb/)
      kbo(:, 3,56, 5) = (/ &
     &4.7372e-01_rb,3.9458e-01_rb,2.7557e-01_rb,1.4657e-01_rb,9.3592e-05_rb/)
      kbo(:, 4,56, 5) = (/ &
     &5.3136e-01_rb,4.4682e-01_rb,3.1425e-01_rb,1.6867e-01_rb,9.8924e-05_rb/)
      kbo(:, 5,56, 5) = (/ &
     &6.0680e-01_rb,5.1356e-01_rb,3.6377e-01_rb,1.9714e-01_rb,1.0388e-04_rb/)
      kbo(:, 1,57, 5) = (/ &
     &3.1256e-01_rb,2.5603e-01_rb,1.7627e-01_rb,9.1926e-02_rb,6.5686e-05_rb/)
      kbo(:, 2,57, 5) = (/ &
     &3.4230e-01_rb,2.8322e-01_rb,1.9630e-01_rb,1.0325e-01_rb,7.0810e-05_rb/)
      kbo(:, 3,57, 5) = (/ &
     &3.7748e-01_rb,3.1574e-01_rb,2.2058e-01_rb,1.1723e-01_rb,7.5533e-05_rb/)
      kbo(:, 4,57, 5) = (/ &
     &4.2229e-01_rb,3.5725e-01_rb,2.5162e-01_rb,1.3506e-01_rb,7.9907e-05_rb/)
      kbo(:, 5,57, 5) = (/ &
     &4.8209e-01_rb,4.1127e-01_rb,2.9188e-01_rb,1.5833e-01_rb,8.4096e-05_rb/)
      kbo(:, 1,58, 5) = (/ &
     &6.5834e-02_rb,6.6132e-02_rb,5.8548e-02_rb,4.2734e-02_rb,5.2667e-05_rb/)
      kbo(:, 2,58, 5) = (/ &
     &7.1968e-02_rb,7.3109e-02_rb,6.5197e-02_rb,4.8002e-02_rb,5.6984e-05_rb/)
      kbo(:, 3,58, 5) = (/ &
     &7.9437e-02_rb,8.1656e-02_rb,7.3428e-02_rb,5.4631e-02_rb,6.0874e-05_rb/)
      kbo(:, 4,58, 5) = (/ &
     &8.8662e-02_rb,9.2383e-02_rb,8.3809e-02_rb,6.3035e-02_rb,6.4619e-05_rb/)
      kbo(:, 5,58, 5) = (/ &
     &1.0118e-01_rb,1.0653e-01_rb,9.7458e-02_rb,7.4110e-02_rb,6.7988e-05_rb/)
      kbo(:, 1,59, 5) = (/ &
     &6.3055e-02_rb,6.1069e-02_rb,5.0451e-02_rb,3.2825e-02_rb,4.2796e-05_rb/)
      kbo(:, 2,59, 5) = (/ &
     &6.8995e-02_rb,6.7782e-02_rb,5.6462e-02_rb,3.7059e-02_rb,4.6289e-05_rb/)
      kbo(:, 3,59, 5) = (/ &
     &7.6280e-02_rb,7.6111e-02_rb,6.4009e-02_rb,4.2464e-02_rb,4.9459e-05_rb/)
      kbo(:, 4,59, 5) = (/ &
     &8.5284e-02_rb,8.6606e-02_rb,7.3583e-02_rb,4.9384e-02_rb,5.2442e-05_rb/)
      kbo(:, 5,59, 5) = (/ &
     &9.7585e-02_rb,1.0050e-01_rb,8.6230e-02_rb,5.8574e-02_rb,5.5198e-05_rb/)
      kbo(:, 1,13, 6) = (/ &
     &7.7180e+02_rb,5.7896e+02_rb,3.8611e+02_rb,1.9327e+02_rb,3.6289e-01_rb/)
      kbo(:, 2,13, 6) = (/ &
     &7.9285e+02_rb,5.9474e+02_rb,3.9664e+02_rb,1.9853e+02_rb,3.7298e-01_rb/)
      kbo(:, 3,13, 6) = (/ &
     &8.1518e+02_rb,6.1149e+02_rb,4.0780e+02_rb,2.0411e+02_rb,3.8121e-01_rb/)
      kbo(:, 4,13, 6) = (/ &
     &8.3776e+02_rb,6.2842e+02_rb,4.1908e+02_rb,2.0974e+02_rb,3.8734e-01_rb/)
      kbo(:, 5,13, 6) = (/ &
     &8.6087e+02_rb,6.4575e+02_rb,4.3063e+02_rb,2.1551e+02_rb,3.9230e-01_rb/)
      kbo(:, 1,14, 6) = (/ &
     &4.1395e+02_rb,3.1056e+02_rb,2.0718e+02_rb,1.0380e+02_rb,3.3381e-01_rb/)
      kbo(:, 2,14, 6) = (/ &
     &4.2539e+02_rb,3.1915e+02_rb,2.1290e+02_rb,1.0666e+02_rb,3.4169e-01_rb/)
      kbo(:, 3,14, 6) = (/ &
     &4.3723e+02_rb,3.2803e+02_rb,2.1882e+02_rb,1.0962e+02_rb,3.4947e-01_rb/)
      kbo(:, 4,14, 6) = (/ &
     &4.4927e+02_rb,3.3705e+02_rb,2.2483e+02_rb,1.1261e+02_rb,3.5567e-01_rb/)
      kbo(:, 5,14, 6) = (/ &
     &4.6195e+02_rb,3.4656e+02_rb,2.3117e+02_rb,1.1578e+02_rb,3.6065e-01_rb/)
      kbo(:, 1,15, 6) = (/ &
     &2.2585e+02_rb,1.6949e+02_rb,1.1313e+02_rb,5.6770e+01_rb,3.0356e-01_rb/)
      kbo(:, 2,15, 6) = (/ &
     &2.3201e+02_rb,1.7411e+02_rb,1.1621e+02_rb,5.8311e+01_rb,3.1091e-01_rb/)
      kbo(:, 3,15, 6) = (/ &
     &2.3839e+02_rb,1.7889e+02_rb,1.1939e+02_rb,5.9899e+01_rb,3.1810e-01_rb/)
      kbo(:, 4,15, 6) = (/ &
     &2.4502e+02_rb,1.8386e+02_rb,1.2270e+02_rb,6.1544e+01_rb,3.2391e-01_rb/)
      kbo(:, 5,15, 6) = (/ &
     &2.5289e+02_rb,1.8977e+02_rb,1.2664e+02_rb,6.3520e+01_rb,3.2850e-01_rb/)
      kbo(:, 1,16, 6) = (/ &
     &1.4635e+02_rb,1.0986e+02_rb,7.3372e+01_rb,3.6890e+01_rb,2.7344e-01_rb/)
      kbo(:, 2,16, 6) = (/ &
     &1.5031e+02_rb,1.1283e+02_rb,7.5356e+01_rb,3.7881e+01_rb,2.8092e-01_rb/)
      kbo(:, 3,16, 6) = (/ &
     &1.5444e+02_rb,1.1592e+02_rb,7.7407e+01_rb,3.8900e+01_rb,2.8781e-01_rb/)
      kbo(:, 4,16, 6) = (/ &
     &1.5922e+02_rb,1.1951e+02_rb,7.9797e+01_rb,4.0091e+01_rb,2.9332e-01_rb/)
      kbo(:, 5,16, 6) = (/ &
     &1.6548e+02_rb,1.2420e+02_rb,8.2930e+01_rb,4.1661e+01_rb,2.9808e-01_rb/)
      kbo(:, 1,17, 6) = (/ &
     &9.8142e+01_rb,7.3705e+01_rb,4.9268e+01_rb,2.4840e+01_rb,2.4429e-01_rb/)
      kbo(:, 2,17, 6) = (/ &
     &1.0076e+02_rb,7.5668e+01_rb,5.0576e+01_rb,2.5493e+01_rb,2.5184e-01_rb/)
      kbo(:, 3,17, 6) = (/ &
     &1.0378e+02_rb,7.7926e+01_rb,5.2073e+01_rb,2.6235e+01_rb,2.5821e-01_rb/)
      kbo(:, 4,17, 6) = (/ &
     &1.0768e+02_rb,8.0849e+01_rb,5.4024e+01_rb,2.7211e+01_rb,2.6339e-01_rb/)
      kbo(:, 5,17, 6) = (/ &
     &1.1267e+02_rb,8.4599e+01_rb,5.6526e+01_rb,2.8462e+01_rb,2.6841e-01_rb/)
      kbo(:, 1,18, 6) = (/ &
     &6.7329e+01_rb,5.0594e+01_rb,3.3862e+01_rb,1.7162e+01_rb,2.1677e-01_rb/)
      kbo(:, 2,18, 6) = (/ &
     &6.9224e+01_rb,5.2009e+01_rb,3.4803e+01_rb,1.7627e+01_rb,2.2376e-01_rb/)
      kbo(:, 3,18, 6) = (/ &
     &7.1671e+01_rb,5.3842e+01_rb,3.6019e+01_rb,1.8223e+01_rb,2.2981e-01_rb/)
      kbo(:, 4,18, 6) = (/ &
     &7.4922e+01_rb,5.6283e+01_rb,3.7648e+01_rb,1.9036e+01_rb,2.3516e-01_rb/)
      kbo(:, 5,18, 6) = (/ &
     &7.8609e+01_rb,5.9048e+01_rb,3.9493e+01_rb,1.9956e+01_rb,2.4055e-01_rb/)
      kbo(:, 1,19, 6) = (/ &
     &4.5351e+01_rb,3.4111e+01_rb,2.2879e+01_rb,1.1748e+01_rb,1.9068e-01_rb/)
      kbo(:, 2,19, 6) = (/ &
     &4.6788e+01_rb,3.5180e+01_rb,2.3587e+01_rb,1.2086e+01_rb,1.9727e-01_rb/)
      kbo(:, 3,19, 6) = (/ &
     &4.8819e+01_rb,3.6704e+01_rb,2.4596e+01_rb,1.2571e+01_rb,2.0322e-01_rb/)
      kbo(:, 4,19, 6) = (/ &
     &5.1243e+01_rb,3.8523e+01_rb,2.5810e+01_rb,1.3165e+01_rb,2.0895e-01_rb/)
      kbo(:, 5,19, 6) = (/ &
     &5.3759e+01_rb,4.0411e+01_rb,2.7071e+01_rb,1.3784e+01_rb,2.1468e-01_rb/)
      kbo(:, 1,20, 6) = (/ &
     &3.3050e+01_rb,2.4885e+01_rb,1.6741e+01_rb,8.7700e+00_rb,1.6695e-01_rb/)
      kbo(:, 2,20, 6) = (/ &
     &3.4346e+01_rb,2.5850e+01_rb,1.7375e+01_rb,9.0666e+00_rb,1.7321e-01_rb/)
      kbo(:, 3,20, 6) = (/ &
     &3.6031e+01_rb,2.7114e+01_rb,1.8213e+01_rb,9.4660e+00_rb,1.7929e-01_rb/)
      kbo(:, 4,20, 6) = (/ &
     &3.7866e+01_rb,2.8491e+01_rb,1.9130e+01_rb,9.9045e+00_rb,1.8515e-01_rb/)
      kbo(:, 5,20, 6) = (/ &
     &3.9713e+01_rb,2.9876e+01_rb,2.0052e+01_rb,1.0350e+01_rb,1.9066e-01_rb/)
      kbo(:, 1,21, 6) = (/ &
     &2.4844e+01_rb,1.8731e+01_rb,1.2673e+01_rb,6.7851e+00_rb,1.4553e-01_rb/)
      kbo(:, 2,21, 6) = (/ &
     &2.6012e+01_rb,1.9600e+01_rb,1.3239e+01_rb,7.0557e+00_rb,1.5180e-01_rb/)
      kbo(:, 3,21, 6) = (/ &
     &2.7347e+01_rb,2.0602e+01_rb,1.3900e+01_rb,7.3743e+00_rb,1.5767e-01_rb/)
      kbo(:, 4,21, 6) = (/ &
     &2.8734e+01_rb,2.1643e+01_rb,1.4586e+01_rb,7.7037e+00_rb,1.6319e-01_rb/)
      kbo(:, 5,21, 6) = (/ &
     &3.0137e+01_rb,2.2696e+01_rb,1.5284e+01_rb,8.0405e+00_rb,1.6831e-01_rb/)
      kbo(:, 1,22, 6) = (/ &
     &1.9244e+01_rb,1.4532e+01_rb,9.9298e+00_rb,5.4125e+00_rb,1.2703e-01_rb/)
      kbo(:, 2,22, 6) = (/ &
     &2.0224e+01_rb,1.5263e+01_rb,1.0402e+01_rb,5.6515e+00_rb,1.3287e-01_rb/)
      kbo(:, 3,22, 6) = (/ &
     &2.1278e+01_rb,1.6053e+01_rb,1.0915e+01_rb,5.9058e+00_rb,1.3815e-01_rb/)
      kbo(:, 4,22, 6) = (/ &
     &2.2359e+01_rb,1.6864e+01_rb,1.1443e+01_rb,6.1676e+00_rb,1.4315e-01_rb/)
      kbo(:, 5,22, 6) = (/ &
     &2.3448e+01_rb,1.7683e+01_rb,1.1980e+01_rb,6.4316e+00_rb,1.4780e-01_rb/)
      kbo(:, 1,23, 6) = (/ &
     &1.5003e+01_rb,1.1361e+01_rb,7.8631e+00_rb,4.3544e+00_rb,1.1053e-01_rb/)
      kbo(:, 2,23, 6) = (/ &
     &1.5788e+01_rb,1.1946e+01_rb,8.2434e+00_rb,4.5541e+00_rb,1.1564e-01_rb/)
      kbo(:, 3,23, 6) = (/ &
     &1.6609e+01_rb,1.2560e+01_rb,8.6387e+00_rb,4.7603e+00_rb,1.2034e-01_rb/)
      kbo(:, 4,23, 6) = (/ &
     &1.7444e+01_rb,1.3184e+01_rb,9.0429e+00_rb,4.9691e+00_rb,1.2479e-01_rb/)
      kbo(:, 5,23, 6) = (/ &
     &1.8318e+01_rb,1.3840e+01_rb,9.4687e+00_rb,5.1869e+00_rb,1.2894e-01_rb/)
      kbo(:, 1,24, 6) = (/ &
     &1.1642e+01_rb,8.8646e+00_rb,6.2180e+00_rb,3.4890e+00_rb,9.5642e-02_rb/)
      kbo(:, 2,24, 6) = (/ &
     &1.2250e+01_rb,9.3143e+00_rb,6.5164e+00_rb,3.6525e+00_rb,1.0007e-01_rb/)
      kbo(:, 3,24, 6) = (/ &
     &1.2878e+01_rb,9.7800e+00_rb,6.8207e+00_rb,3.8184e+00_rb,1.0423e-01_rb/)
      kbo(:, 4,24, 6) = (/ &
     &1.3534e+01_rb,1.0267e+01_rb,7.1387e+00_rb,3.9893e+00_rb,1.0818e-01_rb/)
      kbo(:, 5,24, 6) = (/ &
     &1.4255e+01_rb,1.0806e+01_rb,7.4905e+00_rb,4.1737e+00_rb,1.1190e-01_rb/)
      kbo(:, 1,25, 6) = (/ &
     &9.1475e+00_rb,7.0241e+00_rb,4.9880e+00_rb,2.8234e+00_rb,8.2353e-02_rb/)
      kbo(:, 2,25, 6) = (/ &
     &9.6194e+00_rb,7.3705e+00_rb,5.2236e+00_rb,2.9556e+00_rb,8.6208e-02_rb/)
      kbo(:, 3,25, 6) = (/ &
     &1.0118e+01_rb,7.7356e+00_rb,5.4684e+00_rb,3.0922e+00_rb,8.9882e-02_rb/)
      kbo(:, 4,25, 6) = (/ &
     &1.0662e+01_rb,8.1371e+00_rb,5.7357e+00_rb,3.2382e+00_rb,9.3407e-02_rb/)
      kbo(:, 5,25, 6) = (/ &
     &1.1279e+01_rb,8.5947e+00_rb,6.0377e+00_rb,3.4015e+00_rb,9.6770e-02_rb/)
      kbo(:, 1,26, 6) = (/ &
     &7.3112e+00_rb,5.6725e+00_rb,4.0689e+00_rb,2.3123e+00_rb,7.0640e-02_rb/)
      kbo(:, 2,26, 6) = (/ &
     &7.6950e+00_rb,5.9535e+00_rb,4.2634e+00_rb,2.4227e+00_rb,7.4010e-02_rb/)
      kbo(:, 3,26, 6) = (/ &
     &8.1066e+00_rb,6.2538e+00_rb,4.4719e+00_rb,2.5387e+00_rb,7.7271e-02_rb/)
      kbo(:, 4,26, 6) = (/ &
     &8.5794e+00_rb,6.6014e+00_rb,4.7086e+00_rb,2.6686e+00_rb,8.0450e-02_rb/)
      kbo(:, 5,26, 6) = (/ &
     &9.1099e+00_rb,6.9909e+00_rb,4.9735e+00_rb,2.8158e+00_rb,8.3380e-02_rb/)
      kbo(:, 1,27, 6) = (/ &
     &6.0744e+00_rb,4.7606e+00_rb,3.4368e+00_rb,1.9534e+00_rb,6.0404e-02_rb/)
      kbo(:, 2,27, 6) = (/ &
     &6.4010e+00_rb,4.9996e+00_rb,3.6064e+00_rb,2.0495e+00_rb,6.3391e-02_rb/)
      kbo(:, 3,27, 6) = (/ &
     &6.7712e+00_rb,5.2713e+00_rb,3.7979e+00_rb,2.1556e+00_rb,6.6293e-02_rb/)
      kbo(:, 4,27, 6) = (/ &
     &7.1986e+00_rb,5.5846e+00_rb,4.0153e+00_rb,2.2772e+00_rb,6.9007e-02_rb/)
      kbo(:, 5,27, 6) = (/ &
     &7.6755e+00_rb,5.9357e+00_rb,4.2585e+00_rb,2.4127e+00_rb,7.1247e-02_rb/)
      kbo(:, 1,28, 6) = (/ &
     &5.1866e+00_rb,4.1033e+00_rb,2.9724e+00_rb,1.6878e+00_rb,5.1571e-02_rb/)
      kbo(:, 2,28, 6) = (/ &
     &5.4861e+00_rb,4.3239e+00_rb,3.1305e+00_rb,1.7765e+00_rb,5.4219e-02_rb/)
      kbo(:, 3,28, 6) = (/ &
     &5.8320e+00_rb,4.5795e+00_rb,3.3130e+00_rb,1.8782e+00_rb,5.6690e-02_rb/)
      kbo(:, 4,28, 6) = (/ &
     &6.2247e+00_rb,4.8686e+00_rb,3.5188e+00_rb,1.9940e+00_rb,5.8750e-02_rb/)
      kbo(:, 5,28, 6) = (/ &
     &6.6558e+00_rb,5.1871e+00_rb,3.7446e+00_rb,2.1214e+00_rb,6.0243e-02_rb/)
      kbo(:, 1,29, 6) = (/ &
     &4.7142e+00_rb,3.7524e+00_rb,2.7162e+00_rb,1.5365e+00_rb,4.4039e-02_rb/)
      kbo(:, 2,29, 6) = (/ &
     &5.0090e+00_rb,3.9723e+00_rb,2.8754e+00_rb,1.6258e+00_rb,4.6277e-02_rb/)
      kbo(:, 3,29, 6) = (/ &
     &5.3480e+00_rb,4.2236e+00_rb,3.0565e+00_rb,1.7286e+00_rb,4.8156e-02_rb/)
      kbo(:, 4,29, 6) = (/ &
     &5.7292e+00_rb,4.5074e+00_rb,3.2622e+00_rb,1.8442e+00_rb,4.9560e-02_rb/)
      kbo(:, 5,29, 6) = (/ &
     &6.1329e+00_rb,4.8058e+00_rb,3.4771e+00_rb,1.9687e+00_rb,5.0487e-02_rb/)
      kbo(:, 1,30, 6) = (/ &
     &4.4038e+00_rb,3.5211e+00_rb,2.5447e+00_rb,1.4336e+00_rb,3.7528e-02_rb/)
      kbo(:, 2,30, 6) = (/ &
     &4.6996e+00_rb,3.7441e+00_rb,2.7067e+00_rb,1.5257e+00_rb,3.9262e-02_rb/)
      kbo(:, 3,30, 6) = (/ &
     &5.0396e+00_rb,3.9983e+00_rb,2.8921e+00_rb,1.6308e+00_rb,4.0573e-02_rb/)
      kbo(:, 4,30, 6) = (/ &
     &5.4089e+00_rb,4.2744e+00_rb,3.0942e+00_rb,1.7467e+00_rb,4.1475e-02_rb/)
      kbo(:, 5,30, 6) = (/ &
     &5.8026e+00_rb,4.5655e+00_rb,3.3058e+00_rb,1.8693e+00_rb,4.2084e-02_rb/)
      kbo(:, 1,31, 6) = (/ &
     &4.3406e+00_rb,3.4756e+00_rb,2.5043e+00_rb,1.4029e+00_rb,3.1837e-02_rb/)
      kbo(:, 2,31, 6) = (/ &
     &4.6530e+00_rb,3.7116e+00_rb,2.6766e+00_rb,1.5014e+00_rb,3.3063e-02_rb/)
      kbo(:, 3,31, 6) = (/ &
     &5.0025e+00_rb,3.9754e+00_rb,2.8716e+00_rb,1.6130e+00_rb,3.3948e-02_rb/)
      kbo(:, 4,31, 6) = (/ &
     &5.3785e+00_rb,4.2562e+00_rb,3.0774e+00_rb,1.7319e+00_rb,3.4570e-02_rb/)
      kbo(:, 5,31, 6) = (/ &
     &5.7889e+00_rb,4.5595e+00_rb,3.2989e+00_rb,1.8598e+00_rb,3.4986e-02_rb/)
      kbo(:, 1,32, 6) = (/ &
     &4.3396e+00_rb,3.4769e+00_rb,2.4992e+00_rb,1.3933e+00_rb,2.6827e-02_rb/)
      kbo(:, 2,32, 6) = (/ &
     &4.6714e+00_rb,3.7286e+00_rb,2.6844e+00_rb,1.4997e+00_rb,2.7685e-02_rb/)
      kbo(:, 3,32, 6) = (/ &
     &5.0312e+00_rb,4.0002e+00_rb,2.8855e+00_rb,1.6156e+00_rb,2.8303e-02_rb/)
      kbo(:, 4,32, 6) = (/ &
     &5.4256e+00_rb,4.2946e+00_rb,3.1010e+00_rb,1.7399e+00_rb,2.8748e-02_rb/)
      kbo(:, 5,32, 6) = (/ &
     &5.8665e+00_rb,4.6212e+00_rb,3.3393e+00_rb,1.8767e+00_rb,2.9064e-02_rb/)
      kbo(:, 1,33, 6) = (/ &
     &4.4332e+00_rb,3.5489e+00_rb,2.5452e+00_rb,1.4121e+00_rb,2.2496e-02_rb/)
      kbo(:, 2,33, 6) = (/ &
     &4.7823e+00_rb,3.8149e+00_rb,2.7429e+00_rb,1.5266e+00_rb,2.3111e-02_rb/)
      kbo(:, 3,33, 6) = (/ &
     &5.1647e+00_rb,4.1029e+00_rb,2.9548e+00_rb,1.6483e+00_rb,2.3573e-02_rb/)
      kbo(:, 4,33, 6) = (/ &
     &5.5917e+00_rb,4.4208e+00_rb,3.1884e+00_rb,1.7830e+00_rb,2.3904e-02_rb/)
      kbo(:, 5,33, 6) = (/ &
     &6.0743e+00_rb,4.7788e+00_rb,3.4474e+00_rb,1.9321e+00_rb,2.4134e-02_rb/)
      kbo(:, 1,34, 6) = (/ &
     &4.4125e+00_rb,3.5356e+00_rb,2.5335e+00_rb,1.4023e+00_rb,1.8787e-02_rb/)
      kbo(:, 2,34, 6) = (/ &
     &4.7724e+00_rb,3.8103e+00_rb,2.7374e+00_rb,1.5195e+00_rb,1.9239e-02_rb/)
      kbo(:, 3,34, 6) = (/ &
     &5.1721e+00_rb,4.1112e+00_rb,2.9584e+00_rb,1.6475e+00_rb,1.9590e-02_rb/)
      kbo(:, 4,34, 6) = (/ &
     &5.6244e+00_rb,4.4492e+00_rb,3.2069e+00_rb,1.7903e+00_rb,1.9843e-02_rb/)
      kbo(:, 5,34, 6) = (/ &
     &6.1384e+00_rb,4.8292e+00_rb,3.4834e+00_rb,1.9492e+00_rb,2.0036e-02_rb/)
      kbo(:, 1,35, 6) = (/ &
     &4.3717e+00_rb,3.5073e+00_rb,2.5119e+00_rb,1.3871e+00_rb,1.5622e-02_rb/)
      kbo(:, 2,35, 6) = (/ &
     &4.7431e+00_rb,3.7911e+00_rb,2.7216e+00_rb,1.5076e+00_rb,1.5975e-02_rb/)
      kbo(:, 3,35, 6) = (/ &
     &5.1596e+00_rb,4.1047e+00_rb,2.9535e+00_rb,1.6419e+00_rb,1.6252e-02_rb/)
      kbo(:, 4,35, 6) = (/ &
     &5.6365e+00_rb,4.4612e+00_rb,3.2141e+00_rb,1.7918e+00_rb,1.6461e-02_rb/)
      kbo(:, 5,35, 6) = (/ &
     &6.1774e+00_rb,4.8609e+00_rb,3.5048e+00_rb,1.9589e+00_rb,1.6599e-02_rb/)
      kbo(:, 1,36, 6) = (/ &
     &4.2736e+00_rb,3.4354e+00_rb,2.4585e+00_rb,1.3548e+00_rb,1.2936e-02_rb/)
      kbo(:, 2,36, 6) = (/ &
     &4.6496e+00_rb,3.7234e+00_rb,2.6724e+00_rb,1.4783e+00_rb,1.3228e-02_rb/)
      kbo(:, 3,36, 6) = (/ &
     &5.0769e+00_rb,4.0460e+00_rb,2.9111e+00_rb,1.6163e+00_rb,1.3462e-02_rb/)
      kbo(:, 4,36, 6) = (/ &
     &5.5708e+00_rb,4.4155e+00_rb,3.1813e+00_rb,1.7716e+00_rb,1.3637e-02_rb/)
      kbo(:, 5,36, 6) = (/ &
     &6.1338e+00_rb,4.8311e+00_rb,3.4822e+00_rb,1.9445e+00_rb,1.3753e-02_rb/)
      kbo(:, 1,37, 6) = (/ &
     &4.0234e+00_rb,3.2473e+00_rb,2.3232e+00_rb,1.2788e+00_rb,1.0671e-02_rb/)
      kbo(:, 2,37, 6) = (/ &
     &4.3892e+00_rb,3.5290e+00_rb,2.5324e+00_rb,1.3998e+00_rb,1.0924e-02_rb/)
      kbo(:, 3,37, 6) = (/ &
     &4.8090e+00_rb,3.8477e+00_rb,2.7688e+00_rb,1.5370e+00_rb,1.1122e-02_rb/)
      kbo(:, 4,37, 6) = (/ &
     &5.2982e+00_rb,4.2143e+00_rb,3.0374e+00_rb,1.6915e+00_rb,1.1275e-02_rb/)
      kbo(:, 5,37, 6) = (/ &
     &5.8599e+00_rb,4.6308e+00_rb,3.3399e+00_rb,1.8652e+00_rb,1.1382e-02_rb/)
      kbo(:, 1,38, 6) = (/ &
     &3.8405e+00_rb,3.1094e+00_rb,2.2232e+00_rb,1.2223e+00_rb,8.8011e-03_rb/)
      kbo(:, 2,38, 6) = (/ &
     &4.2026e+00_rb,3.3894e+00_rb,2.4317e+00_rb,1.3429e+00_rb,9.0204e-03_rb/)
      kbo(:, 3,38, 6) = (/ &
     &4.6223e+00_rb,3.7094e+00_rb,2.6688e+00_rb,1.4805e+00_rb,9.1889e-03_rb/)
      kbo(:, 4,38, 6) = (/ &
     &5.1135e+00_rb,4.0786e+00_rb,2.9400e+00_rb,1.6366e+00_rb,9.3266e-03_rb/)
      kbo(:, 5,38, 6) = (/ &
     &5.6812e+00_rb,4.4997e+00_rb,3.2457e+00_rb,1.8125e+00_rb,9.4327e-03_rb/)
      kbo(:, 1,39, 6) = (/ &
     &3.7513e+00_rb,3.0429e+00_rb,2.1735e+00_rb,1.1927e+00_rb,7.2681e-03_rb/)
      kbo(:, 2,39, 6) = (/ &
     &4.1185e+00_rb,3.3272e+00_rb,2.3853e+00_rb,1.3156e+00_rb,7.4517e-03_rb/)
      kbo(:, 3,39, 6) = (/ &
     &4.5480e+00_rb,3.6554e+00_rb,2.6285e+00_rb,1.4564e+00_rb,7.6013e-03_rb/)
      kbo(:, 4,39, 6) = (/ &
     &5.0522e+00_rb,4.0351e+00_rb,2.9074e+00_rb,1.6167e+00_rb,7.7185e-03_rb/)
      kbo(:, 5,39, 6) = (/ &
     &5.6411e+00_rb,4.4721e+00_rb,3.2250e+00_rb,1.7992e+00_rb,7.8245e-03_rb/)
      kbo(:, 1,40, 6) = (/ &
     &3.4100e+00_rb,2.7806e+00_rb,1.9856e+00_rb,1.0896e+00_rb,5.9768e-03_rb/)
      kbo(:, 2,40, 6) = (/ &
     &3.7541e+00_rb,3.0505e+00_rb,2.1868e+00_rb,1.2067e+00_rb,6.1348e-03_rb/)
      kbo(:, 3,40, 6) = (/ &
     &4.1600e+00_rb,3.3636e+00_rb,2.4194e+00_rb,1.3415e+00_rb,6.2676e-03_rb/)
      kbo(:, 4,40, 6) = (/ &
     &4.6404e+00_rb,3.7278e+00_rb,2.6880e+00_rb,1.4963e+00_rb,6.3713e-03_rb/)
      kbo(:, 5,40, 6) = (/ &
     &5.2035e+00_rb,4.1476e+00_rb,2.9948e+00_rb,1.6730e+00_rb,6.4667e-03_rb/)
      kbo(:, 1,41, 6) = (/ &
     &3.0891e+00_rb,2.5323e+00_rb,1.8076e+00_rb,9.9199e-01_rb,4.9106e-03_rb/)
      kbo(:, 2,41, 6) = (/ &
     &3.4100e+00_rb,2.7866e+00_rb,1.9972e+00_rb,1.1027e+00_rb,5.0459e-03_rb/)
      kbo(:, 3,41, 6) = (/ &
     &3.7913e+00_rb,3.0833e+00_rb,2.2185e+00_rb,1.2311e+00_rb,5.1622e-03_rb/)
      kbo(:, 4,41, 6) = (/ &
     &4.2439e+00_rb,3.4304e+00_rb,2.4754e+00_rb,1.3795e+00_rb,5.2594e-03_rb/)
      kbo(:, 5,41, 6) = (/ &
     &4.7807e+00_rb,3.8337e+00_rb,2.7715e+00_rb,1.5507e+00_rb,5.3436e-03_rb/)
      kbo(:, 1,42, 6) = (/ &
     &2.8278e+00_rb,2.3283e+00_rb,1.6611e+00_rb,9.1142e-01_rb,4.0331e-03_rb/)
      kbo(:, 2,42, 6) = (/ &
     &3.1295e+00_rb,2.5702e+00_rb,1.8417e+00_rb,1.0169e+00_rb,4.1507e-03_rb/)
      kbo(:, 3,42, 6) = (/ &
     &3.4910e+00_rb,2.8544e+00_rb,2.0536e+00_rb,1.1401e+00_rb,4.2525e-03_rb/)
      kbo(:, 4,42, 6) = (/ &
     &3.9235e+00_rb,3.1883e+00_rb,2.3016e+00_rb,1.2837e+00_rb,4.3408e-03_rb/)
      kbo(:, 5,42, 6) = (/ &
     &4.4380e+00_rb,3.5784e+00_rb,2.5896e+00_rb,1.4506e+00_rb,4.4170e-03_rb/)
      kbo(:, 1,43, 6) = (/ &
     &2.5971e+00_rb,2.1460e+00_rb,1.5298e+00_rb,8.3854e-01_rb,3.3049e-03_rb/)
      kbo(:, 2,43, 6) = (/ &
     &2.8799e+00_rb,2.3759e+00_rb,1.7013e+00_rb,9.3880e-01_rb,3.4092e-03_rb/)
      kbo(:, 3,43, 6) = (/ &
     &3.2202e+00_rb,2.6461e+00_rb,1.9035e+00_rb,1.0566e+00_rb,3.5003e-03_rb/)
      kbo(:, 4,43, 6) = (/ &
     &3.6329e+00_rb,2.9678e+00_rb,2.1426e+00_rb,1.1949e+00_rb,3.5773e-03_rb/)
      kbo(:, 5,43, 6) = (/ &
     &4.1275e+00_rb,3.3455e+00_rb,2.4216e+00_rb,1.3572e+00_rb,3.6480e-03_rb/)
      kbo(:, 1,44, 6) = (/ &
     &2.4109e+00_rb,1.9983e+00_rb,1.4225e+00_rb,7.7812e-01_rb,2.7054e-03_rb/)
      kbo(:, 2,44, 6) = (/ &
     &2.6776e+00_rb,2.2170e+00_rb,1.5862e+00_rb,8.7403e-01_rb,2.7971e-03_rb/)
      kbo(:, 3,44, 6) = (/ &
     &3.0022e+00_rb,2.4769e+00_rb,1.7806e+00_rb,9.8740e-01_rb,2.8773e-03_rb/)
      kbo(:, 4,44, 6) = (/ &
     &3.3965e+00_rb,2.7871e+00_rb,2.0115e+00_rb,1.1213e+00_rb,2.9478e-03_rb/)
      kbo(:, 5,44, 6) = (/ &
     &3.8747e+00_rb,3.1555e+00_rb,2.2836e+00_rb,1.2791e+00_rb,3.0101e-03_rb/)
      kbo(:, 1,45, 6) = (/ &
     &2.2815e+00_rb,1.8944e+00_rb,1.3464e+00_rb,7.3467e-01_rb,2.2130e-03_rb/)
      kbo(:, 2,45, 6) = (/ &
     &2.5401e+00_rb,2.1081e+00_rb,1.5065e+00_rb,8.2816e-01_rb,2.2948e-03_rb/)
      kbo(:, 3,45, 6) = (/ &
     &2.8533e+00_rb,2.3602e+00_rb,1.6948e+00_rb,9.3826e-01_rb,2.3656e-03_rb/)
      kbo(:, 4,45, 6) = (/ &
     &3.2382e+00_rb,2.6655e+00_rb,1.9218e+00_rb,1.0700e+00_rb,2.4291e-03_rb/)
      kbo(:, 5,45, 6) = (/ &
     &3.7076e+00_rb,3.0298e+00_rb,2.1912e+00_rb,1.2259e+00_rb,2.4852e-03_rb/)
      kbo(:, 1,46, 6) = (/ &
     &2.1752e+00_rb,1.8075e+00_rb,1.2824e+00_rb,6.9758e-01_rb,1.8083e-03_rb/)
      kbo(:, 2,46, 6) = (/ &
     &2.4262e+00_rb,2.0168e+00_rb,1.4391e+00_rb,7.8916e-01_rb,1.8803e-03_rb/)
      kbo(:, 3,46, 6) = (/ &
     &2.7306e+00_rb,2.2636e+00_rb,1.6229e+00_rb,8.9649e-01_rb,1.9453e-03_rb/)
      kbo(:, 4,46, 6) = (/ &
     &3.1079e+00_rb,2.5651e+00_rb,1.8473e+00_rb,1.0263e+00_rb,2.0023e-03_rb/)
      kbo(:, 5,46, 6) = (/ &
     &3.5715e+00_rb,2.9262e+00_rb,2.1144e+00_rb,1.1808e+00_rb,2.0547e-03_rb/)
      kbo(:, 1,47, 6) = (/ &
     &2.0439e+00_rb,1.6998e+00_rb,1.2039e+00_rb,6.5284e-01_rb,1.4728e-03_rb/)
      kbo(:, 2,47, 6) = (/ &
     &2.2849e+00_rb,1.9026e+00_rb,1.3552e+00_rb,7.4124e-01_rb,1.5378e-03_rb/)
      kbo(:, 3,47, 6) = (/ &
     &2.5739e+00_rb,2.1396e+00_rb,1.5317e+00_rb,8.4434e-01_rb,1.5971e-03_rb/)
      kbo(:, 4,47, 6) = (/ &
     &2.9355e+00_rb,2.4308e+00_rb,1.7484e+00_rb,9.6939e-01_rb,1.6478e-03_rb/)
      kbo(:, 5,47, 6) = (/ &
     &3.3837e+00_rb,2.7824e+00_rb,2.0088e+00_rb,1.1198e+00_rb,1.6945e-03_rb/)
      kbo(:, 1,48, 6) = (/ &
     &1.9729e+00_rb,1.6396e+00_rb,1.1585e+00_rb,6.2585e-01_rb,1.1998e-03_rb/)
      kbo(:, 2,48, 6) = (/ &
     &2.2114e+00_rb,1.8415e+00_rb,1.3088e+00_rb,7.1328e-01_rb,1.2575e-03_rb/)
      kbo(:, 3,48, 6) = (/ &
     &2.4937e+00_rb,2.0747e+00_rb,1.4825e+00_rb,8.1439e-01_rb,1.3096e-03_rb/)
      kbo(:, 4,48, 6) = (/ &
     &2.8486e+00_rb,2.3616e+00_rb,1.6960e+00_rb,9.3764e-01_rb,1.3556e-03_rb/)
      kbo(:, 5,48, 6) = (/ &
     &3.2936e+00_rb,2.7131e+00_rb,1.9561e+00_rb,1.0874e+00_rb,1.3979e-03_rb/)
      kbo(:, 1,49, 6) = (/ &
     &1.9817e+00_rb,1.6421e+00_rb,1.1570e+00_rb,6.2210e-01_rb,9.7602e-04_rb/)
      kbo(:, 2,49, 6) = (/ &
     &2.2272e+00_rb,1.8506e+00_rb,1.3118e+00_rb,7.1147e-01_rb,1.0279e-03_rb/)
      kbo(:, 3,49, 6) = (/ &
     &2.5150e+00_rb,2.0896e+00_rb,1.4898e+00_rb,8.1454e-01_rb,1.0745e-03_rb/)
      kbo(:, 4,49, 6) = (/ &
     &2.8769e+00_rb,2.3813e+00_rb,1.7067e+00_rb,9.3965e-01_rb,1.1160e-03_rb/)
      kbo(:, 5,49, 6) = (/ &
     &3.3353e+00_rb,2.7446e+00_rb,1.9747e+00_rb,1.0933e+00_rb,1.1545e-03_rb/)
      kbo(:, 1,50, 6) = (/ &
     &1.8978e+00_rb,1.5720e+00_rb,1.1055e+00_rb,5.9245e-01_rb,7.9308e-04_rb/)
      kbo(:, 2,50, 6) = (/ &
     &2.1399e+00_rb,1.7779e+00_rb,1.2580e+00_rb,6.8007e-01_rb,8.3888e-04_rb/)
      kbo(:, 3,50, 6) = (/ &
     &2.4218e+00_rb,2.0138e+00_rb,1.4333e+00_rb,7.8150e-01_rb,8.7996e-04_rb/)
      kbo(:, 4,50, 6) = (/ &
     &2.7742e+00_rb,2.2995e+00_rb,1.6455e+00_rb,9.0381e-01_rb,9.1698e-04_rb/)
      kbo(:, 5,50, 6) = (/ &
     &3.2255e+00_rb,2.6590e+00_rb,1.9107e+00_rb,1.0555e+00_rb,9.5164e-04_rb/)
      kbo(:, 1,51, 6) = (/ &
     &1.7787e+00_rb,1.4741e+00_rb,1.0351e+00_rb,5.5321e-01_rb,6.4322e-04_rb/)
      kbo(:, 2,51, 6) = (/ &
     &2.0121e+00_rb,1.6731e+00_rb,1.1819e+00_rb,6.3736e-01_rb,6.8396e-04_rb/)
      kbo(:, 3,51, 6) = (/ &
     &2.2825e+00_rb,1.9012e+00_rb,1.3514e+00_rb,7.3514e-01_rb,7.1977e-04_rb/)
      kbo(:, 4,51, 6) = (/ &
     &2.6184e+00_rb,2.1760e+00_rb,1.5550e+00_rb,8.5263e-01_rb,7.5276e-04_rb/)
      kbo(:, 5,51, 6) = (/ &
     &3.0526e+00_rb,2.5248e+00_rb,1.8126e+00_rb,9.9966e-01_rb,7.8322e-04_rb/)
      kbo(:, 1,52, 6) = (/ &
     &1.7138e+00_rb,1.4190e+00_rb,9.9450e-01_rb,5.2987e-01_rb,5.2154e-04_rb/)
      kbo(:, 2,52, 6) = (/ &
     &1.9434e+00_rb,1.6150e+00_rb,1.1387e+00_rb,6.1204e-01_rb,5.5749e-04_rb/)
      kbo(:, 3,52, 6) = (/ &
     &2.2115e+00_rb,1.8422e+00_rb,1.3069e+00_rb,7.0863e-01_rb,5.8864e-04_rb/)
      kbo(:, 4,52, 6) = (/ &
     &2.5398e+00_rb,2.1125e+00_rb,1.5071e+00_rb,8.2393e-01_rb,6.1775e-04_rb/)
      kbo(:, 5,52, 6) = (/ &
     &2.9672e+00_rb,2.4581e+00_rb,1.7623e+00_rb,9.6952e-01_rb,6.4493e-04_rb/)
      kbo(:, 1,53, 6) = (/ &
     &1.7198e+00_rb,1.4203e+00_rb,9.9324e-01_rb,5.2715e-01_rb,4.2262e-04_rb/)
      kbo(:, 2,53, 6) = (/ &
     &1.9529e+00_rb,1.6185e+00_rb,1.1384e+00_rb,6.0925e-01_rb,4.5399e-04_rb/)
      kbo(:, 3,53, 6) = (/ &
     &2.2293e+00_rb,1.8530e+00_rb,1.3116e+00_rb,7.0805e-01_rb,4.8128e-04_rb/)
      kbo(:, 4,53, 6) = (/ &
     &2.5636e+00_rb,2.1294e+00_rb,1.5160e+00_rb,8.2535e-01_rb,5.0713e-04_rb/)
      kbo(:, 5,53, 6) = (/ &
     &3.0004e+00_rb,2.4827e+00_rb,1.7770e+00_rb,9.7383e-01_rb,5.3111e-04_rb/)
      kbo(:, 1,54, 6) = (/ &
     &1.4969e+00_rb,1.2401e+00_rb,8.6613e-01_rb,4.5904e-01_rb,3.4135e-04_rb/)
      kbo(:, 2,54, 6) = (/ &
     &1.7004e+00_rb,1.4146e+00_rb,9.9400e-01_rb,5.3139e-01_rb,3.6835e-04_rb/)
      kbo(:, 3,54, 6) = (/ &
     &1.9475e+00_rb,1.6269e+00_rb,1.1506e+00_rb,6.2081e-01_rb,3.9232e-04_rb/)
      kbo(:, 4,54, 6) = (/ &
     &2.2441e+00_rb,1.8758e+00_rb,1.3350e+00_rb,7.2699e-01_rb,4.1441e-04_rb/)
      kbo(:, 5,54, 6) = (/ &
     &2.6322e+00_rb,2.1931e+00_rb,1.5695e+00_rb,8.6094e-01_rb,4.3501e-04_rb/)
      kbo(:, 1,55, 6) = (/ &
     &1.1902e+00_rb,9.9203e-01_rb,6.9261e-01_rb,3.6696e-01_rb,2.7539e-04_rb/)
      kbo(:, 2,55, 6) = (/ &
     &1.3514e+00_rb,1.1327e+00_rb,7.9579e-01_rb,4.2578e-01_rb,2.9805e-04_rb/)
      kbo(:, 3,55, 6) = (/ &
     &1.5528e+00_rb,1.3090e+00_rb,9.2585e-01_rb,5.0048e-01_rb,3.1883e-04_rb/)
      kbo(:, 4,55, 6) = (/ &
     &1.7945e+00_rb,1.5161e+00_rb,1.0797e+00_rb,5.8958e-01_rb,3.3758e-04_rb/)
      kbo(:, 5,55, 6) = (/ &
     &2.1091e+00_rb,1.7786e+00_rb,1.2743e+00_rb,7.0161e-01_rb,3.5497e-04_rb/)
      kbo(:, 1,56, 6) = (/ &
     &9.4564e-01_rb,7.9256e-01_rb,5.5332e-01_rb,2.9317e-01_rb,2.2189e-04_rb/)
      kbo(:, 2,56, 6) = (/ &
     &1.0727e+00_rb,9.0556e-01_rb,6.3622e-01_rb,3.4063e-01_rb,2.4090e-04_rb/)
      kbo(:, 3,56, 6) = (/ &
     &1.2368e+00_rb,1.0514e+00_rb,7.4402e-01_rb,4.0280e-01_rb,2.5859e-04_rb/)
      kbo(:, 4,56, 6) = (/ &
     &1.4335e+00_rb,1.2238e+00_rb,8.7212e-01_rb,4.7748e-01_rb,2.7478e-04_rb/)
      kbo(:, 5,56, 6) = (/ &
     &1.6876e+00_rb,1.4404e+00_rb,1.0331e+00_rb,5.7092e-01_rb,2.8964e-04_rb/)
      kbo(:, 1,57, 6) = (/ &
     &7.5151e-01_rb,6.3315e-01_rb,4.4196e-01_rb,2.3414e-01_rb,1.7834e-04_rb/)
      kbo(:, 2,57, 6) = (/ &
     &8.5091e-01_rb,7.2306e-01_rb,5.0804e-01_rb,2.7214e-01_rb,1.9471e-04_rb/)
      kbo(:, 3,57, 6) = (/ &
     &9.8353e-01_rb,8.4281e-01_rb,5.9656e-01_rb,3.2337e-01_rb,2.0961e-04_rb/)
      kbo(:, 4,57, 6) = (/ &
     &1.1433e+00_rb,9.8609e-01_rb,7.0325e-01_rb,3.8594e-01_rb,2.2327e-04_rb/)
      kbo(:, 5,57, 6) = (/ &
     &1.3484e+00_rb,1.1650e+00_rb,8.3657e-01_rb,4.6393e-01_rb,2.3604e-04_rb/)
      kbo(:, 1,58, 6) = (/ &
     &1.5793e-01_rb,1.6356e-01_rb,1.4683e-01_rb,1.0896e-01_rb,1.4326e-04_rb/)
      kbo(:, 2,58, 6) = (/ &
     &1.7834e-01_rb,1.8665e-01_rb,1.6872e-01_rb,1.2675e-01_rb,1.5731e-04_rb/)
      kbo(:, 3,58, 6) = (/ &
     &2.0650e-01_rb,2.1828e-01_rb,1.9887e-01_rb,1.5129e-01_rb,1.6986e-04_rb/)
      kbo(:, 4,58, 6) = (/ &
     &2.4084e-01_rb,2.5688e-01_rb,2.3602e-01_rb,1.8197e-01_rb,1.8127e-04_rb/)
      kbo(:, 5,58, 6) = (/ &
     &2.8453e-01_rb,3.0466e-01_rb,2.8209e-01_rb,2.2013e-01_rb,1.9222e-04_rb/)
      kbo(:, 1,59, 6) = (/ &
     &1.5159e-01_rb,1.5229e-01_rb,1.2764e-01_rb,8.4631e-02_rb,1.1680e-04_rb/)
      kbo(:, 2,59, 6) = (/ &
     &1.7166e-01_rb,1.7481e-01_rb,1.4781e-01_rb,9.9479e-02_rb,1.2840e-04_rb/)
      kbo(:, 3,59, 6) = (/ &
     &1.9953e-01_rb,2.0595e-01_rb,1.7587e-01_rb,1.2013e-01_rb,1.3871e-04_rb/)
      kbo(:, 4,59, 6) = (/ &
     &2.3377e-01_rb,2.4439e-01_rb,2.1097e-01_rb,1.4631e-01_rb,1.4804e-04_rb/)
      kbo(:, 5,59, 6) = (/ &
     &2.7769e-01_rb,2.9243e-01_rb,2.5484e-01_rb,1.7920e-01_rb,1.5678e-04_rb/)
      kbo(:, 1,13, 7) = (/ &
     &1.7755e+03_rb,1.3317e+03_rb,8.8796e+02_rb,4.4421e+02_rb,6.5972e-01_rb/)
      kbo(:, 2,13, 7) = (/ &
     &1.8279e+03_rb,1.3710e+03_rb,9.1418e+02_rb,4.5731e+02_rb,6.6173e-01_rb/)
      kbo(:, 3,13, 7) = (/ &
     &1.8885e+03_rb,1.4165e+03_rb,9.4446e+02_rb,4.7244e+02_rb,6.6577e-01_rb/)
      kbo(:, 4,13, 7) = (/ &
     &1.9490e+03_rb,1.4619e+03_rb,9.7472e+02_rb,4.8757e+02_rb,6.7090e-01_rb/)
      kbo(:, 5,13, 7) = (/ &
     &2.0075e+03_rb,1.5058e+03_rb,1.0040e+03_rb,5.0220e+02_rb,6.7193e-01_rb/)
      kbo(:, 1,14, 7) = (/ &
     &9.7594e+02_rb,7.3206e+02_rb,4.8819e+02_rb,2.4432e+02_rb,6.3121e-01_rb/)
      kbo(:, 2,14, 7) = (/ &
     &1.0087e+03_rb,7.5661e+02_rb,5.0455e+02_rb,2.5249e+02_rb,6.3502e-01_rb/)
      kbo(:, 3,14, 7) = (/ &
     &1.0415e+03_rb,7.8124e+02_rb,5.2096e+02_rb,2.6069e+02_rb,6.3710e-01_rb/)
      kbo(:, 4,14, 7) = (/ &
     &1.0744e+03_rb,8.0589e+02_rb,5.3739e+02_rb,2.6890e+02_rb,6.4004e-01_rb/)
      kbo(:, 5,14, 7) = (/ &
     &1.1054e+03_rb,8.2913e+02_rb,5.5288e+02_rb,2.7664e+02_rb,6.4122e-01_rb/)
      kbo(:, 1,15, 7) = (/ &
     &5.4478e+02_rb,4.0870e+02_rb,2.7262e+02_rb,1.3654e+02_rb,5.9523e-01_rb/)
      kbo(:, 2,15, 7) = (/ &
     &5.6305e+02_rb,4.2240e+02_rb,2.8175e+02_rb,1.4110e+02_rb,6.0003e-01_rb/)
      kbo(:, 3,15, 7) = (/ &
     &5.8123e+02_rb,4.3603e+02_rb,2.9083e+02_rb,1.4563e+02_rb,6.0181e-01_rb/)
      kbo(:, 4,15, 7) = (/ &
     &5.9882e+02_rb,4.4922e+02_rb,2.9962e+02_rb,1.5002e+02_rb,6.0320e-01_rb/)
      kbo(:, 5,15, 7) = (/ &
     &6.1530e+02_rb,4.6158e+02_rb,3.0785e+02_rb,1.5413e+02_rb,6.0492e-01_rb/)
      kbo(:, 1,16, 7) = (/ &
     &3.5993e+02_rb,2.7006e+02_rb,1.8020e+02_rb,9.0332e+01_rb,5.5493e-01_rb/)
      kbo(:, 2,16, 7) = (/ &
     &3.7215e+02_rb,2.7922e+02_rb,1.8629e+02_rb,9.3372e+01_rb,5.5851e-01_rb/)
      kbo(:, 3,16, 7) = (/ &
     &3.8394e+02_rb,2.8806e+02_rb,1.9218e+02_rb,9.6308e+01_rb,5.6035e-01_rb/)
      kbo(:, 4,16, 7) = (/ &
     &3.9518e+02_rb,2.9649e+02_rb,1.9779e+02_rb,9.9106e+01_rb,5.6267e-01_rb/)
      kbo(:, 5,16, 7) = (/ &
     &4.0564e+02_rb,3.0433e+02_rb,2.0302e+02_rb,1.0171e+02_rb,5.6476e-01_rb/)
      kbo(:, 1,17, 7) = (/ &
     &2.4595e+02_rb,1.8458e+02_rb,1.2320e+02_rb,6.1838e+01_rb,5.0905e-01_rb/)
      kbo(:, 2,17, 7) = (/ &
     &2.5413e+02_rb,1.9070e+02_rb,1.2728e+02_rb,6.3868e+01_rb,5.1185e-01_rb/)
      kbo(:, 3,17, 7) = (/ &
     &2.6187e+02_rb,1.9651e+02_rb,1.3115e+02_rb,6.5795e+01_rb,5.1474e-01_rb/)
      kbo(:, 4,17, 7) = (/ &
     &2.6921e+02_rb,2.0201e+02_rb,1.3481e+02_rb,6.7615e+01_rb,5.1865e-01_rb/)
      kbo(:, 5,17, 7) = (/ &
     &2.7688e+02_rb,2.0776e+02_rb,1.3865e+02_rb,6.9531e+01_rb,5.2170e-01_rb/)
      kbo(:, 1,18, 7) = (/ &
     &1.7164e+02_rb,1.2884e+02_rb,8.6050e+01_rb,4.3266e+01_rb,4.6055e-01_rb/)
      kbo(:, 2,18, 7) = (/ &
     &1.7713e+02_rb,1.3296e+02_rb,8.8791e+01_rb,4.4628e+01_rb,4.6405e-01_rb/)
      kbo(:, 3,18, 7) = (/ &
     &1.8233e+02_rb,1.3686e+02_rb,9.1384e+01_rb,4.5916e+01_rb,4.6825e-01_rb/)
      kbo(:, 4,18, 7) = (/ &
     &1.8765e+02_rb,1.4084e+02_rb,9.4032e+01_rb,4.7229e+01_rb,4.7329e-01_rb/)
      kbo(:, 5,18, 7) = (/ &
     &1.9405e+02_rb,1.4564e+02_rb,9.7227e+01_rb,4.8824e+01_rb,4.7705e-01_rb/)
      kbo(:, 1,19, 7) = (/ &
     &1.1725e+02_rb,8.8048e+01_rb,5.8853e+01_rb,2.9674e+01_rb,4.1247e-01_rb/)
      kbo(:, 2,19, 7) = (/ &
     &1.2086e+02_rb,9.0759e+01_rb,6.0656e+01_rb,3.0566e+01_rb,4.1661e-01_rb/)
      kbo(:, 3,19, 7) = (/ &
     &1.2440e+02_rb,9.3403e+01_rb,6.2413e+01_rb,3.1438e+01_rb,4.2213e-01_rb/)
      kbo(:, 4,19, 7) = (/ &
     &1.2864e+02_rb,9.6587e+01_rb,6.4532e+01_rb,3.2489e+01_rb,4.2779e-01_rb/)
      kbo(:, 5,19, 7) = (/ &
     &1.3396e+02_rb,1.0057e+02_rb,6.7181e+01_rb,3.3807e+01_rb,4.3260e-01_rb/)
      kbo(:, 1,20, 7) = (/ &
     &8.6468e+01_rb,6.4964e+01_rb,4.3467e+01_rb,2.2017e+01_rb,3.6582e-01_rb/)
      kbo(:, 2,20, 7) = (/ &
     &8.9024e+01_rb,6.6879e+01_rb,4.4738e+01_rb,2.2636e+01_rb,3.7131e-01_rb/)
      kbo(:, 3,20, 7) = (/ &
     &9.1943e+01_rb,6.9064e+01_rb,4.6188e+01_rb,2.3347e+01_rb,3.7766e-01_rb/)
      kbo(:, 4,20, 7) = (/ &
     &9.5737e+01_rb,7.1906e+01_rb,4.8080e+01_rb,2.4283e+01_rb,3.8413e-01_rb/)
      kbo(:, 5,20, 7) = (/ &
     &1.0032e+02_rb,7.5342e+01_rb,5.0362e+01_rb,2.5410e+01_rb,3.9048e-01_rb/)
      kbo(:, 1,21, 7) = (/ &
     &6.5580e+01_rb,4.9300e+01_rb,3.3030e+01_rb,1.6896e+01_rb,3.2248e-01_rb/)
      kbo(:, 2,21, 7) = (/ &
     &6.7679e+01_rb,5.0870e+01_rb,3.4072e+01_rb,1.7386e+01_rb,3.2902e-01_rb/)
      kbo(:, 3,21, 7) = (/ &
     &7.0353e+01_rb,5.2872e+01_rb,3.5398e+01_rb,1.8022e+01_rb,3.3617e-01_rb/)
      kbo(:, 4,21, 7) = (/ &
     &7.3710e+01_rb,5.5384e+01_rb,3.7067e+01_rb,1.8833e+01_rb,3.4337e-01_rb/)
      kbo(:, 5,21, 7) = (/ &
     &7.7378e+01_rb,5.8129e+01_rb,3.8887e+01_rb,1.9718e+01_rb,3.5091e-01_rb/)
      kbo(:, 1,22, 7) = (/ &
     &5.0990e+01_rb,3.8360e+01_rb,2.5747e+01_rb,1.3390e+01_rb,2.8355e-01_rb/)
      kbo(:, 2,22, 7) = (/ &
     &5.2937e+01_rb,3.9813e+01_rb,2.6706e+01_rb,1.3827e+01_rb,2.9084e-01_rb/)
      kbo(:, 3,22, 7) = (/ &
     &5.5472e+01_rb,4.1712e+01_rb,2.7965e+01_rb,1.4417e+01_rb,2.9863e-01_rb/)
      kbo(:, 4,22, 7) = (/ &
     &5.8295e+01_rb,4.3822e+01_rb,2.9362e+01_rb,1.5079e+01_rb,3.0668e-01_rb/)
      kbo(:, 5,22, 7) = (/ &
     &6.1215e+01_rb,4.6005e+01_rb,3.0807e+01_rb,1.5766e+01_rb,3.1501e-01_rb/)
      kbo(:, 1,23, 7) = (/ &
     &3.9928e+01_rb,3.0064e+01_rb,2.0251e+01_rb,1.0753e+01_rb,2.4875e-01_rb/)
      kbo(:, 2,23, 7) = (/ &
     &4.1783e+01_rb,3.1450e+01_rb,2.1160e+01_rb,1.1173e+01_rb,2.5662e-01_rb/)
      kbo(:, 3,23, 7) = (/ &
     &4.3954e+01_rb,3.3072e+01_rb,2.2227e+01_rb,1.1670e+01_rb,2.6505e-01_rb/)
      kbo(:, 4,23, 7) = (/ &
     &4.6239e+01_rb,3.4779e+01_rb,2.3350e+01_rb,1.2195e+01_rb,2.7351e-01_rb/)
      kbo(:, 5,23, 7) = (/ &
     &4.8555e+01_rb,3.6509e+01_rb,2.4489e+01_rb,1.2725e+01_rb,2.8191e-01_rb/)
      kbo(:, 1,24, 7) = (/ &
     &3.1198e+01_rb,2.3518e+01_rb,1.5962e+01_rb,8.6666e+00_rb,2.1812e-01_rb/)
      kbo(:, 2,24, 7) = (/ &
     &3.2836e+01_rb,2.4742e+01_rb,1.6752e+01_rb,9.0412e+00_rb,2.2642e-01_rb/)
      kbo(:, 3,24, 7) = (/ &
     &3.4586e+01_rb,2.6047e+01_rb,1.7597e+01_rb,9.4445e+00_rb,2.3475e-01_rb/)
      kbo(:, 4,24, 7) = (/ &
     &3.6388e+01_rb,2.7390e+01_rb,1.8470e+01_rb,9.8567e+00_rb,2.4289e-01_rb/)
      kbo(:, 5,24, 7) = (/ &
     &3.8237e+01_rb,2.8772e+01_rb,1.9370e+01_rb,1.0281e+01_rb,2.5096e-01_rb/)
      kbo(:, 1,25, 7) = (/ &
     &2.4765e+01_rb,1.8701e+01_rb,1.2839e+01_rb,7.1044e+00_rb,1.9118e-01_rb/)
      kbo(:, 2,25, 7) = (/ &
     &2.6117e+01_rb,1.9708e+01_rb,1.3481e+01_rb,7.4248e+00_rb,1.9923e-01_rb/)
      kbo(:, 3,25, 7) = (/ &
     &2.7518e+01_rb,2.0749e+01_rb,1.4145e+01_rb,7.7545e+00_rb,2.0704e-01_rb/)
      kbo(:, 4,25, 7) = (/ &
     &2.8948e+01_rb,2.1814e+01_rb,1.4825e+01_rb,8.0892e+00_rb,2.1475e-01_rb/)
      kbo(:, 5,25, 7) = (/ &
     &3.0459e+01_rb,2.2943e+01_rb,1.5551e+01_rb,8.4409e+00_rb,2.2240e-01_rb/)
      kbo(:, 1,26, 7) = (/ &
     &1.9997e+01_rb,1.5145e+01_rb,1.0541e+01_rb,5.9203e+00_rb,1.6734e-01_rb/)
      kbo(:, 2,26, 7) = (/ &
     &2.1098e+01_rb,1.5958e+01_rb,1.1060e+01_rb,6.1929e+00_rb,1.7471e-01_rb/)
      kbo(:, 3,26, 7) = (/ &
     &2.2238e+01_rb,1.6802e+01_rb,1.1594e+01_rb,6.4727e+00_rb,1.8196e-01_rb/)
      kbo(:, 4,26, 7) = (/ &
     &2.3407e+01_rb,1.7668e+01_rb,1.2141e+01_rb,6.7596e+00_rb,1.8921e-01_rb/)
      kbo(:, 5,26, 7) = (/ &
     &2.4698e+01_rb,1.8629e+01_rb,1.2749e+01_rb,7.0654e+00_rb,1.9648e-01_rb/)
      kbo(:, 1,27, 7) = (/ &
     &1.6756e+01_rb,1.2745e+01_rb,8.9780e+00_rb,5.0906e+00_rb,1.4596e-01_rb/)
      kbo(:, 2,27, 7) = (/ &
     &1.7682e+01_rb,1.3422e+01_rb,9.4167e+00_rb,5.3320e+00_rb,1.5263e-01_rb/)
      kbo(:, 3,27, 7) = (/ &
     &1.8639e+01_rb,1.4123e+01_rb,9.8653e+00_rb,5.5814e+00_rb,1.5946e-01_rb/)
      kbo(:, 4,27, 7) = (/ &
     &1.9672e+01_rb,1.4884e+01_rb,1.0349e+01_rb,5.8433e+00_rb,1.6641e-01_rb/)
      kbo(:, 5,27, 7) = (/ &
     &2.0853e+01_rb,1.5759e+01_rb,1.0902e+01_rb,6.1333e+00_rb,1.7345e-01_rb/)
      kbo(:, 1,28, 7) = (/ &
     &1.4405e+01_rb,1.1016e+01_rb,7.8395e+00_rb,4.4691e+00_rb,1.2690e-01_rb/)
      kbo(:, 2,28, 7) = (/ &
     &1.5206e+01_rb,1.1594e+01_rb,8.2236e+00_rb,4.6907e+00_rb,1.3310e-01_rb/)
      kbo(:, 3,28, 7) = (/ &
     &1.6055e+01_rb,1.2210e+01_rb,8.6248e+00_rb,4.9200e+00_rb,1.3960e-01_rb/)
      kbo(:, 4,28, 7) = (/ &
     &1.7030e+01_rb,1.2924e+01_rb,9.0823e+00_rb,5.1747e+00_rb,1.4629e-01_rb/)
      kbo(:, 5,28, 7) = (/ &
     &1.8152e+01_rb,1.3752e+01_rb,9.6123e+00_rb,5.4653e+00_rb,1.5306e-01_rb/)
      kbo(:, 1,29, 7) = (/ &
     &1.3149e+01_rb,1.0099e+01_rb,7.2312e+00_rb,4.1269e+00_rb,1.1016e-01_rb/)
      kbo(:, 2,29, 7) = (/ &
     &1.3892e+01_rb,1.0633e+01_rb,7.5916e+00_rb,4.3391e+00_rb,1.1614e-01_rb/)
      kbo(:, 3,29, 7) = (/ &
     &1.4735e+01_rb,1.1242e+01_rb,7.9958e+00_rb,4.5718e+00_rb,1.2237e-01_rb/)
      kbo(:, 4,29, 7) = (/ &
     &1.5717e+01_rb,1.1957e+01_rb,8.4609e+00_rb,4.8366e+00_rb,1.2878e-01_rb/)
      kbo(:, 5,29, 7) = (/ &
     &1.6850e+01_rb,1.2790e+01_rb,8.9991e+00_rb,5.1366e+00_rb,1.3504e-01_rb/)
      kbo(:, 1,30, 7) = (/ &
     &1.2314e+01_rb,9.4945e+00_rb,6.8243e+00_rb,3.8903e+00_rb,9.5721e-02_rb/)
      kbo(:, 2,30, 7) = (/ &
     &1.3054e+01_rb,1.0024e+01_rb,7.1887e+00_rb,4.1054e+00_rb,1.0142e-01_rb/)
      kbo(:, 3,30, 7) = (/ &
     &1.3921e+01_rb,1.0650e+01_rb,7.6093e+00_rb,4.3517e+00_rb,1.0739e-01_rb/)
      kbo(:, 4,30, 7) = (/ &
     &1.4944e+01_rb,1.1393e+01_rb,8.1010e+00_rb,4.6344e+00_rb,1.1328e-01_rb/)
      kbo(:, 5,30, 7) = (/ &
     &1.6104e+01_rb,1.2245e+01_rb,8.6565e+00_rb,4.9501e+00_rb,1.1877e-01_rb/)
      kbo(:, 1,31, 7) = (/ &
     &1.2157e+01_rb,9.3878e+00_rb,6.7548e+00_rb,3.8381e+00_rb,8.3345e-02_rb/)
      kbo(:, 2,31, 7) = (/ &
     &1.2955e+01_rb,9.9594e+00_rb,7.1498e+00_rb,4.0728e+00_rb,8.8771e-02_rb/)
      kbo(:, 3,31, 7) = (/ &
     &1.3911e+01_rb,1.0649e+01_rb,7.6174e+00_rb,4.3452e+00_rb,9.4250e-02_rb/)
      kbo(:, 4,31, 7) = (/ &
     &1.5017e+01_rb,1.1456e+01_rb,8.1532e+00_rb,4.6551e+00_rb,9.9406e-02_rb/)
      kbo(:, 5,31, 7) = (/ &
     &1.6266e+01_rb,1.2373e+01_rb,8.7552e+00_rb,5.0004e+00_rb,1.0401e-01_rb/)
      kbo(:, 1,32, 7) = (/ &
     &1.2189e+01_rb,9.4190e+00_rb,6.7775e+00_rb,3.8398e+00_rb,7.2780e-02_rb/)
      kbo(:, 2,32, 7) = (/ &
     &1.3084e+01_rb,1.0062e+01_rb,7.2229e+00_rb,4.1020e+00_rb,7.7761e-02_rb/)
      kbo(:, 3,32, 7) = (/ &
     &1.4139e+01_rb,1.0827e+01_rb,7.7437e+00_rb,4.4056e+00_rb,8.2541e-02_rb/)
      kbo(:, 4,32, 7) = (/ &
     &1.5347e+01_rb,1.1710e+01_rb,8.3333e+00_rb,4.7481e+00_rb,8.6905e-02_rb/)
      kbo(:, 5,32, 7) = (/ &
     &1.6687e+01_rb,1.2692e+01_rb,8.9809e+00_rb,5.1206e+00_rb,9.0564e-02_rb/)
      kbo(:, 1,33, 7) = (/ &
     &1.2504e+01_rb,9.6581e+00_rb,6.9429e+00_rb,3.9220e+00_rb,6.3643e-02_rb/)
      kbo(:, 2,33, 7) = (/ &
     &1.3523e+01_rb,1.0393e+01_rb,7.4532e+00_rb,4.2218e+00_rb,6.8037e-02_rb/)
      kbo(:, 3,33, 7) = (/ &
     &1.4701e+01_rb,1.1251e+01_rb,8.0374e+00_rb,4.5635e+00_rb,7.2051e-02_rb/)
      kbo(:, 4,33, 7) = (/ &
     &1.6035e+01_rb,1.2227e+01_rb,8.6879e+00_rb,4.9410e+00_rb,7.5520e-02_rb/)
      kbo(:, 5,33, 7) = (/ &
     &1.7498e+01_rb,1.3301e+01_rb,9.3946e+00_rb,5.3470e+00_rb,7.8343e-02_rb/)
      kbo(:, 1,34, 7) = (/ &
     &1.2544e+01_rb,9.6932e+00_rb,6.9709e+00_rb,3.9324e+00_rb,5.5495e-02_rb/)
      kbo(:, 2,34, 7) = (/ &
     &1.3652e+01_rb,1.0495e+01_rb,7.5288e+00_rb,4.2613e+00_rb,5.9207e-02_rb/)
      kbo(:, 3,34, 7) = (/ &
     &1.4931e+01_rb,1.1428e+01_rb,8.1665e+00_rb,4.6328e+00_rb,6.2449e-02_rb/)
      kbo(:, 4,34, 7) = (/ &
     &1.6342e+01_rb,1.2460e+01_rb,8.8567e+00_rb,5.0367e+00_rb,6.5144e-02_rb/)
      kbo(:, 5,34, 7) = (/ &
     &1.7926e+01_rb,1.3623e+01_rb,9.6166e+00_rb,5.4694e+00_rb,6.7469e-02_rb/)
      kbo(:, 1,35, 7) = (/ &
     &1.2550e+01_rb,9.7037e+00_rb,6.9833e+00_rb,3.9358e+00_rb,4.7960e-02_rb/)
      kbo(:, 2,35, 7) = (/ &
     &1.3742e+01_rb,1.0569e+01_rb,7.5854e+00_rb,4.2904e+00_rb,5.1023e-02_rb/)
      kbo(:, 3,35, 7) = (/ &
     &1.5093e+01_rb,1.1555e+01_rb,8.2621e+00_rb,4.6878e+00_rb,5.3624e-02_rb/)
      kbo(:, 4,35, 7) = (/ &
     &1.6591e+01_rb,1.2650e+01_rb,8.9950e+00_rb,5.1151e+00_rb,5.5900e-02_rb/)
      kbo(:, 5,35, 7) = (/ &
     &1.8304e+01_rb,1.3907e+01_rb,9.8148e+00_rb,5.5812e+00_rb,5.7988e-02_rb/)
      kbo(:, 1,36, 7) = (/ &
     &1.2397e+01_rb,9.5976e+00_rb,6.9138e+00_rb,3.8949e+00_rb,4.1048e-02_rb/)
      kbo(:, 2,36, 7) = (/ &
     &1.3655e+01_rb,1.0513e+01_rb,7.5553e+00_rb,4.2704e+00_rb,4.3574e-02_rb/)
      kbo(:, 3,36, 7) = (/ &
     &1.5056e+01_rb,1.1535e+01_rb,8.2577e+00_rb,4.6863e+00_rb,4.5767e-02_rb/)
      kbo(:, 4,36, 7) = (/ &
     &1.6621e+01_rb,1.2678e+01_rb,9.0220e+00_rb,5.1309e+00_rb,4.7778e-02_rb/)
      kbo(:, 5,36, 7) = (/ &
     &1.8454e+01_rb,1.4024e+01_rb,9.9036e+00_rb,5.6291e+00_rb,4.9683e-02_rb/)
      kbo(:, 1,37, 7) = (/ &
     &1.1793e+01_rb,9.1604e+00_rb,6.6146e+00_rb,3.7234e+00_rb,3.4707e-02_rb/)
      kbo(:, 2,37, 7) = (/ &
     &1.3041e+01_rb,1.0070e+01_rb,7.2604e+00_rb,4.1046e+00_rb,3.6864e-02_rb/)
      kbo(:, 3,37, 7) = (/ &
     &1.4439e+01_rb,1.1088e+01_rb,7.9659e+00_rb,4.5244e+00_rb,3.8795e-02_rb/)
      kbo(:, 4,37, 7) = (/ &
     &1.6025e+01_rb,1.2245e+01_rb,8.7442e+00_rb,4.9773e+00_rb,4.0613e-02_rb/)
      kbo(:, 5,37, 7) = (/ &
     &1.7882e+01_rb,1.3605e+01_rb,9.6420e+00_rb,5.4880e+00_rb,4.2339e-02_rb/)
      kbo(:, 1,38, 7) = (/ &
     &1.1383e+01_rb,8.8666e+00_rb,6.4134e+00_rb,3.6072e+00_rb,2.9298e-02_rb/)
      kbo(:, 2,38, 7) = (/ &
     &1.2643e+01_rb,9.7859e+00_rb,7.0699e+00_rb,3.9972e+00_rb,3.1148e-02_rb/)
      kbo(:, 3,38, 7) = (/ &
     &1.4056e+01_rb,1.0814e+01_rb,7.7862e+00_rb,4.4229e+00_rb,3.2867e-02_rb/)
      kbo(:, 4,38, 7) = (/ &
     &1.5686e+01_rb,1.2002e+01_rb,8.5915e+00_rb,4.8935e+00_rb,3.4517e-02_rb/)
      kbo(:, 5,38, 7) = (/ &
     &1.7595e+01_rb,1.3400e+01_rb,9.5179e+00_rb,5.4208e+00_rb,3.6067e-02_rb/)
      kbo(:, 1,39, 7) = (/ &
     &1.1254e+01_rb,8.7785e+00_rb,6.3532e+00_rb,3.5698e+00_rb,2.4703e-02_rb/)
      kbo(:, 2,39, 7) = (/ &
     &1.2546e+01_rb,9.7238e+00_rb,7.0319e+00_rb,3.9725e+00_rb,2.6328e-02_rb/)
      kbo(:, 3,39, 7) = (/ &
     &1.4010e+01_rb,1.0789e+01_rb,7.7764e+00_rb,4.4152e+00_rb,2.7854e-02_rb/)
      kbo(:, 4,39, 7) = (/ &
     &1.5714e+01_rb,1.2031e+01_rb,8.6190e+00_rb,4.9070e+00_rb,2.9325e-02_rb/)
      kbo(:, 5,39, 7) = (/ &
     &1.7752e+01_rb,1.3525e+01_rb,9.6124e+00_rb,5.4718e+00_rb,3.0788e-02_rb/)
      kbo(:, 1,40, 7) = (/ &
     &1.0343e+01_rb,8.1149e+00_rb,5.8880e+00_rb,3.3077e+00_rb,2.0662e-02_rb/)
      kbo(:, 2,40, 7) = (/ &
     &1.1581e+01_rb,9.0215e+00_rb,6.5477e+00_rb,3.7008e+00_rb,2.2084e-02_rb/)
      kbo(:, 3,40, 7) = (/ &
     &1.2986e+01_rb,1.0044e+01_rb,7.2710e+00_rb,4.1322e+00_rb,2.3431e-02_rb/)
      kbo(:, 4,40, 7) = (/ &
     &1.4638e+01_rb,1.1247e+01_rb,8.0993e+00_rb,4.6196e+00_rb,2.4776e-02_rb/)
      kbo(:, 5,40, 7) = (/ &
     &1.6625e+01_rb,1.2700e+01_rb,9.0740e+00_rb,5.1787e+00_rb,2.6103e-02_rb/)
      kbo(:, 1,41, 7) = (/ &
     &9.4683e+00_rb,7.4757e+00_rb,5.4378e+00_rb,3.0540e+00_rb,1.7243e-02_rb/)
      kbo(:, 2,41, 7) = (/ &
     &1.0645e+01_rb,8.3400e+00_rb,6.0743e+00_rb,3.4334e+00_rb,1.8487e-02_rb/)
      kbo(:, 3,41, 7) = (/ &
     &1.1996e+01_rb,9.3250e+00_rb,6.7794e+00_rb,3.8546e+00_rb,1.9684e-02_rb/)
      kbo(:, 4,41, 7) = (/ &
     &1.3587e+01_rb,1.0480e+01_rb,7.5843e+00_rb,4.3322e+00_rb,2.0909e-02_rb/)
      kbo(:, 5,41, 7) = (/ &
     &1.5513e+01_rb,1.1886e+01_rb,8.5396e+00_rb,4.8864e+00_rb,2.2113e-02_rb/)
      kbo(:, 1,42, 7) = (/ &
     &8.7596e+00_rb,6.9575e+00_rb,5.0695e+00_rb,2.8442e+00_rb,1.4375e-02_rb/)
      kbo(:, 2,42, 7) = (/ &
     &9.8918e+00_rb,7.7921e+00_rb,5.6895e+00_rb,3.2145e+00_rb,1.5469e-02_rb/)
      kbo(:, 3,42, 7) = (/ &
     &1.1196e+01_rb,8.7450e+00_rb,6.3808e+00_rb,3.6290e+00_rb,1.6542e-02_rb/)
      kbo(:, 4,42, 7) = (/ &
     &1.2750e+01_rb,9.8741e+00_rb,7.1757e+00_rb,4.1018e+00_rb,1.7643e-02_rb/)
      kbo(:, 5,42, 7) = (/ &
     &1.4631e+01_rb,1.1244e+01_rb,8.1146e+00_rb,4.6494e+00_rb,1.8736e-02_rb/)
      kbo(:, 1,43, 7) = (/ &
     &8.1129e+00_rb,6.4828e+00_rb,4.7282e+00_rb,2.6502e+00_rb,1.1920e-02_rb/)
      kbo(:, 2,43, 7) = (/ &
     &9.2040e+00_rb,7.2919e+00_rb,5.3332e+00_rb,3.0110e+00_rb,1.2884e-02_rb/)
      kbo(:, 3,43, 7) = (/ &
     &1.0467e+01_rb,8.2176e+00_rb,6.0123e+00_rb,3.4195e+00_rb,1.3839e-02_rb/)
      kbo(:, 4,43, 7) = (/ &
     &1.1975e+01_rb,9.3139e+00_rb,6.7911e+00_rb,3.8837e+00_rb,1.4824e-02_rb/)
      kbo(:, 5,43, 7) = (/ &
     &1.3821e+01_rb,1.0658e+01_rb,7.7247e+00_rb,4.4294e+00_rb,1.5816e-02_rb/)
      kbo(:, 1,44, 7) = (/ &
     &7.5813e+00_rb,6.0889e+00_rb,4.4438e+00_rb,2.4886e+00_rb,9.8504e-03_rb/)
      kbo(:, 2,44, 7) = (/ &
     &8.6433e+00_rb,6.8838e+00_rb,5.0388e+00_rb,2.8416e+00_rb,1.0695e-02_rb/)
      kbo(:, 3,44, 7) = (/ &
     &9.8815e+00_rb,7.7937e+00_rb,5.7119e+00_rb,3.2452e+00_rb,1.1549e-02_rb/)
      kbo(:, 4,44, 7) = (/ &
     &1.1354e+01_rb,8.8664e+00_rb,6.4827e+00_rb,3.7047e+00_rb,1.2430e-02_rb/)
      kbo(:, 5,44, 7) = (/ &
     &1.3162e+01_rb,1.0183e+01_rb,7.4060e+00_rb,4.2473e+00_rb,1.3335e-02_rb/)
      kbo(:, 1,45, 7) = (/ &
     &7.2186e+00_rb,5.8199e+00_rb,4.2461e+00_rb,2.3747e+00_rb,8.1314e-03_rb/)
      kbo(:, 2,45, 7) = (/ &
     &8.2698e+00_rb,6.6114e+00_rb,4.8385e+00_rb,2.7251e+00_rb,8.8750e-03_rb/)
      kbo(:, 3,45, 7) = (/ &
     &9.5031e+00_rb,7.5246e+00_rb,5.5174e+00_rb,3.1306e+00_rb,9.6297e-03_rb/)
      kbo(:, 4,45, 7) = (/ &
     &1.0974e+01_rb,8.5972e+00_rb,6.2944e+00_rb,3.5936e+00_rb,1.0417e-02_rb/)
      kbo(:, 5,45, 7) = (/ &
     &1.2791e+01_rb,9.9194e+00_rb,7.2286e+00_rb,4.1456e+00_rb,1.1234e-02_rb/)
      kbo(:, 1,46, 7) = (/ &
     &6.9141e+00_rb,5.5940e+00_rb,4.0776e+00_rb,2.2758e+00_rb,6.6876e-03_rb/)
      kbo(:, 2,46, 7) = (/ &
     &7.9503e+00_rb,6.3788e+00_rb,4.6670e+00_rb,2.6226e+00_rb,7.3399e-03_rb/)
      kbo(:, 3,46, 7) = (/ &
     &9.1864e+00_rb,7.2991e+00_rb,5.3560e+00_rb,3.0335e+00_rb,8.0038e-03_rb/)
      kbo(:, 4,46, 7) = (/ &
     &1.0667e+01_rb,8.3803e+00_rb,6.1440e+00_rb,3.5030e+00_rb,8.7040e-03_rb/)
      kbo(:, 5,46, 7) = (/ &
     &1.2488e+01_rb,9.7091e+00_rb,7.0857e+00_rb,4.0595e+00_rb,9.4400e-03_rb/)
      kbo(:, 1,47, 7) = (/ &
     &6.5158e+00_rb,5.2952e+00_rb,3.8553e+00_rb,2.1461e+00_rb,5.4578e-03_rb/)
      kbo(:, 2,47, 7) = (/ &
     &7.5125e+00_rb,6.0542e+00_rb,4.4282e+00_rb,2.4823e+00_rb,6.0287e-03_rb/)
      kbo(:, 3,47, 7) = (/ &
     &8.7277e+00_rb,6.9654e+00_rb,5.1130e+00_rb,2.8905e+00_rb,6.6123e-03_rb/)
      kbo(:, 4,47, 7) = (/ &
     &1.0180e+01_rb,8.0330e+00_rb,5.8977e+00_rb,3.3599e+00_rb,7.2304e-03_rb/)
      kbo(:, 5,47, 7) = (/ &
     &1.1985e+01_rb,9.3498e+00_rb,6.8393e+00_rb,3.9162e+00_rb,7.8875e-03_rb/)
      kbo(:, 1,48, 7) = (/ &
     &6.3104e+00_rb,5.1398e+00_rb,3.7343e+00_rb,2.0712e+00_rb,4.4438e-03_rb/)
      kbo(:, 2,48, 7) = (/ &
     &7.2865e+00_rb,5.8865e+00_rb,4.2999e+00_rb,2.4038e+00_rb,4.9412e-03_rb/)
      kbo(:, 3,48, 7) = (/ &
     &8.5071e+00_rb,6.8079e+00_rb,4.9920e+00_rb,2.8153e+00_rb,5.4521e-03_rb/)
      kbo(:, 4,48, 7) = (/ &
     &9.9728e+00_rb,7.8905e+00_rb,5.7928e+00_rb,3.2942e+00_rb,5.9945e-03_rb/)
      kbo(:, 5,48, 7) = (/ &
     &1.1799e+01_rb,9.2259e+00_rb,6.7537e+00_rb,3.8639e+00_rb,6.5748e-03_rb/)
      kbo(:, 1,49, 7) = (/ &
     &6.3637e+00_rb,5.1793e+00_rb,3.7516e+00_rb,2.0723e+00_rb,3.6085e-03_rb/)
      kbo(:, 2,49, 7) = (/ &
     &7.3474e+00_rb,5.9332e+00_rb,4.3233e+00_rb,2.4087e+00_rb,4.0406e-03_rb/)
      kbo(:, 3,49, 7) = (/ &
     &8.6221e+00_rb,6.8978e+00_rb,5.0458e+00_rb,2.8357e+00_rb,4.4876e-03_rb/)
      kbo(:, 4,49, 7) = (/ &
     &1.0155e+01_rb,8.0340e+00_rb,5.8866e+00_rb,3.3379e+00_rb,4.9641e-03_rb/)
      kbo(:, 5,49, 7) = (/ &
     &1.2071e+01_rb,9.4371e+00_rb,6.8986e+00_rb,3.9361e+00_rb,5.4780e-03_rb/)
      kbo(:, 1,50, 7) = (/ &
     &6.1287e+00_rb,4.9982e+00_rb,3.6130e+00_rb,1.9898e+00_rb,2.9263e-03_rb/)
      kbo(:, 2,50, 7) = (/ &
     &7.0793e+00_rb,5.7315e+00_rb,4.1690e+00_rb,2.3166e+00_rb,3.3022e-03_rb/)
      kbo(:, 3,50, 7) = (/ &
     &8.3385e+00_rb,6.6892e+00_rb,4.8872e+00_rb,2.7388e+00_rb,3.6894e-03_rb/)
      kbo(:, 4,50, 7) = (/ &
     &9.8705e+00_rb,7.8310e+00_rb,5.7364e+00_rb,3.2460e+00_rb,4.1061e-03_rb/)
      kbo(:, 5,50, 7) = (/ &
     &1.1792e+01_rb,9.2428e+00_rb,6.7611e+00_rb,3.8534e+00_rb,4.5631e-03_rb/)
      kbo(:, 1,51, 7) = (/ &
     &5.7768e+00_rb,4.7273e+00_rb,3.4121e+00_rb,1.8743e+00_rb,2.3693e-03_rb/)
      kbo(:, 2,51, 7) = (/ &
     &6.6763e+00_rb,5.4272e+00_rb,3.9426e+00_rb,2.1849e+00_rb,2.6931e-03_rb/)
      kbo(:, 3,51, 7) = (/ &
     &7.8931e+00_rb,6.3576e+00_rb,4.6398e+00_rb,2.5939e+00_rb,3.0298e-03_rb/)
      kbo(:, 4,51, 7) = (/ &
     &9.3905e+00_rb,7.4818e+00_rb,5.4825e+00_rb,3.0966e+00_rb,3.3909e-03_rb/)
      kbo(:, 5,51, 7) = (/ &
     &1.1270e+01_rb,8.8663e+00_rb,6.4953e+00_rb,3.6986e+00_rb,3.7911e-03_rb/)
      kbo(:, 1,52, 7) = (/ &
     &5.5982e+00_rb,4.5875e+00_rb,3.3045e+00_rb,1.8093e+00_rb,1.9133e-03_rb/)
      kbo(:, 2,52, 7) = (/ &
     &6.4669e+00_rb,5.2676e+00_rb,3.8189e+00_rb,2.1105e+00_rb,2.1929e-03_rb/)
      kbo(:, 3,52, 7) = (/ &
     &7.6692e+00_rb,6.1904e+00_rb,4.5114e+00_rb,2.5133e+00_rb,2.4833e-03_rb/)
      kbo(:, 4,52, 7) = (/ &
     &9.1760e+00_rb,7.3261e+00_rb,5.3623e+00_rb,3.0203e+00_rb,2.7953e-03_rb/)
      kbo(:, 5,52, 7) = (/ &
     &1.1059e+01_rb,8.7181e+00_rb,6.3851e+00_rb,3.6291e+00_rb,3.1452e-03_rb/)
      kbo(:, 1,53, 7) = (/ &
     &5.6498e+00_rb,4.6222e+00_rb,3.3203e+00_rb,1.8115e+00_rb,1.5408e-03_rb/)
      kbo(:, 2,53, 7) = (/ &
     &6.5197e+00_rb,5.3054e+00_rb,3.8365e+00_rb,2.1129e+00_rb,1.7821e-03_rb/)
      kbo(:, 3,53, 7) = (/ &
     &7.7464e+00_rb,6.2478e+00_rb,4.5414e+00_rb,2.5222e+00_rb,2.0319e-03_rb/)
      kbo(:, 4,53, 7) = (/ &
     &9.3204e+00_rb,7.4376e+00_rb,5.4308e+00_rb,3.0476e+00_rb,2.3029e-03_rb/)
      kbo(:, 5,53, 7) = (/ &
     &1.1294e+01_rb,8.9013e+00_rb,6.5042e+00_rb,3.6848e+00_rb,2.6089e-03_rb/)
      kbo(:, 1,54, 7) = (/ &
     &4.9410e+00_rb,4.0701e+00_rb,2.9220e+00_rb,1.5920e+00_rb,1.2393e-03_rb/)
      kbo(:, 2,54, 7) = (/ &
     &5.7011e+00_rb,4.6779e+00_rb,3.3818e+00_rb,1.8599e+00_rb,1.4465e-03_rb/)
      kbo(:, 3,54, 7) = (/ &
     &6.7851e+00_rb,5.5214e+00_rb,4.0167e+00_rb,2.2288e+00_rb,1.6602e-03_rb/)
      kbo(:, 4,54, 7) = (/ &
     &8.2133e+00_rb,6.6133e+00_rb,4.8372e+00_rb,2.7129e+00_rb,1.8918e-03_rb/)
      kbo(:, 5,54, 7) = (/ &
     &1.0009e+01_rb,7.9570e+00_rb,5.8358e+00_rb,3.3045e+00_rb,2.1561e-03_rb/)
      kbo(:, 1,55, 7) = (/ &
     &3.9448e+00_rb,3.2894e+00_rb,2.3617e+00_rb,1.2863e+00_rb,9.9479e-04_rb/)
      kbo(:, 2,55, 7) = (/ &
     &4.5510e+00_rb,3.7868e+00_rb,2.7406e+00_rb,1.5076e+00_rb,1.1698e-03_rb/)
      kbo(:, 3,55, 7) = (/ &
     &5.4272e+00_rb,4.4849e+00_rb,3.2681e+00_rb,1.8142e+00_rb,1.3510e-03_rb/)
      kbo(:, 4,55, 7) = (/ &
     &6.6050e+00_rb,5.4031e+00_rb,3.9651e+00_rb,2.2259e+00_rb,1.5494e-03_rb/)
      kbo(:, 5,55, 7) = (/ &
     &8.0992e+00_rb,6.5420e+00_rb,4.8249e+00_rb,2.7373e+00_rb,1.7731e-03_rb/)
      kbo(:, 1,56, 7) = (/ &
     &3.1456e+00_rb,2.6539e+00_rb,1.9051e+00_rb,1.0370e+00_rb,7.9618e-04_rb/)
      kbo(:, 2,56, 7) = (/ &
     &3.6287e+00_rb,3.0618e+00_rb,2.2169e+00_rb,1.2199e+00_rb,9.4437e-04_rb/)
      kbo(:, 3,56, 7) = (/ &
     &4.3315e+00_rb,3.6363e+00_rb,2.6526e+00_rb,1.4737e+00_rb,1.0978e-03_rb/)
      kbo(:, 4,56, 7) = (/ &
     &5.3003e+00_rb,4.4092e+00_rb,3.2425e+00_rb,1.8210e+00_rb,1.2654e-03_rb/)
      kbo(:, 5,56, 7) = (/ &
     &6.5412e+00_rb,5.3749e+00_rb,3.9813e+00_rb,2.2622e+00_rb,1.4546e-03_rb/)
      kbo(:, 1,57, 7) = (/ &
     &2.5050e+00_rb,2.1372e+00_rb,1.5333e+00_rb,8.3409e-01_rb,6.3524e-04_rb/)
      kbo(:, 2,57, 7) = (/ &
     &2.8897e+00_rb,2.4717e+00_rb,1.7900e+00_rb,9.8489e-01_rb,7.5959e-04_rb/)
      kbo(:, 3,57, 7) = (/ &
     &3.4502e+00_rb,2.9412e+00_rb,2.1481e+00_rb,1.1943e+00_rb,8.8948e-04_rb/)
      kbo(:, 4,57, 7) = (/ &
     &4.2411e+00_rb,3.5892e+00_rb,2.6434e+00_rb,1.4858e+00_rb,1.0311e-03_rb/)
      kbo(:, 5,57, 7) = (/ &
     &5.2712e+00_rb,4.4105e+00_rb,3.2776e+00_rb,1.8647e+00_rb,1.1913e-03_rb/)
      kbo(:, 1,58, 7) = (/ &
     &5.2639e-01_rb,5.5628e-01_rb,5.1310e-01_rb,3.9091e-01_rb,5.0641e-04_rb/)
      kbo(:, 2,58, 7) = (/ &
     &6.0772e-01_rb,6.4553e-01_rb,6.0135e-01_rb,4.6372e-01_rb,6.1035e-04_rb/)
      kbo(:, 3,58, 7) = (/ &
     &7.2539e-01_rb,7.7016e-01_rb,7.2403e-01_rb,5.6463e-01_rb,7.2056e-04_rb/)
      kbo(:, 4,58, 7) = (/ &
     &8.9558e-01_rb,9.4507e-01_rb,8.9682e-01_rb,7.0739e-01_rb,8.3918e-04_rb/)
      kbo(:, 5,58, 7) = (/ &
     &1.1215e+00_rb,1.1728e+00_rb,1.1240e+00_rb,8.9769e-01_rb,9.7498e-04_rb/)
      kbo(:, 1,59, 7) = (/ &
     &5.0774e-01_rb,5.2725e-01_rb,4.5400e-01_rb,3.0925e-01_rb,4.1598e-04_rb/)
      kbo(:, 2,59, 7) = (/ &
     &5.8906e-01_rb,6.1772e-01_rb,5.3777e-01_rb,3.7151e-01_rb,5.0342e-04_rb/)
      kbo(:, 3,59, 7) = (/ &
     &7.0796e-01_rb,7.4490e-01_rb,6.5525e-01_rb,4.5851e-01_rb,5.9609e-04_rb/)
      kbo(:, 4,59, 7) = (/ &
     &8.8130e-01_rb,9.2517e-01_rb,8.2251e-01_rb,5.8308e-01_rb,6.9661e-04_rb/)
      kbo(:, 5,59, 7) = (/ &
     &1.1130e+00_rb,1.1619e+00_rb,1.0449e+00_rb,7.5118e-01_rb,8.1226e-04_rb/)
      kbo(:, 1,13, 8) = (/ &
     &5.5674e+03_rb,4.1757e+03_rb,2.7839e+03_rb,1.3922e+03_rb,1.1827e+00_rb/)
      kbo(:, 2,13, 8) = (/ &
     &5.5672e+03_rb,4.1755e+03_rb,2.7838e+03_rb,1.3922e+03_rb,1.1780e+00_rb/)
      kbo(:, 3,13, 8) = (/ &
     &5.5974e+03_rb,4.1981e+03_rb,2.7989e+03_rb,1.3997e+03_rb,1.1714e+00_rb/)
      kbo(:, 4,13, 8) = (/ &
     &5.6743e+03_rb,4.2558e+03_rb,2.8374e+03_rb,1.4189e+03_rb,1.1612e+00_rb/)
      kbo(:, 5,13, 8) = (/ &
     &5.7999e+03_rb,4.3500e+03_rb,2.9002e+03_rb,1.4503e+03_rb,1.1566e+00_rb/)
      kbo(:, 1,14, 8) = (/ &
     &3.0072e+03_rb,2.2555e+03_rb,1.5038e+03_rb,7.5211e+02_rb,1.1873e+00_rb/)
      kbo(:, 2,14, 8) = (/ &
     &3.0260e+03_rb,2.2696e+03_rb,1.5132e+03_rb,7.5683e+02_rb,1.1778e+00_rb/)
      kbo(:, 3,14, 8) = (/ &
     &3.0739e+03_rb,2.3055e+03_rb,1.5371e+03_rb,7.6880e+02_rb,1.1759e+00_rb/)
      kbo(:, 4,14, 8) = (/ &
     &3.1534e+03_rb,2.3651e+03_rb,1.5769e+03_rb,7.8868e+02_rb,1.1686e+00_rb/)
      kbo(:, 5,14, 8) = (/ &
     &3.2601e+03_rb,2.4451e+03_rb,1.6302e+03_rb,8.1536e+02_rb,1.1588e+00_rb/)
      kbo(:, 1,15, 8) = (/ &
     &1.6681e+03_rb,1.2512e+03_rb,8.3428e+02_rb,4.1737e+02_rb,1.1796e+00_rb/)
      kbo(:, 2,15, 8) = (/ &
     &1.6959e+03_rb,1.2720e+03_rb,8.4818e+02_rb,4.2432e+02_rb,1.1712e+00_rb/)
      kbo(:, 3,15, 8) = (/ &
     &1.7438e+03_rb,1.3080e+03_rb,8.7214e+02_rb,4.3631e+02_rb,1.1630e+00_rb/)
      kbo(:, 4,15, 8) = (/ &
     &1.8057e+03_rb,1.3544e+03_rb,9.0311e+02_rb,4.5180e+02_rb,1.1584e+00_rb/)
      kbo(:, 5,15, 8) = (/ &
     &1.8729e+03_rb,1.4048e+03_rb,9.3671e+02_rb,4.6860e+02_rb,1.1490e+00_rb/)
      kbo(:, 1,16, 8) = (/ &
     &1.1127e+03_rb,8.3463e+02_rb,5.5657e+02_rb,2.7851e+02_rb,1.1544e+00_rb/)
      kbo(:, 2,16, 8) = (/ &
     &1.1452e+03_rb,8.5902e+02_rb,5.7284e+02_rb,2.8665e+02_rb,1.1489e+00_rb/)
      kbo(:, 3,16, 8) = (/ &
     &1.1859e+03_rb,8.8958e+02_rb,5.9321e+02_rb,2.9685e+02_rb,1.1423e+00_rb/)
      kbo(:, 4,16, 8) = (/ &
     &1.2314e+03_rb,9.2370e+02_rb,6.1596e+02_rb,3.0822e+02_rb,1.1380e+00_rb/)
      kbo(:, 5,16, 8) = (/ &
     &1.2793e+03_rb,9.5963e+02_rb,6.3991e+02_rb,3.2020e+02_rb,1.1307e+00_rb/)
      kbo(:, 1,17, 8) = (/ &
     &7.7661e+02_rb,5.8258e+02_rb,3.8854e+02_rb,1.9450e+02_rb,1.1162e+00_rb/)
      kbo(:, 2,17, 8) = (/ &
     &8.0415e+02_rb,6.0323e+02_rb,4.0231e+02_rb,2.0139e+02_rb,1.1130e+00_rb/)
      kbo(:, 3,17, 8) = (/ &
     &8.3554e+02_rb,6.2677e+02_rb,4.1801e+02_rb,2.0924e+02_rb,1.1093e+00_rb/)
      kbo(:, 4,17, 8) = (/ &
     &8.6787e+02_rb,6.5102e+02_rb,4.3418e+02_rb,2.1733e+02_rb,1.1057e+00_rb/)
      kbo(:, 5,17, 8) = (/ &
     &9.0039e+02_rb,6.7541e+02_rb,4.5043e+02_rb,2.2545e+02_rb,1.0994e+00_rb/)
      kbo(:, 1,18, 8) = (/ &
     &5.5593e+02_rb,4.1706e+02_rb,2.7819e+02_rb,1.3933e+02_rb,1.0653e+00_rb/)
      kbo(:, 2,18, 8) = (/ &
     &5.7731e+02_rb,4.3309e+02_rb,2.8888e+02_rb,1.4467e+02_rb,1.0637e+00_rb/)
      kbo(:, 3,18, 8) = (/ &
     &5.9962e+02_rb,4.4983e+02_rb,3.0005e+02_rb,1.5026e+02_rb,1.0625e+00_rb/)
      kbo(:, 4,18, 8) = (/ &
     &6.2213e+02_rb,4.6672e+02_rb,3.1130e+02_rb,1.5589e+02_rb,1.0595e+00_rb/)
      kbo(:, 5,18, 8) = (/ &
     &6.4470e+02_rb,4.8364e+02_rb,3.2258e+02_rb,1.6152e+02_rb,1.0552e+00_rb/)
      kbo(:, 1,19, 8) = (/ &
     &3.9059e+02_rb,2.9306e+02_rb,1.9552e+02_rb,9.7992e+01_rb,1.0025e+00_rb/)
      kbo(:, 2,19, 8) = (/ &
     &4.0544e+02_rb,3.0420e+02_rb,2.0295e+02_rb,1.0172e+02_rb,1.0019e+00_rb/)
      kbo(:, 3,19, 8) = (/ &
     &4.2056e+02_rb,3.1554e+02_rb,2.1052e+02_rb,1.0550e+02_rb,1.0025e+00_rb/)
      kbo(:, 4,19, 8) = (/ &
     &4.3576e+02_rb,3.2693e+02_rb,2.1811e+02_rb,1.0930e+02_rb,1.0012e+00_rb/)
      kbo(:, 5,19, 8) = (/ &
     &4.5126e+02_rb,3.3856e+02_rb,2.2586e+02_rb,1.1317e+02_rb,9.9899e-01_rb/)
      kbo(:, 1,20, 8) = (/ &
     &2.9617e+02_rb,2.2224e+02_rb,1.4831e+02_rb,7.4392e+01_rb,9.3059e-01_rb/)
      kbo(:, 2,20, 8) = (/ &
     &3.0716e+02_rb,2.3049e+02_rb,1.5382e+02_rb,7.7154e+01_rb,9.3219e-01_rb/)
      kbo(:, 3,20, 8) = (/ &
     &3.1816e+02_rb,2.3874e+02_rb,1.5932e+02_rb,7.9908e+01_rb,9.3370e-01_rb/)
      kbo(:, 4,20, 8) = (/ &
     &3.2952e+02_rb,2.4726e+02_rb,1.6500e+02_rb,8.2741e+01_rb,9.3435e-01_rb/)
      kbo(:, 5,20, 8) = (/ &
     &3.4150e+02_rb,2.5625e+02_rb,1.7099e+02_rb,8.5741e+01_rb,9.3474e-01_rb/)
      kbo(:, 1,21, 8) = (/ &
     &2.3042e+02_rb,1.7293e+02_rb,1.1544e+02_rb,5.7965e+01_rb,8.5235e-01_rb/)
      kbo(:, 2,21, 8) = (/ &
     &2.3880e+02_rb,1.7922e+02_rb,1.1964e+02_rb,6.0070e+01_rb,8.5599e-01_rb/)
      kbo(:, 3,21, 8) = (/ &
     &2.4721e+02_rb,1.8553e+02_rb,1.2385e+02_rb,6.2177e+01_rb,8.5938e-01_rb/)
      kbo(:, 4,21, 8) = (/ &
     &2.5622e+02_rb,1.9229e+02_rb,1.2835e+02_rb,6.4426e+01_rb,8.6241e-01_rb/)
      kbo(:, 5,21, 8) = (/ &
     &2.6653e+02_rb,2.0002e+02_rb,1.3351e+02_rb,6.7009e+01_rb,8.6670e-01_rb/)
      kbo(:, 1,22, 8) = (/ &
     &1.8346e+02_rb,1.3771e+02_rb,9.1962e+01_rb,4.6247e+01_rb,7.7175e-01_rb/)
      kbo(:, 2,22, 8) = (/ &
     &1.8996e+02_rb,1.4259e+02_rb,9.5222e+01_rb,4.7880e+01_rb,7.7757e-01_rb/)
      kbo(:, 3,22, 8) = (/ &
     &1.9685e+02_rb,1.4775e+02_rb,9.8665e+01_rb,4.9599e+01_rb,7.8369e-01_rb/)
      kbo(:, 4,22, 8) = (/ &
     &2.0481e+02_rb,1.5373e+02_rb,1.0265e+02_rb,5.1591e+01_rb,7.9052e-01_rb/)
      kbo(:, 5,22, 8) = (/ &
     &2.1438e+02_rb,1.6091e+02_rb,1.0745e+02_rb,5.3989e+01_rb,7.9865e-01_rb/)
      kbo(:, 1,23, 8) = (/ &
     &1.4619e+02_rb,1.0976e+02_rb,7.3333e+01_rb,3.7009e+01_rb,6.9269e-01_rb/)
      kbo(:, 2,23, 8) = (/ &
     &1.5146e+02_rb,1.1371e+02_rb,7.5969e+01_rb,3.8312e+01_rb,7.0058e-01_rb/)
      kbo(:, 3,23, 8) = (/ &
     &1.5752e+02_rb,1.1826e+02_rb,7.9004e+01_rb,3.9812e+01_rb,7.0997e-01_rb/)
      kbo(:, 4,23, 8) = (/ &
     &1.6498e+02_rb,1.2386e+02_rb,8.2742e+01_rb,4.1669e+01_rb,7.2082e-01_rb/)
      kbo(:, 5,23, 8) = (/ &
     &1.7382e+02_rb,1.3049e+02_rb,8.7175e+01_rb,4.3881e+01_rb,7.3247e-01_rb/)
      kbo(:, 1,24, 8) = (/ &
     &1.1546e+02_rb,8.6713e+01_rb,5.7975e+01_rb,2.9484e+01_rb,6.1736e-01_rb/)
      kbo(:, 2,24, 8) = (/ &
     &1.1997e+02_rb,9.0096e+01_rb,6.0229e+01_rb,3.0574e+01_rb,6.2822e-01_rb/)
      kbo(:, 3,24, 8) = (/ &
     &1.2560e+02_rb,9.4324e+01_rb,6.3051e+01_rb,3.1946e+01_rb,6.4118e-01_rb/)
      kbo(:, 4,24, 8) = (/ &
     &1.3251e+02_rb,9.9515e+01_rb,6.6517e+01_rb,3.3646e+01_rb,6.5583e-01_rb/)
      kbo(:, 5,24, 8) = (/ &
     &1.4039e+02_rb,1.0542e+02_rb,7.0467e+01_rb,3.5600e+01_rb,6.7045e-01_rb/)
      kbo(:, 1,25, 8) = (/ &
     &9.2335e+01_rb,6.9366e+01_rb,4.6428e+01_rb,2.3921e+01_rb,5.4857e-01_rb/)
      kbo(:, 2,25, 8) = (/ &
     &9.6533e+01_rb,7.2518e+01_rb,4.8524e+01_rb,2.4911e+01_rb,5.6301e-01_rb/)
      kbo(:, 3,25, 8) = (/ &
     &1.0186e+02_rb,7.6521e+01_rb,5.1191e+01_rb,2.6188e+01_rb,5.7925e-01_rb/)
      kbo(:, 4,25, 8) = (/ &
     &1.0808e+02_rb,8.1193e+01_rb,5.4315e+01_rb,2.7693e+01_rb,5.9615e-01_rb/)
      kbo(:, 5,25, 8) = (/ &
     &1.1475e+02_rb,8.6195e+01_rb,5.7652e+01_rb,2.9318e+01_rb,6.1289e-01_rb/)
      kbo(:, 1,26, 8) = (/ &
     &7.5309e+01_rb,5.6599e+01_rb,3.7963e+01_rb,1.9911e+01_rb,4.8832e-01_rb/)
      kbo(:, 2,26, 8) = (/ &
     &7.9402e+01_rb,5.9672e+01_rb,3.9995e+01_rb,2.0863e+01_rb,5.0531e-01_rb/)
      kbo(:, 3,26, 8) = (/ &
     &8.4365e+01_rb,6.3403e+01_rb,4.2472e+01_rb,2.2029e+01_rb,5.2357e-01_rb/)
      kbo(:, 4,26, 8) = (/ &
     &8.9751e+01_rb,6.7447e+01_rb,4.5169e+01_rb,2.3307e+01_rb,5.4165e-01_rb/)
      kbo(:, 5,26, 8) = (/ &
     &9.5432e+01_rb,7.1708e+01_rb,4.8005e+01_rb,2.4657e+01_rb,5.6018e-01_rb/)
      kbo(:, 1,27, 8) = (/ &
     &6.4036e+01_rb,4.8147e+01_rb,3.2405e+01_rb,1.7310e+01_rb,4.3584e-01_rb/)
      kbo(:, 2,27, 8) = (/ &
     &6.8098e+01_rb,5.1198e+01_rb,3.4406e+01_rb,1.8250e+01_rb,4.5420e-01_rb/)
      kbo(:, 3,27, 8) = (/ &
     &7.2623e+01_rb,5.4602e+01_rb,3.6656e+01_rb,1.9303e+01_rb,4.7326e-01_rb/)
      kbo(:, 4,27, 8) = (/ &
     &7.7443e+01_rb,5.8217e+01_rb,3.9050e+01_rb,2.0426e+01_rb,4.9239e-01_rb/)
      kbo(:, 5,27, 8) = (/ &
     &8.2321e+01_rb,6.1875e+01_rb,4.1476e+01_rb,2.1557e+01_rb,5.1192e-01_rb/)
      kbo(:, 1,28, 8) = (/ &
     &5.6234e+01_rb,4.2299e+01_rb,2.8597e+01_rb,1.5531e+01_rb,3.8958e-01_rb/)
      kbo(:, 2,28, 8) = (/ &
     &6.0088e+01_rb,4.5199e+01_rb,3.0486e+01_rb,1.6430e+01_rb,4.0865e-01_rb/)
      kbo(:, 3,28, 8) = (/ &
     &6.4206e+01_rb,4.8292e+01_rb,3.2515e+01_rb,1.7385e+01_rb,4.2800e-01_rb/)
      kbo(:, 4,28, 8) = (/ &
     &6.8496e+01_rb,5.1510e+01_rb,3.4630e+01_rb,1.8368e+01_rb,4.4785e-01_rb/)
      kbo(:, 5,28, 8) = (/ &
     &7.2933e+01_rb,5.4831e+01_rb,3.6814e+01_rb,1.9374e+01_rb,4.6830e-01_rb/)
      kbo(:, 1,29, 8) = (/ &
     &5.2522e+01_rb,3.9524e+01_rb,2.6825e+01_rb,1.4721e+01_rb,3.4924e-01_rb/)
      kbo(:, 2,29, 8) = (/ &
     &5.6330e+01_rb,4.2387e+01_rb,2.8683e+01_rb,1.5618e+01_rb,3.6822e-01_rb/)
      kbo(:, 3,29, 8) = (/ &
     &6.0271e+01_rb,4.5343e+01_rb,3.0606e+01_rb,1.6531e+01_rb,3.8784e-01_rb/)
      kbo(:, 4,29, 8) = (/ &
     &6.4389e+01_rb,4.8431e+01_rb,3.2623e+01_rb,1.7468e+01_rb,4.0814e-01_rb/)
      kbo(:, 5,29, 8) = (/ &
     &6.8669e+01_rb,5.1633e+01_rb,3.4713e+01_rb,1.8422e+01_rb,4.2953e-01_rb/)
      kbo(:, 1,30, 8) = (/ &
     &5.0335e+01_rb,3.7896e+01_rb,2.5810e+01_rb,1.4271e+01_rb,3.1347e-01_rb/)
      kbo(:, 2,30, 8) = (/ &
     &5.4100e+01_rb,4.0719e+01_rb,2.7633e+01_rb,1.5165e+01_rb,3.3234e-01_rb/)
      kbo(:, 3,30, 8) = (/ &
     &5.7975e+01_rb,4.3625e+01_rb,2.9511e+01_rb,1.6060e+01_rb,3.5217e-01_rb/)
      kbo(:, 4,30, 8) = (/ &
     &6.2057e+01_rb,4.6681e+01_rb,3.1490e+01_rb,1.6979e+01_rb,3.7324e-01_rb/)
      kbo(:, 5,30, 8) = (/ &
     &6.6429e+01_rb,4.9951e+01_rb,3.3616e+01_rb,1.7947e+01_rb,3.9520e-01_rb/)
      kbo(:, 1,31, 8) = (/ &
     &5.0736e+01_rb,3.8204e+01_rb,2.6057e+01_rb,1.4444e+01_rb,2.8199e-01_rb/)
      kbo(:, 2,31, 8) = (/ &
     &5.4607e+01_rb,4.1105e+01_rb,2.7922e+01_rb,1.5358e+01_rb,3.0094e-01_rb/)
      kbo(:, 3,31, 8) = (/ &
     &5.8681e+01_rb,4.4155e+01_rb,2.9887e+01_rb,1.6295e+01_rb,3.2130e-01_rb/)
      kbo(:, 4,31, 8) = (/ &
     &6.3068e+01_rb,4.7436e+01_rb,3.2008e+01_rb,1.7279e+01_rb,3.4300e-01_rb/)
      kbo(:, 5,31, 8) = (/ &
     &6.7873e+01_rb,5.1031e+01_rb,3.4344e+01_rb,1.8340e+01_rb,3.6582e-01_rb/)
      kbo(:, 1,32, 8) = (/ &
     &5.1816e+01_rb,3.9019e+01_rb,2.6627e+01_rb,1.4770e+01_rb,2.5449e-01_rb/)
      kbo(:, 2,32, 8) = (/ &
     &5.5914e+01_rb,4.2088e+01_rb,2.8597e+01_rb,1.5735e+01_rb,2.7379e-01_rb/)
      kbo(:, 3,32, 8) = (/ &
     &6.0292e+01_rb,4.5361e+01_rb,3.0700e+01_rb,1.6736e+01_rb,2.9480e-01_rb/)
      kbo(:, 4,32, 8) = (/ &
     &6.5110e+01_rb,4.8965e+01_rb,3.3030e+01_rb,1.7812e+01_rb,3.1717e-01_rb/)
      kbo(:, 5,32, 8) = (/ &
     &7.0477e+01_rb,5.2981e+01_rb,3.5640e+01_rb,1.8999e+01_rb,3.4101e-01_rb/)
      kbo(:, 1,33, 8) = (/ &
     &5.3927e+01_rb,4.0606e+01_rb,2.7695e+01_rb,1.5341e+01_rb,2.3093e-01_rb/)
      kbo(:, 2,33, 8) = (/ &
     &5.8395e+01_rb,4.3947e+01_rb,2.9836e+01_rb,1.6383e+01_rb,2.5073e-01_rb/)
      kbo(:, 3,33, 8) = (/ &
     &6.3254e+01_rb,4.7578e+01_rb,3.2172e+01_rb,1.7485e+01_rb,2.7229e-01_rb/)
      kbo(:, 4,33, 8) = (/ &
     &6.8732e+01_rb,5.1678e+01_rb,3.4827e+01_rb,1.8707e+01_rb,2.9562e-01_rb/)
      kbo(:, 5,33, 8) = (/ &
     &7.4993e+01_rb,5.6366e+01_rb,3.7885e+01_rb,2.0105e+01_rb,3.2058e-01_rb/)
      kbo(:, 1,34, 8) = (/ &
     &5.4725e+01_rb,4.1207e+01_rb,2.8111e+01_rb,1.5582e+01_rb,2.1030e-01_rb/)
      kbo(:, 2,34, 8) = (/ &
     &5.9482e+01_rb,4.4761e+01_rb,3.0384e+01_rb,1.6682e+01_rb,2.3046e-01_rb/)
      kbo(:, 3,34, 8) = (/ &
     &6.4816e+01_rb,4.8747e+01_rb,3.2949e+01_rb,1.7887e+01_rb,2.5288e-01_rb/)
      kbo(:, 4,34, 8) = (/ &
     &7.0970e+01_rb,5.3352e+01_rb,3.5935e+01_rb,1.9263e+01_rb,2.7720e-01_rb/)
      kbo(:, 5,34, 8) = (/ &
     &7.8000e+01_rb,5.8622e+01_rb,3.9384e+01_rb,2.0853e+01_rb,3.0314e-01_rb/)
      kbo(:, 1,35, 8) = (/ &
     &5.5341e+01_rb,4.1670e+01_rb,2.8432e+01_rb,1.5766e+01_rb,1.9105e-01_rb/)
      kbo(:, 2,35, 8) = (/ &
     &6.0475e+01_rb,4.5504e+01_rb,3.0887e+01_rb,1.6952e+01_rb,2.1167e-01_rb/)
      kbo(:, 3,35, 8) = (/ &
     &6.6351e+01_rb,4.9895e+01_rb,3.3711e+01_rb,1.8277e+01_rb,2.3461e-01_rb/)
      kbo(:, 4,35, 8) = (/ &
     &7.3222e+01_rb,5.5040e+01_rb,3.7057e+01_rb,1.9828e+01_rb,2.5964e-01_rb/)
      kbo(:, 5,35, 8) = (/ &
     &8.0936e+01_rb,6.0824e+01_rb,4.0848e+01_rb,2.1583e+01_rb,2.8629e-01_rb/)
      kbo(:, 1,36, 8) = (/ &
     &5.5260e+01_rb,4.1610e+01_rb,2.8415e+01_rb,1.5779e+01_rb,1.7255e-01_rb/)
      kbo(:, 2,36, 8) = (/ &
     &6.0687e+01_rb,4.5661e+01_rb,3.1003e+01_rb,1.7034e+01_rb,1.9343e-01_rb/)
      kbo(:, 3,36, 8) = (/ &
     &6.7095e+01_rb,5.0451e+01_rb,3.4089e+01_rb,1.8486e+01_rb,2.1666e-01_rb/)
      kbo(:, 4,36, 8) = (/ &
     &7.4593e+01_rb,5.6070e+01_rb,3.7750e+01_rb,2.0193e+01_rb,2.4210e-01_rb/)
      kbo(:, 5,36, 8) = (/ &
     &8.2912e+01_rb,6.2305e+01_rb,4.1836e+01_rb,2.2093e+01_rb,2.6939e-01_rb/)
      kbo(:, 1,37, 8) = (/ &
     &5.3136e+01_rb,4.0024e+01_rb,2.7403e+01_rb,1.5295e+01_rb,1.5374e-01_rb/)
      kbo(:, 2,37, 8) = (/ &
     &5.8700e+01_rb,4.4173e+01_rb,3.0048e+01_rb,1.6588e+01_rb,1.7430e-01_rb/)
      kbo(:, 3,37, 8) = (/ &
     &6.5370e+01_rb,4.9158e+01_rb,3.3257e+01_rb,1.8110e+01_rb,1.9744e-01_rb/)
      kbo(:, 4,37, 8) = (/ &
     &7.3079e+01_rb,5.4934e+01_rb,3.7017e+01_rb,1.9876e+01_rb,2.2299e-01_rb/)
      kbo(:, 5,37, 8) = (/ &
     &8.1729e+01_rb,6.1417e+01_rb,4.1258e+01_rb,2.1852e+01_rb,2.5074e-01_rb/)
      kbo(:, 1,38, 8) = (/ &
     &5.1889e+01_rb,3.9094e+01_rb,2.6819e+01_rb,1.5017e+01_rb,1.3711e-01_rb/)
      kbo(:, 2,38, 8) = (/ &
     &5.7665e+01_rb,4.3397e+01_rb,2.9561e+01_rb,1.6368e+01_rb,1.5734e-01_rb/)
      kbo(:, 3,38, 8) = (/ &
     &6.4668e+01_rb,4.8631e+01_rb,3.2933e+01_rb,1.7981e+01_rb,1.8028e-01_rb/)
      kbo(:, 4,38, 8) = (/ &
     &7.2785e+01_rb,5.4713e+01_rb,3.6887e+01_rb,1.9846e+01_rb,2.0601e-01_rb/)
      kbo(:, 5,38, 8) = (/ &
     &8.1846e+01_rb,6.1504e+01_rb,4.1328e+01_rb,2.1921e+01_rb,2.3415e-01_rb/)
      kbo(:, 1,39, 8) = (/ &
     &5.1946e+01_rb,3.9141e+01_rb,2.6874e+01_rb,1.5056e+01_rb,1.2254e-01_rb/)
      kbo(:, 2,39, 8) = (/ &
     &5.8115e+01_rb,4.3735e+01_rb,2.9803e+01_rb,1.6507e+01_rb,1.4241e-01_rb/)
      kbo(:, 3,39, 8) = (/ &
     &6.5638e+01_rb,4.9361e+01_rb,3.3432e+01_rb,1.8252e+01_rb,1.6535e-01_rb/)
      kbo(:, 4,39, 8) = (/ &
     &7.4281e+01_rb,5.5835e+01_rb,3.7644e+01_rb,2.0246e+01_rb,1.9114e-01_rb/)
      kbo(:, 5,39, 8) = (/ &
     &8.4117e+01_rb,6.3205e+01_rb,4.2465e+01_rb,2.2507e+01_rb,2.1954e-01_rb/)
      kbo(:, 1,40, 8) = (/ &
     &4.8372e+01_rb,3.6476e+01_rb,2.5167e+01_rb,1.4200e+01_rb,1.0786e-01_rb/)
      kbo(:, 2,40, 8) = (/ &
     &5.4435e+01_rb,4.0985e+01_rb,2.8035e+01_rb,1.5644e+01_rb,1.2713e-01_rb/)
      kbo(:, 3,40, 8) = (/ &
     &6.1814e+01_rb,4.6498e+01_rb,3.1582e+01_rb,1.7371e+01_rb,1.4957e-01_rb/)
      kbo(:, 4,40, 8) = (/ &
     &7.0455e+01_rb,5.2966e+01_rb,3.5777e+01_rb,1.9373e+01_rb,1.7507e-01_rb/)
      kbo(:, 5,40, 8) = (/ &
     &8.0314e+01_rb,6.0352e+01_rb,4.0598e+01_rb,2.1643e+01_rb,2.0339e-01_rb/)
      kbo(:, 1,41, 8) = (/ &
     &4.4863e+01_rb,3.3865e+01_rb,2.3494e+01_rb,1.3347e+01_rb,9.4622e-02_rb/)
      kbo(:, 2,41, 8) = (/ &
     &5.0838e+01_rb,3.8301e+01_rb,2.6314e+01_rb,1.4793e+01_rb,1.1320e-01_rb/)
      kbo(:, 3,41, 8) = (/ &
     &5.8061e+01_rb,4.3692e+01_rb,2.9774e+01_rb,1.6500e+01_rb,1.3504e-01_rb/)
      kbo(:, 4,41, 8) = (/ &
     &6.6603e+01_rb,5.0081e+01_rb,3.3908e+01_rb,1.8494e+01_rb,1.6019e-01_rb/)
      kbo(:, 5,41, 8) = (/ &
     &7.6448e+01_rb,5.7454e+01_rb,3.8706e+01_rb,2.0765e+01_rb,1.8828e-01_rb/)
      kbo(:, 1,42, 8) = (/ &
     &4.2143e+01_rb,3.1846e+01_rb,2.2204e+01_rb,1.2680e+01_rb,8.2928e-02_rb/)
      kbo(:, 2,42, 8) = (/ &
     &4.7973e+01_rb,3.6167e+01_rb,2.4953e+01_rb,1.4117e+01_rb,1.0072e-01_rb/)
      kbo(:, 3,42, 8) = (/ &
     &5.5183e+01_rb,4.1542e+01_rb,2.8395e+01_rb,1.5835e+01_rb,1.2195e-01_rb/)
      kbo(:, 4,42, 8) = (/ &
     &6.3709e+01_rb,4.7914e+01_rb,3.2510e+01_rb,1.7836e+01_rb,1.4663e-01_rb/)
      kbo(:, 5,42, 8) = (/ &
     &7.3621e+01_rb,5.5334e+01_rb,3.7330e+01_rb,2.0134e+01_rb,1.7447e-01_rb/)
      kbo(:, 1,43, 8) = (/ &
     &3.9594e+01_rb,2.9962e+01_rb,2.1003e+01_rb,1.2048e+01_rb,7.1693e-02_rb/)
      kbo(:, 2,43, 8) = (/ &
     &4.5330e+01_rb,3.4203e+01_rb,2.3706e+01_rb,1.3485e+01_rb,8.8502e-02_rb/)
      kbo(:, 3,43, 8) = (/ &
     &5.2493e+01_rb,3.9537e+01_rb,2.7118e+01_rb,1.5213e+01_rb,1.0887e-01_rb/)
      kbo(:, 4,43, 8) = (/ &
     &6.1012e+01_rb,4.5896e+01_rb,3.1221e+01_rb,1.7230e+01_rb,1.3283e-01_rb/)
      kbo(:, 5,43, 8) = (/ &
     &7.1011e+01_rb,5.3379e+01_rb,3.6070e+01_rb,1.9559e+01_rb,1.6011e-01_rb/)
      kbo(:, 1,44, 8) = (/ &
     &3.7616e+01_rb,2.8507e+01_rb,2.0078e+01_rb,1.1553e+01_rb,6.1401e-02_rb/)
      kbo(:, 2,44, 8) = (/ &
     &4.3274e+01_rb,3.2681e+01_rb,2.2747e+01_rb,1.2992e+01_rb,7.7099e-02_rb/)
      kbo(:, 3,44, 8) = (/ &
     &5.0385e+01_rb,3.7971e+01_rb,2.6133e+01_rb,1.4732e+01_rb,9.6479e-02_rb/)
      kbo(:, 4,44, 8) = (/ &
     &5.8976e+01_rb,4.4379e+01_rb,3.0259e+01_rb,1.6785e+01_rb,1.1952e-01_rb/)
      kbo(:, 5,44, 8) = (/ &
     &6.9172e+01_rb,5.2003e+01_rb,3.5195e+01_rb,1.9170e+01_rb,1.4606e-01_rb/)
      kbo(:, 1,45, 8) = (/ &
     &3.6417e+01_rb,2.7634e+01_rb,1.9534e+01_rb,1.1252e+01_rb,5.2391e-02_rb/)
      kbo(:, 2,45, 8) = (/ &
     &4.2083e+01_rb,3.1808e+01_rb,2.2209e+01_rb,1.2716e+01_rb,6.6909e-02_rb/)
      kbo(:, 3,45, 8) = (/ &
     &4.9318e+01_rb,3.7181e+01_rb,2.5653e+01_rb,1.4499e+01_rb,8.5223e-02_rb/)
      kbo(:, 4,45, 8) = (/ &
     &5.8109e+01_rb,4.3735e+01_rb,2.9873e+01_rb,1.6620e+01_rb,1.0731e-01_rb/)
      kbo(:, 5,45, 8) = (/ &
     &6.8675e+01_rb,5.1635e+01_rb,3.4984e+01_rb,1.9103e+01_rb,1.3302e-01_rb/)
      kbo(:, 1,46, 8) = (/ &
     &3.5397e+01_rb,2.6895e+01_rb,1.9070e+01_rb,1.0989e+01_rb,4.4228e-02_rb/)
      kbo(:, 2,46, 8) = (/ &
     &4.1150e+01_rb,3.1127e+01_rb,2.1795e+01_rb,1.2497e+01_rb,5.7487e-02_rb/)
      kbo(:, 3,46, 8) = (/ &
     &4.8494e+01_rb,3.6574e+01_rb,2.5287e+01_rb,1.4324e+01_rb,7.4577e-02_rb/)
      kbo(:, 4,46, 8) = (/ &
     &5.7598e+01_rb,4.3359e+01_rb,2.9657e+01_rb,1.6535e+01_rb,9.5575e-02_rb/)
      kbo(:, 5,46, 8) = (/ &
     &6.8559e+01_rb,5.1549e+01_rb,3.4961e+01_rb,1.9128e+01_rb,1.2028e-01_rb/)
      kbo(:, 1,47, 8) = (/ &
     &3.3892e+01_rb,2.5799e+01_rb,1.8369e+01_rb,1.0592e+01_rb,3.6658e-02_rb/)
      kbo(:, 2,47, 8) = (/ &
     &3.9558e+01_rb,2.9961e+01_rb,2.1065e+01_rb,1.2104e+01_rb,4.8512e-02_rb/)
      kbo(:, 3,47, 8) = (/ &
     &4.6821e+01_rb,3.5338e+01_rb,2.4514e+01_rb,1.3933e+01_rb,6.4199e-02_rb/)
      kbo(:, 4,47, 8) = (/ &
     &5.6038e+01_rb,4.2200e+01_rb,2.8934e+01_rb,1.6192e+01_rb,8.3849e-02_rb/)
      kbo(:, 5,47, 8) = (/ &
     &6.7239e+01_rb,5.0568e+01_rb,3.4350e+01_rb,1.8859e+01_rb,1.0731e-01_rb/)
      kbo(:, 1,48, 8) = (/ &
     &3.3363e+01_rb,2.5430e+01_rb,1.8144e+01_rb,1.0446e+01_rb,3.0177e-02_rb/)
      kbo(:, 2,48, 8) = (/ &
     &3.9055e+01_rb,2.9607e+01_rb,2.0861e+01_rb,1.1982e+01_rb,4.0640e-02_rb/)
      kbo(:, 3,48, 8) = (/ &
     &4.6430e+01_rb,3.5057e+01_rb,2.4367e+01_rb,1.3859e+01_rb,5.4873e-02_rb/)
      kbo(:, 4,48, 8) = (/ &
     &5.5939e+01_rb,4.2134e+01_rb,2.8926e+01_rb,1.6205e+01_rb,7.3130e-02_rb/)
      kbo(:, 5,48, 8) = (/ &
     &6.7675e+01_rb,5.0896e+01_rb,3.4598e+01_rb,1.9010e+01_rb,9.5352e-02_rb/)
      kbo(:, 1,49, 8) = (/ &
     &3.4087e+01_rb,2.5989e+01_rb,1.8530e+01_rb,1.0628e+01_rb,2.4642e-02_rb/)
      kbo(:, 2,49, 8) = (/ &
     &4.0143e+01_rb,3.0436e+01_rb,2.1433e+01_rb,1.2264e+01_rb,3.3771e-02_rb/)
      kbo(:, 3,49, 8) = (/ &
     &4.7859e+01_rb,3.6137e+01_rb,2.5106e+01_rb,1.4243e+01_rb,4.6570e-02_rb/)
      kbo(:, 4,49, 8) = (/ &
     &5.8042e+01_rb,4.3715e+01_rb,2.9997e+01_rb,1.6759e+01_rb,6.3406e-02_rb/)
      kbo(:, 5,49, 8) = (/ &
     &7.0774e+01_rb,5.3220e+01_rb,3.6156e+01_rb,1.9813e+01_rb,8.4317e-02_rb/)
      kbo(:, 1,50, 8) = (/ &
     &3.3271e+01_rb,2.5407e+01_rb,1.8155e+01_rb,1.0394e+01_rb,2.0105e-02_rb/)
      kbo(:, 2,50, 8) = (/ &
     &3.9337e+01_rb,2.9859e+01_rb,2.1080e+01_rb,1.2062e+01_rb,2.8045e-02_rb/)
      kbo(:, 3,50, 8) = (/ &
     &4.7124e+01_rb,3.5606e+01_rb,2.4793e+01_rb,1.4082e+01_rb,3.9522e-02_rb/)
      kbo(:, 4,50, 8) = (/ &
     &5.7525e+01_rb,4.3340e+01_rb,2.9788e+01_rb,1.6668e+01_rb,5.4974e-02_rb/)
      kbo(:, 5,50, 8) = (/ &
     &7.0694e+01_rb,5.3165e+01_rb,3.6155e+01_rb,1.9842e+01_rb,7.4569e-02_rb/)
      kbo(:, 1,51, 8) = (/ &
     &3.1799e+01_rb,2.4339e+01_rb,1.7448e+01_rb,9.9788e+00_rb,1.6322e-02_rb/)
      kbo(:, 2,51, 8) = (/ &
     &3.7729e+01_rb,2.8686e+01_rb,2.0331e+01_rb,1.1646e+01_rb,2.3187e-02_rb/)
      kbo(:, 3,51, 8) = (/ &
     &4.5428e+01_rb,3.4361e+01_rb,2.4014e+01_rb,1.3675e+01_rb,3.3395e-02_rb/)
      kbo(:, 4,51, 8) = (/ &
     &5.5764e+01_rb,4.2036e+01_rb,2.8971e+01_rb,1.6259e+01_rb,4.7474e-02_rb/)
      kbo(:, 5,51, 8) = (/ &
     &6.9086e+01_rb,5.1969e+01_rb,3.5405e+01_rb,1.9496e+01_rb,6.5720e-02_rb/)
      kbo(:, 1,52, 8) = (/ &
     &3.1125e+01_rb,2.3861e+01_rb,1.7130e+01_rb,9.7762e+00_rb,1.3165e-02_rb/)
      kbo(:, 2,52, 8) = (/ &
     &3.7206e+01_rb,2.8320e+01_rb,2.0110e+01_rb,1.1507e+01_rb,1.9020e-02_rb/)
      kbo(:, 3,52, 8) = (/ &
     &4.4966e+01_rb,3.4037e+01_rb,2.3832e+01_rb,1.3576e+01_rb,2.8007e-02_rb/)
      kbo(:, 4,52, 8) = (/ &
     &5.5475e+01_rb,4.1834e+01_rb,2.8875e+01_rb,1.6222e+01_rb,4.0752e-02_rb/)
      kbo(:, 5,52, 8) = (/ &
     &6.9286e+01_rb,5.2127e+01_rb,3.5547e+01_rb,1.9593e+01_rb,5.7616e-02_rb/)
      kbo(:, 1,53, 8) = (/ &
     &3.1628e+01_rb,2.4253e+01_rb,1.7391e+01_rb,9.8844e+00_rb,1.0543e-02_rb/)
      kbo(:, 2,53, 8) = (/ &
     &3.8058e+01_rb,2.8975e+01_rb,2.0560e+01_rb,1.1731e+01_rb,1.5472e-02_rb/)
      kbo(:, 3,53, 8) = (/ &
     &4.6305e+01_rb,3.5054e+01_rb,2.4529e+01_rb,1.3933e+01_rb,2.3276e-02_rb/)
      kbo(:, 4,53, 8) = (/ &
     &5.7328e+01_rb,4.3229e+01_rb,2.9824e+01_rb,1.6717e+01_rb,3.4695e-02_rb/)
      kbo(:, 5,53, 8) = (/ &
     &7.2228e+01_rb,5.4335e+01_rb,3.7034e+01_rb,2.0360e+01_rb,5.0210e-02_rb/)
      kbo(:, 1,54, 8) = (/ &
     &2.7866e+01_rb,2.1495e+01_rb,1.5519e+01_rb,8.8346e+00_rb,8.4580e-03_rb/)
      kbo(:, 2,54, 8) = (/ &
     &3.3763e+01_rb,2.5815e+01_rb,1.8473e+01_rb,1.0588e+01_rb,1.2602e-02_rb/)
      kbo(:, 3,54, 8) = (/ &
     &4.1361e+01_rb,3.1401e+01_rb,2.2155e+01_rb,1.2668e+01_rb,1.9366e-02_rb/)
      kbo(:, 4,54, 8) = (/ &
     &5.1507e+01_rb,3.8906e+01_rb,2.7027e+01_rb,1.5284e+01_rb,2.9589e-02_rb/)
      kbo(:, 5,54, 8) = (/ &
     &6.5378e+01_rb,4.9225e+01_rb,3.3718e+01_rb,1.8724e+01_rb,4.3797e-02_rb/)
      kbo(:, 1,55, 8) = (/ &
     &2.2404e+01_rb,1.7484e+01_rb,1.2754e+01_rb,7.2917e+00_rb,6.7620e-03_rb/)
      kbo(:, 2,55, 8) = (/ &
     &2.7309e+01_rb,2.1069e+01_rb,1.5293e+01_rb,8.8299e+00_rb,1.0220e-02_rb/)
      kbo(:, 3,55, 8) = (/ &
     &3.3683e+01_rb,2.5733e+01_rb,1.8431e+01_rb,1.0660e+01_rb,1.6054e-02_rb/)
      kbo(:, 4,55, 8) = (/ &
     &4.2194e+01_rb,3.1999e+01_rb,2.2529e+01_rb,1.2931e+01_rb,2.5150e-02_rb/)
      kbo(:, 5,55, 8) = (/ &
     &5.4011e+01_rb,4.0754e+01_rb,2.8208e+01_rb,1.5935e+01_rb,3.8077e-02_rb/)
      kbo(:, 1,56, 8) = (/ &
     &1.7963e+01_rb,1.4222e+01_rb,1.0460e+01_rb,5.9968e+00_rb,5.3696e-03_rb/)
      kbo(:, 2,56, 8) = (/ &
     &2.2011e+01_rb,1.7185e+01_rb,1.2632e+01_rb,7.3359e+00_rb,8.2238e-03_rb/)
      kbo(:, 3,56, 8) = (/ &
     &2.7341e+01_rb,2.1070e+01_rb,1.5329e+01_rb,8.9492e+00_rb,1.3200e-02_rb/)
      kbo(:, 4,56, 8) = (/ &
     &3.4490e+01_rb,2.6306e+01_rb,1.8809e+01_rb,1.0943e+01_rb,2.1214e-02_rb/)
      kbo(:, 5,56, 8) = (/ &
     &4.4535e+01_rb,3.3715e+01_rb,2.3630e+01_rb,1.3559e+01_rb,3.2915e-02_rb/)
      kbo(:, 1,57, 8) = (/ &
     &1.4368e+01_rb,1.1564e+01_rb,8.5550e+00_rb,4.9118e+00_rb,4.2402e-03_rb/)
      kbo(:, 2,57, 8) = (/ &
     &1.7718e+01_rb,1.4043e+01_rb,1.0433e+01_rb,6.0838e+00_rb,6.5672e-03_rb/)
      kbo(:, 3,57, 8) = (/ &
     &2.2119e+01_rb,1.7250e+01_rb,1.2736e+01_rb,7.4914e+00_rb,1.0750e-02_rb/)
      kbo(:, 4,57, 8) = (/ &
     &2.8111e+01_rb,2.1614e+01_rb,1.5710e+01_rb,9.2466e+00_rb,1.7729e-02_rb/)
      kbo(:, 5,57, 8) = (/ &
     &3.6581e+01_rb,2.7832e+01_rb,1.9800e+01_rb,1.1525e+01_rb,2.8247e-02_rb/)
      kbo(:, 1,58, 8) = (/ &
     &3.0334e+00_rb,3.0458e+00_rb,2.9118e+00_rb,2.3449e+00_rb,3.3480e-03_rb/)
      kbo(:, 2,58, 8) = (/ &
     &3.7595e+00_rb,3.7165e+00_rb,3.5857e+00_rb,2.9410e+00_rb,5.2345e-03_rb/)
      kbo(:, 3,58, 8) = (/ &
     &4.7309e+00_rb,4.5866e+00_rb,4.4204e+00_rb,3.6685e+00_rb,8.7466e-03_rb/)
      kbo(:, 4,58, 8) = (/ &
     &6.0543e+00_rb,5.7620e+00_rb,5.4869e+00_rb,4.5723e+00_rb,1.4810e-02_rb/)
      kbo(:, 5,58, 8) = (/ &
     &7.9336e+00_rb,7.4372e+00_rb,6.9325e+00_rb,5.7367e+00_rb,2.4215e-02_rb/)
      kbo(:, 1,59, 8) = (/ &
     &2.9765e+00_rb,2.9765e+00_rb,2.6845e+00_rb,1.9427e+00_rb,2.8320e-03_rb/)
      kbo(:, 2,59, 8) = (/ &
     &3.7193e+00_rb,3.6697e+00_rb,3.3576e+00_rb,2.4792e+00_rb,4.5261e-03_rb/)
      kbo(:, 3,59, 8) = (/ &
     &4.7245e+00_rb,4.5542e+00_rb,4.1916e+00_rb,3.1428e+00_rb,7.7669e-03_rb/)
      kbo(:, 4,59, 8) = (/ &
     &6.1120e+00_rb,5.7342e+00_rb,5.2520e+00_rb,3.9688e+00_rb,1.3461e-02_rb/)
      kbo(:, 5,59, 8) = (/ &
     &8.1086e+00_rb,7.4151e+00_rb,6.6769e+00_rb,5.0327e+00_rb,2.2363e-02_rb/)
      kbo(:, 1,13, 9) = (/ &
     &3.9255e+04_rb,2.9441e+04_rb,1.9628e+04_rb,9.8141e+03_rb,2.1650e+00_rb/)
      kbo(:, 2,13, 9) = (/ &
     &3.9490e+04_rb,2.9618e+04_rb,1.9745e+04_rb,9.8730e+03_rb,2.1321e+00_rb/)
      kbo(:, 3,13, 9) = (/ &
     &3.9631e+04_rb,2.9723e+04_rb,1.9816e+04_rb,9.9081e+03_rb,2.0803e+00_rb/)
      kbo(:, 4,13, 9) = (/ &
     &3.9707e+04_rb,2.9781e+04_rb,1.9854e+04_rb,9.9272e+03_rb,2.0587e+00_rb/)
      kbo(:, 5,13, 9) = (/ &
     &3.9690e+04_rb,2.9768e+04_rb,1.9845e+04_rb,9.9229e+03_rb,2.0351e+00_rb/)
      kbo(:, 1,14, 9) = (/ &
     &2.2391e+04_rb,1.6793e+04_rb,1.1196e+04_rb,5.5981e+03_rb,2.2958e+00_rb/)
      kbo(:, 2,14, 9) = (/ &
     &2.2483e+04_rb,1.6862e+04_rb,1.1241e+04_rb,5.6209e+03_rb,2.2646e+00_rb/)
      kbo(:, 3,14, 9) = (/ &
     &2.2517e+04_rb,1.6888e+04_rb,1.1259e+04_rb,5.6296e+03_rb,2.2164e+00_rb/)
      kbo(:, 4,14, 9) = (/ &
     &2.2511e+04_rb,1.6883e+04_rb,1.1256e+04_rb,5.6281e+03_rb,2.1692e+00_rb/)
      kbo(:, 5,14, 9) = (/ &
     &2.2470e+04_rb,1.6853e+04_rb,1.1235e+04_rb,5.6178e+03_rb,2.1488e+00_rb/)
      kbo(:, 1,15, 9) = (/ &
     &1.2806e+04_rb,9.6046e+03_rb,6.4033e+03_rb,3.2019e+03_rb,2.4251e+00_rb/)
      kbo(:, 2,15, 9) = (/ &
     &1.2828e+04_rb,9.6209e+03_rb,6.4142e+03_rb,3.2073e+03_rb,2.3960e+00_rb/)
      kbo(:, 3,15, 9) = (/ &
     &1.2826e+04_rb,9.6195e+03_rb,6.4132e+03_rb,3.2069e+03_rb,2.3588e+00_rb/)
      kbo(:, 4,15, 9) = (/ &
     &1.2818e+04_rb,9.6139e+03_rb,6.4095e+03_rb,3.2050e+03_rb,2.3106e+00_rb/)
      kbo(:, 5,15, 9) = (/ &
     &1.2808e+04_rb,9.6063e+03_rb,6.4044e+03_rb,3.2025e+03_rb,2.2722e+00_rb/)
      kbo(:, 1,16, 9) = (/ &
     &8.5967e+03_rb,6.4476e+03_rb,4.2986e+03_rb,2.1496e+03_rb,2.5476e+00_rb/)
      kbo(:, 2,16, 9) = (/ &
     &8.5987e+03_rb,6.4492e+03_rb,4.2996e+03_rb,2.1501e+03_rb,2.5204e+00_rb/)
      kbo(:, 3,16, 9) = (/ &
     &8.5999e+03_rb,6.4500e+03_rb,4.3002e+03_rb,2.1504e+03_rb,2.4878e+00_rb/)
      kbo(:, 4,16, 9) = (/ &
     &8.6043e+03_rb,6.4534e+03_rb,4.3024e+03_rb,2.1515e+03_rb,2.4397e+00_rb/)
      kbo(:, 5,16, 9) = (/ &
     &8.6176e+03_rb,6.4634e+03_rb,4.3091e+03_rb,2.1548e+03_rb,2.4008e+00_rb/)
      kbo(:, 1,17, 9) = (/ &
     &5.9284e+03_rb,4.4464e+03_rb,2.9645e+03_rb,1.4825e+03_rb,2.6544e+00_rb/)
      kbo(:, 2,17, 9) = (/ &
     &5.9327e+03_rb,4.4497e+03_rb,2.9666e+03_rb,1.4836e+03_rb,2.6300e+00_rb/)
      kbo(:, 3,17, 9) = (/ &
     &5.9411e+03_rb,4.4560e+03_rb,2.9708e+03_rb,1.4857e+03_rb,2.5993e+00_rb/)
      kbo(:, 4,17, 9) = (/ &
     &5.9634e+03_rb,4.4727e+03_rb,2.9820e+03_rb,1.4912e+03_rb,2.5553e+00_rb/)
      kbo(:, 5,17, 9) = (/ &
     &6.0032e+03_rb,4.5025e+03_rb,3.0018e+03_rb,1.5012e+03_rb,2.5167e+00_rb/)
      kbo(:, 1,18, 9) = (/ &
     &4.1696e+03_rb,3.1274e+03_rb,2.0851e+03_rb,1.0428e+03_rb,2.7386e+00_rb/)
      kbo(:, 2,18, 9) = (/ &
     &4.1796e+03_rb,3.1348e+03_rb,2.0901e+03_rb,1.0453e+03_rb,2.7184e+00_rb/)
      kbo(:, 3,18, 9) = (/ &
     &4.2028e+03_rb,3.1522e+03_rb,2.1017e+03_rb,1.0511e+03_rb,2.6888e+00_rb/)
      kbo(:, 4,18, 9) = (/ &
     &4.2429e+03_rb,3.1823e+03_rb,2.1217e+03_rb,1.0611e+03_rb,2.6491e+00_rb/)
      kbo(:, 5,18, 9) = (/ &
     &4.2973e+03_rb,3.2231e+03_rb,2.1489e+03_rb,1.0747e+03_rb,2.6124e+00_rb/)
      kbo(:, 1,19, 9) = (/ &
     &2.8753e+03_rb,2.1566e+03_rb,1.4379e+03_rb,7.1922e+02_rb,2.7969e+00_rb/)
      kbo(:, 2,19, 9) = (/ &
     &2.8953e+03_rb,2.1716e+03_rb,1.4479e+03_rb,7.2422e+02_rb,2.7817e+00_rb/)
      kbo(:, 3,19, 9) = (/ &
     &2.9302e+03_rb,2.1978e+03_rb,1.4654e+03_rb,7.3295e+02_rb,2.7548e+00_rb/)
      kbo(:, 4,19, 9) = (/ &
     &2.9782e+03_rb,2.2338e+03_rb,1.4894e+03_rb,7.4494e+02_rb,2.7189e+00_rb/)
      kbo(:, 5,19, 9) = (/ &
     &3.0360e+03_rb,2.2771e+03_rb,1.5183e+03_rb,7.5939e+02_rb,2.6829e+00_rb/)
      kbo(:, 1,20, 9) = (/ &
     &2.1517e+03_rb,1.6139e+03_rb,1.0761e+03_rb,5.3833e+02_rb,2.8251e+00_rb/)
      kbo(:, 2,20, 9) = (/ &
     &2.1814e+03_rb,1.6362e+03_rb,1.0910e+03_rb,5.4575e+02_rb,2.8122e+00_rb/)
      kbo(:, 3,20, 9) = (/ &
     &2.2241e+03_rb,1.6682e+03_rb,1.1123e+03_rb,5.5643e+02_rb,2.7893e+00_rb/)
      kbo(:, 4,20, 9) = (/ &
     &2.2758e+03_rb,1.7069e+03_rb,1.1381e+03_rb,5.6933e+02_rb,2.7572e+00_rb/)
      kbo(:, 5,20, 9) = (/ &
     &2.3338e+03_rb,1.7505e+03_rb,1.1672e+03_rb,5.8384e+02_rb,2.7245e+00_rb/)
      kbo(:, 1,21, 9) = (/ &
     &1.6700e+03_rb,1.2526e+03_rb,8.3526e+02_rb,4.1790e+02_rb,2.8188e+00_rb/)
      kbo(:, 2,21, 9) = (/ &
     &1.7058e+03_rb,1.2795e+03_rb,8.5318e+02_rb,4.2686e+02_rb,2.8087e+00_rb/)
      kbo(:, 3,21, 9) = (/ &
     &1.7510e+03_rb,1.3134e+03_rb,8.7578e+02_rb,4.3816e+02_rb,2.7899e+00_rb/)
      kbo(:, 4,21, 9) = (/ &
     &1.8029e+03_rb,1.3523e+03_rb,9.0173e+02_rb,4.5113e+02_rb,2.7632e+00_rb/)
      kbo(:, 5,21, 9) = (/ &
     &1.8601e+03_rb,1.3952e+03_rb,9.3032e+02_rb,4.6542e+02_rb,2.7328e+00_rb/)
      kbo(:, 1,22, 9) = (/ &
     &1.3403e+03_rb,1.0054e+03_rb,6.7043e+02_rb,3.3549e+02_rb,2.7778e+00_rb/)
      kbo(:, 2,22, 9) = (/ &
     &1.3792e+03_rb,1.0345e+03_rb,6.8985e+02_rb,3.4520e+02_rb,2.7715e+00_rb/)
      kbo(:, 3,22, 9) = (/ &
     &1.4248e+03_rb,1.0688e+03_rb,7.1269e+02_rb,3.5661e+02_rb,2.7559e+00_rb/)
      kbo(:, 4,22, 9) = (/ &
     &1.4749e+03_rb,1.1063e+03_rb,7.3770e+02_rb,3.6911e+02_rb,2.7316e+00_rb/)
      kbo(:, 5,22, 9) = (/ &
     &1.5223e+03_rb,1.1418e+03_rb,7.6138e+02_rb,3.8094e+02_rb,2.7088e+00_rb/)
      kbo(:, 1,23, 9) = (/ &
     &1.0891e+03_rb,8.1698e+02_rb,5.4484e+02_rb,2.7270e+02_rb,2.7041e+00_rb/)
      kbo(:, 2,23, 9) = (/ &
     &1.1283e+03_rb,8.4637e+02_rb,5.6443e+02_rb,2.8249e+02_rb,2.7027e+00_rb/)
      kbo(:, 3,23, 9) = (/ &
     &1.1704e+03_rb,8.7791e+02_rb,5.8545e+02_rb,2.9300e+02_rb,2.6907e+00_rb/)
      kbo(:, 4,23, 9) = (/ &
     &1.2113e+03_rb,9.0862e+02_rb,6.0592e+02_rb,3.0322e+02_rb,2.6730e+00_rb/)
      kbo(:, 5,23, 9) = (/ &
     &1.2507e+03_rb,9.3816e+02_rb,6.2560e+02_rb,3.1305e+02_rb,2.6594e+00_rb/)
      kbo(:, 1,24, 9) = (/ &
     &8.8581e+02_rb,6.6450e+02_rb,4.4318e+02_rb,2.2188e+02_rb,2.6023e+00_rb/)
      kbo(:, 2,24, 9) = (/ &
     &9.2040e+02_rb,6.9044e+02_rb,4.6047e+02_rb,2.3052e+02_rb,2.6060e+00_rb/)
      kbo(:, 3,24, 9) = (/ &
     &9.5496e+02_rb,7.1635e+02_rb,4.7775e+02_rb,2.3914e+02_rb,2.6005e+00_rb/)
      kbo(:, 4,24, 9) = (/ &
     &9.8776e+02_rb,7.4094e+02_rb,4.9413e+02_rb,2.4733e+02_rb,2.5922e+00_rb/)
      kbo(:, 5,24, 9) = (/ &
     &1.0196e+03_rb,7.6479e+02_rb,5.1001e+02_rb,2.5525e+02_rb,2.5914e+00_rb/)
      kbo(:, 1,25, 9) = (/ &
     &7.3035e+02_rb,5.4790e+02_rb,3.6545e+02_rb,1.8302e+02_rb,2.4806e+00_rb/)
      kbo(:, 2,25, 9) = (/ &
     &7.5932e+02_rb,5.6963e+02_rb,3.7993e+02_rb,1.9025e+02_rb,2.4895e+00_rb/)
      kbo(:, 3,25, 9) = (/ &
     &7.8665e+02_rb,5.9012e+02_rb,3.9359e+02_rb,1.9707e+02_rb,2.4921e+00_rb/)
      kbo(:, 4,25, 9) = (/ &
     &8.1328e+02_rb,6.1007e+02_rb,4.0688e+02_rb,2.0370e+02_rb,2.4969e+00_rb/)
      kbo(:, 5,25, 9) = (/ &
     &8.3956e+02_rb,6.2979e+02_rb,4.2001e+02_rb,2.1025e+02_rb,2.5085e+00_rb/)
      kbo(:, 1,26, 9) = (/ &
     &6.1100e+02_rb,4.5838e+02_rb,3.0577e+02_rb,1.5319e+02_rb,2.3442e+00_rb/)
      kbo(:, 2,26, 9) = (/ &
     &6.3406e+02_rb,4.7568e+02_rb,3.1730e+02_rb,1.5893e+02_rb,2.3618e+00_rb/)
      kbo(:, 3,26, 9) = (/ &
     &6.5658e+02_rb,4.9255e+02_rb,3.2854e+02_rb,1.6455e+02_rb,2.3770e+00_rb/)
      kbo(:, 4,26, 9) = (/ &
     &6.7915e+02_rb,5.0948e+02_rb,3.3981e+02_rb,1.7016e+02_rb,2.3964e+00_rb/)
      kbo(:, 5,26, 9) = (/ &
     &7.0247e+02_rb,5.2696e+02_rb,3.5146e+02_rb,1.7598e+02_rb,2.4222e+00_rb/)
      kbo(:, 1,27, 9) = (/ &
     &5.2844e+02_rb,3.9646e+02_rb,2.6449e+02_rb,1.3259e+02_rb,2.2032e+00_rb/)
      kbo(:, 2,27, 9) = (/ &
     &5.4789e+02_rb,4.1105e+02_rb,2.7421e+02_rb,1.3742e+02_rb,2.2331e+00_rb/)
      kbo(:, 3,27, 9) = (/ &
     &5.6778e+02_rb,4.2595e+02_rb,2.8414e+02_rb,1.4236e+02_rb,2.2623e+00_rb/)
      kbo(:, 4,27, 9) = (/ &
     &5.8850e+02_rb,4.4149e+02_rb,2.9449e+02_rb,1.4751e+02_rb,2.3009e+00_rb/)
      kbo(:, 5,27, 9) = (/ &
     &6.1090e+02_rb,4.5829e+02_rb,3.0567e+02_rb,1.5309e+02_rb,2.3382e+00_rb/)
      kbo(:, 1,28, 9) = (/ &
     &4.6826e+02_rb,3.5133e+02_rb,2.3440e+02_rb,1.1763e+02_rb,2.0680e+00_rb/)
      kbo(:, 2,28, 9) = (/ &
     &4.8590e+02_rb,3.6454e+02_rb,2.4321e+02_rb,1.2197e+02_rb,2.1100e+00_rb/)
      kbo(:, 3,28, 9) = (/ &
     &5.0445e+02_rb,3.7846e+02_rb,2.5247e+02_rb,1.2655e+02_rb,2.1567e+00_rb/)
      kbo(:, 4,28, 9) = (/ &
     &5.2474e+02_rb,3.9367e+02_rb,2.6260e+02_rb,1.3159e+02_rb,2.2098e+00_rb/)
      kbo(:, 5,28, 9) = (/ &
     &5.4720e+02_rb,4.1051e+02_rb,2.7382e+02_rb,1.3717e+02_rb,2.2620e+00_rb/)
      kbo(:, 1,29, 9) = (/ &
     &4.4072e+02_rb,3.3067e+02_rb,2.2063e+02_rb,1.1082e+02_rb,1.9419e+00_rb/)
      kbo(:, 2,29, 9) = (/ &
     &4.5821e+02_rb,3.4378e+02_rb,2.2936e+02_rb,1.1511e+02_rb,1.9998e+00_rb/)
      kbo(:, 3,29, 9) = (/ &
     &4.7729e+02_rb,3.5808e+02_rb,2.3889e+02_rb,1.1980e+02_rb,2.0671e+00_rb/)
      kbo(:, 4,29, 9) = (/ &
     &4.9882e+02_rb,3.7422e+02_rb,2.4964e+02_rb,1.2512e+02_rb,2.1341e+00_rb/)
      kbo(:, 5,29, 9) = (/ &
     &5.2331e+02_rb,3.9259e+02_rb,2.6188e+02_rb,1.3121e+02_rb,2.1979e+00_rb/)
      kbo(:, 1,30, 9) = (/ &
     &4.2556e+02_rb,3.1930e+02_rb,2.1305e+02_rb,1.0711e+02_rb,1.8309e+00_rb/)
      kbo(:, 2,30, 9) = (/ &
     &4.4407e+02_rb,3.3317e+02_rb,2.2229e+02_rb,1.1161e+02_rb,1.9101e+00_rb/)
      kbo(:, 3,30, 9) = (/ &
     &4.6505e+02_rb,3.4890e+02_rb,2.3276e+02_rb,1.1676e+02_rb,1.9914e+00_rb/)
      kbo(:, 4,30, 9) = (/ &
     &4.8908e+02_rb,3.6692e+02_rb,2.4477e+02_rb,1.2270e+02_rb,2.0709e+00_rb/)
      kbo(:, 5,30, 9) = (/ &
     &5.1660e+02_rb,3.8756e+02_rb,2.5853e+02_rb,1.2954e+02_rb,2.1480e+00_rb/)
      kbo(:, 1,31, 9) = (/ &
     &4.3357e+02_rb,3.2530e+02_rb,2.1704e+02_rb,1.0913e+02_rb,1.7458e+00_rb/)
      kbo(:, 2,31, 9) = (/ &
     &4.5478e+02_rb,3.4120e+02_rb,2.2764e+02_rb,1.1430e+02_rb,1.8393e+00_rb/)
      kbo(:, 3,31, 9) = (/ &
     &4.7940e+02_rb,3.5966e+02_rb,2.3993e+02_rb,1.2035e+02_rb,1.9333e+00_rb/)
      kbo(:, 4,31, 9) = (/ &
     &5.0804e+02_rb,3.8115e+02_rb,2.5425e+02_rb,1.2744e+02_rb,2.0234e+00_rb/)
      kbo(:, 5,31, 9) = (/ &
     &5.4033e+02_rb,4.0536e+02_rb,2.7039e+02_rb,1.3548e+02_rb,2.1150e+00_rb/)
      kbo(:, 1,32, 9) = (/ &
     &4.4897e+02_rb,3.3685e+02_rb,2.2474e+02_rb,1.1298e+02_rb,1.6787e+00_rb/)
      kbo(:, 2,32, 9) = (/ &
     &4.7419e+02_rb,3.5576e+02_rb,2.3734e+02_rb,1.1915e+02_rb,1.7859e+00_rb/)
      kbo(:, 3,32, 9) = (/ &
     &5.0379e+02_rb,3.7796e+02_rb,2.5213e+02_rb,1.2644e+02_rb,1.8899e+00_rb/)
      kbo(:, 4,32, 9) = (/ &
     &5.3795e+02_rb,4.0357e+02_rb,2.6921e+02_rb,1.3492e+02_rb,1.9968e+00_rb/)
      kbo(:, 5,32, 9) = (/ &
     &5.7556e+02_rb,4.3179e+02_rb,2.8802e+02_rb,1.4429e+02_rb,2.0977e+00_rb/)
      kbo(:, 1,33, 9) = (/ &
     &4.7613e+02_rb,3.5721e+02_rb,2.3831e+02_rb,1.1974e+02_rb,1.6313e+00_rb/)
      kbo(:, 2,33, 9) = (/ &
     &5.0727e+02_rb,3.8056e+02_rb,2.5387e+02_rb,1.2739e+02_rb,1.7486e+00_rb/)
      kbo(:, 3,33, 9) = (/ &
     &5.4363e+02_rb,4.0783e+02_rb,2.7205e+02_rb,1.3639e+02_rb,1.8695e+00_rb/)
      kbo(:, 4,33, 9) = (/ &
     &5.8421e+02_rb,4.3828e+02_rb,2.9234e+02_rb,1.4648e+02_rb,1.9835e+00_rb/)
      kbo(:, 5,33, 9) = (/ &
     &6.2742e+02_rb,4.7068e+02_rb,3.1394e+02_rb,1.5725e+02_rb,2.0938e+00_rb/)
      kbo(:, 1,34, 9) = (/ &
     &4.9479e+02_rb,3.7120e+02_rb,2.4764e+02_rb,1.2439e+02_rb,1.5951e+00_rb/)
      kbo(:, 2,34, 9) = (/ &
     &5.3209e+02_rb,3.9918e+02_rb,2.6628e+02_rb,1.3359e+02_rb,1.7258e+00_rb/)
      kbo(:, 3,34, 9) = (/ &
     &5.7463e+02_rb,4.3109e+02_rb,2.8755e+02_rb,1.4413e+02_rb,1.8546e+00_rb/)
      kbo(:, 4,34, 9) = (/ &
     &6.2052e+02_rb,4.6551e+02_rb,3.1050e+02_rb,1.5556e+02_rb,1.9770e+00_rb/)
      kbo(:, 5,34, 9) = (/ &
     &6.6892e+02_rb,5.0180e+02_rb,3.3469e+02_rb,1.6762e+02_rb,2.0917e+00_rb/)
      kbo(:, 1,35, 9) = (/ &
     &5.1474e+02_rb,3.8617e+02_rb,2.5762e+02_rb,1.2939e+02_rb,1.5620e+00_rb/)
      kbo(:, 2,35, 9) = (/ &
     &5.5847e+02_rb,4.1897e+02_rb,2.7948e+02_rb,1.4019e+02_rb,1.7022e+00_rb/)
      kbo(:, 3,35, 9) = (/ &
     &6.0709e+02_rb,4.5544e+02_rb,3.0379e+02_rb,1.5226e+02_rb,1.8365e+00_rb/)
      kbo(:, 4,35, 9) = (/ &
     &6.5852e+02_rb,4.9401e+02_rb,3.2950e+02_rb,1.6506e+02_rb,1.9663e+00_rb/)
      kbo(:, 5,35, 9) = (/ &
     &7.1282e+02_rb,5.3473e+02_rb,3.5664e+02_rb,1.7860e+02_rb,2.0876e+00_rb/)
      kbo(:, 1,36, 9) = (/ &
     &5.3040e+02_rb,3.9792e+02_rb,2.6545e+02_rb,1.3332e+02_rb,1.5229e+00_rb/)
      kbo(:, 2,36, 9) = (/ &
     &5.8002e+02_rb,4.3513e+02_rb,2.9026e+02_rb,1.4559e+02_rb,1.6698e+00_rb/)
      kbo(:, 3,36, 9) = (/ &
     &6.3422e+02_rb,4.7578e+02_rb,3.1736e+02_rb,1.5904e+02_rb,1.8112e+00_rb/)
      kbo(:, 4,36, 9) = (/ &
     &6.9154e+02_rb,5.1877e+02_rb,3.4601e+02_rb,1.7331e+02_rb,1.9470e+00_rb/)
      kbo(:, 5,36, 9) = (/ &
     &7.5259e+02_rb,5.6456e+02_rb,3.7653e+02_rb,1.8854e+02_rb,2.0786e+00_rb/)
      kbo(:, 1,37, 9) = (/ &
     &5.2761e+02_rb,3.9583e+02_rb,2.6405e+02_rb,1.3269e+02_rb,1.4669e+00_rb/)
      kbo(:, 2,37, 9) = (/ &
     &5.8107e+02_rb,4.3593e+02_rb,2.9079e+02_rb,1.4590e+02_rb,1.6200e+00_rb/)
      kbo(:, 3,37, 9) = (/ &
     &6.3936e+02_rb,4.7964e+02_rb,3.1993e+02_rb,1.6036e+02_rb,1.7687e+00_rb/)
      kbo(:, 4,37, 9) = (/ &
     &7.0149e+02_rb,5.2623e+02_rb,3.5098e+02_rb,1.7581e+02_rb,1.9104e+00_rb/)
      kbo(:, 5,37, 9) = (/ &
     &7.6801e+02_rb,5.7612e+02_rb,3.8424e+02_rb,1.9240e+02_rb,2.0492e+00_rb/)
      kbo(:, 1,38, 9) = (/ &
     &5.3320e+02_rb,4.0002e+02_rb,2.6686e+02_rb,1.3414e+02_rb,1.4150e+00_rb/)
      kbo(:, 2,38, 9) = (/ &
     &5.9148e+02_rb,4.4373e+02_rb,2.9599e+02_rb,1.4854e+02_rb,1.5738e+00_rb/)
      kbo(:, 3,38, 9) = (/ &
     &6.5505e+02_rb,4.9141e+02_rb,3.2778e+02_rb,1.6430e+02_rb,1.7289e+00_rb/)
      kbo(:, 4,38, 9) = (/ &
     &7.2345e+02_rb,5.4270e+02_rb,3.6197e+02_rb,1.8131e+02_rb,1.8759e+00_rb/)
      kbo(:, 5,38, 9) = (/ &
     &7.9740e+02_rb,5.9816e+02_rb,3.9894e+02_rb,1.9975e+02_rb,2.0221e+00_rb/)
      kbo(:, 1,39, 9) = (/ &
     &5.5268e+02_rb,4.1463e+02_rb,2.7660e+02_rb,1.3905e+02_rb,1.3683e+00_rb/)
      kbo(:, 2,39, 9) = (/ &
     &6.1737e+02_rb,4.6316e+02_rb,3.0895e+02_rb,1.5504e+02_rb,1.5320e+00_rb/)
      kbo(:, 3,39, 9) = (/ &
     &6.8821e+02_rb,5.1628e+02_rb,3.4436e+02_rb,1.7260e+02_rb,1.6920e+00_rb/)
      kbo(:, 4,39, 9) = (/ &
     &7.6555e+02_rb,5.7429e+02_rb,3.8302e+02_rb,1.9184e+02_rb,1.8468e+00_rb/)
      kbo(:, 5,39, 9) = (/ &
     &8.4965e+02_rb,6.3736e+02_rb,4.2508e+02_rb,2.1282e+02_rb,1.9969e+00_rb/)
      kbo(:, 1,40, 9) = (/ &
     &5.3311e+02_rb,3.9996e+02_rb,2.6682e+02_rb,1.3428e+02_rb,1.3069e+00_rb/)
      kbo(:, 2,40, 9) = (/ &
     &5.9979e+02_rb,4.4997e+02_rb,3.0016e+02_rb,1.5072e+02_rb,1.4753e+00_rb/)
      kbo(:, 3,40, 9) = (/ &
     &6.7343e+02_rb,5.0519e+02_rb,3.3697e+02_rb,1.6896e+02_rb,1.6406e+00_rb/)
      kbo(:, 4,40, 9) = (/ &
     &7.5472e+02_rb,5.6615e+02_rb,3.7760e+02_rb,1.8916e+02_rb,1.7999e+00_rb/)
      kbo(:, 5,40, 9) = (/ &
     &8.4379e+02_rb,6.3296e+02_rb,4.2214e+02_rb,2.1136e+02_rb,1.9558e+00_rb/)
      kbo(:, 1,41, 9) = (/ &
     &5.1260e+02_rb,3.8458e+02_rb,2.5658e+02_rb,1.2928e+02_rb,1.2459e+00_rb/)
      kbo(:, 2,41, 9) = (/ &
     &5.8131e+02_rb,4.3611e+02_rb,2.9093e+02_rb,1.4621e+02_rb,1.4179e+00_rb/)
      kbo(:, 3,41, 9) = (/ &
     &6.5741e+02_rb,4.9318e+02_rb,3.2896e+02_rb,1.6503e+02_rb,1.5871e+00_rb/)
      kbo(:, 4,41, 9) = (/ &
     &7.4227e+02_rb,5.5683e+02_rb,3.7139e+02_rb,1.8609e+02_rb,1.7498e+00_rb/)
      kbo(:, 5,41, 9) = (/ &
     &8.3623e+02_rb,6.2730e+02_rb,4.1837e+02_rb,2.0950e+02_rb,1.9122e+00_rb/)
      kbo(:, 1,42, 9) = (/ &
     &4.9831e+02_rb,3.7386e+02_rb,2.4944e+02_rb,1.2584e+02_rb,1.1865e+00_rb/)
      kbo(:, 2,42, 9) = (/ &
     &5.6980e+02_rb,4.2748e+02_rb,2.8517e+02_rb,1.4342e+02_rb,1.3622e+00_rb/)
      kbo(:, 3,42, 9) = (/ &
     &6.4955e+02_rb,4.8728e+02_rb,3.2503e+02_rb,1.6313e+02_rb,1.5361e+00_rb/)
      kbo(:, 4,42, 9) = (/ &
     &7.3900e+02_rb,5.5437e+02_rb,3.6975e+02_rb,1.8531e+02_rb,1.7026e+00_rb/)
      kbo(:, 5,42, 9) = (/ &
     &8.3900e+02_rb,6.2937e+02_rb,4.1975e+02_rb,2.1021e+02_rb,1.8688e+00_rb/)
      kbo(:, 1,43, 9) = (/ &
     &4.8522e+02_rb,3.6405e+02_rb,2.4292e+02_rb,1.2271e+02_rb,1.1188e+00_rb/)
      kbo(:, 2,43, 9) = (/ &
     &5.5936e+02_rb,4.1965e+02_rb,2.7996e+02_rb,1.4092e+02_rb,1.2979e+00_rb/)
      kbo(:, 3,43, 9) = (/ &
     &6.4330e+02_rb,4.8260e+02_rb,3.2191e+02_rb,1.6164e+02_rb,1.4736e+00_rb/)
      kbo(:, 4,43, 9) = (/ &
     &7.3773e+02_rb,5.5342e+02_rb,3.6912e+02_rb,1.8505e+02_rb,1.6463e+00_rb/)
      kbo(:, 5,43, 9) = (/ &
     &8.4419e+02_rb,6.3327e+02_rb,4.2235e+02_rb,2.1154e+02_rb,1.8168e+00_rb/)
      kbo(:, 1,44, 9) = (/ &
     &4.7662e+02_rb,3.5759e+02_rb,2.3863e+02_rb,1.2069e+02_rb,1.0480e+00_rb/)
      kbo(:, 2,44, 9) = (/ &
     &5.5453e+02_rb,4.1603e+02_rb,2.7755e+02_rb,1.3982e+02_rb,1.2297e+00_rb/)
      kbo(:, 3,44, 9) = (/ &
     &6.4348e+02_rb,4.8273e+02_rb,3.2200e+02_rb,1.6176e+02_rb,1.4085e+00_rb/)
      kbo(:, 4,44, 9) = (/ &
     &7.4447e+02_rb,5.5848e+02_rb,3.7249e+02_rb,1.8678e+02_rb,1.5847e+00_rb/)
      kbo(:, 5,44, 9) = (/ &
     &8.5855e+02_rb,6.4403e+02_rb,4.2953e+02_rb,2.1515e+02_rb,1.7587e+00_rb/)
      kbo(:, 1,45, 9) = (/ &
     &4.7694e+02_rb,3.5784e+02_rb,2.3881e+02_rb,1.2088e+02_rb,9.8025e-01_rb/)
      kbo(:, 2,45, 9) = (/ &
     &5.6049e+02_rb,4.2050e+02_rb,2.8054e+02_rb,1.4139e+02_rb,1.1623e+00_rb/)
      kbo(:, 3,45, 9) = (/ &
     &6.5632e+02_rb,4.9237e+02_rb,3.2842e+02_rb,1.6503e+02_rb,1.3439e+00_rb/)
      kbo(:, 4,45, 9) = (/ &
     &7.6635e+02_rb,5.7488e+02_rb,3.8343e+02_rb,1.9228e+02_rb,1.5236e+00_rb/)
      kbo(:, 5,45, 9) = (/ &
     &8.9123e+02_rb,6.6854e+02_rb,4.4587e+02_rb,2.2334e+02_rb,1.7007e+00_rb/)
      kbo(:, 1,46, 9) = (/ &
     &4.7924e+02_rb,3.5957e+02_rb,2.3999e+02_rb,1.2157e+02_rb,9.0983e-01_rb/)
      kbo(:, 2,46, 9) = (/ &
     &5.6922e+02_rb,4.2705e+02_rb,2.8491e+02_rb,1.4366e+02_rb,1.0922e+00_rb/)
      kbo(:, 3,46, 9) = (/ &
     &6.7352e+02_rb,5.0527e+02_rb,3.3703e+02_rb,1.6939e+02_rb,1.2757e+00_rb/)
      kbo(:, 4,46, 9) = (/ &
     &7.9367e+02_rb,5.9538e+02_rb,3.9709e+02_rb,1.9915e+02_rb,1.4579e+00_rb/)
      kbo(:, 5,46, 9) = (/ &
     &9.3189e+02_rb,6.9904e+02_rb,4.6620e+02_rb,2.3352e+02_rb,1.6382e+00_rb/)
      kbo(:, 1,47, 9) = (/ &
     &4.7215e+02_rb,3.5425e+02_rb,2.3648e+02_rb,1.1995e+02_rb,8.3319e-01_rb/)
      kbo(:, 2,47, 9) = (/ &
     &5.6710e+02_rb,4.2546e+02_rb,2.8387e+02_rb,1.4325e+02_rb,1.0142e+00_rb/)
      kbo(:, 3,47, 9) = (/ &
     &6.7881e+02_rb,5.0924e+02_rb,3.3968e+02_rb,1.7081e+02_rb,1.1988e+00_rb/)
      kbo(:, 4,47, 9) = (/ &
     &8.0796e+02_rb,6.0609e+02_rb,4.0424e+02_rb,2.0278e+02_rb,1.3835e+00_rb/)
      kbo(:, 5,47, 9) = (/ &
     &9.5794e+02_rb,7.1858e+02_rb,4.7923e+02_rb,2.4007e+02_rb,1.5668e+00_rb/)
      kbo(:, 1,48, 9) = (/ &
     &4.7667e+02_rb,3.5764e+02_rb,2.3876e+02_rb,1.2119e+02_rb,7.5823e-01_rb/)
      kbo(:, 2,48, 9) = (/ &
     &5.7865e+02_rb,4.3413e+02_rb,2.8965e+02_rb,1.4623e+02_rb,9.3799e-01_rb/)
      kbo(:, 3,48, 9) = (/ &
     &7.0145e+02_rb,5.2622e+02_rb,3.5101e+02_rb,1.7653e+02_rb,1.1229e+00_rb/)
      kbo(:, 4,48, 9) = (/ &
     &8.4427e+02_rb,6.3333e+02_rb,4.2240e+02_rb,2.1190e+02_rb,1.3089e+00_rb/)
      kbo(:, 5,48, 9) = (/ &
     &1.0109e+03_rb,7.5832e+02_rb,5.0573e+02_rb,2.5334e+02_rb,1.4951e+00_rb/)
      kbo(:, 1,49, 9) = (/ &
     &4.9957e+02_rb,3.7481e+02_rb,2.5022e+02_rb,1.2697e+02_rb,6.8631e-01_rb/)
      kbo(:, 2,49, 9) = (/ &
     &6.1224e+02_rb,4.5931e+02_rb,3.0645e+02_rb,1.5466e+02_rb,8.6353e-01_rb/)
      kbo(:, 3,49, 9) = (/ &
     &7.5197e+02_rb,5.6411e+02_rb,3.7627e+02_rb,1.8917e+02_rb,1.0480e+00_rb/)
      kbo(:, 4,49, 9) = (/ &
     &9.1643e+02_rb,6.8745e+02_rb,4.5848e+02_rb,2.2994e+02_rb,1.2341e+00_rb/)
      kbo(:, 5,49, 9) = (/ &
     &1.1094e+03_rb,8.3219e+02_rb,5.5497e+02_rb,2.7795e+02_rb,1.4234e+00_rb/)
      kbo(:, 1,50, 9) = (/ &
     &4.9927e+02_rb,3.7459e+02_rb,2.5011e+02_rb,1.2702e+02_rb,6.2082e-01_rb/)
      kbo(:, 2,50, 9) = (/ &
     &6.1745e+02_rb,4.6322e+02_rb,3.0907e+02_rb,1.5606e+02_rb,7.9337e-01_rb/)
      kbo(:, 3,50, 9) = (/ &
     &7.6850e+02_rb,5.7651e+02_rb,3.8454e+02_rb,1.9338e+02_rb,9.7702e-01_rb/)
      kbo(:, 4,50, 9) = (/ &
     &9.4909e+02_rb,7.1195e+02_rb,4.7482e+02_rb,2.3816e+02_rb,1.1637e+00_rb/)
      kbo(:, 5,50, 9) = (/ &
     &1.1628e+03_rb,8.7227e+02_rb,5.8169e+02_rb,2.9133e+02_rb,1.3527e+00_rb/)
      kbo(:, 1,51, 9) = (/ &
     &4.8758e+02_rb,3.6583e+02_rb,2.4432e+02_rb,1.2426e+02_rb,5.5888e-01_rb/)
      kbo(:, 2,51, 9) = (/ &
     &6.0851e+02_rb,4.5652e+02_rb,3.0463e+02_rb,1.5396e+02_rb,7.2702e-01_rb/)
      kbo(:, 3,51, 9) = (/ &
     &7.6706e+02_rb,5.7543e+02_rb,3.8383e+02_rb,1.9312e+02_rb,9.0804e-01_rb/)
      kbo(:, 4,51, 9) = (/ &
     &9.6042e+02_rb,7.2045e+02_rb,4.8049e+02_rb,2.4107e+02_rb,1.0946e+00_rb/)
      kbo(:, 5,51, 9) = (/ &
     &1.1920e+03_rb,8.9415e+02_rb,5.9629e+02_rb,2.9867e+02_rb,1.2844e+00_rb/)
      kbo(:, 1,52, 9) = (/ &
     &4.8860e+02_rb,3.6660e+02_rb,2.4488e+02_rb,1.2464e+02_rb,4.9962e-01_rb/)
      kbo(:, 2,52, 9) = (/ &
     &6.1451e+02_rb,4.6102e+02_rb,3.0765e+02_rb,1.5555e+02_rb,6.6320e-01_rb/)
      kbo(:, 3,52, 9) = (/ &
     &7.8418e+02_rb,5.8827e+02_rb,3.9240e+02_rb,1.9747e+02_rb,8.4128e-01_rb/)
      kbo(:, 4,52, 9) = (/ &
     &9.9602e+02_rb,7.4715e+02_rb,4.9829e+02_rb,2.5001e+02_rb,1.0268e+00_rb/)
      kbo(:, 5,52, 9) = (/ &
     &1.2534e+03_rb,9.4018e+02_rb,6.2698e+02_rb,3.1403e+02_rb,1.2165e+00_rb/)
      kbo(:, 1,53, 9) = (/ &
     &5.0881e+02_rb,3.8175e+02_rb,2.5500e+02_rb,1.2975e+02_rb,4.4418e-01_rb/)
      kbo(:, 2,53, 9) = (/ &
     &6.4426e+02_rb,4.8334e+02_rb,3.2253e+02_rb,1.6302e+02_rb,6.0138e-01_rb/)
      kbo(:, 3,53, 9) = (/ &
     &8.3158e+02_rb,6.2382e+02_rb,4.1611e+02_rb,2.0933e+02_rb,7.7570e-01_rb/)
      kbo(:, 4,53, 9) = (/ &
     &1.0715e+03_rb,8.0375e+02_rb,5.3603e+02_rb,2.6888e+02_rb,9.5940e-01_rb/)
      kbo(:, 5,53, 9) = (/ &
     &1.3680e+03_rb,1.0262e+03_rb,6.8430e+02_rb,3.4269e+02_rb,1.1487e+00_rb/)
      kbo(:, 1,54, 9) = (/ &
     &4.6019e+02_rb,3.4529e+02_rb,2.3082e+02_rb,1.1788e+02_rb,3.9422e-01_rb/)
      kbo(:, 2,54, 9) = (/ &
     &5.8633e+02_rb,4.3988e+02_rb,2.9364e+02_rb,1.4879e+02_rb,5.4422e-01_rb/)
      kbo(:, 3,54, 9) = (/ &
     &7.6477e+02_rb,5.7372e+02_rb,3.8273e+02_rb,1.9285e+02_rb,7.1419e-01_rb/)
      kbo(:, 4,54, 9) = (/ &
     &9.9975e+02_rb,7.4995e+02_rb,5.0017e+02_rb,2.5112e+02_rb,8.9603e-01_rb/)
      kbo(:, 5,54, 9) = (/ &
     &1.2952e+03_rb,9.7153e+02_rb,6.4788e+02_rb,3.2458e+02_rb,1.0830e+00_rb/)
      kbo(:, 1,55, 9) = (/ &
     &3.8020e+02_rb,2.8531e+02_rb,1.9107e+02_rb,9.8307e+01_rb,3.4739e-01_rb/)
      kbo(:, 2,55, 9) = (/ &
     &4.8735e+02_rb,3.6565e+02_rb,2.4430e+02_rb,1.2445e+02_rb,4.9078e-01_rb/)
      kbo(:, 3,55, 9) = (/ &
     &6.4163e+02_rb,4.8136e+02_rb,3.2124e+02_rb,1.6243e+02_rb,6.5484e-01_rb/)
      kbo(:, 4,55, 9) = (/ &
     &8.5082e+02_rb,6.3825e+02_rb,4.2573e+02_rb,2.1419e+02_rb,8.3371e-01_rb/)
      kbo(:, 5,55, 9) = (/ &
     &1.1188e+03_rb,8.3927e+02_rb,5.5970e+02_rb,2.8072e+02_rb,1.0198e+00_rb/)
      kbo(:, 1,56, 9) = (/ &
     &3.1368e+02_rb,2.3545e+02_rb,1.5809e+02_rb,8.2086e+01_rb,3.0410e-01_rb/)
      kbo(:, 2,56, 9) = (/ &
     &4.0416e+02_rb,3.0326e+02_rb,2.0292e+02_rb,1.0407e+02_rb,4.3985e-01_rb/)
      kbo(:, 3,56, 9) = (/ &
     &5.3658e+02_rb,4.0257e+02_rb,2.6883e+02_rb,1.3655e+02_rb,5.9766e-01_rb/)
      kbo(:, 4,56, 9) = (/ &
     &7.2129e+02_rb,5.4110e+02_rb,3.6102e+02_rb,1.8214e+02_rb,7.7256e-01_rb/)
      kbo(:, 5,56, 9) = (/ &
     &9.6339e+02_rb,7.2268e+02_rb,4.8200e+02_rb,2.4214e+02_rb,9.5689e-01_rb/)
      kbo(:, 1,57, 9) = (/ &
     &2.5819e+02_rb,1.9390e+02_rb,1.3066e+02_rb,6.8602e+01_rb,2.6463e-01_rb/)
      kbo(:, 2,57, 9) = (/ &
     &3.3457e+02_rb,2.5110e+02_rb,1.6838e+02_rb,8.7076e+01_rb,3.9133e-01_rb/)
      kbo(:, 3,57, 9) = (/ &
     &4.4726e+02_rb,3.3558e+02_rb,2.2435e+02_rb,1.1462e+02_rb,5.4231e-01_rb/)
      kbo(:, 4,57, 9) = (/ &
     &6.0897e+02_rb,4.5687e+02_rb,3.0496e+02_rb,1.5443e+02_rb,7.1198e-01_rb/)
      kbo(:, 5,57, 9) = (/ &
     &8.2628e+02_rb,6.1985e+02_rb,4.1347e+02_rb,2.0818e+02_rb,8.9451e-01_rb/)
      kbo(:, 1,58, 9) = (/ &
     &5.6144e+01_rb,5.1598e+01_rb,4.4952e+01_rb,3.3575e+01_rb,2.2945e-01_rb/)
      kbo(:, 2,58, 9) = (/ &
     &7.3248e+01_rb,6.7230e+01_rb,5.8185e+01_rb,4.2642e+01_rb,3.4680e-01_rb/)
      kbo(:, 3,58, 9) = (/ &
     &9.8566e+01_rb,9.0421e+01_rb,7.7903e+01_rb,5.6215e+01_rb,4.9127e-01_rb/)
      kbo(:, 4,58, 9) = (/ &
     &1.3577e+02_rb,1.2452e+02_rb,1.0700e+02_rb,7.6316e+01_rb,6.5580e-01_rb/)
      kbo(:, 5,58, 9) = (/ &
     &1.8716e+02_rb,1.7164e+02_rb,1.4731e+02_rb,1.0424e+02_rb,8.3481e-01_rb/)
      kbo(:, 1,59, 9) = (/ &
     &5.7246e+01_rb,5.0299e+01_rb,4.1347e+01_rb,2.8542e+01_rb,2.1390e-01_rb/)
      kbo(:, 2,59, 9) = (/ &
     &7.5569e+01_rb,6.6181e+01_rb,5.3818e+01_rb,3.6138e+01_rb,3.2687e-01_rb/)
      kbo(:, 3,59, 9) = (/ &
     &1.0297e+02_rb,9.0031e+01_rb,7.2620e+01_rb,4.7606e+01_rb,4.6582e-01_rb/)
      kbo(:, 4,59, 9) = (/ &
     &1.4369e+02_rb,1.2555e+02_rb,1.0072e+02_rb,6.4807e+01_rb,6.2712e-01_rb/)
      kbo(:, 5,59, 9) = (/ &
     &2.0105e+02_rb,1.7562e+02_rb,1.4046e+02_rb,8.9199e+01_rb,8.0292e-01_rb/)
      kbo(:, 1,13,10) = (/ &
     &1.3594e+05_rb,1.0195e+05_rb,6.7969e+04_rb,3.3985e+04_rb,3.3627e+00_rb/)
      kbo(:, 2,13,10) = (/ &
     &1.3605e+05_rb,1.0204e+05_rb,6.8026e+04_rb,3.4013e+04_rb,3.2528e+00_rb/)
      kbo(:, 3,13,10) = (/ &
     &1.3597e+05_rb,1.0198e+05_rb,6.7984e+04_rb,3.3992e+04_rb,3.1814e+00_rb/)
      kbo(:, 4,13,10) = (/ &
     &1.3504e+05_rb,1.0128e+05_rb,6.7520e+04_rb,3.3760e+04_rb,3.0515e+00_rb/)
      kbo(:, 5,13,10) = (/ &
     &1.3382e+05_rb,1.0037e+05_rb,6.6910e+04_rb,3.3456e+04_rb,3.0303e+00_rb/)
      kbo(:, 1,14,10) = (/ &
     &8.3672e+04_rb,6.2754e+04_rb,4.1836e+04_rb,2.0918e+04_rb,3.6220e+00_rb/)
      kbo(:, 2,14,10) = (/ &
     &8.3355e+04_rb,6.2516e+04_rb,4.1677e+04_rb,2.0839e+04_rb,3.5454e+00_rb/)
      kbo(:, 3,14,10) = (/ &
     &8.2924e+04_rb,6.2193e+04_rb,4.1462e+04_rb,2.0731e+04_rb,3.4328e+00_rb/)
      kbo(:, 4,14,10) = (/ &
     &8.2189e+04_rb,6.1641e+04_rb,4.1094e+04_rb,2.0547e+04_rb,3.3922e+00_rb/)
      kbo(:, 5,14,10) = (/ &
     &8.1205e+04_rb,6.0904e+04_rb,4.0602e+04_rb,2.0301e+04_rb,3.2425e+00_rb/)
      kbo(:, 1,15,10) = (/ &
     &5.1150e+04_rb,3.8363e+04_rb,2.5576e+04_rb,1.2788e+04_rb,3.9145e+00_rb/)
      kbo(:, 2,15,10) = (/ &
     &5.0875e+04_rb,3.8157e+04_rb,2.5438e+04_rb,1.2719e+04_rb,3.8152e+00_rb/)
      kbo(:, 3,15,10) = (/ &
     &5.0407e+04_rb,3.7806e+04_rb,2.5204e+04_rb,1.2602e+04_rb,3.7733e+00_rb/)
      kbo(:, 4,15,10) = (/ &
     &4.9693e+04_rb,3.7270e+04_rb,2.4847e+04_rb,1.2424e+04_rb,3.6717e+00_rb/)
      kbo(:, 5,15,10) = (/ &
     &4.8987e+04_rb,3.6740e+04_rb,2.4494e+04_rb,1.2247e+04_rb,3.5647e+00_rb/)
      kbo(:, 1,16,10) = (/ &
     &3.6535e+04_rb,2.7401e+04_rb,1.8268e+04_rb,9.1341e+03_rb,4.2542e+00_rb/)
      kbo(:, 2,16,10) = (/ &
     &3.6151e+04_rb,2.7114e+04_rb,1.8076e+04_rb,9.0383e+03_rb,4.1666e+00_rb/)
      kbo(:, 3,16,10) = (/ &
     &3.5666e+04_rb,2.6750e+04_rb,1.7833e+04_rb,8.9169e+03_rb,4.1063e+00_rb/)
      kbo(:, 4,16,10) = (/ &
     &3.5133e+04_rb,2.6350e+04_rb,1.7567e+04_rb,8.7838e+03_rb,4.0309e+00_rb/)
      kbo(:, 5,16,10) = (/ &
     &3.4526e+04_rb,2.5894e+04_rb,1.7263e+04_rb,8.6318e+03_rb,3.8978e+00_rb/)
      kbo(:, 1,17,10) = (/ &
     &2.6414e+04_rb,1.9811e+04_rb,1.3207e+04_rb,6.6039e+03_rb,4.6454e+00_rb/)
      kbo(:, 2,17,10) = (/ &
     &2.6068e+04_rb,1.9551e+04_rb,1.3034e+04_rb,6.5174e+03_rb,4.5633e+00_rb/)
      kbo(:, 3,17,10) = (/ &
     &2.5689e+04_rb,1.9267e+04_rb,1.2845e+04_rb,6.4225e+03_rb,4.4826e+00_rb/)
      kbo(:, 4,17,10) = (/ &
     &2.5209e+04_rb,1.8907e+04_rb,1.2605e+04_rb,6.3027e+03_rb,4.3889e+00_rb/)
      kbo(:, 5,17,10) = (/ &
     &2.4661e+04_rb,1.8496e+04_rb,1.2331e+04_rb,6.1658e+03_rb,4.2589e+00_rb/)
      kbo(:, 1,18,10) = (/ &
     &1.9208e+04_rb,1.4406e+04_rb,9.6044e+03_rb,4.8025e+03_rb,5.0342e+00_rb/)
      kbo(:, 2,18,10) = (/ &
     &1.8927e+04_rb,1.4195e+04_rb,9.4636e+03_rb,4.7320e+03_rb,4.9453e+00_rb/)
      kbo(:, 3,18,10) = (/ &
     &1.8575e+04_rb,1.3931e+04_rb,9.2875e+03_rb,4.6440e+03_rb,4.8565e+00_rb/)
      kbo(:, 4,18,10) = (/ &
     &1.8151e+04_rb,1.3614e+04_rb,9.0760e+03_rb,4.5383e+03_rb,4.7616e+00_rb/)
      kbo(:, 5,18,10) = (/ &
     &1.7715e+04_rb,1.3287e+04_rb,8.8579e+03_rb,4.4292e+03_rb,4.6381e+00_rb/)
      kbo(:, 1,19,10) = (/ &
     &1.3561e+04_rb,1.0171e+04_rb,6.7808e+03_rb,3.3906e+03_rb,5.4045e+00_rb/)
      kbo(:, 2,19,10) = (/ &
     &1.3305e+04_rb,9.9790e+03_rb,6.6528e+03_rb,3.3267e+03_rb,5.3114e+00_rb/)
      kbo(:, 3,19,10) = (/ &
     &1.3001e+04_rb,9.7507e+03_rb,6.5006e+03_rb,3.2506e+03_rb,5.2250e+00_rb/)
      kbo(:, 4,19,10) = (/ &
     &1.2687e+04_rb,9.5155e+03_rb,6.3438e+03_rb,3.1722e+03_rb,5.1407e+00_rb/)
      kbo(:, 5,19,10) = (/ &
     &1.2394e+04_rb,9.2960e+03_rb,6.1975e+03_rb,3.0991e+03_rb,5.0251e+00_rb/)
      kbo(:, 1,20,10) = (/ &
     &1.0195e+04_rb,7.6466e+03_rb,5.0979e+03_rb,2.5492e+03_rb,5.7652e+00_rb/)
      kbo(:, 2,20,10) = (/ &
     &9.9631e+03_rb,7.4725e+03_rb,4.9818e+03_rb,2.4912e+03_rb,5.6804e+00_rb/)
      kbo(:, 3,20,10) = (/ &
     &9.7293e+03_rb,7.2971e+03_rb,4.8649e+03_rb,2.4328e+03_rb,5.6092e+00_rb/)
      kbo(:, 4,20,10) = (/ &
     &9.5227e+03_rb,7.1421e+03_rb,4.7616e+03_rb,2.3811e+03_rb,5.5285e+00_rb/)
      kbo(:, 5,20,10) = (/ &
     &9.3625e+03_rb,7.0220e+03_rb,4.6816e+03_rb,2.3411e+03_rb,5.3995e+00_rb/)
      kbo(:, 1,21,10) = (/ &
     &7.7885e+03_rb,5.8415e+03_rb,3.8945e+03_rb,1.9475e+03_rb,6.1280e+00_rb/)
      kbo(:, 2,21,10) = (/ &
     &7.6092e+03_rb,5.7071e+03_rb,3.8049e+03_rb,1.9027e+03_rb,6.0467e+00_rb/)
      kbo(:, 3,21,10) = (/ &
     &7.4658e+03_rb,5.5995e+03_rb,3.7332e+03_rb,1.8669e+03_rb,5.9782e+00_rb/)
      kbo(:, 4,21,10) = (/ &
     &7.3647e+03_rb,5.5237e+03_rb,3.6826e+03_rb,1.8416e+03_rb,5.8911e+00_rb/)
      kbo(:, 5,21,10) = (/ &
     &7.2928e+03_rb,5.4698e+03_rb,3.6467e+03_rb,1.8237e+03_rb,5.7660e+00_rb/)
      kbo(:, 1,22,10) = (/ &
     &6.0472e+03_rb,4.5355e+03_rb,3.0239e+03_rb,1.5122e+03_rb,6.4389e+00_rb/)
      kbo(:, 2,22,10) = (/ &
     &5.9457e+03_rb,4.4594e+03_rb,2.9731e+03_rb,1.4868e+03_rb,6.3534e+00_rb/)
      kbo(:, 3,22,10) = (/ &
     &5.8843e+03_rb,4.4133e+03_rb,2.9424e+03_rb,1.4715e+03_rb,6.3043e+00_rb/)
      kbo(:, 4,22,10) = (/ &
     &5.8543e+03_rb,4.3909e+03_rb,2.9274e+03_rb,1.4640e+03_rb,6.2309e+00_rb/)
      kbo(:, 5,22,10) = (/ &
     &5.8906e+03_rb,4.4181e+03_rb,2.9456e+03_rb,1.4732e+03_rb,6.0993e+00_rb/)
      kbo(:, 1,23,10) = (/ &
     &4.7512e+03_rb,3.5635e+03_rb,2.3758e+03_rb,1.1882e+03_rb,6.6888e+00_rb/)
      kbo(:, 2,23,10) = (/ &
     &4.7151e+03_rb,3.5365e+03_rb,2.3578e+03_rb,1.1792e+03_rb,6.6289e+00_rb/)
      kbo(:, 3,23,10) = (/ &
     &4.7153e+03_rb,3.5366e+03_rb,2.3579e+03_rb,1.1792e+03_rb,6.5888e+00_rb/)
      kbo(:, 4,23,10) = (/ &
     &4.7757e+03_rb,3.5820e+03_rb,2.3882e+03_rb,1.1944e+03_rb,6.5103e+00_rb/)
      kbo(:, 5,23,10) = (/ &
     &4.8784e+03_rb,3.6590e+03_rb,2.4396e+03_rb,1.2201e+03_rb,6.3750e+00_rb/)
      kbo(:, 1,24,10) = (/ &
     &3.7664e+03_rb,2.8249e+03_rb,1.8835e+03_rb,9.4199e+02_rb,6.8939e+00_rb/)
      kbo(:, 2,24,10) = (/ &
     &3.7833e+03_rb,2.8376e+03_rb,1.8919e+03_rb,9.4626e+02_rb,6.8560e+00_rb/)
      kbo(:, 3,24,10) = (/ &
     &3.8524e+03_rb,2.8895e+03_rb,1.9265e+03_rb,9.6361e+02_rb,6.8233e+00_rb/)
      kbo(:, 4,24,10) = (/ &
     &3.9503e+03_rb,2.9629e+03_rb,1.9755e+03_rb,9.8811e+02_rb,6.7364e+00_rb/)
      kbo(:, 5,24,10) = (/ &
     &4.0500e+03_rb,3.0377e+03_rb,2.0254e+03_rb,1.0130e+03_rb,6.5861e+00_rb/)
      kbo(:, 1,25,10) = (/ &
     &3.0705e+03_rb,2.3030e+03_rb,1.5356e+03_rb,7.6808e+02_rb,7.0352e+00_rb/)
      kbo(:, 2,25,10) = (/ &
     &3.1368e+03_rb,2.3527e+03_rb,1.5687e+03_rb,7.8472e+02_rb,7.0310e+00_rb/)
      kbo(:, 3,25,10) = (/ &
     &3.2299e+03_rb,2.4226e+03_rb,1.6153e+03_rb,8.0803e+02_rb,7.0074e+00_rb/)
      kbo(:, 4,25,10) = (/ &
     &3.3291e+03_rb,2.4970e+03_rb,1.6649e+03_rb,8.3282e+02_rb,6.9138e+00_rb/)
      kbo(:, 5,25,10) = (/ &
     &3.4334e+03_rb,2.5753e+03_rb,1.7171e+03_rb,8.5889e+02_rb,6.7836e+00_rb/)
      kbo(:, 1,26,10) = (/ &
     &2.5973e+03_rb,1.9481e+03_rb,1.2990e+03_rb,6.4986e+02_rb,7.1301e+00_rb/)
      kbo(:, 2,26,10) = (/ &
     &2.6809e+03_rb,2.0109e+03_rb,1.3408e+03_rb,6.7078e+02_rb,7.1546e+00_rb/)
      kbo(:, 3,26,10) = (/ &
     &2.7748e+03_rb,2.0813e+03_rb,1.3878e+03_rb,6.9426e+02_rb,7.1359e+00_rb/)
      kbo(:, 4,26,10) = (/ &
     &2.8656e+03_rb,2.1494e+03_rb,1.4332e+03_rb,7.1694e+02_rb,7.0388e+00_rb/)
      kbo(:, 5,26,10) = (/ &
     &2.9554e+03_rb,2.2167e+03_rb,1.4780e+03_rb,7.3936e+02_rb,6.9309e+00_rb/)
      kbo(:, 1,27,10) = (/ &
     &2.3040e+03_rb,1.7282e+03_rb,1.1524e+03_rb,5.7657e+02_rb,7.1805e+00_rb/)
      kbo(:, 2,27,10) = (/ &
     &2.3938e+03_rb,1.7956e+03_rb,1.1973e+03_rb,5.9902e+02_rb,7.2265e+00_rb/)
      kbo(:, 3,27,10) = (/ &
     &2.4776e+03_rb,1.8584e+03_rb,1.2392e+03_rb,6.1996e+02_rb,7.2160e+00_rb/)
      kbo(:, 4,27,10) = (/ &
     &2.5592e+03_rb,1.9196e+03_rb,1.2799e+03_rb,6.4031e+02_rb,7.1019e+00_rb/)
      kbo(:, 5,27,10) = (/ &
     &2.6344e+03_rb,1.9760e+03_rb,1.3176e+03_rb,6.5914e+02_rb,7.0533e+00_rb/)
      kbo(:, 1,28,10) = (/ &
     &2.1016e+03_rb,1.5764e+03_rb,1.0512e+03_rb,5.2597e+02_rb,7.1943e+00_rb/)
      kbo(:, 2,28,10) = (/ &
     &2.1832e+03_rb,1.6376e+03_rb,1.0920e+03_rb,5.4637e+02_rb,7.2562e+00_rb/)
      kbo(:, 3,28,10) = (/ &
     &2.2626e+03_rb,1.6971e+03_rb,1.1316e+03_rb,5.6618e+02_rb,7.2356e+00_rb/)
      kbo(:, 4,28,10) = (/ &
     &2.3367e+03_rb,1.7527e+03_rb,1.1687e+03_rb,5.8470e+02_rb,7.1672e+00_rb/)
      kbo(:, 5,28,10) = (/ &
     &2.4052e+03_rb,1.8041e+03_rb,1.2030e+03_rb,6.0186e+02_rb,7.1806e+00_rb/)
      kbo(:, 1,29,10) = (/ &
     &2.0213e+03_rb,1.5162e+03_rb,1.0110e+03_rb,5.0592e+02_rb,7.1826e+00_rb/)
      kbo(:, 2,29,10) = (/ &
     &2.1025e+03_rb,1.5771e+03_rb,1.0516e+03_rb,5.2618e+02_rb,7.2449e+00_rb/)
      kbo(:, 3,29,10) = (/ &
     &2.1817e+03_rb,1.6365e+03_rb,1.0912e+03_rb,5.4596e+02_rb,7.2200e+00_rb/)
      kbo(:, 4,29,10) = (/ &
     &2.2556e+03_rb,1.6919e+03_rb,1.1282e+03_rb,5.6447e+02_rb,7.2591e+00_rb/)
      kbo(:, 5,29,10) = (/ &
     &2.3246e+03_rb,1.7436e+03_rb,1.1626e+03_rb,5.8167e+02_rb,7.2912e+00_rb/)
      kbo(:, 1,30,10) = (/ &
     &1.9891e+03_rb,1.4920e+03_rb,9.9489e+02_rb,4.9783e+02_rb,7.1696e+00_rb/)
      kbo(:, 2,30,10) = (/ &
     &2.0704e+03_rb,1.5530e+03_rb,1.0355e+03_rb,5.1812e+02_rb,7.1996e+00_rb/)
      kbo(:, 3,30,10) = (/ &
     &2.1486e+03_rb,1.6117e+03_rb,1.0747e+03_rb,5.3773e+02_rb,7.2564e+00_rb/)
      kbo(:, 4,30,10) = (/ &
     &2.2243e+03_rb,1.6684e+03_rb,1.1125e+03_rb,5.5660e+02_rb,7.3491e+00_rb/)
      kbo(:, 5,30,10) = (/ &
     &2.2969e+03_rb,1.7228e+03_rb,1.1488e+03_rb,5.7471e+02_rb,7.3950e+00_rb/)
      kbo(:, 1,31,10) = (/ &
     &2.0600e+03_rb,1.5452e+03_rb,1.0303e+03_rb,5.1552e+02_rb,7.0891e+00_rb/)
      kbo(:, 2,31,10) = (/ &
     &2.1442e+03_rb,1.6083e+03_rb,1.0725e+03_rb,5.3661e+02_rb,7.1940e+00_rb/)
      kbo(:, 3,31,10) = (/ &
     &2.2267e+03_rb,1.6702e+03_rb,1.1137e+03_rb,5.5720e+02_rb,7.3519e+00_rb/)
      kbo(:, 4,31,10) = (/ &
     &2.3069e+03_rb,1.7303e+03_rb,1.1538e+03_rb,5.7721e+02_rb,7.4783e+00_rb/)
      kbo(:, 5,31,10) = (/ &
     &2.3951e+03_rb,1.7965e+03_rb,1.1978e+03_rb,5.9922e+02_rb,7.4978e+00_rb/)
      kbo(:, 1,32,10) = (/ &
     &2.1607e+03_rb,1.6207e+03_rb,1.0807e+03_rb,5.4073e+02_rb,7.0921e+00_rb/)
      kbo(:, 2,32,10) = (/ &
     &2.2499e+03_rb,1.6876e+03_rb,1.1253e+03_rb,5.6302e+02_rb,7.2853e+00_rb/)
      kbo(:, 3,32,10) = (/ &
     &2.3389e+03_rb,1.7543e+03_rb,1.1698e+03_rb,5.8523e+02_rb,7.4637e+00_rb/)
      kbo(:, 4,32,10) = (/ &
     &2.4345e+03_rb,1.8261e+03_rb,1.2176e+03_rb,6.0908e+02_rb,7.5300e+00_rb/)
      kbo(:, 5,32,10) = (/ &
     &2.5534e+03_rb,1.9152e+03_rb,1.2769e+03_rb,6.3872e+02_rb,7.5836e+00_rb/)
      kbo(:, 1,33,10) = (/ &
     &2.3147e+03_rb,1.7362e+03_rb,1.1577e+03_rb,5.7922e+02_rb,7.1422e+00_rb/)
      kbo(:, 2,33,10) = (/ &
     &2.4114e+03_rb,1.8087e+03_rb,1.2060e+03_rb,6.0335e+02_rb,7.3798e+00_rb/)
      kbo(:, 3,33,10) = (/ &
     &2.5142e+03_rb,1.8858e+03_rb,1.2574e+03_rb,6.2900e+02_rb,7.4990e+00_rb/)
      kbo(:, 4,33,10) = (/ &
     &2.6436e+03_rb,1.9829e+03_rb,1.3221e+03_rb,6.6129e+02_rb,7.6018e+00_rb/)
      kbo(:, 5,33,10) = (/ &
     &2.8166e+03_rb,2.1125e+03_rb,1.4085e+03_rb,7.0447e+02_rb,7.6468e+00_rb/)
      kbo(:, 1,34,10) = (/ &
     &2.4210e+03_rb,1.8159e+03_rb,1.2108e+03_rb,6.0577e+02_rb,7.2083e+00_rb/)
      kbo(:, 2,34,10) = (/ &
     &2.5284e+03_rb,1.8965e+03_rb,1.2645e+03_rb,6.3255e+02_rb,7.4180e+00_rb/)
      kbo(:, 3,34,10) = (/ &
     &2.6591e+03_rb,1.9944e+03_rb,1.3298e+03_rb,6.6516e+02_rb,7.5418e+00_rb/)
      kbo(:, 4,34,10) = (/ &
     &2.8359e+03_rb,2.1270e+03_rb,1.4182e+03_rb,7.0928e+02_rb,7.6931e+00_rb/)
      kbo(:, 5,34,10) = (/ &
     &3.0680e+03_rb,2.3011e+03_rb,1.5342e+03_rb,7.6730e+02_rb,7.7811e+00_rb/)
      kbo(:, 1,35,10) = (/ &
     &2.5363e+03_rb,1.9024e+03_rb,1.2685e+03_rb,6.3456e+02_rb,7.2309e+00_rb/)
      kbo(:, 2,35,10) = (/ &
     &2.6634e+03_rb,1.9977e+03_rb,1.3320e+03_rb,6.6631e+02_rb,7.4393e+00_rb/)
      kbo(:, 3,35,10) = (/ &
     &2.8345e+03_rb,2.1260e+03_rb,1.4175e+03_rb,7.0897e+02_rb,7.6569e+00_rb/)
      kbo(:, 4,35,10) = (/ &
     &3.0674e+03_rb,2.3007e+03_rb,1.5339e+03_rb,7.6716e+02_rb,7.7801e+00_rb/)
      kbo(:, 5,35,10) = (/ &
     &3.3573e+03_rb,2.5181e+03_rb,1.6788e+03_rb,8.3963e+02_rb,7.8561e+00_rb/)
      kbo(:, 1,36,10) = (/ &
     &2.6451e+03_rb,1.9840e+03_rb,1.3229e+03_rb,6.6175e+02_rb,7.2395e+00_rb/)
      kbo(:, 2,36,10) = (/ &
     &2.7976e+03_rb,2.0983e+03_rb,1.3991e+03_rb,6.9981e+02_rb,7.4813e+00_rb/)
      kbo(:, 3,36,10) = (/ &
     &3.0125e+03_rb,2.2595e+03_rb,1.5065e+03_rb,7.5346e+02_rb,7.7309e+00_rb/)
      kbo(:, 4,36,10) = (/ &
     &3.2979e+03_rb,2.4736e+03_rb,1.6492e+03_rb,8.2479e+02_rb,7.8735e+00_rb/)
      kbo(:, 5,36,10) = (/ &
     &3.6475e+03_rb,2.7357e+03_rb,1.8239e+03_rb,9.1218e+02_rb,7.9027e+00_rb/)
      kbo(:, 1,37,10) = (/ &
     &2.6819e+03_rb,2.0116e+03_rb,1.3413e+03_rb,6.7096e+02_rb,7.2017e+00_rb/)
      kbo(:, 2,37,10) = (/ &
     &2.8583e+03_rb,2.1439e+03_rb,1.4294e+03_rb,7.1495e+02_rb,7.5003e+00_rb/)
      kbo(:, 3,37,10) = (/ &
     &3.1042e+03_rb,2.3283e+03_rb,1.5523e+03_rb,7.7640e+02_rb,7.7646e+00_rb/)
      kbo(:, 4,37,10) = (/ &
     &3.4272e+03_rb,2.5705e+03_rb,1.7138e+03_rb,8.5714e+02_rb,7.9233e+00_rb/)
      kbo(:, 5,37,10) = (/ &
     &3.8225e+03_rb,2.8670e+03_rb,1.9115e+03_rb,9.5596e+02_rb,7.9456e+00_rb/)
      kbo(:, 1,38,10) = (/ &
     &2.7669e+03_rb,2.0753e+03_rb,1.3837e+03_rb,6.9217e+02_rb,7.1724e+00_rb/)
      kbo(:, 2,38,10) = (/ &
     &2.9772e+03_rb,2.2330e+03_rb,1.4888e+03_rb,7.4467e+02_rb,7.5382e+00_rb/)
      kbo(:, 3,38,10) = (/ &
     &3.2630e+03_rb,2.4473e+03_rb,1.6317e+03_rb,8.1609e+02_rb,7.7857e+00_rb/)
      kbo(:, 4,38,10) = (/ &
     &3.6292e+03_rb,2.7220e+03_rb,1.8149e+03_rb,9.0769e+02_rb,7.9649e+00_rb/)
      kbo(:, 5,38,10) = (/ &
     &4.0743e+03_rb,3.0559e+03_rb,2.0374e+03_rb,1.0190e+03_rb,7.9771e+00_rb/)
      kbo(:, 1,39,10) = (/ &
     &2.9268e+03_rb,2.1953e+03_rb,1.4637e+03_rb,7.3213e+02_rb,7.1691e+00_rb/)
      kbo(:, 2,39,10) = (/ &
     &3.1832e+03_rb,2.3875e+03_rb,1.5919e+03_rb,7.9619e+02_rb,7.5536e+00_rb/)
      kbo(:, 3,39,10) = (/ &
     &3.5251e+03_rb,2.6440e+03_rb,1.7628e+03_rb,8.8168e+02_rb,7.8089e+00_rb/)
      kbo(:, 4,39,10) = (/ &
     &3.9504e+03_rb,2.9629e+03_rb,1.9755e+03_rb,9.8804e+02_rb,7.9786e+00_rb/)
      kbo(:, 5,39,10) = (/ &
     &4.4598e+03_rb,3.3450e+03_rb,2.2302e+03_rb,1.1154e+03_rb,8.0073e+00_rb/)
      kbo(:, 1,40,10) = (/ &
     &2.8874e+03_rb,2.1657e+03_rb,1.4439e+03_rb,7.2221e+02_rb,7.0746e+00_rb/)
      kbo(:, 2,40,10) = (/ &
     &3.1724e+03_rb,2.3794e+03_rb,1.5864e+03_rb,7.9346e+02_rb,7.4831e+00_rb/)
      kbo(:, 3,40,10) = (/ &
     &3.5465e+03_rb,2.6600e+03_rb,1.7735e+03_rb,8.8703e+02_rb,7.7692e+00_rb/)
      kbo(:, 4,40,10) = (/ &
     &4.0044e+03_rb,3.0035e+03_rb,2.0025e+03_rb,1.0016e+03_rb,7.9632e+00_rb/)
      kbo(:, 5,40,10) = (/ &
     &4.5454e+03_rb,3.4092e+03_rb,2.2730e+03_rb,1.1368e+03_rb,8.0081e+00_rb/)
      kbo(:, 1,41,10) = (/ &
     &2.8406e+03_rb,2.1306e+03_rb,1.4205e+03_rb,7.1050e+02_rb,6.9600e+00_rb/)
      kbo(:, 2,41,10) = (/ &
     &3.1484e+03_rb,2.3614e+03_rb,1.5744e+03_rb,7.8747e+02_rb,7.3954e+00_rb/)
      kbo(:, 3,41,10) = (/ &
     &3.5576e+03_rb,2.6684e+03_rb,1.7791e+03_rb,8.8987e+02_rb,7.7179e+00_rb/)
      kbo(:, 4,41,10) = (/ &
     &4.0527e+03_rb,3.0397e+03_rb,2.0267e+03_rb,1.0137e+03_rb,7.9444e+00_rb/)
      kbo(:, 5,41,10) = (/ &
     &4.6270e+03_rb,3.4704e+03_rb,2.3138e+03_rb,1.1572e+03_rb,8.0108e+00_rb/)
      kbo(:, 1,42,10) = (/ &
     &2.8258e+03_rb,2.1194e+03_rb,1.4131e+03_rb,7.0678e+02_rb,6.8517e+00_rb/)
      kbo(:, 2,42,10) = (/ &
     &3.1581e+03_rb,2.3687e+03_rb,1.5793e+03_rb,7.8990e+02_rb,7.3049e+00_rb/)
      kbo(:, 3,42,10) = (/ &
     &3.6059e+03_rb,2.7046e+03_rb,1.8032e+03_rb,9.0193e+02_rb,7.6524e+00_rb/)
      kbo(:, 4,42,10) = (/ &
     &4.1499e+03_rb,3.1126e+03_rb,2.0753e+03_rb,1.0380e+03_rb,7.9111e+00_rb/)
      kbo(:, 5,42,10) = (/ &
     &4.7763e+03_rb,3.5824e+03_rb,2.3885e+03_rb,1.1945e+03_rb,8.0026e+00_rb/)
      kbo(:, 1,43,10) = (/ &
     &2.8194e+03_rb,2.1147e+03_rb,1.4100e+03_rb,7.0522e+02_rb,6.7032e+00_rb/)
      kbo(:, 2,43,10) = (/ &
     &3.1747e+03_rb,2.3812e+03_rb,1.5876e+03_rb,7.9408e+02_rb,7.1735e+00_rb/)
      kbo(:, 3,43,10) = (/ &
     &3.6549e+03_rb,2.7413e+03_rb,1.8278e+03_rb,9.1420e+02_rb,7.5844e+00_rb/)
      kbo(:, 4,43,10) = (/ &
     &4.2522e+03_rb,3.1893e+03_rb,2.1264e+03_rb,1.0635e+03_rb,7.8420e+00_rb/)
      kbo(:, 5,43,10) = (/ &
     &4.9421e+03_rb,3.7067e+03_rb,2.4714e+03_rb,1.2360e+03_rb,7.9777e+00_rb/)
      kbo(:, 1,44,10) = (/ &
     &2.8469e+03_rb,2.1353e+03_rb,1.4237e+03_rb,7.1206e+02_rb,6.5380e+00_rb/)
      kbo(:, 2,44,10) = (/ &
     &3.2203e+03_rb,2.4153e+03_rb,1.6104e+03_rb,8.0548e+02_rb,7.0393e+00_rb/)
      kbo(:, 3,44,10) = (/ &
     &3.7360e+03_rb,2.8021e+03_rb,1.8683e+03_rb,9.3448e+02_rb,7.4778e+00_rb/)
      kbo(:, 4,44,10) = (/ &
     &4.3910e+03_rb,3.2934e+03_rb,2.1958e+03_rb,1.0982e+03_rb,7.7716e+00_rb/)
      kbo(:, 5,44,10) = (/ &
     &5.1620e+03_rb,3.8716e+03_rb,2.5813e+03_rb,1.2909e+03_rb,7.9568e+00_rb/)
      kbo(:, 1,45,10) = (/ &
     &2.9370e+03_rb,2.2029e+03_rb,1.4687e+03_rb,7.3461e+02_rb,6.3596e+00_rb/)
      kbo(:, 2,45,10) = (/ &
     &3.3323e+03_rb,2.4994e+03_rb,1.6664e+03_rb,8.3351e+02_rb,6.8956e+00_rb/)
      kbo(:, 3,45,10) = (/ &
     &3.8921e+03_rb,2.9193e+03_rb,1.9464e+03_rb,9.7354e+02_rb,7.3748e+00_rb/)
      kbo(:, 4,45,10) = (/ &
     &4.6196e+03_rb,3.4648e+03_rb,2.3101e+03_rb,1.1554e+03_rb,7.7094e+00_rb/)
      kbo(:, 5,45,10) = (/ &
     &5.4975e+03_rb,4.1233e+03_rb,2.7490e+03_rb,1.3748e+03_rb,7.9394e+00_rb/)
      kbo(:, 1,46,10) = (/ &
     &3.0543e+03_rb,2.2908e+03_rb,1.5274e+03_rb,7.6393e+02_rb,6.1665e+00_rb/)
      kbo(:, 2,46,10) = (/ &
     &3.4757e+03_rb,2.6069e+03_rb,1.7382e+03_rb,8.6940e+02_rb,6.7524e+00_rb/)
      kbo(:, 3,46,10) = (/ &
     &4.0758e+03_rb,3.0570e+03_rb,2.0382e+03_rb,1.0195e+03_rb,7.2688e+00_rb/)
      kbo(:, 4,46,10) = (/ &
     &4.8832e+03_rb,3.6626e+03_rb,2.4419e+03_rb,1.2213e+03_rb,7.6299e+00_rb/)
      kbo(:, 5,46,10) = (/ &
     &5.8799e+03_rb,4.4101e+03_rb,2.9402e+03_rb,1.4704e+03_rb,7.9039e+00_rb/)
      kbo(:, 1,47,10) = (/ &
     &3.1321e+03_rb,2.3492e+03_rb,1.5663e+03_rb,7.8341e+02_rb,5.9258e+00_rb/)
      kbo(:, 2,47,10) = (/ &
     &3.5703e+03_rb,2.6779e+03_rb,1.7855e+03_rb,8.9306e+02_rb,6.5564e+00_rb/)
      kbo(:, 3,47,10) = (/ &
     &4.1981e+03_rb,3.1488e+03_rb,2.0994e+03_rb,1.0501e+03_rb,7.1267e+00_rb/)
      kbo(:, 4,47,10) = (/ &
     &5.0652e+03_rb,3.7991e+03_rb,2.5329e+03_rb,1.2668e+03_rb,7.5455e+00_rb/)
      kbo(:, 5,47,10) = (/ &
     &6.1689e+03_rb,4.6268e+03_rb,3.0847e+03_rb,1.5426e+03_rb,7.8470e+00_rb/)
      kbo(:, 1,48,10) = (/ &
     &3.3047e+03_rb,2.4787e+03_rb,1.6526e+03_rb,8.2657e+02_rb,5.6694e+00_rb/)
      kbo(:, 2,48,10) = (/ &
     &3.7747e+03_rb,2.8312e+03_rb,1.8877e+03_rb,9.4418e+02_rb,6.3589e+00_rb/)
      kbo(:, 3,48,10) = (/ &
     &4.4483e+03_rb,3.3364e+03_rb,2.2245e+03_rb,1.1126e+03_rb,6.9569e+00_rb/)
      kbo(:, 4,48,10) = (/ &
     &5.3940e+03_rb,4.0457e+03_rb,2.6973e+03_rb,1.3490e+03_rb,7.4395e+00_rb/)
      kbo(:, 5,48,10) = (/ &
     &6.6398e+03_rb,4.9800e+03_rb,3.3201e+03_rb,1.6603e+03_rb,7.7751e+00_rb/)
      kbo(:, 1,49,10) = (/ &
     &3.6256e+03_rb,2.7194e+03_rb,1.8131e+03_rb,9.0682e+02_rb,5.4147e+00_rb/)
      kbo(:, 2,49,10) = (/ &
     &4.1643e+03_rb,3.1234e+03_rb,2.0825e+03_rb,1.0416e+03_rb,6.1453e+00_rb/)
      kbo(:, 3,49,10) = (/ &
     &4.9094e+03_rb,3.6822e+03_rb,2.4550e+03_rb,1.2279e+03_rb,6.7872e+00_rb/)
      kbo(:, 4,49,10) = (/ &
     &5.9847e+03_rb,4.4886e+03_rb,2.9926e+03_rb,1.4966e+03_rb,7.3168e+00_rb/)
      kbo(:, 5,49,10) = (/ &
     &7.4258e+03_rb,5.5695e+03_rb,3.7131e+03_rb,1.8568e+03_rb,7.6869e+00_rb/)
      kbo(:, 1,50,10) = (/ &
     &3.7913e+03_rb,2.8436e+03_rb,1.8959e+03_rb,9.4825e+02_rb,5.1338e+00_rb/)
      kbo(:, 2,50,10) = (/ &
     &4.3920e+03_rb,3.2942e+03_rb,2.1963e+03_rb,1.0985e+03_rb,5.9203e+00_rb/)
      kbo(:, 3,50,10) = (/ &
     &5.1851e+03_rb,3.8890e+03_rb,2.5929e+03_rb,1.2968e+03_rb,6.5802e+00_rb/)
      kbo(:, 4,50,10) = (/ &
     &6.3489e+03_rb,4.7618e+03_rb,3.1748e+03_rb,1.5877e+03_rb,7.1705e+00_rb/)
      kbo(:, 5,50,10) = (/ &
     &7.9377e+03_rb,5.9534e+03_rb,3.9691e+03_rb,1.9848e+03_rb,7.5922e+00_rb/)
      kbo(:, 1,51,10) = (/ &
     &3.8669e+03_rb,2.9003e+03_rb,1.9338e+03_rb,9.6719e+02_rb,4.8302e+00_rb/)
      kbo(:, 2,51,10) = (/ &
     &4.5318e+03_rb,3.3991e+03_rb,2.2663e+03_rb,1.1335e+03_rb,5.6653e+00_rb/)
      kbo(:, 3,51,10) = (/ &
     &5.3773e+03_rb,4.0331e+03_rb,2.6890e+03_rb,1.3448e+03_rb,6.3736e+00_rb/)
      kbo(:, 4,51,10) = (/ &
     &6.6015e+03_rb,4.9513e+03_rb,3.3010e+03_rb,1.6508e+03_rb,7.0028e+00_rb/)
      kbo(:, 5,51,10) = (/ &
     &8.3208e+03_rb,6.2407e+03_rb,4.1606e+03_rb,2.0805e+03_rb,7.4729e+00_rb/)
      kbo(:, 1,52,10) = (/ &
     &4.0341e+03_rb,3.0258e+03_rb,2.0174e+03_rb,1.0090e+03_rb,4.5505e+00_rb/)
      kbo(:, 2,52,10) = (/ &
     &4.7982e+03_rb,3.5989e+03_rb,2.3995e+03_rb,1.2001e+03_rb,5.4004e+00_rb/)
      kbo(:, 3,52,10) = (/ &
     &5.7356e+03_rb,4.3019e+03_rb,2.8682e+03_rb,1.4344e+03_rb,6.1556e+00_rb/)
      kbo(:, 4,52,10) = (/ &
     &7.0672e+03_rb,5.3005e+03_rb,3.5339e+03_rb,1.7672e+03_rb,6.8208e+00_rb/)
      kbo(:, 5,52,10) = (/ &
     &8.9657e+03_rb,6.7244e+03_rb,4.4831e+03_rb,2.2418e+03_rb,7.3432e+00_rb/)
      kbo(:, 1,53,10) = (/ &
     &4.3581e+03_rb,3.2687e+03_rb,2.1794e+03_rb,1.0900e+03_rb,4.2696e+00_rb/)
      kbo(:, 2,53,10) = (/ &
     &5.2688e+03_rb,3.9518e+03_rb,2.6348e+03_rb,1.3178e+03_rb,5.1422e+00_rb/)
      kbo(:, 3,53,10) = (/ &
     &6.3653e+03_rb,4.7742e+03_rb,3.1830e+03_rb,1.5919e+03_rb,5.9288e+00_rb/)
      kbo(:, 4,53,10) = (/ &
     &7.8870e+03_rb,5.9154e+03_rb,3.9438e+03_rb,1.9722e+03_rb,6.6345e+00_rb/)
      kbo(:, 5,53,10) = (/ &
     &1.0051e+04_rb,7.5383e+03_rb,5.0257e+03_rb,2.5131e+03_rb,7.2054e+00_rb/)
      kbo(:, 1,54,10) = (/ &
     &4.0664e+03_rb,3.0500e+03_rb,2.0336e+03_rb,1.0171e+03_rb,3.9912e+00_rb/)
      kbo(:, 2,54,10) = (/ &
     &5.0070e+03_rb,3.7555e+03_rb,2.5039e+03_rb,1.2523e+03_rb,4.8673e+00_rb/)
      kbo(:, 3,54,10) = (/ &
     &6.1329e+03_rb,4.5998e+03_rb,3.0668e+03_rb,1.5337e+03_rb,5.6727e+00_rb/)
      kbo(:, 4,54,10) = (/ &
     &7.6570e+03_rb,5.7429e+03_rb,3.8288e+03_rb,1.9147e+03_rb,6.4112e+00_rb/)
      kbo(:, 5,54,10) = (/ &
     &9.8073e+03_rb,7.3556e+03_rb,4.9039e+03_rb,2.4522e+03_rb,7.0431e+00_rb/)
      kbo(:, 1,55,10) = (/ &
     &3.4487e+03_rb,2.5868e+03_rb,1.7247e+03_rb,8.6275e+02_rb,3.7023e+00_rb/)
      kbo(:, 2,55,10) = (/ &
     &4.3277e+03_rb,3.2460e+03_rb,2.1642e+03_rb,1.0825e+03_rb,4.5324e+00_rb/)
      kbo(:, 3,55,10) = (/ &
     &5.3892e+03_rb,4.0421e+03_rb,2.6950e+03_rb,1.3479e+03_rb,5.4035e+00_rb/)
      kbo(:, 4,55,10) = (/ &
     &6.7963e+03_rb,5.0973e+03_rb,3.3984e+03_rb,1.6995e+03_rb,6.1546e+00_rb/)
      kbo(:, 5,55,10) = (/ &
     &8.7627e+03_rb,6.5722e+03_rb,4.3816e+03_rb,2.1910e+03_rb,6.8267e+00_rb/)
      kbo(:, 1,56,10) = (/ &
     &2.9072e+03_rb,2.1806e+03_rb,1.4540e+03_rb,7.2734e+02_rb,3.4115e+00_rb/)
      kbo(:, 2,56,10) = (/ &
     &3.7198e+03_rb,2.7901e+03_rb,1.8603e+03_rb,9.3056e+02_rb,4.2571e+00_rb/)
      kbo(:, 3,56,10) = (/ &
     &4.7203e+03_rb,3.5403e+03_rb,2.3605e+03_rb,1.1806e+03_rb,5.1151e+00_rb/)
      kbo(:, 4,56,10) = (/ &
     &6.0279e+03_rb,4.5211e+03_rb,3.0142e+03_rb,1.5074e+03_rb,5.8984e+00_rb/)
      kbo(:, 5,56,10) = (/ &
     &7.8366e+03_rb,5.8775e+03_rb,3.9185e+03_rb,1.9594e+03_rb,6.6003e+00_rb/)
      kbo(:, 1,57,10) = (/ &
     &2.4367e+03_rb,1.8277e+03_rb,1.2187e+03_rb,6.0970e+02_rb,3.1098e+00_rb/)
      kbo(:, 2,57,10) = (/ &
     &3.1766e+03_rb,2.3826e+03_rb,1.5887e+03_rb,7.9475e+02_rb,3.9704e+00_rb/)
      kbo(:, 3,57,10) = (/ &
     &4.1136e+03_rb,3.0854e+03_rb,2.0572e+03_rb,1.0290e+03_rb,4.8038e+00_rb/)
      kbo(:, 4,57,10) = (/ &
     &5.3345e+03_rb,4.0010e+03_rb,2.6675e+03_rb,1.3340e+03_rb,5.6428e+00_rb/)
      kbo(:, 5,57,10) = (/ &
     &7.0017e+03_rb,5.2513e+03_rb,3.5010e+03_rb,1.7507e+03_rb,6.3564e+00_rb/)
      kbo(:, 1,58,10) = (/ &
     &5.3797e+02_rb,4.9330e+02_rb,4.2306e+02_rb,2.9653e+02_rb,2.8402e+00_rb/)
      kbo(:, 2,58,10) = (/ &
     &7.1438e+02_rb,6.5505e+02_rb,5.6177e+02_rb,3.9372e+02_rb,3.6954e+00_rb/)
      kbo(:, 3,58,10) = (/ &
     &9.4508e+02_rb,8.6656e+02_rb,7.4311e+02_rb,5.2072e+02_rb,4.5299e+00_rb/)
      kbo(:, 4,58,10) = (/ &
     &1.2473e+03_rb,1.1436e+03_rb,9.8061e+02_rb,6.8699e+02_rb,5.3595e+00_rb/)
      kbo(:, 5,58,10) = (/ &
     &1.6559e+03_rb,1.5182e+03_rb,1.3018e+03_rb,9.1191e+02_rb,6.1054e+00_rb/)
      kbo(:, 1,59,10) = (/ &
     &5.5953e+02_rb,4.8866e+02_rb,3.8993e+02_rb,2.4311e+02_rb,2.7081e+00_rb/)
      kbo(:, 2,59,10) = (/ &
     &7.5600e+02_rb,6.6022e+02_rb,5.2680e+02_rb,3.2817e+02_rb,3.5280e+00_rb/)
      kbo(:, 3,59,10) = (/ &
     &1.0213e+03_rb,8.9185e+02_rb,7.1155e+02_rb,4.4309e+02_rb,4.3511e+00_rb/)
      kbo(:, 4,59,10) = (/ &
     &1.3747e+03_rb,1.2004e+03_rb,9.5765e+02_rb,5.9615e+02_rb,5.1215e+00_rb/)
      kbo(:, 5,59,10) = (/ &
     &1.8474e+03_rb,1.6131e+03_rb,1.2868e+03_rb,8.0091e+02_rb,5.9179e+00_rb/)
      kbo(:, 1,13,11) = (/ &
     &1.9556e+05_rb,1.4667e+05_rb,9.7780e+04_rb,4.8890e+04_rb,3.8293e+00_rb/)
      kbo(:, 2,13,11) = (/ &
     &1.9579e+05_rb,1.4684e+05_rb,9.7894e+04_rb,4.8947e+04_rb,3.6477e+00_rb/)
      kbo(:, 3,13,11) = (/ &
     &1.9489e+05_rb,1.4617e+05_rb,9.7447e+04_rb,4.8724e+04_rb,3.6240e+00_rb/)
      kbo(:, 4,13,11) = (/ &
     &1.9354e+05_rb,1.4515e+05_rb,9.6770e+04_rb,4.8385e+04_rb,3.4253e+00_rb/)
      kbo(:, 5,13,11) = (/ &
     &1.9176e+05_rb,1.4382e+05_rb,9.5882e+04_rb,4.7942e+04_rb,3.3965e+00_rb/)
      kbo(:, 1,14,11) = (/ &
     &1.2532e+05_rb,9.3990e+04_rb,6.2660e+04_rb,3.1330e+04_rb,4.2132e+00_rb/)
      kbo(:, 2,14,11) = (/ &
     &1.2490e+05_rb,9.3675e+04_rb,6.2450e+04_rb,3.1225e+04_rb,4.0914e+00_rb/)
      kbo(:, 3,14,11) = (/ &
     &1.2400e+05_rb,9.3003e+04_rb,6.2001e+04_rb,3.1001e+04_rb,4.0126e+00_rb/)
      kbo(:, 4,14,11) = (/ &
     &1.2262e+05_rb,9.1963e+04_rb,6.1309e+04_rb,3.0654e+04_rb,3.8288e+00_rb/)
      kbo(:, 5,14,11) = (/ &
     &1.2099e+05_rb,9.0745e+04_rb,6.0496e+04_rb,3.0248e+04_rb,3.6494e+00_rb/)
      kbo(:, 1,15,11) = (/ &
     &8.0146e+04_rb,6.0110e+04_rb,4.0074e+04_rb,2.0037e+04_rb,4.6025e+00_rb/)
      kbo(:, 2,15,11) = (/ &
     &7.9365e+04_rb,5.9524e+04_rb,3.9683e+04_rb,1.9842e+04_rb,4.5011e+00_rb/)
      kbo(:, 3,15,11) = (/ &
     &7.8437e+04_rb,5.8828e+04_rb,3.9219e+04_rb,1.9610e+04_rb,4.3995e+00_rb/)
      kbo(:, 4,15,11) = (/ &
     &7.7471e+04_rb,5.8104e+04_rb,3.8736e+04_rb,1.9369e+04_rb,4.3087e+00_rb/)
      kbo(:, 5,15,11) = (/ &
     &7.6167e+04_rb,5.7126e+04_rb,3.8084e+04_rb,1.9043e+04_rb,4.1657e+00_rb/)
      kbo(:, 1,16,11) = (/ &
     &5.9412e+04_rb,4.4559e+04_rb,2.9706e+04_rb,1.4854e+04_rb,4.9879e+00_rb/)
      kbo(:, 2,16,11) = (/ &
     &5.8737e+04_rb,4.4053e+04_rb,2.9369e+04_rb,1.4685e+04_rb,4.8952e+00_rb/)
      kbo(:, 3,16,11) = (/ &
     &5.7913e+04_rb,4.3435e+04_rb,2.8957e+04_rb,1.4479e+04_rb,4.7931e+00_rb/)
      kbo(:, 4,16,11) = (/ &
     &5.6912e+04_rb,4.2684e+04_rb,2.8456e+04_rb,1.4229e+04_rb,4.7058e+00_rb/)
      kbo(:, 5,16,11) = (/ &
     &5.5858e+04_rb,4.1894e+04_rb,2.7929e+04_rb,1.3965e+04_rb,4.5654e+00_rb/)
      kbo(:, 1,17,11) = (/ &
     &4.4739e+04_rb,3.3554e+04_rb,2.2370e+04_rb,1.1185e+04_rb,5.4258e+00_rb/)
      kbo(:, 2,17,11) = (/ &
     &4.4040e+04_rb,3.3030e+04_rb,2.2020e+04_rb,1.1011e+04_rb,5.3316e+00_rb/)
      kbo(:, 3,17,11) = (/ &
     &4.3278e+04_rb,3.2459e+04_rb,2.1639e+04_rb,1.0820e+04_rb,5.2258e+00_rb/)
      kbo(:, 4,17,11) = (/ &
     &4.2484e+04_rb,3.1863e+04_rb,2.1242e+04_rb,1.0622e+04_rb,5.1306e+00_rb/)
      kbo(:, 5,17,11) = (/ &
     &4.1610e+04_rb,3.1208e+04_rb,2.0805e+04_rb,1.0403e+04_rb,4.9611e+00_rb/)
      kbo(:, 1,18,11) = (/ &
     &3.3808e+04_rb,2.5356e+04_rb,1.6904e+04_rb,8.4526e+03_rb,5.9588e+00_rb/)
      kbo(:, 2,18,11) = (/ &
     &3.3205e+04_rb,2.4904e+04_rb,1.6603e+04_rb,8.3019e+03_rb,5.8553e+00_rb/)
      kbo(:, 3,18,11) = (/ &
     &3.2587e+04_rb,2.4440e+04_rb,1.6294e+04_rb,8.1473e+03_rb,5.7407e+00_rb/)
      kbo(:, 4,18,11) = (/ &
     &3.1890e+04_rb,2.3917e+04_rb,1.5945e+04_rb,7.9729e+03_rb,5.6396e+00_rb/)
      kbo(:, 5,18,11) = (/ &
     &3.1157e+04_rb,2.3368e+04_rb,1.5579e+04_rb,7.7897e+03_rb,5.4685e+00_rb/)
      kbo(:, 1,19,11) = (/ &
     &2.4641e+04_rb,1.8481e+04_rb,1.2321e+04_rb,6.1608e+03_rb,6.5634e+00_rb/)
      kbo(:, 2,19,11) = (/ &
     &2.4168e+04_rb,1.8126e+04_rb,1.2084e+04_rb,6.0424e+03_rb,6.4593e+00_rb/)
      kbo(:, 3,19,11) = (/ &
     &2.3646e+04_rb,1.7735e+04_rb,1.1823e+04_rb,5.9120e+03_rb,6.3292e+00_rb/)
      kbo(:, 4,19,11) = (/ &
     &2.3077e+04_rb,1.7308e+04_rb,1.1539e+04_rb,5.7697e+03_rb,6.2034e+00_rb/)
      kbo(:, 5,19,11) = (/ &
     &2.2489e+04_rb,1.6867e+04_rb,1.1245e+04_rb,5.6226e+03_rb,6.0369e+00_rb/)
      kbo(:, 1,20,11) = (/ &
     &1.9106e+04_rb,1.4330e+04_rb,9.5533e+03_rb,4.7770e+03_rb,7.1646e+00_rb/)
      kbo(:, 2,20,11) = (/ &
     &1.8682e+04_rb,1.4011e+04_rb,9.3411e+03_rb,4.6709e+03_rb,7.0409e+00_rb/)
      kbo(:, 3,20,11) = (/ &
     &1.8224e+04_rb,1.3668e+04_rb,9.1125e+03_rb,4.5566e+03_rb,6.8869e+00_rb/)
      kbo(:, 4,20,11) = (/ &
     &1.7734e+04_rb,1.3300e+04_rb,8.8671e+03_rb,4.4339e+03_rb,6.7557e+00_rb/)
      kbo(:, 5,20,11) = (/ &
     &1.7241e+04_rb,1.2931e+04_rb,8.6211e+03_rb,4.3109e+03_rb,6.6027e+00_rb/)
      kbo(:, 1,21,11) = (/ &
     &1.5039e+04_rb,1.1280e+04_rb,7.5200e+03_rb,3.7604e+03_rb,7.7170e+00_rb/)
      kbo(:, 2,21,11) = (/ &
     &1.4660e+04_rb,1.0995e+04_rb,7.3306e+03_rb,3.6657e+03_rb,7.5835e+00_rb/)
      kbo(:, 3,21,11) = (/ &
     &1.4253e+04_rb,1.0690e+04_rb,7.1270e+03_rb,3.5639e+03_rb,7.4413e+00_rb/)
      kbo(:, 4,21,11) = (/ &
     &1.3848e+04_rb,1.0387e+04_rb,6.9246e+03_rb,3.4627e+03_rb,7.3214e+00_rb/)
      kbo(:, 5,21,11) = (/ &
     &1.3474e+04_rb,1.0106e+04_rb,6.7374e+03_rb,3.3691e+03_rb,7.1721e+00_rb/)
      kbo(:, 1,22,11) = (/ &
     &1.1928e+04_rb,8.9459e+03_rb,5.9642e+03_rb,2.9825e+03_rb,8.2536e+00_rb/)
      kbo(:, 2,22,11) = (/ &
     &1.1588e+04_rb,8.6914e+03_rb,5.7946e+03_rb,2.8977e+03_rb,8.1353e+00_rb/)
      kbo(:, 3,22,11) = (/ &
     &1.1261e+04_rb,8.4462e+03_rb,5.6311e+03_rb,2.8159e+03_rb,8.0046e+00_rb/)
      kbo(:, 4,22,11) = (/ &
     &1.0981e+04_rb,8.2362e+03_rb,5.4911e+03_rb,2.7459e+03_rb,7.8868e+00_rb/)
      kbo(:, 5,22,11) = (/ &
     &1.0767e+04_rb,8.0756e+03_rb,5.3840e+03_rb,2.6924e+03_rb,7.7103e+00_rb/)
      kbo(:, 1,23,11) = (/ &
     &9.3738e+03_rb,7.0305e+03_rb,4.6873e+03_rb,2.3441e+03_rb,8.7957e+00_rb/)
      kbo(:, 2,23,11) = (/ &
     &9.1120e+03_rb,6.8342e+03_rb,4.5564e+03_rb,2.2786e+03_rb,8.6750e+00_rb/)
      kbo(:, 3,23,11) = (/ &
     &8.9119e+03_rb,6.6842e+03_rb,4.4564e+03_rb,2.2286e+03_rb,8.5667e+00_rb/)
      kbo(:, 4,23,11) = (/ &
     &8.7698e+03_rb,6.5776e+03_rb,4.3853e+03_rb,2.1931e+03_rb,8.4437e+00_rb/)
      kbo(:, 5,23,11) = (/ &
     &8.7064e+03_rb,6.5300e+03_rb,4.3536e+03_rb,2.1772e+03_rb,8.2499e+00_rb/)
      kbo(:, 1,24,11) = (/ &
     &7.2818e+03_rb,5.4615e+03_rb,3.6413e+03_rb,1.8211e+03_rb,9.3047e+00_rb/)
      kbo(:, 2,24,11) = (/ &
     &7.1398e+03_rb,5.3551e+03_rb,3.5703e+03_rb,1.7856e+03_rb,9.1844e+00_rb/)
      kbo(:, 3,24,11) = (/ &
     &7.0551e+03_rb,5.2916e+03_rb,3.5280e+03_rb,1.7644e+03_rb,9.0906e+00_rb/)
      kbo(:, 4,24,11) = (/ &
     &7.0561e+03_rb,5.2923e+03_rb,3.5284e+03_rb,1.7646e+03_rb,8.9639e+00_rb/)
      kbo(:, 5,24,11) = (/ &
     &7.1200e+03_rb,5.3402e+03_rb,3.5604e+03_rb,1.7806e+03_rb,8.7597e+00_rb/)
      kbo(:, 1,25,11) = (/ &
     &5.7829e+03_rb,4.3374e+03_rb,2.8918e+03_rb,1.4463e+03_rb,9.7531e+00_rb/)
      kbo(:, 2,25,11) = (/ &
     &5.7416e+03_rb,4.3064e+03_rb,2.8712e+03_rb,1.4360e+03_rb,9.6333e+00_rb/)
      kbo(:, 3,25,11) = (/ &
     &5.7703e+03_rb,4.3279e+03_rb,2.8855e+03_rb,1.4432e+03_rb,9.5575e+00_rb/)
      kbo(:, 4,25,11) = (/ &
     &5.8528e+03_rb,4.3898e+03_rb,2.9268e+03_rb,1.4638e+03_rb,9.4320e+00_rb/)
      kbo(:, 5,25,11) = (/ &
     &5.9946e+03_rb,4.4961e+03_rb,2.9977e+03_rb,1.4993e+03_rb,9.2048e+00_rb/)
      kbo(:, 1,26,11) = (/ &
     &4.7547e+03_rb,3.5662e+03_rb,2.3777e+03_rb,1.1892e+03_rb,1.0127e+01_rb/)
      kbo(:, 2,26,11) = (/ &
     &4.7969e+03_rb,3.5979e+03_rb,2.3989e+03_rb,1.1999e+03_rb,1.0014e+01_rb/)
      kbo(:, 3,26,11) = (/ &
     &4.8916e+03_rb,3.6689e+03_rb,2.4462e+03_rb,1.2235e+03_rb,9.9567e+00_rb/)
      kbo(:, 4,26,11) = (/ &
     &5.0339e+03_rb,3.7756e+03_rb,2.5174e+03_rb,1.2591e+03_rb,9.8430e+00_rb/)
      kbo(:, 5,26,11) = (/ &
     &5.1881e+03_rb,3.8913e+03_rb,2.5944e+03_rb,1.2976e+03_rb,9.5879e+00_rb/)
      kbo(:, 1,27,11) = (/ &
     &4.1434e+03_rb,3.1078e+03_rb,2.0721e+03_rb,1.0365e+03_rb,1.0438e+01_rb/)
      kbo(:, 2,27,11) = (/ &
     &4.2449e+03_rb,3.1839e+03_rb,2.1229e+03_rb,1.0619e+03_rb,1.0375e+01_rb/)
      kbo(:, 3,27,11) = (/ &
     &4.3814e+03_rb,3.2863e+03_rb,2.1911e+03_rb,1.0960e+03_rb,1.0314e+01_rb/)
      kbo(:, 4,27,11) = (/ &
     &4.5344e+03_rb,3.4010e+03_rb,2.2676e+03_rb,1.1342e+03_rb,1.0178e+01_rb/)
      kbo(:, 5,27,11) = (/ &
     &4.7035e+03_rb,3.5278e+03_rb,2.3521e+03_rb,1.1764e+03_rb,9.9184e+00_rb/)
      kbo(:, 1,28,11) = (/ &
     &3.7712e+03_rb,2.8287e+03_rb,1.8861e+03_rb,9.4351e+02_rb,1.0688e+01_rb/)
      kbo(:, 2,28,11) = (/ &
     &3.9032e+03_rb,2.9276e+03_rb,1.9520e+03_rb,9.7644e+02_rb,1.0679e+01_rb/)
      kbo(:, 3,28,11) = (/ &
     &4.0571e+03_rb,3.0430e+03_rb,2.0289e+03_rb,1.0148e+03_rb,1.0623e+01_rb/)
      kbo(:, 4,28,11) = (/ &
     &4.2272e+03_rb,3.1706e+03_rb,2.1139e+03_rb,1.0573e+03_rb,1.0454e+01_rb/)
      kbo(:, 5,28,11) = (/ &
     &4.4031e+03_rb,3.3025e+03_rb,2.2018e+03_rb,1.1012e+03_rb,1.0168e+01_rb/)
      kbo(:, 1,29,11) = (/ &
     &3.6879e+03_rb,2.7661e+03_rb,1.8444e+03_rb,9.2260e+02_rb,1.0914e+01_rb/)
      kbo(:, 2,29,11) = (/ &
     &3.8440e+03_rb,2.8832e+03_rb,1.9224e+03_rb,9.6157e+02_rb,1.0945e+01_rb/)
      kbo(:, 3,29,11) = (/ &
     &4.0134e+03_rb,3.0102e+03_rb,2.0070e+03_rb,1.0038e+03_rb,1.0853e+01_rb/)
      kbo(:, 4,29,11) = (/ &
     &4.1909e+03_rb,3.1433e+03_rb,2.0957e+03_rb,1.0481e+03_rb,1.0578e+01_rb/)
      kbo(:, 5,29,11) = (/ &
     &4.3726e+03_rb,3.2796e+03_rb,2.1866e+03_rb,1.0936e+03_rb,1.0403e+01_rb/)
      kbo(:, 1,30,11) = (/ &
     &3.7192e+03_rb,2.7896e+03_rb,1.8600e+03_rb,9.3035e+02_rb,1.1141e+01_rb/)
      kbo(:, 2,30,11) = (/ &
     &3.8944e+03_rb,2.9210e+03_rb,1.9475e+03_rb,9.7411e+02_rb,1.1139e+01_rb/)
      kbo(:, 3,30,11) = (/ &
     &4.0753e+03_rb,3.0566e+03_rb,2.0379e+03_rb,1.0192e+03_rb,1.0970e+01_rb/)
      kbo(:, 4,30,11) = (/ &
     &4.2621e+03_rb,3.1967e+03_rb,2.1313e+03_rb,1.0660e+03_rb,1.0736e+01_rb/)
      kbo(:, 5,30,11) = (/ &
     &4.4544e+03_rb,3.3410e+03_rb,2.2275e+03_rb,1.1141e+03_rb,1.0663e+01_rb/)
      kbo(:, 1,31,11) = (/ &
     &3.9494e+03_rb,2.9622e+03_rb,1.9750e+03_rb,9.8786e+02_rb,1.1309e+01_rb/)
      kbo(:, 2,31,11) = (/ &
     &4.1497e+03_rb,3.1124e+03_rb,2.0751e+03_rb,1.0378e+03_rb,1.1233e+01_rb/)
      kbo(:, 3,31,11) = (/ &
     &4.3564e+03_rb,3.2674e+03_rb,2.1785e+03_rb,1.0895e+03_rb,1.1031e+01_rb/)
      kbo(:, 4,31,11) = (/ &
     &4.5677e+03_rb,3.4259e+03_rb,2.2841e+03_rb,1.1423e+03_rb,1.1018e+01_rb/)
      kbo(:, 5,31,11) = (/ &
     &4.7823e+03_rb,3.5869e+03_rb,2.3915e+03_rb,1.1960e+03_rb,1.0922e+01_rb/)
      kbo(:, 1,32,11) = (/ &
     &4.2398e+03_rb,3.1800e+03_rb,2.1202e+03_rb,1.0604e+03_rb,1.1395e+01_rb/)
      kbo(:, 2,32,11) = (/ &
     &4.4759e+03_rb,3.3571e+03_rb,2.2382e+03_rb,1.1194e+03_rb,1.1282e+01_rb/)
      kbo(:, 3,32,11) = (/ &
     &4.7161e+03_rb,3.5372e+03_rb,2.3583e+03_rb,1.1795e+03_rb,1.1286e+01_rb/)
      kbo(:, 4,32,11) = (/ &
     &4.9603e+03_rb,3.7204e+03_rb,2.4805e+03_rb,1.2406e+03_rb,1.1267e+01_rb/)
      kbo(:, 5,32,11) = (/ &
     &5.2103e+03_rb,3.9079e+03_rb,2.6055e+03_rb,1.3031e+03_rb,1.1177e+01_rb/)
      kbo(:, 1,33,11) = (/ &
     &4.6404e+03_rb,3.4804e+03_rb,2.3205e+03_rb,1.1605e+03_rb,1.1433e+01_rb/)
      kbo(:, 2,33,11) = (/ &
     &4.9200e+03_rb,3.6901e+03_rb,2.4603e+03_rb,1.2304e+03_rb,1.1468e+01_rb/)
      kbo(:, 3,33,11) = (/ &
     &5.2076e+03_rb,3.9059e+03_rb,2.6041e+03_rb,1.3024e+03_rb,1.1541e+01_rb/)
      kbo(:, 4,33,11) = (/ &
     &5.4986e+03_rb,4.1241e+03_rb,2.7496e+03_rb,1.3751e+03_rb,1.1519e+01_rb/)
      kbo(:, 5,33,11) = (/ &
     &5.8045e+03_rb,4.3535e+03_rb,2.9026e+03_rb,1.4516e+03_rb,1.1398e+01_rb/)
      kbo(:, 1,34,11) = (/ &
     &4.9577e+03_rb,3.7184e+03_rb,2.4791e+03_rb,1.2399e+03_rb,1.1582e+01_rb/)
      kbo(:, 2,34,11) = (/ &
     &5.2771e+03_rb,3.9580e+03_rb,2.6389e+03_rb,1.3198e+03_rb,1.1708e+01_rb/)
      kbo(:, 3,34,11) = (/ &
     &5.6109e+03_rb,4.2083e+03_rb,2.8058e+03_rb,1.4032e+03_rb,1.1770e+01_rb/)
      kbo(:, 4,34,11) = (/ &
     &5.9588e+03_rb,4.4693e+03_rb,2.9798e+03_rb,1.4902e+03_rb,1.1629e+01_rb/)
      kbo(:, 5,34,11) = (/ &
     &6.3300e+03_rb,4.7477e+03_rb,3.1654e+03_rb,1.5830e+03_rb,1.1520e+01_rb/)
      kbo(:, 1,35,11) = (/ &
     &5.2909e+03_rb,3.9683e+03_rb,2.6458e+03_rb,1.3232e+03_rb,1.1741e+01_rb/)
      kbo(:, 2,35,11) = (/ &
     &5.6548e+03_rb,4.2413e+03_rb,2.8277e+03_rb,1.4142e+03_rb,1.1903e+01_rb/)
      kbo(:, 3,35,11) = (/ &
     &6.0420e+03_rb,4.5317e+03_rb,3.0214e+03_rb,1.5110e+03_rb,1.1866e+01_rb/)
      kbo(:, 4,35,11) = (/ &
     &6.4601e+03_rb,4.8453e+03_rb,3.2304e+03_rb,1.6156e+03_rb,1.1803e+01_rb/)
      kbo(:, 5,35,11) = (/ &
     &6.9268e+03_rb,5.1953e+03_rb,3.4637e+03_rb,1.7322e+03_rb,1.1652e+01_rb/)
      kbo(:, 1,36,11) = (/ &
     &5.5884e+03_rb,4.1915e+03_rb,2.7945e+03_rb,1.3976e+03_rb,1.1878e+01_rb/)
      kbo(:, 2,36,11) = (/ &
     &6.0018e+03_rb,4.5015e+03_rb,3.0012e+03_rb,1.5010e+03_rb,1.2049e+01_rb/)
      kbo(:, 3,36,11) = (/ &
     &6.4517e+03_rb,4.8389e+03_rb,3.2262e+03_rb,1.6135e+03_rb,1.1951e+01_rb/)
      kbo(:, 4,36,11) = (/ &
     &6.9515e+03_rb,5.2138e+03_rb,3.4761e+03_rb,1.7384e+03_rb,1.1938e+01_rb/)
      kbo(:, 5,36,11) = (/ &
     &7.5289e+03_rb,5.6469e+03_rb,3.7648e+03_rb,1.8827e+03_rb,1.1892e+01_rb/)
      kbo(:, 1,37,11) = (/ &
     &5.7114e+03_rb,4.2838e+03_rb,2.8561e+03_rb,1.4284e+03_rb,1.1983e+01_rb/)
      kbo(:, 2,37,11) = (/ &
     &6.1629e+03_rb,4.6224e+03_rb,3.0818e+03_rb,1.5413e+03_rb,1.2105e+01_rb/)
      kbo(:, 3,37,11) = (/ &
     &6.6668e+03_rb,5.0003e+03_rb,3.3338e+03_rb,1.6673e+03_rb,1.2063e+01_rb/)
      kbo(:, 4,37,11) = (/ &
     &7.2441e+03_rb,5.4333e+03_rb,3.6224e+03_rb,1.8115e+03_rb,1.2103e+01_rb/)
      kbo(:, 5,37,11) = (/ &
     &7.9256e+03_rb,5.9443e+03_rb,3.9631e+03_rb,1.9819e+03_rb,1.2052e+01_rb/)
      kbo(:, 1,38,11) = (/ &
     &5.9457e+03_rb,4.4595e+03_rb,2.9732e+03_rb,1.4870e+03_rb,1.2040e+01_rb/)
      kbo(:, 2,38,11) = (/ &
     &6.4404e+03_rb,4.8305e+03_rb,3.2206e+03_rb,1.6107e+03_rb,1.2135e+01_rb/)
      kbo(:, 3,38,11) = (/ &
     &7.0103e+03_rb,5.2579e+03_rb,3.5055e+03_rb,1.7531e+03_rb,1.2201e+01_rb/)
      kbo(:, 4,38,11) = (/ &
     &7.6883e+03_rb,5.7664e+03_rb,3.8444e+03_rb,1.9225e+03_rb,1.2264e+01_rb/)
      kbo(:, 5,38,11) = (/ &
     &8.5003e+03_rb,6.3754e+03_rb,4.2504e+03_rb,2.1255e+03_rb,1.2214e+01_rb/)
      kbo(:, 1,39,11) = (/ &
     &6.3574e+03_rb,4.7683e+03_rb,3.1792e+03_rb,1.5900e+03_rb,1.2058e+01_rb/)
      kbo(:, 2,39,11) = (/ &
     &6.9172e+03_rb,5.1882e+03_rb,3.4591e+03_rb,1.7299e+03_rb,1.2221e+01_rb/)
      kbo(:, 3,39,11) = (/ &
     &7.5768e+03_rb,5.6828e+03_rb,3.7888e+03_rb,1.8948e+03_rb,1.2385e+01_rb/)
      kbo(:, 4,39,11) = (/ &
     &8.3852e+03_rb,6.2891e+03_rb,4.1930e+03_rb,2.0968e+03_rb,1.2417e+01_rb/)
      kbo(:, 5,39,11) = (/ &
     &9.3744e+03_rb,7.0310e+03_rb,4.6875e+03_rb,2.3441e+03_rb,1.2400e+01_rb/)
      kbo(:, 1,40,11) = (/ &
     &6.3544e+03_rb,4.7660e+03_rb,3.1776e+03_rb,1.5893e+03_rb,1.2034e+01_rb/)
      kbo(:, 2,40,11) = (/ &
     &6.9547e+03_rb,5.2162e+03_rb,3.4777e+03_rb,1.7393e+03_rb,1.2252e+01_rb/)
      kbo(:, 3,40,11) = (/ &
     &7.6538e+03_rb,5.7405e+03_rb,3.8272e+03_rb,1.9140e+03_rb,1.2461e+01_rb/)
      kbo(:, 4,40,11) = (/ &
     &8.5373e+03_rb,6.4031e+03_rb,4.2689e+03_rb,2.1348e+03_rb,1.2533e+01_rb/)
      kbo(:, 5,40,11) = (/ &
     &9.6388e+03_rb,7.2292e+03_rb,4.8196e+03_rb,2.4101e+03_rb,1.2526e+01_rb/)
      kbo(:, 1,41,11) = (/ &
     &6.3521e+03_rb,4.7643e+03_rb,3.1765e+03_rb,1.5887e+03_rb,1.1995e+01_rb/)
      kbo(:, 2,41,11) = (/ &
     &6.9920e+03_rb,5.2442e+03_rb,3.4964e+03_rb,1.7486e+03_rb,1.2280e+01_rb/)
      kbo(:, 3,41,11) = (/ &
     &7.7375e+03_rb,5.8033e+03_rb,3.8691e+03_rb,1.9349e+03_rb,1.2499e+01_rb/)
      kbo(:, 4,41,11) = (/ &
     &8.6900e+03_rb,6.5177e+03_rb,4.3453e+03_rb,2.1730e+03_rb,1.2628e+01_rb/)
      kbo(:, 5,41,11) = (/ &
     &9.9008e+03_rb,7.4257e+03_rb,4.9507e+03_rb,2.4756e+03_rb,1.2622e+01_rb/)
      kbo(:, 1,42,11) = (/ &
     &6.4377e+03_rb,4.8285e+03_rb,3.2193e+03_rb,1.6101e+03_rb,1.1943e+01_rb/)
      kbo(:, 2,42,11) = (/ &
     &7.1284e+03_rb,5.3465e+03_rb,3.5646e+03_rb,1.7827e+03_rb,1.2311e+01_rb/)
      kbo(:, 3,42,11) = (/ &
     &7.9472e+03_rb,5.9606e+03_rb,3.9739e+03_rb,1.9873e+03_rb,1.2580e+01_rb/)
      kbo(:, 4,42,11) = (/ &
     &8.9767e+03_rb,6.7326e+03_rb,4.4886e+03_rb,2.2446e+03_rb,1.2710e+01_rb/)
      kbo(:, 5,42,11) = (/ &
     &1.0302e+04_rb,7.7266e+03_rb,5.1512e+03_rb,2.5758e+03_rb,1.2734e+01_rb/)
      kbo(:, 1,43,11) = (/ &
     &6.5684e+03_rb,4.9266e+03_rb,3.2847e+03_rb,1.6428e+03_rb,1.1839e+01_rb/)
      kbo(:, 2,43,11) = (/ &
     &7.3100e+03_rb,5.4827e+03_rb,3.6554e+03_rb,1.8281e+03_rb,1.2298e+01_rb/)
      kbo(:, 3,43,11) = (/ &
     &8.2069e+03_rb,6.1553e+03_rb,4.1038e+03_rb,2.0523e+03_rb,1.2585e+01_rb/)
      kbo(:, 4,43,11) = (/ &
     &9.3215e+03_rb,6.9913e+03_rb,4.6611e+03_rb,2.3308e+03_rb,1.2796e+01_rb/)
      kbo(:, 5,43,11) = (/ &
     &1.0756e+04_rb,8.0671e+03_rb,5.3782e+03_rb,2.6894e+03_rb,1.2802e+01_rb/)
      kbo(:, 1,44,11) = (/ &
     &6.7770e+03_rb,5.0829e+03_rb,3.3889e+03_rb,1.6949e+03_rb,1.1696e+01_rb/)
      kbo(:, 2,44,11) = (/ &
     &7.6014e+03_rb,5.7012e+03_rb,3.8011e+03_rb,1.9009e+03_rb,1.2235e+01_rb/)
      kbo(:, 3,44,11) = (/ &
     &8.5905e+03_rb,6.4430e+03_rb,4.2955e+03_rb,2.1481e+03_rb,1.2598e+01_rb/)
      kbo(:, 4,44,11) = (/ &
     &9.8167e+03_rb,7.3626e+03_rb,4.9086e+03_rb,2.4546e+03_rb,1.2843e+01_rb/)
      kbo(:, 5,44,11) = (/ &
     &1.1370e+04_rb,8.5274e+03_rb,5.6850e+03_rb,2.8427e+03_rb,1.2862e+01_rb/)
      kbo(:, 1,45,11) = (/ &
     &7.1337e+03_rb,5.3505e+03_rb,3.5673e+03_rb,1.7841e+03_rb,1.1553e+01_rb/)
      kbo(:, 2,45,11) = (/ &
     &8.0813e+03_rb,6.0612e+03_rb,4.0411e+03_rb,2.0209e+03_rb,1.2213e+01_rb/)
      kbo(:, 3,45,11) = (/ &
     &9.1915e+03_rb,6.8938e+03_rb,4.5961e+03_rb,2.2984e+03_rb,1.2577e+01_rb/)
      kbo(:, 4,45,11) = (/ &
     &1.0572e+04_rb,7.9290e+03_rb,5.2862e+03_rb,2.6434e+03_rb,1.2862e+01_rb/)
      kbo(:, 5,45,11) = (/ &
     &1.2294e+04_rb,9.2203e+03_rb,6.1470e+03_rb,3.0738e+03_rb,1.2905e+01_rb/)
      kbo(:, 1,46,11) = (/ &
     &7.5574e+03_rb,5.6683e+03_rb,3.7792e+03_rb,1.8901e+03_rb,1.1372e+01_rb/)
      kbo(:, 2,46,11) = (/ &
     &8.6566e+03_rb,6.4927e+03_rb,4.3287e+03_rb,2.1648e+03_rb,1.2098e+01_rb/)
      kbo(:, 3,46,11) = (/ &
     &9.9311e+03_rb,7.4485e+03_rb,4.9659e+03_rb,2.4833e+03_rb,1.2508e+01_rb/)
      kbo(:, 4,46,11) = (/ &
     &1.1483e+04_rb,8.6121e+03_rb,5.7416e+03_rb,2.8711e+03_rb,1.2889e+01_rb/)
      kbo(:, 5,46,11) = (/ &
     &1.3417e+04_rb,1.0063e+04_rb,6.7088e+03_rb,3.3546e+03_rb,1.2962e+01_rb/)
      kbo(:, 1,47,11) = (/ &
     &7.8727e+03_rb,5.9048e+03_rb,3.9368e+03_rb,1.9689e+03_rb,1.1120e+01_rb/)
      kbo(:, 2,47,11) = (/ &
     &9.1356e+03_rb,6.8519e+03_rb,4.5682e+03_rb,2.2845e+03_rb,1.1918e+01_rb/)
      kbo(:, 3,47,11) = (/ &
     &1.0582e+04_rb,7.9368e+03_rb,5.2914e+03_rb,2.6461e+03_rb,1.2406e+01_rb/)
      kbo(:, 4,47,11) = (/ &
     &1.2301e+04_rb,9.2260e+03_rb,6.1509e+03_rb,3.0757e+03_rb,1.2816e+01_rb/)
      kbo(:, 5,47,11) = (/ &
     &1.4433e+04_rb,1.0825e+04_rb,7.2167e+03_rb,3.6086e+03_rb,1.2983e+01_rb/)
      kbo(:, 1,48,11) = (/ &
     &8.4035e+03_rb,6.3029e+03_rb,4.2022e+03_rb,2.1016e+03_rb,1.0897e+01_rb/)
      kbo(:, 2,48,11) = (/ &
     &9.8931e+03_rb,7.4200e+03_rb,4.9470e+03_rb,2.4739e+03_rb,1.1706e+01_rb/)
      kbo(:, 3,48,11) = (/ &
     &1.1589e+04_rb,8.6922e+03_rb,5.7950e+03_rb,2.8979e+03_rb,1.2313e+01_rb/)
      kbo(:, 4,48,11) = (/ &
     &1.3567e+04_rb,1.0175e+04_rb,6.7839e+03_rb,3.3922e+03_rb,1.2741e+01_rb/)
      kbo(:, 5,48,11) = (/ &
     &1.5977e+04_rb,1.1983e+04_rb,7.9888e+03_rb,3.9947e+03_rb,1.2994e+01_rb/)
      kbo(:, 1,49,11) = (/ &
     &9.3222e+03_rb,6.9919e+03_rb,4.6615e+03_rb,2.3312e+03_rb,1.0633e+01_rb/)
      kbo(:, 2,49,11) = (/ &
     &1.1123e+04_rb,8.3424e+03_rb,5.5619e+03_rb,2.7813e+03_rb,1.1478e+01_rb/)
      kbo(:, 3,49,11) = (/ &
     &1.3212e+04_rb,9.9089e+03_rb,6.6061e+03_rb,3.3034e+03_rb,1.2190e+01_rb/)
      kbo(:, 4,49,11) = (/ &
     &1.5603e+04_rb,1.1702e+04_rb,7.8017e+03_rb,3.9011e+03_rb,1.2697e+01_rb/)
      kbo(:, 5,49,11) = (/ &
     &1.8442e+04_rb,1.3832e+04_rb,9.2215e+03_rb,4.6110e+03_rb,1.3027e+01_rb/)
      kbo(:, 1,50,11) = (/ &
     &9.8729e+03_rb,7.4049e+03_rb,4.9369e+03_rb,2.4688e+03_rb,1.0346e+01_rb/)
      kbo(:, 2,50,11) = (/ &
     &1.1927e+04_rb,8.9452e+03_rb,5.9637e+03_rb,2.9822e+03_rb,1.1213e+01_rb/)
      kbo(:, 3,50,11) = (/ &
     &1.4376e+04_rb,1.0782e+04_rb,7.1884e+03_rb,3.5945e+03_rb,1.2042e+01_rb/)
      kbo(:, 4,50,11) = (/ &
     &1.7156e+04_rb,1.2867e+04_rb,8.5782e+03_rb,4.2894e+03_rb,1.2592e+01_rb/)
      kbo(:, 5,50,11) = (/ &
     &2.0387e+04_rb,1.5290e+04_rb,1.0194e+04_rb,5.0970e+03_rb,1.3035e+01_rb/)
      kbo(:, 1,51,11) = (/ &
     &1.0246e+04_rb,7.6846e+03_rb,5.1234e+03_rb,2.5621e+03_rb,1.0080e+01_rb/)
      kbo(:, 2,51,11) = (/ &
     &1.2504e+04_rb,9.3781e+03_rb,6.2523e+03_rb,3.1265e+03_rb,1.0959e+01_rb/)
      kbo(:, 3,51,11) = (/ &
     &1.5294e+04_rb,1.1471e+04_rb,7.6476e+03_rb,3.8241e+03_rb,1.1913e+01_rb/)
      kbo(:, 4,51,11) = (/ &
     &1.8483e+04_rb,1.3863e+04_rb,9.2420e+03_rb,4.6213e+03_rb,1.2523e+01_rb/)
      kbo(:, 5,51,11) = (/ &
     &2.2103e+04_rb,1.6577e+04_rb,1.1052e+04_rb,5.5261e+03_rb,1.3008e+01_rb/)
      kbo(:, 1,52,11) = (/ &
     &1.0919e+04_rb,8.1898e+03_rb,5.4601e+03_rb,2.7304e+03_rb,9.7365e+00_rb/)
      kbo(:, 2,52,11) = (/ &
     &1.3474e+04_rb,1.0106e+04_rb,6.7375e+03_rb,3.3691e+03_rb,1.0715e+01_rb/)
      kbo(:, 3,52,11) = (/ &
     &1.6692e+04_rb,1.2519e+04_rb,8.3465e+03_rb,4.1736e+03_rb,1.1714e+01_rb/)
      kbo(:, 4,52,11) = (/ &
     &2.0445e+04_rb,1.5334e+04_rb,1.0223e+04_rb,5.1117e+03_rb,1.2415e+01_rb/)
      kbo(:, 5,52,11) = (/ &
     &2.4642e+04_rb,1.8481e+04_rb,1.2321e+04_rb,6.1608e+03_rb,1.2964e+01_rb/)
      kbo(:, 1,53,11) = (/ &
     &1.2096e+04_rb,9.0720e+03_rb,6.0483e+03_rb,3.0245e+03_rb,9.3807e+00_rb/)
      kbo(:, 2,53,11) = (/ &
     &1.5115e+04_rb,1.1337e+04_rb,7.5582e+03_rb,3.7794e+03_rb,1.0467e+01_rb/)
      kbo(:, 3,53,11) = (/ &
     &1.8931e+04_rb,1.4199e+04_rb,9.4661e+03_rb,4.7334e+03_rb,1.1520e+01_rb/)
      kbo(:, 4,53,11) = (/ &
     &2.3491e+04_rb,1.7618e+04_rb,1.1746e+04_rb,5.8732e+03_rb,1.2292e+01_rb/)
      kbo(:, 5,53,11) = (/ &
     &2.8602e+04_rb,2.1452e+04_rb,1.4301e+04_rb,7.1510e+03_rb,1.2913e+01_rb/)
      kbo(:, 1,54,11) = (/ &
     &1.1628e+04_rb,8.7215e+03_rb,5.8146e+03_rb,2.9077e+03_rb,8.9653e+00_rb/)
      kbo(:, 2,54,11) = (/ &
     &1.4721e+04_rb,1.1041e+04_rb,7.3608e+03_rb,3.6808e+03_rb,1.0130e+01_rb/)
      kbo(:, 3,54,11) = (/ &
     &1.8638e+04_rb,1.3979e+04_rb,9.3193e+03_rb,4.6600e+03_rb,1.1195e+01_rb/)
      kbo(:, 4,54,11) = (/ &
     &2.3417e+04_rb,1.7563e+04_rb,1.1709e+04_rb,5.8547e+03_rb,1.2060e+01_rb/)
      kbo(:, 5,54,11) = (/ &
     &2.8828e+04_rb,2.1621e+04_rb,1.4414e+04_rb,7.2075e+03_rb,1.2739e+01_rb/)
      kbo(:, 1,55,11) = (/ &
     &1.0209e+04_rb,7.6567e+03_rb,5.1047e+03_rb,2.5527e+03_rb,8.5323e+00_rb/)
      kbo(:, 2,55,11) = (/ &
     &1.3095e+04_rb,9.8211e+03_rb,6.5477e+03_rb,3.2742e+03_rb,9.7667e+00_rb/)
      kbo(:, 3,55,11) = (/ &
     &1.6780e+04_rb,1.2585e+04_rb,8.3901e+03_rb,4.1954e+03_rb,1.0760e+01_rb/)
      kbo(:, 4,55,11) = (/ &
     &2.1308e+04_rb,1.5981e+04_rb,1.0654e+04_rb,5.3275e+03_rb,1.1787e+01_rb/)
      kbo(:, 5,55,11) = (/ &
     &2.6526e+04_rb,1.9895e+04_rb,1.3264e+04_rb,6.6321e+03_rb,1.2490e+01_rb/)
      kbo(:, 1,56,11) = (/ &
     &8.9242e+03_rb,6.6933e+03_rb,4.4624e+03_rb,2.2315e+03_rb,8.1192e+00_rb/)
      kbo(:, 2,56,11) = (/ &
     &1.1623e+04_rb,8.7170e+03_rb,5.8115e+03_rb,2.9061e+03_rb,9.3346e+00_rb/)
      kbo(:, 3,56,11) = (/ &
     &1.5096e+04_rb,1.1322e+04_rb,7.5482e+03_rb,3.7744e+03_rb,1.0408e+01_rb/)
      kbo(:, 4,56,11) = (/ &
     &1.9354e+04_rb,1.4515e+04_rb,9.6771e+03_rb,4.8388e+03_rb,1.1396e+01_rb/)
      kbo(:, 5,56,11) = (/ &
     &2.4338e+04_rb,1.8253e+04_rb,1.2169e+04_rb,6.0847e+03_rb,1.2212e+01_rb/)
      kbo(:, 1,57,11) = (/ &
     &7.7622e+03_rb,5.8218e+03_rb,3.8814e+03_rb,1.9410e+03_rb,7.7106e+00_rb/)
      kbo(:, 2,57,11) = (/ &
     &1.0290e+04_rb,7.7178e+03_rb,5.1454e+03_rb,2.5730e+03_rb,8.9363e+00_rb/)
      kbo(:, 3,57,11) = (/ &
     &1.3552e+04_rb,1.0164e+04_rb,6.7760e+03_rb,3.3882e+03_rb,1.0051e+01_rb/)
      kbo(:, 4,57,11) = (/ &
     &1.7543e+04_rb,1.3157e+04_rb,8.7715e+03_rb,4.3860e+03_rb,1.0990e+01_rb/)
      kbo(:, 5,57,11) = (/ &
     &2.2284e+04_rb,1.6713e+04_rb,1.1142e+04_rb,5.5714e+03_rb,1.1934e+01_rb/)
      kbo(:, 1,58,11) = (/ &
     &1.7767e+03_rb,1.6290e+03_rb,1.3968e+03_rb,9.7855e+02_rb,7.2858e+00_rb/)
      kbo(:, 2,58,11) = (/ &
     &2.4068e+03_rb,2.2067e+03_rb,1.8922e+03_rb,1.3254e+03_rb,8.5609e+00_rb/)
      kbo(:, 3,58,11) = (/ &
     &3.2122e+03_rb,2.9452e+03_rb,2.5253e+03_rb,1.7688e+03_rb,9.6597e+00_rb/)
      kbo(:, 4,58,11) = (/ &
     &4.2027e+03_rb,3.8533e+03_rb,3.3039e+03_rb,2.3142e+03_rb,1.0637e+01_rb/)
      kbo(:, 5,58,11) = (/ &
     &5.3882e+03_rb,4.9402e+03_rb,4.2359e+03_rb,2.9669e+03_rb,1.1542e+01_rb/)
      kbo(:, 1,59,11) = (/ &
     &1.9233e+03_rb,1.6794e+03_rb,1.3397e+03_rb,8.3393e+02_rb,7.1275e+00_rb/)
      kbo(:, 2,59,11) = (/ &
     &2.6585e+03_rb,2.3214e+03_rb,1.8518e+03_rb,1.1525e+03_rb,8.4302e+00_rb/)
      kbo(:, 3,59,11) = (/ &
     &3.5898e+03_rb,3.1345e+03_rb,2.5004e+03_rb,1.5561e+03_rb,9.5360e+00_rb/)
      kbo(:, 4,59,11) = (/ &
     &4.7292e+03_rb,4.1294e+03_rb,3.2940e+03_rb,2.0499e+03_rb,1.0472e+01_rb/)
      kbo(:, 5,59,11) = (/ &
     &6.0902e+03_rb,5.3178e+03_rb,4.2419e+03_rb,2.6398e+03_rb,1.1150e+01_rb/)
      kbo(:, 1,13,12) = (/ &
     &2.7240e+05_rb,2.0430e+05_rb,1.3620e+05_rb,6.8101e+04_rb,4.4161e+00_rb/)
      kbo(:, 2,13,12) = (/ &
     &2.7176e+05_rb,2.0382e+05_rb,1.3588e+05_rb,6.7942e+04_rb,4.2840e+00_rb/)
      kbo(:, 3,13,12) = (/ &
     &2.7001e+05_rb,2.0251e+05_rb,1.3501e+05_rb,6.7503e+04_rb,4.1173e+00_rb/)
      kbo(:, 4,13,12) = (/ &
     &2.6763e+05_rb,2.0073e+05_rb,1.3382e+05_rb,6.6909e+04_rb,3.9336e+00_rb/)
      kbo(:, 5,13,12) = (/ &
     &2.6442e+05_rb,1.9831e+05_rb,1.3221e+05_rb,6.6105e+04_rb,3.6961e+00_rb/)
      kbo(:, 1,14,12) = (/ &
     &1.8113e+05_rb,1.3585e+05_rb,9.0565e+04_rb,4.5283e+04_rb,4.8454e+00_rb/)
      kbo(:, 2,14,12) = (/ &
     &1.8030e+05_rb,1.3523e+05_rb,9.0150e+04_rb,4.5075e+04_rb,4.6873e+00_rb/)
      kbo(:, 3,14,12) = (/ &
     &1.7856e+05_rb,1.3392e+05_rb,8.9279e+04_rb,4.4640e+04_rb,4.6210e+00_rb/)
      kbo(:, 4,14,12) = (/ &
     &1.7661e+05_rb,1.3246e+05_rb,8.8303e+04_rb,4.4152e+04_rb,4.4506e+00_rb/)
      kbo(:, 5,14,12) = (/ &
     &1.7414e+05_rb,1.3060e+05_rb,8.7069e+04_rb,4.3535e+04_rb,4.3231e+00_rb/)
      kbo(:, 1,15,12) = (/ &
     &1.2085e+05_rb,9.0635e+04_rb,6.0424e+04_rb,3.0213e+04_rb,5.3476e+00_rb/)
      kbo(:, 2,15,12) = (/ &
     &1.1987e+05_rb,8.9903e+04_rb,5.9936e+04_rb,2.9969e+04_rb,5.2285e+00_rb/)
      kbo(:, 3,15,12) = (/ &
     &1.1863e+05_rb,8.8976e+04_rb,5.9318e+04_rb,2.9660e+04_rb,5.0918e+00_rb/)
      kbo(:, 4,15,12) = (/ &
     &1.1686e+05_rb,8.7649e+04_rb,5.8433e+04_rb,2.9217e+04_rb,4.9903e+00_rb/)
      kbo(:, 5,15,12) = (/ &
     &1.1485e+05_rb,8.6139e+04_rb,5.7427e+04_rb,2.8714e+04_rb,4.7856e+00_rb/)
      kbo(:, 1,16,12) = (/ &
     &9.4371e+04_rb,7.0779e+04_rb,4.7186e+04_rb,2.3593e+04_rb,5.9193e+00_rb/)
      kbo(:, 2,16,12) = (/ &
     &9.3237e+04_rb,6.9928e+04_rb,4.6619e+04_rb,2.3310e+04_rb,5.7778e+00_rb/)
      kbo(:, 3,16,12) = (/ &
     &9.1868e+04_rb,6.8901e+04_rb,4.5935e+04_rb,2.2968e+04_rb,5.6214e+00_rb/)
      kbo(:, 4,16,12) = (/ &
     &9.0372e+04_rb,6.7780e+04_rb,4.5187e+04_rb,2.2594e+04_rb,5.5103e+00_rb/)
      kbo(:, 5,16,12) = (/ &
     &8.8699e+04_rb,6.6525e+04_rb,4.4350e+04_rb,2.2176e+04_rb,5.3472e+00_rb/)
      kbo(:, 1,17,12) = (/ &
     &7.4737e+04_rb,5.6053e+04_rb,3.7369e+04_rb,1.8685e+04_rb,6.5296e+00_rb/)
      kbo(:, 2,17,12) = (/ &
     &7.3678e+04_rb,5.5259e+04_rb,3.6840e+04_rb,1.8420e+04_rb,6.3899e+00_rb/)
      kbo(:, 3,17,12) = (/ &
     &7.2463e+04_rb,5.4347e+04_rb,3.6232e+04_rb,1.8116e+04_rb,6.2234e+00_rb/)
      kbo(:, 4,17,12) = (/ &
     &7.1178e+04_rb,5.3383e+04_rb,3.5589e+04_rb,1.7795e+04_rb,6.1262e+00_rb/)
      kbo(:, 5,17,12) = (/ &
     &6.9846e+04_rb,5.2385e+04_rb,3.4923e+04_rb,1.7462e+04_rb,6.0038e+00_rb/)
      kbo(:, 1,18,12) = (/ &
     &5.9561e+04_rb,4.4671e+04_rb,2.9781e+04_rb,1.4891e+04_rb,7.2106e+00_rb/)
      kbo(:, 2,18,12) = (/ &
     &5.8573e+04_rb,4.3930e+04_rb,2.9287e+04_rb,1.4644e+04_rb,7.0772e+00_rb/)
      kbo(:, 3,18,12) = (/ &
     &5.7514e+04_rb,4.3136e+04_rb,2.8757e+04_rb,1.4379e+04_rb,6.9067e+00_rb/)
      kbo(:, 4,18,12) = (/ &
     &5.6474e+04_rb,4.2356e+04_rb,2.8238e+04_rb,1.4119e+04_rb,6.7942e+00_rb/)
      kbo(:, 5,18,12) = (/ &
     &5.5403e+04_rb,4.1552e+04_rb,2.7702e+04_rb,1.3851e+04_rb,6.6477e+00_rb/)
      kbo(:, 1,19,12) = (/ &
     &4.5689e+04_rb,3.4267e+04_rb,2.2845e+04_rb,1.1423e+04_rb,7.9750e+00_rb/)
      kbo(:, 2,19,12) = (/ &
     &4.4876e+04_rb,3.3657e+04_rb,2.2438e+04_rb,1.1220e+04_rb,7.8278e+00_rb/)
      kbo(:, 3,19,12) = (/ &
     &4.4040e+04_rb,3.3030e+04_rb,2.2021e+04_rb,1.1011e+04_rb,7.6237e+00_rb/)
      kbo(:, 4,19,12) = (/ &
     &4.3227e+04_rb,3.2420e+04_rb,2.1614e+04_rb,1.0807e+04_rb,7.5070e+00_rb/)
      kbo(:, 5,19,12) = (/ &
     &4.2374e+04_rb,3.1781e+04_rb,2.1187e+04_rb,1.0594e+04_rb,7.3430e+00_rb/)
      kbo(:, 1,20,12) = (/ &
     &3.7246e+04_rb,2.7935e+04_rb,1.8623e+04_rb,9.3122e+03_rb,8.8153e+00_rb/)
      kbo(:, 2,20,12) = (/ &
     &3.6550e+04_rb,2.7413e+04_rb,1.8276e+04_rb,9.1383e+03_rb,8.6532e+00_rb/)
      kbo(:, 3,20,12) = (/ &
     &3.5840e+04_rb,2.6880e+04_rb,1.7920e+04_rb,8.9607e+03_rb,8.4391e+00_rb/)
      kbo(:, 4,20,12) = (/ &
     &3.5149e+04_rb,2.6362e+04_rb,1.7575e+04_rb,8.7878e+03_rb,8.2995e+00_rb/)
      kbo(:, 5,20,12) = (/ &
     &3.4405e+04_rb,2.5804e+04_rb,1.7203e+04_rb,8.6018e+03_rb,8.1167e+00_rb/)
      kbo(:, 1,21,12) = (/ &
     &3.0709e+04_rb,2.3032e+04_rb,1.5355e+04_rb,7.6780e+03_rb,9.7372e+00_rb/)
      kbo(:, 2,21,12) = (/ &
     &3.0108e+04_rb,2.2581e+04_rb,1.5055e+04_rb,7.5277e+03_rb,9.5645e+00_rb/)
      kbo(:, 3,21,12) = (/ &
     &2.9509e+04_rb,2.2132e+04_rb,1.4755e+04_rb,7.3780e+03_rb,9.3267e+00_rb/)
      kbo(:, 4,21,12) = (/ &
     &2.8892e+04_rb,2.1670e+04_rb,1.4447e+04_rb,7.2238e+03_rb,9.1690e+00_rb/)
      kbo(:, 5,21,12) = (/ &
     &2.8241e+04_rb,2.1181e+04_rb,1.4121e+04_rb,7.0607e+03_rb,8.9671e+00_rb/)
      kbo(:, 1,22,12) = (/ &
     &2.5484e+04_rb,1.9113e+04_rb,1.2742e+04_rb,6.3717e+03_rb,1.0681e+01_rb/)
      kbo(:, 2,22,12) = (/ &
     &2.4964e+04_rb,1.8723e+04_rb,1.2483e+04_rb,6.2417e+03_rb,1.0497e+01_rb/)
      kbo(:, 3,22,12) = (/ &
     &2.4423e+04_rb,1.8317e+04_rb,1.2212e+04_rb,6.1064e+03_rb,1.0234e+01_rb/)
      kbo(:, 4,22,12) = (/ &
     &2.3869e+04_rb,1.7902e+04_rb,1.1935e+04_rb,5.9678e+03_rb,1.0059e+01_rb/)
      kbo(:, 5,22,12) = (/ &
     &2.3286e+04_rb,1.7465e+04_rb,1.1643e+04_rb,5.8221e+03_rb,9.8315e+00_rb/)
      kbo(:, 1,23,12) = (/ &
     &2.0918e+04_rb,1.5688e+04_rb,1.0459e+04_rb,5.2302e+03_rb,1.1614e+01_rb/)
      kbo(:, 2,23,12) = (/ &
     &2.0451e+04_rb,1.5339e+04_rb,1.0226e+04_rb,5.1135e+03_rb,1.1417e+01_rb/)
      kbo(:, 3,23,12) = (/ &
     &1.9969e+04_rb,1.4977e+04_rb,9.9848e+03_rb,4.9928e+03_rb,1.1125e+01_rb/)
      kbo(:, 4,23,12) = (/ &
     &1.9481e+04_rb,1.4611e+04_rb,9.7407e+03_rb,4.8708e+03_rb,1.0942e+01_rb/)
      kbo(:, 5,23,12) = (/ &
     &1.8998e+04_rb,1.4249e+04_rb,9.4994e+03_rb,4.7501e+03_rb,1.0699e+01_rb/)
      kbo(:, 1,24,12) = (/ &
     &1.6828e+04_rb,1.2621e+04_rb,8.4145e+03_rb,4.2078e+03_rb,1.2522e+01_rb/)
      kbo(:, 2,24,12) = (/ &
     &1.6415e+04_rb,1.2311e+04_rb,8.2079e+03_rb,4.1044e+03_rb,1.2299e+01_rb/)
      kbo(:, 3,24,12) = (/ &
     &1.6005e+04_rb,1.2004e+04_rb,8.0027e+03_rb,4.0018e+03_rb,1.2004e+01_rb/)
      kbo(:, 4,24,12) = (/ &
     &1.5632e+04_rb,1.1724e+04_rb,7.8164e+03_rb,3.9086e+03_rb,1.1796e+01_rb/)
      kbo(:, 5,24,12) = (/ &
     &1.5350e+04_rb,1.1513e+04_rb,7.6755e+03_rb,3.8381e+03_rb,1.1532e+01_rb/)
      kbo(:, 1,25,12) = (/ &
     &1.3517e+04_rb,1.0138e+04_rb,6.7590e+03_rb,3.3800e+03_rb,1.3378e+01_rb/)
      kbo(:, 2,25,12) = (/ &
     &1.3176e+04_rb,9.8826e+03_rb,6.5887e+03_rb,3.2948e+03_rb,1.3150e+01_rb/)
      kbo(:, 3,25,12) = (/ &
     &1.2900e+04_rb,9.6753e+03_rb,6.4505e+03_rb,3.2257e+03_rb,1.2858e+01_rb/)
      kbo(:, 4,25,12) = (/ &
     &1.2719e+04_rb,9.5392e+03_rb,6.3597e+03_rb,3.1803e+03_rb,1.2628e+01_rb/)
      kbo(:, 5,25,12) = (/ &
     &1.2623e+04_rb,9.4672e+03_rb,6.3117e+03_rb,3.1562e+03_rb,1.2342e+01_rb/)
      kbo(:, 1,26,12) = (/ &
     &1.0917e+04_rb,8.1877e+03_rb,5.4588e+03_rb,2.7299e+03_rb,1.4213e+01_rb/)
      kbo(:, 2,26,12) = (/ &
     &1.0711e+04_rb,8.0332e+03_rb,5.3557e+03_rb,2.6783e+03_rb,1.3979e+01_rb/)
      kbo(:, 3,26,12) = (/ &
     &1.0602e+04_rb,7.9514e+03_rb,5.3012e+03_rb,2.6510e+03_rb,1.3677e+01_rb/)
      kbo(:, 4,26,12) = (/ &
     &1.0604e+04_rb,7.9535e+03_rb,5.3026e+03_rb,2.6517e+03_rb,1.3410e+01_rb/)
      kbo(:, 5,26,12) = (/ &
     &1.0730e+04_rb,8.0477e+03_rb,5.3654e+03_rb,2.6831e+03_rb,1.3099e+01_rb/)
      kbo(:, 1,27,12) = (/ &
     &9.1749e+03_rb,6.8814e+03_rb,4.5879e+03_rb,2.2944e+03_rb,1.5000e+01_rb/)
      kbo(:, 2,27,12) = (/ &
     &9.1157e+03_rb,6.8370e+03_rb,4.5583e+03_rb,2.2796e+03_rb,1.4706e+01_rb/)
      kbo(:, 3,27,12) = (/ &
     &9.1873e+03_rb,6.8907e+03_rb,4.5941e+03_rb,2.2975e+03_rb,1.4431e+01_rb/)
      kbo(:, 4,27,12) = (/ &
     &9.3840e+03_rb,7.0382e+03_rb,4.6924e+03_rb,2.3467e+03_rb,1.4137e+01_rb/)
      kbo(:, 5,27,12) = (/ &
     &9.6488e+03_rb,7.2368e+03_rb,4.8249e+03_rb,2.4129e+03_rb,1.3775e+01_rb/)
      kbo(:, 1,28,12) = (/ &
     &8.0307e+03_rb,6.0232e+03_rb,4.0158e+03_rb,2.0083e+03_rb,1.5711e+01_rb/)
      kbo(:, 2,28,12) = (/ &
     &8.1446e+03_rb,6.1087e+03_rb,4.0727e+03_rb,2.0368e+03_rb,1.5384e+01_rb/)
      kbo(:, 3,28,12) = (/ &
     &8.3784e+03_rb,6.2841e+03_rb,4.1897e+03_rb,2.0953e+03_rb,1.5123e+01_rb/)
      kbo(:, 4,28,12) = (/ &
     &8.6648e+03_rb,6.4989e+03_rb,4.3329e+03_rb,2.1669e+03_rb,1.4791e+01_rb/)
      kbo(:, 5,28,12) = (/ &
     &9.0065e+03_rb,6.7551e+03_rb,4.5037e+03_rb,2.2523e+03_rb,1.4343e+01_rb/)
      kbo(:, 1,29,12) = (/ &
     &7.6471e+03_rb,5.7356e+03_rb,3.8240e+03_rb,1.9124e+03_rb,1.6341e+01_rb/)
      kbo(:, 2,29,12) = (/ &
     &7.9071e+03_rb,5.9306e+03_rb,3.9540e+03_rb,1.9775e+03_rb,1.6013e+01_rb/)
      kbo(:, 3,29,12) = (/ &
     &8.2292e+03_rb,6.1721e+03_rb,4.1151e+03_rb,2.0580e+03_rb,1.5732e+01_rb/)
      kbo(:, 4,29,12) = (/ &
     &8.6240e+03_rb,6.4682e+03_rb,4.3125e+03_rb,2.1567e+03_rb,1.5372e+01_rb/)
      kbo(:, 5,29,12) = (/ &
     &9.0598e+03_rb,6.7951e+03_rb,4.5304e+03_rb,2.2657e+03_rb,1.4767e+01_rb/)
      kbo(:, 1,30,12) = (/ &
     &7.6323e+03_rb,5.7245e+03_rb,3.8166e+03_rb,1.9088e+03_rb,1.6832e+01_rb/)
      kbo(:, 2,30,12) = (/ &
     &7.9856e+03_rb,5.9894e+03_rb,3.9933e+03_rb,1.9972e+03_rb,1.6592e+01_rb/)
      kbo(:, 3,30,12) = (/ &
     &8.4256e+03_rb,6.3195e+03_rb,4.2133e+03_rb,2.1072e+03_rb,1.6260e+01_rb/)
      kbo(:, 4,30,12) = (/ &
     &8.9027e+03_rb,6.6772e+03_rb,4.4518e+03_rb,2.2264e+03_rb,1.5767e+01_rb/)
      kbo(:, 5,30,12) = (/ &
     &9.4138e+03_rb,7.0606e+03_rb,4.7073e+03_rb,2.3541e+03_rb,1.5140e+01_rb/)
      kbo(:, 1,31,12) = (/ &
     &8.1459e+03_rb,6.1097e+03_rb,4.0735e+03_rb,2.0373e+03_rb,1.7334e+01_rb/)
      kbo(:, 2,31,12) = (/ &
     &8.6243e+03_rb,6.4685e+03_rb,4.3127e+03_rb,2.1569e+03_rb,1.7064e+01_rb/)
      kbo(:, 3,31,12) = (/ &
     &9.1518e+03_rb,6.8642e+03_rb,4.5764e+03_rb,2.2887e+03_rb,1.6609e+01_rb/)
      kbo(:, 4,31,12) = (/ &
     &9.7356e+03_rb,7.3019e+03_rb,4.8683e+03_rb,2.4346e+03_rb,1.5848e+01_rb/)
      kbo(:, 5,31,12) = (/ &
     &1.0365e+04_rb,7.7736e+03_rb,5.1827e+03_rb,2.5918e+03_rb,1.5376e+01_rb/)
      kbo(:, 1,32,12) = (/ &
     &8.9083e+03_rb,6.6815e+03_rb,4.4547e+03_rb,2.2279e+03_rb,1.7730e+01_rb/)
      kbo(:, 2,32,12) = (/ &
     &9.4979e+03_rb,7.1237e+03_rb,4.7495e+03_rb,2.3753e+03_rb,1.7331e+01_rb/)
      kbo(:, 3,32,12) = (/ &
     &1.0147e+04_rb,7.6108e+03_rb,5.0742e+03_rb,2.5376e+03_rb,1.6589e+01_rb/)
      kbo(:, 4,32,12) = (/ &
     &1.0867e+04_rb,8.1507e+03_rb,5.4341e+03_rb,2.7175e+03_rb,1.6156e+01_rb/)
      kbo(:, 5,32,12) = (/ &
     &1.1646e+04_rb,8.7350e+03_rb,5.8236e+03_rb,2.9122e+03_rb,1.5848e+01_rb/)
      kbo(:, 1,33,12) = (/ &
     &9.9979e+03_rb,7.4987e+03_rb,4.9995e+03_rb,2.5003e+03_rb,1.7989e+01_rb/)
      kbo(:, 2,33,12) = (/ &
     &1.0743e+04_rb,8.0578e+03_rb,5.3722e+03_rb,2.6866e+03_rb,1.7365e+01_rb/)
      kbo(:, 3,33,12) = (/ &
     &1.1568e+04_rb,8.6761e+03_rb,5.7844e+03_rb,2.8926e+03_rb,1.6901e+01_rb/)
      kbo(:, 4,33,12) = (/ &
     &1.2466e+04_rb,9.3497e+03_rb,6.2334e+03_rb,3.1171e+03_rb,1.6592e+01_rb/)
      kbo(:, 5,33,12) = (/ &
     &1.3441e+04_rb,1.0081e+04_rb,6.7208e+03_rb,3.3608e+03_rb,1.6160e+01_rb/)
      kbo(:, 1,34,12) = (/ &
     &1.1005e+04_rb,8.2543e+03_rb,5.5032e+03_rb,2.7521e+03_rb,1.8094e+01_rb/)
      kbo(:, 2,34,12) = (/ &
     &1.1925e+04_rb,8.9436e+03_rb,5.9627e+03_rb,2.9818e+03_rb,1.7503e+01_rb/)
      kbo(:, 3,34,12) = (/ &
     &1.2929e+04_rb,9.6970e+03_rb,6.4650e+03_rb,3.2329e+03_rb,1.7258e+01_rb/)
      kbo(:, 4,34,12) = (/ &
     &1.4013e+04_rb,1.0510e+04_rb,7.0067e+03_rb,3.5037e+03_rb,1.6964e+01_rb/)
      kbo(:, 5,34,12) = (/ &
     &1.5204e+04_rb,1.1403e+04_rb,7.6022e+03_rb,3.8015e+03_rb,1.6506e+01_rb/)
      kbo(:, 1,35,12) = (/ &
     &1.2135e+04_rb,9.1016e+03_rb,6.0680e+03_rb,3.0345e+03_rb,1.8232e+01_rb/)
      kbo(:, 2,35,12) = (/ &
     &1.3244e+04_rb,9.9331e+03_rb,6.6223e+03_rb,3.3116e+03_rb,1.7908e+01_rb/)
      kbo(:, 3,35,12) = (/ &
     &1.4457e+04_rb,1.0843e+04_rb,7.2288e+03_rb,3.6148e+03_rb,1.7659e+01_rb/)
      kbo(:, 4,35,12) = (/ &
     &1.5776e+04_rb,1.1833e+04_rb,7.8886e+03_rb,3.9447e+03_rb,1.7305e+01_rb/)
      kbo(:, 5,35,12) = (/ &
     &1.7218e+04_rb,1.2914e+04_rb,8.6094e+03_rb,4.3051e+03_rb,1.6889e+01_rb/)
      kbo(:, 1,36,12) = (/ &
     &1.3247e+04_rb,9.9356e+03_rb,6.6240e+03_rb,3.3125e+03_rb,1.8488e+01_rb/)
      kbo(:, 2,36,12) = (/ &
     &1.4560e+04_rb,1.0920e+04_rb,7.2802e+03_rb,3.6405e+03_rb,1.8270e+01_rb/)
      kbo(:, 3,36,12) = (/ &
     &1.6016e+04_rb,1.2012e+04_rb,8.0085e+03_rb,4.0046e+03_rb,1.8063e+01_rb/)
      kbo(:, 4,36,12) = (/ &
     &1.7602e+04_rb,1.3202e+04_rb,8.8012e+03_rb,4.4010e+03_rb,1.7553e+01_rb/)
      kbo(:, 5,36,12) = (/ &
     &1.9328e+04_rb,1.4496e+04_rb,9.6641e+03_rb,4.8324e+03_rb,1.7048e+01_rb/)
      kbo(:, 1,37,12) = (/ &
     &1.3952e+04_rb,1.0464e+04_rb,6.9765e+03_rb,3.4887e+03_rb,1.8801e+01_rb/)
      kbo(:, 2,37,12) = (/ &
     &1.5457e+04_rb,1.1593e+04_rb,7.7287e+03_rb,3.8648e+03_rb,1.8637e+01_rb/)
      kbo(:, 3,37,12) = (/ &
     &1.7141e+04_rb,1.2856e+04_rb,8.5707e+03_rb,4.2857e+03_rb,1.8420e+01_rb/)
      kbo(:, 4,37,12) = (/ &
     &1.8985e+04_rb,1.4239e+04_rb,9.4928e+03_rb,4.7467e+03_rb,1.7805e+01_rb/)
      kbo(:, 5,37,12) = (/ &
     &2.0966e+04_rb,1.5724e+04_rb,1.0483e+04_rb,5.2419e+03_rb,1.7343e+01_rb/)
      kbo(:, 1,38,12) = (/ &
     &1.4936e+04_rb,1.1202e+04_rb,7.4684e+03_rb,3.7346e+03_rb,1.9067e+01_rb/)
      kbo(:, 2,38,12) = (/ &
     &1.6693e+04_rb,1.2520e+04_rb,8.3467e+03_rb,4.1737e+03_rb,1.8968e+01_rb/)
      kbo(:, 3,38,12) = (/ &
     &1.8667e+04_rb,1.4000e+04_rb,9.3337e+03_rb,4.6672e+03_rb,1.8667e+01_rb/)
      kbo(:, 4,38,12) = (/ &
     &2.0838e+04_rb,1.5628e+04_rb,1.0419e+04_rb,5.2098e+03_rb,1.8090e+01_rb/)
      kbo(:, 5,38,12) = (/ &
     &2.3152e+04_rb,1.7364e+04_rb,1.1576e+04_rb,5.7883e+03_rb,1.7634e+01_rb/)
      kbo(:, 1,39,12) = (/ &
     &1.6411e+04_rb,1.2309e+04_rb,8.2062e+03_rb,4.1035e+03_rb,1.9362e+01_rb/)
      kbo(:, 2,39,12) = (/ &
     &1.8520e+04_rb,1.3890e+04_rb,9.2603e+03_rb,4.6305e+03_rb,1.9256e+01_rb/)
      kbo(:, 3,39,12) = (/ &
     &2.0878e+04_rb,1.5659e+04_rb,1.0439e+04_rb,5.2200e+03_rb,1.8842e+01_rb/)
      kbo(:, 4,39,12) = (/ &
     &2.3481e+04_rb,1.7611e+04_rb,1.1741e+04_rb,5.8709e+03_rb,1.8427e+01_rb/)
      kbo(:, 5,39,12) = (/ &
     &2.6258e+04_rb,1.9693e+04_rb,1.3129e+04_rb,6.5649e+03_rb,1.8005e+01_rb/)
      kbo(:, 1,40,12) = (/ &
     &1.6779e+04_rb,1.2584e+04_rb,8.3898e+03_rb,4.1952e+03_rb,1.9573e+01_rb/)
      kbo(:, 2,40,12) = (/ &
     &1.9118e+04_rb,1.4339e+04_rb,9.5593e+03_rb,4.7800e+03_rb,1.9496e+01_rb/)
      kbo(:, 3,40,12) = (/ &
     &2.1747e+04_rb,1.6310e+04_rb,1.0874e+04_rb,5.4372e+03_rb,1.9078e+01_rb/)
      kbo(:, 4,40,12) = (/ &
     &2.4641e+04_rb,1.8481e+04_rb,1.2321e+04_rb,6.1605e+03_rb,1.8680e+01_rb/)
      kbo(:, 5,40,12) = (/ &
     &2.7741e+04_rb,2.0806e+04_rb,1.3871e+04_rb,6.9356e+03_rb,1.8305e+01_rb/)
      kbo(:, 1,41,12) = (/ &
     &1.7108e+04_rb,1.2831e+04_rb,8.5543e+03_rb,4.2775e+03_rb,1.9752e+01_rb/)
      kbo(:, 2,41,12) = (/ &
     &1.9696e+04_rb,1.4772e+04_rb,9.8483e+03_rb,4.9245e+03_rb,1.9665e+01_rb/)
      kbo(:, 3,41,12) = (/ &
     &2.2603e+04_rb,1.6953e+04_rb,1.1302e+04_rb,5.6514e+03_rb,1.9347e+01_rb/)
      kbo(:, 4,41,12) = (/ &
     &2.5797e+04_rb,1.9348e+04_rb,1.2899e+04_rb,6.4497e+03_rb,1.8925e+01_rb/)
      kbo(:, 5,41,12) = (/ &
     &2.9237e+04_rb,2.1928e+04_rb,1.4619e+04_rb,7.3098e+03_rb,1.8587e+01_rb/)
      kbo(:, 1,42,12) = (/ &
     &1.7650e+04_rb,1.3238e+04_rb,8.8253e+03_rb,4.4130e+03_rb,1.9913e+01_rb/)
      kbo(:, 2,42,12) = (/ &
     &2.0552e+04_rb,1.5414e+04_rb,1.0276e+04_rb,5.1385e+03_rb,1.9817e+01_rb/)
      kbo(:, 3,42,12) = (/ &
     &2.3780e+04_rb,1.7835e+04_rb,1.1890e+04_rb,5.9455e+03_rb,1.9487e+01_rb/)
      kbo(:, 4,42,12) = (/ &
     &2.7331e+04_rb,2.0498e+04_rb,1.3666e+04_rb,6.8331e+03_rb,1.9141e+01_rb/)
      kbo(:, 5,42,12) = (/ &
     &3.1174e+04_rb,2.3380e+04_rb,1.5587e+04_rb,7.7938e+03_rb,1.8842e+01_rb/)
      kbo(:, 1,43,12) = (/ &
     &1.8226e+04_rb,1.3670e+04_rb,9.1135e+03_rb,4.5571e+03_rb,2.0107e+01_rb/)
      kbo(:, 2,43,12) = (/ &
     &2.1466e+04_rb,1.6100e+04_rb,1.0734e+04_rb,5.3672e+03_rb,1.9969e+01_rb/)
      kbo(:, 3,43,12) = (/ &
     &2.5067e+04_rb,1.8800e+04_rb,1.2534e+04_rb,6.2672e+03_rb,1.9715e+01_rb/)
      kbo(:, 4,43,12) = (/ &
     &2.9022e+04_rb,2.1767e+04_rb,1.4512e+04_rb,7.2561e+03_rb,1.9331e+01_rb/)
      kbo(:, 5,43,12) = (/ &
     &3.3314e+04_rb,2.4986e+04_rb,1.6658e+04_rb,8.3291e+03_rb,1.9158e+01_rb/)
      kbo(:, 1,44,12) = (/ &
     &1.9005e+04_rb,1.4254e+04_rb,9.5029e+03_rb,4.7518e+03_rb,2.0271e+01_rb/)
      kbo(:, 2,44,12) = (/ &
     &2.2627e+04_rb,1.6970e+04_rb,1.1314e+04_rb,5.6570e+03_rb,2.0045e+01_rb/)
      kbo(:, 3,44,12) = (/ &
     &2.6689e+04_rb,2.0017e+04_rb,1.3345e+04_rb,6.6725e+03_rb,1.9909e+01_rb/)
      kbo(:, 4,44,12) = (/ &
     &3.1132e+04_rb,2.3349e+04_rb,1.5566e+04_rb,7.7832e+03_rb,1.9596e+01_rb/)
      kbo(:, 5,44,12) = (/ &
     &3.5955e+04_rb,2.6966e+04_rb,1.7978e+04_rb,8.9890e+03_rb,1.9454e+01_rb/)
      kbo(:, 1,45,12) = (/ &
     &2.0230e+04_rb,1.5173e+04_rb,1.0115e+04_rb,5.0580e+03_rb,2.0360e+01_rb/)
      kbo(:, 2,45,12) = (/ &
     &2.4315e+04_rb,1.8237e+04_rb,1.2158e+04_rb,6.0793e+03_rb,2.0061e+01_rb/)
      kbo(:, 3,45,12) = (/ &
     &2.8985e+04_rb,2.1739e+04_rb,1.4493e+04_rb,7.2467e+03_rb,2.0123e+01_rb/)
      kbo(:, 4,45,12) = (/ &
     &3.4069e+04_rb,2.5552e+04_rb,1.7035e+04_rb,8.5176e+03_rb,1.9825e+01_rb/)
      kbo(:, 5,45,12) = (/ &
     &3.9583e+04_rb,2.9687e+04_rb,1.9792e+04_rb,9.8961e+03_rb,1.9819e+01_rb/)
      kbo(:, 1,46,12) = (/ &
     &2.1692e+04_rb,1.6269e+04_rb,1.0846e+04_rb,5.4233e+03_rb,2.0397e+01_rb/)
      kbo(:, 2,46,12) = (/ &
     &2.6275e+04_rb,1.9706e+04_rb,1.3138e+04_rb,6.5691e+03_rb,2.0140e+01_rb/)
      kbo(:, 3,46,12) = (/ &
     &3.1629e+04_rb,2.3722e+04_rb,1.5815e+04_rb,7.9076e+03_rb,2.0308e+01_rb/)
      kbo(:, 4,46,12) = (/ &
     &3.7518e+04_rb,2.8139e+04_rb,1.8759e+04_rb,9.3799e+03_rb,2.0045e+01_rb/)
      kbo(:, 5,46,12) = (/ &
     &4.3856e+04_rb,3.2892e+04_rb,2.1928e+04_rb,1.0964e+04_rb,2.0102e+01_rb/)
      kbo(:, 1,47,12) = (/ &
     &2.2886e+04_rb,1.7165e+04_rb,1.1443e+04_rb,5.7220e+03_rb,2.0376e+01_rb/)
      kbo(:, 2,47,12) = (/ &
     &2.7889e+04_rb,2.0917e+04_rb,1.3945e+04_rb,6.9727e+03_rb,2.0171e+01_rb/)
      kbo(:, 3,47,12) = (/ &
     &3.3867e+04_rb,2.5401e+04_rb,1.6934e+04_rb,8.4673e+03_rb,2.0409e+01_rb/)
      kbo(:, 4,47,12) = (/ &
     &4.0597e+04_rb,3.0448e+04_rb,2.0299e+04_rb,1.0150e+04_rb,2.0291e+01_rb/)
      kbo(:, 5,47,12) = (/ &
     &4.7822e+04_rb,3.5866e+04_rb,2.3911e+04_rb,1.1956e+04_rb,2.0384e+01_rb/)
      kbo(:, 1,48,12) = (/ &
     &2.4867e+04_rb,1.8651e+04_rb,1.2434e+04_rb,6.2173e+03_rb,2.0295e+01_rb/)
      kbo(:, 2,48,12) = (/ &
     &3.0432e+04_rb,2.2824e+04_rb,1.5216e+04_rb,7.6085e+03_rb,2.0270e+01_rb/)
      kbo(:, 3,48,12) = (/ &
     &3.7235e+04_rb,2.7926e+04_rb,1.8618e+04_rb,9.3091e+03_rb,2.0484e+01_rb/)
      kbo(:, 4,48,12) = (/ &
     &4.5063e+04_rb,3.3797e+04_rb,2.2532e+04_rb,1.1266e+04_rb,2.0580e+01_rb/)
      kbo(:, 5,48,12) = (/ &
     &5.3530e+04_rb,4.0148e+04_rb,2.6765e+04_rb,1.3383e+04_rb,2.0621e+01_rb/)
      kbo(:, 1,49,12) = (/ &
     &2.8095e+04_rb,2.1071e+04_rb,1.4048e+04_rb,7.0242e+03_rb,2.0197e+01_rb/)
      kbo(:, 2,49,12) = (/ &
     &3.4618e+04_rb,2.5963e+04_rb,1.7309e+04_rb,8.6547e+03_rb,2.0319e+01_rb/)
      kbo(:, 3,49,12) = (/ &
     &4.2591e+04_rb,3.1943e+04_rb,2.1296e+04_rb,1.0648e+04_rb,2.0559e+01_rb/)
      kbo(:, 4,49,12) = (/ &
     &5.1945e+04_rb,3.8959e+04_rb,2.5973e+04_rb,1.2987e+04_rb,2.0853e+01_rb/)
      kbo(:, 5,49,12) = (/ &
     &6.2325e+04_rb,4.6744e+04_rb,3.1163e+04_rb,1.5581e+04_rb,2.0890e+01_rb/)
      kbo(:, 1,50,12) = (/ &
     &3.0354e+04_rb,2.2766e+04_rb,1.5178e+04_rb,7.5891e+03_rb,2.0062e+01_rb/)
      kbo(:, 2,50,12) = (/ &
     &3.7734e+04_rb,2.8300e+04_rb,1.8867e+04_rb,9.4338e+03_rb,2.0363e+01_rb/)
      kbo(:, 3,50,12) = (/ &
     &4.6589e+04_rb,3.4942e+04_rb,2.3295e+04_rb,1.1648e+04_rb,2.0543e+01_rb/)
      kbo(:, 4,50,12) = (/ &
     &5.7214e+04_rb,4.2910e+04_rb,2.8607e+04_rb,1.4304e+04_rb,2.0993e+01_rb/)
      kbo(:, 5,50,12) = (/ &
     &6.9238e+04_rb,5.1929e+04_rb,3.4619e+04_rb,1.7310e+04_rb,2.1035e+01_rb/)
      kbo(:, 1,51,12) = (/ &
     &3.2115e+04_rb,2.4086e+04_rb,1.6058e+04_rb,8.0292e+03_rb,1.9809e+01_rb/)
      kbo(:, 2,51,12) = (/ &
     &4.0277e+04_rb,3.0208e+04_rb,2.0139e+04_rb,1.0070e+04_rb,2.0313e+01_rb/)
      kbo(:, 3,51,12) = (/ &
     &4.9958e+04_rb,3.7469e+04_rb,2.4979e+04_rb,1.2490e+04_rb,2.0455e+01_rb/)
      kbo(:, 4,51,12) = (/ &
     &6.1681e+04_rb,4.6261e+04_rb,3.0841e+04_rb,1.5421e+04_rb,2.0964e+01_rb/)
      kbo(:, 5,51,12) = (/ &
     &7.5206e+04_rb,5.6405e+04_rb,3.7603e+04_rb,1.8802e+04_rb,2.1060e+01_rb/)
      kbo(:, 1,52,12) = (/ &
     &3.4912e+04_rb,2.6184e+04_rb,1.7456e+04_rb,8.7285e+03_rb,1.9582e+01_rb/)
      kbo(:, 2,52,12) = (/ &
     &4.4146e+04_rb,3.3110e+04_rb,2.2073e+04_rb,1.1037e+04_rb,2.0163e+01_rb/)
      kbo(:, 3,52,12) = (/ &
     &5.5147e+04_rb,4.1361e+04_rb,2.7574e+04_rb,1.3787e+04_rb,2.0386e+01_rb/)
      kbo(:, 4,52,12) = (/ &
     &6.8326e+04_rb,5.1245e+04_rb,3.4163e+04_rb,1.7082e+04_rb,2.0999e+01_rb/)
      kbo(:, 5,52,12) = (/ &
     &8.3853e+04_rb,6.2890e+04_rb,4.1927e+04_rb,2.0964e+04_rb,2.1183e+01_rb/)
      kbo(:, 1,53,12) = (/ &
     &3.9402e+04_rb,2.9552e+04_rb,1.9702e+04_rb,9.8513e+03_rb,1.9267e+01_rb/)
      kbo(:, 2,53,12) = (/ &
     &5.0308e+04_rb,3.7732e+04_rb,2.5155e+04_rb,1.2578e+04_rb,1.9908e+01_rb/)
      kbo(:, 3,53,12) = (/ &
     &6.3383e+04_rb,4.7538e+04_rb,3.1692e+04_rb,1.5846e+04_rb,2.0279e+01_rb/)
      kbo(:, 4,53,12) = (/ &
     &7.8778e+04_rb,5.9084e+04_rb,3.9389e+04_rb,1.9695e+04_rb,2.1039e+01_rb/)
      kbo(:, 5,53,12) = (/ &
     &9.7229e+04_rb,7.2923e+04_rb,4.8615e+04_rb,2.4308e+04_rb,2.1293e+01_rb/)
      kbo(:, 1,54,12) = (/ &
     &3.8624e+04_rb,2.8968e+04_rb,1.9312e+04_rb,9.6566e+03_rb,1.8946e+01_rb/)
      kbo(:, 2,54,12) = (/ &
     &4.9869e+04_rb,3.7402e+04_rb,2.4935e+04_rb,1.2468e+04_rb,1.9724e+01_rb/)
      kbo(:, 3,54,12) = (/ &
     &6.3267e+04_rb,4.7450e+04_rb,3.1634e+04_rb,1.5817e+04_rb,2.0221e+01_rb/)
      kbo(:, 4,54,12) = (/ &
     &7.9005e+04_rb,5.9254e+04_rb,3.9503e+04_rb,1.9752e+04_rb,2.0891e+01_rb/)
      kbo(:, 5,54,12) = (/ &
     &9.7899e+04_rb,7.3425e+04_rb,4.8950e+04_rb,2.4475e+04_rb,2.1235e+01_rb/)
      kbo(:, 1,55,12) = (/ &
     &3.4504e+04_rb,2.5878e+04_rb,1.7253e+04_rb,8.6267e+03_rb,1.8561e+01_rb/)
      kbo(:, 2,55,12) = (/ &
     &4.5154e+04_rb,3.3866e+04_rb,2.2578e+04_rb,1.1289e+04_rb,1.9514e+01_rb/)
      kbo(:, 3,55,12) = (/ &
     &5.7700e+04_rb,4.3275e+04_rb,2.8850e+04_rb,1.4425e+04_rb,2.0143e+01_rb/)
      kbo(:, 4,55,12) = (/ &
     &7.2513e+04_rb,5.4385e+04_rb,3.6257e+04_rb,1.8129e+04_rb,2.0526e+01_rb/)
      kbo(:, 5,55,12) = (/ &
     &9.0097e+04_rb,6.7573e+04_rb,4.5049e+04_rb,2.2525e+04_rb,2.1145e+01_rb/)
      kbo(:, 1,56,12) = (/ &
     &3.0670e+04_rb,2.3003e+04_rb,1.5335e+04_rb,7.6681e+03_rb,1.8208e+01_rb/)
      kbo(:, 2,56,12) = (/ &
     &4.0775e+04_rb,3.0581e+04_rb,2.0388e+04_rb,1.0194e+04_rb,1.9345e+01_rb/)
      kbo(:, 3,56,12) = (/ &
     &5.2534e+04_rb,3.9401e+04_rb,2.6267e+04_rb,1.3134e+04_rb,1.9986e+01_rb/)
      kbo(:, 4,56,12) = (/ &
     &6.6490e+04_rb,4.9868e+04_rb,3.3245e+04_rb,1.6623e+04_rb,2.0300e+01_rb/)
      kbo(:, 5,56,12) = (/ &
     &8.2863e+04_rb,6.2147e+04_rb,4.1431e+04_rb,2.0716e+04_rb,2.0929e+01_rb/)
      kbo(:, 1,57,12) = (/ &
     &2.7134e+04_rb,2.0351e+04_rb,1.3567e+04_rb,6.7841e+03_rb,1.7766e+01_rb/)
      kbo(:, 2,57,12) = (/ &
     &3.6730e+04_rb,2.7547e+04_rb,1.8365e+04_rb,9.1827e+03_rb,1.8983e+01_rb/)
      kbo(:, 3,57,12) = (/ &
     &4.7798e+04_rb,3.5848e+04_rb,2.3899e+04_rb,1.1950e+04_rb,1.9837e+01_rb/)
      kbo(:, 4,57,12) = (/ &
     &6.0858e+04_rb,4.5644e+04_rb,3.0429e+04_rb,1.5215e+04_rb,2.0288e+01_rb/)
      kbo(:, 5,57,12) = (/ &
     &7.6170e+04_rb,5.7127e+04_rb,3.8085e+04_rb,1.9043e+04_rb,2.0539e+01_rb/)
      kbo(:, 1,58,12) = (/ &
     &6.3468e+03_rb,5.8191e+03_rb,4.9895e+03_rb,3.4949e+03_rb,1.7351e+01_rb/)
      kbo(:, 2,58,12) = (/ &
     &8.7238e+03_rb,7.9984e+03_rb,6.8580e+03_rb,4.8035e+03_rb,1.8673e+01_rb/)
      kbo(:, 3,58,12) = (/ &
     &1.1493e+04_rb,1.0537e+04_rb,9.0349e+03_rb,6.3281e+03_rb,1.9708e+01_rb/)
      kbo(:, 4,58,12) = (/ &
     &1.4729e+04_rb,1.3504e+04_rb,1.1579e+04_rb,8.1099e+03_rb,2.0210e+01_rb/)
      kbo(:, 5,58,12) = (/ &
     &1.8531e+04_rb,1.6990e+04_rb,1.4568e+04_rb,1.0203e+04_rb,2.0346e+01_rb/)
      kbo(:, 1,59,12) = (/ &
     &7.0593e+03_rb,6.1640e+03_rb,4.9170e+03_rb,3.0600e+03_rb,1.7147e+01_rb/)
      kbo(:, 2,59,12) = (/ &
     &9.7799e+03_rb,8.5396e+03_rb,6.8118e+03_rb,4.2390e+03_rb,1.8584e+01_rb/)
      kbo(:, 3,59,12) = (/ &
     &1.2964e+04_rb,1.1320e+04_rb,9.0293e+03_rb,5.6188e+03_rb,1.9434e+01_rb/)
      kbo(:, 4,59,12) = (/ &
     &1.6671e+04_rb,1.4556e+04_rb,1.1611e+04_rb,7.2253e+03_rb,2.0175e+01_rb/)
      kbo(:, 5,59,12) = (/ &
     &2.1035e+04_rb,1.8367e+04_rb,1.4651e+04_rb,9.1168e+03_rb,2.0516e+01_rb/)
      kbo(:, 1,13,13) = (/ &
     &3.8843e+05_rb,2.9132e+05_rb,1.9422e+05_rb,9.7109e+04_rb,5.4111e+00_rb/)
      kbo(:, 2,13,13) = (/ &
     &3.8116e+05_rb,2.8587e+05_rb,1.9058e+05_rb,9.5291e+04_rb,5.1021e+00_rb/)
      kbo(:, 3,13,13) = (/ &
     &3.7364e+05_rb,2.8023e+05_rb,1.8682e+05_rb,9.3410e+04_rb,4.8974e+00_rb/)
      kbo(:, 4,13,13) = (/ &
     &3.6551e+05_rb,2.7413e+05_rb,1.8276e+05_rb,9.1378e+04_rb,4.4934e+00_rb/)
      kbo(:, 5,13,13) = (/ &
     &3.5750e+05_rb,2.6813e+05_rb,1.7875e+05_rb,8.9377e+04_rb,3.9773e+00_rb/)
      kbo(:, 1,14,13) = (/ &
     &2.5525e+05_rb,1.9143e+05_rb,1.2762e+05_rb,6.3811e+04_rb,5.8071e+00_rb/)
      kbo(:, 2,14,13) = (/ &
     &2.5096e+05_rb,1.8822e+05_rb,1.2548e+05_rb,6.2740e+04_rb,5.6102e+00_rb/)
      kbo(:, 3,14,13) = (/ &
     &2.4685e+05_rb,1.8514e+05_rb,1.2343e+05_rb,6.1713e+04_rb,5.4048e+00_rb/)
      kbo(:, 4,14,13) = (/ &
     &2.4253e+05_rb,1.8190e+05_rb,1.2126e+05_rb,6.0632e+04_rb,5.2557e+00_rb/)
      kbo(:, 5,14,13) = (/ &
     &2.3845e+05_rb,1.7884e+05_rb,1.1922e+05_rb,5.9612e+04_rb,4.9044e+00_rb/)
      kbo(:, 1,15,13) = (/ &
     &1.7262e+05_rb,1.2947e+05_rb,8.6313e+04_rb,4.3157e+04_rb,6.3397e+00_rb/)
      kbo(:, 2,15,13) = (/ &
     &1.7029e+05_rb,1.2772e+05_rb,8.5146e+04_rb,4.2574e+04_rb,6.1559e+00_rb/)
      kbo(:, 3,15,13) = (/ &
     &1.6778e+05_rb,1.2583e+05_rb,8.3891e+04_rb,4.1946e+04_rb,5.9275e+00_rb/)
      kbo(:, 4,15,13) = (/ &
     &1.6564e+05_rb,1.2423e+05_rb,8.2824e+04_rb,4.1413e+04_rb,5.8142e+00_rb/)
      kbo(:, 5,15,13) = (/ &
     &1.6363e+05_rb,1.2273e+05_rb,8.1819e+04_rb,4.0910e+04_rb,5.6466e+00_rb/)
      kbo(:, 1,16,13) = (/ &
     &1.3945e+05_rb,1.0459e+05_rb,6.9724e+04_rb,3.4863e+04_rb,7.0725e+00_rb/)
      kbo(:, 2,16,13) = (/ &
     &1.3799e+05_rb,1.0349e+05_rb,6.8996e+04_rb,3.4499e+04_rb,6.9039e+00_rb/)
      kbo(:, 3,16,13) = (/ &
     &1.3667e+05_rb,1.0250e+05_rb,6.8336e+04_rb,3.4169e+04_rb,6.6526e+00_rb/)
      kbo(:, 4,16,13) = (/ &
     &1.3532e+05_rb,1.0149e+05_rb,6.7659e+04_rb,3.3830e+04_rb,6.5506e+00_rb/)
      kbo(:, 5,16,13) = (/ &
     &1.3401e+05_rb,1.0051e+05_rb,6.7008e+04_rb,3.3505e+04_rb,6.3946e+00_rb/)
      kbo(:, 1,17,13) = (/ &
     &1.1728e+05_rb,8.7962e+04_rb,5.8642e+04_rb,2.9322e+04_rb,7.9743e+00_rb/)
      kbo(:, 2,17,13) = (/ &
     &1.1635e+05_rb,8.7266e+04_rb,5.8178e+04_rb,2.9090e+04_rb,7.7873e+00_rb/)
      kbo(:, 3,17,13) = (/ &
     &1.1534e+05_rb,8.6504e+04_rb,5.7670e+04_rb,2.8835e+04_rb,7.5449e+00_rb/)
      kbo(:, 4,17,13) = (/ &
     &1.1432e+05_rb,8.5738e+04_rb,5.7159e+04_rb,2.8580e+04_rb,7.3894e+00_rb/)
      kbo(:, 5,17,13) = (/ &
     &1.1312e+05_rb,8.4838e+04_rb,5.6559e+04_rb,2.8280e+04_rb,7.2243e+00_rb/)
      kbo(:, 1,18,13) = (/ &
     &1.0020e+05_rb,7.5148e+04_rb,5.0099e+04_rb,2.5050e+04_rb,8.9803e+00_rb/)
      kbo(:, 2,18,13) = (/ &
     &9.9423e+04_rb,7.4567e+04_rb,4.9712e+04_rb,2.4856e+04_rb,8.7551e+00_rb/)
      kbo(:, 3,18,13) = (/ &
     &9.8635e+04_rb,7.3977e+04_rb,4.9318e+04_rb,2.4660e+04_rb,8.5049e+00_rb/)
      kbo(:, 4,18,13) = (/ &
     &9.7679e+04_rb,7.3260e+04_rb,4.8840e+04_rb,2.4421e+04_rb,8.3083e+00_rb/)
      kbo(:, 5,18,13) = (/ &
     &9.6616e+04_rb,7.2462e+04_rb,4.8308e+04_rb,2.4155e+04_rb,8.1253e+00_rb/)
      kbo(:, 1,19,13) = (/ &
     &8.3124e+04_rb,6.2343e+04_rb,4.1562e+04_rb,2.0782e+04_rb,1.0036e+01_rb/)
      kbo(:, 2,19,13) = (/ &
     &8.2434e+04_rb,6.1826e+04_rb,4.1217e+04_rb,2.0609e+04_rb,9.7882e+00_rb/)
      kbo(:, 3,19,13) = (/ &
     &8.1692e+04_rb,6.1269e+04_rb,4.0847e+04_rb,2.0424e+04_rb,9.5505e+00_rb/)
      kbo(:, 4,19,13) = (/ &
     &8.0821e+04_rb,6.0616e+04_rb,4.0411e+04_rb,2.0206e+04_rb,9.2995e+00_rb/)
      kbo(:, 5,19,13) = (/ &
     &7.9844e+04_rb,5.9883e+04_rb,3.9922e+04_rb,1.9962e+04_rb,9.1204e+00_rb/)
      kbo(:, 1,20,13) = (/ &
     &7.3434e+04_rb,5.5076e+04_rb,3.6717e+04_rb,1.8359e+04_rb,1.1173e+01_rb/)
      kbo(:, 2,20,13) = (/ &
     &7.2781e+04_rb,5.4586e+04_rb,3.6391e+04_rb,1.8196e+04_rb,1.0919e+01_rb/)
      kbo(:, 3,20,13) = (/ &
     &7.2073e+04_rb,5.4055e+04_rb,3.6037e+04_rb,1.8019e+04_rb,1.0676e+01_rb/)
      kbo(:, 4,20,13) = (/ &
     &7.1226e+04_rb,5.3419e+04_rb,3.5613e+04_rb,1.7807e+04_rb,1.0406e+01_rb/)
      kbo(:, 5,20,13) = (/ &
     &7.0326e+04_rb,5.2745e+04_rb,3.5164e+04_rb,1.7582e+04_rb,1.0205e+01_rb/)
      kbo(:, 1,21,13) = (/ &
     &6.5684e+04_rb,4.9263e+04_rb,3.2843e+04_rb,1.6422e+04_rb,1.2437e+01_rb/)
      kbo(:, 2,21,13) = (/ &
     &6.5060e+04_rb,4.8795e+04_rb,3.2531e+04_rb,1.6266e+04_rb,1.2175e+01_rb/)
      kbo(:, 3,21,13) = (/ &
     &6.4347e+04_rb,4.8261e+04_rb,3.2174e+04_rb,1.6088e+04_rb,1.1913e+01_rb/)
      kbo(:, 4,21,13) = (/ &
     &6.3551e+04_rb,4.7664e+04_rb,3.1776e+04_rb,1.5889e+04_rb,1.1602e+01_rb/)
      kbo(:, 5,21,13) = (/ &
     &6.2699e+04_rb,4.7025e+04_rb,3.1350e+04_rb,1.5676e+04_rb,1.1366e+01_rb/)
      kbo(:, 1,22,13) = (/ &
     &5.8992e+04_rb,4.4244e+04_rb,2.9497e+04_rb,1.4749e+04_rb,1.3817e+01_rb/)
      kbo(:, 2,22,13) = (/ &
     &5.8377e+04_rb,4.3783e+04_rb,2.9189e+04_rb,1.4595e+04_rb,1.3522e+01_rb/)
      kbo(:, 3,22,13) = (/ &
     &5.7726e+04_rb,4.3295e+04_rb,2.8863e+04_rb,1.4432e+04_rb,1.3217e+01_rb/)
      kbo(:, 4,22,13) = (/ &
     &5.7003e+04_rb,4.2753e+04_rb,2.8502e+04_rb,1.4252e+04_rb,1.2864e+01_rb/)
      kbo(:, 5,22,13) = (/ &
     &5.6242e+04_rb,4.2182e+04_rb,2.8121e+04_rb,1.4061e+04_rb,1.2581e+01_rb/)
      kbo(:, 1,23,13) = (/ &
     &5.2234e+04_rb,3.9176e+04_rb,2.6118e+04_rb,1.3059e+04_rb,1.5301e+01_rb/)
      kbo(:, 2,23,13) = (/ &
     &5.1719e+04_rb,3.8789e+04_rb,2.5860e+04_rb,1.2931e+04_rb,1.4943e+01_rb/)
      kbo(:, 3,23,13) = (/ &
     &5.1144e+04_rb,3.8358e+04_rb,2.5572e+04_rb,1.2787e+04_rb,1.4585e+01_rb/)
      kbo(:, 4,23,13) = (/ &
     &5.0530e+04_rb,3.7898e+04_rb,2.5266e+04_rb,1.2633e+04_rb,1.4170e+01_rb/)
      kbo(:, 5,23,13) = (/ &
     &4.9854e+04_rb,3.7391e+04_rb,2.4928e+04_rb,1.2464e+04_rb,1.3822e+01_rb/)
      kbo(:, 1,24,13) = (/ &
     &4.5231e+04_rb,3.3923e+04_rb,2.2616e+04_rb,1.1309e+04_rb,1.6829e+01_rb/)
      kbo(:, 2,24,13) = (/ &
     &4.4807e+04_rb,3.3606e+04_rb,2.2404e+04_rb,1.1203e+04_rb,1.6434e+01_rb/)
      kbo(:, 3,24,13) = (/ &
     &4.4319e+04_rb,3.3240e+04_rb,2.2160e+04_rb,1.1081e+04_rb,1.5981e+01_rb/)
      kbo(:, 4,24,13) = (/ &
     &4.3813e+04_rb,3.2860e+04_rb,2.1907e+04_rb,1.0954e+04_rb,1.5519e+01_rb/)
      kbo(:, 5,24,13) = (/ &
     &4.3229e+04_rb,3.2422e+04_rb,2.1615e+04_rb,1.0808e+04_rb,1.5118e+01_rb/)
      kbo(:, 1,25,13) = (/ &
     &3.9084e+04_rb,2.9314e+04_rb,1.9543e+04_rb,9.7719e+03_rb,1.8397e+01_rb/)
      kbo(:, 2,25,13) = (/ &
     &3.8734e+04_rb,2.9050e+04_rb,1.9367e+04_rb,9.6842e+03_rb,1.7930e+01_rb/)
      kbo(:, 3,25,13) = (/ &
     &3.8359e+04_rb,2.8770e+04_rb,1.9180e+04_rb,9.5907e+03_rb,1.7387e+01_rb/)
      kbo(:, 4,25,13) = (/ &
     &3.7930e+04_rb,2.8448e+04_rb,1.8965e+04_rb,9.4832e+03_rb,1.6873e+01_rb/)
      kbo(:, 5,25,13) = (/ &
     &3.7467e+04_rb,2.8100e+04_rb,1.8734e+04_rb,9.3675e+03_rb,1.6400e+01_rb/)
      kbo(:, 1,26,13) = (/ &
     &3.3890e+04_rb,2.5418e+04_rb,1.6946e+04_rb,8.4734e+03_rb,1.9920e+01_rb/)
      kbo(:, 2,26,13) = (/ &
     &3.3621e+04_rb,2.5216e+04_rb,1.6811e+04_rb,8.4060e+03_rb,1.9382e+01_rb/)
      kbo(:, 3,26,13) = (/ &
     &3.3332e+04_rb,2.4999e+04_rb,1.6667e+04_rb,8.3338e+03_rb,1.8740e+01_rb/)
      kbo(:, 4,26,13) = (/ &
     &3.3002e+04_rb,2.4752e+04_rb,1.6501e+04_rb,8.2512e+03_rb,1.8176e+01_rb/)
      kbo(:, 5,26,13) = (/ &
     &3.2656e+04_rb,2.4492e+04_rb,1.6328e+04_rb,8.1646e+03_rb,1.7620e+01_rb/)
      kbo(:, 1,27,13) = (/ &
     &3.0166e+04_rb,2.2624e+04_rb,1.5083e+04_rb,7.5423e+03_rb,2.1376e+01_rb/)
      kbo(:, 2,27,13) = (/ &
     &2.9996e+04_rb,2.2497e+04_rb,1.4998e+04_rb,7.4998e+03_rb,2.0763e+01_rb/)
      kbo(:, 3,27,13) = (/ &
     &2.9787e+04_rb,2.2341e+04_rb,1.4894e+04_rb,7.4476e+03_rb,2.0015e+01_rb/)
      kbo(:, 4,27,13) = (/ &
     &2.9581e+04_rb,2.2186e+04_rb,1.4791e+04_rb,7.3962e+03_rb,1.9407e+01_rb/)
      kbo(:, 5,27,13) = (/ &
     &2.9406e+04_rb,2.2055e+04_rb,1.4703e+04_rb,7.3522e+03_rb,1.8770e+01_rb/)
      kbo(:, 1,28,13) = (/ &
     &2.7326e+04_rb,2.0495e+04_rb,1.3664e+04_rb,6.8324e+03_rb,2.2732e+01_rb/)
      kbo(:, 2,28,13) = (/ &
     &2.7236e+04_rb,2.0428e+04_rb,1.3619e+04_rb,6.8100e+03_rb,2.2041e+01_rb/)
      kbo(:, 3,28,13) = (/ &
     &2.7174e+04_rb,2.0381e+04_rb,1.3588e+04_rb,6.7945e+03_rb,2.1170e+01_rb/)
      kbo(:, 4,28,13) = (/ &
     &2.7191e+04_rb,2.0394e+04_rb,1.3596e+04_rb,6.7985e+03_rb,2.0533e+01_rb/)
      kbo(:, 5,28,13) = (/ &
     &2.7306e+04_rb,2.0480e+04_rb,1.3654e+04_rb,6.8273e+03_rb,1.9803e+01_rb/)
      kbo(:, 1,29,13) = (/ &
     &2.6107e+04_rb,1.9580e+04_rb,1.3054e+04_rb,6.5276e+03_rb,2.3952e+01_rb/)
      kbo(:, 2,29,13) = (/ &
     &2.6178e+04_rb,1.9634e+04_rb,1.3090e+04_rb,6.5453e+03_rb,2.3128e+01_rb/)
      kbo(:, 3,29,13) = (/ &
     &2.6373e+04_rb,1.9780e+04_rb,1.3187e+04_rb,6.5941e+03_rb,2.2276e+01_rb/)
      kbo(:, 4,29,13) = (/ &
     &2.6674e+04_rb,2.0006e+04_rb,1.3338e+04_rb,6.6693e+03_rb,2.1526e+01_rb/)
      kbo(:, 5,29,13) = (/ &
     &2.7070e+04_rb,2.0303e+04_rb,1.3535e+04_rb,6.7681e+03_rb,2.0703e+01_rb/)
      kbo(:, 1,30,13) = (/ &
     &2.5565e+04_rb,1.9174e+04_rb,1.2783e+04_rb,6.3921e+03_rb,2.5015e+01_rb/)
      kbo(:, 2,30,13) = (/ &
     &2.5924e+04_rb,1.9443e+04_rb,1.2962e+04_rb,6.4818e+03_rb,2.4026e+01_rb/)
      kbo(:, 3,30,13) = (/ &
     &2.6425e+04_rb,1.9819e+04_rb,1.3213e+04_rb,6.6070e+03_rb,2.3238e+01_rb/)
      kbo(:, 4,30,13) = (/ &
     &2.7081e+04_rb,2.0311e+04_rb,1.3541e+04_rb,6.7710e+03_rb,2.2369e+01_rb/)
      kbo(:, 5,30,13) = (/ &
     &2.7860e+04_rb,2.0896e+04_rb,1.3931e+04_rb,6.9658e+03_rb,2.1263e+01_rb/)
      kbo(:, 1,31,13) = (/ &
     &2.6519e+04_rb,1.9890e+04_rb,1.3260e+04_rb,6.6306e+03_rb,2.5829e+01_rb/)
      kbo(:, 2,31,13) = (/ &
     &2.7266e+04_rb,2.0450e+04_rb,1.3634e+04_rb,6.8173e+03_rb,2.4917e+01_rb/)
      kbo(:, 3,31,13) = (/ &
     &2.8229e+04_rb,2.1172e+04_rb,1.4115e+04_rb,7.0579e+03_rb,2.4029e+01_rb/)
      kbo(:, 4,31,13) = (/ &
     &2.9353e+04_rb,2.2015e+04_rb,1.4677e+04_rb,7.3390e+03_rb,2.2989e+01_rb/)
      kbo(:, 5,31,13) = (/ &
     &3.0590e+04_rb,2.2943e+04_rb,1.5295e+04_rb,7.6481e+03_rb,2.1638e+01_rb/)
      kbo(:, 1,32,13) = (/ &
     &2.8188e+04_rb,2.1141e+04_rb,1.4095e+04_rb,7.0478e+03_rb,2.6468e+01_rb/)
      kbo(:, 2,32,13) = (/ &
     &2.9479e+04_rb,2.2110e+04_rb,1.4740e+04_rb,7.3705e+03_rb,2.5641e+01_rb/)
      kbo(:, 3,32,13) = (/ &
     &3.0998e+04_rb,2.3249e+04_rb,1.5500e+04_rb,7.7503e+03_rb,2.4646e+01_rb/)
      kbo(:, 4,32,13) = (/ &
     &3.2677e+04_rb,2.4508e+04_rb,1.6339e+04_rb,8.1700e+03_rb,2.3083e+01_rb/)
      kbo(:, 5,32,13) = (/ &
     &3.4456e+04_rb,2.5842e+04_rb,1.7228e+04_rb,8.6146e+03_rb,2.1888e+01_rb/)
      kbo(:, 1,33,13) = (/ &
     &3.1038e+04_rb,2.3278e+04_rb,1.5519e+04_rb,7.7602e+03_rb,2.7208e+01_rb/)
      kbo(:, 2,33,13) = (/ &
     &3.2999e+04_rb,2.4749e+04_rb,1.6500e+04_rb,8.2505e+03_rb,2.6145e+01_rb/)
      kbo(:, 3,33,13) = (/ &
     &3.5228e+04_rb,2.6421e+04_rb,1.7614e+04_rb,8.8077e+03_rb,2.4544e+01_rb/)
      kbo(:, 4,33,13) = (/ &
     &3.7600e+04_rb,2.8200e+04_rb,1.8800e+04_rb,9.4007e+03_rb,2.3159e+01_rb/)
      kbo(:, 5,33,13) = (/ &
     &4.0073e+04_rb,3.0055e+04_rb,2.0037e+04_rb,1.0019e+04_rb,2.2387e+01_rb/)
      kbo(:, 1,34,13) = (/ &
     &3.3858e+04_rb,2.5394e+04_rb,1.6930e+04_rb,8.4653e+03_rb,2.7625e+01_rb/)
      kbo(:, 2,34,13) = (/ &
     &3.6566e+04_rb,2.7425e+04_rb,1.8284e+04_rb,9.1422e+03_rb,2.6243e+01_rb/)
      kbo(:, 3,34,13) = (/ &
     &3.9555e+04_rb,2.9666e+04_rb,1.9778e+04_rb,9.8893e+03_rb,2.4430e+01_rb/)
      kbo(:, 4,34,13) = (/ &
     &4.2704e+04_rb,3.2028e+04_rb,2.1352e+04_rb,1.0677e+04_rb,2.3679e+01_rb/)
      kbo(:, 5,34,13) = (/ &
     &4.5894e+04_rb,3.4420e+04_rb,2.2947e+04_rb,1.1474e+04_rb,2.2773e+01_rb/)
      kbo(:, 1,35,13) = (/ &
     &3.7312e+04_rb,2.7984e+04_rb,1.8657e+04_rb,9.3288e+03_rb,2.7942e+01_rb/)
      kbo(:, 2,35,13) = (/ &
     &4.0881e+04_rb,3.0661e+04_rb,2.0441e+04_rb,1.0221e+04_rb,2.6106e+01_rb/)
      kbo(:, 3,35,13) = (/ &
     &4.4738e+04_rb,3.3554e+04_rb,2.2369e+04_rb,1.1185e+04_rb,2.5080e+01_rb/)
      kbo(:, 4,35,13) = (/ &
     &4.8754e+04_rb,3.6566e+04_rb,2.4377e+04_rb,1.2189e+04_rb,2.4179e+01_rb/)
      kbo(:, 5,35,13) = (/ &
     &5.2762e+04_rb,3.9572e+04_rb,2.6381e+04_rb,1.3191e+04_rb,2.3224e+01_rb/)
      kbo(:, 1,36,13) = (/ &
     &4.0999e+04_rb,3.0749e+04_rb,2.0500e+04_rb,1.0250e+04_rb,2.8213e+01_rb/)
      kbo(:, 2,36,13) = (/ &
     &4.5493e+04_rb,3.4120e+04_rb,2.2747e+04_rb,1.1374e+04_rb,2.6516e+01_rb/)
      kbo(:, 3,36,13) = (/ &
     &5.0302e+04_rb,3.7726e+04_rb,2.5151e+04_rb,1.2576e+04_rb,2.5690e+01_rb/)
      kbo(:, 4,36,13) = (/ &
     &5.5257e+04_rb,4.1443e+04_rb,2.7629e+04_rb,1.3815e+04_rb,2.4819e+01_rb/)
      kbo(:, 5,36,13) = (/ &
     &6.0175e+04_rb,4.5131e+04_rb,3.0088e+04_rb,1.5044e+04_rb,2.3837e+01_rb/)
      kbo(:, 1,37,13) = (/ &
     &4.3597e+04_rb,3.2698e+04_rb,2.1799e+04_rb,1.0900e+04_rb,2.8678e+01_rb/)
      kbo(:, 2,37,13) = (/ &
     &4.8963e+04_rb,3.6723e+04_rb,2.4482e+04_rb,1.2241e+04_rb,2.7079e+01_rb/)
      kbo(:, 3,37,13) = (/ &
     &5.4655e+04_rb,4.0991e+04_rb,2.7328e+04_rb,1.3664e+04_rb,2.6274e+01_rb/)
      kbo(:, 4,37,13) = (/ &
     &6.0506e+04_rb,4.5380e+04_rb,3.0253e+04_rb,1.5127e+04_rb,2.5504e+01_rb/)
      kbo(:, 5,37,13) = (/ &
     &6.6353e+04_rb,4.9765e+04_rb,3.3177e+04_rb,1.6589e+04_rb,2.4385e+01_rb/)
      kbo(:, 1,38,13) = (/ &
     &4.7296e+04_rb,3.5472e+04_rb,2.3648e+04_rb,1.1825e+04_rb,2.9114e+01_rb/)
      kbo(:, 2,38,13) = (/ &
     &5.3691e+04_rb,4.0269e+04_rb,2.6846e+04_rb,1.3423e+04_rb,2.7730e+01_rb/)
      kbo(:, 3,38,13) = (/ &
     &6.0448e+04_rb,4.5336e+04_rb,3.0224e+04_rb,1.5113e+04_rb,2.6995e+01_rb/)
      kbo(:, 4,38,13) = (/ &
     &6.7408e+04_rb,5.0556e+04_rb,3.3704e+04_rb,1.6852e+04_rb,2.6164e+01_rb/)
      kbo(:, 5,38,13) = (/ &
     &7.4361e+04_rb,5.5771e+04_rb,3.7181e+04_rb,1.8591e+04_rb,2.5025e+01_rb/)
      kbo(:, 1,39,13) = (/ &
     &5.2787e+04_rb,3.9591e+04_rb,2.6394e+04_rb,1.3198e+04_rb,2.9543e+01_rb/)
      kbo(:, 2,39,13) = (/ &
     &6.0481e+04_rb,4.5362e+04_rb,3.0242e+04_rb,1.5121e+04_rb,2.8398e+01_rb/)
      kbo(:, 3,39,13) = (/ &
     &6.8644e+04_rb,5.1483e+04_rb,3.4323e+04_rb,1.7162e+04_rb,2.7686e+01_rb/)
      kbo(:, 4,39,13) = (/ &
     &7.7024e+04_rb,5.7768e+04_rb,3.8513e+04_rb,1.9257e+04_rb,2.6627e+01_rb/)
      kbo(:, 5,39,13) = (/ &
     &8.5404e+04_rb,6.4053e+04_rb,4.2703e+04_rb,2.1352e+04_rb,2.5499e+01_rb/)
      kbo(:, 1,40,13) = (/ &
     &5.4787e+04_rb,4.1090e+04_rb,2.7394e+04_rb,1.3697e+04_rb,3.0183e+01_rb/)
      kbo(:, 2,40,13) = (/ &
     &6.3323e+04_rb,4.7492e+04_rb,3.1662e+04_rb,1.5831e+04_rb,2.8961e+01_rb/)
      kbo(:, 3,40,13) = (/ &
     &7.2463e+04_rb,5.4348e+04_rb,3.6232e+04_rb,1.8116e+04_rb,2.8295e+01_rb/)
      kbo(:, 4,40,13) = (/ &
     &8.1867e+04_rb,6.1400e+04_rb,4.0933e+04_rb,2.0467e+04_rb,2.7184e+01_rb/)
      kbo(:, 5,40,13) = (/ &
     &9.1325e+04_rb,6.8494e+04_rb,4.5663e+04_rb,2.2832e+04_rb,2.5988e+01_rb/)
      kbo(:, 1,41,13) = (/ &
     &5.6764e+04_rb,4.2573e+04_rb,2.8383e+04_rb,1.4192e+04_rb,3.0791e+01_rb/)
      kbo(:, 2,41,13) = (/ &
     &6.6130e+04_rb,4.9598e+04_rb,3.3066e+04_rb,1.6533e+04_rb,2.9538e+01_rb/)
      kbo(:, 3,41,13) = (/ &
     &7.6243e+04_rb,5.7183e+04_rb,3.8122e+04_rb,1.9062e+04_rb,2.8774e+01_rb/)
      kbo(:, 4,41,13) = (/ &
     &8.6738e+04_rb,6.5054e+04_rb,4.3370e+04_rb,2.1685e+04_rb,2.7771e+01_rb/)
      kbo(:, 5,41,13) = (/ &
     &9.7338e+04_rb,7.3004e+04_rb,4.8670e+04_rb,2.4335e+04_rb,2.6481e+01_rb/)
      kbo(:, 1,42,13) = (/ &
     &5.9574e+04_rb,4.4680e+04_rb,2.9787e+04_rb,1.4894e+04_rb,3.1338e+01_rb/)
      kbo(:, 2,42,13) = (/ &
     &6.9874e+04_rb,5.2405e+04_rb,3.4937e+04_rb,1.7469e+04_rb,3.0066e+01_rb/)
      kbo(:, 3,42,13) = (/ &
     &8.1118e+04_rb,6.0839e+04_rb,4.0559e+04_rb,2.0280e+04_rb,2.9330e+01_rb/)
      kbo(:, 4,42,13) = (/ &
     &9.2915e+04_rb,6.9686e+04_rb,4.6457e+04_rb,2.3229e+04_rb,2.8281e+01_rb/)
      kbo(:, 5,42,13) = (/ &
     &1.0485e+05_rb,7.8636e+04_rb,5.2424e+04_rb,2.6212e+04_rb,2.6956e+01_rb/)
      kbo(:, 1,43,13) = (/ &
     &6.2565e+04_rb,4.6924e+04_rb,3.1283e+04_rb,1.5642e+04_rb,3.1947e+01_rb/)
      kbo(:, 2,43,13) = (/ &
     &7.3881e+04_rb,5.5411e+04_rb,3.6941e+04_rb,1.8471e+04_rb,3.0710e+01_rb/)
      kbo(:, 3,43,13) = (/ &
     &8.6376e+04_rb,6.4783e+04_rb,4.3189e+04_rb,2.1595e+04_rb,2.9823e+01_rb/)
      kbo(:, 4,43,13) = (/ &
     &9.9633e+04_rb,7.4725e+04_rb,4.9817e+04_rb,2.4909e+04_rb,2.8916e+01_rb/)
      kbo(:, 5,43,13) = (/ &
     &1.1317e+05_rb,8.4879e+04_rb,5.6586e+04_rb,2.8294e+04_rb,2.7433e+01_rb/)
      kbo(:, 1,44,13) = (/ &
     &6.6377e+04_rb,4.9783e+04_rb,3.3188e+04_rb,1.6595e+04_rb,3.2537e+01_rb/)
      kbo(:, 2,44,13) = (/ &
     &7.8875e+04_rb,5.9156e+04_rb,3.9437e+04_rb,1.9719e+04_rb,3.1463e+01_rb/)
      kbo(:, 3,44,13) = (/ &
     &9.2826e+04_rb,6.9619e+04_rb,4.6413e+04_rb,2.3207e+04_rb,3.0339e+01_rb/)
      kbo(:, 4,44,13) = (/ &
     &1.0782e+05_rb,8.0864e+04_rb,5.3909e+04_rb,2.6955e+04_rb,2.9519e+01_rb/)
      kbo(:, 5,44,13) = (/ &
     &1.2335e+05_rb,9.2508e+04_rb,6.1672e+04_rb,3.0836e+04_rb,2.7886e+01_rb/)
      kbo(:, 1,45,13) = (/ &
     &7.1874e+04_rb,5.3905e+04_rb,3.5937e+04_rb,1.7969e+04_rb,3.3146e+01_rb/)
      kbo(:, 2,45,13) = (/ &
     &8.5887e+04_rb,6.4415e+04_rb,4.2944e+04_rb,2.1472e+04_rb,3.2123e+01_rb/)
      kbo(:, 3,45,13) = (/ &
     &1.0168e+05_rb,7.6262e+04_rb,5.0841e+04_rb,2.5421e+04_rb,3.0727e+01_rb/)
      kbo(:, 4,45,13) = (/ &
     &1.1891e+05_rb,8.9181e+04_rb,5.9454e+04_rb,2.9727e+04_rb,3.0052e+01_rb/)
      kbo(:, 5,45,13) = (/ &
     &1.3695e+05_rb,1.0272e+05_rb,6.8477e+04_rb,3.4239e+04_rb,2.8423e+01_rb/)
      kbo(:, 1,46,13) = (/ &
     &7.8328e+04_rb,5.8747e+04_rb,3.9165e+04_rb,1.9583e+04_rb,3.3800e+01_rb/)
      kbo(:, 2,46,13) = (/ &
     &9.4082e+04_rb,7.0562e+04_rb,4.7041e+04_rb,2.3521e+04_rb,3.2751e+01_rb/)
      kbo(:, 3,46,13) = (/ &
     &1.1205e+05_rb,8.4041e+04_rb,5.6028e+04_rb,2.8014e+04_rb,3.1210e+01_rb/)
      kbo(:, 4,46,13) = (/ &
     &1.3188e+05_rb,9.8910e+04_rb,6.5940e+04_rb,3.2971e+04_rb,3.0670e+01_rb/)
      kbo(:, 5,46,13) = (/ &
     &1.5296e+05_rb,1.1472e+05_rb,7.6478e+04_rb,3.8239e+04_rb,2.9020e+01_rb/)
      kbo(:, 1,47,13) = (/ &
     &8.3810e+04_rb,6.2858e+04_rb,4.1906e+04_rb,2.0953e+04_rb,3.4456e+01_rb/)
      kbo(:, 2,47,13) = (/ &
     &1.0129e+05_rb,7.5968e+04_rb,5.0646e+04_rb,2.5323e+04_rb,3.3326e+01_rb/)
      kbo(:, 3,47,13) = (/ &
     &1.2133e+05_rb,9.0998e+04_rb,6.0666e+04_rb,3.0333e+04_rb,3.1897e+01_rb/)
      kbo(:, 4,47,13) = (/ &
     &1.4374e+05_rb,1.0781e+05_rb,7.1871e+04_rb,3.5936e+04_rb,3.1165e+01_rb/)
      kbo(:, 5,47,13) = (/ &
     &1.6790e+05_rb,1.2593e+05_rb,8.3952e+04_rb,4.1976e+04_rb,2.9653e+01_rb/)
      kbo(:, 1,48,13) = (/ &
     &9.2009e+04_rb,6.9007e+04_rb,4.6005e+04_rb,2.3003e+04_rb,3.5004e+01_rb/)
      kbo(:, 2,48,13) = (/ &
     &1.1207e+05_rb,8.4051e+04_rb,5.6034e+04_rb,2.8018e+04_rb,3.3918e+01_rb/)
      kbo(:, 3,48,13) = (/ &
     &1.3494e+05_rb,1.0121e+05_rb,6.7471e+04_rb,3.3736e+04_rb,3.2425e+01_rb/)
      kbo(:, 4,48,13) = (/ &
     &1.6083e+05_rb,1.2063e+05_rb,8.0418e+04_rb,4.0210e+04_rb,3.1652e+01_rb/)
      kbo(:, 5,48,13) = (/ &
     &1.8920e+05_rb,1.4190e+05_rb,9.4599e+04_rb,4.7300e+04_rb,3.0396e+01_rb/)
      kbo(:, 1,49,13) = (/ &
     &1.0515e+05_rb,7.8866e+04_rb,5.2577e+04_rb,2.6289e+04_rb,3.5598e+01_rb/)
      kbo(:, 2,49,13) = (/ &
     &1.2899e+05_rb,9.6745e+04_rb,6.4497e+04_rb,3.2249e+04_rb,3.4541e+01_rb/)
      kbo(:, 3,49,13) = (/ &
     &1.5618e+05_rb,1.1714e+05_rb,7.8092e+04_rb,3.9046e+04_rb,3.3166e+01_rb/)
      kbo(:, 4,49,13) = (/ &
     &1.8721e+05_rb,1.4041e+05_rb,9.3606e+04_rb,4.6803e+04_rb,3.2127e+01_rb/)
      kbo(:, 5,49,13) = (/ &
     &2.2166e+05_rb,1.6624e+05_rb,1.1083e+05_rb,5.5414e+04_rb,3.1114e+01_rb/)
      kbo(:, 1,50,13) = (/ &
     &1.1492e+05_rb,8.6190e+04_rb,5.7460e+04_rb,2.8730e+04_rb,3.5920e+01_rb/)
      kbo(:, 2,50,13) = (/ &
     &1.4177e+05_rb,1.0633e+05_rb,7.0884e+04_rb,3.5442e+04_rb,3.5009e+01_rb/)
      kbo(:, 3,50,13) = (/ &
     &1.7284e+05_rb,1.2963e+05_rb,8.6422e+04_rb,4.3212e+04_rb,3.3840e+01_rb/)
      kbo(:, 4,50,13) = (/ &
     &2.0816e+05_rb,1.5612e+05_rb,1.0408e+05_rb,5.2042e+04_rb,3.2614e+01_rb/)
      kbo(:, 5,50,13) = (/ &
     &2.4789e+05_rb,1.8591e+05_rb,1.2394e+05_rb,6.1972e+04_rb,3.1787e+01_rb/)
      kbo(:, 1,51,13) = (/ &
     &1.2316e+05_rb,9.2372e+04_rb,6.1582e+04_rb,3.0791e+04_rb,3.6248e+01_rb/)
      kbo(:, 2,51,13) = (/ &
     &1.5257e+05_rb,1.1443e+05_rb,7.6284e+04_rb,3.8142e+04_rb,3.5464e+01_rb/)
      kbo(:, 3,51,13) = (/ &
     &1.8735e+05_rb,1.4051e+05_rb,9.3677e+04_rb,4.6839e+04_rb,3.4431e+01_rb/)
      kbo(:, 4,51,13) = (/ &
     &2.2663e+05_rb,1.6997e+05_rb,1.1332e+05_rb,5.6659e+04_rb,3.2971e+01_rb/)
      kbo(:, 5,51,13) = (/ &
     &2.7128e+05_rb,2.0346e+05_rb,1.3564e+05_rb,6.7820e+04_rb,3.2477e+01_rb/)
      kbo(:, 1,52,13) = (/ &
     &1.3564e+05_rb,1.0173e+05_rb,6.7822e+04_rb,3.3911e+04_rb,3.6511e+01_rb/)
      kbo(:, 2,52,13) = (/ &
     &1.6873e+05_rb,1.2655e+05_rb,8.4367e+04_rb,4.2184e+04_rb,3.5955e+01_rb/)
      kbo(:, 3,52,13) = (/ &
     &2.0841e+05_rb,1.5631e+05_rb,1.0421e+05_rb,5.2104e+04_rb,3.4988e+01_rb/)
      kbo(:, 4,52,13) = (/ &
     &2.5356e+05_rb,1.9017e+05_rb,1.2678e+05_rb,6.3390e+04_rb,3.3412e+01_rb/)
      kbo(:, 5,52,13) = (/ &
     &3.0501e+05_rb,2.2876e+05_rb,1.5250e+05_rb,7.6253e+04_rb,3.3108e+01_rb/)
      kbo(:, 1,53,13) = (/ &
     &1.5552e+05_rb,1.1664e+05_rb,7.7762e+04_rb,3.8882e+04_rb,3.6856e+01_rb/)
      kbo(:, 2,53,13) = (/ &
     &1.9425e+05_rb,1.4569e+05_rb,9.7128e+04_rb,4.8565e+04_rb,3.6432e+01_rb/)
      kbo(:, 3,53,13) = (/ &
     &2.4103e+05_rb,1.8077e+05_rb,1.2051e+05_rb,6.0258e+04_rb,3.5535e+01_rb/)
      kbo(:, 4,53,13) = (/ &
     &2.9532e+05_rb,2.2149e+05_rb,1.4766e+05_rb,7.3832e+04_rb,3.3878e+01_rb/)
      kbo(:, 5,53,13) = (/ &
     &3.5673e+05_rb,2.6755e+05_rb,1.7837e+05_rb,8.9184e+04_rb,3.3830e+01_rb/)
      kbo(:, 1,54,13) = (/ &
     &1.5519e+05_rb,1.1639e+05_rb,7.7595e+04_rb,3.8798e+04_rb,3.7137e+01_rb/)
      kbo(:, 2,54,13) = (/ &
     &1.9455e+05_rb,1.4591e+05_rb,9.7277e+04_rb,4.8639e+04_rb,3.6719e+01_rb/)
      kbo(:, 3,54,13) = (/ &
     &2.4223e+05_rb,1.8167e+05_rb,1.2111e+05_rb,6.0558e+04_rb,3.5737e+01_rb/)
      kbo(:, 4,54,13) = (/ &
     &2.9863e+05_rb,2.2398e+05_rb,1.4932e+05_rb,7.4660e+04_rb,3.4182e+01_rb/)
      kbo(:, 5,54,13) = (/ &
     &3.6225e+05_rb,2.7169e+05_rb,1.8113e+05_rb,9.0564e+04_rb,3.4090e+01_rb/)
      kbo(:, 1,55,13) = (/ &
     &1.4179e+05_rb,1.0634e+05_rb,7.0895e+04_rb,3.5448e+04_rb,3.7500e+01_rb/)
      kbo(:, 2,55,13) = (/ &
     &1.7805e+05_rb,1.3354e+05_rb,8.9025e+04_rb,4.4513e+04_rb,3.6673e+01_rb/)
      kbo(:, 3,55,13) = (/ &
     &2.2250e+05_rb,1.6688e+05_rb,1.1125e+05_rb,5.5626e+04_rb,3.6002e+01_rb/)
      kbo(:, 4,55,13) = (/ &
     &2.7565e+05_rb,2.0674e+05_rb,1.3783e+05_rb,6.8913e+04_rb,3.4967e+01_rb/)
      kbo(:, 5,55,13) = (/ &
     &3.3613e+05_rb,2.5210e+05_rb,1.6807e+05_rb,8.4034e+04_rb,3.3605e+01_rb/)
      kbo(:, 1,56,13) = (/ &
     &1.2955e+05_rb,9.7160e+04_rb,6.4773e+04_rb,3.2387e+04_rb,3.7592e+01_rb/)
      kbo(:, 2,56,13) = (/ &
     &1.6291e+05_rb,1.2218e+05_rb,8.1452e+04_rb,4.0726e+04_rb,3.6892e+01_rb/)
      kbo(:, 3,56,13) = (/ &
     &2.0430e+05_rb,1.5322e+05_rb,1.0215e+05_rb,5.1074e+04_rb,3.6401e+01_rb/)
      kbo(:, 4,56,13) = (/ &
     &2.5407e+05_rb,1.9055e+05_rb,1.2704e+05_rb,6.3518e+04_rb,3.5416e+01_rb/)
      kbo(:, 5,56,13) = (/ &
     &3.1173e+05_rb,2.3380e+05_rb,1.5586e+05_rb,7.7932e+04_rb,3.3725e+01_rb/)
      kbo(:, 1,57,13) = (/ &
     &1.1844e+05_rb,8.8830e+04_rb,5.9220e+04_rb,2.9610e+04_rb,3.7590e+01_rb/)
      kbo(:, 2,57,13) = (/ &
     &1.4902e+05_rb,1.1176e+05_rb,7.4508e+04_rb,3.7254e+04_rb,3.7340e+01_rb/)
      kbo(:, 3,57,13) = (/ &
     &1.8747e+05_rb,1.4060e+05_rb,9.3736e+04_rb,4.6868e+04_rb,3.6437e+01_rb/)
      kbo(:, 4,57,13) = (/ &
     &2.3395e+05_rb,1.7546e+05_rb,1.1697e+05_rb,5.8487e+04_rb,3.5598e+01_rb/)
      kbo(:, 5,57,13) = (/ &
     &2.8870e+05_rb,2.1653e+05_rb,1.4435e+05_rb,7.2175e+04_rb,3.4562e+01_rb/)
      kbo(:, 1,58,13) = (/ &
     &2.8654e+04_rb,2.6272e+04_rb,2.2526e+04_rb,1.5777e+04_rb,3.7286e+01_rb/)
      kbo(:, 2,58,13) = (/ &
     &3.6090e+04_rb,3.3089e+04_rb,2.8371e+04_rb,1.9871e+04_rb,3.7547e+01_rb/)
      kbo(:, 3,58,13) = (/ &
     &4.5492e+04_rb,4.1710e+04_rb,3.5762e+04_rb,2.5048e+04_rb,3.6650e+01_rb/)
      kbo(:, 4,58,13) = (/ &
     &5.6953e+04_rb,5.2217e+04_rb,4.4772e+04_rb,3.1358e+04_rb,3.6005e+01_rb/)
      kbo(:, 5,58,13) = (/ &
     &7.0617e+04_rb,6.4745e+04_rb,5.5513e+04_rb,3.8881e+04_rb,3.4983e+01_rb/)
      kbo(:, 1,59,13) = (/ &
     &3.2425e+04_rb,2.8312e+04_rb,2.2583e+04_rb,1.4053e+04_rb,3.7337e+01_rb/)
      kbo(:, 2,59,13) = (/ &
     &4.0874e+04_rb,3.5689e+04_rb,2.8468e+04_rb,1.7715e+04_rb,3.7354e+01_rb/)
      kbo(:, 3,59,13) = (/ &
     &5.1569e+04_rb,4.5029e+04_rb,3.5917e+04_rb,2.2350e+04_rb,3.7146e+01_rb/)
      kbo(:, 4,59,13) = (/ &
     &6.4659e+04_rb,5.6458e+04_rb,4.5034e+04_rb,2.8023e+04_rb,3.5794e+01_rb/)
      kbo(:, 5,59,13) = (/ &
     &8.0312e+04_rb,7.0125e+04_rb,5.5936e+04_rb,3.4807e+04_rb,3.4858e+01_rb/)
      kbo(:, 1,13,14) = (/ &
     &5.5354e+05_rb,4.1516e+05_rb,2.7677e+05_rb,1.3839e+05_rb,6.6359e+00_rb/)
      kbo(:, 2,13,14) = (/ &
     &5.3662e+05_rb,4.0247e+05_rb,2.6831e+05_rb,1.3416e+05_rb,6.3521e+00_rb/)
      kbo(:, 3,13,14) = (/ &
     &5.2321e+05_rb,3.9241e+05_rb,2.6161e+05_rb,1.3080e+05_rb,6.1777e+00_rb/)
      kbo(:, 4,13,14) = (/ &
     &5.1223e+05_rb,3.8417e+05_rb,2.5612e+05_rb,1.2806e+05_rb,5.8113e+00_rb/)
      kbo(:, 5,13,14) = (/ &
     &5.0254e+05_rb,3.7690e+05_rb,2.5127e+05_rb,1.2564e+05_rb,5.1461e+00_rb/)
      kbo(:, 1,14,14) = (/ &
     &3.6531e+05_rb,2.7398e+05_rb,1.8265e+05_rb,9.1327e+04_rb,7.3146e+00_rb/)
      kbo(:, 2,14,14) = (/ &
     &3.5777e+05_rb,2.6833e+05_rb,1.7888e+05_rb,8.9442e+04_rb,7.0661e+00_rb/)
      kbo(:, 3,14,14) = (/ &
     &3.5166e+05_rb,2.6375e+05_rb,1.7583e+05_rb,8.7915e+04_rb,6.6597e+00_rb/)
      kbo(:, 4,14,14) = (/ &
     &3.4675e+05_rb,2.6006e+05_rb,1.7337e+05_rb,8.6687e+04_rb,6.5607e+00_rb/)
      kbo(:, 5,14,14) = (/ &
     &3.4224e+05_rb,2.5668e+05_rb,1.7112e+05_rb,8.5559e+04_rb,6.0728e+00_rb/)
      kbo(:, 1,15,14) = (/ &
     &2.5038e+05_rb,1.8779e+05_rb,1.2519e+05_rb,6.2598e+04_rb,8.1437e+00_rb/)
      kbo(:, 2,15,14) = (/ &
     &2.4740e+05_rb,1.8555e+05_rb,1.2370e+05_rb,6.1853e+04_rb,7.8645e+00_rb/)
      kbo(:, 3,15,14) = (/ &
     &2.4508e+05_rb,1.8381e+05_rb,1.2254e+05_rb,6.1272e+04_rb,7.4907e+00_rb/)
      kbo(:, 4,15,14) = (/ &
     &2.4271e+05_rb,1.8203e+05_rb,1.2136e+05_rb,6.0680e+04_rb,7.2641e+00_rb/)
      kbo(:, 5,15,14) = (/ &
     &2.4052e+05_rb,1.8039e+05_rb,1.2026e+05_rb,6.0131e+04_rb,7.0682e+00_rb/)
      kbo(:, 1,16,14) = (/ &
     &2.0782e+05_rb,1.5587e+05_rb,1.0391e+05_rb,5.1957e+04_rb,9.0703e+00_rb/)
      kbo(:, 2,16,14) = (/ &
     &2.0661e+05_rb,1.5496e+05_rb,1.0331e+05_rb,5.1654e+04_rb,8.7622e+00_rb/)
      kbo(:, 3,16,14) = (/ &
     &2.0513e+05_rb,1.5385e+05_rb,1.0257e+05_rb,5.1284e+04_rb,8.4635e+00_rb/)
      kbo(:, 4,16,14) = (/ &
     &2.0374e+05_rb,1.5280e+05_rb,1.0187e+05_rb,5.0936e+04_rb,8.1015e+00_rb/)
      kbo(:, 5,16,14) = (/ &
     &2.0191e+05_rb,1.5143e+05_rb,1.0096e+05_rb,5.0479e+04_rb,7.9045e+00_rb/)
      kbo(:, 1,17,14) = (/ &
     &1.8037e+05_rb,1.3528e+05_rb,9.0185e+04_rb,4.5093e+04_rb,1.0108e+01_rb/)
      kbo(:, 2,17,14) = (/ &
     &1.7972e+05_rb,1.3479e+05_rb,8.9862e+04_rb,4.4932e+04_rb,9.7857e+00_rb/)
      kbo(:, 3,17,14) = (/ &
     &1.7906e+05_rb,1.3430e+05_rb,8.9533e+04_rb,4.4767e+04_rb,9.4761e+00_rb/)
      kbo(:, 4,17,14) = (/ &
     &1.7788e+05_rb,1.3341e+05_rb,8.8942e+04_rb,4.4472e+04_rb,9.0551e+00_rb/)
      kbo(:, 5,17,14) = (/ &
     &1.7662e+05_rb,1.3247e+05_rb,8.8311e+04_rb,4.4156e+04_rb,8.8726e+00_rb/)
      kbo(:, 1,18,14) = (/ &
     &1.6160e+05_rb,1.2120e+05_rb,8.0801e+04_rb,4.0401e+04_rb,1.1327e+01_rb/)
      kbo(:, 2,18,14) = (/ &
     &1.6141e+05_rb,1.2106e+05_rb,8.0704e+04_rb,4.0353e+04_rb,1.1010e+01_rb/)
      kbo(:, 3,18,14) = (/ &
     &1.6076e+05_rb,1.2057e+05_rb,8.0382e+04_rb,4.0192e+04_rb,1.0693e+01_rb/)
      kbo(:, 4,18,14) = (/ &
     &1.6001e+05_rb,1.2001e+05_rb,8.0005e+04_rb,4.0003e+04_rb,1.0193e+01_rb/)
      kbo(:, 5,18,14) = (/ &
     &1.5881e+05_rb,1.1911e+05_rb,7.9406e+04_rb,3.9704e+04_rb,1.0071e+01_rb/)
      kbo(:, 1,19,14) = (/ &
     &1.4270e+05_rb,1.0702e+05_rb,7.1350e+04_rb,3.5676e+04_rb,1.2821e+01_rb/)
      kbo(:, 2,19,14) = (/ &
     &1.4247e+05_rb,1.0685e+05_rb,7.1236e+04_rb,3.5619e+04_rb,1.2498e+01_rb/)
      kbo(:, 3,19,14) = (/ &
     &1.4204e+05_rb,1.0653e+05_rb,7.1022e+04_rb,3.5511e+04_rb,1.2171e+01_rb/)
      kbo(:, 4,19,14) = (/ &
     &1.4126e+05_rb,1.0594e+05_rb,7.0629e+04_rb,3.5315e+04_rb,1.1680e+01_rb/)
      kbo(:, 5,19,14) = (/ &
     &1.4024e+05_rb,1.0518e+05_rb,7.0119e+04_rb,3.5060e+04_rb,1.1491e+01_rb/)
      kbo(:, 1,20,14) = (/ &
     &1.3563e+05_rb,1.0173e+05_rb,6.7817e+04_rb,3.3909e+04_rb,1.4618e+01_rb/)
      kbo(:, 2,20,14) = (/ &
     &1.3541e+05_rb,1.0156e+05_rb,6.7708e+04_rb,3.3855e+04_rb,1.4263e+01_rb/)
      kbo(:, 3,20,14) = (/ &
     &1.3484e+05_rb,1.0113e+05_rb,6.7419e+04_rb,3.3710e+04_rb,1.3906e+01_rb/)
      kbo(:, 4,20,14) = (/ &
     &1.3404e+05_rb,1.0053e+05_rb,6.7023e+04_rb,3.3512e+04_rb,1.3385e+01_rb/)
      kbo(:, 5,20,14) = (/ &
     &1.3293e+05_rb,9.9699e+04_rb,6.6466e+04_rb,3.3234e+04_rb,1.3122e+01_rb/)
      kbo(:, 1,21,14) = (/ &
     &1.3198e+05_rb,9.8983e+04_rb,6.5989e+04_rb,3.2995e+04_rb,1.6661e+01_rb/)
      kbo(:, 2,21,14) = (/ &
     &1.3158e+05_rb,9.8688e+04_rb,6.5792e+04_rb,3.2897e+04_rb,1.6248e+01_rb/)
      kbo(:, 3,21,14) = (/ &
     &1.3094e+05_rb,9.8202e+04_rb,6.5468e+04_rb,3.2735e+04_rb,1.5812e+01_rb/)
      kbo(:, 4,21,14) = (/ &
     &1.3001e+05_rb,9.7504e+04_rb,6.5003e+04_rb,3.2502e+04_rb,1.5225e+01_rb/)
      kbo(:, 5,21,14) = (/ &
     &1.2884e+05_rb,9.6632e+04_rb,6.4422e+04_rb,3.2211e+04_rb,1.4867e+01_rb/)
      kbo(:, 1,22,14) = (/ &
     &1.3028e+05_rb,9.7707e+04_rb,6.5139e+04_rb,3.2570e+04_rb,1.8806e+01_rb/)
      kbo(:, 2,22,14) = (/ &
     &1.2981e+05_rb,9.7355e+04_rb,6.4904e+04_rb,3.2452e+04_rb,1.8310e+01_rb/)
      kbo(:, 3,22,14) = (/ &
     &1.2902e+05_rb,9.6763e+04_rb,6.4509e+04_rb,3.2255e+04_rb,1.7798e+01_rb/)
      kbo(:, 4,22,14) = (/ &
     &1.2795e+05_rb,9.5966e+04_rb,6.3978e+04_rb,3.1989e+04_rb,1.7136e+01_rb/)
      kbo(:, 5,22,14) = (/ &
     &1.2666e+05_rb,9.4996e+04_rb,6.3331e+04_rb,3.1666e+04_rb,1.6711e+01_rb/)
      kbo(:, 1,23,14) = (/ &
     &1.2794e+05_rb,9.5952e+04_rb,6.3968e+04_rb,3.1985e+04_rb,2.1073e+01_rb/)
      kbo(:, 2,23,14) = (/ &
     &1.2735e+05_rb,9.5515e+04_rb,6.3677e+04_rb,3.1839e+04_rb,2.0511e+01_rb/)
      kbo(:, 3,23,14) = (/ &
     &1.2647e+05_rb,9.4850e+04_rb,6.3234e+04_rb,3.1617e+04_rb,1.9930e+01_rb/)
      kbo(:, 4,23,14) = (/ &
     &1.2537e+05_rb,9.4025e+04_rb,6.2684e+04_rb,3.1342e+04_rb,1.9174e+01_rb/)
      kbo(:, 5,23,14) = (/ &
     &1.2398e+05_rb,9.2985e+04_rb,6.1990e+04_rb,3.0996e+04_rb,1.8667e+01_rb/)
      kbo(:, 1,24,14) = (/ &
     &1.2343e+05_rb,9.2570e+04_rb,6.1714e+04_rb,3.0858e+04_rb,2.3485e+01_rb/)
      kbo(:, 2,24,14) = (/ &
     &1.2276e+05_rb,9.2074e+04_rb,6.1383e+04_rb,3.0692e+04_rb,2.2824e+01_rb/)
      kbo(:, 3,24,14) = (/ &
     &1.2194e+05_rb,9.1458e+04_rb,6.0973e+04_rb,3.0487e+04_rb,2.2153e+01_rb/)
      kbo(:, 4,24,14) = (/ &
     &1.2080e+05_rb,9.0601e+04_rb,6.0401e+04_rb,3.0201e+04_rb,2.1291e+01_rb/)
      kbo(:, 5,24,14) = (/ &
     &1.1949e+05_rb,8.9617e+04_rb,5.9745e+04_rb,2.9873e+04_rb,2.0633e+01_rb/)
      kbo(:, 1,25,14) = (/ &
     &1.1905e+05_rb,8.9291e+04_rb,5.9528e+04_rb,2.9764e+04_rb,2.5966e+01_rb/)
      kbo(:, 2,25,14) = (/ &
     &1.1853e+05_rb,8.8894e+04_rb,5.9263e+04_rb,2.9632e+04_rb,2.5184e+01_rb/)
      kbo(:, 3,25,14) = (/ &
     &1.1769e+05_rb,8.8271e+04_rb,5.8848e+04_rb,2.9424e+04_rb,2.4373e+01_rb/)
      kbo(:, 4,25,14) = (/ &
     &1.1666e+05_rb,8.7497e+04_rb,5.8331e+04_rb,2.9166e+04_rb,2.3367e+01_rb/)
      kbo(:, 5,25,14) = (/ &
     &1.1542e+05_rb,8.6565e+04_rb,5.7710e+04_rb,2.8855e+04_rb,2.2586e+01_rb/)
      kbo(:, 1,26,14) = (/ &
     &1.1553e+05_rb,8.6651e+04_rb,5.7768e+04_rb,2.8884e+04_rb,2.8441e+01_rb/)
      kbo(:, 2,26,14) = (/ &
     &1.1511e+05_rb,8.6329e+04_rb,5.7553e+04_rb,2.8777e+04_rb,2.7500e+01_rb/)
      kbo(:, 3,26,14) = (/ &
     &1.1443e+05_rb,8.5820e+04_rb,5.7213e+04_rb,2.8607e+04_rb,2.6532e+01_rb/)
      kbo(:, 4,26,14) = (/ &
     &1.1356e+05_rb,8.5172e+04_rb,5.6781e+04_rb,2.8391e+04_rb,2.5362e+01_rb/)
      kbo(:, 5,26,14) = (/ &
     &1.1241e+05_rb,8.4309e+04_rb,5.6206e+04_rb,2.8103e+04_rb,2.4468e+01_rb/)
      kbo(:, 1,27,14) = (/ &
     &1.1545e+05_rb,8.6587e+04_rb,5.7725e+04_rb,2.8863e+04_rb,3.0839e+01_rb/)
      kbo(:, 2,27,14) = (/ &
     &1.1514e+05_rb,8.6352e+04_rb,5.7568e+04_rb,2.8785e+04_rb,2.9713e+01_rb/)
      kbo(:, 3,27,14) = (/ &
     &1.1466e+05_rb,8.5998e+04_rb,5.7332e+04_rb,2.8667e+04_rb,2.8580e+01_rb/)
      kbo(:, 4,27,14) = (/ &
     &1.1391e+05_rb,8.5436e+04_rb,5.6958e+04_rb,2.8479e+04_rb,2.7213e+01_rb/)
      kbo(:, 5,27,14) = (/ &
     &1.1293e+05_rb,8.4701e+04_rb,5.6467e+04_rb,2.8234e+04_rb,2.6212e+01_rb/)
      kbo(:, 1,28,14) = (/ &
     &1.1769e+05_rb,8.8268e+04_rb,5.8845e+04_rb,2.9423e+04_rb,3.3091e+01_rb/)
      kbo(:, 2,28,14) = (/ &
     &1.1765e+05_rb,8.8234e+04_rb,5.8823e+04_rb,2.9412e+04_rb,3.1766e+01_rb/)
      kbo(:, 3,28,14) = (/ &
     &1.1733e+05_rb,8.7997e+04_rb,5.8665e+04_rb,2.9333e+04_rb,3.0463e+01_rb/)
      kbo(:, 4,28,14) = (/ &
     &1.1677e+05_rb,8.7580e+04_rb,5.8387e+04_rb,2.9194e+04_rb,2.8874e+01_rb/)
      kbo(:, 5,28,14) = (/ &
     &1.1593e+05_rb,8.6950e+04_rb,5.7967e+04_rb,2.8984e+04_rb,2.7821e+01_rb/)
      kbo(:, 1,29,14) = (/ &
     &1.2676e+05_rb,9.5071e+04_rb,6.3381e+04_rb,3.1691e+04_rb,3.5131e+01_rb/)
      kbo(:, 2,29,14) = (/ &
     &1.2709e+05_rb,9.5318e+04_rb,6.3545e+04_rb,3.1773e+04_rb,3.3613e+01_rb/)
      kbo(:, 3,29,14) = (/ &
     &1.2701e+05_rb,9.5260e+04_rb,6.3507e+04_rb,3.1754e+04_rb,3.1981e+01_rb/)
      kbo(:, 4,29,14) = (/ &
     &1.2660e+05_rb,9.4952e+04_rb,6.3301e+04_rb,3.1651e+04_rb,3.0369e+01_rb/)
      kbo(:, 5,29,14) = (/ &
     &1.2589e+05_rb,9.4414e+04_rb,6.2943e+04_rb,3.1472e+04_rb,2.9250e+01_rb/)
      kbo(:, 1,30,14) = (/ &
     &1.3959e+05_rb,1.0469e+05_rb,6.9793e+04_rb,3.4897e+04_rb,3.6931e+01_rb/)
      kbo(:, 2,30,14) = (/ &
     &1.4034e+05_rb,1.0525e+05_rb,7.0169e+04_rb,3.5085e+04_rb,3.5216e+01_rb/)
      kbo(:, 3,30,14) = (/ &
     &1.4057e+05_rb,1.0543e+05_rb,7.0285e+04_rb,3.5143e+04_rb,3.3253e+01_rb/)
      kbo(:, 4,30,14) = (/ &
     &1.4040e+05_rb,1.0530e+05_rb,7.0200e+04_rb,3.5100e+04_rb,3.1871e+01_rb/)
      kbo(:, 5,30,14) = (/ &
     &1.3970e+05_rb,1.0477e+05_rb,6.9850e+04_rb,3.4925e+04_rb,3.0435e+01_rb/)
      kbo(:, 1,31,14) = (/ &
     &1.6150e+05_rb,1.2112e+05_rb,8.0748e+04_rb,4.0374e+04_rb,3.8448e+01_rb/)
      kbo(:, 2,31,14) = (/ &
     &1.6274e+05_rb,1.2205e+05_rb,8.1369e+04_rb,4.0685e+04_rb,3.6340e+01_rb/)
      kbo(:, 3,31,14) = (/ &
     &1.6341e+05_rb,1.2256e+05_rb,8.1707e+04_rb,4.0854e+04_rb,3.4532e+01_rb/)
      kbo(:, 4,31,14) = (/ &
     &1.6341e+05_rb,1.2256e+05_rb,8.1706e+04_rb,4.0853e+04_rb,3.3032e+01_rb/)
      kbo(:, 5,31,14) = (/ &
     &1.6281e+05_rb,1.2211e+05_rb,8.1407e+04_rb,4.0704e+04_rb,3.1374e+01_rb/)
      kbo(:, 1,32,14) = (/ &
     &1.8902e+05_rb,1.4177e+05_rb,9.4511e+04_rb,4.7256e+04_rb,3.9673e+01_rb/)
      kbo(:, 2,32,14) = (/ &
     &1.9095e+05_rb,1.4322e+05_rb,9.5478e+04_rb,4.7739e+04_rb,3.7210e+01_rb/)
      kbo(:, 3,32,14) = (/ &
     &1.9208e+05_rb,1.4406e+05_rb,9.6041e+04_rb,4.8021e+04_rb,3.5668e+01_rb/)
      kbo(:, 4,32,14) = (/ &
     &1.9230e+05_rb,1.4423e+05_rb,9.6153e+04_rb,4.8077e+04_rb,3.3902e+01_rb/)
      kbo(:, 5,32,14) = (/ &
     &1.9178e+05_rb,1.4383e+05_rb,9.5890e+04_rb,4.7945e+04_rb,3.1255e+01_rb/)
      kbo(:, 1,33,14) = (/ &
     &2.2548e+05_rb,1.6911e+05_rb,1.1274e+05_rb,5.6370e+04_rb,4.0253e+01_rb/)
      kbo(:, 2,33,14) = (/ &
     &2.2820e+05_rb,1.7115e+05_rb,1.1410e+05_rb,5.7051e+04_rb,3.8356e+01_rb/)
      kbo(:, 3,33,14) = (/ &
     &2.2980e+05_rb,1.7235e+05_rb,1.1490e+05_rb,5.7450e+04_rb,3.6454e+01_rb/)
      kbo(:, 4,33,14) = (/ &
     &2.3033e+05_rb,1.7275e+05_rb,1.1517e+05_rb,5.7584e+04_rb,3.3797e+01_rb/)
      kbo(:, 5,33,14) = (/ &
     &2.2985e+05_rb,1.7239e+05_rb,1.1492e+05_rb,5.7463e+04_rb,3.1363e+01_rb/)
      kbo(:, 1,34,14) = (/ &
     &2.6250e+05_rb,1.9687e+05_rb,1.3125e+05_rb,6.5624e+04_rb,4.1136e+01_rb/)
      kbo(:, 2,34,14) = (/ &
     &2.6605e+05_rb,1.9954e+05_rb,1.3302e+05_rb,6.6512e+04_rb,3.9143e+01_rb/)
      kbo(:, 3,34,14) = (/ &
     &2.6820e+05_rb,2.0115e+05_rb,1.3410e+05_rb,6.7050e+04_rb,3.6807e+01_rb/)
      kbo(:, 4,34,14) = (/ &
     &2.6900e+05_rb,2.0175e+05_rb,1.3450e+05_rb,6.7251e+04_rb,3.3160e+01_rb/)
      kbo(:, 5,34,14) = (/ &
     &2.6861e+05_rb,2.0146e+05_rb,1.3431e+05_rb,6.7153e+04_rb,3.1344e+01_rb/)
      kbo(:, 1,35,14) = (/ &
     &3.0587e+05_rb,2.2940e+05_rb,1.5293e+05_rb,7.6467e+04_rb,4.2099e+01_rb/)
      kbo(:, 2,35,14) = (/ &
     &3.1048e+05_rb,2.3286e+05_rb,1.5524e+05_rb,7.7621e+04_rb,3.9785e+01_rb/)
      kbo(:, 3,35,14) = (/ &
     &3.1341e+05_rb,2.3506e+05_rb,1.5670e+05_rb,7.8352e+04_rb,3.6072e+01_rb/)
      kbo(:, 4,35,14) = (/ &
     &3.1462e+05_rb,2.3596e+05_rb,1.5731e+05_rb,7.8655e+04_rb,3.3490e+01_rb/)
      kbo(:, 5,35,14) = (/ &
     &3.1438e+05_rb,2.3579e+05_rb,1.5719e+05_rb,7.8596e+04_rb,3.1828e+01_rb/)
      kbo(:, 1,36,14) = (/ &
     &3.5380e+05_rb,2.6535e+05_rb,1.7690e+05_rb,8.8450e+04_rb,4.2948e+01_rb/)
      kbo(:, 2,36,14) = (/ &
     &3.5987e+05_rb,2.6990e+05_rb,1.7993e+05_rb,8.9967e+04_rb,3.9945e+01_rb/)
      kbo(:, 3,36,14) = (/ &
     &3.6383e+05_rb,2.7287e+05_rb,1.8191e+05_rb,9.0957e+04_rb,3.5908e+01_rb/)
      kbo(:, 4,36,14) = (/ &
     &3.6571e+05_rb,2.7429e+05_rb,1.8286e+05_rb,9.1429e+04_rb,3.3881e+01_rb/)
      kbo(:, 5,36,14) = (/ &
     &3.6576e+05_rb,2.7432e+05_rb,1.8288e+05_rb,9.1441e+04_rb,3.2580e+01_rb/)
      kbo(:, 1,37,14) = (/ &
     &3.9789e+05_rb,2.9842e+05_rb,1.9894e+05_rb,9.9472e+04_rb,4.3924e+01_rb/)
      kbo(:, 2,37,14) = (/ &
     &4.0582e+05_rb,3.0436e+05_rb,2.0291e+05_rb,1.0145e+05_rb,4.0613e+01_rb/)
      kbo(:, 3,37,14) = (/ &
     &4.1126e+05_rb,3.0845e+05_rb,2.0563e+05_rb,1.0282e+05_rb,3.6440e+01_rb/)
      kbo(:, 4,37,14) = (/ &
     &4.1422e+05_rb,3.1066e+05_rb,2.0711e+05_rb,1.0355e+05_rb,3.4966e+01_rb/)
      kbo(:, 5,37,14) = (/ &
     &4.1491e+05_rb,3.1118e+05_rb,2.0745e+05_rb,1.0373e+05_rb,3.3736e+01_rb/)
      kbo(:, 1,38,14) = (/ &
     &4.5404e+05_rb,3.4053e+05_rb,2.2702e+05_rb,1.1351e+05_rb,4.4908e+01_rb/)
      kbo(:, 2,38,14) = (/ &
     &4.6441e+05_rb,3.4830e+05_rb,2.3220e+05_rb,1.1610e+05_rb,4.0968e+01_rb/)
      kbo(:, 3,38,14) = (/ &
     &4.7167e+05_rb,3.5375e+05_rb,2.3583e+05_rb,1.1792e+05_rb,3.7185e+01_rb/)
      kbo(:, 4,38,14) = (/ &
     &4.7591e+05_rb,3.5693e+05_rb,2.3796e+05_rb,1.1898e+05_rb,3.5958e+01_rb/)
      kbo(:, 5,38,14) = (/ &
     &4.7747e+05_rb,3.5810e+05_rb,2.3873e+05_rb,1.1937e+05_rb,3.4345e+01_rb/)
      kbo(:, 1,39,14) = (/ &
     &5.3006e+05_rb,3.9755e+05_rb,2.6503e+05_rb,1.3252e+05_rb,4.5571e+01_rb/)
      kbo(:, 2,39,14) = (/ &
     &5.4370e+05_rb,4.0778e+05_rb,2.7185e+05_rb,1.3593e+05_rb,4.1350e+01_rb/)
      kbo(:, 3,39,14) = (/ &
     &5.5324e+05_rb,4.1494e+05_rb,2.7663e+05_rb,1.3831e+05_rb,3.8461e+01_rb/)
      kbo(:, 4,39,14) = (/ &
     &5.5931e+05_rb,4.1948e+05_rb,2.7966e+05_rb,1.3983e+05_rb,3.7172e+01_rb/)
      kbo(:, 5,39,14) = (/ &
     &5.6198e+05_rb,4.2149e+05_rb,2.8099e+05_rb,1.4050e+05_rb,3.5387e+01_rb/)
      kbo(:, 1,40,14) = (/ &
     &5.7962e+05_rb,4.3471e+05_rb,2.8981e+05_rb,1.4490e+05_rb,4.6708e+01_rb/)
      kbo(:, 2,40,14) = (/ &
     &5.9661e+05_rb,4.4745e+05_rb,2.9830e+05_rb,1.4915e+05_rb,4.2510e+01_rb/)
      kbo(:, 3,40,14) = (/ &
     &6.0879e+05_rb,4.5659e+05_rb,3.0439e+05_rb,1.5220e+05_rb,3.9273e+01_rb/)
      kbo(:, 4,40,14) = (/ &
     &6.1695e+05_rb,4.6271e+05_rb,3.0847e+05_rb,1.5424e+05_rb,3.8101e+01_rb/)
      kbo(:, 5,40,14) = (/ &
     &6.2122e+05_rb,4.6592e+05_rb,3.1061e+05_rb,1.5530e+05_rb,3.6493e+01_rb/)
      kbo(:, 1,41,14) = (/ &
     &6.3170e+05_rb,4.7378e+05_rb,3.1585e+05_rb,1.5793e+05_rb,4.7871e+01_rb/)
      kbo(:, 2,41,14) = (/ &
     &6.5265e+05_rb,4.8949e+05_rb,3.2633e+05_rb,1.6317e+05_rb,4.3729e+01_rb/)
      kbo(:, 3,41,14) = (/ &
     &6.6799e+05_rb,5.0099e+05_rb,3.3400e+05_rb,1.6700e+05_rb,4.0263e+01_rb/)
      kbo(:, 4,41,14) = (/ &
     &6.7856e+05_rb,5.0892e+05_rb,3.3928e+05_rb,1.6964e+05_rb,3.8971e+01_rb/)
      kbo(:, 5,41,14) = (/ &
     &6.8482e+05_rb,5.1361e+05_rb,3.4241e+05_rb,1.7121e+05_rb,3.7519e+01_rb/)
      kbo(:, 1,42,14) = (/ &
     &6.9554e+05_rb,5.2166e+05_rb,3.4777e+05_rb,1.7388e+05_rb,4.9144e+01_rb/)
      kbo(:, 2,42,14) = (/ &
     &7.2122e+05_rb,5.4091e+05_rb,3.6061e+05_rb,1.8030e+05_rb,4.4903e+01_rb/)
      kbo(:, 3,42,14) = (/ &
     &7.4054e+05_rb,5.5540e+05_rb,3.7027e+05_rb,1.8513e+05_rb,4.1239e+01_rb/)
      kbo(:, 4,42,14) = (/ &
     &7.5407e+05_rb,5.6555e+05_rb,3.7703e+05_rb,1.8852e+05_rb,4.0019e+01_rb/)
      kbo(:, 5,42,14) = (/ &
     &7.6271e+05_rb,5.7203e+05_rb,3.8135e+05_rb,1.9067e+05_rb,3.8545e+01_rb/)
      kbo(:, 1,43,14) = (/ &
     &7.7014e+05_rb,5.7761e+05_rb,3.8508e+05_rb,1.9254e+05_rb,5.0575e+01_rb/)
      kbo(:, 2,43,14) = (/ &
     &8.0215e+05_rb,6.0161e+05_rb,4.0108e+05_rb,2.0054e+05_rb,4.6180e+01_rb/)
      kbo(:, 3,43,14) = (/ &
     &8.2687e+05_rb,6.2015e+05_rb,4.1344e+05_rb,2.0672e+05_rb,4.2529e+01_rb/)
      kbo(:, 4,43,14) = (/ &
     &8.4441e+05_rb,6.3331e+05_rb,4.2221e+05_rb,2.1110e+05_rb,4.0958e+01_rb/)
      kbo(:, 5,43,14) = (/ &
     &8.5614e+05_rb,6.4211e+05_rb,4.2807e+05_rb,2.1404e+05_rb,3.9632e+01_rb/)
      kbo(:, 1,44,14) = (/ &
     &8.6239e+05_rb,6.4679e+05_rb,4.3119e+05_rb,2.1559e+05_rb,5.2080e+01_rb/)
      kbo(:, 2,44,14) = (/ &
     &9.0261e+05_rb,6.7696e+05_rb,4.5130e+05_rb,2.2565e+05_rb,4.7569e+01_rb/)
      kbo(:, 3,44,14) = (/ &
     &9.3442e+05_rb,7.0081e+05_rb,4.6721e+05_rb,2.3360e+05_rb,4.3904e+01_rb/)
      kbo(:, 4,44,14) = (/ &
     &9.5755e+05_rb,7.1816e+05_rb,4.7877e+05_rb,2.3938e+05_rb,4.2000e+01_rb/)
      kbo(:, 5,44,14) = (/ &
     &9.7341e+05_rb,7.3005e+05_rb,4.8670e+05_rb,2.4335e+05_rb,4.0861e+01_rb/)
      kbo(:, 1,45,14) = (/ &
     &9.8377e+05_rb,7.3783e+05_rb,4.9189e+05_rb,2.4594e+05_rb,5.3464e+01_rb/)
      kbo(:, 2,45,14) = (/ &
     &1.0352e+06_rb,7.7641e+05_rb,5.1760e+05_rb,2.5880e+05_rb,4.8989e+01_rb/)
      kbo(:, 3,45,14) = (/ &
     &1.0762e+06_rb,8.0713e+05_rb,5.3809e+05_rb,2.6904e+05_rb,4.5355e+01_rb/)
      kbo(:, 4,45,14) = (/ &
     &1.1070e+06_rb,8.3023e+05_rb,5.5349e+05_rb,2.7674e+05_rb,4.3217e+01_rb/)
      kbo(:, 5,45,14) = (/ &
     &1.1287e+06_rb,8.4649e+05_rb,5.6433e+05_rb,2.8216e+05_rb,4.1953e+01_rb/)
      kbo(:, 1,46,14) = (/ &
     &1.1307e+06_rb,8.4801e+05_rb,5.6534e+05_rb,2.8267e+05_rb,5.4903e+01_rb/)
      kbo(:, 2,46,14) = (/ &
     &1.1967e+06_rb,8.9750e+05_rb,5.9834e+05_rb,2.9917e+05_rb,5.0473e+01_rb/)
      kbo(:, 3,46,14) = (/ &
     &1.2497e+06_rb,9.3728e+05_rb,6.2485e+05_rb,3.1243e+05_rb,4.7017e+01_rb/)
      kbo(:, 4,46,14) = (/ &
     &1.2910e+06_rb,9.6823e+05_rb,6.4549e+05_rb,3.2274e+05_rb,4.4343e+01_rb/)
      kbo(:, 5,46,14) = (/ &
     &1.3205e+06_rb,9.9037e+05_rb,6.6025e+05_rb,3.3012e+05_rb,4.3085e+01_rb/)
      kbo(:, 1,47,14) = (/ &
     &1.2818e+06_rb,9.6133e+05_rb,6.4089e+05_rb,3.2045e+05_rb,5.6436e+01_rb/)
      kbo(:, 2,47,14) = (/ &
     &1.3655e+06_rb,1.0241e+06_rb,6.8273e+05_rb,3.4137e+05_rb,5.2412e+01_rb/)
      kbo(:, 3,47,14) = (/ &
     &1.4341e+06_rb,1.0756e+06_rb,7.1708e+05_rb,3.5854e+05_rb,4.8254e+01_rb/)
      kbo(:, 4,47,14) = (/ &
     &1.4885e+06_rb,1.1164e+06_rb,7.4427e+05_rb,3.7214e+05_rb,4.5497e+01_rb/)
      kbo(:, 5,47,14) = (/ &
     &1.5285e+06_rb,1.1464e+06_rb,7.6425e+05_rb,3.8213e+05_rb,4.4276e+01_rb/)
      kbo(:, 1,48,14) = (/ &
     &1.4912e+06_rb,1.1184e+06_rb,7.4558e+05_rb,3.7279e+05_rb,5.8070e+01_rb/)
      kbo(:, 2,48,14) = (/ &
     &1.5992e+06_rb,1.1994e+06_rb,7.9959e+05_rb,3.9979e+05_rb,5.4067e+01_rb/)
      kbo(:, 3,48,14) = (/ &
     &1.6898e+06_rb,1.2674e+06_rb,8.4492e+05_rb,4.2246e+05_rb,5.0095e+01_rb/)
      kbo(:, 4,48,14) = (/ &
     &1.7626e+06_rb,1.3220e+06_rb,8.8132e+05_rb,4.4066e+05_rb,4.6945e+01_rb/)
      kbo(:, 5,48,14) = (/ &
     &1.8180e+06_rb,1.3635e+06_rb,9.0900e+05_rb,4.5450e+05_rb,4.5438e+01_rb/)
      kbo(:, 1,49,14) = (/ &
     &1.8033e+06_rb,1.3525e+06_rb,9.0165e+05_rb,4.5083e+05_rb,5.9742e+01_rb/)
      kbo(:, 2,49,14) = (/ &
     &1.9472e+06_rb,1.4604e+06_rb,9.7361e+05_rb,4.8680e+05_rb,5.5654e+01_rb/)
      kbo(:, 3,49,14) = (/ &
     &2.0706e+06_rb,1.5529e+06_rb,1.0353e+06_rb,5.1764e+05_rb,5.1796e+01_rb/)
      kbo(:, 4,49,14) = (/ &
     &2.1711e+06_rb,1.6283e+06_rb,1.0855e+06_rb,5.4277e+05_rb,4.8429e+01_rb/)
      kbo(:, 5,49,14) = (/ &
     &2.2497e+06_rb,1.6872e+06_rb,1.1248e+06_rb,5.6241e+05_rb,4.6805e+01_rb/)
      kbo(:, 1,50,14) = (/ &
     &2.0759e+06_rb,1.5569e+06_rb,1.0379e+06_rb,5.1897e+05_rb,6.1720e+01_rb/)
      kbo(:, 2,50,14) = (/ &
     &2.2580e+06_rb,1.6935e+06_rb,1.1290e+06_rb,5.6451e+05_rb,5.7265e+01_rb/)
      kbo(:, 3,50,14) = (/ &
     &2.4154e+06_rb,1.8116e+06_rb,1.2077e+06_rb,6.0386e+05_rb,5.3335e+01_rb/)
      kbo(:, 4,50,14) = (/ &
     &2.5463e+06_rb,1.9097e+06_rb,1.2731e+06_rb,6.3657e+05_rb,5.0026e+01_rb/)
      kbo(:, 5,50,14) = (/ &
     &2.6508e+06_rb,1.9881e+06_rb,1.3254e+06_rb,6.6270e+05_rb,4.7879e+01_rb/)
      kbo(:, 1,51,14) = (/ &
     &2.3343e+06_rb,1.7507e+06_rb,1.1671e+06_rb,5.8357e+05_rb,6.3629e+01_rb/)
      kbo(:, 2,51,14) = (/ &
     &2.5583e+06_rb,1.9187e+06_rb,1.2791e+06_rb,6.3957e+05_rb,5.8823e+01_rb/)
      kbo(:, 3,51,14) = (/ &
     &2.7536e+06_rb,2.0652e+06_rb,1.3768e+06_rb,6.8840e+05_rb,5.4830e+01_rb/)
      kbo(:, 4,51,14) = (/ &
     &2.9192e+06_rb,2.1894e+06_rb,1.4596e+06_rb,7.2979e+05_rb,5.1797e+01_rb/)
      kbo(:, 5,51,14) = (/ &
     &3.0520e+06_rb,2.2890e+06_rb,1.5260e+06_rb,7.6300e+05_rb,4.8739e+01_rb/)
      kbo(:, 1,52,14) = (/ &
     &2.6927e+06_rb,2.0196e+06_rb,1.3464e+06_rb,6.7319e+05_rb,6.5473e+01_rb/)
      kbo(:, 2,52,14) = (/ &
     &2.9755e+06_rb,2.2317e+06_rb,1.4878e+06_rb,7.4389e+05_rb,6.0202e+01_rb/)
      kbo(:, 3,52,14) = (/ &
     &3.2232e+06_rb,2.4174e+06_rb,1.6116e+06_rb,8.0581e+05_rb,5.6406e+01_rb/)
      kbo(:, 4,52,14) = (/ &
     &3.4356e+06_rb,2.5767e+06_rb,1.7178e+06_rb,8.5891e+05_rb,5.3338e+01_rb/)
      kbo(:, 5,52,14) = (/ &
     &3.6105e+06_rb,2.7079e+06_rb,1.8053e+06_rb,9.0264e+05_rb,4.9535e+01_rb/)
      kbo(:, 1,53,14) = (/ &
     &3.2282e+06_rb,2.4212e+06_rb,1.6141e+06_rb,8.0707e+05_rb,6.7136e+01_rb/)
      kbo(:, 2,53,14) = (/ &
     &3.5968e+06_rb,2.6976e+06_rb,1.7984e+06_rb,8.9921e+05_rb,6.1503e+01_rb/)
      kbo(:, 3,53,14) = (/ &
     &3.9238e+06_rb,2.9429e+06_rb,1.9619e+06_rb,9.8097e+05_rb,5.8062e+01_rb/)
      kbo(:, 4,53,14) = (/ &
     &4.2071e+06_rb,3.1553e+06_rb,2.1035e+06_rb,1.0518e+06_rb,5.4982e+01_rb/)
      kbo(:, 5,53,14) = (/ &
     &4.4446e+06_rb,3.3334e+06_rb,2.2223e+06_rb,1.1112e+06_rb,5.0414e+01_rb/)
      kbo(:, 1,54,14) = (/ &
     &3.3478e+06_rb,2.5109e+06_rb,1.6739e+06_rb,8.3696e+05_rb,6.8647e+01_rb/)
      kbo(:, 2,54,14) = (/ &
     &3.7598e+06_rb,2.8199e+06_rb,1.8799e+06_rb,9.3996e+05_rb,6.3327e+01_rb/)
      kbo(:, 3,54,14) = (/ &
     &4.1308e+06_rb,3.0981e+06_rb,2.0654e+06_rb,1.0327e+06_rb,5.9705e+01_rb/)
      kbo(:, 4,54,14) = (/ &
     &4.4534e+06_rb,3.3401e+06_rb,2.2267e+06_rb,1.1134e+06_rb,5.5841e+01_rb/)
      kbo(:, 5,54,14) = (/ &
     &4.7287e+06_rb,3.5465e+06_rb,2.3644e+06_rb,1.1822e+06_rb,5.1312e+01_rb/)
      kbo(:, 1,55,14) = (/ &
     &3.1636e+06_rb,2.3727e+06_rb,1.5818e+06_rb,7.9090e+05_rb,6.9800e+01_rb/)
      kbo(:, 2,55,14) = (/ &
     &3.5827e+06_rb,2.6870e+06_rb,1.7914e+06_rb,8.9569e+05_rb,6.5650e+01_rb/)
      kbo(:, 3,55,14) = (/ &
     &3.9638e+06_rb,2.9728e+06_rb,1.9819e+06_rb,9.9095e+05_rb,6.0566e+01_rb/)
      kbo(:, 4,55,14) = (/ &
     &4.2985e+06_rb,3.2239e+06_rb,2.1493e+06_rb,1.0746e+06_rb,5.6657e+01_rb/)
      kbo(:, 5,55,14) = (/ &
     &4.5858e+06_rb,3.4394e+06_rb,2.2929e+06_rb,1.1465e+06_rb,5.3531e+01_rb/)
      kbo(:, 1,56,14) = (/ &
     &2.9822e+06_rb,2.2367e+06_rb,1.4911e+06_rb,7.4555e+05_rb,7.1031e+01_rb/)
      kbo(:, 2,56,14) = (/ &
     &3.4067e+06_rb,2.5550e+06_rb,1.7033e+06_rb,8.5166e+05_rb,6.7133e+01_rb/)
      kbo(:, 3,56,14) = (/ &
     &3.7967e+06_rb,2.8475e+06_rb,1.8983e+06_rb,9.4915e+05_rb,6.1784e+01_rb/)
      kbo(:, 4,56,14) = (/ &
     &4.1424e+06_rb,3.1068e+06_rb,2.0712e+06_rb,1.0356e+06_rb,5.8142e+01_rb/)
      kbo(:, 5,56,14) = (/ &
     &4.4422e+06_rb,3.3317e+06_rb,2.2211e+06_rb,1.1105e+06_rb,5.4348e+01_rb/)
      kbo(:, 1,57,14) = (/ &
     &2.8003e+06_rb,2.1002e+06_rb,1.4001e+06_rb,7.0006e+05_rb,7.2686e+01_rb/)
      kbo(:, 2,57,14) = (/ &
     &3.2322e+06_rb,2.4241e+06_rb,1.6161e+06_rb,8.0804e+05_rb,6.8121e+01_rb/)
      kbo(:, 3,57,14) = (/ &
     &3.6290e+06_rb,2.7217e+06_rb,1.8145e+06_rb,9.0724e+05_rb,6.3933e+01_rb/)
      kbo(:, 4,57,14) = (/ &
     &3.9857e+06_rb,2.9893e+06_rb,1.9928e+06_rb,9.9642e+05_rb,5.9242e+01_rb/)
      kbo(:, 5,57,14) = (/ &
     &4.2966e+06_rb,3.2224e+06_rb,2.1482e+06_rb,1.0741e+06_rb,5.5180e+01_rb/)
      kbo(:, 1,58,14) = (/ &
     &6.9277e+05_rb,6.3516e+05_rb,5.4460e+05_rb,3.8143e+05_rb,7.4521e+01_rb/)
      kbo(:, 2,58,14) = (/ &
     &8.0841e+05_rb,7.4119e+05_rb,6.3550e+05_rb,4.4510e+05_rb,6.9466e+01_rb/)
      kbo(:, 3,58,14) = (/ &
     &9.1451e+05_rb,8.3847e+05_rb,7.1891e+05_rb,5.0352e+05_rb,6.5445e+01_rb/)
      kbo(:, 4,58,14) = (/ &
     &1.0112e+06_rb,9.2715e+05_rb,7.9494e+05_rb,5.5677e+05_rb,6.0252e+01_rb/)
      kbo(:, 5,58,14) = (/ &
     &1.0957e+06_rb,1.0046e+06_rb,8.6135e+05_rb,6.0328e+05_rb,5.6261e+01_rb/)
      kbo(:, 1,59,14) = (/ &
     &7.9105e+05_rb,6.9072e+05_rb,5.5096e+05_rb,3.4284e+05_rb,7.5082e+01_rb/)
      kbo(:, 2,59,14) = (/ &
     &9.2759e+05_rb,8.0994e+05_rb,6.4605e+05_rb,4.0201e+05_rb,7.0591e+01_rb/)
      kbo(:, 3,59,14) = (/ &
     &1.0529e+06_rb,9.1931e+05_rb,7.3330e+05_rb,4.5630e+05_rb,6.5689e+01_rb/)
      kbo(:, 4,59,14) = (/ &
     &1.1673e+06_rb,1.0193e+06_rb,8.1302e+05_rb,5.0591e+05_rb,6.1427e+01_rb/)
      kbo(:, 5,59,14) = (/ &
     &1.2680e+06_rb,1.1071e+06_rb,8.8311e+05_rb,5.4953e+05_rb,5.7108e+01_rb/)
      kbo(:, 1,13,15) = (/ &
     &6.8853e+05_rb,5.1640e+05_rb,3.4427e+05_rb,1.7213e+05_rb,8.3646e+00_rb/)
      kbo(:, 2,13,15) = (/ &
     &6.7841e+05_rb,5.0881e+05_rb,3.3921e+05_rb,1.6961e+05_rb,8.1557e+00_rb/)
      kbo(:, 3,13,15) = (/ &
     &6.7058e+05_rb,5.0293e+05_rb,3.3529e+05_rb,1.6765e+05_rb,7.2355e+00_rb/)
      kbo(:, 4,13,15) = (/ &
     &6.6505e+05_rb,4.9879e+05_rb,3.3253e+05_rb,1.6627e+05_rb,7.1104e+00_rb/)
      kbo(:, 5,13,15) = (/ &
     &6.5977e+05_rb,4.9483e+05_rb,3.2989e+05_rb,1.6494e+05_rb,5.0356e+00_rb/)
      kbo(:, 1,14,15) = (/ &
     &4.8859e+05_rb,3.6644e+05_rb,2.4429e+05_rb,1.2215e+05_rb,9.3416e+00_rb/)
      kbo(:, 2,14,15) = (/ &
     &4.8530e+05_rb,3.6397e+05_rb,2.4265e+05_rb,1.2132e+05_rb,9.0997e+00_rb/)
      kbo(:, 3,14,15) = (/ &
     &4.8323e+05_rb,3.6243e+05_rb,2.4161e+05_rb,1.2081e+05_rb,8.4842e+00_rb/)
      kbo(:, 4,14,15) = (/ &
     &4.8076e+05_rb,3.6057e+05_rb,2.4038e+05_rb,1.2019e+05_rb,8.2141e+00_rb/)
      kbo(:, 5,14,15) = (/ &
     &4.7824e+05_rb,3.5868e+05_rb,2.3912e+05_rb,1.1956e+05_rb,7.9624e+00_rb/)
      kbo(:, 1,15,15) = (/ &
     &3.5725e+05_rb,2.6794e+05_rb,1.7863e+05_rb,8.9315e+04_rb,1.0416e+01_rb/)
      kbo(:, 2,15,15) = (/ &
     &3.5701e+05_rb,2.6776e+05_rb,1.7851e+05_rb,8.9255e+04_rb,1.0132e+01_rb/)
      kbo(:, 3,15,15) = (/ &
     &3.5620e+05_rb,2.6715e+05_rb,1.7810e+05_rb,8.9053e+04_rb,9.8342e+00_rb/)
      kbo(:, 4,15,15) = (/ &
     &3.5534e+05_rb,2.6651e+05_rb,1.7767e+05_rb,8.8838e+04_rb,8.7353e+00_rb/)
      kbo(:, 5,15,15) = (/ &
     &3.5411e+05_rb,2.6559e+05_rb,1.7706e+05_rb,8.8531e+04_rb,9.0528e+00_rb/)
      kbo(:, 1,16,15) = (/ &
     &3.1303e+05_rb,2.3478e+05_rb,1.5652e+05_rb,7.8260e+04_rb,1.1678e+01_rb/)
      kbo(:, 2,16,15) = (/ &
     &3.1296e+05_rb,2.3472e+05_rb,1.5648e+05_rb,7.8243e+04_rb,1.1373e+01_rb/)
      kbo(:, 3,16,15) = (/ &
     &3.1311e+05_rb,2.3483e+05_rb,1.5655e+05_rb,7.8278e+04_rb,1.1063e+01_rb/)
      kbo(:, 4,16,15) = (/ &
     &3.1270e+05_rb,2.3452e+05_rb,1.5635e+05_rb,7.8176e+04_rb,1.0185e+01_rb/)
      kbo(:, 5,16,15) = (/ &
     &3.1216e+05_rb,2.3412e+05_rb,1.5608e+05_rb,7.8040e+04_rb,1.0166e+01_rb/)
      kbo(:, 1,17,15) = (/ &
     &2.8440e+05_rb,2.1330e+05_rb,1.4220e+05_rb,7.1101e+04_rb,1.3334e+01_rb/)
      kbo(:, 2,17,15) = (/ &
     &2.8479e+05_rb,2.1359e+05_rb,1.4240e+05_rb,7.1199e+04_rb,1.2996e+01_rb/)
      kbo(:, 3,17,15) = (/ &
     &2.8480e+05_rb,2.1360e+05_rb,1.4240e+05_rb,7.1202e+04_rb,1.2651e+01_rb/)
      kbo(:, 4,17,15) = (/ &
     &2.8481e+05_rb,2.1361e+05_rb,1.4240e+05_rb,7.1203e+04_rb,1.1964e+01_rb/)
      kbo(:, 5,17,15) = (/ &
     &2.8402e+05_rb,2.1301e+05_rb,1.4201e+05_rb,7.1006e+04_rb,1.1649e+01_rb/)
      kbo(:, 1,18,15) = (/ &
     &2.6460e+05_rb,1.9845e+05_rb,1.3230e+05_rb,6.6150e+04_rb,1.5350e+01_rb/)
      kbo(:, 2,18,15) = (/ &
     &2.6490e+05_rb,1.9868e+05_rb,1.3245e+05_rb,6.6227e+04_rb,1.4954e+01_rb/)
      kbo(:, 3,18,15) = (/ &
     &2.6505e+05_rb,1.9879e+05_rb,1.3252e+05_rb,6.6263e+04_rb,1.4553e+01_rb/)
      kbo(:, 4,18,15) = (/ &
     &2.6448e+05_rb,1.9836e+05_rb,1.3224e+05_rb,6.6121e+04_rb,1.4057e+01_rb/)
      kbo(:, 5,18,15) = (/ &
     &2.6357e+05_rb,1.9768e+05_rb,1.3179e+05_rb,6.5894e+04_rb,1.3341e+01_rb/)
      kbo(:, 1,19,15) = (/ &
     &2.4108e+05_rb,1.8081e+05_rb,1.2054e+05_rb,6.0272e+04_rb,1.7626e+01_rb/)
      kbo(:, 2,19,15) = (/ &
     &2.4160e+05_rb,1.8120e+05_rb,1.2080e+05_rb,6.0400e+04_rb,1.7162e+01_rb/)
      kbo(:, 3,19,15) = (/ &
     &2.4138e+05_rb,1.8103e+05_rb,1.2069e+05_rb,6.0347e+04_rb,1.6660e+01_rb/)
      kbo(:, 4,19,15) = (/ &
     &2.4078e+05_rb,1.8058e+05_rb,1.2039e+05_rb,6.0195e+04_rb,1.6133e+01_rb/)
      kbo(:, 5,19,15) = (/ &
     &2.3956e+05_rb,1.7967e+05_rb,1.1978e+05_rb,5.9892e+04_rb,1.5032e+01_rb/)
      kbo(:, 1,20,15) = (/ &
     &2.3801e+05_rb,1.7851e+05_rb,1.1901e+05_rb,5.9503e+04_rb,2.0107e+01_rb/)
      kbo(:, 2,20,15) = (/ &
     &2.3824e+05_rb,1.7868e+05_rb,1.1912e+05_rb,5.9560e+04_rb,1.9545e+01_rb/)
      kbo(:, 3,20,15) = (/ &
     &2.3790e+05_rb,1.7842e+05_rb,1.1895e+05_rb,5.9475e+04_rb,1.8935e+01_rb/)
      kbo(:, 4,20,15) = (/ &
     &2.3694e+05_rb,1.7770e+05_rb,1.1847e+05_rb,5.9235e+04_rb,1.8318e+01_rb/)
      kbo(:, 5,20,15) = (/ &
     &2.3546e+05_rb,1.7659e+05_rb,1.1773e+05_rb,5.8866e+04_rb,1.6971e+01_rb/)
      kbo(:, 1,21,15) = (/ &
     &2.4153e+05_rb,1.8115e+05_rb,1.2077e+05_rb,6.0385e+04_rb,2.2844e+01_rb/)
      kbo(:, 2,21,15) = (/ &
     &2.4153e+05_rb,1.8115e+05_rb,1.2076e+05_rb,6.0383e+04_rb,2.2202e+01_rb/)
      kbo(:, 3,21,15) = (/ &
     &2.4090e+05_rb,1.8068e+05_rb,1.2045e+05_rb,6.0226e+04_rb,2.1535e+01_rb/)
      kbo(:, 4,21,15) = (/ &
     &2.3957e+05_rb,1.7968e+05_rb,1.1978e+05_rb,5.9893e+04_rb,2.0842e+01_rb/)
      kbo(:, 5,21,15) = (/ &
     &2.3762e+05_rb,1.7822e+05_rb,1.1881e+05_rb,5.9406e+04_rb,1.9479e+01_rb/)
      kbo(:, 1,22,15) = (/ &
     &2.5014e+05_rb,1.8760e+05_rb,1.2507e+05_rb,6.2536e+04_rb,2.6039e+01_rb/)
      kbo(:, 2,22,15) = (/ &
     &2.4958e+05_rb,1.8719e+05_rb,1.2479e+05_rb,6.2397e+04_rb,2.5280e+01_rb/)
      kbo(:, 3,22,15) = (/ &
     &2.4838e+05_rb,1.8628e+05_rb,1.2419e+05_rb,6.2096e+04_rb,2.4490e+01_rb/)
      kbo(:, 4,22,15) = (/ &
     &2.4655e+05_rb,1.8492e+05_rb,1.2328e+05_rb,6.1639e+04_rb,2.3671e+01_rb/)
      kbo(:, 5,22,15) = (/ &
     &2.4402e+05_rb,1.8302e+05_rb,1.2201e+05_rb,6.1006e+04_rb,2.2182e+01_rb/)
      kbo(:, 1,23,15) = (/ &
     &2.5892e+05_rb,1.9419e+05_rb,1.2946e+05_rb,6.4730e+04_rb,2.9635e+01_rb/)
      kbo(:, 2,23,15) = (/ &
     &2.5767e+05_rb,1.9325e+05_rb,1.2883e+05_rb,6.4418e+04_rb,2.8708e+01_rb/)
      kbo(:, 3,23,15) = (/ &
     &2.5584e+05_rb,1.9188e+05_rb,1.2792e+05_rb,6.3960e+04_rb,2.7743e+01_rb/)
      kbo(:, 4,23,15) = (/ &
     &2.5321e+05_rb,1.8991e+05_rb,1.2660e+05_rb,6.3303e+04_rb,2.6764e+01_rb/)
      kbo(:, 5,23,15) = (/ &
     &2.5010e+05_rb,1.8758e+05_rb,1.2505e+05_rb,6.2526e+04_rb,2.5122e+01_rb/)
      kbo(:, 1,24,15) = (/ &
     &2.6502e+05_rb,1.9877e+05_rb,1.3251e+05_rb,6.6257e+04_rb,3.3543e+01_rb/)
      kbo(:, 2,24,15) = (/ &
     &2.6308e+05_rb,1.9731e+05_rb,1.3154e+05_rb,6.5771e+04_rb,3.2390e+01_rb/)
      kbo(:, 3,24,15) = (/ &
     &2.6033e+05_rb,1.9524e+05_rb,1.3016e+05_rb,6.5082e+04_rb,3.1202e+01_rb/)
      kbo(:, 4,24,15) = (/ &
     &2.5704e+05_rb,1.9278e+05_rb,1.2852e+05_rb,6.4262e+04_rb,3.0027e+01_rb/)
      kbo(:, 5,24,15) = (/ &
     &2.5313e+05_rb,1.8984e+05_rb,1.2656e+05_rb,6.3282e+04_rb,2.8238e+01_rb/)
      kbo(:, 1,25,15) = (/ &
     &2.7306e+05_rb,2.0480e+05_rb,1.3653e+05_rb,6.8266e+04_rb,3.7656e+01_rb/)
      kbo(:, 2,25,15) = (/ &
     &2.7003e+05_rb,2.0252e+05_rb,1.3501e+05_rb,6.7507e+04_rb,3.6233e+01_rb/)
      kbo(:, 3,25,15) = (/ &
     &2.6644e+05_rb,1.9983e+05_rb,1.3322e+05_rb,6.6611e+04_rb,3.4804e+01_rb/)
      kbo(:, 4,25,15) = (/ &
     &2.6230e+05_rb,1.9672e+05_rb,1.3115e+05_rb,6.5574e+04_rb,3.3361e+01_rb/)
      kbo(:, 5,25,15) = (/ &
     &2.5745e+05_rb,1.9309e+05_rb,1.2873e+05_rb,6.4363e+04_rb,3.1338e+01_rb/)
      kbo(:, 1,26,15) = (/ &
     &2.8347e+05_rb,2.1261e+05_rb,1.4174e+05_rb,7.0869e+04_rb,4.1820e+01_rb/)
      kbo(:, 2,26,15) = (/ &
     &2.7951e+05_rb,2.0963e+05_rb,1.3976e+05_rb,6.9878e+04_rb,4.0076e+01_rb/)
      kbo(:, 3,26,15) = (/ &
     &2.7489e+05_rb,2.0617e+05_rb,1.3745e+05_rb,6.8723e+04_rb,3.8351e+01_rb/)
      kbo(:, 4,26,15) = (/ &
     &2.6958e+05_rb,2.0219e+05_rb,1.3479e+05_rb,6.7396e+04_rb,3.6631e+01_rb/)
      kbo(:, 5,26,15) = (/ &
     &2.6383e+05_rb,1.9788e+05_rb,1.3192e+05_rb,6.5959e+04_rb,3.4286e+01_rb/)
      kbo(:, 1,27,15) = (/ &
     &3.0277e+05_rb,2.2708e+05_rb,1.5139e+05_rb,7.5694e+04_rb,4.5895e+01_rb/)
      kbo(:, 2,27,15) = (/ &
     &2.9771e+05_rb,2.2329e+05_rb,1.4886e+05_rb,7.4429e+04_rb,4.3819e+01_rb/)
      kbo(:, 3,27,15) = (/ &
     &2.9175e+05_rb,2.1881e+05_rb,1.4587e+05_rb,7.2938e+04_rb,4.1746e+01_rb/)
      kbo(:, 4,27,15) = (/ &
     &2.8521e+05_rb,2.1391e+05_rb,1.4261e+05_rb,7.1304e+04_rb,3.9720e+01_rb/)
      kbo(:, 5,27,15) = (/ &
     &2.7826e+05_rb,2.0870e+05_rb,1.3913e+05_rb,6.9566e+04_rb,3.6964e+01_rb/)
      kbo(:, 1,28,15) = (/ &
     &3.2897e+05_rb,2.4673e+05_rb,1.6449e+05_rb,8.2243e+04_rb,4.9767e+01_rb/)
      kbo(:, 2,28,15) = (/ &
     &3.2244e+05_rb,2.4183e+05_rb,1.6122e+05_rb,8.0610e+04_rb,4.7309e+01_rb/)
      kbo(:, 3,28,15) = (/ &
     &3.1503e+05_rb,2.3628e+05_rb,1.5752e+05_rb,7.8759e+04_rb,4.4889e+01_rb/)
      kbo(:, 4,28,15) = (/ &
     &3.0709e+05_rb,2.3032e+05_rb,1.5355e+05_rb,7.6774e+04_rb,4.2541e+01_rb/)
      kbo(:, 5,28,15) = (/ &
     &2.9858e+05_rb,2.2393e+05_rb,1.4929e+05_rb,7.4645e+04_rb,3.9310e+01_rb/)
      kbo(:, 1,29,15) = (/ &
     &3.7585e+05_rb,2.8188e+05_rb,1.8792e+05_rb,9.3962e+04_rb,5.3281e+01_rb/)
      kbo(:, 2,29,15) = (/ &
     &3.6712e+05_rb,2.7534e+05_rb,1.8356e+05_rb,9.1780e+04_rb,5.0424e+01_rb/)
      kbo(:, 3,29,15) = (/ &
     &3.5767e+05_rb,2.6826e+05_rb,1.7884e+05_rb,8.9419e+04_rb,4.7662e+01_rb/)
      kbo(:, 4,29,15) = (/ &
     &3.4765e+05_rb,2.6073e+05_rb,1.7382e+05_rb,8.6912e+04_rb,4.4800e+01_rb/)
      kbo(:, 5,29,15) = (/ &
     &3.3714e+05_rb,2.5285e+05_rb,1.6857e+05_rb,8.4284e+04_rb,4.1166e+01_rb/)
      kbo(:, 1,30,15) = (/ &
     &4.3604e+05_rb,3.2703e+05_rb,2.1802e+05_rb,1.0901e+05_rb,5.6346e+01_rb/)
      kbo(:, 2,30,15) = (/ &
     &4.2462e+05_rb,3.1847e+05_rb,2.1231e+05_rb,1.0616e+05_rb,5.3119e+01_rb/)
      kbo(:, 3,30,15) = (/ &
     &4.1259e+05_rb,3.0944e+05_rb,2.0629e+05_rb,1.0315e+05_rb,5.0028e+01_rb/)
      kbo(:, 4,30,15) = (/ &
     &3.9989e+05_rb,2.9992e+05_rb,1.9995e+05_rb,9.9973e+04_rb,4.6067e+01_rb/)
      kbo(:, 5,30,15) = (/ &
     &3.8696e+05_rb,2.9022e+05_rb,1.9348e+05_rb,9.6741e+04_rb,4.3622e+01_rb/)
      kbo(:, 1,31,15) = (/ &
     &5.2737e+05_rb,3.9553e+05_rb,2.6369e+05_rb,1.3184e+05_rb,5.8898e+01_rb/)
      kbo(:, 2,31,15) = (/ &
     &5.1226e+05_rb,3.8420e+05_rb,2.5613e+05_rb,1.2807e+05_rb,5.5332e+01_rb/)
      kbo(:, 3,31,15) = (/ &
     &4.9638e+05_rb,3.7229e+05_rb,2.4819e+05_rb,1.2410e+05_rb,5.1303e+01_rb/)
      kbo(:, 4,31,15) = (/ &
     &4.8006e+05_rb,3.6004e+05_rb,2.4003e+05_rb,1.2001e+05_rb,4.7154e+01_rb/)
      kbo(:, 5,31,15) = (/ &
     &4.6345e+05_rb,3.4759e+05_rb,2.3173e+05_rb,1.1586e+05_rb,4.5673e+01_rb/)
      kbo(:, 1,32,15) = (/ &
     &6.4038e+05_rb,4.8028e+05_rb,3.2019e+05_rb,1.6010e+05_rb,6.0926e+01_rb/)
      kbo(:, 2,32,15) = (/ &
     &6.2037e+05_rb,4.6528e+05_rb,3.1019e+05_rb,1.5509e+05_rb,5.7062e+01_rb/)
      kbo(:, 3,32,15) = (/ &
     &5.9976e+05_rb,4.4982e+05_rb,2.9988e+05_rb,1.4994e+05_rb,5.1911e+01_rb/)
      kbo(:, 4,32,15) = (/ &
     &5.7886e+05_rb,4.3415e+05_rb,2.8943e+05_rb,1.4472e+05_rb,4.9758e+01_rb/)
      kbo(:, 5,32,15) = (/ &
     &5.5779e+05_rb,4.1835e+05_rb,2.7890e+05_rb,1.3945e+05_rb,4.7124e+01_rb/)
      kbo(:, 1,33,15) = (/ &
     &7.8604e+05_rb,5.8953e+05_rb,3.9302e+05_rb,1.9651e+05_rb,6.2449e+01_rb/)
      kbo(:, 2,33,15) = (/ &
     &7.5994e+05_rb,5.6995e+05_rb,3.7997e+05_rb,1.8999e+05_rb,5.7026e+01_rb/)
      kbo(:, 3,33,15) = (/ &
     &7.3324e+05_rb,5.4993e+05_rb,3.6662e+05_rb,1.8331e+05_rb,5.3959e+01_rb/)
      kbo(:, 4,33,15) = (/ &
     &7.0633e+05_rb,5.2975e+05_rb,3.5316e+05_rb,1.7658e+05_rb,5.1185e+01_rb/)
      kbo(:, 5,33,15) = (/ &
     &6.7933e+05_rb,5.0950e+05_rb,3.3967e+05_rb,1.6983e+05_rb,4.6145e+01_rb/)
      kbo(:, 1,34,15) = (/ &
     &9.3629e+05_rb,7.0222e+05_rb,4.6814e+05_rb,2.3407e+05_rb,6.2950e+01_rb/)
      kbo(:, 2,34,15) = (/ &
     &9.0361e+05_rb,6.7770e+05_rb,4.5180e+05_rb,2.2590e+05_rb,5.7628e+01_rb/)
      kbo(:, 3,34,15) = (/ &
     &8.7048e+05_rb,6.5286e+05_rb,4.3524e+05_rb,2.1762e+05_rb,5.5397e+01_rb/)
      kbo(:, 4,34,15) = (/ &
     &8.3718e+05_rb,6.2788e+05_rb,4.1859e+05_rb,2.0929e+05_rb,5.1885e+01_rb/)
      kbo(:, 5,34,15) = (/ &
     &8.0393e+05_rb,6.0294e+05_rb,4.0196e+05_rb,2.0098e+05_rb,4.4833e+01_rb/)
      kbo(:, 1,35,15) = (/ &
     &1.1151e+06_rb,8.3633e+05_rb,5.5755e+05_rb,2.7878e+05_rb,6.3401e+01_rb/)
      kbo(:, 2,35,15) = (/ &
     &1.0748e+06_rb,8.0608e+05_rb,5.3739e+05_rb,2.6869e+05_rb,5.9975e+01_rb/)
      kbo(:, 3,35,15) = (/ &
     &1.0341e+06_rb,7.7556e+05_rb,5.1704e+05_rb,2.5852e+05_rb,5.6633e+01_rb/)
      kbo(:, 4,35,15) = (/ &
     &9.9325e+05_rb,7.4494e+05_rb,4.9662e+05_rb,2.4831e+05_rb,5.0172e+01_rb/)
      kbo(:, 5,35,15) = (/ &
     &9.5274e+05_rb,7.1455e+05_rb,4.7637e+05_rb,2.3818e+05_rb,4.3180e+01_rb/)
      kbo(:, 1,36,15) = (/ &
     &1.3204e+06_rb,9.9032e+05_rb,6.6021e+05_rb,3.3011e+05_rb,6.4479e+01_rb/)
      kbo(:, 2,36,15) = (/ &
     &1.2713e+06_rb,9.5347e+05_rb,6.3565e+05_rb,3.1782e+05_rb,6.1661e+01_rb/)
      kbo(:, 3,36,15) = (/ &
     &1.2219e+06_rb,9.1640e+05_rb,6.1093e+05_rb,3.0547e+05_rb,5.7543e+01_rb/)
      kbo(:, 4,36,15) = (/ &
     &1.1726e+06_rb,8.7947e+05_rb,5.8632e+05_rb,2.9316e+05_rb,4.9268e+01_rb/)
      kbo(:, 5,36,15) = (/ &
     &1.1240e+06_rb,8.4303e+05_rb,5.6202e+05_rb,2.8101e+05_rb,4.3422e+01_rb/)
      kbo(:, 1,37,15) = (/ &
     &1.5283e+06_rb,1.1462e+06_rb,7.6413e+05_rb,3.8207e+05_rb,6.6367e+01_rb/)
      kbo(:, 2,37,15) = (/ &
     &1.4706e+06_rb,1.1030e+06_rb,7.3532e+05_rb,3.6766e+05_rb,6.3620e+01_rb/)
      kbo(:, 3,37,15) = (/ &
     &1.4128e+06_rb,1.0596e+06_rb,7.0638e+05_rb,3.5319e+05_rb,5.8851e+01_rb/)
      kbo(:, 4,37,15) = (/ &
     &1.3549e+06_rb,1.0162e+06_rb,6.7744e+05_rb,3.3872e+05_rb,4.8460e+01_rb/)
      kbo(:, 5,37,15) = (/ &
     &1.2980e+06_rb,9.7352e+05_rb,6.4901e+05_rb,3.2451e+05_rb,4.3139e+01_rb/)
      kbo(:, 1,38,15) = (/ &
     &1.7917e+06_rb,1.3437e+06_rb,8.9582e+05_rb,4.4791e+05_rb,6.8827e+01_rb/)
      kbo(:, 2,38,15) = (/ &
     &1.7232e+06_rb,1.2924e+06_rb,8.6157e+05_rb,4.3078e+05_rb,6.5495e+01_rb/)
      kbo(:, 3,38,15) = (/ &
     &1.6545e+06_rb,1.2409e+06_rb,8.2724e+05_rb,4.1362e+05_rb,5.8860e+01_rb/)
      kbo(:, 4,38,15) = (/ &
     &1.5862e+06_rb,1.1896e+06_rb,7.9309e+05_rb,3.9654e+05_rb,4.7251e+01_rb/)
      kbo(:, 5,38,15) = (/ &
     &1.5188e+06_rb,1.1391e+06_rb,7.5940e+05_rb,3.7970e+05_rb,4.3731e+01_rb/)
      kbo(:, 1,39,15) = (/ &
     &2.1445e+06_rb,1.6084e+06_rb,1.0723e+06_rb,5.3613e+05_rb,7.1068e+01_rb/)
      kbo(:, 2,39,15) = (/ &
     &2.0613e+06_rb,1.5460e+06_rb,1.0307e+06_rb,5.1533e+05_rb,6.6419e+01_rb/)
      kbo(:, 3,39,15) = (/ &
     &1.9786e+06_rb,1.4840e+06_rb,9.8932e+05_rb,4.9466e+05_rb,5.6988e+01_rb/)
      kbo(:, 4,39,15) = (/ &
     &1.8964e+06_rb,1.4223e+06_rb,9.4822e+05_rb,4.7411e+05_rb,4.6537e+01_rb/)
      kbo(:, 5,39,15) = (/ &
     &1.8152e+06_rb,1.3614e+06_rb,9.0762e+05_rb,4.5381e+05_rb,4.1463e+01_rb/)
      kbo(:, 1,40,15) = (/ &
     &2.4170e+06_rb,1.8127e+06_rb,1.2085e+06_rb,6.0424e+05_rb,7.3096e+01_rb/)
      kbo(:, 2,40,15) = (/ &
     &2.3226e+06_rb,1.7420e+06_rb,1.1613e+06_rb,5.8063e+05_rb,6.8150e+01_rb/)
      kbo(:, 3,40,15) = (/ &
     &2.2296e+06_rb,1.6722e+06_rb,1.1148e+06_rb,5.5739e+05_rb,5.8979e+01_rb/)
      kbo(:, 4,40,15) = (/ &
     &2.1366e+06_rb,1.6025e+06_rb,1.0683e+06_rb,5.3415e+05_rb,4.7816e+01_rb/)
      kbo(:, 5,40,15) = (/ &
     &2.0448e+06_rb,1.5336e+06_rb,1.0224e+06_rb,5.1120e+05_rb,4.2008e+01_rb/)
      kbo(:, 1,41,15) = (/ &
     &2.7164e+06_rb,2.0373e+06_rb,1.3582e+06_rb,6.7911e+05_rb,7.4782e+01_rb/)
      kbo(:, 2,41,15) = (/ &
     &2.6095e+06_rb,1.9571e+06_rb,1.3048e+06_rb,6.5238e+05_rb,6.9972e+01_rb/)
      kbo(:, 3,41,15) = (/ &
     &2.5049e+06_rb,1.8787e+06_rb,1.2525e+06_rb,6.2623e+05_rb,6.0957e+01_rb/)
      kbo(:, 4,41,15) = (/ &
     &2.4006e+06_rb,1.8005e+06_rb,1.2003e+06_rb,6.0017e+05_rb,4.9097e+01_rb/)
      kbo(:, 5,41,15) = (/ &
     &2.2975e+06_rb,1.7231e+06_rb,1.1487e+06_rb,5.7438e+05_rb,4.2980e+01_rb/)
      kbo(:, 1,42,15) = (/ &
     &3.0827e+06_rb,2.3120e+06_rb,1.5413e+06_rb,7.7067e+05_rb,7.6123e+01_rb/)
      kbo(:, 2,42,15) = (/ &
     &2.9612e+06_rb,2.2209e+06_rb,1.4806e+06_rb,7.4030e+05_rb,7.1662e+01_rb/)
      kbo(:, 3,42,15) = (/ &
     &2.8423e+06_rb,2.1317e+06_rb,1.4211e+06_rb,7.1056e+05_rb,6.2602e+01_rb/)
      kbo(:, 4,42,15) = (/ &
     &2.7244e+06_rb,2.0433e+06_rb,1.3622e+06_rb,6.8108e+05_rb,5.0093e+01_rb/)
      kbo(:, 5,42,15) = (/ &
     &2.6071e+06_rb,1.9553e+06_rb,1.3035e+06_rb,6.5176e+05_rb,4.3774e+01_rb/)
      kbo(:, 1,43,15) = (/ &
     &3.5341e+06_rb,2.6506e+06_rb,1.7671e+06_rb,8.8353e+05_rb,7.8436e+01_rb/)
      kbo(:, 2,43,15) = (/ &
     &3.3959e+06_rb,2.5467e+06_rb,1.6978e+06_rb,8.4891e+05_rb,7.3775e+01_rb/)
      kbo(:, 3,43,15) = (/ &
     &3.2585e+06_rb,2.4439e+06_rb,1.6293e+06_rb,8.1464e+05_rb,6.4695e+01_rb/)
      kbo(:, 4,43,15) = (/ &
     &3.1244e+06_rb,2.3433e+06_rb,1.5622e+06_rb,7.8111e+05_rb,5.1912e+01_rb/)
      kbo(:, 5,43,15) = (/ &
     &2.9906e+06_rb,2.2429e+06_rb,1.4953e+06_rb,7.4765e+05_rb,4.5444e+01_rb/)
      kbo(:, 1,44,15) = (/ &
     &4.1097e+06_rb,3.0823e+06_rb,2.0548e+06_rb,1.0274e+06_rb,8.1119e+01_rb/)
      kbo(:, 2,44,15) = (/ &
     &3.9487e+06_rb,2.9615e+06_rb,1.9743e+06_rb,9.8717e+05_rb,7.5797e+01_rb/)
      kbo(:, 3,44,15) = (/ &
     &3.7894e+06_rb,2.8420e+06_rb,1.8947e+06_rb,9.4733e+05_rb,6.6793e+01_rb/)
      kbo(:, 4,44,15) = (/ &
     &3.6341e+06_rb,2.7256e+06_rb,1.8170e+06_rb,9.0851e+05_rb,5.3433e+01_rb/)
      kbo(:, 5,44,15) = (/ &
     &3.4797e+06_rb,2.6097e+06_rb,1.7398e+06_rb,8.6990e+05_rb,4.7406e+01_rb/)
      kbo(:, 1,45,15) = (/ &
     &4.8722e+06_rb,3.6542e+06_rb,2.4361e+06_rb,1.2181e+06_rb,8.3949e+01_rb/)
      kbo(:, 2,45,15) = (/ &
     &4.6827e+06_rb,3.5120e+06_rb,2.3413e+06_rb,1.1707e+06_rb,7.7974e+01_rb/)
      kbo(:, 3,45,15) = (/ &
     &4.4945e+06_rb,3.3709e+06_rb,2.2473e+06_rb,1.1236e+06_rb,6.9073e+01_rb/)
      kbo(:, 4,45,15) = (/ &
     &4.3103e+06_rb,3.2327e+06_rb,2.1551e+06_rb,1.0776e+06_rb,5.6213e+01_rb/)
      kbo(:, 5,45,15) = (/ &
     &4.1290e+06_rb,3.0968e+06_rb,2.0645e+06_rb,1.0323e+06_rb,4.8762e+01_rb/)
      kbo(:, 1,46,15) = (/ &
     &5.8372e+06_rb,4.3779e+06_rb,2.9186e+06_rb,1.4593e+06_rb,8.7007e+01_rb/)
      kbo(:, 2,46,15) = (/ &
     &5.6126e+06_rb,4.2094e+06_rb,2.8063e+06_rb,1.4031e+06_rb,8.0166e+01_rb/)
      kbo(:, 3,46,15) = (/ &
     &5.3885e+06_rb,4.0414e+06_rb,2.6943e+06_rb,1.3471e+06_rb,7.0991e+01_rb/)
      kbo(:, 4,46,15) = (/ &
     &5.1687e+06_rb,3.8765e+06_rb,2.5843e+06_rb,1.2922e+06_rb,5.9015e+01_rb/)
      kbo(:, 5,46,15) = (/ &
     &4.9529e+06_rb,3.7147e+06_rb,2.4764e+06_rb,1.2382e+06_rb,5.0566e+01_rb/)
      kbo(:, 1,47,15) = (/ &
     &6.9385e+06_rb,5.2039e+06_rb,3.4693e+06_rb,1.7346e+06_rb,9.1194e+01_rb/)
      kbo(:, 2,47,15) = (/ &
     &6.6774e+06_rb,5.0081e+06_rb,3.3387e+06_rb,1.6694e+06_rb,8.2417e+01_rb/)
      kbo(:, 3,47,15) = (/ &
     &6.4127e+06_rb,4.8095e+06_rb,3.2064e+06_rb,1.6032e+06_rb,7.5225e+01_rb/)
      kbo(:, 4,47,15) = (/ &
     &6.1518e+06_rb,4.6139e+06_rb,3.0759e+06_rb,1.5380e+06_rb,6.2982e+01_rb/)
      kbo(:, 5,47,15) = (/ &
     &5.8970e+06_rb,4.4228e+06_rb,2.9485e+06_rb,1.4743e+06_rb,5.3315e+01_rb/)
      kbo(:, 1,48,15) = (/ &
     &8.4803e+06_rb,6.3603e+06_rb,4.2402e+06_rb,2.1201e+06_rb,9.5233e+01_rb/)
      kbo(:, 2,48,15) = (/ &
     &8.1666e+06_rb,6.1249e+06_rb,4.0833e+06_rb,2.0417e+06_rb,8.3938e+01_rb/)
      kbo(:, 3,48,15) = (/ &
     &7.8474e+06_rb,5.8855e+06_rb,3.9237e+06_rb,1.9618e+06_rb,7.8581e+01_rb/)
      kbo(:, 4,48,15) = (/ &
     &7.5308e+06_rb,5.6481e+06_rb,3.7654e+06_rb,1.8827e+06_rb,6.5837e+01_rb/)
      kbo(:, 5,48,15) = (/ &
     &7.2194e+06_rb,5.4145e+06_rb,3.6097e+06_rb,1.8049e+06_rb,5.5833e+01_rb/)
      kbo(:, 1,49,15) = (/ &
     &1.0795e+07_rb,8.0961e+06_rb,5.3974e+06_rb,2.6987e+06_rb,9.8694e+01_rb/)
      kbo(:, 2,49,15) = (/ &
     &1.0405e+07_rb,7.8035e+06_rb,5.2023e+06_rb,2.6011e+06_rb,8.7441e+01_rb/)
      kbo(:, 3,49,15) = (/ &
     &1.0006e+07_rb,7.5045e+06_rb,5.0030e+06_rb,2.5015e+06_rb,8.1233e+01_rb/)
      kbo(:, 4,49,15) = (/ &
     &9.6057e+06_rb,7.2043e+06_rb,4.8028e+06_rb,2.4014e+06_rb,6.9160e+01_rb/)
      kbo(:, 5,49,15) = (/ &
     &9.2104e+06_rb,6.9078e+06_rb,4.6052e+06_rb,2.3026e+06_rb,5.7442e+01_rb/)
      kbo(:, 1,50,15) = (/ &
     &1.3074e+07_rb,9.8054e+06_rb,6.5369e+06_rb,3.2685e+06_rb,1.0218e+02_rb/)
      kbo(:, 2,50,15) = (/ &
     &1.2610e+07_rb,9.4575e+06_rb,6.3050e+06_rb,3.1525e+06_rb,9.1395e+01_rb/)
      kbo(:, 3,50,15) = (/ &
     &1.2137e+07_rb,9.1025e+06_rb,6.0683e+06_rb,3.0342e+06_rb,8.4017e+01_rb/)
      kbo(:, 4,50,15) = (/ &
     &1.1655e+07_rb,8.7414e+06_rb,5.8276e+06_rb,2.9138e+06_rb,7.1620e+01_rb/)
      kbo(:, 5,50,15) = (/ &
     &1.1180e+07_rb,8.3848e+06_rb,5.5899e+06_rb,2.7949e+06_rb,6.0411e+01_rb/)
      kbo(:, 1,51,15) = (/ &
     &1.5480e+07_rb,1.1610e+07_rb,7.7399e+06_rb,3.8700e+06_rb,1.0595e+02_rb/)
      kbo(:, 2,51,15) = (/ &
     &1.4942e+07_rb,1.1207e+07_rb,7.4711e+06_rb,3.7356e+06_rb,9.5091e+01_rb/)
      kbo(:, 3,51,15) = (/ &
     &1.4390e+07_rb,1.0793e+07_rb,7.1952e+06_rb,3.5976e+06_rb,8.6843e+01_rb/)
      kbo(:, 4,51,15) = (/ &
     &1.3829e+07_rb,1.0371e+07_rb,6.9143e+06_rb,3.4572e+06_rb,7.4583e+01_rb/)
      kbo(:, 5,51,15) = (/ &
     &1.3268e+07_rb,9.9514e+06_rb,6.6343e+06_rb,3.3171e+06_rb,6.4529e+01_rb/)
      kbo(:, 1,52,15) = (/ &
     &1.8848e+07_rb,1.4136e+07_rb,9.4240e+06_rb,4.7120e+06_rb,1.0974e+02_rb/)
      kbo(:, 2,52,15) = (/ &
     &1.8206e+07_rb,1.3654e+07_rb,9.1030e+06_rb,4.5515e+06_rb,9.9547e+01_rb/)
      kbo(:, 3,52,15) = (/ &
     &1.7547e+07_rb,1.3161e+07_rb,8.7738e+06_rb,4.3869e+06_rb,8.9552e+01_rb/)
      kbo(:, 4,52,15) = (/ &
     &1.6877e+07_rb,1.2658e+07_rb,8.4384e+06_rb,4.2192e+06_rb,7.7635e+01_rb/)
      kbo(:, 5,52,15) = (/ &
     &1.6199e+07_rb,1.2149e+07_rb,8.0997e+06_rb,4.0499e+06_rb,6.8061e+01_rb/)
      kbo(:, 1,53,15) = (/ &
     &2.3903e+07_rb,1.7928e+07_rb,1.1952e+07_rb,5.9760e+06_rb,1.1360e+02_rb/)
      kbo(:, 2,53,15) = (/ &
     &2.3112e+07_rb,1.7334e+07_rb,1.1556e+07_rb,5.7781e+06_rb,1.0455e+02_rb/)
      kbo(:, 3,53,15) = (/ &
     &2.2293e+07_rb,1.6719e+07_rb,1.1146e+07_rb,5.5732e+06_rb,9.2320e+01_rb/)
      kbo(:, 4,53,15) = (/ &
     &2.1455e+07_rb,1.6091e+07_rb,1.0728e+07_rb,5.3638e+06_rb,8.0467e+01_rb/)
      kbo(:, 5,53,15) = (/ &
     &2.0604e+07_rb,1.5453e+07_rb,1.0302e+07_rb,5.1511e+06_rb,7.1688e+01_rb/)
      kbo(:, 1,54,15) = (/ &
     &2.6168e+07_rb,1.9626e+07_rb,1.3084e+07_rb,6.5420e+06_rb,1.1792e+02_rb/)
      kbo(:, 2,54,15) = (/ &
     &2.5326e+07_rb,1.8995e+07_rb,1.2663e+07_rb,6.3316e+06_rb,1.0689e+02_rb/)
      kbo(:, 3,54,15) = (/ &
     &2.4445e+07_rb,1.8334e+07_rb,1.2223e+07_rb,6.1114e+06_rb,9.4623e+01_rb/)
      kbo(:, 4,54,15) = (/ &
     &2.3543e+07_rb,1.7657e+07_rb,1.1772e+07_rb,5.8858e+06_rb,8.6034e+01_rb/)
      kbo(:, 5,54,15) = (/ &
     &2.2624e+07_rb,1.6968e+07_rb,1.1312e+07_rb,5.6560e+06_rb,7.5610e+01_rb/)
      kbo(:, 1,55,15) = (/ &
     &2.6110e+07_rb,1.9583e+07_rb,1.3055e+07_rb,6.5277e+06_rb,1.2293e+02_rb/)
      kbo(:, 2,55,15) = (/ &
     &2.5298e+07_rb,1.8974e+07_rb,1.2649e+07_rb,6.3247e+06_rb,1.0996e+02_rb/)
      kbo(:, 3,55,15) = (/ &
     &2.4435e+07_rb,1.8326e+07_rb,1.2218e+07_rb,6.1089e+06_rb,1.0025e+02_rb/)
      kbo(:, 4,55,15) = (/ &
     &2.3548e+07_rb,1.7661e+07_rb,1.1774e+07_rb,5.8870e+06_rb,8.9943e+01_rb/)
      kbo(:, 5,55,15) = (/ &
     &2.2646e+07_rb,1.6984e+07_rb,1.1323e+07_rb,5.6615e+06_rb,7.8052e+01_rb/)
      kbo(:, 1,56,15) = (/ &
     &2.6049e+07_rb,1.9537e+07_rb,1.3025e+07_rb,6.5123e+06_rb,1.2801e+02_rb/)
      kbo(:, 2,56,15) = (/ &
     &2.5263e+07_rb,1.8947e+07_rb,1.2631e+07_rb,6.3155e+06_rb,1.1426e+02_rb/)
      kbo(:, 3,56,15) = (/ &
     &2.4422e+07_rb,1.8316e+07_rb,1.2211e+07_rb,6.1054e+06_rb,1.0290e+02_rb/)
      kbo(:, 4,56,15) = (/ &
     &2.3554e+07_rb,1.7665e+07_rb,1.1777e+07_rb,5.8883e+06_rb,9.1587e+01_rb/)
      kbo(:, 5,56,15) = (/ &
     &2.2663e+07_rb,1.6997e+07_rb,1.1331e+07_rb,5.6657e+06_rb,8.3726e+01_rb/)
      kbo(:, 1,57,15) = (/ &
     &2.5975e+07_rb,1.9481e+07_rb,1.2987e+07_rb,6.4936e+06_rb,1.3227e+02_rb/)
      kbo(:, 2,57,15) = (/ &
     &2.5221e+07_rb,1.8915e+07_rb,1.2610e+07_rb,6.3050e+06_rb,1.1939e+02_rb/)
      kbo(:, 3,57,15) = (/ &
     &2.4401e+07_rb,1.8301e+07_rb,1.2201e+07_rb,6.1003e+06_rb,1.0672e+02_rb/)
      kbo(:, 4,57,15) = (/ &
     &2.3548e+07_rb,1.7661e+07_rb,1.1774e+07_rb,5.8869e+06_rb,9.6378e+01_rb/)
      kbo(:, 5,57,15) = (/ &
     &2.2676e+07_rb,1.7007e+07_rb,1.1338e+07_rb,5.6688e+06_rb,8.7523e+01_rb/)
      kbo(:, 1,58,15) = (/ &
     &6.8249e+06_rb,6.2574e+06_rb,5.3652e+06_rb,3.7577e+06_rb,1.3651e+02_rb/)
      kbo(:, 2,58,15) = (/ &
     &6.6343e+06_rb,6.0826e+06_rb,5.2153e+06_rb,3.6528e+06_rb,1.2352e+02_rb/)
      kbo(:, 3,58,15) = (/ &
     &6.4258e+06_rb,5.8914e+06_rb,5.0514e+06_rb,3.5379e+06_rb,1.1077e+02_rb/)
      kbo(:, 4,58,15) = (/ &
     &6.2048e+06_rb,5.6888e+06_rb,4.8776e+06_rb,3.4163e+06_rb,9.9878e+01_rb/)
      kbo(:, 5,58,15) = (/ &
     &5.9792e+06_rb,5.4820e+06_rb,4.7004e+06_rb,3.2921e+06_rb,8.9433e+01_rb/)
      kbo(:, 1,59,15) = (/ &
     &7.9949e+06_rb,6.9809e+06_rb,5.5683e+06_rb,3.4650e+06_rb,1.3832e+02_rb/)
      kbo(:, 2,59,15) = (/ &
     &7.7778e+06_rb,6.7913e+06_rb,5.4171e+06_rb,3.3709e+06_rb,1.2505e+02_rb/)
      kbo(:, 3,59,15) = (/ &
     &7.5359e+06_rb,6.5801e+06_rb,5.2486e+06_rb,3.2660e+06_rb,1.1363e+02_rb/)
      kbo(:, 4,59,15) = (/ &
     &7.2794e+06_rb,6.3561e+06_rb,5.0700e+06_rb,3.1549e+06_rb,1.0324e+02_rb/)
      kbo(:, 5,59,15) = (/ &
     &7.0157e+06_rb,6.1258e+06_rb,4.8863e+06_rb,3.0406e+06_rb,9.1722e+01_rb/)
      kbo(:, 1,13,16) = (/ &
     &7.4591e+05_rb,5.5944e+05_rb,3.7296e+05_rb,1.8648e+05_rb,1.0589e+01_rb/)
      kbo(:, 2,13,16) = (/ &
     &7.4121e+05_rb,5.5591e+05_rb,3.7061e+05_rb,1.8530e+05_rb,1.0476e+01_rb/)
      kbo(:, 3,13,16) = (/ &
     &7.3660e+05_rb,5.5245e+05_rb,3.6830e+05_rb,1.8415e+05_rb,6.4518e+00_rb/)
      kbo(:, 4,13,16) = (/ &
     &7.3017e+05_rb,5.4763e+05_rb,3.6509e+05_rb,1.8254e+05_rb,7.2400e-02_rb/)
      kbo(:, 5,13,16) = (/ &
     &7.2523e+05_rb,5.4392e+05_rb,3.6262e+05_rb,1.8131e+05_rb,8.3840e-02_rb/)
      kbo(:, 1,14,16) = (/ &
     &5.4546e+05_rb,4.0910e+05_rb,2.7273e+05_rb,1.3636e+05_rb,1.2083e+01_rb/)
      kbo(:, 2,14,16) = (/ &
     &5.4530e+05_rb,4.0897e+05_rb,2.7264e+05_rb,1.3632e+05_rb,1.1940e+01_rb/)
      kbo(:, 3,14,16) = (/ &
     &5.4326e+05_rb,4.0739e+05_rb,2.7163e+05_rb,1.3581e+05_rb,1.1721e+01_rb/)
      kbo(:, 4,14,16) = (/ &
     &5.4132e+05_rb,4.0599e+05_rb,2.7066e+05_rb,1.3533e+05_rb,4.5827e+00_rb/)
      kbo(:, 5,14,16) = (/ &
     &5.3886e+05_rb,4.0414e+05_rb,2.6943e+05_rb,1.3471e+05_rb,7.8150e-02_rb/)
      kbo(:, 1,15,16) = (/ &
     &4.1181e+05_rb,3.0886e+05_rb,2.0591e+05_rb,1.0295e+05_rb,1.3732e+01_rb/)
      kbo(:, 2,15,16) = (/ &
     &4.1235e+05_rb,3.0926e+05_rb,2.0618e+05_rb,1.0309e+05_rb,1.3536e+01_rb/)
      kbo(:, 3,15,16) = (/ &
     &4.1248e+05_rb,3.0936e+05_rb,2.0624e+05_rb,1.0312e+05_rb,1.3259e+01_rb/)
      kbo(:, 4,15,16) = (/ &
     &4.1181e+05_rb,3.0886e+05_rb,2.0591e+05_rb,1.0296e+05_rb,1.2919e+01_rb/)
      kbo(:, 5,15,16) = (/ &
     &4.1010e+05_rb,3.0757e+05_rb,2.0505e+05_rb,1.0253e+05_rb,4.0719e+00_rb/)
      kbo(:, 1,16,16) = (/ &
     &3.7257e+05_rb,2.7672e+05_rb,1.8628e+05_rb,9.3143e+04_rb,1.5485e+01_rb/)
      kbo(:, 2,16,16) = (/ &
     &3.7431e+05_rb,2.8074e+05_rb,1.8716e+05_rb,9.3579e+04_rb,1.5222e+01_rb/)
      kbo(:, 3,16,16) = (/ &
     &3.7500e+05_rb,2.8129e+05_rb,1.8753e+05_rb,9.3763e+04_rb,1.4862e+01_rb/)
      kbo(:, 4,16,16) = (/ &
     &3.7444e+05_rb,2.8083e+05_rb,1.8722e+05_rb,9.3612e+04_rb,1.4438e+01_rb/)
      kbo(:, 5,16,16) = (/ &
     &3.7332e+05_rb,2.7999e+05_rb,1.8666e+05_rb,9.3332e+04_rb,7.6230e+00_rb/)
      kbo(:, 1,17,16) = (/ &
     &3.5182e+05_rb,2.6386e+05_rb,1.7591e+05_rb,8.7955e+04_rb,1.7290e+01_rb/)
      kbo(:, 2,17,16) = (/ &
     &3.5397e+05_rb,2.6547e+05_rb,1.7698e+05_rb,8.8492e+04_rb,1.6932e+01_rb/)
      kbo(:, 3,17,16) = (/ &
     &3.5462e+05_rb,2.6597e+05_rb,1.7731e+05_rb,8.8656e+04_rb,1.6480e+01_rb/)
      kbo(:, 4,17,16) = (/ &
     &3.5402e+05_rb,2.6552e+05_rb,1.7701e+05_rb,8.8507e+04_rb,1.5962e+01_rb/)
      kbo(:, 5,17,16) = (/ &
     &3.5314e+05_rb,2.6486e+05_rb,1.7657e+05_rb,8.8287e+04_rb,1.0353e+01_rb/)
      kbo(:, 1,18,16) = (/ &
     &3.4176e+05_rb,2.5632e+05_rb,1.7088e+05_rb,8.5439e+04_rb,1.9139e+01_rb/)
      kbo(:, 2,18,16) = (/ &
     &3.4367e+05_rb,2.5775e+05_rb,1.7183e+05_rb,8.5917e+04_rb,1.8641e+01_rb/)
      kbo(:, 3,18,16) = (/ &
     &3.4399e+05_rb,2.5800e+05_rb,1.7200e+05_rb,8.5999e+04_rb,1.8079e+01_rb/)
      kbo(:, 4,18,16) = (/ &
     &3.4376e+05_rb,2.5782e+05_rb,1.7188e+05_rb,8.5941e+04_rb,1.7627e+01_rb/)
      kbo(:, 5,18,16) = (/ &
     &3.4258e+05_rb,2.5694e+05_rb,1.7129e+05_rb,8.5646e+04_rb,1.3174e+01_rb/)
      kbo(:, 1,19,16) = (/ &
     &3.2677e+05_rb,2.4508e+05_rb,1.6338e+05_rb,8.1692e+04_rb,2.1598e+01_rb/)
      kbo(:, 2,19,16) = (/ &
     &3.2775e+05_rb,2.4581e+05_rb,1.6387e+05_rb,8.1937e+04_rb,2.1013e+01_rb/)
      kbo(:, 3,19,16) = (/ &
     &3.2835e+05_rb,2.4626e+05_rb,1.6417e+05_rb,8.2068e+04_rb,2.0530e+01_rb/)
      kbo(:, 4,19,16) = (/ &
     &3.2780e+05_rb,2.4585e+05_rb,1.6390e+05_rb,8.1950e+04_rb,2.0057e+01_rb/)
      kbo(:, 5,19,16) = (/ &
     &3.2322e+05_rb,2.4484e+05_rb,1.6323e+05_rb,8.1613e+04_rb,1.8218e+01_rb/)
      kbo(:, 1,20,16) = (/ &
     &3.3791e+05_rb,2.5343e+05_rb,1.6896e+05_rb,8.4478e+04_rb,2.4811e+01_rb/)
      kbo(:, 2,20,16) = (/ &
     &3.3834e+05_rb,2.5376e+05_rb,1.6917e+05_rb,8.4586e+04_rb,2.4163e+01_rb/)
      kbo(:, 3,20,16) = (/ &
     &3.3842e+05_rb,2.5381e+05_rb,1.6921e+05_rb,8.4605e+04_rb,2.3647e+01_rb/)
      kbo(:, 4,20,16) = (/ &
     &3.3728e+05_rb,2.5296e+05_rb,1.6864e+05_rb,8.4321e+04_rb,2.3144e+01_rb/)
      kbo(:, 5,20,16) = (/ &
     &3.3569e+05_rb,2.5177e+05_rb,1.6785e+05_rb,8.3923e+04_rb,2.2618e+01_rb/)
      kbo(:, 1,21,16) = (/ &
     &3.5913e+05_rb,2.6935e+05_rb,1.7956e+05_rb,8.9782e+04_rb,2.8889e+01_rb/)
      kbo(:, 2,21,16) = (/ &
     &3.5914e+05_rb,2.6936e+05_rb,1.7957e+05_rb,8.9786e+04_rb,2.8114e+01_rb/)
      kbo(:, 3,21,16) = (/ &
     &3.5794e+05_rb,2.6846e+05_rb,1.7897e+05_rb,8.9486e+04_rb,2.7459e+01_rb/)
      kbo(:, 4,21,16) = (/ &
     &3.5626e+05_rb,2.6720e+05_rb,1.7813e+05_rb,8.9066e+04_rb,2.6836e+01_rb/)
      kbo(:, 5,21,16) = (/ &
     &3.5369e+05_rb,2.6527e+05_rb,1.7684e+05_rb,8.8423e+04_rb,2.6161e+01_rb/)
      kbo(:, 1,22,16) = (/ &
     &3.8948e+05_rb,2.9211e+05_rb,1.9474e+05_rb,9.7369e+04_rb,3.3724e+01_rb/)
      kbo(:, 2,22,16) = (/ &
     &3.8828e+05_rb,2.9121e+05_rb,1.9414e+05_rb,9.7069e+04_rb,3.2720e+01_rb/)
      kbo(:, 3,22,16) = (/ &
     &3.8560e+05_rb,2.8920e+05_rb,1.9280e+05_rb,9.6401e+04_rb,3.1834e+01_rb/)
      kbo(:, 4,22,16) = (/ &
     &3.8253e+05_rb,2.8690e+05_rb,1.9127e+05_rb,9.5633e+04_rb,3.0971e+01_rb/)
      kbo(:, 5,22,16) = (/ &
     &3.7845e+05_rb,2.8383e+05_rb,1.8922e+05_rb,9.4612e+04_rb,3.0080e+01_rb/)
      kbo(:, 1,23,16) = (/ &
     &4.2147e+05_rb,3.1611e+05_rb,2.1074e+05_rb,1.0537e+05_rb,3.9276e+01_rb/)
      kbo(:, 2,23,16) = (/ &
     &4.1843e+05_rb,3.1382e+05_rb,2.0921e+05_rb,1.0461e+05_rb,3.7947e+01_rb/)
      kbo(:, 3,23,16) = (/ &
     &4.1438e+05_rb,3.1079e+05_rb,2.0719e+05_rb,1.0360e+05_rb,3.6734e+01_rb/)
      kbo(:, 4,23,16) = (/ &
     &4.0918e+05_rb,3.0688e+05_rb,2.0459e+05_rb,1.0230e+05_rb,3.5522e+01_rb/)
      kbo(:, 5,23,16) = (/ &
     &4.0269e+05_rb,3.0202e+05_rb,2.0135e+05_rb,1.0067e+05_rb,3.4322e+01_rb/)
      kbo(:, 1,24,16) = (/ &
     &4.4910e+05_rb,3.3682e+05_rb,2.2455e+05_rb,1.1228e+05_rb,4.5446e+01_rb/)
      kbo(:, 2,24,16) = (/ &
     &4.4436e+05_rb,3.3327e+05_rb,2.2218e+05_rb,1.1109e+05_rb,4.3684e+01_rb/)
      kbo(:, 3,24,16) = (/ &
     &4.3822e+05_rb,3.2867e+05_rb,2.1911e+05_rb,1.0956e+05_rb,4.2010e+01_rb/)
      kbo(:, 4,24,16) = (/ &
     &4.3055e+05_rb,3.2291e+05_rb,2.1528e+05_rb,1.0764e+05_rb,4.0380e+01_rb/)
      kbo(:, 5,24,16) = (/ &
     &4.2183e+05_rb,3.1637e+05_rb,2.1092e+05_rb,1.0546e+05_rb,3.8819e+01_rb/)
      kbo(:, 1,25,16) = (/ &
     &4.7951e+05_rb,3.5963e+05_rb,2.3975e+05_rb,1.1988e+05_rb,5.2030e+01_rb/)
      kbo(:, 2,25,16) = (/ &
     &4.7236e+05_rb,3.5427e+05_rb,2.3618e+05_rb,1.1809e+05_rb,4.9759e+01_rb/)
      kbo(:, 3,25,16) = (/ &
     &4.6345e+05_rb,3.4759e+05_rb,2.3173e+05_rb,1.1586e+05_rb,4.7513e+01_rb/)
      kbo(:, 4,25,16) = (/ &
     &4.5307e+05_rb,3.3980e+05_rb,2.2654e+05_rb,1.1327e+05_rb,4.5411e+01_rb/)
      kbo(:, 5,25,16) = (/ &
     &4.4201e+05_rb,3.3151e+05_rb,2.2101e+05_rb,1.1050e+05_rb,4.3416e+01_rb/)
      kbo(:, 1,26,16) = (/ &
     &5.1412e+05_rb,3.8559e+05_rb,2.5706e+05_rb,1.2853e+05_rb,5.8735e+01_rb/)
      kbo(:, 2,26,16) = (/ &
     &5.0368e+05_rb,3.7776e+05_rb,2.5184e+05_rb,1.2592e+05_rb,5.5868e+01_rb/)
      kbo(:, 3,26,16) = (/ &
     &4.9157e+05_rb,3.6868e+05_rb,2.4579e+05_rb,1.2289e+05_rb,5.3078e+01_rb/)
      kbo(:, 4,26,16) = (/ &
     &4.7856e+05_rb,3.5892e+05_rb,2.3928e+05_rb,1.1964e+05_rb,5.0425e+01_rb/)
      kbo(:, 5,26,16) = (/ &
     &4.6464e+05_rb,3.4848e+05_rb,2.3232e+05_rb,1.1616e+05_rb,4.7921e+01_rb/)
      kbo(:, 1,27,16) = (/ &
     &5.6504e+05_rb,4.2378e+05_rb,2.8252e+05_rb,1.4126e+05_rb,6.5394e+01_rb/)
      kbo(:, 2,27,16) = (/ &
     &5.5002e+05_rb,4.1252e+05_rb,2.7501e+05_rb,1.3751e+05_rb,6.1825e+01_rb/)
      kbo(:, 3,27,16) = (/ &
     &5.3436e+05_rb,4.0077e+05_rb,2.6718e+05_rb,1.3359e+05_rb,5.8497e+01_rb/)
      kbo(:, 4,27,16) = (/ &
     &5.1782e+05_rb,3.8837e+05_rb,2.5891e+05_rb,1.2946e+05_rb,5.5282e+01_rb/)
      kbo(:, 5,27,16) = (/ &
     &5.0051e+05_rb,3.7538e+05_rb,2.5025e+05_rb,1.2513e+05_rb,5.2211e+01_rb/)
      kbo(:, 1,28,16) = (/ &
     &6.2869e+05_rb,4.7152e+05_rb,3.1435e+05_rb,1.5717e+05_rb,7.1700e+01_rb/)
      kbo(:, 2,28,16) = (/ &
     &6.0862e+05_rb,4.5646e+05_rb,3.0431e+05_rb,1.5215e+05_rb,6.7460e+01_rb/)
      kbo(:, 3,28,16) = (/ &
     &5.8860e+05_rb,4.4145e+05_rb,2.9430e+05_rb,1.4715e+05_rb,6.3511e+01_rb/)
      kbo(:, 4,28,16) = (/ &
     &5.6780e+05_rb,4.2585e+05_rb,2.8390e+05_rb,1.4195e+05_rb,5.9746e+01_rb/)
      kbo(:, 5,28,16) = (/ &
     &5.4624e+05_rb,4.0968e+05_rb,2.7312e+05_rb,1.3656e+05_rb,5.6156e+01_rb/)
      kbo(:, 1,29,16) = (/ &
     &7.3240e+05_rb,5.4930e+05_rb,3.6620e+05_rb,1.8310e+05_rb,7.7436e+01_rb/)
      kbo(:, 2,29,16) = (/ &
     &7.0567e+05_rb,5.2925e+05_rb,3.5284e+05_rb,1.7642e+05_rb,7.2532e+01_rb/)
      kbo(:, 3,29,16) = (/ &
     &6.7920e+05_rb,5.0940e+05_rb,3.3960e+05_rb,1.6980e+05_rb,6.7962e+01_rb/)
      kbo(:, 4,29,16) = (/ &
     &6.5231e+05_rb,4.8923e+05_rb,3.2615e+05_rb,1.6308e+05_rb,6.3669e+01_rb/)
      kbo(:, 5,29,16) = (/ &
     &6.2510e+05_rb,4.6882e+05_rb,3.1255e+05_rb,1.5627e+05_rb,5.9600e+01_rb/)
      kbo(:, 1,30,16) = (/ &
     &8.6319e+05_rb,6.4740e+05_rb,4.3160e+05_rb,2.1580e+05_rb,8.2489e+01_rb/)
      kbo(:, 2,30,16) = (/ &
     &8.2811e+05_rb,6.2108e+05_rb,4.1405e+05_rb,2.0703e+05_rb,7.6918e+01_rb/)
      kbo(:, 3,30,16) = (/ &
     &7.9339e+05_rb,5.9504e+05_rb,3.9669e+05_rb,1.9835e+05_rb,7.1793e+01_rb/)
      kbo(:, 4,30,16) = (/ &
     &7.5902e+05_rb,5.6926e+05_rb,3.7951e+05_rb,1.8975e+05_rb,6.6976e+01_rb/)
      kbo(:, 5,30,16) = (/ &
     &7.2513e+05_rb,5.4385e+05_rb,3.6256e+05_rb,1.8128e+05_rb,5.5921e+01_rb/)
      kbo(:, 1,31,16) = (/ &
     &1.0572e+06_rb,7.9291e+05_rb,5.2861e+05_rb,2.6430e+05_rb,8.6704e+01_rb/)
      kbo(:, 2,31,16) = (/ &
     &1.0099e+06_rb,7.5744e+05_rb,5.0496e+05_rb,2.5248e+05_rb,8.0546e+01_rb/)
      kbo(:, 3,31,16) = (/ &
     &9.6381e+05_rb,7.2285e+05_rb,4.8190e+05_rb,2.4095e+05_rb,7.4889e+01_rb/)
      kbo(:, 4,31,16) = (/ &
     &9.1917e+05_rb,6.8937e+05_rb,4.5958e+05_rb,2.2979e+05_rb,6.9630e+01_rb/)
      kbo(:, 5,31,16) = (/ &
     &8.7558e+05_rb,6.5669e+05_rb,4.3779e+05_rb,2.1889e+05_rb,5.1048e+01_rb/)
      kbo(:, 1,32,16) = (/ &
     &1.2953e+06_rb,9.7146e+05_rb,6.4764e+05_rb,3.2382e+05_rb,9.0053e+01_rb/)
      kbo(:, 2,32,16) = (/ &
     &1.2332e+06_rb,9.2492e+05_rb,6.1662e+05_rb,3.0831e+05_rb,8.3375e+01_rb/)
      kbo(:, 3,32,16) = (/ &
     &1.1732e+06_rb,8.7992e+05_rb,5.8661e+05_rb,2.9331e+05_rb,7.7283e+01_rb/)
      kbo(:, 4,32,16) = (/ &
     &1.1156e+06_rb,8.3672e+05_rb,5.5781e+05_rb,2.7891e+05_rb,5.9068e+01_rb/)
      kbo(:, 5,32,16) = (/ &
     &1.0600e+06_rb,7.9499e+05_rb,5.3000e+05_rb,2.6500e+05_rb,4.7615e+01_rb/)
      kbo(:, 1,33,16) = (/ &
     &1.6005e+06_rb,1.2004e+06_rb,8.0025e+05_rb,4.0013e+05_rb,9.2564e+01_rb/)
      kbo(:, 2,33,16) = (/ &
     &1.5042e+06_rb,1.1396e+06_rb,7.5976e+05_rb,3.7988e+05_rb,8.5451e+01_rb/)
      kbo(:, 3,33,16) = (/ &
     &1.4412e+06_rb,1.0809e+06_rb,7.2060e+05_rb,3.6030e+05_rb,6.7956e+01_rb/)
      kbo(:, 4,33,16) = (/ &
     &1.3676e+06_rb,1.0257e+06_rb,6.8379e+05_rb,3.4189e+05_rb,5.3693e+01_rb/)
      kbo(:, 5,33,16) = (/ &
     &1.2966e+06_rb,9.7245e+05_rb,6.4830e+05_rb,3.2415e+05_rb,4.3539e+01_rb/)
      kbo(:, 1,34,16) = (/ &
     &1.9162e+06_rb,1.4372e+06_rb,9.5812e+05_rb,4.7906e+05_rb,9.4564e+01_rb/)
      kbo(:, 2,34,16) = (/ &
     &1.8147e+06_rb,1.3610e+06_rb,9.0734e+05_rb,4.5367e+05_rb,8.5602e+01_rb/)
      kbo(:, 3,34,16) = (/ &
     &1.7177e+06_rb,1.2883e+06_rb,8.5884e+05_rb,4.2942e+05_rb,6.2695e+01_rb/)
      kbo(:, 4,34,16) = (/ &
     &1.6266e+06_rb,1.2199e+06_rb,8.1329e+05_rb,4.0664e+05_rb,5.0461e+01_rb/)
      kbo(:, 5,34,16) = (/ &
     &1.5395e+06_rb,1.1546e+06_rb,7.6973e+05_rb,3.8486e+05_rb,4.1009e+01_rb/)
      kbo(:, 1,35,16) = (/ &
     &2.2940e+06_rb,1.7205e+06_rb,1.1470e+06_rb,5.7350e+05_rb,9.6683e+01_rb/)
      kbo(:, 2,35,16) = (/ &
     &2.1679e+06_rb,1.6259e+06_rb,1.0839e+06_rb,5.4197e+05_rb,7.5706e+01_rb/)
      kbo(:, 3,35,16) = (/ &
     &2.0487e+06_rb,1.5365e+06_rb,1.0243e+06_rb,5.1216e+05_rb,5.9415e+01_rb/)
      kbo(:, 4,35,16) = (/ &
     &1.9369e+06_rb,1.4527e+06_rb,9.6844e+05_rb,4.8422e+05_rb,4.7837e+01_rb/)
      kbo(:, 5,35,16) = (/ &
     &1.8303e+06_rb,1.3728e+06_rb,9.1517e+05_rb,4.5758e+05_rb,3.3167e+01_rb/)
      kbo(:, 1,36,16) = (/ &
     &2.7323e+06_rb,2.0492e+06_rb,1.3661e+06_rb,6.8307e+05_rb,9.9200e+01_rb/)
      kbo(:, 2,36,16) = (/ &
     &2.5780e+06_rb,1.9335e+06_rb,1.2890e+06_rb,6.4450e+05_rb,7.2688e+01_rb/)
      kbo(:, 3,36,16) = (/ &
     &2.4326e+06_rb,1.8244e+06_rb,1.2163e+06_rb,6.0814e+05_rb,5.7708e+01_rb/)
      kbo(:, 4,36,16) = (/ &
     &2.2967e+06_rb,1.7225e+06_rb,1.1483e+06_rb,5.7417e+05_rb,4.6712e+01_rb/)
      kbo(:, 5,36,16) = (/ &
     &2.1679e+06_rb,1.6259e+06_rb,1.0839e+06_rb,5.4197e+05_rb,1.2791e+01_rb/)
      kbo(:, 1,37,16) = (/ &
     &3.1885e+06_rb,2.3914e+06_rb,1.5942e+06_rb,7.9711e+05_rb,1.0268e+02_rb/)
      kbo(:, 2,37,16) = (/ &
     &3.0040e+06_rb,2.2530e+06_rb,1.5020e+06_rb,7.5099e+05_rb,7.3443e+01_rb/)
      kbo(:, 3,37,16) = (/ &
     &2.8309e+06_rb,2.1232e+06_rb,1.4155e+06_rb,7.0773e+05_rb,5.8756e+01_rb/)
      kbo(:, 4,37,16) = (/ &
     &2.6696e+06_rb,2.0022e+06_rb,1.3348e+06_rb,6.6739e+05_rb,4.7388e+01_rb/)
      kbo(:, 5,37,16) = (/ &
     &2.5177e+06_rb,1.8883e+06_rb,1.2588e+06_rb,6.2942e+05_rb,7.8483e+00_rb/)
      kbo(:, 1,38,16) = (/ &
     &3.7660e+06_rb,2.8245e+06_rb,1.8830e+06_rb,9.4149e+05_rb,1.0053e+02_rb/)
      kbo(:, 2,38,16) = (/ &
     &3.5441e+06_rb,2.6581e+06_rb,1.7720e+06_rb,8.8602e+05_rb,7.3960e+01_rb/)
      kbo(:, 3,38,16) = (/ &
     &3.3357e+06_rb,2.5018e+06_rb,1.6678e+06_rb,8.3391e+05_rb,5.9673e+01_rb/)
      kbo(:, 4,38,16) = (/ &
     &3.1421e+06_rb,2.3566e+06_rb,1.5711e+06_rb,7.8553e+05_rb,4.7953e+01_rb/)
      kbo(:, 5,38,16) = (/ &
     &2.9610e+06_rb,2.2207e+06_rb,1.4805e+06_rb,7.4024e+05_rb,7.1016e-04_rb/)
      kbo(:, 1,39,16) = (/ &
     &4.5378e+06_rb,3.4034e+06_rb,2.2689e+06_rb,1.1345e+06_rb,9.8727e+01_rb/)
      kbo(:, 2,39,16) = (/ &
     &4.2667e+06_rb,3.2000e+06_rb,2.1334e+06_rb,1.0667e+06_rb,7.4242e+01_rb/)
      kbo(:, 3,39,16) = (/ &
     &4.0124e+06_rb,3.0093e+06_rb,2.0062e+06_rb,1.0031e+06_rb,5.9575e+01_rb/)
      kbo(:, 4,39,16) = (/ &
     &3.7760e+06_rb,2.8320e+06_rb,1.8880e+06_rb,9.4400e+05_rb,3.9674e+01_rb/)
      kbo(:, 5,39,16) = (/ &
     &3.5558e+06_rb,2.6669e+06_rb,1.7591e+06_rb,8.8898e+05_rb,5.7405e-04_rb/)
      kbo(:, 1,40,16) = (/ &
     &5.1586e+06_rb,3.8690e+06_rb,2.5793e+06_rb,1.2896e+06_rb,1.0433e+02_rb/)
      kbo(:, 2,40,16) = (/ &
     &4.8469e+06_rb,3.6347e+06_rb,2.4234e+06_rb,1.2117e+06_rb,7.7760e+01_rb/)
      kbo(:, 3,40,16) = (/ &
     &4.5541e+06_rb,3.4156e+06_rb,2.2770e+06_rb,1.1385e+06_rb,6.1585e+01_rb/)
      kbo(:, 4,40,16) = (/ &
     &4.2818e+06_rb,3.2114e+06_rb,2.1409e+06_rb,1.0704e+06_rb,4.0940e+01_rb/)
      kbo(:, 5,40,16) = (/ &
     &4.0296e+06_rb,3.0222e+06_rb,2.0148e+06_rb,1.0074e+06_rb,4.6888e-04_rb/)
      kbo(:, 1,41,16) = (/ &
     &5.8465e+06_rb,4.3849e+06_rb,2.9233e+06_rb,1.4616e+06_rb,1.1317e+02_rb/)
      kbo(:, 2,41,16) = (/ &
     &5.4907e+06_rb,4.1180e+06_rb,2.7454e+06_rb,1.3727e+06_rb,8.1398e+01_rb/)
      kbo(:, 3,41,16) = (/ &
     &5.1550e+06_rb,3.8663e+06_rb,2.5775e+06_rb,1.2888e+06_rb,6.4536e+01_rb/)
      kbo(:, 4,41,16) = (/ &
     &4.8434e+06_rb,3.6326e+06_rb,2.4217e+06_rb,1.2109e+06_rb,4.3960e+01_rb/)
      kbo(:, 5,41,16) = (/ &
     &4.5548e+06_rb,3.4161e+06_rb,2.2774e+06_rb,1.1387e+06_rb,3.7799e-04_rb/)
      kbo(:, 1,42,16) = (/ &
     &6.6900e+06_rb,5.0175e+06_rb,3.3450e+06_rb,1.6725e+06_rb,1.2124e+02_rb/)
      kbo(:, 2,42,16) = (/ &
     &6.2794e+06_rb,4.7096e+06_rb,3.1397e+06_rb,1.5698e+06_rb,8.5097e+01_rb/)
      kbo(:, 3,42,16) = (/ &
     &5.8926e+06_rb,4.4195e+06_rb,2.9463e+06_rb,1.4731e+06_rb,6.7520e+01_rb/)
      kbo(:, 4,42,16) = (/ &
     &5.5323e+06_rb,4.1492e+06_rb,2.7661e+06_rb,1.3831e+06_rb,4.6182e+01_rb/)
      kbo(:, 5,42,16) = (/ &
     &5.1983e+06_rb,3.8987e+06_rb,2.5991e+06_rb,1.2996e+06_rb,3.0509e-04_rb/)
      kbo(:, 1,43,16) = (/ &
     &7.7419e+06_rb,5.8065e+06_rb,3.8710e+06_rb,1.9355e+06_rb,1.2600e+02_rb/)
      kbo(:, 2,43,16) = (/ &
     &7.2619e+06_rb,5.4481e+06_rb,3.6321e+06_rb,1.8161e+06_rb,9.0385e+01_rb/)
      kbo(:, 3,43,16) = (/ &
     &6.8139e+06_rb,5.1104e+06_rb,3.4070e+06_rb,1.7035e+06_rb,7.0944e+01_rb/)
      kbo(:, 4,43,16) = (/ &
     &6.3928e+06_rb,4.7946e+06_rb,3.1964e+06_rb,1.5982e+06_rb,5.0507e+01_rb/)
      kbo(:, 5,43,16) = (/ &
     &6.0029e+06_rb,4.5022e+06_rb,3.0015e+06_rb,1.5007e+06_rb,2.4615e-04_rb/)
      kbo(:, 1,44,16) = (/ &
     &9.0918e+06_rb,6.8188e+06_rb,4.5458e+06_rb,2.2729e+06_rb,1.3122e+02_rb/)
      kbo(:, 2,44,16) = (/ &
     &8.5266e+06_rb,6.3949e+06_rb,4.2633e+06_rb,2.1316e+06_rb,9.7168e+01_rb/)
      kbo(:, 3,44,16) = (/ &
     &7.9959e+06_rb,5.9969e+06_rb,3.9979e+06_rb,1.9989e+06_rb,7.5640e+01_rb/)
      kbo(:, 4,44,16) = (/ &
     &7.4967e+06_rb,5.6225e+06_rb,3.7483e+06_rb,1.8741e+06_rb,5.7032e+01_rb/)
      kbo(:, 5,44,16) = (/ &
     &7.0357e+06_rb,5.2767e+06_rb,3.5178e+06_rb,1.7589e+06_rb,1.1125e-02_rb/)
      kbo(:, 1,45,16) = (/ &
     &1.0890e+07_rb,8.1678e+06_rb,5.4452e+06_rb,2.7226e+06_rb,1.3511e+02_rb/)
      kbo(:, 2,45,16) = (/ &
     &1.0206e+07_rb,7.6543e+06_rb,5.1028e+06_rb,2.5514e+06_rb,1.0320e+02_rb/)
      kbo(:, 3,45,16) = (/ &
     &9.5676e+06_rb,7.1757e+06_rb,4.7838e+06_rb,2.3919e+06_rb,7.9521e+01_rb/)
      kbo(:, 4,45,16) = (/ &
     &8.9682e+06_rb,6.7261e+06_rb,4.4841e+06_rb,2.2420e+06_rb,5.3314e+01_rb/)
      kbo(:, 5,45,16) = (/ &
     &8.4107e+06_rb,6.3080e+06_rb,4.2053e+06_rb,2.1027e+06_rb,2.2542e-04_rb/)
      kbo(:, 1,46,16) = (/ &
     &1.3200e+07_rb,9.8998e+06_rb,6.5999e+06_rb,3.2999e+06_rb,1.4245e+02_rb/)
      kbo(:, 2,46,16) = (/ &
     &1.2354e+07_rb,9.2658e+06_rb,6.1772e+06_rb,3.0886e+06_rb,1.1090e+02_rb/)
      kbo(:, 3,46,16) = (/ &
     &1.1578e+07_rb,8.6832e+06_rb,5.7888e+06_rb,2.8944e+06_rb,8.3717e+01_rb/)
      kbo(:, 4,46,16) = (/ &
     &1.0851e+07_rb,8.1384e+06_rb,5.4256e+06_rb,2.7128e+06_rb,5.1335e+01_rb/)
      kbo(:, 5,46,16) = (/ &
     &1.0172e+07_rb,7.6289e+06_rb,5.0860e+06_rb,2.5430e+06_rb,1.9729e-04_rb/)
      kbo(:, 1,47,16) = (/ &
     &1.5899e+07_rb,1.1924e+07_rb,7.9494e+06_rb,3.9747e+06_rb,1.4926e+02_rb/)
      kbo(:, 2,47,16) = (/ &
     &1.4859e+07_rb,1.1144e+07_rb,7.4297e+06_rb,3.7148e+06_rb,1.2298e+02_rb/)
      kbo(:, 3,47,16) = (/ &
     &1.3924e+07_rb,1.0443e+07_rb,6.9621e+06_rb,3.4810e+06_rb,8.5230e+01_rb/)
      kbo(:, 4,47,16) = (/ &
     &1.3046e+07_rb,9.7847e+06_rb,6.5231e+06_rb,3.2616e+06_rb,5.3561e+01_rb/)
      kbo(:, 5,47,16) = (/ &
     &1.2222e+07_rb,9.1665e+06_rb,6.1110e+06_rb,3.0555e+06_rb,1.6606e-04_rb/)
      kbo(:, 1,48,16) = (/ &
     &1.9701e+07_rb,1.4775e+07_rb,9.8503e+06_rb,4.9251e+06_rb,1.5645e+02_rb/)
      kbo(:, 2,48,16) = (/ &
     &1.8394e+07_rb,1.3796e+07_rb,9.1972e+06_rb,4.5986e+06_rb,1.4059e+02_rb/)
      kbo(:, 3,48,16) = (/ &
     &1.7217e+07_rb,1.2913e+07_rb,8.6087e+06_rb,4.3044e+06_rb,8.5580e+01_rb/)
      kbo(:, 4,48,16) = (/ &
     &1.6130e+07_rb,1.2097e+07_rb,8.0649e+06_rb,4.0324e+06_rb,5.5884e+01_rb/)
      kbo(:, 5,48,16) = (/ &
     &1.5108e+07_rb,1.1331e+07_rb,7.5542e+06_rb,3.7771e+06_rb,1.4110e-04_rb/)
      kbo(:, 1,49,16) = (/ &
     &2.5446e+07_rb,1.9085e+07_rb,1.2723e+07_rb,6.3615e+06_rb,1.6402e+02_rb/)
      kbo(:, 2,49,16) = (/ &
     &2.3735e+07_rb,1.7801e+07_rb,1.1868e+07_rb,5.9337e+06_rb,1.4463e+02_rb/)
      kbo(:, 3,49,16) = (/ &
     &2.2185e+07_rb,1.6639e+07_rb,1.1093e+07_rb,5.5463e+06_rb,8.5806e+01_rb/)
      kbo(:, 4,49,16) = (/ &
     &2.0781e+07_rb,1.5586e+07_rb,1.0390e+07_rb,5.1952e+06_rb,5.2515e+01_rb/)
      kbo(:, 5,49,16) = (/ &
     &1.9463e+07_rb,1.4597e+07_rb,9.7313e+06_rb,4.8656e+06_rb,1.1804e-04_rb/)
      kbo(:, 1,50,16) = (/ &
     &3.1256e+07_rb,2.3442e+07_rb,1.5628e+07_rb,7.8139e+06_rb,1.7145e+02_rb/)
      kbo(:, 2,50,16) = (/ &
     &2.9123e+07_rb,2.1842e+07_rb,1.4562e+07_rb,7.2808e+06_rb,1.4715e+02_rb/)
      kbo(:, 3,50,16) = (/ &
     &2.7197e+07_rb,2.0398e+07_rb,1.3599e+07_rb,6.7993e+06_rb,9.2079e+01_rb/)
      kbo(:, 4,50,16) = (/ &
     &2.5458e+07_rb,1.9094e+07_rb,1.2729e+07_rb,6.3645e+06_rb,5.7113e+01_rb/)
      kbo(:, 5,50,16) = (/ &
     &2.3842e+07_rb,1.7882e+07_rb,1.1921e+07_rb,5.9606e+06_rb,9.7879e-05_rb/)
      kbo(:, 1,51,16) = (/ &
     &3.7532e+07_rb,2.8149e+07_rb,1.8766e+07_rb,9.3831e+06_rb,1.7913e+02_rb/)
      kbo(:, 2,51,16) = (/ &
     &3.4951e+07_rb,2.6213e+07_rb,1.7476e+07_rb,8.7379e+06_rb,1.5348e+02_rb/)
      kbo(:, 3,51,16) = (/ &
     &3.2604e+07_rb,2.4453e+07_rb,1.6302e+07_rb,8.1511e+06_rb,9.9906e+01_rb/)
      kbo(:, 4,51,16) = (/ &
     &3.0489e+07_rb,2.2867e+07_rb,1.5245e+07_rb,7.6224e+06_rb,6.1926e+01_rb/)
      kbo(:, 5,51,16) = (/ &
     &2.8553e+07_rb,2.1415e+07_rb,1.4277e+07_rb,7.1383e+06_rb,8.2332e-05_rb/)
      kbo(:, 1,52,16) = (/ &
     &4.6363e+07_rb,3.4773e+07_rb,2.3182e+07_rb,1.1591e+07_rb,1.8724e+02_rb/)
      kbo(:, 2,52,16) = (/ &
     &4.3154e+07_rb,3.2365e+07_rb,2.1577e+07_rb,1.0789e+07_rb,1.5735e+02_rb/)
      kbo(:, 3,52,16) = (/ &
     &4.0219e+07_rb,3.0165e+07_rb,2.0110e+07_rb,1.0055e+07_rb,1.0819e+02_rb/)
      kbo(:, 4,52,16) = (/ &
     &3.7571e+07_rb,2.8178e+07_rb,1.8786e+07_rb,9.3929e+06_rb,6.5746e+01_rb/)
      kbo(:, 5,52,16) = (/ &
     &3.5175e+07_rb,2.6382e+07_rb,1.7588e+07_rb,8.7937e+06_rb,2.5991e+00_rb/)
      kbo(:, 1,53,16) = (/ &
     &5.9694e+07_rb,4.4771e+07_rb,2.9847e+07_rb,1.4924e+07_rb,1.9593e+02_rb/)
      kbo(:, 2,53,16) = (/ &
     &5.5520e+07_rb,4.1640e+07_rb,2.7760e+07_rb,1.3880e+07_rb,1.5986e+02_rb/)
      kbo(:, 3,53,16) = (/ &
     &5.1713e+07_rb,3.8785e+07_rb,2.5857e+07_rb,1.2928e+07_rb,1.1418e+02_rb/)
      kbo(:, 4,53,16) = (/ &
     &4.8267e+07_rb,3.6200e+07_rb,2.4134e+07_rb,1.2067e+07_rb,6.7101e+01_rb/)
      kbo(:, 5,53,16) = (/ &
     &4.5154e+07_rb,3.3866e+07_rb,2.2577e+07_rb,1.1289e+07_rb,5.4836e-05_rb/)
      kbo(:, 1,54,16) = (/ &
     &6.6291e+07_rb,4.9719e+07_rb,3.3146e+07_rb,1.6573e+07_rb,2.0438e+02_rb/)
      kbo(:, 2,54,16) = (/ &
     &6.1631e+07_rb,4.6223e+07_rb,3.0816e+07_rb,1.5408e+07_rb,1.7539e+02_rb/)
      kbo(:, 3,54,16) = (/ &
     &5.7374e+07_rb,4.3031e+07_rb,2.8687e+07_rb,1.4344e+07_rb,1.3582e+02_rb/)
      kbo(:, 4,54,16) = (/ &
     &5.3498e+07_rb,4.0123e+07_rb,2.6749e+07_rb,1.3375e+07_rb,7.9383e+01_rb/)
      kbo(:, 5,54,16) = (/ &
     &5.0001e+07_rb,3.7501e+07_rb,2.5001e+07_rb,1.2501e+07_rb,1.9380e+01_rb/)
      kbo(:, 1,55,16) = (/ &
     &6.7110e+07_rb,5.0333e+07_rb,3.3555e+07_rb,1.6778e+07_rb,2.1302e+02_rb/)
      kbo(:, 2,55,16) = (/ &
     &6.2350e+07_rb,4.6763e+07_rb,3.1175e+07_rb,1.5588e+07_rb,1.8842e+02_rb/)
      kbo(:, 3,55,16) = (/ &
     &5.8012e+07_rb,4.3509e+07_rb,2.9006e+07_rb,1.4503e+07_rb,1.5117e+02_rb/)
      kbo(:, 4,55,16) = (/ &
     &5.4054e+07_rb,4.0541e+07_rb,2.7027e+07_rb,1.3514e+07_rb,1.0060e+02_rb/)
      kbo(:, 5,55,16) = (/ &
     &5.0479e+07_rb,3.7859e+07_rb,2.5240e+07_rb,1.2620e+07_rb,5.1596e+01_rb/)
      kbo(:, 1,56,16) = (/ &
     &6.7946e+07_rb,5.0959e+07_rb,3.3973e+07_rb,1.6986e+07_rb,2.2213e+02_rb/)
      kbo(:, 2,56,16) = (/ &
     &6.3087e+07_rb,4.7315e+07_rb,3.1543e+07_rb,1.5771e+07_rb,1.9599e+02_rb/)
      kbo(:, 3,56,16) = (/ &
     &5.8660e+07_rb,4.3995e+07_rb,2.9330e+07_rb,1.4665e+07_rb,1.7426e+02_rb/)
      kbo(:, 4,56,16) = (/ &
     &5.4632e+07_rb,4.0974e+07_rb,2.7316e+07_rb,1.3658e+07_rb,1.3341e+02_rb/)
      kbo(:, 5,56,16) = (/ &
     &5.0980e+07_rb,3.8235e+07_rb,2.5490e+07_rb,1.2745e+07_rb,7.5113e+01_rb/)
      kbo(:, 1,57,16) = (/ &
     &6.8848e+07_rb,5.1636e+07_rb,3.4424e+07_rb,1.7212e+07_rb,2.3193e+02_rb/)
      kbo(:, 2,57,16) = (/ &
     &6.3831e+07_rb,4.7873e+07_rb,3.1915e+07_rb,1.5958e+07_rb,2.0396e+02_rb/)
      kbo(:, 3,57,16) = (/ &
     &5.9337e+07_rb,4.4503e+07_rb,2.9325e+07_rb,1.4834e+07_rb,1.8090e+02_rb/)
      kbo(:, 4,57,16) = (/ &
     &5.5232e+07_rb,4.1423e+07_rb,2.7615e+07_rb,1.3808e+07_rb,1.4946e+02_rb/)
      kbo(:, 5,57,16) = (/ &
     &5.1498e+07_rb,3.8623e+07_rb,2.5749e+07_rb,1.2874e+07_rb,9.6802e+01_rb/)
      kbo(:, 1,58,16) = (/ &
     &1.8392e+07_rb,1.6863e+07_rb,1.4458e+07_rb,1.0126e+07_rb,2.4193e+02_rb/)
      kbo(:, 2,58,16) = (/ &
     &1.7018e+07_rb,1.5603e+07_rb,1.3378e+07_rb,9.3701e+06_rb,2.1195e+02_rb/)
      kbo(:, 3,58,16) = (/ &
     &1.5812e+07_rb,1.4497e+07_rb,1.2430e+07_rb,8.7057e+06_rb,1.8755e+02_rb/)
      kbo(:, 4,58,16) = (/ &
     &1.4710e+07_rb,1.3487e+07_rb,1.1564e+07_rb,8.0993e+06_rb,1.6722e+02_rb/)
      kbo(:, 5,58,16) = (/ &
     &1.3707e+07_rb,1.2567e+07_rb,1.0775e+07_rb,7.5470e+06_rb,1.3461e+02_rb/)
      kbo(:, 1,59,16) = (/ &
     &2.1701e+07_rb,1.8948e+07_rb,1.5114e+07_rb,9.4051e+06_rb,2.4616e+02_rb/)
      kbo(:, 2,59,16) = (/ &
     &2.0062e+07_rb,1.7517e+07_rb,1.3973e+07_rb,8.5918e+06_rb,2.1536e+02_rb/)
      kbo(:, 3,59,16) = (/ &
     &1.8637e+07_rb,1.6273e+07_rb,1.2980e+07_rb,8.0770e+06_rb,1.9038e+02_rb/)
      kbo(:, 4,59,16) = (/ &
     &1.7332e+07_rb,1.5134e+07_rb,1.2072e+07_rb,7.5118e+06_rb,1.6959e+02_rb/)
      kbo(:, 5,59,16) = (/ &
     &1.6145e+07_rb,1.4097e+07_rb,1.1245e+07_rb,6.9973e+06_rb,1.5180e+02_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &3.3839e-04_rb,2.4739e-04_rb,2.2846e-04_rb,2.3376e-04_rb,2.2622e-04_rb,2.3188e-04_rb, &
     &2.2990e-04_rb,2.2532e-04_rb,2.1233e-04_rb,2.0593e-04_rb,2.0716e-04_rb,2.0809e-04_rb, &
     &2.0889e-04_rb,2.0932e-04_rb,2.0944e-04_rb,2.0945e-04_rb/)
      forrefo(2,:) = (/ &
     &3.4391e-04_rb,2.6022e-04_rb,2.3449e-04_rb,2.4544e-04_rb,2.3831e-04_rb,2.3014e-04_rb, &
     &2.3729e-04_rb,2.2726e-04_rb,2.1892e-04_rb,1.9223e-04_rb,2.1291e-04_rb,2.1406e-04_rb, &
     &2.1491e-04_rb,2.1548e-04_rb,2.1562e-04_rb,2.1567e-04_rb/)
      forrefo(3,:) = (/ &
     &3.4219e-04_rb,2.7334e-04_rb,2.3727e-04_rb,2.4515e-04_rb,2.5272e-04_rb,2.4212e-04_rb, &
     &2.3824e-04_rb,2.3615e-04_rb,2.2724e-04_rb,2.2381e-04_rb,1.9634e-04_rb,2.1625e-04_rb, &
     &2.1963e-04_rb,2.2032e-04_rb,2.2057e-04_rb,2.2058e-04_rb/)
      forrefo(4,:) = (/ &
     &3.1684e-04_rb,2.4823e-04_rb,2.4890e-04_rb,2.4577e-04_rb,2.4106e-04_rb,2.4353e-04_rb, &
     &2.4038e-04_rb,2.3932e-04_rb,2.3604e-04_rb,2.3773e-04_rb,2.4243e-04_rb,2.2597e-04_rb, &
     &2.2879e-04_rb,2.2440e-04_rb,2.1104e-04_rb,2.1460e-04_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 2.62922e-01_rb, 2.29106e-01_rb, 1.99640e-01_rb, 1.73964e-01_rb, 1.51589e-01_rb, &
     & 1.32093e-01_rb, 1.15104e-01_rb, 1.00300e-01_rb, 8.74000e-02_rb, 7.61592e-02_rb/)
      selfrefo(:, 2) = (/ &
     & 2.45448e-01_rb, 2.13212e-01_rb, 1.85210e-01_rb, 1.60886e-01_rb, 1.39756e-01_rb, &
     & 1.21401e-01_rb, 1.05457e-01_rb, 9.16070e-02_rb, 7.95759e-02_rb, 6.91249e-02_rb/)
      selfrefo(:, 3) = (/ &
     & 2.41595e-01_rb, 2.09697e-01_rb, 1.82010e-01_rb, 1.57979e-01_rb, 1.37121e-01_rb, &
     & 1.19016e-01_rb, 1.03302e-01_rb, 8.96630e-02_rb, 7.78246e-02_rb, 6.75492e-02_rb/)
      selfrefo(:, 4) = (/ &
     & 2.44818e-01_rb, 2.12172e-01_rb, 1.83880e-01_rb, 1.59360e-01_rb, 1.38110e-01_rb, &
     & 1.19694e-01_rb, 1.03733e-01_rb, 8.99010e-02_rb, 7.79131e-02_rb, 6.75238e-02_rb/)
      selfrefo(:, 5) = (/ &
     & 2.43458e-01_rb, 2.10983e-01_rb, 1.82840e-01_rb, 1.58451e-01_rb, 1.37315e-01_rb, &
     & 1.18998e-01_rb, 1.03125e-01_rb, 8.93690e-02_rb, 7.74480e-02_rb, 6.71171e-02_rb/)
      selfrefo(:, 6) = (/ &
     & 2.40186e-01_rb, 2.08745e-01_rb, 1.81420e-01_rb, 1.57672e-01_rb, 1.37032e-01_rb, &
     & 1.19095e-01_rb, 1.03505e-01_rb, 8.99560e-02_rb, 7.81806e-02_rb, 6.79467e-02_rb/)
      selfrefo(:, 7) = (/ &
     & 2.42752e-01_rb, 2.10579e-01_rb, 1.82670e-01_rb, 1.58460e-01_rb, 1.37459e-01_rb, &
     & 1.19240e-01_rb, 1.03437e-01_rb, 8.97280e-02_rb, 7.78359e-02_rb, 6.75200e-02_rb/)
      selfrefo(:, 8) = (/ &
     & 2.39620e-01_rb, 2.08166e-01_rb, 1.80840e-01_rb, 1.57101e-01_rb, 1.36479e-01_rb, &
     & 1.18563e-01_rb, 1.03000e-01_rb, 8.94790e-02_rb, 7.77332e-02_rb, 6.75292e-02_rb/)
      selfrefo(:, 9) = (/ &
     & 2.38856e-01_rb, 2.07166e-01_rb, 1.79680e-01_rb, 1.55841e-01_rb, 1.35165e-01_rb, &
     & 1.17232e-01_rb, 1.01678e-01_rb, 8.81880e-02_rb, 7.64877e-02_rb, 6.63397e-02_rb/)
      selfrefo(:,10) = (/ &
     & 2.29821e-01_rb, 2.00586e-01_rb, 1.75070e-01_rb, 1.52800e-01_rb, 1.33363e-01_rb, &
     & 1.16398e-01_rb, 1.01591e-01_rb, 8.86680e-02_rb, 7.73887e-02_rb, 6.75443e-02_rb/)
      selfrefo(:,11) = (/ &
     & 2.39945e-01_rb, 2.08186e-01_rb, 1.80630e-01_rb, 1.56722e-01_rb, 1.35978e-01_rb, &
     & 1.17980e-01_rb, 1.02364e-01_rb, 8.88150e-02_rb, 7.70594e-02_rb, 6.68598e-02_rb/)
      selfrefo(:,12) = (/ &
     & 2.40271e-01_rb, 2.08465e-01_rb, 1.80870e-01_rb, 1.56927e-01_rb, 1.36154e-01_rb, &
     & 1.18131e-01_rb, 1.02494e-01_rb, 8.89260e-02_rb, 7.71545e-02_rb, 6.69412e-02_rb/)
      selfrefo(:,13) = (/ &
     & 2.40503e-01_rb, 2.08670e-01_rb, 1.81050e-01_rb, 1.57086e-01_rb, 1.36294e-01_rb, &
     & 1.18254e-01_rb, 1.02602e-01_rb, 8.90210e-02_rb, 7.72380e-02_rb, 6.70147e-02_rb/)
      selfrefo(:,14) = (/ &
     & 2.40670e-01_rb, 2.08811e-01_rb, 1.81170e-01_rb, 1.57188e-01_rb, 1.36380e-01_rb, &
     & 1.18327e-01_rb, 1.02663e-01_rb, 8.90730e-02_rb, 7.72819e-02_rb, 6.70517e-02_rb/)
      selfrefo(:,15) = (/ &
     & 2.40711e-01_rb, 2.08846e-01_rb, 1.81200e-01_rb, 1.57213e-01_rb, 1.36402e-01_rb, &
     & 1.18346e-01_rb, 1.02679e-01_rb, 8.90870e-02_rb, 7.72939e-02_rb, 6.70621e-02_rb/)
      selfrefo(:,16) = (/ &
     & 2.40727e-01_rb, 2.08859e-01_rb, 1.81210e-01_rb, 1.57221e-01_rb, 1.36408e-01_rb, &
     & 1.18350e-01_rb, 1.02682e-01_rb, 8.90890e-02_rb, 7.72952e-02_rb, 6.70627e-02_rb/)

      end subroutine lw_kgb04

! **************************************************************************
      subroutine lw_kgb05
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg05, only : fracrefao, fracrefbo, kao, kbo, kao_mo3, &
                            selfrefo, forrefo, ccl4o

      implicit none
      save

! Planck fraction mapping level : P = 473.42 mb, T = 259.83
      fracrefao(:, 1) = (/ &
        1.4111e-01_rb,1.4222e-01_rb,1.3802e-01_rb,1.3101e-01_rb,1.2244e-01_rb,1.0691e-01_rb, &
        8.8703e-02_rb,6.7130e-02_rb,4.5509e-02_rb,4.9866e-03_rb,4.1214e-03_rb,3.2557e-03_rb, &
        2.3805e-03_rb,1.5450e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 2) = (/ &
        1.4152e-01_rb,1.4271e-01_rb,1.3784e-01_rb,1.3075e-01_rb,1.2215e-01_rb,1.0674e-01_rb, &
        8.8686e-02_rb,6.7135e-02_rb,4.5508e-02_rb,4.9866e-03_rb,4.1214e-03_rb,3.2558e-03_rb, &
        2.3805e-03_rb,1.5450e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 3) = (/ &
        1.4159e-01_rb,1.4300e-01_rb,1.3781e-01_rb,1.3094e-01_rb,1.2192e-01_rb,1.0661e-01_rb, &
        8.8529e-02_rb,6.7127e-02_rb,4.5511e-02_rb,4.9877e-03_rb,4.1214e-03_rb,3.2558e-03_rb, &
        2.3805e-03_rb,1.5450e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 4) = (/ &
        1.4162e-01_rb,1.4337e-01_rb,1.3774e-01_rb,1.3122e-01_rb,1.2172e-01_rb,1.0641e-01_rb, &
        8.8384e-02_rb,6.7056e-02_rb,4.5514e-02_rb,4.9880e-03_rb,4.1214e-03_rb,3.2557e-03_rb, &
        2.3805e-03_rb,1.5450e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 5) = (/ &
        1.4161e-01_rb,1.4370e-01_rb,1.3770e-01_rb,1.3143e-01_rb,1.2173e-01_rb,1.0613e-01_rb, &
        8.8357e-02_rb,6.6874e-02_rb,4.5509e-02_rb,4.9883e-03_rb,4.1214e-03_rb,3.2558e-03_rb, &
        2.3804e-03_rb,1.5450e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 6) = (/ &
        1.4154e-01_rb,1.4405e-01_rb,1.3771e-01_rb,1.3169e-01_rb,1.2166e-01_rb,1.0603e-01_rb, &
        8.8193e-02_rb,6.6705e-02_rb,4.5469e-02_rb,4.9902e-03_rb,4.1214e-03_rb,3.2558e-03_rb, &
        2.3804e-03_rb,1.5450e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 7) = (/ &
        1.4126e-01_rb,1.4440e-01_rb,1.3790e-01_rb,1.3214e-01_rb,1.2153e-01_rb,1.0603e-01_rb, &
        8.7908e-02_rb,6.6612e-02_rb,4.5269e-02_rb,4.9900e-03_rb,4.1256e-03_rb,3.2558e-03_rb, &
        2.3804e-03_rb,1.5451e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 8) = (/ &
        1.4076e-01_rb,1.4415e-01_rb,1.3885e-01_rb,1.3286e-01_rb,1.2147e-01_rb,1.0612e-01_rb, &
        8.7579e-02_rb,6.6280e-02_rb,4.4977e-02_rb,4.9782e-03_rb,4.1200e-03_rb,3.2620e-03_rb, &
        2.3820e-03_rb,1.5452e-03_rb,5.8423e-04_rb,8.2275e-05_rb/)
      fracrefao(:, 9) = (/ &
        1.4205e-01_rb,1.4496e-01_rb,1.4337e-01_rb,1.3504e-01_rb,1.2260e-01_rb,1.0428e-01_rb, &
        8.4946e-02_rb,6.3625e-02_rb,4.2951e-02_rb,4.7313e-03_rb,3.9157e-03_rb,3.0879e-03_rb, &
        2.2666e-03_rb,1.5193e-03_rb,5.7469e-04_rb,8.1674e-05_rb/)

! Planck fraction mapping level : P = 0.2369280 mbar, T = 253.60 K
      fracrefbo(:, 1) = (/ &
        1.4075e-01_rb,1.4196e-01_rb,1.3833e-01_rb,1.3345e-01_rb,1.2234e-01_rb,1.0718e-01_rb, &
        8.8004e-02_rb,6.6308e-02_rb,4.5028e-02_rb,4.9029e-03_rb,4.0377e-03_rb,3.1870e-03_rb, &
        2.3503e-03_rb,1.5146e-03_rb,5.7165e-04_rb,8.2371e-05_rb/)
      fracrefbo(:, 2) = (/ &
        1.4081e-01_rb,1.4225e-01_rb,1.3890e-01_rb,1.3410e-01_rb,1.2254e-01_rb,1.0680e-01_rb, &
        8.7391e-02_rb,6.5819e-02_rb,4.4725e-02_rb,4.9121e-03_rb,4.0420e-03_rb,3.1869e-03_rb, &
        2.3504e-03_rb,1.5146e-03_rb,5.7165e-04_rb,8.2371e-05_rb/)
      fracrefbo(:, 3) = (/ &
        1.4087e-01_rb,1.4227e-01_rb,1.3920e-01_rb,1.3395e-01_rb,1.2270e-01_rb,1.0694e-01_rb, &
        8.7229e-02_rb,6.5653e-02_rb,4.4554e-02_rb,4.8797e-03_rb,4.0460e-03_rb,3.1939e-03_rb, &
        2.3505e-03_rb,1.5146e-03_rb,5.7165e-04_rb,8.1910e-05_rb/)
      fracrefbo(:, 4) = (/ &
        1.4089e-01_rb,1.4238e-01_rb,1.3956e-01_rb,1.3379e-01_rb,1.2284e-01_rb,1.0688e-01_rb, &
        8.7192e-02_rb,6.5490e-02_rb,4.4390e-02_rb,4.8395e-03_rb,4.0173e-03_rb,3.2070e-03_rb, &
        2.3559e-03_rb,1.5146e-03_rb,5.7165e-04_rb,8.2371e-05_rb/)
      fracrefbo(:, 5) = (/ &
        1.4091e-01_rb,1.4417e-01_rb,1.4194e-01_rb,1.3457e-01_rb,1.2167e-01_rb,1.0551e-01_rb, &
        8.6450e-02_rb,6.4889e-02_rb,4.3584e-02_rb,4.7551e-03_rb,3.9509e-03_rb,3.1374e-03_rb, &
        2.3226e-03_rb,1.4942e-03_rb,5.7545e-04_rb,8.0887e-05_rb/)

! Minor gas mapping level :
!     lower - o3, p = 317.34 mbar, t = 240.77 k
!     lower - ccl4

      ccl4o(:) = (/ &
        26.1407_rb, 53.9776_rb, 63.8085_rb, 36.1701_rb, 15.4099_rb, 10.23116_rb, &
        4.82948_rb, 5.03836_rb, 1.75558_rb, 0._rb,      0._rb,      0._rb,       &
        0._rb,      0._rb,      0._rb,      0._rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &2.0627e-06_rb,7.6619e-06_rb,1.2821e-05_rb,1.8547e-05_rb,2.5694e-05_rb,3.6833e-05_rb, &
     &6.0675e-05_rb,1.0691e-04_rb,4.5186e-05_rb/)
      kao(:, 2, 1, 1) = (/ &
     &2.7702e-06_rb,1.0260e-05_rb,1.6067e-05_rb,2.2990e-05_rb,3.0808e-05_rb,4.1244e-05_rb, &
     &5.9411e-05_rb,1.1066e-04_rb,5.2628e-05_rb/)
      kao(:, 3, 1, 1) = (/ &
     &3.6863e-06_rb,1.3689e-05_rb,2.0068e-05_rb,2.8133e-05_rb,3.7359e-05_rb,4.8756e-05_rb, &
     &6.5660e-05_rb,1.0750e-04_rb,6.3147e-05_rb/)
      kao(:, 4, 1, 1) = (/ &
     &4.8209e-06_rb,1.8184e-05_rb,2.5811e-05_rb,3.2670e-05_rb,4.2393e-05_rb,5.7509e-05_rb, &
     &7.7672e-05_rb,1.1739e-04_rb,6.0614e-05_rb/)
      kao(:, 5, 1, 1) = (/ &
     &6.2011e-06_rb,2.3679e-05_rb,3.3185e-05_rb,4.1098e-05_rb,4.8872e-05_rb,6.0199e-05_rb, &
     &8.6830e-05_rb,1.3320e-04_rb,5.7959e-05_rb/)
      kao(:, 1, 2, 1) = (/ &
     &2.3538e-06_rb,6.7359e-06_rb,1.0280e-05_rb,1.4331e-05_rb,1.9304e-05_rb,2.5963e-05_rb, &
     &4.0842e-05_rb,7.0297e-05_rb,3.0786e-05_rb/)
      kao(:, 2, 2, 1) = (/ &
     &3.1800e-06_rb,9.2776e-06_rb,1.3401e-05_rb,1.8014e-05_rb,2.3501e-05_rb,3.0468e-05_rb, &
     &4.1030e-05_rb,7.2758e-05_rb,3.6807e-05_rb/)
      kao(:, 3, 2, 1) = (/ &
     &4.2644e-06_rb,1.2589e-05_rb,1.7414e-05_rb,2.2365e-05_rb,2.8924e-05_rb,3.6996e-05_rb, &
     &4.7459e-05_rb,7.1458e-05_rb,4.4453e-05_rb/)
      kao(:, 4, 2, 1) = (/ &
     &5.6334e-06_rb,1.6843e-05_rb,2.2937e-05_rb,2.8083e-05_rb,3.3609e-05_rb,4.3642e-05_rb, &
     &5.6794e-05_rb,7.9540e-05_rb,3.7769e-05_rb/)
      kao(:, 5, 2, 1) = (/ &
     &7.3165e-06_rb,2.2208e-05_rb,2.9987e-05_rb,3.5834e-05_rb,4.1117e-05_rb,4.7236e-05_rb, &
     &6.4507e-05_rb,9.1784e-05_rb,4.0356e-05_rb/)
      kao(:, 1, 3, 1) = (/ &
     &3.4149e-06_rb,6.8426e-06_rb,8.9161e-06_rb,1.1292e-05_rb,1.3987e-05_rb,1.7689e-05_rb, &
     &2.4623e-05_rb,4.0560e-05_rb,1.8741e-05_rb/)
      kao(:, 2, 3, 1) = (/ &
     &4.6295e-06_rb,9.5910e-06_rb,1.2218e-05_rb,1.4773e-05_rb,1.7906e-05_rb,2.1627e-05_rb, &
     &2.7175e-05_rb,4.1902e-05_rb,2.3338e-05_rb/)
      kao(:, 3, 3, 1) = (/ &
     &6.2620e-06_rb,1.3265e-05_rb,1.6740e-05_rb,1.9517e-05_rb,2.2851e-05_rb,2.6791e-05_rb, &
     &3.2311e-05_rb,4.3092e-05_rb,2.7303e-05_rb/)
      kao(:, 4, 3, 1) = (/ &
     &8.3825e-06_rb,1.8025e-05_rb,2.2633e-05_rb,2.5988e-05_rb,2.8756e-05_rb,3.2402e-05_rb, &
     &3.9084e-05_rb,5.0182e-05_rb,2.3119e-05_rb/)
      kao(:, 5, 3, 1) = (/ &
     &1.1030e-05_rb,2.4184e-05_rb,3.0218e-05_rb,3.4428e-05_rb,3.7379e-05_rb,3.9667e-05_rb, &
     &4.3975e-05_rb,5.8391e-05_rb,2.6206e-05_rb/)
      kao(:, 1, 4, 1) = (/ &
     &5.0171e-06_rb,7.6880e-06_rb,8.7791e-06_rb,9.8121e-06_rb,1.1212e-05_rb,1.2838e-05_rb, &
     &1.6066e-05_rb,2.4377e-05_rb,1.1689e-05_rb/)
      kao(:, 2, 4, 1) = (/ &
     &6.7946e-06_rb,1.0789e-05_rb,1.2297e-05_rb,1.3454e-05_rb,1.4738e-05_rb,1.6388e-05_rb, &
     &1.8978e-05_rb,2.5098e-05_rb,1.5123e-05_rb/)
      kao(:, 3, 4, 1) = (/ &
     &9.2346e-06_rb,1.5122e-05_rb,1.7217e-05_rb,1.8715e-05_rb,1.9870e-05_rb,2.1284e-05_rb, &
     &2.3541e-05_rb,2.8475e-05_rb,1.5817e-05_rb/)
      kao(:, 4, 4, 1) = (/ &
     &1.2485e-05_rb,2.0883e-05_rb,2.3764e-05_rb,2.5687e-05_rb,2.6834e-05_rb,2.7524e-05_rb, &
     &2.9511e-05_rb,3.4495e-05_rb,1.5461e-05_rb/)
      kao(:, 5, 4, 1) = (/ &
     &1.6688e-05_rb,2.8377e-05_rb,3.2282e-05_rb,3.4761e-05_rb,3.6075e-05_rb,3.6212e-05_rb, &
     &3.5832e-05_rb,4.0812e-05_rb,1.7475e-05_rb/)
      kao(:, 1, 5, 1) = (/ &
     &6.8849e-06_rb,8.9001e-06_rb,9.3032e-06_rb,9.4606e-06_rb,9.6746e-06_rb,1.0252e-05_rb, &
     &1.1422e-05_rb,1.5315e-05_rb,7.4550e-06_rb/)
      kao(:, 2, 5, 1) = (/ &
     &9.2829e-06_rb,1.2355e-05_rb,1.3006e-05_rb,1.3223e-05_rb,1.3294e-05_rb,1.3454e-05_rb, &
     &1.4383e-05_rb,1.6505e-05_rb,9.6725e-06_rb/)
      kao(:, 3, 5, 1) = (/ &
     &1.2628e-05_rb,1.7354e-05_rb,1.8341e-05_rb,1.8646e-05_rb,1.8560e-05_rb,1.8269e-05_rb, &
     &1.8381e-05_rb,1.9847e-05_rb,1.0746e-05_rb/)
      kao(:, 4, 5, 1) = (/ &
     &1.7178e-05_rb,2.4178e-05_rb,2.5699e-05_rb,2.6093e-05_rb,2.5857e-05_rb,2.5047e-05_rb, &
     &2.4039e-05_rb,2.4517e-05_rb,1.1239e-05_rb/)
      kao(:, 5, 5, 1) = (/ &
     &2.3232e-05_rb,3.3207e-05_rb,3.5417e-05_rb,3.5933e-05_rb,3.5521e-05_rb,3.4179e-05_rb, &
     &3.1639e-05_rb,3.0107e-05_rb,1.3413e-05_rb/)
      kao(:, 1, 6, 1) = (/ &
     &8.9763e-06_rb,1.0310e-05_rb,1.0152e-05_rb,9.7112e-06_rb,9.1573e-06_rb,8.7286e-06_rb, &
     &8.7100e-06_rb,1.0151e-05_rb,5.0468e-06_rb/)
      kao(:, 2, 6, 1) = (/ &
     &1.1944e-05_rb,1.4094e-05_rb,1.4098e-05_rb,1.3570e-05_rb,1.2771e-05_rb,1.1934e-05_rb, &
     &1.1288e-05_rb,1.1804e-05_rb,6.4542e-06_rb/)
      kao(:, 3, 6, 1) = (/ &
     &1.6264e-05_rb,1.9621e-05_rb,1.9778e-05_rb,1.9110e-05_rb,1.7995e-05_rb,1.6619e-05_rb, &
     &1.5210e-05_rb,1.4762e-05_rb,7.7867e-06_rb/)
      kao(:, 4, 6, 1) = (/ &
     &2.2127e-05_rb,2.7394e-05_rb,2.7774e-05_rb,2.6897e-05_rb,2.5333e-05_rb,2.3366e-05_rb, &
     &2.0855e-05_rb,1.8805e-05_rb,9.1527e-06_rb/)
      kao(:, 5, 6, 1) = (/ &
     &3.0031e-05_rb,3.7976e-05_rb,3.8538e-05_rb,3.7483e-05_rb,3.5390e-05_rb,3.2530e-05_rb, &
     &2.8729e-05_rb,2.4137e-05_rb,1.0787e-05_rb/)
      kao(:, 1, 7, 1) = (/ &
     &1.2661e-05_rb,1.3255e-05_rb,1.2476e-05_rb,1.1360e-05_rb,1.0126e-05_rb,8.8167e-06_rb, &
     &7.6811e-06_rb,7.2819e-06_rb,3.4701e-06_rb/)
      kao(:, 2, 7, 1) = (/ &
     &1.6507e-05_rb,1.7720e-05_rb,1.6880e-05_rb,1.5585e-05_rb,1.3986e-05_rb,1.2151e-05_rb, &
     &1.0259e-05_rb,8.9670e-06_rb,4.6214e-06_rb/)
      kao(:, 3, 7, 1) = (/ &
     &2.2311e-05_rb,2.4431e-05_rb,2.3459e-05_rb,2.1822e-05_rb,1.9662e-05_rb,1.7085e-05_rb, &
     &1.4272e-05_rb,1.1641e-05_rb,5.9077e-06_rb/)
      kao(:, 4, 7, 1) = (/ &
     &3.0470e-05_rb,3.3907e-05_rb,3.2735e-05_rb,3.0550e-05_rb,2.7736e-05_rb,2.4159e-05_rb, &
     &2.0093e-05_rb,1.5711e-05_rb,7.6152e-06_rb/)
      kao(:, 5, 7, 1) = (/ &
     &4.1581e-05_rb,4.6778e-05_rb,4.5634e-05_rb,4.2827e-05_rb,3.9000e-05_rb,3.4010e-05_rb, &
     &2.8185e-05_rb,2.1414e-05_rb,9.5151e-06_rb/)
      kao(:, 1, 8, 1) = (/ &
     &2.2110e-05_rb,2.1516e-05_rb,1.9520e-05_rb,1.7197e-05_rb,1.4610e-05_rb,1.1834e-05_rb, &
     &8.9776e-06_rb,6.4038e-06_rb,2.9294e-06_rb/)
      kao(:, 2, 8, 1) = (/ &
     &2.8119e-05_rb,2.7833e-05_rb,2.5591e-05_rb,2.2815e-05_rb,1.9548e-05_rb,1.6035e-05_rb, &
     &1.2291e-05_rb,8.4633e-06_rb,3.7469e-06_rb/)
      kao(:, 3, 8, 1) = (/ &
     &3.7519e-05_rb,3.7575e-05_rb,3.4963e-05_rb,3.1329e-05_rb,2.6989e-05_rb,2.2379e-05_rb, &
     &1.7209e-05_rb,1.1658e-05_rb,5.1343e-06_rb/)
      kao(:, 4, 8, 1) = (/ &
     &5.1233e-05_rb,5.1900e-05_rb,4.8534e-05_rb,4.3661e-05_rb,3.7852e-05_rb,3.1455e-05_rb, &
     &2.4380e-05_rb,1.6303e-05_rb,6.8677e-06_rb/)
      kao(:, 5, 8, 1) = (/ &
     &7.0275e-05_rb,7.1610e-05_rb,6.7129e-05_rb,6.0809e-05_rb,5.2885e-05_rb,4.4177e-05_rb, &
     &3.4435e-05_rb,2.2981e-05_rb,8.8724e-06_rb/)
      kao(:, 1, 9, 1) = (/ &
     &7.5307e-05_rb,6.8982e-05_rb,6.0564e-05_rb,5.1649e-05_rb,4.2335e-05_rb,3.2789e-05_rb, &
     &2.3004e-05_rb,1.2766e-05_rb,4.1976e-06_rb/)
      kao(:, 2, 9, 1) = (/ &
     &9.3306e-05_rb,8.6322e-05_rb,7.6065e-05_rb,6.5167e-05_rb,5.3786e-05_rb,4.2095e-05_rb, &
     &2.9960e-05_rb,1.6892e-05_rb,6.0950e-06_rb/)
      kao(:, 3, 9, 1) = (/ &
     &1.2202e-04_rb,1.1392e-04_rb,1.0068e-04_rb,8.6696e-05_rb,7.2080e-05_rb,5.6739e-05_rb, &
     &4.0820e-05_rb,2.3292e-05_rb,8.3200e-06_rb/)
      kao(:, 4, 9, 1) = (/ &
     &1.6580e-04_rb,1.5570e-04_rb,1.3769e-04_rb,1.1869e-04_rb,9.9119e-05_rb,7.8735e-05_rb, &
     &5.6759e-05_rb,3.2860e-05_rb,1.0538e-05_rb/)
      kao(:, 5, 9, 1) = (/ &
     &2.2845e-04_rb,2.1403e-04_rb,1.8995e-04_rb,1.6428e-04_rb,1.3747e-04_rb,1.0943e-04_rb, &
     &7.9478e-05_rb,4.6333e-05_rb,1.2645e-05_rb/)
      kao(:, 1,10, 1) = (/ &
     &2.9742e-04_rb,2.6515e-04_rb,2.2972e-04_rb,1.9315e-04_rb,1.5610e-04_rb,1.1880e-04_rb, &
     &8.1117e-05_rb,4.2319e-05_rb,6.3944e-06_rb/)
      kao(:, 2,10, 1) = (/ &
     &3.6180e-04_rb,3.2444e-04_rb,2.8214e-04_rb,2.3809e-04_rb,1.9281e-04_rb,1.4714e-04_rb, &
     &1.0099e-04_rb,5.3082e-05_rb,9.7791e-06_rb/)
      kao(:, 3,10, 1) = (/ &
     &4.6752e-04_rb,4.2137e-04_rb,3.6703e-04_rb,3.1031e-04_rb,2.5194e-04_rb,1.9278e-04_rb, &
     &1.3274e-04_rb,7.0692e-05_rb,1.3731e-05_rb/)
      kao(:, 4,10, 1) = (/ &
     &6.3024e-04_rb,5.6976e-04_rb,4.9706e-04_rb,4.2128e-04_rb,3.4229e-04_rb,2.6250e-04_rb, &
     &1.8103e-04_rb,9.7478e-05_rb,1.8873e-05_rb/)
      kao(:, 5,10, 1) = (/ &
     &8.6816e-04_rb,7.8510e-04_rb,6.8602e-04_rb,5.8224e-04_rb,4.7468e-04_rb,3.6394e-04_rb, &
     &2.5185e-04_rb,1.3590e-04_rb,2.4864e-05_rb/)
      kao(:, 1,11, 1) = (/ &
     &4.4215e-04_rb,3.9273e-04_rb,3.3972e-04_rb,2.8576e-04_rb,2.3079e-04_rb,1.7475e-04_rb, &
     &1.1858e-04_rb,6.1576e-05_rb,6.9970e-06_rb/)
      kao(:, 2,11, 1) = (/ &
     &5.4524e-04_rb,4.8624e-04_rb,4.2158e-04_rb,3.5507e-04_rb,2.8728e-04_rb,2.1797e-04_rb, &
     &1.4849e-04_rb,7.7710e-05_rb,1.0796e-05_rb/)
      kao(:, 3,11, 1) = (/ &
     &7.1212e-04_rb,6.3874e-04_rb,5.5491e-04_rb,4.6791e-04_rb,3.7968e-04_rb,2.8946e-04_rb, &
     &1.9751e-04_rb,1.0405e-04_rb,1.6156e-05_rb/)
      kao(:, 4,11, 1) = (/ &
     &9.7112e-04_rb,8.7198e-04_rb,7.5717e-04_rb,6.3918e-04_rb,5.1919e-04_rb,3.9650e-04_rb, &
     &2.7055e-04_rb,1.4272e-04_rb,2.2761e-05_rb/)
      kao(:, 5,11, 1) = (/ &
     &1.3431e-03_rb,1.2076e-03_rb,1.0484e-03_rb,8.8503e-04_rb,7.1868e-04_rb,5.4992e-04_rb, &
     &3.7609e-04_rb,1.9954e-04_rb,2.9940e-05_rb/)
      kao(:, 1,12, 1) = (/ &
     &4.8443e-04_rb,4.2968e-04_rb,3.7109e-04_rb,3.1171e-04_rb,2.5157e-04_rb,1.9045e-04_rb, &
     &1.2868e-04_rb,6.6606e-05_rb,6.8294e-06_rb/)
      kao(:, 2,12, 1) = (/ &
     &6.0269e-04_rb,5.3630e-04_rb,4.6453e-04_rb,3.9092e-04_rb,3.1605e-04_rb,2.4000e-04_rb, &
     &1.6259e-04_rb,8.4751e-05_rb,1.0182e-05_rb/)
      kao(:, 3,12, 1) = (/ &
     &7.9706e-04_rb,7.1250e-04_rb,6.1805e-04_rb,5.2087e-04_rb,4.2192e-04_rb,3.2125e-04_rb, &
     &2.1834e-04_rb,1.1445e-04_rb,1.5140e-05_rb/)
      kao(:, 4,12, 1) = (/ &
     &1.0910e-03_rb,9.7656e-04_rb,8.4711e-04_rb,7.1442e-04_rb,5.7891e-04_rb,4.4221e-04_rb, &
     &3.0181e-04_rb,1.5855e-04_rb,2.1388e-05_rb/)
      kao(:, 5,12, 1) = (/ &
     &1.5134e-03_rb,1.3551e-03_rb,1.1752e-03_rb,9.9093e-04_rb,8.0388e-04_rb,6.1444e-04_rb, &
     &4.2020e-04_rb,2.2117e-04_rb,2.9059e-05_rb/)
      kao(:, 1,13, 1) = (/ &
     &4.2625e-04_rb,3.7812e-04_rb,3.2711e-04_rb,2.7465e-04_rb,2.2177e-04_rb,1.6800e-04_rb, &
     &1.1342e-04_rb,5.8678e-05_rb,7.1193e-06_rb/)
      kao(:, 2,13, 1) = (/ &
     &5.3759e-04_rb,4.7832e-04_rb,4.1433e-04_rb,3.4844e-04_rb,2.8188e-04_rb,2.1423e-04_rb, &
     &1.4519e-04_rb,7.5588e-05_rb,1.0317e-05_rb/)
      kao(:, 3,13, 1) = (/ &
     &7.2018e-04_rb,6.4309e-04_rb,5.5764e-04_rb,4.6948e-04_rb,3.8030e-04_rb,2.8938e-04_rb, &
     &1.9678e-04_rb,1.0285e-04_rb,1.4041e-05_rb/)
      kao(:, 4,13, 1) = (/ &
     &9.8953e-04_rb,8.8546e-04_rb,7.6800e-04_rb,6.4711e-04_rb,5.2450e-04_rb,3.9956e-04_rb, &
     &2.7312e-04_rb,1.4349e-04_rb,1.9954e-05_rb/)
      kao(:, 5,13, 1) = (/ &
     &1.3790e-03_rb,1.2366e-03_rb,1.0731e-03_rb,9.0477e-04_rb,7.3377e-04_rb,5.6037e-04_rb, &
     &3.8365e-04_rb,2.0147e-04_rb,2.6871e-05_rb/)
      kao(:, 1, 1, 2) = (/ &
     &6.2300e-06_rb,2.3124e-05_rb,2.9698e-05_rb,3.9591e-05_rb,5.8380e-05_rb,9.5125e-05_rb, &
     &1.3537e-04_rb,2.8288e-04_rb,9.7543e-05_rb/)
      kao(:, 2, 1, 2) = (/ &
     &9.0246e-06_rb,3.3229e-05_rb,4.2160e-05_rb,4.9478e-05_rb,6.0779e-05_rb,8.5533e-05_rb, &
     &1.5267e-04_rb,2.3904e-04_rb,8.0422e-05_rb/)
      kao(:, 3, 1, 2) = (/ &
     &1.2807e-05_rb,4.6362e-05_rb,5.9074e-05_rb,6.7253e-05_rb,7.5131e-05_rb,8.9889e-05_rb, &
     &1.2494e-04_rb,2.6418e-04_rb,7.9145e-05_rb/)
      kao(:, 4, 1, 2) = (/ &
     &1.7786e-05_rb,6.3031e-05_rb,8.0225e-05_rb,9.3521e-05_rb,1.0301e-04_rb,1.0987e-04_rb, &
     &1.2899e-04_rb,2.0499e-04_rb,1.0830e-04_rb/)
      kao(:, 5, 1, 2) = (/ &
     &2.4116e-05_rb,8.3783e-05_rb,1.0646e-04_rb,1.2431e-04_rb,1.3944e-04_rb,1.5277e-04_rb, &
     &1.5881e-04_rb,2.0933e-04_rb,1.4344e-04_rb/)
      kao(:, 1, 2, 2) = (/ &
     &7.1421e-06_rb,2.2132e-05_rb,2.7590e-05_rb,3.3376e-05_rb,4.3482e-05_rb,6.6128e-05_rb, &
     &9.2521e-05_rb,1.8166e-04_rb,6.3119e-05_rb/)
      kao(:, 2, 2, 2) = (/ &
     &1.0480e-05_rb,3.2273e-05_rb,3.9804e-05_rb,4.4484e-05_rb,4.9766e-05_rb,6.2224e-05_rb, &
     &1.0181e-04_rb,1.6062e-04_rb,5.5112e-05_rb/)
      kao(:, 3, 2, 2) = (/ &
     &1.5069e-05_rb,4.5805e-05_rb,5.6454e-05_rb,6.2730e-05_rb,6.6262e-05_rb,7.1655e-05_rb, &
     &8.8554e-05_rb,1.7608e-04_rb,5.4833e-05_rb/)
      kao(:, 4, 2, 2) = (/ &
     &2.1158e-05_rb,6.3205e-05_rb,7.7534e-05_rb,8.6669e-05_rb,9.2743e-05_rb,9.3710e-05_rb, &
     &9.8368e-05_rb,1.4175e-04_rb,8.3989e-05_rb/)
      kao(:, 5, 2, 2) = (/ &
     &2.9018e-05_rb,8.5014e-05_rb,1.0387e-04_rb,1.1638e-04_rb,1.2535e-04_rb,1.3160e-04_rb, &
     &1.2952e-04_rb,1.4715e-04_rb,1.0168e-04_rb/)
      kao(:, 1, 3, 2) = (/ &
     &1.0134e-05_rb,2.3698e-05_rb,2.8098e-05_rb,3.0832e-05_rb,3.4559e-05_rb,4.2739e-05_rb, &
     &5.8413e-05_rb,1.0086e-04_rb,3.2546e-05_rb/)
      kao(:, 2, 3, 2) = (/ &
     &1.5067e-05_rb,3.5478e-05_rb,4.1782e-05_rb,4.4927e-05_rb,4.6304e-05_rb,4.9338e-05_rb, &
     &6.0952e-05_rb,9.7126e-05_rb,3.2271e-05_rb/)
      kao(:, 3, 3, 2) = (/ &
     &2.2020e-05_rb,5.1526e-05_rb,6.0415e-05_rb,6.4785e-05_rb,6.5815e-05_rb,6.4970e-05_rb, &
     &6.6401e-05_rb,9.8577e-05_rb,3.7377e-05_rb/)
      kao(:, 4, 3, 2) = (/ &
     &3.1402e-05_rb,7.2606e-05_rb,8.4750e-05_rb,9.0680e-05_rb,9.2879e-05_rb,9.1127e-05_rb, &
     &8.5666e-05_rb,9.2372e-05_rb,5.9881e-05_rb/)
      kao(:, 5, 3, 2) = (/ &
     &4.3798e-05_rb,9.9375e-05_rb,1.1577e-04_rb,1.2361e-04_rb,1.2645e-04_rb,1.2543e-04_rb, &
     &1.1954e-04_rb,1.0799e-04_rb,6.6216e-05_rb/)
      kao(:, 1, 4, 2) = (/ &
     &1.4395e-05_rb,2.6113e-05_rb,2.9242e-05_rb,3.0501e-05_rb,3.0825e-05_rb,3.2335e-05_rb, &
     &3.9824e-05_rb,5.5770e-05_rb,1.8987e-05_rb/)
      kao(:, 2, 4, 2) = (/ &
     &2.1448e-05_rb,3.9639e-05_rb,4.4560e-05_rb,4.6391e-05_rb,4.5970e-05_rb,4.4537e-05_rb, &
     &4.5440e-05_rb,6.3416e-05_rb,1.9764e-05_rb/)
      kao(:, 3, 4, 2) = (/ &
     &3.1834e-05_rb,5.8680e-05_rb,6.6149e-05_rb,6.8528e-05_rb,6.7668e-05_rb,6.3937e-05_rb, &
     &5.8285e-05_rb,6.2232e-05_rb,2.7306e-05_rb/)
      kao(:, 4, 4, 2) = (/ &
     &4.6271e-05_rb,8.4438e-05_rb,9.5181e-05_rb,9.8206e-05_rb,9.6873e-05_rb,9.1747e-05_rb, &
     &8.1681e-05_rb,6.9834e-05_rb,3.9615e-05_rb/)
      kao(:, 5, 4, 2) = (/ &
     &6.5671e-05_rb,1.1809e-04_rb,1.3274e-04_rb,1.3673e-04_rb,1.3457e-04_rb,1.2744e-04_rb, &
     &1.1523e-04_rb,9.3255e-05_rb,4.5946e-05_rb/)
      kao(:, 1, 5, 2) = (/ &
     &1.8932e-05_rb,2.8174e-05_rb,3.0127e-05_rb,3.0189e-05_rb,2.9303e-05_rb,2.7710e-05_rb, &
     &2.8724e-05_rb,3.6981e-05_rb,1.2871e-05_rb/)
      kao(:, 2, 5, 2) = (/ &
     &2.8193e-05_rb,4.3103e-05_rb,4.6483e-05_rb,4.6716e-05_rb,4.5184e-05_rb,4.1972e-05_rb, &
     &3.7796e-05_rb,4.2113e-05_rb,1.5002e-05_rb/)
      kao(:, 3, 5, 2) = (/ &
     &4.2171e-05_rb,6.4890e-05_rb,7.0138e-05_rb,7.0754e-05_rb,6.8197e-05_rb,6.2875e-05_rb, &
     &5.4545e-05_rb,4.7871e-05_rb,1.9971e-05_rb/)
      kao(:, 4, 5, 2) = (/ &
     &6.2206e-05_rb,9.5463e-05_rb,1.0284e-04_rb,1.0393e-04_rb,9.9869e-05_rb,9.1904e-05_rb, &
     &7.9592e-05_rb,6.1772e-05_rb,2.7676e-05_rb/)
      kao(:, 5, 5, 2) = (/ &
     &8.9897e-05_rb,1.3643e-04_rb,1.4635e-04_rb,1.4773e-04_rb,1.4191e-04_rb,1.3018e-04_rb, &
     &1.1303e-04_rb,8.6827e-05_rb,3.3409e-05_rb/)
      kao(:, 1, 6, 2) = (/ &
     &2.3688e-05_rb,3.0481e-05_rb,3.1060e-05_rb,3.0054e-05_rb,2.7961e-05_rb,2.5367e-05_rb, &
     &2.2999e-05_rb,2.6629e-05_rb,1.0275e-05_rb/)
      kao(:, 2, 6, 2) = (/ &
     &3.4582e-05_rb,4.5983e-05_rb,4.7133e-05_rb,4.6002e-05_rb,4.3141e-05_rb,3.8854e-05_rb, &
     &3.3303e-05_rb,2.9897e-05_rb,1.2224e-05_rb/)
      kao(:, 3, 6, 2) = (/ &
     &5.1699e-05_rb,6.9356e-05_rb,7.1684e-05_rb,7.0505e-05_rb,6.6405e-05_rb,5.9819e-05_rb, &
     &5.0541e-05_rb,3.9074e-05_rb,1.6179e-05_rb/)
      kao(:, 4, 6, 2) = (/ &
     &7.7368e-05_rb,1.0342e-04_rb,1.0714e-04_rb,1.0541e-04_rb,9.9595e-05_rb,8.9603e-05_rb, &
     &7.5459e-05_rb,5.5384e-05_rb,2.1206e-05_rb/)
      kao(:, 5, 6, 2) = (/ &
     &1.1347e-04_rb,1.5064e-04_rb,1.5637e-04_rb,1.5316e-04_rb,1.4449e-04_rb,1.2985e-04_rb, &
     &1.0910e-04_rb,8.0324e-05_rb,2.6862e-05_rb/)
      kao(:, 1, 7, 2) = (/ &
     &3.2355e-05_rb,3.6612e-05_rb,3.5642e-05_rb,3.3581e-05_rb,3.0393e-05_rb,2.6088e-05_rb, &
     &2.1415e-05_rb,1.9368e-05_rb,9.5314e-06_rb/)
      kao(:, 2, 7, 2) = (/ &
     &4.6022e-05_rb,5.3816e-05_rb,5.2843e-05_rb,4.9963e-05_rb,4.5493e-05_rb,3.9581e-05_rb, &
     &3.2262e-05_rb,2.4294e-05_rb,1.1772e-05_rb/)
      kao(:, 3, 7, 2) = (/ &
     &6.7685e-05_rb,8.0405e-05_rb,7.9935e-05_rb,7.5649e-05_rb,6.9218e-05_rb,6.0527e-05_rb, &
     &4.9342e-05_rb,3.5155e-05_rb,1.4463e-05_rb/)
      kao(:, 4, 7, 2) = (/ &
     &1.0120e-04_rb,1.2036e-04_rb,1.2034e-04_rb,1.1418e-04_rb,1.0434e-04_rb,9.1774e-05_rb, &
     &7.5257e-05_rb,5.2793e-05_rb,1.8375e-05_rb/)
      kao(:, 5, 7, 2) = (/ &
     &1.5071e-04_rb,1.7822e-04_rb,1.7733e-04_rb,1.6840e-04_rb,1.5410e-04_rb,1.3584e-04_rb, &
     &1.1141e-04_rb,7.8080e-05_rb,2.3868e-05_rb/)
      kao(:, 1, 8, 2) = (/ &
     &5.5226e-05_rb,5.6106e-05_rb,5.2275e-05_rb,4.7400e-05_rb,4.1299e-05_rb,3.4449e-05_rb, &
     &2.6593e-05_rb,1.7838e-05_rb,1.0224e-05_rb/)
      kao(:, 2, 8, 2) = (/ &
     &7.6682e-05_rb,7.9501e-05_rb,7.4950e-05_rb,6.8270e-05_rb,5.9838e-05_rb,5.0360e-05_rb, &
     &3.9211e-05_rb,2.5644e-05_rb,1.3679e-05_rb/)
      kao(:, 3, 8, 2) = (/ &
     &1.1162e-04_rb,1.1744e-04_rb,1.1105e-04_rb,1.0166e-04_rb,8.9544e-05_rb,7.5446e-05_rb, &
     &5.9105e-05_rb,3.8912e-05_rb,1.6924e-05_rb/)
      kao(:, 4, 8, 2) = (/ &
     &1.6426e-04_rb,1.7413e-04_rb,1.6536e-04_rb,1.5206e-04_rb,1.3480e-04_rb,1.1398e-04_rb, &
     &8.9434e-05_rb,5.9125e-05_rb,2.1439e-05_rb/)
      kao(:, 5, 8, 2) = (/ &
     &2.4317e-04_rb,2.5868e-04_rb,2.4524e-04_rb,2.2533e-04_rb,2.0048e-04_rb,1.6943e-04_rb, &
     &1.3297e-04_rb,8.8280e-05_rb,2.6395e-05_rb/)
      kao(:, 1, 9, 2) = (/ &
     &1.8480e-04_rb,1.7106e-04_rb,1.5192e-04_rb,1.3086e-04_rb,1.0905e-04_rb,8.6158e-05_rb, &
     &6.2340e-05_rb,3.6177e-05_rb,1.4903e-05_rb/)
      kao(:, 2, 9, 2) = (/ &
     &2.4641e-04_rb,2.3135e-04_rb,2.0759e-04_rb,1.8017e-04_rb,1.5122e-04_rb,1.2054e-04_rb, &
     &8.8111e-05_rb,5.2211e-05_rb,2.0425e-05_rb/)
      kao(:, 3, 9, 2) = (/ &
     &3.5294e-04_rb,3.3406e-04_rb,3.0047e-04_rb,2.6193e-04_rb,2.2100e-04_rb,1.7727e-04_rb, &
     &1.3000e-04_rb,7.7386e-05_rb,2.5956e-05_rb/)
      kao(:, 4, 9, 2) = (/ &
     &5.2033e-04_rb,4.9504e-04_rb,4.4640e-04_rb,3.9034e-04_rb,3.2978e-04_rb,2.6409e-04_rb, &
     &1.9434e-04_rb,1.1587e-04_rb,3.4101e-05_rb/)
      kao(:, 5, 9, 2) = (/ &
     &7.6495e-04_rb,7.3083e-04_rb,6.5854e-04_rb,5.7617e-04_rb,4.8713e-04_rb,3.9080e-04_rb, &
     &2.8811e-04_rb,1.7251e-04_rb,4.3686e-05_rb/)
      kao(:, 1,10, 2) = (/ &
     &7.1951e-04_rb,6.4170e-04_rb,5.5848e-04_rb,4.7254e-04_rb,3.8543e-04_rb,2.9616e-04_rb, &
     &2.0442e-04_rb,1.0914e-04_rb,2.8683e-05_rb/)
      kao(:, 2,10, 2) = (/ &
     &9.5077e-04_rb,8.5141e-04_rb,7.4327e-04_rb,6.3053e-04_rb,5.1626e-04_rb,3.9828e-04_rb, &
     &2.7613e-04_rb,1.4913e-04_rb,3.8199e-05_rb/)
      kao(:, 3,10, 2) = (/ &
     &1.3355e-03_rb,1.2001e-03_rb,1.0504e-03_rb,8.9360e-04_rb,7.3324e-04_rb,5.6715e-04_rb, &
     &3.9425e-04_rb,2.1480e-04_rb,4.7134e-05_rb/)
      kao(:, 4,10, 2) = (/ &
     &1.9553e-03_rb,1.7643e-03_rb,1.5470e-03_rb,1.3173e-03_rb,1.0803e-03_rb,8.3446e-04_rb, &
     &5.8166e-04_rb,3.1871e-04_rb,5.8212e-05_rb/)
      kao(:, 5,10, 2) = (/ &
     &2.8783e-03_rb,2.5953e-03_rb,2.2770e-03_rb,1.9407e-03_rb,1.5909e-03_rb,1.2316e-03_rb, &
     &8.6003e-04_rb,4.7307e-04_rb,7.1136e-05_rb/)
      kao(:, 1,11, 2) = (/ &
     &1.1139e-03_rb,9.8774e-04_rb,8.5580e-04_rb,7.2091e-04_rb,5.8416e-04_rb,4.4688e-04_rb, &
     &3.0678e-04_rb,1.6222e-04_rb,3.9161e-05_rb/)
      kao(:, 2,11, 2) = (/ &
     &1.5083e-03_rb,1.3423e-03_rb,1.1649e-03_rb,9.8455e-04_rb,8.0047e-04_rb,6.1397e-04_rb, &
     &4.2291e-04_rb,2.2482e-04_rb,5.4701e-05_rb/)
      kao(:, 3,11, 2) = (/ &
     &2.1559e-03_rb,1.9181e-03_rb,1.6686e-03_rb,1.4112e-03_rb,1.1484e-03_rb,8.7957e-04_rb, &
     &6.0584e-04_rb,3.2346e-04_rb,6.6564e-05_rb/)
      kao(:, 4,11, 2) = (/ &
     &3.1472e-03_rb,2.8085e-03_rb,2.4474e-03_rb,2.0740e-03_rb,1.6883e-03_rb,1.2953e-03_rb, &
     &8.9345e-04_rb,4.7890e-04_rb,8.3118e-05_rb/)
      kao(:, 5,11, 2) = (/ &
     &4.6362e-03_rb,4.1388e-03_rb,3.6072e-03_rb,3.0601e-03_rb,2.4938e-03_rb,1.9129e-03_rb, &
     &1.3182e-03_rb,7.0366e-04_rb,1.0053e-04_rb/)
      kao(:, 1,12, 2) = (/ &
     &1.2746e-03_rb,1.1293e-03_rb,9.7744e-04_rb,8.2280e-04_rb,6.6509e-04_rb,5.0694e-04_rb, &
     &3.4671e-04_rb,1.8255e-04_rb,4.0437e-05_rb/)
      kao(:, 2,12, 2) = (/ &
     &1.7556e-03_rb,1.5589e-03_rb,1.3508e-03_rb,1.1390e-03_rb,9.2389e-04_rb,7.0640e-04_rb, &
     &4.8477e-04_rb,2.5611e-04_rb,5.8338e-05_rb/)
      kao(:, 3,12, 2) = (/ &
     &2.5208e-03_rb,2.2390e-03_rb,1.9427e-03_rb,1.6407e-03_rb,1.3325e-03_rb,1.0177e-03_rb, &
     &6.9952e-04_rb,3.6990e-04_rb,7.2977e-05_rb/)
      kao(:, 4,12, 2) = (/ &
     &3.7199e-03_rb,3.3065e-03_rb,2.8736e-03_rb,2.4286e-03_rb,1.9715e-03_rb,1.5043e-03_rb, &
     &1.0308e-03_rb,5.4613e-04_rb,9.0596e-05_rb/)
      kao(:, 5,12, 2) = (/ &
     &5.4892e-03_rb,4.8847e-03_rb,4.2441e-03_rb,3.5888e-03_rb,2.9119e-03_rb,2.2221e-03_rb, &
     &1.5249e-03_rb,8.0704e-04_rb,1.1205e-04_rb/)
      kao(:, 1,13, 2) = (/ &
     &1.1670e-03_rb,1.0343e-03_rb,8.9403e-04_rb,7.5206e-04_rb,6.0815e-04_rb,4.6361e-04_rb, &
     &3.1684e-04_rb,1.6665e-04_rb,3.8297e-05_rb/)
      kao(:, 2,13, 2) = (/ &
     &1.6263e-03_rb,1.4448e-03_rb,1.2518e-03_rb,1.0548e-03_rb,8.5537e-04_rb,6.5308e-04_rb, &
     &4.4734e-04_rb,2.3634e-04_rb,5.4012e-05_rb/)
      kao(:, 3,13, 2) = (/ &
     &2.3576e-03_rb,2.0975e-03_rb,1.8201e-03_rb,1.5368e-03_rb,1.2479e-03_rb,9.5295e-04_rb, &
     &6.5303e-04_rb,3.4532e-04_rb,6.8858e-05_rb/)
      kao(:, 4,13, 2) = (/ &
     &3.5158e-03_rb,3.1277e-03_rb,2.7158e-03_rb,2.2939e-03_rb,1.8617e-03_rb,1.4210e-03_rb, &
     &9.7237e-04_rb,5.1302e-04_rb,8.6284e-05_rb/)
      kao(:, 5,13, 2) = (/ &
     &5.1868e-03_rb,4.6126e-03_rb,4.0043e-03_rb,3.3796e-03_rb,2.7407e-03_rb,2.0921e-03_rb, &
     &1.4302e-03_rb,7.5554e-04_rb,1.0703e-04_rb/)
      kao(:, 1, 1, 3) = (/ &
     &2.1967e-05_rb,7.0847e-05_rb,9.8056e-05_rb,1.1683e-04_rb,1.2672e-04_rb,1.3386e-04_rb, &
     &1.7335e-04_rb,3.0889e-04_rb,1.4104e-04_rb/)
      kao(:, 2, 1, 3) = (/ &
     &3.2805e-05_rb,1.0021e-04_rb,1.3719e-04_rb,1.6469e-04_rb,1.8755e-04_rb,2.0275e-04_rb, &
     &1.9715e-04_rb,2.9590e-04_rb,1.6430e-04_rb/)
      kao(:, 3, 1, 3) = (/ &
     &4.7330e-05_rb,1.3749e-04_rb,1.8575e-04_rb,2.2309e-04_rb,2.5257e-04_rb,2.7701e-04_rb, &
     &3.0293e-04_rb,3.2277e-04_rb,1.5833e-04_rb/)
      kao(:, 4, 1, 3) = (/ &
     &6.6184e-05_rb,1.8358e-04_rb,2.4505e-04_rb,2.9115e-04_rb,3.2275e-04_rb,3.3970e-04_rb, &
     &3.6327e-04_rb,4.7032e-04_rb,1.5296e-04_rb/)
      kao(:, 5, 1, 3) = (/ &
     &8.9953e-05_rb,2.3845e-04_rb,3.1568e-04_rb,3.7021e-04_rb,4.0111e-04_rb,4.1250e-04_rb, &
     &4.1712e-04_rb,4.9013e-04_rb,1.7116e-04_rb/)
      kao(:, 1, 2, 3) = (/ &
     &2.6480e-05_rb,6.7795e-05_rb,8.8273e-05_rb,1.0239e-04_rb,1.1087e-04_rb,1.1252e-04_rb, &
     &1.2626e-04_rb,1.7765e-04_rb,1.0235e-04_rb/)
      kao(:, 2, 2, 3) = (/ &
     &4.0229e-05_rb,9.8115e-05_rb,1.2639e-04_rb,1.4693e-04_rb,1.6171e-04_rb,1.7149e-04_rb, &
     &1.6078e-04_rb,2.0296e-04_rb,1.0513e-04_rb/)
      kao(:, 3, 2, 3) = (/ &
     &5.9019e-05_rb,1.3751e-04_rb,1.7468e-04_rb,2.0166e-04_rb,2.2223e-04_rb,2.3730e-04_rb, &
     &2.4647e-04_rb,2.3569e-04_rb,1.0639e-04_rb/)
      kao(:, 4, 2, 3) = (/ &
     &8.3741e-05_rb,1.8682e-04_rb,2.3419e-04_rb,2.6910e-04_rb,2.9309e-04_rb,3.0629e-04_rb, &
     &3.1256e-04_rb,3.4516e-04_rb,1.0584e-04_rb/)
      kao(:, 5, 2, 3) = (/ &
     &1.1534e-04_rb,2.4682e-04_rb,3.0598e-04_rb,3.4903e-04_rb,3.7559e-04_rb,3.8169e-04_rb, &
     &3.7221e-04_rb,3.8365e-04_rb,1.2786e-04_rb/)
      kao(:, 1, 3, 3) = (/ &
     &3.8574e-05_rb,7.5157e-05_rb,8.9382e-05_rb,9.8254e-05_rb,1.0233e-04_rb,1.0063e-04_rb, &
     &1.0056e-04_rb,1.0870e-04_rb,6.8446e-05_rb/)
      kao(:, 2, 3, 3) = (/ &
     &6.0142e-05_rb,1.1264e-04_rb,1.3244e-04_rb,1.4504e-04_rb,1.5242e-04_rb,1.5259e-04_rb, &
     &1.4806e-04_rb,1.3730e-04_rb,6.4347e-05_rb/)
      kao(:, 3, 3, 3) = (/ &
     &9.0381e-05_rb,1.6247e-04_rb,1.8926e-04_rb,2.0538e-04_rb,2.1407e-04_rb,2.1609e-04_rb, &
     &2.1113e-04_rb,1.9332e-04_rb,6.7739e-05_rb/)
      kao(:, 4, 3, 3) = (/ &
     &1.3106e-04_rb,2.2712e-04_rb,2.6180e-04_rb,2.8099e-04_rb,2.9092e-04_rb,2.9292e-04_rb, &
     &2.8375e-04_rb,2.7092e-04_rb,6.8866e-05_rb/)
      kao(:, 5, 3, 3) = (/ &
     &1.8403e-04_rb,3.0884e-04_rb,3.5100e-04_rb,3.7380e-04_rb,3.8510e-04_rb,3.8362e-04_rb, &
     &3.6289e-04_rb,3.2300e-04_rb,9.2758e-05_rb/)
      kao(:, 1, 4, 3) = (/ &
     &5.4705e-05_rb,8.6072e-05_rb,9.6209e-05_rb,1.0034e-04_rb,9.9953e-05_rb,9.5189e-05_rb, &
     &8.3161e-05_rb,7.9207e-05_rb,3.9423e-05_rb/)
      kao(:, 2, 4, 3) = (/ &
     &8.7751e-05_rb,1.3347e-04_rb,1.4769e-04_rb,1.5271e-04_rb,1.5230e-04_rb,1.4652e-04_rb, &
     &1.3316e-04_rb,1.0796e-04_rb,4.2482e-05_rb/)
      kao(:, 3, 4, 3) = (/ &
     &1.3536e-04_rb,1.9949e-04_rb,2.1780e-04_rb,2.2400e-04_rb,2.2194e-04_rb,2.1254e-04_rb, &
     &1.9475e-04_rb,1.6332e-04_rb,4.5138e-05_rb/)
      kao(:, 4, 4, 3) = (/ &
     &2.0116e-04_rb,2.8783e-04_rb,3.1041e-04_rb,3.1735e-04_rb,3.1196e-04_rb,2.9572e-04_rb, &
     &2.7052e-04_rb,2.3211e-04_rb,5.0851e-05_rb/)
      kao(:, 5, 4, 3) = (/ &
     &2.8895e-04_rb,4.0188e-04_rb,4.2924e-04_rb,4.3500e-04_rb,4.2412e-04_rb,3.9959e-04_rb, &
     &3.6227e-04_rb,3.0072e-04_rb,6.6760e-05_rb/)
      kao(:, 1, 5, 3) = (/ &
     &7.0137e-05_rb,9.4755e-05_rb,1.0063e-04_rb,1.0127e-04_rb,9.8061e-05_rb,9.0698e-05_rb, &
     &7.7439e-05_rb,5.9138e-05_rb,2.6683e-05_rb/)
      kao(:, 2, 5, 3) = (/ &
     &1.1533e-04_rb,1.5194e-04_rb,1.5997e-04_rb,1.5979e-04_rb,1.5340e-04_rb,1.4208e-04_rb, &
     &1.2486e-04_rb,9.3241e-05_rb,2.9592e-05_rb/)
      kao(:, 3, 5, 3) = (/ &
     &1.8276e-04_rb,2.3503e-04_rb,2.4432e-04_rb,2.4189e-04_rb,2.3109e-04_rb,2.1296e-04_rb, &
     &1.8693e-04_rb,1.4538e-04_rb,3.2790e-05_rb/)
      kao(:, 4, 5, 3) = (/ &
     &2.7903e-04_rb,3.5030e-04_rb,3.5953e-04_rb,3.5303e-04_rb,3.3542e-04_rb,3.0702e-04_rb, &
     &2.6606e-04_rb,2.0930e-04_rb,3.9551e-05_rb/)
      kao(:, 5, 5, 3) = (/ &
     &4.1070e-04_rb,5.0357e-04_rb,5.1208e-04_rb,4.9861e-04_rb,4.7008e-04_rb,4.2683e-04_rb, &
     &3.6701e-04_rb,2.8592e-04_rb,5.1146e-05_rb/)
      kao(:, 1, 6, 3) = (/ &
     &8.1368e-05_rb,9.8431e-05_rb,1.0007e-04_rb,9.7542e-05_rb,9.2017e-05_rb,8.3169e-05_rb, &
     &7.0090e-05_rb,4.8919e-05_rb,2.1156e-05_rb/)
      kao(:, 2, 6, 3) = (/ &
     &1.3706e-04_rb,1.6300e-04_rb,1.6503e-04_rb,1.5990e-04_rb,1.4992e-04_rb,1.3472e-04_rb, &
     &1.1399e-04_rb,8.3619e-05_rb,2.5058e-05_rb/)
      kao(:, 3, 6, 3) = (/ &
     &2.2338e-04_rb,2.6147e-04_rb,2.6146e-04_rb,2.5118e-04_rb,2.3339e-04_rb,2.0831e-04_rb, &
     &1.7568e-04_rb,1.3135e-04_rb,2.8387e-05_rb/)
      kao(:, 4, 6, 3) = (/ &
     &3.5127e-04_rb,4.0389e-04_rb,3.9905e-04_rb,3.7918e-04_rb,3.4971e-04_rb,3.1035e-04_rb, &
     &2.5984e-04_rb,1.9266e-04_rb,3.2799e-05_rb/)
      kao(:, 5, 6, 3) = (/ &
     &5.3129e-04_rb,6.0001e-04_rb,5.8622e-04_rb,5.5309e-04_rb,5.0577e-04_rb,4.4569e-04_rb, &
     &3.6955e-04_rb,2.7080e-04_rb,4.1496e-05_rb/)
      kao(:, 1, 7, 3) = (/ &
     &1.0007e-04_rb,1.0969e-04_rb,1.0696e-04_rb,1.0012e-04_rb,9.0860e-05_rb,7.9591e-05_rb, &
     &6.5272e-05_rb,4.4404e-05_rb,1.8023e-05_rb/)
      kao(:, 2, 7, 3) = (/ &
     &1.6864e-04_rb,1.8414e-04_rb,1.7903e-04_rb,1.6759e-04_rb,1.5212e-04_rb,1.3294e-04_rb, &
     &1.0932e-04_rb,7.7110e-05_rb,2.3161e-05_rb/)
      kao(:, 3, 7, 3) = (/ &
     &2.8238e-04_rb,3.0439e-04_rb,2.9352e-04_rb,2.7330e-04_rb,2.4706e-04_rb,2.1459e-04_rb, &
     &1.7451e-04_rb,1.2351e-04_rb,2.8855e-05_rb/)
      kao(:, 4, 7, 3) = (/ &
     &4.5659e-04_rb,4.8716e-04_rb,4.6539e-04_rb,4.2996e-04_rb,3.8506e-04_rb,3.3112e-04_rb, &
     &2.6671e-04_rb,1.8725e-04_rb,3.4923e-05_rb/)
      kao(:, 5, 7, 3) = (/ &
     &7.1027e-04_rb,7.4800e-04_rb,7.1021e-04_rb,6.5078e-04_rb,5.7777e-04_rb,4.9194e-04_rb, &
     &3.9297e-04_rb,2.7155e-04_rb,4.1495e-05_rb/)
      kao(:, 1, 8, 3) = (/ &
     &1.5946e-04_rb,1.5842e-04_rb,1.4746e-04_rb,1.3211e-04_rb,1.1546e-04_rb,9.6168e-05_rb, &
     &7.4324e-05_rb,4.8395e-05_rb,1.7989e-05_rb/)
      kao(:, 2, 8, 3) = (/ &
     &2.5770e-04_rb,2.5704e-04_rb,2.3941e-04_rb,2.1551e-04_rb,1.8837e-04_rb,1.5713e-04_rb, &
     &1.2214e-04_rb,8.1368e-05_rb,2.3968e-05_rb/)
      kao(:, 3, 8, 3) = (/ &
     &4.2472e-04_rb,4.2443e-04_rb,3.9713e-04_rb,3.5763e-04_rb,3.1122e-04_rb,2.5901e-04_rb, &
     &2.0043e-04_rb,1.3331e-04_rb,3.2118e-05_rb/)
      kao(:, 4, 8, 3) = (/ &
     &7.0597e-04_rb,6.9998e-04_rb,6.4911e-04_rb,5.8037e-04_rb,5.0147e-04_rb,4.1570e-04_rb, &
     &3.1990e-04_rb,2.0974e-04_rb,3.9608e-05_rb/)
      kao(:, 5, 8, 3) = (/ &
     &1.1264e-03_rb,1.1057e-03_rb,1.0236e-03_rb,9.1299e-04_rb,7.8475e-04_rb,6.4516e-04_rb, &
     &4.9155e-04_rb,3.1715e-04_rb,4.8541e-05_rb/)
      kao(:, 1, 9, 3) = (/ &
     &5.0227e-04_rb,4.5998e-04_rb,4.0915e-04_rb,3.5429e-04_rb,2.9541e-04_rb,2.3264e-04_rb, &
     &1.6588e-04_rb,9.6165e-05_rb,2.6165e-05_rb/)
      kao(:, 2, 9, 3) = (/ &
     &7.8676e-04_rb,7.2202e-04_rb,6.4423e-04_rb,5.5868e-04_rb,4.6624e-04_rb,3.6840e-04_rb, &
     &2.6387e-04_rb,1.5409e-04_rb,3.5446e-05_rb/)
      kao(:, 3, 9, 3) = (/ &
     &1.2789e-03_rb,1.1712e-03_rb,1.0462e-03_rb,9.0711e-04_rb,7.5900e-04_rb,5.9930e-04_rb, &
     &4.3043e-04_rb,2.5008e-04_rb,4.6358e-05_rb/)
      kao(:, 4, 9, 3) = (/ &
     &2.0840e-03_rb,1.9016e-03_rb,1.6924e-03_rb,1.4648e-03_rb,1.2215e-03_rb,9.6366e-04_rb, &
     &6.9084e-04_rb,4.0072e-04_rb,6.1001e-05_rb/)
      kao(:, 5, 9, 3) = (/ &
     &3.3367e-03_rb,3.0441e-03_rb,2.7105e-03_rb,2.3462e-03_rb,1.9558e-03_rb,1.5435e-03_rb, &
     &1.1056e-03_rb,6.3552e-04_rb,7.9035e-05_rb/)
      kao(:, 1,10, 3) = (/ &
     &1.9385e-03_rb,1.7157e-03_rb,1.4863e-03_rb,1.2534e-03_rb,1.0193e-03_rb,7.7905e-04_rb, &
     &5.3570e-04_rb,2.8621e-04_rb,4.8614e-05_rb/)
      kao(:, 2,10, 3) = (/ &
     &2.9395e-03_rb,2.6008e-03_rb,2.2566e-03_rb,1.9073e-03_rb,1.5534e-03_rb,1.1938e-03_rb, &
     &8.2332e-04_rb,4.4169e-04_rb,7.1291e-05_rb/)
      kao(:, 3,10, 3) = (/ &
     &4.6962e-03_rb,4.1509e-03_rb,3.6037e-03_rb,3.0461e-03_rb,2.4817e-03_rb,1.9064e-03_rb, &
     &1.3190e-03_rb,7.0872e-04_rb,9.5825e-05_rb/)
      kao(:, 4,10, 3) = (/ &
     &7.5852e-03_rb,6.6994e-03_rb,5.8186e-03_rb,4.9195e-03_rb,4.0087e-03_rb,3.0812e-03_rb, &
     &2.1247e-03_rb,1.1368e-03_rb,1.1801e-04_rb/)
      kao(:, 5,10, 3) = (/ &
     &1.2000e-02_rb,1.0601e-02_rb,9.2089e-03_rb,7.7807e-03_rb,6.3408e-03_rb,4.8692e-03_rb, &
     &3.3607e-03_rb,1.8001e-03_rb,1.5301e-04_rb/)
      kao(:, 1,11, 3) = (/ &
     &3.1614e-03_rb,2.7883e-03_rb,2.4150e-03_rb,2.0359e-03_rb,1.6451e-03_rb,1.2526e-03_rb, &
     &8.5416e-04_rb,4.4797e-04_rb,5.8064e-05_rb/)
      kao(:, 2,11, 3) = (/ &
     &4.7730e-03_rb,4.2097e-03_rb,3.6498e-03_rb,3.0671e-03_rb,2.4809e-03_rb,1.8948e-03_rb, &
     &1.2977e-03_rb,6.8686e-04_rb,8.3754e-05_rb/)
      kao(:, 3,11, 3) = (/ &
     &7.6614e-03_rb,6.7521e-03_rb,5.8435e-03_rb,4.9093e-03_rb,3.9758e-03_rb,3.0351e-03_rb, &
     &2.0807e-03_rb,1.0948e-03_rb,1.2253e-04_rb/)
      kao(:, 4,11, 3) = (/ &
     &1.2246e-02_rb,1.0785e-02_rb,9.3301e-03_rb,7.8482e-03_rb,6.3562e-03_rb,4.8536e-03_rb, &
     &3.3260e-03_rb,1.7481e-03_rb,1.5741e-04_rb/)
      kao(:, 5,11, 3) = (/ &
     &1.9021e-02_rb,1.6750e-02_rb,1.4500e-02_rb,1.2206e-02_rb,9.8921e-03_rb,7.5567e-03_rb, &
     &5.1767e-03_rb,2.7251e-03_rb,2.0052e-04_rb/)
      kao(:, 1,12, 3) = (/ &
     &3.8826e-03_rb,3.4185e-03_rb,2.9574e-03_rb,2.4900e-03_rb,2.0123e-03_rb,1.5298e-03_rb, &
     &1.0368e-03_rb,5.3904e-04_rb,6.3688e-05_rb/)
      kao(:, 2,12, 3) = (/ &
     &5.8362e-03_rb,5.1381e-03_rb,4.4457e-03_rb,3.7372e-03_rb,3.0206e-03_rb,2.2947e-03_rb, &
     &1.5681e-03_rb,8.2298e-04_rb,8.9290e-05_rb/)
      kao(:, 3,12, 3) = (/ &
     &9.2251e-03_rb,8.1210e-03_rb,7.0237e-03_rb,5.8967e-03_rb,4.7688e-03_rb,3.6322e-03_rb, &
     &2.4807e-03_rb,1.3032e-03_rb,1.3416e-04_rb/)
      kao(:, 4,12, 3) = (/ &
     &1.4709e-02_rb,1.2946e-02_rb,1.1191e-02_rb,9.4045e-03_rb,7.5956e-03_rb,5.7870e-03_rb, &
     &3.9575e-03_rb,2.0727e-03_rb,1.8001e-04_rb/)
      kao(:, 5,12, 3) = (/ &
     &2.2619e-02_rb,1.9908e-02_rb,1.7216e-02_rb,1.4475e-02_rb,1.1703e-02_rb,8.9157e-03_rb, &
     &6.0792e-03_rb,3.1754e-03_rb,2.2086e-04_rb/)
      kao(:, 1,13, 3) = (/ &
     &3.8309e-03_rb,3.3731e-03_rb,2.9187e-03_rb,2.4542e-03_rb,1.9844e-03_rb,1.5107e-03_rb, &
     &1.0279e-03_rb,5.3642e-04_rb,6.4942e-05_rb/)
      kao(:, 2,13, 3) = (/ &
     &5.8347e-03_rb,5.1376e-03_rb,4.4461e-03_rb,3.7352e-03_rb,3.0197e-03_rb,2.2950e-03_rb, &
     &1.5588e-03_rb,8.1629e-04_rb,9.2767e-05_rb/)
      kao(:, 3,13, 3) = (/ &
     &9.2194e-03_rb,8.1162e-03_rb,7.0173e-03_rb,5.8930e-03_rb,4.7598e-03_rb,3.6175e-03_rb, &
     &2.4591e-03_rb,1.2857e-03_rb,1.3288e-04_rb/)
      kao(:, 4,13, 3) = (/ &
     &1.4386e-02_rb,1.2666e-02_rb,1.0949e-02_rb,9.1979e-03_rb,7.4294e-03_rb,5.6460e-03_rb, &
     &3.8493e-03_rb,2.0075e-03_rb,1.7458e-04_rb/)
      kao(:, 5,13, 3) = (/ &
     &2.1887e-02_rb,1.9271e-02_rb,1.6664e-02_rb,1.4007e-02_rb,1.1332e-02_rb,8.6132e-03_rb, &
     &5.8543e-03_rb,3.0399e-03_rb,2.1866e-04_rb/)
      kao(:, 1, 1, 4) = (/ &
     &1.2767e-04_rb,2.6436e-04_rb,3.0512e-04_rb,3.1799e-04_rb,3.1958e-04_rb,3.3191e-04_rb, &
     &3.9104e-04_rb,5.0710e-04_rb,1.7855e-04_rb/)
      kao(:, 2, 1, 4) = (/ &
     &1.7738e-04_rb,3.5053e-04_rb,3.9839e-04_rb,4.1229e-04_rb,4.0492e-04_rb,3.8983e-04_rb, &
     &4.1500e-04_rb,5.9614e-04_rb,1.6036e-04_rb/)
      kao(:, 3, 1, 4) = (/ &
     &2.3686e-04_rb,4.4965e-04_rb,5.0544e-04_rb,5.1877e-04_rb,5.0600e-04_rb,4.7740e-04_rb, &
     &4.5052e-04_rb,5.7236e-04_rb,1.8787e-04_rb/)
      kao(:, 4, 1, 4) = (/ &
     &3.0650e-04_rb,5.6569e-04_rb,6.3165e-04_rb,6.4530e-04_rb,6.3512e-04_rb,6.1237e-04_rb, &
     &5.6656e-04_rb,5.2891e-04_rb,2.7469e-04_rb/)
      kao(:, 5, 1, 4) = (/ &
     &3.8512e-04_rb,6.9752e-04_rb,7.7525e-04_rb,7.9245e-04_rb,7.8883e-04_rb,7.7085e-04_rb, &
     &7.4060e-04_rb,6.7719e-04_rb,3.6282e-04_rb/)
      kao(:, 1, 2, 4) = (/ &
     &1.5390e-04_rb,2.7494e-04_rb,3.1261e-04_rb,3.2023e-04_rb,3.1151e-04_rb,2.9547e-04_rb, &
     &3.1497e-04_rb,3.9039e-04_rb,1.0989e-04_rb/)
      kao(:, 2, 2, 4) = (/ &
     &2.1714e-04_rb,3.7453e-04_rb,4.1638e-04_rb,4.2413e-04_rb,4.1007e-04_rb,3.7857e-04_rb, &
     &3.5354e-04_rb,4.2401e-04_rb,1.1763e-04_rb/)
      kao(:, 3, 2, 4) = (/ &
     &2.9352e-04_rb,4.8938e-04_rb,5.3784e-04_rb,5.4431e-04_rb,5.2257e-04_rb,4.7749e-04_rb, &
     &4.2355e-04_rb,4.2211e-04_rb,1.4157e-04_rb/)
      kao(:, 4, 2, 4) = (/ &
     &3.8348e-04_rb,6.2214e-04_rb,6.8003e-04_rb,6.8421e-04_rb,6.5590e-04_rb,6.0605e-04_rb, &
     &5.3467e-04_rb,4.5064e-04_rb,2.0612e-04_rb/)
      kao(:, 5, 2, 4) = (/ &
     &4.8704e-04_rb,7.7618e-04_rb,8.4540e-04_rb,8.4784e-04_rb,8.1484e-04_rb,7.6245e-04_rb, &
     &6.8894e-04_rb,5.8453e-04_rb,2.7190e-04_rb/)
      kao(:, 1, 3, 4) = (/ &
     &2.3071e-04_rb,3.3499e-04_rb,3.6845e-04_rb,3.7270e-04_rb,3.5547e-04_rb,3.2203e-04_rb, &
     &2.7656e-04_rb,2.6506e-04_rb,6.2779e-05_rb/)
      kao(:, 2, 3, 4) = (/ &
     &3.3368e-04_rb,4.7302e-04_rb,5.1131e-04_rb,5.0923e-04_rb,4.8101e-04_rb,4.3420e-04_rb, &
     &3.6192e-04_rb,3.1963e-04_rb,7.8022e-05_rb/)
      kao(:, 3, 3, 4) = (/ &
     &4.6167e-04_rb,6.3817e-04_rb,6.7863e-04_rb,6.7148e-04_rb,6.3202e-04_rb,5.6610e-04_rb, &
     &4.7208e-04_rb,3.5841e-04_rb,1.0032e-04_rb/)
      kao(:, 4, 3, 4) = (/ &
     &6.1421e-04_rb,8.2728e-04_rb,8.7092e-04_rb,8.6012e-04_rb,8.0666e-04_rb,7.1818e-04_rb, &
     &5.9937e-04_rb,4.3811e-04_rb,1.4466e-04_rb/)
      kao(:, 5, 3, 4) = (/ &
     &7.9318e-04_rb,1.0499e-03_rb,1.1001e-03_rb,1.0825e-03_rb,1.0111e-03_rb,9.0157e-04_rb, &
     &7.5964e-04_rb,5.7512e-04_rb,1.9432e-04_rb/)
      kao(:, 1, 4, 4) = (/ &
     &3.4200e-04_rb,4.1990e-04_rb,4.4259e-04_rb,4.3661e-04_rb,4.1163e-04_rb,3.6760e-04_rb, &
     &3.0371e-04_rb,2.3149e-04_rb,4.6307e-05_rb/)
      kao(:, 2, 4, 4) = (/ &
     &5.0920e-04_rb,6.1539e-04_rb,6.3721e-04_rb,6.1994e-04_rb,5.7603e-04_rb,5.0827e-04_rb, &
     &4.1402e-04_rb,2.9065e-04_rb,5.1907e-05_rb/)
      kao(:, 3, 4, 4) = (/ &
     &7.2424e-04_rb,8.6063e-04_rb,8.7699e-04_rb,8.4266e-04_rb,7.7721e-04_rb,6.8304e-04_rb, &
     &5.5495e-04_rb,3.8326e-04_rb,6.9160e-05_rb/)
      kao(:, 4, 4, 4) = (/ &
     &9.8776e-04_rb,1.1516e-03_rb,1.1582e-03_rb,1.1044e-03_rb,1.0138e-03_rb,8.8821e-04_rb, &
     &7.1753e-04_rb,4.9169e-04_rb,9.8233e-05_rb/)
      kao(:, 5, 4, 4) = (/ &
     &1.2975e-03_rb,1.4918e-03_rb,1.4869e-03_rb,1.4114e-03_rb,1.2930e-03_rb,1.1277e-03_rb, &
     &9.0840e-04_rb,6.3197e-04_rb,1.3450e-04_rb/)
      kao(:, 1, 5, 4) = (/ &
     &4.6076e-04_rb,5.0474e-04_rb,5.1144e-04_rb,4.9140e-04_rb,4.5232e-04_rb,3.9779e-04_rb, &
     &3.2321e-04_rb,2.2336e-04_rb,3.4201e-05_rb/)
      kao(:, 2, 5, 4) = (/ &
     &7.0989e-04_rb,7.6999e-04_rb,7.6963e-04_rb,7.2952e-04_rb,6.6336e-04_rb,5.7413e-04_rb, &
     &4.5930e-04_rb,3.0843e-04_rb,4.2292e-05_rb/)
      kao(:, 3, 5, 4) = (/ &
     &1.0386e-03_rb,1.1133e-03_rb,1.0985e-03_rb,1.0276e-03_rb,9.2270e-04_rb,7.9078e-04_rb, &
     &6.2825e-04_rb,4.1898e-04_rb,5.3586e-05_rb/)
      kao(:, 4, 5, 4) = (/ &
     &1.4541e-03_rb,1.5404e-03_rb,1.4988e-03_rb,1.3877e-03_rb,1.2375e-03_rb,1.0544e-03_rb, &
     &8.3469e-04_rb,5.5179e-04_rb,6.8395e-05_rb/)
      kao(:, 5, 5, 4) = (/ &
     &1.9528e-03_rb,2.0463e-03_rb,1.9719e-03_rb,1.8145e-03_rb,1.6102e-03_rb,1.3677e-03_rb, &
     &1.0776e-03_rb,7.0877e-04_rb,9.0444e-05_rb/)
      kao(:, 1, 6, 4) = (/ &
     &5.6492e-04_rb,5.7565e-04_rb,5.5931e-04_rb,5.2434e-04_rb,4.7320e-04_rb,4.0777e-04_rb, &
     &3.2512e-04_rb,2.1641e-04_rb,2.6966e-05_rb/)
      kao(:, 2, 6, 4) = (/ &
     &9.0441e-04_rb,9.1447e-04_rb,8.8349e-04_rb,8.2170e-04_rb,7.3208e-04_rb,6.2190e-04_rb, &
     &4.8768e-04_rb,3.1817e-04_rb,3.4982e-05_rb/)
      kao(:, 3, 6, 4) = (/ &
     &1.3664e-03_rb,1.3688e-03_rb,1.3074e-03_rb,1.2015e-03_rb,1.0584e-03_rb,8.8601e-04_rb, &
     &6.8532e-04_rb,4.4190e-04_rb,4.5831e-05_rb/)
      kao(:, 4, 6, 4) = (/ &
     &1.9708e-03_rb,1.9570e-03_rb,1.8529e-03_rb,1.6859e-03_rb,1.4692e-03_rb,1.2199e-03_rb, &
     &9.3583e-04_rb,6.0020e-04_rb,5.8072e-05_rb/)
      kao(:, 5, 6, 4) = (/ &
     &2.7158e-03_rb,2.6789e-03_rb,2.5132e-03_rb,2.2684e-03_rb,1.9663e-03_rb,1.6232e-03_rb, &
     &1.2393e-03_rb,7.8815e-04_rb,7.1453e-05_rb/)
      kao(:, 1, 7, 4) = (/ &
     &7.1612e-04_rb,6.9080e-04_rb,6.4557e-04_rb,5.8741e-04_rb,5.1690e-04_rb,4.3484e-04_rb, &
     &3.3811e-04_rb,2.1958e-04_rb,2.5385e-05_rb/)
      kao(:, 2, 7, 4) = (/ &
     &1.1940e-03_rb,1.1449e-03_rb,1.0668e-03_rb,9.6669e-04_rb,8.4588e-04_rb,7.0528e-04_rb, &
     &5.3962e-04_rb,3.4196e-04_rb,3.2650e-05_rb/)
      kao(:, 3, 7, 4) = (/ &
     &1.8743e-03_rb,1.7868e-03_rb,1.6562e-03_rb,1.4906e-03_rb,1.2906e-03_rb,1.0612e-03_rb, &
     &7.9912e-04_rb,4.9523e-04_rb,4.3298e-05_rb/)
      kao(:, 4, 7, 4) = (/ &
     &2.7760e-03_rb,2.6274e-03_rb,2.4197e-03_rb,2.1609e-03_rb,1.8570e-03_rb,1.5098e-03_rb, &
     &1.1229e-03_rb,6.8654e-04_rb,5.4268e-05_rb/)
      kao(:, 5, 7, 4) = (/ &
     &3.9210e-03_rb,3.6980e-03_rb,3.3854e-03_rb,3.0037e-03_rb,2.5627e-03_rb,2.0710e-03_rb, &
     &1.5295e-03_rb,9.2840e-04_rb,6.7940e-05_rb/)
      kao(:, 1, 8, 4) = (/ &
     &1.0670e-03_rb,9.8692e-04_rb,8.9134e-04_rb,7.8593e-04_rb,6.7073e-04_rb,5.4467e-04_rb, &
     &4.0756e-04_rb,2.5210e-04_rb,2.8359e-05_rb/)
      kao(:, 2, 8, 4) = (/ &
     &1.8582e-03_rb,1.7102e-03_rb,1.5414e-03_rb,1.3557e-03_rb,1.1545e-03_rb,9.3501e-04_rb, &
     &6.9502e-04_rb,4.2320e-04_rb,3.7233e-05_rb/)
      kao(:, 3, 8, 4) = (/ &
     &3.0392e-03_rb,2.7815e-03_rb,2.4987e-03_rb,2.1910e-03_rb,1.8592e-03_rb,1.4974e-03_rb, &
     &1.1016e-03_rb,6.5354e-04_rb,4.6926e-05_rb/)
      kao(:, 4, 8, 4) = (/ &
     &4.6351e-03_rb,4.2251e-03_rb,3.7893e-03_rb,3.3101e-03_rb,2.7928e-03_rb,2.2311e-03_rb, &
     &1.6189e-03_rb,9.4084e-04_rb,6.1285e-05_rb/)
      kao(:, 5, 8, 4) = (/ &
     &6.7053e-03_rb,6.1011e-03_rb,5.4490e-03_rb,4.7367e-03_rb,3.9743e-03_rb,3.1531e-03_rb, &
     &2.2726e-03_rb,1.3072e-03_rb,7.4486e-05_rb/)
      kao(:, 1, 9, 4) = (/ &
     &3.0079e-03_rb,2.6671e-03_rb,2.3362e-03_rb,1.9921e-03_rb,1.6375e-03_rb,1.2739e-03_rb, &
     &8.9947e-04_rb,5.0478e-04_rb,4.8310e-05_rb/)
      kao(:, 2, 9, 4) = (/ &
     &5.3604e-03_rb,4.7462e-03_rb,4.1492e-03_rb,3.5341e-03_rb,2.9054e-03_rb,2.2583e-03_rb, &
     &1.5943e-03_rb,8.9078e-04_rb,6.1843e-05_rb/)
      kao(:, 3, 9, 4) = (/ &
     &9.0324e-03_rb,7.9882e-03_rb,6.9714e-03_rb,5.9320e-03_rb,4.8700e-03_rb,3.7864e-03_rb, &
     &2.6693e-03_rb,1.4872e-03_rb,8.4369e-05_rb/)
      kao(:, 4, 9, 4) = (/ &
     &1.4329e-02_rb,1.2658e-02_rb,1.1032e-02_rb,9.3748e-03_rb,7.6880e-03_rb,5.9624e-03_rb, &
     &4.1846e-03_rb,2.3075e-03_rb,1.0247e-04_rb/)
      kao(:, 5, 9, 4) = (/ &
     &2.1261e-02_rb,1.8755e-02_rb,1.6325e-02_rb,1.3858e-02_rb,1.1339e-02_rb,8.7669e-03_rb, &
     &6.1142e-03_rb,3.3320e-03_rb,1.2163e-04_rb/)
      kao(:, 1,10, 4) = (/ &
     &1.0911e-02_rb,9.5635e-03_rb,8.2330e-03_rb,6.9092e-03_rb,5.5802e-03_rb,4.2396e-03_rb, &
     &2.8819e-03_rb,1.5069e-03_rb,1.0164e-04_rb/)
      kao(:, 2,10, 4) = (/ &
     &1.8579e-02_rb,1.6289e-02_rb,1.4021e-02_rb,1.1757e-02_rb,9.4958e-03_rb,7.2115e-03_rb, &
     &4.9034e-03_rb,2.5692e-03_rb,1.2765e-04_rb/)
      kao(:, 3,10, 4) = (/ &
     &3.1018e-02_rb,2.7193e-02_rb,2.3385e-02_rb,1.9601e-02_rb,1.5823e-02_rb,1.2032e-02_rb, &
     &8.1956e-03_rb,4.3045e-03_rb,1.7856e-04_rb/)
      kao(:, 4,10, 4) = (/ &
     &5.0111e-02_rb,4.3919e-02_rb,3.7756e-02_rb,3.1628e-02_rb,2.5516e-02_rb,1.9374e-02_rb, &
     &1.3194e-02_rb,6.9219e-03_rb,2.2724e-04_rb/)
      kao(:, 5,10, 4) = (/ &
     &7.6376e-02_rb,6.6924e-02_rb,5.7507e-02_rb,4.8151e-02_rb,3.8816e-02_rb,2.9439e-02_rb, &
     &2.0003e-02_rb,1.0433e-02_rb,2.6085e-04_rb/)
      kao(:, 1,11, 4) = (/ &
     &1.8971e-02_rb,1.6611e-02_rb,1.4262e-02_rb,1.1923e-02_rb,9.5992e-03_rb,7.2680e-03_rb, &
     &4.9113e-03_rb,2.5257e-03_rb,1.6715e-04_rb/)
      kao(:, 2,11, 4) = (/ &
     &3.0950e-02_rb,2.7103e-02_rb,2.3271e-02_rb,1.9463e-02_rb,1.5664e-02_rb,1.1847e-02_rb, &
     &7.9981e-03_rb,4.1199e-03_rb,2.1489e-04_rb/)
      kao(:, 3,11, 4) = (/ &
     &4.9685e-02_rb,4.3517e-02_rb,3.7369e-02_rb,3.1256e-02_rb,2.5144e-02_rb,1.9021e-02_rb, &
     &1.2853e-02_rb,6.6396e-03_rb,2.5990e-04_rb/)
      kao(:, 4,11, 4) = (/ &
     &7.7374e-02_rb,6.7750e-02_rb,5.8163e-02_rb,4.8608e-02_rb,3.9067e-02_rb,2.9542e-02_rb, &
     &1.9962e-02_rb,1.0311e-02_rb,3.2116e-04_rb/)
      kao(:, 5,11, 4) = (/ &
     &1.1501e-01_rb,1.0072e-01_rb,8.6431e-02_rb,7.2217e-02_rb,5.8056e-02_rb,4.3898e-02_rb, &
     &2.9647e-02_rb,1.5264e-02_rb,3.7485e-04_rb/)
      kao(:, 1,12, 4) = (/ &
     &2.3401e-02_rb,2.0486e-02_rb,1.7582e-02_rb,1.4686e-02_rb,1.1806e-02_rb,8.9258e-03_rb, &
     &6.0297e-03_rb,3.0936e-03_rb,1.8693e-04_rb/)
      kao(:, 2,12, 4) = (/ &
     &3.8133e-02_rb,3.3387e-02_rb,2.8657e-02_rb,2.3944e-02_rb,1.9250e-02_rb,1.4544e-02_rb, &
     &9.8021e-03_rb,5.0235e-03_rb,2.4530e-04_rb/)
      kao(:, 3,12, 4) = (/ &
     &6.0110e-02_rb,5.2620e-02_rb,4.5149e-02_rb,3.7710e-02_rb,3.0268e-02_rb,2.2852e-02_rb, &
     &1.5390e-02_rb,7.8945e-03_rb,2.8864e-04_rb/)
      kao(:, 4,12, 4) = (/ &
     &9.0890e-02_rb,7.9571e-02_rb,6.8303e-02_rb,5.7049e-02_rb,4.5842e-02_rb,3.4594e-02_rb, &
     &2.3289e-02_rb,1.1923e-02_rb,3.6228e-04_rb/)
      kao(:, 5,12, 4) = (/ &
     &1.3100e-01_rb,1.1462e-01_rb,9.8322e-02_rb,8.2088e-02_rb,6.5959e-02_rb,4.9780e-02_rb, &
     &3.3534e-02_rb,1.7184e-02_rb,4.3158e-04_rb/)
      kao(:, 1,13, 4) = (/ &
     &2.4303e-02_rb,2.1262e-02_rb,1.8230e-02_rb,1.5211e-02_rb,1.2213e-02_rb,9.2155e-03_rb, &
     &6.2046e-03_rb,3.1714e-03_rb,1.9233e-04_rb/)
      kao(:, 2,13, 4) = (/ &
     &3.8296e-02_rb,3.3521e-02_rb,2.8762e-02_rb,2.4031e-02_rb,1.9310e-02_rb,1.4568e-02_rb, &
     &9.8072e-03_rb,5.0047e-03_rb,2.4842e-04_rb/)
      kao(:, 3,13, 4) = (/ &
     &5.8379e-02_rb,5.1065e-02_rb,4.3832e-02_rb,3.6631e-02_rb,2.9434e-02_rb,2.2205e-02_rb, &
     &1.4940e-02_rb,7.6384e-03_rb,2.8734e-04_rb/)
      kao(:, 4,13, 4) = (/ &
     &8.5558e-02_rb,7.4841e-02_rb,6.4227e-02_rb,5.3638e-02_rb,4.3082e-02_rb,3.2499e-02_rb, &
     &2.1863e-02_rb,1.1189e-02_rb,3.6167e-04_rb/)
      kao(:, 5,13, 4) = (/ &
     &1.2105e-01_rb,1.0601e-01_rb,9.0943e-02_rb,7.5945e-02_rb,6.0995e-02_rb,4.5990e-02_rb, &
     &3.0953e-02_rb,1.5858e-02_rb,4.3014e-04_rb/)
      kao(:, 1, 1, 5) = (/ &
     &5.2897e-04_rb,6.3863e-04_rb,6.7716e-04_rb,6.8592e-04_rb,6.6781e-04_rb,6.0937e-04_rb, &
     &5.2069e-04_rb,5.5022e-04_rb,1.8160e-04_rb/)
      kao(:, 2, 1, 5) = (/ &
     &6.5371e-04_rb,8.0837e-04_rb,8.6369e-04_rb,8.8295e-04_rb,8.7230e-04_rb,8.3067e-04_rb, &
     &7.4453e-04_rb,6.6477e-04_rb,2.9044e-04_rb/)
      kao(:, 3, 1, 5) = (/ &
     &7.9309e-04_rb,1.0041e-03_rb,1.0806e-03_rb,1.1131e-03_rb,1.1135e-03_rb,1.0810e-03_rb, &
     &1.0060e-03_rb,8.6363e-04_rb,4.3507e-04_rb/)
      kao(:, 4, 1, 5) = (/ &
     &9.5515e-04_rb,1.2330e-03_rb,1.3320e-03_rb,1.3798e-03_rb,1.3866e-03_rb,1.3599e-03_rb, &
     &1.2978e-03_rb,1.1703e-03_rb,5.4823e-04_rb/)
      kao(:, 5, 1, 5) = (/ &
     &1.1370e-03_rb,1.4922e-03_rb,1.6168e-03_rb,1.6809e-03_rb,1.6969e-03_rb,1.6741e-03_rb, &
     &1.6142e-03_rb,1.5306e-03_rb,7.1233e-04_rb/)
      kao(:, 1, 2, 5) = (/ &
     &6.4530e-04_rb,7.2673e-04_rb,7.3348e-04_rb,7.1463e-04_rb,6.7564e-04_rb,6.1110e-04_rb, &
     &5.0123e-04_rb,4.4222e-04_rb,1.3962e-04_rb/)
      kao(:, 2, 2, 5) = (/ &
     &8.0957e-04_rb,9.2607e-04_rb,9.4414e-04_rb,9.2480e-04_rb,8.8433e-04_rb,8.1497e-04_rb, &
     &7.0697e-04_rb,5.2414e-04_rb,2.1889e-04_rb/)
      kao(:, 3, 2, 5) = (/ &
     &9.9558e-04_rb,1.1596e-03_rb,1.1892e-03_rb,1.1731e-03_rb,1.1295e-03_rb,1.0550e-03_rb, &
     &9.3776e-04_rb,7.6366e-04_rb,3.1779e-04_rb/)
      kao(:, 4, 2, 5) = (/ &
     &1.2093e-03_rb,1.4336e-03_rb,1.4751e-03_rb,1.4631e-03_rb,1.4149e-03_rb,1.3281e-03_rb, &
     &1.2039e-03_rb,1.0264e-03_rb,4.0743e-04_rb/)
      kao(:, 5, 2, 5) = (/ &
     &1.4548e-03_rb,1.7501e-03_rb,1.8041e-03_rb,1.7977e-03_rb,1.7416e-03_rb,1.6424e-03_rb, &
     &1.5037e-03_rb,1.3244e-03_rb,5.3155e-04_rb/)
      kao(:, 1, 3, 5) = (/ &
     &1.0073e-03_rb,1.0305e-03_rb,9.9170e-04_rb,9.1685e-04_rb,8.2334e-04_rb,7.1279e-04_rb, &
     &5.7365e-04_rb,3.9412e-04_rb,1.0025e-04_rb/)
      kao(:, 2, 3, 5) = (/ &
     &1.2880e-03_rb,1.3316e-03_rb,1.2867e-03_rb,1.1967e-03_rb,1.0809e-03_rb,9.4443e-04_rb, &
     &7.8179e-04_rb,5.3700e-04_rb,1.5384e-04_rb/)
      kao(:, 3, 3, 5) = (/ &
     &1.6107e-03_rb,1.6903e-03_rb,1.6390e-03_rb,1.5300e-03_rb,1.3886e-03_rb,1.2242e-03_rb, &
     &1.0235e-03_rb,7.5425e-04_rb,2.1640e-04_rb/)
      kao(:, 4, 3, 5) = (/ &
     &1.9797e-03_rb,2.1107e-03_rb,2.0529e-03_rb,1.9209e-03_rb,1.7527e-03_rb,1.5541e-03_rb, &
     &1.3094e-03_rb,9.9288e-04_rb,2.8803e-04_rb/)
      kao(:, 5, 3, 5) = (/ &
     &2.4113e-03_rb,2.6021e-03_rb,2.5366e-03_rb,2.3815e-03_rb,2.1822e-03_rb,1.9385e-03_rb, &
     &1.6375e-03_rb,1.2689e-03_rb,3.7641e-04_rb/)
      kao(:, 1, 4, 5) = (/ &
     &1.5851e-03_rb,1.5181e-03_rb,1.4032e-03_rb,1.2606e-03_rb,1.0883e-03_rb,8.9739e-04_rb, &
     &6.8560e-04_rb,4.2854e-04_rb,6.6391e-05_rb/)
      kao(:, 2, 4, 5) = (/ &
     &2.0501e-03_rb,1.9754e-03_rb,1.8387e-03_rb,1.6565e-03_rb,1.4386e-03_rb,1.1914e-03_rb, &
     &9.2359e-04_rb,6.1319e-04_rb,1.0381e-04_rb/)
      kao(:, 3, 4, 5) = (/ &
     &2.6137e-03_rb,2.5384e-03_rb,2.3763e-03_rb,2.1489e-03_rb,1.8706e-03_rb,1.5557e-03_rb, &
     &1.2160e-03_rb,8.2392e-04_rb,1.4611e-04_rb/)
      kao(:, 4, 4, 5) = (/ &
     &3.2665e-03_rb,3.2074e-03_rb,3.0134e-03_rb,2.7316e-03_rb,2.3866e-03_rb,1.9939e-03_rb, &
     &1.5682e-03_rb,1.0781e-03_rb,1.9821e-04_rb/)
      kao(:, 5, 4, 5) = (/ &
     &4.0137e-03_rb,3.9873e-03_rb,3.7629e-03_rb,3.4158e-03_rb,2.9921e-03_rb,2.5128e-03_rb, &
     &1.9870e-03_rb,1.3733e-03_rb,2.6370e-04_rb/)
      kao(:, 1, 5, 5) = (/ &
     &2.2692e-03_rb,2.1006e-03_rb,1.8869e-03_rb,1.6579e-03_rb,1.4070e-03_rb,1.1280e-03_rb, &
     &8.2786e-04_rb,4.9332e-04_rb,4.7300e-05_rb/)
      kao(:, 2, 5, 5) = (/ &
     &2.9951e-03_rb,2.7795e-03_rb,2.5097e-03_rb,2.2151e-03_rb,1.8839e-03_rb,1.5178e-03_rb, &
     &1.1209e-03_rb,6.8743e-04_rb,6.4441e-05_rb/)
      kao(:, 3, 5, 5) = (/ &
     &3.8783e-03_rb,3.6150e-03_rb,3.2821e-03_rb,2.9041e-03_rb,2.4789e-03_rb,2.0050e-03_rb, &
     &1.4894e-03_rb,9.2698e-04_rb,9.4008e-05_rb/)
      kao(:, 4, 5, 5) = (/ &
     &4.9258e-03_rb,4.6186e-03_rb,4.2220e-03_rb,3.7446e-03_rb,3.2015e-03_rb,2.5968e-03_rb, &
     &1.9395e-03_rb,1.2217e-03_rb,1.3185e-04_rb/)
      kao(:, 5, 5, 5) = (/ &
     &6.1149e-03_rb,5.7761e-03_rb,5.3147e-03_rb,4.7312e-03_rb,4.0567e-03_rb,3.3012e-03_rb, &
     &2.4775e-03_rb,1.5747e-03_rb,1.8186e-04_rb/)
      kao(:, 1, 6, 5) = (/ &
     &2.9777e-03_rb,2.7020e-03_rb,2.3926e-03_rb,2.0653e-03_rb,1.7240e-03_rb,1.3628e-03_rb, &
     &9.7585e-04_rb,5.5769e-04_rb,3.7018e-05_rb/)
      kao(:, 2, 6, 5) = (/ &
     &4.0405e-03_rb,3.6698e-03_rb,3.2527e-03_rb,2.8150e-03_rb,2.3582e-03_rb,1.8689e-03_rb, &
     &1.3459e-03_rb,7.7825e-04_rb,5.0931e-05_rb/)
      kao(:, 3, 6, 5) = (/ &
     &5.2985e-03_rb,4.8259e-03_rb,4.2980e-03_rb,3.7351e-03_rb,3.1354e-03_rb,2.4956e-03_rb, &
     &1.8046e-03_rb,1.0570e-03_rb,6.8165e-05_rb/)
      kao(:, 4, 6, 5) = (/ &
     &6.8330e-03_rb,6.2458e-03_rb,5.5922e-03_rb,4.8800e-03_rb,4.1117e-03_rb,3.2799e-03_rb, &
     &2.3798e-03_rb,1.4059e-03_rb,8.9401e-05_rb/)
      kao(:, 5, 6, 5) = (/ &
     &8.6003e-03_rb,7.8905e-03_rb,7.1103e-03_rb,6.2229e-03_rb,5.2575e-03_rb,4.2076e-03_rb, &
     &3.0687e-03_rb,1.8306e-03_rb,1.2028e-04_rb/)
      kao(:, 1, 7, 5) = (/ &
     &4.0427e-03_rb,3.6171e-03_rb,3.1732e-03_rb,2.7085e-03_rb,2.2305e-03_rb,1.7374e-03_rb, &
     &1.2245e-03_rb,6.7631e-04_rb,3.7016e-05_rb/)
      kao(:, 2, 7, 5) = (/ &
     &5.6556e-03_rb,5.0622e-03_rb,4.4391e-03_rb,3.7915e-03_rb,3.1262e-03_rb,2.4381e-03_rb, &
     &1.7215e-03_rb,9.5894e-04_rb,4.8862e-05_rb/)
      kao(:, 3, 7, 5) = (/ &
     &7.5823e-03_rb,6.7945e-03_rb,5.9706e-03_rb,5.1108e-03_rb,4.2263e-03_rb,3.3055e-03_rb, &
     &2.3437e-03_rb,1.3169e-03_rb,6.1466e-05_rb/)
      kao(:, 4, 7, 5) = (/ &
     &9.8701e-03_rb,8.8589e-03_rb,7.8113e-03_rb,6.7131e-03_rb,5.5681e-03_rb,4.3736e-03_rb, &
     &3.1150e-03_rb,1.7640e-03_rb,7.9784e-05_rb/)
      kao(:, 5, 7, 5) = (/ &
     &1.2627e-02_rb,1.1349e-02_rb,1.0044e-02_rb,8.6633e-03_rb,7.2033e-03_rb,5.6735e-03_rb, &
     &4.0519e-03_rb,2.3093e-03_rb,9.9338e-05_rb/)
      kao(:, 1, 8, 5) = (/ &
     &6.5162e-03_rb,5.7610e-03_rb,5.0100e-03_rb,4.2382e-03_rb,3.4497e-03_rb,2.6520e-03_rb, &
     &1.8357e-03_rb,9.8841e-04_rb,5.0893e-05_rb/)
      kao(:, 2, 8, 5) = (/ &
     &9.3894e-03_rb,8.3047e-03_rb,7.2233e-03_rb,6.1097e-03_rb,4.9767e-03_rb,3.8267e-03_rb, &
     &2.6500e-03_rb,1.4278e-03_rb,6.6360e-05_rb/)
      kao(:, 3, 8, 5) = (/ &
     &1.2886e-02_rb,1.1402e-02_rb,9.9227e-03_rb,8.3986e-03_rb,6.8476e-03_rb,5.2715e-03_rb, &
     &3.6563e-03_rb,1.9797e-03_rb,8.1964e-05_rb/)
      kao(:, 4, 8, 5) = (/ &
     &1.7029e-02_rb,1.5075e-02_rb,1.3132e-02_rb,1.1136e-02_rb,9.1030e-03_rb,7.0269e-03_rb, &
     &4.8948e-03_rb,2.6694e-03_rb,1.0083e-04_rb/)
      kao(:, 5, 8, 5) = (/ &
     &2.2131e-02_rb,1.9595e-02_rb,1.7101e-02_rb,1.4537e-02_rb,1.1918e-02_rb,9.2249e-03_rb, &
     &6.4389e-03_rb,3.5237e-03_rb,1.2644e-04_rb/)
      kao(:, 1, 9, 5) = (/ &
     &1.9574e-02_rb,1.7171e-02_rb,1.4767e-02_rb,1.2373e-02_rb,9.9784e-03_rb,7.5698e-03_rb, &
     &5.1323e-03_rb,2.6624e-03_rb,7.9787e-05_rb/)
      kao(:, 2, 9, 5) = (/ &
     &2.9063e-02_rb,2.5494e-02_rb,2.1931e-02_rb,1.8384e-02_rb,1.4827e-02_rb,1.1241e-02_rb, &
     &7.6171e-03_rb,3.9520e-03_rb,1.0489e-04_rb/)
      kao(:, 3, 9, 5) = (/ &
     &4.0973e-02_rb,3.5943e-02_rb,3.0923e-02_rb,2.5930e-02_rb,2.0911e-02_rb,1.5846e-02_rb, &
     &1.0739e-02_rb,5.5789e-03_rb,1.2330e-04_rb/)
      kao(:, 4, 9, 5) = (/ &
     &5.5333e-02_rb,4.8546e-02_rb,4.1776e-02_rb,3.5034e-02_rb,2.8259e-02_rb,2.1435e-02_rb, &
     &1.4550e-02_rb,7.5751e-03_rb,1.5097e-04_rb/)
      kao(:, 5, 9, 5) = (/ &
     &7.2795e-02_rb,6.3880e-02_rb,5.4976e-02_rb,4.6114e-02_rb,3.7235e-02_rb,2.8276e-02_rb, &
     &1.9247e-02_rb,1.0060e-02_rb,1.9444e-04_rb/)
      kao(:, 1,10, 5) = (/ &
     &6.7876e-02_rb,5.9426e-02_rb,5.0975e-02_rb,4.2524e-02_rb,3.4085e-02_rb,2.5643e-02_rb, &
     &1.7219e-02_rb,8.7441e-03_rb,2.1379e-04_rb/)
      kao(:, 2,10, 5) = (/ &
     &1.0363e-01_rb,9.0726e-02_rb,7.7824e-02_rb,6.4925e-02_rb,5.2032e-02_rb,3.9153e-02_rb, &
     &2.6288e-02_rb,1.3336e-02_rb,2.7530e-04_rb/)
      kao(:, 3,10, 5) = (/ &
     &1.4868e-01_rb,1.3017e-01_rb,1.1167e-01_rb,9.3161e-02_rb,7.4667e-02_rb,5.6180e-02_rb, &
     &3.7706e-02_rb,1.9125e-02_rb,3.1198e-04_rb/)
      kao(:, 4,10, 5) = (/ &
     &2.0441e-01_rb,1.7897e-01_rb,1.5353e-01_rb,1.2809e-01_rb,1.0267e-01_rb,7.7285e-02_rb, &
     &5.1877e-02_rb,2.6327e-02_rb,3.7361e-04_rb/)
      kao(:, 5,10, 5) = (/ &
     &2.7248e-01_rb,2.3857e-01_rb,2.0467e-01_rb,1.7078e-01_rb,1.3689e-01_rb,1.0307e-01_rb, &
     &6.9236e-02_rb,3.5215e-02_rb,4.4890e-04_rb/)
      kao(:, 1,11, 5) = (/ &
     &1.0259e-01_rb,8.9796e-02_rb,7.7004e-02_rb,6.4217e-02_rb,5.1425e-02_rb,3.8649e-02_rb, &
     &2.5882e-02_rb,1.3106e-02_rb,2.9325e-04_rb/)
      kao(:, 2,11, 5) = (/ &
     &1.5400e-01_rb,1.3480e-01_rb,1.1559e-01_rb,9.6403e-02_rb,7.7200e-02_rb,5.8036e-02_rb, &
     &3.8878e-02_rb,1.9667e-02_rb,3.5327e-04_rb/)
      kao(:, 3,11, 5) = (/ &
     &2.2118e-01_rb,1.9361e-01_rb,1.6603e-01_rb,1.3846e-01_rb,1.1090e-01_rb,8.3368e-02_rb, &
     &5.5861e-02_rb,2.8242e-02_rb,4.6202e-04_rb/)
      kao(:, 4,11, 5) = (/ &
     &3.0353e-01_rb,2.6571e-01_rb,2.2790e-01_rb,1.9008e-01_rb,1.5227e-01_rb,1.1447e-01_rb, &
     &7.6708e-02_rb,3.8764e-02_rb,5.6335e-04_rb/)
      kao(:, 5,11, 5) = (/ &
     &4.0552e-01_rb,3.5495e-01_rb,3.0446e-01_rb,2.5391e-01_rb,2.0338e-01_rb,1.5287e-01_rb, &
     &1.0246e-01_rb,5.1876e-02_rb,6.9233e-04_rb/)
      kao(:, 1,12, 5) = (/ &
     &1.1805e-01_rb,1.0334e-01_rb,8.8617e-02_rb,7.3889e-02_rb,5.9170e-02_rb,4.4431e-02_rb, &
     &2.9722e-02_rb,1.5007e-02_rb,3.3712e-04_rb/)
      kao(:, 2,12, 5) = (/ &
     &1.7261e-01_rb,1.5108e-01_rb,1.2954e-01_rb,1.0804e-01_rb,8.6508e-02_rb,6.5013e-02_rb, &
     &4.3529e-02_rb,2.1984e-02_rb,3.8771e-04_rb/)
      kao(:, 3,12, 5) = (/ &
     &2.4519e-01_rb,2.1462e-01_rb,1.8406e-01_rb,1.5350e-01_rb,1.2297e-01_rb,9.2422e-02_rb, &
     &6.1905e-02_rb,3.1241e-02_rb,5.1427e-04_rb/)
      kao(:, 4,12, 5) = (/ &
     &3.3550e-01_rb,2.9369e-01_rb,2.5183e-01_rb,2.1003e-01_rb,1.6821e-01_rb,1.2645e-01_rb, &
     &8.4714e-02_rb,4.2803e-02_rb,6.2818e-04_rb/)
      kao(:, 5,12, 5) = (/ &
     &4.4900e-01_rb,3.9309e-01_rb,3.3714e-01_rb,2.8120e-01_rb,2.2522e-01_rb,1.6927e-01_rb, &
     &1.1341e-01_rb,5.7326e-02_rb,7.6692e-04_rb/)
      kao(:, 1,13, 5) = (/ &
     &1.0921e-01_rb,9.5613e-02_rb,8.2016e-02_rb,6.8423e-02_rb,5.4814e-02_rb,4.1207e-02_rb, &
     &2.7589e-02_rb,1.3915e-02_rb,3.0996e-04_rb/)
      kao(:, 2,13, 5) = (/ &
     &1.5908e-01_rb,1.3927e-01_rb,1.1945e-01_rb,9.9633e-02_rb,7.9786e-02_rb,5.9973e-02_rb, &
     &4.0176e-02_rb,2.0276e-02_rb,3.7414e-04_rb/)
      kao(:, 3,13, 5) = (/ &
     &2.2086e-01_rb,1.9339e-01_rb,1.6582e-01_rb,1.3829e-01_rb,1.1077e-01_rb,8.3306e-02_rb, &
     &5.5823e-02_rb,2.8176e-02_rb,4.9992e-04_rb/)
      kao(:, 4,13, 5) = (/ &
     &3.0195e-01_rb,2.6439e-01_rb,2.2676e-01_rb,1.8917e-01_rb,1.5156e-01_rb,1.1401e-01_rb, &
     &7.6395e-02_rb,3.8587e-02_rb,6.2293e-04_rb/)
      kao(:, 5,13, 5) = (/ &
     &4.0305e-01_rb,3.5276e-01_rb,3.0259e-01_rb,2.5241e-01_rb,2.0221e-01_rb,1.5209e-01_rb, &
     &1.0190e-01_rb,5.1477e-02_rb,7.6835e-04_rb/)
      kao(:, 1, 1, 6) = (/ &
     &1.3028e-03_rb,1.4792e-03_rb,1.5640e-03_rb,1.5783e-03_rb,1.5338e-03_rb,1.4263e-03_rb, &
     &1.2426e-03_rb,1.0270e-03_rb,3.8769e-04_rb/)
      kao(:, 2, 1, 6) = (/ &
     &1.5941e-03_rb,1.8539e-03_rb,1.9924e-03_rb,2.0205e-03_rb,1.9782e-03_rb,1.8703e-03_rb, &
     &1.6672e-03_rb,1.3063e-03_rb,5.8906e-04_rb/)
      kao(:, 3, 1, 6) = (/ &
     &1.9170e-03_rb,2.2892e-03_rb,2.4874e-03_rb,2.5352e-03_rb,2.4910e-03_rb,2.3785e-03_rb, &
     &2.1678e-03_rb,1.7982e-03_rb,8.4228e-04_rb/)
      kao(:, 4, 1, 6) = (/ &
     &2.2745e-03_rb,2.7857e-03_rb,3.0496e-03_rb,3.1195e-03_rb,3.0784e-03_rb,2.9487e-03_rb, &
     &2.7123e-03_rb,2.3729e-03_rb,1.1599e-03_rb/)
      kao(:, 5, 1, 6) = (/ &
     &2.6664e-03_rb,3.3396e-03_rb,3.6796e-03_rb,3.7742e-03_rb,3.7339e-03_rb,3.5735e-03_rb, &
     &3.3124e-03_rb,2.9540e-03_rb,1.5049e-03_rb/)
      kao(:, 1, 2, 6) = (/ &
     &1.6105e-03_rb,1.7112e-03_rb,1.7440e-03_rb,1.7104e-03_rb,1.6159e-03_rb,1.4650e-03_rb, &
     &1.2347e-03_rb,8.8371e-04_rb,2.8335e-04_rb/)
      kao(:, 2, 2, 6) = (/ &
     &1.9953e-03_rb,2.1575e-03_rb,2.2293e-03_rb,2.1981e-03_rb,2.0893e-03_rb,1.9131e-03_rb, &
     &1.6649e-03_rb,1.2817e-03_rb,4.3740e-04_rb/)
      kao(:, 3, 2, 6) = (/ &
     &2.4284e-03_rb,2.6728e-03_rb,2.7960e-03_rb,2.7642e-03_rb,2.6394e-03_rb,2.4361e-03_rb, &
     &2.1522e-03_rb,1.6930e-03_rb,6.3499e-04_rb/)
      kao(:, 4, 2, 6) = (/ &
     &2.9107e-03_rb,3.2657e-03_rb,3.4379e-03_rb,3.4084e-03_rb,3.2638e-03_rb,3.0336e-03_rb, &
     &2.6919e-03_rb,2.1920e-03_rb,8.6677e-04_rb/)
      kao(:, 5, 2, 6) = (/ &
     &3.4457e-03_rb,3.9315e-03_rb,4.1602e-03_rb,4.1311e-03_rb,3.9674e-03_rb,3.6982e-03_rb, &
     &3.2905e-03_rb,2.7416e-03_rb,1.1277e-03_rb/)
      kao(:, 1, 3, 6) = (/ &
     &2.5362e-03_rb,2.4734e-03_rb,2.3740e-03_rb,2.2299e-03_rb,2.0298e-03_rb,1.7661e-03_rb, &
     &1.4316e-03_rb,9.7765e-04_rb,1.9718e-04_rb/)
      kao(:, 2, 3, 6) = (/ &
     &3.1979e-03_rb,3.1448e-03_rb,3.0502e-03_rb,2.8893e-03_rb,2.6412e-03_rb,2.3120e-03_rb, &
     &1.8934e-03_rb,1.3566e-03_rb,3.0943e-04_rb/)
      kao(:, 3, 3, 6) = (/ &
     &3.9564e-03_rb,3.9245e-03_rb,3.8464e-03_rb,3.6625e-03_rb,3.3542e-03_rb,2.9452e-03_rb, &
     &2.4362e-03_rb,1.8025e-03_rb,4.5105e-04_rb/)
      kao(:, 4, 3, 6) = (/ &
     &4.8099e-03_rb,4.8142e-03_rb,4.7669e-03_rb,4.5499e-03_rb,4.1705e-03_rb,3.6723e-03_rb, &
     &3.0586e-03_rb,2.2944e-03_rb,6.1219e-04_rb/)
      kao(:, 5, 3, 6) = (/ &
     &5.7692e-03_rb,5.8286e-03_rb,5.8075e-03_rb,5.5468e-03_rb,5.0930e-03_rb,4.4966e-03_rb, &
     &3.7644e-03_rb,2.8408e-03_rb,8.0331e-04_rb/)
      kao(:, 1, 4, 6) = (/ &
     &3.9832e-03_rb,3.6741e-03_rb,3.3865e-03_rb,3.0403e-03_rb,2.6684e-03_rb,2.2429e-03_rb, &
     &1.7419e-03_rb,1.1291e-03_rb,1.3015e-04_rb/)
      kao(:, 2, 4, 6) = (/ &
     &5.1171e-03_rb,4.7408e-03_rb,4.3919e-03_rb,3.9718e-03_rb,3.5069e-03_rb,2.9654e-03_rb, &
     &2.3168e-03_rb,1.5306e-03_rb,2.0974e-04_rb/)
      kao(:, 3, 4, 6) = (/ &
     &6.4449e-03_rb,5.9980e-03_rb,5.5858e-03_rb,5.0842e-03_rb,4.5074e-03_rb,3.8196e-03_rb, &
     &2.9911e-03_rb,2.0056e-03_rb,3.1006e-04_rb/)
      kao(:, 4, 4, 6) = (/ &
     &7.9648e-03_rb,7.4434e-03_rb,6.9731e-03_rb,6.3878e-03_rb,5.6676e-03_rb,4.8062e-03_rb, &
     &3.7737e-03_rb,2.5549e-03_rb,4.2568e-04_rb/)
      kao(:, 5, 4, 6) = (/ &
     &9.7092e-03_rb,9.1026e-03_rb,8.5703e-03_rb,7.8734e-03_rb,6.9867e-03_rb,5.9261e-03_rb, &
     &4.6651e-03_rb,3.1844e-03_rb,5.6406e-04_rb/)
      kao(:, 1, 5, 6) = (/ &
     &5.7027e-03_rb,5.1210e-03_rb,4.6077e-03_rb,4.0275e-03_rb,3.4179e-03_rb,2.7812e-03_rb, &
     &2.0863e-03_rb,1.2862e-03_rb,7.8383e-05_rb/)
      kao(:, 2, 5, 6) = (/ &
     &7.4791e-03_rb,6.7335e-03_rb,6.0729e-03_rb,5.3256e-03_rb,4.5457e-03_rb,3.7228e-03_rb, &
     &2.8115e-03_rb,1.7506e-03_rb,1.3561e-04_rb/)
      kao(:, 3, 5, 6) = (/ &
     &9.5940e-03_rb,8.6573e-03_rb,7.8294e-03_rb,6.8962e-03_rb,5.9152e-03_rb,4.8604e-03_rb, &
     &3.6801e-03_rb,2.2991e-03_rb,2.0565e-04_rb/)
      kao(:, 4, 5, 6) = (/ &
     &1.2079e-02_rb,1.0922e-02_rb,9.8984e-03_rb,8.7543e-03_rb,7.5389e-03_rb,6.1967e-03_rb, &
     &4.6995e-03_rb,2.9469e-03_rb,2.9267e-04_rb/)
      kao(:, 5, 5, 6) = (/ &
     &1.4994e-02_rb,1.3590e-02_rb,1.2330e-02_rb,1.0933e-02_rb,9.4206e-03_rb,7.7402e-03_rb, &
     &5.8678e-03_rb,3.6936e-03_rb,3.9433e-04_rb/)
      kao(:, 1, 6, 6) = (/ &
     &7.5110e-03_rb,6.6602e-03_rb,5.8889e-03_rb,5.0761e-03_rb,4.2229e-03_rb,3.3438e-03_rb, &
     &2.4265e-03_rb,1.4290e-03_rb,6.0800e-05_rb/)
      kao(:, 2, 6, 6) = (/ &
     &1.0081e-02_rb,8.9573e-03_rb,7.9340e-03_rb,6.8466e-03_rb,5.7108e-03_rb,4.5453e-03_rb, &
     &3.3197e-03_rb,1.9723e-03_rb,8.3721e-05_rb/)
      kao(:, 3, 6, 6) = (/ &
     &1.3198e-02_rb,1.1747e-02_rb,1.0417e-02_rb,9.0067e-03_rb,7.5375e-03_rb,6.0255e-03_rb, &
     &4.4194e-03_rb,2.6335e-03_rb,1.2422e-04_rb/)
      kao(:, 4, 6, 6) = (/ &
     &1.6984e-02_rb,1.5135e-02_rb,1.3426e-02_rb,1.1628e-02_rb,9.7560e-03_rb,7.8119e-03_rb, &
     &5.7333e-03_rb,3.4188e-03_rb,1.8443e-04_rb/)
      kao(:, 5, 6, 6) = (/ &
     &2.1488e-02_rb,1.9171e-02_rb,1.7013e-02_rb,1.4755e-02_rb,1.2406e-02_rb,9.9290e-03_rb, &
     &7.2811e-03_rb,4.3367e-03_rb,2.5906e-04_rb/)
      kao(:, 1, 7, 6) = (/ &
     &1.0317e-02_rb,9.0822e-03_rb,7.9160e-03_rb,6.7515e-03_rb,5.5440e-03_rb,4.3074e-03_rb, &
     &3.0340e-03_rb,1.7035e-03_rb,6.0426e-05_rb/)
      kao(:, 2, 7, 6) = (/ &
     &1.4191e-02_rb,1.2505e-02_rb,1.0919e-02_rb,9.3201e-03_rb,7.6597e-03_rb,5.9649e-03_rb, &
     &4.2263e-03_rb,2.3919e-03_rb,8.0608e-05_rb/)
      kao(:, 3, 7, 6) = (/ &
     &1.9028e-02_rb,1.6789e-02_rb,1.4664e-02_rb,1.2525e-02_rb,1.0306e-02_rb,8.0458e-03_rb, &
     &5.7261e-03_rb,3.2535e-03_rb,1.0671e-04_rb/)
      kao(:, 4, 7, 6) = (/ &
     &2.5018e-02_rb,2.2107e-02_rb,1.9309e-02_rb,1.6498e-02_rb,1.3592e-02_rb,1.0633e-02_rb, &
     &7.5736e-03_rb,4.3041e-03_rb,1.3667e-04_rb/)
      kao(:, 5, 7, 6) = (/ &
     &3.2278e-02_rb,2.8550e-02_rb,2.4932e-02_rb,2.1309e-02_rb,1.7576e-02_rb,1.3764e-02_rb, &
     &9.7979e-03_rb,5.5674e-03_rb,1.7569e-04_rb/)
      kao(:, 1, 8, 6) = (/ &
     &1.6908e-02_rb,1.4827e-02_rb,1.2781e-02_rb,1.0775e-02_rb,8.7653e-03_rb,6.7156e-03_rb, &
     &4.6263e-03_rb,2.4796e-03_rb,7.2095e-05_rb/)
      kao(:, 2, 8, 6) = (/ &
     &2.3895e-02_rb,2.0964e-02_rb,1.8085e-02_rb,1.5267e-02_rb,1.2424e-02_rb,9.5170e-03_rb, &
     &6.5710e-03_rb,3.5441e-03_rb,9.6806e-05_rb/)
      kao(:, 3, 8, 6) = (/ &
     &3.2855e-02_rb,2.8841e-02_rb,2.4900e-02_rb,2.1032e-02_rb,1.7114e-02_rb,1.3120e-02_rb, &
     &9.0780e-03_rb,4.9231e-03_rb,1.2768e-04_rb/)
      kao(:, 4, 8, 6) = (/ &
     &4.4127e-02_rb,3.8763e-02_rb,3.3492e-02_rb,2.8282e-02_rb,2.3023e-02_rb,1.7668e-02_rb, &
     &1.2245e-02_rb,6.6502e-03_rb,1.6091e-04_rb/)
      kao(:, 5, 8, 6) = (/ &
     &5.7992e-02_rb,5.0977e-02_rb,4.4047e-02_rb,3.7189e-02_rb,3.0275e-02_rb,2.3248e-02_rb, &
     &1.6136e-02_rb,8.7683e-03_rb,1.9428e-04_rb/)
      kao(:, 1, 9, 6) = (/ &
     &5.1709e-02_rb,4.5263e-02_rb,3.8825e-02_rb,3.2404e-02_rb,2.6001e-02_rb,1.9630e-02_rb, &
     &1.3270e-02_rb,6.8433e-03_rb,1.6819e-04_rb/)
      kao(:, 2, 9, 6) = (/ &
     &7.5377e-02_rb,6.5994e-02_rb,5.6614e-02_rb,4.7250e-02_rb,3.7927e-02_rb,2.8667e-02_rb, &
     &1.9405e-02_rb,1.0007e-02_rb,2.0150e-04_rb/)
      kao(:, 3, 9, 6) = (/ &
     &1.0637e-01_rb,9.3142e-02_rb,7.9913e-02_rb,6.6713e-02_rb,5.3579e-02_rb,4.0525e-02_rb, &
     &2.7438e-02_rb,1.4159e-02_rb,2.5395e-04_rb/)
      kao(:, 4, 9, 6) = (/ &
     &1.4601e-01_rb,1.2787e-01_rb,1.0973e-01_rb,9.1640e-02_rb,7.3643e-02_rb,5.5707e-02_rb, &
     &3.7721e-02_rb,1.9493e-02_rb,2.9114e-04_rb/)
      kao(:, 5, 9, 6) = (/ &
     &1.9540e-01_rb,1.7114e-01_rb,1.4689e-01_rb,1.2270e-01_rb,9.8611e-02_rb,7.4576e-02_rb, &
     &5.0506e-02_rb,2.6131e-02_rb,3.2502e-04_rb/)
      kao(:, 1,10, 6) = (/ &
     &1.7908e-01_rb,1.5672e-01_rb,1.3434e-01_rb,1.1197e-01_rb,8.9592e-02_rb,6.7243e-02_rb, &
     &4.4903e-02_rb,2.2650e-02_rb,3.5407e-04_rb/)
      kao(:, 2,10, 6) = (/ &
     &2.6809e-01_rb,2.3461e-01_rb,2.0111e-01_rb,1.6763e-01_rb,1.3416e-01_rb,1.0070e-01_rb, &
     &6.7289e-02_rb,3.4001e-02_rb,4.2267e-04_rb/)
      kao(:, 3,10, 6) = (/ &
     &3.8962e-01_rb,3.4094e-01_rb,2.9228e-01_rb,2.4365e-01_rb,1.9501e-01_rb,1.4639e-01_rb, &
     &9.7852e-02_rb,4.9496e-02_rb,5.5217e-04_rb/)
      kao(:, 4,10, 6) = (/ &
     &5.4709e-01_rb,4.7880e-01_rb,4.1047e-01_rb,3.4221e-01_rb,2.7390e-01_rb,2.0562e-01_rb, &
     &1.3748e-01_rb,6.9566e-02_rb,6.7800e-04_rb/)
      kao(:, 5,10, 6) = (/ &
     &7.4383e-01_rb,6.5097e-01_rb,5.5813e-01_rb,4.6528e-01_rb,3.7246e-01_rb,2.7966e-01_rb, &
     &1.8702e-01_rb,9.4633e-02_rb,7.9930e-04_rb/)
      kao(:, 1,11, 6) = (/ &
     &2.6405e-01_rb,2.3107e-01_rb,1.9807e-01_rb,1.6508e-01_rb,1.3209e-01_rb,9.9087e-02_rb, &
     &6.6140e-02_rb,3.3239e-02_rb,4.6661e-04_rb/)
      kao(:, 2,11, 6) = (/ &
     &3.9885e-01_rb,3.4898e-01_rb,2.9917e-01_rb,2.4935e-01_rb,1.9952e-01_rb,1.4970e-01_rb, &
     &9.9938e-02_rb,5.0287e-02_rb,6.6831e-04_rb/)
      kao(:, 3,11, 6) = (/ &
     &5.8178e-01_rb,5.0910e-01_rb,4.3644e-01_rb,3.6379e-01_rb,2.9110e-01_rb,2.1843e-01_rb, &
     &1.4582e-01_rb,7.3440e-02_rb,7.3058e-04_rb/)
      kao(:, 4,11, 6) = (/ &
     &8.2081e-01_rb,7.1832e-01_rb,6.1577e-01_rb,5.1330e-01_rb,4.1077e-01_rb,3.0826e-01_rb, &
     &2.0583e-01_rb,1.0374e-01_rb,9.0801e-04_rb/)
      kao(:, 5,11, 6) = (/ &
     &1.1196e+00_rb,9.7976e-01_rb,8.3991e-01_rb,7.0013e-01_rb,5.6034e-01_rb,4.2058e-01_rb, &
     &2.8087e-01_rb,1.4156e-01_rb,1.1083e-03_rb/)
      kao(:, 1,12, 6) = (/ &
     &2.9667e-01_rb,2.5960e-01_rb,2.2251e-01_rb,1.8545e-01_rb,1.4837e-01_rb,1.1135e-01_rb, &
     &7.4300e-02_rb,3.7344e-02_rb,4.3166e-04_rb/)
      kao(:, 2,12, 6) = (/ &
     &4.4544e-01_rb,3.8980e-01_rb,3.3414e-01_rb,2.7849e-01_rb,2.2283e-01_rb,1.6719e-01_rb, &
     &1.1158e-01_rb,5.6122e-02_rb,7.5682e-04_rb/)
      kao(:, 3,12, 6) = (/ &
     &6.4877e-01_rb,5.6770e-01_rb,4.8669e-01_rb,4.0564e-01_rb,3.2461e-01_rb,2.4355e-01_rb, &
     &1.6257e-01_rb,8.1850e-02_rb,7.9167e-04_rb/)
      kao(:, 4,12, 6) = (/ &
     &9.1198e-01_rb,7.9809e-01_rb,6.8416e-01_rb,5.7025e-01_rb,4.5635e-01_rb,3.4245e-01_rb, &
     &2.2863e-01_rb,1.1515e-01_rb,9.9906e-04_rb/)
      kao(:, 5,12, 6) = (/ &
     &1.2467e+00_rb,1.0910e+00_rb,9.3529e-01_rb,7.7959e-01_rb,6.2391e-01_rb,4.6827e-01_rb, &
     &3.1271e-01_rb,1.5753e-01_rb,1.2003e-03_rb/)
      kao(:, 1,13, 6) = (/ &
     &2.7690e-01_rb,2.4228e-01_rb,2.0769e-01_rb,1.7308e-01_rb,1.3848e-01_rb,1.0389e-01_rb, &
     &6.9340e-02_rb,3.4888e-02_rb,4.3050e-04_rb/)
      kao(:, 2,13, 6) = (/ &
     &4.0226e-01_rb,3.5198e-01_rb,3.0172e-01_rb,2.5145e-01_rb,2.0123e-01_rb,1.5101e-01_rb, &
     &1.0081e-01_rb,5.0783e-02_rb,7.1649e-04_rb/)
      kao(:, 3,13, 6) = (/ &
     &5.7863e-01_rb,5.0634e-01_rb,4.3410e-01_rb,3.6182e-01_rb,2.8956e-01_rb,2.1728e-01_rb, &
     &1.4512e-01_rb,7.3148e-02_rb,7.7482e-04_rb/)
      kao(:, 4,13, 6) = (/ &
     &8.1324e-01_rb,7.1165e-01_rb,6.1013e-01_rb,5.0851e-01_rb,4.0696e-01_rb,3.0541e-01_rb, &
     &2.0404e-01_rb,1.0287e-01_rb,9.9392e-04_rb/)
      kao(:, 5,13, 6) = (/ &
     &1.1104e+00_rb,9.7179e-01_rb,8.3315e-01_rb,6.9450e-01_rb,5.5586e-01_rb,4.1728e-01_rb, &
     &2.7889e-01_rb,1.4068e-01_rb,1.1816e-03_rb/)
      kao(:, 1, 1, 7) = (/ &
     &3.5054e-03_rb,3.5055e-03_rb,3.5120e-03_rb,3.3810e-03_rb,3.1730e-03_rb,2.8811e-03_rb, &
     &2.4850e-03_rb,1.7479e-03_rb,1.1652e-03_rb/)
      kao(:, 2, 1, 7) = (/ &
     &4.3050e-03_rb,4.3583e-03_rb,4.3910e-03_rb,4.2644e-03_rb,4.0306e-03_rb,3.6880e-03_rb, &
     &3.2528e-03_rb,2.5892e-03_rb,1.6752e-03_rb/)
      kao(:, 3, 1, 7) = (/ &
     &5.1735e-03_rb,5.2976e-03_rb,5.3803e-03_rb,5.2761e-03_rb,5.0137e-03_rb,4.6183e-03_rb, &
     &4.1353e-03_rb,3.5068e-03_rb,2.2495e-03_rb/)
      kao(:, 4, 1, 7) = (/ &
     &6.1057e-03_rb,6.3115e-03_rb,6.4786e-03_rb,6.4038e-03_rb,6.1210e-03_rb,5.6909e-03_rb, &
     &5.1278e-03_rb,4.5615e-03_rb,2.9063e-03_rb/)
      kao(:, 5, 1, 7) = (/ &
     &7.0959e-03_rb,7.4049e-03_rb,7.6778e-03_rb,7.6470e-03_rb,7.3706e-03_rb,6.8965e-03_rb, &
     &6.2418e-03_rb,5.7511e-03_rb,3.6720e-03_rb/)
      kao(:, 1, 2, 7) = (/ &
     &4.4522e-03_rb,4.2827e-03_rb,4.1712e-03_rb,3.9058e-03_rb,3.5730e-03_rb,3.1581e-03_rb, &
     &2.6434e-03_rb,1.9182e-03_rb,8.6485e-04_rb/)
      kao(:, 2, 2, 7) = (/ &
     &5.5183e-03_rb,5.3551e-03_rb,5.2393e-03_rb,4.9436e-03_rb,4.5529e-03_rb,4.0469e-03_rb, &
     &3.4309e-03_rb,2.6479e-03_rb,1.2370e-03_rb/)
      kao(:, 3, 2, 7) = (/ &
     &6.6858e-03_rb,6.5555e-03_rb,6.4351e-03_rb,6.1353e-03_rb,5.6849e-03_rb,5.0737e-03_rb, &
     &4.3365e-03_rb,3.4457e-03_rb,1.6663e-03_rb/)
      kao(:, 4, 2, 7) = (/ &
     &7.9566e-03_rb,7.8606e-03_rb,7.7697e-03_rb,7.4702e-03_rb,6.9634e-03_rb,6.2571e-03_rb, &
     &5.3849e-03_rb,4.3375e-03_rb,2.1739e-03_rb/)
      kao(:, 5, 2, 7) = (/ &
     &9.3107e-03_rb,9.2749e-03_rb,9.2506e-03_rb,8.9459e-03_rb,8.3896e-03_rb,7.5840e-03_rb, &
     &6.5605e-03_rb,5.3408e-03_rb,2.7747e-03_rb/)
      kao(:, 1, 3, 7) = (/ &
     &7.1663e-03_rb,6.5385e-03_rb,6.0832e-03_rb,5.5176e-03_rb,4.8422e-03_rb,4.1012e-03_rb, &
     &3.2557e-03_rb,2.2244e-03_rb,5.9815e-04_rb/)
      kao(:, 2, 3, 7) = (/ &
     &9.0118e-03_rb,8.2694e-03_rb,7.7301e-03_rb,7.0328e-03_rb,6.2053e-03_rb,5.2869e-03_rb, &
     &4.2235e-03_rb,2.9652e-03_rb,8.5322e-04_rb/)
      kao(:, 3, 3, 7) = (/ &
     &1.1073e-02_rb,1.0215e-02_rb,9.5832e-03_rb,8.7498e-03_rb,7.7844e-03_rb,6.6733e-03_rb, &
     &5.3588e-03_rb,3.8104e-03_rb,1.1597e-03_rb/)
      kao(:, 4, 3, 7) = (/ &
     &1.3336e-02_rb,1.2373e-02_rb,1.1642e-02_rb,1.0696e-02_rb,9.5807e-03_rb,8.2482e-03_rb, &
     &6.6726e-03_rb,4.7891e-03_rb,1.5317e-03_rb/)
      kao(:, 5, 3, 7) = (/ &
     &1.5766e-02_rb,1.4727e-02_rb,1.3917e-02_rb,1.2876e-02_rb,1.1585e-02_rb,1.0016e-02_rb, &
     &8.1535e-03_rb,5.8982e-03_rb,1.9785e-03_rb/)
      kao(:, 1, 4, 7) = (/ &
     &1.1443e-02_rb,1.0194e-02_rb,9.0847e-03_rb,8.0229e-03_rb,6.8443e-03_rb,5.5702e-03_rb, &
     &4.2183e-03_rb,2.6813e-03_rb,4.0030e-04_rb/)
      kao(:, 2, 4, 7) = (/ &
     &1.4659e-02_rb,1.3095e-02_rb,1.1714e-02_rb,1.0375e-02_rb,8.8664e-03_rb,7.2483e-03_rb, &
     &5.5253e-03_rb,3.5568e-03_rb,5.7754e-04_rb/)
      kao(:, 3, 4, 7) = (/ &
     &1.8306e-02_rb,1.6401e-02_rb,1.4719e-02_rb,1.3058e-02_rb,1.1190e-02_rb,9.2063e-03_rb, &
     &7.0626e-03_rb,4.5942e-03_rb,7.9824e-04_rb/)
      kao(:, 4, 4, 7) = (/ &
     &2.2363e-02_rb,2.0101e-02_rb,1.8107e-02_rb,1.6087e-02_rb,1.3850e-02_rb,1.1470e-02_rb, &
     &8.8334e-03_rb,5.7997e-03_rb,1.0688e-03_rb/)
      kao(:, 5, 4, 7) = (/ &
     &2.6741e-02_rb,2.4136e-02_rb,2.1831e-02_rb,1.9473e-02_rb,1.6867e-02_rb,1.4029e-02_rb, &
     &1.0846e-02_rb,7.1614e-03_rb,1.3988e-03_rb/)
      kao(:, 1, 5, 7) = (/ &
     &1.6600e-02_rb,1.4668e-02_rb,1.2778e-02_rb,1.1029e-02_rb,9.2361e-03_rb,7.3333e-03_rb, &
     &5.3444e-03_rb,3.2124e-03_rb,2.6040e-04_rb/)
      kao(:, 2, 5, 7) = (/ &
     &2.1709e-02_rb,1.9200e-02_rb,1.6777e-02_rb,1.4511e-02_rb,1.2172e-02_rb,9.6744e-03_rb, &
     &7.0766e-03_rb,4.2987e-03_rb,3.8620e-04_rb/)
      kao(:, 3, 5, 7) = (/ &
     &2.7589e-02_rb,2.4430e-02_rb,2.1405e-02_rb,1.8550e-02_rb,1.5579e-02_rb,1.2414e-02_rb, &
     &9.1361e-03_rb,5.5941e-03_rb,5.4726e-04_rb/)
      kao(:, 4, 5, 7) = (/ &
     &3.4183e-02_rb,3.0324e-02_rb,2.6641e-02_rb,2.3153e-02_rb,1.9480e-02_rb,1.5601e-02_rb, &
     &1.1542e-02_rb,7.1098e-03_rb,7.4652e-04_rb/)
      kao(:, 5, 5, 7) = (/ &
     &4.1421e-02_rb,3.6823e-02_rb,3.2447e-02_rb,2.8275e-02_rb,2.3873e-02_rb,1.9232e-02_rb, &
     &1.4297e-02_rb,8.8472e-03_rb,9.9177e-04_rb/)
      kao(:, 1, 6, 7) = (/ &
     &2.2041e-02_rb,1.9394e-02_rb,1.6757e-02_rb,1.4215e-02_rb,1.1736e-02_rb,9.1749e-03_rb, &
     &6.5141e-03_rb,3.7310e-03_rb,1.4404e-04_rb/)
      kao(:, 2, 6, 7) = (/ &
     &2.9498e-02_rb,2.5961e-02_rb,2.2459e-02_rb,1.9104e-02_rb,1.5796e-02_rb,1.2359e-02_rb, &
     &8.7857e-03_rb,5.0673e-03_rb,2.3466e-04_rb/)
      kao(:, 3, 6, 7) = (/ &
     &3.8260e-02_rb,3.3683e-02_rb,2.9196e-02_rb,2.4887e-02_rb,2.0608e-02_rb,1.6142e-02_rb, &
     &1.1503e-02_rb,6.6775e-03_rb,3.4879e-04_rb/)
      kao(:, 4, 6, 7) = (/ &
     &4.8196e-02_rb,4.2468e-02_rb,3.6884e-02_rb,3.1502e-02_rb,2.6137e-02_rb,2.0520e-02_rb, &
     &1.4704e-02_rb,8.5847e-03_rb,4.9990e-04_rb/)
      kao(:, 5, 6, 7) = (/ &
     &5.9282e-02_rb,5.2285e-02_rb,4.5513e-02_rb,3.8965e-02_rb,3.2393e-02_rb,2.5526e-02_rb, &
     &1.8401e-02_rb,1.0794e-02_rb,6.8674e-04_rb/)
      kao(:, 1, 7, 7) = (/ &
     &3.0340e-02_rb,2.6632e-02_rb,2.2911e-02_rb,1.9238e-02_rb,1.5651e-02_rb,1.2078e-02_rb, &
     &8.4116e-03_rb,4.6225e-03_rb,1.1044e-04_rb/)
      kao(:, 2, 7, 7) = (/ &
     &4.1630e-02_rb,3.6541e-02_rb,3.1448e-02_rb,2.6441e-02_rb,2.1565e-02_rb,1.6662e-02_rb, &
     &1.1608e-02_rb,6.3979e-03_rb,1.5103e-04_rb/)
      kao(:, 3, 7, 7) = (/ &
     &5.5141e-02_rb,4.8404e-02_rb,4.1697e-02_rb,3.5120e-02_rb,2.8696e-02_rb,2.2204e-02_rb, &
     &1.5482e-02_rb,8.5690e-03_rb,2.0480e-04_rb/)
      kao(:, 4, 7, 7) = (/ &
     &7.0797e-02_rb,6.2144e-02_rb,5.3596e-02_rb,4.5212e-02_rb,3.6995e-02_rb,2.8664e-02_rb, &
     &2.0039e-02_rb,1.1163e-02_rb,2.9866e-04_rb/)
      kao(:, 5, 7, 7) = (/ &
     &8.8459e-02_rb,7.7674e-02_rb,6.7088e-02_rb,5.6677e-02_rb,4.6470e-02_rb,3.6087e-02_rb, &
     &2.5332e-02_rb,1.4195e-02_rb,4.2372e-04_rb/)
      kao(:, 1, 8, 7) = (/ &
     &4.9601e-02_rb,4.3472e-02_rb,3.7321e-02_rb,3.1177e-02_rb,2.5072e-02_rb,1.9045e-02_rb, &
     &1.3035e-02_rb,6.9120e-03_rb,1.2554e-04_rb/)
      kao(:, 2, 8, 7) = (/ &
     &6.9873e-02_rb,6.1229e-02_rb,5.2577e-02_rb,4.3926e-02_rb,3.5352e-02_rb,2.6911e-02_rb, &
     &1.8460e-02_rb,9.7927e-03_rb,1.5143e-04_rb/)
      kao(:, 3, 8, 7) = (/ &
     &9.4645e-02_rb,8.2931e-02_rb,7.1217e-02_rb,5.9530e-02_rb,4.7979e-02_rb,3.6597e-02_rb, &
     &2.5135e-02_rb,1.3350e-02_rb,1.8982e-04_rb/)
      kao(:, 4, 8, 7) = (/ &
     &1.2398e-01_rb,1.0863e-01_rb,9.3295e-02_rb,7.8049e-02_rb,6.2983e-02_rb,4.8099e-02_rb, &
     &3.3072e-02_rb,1.7624e-02_rb,2.4338e-04_rb/)
      kao(:, 5, 8, 7) = (/ &
     &1.5756e-01_rb,1.3808e-01_rb,1.1863e-01_rb,9.9354e-02_rb,8.0265e-02_rb,6.1390e-02_rb, &
     &4.2297e-02_rb,2.2655e-02_rb,3.0934e-04_rb/)
      kao(:, 1, 9, 7) = (/ &
     &1.5112e-01_rb,1.3227e-01_rb,1.1344e-01_rb,9.4596e-02_rb,7.5748e-02_rb,5.6892e-02_rb, &
     &3.8080e-02_rb,1.9401e-02_rb,2.5169e-04_rb/)
      kao(:, 2, 9, 7) = (/ &
     &2.1904e-01_rb,1.9173e-01_rb,1.6442e-01_rb,1.3710e-01_rb,1.0978e-01_rb,8.2457e-02_rb, &
     &5.5213e-02_rb,2.8220e-02_rb,3.2207e-04_rb/)
      kao(:, 3, 9, 7) = (/ &
     &3.0409e-01_rb,2.6618e-01_rb,2.2825e-01_rb,1.9033e-01_rb,1.5240e-01_rb,1.1448e-01_rb, &
     &7.6724e-02_rb,3.9326e-02_rb,3.9043e-04_rb/)
      kao(:, 4, 9, 7) = (/ &
     &4.0691e-01_rb,3.5618e-01_rb,3.0544e-01_rb,2.5467e-01_rb,2.0393e-01_rb,1.5326e-01_rb, &
     &1.0281e-01_rb,5.2796e-02_rb,4.7191e-04_rb/)
      kao(:, 5, 9, 7) = (/ &
     &5.2694e-01_rb,4.6123e-01_rb,3.9556e-01_rb,3.2985e-01_rb,2.6417e-01_rb,1.9865e-01_rb, &
     &1.3342e-01_rb,6.8629e-02_rb,5.3966e-04_rb/)
      kao(:, 1,10, 7) = (/ &
     &5.2400e-01_rb,4.5853e-01_rb,3.9306e-01_rb,3.2760e-01_rb,2.6216e-01_rb,1.9669e-01_rb, &
     &1.3123e-01_rb,6.5752e-02_rb,5.0343e-04_rb/)
      kao(:, 2,10, 7) = (/ &
     &7.8161e-01_rb,6.8400e-01_rb,5.8634e-01_rb,4.8867e-01_rb,3.9103e-01_rb,2.9337e-01_rb, &
     &1.9570e-01_rb,9.8059e-02_rb,7.7212e-04_rb/)
      kao(:, 3,10, 7) = (/ &
     &1.1108e+00_rb,9.7208e-01_rb,8.3335e-01_rb,6.9453e-01_rb,5.5575e-01_rb,4.1693e-01_rb, &
     &2.7814e-01_rb,1.3935e-01_rb,8.3303e-04_rb/)
      kao(:, 4,10, 7) = (/ &
     &1.5159e+00_rb,1.3265e+00_rb,1.1372e+00_rb,9.4781e-01_rb,7.5840e-01_rb,5.6900e-01_rb, &
     &3.7960e-01_rb,1.9025e-01_rb,1.0516e-03_rb/)
      kao(:, 5,10, 7) = (/ &
     &2.0008e+00_rb,1.7509e+00_rb,1.5009e+00_rb,1.2510e+00_rb,1.0010e+00_rb,7.5106e-01_rb, &
     &5.0105e-01_rb,2.5132e-01_rb,1.2795e-03_rb/)
      kao(:, 1,11, 7) = (/ &
     &7.6934e-01_rb,6.7322e-01_rb,5.7711e-01_rb,4.8095e-01_rb,3.8483e-01_rb,2.8868e-01_rb, &
     &1.9254e-01_rb,9.6383e-02_rb,4.1213e-04_rb/)
      kao(:, 2,11, 7) = (/ &
     &1.1531e+00_rb,1.0090e+00_rb,8.6492e-01_rb,7.2086e-01_rb,5.7680e-01_rb,4.3268e-01_rb, &
     &2.8857e-01_rb,1.4445e-01_rb,7.8643e-04_rb/)
      kao(:, 3,11, 7) = (/ &
     &1.6479e+00_rb,1.4420e+00_rb,1.2361e+00_rb,1.0302e+00_rb,8.2427e-01_rb,6.1836e-01_rb, &
     &4.1243e-01_rb,2.0646e-01_rb,1.3424e-03_rb/)
      kao(:, 4,11, 7) = (/ &
     &2.2624e+00_rb,1.9796e+00_rb,1.6970e+00_rb,1.4143e+00_rb,1.1316e+00_rb,8.4892e-01_rb, &
     &5.6619e-01_rb,2.8348e-01_rb,1.4044e-03_rb/)
      kao(:, 5,11, 7) = (/ &
     &3.0074e+00_rb,2.6315e+00_rb,2.2558e+00_rb,1.8800e+00_rb,1.5042e+00_rb,1.1285e+00_rb, &
     &7.5266e-01_rb,3.7695e-01_rb,1.6947e-03_rb/)
      kao(:, 1,12, 7) = (/ &
     &8.4478e-01_rb,7.3926e-01_rb,6.3371e-01_rb,5.2813e-01_rb,4.2255e-01_rb,3.1696e-01_rb, &
     &2.1141e-01_rb,1.0578e-01_rb,4.0897e-04_rb/)
      kao(:, 2,12, 7) = (/ &
     &1.2687e+00_rb,1.1101e+00_rb,9.5161e-01_rb,7.9307e-01_rb,6.3449e-01_rb,4.7596e-01_rb, &
     &3.1742e-01_rb,1.5884e-01_rb,7.3572e-04_rb/)
      kao(:, 3,12, 7) = (/ &
     &1.8247e+00_rb,1.5966e+00_rb,1.3686e+00_rb,1.1406e+00_rb,9.1261e-01_rb,6.8459e-01_rb, &
     &4.5656e-01_rb,2.2852e-01_rb,1.3876e-03_rb/)
      kao(:, 4,12, 7) = (/ &
     &2.5294e+00_rb,2.2134e+00_rb,1.8973e+00_rb,1.5812e+00_rb,1.2652e+00_rb,9.4906e-01_rb, &
     &6.3294e-01_rb,3.1685e-01_rb,1.6774e-03_rb/)
      kao(:, 5,12, 7) = (/ &
     &3.3840e+00_rb,2.9611e+00_rb,2.5383e+00_rb,2.1154e+00_rb,1.6926e+00_rb,1.2697e+00_rb, &
     &8.4679e-01_rb,4.2403e-01_rb,1.9227e-03_rb/)
      kao(:, 1,13, 7) = (/ &
     &7.4992e-01_rb,6.5626e-01_rb,5.6252e-01_rb,4.6882e-01_rb,3.7512e-01_rb,2.8140e-01_rb, &
     &1.8768e-01_rb,9.3938e-02_rb,4.2556e-04_rb/)
      kao(:, 2,13, 7) = (/ &
     &1.1304e+00_rb,9.8913e-01_rb,8.4793e-01_rb,7.0671e-01_rb,5.6541e-01_rb,4.2417e-01_rb, &
     &2.8288e-01_rb,1.4159e-01_rb,7.8986e-04_rb/)
      kao(:, 3,13, 7) = (/ &
     &1.6343e+00_rb,1.4300e+00_rb,1.2259e+00_rb,1.0217e+00_rb,8.1745e-01_rb,6.1321e-01_rb, &
     &4.0897e-01_rb,2.0475e-01_rb,1.4019e-03_rb/)
      kao(:, 4,13, 7) = (/ &
     &2.2667e+00_rb,1.9834e+00_rb,1.7002e+00_rb,1.4170e+00_rb,1.1337e+00_rb,8.5047e-01_rb, &
     &5.6722e-01_rb,2.8408e-01_rb,1.6857e-03_rb/)
      kao(:, 5,13, 7) = (/ &
     &3.0450e+00_rb,2.6646e+00_rb,2.2841e+00_rb,1.9036e+00_rb,1.5231e+00_rb,1.1426e+00_rb, &
     &7.6204e-01_rb,3.8179e-01_rb,1.9578e-03_rb/)
      kao(:, 1, 1, 8) = (/ &
     &7.5530e-03_rb,7.1946e-03_rb,6.9309e-03_rb,6.6476e-03_rb,6.1286e-03_rb,5.5188e-03_rb, &
     &4.8784e-03_rb,4.0369e-03_rb,3.2329e-03_rb/)
      kao(:, 2, 1, 8) = (/ &
     &9.1868e-03_rb,8.8285e-03_rb,8.6011e-03_rb,8.3060e-03_rb,7.7227e-03_rb,7.0815e-03_rb, &
     &6.4738e-03_rb,5.8694e-03_rb,4.4599e-03_rb/)
      kao(:, 3, 1, 8) = (/ &
     &1.0993e-02_rb,1.0661e-02_rb,1.0487e-02_rb,1.0178e-02_rb,9.5750e-03_rb,8.9143e-03_rb, &
     &8.3359e-03_rb,7.9019e-03_rb,5.9682e-03_rb/)
      kao(:, 4, 1, 8) = (/ &
     &1.2929e-02_rb,1.2692e-02_rb,1.2580e-02_rb,1.2285e-02_rb,1.1693e-02_rb,1.1035e-02_rb, &
     &1.0548e-02_rb,1.0225e-02_rb,7.8210e-03_rb/)
      kao(:, 5, 1, 8) = (/ &
     &1.4992e-02_rb,1.4924e-02_rb,1.4888e-02_rb,1.4659e-02_rb,1.4068e-02_rb,1.3474e-02_rb, &
     &1.3107e-02_rb,1.2867e-02_rb,1.0020e-02_rb/)
      kao(:, 1, 2, 8) = (/ &
     &1.0207e-02_rb,9.4797e-03_rb,8.8085e-03_rb,8.2309e-03_rb,7.4348e-03_rb,6.4679e-03_rb, &
     &5.3282e-03_rb,4.0056e-03_rb,2.5229e-03_rb/)
      kao(:, 2, 2, 8) = (/ &
     &1.2558e-02_rb,1.1748e-02_rb,1.1002e-02_rb,1.0338e-02_rb,9.3997e-03_rb,8.2648e-03_rb, &
     &6.9317e-03_rb,5.5570e-03_rb,3.5029e-03_rb/)
      kao(:, 3, 2, 8) = (/ &
     &1.5167e-02_rb,1.4277e-02_rb,1.3509e-02_rb,1.2727e-02_rb,1.1653e-02_rb,1.0352e-02_rb, &
     &8.8205e-03_rb,7.4037e-03_rb,4.7140e-03_rb/)
      kao(:, 4, 2, 8) = (/ &
     &1.7986e-02_rb,1.7073e-02_rb,1.6287e-02_rb,1.5417e-02_rb,1.4234e-02_rb,1.2722e-02_rb, &
     &1.1029e-02_rb,9.5450e-03_rb,6.1941e-03_rb/)
      kao(:, 5, 2, 8) = (/ &
     &2.1016e-02_rb,2.0104e-02_rb,1.9317e-02_rb,1.8415e-02_rb,1.7125e-02_rb,1.5397e-02_rb, &
     &1.3573e-02_rb,1.2029e-02_rb,7.9674e-03_rb/)
      kao(:, 1, 3, 8) = (/ &
     &1.7495e-02_rb,1.5793e-02_rb,1.4039e-02_rb,1.2465e-02_rb,1.0907e-02_rb,9.1072e-03_rb, &
     &7.1058e-03_rb,4.8343e-03_rb,1.7832e-03_rb/)
      kao(:, 2, 3, 8) = (/ &
     &2.1893e-02_rb,1.9841e-02_rb,1.7709e-02_rb,1.5801e-02_rb,1.3894e-02_rb,1.1662e-02_rb, &
     &9.2038e-03_rb,6.3817e-03_rb,2.5128e-03_rb/)
      kao(:, 3, 3, 8) = (/ &
     &2.6790e-02_rb,2.4372e-02_rb,2.1871e-02_rb,1.9635e-02_rb,1.7311e-02_rb,1.4624e-02_rb, &
     &1.1658e-02_rb,8.2109e-03_rb,3.4348e-03_rb/)
      kao(:, 4, 3, 8) = (/ &
     &3.2142e-02_rb,2.9358e-02_rb,2.6490e-02_rb,2.3921e-02_rb,2.1182e-02_rb,1.8026e-02_rb, &
     &1.4487e-02_rb,1.0373e-02_rb,4.5646e-03_rb/)
      kao(:, 5, 3, 8) = (/ &
     &3.7933e-02_rb,3.4749e-02_rb,3.1541e-02_rb,2.8628e-02_rb,2.5501e-02_rb,2.1864e-02_rb, &
     &1.7709e-02_rb,1.2883e-02_rb,5.9131e-03_rb/)
      kao(:, 1, 4, 8) = (/ &
     &2.9550e-02_rb,2.6227e-02_rb,2.2910e-02_rb,1.9604e-02_rb,1.6469e-02_rb,1.3339e-02_rb, &
     &9.9415e-03_rb,6.2262e-03_rb,1.2135e-03_rb/)
      kao(:, 2, 4, 8) = (/ &
     &3.7687e-02_rb,3.3528e-02_rb,2.9343e-02_rb,2.5186e-02_rb,2.1249e-02_rb,1.7292e-02_rb, &
     &1.2959e-02_rb,8.2307e-03_rb,1.7374e-03_rb/)
      kao(:, 3, 4, 8) = (/ &
     &4.6817e-02_rb,4.1770e-02_rb,3.6634e-02_rb,3.1558e-02_rb,2.6759e-02_rb,2.1861e-02_rb, &
     &1.6492e-02_rb,1.0607e-02_rb,2.4118e-03_rb/)
      kao(:, 4, 4, 8) = (/ &
     &5.6963e-02_rb,5.0934e-02_rb,4.4739e-02_rb,3.8692e-02_rb,3.2979e-02_rb,2.7058e-02_rb, &
     &2.0566e-02_rb,1.3387e-02_rb,3.2525e-03_rb/)
      kao(:, 5, 4, 8) = (/ &
     &6.8137e-02_rb,6.0987e-02_rb,5.3660e-02_rb,4.6589e-02_rb,3.9864e-02_rb,3.2872e-02_rb, &
     &2.5218e-02_rb,1.6596e-02_rb,4.2777e-03_rb/)
      kao(:, 1, 5, 8) = (/ &
     &4.4958e-02_rb,3.9602e-02_rb,3.4298e-02_rb,2.8957e-02_rb,2.3691e-02_rb,1.8601e-02_rb, &
     &1.3441e-02_rb,7.9068e-03_rb,8.1517e-04_rb/)
      kao(:, 2, 5, 8) = (/ &
     &5.8476e-02_rb,5.1578e-02_rb,4.4743e-02_rb,3.7815e-02_rb,3.1045e-02_rb,2.4482e-02_rb, &
     &1.7770e-02_rb,1.0541e-02_rb,1.1885e-03_rb/)
      kao(:, 3, 5, 8) = (/ &
     &7.3906e-02_rb,6.5268e-02_rb,5.6679e-02_rb,4.8013e-02_rb,3.9537e-02_rb,3.1318e-02_rb, &
     &2.2852e-02_rb,1.3697e-02_rb,1.6746e-03_rb/)
      kao(:, 4, 5, 8) = (/ &
     &9.1296e-02_rb,8.0739e-02_rb,7.0175e-02_rb,5.9533e-02_rb,4.9168e-02_rb,3.9100e-02_rb, &
     &2.8676e-02_rb,1.7397e-02_rb,2.2909e-03_rb/)
      kao(:, 5, 5, 8) = (/ &
     &1.1069e-01_rb,9.7986e-02_rb,8.5202e-02_rb,7.2388e-02_rb,5.9975e-02_rb,4.7864e-02_rb, &
     &3.5291e-02_rb,2.1668e-02_rb,3.0530e-03_rb/)
      kao(:, 1, 6, 8) = (/ &
     &6.2124e-02_rb,5.4544e-02_rb,4.6999e-02_rb,3.9470e-02_rb,3.1923e-02_rb,2.4497e-02_rb, &
     &1.7234e-02_rb,9.7062e-03_rb,5.1266e-04_rb/)
      kao(:, 2, 6, 8) = (/ &
     &8.2639e-02_rb,7.2604e-02_rb,6.2611e-02_rb,5.2626e-02_rb,4.2613e-02_rb,3.2833e-02_rb, &
     &2.3213e-02_rb,1.3144e-02_rb,7.8429e-04_rb/)
      kao(:, 3, 6, 8) = (/ &
     &1.0657e-01_rb,9.3700e-02_rb,8.0879e-02_rb,6.8013e-02_rb,5.5195e-02_rb,4.2665e-02_rb, &
     &3.0306e-02_rb,1.7290e-02_rb,1.1388e-03_rb/)
      kao(:, 4, 6, 8) = (/ &
     &1.3398e-01_rb,1.1789e-01_rb,1.0183e-01_rb,8.5694e-02_rb,6.9665e-02_rb,5.3995e-02_rb, &
     &3.8486e-02_rb,2.2145e-02_rb,1.5829e-03_rb/)
      kao(:, 5, 6, 8) = (/ &
     &1.6490e-01_rb,1.4520e-01_rb,1.2544e-01_rb,1.0568e-01_rb,8.6025e-02_rb,6.6861e-02_rb, &
     &4.7818e-02_rb,2.7758e-02_rb,2.1408e-03_rb/)
      kao(:, 1, 7, 8) = (/ &
     &8.8474e-02_rb,7.7539e-02_rb,6.6627e-02_rb,5.5731e-02_rb,4.4842e-02_rb,3.3990e-02_rb, &
     &2.3276e-02_rb,1.2620e-02_rb,2.5937e-04_rb/)
      kao(:, 2, 7, 8) = (/ &
     &1.2057e-01_rb,1.0571e-01_rb,9.0876e-02_rb,7.6061e-02_rb,6.1253e-02_rb,4.6466e-02_rb, &
     &3.1963e-02_rb,1.7441e-02_rb,4.2731e-04_rb/)
      kao(:, 3, 7, 8) = (/ &
     &1.5880e-01_rb,1.3927e-01_rb,1.1980e-01_rb,1.0034e-01_rb,8.0808e-02_rb,6.1423e-02_rb, &
     &4.2427e-02_rb,2.3279e-02_rb,6.8413e-04_rb/)
      kao(:, 4, 7, 8) = (/ &
     &2.0345e-01_rb,1.7853e-01_rb,1.5363e-01_rb,1.2871e-01_rb,1.0372e-01_rb,7.9011e-02_rb, &
     &5.4743e-02_rb,3.0170e-02_rb,1.0214e-03_rb/)
      kao(:, 5, 7, 8) = (/ &
     &2.5461e-01_rb,2.2346e-01_rb,1.9234e-01_rb,1.6117e-01_rb,1.3002e-01_rb,9.9184e-02_rb, &
     &6.8901e-02_rb,3.8181e-02_rb,1.4552e-03_rb/)
      kao(:, 1, 8, 8) = (/ &
     &1.4868e-01_rb,1.3017e-01_rb,1.1167e-01_rb,9.3201e-02_rb,7.4739e-02_rb,5.6278e-02_rb, &
     &3.7905e-02_rb,1.9693e-02_rb,2.2804e-04_rb/)
      kao(:, 2, 8, 8) = (/ &
     &2.0798e-01_rb,1.8213e-01_rb,1.5630e-01_rb,1.3048e-01_rb,1.0468e-01_rb,7.8891e-02_rb, &
     &5.3152e-02_rb,2.7778e-02_rb,2.9739e-04_rb/)
      kao(:, 3, 8, 8) = (/ &
     &2.8017e-01_rb,2.4539e-01_rb,2.1065e-01_rb,1.7590e-01_rb,1.4119e-01_rb,1.0644e-01_rb, &
     &7.1779e-02_rb,3.7725e-02_rb,3.8946e-04_rb/)
      kao(:, 4, 8, 8) = (/ &
     &3.6604e-01_rb,3.2068e-01_rb,2.7532e-01_rb,2.2999e-01_rb,1.8464e-01_rb,1.3919e-01_rb, &
     &9.4095e-02_rb,4.9659e-02_rb,5.6194e-04_rb/)
      kao(:, 5, 8, 8) = (/ &
     &4.6634e-01_rb,4.0858e-01_rb,3.5083e-01_rb,2.9310e-01_rb,2.3536e-01_rb,1.7756e-01_rb, &
     &1.2024e-01_rb,6.3667e-02_rb,8.0698e-04_rb/)
      kao(:, 1, 9, 8) = (/ &
     &4.6269e-01_rb,4.0493e-01_rb,3.4715e-01_rb,2.8935e-01_rb,2.3157e-01_rb,1.7382e-01_rb, &
     &1.1609e-01_rb,5.8382e-02_rb,3.9623e-04_rb/)
      kao(:, 2, 9, 8) = (/ &
     &6.6628e-01_rb,5.8309e-01_rb,4.9991e-01_rb,4.1673e-01_rb,3.3358e-01_rb,2.5043e-01_rb, &
     &1.6730e-01_rb,8.4243e-02_rb,5.2033e-04_rb/)
      kao(:, 3, 9, 8) = (/ &
     &9.1981e-01_rb,8.0501e-01_rb,6.9024e-01_rb,5.7543e-01_rb,4.6067e-01_rb,3.4594e-01_rb, &
     &2.3120e-01_rb,1.1646e-01_rb,6.1848e-04_rb/)
      kao(:, 4, 9, 8) = (/ &
     &1.2275e+00_rb,1.0743e+00_rb,9.2117e-01_rb,7.6798e-01_rb,6.1489e-01_rb,4.6180e-01_rb, &
     &3.0869e-01_rb,1.5554e-01_rb,7.6432e-04_rb/)
      kao(:, 5, 9, 8) = (/ &
     &1.5931e+00_rb,1.3944e+00_rb,1.1956e+00_rb,9.9699e-01_rb,7.9825e-01_rb,5.9958e-01_rb, &
     &4.0095e-01_rb,2.0227e-01_rb,9.3678e-04_rb/)
      kao(:, 1,10, 8) = (/ &
     &1.6306e+00_rb,1.4267e+00_rb,1.2230e+00_rb,1.0192e+00_rb,8.1547e-01_rb,6.1164e-01_rb, &
     &4.0787e-01_rb,2.0413e-01_rb,4.5853e-04_rb/)
      kao(:, 2,10, 8) = (/ &
     &2.4126e+00_rb,2.1111e+00_rb,1.8096e+00_rb,1.5081e+00_rb,1.2066e+00_rb,9.0513e-01_rb, &
     &6.0364e-01_rb,3.0217e-01_rb,8.3201e-04_rb/)
      kao(:, 3,10, 8) = (/ &
     &3.4093e+00_rb,2.9833e+00_rb,2.5573e+00_rb,2.1313e+00_rb,1.7053e+00_rb,1.2792e+00_rb, &
     &8.5320e-01_rb,4.2724e-01_rb,1.5974e-03_rb/)
      kao(:, 4,10, 8) = (/ &
     &4.6433e+00_rb,4.0630e+00_rb,3.4826e+00_rb,2.9025e+00_rb,2.3226e+00_rb,1.7423e+00_rb, &
     &1.1621e+00_rb,5.8201e-01_rb,1.5962e-03_rb/)
      kao(:, 5,10, 8) = (/ &
     &6.1277e+00_rb,5.3619e+00_rb,4.5964e+00_rb,3.8308e+00_rb,3.0651e+00_rb,2.2994e+00_rb, &
     &1.5338e+00_rb,7.6843e-01_rb,2.0260e-03_rb/)
      kao(:, 1,11, 8) = (/ &
     &2.4162e+00_rb,2.1143e+00_rb,1.8124e+00_rb,1.5103e+00_rb,1.2083e+00_rb,9.0631e-01_rb, &
     &6.0429e-01_rb,3.0233e-01_rb,6.1607e-04_rb/)
      kao(:, 2,11, 8) = (/ &
     &3.5951e+00_rb,3.1458e+00_rb,2.6963e+00_rb,2.2470e+00_rb,1.7978e+00_rb,1.3484e+00_rb, &
     &8.9917e-01_rb,4.4993e-01_rb,6.5176e-04_rb/)
      kao(:, 3,11, 8) = (/ &
     &5.1060e+00_rb,4.4678e+00_rb,3.8297e+00_rb,3.1915e+00_rb,2.5535e+00_rb,1.9153e+00_rb, &
     &1.2772e+00_rb,6.3920e-01_rb,1.2930e-03_rb/)
      kao(:, 4,11, 8) = (/ &
     &6.9828e+00_rb,6.1098e+00_rb,5.2373e+00_rb,4.3645e+00_rb,3.4923e+00_rb,2.6195e+00_rb, &
     &1.7469e+00_rb,8.7439e-01_rb,2.6451e-03_rb/)
      kao(:, 5,11, 8) = (/ &
     &9.2372e+00_rb,8.0825e+00_rb,6.9284e+00_rb,5.7745e+00_rb,4.6202e+00_rb,3.4655e+00_rb, &
     &2.3113e+00_rb,1.1571e+00_rb,3.2091e-03_rb/)
      kao(:, 1,12, 8) = (/ &
     &2.6551e+00_rb,2.3231e+00_rb,1.9913e+00_rb,1.6595e+00_rb,1.3276e+00_rb,9.9570e-01_rb, &
     &6.6396e-01_rb,3.3215e-01_rb,5.7595e-04_rb/)
      kao(:, 2,12, 8) = (/ &
     &3.9875e+00_rb,3.4892e+00_rb,2.9906e+00_rb,2.4922e+00_rb,1.9940e+00_rb,1.4956e+00_rb, &
     &9.9727e-01_rb,4.9897e-01_rb,7.2773e-04_rb/)
      kao(:, 3,12, 8) = (/ &
     &5.6951e+00_rb,4.9832e+00_rb,4.2714e+00_rb,3.5598e+00_rb,2.8480e+00_rb,2.1362e+00_rb, &
     &1.4245e+00_rb,7.1282e-01_rb,1.1732e-03_rb/)
      kao(:, 4,12, 8) = (/ &
     &7.8078e+00_rb,6.8322e+00_rb,5.8562e+00_rb,4.8804e+00_rb,3.9048e+00_rb,2.9289e+00_rb, &
     &1.9532e+00_rb,9.7758e-01_rb,2.5629e-03_rb/)
      kao(:, 5,12, 8) = (/ &
     &1.0362e+01_rb,9.0671e+00_rb,7.7724e+00_rb,6.4775e+00_rb,5.1823e+00_rb,3.8873e+00_rb, &
     &2.5926e+00_rb,1.2977e+00_rb,3.8105e-03_rb/)
      kao(:, 1,13, 8) = (/ &
     &2.3258e+00_rb,2.0350e+00_rb,1.7444e+00_rb,1.4537e+00_rb,1.1630e+00_rb,8.7226e-01_rb, &
     &5.8161e-01_rb,2.9098e-01_rb,5.9908e-04_rb/)
      kao(:, 2,13, 8) = (/ &
     &3.5360e+00_rb,3.0938e+00_rb,2.6520e+00_rb,2.2101e+00_rb,1.7681e+00_rb,1.3262e+00_rb, &
     &8.8435e-01_rb,4.4250e-01_rb,8.1555e-04_rb/)
      kao(:, 3,13, 8) = (/ &
     &5.0896e+00_rb,4.4536e+00_rb,3.8175e+00_rb,3.1814e+00_rb,2.5452e+00_rb,1.9091e+00_rb, &
     &1.2732e+00_rb,6.3712e-01_rb,1.2640e-03_rb/)
      kao(:, 4,13, 8) = (/ &
     &7.0210e+00_rb,6.1435e+00_rb,5.2660e+00_rb,4.3886e+00_rb,3.5112e+00_rb,2.6338e+00_rb, &
     &1.7565e+00_rb,8.7919e-01_rb,2.6701e-03_rb/)
      kao(:, 5,13, 8) = (/ &
     &9.3671e+00_rb,8.1963e+00_rb,7.0260e+00_rb,5.8554e+00_rb,4.6849e+00_rb,3.5142e+00_rb, &
     &2.3437e+00_rb,1.1734e+00_rb,3.9606e-03_rb/)
      kao(:, 1, 1, 9) = (/ &
     &1.7557e-02_rb,1.6098e-02_rb,1.5252e-02_rb,1.4591e-02_rb,1.4194e-02_rb,1.3841e-02_rb, &
     &1.3213e-02_rb,1.1760e-02_rb,1.0941e-02_rb/)
      kao(:, 2, 1, 9) = (/ &
     &2.1322e-02_rb,1.9727e-02_rb,1.8932e-02_rb,1.8315e-02_rb,1.8048e-02_rb,1.7757e-02_rb, &
     &1.7105e-02_rb,1.5377e-02_rb,1.4897e-02_rb/)
      kao(:, 3, 1, 9) = (/ &
     &2.5364e-02_rb,2.3730e-02_rb,2.3058e-02_rb,2.2569e-02_rb,2.2483e-02_rb,2.2337e-02_rb, &
     &2.1647e-02_rb,1.9659e-02_rb,1.9740e-02_rb/)
      kao(:, 4, 1, 9) = (/ &
     &2.9651e-02_rb,2.8091e-02_rb,2.7567e-02_rb,2.7373e-02_rb,2.7544e-02_rb,2.7572e-02_rb, &
     &2.6885e-02_rb,2.4698e-02_rb,2.5506e-02_rb/)
      kao(:, 5, 1, 9) = (/ &
     &3.4147e-02_rb,3.2801e-02_rb,3.2509e-02_rb,3.2696e-02_rb,3.3242e-02_rb,3.3466e-02_rb, &
     &3.2763e-02_rb,3.0530e-02_rb,3.2414e-02_rb/)
      kao(:, 1, 2, 9) = (/ &
     &2.4912e-02_rb,2.2331e-02_rb,2.0288e-02_rb,1.8446e-02_rb,1.6840e-02_rb,1.5437e-02_rb, &
     &1.4050e-02_rb,1.2105e-02_rb,9.2979e-03_rb/)
      kao(:, 2, 2, 9) = (/ &
     &3.0564e-02_rb,2.7515e-02_rb,2.5261e-02_rb,2.3219e-02_rb,2.1400e-02_rb,1.9828e-02_rb, &
     &1.8282e-02_rb,1.5938e-02_rb,1.2847e-02_rb/)
      kao(:, 3, 2, 9) = (/ &
     &3.6716e-02_rb,3.3250e-02_rb,3.0813e-02_rb,2.8576e-02_rb,2.6652e-02_rb,2.4969e-02_rb, &
     &2.3242e-02_rb,2.0434e-02_rb,1.7255e-02_rb/)
      kao(:, 4, 2, 9) = (/ &
     &4.3274e-02_rb,3.9470e-02_rb,3.6945e-02_rb,3.4546e-02_rb,3.2567e-02_rb,3.0902e-02_rb, &
     &2.8990e-02_rb,2.5608e-02_rb,2.2622e-02_rb/)
      kao(:, 5, 2, 9) = (/ &
     &5.0151e-02_rb,4.6154e-02_rb,4.3548e-02_rb,4.1147e-02_rb,3.9208e-02_rb,3.7617e-02_rb, &
     &3.5476e-02_rb,3.1506e-02_rb,2.9070e-02_rb/)
      kao(:, 1, 3, 9) = (/ &
     &4.5824e-02_rb,4.0428e-02_rb,3.5360e-02_rb,3.0531e-02_rb,2.5873e-02_rb,2.1515e-02_rb, &
     &1.7437e-02_rb,1.3182e-02_rb,7.2135e-03_rb/)
      kao(:, 2, 3, 9) = (/ &
     &5.7122e-02_rb,5.0441e-02_rb,4.4296e-02_rb,3.8535e-02_rb,3.2887e-02_rb,2.7626e-02_rb, &
     &2.2637e-02_rb,1.7470e-02_rb,1.0156e-02_rb/)
      kao(:, 3, 3, 9) = (/ &
     &6.9514e-02_rb,6.1484e-02_rb,5.4267e-02_rb,4.7520e-02_rb,4.0893e-02_rb,3.4647e-02_rb, &
     &2.8731e-02_rb,2.2576e-02_rb,1.3878e-02_rb/)
      kao(:, 4, 3, 9) = (/ &
     &8.2919e-02_rb,7.3512e-02_rb,6.5273e-02_rb,5.7524e-02_rb,4.9783e-02_rb,4.2603e-02_rb, &
     &3.5784e-02_rb,2.8563e-02_rb,1.8502e-02_rb/)
      kao(:, 5, 3, 9) = (/ &
     &9.7170e-02_rb,8.6431e-02_rb,7.7261e-02_rb,6.8346e-02_rb,5.9584e-02_rb,5.1514e-02_rb, &
     &4.3768e-02_rb,3.5379e-02_rb,2.4141e-02_rb/)
      kao(:, 1, 4, 9) = (/ &
     &8.5028e-02_rb,7.4675e-02_rb,6.4408e-02_rb,5.4349e-02_rb,4.4385e-02_rb,3.4667e-02_rb, &
     &2.5298e-02_rb,1.6347e-02_rb,5.3723e-03_rb/)
      kao(:, 2, 4, 9) = (/ &
     &1.0799e-01_rb,9.4832e-02_rb,8.1887e-02_rb,6.9265e-02_rb,5.6849e-02_rb,4.4649e-02_rb, &
     &3.2963e-02_rb,2.1613e-02_rb,7.7235e-03_rb/)
      kao(:, 3, 4, 9) = (/ &
     &1.3350e-01_rb,1.1723e-01_rb,1.0141e-01_rb,8.6053e-02_rb,7.1006e-02_rb,5.6113e-02_rb, &
     &4.1792e-02_rb,2.7868e-02_rb,1.0747e-02_rb/)
      kao(:, 4, 4, 9) = (/ &
     &1.6142e-01_rb,1.4180e-01_rb,1.2297e-01_rb,1.0476e-01_rb,8.6810e-02_rb,6.8956e-02_rb, &
     &5.1841e-02_rb,3.5143e-02_rb,1.4564e-02_rb/)
      kao(:, 5, 4, 9) = (/ &
     &1.9126e-01_rb,1.6822e-01_rb,1.4629e-01_rb,1.2515e-01_rb,1.0422e-01_rb,8.3176e-02_rb, &
     &6.3089e-02_rb,4.3500e-02_rb,1.9267e-02_rb/)
      kao(:, 1, 5, 9) = (/ &
     &1.4488e-01_rb,1.2702e-01_rb,1.0917e-01_rb,9.1352e-02_rb,7.3672e-02_rb,5.6082e-02_rb, &
     &3.8867e-02_rb,2.2212e-02_rb,3.9357e-03_rb/)
      kao(:, 2, 5, 9) = (/ &
     &1.8767e-01_rb,1.6453e-01_rb,1.4140e-01_rb,1.1846e-01_rb,9.5672e-02_rb,7.3107e-02_rb, &
     &5.0961e-02_rb,2.9559e-02_rb,5.7686e-03_rb/)
      kao(:, 3, 5, 9) = (/ &
     &2.3604e-01_rb,2.0693e-01_rb,1.7791e-01_rb,1.4922e-01_rb,1.2079e-01_rb,9.2713e-02_rb, &
     &6.4987e-02_rb,3.8154e-02_rb,8.1725e-03_rb/)
      kao(:, 4, 5, 9) = (/ &
     &2.8921e-01_rb,2.5355e-01_rb,2.1818e-01_rb,1.8330e-01_rb,1.4882e-01_rb,1.1469e-01_rb, &
     &8.0893e-02_rb,4.8061e-02_rb,1.1257e-02_rb/)
      kao(:, 5, 5, 9) = (/ &
     &3.4679e-01_rb,3.0409e-01_rb,2.6204e-01_rb,2.2057e-01_rb,1.7963e-01_rb,1.3899e-01_rb, &
     &9.8558e-02_rb,5.9364e-02_rb,1.5113e-02_rb/)
      kao(:, 1, 6, 9) = (/ &
     &2.2687e-01_rb,1.9870e-01_rb,1.7053e-01_rb,1.4235e-01_rb,1.1424e-01_rb,8.6261e-02_rb, &
     &5.8407e-02_rb,3.1198e-02_rb,2.7719e-03_rb/)
      kao(:, 2, 6, 9) = (/ &
     &3.0049e-01_rb,2.6318e-01_rb,2.2586e-01_rb,1.8856e-01_rb,1.5147e-01_rb,1.1450e-01_rb, &
     &7.7818e-02_rb,4.1952e-02_rb,4.1659e-03_rb/)
      kao(:, 3, 6, 9) = (/ &
     &3.8494e-01_rb,3.3714e-01_rb,2.8934e-01_rb,2.4170e-01_rb,1.9431e-01_rb,1.4714e-01_rb, &
     &1.0048e-01_rb,5.4667e-02_rb,6.0249e-03_rb/)
      kao(:, 4, 6, 9) = (/ &
     &4.7888e-01_rb,4.1941e-01_rb,3.6000e-01_rb,3.0097e-01_rb,2.4222e-01_rb,1.8390e-01_rb, &
     &1.2612e-01_rb,6.9245e-02_rb,8.4412e-03_rb/)
      kao(:, 5, 6, 9) = (/ &
     &5.8181e-01_rb,5.0960e-01_rb,4.3765e-01_rb,3.6614e-01_rb,2.9513e-01_rb,2.2467e-01_rb, &
     &1.5478e-01_rb,8.5677e-02_rb,1.1500e-02_rb/)
      kao(:, 1, 7, 9) = (/ &
     &3.6399e-01_rb,3.1864e-01_rb,2.7330e-01_rb,2.2796e-01_rb,1.8261e-01_rb,1.3731e-01_rb, &
     &9.2161e-02_rb,4.7386e-02_rb,1.8335e-03_rb/)
      kao(:, 2, 7, 9) = (/ &
     &4.9404e-01_rb,4.3249e-01_rb,3.7094e-01_rb,3.0937e-01_rb,2.4784e-01_rb,1.8651e-01_rb, &
     &1.2531e-01_rb,6.4814e-02_rb,2.9040e-03_rb/)
      kao(:, 3, 7, 9) = (/ &
     &6.4560e-01_rb,5.6518e-01_rb,4.8472e-01_rb,4.0430e-01_rb,3.2403e-01_rb,2.4402e-01_rb, &
     &1.6421e-01_rb,8.5452e-02_rb,4.3365e-03_rb/)
      kao(:, 4, 7, 9) = (/ &
     &8.1690e-01_rb,7.1513e-01_rb,6.1336e-01_rb,5.1171e-01_rb,4.1038e-01_rb,3.0928e-01_rb, &
     &2.0863e-01_rb,1.0932e-01_rb,6.1934e-03_rb/)
      kao(:, 5, 7, 9) = (/ &
     &1.0068e+00_rb,8.8147e-01_rb,7.5616e-01_rb,6.3106e-01_rb,5.0635e-01_rb,3.8205e-01_rb, &
     &2.5843e-01_rb,1.3634e-01_rb,8.5588e-03_rb/)
      kao(:, 1, 8, 9) = (/ &
     &6.8384e-01_rb,5.9848e-01_rb,5.1310e-01_rb,4.2775e-01_rb,3.4242e-01_rb,2.5705e-01_rb, &
     &1.7171e-01_rb,8.6607e-02_rb,8.9761e-04_rb/)
      kao(:, 2, 8, 9) = (/ &
     &9.5272e-01_rb,8.3382e-01_rb,7.1490e-01_rb,5.9597e-01_rb,4.7702e-01_rb,3.5811e-01_rb, &
     &2.3935e-01_rb,1.2088e-01_rb,1.5768e-03_rb/)
      kao(:, 3, 8, 9) = (/ &
     &1.2716e+00_rb,1.1128e+00_rb,9.5409e-01_rb,7.9538e-01_rb,6.3666e-01_rb,4.7804e-01_rb, &
     &3.1975e-01_rb,1.6179e-01_rb,2.6889e-03_rb/)
      kao(:, 4, 8, 9) = (/ &
     &1.6381e+00_rb,1.4337e+00_rb,1.2291e+00_rb,1.0247e+00_rb,8.2031e-01_rb,6.1622e-01_rb, &
     &4.1239e-01_rb,2.0929e-01_rb,4.1353e-03_rb/)
      kao(:, 5, 8, 9) = (/ &
     &2.0497e+00_rb,1.7940e+00_rb,1.5381e+00_rb,1.2824e+00_rb,1.0268e+00_rb,7.7156e-01_rb, &
     &5.1676e-01_rb,2.6319e-01_rb,6.0327e-03_rb/)
      kao(:, 1, 9, 9) = (/ &
     &2.3562e+00_rb,2.0618e+00_rb,1.7675e+00_rb,1.4730e+00_rb,1.1786e+00_rb,8.8409e-01_rb, &
     &5.8968e-01_rb,2.9523e-01_rb,7.8820e-04_rb/)
      kao(:, 2, 9, 9) = (/ &
     &3.3780e+00_rb,2.9558e+00_rb,2.5337e+00_rb,2.1117e+00_rb,1.6896e+00_rb,1.2675e+00_rb, &
     &8.4537e-01_rb,4.2322e-01_rb,1.1025e-03_rb/)
      kao(:, 3, 9, 9) = (/ &
     &4.6138e+00_rb,4.0373e+00_rb,3.4607e+00_rb,2.8840e+00_rb,2.3076e+00_rb,1.7311e+00_rb, &
     &1.1546e+00_rb,5.7817e-01_rb,1.6643e-03_rb/)
      kao(:, 4, 9, 9) = (/ &
     &6.0608e+00_rb,5.3032e+00_rb,4.5462e+00_rb,3.7886e+00_rb,3.0313e+00_rb,2.2740e+00_rb, &
     &1.5168e+00_rb,7.6000e-01_rb,2.3974e-03_rb/)
      kao(:, 5, 9, 9) = (/ &
     &7.7085e+00_rb,6.7454e+00_rb,5.7823e+00_rb,4.8189e+00_rb,3.8558e+00_rb,2.8925e+00_rb, &
     &1.9295e+00_rb,9.6718e-01_rb,3.2535e-03_rb/)
      kao(:, 1,10, 9) = (/ &
     &9.1055e+00_rb,7.9675e+00_rb,6.8292e+00_rb,5.6913e+00_rb,4.5531e+00_rb,3.4149e+00_rb, &
     &2.2769e+00_rb,1.1388e+00_rb,5.8809e-04_rb/)
      kao(:, 2,10, 9) = (/ &
     &1.3404e+01_rb,1.1729e+01_rb,1.0053e+01_rb,8.3780e+00_rb,6.7024e+00_rb,5.0271e+00_rb, &
     &3.3517e+00_rb,1.6763e+00_rb,9.0563e-04_rb/)
      kao(:, 3,10, 9) = (/ &
     &1.8696e+01_rb,1.6359e+01_rb,1.4022e+01_rb,1.1685e+01_rb,9.3484e+00_rb,7.0117e+00_rb, &
     &4.6747e+00_rb,2.3381e+00_rb,1.7881e-03_rb/)
      kao(:, 4,10, 9) = (/ &
     &2.4995e+01_rb,2.1869e+01_rb,1.8747e+01_rb,1.5622e+01_rb,1.2498e+01_rb,9.3741e+00_rb, &
     &6.2500e+00_rb,3.1259e+00_rb,3.1813e-03_rb/)
      kao(:, 5,10, 9) = (/ &
     &3.2273e+01_rb,2.8239e+01_rb,2.4206e+01_rb,2.0172e+01_rb,1.6138e+01_rb,1.2104e+01_rb, &
     &8.0701e+00_rb,4.0364e+00_rb,3.7345e-03_rb/)
      kao(:, 1,11, 9) = (/ &
     &1.4610e+01_rb,1.2784e+01_rb,1.0958e+01_rb,9.1318e+00_rb,7.3053e+00_rb,5.4792e+00_rb, &
     &3.6529e+00_rb,1.8269e+00_rb,9.3673e-05_rb/)
      kao(:, 2,11, 9) = (/ &
     &2.1482e+01_rb,1.8797e+01_rb,1.6112e+01_rb,1.3427e+01_rb,1.0742e+01_rb,8.0564e+00_rb, &
     &5.3711e+00_rb,2.6860e+00_rb,1.0848e-03_rb/)
      kao(:, 3,11, 9) = (/ &
     &2.9982e+01_rb,2.6234e+01_rb,2.2487e+01_rb,1.8739e+01_rb,1.4992e+01_rb,1.1244e+01_rb, &
     &7.4965e+00_rb,3.7489e+00_rb,1.4593e-03_rb/)
      kao(:, 4,11, 9) = (/ &
     &4.0102e+01_rb,3.5090e+01_rb,3.0079e+01_rb,2.5065e+01_rb,2.0053e+01_rb,1.5039e+01_rb, &
     &1.0027e+01_rb,5.0144e+00_rb,2.0582e-03_rb/)
      kao(:, 5,11, 9) = (/ &
     &5.1873e+01_rb,4.5387e+01_rb,3.8905e+01_rb,3.2421e+01_rb,2.5937e+01_rb,1.9453e+01_rb, &
     &1.2970e+01_rb,6.4862e+00_rb,4.4703e-03_rb/)
      kao(:, 1,12, 9) = (/ &
     &1.7189e+01_rb,1.5041e+01_rb,1.2892e+01_rb,1.0744e+01_rb,8.5949e+00_rb,6.4462e+00_rb, &
     &4.2976e+00_rb,2.1491e+00_rb,3.9266e-05_rb/)
      kao(:, 2,12, 9) = (/ &
     &2.5282e+01_rb,2.2121e+01_rb,1.8962e+01_rb,1.5801e+01_rb,1.2641e+01_rb,9.4811e+00_rb, &
     &6.3210e+00_rb,3.1609e+00_rb,6.4444e-04_rb/)
      kao(:, 3,12, 9) = (/ &
     &3.5289e+01_rb,3.0879e+01_rb,2.6468e+01_rb,2.2056e+01_rb,1.7646e+01_rb,1.3235e+01_rb, &
     &8.8231e+00_rb,4.4124e+00_rb,1.8228e-03_rb/)
      kao(:, 4,12, 9) = (/ &
     &4.7291e+01_rb,4.1381e+01_rb,3.5468e+01_rb,2.9558e+01_rb,2.3647e+01_rb,1.7735e+01_rb, &
     &1.1825e+01_rb,5.9130e+00_rb,2.3757e-03_rb/)
      kao(:, 5,12, 9) = (/ &
     &6.1297e+01_rb,5.3636e+01_rb,4.5972e+01_rb,3.8311e+01_rb,3.0649e+01_rb,2.2988e+01_rb, &
     &1.5326e+01_rb,7.6642e+00_rb,3.9341e-03_rb/)
      kao(:, 1,13, 9) = (/ &
     &1.6000e+01_rb,1.4000e+01_rb,1.2000e+01_rb,9.9998e+00_rb,8.0002e+00_rb,6.0002e+00_rb, &
     &4.0003e+00_rb,2.0005e+00_rb,3.8486e-05_rb/)
      kao(:, 2,13, 9) = (/ &
     &2.3551e+01_rb,2.0607e+01_rb,1.7663e+01_rb,1.4719e+01_rb,1.1776e+01_rb,8.8325e+00_rb, &
     &5.8883e+00_rb,2.9447e+00_rb,5.3945e-04_rb/)
      kao(:, 3,13, 9) = (/ &
     &3.2950e+01_rb,2.8830e+01_rb,2.4711e+01_rb,2.0594e+01_rb,1.6475e+01_rb,1.2356e+01_rb, &
     &8.2382e+00_rb,4.1197e+00_rb,1.7722e-03_rb/)
      kao(:, 4,13, 9) = (/ &
     &4.4263e+01_rb,3.8730e+01_rb,3.3198e+01_rb,2.7666e+01_rb,2.2132e+01_rb,1.6600e+01_rb, &
     &1.1067e+01_rb,5.5345e+00_rb,2.5081e-03_rb/)
      kao(:, 5,13, 9) = (/ &
     &5.7461e+01_rb,5.0277e+01_rb,4.3096e+01_rb,3.5913e+01_rb,2.8731e+01_rb,2.1549e+01_rb, &
     &1.4366e+01_rb,7.1843e+00_rb,4.3602e-03_rb/)
      kao(:, 1, 1,10) = (/ &
     &4.8312e-02_rb,4.2395e-02_rb,3.7029e-02_rb,3.4268e-02_rb,3.1631e-02_rb,2.8012e-02_rb, &
     &2.4227e-02_rb,1.9340e-02_rb,2.3819e-02_rb/)
      kao(:, 2, 1,10) = (/ &
     &5.7978e-02_rb,5.0880e-02_rb,4.5259e-02_rb,4.2684e-02_rb,3.9233e-02_rb,3.5182e-02_rb, &
     &3.0643e-02_rb,2.5810e-02_rb,3.1295e-02_rb/)
      kao(:, 3, 1,10) = (/ &
     &6.8091e-02_rb,5.9773e-02_rb,5.4735e-02_rb,5.1517e-02_rb,4.7810e-02_rb,4.2960e-02_rb, &
     &3.8310e-02_rb,3.3678e-02_rb,4.0832e-02_rb/)
      kao(:, 4, 1,10) = (/ &
     &7.8444e-02_rb,6.8870e-02_rb,6.5474e-02_rb,6.0764e-02_rb,5.6861e-02_rb,5.2020e-02_rb, &
     &4.6525e-02_rb,4.3019e-02_rb,5.3025e-02_rb/)
      kao(:, 5, 1,10) = (/ &
     &8.8955e-02_rb,7.8190e-02_rb,7.6227e-02_rb,7.0877e-02_rb,6.6771e-02_rb,6.2214e-02_rb, &
     &5.5680e-02_rb,5.4857e-02_rb,6.6965e-02_rb/)
      kao(:, 1, 2,10) = (/ &
     &6.8494e-02_rb,6.0019e-02_rb,5.1535e-02_rb,4.5753e-02_rb,4.0717e-02_rb,3.4636e-02_rb, &
     &2.8735e-02_rb,2.2359e-02_rb,2.2396e-02_rb/)
      kao(:, 2, 2,10) = (/ &
     &8.3057e-02_rb,7.2786e-02_rb,6.2703e-02_rb,5.6353e-02_rb,5.0990e-02_rb,4.3733e-02_rb, &
     &3.6309e-02_rb,2.8660e-02_rb,2.9863e-02_rb/)
      kao(:, 3, 2,10) = (/ &
     &9.8445e-02_rb,8.6268e-02_rb,7.5172e-02_rb,6.9101e-02_rb,6.1861e-02_rb,5.3608e-02_rb, &
     &4.5462e-02_rb,3.5628e-02_rb,3.9584e-02_rb/)
      kao(:, 4, 2,10) = (/ &
     &1.1465e-01_rb,1.0047e-01_rb,8.9322e-02_rb,8.2399e-02_rb,7.3847e-02_rb,6.4459e-02_rb, &
     &5.5729e-02_rb,4.5259e-02_rb,5.1011e-02_rb/)
      kao(:, 5, 2,10) = (/ &
     &1.3121e-01_rb,1.1499e-01_rb,1.0547e-01_rb,9.5942e-02_rb,8.6700e-02_rb,7.6912e-02_rb, &
     &6.7683e-02_rb,5.7087e-02_rb,6.4331e-02_rb/)
      kao(:, 1, 3,10) = (/ &
     &1.2396e-01_rb,1.0851e-01_rb,9.3078e-02_rb,7.7936e-02_rb,6.5430e-02_rb,5.3555e-02_rb, &
     &4.0149e-02_rb,2.8165e-02_rb,1.9103e-02_rb/)
      kao(:, 2, 3,10) = (/ &
     &1.5308e-01_rb,1.3401e-01_rb,1.1495e-01_rb,9.6317e-02_rb,8.2061e-02_rb,6.7720e-02_rb, &
     &5.1563e-02_rb,3.6346e-02_rb,2.6147e-02_rb/)
      kao(:, 3, 3,10) = (/ &
     &1.8438e-01_rb,1.6143e-01_rb,1.3846e-01_rb,1.1685e-01_rb,1.0065e-01_rb,8.3573e-02_rb, &
     &6.4169e-02_rb,4.6273e-02_rb,3.5249e-02_rb/)
      kao(:, 4, 3,10) = (/ &
     &2.1730e-01_rb,1.9024e-01_rb,1.6317e-01_rb,1.3937e-01_rb,1.2182e-01_rb,1.0076e-01_rb, &
     &7.7802e-02_rb,5.7458e-02_rb,4.6106e-02_rb/)
      kao(:, 5, 3,10) = (/ &
     &2.5114e-01_rb,2.1989e-01_rb,1.8862e-01_rb,1.6558e-01_rb,1.4330e-01_rb,1.1936e-01_rb, &
     &9.3594e-02_rb,7.0438e-02_rb,5.9048e-02_rb/)
      kao(:, 1, 4,10) = (/ &
     &2.2307e-01_rb,1.9520e-01_rb,1.6737e-01_rb,1.3960e-01_rb,1.1281e-01_rb,8.7662e-02_rb, &
     &6.3355e-02_rb,3.7440e-02_rb,1.5143e-02_rb/)
      kao(:, 2, 4,10) = (/ &
     &2.8121e-01_rb,2.4610e-01_rb,2.1101e-01_rb,1.7601e-01_rb,1.4219e-01_rb,1.1180e-01_rb, &
     &8.0961e-02_rb,4.9348e-02_rb,2.1248e-02_rb/)
      kao(:, 3, 4,10) = (/ &
     &3.4426e-01_rb,3.0128e-01_rb,2.5831e-01_rb,2.1545e-01_rb,1.7407e-01_rb,1.3850e-01_rb, &
     &1.0170e-01_rb,6.2450e-02_rb,2.9486e-02_rb/)
      kao(:, 4, 4,10) = (/ &
     &4.1162e-01_rb,3.6021e-01_rb,3.0885e-01_rb,2.5755e-01_rb,2.0938e-01_rb,1.6873e-01_rb, &
     &1.2428e-01_rb,7.7637e-02_rb,3.9623e-02_rb/)
      kao(:, 5, 4,10) = (/ &
     &4.8400e-01_rb,4.2357e-01_rb,3.6318e-01_rb,3.0275e-01_rb,2.4837e-01_rb,2.0175e-01_rb, &
     &1.4904e-01_rb,9.5123e-02_rb,5.2032e-02_rb/)
      kao(:, 1, 5,10) = (/ &
     &3.6240e-01_rb,3.1713e-01_rb,2.7189e-01_rb,2.2715e-01_rb,1.8287e-01_rb,1.3915e-01_rb, &
     &9.6038e-02_rb,5.3151e-02_rb,1.1469e-02_rb/)
      kao(:, 2, 5,10) = (/ &
     &4.6684e-01_rb,4.0853e-01_rb,3.5029e-01_rb,2.9260e-01_rb,2.3543e-01_rb,1.7892e-01_rb, &
     &1.2501e-01_rb,6.9710e-02_rb,1.6590e-02_rb/)
      kao(:, 3, 5,10) = (/ &
     &5.8305e-01_rb,5.1017e-01_rb,4.3746e-01_rb,3.6540e-01_rb,2.9385e-01_rb,2.2314e-01_rb, &
     &1.5770e-01_rb,8.9552e-02_rb,2.3682e-02_rb/)
      kao(:, 4, 5,10) = (/ &
     &7.1039e-01_rb,6.2164e-01_rb,5.3285e-01_rb,4.4486e-01_rb,3.5759e-01_rb,2.7242e-01_rb, &
     &1.9452e-01_rb,1.1176e-01_rb,3.2616e-02_rb/)
      kao(:, 5, 5,10) = (/ &
     &8.4585e-01_rb,7.4016e-01_rb,6.3454e-01_rb,5.2965e-01_rb,4.2558e-01_rb,3.2651e-01_rb, &
     &2.3601e-01_rb,1.3577e-01_rb,4.3721e-02_rb/)
      kao(:, 1, 6,10) = (/ &
     &5.4565e-01_rb,4.7774e-01_rb,4.0998e-01_rb,3.4242e-01_rb,2.7503e-01_rb,2.0802e-01_rb, &
     &1.4092e-01_rb,7.4294e-02_rb,8.5994e-03_rb/)
      kao(:, 2, 6,10) = (/ &
     &7.2281e-01_rb,6.3285e-01_rb,5.4312e-01_rb,4.5353e-01_rb,3.6423e-01_rb,2.7526e-01_rb, &
     &1.8607e-01_rb,9.8990e-02_rb,1.2620e-02_rb/)
      kao(:, 3, 6,10) = (/ &
     &9.2293e-01_rb,8.0812e-01_rb,6.9347e-01_rb,5.7897e-01_rb,4.6479e-01_rb,3.5103e-01_rb, &
     &2.3728e-01_rb,1.2747e-01_rb,1.8407e-02_rb/)
      kao(:, 4, 6,10) = (/ &
     &1.1448e+00_rb,1.0023e+00_rb,8.6003e-01_rb,7.1795e-01_rb,5.7619e-01_rb,4.3493e-01_rb, &
     &2.9519e-01_rb,1.6083e-01_rb,2.5918e-02_rb/)
      kao(:, 5, 6,10) = (/ &
     &1.3870e+00_rb,1.2144e+00_rb,1.0419e+00_rb,8.6977e-01_rb,6.9789e-01_rb,5.2651e-01_rb, &
     &3.5987e-01_rb,1.9795e-01_rb,3.5516e-02_rb/)
      kao(:, 1, 7,10) = (/ &
     &9.0563e-01_rb,7.9294e-01_rb,6.8033e-01_rb,5.6775e-01_rb,4.5526e-01_rb,3.4288e-01_rb, &
     &2.2993e-01_rb,1.1636e-01_rb,6.3281e-03_rb/)
      kao(:, 2, 7,10) = (/ &
     &1.2304e+00_rb,1.0774e+00_rb,9.2435e-01_rb,7.7129e-01_rb,6.1846e-01_rb,4.6574e-01_rb, &
     &3.1230e-01_rb,1.5802e-01_rb,9.6345e-03_rb/)
      kao(:, 3, 7,10) = (/ &
     &1.6067e+00_rb,1.4068e+00_rb,1.2068e+00_rb,1.0069e+00_rb,8.0717e-01_rb,6.0759e-01_rb, &
     &4.0707e-01_rb,2.0698e-01_rb,1.4230e-02_rb/)
      kao(:, 4, 7,10) = (/ &
     &2.0379e+00_rb,1.7842e+00_rb,1.5305e+00_rb,1.2770e+00_rb,1.0235e+00_rb,7.6992e-01_rb, &
     &5.1539e-01_rb,2.6359e-01_rb,2.0362e-02_rb/)
      kao(:, 5, 7,10) = (/ &
     &2.5042e+00_rb,2.1925e+00_rb,1.8806e+00_rb,1.5690e+00_rb,1.2574e+00_rb,9.4558e-01_rb, &
     &6.3281e-01_rb,3.2569e-01_rb,2.8537e-02_rb/)
      kao(:, 1, 8,10) = (/ &
     &1.8551e+00_rb,1.6239e+00_rb,1.3926e+00_rb,1.1613e+00_rb,9.2995e-01_rb,6.9848e-01_rb, &
     &4.6667e-01_rb,2.3461e-01_rb,3.7282e-03_rb/)
      kao(:, 2, 8,10) = (/ &
     &2.5887e+00_rb,2.2659e+00_rb,1.9430e+00_rb,1.6201e+00_rb,1.2969e+00_rb,9.7383e-01_rb, &
     &6.5051e-01_rb,3.2684e-01_rb,7.1685e-03_rb/)
      kao(:, 3, 8,10) = (/ &
     &3.4601e+00_rb,3.0282e+00_rb,2.5964e+00_rb,2.1645e+00_rb,1.7327e+00_rb,1.3007e+00_rb, &
     &8.6869e-01_rb,4.3618e-01_rb,1.0869e-02_rb/)
      kao(:, 4, 8,10) = (/ &
     &4.4635e+00_rb,3.9062e+00_rb,3.3492e+00_rb,2.7923e+00_rb,2.2350e+00_rb,1.6778e+00_rb, &
     &1.1202e+00_rb,5.6241e-01_rb,1.5855e-02_rb/)
      kao(:, 5, 8,10) = (/ &
     &5.5614e+00_rb,4.8670e+00_rb,4.1730e+00_rb,3.4788e+00_rb,2.7845e+00_rb,2.0899e+00_rb, &
     &1.3954e+00_rb,7.0042e-01_rb,2.2369e-02_rb/)
      kao(:, 1, 9,10) = (/ &
     &7.0057e+00_rb,6.1306e+00_rb,5.2555e+00_rb,4.3802e+00_rb,3.5047e+00_rb,2.6291e+00_rb, &
     &1.7537e+00_rb,8.7821e-01_rb,2.6651e-03_rb/)
      kao(:, 2, 9,10) = (/ &
     &1.0071e+01_rb,8.8120e+00_rb,7.5539e+00_rb,6.2955e+00_rb,5.0368e+00_rb,3.7786e+00_rb, &
     &2.5204e+00_rb,1.2618e+00_rb,3.6187e-03_rb/)
      kao(:, 3, 9,10) = (/ &
     &1.3793e+01_rb,1.2069e+01_rb,1.0345e+01_rb,8.6215e+00_rb,6.8979e+00_rb,5.1747e+00_rb, &
     &3.4512e+00_rb,1.7274e+00_rb,5.0886e-03_rb/)
      kao(:, 4, 9,10) = (/ &
     &1.8095e+01_rb,1.5834e+01_rb,1.3574e+01_rb,1.1311e+01_rb,9.0505e+00_rb,6.7893e+00_rb, &
     &4.5277e+00_rb,2.2659e+00_rb,6.6445e-03_rb/)
      kao(:, 5, 9,10) = (/ &
     &2.2909e+01_rb,2.0046e+01_rb,1.7183e+01_rb,1.4320e+01_rb,1.1458e+01_rb,8.5941e+00_rb, &
     &5.7315e+00_rb,2.8684e+00_rb,1.0715e-02_rb/)
      kao(:, 1,10,10) = (/ &
     &2.9637e+01_rb,2.5932e+01_rb,2.2227e+01_rb,1.8524e+01_rb,1.4818e+01_rb,1.1115e+01_rb, &
     &7.4105e+00_rb,3.7062e+00_rb,1.7663e-03_rb/)
      kao(:, 2,10,10) = (/ &
     &4.3734e+01_rb,3.8266e+01_rb,3.2801e+01_rb,2.7334e+01_rb,2.1867e+01_rb,1.6402e+01_rb, &
     &1.0936e+01_rb,5.4691e+00_rb,2.0817e-03_rb/)
      kao(:, 3,10,10) = (/ &
     &6.1232e+01_rb,5.3576e+01_rb,4.5921e+01_rb,3.8272e+01_rb,3.0616e+01_rb,2.2963e+01_rb, &
     &1.5310e+01_rb,7.6576e+00_rb,1.7504e-03_rb/)
      kao(:, 4,10,10) = (/ &
     &8.1674e+01_rb,7.1459e+01_rb,6.1256e+01_rb,5.1048e+01_rb,4.0839e+01_rb,3.0631e+01_rb, &
     &2.0421e+01_rb,1.0211e+01_rb,7.5551e-03_rb/)
      kao(:, 5,10,10) = (/ &
     &1.0484e+02_rb,9.1726e+01_rb,7.8618e+01_rb,6.5522e+01_rb,5.2414e+01_rb,3.9315e+01_rb, &
     &2.6210e+01_rb,1.3108e+01_rb,1.2092e-02_rb/)
      kao(:, 1,11,10) = (/ &
     &5.2035e+01_rb,4.5533e+01_rb,3.9026e+01_rb,3.2522e+01_rb,2.6019e+01_rb,1.9514e+01_rb, &
     &1.3010e+01_rb,6.5060e+00_rb,2.3377e-05_rb/)
      kao(:, 2,11,10) = (/ &
     &7.6878e+01_rb,6.7270e+01_rb,5.7660e+01_rb,4.8051e+01_rb,3.8444e+01_rb,2.8832e+01_rb, &
     &1.9221e+01_rb,9.6129e+00_rb,3.4131e-05_rb/)
      kao(:, 3,11,10) = (/ &
     &1.0695e+02_rb,9.3584e+01_rb,8.0221e+01_rb,6.6851e+01_rb,5.3480e+01_rb,4.0109e+01_rb, &
     &2.6742e+01_rb,1.3372e+01_rb,6.5532e-03_rb/)
      kao(:, 4,11,10) = (/ &
     &1.4204e+02_rb,1.2428e+02_rb,1.0653e+02_rb,8.8779e+01_rb,7.1025e+01_rb,5.3267e+01_rb, &
     &3.5513e+01_rb,1.7760e+01_rb,3.6230e-03_rb/)
      kao(:, 5,11,10) = (/ &
     &1.8146e+02_rb,1.5877e+02_rb,1.3608e+02_rb,1.1341e+02_rb,9.0723e+01_rb,6.8041e+01_rb, &
     &4.5364e+01_rb,2.2685e+01_rb,5.3060e-03_rb/)
      kao(:, 1,12,10) = (/ &
     &6.6709e+01_rb,5.8372e+01_rb,5.0033e+01_rb,4.1695e+01_rb,3.3357e+01_rb,2.5018e+01_rb, &
     &1.6679e+01_rb,8.3409e+00_rb,1.6896e-05_rb/)
      kao(:, 2,12,10) = (/ &
     &9.8021e+01_rb,8.5767e+01_rb,7.3510e+01_rb,6.1263e+01_rb,4.9006e+01_rb,3.6758e+01_rb, &
     &2.4506e+01_rb,1.2254e+01_rb,2.4443e-05_rb/)
      kao(:, 3,12,10) = (/ &
     &1.3575e+02_rb,1.1880e+02_rb,1.0183e+02_rb,8.4855e+01_rb,6.7883e+01_rb,5.0907e+01_rb, &
     &3.3942e+01_rb,1.6972e+01_rb,3.9041e-05_rb/)
      kao(:, 4,12,10) = (/ &
     &1.7930e+02_rb,1.5689e+02_rb,1.3447e+02_rb,1.1206e+02_rb,8.9651e+01_rb,6.7236e+01_rb, &
     &4.4831e+01_rb,2.2414e+01_rb,2.5406e-03_rb/)
      kao(:, 5,12,10) = (/ &
     &2.2752e+02_rb,1.9909e+02_rb,1.7067e+02_rb,1.4220e+02_rb,1.1376e+02_rb,8.5325e+01_rb, &
     &5.6889e+01_rb,2.8448e+01_rb,5.4913e-03_rb/)
      kao(:, 1,13,10) = (/ &
     &6.7103e+01_rb,5.8720e+01_rb,5.0329e+01_rb,4.1943e+01_rb,3.3555e+01_rb,2.5165e+01_rb, &
     &1.6777e+01_rb,8.3902e+00_rb,1.5511e-05_rb/)
      kao(:, 2,13,10) = (/ &
     &9.7787e+01_rb,8.5564e+01_rb,7.3344e+01_rb,6.1125e+01_rb,4.8896e+01_rb,3.6672e+01_rb, &
     &2.4451e+01_rb,1.2226e+01_rb,2.1977e-05_rb/)
      kao(:, 3,13,10) = (/ &
     &1.3463e+02_rb,1.1780e+02_rb,1.0097e+02_rb,8.4145e+01_rb,6.7313e+01_rb,5.0486e+01_rb, &
     &3.3658e+01_rb,1.6832e+01_rb,3.2342e-05_rb/)
      kao(:, 4,13,10) = (/ &
     &1.7688e+02_rb,1.5477e+02_rb,1.3267e+02_rb,1.1056e+02_rb,8.8438e+01_rb,6.6334e+01_rb, &
     &4.4220e+01_rb,2.2113e+01_rb,5.6189e-03_rb/)
      kao(:, 5,13,10) = (/ &
     &2.2476e+02_rb,1.9667e+02_rb,1.6857e+02_rb,1.4047e+02_rb,1.1238e+02_rb,8.4289e+01_rb, &
     &5.6195e+01_rb,2.8100e+01_rb,6.0459e-03_rb/)
      kao(:, 1, 1,11) = (/ &
     &6.9001e-02_rb,6.0529e-02_rb,5.2064e-02_rb,4.5590e-02_rb,4.0628e-02_rb,3.5945e-02_rb, &
     &2.9389e-02_rb,2.3144e-02_rb,2.9176e-02_rb/)
      kao(:, 2, 1,11) = (/ &
     &8.2103e-02_rb,7.2020e-02_rb,6.1937e-02_rb,5.5694e-02_rb,5.0508e-02_rb,4.4515e-02_rb, &
     &3.6553e-02_rb,3.0757e-02_rb,3.9695e-02_rb/)
      kao(:, 3, 1,11) = (/ &
     &9.5630e-02_rb,8.3878e-02_rb,7.2481e-02_rb,6.7965e-02_rb,6.0680e-02_rb,5.4180e-02_rb, &
     &4.4143e-02_rb,4.0422e-02_rb,5.1968e-02_rb/)
      kao(:, 4, 1,11) = (/ &
     &1.0928e-01_rb,9.5880e-02_rb,8.4382e-02_rb,8.0469e-02_rb,7.1798e-02_rb,6.4108e-02_rb, &
     &5.3608e-02_rb,5.2409e-02_rb,6.6870e-02_rb/)
      kao(:, 5, 1,11) = (/ &
     &1.2299e-01_rb,1.0790e-01_rb,9.8888e-02_rb,9.2808e-02_rb,8.4009e-02_rb,7.5069e-02_rb, &
     &6.6004e-02_rb,6.6900e-02_rb,8.4455e-02_rb/)
      kao(:, 1, 2,11) = (/ &
     &1.0031e-01_rb,8.7857e-02_rb,7.5407e-02_rb,6.3060e-02_rb,5.4499e-02_rb,4.6504e-02_rb, &
     &3.6770e-02_rb,2.6380e-02_rb,2.8413e-02_rb/)
      kao(:, 2, 2,11) = (/ &
     &1.2053e-01_rb,1.0559e-01_rb,9.0627e-02_rb,7.6829e-02_rb,6.7033e-02_rb,5.8104e-02_rb, &
     &4.6509e-02_rb,3.3638e-02_rb,3.8310e-02_rb/)
      kao(:, 3, 2,11) = (/ &
     &1.4155e-01_rb,1.2398e-01_rb,1.0644e-01_rb,9.2248e-02_rb,8.1983e-02_rb,7.0441e-02_rb, &
     &5.6590e-02_rb,4.3821e-02_rb,5.0130e-02_rb/)
      kao(:, 4, 2,11) = (/ &
     &1.6305e-01_rb,1.4282e-01_rb,1.2262e-01_rb,1.1049e-01_rb,9.7062e-02_rb,8.3848e-02_rb, &
     &6.7975e-02_rb,5.5529e-02_rb,6.5378e-02_rb/)
      kao(:, 5, 2,11) = (/ &
     &1.8507e-01_rb,1.6212e-01_rb,1.3962e-01_rb,1.2935e-01_rb,1.1281e-01_rb,9.7726e-02_rb, &
     &8.0117e-02_rb,6.9954e-02_rb,8.3371e-02_rb/)
      kao(:, 1, 3,11) = (/ &
     &1.8667e-01_rb,1.6338e-01_rb,1.4010e-01_rb,1.1682e-01_rb,9.3557e-02_rb,7.4325e-02_rb, &
     &5.5984e-02_rb,3.5878e-02_rb,2.5531e-02_rb/)
      kao(:, 2, 3,11) = (/ &
     &2.2795e-01_rb,1.9954e-01_rb,1.7113e-01_rb,1.4270e-01_rb,1.1480e-01_rb,9.2509e-02_rb, &
     &7.0623e-02_rb,4.6178e-02_rb,3.4917e-02_rb/)
      kao(:, 3, 3,11) = (/ &
     &2.7211e-01_rb,2.3816e-01_rb,2.0423e-01_rb,1.7032e-01_rb,1.3866e-01_rb,1.1338e-01_rb, &
     &8.6877e-02_rb,5.7191e-02_rb,4.6054e-02_rb/)
      kao(:, 4, 3,11) = (/ &
     &3.1770e-01_rb,2.7809e-01_rb,2.3849e-01_rb,1.9887e-01_rb,1.6548e-01_rb,1.3614e-01_rb, &
     &1.0464e-01_rb,7.0039e-02_rb,6.0217e-02_rb/)
      kao(:, 5, 3,11) = (/ &
     &3.6403e-01_rb,3.1863e-01_rb,2.7328e-01_rb,2.2795e-01_rb,1.9641e-01_rb,1.5908e-01_rb, &
     &1.2312e-01_rb,8.3529e-02_rb,7.7572e-02_rb/)
      kao(:, 1, 4,11) = (/ &
     &3.4588e-01_rb,3.0268e-01_rb,2.5948e-01_rb,2.1629e-01_rb,1.7307e-01_rb,1.2990e-01_rb, &
     &9.1217e-02_rb,5.3135e-02_rb,2.1767e-02_rb/)
      kao(:, 2, 4,11) = (/ &
     &4.3145e-01_rb,3.7755e-01_rb,3.2366e-01_rb,2.6977e-01_rb,2.1589e-01_rb,1.6201e-01_rb, &
     &1.1572e-01_rb,6.8208e-02_rb,3.0409e-02_rb/)
      kao(:, 3, 4,11) = (/ &
     &5.2311e-01_rb,4.5776e-01_rb,3.9244e-01_rb,3.2707e-01_rb,2.6176e-01_rb,1.9718e-01_rb, &
     &1.4225e-01_rb,8.5649e-02_rb,4.0906e-02_rb/)
      kao(:, 4, 4,11) = (/ &
     &6.1949e-01_rb,5.4214e-01_rb,4.6476e-01_rb,3.8737e-01_rb,3.1001e-01_rb,2.3606e-01_rb, &
     &1.7310e-01_rb,1.0459e-01_rb,5.4382e-02_rb/)
      kao(:, 5, 4,11) = (/ &
     &7.1941e-01_rb,6.2950e-01_rb,5.3975e-01_rb,4.4985e-01_rb,3.6002e-01_rb,2.8013e-01_rb, &
     &2.0492e-01_rb,1.2467e-01_rb,7.0400e-02_rb/)
      kao(:, 1, 5,11) = (/ &
     &5.8048e-01_rb,5.0796e-01_rb,4.3542e-01_rb,3.6286e-01_rb,2.9032e-01_rb,2.1780e-01_rb, &
     &1.4642e-01_rb,8.0688e-02_rb,1.7624e-02_rb/)
      kao(:, 2, 5,11) = (/ &
     &7.4078e-01_rb,6.4817e-01_rb,5.5562e-01_rb,4.6305e-01_rb,3.7047e-01_rb,2.7793e-01_rb, &
     &1.8667e-01_rb,1.0393e-01_rb,2.5321e-02_rb/)
      kao(:, 3, 5,11) = (/ &
     &9.1562e-01_rb,8.0121e-01_rb,6.8680e-01_rb,5.7241e-01_rb,4.5798e-01_rb,3.4354e-01_rb, &
     &2.3145e-01_rb,1.3051e-01_rb,3.5008e-02_rb/)
      kao(:, 4, 5,11) = (/ &
     &1.1041e+00_rb,9.6607e-01_rb,8.2810e-01_rb,6.9018e-01_rb,5.5220e-01_rb,4.1422e-01_rb, &
     &2.8137e-01_rb,1.6106e-01_rb,4.7504e-02_rb/)
      kao(:, 5, 5,11) = (/ &
     &1.3000e+00_rb,1.1374e+00_rb,9.7498e-01_rb,8.1255e-01_rb,6.5013e-01_rb,4.8767e-01_rb, &
     &3.3622e-01_rb,1.9350e-01_rb,6.2782e-02_rb/)
      kao(:, 1, 6,11) = (/ &
     &8.9158e-01_rb,7.8014e-01_rb,6.6870e-01_rb,5.5724e-01_rb,4.4582e-01_rb,3.3441e-01_rb, &
     &2.2372e-01_rb,1.1615e-01_rb,1.3452e-02_rb/)
      kao(:, 2, 6,11) = (/ &
     &1.1678e+00_rb,1.0218e+00_rb,8.7586e-01_rb,7.2991e-01_rb,5.8393e-01_rb,4.3799e-01_rb, &
     &2.9305e-01_rb,1.5296e-01_rb,1.9872e-02_rb/)
      kao(:, 3, 6,11) = (/ &
     &1.4777e+00_rb,1.2930e+00_rb,1.1083e+00_rb,9.2359e-01_rb,7.3888e-01_rb,5.5427e-01_rb, &
     &3.7059e-01_rb,1.9566e-01_rb,2.8227e-02_rb/)
      kao(:, 4, 6,11) = (/ &
     &1.8119e+00_rb,1.5855e+00_rb,1.3592e+00_rb,1.1325e+00_rb,9.0614e-01_rb,6.7964e-01_rb, &
     &4.5430e-01_rb,2.4269e-01_rb,3.9342e-02_rb/)
      kao(:, 5, 6,11) = (/ &
     &2.1657e+00_rb,1.8951e+00_rb,1.6242e+00_rb,1.3536e+00_rb,1.0830e+00_rb,8.1233e-01_rb, &
     &5.4303e-01_rb,2.9520e-01_rb,5.3415e-02_rb/)
      kao(:, 1, 7,11) = (/ &
     &1.4142e+00_rb,1.2374e+00_rb,1.0606e+00_rb,8.8396e-01_rb,7.0710e-01_rb,5.3041e-01_rb, &
     &3.5484e-01_rb,1.8101e-01_rb,1.0092e-02_rb/)
      kao(:, 2, 7,11) = (/ &
     &1.9068e+00_rb,1.6686e+00_rb,1.4303e+00_rb,1.1919e+00_rb,9.5353e-01_rb,7.1519e-01_rb, &
     &4.7813e-01_rb,2.4337e-01_rb,1.5041e-02_rb/)
      kao(:, 3, 7,11) = (/ &
     &2.4672e+00_rb,2.1587e+00_rb,1.8505e+00_rb,1.5421e+00_rb,1.2336e+00_rb,9.2548e-01_rb, &
     &6.1872e-01_rb,3.1448e-01_rb,2.1900e-02_rb/)
      kao(:, 4, 7,11) = (/ &
     &3.0809e+00_rb,2.6959e+00_rb,2.3109e+00_rb,1.9258e+00_rb,1.5405e+00_rb,1.1559e+00_rb, &
     &7.7290e-01_rb,3.9302e-01_rb,3.1410e-02_rb/)
      kao(:, 5, 7,11) = (/ &
     &3.7474e+00_rb,3.2787e+00_rb,2.8103e+00_rb,2.3419e+00_rb,1.8737e+00_rb,1.4060e+00_rb, &
     &9.3991e-01_rb,4.7981e-01_rb,4.3693e-02_rb/)
      kao(:, 1, 8,11) = (/ &
     &2.6683e+00_rb,2.3349e+00_rb,2.0012e+00_rb,1.6677e+00_rb,1.3344e+00_rb,1.0014e+00_rb, &
     &6.6874e-01_rb,3.3701e-01_rb,7.4648e-03_rb/)
      kao(:, 2, 8,11) = (/ &
     &3.7060e+00_rb,3.2430e+00_rb,2.7797e+00_rb,2.3168e+00_rb,1.8539e+00_rb,1.3914e+00_rb, &
     &9.2912e-01_rb,4.6800e-01_rb,1.1435e-02_rb/)
      kao(:, 3, 8,11) = (/ &
     &4.9130e+00_rb,4.2993e+00_rb,3.6854e+00_rb,3.0718e+00_rb,2.4583e+00_rb,1.8451e+00_rb, &
     &1.2322e+00_rb,6.2028e-01_rb,1.6885e-02_rb/)
      kao(:, 4, 8,11) = (/ &
     &6.2797e+00_rb,5.4950e+00_rb,4.7108e+00_rb,3.9265e+00_rb,3.1420e+00_rb,2.3582e+00_rb, &
     &1.5746e+00_rb,7.9222e-01_rb,2.4688e-02_rb/)
      kao(:, 5, 8,11) = (/ &
     &7.7638e+00_rb,6.7943e+00_rb,5.8243e+00_rb,4.8546e+00_rb,3.8850e+00_rb,2.9154e+00_rb, &
     &1.9466e+00_rb,9.7896e-01_rb,3.5101e-02_rb/)
      kao(:, 1, 9,11) = (/ &
     &9.7135e+00_rb,8.4998e+00_rb,7.2860e+00_rb,6.0723e+00_rb,4.8583e+00_rb,3.6448e+00_rb, &
     &2.4304e+00_rb,1.2171e+00_rb,3.5233e-03_rb/)
      kao(:, 2, 9,11) = (/ &
     &1.3929e+01_rb,1.2189e+01_rb,1.0449e+01_rb,8.7078e+00_rb,6.9673e+00_rb,5.2263e+00_rb, &
     &3.4856e+00_rb,1.7453e+00_rb,4.1308e-03_rb/)
      kao(:, 3, 9,11) = (/ &
     &1.8989e+01_rb,1.6615e+01_rb,1.4243e+01_rb,1.1869e+01_rb,9.4966e+00_rb,7.1244e+00_rb, &
     &4.7516e+00_rb,2.3792e+00_rb,6.6319e-03_rb/)
      kao(:, 4, 9,11) = (/ &
     &2.4785e+01_rb,2.1688e+01_rb,1.8591e+01_rb,1.5493e+01_rb,1.2396e+01_rb,9.2991e+00_rb, &
     &6.2019e+00_rb,3.1051e+00_rb,1.1490e-02_rb/)
      kao(:, 5, 9,11) = (/ &
     &3.1283e+01_rb,2.7373e+01_rb,2.3465e+01_rb,1.9556e+01_rb,1.5646e+01_rb,1.1737e+01_rb, &
     &7.8279e+00_rb,3.9188e+00_rb,2.1019e-02_rb/)
      kao(:, 1,10,11) = (/ &
     &4.1666e+01_rb,3.6458e+01_rb,3.1251e+01_rb,2.6043e+01_rb,2.0835e+01_rb,1.5628e+01_rb, &
     &1.0419e+01_rb,5.2119e+00_rb,1.6494e-03_rb/)
      kao(:, 2,10,11) = (/ &
     &6.1694e+01_rb,5.3987e+01_rb,4.6271e+01_rb,3.8561e+01_rb,3.0850e+01_rb,2.3139e+01_rb, &
     &1.5428e+01_rb,7.7161e+00_rb,4.8320e-03_rb/)
      kao(:, 3,10,11) = (/ &
     &8.6109e+01_rb,7.5347e+01_rb,6.4581e+01_rb,5.3821e+01_rb,4.3056e+01_rb,3.2294e+01_rb, &
     &2.1533e+01_rb,1.0770e+01_rb,3.2818e-03_rb/)
      kao(:, 4,10,11) = (/ &
     &1.1458e+02_rb,1.0025e+02_rb,8.5946e+01_rb,7.1620e+01_rb,5.7298e+01_rb,4.2975e+01_rb, &
     &2.8651e+01_rb,1.4330e+01_rb,1.0923e-02_rb/)
      kao(:, 5,10,11) = (/ &
     &1.4689e+02_rb,1.2853e+02_rb,1.1017e+02_rb,9.1808e+01_rb,7.3452e+01_rb,5.5088e+01_rb, &
     &3.6731e+01_rb,1.8371e+01_rb,1.7421e-02_rb/)
      kao(:, 1,11,11) = (/ &
     &7.6402e+01_rb,6.6855e+01_rb,5.7300e+01_rb,4.7753e+01_rb,3.8204e+01_rb,2.8655e+01_rb, &
     &1.9105e+01_rb,9.5538e+00_rb,4.1075e-05_rb/)
      kao(:, 2,11,11) = (/ &
     &1.1253e+02_rb,9.8463e+01_rb,8.4399e+01_rb,7.0331e+01_rb,5.6269e+01_rb,4.2202e+01_rb, &
     &2.8136e+01_rb,1.4070e+01_rb,5.7635e-05_rb/)
      kao(:, 3,11,11) = (/ &
     &1.5636e+02_rb,1.3680e+02_rb,1.1725e+02_rb,9.7724e+01_rb,7.8173e+01_rb,5.8632e+01_rb, &
     &3.9090e+01_rb,1.9548e+01_rb,8.0706e-05_rb/)
      kao(:, 4,11,11) = (/ &
     &2.0774e+02_rb,1.8177e+02_rb,1.5580e+02_rb,1.2984e+02_rb,1.0387e+02_rb,7.7908e+01_rb, &
     &5.1937e+01_rb,2.5972e+01_rb,1.1256e-02_rb/)
      kao(:, 5,11,11) = (/ &
     &2.6504e+02_rb,2.3192e+02_rb,1.9880e+02_rb,1.6566e+02_rb,1.3254e+02_rb,9.9402e+01_rb, &
     &6.6269e+01_rb,3.3138e+01_rb,8.3317e-03_rb/)
      kao(:, 1,12,11) = (/ &
     &1.0209e+02_rb,8.9323e+01_rb,7.6564e+01_rb,6.3800e+01_rb,5.1042e+01_rb,3.8285e+01_rb, &
     &2.5522e+01_rb,1.2763e+01_rb,2.5628e-05_rb/)
      kao(:, 2,12,11) = (/ &
     &1.4952e+02_rb,1.3083e+02_rb,1.1214e+02_rb,9.3447e+01_rb,7.4760e+01_rb,5.6068e+01_rb, &
     &3.7381e+01_rb,1.8692e+01_rb,3.7727e-05_rb/)
      kao(:, 3,12,11) = (/ &
     &2.0709e+02_rb,1.8121e+02_rb,1.5533e+02_rb,1.2944e+02_rb,1.0355e+02_rb,7.7656e+01_rb, &
     &5.1777e+01_rb,2.5887e+01_rb,4.7198e-05_rb/)
      kao(:, 4,12,11) = (/ &
     &2.7380e+02_rb,2.3961e+02_rb,2.0536e+02_rb,1.7114e+02_rb,1.3691e+02_rb,1.0268e+02_rb, &
     &6.8461e+01_rb,3.4230e+01_rb,7.7074e-03_rb/)
      kao(:, 5,12,11) = (/ &
     &3.4895e+02_rb,3.0533e+02_rb,2.6172e+02_rb,2.1811e+02_rb,1.7449e+02_rb,1.3086e+02_rb, &
     &8.7241e+01_rb,4.3626e+01_rb,8.7565e-03_rb/)
      kao(:, 1,13,11) = (/ &
     &1.0675e+02_rb,9.3411e+01_rb,8.0067e+01_rb,6.6725e+01_rb,5.3379e+01_rb,4.0032e+01_rb, &
     &2.6688e+01_rb,1.3347e+01_rb,1.6230e-05_rb/)
      kao(:, 2,13,11) = (/ &
     &1.5608e+02_rb,1.3657e+02_rb,1.1706e+02_rb,9.7552e+01_rb,7.8046e+01_rb,5.8531e+01_rb, &
     &3.9024e+01_rb,1.9513e+01_rb,2.2466e-05_rb/)
      kao(:, 3,13,11) = (/ &
     &2.1549e+02_rb,1.8855e+02_rb,1.6162e+02_rb,1.3468e+02_rb,1.0774e+02_rb,8.0803e+01_rb, &
     &5.3874e+01_rb,2.6939e+01_rb,3.0542e-05_rb/)
      kao(:, 4,13,11) = (/ &
     &2.8371e+02_rb,2.4826e+02_rb,2.1280e+02_rb,1.7733e+02_rb,1.4186e+02_rb,1.0640e+02_rb, &
     &7.0935e+01_rb,3.5470e+01_rb,1.3758e-03_rb/)
      kao(:, 5,13,11) = (/ &
     &3.5962e+02_rb,3.1467e+02_rb,2.6973e+02_rb,2.2478e+02_rb,1.7981e+02_rb,1.3487e+02_rb, &
     &8.9910e+01_rb,4.4960e+01_rb,3.9903e-03_rb/)
      kao(:, 1, 1,12) = (/ &
     &9.6318e-02_rb,8.4306e-02_rb,7.2320e-02_rb,6.0347e-02_rb,5.2079e-02_rb,4.4235e-02_rb, &
     &3.5296e-02_rb,3.0172e-02_rb,3.8372e-02_rb/)
      kao(:, 2, 1,12) = (/ &
     &1.1333e-01_rb,9.9268e-02_rb,8.5214e-02_rb,7.2014e-02_rb,6.3588e-02_rb,5.4323e-02_rb, &
     &4.4306e-02_rb,4.1727e-02_rb,5.1292e-02_rb/)
      kao(:, 3, 1,12) = (/ &
     &1.3065e-01_rb,1.1448e-01_rb,9.8324e-02_rb,8.5358e-02_rb,7.7141e-02_rb,6.4277e-02_rb, &
     &5.5750e-02_rb,5.5415e-02_rb,6.7933e-02_rb/)
      kao(:, 4, 1,12) = (/ &
     &1.4795e-01_rb,1.2969e-01_rb,1.1146e-01_rb,1.0185e-01_rb,9.0045e-02_rb,7.5178e-02_rb, &
     &6.8885e-02_rb,7.0700e-02_rb,8.8138e-02_rb/)
      kao(:, 5, 1,12) = (/ &
     &1.6491e-01_rb,1.4461e-01_rb,1.2472e-01_rb,1.1761e-01_rb,1.0268e-01_rb,8.7802e-02_rb, &
     &8.5057e-02_rb,9.0208e-02_rb,1.1284e-01_rb/)
      kao(:, 1, 2,12) = (/ &
     &1.4316e-01_rb,1.2534e-01_rb,1.0752e-01_rb,8.9701e-02_rb,7.2577e-02_rb,5.9608e-02_rb, &
     &4.6628e-02_rb,3.2233e-02_rb,3.6922e-02_rb/)
      kao(:, 2, 2,12) = (/ &
     &1.7023e-01_rb,1.4907e-01_rb,1.2790e-01_rb,1.0674e-01_rb,8.8738e-02_rb,7.3250e-02_rb, &
     &5.7353e-02_rb,4.3254e-02_rb,4.9659e-02_rb/)
      kao(:, 3, 2,12) = (/ &
     &1.9792e-01_rb,1.7332e-01_rb,1.4875e-01_rb,1.2418e-01_rb,1.0651e-01_rb,8.8601e-02_rb, &
     &6.9038e-02_rb,5.5574e-02_rb,6.6062e-02_rb/)
      kao(:, 4, 2,12) = (/ &
     &2.2567e-01_rb,1.9768e-01_rb,1.6971e-01_rb,1.4278e-01_rb,1.2696e-01_rb,1.0366e-01_rb, &
     &8.1948e-02_rb,7.1191e-02_rb,8.6127e-02_rb/)
      kao(:, 5, 2,12) = (/ &
     &2.5314e-01_rb,2.2175e-01_rb,1.9032e-01_rb,1.6565e-01_rb,1.4606e-01_rb,1.1970e-01_rb, &
     &9.8165e-02_rb,9.0526e-02_rb,1.1061e-01_rb/)
      kao(:, 1, 3,12) = (/ &
     &2.7385e-01_rb,2.3971e-01_rb,2.0552e-01_rb,1.7136e-01_rb,1.3716e-01_rb,1.0300e-01_rb, &
     &7.5178e-02_rb,4.5691e-02_rb,3.3533e-02_rb/)
      kao(:, 2, 3,12) = (/ &
     &3.3090e-01_rb,2.8961e-01_rb,2.4836e-01_rb,2.0707e-01_rb,1.6581e-01_rb,1.2597e-01_rb, &
     &9.2542e-02_rb,5.7168e-02_rb,4.5647e-02_rb/)
      kao(:, 3, 3,12) = (/ &
     &3.8960e-01_rb,3.4098e-01_rb,2.9239e-01_rb,2.4379e-01_rb,1.9517e-01_rb,1.5181e-01_rb, &
     &1.1308e-01_rb,7.0212e-02_rb,6.1610e-02_rb/)
      kao(:, 4, 3,12) = (/ &
     &4.4888e-01_rb,3.9285e-01_rb,3.3686e-01_rb,2.8085e-01_rb,2.2487e-01_rb,1.8074e-01_rb, &
     &1.3367e-01_rb,8.4795e-02_rb,8.0803e-02_rb/)
      kao(:, 5, 3,12) = (/ &
     &5.0777e-01_rb,4.4444e-01_rb,3.8104e-01_rb,3.1773e-01_rb,2.5631e-01_rb,2.1146e-01_rb, &
     &1.5502e-01_rb,1.0449e-01_rb,1.0410e-01_rb/)
      kao(:, 1, 4,12) = (/ &
     &5.2233e-01_rb,4.5708e-01_rb,3.9182e-01_rb,3.2654e-01_rb,2.6131e-01_rb,1.9604e-01_rb, &
     &1.3079e-01_rb,7.4640e-02_rb,2.9498e-02_rb/)
      kao(:, 2, 4,12) = (/ &
     &6.4217e-01_rb,5.6200e-01_rb,4.8174e-01_rb,4.0151e-01_rb,3.2131e-01_rb,2.4104e-01_rb, &
     &1.6124e-01_rb,9.3279e-02_rb,4.1076e-02_rb/)
      kao(:, 3, 4,12) = (/ &
     &7.6818e-01_rb,6.7216e-01_rb,5.7628e-01_rb,4.8026e-01_rb,3.8431e-01_rb,2.8834e-01_rb, &
     &1.9584e-01_rb,1.1499e-01_rb,5.5764e-02_rb/)
      kao(:, 4, 4,12) = (/ &
     &8.9714e-01_rb,7.8510e-01_rb,6.7304e-01_rb,5.6095e-01_rb,4.4882e-01_rb,3.3677e-01_rb, &
     &2.3398e-01_rb,1.3732e-01_rb,7.3695e-02_rb/)
      kao(:, 5, 4,12) = (/ &
     &1.0267e+00_rb,8.9838e-01_rb,7.7018e-01_rb,6.4193e-01_rb,5.1368e-01_rb,3.8545e-01_rb, &
     &2.7687e-01_rb,1.6073e-01_rb,9.6046e-02_rb/)
      kao(:, 1, 5,12) = (/ &
     &9.0551e-01_rb,7.9235e-01_rb,6.7918e-01_rb,5.6601e-01_rb,4.5286e-01_rb,3.3967e-01_rb, &
     &2.2649e-01_rb,1.1543e-01_rb,2.5575e-02_rb/)
      kao(:, 2, 5,12) = (/ &
     &1.1369e+00_rb,9.9490e-01_rb,8.5273e-01_rb,7.1065e-01_rb,5.6855e-01_rb,4.2647e-01_rb, &
     &2.8441e-01_rb,1.4706e-01_rb,3.6472e-02_rb/)
      kao(:, 3, 5,12) = (/ &
     &1.3835e+00_rb,1.2106e+00_rb,1.0376e+00_rb,8.6468e-01_rb,6.9181e-01_rb,5.1896e-01_rb, &
     &3.4607e-01_rb,1.8250e-01_rb,5.0201e-02_rb/)
      kao(:, 4, 5,12) = (/ &
     &1.6400e+00_rb,1.4350e+00_rb,1.2302e+00_rb,1.0252e+00_rb,8.2021e-01_rb,6.1526e-01_rb, &
     &4.1028e-01_rb,2.2034e-01_rb,6.6839e-02_rb/)
      kao(:, 5, 5,12) = (/ &
     &1.9010e+00_rb,1.6635e+00_rb,1.4259e+00_rb,1.1884e+00_rb,9.5076e-01_rb,7.1323e-01_rb, &
     &4.7563e-01_rb,2.6201e-01_rb,8.7568e-02_rb/)
      kao(:, 1, 6,12) = (/ &
     &1.4399e+00_rb,1.2599e+00_rb,1.0800e+00_rb,9.0001e-01_rb,7.2000e-01_rb,5.4003e-01_rb, &
     &3.6008e-01_rb,1.8026e-01_rb,2.1293e-02_rb/)
      kao(:, 2, 6,12) = (/ &
     &1.8513e+00_rb,1.6198e+00_rb,1.3886e+00_rb,1.1570e+00_rb,9.2575e-01_rb,6.9429e-01_rb, &
     &4.6294e-01_rb,2.3182e-01_rb,3.1128e-02_rb/)
      kao(:, 3, 6,12) = (/ &
     &2.3002e+00_rb,2.0128e+00_rb,1.7253e+00_rb,1.4377e+00_rb,1.1502e+00_rb,8.6271e-01_rb, &
     &5.7522e-01_rb,2.8786e-01_rb,4.3673e-02_rb/)
      kao(:, 4, 6,12) = (/ &
     &2.7724e+00_rb,2.4259e+00_rb,2.0792e+00_rb,1.7327e+00_rb,1.3862e+00_rb,1.0398e+00_rb, &
     &6.9321e-01_rb,3.4832e-01_rb,5.9465e-02_rb/)
      kao(:, 5, 6,12) = (/ &
     &3.2575e+00_rb,2.8504e+00_rb,2.4432e+00_rb,2.0359e+00_rb,1.6288e+00_rb,1.2216e+00_rb, &
     &8.1457e-01_rb,4.1431e-01_rb,7.8852e-02_rb/)
      kao(:, 1, 7,12) = (/ &
     &2.3706e+00_rb,2.0741e+00_rb,1.7778e+00_rb,1.4816e+00_rb,1.1852e+00_rb,8.8892e-01_rb, &
     &5.9261e-01_rb,2.9636e-01_rb,1.6814e-02_rb/)
      kao(:, 2, 7,12) = (/ &
     &3.1315e+00_rb,2.7398e+00_rb,2.3484e+00_rb,1.9572e+00_rb,1.5657e+00_rb,1.1742e+00_rb, &
     &7.8297e-01_rb,3.9150e-01_rb,2.5425e-02_rb/)
      kao(:, 3, 7,12) = (/ &
     &3.9698e+00_rb,3.4734e+00_rb,2.9772e+00_rb,2.4811e+00_rb,1.9850e+00_rb,1.4887e+00_rb, &
     &9.9245e-01_rb,4.9628e-01_rb,3.6698e-02_rb/)
      kao(:, 4, 7,12) = (/ &
     &4.8729e+00_rb,4.2639e+00_rb,3.6548e+00_rb,3.0457e+00_rb,2.4366e+00_rb,1.8275e+00_rb, &
     &1.2184e+00_rb,6.0927e-01_rb,5.1098e-02_rb/)
      kao(:, 5, 7,12) = (/ &
     &5.8302e+00_rb,5.1015e+00_rb,4.3726e+00_rb,3.6438e+00_rb,2.9154e+00_rb,2.1865e+00_rb, &
     &1.4578e+00_rb,7.2896e-01_rb,6.9201e-02_rb/)
      kao(:, 1, 8,12) = (/ &
     &4.6083e+00_rb,4.0328e+00_rb,3.4566e+00_rb,2.8804e+00_rb,2.3046e+00_rb,1.7283e+00_rb, &
     &1.1523e+00_rb,5.7615e-01_rb,1.2896e-02_rb/)
      kao(:, 2, 8,12) = (/ &
     &6.2568e+00_rb,5.4739e+00_rb,4.6922e+00_rb,3.9099e+00_rb,3.1281e+00_rb,2.3461e+00_rb, &
     &1.5641e+00_rb,7.8209e-01_rb,1.9880e-02_rb/)
      kao(:, 3, 8,12) = (/ &
     &8.1358e+00_rb,7.1187e+00_rb,6.1015e+00_rb,5.0849e+00_rb,4.0679e+00_rb,3.0509e+00_rb, &
     &2.0338e+00_rb,1.0170e+00_rb,2.9497e-02_rb/)
      kao(:, 4, 8,12) = (/ &
     &1.0204e+01_rb,8.9284e+00_rb,7.6531e+00_rb,6.3773e+00_rb,5.1020e+00_rb,3.8269e+00_rb, &
     &2.5510e+00_rb,1.2755e+00_rb,4.2115e-02_rb/)
      kao(:, 5, 8,12) = (/ &
     &1.2434e+01_rb,1.0880e+01_rb,9.3260e+00_rb,7.7708e+00_rb,6.2169e+00_rb,4.6632e+00_rb, &
     &3.1088e+00_rb,1.5543e+00_rb,5.8665e-02_rb/)
      kao(:, 1, 9,12) = (/ &
     &1.6573e+01_rb,1.4502e+01_rb,1.2430e+01_rb,1.0358e+01_rb,8.2865e+00_rb,6.2149e+00_rb, &
     &4.1429e+00_rb,2.0717e+00_rb,3.2506e-03_rb/)
      kao(:, 2, 9,12) = (/ &
     &2.3241e+01_rb,2.0336e+01_rb,1.7432e+01_rb,1.4525e+01_rb,1.1622e+01_rb,8.7162e+00_rb, &
     &5.8106e+00_rb,2.9051e+00_rb,8.0054e-03_rb/)
      kao(:, 3, 9,12) = (/ &
     &3.1035e+01_rb,2.7156e+01_rb,2.3277e+01_rb,1.9396e+01_rb,1.5516e+01_rb,1.1638e+01_rb, &
     &7.7589e+00_rb,3.8795e+00_rb,1.2478e-02_rb/)
      kao(:, 4, 9,12) = (/ &
     &3.9794e+01_rb,3.4820e+01_rb,2.9846e+01_rb,2.4871e+01_rb,1.9897e+01_rb,1.4923e+01_rb, &
     &9.9485e+00_rb,4.9745e+00_rb,2.7934e-02_rb/)
      kao(:, 5, 9,12) = (/ &
     &4.9259e+01_rb,4.3099e+01_rb,3.6943e+01_rb,3.0785e+01_rb,2.4628e+01_rb,1.8472e+01_rb, &
     &1.2315e+01_rb,6.1576e+00_rb,4.7661e-02_rb/)
      kao(:, 1,10,12) = (/ &
     &6.7160e+01_rb,5.8765e+01_rb,5.0369e+01_rb,4.1974e+01_rb,3.3581e+01_rb,2.5186e+01_rb, &
     &1.6790e+01_rb,8.3953e+00_rb,3.7061e-05_rb/)
      kao(:, 2,10,12) = (/ &
     &9.7116e+01_rb,8.4979e+01_rb,7.2826e+01_rb,6.0693e+01_rb,4.8559e+01_rb,3.6416e+01_rb, &
     &2.4277e+01_rb,1.2139e+01_rb,5.3890e-03_rb/)
      kao(:, 3,10,12) = (/ &
     &1.3274e+02_rb,1.1614e+02_rb,9.9552e+01_rb,8.2959e+01_rb,6.6370e+01_rb,4.9775e+01_rb, &
     &3.3183e+01_rb,1.6592e+01_rb,5.5995e-03_rb/)
      kao(:, 4,10,12) = (/ &
     &1.7342e+02_rb,1.5174e+02_rb,1.3008e+02_rb,1.0839e+02_rb,8.6716e+01_rb,6.5036e+01_rb, &
     &4.3354e+01_rb,2.1678e+01_rb,7.3568e-03_rb/)
      kao(:, 5,10,12) = (/ &
     &2.1869e+02_rb,1.9137e+02_rb,1.6402e+02_rb,1.3669e+02_rb,1.0934e+02_rb,8.2010e+01_rb, &
     &5.4674e+01_rb,2.7339e+01_rb,1.9171e-02_rb/)
      kao(:, 1,11,12) = (/ &
     &1.1512e+02_rb,1.0073e+02_rb,8.6335e+01_rb,7.1940e+01_rb,5.7559e+01_rb,4.3164e+01_rb, &
     &2.8776e+01_rb,1.4390e+01_rb,3.3905e-05_rb/)
      kao(:, 2,11,12) = (/ &
     &1.6697e+02_rb,1.4609e+02_rb,1.2523e+02_rb,1.0436e+02_rb,8.3482e+01_rb,6.2612e+01_rb, &
     &4.1744e+01_rb,2.0873e+01_rb,5.9590e-05_rb/)
      kao(:, 3,11,12) = (/ &
     &2.2930e+02_rb,2.0064e+02_rb,1.7196e+02_rb,1.4331e+02_rb,1.1466e+02_rb,8.5994e+01_rb, &
     &5.7327e+01_rb,2.8668e+01_rb,8.9833e-05_rb/)
      kao(:, 4,11,12) = (/ &
     &3.0132e+02_rb,2.6364e+02_rb,2.2598e+02_rb,1.8835e+02_rb,1.5066e+02_rb,1.1300e+02_rb, &
     &7.5334e+01_rb,3.7671e+01_rb,8.4167e-03_rb/)
      kao(:, 5,11,12) = (/ &
     &3.8162e+02_rb,3.3392e+02_rb,2.8622e+02_rb,2.3853e+02_rb,1.9083e+02_rb,1.4312e+02_rb, &
     &9.5410e+01_rb,4.7709e+01_rb,1.8161e-02_rb/)
      kao(:, 1,12,12) = (/ &
     &1.5129e+02_rb,1.3238e+02_rb,1.1347e+02_rb,9.4558e+01_rb,7.5645e+01_rb,5.6733e+01_rb, &
     &3.7823e+01_rb,1.8915e+01_rb,3.6770e-05_rb/)
      kao(:, 2,12,12) = (/ &
     &2.2053e+02_rb,1.9297e+02_rb,1.6541e+02_rb,1.3784e+02_rb,1.1026e+02_rb,8.2702e+01_rb, &
     &5.5137e+01_rb,2.7573e+01_rb,5.2389e-05_rb/)
      kao(:, 3,12,12) = (/ &
     &3.0464e+02_rb,2.6657e+02_rb,2.2849e+02_rb,1.9040e+02_rb,1.5233e+02_rb,1.1424e+02_rb, &
     &7.6159e+01_rb,3.8090e+01_rb,7.1306e-05_rb/)
      kao(:, 4,12,12) = (/ &
     &4.0144e+02_rb,3.5128e+02_rb,3.0110e+02_rb,2.5092e+02_rb,2.0072e+02_rb,1.5055e+02_rb, &
     &1.0037e+02_rb,5.0190e+01_rb,9.4766e-05_rb/)
      kao(:, 5,12,12) = (/ &
     &5.1041e+02_rb,4.4660e+02_rb,3.8281e+02_rb,3.1900e+02_rb,2.5521e+02_rb,1.9140e+02_rb, &
     &1.2761e+02_rb,6.3809e+01_rb,8.9549e-03_rb/)
      kao(:, 1,13,12) = (/ &
     &1.6406e+02_rb,1.4356e+02_rb,1.2306e+02_rb,1.0255e+02_rb,8.2037e+01_rb,6.1534e+01_rb, &
     &4.1023e+01_rb,2.0513e+01_rb,2.9105e-05_rb/)
      kao(:, 2,13,12) = (/ &
     &2.3966e+02_rb,2.0971e+02_rb,1.7976e+02_rb,1.4978e+02_rb,1.1983e+02_rb,8.9879e+01_rb, &
     &5.9920e+01_rb,2.9966e+01_rb,3.6736e-05_rb/)
      kao(:, 3,13,12) = (/ &
     &3.3040e+02_rb,2.8909e+02_rb,2.4781e+02_rb,2.0651e+02_rb,1.6521e+02_rb,1.2390e+02_rb, &
     &8.2608e+01_rb,4.1307e+01_rb,4.7139e-05_rb/)
      kao(:, 4,13,12) = (/ &
     &4.3608e+02_rb,3.8160e+02_rb,3.2708e+02_rb,2.7256e+02_rb,2.1804e+02_rb,1.6354e+02_rb, &
     &1.0903e+02_rb,5.4517e+01_rb,5.9953e-05_rb/)
      kao(:, 5,13,12) = (/ &
     &5.5401e+02_rb,4.8478e+02_rb,4.1553e+02_rb,3.4627e+02_rb,2.7701e+02_rb,2.0776e+02_rb, &
     &1.3851e+02_rb,6.9261e+01_rb,1.3826e-02_rb/)
      kao(:, 1, 1,13) = (/ &
     &1.2490e-01_rb,1.0932e-01_rb,9.3720e-02_rb,7.8121e-02_rb,6.3794e-02_rb,5.1600e-02_rb, &
     &4.4680e-02_rb,4.2635e-02_rb,5.5634e-02_rb/)
      kao(:, 2, 1,13) = (/ &
     &1.4548e-01_rb,1.2733e-01_rb,1.0918e-01_rb,9.1021e-02_rb,7.6620e-02_rb,6.3500e-02_rb, &
     &5.7581e-02_rb,5.7524e-02_rb,7.4793e-02_rb/)
      kao(:, 3, 1,13) = (/ &
     &1.6600e-01_rb,1.4529e-01_rb,1.2460e-01_rb,1.0392e-01_rb,9.0379e-02_rb,7.8694e-02_rb, &
     &7.4384e-02_rb,7.8232e-02_rb,9.9775e-02_rb/)
      kao(:, 4, 1,13) = (/ &
     &1.8636e-01_rb,1.6314e-01_rb,1.3994e-01_rb,1.1863e-01_rb,1.0661e-01_rb,9.7110e-02_rb, &
     &9.8012e-02_rb,1.0654e-01_rb,1.3265e-01_rb/)
      kao(:, 5, 1,13) = (/ &
     &2.0606e-01_rb,1.8040e-01_rb,1.5476e-01_rb,1.3669e-01_rb,1.2405e-01_rb,1.1873e-01_rb, &
     &1.2869e-01_rb,1.4070e-01_rb,1.7529e-01_rb/)
      kao(:, 1, 2,13) = (/ &
     &1.8895e-01_rb,1.6535e-01_rb,1.4175e-01_rb,1.1814e-01_rb,9.4551e-02_rb,7.3809e-02_rb, &
     &5.5615e-02_rb,4.4637e-02_rb,5.3030e-02_rb/)
      kao(:, 2, 2,13) = (/ &
     &2.2250e-01_rb,1.9470e-01_rb,1.6692e-01_rb,1.3914e-01_rb,1.1136e-01_rb,8.8262e-02_rb, &
     &7.0022e-02_rb,6.0363e-02_rb,7.3472e-02_rb/)
      kao(:, 3, 2,13) = (/ &
     &2.5567e-01_rb,2.2376e-01_rb,1.9185e-01_rb,1.5991e-01_rb,1.3005e-01_rb,1.0599e-01_rb, &
     &8.7655e-02_rb,7.9726e-02_rb,9.6994e-02_rb/)
      kao(:, 4, 2,13) = (/ &
     &2.8747e-01_rb,2.5157e-01_rb,2.1569e-01_rb,1.7980e-01_rb,1.5083e-01_rb,1.2699e-01_rb, &
     &1.0917e-01_rb,1.0493e-01_rb,1.2703e-01_rb/)
      kao(:, 5, 2,13) = (/ &
     &3.1781e-01_rb,2.7817e-01_rb,2.3857e-01_rb,1.9898e-01_rb,1.7411e-01_rb,1.5016e-01_rb, &
     &1.3454e-01_rb,1.3707e-01_rb,1.6656e-01_rb/)
      kao(:, 1, 3,13) = (/ &
     &3.6978e-01_rb,3.2357e-01_rb,2.7737e-01_rb,2.3116e-01_rb,1.8494e-01_rb,1.3872e-01_rb, &
     &9.4432e-02_rb,5.7465e-02_rb,4.7026e-02_rb/)
      kao(:, 2, 3,13) = (/ &
     &4.3903e-01_rb,3.8415e-01_rb,3.2929e-01_rb,2.7445e-01_rb,2.1956e-01_rb,1.6470e-01_rb, &
     &1.1482e-01_rb,7.4394e-02_rb,6.6138e-02_rb/)
      kao(:, 3, 3,13) = (/ &
     &5.0855e-01_rb,4.4506e-01_rb,3.8154e-01_rb,3.1805e-01_rb,2.5451e-01_rb,1.9102e-01_rb, &
     &1.3670e-01_rb,9.4825e-02_rb,8.9406e-02_rb/)
      kao(:, 4, 3,13) = (/ &
     &5.7817e-01_rb,5.0599e-01_rb,4.3381e-01_rb,3.6166e-01_rb,2.8948e-01_rb,2.2042e-01_rb, &
     &1.6420e-01_rb,1.1827e-01_rb,1.1730e-01_rb/)
      kao(:, 5, 3,13) = (/ &
     &6.4715e-01_rb,5.6638e-01_rb,4.8566e-01_rb,4.0494e-01_rb,3.2419e-01_rb,2.5446e-01_rb, &
     &1.9367e-01_rb,1.4612e-01_rb,1.5219e-01_rb/)
      kao(:, 1, 4,13) = (/ &
     &7.1857e-01_rb,6.2877e-01_rb,5.3894e-01_rb,4.4919e-01_rb,3.5935e-01_rb,2.6957e-01_rb, &
     &1.7976e-01_rb,9.3998e-02_rb,4.1187e-02_rb/)
      kao(:, 2, 4,13) = (/ &
     &8.6835e-01_rb,7.5986e-01_rb,6.5131e-01_rb,5.4283e-01_rb,4.3435e-01_rb,3.2583e-01_rb, &
     &2.1731e-01_rb,1.1522e-01_rb,5.8302e-02_rb/)
      kao(:, 3, 4,13) = (/ &
     &1.0243e+00_rb,8.9626e-01_rb,7.6836e-01_rb,6.4035e-01_rb,5.1240e-01_rb,3.8438e-01_rb, &
     &2.5643e-01_rb,1.3949e-01_rb,7.9878e-02_rb/)
      kao(:, 4, 4,13) = (/ &
     &1.1800e+00_rb,1.0326e+00_rb,8.8512e-01_rb,7.3767e-01_rb,5.9020e-01_rb,4.4282e-01_rb, &
     &2.9609e-01_rb,1.6997e-01_rb,1.0651e-01_rb/)
      kao(:, 5, 4,13) = (/ &
     &1.3277e+00_rb,1.1619e+00_rb,9.9598e-01_rb,8.3009e-01_rb,6.6410e-01_rb,4.9823e-01_rb, &
     &3.3917e-01_rb,2.0392e-01_rb,1.3923e-01_rb/)
      kao(:, 1, 5,13) = (/ &
     &1.2769e+00_rb,1.1173e+00_rb,9.5773e-01_rb,7.9808e-01_rb,6.3854e-01_rb,4.7892e-01_rb, &
     &3.1932e-01_rb,1.5974e-01_rb,3.6366e-02_rb/)
      kao(:, 2, 5,13) = (/ &
     &1.5754e+00_rb,1.3783e+00_rb,1.1815e+00_rb,9.8457e-01_rb,7.8769e-01_rb,5.9081e-01_rb, &
     &3.9400e-01_rb,1.9711e-01_rb,5.2068e-02_rb/)
      kao(:, 3, 5,13) = (/ &
     &1.8818e+00_rb,1.6466e+00_rb,1.4114e+00_rb,1.1761e+00_rb,9.4096e-01_rb,7.0577e-01_rb, &
     &4.7057e-01_rb,2.3659e-01_rb,7.1428e-02_rb/)
      kao(:, 4, 5,13) = (/ &
     &2.1862e+00_rb,1.9130e+00_rb,1.6397e+00_rb,1.3666e+00_rb,1.0933e+00_rb,8.2000e-01_rb, &
     &5.4672e-01_rb,2.8019e-01_rb,9.6282e-02_rb/)
      kao(:, 5, 5,13) = (/ &
     &2.4918e+00_rb,2.1806e+00_rb,1.8691e+00_rb,1.5576e+00_rb,1.2460e+00_rb,9.3474e-01_rb, &
     &6.2322e-01_rb,3.2775e-01_rb,1.2758e-01_rb/)
      kao(:, 1, 6,13) = (/ &
     &2.0873e+00_rb,1.8265e+00_rb,1.5654e+00_rb,1.3046e+00_rb,1.0437e+00_rb,7.8276e-01_rb, &
     &5.2187e-01_rb,2.6100e-01_rb,3.1022e-02_rb/)
      kao(:, 2, 6,13) = (/ &
     &2.6230e+00_rb,2.2953e+00_rb,1.9673e+00_rb,1.6394e+00_rb,1.3116e+00_rb,9.8380e-01_rb, &
     &6.5587e-01_rb,3.2796e-01_rb,4.5564e-02_rb/)
      kao(:, 3, 6,13) = (/ &
     &3.1789e+00_rb,2.7816e+00_rb,2.3843e+00_rb,1.9870e+00_rb,1.5894e+00_rb,1.1921e+00_rb, &
     &7.9481e-01_rb,3.9748e-01_rb,6.4081e-02_rb/)
      kao(:, 4, 6,13) = (/ &
     &3.7665e+00_rb,3.2952e+00_rb,2.8247e+00_rb,2.3538e+00_rb,1.8832e+00_rb,1.4124e+00_rb, &
     &9.4165e-01_rb,4.7095e-01_rb,8.7003e-02_rb/)
      kao(:, 5, 6,13) = (/ &
     &4.3632e+00_rb,3.8180e+00_rb,3.2726e+00_rb,2.7273e+00_rb,2.1819e+00_rb,1.6363e+00_rb, &
     &1.0910e+00_rb,5.4560e-01_rb,1.1609e-01_rb/)
      kao(:, 1, 7,13) = (/ &
     &3.5237e+00_rb,3.0834e+00_rb,2.6428e+00_rb,2.2023e+00_rb,1.7619e+00_rb,1.3214e+00_rb, &
     &8.8102e-01_rb,4.4047e-01_rb,2.5898e-02_rb/)
      kao(:, 2, 7,13) = (/ &
     &4.5331e+00_rb,3.9665e+00_rb,3.3999e+00_rb,2.8333e+00_rb,2.2667e+00_rb,1.6999e+00_rb, &
     &1.1334e+00_rb,5.6671e-01_rb,3.9066e-02_rb/)
      kao(:, 3, 7,13) = (/ &
     &5.6378e+00_rb,4.9326e+00_rb,4.2280e+00_rb,3.5234e+00_rb,2.8189e+00_rb,2.1142e+00_rb, &
     &1.4094e+00_rb,7.0475e-01_rb,5.6679e-02_rb/)
      kao(:, 4, 7,13) = (/ &
     &6.8044e+00_rb,5.9541e+00_rb,5.1033e+00_rb,4.2525e+00_rb,3.4021e+00_rb,2.5518e+00_rb, &
     &1.7012e+00_rb,8.5062e-01_rb,7.8214e-02_rb/)
      kao(:, 5, 7,13) = (/ &
     &7.9748e+00_rb,6.9785e+00_rb,5.9814e+00_rb,4.9841e+00_rb,3.9872e+00_rb,2.9906e+00_rb, &
     &1.9939e+00_rb,9.9695e-01_rb,1.0573e-01_rb/)
      kao(:, 1, 8,13) = (/ &
     &7.0539e+00_rb,6.1726e+00_rb,5.2906e+00_rb,4.4088e+00_rb,3.5267e+00_rb,2.6453e+00_rb, &
     &1.7634e+00_rb,8.8179e-01_rb,2.1249e-02_rb/)
      kao(:, 2, 8,13) = (/ &
     &9.3702e+00_rb,8.1985e+00_rb,7.0273e+00_rb,5.8560e+00_rb,4.6849e+00_rb,3.5137e+00_rb, &
     &2.3424e+00_rb,1.1713e+00_rb,3.3095e-02_rb/)
      kao(:, 3, 8,13) = (/ &
     &1.1943e+01_rb,1.0450e+01_rb,8.9569e+00_rb,7.4646e+00_rb,5.9714e+00_rb,4.4785e+00_rb, &
     &2.9858e+00_rb,1.4929e+00_rb,4.9465e-02_rb/)
      kao(:, 4, 8,13) = (/ &
     &1.4646e+01_rb,1.2815e+01_rb,1.0984e+01_rb,9.1536e+00_rb,7.3228e+00_rb,5.4925e+00_rb, &
     &3.6616e+00_rb,1.8307e+00_rb,7.0099e-02_rb/)
      kao(:, 5, 8,13) = (/ &
     &1.7489e+01_rb,1.5303e+01_rb,1.3117e+01_rb,1.0930e+01_rb,8.7446e+00_rb,6.5589e+00_rb, &
     &4.3724e+00_rb,2.1863e+00_rb,9.5981e-02_rb/)
      kao(:, 1, 9,13) = (/ &
     &2.6371e+01_rb,2.3078e+01_rb,1.9780e+01_rb,1.6484e+01_rb,1.3187e+01_rb,9.8900e+00_rb, &
     &6.5930e+00_rb,3.2967e+00_rb,6.8943e-03_rb/)
      kao(:, 2, 9,13) = (/ &
     &3.6211e+01_rb,3.1684e+01_rb,2.7158e+01_rb,2.2631e+01_rb,1.8105e+01_rb,1.3579e+01_rb, &
     &9.0530e+00_rb,4.5262e+00_rb,1.7485e-02_rb/)
      kao(:, 3, 9,13) = (/ &
     &4.7281e+01_rb,4.1374e+01_rb,3.5462e+01_rb,2.9549e+01_rb,2.3642e+01_rb,1.7731e+01_rb, &
     &1.1820e+01_rb,5.9103e+00_rb,4.0011e-02_rb/)
      kao(:, 4, 9,13) = (/ &
     &5.9381e+01_rb,5.1957e+01_rb,4.4537e+01_rb,3.7114e+01_rb,2.9690e+01_rb,2.2267e+01_rb, &
     &1.4846e+01_rb,7.4229e+00_rb,6.1300e-02_rb/)
      kao(:, 5, 9,13) = (/ &
     &7.2474e+01_rb,6.3419e+01_rb,5.4356e+01_rb,4.5296e+01_rb,3.6239e+01_rb,2.7178e+01_rb, &
     &1.8119e+01_rb,9.0595e+00_rb,8.5682e-02_rb/)
      kao(:, 1,10,13) = (/ &
     &1.1184e+02_rb,9.7849e+01_rb,8.3875e+01_rb,6.9894e+01_rb,5.5915e+01_rb,4.1936e+01_rb, &
     &2.7957e+01_rb,1.3979e+01_rb,1.8093e-05_rb/)
      kao(:, 2,10,13) = (/ &
     &1.5816e+02_rb,1.3839e+02_rb,1.1862e+02_rb,9.8846e+01_rb,7.9081e+01_rb,5.9311e+01_rb, &
     &3.9538e+01_rb,1.9769e+01_rb,2.0368e-03_rb/)
      kao(:, 3,10,13) = (/ &
     &2.1176e+02_rb,1.8530e+02_rb,1.5882e+02_rb,1.3236e+02_rb,1.0588e+02_rb,7.9412e+01_rb, &
     &5.2940e+01_rb,2.6470e+01_rb,1.0379e-02_rb/)
      kao(:, 4,10,13) = (/ &
     &2.7234e+02_rb,2.3829e+02_rb,2.0425e+02_rb,1.7021e+02_rb,1.3616e+02_rb,1.0213e+02_rb, &
     &6.8082e+01_rb,3.4042e+01_rb,8.3550e-03_rb/)
      kao(:, 5,10,13) = (/ &
     &3.3838e+02_rb,2.9608e+02_rb,2.5378e+02_rb,2.1148e+02_rb,1.6919e+02_rb,1.2688e+02_rb, &
     &8.4588e+01_rb,4.2296e+01_rb,2.4237e-02_rb/)
      kao(:, 1,11,13) = (/ &
     &1.9607e+02_rb,1.7156e+02_rb,1.4706e+02_rb,1.2254e+02_rb,9.8035e+01_rb,7.3528e+01_rb, &
     &4.9019e+01_rb,2.4510e+01_rb,1.9563e-05_rb/)
      kao(:, 2,11,13) = (/ &
     &2.7783e+02_rb,2.4308e+02_rb,2.0836e+02_rb,1.7363e+02_rb,1.3891e+02_rb,1.0418e+02_rb, &
     &6.9451e+01_rb,3.4727e+01_rb,2.1737e-05_rb/)
      kao(:, 3,11,13) = (/ &
     &3.7392e+02_rb,3.2720e+02_rb,2.8047e+02_rb,2.3371e+02_rb,1.8697e+02_rb,1.4023e+02_rb, &
     &9.3485e+01_rb,4.6744e+01_rb,2.8098e-05_rb/)
      kao(:, 4,11,13) = (/ &
     &4.8258e+02_rb,4.2223e+02_rb,3.6194e+02_rb,3.0162e+02_rb,2.4128e+02_rb,1.8096e+02_rb, &
     &1.2064e+02_rb,6.0325e+01_rb,6.2823e-03_rb/)
      kao(:, 5,11,13) = (/ &
     &6.0149e+02_rb,5.2626e+02_rb,4.5111e+02_rb,3.7591e+02_rb,3.0076e+02_rb,2.2554e+02_rb, &
     &1.5037e+02_rb,7.5184e+01_rb,1.4409e-02_rb/)
      kao(:, 1,12,13) = (/ &
     &2.5475e+02_rb,2.2289e+02_rb,1.9106e+02_rb,1.5923e+02_rb,1.2737e+02_rb,9.5526e+01_rb, &
     &6.3684e+01_rb,3.1843e+01_rb,2.0642e-05_rb/)
      kao(:, 2,12,13) = (/ &
     &3.6233e+02_rb,3.1704e+02_rb,2.7175e+02_rb,2.2647e+02_rb,1.8117e+02_rb,1.3587e+02_rb, &
     &9.0584e+01_rb,4.5292e+01_rb,3.1983e-05_rb/)
      kao(:, 3,12,13) = (/ &
     &4.8916e+02_rb,4.2803e+02_rb,3.6684e+02_rb,3.0571e+02_rb,2.4456e+02_rb,1.8343e+02_rb, &
     &1.2228e+02_rb,6.1143e+01_rb,4.9821e-05_rb/)
      kao(:, 4,12,13) = (/ &
     &6.3314e+02_rb,5.5397e+02_rb,4.7487e+02_rb,3.9571e+02_rb,3.1654e+02_rb,2.3742e+02_rb, &
     &1.5828e+02_rb,7.9139e+01_rb,7.0156e-05_rb/)
      kao(:, 5,12,13) = (/ &
     &7.9128e+02_rb,6.9231e+02_rb,5.9347e+02_rb,4.9456e+02_rb,3.9565e+02_rb,2.9675e+02_rb, &
     &1.9782e+02_rb,9.8911e+01_rb,2.0440e-02_rb/)
      kao(:, 1,13,13) = (/ &
     &2.6423e+02_rb,2.3120e+02_rb,1.9817e+02_rb,1.6514e+02_rb,1.3211e+02_rb,9.9077e+01_rb, &
     &6.6055e+01_rb,3.3026e+01_rb,2.0316e-05_rb/)
      kao(:, 2,13,13) = (/ &
     &3.7762e+02_rb,3.3041e+02_rb,2.8322e+02_rb,2.3600e+02_rb,1.8882e+02_rb,1.4161e+02_rb, &
     &9.4405e+01_rb,4.7199e+01_rb,3.9220e-05_rb/)
      kao(:, 3,13,13) = (/ &
     &5.1221e+02_rb,4.4817e+02_rb,3.8414e+02_rb,3.2014e+02_rb,2.5611e+02_rb,1.9208e+02_rb, &
     &1.2806e+02_rb,6.4027e+01_rb,5.9611e-05_rb/)
      kao(:, 4,13,13) = (/ &
     &6.6615e+02_rb,5.8290e+02_rb,4.9961e+02_rb,4.1636e+02_rb,3.3309e+02_rb,2.4982e+02_rb, &
     &1.6655e+02_rb,8.3275e+01_rb,8.3600e-05_rb/)
      kao(:, 5,13,13) = (/ &
     &8.3847e+02_rb,7.3367e+02_rb,6.2884e+02_rb,5.2405e+02_rb,4.1925e+02_rb,3.1441e+02_rb, &
     &2.0962e+02_rb,1.0481e+02_rb,8.6814e-03_rb/)
      kao(:, 1, 1,14) = (/ &
     &1.4546e-01_rb,1.2729e-01_rb,1.0913e-01_rb,9.0969e-02_rb,7.2816e-02_rb,6.3783e-02_rb, &
     &5.7732e-02_rb,5.8710e-02_rb,8.0301e-02_rb/)
      kao(:, 2, 1,14) = (/ &
     &1.6927e-01_rb,1.4814e-01_rb,1.2702e-01_rb,1.0589e-01_rb,8.6710e-02_rb,8.1091e-02_rb, &
     &8.1369e-02_rb,8.2706e-02_rb,1.1649e-01_rb/)
      kao(:, 3, 1,14) = (/ &
     &1.9294e-01_rb,1.6889e-01_rb,1.4479e-01_rb,1.2072e-01_rb,1.0647e-01_rb,1.0625e-01_rb, &
     &1.1521e-01_rb,1.1490e-01_rb,1.6620e-01_rb/)
      kao(:, 4, 1,14) = (/ &
     &2.1639e-01_rb,1.8940e-01_rb,1.6240e-01_rb,1.3541e-01_rb,1.3447e-01_rb,1.4239e-01_rb, &
     &1.5920e-01_rb,1.5808e-01_rb,2.2919e-01_rb/)
      kao(:, 5, 1,14) = (/ &
     &2.3882e-01_rb,2.0905e-01_rb,1.7929e-01_rb,1.5673e-01_rb,1.6730e-01_rb,1.8853e-01_rb, &
     &2.1227e-01_rb,2.1249e-01_rb,3.0359e-01_rb/)
      kao(:, 1, 2,14) = (/ &
     &2.2939e-01_rb,2.0073e-01_rb,1.7206e-01_rb,1.4339e-01_rb,1.1475e-01_rb,8.6951e-02_rb, &
     &7.2184e-02_rb,6.3155e-02_rb,8.0463e-02_rb/)
      kao(:, 2, 2,14) = (/ &
     &2.6931e-01_rb,2.3569e-01_rb,2.0200e-01_rb,1.6840e-01_rb,1.3475e-01_rb,1.0880e-01_rb, &
     &9.2729e-02_rb,8.6720e-02_rb,1.1351e-01_rb/)
      kao(:, 3, 2,14) = (/ &
     &3.0958e-01_rb,2.7100e-01_rb,2.3236e-01_rb,1.9370e-01_rb,1.5506e-01_rb,1.3547e-01_rb, &
     &1.2355e-01_rb,1.2394e-01_rb,1.6330e-01_rb/)
      kao(:, 4, 2,14) = (/ &
     &3.5073e-01_rb,3.0694e-01_rb,2.6319e-01_rb,2.1943e-01_rb,1.7928e-01_rb,1.7126e-01_rb, &
     &1.6643e-01_rb,1.7360e-01_rb,2.2783e-01_rb/)
      kao(:, 5, 2,14) = (/ &
     &3.9134e-01_rb,3.4248e-01_rb,2.9368e-01_rb,2.4496e-01_rb,2.1593e-01_rb,2.1367e-01_rb, &
     &2.2253e-01_rb,2.3537e-01_rb,3.0544e-01_rb/)
      kao(:, 1, 3,14) = (/ &
     &4.7010e-01_rb,4.1135e-01_rb,3.5261e-01_rb,2.9387e-01_rb,2.3512e-01_rb,1.7638e-01_rb, &
     &1.1764e-01_rb,7.8616e-02_rb,7.6241e-02_rb/)
      kao(:, 2, 3,14) = (/ &
     &5.6247e-01_rb,4.9221e-01_rb,4.2195e-01_rb,3.5165e-01_rb,2.8136e-01_rb,2.1111e-01_rb, &
     &1.4480e-01_rb,1.0039e-01_rb,1.0523e-01_rb/)
      kao(:, 3, 3,14) = (/ &
     &6.5749e-01_rb,5.7537e-01_rb,4.9319e-01_rb,4.1105e-01_rb,3.2895e-01_rb,2.4684e-01_rb, &
     &1.7889e-01_rb,1.3157e-01_rb,1.4752e-01_rb/)
      kao(:, 4, 3,14) = (/ &
     &7.5292e-01_rb,6.5895e-01_rb,5.6476e-01_rb,4.7081e-01_rb,3.7678e-01_rb,2.8272e-01_rb, &
     &2.2206e-01_rb,1.7756e-01_rb,2.0769e-01_rb/)
      kao(:, 5, 3,14) = (/ &
     &8.4721e-01_rb,7.4147e-01_rb,6.3570e-01_rb,5.2989e-01_rb,4.2402e-01_rb,3.2414e-01_rb, &
     &2.7831e-01_rb,2.3784e-01_rb,2.8406e-01_rb/)
      kao(:, 1, 4,14) = (/ &
     &9.7171e-01_rb,8.5020e-01_rb,7.2877e-01_rb,6.0734e-01_rb,4.8590e-01_rb,3.6445e-01_rb, &
     &2.4301e-01_rb,1.2213e-01_rb,6.9724e-02_rb/)
      kao(:, 2, 4,14) = (/ &
     &1.1819e+00_rb,1.0342e+00_rb,8.8647e-01_rb,7.3863e-01_rb,5.9101e-01_rb,4.4338e-01_rb, &
     &2.9560e-01_rb,1.5522e-01_rb,9.6103e-02_rb/)
      kao(:, 3, 4,14) = (/ &
     &1.3997e+00_rb,1.2249e+00_rb,1.0500e+00_rb,8.7515e-01_rb,7.0004e-01_rb,5.2502e-01_rb, &
     &3.5022e-01_rb,1.9543e-01_rb,1.3302e-01_rb/)
      kao(:, 4, 4,14) = (/ &
     &1.6223e+00_rb,1.4197e+00_rb,1.2167e+00_rb,1.0141e+00_rb,8.1148e-01_rb,6.0893e-01_rb, &
     &4.0620e-01_rb,2.4484e-01_rb,1.8472e-01_rb/)
      kao(:, 5, 4,14) = (/ &
     &1.8526e+00_rb,1.6213e+00_rb,1.3899e+00_rb,1.1585e+00_rb,9.2711e-01_rb,6.9563e-01_rb, &
     &4.6462e-01_rb,3.0719e-01_rb,2.5304e-01_rb/)
      kao(:, 1, 5,14) = (/ &
     &1.8192e+00_rb,1.5916e+00_rb,1.3645e+00_rb,1.1369e+00_rb,9.0965e-01_rb,6.8222e-01_rb, &
     &4.5478e-01_rb,2.2743e-01_rb,6.1385e-02_rb/)
      kao(:, 2, 5,14) = (/ &
     &2.2491e+00_rb,1.9677e+00_rb,1.6869e+00_rb,1.4056e+00_rb,1.1246e+00_rb,8.4332e-01_rb, &
     &5.6233e-01_rb,2.8119e-01_rb,8.6006e-02_rb/)
      kao(:, 3, 5,14) = (/ &
     &2.7102e+00_rb,2.3722e+00_rb,2.0328e+00_rb,1.6939e+00_rb,1.3553e+00_rb,1.0165e+00_rb, &
     &6.7791e-01_rb,3.3919e-01_rb,1.2062e-01_rb/)
      kao(:, 4, 5,14) = (/ &
     &3.1976e+00_rb,2.7982e+00_rb,2.3984e+00_rb,1.9988e+00_rb,1.5996e+00_rb,1.1997e+00_rb, &
     &8.0003e-01_rb,4.0180e-01_rb,1.6649e-01_rb/)
      kao(:, 5, 5,14) = (/ &
     &3.6973e+00_rb,3.2349e+00_rb,2.7730e+00_rb,2.3110e+00_rb,1.8489e+00_rb,1.3873e+00_rb, &
     &9.2506e-01_rb,4.7326e-01_rb,2.2654e-01_rb/)
      kao(:, 1, 6,14) = (/ &
     &3.1019e+00_rb,2.7140e+00_rb,2.3265e+00_rb,1.9387e+00_rb,1.5510e+00_rb,1.1633e+00_rb, &
     &7.7552e-01_rb,3.8777e-01_rb,5.2928e-02_rb/)
      kao(:, 2, 6,14) = (/ &
     &3.9302e+00_rb,3.4386e+00_rb,2.9473e+00_rb,2.4561e+00_rb,1.9652e+00_rb,1.4738e+00_rb, &
     &9.8267e-01_rb,4.9148e-01_rb,7.5058e-02_rb/)
      kao(:, 3, 6,14) = (/ &
     &4.8343e+00_rb,4.2298e+00_rb,3.6258e+00_rb,3.0217e+00_rb,2.4174e+00_rb,1.8133e+00_rb, &
     &1.2090e+00_rb,6.0467e-01_rb,1.0698e-01_rb/)
      kao(:, 4, 6,14) = (/ &
     &5.7912e+00_rb,5.0677e+00_rb,4.3433e+00_rb,3.6195e+00_rb,2.8961e+00_rb,2.1723e+00_rb, &
     &1.4483e+00_rb,7.2440e-01_rb,1.4901e-01_rb/)
      kao(:, 5, 6,14) = (/ &
     &6.7811e+00_rb,5.9338e+00_rb,5.0864e+00_rb,4.2388e+00_rb,3.3910e+00_rb,2.5430e+00_rb, &
     &1.6957e+00_rb,8.4831e-01_rb,2.0161e-01_rb/)
      kao(:, 1, 7,14) = (/ &
     &5.4638e+00_rb,4.7799e+00_rb,4.0972e+00_rb,3.4147e+00_rb,2.7314e+00_rb,2.0488e+00_rb, &
     &1.3659e+00_rb,6.8294e-01_rb,4.5673e-02_rb/)
      kao(:, 2, 7,14) = (/ &
     &7.1063e+00_rb,6.2177e+00_rb,5.3292e+00_rb,4.4410e+00_rb,3.5530e+00_rb,2.6647e+00_rb, &
     &1.7765e+00_rb,8.8835e-01_rb,6.5608e-02_rb/)
      kao(:, 3, 7,14) = (/ &
     &8.9165e+00_rb,7.8008e+00_rb,6.6867e+00_rb,5.5727e+00_rb,4.4582e+00_rb,3.3435e+00_rb, &
     &2.2293e+00_rb,1.1147e+00_rb,9.3558e-02_rb/)
      kao(:, 4, 7,14) = (/ &
     &1.0858e+01_rb,9.5015e+00_rb,8.1441e+00_rb,6.7868e+00_rb,5.4291e+00_rb,4.0729e+00_rb, &
     &2.7150e+00_rb,1.3577e+00_rb,1.3310e-01_rb/)
      kao(:, 5, 7,14) = (/ &
     &1.2930e+01_rb,1.1317e+01_rb,9.6983e+00_rb,8.0819e+00_rb,6.4667e+00_rb,4.8496e+00_rb, &
     &3.2333e+00_rb,1.6168e+00_rb,1.8236e-01_rb/)
      kao(:, 1, 8,14) = (/ &
     &1.1346e+01_rb,9.9283e+00_rb,8.5084e+00_rb,7.0912e+00_rb,5.6732e+00_rb,4.2544e+00_rb, &
     &2.8362e+00_rb,1.4181e+00_rb,3.9157e-02_rb/)
      kao(:, 2, 8,14) = (/ &
     &1.5150e+01_rb,1.3257e+01_rb,1.1363e+01_rb,9.4692e+00_rb,7.5753e+00_rb,5.6816e+00_rb, &
     &3.7878e+00_rb,1.8940e+00_rb,5.7302e-02_rb/)
      kao(:, 3, 8,14) = (/ &
     &1.9415e+01_rb,1.6989e+01_rb,1.4562e+01_rb,1.2134e+01_rb,9.7069e+00_rb,7.2806e+00_rb, &
     &4.8539e+00_rb,2.4270e+00_rb,8.2333e-02_rb/)
      kao(:, 4, 8,14) = (/ &
     &2.4092e+01_rb,2.1079e+01_rb,1.8070e+01_rb,1.5059e+01_rb,1.2046e+01_rb,9.0355e+00_rb, &
     &6.0239e+00_rb,3.0117e+00_rb,1.1787e-01_rb/)
      kao(:, 5, 8,14) = (/ &
     &2.9062e+01_rb,2.5429e+01_rb,2.1796e+01_rb,1.8164e+01_rb,1.4531e+01_rb,1.0899e+01_rb, &
     &7.2663e+00_rb,3.6329e+00_rb,1.6482e-01_rb/)
      kao(:, 1, 9,14) = (/ &
     &4.3556e+01_rb,3.8120e+01_rb,3.2674e+01_rb,2.7224e+01_rb,2.1783e+01_rb,1.6334e+01_rb, &
     &1.0888e+01_rb,5.4440e+00_rb,2.2652e-02_rb/)
      kao(:, 2, 9,14) = (/ &
     &5.9826e+01_rb,5.2350e+01_rb,4.4870e+01_rb,3.7394e+01_rb,2.9916e+01_rb,2.2436e+01_rb, &
     &1.4957e+01_rb,7.4785e+00_rb,4.6893e-02_rb/)
      kao(:, 3, 9,14) = (/ &
     &7.8381e+01_rb,6.8585e+01_rb,5.8789e+01_rb,4.8993e+01_rb,3.9194e+01_rb,2.9396e+01_rb, &
     &1.9597e+01_rb,9.7990e+00_rb,7.2210e-02_rb/)
      kao(:, 4, 9,14) = (/ &
     &9.9048e+01_rb,8.6676e+01_rb,7.4294e+01_rb,6.1915e+01_rb,4.9526e+01_rb,3.7147e+01_rb, &
     &2.4767e+01_rb,1.2383e+01_rb,1.0452e-01_rb/)
      kao(:, 5, 9,14) = (/ &
     &1.2130e+02_rb,1.0614e+02_rb,9.0980e+01_rb,7.5818e+01_rb,6.0656e+01_rb,4.5493e+01_rb, &
     &3.0327e+01_rb,1.5163e+01_rb,1.4798e-01_rb/)
      kao(:, 1,10,14) = (/ &
     &1.8821e+02_rb,1.6469e+02_rb,1.4116e+02_rb,1.1763e+02_rb,9.4105e+01_rb,7.0580e+01_rb, &
     &4.7054e+01_rb,2.3527e+01_rb,9.8284e-06_rb/)
      kao(:, 2,10,14) = (/ &
     &2.6476e+02_rb,2.3166e+02_rb,1.9858e+02_rb,1.6548e+02_rb,1.3239e+02_rb,9.9294e+01_rb, &
     &6.6189e+01_rb,3.3094e+01_rb,1.2967e-05_rb/)
      kao(:, 3,10,14) = (/ &
     &3.5496e+02_rb,3.1059e+02_rb,2.6621e+02_rb,2.2185e+02_rb,1.7748e+02_rb,1.3310e+02_rb, &
     &8.8736e+01_rb,4.4370e+01_rb,2.4344e-02_rb/)
      kao(:, 4,10,14) = (/ &
     &4.5706e+02_rb,3.9992e+02_rb,3.4280e+02_rb,2.8566e+02_rb,2.2852e+02_rb,1.7137e+02_rb, &
     &1.1427e+02_rb,5.7136e+01_rb,2.2864e-02_rb/)
      kao(:, 5,10,14) = (/ &
     &5.6918e+02_rb,4.9801e+02_rb,4.2687e+02_rb,3.5573e+02_rb,2.8459e+02_rb,2.1346e+02_rb, &
     &1.4231e+02_rb,7.1148e+01_rb,3.1037e-02_rb/)
      kao(:, 1,11,14) = (/ &
     &3.3285e+02_rb,2.9123e+02_rb,2.4962e+02_rb,2.0803e+02_rb,1.6643e+02_rb,1.2482e+02_rb, &
     &8.3209e+01_rb,4.1605e+01_rb,7.6787e-06_rb/)
      kao(:, 2,11,14) = (/ &
     &4.7091e+02_rb,4.1207e+02_rb,3.5320e+02_rb,2.9434e+02_rb,2.3549e+02_rb,1.7660e+02_rb, &
     &1.1774e+02_rb,5.8871e+01_rb,1.2053e-05_rb/)
      kao(:, 3,11,14) = (/ &
     &6.3434e+02_rb,5.5505e+02_rb,4.7579e+02_rb,3.9648e+02_rb,3.1719e+02_rb,2.3787e+02_rb, &
     &1.5859e+02_rb,7.9298e+01_rb,1.7979e-05_rb/)
      kao(:, 4,11,14) = (/ &
     &8.2080e+02_rb,7.1819e+02_rb,6.1562e+02_rb,5.1300e+02_rb,4.1039e+02_rb,3.0781e+02_rb, &
     &2.0518e+02_rb,1.0261e+02_rb,2.6043e-05_rb/)
      kao(:, 5,11,14) = (/ &
     &1.0279e+03_rb,8.9936e+02_rb,7.7089e+02_rb,6.4243e+02_rb,5.1399e+02_rb,3.8545e+02_rb, &
     &2.5700e+02_rb,1.2849e+02_rb,3.6811e-02_rb/)
      kao(:, 1,12,14) = (/ &
     &4.4072e+02_rb,3.8565e+02_rb,3.3056e+02_rb,2.7546e+02_rb,2.2036e+02_rb,1.6527e+02_rb, &
     &1.1018e+02_rb,5.5100e+01_rb,6.3836e-06_rb/)
      kao(:, 2,12,14) = (/ &
     &6.2804e+02_rb,5.4949e+02_rb,4.7103e+02_rb,3.9252e+02_rb,3.1400e+02_rb,2.3550e+02_rb, &
     &1.5700e+02_rb,7.8501e+01_rb,1.0354e-05_rb/)
      kao(:, 3,12,14) = (/ &
     &8.4964e+02_rb,7.4346e+02_rb,6.3726e+02_rb,5.3106e+02_rb,4.2484e+02_rb,3.1862e+02_rb, &
     &2.1241e+02_rb,1.0621e+02_rb,1.6673e-05_rb/)
      kao(:, 4,12,14) = (/ &
     &1.1033e+03_rb,9.6532e+02_rb,8.2745e+02_rb,6.8953e+02_rb,5.5162e+02_rb,4.1376e+02_rb, &
     &2.7582e+02_rb,1.3790e+02_rb,2.5500e-05_rb/)
      kao(:, 5,12,14) = (/ &
     &1.3847e+03_rb,1.2117e+03_rb,1.0386e+03_rb,8.6544e+02_rb,6.9241e+02_rb,5.1925e+02_rb, &
     &3.4620e+02_rb,1.7310e+02_rb,3.8579e-05_rb/)
      kao(:, 1,13,14) = (/ &
     &4.7310e+02_rb,4.1397e+02_rb,3.5483e+02_rb,2.9569e+02_rb,2.3655e+02_rb,1.7741e+02_rb, &
     &1.1827e+02_rb,5.9135e+01_rb,7.3062e-06_rb/)
      kao(:, 2,13,14) = (/ &
     &6.7569e+02_rb,5.9130e+02_rb,5.0679e+02_rb,4.2232e+02_rb,3.3786e+02_rb,2.5340e+02_rb, &
     &1.6892e+02_rb,8.4465e+01_rb,8.6740e-06_rb/)
      kao(:, 3,13,14) = (/ &
     &9.1788e+02_rb,8.0315e+02_rb,6.8841e+02_rb,5.7369e+02_rb,4.5893e+02_rb,3.4424e+02_rb, &
     &2.2946e+02_rb,1.1474e+02_rb,1.3663e-05_rb/)
      kao(:, 4,13,14) = (/ &
     &1.1975e+03_rb,1.0479e+03_rb,8.9816e+02_rb,7.4843e+02_rb,5.9874e+02_rb,4.4905e+02_rb, &
     &2.9938e+02_rb,1.4969e+02_rb,2.1412e-05_rb/)
      kao(:, 5,13,14) = (/ &
     &1.5101e+03_rb,1.3214e+03_rb,1.1326e+03_rb,9.4376e+02_rb,7.5504e+02_rb,5.6626e+02_rb, &
     &3.7752e+02_rb,1.8876e+02_rb,3.3615e-05_rb/)
      kao(:, 1, 1,15) = (/ &
     &1.9051e-01_rb,1.6674e-01_rb,1.4292e-01_rb,1.1918e-01_rb,9.5396e-02_rb,7.5613e-02_rb, &
     &8.0457e-02_rb,7.8890e-02_rb,1.1694e-01_rb/)
      kao(:, 2, 1,15) = (/ &
     &2.2168e-01_rb,1.9400e-01_rb,1.6634e-01_rb,1.3865e-01_rb,1.1100e-01_rb,1.0009e-01_rb, &
     &1.1113e-01_rb,1.0913e-01_rb,1.6166e-01_rb/)
      kao(:, 3, 1,15) = (/ &
     &2.5284e-01_rb,2.2130e-01_rb,1.8975e-01_rb,1.5816e-01_rb,1.2778e-01_rb,1.3319e-01_rb, &
     &1.4805e-01_rb,1.4559e-01_rb,2.1625e-01_rb/)
      kao(:, 4, 1,15) = (/ &
     &2.8343e-01_rb,2.4807e-01_rb,2.1270e-01_rb,1.7731e-01_rb,1.5213e-01_rb,1.7480e-01_rb, &
     &1.9314e-01_rb,1.8830e-01_rb,2.8691e-01_rb/)
      kao(:, 5, 1,15) = (/ &
     &3.1350e-01_rb,2.7439e-01_rb,2.3527e-01_rb,1.9622e-01_rb,1.9594e-01_rb,2.3135e-01_rb, &
     &2.5307e-01_rb,2.4156e-01_rb,3.8082e-01_rb/)
      kao(:, 1, 2,15) = (/ &
     &3.2111e-01_rb,2.8102e-01_rb,2.4090e-01_rb,2.0079e-01_rb,1.6069e-01_rb,1.2054e-01_rb, &
     &9.0034e-02_rb,8.9831e-02_rb,1.2243e-01_rb/)
      kao(:, 2, 2,15) = (/ &
     &3.7636e-01_rb,3.2926e-01_rb,2.8226e-01_rb,2.3534e-01_rb,1.8831e-01_rb,1.4126e-01_rb, &
     &1.2281e-01_rb,1.2620e-01_rb,1.7187e-01_rb/)
      kao(:, 3, 2,15) = (/ &
     &4.3275e-01_rb,3.7860e-01_rb,3.2463e-01_rb,2.7055e-01_rb,2.1650e-01_rb,1.6706e-01_rb, &
     &1.6578e-01_rb,1.7060e-01_rb,2.3384e-01_rb/)
      kao(:, 4, 2,15) = (/ &
     &4.8952e-01_rb,4.2824e-01_rb,3.6725e-01_rb,3.0597e-01_rb,2.4488e-01_rb,2.0300e-01_rb, &
     &2.2012e-01_rb,2.2396e-01_rb,3.1375e-01_rb/)
      kao(:, 5, 2,15) = (/ &
     &5.4581e-01_rb,4.7748e-01_rb,4.0943e-01_rb,3.4119e-01_rb,2.7312e-01_rb,2.6005e-01_rb, &
     &2.9166e-01_rb,2.9212e-01_rb,4.1866e-01_rb/)
      kao(:, 1, 3,15) = (/ &
     &6.9979e-01_rb,6.1213e-01_rb,5.2483e-01_rb,4.3719e-01_rb,3.4984e-01_rb,2.6243e-01_rb, &
     &1.7501e-01_rb,9.9676e-02_rb,1.1688e-01_rb/)
      kao(:, 2, 3,15) = (/ &
     &8.3387e-01_rb,7.2959e-01_rb,6.2545e-01_rb,5.2112e-01_rb,4.1694e-01_rb,3.1278e-01_rb, &
     &2.0862e-01_rb,1.3660e-01_rb,1.6827e-01_rb/)
      kao(:, 3, 3,15) = (/ &
     &9.7266e-01_rb,8.5098e-01_rb,7.2958e-01_rb,6.0802e-01_rb,4.8655e-01_rb,3.6492e-01_rb, &
     &2.4335e-01_rb,1.8790e-01_rb,2.3450e-01_rb/)
      kao(:, 4, 3,15) = (/ &
     &1.1134e+00_rb,9.7421e-01_rb,8.3530e-01_rb,6.9606e-01_rb,5.5697e-01_rb,4.1779e-01_rb, &
     &2.8251e-01_rb,2.5378e-01_rb,3.2011e-01_rb/)
      kao(:, 5, 3,15) = (/ &
     &1.2541e+00_rb,1.0974e+00_rb,9.4084e-01_rb,7.8396e-01_rb,6.2738e-01_rb,4.7061e-01_rb, &
     &3.3812e-01_rb,3.3805e-01_rb,4.3341e-01_rb/)
      kao(:, 1, 4,15) = (/ &
     &1.5338e+00_rb,1.3422e+00_rb,1.1503e+00_rb,9.5870e-01_rb,7.6686e-01_rb,5.7527e-01_rb, &
     &3.8351e-01_rb,1.9180e-01_rb,1.0723e-01_rb/)
      kao(:, 2, 4,15) = (/ &
     &1.8631e+00_rb,1.6302e+00_rb,1.3974e+00_rb,1.1644e+00_rb,9.3180e-01_rb,6.9872e-01_rb, &
     &4.6589e-01_rb,2.3301e-01_rb,1.5610e-01_rb/)
      kao(:, 3, 4,15) = (/ &
     &2.2077e+00_rb,1.9315e+00_rb,1.6557e+00_rb,1.3803e+00_rb,1.1039e+00_rb,8.2801e-01_rb, &
     &5.5191e-01_rb,2.7612e-01_rb,2.2377e-01_rb/)
      kao(:, 4, 4,15) = (/ &
     &2.5609e+00_rb,2.2407e+00_rb,1.9209e+00_rb,1.6005e+00_rb,1.2804e+00_rb,9.6050e-01_rb, &
     &6.4039e-01_rb,3.2615e-01_rb,3.1195e-01_rb/)
      kao(:, 5, 4,15) = (/ &
     &2.9154e+00_rb,2.5514e+00_rb,2.1867e+00_rb,1.8223e+00_rb,1.4581e+00_rb,1.0937e+00_rb, &
     &7.2913e-01_rb,3.9607e-01_rb,4.2846e-01_rb/)
      kao(:, 1, 5,15) = (/ &
     &3.0736e+00_rb,2.6896e+00_rb,2.3057e+00_rb,1.9212e+00_rb,1.5370e+00_rb,1.1527e+00_rb, &
     &7.6854e-01_rb,3.8431e-01_rb,1.0147e-01_rb/)
      kao(:, 2, 5,15) = (/ &
     &3.8075e+00_rb,3.3314e+00_rb,2.8555e+00_rb,2.3797e+00_rb,1.9037e+00_rb,1.4279e+00_rb, &
     &9.5192e-01_rb,4.7599e-01_rb,1.4471e-01_rb/)
      kao(:, 3, 5,15) = (/ &
     &4.5849e+00_rb,4.0117e+00_rb,3.4370e+00_rb,2.8652e+00_rb,2.2919e+00_rb,1.7190e+00_rb, &
     &1.1462e+00_rb,5.7315e-01_rb,2.0749e-01_rb/)
      kao(:, 4, 5,15) = (/ &
     &5.3872e+00_rb,4.7143e+00_rb,4.0418e+00_rb,3.3672e+00_rb,2.6939e+00_rb,2.0205e+00_rb, &
     &1.3469e+00_rb,6.7361e-01_rb,2.9614e-01_rb/)
      kao(:, 5, 5,15) = (/ &
     &6.2028e+00_rb,5.4263e+00_rb,4.6517e+00_rb,3.8766e+00_rb,3.1014e+00_rb,2.3260e+00_rb, &
     &1.5507e+00_rb,7.7527e-01_rb,4.1447e-01_rb/)
      kao(:, 1, 6,15) = (/ &
     &5.6666e+00_rb,4.9583e+00_rb,4.2496e+00_rb,3.5417e+00_rb,2.8332e+00_rb,2.1246e+00_rb, &
     &1.4165e+00_rb,7.0837e-01_rb,9.5689e-02_rb/)
      kao(:, 2, 6,15) = (/ &
     &7.1702e+00_rb,6.2735e+00_rb,5.3788e+00_rb,4.4810e+00_rb,3.5859e+00_rb,2.6888e+00_rb, &
     &1.7925e+00_rb,8.9634e-01_rb,1.3582e-01_rb/)
      kao(:, 3, 6,15) = (/ &
     &8.7850e+00_rb,7.6873e+00_rb,6.5880e+00_rb,5.4902e+00_rb,4.3923e+00_rb,3.2943e+00_rb, &
     &2.1960e+00_rb,1.0981e+00_rb,1.9014e-01_rb/)
      kao(:, 4, 6,15) = (/ &
     &1.0471e+01_rb,9.1623e+00_rb,7.8537e+00_rb,6.5443e+00_rb,5.2362e+00_rb,3.9265e+00_rb, &
     &2.6173e+00_rb,1.3091e+00_rb,2.7084e-01_rb/)
      kao(:, 5, 6,15) = (/ &
     &1.2195e+01_rb,1.0670e+01_rb,9.1465e+00_rb,7.6219e+00_rb,6.0976e+00_rb,4.5729e+00_rb, &
     &3.0483e+00_rb,1.5247e+00_rb,3.8788e-01_rb/)
      kao(:, 1, 7,15) = (/ &
     &1.0821e+01_rb,9.4694e+00_rb,8.1159e+00_rb,6.7641e+00_rb,5.4109e+00_rb,4.0578e+00_rb, &
     &2.7049e+00_rb,1.3527e+00_rb,8.8090e-02_rb/)
      kao(:, 2, 7,15) = (/ &
     &1.4005e+01_rb,1.2256e+01_rb,1.0505e+01_rb,8.7544e+00_rb,7.0039e+00_rb,5.2527e+00_rb, &
     &3.5019e+00_rb,1.7510e+00_rb,1.2836e-01_rb/)
      kao(:, 3, 7,15) = (/ &
     &1.7472e+01_rb,1.5288e+01_rb,1.3104e+01_rb,1.0920e+01_rb,8.7370e+00_rb,6.5526e+00_rb, &
     &4.3682e+00_rb,2.1842e+00_rb,1.7912e-01_rb/)
      kao(:, 4, 7,15) = (/ &
     &2.1133e+01_rb,1.8491e+01_rb,1.5850e+01_rb,1.3209e+01_rb,1.0568e+01_rb,7.9265e+00_rb, &
     &5.2835e+00_rb,2.6416e+00_rb,2.4968e-01_rb/)
      kao(:, 5, 7,15) = (/ &
     &2.4946e+01_rb,2.1829e+01_rb,1.8712e+01_rb,1.5594e+01_rb,1.2475e+01_rb,9.3563e+00_rb, &
     &6.2369e+00_rb,3.1183e+00_rb,3.5418e-01_rb/)
      kao(:, 1, 8,15) = (/ &
     &2.4402e+01_rb,2.1362e+01_rb,1.8302e+01_rb,1.5253e+01_rb,1.2203e+01_rb,9.1488e+00_rb, &
     &6.1012e+00_rb,3.0503e+00_rb,8.0172e-02_rb/)
      kao(:, 2, 8,15) = (/ &
     &3.2338e+01_rb,2.8299e+01_rb,2.4254e+01_rb,2.0211e+01_rb,1.6168e+01_rb,1.2127e+01_rb, &
     &8.0850e+00_rb,4.0423e+00_rb,1.1917e-01_rb/)
      kao(:, 3, 8,15) = (/ &
     &4.1117e+01_rb,3.5977e+01_rb,3.0837e+01_rb,2.5703e+01_rb,2.0563e+01_rb,1.5418e+01_rb, &
     &1.0280e+01_rb,5.1397e+00_rb,1.6937e-01_rb/)
      kao(:, 4, 8,15) = (/ &
     &5.0705e+01_rb,4.4368e+01_rb,3.8026e+01_rb,3.1698e+01_rb,2.5350e+01_rb,1.9014e+01_rb, &
     &1.2673e+01_rb,6.3384e+00_rb,2.3470e-01_rb/)
      kao(:, 5, 8,15) = (/ &
     &6.0900e+01_rb,5.3282e+01_rb,4.5673e+01_rb,3.8064e+01_rb,3.0449e+01_rb,2.2835e+01_rb, &
     &1.5224e+01_rb,7.6126e+00_rb,3.2704e-01_rb/)
      kao(:, 1, 9,15) = (/ &
     &1.0170e+02_rb,8.8996e+01_rb,7.6286e+01_rb,6.3569e+01_rb,5.0854e+01_rb,3.8145e+01_rb, &
     &2.5429e+01_rb,1.2713e+01_rb,7.1359e-02_rb/)
      kao(:, 2, 9,15) = (/ &
     &1.3845e+02_rb,1.2114e+02_rb,1.0384e+02_rb,8.6532e+01_rb,6.9224e+01_rb,5.1921e+01_rb, &
     &3.4611e+01_rb,1.7307e+01_rb,1.0767e-01_rb/)
      kao(:, 3, 9,15) = (/ &
     &1.8073e+02_rb,1.5810e+02_rb,1.3551e+02_rb,1.1293e+02_rb,9.0345e+01_rb,6.7755e+01_rb, &
     &4.5176e+01_rb,2.2583e+01_rb,1.5849e-01_rb/)
      kao(:, 4, 9,15) = (/ &
     &2.2745e+02_rb,1.9903e+02_rb,1.7061e+02_rb,1.4217e+02_rb,1.1373e+02_rb,8.5301e+01_rb, &
     &5.6875e+01_rb,2.8433e+01_rb,2.2030e-01_rb/)
      kao(:, 5, 9,15) = (/ &
     &2.7773e+02_rb,2.4298e+02_rb,2.0829e+02_rb,1.7358e+02_rb,1.3886e+02_rb,1.0414e+02_rb, &
     &6.9430e+01_rb,3.4715e+01_rb,3.0575e-01_rb/)
      kao(:, 1,10,15) = (/ &
     &4.7674e+02_rb,4.1715e+02_rb,3.5753e+02_rb,2.9794e+02_rb,2.3835e+02_rb,1.7878e+02_rb, &
     &1.1918e+02_rb,5.9592e+01_rb,7.9888e-06_rb/)
      kao(:, 2,10,15) = (/ &
     &6.6856e+02_rb,5.8521e+02_rb,5.0164e+02_rb,4.1786e+02_rb,3.3435e+02_rb,2.5071e+02_rb, &
     &1.6719e+02_rb,8.3597e+01_rb,1.0572e-05_rb/)
      kao(:, 3,10,15) = (/ &
     &8.9281e+02_rb,7.8118e+02_rb,6.6956e+02_rb,5.5797e+02_rb,4.4639e+02_rb,3.3479e+02_rb, &
     &2.2318e+02_rb,1.1160e+02_rb,1.3235e-05_rb/)
      kao(:, 4,10,15) = (/ &
     &1.1439e+03_rb,1.0009e+03_rb,8.5793e+02_rb,7.1497e+02_rb,5.7192e+02_rb,4.2885e+02_rb, &
     &2.8596e+02_rb,1.4299e+02_rb,7.9928e-02_rb/)
      kao(:, 5,10,15) = (/ &
     &1.4160e+03_rb,1.2390e+03_rb,1.0620e+03_rb,8.8492e+02_rb,7.0798e+02_rb,5.3098e+02_rb, &
     &3.5397e+02_rb,1.7699e+02_rb,2.0557e-01_rb/)
      kao(:, 1,11,15) = (/ &
     &9.1777e+02_rb,8.0296e+02_rb,6.8835e+02_rb,5.7354e+02_rb,4.5882e+02_rb,3.4414e+02_rb, &
     &2.2941e+02_rb,1.1472e+02_rb,9.0079e-06_rb/)
      kao(:, 2,11,15) = (/ &
     &1.2897e+03_rb,1.1284e+03_rb,9.6718e+02_rb,8.0605e+02_rb,6.4484e+02_rb,4.8368e+02_rb, &
     &3.2242e+02_rb,1.6120e+02_rb,1.0534e-05_rb/)
      kao(:, 3,11,15) = (/ &
     &1.7217e+03_rb,1.5064e+03_rb,1.2912e+03_rb,1.0761e+03_rb,8.6072e+02_rb,6.4564e+02_rb, &
     &4.3042e+02_rb,2.1520e+02_rb,1.2890e-05_rb/)
      kao(:, 4,11,15) = (/ &
     &2.2030e+03_rb,1.9276e+03_rb,1.6522e+03_rb,1.3769e+03_rb,1.1015e+03_rb,8.2614e+02_rb, &
     &5.5075e+02_rb,2.7537e+02_rb,1.8773e-05_rb/)
      kao(:, 5,11,15) = (/ &
     &2.7288e+03_rb,2.3877e+03_rb,2.0468e+03_rb,1.7056e+03_rb,1.3645e+03_rb,1.0232e+03_rb, &
     &6.8218e+02_rb,3.4109e+02_rb,2.8192e-05_rb/)
      kao(:, 1,12,15) = (/ &
     &1.3049e+03_rb,1.1418e+03_rb,9.7855e+02_rb,8.1546e+02_rb,6.5249e+02_rb,4.8934e+02_rb, &
     &3.2625e+02_rb,1.6311e+02_rb,7.4201e-06_rb/)
      kao(:, 2,12,15) = (/ &
     &1.8314e+03_rb,1.6024e+03_rb,1.3735e+03_rb,1.1446e+03_rb,9.1569e+02_rb,6.8677e+02_rb, &
     &4.5789e+02_rb,2.2893e+02_rb,9.0712e-06_rb/)
      kao(:, 3,12,15) = (/ &
     &2.4447e+03_rb,2.1390e+03_rb,1.8335e+03_rb,1.5279e+03_rb,1.2222e+03_rb,9.1674e+02_rb, &
     &6.1121e+02_rb,3.0557e+02_rb,9.2932e-06_rb/)
      kao(:, 4,12,15) = (/ &
     &3.1395e+03_rb,2.7470e+03_rb,2.3545e+03_rb,1.9622e+03_rb,1.5697e+03_rb,1.1774e+03_rb, &
     &7.8488e+02_rb,3.9241e+02_rb,1.2651e-05_rb/)
      kao(:, 5,12,15) = (/ &
     &3.8988e+03_rb,3.4115e+03_rb,2.9243e+03_rb,2.4367e+03_rb,1.9495e+03_rb,1.4621e+03_rb, &
     &9.7464e+02_rb,4.8737e+02_rb,2.0431e-05_rb/)
      kao(:, 1,13,15) = (/ &
     &1.4703e+03_rb,1.2867e+03_rb,1.1028e+03_rb,9.1904e+02_rb,7.3532e+02_rb,5.5150e+02_rb, &
     &3.6759e+02_rb,1.8380e+02_rb,5.1458e-06_rb/)
      kao(:, 2,13,15) = (/ &
     &2.0704e+03_rb,1.8115e+03_rb,1.5528e+03_rb,1.2939e+03_rb,1.0351e+03_rb,7.7638e+02_rb, &
     &5.1765e+02_rb,2.5875e+02_rb,6.1867e-06_rb/)
      kao(:, 3,13,15) = (/ &
     &2.7739e+03_rb,2.4272e+03_rb,2.0805e+03_rb,1.7336e+03_rb,1.3869e+03_rb,1.0403e+03_rb, &
     &6.9356e+02_rb,3.4674e+02_rb,7.7061e-06_rb/)
      kao(:, 4,13,15) = (/ &
     &3.5658e+03_rb,3.1198e+03_rb,2.6740e+03_rb,2.2287e+03_rb,1.7830e+03_rb,1.3371e+03_rb, &
     &8.9142e+02_rb,4.4572e+02_rb,1.1573e-05_rb/)
      kao(:, 5,13,15) = (/ &
     &4.4270e+03_rb,3.8738e+03_rb,3.3205e+03_rb,2.7670e+03_rb,2.2135e+03_rb,1.6602e+03_rb, &
     &1.1068e+03_rb,5.5337e+02_rb,1.8919e-05_rb/)
      kao(:, 1, 1,16) = (/ &
     &2.0584e-01_rb,1.8014e-01_rb,1.5444e-01_rb,1.2874e-01_rb,1.0304e-01_rb,7.7345e-02_rb, &
     &8.4273e-02_rb,8.1561e-02_rb,1.2397e-01_rb/)
      kao(:, 2, 1,16) = (/ &
     &2.4103e-01_rb,2.1093e-01_rb,1.8085e-01_rb,1.5075e-01_rb,1.2067e-01_rb,1.0583e-01_rb, &
     &1.1670e-01_rb,1.1303e-01_rb,1.7190e-01_rb/)
      kao(:, 3, 1,16) = (/ &
     &2.7664e-01_rb,2.4213e-01_rb,2.0759e-01_rb,1.7305e-01_rb,1.3852e-01_rb,1.4127e-01_rb, &
     &1.5588e-01_rb,1.5105e-01_rb,2.2980e-01_rb/)
      kao(:, 4, 1,16) = (/ &
     &3.1217e-01_rb,2.7321e-01_rb,2.3426e-01_rb,1.9529e-01_rb,1.5635e-01_rb,1.8285e-01_rb, &
     &2.0187e-01_rb,1.9571e-01_rb,2.9781e-01_rb/)
      kao(:, 5, 1,16) = (/ &
     &3.4704e-01_rb,3.0374e-01_rb,2.6044e-01_rb,2.1712e-01_rb,2.0704e-01_rb,2.4179e-01_rb, &
     &2.6249e-01_rb,2.4688e-01_rb,4.0744e-01_rb/)
      kao(:, 1, 2,16) = (/ &
     &3.5775e-01_rb,3.1302e-01_rb,2.6832e-01_rb,2.2363e-01_rb,1.7894e-01_rb,1.3423e-01_rb, &
     &9.4199e-02_rb,9.5152e-02_rb,1.3347e-01_rb/)
      kao(:, 2, 2,16) = (/ &
     &4.2272e-01_rb,3.6993e-01_rb,3.1710e-01_rb,2.6431e-01_rb,2.1146e-01_rb,1.5866e-01_rb, &
     &1.3268e-01_rb,1.3412e-01_rb,1.8823e-01_rb/)
      kao(:, 3, 2,16) = (/ &
     &4.8911e-01_rb,4.2798e-01_rb,3.6687e-01_rb,3.0577e-01_rb,2.4466e-01_rb,1.8360e-01_rb, &
     &1.7978e-01_rb,1.8185e-01_rb,2.5529e-01_rb/)
      kao(:, 4, 2,16) = (/ &
     &5.5559e-01_rb,4.8623e-01_rb,4.1679e-01_rb,3.4735e-01_rb,2.7796e-01_rb,2.0932e-01_rb, &
     &2.3570e-01_rb,2.3853e-01_rb,3.3493e-01_rb/)
      kao(:, 5, 2,16) = (/ &
     &6.2130e-01_rb,5.4374e-01_rb,4.6609e-01_rb,3.8845e-01_rb,3.1090e-01_rb,2.6998e-01_rb, &
     &3.0230e-01_rb,3.0412e-01_rb,4.4787e-01_rb/)
      kao(:, 1, 3,16) = (/ &
     &8.0759e-01_rb,7.0655e-01_rb,6.0564e-01_rb,5.0473e-01_rb,4.0382e-01_rb,3.0291e-01_rb, &
     &2.0195e-01_rb,1.0457e-01_rb,1.3240e-01_rb/)
      kao(:, 2, 3,16) = (/ &
     &9.6833e-01_rb,8.4737e-01_rb,7.2636e-01_rb,6.0527e-01_rb,4.8421e-01_rb,3.6321e-01_rb, &
     &2.4220e-01_rb,1.5138e-01_rb,1.9184e-01_rb/)
      kao(:, 3, 3,16) = (/ &
     &1.1342e+00_rb,9.9245e-01_rb,8.5072e-01_rb,7.0903e-01_rb,5.6727e-01_rb,4.2548e-01_rb, &
     &2.8373e-01_rb,2.0993e-01_rb,2.6621e-01_rb/)
      kao(:, 4, 3,16) = (/ &
     &1.3024e+00_rb,1.1395e+00_rb,9.7685e-01_rb,8.1409e-01_rb,6.5126e-01_rb,4.8855e-01_rb, &
     &3.2575e-01_rb,2.8072e-01_rb,3.5605e-01_rb/)
      kao(:, 5, 3,16) = (/ &
     &1.4698e+00_rb,1.2860e+00_rb,1.1024e+00_rb,9.1880e-01_rb,7.3505e-01_rb,5.5132e-01_rb, &
     &3.6767e-01_rb,3.6382e-01_rb,4.6256e-01_rb/)
      kao(:, 1, 4,16) = (/ &
     &1.8354e+00_rb,1.6059e+00_rb,1.3766e+00_rb,1.1471e+00_rb,9.1770e-01_rb,6.8825e-01_rb, &
     &4.5887e-01_rb,2.2948e-01_rb,1.2512e-01_rb/)
      kao(:, 2, 4,16) = (/ &
     &2.2391e+00_rb,1.9593e+00_rb,1.6796e+00_rb,1.3998e+00_rb,1.1197e+00_rb,8.3982e-01_rb, &
     &5.5988e-01_rb,2.7997e-01_rb,1.8724e-01_rb/)
      kao(:, 3, 4,16) = (/ &
     &2.6616e+00_rb,2.3290e+00_rb,1.9963e+00_rb,1.6635e+00_rb,1.3310e+00_rb,9.9835e-01_rb, &
     &6.6551e-01_rb,3.3282e-01_rb,2.6695e-01_rb/)
      kao(:, 4, 4,16) = (/ &
     &3.0942e+00_rb,2.7077e+00_rb,2.3212e+00_rb,1.9341e+00_rb,1.5474e+00_rb,1.1605e+00_rb, &
     &7.7373e-01_rb,3.8698e-01_rb,3.6534e-01_rb/)
      kao(:, 5, 4,16) = (/ &
     &3.5299e+00_rb,3.0886e+00_rb,2.6475e+00_rb,2.2065e+00_rb,1.7650e+00_rb,1.3239e+00_rb, &
     &8.8260e-01_rb,4.4141e-01_rb,4.8300e-01_rb/)
      kao(:, 1, 5,16) = (/ &
     &3.8226e+00_rb,3.3446e+00_rb,2.8509e+00_rb,2.3892e+00_rb,1.9114e+00_rb,1.4335e+00_rb, &
     &9.5565e-01_rb,4.7787e-01_rb,1.1560e-01_rb/)
      kao(:, 2, 5,16) = (/ &
     &4.7529e+00_rb,4.1589e+00_rb,3.5644e+00_rb,2.9706e+00_rb,2.3763e+00_rb,1.7823e+00_rb, &
     &1.1883e+00_rb,5.9413e-01_rb,1.7922e-01_rb/)
      kao(:, 3, 5,16) = (/ &
     &5.7385e+00_rb,5.0213e+00_rb,4.3040e+00_rb,3.5869e+00_rb,2.8697e+00_rb,2.1522e+00_rb, &
     &1.4347e+00_rb,7.1742e-01_rb,2.6311e-01_rb/)
      kao(:, 4, 5,16) = (/ &
     &6.7615e+00_rb,5.9164e+00_rb,5.0713e+00_rb,4.2261e+00_rb,3.3806e+00_rb,2.5357e+00_rb, &
     &1.6905e+00_rb,8.4524e-01_rb,3.6914e-01_rb/)
      kao(:, 5, 5,16) = (/ &
     &7.8020e+00_rb,6.8269e+00_rb,5.8507e+00_rb,4.8765e+00_rb,3.9008e+00_rb,2.9261e+00_rb, &
     &1.9507e+00_rb,9.7534e-01_rb,4.9832e-01_rb/)
      kao(:, 1, 6,16) = (/ &
     &7.3571e+00_rb,6.4369e+00_rb,5.5172e+00_rb,4.5985e+00_rb,3.6783e+00_rb,2.7589e+00_rb, &
     &1.8291e+00_rb,9.1965e-01_rb,1.0645e-01_rb/)
      kao(:, 2, 6,16) = (/ &
     &9.3468e+00_rb,8.1778e+00_rb,7.0095e+00_rb,5.8414e+00_rb,4.6732e+00_rb,3.5051e+00_rb, &
     &2.3366e+00_rb,1.1682e+00_rb,1.6617e-01_rb/)
      kao(:, 3, 6,16) = (/ &
     &1.1491e+01_rb,1.0055e+01_rb,8.6175e+00_rb,7.1820e+00_rb,5.7127e+00_rb,4.3090e+00_rb, &
     &2.8724e+00_rb,1.4282e+00_rb,2.5233e-01_rb/)
      kao(:, 4, 6,16) = (/ &
     &1.3744e+01_rb,1.2025e+01_rb,1.0308e+01_rb,8.5898e+00_rb,6.8725e+00_rb,5.1546e+00_rb, &
     &3.4361e+00_rb,1.7180e+00_rb,3.6410e-01_rb/)
      kao(:, 5, 6,16) = (/ &
     &1.6063e+01_rb,1.4056e+01_rb,1.2048e+01_rb,1.0040e+01_rb,8.0325e+00_rb,6.0238e+00_rb, &
     &4.0162e+00_rb,2.0081e+00_rb,5.0340e-01_rb/)
      kao(:, 1, 7,16) = (/ &
     &1.4766e+01_rb,1.2923e+01_rb,1.1075e+01_rb,9.2294e+00_rb,7.3836e+00_rb,5.5378e+00_rb, &
     &3.6915e+00_rb,1.8460e+00_rb,1.0988e-01_rb/)
      kao(:, 2, 7,16) = (/ &
     &1.9210e+01_rb,1.6809e+01_rb,1.4406e+01_rb,1.2004e+01_rb,9.6049e+00_rb,7.2040e+00_rb, &
     &4.8023e+00_rb,2.4010e+00_rb,1.5119e-01_rb/)
      kao(:, 3, 7,16) = (/ &
     &2.4080e+01_rb,2.1071e+01_rb,1.8060e+01_rb,1.5050e+01_rb,1.2042e+01_rb,9.0305e+00_rb, &
     &6.0199e+00_rb,2.9934e+00_rb,2.3822e-01_rb/)
      kao(:, 4, 7,16) = (/ &
     &2.9278e+01_rb,2.5619e+01_rb,2.1957e+01_rb,1.8300e+01_rb,1.4640e+01_rb,1.0980e+01_rb, &
     &7.3208e+00_rb,3.6596e+00_rb,3.5444e-01_rb/)
      kao(:, 5, 7,16) = (/ &
     &3.4690e+01_rb,3.0352e+01_rb,2.6017e+01_rb,2.1683e+01_rb,1.7349e+01_rb,1.3009e+01_rb, &
     &8.6734e+00_rb,4.3366e+00_rb,5.0281e-01_rb/)
      kao(:, 1, 8,16) = (/ &
     &3.5305e+01_rb,3.0898e+01_rb,2.6479e+01_rb,2.2066e+01_rb,1.7653e+01_rb,1.3237e+01_rb, &
     &8.8267e+00_rb,4.4136e+00_rb,1.1225e-01_rb/)
      kao(:, 2, 8,16) = (/ &
     &4.7105e+01_rb,4.1217e+01_rb,3.5327e+01_rb,2.9443e+01_rb,2.3551e+01_rb,1.7664e+01_rb, &
     &1.1776e+01_rb,5.8880e+00_rb,1.4909e-01_rb/)
      kao(:, 3, 8,16) = (/ &
     &6.0289e+01_rb,5.2760e+01_rb,4.5222e+01_rb,3.7680e+01_rb,3.0141e+01_rb,2.2609e+01_rb, &
     &1.5075e+01_rb,7.5364e+00_rb,2.2235e-01_rb/)
      kao(:, 4, 8,16) = (/ &
     &7.4572e+01_rb,6.4882e+01_rb,5.5930e+01_rb,4.6614e+01_rb,3.7287e+01_rb,2.7965e+01_rb, &
     &1.8642e+01_rb,9.3209e+00_rb,3.4182e-01_rb/)
      kao(:, 5, 8,16) = (/ &
     &8.9645e+01_rb,7.8444e+01_rb,6.7227e+01_rb,5.6021e+01_rb,4.4815e+01_rb,3.3612e+01_rb, &
     &2.2408e+01_rb,1.1205e+01_rb,4.9824e-01_rb/)
      kao(:, 1, 9,16) = (/ &
     &1.5759e+02_rb,1.3788e+02_rb,1.1817e+02_rb,9.8475e+01_rb,7.8796e+01_rb,5.9083e+01_rb, &
     &3.9390e+01_rb,1.9699e+01_rb,1.1304e-01_rb/)
      kao(:, 2, 9,16) = (/ &
     &2.1616e+02_rb,1.8914e+02_rb,1.6212e+02_rb,1.3510e+02_rb,1.0808e+02_rb,8.1068e+01_rb, &
     &5.4043e+01_rb,2.7021e+01_rb,1.5445e-01_rb/)
      kao(:, 3, 9,16) = (/ &
     &2.8258e+02_rb,2.4760e+02_rb,2.1223e+02_rb,1.7686e+02_rb,1.4149e+02_rb,1.0611e+02_rb, &
     &7.0744e+01_rb,3.5373e+01_rb,2.0441e-01_rb/)
      kao(:, 4, 9,16) = (/ &
     &3.5653e+02_rb,3.1192e+02_rb,2.6735e+02_rb,2.2280e+02_rb,1.7825e+02_rb,1.3369e+02_rb, &
     &8.9123e+01_rb,4.4565e+01_rb,3.2503e-01_rb/)
      kao(:, 5, 9,16) = (/ &
     &4.3513e+02_rb,3.8075e+02_rb,3.2632e+02_rb,2.7197e+02_rb,2.1754e+02_rb,1.6316e+02_rb, &
     &1.0877e+02_rb,5.4390e+01_rb,4.8785e-01_rb/)
      kao(:, 1,10,16) = (/ &
     &7.9828e+02_rb,6.9859e+02_rb,5.9871e+02_rb,4.9899e+02_rb,3.9914e+02_rb,2.9935e+02_rb, &
     &1.9957e+02_rb,9.9786e+01_rb,1.4112e-06_rb/)
      kao(:, 2,10,16) = (/ &
     &1.1235e+03_rb,9.8329e+02_rb,8.4282e+02_rb,7.0236e+02_rb,5.6188e+02_rb,4.2131e+02_rb, &
     &2.8094e+02_rb,1.4047e+02_rb,1.8812e-06_rb/)
      kao(:, 3,10,16) = (/ &
     &1.5014e+03_rb,1.3138e+03_rb,1.1261e+03_rb,9.3840e+02_rb,7.5072e+02_rb,5.6311e+02_rb, &
     &3.7536e+02_rb,1.8771e+02_rb,2.5472e-06_rb/)
      kao(:, 4,10,16) = (/ &
     &1.9233e+03_rb,1.6832e+03_rb,1.4427e+03_rb,1.2020e+03_rb,9.6160e+02_rb,7.2135e+02_rb, &
     &4.8081e+02_rb,2.4039e+02_rb,3.1592e-01_rb/)
      kao(:, 5,10,16) = (/ &
     &2.3793e+03_rb,2.0824e+03_rb,1.7849e+03_rb,1.4876e+03_rb,1.1900e+03_rb,8.9243e+02_rb, &
     &5.9503e+02_rb,2.9747e+02_rb,4.8703e-01_rb/)
      kao(:, 1,11,16) = (/ &
     &1.6695e+03_rb,1.4608e+03_rb,1.2521e+03_rb,1.0434e+03_rb,8.3472e+02_rb,6.2605e+02_rb, &
     &4.1737e+02_rb,2.0867e+02_rb,1.1529e-06_rb/)
      kao(:, 2,11,16) = (/ &
     &2.3476e+03_rb,2.0540e+03_rb,1.7608e+03_rb,1.4673e+03_rb,1.1739e+03_rb,8.8029e+02_rb, &
     &5.8691e+02_rb,2.9349e+02_rb,1.5382e-06_rb/)
      kao(:, 3,11,16) = (/ &
     &3.1326e+03_rb,2.7416e+03_rb,2.3500e+03_rb,1.9579e+03_rb,1.5664e+03_rb,1.1749e+03_rb, &
     &7.8334e+02_rb,3.9162e+02_rb,2.0823e-06_rb/)
      kao(:, 4,11,16) = (/ &
     &4.0087e+03_rb,3.5076e+03_rb,3.0065e+03_rb,2.5054e+03_rb,2.0043e+03_rb,1.5033e+03_rb, &
     &1.0022e+03_rb,5.0110e+02_rb,2.4008e-06_rb/)
      kao(:, 5,11,16) = (/ &
     &4.9540e+03_rb,4.3346e+03_rb,3.7151e+03_rb,3.0963e+03_rb,2.4766e+03_rb,1.8577e+03_rb, &
     &1.2385e+03_rb,6.1917e+02_rb,1.3302e-05_rb/)
      kao(:, 1,12,16) = (/ &
     &2.5940e+03_rb,2.2689e+03_rb,1.9455e+03_rb,1.6210e+03_rb,1.2970e+03_rb,9.7271e+02_rb, &
     &6.4849e+02_rb,3.2418e+02_rb,9.4397e-07_rb/)
      kao(:, 2,12,16) = (/ &
     &3.6401e+03_rb,3.1850e+03_rb,2.7301e+03_rb,2.2751e+03_rb,1.8200e+03_rb,1.3650e+03_rb, &
     &9.1001e+02_rb,4.5497e+02_rb,1.2594e-06_rb/)
      kao(:, 3,12,16) = (/ &
     &4.8482e+03_rb,4.2436e+03_rb,3.6368e+03_rb,3.0301e+03_rb,2.4250e+03_rb,1.8187e+03_rb, &
     &1.2120e+03_rb,6.0606e+02_rb,1.2513e-05_rb/)
      kao(:, 4,12,16) = (/ &
     &6.1914e+03_rb,5.4187e+03_rb,4.6435e+03_rb,3.8696e+03_rb,3.0964e+03_rb,2.3220e+03_rb, &
     &1.5479e+03_rb,7.7403e+02_rb,2.0845e-05_rb/)
      kao(:, 5,12,16) = (/ &
     &7.6366e+03_rb,6.6419e+03_rb,5.7257e+03_rb,4.7728e+03_rb,3.8188e+03_rb,2.8637e+03_rb, &
     &1.9092e+03_rb,9.4962e+02_rb,2.1463e-05_rb/)
      kao(:, 1,13,16) = (/ &
     &3.2192e+03_rb,2.8160e+03_rb,2.4141e+03_rb,2.0117e+03_rb,1.6091e+03_rb,1.2070e+03_rb, &
     &8.0460e+02_rb,4.0235e+02_rb,2.5866e-06_rb/)
      kao(:, 2,13,16) = (/ &
     &4.5059e+03_rb,3.9428e+03_rb,3.3791e+03_rb,2.8160e+03_rb,2.2531e+03_rb,1.6897e+03_rb, &
     &1.1264e+03_rb,5.6328e+02_rb,9.4719e-06_rb/)
      kao(:, 3,13,16) = (/ &
     &5.9881e+03_rb,5.2388e+03_rb,4.4907e+03_rb,3.7423e+03_rb,2.9935e+03_rb,2.2450e+03_rb, &
     &1.4968e+03_rb,7.4841e+02_rb,1.6561e-05_rb/)
      kao(:, 4,13,16) = (/ &
     &7.6265e+03_rb,6.6729e+03_rb,5.7200e+03_rb,4.7671e+03_rb,3.8133e+03_rb,2.8600e+03_rb, &
     &1.9066e+03_rb,9.4757e+02_rb,2.1537e-05_rb/)
      kao(:, 5,13,16) = (/ &
     &9.3827e+03_rb,8.2086e+03_rb,7.0366e+03_rb,5.8639e+03_rb,4.6917e+03_rb,3.5181e+03_rb, &
     &2.3456e+03_rb,1.1729e+03_rb,2.1283e-05_rb/)

!     The array KBO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels  < ~100mb, temperatures, and ratios
!     of H2O to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index, JT, which
!     runs from 1 to 5, corresponds to different temperatures.  More 
!     specifically, JT = 3 means that the data are for the corresponding 
!     reference temperature TREF for this  pressure level, JT = 2 refers 
!     to the TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and
!     JT = 5 is for TREF+30.  The third index, JP, runs from 13 to 59 and
!     refers to the corresponding pressure level in PREF (e.g. JP = 13 is
!     for a pressure of 95.5835 mb).  The fourth index, IG, goes from 1 to
!     16, and tells us which g-interval the absorption coefficients are for.

      kbo(:, 1,13, 1) = (/ &
     &1.5731e-03_rb,1.8696e-03_rb,1.8486e-03_rb,1.7186e-03_rb,1.3898e-03_rb/)
      kbo(:, 2,13, 1) = (/ &
     &1.8623e-03_rb,2.3963e-03_rb,2.4362e-03_rb,2.2990e-03_rb,1.8223e-03_rb/)
      kbo(:, 3,13, 1) = (/ &
     &2.3718e-03_rb,3.1878e-03_rb,3.2540e-03_rb,3.0430e-03_rb,2.2915e-03_rb/)
      kbo(:, 4,13, 1) = (/ &
     &3.1921e-03_rb,4.3104e-03_rb,4.3590e-03_rb,3.9821e-03_rb,2.7709e-03_rb/)
      kbo(:, 5,13, 1) = (/ &
     &4.4313e-03_rb,5.8691e-03_rb,5.8118e-03_rb,5.1528e-03_rb,3.2426e-03_rb/)
      kbo(:, 1,14, 1) = (/ &
     &8.1496e-04_rb,1.1449e-03_rb,1.2521e-03_rb,1.2864e-03_rb,1.1830e-03_rb/)
      kbo(:, 2,14, 1) = (/ &
     &9.7291e-04_rb,1.4990e-03_rb,1.6700e-03_rb,1.7215e-03_rb,1.5434e-03_rb/)
      kbo(:, 3,14, 1) = (/ &
     &1.2487e-03_rb,2.0089e-03_rb,2.2296e-03_rb,2.2624e-03_rb,1.9289e-03_rb/)
      kbo(:, 4,14, 1) = (/ &
     &1.6881e-03_rb,2.7166e-03_rb,2.9639e-03_rb,2.9248e-03_rb,2.3207e-03_rb/)
      kbo(:, 5,14, 1) = (/ &
     &2.3524e-03_rb,3.6802e-03_rb,3.9082e-03_rb,3.7262e-03_rb,2.7008e-03_rb/)
      kbo(:, 1,15, 1) = (/ &
     &4.3177e-04_rb,7.4258e-04_rb,8.9651e-04_rb,1.0013e-03_rb,1.0098e-03_rb/)
      kbo(:, 2,15, 1) = (/ &
     &5.2028e-04_rb,9.8643e-04_rb,1.2013e-03_rb,1.3351e-03_rb,1.3087e-03_rb/)
      kbo(:, 3,15, 1) = (/ &
     &6.7289e-04_rb,1.3258e-03_rb,1.5978e-03_rb,1.7375e-03_rb,1.6233e-03_rb/)
      kbo(:, 4,15, 1) = (/ &
     &9.1389e-04_rb,1.7863e-03_rb,2.1013e-03_rb,2.2138e-03_rb,1.9402e-03_rb/)
      kbo(:, 5,15, 1) = (/ &
     &1.2783e-03_rb,2.4005e-03_rb,2.7322e-03_rb,2.7722e-03_rb,2.2422e-03_rb/)
      kbo(:, 1,16, 1) = (/ &
     &2.7315e-04_rb,5.4830e-04_rb,7.0218e-04_rb,8.1997e-04_rb,8.6240e-04_rb/)
      kbo(:, 2,16, 1) = (/ &
     &3.3240e-04_rb,7.3351e-04_rb,9.4112e-04_rb,1.0872e-03_rb,1.1098e-03_rb/)
      kbo(:, 3,16, 1) = (/ &
     &4.3314e-04_rb,9.8584e-04_rb,1.2457e-03_rb,1.4034e-03_rb,1.3664e-03_rb/)
      kbo(:, 4,16, 1) = (/ &
     &5.9092e-04_rb,1.3229e-03_rb,1.6223e-03_rb,1.7684e-03_rb,1.6211e-03_rb/)
      kbo(:, 5,16, 1) = (/ &
     &8.2878e-04_rb,1.7642e-03_rb,2.0878e-03_rb,2.1890e-03_rb,1.8647e-03_rb/)
      kbo(:, 1,17, 1) = (/ &
     &1.7965e-04_rb,4.1801e-04_rb,5.6155e-04_rb,6.7756e-04_rb,7.3362e-04_rb/)
      kbo(:, 2,17, 1) = (/ &
     &2.2058e-04_rb,5.6121e-04_rb,7.5064e-04_rb,8.9254e-04_rb,9.3681e-04_rb/)
      kbo(:, 3,17, 1) = (/ &
     &2.8939e-04_rb,7.5257e-04_rb,9.8597e-04_rb,1.1419e-03_rb,1.1452e-03_rb/)
      kbo(:, 4,17, 1) = (/ &
     &3.9639e-04_rb,1.0037e-03_rb,1.2718e-03_rb,1.4256e-03_rb,1.3514e-03_rb/)
      kbo(:, 5,17, 1) = (/ &
     &5.5678e-04_rb,1.3263e-03_rb,1.6209e-03_rb,1.7480e-03_rb,1.5479e-03_rb/)
      kbo(:, 1,18, 1) = (/ &
     &1.2144e-04_rb,3.2556e-04_rb,4.5465e-04_rb,5.6224e-04_rb,6.2157e-04_rb/)
      kbo(:, 2,18, 1) = (/ &
     &1.5036e-04_rb,4.3710e-04_rb,6.0499e-04_rb,7.3530e-04_rb,7.8763e-04_rb/)
      kbo(:, 3,18, 1) = (/ &
     &1.9855e-04_rb,5.8365e-04_rb,7.8791e-04_rb,9.3325e-04_rb,9.5745e-04_rb/)
      kbo(:, 4,18, 1) = (/ &
     &2.7288e-04_rb,7.7253e-04_rb,1.0079e-03_rb,1.1563e-03_rb,1.1247e-03_rb/)
      kbo(:, 5,18, 1) = (/ &
     &3.8348e-04_rb,1.0128e-03_rb,1.2724e-03_rb,1.4068e-03_rb,1.2838e-03_rb/)
      kbo(:, 1,19, 1) = (/ &
     &8.0908e-05_rb,2.5355e-04_rb,3.6780e-04_rb,4.6534e-04_rb,5.2420e-04_rb/)
      kbo(:, 2,19, 1) = (/ &
     &1.0097e-04_rb,3.3963e-04_rb,4.8637e-04_rb,6.0408e-04_rb,6.5997e-04_rb/)
      kbo(:, 3,19, 1) = (/ &
     &1.3403e-04_rb,4.5064e-04_rb,6.2806e-04_rb,7.6126e-04_rb,7.9854e-04_rb/)
      kbo(:, 4,19, 1) = (/ &
     &1.8471e-04_rb,5.9188e-04_rb,7.9637e-04_rb,9.3587e-04_rb,9.3436e-04_rb/)
      kbo(:, 5,19, 1) = (/ &
     &2.5970e-04_rb,7.6935e-04_rb,9.9605e-04_rb,1.1301e-03_rb,1.0631e-03_rb/)
      kbo(:, 1,20, 1) = (/ &
     &5.8651e-05_rb,2.0533e-04_rb,3.0468e-04_rb,3.8991e-04_rb,4.4299e-04_rb/)
      kbo(:, 2,20, 1) = (/ &
     &7.3858e-05_rb,2.7425e-04_rb,4.0056e-04_rb,5.0325e-04_rb,5.5439e-04_rb/)
      kbo(:, 3,20, 1) = (/ &
     &9.8558e-05_rb,3.6201e-04_rb,5.1389e-04_rb,6.3043e-04_rb,6.6745e-04_rb/)
      kbo(:, 4,20, 1) = (/ &
     &1.3624e-04_rb,4.7289e-04_rb,6.4732e-04_rb,7.7044e-04_rb,7.7770e-04_rb/)
      kbo(:, 5,20, 1) = (/ &
     &1.9168e-04_rb,6.1111e-04_rb,8.0443e-04_rb,9.2550e-04_rb,8.8213e-04_rb/)
      kbo(:, 1,21, 1) = (/ &
     &4.3969e-05_rb,1.6839e-04_rb,2.5404e-04_rb,3.2750e-04_rb,3.7386e-04_rb/)
      kbo(:, 2,21, 1) = (/ &
     &5.5807e-05_rb,2.2425e-04_rb,3.3214e-04_rb,4.2065e-04_rb,4.6519e-04_rb/)
      kbo(:, 3,21, 1) = (/ &
     &7.4871e-05_rb,2.9470e-04_rb,4.2375e-04_rb,5.2400e-04_rb,5.5744e-04_rb/)
      kbo(:, 4,21, 1) = (/ &
     &1.0381e-04_rb,3.8330e-04_rb,5.3089e-04_rb,6.3750e-04_rb,6.4727e-04_rb/)
      kbo(:, 5,21, 1) = (/ &
     &1.4611e-04_rb,4.9243e-04_rb,6.5637e-04_rb,7.6218e-04_rb,7.3199e-04_rb/)
      kbo(:, 1,22, 1) = (/ &
     &3.4337e-05_rb,1.4203e-04_rb,2.1658e-04_rb,2.8040e-04_rb,3.2005e-04_rb/)
      kbo(:, 2,22, 1) = (/ &
     &4.4095e-05_rb,1.8845e-04_rb,2.8132e-04_rb,3.5774e-04_rb,3.9514e-04_rb/)
      kbo(:, 3,22, 1) = (/ &
     &5.9627e-05_rb,2.4659e-04_rb,3.5661e-04_rb,4.4266e-04_rb,4.7049e-04_rb/)
      kbo(:, 4,22, 1) = (/ &
     &8.3001e-05_rb,3.1905e-04_rb,4.4430e-04_rb,5.3537e-04_rb,5.4306e-04_rb/)
      kbo(:, 5,22, 1) = (/ &
     &1.1688e-04_rb,4.0732e-04_rb,5.4669e-04_rb,6.3703e-04_rb,6.1162e-04_rb/)
      kbo(:, 1,23, 1) = (/ &
     &2.7088e-05_rb,1.2024e-04_rb,1.8486e-04_rb,2.4028e-04_rb,2.7381e-04_rb/)
      kbo(:, 2,23, 1) = (/ &
     &3.5182e-05_rb,1.5885e-04_rb,2.3849e-04_rb,3.0423e-04_rb,3.3547e-04_rb/)
      kbo(:, 3,23, 1) = (/ &
     &4.7910e-05_rb,2.0687e-04_rb,3.0043e-04_rb,3.7399e-04_rb,3.9686e-04_rb/)
      kbo(:, 4,23, 1) = (/ &
     &6.6975e-05_rb,2.6613e-04_rb,3.7259e-04_rb,4.4994e-04_rb,4.5570e-04_rb/)
      kbo(:, 5,23, 1) = (/ &
     &9.4305e-05_rb,3.3770e-04_rb,4.5596e-04_rb,5.3292e-04_rb,5.1097e-04_rb/)
      kbo(:, 1,24, 1) = (/ &
     &2.1382e-05_rb,1.0191e-04_rb,1.5778e-04_rb,2.0583e-04_rb,2.3424e-04_rb/)
      kbo(:, 2,24, 1) = (/ &
     &2.8085e-05_rb,1.3391e-04_rb,2.0214e-04_rb,2.5863e-04_rb,2.8484e-04_rb/)
      kbo(:, 3,24, 1) = (/ &
     &3.8505e-05_rb,1.7343e-04_rb,2.5313e-04_rb,3.1592e-04_rb,3.3477e-04_rb/)
      kbo(:, 4,24, 1) = (/ &
     &5.3997e-05_rb,2.2171e-04_rb,3.1228e-04_rb,3.7804e-04_rb,3.8231e-04_rb/)
      kbo(:, 5,24, 1) = (/ &
     &7.5953e-05_rb,2.7966e-04_rb,3.7978e-04_rb,4.4568e-04_rb,4.2681e-04_rb/)
      kbo(:, 1,25, 1) = (/ &
     &1.7217e-05_rb,8.6935e-05_rb,1.3520e-04_rb,1.7667e-04_rb,2.0051e-04_rb/)
      kbo(:, 2,25, 1) = (/ &
     &2.2860e-05_rb,1.1361e-04_rb,1.7201e-04_rb,2.2028e-04_rb,2.4198e-04_rb/)
      kbo(:, 3,25, 1) = (/ &
     &3.1540e-05_rb,1.4632e-04_rb,2.1426e-04_rb,2.6745e-04_rb,2.8247e-04_rb/)
      kbo(:, 4,25, 1) = (/ &
     &4.4330e-05_rb,1.8594e-04_rb,2.6272e-04_rb,3.1846e-04_rb,3.2089e-04_rb/)
      kbo(:, 5,25, 1) = (/ &
     &6.2260e-05_rb,2.3327e-04_rb,3.1770e-04_rb,3.7373e-04_rb,3.5666e-04_rb/)
      kbo(:, 1,26, 1) = (/ &
     &1.4222e-05_rb,7.4853e-05_rb,1.1659e-04_rb,1.5229e-04_rb,1.7205e-04_rb/)
      kbo(:, 2,26, 1) = (/ &
     &1.9097e-05_rb,9.7290e-05_rb,1.4733e-04_rb,1.8840e-04_rb,2.0595e-04_rb/)
      kbo(:, 3,26, 1) = (/ &
     &2.6498e-05_rb,1.2462e-04_rb,1.8247e-04_rb,2.2740e-04_rb,2.3871e-04_rb/)
      kbo(:, 4,26, 1) = (/ &
     &3.7304e-05_rb,1.5749e-04_rb,2.2244e-04_rb,2.6950e-04_rb,2.6966e-04_rb/)
      kbo(:, 5,26, 1) = (/ &
     &5.2263e-05_rb,1.9669e-04_rb,2.6764e-04_rb,3.1472e-04_rb,2.9848e-04_rb/)
      kbo(:, 1,27, 1) = (/ &
     &1.2263e-05_rb,6.5144e-05_rb,1.0108e-04_rb,1.3156e-04_rb,1.4744e-04_rb/)
      kbo(:, 2,27, 1) = (/ &
     &1.6644e-05_rb,8.4290e-05_rb,1.2696e-04_rb,1.6163e-04_rb,1.7509e-04_rb/)
      kbo(:, 3,27, 1) = (/ &
     &2.3202e-05_rb,1.0755e-04_rb,1.5651e-04_rb,1.9419e-04_rb,2.0159e-04_rb/)
      kbo(:, 4,27, 1) = (/ &
     &3.2669e-05_rb,1.3540e-04_rb,1.9003e-04_rb,2.2917e-04_rb,2.2657e-04_rb/)
      kbo(:, 5,27, 1) = (/ &
     &4.5636e-05_rb,1.6849e-04_rb,2.2778e-04_rb,2.6648e-04_rb,2.4966e-04_rb/)
      kbo(:, 1,28, 1) = (/ &
     &1.0907e-05_rb,5.7118e-05_rb,8.7946e-05_rb,1.1378e-04_rb,1.2622e-04_rb/)
      kbo(:, 2,28, 1) = (/ &
     &1.4925e-05_rb,7.3661e-05_rb,1.0996e-04_rb,1.3907e-04_rb,1.4872e-04_rb/)
      kbo(:, 3,28, 1) = (/ &
     &2.0909e-05_rb,9.3671e-05_rb,1.3497e-04_rb,1.6629e-04_rb,1.7014e-04_rb/)
      kbo(:, 4,28, 1) = (/ &
     &2.9425e-05_rb,1.1761e-04_rb,1.6330e-04_rb,1.9540e-04_rb,1.9022e-04_rb/)
      kbo(:, 5,28, 1) = (/ &
     &4.0966e-05_rb,1.4604e-04_rb,1.9535e-04_rb,2.2654e-04_rb,2.0877e-04_rb/)
      kbo(:, 1,29, 1) = (/ &
     &1.0363e-05_rb,5.1128e-05_rb,7.7457e-05_rb,9.9099e-05_rb,1.0807e-04_rb/)
      kbo(:, 2,29, 1) = (/ &
     &1.4287e-05_rb,6.5810e-05_rb,9.6485e-05_rb,1.2056e-04_rb,1.2631e-04_rb/)
      kbo(:, 3,29, 1) = (/ &
     &2.0078e-05_rb,8.3622e-05_rb,1.1822e-04_rb,1.4364e-04_rb,1.4366e-04_rb/)
      kbo(:, 4,29, 1) = (/ &
     &2.8213e-05_rb,1.0495e-04_rb,1.4288e-04_rb,1.6839e-04_rb,1.5982e-04_rb/)
      kbo(:, 5,29, 1) = (/ &
     &3.9150e-05_rb,1.3025e-04_rb,1.7086e-04_rb,1.9491e-04_rb,1.7468e-04_rb/)
      kbo(:, 1,30, 1) = (/ &
     &1.0130e-05_rb,4.6183e-05_rb,6.8547e-05_rb,8.6516e-05_rb,9.2360e-05_rb/)
      kbo(:, 2,30, 1) = (/ &
     &1.4060e-05_rb,5.9445e-05_rb,8.5241e-05_rb,1.0481e-04_rb,1.0719e-04_rb/)
      kbo(:, 3,30, 1) = (/ &
     &1.9789e-05_rb,7.5542e-05_rb,1.0433e-04_rb,1.2454e-04_rb,1.2117e-04_rb/)
      kbo(:, 4,30, 1) = (/ &
     &2.7737e-05_rb,9.4871e-05_rb,1.2613e-04_rb,1.4578e-04_rb,1.3421e-04_rb/)
      kbo(:, 5,30, 1) = (/ &
     &3.8345e-05_rb,1.1778e-04_rb,1.5088e-04_rb,1.6869e-04_rb,1.4612e-04_rb/)
      kbo(:, 1,31, 1) = (/ &
     &1.0468e-05_rb,4.2679e-05_rb,6.1533e-05_rb,7.6101e-05_rb,7.8936e-05_rb/)
      kbo(:, 2,31, 1) = (/ &
     &1.4602e-05_rb,5.5015e-05_rb,7.6472e-05_rb,9.1912e-05_rb,9.0925e-05_rb/)
      kbo(:, 3,31, 1) = (/ &
     &2.0565e-05_rb,7.0045e-05_rb,9.3754e-05_rb,1.0914e-04_rb,1.0226e-04_rb/)
      kbo(:, 4,31, 1) = (/ &
     &2.8741e-05_rb,8.8177e-05_rb,1.1361e-04_rb,1.2783e-04_rb,1.1277e-04_rb/)
      kbo(:, 5,31, 1) = (/ &
     &3.9551e-05_rb,1.0965e-04_rb,1.3625e-04_rb,1.4816e-04_rb,1.2240e-04_rb/)
      kbo(:, 1,32, 1) = (/ &
     &1.0980e-05_rb,3.9860e-05_rb,5.5596e-05_rb,6.7154e-05_rb,6.7396e-05_rb/)
      kbo(:, 2,32, 1) = (/ &
     &1.5378e-05_rb,5.1481e-05_rb,6.9207e-05_rb,8.1024e-05_rb,7.7105e-05_rb/)
      kbo(:, 3,32, 1) = (/ &
     &2.1641e-05_rb,6.5723e-05_rb,8.5005e-05_rb,9.6172e-05_rb,8.6258e-05_rb/)
      kbo(:, 4,32, 1) = (/ &
     &3.0131e-05_rb,8.2891e-05_rb,1.0329e-04_rb,1.1283e-04_rb,9.4767e-05_rb/)
      kbo(:, 5,32, 1) = (/ &
     &4.1276e-05_rb,1.0326e-04_rb,1.2427e-04_rb,1.3104e-04_rb,1.0241e-04_rb/)
      kbo(:, 1,33, 1) = (/ &
     &1.1748e-05_rb,3.7749e-05_rb,5.0777e-05_rb,5.9616e-05_rb,5.7446e-05_rb/)
      kbo(:, 2,33, 1) = (/ &
     &1.6501e-05_rb,4.8915e-05_rb,6.3363e-05_rb,7.1901e-05_rb,6.5353e-05_rb/)
      kbo(:, 3,33, 1) = (/ &
     &2.3188e-05_rb,6.2624e-05_rb,7.8070e-05_rb,8.5481e-05_rb,7.2776e-05_rb/)
      kbo(:, 4,33, 1) = (/ &
     &3.2156e-05_rb,7.9179e-05_rb,9.5194e-05_rb,1.0052e-04_rb,7.9571e-05_rb/)
      kbo(:, 5,33, 1) = (/ &
     &4.3815e-05_rb,9.8786e-05_rb,1.1493e-04_rb,1.1713e-04_rb,8.5692e-05_rb/)
      kbo(:, 1,34, 1) = (/ &
     &1.2122e-05_rb,3.5250e-05_rb,4.5906e-05_rb,5.2480e-05_rb,4.8658e-05_rb/)
      kbo(:, 2,34, 1) = (/ &
     &1.7058e-05_rb,4.5800e-05_rb,5.7395e-05_rb,6.3326e-05_rb,5.5101e-05_rb/)
      kbo(:, 3,34, 1) = (/ &
     &2.3918e-05_rb,5.8762e-05_rb,7.0946e-05_rb,7.5460e-05_rb,6.1101e-05_rb/)
      kbo(:, 4,34, 1) = (/ &
     &3.3054e-05_rb,7.4431e-05_rb,8.6803e-05_rb,8.8957e-05_rb,6.6540e-05_rb/)
      kbo(:, 5,34, 1) = (/ &
     &4.4827e-05_rb,9.2907e-05_rb,1.0510e-04_rb,1.0393e-04_rb,7.1470e-05_rb/)
      kbo(:, 1,35, 1) = (/ &
     &1.2136e-05_rb,3.2274e-05_rb,4.0784e-05_rb,4.5493e-05_rb,4.0671e-05_rb/)
      kbo(:, 2,35, 1) = (/ &
     &1.7085e-05_rb,4.2082e-05_rb,5.1198e-05_rb,5.5056e-05_rb,4.5918e-05_rb/)
      kbo(:, 3,35, 1) = (/ &
     &2.3916e-05_rb,5.4183e-05_rb,6.3553e-05_rb,6.5795e-05_rb,5.0794e-05_rb/)
      kbo(:, 4,35, 1) = (/ &
     &3.2983e-05_rb,6.8784e-05_rb,7.8061e-05_rb,7.7848e-05_rb,5.5215e-05_rb/)
      kbo(:, 5,35, 1) = (/ &
     &4.4608e-05_rb,8.5962e-05_rb,9.4799e-05_rb,9.1316e-05_rb,5.9185e-05_rb/)
      kbo(:, 1,36, 1) = (/ &
     &1.1639e-05_rb,2.8710e-05_rb,3.5389e-05_rb,3.8647e-05_rb,3.3496e-05_rb/)
      kbo(:, 2,36, 1) = (/ &
     &1.6387e-05_rb,3.7611e-05_rb,4.4660e-05_rb,4.6982e-05_rb,3.7819e-05_rb/)
      kbo(:, 3,36, 1) = (/ &
     &2.2931e-05_rb,4.8613e-05_rb,5.5727e-05_rb,5.6405e-05_rb,4.1793e-05_rb/)
      kbo(:, 4,36, 1) = (/ &
     &3.1614e-05_rb,6.1937e-05_rb,6.8745e-05_rb,6.7054e-05_rb,4.5416e-05_rb/)
      kbo(:, 5,36, 1) = (/ &
     &4.2736e-05_rb,7.7667e-05_rb,8.3834e-05_rb,7.9002e-05_rb,4.8700e-05_rb/)
      kbo(:, 1,37, 1) = (/ &
     &1.0334e-05_rb,2.4228e-05_rb,2.9412e-05_rb,3.1719e-05_rb,2.7018e-05_rb/)
      kbo(:, 2,37, 1) = (/ &
     &1.4545e-05_rb,3.1907e-05_rb,3.7329e-05_rb,3.8760e-05_rb,3.0586e-05_rb/)
      kbo(:, 3,37, 1) = (/ &
     &2.0385e-05_rb,4.1462e-05_rb,4.6849e-05_rb,4.6806e-05_rb,3.3891e-05_rb/)
      kbo(:, 4,37, 1) = (/ &
     &2.8167e-05_rb,5.3075e-05_rb,5.8122e-05_rb,5.5962e-05_rb,3.6915e-05_rb/)
      kbo(:, 5,37, 1) = (/ &
     &3.8188e-05_rb,6.6903e-05_rb,7.1250e-05_rb,6.6274e-05_rb,3.9679e-05_rb/)
      kbo(:, 1,38, 1) = (/ &
     &9.2873e-06_rb,2.0587e-05_rb,2.4545e-05_rb,2.6080e-05_rb,2.1759e-05_rb/)
      kbo(:, 2,38, 1) = (/ &
     &1.3062e-05_rb,2.7262e-05_rb,3.1359e-05_rb,3.2085e-05_rb,2.4714e-05_rb/)
      kbo(:, 3,38, 1) = (/ &
     &1.8340e-05_rb,3.5628e-05_rb,3.9617e-05_rb,3.8992e-05_rb,2.7465e-05_rb/)
      kbo(:, 4,38, 1) = (/ &
     &2.5394e-05_rb,4.5875e-05_rb,4.9461e-05_rb,4.6884e-05_rb,2.9998e-05_rb/)
      kbo(:, 5,38, 1) = (/ &
     &3.4528e-05_rb,5.8131e-05_rb,6.0967e-05_rb,5.5861e-05_rb,3.2334e-05_rb/)
      kbo(:, 1,39, 1) = (/ &
     &8.5453e-06_rb,1.7761e-05_rb,2.0713e-05_rb,2.1598e-05_rb,1.7527e-05_rb/)
      kbo(:, 2,39, 1) = (/ &
     &1.2008e-05_rb,2.3655e-05_rb,2.6655e-05_rb,2.6765e-05_rb,1.9983e-05_rb/)
      kbo(:, 3,39, 1) = (/ &
     &1.6881e-05_rb,3.1100e-05_rb,3.3909e-05_rb,3.2745e-05_rb,2.2276e-05_rb/)
      kbo(:, 4,39, 1) = (/ &
     &2.3421e-05_rb,4.0278e-05_rb,4.2620e-05_rb,3.9657e-05_rb,2.4422e-05_rb/)
      kbo(:, 5,39, 1) = (/ &
     &3.1935e-05_rb,5.1364e-05_rb,5.2889e-05_rb,4.7569e-05_rb,2.6373e-05_rb/)
      kbo(:, 1,40, 1) = (/ &
     &7.1461e-06_rb,1.4361e-05_rb,1.6604e-05_rb,1.7207e-05_rb,1.3880e-05_rb/)
      kbo(:, 2,40, 1) = (/ &
     &1.0009e-05_rb,1.9228e-05_rb,2.1496e-05_rb,2.1462e-05_rb,1.5923e-05_rb/)
      kbo(:, 3,40, 1) = (/ &
     &1.4089e-05_rb,2.5420e-05_rb,2.7521e-05_rb,2.6424e-05_rb,1.7845e-05_rb/)
      kbo(:, 4,40, 1) = (/ &
     &1.9620e-05_rb,3.3119e-05_rb,3.4800e-05_rb,3.2193e-05_rb,1.9645e-05_rb/)
      kbo(:, 5,40, 1) = (/ &
     &2.6885e-05_rb,4.2499e-05_rb,4.3444e-05_rb,3.8836e-05_rb,2.1269e-05_rb/)
      kbo(:, 1,41, 1) = (/ &
     &5.9442e-06_rb,1.1549e-05_rb,1.3242e-05_rb,1.3641e-05_rb,1.0951e-05_rb/)
      kbo(:, 2,41, 1) = (/ &
     &8.2890e-06_rb,1.5538e-05_rb,1.7245e-05_rb,1.7129e-05_rb,1.2647e-05_rb/)
      kbo(:, 3,41, 1) = (/ &
     &1.1675e-05_rb,2.0658e-05_rb,2.2220e-05_rb,2.1226e-05_rb,1.4267e-05_rb/)
      kbo(:, 4,41, 1) = (/ &
     &1.6317e-05_rb,2.7083e-05_rb,2.8276e-05_rb,2.6013e-05_rb,1.5751e-05_rb/)
      kbo(:, 5,41, 1) = (/ &
     &2.2462e-05_rb,3.4966e-05_rb,3.5519e-05_rb,3.1566e-05_rb,1.7118e-05_rb/)
      kbo(:, 1,42, 1) = (/ &
     &5.0039e-06_rb,9.3487e-06_rb,1.0601e-05_rb,1.0831e-05_rb,8.6234e-06_rb/)
      kbo(:, 2,42, 1) = (/ &
     &6.9419e-06_rb,1.2631e-05_rb,1.3888e-05_rb,1.3698e-05_rb,1.0034e-05_rb/)
      kbo(:, 3,42, 1) = (/ &
     &9.7739e-06_rb,1.6890e-05_rb,1.8015e-05_rb,1.7096e-05_rb,1.1383e-05_rb/)
      kbo(:, 4,42, 1) = (/ &
     &1.3702e-05_rb,2.2284e-05_rb,2.3078e-05_rb,2.1084e-05_rb,1.2617e-05_rb/)
      kbo(:, 5,42, 1) = (/ &
     &1.8946e-05_rb,2.8958e-05_rb,2.9188e-05_rb,2.5746e-05_rb,1.3767e-05_rb/)
      kbo(:, 1,43, 1) = (/ &
     &4.1873e-06_rb,7.4795e-06_rb,8.3830e-06_rb,8.4906e-06_rb,6.7096e-06_rb/)
      kbo(:, 2,43, 1) = (/ &
     &5.7609e-06_rb,1.0144e-05_rb,1.1051e-05_rb,1.0826e-05_rb,7.8872e-06_rb/)
      kbo(:, 3,43, 1) = (/ &
     &8.0947e-06_rb,1.3657e-05_rb,1.4441e-05_rb,1.3617e-05_rb,9.0007e-06_rb/)
      kbo(:, 4,43, 1) = (/ &
     &1.1379e-05_rb,1.8143e-05_rb,1.8645e-05_rb,1.6928e-05_rb,1.0041e-05_rb/)
      kbo(:, 5,43, 1) = (/ &
     &1.5813e-05_rb,2.3749e-05_rb,2.3760e-05_rb,2.0820e-05_rb,1.1010e-05_rb/)
      kbo(:, 1,44, 1) = (/ &
     &3.5435e-06_rb,5.9971e-06_rb,6.6151e-06_rb,6.6257e-06_rb,5.1813e-06_rb/)
      kbo(:, 2,44, 1) = (/ &
     &4.8184e-06_rb,8.1565e-06_rb,8.7802e-06_rb,8.5231e-06_rb,6.1554e-06_rb/)
      kbo(:, 3,44, 1) = (/ &
     &6.7381e-06_rb,1.1044e-05_rb,1.1560e-05_rb,1.0814e-05_rb,7.0815e-06_rb/)
      kbo(:, 4,44, 1) = (/ &
     &9.4910e-06_rb,1.4784e-05_rb,1.5050e-05_rb,1.3555e-05_rb,7.9532e-06_rb/)
      kbo(:, 5,44, 1) = (/ &
     &1.3248e-05_rb,1.9506e-05_rb,1.9344e-05_rb,1.6815e-05_rb,8.7723e-06_rb/)
      kbo(:, 1,45, 1) = (/ &
     &3.0781e-06_rb,4.8864e-06_rb,5.2730e-06_rb,5.1910e-06_rb,3.9885e-06_rb/)
      kbo(:, 2,45, 1) = (/ &
     &4.1281e-06_rb,6.6470e-06_rb,7.0382e-06_rb,6.7401e-06_rb,4.7888e-06_rb/)
      kbo(:, 3,45, 1) = (/ &
     &5.7272e-06_rb,9.0419e-06_rb,9.3373e-06_rb,8.6331e-06_rb,5.5575e-06_rb/)
      kbo(:, 4,45, 1) = (/ &
     &8.0675e-06_rb,1.2195e-05_rb,1.2269e-05_rb,1.0924e-05_rb,6.2888e-06_rb/)
      kbo(:, 5,45, 1) = (/ &
     &1.1304e-05_rb,1.6230e-05_rb,1.5911e-05_rb,1.3670e-05_rb,6.9809e-06_rb/)
      kbo(:, 1,46, 1) = (/ &
     &2.7031e-06_rb,4.0051e-06_rb,4.2052e-06_rb,4.0513e-06_rb,3.0391e-06_rb/)
      kbo(:, 2,46, 1) = (/ &
     &3.5629e-06_rb,5.4291e-06_rb,5.6384e-06_rb,5.3109e-06_rb,3.6957e-06_rb/)
      kbo(:, 3,46, 1) = (/ &
     &4.8917e-06_rb,7.4052e-06_rb,7.5318e-06_rb,6.8686e-06_rb,4.3368e-06_rb/)
      kbo(:, 4,46, 1) = (/ &
     &6.8735e-06_rb,1.0056e-05_rb,9.9863e-06_rb,8.7795e-06_rb,4.9486e-06_rb/)
      kbo(:, 5,46, 1) = (/ &
     &9.6601e-06_rb,1.3493e-05_rb,1.3073e-05_rb,1.1090e-05_rb,5.5328e-06_rb/)
      kbo(:, 1,47, 1) = (/ &
     &2.3513e-06_rb,3.2459e-06_rb,3.3109e-06_rb,3.1126e-06_rb,2.2809e-06_rb/)
      kbo(:, 2,47, 1) = (/ &
     &3.0285e-06_rb,4.3632e-06_rb,4.4435e-06_rb,4.1169e-06_rb,2.8160e-06_rb/)
      kbo(:, 3,47, 1) = (/ &
     &4.0963e-06_rb,5.9519e-06_rb,5.9675e-06_rb,5.3774e-06_rb,3.3494e-06_rb/)
      kbo(:, 4,47, 1) = (/ &
     &5.7200e-06_rb,8.1250e-06_rb,7.9789e-06_rb,6.9425e-06_rb,3.8611e-06_rb/)
      kbo(:, 5,47, 1) = (/ &
     &8.0548e-06_rb,1.0990e-05_rb,1.0545e-05_rb,8.8570e-06_rb,4.3522e-06_rb/)
      kbo(:, 1,48, 1) = (/ &
     &2.1277e-06_rb,2.7115e-06_rb,2.6627e-06_rb,2.4165e-06_rb,1.7021e-06_rb/)
      kbo(:, 2,48, 1) = (/ &
     &2.6703e-06_rb,3.5982e-06_rb,3.5641e-06_rb,3.2218e-06_rb,2.1347e-06_rb/)
      kbo(:, 3,48, 1) = (/ &
     &3.5476e-06_rb,4.8904e-06_rb,4.8030e-06_rb,4.2482e-06_rb,2.5746e-06_rb/)
      kbo(:, 4,48, 1) = (/ &
     &4.9045e-06_rb,6.6952e-06_rb,6.4687e-06_rb,5.5412e-06_rb,3.0044e-06_rb/)
      kbo(:, 5,48, 1) = (/ &
     &6.9032e-06_rb,9.1233e-06_rb,8.6345e-06_rb,7.1468e-06_rb,3.4162e-06_rb/)
      kbo(:, 1,49, 1) = (/ &
     &2.0328e-06_rb,2.3718e-06_rb,2.2199e-06_rb,1.9157e-06_rb,1.2610e-06_rb/)
      kbo(:, 2,49, 1) = (/ &
     &2.4767e-06_rb,3.0887e-06_rb,2.9459e-06_rb,2.5646e-06_rb,1.6091e-06_rb/)
      kbo(:, 3,49, 1) = (/ &
     &3.2245e-06_rb,4.1594e-06_rb,3.9687e-06_rb,3.4109e-06_rb,1.9688e-06_rb/)
      kbo(:, 4,49, 1) = (/ &
     &4.3950e-06_rb,5.6964e-06_rb,5.3766e-06_rb,4.4963e-06_rb,2.3270e-06_rb/)
      kbo(:, 5,49, 1) = (/ &
     &6.1628e-06_rb,7.8088e-06_rb,7.2467e-06_rb,5.8704e-06_rb,2.6722e-06_rb/)
      kbo(:, 1,50, 1) = (/ &
     &1.8800e-06_rb,2.0468e-06_rb,1.8384e-06_rb,1.5142e-06_rb,9.3123e-07_rb/)
      kbo(:, 2,50, 1) = (/ &
     &2.2261e-06_rb,2.6068e-06_rb,2.4062e-06_rb,2.0250e-06_rb,1.2100e-06_rb/)
      kbo(:, 3,50, 1) = (/ &
     &2.8323e-06_rb,3.4659e-06_rb,3.2264e-06_rb,2.7071e-06_rb,1.5035e-06_rb/)
      kbo(:, 4,50, 1) = (/ &
     &3.7999e-06_rb,4.7249e-06_rb,4.3788e-06_rb,3.5953e-06_rb,1.7997e-06_rb/)
      kbo(:, 5,50, 1) = (/ &
     &5.2884e-06_rb,6.4943e-06_rb,5.9393e-06_rb,4.7367e-06_rb,2.0882e-06_rb/)
      kbo(:, 1,51, 1) = (/ &
     &1.7210e-06_rb,1.7678e-06_rb,1.5277e-06_rb,1.2003e-06_rb,6.8440e-07_rb/)
      kbo(:, 2,51, 1) = (/ &
     &1.9818e-06_rb,2.1939e-06_rb,1.9638e-06_rb,1.5963e-06_rb,9.0698e-07_rb/)
      kbo(:, 3,51, 1) = (/ &
     &2.4588e-06_rb,2.8718e-06_rb,2.6104e-06_rb,2.1370e-06_rb,1.1441e-06_rb/)
      kbo(:, 4,51, 1) = (/ &
     &3.2428e-06_rb,3.8817e-06_rb,3.5358e-06_rb,2.8541e-06_rb,1.3878e-06_rb/)
      kbo(:, 5,51, 1) = (/ &
     &4.4645e-06_rb,5.3335e-06_rb,4.8140e-06_rb,3.7885e-06_rb,1.6275e-06_rb/)
      kbo(:, 1,52, 1) = (/ &
     &1.6367e-06_rb,1.5882e-06_rb,1.3178e-06_rb,9.7791e-07_rb,4.9920e-07_rb/)
      kbo(:, 2,52, 1) = (/ &
     &1.8355e-06_rb,1.9143e-06_rb,1.6527e-06_rb,1.2853e-06_rb,6.7537e-07_rb/)
      kbo(:, 3,52, 1) = (/ &
     &2.2162e-06_rb,2.4540e-06_rb,2.1676e-06_rb,1.7156e-06_rb,8.6642e-07_rb/)
      kbo(:, 4,52, 1) = (/ &
     &2.8651e-06_rb,3.2774e-06_rb,2.9188e-06_rb,2.2996e-06_rb,1.0660e-06_rb/)
      kbo(:, 5,52, 1) = (/ &
     &3.8903e-06_rb,4.4871e-06_rb,3.9803e-06_rb,3.0728e-06_rb,1.2649e-06_rb/)
      kbo(:, 1,53, 1) = (/ &
     &1.6359e-06_rb,1.5065e-06_rb,1.1971e-06_rb,8.3215e-07_rb,3.6108e-07_rb/)
      kbo(:, 2,53, 1) = (/ &
     &1.7884e-06_rb,1.7570e-06_rb,1.4552e-06_rb,1.0704e-06_rb,4.9954e-07_rb/)
      kbo(:, 3,53, 1) = (/ &
     &2.1010e-06_rb,2.1926e-06_rb,1.8706e-06_rb,1.4156e-06_rb,6.5257e-07_rb/)
      kbo(:, 4,53, 1) = (/ &
     &2.6543e-06_rb,2.8821e-06_rb,2.4935e-06_rb,1.8975e-06_rb,8.1519e-07_rb/)
      kbo(:, 5,53, 1) = (/ &
     &3.5462e-06_rb,3.9143e-06_rb,3.3945e-06_rb,2.5503e-06_rb,9.8011e-07_rb/)
      kbo(:, 1,54, 1) = (/ &
     &1.4300e-06_rb,1.2802e-06_rb,9.9257e-07_rb,6.6497e-07_rb,2.6129e-07_rb/)
      kbo(:, 2,54, 1) = (/ &
     &1.5262e-06_rb,1.4508e-06_rb,1.1768e-06_rb,8.3958e-07_rb,3.6958e-07_rb/)
      kbo(:, 3,54, 1) = (/ &
     &1.7484e-06_rb,1.7685e-06_rb,1.4827e-06_rb,1.0992e-06_rb,4.9169e-07_rb/)
      kbo(:, 4,54, 1) = (/ &
     &2.1569e-06_rb,2.2883e-06_rb,1.9539e-06_rb,1.4679e-06_rb,6.2244e-07_rb/)
      kbo(:, 5,54, 1) = (/ &
     &2.8354e-06_rb,3.0764e-06_rb,2.6455e-06_rb,1.9748e-06_rb,7.5773e-07_rb/)
      kbo(:, 1,55, 1) = (/ &
     &1.1496e-06_rb,1.0107e-06_rb,7.7300e-07_rb,5.0639e-07_rb,1.8790e-07_rb/)
      kbo(:, 2,55, 1) = (/ &
     &1.1995e-06_rb,1.1187e-06_rb,8.9719e-07_rb,6.3009e-07_rb,2.7269e-07_rb/)
      kbo(:, 3,55, 1) = (/ &
     &1.3422e-06_rb,1.3346e-06_rb,1.1096e-06_rb,8.1740e-07_rb,3.6933e-07_rb/)
      kbo(:, 4,55, 1) = (/ &
     &1.6169e-06_rb,1.6973e-06_rb,1.4454e-06_rb,1.0866e-06_rb,4.7429e-07_rb/)
      kbo(:, 5,55, 1) = (/ &
     &2.0871e-06_rb,2.2587e-06_rb,1.9432e-06_rb,1.4592e-06_rb,5.8406e-07_rb/)
      kbo(:, 1,56, 1) = (/ &
     &9.3098e-07_rb,8.0374e-07_rb,6.0662e-07_rb,3.8780e-07_rb,1.3395e-07_rb/)
      kbo(:, 2,56, 1) = (/ &
     &9.4853e-07_rb,8.6979e-07_rb,6.8824e-07_rb,4.7493e-07_rb,1.9978e-07_rb/)
      kbo(:, 3,56, 1) = (/ &
     &1.0383e-06_rb,1.0136e-06_rb,8.3527e-07_rb,6.0949e-07_rb,2.7592e-07_rb/)
      kbo(:, 4,56, 1) = (/ &
     &1.2210e-06_rb,1.2652e-06_rb,1.0730e-06_rb,8.0519e-07_rb,3.6001e-07_rb/)
      kbo(:, 5,56, 1) = (/ &
     &1.5443e-06_rb,1.6630e-06_rb,1.4302e-06_rb,1.0789e-06_rb,4.4877e-07_rb/)
      kbo(:, 1,57, 1) = (/ &
     &7.5869e-07_rb,6.4433e-07_rb,4.7913e-07_rb,2.9908e-07_rb,9.4559e-08_rb/)
      kbo(:, 2,57, 1) = (/ &
     &7.5680e-07_rb,6.8154e-07_rb,5.3140e-07_rb,3.5955e-07_rb,1.4504e-07_rb/)
      kbo(:, 3,57, 1) = (/ &
     &8.0958e-07_rb,7.7512e-07_rb,6.3252e-07_rb,4.5567e-07_rb,2.0501e-07_rb/)
      kbo(:, 4,57, 1) = (/ &
     &9.2953e-07_rb,9.4815e-07_rb,7.9932e-07_rb,5.9759e-07_rb,2.7208e-07_rb/)
      kbo(:, 5,57, 1) = (/ &
     &1.1491e-06_rb,1.2288e-06_rb,1.0552e-06_rb,7.9844e-07_rb,3.4355e-07_rb/)
      kbo(:, 1,58, 1) = (/ &
     &1.6388e-07_rb,1.6804e-07_rb,1.5881e-07_rb,1.3627e-07_rb,6.6479e-08_rb/)
      kbo(:, 2,58, 1) = (/ &
     &1.6042e-07_rb,1.7409e-07_rb,1.7274e-07_rb,1.6106e-07_rb,1.0511e-07_rb/)
      kbo(:, 3,58, 1) = (/ &
     &1.6792e-07_rb,1.9377e-07_rb,2.0193e-07_rb,2.0193e-07_rb,1.5191e-07_rb/)
      kbo(:, 4,58, 1) = (/ &
     &1.8861e-07_rb,2.3235e-07_rb,2.5117e-07_rb,2.6306e-07_rb,2.0537e-07_rb/)
      kbo(:, 5,58, 1) = (/ &
     &2.2793e-07_rb,2.9684e-07_rb,3.2824e-07_rb,3.5018e-07_rb,2.6292e-07_rb/)
      kbo(:, 1,59, 1) = (/ &
     &1.5765e-07_rb,1.5618e-07_rb,1.3958e-07_rb,1.1007e-07_rb,5.0739e-08_rb/)
      kbo(:, 2,59, 1) = (/ &
     &1.5325e-07_rb,1.6219e-07_rb,1.5348e-07_rb,1.3308e-07_rb,8.1077e-08_rb/)
      kbo(:, 3,59, 1) = (/ &
     &1.5900e-07_rb,1.8099e-07_rb,1.8121e-07_rb,1.6979e-07_rb,1.1831e-07_rb/)
      kbo(:, 4,59, 1) = (/ &
     &1.7701e-07_rb,2.1714e-07_rb,2.2693e-07_rb,2.2335e-07_rb,1.6137e-07_rb/)
      kbo(:, 5,59, 1) = (/ &
     &2.1198e-07_rb,2.7749e-07_rb,2.9703e-07_rb,2.9803e-07_rb,2.0840e-07_rb/)
      kbo(:, 1,13, 2) = (/ &
     &3.6279e-03_rb,6.1760e-03_rb,7.4137e-03_rb,8.1756e-03_rb,7.6678e-03_rb/)
      kbo(:, 2,13, 2) = (/ &
     &4.7275e-03_rb,8.3099e-03_rb,9.8779e-03_rb,1.0590e-02_rb,9.2190e-03_rb/)
      kbo(:, 3,13, 2) = (/ &
     &6.7817e-03_rb,1.1546e-02_rb,1.3324e-02_rb,1.3748e-02_rb,1.0778e-02_rb/)
      kbo(:, 4,13, 2) = (/ &
     &1.0391e-02_rb,1.6290e-02_rb,1.8048e-02_rb,1.7786e-02_rb,1.2299e-02_rb/)
      kbo(:, 5,13, 2) = (/ &
     &1.6231e-02_rb,2.2973e-02_rb,2.4369e-02_rb,2.2866e-02_rb,1.3775e-02_rb/)
      kbo(:, 1,14, 2) = (/ &
     &1.9069e-03_rb,4.2185e-03_rb,5.5573e-03_rb,6.5712e-03_rb,6.6234e-03_rb/)
      kbo(:, 2,14, 2) = (/ &
     &2.5180e-03_rb,5.7108e-03_rb,7.3682e-03_rb,8.4311e-03_rb,7.9352e-03_rb/)
      kbo(:, 3,14, 2) = (/ &
     &3.6614e-03_rb,7.8938e-03_rb,9.8216e-03_rb,1.0792e-02_rb,9.2480e-03_rb/)
      kbo(:, 4,14, 2) = (/ &
     &5.6530e-03_rb,1.1002e-02_rb,1.3072e-02_rb,1.3740e-02_rb,1.0517e-02_rb/)
      kbo(:, 5,14, 2) = (/ &
     &8.8619e-03_rb,1.5292e-02_rb,1.7338e-02_rb,1.7343e-02_rb,1.1731e-02_rb/)
      kbo(:, 1,15, 2) = (/ &
     &1.0269e-03_rb,3.0313e-03_rb,4.3258e-03_rb,5.3827e-03_rb,5.7089e-03_rb/)
      kbo(:, 2,15, 2) = (/ &
     &1.3765e-03_rb,4.1045e-03_rb,5.6876e-03_rb,6.8515e-03_rb,6.8087e-03_rb/)
      kbo(:, 3,15, 2) = (/ &
     &2.0273e-03_rb,5.6227e-03_rb,7.4759e-03_rb,8.6449e-03_rb,7.9051e-03_rb/)
      kbo(:, 4,15, 2) = (/ &
     &3.1553e-03_rb,7.7214e-03_rb,9.7810e-03_rb,1.0828e-02_rb,8.9583e-03_rb/)
      kbo(:, 5,15, 2) = (/ &
     &4.9678e-03_rb,1.0557e-02_rb,1.2725e-02_rb,1.3447e-02_rb,9.9499e-03_rb/)
      kbo(:, 1,16, 2) = (/ &
     &6.6110e-04_rb,2.3925e-03_rb,3.5607e-03_rb,4.5272e-03_rb,4.9025e-03_rb/)
      kbo(:, 2,16, 2) = (/ &
     &8.9912e-04_rb,3.2391e-03_rb,4.6534e-03_rb,5.7377e-03_rb,5.8262e-03_rb/)
      kbo(:, 3,16, 2) = (/ &
     &1.3419e-03_rb,4.4091e-03_rb,6.0651e-03_rb,7.1821e-03_rb,6.7308e-03_rb/)
      kbo(:, 4,16, 2) = (/ &
     &2.1065e-03_rb,5.9970e-03_rb,7.8566e-03_rb,8.9138e-03_rb,7.5936e-03_rb/)
      kbo(:, 5,16, 2) = (/ &
     &3.3254e-03_rb,8.1145e-03_rb,1.0111e-02_rb,1.0963e-02_rb,8.4021e-03_rb/)
      kbo(:, 1,17, 2) = (/ &
     &4.4242e-04_rb,1.9273e-03_rb,2.9584e-03_rb,3.8169e-03_rb,4.1910e-03_rb/)
      kbo(:, 2,17, 2) = (/ &
     &6.0993e-04_rb,2.6019e-03_rb,3.8415e-03_rb,4.8112e-03_rb,4.9579e-03_rb/)
      kbo(:, 3,17, 2) = (/ &
     &9.2221e-04_rb,3.5184e-03_rb,4.9669e-03_rb,5.9811e-03_rb,5.6981e-03_rb/)
      kbo(:, 4,17, 2) = (/ &
     &1.4572e-03_rb,4.7461e-03_rb,6.3751e-03_rb,7.3581e-03_rb,6.3963e-03_rb/)
      kbo(:, 5,17, 2) = (/ &
     &2.3036e-03_rb,6.3581e-03_rb,8.1281e-03_rb,8.9737e-03_rb,7.0378e-03_rb/)
      kbo(:, 1,18, 2) = (/ &
     &3.0388e-04_rb,1.5707e-03_rb,2.4656e-03_rb,3.2158e-03_rb,3.5652e-03_rb/)
      kbo(:, 2,18, 2) = (/ &
     &4.2513e-04_rb,2.1109e-03_rb,3.1840e-03_rb,4.0302e-03_rb,4.1950e-03_rb/)
      kbo(:, 3,18, 2) = (/ &
     &6.4993e-04_rb,2.8372e-03_rb,4.0815e-03_rb,4.9772e-03_rb,4.7960e-03_rb/)
      kbo(:, 4,18, 2) = (/ &
     &1.0315e-03_rb,3.7953e-03_rb,5.1949e-03_rb,6.0778e-03_rb,5.3518e-03_rb/)
      kbo(:, 5,18, 2) = (/ &
     &1.6331e-03_rb,5.0371e-03_rb,6.5707e-03_rb,7.3465e-03_rb,5.8586e-03_rb/)
      kbo(:, 1,19, 2) = (/ &
     &2.0547e-04_rb,1.2763e-03_rb,2.0458e-03_rb,2.6959e-03_rb,3.0176e-03_rb/)
      kbo(:, 2,19, 2) = (/ &
     &2.9169e-04_rb,1.7044e-03_rb,2.6231e-03_rb,3.3550e-03_rb,3.5313e-03_rb/)
      kbo(:, 3,19, 2) = (/ &
     &4.4987e-04_rb,2.2721e-03_rb,3.3322e-03_rb,4.1113e-03_rb,4.0121e-03_rb/)
      kbo(:, 4,19, 2) = (/ &
     &7.1653e-04_rb,3.0090e-03_rb,4.2013e-03_rb,4.9750e-03_rb,4.4532e-03_rb/)
      kbo(:, 5,19, 2) = (/ &
     &1.1358e-03_rb,3.9520e-03_rb,5.2567e-03_rb,5.9542e-03_rb,4.8572e-03_rb/)
      kbo(:, 1,20, 2) = (/ &
     &1.5174e-04_rb,1.0642e-03_rb,1.7200e-03_rb,2.2761e-03_rb,2.5514e-03_rb/)
      kbo(:, 2,20, 2) = (/ &
     &2.1846e-04_rb,1.4153e-03_rb,2.1942e-03_rb,2.8165e-03_rb,2.9650e-03_rb/)
      kbo(:, 3,20, 2) = (/ &
     &3.3982e-04_rb,1.8745e-03_rb,2.7705e-03_rb,3.4277e-03_rb,3.3482e-03_rb/)
      kbo(:, 4,20, 2) = (/ &
     &5.4313e-04_rb,2.4649e-03_rb,3.4676e-03_rb,4.1183e-03_rb,3.7008e-03_rb/)
      kbo(:, 5,20, 2) = (/ &
     &8.6061e-04_rb,3.2127e-03_rb,4.3062e-03_rb,4.9014e-03_rb,4.0252e-03_rb/)
      kbo(:, 1,21, 2) = (/ &
     &1.1592e-04_rb,8.9297e-04_rb,1.4493e-03_rb,1.9202e-03_rb,2.1482e-03_rb/)
      kbo(:, 2,21, 2) = (/ &
     &1.6918e-04_rb,1.1825e-03_rb,1.8389e-03_rb,2.3614e-03_rb,2.4794e-03_rb/)
      kbo(:, 3,21, 2) = (/ &
     &2.6522e-04_rb,1.5570e-03_rb,2.3088e-03_rb,2.8574e-03_rb,2.7868e-03_rb/)
      kbo(:, 4,21, 2) = (/ &
     &4.2461e-04_rb,2.0332e-03_rb,2.8744e-03_rb,3.4170e-03_rb,3.0702e-03_rb/)
      kbo(:, 5,21, 2) = (/ &
     &6.7177e-04_rb,2.6322e-03_rb,3.5494e-03_rb,4.0519e-03_rb,3.3309e-03_rb/)
      kbo(:, 1,22, 2) = (/ &
     &9.3055e-05_rb,7.6606e-04_rb,1.2409e-03_rb,1.6398e-03_rb,1.8197e-03_rb/)
      kbo(:, 2,22, 2) = (/ &
     &1.3841e-04_rb,1.0092e-03_rb,1.5660e-03_rb,2.0038e-03_rb,2.0843e-03_rb/)
      kbo(:, 3,22, 2) = (/ &
     &2.1899e-04_rb,1.3206e-03_rb,1.9566e-03_rb,2.4125e-03_rb,2.3311e-03_rb/)
      kbo(:, 4,22, 2) = (/ &
     &3.5071e-04_rb,1.7145e-03_rb,2.4240e-03_rb,2.8744e-03_rb,2.5586e-03_rb/)
      kbo(:, 5,22, 2) = (/ &
     &5.5216e-04_rb,2.2067e-03_rb,2.9792e-03_rb,3.3952e-03_rb,2.7680e-03_rb/)
      kbo(:, 1,23, 2) = (/ &
     &7.5659e-05_rb,6.5668e-04_rb,1.0605e-03_rb,1.3970e-03_rb,1.5365e-03_rb/)
      kbo(:, 2,23, 2) = (/ &
     &1.1449e-04_rb,8.6018e-04_rb,1.3327e-03_rb,1.6978e-03_rb,1.7494e-03_rb/)
      kbo(:, 3,23, 2) = (/ &
     &1.8237e-04_rb,1.1195e-03_rb,1.6581e-03_rb,2.0367e-03_rb,1.9476e-03_rb/)
      kbo(:, 4,23, 2) = (/ &
     &2.9157e-04_rb,1.4455e-03_rb,2.0441e-03_rb,2.4187e-03_rb,2.1305e-03_rb/)
      kbo(:, 5,23, 2) = (/ &
     &4.5638e-04_rb,1.8506e-03_rb,2.4994e-03_rb,2.8459e-03_rb,2.2986e-03_rb/)
      kbo(:, 1,24, 2) = (/ &
     &6.1686e-05_rb,5.6133e-04_rb,9.0530e-04_rb,1.1887e-03_rb,1.2959e-03_rb/)
      kbo(:, 2,24, 2) = (/ &
     &9.4868e-05_rb,7.3156e-04_rb,1.1327e-03_rb,1.4379e-03_rb,1.4672e-03_rb/)
      kbo(:, 3,24, 2) = (/ &
     &1.5170e-04_rb,9.4694e-04_rb,1.4023e-03_rb,1.7186e-03_rb,1.6265e-03_rb/)
      kbo(:, 4,24, 2) = (/ &
     &2.4184e-04_rb,1.2160e-03_rb,1.7202e-03_rb,2.0324e-03_rb,1.7732e-03_rb/)
      kbo(:, 5,24, 2) = (/ &
     &3.7584e-04_rb,1.5479e-03_rb,2.0924e-03_rb,2.3822e-03_rb,1.9076e-03_rb/)
      kbo(:, 1,25, 2) = (/ &
     &5.1420e-05_rb,4.8151e-04_rb,7.7480e-04_rb,1.0127e-03_rb,1.0925e-03_rb/)
      kbo(:, 2,25, 2) = (/ &
     &8.0205e-05_rb,6.2472e-04_rb,9.6537e-04_rb,1.2206e-03_rb,1.2305e-03_rb/)
      kbo(:, 3,25, 2) = (/ &
     &1.2852e-04_rb,8.0481e-04_rb,1.1895e-03_rb,1.4535e-03_rb,1.3583e-03_rb/)
      kbo(:, 4,25, 2) = (/ &
     &2.0384e-04_rb,1.0284e-03_rb,1.4523e-03_rb,1.7128e-03_rb,1.4762e-03_rb/)
      kbo(:, 5,25, 2) = (/ &
     &3.1446e-04_rb,1.3022e-03_rb,1.7587e-03_rb,1.9996e-03_rb,1.5839e-03_rb/)
      kbo(:, 1,26, 2) = (/ &
     &4.4173e-05_rb,4.1605e-04_rb,6.6653e-04_rb,8.6651e-04_rb,9.2186e-04_rb/)
      kbo(:, 2,26, 2) = (/ &
     &6.9706e-05_rb,5.3779e-04_rb,8.2701e-04_rb,1.0408e-03_rb,1.0329e-03_rb/)
      kbo(:, 3,26, 2) = (/ &
     &1.1164e-04_rb,6.8982e-04_rb,1.0147e-03_rb,1.2348e-03_rb,1.1353e-03_rb/)
      kbo(:, 4,26, 2) = (/ &
     &1.7590e-04_rb,8.7786e-04_rb,1.2337e-03_rb,1.4499e-03_rb,1.2294e-03_rb/)
      kbo(:, 5,26, 2) = (/ &
     &2.6942e-04_rb,1.1054e-03_rb,1.4885e-03_rb,1.6863e-03_rb,1.3161e-03_rb/)
      kbo(:, 1,27, 2) = (/ &
     &3.9675e-05_rb,3.6275e-04_rb,5.7658e-04_rb,7.4423e-04_rb,7.7727e-04_rb/)
      kbo(:, 2,27, 2) = (/ &
     &6.3079e-05_rb,4.6769e-04_rb,7.1304e-04_rb,8.9128e-04_rb,8.6622e-04_rb/)
      kbo(:, 3,27, 2) = (/ &
     &1.0080e-04_rb,5.9857e-04_rb,8.7226e-04_rb,1.0546e-03_rb,9.4873e-04_rb/)
      kbo(:, 4,27, 2) = (/ &
     &1.5784e-04_rb,7.5914e-04_rb,1.0582e-03_rb,1.2346e-03_rb,1.0243e-03_rb/)
      kbo(:, 5,27, 2) = (/ &
     &2.3988e-04_rb,9.5199e-04_rb,1.2727e-03_rb,1.4326e-03_rb,1.0935e-03_rb/)
      kbo(:, 1,28, 2) = (/ &
     &3.6787e-05_rb,3.1827e-04_rb,5.0054e-04_rb,6.4070e-04_rb,6.5436e-04_rb/)
      kbo(:, 2,28, 2) = (/ &
     &5.8744e-05_rb,4.0999e-04_rb,6.1769e-04_rb,7.6553e-04_rb,7.2608e-04_rb/)
      kbo(:, 3,28, 2) = (/ &
     &9.3496e-05_rb,5.2404e-04_rb,7.5434e-04_rb,9.0364e-04_rb,7.9234e-04_rb/)
      kbo(:, 4,28, 2) = (/ &
     &1.4539e-04_rb,6.6291e-04_rb,9.1412e-04_rb,1.0560e-03_rb,8.5294e-04_rb/)
      kbo(:, 5,28, 2) = (/ &
     &2.1931e-04_rb,8.2828e-04_rb,1.0967e-03_rb,1.2236e-03_rb,9.0840e-04_rb/)
      kbo(:, 1,29, 2) = (/ &
     &3.6448e-05_rb,2.8470e-04_rb,4.4017e-04_rb,5.5630e-04_rb,5.5097e-04_rb/)
      kbo(:, 2,29, 2) = (/ &
     &5.8278e-05_rb,3.6727e-04_rb,5.4285e-04_rb,6.6380e-04_rb,6.0864e-04_rb/)
      kbo(:, 3,29, 2) = (/ &
     &9.2358e-05_rb,4.6962e-04_rb,6.6368e-04_rb,7.8295e-04_rb,6.6201e-04_rb/)
      kbo(:, 4,29, 2) = (/ &
     &1.4253e-04_rb,5.9354e-04_rb,8.0405e-04_rb,9.1518e-04_rb,7.1060e-04_rb/)
      kbo(:, 5,29, 2) = (/ &
     &2.1329e-04_rb,7.4124e-04_rb,9.6341e-04_rb,1.0599e-03_rb,7.5499e-04_rb/)
      kbo(:, 1,30, 2) = (/ &
     &3.7091e-05_rb,2.5711e-04_rb,3.8922e-04_rb,4.8469e-04_rb,4.6359e-04_rb/)
      kbo(:, 2,30, 2) = (/ &
     &5.9310e-05_rb,3.3239e-04_rb,4.8070e-04_rb,5.7820e-04_rb,5.1002e-04_rb/)
      kbo(:, 3,30, 2) = (/ &
     &9.3453e-05_rb,4.2556e-04_rb,5.8851e-04_rb,6.8238e-04_rb,5.5293e-04_rb/)
      kbo(:, 4,30, 2) = (/ &
     &1.4319e-04_rb,5.3792e-04_rb,7.1341e-04_rb,7.9807e-04_rb,5.9204e-04_rb/)
      kbo(:, 5,30, 2) = (/ &
     &2.1251e-04_rb,6.7254e-04_rb,8.5446e-04_rb,9.2424e-04_rb,6.2785e-04_rb/)
      kbo(:, 1,31, 2) = (/ &
     &3.9864e-05_rb,2.3712e-04_rb,3.4911e-04_rb,4.2654e-04_rb,3.8998e-04_rb/)
      kbo(:, 2,31, 2) = (/ &
     &6.3592e-05_rb,3.0797e-04_rb,4.3283e-04_rb,5.0926e-04_rb,4.2741e-04_rb/)
      kbo(:, 3,31, 2) = (/ &
     &9.9603e-05_rb,3.9506e-04_rb,5.3137e-04_rb,6.0258e-04_rb,4.6195e-04_rb/)
      kbo(:, 4,31, 2) = (/ &
     &1.5142e-04_rb,5.0071e-04_rb,6.4518e-04_rb,7.0554e-04_rb,4.9395e-04_rb/)
      kbo(:, 5,31, 2) = (/ &
     &2.2297e-04_rb,6.2830e-04_rb,7.7442e-04_rb,8.1724e-04_rb,5.2260e-04_rb/)
      kbo(:, 1,32, 2) = (/ &
     &4.3430e-05_rb,2.2111e-04_rb,3.1573e-04_rb,3.7714e-04_rb,3.2795e-04_rb/)
      kbo(:, 2,32, 2) = (/ &
     &6.9083e-05_rb,2.8816e-04_rb,3.9276e-04_rb,4.5160e-04_rb,3.5813e-04_rb/)
      kbo(:, 3,32, 2) = (/ &
     &1.0740e-04_rb,3.7075e-04_rb,4.8372e-04_rb,5.3513e-04_rb,3.8620e-04_rb/)
      kbo(:, 4,32, 2) = (/ &
     &1.6196e-04_rb,4.7193e-04_rb,5.8843e-04_rb,6.2745e-04_rb,4.1195e-04_rb/)
      kbo(:, 5,32, 2) = (/ &
     &2.3647e-04_rb,5.9479e-04_rb,7.0866e-04_rb,7.2794e-04_rb,4.3506e-04_rb/)
      kbo(:, 1,33, 2) = (/ &
     &4.8220e-05_rb,2.0906e-04_rb,2.8832e-04_rb,3.3596e-04_rb,2.7570e-04_rb/)
      kbo(:, 2,33, 2) = (/ &
     &7.6368e-05_rb,2.7329e-04_rb,3.6023e-04_rb,4.0330e-04_rb,3.0019e-04_rb/)
      kbo(:, 3,33, 2) = (/ &
     &1.1780e-04_rb,3.5332e-04_rb,4.4506e-04_rb,4.7900e-04_rb,3.2295e-04_rb/)
      kbo(:, 4,33, 2) = (/ &
     &1.7613e-04_rb,4.5198e-04_rb,5.4349e-04_rb,5.6284e-04_rb,3.4357e-04_rb/)
      kbo(:, 5,33, 2) = (/ &
     &2.5515e-04_rb,5.7259e-04_rb,6.5773e-04_rb,6.5473e-04_rb,3.6197e-04_rb/)
      kbo(:, 1,34, 2) = (/ &
     &5.1326e-05_rb,1.9484e-04_rb,2.6083e-04_rb,2.9705e-04_rb,2.3085e-04_rb/)
      kbo(:, 2,34, 2) = (/ &
     &8.0913e-05_rb,2.5559e-04_rb,3.2688e-04_rb,3.5738e-04_rb,2.5079e-04_rb/)
      kbo(:, 3,34, 2) = (/ &
     &1.2390e-04_rb,3.3190e-04_rb,4.0497e-04_rb,4.2508e-04_rb,2.6902e-04_rb/)
      kbo(:, 4,34, 2) = (/ &
     &1.8386e-04_rb,4.2648e-04_rb,4.9662e-04_rb,5.0061e-04_rb,2.8571e-04_rb/)
      kbo(:, 5,34, 2) = (/ &
     &2.6467e-04_rb,5.4249e-04_rb,6.0361e-04_rb,5.8398e-04_rb,3.0059e-04_rb/)
      kbo(:, 1,35, 2) = (/ &
     &5.2281e-05_rb,1.7754e-04_rb,2.3171e-04_rb,2.5859e-04_rb,1.9168e-04_rb/)
      kbo(:, 2,35, 2) = (/ &
     &8.2157e-05_rb,2.3435e-04_rb,2.9174e-04_rb,3.1208e-04_rb,2.0781e-04_rb/)
      kbo(:, 3,35, 2) = (/ &
     &1.2528e-04_rb,3.0602e-04_rb,3.6300e-04_rb,3.7259e-04_rb,2.2270e-04_rb/)
      kbo(:, 4,35, 2) = (/ &
     &1.8516e-04_rb,3.9534e-04_rb,4.4759e-04_rb,4.4038e-04_rb,2.3630e-04_rb/)
      kbo(:, 5,35, 2) = (/ &
     &2.6552e-04_rb,5.0554e-04_rb,5.4703e-04_rb,5.1605e-04_rb,2.4840e-04_rb/)
      kbo(:, 1,36, 2) = (/ &
     &5.0331e-05_rb,1.5710e-04_rb,2.0093e-04_rb,2.2058e-04_rb,1.5759e-04_rb/)
      kbo(:, 2,36, 2) = (/ &
     &7.9043e-05_rb,2.0874e-04_rb,2.5435e-04_rb,2.6739e-04_rb,1.7077e-04_rb/)
      kbo(:, 3,36, 2) = (/ &
     &1.2045e-04_rb,2.7440e-04_rb,3.1834e-04_rb,3.2085e-04_rb,1.8310e-04_rb/)
      kbo(:, 4,36, 2) = (/ &
     &1.7786e-04_rb,3.5687e-04_rb,3.9494e-04_rb,3.8113e-04_rb,1.9431e-04_rb/)
      kbo(:, 5,36, 2) = (/ &
     &2.5490e-04_rb,4.5880e-04_rb,4.8570e-04_rb,4.4889e-04_rb,2.0416e-04_rb/)
      kbo(:, 1,37, 2) = (/ &
     &4.4029e-05_rb,1.3168e-04_rb,1.6668e-04_rb,1.8164e-04_rb,1.2776e-04_rb/)
      kbo(:, 2,37, 2) = (/ &
     &6.9369e-05_rb,1.7622e-04_rb,2.1241e-04_rb,2.2140e-04_rb,1.3874e-04_rb/)
      kbo(:, 3,37, 2) = (/ &
     &1.0612e-04_rb,2.3332e-04_rb,2.6770e-04_rb,2.6723e-04_rb,1.4898e-04_rb/)
      kbo(:, 4,37, 2) = (/ &
     &1.5731e-04_rb,3.0549e-04_rb,3.3418e-04_rb,3.1918e-04_rb,1.5836e-04_rb/)
      kbo(:, 5,37, 2) = (/ &
     &2.2631e-04_rb,3.9537e-04_rb,4.1350e-04_rb,3.7792e-04_rb,1.6663e-04_rb/)
      kbo(:, 1,38, 2) = (/ &
     &3.8922e-05_rb,1.1094e-04_rb,1.3878e-04_rb,1.4984e-04_rb,1.0348e-04_rb/)
      kbo(:, 2,38, 2) = (/ &
     &6.1507e-05_rb,1.4964e-04_rb,1.7814e-04_rb,1.8378e-04_rb,1.1264e-04_rb/)
      kbo(:, 3,38, 2) = (/ &
     &9.4510e-05_rb,1.9970e-04_rb,2.2619e-04_rb,2.2325e-04_rb,1.2117e-04_rb/)
      kbo(:, 4,38, 2) = (/ &
     &1.4072e-04_rb,2.6356e-04_rb,2.8447e-04_rb,2.6849e-04_rb,1.2897e-04_rb/)
      kbo(:, 5,38, 2) = (/ &
     &2.0319e-04_rb,3.4350e-04_rb,3.5441e-04_rb,3.1981e-04_rb,1.3588e-04_rb/)
      kbo(:, 1,39, 2) = (/ &
     &3.5197e-05_rb,9.4645e-05_rb,1.1658e-04_rb,1.2439e-04_rb,8.3852e-05_rb/)
      kbo(:, 2,39, 2) = (/ &
     &5.5773e-05_rb,1.2882e-04_rb,1.5091e-04_rb,1.5369e-04_rb,9.1477e-05_rb/)
      kbo(:, 3,39, 2) = (/ &
     &8.6079e-05_rb,1.7350e-04_rb,1.9331e-04_rb,1.8814e-04_rb,9.8545e-05_rb/)
      kbo(:, 4,39, 2) = (/ &
     &1.2878e-04_rb,2.3093e-04_rb,2.4509e-04_rb,2.2784e-04_rb,1.0505e-04_rb/)
      kbo(:, 5,39, 2) = (/ &
     &1.8672e-04_rb,3.0343e-04_rb,3.0781e-04_rb,2.7347e-04_rb,1.1084e-04_rb/)
      kbo(:, 1,40, 2) = (/ &
     &2.8482e-05_rb,7.5721e-05_rb,9.3100e-05_rb,9.9288e-05_rb,6.7184e-05_rb/)
      kbo(:, 2,40, 2) = (/ &
     &4.5322e-05_rb,1.0379e-04_rb,1.2136e-04_rb,1.2342e-04_rb,7.3552e-05_rb/)
      kbo(:, 3,40, 2) = (/ &
     &7.0466e-05_rb,1.4083e-04_rb,1.5650e-04_rb,1.5207e-04_rb,7.9463e-05_rb/)
      kbo(:, 4,40, 2) = (/ &
     &1.0624e-04_rb,1.8883e-04_rb,1.9975e-04_rb,1.8533e-04_rb,8.4957e-05_rb/)
      kbo(:, 5,40, 2) = (/ &
     &1.5520e-04_rb,2.4983e-04_rb,2.5250e-04_rb,2.2369e-04_rb,8.9879e-05_rb/)
      kbo(:, 1,41, 2) = (/ &
     &2.2852e-05_rb,6.0191e-05_rb,7.3911e-05_rb,7.8847e-05_rb,5.3707e-05_rb/)
      kbo(:, 2,41, 2) = (/ &
     &3.6477e-05_rb,8.3074e-05_rb,9.7030e-05_rb,9.8640e-05_rb,5.9019e-05_rb/)
      kbo(:, 3,41, 2) = (/ &
     &5.7117e-05_rb,1.1353e-04_rb,1.2598e-04_rb,1.2230e-04_rb,6.3954e-05_rb/)
      kbo(:, 4,41, 2) = (/ &
     &8.6797e-05_rb,1.5338e-04_rb,1.6193e-04_rb,1.5005e-04_rb,6.8580e-05_rb/)
      kbo(:, 5,41, 2) = (/ &
     &1.2784e-04_rb,2.0441e-04_rb,2.0604e-04_rb,1.8219e-04_rb,7.2757e-05_rb/)
      kbo(:, 1,42, 2) = (/ &
     &1.8532e-05_rb,4.8021e-05_rb,5.8809e-05_rb,6.2677e-05_rb,4.2889e-05_rb/)
      kbo(:, 2,42, 2) = (/ &
     &2.9639e-05_rb,6.6772e-05_rb,7.7781e-05_rb,7.8945e-05_rb,4.7319e-05_rb/)
      kbo(:, 3,42, 2) = (/ &
     &4.6699e-05_rb,9.1970e-05_rb,1.0173e-04_rb,9.8516e-05_rb,5.1447e-05_rb/)
      kbo(:, 4,42, 2) = (/ &
     &7.1536e-05_rb,1.2527e-04_rb,1.3174e-04_rb,1.2174e-04_rb,5.5341e-05_rb/)
      kbo(:, 5,42, 2) = (/ &
     &1.0622e-04_rb,1.6829e-04_rb,1.6888e-04_rb,1.4882e-04_rb,5.8857e-05_rb/)
      kbo(:, 1,43, 2) = (/ &
     &1.4784e-05_rb,3.7715e-05_rb,4.6105e-05_rb,4.9162e-05_rb,3.3989e-05_rb/)
      kbo(:, 2,43, 2) = (/ &
     &2.3635e-05_rb,5.2843e-05_rb,6.1488e-05_rb,6.2402e-05_rb,3.7696e-05_rb/)
      kbo(:, 3,43, 2) = (/ &
     &3.7482e-05_rb,7.3434e-05_rb,8.1121e-05_rb,7.8491e-05_rb,4.1167e-05_rb/)
      kbo(:, 4,43, 2) = (/ &
     &5.7957e-05_rb,1.0094e-04_rb,1.0596e-04_rb,9.7736e-05_rb,4.4450e-05_rb/)
      kbo(:, 5,43, 2) = (/ &
     &8.6918e-05_rb,1.3688e-04_rb,1.3695e-04_rb,1.2041e-04_rb,4.7444e-05_rb/)
      kbo(:, 1,44, 2) = (/ &
     &1.1851e-05_rb,2.9526e-05_rb,3.5985e-05_rb,3.8353e-05_rb,2.6802e-05_rb/)
      kbo(:, 2,44, 2) = (/ &
     &1.8865e-05_rb,4.1676e-05_rb,4.8402e-05_rb,4.9101e-05_rb,2.9917e-05_rb/)
      kbo(:, 3,44, 2) = (/ &
     &3.0080e-05_rb,5.8471e-05_rb,6.4462e-05_rb,6.2284e-05_rb,3.2834e-05_rb/)
      kbo(:, 4,44, 2) = (/ &
     &4.6945e-05_rb,8.1180e-05_rb,8.5021e-05_rb,7.8278e-05_rb,3.5598e-05_rb/)
      kbo(:, 5,44, 2) = (/ &
     &7.1168e-05_rb,1.1121e-04_rb,1.1093e-04_rb,9.7235e-05_rb,3.8149e-05_rb/)
      kbo(:, 1,45, 2) = (/ &
     &9.7426e-06_rb,2.3333e-05_rb,2.8230e-05_rb,2.9988e-05_rb,2.1086e-05_rb/)
      kbo(:, 2,45, 2) = (/ &
     &1.5361e-05_rb,3.3173e-05_rb,3.8317e-05_rb,3.8748e-05_rb,2.3700e-05_rb/)
      kbo(:, 3,45, 2) = (/ &
     &2.4573e-05_rb,4.7011e-05_rb,5.1561e-05_rb,4.9622e-05_rb,2.6158e-05_rb/)
      kbo(:, 4,45, 2) = (/ &
     &3.8674e-05_rb,6.5958e-05_rb,6.8724e-05_rb,6.2989e-05_rb,2.8488e-05_rb/)
      kbo(:, 5,45, 2) = (/ &
     &5.9269e-05_rb,9.1392e-05_rb,9.0638e-05_rb,7.9041e-05_rb,3.0648e-05_rb/)
      kbo(:, 1,46, 2) = (/ &
     &8.0813e-06_rb,1.8394e-05_rb,2.2048e-05_rb,2.3302e-05_rb,1.6499e-05_rb/)
      kbo(:, 2,46, 2) = (/ &
     &1.2533e-05_rb,2.6309e-05_rb,3.0179e-05_rb,3.0408e-05_rb,1.8696e-05_rb/)
      kbo(:, 3,46, 2) = (/ &
     &2.0044e-05_rb,3.7660e-05_rb,4.1056e-05_rb,3.9350e-05_rb,2.0768e-05_rb/)
      kbo(:, 4,46, 2) = (/ &
     &3.1770e-05_rb,5.3410e-05_rb,5.5319e-05_rb,5.0478e-05_rb,2.2728e-05_rb/)
      kbo(:, 5,46, 2) = (/ &
     &4.9215e-05_rb,7.4898e-05_rb,7.3800e-05_rb,6.4019e-05_rb,2.4552e-05_rb/)
      kbo(:, 1,47, 2) = (/ &
     &6.6192e-06_rb,1.4238e-05_rb,1.6907e-05_rb,1.7806e-05_rb,1.2778e-05_rb/)
      kbo(:, 2,47, 2) = (/ &
     &9.9788e-06_rb,2.0419e-05_rb,2.3301e-05_rb,2.3456e-05_rb,1.4630e-05_rb/)
      kbo(:, 3,47, 2) = (/ &
     &1.5864e-05_rb,2.9439e-05_rb,3.2011e-05_rb,3.0659e-05_rb,1.6375e-05_rb/)
      kbo(:, 4,47, 2) = (/ &
     &2.5298e-05_rb,4.2209e-05_rb,4.3617e-05_rb,3.9755e-05_rb,1.8027e-05_rb/)
      kbo(:, 5,47, 2) = (/ &
     &3.9602e-05_rb,5.9887e-05_rb,5.8840e-05_rb,5.0962e-05_rb,1.9570e-05_rb/)
      kbo(:, 1,48, 2) = (/ &
     &5.6786e-06_rb,1.1254e-05_rb,1.3112e-05_rb,1.3664e-05_rb,9.8559e-06_rb/)
      kbo(:, 2,48, 2) = (/ &
     &8.2415e-06_rb,1.6109e-05_rb,1.8176e-05_rb,1.8172e-05_rb,1.1406e-05_rb/)
      kbo(:, 3,48, 2) = (/ &
     &1.2920e-05_rb,2.3359e-05_rb,2.5209e-05_rb,2.4020e-05_rb,1.2882e-05_rb/)
      kbo(:, 4,48, 2) = (/ &
     &2.0668e-05_rb,3.3871e-05_rb,3.4762e-05_rb,3.1514e-05_rb,1.4268e-05_rb/)
      kbo(:, 5,48, 2) = (/ &
     &3.2629e-05_rb,4.8666e-05_rb,4.7495e-05_rb,4.0906e-05_rb,1.5578e-05_rb/)
      kbo(:, 1,49, 2) = (/ &
     &5.1748e-06_rb,9.2021e-06_rb,1.0379e-05_rb,1.0585e-05_rb,7.5665e-06_rb/)
      kbo(:, 2,49, 2) = (/ &
     &7.1867e-06_rb,1.3062e-05_rb,1.4427e-05_rb,1.4215e-05_rb,8.8627e-06_rb/)
      kbo(:, 3,49, 2) = (/ &
     &1.1001e-05_rb,1.9012e-05_rb,2.0206e-05_rb,1.9017e-05_rb,1.0112e-05_rb/)
      kbo(:, 4,49, 2) = (/ &
     &1.7557e-05_rb,2.7865e-05_rb,2.8224e-05_rb,2.5298e-05_rb,1.1279e-05_rb/)
      kbo(:, 5,49, 2) = (/ &
     &2.7899e-05_rb,4.0582e-05_rb,3.9130e-05_rb,3.3306e-05_rb,1.2389e-05_rb/)
      kbo(:, 1,50, 2) = (/ &
     &4.6020e-06_rb,7.4638e-06_rb,8.1645e-06_rb,8.1559e-06_rb,5.7980e-06_rb/)
      kbo(:, 2,50, 2) = (/ &
     &6.1191e-06_rb,1.0436e-05_rb,1.1320e-05_rb,1.1020e-05_rb,6.8814e-06_rb/)
      kbo(:, 3,50, 2) = (/ &
     &9.0820e-06_rb,1.5149e-05_rb,1.5926e-05_rb,1.4870e-05_rb,7.9274e-06_rb/)
      kbo(:, 4,50, 2) = (/ &
     &1.4367e-05_rb,2.2331e-05_rb,2.2448e-05_rb,1.9990e-05_rb,8.9091e-06_rb/)
      kbo(:, 5,50, 2) = (/ &
     &2.2930e-05_rb,3.2870e-05_rb,3.1485e-05_rb,2.6634e-05_rb,9.8428e-06_rb/)
      kbo(:, 1,51, 2) = (/ &
     &4.0738e-06_rb,6.0767e-06_rb,6.4322e-06_rb,6.2775e-06_rb,4.4265e-06_rb/)
      kbo(:, 2,51, 2) = (/ &
     &5.2028e-06_rb,8.3182e-06_rb,8.8519e-06_rb,8.5040e-06_rb,5.3268e-06_rb/)
      kbo(:, 3,51, 2) = (/ &
     &7.4386e-06_rb,1.1969e-05_rb,1.2454e-05_rb,1.1549e-05_rb,6.2015e-06_rb/)
      kbo(:, 4,51, 2) = (/ &
     &1.1580e-05_rb,1.7676e-05_rb,1.7666e-05_rb,1.5659e-05_rb,7.0249e-06_rb/)
      kbo(:, 5,51, 2) = (/ &
     &1.8500e-05_rb,2.6231e-05_rb,2.5018e-05_rb,2.1082e-05_rb,7.8088e-06_rb/)
      kbo(:, 1,52, 2) = (/ &
     &3.7602e-06_rb,5.1251e-06_rb,5.1954e-06_rb,4.8922e-06_rb,3.3637e-06_rb/)
      kbo(:, 2,52, 2) = (/ &
     &4.6219e-06_rb,6.8269e-06_rb,7.0581e-06_rb,6.6280e-06_rb,4.1074e-06_rb/)
      kbo(:, 3,52, 2) = (/ &
     &6.3416e-06_rb,9.6823e-06_rb,9.8934e-06_rb,9.0485e-06_rb,4.8363e-06_rb/)
      kbo(:, 4,52, 2) = (/ &
     &9.6360e-06_rb,1.4271e-05_rb,1.4108e-05_rb,1.2374e-05_rb,5.5264e-06_rb/)
      kbo(:, 5,52, 2) = (/ &
     &1.5337e-05_rb,2.1320e-05_rb,2.0165e-05_rb,1.6846e-05_rb,6.1865e-06_rb/)
      kbo(:, 1,53, 2) = (/ &
     &3.6702e-06_rb,4.5390e-06_rb,4.3659e-06_rb,3.9002e-06_rb,2.5423e-06_rb/)
      kbo(:, 2,53, 2) = (/ &
     &4.3437e-06_rb,5.8533e-06_rb,5.8072e-06_rb,5.2588e-06_rb,3.1537e-06_rb/)
      kbo(:, 3,53, 2) = (/ &
     &5.7127e-06_rb,8.1219e-06_rb,8.0674e-06_rb,7.2006e-06_rb,3.7613e-06_rb/)
      kbo(:, 4,53, 2) = (/ &
     &8.4068e-06_rb,1.1886e-05_rb,1.1528e-05_rb,9.9298e-06_rb,4.3399e-06_rb/)
      kbo(:, 5,53, 2) = (/ &
     &1.3242e-05_rb,1.7846e-05_rb,1.6632e-05_rb,1.3680e-05_rb,4.8970e-06_rb/)
      kbo(:, 1,54, 2) = (/ &
     &3.1403e-06_rb,3.6924e-06_rb,3.4557e-06_rb,2.9996e-06_rb,1.9177e-06_rb/)
      kbo(:, 2,54, 2) = (/ &
     &3.5986e-06_rb,4.6301e-06_rb,4.4986e-06_rb,4.0115e-06_rb,2.4155e-06_rb/)
      kbo(:, 3,54, 2) = (/ &
     &4.5617e-06_rb,6.2669e-06_rb,6.1686e-06_rb,5.4741e-06_rb,2.9175e-06_rb/)
      kbo(:, 4,54, 2) = (/ &
     &6.4783e-06_rb,9.0433e-06_rb,8.7626e-06_rb,7.5562e-06_rb,3.4007e-06_rb/)
      kbo(:, 5,54, 2) = (/ &
     &1.0042e-05_rb,1.3533e-05_rb,1.2655e-05_rb,1.0455e-05_rb,3.8639e-06_rb/)
      kbo(:, 1,55, 2) = (/ &
     &2.4739e-06_rb,2.8343e-06_rb,2.6165e-06_rb,2.2420e-06_rb,1.4427e-06_rb/)
      kbo(:, 2,55, 2) = (/ &
     &2.7553e-06_rb,3.4672e-06_rb,3.3435e-06_rb,2.9758e-06_rb,1.8442e-06_rb/)
      kbo(:, 3,55, 2) = (/ &
     &3.3782e-06_rb,4.5817e-06_rb,4.5186e-06_rb,4.0371e-06_rb,2.2544e-06_rb/)
      kbo(:, 4,55, 2) = (/ &
     &4.6246e-06_rb,6.4927e-06_rb,6.3529e-06_rb,5.5548e-06_rb,2.6558e-06_rb/)
      kbo(:, 5,55, 2) = (/ &
     &7.0125e-06_rb,9.6319e-06_rb,9.1393e-06_rb,7.6813e-06_rb,3.0372e-06_rb/)
      kbo(:, 1,56, 2) = (/ &
     &1.9604e-06_rb,2.1929e-06_rb,1.9920e-06_rb,1.6797e-06_rb,1.0797e-06_rb/)
      kbo(:, 2,56, 2) = (/ &
     &2.1351e-06_rb,2.6170e-06_rb,2.4998e-06_rb,2.2137e-06_rb,1.4022e-06_rb/)
      kbo(:, 3,56, 2) = (/ &
     &2.5317e-06_rb,3.3754e-06_rb,3.3236e-06_rb,2.9840e-06_rb,1.7372e-06_rb/)
      kbo(:, 4,56, 2) = (/ &
     &3.3387e-06_rb,4.6862e-06_rb,4.6202e-06_rb,4.0882e-06_rb,2.0679e-06_rb/)
      kbo(:, 5,56, 2) = (/ &
     &4.9234e-06_rb,6.8724e-06_rb,6.6108e-06_rb,5.6459e-06_rb,2.3854e-06_rb/)
      kbo(:, 1,57, 2) = (/ &
     &1.5649e-06_rb,1.7047e-06_rb,1.5250e-06_rb,1.2613e-06_rb,8.0294e-07_rb/)
      kbo(:, 2,57, 2) = (/ &
     &1.6685e-06_rb,1.9908e-06_rb,1.8801e-06_rb,1.6502e-06_rb,1.0613e-06_rb/)
      kbo(:, 3,57, 2) = (/ &
     &1.9178e-06_rb,2.5066e-06_rb,2.4556e-06_rb,2.2101e-06_rb,1.3329e-06_rb/)
      kbo(:, 4,57, 2) = (/ &
     &2.4407e-06_rb,3.4025e-06_rb,3.3722e-06_rb,3.0127e-06_rb,1.6045e-06_rb/)
      kbo(:, 5,57, 2) = (/ &
     &3.4802e-06_rb,4.9175e-06_rb,4.7870e-06_rb,4.1514e-06_rb,1.8688e-06_rb/)
      kbo(:, 1,58, 2) = (/ &
     &3.3188e-07_rb,4.3313e-07_rb,4.9294e-07_rb,5.6291e-07_rb,5.9615e-07_rb/)
      kbo(:, 2,58, 2) = (/ &
     &3.4670e-07_rb,4.9696e-07_rb,5.9930e-07_rb,7.3314e-07_rb,8.0253e-07_rb/)
      kbo(:, 3,58, 2) = (/ &
     &3.8806e-07_rb,6.1237e-07_rb,7.7019e-07_rb,9.7654e-07_rb,1.0216e-06_rb/)
      kbo(:, 4,58, 2) = (/ &
     &4.7831e-07_rb,8.1333e-07_rb,1.0442e-06_rb,1.3240e-06_rb,1.2449e-06_rb/)
      kbo(:, 5,58, 2) = (/ &
     &6.5898e-07_rb,1.1558e-06_rb,1.4682e-06_rb,1.8176e-06_rb,1.4615e-06_rb/)
      kbo(:, 1,59, 2) = (/ &
     &3.1712e-07_rb,4.1538e-07_rb,4.5873e-07_rb,4.9478e-07_rb,4.6737e-07_rb/)
      kbo(:, 2,59, 2) = (/ &
     &3.2863e-07_rb,4.7977e-07_rb,5.6611e-07_rb,6.5498e-07_rb,6.3397e-07_rb/)
      kbo(:, 3,59, 2) = (/ &
     &3.6418e-07_rb,5.9318e-07_rb,7.3191e-07_rb,8.7648e-07_rb,8.1119e-07_rb/)
      kbo(:, 4,59, 2) = (/ &
     &4.4347e-07_rb,7.8664e-07_rb,9.8989e-07_rb,1.1827e-06_rb,9.9218e-07_rb/)
      kbo(:, 5,59, 2) = (/ &
     &6.0215e-07_rb,1.1090e-06_rb,1.3813e-06_rb,1.6064e-06_rb,1.1684e-06_rb/)
      kbo(:, 1,13, 3) = (/ &
     &9.1540e-03_rb,2.0983e-02_rb,2.7409e-02_rb,3.0450e-02_rb,2.3144e-02_rb/)
      kbo(:, 2,13, 3) = (/ &
     &1.5150e-02_rb,2.9836e-02_rb,3.7088e-02_rb,3.9562e-02_rb,2.6157e-02_rb/)
      kbo(:, 3,13, 3) = (/ &
     &2.6705e-02_rb,4.3495e-02_rb,5.0489e-02_rb,5.0868e-02_rb,2.9169e-02_rb/)
      kbo(:, 4,13, 3) = (/ &
     &4.6501e-02_rb,6.3525e-02_rb,6.8930e-02_rb,6.5213e-02_rb,3.2207e-02_rb/)
      kbo(:, 5,13, 3) = (/ &
     &7.7883e-02_rb,9.1957e-02_rb,9.3471e-02_rb,8.3394e-02_rb,3.5205e-02_rb/)
      kbo(:, 1,14, 3) = (/ &
     &4.9769e-03_rb,1.5542e-02_rb,2.1404e-02_rb,2.4888e-02_rb,2.0102e-02_rb/)
      kbo(:, 2,14, 3) = (/ &
     &8.3891e-03_rb,2.1811e-02_rb,2.8513e-02_rb,3.1898e-02_rb,2.2689e-02_rb/)
      kbo(:, 3,14, 3) = (/ &
     &1.4857e-02_rb,3.0975e-02_rb,3.8046e-02_rb,4.0288e-02_rb,2.5289e-02_rb/)
      kbo(:, 4,14, 3) = (/ &
     &2.5874e-02_rb,4.3884e-02_rb,5.0681e-02_rb,5.0529e-02_rb,2.7965e-02_rb/)
      kbo(:, 5,14, 3) = (/ &
     &4.3258e-02_rb,6.1604e-02_rb,6.6800e-02_rb,6.3228e-02_rb,3.0564e-02_rb/)
      kbo(:, 1,15, 3) = (/ &
     &2.7780e-03_rb,1.1828e-02_rb,1.7073e-02_rb,2.0490e-02_rb,1.7377e-02_rb/)
      kbo(:, 2,15, 3) = (/ &
     &4.7591e-03_rb,1.6367e-02_rb,2.2382e-02_rb,2.5900e-02_rb,1.9610e-02_rb/)
      kbo(:, 3,15, 3) = (/ &
     &8.4707e-03_rb,2.2715e-02_rb,2.9226e-02_rb,3.2287e-02_rb,2.1849e-02_rb/)
      kbo(:, 4,15, 3) = (/ &
     &1.4726e-02_rb,3.1294e-02_rb,3.8074e-02_rb,3.9870e-02_rb,2.4127e-02_rb/)
      kbo(:, 5,15, 3) = (/ &
     &2.4582e-02_rb,4.2700e-02_rb,4.9087e-02_rb,4.8962e-02_rb,2.6357e-02_rb/)
      kbo(:, 1,16, 3) = (/ &
     &1.8575e-03_rb,9.6428e-03_rb,1.4258e-02_rb,1.7308e-02_rb,1.4967e-02_rb/)
      kbo(:, 2,16, 3) = (/ &
     &3.2295e-03_rb,1.3222e-02_rb,1.8534e-02_rb,2.1716e-02_rb,1.6894e-02_rb/)
      kbo(:, 3,16, 3) = (/ &
     &5.7622e-03_rb,1.8134e-02_rb,2.3952e-02_rb,2.6895e-02_rb,1.8844e-02_rb/)
      kbo(:, 4,16, 3) = (/ &
     &1.0014e-02_rb,2.4628e-02_rb,3.0790e-02_rb,3.2975e-02_rb,2.0792e-02_rb/)
      kbo(:, 5,16, 3) = (/ &
     &1.6651e-02_rb,3.3072e-02_rb,3.9202e-02_rb,4.0100e-02_rb,2.2651e-02_rb/)
      kbo(:, 1,17, 3) = (/ &
     &1.2913e-03_rb,7.9469e-03_rb,1.1927e-02_rb,1.4612e-02_rb,1.2832e-02_rb/)
      kbo(:, 2,17, 3) = (/ &
     &2.2741e-03_rb,1.0816e-02_rb,1.5424e-02_rb,1.8211e-02_rb,1.4484e-02_rb/)
      kbo(:, 3,17, 3) = (/ &
     &4.0659e-03_rb,1.4661e-02_rb,1.9799e-02_rb,2.2441e-02_rb,1.6158e-02_rb/)
      kbo(:, 4,17, 3) = (/ &
     &7.0575e-03_rb,1.9668e-02_rb,2.5205e-02_rb,2.7372e-02_rb,1.7784e-02_rb/)
      kbo(:, 5,17, 3) = (/ &
     &1.1701e-02_rb,2.6124e-02_rb,3.1755e-02_rb,3.3025e-02_rb,1.9312e-02_rb/)
      kbo(:, 1,18, 3) = (/ &
     &9.2490e-04_rb,6.5924e-03_rb,1.0000e-02_rb,1.2327e-02_rb,1.0954e-02_rb/)
      kbo(:, 2,18, 3) = (/ &
     &1.6450e-03_rb,8.9024e-03_rb,1.2876e-02_rb,1.5279e-02_rb,1.2362e-02_rb/)
      kbo(:, 3,18, 3) = (/ &
     &2.9476e-03_rb,1.1938e-02_rb,1.6432e-02_rb,1.8727e-02_rb,1.3771e-02_rb/)
      kbo(:, 4,18, 3) = (/ &
     &5.1050e-03_rb,1.5852e-02_rb,2.0747e-02_rb,2.2702e-02_rb,1.5108e-02_rb/)
      kbo(:, 5,18, 3) = (/ &
     &8.4407e-03_rb,2.0877e-02_rb,2.5905e-02_rb,2.7220e-02_rb,1.6353e-02_rb/)
      kbo(:, 1,19, 3) = (/ &
     &6.5306e-04_rb,5.4276e-03_rb,8.3228e-03_rb,1.0321e-02_rb,9.3064e-03_rb/)
      kbo(:, 2,19, 3) = (/ &
     &1.1720e-03_rb,7.2659e-03_rb,1.0659e-02_rb,1.2719e-02_rb,1.0490e-02_rb/)
      kbo(:, 3,19, 3) = (/ &
     &2.1010e-03_rb,9.6452e-03_rb,1.3501e-02_rb,1.5485e-02_rb,1.1656e-02_rb/)
      kbo(:, 4,19, 3) = (/ &
     &3.6298e-03_rb,1.2682e-02_rb,1.6890e-02_rb,1.8630e-02_rb,1.2744e-02_rb/)
      kbo(:, 5,19, 3) = (/ &
     &5.9733e-03_rb,1.6533e-02_rb,2.0894e-02_rb,2.2160e-02_rb,1.3725e-02_rb/)
      kbo(:, 1,20, 3) = (/ &
     &5.0525e-04_rb,4.5786e-03_rb,7.0416e-03_rb,8.7290e-03_rb,7.8885e-03_rb/)
      kbo(:, 2,20, 3) = (/ &
     &9.1391e-04_rb,6.1018e-03_rb,8.9804e-03_rb,1.0717e-02_rb,8.8768e-03_rb/)
      kbo(:, 3,20, 3) = (/ &
     &1.6371e-03_rb,8.0525e-03_rb,1.1310e-02_rb,1.2976e-02_rb,9.8259e-03_rb/)
      kbo(:, 4,20, 3) = (/ &
     &2.8150e-03_rb,1.0522e-02_rb,1.4057e-02_rb,1.5517e-02_rb,1.0692e-02_rb/)
      kbo(:, 5,20, 3) = (/ &
     &4.6066e-03_rb,1.3646e-02_rb,1.7296e-02_rb,1.8359e-02_rb,1.1448e-02_rb/)
      kbo(:, 1,21, 3) = (/ &
     &4.0425e-04_rb,3.8829e-03_rb,5.9693e-03_rb,7.3863e-03_rb,6.6651e-03_rb/)
      kbo(:, 2,21, 3) = (/ &
     &7.3484e-04_rb,5.1547e-03_rb,7.5787e-03_rb,9.0292e-03_rb,7.4787e-03_rb/)
      kbo(:, 3,21, 3) = (/ &
     &1.3153e-03_rb,6.7705e-03_rb,9.4934e-03_rb,1.0877e-02_rb,8.2368e-03_rb/)
      kbo(:, 4,21, 3) = (/ &
     &2.2503e-03_rb,8.8142e-03_rb,1.1739e-02_rb,1.2939e-02_rb,8.9114e-03_rb/)
      kbo(:, 5,21, 3) = (/ &
     &3.6575e-03_rb,1.1380e-02_rb,1.4377e-02_rb,1.5231e-02_rb,9.5025e-03_rb/)
      kbo(:, 1,22, 3) = (/ &
     &3.4523e-04_rb,3.3635e-03_rb,5.1429e-03_rb,6.3294e-03_rb,5.6535e-03_rb/)
      kbo(:, 2,22, 3) = (/ &
     &6.2993e-04_rb,4.4492e-03_rb,6.4923e-03_rb,7.6957e-03_rb,6.3107e-03_rb/)
      kbo(:, 3,22, 3) = (/ &
     &1.1206e-03_rb,5.8192e-03_rb,8.0871e-03_rb,9.2166e-03_rb,6.9018e-03_rb/)
      kbo(:, 4,22, 3) = (/ &
     &1.8990e-03_rb,7.5458e-03_rb,9.9517e-03_rb,1.0900e-02_rb,7.4316e-03_rb/)
      kbo(:, 5,22, 3) = (/ &
     &3.0583e-03_rb,9.7051e-03_rb,1.2139e-02_rb,1.2766e-02_rb,7.8941e-03_rb/)
      kbo(:, 1,23, 3) = (/ &
     &2.9783e-04_rb,2.9108e-03_rb,4.4177e-03_rb,5.4056e-03_rb,4.7807e-03_rb/)
      kbo(:, 2,23, 3) = (/ &
     &5.4387e-04_rb,3.8320e-03_rb,5.5435e-03_rb,6.5345e-03_rb,5.2974e-03_rb/)
      kbo(:, 3,23, 3) = (/ &
     &9.5952e-04_rb,4.9950e-03_rb,6.8655e-03_rb,7.7757e-03_rb,5.7630e-03_rb/)
      kbo(:, 4,23, 3) = (/ &
     &1.6092e-03_rb,6.4496e-03_rb,8.4095e-03_rb,9.1440e-03_rb,6.1817e-03_rb/)
      kbo(:, 5,23, 3) = (/ &
     &2.5684e-03_rb,8.2494e-03_rb,1.0215e-02_rb,1.0668e-02_rb,6.5447e-03_rb/)
      kbo(:, 1,24, 3) = (/ &
     &2.5715e-04_rb,2.5095e-03_rb,3.7777e-03_rb,4.5997e-03_rb,4.0260e-03_rb/)
      kbo(:, 2,24, 3) = (/ &
     &4.6827e-04_rb,3.2888e-03_rb,4.7107e-03_rb,5.5207e-03_rb,4.4325e-03_rb/)
      kbo(:, 3,24, 3) = (/ &
     &8.1828e-04_rb,4.2677e-03_rb,5.8000e-03_rb,6.5285e-03_rb,4.8028e-03_rb/)
      kbo(:, 4,24, 3) = (/ &
     &1.3581e-03_rb,5.4776e-03_rb,7.0696e-03_rb,7.6441e-03_rb,5.1338e-03_rb/)
      kbo(:, 5,24, 3) = (/ &
     &2.1494e-03_rb,6.9550e-03_rb,8.5500e-03_rb,8.8910e-03_rb,5.4193e-03_rb/)
      kbo(:, 1,25, 3) = (/ &
     &2.2632e-04_rb,2.1696e-03_rb,3.2330e-03_rb,3.9113e-03_rb,3.3801e-03_rb/)
      kbo(:, 2,25, 3) = (/ &
     &4.0936e-04_rb,2.8320e-03_rb,4.0052e-03_rb,4.6615e-03_rb,3.7035e-03_rb/)
      kbo(:, 3,25, 3) = (/ &
     &7.0836e-04_rb,3.6547e-03_rb,4.9096e-03_rb,5.4885e-03_rb,3.9989e-03_rb/)
      kbo(:, 4,25, 3) = (/ &
     &1.1642e-03_rb,4.6613e-03_rb,5.9642e-03_rb,6.4094e-03_rb,4.2600e-03_rb/)
      kbo(:, 5,25, 3) = (/ &
     &1.8243e-03_rb,5.8764e-03_rb,7.1830e-03_rb,7.4369e-03_rb,4.4844e-03_rb/)
      kbo(:, 1,26, 3) = (/ &
     &2.0471e-04_rb,1.8893e-03_rb,2.7768e-03_rb,3.3315e-03_rb,2.8361e-03_rb/)
      kbo(:, 2,26, 3) = (/ &
     &3.6709e-04_rb,2.4543e-03_rb,3.4258e-03_rb,3.9516e-03_rb,3.0939e-03_rb/)
      kbo(:, 3,26, 3) = (/ &
     &6.2789e-04_rb,3.1504e-03_rb,4.1849e-03_rb,4.6387e-03_rb,3.3286e-03_rb/)
      kbo(:, 4,26, 3) = (/ &
     &1.0213e-03_rb,3.9950e-03_rb,5.0687e-03_rb,5.4088e-03_rb,3.5346e-03_rb/)
      kbo(:, 5,26, 3) = (/ &
     &1.5815e-03_rb,5.0074e-03_rb,6.0787e-03_rb,6.2593e-03_rb,3.7107e-03_rb/)
      kbo(:, 1,27, 3) = (/ &
     &1.9251e-04_rb,1.6617e-03_rb,2.4029e-03_rb,2.8514e-03_rb,2.3762e-03_rb/)
      kbo(:, 2,27, 3) = (/ &
     &3.4185e-04_rb,2.1525e-03_rb,2.9600e-03_rb,3.3730e-03_rb,2.5825e-03_rb/)
      kbo(:, 3,27, 3) = (/ &
     &5.7778e-04_rb,2.7512e-03_rb,3.6092e-03_rb,3.9563e-03_rb,2.7684e-03_rb/)
      kbo(:, 4,27, 3) = (/ &
     &9.2854e-04_rb,3.4753e-03_rb,4.3621e-03_rb,4.6091e-03_rb,2.9308e-03_rb/)
      kbo(:, 5,27, 3) = (/ &
     &1.4199e-03_rb,4.3436e-03_rb,5.2156e-03_rb,5.3258e-03_rb,3.0691e-03_rb/)
      kbo(:, 1,28, 3) = (/ &
     &1.8568e-04_rb,1.4731e-03_rb,2.0943e-03_rb,2.4502e-03_rb,1.9883e-03_rb/)
      kbo(:, 2,28, 3) = (/ &
     &3.2639e-04_rb,1.9038e-03_rb,2.5783e-03_rb,2.8978e-03_rb,2.1535e-03_rb/)
      kbo(:, 3,28, 3) = (/ &
     &5.4487e-04_rb,2.4257e-03_rb,3.1398e-03_rb,3.3991e-03_rb,2.3010e-03_rb/)
      kbo(:, 4,28, 3) = (/ &
     &8.6497e-04_rb,3.0580e-03_rb,3.7865e-03_rb,3.9564e-03_rb,2.4290e-03_rb/)
      kbo(:, 5,28, 3) = (/ &
     &1.3039e-03_rb,3.8186e-03_rb,4.5211e-03_rb,4.5655e-03_rb,2.5382e-03_rb/)
      kbo(:, 1,29, 3) = (/ &
     &1.9062e-04_rb,1.3364e-03_rb,1.8603e-03_rb,2.1354e-03_rb,1.6643e-03_rb/)
      kbo(:, 2,29, 3) = (/ &
     &3.3121e-04_rb,1.7267e-03_rb,2.2927e-03_rb,2.5300e-03_rb,1.7960e-03_rb/)
      kbo(:, 3,29, 3) = (/ &
     &5.4570e-04_rb,2.2011e-03_rb,2.7931e-03_rb,2.9723e-03_rb,1.9129e-03_rb/)
      kbo(:, 4,29, 3) = (/ &
     &8.5435e-04_rb,2.7794e-03_rb,3.3678e-03_rb,3.4607e-03_rb,2.0139e-03_rb/)
      kbo(:, 5,29, 3) = (/ &
     &1.2701e-03_rb,3.4715e-03_rb,4.0279e-03_rb,3.9937e-03_rb,2.1000e-03_rb/)
      kbo(:, 1,30, 3) = (/ &
     &2.0043e-04_rb,1.2258e-03_rb,1.6689e-03_rb,1.8766e-03_rb,1.3916e-03_rb/)
      kbo(:, 2,30, 3) = (/ &
     &3.4350e-04_rb,1.5850e-03_rb,2.0586e-03_rb,2.2288e-03_rb,1.4967e-03_rb/)
      kbo(:, 3,30, 3) = (/ &
     &5.5868e-04_rb,2.0253e-03_rb,2.5096e-03_rb,2.6211e-03_rb,1.5893e-03_rb/)
      kbo(:, 4,30, 3) = (/ &
     &8.6173e-04_rb,2.5636e-03_rb,3.0320e-03_rb,3.0532e-03_rb,1.6692e-03_rb/)
      kbo(:, 5,30, 3) = (/ &
     &1.2648e-03_rb,3.2006e-03_rb,3.6329e-03_rb,3.5289e-03_rb,1.7373e-03_rb/)
      kbo(:, 1,31, 3) = (/ &
     &2.2172e-04_rb,1.1517e-03_rb,1.5273e-03_rb,1.6764e-03_rb,1.1637e-03_rb/)
      kbo(:, 2,31, 3) = (/ &
     &3.7478e-04_rb,1.4949e-03_rb,1.8875e-03_rb,1.9960e-03_rb,1.2474e-03_rb/)
      kbo(:, 3,31, 3) = (/ &
     &6.0019e-04_rb,1.9228e-03_rb,2.3094e-03_rb,2.3523e-03_rb,1.3210e-03_rb/)
      kbo(:, 4,31, 3) = (/ &
     &9.1186e-04_rb,2.4398e-03_rb,2.8016e-03_rb,2.7471e-03_rb,1.3843e-03_rb/)
      kbo(:, 5,31, 3) = (/ &
     &1.3231e-03_rb,3.0462e-03_rb,3.3613e-03_rb,3.1855e-03_rb,1.4373e-03_rb/)
      kbo(:, 1,32, 3) = (/ &
     &2.4768e-04_rb,1.0937e-03_rb,1.4089e-03_rb,1.5091e-03_rb,9.7256e-04_rb/)
      kbo(:, 2,32, 3) = (/ &
     &4.1248e-04_rb,1.4292e-03_rb,1.7477e-03_rb,1.8006e-03_rb,1.0394e-03_rb/)
      kbo(:, 3,32, 3) = (/ &
     &6.5047e-04_rb,1.8467e-03_rb,2.1484e-03_rb,2.1275e-03_rb,1.0979e-03_rb/)
      kbo(:, 4,32, 3) = (/ &
     &9.7457e-04_rb,2.3473e-03_rb,2.6140e-03_rb,2.4947e-03_rb,1.1482e-03_rb/)
      kbo(:, 5,32, 3) = (/ &
     &1.3990e-03_rb,2.9324e-03_rb,3.1399e-03_rb,2.8984e-03_rb,1.1898e-03_rb/)
      kbo(:, 1,33, 3) = (/ &
     &2.8078e-04_rb,1.0552e-03_rb,1.3156e-03_rb,1.3710e-03_rb,8.1280e-04_rb/)
      kbo(:, 2,33, 3) = (/ &
     &4.6027e-04_rb,1.3891e-03_rb,1.6405e-03_rb,1.6404e-03_rb,8.6605e-04_rb/)
      kbo(:, 3,33, 3) = (/ &
     &7.1527e-04_rb,1.8015e-03_rb,2.0264e-03_rb,1.9473e-03_rb,9.1257e-04_rb/)
      kbo(:, 4,33, 3) = (/ &
     &1.0579e-03_rb,2.2936e-03_rb,2.4711e-03_rb,2.2904e-03_rb,9.5249e-04_rb/)
      kbo(:, 5,33, 3) = (/ &
     &1.5037e-03_rb,2.8717e-03_rb,2.9734e-03_rb,2.6646e-03_rb,9.8557e-04_rb/)
      kbo(:, 1,34, 3) = (/ &
     &3.0359e-04_rb,1.0038e-03_rb,1.2140e-03_rb,1.2334e-03_rb,6.7695e-04_rb/)
      kbo(:, 2,34, 3) = (/ &
     &4.9090e-04_rb,1.3275e-03_rb,1.5216e-03_rb,1.4814e-03_rb,7.1957e-04_rb/)
      kbo(:, 3,34, 3) = (/ &
     &7.5358e-04_rb,1.7240e-03_rb,1.8846e-03_rb,1.7651e-03_rb,7.5712e-04_rb/)
      kbo(:, 4,34, 3) = (/ &
     &1.1033e-03_rb,2.1975e-03_rb,2.2999e-03_rb,2.0785e-03_rb,7.8887e-04_rb/)
      kbo(:, 5,34, 3) = (/ &
     &1.5542e-03_rb,2.7592e-03_rb,2.7749e-03_rb,2.4194e-03_rb,8.1541e-04_rb/)
      kbo(:, 1,35, 3) = (/ &
     &3.1210e-04_rb,9.3394e-04_rb,1.1014e-03_rb,1.0942e-03_rb,5.6016e-04_rb/)
      kbo(:, 2,35, 3) = (/ &
     &5.0058e-04_rb,1.2412e-03_rb,1.3879e-03_rb,1.3212e-03_rb,5.9529e-04_rb/)
      kbo(:, 3,35, 3) = (/ &
     &7.6299e-04_rb,1.6182e-03_rb,1.7248e-03_rb,1.5791e-03_rb,6.2556e-04_rb/)
      kbo(:, 4,35, 3) = (/ &
     &1.1110e-03_rb,2.0696e-03_rb,2.1107e-03_rb,1.8624e-03_rb,6.5155e-04_rb/)
      kbo(:, 5,35, 3) = (/ &
     &1.5579e-03_rb,2.6064e-03_rb,2.5562e-03_rb,2.1725e-03_rb,6.7276e-04_rb/)
      kbo(:, 1,36, 3) = (/ &
     &3.0156e-04_rb,8.4185e-04_rb,9.7484e-04_rb,9.5194e-04_rb,4.6045e-04_rb/)
      kbo(:, 2,36, 3) = (/ &
     &4.8313e-04_rb,1.1258e-03_rb,1.2357e-03_rb,1.1559e-03_rb,4.8953e-04_rb/)
      kbo(:, 3,36, 3) = (/ &
     &7.3505e-04_rb,1.4756e-03_rb,1.5422e-03_rb,1.3864e-03_rb,5.1450e-04_rb/)
      kbo(:, 4,36, 3) = (/ &
     &1.0692e-03_rb,1.8983e-03_rb,1.8970e-03_rb,1.6403e-03_rb,5.3594e-04_rb/)
      kbo(:, 5,36, 3) = (/ &
     &1.4983e-03_rb,2.4025e-03_rb,2.3079e-03_rb,1.9210e-03_rb,5.5373e-04_rb/)
      kbo(:, 1,37, 3) = (/ &
     &2.6336e-04_rb,7.1618e-04_rb,8.2306e-04_rb,7.9778e-04_rb,3.7474e-04_rb/)
      kbo(:, 2,37, 3) = (/ &
     &4.2453e-04_rb,9.6471e-04_rb,1.0505e-03_rb,9.7428e-04_rb,3.9893e-04_rb/)
      kbo(:, 3,37, 3) = (/ &
     &6.5002e-04_rb,1.2743e-03_rb,1.3189e-03_rb,1.1741e-03_rb,4.2011e-04_rb/)
      kbo(:, 4,37, 3) = (/ &
     &9.5041e-04_rb,1.6515e-03_rb,1.6323e-03_rb,1.3957e-03_rb,4.3822e-04_rb/)
      kbo(:, 5,37, 3) = (/ &
     &1.3379e-03_rb,2.1029e-03_rb,1.9967e-03_rb,1.6424e-03_rb,4.5330e-04_rb/)
      kbo(:, 1,38, 3) = (/ &
     &2.3216e-04_rb,6.1241e-04_rb,6.9804e-04_rb,6.7090e-04_rb,3.0479e-04_rb/)
      kbo(:, 2,38, 3) = (/ &
     &3.7692e-04_rb,8.3215e-04_rb,8.9807e-04_rb,8.2465e-04_rb,3.2506e-04_rb/)
      kbo(:, 3,38, 3) = (/ &
     &5.8109e-04_rb,1.1090e-03_rb,1.1350e-03_rb,9.9936e-04_rb,3.4289e-04_rb/)
      kbo(:, 4,38, 3) = (/ &
     &8.5435e-04_rb,1.4493e-03_rb,1.4148e-03_rb,1.1944e-03_rb,3.5825e-04_rb/)
      kbo(:, 5,38, 3) = (/ &
     &1.2083e-03_rb,1.8590e-03_rb,1.7421e-03_rb,1.4132e-03_rb,3.7105e-04_rb/)
      kbo(:, 1,39, 3) = (/ &
     &2.0931e-04_rb,5.3070e-04_rb,5.9845e-04_rb,5.6903e-04_rb,2.4791e-04_rb/)
      kbo(:, 2,39, 3) = (/ &
     &3.4220e-04_rb,7.2810e-04_rb,7.7651e-04_rb,7.0438e-04_rb,2.6488e-04_rb/)
      kbo(:, 3,39, 3) = (/ &
     &5.3115e-04_rb,9.8088e-04_rb,9.9044e-04_rb,8.5969e-04_rb,2.7995e-04_rb/)
      kbo(:, 4,39, 3) = (/ &
     &7.8539e-04_rb,1.2946e-03_rb,1.2451e-03_rb,1.0342e-03_rb,2.9299e-04_rb/)
      kbo(:, 5,39, 3) = (/ &
     &1.1164e-03_rb,1.6738e-03_rb,1.5453e-03_rb,1.2315e-03_rb,3.0388e-04_rb/)
      kbo(:, 1,40, 3) = (/ &
     &1.6775e-04_rb,4.2853e-04_rb,4.8421e-04_rb,4.6094e-04_rb,1.9976e-04_rb/)
      kbo(:, 2,40, 3) = (/ &
     &2.7762e-04_rb,5.9348e-04_rb,6.3343e-04_rb,5.7451e-04_rb,2.1416e-04_rb/)
      kbo(:, 3,40, 3) = (/ &
     &4.3621e-04_rb,8.0667e-04_rb,8.1437e-04_rb,7.0551e-04_rb,2.2701e-04_rb/)
      kbo(:, 4,40, 3) = (/ &
     &6.5215e-04_rb,1.0751e-03_rb,1.0317e-03_rb,8.5401e-04_rb,2.3821e-04_rb/)
      kbo(:, 5,40, 3) = (/ &
     &9.3504e-04_rb,1.4004e-03_rb,1.2886e-03_rb,1.0222e-03_rb,2.4759e-04_rb/)
      kbo(:, 1,41, 3) = (/ &
     &1.3280e-04_rb,3.4319e-04_rb,3.8888e-04_rb,3.7131e-04_rb,1.6067e-04_rb/)
      kbo(:, 2,41, 3) = (/ &
     &2.2274e-04_rb,4.8016e-04_rb,5.1341e-04_rb,4.6640e-04_rb,1.7291e-04_rb/)
      kbo(:, 3,41, 3) = (/ &
     &3.5456e-04_rb,6.5866e-04_rb,6.6545e-04_rb,5.7628e-04_rb,1.8383e-04_rb/)
      kbo(:, 4,41, 3) = (/ &
     &5.3629e-04_rb,8.8588e-04_rb,8.4970e-04_rb,7.0181e-04_rb,1.9338e-04_rb/)
      kbo(:, 5,41, 3) = (/ &
     &7.7683e-04_rb,1.1654e-03_rb,1.0693e-03_rb,8.4547e-04_rb,2.0151e-04_rb/)
      kbo(:, 1,42, 3) = (/ &
     &1.0593e-04_rb,2.7588e-04_rb,3.1299e-04_rb,2.9948e-04_rb,1.2916e-04_rb/)
      kbo(:, 2,42, 3) = (/ &
     &1.8005e-04_rb,3.9002e-04_rb,4.1742e-04_rb,3.7944e-04_rb,1.3949e-04_rb/)
      kbo(:, 3,42, 3) = (/ &
     &2.9034e-04_rb,5.4056e-04_rb,5.4599e-04_rb,4.7208e-04_rb,1.4879e-04_rb/)
      kbo(:, 4,42, 3) = (/ &
     &4.4476e-04_rb,7.3427e-04_rb,7.0341e-04_rb,5.7900e-04_rb,1.5686e-04_rb/)
      kbo(:, 5,42, 3) = (/ &
     &6.5058e-04_rb,9.7580e-04_rb,8.9207e-04_rb,7.0236e-04_rb,1.6390e-04_rb/)
      kbo(:, 1,43, 3) = (/ &
     &8.2227e-05_rb,2.1779e-04_rb,2.4797e-04_rb,2.3838e-04_rb,1.0321e-04_rb/)
      kbo(:, 2,43, 3) = (/ &
     &1.4211e-04_rb,3.1159e-04_rb,3.3460e-04_rb,3.0498e-04_rb,1.1196e-04_rb/)
      kbo(:, 3,43, 3) = (/ &
     &2.3284e-04_rb,4.3690e-04_rb,4.4252e-04_rb,3.8287e-04_rb,1.1989e-04_rb/)
      kbo(:, 4,43, 3) = (/ &
     &3.6233e-04_rb,6.0081e-04_rb,5.7601e-04_rb,4.7373e-04_rb,1.2682e-04_rb/)
      kbo(:, 5,43, 3) = (/ &
     &5.3702e-04_rb,8.0864e-04_rb,7.3797e-04_rb,5.7931e-04_rb,1.3294e-04_rb/)
      kbo(:, 1,44, 3) = (/ &
     &6.3384e-05_rb,1.7094e-04_rb,1.9525e-04_rb,1.8879e-04_rb,8.2201e-05_rb/)
      kbo(:, 2,44, 3) = (/ &
     &1.1160e-04_rb,2.4797e-04_rb,2.6713e-04_rb,2.4425e-04_rb,8.9566e-05_rb/)
      kbo(:, 3,44, 3) = (/ &
     &1.8608e-04_rb,3.5211e-04_rb,3.5750e-04_rb,3.0979e-04_rb,9.6374e-05_rb/)
      kbo(:, 4,44, 3) = (/ &
     &2.9467e-04_rb,4.9085e-04_rb,4.7122e-04_rb,3.8724e-04_rb,1.0229e-04_rb/)
      kbo(:, 5,44, 3) = (/ &
     &4.4331e-04_rb,6.6909e-04_rb,6.1037e-04_rb,4.7753e-04_rb,1.0759e-04_rb/)
      kbo(:, 1,45, 3) = (/ &
     &4.9661e-05_rb,1.3506e-04_rb,1.5438e-04_rb,1.4993e-04_rb,6.5369e-05_rb/)
      kbo(:, 2,45, 3) = (/ &
     &8.8927e-05_rb,1.9883e-04_rb,2.1432e-04_rb,1.9642e-04_rb,7.1593e-05_rb/)
      kbo(:, 3,45, 3) = (/ &
     &1.5097e-04_rb,2.8657e-04_rb,2.9095e-04_rb,2.5226e-04_rb,7.7369e-05_rb/)
      kbo(:, 4,45, 3) = (/ &
     &2.4324e-04_rb,4.0505e-04_rb,3.8855e-04_rb,3.1866e-04_rb,8.2473e-05_rb/)
      kbo(:, 5,45, 3) = (/ &
     &3.7189e-04_rb,5.6030e-04_rb,5.1022e-04_rb,3.9740e-04_rb,8.7038e-05_rb/)
      kbo(:, 1,46, 3) = (/ &
     &3.8628e-05_rb,1.0586e-04_rb,1.2112e-04_rb,1.1814e-04_rb,5.1792e-05_rb/)
      kbo(:, 2,46, 3) = (/ &
     &7.0302e-05_rb,1.5840e-04_rb,1.7081e-04_rb,1.5701e-04_rb,5.7032e-05_rb/)
      kbo(:, 3,46, 3) = (/ &
     &1.2164e-04_rb,2.3200e-04_rb,2.3559e-04_rb,2.0456e-04_rb,6.1939e-05_rb/)
      kbo(:, 4,46, 3) = (/ &
     &1.9963e-04_rb,3.3271e-04_rb,3.1891e-04_rb,2.6146e-04_rb,6.6327e-05_rb/)
      kbo(:, 5,46, 3) = (/ &
     &3.1076e-04_rb,4.6749e-04_rb,4.2492e-04_rb,3.2997e-04_rb,7.0248e-05_rb/)
      kbo(:, 1,47, 3) = (/ &
     &2.8886e-05_rb,8.0656e-05_rb,9.2760e-05_rb,9.1121e-05_rb,4.0766e-05_rb/)
      kbo(:, 2,47, 3) = (/ &
     &5.3342e-05_rb,1.2263e-04_rb,1.3276e-04_rb,1.2283e-04_rb,4.5144e-05_rb/)
      kbo(:, 3,47, 3) = (/ &
     &9.4301e-05_rb,1.8266e-04_rb,1.8617e-04_rb,1.6258e-04_rb,4.9298e-05_rb/)
      kbo(:, 4,47, 3) = (/ &
     &1.5800e-04_rb,2.6616e-04_rb,2.5579e-04_rb,2.1061e-04_rb,5.3068e-05_rb/)
      kbo(:, 5,47, 3) = (/ &
     &2.5109e-04_rb,3.7981e-04_rb,3.4582e-04_rb,2.6887e-04_rb,5.6458e-05_rb/)
      kbo(:, 1,48, 3) = (/ &
     &2.2227e-05_rb,6.2023e-05_rb,7.1458e-05_rb,7.0462e-05_rb,3.1996e-05_rb/)
      kbo(:, 2,48, 3) = (/ &
     &4.1346e-05_rb,9.5908e-05_rb,1.0391e-04_rb,9.6415e-05_rb,3.5660e-05_rb/)
      kbo(:, 3,48, 3) = (/ &
     &7.4574e-05_rb,1.4557e-04_rb,1.4830e-04_rb,1.2990e-04_rb,3.9158e-05_rb/)
      kbo(:, 4,48, 3) = (/ &
     &1.2768e-04_rb,2.1605e-04_rb,2.0753e-04_rb,1.7110e-04_rb,4.2391e-05_rb/)
      kbo(:, 5,48, 3) = (/ &
     &2.0727e-04_rb,3.1336e-04_rb,2.8498e-04_rb,2.2145e-04_rb,4.5305e-05_rb/)
      kbo(:, 1,49, 3) = (/ &
     &1.7958e-05_rb,4.8535e-05_rb,5.5723e-05_rb,5.4851e-05_rb,2.5055e-05_rb/)
      kbo(:, 2,49, 3) = (/ &
     &3.3224e-05_rb,7.6396e-05_rb,8.2450e-05_rb,7.6323e-05_rb,2.8114e-05_rb/)
      kbo(:, 3,49, 3) = (/ &
     &6.0934e-05_rb,1.1837e-04_rb,1.1999e-04_rb,1.0483e-04_rb,3.1052e-05_rb/)
      kbo(:, 4,49, 3) = (/ &
     &1.0660e-04_rb,1.7943e-04_rb,1.7133e-04_rb,1.4083e-04_rb,3.3806e-05_rb/)
      kbo(:, 5,49, 3) = (/ &
     &1.7683e-04_rb,2.6555e-04_rb,2.4004e-04_rb,1.8562e-04_rb,3.6333e-05_rb/)
      kbo(:, 1,50, 3) = (/ &
     &1.4221e-05_rb,3.7289e-05_rb,4.2772e-05_rb,4.2112e-05_rb,1.9622e-05_rb/)
      kbo(:, 2,50, 3) = (/ &
     &2.5770e-05_rb,5.9303e-05_rb,6.4074e-05_rb,5.9411e-05_rb,2.2155e-05_rb/)
      kbo(:, 3,50, 3) = (/ &
     &4.7760e-05_rb,9.3560e-05_rb,9.4851e-05_rb,8.2933e-05_rb,2.4619e-05_rb/)
      kbo(:, 4,50, 3) = (/ &
     &8.5239e-05_rb,1.4440e-04_rb,1.3769e-04_rb,1.1329e-04_rb,2.6943e-05_rb/)
      kbo(:, 5,50, 3) = (/ &
     &1.4427e-04_rb,2.1753e-04_rb,1.9626e-04_rb,1.5178e-04_rb,2.9098e-05_rb/)
      kbo(:, 1,51, 3) = (/ &
     &1.1317e-05_rb,2.8454e-05_rb,3.2589e-05_rb,3.2092e-05_rb,1.5353e-05_rb/)
      kbo(:, 2,51, 3) = (/ &
     &1.9742e-05_rb,4.5457e-05_rb,4.9229e-05_rb,4.5791e-05_rb,1.7432e-05_rb/)
      kbo(:, 3,51, 3) = (/ &
     &3.6711e-05_rb,7.2714e-05_rb,7.3963e-05_rb,6.4841e-05_rb,1.9496e-05_rb/)
      kbo(:, 4,51, 3) = (/ &
     &6.6639e-05_rb,1.1416e-04_rb,1.0901e-04_rb,8.9977e-05_rb,2.1439e-05_rb/)
      kbo(:, 5,51, 3) = (/ &
     &1.1501e-04_rb,1.7496e-04_rb,1.5780e-04_rb,1.2232e-04_rb,2.3266e-05_rb/)
      kbo(:, 1,52, 3) = (/ &
     &9.5193e-06_rb,2.2151e-05_rb,2.5124e-05_rb,2.4603e-05_rb,1.1980e-05_rb/)
      kbo(:, 2,52, 3) = (/ &
     &1.5692e-05_rb,3.5362e-05_rb,3.8197e-05_rb,3.5510e-05_rb,1.3701e-05_rb/)
      kbo(:, 3,52, 3) = (/ &
     &2.8933e-05_rb,5.7282e-05_rb,5.8273e-05_rb,5.1019e-05_rb,1.5415e-05_rb/)
      kbo(:, 4,52, 3) = (/ &
     &5.3248e-05_rb,9.1604e-05_rb,8.7350e-05_rb,7.2017e-05_rb,1.7048e-05_rb/)
      kbo(:, 5,52, 3) = (/ &
     &9.3646e-05_rb,1.4304e-04_rb,1.2865e-04_rb,9.9576e-05_rb,1.8583e-05_rb/)
      kbo(:, 1,53, 3) = (/ &
     &8.6013e-06_rb,1.7806e-05_rb,1.9756e-05_rb,1.9080e-05_rb,9.3222e-06_rb/)
      kbo(:, 2,53, 3) = (/ &
     &1.3174e-05_rb,2.8179e-05_rb,3.0144e-05_rb,2.7850e-05_rb,1.0743e-05_rb/)
      kbo(:, 3,53, 3) = (/ &
     &2.3711e-05_rb,4.6162e-05_rb,4.6700e-05_rb,4.0629e-05_rb,1.2161e-05_rb/)
      kbo(:, 4,53, 3) = (/ &
     &4.4006e-05_rb,7.5218e-05_rb,7.1366e-05_rb,5.8426e-05_rb,1.3528e-05_rb/)
      kbo(:, 5,53, 3) = (/ &
     &7.8885e-05_rb,1.1994e-04_rb,1.0719e-04_rb,8.2389e-05_rb,1.4819e-05_rb/)
      kbo(:, 1,54, 3) = (/ &
     &6.9789e-06_rb,1.3437e-05_rb,1.4771e-05_rb,1.4238e-05_rb,7.2546e-06_rb/)
      kbo(:, 2,54, 3) = (/ &
     &9.9119e-06_rb,2.0811e-05_rb,2.2371e-05_rb,2.0824e-05_rb,8.4185e-06_rb/)
      kbo(:, 3,54, 3) = (/ &
     &1.7167e-05_rb,3.4065e-05_rb,3.4800e-05_rb,3.0574e-05_rb,9.5808e-06_rb/)
      kbo(:, 4,54, 3) = (/ &
     &3.1909e-05_rb,5.5956e-05_rb,5.3708e-05_rb,4.4436e-05_rb,1.0721e-05_rb/)
      kbo(:, 5,54, 3) = (/ &
     &5.8081e-05_rb,9.0422e-05_rb,8.1544e-05_rb,6.3307e-05_rb,1.1799e-05_rb/)
      kbo(:, 1,55, 3) = (/ &
     &5.3128e-06_rb,9.7390e-06_rb,1.0688e-05_rb,1.0358e-05_rb,5.6367e-06_rb/)
      kbo(:, 2,55, 3) = (/ &
     &6.9987e-06_rb,1.4672e-05_rb,1.5984e-05_rb,1.5113e-05_rb,6.5876e-06_rb/)
      kbo(:, 3,55, 3) = (/ &
     &1.1522e-05_rb,2.3785e-05_rb,2.4757e-05_rb,2.2203e-05_rb,7.5366e-06_rb/)
      kbo(:, 4,55, 3) = (/ &
     &2.1231e-05_rb,3.9099e-05_rb,3.8355e-05_rb,3.2404e-05_rb,8.4801e-06_rb/)
      kbo(:, 5,55, 3) = (/ &
     &3.9115e-05_rb,6.3594e-05_rb,5.8504e-05_rb,4.6398e-05_rb,9.3778e-06_rb/)
      kbo(:, 1,56, 3) = (/ &
     &4.1255e-06_rb,7.1364e-06_rb,7.7808e-06_rb,7.5482e-06_rb,4.3681e-06_rb/)
      kbo(:, 2,56, 3) = (/ &
     &5.0603e-06_rb,1.0419e-05_rb,1.1462e-05_rb,1.0974e-05_rb,5.1464e-06_rb/)
      kbo(:, 3,56, 3) = (/ &
     &7.8194e-06_rb,1.6631e-05_rb,1.7619e-05_rb,1.6112e-05_rb,5.9174e-06_rb/)
      kbo(:, 4,56, 3) = (/ &
     &1.4119e-05_rb,2.7308e-05_rb,2.7375e-05_rb,2.3608e-05_rb,6.6946e-06_rb/)
      kbo(:, 5,56, 3) = (/ &
     &2.6208e-05_rb,4.4637e-05_rb,4.1923e-05_rb,3.3985e-05_rb,7.4429e-06_rb/)
      kbo(:, 1,57, 3) = (/ &
     &3.2493e-06_rb,5.2971e-06_rb,5.6993e-06_rb,5.5117e-06_rb,3.3724e-06_rb/)
      kbo(:, 2,57, 3) = (/ &
     &3.7488e-06_rb,7.4571e-06_rb,8.2480e-06_rb,7.9737e-06_rb,4.0105e-06_rb/)
      kbo(:, 3,57, 3) = (/ &
     &5.3914e-06_rb,1.1658e-05_rb,1.2555e-05_rb,1.1685e-05_rb,4.6429e-06_rb/)
      kbo(:, 4,57, 3) = (/ &
     &9.4028e-06_rb,1.9067e-05_rb,1.9516e-05_rb,1.7175e-05_rb,5.2749e-06_rb/)
      kbo(:, 5,57, 3) = (/ &
     &1.7491e-05_rb,3.1266e-05_rb,3.0002e-05_rb,2.4856e-05_rb,5.8965e-06_rb/)
      kbo(:, 1,58, 3) = (/ &
     &6.8210e-07_rb,1.3006e-06_rb,1.7801e-06_rb,2.4088e-06_rb,2.6011e-06_rb/)
      kbo(:, 2,58, 3) = (/ &
     &7.5233e-07_rb,1.7695e-06_rb,2.5293e-06_rb,3.4612e-06_rb,3.1260e-06_rb/)
      kbo(:, 3,58, 3) = (/ &
     &1.0077e-06_rb,2.6981e-06_rb,3.8040e-06_rb,5.0545e-06_rb,3.6427e-06_rb/)
      kbo(:, 4,58, 3) = (/ &
     &1.6789e-06_rb,4.3767e-06_rb,5.8928e-06_rb,7.4329e-06_rb,4.1541e-06_rb/)
      kbo(:, 5,58, 3) = (/ &
     &3.1035e-06_rb,7.1707e-06_rb,9.0733e-06_rb,1.0789e-05_rb,4.6675e-06_rb/)
      kbo(:, 1,59, 3) = (/ &
     &6.5007e-07_rb,1.2934e-06_rb,1.7266e-06_rb,2.1835e-06_rb,2.0738e-06_rb/)
      kbo(:, 2,59, 3) = (/ &
     &7.0640e-07_rb,1.7649e-06_rb,2.4593e-06_rb,3.1227e-06_rb,2.5027e-06_rb/)
      kbo(:, 3,59, 3) = (/ &
     &9.2077e-07_rb,2.6612e-06_rb,3.6625e-06_rb,4.5103e-06_rb,2.9231e-06_rb/)
      kbo(:, 4,59, 3) = (/ &
     &1.5013e-06_rb,4.2541e-06_rb,5.5816e-06_rb,6.5373e-06_rb,3.3365e-06_rb/)
      kbo(:, 5,59, 3) = (/ &
     &2.7629e-06_rb,6.8551e-06_rb,8.4568e-06_rb,9.3583e-06_rb,3.7483e-06_rb/)
      kbo(:, 1,13, 4) = (/ &
     &6.7049e-02_rb,1.0578e-01_rb,1.2049e-01_rb,1.1447e-01_rb,5.3996e-02_rb/)
      kbo(:, 2,13, 4) = (/ &
     &1.2378e-01_rb,1.5839e-01_rb,1.6716e-01_rb,1.4967e-01_rb,5.8724e-02_rb/)
      kbo(:, 3,13, 4) = (/ &
     &2.1508e-01_rb,2.3357e-01_rb,2.2807e-01_rb,1.9256e-01_rb,6.3468e-02_rb/)
      kbo(:, 4,13, 4) = (/ &
     &3.4658e-01_rb,3.3547e-01_rb,3.0532e-01_rb,2.4368e-01_rb,6.7895e-02_rb/)
      kbo(:, 5,13, 4) = (/ &
     &5.1918e-01_rb,4.6542e-01_rb,4.0071e-01_rb,3.0351e-01_rb,7.2180e-02_rb/)
      kbo(:, 1,14, 4) = (/ &
     &3.7591e-02_rb,7.5935e-02_rb,8.9834e-02_rb,8.8154e-02_rb,4.7370e-02_rb/)
      kbo(:, 2,14, 4) = (/ &
     &6.9282e-02_rb,1.0939e-01_rb,1.2082e-01_rb,1.1264e-01_rb,5.1597e-02_rb/)
      kbo(:, 3,14, 4) = (/ &
     &1.1973e-01_rb,1.5456e-01_rb,1.5973e-01_rb,1.4179e-01_rb,5.5698e-02_rb/)
      kbo(:, 4,14, 4) = (/ &
     &1.9141e-01_rb,2.1319e-01_rb,2.0757e-01_rb,1.7567e-01_rb,5.9503e-02_rb/)
      kbo(:, 5,14, 4) = (/ &
     &2.8419e-01_rb,2.8689e-01_rb,2.6537e-01_rb,2.1426e-01_rb,6.3167e-02_rb/)
      kbo(:, 1,15, 4) = (/ &
     &2.1562e-02_rb,5.5383e-02_rb,6.7683e-02_rb,6.8583e-02_rb,4.1403e-02_rb/)
      kbo(:, 2,15, 4) = (/ &
     &3.9720e-02_rb,7.6977e-02_rb,8.8827e-02_rb,8.5804e-02_rb,4.5107e-02_rb/)
      kbo(:, 3,15, 4) = (/ &
     &6.8049e-02_rb,1.0522e-01_rb,1.1458e-01_rb,1.0592e-01_rb,4.8636e-02_rb/)
      kbo(:, 4,15, 4) = (/ &
     &1.0779e-01_rb,1.4096e-01_rb,1.4532e-01_rb,1.2906e-01_rb,5.1972e-02_rb/)
      kbo(:, 5,15, 4) = (/ &
     &1.5865e-01_rb,1.8497e-01_rb,1.8175e-01_rb,1.5501e-01_rb,5.5102e-02_rb/)
      kbo(:, 1,16, 4) = (/ &
     &1.4772e-02_rb,4.3932e-02_rb,5.4513e-02_rb,5.6391e-02_rb,3.6009e-02_rb/)
      kbo(:, 2,16, 4) = (/ &
     &2.7132e-02_rb,5.9972e-02_rb,7.0550e-02_rb,6.9709e-02_rb,3.9169e-02_rb/)
      kbo(:, 3,16, 4) = (/ &
     &4.6049e-02_rb,8.0444e-02_rb,8.9769e-02_rb,8.5125e-02_rb,4.2209e-02_rb/)
      kbo(:, 4,16, 4) = (/ &
     &7.2213e-02_rb,1.0588e-01_rb,1.1245e-01_rb,1.0271e-01_rb,4.5097e-02_rb/)
      kbo(:, 5,16, 4) = (/ &
     &1.0534e-01_rb,1.3693e-01_rb,1.3910e-01_rb,1.2239e-01_rb,4.7833e-02_rb/)
      kbo(:, 1,17, 4) = (/ &
     &1.0505e-02_rb,3.5195e-02_rb,4.4278e-02_rb,4.6677e-02_rb,3.1091e-02_rb/)
      kbo(:, 2,17, 4) = (/ &
     &1.9166e-02_rb,4.7335e-02_rb,5.6632e-02_rb,5.7162e-02_rb,3.3802e-02_rb/)
      kbo(:, 3,17, 4) = (/ &
     &3.2242e-02_rb,6.2594e-02_rb,7.1319e-02_rb,6.9208e-02_rb,3.6407e-02_rb/)
      kbo(:, 4,17, 4) = (/ &
     &4.9961e-02_rb,8.1341e-02_rb,8.8470e-02_rb,8.2810e-02_rb,3.8893e-02_rb/)
      kbo(:, 5,17, 4) = (/ &
     &7.2404e-02_rb,1.0400e-01_rb,1.0862e-01_rb,9.8084e-02_rb,4.1273e-02_rb/)
      kbo(:, 1,18, 4) = (/ &
     &7.6562e-03_rb,2.8333e-02_rb,3.6121e-02_rb,3.8835e-02_rb,2.6686e-02_rb/)
      kbo(:, 2,18, 4) = (/ &
     &1.3867e-02_rb,3.7676e-02_rb,4.5765e-02_rb,4.7147e-02_rb,2.9013e-02_rb/)
      kbo(:, 3,18, 4) = (/ &
     &2.3086e-02_rb,4.9274e-02_rb,5.7143e-02_rb,5.6661e-02_rb,3.1231e-02_rb/)
      kbo(:, 4,18, 4) = (/ &
     &3.5350e-02_rb,6.3412e-02_rb,7.0379e-02_rb,6.7363e-02_rb,3.3367e-02_rb/)
      kbo(:, 5,18, 4) = (/ &
     &5.0974e-02_rb,8.0253e-02_rb,8.5775e-02_rb,7.9284e-02_rb,3.5380e-02_rb/)
      kbo(:, 1,19, 4) = (/ &
     &5.4803e-03_rb,2.2514e-02_rb,2.9208e-02_rb,3.2099e-02_rb,2.2789e-02_rb/)
      kbo(:, 2,19, 4) = (/ &
     &9.8317e-03_rb,2.9614e-02_rb,3.6650e-02_rb,3.8632e-02_rb,2.4757e-02_rb/)
      kbo(:, 3,19, 4) = (/ &
     &1.6184e-02_rb,3.8312e-02_rb,4.5365e-02_rb,4.6081e-02_rb,2.6634e-02_rb/)
      kbo(:, 4,19, 4) = (/ &
     &2.4529e-02_rb,4.8776e-02_rb,5.5440e-02_rb,5.4451e-02_rb,2.8434e-02_rb/)
      kbo(:, 5,19, 4) = (/ &
     &3.5233e-02_rb,6.1231e-02_rb,6.7107e-02_rb,6.3779e-02_rb,3.0105e-02_rb/)
      kbo(:, 1,20, 4) = (/ &
     &4.2828e-03_rb,1.8555e-02_rb,2.4277e-02_rb,2.6995e-02_rb,1.9406e-02_rb/)
      kbo(:, 2,20, 4) = (/ &
     &7.5931e-03_rb,2.4236e-02_rb,3.0290e-02_rb,3.2347e-02_rb,2.1050e-02_rb/)
      kbo(:, 3,20, 4) = (/ &
     &1.2330e-02_rb,3.1123e-02_rb,3.7288e-02_rb,3.8414e-02_rb,2.2621e-02_rb/)
      kbo(:, 4,20, 4) = (/ &
     &1.8510e-02_rb,3.9358e-02_rb,4.5374e-02_rb,4.5224e-02_rb,2.4107e-02_rb/)
      kbo(:, 5,20, 4) = (/ &
     &2.6546e-02_rb,4.9149e-02_rb,5.4720e-02_rb,5.2789e-02_rb,2.5443e-02_rb/)
      kbo(:, 1,21, 4) = (/ &
     &3.4446e-03_rb,1.5454e-02_rb,2.0322e-02_rb,2.2766e-02_rb,1.6445e-02_rb/)
      kbo(:, 2,21, 4) = (/ &
     &6.0284e-03_rb,2.0063e-02_rb,2.5258e-02_rb,2.7191e-02_rb,1.7819e-02_rb/)
      kbo(:, 3,21, 4) = (/ &
     &9.6698e-03_rb,2.5633e-02_rb,3.0993e-02_rb,3.2202e-02_rb,1.9116e-02_rb/)
      kbo(:, 4,21, 4) = (/ &
     &1.4416e-02_rb,3.2230e-02_rb,3.7573e-02_rb,3.7789e-02_rb,2.0305e-02_rb/)
      kbo(:, 5,21, 4) = (/ &
     &2.0653e-02_rb,4.0104e-02_rb,4.5153e-02_rb,4.3956e-02_rb,2.1322e-02_rb/)
      kbo(:, 1,22, 4) = (/ &
     &2.9417e-03_rb,1.3191e-02_rb,1.7364e-02_rb,1.9451e-02_rb,1.3960e-02_rb/)
      kbo(:, 2,22, 4) = (/ &
     &5.0614e-03_rb,1.7031e-02_rb,2.1496e-02_rb,2.3163e-02_rb,1.5084e-02_rb/)
      kbo(:, 3,22, 4) = (/ &
     &7.9857e-03_rb,2.1643e-02_rb,2.6269e-02_rb,2.7349e-02_rb,1.6128e-02_rb/)
      kbo(:, 4,22, 4) = (/ &
     &1.1815e-02_rb,2.7096e-02_rb,3.1716e-02_rb,3.1974e-02_rb,1.7035e-02_rb/)
      kbo(:, 5,22, 4) = (/ &
     &1.6874e-02_rb,3.3630e-02_rb,3.8023e-02_rb,3.7066e-02_rb,1.7780e-02_rb/)
      kbo(:, 1,23, 4) = (/ &
     &2.5270e-03_rb,1.1266e-02_rb,1.4840e-02_rb,1.6589e-02_rb,1.1804e-02_rb/)
      kbo(:, 2,23, 4) = (/ &
     &4.2729e-03_rb,1.4464e-02_rb,1.8293e-02_rb,1.9688e-02_rb,1.2713e-02_rb/)
      kbo(:, 3,23, 4) = (/ &
     &6.6524e-03_rb,1.8294e-02_rb,2.2254e-02_rb,2.3162e-02_rb,1.3520e-02_rb/)
      kbo(:, 4,23, 4) = (/ &
     &9.7631e-03_rb,2.2820e-02_rb,2.6759e-02_rb,2.6972e-02_rb,1.4190e-02_rb/)
      kbo(:, 5,23, 4) = (/ &
     &1.3880e-02_rb,2.8296e-02_rb,3.2015e-02_rb,3.1138e-02_rb,1.4756e-02_rb/)
      kbo(:, 1,24, 4) = (/ &
     &2.1649e-03_rb,9.5893e-03_rb,1.2641e-02_rb,1.4093e-02_rb,9.9437e-03_rb/)
      kbo(:, 2,24, 4) = (/ &
     &3.5984e-03_rb,1.2238e-02_rb,1.5497e-02_rb,1.6660e-02_rb,1.0660e-02_rb/)
      kbo(:, 3,24, 4) = (/ &
     &5.5343e-03_rb,1.5407e-02_rb,1.8761e-02_rb,1.9506e-02_rb,1.1265e-02_rb/)
      kbo(:, 4,24, 4) = (/ &
     &8.0630e-03_rb,1.9178e-02_rb,2.2493e-02_rb,2.2596e-02_rb,1.1775e-02_rb/)
      kbo(:, 5,24, 4) = (/ &
     &1.1384e-02_rb,2.3746e-02_rb,2.6831e-02_rb,2.5965e-02_rb,1.2211e-02_rb/)
      kbo(:, 1,25, 4) = (/ &
     &1.8852e-03_rb,8.1978e-03_rb,1.0793e-02_rb,1.1983e-02_rb,8.3500e-03_rb/)
      kbo(:, 2,25, 4) = (/ &
     &3.0822e-03_rb,1.0410e-02_rb,1.3161e-02_rb,1.4105e-02_rb,8.8930e-03_rb/)
      kbo(:, 3,25, 4) = (/ &
     &4.6788e-03_rb,1.3060e-02_rb,1.5866e-02_rb,1.6423e-02_rb,9.3555e-03_rb/)
      kbo(:, 4,25, 4) = (/ &
     &6.7672e-03_rb,1.6246e-02_rb,1.8965e-02_rb,1.8919e-02_rb,9.7508e-03_rb/)
      kbo(:, 5,25, 4) = (/ &
     &9.4703e-03_rb,2.0069e-02_rb,2.2560e-02_rb,2.1660e-02_rb,1.0087e-02_rb/)
      kbo(:, 1,26, 4) = (/ &
     &1.6838e-03_rb,7.0691e-03_rb,9.2649e-03_rb,1.0222e-02_rb,6.9856e-03_rb/)
      kbo(:, 2,26, 4) = (/ &
     &2.7021e-03_rb,8.9432e-03_rb,1.1240e-02_rb,1.1970e-02_rb,7.4013e-03_rb/)
      kbo(:, 3,26, 4) = (/ &
     &4.0452e-03_rb,1.1195e-02_rb,1.3497e-02_rb,1.3851e-02_rb,7.7588e-03_rb/)
      kbo(:, 4,26, 4) = (/ &
     &5.8017e-03_rb,1.3912e-02_rb,1.6094e-02_rb,1.5906e-02_rb,8.0669e-03_rb/)
      kbo(:, 5,26, 4) = (/ &
     &8.0448e-03_rb,1.7140e-02_rb,1.9095e-02_rb,1.8175e-02_rb,8.3263e-03_rb/)
      kbo(:, 1,27, 4) = (/ &
     &1.5607e-03_rb,6.1890e-03_rb,8.0258e-03_rb,8.7683e-03_rb,5.8228e-03_rb/)
      kbo(:, 2,27, 4) = (/ &
     &2.4561e-03_rb,7.8183e-03_rb,9.7010e-03_rb,1.0217e-02_rb,6.1459e-03_rb/)
      kbo(:, 3,27, 4) = (/ &
     &3.6253e-03_rb,9.7806e-03_rb,1.1625e-02_rb,1.1787e-02_rb,6.4270e-03_rb/)
      kbo(:, 4,27, 4) = (/ &
     &5.1578e-03_rb,1.2146e-02_rb,1.3856e-02_rb,1.3525e-02_rb,6.6670e-03_rb/)
      kbo(:, 5,27, 4) = (/ &
     &7.0917e-03_rb,1.4929e-02_rb,1.6419e-02_rb,1.5452e-02_rb,6.8676e-03_rb/)
      kbo(:, 1,28, 4) = (/ &
     &1.4790e-03_rb,5.4823e-03_rb,6.9976e-03_rb,7.5478e-03_rb,4.8444e-03_rb/)
      kbo(:, 2,28, 4) = (/ &
     &2.2841e-03_rb,6.9262e-03_rb,8.4418e-03_rb,8.7670e-03_rb,5.0989e-03_rb/)
      kbo(:, 3,28, 4) = (/ &
     &3.3280e-03_rb,8.6594e-03_rb,1.0117e-02_rb,1.0108e-02_rb,5.3174e-03_rb/)
      kbo(:, 4,28, 4) = (/ &
     &4.7002e-03_rb,1.0747e-02_rb,1.2065e-02_rb,1.1610e-02_rb,5.5053e-03_rb/)
      kbo(:, 5,28, 4) = (/ &
     &6.4216e-03_rb,1.3181e-02_rb,1.4294e-02_rb,1.3272e-02_rb,5.6597e-03_rb/)
      kbo(:, 1,29, 4) = (/ &
     &1.4846e-03_rb,5.0115e-03_rb,6.2330e-03_rb,6.5969e-03_rb,4.0271e-03_rb/)
      kbo(:, 2,29, 4) = (/ &
     &2.2530e-03_rb,6.3449e-03_rb,7.5362e-03_rb,7.6648e-03_rb,4.2267e-03_rb/)
      kbo(:, 3,29, 4) = (/ &
     &3.2510e-03_rb,7.9458e-03_rb,9.0603e-03_rb,8.8629e-03_rb,4.3989e-03_rb/)
      kbo(:, 4,29, 4) = (/ &
     &4.5516e-03_rb,9.8569e-03_rb,1.0828e-02_rb,1.0206e-02_rb,4.5467e-03_rb/)
      kbo(:, 5,29, 4) = (/ &
     &6.1805e-03_rb,1.2091e-02_rb,1.2839e-02_rb,1.1688e-02_rb,4.6649e-03_rb/)
      kbo(:, 1,30, 4) = (/ &
     &1.5199e-03_rb,4.6536e-03_rb,5.6184e-03_rb,5.8111e-03_rb,3.3446e-03_rb/)
      kbo(:, 2,30, 4) = (/ &
     &2.2691e-03_rb,5.9027e-03_rb,6.8239e-03_rb,6.7733e-03_rb,3.5018e-03_rb/)
      kbo(:, 3,30, 4) = (/ &
     &3.2488e-03_rb,7.4072e-03_rb,8.2321e-03_rb,7.8632e-03_rb,3.6376e-03_rb/)
      kbo(:, 4,30, 4) = (/ &
     &4.5119e-03_rb,9.1886e-03_rb,9.8595e-03_rb,9.0823e-03_rb,3.7530e-03_rb/)
      kbo(:, 5,30, 4) = (/ &
     &6.0862e-03_rb,1.1295e-02_rb,1.1709e-02_rb,1.0421e-02_rb,3.8437e-03_rb/)
      kbo(:, 1,31, 4) = (/ &
     &1.6311e-03_rb,4.4648e-03_rb,5.2027e-03_rb,5.2222e-03_rb,2.7768e-03_rb/)
      kbo(:, 2,31, 4) = (/ &
     &2.3988e-03_rb,5.6776e-03_rb,6.3545e-03_rb,6.1217e-03_rb,2.9011e-03_rb/)
      kbo(:, 3,31, 4) = (/ &
     &3.4145e-03_rb,7.1414e-03_rb,7.7030e-03_rb,7.1444e-03_rb,3.0093e-03_rb/)
      kbo(:, 4,31, 4) = (/ &
     &4.7033e-03_rb,8.8852e-03_rb,9.2526e-03_rb,8.2855e-03_rb,3.0977e-03_rb/)
      kbo(:, 5,31, 4) = (/ &
     &6.2907e-03_rb,1.0964e-02_rb,1.1034e-02_rb,9.5381e-03_rb,3.1698e-03_rb/)
      kbo(:, 1,32, 4) = (/ &
     &1.7646e-03_rb,4.3392e-03_rb,4.8857e-03_rb,4.7446e-03_rb,2.3054e-03_rb/)
      kbo(:, 2,32, 4) = (/ &
     &2.5651e-03_rb,5.5295e-03_rb,5.9929e-03_rb,5.5951e-03_rb,2.4036e-03_rb/)
      kbo(:, 3,32, 4) = (/ &
     &3.6260e-03_rb,6.9759e-03_rb,7.2903e-03_rb,6.5646e-03_rb,2.4876e-03_rb/)
      kbo(:, 4,32, 4) = (/ &
     &4.9515e-03_rb,8.7151e-03_rb,8.7927e-03_rb,7.6385e-03_rb,2.5574e-03_rb/)
      kbo(:, 5,32, 4) = (/ &
     &6.5598e-03_rb,1.0782e-02_rb,1.0540e-02_rb,8.8332e-03_rb,2.6142e-03_rb/)
      kbo(:, 1,33, 4) = (/ &
     &1.9374e-03_rb,4.2874e-03_rb,4.6591e-03_rb,4.3717e-03_rb,1.9130e-03_rb/)
      kbo(:, 2,33, 4) = (/ &
     &2.7949e-03_rb,5.4803e-03_rb,5.7408e-03_rb,5.1902e-03_rb,1.9904e-03_rb/)
      kbo(:, 3,33, 4) = (/ &
     &3.9103e-03_rb,6.9411e-03_rb,7.0122e-03_rb,6.1134e-03_rb,2.0570e-03_rb/)
      kbo(:, 4,33, 4) = (/ &
     &5.2898e-03_rb,8.7072e-03_rb,8.5064e-03_rb,7.1438e-03_rb,2.1115e-03_rb/)
      kbo(:, 5,33, 4) = (/ &
     &6.9527e-03_rb,1.0788e-02_rb,1.0239e-02_rb,8.3069e-03_rb,2.1568e-03_rb/)
      kbo(:, 1,34, 4) = (/ &
     &2.0406e-03_rb,4.1540e-03_rb,4.3826e-03_rb,3.9946e-03_rb,1.5845e-03_rb/)
      kbo(:, 2,34, 4) = (/ &
     &2.9261e-03_rb,5.3300e-03_rb,5.4194e-03_rb,4.7632e-03_rb,1.6461e-03_rb/)
      kbo(:, 3,34, 4) = (/ &
     &4.0546e-03_rb,6.7773e-03_rb,6.6513e-03_rb,5.6287e-03_rb,1.6983e-03_rb/)
      kbo(:, 4,34, 4) = (/ &
     &5.4348e-03_rb,8.5143e-03_rb,8.1040e-03_rb,6.6107e-03_rb,1.7421e-03_rb/)
      kbo(:, 5,34, 4) = (/ &
     &7.1060e-03_rb,1.0544e-02_rb,9.7699e-03_rb,7.7212e-03_rb,1.7780e-03_rb/)
      kbo(:, 1,35, 4) = (/ &
     &2.0728e-03_rb,3.9485e-03_rb,4.0574e-03_rb,3.6081e-03_rb,1.3068e-03_rb/)
      kbo(:, 2,35, 4) = (/ &
     &2.9602e-03_rb,5.0932e-03_rb,5.0438e-03_rb,4.3192e-03_rb,1.3565e-03_rb/)
      kbo(:, 3,35, 4) = (/ &
     &4.0769e-03_rb,6.5016e-03_rb,6.2270e-03_rb,5.1307e-03_rb,1.3990e-03_rb/)
      kbo(:, 4,35, 4) = (/ &
     &5.4354e-03_rb,8.1764e-03_rb,7.6137e-03_rb,6.0597e-03_rb,1.4343e-03_rb/)
      kbo(:, 5,35, 4) = (/ &
     &7.0821e-03_rb,1.0134e-02_rb,9.1905e-03_rb,7.1011e-03_rb,1.4630e-03_rb/)
      kbo(:, 1,36, 4) = (/ &
     &2.0103e-03_rb,3.6479e-03_rb,3.6691e-03_rb,3.1952e-03_rb,1.0729e-03_rb/)
      kbo(:, 2,36, 4) = (/ &
     &2.8662e-03_rb,4.7338e-03_rb,4.5904e-03_rb,3.8448e-03_rb,1.1132e-03_rb/)
      kbo(:, 3,36, 4) = (/ &
     &3.9368e-03_rb,6.0663e-03_rb,5.7008e-03_rb,4.5973e-03_rb,1.1487e-03_rb/)
      kbo(:, 4,36, 4) = (/ &
     &5.2410e-03_rb,7.6444e-03_rb,6.9919e-03_rb,5.4585e-03_rb,1.1779e-03_rb/)
      kbo(:, 5,36, 4) = (/ &
     &6.8204e-03_rb,9.5005e-03_rb,8.4610e-03_rb,6.4164e-03_rb,1.2022e-03_rb/)
      kbo(:, 1,37, 4) = (/ &
     &1.7959e-03_rb,3.1864e-03_rb,3.1663e-03_rb,2.7239e-03_rb,8.7469e-04_rb/)
      kbo(:, 2,37, 4) = (/ &
     &2.5697e-03_rb,4.1617e-03_rb,3.9876e-03_rb,3.2966e-03_rb,9.0931e-04_rb/)
      kbo(:, 3,37, 4) = (/ &
     &3.5391e-03_rb,5.3610e-03_rb,4.9818e-03_rb,3.9676e-03_rb,9.3897e-04_rb/)
      kbo(:, 4,37, 4) = (/ &
     &4.7240e-03_rb,6.7847e-03_rb,6.1364e-03_rb,4.7358e-03_rb,9.6399e-04_rb/)
      kbo(:, 5,37, 4) = (/ &
     &6.1586e-03_rb,8.4655e-03_rb,7.4564e-03_rb,5.5875e-03_rb,9.8533e-04_rb/)
      kbo(:, 1,38, 4) = (/ &
     &1.6221e-03_rb,2.8081e-03_rb,2.7522e-03_rb,2.3347e-03_rb,7.1314e-04_rb/)
      kbo(:, 2,38, 4) = (/ &
     &2.3291e-03_rb,3.6929e-03_rb,3.4899e-03_rb,2.8446e-03_rb,7.4249e-04_rb/)
      kbo(:, 3,38, 4) = (/ &
     &3.2190e-03_rb,4.7846e-03_rb,4.3892e-03_rb,3.4480e-03_rb,7.6765e-04_rb/)
      kbo(:, 4,38, 4) = (/ &
     &4.3100e-03_rb,6.0843e-03_rb,5.4322e-03_rb,4.1379e-03_rb,7.8970e-04_rb/)
      kbo(:, 5,38, 4) = (/ &
     &5.6295e-03_rb,7.6227e-03_rb,6.6325e-03_rb,4.9024e-03_rb,8.0762e-04_rb/)
      kbo(:, 1,39, 4) = (/ &
     &1.4985e-03_rb,2.5181e-03_rb,2.4272e-03_rb,2.0247e-03_rb,5.8188e-04_rb/)
      kbo(:, 2,39, 4) = (/ &
     &2.1580e-03_rb,3.3353e-03_rb,3.1025e-03_rb,2.4863e-03_rb,6.0647e-04_rb/)
      kbo(:, 3,39, 4) = (/ &
     &2.9946e-03_rb,4.3491e-03_rb,3.9298e-03_rb,3.0357e-03_rb,6.2817e-04_rb/)
      kbo(:, 4,39, 4) = (/ &
     &4.0211e-03_rb,5.5590e-03_rb,4.8918e-03_rb,3.6649e-03_rb,6.4668e-04_rb/)
      kbo(:, 5,39, 4) = (/ &
     &5.2642e-03_rb,6.9995e-03_rb,6.0040e-03_rb,4.3661e-03_rb,6.6239e-04_rb/)
      kbo(:, 1,40, 4) = (/ &
     &1.2480e-03_rb,2.0953e-03_rb,2.0102e-03_rb,1.6690e-03_rb,4.7210e-04_rb/)
      kbo(:, 2,40, 4) = (/ &
     &1.8115e-03_rb,2.7936e-03_rb,2.5861e-03_rb,2.0625e-03_rb,4.9296e-04_rb/)
      kbo(:, 3,40, 4) = (/ &
     &2.5332e-03_rb,3.6693e-03_rb,3.2977e-03_rb,2.5337e-03_rb,5.1142e-04_rb/)
      kbo(:, 4,40, 4) = (/ &
     &3.4226e-03_rb,4.7175e-03_rb,4.1297e-03_rb,3.0758e-03_rb,5.2719e-04_rb/)
      kbo(:, 5,40, 4) = (/ &
     &4.4988e-03_rb,5.9686e-03_rb,5.0929e-03_rb,3.6827e-03_rb,5.4108e-04_rb/)
      kbo(:, 1,41, 4) = (/ &
     &1.0279e-03_rb,1.7303e-03_rb,1.6546e-03_rb,1.3684e-03_rb,3.8259e-04_rb/)
      kbo(:, 2,41, 4) = (/ &
     &1.5100e-03_rb,2.3260e-03_rb,2.1442e-03_rb,1.7025e-03_rb,4.0029e-04_rb/)
      kbo(:, 3,41, 4) = (/ &
     &2.1257e-03_rb,3.0759e-03_rb,2.7514e-03_rb,2.1044e-03_rb,4.1598e-04_rb/)
      kbo(:, 4,41, 4) = (/ &
     &2.8920e-03_rb,3.9799e-03_rb,3.4670e-03_rb,2.5693e-03_rb,4.2980e-04_rb/)
      kbo(:, 5,41, 4) = (/ &
     &3.8245e-03_rb,5.0645e-03_rb,4.3009e-03_rb,3.0935e-03_rb,4.4154e-04_rb/)
      kbo(:, 1,42, 4) = (/ &
     &8.5248e-04_rb,1.4366e-03_rb,1.3681e-03_rb,1.1259e-03_rb,3.0986e-04_rb/)
      kbo(:, 2,42, 4) = (/ &
     &1.2693e-03_rb,1.9496e-03_rb,1.7879e-03_rb,1.4116e-03_rb,3.2494e-04_rb/)
      kbo(:, 3,42, 4) = (/ &
     &1.7992e-03_rb,2.5962e-03_rb,2.3093e-03_rb,1.7561e-03_rb,3.3826e-04_rb/)
      kbo(:, 4,42, 4) = (/ &
     &2.4657e-03_rb,3.3829e-03_rb,2.9302e-03_rb,2.1577e-03_rb,3.5033e-04_rb/)
      kbo(:, 5,42, 4) = (/ &
     &3.2835e-03_rb,4.3314e-03_rb,3.6581e-03_rb,2.6128e-03_rb,3.6051e-04_rb/)
      kbo(:, 1,43, 4) = (/ &
     &6.9299e-04_rb,1.1756e-03_rb,1.1172e-03_rb,9.1666e-04_rb,2.5008e-04_rb/)
      kbo(:, 2,43, 4) = (/ &
     &1.0504e-03_rb,1.6147e-03_rb,1.4746e-03_rb,1.1591e-03_rb,2.6296e-04_rb/)
      kbo(:, 3,43, 4) = (/ &
     &1.5042e-03_rb,2.1683e-03_rb,1.9190e-03_rb,1.4524e-03_rb,2.7439e-04_rb/)
      kbo(:, 4,43, 4) = (/ &
     &2.0819e-03_rb,2.8510e-03_rb,2.4566e-03_rb,1.7990e-03_rb,2.8482e-04_rb/)
      kbo(:, 5,43, 4) = (/ &
     &2.7959e-03_rb,3.6773e-03_rb,3.0899e-03_rb,2.1936e-03_rb,2.9376e-04_rb/)
      kbo(:, 1,44, 4) = (/ &
     &5.6092e-04_rb,9.5981e-04_rb,9.1038e-04_rb,7.4434e-04_rb,2.0138e-04_rb/)
      kbo(:, 2,44, 4) = (/ &
     &8.6869e-04_rb,1.3365e-03_rb,1.2153e-03_rb,9.5085e-04_rb,2.1240e-04_rb/)
      kbo(:, 3,44, 4) = (/ &
     &1.2605e-03_rb,1.8133e-03_rb,1.5968e-03_rb,1.2017e-03_rb,2.2220e-04_rb/)
      kbo(:, 4,44, 4) = (/ &
     &1.7639e-03_rb,2.4086e-03_rb,2.0630e-03_rb,1.5011e-03_rb,2.3130e-04_rb/)
      kbo(:, 5,44, 4) = (/ &
     &2.3892e-03_rb,3.1325e-03_rb,2.6172e-03_rb,1.8457e-03_rb,2.3904e-04_rb/)
      kbo(:, 1,45, 4) = (/ &
     &4.5956e-04_rb,7.9161e-04_rb,7.4840e-04_rb,6.0859e-04_rb,1.6203e-04_rb/)
      kbo(:, 2,45, 4) = (/ &
     &7.2822e-04_rb,1.1187e-03_rb,1.0117e-03_rb,7.8592e-04_rb,1.7142e-04_rb/)
      kbo(:, 3,45, 4) = (/ &
     &1.0767e-03_rb,1.5389e-03_rb,1.3458e-03_rb,1.0038e-03_rb,1.7986e-04_rb/)
      kbo(:, 4,45, 4) = (/ &
     &1.5195e-03_rb,2.0634e-03_rb,1.7544e-03_rb,1.2651e-03_rb,1.8773e-04_rb/)
      kbo(:, 5,45, 4) = (/ &
     &2.0790e-03_rb,2.7102e-03_rb,2.2473e-03_rb,1.5700e-03_rb,1.9451e-04_rb/)
      kbo(:, 1,46, 4) = (/ &
     &3.7292e-04_rb,6.4911e-04_rb,6.1209e-04_rb,4.9545e-04_rb,1.3008e-04_rb/)
      kbo(:, 2,46, 4) = (/ &
     &6.0673e-04_rb,9.3287e-04_rb,8.3976e-04_rb,6.4802e-04_rb,1.3811e-04_rb/)
      kbo(:, 3,46, 4) = (/ &
     &9.1635e-04_rb,1.3033e-03_rb,1.1323e-03_rb,8.3718e-04_rb,1.4541e-04_rb/)
      kbo(:, 4,46, 4) = (/ &
     &1.3083e-03_rb,1.7659e-03_rb,1.4916e-03_rb,1.0659e-03_rb,1.5214e-04_rb/)
      kbo(:, 5,46, 4) = (/ &
     &1.8099e-03_rb,2.3458e-03_rb,1.9309e-03_rb,1.3365e-03_rb,1.5810e-04_rb/)
      kbo(:, 1,47, 4) = (/ &
     &2.8950e-04_rb,5.1677e-04_rb,4.8802e-04_rb,3.9502e-04_rb,1.0398e-04_rb/)
      kbo(:, 2,47, 4) = (/ &
     &4.8670e-04_rb,7.5660e-04_rb,6.8049e-04_rb,5.2378e-04_rb,1.1085e-04_rb/)
      kbo(:, 3,47, 4) = (/ &
     &7.5525e-04_rb,1.0762e-03_rb,9.3123e-04_rb,6.8496e-04_rb,1.1715e-04_rb/)
      kbo(:, 4,47, 4) = (/ &
     &1.0969e-03_rb,1.4777e-03_rb,1.2426e-03_rb,8.8223e-04_rb,1.2295e-04_rb/)
      kbo(:, 5,47, 4) = (/ &
     &1.5354e-03_rb,1.9852e-03_rb,1.6256e-03_rb,1.1178e-03_rb,1.2805e-04_rb/)
      kbo(:, 1,48, 4) = (/ &
     &2.2732e-04_rb,4.1628e-04_rb,3.9316e-04_rb,3.1737e-04_rb,8.2969e-05_rb/)
      kbo(:, 2,48, 4) = (/ &
     &3.9696e-04_rb,6.2251e-04_rb,5.5852e-04_rb,4.2777e-04_rb,8.8908e-05_rb/)
      kbo(:, 3,48, 4) = (/ &
     &6.3462e-04_rb,9.0305e-04_rb,7.7729e-04_rb,5.6732e-04_rb,9.4308e-05_rb/)
      kbo(:, 4,48, 4) = (/ &
     &9.4204e-04_rb,1.2622e-03_rb,1.0539e-03_rb,7.4093e-04_rb,9.9236e-05_rb/)
      kbo(:, 5,48, 4) = (/ &
     &1.3331e-03_rb,1.7143e-03_rb,1.3938e-03_rb,9.4935e-04_rb,1.0368e-04_rb/)
      kbo(:, 1,49, 4) = (/ &
     &1.8300e-04_rb,3.4217e-04_rb,3.2264e-04_rb,2.5877e-04_rb,6.6118e-05_rb/)
      kbo(:, 2,49, 4) = (/ &
     &3.3312e-04_rb,5.2483e-04_rb,4.6829e-04_rb,3.5552e-04_rb,7.1194e-05_rb/)
      kbo(:, 3,49, 4) = (/ &
     &5.4973e-04_rb,7.7839e-04_rb,6.6473e-04_rb,4.7940e-04_rb,7.5813e-05_rb/)
      kbo(:, 4,49, 4) = (/ &
     &8.3663e-04_rb,1.1105e-03_rb,9.1800e-04_rb,6.3658e-04_rb,8.0100e-05_rb/)
      kbo(:, 5,49, 4) = (/ &
     &1.2004e-03_rb,1.5281e-03_rb,1.2307e-03_rb,8.2653e-04_rb,8.3935e-05_rb/)
      kbo(:, 1,50, 4) = (/ &
     &1.4039e-04_rb,2.7151e-04_rb,2.5713e-04_rb,2.0628e-04_rb,5.2648e-05_rb/)
      kbo(:, 2,50, 4) = (/ &
     &2.6610e-04_rb,4.2689e-04_rb,3.8091e-04_rb,2.8831e-04_rb,5.6961e-05_rb/)
      kbo(:, 3,50, 4) = (/ &
     &4.5427e-04_rb,6.4624e-04_rb,5.5025e-04_rb,3.9479e-04_rb,6.0907e-05_rb/)
      kbo(:, 4,50, 4) = (/ &
     &7.0950e-04_rb,9.4070e-04_rb,7.7361e-04_rb,5.3217e-04_rb,6.4571e-05_rb/)
      kbo(:, 5,50, 4) = (/ &
     &1.0374e-03_rb,1.3149e-03_rb,1.0530e-03_rb,7.0038e-04_rb,6.7890e-05_rb/)
      kbo(:, 1,51, 4) = (/ &
     &1.0489e-04_rb,2.1109e-04_rb,2.0141e-04_rb,1.6218e-04_rb,4.1878e-05_rb/)
      kbo(:, 2,51, 4) = (/ &
     &2.0633e-04_rb,3.4016e-04_rb,3.0444e-04_rb,2.3054e-04_rb,4.5538e-05_rb/)
      kbo(:, 3,51, 4) = (/ &
     &3.6536e-04_rb,5.2580e-04_rb,4.4746e-04_rb,3.2051e-04_rb,4.8874e-05_rb/)
      kbo(:, 4,51, 4) = (/ &
     &5.8696e-04_rb,7.8085e-04_rb,6.4023e-04_rb,4.3829e-04_rb,5.1997e-05_rb/)
      kbo(:, 5,51, 4) = (/ &
     &8.7556e-04_rb,1.1091e-03_rb,8.8460e-04_rb,5.8473e-04_rb,5.4858e-05_rb/)
      kbo(:, 1,52, 4) = (/ &
     &7.9793e-05_rb,1.6601e-04_rb,1.5918e-04_rb,1.2853e-04_rb,3.3276e-05_rb/)
      kbo(:, 2,52, 4) = (/ &
     &1.6247e-04_rb,2.7468e-04_rb,2.4631e-04_rb,1.8617e-04_rb,3.6357e-05_rb/)
      kbo(:, 3,52, 4) = (/ &
     &2.9905e-04_rb,4.3474e-04_rb,3.6924e-04_rb,2.6353e-04_rb,3.9196e-05_rb/)
      kbo(:, 4,52, 4) = (/ &
     &4.9530e-04_rb,6.5976e-04_rb,5.3847e-04_rb,3.6618e-04_rb,4.1839e-05_rb/)
      kbo(:, 5,52, 4) = (/ &
     &7.5546e-04_rb,9.5475e-04_rb,7.5721e-04_rb,4.9627e-04_rb,4.4317e-05_rb/)
      kbo(:, 1,53, 4) = (/ &
     &6.2729e-05_rb,1.3307e-04_rb,1.2792e-04_rb,1.0321e-04_rb,2.6401e-05_rb/)
      kbo(:, 2,53, 4) = (/ &
     &1.3138e-04_rb,2.2658e-04_rb,2.0324e-04_rb,1.5289e-04_rb,2.9006e-05_rb/)
      kbo(:, 3,53, 4) = (/ &
     &2.5160e-04_rb,3.6877e-04_rb,3.1205e-04_rb,2.2094e-04_rb,3.1410e-05_rb/)
      kbo(:, 4,53, 4) = (/ &
     &4.3128e-04_rb,5.7326e-04_rb,4.6479e-04_rb,3.1290e-04_rb,3.3663e-05_rb/)
      kbo(:, 5,53, 4) = (/ &
     &6.7436e-04_rb,8.4743e-04_rb,6.6688e-04_rb,4.3200e-04_rb,3.5790e-05_rb/)
      kbo(:, 1,54, 4) = (/ &
     &4.3536e-05_rb,9.6993e-05_rb,9.4650e-05_rb,7.7465e-05_rb,2.0921e-05_rb/)
      kbo(:, 2,54, 4) = (/ &
     &9.2752e-05_rb,1.6814e-04_rb,1.5268e-04_rb,1.1626e-04_rb,2.3103e-05_rb/)
      kbo(:, 3,54, 4) = (/ &
     &1.8378e-04_rb,2.7919e-04_rb,2.3835e-04_rb,1.7016e-04_rb,2.5121e-05_rb/)
      kbo(:, 4,54, 4) = (/ &
     &3.2645e-04_rb,4.4256e-04_rb,3.6033e-04_rb,2.4382e-04_rb,2.7012e-05_rb/)
      kbo(:, 5,54, 4) = (/ &
     &5.2349e-04_rb,6.6571e-04_rb,5.2497e-04_rb,3.4104e-04_rb,2.8788e-05_rb/)
      kbo(:, 1,55, 4) = (/ &
     &2.7968e-05_rb,6.6619e-05_rb,6.6506e-05_rb,5.5712e-05_rb,1.6550e-05_rb/)
      kbo(:, 2,55, 4) = (/ &
     &5.9858e-05_rb,1.1651e-04_rb,1.0803e-04_rb,8.4126e-05_rb,1.8363e-05_rb/)
      kbo(:, 3,55, 4) = (/ &
     &1.2216e-04_rb,1.9620e-04_rb,1.7047e-04_rb,1.2407e-04_rb,2.0057e-05_rb/)
      kbo(:, 4,55, 4) = (/ &
     &2.2490e-04_rb,3.1581e-04_rb,2.6032e-04_rb,1.7892e-04_rb,2.1634e-05_rb/)
      kbo(:, 5,55, 4) = (/ &
     &3.7032e-04_rb,4.8266e-04_rb,3.8398e-04_rb,2.5251e-04_rb,2.3108e-05_rb/)
      kbo(:, 1,56, 4) = (/ &
     &1.8187e-05_rb,4.5780e-05_rb,4.6700e-05_rb,4.0035e-05_rb,1.3073e-05_rb/)
      kbo(:, 2,56, 4) = (/ &
     &3.8391e-05_rb,8.0472e-05_rb,7.6243e-05_rb,6.0757e-05_rb,1.4582e-05_rb/)
      kbo(:, 3,56, 4) = (/ &
     &8.0424e-05_rb,1.3728e-04_rb,1.2154e-04_rb,9.0284e-05_rb,1.6006e-05_rb/)
      kbo(:, 4,56, 4) = (/ &
     &1.5350e-04_rb,2.2433e-04_rb,1.8750e-04_rb,1.3109e-04_rb,1.7305e-05_rb/)
      kbo(:, 5,56, 4) = (/ &
     &2.6031e-04_rb,3.4853e-04_rb,2.7994e-04_rb,1.8653e-04_rb,1.8526e-05_rb/)
      kbo(:, 1,57, 4) = (/ &
     &1.2069e-05_rb,3.1511e-05_rb,3.2815e-05_rb,2.8783e-05_rb,1.0310e-05_rb/)
      kbo(:, 2,57, 4) = (/ &
     &2.4551e-05_rb,5.5416e-05_rb,5.3688e-05_rb,4.3795e-05_rb,1.1563e-05_rb/)
      kbo(:, 3,57, 4) = (/ &
     &5.2372e-05_rb,9.5634e-05_rb,8.6379e-05_rb,6.5547e-05_rb,1.2748e-05_rb/)
      kbo(:, 4,57, 4) = (/ &
     &1.0355e-04_rb,1.5851e-04_rb,1.3458e-04_rb,9.5855e-05_rb,1.3838e-05_rb/)
      kbo(:, 5,57, 4) = (/ &
     &1.8156e-04_rb,2.5040e-04_rb,2.0330e-04_rb,1.3740e-04_rb,1.4854e-05_rb/)
      kbo(:, 1,58, 4) = (/ &
     &2.1923e-06_rb,7.1667e-06_rb,9.7990e-06_rb,1.2334e-05_rb,8.1331e-06_rb/)
      kbo(:, 2,58, 4) = (/ &
     &4.2014e-06_rb,1.2514e-05_rb,1.5980e-05_rb,1.8735e-05_rb,9.1661e-06_rb/)
      kbo(:, 3,58, 4) = (/ &
     &9.0349e-06_rb,2.1751e-05_rb,2.5847e-05_rb,2.8162e-05_rb,1.0154e-05_rb/)
      kbo(:, 4,58, 4) = (/ &
     &1.8408e-05_rb,3.6461e-05_rb,4.0575e-05_rb,4.1376e-05_rb,1.1069e-05_rb/)
      kbo(:, 5,58, 4) = (/ &
     &3.3398e-05_rb,5.8421e-05_rb,6.1871e-05_rb,5.9629e-05_rb,1.1910e-05_rb/)
      kbo(:, 1,59, 4) = (/ &
     &1.9895e-06_rb,7.0146e-06_rb,9.2666e-06_rb,1.0819e-05_rb,6.5530e-06_rb/)
      kbo(:, 2,59, 4) = (/ &
     &3.7103e-06_rb,1.1985e-05_rb,1.4798e-05_rb,1.6119e-05_rb,7.3922e-06_rb/)
      kbo(:, 3,59, 4) = (/ &
     &7.9795e-06_rb,2.0481e-05_rb,2.3504e-05_rb,2.3808e-05_rb,8.1931e-06_rb/)
      kbo(:, 4,59, 4) = (/ &
     &1.6446e-05_rb,3.3891e-05_rb,3.6329e-05_rb,3.4426e-05_rb,8.9453e-06_rb/)
      kbo(:, 5,59, 4) = (/ &
     &3.0202e-05_rb,5.3874e-05_rb,5.4799e-05_rb,4.8960e-05_rb,9.6374e-06_rb/)
      kbo(:, 1,13, 5) = (/ &
     &4.3236e-01_rb,4.3213e-01_rb,3.9701e-01_rb,3.2546e-01_rb,1.1197e-01_rb/)
      kbo(:, 2,13, 5) = (/ &
     &6.6372e-01_rb,6.1379e-01_rb,5.3578e-01_rb,4.1378e-01_rb,1.1749e-01_rb/)
      kbo(:, 3,13, 5) = (/ &
     &9.6012e-01_rb,8.4543e-01_rb,7.0690e-01_rb,5.2039e-01_rb,1.2229e-01_rb/)
      kbo(:, 4,13, 5) = (/ &
     &1.3340e+00_rb,1.1343e+00_rb,9.1491e-01_rb,6.4651e-01_rb,1.2717e-01_rb/)
      kbo(:, 5,13, 5) = (/ &
     &1.8059e+00_rb,1.4945e+00_rb,1.1687e+00_rb,7.9501e-01_rb,1.3200e-01_rb/)
      kbo(:, 1,14, 5) = (/ &
     &2.3576e-01_rb,2.7360e-01_rb,2.7141e-01_rb,2.4039e-01_rb,9.8792e-02_rb/)
      kbo(:, 2,14, 5) = (/ &
     &3.5903e-01_rb,3.7743e-01_rb,3.5638e-01_rb,2.9754e-01_rb,1.0359e-01_rb/)
      kbo(:, 3,14, 5) = (/ &
     &5.1761e-01_rb,5.0746e-01_rb,4.5981e-01_rb,3.6575e-01_rb,1.0816e-01_rb/)
      kbo(:, 4,14, 5) = (/ &
     &7.1963e-01_rb,6.6997e-01_rb,5.8434e-01_rb,4.4555e-01_rb,1.1272e-01_rb/)
      kbo(:, 5,14, 5) = (/ &
     &9.7673e-01_rb,8.7084e-01_rb,7.3356e-01_rb,5.3881e-01_rb,1.1732e-01_rb/)
      kbo(:, 1,15, 5) = (/ &
     &1.3110e-01_rb,1.8103e-01_rb,1.9281e-01_rb,1.8264e-01_rb,8.6582e-02_rb/)
      kbo(:, 2,15, 5) = (/ &
     &1.9817e-01_rb,2.4413e-01_rb,2.4684e-01_rb,2.2172e-01_rb,9.0925e-02_rb/)
      kbo(:, 3,15, 5) = (/ &
     &2.8519e-01_rb,3.2198e-01_rb,3.1178e-01_rb,2.6742e-01_rb,9.5231e-02_rb/)
      kbo(:, 4,15, 5) = (/ &
     &3.9693e-01_rb,4.1797e-01_rb,3.8987e-01_rb,3.1982e-01_rb,9.9470e-02_rb/)
      kbo(:, 5,15, 5) = (/ &
     &5.3979e-01_rb,5.3546e-01_rb,4.8283e-01_rb,3.8058e-01_rb,1.0367e-01_rb/)
      kbo(:, 1,16, 5) = (/ &
     &8.6780e-02_rb,1.3641e-01_rb,1.5186e-01_rb,1.4818e-01_rb,7.5514e-02_rb/)
      kbo(:, 2,16, 5) = (/ &
     &1.3047e-01_rb,1.8113e-01_rb,1.9135e-01_rb,1.7835e-01_rb,7.9533e-02_rb/)
      kbo(:, 3,16, 5) = (/ &
     &1.8753e-01_rb,2.3639e-01_rb,2.3875e-01_rb,2.1304e-01_rb,8.3481e-02_rb/)
      kbo(:, 4,16, 5) = (/ &
     &2.6140e-01_rb,3.0428e-01_rb,2.9551e-01_rb,2.5256e-01_rb,8.7293e-02_rb/)
      kbo(:, 5,16, 5) = (/ &
     &3.5574e-01_rb,3.8652e-01_rb,3.6267e-01_rb,2.9793e-01_rb,9.1094e-02_rb/)
      kbo(:, 1,17, 5) = (/ &
     &5.9400e-02_rb,1.0544e-01_rb,1.2175e-01_rb,1.2140e-01_rb,6.5571e-02_rb/)
      kbo(:, 2,17, 5) = (/ &
     &8.8982e-02_rb,1.3824e-01_rb,1.5175e-01_rb,1.4491e-01_rb,6.9197e-02_rb/)
      kbo(:, 3,17, 5) = (/ &
     &1.2788e-01_rb,1.7890e-01_rb,1.8755e-01_rb,1.7185e-01_rb,7.2708e-02_rb/)
      kbo(:, 4,17, 5) = (/ &
     &1.7835e-01_rb,2.2856e-01_rb,2.3024e-01_rb,2.0241e-01_rb,7.6161e-02_rb/)
      kbo(:, 5,17, 5) = (/ &
     &2.4297e-01_rb,2.8834e-01_rb,2.8043e-01_rb,2.3722e-01_rb,7.9603e-02_rb/)
      kbo(:, 1,18, 5) = (/ &
     &4.1651e-02_rb,8.2911e-02_rb,9.8455e-02_rb,9.9593e-02_rb,5.6662e-02_rb/)
      kbo(:, 2,18, 5) = (/ &
     &6.2251e-02_rb,1.0765e-01_rb,1.2173e-01_rb,1.1843e-01_rb,5.9831e-02_rb/)
      kbo(:, 3,18, 5) = (/ &
     &8.9419e-02_rb,1.3828e-01_rb,1.4944e-01_rb,1.3970e-01_rb,6.2923e-02_rb/)
      kbo(:, 4,18, 5) = (/ &
     &1.2470e-01_rb,1.7541e-01_rb,1.8216e-01_rb,1.6372e-01_rb,6.6077e-02_rb/)
      kbo(:, 5,18, 5) = (/ &
     &1.6992e-01_rb,2.1986e-01_rb,2.2047e-01_rb,1.9095e-01_rb,6.9212e-02_rb/)
      kbo(:, 1,19, 5) = (/ &
     &2.8717e-02_rb,6.4742e-02_rb,7.8819e-02_rb,8.0895e-02_rb,4.8664e-02_rb/)
      kbo(:, 2,19, 5) = (/ &
     &4.2849e-02_rb,8.3244e-02_rb,9.6815e-02_rb,9.5751e-02_rb,5.1433e-02_rb/)
      kbo(:, 3,19, 5) = (/ &
     &6.1454e-02_rb,1.0605e-01_rb,1.1801e-01_rb,1.1248e-01_rb,5.4215e-02_rb/)
      kbo(:, 4,19, 5) = (/ &
     &8.5699e-02_rb,1.3356e-01_rb,1.4292e-01_rb,1.3115e-01_rb,5.7031e-02_rb/)
      kbo(:, 5,19, 5) = (/ &
     &1.1681e-01_rb,1.6622e-01_rb,1.7182e-01_rb,1.5204e-01_rb,5.9790e-02_rb/)
      kbo(:, 1,20, 5) = (/ &
     &2.1632e-02_rb,5.2943e-02_rb,6.5235e-02_rb,6.7363e-02_rb,4.1610e-02_rb/)
      kbo(:, 2,20, 5) = (/ &
     &3.2163e-02_rb,6.7696e-02_rb,7.9852e-02_rb,7.9447e-02_rb,4.4062e-02_rb/)
      kbo(:, 3,20, 5) = (/ &
     &4.6057e-02_rb,8.5815e-02_rb,9.6980e-02_rb,9.3049e-02_rb,4.6553e-02_rb/)
      kbo(:, 4,20, 5) = (/ &
     &6.4136e-02_rb,1.0754e-01_rb,1.1694e-01_rb,1.0820e-01_rb,4.9013e-02_rb/)
      kbo(:, 5,20, 5) = (/ &
     &8.7379e-02_rb,1.3322e-01_rb,1.3995e-01_rb,1.2502e-01_rb,5.1455e-02_rb/)
      kbo(:, 1,21, 5) = (/ &
     &1.6823e-02_rb,4.3877e-02_rb,5.4527e-02_rb,5.6402e-02_rb,3.5455e-02_rb/)
      kbo(:, 2,21, 5) = (/ &
     &2.4936e-02_rb,5.5953e-02_rb,6.6511e-02_rb,6.6307e-02_rb,3.7602e-02_rb/)
      kbo(:, 3,21, 5) = (/ &
     &3.5634e-02_rb,7.0642e-02_rb,8.0513e-02_rb,7.7454e-02_rb,3.9786e-02_rb/)
      kbo(:, 4,21, 5) = (/ &
     &4.9531e-02_rb,8.8177e-02_rb,9.6745e-02_rb,8.9935e-02_rb,4.1920e-02_rb/)
      kbo(:, 5,21, 5) = (/ &
     &6.7455e-02_rb,1.0884e-01_rb,1.1539e-01_rb,1.0375e-01_rb,4.4030e-02_rb/)
      kbo(:, 1,22, 5) = (/ &
     &1.3789e-02_rb,3.7331e-02_rb,4.6413e-02_rb,4.7878e-02_rb,3.0220e-02_rb/)
      kbo(:, 2,22, 5) = (/ &
     &2.0294e-02_rb,4.7459e-02_rb,5.6444e-02_rb,5.6212e-02_rb,3.2091e-02_rb/)
      kbo(:, 3,22, 5) = (/ &
     &2.8887e-02_rb,5.9685e-02_rb,6.8142e-02_rb,6.5522e-02_rb,3.3949e-02_rb/)
      kbo(:, 4,22, 5) = (/ &
     &4.0024e-02_rb,7.4193e-02_rb,8.1621e-02_rb,7.5940e-02_rb,3.5793e-02_rb/)
      kbo(:, 5,22, 5) = (/ &
     &5.4487e-02_rb,9.1298e-02_rb,9.7079e-02_rb,8.7446e-02_rb,3.7515e-02_rb/)
      kbo(:, 1,23, 5) = (/ &
     &1.1373e-02_rb,3.1797e-02_rb,3.9445e-02_rb,4.0631e-02_rb,2.5676e-02_rb/)
      kbo(:, 2,23, 5) = (/ &
     &1.6630e-02_rb,4.0276e-02_rb,4.7874e-02_rb,4.7621e-02_rb,2.7276e-02_rb/)
      kbo(:, 3,23, 5) = (/ &
     &2.3572e-02_rb,5.0445e-02_rb,5.7632e-02_rb,5.5405e-02_rb,2.8857e-02_rb/)
      kbo(:, 4,23, 5) = (/ &
     &3.2590e-02_rb,6.2506e-02_rb,6.8883e-02_rb,6.4105e-02_rb,3.0360e-02_rb/)
      kbo(:, 5,23, 5) = (/ &
     &4.4383e-02_rb,7.6716e-02_rb,8.1703e-02_rb,7.3646e-02_rb,3.1632e-02_rb/)
      kbo(:, 1,24, 5) = (/ &
     &9.3521e-03_rb,2.6988e-02_rb,3.3405e-02_rb,3.4382e-02_rb,2.1758e-02_rb/)
      kbo(:, 2,24, 5) = (/ &
     &1.3592e-02_rb,3.4040e-02_rb,4.0452e-02_rb,4.0217e-02_rb,2.3106e-02_rb/)
      kbo(:, 3,24, 5) = (/ &
     &1.9190e-02_rb,4.2486e-02_rb,4.8570e-02_rb,4.6701e-02_rb,2.4399e-02_rb/)
      kbo(:, 4,24, 5) = (/ &
     &2.6527e-02_rb,5.2454e-02_rb,5.7886e-02_rb,5.3878e-02_rb,2.5528e-02_rb/)
      kbo(:, 5,24, 5) = (/ &
     &3.6127e-02_rb,6.4240e-02_rb,6.8480e-02_rb,6.1740e-02_rb,2.6418e-02_rb/)
      kbo(:, 1,25, 5) = (/ &
     &7.8143e-03_rb,2.3046e-02_rb,2.8423e-02_rb,2.9180e-02_rb,1.8391e-02_rb/)
      kbo(:, 2,25, 5) = (/ &
     &1.1292e-02_rb,2.8965e-02_rb,3.4350e-02_rb,3.4059e-02_rb,1.9498e-02_rb/)
      kbo(:, 3,25, 5) = (/ &
     &1.5906e-02_rb,3.6033e-02_rb,4.1158e-02_rb,3.9460e-02_rb,2.0484e-02_rb/)
      kbo(:, 4,25, 5) = (/ &
     &2.2015e-02_rb,4.4387e-02_rb,4.8915e-02_rb,4.5425e-02_rb,2.1291e-02_rb/)
      kbo(:, 5,25, 5) = (/ &
     &2.9929e-02_rb,5.4292e-02_rb,5.7748e-02_rb,5.1892e-02_rb,2.1939e-02_rb/)
      kbo(:, 1,26, 5) = (/ &
     &6.6841e-03_rb,1.9882e-02_rb,2.4386e-02_rb,2.4902e-02_rb,1.5507e-02_rb/)
      kbo(:, 2,26, 5) = (/ &
     &9.6187e-03_rb,2.4917e-02_rb,2.9404e-02_rb,2.8998e-02_rb,1.6368e-02_rb/)
      kbo(:, 3,26, 5) = (/ &
     &1.3541e-02_rb,3.0916e-02_rb,3.5147e-02_rb,3.3522e-02_rb,1.7088e-02_rb/)
      kbo(:, 4,26, 5) = (/ &
     &1.8734e-02_rb,3.8045e-02_rb,4.1676e-02_rb,3.8461e-02_rb,1.7679e-02_rb/)
      kbo(:, 5,26, 5) = (/ &
     &2.5382e-02_rb,4.6534e-02_rb,4.9183e-02_rb,4.3795e-02_rb,1.8164e-02_rb/)
      kbo(:, 1,27, 5) = (/ &
     &5.9464e-03_rb,1.7461e-02_rb,2.1207e-02_rb,2.1457e-02_rb,1.3019e-02_rb/)
      kbo(:, 2,27, 5) = (/ &
     &8.5411e-03_rb,2.1855e-02_rb,2.5534e-02_rb,2.4945e-02_rb,1.3658e-02_rb/)
      kbo(:, 3,27, 5) = (/ &
     &1.2012e-02_rb,2.7098e-02_rb,3.0472e-02_rb,2.8773e-02_rb,1.4192e-02_rb/)
      kbo(:, 4,27, 5) = (/ &
     &1.6580e-02_rb,3.3382e-02_rb,3.6146e-02_rb,3.2904e-02_rb,1.4638e-02_rb/)
      kbo(:, 5,27, 5) = (/ &
     &2.2392e-02_rb,4.0890e-02_rb,4.2671e-02_rb,3.7399e-02_rb,1.5005e-02_rb/)
      kbo(:, 1,28, 5) = (/ &
     &5.4489e-03_rb,1.5531e-02_rb,1.8614e-02_rb,1.8613e-02_rb,1.0873e-02_rb/)
      kbo(:, 2,28, 5) = (/ &
     &7.8148e-03_rb,1.9440e-02_rb,2.2401e-02_rb,2.1604e-02_rb,1.1349e-02_rb/)
      kbo(:, 3,28, 5) = (/ &
     &1.0969e-02_rb,2.4138e-02_rb,2.6729e-02_rb,2.4848e-02_rb,1.1757e-02_rb/)
      kbo(:, 4,28, 5) = (/ &
     &1.5087e-02_rb,2.9815e-02_rb,3.1728e-02_rb,2.8368e-02_rb,1.2101e-02_rb/)
      kbo(:, 5,28, 5) = (/ &
     &2.0299e-02_rb,3.6555e-02_rb,3.7502e-02_rb,3.2280e-02_rb,1.2378e-02_rb/)
      kbo(:, 1,29, 5) = (/ &
     &5.3270e-03_rb,1.4285e-02_rb,1.6775e-02_rb,1.6453e-02_rb,9.0474e-03_rb/)
      kbo(:, 2,29, 5) = (/ &
     &7.6276e-03_rb,1.7927e-02_rb,2.0207e-02_rb,1.9083e-02_rb,9.4132e-03_rb/)
      kbo(:, 3,29, 5) = (/ &
     &1.0673e-02_rb,2.2373e-02_rb,2.4172e-02_rb,2.1940e-02_rb,9.7284e-03_rb/)
      kbo(:, 4,29, 5) = (/ &
     &1.4614e-02_rb,2.7730e-02_rb,2.8795e-02_rb,2.5116e-02_rb,9.9926e-03_rb/)
      kbo(:, 5,29, 5) = (/ &
     &1.9579e-02_rb,3.4081e-02_rb,3.4110e-02_rb,2.8684e-02_rb,1.0207e-02_rb/)
      kbo(:, 1,30, 5) = (/ &
     &5.3535e-03_rb,1.3353e-02_rb,1.5300e-02_rb,1.4666e-02_rb,7.5129e-03_rb/)
      kbo(:, 2,30, 5) = (/ &
     &7.6380e-03_rb,1.6853e-02_rb,1.8473e-02_rb,1.7010e-02_rb,7.7968e-03_rb/)
      kbo(:, 3,30, 5) = (/ &
     &1.0647e-02_rb,2.1132e-02_rb,2.2202e-02_rb,1.9612e-02_rb,8.0434e-03_rb/)
      kbo(:, 4,30, 5) = (/ &
     &1.4510e-02_rb,2.6289e-02_rb,2.6550e-02_rb,2.2553e-02_rb,8.2450e-03_rb/)
      kbo(:, 5,30, 5) = (/ &
     &1.9367e-02_rb,3.2371e-02_rb,3.1546e-02_rb,2.5868e-02_rb,8.4082e-03_rb/)
      kbo(:, 1,31, 5) = (/ &
     &5.6746e-03_rb,1.2936e-02_rb,1.4343e-02_rb,1.3343e-02_rb,6.2303e-03_rb/)
      kbo(:, 2,31, 5) = (/ &
     &8.0577e-03_rb,1.6447e-02_rb,1.7425e-02_rb,1.5532e-02_rb,6.4556e-03_rb/)
      kbo(:, 3,31, 5) = (/ &
     &1.1174e-02_rb,2.0739e-02_rb,2.1078e-02_rb,1.8020e-02_rb,6.6441e-03_rb/)
      kbo(:, 4,31, 5) = (/ &
     &1.5165e-02_rb,2.5916e-02_rb,2.5346e-02_rb,2.0861e-02_rb,6.8034e-03_rb/)
      kbo(:, 5,31, 5) = (/ &
     &2.0188e-02_rb,3.1999e-02_rb,3.0259e-02_rb,2.4055e-02_rb,6.9268e-03_rb/)
      kbo(:, 1,32, 5) = (/ &
     &6.0895e-03_rb,1.2732e-02_rb,1.3622e-02_rb,1.2258e-02_rb,5.1656e-03_rb/)
      kbo(:, 2,32, 5) = (/ &
     &8.5996e-03_rb,1.6298e-02_rb,1.6681e-02_rb,1.4366e-02_rb,5.3392e-03_rb/)
      kbo(:, 3,32, 5) = (/ &
     &1.1871e-02_rb,2.0666e-02_rb,2.0302e-02_rb,1.6787e-02_rb,5.4899e-03_rb/)
      kbo(:, 4,32, 5) = (/ &
     &1.6059e-02_rb,2.5901e-02_rb,2.4547e-02_rb,1.9547e-02_rb,5.6119e-03_rb/)
      kbo(:, 5,32, 5) = (/ &
     &2.1313e-02_rb,3.2082e-02_rb,2.9454e-02_rb,2.2664e-02_rb,5.7070e-03_rb/)
      kbo(:, 1,33, 5) = (/ &
     &6.6515e-03_rb,1.2796e-02_rb,1.3188e-02_rb,1.1442e-02_rb,4.2781e-03_rb/)
      kbo(:, 2,33, 5) = (/ &
     &9.3384e-03_rb,1.6485e-02_rb,1.6266e-02_rb,1.3517e-02_rb,4.4167e-03_rb/)
      kbo(:, 3,33, 5) = (/ &
     &1.2846e-02_rb,2.0999e-02_rb,1.9943e-02_rb,1.5910e-02_rb,4.5341e-03_rb/)
      kbo(:, 4,33, 5) = (/ &
     &1.7327e-02_rb,2.6399e-02_rb,2.4247e-02_rb,1.8651e-02_rb,4.6303e-03_rb/)
      kbo(:, 5,33, 5) = (/ &
     &2.2895e-02_rb,3.2829e-02_rb,2.9267e-02_rb,2.1750e-02_rb,4.7019e-03_rb/)
      kbo(:, 1,34, 5) = (/ &
     &6.9862e-03_rb,1.2631e-02_rb,1.2600e-02_rb,1.0591e-02_rb,3.5376e-03_rb/)
      kbo(:, 2,34, 5) = (/ &
     &9.7726e-03_rb,1.6353e-02_rb,1.5653e-02_rb,1.2600e-02_rb,3.6473e-03_rb/)
      kbo(:, 3,34, 5) = (/ &
     &1.3409e-02_rb,2.0887e-02_rb,1.9295e-02_rb,1.4929e-02_rb,3.7406e-03_rb/)
      kbo(:, 4,34, 5) = (/ &
     &1.8037e-02_rb,2.6346e-02_rb,2.3581e-02_rb,1.7591e-02_rb,3.8148e-03_rb/)
      kbo(:, 5,34, 5) = (/ &
     &2.3704e-02_rb,3.2885e-02_rb,2.8616e-02_rb,2.0638e-02_rb,3.8707e-03_rb/)
      kbo(:, 1,35, 5) = (/ &
     &7.0977e-03_rb,1.2235e-02_rb,1.1878e-02_rb,9.7122e-03_rb,2.9156e-03_rb/)
      kbo(:, 2,35, 5) = (/ &
     &9.9249e-03_rb,1.5915e-02_rb,1.4862e-02_rb,1.1642e-02_rb,3.0042e-03_rb/)
      kbo(:, 3,35, 5) = (/ &
     &1.3615e-02_rb,2.0413e-02_rb,1.8425e-02_rb,1.3882e-02_rb,3.0791e-03_rb/)
      kbo(:, 4,35, 5) = (/ &
     &1.8271e-02_rb,2.5881e-02_rb,2.2655e-02_rb,1.6461e-02_rb,3.1387e-03_rb/)
      kbo(:, 5,35, 5) = (/ &
     &2.3900e-02_rb,3.2414e-02_rb,2.7651e-02_rb,1.9450e-02_rb,3.1837e-03_rb/)
      kbo(:, 1,36, 5) = (/ &
     &6.9060e-03_rb,1.1498e-02_rb,1.0932e-02_rb,8.7493e-03_rb,2.3952e-03_rb/)
      kbo(:, 2,36, 5) = (/ &
     &9.6735e-03_rb,1.5042e-02_rb,1.3779e-02_rb,1.0573e-02_rb,2.4672e-03_rb/)
      kbo(:, 3,36, 5) = (/ &
     &1.3291e-02_rb,1.9415e-02_rb,1.7205e-02_rb,1.2691e-02_rb,2.5290e-03_rb/)
      kbo(:, 4,36, 5) = (/ &
     &1.7820e-02_rb,2.4753e-02_rb,2.1299e-02_rb,1.5151e-02_rb,2.5791e-03_rb/)
      kbo(:, 5,36, 5) = (/ &
     &2.3253e-02_rb,3.1113e-02_rb,2.6141e-02_rb,1.8030e-02_rb,2.6175e-03_rb/)
      kbo(:, 1,37, 5) = (/ &
     &6.2139e-03_rb,1.0192e-02_rb,9.5802e-03_rb,7.5798e-03_rb,1.9567e-03_rb/)
      kbo(:, 2,37, 5) = (/ &
     &8.7456e-03_rb,1.3428e-02_rb,1.2164e-02_rb,9.2244e-03_rb,2.0181e-03_rb/)
      kbo(:, 3,37, 5) = (/ &
     &1.2068e-02_rb,1.7456e-02_rb,1.5302e-02_rb,1.1147e-02_rb,2.0706e-03_rb/)
      kbo(:, 4,37, 5) = (/ &
     &1.6229e-02_rb,2.2411e-02_rb,1.9089e-02_rb,1.3408e-02_rb,2.1134e-03_rb/)
      kbo(:, 5,37, 5) = (/ &
     &2.1208e-02_rb,2.8314e-02_rb,2.3560e-02_rb,1.6063e-02_rb,2.1467e-03_rb/)
      kbo(:, 1,38, 5) = (/ &
     &5.6546e-03_rb,9.1081e-03_rb,8.4597e-03_rb,6.6101e-03_rb,1.5995e-03_rb/)
      kbo(:, 2,38, 5) = (/ &
     &7.9966e-03_rb,1.2093e-02_rb,1.0832e-02_rb,8.1085e-03_rb,1.6506e-03_rb/)
      kbo(:, 3,38, 5) = (/ &
     &1.1084e-02_rb,1.5851e-02_rb,1.3740e-02_rb,9.8767e-03_rb,1.6948e-03_rb/)
      kbo(:, 4,38, 5) = (/ &
     &1.4944e-02_rb,2.0494e-02_rb,1.7272e-02_rb,1.1975e-02_rb,1.7322e-03_rb/)
      kbo(:, 5,38, 5) = (/ &
     &1.9571e-02_rb,2.6031e-02_rb,2.1445e-02_rb,1.4447e-02_rb,1.7614e-03_rb/)
      kbo(:, 1,39, 5) = (/ &
     &5.2606e-03_rb,8.2861e-03_rb,7.5918e-03_rb,5.8448e-03_rb,1.3077e-03_rb/)
      kbo(:, 2,39, 5) = (/ &
     &7.4758e-03_rb,1.1098e-02_rb,9.8108e-03_rb,7.2343e-03_rb,1.3505e-03_rb/)
      kbo(:, 3,39, 5) = (/ &
     &1.0407e-02_rb,1.4668e-02_rb,1.2553e-02_rb,8.8880e-03_rb,1.3888e-03_rb/)
      kbo(:, 4,39, 5) = (/ &
     &1.4072e-02_rb,1.9102e-02_rb,1.5911e-02_rb,1.0872e-02_rb,1.4211e-03_rb/)
      kbo(:, 5,39, 5) = (/ &
     &1.8472e-02_rb,2.4391e-02_rb,1.9883e-02_rb,1.3211e-02_rb,1.4470e-03_rb/)
      kbo(:, 1,40, 5) = (/ &
     &4.4239e-03_rb,6.9683e-03_rb,6.3619e-03_rb,4.8825e-03_rb,1.0643e-03_rb/)
      kbo(:, 2,40, 5) = (/ &
     &6.3289e-03_rb,9.4143e-03_rb,8.2870e-03_rb,6.0882e-03_rb,1.1007e-03_rb/)
      kbo(:, 3,40, 5) = (/ &
     &8.8723e-03_rb,1.2550e-02_rb,1.0689e-02_rb,7.5344e-03_rb,1.1340e-03_rb/)
      kbo(:, 4,40, 5) = (/ &
     &1.2075e-02_rb,1.6469e-02_rb,1.3655e-02_rb,9.2842e-03_rb,1.1620e-03_rb/)
      kbo(:, 5,40, 5) = (/ &
     &1.5933e-02_rb,2.1174e-02_rb,1.7180e-02_rb,1.1354e-02_rb,1.1848e-03_rb/)
      kbo(:, 1,41, 5) = (/ &
     &3.6867e-03_rb,5.8128e-03_rb,5.2949e-03_rb,4.0553e-03_rb,8.6561e-04_rb/)
      kbo(:, 2,41, 5) = (/ &
     &5.3143e-03_rb,7.9271e-03_rb,6.9551e-03_rb,5.0947e-03_rb,8.9710e-04_rb/)
      kbo(:, 3,41, 5) = (/ &
     &7.5044e-03_rb,1.0658e-02_rb,9.0449e-03_rb,6.3522e-03_rb,9.2517e-04_rb/)
      kbo(:, 4,41, 5) = (/ &
     &1.0281e-02_rb,1.4099e-02_rb,1.1643e-02_rb,7.8856e-03_rb,9.4938e-04_rb/)
      kbo(:, 5,41, 5) = (/ &
     &1.3649e-02_rb,1.8248e-02_rb,1.4748e-02_rb,9.7057e-03_rb,9.6963e-04_rb/)
      kbo(:, 1,42, 5) = (/ &
     &3.0985e-03_rb,4.8781e-03_rb,4.4299e-03_rb,3.3820e-03_rb,7.0400e-04_rb/)
      kbo(:, 2,42, 5) = (/ &
     &4.5009e-03_rb,6.7165e-03_rb,5.8698e-03_rb,4.2829e-03_rb,7.3071e-04_rb/)
      kbo(:, 3,42, 5) = (/ &
     &6.4001e-03_rb,9.1103e-03_rb,7.7007e-03_rb,5.3843e-03_rb,7.5479e-04_rb/)
      kbo(:, 4,42, 5) = (/ &
     &8.8324e-03_rb,1.2154e-02_rb,9.9941e-03_rb,6.7362e-03_rb,7.7572e-04_rb/)
      kbo(:, 5,42, 5) = (/ &
     &1.1799e-02_rb,1.5850e-02_rb,1.2750e-02_rb,8.3487e-03_rb,7.9315e-04_rb/)
      kbo(:, 1,43, 5) = (/ &
     &2.5629e-03_rb,4.0329e-03_rb,3.6588e-03_rb,2.7901e-03_rb,5.7114e-04_rb/)
      kbo(:, 2,43, 5) = (/ &
     &3.7578e-03_rb,5.6142e-03_rb,4.8977e-03_rb,3.5662e-03_rb,5.9379e-04_rb/)
      kbo(:, 3,43, 5) = (/ &
     &5.3881e-03_rb,7.6915e-03_rb,6.4844e-03_rb,4.5229e-03_rb,6.1455e-04_rb/)
      kbo(:, 4,43, 5) = (/ &
     &7.5091e-03_rb,1.0367e-02_rb,8.4990e-03_rb,5.7086e-03_rb,6.3259e-04_rb/)
      kbo(:, 5,43, 5) = (/ &
     &1.0110e-02_rb,1.3645e-02_rb,1.0937e-02_rb,7.1314e-03_rb,6.4809e-04_rb/)
      kbo(:, 1,44, 5) = (/ &
     &2.1195e-03_rb,3.3263e-03_rb,3.0153e-03_rb,2.2974e-03_rb,4.6259e-04_rb/)
      kbo(:, 2,44, 5) = (/ &
     &3.1405e-03_rb,4.6890e-03_rb,4.0834e-03_rb,2.9671e-03_rb,4.8211e-04_rb/)
      kbo(:, 3,44, 5) = (/ &
     &4.5449e-03_rb,6.4956e-03_rb,5.4625e-03_rb,3.8015e-03_rb,4.9986e-04_rb/)
      kbo(:, 4,44, 5) = (/ &
     &6.4001e-03_rb,8.8502e-03_rb,7.2321e-03_rb,4.8421e-03_rb,5.1551e-04_rb/)
      kbo(:, 5,44, 5) = (/ &
     &8.6977e-03_rb,1.1779e-02_rb,9.4064e-03_rb,6.1062e-03_rb,5.2910e-04_rb/)
      kbo(:, 1,45, 5) = (/ &
     &1.7814e-03_rb,2.7759e-03_rb,2.5092e-03_rb,1.9068e-03_rb,3.7475e-04_rb/)
      kbo(:, 2,45, 5) = (/ &
     &2.6678e-03_rb,3.9635e-03_rb,3.4407e-03_rb,2.4906e-03_rb,3.9120e-04_rb/)
      kbo(:, 3,45, 5) = (/ &
     &3.8999e-03_rb,5.5595e-03_rb,4.6558e-03_rb,3.2267e-03_rb,4.0640e-04_rb/)
      kbo(:, 4,45, 5) = (/ &
     &5.5475e-03_rb,7.6646e-03_rb,6.2331e-03_rb,4.1536e-03_rb,4.1988e-04_rb/)
      kbo(:, 5,45, 5) = (/ &
     &7.6146e-03_rb,1.0314e-02_rb,8.1967e-03_rb,5.2910e-03_rb,4.3189e-04_rb/)
      kbo(:, 1,46, 5) = (/ &
     &1.4913e-03_rb,2.3049e-03_rb,2.0781e-03_rb,1.5752e-03_rb,3.0303e-04_rb/)
      kbo(:, 2,46, 5) = (/ &
     &2.2619e-03_rb,3.3394e-03_rb,2.8896e-03_rb,2.0839e-03_rb,3.1705e-04_rb/)
      kbo(:, 3,46, 5) = (/ &
     &3.3424e-03_rb,4.7455e-03_rb,3.9592e-03_rb,2.7324e-03_rb,3.2997e-04_rb/)
      kbo(:, 4,46, 5) = (/ &
     &4.8041e-03_rb,6.6271e-03_rb,5.3667e-03_rb,3.5579e-03_rb,3.4174e-04_rb/)
      kbo(:, 5,46, 5) = (/ &
     &6.6700e-03_rb,9.0306e-03_rb,7.1468e-03_rb,4.5847e-03_rb,3.5220e-04_rb/)
      kbo(:, 1,47, 5) = (/ &
     &1.2089e-03_rb,1.8595e-03_rb,1.6774e-03_rb,1.2733e-03_rb,2.4435e-04_rb/)
      kbo(:, 2,47, 5) = (/ &
     &1.8589e-03_rb,2.7359e-03_rb,2.3669e-03_rb,1.7064e-03_rb,2.5622e-04_rb/)
      kbo(:, 3,47, 5) = (/ &
     &2.7805e-03_rb,3.9417e-03_rb,3.2867e-03_rb,2.2649e-03_rb,2.6731e-04_rb/)
      kbo(:, 4,47, 5) = (/ &
     &4.0468e-03_rb,5.5810e-03_rb,4.5086e-03_rb,2.9823e-03_rb,2.7740e-04_rb/)
      kbo(:, 5,47, 5) = (/ &
     &5.6890e-03_rb,7.7116e-03_rb,6.0862e-03_rb,3.8914e-03_rb,2.8663e-04_rb/)
      kbo(:, 1,48, 5) = (/ &
     &1.0010e-03_rb,1.5219e-03_rb,1.3705e-03_rb,1.0399e-03_rb,1.9690e-04_rb/)
      kbo(:, 2,48, 5) = (/ &
     &1.5623e-03_rb,2.2781e-03_rb,1.9661e-03_rb,1.4139e-03_rb,2.0690e-04_rb/)
      kbo(:, 3,48, 5) = (/ &
     &2.3663e-03_rb,3.3328e-03_rb,2.7724e-03_rb,1.9027e-03_rb,2.1643e-04_rb/)
      kbo(:, 4,48, 5) = (/ &
     &3.4895e-03_rb,4.7876e-03_rb,3.8543e-03_rb,2.5388e-03_rb,2.2513e-04_rb/)
      kbo(:, 5,48, 5) = (/ &
     &4.9716e-03_rb,6.7160e-03_rb,5.2797e-03_rb,3.3578e-03_rb,2.3320e-04_rb/)
      kbo(:, 1,49, 5) = (/ &
     &8.5765e-04_rb,1.2765e-03_rb,1.1434e-03_rb,8.6362e-04_rb,1.5857e-04_rb/)
      kbo(:, 2,49, 5) = (/ &
     &1.3602e-03_rb,1.9488e-03_rb,1.6723e-03_rb,1.1956e-03_rb,1.6712e-04_rb/)
      kbo(:, 3,49, 5) = (/ &
     &2.0873e-03_rb,2.8996e-03_rb,2.4000e-03_rb,1.6350e-03_rb,1.7515e-04_rb/)
      kbo(:, 4,49, 5) = (/ &
     &3.1195e-03_rb,4.2309e-03_rb,3.3880e-03_rb,2.2146e-03_rb,1.8269e-04_rb/)
      kbo(:, 5,49, 5) = (/ &
     &4.5042e-03_rb,6.0317e-03_rb,4.7143e-03_rb,2.9742e-03_rb,1.8971e-04_rb/)
      kbo(:, 1,50, 5) = (/ &
     &7.0327e-04_rb,1.0357e-03_rb,9.2617e-04_rb,6.9913e-04_rb,1.2768e-04_rb/)
      kbo(:, 2,50, 5) = (/ &
     &1.1341e-03_rb,1.6088e-03_rb,1.3773e-03_rb,9.8362e-04_rb,1.3488e-04_rb/)
      kbo(:, 3,50, 5) = (/ &
     &1.7633e-03_rb,2.4334e-03_rb,2.0087e-03_rb,1.3647e-03_rb,1.4174e-04_rb/)
      kbo(:, 4,50, 5) = (/ &
     &2.6686e-03_rb,3.6011e-03_rb,2.8760e-03_rb,1.8731e-03_rb,1.4819e-04_rb/)
      kbo(:, 5,50, 5) = (/ &
     &3.9042e-03_rb,5.2110e-03_rb,4.0582e-03_rb,2.5499e-03_rb,1.5417e-04_rb/)
      kbo(:, 1,51, 5) = (/ &
     &5.6283e-04_rb,8.2479e-04_rb,7.3777e-04_rb,5.5772e-04_rb,1.0280e-04_rb/)
      kbo(:, 2,51, 5) = (/ &
     &9.2490e-04_rb,1.3030e-03_rb,1.1149e-03_rb,7.9673e-04_rb,1.0882e-04_rb/)
      kbo(:, 3,51, 5) = (/ &
     &1.4569e-03_rb,2.0026e-03_rb,1.6508e-03_rb,1.1213e-03_rb,1.1463e-04_rb/)
      kbo(:, 4,51, 5) = (/ &
     &2.2322e-03_rb,3.0043e-03_rb,2.3967e-03_rb,1.5580e-03_rb,1.2010e-04_rb/)
      kbo(:, 5,51, 5) = (/ &
     &3.3090e-03_rb,4.4133e-03_rb,3.4281e-03_rb,2.1477e-03_rb,1.2516e-04_rb/)
      kbo(:, 1,52, 5) = (/ &
     &4.5964e-04_rb,6.6706e-04_rb,5.9570e-04_rb,4.4969e-04_rb,8.2695e-05_rb/)
      kbo(:, 2,52, 5) = (/ &
     &7.7113e-04_rb,1.0739e-03_rb,9.1646e-04_rb,6.5362e-04_rb,8.7774e-05_rb/)
      kbo(:, 3,52, 5) = (/ &
     &1.2329e-03_rb,1.6791e-03_rb,1.3796e-03_rb,9.3460e-04_rb,9.2675e-05_rb/)
      kbo(:, 4,52, 5) = (/ &
     &1.9124e-03_rb,2.5569e-03_rb,2.0345e-03_rb,1.3172e-03_rb,9.7314e-05_rb/)
      kbo(:, 5,52, 5) = (/ &
     &2.8744e-03_rb,3.8139e-03_rb,2.9526e-03_rb,1.8404e-03_rb,1.0157e-04_rb/)
      kbo(:, 1,53, 5) = (/ &
     &3.8730e-04_rb,5.5282e-04_rb,4.9108e-04_rb,3.6874e-04_rb,6.6480e-05_rb/)
      kbo(:, 2,53, 5) = (/ &
     &6.6529e-04_rb,9.0913e-04_rb,7.7130e-04_rb,5.4660e-04_rb,7.0743e-05_rb/)
      kbo(:, 3,53, 5) = (/ &
     &1.0811e-03_rb,1.4483e-03_rb,1.1832e-03_rb,7.9661e-04_rb,7.4902e-05_rb/)
      kbo(:, 4,53, 5) = (/ &
     &1.6977e-03_rb,2.2425e-03_rb,1.7750e-03_rb,1.1419e-03_rb,7.8801e-05_rb/)
      kbo(:, 5,53, 5) = (/ &
     &2.5875e-03_rb,3.3978e-03_rb,2.6180e-03_rb,1.6189e-03_rb,8.2434e-05_rb/)
      kbo(:, 1,54, 5) = (/ &
     &2.8438e-04_rb,4.1240e-04_rb,3.6923e-04_rb,2.8032e-04_rb,5.3391e-05_rb/)
      kbo(:, 2,54, 5) = (/ &
     &5.0038e-04_rb,6.8881e-04_rb,5.8764e-04_rb,4.1957e-04_rb,5.6986e-05_rb/)
      kbo(:, 3,54, 5) = (/ &
     &8.2710e-04_rb,1.1130e-03_rb,9.1323e-04_rb,6.1896e-04_rb,6.0464e-05_rb/)
      kbo(:, 4,54, 5) = (/ &
     &1.3140e-03_rb,1.7467e-03_rb,1.3863e-03_rb,8.9674e-04_rb,6.3698e-05_rb/)
      kbo(:, 5,54, 5) = (/ &
     &2.0289e-03_rb,2.6791e-03_rb,2.0677e-03_rb,1.2826e-03_rb,6.6754e-05_rb/)
      kbo(:, 1,55, 5) = (/ &
     &1.9032e-04_rb,2.8714e-04_rb,2.6164e-04_rb,2.0312e-04_rb,4.2857e-05_rb/)
      kbo(:, 2,55, 5) = (/ &
     &3.4354e-04_rb,4.8514e-04_rb,4.1940e-04_rb,3.0487e-04_rb,4.5872e-05_rb/)
      kbo(:, 3,55, 5) = (/ &
     &5.7798e-04_rb,7.9314e-04_rb,6.5789e-04_rb,4.5289e-04_rb,4.8737e-05_rb/)
      kbo(:, 4,55, 5) = (/ &
     &9.2977e-04_rb,1.2582e-03_rb,1.0072e-03_rb,6.6019e-04_rb,5.1425e-05_rb/)
      kbo(:, 5,55, 5) = (/ &
     &1.4533e-03_rb,1.9484e-03_rb,1.5135e-03_rb,9.4927e-04_rb,5.3965e-05_rb/)
      kbo(:, 1,56, 5) = (/ &
     &1.2624e-04_rb,1.9945e-04_rb,1.8526e-04_rb,1.4713e-04_rb,3.4367e-05_rb/)
      kbo(:, 2,56, 5) = (/ &
     &2.3425e-04_rb,3.4034e-04_rb,2.9855e-04_rb,2.2108e-04_rb,3.6885e-05_rb/)
      kbo(:, 3,56, 5) = (/ &
     &4.0201e-04_rb,5.6312e-04_rb,4.7257e-04_rb,3.3069e-04_rb,3.9253e-05_rb/)
      kbo(:, 4,56, 5) = (/ &
     &6.5545e-04_rb,9.0280e-04_rb,7.2955e-04_rb,4.8511e-04_rb,4.1521e-05_rb/)
      kbo(:, 5,56, 5) = (/ &
     &1.0372e-03_rb,1.4120e-03_rb,1.1051e-03_rb,7.0135e-04_rb,4.3631e-05_rb/)
      kbo(:, 1,57, 5) = (/ &
     &8.2825e-05_rb,1.3826e-04_rb,1.3105e-04_rb,1.0648e-04_rb,2.7541e-05_rb/)
      kbo(:, 2,57, 5) = (/ &
     &1.5848e-04_rb,2.3765e-04_rb,2.1180e-04_rb,1.6006e-04_rb,2.9646e-05_rb/)
      kbo(:, 3,57, 5) = (/ &
     &2.7802e-04_rb,3.9802e-04_rb,3.3815e-04_rb,2.4085e-04_rb,3.1604e-05_rb/)
      kbo(:, 4,57, 5) = (/ &
     &4.6014e-04_rb,6.4521e-04_rb,5.2696e-04_rb,3.5556e-04_rb,3.3502e-05_rb/)
      kbo(:, 5,57, 5) = (/ &
     &7.3676e-04_rb,1.0192e-03_rb,8.0418e-04_rb,5.1690e-04_rb,3.5248e-05_rb/)
      kbo(:, 1,58, 5) = (/ &
     &1.4413e-05_rb,3.1415e-05_rb,3.9206e-05_rb,4.5793e-05_rb,2.2070e-05_rb/)
      kbo(:, 2,58, 5) = (/ &
     &2.8319e-05_rb,5.4166e-05_rb,6.3295e-05_rb,6.8680e-05_rb,2.3823e-05_rb/)
      kbo(:, 3,58, 5) = (/ &
     &5.0851e-05_rb,9.1643e-05_rb,1.0172e-04_rb,1.0358e-04_rb,2.5450e-05_rb/)
      kbo(:, 4,58, 5) = (/ &
     &8.5536e-05_rb,1.4999e-04_rb,1.5967e-04_rb,1.5367e-04_rb,2.7016e-05_rb/)
      kbo(:, 5,58, 5) = (/ &
     &1.3845e-04_rb,2.3895e-04_rb,2.4509e-04_rb,2.2418e-04_rb,2.8488e-05_rb/)
      kbo(:, 1,59, 5) = (/ &
     &1.2766e-05_rb,2.9930e-05_rb,3.6342e-05_rb,3.9546e-05_rb,1.7924e-05_rb/)
      kbo(:, 2,59, 5) = (/ &
     &2.5308e-05_rb,5.0886e-05_rb,5.7559e-05_rb,5.8137e-05_rb,1.9351e-05_rb/)
      kbo(:, 3,59, 5) = (/ &
     &4.5886e-05_rb,8.5371e-05_rb,9.1314e-05_rb,8.6129e-05_rb,2.0681e-05_rb/)
      kbo(:, 4,59, 5) = (/ &
     &7.7783e-05_rb,1.3897e-04_rb,1.4192e-04_rb,1.2608e-04_rb,2.1937e-05_rb/)
      kbo(:, 5,59, 5) = (/ &
     &1.2671e-04_rb,2.2060e-04_rb,2.1586e-04_rb,1.8160e-04_rb,2.3122e-05_rb/)
      kbo(:, 1,13, 6) = (/ &
     &1.2091e+00_rb,1.0925e+00_rb,9.7132e-01_rb,7.6832e-01_rb,2.2429e-01_rb/)
      kbo(:, 2,13, 6) = (/ &
     &1.8464e+00_rb,1.5640e+00_rb,1.2977e+00_rb,9.6738e-01_rb,2.2945e-01_rb/)
      kbo(:, 3,13, 6) = (/ &
     &2.7006e+00_rb,2.1933e+00_rb,1.7174e+00_rb,1.2089e+00_rb,2.3586e-01_rb/)
      kbo(:, 4,13, 6) = (/ &
     &3.8041e+00_rb,3.0098e+00_rb,2.2534e+00_rb,1.4966e+00_rb,2.4148e-01_rb/)
      kbo(:, 5,13, 6) = (/ &
     &5.1874e+00_rb,4.0404e+00_rb,2.9277e+00_rb,1.8384e+00_rb,2.4717e-01_rb/)
      kbo(:, 1,14, 6) = (/ &
     &6.6672e-01_rb,6.8855e-01_rb,6.6353e-01_rb,5.5608e-01_rb,2.0092e-01_rb/)
      kbo(:, 2,14, 6) = (/ &
     &1.0160e+00_rb,9.4963e-01_rb,8.6392e-01_rb,6.8590e-01_rb,2.0621e-01_rb/)
      kbo(:, 3,14, 6) = (/ &
     &1.4836e+00_rb,1.2937e+00_rb,1.1112e+00_rb,8.3934e-01_rb,2.1228e-01_rb/)
      kbo(:, 4,14, 6) = (/ &
     &2.0856e+00_rb,1.7366e+00_rb,1.4142e+00_rb,1.0186e+00_rb,2.1791e-01_rb/)
      kbo(:, 5,14, 6) = (/ &
     &2.8402e+00_rb,2.2928e+00_rb,1.7814e+00_rb,1.2275e+00_rb,2.2338e-01_rb/)
      kbo(:, 1,15, 6) = (/ &
     &3.7534e-01_rb,4.5732e-01_rb,4.6750e-01_rb,4.0917e-01_rb,1.7874e-01_rb/)
      kbo(:, 2,15, 6) = (/ &
     &5.7029e-01_rb,6.1390e-01_rb,5.9540e-01_rb,4.9734e-01_rb,1.8407e-01_rb/)
      kbo(:, 3,15, 6) = (/ &
     &8.2994e-01_rb,8.1258e-01_rb,7.5038e-01_rb,5.9870e-01_rb,1.8982e-01_rb/)
      kbo(:, 4,15, 6) = (/ &
     &1.1635e+00_rb,1.0593e+00_rb,9.3529e-01_rb,7.1512e-01_rb,1.9538e-01_rb/)
      kbo(:, 5,15, 6) = (/ &
     &1.5834e+00_rb,1.3649e+00_rb,1.1537e+00_rb,8.4763e-01_rb,2.0092e-01_rb/)
      kbo(:, 1,16, 6) = (/ &
     &2.5139e-01_rb,3.4640e-01_rb,3.6560e-01_rb,3.2723e-01_rb,1.5796e-01_rb/)
      kbo(:, 2,16, 6) = (/ &
     &3.8033e-01_rb,4.5844e-01_rb,4.5962e-01_rb,3.9345e-01_rb,1.6322e-01_rb/)
      kbo(:, 3,16, 6) = (/ &
     &5.5135e-01_rb,5.9731e-01_rb,5.7234e-01_rb,4.6896e-01_rb,1.6890e-01_rb/)
      kbo(:, 4,16, 6) = (/ &
     &7.7178e-01_rb,7.6699e-01_rb,7.0504e-01_rb,5.5557e-01_rb,1.7446e-01_rb/)
      kbo(:, 5,16, 6) = (/ &
     &1.0505e+00_rb,9.7332e-01_rb,8.6035e-01_rb,6.5296e-01_rb,1.7998e-01_rb/)
      kbo(:, 1,17, 6) = (/ &
     &1.7391e-01_rb,2.6830e-01_rb,2.8935e-01_rb,2.6535e-01_rb,1.3871e-01_rb/)
      kbo(:, 2,17, 6) = (/ &
     &2.6196e-01_rb,3.5091e-01_rb,3.6089e-01_rb,3.1647e-01_rb,1.4388e-01_rb/)
      kbo(:, 3,17, 6) = (/ &
     &3.7888e-01_rb,4.5188e-01_rb,4.4522e-01_rb,3.7420e-01_rb,1.4950e-01_rb/)
      kbo(:, 4,17, 6) = (/ &
     &5.3041e-01_rb,5.7470e-01_rb,5.4379e-01_rb,4.3949e-01_rb,1.5497e-01_rb/)
      kbo(:, 5,17, 6) = (/ &
     &7.2254e-01_rb,7.2344e-01_rb,6.5849e-01_rb,5.1285e-01_rb,1.6018e-01_rb/)
      kbo(:, 1,18, 6) = (/ &
     &1.2310e-01_rb,2.1053e-01_rb,2.3152e-01_rb,2.1713e-01_rb,1.2112e-01_rb/)
      kbo(:, 2,18, 6) = (/ &
     &1.8497e-01_rb,2.7232e-01_rb,2.8648e-01_rb,2.5684e-01_rb,1.2635e-01_rb/)
      kbo(:, 3,18, 6) = (/ &
     &2.6727e-01_rb,3.4798e-01_rb,3.5045e-01_rb,3.0154e-01_rb,1.3167e-01_rb/)
      kbo(:, 4,18, 6) = (/ &
     &3.7442e-01_rb,4.3958e-01_rb,4.2522e-01_rb,3.5185e-01_rb,1.3670e-01_rb/)
      kbo(:, 5,18, 6) = (/ &
     &5.0994e-01_rb,5.4989e-01_rb,5.1174e-01_rb,4.0832e-01_rb,1.4125e-01_rb/)
      kbo(:, 1,19, 6) = (/ &
     &8.5532e-02_rb,1.6314e-01_rb,1.8367e-01_rb,1.7636e-01_rb,1.0532e-01_rb/)
      kbo(:, 2,19, 6) = (/ &
     &1.2828e-01_rb,2.0893e-01_rb,2.2484e-01_rb,2.0705e-01_rb,1.1032e-01_rb/)
      kbo(:, 3,19, 6) = (/ &
     &1.8543e-01_rb,2.6474e-01_rb,2.7268e-01_rb,2.4148e-01_rb,1.1512e-01_rb/)
      kbo(:, 4,19, 6) = (/ &
     &2.5984e-01_rb,3.3238e-01_rb,3.2852e-01_rb,2.8001e-01_rb,1.1952e-01_rb/)
      kbo(:, 5,19, 6) = (/ &
     &3.5320e-01_rb,4.1295e-01_rb,3.9317e-01_rb,3.2333e-01_rb,1.2366e-01_rb/)
      kbo(:, 1,20, 6) = (/ &
     &6.4796e-02_rb,1.3254e-01_rb,1.5120e-01_rb,1.4730e-01_rb,9.1341e-02_rb/)
      kbo(:, 2,20, 6) = (/ &
     &9.6990e-02_rb,1.6885e-01_rb,1.8400e-01_rb,1.7239e-01_rb,9.5796e-02_rb/)
      kbo(:, 3,20, 6) = (/ &
     &1.4017e-01_rb,2.1315e-01_rb,2.2212e-01_rb,2.0058e-01_rb,9.9929e-02_rb/)
      kbo(:, 4,20, 6) = (/ &
     &1.9600e-01_rb,2.6649e-01_rb,2.6658e-01_rb,2.3206e-01_rb,1.0386e-01_rb/)
      kbo(:, 5,20, 6) = (/ &
     &2.6559e-01_rb,3.2963e-01_rb,3.1802e-01_rb,2.6726e-01_rb,1.0762e-01_rb/)
      kbo(:, 1,21, 6) = (/ &
     &5.0649e-02_rb,1.0928e-01_rb,1.2583e-01_rb,1.2405e-01_rb,7.8779e-02_rb/)
      kbo(:, 2,21, 6) = (/ &
     &7.5738e-02_rb,1.3875e-01_rb,1.5264e-01_rb,1.4489e-01_rb,8.2609e-02_rb/)
      kbo(:, 3,21, 6) = (/ &
     &1.0925e-01_rb,1.7464e-01_rb,1.8399e-01_rb,1.6832e-01_rb,8.6258e-02_rb/)
      kbo(:, 4,21, 6) = (/ &
     &1.5216e-01_rb,2.1765e-01_rb,2.2035e-01_rb,1.9451e-01_rb,8.9821e-02_rb/)
      kbo(:, 5,21, 6) = (/ &
     &2.0550e-01_rb,2.6822e-01_rb,2.6230e-01_rb,2.2354e-01_rb,9.3160e-02_rb/)
      kbo(:, 1,22, 6) = (/ &
     &4.1767e-02_rb,9.2608e-02_rb,1.0699e-01_rb,1.0611e-01_rb,6.7766e-02_rb/)
      kbo(:, 2,22, 6) = (/ &
     &6.2192e-02_rb,1.1726e-01_rb,1.2959e-01_rb,1.2363e-01_rb,7.1096e-02_rb/)
      kbo(:, 3,22, 6) = (/ &
     &8.9136e-02_rb,1.4711e-01_rb,1.5605e-01_rb,1.4343e-01_rb,7.4363e-02_rb/)
      kbo(:, 4,22, 6) = (/ &
     &1.2346e-01_rb,1.8283e-01_rb,1.8672e-01_rb,1.6558e-01_rb,7.7494e-02_rb/)
      kbo(:, 5,22, 6) = (/ &
     &1.6595e-01_rb,2.2428e-01_rb,2.2184e-01_rb,1.9008e-01_rb,8.0478e-02_rb/)
      kbo(:, 1,23, 6) = (/ &
     &3.4739e-02_rb,7.8664e-02_rb,9.1136e-02_rb,9.0610e-02_rb,5.8065e-02_rb/)
      kbo(:, 2,23, 6) = (/ &
     &5.1379e-02_rb,9.9376e-02_rb,1.1032e-01_rb,1.0537e-01_rb,6.0980e-02_rb/)
      kbo(:, 3,23, 6) = (/ &
     &7.3154e-02_rb,1.2441e-01_rb,1.3261e-01_rb,1.2218e-01_rb,6.3866e-02_rb/)
      kbo(:, 4,23, 6) = (/ &
     &1.0075e-01_rb,1.5392e-01_rb,1.5845e-01_rb,1.4087e-01_rb,6.6610e-02_rb/)
      kbo(:, 5,23, 6) = (/ &
     &1.3481e-01_rb,1.8817e-01_rb,1.8784e-01_rb,1.6172e-01_rb,6.9269e-02_rb/)
      kbo(:, 1,24, 6) = (/ &
     &2.8818e-02_rb,6.6682e-02_rb,7.7471e-02_rb,7.7057e-02_rb,4.9574e-02_rb/)
      kbo(:, 2,24, 6) = (/ &
     &4.2313e-02_rb,8.4062e-02_rb,9.3582e-02_rb,8.9537e-02_rb,5.2146e-02_rb/)
      kbo(:, 3,24, 6) = (/ &
     &5.9843e-02_rb,1.0485e-01_rb,1.1232e-01_rb,1.0372e-01_rb,5.4645e-02_rb/)
      kbo(:, 4,24, 6) = (/ &
     &8.1952e-02_rb,1.2924e-01_rb,1.3388e-01_rb,1.1954e-01_rb,5.7051e-02_rb/)
      kbo(:, 5,24, 6) = (/ &
     &1.0925e-01_rb,1.5756e-01_rb,1.5837e-01_rb,1.3710e-01_rb,5.9328e-02_rb/)
      kbo(:, 1,25, 6) = (/ &
     &2.4289e-02_rb,5.7018e-02_rb,6.6182e-02_rb,6.5743e-02_rb,4.2224e-02_rb/)
      kbo(:, 2,25, 6) = (/ &
     &3.5371e-02_rb,7.1685e-02_rb,7.9860e-02_rb,7.6400e-02_rb,4.4451e-02_rb/)
      kbo(:, 3,25, 6) = (/ &
     &4.9695e-02_rb,8.9091e-02_rb,9.5704e-02_rb,8.8452e-02_rb,4.6603e-02_rb/)
      kbo(:, 4,25, 6) = (/ &
     &6.7687e-02_rb,1.0953e-01_rb,1.1385e-01_rb,1.0188e-01_rb,4.8655e-02_rb/)
      kbo(:, 5,25, 6) = (/ &
     &9.0080e-02_rb,1.3324e-01_rb,1.3448e-01_rb,1.1679e-01_rb,5.0366e-02_rb/)
      kbo(:, 1,26, 6) = (/ &
     &2.0940e-02_rb,4.9340e-02_rb,5.7029e-02_rb,5.6467e-02_rb,3.5910e-02_rb/)
      kbo(:, 2,26, 6) = (/ &
     &3.0229e-02_rb,6.1848e-02_rb,6.8786e-02_rb,6.5644e-02_rb,3.7816e-02_rb/)
      kbo(:, 3,26, 6) = (/ &
     &4.2173e-02_rb,7.6666e-02_rb,8.2327e-02_rb,7.6015e-02_rb,3.9629e-02_rb/)
      kbo(:, 4,26, 6) = (/ &
     &5.7238e-02_rb,9.4072e-02_rb,9.7854e-02_rb,8.7548e-02_rb,4.1173e-02_rb/)
      kbo(:, 5,26, 6) = (/ &
     &7.6058e-02_rb,1.1436e-01_rb,1.1545e-01_rb,1.0034e-01_rb,4.2300e-02_rb/)
      kbo(:, 1,27, 6) = (/ &
     &1.8730e-02_rb,4.3572e-02_rb,4.9937e-02_rb,4.9064e-02_rb,3.0474e-02_rb/)
      kbo(:, 2,27, 6) = (/ &
     &2.6831e-02_rb,5.4541e-02_rb,6.0259e-02_rb,5.7134e-02_rb,3.2064e-02_rb/)
      kbo(:, 3,27, 6) = (/ &
     &3.7216e-02_rb,6.7578e-02_rb,7.2151e-02_rb,6.6197e-02_rb,3.3458e-02_rb/)
      kbo(:, 4,27, 6) = (/ &
     &5.0417e-02_rb,8.2889e-02_rb,8.5762e-02_rb,7.6299e-02_rb,3.4507e-02_rb/)
      kbo(:, 5,27, 6) = (/ &
     &6.6884e-02_rb,1.0088e-01_rb,1.0129e-01_rb,8.7542e-02_rb,3.5247e-02_rb/)
      kbo(:, 1,28, 6) = (/ &
     &1.7190e-02_rb,3.9051e-02_rb,4.4263e-02_rb,4.3005e-02_rb,2.5796e-02_rb/)
      kbo(:, 2,28, 6) = (/ &
     &2.4452e-02_rb,4.8897e-02_rb,5.3477e-02_rb,5.0169e-02_rb,2.7044e-02_rb/)
      kbo(:, 3,28, 6) = (/ &
     &3.3807e-02_rb,6.0596e-02_rb,6.4111e-02_rb,5.8195e-02_rb,2.8015e-02_rb/)
      kbo(:, 4,28, 6) = (/ &
     &4.5722e-02_rb,7.4408e-02_rb,7.6314e-02_rb,6.7197e-02_rb,2.8722e-02_rb/)
      kbo(:, 5,28, 6) = (/ &
     &6.0503e-02_rb,9.0881e-02_rb,9.0351e-02_rb,7.7127e-02_rb,2.9243e-02_rb/)
      kbo(:, 1,29, 6) = (/ &
     &1.6778e-02_rb,3.6309e-02_rb,4.0421e-02_rb,3.8540e-02_rb,2.1757e-02_rb/)
      kbo(:, 2,29, 6) = (/ &
     &2.3732e-02_rb,4.5567e-02_rb,4.8987e-02_rb,4.5076e-02_rb,2.2654e-02_rb/)
      kbo(:, 3,29, 6) = (/ &
     &3.2754e-02_rb,5.6621e-02_rb,5.8900e-02_rb,5.2465e-02_rb,2.3325e-02_rb/)
      kbo(:, 4,29, 6) = (/ &
     &4.4197e-02_rb,6.9861e-02_rb,7.0379e-02_rb,6.0698e-02_rb,2.3826e-02_rb/)
      kbo(:, 5,29, 6) = (/ &
     &5.8243e-02_rb,8.5699e-02_rb,8.3806e-02_rb,6.9875e-02_rb,2.4188e-02_rb/)
      kbo(:, 1,30, 6) = (/ &
     &1.6788e-02_rb,3.4359e-02_rb,3.7447e-02_rb,3.4898e-02_rb,1.8241e-02_rb/)
      kbo(:, 2,30, 6) = (/ &
     &2.3674e-02_rb,4.3260e-02_rb,4.5554e-02_rb,4.0970e-02_rb,1.8874e-02_rb/)
      kbo(:, 3,30, 6) = (/ &
     &3.2611e-02_rb,5.3991e-02_rb,5.4995e-02_rb,4.7819e-02_rb,1.9352e-02_rb/)
      kbo(:, 4,30, 6) = (/ &
     &4.3823e-02_rb,6.7000e-02_rb,6.6123e-02_rb,5.5495e-02_rb,1.9717e-02_rb/)
      kbo(:, 5,30, 6) = (/ &
     &5.7496e-02_rb,8.2562e-02_rb,7.9216e-02_rb,6.4205e-02_rb,1.9974e-02_rb/)
      kbo(:, 1,31, 6) = (/ &
     &1.7729e-02_rb,3.3726e-02_rb,3.5756e-02_rb,3.2351e-02_rb,1.5217e-02_rb/)
      kbo(:, 2,31, 6) = (/ &
     &2.4948e-02_rb,4.2686e-02_rb,4.3734e-02_rb,3.8162e-02_rb,1.5670e-02_rb/)
      kbo(:, 3,31, 6) = (/ &
     &3.4225e-02_rb,5.3676e-02_rb,5.3194e-02_rb,4.4735e-02_rb,1.6031e-02_rb/)
      kbo(:, 4,31, 6) = (/ &
     &4.5784e-02_rb,6.7038e-02_rb,6.4468e-02_rb,5.2256e-02_rb,1.6295e-02_rb/)
      kbo(:, 5,31, 6) = (/ &
     &5.9817e-02_rb,8.2952e-02_rb,7.7737e-02_rb,6.0918e-02_rb,1.6485e-02_rb/)
      kbo(:, 1,32, 6) = (/ &
     &1.9003e-02_rb,3.3583e-02_rb,3.4583e-02_rb,3.0306e-02_rb,1.2654e-02_rb/)
      kbo(:, 2,32, 6) = (/ &
     &2.6663e-02_rb,4.2837e-02_rb,4.2596e-02_rb,3.5915e-02_rb,1.2998e-02_rb/)
      kbo(:, 3,32, 6) = (/ &
     &3.6385e-02_rb,5.4256e-02_rb,5.2269e-02_rb,4.2382e-02_rb,1.3260e-02_rb/)
      kbo(:, 4,32, 6) = (/ &
     &4.8425e-02_rb,6.8084e-02_rb,6.3837e-02_rb,4.9934e-02_rb,1.3455e-02_rb/)
      kbo(:, 5,32, 6) = (/ &
     &6.3023e-02_rb,8.4608e-02_rb,7.7399e-02_rb,5.8694e-02_rb,1.3589e-02_rb/)
      kbo(:, 1,33, 6) = (/ &
     &2.0794e-02_rb,3.4164e-02_rb,3.4049e-02_rb,2.8789e-02_rb,1.0506e-02_rb/)
      kbo(:, 2,33, 6) = (/ &
     &2.9020e-02_rb,4.3934e-02_rb,4.2358e-02_rb,3.4361e-02_rb,1.0761e-02_rb/)
      kbo(:, 3,33, 6) = (/ &
     &3.9409e-02_rb,5.5981e-02_rb,5.2449e-02_rb,4.0931e-02_rb,1.0954e-02_rb/)
      kbo(:, 4,33, 6) = (/ &
     &5.2184e-02_rb,7.0595e-02_rb,6.4456e-02_rb,4.8674e-02_rb,1.1100e-02_rb/)
      kbo(:, 5,33, 6) = (/ &
     &6.7723e-02_rb,8.8066e-02_rb,7.8614e-02_rb,5.7672e-02_rb,1.1201e-02_rb/)
      kbo(:, 1,34, 6) = (/ &
     &2.1935e-02_rb,3.4165e-02_rb,3.3105e-02_rb,2.7057e-02_rb,8.7014e-03_rb/)
      kbo(:, 2,34, 6) = (/ &
     &3.0477e-02_rb,4.4226e-02_rb,4.1562e-02_rb,3.2560e-02_rb,8.8925e-03_rb/)
      kbo(:, 3,34, 6) = (/ &
     &4.1199e-02_rb,5.6620e-02_rb,5.1809e-02_rb,3.9153e-02_rb,9.0424e-03_rb/)
      kbo(:, 4,34, 6) = (/ &
     &5.4372e-02_rb,7.1709e-02_rb,6.4039e-02_rb,4.6915e-02_rb,9.1557e-03_rb/)
      kbo(:, 5,34, 6) = (/ &
     &7.0476e-02_rb,8.9710e-02_rb,7.8542e-02_rb,5.5979e-02_rb,9.2318e-03_rb/)
      kbo(:, 1,35, 6) = (/ &
     &2.2432e-02_rb,3.3575e-02_rb,3.1770e-02_rb,2.5159e-02_rb,7.1825e-03_rb/)
      kbo(:, 2,35, 6) = (/ &
     &3.1101e-02_rb,4.3743e-02_rb,4.0231e-02_rb,3.0588e-02_rb,7.3372e-03_rb/)
      kbo(:, 3,35, 6) = (/ &
     &4.1959e-02_rb,5.6323e-02_rb,5.0491e-02_rb,3.7114e-02_rb,7.4571e-03_rb/)
      kbo(:, 4,35, 6) = (/ &
     &5.5361e-02_rb,7.1664e-02_rb,6.2827e-02_rb,4.4829e-02_rb,7.5457e-03_rb/)
      kbo(:, 5,35, 6) = (/ &
     &7.1841e-02_rb,8.9995e-02_rb,7.7538e-02_rb,5.3898e-02_rb,7.6098e-03_rb/)
      kbo(:, 1,36, 6) = (/ &
     &2.1971e-02_rb,3.2052e-02_rb,2.9773e-02_rb,2.2980e-02_rb,5.9169e-03_rb/)
      kbo(:, 2,36, 6) = (/ &
     &3.0496e-02_rb,4.2068e-02_rb,3.8012e-02_rb,2.8228e-02_rb,6.0417e-03_rb/)
      kbo(:, 3,36, 6) = (/ &
     &4.1195e-02_rb,5.4532e-02_rb,4.8068e-02_rb,3.4542e-02_rb,6.1403e-03_rb/)
      kbo(:, 4,36, 6) = (/ &
     &5.4471e-02_rb,6.9759e-02_rb,6.0254e-02_rb,4.2072e-02_rb,6.2148e-03_rb/)
      kbo(:, 5,36, 6) = (/ &
     &7.0911e-02_rb,8.8088e-02_rb,7.4912e-02_rb,5.0992e-02_rb,6.2662e-03_rb/)
      kbo(:, 1,37, 6) = (/ &
     &1.9922e-02_rb,2.8852e-02_rb,2.6493e-02_rb,2.0135e-02_rb,4.8546e-03_rb/)
      kbo(:, 2,37, 6) = (/ &
     &2.7802e-02_rb,3.8202e-02_rb,3.4133e-02_rb,2.4969e-02_rb,4.9597e-03_rb/)
      kbo(:, 3,37, 6) = (/ &
     &3.7750e-02_rb,4.9930e-02_rb,4.3532e-02_rb,3.0816e-02_rb,5.0441e-03_rb/)
      kbo(:, 4,37, 6) = (/ &
     &5.0184e-02_rb,6.4346e-02_rb,5.5012e-02_rb,3.7855e-02_rb,5.1082e-03_rb/)
      kbo(:, 5,37, 6) = (/ &
     &6.5650e-02_rb,8.1819e-02_rb,6.8970e-02_rb,4.6262e-02_rb,5.1546e-03_rb/)
      kbo(:, 1,38, 6) = (/ &
     &1.8261e-02_rb,2.6220e-02_rb,2.3783e-02_rb,1.7789e-02_rb,3.9817e-03_rb/)
      kbo(:, 2,38, 6) = (/ &
     &2.5634e-02_rb,3.5047e-02_rb,3.0941e-02_rb,2.2280e-02_rb,4.0729e-03_rb/)
      kbo(:, 3,38, 6) = (/ &
     &3.5007e-02_rb,4.6188e-02_rb,3.9826e-02_rb,2.7761e-02_rb,4.1447e-03_rb/)
      kbo(:, 4,38, 6) = (/ &
     &4.6818e-02_rb,5.9985e-02_rb,5.0782e-02_rb,3.4412e-02_rb,4.1996e-03_rb/)
      kbo(:, 5,38, 6) = (/ &
     &6.1549e-02_rb,7.6851e-02_rb,6.4240e-02_rb,4.2442e-02_rb,4.2409e-03_rb/)
      kbo(:, 1,39, 6) = (/ &
     &1.7120e-02_rb,2.4268e-02_rb,2.1723e-02_rb,1.5967e-02_rb,3.2683e-03_rb/)
      kbo(:, 2,39, 6) = (/ &
     &2.4172e-02_rb,3.2753e-02_rb,2.8567e-02_rb,2.0215e-02_rb,3.3454e-03_rb/)
      kbo(:, 3,39, 6) = (/ &
     &3.3206e-02_rb,4.3552e-02_rb,3.7135e-02_rb,2.5455e-02_rb,3.4062e-03_rb/)
      kbo(:, 4,39, 6) = (/ &
     &4.4652e-02_rb,5.7021e-02_rb,4.7802e-02_rb,3.1863e-02_rb,3.4542e-03_rb/)
      kbo(:, 5,39, 6) = (/ &
     &5.8989e-02_rb,7.3628e-02_rb,6.1045e-02_rb,3.9701e-02_rb,3.4927e-03_rb/)
      kbo(:, 1,40, 6) = (/ &
     &1.4508e-02_rb,2.0729e-02_rb,1.8466e-02_rb,1.3481e-02_rb,2.6735e-03_rb/)
      kbo(:, 2,40, 6) = (/ &
     &2.0675e-02_rb,2.8264e-02_rb,2.4526e-02_rb,1.7230e-02_rb,2.7394e-03_rb/)
      kbo(:, 3,40, 6) = (/ &
     &2.8640e-02_rb,3.7952e-02_rb,3.2186e-02_rb,2.1896e-02_rb,2.7930e-03_rb/)
      kbo(:, 4,40, 6) = (/ &
     &3.8810e-02_rb,5.0166e-02_rb,4.1826e-02_rb,2.7660e-02_rb,2.8356e-03_rb/)
      kbo(:, 5,40, 6) = (/ &
     &5.1624e-02_rb,6.5327e-02_rb,5.3925e-02_rb,3.4781e-02_rb,2.8708e-03_rb/)
      kbo(:, 1,41, 6) = (/ &
     &1.2173e-02_rb,1.7558e-02_rb,1.5577e-02_rb,1.1317e-02_rb,2.1860e-03_rb/)
      kbo(:, 2,41, 6) = (/ &
     &1.7521e-02_rb,2.4204e-02_rb,2.0905e-02_rb,1.4606e-02_rb,2.2431e-03_rb/)
      kbo(:, 3,41, 6) = (/ &
     &2.4494e-02_rb,3.2840e-02_rb,2.7710e-02_rb,1.8738e-02_rb,2.2897e-03_rb/)
      kbo(:, 4,41, 6) = (/ &
     &3.3465e-02_rb,4.3838e-02_rb,3.6359e-02_rb,2.3890e-02_rb,2.3273e-03_rb/)
      kbo(:, 5,41, 6) = (/ &
     &4.4815e-02_rb,5.7614e-02_rb,4.7337e-02_rb,3.0319e-02_rb,2.3593e-03_rb/)
      kbo(:, 1,42, 6) = (/ &
     &1.0295e-02_rb,1.4955e-02_rb,1.3204e-02_rb,9.5463e-03_rb,1.7869e-03_rb/)
      kbo(:, 2,42, 6) = (/ &
     &1.4969e-02_rb,2.0854e-02_rb,1.7923e-02_rb,1.2449e-02_rb,1.8368e-03_rb/)
      kbo(:, 3,42, 6) = (/ &
     &2.1120e-02_rb,2.8597e-02_rb,2.4004e-02_rb,1.6130e-02_rb,1.8766e-03_rb/)
      kbo(:, 4,42, 6) = (/ &
     &2.9097e-02_rb,3.8565e-02_rb,3.1822e-02_rb,2.0764e-02_rb,1.9105e-03_rb/)
      kbo(:, 5,42, 6) = (/ &
     &3.9250e-02_rb,5.1179e-02_rb,4.1851e-02_rb,2.6620e-02_rb,1.9390e-03_rb/)
      kbo(:, 1,43, 6) = (/ &
     &8.5600e-03_rb,1.2537e-02_rb,1.1032e-02_rb,7.9541e-03_rb,1.4581e-03_rb/)
      kbo(:, 2,43, 6) = (/ &
     &1.2604e-02_rb,1.7710e-02_rb,1.5163e-02_rb,1.0490e-02_rb,1.5008e-03_rb/)
      kbo(:, 3,43, 6) = (/ &
     &1.7986e-02_rb,2.4578e-02_rb,2.0552e-02_rb,1.3738e-02_rb,1.5357e-03_rb/)
      kbo(:, 4,43, 6) = (/ &
     &2.5016e-02_rb,3.3532e-02_rb,2.7550e-02_rb,1.7878e-02_rb,1.5671e-03_rb/)
      kbo(:, 5,43, 6) = (/ &
     &3.4031e-02_rb,4.4978e-02_rb,3.6624e-02_rb,2.3164e-02_rb,1.5928e-03_rb/)
      kbo(:, 1,44, 6) = (/ &
     &7.1102e-03_rb,1.0482e-02_rb,9.1923e-03_rb,6.6123e-03_rb,1.1883e-03_rb/)
      kbo(:, 2,44, 6) = (/ &
     &1.0618e-02_rb,1.5014e-02_rb,1.2809e-02_rb,8.8284e-03_rb,1.2254e-03_rb/)
      kbo(:, 3,44, 6) = (/ &
     &1.5336e-02_rb,2.1109e-02_rb,1.7587e-02_rb,1.1700e-02_rb,1.2566e-03_rb/)
      kbo(:, 4,44, 6) = (/ &
     &2.1558e-02_rb,2.9159e-02_rb,2.3862e-02_rb,1.5404e-02_rb,1.2849e-03_rb/)
      kbo(:, 5,44, 6) = (/ &
     &2.9601e-02_rb,3.9572e-02_rb,3.2092e-02_rb,2.0188e-02_rb,1.3081e-03_rb/)
      kbo(:, 1,45, 6) = (/ &
     &5.9959e-03_rb,8.8564e-03_rb,7.7363e-03_rb,5.5453e-03_rb,9.6825e-04_rb/)
      kbo(:, 2,45, 6) = (/ &
     &9.0798e-03_rb,1.2878e-02_rb,1.0936e-02_rb,7.5039e-03_rb,1.0003e-03_rb/)
      kbo(:, 3,45, 6) = (/ &
     &1.3290e-02_rb,1.8346e-02_rb,1.5227e-02_rb,1.0074e-02_rb,1.0284e-03_rb/)
      kbo(:, 4,45, 6) = (/ &
     &1.8884e-02_rb,2.5678e-02_rb,2.0927e-02_rb,1.3427e-02_rb,1.0535e-03_rb/)
      kbo(:, 5,45, 6) = (/ &
     &2.6185e-02_rb,3.5270e-02_rb,2.8485e-02_rb,1.7812e-02_rb,1.0746e-03_rb/)
      kbo(:, 1,46, 6) = (/ &
     &5.0299e-03_rb,7.4350e-03_rb,6.4749e-03_rb,4.6277e-03_rb,7.8776e-04_rb/)
      kbo(:, 2,46, 6) = (/ &
     &7.7400e-03_rb,1.0992e-02_rb,9.2918e-03_rb,6.3536e-03_rb,8.1609e-04_rb/)
      kbo(:, 3,46, 6) = (/ &
     &1.1482e-02_rb,1.5887e-02_rb,1.3136e-02_rb,8.6493e-03_rb,8.4114e-04_rb/)
      kbo(:, 4,46, 6) = (/ &
     &1.6521e-02_rb,2.2547e-02_rb,1.8306e-02_rb,1.1680e-02_rb,8.6343e-04_rb/)
      kbo(:, 5,46, 6) = (/ &
     &2.3160e-02_rb,3.1365e-02_rb,2.5232e-02_rb,1.5694e-02_rb,8.8222e-04_rb/)
      kbo(:, 1,47, 6) = (/ &
     &4.0735e-03_rb,6.0481e-03_rb,5.2664e-03_rb,3.7670e-03_rb,6.3966e-04_rb/)
      kbo(:, 2,47, 6) = (/ &
     &6.3819e-03_rb,9.1048e-03_rb,7.6804e-03_rb,5.2485e-03_rb,6.6459e-04_rb/)
      kbo(:, 3,47, 6) = (/ &
     &9.6180e-03_rb,1.3376e-02_rb,1.1030e-02_rb,7.2481e-03_rb,6.8646e-04_rb/)
      kbo(:, 4,47, 6) = (/ &
     &1.4038e-02_rb,1.9266e-02_rb,1.5600e-02_rb,9.9209e-03_rb,7.0632e-04_rb/)
      kbo(:, 5,47, 6) = (/ &
     &1.9923e-02_rb,2.7168e-02_rb,2.1786e-02_rb,1.3500e-02_rb,7.2294e-04_rb/)
      kbo(:, 1,48, 6) = (/ &
     &3.3654e-03_rb,4.9911e-03_rb,4.3393e-03_rb,3.1001e-03_rb,5.1894e-04_rb/)
      kbo(:, 2,48, 6) = (/ &
     &5.3738e-03_rb,7.6583e-03_rb,6.4413e-03_rb,4.3896e-03_rb,5.4055e-04_rb/)
      kbo(:, 3,48, 6) = (/ &
     &8.2348e-03_rb,1.1452e-02_rb,9.4090e-03_rb,6.1594e-03_rb,5.5982e-04_rb/)
      kbo(:, 4,48, 6) = (/ &
     &1.2194e-02_rb,1.6751e-02_rb,1.3518e-02_rb,8.5585e-03_rb,5.7759e-04_rb/)
      kbo(:, 5,48, 6) = (/ &
     &1.7535e-02_rb,2.3969e-02_rb,1.9153e-02_rb,1.1809e-02_rb,5.9223e-04_rb/)
      kbo(:, 1,49, 6) = (/ &
     &2.8714e-03_rb,4.2192e-03_rb,3.6533e-03_rb,2.6003e-03_rb,4.2066e-04_rb/)
      kbo(:, 2,49, 6) = (/ &
     &4.6796e-03_rb,6.6096e-03_rb,5.5332e-03_rb,3.7482e-03_rb,4.3948e-04_rb/)
      kbo(:, 3,49, 6) = (/ &
     &7.3002e-03_rb,1.0080e-02_rb,8.2347e-03_rb,5.3596e-03_rb,4.5652e-04_rb/)
      kbo(:, 4,49, 6) = (/ &
     &1.0971e-02_rb,1.4981e-02_rb,1.2046e-02_rb,7.5739e-03_rb,4.7234e-04_rb/)
      kbo(:, 5,49, 6) = (/ &
     &1.5995e-02_rb,2.1767e-02_rb,1.7330e-02_rb,1.0614e-02_rb,4.8568e-04_rb/)
      kbo(:, 1,50, 6) = (/ &
     &2.3434e-03_rb,3.4410e-03_rb,2.9786e-03_rb,2.1205e-03_rb,3.4069e-04_rb/)
      kbo(:, 2,50, 6) = (/ &
     &3.8971e-03_rb,5.4992e-03_rb,4.5940e-03_rb,3.1036e-03_rb,3.5714e-04_rb/)
      kbo(:, 3,50, 6) = (/ &
     &6.1900e-03_rb,8.5460e-03_rb,6.9553e-03_rb,4.5152e-03_rb,3.7212e-04_rb/)
      kbo(:, 4,50, 6) = (/ &
     &9.4451e-03_rb,1.2909e-02_rb,1.0349e-02_rb,6.4854e-03_rb,3.8582e-04_rb/)
      kbo(:, 5,50, 6) = (/ &
     &1.3954e-02_rb,1.9042e-02_rb,1.5119e-02_rb,9.2171e-03_rb,3.9788e-04_rb/)
      kbo(:, 1,51, 6) = (/ &
     &1.8670e-03_rb,2.7492e-03_rb,2.3841e-03_rb,1.7021e-03_rb,2.7578e-04_rb/)
      kbo(:, 2,51, 6) = (/ &
     &3.1678e-03_rb,4.4826e-03_rb,3.7408e-03_rb,2.5267e-03_rb,2.9005e-04_rb/)
      kbo(:, 3,51, 6) = (/ &
     &5.1260e-03_rb,7.0985e-03_rb,5.7656e-03_rb,3.7383e-03_rb,3.0300e-04_rb/)
      kbo(:, 4,51, 6) = (/ &
     &7.9439e-03_rb,1.0904e-02_rb,8.7196e-03_rb,5.4539e-03_rb,3.1487e-04_rb/)
      kbo(:, 5,51, 6) = (/ &
     &1.1900e-02_rb,1.6326e-02_rb,1.2929e-02_rb,7.8597e-03_rb,3.2578e-04_rb/)
      kbo(:, 1,52, 6) = (/ &
     &1.5181e-03_rb,2.2298e-03_rb,1.9337e-03_rb,1.3821e-03_rb,2.2294e-04_rb/)
      kbo(:, 2,52, 6) = (/ &
     &2.6323e-03_rb,3.7113e-03_rb,3.0904e-03_rb,2.0844e-03_rb,2.3535e-04_rb/)
      kbo(:, 3,52, 6) = (/ &
     &4.3389e-03_rb,5.9943e-03_rb,4.8574e-03_rb,3.1382e-03_rb,2.4655e-04_rb/)
      kbo(:, 4,52, 6) = (/ &
     &6.8363e-03_rb,9.3770e-03_rb,7.4736e-03_rb,4.6599e-03_rb,2.5711e-04_rb/)
      kbo(:, 5,52, 6) = (/ &
     &1.0389e-02_rb,1.4256e-02_rb,1.1263e-02_rb,6.8242e-03_rb,2.6665e-04_rb/)
      kbo(:, 1,53, 6) = (/ &
     &1.2752e-03_rb,1.8531e-03_rb,1.6024e-03_rb,1.1428e-03_rb,1.8014e-04_rb/)
      kbo(:, 2,53, 6) = (/ &
     &2.2594e-03_rb,3.1508e-03_rb,2.6140e-03_rb,1.7561e-03_rb,1.9085e-04_rb/)
      kbo(:, 3,53, 6) = (/ &
     &3.7990e-03_rb,5.2013e-03_rb,4.1982e-03_rb,2.6961e-03_rb,2.0067e-04_rb/)
      kbo(:, 4,53, 6) = (/ &
     &6.0922e-03_rb,8.2959e-03_rb,6.5816e-03_rb,4.0835e-03_rb,2.0982e-04_rb/)
      kbo(:, 5,53, 6) = (/ &
     &9.3947e-03_rb,1.2809e-02_rb,1.0095e-02_rb,6.0875e-03_rb,2.1819e-04_rb/)
      kbo(:, 1,54, 6) = (/ &
     &9.3663e-04_rb,1.3820e-03_rb,1.2040e-03_rb,8.6807e-04_rb,1.4544e-04_rb/)
      kbo(:, 2,54, 6) = (/ &
     &1.6914e-03_rb,2.3887e-03_rb,1.9911e-03_rb,1.3482e-03_rb,1.5460e-04_rb/)
      kbo(:, 3,54, 6) = (/ &
     &2.8997e-03_rb,4.0201e-03_rb,3.2494e-03_rb,2.0953e-03_rb,1.6297e-04_rb/)
      kbo(:, 4,54, 6) = (/ &
     &4.7306e-03_rb,6.5273e-03_rb,5.1758e-03_rb,3.2189e-03_rb,1.7075e-04_rb/)
      kbo(:, 5,54, 6) = (/ &
     &7.3984e-03_rb,1.0238e-02_rb,8.0539e-03_rb,4.8622e-03_rb,1.7797e-04_rb/)
      kbo(:, 1,55, 6) = (/ &
     &6.2935e-04_rb,9.5969e-04_rb,8.4900e-04_rb,6.2465e-04_rb,1.1738e-04_rb/)
      kbo(:, 2,55, 6) = (/ &
     &1.1558e-03_rb,1.6799e-03_rb,1.4161e-03_rb,9.7472e-04_rb,1.2508e-04_rb/)
      kbo(:, 3,55, 6) = (/ &
     &2.0209e-03_rb,2.8749e-03_rb,2.3378e-03_rb,1.5244e-03_rb,1.3211e-04_rb/)
      kbo(:, 4,55, 6) = (/ &
     &3.3522e-03_rb,4.7482e-03_rb,3.7736e-03_rb,2.3654e-03_rb,1.3870e-04_rb/)
      kbo(:, 5,55, 6) = (/ &
     &5.3187e-03_rb,7.5686e-03_rb,5.9480e-03_rb,3.6106e-03_rb,1.4478e-04_rb/)
      kbo(:, 1,56, 6) = (/ &
     &4.2101e-04_rb,6.6332e-04_rb,5.9672e-04_rb,4.4860e-04_rb,9.4587e-05_rb/)
      kbo(:, 2,56, 6) = (/ &
     &7.8466e-04_rb,1.1752e-03_rb,1.0029e-03_rb,7.0318e-04_rb,1.0112e-04_rb/)
      kbo(:, 3,56, 6) = (/ &
     &1.3999e-03_rb,2.0432e-03_rb,1.6739e-03_rb,1.1053e-03_rb,1.0709e-04_rb/)
      kbo(:, 4,56, 6) = (/ &
     &2.3635e-03_rb,3.4329e-03_rb,2.7372e-03_rb,1.7318e-03_rb,1.1258e-04_rb/)
      kbo(:, 5,56, 6) = (/ &
     &3.8069e-03_rb,5.5637e-03_rb,4.3696e-03_rb,2.6702e-03_rb,1.1773e-04_rb/)
      kbo(:, 1,57, 6) = (/ &
     &2.8076e-04_rb,4.5641e-04_rb,4.1815e-04_rb,3.2142e-04_rb,7.6115e-05_rb/)
      kbo(:, 2,57, 6) = (/ &
     &5.2895e-04_rb,8.1778e-04_rb,7.0771e-04_rb,5.0616e-04_rb,8.1716e-05_rb/)
      kbo(:, 3,57, 6) = (/ &
     &9.6334e-04_rb,1.4422e-03_rb,1.1917e-03_rb,7.9834e-04_rb,8.6757e-05_rb/)
      kbo(:, 4,57, 6) = (/ &
     &1.6571e-03_rb,2.4651e-03_rb,1.9746e-03_rb,1.2622e-03_rb,9.1332e-05_rb/)
      kbo(:, 5,57, 6) = (/ &
     &2.7119e-03_rb,4.0628e-03_rb,3.1926e-03_rb,1.9661e-03_rb,9.5684e-05_rb/)
      kbo(:, 1,58, 6) = (/ &
     &4.9720e-05_rb,1.0269e-04_rb,1.2366e-04_rb,1.3663e-04_rb,6.1273e-05_rb/)
      kbo(:, 2,58, 6) = (/ &
     &9.4473e-05_rb,1.8553e-04_rb,2.1007e-04_rb,2.1548e-04_rb,6.5992e-05_rb/)
      kbo(:, 3,58, 6) = (/ &
     &1.7530e-04_rb,3.3103e-04_rb,3.5575e-04_rb,3.4010e-04_rb,7.0239e-05_rb/)
      kbo(:, 4,58, 6) = (/ &
     &3.0720e-04_rb,5.7469e-04_rb,5.9600e-04_rb,5.4082e-04_rb,7.4141e-05_rb/)
      kbo(:, 5,58, 6) = (/ &
     &5.1103e-04_rb,9.6242e-04_rb,9.7487e-04_rb,8.4973e-04_rb,7.7747e-05_rb/)
      kbo(:, 1,59, 6) = (/ &
     &4.4314e-05_rb,9.6972e-05_rb,1.1293e-04_rb,1.1582e-04_rb,4.9949e-05_rb/)
      kbo(:, 2,59, 6) = (/ &
     &8.4700e-05_rb,1.7358e-04_rb,1.8881e-04_rb,1.7917e-04_rb,5.3859e-05_rb/)
      kbo(:, 3,59, 6) = (/ &
     &1.5856e-04_rb,3.0848e-04_rb,3.1562e-04_rb,2.7815e-04_rb,5.7310e-05_rb/)
      kbo(:, 4,59, 6) = (/ &
     &2.8032e-04_rb,5.3615e-04_rb,5.2587e-04_rb,4.3688e-04_rb,6.0507e-05_rb/)
      kbo(:, 5,59, 6) = (/ &
     &4.7008e-04_rb,9.0261e-04_rb,8.5908e-04_rb,6.8167e-04_rb,6.3447e-05_rb/)
      kbo(:, 1,13, 7) = (/ &
     &3.5002e+00_rb,2.8426e+00_rb,2.2229e+00_rb,1.5940e+00_rb,4.6284e-01_rb/)
      kbo(:, 2,13, 7) = (/ &
     &5.2764e+00_rb,4.1532e+00_rb,3.0774e+00_rb,2.0245e+00_rb,4.6449e-01_rb/)
      kbo(:, 3,13, 7) = (/ &
     &7.6096e+00_rb,5.8846e+00_rb,4.2048e+00_rb,2.5664e+00_rb,4.6564e-01_rb/)
      kbo(:, 4,13, 7) = (/ &
     &1.0545e+01_rb,8.0718e+00_rb,5.6353e+00_rb,3.2573e+00_rb,4.6805e-01_rb/)
      kbo(:, 5,13, 7) = (/ &
     &1.4101e+01_rb,1.0726e+01_rb,7.3776e+00_rb,4.1071e+00_rb,4.6959e-01_rb/)
      kbo(:, 1,14, 7) = (/ &
     &1.9332e+00_rb,1.6833e+00_rb,1.4584e+00_rb,1.1419e+00_rb,4.2549e-01_rb/)
      kbo(:, 2,14, 7) = (/ &
     &2.9208e+00_rb,2.4028e+00_rb,1.9264e+00_rb,1.4148e+00_rb,4.2845e-01_rb/)
      kbo(:, 3,14, 7) = (/ &
     &4.2111e+00_rb,3.3491e+00_rb,2.5401e+00_rb,1.7447e+00_rb,4.2997e-01_rb/)
      kbo(:, 4,14, 7) = (/ &
     &5.8219e+00_rb,4.5369e+00_rb,3.3137e+00_rb,2.1374e+00_rb,4.3233e-01_rb/)
      kbo(:, 5,14, 7) = (/ &
     &7.7805e+00_rb,5.9890e+00_rb,4.2588e+00_rb,2.5955e+00_rb,4.3475e-01_rb/)
      kbo(:, 1,15, 7) = (/ &
     &1.0958e+00_rb,1.0674e+00_rb,1.0130e+00_rb,8.4961e-01_rb,3.8730e-01_rb/)
      kbo(:, 2,15, 7) = (/ &
     &1.6536e+00_rb,1.4690e+00_rb,1.3041e+00_rb,1.0292e+00_rb,3.9101e-01_rb/)
      kbo(:, 3,15, 7) = (/ &
     &2.3745e+00_rb,1.9914e+00_rb,1.6603e+00_rb,1.2452e+00_rb,3.9344e-01_rb/)
      kbo(:, 4,15, 7) = (/ &
     &3.2750e+00_rb,2.6468e+00_rb,2.0887e+00_rb,1.4951e+00_rb,3.9613e-01_rb/)
      kbo(:, 5,15, 7) = (/ &
     &4.3749e+00_rb,3.4526e+00_rb,2.6092e+00_rb,1.7780e+00_rb,3.9900e-01_rb/)
      kbo(:, 1,16, 7) = (/ &
     &7.4165e-01_rb,8.0405e-01_rb,7.9283e-01_rb,6.9114e-01_rb,3.4911e-01_rb/)
      kbo(:, 2,16, 7) = (/ &
     &1.1136e+00_rb,1.0786e+00_rb,1.0091e+00_rb,8.2862e-01_rb,3.5314e-01_rb/)
      kbo(:, 3,16, 7) = (/ &
     &1.5923e+00_rb,1.4242e+00_rb,1.2681e+00_rb,9.9143e-01_rb,3.5636e-01_rb/)
      kbo(:, 4,16, 7) = (/ &
     &2.1926e+00_rb,1.8566e+00_rb,1.5712e+00_rb,1.1771e+00_rb,3.6013e-01_rb/)
      kbo(:, 5,16, 7) = (/ &
     &2.9285e+00_rb,2.3901e+00_rb,1.9279e+00_rb,1.3859e+00_rb,3.6399e-01_rb/)
      kbo(:, 1,17, 7) = (/ &
     &5.1887e-01_rb,6.2476e-01_rb,6.3724e-01_rb,5.7083e-01_rb,3.1167e-01_rb/)
      kbo(:, 2,17, 7) = (/ &
     &7.7486e-01_rb,8.2693e-01_rb,8.0143e-01_rb,6.7854e-01_rb,3.1621e-01_rb/)
      kbo(:, 3,17, 7) = (/ &
     &1.1045e+00_rb,1.0746e+00_rb,9.9649e-01_rb,8.0449e-01_rb,3.2014e-01_rb/)
      kbo(:, 4,17, 7) = (/ &
     &1.5180e+00_rb,1.3742e+00_rb,1.2231e+00_rb,9.4686e-01_rb,3.2461e-01_rb/)
      kbo(:, 5,17, 7) = (/ &
     &2.0257e+00_rb,1.7386e+00_rb,1.4858e+00_rb,1.1069e+00_rb,3.2955e-01_rb/)
      kbo(:, 1,18, 7) = (/ &
     &3.7097e-01_rb,4.9432e-01_rb,5.1840e-01_rb,4.7504e-01_rb,2.7614e-01_rb/)
      kbo(:, 2,18, 7) = (/ &
     &5.5115e-01_rb,6.4712e-01_rb,6.4514e-01_rb,5.6072e-01_rb,2.8084e-01_rb/)
      kbo(:, 3,18, 7) = (/ &
     &7.8296e-01_rb,8.3225e-01_rb,7.9447e-01_rb,6.5954e-01_rb,2.8571e-01_rb/)
      kbo(:, 4,18, 7) = (/ &
     &1.0738e+00_rb,1.0535e+00_rb,9.6779e-01_rb,7.7088e-01_rb,2.9112e-01_rb/)
      kbo(:, 5,18, 7) = (/ &
     &1.4335e+00_rb,1.3170e+00_rb,1.1664e+00_rb,8.9595e-01_rb,2.9703e-01_rb/)
      kbo(:, 1,19, 7) = (/ &
     &2.5985e-01_rb,3.8651e-01_rb,4.1602e-01_rb,3.9308e-01_rb,2.4307e-01_rb/)
      kbo(:, 2,19, 7) = (/ &
     &3.8422e-01_rb,5.0067e-01_rb,5.1275e-01_rb,4.5968e-01_rb,2.4809e-01_rb/)
      kbo(:, 3,19, 7) = (/ &
     &5.4420e-01_rb,6.3764e-01_rb,6.2617e-01_rb,5.3586e-01_rb,2.5358e-01_rb/)
      kbo(:, 4,19, 7) = (/ &
     &7.4603e-01_rb,7.9981e-01_rb,7.5671e-01_rb,6.2198e-01_rb,2.5998e-01_rb/)
      kbo(:, 5,19, 7) = (/ &
     &9.9708e-01_rb,9.9236e-01_rb,9.0617e-01_rb,7.1829e-01_rb,2.6615e-01_rb/)
      kbo(:, 1,20, 7) = (/ &
     &1.9760e-01_rb,3.1836e-01_rb,3.4770e-01_rb,3.3477e-01_rb,2.1277e-01_rb/)
      kbo(:, 2,20, 7) = (/ &
     &2.9095e-01_rb,4.0925e-01_rb,4.2592e-01_rb,3.8937e-01_rb,2.1827e-01_rb/)
      kbo(:, 3,20, 7) = (/ &
     &4.1127e-01_rb,5.1768e-01_rb,5.1739e-01_rb,4.5140e-01_rb,2.2458e-01_rb/)
      kbo(:, 4,20, 7) = (/ &
     &5.6427e-01_rb,6.4625e-01_rb,6.2247e-01_rb,5.2174e-01_rb,2.3101e-01_rb/)
      kbo(:, 5,20, 7) = (/ &
     &7.5526e-01_rb,7.9836e-01_rb,7.4304e-01_rb,6.0027e-01_rb,2.3711e-01_rb/)
      kbo(:, 1,21, 7) = (/ &
     &1.5484e-01_rb,2.6582e-01_rb,2.9382e-01_rb,2.8708e-01_rb,1.8550e-01_rb/)
      kbo(:, 2,21, 7) = (/ &
     &2.2722e-01_rb,3.3963e-01_rb,3.5832e-01_rb,3.3257e-01_rb,1.9141e-01_rb/)
      kbo(:, 3,21, 7) = (/ &
     &3.2108e-01_rb,4.2782e-01_rb,4.3333e-01_rb,3.8435e-01_rb,1.9782e-01_rb/)
      kbo(:, 4,21, 7) = (/ &
     &4.4098e-01_rb,5.3304e-01_rb,5.1997e-01_rb,4.4275e-01_rb,2.0379e-01_rb/)
      kbo(:, 5,21, 7) = (/ &
     &5.9030e-01_rb,6.5709e-01_rb,6.1970e-01_rb,5.0864e-01_rb,2.0956e-01_rb/)
      kbo(:, 1,22, 7) = (/ &
     &1.2753e-01_rb,2.2781e-01_rb,2.5360e-01_rb,2.4962e-01_rb,1.6164e-01_rb/)
      kbo(:, 2,22, 7) = (/ &
     &1.8637e-01_rb,2.8970e-01_rb,3.0789e-01_rb,2.8854e-01_rb,1.6761e-01_rb/)
      kbo(:, 3,22, 7) = (/ &
     &2.6299e-01_rb,3.6404e-01_rb,3.7114e-01_rb,3.3280e-01_rb,1.7333e-01_rb/)
      kbo(:, 4,22, 7) = (/ &
     &3.6059e-01_rb,4.5267e-01_rb,4.4472e-01_rb,3.8285e-01_rb,1.7883e-01_rb/)
      kbo(:, 5,22, 7) = (/ &
     &4.8104e-01_rb,5.5702e-01_rb,5.2923e-01_rb,4.3976e-01_rb,1.8428e-01_rb/)
      kbo(:, 1,23, 7) = (/ &
     &1.0589e-01_rb,1.9529e-01_rb,2.1876e-01_rb,2.1679e-01_rb,1.4037e-01_rb/)
      kbo(:, 2,23, 7) = (/ &
     &1.5420e-01_rb,2.4758e-01_rb,2.6461e-01_rb,2.5042e-01_rb,1.4579e-01_rb/)
      kbo(:, 3,23, 7) = (/ &
     &2.1702e-01_rb,3.1068e-01_rb,3.1854e-01_rb,2.8852e-01_rb,1.5094e-01_rb/)
      kbo(:, 4,23, 7) = (/ &
     &2.9661e-01_rb,3.8564e-01_rb,3.8128e-01_rb,3.3195e-01_rb,1.5614e-01_rb/)
      kbo(:, 5,23, 7) = (/ &
     &3.9397e-01_rb,4.7355e-01_rb,4.5341e-01_rb,3.8121e-01_rb,1.6132e-01_rb/)
      kbo(:, 1,24, 7) = (/ &
     &8.7788e-02_rb,1.6668e-01_rb,1.8799e-01_rb,1.8780e-01_rb,1.2140e-01_rb/)
      kbo(:, 2,24, 7) = (/ &
     &1.2734e-01_rb,2.1083e-01_rb,2.2703e-01_rb,2.1652e-01_rb,1.2618e-01_rb/)
      kbo(:, 3,24, 7) = (/ &
     &1.7857e-01_rb,2.6413e-01_rb,2.7303e-01_rb,2.4936e-01_rb,1.3098e-01_rb/)
      kbo(:, 4,24, 7) = (/ &
     &2.4296e-01_rb,3.2721e-01_rb,3.2661e-01_rb,2.8698e-01_rb,1.3586e-01_rb/)
      kbo(:, 5,24, 7) = (/ &
     &3.2098e-01_rb,4.0055e-01_rb,3.8785e-01_rb,3.2936e-01_rb,1.4075e-01_rb/)
      kbo(:, 1,25, 7) = (/ &
     &7.4114e-02_rb,1.4357e-01_rb,1.6268e-01_rb,1.6299e-01_rb,1.0454e-01_rb/)
      kbo(:, 2,25, 7) = (/ &
     &1.0696e-01_rb,1.8148e-01_rb,1.9639e-01_rb,1.8781e-01_rb,1.0890e-01_rb/)
      kbo(:, 3,25, 7) = (/ &
     &1.4921e-01_rb,2.2686e-01_rb,2.3627e-01_rb,2.1648e-01_rb,1.1337e-01_rb/)
      kbo(:, 4,25, 7) = (/ &
     &2.0169e-01_rb,2.8030e-01_rb,2.8233e-01_rb,2.4913e-01_rb,1.1793e-01_rb/)
      kbo(:, 5,25, 7) = (/ &
     &2.6511e-01_rb,3.4240e-01_rb,3.3494e-01_rb,2.8598e-01_rb,1.2261e-01_rb/)
      kbo(:, 1,26, 7) = (/ &
     &6.4102e-02_rb,1.2562e-01_rb,1.4220e-01_rb,1.4209e-01_rb,8.9836e-02_rb/)
      kbo(:, 2,26, 7) = (/ &
     &9.1903e-02_rb,1.5853e-01_rb,1.7179e-01_rb,1.6391e-01_rb,9.3830e-02_rb/)
      kbo(:, 3,26, 7) = (/ &
     &1.2732e-01_rb,1.9782e-01_rb,2.0670e-01_rb,1.8917e-01_rb,9.8002e-02_rb/)
      kbo(:, 4,26, 7) = (/ &
     &1.7095e-01_rb,2.4388e-01_rb,2.4691e-01_rb,2.1801e-01_rb,1.0234e-01_rb/)
      kbo(:, 5,26, 7) = (/ &
     &2.2376e-01_rb,2.9731e-01_rb,2.9277e-01_rb,2.5044e-01_rb,1.0674e-01_rb/)
      kbo(:, 1,27, 7) = (/ &
     &5.7622e-02_rb,1.1249e-01_rb,1.2636e-01_rb,1.2511e-01_rb,7.7055e-02_rb/)
      kbo(:, 2,27, 7) = (/ &
     &8.1983e-02_rb,1.4196e-01_rb,1.5312e-01_rb,1.4485e-01_rb,8.0791e-02_rb/)
      kbo(:, 3,27, 7) = (/ &
     &1.1275e-01_rb,1.7697e-01_rb,1.8437e-01_rb,1.6781e-01_rb,8.4713e-02_rb/)
      kbo(:, 4,27, 7) = (/ &
     &1.5055e-01_rb,2.1794e-01_rb,2.2039e-01_rb,1.9390e-01_rb,8.8731e-02_rb/)
      kbo(:, 5,27, 7) = (/ &
     &1.9638e-01_rb,2.6555e-01_rb,2.6132e-01_rb,2.2312e-01_rb,9.2655e-02_rb/)
      kbo(:, 1,28, 7) = (/ &
     &5.3140e-02_rb,1.0241e-01_rb,1.1379e-01_rb,1.1112e-01_rb,6.6059e-02_rb/)
      kbo(:, 2,28, 7) = (/ &
     &7.5045e-02_rb,1.2928e-01_rb,1.3817e-01_rb,1.2935e-01_rb,6.9529e-02_rb/)
      kbo(:, 3,28, 7) = (/ &
     &1.0252e-01_rb,1.6119e-01_rb,1.6672e-01_rb,1.5050e-01_rb,7.3142e-02_rb/)
      kbo(:, 4,28, 7) = (/ &
     &1.3625e-01_rb,1.9862e-01_rb,1.9953e-01_rb,1.7444e-01_rb,7.6746e-02_rb/)
      kbo(:, 5,28, 7) = (/ &
     &1.7752e-01_rb,2.4214e-01_rb,2.3714e-01_rb,2.0133e-01_rb,7.9941e-02_rb/)
      kbo(:, 1,29, 7) = (/ &
     &5.2099e-02_rb,9.6941e-02_rb,1.0563e-01_rb,1.0097e-01_rb,5.6668e-02_rb/)
      kbo(:, 2,29, 7) = (/ &
     &7.3009e-02_rb,1.2266e-01_rb,1.2890e-01_rb,1.1850e-01_rb,5.9859e-02_rb/)
      kbo(:, 3,29, 7) = (/ &
     &9.9110e-02_rb,1.5322e-01_rb,1.5608e-01_rb,1.3860e-01_rb,6.3089e-02_rb/)
      kbo(:, 4,29, 7) = (/ &
     &1.3136e-01_rb,1.8918e-01_rb,1.8750e-01_rb,1.6146e-01_rb,6.6000e-02_rb/)
      kbo(:, 5,29, 7) = (/ &
     &1.7133e-01_rb,2.3120e-01_rb,2.2375e-01_rb,1.8721e-01_rb,6.8384e-02_rb/)
      kbo(:, 1,30, 7) = (/ &
     &5.2298e-02_rb,9.3477e-02_rb,9.9631e-02_rb,9.3041e-02_rb,4.8653e-02_rb/)
      kbo(:, 2,30, 7) = (/ &
     &7.2744e-02_rb,1.1854e-01_rb,1.2224e-01_rb,1.0996e-01_rb,5.1507e-02_rb/)
      kbo(:, 3,30, 7) = (/ &
     &9.8307e-02_rb,1.4854e-01_rb,1.4866e-01_rb,1.2943e-01_rb,5.4141e-02_rb/)
      kbo(:, 4,30, 7) = (/ &
     &1.3030e-01_rb,1.8393e-01_rb,1.7946e-01_rb,1.5167e-01_rb,5.6316e-02_rb/)
      kbo(:, 5,30, 7) = (/ &
     &1.7017e-01_rb,2.2583e-01_rb,2.1516e-01_rb,1.7690e-01_rb,5.8003e-02_rb/)
      kbo(:, 1,31, 7) = (/ &
     &5.5237e-02_rb,9.3530e-02_rb,9.7080e-02_rb,8.7885e-02_rb,4.1801e-02_rb/)
      kbo(:, 2,31, 7) = (/ &
     &7.6342e-02_rb,1.1917e-01_rb,1.1987e-01_rb,1.0485e-01_rb,4.4162e-02_rb/)
      kbo(:, 3,31, 7) = (/ &
     &1.0297e-01_rb,1.4987e-01_rb,1.4665e-01_rb,1.2435e-01_rb,4.6139e-02_rb/)
      kbo(:, 4,31, 7) = (/ &
     &1.3657e-01_rb,1.8654e-01_rb,1.7810e-01_rb,1.4680e-01_rb,4.7704e-02_rb/)
      kbo(:, 5,31, 7) = (/ &
     &1.7842e-01_rb,2.3057e-01_rb,2.1509e-01_rb,1.7253e-01_rb,4.8947e-02_rb/)
      kbo(:, 1,32, 7) = (/ &
     &5.9027e-02_rb,9.4969e-02_rb,9.5943e-02_rb,8.4194e-02_rb,3.5812e-02_rb/)
      kbo(:, 2,32, 7) = (/ &
     &8.1201e-02_rb,1.2151e-01_rb,1.1922e-01_rb,1.0130e-01_rb,3.7617e-02_rb/)
      kbo(:, 3,32, 7) = (/ &
     &1.0955e-01_rb,1.5358e-01_rb,1.4681e-01_rb,1.2117e-01_rb,3.9058e-02_rb/)
      kbo(:, 4,32, 7) = (/ &
     &1.4540e-01_rb,1.9256e-01_rb,1.7966e-01_rb,1.4421e-01_rb,4.0236e-02_rb/)
      kbo(:, 5,32, 7) = (/ &
     &1.8982e-01_rb,2.3938e-01_rb,2.1889e-01_rb,1.7084e-01_rb,4.1181e-02_rb/)
      kbo(:, 1,33, 7) = (/ &
     &6.4219e-02_rb,9.8255e-02_rb,9.6625e-02_rb,8.2099e-02_rb,3.0519e-02_rb/)
      kbo(:, 2,33, 7) = (/ &
     &8.8246e-02_rb,1.2633e-01_rb,1.2088e-01_rb,9.9705e-02_rb,3.1855e-02_rb/)
      kbo(:, 3,33, 7) = (/ &
     &1.1912e-01_rb,1.6083e-01_rb,1.5001e-01_rb,1.2028e-01_rb,3.2953e-02_rb/)
      kbo(:, 4,33, 7) = (/ &
     &1.5802e-01_rb,2.0300e-01_rb,1.8528e-01_rb,1.4442e-01_rb,3.3850e-02_rb/)
      kbo(:, 5,33, 7) = (/ &
     &2.0596e-01_rb,2.5392e-01_rb,2.2791e-01_rb,1.7303e-01_rb,3.4585e-02_rb/)
      kbo(:, 1,34, 7) = (/ &
     &6.7485e-02_rb,9.9620e-02_rb,9.5868e-02_rb,7.9332e-02_rb,2.5830e-02_rb/)
      kbo(:, 2,34, 7) = (/ &
     &9.2815e-02_rb,1.2890e-01_rb,1.2075e-01_rb,9.7213e-02_rb,2.6848e-02_rb/)
      kbo(:, 3,34, 7) = (/ &
     &1.2537e-01_rb,1.6529e-01_rb,1.5122e-01_rb,1.1822e-01_rb,2.7699e-02_rb/)
      kbo(:, 4,34, 7) = (/ &
     &1.6617e-01_rb,2.0989e-01_rb,1.8860e-01_rb,1.4346e-01_rb,2.8411e-02_rb/)
      kbo(:, 5,34, 7) = (/ &
     &2.1641e-01_rb,2.6338e-01_rb,2.3365e-01_rb,1.7367e-01_rb,2.9006e-02_rb/)
      kbo(:, 1,35, 7) = (/ &
     &6.9002e-02_rb,9.9168e-02_rb,9.3611e-02_rb,7.5846e-02_rb,2.1704e-02_rb/)
      kbo(:, 2,35, 7) = (/ &
     &9.5177e-02_rb,1.2935e-01_rb,1.1903e-01_rb,9.3690e-02_rb,2.2517e-02_rb/)
      kbo(:, 3,35, 7) = (/ &
     &1.2872e-01_rb,1.6713e-01_rb,1.5066e-01_rb,1.1518e-01_rb,2.3202e-02_rb/)
      kbo(:, 4,35, 7) = (/ &
     &1.7076e-01_rb,2.1347e-01_rb,1.8964e-01_rb,1.4135e-01_rb,2.3785e-02_rb/)
      kbo(:, 5,35, 7) = (/ &
     &2.2264e-01_rb,2.6908e-01_rb,2.3665e-01_rb,1.7298e-01_rb,2.4252e-02_rb/)
      kbo(:, 1,36, 7) = (/ &
     &6.7865e-02_rb,9.5879e-02_rb,8.9211e-02_rb,7.0981e-02_rb,1.8133e-02_rb/)
      kbo(:, 2,36, 7) = (/ &
     &9.4007e-02_rb,1.2625e-01_rb,1.1466e-01_rb,8.8573e-02_rb,1.8797e-02_rb/)
      kbo(:, 3,36, 7) = (/ &
     &1.2751e-01_rb,1.6450e-01_rb,1.4669e-01_rb,1.1023e-01_rb,1.9364e-02_rb/)
      kbo(:, 4,36, 7) = (/ &
     &1.6969e-01_rb,2.1129e-01_rb,1.8634e-01_rb,1.3684e-01_rb,1.9848e-02_rb/)
      kbo(:, 5,36, 7) = (/ &
     &2.2199e-01_rb,2.6776e-01_rb,2.3416e-01_rb,1.6920e-01_rb,2.0253e-02_rb/)
      kbo(:, 1,37, 7) = (/ &
     &6.1910e-02_rb,8.7392e-02_rb,8.0722e-02_rb,6.3503e-02_rb,1.5050e-02_rb/)
      kbo(:, 2,37, 7) = (/ &
     &8.6402e-02_rb,1.1634e-01_rb,1.0496e-01_rb,8.0172e-02_rb,1.5605e-02_rb/)
      kbo(:, 3,37, 7) = (/ &
     &1.1792e-01_rb,1.5301e-01_rb,1.3575e-01_rb,1.0102e-01_rb,1.6089e-02_rb/)
      kbo(:, 4,37, 7) = (/ &
     &1.5786e-01_rb,1.9811e-01_rb,1.7416e-01_rb,1.2688e-01_rb,1.6504e-02_rb/)
      kbo(:, 5,37, 7) = (/ &
     &2.0776e-01_rb,2.5283e-01_rb,2.2073e-01_rb,1.5856e-01_rb,1.6861e-02_rb/)
      kbo(:, 1,38, 7) = (/ &
     &5.7180e-02_rb,8.0434e-02_rb,7.3641e-02_rb,5.7228e-02_rb,1.2479e-02_rb/)
      kbo(:, 2,38, 7) = (/ &
     &8.0361e-02_rb,1.0828e-01_rb,9.6955e-02_rb,7.3200e-02_rb,1.2947e-02_rb/)
      kbo(:, 3,38, 7) = (/ &
     &1.1043e-01_rb,1.4379e-01_rb,1.2688e-01_rb,9.3375e-02_rb,1.3365e-02_rb/)
      kbo(:, 4,38, 7) = (/ &
     &1.4883e-01_rb,1.8786e-01_rb,1.6450e-01_rb,1.1879e-01_rb,1.3732e-02_rb/)
      kbo(:, 5,38, 7) = (/ &
     &1.9707e-01_rb,2.4129e-01_rb,2.1013e-01_rb,1.4998e-01_rb,1.4049e-02_rb/)
      kbo(:, 1,39, 7) = (/ &
     &5.4044e-02_rb,7.5469e-02_rb,6.8324e-02_rb,5.2315e-02_rb,1.0342e-02_rb/)
      kbo(:, 2,39, 7) = (/ &
     &7.6525e-02_rb,1.0279e-01_rb,9.1154e-02_rb,6.7832e-02_rb,1.0747e-02_rb/)
      kbo(:, 3,39, 7) = (/ &
     &1.0591e-01_rb,1.3783e-01_rb,1.2080e-01_rb,8.7775e-02_rb,1.1116e-02_rb/)
      kbo(:, 4,39, 7) = (/ &
     &1.4369e-01_rb,1.8148e-01_rb,1.5809e-01_rb,1.1309e-01_rb,1.1438e-02_rb/)
      kbo(:, 5,39, 7) = (/ &
     &1.9144e-01_rb,2.3501e-01_rb,2.0393e-01_rb,1.4445e-01_rb,1.1716e-02_rb/)
      kbo(:, 1,40, 7) = (/ &
     &4.6217e-02_rb,6.5370e-02_rb,5.9012e-02_rb,4.4989e-02_rb,8.5358e-03_rb/)
      kbo(:, 2,40, 7) = (/ &
     &6.6114e-02_rb,9.0173e-02_rb,7.9777e-02_rb,5.9045e-02_rb,8.8901e-03_rb/)
      kbo(:, 3,40, 7) = (/ &
     &9.2339e-02_rb,1.2222e-01_rb,1.0699e-01_rb,7.7378e-02_rb,9.2084e-03_rb/)
      kbo(:, 4,40, 7) = (/ &
     &1.2639e-01_rb,1.6256e-01_rb,1.4164e-01_rb,1.0101e-01_rb,9.4930e-03_rb/)
      kbo(:, 5,40, 7) = (/ &
     &1.6986e-01_rb,2.1255e-01_rb,1.8467e-01_rb,1.3055e-01_rb,9.7467e-03_rb/)
      kbo(:, 1,41, 7) = (/ &
     &3.9155e-02_rb,5.6183e-02_rb,5.0596e-02_rb,3.8424e-02_rb,7.0383e-03_rb/)
      kbo(:, 2,41, 7) = (/ &
     &5.6659e-02_rb,7.8503e-02_rb,6.9340e-02_rb,5.1085e-02_rb,7.3470e-03_rb/)
      kbo(:, 3,41, 7) = (/ &
     &7.9928e-02_rb,1.0766e-01_rb,9.4225e-02_rb,6.7858e-02_rb,7.6230e-03_rb/)
      kbo(:, 4,41, 7) = (/ &
     &1.1042e-01_rb,1.4475e-01_rb,1.2626e-01_rb,8.9781e-02_rb,7.8762e-03_rb/)
      kbo(:, 5,41, 7) = (/ &
     &1.4975e-01_rb,1.9111e-01_rb,1.6637e-01_rb,1.1739e-01_rb,8.1039e-03_rb/)
      kbo(:, 1,42, 7) = (/ &
     &3.3442e-02_rb,4.8574e-02_rb,4.3613e-02_rb,3.2930e-02_rb,5.8050e-03_rb/)
      kbo(:, 2,42, 7) = (/ &
     &4.8965e-02_rb,6.8792e-02_rb,6.0620e-02_rb,4.4408e-02_rb,6.0682e-03_rb/)
      kbo(:, 3,42, 7) = (/ &
     &6.9803e-02_rb,9.5533e-02_rb,8.3532e-02_rb,5.9826e-02_rb,6.3116e-03_rb/)
      kbo(:, 4,42, 7) = (/ &
     &9.7360e-02_rb,1.2988e-01_rb,1.1328e-01_rb,8.0240e-02_rb,6.5330e-03_rb/)
      kbo(:, 5,42, 7) = (/ &
     &1.3333e-01_rb,1.7321e-01_rb,1.5102e-01_rb,1.0628e-01_rb,6.7345e-03_rb/)
      kbo(:, 1,43, 7) = (/ &
     &2.8074e-02_rb,4.1293e-02_rb,3.7000e-02_rb,2.7812e-02_rb,4.7712e-03_rb/)
      kbo(:, 2,43, 7) = (/ &
     &4.1661e-02_rb,5.9380e-02_rb,5.2205e-02_rb,3.8045e-02_rb,4.9985e-03_rb/)
      kbo(:, 3,43, 7) = (/ &
     &6.0156e-02_rb,8.3670e-02_rb,7.3060e-02_rb,5.2075e-02_rb,5.2119e-03_rb/)
      kbo(:, 4,43, 7) = (/ &
     &8.4838e-02_rb,1.1520e-01_rb,1.0043e-01_rb,7.0829e-02_rb,5.4068e-03_rb/)
      kbo(:, 5,43, 7) = (/ &
     &1.1748e-01_rb,1.5547e-01_rb,1.3568e-01_rb,9.5209e-02_rb,5.5907e-03_rb/)
      kbo(:, 1,44, 7) = (/ &
     &2.3517e-02_rb,3.4984e-02_rb,3.1261e-02_rb,2.3397e-02_rb,3.9133e-03_rb/)
      kbo(:, 2,44, 7) = (/ &
     &3.5440e-02_rb,5.1145e-02_rb,4.4838e-02_rb,3.2508e-02_rb,4.1113e-03_rb/)
      kbo(:, 3,44, 7) = (/ &
     &5.1856e-02_rb,7.3218e-02_rb,6.3748e-02_rb,4.5187e-02_rb,4.2980e-03_rb/)
      kbo(:, 4,44, 7) = (/ &
     &7.4068e-02_rb,1.0225e-01_rb,8.9058e-02_rb,6.2496e-02_rb,4.4682e-03_rb/)
      kbo(:, 5,44, 7) = (/ &
     &1.0378e-01_rb,1.3965e-01_rb,1.2187e-01_rb,8.5218e-02_rb,4.6319e-03_rb/)
      kbo(:, 1,45, 7) = (/ &
     &1.9987e-02_rb,2.9948e-02_rb,2.6638e-02_rb,1.9815e-02_rb,3.2076e-03_rb/)
      kbo(:, 2,45, 7) = (/ &
     &3.0595e-02_rb,4.4514e-02_rb,3.8859e-02_rb,2.7978e-02_rb,3.3798e-03_rb/)
      kbo(:, 3,45, 7) = (/ &
     &4.5403e-02_rb,6.4811e-02_rb,5.6194e-02_rb,3.9557e-02_rb,3.5406e-03_rb/)
      kbo(:, 4,45, 7) = (/ &
     &6.5696e-02_rb,9.1866e-02_rb,7.9797e-02_rb,5.5645e-02_rb,3.6912e-03_rb/)
      kbo(:, 5,45, 7) = (/ &
     &9.3194e-02_rb,1.2710e-01_rb,1.1078e-01_rb,7.7097e-02_rb,3.8387e-03_rb/)
      kbo(:, 1,46, 7) = (/ &
     &1.6883e-02_rb,2.5455e-02_rb,2.2540e-02_rb,1.6677e-02_rb,2.6234e-03_rb/)
      kbo(:, 2,46, 7) = (/ &
     &2.6297e-02_rb,3.8534e-02_rb,3.3486e-02_rb,2.3954e-02_rb,2.7725e-03_rb/)
      kbo(:, 3,46, 7) = (/ &
     &3.9636e-02_rb,5.7120e-02_rb,4.9278e-02_rb,3.4454e-02_rb,2.9133e-03_rb/)
      kbo(:, 4,46, 7) = (/ &
     &5.8158e-02_rb,8.2259e-02_rb,7.1230e-02_rb,4.9351e-02_rb,3.0483e-03_rb/)
      kbo(:, 5,46, 7) = (/ &
     &8.3561e-02_rb,1.1543e-01_rb,1.0042e-01_rb,6.9545e-02_rb,3.1794e-03_rb/)
      kbo(:, 1,47, 7) = (/ &
     &1.3750e-02_rb,2.0931e-02_rb,1.8489e-02_rb,1.3652e-02_rb,2.1369e-03_rb/)
      kbo(:, 2,47, 7) = (/ &
     &2.1838e-02_rb,3.2305e-02_rb,2.7989e-02_rb,1.9957e-02_rb,2.2678e-03_rb/)
      kbo(:, 3,47, 7) = (/ &
     &3.3497e-02_rb,4.8835e-02_rb,4.1985e-02_rb,2.9214e-02_rb,2.3899e-03_rb/)
      kbo(:, 4,47, 7) = (/ &
     &4.9929e-02_rb,7.1624e-02_rb,6.1836e-02_rb,4.2605e-02_rb,2.5094e-03_rb/)
      kbo(:, 5,47, 7) = (/ &
     &7.2757e-02_rb,1.0217e-01_rb,8.8738e-02_rb,6.1180e-02_rb,2.6247e-03_rb/)
      kbo(:, 1,48, 7) = (/ &
     &1.1414e-02_rb,1.7437e-02_rb,1.5334e-02_rb,1.1272e-02_rb,1.7386e-03_rb/)
      kbo(:, 2,48, 7) = (/ &
     &1.8495e-02_rb,2.7458e-02_rb,2.3694e-02_rb,1.6798e-02_rb,1.8526e-03_rb/)
      kbo(:, 3,48, 7) = (/ &
     &2.8896e-02_rb,4.2355e-02_rb,3.6253e-02_rb,2.5061e-02_rb,1.9592e-03_rb/)
      kbo(:, 4,48, 7) = (/ &
     &4.3780e-02_rb,6.3320e-02_rb,5.4439e-02_rb,3.7255e-02_rb,2.0628e-03_rb/)
      kbo(:, 5,48, 7) = (/ &
     &6.4736e-02_rb,9.1889e-02_rb,7.9609e-02_rb,5.4563e-02_rb,2.1644e-03_rb/)
      kbo(:, 1,49, 7) = (/ &
     &9.7792e-03_rb,1.4862e-02_rb,1.2975e-02_rb,9.4608e-03_rb,1.4121e-03_rb/)
      kbo(:, 2,49, 7) = (/ &
     &1.6175e-02_rb,2.3898e-02_rb,2.0488e-02_rb,1.4406e-02_rb,1.5117e-03_rb/)
      kbo(:, 3,49, 7) = (/ &
     &2.5763e-02_rb,3.7620e-02_rb,3.1995e-02_rb,2.1936e-02_rb,1.6039e-03_rb/)
      kbo(:, 4,49, 7) = (/ &
     &3.9714e-02_rb,5.7411e-02_rb,4.9041e-02_rb,3.3299e-02_rb,1.6946e-03_rb/)
      kbo(:, 5,49, 7) = (/ &
     &5.9638e-02_rb,8.4793e-02_rb,7.3080e-02_rb,4.9749e-02_rb,1.7852e-03_rb/)
      kbo(:, 1,50, 7) = (/ &
     &8.0131e-03_rb,1.2216e-02_rb,1.0631e-02_rb,7.7241e-03_rb,1.1467e-03_rb/)
      kbo(:, 2,50, 7) = (/ &
     &1.3522e-02_rb,2.0069e-02_rb,1.7141e-02_rb,1.2008e-02_rb,1.2327e-03_rb/)
      kbo(:, 3,50, 7) = (/ &
     &2.1961e-02_rb,3.2239e-02_rb,2.7314e-02_rb,1.8640e-02_rb,1.3124e-03_rb/)
      kbo(:, 4,50, 7) = (/ &
     &3.4440e-02_rb,5.0209e-02_rb,4.2737e-02_rb,2.8885e-02_rb,1.3920e-03_rb/)
      kbo(:, 5,50, 7) = (/ &
     &5.2495e-02_rb,7.5521e-02_rb,6.4947e-02_rb,4.4030e-02_rb,1.4733e-03_rb/)
      kbo(:, 1,51, 7) = (/ &
     &6.4036e-03_rb,9.8290e-03_rb,8.5406e-03_rb,6.2032e-03_rb,9.3043e-04_rb/)
      kbo(:, 2,51, 7) = (/ &
     &1.1026e-02_rb,1.6501e-02_rb,1.4064e-02_rb,9.8397e-03_rb,1.0045e-03_rb/)
      kbo(:, 3,51, 7) = (/ &
     &1.8273e-02_rb,2.7056e-02_rb,2.2864e-02_rb,1.5565e-02_rb,1.0742e-03_rb/)
      kbo(:, 4,51, 7) = (/ &
     &2.9163e-02_rb,4.3028e-02_rb,3.6523e-02_rb,2.4601e-02_rb,1.1437e-03_rb/)
      kbo(:, 5,51, 7) = (/ &
     &4.5145e-02_rb,6.5958e-02_rb,5.6628e-02_rb,3.8264e-02_rb,1.2140e-03_rb/)
      kbo(:, 1,52, 7) = (/ &
     &5.2190e-03_rb,8.0088e-03_rb,6.9367e-03_rb,5.0245e-03_rb,7.5434e-04_rb/)
      kbo(:, 2,52, 7) = (/ &
     &9.1694e-03_rb,1.3764e-02_rb,1.1684e-02_rb,8.1486e-03_rb,8.1834e-04_rb/)
      kbo(:, 3,52, 7) = (/ &
     &1.5524e-02_rb,2.3049e-02_rb,1.9394e-02_rb,1.3151e-02_rb,8.7913e-04_rb/)
      kbo(:, 4,52, 7) = (/ &
     &2.5232e-02_rb,3.7442e-02_rb,3.1647e-02_rb,2.1215e-02_rb,9.3902e-04_rb/)
      kbo(:, 5,52, 7) = (/ &
     &3.9698e-02_rb,5.8529e-02_rb,5.0101e-02_rb,3.3714e-02_rb,1.0010e-03_rb/)
      kbo(:, 1,53, 7) = (/ &
     &4.3882e-03_rb,6.6717e-03_rb,5.7454e-03_rb,4.1343e-03_rb,6.1055e-04_rb/)
      kbo(:, 2,53, 7) = (/ &
     &7.8762e-03_rb,1.1762e-02_rb,9.9163e-03_rb,6.8660e-03_rb,6.6611e-04_rb/)
      kbo(:, 3,53, 7) = (/ &
     &1.3621e-02_rb,2.0115e-02_rb,1.6825e-02_rb,1.1330e-02_rb,7.1874e-04_rb/)
      kbo(:, 4,53, 7) = (/ &
     &2.2577e-02_rb,3.3386e-02_rb,2.8049e-02_rb,1.8668e-02_rb,7.7087e-04_rb/)
      kbo(:, 5,53, 7) = (/ &
     &3.6135e-02_rb,5.3261e-02_rb,4.5341e-02_rb,3.0337e-02_rb,8.2493e-04_rb/)
      kbo(:, 1,54, 7) = (/ &
     &3.2171e-03_rb,4.9791e-03_rb,4.3152e-03_rb,3.1327e-03_rb,4.9375e-04_rb/)
      kbo(:, 2,54, 7) = (/ &
     &5.9006e-03_rb,8.9891e-03_rb,7.6027e-03_rb,5.2931e-03_rb,5.4126e-04_rb/)
      kbo(:, 3,54, 7) = (/ &
     &1.0411e-02_rb,1.5699e-02_rb,1.3153e-02_rb,8.8893e-03_rb,5.8635e-04_rb/)
      kbo(:, 4,54, 7) = (/ &
     &1.7599e-02_rb,2.6636e-02_rb,2.2397e-02_rb,1.4923e-02_rb,6.3108e-04_rb/)
      kbo(:, 5,54, 7) = (/ &
     &2.8648e-02_rb,4.3428e-02_rb,3.7016e-02_rb,2.4766e-02_rb,6.7747e-04_rb/)
      kbo(:, 1,55, 7) = (/ &
     &2.1501e-03_rb,3.4516e-03_rb,3.0360e-03_rb,2.2475e-03_rb,3.9872e-04_rb/)
      kbo(:, 2,55, 7) = (/ &
     &4.0344e-03_rb,6.3713e-03_rb,5.4431e-03_rb,3.8454e-03_rb,4.3910e-04_rb/)
      kbo(:, 3,55, 7) = (/ &
     &7.2610e-03_rb,1.1366e-02_rb,9.5883e-03_rb,6.5518e-03_rb,4.7768e-04_rb/)
      kbo(:, 4,55, 7) = (/ &
     &1.2517e-02_rb,1.9714e-02_rb,1.6657e-02_rb,1.1171e-02_rb,5.1535e-04_rb/)
      kbo(:, 5,55, 7) = (/ &
     &2.0727e-02_rb,3.2905e-02_rb,2.8181e-02_rb,1.8908e-02_rb,5.5486e-04_rb/)
      kbo(:, 1,56, 7) = (/ &
     &1.4245e-03_rb,2.3749e-03_rb,2.1236e-03_rb,1.6049e-03_rb,3.2130e-04_rb/)
      kbo(:, 2,56, 7) = (/ &
     &2.7381e-03_rb,4.4799e-03_rb,3.8718e-03_rb,2.7798e-03_rb,3.5587e-04_rb/)
      kbo(:, 3,56, 7) = (/ &
     &5.0259e-03_rb,8.1674e-03_rb,6.9450e-03_rb,4.8004e-03_rb,3.8862e-04_rb/)
      kbo(:, 4,56, 7) = (/ &
     &8.8432e-03_rb,1.4489e-02_rb,1.2307e-02_rb,8.3071e-03_rb,4.2064e-04_rb/)
      kbo(:, 5,56, 7) = (/ &
     &1.4910e-02_rb,2.4779e-02_rb,2.1310e-02_rb,1.4337e-02_rb,4.5360e-04_rb/)
      kbo(:, 1,57, 7) = (/ &
     &9.3471e-04_rb,1.6209e-03_rb,1.4751e-03_rb,1.1411e-03_rb,2.5838e-04_rb/)
      kbo(:, 2,57, 7) = (/ &
     &1.8436e-03_rb,3.1233e-03_rb,2.7344e-03_rb,1.9986e-03_rb,2.8780e-04_rb/)
      kbo(:, 3,57, 7) = (/ &
     &3.4502e-03_rb,5.8213e-03_rb,4.9934e-03_rb,3.4957e-03_rb,3.1565e-04_rb/)
      kbo(:, 4,57, 7) = (/ &
     &6.2020e-03_rb,1.0562e-02_rb,9.0212e-03_rb,6.1326e-03_rb,3.4283e-04_rb/)
      kbo(:, 5,57, 7) = (/ &
     &1.0656e-02_rb,1.8525e-02_rb,1.5995e-02_rb,1.0788e-02_rb,3.7043e-04_rb/)
      kbo(:, 1,58, 7) = (/ &
     &1.6224e-04_rb,3.6080e-04_rb,4.3166e-04_rb,4.8123e-04_rb,2.0767e-04_rb/)
      kbo(:, 2,58, 7) = (/ &
     &3.2835e-04_rb,7.0851e-04_rb,8.1160e-04_rb,8.4893e-04_rb,2.3260e-04_rb/)
      kbo(:, 3,58, 7) = (/ &
     &6.2650e-04_rb,1.3493e-03_rb,1.5057e-03_rb,1.4999e-03_rb,2.5617e-04_rb/)
      kbo(:, 4,58, 7) = (/ &
     &1.1500e-03_rb,2.5038e-03_rb,2.7715e-03_rb,2.6635e-03_rb,2.7921e-04_rb/)
      kbo(:, 5,58, 7) = (/ &
     &2.0128e-03_rb,4.5008e-03_rb,5.0240e-03_rb,4.7643e-03_rb,3.0249e-04_rb/)
      kbo(:, 1,59, 7) = (/ &
     &1.4374e-04_rb,3.4167e-04_rb,3.9743e-04_rb,4.1474e-04_rb,1.7016e-04_rb/)
      kbo(:, 2,59, 7) = (/ &
     &2.9460e-04_rb,6.7320e-04_rb,7.4449e-04_rb,7.2184e-04_rb,1.9072e-04_rb/)
      kbo(:, 3,59, 7) = (/ &
     &5.6936e-04_rb,1.2962e-03_rb,1.3859e-03_rb,1.2659e-03_rb,2.1044e-04_rb/)
      kbo(:, 4,59, 7) = (/ &
     &1.0579e-03_rb,2.4467e-03_rb,2.5761e-03_rb,2.2456e-03_rb,2.2959e-04_rb/)
      kbo(:, 5,59, 7) = (/ &
     &1.8733e-03_rb,4.4959e-03_rb,4.7523e-03_rb,4.0463e-03_rb,2.4889e-04_rb/)
      kbo(:, 1,13, 8) = (/ &
     &1.0984e+01_rb,8.3644e+00_rb,5.8323e+00_rb,3.5207e+00_rb,9.5573e-01_rb/)
      kbo(:, 2,13, 8) = (/ &
     &1.6457e+01_rb,1.2460e+01_rb,8.5024e+00_rb,4.7543e+00_rb,9.4898e-01_rb/)
      kbo(:, 3,13, 8) = (/ &
     &2.3514e+01_rb,1.7749e+01_rb,1.1998e+01_rb,6.4125e+00_rb,9.3955e-01_rb/)
      kbo(:, 4,13, 8) = (/ &
     &3.2333e+01_rb,2.4359e+01_rb,1.6389e+01_rb,8.5253e+00_rb,9.2689e-01_rb/)
      kbo(:, 5,13, 8) = (/ &
     &4.3124e+01_rb,3.2448e+01_rb,2.1775e+01_rb,1.1152e+01_rb,9.1779e-01_rb/)
      kbo(:, 1,14, 8) = (/ &
     &6.1000e+00_rb,4.7207e+00_rb,3.5104e+00_rb,2.4718e+00_rb,9.1734e-01_rb/)
      kbo(:, 2,14, 8) = (/ &
     &9.1253e+00_rb,6.9681e+00_rb,4.9378e+00_rb,3.1340e+00_rb,9.1181e-01_rb/)
      kbo(:, 3,14, 8) = (/ &
     &1.3035e+01_rb,9.8900e+00_rb,6.8208e+00_rb,3.9957e+00_rb,9.0546e-01_rb/)
      kbo(:, 4,14, 8) = (/ &
     &1.7969e+01_rb,1.3584e+01_rb,9.2328e+00_rb,5.1032e+00_rb,8.9760e-01_rb/)
      kbo(:, 5,14, 8) = (/ &
     &2.4021e+01_rb,1.8119e+01_rb,1.2229e+01_rb,6.5133e+00_rb,8.8920e-01_rb/)
      kbo(:, 1,15, 8) = (/ &
     &3.4743e+00_rb,2.8067e+00_rb,2.3437e+00_rb,1.8996e+00_rb,8.7156e-01_rb/)
      kbo(:, 2,15, 8) = (/ &
     &5.1884e+00_rb,4.0500e+00_rb,3.1125e+00_rb,2.2977e+00_rb,8.6786e-01_rb/)
      kbo(:, 3,15, 8) = (/ &
     &7.4263e+00_rb,5.6964e+00_rb,4.1330e+00_rb,2.7795e+00_rb,8.6394e-01_rb/)
      kbo(:, 4,15, 8) = (/ &
     &1.0261e+01_rb,7.8059e+00_rb,5.4699e+00_rb,3.3858e+00_rb,8.5908e-01_rb/)
      kbo(:, 5,15, 8) = (/ &
     &1.3736e+01_rb,1.0405e+01_rb,7.1459e+00_rb,4.1475e+00_rb,8.5337e-01_rb/)
      kbo(:, 1,16, 8) = (/ &
     &2.3668e+00_rb,2.0348e+00_rb,1.8716e+00_rb,1.6135e+00_rb,8.1894e-01_rb/)
      kbo(:, 2,16, 8) = (/ &
     &3.5375e+00_rb,2.8566e+00_rb,2.3918e+00_rb,1.9179e+00_rb,8.1793e-01_rb/)
      kbo(:, 3,16, 8) = (/ &
     &5.0738e+00_rb,3.9641e+00_rb,3.0738e+00_rb,2.2799e+00_rb,8.1574e-01_rb/)
      kbo(:, 4,16, 8) = (/ &
     &7.0147e+00_rb,5.3888e+00_rb,3.9564e+00_rb,2.7115e+00_rb,8.1291e-01_rb/)
      kbo(:, 5,16, 8) = (/ &
     &9.3829e+00_rb,7.1458e+00_rb,5.0571e+00_rb,3.2258e+00_rb,8.0943e-01_rb/)
      kbo(:, 1,17, 8) = (/ &
     &1.6738e+00_rb,1.5709e+00_rb,1.5512e+00_rb,1.3915e+00_rb,7.6163e-01_rb/)
      kbo(:, 2,17, 8) = (/ &
     &2.5071e+00_rb,2.1423e+00_rb,1.9466e+00_rb,1.6331e+00_rb,7.6194e-01_rb/)
      kbo(:, 3,17, 8) = (/ &
     &3.5973e+00_rb,2.9044e+00_rb,2.4376e+00_rb,1.9200e+00_rb,7.6176e-01_rb/)
      kbo(:, 4,17, 8) = (/ &
     &4.9653e+00_rb,3.8871e+00_rb,3.0499e+00_rb,2.2591e+00_rb,7.6112e-01_rb/)
      kbo(:, 5,17, 8) = (/ &
     &6.6200e+00_rb,5.0972e+00_rb,3.7961e+00_rb,2.6511e+00_rb,7.6001e-01_rb/)
      kbo(:, 1,18, 8) = (/ &
     &1.2159e+00_rb,1.2682e+00_rb,1.3055e+00_rb,1.2114e+00_rb,6.9972e-01_rb/)
      kbo(:, 2,18, 8) = (/ &
     &1.8215e+00_rb,1.6846e+00_rb,1.6207e+00_rb,1.4074e+00_rb,7.0172e-01_rb/)
      kbo(:, 3,18, 8) = (/ &
     &2.6063e+00_rb,2.2250e+00_rb,2.0059e+00_rb,1.6407e+00_rb,7.0349e-01_rb/)
      kbo(:, 4,18, 8) = (/ &
     &3.5864e+00_rb,2.9083e+00_rb,2.4603e+00_rb,1.9137e+00_rb,7.0440e-01_rb/)
      kbo(:, 5,18, 8) = (/ &
     &4.7701e+00_rb,3.7503e+00_rb,2.9936e+00_rb,2.2226e+00_rb,7.0559e-01_rb/)
      kbo(:, 1,19, 8) = (/ &
     &8.6896e-01_rb,1.0218e+00_rb,1.0917e+00_rb,1.0431e+00_rb,6.3571e-01_rb/)
      kbo(:, 2,19, 8) = (/ &
     &1.2971e+00_rb,1.3348e+00_rb,1.3390e+00_rb,1.2022e+00_rb,6.3915e-01_rb/)
      kbo(:, 3,19, 8) = (/ &
     &1.8500e+00_rb,1.7190e+00_rb,1.6367e+00_rb,1.3904e+00_rb,6.4265e-01_rb/)
      kbo(:, 4,19, 8) = (/ &
     &2.5377e+00_rb,2.1918e+00_rb,1.9845e+00_rb,1.6067e+00_rb,6.4523e-01_rb/)
      kbo(:, 5,19, 8) = (/ &
     &3.3694e+00_rb,2.7695e+00_rb,2.3848e+00_rb,1.8478e+00_rb,6.4962e-01_rb/)
      kbo(:, 1,20, 8) = (/ &
     &6.7593e-01_rb,8.7052e-01_rb,9.5066e-01_rb,9.1950e-01_rb,5.7154e-01_rb/)
      kbo(:, 2,20, 8) = (/ &
     &1.0037e+00_rb,1.1265e+00_rb,1.1588e+00_rb,1.0572e+00_rb,5.7670e-01_rb/)
      kbo(:, 3,20, 8) = (/ &
     &1.4256e+00_rb,1.4357e+00_rb,1.4052e+00_rb,1.2186e+00_rb,5.8150e-01_rb/)
      kbo(:, 4,20, 8) = (/ &
     &1.9488e+00_rb,1.8041e+00_rb,1.6911e+00_rb,1.4006e+00_rb,5.8703e-01_rb/)
      kbo(:, 5,20, 8) = (/ &
     &2.5835e+00_rb,2.2440e+00_rb,2.0193e+00_rb,1.6034e+00_rb,5.9399e-01_rb/)
      kbo(:, 1,21, 8) = (/ &
     &5.4036e-01_rb,7.5420e-01_rb,8.3572e-01_rb,8.1199e-01_rb,5.0987e-01_rb/)
      kbo(:, 2,21, 8) = (/ &
     &7.9845e-01_rb,9.6809e-01_rb,1.0141e+00_rb,9.3456e-01_rb,5.1627e-01_rb/)
      kbo(:, 3,21, 8) = (/ &
     &1.1289e+00_rb,1.2252e+00_rb,1.2225e+00_rb,1.0751e+00_rb,5.2292e-01_rb/)
      kbo(:, 4,21, 8) = (/ &
     &1.5394e+00_rb,1.5293e+00_rb,1.4635e+00_rb,1.2321e+00_rb,5.3121e-01_rb/)
      kbo(:, 5,21, 8) = (/ &
     &2.0388e+00_rb,1.8858e+00_rb,1.7395e+00_rb,1.4066e+00_rb,5.4074e-01_rb/)
      kbo(:, 1,22, 8) = (/ &
     &4.5347e-01_rb,6.7088e-01_rb,7.4705e-01_rb,7.2463e-01_rb,4.5233e-01_rb/)
      kbo(:, 2,22, 8) = (/ &
     &6.6420e-01_rb,8.5466e-01_rb,9.0352e-01_rb,8.3532e-01_rb,4.6020e-01_rb/)
      kbo(:, 3,22, 8) = (/ &
     &9.3312e-01_rb,1.0751e+00_rb,1.0854e+00_rb,9.5995e-01_rb,4.6942e-01_rb/)
      kbo(:, 4,22, 8) = (/ &
     &1.2674e+00_rb,1.3357e+00_rb,1.2944e+00_rb,1.0988e+00_rb,4.8015e-01_rb/)
      kbo(:, 5,22, 8) = (/ &
     &1.6769e+00_rb,1.6408e+00_rb,1.5338e+00_rb,1.2527e+00_rb,4.9182e-01_rb/)
      kbo(:, 1,23, 8) = (/ &
     &3.8192e-01_rb,5.9551e-01_rb,6.6526e-01_rb,6.4489e-01_rb,3.9997e-01_rb/)
      kbo(:, 2,23, 8) = (/ &
     &5.5484e-01_rb,7.5433e-01_rb,8.0227e-01_rb,7.4370e-01_rb,4.0956e-01_rb/)
      kbo(:, 3,23, 8) = (/ &
     &7.7517e-01_rb,9.4347e-01_rb,9.6125e-01_rb,8.5431e-01_rb,4.2064e-01_rb/)
      kbo(:, 4,23, 8) = (/ &
     &1.0504e+00_rb,1.1678e+00_rb,1.1437e+00_rb,9.7728e-01_rb,4.3319e-01_rb/)
      kbo(:, 5,23, 8) = (/ &
     &1.3891e+00_rb,1.4314e+00_rb,1.3533e+00_rb,1.1142e+00_rb,4.4599e-01_rb/)
      kbo(:, 1,24, 8) = (/ &
     &3.1980e-01_rb,5.2485e-01_rb,5.8816e-01_rb,5.7121e-01_rb,3.5303e-01_rb/)
      kbo(:, 2,24, 8) = (/ &
     &4.6136e-01_rb,6.6155e-01_rb,7.0757e-01_rb,6.5873e-01_rb,3.6427e-01_rb/)
      kbo(:, 3,24, 8) = (/ &
     &6.4223e-01_rb,8.2371e-01_rb,8.4561e-01_rb,7.5649e-01_rb,3.7674e-01_rb/)
      kbo(:, 4,24, 8) = (/ &
     &8.6939e-01_rb,1.0172e+00_rb,1.0047e+00_rb,8.6558e-01_rb,3.8981e-01_rb/)
      kbo(:, 5,24, 8) = (/ &
     &1.1484e+00_rb,1.2449e+00_rb,1.1880e+00_rb,9.8717e-01_rb,4.0255e-01_rb/)
      kbo(:, 1,25, 8) = (/ &
     &2.7156e-01_rb,4.6457e-01_rb,5.2127e-01_rb,5.0714e-01_rb,3.1176e-01_rb/)
      kbo(:, 2,25, 8) = (/ &
     &3.8962e-01_rb,5.8326e-01_rb,6.2605e-01_rb,5.8521e-01_rb,3.2376e-01_rb/)
      kbo(:, 3,25, 8) = (/ &
     &5.4140e-01_rb,7.2573e-01_rb,7.4721e-01_rb,6.7267e-01_rb,3.3654e-01_rb/)
      kbo(:, 4,25, 8) = (/ &
     &7.3158e-01_rb,8.9514e-01_rb,8.8800e-01_rb,7.7033e-01_rb,3.4916e-01_rb/)
      kbo(:, 5,25, 8) = (/ &
     &9.6557e-01_rb,1.0940e+00_rb,1.0510e+00_rb,8.7991e-01_rb,3.6188e-01_rb/)
      kbo(:, 1,26, 8) = (/ &
     &2.3603e-01_rb,4.1514e-01_rb,4.6570e-01_rb,4.5317e-01_rb,2.7528e-01_rb/)
      kbo(:, 2,26, 8) = (/ &
     &3.3729e-01_rb,5.2062e-01_rb,5.5904e-01_rb,5.2356e-01_rb,2.8722e-01_rb/)
      kbo(:, 3,26, 8) = (/ &
     &4.6762e-01_rb,6.4812e-01_rb,6.6791e-01_rb,6.0289e-01_rb,2.9963e-01_rb/)
      kbo(:, 4,26, 8) = (/ &
     &6.3043e-01_rb,7.9949e-01_rb,7.9540e-01_rb,6.9207e-01_rb,3.1197e-01_rb/)
      kbo(:, 5,26, 8) = (/ &
     &8.2894e-01_rb,9.7604e-01_rb,9.4222e-01_rb,7.9275e-01_rb,3.2460e-01_rb/)
      kbo(:, 1,27, 8) = (/ &
     &2.1329e-01_rb,3.7886e-01_rb,4.2295e-01_rb,4.0982e-01_rb,2.4262e-01_rb/)
      kbo(:, 2,27, 8) = (/ &
     &3.0405e-01_rb,4.7609e-01_rb,5.0881e-01_rb,4.7490e-01_rb,2.5434e-01_rb/)
      kbo(:, 3,27, 8) = (/ &
     &4.2047e-01_rb,5.9404e-01_rb,6.1027e-01_rb,5.4870e-01_rb,2.6611e-01_rb/)
      kbo(:, 4,27, 8) = (/ &
     &5.6493e-01_rb,7.3338e-01_rb,7.2904e-01_rb,6.3251e-01_rb,2.7831e-01_rb/)
      kbo(:, 5,27, 8) = (/ &
     &7.3961e-01_rb,8.9452e-01_rb,8.6490e-01_rb,7.2735e-01_rb,2.9101e-01_rb/)
      kbo(:, 1,28, 8) = (/ &
     &1.9857e-01_rb,3.5152e-01_rb,3.8897e-01_rb,3.7421e-01_rb,2.1366e-01_rb/)
      kbo(:, 2,28, 8) = (/ &
     &2.8199e-01_rb,4.4350e-01_rb,4.7029e-01_rb,4.3552e-01_rb,2.2475e-01_rb/)
      kbo(:, 3,28, 8) = (/ &
     &3.8828e-01_rb,5.5465e-01_rb,5.6697e-01_rb,5.0558e-01_rb,2.3636e-01_rb/)
      kbo(:, 4,28, 8) = (/ &
     &5.1944e-01_rb,6.8554e-01_rb,6.7950e-01_rb,5.8599e-01_rb,2.4843e-01_rb/)
      kbo(:, 5,28, 8) = (/ &
     &6.7804e-01_rb,8.3583e-01_rb,8.0789e-01_rb,6.7686e-01_rb,2.6132e-01_rb/)
      kbo(:, 1,29, 8) = (/ &
     &1.9699e-01_rb,3.4001e-01_rb,3.6948e-01_rb,3.4985e-01_rb,1.8812e-01_rb/)
      kbo(:, 2,29, 8) = (/ &
     &2.7846e-01_rb,4.3155e-01_rb,4.5041e-01_rb,4.0987e-01_rb,1.9891e-01_rb/)
      kbo(:, 3,29, 8) = (/ &
     &3.8140e-01_rb,5.4168e-01_rb,5.4658e-01_rb,4.7968e-01_rb,2.1027e-01_rb/)
      kbo(:, 4,29, 8) = (/ &
     &5.0817e-01_rb,6.7028e-01_rb,6.5795e-01_rb,5.5982e-01_rb,2.2267e-01_rb/)
      kbo(:, 5,29, 8) = (/ &
     &6.6092e-01_rb,8.1811e-01_rb,7.8466e-01_rb,6.5003e-01_rb,2.3569e-01_rb/)
      kbo(:, 1,30, 8) = (/ &
     &2.0038e-01_rb,3.3558e-01_rb,3.5744e-01_rb,3.3111e-01_rb,1.6586e-01_rb/)
      kbo(:, 2,30, 8) = (/ &
     &2.8170e-01_rb,4.2849e-01_rb,4.3945e-01_rb,3.9159e-01_rb,1.7645e-01_rb/)
      kbo(:, 3,30, 8) = (/ &
     &3.8386e-01_rb,5.3939e-01_rb,5.3663e-01_rb,4.6240e-01_rb,1.8797e-01_rb/)
      kbo(:, 4,30, 8) = (/ &
     &5.0944e-01_rb,6.6904e-01_rb,6.4909e-01_rb,5.4342e-01_rb,2.0032e-01_rb/)
      kbo(:, 5,30, 8) = (/ &
     &6.6110e-01_rb,8.1719e-01_rb,7.7645e-01_rb,6.3464e-01_rb,2.1343e-01_rb/)
      kbo(:, 1,31, 8) = (/ &
     &2.1457e-01_rb,3.4491e-01_rb,3.5799e-01_rb,3.2199e-01_rb,1.4677e-01_rb/)
      kbo(:, 2,31, 8) = (/ &
     &3.0025e-01_rb,4.4295e-01_rb,4.4440e-01_rb,3.8528e-01_rb,1.5733e-01_rb/)
      kbo(:, 3,31, 8) = (/ &
     &4.0706e-01_rb,5.5909e-01_rb,5.4635e-01_rb,4.5932e-01_rb,1.6888e-01_rb/)
      kbo(:, 4,31, 8) = (/ &
     &5.3865e-01_rb,6.9458e-01_rb,6.6400e-01_rb,5.4394e-01_rb,1.8129e-01_rb/)
      kbo(:, 5,31, 8) = (/ &
     &6.9836e-01_rb,8.4979e-01_rb,7.9754e-01_rb,6.3956e-01_rb,1.9424e-01_rb/)
      kbo(:, 1,32, 8) = (/ &
     &2.3301e-01_rb,3.6002e-01_rb,3.6447e-01_rb,3.1806e-01_rb,1.3061e-01_rb/)
      kbo(:, 2,32, 8) = (/ &
     &3.2391e-01_rb,4.6378e-01_rb,4.5628e-01_rb,3.8494e-01_rb,1.4116e-01_rb/)
      kbo(:, 3,32, 8) = (/ &
     &4.3762e-01_rb,5.8716e-01_rb,5.6420e-01_rb,4.6300e-01_rb,1.5266e-01_rb/)
      kbo(:, 4,32, 8) = (/ &
     &5.7844e-01_rb,7.3058e-01_rb,6.8894e-01_rb,5.5249e-01_rb,1.6489e-01_rb/)
      kbo(:, 5,32, 8) = (/ &
     &7.5012e-01_rb,8.9521e-01_rb,8.3056e-01_rb,6.5369e-01_rb,1.7758e-01_rb/)
      kbo(:, 1,33, 8) = (/ &
     &2.5790e-01_rb,3.8312e-01_rb,3.7883e-01_rb,3.2071e-01_rb,1.1691e-01_rb/)
      kbo(:, 2,33, 8) = (/ &
     &3.5669e-01_rb,4.9524e-01_rb,4.7746e-01_rb,3.9236e-01_rb,1.2744e-01_rb/)
      kbo(:, 3,33, 8) = (/ &
     &4.8068e-01_rb,6.2777e-01_rb,5.9359e-01_rb,4.7601e-01_rb,1.3874e-01_rb/)
      kbo(:, 4,33, 8) = (/ &
     &6.3465e-01_rb,7.8244e-01_rb,7.2784e-01_rb,5.7215e-01_rb,1.5065e-01_rb/)
      kbo(:, 5,33, 8) = (/ &
     &8.2463e-01_rb,9.6158e-01_rb,8.8094e-01_rb,6.8112e-01_rb,1.6290e-01_rb/)
      kbo(:, 1,34, 8) = (/ &
     &2.7501e-01_rb,3.9928e-01_rb,3.8789e-01_rb,3.2093e-01_rb,1.0495e-01_rb/)
      kbo(:, 2,34, 8) = (/ &
     &3.7955e-01_rb,5.1722e-01_rb,4.9202e-01_rb,3.9633e-01_rb,1.1524e-01_rb/)
      kbo(:, 3,34, 8) = (/ &
     &5.1074e-01_rb,6.5671e-01_rb,6.1416e-01_rb,4.8473e-01_rb,1.2623e-01_rb/)
      kbo(:, 4,34, 8) = (/ &
     &6.7545e-01_rb,8.2065e-01_rb,7.5592e-01_rb,5.8631e-01_rb,1.3772e-01_rb/)
      kbo(:, 5,34, 8) = (/ &
     &8.8010e-01_rb,1.0115e+00_rb,9.1893e-01_rb,7.0176e-01_rb,1.4967e-01_rb/)
      kbo(:, 1,35, 8) = (/ &
     &2.8491e-01_rb,4.0792e-01_rb,3.9124e-01_rb,3.1768e-01_rb,9.3815e-02_rb/)
      kbo(:, 2,35, 8) = (/ &
     &3.9304e-01_rb,5.3008e-01_rb,4.9917e-01_rb,3.9612e-01_rb,1.0372e-01_rb/)
      kbo(:, 3,35, 8) = (/ &
     &5.3059e-01_rb,6.7569e-01_rb,6.2665e-01_rb,4.8824e-01_rb,1.1431e-01_rb/)
      kbo(:, 4,35, 8) = (/ &
     &7.0463e-01_rb,8.4744e-01_rb,7.7517e-01_rb,5.9483e-01_rb,1.2547e-01_rb/)
      kbo(:, 5,35, 8) = (/ &
     &9.2289e-01_rb,1.0490e+00_rb,9.4773e-01_rb,7.1638e-01_rb,1.3729e-01_rb/)
      kbo(:, 1,36, 8) = (/ &
     &2.8335e-01_rb,4.0444e-01_rb,3.8444e-01_rb,3.0785e-01_rb,8.3181e-02_rb/)
      kbo(:, 2,36, 8) = (/ &
     &3.9231e-01_rb,5.2837e-01_rb,4.9425e-01_rb,3.8787e-01_rb,9.2674e-02_rb/)
      kbo(:, 3,36, 8) = (/ &
     &5.3277e-01_rb,6.7716e-01_rb,6.2485e-01_rb,4.8257e-01_rb,1.0284e-01_rb/)
      kbo(:, 4,36, 8) = (/ &
     &7.1164e-01_rb,8.5400e-01_rb,7.7825e-01_rb,5.9239e-01_rb,1.1370e-01_rb/)
      kbo(:, 5,36, 8) = (/ &
     &9.3841e-01_rb,1.0634e+00_rb,9.5733e-01_rb,7.1881e-01_rb,1.2521e-01_rb/)
      kbo(:, 1,37, 8) = (/ &
     &2.6099e-01_rb,3.7794e-01_rb,3.5839e-01_rb,2.8510e-01_rb,7.2608e-02_rb/)
      kbo(:, 2,37, 8) = (/ &
     &3.6453e-01_rb,4.9803e-01_rb,4.6544e-01_rb,3.6354e-01_rb,8.1599e-02_rb/)
      kbo(:, 3,37, 8) = (/ &
     &4.9925e-01_rb,6.4313e-01_rb,5.9376e-01_rb,4.5692e-01_rb,9.1324e-02_rb/)
      kbo(:, 4,37, 8) = (/ &
     &6.7318e-01_rb,8.1735e-01_rb,7.4579e-01_rb,5.6616e-01_rb,1.0177e-01_rb/)
      kbo(:, 5,37, 8) = (/ &
     &8.9497e-01_rb,1.0258e+00_rb,9.2402e-01_rb,6.9303e-01_rb,1.1300e-01_rb/)
      kbo(:, 1,38, 8) = (/ &
     &2.4331e-01_rb,3.5651e-01_rb,3.3705e-01_rb,2.6611e-01_rb,6.3364e-02_rb/)
      kbo(:, 2,38, 8) = (/ &
     &3.4315e-01_rb,4.7401e-01_rb,4.4240e-01_rb,3.4349e-01_rb,7.1841e-02_rb/)
      kbo(:, 3,38, 8) = (/ &
     &4.7415e-01_rb,6.1710e-01_rb,5.6959e-01_rb,4.3645e-01_rb,8.1122e-02_rb/)
      kbo(:, 4,38, 8) = (/ &
     &6.4535e-01_rb,7.9084e-01_rb,7.2136e-01_rb,5.4612e-01_rb,9.1255e-02_rb/)
      kbo(:, 5,38, 8) = (/ &
     &8.6575e-01_rb,1.0005e+00_rb,9.0147e-01_rb,6.7461e-01_rb,1.0216e-01_rb/)
      kbo(:, 1,39, 8) = (/ &
     &2.3205e-01_rb,3.4217e-01_rb,3.2219e-01_rb,2.5204e-01_rb,5.5366e-02_rb/)
      kbo(:, 2,39, 8) = (/ &
     &3.3051e-01_rb,4.5923e-01_rb,4.2752e-01_rb,3.2956e-01_rb,6.3325e-02_rb/)
      kbo(:, 3,39, 8) = (/ &
     &4.6126e-01_rb,6.0310e-01_rb,5.5564e-01_rb,4.2338e-01_rb,7.2166e-02_rb/)
      kbo(:, 4,39, 8) = (/ &
     &6.3405e-01_rb,7.7976e-01_rb,7.0985e-01_rb,5.3501e-01_rb,8.2008e-02_rb/)
      kbo(:, 5,39, 8) = (/ &
     &8.5903e-01_rb,9.9488e-01_rb,8.9458e-01_rb,6.6726e-01_rb,9.2704e-02_rb/)
      kbo(:, 1,40, 8) = (/ &
     &2.0024e-01_rb,3.0416e-01_rb,2.8720e-01_rb,2.2448e-01_rb,4.7646e-02_rb/)
      kbo(:, 2,40, 8) = (/ &
     &2.8846e-01_rb,4.1256e-01_rb,3.8572e-01_rb,2.9759e-01_rb,5.5021e-02_rb/)
      kbo(:, 3,40, 8) = (/ &
     &4.0740e-01_rb,5.4726e-01_rb,5.0694e-01_rb,3.8698e-01_rb,6.3373e-02_rb/)
      kbo(:, 4,40, 8) = (/ &
     &5.6679e-01_rb,7.1454e-01_rb,6.5407e-01_rb,4.9457e-01_rb,7.2804e-02_rb/)
      kbo(:, 5,40, 8) = (/ &
     &7.7703e-01_rb,9.2066e-01_rb,8.3239e-01_rb,6.2266e-01_rb,8.3210e-02_rb/)
      kbo(:, 1,41, 8) = (/ &
     &1.7113e-01_rb,2.6826e-01_rb,2.5417e-01_rb,1.9867e-01_rb,4.0876e-02_rb/)
      kbo(:, 2,41, 8) = (/ &
     &2.4978e-01_rb,3.6831e-01_rb,3.4599e-01_rb,2.6723e-01_rb,4.7680e-02_rb/)
      kbo(:, 3,41, 8) = (/ &
     &3.5705e-01_rb,4.9395e-01_rb,4.5991e-01_rb,3.5185e-01_rb,5.5528e-02_rb/)
      kbo(:, 4,41, 8) = (/ &
     &5.0326e-01_rb,6.5160e-01_rb,5.9993e-01_rb,4.5499e-01_rb,6.4535e-02_rb/)
      kbo(:, 5,41, 8) = (/ &
     &6.9778e-01_rb,8.4752e-01_rb,7.7088e-01_rb,5.7861e-01_rb,7.4659e-02_rb/)
      kbo(:, 1,42, 8) = (/ &
     &1.4736e-01_rb,2.3782e-01_rb,2.2594e-01_rb,1.7650e-01_rb,3.4992e-02_rb/)
      kbo(:, 2,42, 8) = (/ &
     &2.1817e-01_rb,3.3087e-01_rb,3.1198e-01_rb,2.4105e-01_rb,4.1263e-02_rb/)
      kbo(:, 3,42, 8) = (/ &
     &3.1590e-01_rb,4.4893e-01_rb,4.1987e-01_rb,3.2183e-01_rb,4.8631e-02_rb/)
      kbo(:, 4,42, 8) = (/ &
     &4.5063e-01_rb,5.9836e-01_rb,5.5343e-01_rb,4.2092e-01_rb,5.7226e-02_rb/)
      kbo(:, 5,42, 8) = (/ &
     &6.3264e-01_rb,7.8621e-01_rb,7.1837e-01_rb,5.4085e-01_rb,6.7074e-02_rb/)
      kbo(:, 1,43, 8) = (/ &
     &1.2471e-01_rb,2.0767e-01_rb,1.9765e-01_rb,1.5433e-01_rb,2.9617e-02_rb/)
      kbo(:, 2,43, 8) = (/ &
     &1.8736e-01_rb,2.9309e-01_rb,2.7736e-01_rb,2.1438e-01_rb,3.5333e-02_rb/)
      kbo(:, 3,43, 8) = (/ &
     &2.7561e-01_rb,4.0307e-01_rb,3.7854e-01_rb,2.9057e-01_rb,4.2142e-02_rb/)
      kbo(:, 4,43, 8) = (/ &
     &3.9834e-01_rb,5.4356e-01_rb,5.0521e-01_rb,3.8507e-01_rb,5.0246e-02_rb/)
      kbo(:, 5,43, 8) = (/ &
     &5.6689e-01_rb,7.2237e-01_rb,6.6289e-01_rb,5.0065e-01_rb,5.9725e-02_rb/)
      kbo(:, 1,44, 8) = (/ &
     &1.0535e-01_rb,1.8057e-01_rb,1.7198e-01_rb,1.3418e-01_rb,2.4874e-02_rb/)
      kbo(:, 2,44, 8) = (/ &
     &1.6101e-01_rb,2.5912e-01_rb,2.4579e-01_rb,1.8994e-01_rb,3.0018e-02_rb/)
      kbo(:, 3,44, 8) = (/ &
     &2.4038e-01_rb,3.6143e-01_rb,3.4064e-01_rb,2.6166e-01_rb,3.6275e-02_rb/)
      kbo(:, 4,44, 8) = (/ &
     &3.5302e-01_rb,4.9391e-01_rb,4.6079e-01_rb,3.5170e-01_rb,4.3858e-02_rb/)
      kbo(:, 5,44, 8) = (/ &
     &5.0915e-01_rb,6.6446e-01_rb,6.1193e-01_rb,4.6316e-01_rb,5.2911e-02_rb/)
      kbo(:, 1,45, 8) = (/ &
     &9.0316e-02_rb,1.5846e-01_rb,1.5081e-01_rb,1.1736e-01_rb,2.0815e-02_rb/)
      kbo(:, 2,45, 8) = (/ &
     &1.4022e-01_rb,2.3128e-01_rb,2.1952e-01_rb,1.6935e-01_rb,2.5419e-02_rb/)
      kbo(:, 3,45, 8) = (/ &
     &2.1290e-01_rb,3.2756e-01_rb,3.0923e-01_rb,2.3736e-01_rb,3.1129e-02_rb/)
      kbo(:, 4,45, 8) = (/ &
     &3.1767e-01_rb,4.5386e-01_rb,4.2443e-01_rb,3.2399e-01_rb,3.8187e-02_rb/)
      kbo(:, 5,45, 8) = (/ &
     &4.6514e-01_rb,6.1840e-01_rb,5.7081e-01_rb,4.3233e-01_rb,4.6782e-02_rb/)
      kbo(:, 1,46, 8) = (/ &
     &7.6970e-02_rb,1.3789e-01_rb,1.3114e-01_rb,1.0166e-01_rb,1.7264e-02_rb/)
      kbo(:, 2,46, 8) = (/ &
     &1.2176e-01_rb,2.0534e-01_rb,1.9482e-01_rb,1.4994e-01_rb,2.1346e-02_rb/)
      kbo(:, 3,46, 8) = (/ &
     &1.8790e-01_rb,2.9566e-01_rb,2.7941e-01_rb,2.1415e-01_rb,2.6490e-02_rb/)
      kbo(:, 4,46, 8) = (/ &
     &2.8511e-01_rb,4.1574e-01_rb,3.8936e-01_rb,2.9707e-01_rb,3.3007e-02_rb/)
      kbo(:, 5,46, 8) = (/ &
     &4.2418e-01_rb,5.7427e-01_rb,5.3095e-01_rb,4.0219e-01_rb,4.1080e-02_rb/)
      kbo(:, 1,47, 8) = (/ &
     &6.3210e-02_rb,1.1622e-01_rb,1.1043e-01_rb,8.5380e-02_rb,1.4119e-02_rb/)
      kbo(:, 2,47, 8) = (/ &
     &1.0215e-01_rb,1.7705e-01_rb,1.6808e-01_rb,1.2908e-01_rb,1.7662e-02_rb/)
      kbo(:, 3,47, 8) = (/ &
     &1.6053e-01_rb,2.5968e-01_rb,2.4579e-01_rb,1.8828e-01_rb,2.2222e-02_rb/)
      kbo(:, 4,47, 8) = (/ &
     &2.4774e-01_rb,3.7111e-01_rb,3.4851e-01_rb,2.6604e-01_rb,2.8126e-02_rb/)
      kbo(:, 5,47, 8) = (/ &
     &3.7525e-01_rb,5.2022e-01_rb,4.8241e-01_rb,3.6582e-01_rb,3.5589e-02_rb/)
      kbo(:, 1,48, 8) = (/ &
     &5.2856e-02_rb,9.8941e-02_rb,9.3700e-02_rb,7.2138e-02_rb,1.1481e-02_rb/)
      kbo(:, 2,48, 8) = (/ &
     &8.7380e-02_rb,1.5439e-01_rb,1.4631e-01_rb,1.1192e-01_rb,1.4532e-02_rb/)
      kbo(:, 3,48, 8) = (/ &
     &1.4008e-01_rb,2.3111e-01_rb,2.1862e-01_rb,1.6701e-01_rb,1.8538e-02_rb/)
      kbo(:, 4,48, 8) = (/ &
     &2.2000e-01_rb,3.3600e-01_rb,3.1573e-01_rb,2.4065e-01_rb,2.3829e-02_rb/)
      kbo(:, 5,48, 8) = (/ &
     &3.3939e-01_rb,4.7860e-01_rb,4.4403e-01_rb,3.3641e-01_rb,3.0679e-02_rb/)
      kbo(:, 1,49, 8) = (/ &
     &4.5594e-02_rb,8.5787e-02_rb,8.0733e-02_rb,6.1668e-02_rb,9.2832e-03_rb/)
      kbo(:, 2,49, 8) = (/ &
     &7.7236e-02_rb,1.3755e-01_rb,1.2964e-01_rb,9.8500e-02_rb,1.1885e-02_rb/)
      kbo(:, 3,49, 8) = (/ &
     &1.2646e-01_rb,2.1033e-01_rb,1.9831e-01_rb,1.5059e-01_rb,1.5367e-02_rb/)
      kbo(:, 4,49, 8) = (/ &
     &2.0223e-01_rb,3.1150e-01_rb,2.9194e-01_rb,2.2146e-01_rb,2.0076e-02_rb/)
      kbo(:, 5,49, 8) = (/ &
     &3.1773e-01_rb,4.5116e-01_rb,4.1757e-01_rb,3.1527e-01_rb,2.6302e-02_rb/)
      kbo(:, 1,50, 8) = (/ &
     &3.7552e-02_rb,7.1895e-02_rb,6.7424e-02_rb,5.1302e-02_rb,7.5009e-03_rb/)
      kbo(:, 2,50, 8) = (/ &
     &6.5207e-02_rb,1.1840e-01_rb,1.1139e-01_rb,8.4335e-02_rb,9.7205e-03_rb/)
      kbo(:, 3,50, 8) = (/ &
     &1.0908e-01_rb,1.8529e-01_rb,1.7464e-01_rb,1.3230e-01_rb,1.2741e-02_rb/)
      kbo(:, 4,50, 8) = (/ &
     &1.7795e-01_rb,2.7969e-01_rb,2.6224e-01_rb,1.9866e-01_rb,1.6912e-02_rb/)
      kbo(:, 5,50, 8) = (/ &
     &2.8480e-01_rb,4.1182e-01_rb,3.8156e-01_rb,2.8786e-01_rb,2.2557e-02_rb/)
      kbo(:, 1,51, 8) = (/ &
     &3.0119e-02_rb,5.9038e-02_rb,5.5227e-02_rb,4.1909e-02_rb,6.0436e-03_rb/)
      kbo(:, 2,51, 8) = (/ &
     &5.3723e-02_rb,1.0012e-01_rb,9.4083e-02_rb,7.1011e-02_rb,7.9244e-03_rb/)
      kbo(:, 3,51, 8) = (/ &
     &9.1816e-02_rb,1.6026e-01_rb,1.5113e-01_rb,1.1438e-01_rb,1.0530e-02_rb/)
      kbo(:, 4,51, 8) = (/ &
     &1.5289e-01_rb,2.4665e-01_rb,2.3169e-01_rb,1.7549e-01_rb,1.4198e-02_rb/)
      kbo(:, 5,51, 8) = (/ &
     &2.4923e-01_rb,3.6941e-01_rb,3.4309e-01_rb,2.5907e-01_rb,1.9285e-02_rb/)
      kbo(:, 1,52, 8) = (/ &
     &2.4575e-02_rb,4.8862e-02_rb,4.5516e-02_rb,3.4388e-02_rb,4.8448e-03_rb/)
      kbo(:, 2,52, 8) = (/ &
     &4.5073e-02_rb,8.5520e-02_rb,8.0105e-02_rb,6.0149e-02_rb,6.4237e-03_rb/)
      kbo(:, 3,52, 8) = (/ &
     &7.8951e-02_rb,1.4049e-01_rb,1.3220e-01_rb,9.9732e-02_rb,8.6480e-03_rb/)
      kbo(:, 4,52, 8) = (/ &
     &1.3416e-01_rb,2.2067e-01_rb,2.0720e-01_rb,1.5657e-01_rb,1.1852e-02_rb/)
      kbo(:, 5,52, 8) = (/ &
     &2.2319e-01_rb,3.3661e-01_rb,3.1253e-01_rb,2.3574e-01_rb,1.6412e-02_rb/)
      kbo(:, 1,53, 8) = (/ &
     &2.0612e-02_rb,4.1138e-02_rb,3.8068e-02_rb,2.8547e-02_rb,3.8662e-03_rb/)
      kbo(:, 2,53, 8) = (/ &
     &3.9008e-02_rb,7.4441e-02_rb,6.9299e-02_rb,5.1635e-02_rb,5.1786e-03_rb/)
      kbo(:, 3,53, 8) = (/ &
     &7.0078e-02_rb,1.2575e-01_rb,1.1775e-01_rb,8.8269e-02_rb,7.0623e-03_rb/)
      kbo(:, 4,53, 8) = (/ &
     &1.2164e-01_rb,2.0192e-01_rb,1.8894e-01_rb,1.4209e-01_rb,9.8285e-03_rb/)
      kbo(:, 5,53, 8) = (/ &
     &2.0657e-01_rb,3.1414e-01_rb,2.9077e-01_rb,2.1839e-01_rb,1.3889e-02_rb/)
      kbo(:, 1,54, 8) = (/ &
     &1.5065e-02_rb,3.1280e-02_rb,2.9021e-02_rb,2.1865e-02_rb,3.0881e-03_rb/)
      kbo(:, 2,54, 8) = (/ &
     &2.9415e-02_rb,5.8678e-02_rb,5.4740e-02_rb,4.0841e-02_rb,4.1803e-03_rb/)
      kbo(:, 3,54, 8) = (/ &
     &5.4186e-02_rb,1.0195e-01_rb,9.5873e-02_rb,7.1991e-02_rb,5.7700e-03_rb/)
      kbo(:, 4,54, 8) = (/ &
     &9.6215e-02_rb,1.6747e-01_rb,1.5763e-01_rb,1.1902e-01_rb,8.1641e-03_rb/)
      kbo(:, 5,54, 8) = (/ &
     &1.6671e-01_rb,2.6518e-01_rb,2.4733e-01_rb,1.8663e-01_rb,1.1760e-02_rb/)
      kbo(:, 1,55, 8) = (/ &
     &1.0025e-02_rb,2.2183e-02_rb,2.0776e-02_rb,1.5843e-02_rb,2.4606e-03_rb/)
      kbo(:, 2,55, 8) = (/ &
     &2.0216e-02_rb,4.3248e-02_rb,4.0605e-02_rb,3.0502e-02_rb,3.3650e-03_rb/)
      kbo(:, 3,55, 8) = (/ &
     &3.8225e-02_rb,7.7452e-02_rb,7.3454e-02_rb,5.5488e-02_rb,4.7010e-03_rb/)
      kbo(:, 4,55, 8) = (/ &
     &6.9405e-02_rb,1.3019e-01_rb,1.2386e-01_rb,9.4370e-02_rb,6.7604e-03_rb/)
      kbo(:, 5,55, 8) = (/ &
     &1.2275e-01_rb,2.0976e-01_rb,1.9836e-01_rb,1.5118e-01_rb,9.9237e-03_rb/)
      kbo(:, 1,56, 8) = (/ &
     &6.5953e-03_rb,1.5532e-02_rb,1.4697e-02_rb,1.1369e-02_rb,1.9523e-03_rb/)
      kbo(:, 2,56, 8) = (/ &
     &1.3758e-02_rb,3.1499e-02_rb,2.9770e-02_rb,2.2525e-02_rb,2.6931e-03_rb/)
      kbo(:, 3,56, 8) = (/ &
     &2.6733e-02_rb,5.8400e-02_rb,5.5794e-02_rb,4.2378e-02_rb,3.8077e-03_rb/)
      kbo(:, 4,56, 8) = (/ &
     &4.9676e-02_rb,1.0081e-01_rb,9.6783e-02_rb,7.4327e-02_rb,5.5587e-03_rb/)
      kbo(:, 5,56, 8) = (/ &
     &8.9711e-02_rb,1.6544e-01_rb,1.5844e-01_rb,1.2195e-01_rb,8.3185e-03_rb/)
      kbo(:, 1,57, 8) = (/ &
     &4.2820e-03_rb,1.0722e-02_rb,1.0258e-02_rb,8.0796e-03_rb,1.5428e-03_rb/)
      kbo(:, 2,57, 8) = (/ &
     &9.2544e-03_rb,2.2704e-02_rb,2.1598e-02_rb,1.6468e-02_rb,2.1441e-03_rb/)
      kbo(:, 3,57, 8) = (/ &
     &1.8521e-02_rb,4.3686e-02_rb,4.1983e-02_rb,3.2044e-02_rb,3.0664e-03_rb/)
      kbo(:, 4,57, 8) = (/ &
     &3.5250e-02_rb,7.7586e-02_rb,7.5143e-02_rb,5.8070e-02_rb,4.5371e-03_rb/)
      kbo(:, 5,57, 8) = (/ &
     &6.5084e-02_rb,1.3004e-01_rb,1.2594e-01_rb,9.7902e-02_rb,6.9329e-03_rb/)
      kbo(:, 1,58, 8) = (/ &
     &7.3360e-04_rb,2.4101e-03_rb,3.0107e-03_rb,3.3982e-03_rb,1.2193e-03_rb/)
      kbo(:, 2,58, 8) = (/ &
     &1.6434e-03_rb,5.3110e-03_rb,6.5569e-03_rb,7.0857e-03_rb,1.7049e-03_rb/)
      kbo(:, 3,58, 8) = (/ &
     &3.3887e-03_rb,1.0643e-02_rb,1.3244e-02_rb,1.4263e-02_rb,2.4670e-03_rb/)
      kbo(:, 4,58, 8) = (/ &
     &6.6116e-03_rb,1.9492e-02_rb,2.4519e-02_rb,2.6730e-02_rb,3.7032e-03_rb/)
      kbo(:, 5,58, 8) = (/ &
     &1.2475e-02_rb,3.3432e-02_rb,4.2135e-02_rb,4.6396e-02_rb,5.7742e-03_rb/)
      kbo(:, 1,59, 8) = (/ &
     &6.5322e-04_rb,2.3892e-03_rb,2.8729e-03_rb,2.9916e-03_rb,1.0154e-03_rb/)
      kbo(:, 2,59, 8) = (/ &
     &1.4925e-03_rb,5.3944e-03_rb,6.3528e-03_rb,6.2709e-03_rb,1.4376e-03_rb/)
      kbo(:, 3,59, 8) = (/ &
     &3.1352e-03_rb,1.1095e-02_rb,1.3136e-02_rb,1.2884e-02_rb,2.1132e-03_rb/)
      kbo(:, 4,59, 8) = (/ &
     &6.2265e-03_rb,2.0748e-02_rb,2.4885e-02_rb,2.4670e-02_rb,3.2435e-03_rb/)
      kbo(:, 5,59, 8) = (/ &
     &1.1971e-02_rb,3.6140e-02_rb,4.3523e-02_rb,4.3765e-02_rb,5.1685e-03_rb/)
      kbo(:, 1,13, 9) = (/ &
     &7.3666e+01_rb,5.5396e+01_rb,3.7129e+01_rb,1.8928e+01_rb,2.0240e+00_rb/)
      kbo(:, 2,13, 9) = (/ &
     &1.0840e+02_rb,8.1447e+01_rb,5.4490e+01_rb,2.7556e+01_rb,1.9929e+00_rb/)
      kbo(:, 3,13, 9) = (/ &
     &1.5167e+02_rb,1.1389e+02_rb,7.6113e+01_rb,3.8343e+01_rb,1.9601e+00_rb/)
      kbo(:, 4,13, 9) = (/ &
     &2.0374e+02_rb,1.5294e+02_rb,1.0214e+02_rb,5.1349e+01_rb,1.9301e+00_rb/)
      kbo(:, 5,13, 9) = (/ &
     &2.6449e+02_rb,1.9850e+02_rb,1.3251e+02_rb,6.6525e+01_rb,1.8945e+00_rb/)
      kbo(:, 1,14, 9) = (/ &
     &4.2021e+01_rb,3.1664e+01_rb,2.1327e+01_rb,1.1250e+01_rb,2.1001e+00_rb/)
      kbo(:, 2,14, 9) = (/ &
     &6.1662e+01_rb,4.6390e+01_rb,3.1123e+01_rb,1.5991e+01_rb,2.0721e+00_rb/)
      kbo(:, 3,14, 9) = (/ &
     &8.6125e+01_rb,6.4733e+01_rb,4.3345e+01_rb,2.2004e+01_rb,2.0419e+00_rb/)
      kbo(:, 4,14, 9) = (/ &
     &1.1550e+02_rb,8.6758e+01_rb,5.8024e+01_rb,2.9304e+01_rb,2.0094e+00_rb/)
      kbo(:, 5,14, 9) = (/ &
     &1.4971e+02_rb,1.1242e+02_rb,7.5125e+01_rb,3.7838e+01_rb,1.9780e+00_rb/)
      kbo(:, 1,15, 9) = (/ &
     &2.4325e+01_rb,1.8394e+01_rb,1.2572e+01_rb,7.1971e+00_rb,2.1558e+00_rb/)
      kbo(:, 2,15, 9) = (/ &
     &3.5582e+01_rb,2.6831e+01_rb,1.8117e+01_rb,9.7575e+00_rb,2.1305e+00_rb/)
      kbo(:, 3,15, 9) = (/ &
     &4.9567e+01_rb,3.7318e+01_rb,2.5078e+01_rb,1.3071e+01_rb,2.1029e+00_rb/)
      kbo(:, 4,15, 9) = (/ &
     &6.6327e+01_rb,4.9885e+01_rb,3.3445e+01_rb,1.7135e+01_rb,2.0733e+00_rb/)
      kbo(:, 5,15, 9) = (/ &
     &8.5909e+01_rb,6.4569e+01_rb,4.3231e+01_rb,2.1949e+01_rb,2.0415e+00_rb/)
      kbo(:, 1,16, 9) = (/ &
     &1.6702e+01_rb,1.2688e+01_rb,8.9156e+00_rb,5.6745e+00_rb,2.1884e+00_rb/)
      kbo(:, 2,16, 9) = (/ &
     &2.4341e+01_rb,1.8405e+01_rb,1.2602e+01_rb,7.2943e+00_rb,2.1658e+00_rb/)
      kbo(:, 3,16, 9) = (/ &
     &3.3812e+01_rb,2.5504e+01_rb,1.7253e+01_rb,9.4143e+00_rb,2.1418e+00_rb/)
      kbo(:, 4,16, 9) = (/ &
     &4.5185e+01_rb,3.4031e+01_rb,2.2894e+01_rb,1.2063e+01_rb,2.1149e+00_rb/)
      kbo(:, 5,16, 9) = (/ &
     &5.8502e+01_rb,4.4016e+01_rb,2.9536e+01_rb,1.5253e+01_rb,2.0863e+00_rb/)
      kbo(:, 1,17, 9) = (/ &
     &1.1824e+01_rb,9.0729e+00_rb,6.6968e+00_rb,4.8304e+00_rb,2.1968e+00_rb/)
      kbo(:, 2,17, 9) = (/ &
     &1.7172e+01_rb,1.3045e+01_rb,9.1828e+00_rb,5.8938e+00_rb,2.1784e+00_rb/)
      kbo(:, 3,17, 9) = (/ &
     &2.3815e+01_rb,1.8013e+01_rb,1.2370e+01_rb,7.2830e+00_rb,2.1580e+00_rb/)
      kbo(:, 4,17, 9) = (/ &
     &3.1829e+01_rb,2.4018e+01_rb,1.6287e+01_rb,9.0400e+00_rb,2.1352e+00_rb/)
      kbo(:, 5,17, 9) = (/ &
     &4.1298e+01_rb,3.1117e+01_rb,2.0970e+01_rb,1.1193e+01_rb,2.1100e+00_rb/)
      kbo(:, 1,18, 9) = (/ &
     &8.5582e+00_rb,6.6960e+00_rb,5.3232e+00_rb,4.2870e+00_rb,2.1819e+00_rb/)
      kbo(:, 2,18, 9) = (/ &
     &1.2402e+01_rb,9.5125e+00_rb,7.0290e+00_rb,5.0519e+00_rb,2.1680e+00_rb/)
      kbo(:, 3,18, 9) = (/ &
     &1.7200e+01_rb,1.3071e+01_rb,9.2389e+00_rb,6.0101e+00_rb,2.1517e+00_rb/)
      kbo(:, 4,18, 9) = (/ &
     &2.3032e+01_rb,1.7428e+01_rb,1.2014e+01_rb,7.2086e+00_rb,2.1334e+00_rb/)
      kbo(:, 5,18, 9) = (/ &
     &2.9959e+01_rb,2.2618e+01_rb,1.5385e+01_rb,8.6965e+00_rb,2.1122e+00_rb/)
      kbo(:, 1,19, 9) = (/ &
     &6.0876e+00_rb,4.9589e+00_rb,4.3617e+00_rb,3.8537e+00_rb,2.1426e+00_rb/)
      kbo(:, 2,19, 9) = (/ &
     &8.8178e+00_rb,6.9019e+00_rb,5.5120e+00_rb,4.4148e+00_rb,2.1341e+00_rb/)
      kbo(:, 3,19, 9) = (/ &
     &1.2247e+01_rb,9.4065e+00_rb,7.0194e+00_rb,5.0977e+00_rb,2.1226e+00_rb/)
      kbo(:, 4,19, 9) = (/ &
     &1.6435e+01_rb,1.2508e+01_rb,8.9285e+00_rb,5.9346e+00_rb,2.1084e+00_rb/)
      kbo(:, 5,19, 9) = (/ &
     &2.1395e+01_rb,1.6206e+01_rb,1.1260e+01_rb,6.9507e+00_rb,2.0906e+00_rb/)
      kbo(:, 1,20, 9) = (/ &
     &4.7338e+00_rb,4.0618e+00_rb,3.8673e+00_rb,3.5847e+00_rb,2.0812e+00_rb/)
      kbo(:, 2,20, 9) = (/ &
     &6.8571e+00_rb,5.5334e+00_rb,4.7582e+00_rb,4.0685e+00_rb,2.0765e+00_rb/)
      kbo(:, 3,20, 9) = (/ &
     &9.5336e+00_rb,7.4447e+00_rb,5.9012e+00_rb,4.6250e+00_rb,2.0694e+00_rb/)
      kbo(:, 4,20, 9) = (/ &
     &1.2791e+01_rb,9.8219e+00_rb,7.3358e+00_rb,5.2854e+00_rb,2.0598e+00_rb/)
      kbo(:, 5,20, 9) = (/ &
     &1.6650e+01_rb,1.2677e+01_rb,9.0899e+00_rb,6.0678e+00_rb,2.0475e+00_rb/)
      kbo(:, 1,21, 9) = (/ &
     &3.8172e+00_rb,3.4906e+00_rb,3.5314e+00_rb,3.3393e+00_rb,1.9987e+00_rb/)
      kbo(:, 2,21, 9) = (/ &
     &5.5293e+00_rb,4.6505e+00_rb,4.2636e+00_rb,3.7764e+00_rb,1.9988e+00_rb/)
      kbo(:, 3,21, 9) = (/ &
     &7.6789e+00_rb,6.1523e+00_rb,5.1831e+00_rb,4.2770e+00_rb,1.9964e+00_rb/)
      kbo(:, 4,21, 9) = (/ &
     &1.0301e+01_rb,8.0282e+00_rb,6.3179e+00_rb,4.8454e+00_rb,1.9916e+00_rb/)
      kbo(:, 5,21, 9) = (/ &
     &1.3413e+01_rb,1.0296e+01_rb,7.6922e+00_rb,5.4909e+00_rb,1.9849e+00_rb/)
      kbo(:, 1,22, 9) = (/ &
     &3.2538e+00_rb,3.1636e+00_rb,3.3083e+00_rb,3.1336e+00_rb,1.8994e+00_rb/)
      kbo(:, 2,22, 9) = (/ &
     &4.6904e+00_rb,4.1275e+00_rb,3.9601e+00_rb,3.5392e+00_rb,1.9038e+00_rb/)
      kbo(:, 3,22, 9) = (/ &
     &6.4936e+00_rb,5.3647e+00_rb,4.7415e+00_rb,4.0033e+00_rb,1.9062e+00_rb/)
      kbo(:, 4,22, 9) = (/ &
     &8.6858e+00_rb,6.9049e+00_rb,5.6919e+00_rb,4.5273e+00_rb,1.9068e+00_rb/)
      kbo(:, 5,22, 9) = (/ &
     &1.1287e+01_rb,8.7705e+00_rb,6.8305e+00_rb,5.1089e+00_rb,1.9070e+00_rb/)
      kbo(:, 1,23, 9) = (/ &
     &2.7990e+00_rb,2.9004e+00_rb,3.0888e+00_rb,2.9221e+00_rb,1.7866e+00_rb/)
      kbo(:, 2,23, 9) = (/ &
     &4.0164e+00_rb,3.7193e+00_rb,3.6884e+00_rb,3.3009e+00_rb,1.7963e+00_rb/)
      kbo(:, 3,23, 9) = (/ &
     &5.5385e+00_rb,4.7530e+00_rb,4.3835e+00_rb,3.7336e+00_rb,1.8046e+00_rb/)
      kbo(:, 4,23, 9) = (/ &
     &7.3874e+00_rb,6.0323e+00_rb,5.1957e+00_rb,4.2187e+00_rb,1.8111e+00_rb/)
      kbo(:, 5,23, 9) = (/ &
     &9.5697e+00_rb,7.5707e+00_rb,6.1486e+00_rb,4.7536e+00_rb,1.8191e+00_rb/)
      kbo(:, 1,24, 9) = (/ &
     &2.4093e+00_rb,2.6666e+00_rb,2.8613e+00_rb,2.7031e+00_rb,1.6662e+00_rb/)
      kbo(:, 2,24, 9) = (/ &
     &3.4388e+00_rb,3.3672e+00_rb,3.4097e+00_rb,3.0567e+00_rb,1.6812e+00_rb/)
      kbo(:, 3,24, 9) = (/ &
     &4.7196e+00_rb,4.2384e+00_rb,4.0418e+00_rb,3.4577e+00_rb,1.6956e+00_rb/)
      kbo(:, 4,24, 9) = (/ &
     &6.2610e+00_rb,5.2923e+00_rb,4.7542e+00_rb,3.9053e+00_rb,1.7103e+00_rb/)
      kbo(:, 5,24, 9) = (/ &
     &8.0725e+00_rb,6.5459e+00_rb,5.5528e+00_rb,4.3943e+00_rb,1.7280e+00_rb/)
      kbo(:, 1,25, 9) = (/ &
     &2.1093e+00_rb,2.4777e+00_rb,2.6576e+00_rb,2.5025e+00_rb,1.5428e+00_rb/)
      kbo(:, 2,25, 9) = (/ &
     &2.9902e+00_rb,3.0964e+00_rb,3.1653e+00_rb,2.8347e+00_rb,1.5643e+00_rb/)
      kbo(:, 3,25, 9) = (/ &
     &4.0768e+00_rb,3.8394e+00_rb,3.7463e+00_rb,3.2106e+00_rb,1.5865e+00_rb/)
      kbo(:, 4,25, 9) = (/ &
     &5.3785e+00_rb,4.7237e+00_rb,4.3881e+00_rb,3.6281e+00_rb,1.6115e+00_rb/)
      kbo(:, 5,25, 9) = (/ &
     &6.9128e+00_rb,5.7727e+00_rb,5.0863e+00_rb,4.0788e+00_rb,1.6376e+00_rb/)
      kbo(:, 1,26, 9) = (/ &
     &1.8856e+00_rb,2.3262e+00_rb,2.4875e+00_rb,2.3287e+00_rb,1.4224e+00_rb/)
      kbo(:, 2,26, 9) = (/ &
     &2.6493e+00_rb,2.8890e+00_rb,2.9641e+00_rb,2.6458e+00_rb,1.4515e+00_rb/)
      kbo(:, 3,26, 9) = (/ &
     &3.5871e+00_rb,3.5368e+00_rb,3.5013e+00_rb,3.0021e+00_rb,1.4824e+00_rb/)
      kbo(:, 4,26, 9) = (/ &
     &4.7140e+00_rb,4.3024e+00_rb,4.0862e+00_rb,3.3950e+00_rb,1.5166e+00_rb/)
      kbo(:, 5,26, 9) = (/ &
     &6.0446e+00_rb,5.2045e+00_rb,4.7238e+00_rb,3.8145e+00_rb,1.5518e+00_rb/)
      kbo(:, 1,27, 9) = (/ &
     &1.7455e+00_rb,2.2279e+00_rb,2.3691e+00_rb,2.1938e+00_rb,1.3093e+00_rb/)
      kbo(:, 2,27, 9) = (/ &
     &2.4331e+00_rb,2.7544e+00_rb,2.8270e+00_rb,2.5043e+00_rb,1.3459e+00_rb/)
      kbo(:, 3,27, 9) = (/ &
     &3.2766e+00_rb,3.3516e+00_rb,3.3333e+00_rb,2.8520e+00_rb,1.3862e+00_rb/)
      kbo(:, 4,27, 9) = (/ &
     &4.2923e+00_rb,4.0432e+00_rb,3.8852e+00_rb,3.2303e+00_rb,1.4289e+00_rb/)
      kbo(:, 5,27, 9) = (/ &
     &5.4942e+00_rb,4.8565e+00_rb,4.4878e+00_rb,3.6320e+00_rb,1.4726e+00_rb/)
      kbo(:, 1,28, 9) = (/ &
     &1.6557e+00_rb,2.1623e+00_rb,2.2857e+00_rb,2.0892e+00_rb,1.2058e+00_rb/)
      kbo(:, 2,28, 9) = (/ &
     &2.2924e+00_rb,2.6651e+00_rb,2.7290e+00_rb,2.3996e+00_rb,1.2506e+00_rb/)
      kbo(:, 3,28, 9) = (/ &
     &3.0749e+00_rb,3.2332e+00_rb,3.2167e+00_rb,2.7436e+00_rb,1.2996e+00_rb/)
      kbo(:, 4,28, 9) = (/ &
     &4.0186e+00_rb,3.8816e+00_rb,3.7505e+00_rb,3.1131e+00_rb,1.3503e+00_rb/)
      kbo(:, 5,28, 9) = (/ &
     &5.1433e+00_rb,4.6442e+00_rb,4.3346e+00_rb,3.5087e+00_rb,1.4023e+00_rb/)
      kbo(:, 1,29, 9) = (/ &
     &1.6682e+00_rb,2.1786e+00_rb,2.2786e+00_rb,2.0447e+00_rb,1.1144e+00_rb/)
      kbo(:, 2,29, 9) = (/ &
     &2.2981e+00_rb,2.6823e+00_rb,2.7239e+00_rb,2.3673e+00_rb,1.1675e+00_rb/)
      kbo(:, 3,29, 9) = (/ &
     &3.0738e+00_rb,3.2536e+00_rb,3.2170e+00_rb,2.7185e+00_rb,1.2244e+00_rb/)
      kbo(:, 4,29, 9) = (/ &
     &4.0119e+00_rb,3.9041e+00_rb,3.7595e+00_rb,3.0962e+00_rb,1.2824e+00_rb/)
      kbo(:, 5,29, 9) = (/ &
     &5.1340e+00_rb,4.6678e+00_rb,4.3550e+00_rb,3.5038e+00_rb,1.3425e+00_rb/)
      kbo(:, 1,30, 9) = (/ &
     &1.7221e+00_rb,2.2297e+00_rb,2.3038e+00_rb,2.0342e+00_rb,1.0364e+00_rb/)
      kbo(:, 2,30, 9) = (/ &
     &2.3632e+00_rb,2.7462e+00_rb,2.7611e+00_rb,2.3703e+00_rb,1.0967e+00_rb/)
      kbo(:, 3,30, 9) = (/ &
     &3.1545e+00_rb,3.3332e+00_rb,3.2697e+00_rb,2.7338e+00_rb,1.1597e+00_rb/)
      kbo(:, 4,30, 9) = (/ &
     &4.1176e+00_rb,4.0031e+00_rb,3.8327e+00_rb,3.1282e+00_rb,1.2250e+00_rb/)
      kbo(:, 5,30, 9) = (/ &
     &5.2710e+00_rb,4.7910e+00_rb,4.4538e+00_rb,3.5548e+00_rb,1.2942e+00_rb/)
      kbo(:, 1,31, 9) = (/ &
     &1.8733e+00_rb,2.3643e+00_rb,2.4037e+00_rb,2.0831e+00_rb,9.7071e-01_rb/)
      kbo(:, 2,31, 9) = (/ &
     &2.5621e+00_rb,2.9163e+00_rb,2.8917e+00_rb,2.4413e+00_rb,1.0371e+00_rb/)
      kbo(:, 3,31, 9) = (/ &
     &3.4185e+00_rb,3.5458e+00_rb,3.4392e+00_rb,2.8321e+00_rb,1.1068e+00_rb/)
      kbo(:, 4,31, 9) = (/ &
     &4.4630e+00_rb,4.2725e+00_rb,4.0472e+00_rb,3.2583e+00_rb,1.1813e+00_rb/)
      kbo(:, 5,31, 9) = (/ &
     &5.7201e+00_rb,5.1339e+00_rb,4.7206e+00_rb,3.7217e+00_rb,1.2572e+00_rb/)
      kbo(:, 1,32, 9) = (/ &
     &2.0653e+00_rb,2.5337e+00_rb,2.5386e+00_rb,2.1600e+00_rb,9.1680e-01_rb/)
      kbo(:, 2,32, 9) = (/ &
     &2.8216e+00_rb,3.1306e+00_rb,3.0669e+00_rb,2.5463e+00_rb,9.8936e-01_rb/)
      kbo(:, 3,32, 9) = (/ &
     &3.7648e+00_rb,3.8151e+00_rb,3.6619e+00_rb,2.9702e+00_rb,1.0685e+00_rb/)
      kbo(:, 4,32, 9) = (/ &
     &4.9201e+00_rb,4.6180e+00_rb,4.3259e+00_rb,3.4358e+00_rb,1.1502e+00_rb/)
      kbo(:, 5,32, 9) = (/ &
     &6.3088e+00_rb,5.5739e+00_rb,5.0619e+00_rb,3.9444e+00_rb,1.2333e+00_rb/)
      kbo(:, 1,33, 9) = (/ &
     &2.3272e+00_rb,2.7583e+00_rb,2.7254e+00_rb,2.2760e+00_rb,8.7514e-01_rb/)
      kbo(:, 2,33, 9) = (/ &
     &3.1779e+00_rb,3.4161e+00_rb,3.3066e+00_rb,2.6982e+00_rb,9.5639e-01_rb/)
      kbo(:, 3,33, 9) = (/ &
     &4.2411e+00_rb,4.1786e+00_rb,3.9626e+00_rb,3.1651e+00_rb,1.0431e+00_rb/)
      kbo(:, 4,33, 9) = (/ &
     &5.5462e+00_rb,5.0836e+00_rb,4.6978e+00_rb,3.6805e+00_rb,1.1309e+00_rb/)
      kbo(:, 5,33, 9) = (/ &
     &7.1174e+00_rb,6.1675e+00_rb,5.5124e+00_rb,4.2455e+00_rb,1.2216e+00_rb/)
      kbo(:, 1,34, 9) = (/ &
     &2.5377e+00_rb,2.9389e+00_rb,2.8782e+00_rb,2.3730e+00_rb,8.4364e-01_rb/)
      kbo(:, 2,34, 9) = (/ &
     &3.4681e+00_rb,3.6495e+00_rb,3.5041e+00_rb,2.8278e+00_rb,9.3211e-01_rb/)
      kbo(:, 3,34, 9) = (/ &
     &4.6355e+00_rb,4.4791e+00_rb,4.2141e+00_rb,3.3332e+00_rb,1.0240e+00_rb/)
      kbo(:, 4,34, 9) = (/ &
     &6.0678e+00_rb,5.4727e+00_rb,5.0128e+00_rb,3.8930e+00_rb,1.1193e+00_rb/)
      kbo(:, 5,34, 9) = (/ &
     &7.8056e+00_rb,6.6744e+00_rb,5.8925e+00_rb,4.5074e+00_rb,1.2154e+00_rb/)
      kbo(:, 1,35, 9) = (/ &
     &2.6973e+00_rb,3.0775e+00_rb,2.9953e+00_rb,2.4483e+00_rb,8.1381e-01_rb/)
      kbo(:, 2,35, 9) = (/ &
     &3.6991e+00_rb,3.8360e+00_rb,3.6636e+00_rb,2.9345e+00_rb,9.0760e-01_rb/)
      kbo(:, 3,35, 9) = (/ &
     &4.9601e+00_rb,4.7300e+00_rb,4.4278e+00_rb,3.4777e+00_rb,1.0051e+00_rb/)
      kbo(:, 4,35, 9) = (/ &
     &6.5186e+00_rb,5.8127e+00_rb,5.2832e+00_rb,4.0814e+00_rb,1.1048e+00_rb/)
      kbo(:, 5,35, 9) = (/ &
     &8.4126e+00_rb,7.1259e+00_rb,6.2273e+00_rb,4.7423e+00_rb,1.2066e+00_rb/)
      kbo(:, 1,36, 9) = (/ &
     &2.7598e+00_rb,3.1389e+00_rb,3.0464e+00_rb,2.4794e+00_rb,7.8185e-01_rb/)
      kbo(:, 2,36, 9) = (/ &
     &3.8096e+00_rb,3.9341e+00_rb,3.7503e+00_rb,2.9909e+00_rb,8.7896e-01_rb/)
      kbo(:, 3,36, 9) = (/ &
     &5.1379e+00_rb,4.8766e+00_rb,4.5549e+00_rb,3.5652e+00_rb,9.8107e-01_rb/)
      kbo(:, 4,36, 9) = (/ &
     &6.7937e+00_rb,6.0302e+00_rb,5.4578e+00_rb,4.2069e+00_rb,1.0852e+00_rb/)
      kbo(:, 5,36, 9) = (/ &
     &8.8141e+00_rb,7.4326e+00_rb,6.4589e+00_rb,4.9083e+00_rb,1.1912e+00_rb/)
      kbo(:, 1,37, 9) = (/ &
     &2.6230e+00_rb,3.0408e+00_rb,2.9596e+00_rb,2.4124e+00_rb,7.3962e-01_rb/)
      kbo(:, 2,37, 9) = (/ &
     &3.6580e+00_rb,3.8393e+00_rb,3.6702e+00_rb,2.9331e+00_rb,8.3992e-01_rb/)
      kbo(:, 3,37, 9) = (/ &
     &4.9824e+00_rb,4.7866e+00_rb,4.4878e+00_rb,3.5210e+00_rb,9.4491e-01_rb/)
      kbo(:, 4,37, 9) = (/ &
     &6.6454e+00_rb,5.9484e+00_rb,5.4110e+00_rb,4.1812e+00_rb,1.0538e+00_rb/)
      kbo(:, 5,37, 9) = (/ &
     &8.6934e+00_rb,7.3704e+00_rb,6.4379e+00_rb,4.9067e+00_rb,1.1625e+00_rb/)
      kbo(:, 1,38, 9) = (/ &
     &2.5251e+00_rb,2.9721e+00_rb,2.8975e+00_rb,2.3630e+00_rb,7.0093e-01_rb/)
      kbo(:, 2,38, 9) = (/ &
     &3.5600e+00_rb,3.7826e+00_rb,3.6219e+00_rb,2.8967e+00_rb,8.0383e-01_rb/)
      kbo(:, 3,38, 9) = (/ &
     &4.9004e+00_rb,4.7489e+00_rb,4.4611e+00_rb,3.5032e+00_rb,9.1176e-01_rb/)
      kbo(:, 4,38, 9) = (/ &
     &6.5939e+00_rb,5.9343e+00_rb,5.4149e+00_rb,4.1893e+00_rb,1.0245e+00_rb/)
      kbo(:, 5,38, 9) = (/ &
     &8.7067e+00_rb,7.4030e+00_rb,6.4798e+00_rb,4.9449e+00_rb,1.1358e+00_rb/)
      kbo(:, 1,39, 9) = (/ &
     &2.4904e+00_rb,2.9537e+00_rb,2.8793e+00_rb,2.3443e+00_rb,6.6673e-01_rb/)
      kbo(:, 2,39, 9) = (/ &
     &3.5509e+00_rb,3.7917e+00_rb,3.6298e+00_rb,2.8981e+00_rb,7.7181e-01_rb/)
      kbo(:, 3,39, 9) = (/ &
     &4.9392e+00_rb,4.7973e+00_rb,4.5055e+00_rb,3.5337e+00_rb,8.8209e-01_rb/)
      kbo(:, 4,39, 9) = (/ &
     &6.7119e+00_rb,6.0406e+00_rb,5.5079e+00_rb,4.2551e+00_rb,9.9828e-01_rb/)
      kbo(:, 5,39, 9) = (/ &
     &8.9447e+00_rb,7.5957e+00_rb,6.6321e+00_rb,5.0544e+00_rb,1.1121e+00_rb/)
      kbo(:, 1,40, 9) = (/ &
     &2.2209e+00_rb,2.7364e+00_rb,2.6885e+00_rb,2.2038e+00_rb,6.2428e-01_rb/)
      kbo(:, 2,40, 9) = (/ &
     &3.2135e+00_rb,3.5469e+00_rb,3.4206e+00_rb,2.7501e+00_rb,7.3025e-01_rb/)
      kbo(:, 3,40, 9) = (/ &
     &4.5302e+00_rb,4.5210e+00_rb,4.2832e+00_rb,3.3810e+00_rb,8.4222e-01_rb/)
      kbo(:, 4,40, 9) = (/ &
     &6.2335e+00_rb,5.7214e+00_rb,5.2769e+00_rb,4.1014e+00_rb,9.6000e-01_rb/)
      kbo(:, 5,40, 9) = (/ &
     &8.4037e+00_rb,7.2349e+00_rb,6.3985e+00_rb,4.9070e+00_rb,1.0766e+00_rb/)
      kbo(:, 1,41, 9) = (/ &
     &1.9647e+00_rb,2.5201e+00_rb,2.4974e+00_rb,2.0614e+00_rb,5.8325e-01_rb/)
      kbo(:, 2,41, 9) = (/ &
     &2.8852e+00_rb,3.2999e+00_rb,3.2081e+00_rb,2.5978e+00_rb,6.8959e-01_rb/)
      kbo(:, 3,41, 9) = (/ &
     &4.1243e+00_rb,4.2421e+00_rb,4.0540e+00_rb,3.2221e+00_rb,8.0263e-01_rb/)
      kbo(:, 4,41, 9) = (/ &
     &5.7517e+00_rb,5.3978e+00_rb,5.0364e+00_rb,3.9392e+00_rb,9.2181e-01_rb/)
      kbo(:, 5,41, 9) = (/ &
     &7.8523e+00_rb,6.8657e+00_rb,6.1521e+00_rb,4.7480e+00_rb,1.0405e+00_rb/)
      kbo(:, 1,42, 9) = (/ &
     &1.7512e+00_rb,2.3349e+00_rb,2.3318e+00_rb,1.9366e+00_rb,5.4463e-01_rb/)
      kbo(:, 2,42, 9) = (/ &
     &2.6144e+00_rb,3.0905e+00_rb,3.0258e+00_rb,2.4663e+00_rb,6.5106e-01_rb/)
      kbo(:, 3,42, 9) = (/ &
     &3.7920e+00_rb,4.0117e+00_rb,3.8605e+00_rb,3.0859e+00_rb,7.6489e-01_rb/)
      kbo(:, 4,42, 9) = (/ &
     &5.3603e+00_rb,5.1368e+00_rb,4.8392e+00_rb,3.8039e+00_rb,8.8471e-01_rb/)
      kbo(:, 5,42, 9) = (/ &
     &7.4143e+00_rb,6.5758e+00_rb,5.9582e+00_rb,4.6208e+00_rb,1.0054e+00_rb/)
      kbo(:, 1,43, 9) = (/ &
     &1.5310e+00_rb,2.1360e+00_rb,2.1508e+00_rb,1.7978e+00_rb,5.0265e-01_rb/)
      kbo(:, 2,43, 9) = (/ &
     &2.3335e+00_rb,2.8636e+00_rb,2.8240e+00_rb,2.3172e+00_rb,6.0791e-01_rb/)
      kbo(:, 3,43, 9) = (/ &
     &3.4398e+00_rb,3.7600e+00_rb,3.6428e+00_rb,2.9301e+00_rb,7.2170e-01_rb/)
      kbo(:, 4,43, 9) = (/ &
     &4.9392e+00_rb,4.8535e+00_rb,4.6142e+00_rb,3.6458e+00_rb,8.4218e-01_rb/)
      kbo(:, 5,43, 9) = (/ &
     &6.9338e+00_rb,6.2557e+00_rb,5.7358e+00_rb,4.4687e+00_rb,9.6366e-01_rb/)
      kbo(:, 1,44, 9) = (/ &
     &1.3351e+00_rb,1.9503e+00_rb,1.9789e+00_rb,1.6632e+00_rb,4.6023e-01_rb/)
      kbo(:, 2,44, 9) = (/ &
     &2.0772e+00_rb,2.6507e+00_rb,2.6321e+00_rb,2.1724e+00_rb,5.6381e-01_rb/)
      kbo(:, 3,44, 9) = (/ &
     &3.1248e+00_rb,3.5269e+00_rb,3.4366e+00_rb,2.7797e+00_rb,6.7693e-01_rb/)
      kbo(:, 4,44, 9) = (/ &
     &4.5611e+00_rb,4.5989e+00_rb,4.4033e+00_rb,3.4949e+00_rb,7.9737e-01_rb/)
      kbo(:, 5,44, 9) = (/ &
     &6.5042e+00_rb,5.9719e+00_rb,5.5317e+00_rb,4.3254e+00_rb,9.1937e-01_rb/)
      kbo(:, 1,45, 9) = (/ &
     &1.1794e+00_rb,1.7969e+00_rb,1.8343e+00_rb,1.5476e+00_rb,4.2007e-01_rb/)
      kbo(:, 2,45, 9) = (/ &
     &1.8762e+00_rb,2.4795e+00_rb,2.4742e+00_rb,2.0501e+00_rb,5.2163e-01_rb/)
      kbo(:, 3,45, 9) = (/ &
     &2.8809e+00_rb,3.3442e+00_rb,3.2721e+00_rb,2.6566e+00_rb,6.3409e-01_rb/)
      kbo(:, 4,45, 9) = (/ &
     &4.2825e+00_rb,4.4131e+00_rb,4.2443e+00_rb,3.3788e+00_rb,7.5354e-01_rb/)
      kbo(:, 5,45, 9) = (/ &
     &6.2077e+00_rb,5.7830e+00_rb,5.3934e+00_rb,4.2249e+00_rb,8.7651e-01_rb/)
      kbo(:, 1,46, 9) = (/ &
     &1.0327e+00_rb,1.6453e+00_rb,1.6900e+00_rb,1.4307e+00_rb,3.8001e-01_rb/)
      kbo(:, 2,46, 9) = (/ &
     &1.6840e+00_rb,2.3071e+00_rb,2.3154e+00_rb,1.9251e+00_rb,4.7892e-01_rb/)
      kbo(:, 3,46, 9) = (/ &
     &2.6449e+00_rb,3.1623e+00_rb,3.1054e+00_rb,2.5294e+00_rb,5.8968e-01_rb/)
      kbo(:, 4,46, 9) = (/ &
     &4.0149e+00_rb,4.2299e+00_rb,4.0828e+00_rb,3.2577e+00_rb,7.0805e-01_rb/)
      kbo(:, 5,46, 9) = (/ &
     &5.9211e+00_rb,5.5984e+00_rb,5.2528e+00_rb,4.1223e+00_rb,8.3110e-01_rb/)
      kbo(:, 1,47, 9) = (/ &
     &8.6791e-01_rb,1.4650e+00_rb,1.5175e+00_rb,1.2909e+00_rb,3.3829e-01_rb/)
      kbo(:, 2,47, 9) = (/ &
     &1.4548e+00_rb,2.0919e+00_rb,2.1155e+00_rb,1.7684e+00_rb,4.3374e-01_rb/)
      kbo(:, 3,47, 9) = (/ &
     &2.3437e+00_rb,2.9171e+00_rb,2.8820e+00_rb,2.3592e+00_rb,5.4146e-01_rb/)
      kbo(:, 4,47, 9) = (/ &
     &3.6410e+00_rb,3.9658e+00_rb,3.8455e+00_rb,3.0807e+00_rb,6.5808e-01_rb/)
      kbo(:, 5,47, 9) = (/ &
     &5.4757e+00_rb,5.2982e+00_rb,5.0168e+00_rb,3.9488e+00_rb,7.8042e-01_rb/)
      kbo(:, 1,48, 9) = (/ &
     &7.4197e-01_rb,1.3175e+00_rb,1.3727e+00_rb,1.1710e+00_rb,2.9912e-01_rb/)
      kbo(:, 2,48, 9) = (/ &
     &1.2795e+00_rb,1.9175e+00_rb,1.9504e+00_rb,1.6359e+00_rb,3.9088e-01_rb/)
      kbo(:, 3,48, 9) = (/ &
     &2.1163e+00_rb,2.7243e+00_rb,2.7026e+00_rb,2.2191e+00_rb,4.9535e-01_rb/)
      kbo(:, 4,48, 9) = (/ &
     &3.3685e+00_rb,3.7692e+00_rb,3.6648e+00_rb,2.9415e+00_rb,6.0991e-01_rb/)
      kbo(:, 5,48, 9) = (/ &
     &5.1784e+00_rb,5.1043e+00_rb,4.8553e+00_rb,3.8227e+00_rb,7.3131e-01_rb/)
      kbo(:, 1,49, 9) = (/ &
     &6.5229e-01_rb,1.2051e+00_rb,1.2591e+00_rb,1.0738e+00_rb,2.6269e-01_rb/)
      kbo(:, 2,49, 9) = (/ &
     &1.1591e+00_rb,1.7908e+00_rb,1.8269e+00_rb,1.5329e+00_rb,3.5032e-01_rb/)
      kbo(:, 3,49, 9) = (/ &
     &1.9719e+00_rb,2.5959e+00_rb,2.5784e+00_rb,2.1177e+00_rb,4.5127e-01_rb/)
      kbo(:, 4,49, 9) = (/ &
     &3.2176e+00_rb,3.6617e+00_rb,3.5592e+00_rb,2.8538e+00_rb,5.6309e-01_rb/)
      kbo(:, 5,49, 9) = (/ &
     &5.0660e+00_rb,5.0444e+00_rb,4.7964e+00_rb,3.7669e+00_rb,6.8298e-01_rb/)
      kbo(:, 1,50, 9) = (/ &
     &5.4718e-01_rb,1.0700e+00_rb,1.1242e+00_rb,9.6269e-01_rb,2.3041e-01_rb/)
      kbo(:, 2,50, 9) = (/ &
     &1.0029e+00_rb,1.6230e+00_rb,1.6660e+00_rb,1.4048e+00_rb,3.1350e-01_rb/)
      kbo(:, 3,50, 9) = (/ &
     &1.7557e+00_rb,2.3977e+00_rb,2.3952e+00_rb,1.9754e+00_rb,4.1052e-01_rb/)
      kbo(:, 4,50, 9) = (/ &
     &2.9398e+00_rb,3.4467e+00_rb,3.3608e+00_rb,2.7053e+00_rb,5.1916e-01_rb/)
      kbo(:, 5,50, 9) = (/ &
     &4.7374e+00_rb,4.8217e+00_rb,4.6033e+00_rb,3.6228e+00_rb,6.3755e-01_rb/)
      kbo(:, 1,51, 9) = (/ &
     &4.4659e-01_rb,9.3305e-01_rb,9.8713e-01_rb,8.5042e-01_rb,2.0131e-01_rb/)
      kbo(:, 2,51, 9) = (/ &
     &8.4503e-01_rb,1.4458e+00_rb,1.4962e+00_rb,1.2698e+00_rb,2.7950e-01_rb/)
      kbo(:, 3,51, 9) = (/ &
     &1.5227e+00_rb,2.1776e+00_rb,2.1908e+00_rb,1.8185e+00_rb,3.7211e-01_rb/)
      kbo(:, 4,51, 9) = (/ &
     &2.6197e+00_rb,3.1878e+00_rb,3.1271e+00_rb,2.5305e+00_rb,4.7775e-01_rb/)
      kbo(:, 5,51, 9) = (/ &
     &4.3246e+00_rb,4.5333e+00_rb,4.3504e+00_rb,3.4392e+00_rb,5.9331e-01_rb/)
      kbo(:, 1,52, 9) = (/ &
     &3.7000e-01_rb,8.2088e-01_rb,8.7232e-01_rb,7.5398e-01_rb,1.7473e-01_rb/)
      kbo(:, 2,52, 9) = (/ &
     &7.2411e-01_rb,1.3013e+00_rb,1.3548e+00_rb,1.1545e+00_rb,2.4775e-01_rb/)
      kbo(:, 3,52, 9) = (/ &
     &1.3455e+00_rb,2.0001e+00_rb,2.0236e+00_rb,1.6871e+00_rb,3.3555e-01_rb/)
      kbo(:, 4,52, 9) = (/ &
     &2.3815e+00_rb,2.9886e+00_rb,2.9408e+00_rb,2.3888e+00_rb,4.3798e-01_rb/)
      kbo(:, 5,52, 9) = (/ &
     &4.0315e+00_rb,4.3264e+00_rb,4.1654e+00_rb,3.2990e+00_rb,5.5079e-01_rb/)
      kbo(:, 1,53, 9) = (/ &
     &3.1522e-01_rb,7.3236e-01_rb,7.7935e-01_rb,6.7369e-01_rb,1.5034e-01_rb/)
      kbo(:, 2,53, 9) = (/ &
     &6.3787e-01_rb,1.1908e+00_rb,1.2433e+00_rb,1.0610e+00_rb,2.1835e-01_rb/)
      kbo(:, 3,53, 9) = (/ &
     &1.2250e+00_rb,1.8717e+00_rb,1.8985e+00_rb,1.5852e+00_rb,3.0124e-01_rb/)
      kbo(:, 4,53, 9) = (/ &
     &2.2328e+00_rb,2.8593e+00_rb,2.8149e+00_rb,2.2876e+00_rb,3.9993e-01_rb/)
      kbo(:, 5,53, 9) = (/ &
     &3.8806e+00_rb,4.2255e+00_rb,4.0654e+00_rb,3.2164e+00_rb,5.0965e-01_rb/)
      kbo(:, 1,54, 9) = (/ &
     &2.3422e-01_rb,6.0076e-01_rb,6.4671e-01_rb,5.6503e-01_rb,1.2928e-01_rb/)
      kbo(:, 2,54, 9) = (/ &
     &4.9005e-01_rb,1.0010e+00_rb,1.0587e+00_rb,9.1460e-01_rb,1.9217e-01_rb/)
      kbo(:, 3,54, 9) = (/ &
     &9.7166e-01_rb,1.6014e+00_rb,1.6473e+00_rb,1.3943e+00_rb,2.7068e-01_rb/)
      kbo(:, 4,54, 9) = (/ &
     &1.8237e+00_rb,2.4909e+00_rb,2.4828e+00_rb,2.0443e+00_rb,3.6438e-01_rb/)
      kbo(:, 5,54, 9) = (/ &
     &3.2575e+00_rb,3.7456e+00_rb,3.6419e+00_rb,2.9148e+00_rb,4.7112e-01_rb/)
      kbo(:, 1,55, 9) = (/ &
     &1.5838e-01_rb,4.6639e-01_rb,5.1049e-01_rb,4.5411e-01_rb,1.1083e-01_rb/)
      kbo(:, 2,55, 9) = (/ &
     &3.4306e-01_rb,7.9590e-01_rb,8.5878e-01_rb,7.5589e-01_rb,1.6856e-01_rb/)
      kbo(:, 3,55, 9) = (/ &
     &7.0266e-01_rb,1.2949e+00_rb,1.3605e+00_rb,1.1760e+00_rb,2.4233e-01_rb/)
      kbo(:, 4,55, 9) = (/ &
     &1.3593e+00_rb,2.0439e+00_rb,2.0790e+00_rb,1.7490e+00_rb,3.3108e-01_rb/)
      kbo(:, 5,55, 9) = (/ &
     &2.4959e+00_rb,3.1221e+00_rb,3.0894e+00_rb,2.5235e+00_rb,4.3367e-01_rb/)
      kbo(:, 1,56, 9) = (/ &
     &1.0564e-01_rb,3.5968e-01_rb,4.0062e-01_rb,3.6250e-01_rb,9.4093e-02_rb/)
      kbo(:, 2,56, 9) = (/ &
     &2.3778e-01_rb,6.3076e-01_rb,6.9353e-01_rb,6.2192e-01_rb,1.4691e-01_rb/)
      kbo(:, 3,56, 9) = (/ &
     &5.0289e-01_rb,1.0440e+00_rb,1.1212e+00_rb,9.8957e-01_rb,2.1564e-01_rb/)
      kbo(:, 4,56, 9) = (/ &
     &1.0047e+00_rb,1.6731e+00_rb,1.7388e+00_rb,1.4948e+00_rb,2.9947e-01_rb/)
      kbo(:, 5,56, 9) = (/ &
     &1.8976e+00_rb,2.5969e+00_rb,2.6179e+00_rb,2.1832e+00_rb,3.9803e-01_rb/)
      kbo(:, 1,57, 9) = (/ &
     &6.9404e-02_rb,2.7490e-01_rb,3.1190e-01_rb,2.8693e-01_rb,7.9174e-02_rb/)
      kbo(:, 2,57, 9) = (/ &
     &1.6275e-01_rb,4.9714e-01_rb,5.5717e-01_rb,5.0875e-01_rb,1.2713e-01_rb/)
      kbo(:, 3,57, 9) = (/ &
     &3.5620e-01_rb,8.3998e-01_rb,9.2115e-01_rb,8.2990e-01_rb,1.9063e-01_rb/)
      kbo(:, 4,57, 9) = (/ &
     &7.3515e-01_rb,1.3656e+00_rb,1.4515e+00_rb,1.2750e+00_rb,2.6981e-01_rb/)
      kbo(:, 5,57, 9) = (/ &
     &1.4305e+00_rb,2.1547e+00_rb,2.2152e+00_rb,1.8869e+00_rb,3.6361e-01_rb/)
      kbo(:, 1,58, 9) = (/ &
     &1.2018e-02_rb,6.8819e-02_rb,1.0253e-01_rb,1.3454e-01_rb,6.6409e-02_rb/)
      kbo(:, 2,58, 9) = (/ &
     &2.9376e-02_rb,1.2869e-01_rb,1.8932e-01_rb,2.4688e-01_rb,1.0988e-01_rb/)
      kbo(:, 3,58, 9) = (/ &
     &6.6678e-02_rb,2.2200e-01_rb,3.2071e-01_rb,4.1332e-01_rb,1.6852e-01_rb/)
      kbo(:, 4,58, 9) = (/ &
     &1.4218e-01_rb,3.6629e-01_rb,5.1309e-01_rb,6.4691e-01_rb,2.4300e-01_rb/)
      kbo(:, 5,58, 9) = (/ &
     &2.8506e-01_rb,5.8551e-01_rb,7.9229e-01_rb,9.6816e-01_rb,3.3198e-01_rb/)
      kbo(:, 1,59, 9) = (/ &
     &1.0974e-02_rb,7.7674e-02_rb,1.1174e-01_rb,1.3484e-01_rb,6.0877e-02_rb/)
      kbo(:, 2,59, 9) = (/ &
     &2.7572e-02_rb,1.4726e-01_rb,2.0947e-01_rb,2.5117e-01_rb,1.0237e-01_rb/)
      kbo(:, 3,59, 9) = (/ &
     &6.4218e-02_rb,2.5542e-01_rb,3.5789e-01_rb,4.2487e-01_rb,1.5885e-01_rb/)
      kbo(:, 4,59, 9) = (/ &
     &1.3998e-01_rb,4.2159e-01_rb,5.7382e-01_rb,6.6846e-01_rb,2.3069e-01_rb/)
      kbo(:, 5,59, 9) = (/ &
     &2.8653e-01_rb,6.7350e-01_rb,8.8507e-01_rb,9.9974e-01_rb,3.1701e-01_rb/)
      kbo(:, 1,13,10) = (/ &
     &3.0887e+02_rb,2.3182e+02_rb,1.5477e+02_rb,7.7734e+01_rb,3.2340e+00_rb/)
      kbo(:, 2,13,10) = (/ &
     &4.5010e+02_rb,3.3773e+02_rb,2.2537e+02_rb,1.1301e+02_rb,3.1679e+00_rb/)
      kbo(:, 3,13,10) = (/ &
     &6.1970e+02_rb,4.6492e+02_rb,3.1014e+02_rb,1.5536e+02_rb,3.1118e+00_rb/)
      kbo(:, 4,13,10) = (/ &
     &8.1413e+02_rb,6.1074e+02_rb,4.0734e+02_rb,2.0395e+02_rb,3.0351e+00_rb/)
      kbo(:, 5,13,10) = (/ &
     &1.0345e+03_rb,7.7604e+02_rb,5.1753e+02_rb,2.5903e+02_rb,2.9847e+00_rb/)
      kbo(:, 1,14,10) = (/ &
     &1.8722e+02_rb,1.4057e+02_rb,9.3932e+01_rb,4.7308e+01_rb,3.5502e+00_rb/)
      kbo(:, 2,14,10) = (/ &
     &2.7007e+02_rb,2.0269e+02_rb,1.3533e+02_rb,6.7970e+01_rb,3.4874e+00_rb/)
      kbo(:, 3,14,10) = (/ &
     &3.6889e+02_rb,2.7681e+02_rb,1.8473e+02_rb,9.2654e+01_rb,3.4323e+00_rb/)
      kbo(:, 4,14,10) = (/ &
     &4.8380e+02_rb,3.6298e+02_rb,2.4217e+02_rb,1.2136e+02_rb,3.3766e+00_rb/)
      kbo(:, 5,14,10) = (/ &
     &6.1384e+02_rb,4.6051e+02_rb,3.0717e+02_rb,1.5385e+02_rb,3.3022e+00_rb/)
      kbo(:, 1,15,10) = (/ &
     &1.1327e+02_rb,8.5100e+01_rb,5.6942e+01_rb,2.8827e+01_rb,3.8722e+00_rb/)
      kbo(:, 2,15,10) = (/ &
     &1.6194e+02_rb,1.2160e+02_rb,8.1262e+01_rb,4.0948e+01_rb,3.8151e+00_rb/)
      kbo(:, 3,15,10) = (/ &
     &2.2047e+02_rb,1.6549e+02_rb,1.1051e+02_rb,5.5552e+01_rb,3.7597e+00_rb/)
      kbo(:, 4,15,10) = (/ &
     &2.8853e+02_rb,2.1653e+02_rb,1.4453e+02_rb,7.2538e+01_rb,3.6967e+00_rb/)
      kbo(:, 5,15,10) = (/ &
     &3.6531e+02_rb,2.7411e+02_rb,1.8291e+02_rb,9.1708e+01_rb,3.6313e+00_rb/)
      kbo(:, 1,16,10) = (/ &
     &8.0065e+01_rb,6.0189e+01_rb,4.0328e+01_rb,2.0552e+01_rb,4.2109e+00_rb/)
      kbo(:, 2,16,10) = (/ &
     &1.1401e+02_rb,8.5639e+01_rb,5.7279e+01_rb,2.8971e+01_rb,4.1507e+00_rb/)
      kbo(:, 3,16,10) = (/ &
     &1.5472e+02_rb,1.1617e+02_rb,7.7619e+01_rb,3.9099e+01_rb,4.0906e+00_rb/)
      kbo(:, 4,16,10) = (/ &
     &2.0197e+02_rb,1.5160e+02_rb,1.0123e+02_rb,5.0878e+01_rb,4.0252e+00_rb/)
      kbo(:, 5,16,10) = (/ &
     &2.5492e+02_rb,1.9130e+02_rb,1.2769e+02_rb,6.4095e+01_rb,3.9532e+00_rb/)
      kbo(:, 1,17,10) = (/ &
     &5.7968e+01_rb,4.3609e+01_rb,2.9277e+01_rb,1.5223e+01_rb,4.5287e+00_rb/)
      kbo(:, 2,17,10) = (/ &
     &8.2190e+01_rb,6.1765e+01_rb,4.1354e+01_rb,2.1049e+01_rb,4.4713e+00_rb/)
      kbo(:, 3,17,10) = (/ &
     &1.1116e+02_rb,8.3481e+01_rb,5.5816e+01_rb,2.8216e+01_rb,4.4079e+00_rb/)
      kbo(:, 4,17,10) = (/ &
     &1.4447e+02_rb,1.0846e+02_rb,7.2459e+01_rb,3.6495e+01_rb,4.3339e+00_rb/)
      kbo(:, 5,17,10) = (/ &
     &1.8168e+02_rb,1.3637e+02_rb,9.1060e+01_rb,4.5781e+01_rb,4.2592e+00_rb/)
      kbo(:, 1,18,10) = (/ &
     &4.2612e+01_rb,3.2081e+01_rb,2.1612e+01_rb,1.2271e+01_rb,4.8071e+00_rb/)
      kbo(:, 2,18,10) = (/ &
     &6.0147e+01_rb,4.5223e+01_rb,3.0329e+01_rb,1.5934e+01_rb,4.7499e+00_rb/)
      kbo(:, 3,18,10) = (/ &
     &8.0941e+01_rb,6.0812e+01_rb,4.0702e+01_rb,2.0761e+01_rb,4.6797e+00_rb/)
      kbo(:, 4,18,10) = (/ &
     &1.0473e+02_rb,7.8652e+01_rb,5.2586e+01_rb,2.6600e+01_rb,4.6080e+00_rb/)
      kbo(:, 5,18,10) = (/ &
     &1.3143e+02_rb,9.8677e+01_rb,6.5930e+01_rb,3.3232e+01_rb,4.5353e+00_rb/)
      kbo(:, 1,19,10) = (/ &
     &3.0448e+01_rb,2.2951e+01_rb,1.5620e+01_rb,1.0168e+01_rb,5.0350e+00_rb/)
      kbo(:, 2,19,10) = (/ &
     &4.2741e+01_rb,3.2162e+01_rb,2.1658e+01_rb,1.2576e+01_rb,4.9712e+00_rb/)
      kbo(:, 3,19,10) = (/ &
     &5.7239e+01_rb,4.3029e+01_rb,2.8857e+01_rb,1.5561e+01_rb,4.9060e+00_rb/)
      kbo(:, 4,19,10) = (/ &
     &7.3990e+01_rb,5.5595e+01_rb,3.7227e+01_rb,1.9263e+01_rb,4.8462e+00_rb/)
      kbo(:, 5,19,10) = (/ &
     &9.3283e+01_rb,7.0062e+01_rb,4.6859e+01_rb,2.3820e+01_rb,4.7826e+00_rb/)
      kbo(:, 1,20,10) = (/ &
     &2.3494e+01_rb,1.7742e+01_rb,1.2645e+01_rb,9.3657e+00_rb,5.1974e+00_rb/)
      kbo(:, 2,20,10) = (/ &
     &3.2767e+01_rb,2.4679e+01_rb,1.6836e+01_rb,1.0930e+01_rb,5.1468e+00_rb/)
      kbo(:, 3,20,10) = (/ &
     &4.3805e+01_rb,3.2955e+01_rb,2.2186e+01_rb,1.3029e+01_rb,5.0951e+00_rb/)
      kbo(:, 4,20,10) = (/ &
     &5.6900e+01_rb,4.2771e+01_rb,2.8686e+01_rb,1.5712e+01_rb,5.0360e+00_rb/)
      kbo(:, 5,20,10) = (/ &
     &7.2009e+01_rb,5.4098e+01_rb,3.6220e+01_rb,1.9012e+01_rb,4.9716e+00_rb/)
      kbo(:, 1,21,10) = (/ &
     &1.8574e+01_rb,1.4067e+01_rb,1.0798e+01_rb,9.1144e+00_rb,5.2989e+00_rb/)
      kbo(:, 2,21,10) = (/ &
     &2.5833e+01_rb,1.9488e+01_rb,1.3882e+01_rb,1.0250e+01_rb,5.2602e+00_rb/)
      kbo(:, 3,21,10) = (/ &
     &3.4706e+01_rb,2.6128e+01_rb,1.7872e+01_rb,1.1627e+01_rb,5.2166e+00_rb/)
      kbo(:, 4,21,10) = (/ &
     &4.5183e+01_rb,3.3979e+01_rb,2.2895e+01_rb,1.3514e+01_rb,5.1658e+00_rb/)
      kbo(:, 5,21,10) = (/ &
     &5.7162e+01_rb,4.2958e+01_rb,2.8816e+01_rb,1.5946e+01_rb,5.1078e+00_rb/)
      kbo(:, 1,22,10) = (/ &
     &1.5385e+01_rb,1.1760e+01_rb,9.8072e+00_rb,8.9715e+00_rb,5.3316e+00_rb/)
      kbo(:, 2,22,10) = (/ &
     &2.1421e+01_rb,1.6187e+01_rb,1.2154e+01_rb,1.0019e+01_rb,5.3065e+00_rb/)
      kbo(:, 3,22,10) = (/ &
     &2.8763e+01_rb,2.1675e+01_rb,1.5363e+01_rb,1.1158e+01_rb,5.2767e+00_rb/)
      kbo(:, 4,22,10) = (/ &
     &3.7378e+01_rb,2.8123e+01_rb,1.9262e+01_rb,1.2474e+01_rb,5.2393e+00_rb/)
      kbo(:, 5,22,10) = (/ &
     &4.7335e+01_rb,3.5583e+01_rb,2.3996e+01_rb,1.4183e+01_rb,5.1848e+00_rb/)
      kbo(:, 1,23,10) = (/ &
     &1.2958e+01_rb,1.0148e+01_rb,9.3026e+00_rb,8.7646e+00_rb,5.3174e+00_rb/)
      kbo(:, 2,23,10) = (/ &
     &1.8016e+01_rb,1.3712e+01_rb,1.1027e+01_rb,9.7388e+00_rb,5.3065e+00_rb/)
      kbo(:, 3,23,10) = (/ &
     &2.4116e+01_rb,1.8203e+01_rb,1.3458e+01_rb,1.0800e+01_rb,5.2863e+00_rb/)
      kbo(:, 4,23,10) = (/ &
     &3.1331e+01_rb,2.3589e+01_rb,1.6625e+01_rb,1.1918e+01_rb,5.2605e+00_rb/)
      kbo(:, 5,23,10) = (/ &
     &3.9751e+01_rb,2.9901e+01_rb,2.0473e+01_rb,1.3180e+01_rb,5.2164e+00_rb/)
      kbo(:, 1,24,10) = (/ &
     &1.0965e+01_rb,8.9178e+00_rb,8.8968e+00_rb,8.4756e+00_rb,5.2609e+00_rb/)
      kbo(:, 2,24,10) = (/ &
     &1.5180e+01_rb,1.1787e+01_rb,1.0354e+01_rb,9.3845e+00_rb,5.2622e+00_rb/)
      kbo(:, 3,24,10) = (/ &
     &2.0304e+01_rb,1.5418e+01_rb,1.2131e+01_rb,1.0371e+01_rb,5.2552e+00_rb/)
      kbo(:, 4,24,10) = (/ &
     &2.6453e+01_rb,1.9955e+01_rb,1.4586e+01_rb,1.1412e+01_rb,5.2428e+00_rb/)
      kbo(:, 5,24,10) = (/ &
     &3.3727e+01_rb,2.5409e+01_rb,1.7807e+01_rb,1.2537e+01_rb,5.2080e+00_rb/)
      kbo(:, 1,25,10) = (/ &
     &9.4802e+00_rb,8.0678e+00_rb,8.5346e+00_rb,8.1689e+00_rb,5.1602e+00_rb/)
      kbo(:, 2,25,10) = (/ &
     &1.3091e+01_rb,1.0412e+01_rb,9.8804e+00_rb,9.0420e+00_rb,5.1798e+00_rb/)
      kbo(:, 3,25,10) = (/ &
     &1.7556e+01_rb,1.3502e+01_rb,1.1390e+01_rb,9.9740e+00_rb,5.1919e+00_rb/)
      kbo(:, 4,25,10) = (/ &
     &2.2937e+01_rb,1.7389e+01_rb,1.3318e+01_rb,1.0956e+01_rb,5.1855e+00_rb/)
      kbo(:, 5,25,10) = (/ &
     &2.9162e+01_rb,2.2007e+01_rb,1.5910e+01_rb,1.2043e+01_rb,5.1753e+00_rb/)
      kbo(:, 1,26,10) = (/ &
     &8.4883e+00_rb,7.6029e+00_rb,8.2305e+00_rb,7.8781e+00_rb,5.0300e+00_rb/)
      kbo(:, 2,26,10) = (/ &
     &1.1732e+01_rb,9.5611e+00_rb,9.4929e+00_rb,8.7340e+00_rb,5.0703e+00_rb/)
      kbo(:, 3,26,10) = (/ &
     &1.5710e+01_rb,1.2274e+01_rb,1.0941e+01_rb,9.6483e+00_rb,5.1008e+00_rb/)
      kbo(:, 4,26,10) = (/ &
     &2.0377e+01_rb,1.5577e+01_rb,1.2626e+01_rb,1.0617e+01_rb,5.1143e+00_rb/)
      kbo(:, 5,26,10) = (/ &
     &2.5762e+01_rb,1.9497e+01_rb,1.4646e+01_rb,1.1691e+01_rb,5.1337e+00_rb/)
      kbo(:, 1,27,10) = (/ &
     &7.9831e+00_rb,7.4657e+00_rb,8.0386e+00_rb,7.6712e+00_rb,4.8762e+00_rb/)
      kbo(:, 2,27,10) = (/ &
     &1.1007e+01_rb,9.2258e+00_rb,9.2928e+00_rb,8.5396e+00_rb,4.9402e+00_rb/)
      kbo(:, 3,27,10) = (/ &
     &1.4621e+01_rb,1.1594e+01_rb,1.0757e+01_rb,9.4652e+00_rb,4.9915e+00_rb/)
      kbo(:, 4,27,10) = (/ &
     &1.8847e+01_rb,1.4547e+01_rb,1.2326e+01_rb,1.0475e+01_rb,5.0304e+00_rb/)
      kbo(:, 5,27,10) = (/ &
     &2.3744e+01_rb,1.8045e+01_rb,1.4056e+01_rb,1.1545e+01_rb,5.0808e+00_rb/)
      kbo(:, 1,28,10) = (/ &
     &7.7237e+00_rb,7.4713e+00_rb,7.9486e+00_rb,7.5349e+00_rb,4.7172e+00_rb/)
      kbo(:, 2,28,10) = (/ &
     &1.0564e+01_rb,9.1113e+00_rb,9.2602e+00_rb,8.4182e+00_rb,4.8066e+00_rb/)
      kbo(:, 3,28,10) = (/ &
     &1.3950e+01_rb,1.1225e+01_rb,1.0715e+01_rb,9.3795e+00_rb,4.8773e+00_rb/)
      kbo(:, 4,28,10) = (/ &
     &1.7938e+01_rb,1.3957e+01_rb,1.2227e+01_rb,1.0437e+01_rb,4.9557e+00_rb/)
      kbo(:, 5,28,10) = (/ &
     &2.2520e+01_rb,1.7201e+01_rb,1.3827e+01_rb,1.1495e+01_rb,5.0256e+00_rb/)
      kbo(:, 1,29,10) = (/ &
     &7.9252e+00_rb,7.7575e+00_rb,8.1492e+00_rb,7.5791e+00_rb,4.5608e+00_rb/)
      kbo(:, 2,29,10) = (/ &
     &1.0756e+01_rb,9.3871e+00_rb,9.5612e+00_rb,8.5083e+00_rb,4.6721e+00_rb/)
      kbo(:, 3,29,10) = (/ &
     &1.4151e+01_rb,1.1455e+01_rb,1.1026e+01_rb,9.5615e+00_rb,4.7742e+00_rb/)
      kbo(:, 4,29,10) = (/ &
     &1.8135e+01_rb,1.4153e+01_rb,1.2557e+01_rb,1.0665e+01_rb,4.8851e+00_rb/)
      kbo(:, 5,29,10) = (/ &
     &2.2757e+01_rb,1.7409e+01_rb,1.4161e+01_rb,1.1730e+01_rb,4.9738e+00_rb/)
      kbo(:, 1,30,10) = (/ &
     &8.3260e+00_rb,8.1445e+00_rb,8.5115e+00_rb,7.7075e+00_rb,4.4112e+00_rb/)
      kbo(:, 2,30,10) = (/ &
     &1.1239e+01_rb,9.8059e+00_rb,9.9796e+00_rb,8.7369e+00_rb,4.5506e+00_rb/)
      kbo(:, 3,30,10) = (/ &
     &1.4731e+01_rb,1.1916e+01_rb,1.1495e+01_rb,9.8748e+00_rb,4.7011e+00_rb/)
      kbo(:, 4,30,10) = (/ &
     &1.8856e+01_rb,1.4710e+01_rb,1.3064e+01_rb,1.1008e+01_rb,4.8328e+00_rb/)
      kbo(:, 5,30,10) = (/ &
     &2.3705e+01_rb,1.8124e+01_rb,1.4716e+01_rb,1.2119e+01_rb,4.9257e+00_rb/)
      kbo(:, 1,31,10) = (/ &
     &9.2196e+00_rb,8.8116e+00_rb,9.1465e+00_rb,8.0622e+00_rb,4.2873e+00_rb/)
      kbo(:, 2,31,10) = (/ &
     &1.2392e+01_rb,1.0607e+01_rb,1.0704e+01_rb,9.2367e+00_rb,4.4733e+00_rb/)
      kbo(:, 3,31,10) = (/ &
     &1.6211e+01_rb,1.2983e+01_rb,1.2309e+01_rb,1.0449e+01_rb,4.6511e+00_rb/)
      kbo(:, 4,31,10) = (/ &
     &2.0762e+01_rb,1.6100e+01_rb,1.3996e+01_rb,1.1659e+01_rb,4.7878e+00_rb/)
      kbo(:, 5,31,10) = (/ &
     &2.6150e+01_rb,1.9908e+01_rb,1.5819e+01_rb,1.2859e+01_rb,4.9198e+00_rb/)
      kbo(:, 1,32,10) = (/ &
     &1.0359e+01_rb,9.6137e+00_rb,9.8752e+00_rb,8.5830e+00_rb,4.2174e+00_rb/)
      kbo(:, 2,32,10) = (/ &
     &1.3877e+01_rb,1.1631e+01_rb,1.1538e+01_rb,9.8576e+00_rb,4.4285e+00_rb/)
      kbo(:, 3,32,10) = (/ &
     &1.8140e+01_rb,1.4380e+01_rb,1.3269e+01_rb,1.1160e+01_rb,4.5970e+00_rb/)
      kbo(:, 4,32,10) = (/ &
     &2.3264e+01_rb,1.7927e+01_rb,1.5118e+01_rb,1.2461e+01_rb,4.7493e+00_rb/)
      kbo(:, 5,32,10) = (/ &
     &2.9418e+01_rb,2.2303e+01_rb,1.7227e+01_rb,1.3746e+01_rb,4.9087e+00_rb/)
      kbo(:, 1,33,10) = (/ &
     &1.1859e+01_rb,1.0640e+01_rb,1.0773e+01_rb,9.2560e+00_rb,4.1651e+00_rb/)
      kbo(:, 2,33,10) = (/ &
     &1.5884e+01_rb,1.3011e+01_rb,1.2572e+01_rb,1.0635e+01_rb,4.3715e+00_rb/)
      kbo(:, 3,33,10) = (/ &
     &2.0782e+01_rb,1.6260e+01_rb,1.4502e+01_rb,1.2047e+01_rb,4.5449e+00_rb/)
      kbo(:, 4,33,10) = (/ &
     &2.6731e+01_rb,2.0436e+01_rb,1.6585e+01_rb,1.3454e+01_rb,4.7445e+00_rb/)
      kbo(:, 5,33,10) = (/ &
     &3.3983e+01_rb,2.5659e+01_rb,1.9186e+01_rb,1.4880e+01_rb,4.8947e+00_rb/)
      kbo(:, 1,34,10) = (/ &
     &1.3139e+01_rb,1.1513e+01_rb,1.1537e+01_rb,9.8375e+00_rb,4.1077e+00_rb/)
      kbo(:, 2,34,10) = (/ &
     &1.7636e+01_rb,1.4225e+01_rb,1.3489e+01_rb,1.1311e+01_rb,4.3160e+00_rb/)
      kbo(:, 3,34,10) = (/ &
     &2.3141e+01_rb,1.7962e+01_rb,1.5594e+01_rb,1.2810e+01_rb,4.5443e+00_rb/)
      kbo(:, 4,34,10) = (/ &
     &2.9960e+01_rb,2.2778e+01_rb,1.7924e+01_rb,1.4335e+01_rb,4.7397e+00_rb/)
      kbo(:, 5,34,10) = (/ &
     &3.8261e+01_rb,2.8825e+01_rb,2.1079e+01_rb,1.5921e+01_rb,4.9149e+00_rb/)
      kbo(:, 1,35,10) = (/ &
     &1.4201e+01_rb,1.2261e+01_rb,1.2213e+01_rb,1.0330e+01_rb,4.0399e+00_rb/)
      kbo(:, 2,35,10) = (/ &
     &1.9135e+01_rb,1.5305e+01_rb,1.4323e+01_rb,1.1895e+01_rb,4.2894e+00_rb/)
      kbo(:, 3,35,10) = (/ &
     &2.5318e+01_rb,1.9545e+01_rb,1.6589e+01_rb,1.3508e+01_rb,4.5397e+00_rb/)
      kbo(:, 4,35,10) = (/ &
     &3.3021e+01_rb,2.5025e+01_rb,1.9285e+01_rb,1.5170e+01_rb,4.7565e+00_rb/)
      kbo(:, 5,35,10) = (/ &
     &4.2461e+01_rb,3.1954e+01_rb,2.3012e+01_rb,1.6958e+01_rb,4.9332e+00_rb/)
      kbo(:, 1,36,10) = (/ &
     &1.4798e+01_rb,1.2717e+01_rb,1.2670e+01_rb,1.0657e+01_rb,3.9724e+00_rb/)
      kbo(:, 2,36,10) = (/ &
     &2.0092e+01_rb,1.6016e+01_rb,1.4899e+01_rb,1.2316e+01_rb,4.2681e+00_rb/)
      kbo(:, 3,36,10) = (/ &
     &2.6852e+01_rb,2.0677e+01_rb,1.7358e+01_rb,1.4052e+01_rb,4.5368e+00_rb/)
      kbo(:, 4,36,10) = (/ &
     &3.5362e+01_rb,2.6747e+01_rb,2.0391e+01_rb,1.5864e+01_rb,4.7528e+00_rb/)
      kbo(:, 5,36,10) = (/ &
     &4.5869e+01_rb,3.4503e+01_rb,2.4644e+01_rb,1.7859e+01_rb,4.9480e+00_rb/)
      kbo(:, 1,37,10) = (/ &
     &1.4369e+01_rb,1.2549e+01_rb,1.2615e+01_rb,1.0629e+01_rb,3.8875e+00_rb/)
      kbo(:, 2,37,10) = (/ &
     &1.9757e+01_rb,1.5840e+01_rb,1.4922e+01_rb,1.2359e+01_rb,4.2086e+00_rb/)
      kbo(:, 3,37,10) = (/ &
     &2.6704e+01_rb,2.0605e+01_rb,1.7493e+01_rb,1.4185e+01_rb,4.4864e+00_rb/)
      kbo(:, 4,37,10) = (/ &
     &3.5588e+01_rb,2.6935e+01_rb,2.0650e+01_rb,1.6109e+01_rb,4.7274e+00_rb/)
      kbo(:, 5,37,10) = (/ &
     &4.6659e+01_rb,3.5099e+01_rb,2.5115e+01_rb,1.8232e+01_rb,4.9396e+00_rb/)
      kbo(:, 1,38,10) = (/ &
     &1.4144e+01_rb,1.2516e+01_rb,1.2646e+01_rb,1.0674e+01_rb,3.8121e+00_rb/)
      kbo(:, 2,38,10) = (/ &
     &1.9692e+01_rb,1.5869e+01_rb,1.5055e+01_rb,1.2490e+01_rb,4.1526e+00_rb/)
      kbo(:, 3,38,10) = (/ &
     &2.6928e+01_rb,2.0800e+01_rb,1.7766e+01_rb,1.4428e+01_rb,4.4461e+00_rb/)
      kbo(:, 4,38,10) = (/ &
     &3.6364e+01_rb,2.7528e+01_rb,2.1122e+01_rb,1.6470e+01_rb,4.6931e+00_rb/)
      kbo(:, 5,38,10) = (/ &
     &4.8190e+01_rb,3.6248e+01_rb,2.5894e+01_rb,1.8774e+01_rb,4.9281e+00_rb/)
      kbo(:, 1,39,10) = (/ &
     &1.4265e+01_rb,1.2699e+01_rb,1.2840e+01_rb,1.0848e+01_rb,3.7413e+00_rb/)
      kbo(:, 2,39,10) = (/ &
     &2.0117e+01_rb,1.6218e+01_rb,1.5387e+01_rb,1.2783e+01_rb,4.1060e+00_rb/)
      kbo(:, 3,39,10) = (/ &
     &2.7852e+01_rb,2.1494e+01_rb,1.8320e+01_rb,1.4851e+01_rb,4.4149e+00_rb/)
      kbo(:, 4,39,10) = (/ &
     &3.8047e+01_rb,2.8782e+01_rb,2.1981e+01_rb,1.7084e+01_rb,4.6584e+00_rb/)
      kbo(:, 5,39,10) = (/ &
     &5.1036e+01_rb,3.8378e+01_rb,2.7311e+01_rb,1.9599e+01_rb,4.9209e+00_rb/)
      kbo(:, 1,40,10) = (/ &
     &1.3045e+01_rb,1.2072e+01_rb,1.2332e+01_rb,1.0515e+01_rb,3.6349e+00_rb/)
      kbo(:, 2,40,10) = (/ &
     &1.8669e+01_rb,1.5342e+01_rb,1.4892e+01_rb,1.2474e+01_rb,4.0124e+00_rb/)
      kbo(:, 3,40,10) = (/ &
     &2.6244e+01_rb,2.0390e+01_rb,1.7841e+01_rb,1.4589e+01_rb,4.3337e+00_rb/)
      kbo(:, 4,40,10) = (/ &
     &3.6291e+01_rb,2.7528e+01_rb,2.1453e+01_rb,1.6896e+01_rb,4.5936e+00_rb/)
      kbo(:, 5,40,10) = (/ &
     &4.9350e+01_rb,3.7123e+01_rb,2.6680e+01_rb,1.9495e+01_rb,4.8667e+00_rb/)
      kbo(:, 1,41,10) = (/ &
     &1.1803e+01_rb,1.1439e+01_rb,1.1786e+01_rb,1.0149e+01_rb,3.5171e+00_rb/)
      kbo(:, 2,41,10) = (/ &
     &1.7194e+01_rb,1.4495e+01_rb,1.4344e+01_rb,1.2127e+01_rb,3.9113e+00_rb/)
      kbo(:, 3,41,10) = (/ &
     &2.4539e+01_rb,1.9243e+01_rb,1.7307e+01_rb,1.4275e+01_rb,4.2454e+00_rb/)
      kbo(:, 4,41,10) = (/ &
     &3.4381e+01_rb,2.6175e+01_rb,2.0871e+01_rb,1.6646e+01_rb,4.5236e+00_rb/)
      kbo(:, 5,41,10) = (/ &
     &4.7387e+01_rb,3.5674e+01_rb,2.5938e+01_rb,1.9314e+01_rb,4.8021e+00_rb/)
      kbo(:, 1,42,10) = (/ &
     &1.0775e+01_rb,1.0903e+01_rb,1.1326e+01_rb,9.8291e+00_rb,3.4027e+00_rb/)
      kbo(:, 2,42,10) = (/ &
     &1.5966e+01_rb,1.3847e+01_rb,1.3900e+01_rb,1.1829e+01_rb,3.8068e+00_rb/)
      kbo(:, 3,42,10) = (/ &
     &2.3163e+01_rb,1.8350e+01_rb,1.6888e+01_rb,1.4033e+01_rb,4.1524e+00_rb/)
      kbo(:, 4,42,10) = (/ &
     &3.2905e+01_rb,2.5146e+01_rb,2.0462e+01_rb,1.6474e+01_rb,4.4507e+00_rb/)
      kbo(:, 5,42,10) = (/ &
     &4.5959e+01_rb,3.4632e+01_rb,2.5458e+01_rb,1.9233e+01_rb,4.7372e+00_rb/)
      kbo(:, 1,43,10) = (/ &
     &9.6785e+00_rb,1.0293e+01_rb,1.0788e+01_rb,9.4380e+00_rb,3.2651e+00_rb/)
      kbo(:, 2,43,10) = (/ &
     &1.4578e+01_rb,1.3133e+01_rb,1.3380e+01_rb,1.1460e+01_rb,3.6820e+00_rb/)
      kbo(:, 3,43,10) = (/ &
     &2.1568e+01_rb,1.7343e+01_rb,1.6367e+01_rb,1.3704e+01_rb,4.0432e+00_rb/)
      kbo(:, 4,43,10) = (/ &
     &3.1160e+01_rb,2.3941e+01_rb,1.9961e+01_rb,1.6215e+01_rb,4.3540e+00_rb/)
      kbo(:, 5,43,10) = (/ &
     &4.4112e+01_rb,3.3295e+01_rb,2.4834e+01_rb,1.9063e+01_rb,4.6487e+00_rb/)
      kbo(:, 1,44,10) = (/ &
     &8.6703e+00_rb,9.7103e+00_rb,1.0263e+01_rb,9.0428e+00_rb,3.1167e+00_rb/)
      kbo(:, 2,44,10) = (/ &
     &1.3323e+01_rb,1.2507e+01_rb,1.2872e+01_rb,1.1099e+01_rb,3.5429e+00_rb/)
      kbo(:, 3,44,10) = (/ &
     &2.0055e+01_rb,1.6451e+01_rb,1.5900e+01_rb,1.3381e+01_rb,3.9213e+00_rb/)
      kbo(:, 4,44,10) = (/ &
     &2.9535e+01_rb,2.2835e+01_rb,1.9530e+01_rb,1.5970e+01_rb,4.2481e+00_rb/)
      kbo(:, 5,44,10) = (/ &
     &4.2449e+01_rb,3.2099e+01_rb,2.4329e+01_rb,1.8926e+01_rb,4.5571e+00_rb/)
      kbo(:, 1,45,10) = (/ &
     &7.8818e+00_rb,9.2454e+00_rb,9.8403e+00_rb,8.7130e+00_rb,2.9638e+00_rb/)
      kbo(:, 2,45,10) = (/ &
     &1.2355e+01_rb,1.2035e+01_rb,1.2482e+01_rb,1.0822e+01_rb,3.4038e+00_rb/)
      kbo(:, 3,45,10) = (/ &
     &1.8948e+01_rb,1.5842e+01_rb,1.5587e+01_rb,1.3162e+01_rb,3.7919e+00_rb/)
      kbo(:, 4,45,10) = (/ &
     &2.8443e+01_rb,2.2122e+01_rb,1.9322e+01_rb,1.5846e+01_rb,4.1405e+00_rb/)
      kbo(:, 5,45,10) = (/ &
     &4.1558e+01_rb,3.1475e+01_rb,2.4163e+01_rb,1.8949e+01_rb,4.4650e+00_rb/)
      kbo(:, 1,46,10) = (/ &
     &7.1089e+00_rb,8.7733e+00_rb,9.3941e+00_rb,8.3547e+00_rb,2.8054e+00_rb/)
      kbo(:, 2,46,10) = (/ &
     &1.1401e+01_rb,1.1562e+01_rb,1.2067e+01_rb,1.0520e+01_rb,3.2536e+00_rb/)
      kbo(:, 3,46,10) = (/ &
     &1.7834e+01_rb,1.5248e+01_rb,1.5247e+01_rb,1.2927e+01_rb,3.6528e+00_rb/)
      kbo(:, 4,46,10) = (/ &
     &2.7249e+01_rb,2.1375e+01_rb,1.9093e+01_rb,1.5705e+01_rb,4.0192e+00_rb/)
      kbo(:, 5,46,10) = (/ &
     &4.0651e+01_rb,3.0866e+01_rb,2.4015e+01_rb,1.8943e+01_rb,4.3574e+00_rb/)
      kbo(:, 1,47,10) = (/ &
     &6.1649e+00_rb,8.1403e+00_rb,8.7798e+00_rb,7.8562e+00_rb,2.6253e+00_rb/)
      kbo(:, 2,47,10) = (/ &
     &1.0138e+01_rb,1.0874e+01_rb,1.1437e+01_rb,1.0055e+01_rb,3.0768e+00_rb/)
      kbo(:, 3,47,10) = (/ &
     &1.6220e+01_rb,1.4389e+01_rb,1.4631e+01_rb,1.2502e+01_rb,3.4927e+00_rb/)
      kbo(:, 4,47,10) = (/ &
     &2.5292e+01_rb,2.0083e+01_rb,1.8506e+01_rb,1.5334e+01_rb,3.8713e+00_rb/)
      kbo(:, 5,47,10) = (/ &
     &3.8579e+01_rb,2.9419e+01_rb,2.3431e+01_rb,1.8659e+01_rb,4.2295e+00_rb/)
      kbo(:, 1,48,10) = (/ &
     &5.4216e+00_rb,7.6284e+00_rb,8.2781e+00_rb,7.4282e+00_rb,2.4448e+00_rb/)
      kbo(:, 2,48,10) = (/ &
     &9.1674e+00_rb,1.0353e+01_rb,1.0955e+01_rb,9.6794e+00_rb,2.8941e+00_rb/)
      kbo(:, 3,48,10) = (/ &
     &1.5043e+01_rb,1.3839e+01_rb,1.4203e+01_rb,1.2197e+01_rb,3.3245e+00_rb/)
      kbo(:, 4,48,10) = (/ &
     &2.3974e+01_rb,1.9288e+01_rb,1.8167e+01_rb,1.5125e+01_rb,3.7165e+00_rb/)
      kbo(:, 5,48,10) = (/ &
     &3.7305e+01_rb,2.8554e+01_rb,2.3217e+01_rb,1.8594e+01_rb,4.0932e+00_rb/)
      kbo(:, 1,49,10) = (/ &
     &4.8939e+00_rb,7.2548e+00_rb,7.9080e+00_rb,7.0906e+00_rb,2.2660e+00_rb/)
      kbo(:, 2,49,10) = (/ &
     &8.5415e+00_rb,1.0039e+01_rb,1.0662e+01_rb,9.4308e+00_rb,2.7166e+00_rb/)
      kbo(:, 3,49,10) = (/ &
     &1.4366e+01_rb,1.3618e+01_rb,1.4041e+01_rb,1.2074e+01_rb,3.1539e+00_rb/)
      kbo(:, 4,49,10) = (/ &
     &2.3519e+01_rb,1.9097e+01_rb,1.8194e+01_rb,1.5161e+01_rb,3.5625e+00_rb/)
      kbo(:, 5,49,10) = (/ &
     &3.7354e+01_rb,2.8637e+01_rb,2.3540e+01_rb,1.8862e+01_rb,3.9655e+00_rb/)
      kbo(:, 1,50,10) = (/ &
     &4.2089e+00_rb,6.7050e+00_rb,7.3787e+00_rb,6.6334e+00_rb,2.0951e+00_rb/)
      kbo(:, 2,50,10) = (/ &
     &7.5781e+00_rb,9.4477e+00_rb,1.0132e+01_rb,8.9977e+00_rb,2.5467e+00_rb/)
      kbo(:, 3,50,10) = (/ &
     &1.3098e+01_rb,1.3002e+01_rb,1.3521e+01_rb,1.1708e+01_rb,2.9888e+00_rb/)
      kbo(:, 4,50,10) = (/ &
     &2.2021e+01_rb,1.8246e+01_rb,1.7755e+01_rb,1.4869e+01_rb,3.4160e+00_rb/)
      kbo(:, 5,50,10) = (/ &
     &3.5802e+01_rb,2.7586e+01_rb,2.3214e+01_rb,1.8702e+01_rb,3.8321e+00_rb/)
      kbo(:, 1,51,10) = (/ &
     &3.5246e+00_rb,6.0952e+00_rb,6.7836e+00_rb,6.1221e+00_rb,1.9285e+00_rb/)
      kbo(:, 2,51,10) = (/ &
     &6.5392e+00_rb,8.7579e+00_rb,9.5007e+00_rb,8.4899e+00_rb,2.3790e+00_rb/)
      kbo(:, 3,51,10) = (/ &
     &1.1662e+01_rb,1.2207e+01_rb,1.2854e+01_rb,1.1229e+01_rb,2.8289e+00_rb/)
      kbo(:, 4,51,10) = (/ &
     &2.0094e+01_rb,1.7146e+01_rb,1.7077e+01_rb,1.4419e+01_rb,3.2637e+00_rb/)
      kbo(:, 5,51,10) = (/ &
     &3.3544e+01_rb,2.6031e+01_rb,2.2586e+01_rb,1.8311e+01_rb,3.6955e+00_rb/)
      kbo(:, 1,52,10) = (/ &
     &2.9972e+00_rb,5.5809e+00_rb,6.2670e+00_rb,5.6729e+00_rb,1.7637e+00_rb/)
      kbo(:, 2,52,10) = (/ &
     &5.7401e+00_rb,8.2025e+00_rb,8.9834e+00_rb,8.0597e+00_rb,2.2115e+00_rb/)
      kbo(:, 3,52,10) = (/ &
     &1.0553e+01_rb,1.1618e+01_rb,1.2357e+01_rb,1.0861e+01_rb,2.6710e+00_rb/)
      kbo(:, 4,52,10) = (/ &
     &1.8666e+01_rb,1.6392e+01_rb,1.6645e+01_rb,1.4126e+01_rb,3.1091e+00_rb/)
      kbo(:, 5,52,10) = (/ &
     &3.2058e+01_rb,2.5069e+01_rb,2.2259e+01_rb,1.8128e+01_rb,3.5578e+00_rb/)
      kbo(:, 1,53,10) = (/ &
     &2.6079e+00_rb,5.1734e+00_rb,5.8449e+00_rb,5.2987e+00_rb,1.6026e+00_rb/)
      kbo(:, 2,53,10) = (/ &
     &5.1862e+00_rb,7.8030e+00_rb,8.6055e+00_rb,7.7246e+00_rb,2.0456e+00_rb/)
      kbo(:, 3,53,10) = (/ &
     &9.8254e+00_rb,1.1267e+01_rb,1.2065e+01_rb,1.0633e+01_rb,2.5075e+00_rb/)
      kbo(:, 4,53,10) = (/ &
     &1.7902e+01_rb,1.6069e+01_rb,1.6506e+01_rb,1.4053e+01_rb,2.9528e+00_rb/)
      kbo(:, 5,53,10) = (/ &
     &3.1626e+01_rb,2.4842e+01_rb,2.2382e+01_rb,1.8272e+01_rb,3.4166e+00_rb/)
      kbo(:, 1,54,10) = (/ &
     &1.9692e+00_rb,4.4492e+00_rb,5.0971e+00_rb,4.6730e+00_rb,1.4560e+00_rb/)
      kbo(:, 2,54,10) = (/ &
     &4.0777e+00_rb,6.8483e+00_rb,7.6942e+00_rb,6.9748e+00_rb,1.8918e+00_rb/)
      kbo(:, 3,54,10) = (/ &
     &7.9636e+00_rb,1.0044e+01_rb,1.0960e+01_rb,9.7978e+00_rb,2.3486e+00_rb/)
      kbo(:, 4,54,10) = (/ &
     &1.4982e+01_rb,1.4434e+01_rb,1.5164e+01_rb,1.3125e+01_rb,2.8028e+00_rb/)
      kbo(:, 5,54,10) = (/ &
     &2.7158e+01_rb,2.1902e+01_rb,2.0733e+01_rb,1.7211e+01_rb,3.2543e+00_rb/)
      kbo(:, 1,55,10) = (/ &
     &1.3507e+00_rb,3.6433e+00_rb,4.2566e+00_rb,3.9661e+00_rb,1.3130e+00_rb/)
      kbo(:, 2,55,10) = (/ &
     &2.9154e+00_rb,5.7215e+00_rb,6.5830e+00_rb,6.0536e+00_rb,1.7424e+00_rb/)
      kbo(:, 3,55,10) = (/ &
     &5.8789e+00_rb,8.5105e+00_rb,9.5339e+00_rb,8.6749e+00_rb,2.1951e+00_rb/)
      kbo(:, 4,55,10) = (/ &
     &1.1400e+01_rb,1.2318e+01_rb,1.3311e+01_rb,1.1788e+01_rb,2.6530e+00_rb/)
      kbo(:, 5,55,10) = (/ &
     &2.1258e+01_rb,1.8155e+01_rb,1.8294e+01_rb,1.5532e+01_rb,3.0990e+00_rb/)
      kbo(:, 1,56,10) = (/ &
     &9.1417e-01_rb,2.9676e+00_rb,3.5310e+00_rb,3.3437e+00_rb,1.1797e+00_rb/)
      kbo(:, 2,56,10) = (/ &
     &2.0541e+00_rb,4.7744e+00_rb,5.6130e+00_rb,5.2303e+00_rb,1.5997e+00_rb/)
      kbo(:, 3,56,10) = (/ &
     &4.2988e+00_rb,7.2236e+00_rb,8.2994e+00_rb,7.6574e+00_rb,2.0435e+00_rb/)
      kbo(:, 4,56,10) = (/ &
     &8.5903e+00_rb,1.0539e+01_rb,1.1687e+01_rb,1.0572e+01_rb,2.5005e+00_rb/)
      kbo(:, 5,56,10) = (/ &
     &1.6506e+01_rb,1.5331e+01_rb,1.6174e+01_rb,1.4030e+01_rb,2.9531e+00_rb/)
      kbo(:, 1,57,10) = (/ &
     &6.0887e-01_rb,2.4030e+00_rb,2.9065e+00_rb,2.7971e+00_rb,1.0495e+00_rb/)
      kbo(:, 2,57,10) = (/ &
     &1.4281e+00_rb,3.9782e+00_rb,4.7656e+00_rb,4.4965e+00_rb,1.4561e+00_rb/)
      kbo(:, 3,57,10) = (/ &
     &3.1054e+00_rb,6.1238e+00_rb,7.2122e+00_rb,6.7209e+00_rb,1.8934e+00_rb/)
      kbo(:, 4,57,10) = (/ &
     &6.4056e+00_rb,9.0325e+00_rb,1.0294e+01_rb,9.4596e+00_rb,2.3523e+00_rb/)
      kbo(:, 5,57,10) = (/ &
     &1.2719e+01_rb,1.3167e+01_rb,1.4312e+01_rb,1.2682e+01_rb,2.8029e+00_rb/)
      kbo(:, 1,58,10) = (/ &
     &1.0707e-01_rb,6.3953e-01_rb,1.0127e+00_rb,1.3896e+00_rb,9.3114e-01_rb/)
      kbo(:, 2,58,10) = (/ &
     &2.6195e-01_rb,1.0902e+00_rb,1.7100e+00_rb,2.2949e+00_rb,1.3241e+00_rb/)
      kbo(:, 3,58,10) = (/ &
     &5.9147e-01_rb,1.7109e+00_rb,2.6503e+00_rb,3.5010e+00_rb,1.7544e+00_rb/)
      kbo(:, 4,58,10) = (/ &
     &1.2606e+00_rb,2.5523e+00_rb,3.8433e+00_rb,5.0138e+00_rb,2.2018e+00_rb/)
      kbo(:, 5,58,10) = (/ &
     &2.5874e+00_rb,3.7372e+00_rb,5.3689e+00_rb,6.8137e+00_rb,2.6612e+00_rb/)
      kbo(:, 1,59,10) = (/ &
     &9.9298e-02_rb,7.6667e-01_rb,1.1645e+00_rb,1.4612e+00_rb,8.8547e-01_rb/)
      kbo(:, 2,59,10) = (/ &
     &2.4968e-01_rb,1.3224e+00_rb,1.9807e+00_rb,2.4248e+00_rb,1.2655e+00_rb/)
      kbo(:, 3,59,10) = (/ &
     &5.7897e-01_rb,2.0921e+00_rb,3.0941e+00_rb,3.7073e+00_rb,1.6878e+00_rb/)
      kbo(:, 4,59,10) = (/ &
     &1.2660e+00_rb,3.1094e+00_rb,4.5051e+00_rb,5.3308e+00_rb,2.1316e+00_rb/)
      kbo(:, 5,59,10) = (/ &
     &2.6665e+00_rb,4.5101e+00_rb,6.2520e+00_rb,7.2627e+00_rb,2.5824e+00_rb/)
      kbo(:, 1,13,11) = (/ &
     &4.9137e+02_rb,3.6866e+02_rb,2.4595e+02_rb,1.2325e+02_rb,3.7154e+00_rb/)
      kbo(:, 2,13,11) = (/ &
     &7.1842e+02_rb,5.3894e+02_rb,3.5946e+02_rb,1.7998e+02_rb,3.6255e+00_rb/)
      kbo(:, 3,13,11) = (/ &
     &9.9186e+02_rb,7.4402e+02_rb,4.9618e+02_rb,2.4833e+02_rb,3.5452e+00_rb/)
      kbo(:, 4,13,11) = (/ &
     &1.3059e+03_rb,9.7954e+02_rb,6.5319e+02_rb,3.2683e+02_rb,3.4736e+00_rb/)
      kbo(:, 5,13,11) = (/ &
     &1.6553e+03_rb,1.2416e+03_rb,8.2787e+02_rb,4.1417e+02_rb,3.4158e+00_rb/)
      kbo(:, 1,14,11) = (/ &
     &3.1242e+02_rb,2.3443e+02_rb,1.5644e+02_rb,7.8454e+01_rb,4.0956e+00_rb/)
      kbo(:, 2,14,11) = (/ &
     &4.5149e+02_rb,3.3873e+02_rb,2.2597e+02_rb,1.1322e+02_rb,4.0089e+00_rb/)
      kbo(:, 3,14,11) = (/ &
     &6.1672e+02_rb,4.6265e+02_rb,3.0858e+02_rb,1.5452e+02_rb,3.9350e+00_rb/)
      kbo(:, 4,14,11) = (/ &
     &8.0574e+02_rb,6.0442e+02_rb,4.0309e+02_rb,2.0177e+02_rb,3.8665e+00_rb/)
      kbo(:, 5,14,11) = (/ &
     &1.0145e+03_rb,7.6099e+02_rb,5.0748e+02_rb,2.5397e+02_rb,3.7755e+00_rb/)
      kbo(:, 1,15,11) = (/ &
     &1.9759e+02_rb,1.4830e+02_rb,9.9018e+01_rb,4.9739e+01_rb,4.5712e+00_rb/)
      kbo(:, 2,15,11) = (/ &
     &2.8230e+02_rb,2.1184e+02_rb,1.4137e+02_rb,7.0907e+01_rb,4.4810e+00_rb/)
      kbo(:, 3,15,11) = (/ &
     &3.8219e+02_rb,2.8675e+02_rb,1.9132e+02_rb,9.5881e+01_rb,4.3973e+00_rb/)
      kbo(:, 4,15,11) = (/ &
     &4.9586e+02_rb,3.7201e+02_rb,2.4816e+02_rb,1.2430e+02_rb,4.3207e+00_rb/)
      kbo(:, 5,15,11) = (/ &
     &6.2301e+02_rb,4.6738e+02_rb,3.1174e+02_rb,1.5610e+02_rb,4.2419e+00_rb/)
      kbo(:, 1,16,11) = (/ &
     &1.4534e+02_rb,1.0912e+02_rb,7.2899e+01_rb,3.6687e+01_rb,5.0599e+00_rb/)
      kbo(:, 2,16,11) = (/ &
     &2.0548e+02_rb,1.5423e+02_rb,1.0297e+02_rb,5.1715e+01_rb,4.9736e+00_rb/)
      kbo(:, 3,16,11) = (/ &
     &2.7637e+02_rb,2.0739e+02_rb,1.3841e+02_rb,6.9439e+01_rb,4.8882e+00_rb/)
      kbo(:, 4,16,11) = (/ &
     &3.5789e+02_rb,2.6854e+02_rb,1.7918e+02_rb,8.9827e+01_rb,4.8052e+00_rb/)
      kbo(:, 5,16,11) = (/ &
     &4.4907e+02_rb,3.3692e+02_rb,2.2477e+02_rb,1.1262e+02_rb,4.7153e+00_rb/)
      kbo(:, 1,17,11) = (/ &
     &1.0823e+02_rb,8.1288e+01_rb,5.4344e+01_rb,2.7412e+01_rb,5.5645e+00_rb/)
      kbo(:, 2,17,11) = (/ &
     &1.5200e+02_rb,1.1412e+02_rb,7.6233e+01_rb,3.8355e+01_rb,5.4748e+00_rb/)
      kbo(:, 3,17,11) = (/ &
     &2.0403e+02_rb,1.5314e+02_rb,1.0225e+02_rb,5.1364e+01_rb,5.3859e+00_rb/)
      kbo(:, 4,17,11) = (/ &
     &2.6361e+02_rb,1.9782e+02_rb,1.3204e+02_rb,6.6255e+01_rb,5.2931e+00_rb/)
      kbo(:, 5,17,11) = (/ &
     &3.3009e+02_rb,2.4768e+02_rb,1.6528e+02_rb,8.2867e+01_rb,5.1961e+00_rb/)
      kbo(:, 1,18,11) = (/ &
     &8.1232e+01_rb,6.1036e+01_rb,4.0843e+01_rb,2.0768e+01_rb,6.0688e+00_rb/)
      kbo(:, 2,18,11) = (/ &
     &1.1380e+02_rb,8.5466e+01_rb,5.7129e+01_rb,2.8802e+01_rb,5.9719e+00_rb/)
      kbo(:, 3,18,11) = (/ &
     &1.5225e+02_rb,1.1430e+02_rb,7.6349e+01_rb,3.8404e+01_rb,5.8780e+00_rb/)
      kbo(:, 4,18,11) = (/ &
     &1.9624e+02_rb,1.4729e+02_rb,9.8340e+01_rb,4.9394e+01_rb,5.7787e+00_rb/)
      kbo(:, 5,18,11) = (/ &
     &2.4493e+02_rb,1.8381e+02_rb,1.2268e+02_rb,6.1561e+01_rb,5.6709e+00_rb/)
      kbo(:, 1,19,11) = (/ &
     &5.9301e+01_rb,4.4585e+01_rb,2.9875e+01_rb,1.6238e+01_rb,6.5385e+00_rb/)
      kbo(:, 2,19,11) = (/ &
     &8.2717e+01_rb,6.2145e+01_rb,4.1575e+01_rb,2.1335e+01_rb,6.4509e+00_rb/)
      kbo(:, 3,19,11) = (/ &
     &1.1031e+02_rb,8.2837e+01_rb,5.5366e+01_rb,2.7948e+01_rb,6.3522e+00_rb/)
      kbo(:, 4,19,11) = (/ &
     &1.4167e+02_rb,1.0635e+02_rb,7.1041e+01_rb,3.5737e+01_rb,6.2392e+00_rb/)
      kbo(:, 5,19,11) = (/ &
     &1.7630e+02_rb,1.3232e+02_rb,8.8349e+01_rb,4.4378e+01_rb,6.1191e+00_rb/)
      kbo(:, 1,20,11) = (/ &
     &4.6675e+01_rb,3.5111e+01_rb,2.3575e+01_rb,1.4266e+01_rb,6.9620e+00_rb/)
      kbo(:, 2,20,11) = (/ &
     &6.4787e+01_rb,4.8692e+01_rb,3.2603e+01_rb,1.7833e+01_rb,6.8738e+00_rb/)
      kbo(:, 3,20,11) = (/ &
     &8.5965e+01_rb,6.4574e+01_rb,4.3185e+01_rb,2.2381e+01_rb,6.7710e+00_rb/)
      kbo(:, 4,20,11) = (/ &
     &1.0993e+02_rb,8.2545e+01_rb,5.5164e+01_rb,2.7968e+01_rb,6.6598e+00_rb/)
      kbo(:, 5,20,11) = (/ &
     &1.3666e+02_rb,1.0259e+02_rb,6.8531e+01_rb,3.4499e+01_rb,6.5455e+00_rb/)
      kbo(:, 1,21,11) = (/ &
     &3.7501e+01_rb,2.8227e+01_rb,1.9281e+01_rb,1.3115e+01_rb,7.3104e+00_rb/)
      kbo(:, 2,21,11) = (/ &
     &5.1743e+01_rb,3.8905e+01_rb,2.6108e+01_rb,1.5684e+01_rb,7.2251e+00_rb/)
      kbo(:, 3,21,11) = (/ &
     &6.8349e+01_rb,5.1361e+01_rb,3.4375e+01_rb,1.9019e+01_rb,7.1334e+00_rb/)
      kbo(:, 4,21,11) = (/ &
     &8.7361e+01_rb,6.5620e+01_rb,4.3882e+01_rb,2.3034e+01_rb,7.0346e+00_rb/)
      kbo(:, 5,21,11) = (/ &
     &1.0904e+02_rb,8.1880e+01_rb,5.4724e+01_rb,2.7964e+01_rb,6.9286e+00_rb/)
      kbo(:, 1,22,11) = (/ &
     &3.1289e+01_rb,2.3566e+01_rb,1.6825e+01_rb,1.2828e+01_rb,7.5760e+00_rb/)
      kbo(:, 2,22,11) = (/ &
     &4.2825e+01_rb,3.2216e+01_rb,2.1971e+01_rb,1.4588e+01_rb,7.5079e+00_rb/)
      kbo(:, 3,22,11) = (/ &
     &5.6403e+01_rb,4.2401e+01_rb,2.8462e+01_rb,1.6984e+01_rb,7.4270e+00_rb/)
      kbo(:, 4,22,11) = (/ &
     &7.2294e+01_rb,5.4320e+01_rb,3.6353e+01_rb,2.0165e+01_rb,7.3380e+00_rb/)
      kbo(:, 5,22,11) = (/ &
     &9.0391e+01_rb,6.7890e+01_rb,4.5398e+01_rb,2.4037e+01_rb,7.2398e+00_rb/)
      kbo(:, 1,23,11) = (/ &
     &2.6113e+01_rb,1.9694e+01_rb,1.4984e+01_rb,1.2807e+01_rb,7.7662e+00_rb/)
      kbo(:, 2,23,11) = (/ &
     &3.5611e+01_rb,2.6805e+01_rb,1.8996e+01_rb,1.4145e+01_rb,7.7141e+00_rb/)
      kbo(:, 3,23,11) = (/ &
     &4.7007e+01_rb,3.5353e+01_rb,2.4112e+01_rb,1.5803e+01_rb,7.6529e+00_rb/)
      kbo(:, 4,23,11) = (/ &
     &6.0260e+01_rb,4.5296e+01_rb,3.0446e+01_rb,1.8119e+01_rb,7.5779e+00_rb/)
      kbo(:, 5,23,11) = (/ &
     &7.5402e+01_rb,5.6642e+01_rb,3.7917e+01_rb,2.1118e+01_rb,7.4889e+00_rb/)
      kbo(:, 1,24,11) = (/ &
     &2.1744e+01_rb,1.6531e+01_rb,1.3667e+01_rb,1.2705e+01_rb,7.8734e+00_rb/)
      kbo(:, 2,24,11) = (/ &
     &2.9665e+01_rb,2.2353e+01_rb,1.6710e+01_rb,1.3918e+01_rb,7.8466e+00_rb/)
      kbo(:, 3,24,11) = (/ &
     &3.9143e+01_rb,2.9459e+01_rb,2.0787e+01_rb,1.5226e+01_rb,7.8065e+00_rb/)
      kbo(:, 4,24,11) = (/ &
     &5.0222e+01_rb,3.7761e+01_rb,2.5773e+01_rb,1.6824e+01_rb,7.7511e+00_rb/)
      kbo(:, 5,24,11) = (/ &
     &6.2928e+01_rb,4.7282e+01_rb,3.1824e+01_rb,1.8958e+01_rb,7.6816e+00_rb/)
      kbo(:, 1,25,11) = (/ &
     &1.8556e+01_rb,1.4391e+01_rb,1.2986e+01_rb,1.2542e+01_rb,7.9223e+00_rb/)
      kbo(:, 2,25,11) = (/ &
     &2.5288e+01_rb,1.9166e+01_rb,1.5272e+01_rb,1.3696e+01_rb,7.9207e+00_rb/)
      kbo(:, 3,25,11) = (/ &
     &3.3328e+01_rb,2.5101e+01_rb,1.8448e+01_rb,1.4913e+01_rb,7.9039e+00_rb/)
      kbo(:, 4,25,11) = (/ &
     &4.2799e+01_rb,3.2187e+01_rb,2.2557e+01_rb,1.6216e+01_rb,7.8725e+00_rb/)
      kbo(:, 5,25,11) = (/ &
     &5.3844e+01_rb,4.0473e+01_rb,2.7585e+01_rb,1.7777e+01_rb,7.8216e+00_rb/)
      kbo(:, 1,26,11) = (/ &
     &1.6321e+01_rb,1.3012e+01_rb,1.2671e+01_rb,1.2389e+01_rb,7.9293e+00_rb/)
      kbo(:, 2,26,11) = (/ &
     &2.2176e+01_rb,1.7013e+01_rb,1.4541e+01_rb,1.3492e+01_rb,7.9549e+00_rb/)
      kbo(:, 3,26,11) = (/ &
     &2.9236e+01_rb,2.2092e+01_rb,1.7014e+01_rb,1.4647e+01_rb,7.9628e+00_rb/)
      kbo(:, 4,26,11) = (/ &
     &3.7703e+01_rb,2.8378e+01_rb,2.0439e+01_rb,1.5868e+01_rb,7.9537e+00_rb/)
      kbo(:, 5,26,11) = (/ &
     &4.7623e+01_rb,3.5826e+01_rb,2.4857e+01_rb,1.7188e+01_rb,7.9177e+00_rb/)
      kbo(:, 1,27,11) = (/ &
     &1.5028e+01_rb,1.2264e+01_rb,1.2582e+01_rb,1.2313e+01_rb,7.9116e+00_rb/)
      kbo(:, 2,27,11) = (/ &
     &2.0393e+01_rb,1.5855e+01_rb,1.4339e+01_rb,1.3407e+01_rb,7.9620e+00_rb/)
      kbo(:, 3,27,11) = (/ &
     &2.6998e+01_rb,2.0503e+01_rb,1.6441e+01_rb,1.4550e+01_rb,7.9958e+00_rb/)
      kbo(:, 4,27,11) = (/ &
     &3.4948e+01_rb,2.6342e+01_rb,1.9460e+01_rb,1.5757e+01_rb,8.0119e+00_rb/)
      kbo(:, 5,27,11) = (/ &
     &4.4164e+01_rb,3.3250e+01_rb,2.3464e+01_rb,1.7068e+01_rb,7.9872e+00_rb/)
      kbo(:, 1,28,11) = (/ &
     &1.4343e+01_rb,1.1935e+01_rb,1.2606e+01_rb,1.2286e+01_rb,7.8708e+00_rb/)
      kbo(:, 2,28,11) = (/ &
     &1.9534e+01_rb,1.5338e+01_rb,1.4333e+01_rb,1.3405e+01_rb,7.9546e+00_rb/)
      kbo(:, 3,28,11) = (/ &
     &2.5909e+01_rb,1.9792e+01_rb,1.6354e+01_rb,1.4575e+01_rb,8.0164e+00_rb/)
      kbo(:, 4,28,11) = (/ &
     &3.3450e+01_rb,2.5271e+01_rb,1.9104e+01_rb,1.5810e+01_rb,8.0394e+00_rb/)
      kbo(:, 5,28,11) = (/ &
     &4.2201e+01_rb,3.1789e+01_rb,2.2776e+01_rb,1.7191e+01_rb,8.0384e+00_rb/)
      kbo(:, 1,29,11) = (/ &
     &1.4799e+01_rb,1.2334e+01_rb,1.2951e+01_rb,1.2470e+01_rb,7.8248e+00_rb/)
      kbo(:, 2,29,11) = (/ &
     &2.0134e+01_rb,1.5845e+01_rb,1.4778e+01_rb,1.3682e+01_rb,7.9390e+00_rb/)
      kbo(:, 3,29,11) = (/ &
     &2.6563e+01_rb,2.0336e+01_rb,1.6926e+01_rb,1.4933e+01_rb,8.0194e+00_rb/)
      kbo(:, 4,29,11) = (/ &
     &3.4143e+01_rb,2.5818e+01_rb,1.9648e+01_rb,1.6307e+01_rb,8.0590e+00_rb/)
      kbo(:, 5,29,11) = (/ &
     &4.2969e+01_rb,3.2368e+01_rb,2.3282e+01_rb,1.7794e+01_rb,8.0931e+00_rb/)
      kbo(:, 1,30,11) = (/ &
     &1.5772e+01_rb,1.3097e+01_rb,1.3464e+01_rb,1.2783e+01_rb,7.7812e+00_rb/)
      kbo(:, 2,30,11) = (/ &
     &2.1344e+01_rb,1.6785e+01_rb,1.5505e+01_rb,1.4108e+01_rb,7.9232e+00_rb/)
      kbo(:, 3,30,11) = (/ &
     &2.7999e+01_rb,2.1429e+01_rb,1.7759e+01_rb,1.5492e+01_rb,8.0114e+00_rb/)
      kbo(:, 4,30,11) = (/ &
     &3.5848e+01_rb,2.7096e+01_rb,2.0562e+01_rb,1.7031e+01_rb,8.0836e+00_rb/)
      kbo(:, 5,30,11) = (/ &
     &4.5004e+01_rb,3.3889e+01_rb,2.4347e+01_rb,1.8589e+01_rb,8.1429e+00_rb/)
      kbo(:, 1,31,11) = (/ &
     &1.7716e+01_rb,1.4511e+01_rb,1.4478e+01_rb,1.3396e+01_rb,7.7478e+00_rb/)
      kbo(:, 2,31,11) = (/ &
     &2.3853e+01_rb,1.8632e+01_rb,1.6766e+01_rb,1.4874e+01_rb,7.8974e+00_rb/)
      kbo(:, 3,31,11) = (/ &
     &3.1182e+01_rb,2.3768e+01_rb,1.9223e+01_rb,1.6499e+01_rb,8.0055e+00_rb/)
      kbo(:, 4,31,11) = (/ &
     &3.9834e+01_rb,3.0054e+01_rb,2.2404e+01_rb,1.8165e+01_rb,8.1138e+00_rb/)
      kbo(:, 5,31,11) = (/ &
     &4.9938e+01_rb,3.7588e+01_rb,2.6690e+01_rb,1.9788e+01_rb,8.1481e+00_rb/)
      kbo(:, 1,32,11) = (/ &
     &2.0151e+01_rb,1.6250e+01_rb,1.5786e+01_rb,1.4153e+01_rb,7.7024e+00_rb/)
      kbo(:, 2,32,11) = (/ &
     &2.7022e+01_rb,2.0930e+01_rb,1.8243e+01_rb,1.5874e+01_rb,7.8766e+00_rb/)
      kbo(:, 3,32,11) = (/ &
     &3.5292e+01_rb,2.6776e+01_rb,2.1045e+01_rb,1.7666e+01_rb,8.0370e+00_rb/)
      kbo(:, 4,32,11) = (/ &
     &4.5050e+01_rb,3.3940e+01_rb,2.4811e+01_rb,1.9434e+01_rb,8.1312e+00_rb/)
      kbo(:, 5,32,11) = (/ &
     &5.6497e+01_rb,4.2506e+01_rb,2.9790e+01_rb,2.1188e+01_rb,8.1409e+00_rb/)
      kbo(:, 1,33,11) = (/ &
     &2.3401e+01_rb,1.8533e+01_rb,1.7371e+01_rb,1.5193e+01_rb,7.6839e+00_rb/)
      kbo(:, 2,33,11) = (/ &
     &3.1308e+01_rb,2.4009e+01_rb,2.0094e+01_rb,1.7155e+01_rb,7.9059e+00_rb/)
      kbo(:, 3,33,11) = (/ &
     &4.0829e+01_rb,3.0856e+01_rb,2.3455e+01_rb,1.9076e+01_rb,8.0587e+00_rb/)
      kbo(:, 4,33,11) = (/ &
     &5.2189e+01_rb,3.9272e+01_rb,2.8099e+01_rb,2.1008e+01_rb,8.1274e+00_rb/)
      kbo(:, 5,33,11) = (/ &
     &6.5586e+01_rb,4.9320e+01_rb,3.4075e+01_rb,2.2993e+01_rb,8.2027e+00_rb/)
      kbo(:, 1,34,11) = (/ &
     &2.6313e+01_rb,2.0607e+01_rb,1.8746e+01_rb,1.6201e+01_rb,7.7021e+00_rb/)
      kbo(:, 2,34,11) = (/ &
     &3.5155e+01_rb,2.6805e+01_rb,2.1758e+01_rb,1.8283e+01_rb,7.9397e+00_rb/)
      kbo(:, 3,34,11) = (/ &
     &4.5927e+01_rb,3.4632e+01_rb,2.5705e+01_rb,2.0353e+01_rb,8.0605e+00_rb/)
      kbo(:, 4,34,11) = (/ &
     &5.8835e+01_rb,4.4252e+01_rb,3.1197e+01_rb,2.2458e+01_rb,8.1311e+00_rb/)
      kbo(:, 5,34,11) = (/ &
     &7.4250e+01_rb,5.5815e+01_rb,3.8184e+01_rb,2.4701e+01_rb,8.2150e+00_rb/)
      kbo(:, 1,35,11) = (/ &
     &2.8876e+01_rb,2.2455e+01_rb,1.9946e+01_rb,1.7124e+01_rb,7.7224e+00_rb/)
      kbo(:, 2,35,11) = (/ &
     &3.8715e+01_rb,2.9405e+01_rb,2.3284e+01_rb,1.9326e+01_rb,7.9252e+00_rb/)
      kbo(:, 3,35,11) = (/ &
     &5.0778e+01_rb,3.8237e+01_rb,2.7902e+01_rb,2.1577e+01_rb,8.0258e+00_rb/)
      kbo(:, 4,35,11) = (/ &
     &6.5355e+01_rb,4.9140e+01_rb,3.4259e+01_rb,2.3860e+01_rb,8.1543e+00_rb/)
      kbo(:, 5,35,11) = (/ &
     &8.3003e+01_rb,6.2372e+01_rb,4.2365e+01_rb,2.6463e+01_rb,8.1938e+00_rb/)
      kbo(:, 1,36,11) = (/ &
     &3.0603e+01_rb,2.3731e+01_rb,2.0817e+01_rb,1.7816e+01_rb,7.6876e+00_rb/)
      kbo(:, 2,36,11) = (/ &
     &4.1304e+01_rb,3.1313e+01_rb,2.4462e+01_rb,2.0172e+01_rb,7.8715e+00_rb/)
      kbo(:, 3,36,11) = (/ &
     &5.4533e+01_rb,4.1035e+01_rb,2.9662e+01_rb,2.2576e+01_rb,7.9878e+00_rb/)
      kbo(:, 4,36,11) = (/ &
     &7.0736e+01_rb,5.3175e+01_rb,3.6832e+01_rb,2.5052e+01_rb,8.1404e+00_rb/)
      kbo(:, 5,36,11) = (/ &
     &9.0535e+01_rb,6.8017e+01_rb,4.6007e+01_rb,2.8044e+01_rb,8.2152e+00_rb/)
      kbo(:, 1,37,11) = (/ &
     &3.0255e+01_rb,2.3559e+01_rb,2.0925e+01_rb,1.7969e+01_rb,7.6093e+00_rb/)
      kbo(:, 2,37,11) = (/ &
     &4.1246e+01_rb,3.1319e+01_rb,2.4672e+01_rb,2.0442e+01_rb,7.8000e+00_rb/)
      kbo(:, 3,37,11) = (/ &
     &5.5008e+01_rb,4.1404e+01_rb,3.0029e+01_rb,2.2965e+01_rb,7.9652e+00_rb/)
      kbo(:, 4,37,11) = (/ &
     &7.2100e+01_rb,5.4198e+01_rb,3.7564e+01_rb,2.5602e+01_rb,8.1174e+00_rb/)
      kbo(:, 5,37,11) = (/ &
     &9.3252e+01_rb,7.0052e+01_rb,4.7363e+01_rb,2.8813e+01_rb,8.2379e+00_rb/)
      kbo(:, 1,38,11) = (/ &
     &3.0319e+01_rb,2.3676e+01_rb,2.1203e+01_rb,1.8223e+01_rb,7.5198e+00_rb/)
      kbo(:, 2,38,11) = (/ &
     &4.1784e+01_rb,3.1741e+01_rb,2.5125e+01_rb,2.0837e+01_rb,7.7243e+00_rb/)
      kbo(:, 3,38,11) = (/ &
     &5.6316e+01_rb,4.2385e+01_rb,3.0775e+01_rb,2.3517e+01_rb,7.9363e+00_rb/)
      kbo(:, 4,38,11) = (/ &
     &7.4582e+01_rb,5.6053e+01_rb,3.8823e+01_rb,2.6361e+01_rb,8.1242e+00_rb/)
      kbo(:, 5,38,11) = (/ &
     &9.7510e+01_rb,7.3243e+01_rb,4.9498e+01_rb,2.9889e+01_rb,8.2559e+00_rb/)
      kbo(:, 1,39,11) = (/ &
     &3.1111e+01_rb,2.4293e+01_rb,2.1786e+01_rb,1.8666e+01_rb,7.4239e+00_rb/)
      kbo(:, 2,39,11) = (/ &
     &4.3408e+01_rb,3.2962e+01_rb,2.6016e+01_rb,2.1461e+01_rb,7.6345e+00_rb/)
      kbo(:, 3,39,11) = (/ &
     &5.9150e+01_rb,4.4506e+01_rb,3.2172e+01_rb,2.4357e+01_rb,7.9192e+00_rb/)
      kbo(:, 4,39,11) = (/ &
     &7.9245e+01_rb,5.9549e+01_rb,4.1085e+01_rb,2.7494e+01_rb,8.1496e+00_rb/)
      kbo(:, 5,39,11) = (/ &
     &1.0467e+02_rb,7.8610e+01_rb,5.2990e+01_rb,3.1523e+01_rb,8.2789e+00_rb/)
      kbo(:, 1,40,11) = (/ &
     &2.8917e+01_rb,2.2849e+01_rb,2.1212e+01_rb,1.8303e+01_rb,7.3082e+00_rb/)
      kbo(:, 2,40,11) = (/ &
     &4.0995e+01_rb,3.1270e+01_rb,2.5375e+01_rb,2.1193e+01_rb,7.5582e+00_rb/)
      kbo(:, 3,40,11) = (/ &
     &5.6673e+01_rb,4.2684e+01_rb,3.1298e+01_rb,2.4183e+01_rb,7.8695e+00_rb/)
      kbo(:, 4,40,11) = (/ &
     &7.6946e+01_rb,5.7826e+01_rb,4.0125e+01_rb,2.7426e+01_rb,8.1312e+00_rb/)
      kbo(:, 5,40,11) = (/ &
     &1.0293e+02_rb,7.7305e+01_rb,5.2205e+01_rb,3.1496e+01_rb,8.2689e+00_rb/)
      kbo(:, 1,41,11) = (/ &
     &2.6634e+01_rb,2.1385e+01_rb,2.0561e+01_rb,1.7885e+01_rb,7.1901e+00_rb/)
      kbo(:, 2,41,11) = (/ &
     &3.8375e+01_rb,2.9451e+01_rb,2.4682e+01_rb,2.0867e+01_rb,7.4740e+00_rb/)
      kbo(:, 3,41,11) = (/ &
     &5.3928e+01_rb,4.0671e+01_rb,3.0356e+01_rb,2.3955e+01_rb,7.8182e+00_rb/)
      kbo(:, 4,41,11) = (/ &
     &7.4310e+01_rb,5.5850e+01_rb,3.9033e+01_rb,2.7309e+01_rb,8.0950e+00_rb/)
      kbo(:, 5,41,11) = (/ &
     &1.0067e+02_rb,7.5610e+01_rb,5.1206e+01_rb,3.1414e+01_rb,8.2644e+00_rb/)
      kbo(:, 1,42,11) = (/ &
     &2.4738e+01_rb,2.0233e+01_rb,2.0020e+01_rb,1.7547e+01_rb,7.0619e+00_rb/)
      kbo(:, 2,42,11) = (/ &
     &3.6234e+01_rb,2.7982e+01_rb,2.4175e+01_rb,2.0637e+01_rb,7.4025e+00_rb/)
      kbo(:, 3,42,11) = (/ &
     &5.1779e+01_rb,3.9115e+01_rb,2.9714e+01_rb,2.3853e+01_rb,7.7568e+00_rb/)
      kbo(:, 4,42,11) = (/ &
     &7.2527e+01_rb,5.4511e+01_rb,3.8357e+01_rb,2.7357e+01_rb,8.0593e+00_rb/)
      kbo(:, 5,42,11) = (/ &
     &9.9602e+01_rb,7.4807e+01_rb,5.0774e+01_rb,3.1579e+01_rb,8.2618e+00_rb/)
      kbo(:, 1,43,11) = (/ &
     &2.2598e+01_rb,1.8979e+01_rb,1.9339e+01_rb,1.7092e+01_rb,6.9063e+00_rb/)
      kbo(:, 2,43,11) = (/ &
     &3.3721e+01_rb,2.6289e+01_rb,2.3541e+01_rb,2.0300e+01_rb,7.2922e+00_rb/)
      kbo(:, 3,43,11) = (/ &
     &4.9059e+01_rb,3.7168e+01_rb,2.8920e+01_rb,2.3665e+01_rb,7.6599e+00_rb/)
      kbo(:, 4,43,11) = (/ &
     &6.9947e+01_rb,5.2582e+01_rb,3.7401e+01_rb,2.7324e+01_rb,7.9953e+00_rb/)
      kbo(:, 5,43,11) = (/ &
     &9.7637e+01_rb,7.3334e+01_rb,4.9954e+01_rb,3.1634e+01_rb,8.2449e+00_rb/)
      kbo(:, 1,44,11) = (/ &
     &2.0598e+01_rb,1.7877e+01_rb,1.8670e+01_rb,1.6634e+01_rb,6.7349e+00_rb/)
      kbo(:, 2,44,11) = (/ &
     &3.1385e+01_rb,2.4755e+01_rb,2.2973e+01_rb,1.9973e+01_rb,7.1646e+00_rb/)
      kbo(:, 3,44,11) = (/ &
     &4.6533e+01_rb,3.5381e+01_rb,2.8231e+01_rb,2.3518e+01_rb,7.5534e+00_rb/)
      kbo(:, 4,44,11) = (/ &
     &6.7572e+01_rb,5.0818e+01_rb,3.6582e+01_rb,2.7361e+01_rb,7.9184e+00_rb/)
      kbo(:, 5,44,11) = (/ &
     &9.6116e+01_rb,7.2195e+01_rb,4.9367e+01_rb,3.1819e+01_rb,8.2107e+00_rb/)
      kbo(:, 1,45,11) = (/ &
     &1.9103e+01_rb,1.7116e+01_rb,1.8163e+01_rb,1.6283e+01_rb,6.5605e+00_rb/)
      kbo(:, 2,45,11) = (/ &
     &2.9658e+01_rb,2.3682e+01_rb,2.2618e+01_rb,1.9789e+01_rb,7.0239e+00_rb/)
      kbo(:, 3,45,11) = (/ &
     &4.4853e+01_rb,3.4227e+01_rb,2.7909e+01_rb,2.3555e+01_rb,7.4383e+00_rb/)
      kbo(:, 4,45,11) = (/ &
     &6.6390e+01_rb,4.9954e+01_rb,3.6314e+01_rb,2.7640e+01_rb,7.8580e+00_rb/)
      kbo(:, 5,45,11) = (/ &
     &9.6230e+01_rb,7.2282e+01_rb,4.9573e+01_rb,3.2349e+01_rb,8.1505e+00_rb/)
      kbo(:, 1,46,11) = (/ &
     &1.7603e+01_rb,1.6408e+01_rb,1.7601e+01_rb,1.5877e+01_rb,6.3549e+00_rb/)
      kbo(:, 2,46,11) = (/ &
     &2.7890e+01_rb,2.2645e+01_rb,2.2217e+01_rb,1.9546e+01_rb,6.8535e+00_rb/)
      kbo(:, 3,46,11) = (/ &
     &4.3095e+01_rb,3.3035e+01_rb,2.7597e+01_rb,2.3556e+01_rb,7.2985e+00_rb/)
      kbo(:, 4,46,11) = (/ &
     &6.5080e+01_rb,4.9001e+01_rb,3.6011e+01_rb,2.7911e+01_rb,7.7668e+00_rb/)
      kbo(:, 5,46,11) = (/ &
     &9.6111e+01_rb,7.2194e+01_rb,4.9702e+01_rb,3.2895e+01_rb,8.0887e+00_rb/)
      kbo(:, 1,47,11) = (/ &
     &1.5582e+01_rb,1.5416e+01_rb,1.6712e+01_rb,1.5202e+01_rb,6.1058e+00_rb/)
      kbo(:, 2,47,11) = (/ &
     &2.5371e+01_rb,2.1144e+01_rb,2.1438e+01_rb,1.8984e+01_rb,6.6529e+00_rb/)
      kbo(:, 3,47,11) = (/ &
     &4.0058e+01_rb,3.0957e+01_rb,2.6834e+01_rb,2.3189e+01_rb,7.1345e+00_rb/)
      kbo(:, 4,47,11) = (/ &
     &6.1807e+01_rb,4.6615e+01_rb,3.4933e+01_rb,2.7758e+01_rb,7.6397e+00_rb/)
      kbo(:, 5,47,11) = (/ &
     &9.3127e+01_rb,6.9962e+01_rb,4.8499e+01_rb,3.2952e+01_rb,7.9851e+00_rb/)
      kbo(:, 1,48,11) = (/ &
     &1.4034e+01_rb,1.4681e+01_rb,1.5989e+01_rb,1.4629e+01_rb,5.8421e+00_rb/)
      kbo(:, 2,48,11) = (/ &
     &2.3558e+01_rb,2.0186e+01_rb,2.0890e+01_rb,1.8564e+01_rb,6.4412e+00_rb/)
      kbo(:, 3,48,11) = (/ &
     &3.8004e+01_rb,2.9609e+01_rb,2.6472e+01_rb,2.3010e+01_rb,6.9605e+00_rb/)
      kbo(:, 4,48,11) = (/ &
     &5.9949e+01_rb,4.5296e+01_rb,3.4518e+01_rb,2.7885e+01_rb,7.5000e+00_rb/)
      kbo(:, 5,48,11) = (/ &
     &9.2277e+01_rb,6.9322e+01_rb,4.8288e+01_rb,3.3402e+01_rb,7.8874e+00_rb/)
      kbo(:, 1,49,11) = (/ &
     &1.3020e+01_rb,1.4223e+01_rb,1.5508e+01_rb,1.4222e+01_rb,5.5629e+00_rb/)
      kbo(:, 2,49,11) = (/ &
     &2.2531e+01_rb,1.9765e+01_rb,2.0674e+01_rb,1.8360e+01_rb,6.2033e+00_rb/)
      kbo(:, 3,49,11) = (/ &
     &3.7326e+01_rb,2.9255e+01_rb,2.6622e+01_rb,2.3133e+01_rb,6.7696e+00_rb/)
      kbo(:, 4,49,11) = (/ &
     &6.0141e+01_rb,4.5507e+01_rb,3.5009e+01_rb,2.8438e+01_rb,7.3484e+00_rb/)
      kbo(:, 5,49,11) = (/ &
     &9.4669e+01_rb,7.1125e+01_rb,4.9613e+01_rb,3.4451e+01_rb,7.7676e+00_rb/)
      kbo(:, 1,50,11) = (/ &
     &1.1528e+01_rb,1.3385e+01_rb,1.4676e+01_rb,1.3553e+01_rb,5.2886e+00_rb/)
      kbo(:, 2,50,11) = (/ &
     &2.0590e+01_rb,1.8779e+01_rb,1.9954e+01_rb,1.7785e+01_rb,5.9683e+00_rb/)
      kbo(:, 3,50,11) = (/ &
     &3.5112e+01_rb,2.7878e+01_rb,2.6128e+01_rb,2.2750e+01_rb,6.5812e+00_rb/)
      kbo(:, 4,50,11) = (/ &
     &5.7821e+01_rb,4.3871e+01_rb,3.4428e+01_rb,2.8358e+01_rb,7.1800e+00_rb/)
      kbo(:, 5,50,11) = (/ &
     &9.2920e+01_rb,6.9817e+01_rb,4.9041e+01_rb,3.4691e+01_rb,7.6346e+00_rb/)
      kbo(:, 1,51,11) = (/ &
     &9.9087e+00_rb,1.2389e+01_rb,1.3702e+01_rb,1.2771e+01_rb,5.0110e+00_rb/)
      kbo(:, 2,51,11) = (/ &
     &1.8347e+01_rb,1.7625e+01_rb,1.8991e+01_rb,1.7018e+01_rb,5.7321e+00_rb/)
      kbo(:, 3,51,11) = (/ &
     &3.2210e+01_rb,2.6119e+01_rb,2.5308e+01_rb,2.2105e+01_rb,6.3824e+00_rb/)
      kbo(:, 4,51,11) = (/ &
     &5.4374e+01_rb,4.1438e+01_rb,3.3397e+01_rb,2.7944e+01_rb,7.0038e+00_rb/)
      kbo(:, 5,51,11) = (/ &
     &8.9118e+01_rb,6.6984e+01_rb,4.7570e+01_rb,3.4540e+01_rb,7.4941e+00_rb/)
      kbo(:, 1,52,11) = (/ &
     &8.6167e+00_rb,1.1576e+01_rb,1.2899e+01_rb,1.2089e+01_rb,4.7245e+00_rb/)
      kbo(:, 2,52,11) = (/ &
     &1.6651e+01_rb,1.6818e+01_rb,1.8231e+01_rb,1.6395e+01_rb,5.4842e+00_rb/)
      kbo(:, 3,52,11) = (/ &
     &3.0138e+01_rb,2.4957e+01_rb,2.4762e+01_rb,2.1646e+01_rb,6.1679e+00_rb/)
      kbo(:, 4,52,11) = (/ &
     &5.2273e+01_rb,4.0011e+01_rb,3.2952e+01_rb,2.7789e+01_rb,6.8199e+00_rb/)
      kbo(:, 5,52,11) = (/ &
     &8.7481e+01_rb,6.5781e+01_rb,4.7111e+01_rb,3.4770e+01_rb,7.3415e+00_rb/)
      kbo(:, 1,53,11) = (/ &
     &7.6702e+00_rb,1.0979e+01_rb,1.2289e+01_rb,1.1530e+01_rb,4.4295e+00_rb/)
      kbo(:, 2,53,11) = (/ &
     &1.5508e+01_rb,1.6366e+01_rb,1.7771e+01_rb,1.5970e+01_rb,5.2253e+00_rb/)
      kbo(:, 3,53,11) = (/ &
     &2.9086e+01_rb,2.4490e+01_rb,2.4633e+01_rb,2.1484e+01_rb,5.9485e+00_rb/)
      kbo(:, 4,53,11) = (/ &
     &5.1881e+01_rb,3.9852e+01_rb,3.3235e+01_rb,2.8055e+01_rb,6.6238e+00_rb/)
      kbo(:, 5,53,11) = (/ &
     &8.8988e+01_rb,6.6930e+01_rb,4.8082e+01_rb,3.5589e+01_rb,7.1849e+00_rb/)
      kbo(:, 1,54,11) = (/ &
     &5.9450e+00_rb,9.6177e+00_rb,1.0972e+01_rb,1.0431e+01_rb,4.1455e+00_rb/)
      kbo(:, 2,54,11) = (/ &
     &1.2556e+01_rb,1.4668e+01_rb,1.6126e+01_rb,1.4708e+01_rb,4.9524e+00_rb/)
      kbo(:, 3,54,11) = (/ &
     &2.4491e+01_rb,2.1770e+01_rb,2.2780e+01_rb,2.0031e+01_rb,5.7134e+00_rb/)
      kbo(:, 4,54,11) = (/ &
     &4.4954e+01_rb,3.5031e+01_rb,3.0908e+01_rb,2.6515e+01_rb,6.4122e+00_rb/)
      kbo(:, 5,54,11) = (/ &
     &7.9027e+01_rb,5.9569e+01_rb,4.4065e+01_rb,3.4001e+01_rb,7.0090e+00_rb/)
      kbo(:, 1,55,11) = (/ &
     &4.1710e+00_rb,8.0225e+00_rb,9.3978e+00_rb,9.1226e+00_rb,3.8682e+00_rb/)
      kbo(:, 2,55,11) = (/ &
     &9.2293e+00_rb,1.2486e+01_rb,1.4001e+01_rb,1.3070e+01_rb,4.6969e+00_rb/)
      kbo(:, 3,55,11) = (/ &
     &1.8778e+01_rb,1.8456e+01_rb,2.0084e+01_rb,1.7971e+01_rb,5.4658e+00_rb/)
      kbo(:, 4,55,11) = (/ &
     &3.5574e+01_rb,2.8777e+01_rb,2.7568e+01_rb,2.4021e+01_rb,6.1617e+00_rb/)
      kbo(:, 5,55,11) = (/ &
     &6.4114e+01_rb,4.8674e+01_rb,3.8113e+01_rb,3.1115e+01_rb,6.8236e+00_rb/)
      kbo(:, 1,56,11) = (/ &
     &2.8728e+00_rb,6.6889e+00_rb,8.0328e+00_rb,7.9557e+00_rb,3.5989e+00_rb/)
      kbo(:, 2,56,11) = (/ &
     &6.6874e+00_rb,1.0605e+01_rb,1.2156e+01_rb,1.1597e+01_rb,4.4247e+00_rb/)
      kbo(:, 3,56,11) = (/ &
     &1.4226e+01_rb,1.5908e+01_rb,1.7667e+01_rb,1.6104e+01_rb,5.2042e+00_rb/)
      kbo(:, 4,56,11) = (/ &
     &2.7945e+01_rb,2.3989e+01_rb,2.4626e+01_rb,2.1735e+01_rb,5.9393e+00_rb/)
      kbo(:, 5,56,11) = (/ &
     &5.1707e+01_rb,3.9775e+01_rb,3.3431e+01_rb,2.8442e+01_rb,6.5979e+00_rb/)
      kbo(:, 1,57,11) = (/ &
     &1.9429e+00_rb,5.5686e+00_rb,6.8405e+00_rb,6.9151e+00_rb,3.3226e+00_rb/)
      kbo(:, 2,57,11) = (/ &
     &4.7724e+00_rb,8.9791e+00_rb,1.0545e+01_rb,1.0240e+01_rb,4.1527e+00_rb/)
      kbo(:, 3,57,11) = (/ &
     &1.0641e+01_rb,1.3735e+01_rb,1.5504e+01_rb,1.4420e+01_rb,4.9657e+00_rb/)
      kbo(:, 4,57,11) = (/ &
     &2.1776e+01_rb,2.0320e+01_rb,2.1929e+01_rb,1.9615e+01_rb,5.6966e+00_rb/)
      kbo(:, 5,57,11) = (/ &
     &4.1383e+01_rb,3.2617e+01_rb,2.9771e+01_rb,2.5932e+01_rb,6.3597e+00_rb/)
      kbo(:, 1,58,11) = (/ &
     &3.4498e-01_rb,1.5267e+00_rb,2.4690e+00_rb,3.5849e+00_rb,3.0713e+00_rb/)
      kbo(:, 2,58,11) = (/ &
     &8.9647e-01_rb,2.5018e+00_rb,3.8790e+00_rb,5.3697e+00_rb,3.9024e+00_rb/)
      kbo(:, 3,58,11) = (/ &
     &2.0963e+00_rb,3.8966e+00_rb,5.7628e+00_rb,7.6647e+00_rb,4.7137e+00_rb/)
      kbo(:, 4,58,11) = (/ &
     &4.4774e+00_rb,5.7497e+00_rb,8.2495e+00_rb,1.0504e+01_rb,5.4718e+00_rb/)
      kbo(:, 5,58,11) = (/ &
     &8.7673e+00_rb,8.8663e+00_rb,1.1318e+01_rb,1.3996e+01_rb,6.1538e+00_rb/)
      kbo(:, 1,59,11) = (/ &
     &3.2685e-01_rb,1.8752e+00_rb,2.9232e+00_rb,3.9328e+00_rb,2.9672e+00_rb/)
      kbo(:, 2,59,11) = (/ &
     &8.8018e-01_rb,3.0635e+00_rb,4.5809e+00_rb,5.8034e+00_rb,3.8049e+00_rb/)
      kbo(:, 3,59,11) = (/ &
     &2.1278e+00_rb,4.7401e+00_rb,6.7591e+00_rb,8.2573e+00_rb,4.6076e+00_rb/)
      kbo(:, 4,59,11) = (/ &
     &4.6712e+00_rb,6.9756e+00_rb,9.6079e+00_rb,1.1224e+01_rb,5.3733e+00_rb/)
      kbo(:, 5,59,11) = (/ &
     &9.3271e+00_rb,1.0056e+01_rb,1.3155e+01_rb,1.4840e+01_rb,6.0503e+00_rb/)
      kbo(:, 1,13,12) = (/ &
     &7.5516e+02_rb,5.6651e+02_rb,3.7786e+02_rb,1.8921e+02_rb,4.6057e+00_rb/)
      kbo(:, 2,13,12) = (/ &
     &1.1031e+03_rb,8.2746e+02_rb,5.5183e+02_rb,2.7620e+02_rb,4.4752e+00_rb/)
      kbo(:, 3,13,12) = (/ &
     &1.5208e+03_rb,1.1407e+03_rb,7.6067e+02_rb,3.8062e+02_rb,4.3357e+00_rb/)
      kbo(:, 4,13,12) = (/ &
     &2.0072e+03_rb,1.5056e+03_rb,1.0039e+03_rb,5.0221e+02_rb,4.1960e+00_rb/)
      kbo(:, 5,13,12) = (/ &
     &2.5500e+03_rb,1.9126e+03_rb,1.2753e+03_rb,6.3790e+02_rb,4.0223e+00_rb/)
      kbo(:, 1,14,12) = (/ &
     &5.0313e+02_rb,3.7749e+02_rb,2.5185e+02_rb,1.2621e+02_rb,5.0046e+00_rb/)
      kbo(:, 2,14,12) = (/ &
     &7.2897e+02_rb,5.4687e+02_rb,3.6476e+02_rb,1.8266e+02_rb,4.8590e+00_rb/)
      kbo(:, 3,14,12) = (/ &
     &9.9893e+02_rb,7.4933e+02_rb,4.9972e+02_rb,2.5013e+02_rb,4.7095e+00_rb/)
      kbo(:, 4,14,12) = (/ &
     &1.3102e+03_rb,9.8276e+02_rb,6.5533e+02_rb,3.2791e+02_rb,4.5769e+00_rb/)
      kbo(:, 5,14,12) = (/ &
     &1.6551e+03_rb,1.2414e+03_rb,8.2775e+02_rb,4.1411e+02_rb,4.4631e+00_rb/)
      kbo(:, 1,15,12) = (/ &
     &3.3639e+02_rb,2.5243e+02_rb,1.6847e+02_rb,8.4507e+01_rb,5.4523e+00_rb/)
      kbo(:, 2,15,12) = (/ &
     &4.8280e+02_rb,3.6223e+02_rb,2.4166e+02_rb,1.2109e+02_rb,5.3133e+00_rb/)
      kbo(:, 3,15,12) = (/ &
     &6.5719e+02_rb,4.9302e+02_rb,3.2884e+02_rb,1.6466e+02_rb,5.1834e+00_rb/)
      kbo(:, 4,15,12) = (/ &
     &8.5550e+02_rb,6.4175e+02_rb,4.2799e+02_rb,2.1423e+02_rb,5.0660e+00_rb/)
      kbo(:, 5,15,12) = (/ &
     &1.0728e+03_rb,8.0472e+02_rb,5.3663e+02_rb,2.6854e+02_rb,4.9623e+00_rb/)
      kbo(:, 1,16,12) = (/ &
     &2.6229e+02_rb,1.9684e+02_rb,1.3139e+02_rb,6.5935e+01_rb,6.0512e+00_rb/)
      kbo(:, 2,16,12) = (/ &
     &3.7303e+02_rb,2.7989e+02_rb,1.8674e+02_rb,9.3604e+01_rb,5.9120e+00_rb/)
      kbo(:, 3,16,12) = (/ &
     &5.0297e+02_rb,3.7733e+02_rb,2.5170e+02_rb,1.2607e+02_rb,5.7872e+00_rb/)
      kbo(:, 4,16,12) = (/ &
     &6.4911e+02_rb,4.8694e+02_rb,3.2477e+02_rb,1.6260e+02_rb,5.6679e+00_rb/)
      kbo(:, 5,16,12) = (/ &
     &8.0986e+02_rb,6.0750e+02_rb,4.0515e+02_rb,2.0279e+02_rb,5.5547e+00_rb/)
      kbo(:, 1,17,12) = (/ &
     &2.0705e+02_rb,1.5540e+02_rb,1.0375e+02_rb,5.2106e+01_rb,6.7730e+00_rb/)
      kbo(:, 2,17,12) = (/ &
     &2.9135e+02_rb,2.1862e+02_rb,1.4589e+02_rb,7.3163e+01_rb,6.6333e+00_rb/)
      kbo(:, 3,17,12) = (/ &
     &3.8916e+02_rb,2.9197e+02_rb,1.9479e+02_rb,9.7603e+01_rb,6.4989e+00_rb/)
      kbo(:, 4,17,12) = (/ &
     &4.9976e+02_rb,3.7492e+02_rb,2.5009e+02_rb,1.2525e+02_rb,6.3687e+00_rb/)
      kbo(:, 5,17,12) = (/ &
     &6.2230e+02_rb,4.6683e+02_rb,3.1136e+02_rb,1.5589e+02_rb,6.2358e+00_rb/)
      kbo(:, 1,18,12) = (/ &
     &1.6333e+02_rb,1.2261e+02_rb,8.1888e+01_rb,4.1178e+01_rb,7.5424e+00_rb/)
      kbo(:, 2,18,12) = (/ &
     &2.2759e+02_rb,1.7080e+02_rb,1.1401e+02_rb,5.7226e+01_rb,7.4084e+00_rb/)
      kbo(:, 3,18,12) = (/ &
     &3.0244e+02_rb,2.2693e+02_rb,1.5143e+02_rb,7.5931e+01_rb,7.2669e+00_rb/)
      kbo(:, 4,18,12) = (/ &
     &3.8758e+02_rb,2.9079e+02_rb,1.9400e+02_rb,9.7216e+01_rb,7.1208e+00_rb/)
      kbo(:, 5,18,12) = (/ &
     &4.8269e+02_rb,3.6212e+02_rb,2.4155e+02_rb,1.2098e+02_rb,6.9720e+00_rb/)
      kbo(:, 1,19,12) = (/ &
     &1.2382e+02_rb,9.2977e+01_rb,6.2136e+01_rb,3.1325e+01_rb,8.3597e+00_rb/)
      kbo(:, 2,19,12) = (/ &
     &1.7154e+02_rb,1.2877e+02_rb,8.5994e+01_rb,4.3233e+01_rb,8.2061e+00_rb/)
      kbo(:, 3,19,12) = (/ &
     &2.2748e+02_rb,1.7072e+02_rb,1.1396e+02_rb,5.7206e+01_rb,8.0513e+00_rb/)
      kbo(:, 4,19,12) = (/ &
     &2.9138e+02_rb,2.1864e+02_rb,1.4590e+02_rb,7.3167e+01_rb,7.8994e+00_rb/)
      kbo(:, 5,19,12) = (/ &
     &3.6253e+02_rb,2.7200e+02_rb,1.8147e+02_rb,9.0939e+01_rb,7.7452e+00_rb/)
      kbo(:, 1,20,12) = (/ &
     &1.0042e+02_rb,7.5430e+01_rb,5.0444e+01_rb,2.5891e+01_rb,9.1675e+00_rb/)
      kbo(:, 2,20,12) = (/ &
     &1.3861e+02_rb,1.0407e+02_rb,6.9532e+01_rb,3.5031e+01_rb,9.0030e+00_rb/)
      kbo(:, 3,20,12) = (/ &
     &1.8351e+02_rb,1.3774e+02_rb,9.1975e+01_rb,4.6211e+01_rb,8.8390e+00_rb/)
      kbo(:, 4,20,12) = (/ &
     &2.3463e+02_rb,1.7607e+02_rb,1.1752e+02_rb,5.8971e+01_rb,8.6735e+00_rb/)
      kbo(:, 5,20,12) = (/ &
     &2.9128e+02_rb,2.1856e+02_rb,1.4583e+02_rb,7.3117e+01_rb,8.5016e+00_rb/)
      kbo(:, 1,21,12) = (/ &
     &8.3057e+01_rb,6.2407e+01_rb,4.1762e+01_rb,2.2683e+01_rb,9.9550e+00_rb/)
      kbo(:, 2,21,12) = (/ &
     &1.1435e+02_rb,8.5868e+01_rb,5.7394e+01_rb,2.9474e+01_rb,9.7863e+00_rb/)
      kbo(:, 3,21,12) = (/ &
     &1.5096e+02_rb,1.1332e+02_rb,7.5686e+01_rb,3.8128e+01_rb,9.6108e+00_rb/)
      kbo(:, 4,21,12) = (/ &
     &1.9244e+02_rb,1.4443e+02_rb,9.6415e+01_rb,4.8410e+01_rb,9.4311e+00_rb/)
      kbo(:, 5,21,12) = (/ &
     &2.3840e+02_rb,1.7889e+02_rb,1.1939e+02_rb,5.9885e+01_rb,9.2417e+00_rb/)
      kbo(:, 1,22,12) = (/ &
     &7.1250e+01_rb,5.3549e+01_rb,3.5853e+01_rb,2.1126e+01_rb,1.0685e+01_rb/)
      kbo(:, 2,22,12) = (/ &
     &9.7414e+01_rb,7.3164e+01_rb,4.8918e+01_rb,2.6307e+01_rb,1.0512e+01_rb/)
      kbo(:, 3,22,12) = (/ &
     &1.2780e+02_rb,9.5945e+01_rb,6.4097e+01_rb,3.2967e+01_rb,1.0335e+01_rb/)
      kbo(:, 4,22,12) = (/ &
     &1.6218e+02_rb,1.2173e+02_rb,8.1281e+01_rb,4.1038e+01_rb,1.0145e+01_rb/)
      kbo(:, 5,22,12) = (/ &
     &2.0041e+02_rb,1.5040e+02_rb,1.0039e+02_rb,5.0398e+01_rb,9.9454e+00_rb/)
      kbo(:, 1,23,12) = (/ &
     &6.0942e+01_rb,4.5813e+01_rb,3.0934e+01_rb,2.0012e+01_rb,1.1340e+01_rb/)
      kbo(:, 2,23,12) = (/ &
     &8.2640e+01_rb,6.2081e+01_rb,4.1527e+01_rb,2.4053e+01_rb,1.1180e+01_rb/)
      kbo(:, 3,23,12) = (/ &
     &1.0785e+02_rb,8.0983e+01_rb,5.4124e+01_rb,2.9110e+01_rb,1.1001e+01_rb/)
      kbo(:, 4,23,12) = (/ &
     &1.3652e+02_rb,1.0249e+02_rb,6.8454e+01_rb,3.5343e+01_rb,1.0812e+01_rb/)
      kbo(:, 5,23,12) = (/ &
     &1.6894e+02_rb,1.2680e+02_rb,8.4663e+01_rb,4.2880e+01_rb,1.0609e+01_rb/)
      kbo(:, 1,24,12) = (/ &
     &5.1406e+01_rb,3.8659e+01_rb,2.6835e+01_rb,1.9368e+01_rb,1.1917e+01_rb/)
      kbo(:, 2,24,12) = (/ &
     &6.9266e+01_rb,5.2052e+01_rb,3.5118e+01_rb,2.2283e+01_rb,1.1770e+01_rb/)
      kbo(:, 3,24,12) = (/ &
     &9.0127e+01_rb,6.7693e+01_rb,4.5282e+01_rb,2.6104e+01_rb,1.1607e+01_rb/)
      kbo(:, 4,24,12) = (/ &
     &1.1424e+02_rb,8.5780e+01_rb,5.7328e+01_rb,3.0937e+01_rb,1.1421e+01_rb/)
      kbo(:, 5,24,12) = (/ &
     &1.4182e+02_rb,1.0646e+02_rb,7.1109e+01_rb,3.6894e+01_rb,1.1218e+01_rb/)
      kbo(:, 1,25,12) = (/ &
     &4.3762e+01_rb,3.2928e+01_rb,2.3974e+01_rb,1.9297e+01_rb,1.2406e+01_rb/)
      kbo(:, 2,25,12) = (/ &
     &5.8726e+01_rb,4.4147e+01_rb,3.0453e+01_rb,2.1379e+01_rb,1.2284e+01_rb/)
      kbo(:, 3,25,12) = (/ &
     &7.6463e+01_rb,5.7453e+01_rb,3.8735e+01_rb,2.4187e+01_rb,1.2134e+01_rb/)
      kbo(:, 4,25,12) = (/ &
     &9.7222e+01_rb,7.3021e+01_rb,4.8859e+01_rb,2.7987e+01_rb,1.1961e+01_rb/)
      kbo(:, 5,25,12) = (/ &
     &1.2098e+02_rb,9.0828e+01_rb,6.0694e+01_rb,3.2765e+01_rb,1.1759e+01_rb/)
      kbo(:, 1,26,12) = (/ &
     &3.8022e+01_rb,2.8659e+01_rb,2.2105e+01_rb,1.9516e+01_rb,1.2805e+01_rb/)
      kbo(:, 2,26,12) = (/ &
     &5.0994e+01_rb,3.8356e+01_rb,2.7362e+01_rb,2.1131e+01_rb,1.2709e+01_rb/)
      kbo(:, 3,26,12) = (/ &
     &6.6538e+01_rb,5.0014e+01_rb,3.4258e+01_rb,2.3250e+01_rb,1.2582e+01_rb/)
      kbo(:, 4,26,12) = (/ &
     &8.4702e+01_rb,6.3631e+01_rb,4.2833e+01_rb,2.6169e+01_rb,1.2418e+01_rb/)
      kbo(:, 5,26,12) = (/ &
     &1.0565e+02_rb,7.9327e+01_rb,5.3062e+01_rb,3.0028e+01_rb,1.2216e+01_rb/)
      kbo(:, 1,27,12) = (/ &
     &3.4489e+01_rb,2.6132e+01_rb,2.1235e+01_rb,1.9878e+01_rb,1.3122e+01_rb/)
      kbo(:, 2,27,12) = (/ &
     &4.6305e+01_rb,3.4844e+01_rb,2.5714e+01_rb,2.1342e+01_rb,1.3062e+01_rb/)
      kbo(:, 3,27,12) = (/ &
     &6.0441e+01_rb,4.5443e+01_rb,3.1734e+01_rb,2.3085e+01_rb,1.2955e+01_rb/)
      kbo(:, 4,27,12) = (/ &
     &7.7109e+01_rb,5.7931e+01_rb,3.9347e+01_rb,2.5409e+01_rb,1.2800e+01_rb/)
      kbo(:, 5,27,12) = (/ &
     &9.6578e+01_rb,7.2521e+01_rb,4.8662e+01_rb,2.8654e+01_rb,1.2602e+01_rb/)
      kbo(:, 1,28,12) = (/ &
     &3.2427e+01_rb,2.4737e+01_rb,2.0948e+01_rb,2.0280e+01_rb,1.3386e+01_rb/)
      kbo(:, 2,28,12) = (/ &
     &4.3501e+01_rb,3.2787e+01_rb,2.4919e+01_rb,2.1714e+01_rb,1.3352e+01_rb/)
      kbo(:, 3,28,12) = (/ &
     &5.6891e+01_rb,4.2776e+01_rb,3.0416e+01_rb,2.3252e+01_rb,1.3262e+01_rb/)
      kbo(:, 4,28,12) = (/ &
     &7.2922e+01_rb,5.4789e+01_rb,3.7531e+01_rb,2.5273e+01_rb,1.3118e+01_rb/)
      kbo(:, 5,28,12) = (/ &
     &9.1872e+01_rb,6.8992e+01_rb,4.6457e+01_rb,2.8166e+01_rb,1.2926e+01_rb/)
      kbo(:, 1,29,12) = (/ &
     &3.2667e+01_rb,2.5018e+01_rb,2.1498e+01_rb,2.0955e+01_rb,1.3603e+01_rb/)
      kbo(:, 2,29,12) = (/ &
     &4.3912e+01_rb,3.3123e+01_rb,2.5431e+01_rb,2.2422e+01_rb,1.3591e+01_rb/)
      kbo(:, 3,29,12) = (/ &
     &5.7701e+01_rb,4.3385e+01_rb,3.1008e+01_rb,2.3974e+01_rb,1.3517e+01_rb/)
      kbo(:, 4,29,12) = (/ &
     &7.4275e+01_rb,5.5806e+01_rb,3.8313e+01_rb,2.6005e+01_rb,1.3383e+01_rb/)
      kbo(:, 5,29,12) = (/ &
     &9.3871e+01_rb,7.0500e+01_rb,4.7521e+01_rb,2.8930e+01_rb,1.3178e+01_rb/)
      kbo(:, 1,30,12) = (/ &
     &3.4156e+01_rb,2.6153e+01_rb,2.2429e+01_rb,2.1713e+01_rb,1.3788e+01_rb/)
      kbo(:, 2,30,12) = (/ &
     &4.6027e+01_rb,3.4711e+01_rb,2.6560e+01_rb,2.3234e+01_rb,1.3795e+01_rb/)
      kbo(:, 3,30,12) = (/ &
     &6.0622e+01_rb,4.5582e+01_rb,3.2488e+01_rb,2.4921e+01_rb,1.3734e+01_rb/)
      kbo(:, 4,30,12) = (/ &
     &7.8321e+01_rb,5.8857e+01_rb,4.0368e+01_rb,2.7080e+01_rb,1.3599e+01_rb/)
      kbo(:, 5,30,12) = (/ &
     &9.9301e+01_rb,7.4592e+01_rb,5.0256e+01_rb,3.0286e+01_rb,1.3384e+01_rb/)
      kbo(:, 1,31,12) = (/ &
     &3.8076e+01_rb,2.9005e+01_rb,2.4159e+01_rb,2.2772e+01_rb,1.3950e+01_rb/)
      kbo(:, 2,31,12) = (/ &
     &5.1386e+01_rb,3.8696e+01_rb,2.8953e+01_rb,2.4505e+01_rb,1.3973e+01_rb/)
      kbo(:, 3,31,12) = (/ &
     &6.7843e+01_rb,5.1009e+01_rb,3.5918e+01_rb,2.6433e+01_rb,1.3920e+01_rb/)
      kbo(:, 4,31,12) = (/ &
     &8.7812e+01_rb,6.5990e+01_rb,4.4995e+01_rb,2.9071e+01_rb,1.3753e+01_rb/)
      kbo(:, 5,31,12) = (/ &
     &1.1160e+02_rb,8.3824e+01_rb,5.6340e+01_rb,3.3050e+01_rb,1.3573e+01_rb/)
      kbo(:, 1,32,12) = (/ &
     &4.3422e+01_rb,3.2909e+01_rb,2.6354e+01_rb,2.4011e+01_rb,1.4101e+01_rb/)
      kbo(:, 2,32,12) = (/ &
     &5.8749e+01_rb,4.4202e+01_rb,3.2268e+01_rb,2.6008e+01_rb,1.4129e+01_rb/)
      kbo(:, 3,32,12) = (/ &
     &7.7682e+01_rb,5.8398e+01_rb,4.0569e+01_rb,2.8355e+01_rb,1.4037e+01_rb/)
      kbo(:, 4,32,12) = (/ &
     &1.0053e+02_rb,7.5534e+01_rb,5.1158e+01_rb,3.1768e+01_rb,1.3909e+01_rb/)
      kbo(:, 5,32,12) = (/ &
     &1.2780e+02_rb,9.5988e+01_rb,6.4344e+01_rb,3.6705e+01_rb,1.3757e+01_rb/)
      kbo(:, 1,33,12) = (/ &
     &5.0972e+01_rb,3.8474e+01_rb,2.9509e+01_rb,2.5608e+01_rb,1.4236e+01_rb/)
      kbo(:, 2,33,12) = (/ &
     &6.8978e+01_rb,5.1873e+01_rb,3.6983e+01_rb,2.7940e+01_rb,1.4225e+01_rb/)
      kbo(:, 3,33,12) = (/ &
     &9.1196e+01_rb,6.8536e+01_rb,4.6964e+01_rb,3.1045e+01_rb,1.4156e+01_rb/)
      kbo(:, 4,33,12) = (/ &
     &1.1806e+02_rb,8.8684e+01_rb,5.9688e+01_rb,3.5479e+01_rb,1.4053e+01_rb/)
      kbo(:, 5,33,12) = (/ &
     &1.5026e+02_rb,1.1283e+02_rb,7.5476e+01_rb,4.1722e+01_rb,1.3797e+01_rb/)
      kbo(:, 1,34,12) = (/ &
     &5.8196e+01_rb,4.3842e+01_rb,3.2696e+01_rb,2.7075e+01_rb,1.4322e+01_rb/)
      kbo(:, 2,34,12) = (/ &
     &7.8717e+01_rb,5.9180e+01_rb,4.1505e+01_rb,2.9899e+01_rb,1.4304e+01_rb/)
      kbo(:, 3,34,12) = (/ &
     &1.0408e+02_rb,7.8203e+01_rb,5.3150e+01_rb,3.3676e+01_rb,1.4279e+01_rb/)
      kbo(:, 4,34,12) = (/ &
     &1.3497e+02_rb,1.0137e+02_rb,6.8004e+01_rb,3.9142e+01_rb,1.4130e+01_rb/)
      kbo(:, 5,34,12) = (/ &
     &1.7215e+02_rb,1.2926e+02_rb,8.6400e+01_rb,4.6696e+01_rb,1.3817e+01_rb/)
      kbo(:, 1,35,12) = (/ &
     &6.4953e+01_rb,4.8885e+01_rb,3.5777e+01_rb,2.8513e+01_rb,1.4376e+01_rb/)
      kbo(:, 2,35,12) = (/ &
     &8.7973e+01_rb,6.6128e+01_rb,4.5876e+01_rb,3.1844e+01_rb,1.4423e+01_rb/)
      kbo(:, 3,35,12) = (/ &
     &1.1659e+02_rb,8.7588e+01_rb,5.9213e+01_rb,3.6344e+01_rb,1.4380e+01_rb/)
      kbo(:, 4,35,12) = (/ &
     &1.5177e+02_rb,1.1397e+02_rb,7.6316e+01_rb,4.2891e+01_rb,1.4142e+01_rb/)
      kbo(:, 5,35,12) = (/ &
     &1.9438e+02_rb,1.4593e+02_rb,9.7521e+01_rb,5.1793e+01_rb,1.3951e+01_rb/)
      kbo(:, 1,36,12) = (/ &
     &7.0028e+01_rb,5.2692e+01_rb,3.8196e+01_rb,2.9764e+01_rb,1.4429e+01_rb/)
      kbo(:, 2,36,12) = (/ &
     &9.5230e+01_rb,7.1580e+01_rb,4.9388e+01_rb,3.3491e+01_rb,1.4515e+01_rb/)
      kbo(:, 3,36,12) = (/ &
     &1.2679e+02_rb,9.5251e+01_rb,6.4219e+01_rb,3.8654e+01_rb,1.4447e+01_rb/)
      kbo(:, 4,36,12) = (/ &
     &1.6604e+02_rb,1.2468e+02_rb,8.3408e+01_rb,4.6199e+01_rb,1.4226e+01_rb/)
      kbo(:, 5,36,12) = (/ &
     &2.1406e+02_rb,1.6070e+02_rb,1.0737e+02_rb,5.6401e+01_rb,1.4079e+01_rb/)
      kbo(:, 1,37,12) = (/ &
     &7.0336e+01_rb,5.2944e+01_rb,3.8605e+01_rb,3.0262e+01_rb,1.4492e+01_rb/)
      kbo(:, 2,37,12) = (/ &
     &9.6465e+01_rb,7.2514e+01_rb,5.0131e+01_rb,3.4164e+01_rb,1.4585e+01_rb/)
      kbo(:, 3,37,12) = (/ &
     &1.2955e+02_rb,9.7324e+01_rb,6.5642e+01_rb,3.9574e+01_rb,1.4537e+01_rb/)
      kbo(:, 4,37,12) = (/ &
     &1.7114e+02_rb,1.2851e+02_rb,8.5971e+01_rb,4.7570e+01_rb,1.4312e+01_rb/)
      kbo(:, 5,37,12) = (/ &
     &2.2257e+02_rb,1.6708e+02_rb,1.1162e+02_rb,5.8523e+01_rb,1.4157e+01_rb/)
      kbo(:, 1,38,12) = (/ &
     &7.1653e+01_rb,5.3935e+01_rb,3.9443e+01_rb,3.0956e+01_rb,1.4537e+01_rb/)
      kbo(:, 2,38,12) = (/ &
     &9.9371e+01_rb,7.4697e+01_rb,5.1646e+01_rb,3.5117e+01_rb,1.4670e+01_rb/)
      kbo(:, 3,38,12) = (/ &
     &1.3467e+02_rb,1.0116e+02_rb,6.8199e+01_rb,4.0938e+01_rb,1.4595e+01_rb/)
      kbo(:, 4,38,12) = (/ &
     &1.7944e+02_rb,1.3474e+02_rb,9.0110e+01_rb,4.9614e+01_rb,1.4367e+01_rb/)
      kbo(:, 5,38,12) = (/ &
     &2.3537e+02_rb,1.7668e+02_rb,1.1802e+02_rb,6.1608e+01_rb,1.4213e+01_rb/)
      kbo(:, 1,39,12) = (/ &
     &7.4953e+01_rb,5.6404e+01_rb,4.1108e+01_rb,3.2048e+01_rb,1.4593e+01_rb/)
      kbo(:, 2,39,12) = (/ &
     &1.0508e+02_rb,7.8982e+01_rb,5.4463e+01_rb,3.6606e+01_rb,1.4737e+01_rb/)
      kbo(:, 3,39,12) = (/ &
     &1.4387e+02_rb,1.0807e+02_rb,7.2745e+01_rb,4.3123e+01_rb,1.4593e+01_rb/)
      kbo(:, 4,39,12) = (/ &
     &1.9336e+02_rb,1.4518e+02_rb,9.7046e+01_rb,5.2882e+01_rb,1.4422e+01_rb/)
      kbo(:, 5,39,12) = (/ &
     &2.5579e+02_rb,1.9200e+02_rb,1.2823e+02_rb,6.6478e+01_rb,1.4241e+01_rb/)
      kbo(:, 1,40,12) = (/ &
     &7.1125e+01_rb,5.3576e+01_rb,3.9744e+01_rb,3.1845e+01_rb,1.4584e+01_rb/)
      kbo(:, 2,40,12) = (/ &
     &1.0123e+02_rb,7.6093e+01_rb,5.2813e+01_rb,3.6464e+01_rb,1.4753e+01_rb/)
      kbo(:, 3,40,12) = (/ &
     &1.4047e+02_rb,1.0551e+02_rb,7.1189e+01_rb,4.2876e+01_rb,1.4635e+01_rb/)
      kbo(:, 4,40,12) = (/ &
     &1.9075e+02_rb,1.4322e+02_rb,9.5759e+01_rb,5.2592e+01_rb,1.4461e+01_rb/)
      kbo(:, 5,40,12) = (/ &
     &2.5487e+02_rb,1.9131e+02_rb,1.2777e+02_rb,6.6443e+01_rb,1.4311e+01_rb/)
      kbo(:, 1,41,12) = (/ &
     &6.6999e+01_rb,5.0553e+01_rb,3.8328e+01_rb,3.1540e+01_rb,1.4557e+01_rb/)
      kbo(:, 2,41,12) = (/ &
     &9.6989e+01_rb,7.2904e+01_rb,5.1038e+01_rb,3.6239e+01_rb,1.4754e+01_rb/)
      kbo(:, 3,41,12) = (/ &
     &1.3645e+02_rb,1.0250e+02_rb,6.9354e+01_rb,4.2559e+01_rb,1.4660e+01_rb/)
      kbo(:, 4,41,12) = (/ &
     &1.8750e+02_rb,1.4078e+02_rb,9.4178e+01_rb,5.2196e+01_rb,1.4491e+01_rb/)
      kbo(:, 5,41,12) = (/ &
     &2.5297e+02_rb,1.8988e+02_rb,1.2682e+02_rb,6.6184e+01_rb,1.4389e+01_rb/)
      kbo(:, 1,42,12) = (/ &
     &6.3698e+01_rb,4.8173e+01_rb,3.7314e+01_rb,3.1356e+01_rb,1.4515e+01_rb/)
      kbo(:, 2,42,12) = (/ &
     &9.3921e+01_rb,7.0600e+01_rb,4.9842e+01_rb,3.6216e+01_rb,1.4716e+01_rb/)
      kbo(:, 3,42,12) = (/ &
     &1.3407e+02_rb,1.0072e+02_rb,6.8313e+01_rb,4.2558e+01_rb,1.4673e+01_rb/)
      kbo(:, 4,42,12) = (/ &
     &1.8663e+02_rb,1.4013e+02_rb,9.3782e+01_rb,5.2338e+01_rb,1.4521e+01_rb/)
      kbo(:, 5,42,12) = (/ &
     &2.5424e+02_rb,1.9084e+02_rb,1.2745e+02_rb,6.6667e+01_rb,1.4444e+01_rb/)
      kbo(:, 1,43,12) = (/ &
     &5.9556e+01_rb,4.5221e+01_rb,3.6039e+01_rb,3.0985e+01_rb,1.4424e+01_rb/)
      kbo(:, 2,43,12) = (/ &
     &8.9715e+01_rb,6.7448e+01_rb,4.8196e+01_rb,3.6053e+01_rb,1.4672e+01_rb/)
      kbo(:, 3,43,12) = (/ &
     &1.3041e+02_rb,9.7958e+01_rb,6.6682e+01_rb,4.2356e+01_rb,1.4685e+01_rb/)
      kbo(:, 4,43,12) = (/ &
     &1.8413e+02_rb,1.3825e+02_rb,9.2592e+01_rb,5.2204e+01_rb,1.4530e+01_rb/)
      kbo(:, 5,43,12) = (/ &
     &2.5372e+02_rb,1.9044e+02_rb,1.2719e+02_rb,6.6793e+01_rb,1.4531e+01_rb/)
      kbo(:, 1,44,12) = (/ &
     &5.5617e+01_rb,4.2462e+01_rb,3.4926e+01_rb,3.0616e+01_rb,1.4301e+01_rb/)
      kbo(:, 2,44,12) = (/ &
     &8.5711e+01_rb,6.4467e+01_rb,4.6726e+01_rb,3.5948e+01_rb,1.4602e+01_rb/)
      kbo(:, 3,44,12) = (/ &
     &1.2713e+02_rb,9.5499e+01_rb,6.5283e+01_rb,4.2294e+01_rb,1.4674e+01_rb/)
      kbo(:, 4,44,12) = (/ &
     &1.8238e+02_rb,1.3694e+02_rb,9.1783e+01_rb,5.2246e+01_rb,1.4528e+01_rb/)
      kbo(:, 5,44,12) = (/ &
     &2.5466e+02_rb,1.9115e+02_rb,1.2766e+02_rb,6.7294e+01_rb,1.4570e+01_rb/)
      kbo(:, 1,45,12) = (/ &
     &5.2685e+01_rb,4.0486e+01_rb,3.4264e+01_rb,3.0427e+01_rb,1.4151e+01_rb/)
      kbo(:, 2,45,12) = (/ &
     &8.3179e+01_rb,6.2605e+01_rb,4.5965e+01_rb,3.6072e+01_rb,1.4507e+01_rb/)
      kbo(:, 3,45,12) = (/ &
     &1.2601e+02_rb,9.4652e+01_rb,6.4923e+01_rb,4.2669e+01_rb,1.4643e+01_rb/)
      kbo(:, 4,45,12) = (/ &
     &1.8401e+02_rb,1.3816e+02_rb,9.2634e+01_rb,5.3005e+01_rb,1.4503e+01_rb/)
      kbo(:, 5,45,12) = (/ &
     &2.6050e+02_rb,1.9553e+02_rb,1.3058e+02_rb,6.8905e+01_rb,1.4618e+01_rb/)
      kbo(:, 1,46,12) = (/ &
     &4.9550e+01_rb,3.8366e+01_rb,3.3605e+01_rb,3.0155e+01_rb,1.3963e+01_rb/)
      kbo(:, 2,46,12) = (/ &
     &8.0337e+01_rb,6.0538e+01_rb,4.5156e+01_rb,3.6142e+01_rb,1.4400e+01_rb/)
      kbo(:, 3,46,12) = (/ &
     &1.2453e+02_rb,9.3545e+01_rb,6.4429e+01_rb,4.3023e+01_rb,1.4594e+01_rb/)
      kbo(:, 4,46,12) = (/ &
     &1.8550e+02_rb,1.3927e+02_rb,9.3438e+01_rb,5.3784e+01_rb,1.4498e+01_rb/)
      kbo(:, 5,46,12) = (/ &
     &2.6666e+02_rb,2.0015e+02_rb,1.3366e+02_rb,7.0579e+01_rb,1.4676e+01_rb/)
      kbo(:, 1,47,12) = (/ &
     &4.4761e+01_rb,3.5144e+01_rb,3.2385e+01_rb,2.9408e+01_rb,1.3714e+01_rb/)
      kbo(:, 2,47,12) = (/ &
     &7.4836e+01_rb,5.6569e+01_rb,4.3337e+01_rb,3.5671e+01_rb,1.4241e+01_rb/)
      kbo(:, 3,47,12) = (/ &
     &1.1904e+02_rb,8.9427e+01_rb,6.2124e+01_rb,4.2720e+01_rb,1.4513e+01_rb/)
      kbo(:, 4,47,12) = (/ &
     &1.8135e+02_rb,1.3615e+02_rb,9.1529e+01_rb,5.3518e+01_rb,1.4491e+01_rb/)
      kbo(:, 5,47,12) = (/ &
     &2.6557e+02_rb,1.9933e+02_rb,1.3311e+02_rb,7.0671e+01_rb,1.4708e+01_rb/)
      kbo(:, 1,48,12) = (/ &
     &4.1069e+01_rb,3.2801e+01_rb,3.1586e+01_rb,2.8865e+01_rb,1.3457e+01_rb/)
      kbo(:, 2,48,12) = (/ &
     &7.0975e+01_rb,5.3819e+01_rb,4.2302e+01_rb,3.5478e+01_rb,1.4056e+01_rb/)
      kbo(:, 3,48,12) = (/ &
     &1.1614e+02_rb,8.7254e+01_rb,6.1078e+01_rb,4.2908e+01_rb,1.4399e+01_rb/)
      kbo(:, 4,48,12) = (/ &
     &1.8113e+02_rb,1.3599e+02_rb,9.1560e+01_rb,5.4099e+01_rb,1.4482e+01_rb/)
      kbo(:, 5,48,12) = (/ &
     &2.7078e+02_rb,2.0323e+02_rb,1.3570e+02_rb,7.2222e+01_rb,1.4704e+01_rb/)
      kbo(:, 1,49,12) = (/ &
     &3.8731e+01_rb,3.1486e+01_rb,3.1265e+01_rb,2.8628e+01_rb,1.3159e+01_rb/)
      kbo(:, 2,49,12) = (/ &
     &6.9397e+01_rb,5.2759e+01_rb,4.2223e+01_rb,3.5759e+01_rb,1.3841e+01_rb/)
      kbo(:, 3,49,12) = (/ &
     &1.1709e+02_rb,8.7966e+01_rb,6.1851e+01_rb,4.3836e+01_rb,1.4254e+01_rb/)
      kbo(:, 4,49,12) = (/ &
     &1.8714e+02_rb,1.4049e+02_rb,9.4623e+01_rb,5.5973e+01_rb,1.4472e+01_rb/)
      kbo(:, 5,49,12) = (/ &
     &2.8581e+02_rb,2.1450e+02_rb,1.4321e+02_rb,7.6013e+01_rb,1.4713e+01_rb/)
      kbo(:, 1,50,12) = (/ &
     &3.4845e+01_rb,2.9222e+01_rb,3.0219e+01_rb,2.7841e+01_rb,1.2848e+01_rb/)
      kbo(:, 2,50,12) = (/ &
     &6.4780e+01_rb,4.9517e+01_rb,4.0909e+01_rb,3.5306e+01_rb,1.3605e+01_rb/)
      kbo(:, 3,50,12) = (/ &
     &1.1280e+02_rb,8.4759e+01_rb,6.0286e+01_rb,4.3805e+01_rb,1.4114e+01_rb/)
      kbo(:, 4,50,12) = (/ &
     &1.8491e+02_rb,1.3882e+02_rb,9.3723e+01_rb,5.6179e+01_rb,1.4404e+01_rb/)
      kbo(:, 5,50,12) = (/ &
     &2.8849e+02_rb,2.1650e+02_rb,1.4455e+02_rb,7.7018e+01_rb,1.4673e+01_rb/)
      kbo(:, 1,51,12) = (/ &
     &3.0519e+01_rb,2.6754e+01_rb,2.8814e+01_rb,2.6775e+01_rb,1.2516e+01_rb/)
      kbo(:, 2,51,12) = (/ &
     &5.8895e+01_rb,4.5411e+01_rb,3.9106e+01_rb,3.4462e+01_rb,1.3344e+01_rb/)
      kbo(:, 3,51,12) = (/ &
     &1.0594e+02_rb,7.9673e+01_rb,5.7668e+01_rb,4.3297e+01_rb,1.3940e+01_rb/)
      kbo(:, 4,51,12) = (/ &
     &1.7846e+02_rb,1.3399e+02_rb,9.0800e+01_rb,5.5546e+01_rb,1.4316e+01_rb/)
      kbo(:, 5,51,12) = (/ &
     &2.8452e+02_rb,2.1353e+02_rb,1.4260e+02_rb,7.6552e+01_rb,1.4603e+01_rb/)
      kbo(:, 1,52,12) = (/ &
     &2.7172e+01_rb,2.4938e+01_rb,2.7638e+01_rb,2.5911e+01_rb,1.2154e+01_rb/)
      kbo(:, 2,52,12) = (/ &
     &5.4443e+01_rb,4.2404e+01_rb,3.7930e+01_rb,3.3867e+01_rb,1.3047e+01_rb/)
      kbo(:, 3,52,12) = (/ &
     &1.0140e+02_rb,7.6335e+01_rb,5.6140e+01_rb,4.3191e+01_rb,1.3744e+01_rb/)
      kbo(:, 4,52,12) = (/ &
     &1.7597e+02_rb,1.3212e+02_rb,8.9791e+01_rb,5.5744e+01_rb,1.4204e+01_rb/)
      kbo(:, 5,52,12) = (/ &
     &2.8690e+02_rb,2.1531e+02_rb,1.4382e+02_rb,7.7564e+01_rb,1.4539e+01_rb/)
      kbo(:, 1,53,12) = (/ &
     &2.4852e+01_rb,2.3783e+01_rb,2.6797e+01_rb,2.5306e+01_rb,1.1753e+01_rb/)
      kbo(:, 2,53,12) = (/ &
     &5.1806e+01_rb,4.0721e+01_rb,3.7447e+01_rb,3.3656e+01_rb,1.2720e+01_rb/)
      kbo(:, 3,53,12) = (/ &
     &1.0002e+02_rb,7.5369e+01_rb,5.6019e+01_rb,4.3667e+01_rb,1.3525e+01_rb/)
      kbo(:, 4,53,12) = (/ &
     &1.7916e+02_rb,1.3451e+02_rb,9.1517e+01_rb,5.7134e+01_rb,1.4067e+01_rb/)
      kbo(:, 5,53,12) = (/ &
     &2.9931e+02_rb,2.2462e+02_rb,1.5004e+02_rb,8.0825e+01_rb,1.4479e+01_rb/)
      kbo(:, 1,54,12) = (/ &
     &1.9814e+01_rb,2.1046e+01_rb,2.4392e+01_rb,2.3477e+01_rb,1.1367e+01_rb/)
      kbo(:, 2,54,12) = (/ &
     &4.3028e+01_rb,3.5019e+01_rb,3.4545e+01_rb,3.1504e+01_rb,1.2419e+01_rb/)
      kbo(:, 3,54,12) = (/ &
     &8.5988e+01_rb,6.5126e+01_rb,5.0588e+01_rb,4.1403e+01_rb,1.3282e+01_rb/)
      kbo(:, 4,54,12) = (/ &
     &1.5878e+02_rb,1.1923e+02_rb,8.2044e+01_rb,5.3856e+01_rb,1.3889e+01_rb/)
      kbo(:, 5,54,12) = (/ &
     &2.7225e+02_rb,2.0433e+02_rb,1.3669e+02_rb,7.5460e+01_rb,1.4357e+01_rb/)
      kbo(:, 1,55,12) = (/ &
     &1.4388e+01_rb,1.7950e+01_rb,2.1372e+01_rb,2.1168e+01_rb,1.0974e+01_rb/)
      kbo(:, 2,55,12) = (/ &
     &3.2596e+01_rb,2.8570e+01_rb,3.0630e+01_rb,2.8429e+01_rb,1.2112e+01_rb/)
      kbo(:, 3,55,12) = (/ &
     &6.7426e+01_rb,5.1747e+01_rb,4.3446e+01_rb,3.7705e+01_rb,1.3016e+01_rb/)
      kbo(:, 4,55,12) = (/ &
     &1.2839e+02_rb,9.6452e+01_rb,6.8218e+01_rb,4.8732e+01_rb,1.3748e+01_rb/)
      kbo(:, 5,55,12) = (/ &
     &2.2612e+02_rb,1.6973e+02_rb,1.1415e+02_rb,6.6296e+01_rb,1.4200e+01_rb/)
      kbo(:, 1,56,12) = (/ &
     &1.0289e+01_rb,1.5296e+01_rb,1.8729e+01_rb,1.9182e+01_rb,1.0528e+01_rb/)
      kbo(:, 2,56,12) = (/ &
     &2.4440e+01_rb,2.3761e+01_rb,2.7083e+01_rb,2.5668e+01_rb,1.1786e+01_rb/)
      kbo(:, 3,56,12) = (/ &
     &5.2397e+01_rb,4.1238e+01_rb,3.7979e+01_rb,3.4254e+01_rb,1.2753e+01_rb/)
      kbo(:, 4,56,12) = (/ &
     &1.0299e+02_rb,7.7598e+01_rb,5.7468e+01_rb,4.4519e+01_rb,1.3526e+01_rb/)
      kbo(:, 5,56,12) = (/ &
     &1.8650e+02_rb,1.4002e+02_rb,9.5113e+01_rb,5.8818e+01_rb,1.4062e+01_rb/)
      kbo(:, 1,57,12) = (/ &
     &7.2296e+00_rb,1.3018e+01_rb,1.6426e+01_rb,1.7434e+01_rb,1.0060e+01_rb/)
      kbo(:, 2,57,12) = (/ &
     &1.8079e+01_rb,2.0222e+01_rb,2.3856e+01_rb,2.3179e+01_rb,1.1422e+01_rb/)
      kbo(:, 3,57,12) = (/ &
     &4.0328e+01_rb,3.3389e+01_rb,3.3719e+01_rb,3.1042e+01_rb,1.2469e+01_rb/)
      kbo(:, 4,57,12) = (/ &
     &8.1861e+01_rb,6.2161e+01_rb,4.9031e+01_rb,4.0727e+01_rb,1.3313e+01_rb/)
      kbo(:, 5,57,12) = (/ &
     &1.5269e+02_rb,1.1466e+02_rb,7.9200e+01_rb,5.2806e+01_rb,1.3923e+01_rb/)
      kbo(:, 1,58,12) = (/ &
     &1.3378e+00_rb,3.6524e+00_rb,6.1661e+00_rb,9.4777e+00_rb,9.5880e+00_rb/)
      kbo(:, 2,58,12) = (/ &
     &3.5257e+00_rb,5.7408e+00_rb,8.9221e+00_rb,1.2503e+01_rb,1.1030e+01_rb/)
      kbo(:, 3,58,12) = (/ &
     &8.2101e+00_rb,9.0368e+00_rb,1.2733e+01_rb,1.6723e+01_rb,1.2197e+01_rb/)
      kbo(:, 4,58,12) = (/ &
     &1.7210e+01_rb,1.6233e+01_rb,1.7913e+01_rb,2.2101e+01_rb,1.3072e+01_rb/)
      kbo(:, 5,58,12) = (/ &
     &3.3035e+01_rb,3.0353e+01_rb,2.7850e+01_rb,2.8452e+01_rb,1.3763e+01_rb/)
      kbo(:, 1,59,12) = (/ &
     &1.3260e+00_rb,4.5998e+00_rb,7.5889e+00_rb,1.0717e+01_rb,9.4033e+00_rb/)
      kbo(:, 2,59,12) = (/ &
     &3.6097e+00_rb,7.0919e+00_rb,1.0659e+01_rb,1.3898e+01_rb,1.0849e+01_rb/)
      kbo(:, 3,59,12) = (/ &
     &8.5987e+00_rb,1.0607e+01_rb,1.5006e+01_rb,1.8102e+01_rb,1.2070e+01_rb/)
      kbo(:, 4,59,12) = (/ &
     &1.8312e+01_rb,1.7368e+01_rb,2.0492e+01_rb,2.3593e+01_rb,1.3009e+01_rb/)
      kbo(:, 5,59,12) = (/ &
     &3.5634e+01_rb,3.1449e+01_rb,2.9305e+01_rb,3.0122e+01_rb,1.3723e+01_rb/)
      kbo(:, 1,13,13) = (/ &
     &1.2162e+03_rb,9.1228e+02_rb,6.0835e+02_rb,3.0442e+02_rb,5.7609e+00_rb/)
      kbo(:, 2,13,13) = (/ &
     &1.7381e+03_rb,1.3037e+03_rb,8.6931e+02_rb,4.3490e+02_rb,5.6432e+00_rb/)
      kbo(:, 3,13,13) = (/ &
     &2.3576e+03_rb,1.7683e+03_rb,1.1791e+03_rb,5.8977e+02_rb,5.5255e+00_rb/)
      kbo(:, 4,13,13) = (/ &
     &3.0662e+03_rb,2.2998e+03_rb,1.5334e+03_rb,7.6694e+02_rb,5.4044e+00_rb/)
      kbo(:, 5,13,13) = (/ &
     &3.8594e+03_rb,2.8947e+03_rb,1.9299e+03_rb,9.6523e+02_rb,5.2355e+00_rb/)
      kbo(:, 1,14,13) = (/ &
     &7.9362e+02_rb,5.9533e+02_rb,3.9705e+02_rb,1.9877e+02_rb,6.3709e+00_rb/)
      kbo(:, 2,14,13) = (/ &
     &1.1345e+03_rb,8.5102e+02_rb,5.6751e+02_rb,2.8400e+02_rb,6.2388e+00_rb/)
      kbo(:, 3,14,13) = (/ &
     &1.5436e+03_rb,1.1578e+03_rb,7.7203e+02_rb,3.8627e+02_rb,6.0986e+00_rb/)
      kbo(:, 4,14,13) = (/ &
     &2.0139e+03_rb,1.5105e+03_rb,1.0072e+03_rb,5.0384e+02_rb,5.9465e+00_rb/)
      kbo(:, 5,14,13) = (/ &
     &2.5413e+03_rb,1.9061e+03_rb,1.2709e+03_rb,6.3569e+02_rb,5.7898e+00_rb/)
      kbo(:, 1,15,13) = (/ &
     &5.3943e+02_rb,4.0470e+02_rb,2.6997e+02_rb,1.3524e+02_rb,7.0503e+00_rb/)
      kbo(:, 2,15,13) = (/ &
     &7.7270e+02_rb,5.7965e+02_rb,3.8660e+02_rb,1.9355e+02_rb,6.8771e+00_rb/)
      kbo(:, 3,15,13) = (/ &
     &1.0520e+03_rb,7.8913e+02_rb,5.2626e+02_rb,2.6338e+02_rb,6.6997e+00_rb/)
      kbo(:, 4,15,13) = (/ &
     &1.3743e+03_rb,1.0308e+03_rb,6.8738e+02_rb,3.4394e+02_rb,6.5201e+00_rb/)
      kbo(:, 5,15,13) = (/ &
     &1.7371e+03_rb,1.3030e+03_rb,8.6881e+02_rb,4.3464e+02_rb,6.3410e+00_rb/)
      kbo(:, 1,16,13) = (/ &
     &4.4449e+02_rb,3.3349e+02_rb,2.2249e+02_rb,1.1150e+02_rb,7.7448e+00_rb/)
      kbo(:, 2,16,13) = (/ &
     &6.3512e+02_rb,4.7646e+02_rb,3.1780e+02_rb,1.5915e+02_rb,7.5481e+00_rb/)
      kbo(:, 3,16,13) = (/ &
     &8.6257e+02_rb,6.4705e+02_rb,4.3152e+02_rb,2.1600e+02_rb,7.3457e+00_rb/)
      kbo(:, 4,16,13) = (/ &
     &1.1249e+03_rb,8.4376e+02_rb,5.6265e+02_rb,2.8155e+02_rb,7.1472e+00_rb/)
      kbo(:, 5,16,13) = (/ &
     &1.4185e+03_rb,1.0640e+03_rb,7.0946e+02_rb,3.5494e+02_rb,6.9622e+00_rb/)
      kbo(:, 1,17,13) = (/ &
     &3.7725e+02_rb,2.8305e+02_rb,1.8886e+02_rb,9.4667e+01_rb,8.5153e+00_rb/)
      kbo(:, 2,17,13) = (/ &
     &5.3579e+02_rb,4.0195e+02_rb,2.6812e+02_rb,1.3428e+02_rb,8.2959e+00_rb/)
      kbo(:, 3,17,13) = (/ &
     &7.2432e+02_rb,5.4334e+02_rb,3.6237e+02_rb,1.8140e+02_rb,8.0790e+00_rb/)
      kbo(:, 4,17,13) = (/ &
     &9.4095e+02_rb,7.0582e+02_rb,4.7068e+02_rb,2.3555e+02_rb,7.8782e+00_rb/)
      kbo(:, 5,17,13) = (/ &
     &1.1800e+03_rb,8.8510e+02_rb,5.9020e+02_rb,2.9531e+02_rb,7.6889e+00_rb/)
      kbo(:, 1,18,13) = (/ &
     &3.2277e+02_rb,2.4218e+02_rb,1.6159e+02_rb,8.1002e+01_rb,9.4611e+00_rb/)
      kbo(:, 2,18,13) = (/ &
     &4.5535e+02_rb,3.4161e+02_rb,2.2787e+02_rb,1.1413e+02_rb,9.2187e+00_rb/)
      kbo(:, 3,18,13) = (/ &
     &6.1248e+02_rb,4.5945e+02_rb,3.0643e+02_rb,1.5340e+02_rb,8.9925e+00_rb/)
      kbo(:, 4,18,13) = (/ &
     &7.9039e+02_rb,5.9288e+02_rb,3.9538e+02_rb,1.9787e+02_rb,8.7824e+00_rb/)
      kbo(:, 5,18,13) = (/ &
     &9.8664e+02_rb,7.4007e+02_rb,4.9351e+02_rb,2.4694e+02_rb,8.5832e+00_rb/)
      kbo(:, 1,19,13) = (/ &
     &2.6617e+02_rb,1.9972e+02_rb,1.3326e+02_rb,6.6812e+01_rb,1.0581e+01_rb/)
      kbo(:, 2,19,13) = (/ &
     &3.7316e+02_rb,2.7996e+02_rb,1.8675e+02_rb,9.3549e+01_rb,1.0334e+01_rb/)
      kbo(:, 3,19,13) = (/ &
     &4.9811e+02_rb,3.7366e+02_rb,2.4922e+02_rb,1.2478e+02_rb,1.0097e+01_rb/)
      kbo(:, 4,19,13) = (/ &
     &6.3941e+02_rb,4.7964e+02_rb,3.1987e+02_rb,1.6011e+02_rb,9.8657e+00_rb/)
      kbo(:, 5,19,13) = (/ &
     &7.9554e+02_rb,5.9674e+02_rb,3.9794e+02_rb,1.9915e+02_rb,9.6319e+00_rb/)
      kbo(:, 1,20,13) = (/ &
     &2.3446e+02_rb,1.7592e+02_rb,1.1738e+02_rb,5.8849e+01_rb,1.1882e+01_rb/)
      kbo(:, 2,20,13) = (/ &
     &3.2565e+02_rb,2.4431e+02_rb,1.6298e+02_rb,8.1640e+01_rb,1.1619e+01_rb/)
      kbo(:, 3,20,13) = (/ &
     &4.3162e+02_rb,3.2380e+02_rb,2.1597e+02_rb,1.0814e+02_rb,1.1360e+01_rb/)
      kbo(:, 4,20,13) = (/ &
     &5.5163e+02_rb,4.1380e+02_rb,2.7598e+02_rb,1.3815e+02_rb,1.1092e+01_rb/)
      kbo(:, 5,20,13) = (/ &
     &6.8433e+02_rb,5.1333e+02_rb,3.4234e+02_rb,1.7134e+02_rb,1.0817e+01_rb/)
      kbo(:, 1,21,13) = (/ &
     &2.0791e+02_rb,1.5600e+02_rb,1.0410e+02_rb,5.2196e+01_rb,1.3299e+01_rb/)
      kbo(:, 2,21,13) = (/ &
     &2.8639e+02_rb,2.1487e+02_rb,1.4335e+02_rb,7.1825e+01_rb,1.3019e+01_rb/)
      kbo(:, 3,21,13) = (/ &
     &3.7808e+02_rb,2.8364e+02_rb,1.8920e+02_rb,9.4762e+01_rb,1.2725e+01_rb/)
      kbo(:, 4,21,13) = (/ &
     &4.8188e+02_rb,3.6150e+02_rb,2.4111e+02_rb,1.2072e+02_rb,1.2414e+01_rb/)
      kbo(:, 5,21,13) = (/ &
     &5.9751e+02_rb,4.4821e+02_rb,2.9892e+02_rb,1.4963e+02_rb,1.2098e+01_rb/)
      kbo(:, 1,22,13) = (/ &
     &1.8905e+02_rb,1.4186e+02_rb,9.4669e+01_rb,4.7658e+01_rb,1.4760e+01_rb/)
      kbo(:, 2,22,13) = (/ &
     &2.5846e+02_rb,1.9392e+02_rb,1.2939e+02_rb,6.4852e+01_rb,1.4444e+01_rb/)
      kbo(:, 3,22,13) = (/ &
     &3.3936e+02_rb,2.5460e+02_rb,1.6984e+02_rb,8.5086e+01_rb,1.4104e+01_rb/)
      kbo(:, 4,22,13) = (/ &
     &4.3137e+02_rb,3.2361e+02_rb,2.1585e+02_rb,1.0809e+02_rb,1.3746e+01_rb/)
      kbo(:, 5,22,13) = (/ &
     &5.3388e+02_rb,4.0049e+02_rb,2.6710e+02_rb,1.3372e+02_rb,1.3378e+01_rb/)
      kbo(:, 1,23,13) = (/ &
     &1.7012e+02_rb,1.2766e+02_rb,8.5209e+01_rb,4.3736e+01_rb,1.6211e+01_rb/)
      kbo(:, 2,23,13) = (/ &
     &2.3107e+02_rb,1.7338e+02_rb,1.1569e+02_rb,5.8148e+01_rb,1.5847e+01_rb/)
      kbo(:, 3,23,13) = (/ &
     &3.0241e+02_rb,2.2689e+02_rb,1.5137e+02_rb,7.5851e+01_rb,1.5467e+01_rb/)
      kbo(:, 4,23,13) = (/ &
     &3.8349e+02_rb,2.8770e+02_rb,1.9191e+02_rb,9.6125e+01_rb,1.5066e+01_rb/)
      kbo(:, 5,23,13) = (/ &
     &4.7369e+02_rb,3.5535e+02_rb,2.3701e+02_rb,1.1867e+02_rb,1.4651e+01_rb/)
      kbo(:, 1,24,13) = (/ &
     &1.5057e+02_rb,1.1300e+02_rb,7.5436e+01_rb,4.0246e+01_rb,1.7609e+01_rb/)
      kbo(:, 2,24,13) = (/ &
     &2.0367e+02_rb,1.5283e+02_rb,1.0199e+02_rb,5.2083e+01_rb,1.7203e+01_rb/)
      kbo(:, 3,24,13) = (/ &
     &2.6558e+02_rb,1.9927e+02_rb,1.3295e+02_rb,6.6835e+01_rb,1.6770e+01_rb/)
      kbo(:, 4,24,13) = (/ &
     &3.3594e+02_rb,2.5203e+02_rb,1.6813e+02_rb,8.4226e+01_rb,1.6325e+01_rb/)
      kbo(:, 5,24,13) = (/ &
     &4.1379e+02_rb,3.1042e+02_rb,2.0705e+02_rb,1.0369e+02_rb,1.5864e+01_rb/)
      kbo(:, 1,25,13) = (/ &
     &1.3420e+02_rb,1.0073e+02_rb,6.7255e+01_rb,3.7941e+01_rb,1.8922e+01_rb/)
      kbo(:, 2,25,13) = (/ &
     &1.8061e+02_rb,1.3554e+02_rb,9.0464e+01_rb,4.7603e+01_rb,1.8471e+01_rb/)
      kbo(:, 3,25,13) = (/ &
     &2.3464e+02_rb,1.7606e+02_rb,1.1747e+02_rb,5.9840e+01_rb,1.7998e+01_rb/)
      kbo(:, 4,25,13) = (/ &
     &2.9580e+02_rb,2.2193e+02_rb,1.4806e+02_rb,7.4446e+01_rb,1.7506e+01_rb/)
      kbo(:, 5,25,13) = (/ &
     &3.6422e+02_rb,2.7325e+02_rb,1.8228e+02_rb,9.1309e+01_rb,1.6997e+01_rb/)
      kbo(:, 1,26,13) = (/ &
     &1.2114e+02_rb,9.0934e+01_rb,6.0946e+01_rb,3.6699e+01_rb,2.0129e+01_rb/)
      kbo(:, 2,26,13) = (/ &
     &1.6224e+02_rb,1.2175e+02_rb,8.1268e+01_rb,4.4540e+01_rb,1.9645e+01_rb/)
      kbo(:, 3,26,13) = (/ &
     &2.0993e+02_rb,1.5753e+02_rb,1.0512e+02_rb,5.4768e+01_rb,1.9134e+01_rb/)
      kbo(:, 4,26,13) = (/ &
     &2.6457e+02_rb,1.9850e+02_rb,1.3244e+02_rb,6.7280e+01_rb,1.8599e+01_rb/)
      kbo(:, 5,26,13) = (/ &
     &3.2617e+02_rb,2.4471e+02_rb,1.6326e+02_rb,8.2064e+01_rb,1.8043e+01_rb/)
      kbo(:, 1,27,13) = (/ &
     &1.1288e+02_rb,8.4731e+01_rb,5.7258e+01_rb,3.6591e+01_rb,2.1235e+01_rb/)
      kbo(:, 2,27,13) = (/ &
     &1.5040e+02_rb,1.1288e+02_rb,7.5447e+01_rb,4.3146e+01_rb,2.0722e+01_rb/)
      kbo(:, 3,27,13) = (/ &
     &1.9447e+02_rb,1.4593e+02_rb,9.7397e+01_rb,5.2019e+01_rb,2.0175e+01_rb/)
      kbo(:, 4,27,13) = (/ &
     &2.4522e+02_rb,1.8400e+02_rb,1.2279e+02_rb,6.3250e+01_rb,1.9596e+01_rb/)
      kbo(:, 5,27,13) = (/ &
     &3.0305e+02_rb,2.2738e+02_rb,1.5172e+02_rb,7.6751e+01_rb,1.8997e+01_rb/)
      kbo(:, 1,28,13) = (/ &
     &1.0746e+02_rb,8.0667e+01_rb,5.5081e+01_rb,3.7035e+01_rb,2.2229e+01_rb/)
      kbo(:, 2,28,13) = (/ &
     &1.4298e+02_rb,1.0731e+02_rb,7.1965e+01_rb,4.2784e+01_rb,2.1691e+01_rb/)
      kbo(:, 3,28,13) = (/ &
     &1.8493e+02_rb,1.3878e+02_rb,9.2649e+01_rb,5.0725e+01_rb,2.1105e+01_rb/)
      kbo(:, 4,28,13) = (/ &
     &2.3378e+02_rb,1.7543e+02_rb,1.1708e+02_rb,6.1166e+01_rb,2.0486e+01_rb/)
      kbo(:, 5,28,13) = (/ &
     &2.9002e+02_rb,2.1761e+02_rb,1.4521e+02_rb,7.3973e+01_rb,1.9841e+01_rb/)
      kbo(:, 1,29,13) = (/ &
     &1.0862e+02_rb,8.1537e+01_rb,5.6008e+01_rb,3.8525e+01_rb,2.3108e+01_rb/)
      kbo(:, 2,29,13) = (/ &
     &1.4451e+02_rb,1.0847e+02_rb,7.2927e+01_rb,4.4121e+01_rb,2.2541e+01_rb/)
      kbo(:, 3,29,13) = (/ &
     &1.8742e+02_rb,1.4066e+02_rb,9.3939e+01_rb,5.2003e+01_rb,2.1917e+01_rb/)
      kbo(:, 4,29,13) = (/ &
     &2.3814e+02_rb,1.7870e+02_rb,1.1927e+02_rb,6.2626e+01_rb,2.1254e+01_rb/)
      kbo(:, 5,29,13) = (/ &
     &2.9712e+02_rb,2.2294e+02_rb,1.4877e+02_rb,7.5955e+01_rb,2.0559e+01_rb/)
      kbo(:, 1,30,13) = (/ &
     &1.1263e+02_rb,8.4559e+01_rb,5.8240e+01_rb,4.0363e+01_rb,2.3867e+01_rb/)
      kbo(:, 2,30,13) = (/ &
     &1.5031e+02_rb,1.1283e+02_rb,7.5908e+01_rb,4.6135e+01_rb,2.3263e+01_rb/)
      kbo(:, 3,30,13) = (/ &
     &1.9606e+02_rb,1.4715e+02_rb,9.8280e+01_rb,5.4419e+01_rb,2.2598e+01_rb/)
      kbo(:, 4,30,13) = (/ &
     &2.5065e+02_rb,1.8809e+02_rb,1.2553e+02_rb,6.5875e+01_rb,2.1889e+01_rb/)
      kbo(:, 5,30,13) = (/ &
     &3.1497e+02_rb,2.3633e+02_rb,1.5769e+02_rb,8.0462e+01_rb,2.1149e+01_rb/)
      kbo(:, 1,31,13) = (/ &
     &1.2378e+02_rb,9.2930e+01_rb,6.3680e+01_rb,4.3209e+01_rb,2.4494e+01_rb/)
      kbo(:, 2,31,13) = (/ &
     &1.6598e+02_rb,1.2458e+02_rb,8.3650e+01_rb,4.9789e+01_rb,2.3857e+01_rb/)
      kbo(:, 3,31,13) = (/ &
     &2.1788e+02_rb,1.6351e+02_rb,1.0916e+02_rb,5.9548e+01_rb,2.3152e+01_rb/)
      kbo(:, 4,31,13) = (/ &
     &2.8056e+02_rb,2.1051e+02_rb,1.4048e+02_rb,7.3069e+01_rb,2.2400e+01_rb/)
      kbo(:, 5,31,13) = (/ &
     &3.5485e+02_rb,2.6624e+02_rb,1.7763e+02_rb,9.0172e+01_rb,2.1588e+01_rb/)
      kbo(:, 1,32,13) = (/ &
     &1.3906e+02_rb,1.0439e+02_rb,7.1039e+01_rb,4.6595e+01_rb,2.5004e+01_rb/)
      kbo(:, 2,32,13) = (/ &
     &1.8760e+02_rb,1.4080e+02_rb,9.4311e+01_rb,5.4496e+01_rb,2.4327e+01_rb/)
      kbo(:, 3,32,13) = (/ &
     &2.4774e+02_rb,1.8590e+02_rb,1.2408e+02_rb,6.6467e+01_rb,2.3584e+01_rb/)
      kbo(:, 4,32,13) = (/ &
     &3.2109e+02_rb,2.4091e+02_rb,1.6074e+02_rb,8.2698e+01_rb,2.2768e+01_rb/)
      kbo(:, 5,32,13) = (/ &
     &4.0872e+02_rb,3.0664e+02_rb,2.0456e+02_rb,1.0327e+02_rb,2.1879e+01_rb/)
      kbo(:, 1,33,13) = (/ &
     &1.6104e+02_rb,1.2087e+02_rb,8.1641e+01_rb,5.1002e+01_rb,2.5402e+01_rb/)
      kbo(:, 2,33,13) = (/ &
     &2.1845e+02_rb,1.6394e+02_rb,1.0954e+02_rb,6.1146e+01_rb,2.4688e+01_rb/)
      kbo(:, 3,33,13) = (/ &
     &2.9039e+02_rb,2.1789e+02_rb,1.4539e+02_rb,7.6233e+01_rb,2.3897e+01_rb/)
      kbo(:, 4,33,13) = (/ &
     &3.7885e+02_rb,2.8424e+02_rb,1.8963e+02_rb,9.6481e+01_rb,2.2987e+01_rb/)
      kbo(:, 5,33,13) = (/ &
     &4.8539e+02_rb,3.6414e+02_rb,2.4290e+02_rb,1.2206e+02_rb,2.2183e+01_rb/)
      kbo(:, 1,34,13) = (/ &
     &1.8189e+02_rb,1.3651e+02_rb,9.1768e+01_rb,5.5207e+01_rb,2.5729e+01_rb/)
      kbo(:, 2,34,13) = (/ &
     &2.4868e+02_rb,1.8660e+02_rb,1.2456e+02_rb,6.7745e+01_rb,2.4984e+01_rb/)
      kbo(:, 3,34,13) = (/ &
     &3.3298e+02_rb,2.4983e+02_rb,1.6669e+02_rb,8.6174e+01_rb,2.4083e+01_rb/)
      kbo(:, 4,34,13) = (/ &
     &4.3751e+02_rb,3.2823e+02_rb,2.1895e+02_rb,1.1064e+02_rb,2.3253e+01_rb/)
      kbo(:, 5,34,13) = (/ &
     &5.6416e+02_rb,4.2322e+02_rb,2.8227e+02_rb,1.4151e+02_rb,2.2432e+01_rb/)
      kbo(:, 1,35,13) = (/ &
     &2.0192e+02_rb,1.5153e+02_rb,1.0158e+02_rb,5.9346e+01_rb,2.6044e+01_rb/)
      kbo(:, 2,35,13) = (/ &
     &2.7880e+02_rb,2.0919e+02_rb,1.3960e+02_rb,7.4527e+01_rb,2.5204e+01_rb/)
      kbo(:, 3,35,13) = (/ &
     &3.7683e+02_rb,2.8271e+02_rb,1.8860e+02_rb,9.6532e+01_rb,2.4315e+01_rb/)
      kbo(:, 4,35,13) = (/ &
     &4.9906e+02_rb,3.7439e+02_rb,2.4972e+02_rb,1.2566e+02_rb,2.3549e+01_rb/)
      kbo(:, 5,35,13) = (/ &
     &6.4817e+02_rb,4.8622e+02_rb,3.2427e+02_rb,1.6242e+02_rb,2.2556e+01_rb/)
      kbo(:, 1,36,13) = (/ &
     &2.1718e+02_rb,1.6298e+02_rb,1.0911e+02_rb,6.2682e+01_rb,2.6372e+01_rb/)
      kbo(:, 2,36,13) = (/ &
     &3.0353e+02_rb,2.2773e+02_rb,1.5196e+02_rb,8.0255e+01_rb,2.5484e+01_rb/)
      kbo(:, 3,36,13) = (/ &
     &4.1487e+02_rb,3.1124e+02_rb,2.0762e+02_rb,1.0565e+02_rb,2.4690e+01_rb/)
      kbo(:, 4,36,13) = (/ &
     &5.5465e+02_rb,4.1607e+02_rb,2.7750e+02_rb,1.3934e+02_rb,2.3815e+01_rb/)
      kbo(:, 5,36,13) = (/ &
     &7.2608e+02_rb,5.4465e+02_rb,3.6322e+02_rb,1.8189e+02_rb,2.2654e+01_rb/)
      kbo(:, 1,37,13) = (/ &
     &2.1788e+02_rb,1.6349e+02_rb,1.0950e+02_rb,6.3289e+01_rb,2.6719e+01_rb/)
      kbo(:, 2,37,13) = (/ &
     &3.0912e+02_rb,2.3193e+02_rb,1.5476e+02_rb,8.1797e+01_rb,2.5860e+01_rb/)
      kbo(:, 3,37,13) = (/ &
     &4.2846e+02_rb,3.2142e+02_rb,2.1440e+02_rb,1.0906e+02_rb,2.5054e+01_rb/)
      kbo(:, 4,37,13) = (/ &
     &5.7982e+02_rb,4.3495e+02_rb,2.9008e+02_rb,1.4561e+02_rb,2.4158e+01_rb/)
      kbo(:, 5,37,13) = (/ &
     &7.6717e+02_rb,5.7546e+02_rb,3.8377e+02_rb,1.9217e+02_rb,2.2948e+01_rb/)
      kbo(:, 1,38,13) = (/ &
     &2.2254e+02_rb,1.6699e+02_rb,1.1184e+02_rb,6.4633e+01_rb,2.7030e+01_rb/)
      kbo(:, 2,38,13) = (/ &
     &3.2031e+02_rb,2.4031e+02_rb,1.6035e+02_rb,8.4610e+01_rb,2.6154e+01_rb/)
      kbo(:, 3,38,13) = (/ &
     &4.5000e+02_rb,3.3758e+02_rb,2.2517e+02_rb,1.1439e+02_rb,2.5377e+01_rb/)
      kbo(:, 4,38,13) = (/ &
     &6.1626e+02_rb,4.6228e+02_rb,3.0831e+02_rb,1.5466e+02_rb,2.4449e+01_rb/)
      kbo(:, 5,38,13) = (/ &
     &8.2391e+02_rb,6.1802e+02_rb,4.1215e+02_rb,2.0635e+02_rb,2.3228e+01_rb/)
      kbo(:, 1,39,13) = (/ &
     &2.3401e+02_rb,1.7560e+02_rb,1.1754e+02_rb,6.7259e+01_rb,2.7265e+01_rb/)
      kbo(:, 2,39,13) = (/ &
     &3.4141e+02_rb,2.5613e+02_rb,1.7089e+02_rb,8.9638e+01_rb,2.6464e+01_rb/)
      kbo(:, 3,39,13) = (/ &
     &4.8579e+02_rb,3.6442e+02_rb,2.4308e+02_rb,1.2315e+02_rb,2.5687e+01_rb/)
      kbo(:, 4,39,13) = (/ &
     &6.7297e+02_rb,5.0482e+02_rb,3.3668e+02_rb,1.6875e+02_rb,2.4654e+01_rb/)
      kbo(:, 5,39,13) = (/ &
     &9.0884e+02_rb,6.8173e+02_rb,4.5462e+02_rb,2.2758e+02_rb,2.3620e+01_rb/)
      kbo(:, 1,40,13) = (/ &
     &2.2310e+02_rb,1.6742e+02_rb,1.1224e+02_rb,6.5625e+01_rb,2.7589e+01_rb/)
      kbo(:, 2,40,13) = (/ &
     &3.3051e+02_rb,2.4796e+02_rb,1.6546e+02_rb,8.7436e+01_rb,2.6816e+01_rb/)
      kbo(:, 3,40,13) = (/ &
     &4.7705e+02_rb,3.5787e+02_rb,2.3871e+02_rb,1.2127e+02_rb,2.6052e+01_rb/)
      kbo(:, 4,40,13) = (/ &
     &6.7033e+02_rb,5.0284e+02_rb,3.3536e+02_rb,1.6816e+02_rb,2.5038e+01_rb/)
      kbo(:, 5,40,13) = (/ &
     &9.1616e+02_rb,6.8722e+02_rb,4.5828e+02_rb,2.2941e+02_rb,2.4007e+01_rb/)
      kbo(:, 1,41,13) = (/ &
     &2.1165e+02_rb,1.5884e+02_rb,1.0674e+02_rb,6.4017e+01_rb,2.7888e+01_rb/)
      kbo(:, 2,41,13) = (/ &
     &3.1819e+02_rb,2.3873e+02_rb,1.5932e+02_rb,8.5003e+01_rb,2.7118e+01_rb/)
      kbo(:, 3,41,13) = (/ &
     &4.6591e+02_rb,3.4951e+02_rb,2.3314e+02_rb,1.1880e+02_rb,2.6396e+01_rb/)
      kbo(:, 4,41,13) = (/ &
     &6.6349e+02_rb,4.9771e+02_rb,3.3195e+02_rb,1.6656e+02_rb,2.5465e+01_rb/)
      kbo(:, 5,41,13) = (/ &
     &9.1847e+02_rb,6.8895e+02_rb,4.5944e+02_rb,2.3000e+02_rb,2.4337e+01_rb/)
      kbo(:, 1,42,13) = (/ &
     &2.0335e+02_rb,1.5260e+02_rb,1.0282e+02_rb,6.3023e+01_rb,2.8153e+01_rb/)
      kbo(:, 2,42,13) = (/ &
     &3.1003e+02_rb,2.3260e+02_rb,1.5524e+02_rb,8.3539e+01_rb,2.7395e+01_rb/)
      kbo(:, 3,42,13) = (/ &
     &4.6024e+02_rb,3.4526e+02_rb,2.3031e+02_rb,1.1765e+02_rb,2.6718e+01_rb/)
      kbo(:, 4,42,13) = (/ &
     &6.6381e+02_rb,4.9795e+02_rb,3.3210e+02_rb,1.6673e+02_rb,2.5846e+01_rb/)
      kbo(:, 5,42,13) = (/ &
     &9.3041e+02_rb,6.9790e+02_rb,4.6540e+02_rb,2.3298e+02_rb,2.4647e+01_rb/)
      kbo(:, 1,43,13) = (/ &
     &1.9244e+02_rb,1.4443e+02_rb,9.7744e+01_rb,6.1682e+01_rb,2.8431e+01_rb/)
      kbo(:, 2,43,13) = (/ &
     &2.9785e+02_rb,2.2347e+02_rb,1.4920e+02_rb,8.1276e+01_rb,2.7711e+01_rb/)
      kbo(:, 3,43,13) = (/ &
     &4.4872e+02_rb,3.3663e+02_rb,2.2458e+02_rb,1.1517e+02_rb,2.7073e+01_rb/)
      kbo(:, 4,43,13) = (/ &
     &6.5646e+02_rb,4.9243e+02_rb,3.2842e+02_rb,1.6500e+02_rb,2.6290e+01_rb/)
      kbo(:, 5,43,13) = (/ &
     &9.3268e+02_rb,6.9961e+02_rb,4.6654e+02_rb,2.3355e+02_rb,2.5000e+01_rb/)
      kbo(:, 1,44,13) = (/ &
     &1.8195e+02_rb,1.3656e+02_rb,9.2955e+01_rb,6.0559e+01_rb,2.8683e+01_rb/)
      kbo(:, 2,44,13) = (/ &
     &2.8673e+02_rb,2.1514e+02_rb,1.4371e+02_rb,7.9343e+01_rb,2.8022e+01_rb/)
      kbo(:, 3,44,13) = (/ &
     &4.3845e+02_rb,3.2893e+02_rb,2.1944e+02_rb,1.1299e+02_rb,2.7430e+01_rb/)
      kbo(:, 4,44,13) = (/ &
     &6.5082e+02_rb,4.8820e+02_rb,3.2560e+02_rb,1.6371e+02_rb,2.6724e+01_rb/)
      kbo(:, 5,44,13) = (/ &
     &9.3739e+02_rb,7.0313e+02_rb,4.6889e+02_rb,2.3472e+02_rb,2.5409e+01_rb/)
      kbo(:, 1,45,13) = (/ &
     &1.7490e+02_rb,1.3128e+02_rb,8.9889e+01_rb,6.0087e+01_rb,2.8892e+01_rb/)
      kbo(:, 2,45,13) = (/ &
     &2.8115e+02_rb,2.1096e+02_rb,1.4101e+02_rb,7.8697e+01_rb,2.8297e+01_rb/)
      kbo(:, 3,45,13) = (/ &
     &4.3612e+02_rb,3.2718e+02_rb,2.1828e+02_rb,1.1266e+02_rb,2.7806e+01_rb/)
      kbo(:, 4,45,13) = (/ &
     &6.5668e+02_rb,4.9260e+02_rb,3.2854e+02_rb,1.6526e+02_rb,2.7097e+01_rb/)
      kbo(:, 5,45,13) = (/ &
     &9.5876e+02_rb,7.1917e+02_rb,4.7958e+02_rb,2.4006e+02_rb,2.5776e+01_rb/)
      kbo(:, 1,46,13) = (/ &
     &1.6728e+02_rb,1.2556e+02_rb,8.6592e+01_rb,5.9568e+01_rb,2.9082e+01_rb/)
      kbo(:, 2,46,13) = (/ &
     &2.7457e+02_rb,2.0602e+02_rb,1.3784e+02_rb,7.7967e+01_rb,2.8545e+01_rb/)
      kbo(:, 3,46,13) = (/ &
     &4.3313e+02_rb,3.2494e+02_rb,2.1678e+02_rb,1.1222e+02_rb,2.8139e+01_rb/)
      kbo(:, 4,46,13) = (/ &
     &6.6163e+02_rb,4.9632e+02_rb,3.3103e+02_rb,1.6658e+02_rb,2.7449e+01_rb/)
      kbo(:, 5,46,13) = (/ &
     &9.8004e+02_rb,7.3512e+02_rb,4.9021e+02_rb,2.4539e+02_rb,2.6122e+01_rb/)
      kbo(:, 1,47,13) = (/ &
     &1.5431e+02_rb,1.1585e+02_rb,8.0931e+01_rb,5.8232e+01_rb,2.9262e+01_rb/)
      kbo(:, 2,47,13) = (/ &
     &2.5885e+02_rb,1.9423e+02_rb,1.3026e+02_rb,7.5643e+01_rb,2.8798e+01_rb/)
      kbo(:, 3,47,13) = (/ &
     &4.1682e+02_rb,3.1271e+02_rb,2.0862e+02_rb,1.0877e+02_rb,2.8446e+01_rb/)
      kbo(:, 4,47,13) = (/ &
     &6.4684e+02_rb,4.8523e+02_rb,3.2363e+02_rb,1.6308e+02_rb,2.7814e+01_rb/)
      kbo(:, 5,47,13) = (/ &
     &9.7279e+02_rb,7.2968e+02_rb,4.8659e+02_rb,2.4359e+02_rb,2.6548e+01_rb/)
      kbo(:, 1,48,13) = (/ &
     &1.4538e+02_rb,1.0915e+02_rb,7.7289e+01_rb,5.7662e+01_rb,2.9351e+01_rb/)
      kbo(:, 2,48,13) = (/ &
     &2.4931e+02_rb,1.8708e+02_rb,1.2577e+02_rb,7.4691e+01_rb,2.8998e+01_rb/)
      kbo(:, 3,48,13) = (/ &
     &4.1014e+02_rb,3.0770e+02_rb,2.0528e+02_rb,1.0766e+02_rb,2.8726e+01_rb/)
      kbo(:, 4,48,13) = (/ &
     &6.4756e+02_rb,4.8577e+02_rb,3.2399e+02_rb,1.6338e+02_rb,2.8115e+01_rb/)
      kbo(:, 5,48,13) = (/ &
     &9.8831e+02_rb,7.4133e+02_rb,4.9436e+02_rb,2.4748e+02_rb,2.6944e+01_rb/)
      kbo(:, 1,49,13) = (/ &
     &1.4120e+02_rb,1.0604e+02_rb,7.5941e+01_rb,5.8065e+01_rb,2.9380e+01_rb/)
      kbo(:, 2,49,13) = (/ &
     &2.4862e+02_rb,1.8657e+02_rb,1.2564e+02_rb,7.5519e+01_rb,2.9233e+01_rb/)
      kbo(:, 3,49,13) = (/ &
     &4.1758e+02_rb,3.1328e+02_rb,2.0900e+02_rb,1.0986e+02_rb,2.8989e+01_rb/)
      kbo(:, 4,49,13) = (/ &
     &6.7203e+02_rb,5.0412e+02_rb,3.3622e+02_rb,1.6950e+02_rb,2.8333e+01_rb/)
      kbo(:, 5,49,13) = (/ &
     &1.0415e+03_rb,7.8124e+02_rb,5.2096e+02_rb,2.6077e+02_rb,2.7256e+01_rb/)
      kbo(:, 1,50,13) = (/ &
     &1.3115e+02_rb,9.8605e+01_rb,7.2172e+01_rb,5.7503e+01_rb,2.9363e+01_rb/)
      kbo(:, 2,50,13) = (/ &
     &2.3790e+02_rb,1.7854e+02_rb,1.2070e+02_rb,7.4407e+01_rb,2.9364e+01_rb/)
      kbo(:, 3,50,13) = (/ &
     &4.0744e+02_rb,3.0568e+02_rb,2.0394e+02_rb,1.0804e+02_rb,2.9149e+01_rb/)
      kbo(:, 4,50,13) = (/ &
     &6.6872e+02_rb,5.0163e+02_rb,3.3456e+02_rb,1.6889e+02_rb,2.8599e+01_rb/)
      kbo(:, 5,50,13) = (/ &
     &1.0519e+03_rb,7.8905e+02_rb,5.2618e+02_rb,2.6337e+02_rb,2.7606e+01_rb/)
      kbo(:, 1,51,13) = (/ &
     &1.1865e+02_rb,8.9445e+01_rb,6.7680e+01_rb,5.6587e+01_rb,2.9290e+01_rb/)
      kbo(:, 2,51,13) = (/ &
     &2.2216e+02_rb,1.6674e+02_rb,1.1346e+02_rb,7.2415e+01_rb,2.9435e+01_rb/)
      kbo(:, 3,51,13) = (/ &
     &3.8895e+02_rb,2.9182e+02_rb,1.9472e+02_rb,1.0439e+02_rb,2.9207e+01_rb/)
      kbo(:, 4,51,13) = (/ &
     &6.5048e+02_rb,4.8795e+02_rb,3.2544e+02_rb,1.6473e+02_rb,2.8819e+01_rb/)
      kbo(:, 5,51,13) = (/ &
     &1.0401e+03_rb,7.8020e+02_rb,5.2027e+02_rb,2.6042e+02_rb,2.7943e+01_rb/)
      kbo(:, 1,52,13) = (/ &
     &1.0900e+02_rb,8.2517e+01_rb,6.4683e+01_rb,5.6184e+01_rb,2.9159e+01_rb/)
      kbo(:, 2,52,13) = (/ &
     &2.1154e+02_rb,1.5879e+02_rb,1.0876e+02_rb,7.1453e+01_rb,2.9466e+01_rb/)
      kbo(:, 3,52,13) = (/ &
     &3.8014e+02_rb,2.8522e+02_rb,1.9038e+02_rb,1.0306e+02_rb,2.9320e+01_rb/)
      kbo(:, 4,52,13) = (/ &
     &6.4723e+02_rb,4.8552e+02_rb,3.2381e+02_rb,1.6425e+02_rb,2.8993e+01_rb/)
      kbo(:, 5,52,13) = (/ &
     &1.0537e+03_rb,7.9041e+02_rb,5.2708e+02_rb,2.6382e+02_rb,2.8208e+01_rb/)
      kbo(:, 1,53,13) = (/ &
     &1.0283e+02_rb,7.8224e+01_rb,6.3272e+01_rb,5.6425e+01_rb,2.8966e+01_rb/)
      kbo(:, 2,53,13) = (/ &
     &2.0760e+02_rb,1.5584e+02_rb,1.0728e+02_rb,7.1858e+01_rb,2.9467e+01_rb/)
      kbo(:, 3,53,13) = (/ &
     &3.8429e+02_rb,2.8834e+02_rb,1.9251e+02_rb,1.0470e+02_rb,2.9381e+01_rb/)
      kbo(:, 4,53,13) = (/ &
     &6.6697e+02_rb,5.0033e+02_rb,3.3369e+02_rb,1.6926e+02_rb,2.9144e+01_rb/)
      kbo(:, 5,53,13) = (/ &
     &1.1061e+03_rb,8.2971e+02_rb,5.5328e+02_rb,2.7691e+02_rb,2.8402e+01_rb/)
      kbo(:, 1,54,13) = (/ &
     &8.4612e+01_rb,6.5337e+01_rb,5.7223e+01_rb,5.4259e+01_rb,2.8744e+01_rb/)
      kbo(:, 2,54,13) = (/ &
     &1.7755e+02_rb,1.3329e+02_rb,9.3740e+01_rb,6.7487e+01_rb,2.9376e+01_rb/)
      kbo(:, 3,54,13) = (/ &
     &3.3886e+02_rb,2.5427e+02_rb,1.7017e+02_rb,9.5886e+01_rb,2.9463e+01_rb/)
      kbo(:, 4,54,13) = (/ &
     &6.0067e+02_rb,4.5060e+02_rb,3.0055e+02_rb,1.5369e+02_rb,2.9315e+01_rb/)
      kbo(:, 5,54,13) = (/ &
     &1.0127e+03_rb,7.5962e+02_rb,5.0655e+02_rb,2.5368e+02_rb,2.8705e+01_rb/)
      kbo(:, 1,55,13) = (/ &
     &6.3387e+01_rb,5.0935e+01_rb,5.0168e+01_rb,5.0996e+01_rb,2.8487e+01_rb/)
      kbo(:, 2,55,13) = (/ &
     &1.3831e+02_rb,1.0410e+02_rb,7.7011e+01_rb,6.1667e+01_rb,2.9169e+01_rb/)
      kbo(:, 3,55,13) = (/ &
     &2.7282e+02_rb,2.0475e+02_rb,1.3822e+02_rb,8.3541e+01_rb,2.9521e+01_rb/)
      kbo(:, 4,55,13) = (/ &
     &4.9509e+02_rb,3.7143e+02_rb,2.4779e+02_rb,1.2925e+02_rb,2.9355e+01_rb/)
      kbo(:, 5,55,13) = (/ &
     &8.4764e+02_rb,6.3583e+02_rb,4.2402e+02_rb,2.1299e+02_rb,2.9013e+01_rb/)
      kbo(:, 1,56,13) = (/ &
     &4.6914e+01_rb,4.0782e+01_rb,4.5015e+01_rb,4.7952e+01_rb,2.8186e+01_rb/)
      kbo(:, 2,56,13) = (/ &
     &1.0661e+02_rb,8.1027e+01_rb,6.4934e+01_rb,5.7313e+01_rb,2.9008e+01_rb/)
      kbo(:, 3,56,13) = (/ &
     &2.1790e+02_rb,1.6356e+02_rb,1.1223e+02_rb,7.3951e+01_rb,2.9460e+01_rb/)
      kbo(:, 4,56,13) = (/ &
     &4.0585e+02_rb,3.0451e+02_rb,2.0325e+02_rb,1.0953e+02_rb,2.9455e+01_rb/)
      kbo(:, 5,56,13) = (/ &
     &7.0659e+02_rb,5.3004e+02_rb,3.5349e+02_rb,1.7883e+02_rb,2.9213e+01_rb/)
      kbo(:, 1,57,13) = (/ &
     &3.4230e+01_rb,3.3758e+01_rb,4.1164e+01_rb,4.5068e+01_rb,2.7829e+01_rb/)
      kbo(:, 2,57,13) = (/ &
     &8.1249e+01_rb,6.3007e+01_rb,5.6061e+01_rb,5.3761e+01_rb,2.8811e+01_rb/)
      kbo(:, 3,57,13) = (/ &
     &1.7223e+02_rb,1.2931e+02_rb,9.1411e+01_rb,6.6606e+01_rb,2.9335e+01_rb/)
      kbo(:, 4,57,13) = (/ &
     &3.3026e+02_rb,2.4783e+02_rb,1.6598e+02_rb,9.4193e+01_rb,2.9500e+01_rb/)
      kbo(:, 5,57,13) = (/ &
     &5.8636e+02_rb,4.3987e+02_rb,2.9340e+02_rb,1.5031e+02_rb,2.9335e+01_rb/)
      kbo(:, 1,58,13) = (/ &
     &6.5933e+00_rb,9.4893e+00_rb,1.6089e+01_rb,2.5300e+01_rb,2.7425e+01_rb/)
      kbo(:, 2,58,13) = (/ &
     &1.6360e+01_rb,1.6224e+01_rb,2.0957e+01_rb,3.0182e+01_rb,2.8583e+01_rb/)
      kbo(:, 3,58,13) = (/ &
     &3.5957e+01_rb,3.3123e+01_rb,3.1853e+01_rb,3.6291e+01_rb,2.9233e+01_rb/)
      kbo(:, 4,58,13) = (/ &
     &7.1076e+01_rb,6.5212e+01_rb,5.6746e+01_rb,4.8699e+01_rb,2.9472e+01_rb/)
      kbo(:, 5,58,13) = (/ &
     &1.2897e+02_rb,1.1828e+02_rb,1.0149e+02_rb,7.4519e+01_rb,2.9400e+01_rb/)
      kbo(:, 1,59,13) = (/ &
     &6.7458e+00_rb,1.2021e+01_rb,2.0178e+01_rb,2.9391e+01_rb,2.7237e+01_rb/)
      kbo(:, 2,59,13) = (/ &
     &1.7109e+01_rb,1.8183e+01_rb,2.5274e+01_rb,3.4440e+01_rb,2.8519e+01_rb/)
      kbo(:, 3,59,13) = (/ &
     &3.8233e+01_rb,3.4158e+01_rb,3.4619e+01_rb,4.0204e+01_rb,2.9240e+01_rb/)
      kbo(:, 4,59,13) = (/ &
     &7.6647e+01_rb,6.6998e+01_rb,5.6073e+01_rb,4.9489e+01_rb,2.9509e+01_rb/)
      kbo(:, 5,59,13) = (/ &
     &1.4043e+02_rb,1.2268e+02_rb,9.8230e+01_rb,6.8889e+01_rb,2.9402e+01_rb/)
      kbo(:, 1,13,14) = (/ &
     &2.1776e+03_rb,1.6333e+03_rb,1.0891e+03_rb,5.4478e+02_rb,7.3262e+00_rb/)
      kbo(:, 2,13,14) = (/ &
     &3.1101e+03_rb,2.3327e+03_rb,1.5553e+03_rb,7.7788e+02_rb,7.3124e+00_rb/)
      kbo(:, 3,13,14) = (/ &
     &4.2249e+03_rb,3.1687e+03_rb,2.1126e+03_rb,1.0565e+03_rb,7.2480e+00_rb/)
      kbo(:, 4,13,14) = (/ &
     &5.5118e+03_rb,4.1339e+03_rb,2.7561e+03_rb,1.3783e+03_rb,7.1568e+00_rb/)
      kbo(:, 5,13,14) = (/ &
     &6.9506e+03_rb,5.2131e+03_rb,3.4755e+03_rb,1.7380e+03_rb,7.0367e+00_rb/)
      kbo(:, 1,14,14) = (/ &
     &1.4496e+03_rb,1.0873e+03_rb,7.2501e+02_rb,3.6272e+02_rb,8.3339e+00_rb/)
      kbo(:, 2,14,14) = (/ &
     &2.0734e+03_rb,1.5551e+03_rb,1.0369e+03_rb,5.1863e+02_rb,8.2705e+00_rb/)
      kbo(:, 3,14,14) = (/ &
     &2.8238e+03_rb,2.1179e+03_rb,1.4121e+03_rb,7.0623e+02_rb,8.1703e+00_rb/)
      kbo(:, 4,14,14) = (/ &
     &3.6841e+03_rb,2.7632e+03_rb,1.8422e+03_rb,9.2131e+02_rb,8.0387e+00_rb/)
      kbo(:, 5,14,14) = (/ &
     &4.6457e+03_rb,3.4843e+03_rb,2.3230e+03_rb,1.1617e+03_rb,7.8896e+00_rb/)
      kbo(:, 1,15,14) = (/ &
     &9.9772e+02_rb,7.4839e+02_rb,4.9906e+02_rb,2.4972e+02_rb,9.4242e+00_rb/)
      kbo(:, 2,15,14) = (/ &
     &1.4271e+03_rb,1.0704e+03_rb,7.1374e+02_rb,3.5707e+02_rb,9.3124e+00_rb/)
      kbo(:, 3,15,14) = (/ &
     &1.9364e+03_rb,1.4524e+03_rb,9.6841e+02_rb,4.8440e+02_rb,9.1590e+00_rb/)
      kbo(:, 4,15,14) = (/ &
     &2.5268e+03_rb,1.8952e+03_rb,1.2636e+03_rb,6.3198e+02_rb,8.9837e+00_rb/)
      kbo(:, 5,15,14) = (/ &
     &3.1888e+03_rb,2.3917e+03_rb,1.5946e+03_rb,7.9748e+02_rb,8.7913e+00_rb/)
      kbo(:, 1,16,14) = (/ &
     &8.2535e+02_rb,6.1910e+02_rb,4.1286e+02_rb,2.0662e+02_rb,1.0617e+01_rb/)
      kbo(:, 2,16,14) = (/ &
     &1.1765e+03_rb,8.8250e+02_rb,5.8845e+02_rb,2.9441e+02_rb,1.0441e+01_rb/)
      kbo(:, 3,16,14) = (/ &
     &1.5988e+03_rb,1.1992e+03_rb,7.9956e+02_rb,3.9995e+02_rb,1.0240e+01_rb/)
      kbo(:, 4,16,14) = (/ &
     &2.0853e+03_rb,1.5640e+03_rb,1.0428e+03_rb,5.2160e+02_rb,1.0022e+01_rb/)
      kbo(:, 5,16,14) = (/ &
     &2.6260e+03_rb,1.9696e+03_rb,1.3132e+03_rb,6.5679e+02_rb,9.7853e+00_rb/)
      kbo(:, 1,17,14) = (/ &
     &7.1446e+02_rb,5.3593e+02_rb,3.5740e+02_rb,1.7888e+02_rb,1.1902e+01_rb/)
      kbo(:, 2,17,14) = (/ &
     &1.0170e+03_rb,7.6281e+02_rb,5.0866e+02_rb,2.5450e+02_rb,1.1678e+01_rb/)
      kbo(:, 3,17,14) = (/ &
     &1.3782e+03_rb,1.0338e+03_rb,6.8929e+02_rb,3.4482e+02_rb,1.1434e+01_rb/)
      kbo(:, 4,17,14) = (/ &
     &1.7913e+03_rb,1.3436e+03_rb,8.9585e+02_rb,4.4810e+02_rb,1.1160e+01_rb/)
      kbo(:, 5,17,14) = (/ &
     &2.2531e+03_rb,1.6899e+03_rb,1.1267e+03_rb,5.6354e+02_rb,1.0872e+01_rb/)
      kbo(:, 1,18,14) = (/ &
     &6.3627e+02_rb,4.7728e+02_rb,3.1830e+02_rb,1.5932e+02_rb,1.3306e+01_rb/)
      kbo(:, 2,18,14) = (/ &
     &9.0279e+02_rb,6.7718e+02_rb,4.5157e+02_rb,2.2596e+02_rb,1.3021e+01_rb/)
      kbo(:, 3,18,14) = (/ &
     &1.2176e+03_rb,9.1326e+02_rb,6.0896e+02_rb,3.0466e+02_rb,1.2705e+01_rb/)
      kbo(:, 4,18,14) = (/ &
     &1.5793e+03_rb,1.1846e+03_rb,7.8983e+02_rb,3.9509e+02_rb,1.2374e+01_rb/)
      kbo(:, 5,18,14) = (/ &
     &1.9821e+03_rb,1.4867e+03_rb,9.9125e+02_rb,4.9580e+02_rb,1.2038e+01_rb/)
      kbo(:, 1,19,14) = (/ &
     &5.5701e+02_rb,4.1784e+02_rb,2.7868e+02_rb,1.3951e+02_rb,1.4835e+01_rb/)
      kbo(:, 2,19,14) = (/ &
     &7.8555e+02_rb,5.8924e+02_rb,3.9294e+02_rb,1.9665e+02_rb,1.4471e+01_rb/)
      kbo(:, 3,19,14) = (/ &
     &1.0565e+03_rb,7.9246e+02_rb,5.2842e+02_rb,2.6438e+02_rb,1.4088e+01_rb/)
      kbo(:, 4,19,14) = (/ &
     &1.3666e+03_rb,1.0251e+03_rb,6.8348e+02_rb,3.4191e+02_rb,1.3700e+01_rb/)
      kbo(:, 5,19,14) = (/ &
     &1.7097e+03_rb,1.2824e+03_rb,8.5504e+02_rb,4.2769e+02_rb,1.3330e+01_rb/)
      kbo(:, 1,20,14) = (/ &
     &5.2902e+02_rb,3.9685e+02_rb,2.6468e+02_rb,1.3251e+02_rb,1.6461e+01_rb/)
      kbo(:, 2,20,14) = (/ &
     &7.4221e+02_rb,5.5674e+02_rb,3.7128e+02_rb,1.8581e+02_rb,1.6028e+01_rb/)
      kbo(:, 3,20,14) = (/ &
     &9.9413e+02_rb,7.4568e+02_rb,4.9723e+02_rb,2.4879e+02_rb,1.5588e+01_rb/)
      kbo(:, 4,20,14) = (/ &
     &1.2802e+03_rb,9.6022e+02_rb,6.4026e+02_rb,3.2030e+02_rb,1.5170e+01_rb/)
      kbo(:, 5,20,14) = (/ &
     &1.5959e+03_rb,1.1970e+03_rb,7.9814e+02_rb,3.9924e+02_rb,1.4766e+01_rb/)
      kbo(:, 1,21,14) = (/ &
     &5.1643e+02_rb,3.8740e+02_rb,2.5838e+02_rb,1.2935e+02_rb,1.8266e+01_rb/)
      kbo(:, 2,21,14) = (/ &
     &7.2079e+02_rb,5.4067e+02_rb,3.6056e+02_rb,1.8045e+02_rb,1.7763e+01_rb/)
      kbo(:, 3,21,14) = (/ &
     &9.5965e+02_rb,7.1982e+02_rb,4.7999e+02_rb,2.4017e+02_rb,1.7272e+01_rb/)
      kbo(:, 4,21,14) = (/ &
     &1.2302e+03_rb,9.2274e+02_rb,6.1527e+02_rb,3.0781e+02_rb,1.6803e+01_rb/)
      kbo(:, 5,21,14) = (/ &
     &1.5280e+03_rb,1.1461e+03_rb,7.6416e+02_rb,3.8225e+02_rb,1.6348e+01_rb/)
      kbo(:, 1,22,14) = (/ &
     &5.2623e+02_rb,3.9475e+02_rb,2.6327e+02_rb,1.3179e+02_rb,2.0243e+01_rb/)
      kbo(:, 2,22,14) = (/ &
     &7.2644e+02_rb,5.4491e+02_rb,3.6339e+02_rb,1.8186e+02_rb,1.9664e+01_rb/)
      kbo(:, 3,22,14) = (/ &
     &9.5925e+02_rb,7.1952e+02_rb,4.7979e+02_rb,2.4006e+02_rb,1.9116e+01_rb/)
      kbo(:, 4,22,14) = (/ &
     &1.2211e+03_rb,9.1590e+02_rb,6.1071e+02_rb,3.0553e+02_rb,1.8593e+01_rb/)
      kbo(:, 5,22,14) = (/ &
     &1.5078e+03_rb,1.1310e+03_rb,7.5409e+02_rb,3.7722e+02_rb,1.8080e+01_rb/)
      kbo(:, 1,23,14) = (/ &
     &5.3158e+02_rb,3.9876e+02_rb,2.6595e+02_rb,1.3313e+02_rb,2.2446e+01_rb/)
      kbo(:, 2,23,14) = (/ &
     &7.2667e+02_rb,5.4508e+02_rb,3.6349e+02_rb,1.8191e+02_rb,2.1789e+01_rb/)
      kbo(:, 3,23,14) = (/ &
     &9.5159e+02_rb,7.1378e+02_rb,4.7596e+02_rb,2.3814e+02_rb,2.1156e+01_rb/)
      kbo(:, 4,23,14) = (/ &
     &1.2042e+03_rb,9.0323e+02_rb,6.0227e+02_rb,3.0131e+02_rb,2.0541e+01_rb/)
      kbo(:, 5,23,14) = (/ &
     &1.4791e+03_rb,1.1094e+03_rb,7.3972e+02_rb,3.7004e+02_rb,1.9936e+01_rb/)
      kbo(:, 1,24,14) = (/ &
     &5.2674e+02_rb,3.9513e+02_rb,2.6351e+02_rb,1.3189e+02_rb,2.4861e+01_rb/)
      kbo(:, 2,24,14) = (/ &
     &7.1320e+02_rb,5.3497e+02_rb,3.5675e+02_rb,1.7852e+02_rb,2.4105e+01_rb/)
      kbo(:, 3,24,14) = (/ &
     &9.2799e+02_rb,6.9608e+02_rb,4.6416e+02_rb,2.3224e+02_rb,2.3357e+01_rb/)
      kbo(:, 4,24,14) = (/ &
     &1.1676e+03_rb,8.7581e+02_rb,5.8399e+02_rb,2.9218e+02_rb,2.2617e+01_rb/)
      kbo(:, 5,24,14) = (/ &
     &1.4298e+03_rb,1.0724e+03_rb,7.1508e+02_rb,3.5774e+02_rb,2.1888e+01_rb/)
      kbo(:, 1,25,14) = (/ &
     &5.2185e+02_rb,3.9145e+02_rb,2.6105e+02_rb,1.3066e+02_rb,2.7396e+01_rb/)
      kbo(:, 2,25,14) = (/ &
     &7.0140e+02_rb,5.2612e+02_rb,3.5084e+02_rb,1.7556e+02_rb,2.6509e+01_rb/)
      kbo(:, 3,25,14) = (/ &
     &9.0713e+02_rb,6.8043e+02_rb,4.5373e+02_rb,2.2703e+02_rb,2.5618e+01_rb/)
      kbo(:, 4,25,14) = (/ &
     &1.1371e+03_rb,8.5288e+02_rb,5.6870e+02_rb,2.8453e+02_rb,2.4730e+01_rb/)
      kbo(:, 5,25,14) = (/ &
     &1.3891e+03_rb,1.0419e+03_rb,6.9474e+02_rb,3.4757e+02_rb,2.3860e+01_rb/)
      kbo(:, 1,26,14) = (/ &
     &5.2251e+02_rb,3.9195e+02_rb,2.6138e+02_rb,1.3085e+02_rb,2.9927e+01_rb/)
      kbo(:, 2,26,14) = (/ &
     &6.9730e+02_rb,5.2305e+02_rb,3.4880e+02_rb,1.7455e+02_rb,2.8877e+01_rb/)
      kbo(:, 3,26,14) = (/ &
     &8.9735e+02_rb,6.7309e+02_rb,4.4884e+02_rb,2.2459e+02_rb,2.7820e+01_rb/)
      kbo(:, 4,26,14) = (/ &
     &1.1213e+03_rb,8.4105e+02_rb,5.6081e+02_rb,2.8058e+02_rb,2.6772e+01_rb/)
      kbo(:, 5,26,14) = (/ &
     &1.3665e+03_rb,1.0250e+03_rb,6.8346e+02_rb,3.4192e+02_rb,2.5741e+01_rb/)
      kbo(:, 1,27,14) = (/ &
     &5.3834e+02_rb,4.0383e+02_rb,2.6931e+02_rb,1.3510e+02_rb,3.2357e+01_rb/)
      kbo(:, 2,27,14) = (/ &
     &7.1439e+02_rb,5.3587e+02_rb,3.5735e+02_rb,1.7883e+02_rb,3.1116e+01_rb/)
      kbo(:, 3,27,14) = (/ &
     &9.1626e+02_rb,6.8728e+02_rb,4.5830e+02_rb,2.2932e+02_rb,2.9881e+01_rb/)
      kbo(:, 4,27,14) = (/ &
     &1.1424e+03_rb,8.5692e+02_rb,5.7140e+02_rb,2.8587e+02_rb,2.8666e+01_rb/)
      kbo(:, 5,27,14) = (/ &
     &1.3901e+03_rb,1.0427e+03_rb,6.9525e+02_rb,3.4781e+02_rb,2.7467e+01_rb/)
      kbo(:, 1,28,14) = (/ &
     &5.6592e+02_rb,4.2451e+02_rb,2.8310e+02_rb,1.4237e+02_rb,3.4583e+01_rb/)
      kbo(:, 2,28,14) = (/ &
     &7.4785e+02_rb,5.6096e+02_rb,3.7408e+02_rb,1.8720e+02_rb,3.3140e+01_rb/)
      kbo(:, 3,28,14) = (/ &
     &9.5690e+02_rb,7.1776e+02_rb,4.7862e+02_rb,2.3947e+02_rb,3.1724e+01_rb/)
      kbo(:, 4,28,14) = (/ &
     &1.1915e+03_rb,8.9371e+02_rb,5.9592e+02_rb,2.9814e+02_rb,3.0340e+01_rb/)
      kbo(:, 5,28,14) = (/ &
     &1.4480e+03_rb,1.0861e+03_rb,7.2421e+02_rb,3.6229e+02_rb,2.8988e+01_rb/)
      kbo(:, 1,29,14) = (/ &
     &6.3031e+02_rb,4.7280e+02_rb,3.1529e+02_rb,1.5867e+02_rb,3.6546e+01_rb/)
      kbo(:, 2,29,14) = (/ &
     &8.3056e+02_rb,6.2300e+02_rb,4.1543e+02_rb,2.0788e+02_rb,3.4912e+01_rb/)
      kbo(:, 3,29,14) = (/ &
     &1.0608e+03_rb,7.9566e+02_rb,5.3055e+02_rb,2.6544e+02_rb,3.3320e+01_rb/)
      kbo(:, 4,29,14) = (/ &
     &1.3185e+03_rb,9.8899e+02_rb,6.5945e+02_rb,3.2990e+02_rb,3.1769e+01_rb/)
      kbo(:, 5,29,14) = (/ &
     &1.6020e+03_rb,1.2016e+03_rb,8.0116e+02_rb,4.0076e+02_rb,3.0274e+01_rb/)
      kbo(:, 1,30,14) = (/ &
     &7.1878e+02_rb,5.3916e+02_rb,3.5953e+02_rb,1.8082e+02_rb,3.8231e+01_rb/)
      kbo(:, 2,30,14) = (/ &
     &9.4502e+02_rb,7.0884e+02_rb,4.7266e+02_rb,2.3650e+02_rb,3.6421e+01_rb/)
      kbo(:, 3,30,14) = (/ &
     &1.2048e+03_rb,9.0370e+02_rb,6.0258e+02_rb,3.0145e+02_rb,3.4660e+01_rb/)
      kbo(:, 4,30,14) = (/ &
     &1.4966e+03_rb,1.1225e+03_rb,7.4846e+02_rb,3.7440e+02_rb,3.2964e+01_rb/)
      kbo(:, 5,30,14) = (/ &
     &1.8180e+03_rb,1.3636e+03_rb,9.0918e+02_rb,4.5477e+02_rb,3.1338e+01_rb/)
      kbo(:, 1,31,14) = (/ &
     &8.6352e+02_rb,6.4770e+02_rb,4.3189e+02_rb,2.1681e+02_rb,3.9626e+01_rb/)
      kbo(:, 2,31,14) = (/ &
     &1.1338e+03_rb,8.5039e+02_rb,5.6702e+02_rb,2.8366e+02_rb,3.7649e+01_rb/)
      kbo(:, 3,31,14) = (/ &
     &1.4436e+03_rb,1.0827e+03_rb,7.2193e+02_rb,3.6112e+02_rb,3.5738e+01_rb/)
      kbo(:, 4,31,14) = (/ &
     &1.7919e+03_rb,1.3440e+03_rb,8.9611e+02_rb,4.4823e+02_rb,3.3911e+01_rb/)
      kbo(:, 5,31,14) = (/ &
     &2.1766e+03_rb,1.6325e+03_rb,1.0885e+03_rb,5.4442e+02_rb,3.2168e+01_rb/)
      kbo(:, 1,32,14) = (/ &
     &1.0534e+03_rb,7.9009e+02_rb,5.2682e+02_rb,2.6397e+02_rb,4.0725e+01_rb/)
      kbo(:, 2,32,14) = (/ &
     &1.3802e+03_rb,1.0352e+03_rb,6.9024e+02_rb,3.4527e+02_rb,3.8603e+01_rb/)
      kbo(:, 3,32,14) = (/ &
     &1.7557e+03_rb,1.3169e+03_rb,8.7802e+02_rb,4.3918e+02_rb,3.6565e+01_rb/)
      kbo(:, 4,32,14) = (/ &
     &2.1789e+03_rb,1.6342e+03_rb,1.0896e+03_rb,5.4499e+02_rb,3.4625e+01_rb/)
      kbo(:, 5,32,14) = (/ &
     &2.6446e+03_rb,1.9836e+03_rb,1.3225e+03_rb,6.6145e+02_rb,3.2784e+01_rb/)
      kbo(:, 1,33,14) = (/ &
     &1.3124e+03_rb,9.8436e+02_rb,6.5633e+02_rb,3.2843e+02_rb,4.1548e+01_rb/)
      kbo(:, 2,33,14) = (/ &
     &1.7177e+03_rb,1.2883e+03_rb,8.5900e+02_rb,4.2966e+02_rb,3.9303e+01_rb/)
      kbo(:, 3,33,14) = (/ &
     &2.1832e+03_rb,1.6375e+03_rb,1.0918e+03_rb,5.4605e+02_rb,3.7160e+01_rb/)
      kbo(:, 4,33,14) = (/ &
     &2.7063e+03_rb,2.0298e+03_rb,1.3533e+03_rb,6.7686e+02_rb,3.5130e+01_rb/)
      kbo(:, 5,33,14) = (/ &
     &3.2806e+03_rb,2.4606e+03_rb,1.6405e+03_rb,8.2046e+02_rb,3.3063e+01_rb/)
      kbo(:, 1,34,14) = (/ &
     &1.5860e+03_rb,1.1896e+03_rb,7.9315e+02_rb,3.9672e+02_rb,4.2202e+01_rb/)
      kbo(:, 2,34,14) = (/ &
     &2.0745e+03_rb,1.5559e+03_rb,1.0374e+03_rb,5.1886e+02_rb,3.9853e+01_rb/)
      kbo(:, 3,34,14) = (/ &
     &2.6355e+03_rb,1.9767e+03_rb,1.3179e+03_rb,6.5913e+02_rb,3.7623e+01_rb/)
      kbo(:, 4,34,14) = (/ &
     &3.2644e+03_rb,2.4484e+03_rb,1.6324e+03_rb,8.1637e+02_rb,3.5405e+01_rb/)
      kbo(:, 5,34,14) = (/ &
     &3.9520e+03_rb,2.9641e+03_rb,1.9762e+03_rb,9.8832e+02_rb,3.3276e+01_rb/)
      kbo(:, 1,35,14) = (/ &
     &1.8756e+03_rb,1.4068e+03_rb,9.3797e+02_rb,4.6914e+02_rb,4.2878e+01_rb/)
      kbo(:, 2,35,14) = (/ &
     &2.4578e+03_rb,1.8434e+03_rb,1.2290e+03_rb,6.1469e+02_rb,4.0440e+01_rb/)
      kbo(:, 3,35,14) = (/ &
     &3.1268e+03_rb,2.3452e+03_rb,1.5636e+03_rb,7.8197e+02_rb,3.8129e+01_rb/)
      kbo(:, 4,35,14) = (/ &
     &3.8751e+03_rb,2.9064e+03_rb,1.9378e+03_rb,9.6909e+02_rb,3.5715e+01_rb/)
      kbo(:, 5,35,14) = (/ &
     &4.6936e+03_rb,3.5203e+03_rb,2.3471e+03_rb,1.1738e+03_rb,3.3797e+01_rb/)
      kbo(:, 1,36,14) = (/ &
     &2.1453e+03_rb,1.6090e+03_rb,1.0728e+03_rb,5.3655e+02_rb,4.3664e+01_rb/)
      kbo(:, 2,36,14) = (/ &
     &2.8245e+03_rb,2.1184e+03_rb,1.4124e+03_rb,7.0636e+02_rb,4.1138e+01_rb/)
      kbo(:, 3,36,14) = (/ &
     &3.6063e+03_rb,2.7048e+03_rb,1.8033e+03_rb,9.0186e+02_rb,3.8589e+01_rb/)
      kbo(:, 4,36,14) = (/ &
     &4.4821e+03_rb,3.3616e+03_rb,2.2412e+03_rb,1.1208e+03_rb,3.6195e+01_rb/)
      kbo(:, 5,36,14) = (/ &
     &5.4418e+03_rb,4.0814e+03_rb,2.7211e+03_rb,1.3608e+03_rb,3.4367e+01_rb/)
      kbo(:, 1,37,14) = (/ &
     &2.2940e+03_rb,1.7205e+03_rb,1.1471e+03_rb,5.7372e+02_rb,4.4724e+01_rb/)
      kbo(:, 2,37,14) = (/ &
     &3.0476e+03_rb,2.2858e+03_rb,1.5240e+03_rb,7.6217e+02_rb,4.2103e+01_rb/)
      kbo(:, 3,37,14) = (/ &
     &3.9216e+03_rb,2.9413e+03_rb,1.9610e+03_rb,9.8070e+02_rb,3.9415e+01_rb/)
      kbo(:, 4,37,14) = (/ &
     &4.9050e+03_rb,3.6789e+03_rb,2.4527e+03_rb,1.2266e+03_rb,3.7136e+01_rb/)
      kbo(:, 5,37,14) = (/ &
     &5.9876e+03_rb,4.4908e+03_rb,2.9940e+03_rb,1.4972e+03_rb,3.5121e+01_rb/)
      kbo(:, 1,38,14) = (/ &
     &2.4855e+03_rb,1.8642e+03_rb,1.2429e+03_rb,6.2161e+02_rb,4.5721e+01_rb/)
      kbo(:, 2,38,14) = (/ &
     &3.3318e+03_rb,2.4989e+03_rb,1.6661e+03_rb,8.3322e+02_rb,4.3010e+01_rb/)
      kbo(:, 3,38,14) = (/ &
     &4.3213e+03_rb,3.2410e+03_rb,2.1608e+03_rb,1.0806e+03_rb,4.0195e+01_rb/)
      kbo(:, 4,38,14) = (/ &
     &5.4403e+03_rb,4.0803e+03_rb,2.7203e+03_rb,1.3604e+03_rb,3.7939e+01_rb/)
      kbo(:, 5,38,14) = (/ &
     &6.6754e+03_rb,5.0066e+03_rb,3.3379e+03_rb,1.6692e+03_rb,3.5824e+01_rb/)
      kbo(:, 1,39,14) = (/ &
     &2.7579e+03_rb,2.0685e+03_rb,1.3791e+03_rb,6.8973e+02_rb,4.6636e+01_rb/)
      kbo(:, 2,39,14) = (/ &
     &3.7311e+03_rb,2.7984e+03_rb,1.8657e+03_rb,9.3308e+02_rb,4.3764e+01_rb/)
      kbo(:, 3,39,14) = (/ &
     &4.8740e+03_rb,3.6556e+03_rb,2.4372e+03_rb,1.2188e+03_rb,4.0901e+01_rb/)
      kbo(:, 4,39,14) = (/ &
     &6.1755e+03_rb,4.6318e+03_rb,3.0880e+03_rb,1.5442e+03_rb,3.8692e+01_rb/)
      kbo(:, 5,39,14) = (/ &
     &7.6168e+03_rb,5.7128e+03_rb,3.8087e+03_rb,1.9046e+03_rb,3.6244e+01_rb/)
      kbo(:, 1,40,14) = (/ &
     &2.7806e+03_rb,2.0855e+03_rb,1.3905e+03_rb,6.9541e+02_rb,4.7781e+01_rb/)
      kbo(:, 2,40,14) = (/ &
     &3.8105e+03_rb,2.8580e+03_rb,1.9054e+03_rb,9.5291e+02_rb,4.4820e+01_rb/)
      kbo(:, 3,40,14) = (/ &
     &5.0317e+03_rb,3.7739e+03_rb,2.5160e+03_rb,1.2582e+03_rb,4.1870e+01_rb/)
      kbo(:, 4,40,14) = (/ &
     &6.4348e+03_rb,4.8262e+03_rb,3.2176e+03_rb,1.6090e+03_rb,3.9597e+01_rb/)
      kbo(:, 5,40,14) = (/ &
     &7.9965e+03_rb,5.9975e+03_rb,3.9985e+03_rb,1.9995e+03_rb,3.7061e+01_rb/)
      kbo(:, 1,41,14) = (/ &
     &2.7787e+03_rb,2.0841e+03_rb,1.3895e+03_rb,6.9500e+02_rb,4.8925e+01_rb/)
      kbo(:, 2,41,14) = (/ &
     &3.8614e+03_rb,2.8961e+03_rb,1.9309e+03_rb,9.6566e+02_rb,4.5922e+01_rb/)
      kbo(:, 3,41,14) = (/ &
     &5.1574e+03_rb,3.8681e+03_rb,2.5789e+03_rb,1.2897e+03_rb,4.2853e+01_rb/)
      kbo(:, 4,41,14) = (/ &
     &6.6587e+03_rb,4.9941e+03_rb,3.3296e+03_rb,1.6650e+03_rb,4.0428e+01_rb/)
      kbo(:, 5,41,14) = (/ &
     &8.3439e+03_rb,6.2581e+03_rb,4.1723e+03_rb,2.0864e+03_rb,3.7900e+01_rb/)
      kbo(:, 1,42,14) = (/ &
     &2.7998e+03_rb,2.0999e+03_rb,1.4001e+03_rb,7.0041e+02_rb,5.0031e+01_rb/)
      kbo(:, 2,42,14) = (/ &
     &3.9460e+03_rb,2.9596e+03_rb,1.9732e+03_rb,9.8680e+02_rb,4.6998e+01_rb/)
      kbo(:, 3,42,14) = (/ &
     &5.3323e+03_rb,3.9993e+03_rb,2.6663e+03_rb,1.3334e+03_rb,4.3819e+01_rb/)
      kbo(:, 4,42,14) = (/ &
     &6.9531e+03_rb,5.2149e+03_rb,3.4767e+03_rb,1.7386e+03_rb,4.1253e+01_rb/)
      kbo(:, 5,42,14) = (/ &
     &8.7873e+03_rb,6.5906e+03_rb,4.3939e+03_rb,2.1972e+03_rb,3.8745e+01_rb/)
      kbo(:, 1,43,14) = (/ &
     &2.7784e+03_rb,2.0839e+03_rb,1.3894e+03_rb,6.9544e+02_rb,5.1292e+01_rb/)
      kbo(:, 2,43,14) = (/ &
     &3.9830e+03_rb,2.9874e+03_rb,1.9917e+03_rb,9.9606e+02_rb,4.8177e+01_rb/)
      kbo(:, 3,43,14) = (/ &
     &5.4603e+03_rb,4.0953e+03_rb,2.7304e+03_rb,1.3654e+03_rb,4.4925e+01_rb/)
      kbo(:, 4,43,14) = (/ &
     &7.2065e+03_rb,5.4050e+03_rb,3.6035e+03_rb,1.8020e+03_rb,4.2242e+01_rb/)
      kbo(:, 5,43,14) = (/ &
     &9.2025e+03_rb,6.9020e+03_rb,4.6015e+03_rb,2.3010e+03_rb,3.9649e+01_rb/)
      kbo(:, 1,44,14) = (/ &
     &2.7542e+03_rb,2.0657e+03_rb,1.3772e+03_rb,6.8977e+02_rb,5.2617e+01_rb/)
      kbo(:, 2,44,14) = (/ &
     &4.0230e+03_rb,3.0173e+03_rb,2.0117e+03_rb,1.0060e+03_rb,4.9427e+01_rb/)
      kbo(:, 3,44,14) = (/ &
     &5.6050e+03_rb,4.2038e+03_rb,2.8027e+03_rb,1.4015e+03_rb,4.6077e+01_rb/)
      kbo(:, 4,44,14) = (/ &
     &7.4959e+03_rb,5.6220e+03_rb,3.7481e+03_rb,1.8743e+03_rb,4.3276e+01_rb/)
      kbo(:, 5,44,14) = (/ &
     &9.6875e+03_rb,7.2657e+03_rb,4.8439e+03_rb,2.4222e+03_rb,4.0647e+01_rb/)
      kbo(:, 1,45,14) = (/ &
     &2.7680e+03_rb,2.0761e+03_rb,1.3842e+03_rb,6.9371e+02_rb,5.3927e+01_rb/)
      kbo(:, 2,45,14) = (/ &
     &4.1247e+03_rb,3.0936e+03_rb,2.0625e+03_rb,1.0314e+03_rb,5.0672e+01_rb/)
      kbo(:, 3,45,14) = (/ &
     &5.8448e+03_rb,4.3837e+03_rb,2.9226e+03_rb,1.4615e+03_rb,4.7150e+01_rb/)
      kbo(:, 4,45,14) = (/ &
     &7.9284e+03_rb,5.9463e+03_rb,3.9644e+03_rb,1.9824e+03_rb,4.4329e+01_rb/)
      kbo(:, 5,45,14) = (/ &
     &1.0369e+04_rb,7.7768e+03_rb,5.1847e+03_rb,2.5926e+03_rb,4.1634e+01_rb/)
      kbo(:, 1,46,14) = (/ &
     &2.7642e+03_rb,2.0732e+03_rb,1.3822e+03_rb,6.9344e+02_rb,5.5299e+01_rb/)
      kbo(:, 2,46,14) = (/ &
     &4.2127e+03_rb,3.1596e+03_rb,2.1065e+03_rb,1.0534e+03_rb,5.1989e+01_rb/)
      kbo(:, 3,46,14) = (/ &
     &6.0817e+03_rb,4.5614e+03_rb,3.0411e+03_rb,1.5207e+03_rb,4.8362e+01_rb/)
      kbo(:, 4,46,14) = (/ &
     &8.3835e+03_rb,6.2877e+03_rb,4.1920e+03_rb,2.0962e+03_rb,4.5451e+01_rb/)
      kbo(:, 5,46,14) = (/ &
     &1.1109e+04_rb,8.3315e+03_rb,5.5545e+03_rb,2.7775e+03_rb,4.2710e+01_rb/)
      kbo(:, 1,47,14) = (/ &
     &2.6579e+03_rb,1.9935e+03_rb,1.3291e+03_rb,6.6810e+02_rb,5.6805e+01_rb/)
      kbo(:, 2,47,14) = (/ &
     &4.1618e+03_rb,3.1214e+03_rb,2.0811e+03_rb,1.0407e+03_rb,5.3447e+01_rb/)
      kbo(:, 3,47,14) = (/ &
     &6.1402e+03_rb,4.6053e+03_rb,3.0703e+03_rb,1.5354e+03_rb,4.9761e+01_rb/)
      kbo(:, 4,47,14) = (/ &
     &8.6262e+03_rb,6.4697e+03_rb,4.3133e+03_rb,2.1569e+03_rb,4.6695e+01_rb/)
      kbo(:, 5,47,14) = (/ &
     &1.1609e+04_rb,8.7071e+03_rb,5.8049e+03_rb,2.9027e+03_rb,4.4026e+01_rb/)
      kbo(:, 1,48,14) = (/ &
     &2.5983e+03_rb,1.9488e+03_rb,1.2993e+03_rb,6.5451e+02_rb,5.8316e+01_rb/)
      kbo(:, 2,48,14) = (/ &
     &4.1890e+03_rb,3.1418e+03_rb,2.0947e+03_rb,1.0476e+03_rb,5.4906e+01_rb/)
      kbo(:, 3,48,14) = (/ &
     &6.3280e+03_rb,4.7461e+03_rb,3.1642e+03_rb,1.5823e+03_rb,5.1153e+01_rb/)
      kbo(:, 4,48,14) = (/ &
     &9.0666e+03_rb,6.8001e+03_rb,4.5335e+03_rb,2.2670e+03_rb,4.7961e+01_rb/)
      kbo(:, 5,48,14) = (/ &
     &1.2413e+04_rb,9.3096e+03_rb,6.2066e+03_rb,3.1035e+03_rb,4.5348e+01_rb/)
      kbo(:, 1,49,14) = (/ &
     &2.6146e+03_rb,1.9610e+03_rb,1.3074e+03_rb,6.5952e+02_rb,5.9823e+01_rb/)
      kbo(:, 2,49,14) = (/ &
     &4.3479e+03_rb,3.2610e+03_rb,2.1741e+03_rb,1.0874e+03_rb,5.6229e+01_rb/)
      kbo(:, 3,49,14) = (/ &
     &6.7401e+03_rb,5.0552e+03_rb,3.3702e+03_rb,1.6853e+03_rb,5.2522e+01_rb/)
      kbo(:, 4,49,14) = (/ &
     &9.8602e+03_rb,7.3953e+03_rb,4.9303e+03_rb,2.4653e+03_rb,4.9326e+01_rb/)
      kbo(:, 5,49,14) = (/ &
     &1.3745e+04_rb,1.0309e+04_rb,6.8727e+03_rb,3.4366e+03_rb,4.6702e+01_rb/)
      kbo(:, 1,50,14) = (/ &
     &2.5099e+03_rb,1.8825e+03_rb,1.2551e+03_rb,6.3476e+02_rb,6.1230e+01_rb/)
      kbo(:, 2,50,14) = (/ &
     &4.3054e+03_rb,3.2291e+03_rb,2.1528e+03_rb,1.0772e+03_rb,5.7584e+01_rb/)
      kbo(:, 3,50,14) = (/ &
     &6.8503e+03_rb,5.1378e+03_rb,3.4253e+03_rb,1.7128e+03_rb,5.3876e+01_rb/)
      kbo(:, 4,50,14) = (/ &
     &1.0235e+04_rb,7.6760e+03_rb,5.1175e+03_rb,2.5589e+03_rb,5.0609e+01_rb/)
      kbo(:, 5,50,14) = (/ &
     &1.4527e+04_rb,1.0895e+04_rb,7.2636e+03_rb,3.6320e+03_rb,4.7904e+01_rb/)
      kbo(:, 1,51,14) = (/ &
     &2.3421e+03_rb,1.7566e+03_rb,1.1712e+03_rb,5.9456e+02_rb,6.2588e+01_rb/)
      kbo(:, 2,51,14) = (/ &
     &4.1517e+03_rb,3.1138e+03_rb,2.0760e+03_rb,1.0393e+03_rb,5.8927e+01_rb/)
      kbo(:, 3,51,14) = (/ &
     &6.7861e+03_rb,5.0897e+03_rb,3.3932e+03_rb,1.6968e+03_rb,5.5331e+01_rb/)
      kbo(:, 4,51,14) = (/ &
     &1.0366e+04_rb,7.7746e+03_rb,5.1832e+03_rb,2.5918e+03_rb,5.1728e+01_rb/)
      kbo(:, 5,51,14) = (/ &
     &1.4983e+04_rb,1.1238e+04_rb,7.4919e+03_rb,3.7462e+03_rb,4.9071e+01_rb/)
      kbo(:, 1,52,14) = (/ &
     &2.2192e+03_rb,1.6645e+03_rb,1.1098e+03_rb,5.6550e+02_rb,6.3929e+01_rb/)
      kbo(:, 2,52,14) = (/ &
     &4.0774e+03_rb,3.0581e+03_rb,2.0388e+03_rb,1.0215e+03_rb,6.0266e+01_rb/)
      kbo(:, 3,52,14) = (/ &
     &6.8574e+03_rb,5.1432e+03_rb,3.4289e+03_rb,1.7146e+03_rb,5.6630e+01_rb/)
      kbo(:, 4,52,14) = (/ &
     &1.0728e+04_rb,8.0459e+03_rb,5.3641e+03_rb,2.6822e+03_rb,5.3054e+01_rb/)
      kbo(:, 5,52,14) = (/ &
     &1.5806e+04_rb,1.1855e+04_rb,7.9035e+03_rb,3.9520e+03_rb,5.0321e+01_rb/)
      kbo(:, 1,53,14) = (/ &
     &2.1588e+03_rb,1.6192e+03_rb,1.0797e+03_rb,5.5175e+02_rb,6.5259e+01_rb/)
      kbo(:, 2,53,14) = (/ &
     &4.1225e+03_rb,3.0919e+03_rb,2.0614e+03_rb,1.0332e+03_rb,6.1546e+01_rb/)
      kbo(:, 3,53,14) = (/ &
     &7.1492e+03_rb,5.3620e+03_rb,3.5747e+03_rb,1.7875e+03_rb,5.7918e+01_rb/)
      kbo(:, 4,53,14) = (/ &
     &1.1475e+04_rb,8.6063e+03_rb,5.7376e+03_rb,2.8690e+03_rb,5.4409e+01_rb/)
      kbo(:, 5,53,14) = (/ &
     &1.7262e+04_rb,1.2946e+04_rb,8.6311e+03_rb,4.3158e+03_rb,5.1571e+01_rb/)
      kbo(:, 1,54,14) = (/ &
     &1.8282e+03_rb,1.3713e+03_rb,9.1520e+02_rb,4.7290e+02_rb,6.6448e+01_rb/)
      kbo(:, 2,54,14) = (/ &
     &3.6279e+03_rb,2.7210e+03_rb,1.8141e+03_rb,9.1135e+02_rb,6.2844e+01_rb/)
      kbo(:, 3,54,14) = (/ &
     &6.4841e+03_rb,4.8631e+03_rb,3.2422e+03_rb,1.6212e+03_rb,5.9155e+01_rb/)
      kbo(:, 4,54,14) = (/ &
     &1.0670e+04_rb,8.0025e+03_rb,5.3351e+03_rb,2.6677e+03_rb,5.5552e+01_rb/)
      kbo(:, 5,54,14) = (/ &
     &1.6384e+04_rb,1.2288e+04_rb,8.1922e+03_rb,4.0963e+03_rb,5.2636e+01_rb/)
      kbo(:, 1,55,14) = (/ &
     &1.4067e+03_rb,1.0551e+03_rb,7.0638e+02_rb,3.7354e+02_rb,6.7558e+01_rb/)
      kbo(:, 2,55,14) = (/ &
     &2.9041e+03_rb,2.1781e+03_rb,1.4522e+03_rb,7.3353e+02_rb,6.4148e+01_rb/)
      kbo(:, 3,55,14) = (/ &
     &5.3593e+03_rb,4.0195e+03_rb,2.6797e+03_rb,1.3404e+03_rb,6.0390e+01_rb/)
      kbo(:, 4,55,14) = (/ &
     &9.0439e+03_rb,6.7830e+03_rb,4.5221e+03_rb,2.2612e+03_rb,5.6702e+01_rb/)
      kbo(:, 5,55,14) = (/ &
     &1.4184e+04_rb,1.0638e+04_rb,7.0920e+03_rb,3.5462e+03_rb,5.3319e+01_rb/)
      kbo(:, 1,56,14) = (/ &
     &1.0676e+03_rb,8.0078e+02_rb,5.3961e+02_rb,2.9566e+02_rb,6.8630e+01_rb/)
      kbo(:, 2,56,14) = (/ &
     &2.3009e+03_rb,1.7258e+03_rb,1.1507e+03_rb,5.8655e+02_rb,6.5294e+01_rb/)
      kbo(:, 3,56,14) = (/ &
     &4.3913e+03_rb,3.2935e+03_rb,2.1958e+03_rb,1.1000e+03_rb,6.1700e+01_rb/)
      kbo(:, 4,56,14) = (/ &
     &7.6108e+03_rb,5.7081e+03_rb,3.8055e+03_rb,1.9028e+03_rb,5.7982e+01_rb/)
      kbo(:, 5,56,14) = (/ &
     &1.2207e+04_rb,9.1554e+03_rb,6.1037e+03_rb,3.0520e+03_rb,5.4269e+01_rb/)
      kbo(:, 1,57,14) = (/ &
     &7.9773e+02_rb,5.9861e+02_rb,4.0826e+02_rb,2.3574e+02_rb,6.9656e+01_rb/)
      kbo(:, 2,57,14) = (/ &
     &1.8005e+03_rb,1.3505e+03_rb,9.0146e+02_rb,4.6638e+02_rb,6.6420e+01_rb/)
      kbo(:, 3,57,14) = (/ &
     &3.5629e+03_rb,2.6722e+03_rb,1.7816e+03_rb,8.9534e+02_rb,6.2948e+01_rb/)
      kbo(:, 4,57,14) = (/ &
     &6.3553e+03_rb,4.7665e+03_rb,3.1777e+03_rb,1.5890e+03_rb,5.9228e+01_rb/)
      kbo(:, 5,57,14) = (/ &
     &1.0437e+04_rb,7.8281e+03_rb,5.2188e+03_rb,2.6095e+03_rb,5.5502e+01_rb/)
      kbo(:, 1,58,14) = (/ &
     &1.5691e+02_rb,1.4430e+02_rb,1.2945e+02_rb,1.1296e+02_rb,7.0583e+01_rb/)
      kbo(:, 2,58,14) = (/ &
     &3.7146e+02_rb,3.4061e+02_rb,2.9343e+02_rb,2.1801e+02_rb,6.7460e+01_rb/)
      kbo(:, 3,58,14) = (/ &
     &7.6282e+02_rb,6.9942e+02_rb,5.9974e+02_rb,4.2478e+02_rb,6.4030e+01_rb/)
      kbo(:, 4,58,14) = (/ &
     &1.4014e+03_rb,1.2849e+03_rb,1.1017e+03_rb,7.7206e+02_rb,6.0507e+01_rb/)
      kbo(:, 5,58,14) = (/ &
     &2.3564e+03_rb,2.1605e+03_rb,1.8525e+03_rb,1.2975e+03_rb,5.6760e+01_rb/)
      kbo(:, 1,59,14) = (/ &
     &1.6269e+02_rb,1.4372e+02_rb,1.2601e+02_rb,1.1040e+02_rb,7.0952e+01_rb/)
      kbo(:, 2,59,14) = (/ &
     &3.9338e+02_rb,3.4354e+02_rb,2.7833e+02_rb,1.9401e+02_rb,6.7878e+01_rb/)
      kbo(:, 3,59,14) = (/ &
     &8.2073e+02_rb,7.1668e+02_rb,5.7208e+02_rb,3.6616e+02_rb,6.4473e+01_rb/)
      kbo(:, 4,59,14) = (/ &
     &1.5267e+03_rb,1.3331e+03_rb,1.0634e+03_rb,6.6456e+02_rb,6.0953e+01_rb/)
      kbo(:, 5,59,14) = (/ &
     &2.5924e+03_rb,2.2637e+03_rb,1.8057e+03_rb,1.1237e+03_rb,5.7434e+01_rb/)
      kbo(:, 1,13,15) = (/ &
     &6.7676e+03_rb,5.0758e+03_rb,3.3840e+03_rb,1.6922e+03_rb,9.0910e+00_rb/)
      kbo(:, 2,13,15) = (/ &
     &9.5300e+03_rb,7.1475e+03_rb,4.7651e+03_rb,2.3827e+03_rb,8.9593e+00_rb/)
      kbo(:, 3,13,15) = (/ &
     &1.2768e+04_rb,9.5761e+03_rb,6.3842e+03_rb,3.1923e+03_rb,8.8860e+00_rb/)
      kbo(:, 4,13,15) = (/ &
     &1.6413e+04_rb,1.2310e+04_rb,8.2067e+03_rb,4.1035e+03_rb,8.8094e+00_rb/)
      kbo(:, 5,13,15) = (/ &
     &2.0377e+04_rb,1.5283e+04_rb,1.0189e+04_rb,5.0946e+03_rb,8.7010e+00_rb/)
      kbo(:, 1,14,15) = (/ &
     &4.6855e+03_rb,3.5142e+03_rb,2.3429e+03_rb,1.1716e+03_rb,1.0351e+01_rb/)
      kbo(:, 2,14,15) = (/ &
     &6.5777e+03_rb,4.9333e+03_rb,3.2890e+03_rb,1.6447e+03_rb,1.0245e+01_rb/)
      kbo(:, 3,14,15) = (/ &
     &8.7780e+03_rb,6.5835e+03_rb,4.3891e+03_rb,2.1947e+03_rb,1.0189e+01_rb/)
      kbo(:, 4,14,15) = (/ &
     &1.1257e+04_rb,8.4428e+03_rb,5.6286e+03_rb,2.8145e+03_rb,1.0105e+01_rb/)
      kbo(:, 5,14,15) = (/ &
     &1.3964e+04_rb,1.0473e+04_rb,6.9821e+03_rb,3.4912e+03_rb,9.9926e+00_rb/)
      kbo(:, 1,15,15) = (/ &
     &3.2854e+03_rb,2.4642e+03_rb,1.6429e+03_rb,8.2169e+02_rb,1.1907e+01_rb/)
      kbo(:, 2,15,15) = (/ &
     &4.5889e+03_rb,3.4418e+03_rb,2.2947e+03_rb,1.1475e+03_rb,1.1847e+01_rb/)
      kbo(:, 3,15,15) = (/ &
     &6.1056e+03_rb,4.5793e+03_rb,3.0530e+03_rb,1.5267e+03_rb,1.1796e+01_rb/)
      kbo(:, 4,15,15) = (/ &
     &7.8011e+03_rb,5.8510e+03_rb,3.9008e+03_rb,1.9506e+03_rb,1.1682e+01_rb/)
      kbo(:, 5,15,15) = (/ &
     &9.6352e+03_rb,7.2265e+03_rb,4.8178e+03_rb,2.4091e+03_rb,1.1548e+01_rb/)
      kbo(:, 1,16,15) = (/ &
     &2.7156e+03_rb,2.0368e+03_rb,1.3580e+03_rb,6.7918e+02_rb,1.3833e+01_rb/)
      kbo(:, 2,16,15) = (/ &
     &3.7730e+03_rb,2.8298e+03_rb,1.8867e+03_rb,9.4350e+02_rb,1.3786e+01_rb/)
      kbo(:, 3,16,15) = (/ &
     &4.9942e+03_rb,3.7457e+03_rb,2.4973e+03_rb,1.2488e+03_rb,1.3682e+01_rb/)
      kbo(:, 4,16,15) = (/ &
     &6.3607e+03_rb,4.7706e+03_rb,3.1805e+03_rb,1.5904e+03_rb,1.3533e+01_rb/)
      kbo(:, 5,16,15) = (/ &
     &7.8514e+03_rb,5.8886e+03_rb,3.9259e+03_rb,1.9631e+03_rb,1.3377e+01_rb/)
      kbo(:, 1,17,15) = (/ &
     &2.3075e+03_rb,1.7307e+03_rb,1.1539e+03_rb,5.7711e+02_rb,1.6126e+01_rb/)
      kbo(:, 2,17,15) = (/ &
     &3.1956e+03_rb,2.3967e+03_rb,1.5979e+03_rb,7.9911e+02_rb,1.6029e+01_rb/)
      kbo(:, 3,17,15) = (/ &
     &4.2241e+03_rb,3.1681e+03_rb,2.1122e+03_rb,1.0562e+03_rb,1.5863e+01_rb/)
      kbo(:, 4,17,15) = (/ &
     &5.3821e+03_rb,4.0366e+03_rb,2.6912e+03_rb,1.3457e+03_rb,1.5685e+01_rb/)
      kbo(:, 5,17,15) = (/ &
     &6.6465e+03_rb,4.9850e+03_rb,3.3234e+03_rb,1.6618e+03_rb,1.5496e+01_rb/)
      kbo(:, 1,18,15) = (/ &
     &2.0119e+03_rb,1.5090e+03_rb,1.0061e+03_rb,5.0318e+02_rb,1.8668e+01_rb/)
      kbo(:, 2,18,15) = (/ &
     &2.7815e+03_rb,2.0862e+03_rb,1.3909e+03_rb,6.9557e+02_rb,1.8505e+01_rb/)
      kbo(:, 3,18,15) = (/ &
     &3.6794e+03_rb,2.7596e+03_rb,1.8398e+03_rb,9.2005e+02_rb,1.8343e+01_rb/)
      kbo(:, 4,18,15) = (/ &
     &4.6884e+03_rb,3.5164e+03_rb,2.3443e+03_rb,1.1723e+03_rb,1.8140e+01_rb/)
      kbo(:, 5,18,15) = (/ &
     &5.7908e+03_rb,4.3432e+03_rb,2.8955e+03_rb,1.4479e+03_rb,1.7901e+01_rb/)
      kbo(:, 1,19,15) = (/ &
     &1.7281e+03_rb,1.2962e+03_rb,8.6418e+02_rb,4.3222e+02_rb,2.1496e+01_rb/)
      kbo(:, 2,19,15) = (/ &
     &2.3903e+03_rb,1.7928e+03_rb,1.1953e+03_rb,5.9777e+02_rb,2.1311e+01_rb/)
      kbo(:, 3,19,15) = (/ &
     &3.1615e+03_rb,2.3712e+03_rb,1.5809e+03_rb,7.9055e+02_rb,2.1106e+01_rb/)
      kbo(:, 4,19,15) = (/ &
     &4.0222e+03_rb,3.0167e+03_rb,2.0112e+03_rb,1.0057e+03_rb,2.0849e+01_rb/)
      kbo(:, 5,19,15) = (/ &
     &4.9621e+03_rb,3.7216e+03_rb,2.4811e+03_rb,1.2407e+03_rb,2.0531e+01_rb/)
      kbo(:, 1,20,15) = (/ &
     &1.6235e+03_rb,1.2177e+03_rb,8.1188e+02_rb,4.0606e+02_rb,2.4735e+01_rb/)
      kbo(:, 2,20,15) = (/ &
     &2.2387e+03_rb,1.6791e+03_rb,1.1195e+03_rb,5.5984e+02_rb,2.4475e+01_rb/)
      kbo(:, 3,20,15) = (/ &
     &2.9488e+03_rb,2.2116e+03_rb,1.4745e+03_rb,7.3737e+02_rb,2.4178e+01_rb/)
      kbo(:, 4,20,15) = (/ &
     &3.7459e+03_rb,2.8095e+03_rb,1.8731e+03_rb,9.3665e+02_rb,2.3818e+01_rb/)
      kbo(:, 5,20,15) = (/ &
     &4.6161e+03_rb,3.4621e+03_rb,2.3081e+03_rb,1.1542e+03_rb,2.3401e+01_rb/)
      kbo(:, 1,21,15) = (/ &
     &1.5705e+03_rb,1.1779e+03_rb,7.8536e+02_rb,3.9279e+02_rb,2.8368e+01_rb/)
      kbo(:, 2,21,15) = (/ &
     &2.1555e+03_rb,1.6166e+03_rb,1.0778e+03_rb,5.3903e+02_rb,2.7972e+01_rb/)
      kbo(:, 3,21,15) = (/ &
     &2.8340e+03_rb,2.1256e+03_rb,1.4171e+03_rb,7.0866e+02_rb,2.7543e+01_rb/)
      kbo(:, 4,21,15) = (/ &
     &3.5949e+03_rb,2.6962e+03_rb,1.7975e+03_rb,8.9888e+02_rb,2.7047e+01_rb/)
      kbo(:, 5,21,15) = (/ &
     &4.4249e+03_rb,3.3187e+03_rb,2.2126e+03_rb,1.1064e+03_rb,2.6508e+01_rb/)
      kbo(:, 1,22,15) = (/ &
     &1.5869e+03_rb,1.1902e+03_rb,7.9356e+02_rb,3.9688e+02_rb,3.2318e+01_rb/)
      kbo(:, 2,22,15) = (/ &
     &2.1673e+03_rb,1.6255e+03_rb,1.0838e+03_rb,5.4198e+02_rb,3.1767e+01_rb/)
      kbo(:, 3,22,15) = (/ &
     &2.8378e+03_rb,2.1284e+03_rb,1.4190e+03_rb,7.0961e+02_rb,3.1126e+01_rb/)
      kbo(:, 4,22,15) = (/ &
     &3.5877e+03_rb,2.6908e+03_rb,1.7940e+03_rb,8.9709e+02_rb,3.0424e+01_rb/)
      kbo(:, 5,22,15) = (/ &
     &4.4005e+03_rb,3.3004e+03_rb,2.2004e+03_rb,1.1003e+03_rb,2.9694e+01_rb/)
      kbo(:, 1,23,15) = (/ &
     &1.6232e+03_rb,1.2174e+03_rb,8.1169e+02_rb,4.0594e+02_rb,3.6426e+01_rb/)
      kbo(:, 2,23,15) = (/ &
     &2.2045e+03_rb,1.6534e+03_rb,1.1023e+03_rb,5.5127e+02_rb,3.5632e+01_rb/)
      kbo(:, 3,23,15) = (/ &
     &2.8697e+03_rb,2.1523e+03_rb,1.4350e+03_rb,7.1760e+02_rb,3.4766e+01_rb/)
      kbo(:, 4,23,15) = (/ &
     &3.6072e+03_rb,2.7054e+03_rb,1.8037e+03_rb,9.0197e+02_rb,3.3867e+01_rb/)
      kbo(:, 5,23,15) = (/ &
     &4.4070e+03_rb,3.3053e+03_rb,2.2036e+03_rb,1.1019e+03_rb,3.2942e+01_rb/)
      kbo(:, 1,24,15) = (/ &
     &1.6638e+03_rb,1.2479e+03_rb,8.3204e+02_rb,4.1615e+02_rb,4.0555e+01_rb/)
      kbo(:, 2,24,15) = (/ &
     &2.2399e+03_rb,1.6800e+03_rb,1.1201e+03_rb,5.6017e+02_rb,3.9512e+01_rb/)
      kbo(:, 3,24,15) = (/ &
     &2.8950e+03_rb,2.1713e+03_rb,1.4476e+03_rb,7.2392e+02_rb,3.8430e+01_rb/)
      kbo(:, 4,24,15) = (/ &
     &3.6214e+03_rb,2.7161e+03_rb,1.8108e+03_rb,9.0552e+02_rb,3.7312e+01_rb/)
      kbo(:, 5,24,15) = (/ &
     &4.4048e+03_rb,3.3036e+03_rb,2.2025e+03_rb,1.1014e+03_rb,3.6172e+01_rb/)
      kbo(:, 1,25,15) = (/ &
     &1.7332e+03_rb,1.2999e+03_rb,8.6673e+02_rb,4.3352e+02_rb,4.4719e+01_rb/)
      kbo(:, 2,25,15) = (/ &
     &2.3131e+03_rb,1.7349e+03_rb,1.1567e+03_rb,5.7851e+02_rb,4.3381e+01_rb/)
      kbo(:, 3,25,15) = (/ &
     &2.9711e+03_rb,2.2284e+03_rb,1.4857e+03_rb,7.4302e+02_rb,4.2032e+01_rb/)
      kbo(:, 4,25,15) = (/ &
     &3.6954e+03_rb,2.7716e+03_rb,1.8478e+03_rb,9.2407e+02_rb,4.0668e+01_rb/)
      kbo(:, 5,25,15) = (/ &
     &4.4711e+03_rb,3.3534e+03_rb,2.2357e+03_rb,1.1180e+03_rb,3.9312e+01_rb/)
      kbo(:, 1,26,15) = (/ &
     &1.8457e+03_rb,1.3843e+03_rb,9.2300e+02_rb,4.6166e+02_rb,4.8798e+01_rb/)
      kbo(:, 2,26,15) = (/ &
     &2.4421e+03_rb,1.8317e+03_rb,1.2212e+03_rb,6.1078e+02_rb,4.7150e+01_rb/)
      kbo(:, 3,26,15) = (/ &
     &3.1127e+03_rb,2.3346e+03_rb,1.5565e+03_rb,7.7843e+02_rb,4.5507e+01_rb/)
      kbo(:, 4,26,15) = (/ &
     &3.8432e+03_rb,2.8825e+03_rb,1.9217e+03_rb,9.6104e+02_rb,4.3884e+01_rb/)
      kbo(:, 5,26,15) = (/ &
     &4.6197e+03_rb,3.4648e+03_rb,2.3100e+03_rb,1.1552e+03_rb,4.2296e+01_rb/)
      kbo(:, 1,27,15) = (/ &
     &2.0401e+03_rb,1.5302e+03_rb,1.0202e+03_rb,5.1030e+02_rb,5.2748e+01_rb/)
      kbo(:, 2,27,15) = (/ &
     &2.6727e+03_rb,2.0046e+03_rb,1.3365e+03_rb,6.6845e+02_rb,5.0763e+01_rb/)
      kbo(:, 3,27,15) = (/ &
     &3.3765e+03_rb,2.5325e+03_rb,1.6885e+03_rb,8.4441e+02_rb,4.8815e+01_rb/)
      kbo(:, 4,27,15) = (/ &
     &4.1373e+03_rb,3.1031e+03_rb,2.0688e+03_rb,1.0346e+03_rb,4.6912e+01_rb/)
      kbo(:, 5,27,15) = (/ &
     &4.9426e+03_rb,3.7071e+03_rb,2.4715e+03_rb,1.2359e+03_rb,4.5084e+01_rb/)
      kbo(:, 1,28,15) = (/ &
     &2.3062e+03_rb,1.7297e+03_rb,1.1533e+03_rb,5.7684e+02_rb,5.6494e+01_rb/)
      kbo(:, 2,28,15) = (/ &
     &2.9911e+03_rb,2.2434e+03_rb,1.4957e+03_rb,7.4806e+02_rb,5.4164e+01_rb/)
      kbo(:, 3,28,15) = (/ &
     &3.7467e+03_rb,2.8101e+03_rb,1.8736e+03_rb,9.3698e+02_rb,5.1914e+01_rb/)
      kbo(:, 4,28,15) = (/ &
     &4.5581e+03_rb,3.4187e+03_rb,2.2792e+03_rb,1.1398e+03_rb,4.9734e+01_rb/)
      kbo(:, 5,28,15) = (/ &
     &5.4121e+03_rb,4.0592e+03_rb,2.7063e+03_rb,1.3533e+03_rb,4.7639e+01_rb/)
      kbo(:, 1,29,15) = (/ &
     &2.7587e+03_rb,2.0691e+03_rb,1.3796e+03_rb,6.8998e+02_rb,5.9883e+01_rb/)
      kbo(:, 2,29,15) = (/ &
     &3.5427e+03_rb,2.6571e+03_rb,1.7716e+03_rb,8.8598e+02_rb,5.7214e+01_rb/)
      kbo(:, 3,29,15) = (/ &
     &4.3997e+03_rb,3.2998e+03_rb,2.2000e+03_rb,1.1002e+03_rb,5.4658e+01_rb/)
      kbo(:, 4,29,15) = (/ &
     &5.3158e+03_rb,3.9869e+03_rb,2.6581e+03_rb,1.3292e+03_rb,5.2215e+01_rb/)
      kbo(:, 5,29,15) = (/ &
     &6.2710e+03_rb,4.7033e+03_rb,3.1357e+03_rb,1.5680e+03_rb,4.9872e+01_rb/)
      kbo(:, 1,30,15) = (/ &
     &3.3645e+03_rb,2.5235e+03_rb,1.6825e+03_rb,8.4145e+02_rb,6.2847e+01_rb/)
      kbo(:, 2,30,15) = (/ &
     &4.2782e+03_rb,3.2088e+03_rb,2.1393e+03_rb,1.0699e+03_rb,5.9850e+01_rb/)
      kbo(:, 3,30,15) = (/ &
     &5.2706e+03_rb,3.9531e+03_rb,2.6355e+03_rb,1.3180e+03_rb,5.7012e+01_rb/)
      kbo(:, 4,30,15) = (/ &
     &6.3236e+03_rb,4.7428e+03_rb,3.1620e+03_rb,1.5812e+03_rb,5.4322e+01_rb/)
      kbo(:, 5,30,15) = (/ &
     &7.4122e+03_rb,5.5592e+03_rb,3.7063e+03_rb,1.8534e+03_rb,5.1754e+01_rb/)
      kbo(:, 1,31,15) = (/ &
     &4.2971e+03_rb,3.2230e+03_rb,2.1488e+03_rb,1.0746e+03_rb,6.5331e+01_rb/)
      kbo(:, 2,31,15) = (/ &
     &5.4121e+03_rb,4.0592e+03_rb,2.7063e+03_rb,1.3534e+03_rb,6.2034e+01_rb/)
      kbo(:, 3,31,15) = (/ &
     &6.6141e+03_rb,4.9607e+03_rb,3.3073e+03_rb,1.6539e+03_rb,5.8947e+01_rb/)
      kbo(:, 4,31,15) = (/ &
     &7.8758e+03_rb,5.9069e+03_rb,3.9381e+03_rb,1.9693e+03_rb,5.6027e+01_rb/)
      kbo(:, 5,31,15) = (/ &
     &9.1717e+03_rb,6.8788e+03_rb,4.5860e+03_rb,2.2932e+03_rb,5.3266e+01_rb/)
      kbo(:, 1,32,15) = (/ &
     &5.5229e+03_rb,4.1423e+03_rb,2.7617e+03_rb,1.3811e+03_rb,6.7311e+01_rb/)
      kbo(:, 2,32,15) = (/ &
     &6.8909e+03_rb,5.1683e+03_rb,3.4457e+03_rb,1.7231e+03_rb,6.3759e+01_rb/)
      kbo(:, 3,32,15) = (/ &
     &8.3530e+03_rb,6.2649e+03_rb,4.1768e+03_rb,2.0886e+03_rb,6.0452e+01_rb/)
      kbo(:, 4,32,15) = (/ &
     &9.8746e+03_rb,7.4061e+03_rb,4.9375e+03_rb,2.4690e+03_rb,5.7340e+01_rb/)
      kbo(:, 5,32,15) = (/ &
     &1.1429e+04_rb,8.5717e+03_rb,5.7146e+03_rb,2.8575e+03_rb,5.4420e+01_rb/)
      kbo(:, 1,33,15) = (/ &
     &7.1868e+03_rb,5.3902e+03_rb,3.5936e+03_rb,1.7971e+03_rb,6.8810e+01_rb/)
      kbo(:, 2,33,15) = (/ &
     &8.8853e+03_rb,6.6641e+03_rb,4.4429e+03_rb,2.2217e+03_rb,6.5053e+01_rb/)
      kbo(:, 3,33,15) = (/ &
     &1.0684e+04_rb,8.0128e+03_rb,5.3420e+03_rb,2.6713e+03_rb,6.1559e+01_rb/)
      kbo(:, 4,33,15) = (/ &
     &1.2545e+04_rb,9.4087e+03_rb,6.2726e+03_rb,3.1365e+03_rb,5.8295e+01_rb/)
      kbo(:, 5,33,15) = (/ &
     &1.4437e+04_rb,1.0828e+04_rb,7.2187e+03_rb,3.6096e+03_rb,5.5243e+01_rb/)
      kbo(:, 1,34,15) = (/ &
     &8.9946e+03_rb,6.7461e+03_rb,4.4975e+03_rb,2.2490e+03_rb,7.0022e+01_rb/)
      kbo(:, 2,34,15) = (/ &
     &1.1036e+04_rb,8.2772e+03_rb,5.5183e+03_rb,2.7594e+03_rb,6.6087e+01_rb/)
      kbo(:, 3,34,15) = (/ &
     &1.3184e+04_rb,9.8877e+03_rb,6.5920e+03_rb,3.2962e+03_rb,6.2443e+01_rb/)
      kbo(:, 4,34,15) = (/ &
     &1.5394e+04_rb,1.1546e+04_rb,7.6972e+03_rb,3.8488e+03_rb,5.9052e+01_rb/)
      kbo(:, 5,34,15) = (/ &
     &1.7632e+04_rb,1.3224e+04_rb,8.8161e+03_rb,4.4083e+03_rb,5.5884e+01_rb/)
      kbo(:, 1,35,15) = (/ &
     &1.0993e+04_rb,8.2446e+03_rb,5.4966e+03_rb,2.7485e+03_rb,7.1272e+01_rb/)
      kbo(:, 2,35,15) = (/ &
     &1.3428e+04_rb,1.0071e+04_rb,6.7141e+03_rb,3.3573e+03_rb,6.7165e+01_rb/)
      kbo(:, 3,35,15) = (/ &
     &1.5977e+04_rb,1.1983e+04_rb,7.9889e+03_rb,3.9949e+03_rb,6.3376e+01_rb/)
      kbo(:, 4,35,15) = (/ &
     &1.8595e+04_rb,1.3946e+04_rb,9.2975e+03_rb,4.6490e+03_rb,5.9869e+01_rb/)
      kbo(:, 5,35,15) = (/ &
     &2.1231e+04_rb,1.5923e+04_rb,1.0616e+04_rb,5.3081e+03_rb,5.6025e+01_rb/)
      kbo(:, 1,36,15) = (/ &
     &1.3006e+04_rb,9.7547e+03_rb,6.5033e+03_rb,3.2519e+03_rb,7.2705e+01_rb/)
      kbo(:, 2,36,15) = (/ &
     &1.5874e+04_rb,1.1905e+04_rb,7.9370e+03_rb,3.9687e+03_rb,6.8413e+01_rb/)
      kbo(:, 3,36,15) = (/ &
     &1.8861e+04_rb,1.4146e+04_rb,9.4305e+03_rb,4.7155e+03_rb,6.4479e+01_rb/)
      kbo(:, 4,36,15) = (/ &
     &2.1927e+04_rb,1.6445e+04_rb,1.0964e+04_rb,5.4821e+03_rb,6.0843e+01_rb/)
      kbo(:, 5,36,15) = (/ &
     &2.5014e+04_rb,1.8760e+04_rb,1.2507e+04_rb,6.2537e+03_rb,5.6544e+01_rb/)
      kbo(:, 1,37,15) = (/ &
     &1.4489e+04_rb,1.0867e+04_rb,7.2447e+03_rb,3.6226e+03_rb,7.4617e+01_rb/)
      kbo(:, 2,37,15) = (/ &
     &1.7744e+04_rb,1.3308e+04_rb,8.8720e+03_rb,4.4362e+03_rb,7.0104e+01_rb/)
      kbo(:, 3,37,15) = (/ &
     &2.1142e+04_rb,1.5857e+04_rb,1.0571e+04_rb,5.2858e+03_rb,6.5985e+01_rb/)
      kbo(:, 4,37,15) = (/ &
     &2.4633e+04_rb,1.8475e+04_rb,1.2317e+04_rb,6.1586e+03_rb,6.1723e+01_rb/)
      kbo(:, 5,37,15) = (/ &
     &2.8152e+04_rb,2.1114e+04_rb,1.4076e+04_rb,7.0383e+03_rb,5.7616e+01_rb/)
      kbo(:, 1,38,15) = (/ &
     &1.6314e+04_rb,1.2236e+04_rb,8.1572e+03_rb,4.0788e+03_rb,7.6413e+01_rb/)
      kbo(:, 2,38,15) = (/ &
     &2.0055e+04_rb,1.5041e+04_rb,1.0028e+04_rb,5.0140e+03_rb,7.1696e+01_rb/)
      kbo(:, 3,38,15) = (/ &
     &2.3973e+04_rb,1.7980e+04_rb,1.1987e+04_rb,5.9936e+03_rb,6.7406e+01_rb/)
      kbo(:, 4,38,15) = (/ &
     &2.8001e+04_rb,2.1001e+04_rb,1.4001e+04_rb,7.0006e+03_rb,6.2768e+01_rb/)
      kbo(:, 5,38,15) = (/ &
     &3.2071e+04_rb,2.4053e+04_rb,1.6036e+04_rb,8.0180e+03_rb,5.8602e+01_rb/)
      kbo(:, 1,39,15) = (/ &
     &1.8759e+04_rb,1.4069e+04_rb,9.3797e+03_rb,4.6901e+03_rb,7.8086e+01_rb/)
      kbo(:, 2,39,15) = (/ &
     &2.3153e+04_rb,1.7365e+04_rb,1.1577e+04_rb,5.7886e+03_rb,7.3162e+01_rb/)
      kbo(:, 3,39,15) = (/ &
     &2.7770e+04_rb,2.0828e+04_rb,1.3886e+04_rb,6.9431e+03_rb,6.8719e+01_rb/)
      kbo(:, 4,39,15) = (/ &
     &3.2517e+04_rb,2.4388e+04_rb,1.6259e+04_rb,8.1299e+03_rb,6.3661e+01_rb/)
      kbo(:, 5,39,15) = (/ &
     &3.7323e+04_rb,2.7992e+04_rb,1.8662e+04_rb,9.3312e+03_rb,5.9468e+01_rb/)
      kbo(:, 1,40,15) = (/ &
     &1.9750e+04_rb,1.4812e+04_rb,9.8751e+03_rb,4.9378e+03_rb,8.0192e+01_rb/)
      kbo(:, 2,40,15) = (/ &
     &2.4562e+04_rb,1.8422e+04_rb,1.2281e+04_rb,6.1409e+03_rb,7.5032e+01_rb/)
      kbo(:, 3,40,15) = (/ &
     &2.9646e+04_rb,2.2235e+04_rb,1.4823e+04_rb,7.4118e+03_rb,7.0393e+01_rb/)
      kbo(:, 4,40,15) = (/ &
     &3.4890e+04_rb,2.6167e+04_rb,1.7445e+04_rb,8.7227e+03_rb,6.5151e+01_rb/)
      kbo(:, 5,40,15) = (/ &
     &4.0218e+04_rb,3.0164e+04_rb,2.0109e+04_rb,1.0055e+04_rb,6.0809e+01_rb/)
      kbo(:, 1,41,15) = (/ &
     &2.0620e+04_rb,1.5465e+04_rb,1.0310e+04_rb,5.1555e+03_rb,8.2333e+01_rb/)
      kbo(:, 2,41,15) = (/ &
     &2.5862e+04_rb,1.9396e+04_rb,1.2931e+04_rb,6.4659e+03_rb,7.6930e+01_rb/)
      kbo(:, 3,41,15) = (/ &
     &3.1429e+04_rb,2.3571e+04_rb,1.5714e+04_rb,7.8575e+03_rb,7.2080e+01_rb/)
      kbo(:, 4,41,15) = (/ &
     &3.7207e+04_rb,2.7905e+04_rb,1.8604e+04_rb,9.3022e+03_rb,6.6689e+01_rb/)
      kbo(:, 5,41,15) = (/ &
     &4.3090e+04_rb,3.2318e+04_rb,2.1545e+04_rb,1.0773e+04_rb,6.2194e+01_rb/)
      kbo(:, 1,42,15) = (/ &
     &2.1691e+04_rb,1.6268e+04_rb,1.0846e+04_rb,5.4231e+03_rb,8.4448e+01_rb/)
      kbo(:, 2,42,15) = (/ &
     &2.7444e+04_rb,2.0583e+04_rb,1.3722e+04_rb,6.8623e+03_rb,7.8791e+01_rb/)
      kbo(:, 3,42,15) = (/ &
     &3.3603e+04_rb,2.5202e+04_rb,1.6802e+04_rb,8.4010e+03_rb,7.3737e+01_rb/)
      kbo(:, 4,42,15) = (/ &
     &4.0024e+04_rb,3.0018e+04_rb,2.0012e+04_rb,1.0006e+04_rb,6.8167e+01_rb/)
      kbo(:, 5,42,15) = (/ &
     &4.6588e+04_rb,3.4941e+04_rb,2.3294e+04_rb,1.1647e+04_rb,6.3488e+01_rb/)
      kbo(:, 1,43,15) = (/ &
     &2.2603e+04_rb,1.6952e+04_rb,1.1302e+04_rb,5.6511e+03_rb,8.6896e+01_rb/)
      kbo(:, 2,43,15) = (/ &
     &2.8935e+04_rb,2.1702e+04_rb,1.4468e+04_rb,7.2343e+03_rb,8.0969e+01_rb/)
      kbo(:, 3,43,15) = (/ &
     &3.5766e+04_rb,2.6825e+04_rb,1.7883e+04_rb,8.9419e+03_rb,7.5666e+01_rb/)
      kbo(:, 4,43,15) = (/ &
     &4.2940e+04_rb,3.2205e+04_rb,2.1471e+04_rb,1.0736e+04_rb,6.9870e+01_rb/)
      kbo(:, 5,43,15) = (/ &
     &5.0307e+04_rb,3.7731e+04_rb,2.5154e+04_rb,1.2577e+04_rb,6.5095e+01_rb/)
      kbo(:, 1,44,15) = (/ &
     &2.3606e+04_rb,1.7705e+04_rb,1.1803e+04_rb,5.9018e+03_rb,8.9540e+01_rb/)
      kbo(:, 2,44,15) = (/ &
     &3.0627e+04_rb,2.2970e+04_rb,1.5314e+04_rb,7.6570e+03_rb,8.3300e+01_rb/)
      kbo(:, 3,44,15) = (/ &
     &3.8272e+04_rb,2.8704e+04_rb,1.9136e+04_rb,9.5682e+03_rb,7.7742e+01_rb/)
      kbo(:, 4,44,15) = (/ &
     &4.6369e+04_rb,3.4777e+04_rb,2.3185e+04_rb,1.1593e+04_rb,7.1787e+01_rb/)
      kbo(:, 5,44,15) = (/ &
     &5.4746e+04_rb,4.1060e+04_rb,2.7373e+04_rb,1.3687e+04_rb,6.6743e+01_rb/)
      kbo(:, 1,45,15) = (/ &
     &2.5020e+04_rb,1.8765e+04_rb,1.2510e+04_rb,6.2554e+03_rb,9.2249e+01_rb/)
      kbo(:, 2,45,15) = (/ &
     &3.2921e+04_rb,2.4691e+04_rb,1.6461e+04_rb,8.2305e+03_rb,8.5687e+01_rb/)
      kbo(:, 3,45,15) = (/ &
     &4.1630e+04_rb,3.1222e+04_rb,2.0815e+04_rb,1.0408e+04_rb,7.9854e+01_rb/)
      kbo(:, 4,45,15) = (/ &
     &5.0934e+04_rb,3.8200e+04_rb,2.5467e+04_rb,1.2734e+04_rb,7.3688e+01_rb/)
      kbo(:, 5,45,15) = (/ &
     &6.0623e+04_rb,4.5467e+04_rb,3.0312e+04_rb,1.5156e+04_rb,6.8450e+01_rb/)
      kbo(:, 1,46,15) = (/ &
     &2.6462e+04_rb,1.9847e+04_rb,1.3231e+04_rb,6.6159e+03_rb,9.5173e+01_rb/)
      kbo(:, 2,46,15) = (/ &
     &3.5388e+04_rb,2.6541e+04_rb,1.7694e+04_rb,8.8474e+03_rb,8.8256e+01_rb/)
      kbo(:, 3,46,15) = (/ &
     &4.5343e+04_rb,3.4008e+04_rb,2.2672e+04_rb,1.1336e+04_rb,8.2143e+01_rb/)
      kbo(:, 4,46,15) = (/ &
     &5.6094e+04_rb,4.2071e+04_rb,2.8047e+04_rb,1.4024e+04_rb,7.5728e+01_rb/)
      kbo(:, 5,46,15) = (/ &
     &6.7389e+04_rb,5.0542e+04_rb,3.3695e+04_rb,1.6848e+04_rb,7.0196e+01_rb/)
      kbo(:, 1,47,15) = (/ &
     &2.7156e+04_rb,2.0367e+04_rb,1.3578e+04_rb,6.7894e+03_rb,9.8564e+01_rb/)
      kbo(:, 2,47,15) = (/ &
     &3.7028e+04_rb,2.7771e+04_rb,1.8514e+04_rb,9.2574e+03_rb,9.1210e+01_rb/)
      kbo(:, 3,47,15) = (/ &
     &4.8225e+04_rb,3.6169e+04_rb,2.4113e+04_rb,1.2057e+04_rb,8.4753e+01_rb/)
      kbo(:, 4,47,15) = (/ &
     &6.0453e+04_rb,4.5340e+04_rb,3.0227e+04_rb,1.5114e+04_rb,7.8168e+01_rb/)
      kbo(:, 5,47,15) = (/ &
     &7.3441e+04_rb,5.5081e+04_rb,3.6721e+04_rb,1.8361e+04_rb,7.1964e+01_rb/)
      kbo(:, 1,48,15) = (/ &
     &2.8408e+04_rb,2.1306e+04_rb,1.4204e+04_rb,7.1024e+03_rb,1.0210e+02_rb/)
      kbo(:, 2,48,15) = (/ &
     &3.9561e+04_rb,2.9671e+04_rb,1.9781e+04_rb,9.8907e+03_rb,9.4282e+01_rb/)
      kbo(:, 3,48,15) = (/ &
     &5.2433e+04_rb,3.9325e+04_rb,2.6217e+04_rb,1.3109e+04_rb,8.7454e+01_rb/)
      kbo(:, 4,48,15) = (/ &
     &6.6685e+04_rb,5.0014e+04_rb,3.3343e+04_rb,1.6672e+04_rb,8.0648e+01_rb/)
      kbo(:, 5,48,15) = (/ &
     &8.1969e+04_rb,6.1477e+04_rb,4.0985e+04_rb,2.0493e+04_rb,7.3773e+01_rb/)
      kbo(:, 1,49,15) = (/ &
     &3.0665e+04_rb,2.2999e+04_rb,1.5333e+04_rb,7.6665e+03_rb,1.0579e+02_rb/)
      kbo(:, 2,49,15) = (/ &
     &4.3701e+04_rb,3.2776e+04_rb,2.1851e+04_rb,1.0926e+04_rb,9.7512e+01_rb/)
      kbo(:, 3,49,15) = (/ &
     &5.9013e+04_rb,4.4260e+04_rb,2.9506e+04_rb,1.4753e+04_rb,9.0273e+01_rb/)
      kbo(:, 4,49,15) = (/ &
     &7.6235e+04_rb,5.7177e+04_rb,3.8118e+04_rb,1.9059e+04_rb,8.3012e+01_rb/)
      kbo(:, 5,49,15) = (/ &
     &9.4928e+04_rb,7.1196e+04_rb,4.7464e+04_rb,2.3732e+04_rb,7.5643e+01_rb/)
      kbo(:, 1,50,15) = (/ &
     &3.1512e+04_rb,2.3634e+04_rb,1.5756e+04_rb,7.8783e+03_rb,1.0944e+02_rb/)
      kbo(:, 2,50,15) = (/ &
     &4.5989e+04_rb,3.4492e+04_rb,2.2995e+04_rb,1.1498e+04_rb,1.0069e+02_rb/)
      kbo(:, 3,50,15) = (/ &
     &6.3288e+04_rb,4.7466e+04_rb,3.1644e+04_rb,1.5822e+04_rb,9.3050e+01_rb/)
      kbo(:, 4,50,15) = (/ &
     &8.3061e+04_rb,6.2296e+04_rb,4.1531e+04_rb,2.0766e+04_rb,8.5379e+01_rb/)
      kbo(:, 5,50,15) = (/ &
     &1.0477e+05_rb,7.8579e+04_rb,5.2386e+04_rb,2.6193e+04_rb,7.7745e+01_rb/)
      kbo(:, 1,51,15) = (/ &
     &3.1512e+04_rb,2.3634e+04_rb,1.5756e+04_rb,7.8783e+03_rb,1.1317e+02_rb/)
      kbo(:, 2,51,15) = (/ &
     &4.7162e+04_rb,3.5373e+04_rb,2.3581e+04_rb,1.1791e+04_rb,1.0391e+02_rb/)
      kbo(:, 3,51,15) = (/ &
     &6.6209e+04_rb,4.9657e+04_rb,3.3105e+04_rb,1.6553e+04_rb,9.5852e+01_rb/)
      kbo(:, 4,51,15) = (/ &
     &8.8316e+04_rb,6.6238e+04_rb,4.4164e+04_rb,2.2080e+04_rb,8.8232e+01_rb/)
      kbo(:, 5,51,15) = (/ &
     &1.1292e+05_rb,8.4693e+04_rb,5.6462e+04_rb,2.8231e+04_rb,8.0003e+01_rb/)
      kbo(:, 1,52,15) = (/ &
     &3.2076e+04_rb,2.4057e+04_rb,1.6038e+04_rb,8.0198e+03_rb,1.1706e+02_rb/)
      kbo(:, 2,52,15) = (/ &
     &4.9342e+04_rb,3.7006e+04_rb,2.4671e+04_rb,1.2336e+04_rb,1.0727e+02_rb/)
      kbo(:, 3,52,15) = (/ &
     &7.0777e+04_rb,5.3083e+04_rb,3.5389e+04_rb,1.7695e+04_rb,9.8793e+01_rb/)
      kbo(:, 4,52,15) = (/ &
     &9.6072e+04_rb,7.2054e+04_rb,4.8037e+04_rb,2.4019e+04_rb,9.0662e+01_rb/)
      kbo(:, 5,52,15) = (/ &
     &1.2464e+05_rb,9.3484e+04_rb,6.2323e+04_rb,3.1162e+04_rb,8.2138e+01_rb/)
      kbo(:, 1,53,15) = (/ &
     &3.3608e+04_rb,2.5207e+04_rb,1.6805e+04_rb,8.4025e+03_rb,1.2114e+02_rb/)
      kbo(:, 2,53,15) = (/ &
     &5.3290e+04_rb,3.9968e+04_rb,2.6645e+04_rb,1.3323e+04_rb,1.1081e+02_rb/)
      kbo(:, 3,53,15) = (/ &
     &7.8269e+04_rb,5.8702e+04_rb,3.9135e+04_rb,1.9568e+04_rb,1.0185e+02_rb/)
      kbo(:, 4,53,15) = (/ &
     &1.0824e+05_rb,8.1183e+04_rb,5.4123e+04_rb,2.7062e+04_rb,9.3093e+01_rb/)
      kbo(:, 5,53,15) = (/ &
     &1.4266e+05_rb,1.0700e+05_rb,7.1333e+04_rb,3.5667e+04_rb,8.4356e+01_rb/)
      kbo(:, 1,54,15) = (/ &
     &3.0554e+04_rb,2.2916e+04_rb,1.5277e+04_rb,7.6389e+03_rb,1.2510e+02_rb/)
      kbo(:, 2,54,15) = (/ &
     &4.9936e+04_rb,3.7452e+04_rb,2.4968e+04_rb,1.2484e+04_rb,1.1423e+02_rb/)
      kbo(:, 3,54,15) = (/ &
     &7.5079e+04_rb,5.6309e+04_rb,3.7540e+04_rb,1.8770e+04_rb,1.0480e+02_rb/)
      kbo(:, 4,54,15) = (/ &
     &1.0578e+05_rb,7.9333e+04_rb,5.2889e+04_rb,2.6445e+04_rb,9.5981e+01_rb/)
      kbo(:, 5,54,15) = (/ &
     &1.4151e+05_rb,1.0613e+05_rb,7.0756e+04_rb,3.5378e+04_rb,8.6958e+01_rb/)
      kbo(:, 1,55,15) = (/ &
     &2.5242e+04_rb,1.8932e+04_rb,1.2621e+04_rb,6.3108e+03_rb,1.2910e+02_rb/)
      kbo(:, 2,55,15) = (/ &
     &4.2561e+04_rb,3.1921e+04_rb,2.1281e+04_rb,1.0641e+04_rb,1.1763e+02_rb/)
      kbo(:, 3,55,15) = (/ &
     &6.5579e+04_rb,4.9184e+04_rb,3.2790e+04_rb,1.6395e+04_rb,1.0775e+02_rb/)
      kbo(:, 4,55,15) = (/ &
     &9.4190e+04_rb,7.0643e+04_rb,4.7095e+04_rb,2.3548e+04_rb,9.9196e+01_rb/)
      kbo(:, 5,55,15) = (/ &
     &1.2798e+05_rb,9.5983e+04_rb,6.3989e+04_rb,3.1995e+04_rb,9.0570e+01_rb/)
      kbo(:, 1,56,15) = (/ &
     &2.0611e+04_rb,1.5459e+04_rb,1.0306e+04_rb,5.1530e+03_rb,1.3327e+02_rb/)
      kbo(:, 2,56,15) = (/ &
     &3.5961e+04_rb,2.6970e+04_rb,1.7980e+04_rb,8.9902e+03_rb,1.2118e+02_rb/)
      kbo(:, 3,56,15) = (/ &
     &5.6902e+04_rb,4.2676e+04_rb,2.8451e+04_rb,1.4227e+04_rb,1.1083e+02_rb/)
      kbo(:, 4,56,15) = (/ &
     &8.3426e+04_rb,6.2569e+04_rb,4.1713e+04_rb,2.0856e+04_rb,1.0187e+02_rb/)
      kbo(:, 5,56,15) = (/ &
     &1.1523e+05_rb,8.6424e+04_rb,5.7616e+04_rb,2.8808e+04_rb,9.3771e+01_rb/)
      kbo(:, 1,57,15) = (/ &
     &1.6610e+04_rb,1.2457e+04_rb,8.3049e+03_rb,4.1526e+03_rb,1.3769e+02_rb/)
      kbo(:, 2,57,15) = (/ &
     &3.0089e+04_rb,2.2567e+04_rb,1.5044e+04_rb,7.5226e+03_rb,1.2490e+02_rb/)
      kbo(:, 3,57,15) = (/ &
     &4.8979e+04_rb,3.6734e+04_rb,2.4489e+04_rb,1.2245e+04_rb,1.1405e+02_rb/)
      kbo(:, 4,57,15) = (/ &
     &7.3462e+04_rb,5.5096e+04_rb,3.6731e+04_rb,1.8365e+04_rb,1.0465e+02_rb/)
      kbo(:, 5,57,15) = (/ &
     &1.0326e+05_rb,7.7443e+04_rb,5.1628e+04_rb,2.5814e+04_rb,9.6480e+01_rb/)
      kbo(:, 1,58,15) = (/ &
     &3.5201e+03_rb,3.2274e+03_rb,2.7672e+03_rb,1.9382e+03_rb,1.4211e+02_rb/)
      kbo(:, 2,58,15) = (/ &
     &6.6315e+03_rb,6.0801e+03_rb,5.2132e+03_rb,3.6514e+03_rb,1.2863e+02_rb/)
      kbo(:, 3,58,15) = (/ &
     &1.1117e+04_rb,1.0193e+04_rb,8.7395e+03_rb,6.1212e+03_rb,1.1722e+02_rb/)
      kbo(:, 4,58,15) = (/ &
     &1.7057e+04_rb,1.5638e+04_rb,1.3409e+04_rb,9.3914e+03_rb,1.0740e+02_rb/)
      kbo(:, 5,58,15) = (/ &
     &2.4410e+04_rb,2.2380e+04_rb,1.9189e+04_rb,1.3440e+04_rb,9.8884e+01_rb/)
      kbo(:, 1,59,15) = (/ &
     &3.7699e+03_rb,3.2918e+03_rb,2.6257e+03_rb,1.6340e+03_rb,1.4398e+02_rb/)
      kbo(:, 2,59,15) = (/ &
     &7.2215e+03_rb,6.3055e+03_rb,5.0297e+03_rb,3.1299e+03_rb,1.3019e+02_rb/)
      kbo(:, 3,59,15) = (/ &
     &1.2258e+04_rb,1.0703e+04_rb,8.5377e+03_rb,5.3128e+03_rb,1.1856e+02_rb/)
      kbo(:, 4,59,15) = (/ &
     &1.8986e+04_rb,1.6578e+04_rb,1.3224e+04_rb,8.2288e+03_rb,1.0855e+02_rb/)
      kbo(:, 5,59,15) = (/ &
     &2.7379e+04_rb,2.3906e+04_rb,1.9069e+04_rb,1.1866e+04_rb,9.9884e+01_rb/)
      kbo(:, 1,13,16) = (/ &
     &1.4818e+04_rb,1.1113e+04_rb,7.4089e+03_rb,3.7045e+03_rb,1.1371e+01_rb/)
      kbo(:, 2,13,16) = (/ &
     &2.0739e+04_rb,1.5555e+04_rb,1.0370e+04_rb,5.1851e+03_rb,1.0985e+01_rb/)
      kbo(:, 3,13,16) = (/ &
     &2.7562e+04_rb,2.0672e+04_rb,1.3781e+04_rb,6.8906e+03_rb,1.0720e+01_rb/)
      kbo(:, 4,13,16) = (/ &
     &3.5104e+04_rb,2.6328e+04_rb,1.7552e+04_rb,8.7760e+03_rb,1.0594e+01_rb/)
      kbo(:, 5,13,16) = (/ &
     &4.3187e+04_rb,3.2390e+04_rb,2.1594e+04_rb,1.0797e+04_rb,1.0676e+01_rb/)
      kbo(:, 1,14,16) = (/ &
     &1.1265e+04_rb,8.4488e+03_rb,5.6325e+03_rb,2.8163e+03_rb,1.3004e+01_rb/)
      kbo(:, 2,14,16) = (/ &
     &1.5638e+04_rb,1.1729e+04_rb,7.8191e+03_rb,3.9096e+03_rb,1.2745e+01_rb/)
      kbo(:, 3,14,16) = (/ &
     &2.0632e+04_rb,1.5474e+04_rb,1.0316e+04_rb,5.1580e+03_rb,1.2511e+01_rb/)
      kbo(:, 4,14,16) = (/ &
     &2.6158e+04_rb,1.9619e+04_rb,1.3079e+04_rb,6.5395e+03_rb,1.2514e+01_rb/)
      kbo(:, 5,14,16) = (/ &
     &3.2161e+04_rb,2.4120e+04_rb,1.6080e+04_rb,8.0400e+03_rb,1.2574e+01_rb/)
      kbo(:, 1,15,16) = (/ &
     &8.6626e+03_rb,6.4970e+03_rb,4.3314e+03_rb,2.1657e+03_rb,1.5047e+01_rb/)
      kbo(:, 2,15,16) = (/ &
     &1.1944e+04_rb,8.8995e+03_rb,5.9720e+03_rb,2.9860e+03_rb,1.4804e+01_rb/)
      kbo(:, 3,15,16) = (/ &
     &1.5730e+04_rb,1.1798e+04_rb,7.8651e+03_rb,3.9326e+03_rb,1.4621e+01_rb/)
      kbo(:, 4,15,16) = (/ &
     &1.9953e+04_rb,1.4965e+04_rb,9.9768e+03_rb,4.9885e+03_rb,1.4704e+01_rb/)
      kbo(:, 5,15,16) = (/ &
     &2.4510e+04_rb,1.8382e+04_rb,1.2255e+04_rb,6.1276e+03_rb,1.4738e+01_rb/)
      kbo(:, 1,16,16) = (/ &
     &7.9177e+03_rb,5.9383e+03_rb,3.9589e+03_rb,1.9795e+03_rb,1.7379e+01_rb/)
      kbo(:, 2,16,16) = (/ &
     &1.0905e+04_rb,8.1790e+03_rb,5.4527e+03_rb,2.7264e+03_rb,1.7087e+01_rb/)
      kbo(:, 3,16,16) = (/ &
     &1.4336e+04_rb,1.0752e+04_rb,7.1680e+03_rb,3.5840e+03_rb,1.7079e+01_rb/)
      kbo(:, 4,16,16) = (/ &
     &1.8134e+04_rb,1.3600e+04_rb,9.0670e+03_rb,4.5335e+03_rb,1.7162e+01_rb/)
      kbo(:, 5,16,16) = (/ &
     &2.2206e+04_rb,1.6655e+04_rb,1.1103e+04_rb,5.5517e+03_rb,1.7193e+01_rb/)
      kbo(:, 1,17,16) = (/ &
     &7.5178e+03_rb,5.6384e+03_rb,3.7589e+03_rb,1.8795e+03_rb,1.9948e+01_rb/)
      kbo(:, 2,17,16) = (/ &
     &1.0311e+04_rb,7.7335e+03_rb,5.1557e+03_rb,2.5779e+03_rb,1.9796e+01_rb/)
      kbo(:, 3,17,16) = (/ &
     &1.3498e+04_rb,1.0124e+04_rb,6.7488e+03_rb,3.3746e+03_rb,1.9858e+01_rb/)
      kbo(:, 4,17,16) = (/ &
     &1.6996e+04_rb,1.2747e+04_rb,8.4978e+03_rb,4.2489e+03_rb,1.9944e+01_rb/)
      kbo(:, 5,17,16) = (/ &
     &2.0726e+04_rb,1.5544e+04_rb,1.0363e+04_rb,5.1815e+03_rb,1.9956e+01_rb/)
      kbo(:, 1,18,16) = (/ &
     &7.2889e+03_rb,5.4667e+03_rb,3.6445e+03_rb,1.8223e+03_rb,2.3259e+01_rb/)
      kbo(:, 2,18,16) = (/ &
     &9.9353e+03_rb,7.4515e+03_rb,4.9677e+03_rb,2.4839e+03_rb,2.3214e+01_rb/)
      kbo(:, 3,18,16) = (/ &
     &1.2928e+04_rb,9.6956e+03_rb,6.4638e+03_rb,3.2319e+03_rb,2.3116e+01_rb/)
      kbo(:, 4,18,16) = (/ &
     &1.6189e+04_rb,1.2142e+04_rb,8.0944e+03_rb,4.0472e+03_rb,2.3106e+01_rb/)
      kbo(:, 5,18,16) = (/ &
     &1.9640e+04_rb,1.4730e+04_rb,9.8201e+03_rb,4.9101e+03_rb,2.3045e+01_rb/)
      kbo(:, 1,19,16) = (/ &
     &6.8892e+03_rb,5.1669e+03_rb,3.4446e+03_rb,1.7223e+03_rb,2.7312e+01_rb/)
      kbo(:, 2,19,16) = (/ &
     &9.3210e+03_rb,6.9908e+03_rb,4.6605e+03_rb,2.3303e+03_rb,2.7199e+01_rb/)
      kbo(:, 3,19,16) = (/ &
     &1.2050e+04_rb,9.0375e+03_rb,6.0250e+03_rb,3.0125e+03_rb,2.6980e+01_rb/)
      kbo(:, 4,19,16) = (/ &
     &1.5046e+04_rb,1.1284e+04_rb,7.5228e+03_rb,3.7614e+03_rb,2.6786e+01_rb/)
      kbo(:, 5,19,16) = (/ &
     &1.8241e+04_rb,1.3681e+04_rb,9.1205e+03_rb,4.5603e+03_rb,2.6595e+01_rb/)
      kbo(:, 1,20,16) = (/ &
     &7.0094e+03_rb,5.2570e+03_rb,3.5047e+03_rb,1.7524e+03_rb,3.2009e+01_rb/)
      kbo(:, 2,20,16) = (/ &
     &9.4364e+03_rb,7.0773e+03_rb,4.7182e+03_rb,2.3591e+03_rb,3.1788e+01_rb/)
      kbo(:, 3,20,16) = (/ &
     &1.2173e+04_rb,9.1297e+03_rb,6.0865e+03_rb,3.0433e+03_rb,3.1450e+01_rb/)
      kbo(:, 4,20,16) = (/ &
     &1.5155e+04_rb,1.1367e+04_rb,7.5778e+03_rb,3.7889e+03_rb,3.1059e+01_rb/)
      kbo(:, 5,20,16) = (/ &
     &1.8307e+04_rb,1.3730e+04_rb,9.1534e+03_rb,4.5767e+03_rb,3.0674e+01_rb/)
      kbo(:, 1,21,16) = (/ &
     &7.3306e+03_rb,5.4980e+03_rb,3.6654e+03_rb,1.8327e+03_rb,3.7355e+01_rb/)
      kbo(:, 2,21,16) = (/ &
     &9.8238e+03_rb,7.3679e+03_rb,4.9119e+03_rb,2.4560e+03_rb,3.6998e+01_rb/)
      kbo(:, 3,21,16) = (/ &
     &1.2609e+04_rb,9.4571e+03_rb,6.3047e+03_rb,3.1524e+03_rb,3.6493e+01_rb/)
      kbo(:, 4,21,16) = (/ &
     &1.5617e+04_rb,1.1713e+04_rb,7.8084e+03_rb,3.9042e+03_rb,3.5962e+01_rb/)
      kbo(:, 5,21,16) = (/ &
     &1.8773e+04_rb,1.4079e+04_rb,9.3863e+03_rb,4.6932e+03_rb,3.5346e+01_rb/)
      kbo(:, 1,22,16) = (/ &
     &7.9668e+03_rb,5.9751e+03_rb,3.9834e+03_rb,1.9917e+03_rb,4.3325e+01_rb/)
      kbo(:, 2,22,16) = (/ &
     &1.0571e+04_rb,7.9286e+03_rb,5.2858e+03_rb,2.6429e+03_rb,4.2793e+01_rb/)
      kbo(:, 3,22,16) = (/ &
     &1.3449e+04_rb,1.0087e+04_rb,6.7244e+03_rb,3.3622e+03_rb,4.2221e+01_rb/)
      kbo(:, 4,22,16) = (/ &
     &1.6524e+04_rb,1.2393e+04_rb,8.2619e+03_rb,4.1310e+03_rb,4.1607e+01_rb/)
      kbo(:, 5,22,16) = (/ &
     &1.9754e+04_rb,1.4816e+04_rb,9.8771e+03_rb,4.9386e+03_rb,4.0888e+01_rb/)
      kbo(:, 1,23,16) = (/ &
     &8.6210e+03_rb,6.4658e+03_rb,4.3105e+03_rb,2.1553e+03_rb,5.0428e+01_rb/)
      kbo(:, 2,23,16) = (/ &
     &1.1316e+04_rb,8.4866e+03_rb,5.6578e+03_rb,2.8289e+03_rb,4.9764e+01_rb/)
      kbo(:, 3,23,16) = (/ &
     &1.4291e+04_rb,1.0718e+04_rb,7.1454e+03_rb,3.5727e+03_rb,4.9049e+01_rb/)
      kbo(:, 4,23,16) = (/ &
     &1.7469e+04_rb,1.3102e+04_rb,8.7344e+03_rb,4.3672e+03_rb,4.8146e+01_rb/)
      kbo(:, 5,23,16) = (/ &
     &2.0774e+04_rb,1.5580e+04_rb,1.0387e+04_rb,5.1935e+03_rb,4.7113e+01_rb/)
      kbo(:, 1,24,16) = (/ &
     &9.2015e+03_rb,6.9011e+03_rb,4.6008e+03_rb,2.3004e+03_rb,5.8527e+01_rb/)
      kbo(:, 2,24,16) = (/ &
     &1.1985e+04_rb,8.9232e+03_rb,5.9928e+03_rb,2.9964e+03_rb,5.7489e+01_rb/)
      kbo(:, 3,24,16) = (/ &
     &1.5026e+04_rb,1.1269e+04_rb,7.5129e+03_rb,3.7565e+03_rb,5.6333e+01_rb/)
      kbo(:, 4,24,16) = (/ &
     &1.8238e+04_rb,1.3679e+04_rb,9.1191e+03_rb,4.5596e+03_rb,5.5030e+01_rb/)
      kbo(:, 5,24,16) = (/ &
     &2.1539e+04_rb,1.6154e+04_rb,1.0769e+04_rb,5.3848e+03_rb,5.3599e+01_rb/)
      kbo(:, 1,25,16) = (/ &
     &9.9033e+03_rb,7.4275e+03_rb,4.9517e+03_rb,2.4759e+03_rb,6.7091e+01_rb/)
      kbo(:, 2,25,16) = (/ &
     &1.2768e+04_rb,9.5762e+03_rb,6.3841e+03_rb,3.1921e+03_rb,6.5542e+01_rb/)
      kbo(:, 3,25,16) = (/ &
     &1.5871e+04_rb,1.1903e+04_rb,7.9355e+03_rb,3.9678e+03_rb,6.3831e+01_rb/)
      kbo(:, 4,25,16) = (/ &
     &1.9110e+04_rb,1.4333e+04_rb,9.5551e+03_rb,4.7776e+03_rb,6.2029e+01_rb/)
      kbo(:, 5,25,16) = (/ &
     &2.2417e+04_rb,1.6813e+04_rb,1.1208e+04_rb,5.6043e+03_rb,6.0136e+01_rb/)
      kbo(:, 1,26,16) = (/ &
     &1.0777e+04_rb,8.0825e+03_rb,5.3884e+03_rb,2.6942e+03_rb,7.5807e+01_rb/)
      kbo(:, 2,26,16) = (/ &
     &1.3749e+04_rb,1.0312e+04_rb,6.8746e+03_rb,3.4373e+03_rb,7.3586e+01_rb/)
      kbo(:, 3,26,16) = (/ &
     &1.6921e+04_rb,1.2690e+04_rb,8.4603e+03_rb,4.2302e+03_rb,7.1246e+01_rb/)
      kbo(:, 4,26,16) = (/ &
     &2.0227e+04_rb,1.5170e+04_rb,1.0114e+04_rb,5.0568e+03_rb,6.8870e+01_rb/)
      kbo(:, 5,26,16) = (/ &
     &2.3585e+04_rb,1.7688e+04_rb,1.1792e+04_rb,5.8962e+03_rb,6.6424e+01_rb/)
      kbo(:, 1,27,16) = (/ &
     &1.2052e+04_rb,9.0388e+03_rb,6.0259e+03_rb,3.0130e+03_rb,8.4316e+01_rb/)
      kbo(:, 2,27,16) = (/ &
     &1.5222e+04_rb,1.1417e+04_rb,7.6111e+03_rb,3.8056e+03_rb,8.1386e+01_rb/)
      kbo(:, 3,27,16) = (/ &
     &1.8590e+04_rb,1.3942e+04_rb,9.2948e+03_rb,4.6474e+03_rb,7.8348e+01_rb/)
      kbo(:, 4,27,16) = (/ &
     &2.2058e+04_rb,1.6543e+04_rb,1.1029e+04_rb,5.5145e+03_rb,7.5340e+01_rb/)
      kbo(:, 5,27,16) = (/ &
     &2.5550e+04_rb,1.9162e+04_rb,1.2775e+04_rb,6.3875e+03_rb,7.2329e+01_rb/)
      kbo(:, 1,28,16) = (/ &
     &1.3724e+04_rb,1.0293e+04_rb,6.8620e+03_rb,3.4310e+03_rb,9.2351e+01_rb/)
      kbo(:, 2,28,16) = (/ &
     &1.7171e+04_rb,1.2878e+04_rb,8.5855e+03_rb,4.2928e+03_rb,8.8670e+01_rb/)
      kbo(:, 3,28,16) = (/ &
     &2.0788e+04_rb,1.5591e+04_rb,1.0394e+04_rb,5.1970e+03_rb,8.4928e+01_rb/)
      kbo(:, 4,28,16) = (/ &
     &2.4487e+04_rb,1.8365e+04_rb,1.2244e+04_rb,6.1218e+03_rb,8.1255e+01_rb/)
      kbo(:, 5,28,16) = (/ &
     &2.8173e+04_rb,2.1130e+04_rb,1.4087e+04_rb,7.0433e+03_rb,7.7666e+01_rb/)
      kbo(:, 1,29,16) = (/ &
     &1.6469e+04_rb,1.2352e+04_rb,8.2344e+03_rb,4.1172e+03_rb,9.9675e+01_rb/)
      kbo(:, 2,29,16) = (/ &
     &2.0402e+04_rb,1.5302e+04_rb,1.0201e+04_rb,5.1006e+03_rb,9.5182e+01_rb/)
      kbo(:, 3,29,16) = (/ &
     &2.4499e+04_rb,1.8374e+04_rb,1.2249e+04_rb,6.1246e+03_rb,9.0759e+01_rb/)
      kbo(:, 4,29,16) = (/ &
     &2.8632e+04_rb,2.1474e+04_rb,1.4316e+04_rb,7.1579e+03_rb,8.6444e+01_rb/)
      kbo(:, 5,29,16) = (/ &
     &3.2723e+04_rb,2.4542e+04_rb,1.6361e+04_rb,8.1807e+03_rb,8.2287e+01_rb/)
      kbo(:, 1,30,16) = (/ &
     &2.0078e+04_rb,1.5058e+04_rb,1.0039e+04_rb,5.0195e+03_rb,1.0609e+02_rb/)
      kbo(:, 2,30,16) = (/ &
     &2.4638e+04_rb,1.8479e+04_rb,1.2319e+04_rb,6.1595e+03_rb,1.0084e+02_rb/)
      kbo(:, 3,30,16) = (/ &
     &2.9329e+04_rb,2.1997e+04_rb,1.4664e+04_rb,7.3322e+03_rb,9.5740e+01_rb/)
      kbo(:, 4,30,16) = (/ &
     &3.4034e+04_rb,2.5525e+04_rb,1.7017e+04_rb,8.5085e+03_rb,9.0849e+01_rb/)
      kbo(:, 5,30,16) = (/ &
     &3.8657e+04_rb,2.8993e+04_rb,1.9329e+04_rb,9.6643e+03_rb,8.6169e+01_rb/)
      kbo(:, 1,31,16) = (/ &
     &2.5548e+04_rb,1.9161e+04_rb,1.2774e+04_rb,6.3870e+03_rb,1.1146e+02_rb/)
      kbo(:, 2,31,16) = (/ &
     &3.1052e+04_rb,2.3289e+04_rb,1.5526e+04_rb,7.7630e+03_rb,1.0555e+02_rb/)
      kbo(:, 3,31,16) = (/ &
     &3.6674e+04_rb,2.7505e+04_rb,1.8172e+04_rb,9.1684e+03_rb,9.9833e+01_rb/)
      kbo(:, 4,31,16) = (/ &
     &4.2278e+04_rb,3.1708e+04_rb,2.1139e+04_rb,1.0569e+04_rb,9.4413e+01_rb/)
      kbo(:, 5,31,16) = (/ &
     &4.7725e+04_rb,3.5794e+04_rb,2.3863e+04_rb,1.1931e+04_rb,8.9289e+01_rb/)
      kbo(:, 1,32,16) = (/ &
     &3.2648e+04_rb,2.4486e+04_rb,1.6324e+04_rb,8.1620e+03_rb,1.1579e+02_rb/)
      kbo(:, 2,32,16) = (/ &
     &3.9333e+04_rb,2.9500e+04_rb,1.9666e+04_rb,9.8333e+03_rb,1.0928e+02_rb/)
      kbo(:, 3,32,16) = (/ &
     &4.6114e+04_rb,3.4586e+04_rb,2.3057e+04_rb,1.1529e+04_rb,1.0305e+02_rb/)
      kbo(:, 4,32,16) = (/ &
     &5.2809e+04_rb,3.9607e+04_rb,2.6404e+04_rb,1.3202e+04_rb,9.7184e+01_rb/)
      kbo(:, 5,32,16) = (/ &
     &5.9284e+04_rb,4.4463e+04_rb,2.9311e+04_rb,1.4821e+04_rb,9.1687e+01_rb/)
      kbo(:, 1,33,16) = (/ &
     &4.2212e+04_rb,3.1659e+04_rb,2.1106e+04_rb,1.0553e+04_rb,1.1913e+02_rb/)
      kbo(:, 2,33,16) = (/ &
     &5.0427e+04_rb,3.7820e+04_rb,2.5214e+04_rb,1.2607e+04_rb,1.1210e+02_rb/)
      kbo(:, 3,33,16) = (/ &
     &5.8684e+04_rb,4.4013e+04_rb,2.9342e+04_rb,1.4671e+04_rb,1.0546e+02_rb/)
      kbo(:, 4,33,16) = (/ &
     &6.6793e+04_rb,5.0095e+04_rb,3.3396e+04_rb,1.6698e+04_rb,9.9210e+01_rb/)
      kbo(:, 5,33,16) = (/ &
     &7.4575e+04_rb,5.5932e+04_rb,3.7288e+04_rb,1.8644e+04_rb,9.3412e+01_rb/)
      kbo(:, 1,34,16) = (/ &
     &5.2540e+04_rb,3.9405e+04_rb,2.6270e+04_rb,1.3135e+04_rb,1.2180e+02_rb/)
      kbo(:, 2,34,16) = (/ &
     &6.2330e+04_rb,4.6748e+04_rb,3.1165e+04_rb,1.5582e+04_rb,1.1436e+02_rb/)
      kbo(:, 3,34,16) = (/ &
     &7.2093e+04_rb,5.4070e+04_rb,3.6046e+04_rb,1.8023e+04_rb,1.0734e+02_rb/)
      kbo(:, 4,34,16) = (/ &
     &8.1593e+04_rb,6.1194e+04_rb,4.0796e+04_rb,2.0398e+04_rb,1.0080e+02_rb/)
      kbo(:, 5,34,16) = (/ &
     &9.0697e+04_rb,6.8023e+04_rb,4.5349e+04_rb,2.2674e+04_rb,9.4753e+01_rb/)
      kbo(:, 1,35,16) = (/ &
     &6.4051e+04_rb,4.8038e+04_rb,3.2026e+04_rb,1.6013e+04_rb,1.2446e+02_rb/)
      kbo(:, 2,35,16) = (/ &
     &7.5644e+04_rb,5.6733e+04_rb,3.7822e+04_rb,1.8911e+04_rb,1.1662e+02_rb/)
      kbo(:, 3,35,16) = (/ &
     &8.7172e+04_rb,6.5379e+04_rb,4.3586e+04_rb,2.1792e+04_rb,1.0929e+02_rb/)
      kbo(:, 4,35,16) = (/ &
     &9.8380e+04_rb,7.3786e+04_rb,4.9191e+04_rb,2.4595e+04_rb,1.0249e+02_rb/)
      kbo(:, 5,35,16) = (/ &
     &1.0903e+05_rb,8.1776e+04_rb,5.4517e+04_rb,2.7259e+04_rb,9.6197e+01_rb/)
      kbo(:, 1,36,16) = (/ &
     &7.5878e+04_rb,5.6908e+04_rb,3.7939e+04_rb,1.8969e+04_rb,1.2742e+02_rb/)
      kbo(:, 2,36,16) = (/ &
     &8.9471e+04_rb,6.7103e+04_rb,4.4735e+04_rb,2.2368e+04_rb,1.1920e+02_rb/)
      kbo(:, 3,36,16) = (/ &
     &1.0302e+05_rb,7.7263e+04_rb,5.1509e+04_rb,2.5436e+04_rb,1.1154e+02_rb/)
      kbo(:, 4,36,16) = (/ &
     &1.1614e+05_rb,8.7108e+04_rb,5.8072e+04_rb,2.9036e+04_rb,1.0445e+02_rb/)
      kbo(:, 5,36,16) = (/ &
     &1.2862e+05_rb,9.6465e+04_rb,6.4310e+04_rb,3.2155e+04_rb,9.7922e+01_rb/)
      kbo(:, 1,37,16) = (/ &
     &8.5083e+04_rb,6.3812e+04_rb,4.2541e+04_rb,2.1271e+04_rb,1.3125e+02_rb/)
      kbo(:, 2,37,16) = (/ &
     &1.0058e+05_rb,7.5436e+04_rb,5.0291e+04_rb,2.5145e+04_rb,1.2185e+02_rb/)
      kbo(:, 3,37,16) = (/ &
     &1.1610e+05_rb,8.7073e+04_rb,5.8049e+04_rb,2.9024e+04_rb,1.1457e+02_rb/)
      kbo(:, 4,37,16) = (/ &
     &1.3117e+05_rb,9.8374e+04_rb,6.5582e+04_rb,3.2791e+04_rb,1.0715e+02_rb/)
      kbo(:, 5,37,16) = (/ &
     &1.4551e+05_rb,1.0913e+05_rb,7.2757e+04_rb,3.6378e+04_rb,1.0034e+02_rb/)
      kbo(:, 1,38,16) = (/ &
     &9.5077e+04_rb,7.2333e+04_rb,4.8222e+04_rb,2.4111e+04_rb,1.3486e+02_rb/)
      kbo(:, 2,38,16) = (/ &
     &1.1439e+05_rb,8.5793e+04_rb,5.7195e+04_rb,2.8598e+04_rb,1.2580e+02_rb/)
      kbo(:, 3,38,16) = (/ &
     &1.3231e+05_rb,9.9236e+04_rb,6.6157e+04_rb,3.3078e+04_rb,1.1743e+02_rb/)
      kbo(:, 4,38,16) = (/ &
     &1.4989e+05_rb,1.1242e+05_rb,7.4945e+04_rb,3.7472e+04_rb,1.0969e+02_rb/)
      kbo(:, 5,38,16) = (/ &
     &1.6664e+05_rb,1.2498e+05_rb,8.3321e+04_rb,4.1660e+04_rb,1.0261e+02_rb/)
      kbo(:, 1,39,16) = (/ &
     &1.1158e+05_rb,8.3685e+04_rb,5.5790e+04_rb,2.7895e+04_rb,1.3820e+02_rb/)
      kbo(:, 2,39,16) = (/ &
     &1.3280e+05_rb,9.9602e+04_rb,6.6401e+04_rb,3.3201e+04_rb,1.2875e+02_rb/)
      kbo(:, 3,39,16) = (/ &
     &1.5408e+05_rb,1.1556e+05_rb,7.7040e+04_rb,3.8520e+04_rb,1.2007e+02_rb/)
      kbo(:, 4,39,16) = (/ &
     &1.7495e+05_rb,1.3121e+05_rb,8.7476e+04_rb,4.3738e+04_rb,1.1204e+02_rb/)
      kbo(:, 5,39,16) = (/ &
     &1.9488e+05_rb,1.4616e+05_rb,9.7443e+04_rb,4.8722e+04_rb,1.0472e+02_rb/)
      kbo(:, 1,40,16) = (/ &
     &1.1876e+05_rb,8.9071e+04_rb,5.9380e+04_rb,2.9690e+04_rb,1.4235e+02_rb/)
      kbo(:, 2,40,16) = (/ &
     &1.4221e+05_rb,1.0666e+05_rb,7.1106e+04_rb,3.5553e+04_rb,1.3244e+02_rb/)
      kbo(:, 3,40,16) = (/ &
     &1.6587e+05_rb,1.2440e+05_rb,8.2934e+04_rb,4.1467e+04_rb,1.2341e+02_rb/)
      kbo(:, 4,40,16) = (/ &
     &1.8920e+05_rb,1.4190e+05_rb,9.4601e+04_rb,4.7300e+04_rb,1.1506e+02_rb/)
      kbo(:, 5,40,16) = (/ &
     &2.1165e+05_rb,1.5874e+05_rb,1.0583e+05_rb,5.2913e+04_rb,1.0743e+02_rb/)
      kbo(:, 1,41,16) = (/ &
     &1.2546e+05_rb,9.4099e+04_rb,6.2733e+04_rb,3.1367e+04_rb,1.4657e+02_rb/)
      kbo(:, 2,41,16) = (/ &
     &1.5125e+05_rb,1.1344e+05_rb,7.5625e+04_rb,3.7812e+04_rb,1.3531e+02_rb/)
      kbo(:, 3,41,16) = (/ &
     &1.7744e+05_rb,1.3308e+05_rb,8.8723e+04_rb,4.4362e+04_rb,1.2676e+02_rb/)
      kbo(:, 4,41,16) = (/ &
     &2.0341e+05_rb,1.5256e+05_rb,1.0171e+05_rb,5.0854e+04_rb,1.1810e+02_rb/)
      kbo(:, 5,41,16) = (/ &
     &2.2858e+05_rb,1.7144e+05_rb,1.1429e+05_rb,5.7146e+04_rb,1.1015e+02_rb/)
      kbo(:, 1,42,16) = (/ &
     &1.3356e+05_rb,1.0017e+05_rb,6.6777e+04_rb,3.3389e+04_rb,1.5077e+02_rb/)
      kbo(:, 2,42,16) = (/ &
     &1.6221e+05_rb,1.2166e+05_rb,8.1105e+04_rb,4.0545e+04_rb,1.3986e+02_rb/)
      kbo(:, 3,42,16) = (/ &
     &1.9146e+05_rb,1.4359e+05_rb,9.5729e+04_rb,4.7864e+04_rb,1.3005e+02_rb/)
      kbo(:, 4,42,16) = (/ &
     &2.1781e+05_rb,1.6546e+05_rb,1.1031e+05_rb,5.5153e+04_rb,1.2109e+02_rb/)
      kbo(:, 5,42,16) = (/ &
     &2.4912e+05_rb,1.8684e+05_rb,1.2456e+05_rb,6.2279e+04_rb,1.1284e+02_rb/)
      kbo(:, 1,43,16) = (/ &
     &1.4126e+05_rb,1.0594e+05_rb,7.0630e+04_rb,3.5315e+04_rb,1.5567e+02_rb/)
      kbo(:, 2,43,16) = (/ &
     &1.7322e+05_rb,1.2992e+05_rb,8.6612e+04_rb,4.3306e+04_rb,1.4408e+02_rb/)
      kbo(:, 3,43,16) = (/ &
     &2.0611e+05_rb,1.5458e+05_rb,1.0200e+05_rb,5.1528e+04_rb,1.3385e+02_rb/)
      kbo(:, 4,43,16) = (/ &
     &2.3919e+05_rb,1.7940e+05_rb,1.1960e+05_rb,5.9799e+04_rb,1.2454e+02_rb/)
      kbo(:, 5,43,16) = (/ &
     &2.7160e+05_rb,2.0370e+05_rb,1.3580e+05_rb,6.7900e+04_rb,1.1596e+02_rb/)
      kbo(:, 1,44,16) = (/ &
     &1.5000e+05_rb,1.1250e+05_rb,7.5001e+04_rb,3.7500e+04_rb,1.6094e+02_rb/)
      kbo(:, 2,44,16) = (/ &
     &1.8602e+05_rb,1.3952e+05_rb,9.3011e+04_rb,4.6505e+04_rb,1.4866e+02_rb/)
      kbo(:, 3,44,16) = (/ &
     &2.2342e+05_rb,1.6756e+05_rb,1.1171e+05_rb,5.5854e+04_rb,1.3792e+02_rb/)
      kbo(:, 4,44,16) = (/ &
     &2.6122e+05_rb,1.9592e+05_rb,1.3061e+05_rb,6.5304e+04_rb,1.2823e+02_rb/)
      kbo(:, 5,44,16) = (/ &
     &2.9874e+05_rb,2.2406e+05_rb,1.4937e+05_rb,7.4685e+04_rb,1.1932e+02_rb/)
      kbo(:, 1,45,16) = (/ &
     &1.6171e+05_rb,1.2128e+05_rb,8.0854e+04_rb,4.0427e+04_rb,1.6635e+02_rb/)
      kbo(:, 2,45,16) = (/ &
     &2.0304e+05_rb,1.5228e+05_rb,1.0152e+05_rb,5.0760e+04_rb,1.5336e+02_rb/)
      kbo(:, 3,45,16) = (/ &
     &2.4630e+05_rb,1.8473e+05_rb,1.2315e+05_rb,6.1576e+04_rb,1.4204e+02_rb/)
      kbo(:, 4,45,16) = (/ &
     &2.9041e+05_rb,2.1781e+05_rb,1.4520e+05_rb,7.2602e+04_rb,1.3193e+02_rb/)
      kbo(:, 5,45,16) = (/ &
     &3.3433e+05_rb,2.5074e+05_rb,1.6716e+05_rb,8.3581e+04_rb,1.2271e+02_rb/)
      kbo(:, 1,46,16) = (/ &
     &1.7432e+05_rb,1.3074e+05_rb,8.7162e+04_rb,4.3581e+04_rb,1.7227e+02_rb/)
      kbo(:, 2,46,16) = (/ &
     &2.2186e+05_rb,1.6640e+05_rb,1.1093e+05_rb,5.5465e+04_rb,1.5849e+02_rb/)
      kbo(:, 3,46,16) = (/ &
     &2.7221e+05_rb,2.0416e+05_rb,1.3611e+05_rb,6.8053e+04_rb,1.4648e+02_rb/)
      kbo(:, 4,46,16) = (/ &
     &3.2411e+05_rb,2.4308e+05_rb,1.6205e+05_rb,8.1026e+04_rb,1.3593e+02_rb/)
      kbo(:, 5,46,16) = (/ &
     &3.7610e+05_rb,2.8208e+05_rb,1.8805e+05_rb,9.4025e+04_rb,1.2634e+02_rb/)
      kbo(:, 1,47,16) = (/ &
     &1.8289e+05_rb,1.3717e+05_rb,9.1447e+04_rb,4.5723e+04_rb,1.7909e+02_rb/)
      kbo(:, 2,47,16) = (/ &
     &2.3679e+05_rb,1.7759e+05_rb,1.1839e+05_rb,5.9198e+04_rb,1.6436e+02_rb/)
      kbo(:, 3,47,16) = (/ &
     &2.9447e+05_rb,2.2085e+05_rb,1.4724e+05_rb,7.3618e+04_rb,1.5159e+02_rb/)
      kbo(:, 4,47,16) = (/ &
     &3.5454e+05_rb,2.6590e+05_rb,1.7727e+05_rb,8.8635e+04_rb,1.4045e+02_rb/)
      kbo(:, 5,47,16) = (/ &
     &4.1548e+05_rb,3.1161e+05_rb,2.0774e+05_rb,1.0387e+05_rb,1.3046e+02_rb/)
      kbo(:, 1,48,16) = (/ &
     &1.9579e+05_rb,1.4684e+05_rb,9.7896e+04_rb,4.8948e+04_rb,1.8623e+02_rb/)
      kbo(:, 2,48,16) = (/ &
     &2.5824e+05_rb,1.9368e+05_rb,1.2912e+05_rb,6.4559e+04_rb,1.7055e+02_rb/)
      kbo(:, 3,48,16) = (/ &
     &3.2608e+05_rb,2.4456e+05_rb,1.6304e+05_rb,8.0315e+04_rb,1.5696e+02_rb/)
      kbo(:, 4,48,16) = (/ &
     &3.9734e+05_rb,2.9801e+05_rb,1.9867e+05_rb,9.9336e+04_rb,1.4512e+02_rb/)
      kbo(:, 5,48,16) = (/ &
     &4.7039e+05_rb,3.5279e+05_rb,2.3519e+05_rb,1.1760e+05_rb,1.3467e+02_rb/)
      kbo(:, 1,49,16) = (/ &
     &2.1650e+05_rb,1.6238e+05_rb,1.0825e+05_rb,5.4125e+04_rb,1.9381e+02_rb/)
      kbo(:, 2,49,16) = (/ &
     &2.9149e+05_rb,2.1861e+05_rb,1.4574e+05_rb,7.2871e+04_rb,1.7704e+02_rb/)
      kbo(:, 3,49,16) = (/ &
     &3.7402e+05_rb,2.8052e+05_rb,1.8701e+05_rb,9.3505e+04_rb,1.6258e+02_rb/)
      kbo(:, 4,49,16) = (/ &
     &4.6192e+05_rb,3.4644e+05_rb,2.3096e+05_rb,1.1548e+05_rb,1.4999e+02_rb/)
      kbo(:, 5,49,16) = (/ &
     &5.5289e+05_rb,4.1467e+05_rb,2.7644e+05_rb,1.3822e+05_rb,1.3901e+02_rb/)
      kbo(:, 1,50,16) = (/ &
     &2.2791e+05_rb,1.7093e+05_rb,1.1395e+05_rb,5.6977e+04_rb,2.0134e+02_rb/)
      kbo(:, 2,50,16) = (/ &
     &3.1325e+05_rb,2.3494e+05_rb,1.5662e+05_rb,7.8312e+04_rb,1.8345e+02_rb/)
      kbo(:, 3,50,16) = (/ &
     &4.0878e+05_rb,3.0659e+05_rb,2.0439e+05_rb,1.0220e+05_rb,1.6811e+02_rb/)
      kbo(:, 4,50,16) = (/ &
     &5.1152e+05_rb,3.8364e+05_rb,2.5576e+05_rb,1.2788e+05_rb,1.5482e+02_rb/)
      kbo(:, 5,50,16) = (/ &
     &6.1913e+05_rb,4.6434e+05_rb,3.0956e+05_rb,1.5478e+05_rb,1.4323e+02_rb/)
      kbo(:, 1,51,16) = (/ &
     &2.3351e+05_rb,1.7513e+05_rb,1.1675e+05_rb,5.8377e+04_rb,2.0909e+02_rb/)
      kbo(:, 2,51,16) = (/ &
     &3.2811e+05_rb,2.4608e+05_rb,1.6406e+05_rb,8.2029e+04_rb,1.9001e+02_rb/)
      kbo(:, 3,51,16) = (/ &
     &4.3579e+05_rb,3.2684e+05_rb,2.1790e+05_rb,1.0895e+05_rb,1.7377e+02_rb/)
      kbo(:, 4,51,16) = (/ &
     &5.5314e+05_rb,4.1486e+05_rb,2.7653e+05_rb,1.3829e+05_rb,1.5970e+02_rb/)
      kbo(:, 5,51,16) = (/ &
     &6.7717e+05_rb,5.0788e+05_rb,3.3859e+05_rb,1.6929e+05_rb,1.4750e+02_rb/)
      kbo(:, 1,52,16) = (/ &
     &2.4395e+05_rb,1.8296e+05_rb,1.2197e+05_rb,6.0984e+04_rb,2.1739e+02_rb/)
      kbo(:, 2,52,16) = (/ &
     &3.5104e+05_rb,2.6328e+05_rb,1.7552e+05_rb,8.7761e+04_rb,1.9693e+02_rb/)
      kbo(:, 3,52,16) = (/ &
     &4.7523e+05_rb,3.5643e+05_rb,2.3762e+05_rb,1.1881e+05_rb,1.7968e+02_rb/)
      kbo(:, 4,52,16) = (/ &
     &6.1247e+05_rb,4.5936e+05_rb,3.0624e+05_rb,1.5312e+05_rb,1.6481e+02_rb/)
      kbo(:, 5,52,16) = (/ &
     &7.5898e+05_rb,5.6924e+05_rb,3.7949e+05_rb,1.8975e+05_rb,1.5192e+02_rb/)
      kbo(:, 1,53,16) = (/ &
     &2.6284e+05_rb,1.9713e+05_rb,1.3142e+05_rb,6.5712e+04_rb,2.2625e+02_rb/)
      kbo(:, 2,53,16) = (/ &
     &3.8810e+05_rb,2.9108e+05_rb,1.9405e+05_rb,9.7027e+04_rb,2.0424e+02_rb/)
      kbo(:, 3,53,16) = (/ &
     &5.3641e+05_rb,4.0231e+05_rb,2.6821e+05_rb,1.3410e+05_rb,1.8589e+02_rb/)
      kbo(:, 4,53,16) = (/ &
     &7.0283e+05_rb,5.2713e+05_rb,3.5142e+05_rb,1.7571e+05_rb,1.7018e+02_rb/)
      kbo(:, 5,53,16) = (/ &
     &8.8260e+05_rb,6.6195e+05_rb,4.4130e+05_rb,2.2065e+05_rb,1.5658e+02_rb/)
      kbo(:, 1,54,16) = (/ &
     &2.4532e+05_rb,1.8399e+05_rb,1.2266e+05_rb,6.1332e+04_rb,2.3489e+02_rb/)
      kbo(:, 2,54,16) = (/ &
     &3.7212e+05_rb,2.7909e+05_rb,1.8606e+05_rb,9.3031e+04_rb,2.1139e+02_rb/)
      kbo(:, 3,54,16) = (/ &
     &5.1860e+05_rb,3.9351e+05_rb,2.6234e+05_rb,1.3117e+05_rb,1.9188e+02_rb/)
      kbo(:, 4,54,16) = (/ &
     &6.9903e+05_rb,5.2427e+05_rb,3.4952e+05_rb,1.7476e+05_rb,1.7536e+02_rb/)
      kbo(:, 5,54,16) = (/ &
     &8.8946e+05_rb,6.6416e+05_rb,4.4474e+05_rb,2.2237e+05_rb,1.6106e+02_rb/)
      kbo(:, 1,55,16) = (/ &
     &2.0815e+05_rb,1.5612e+05_rb,1.0408e+05_rb,5.2039e+04_rb,2.4370e+02_rb/)
      kbo(:, 2,55,16) = (/ &
     &3.2440e+05_rb,2.4330e+05_rb,1.6220e+05_rb,8.1100e+04_rb,2.1865e+02_rb/)
      kbo(:, 3,55,16) = (/ &
     &4.6741e+05_rb,3.5056e+05_rb,2.3371e+05_rb,1.1685e+05_rb,1.9795e+02_rb/)
      kbo(:, 4,55,16) = (/ &
     &6.3338e+05_rb,4.7504e+05_rb,3.1669e+05_rb,1.5835e+05_rb,1.8053e+02_rb/)
      kbo(:, 5,55,16) = (/ &
     &8.1696e+05_rb,6.1272e+05_rb,4.0848e+05_rb,2.0424e+05_rb,1.6554e+02_rb/)
      kbo(:, 1,56,16) = (/ &
     &1.7485e+05_rb,1.3114e+05_rb,8.7424e+04_rb,4.3712e+04_rb,2.5296e+02_rb/)
      kbo(:, 2,56,16) = (/ &
     &2.8084e+05_rb,2.1063e+05_rb,1.4042e+05_rb,7.0208e+04_rb,2.2634e+02_rb/)
      kbo(:, 3,56,16) = (/ &
     &4.1394e+05_rb,3.1046e+05_rb,2.0697e+05_rb,1.0347e+05_rb,2.0431e+02_rb/)
      kbo(:, 4,56,16) = (/ &
     &5.7118e+05_rb,4.2838e+05_rb,2.8559e+05_rb,1.4279e+05_rb,1.8592e+02_rb/)
      kbo(:, 5,56,16) = (/ &
     &7.4759e+05_rb,5.6069e+05_rb,3.7379e+05_rb,1.8690e+05_rb,1.7020e+02_rb/)
      kbo(:, 1,57,16) = (/ &
     &1.4522e+05_rb,1.0891e+05_rb,7.2607e+04_rb,3.6303e+04_rb,2.6280e+02_rb/)
      kbo(:, 2,57,16) = (/ &
     &2.4098e+05_rb,1.8073e+05_rb,1.2049e+05_rb,6.0242e+04_rb,2.3450e+02_rb/)
      kbo(:, 3,57,16) = (/ &
     &3.6414e+05_rb,2.7311e+05_rb,1.8207e+05_rb,9.1034e+04_rb,2.1103e+02_rb/)
      kbo(:, 4,57,16) = (/ &
     &5.1231e+05_rb,3.8423e+05_rb,2.5615e+05_rb,1.2807e+05_rb,1.9160e+02_rb/)
      kbo(:, 5,57,16) = (/ &
     &6.8106e+05_rb,5.1079e+05_rb,3.4053e+05_rb,1.7026e+05_rb,1.7509e+02_rb/)
      kbo(:, 1,58,16) = (/ &
     &3.1730e+04_rb,2.9092e+04_rb,2.4944e+04_rb,1.7470e+04_rb,2.7269e+02_rb/)
      kbo(:, 2,58,16) = (/ &
     &5.4453e+04_rb,4.9925e+04_rb,4.2806e+04_rb,2.9981e+04_rb,2.4268e+02_rb/)
      kbo(:, 3,58,16) = (/ &
     &8.4447e+04_rb,7.7425e+04_rb,6.6385e+04_rb,4.6495e+04_rb,2.1781e+02_rb/)
      kbo(:, 4,58,16) = (/ &
     &1.2116e+05_rb,1.1108e+05_rb,9.5242e+04_rb,6.6707e+04_rb,1.9723e+02_rb/)
      kbo(:, 5,58,16) = (/ &
     &1.6365e+05_rb,1.5004e+05_rb,1.2865e+05_rb,9.0103e+04_rb,1.7991e+02_rb/)
      kbo(:, 1,59,16) = (/ &
     &3.4421e+04_rb,3.0055e+04_rb,2.3973e+04_rb,1.4918e+04_rb,2.7694e+02_rb/)
      kbo(:, 2,59,16) = (/ &
     &5.9930e+04_rb,5.2328e+04_rb,4.1740e+04_rb,2.5973e+04_rb,2.4614e+02_rb/)
      kbo(:, 3,59,16) = (/ &
     &9.3934e+04_rb,8.2019e+04_rb,6.5423e+04_rb,4.0711e+04_rb,2.2066e+02_rb/)
      kbo(:, 4,59,16) = (/ &
     &1.3595e+05_rb,1.1871e+05_rb,9.4687e+04_rb,5.8920e+04_rb,1.9962e+02_rb/)
      kbo(:, 5,59,16) = (/ &
     &1.8479e+05_rb,1.6135e+05_rb,1.2650e+05_rb,8.0089e+04_rb,1.8195e+02_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The third index 
!     runs over the g-channel (1 to 16).

      kao_mo3( 1, :, 1) = (/ &
     & 9.31040e-03_rb, 1.01286e-02_rb, 1.10186e-02_rb, 1.19869e-02_rb, 1.30403e-02_rb, &
     & 1.41862e-02_rb, 1.54328e-02_rb, 1.67890e-02_rb, 1.82644e-02_rb, 1.98694e-02_rb, &
     & 2.16154e-02_rb, 2.35149e-02_rb, 2.55813e-02_rb, 2.78293e-02_rb, 3.02749e-02_rb, &
     & 3.29353e-02_rb, 3.58295e-02_rb, 3.89781e-02_rb, 4.24034e-02_rb/)
      kao_mo3( 2, :, 1) = (/ &
     & 1.11200e-02_rb, 1.20461e-02_rb, 1.30493e-02_rb, 1.41360e-02_rb, 1.53133e-02_rb, &
     & 1.65886e-02_rb, 1.79701e-02_rb, 1.94666e-02_rb, 2.10878e-02_rb, 2.28440e-02_rb, &
     & 2.47465e-02_rb, 2.68074e-02_rb, 2.90399e-02_rb, 3.14583e-02_rb, 3.40782e-02_rb, &
     & 3.69162e-02_rb, 3.99907e-02_rb, 4.33211e-02_rb, 4.69289e-02_rb/)
      kao_mo3( 3, :, 1) = (/ &
     & 1.21630e-02_rb, 1.31401e-02_rb, 1.41956e-02_rb, 1.53359e-02_rb, 1.65679e-02_rb, &
     & 1.78988e-02_rb, 1.93366e-02_rb, 2.08899e-02_rb, 2.25680e-02_rb, 2.43808e-02_rb, &
     & 2.63394e-02_rb, 2.84552e-02_rb, 3.07410e-02_rb, 3.32104e-02_rb, 3.58782e-02_rb, &
     & 3.87603e-02_rb, 4.18739e-02_rb, 4.52377e-02_rb, 4.88716e-02_rb/)
      kao_mo3( 4, :, 1) = (/ &
     & 1.26231e-02_rb, 1.36243e-02_rb, 1.47049e-02_rb, 1.58713e-02_rb, 1.71301e-02_rb, &
     & 1.84888e-02_rb, 1.99553e-02_rb, 2.15380e-02_rb, 2.32463e-02_rb, 2.50901e-02_rb, &
     & 2.70801e-02_rb, 2.92280e-02_rb, 3.15463e-02_rb, 3.40484e-02_rb, 3.67489e-02_rb, &
     & 3.96637e-02_rb, 4.28097e-02_rb, 4.62051e-02_rb, 4.98699e-02_rb/)
      kao_mo3( 5, :, 1) = (/ &
     & 1.33345e-02_rb, 1.43736e-02_rb, 1.54938e-02_rb, 1.67012e-02_rb, 1.80027e-02_rb, &
     & 1.94057e-02_rb, 2.09180e-02_rb, 2.25481e-02_rb, 2.43053e-02_rb, 2.61994e-02_rb, &
     & 2.82411e-02_rb, 3.04419e-02_rb, 3.28142e-02_rb, 3.53714e-02_rb, 3.81279e-02_rb, &
     & 4.10992e-02_rb, 4.43021e-02_rb, 4.77545e-02_rb, 5.14760e-02_rb/)
      kao_mo3( 6, :, 1) = (/ &
     & 1.43294e-02_rb, 1.54133e-02_rb, 1.65791e-02_rb, 1.78331e-02_rb, 1.91819e-02_rb, &
     & 2.06328e-02_rb, 2.21935e-02_rb, 2.38721e-02_rb, 2.56778e-02_rb, 2.76200e-02_rb, &
     & 2.97091e-02_rb, 3.19562e-02_rb, 3.43733e-02_rb, 3.69732e-02_rb, 3.97698e-02_rb, &
     & 4.27779e-02_rb, 4.60136e-02_rb, 4.94939e-02_rb, 5.32375e-02_rb/)
      kao_mo3( 7, :, 1) = (/ &
     & 1.48298e-02_rb, 1.59503e-02_rb, 1.71554e-02_rb, 1.84517e-02_rb, 1.98458e-02_rb, &
     & 2.13453e-02_rb, 2.29581e-02_rb, 2.46928e-02_rb, 2.65585e-02_rb, 2.85652e-02_rb, &
     & 3.07235e-02_rb, 3.30449e-02_rb, 3.55417e-02_rb, 3.82272e-02_rb, 4.11155e-02_rb, &
     & 4.42221e-02_rb, 4.75634e-02_rb, 5.11572e-02_rb, 5.50225e-02_rb/)
      kao_mo3( 8, :, 1) = (/ &
     & 1.41792e-02_rb, 1.53141e-02_rb, 1.65398e-02_rb, 1.78637e-02_rb, 1.92935e-02_rb, &
     & 2.08378e-02_rb, 2.25057e-02_rb, 2.43071e-02_rb, 2.62526e-02_rb, 2.83539e-02_rb, &
     & 3.06234e-02_rb, 3.30745e-02_rb, 3.57218e-02_rb, 3.85810e-02_rb, 4.16690e-02_rb, &
     & 4.50042e-02_rb, 4.86064e-02_rb, 5.24969e-02_rb, 5.66988e-02_rb/)
      kao_mo3( 9, :, 1) = (/ &
     & 8.82784e-03_rb, 9.48321e-03_rb, 1.01872e-02_rb, 1.09435e-02_rb, 1.17560e-02_rb, &
     & 1.26287e-02_rb, 1.35662e-02_rb, 1.45734e-02_rb, 1.56553e-02_rb, 1.68175e-02_rb, &
     & 1.80660e-02_rb, 1.94072e-02_rb, 2.08480e-02_rb, 2.23958e-02_rb, 2.40584e-02_rb, &
     & 2.58445e-02_rb, 2.77631e-02_rb, 2.98242e-02_rb, 3.20383e-02_rb/)
      kao_mo3( 1, :, 2) = (/ &
     & 4.28238e-02_rb, 4.51015e-02_rb, 4.75003e-02_rb, 5.00266e-02_rb, 5.26873e-02_rb, &
     & 5.54896e-02_rb, 5.84409e-02_rb, 6.15491e-02_rb, 6.48227e-02_rb, 6.82704e-02_rb, &
     & 7.19014e-02_rb, 7.57256e-02_rb, 7.97532e-02_rb, 8.39949e-02_rb, 8.84623e-02_rb, &
     & 9.31673e-02_rb, 9.81225e-02_rb, 1.03341e-01_rb, 1.08838e-01_rb/)
      kao_mo3( 2, :, 2) = (/ &
     & 4.83672e-02_rb, 5.07219e-02_rb, 5.31911e-02_rb, 5.57806e-02_rb, 5.84962e-02_rb, &
     & 6.13440e-02_rb, 6.43303e-02_rb, 6.74621e-02_rb, 7.07464e-02_rb, 7.41905e-02_rb, &
     & 7.78023e-02_rb, 8.15899e-02_rb, 8.55619e-02_rb, 8.97273e-02_rb, 9.40955e-02_rb, &
     & 9.86763e-02_rb, 1.03480e-01_rb, 1.08518e-01_rb, 1.13801e-01_rb/)
      kao_mo3( 3, :, 2) = (/ &
     & 5.24315e-02_rb, 5.48650e-02_rb, 5.74115e-02_rb, 6.00762e-02_rb, 6.28645e-02_rb, &
     & 6.57822e-02_rb, 6.88354e-02_rb, 7.20302e-02_rb, 7.53734e-02_rb, 7.88717e-02_rb, &
     & 8.25324e-02_rb, 8.63630e-02_rb, 9.03714e-02_rb, 9.45658e-02_rb, 9.89549e-02_rb, &
     & 1.03548e-01_rb, 1.08354e-01_rb, 1.13383e-01_rb, 1.18645e-01_rb/)
      kao_mo3( 4, :, 2) = (/ &
     & 5.65191e-02_rb, 5.90383e-02_rb, 6.16699e-02_rb, 6.44187e-02_rb, 6.72901e-02_rb, &
     & 7.02894e-02_rb, 7.34224e-02_rb, 7.66951e-02_rb, 8.01137e-02_rb, 8.36846e-02_rb, &
     & 8.74147e-02_rb, 9.13111e-02_rb, 9.53812e-02_rb, 9.96326e-02_rb, 1.04074e-01_rb, &
     & 1.08712e-01_rb, 1.13558e-01_rb, 1.18620e-01_rb, 1.23907e-01_rb/)
      kao_mo3( 5, :, 2) = (/ &
     & 6.03171e-02_rb, 6.29114e-02_rb, 6.56172e-02_rb, 6.84394e-02_rb, 7.13830e-02_rb, &
     & 7.44532e-02_rb, 7.76555e-02_rb, 8.09955e-02_rb, 8.44791e-02_rb, 8.81125e-02_rb, &
     & 9.19023e-02_rb, 9.58550e-02_rb, 9.99778e-02_rb, 1.04278e-01_rb, 1.08763e-01_rb, &
     & 1.13441e-01_rb, 1.18320e-01_rb, 1.23409e-01_rb, 1.28717e-01_rb/)
      kao_mo3( 6, :, 2) = (/ &
     & 6.51092e-02_rb, 6.77827e-02_rb, 7.05660e-02_rb, 7.34635e-02_rb, 7.64801e-02_rb, &
     & 7.96204e-02_rb, 8.28898e-02_rb, 8.62934e-02_rb, 8.98367e-02_rb, 9.35255e-02_rb, &
     & 9.73658e-02_rb, 1.01364e-01_rb, 1.05526e-01_rb, 1.09859e-01_rb, 1.14370e-01_rb, &
     & 1.19066e-01_rb, 1.23955e-01_rb, 1.29045e-01_rb, 1.34344e-01_rb/)
      kao_mo3( 7, :, 2) = (/ &
     & 7.09653e-02_rb, 7.37378e-02_rb, 7.66187e-02_rb, 7.96121e-02_rb, 8.27225e-02_rb, &
     & 8.59543e-02_rb, 8.93125e-02_rb, 9.28018e-02_rb, 9.64275e-02_rb, 1.00195e-01_rb, &
     & 1.04109e-01_rb, 1.08177e-01_rb, 1.12403e-01_rb, 1.16795e-01_rb, 1.21358e-01_rb, &
     & 1.26099e-01_rb, 1.31026e-01_rb, 1.36145e-01_rb, 1.41464e-01_rb/)
      kao_mo3( 8, :, 2) = (/ &
     & 7.69193e-02_rb, 7.97926e-02_rb, 8.27733e-02_rb, 8.58653e-02_rb, 8.90728e-02_rb, &
     & 9.24002e-02_rb, 9.58518e-02_rb, 9.94324e-02_rb, 1.03147e-01_rb, 1.07000e-01_rb, &
     & 1.10997e-01_rb, 1.15143e-01_rb, 1.19444e-01_rb, 1.23906e-01_rb, 1.28535e-01_rb, &
     & 1.33336e-01_rb, 1.38317e-01_rb, 1.43484e-01_rb, 1.48844e-01_rb/)
      kao_mo3( 9, :, 2) = (/ &
     & 4.57962e-02_rb, 4.76027e-02_rb, 4.94805e-02_rb, 5.14323e-02_rb, 5.34611e-02_rb, &
     & 5.55700e-02_rb, 5.77620e-02_rb, 6.00405e-02_rb, 6.24089e-02_rb, 6.48707e-02_rb, &
     & 6.74296e-02_rb, 7.00895e-02_rb, 7.28542e-02_rb, 7.57281e-02_rb, 7.87153e-02_rb, &
     & 8.18203e-02_rb, 8.50478e-02_rb, 8.84027e-02_rb, 9.18898e-02_rb/)
      kao_mo3( 1, :, 3) = (/ &
     & 1.12607e-01_rb, 1.16047e-01_rb, 1.19591e-01_rb, 1.23244e-01_rb, 1.27009e-01_rb, &
     & 1.30888e-01_rb, 1.34886e-01_rb, 1.39006e-01_rb, 1.43252e-01_rb, 1.47628e-01_rb, &
     & 1.52137e-01_rb, 1.56785e-01_rb, 1.61574e-01_rb, 1.66509e-01_rb, 1.71595e-01_rb, &
     & 1.76836e-01_rb, 1.82238e-01_rb, 1.87804e-01_rb, 1.93541e-01_rb/)
      kao_mo3( 2, :, 3) = (/ &
     & 1.14531e-01_rb, 1.17850e-01_rb, 1.21266e-01_rb, 1.24781e-01_rb, 1.28397e-01_rb, &
     & 1.32119e-01_rb, 1.35948e-01_rb, 1.39888e-01_rb, 1.43943e-01_rb, 1.48115e-01_rb, &
     & 1.52407e-01_rb, 1.56825e-01_rb, 1.61370e-01_rb, 1.66047e-01_rb, 1.70860e-01_rb, &
     & 1.75812e-01_rb, 1.80907e-01_rb, 1.86150e-01_rb, 1.91546e-01_rb/)
      kao_mo3( 3, :, 3) = (/ &
     & 1.13986e-01_rb, 1.17222e-01_rb, 1.20551e-01_rb, 1.23974e-01_rb, 1.27494e-01_rb, &
     & 1.31114e-01_rb, 1.34837e-01_rb, 1.38666e-01_rb, 1.42604e-01_rb, 1.46653e-01_rb, &
     & 1.50817e-01_rb, 1.55099e-01_rb, 1.59503e-01_rb, 1.64032e-01_rb, 1.68690e-01_rb, &
     & 1.73480e-01_rb, 1.78406e-01_rb, 1.83472e-01_rb, 1.88682e-01_rb/)
      kao_mo3( 4, :, 3) = (/ &
     & 1.13713e-01_rb, 1.16892e-01_rb, 1.20160e-01_rb, 1.23519e-01_rb, 1.26972e-01_rb, &
     & 1.30522e-01_rb, 1.34171e-01_rb, 1.37922e-01_rb, 1.41778e-01_rb, 1.45742e-01_rb, &
     & 1.49817e-01_rb, 1.54005e-01_rb, 1.58311e-01_rb, 1.62737e-01_rb, 1.67287e-01_rb, &
     & 1.71964e-01_rb, 1.76771e-01_rb, 1.81714e-01_rb, 1.86794e-01_rb/)
      kao_mo3( 5, :, 3) = (/ &
     & 1.12321e-01_rb, 1.15413e-01_rb, 1.18591e-01_rb, 1.21856e-01_rb, 1.25211e-01_rb, &
     & 1.28658e-01_rb, 1.32200e-01_rb, 1.35840e-01_rb, 1.39580e-01_rb, 1.43423e-01_rb, &
     & 1.47372e-01_rb, 1.51429e-01_rb, 1.55599e-01_rb, 1.59883e-01_rb, 1.64284e-01_rb, &
     & 1.68808e-01_rb, 1.73455e-01_rb, 1.78231e-01_rb, 1.83138e-01_rb/)
      kao_mo3( 6, :, 3) = (/ &
     & 1.14158e-01_rb, 1.17218e-01_rb, 1.20360e-01_rb, 1.23586e-01_rb, 1.26899e-01_rb, &
     & 1.30300e-01_rb, 1.33793e-01_rb, 1.37379e-01_rb, 1.41061e-01_rb, 1.44842e-01_rb, &
     & 1.48724e-01_rb, 1.52711e-01_rb, 1.56804e-01_rb, 1.61007e-01_rb, 1.65322e-01_rb, &
     & 1.69754e-01_rb, 1.74304e-01_rb, 1.78976e-01_rb, 1.83773e-01_rb/)
      kao_mo3( 7, :, 3) = (/ &
     & 1.21015e-01_rb, 1.23989e-01_rb, 1.27036e-01_rb, 1.30157e-01_rb, 1.33355e-01_rb, &
     & 1.36632e-01_rb, 1.39990e-01_rb, 1.43429e-01_rb, 1.46954e-01_rb, 1.50565e-01_rb, &
     & 1.54264e-01_rb, 1.58055e-01_rb, 1.61939e-01_rb, 1.65918e-01_rb, 1.69995e-01_rb, &
     & 1.74172e-01_rb, 1.78452e-01_rb, 1.82836e-01_rb, 1.87329e-01_rb/)
      kao_mo3( 8, :, 3) = (/ &
     & 1.33952e-01_rb, 1.36939e-01_rb, 1.39992e-01_rb, 1.43114e-01_rb, 1.46305e-01_rb, &
     & 1.49567e-01_rb, 1.52902e-01_rb, 1.56311e-01_rb, 1.59797e-01_rb, 1.63360e-01_rb, &
     & 1.67002e-01_rb, 1.70726e-01_rb, 1.74533e-01_rb, 1.78424e-01_rb, 1.82403e-01_rb, &
     & 1.86470e-01_rb, 1.90627e-01_rb, 1.94878e-01_rb, 1.99223e-01_rb/)
      kao_mo3( 9, :, 3) = (/ &
     & 1.01003e-01_rb, 1.03713e-01_rb, 1.06495e-01_rb, 1.09352e-01_rb, 1.12285e-01_rb, &
     & 1.15297e-01_rb, 1.18390e-01_rb, 1.21566e-01_rb, 1.24827e-01_rb, 1.28176e-01_rb, &
     & 1.31614e-01_rb, 1.35145e-01_rb, 1.38770e-01_rb, 1.42493e-01_rb, 1.46315e-01_rb, &
     & 1.50240e-01_rb, 1.54271e-01_rb, 1.58409e-01_rb, 1.62659e-01_rb/)
      kao_mo3( 1, :, 4) = (/ &
     & 2.35597e-01_rb, 2.37975e-01_rb, 2.40376e-01_rb, 2.42802e-01_rb, 2.45253e-01_rb, &
     & 2.47728e-01_rb, 2.50228e-01_rb, 2.52753e-01_rb, 2.55304e-01_rb, 2.57881e-01_rb, &
     & 2.60483e-01_rb, 2.63112e-01_rb, 2.65767e-01_rb, 2.68450e-01_rb, 2.71159e-01_rb, &
     & 2.73895e-01_rb, 2.76660e-01_rb, 2.79452e-01_rb, 2.82272e-01_rb/)
      kao_mo3( 2, :, 4) = (/ &
     & 2.27965e-01_rb, 2.30334e-01_rb, 2.32728e-01_rb, 2.35146e-01_rb, 2.37590e-01_rb, &
     & 2.40059e-01_rb, 2.42554e-01_rb, 2.45075e-01_rb, 2.47621e-01_rb, 2.50195e-01_rb, &
     & 2.52795e-01_rb, 2.55422e-01_rb, 2.58077e-01_rb, 2.60759e-01_rb, 2.63468e-01_rb, &
     & 2.66206e-01_rb, 2.68973e-01_rb, 2.71768e-01_rb, 2.74593e-01_rb/)
      kao_mo3( 3, :, 4) = (/ &
     & 2.25956e-01_rb, 2.28277e-01_rb, 2.30622e-01_rb, 2.32991e-01_rb, 2.35384e-01_rb, &
     & 2.37802e-01_rb, 2.40244e-01_rb, 2.42712e-01_rb, 2.45205e-01_rb, 2.47724e-01_rb, &
     & 2.50268e-01_rb, 2.52839e-01_rb, 2.55436e-01_rb, 2.58060e-01_rb, 2.60711e-01_rb, &
     & 2.63389e-01_rb, 2.66094e-01_rb, 2.68827e-01_rb, 2.71589e-01_rb/)
      kao_mo3( 4, :, 4) = (/ &
     & 2.28371e-01_rb, 2.30595e-01_rb, 2.32840e-01_rb, 2.35107e-01_rb, 2.37397e-01_rb, &
     & 2.39708e-01_rb, 2.42042e-01_rb, 2.44399e-01_rb, 2.46779e-01_rb, 2.49182e-01_rb, &
     & 2.51608e-01_rb, 2.54058e-01_rb, 2.56532e-01_rb, 2.59030e-01_rb, 2.61552e-01_rb, &
     & 2.64099e-01_rb, 2.66671e-01_rb, 2.69267e-01_rb, 2.71889e-01_rb/)
      kao_mo3( 5, :, 4) = (/ &
     & 2.42563e-01_rb, 2.44620e-01_rb, 2.46695e-01_rb, 2.48787e-01_rb, 2.50897e-01_rb, &
     & 2.53024e-01_rb, 2.55170e-01_rb, 2.57334e-01_rb, 2.59516e-01_rb, 2.61717e-01_rb, &
     & 2.63936e-01_rb, 2.66174e-01_rb, 2.68431e-01_rb, 2.70708e-01_rb, 2.73003e-01_rb, &
     & 2.75318e-01_rb, 2.77653e-01_rb, 2.80008e-01_rb, 2.82382e-01_rb/)
      kao_mo3( 6, :, 4) = (/ &
     & 2.54052e-01_rb, 2.56017e-01_rb, 2.57997e-01_rb, 2.59992e-01_rb, 2.62003e-01_rb, &
     & 2.64029e-01_rb, 2.66071e-01_rb, 2.68129e-01_rb, 2.70203e-01_rb, 2.72293e-01_rb, &
     & 2.74398e-01_rb, 2.76521e-01_rb, 2.78659e-01_rb, 2.80814e-01_rb, 2.82986e-01_rb, &
     & 2.85175e-01_rb, 2.87380e-01_rb, 2.89603e-01_rb, 2.91842e-01_rb/)
      kao_mo3( 7, :, 4) = (/ &
     & 2.54061e-01_rb, 2.55982e-01_rb, 2.57917e-01_rb, 2.59867e-01_rb, 2.61832e-01_rb, &
     & 2.63811e-01_rb, 2.65806e-01_rb, 2.67815e-01_rb, 2.69840e-01_rb, 2.71880e-01_rb, &
     & 2.73936e-01_rb, 2.76007e-01_rb, 2.78093e-01_rb, 2.80196e-01_rb, 2.82314e-01_rb, &
     & 2.84449e-01_rb, 2.86599e-01_rb, 2.88766e-01_rb, 2.90949e-01_rb/)
      kao_mo3( 8, :, 4) = (/ &
     & 2.72482e-01_rb, 2.73916e-01_rb, 2.75358e-01_rb, 2.76807e-01_rb, 2.78264e-01_rb, &
     & 2.79729e-01_rb, 2.81201e-01_rb, 2.82681e-01_rb, 2.84169e-01_rb, 2.85665e-01_rb, &
     & 2.87168e-01_rb, 2.88680e-01_rb, 2.90199e-01_rb, 2.91726e-01_rb, 2.93262e-01_rb, &
     & 2.94805e-01_rb, 2.96357e-01_rb, 2.97917e-01_rb, 2.99485e-01_rb/)
      kao_mo3( 9, :, 4) = (/ &
     & 1.93414e-01_rb, 1.95498e-01_rb, 1.97605e-01_rb, 1.99734e-01_rb, 2.01886e-01_rb, &
     & 2.04062e-01_rb, 2.06261e-01_rb, 2.08483e-01_rb, 2.10730e-01_rb, 2.13001e-01_rb, &
     & 2.15296e-01_rb, 2.17616e-01_rb, 2.19961e-01_rb, 2.22331e-01_rb, 2.24727e-01_rb, &
     & 2.27148e-01_rb, 2.29596e-01_rb, 2.32070e-01_rb, 2.34571e-01_rb/)
      kao_mo3( 1, :, 5) = (/ &
     & 5.30785e-01_rb, 5.30477e-01_rb, 5.30169e-01_rb, 5.29861e-01_rb, 5.29553e-01_rb, &
     & 5.29246e-01_rb, 5.28938e-01_rb, 5.28631e-01_rb, 5.28324e-01_rb, 5.28017e-01_rb, &
     & 5.27711e-01_rb, 5.27404e-01_rb, 5.27098e-01_rb, 5.26792e-01_rb, 5.26486e-01_rb, &
     & 5.26180e-01_rb, 5.25875e-01_rb, 5.25569e-01_rb, 5.25264e-01_rb/)
      kao_mo3( 2, :, 5) = (/ &
     & 5.33406e-01_rb, 5.32997e-01_rb, 5.32587e-01_rb, 5.32178e-01_rb, 5.31769e-01_rb, &
     & 5.31360e-01_rb, 5.30952e-01_rb, 5.30544e-01_rb, 5.30137e-01_rb, 5.29729e-01_rb, &
     & 5.29322e-01_rb, 5.28916e-01_rb, 5.28509e-01_rb, 5.28103e-01_rb, 5.27697e-01_rb, &
     & 5.27292e-01_rb, 5.26887e-01_rb, 5.26482e-01_rb, 5.26077e-01_rb/)
      kao_mo3( 3, :, 5) = (/ &
     & 5.39814e-01_rb, 5.39234e-01_rb, 5.38655e-01_rb, 5.38077e-01_rb, 5.37499e-01_rb, &
     & 5.36922e-01_rb, 5.36345e-01_rb, 5.35769e-01_rb, 5.35194e-01_rb, 5.34620e-01_rb, &
     & 5.34045e-01_rb, 5.33472e-01_rb, 5.32899e-01_rb, 5.32327e-01_rb, 5.31756e-01_rb, &
     & 5.31185e-01_rb, 5.30614e-01_rb, 5.30045e-01_rb, 5.29475e-01_rb/)
      kao_mo3( 4, :, 5) = (/ &
     & 5.39054e-01_rb, 5.38348e-01_rb, 5.37643e-01_rb, 5.36938e-01_rb, 5.36235e-01_rb, &
     & 5.35532e-01_rb, 5.34831e-01_rb, 5.34130e-01_rb, 5.33431e-01_rb, 5.32732e-01_rb, &
     & 5.32034e-01_rb, 5.31337e-01_rb, 5.30641e-01_rb, 5.29946e-01_rb, 5.29252e-01_rb, &
     & 5.28559e-01_rb, 5.27866e-01_rb, 5.27175e-01_rb, 5.26484e-01_rb/)
      kao_mo3( 5, :, 5) = (/ &
     & 5.29240e-01_rb, 5.28475e-01_rb, 5.27711e-01_rb, 5.26949e-01_rb, 5.26187e-01_rb, &
     & 5.25427e-01_rb, 5.24668e-01_rb, 5.23909e-01_rb, 5.23152e-01_rb, 5.22396e-01_rb, &
     & 5.21641e-01_rb, 5.20888e-01_rb, 5.20135e-01_rb, 5.19383e-01_rb, 5.18633e-01_rb, &
     & 5.17883e-01_rb, 5.17135e-01_rb, 5.16388e-01_rb, 5.15642e-01_rb/)
      kao_mo3( 6, :, 5) = (/ &
     & 5.21746e-01_rb, 5.20815e-01_rb, 5.19886e-01_rb, 5.18958e-01_rb, 5.18032e-01_rb, &
     & 5.17107e-01_rb, 5.16184e-01_rb, 5.15263e-01_rb, 5.14343e-01_rb, 5.13425e-01_rb, &
     & 5.12509e-01_rb, 5.11594e-01_rb, 5.10681e-01_rb, 5.09770e-01_rb, 5.08860e-01_rb, &
     & 5.07952e-01_rb, 5.07045e-01_rb, 5.06140e-01_rb, 5.05237e-01_rb/)
      kao_mo3( 7, :, 5) = (/ &
     & 5.26752e-01_rb, 5.25550e-01_rb, 5.24352e-01_rb, 5.23156e-01_rb, 5.21963e-01_rb, &
     & 5.20772e-01_rb, 5.19584e-01_rb, 5.18399e-01_rb, 5.17217e-01_rb, 5.16038e-01_rb, &
     & 5.14861e-01_rb, 5.13686e-01_rb, 5.12515e-01_rb, 5.11346e-01_rb, 5.10180e-01_rb, &
     & 5.09016e-01_rb, 5.07855e-01_rb, 5.06697e-01_rb, 5.05541e-01_rb/)
      kao_mo3( 8, :, 5) = (/ &
     & 5.23581e-01_rb, 5.22513e-01_rb, 5.21446e-01_rb, 5.20382e-01_rb, 5.19320e-01_rb, &
     & 5.18260e-01_rb, 5.17203e-01_rb, 5.16147e-01_rb, 5.15094e-01_rb, 5.14042e-01_rb, &
     & 5.12993e-01_rb, 5.11946e-01_rb, 5.10901e-01_rb, 5.09859e-01_rb, 5.08818e-01_rb, &
     & 5.07780e-01_rb, 5.06743e-01_rb, 5.05709e-01_rb, 5.04677e-01_rb/)
      kao_mo3( 9, :, 5) = (/ &
     & 3.80393e-01_rb, 3.80680e-01_rb, 3.80967e-01_rb, 3.81254e-01_rb, 3.81542e-01_rb, &
     & 3.81829e-01_rb, 3.82117e-01_rb, 3.82405e-01_rb, 3.82693e-01_rb, 3.82982e-01_rb, &
     & 3.83271e-01_rb, 3.83559e-01_rb, 3.83849e-01_rb, 3.84138e-01_rb, 3.84428e-01_rb, &
     & 3.84717e-01_rb, 3.85007e-01_rb, 3.85298e-01_rb, 3.85588e-01_rb/)
      kao_mo3( 1, :, 6) = (/ &
     & 6.14818e-01_rb, 6.10664e-01_rb, 6.06539e-01_rb, 6.02441e-01_rb, 5.98372e-01_rb, &
     & 5.94330e-01_rb, 5.90315e-01_rb, 5.86327e-01_rb, 5.82366e-01_rb, 5.78432e-01_rb, &
     & 5.74524e-01_rb, 5.70643e-01_rb, 5.66788e-01_rb, 5.62959e-01_rb, 5.59156e-01_rb, &
     & 5.55379e-01_rb, 5.51627e-01_rb, 5.47901e-01_rb, 5.44199e-01_rb/)
      kao_mo3( 2, :, 6) = (/ &
     & 6.10199e-01_rb, 6.06143e-01_rb, 6.02114e-01_rb, 5.98112e-01_rb, 5.94136e-01_rb, &
     & 5.90187e-01_rb, 5.86264e-01_rb, 5.82367e-01_rb, 5.78496e-01_rb, 5.74651e-01_rb, &
     & 5.70831e-01_rb, 5.67037e-01_rb, 5.63268e-01_rb, 5.59524e-01_rb, 5.55805e-01_rb, &
     & 5.52110e-01_rb, 5.48440e-01_rb, 5.44795e-01_rb, 5.41174e-01_rb/)
      kao_mo3( 3, :, 6) = (/ &
     & 6.02949e-01_rb, 5.99057e-01_rb, 5.95190e-01_rb, 5.91348e-01_rb, 5.87531e-01_rb, &
     & 5.83739e-01_rb, 5.79971e-01_rb, 5.76227e-01_rb, 5.72508e-01_rb, 5.68812e-01_rb, &
     & 5.65140e-01_rb, 5.61493e-01_rb, 5.57868e-01_rb, 5.54267e-01_rb, 5.50690e-01_rb, &
     & 5.47135e-01_rb, 5.43603e-01_rb, 5.40094e-01_rb, 5.36608e-01_rb/)
      kao_mo3( 4, :, 6) = (/ &
     & 6.05047e-01_rb, 6.01155e-01_rb, 5.97289e-01_rb, 5.93448e-01_rb, 5.89631e-01_rb, &
     & 5.85838e-01_rb, 5.82071e-01_rb, 5.78327e-01_rb, 5.74607e-01_rb, 5.70912e-01_rb, &
     & 5.67240e-01_rb, 5.63592e-01_rb, 5.59967e-01_rb, 5.56365e-01_rb, 5.52787e-01_rb, &
     & 5.49232e-01_rb, 5.45699e-01_rb, 5.42190e-01_rb, 5.38703e-01_rb/)
      kao_mo3( 5, :, 6) = (/ &
     & 6.03593e-01_rb, 5.99867e-01_rb, 5.96164e-01_rb, 5.92483e-01_rb, 5.88825e-01_rb, &
     & 5.85190e-01_rb, 5.81577e-01_rb, 5.77987e-01_rb, 5.74419e-01_rb, 5.70872e-01_rb, &
     & 5.67348e-01_rb, 5.63846e-01_rb, 5.60365e-01_rb, 5.56905e-01_rb, 5.53467e-01_rb, &
     & 5.50050e-01_rb, 5.46654e-01_rb, 5.43279e-01_rb, 5.39926e-01_rb/)
      kao_mo3( 6, :, 6) = (/ &
     & 6.03940e-01_rb, 6.00224e-01_rb, 5.96531e-01_rb, 5.92861e-01_rb, 5.89213e-01_rb, &
     & 5.85588e-01_rb, 5.81985e-01_rb, 5.78404e-01_rb, 5.74845e-01_rb, 5.71308e-01_rb, &
     & 5.67793e-01_rb, 5.64299e-01_rb, 5.60827e-01_rb, 5.57377e-01_rb, 5.53947e-01_rb, &
     & 5.50539e-01_rb, 5.47151e-01_rb, 5.43785e-01_rb, 5.40439e-01_rb/)
      kao_mo3( 7, :, 6) = (/ &
     & 6.06242e-01_rb, 6.02257e-01_rb, 5.98299e-01_rb, 5.94367e-01_rb, 5.90461e-01_rb, &
     & 5.86580e-01_rb, 5.82725e-01_rb, 5.78895e-01_rb, 5.75090e-01_rb, 5.71311e-01_rb, &
     & 5.67556e-01_rb, 5.63826e-01_rb, 5.60120e-01_rb, 5.56439e-01_rb, 5.52782e-01_rb, &
     & 5.49149e-01_rb, 5.45540e-01_rb, 5.41954e-01_rb, 5.38393e-01_rb/)
      kao_mo3( 8, :, 6) = (/ &
     & 6.11929e-01_rb, 6.07173e-01_rb, 6.02454e-01_rb, 5.97773e-01_rb, 5.93127e-01_rb, &
     & 5.88518e-01_rb, 5.83944e-01_rb, 5.79406e-01_rb, 5.74903e-01_rb, 5.70436e-01_rb, &
     & 5.66002e-01_rb, 5.61604e-01_rb, 5.57239e-01_rb, 5.52909e-01_rb, 5.48612e-01_rb, &
     & 5.44349e-01_rb, 5.40118e-01_rb, 5.35921e-01_rb, 5.31756e-01_rb/)
      kao_mo3( 9, :, 6) = (/ &
     & 6.21189e-01_rb, 6.17338e-01_rb, 6.13511e-01_rb, 6.09707e-01_rb, 6.05927e-01_rb, &
     & 6.02170e-01_rb, 5.98437e-01_rb, 5.94726e-01_rb, 5.91039e-01_rb, 5.87375e-01_rb, &
     & 5.83733e-01_rb, 5.80114e-01_rb, 5.76517e-01_rb, 5.72943e-01_rb, 5.69390e-01_rb, &
     & 5.65860e-01_rb, 5.62352e-01_rb, 5.58865e-01_rb, 5.55400e-01_rb/)
      kao_mo3( 1, :, 7) = (/ &
     & 7.41310e-01_rb, 7.30108e-01_rb, 7.19075e-01_rb, 7.08209e-01_rb, 6.97507e-01_rb, &
     & 6.86967e-01_rb, 6.76586e-01_rb, 6.66362e-01_rb, 6.56292e-01_rb, 6.46374e-01_rb, &
     & 6.36607e-01_rb, 6.26987e-01_rb, 6.17512e-01_rb, 6.08181e-01_rb, 5.98990e-01_rb, &
     & 5.89939e-01_rb, 5.81024e-01_rb, 5.72244e-01_rb, 5.63597e-01_rb/)
      kao_mo3( 2, :, 7) = (/ &
     & 7.38780e-01_rb, 7.27631e-01_rb, 7.16651e-01_rb, 7.05836e-01_rb, 6.95185e-01_rb, &
     & 6.84695e-01_rb, 6.74362e-01_rb, 6.64186e-01_rb, 6.54163e-01_rb, 6.44292e-01_rb, &
     & 6.34569e-01_rb, 6.24993e-01_rb, 6.15562e-01_rb, 6.06273e-01_rb, 5.97124e-01_rb, &
     & 5.88113e-01_rb, 5.79238e-01_rb, 5.70498e-01_rb, 5.61889e-01_rb/)
      kao_mo3( 3, :, 7) = (/ &
     & 7.33846e-01_rb, 7.22799e-01_rb, 7.11919e-01_rb, 7.01203e-01_rb, 6.90648e-01_rb, &
     & 6.80252e-01_rb, 6.70012e-01_rb, 6.59927e-01_rb, 6.49993e-01_rb, 6.40209e-01_rb, &
     & 6.30572e-01_rb, 6.21080e-01_rb, 6.11731e-01_rb, 6.02523e-01_rb, 5.93453e-01_rb, &
     & 5.84520e-01_rb, 5.75721e-01_rb, 5.67055e-01_rb, 5.58519e-01_rb/)
      kao_mo3( 4, :, 7) = (/ &
     & 7.21218e-01_rb, 7.10492e-01_rb, 6.99926e-01_rb, 6.89517e-01_rb, 6.79262e-01_rb, &
     & 6.69160e-01_rb, 6.59209e-01_rb, 6.49405e-01_rb, 6.39747e-01_rb, 6.30233e-01_rb, &
     & 6.20860e-01_rb, 6.11627e-01_rb, 6.02531e-01_rb, 5.93570e-01_rb, 5.84743e-01_rb, &
     & 5.76047e-01_rb, 5.67480e-01_rb, 5.59040e-01_rb, 5.50726e-01_rb/)
      kao_mo3( 5, :, 7) = (/ &
     & 7.10588e-01_rb, 7.00014e-01_rb, 6.89596e-01_rb, 6.79334e-01_rb, 6.69225e-01_rb, &
     & 6.59266e-01_rb, 6.49455e-01_rb, 6.39790e-01_rb, 6.30269e-01_rb, 6.20889e-01_rb, &
     & 6.11650e-01_rb, 6.02547e-01_rb, 5.93581e-01_rb, 5.84747e-01_rb, 5.76045e-01_rb, &
     & 5.67473e-01_rb, 5.59028e-01_rb, 5.50709e-01_rb, 5.42513e-01_rb/)
      kao_mo3( 6, :, 7) = (/ &
     & 6.98166e-01_rb, 6.87706e-01_rb, 6.77402e-01_rb, 6.67253e-01_rb, 6.57256e-01_rb, &
     & 6.47408e-01_rb, 6.37708e-01_rb, 6.28154e-01_rb, 6.18742e-01_rb, 6.09472e-01_rb, &
     & 6.00340e-01_rb, 5.91346e-01_rb, 5.82486e-01_rb, 5.73758e-01_rb, 5.65162e-01_rb, &
     & 5.56694e-01_rb, 5.48353e-01_rb, 5.40138e-01_rb, 5.32045e-01_rb/)
      kao_mo3( 7, :, 7) = (/ &
     & 6.76974e-01_rb, 6.67034e-01_rb, 6.57240e-01_rb, 6.47590e-01_rb, 6.38081e-01_rb, &
     & 6.28712e-01_rb, 6.19481e-01_rb, 6.10385e-01_rb, 6.01422e-01_rb, 5.92592e-01_rb, &
     & 5.83891e-01_rb, 5.75317e-01_rb, 5.66870e-01_rb, 5.58547e-01_rb, 5.50345e-01_rb, &
     & 5.42265e-01_rb, 5.34303e-01_rb, 5.26457e-01_rb, 5.18727e-01_rb/)
      kao_mo3( 8, :, 7) = (/ &
     & 6.30061e-01_rb, 6.21017e-01_rb, 6.12102e-01_rb, 6.03316e-01_rb, 5.94656e-01_rb, &
     & 5.86120e-01_rb, 5.77706e-01_rb, 5.69414e-01_rb, 5.61240e-01_rb, 5.53184e-01_rb, &
     & 5.45243e-01_rb, 5.37416e-01_rb, 5.29702e-01_rb, 5.22098e-01_rb, 5.14604e-01_rb, &
     & 5.07217e-01_rb, 4.99936e-01_rb, 4.92760e-01_rb, 4.85687e-01_rb/)
      kao_mo3( 9, :, 7) = (/ &
     & 8.97633e-01_rb, 8.87307e-01_rb, 8.77100e-01_rb, 8.67010e-01_rb, 8.57036e-01_rb, &
     & 8.47176e-01_rb, 8.37431e-01_rb, 8.27797e-01_rb, 8.18274e-01_rb, 8.08861e-01_rb, &
     & 7.99555e-01_rb, 7.90357e-01_rb, 7.81265e-01_rb, 7.72278e-01_rb, 7.63393e-01_rb, &
     & 7.54611e-01_rb, 7.45930e-01_rb, 7.37349e-01_rb, 7.28867e-01_rb/)
      kao_mo3( 1, :, 8) = (/ &
     & 4.87356e-01_rb, 4.80743e-01_rb, 4.74220e-01_rb, 4.67785e-01_rb, 4.61437e-01_rb, &
     & 4.55176e-01_rb, 4.49000e-01_rb, 4.42907e-01_rb, 4.36897e-01_rb, 4.30969e-01_rb, &
     & 4.25121e-01_rb, 4.19353e-01_rb, 4.13663e-01_rb, 4.08049e-01_rb, 4.02513e-01_rb, &
     & 3.97051e-01_rb, 3.91663e-01_rb, 3.86349e-01_rb, 3.81106e-01_rb/)
      kao_mo3( 2, :, 8) = (/ &
     & 4.86776e-01_rb, 4.80157e-01_rb, 4.73627e-01_rb, 4.67187e-01_rb, 4.60834e-01_rb, &
     & 4.54567e-01_rb, 4.48386e-01_rb, 4.42289e-01_rb, 4.36274e-01_rb, 4.30342e-01_rb, &
     & 4.24490e-01_rb, 4.18718e-01_rb, 4.13024e-01_rb, 4.07407e-01_rb, 4.01867e-01_rb, &
     & 3.96403e-01_rb, 3.91012e-01_rb, 3.85695e-01_rb, 3.80450e-01_rb/)
      kao_mo3( 3, :, 8) = (/ &
     & 4.86111e-01_rb, 4.79496e-01_rb, 4.72972e-01_rb, 4.66536e-01_rb, 4.60188e-01_rb, &
     & 4.53926e-01_rb, 4.47750e-01_rb, 4.41657e-01_rb, 4.35648e-01_rb, 4.29720e-01_rb, &
     & 4.23873e-01_rb, 4.18105e-01_rb, 4.12416e-01_rb, 4.06804e-01_rb, 4.01269e-01_rb, &
     & 3.95809e-01_rb, 3.90423e-01_rb, 3.85111e-01_rb, 3.79871e-01_rb/)
      kao_mo3( 4, :, 8) = (/ &
     & 4.85501e-01_rb, 4.78880e-01_rb, 4.72350e-01_rb, 4.65908e-01_rb, 4.59554e-01_rb, &
     & 4.53288e-01_rb, 4.47106e-01_rb, 4.41009e-01_rb, 4.34995e-01_rb, 4.29063e-01_rb, &
     & 4.23211e-01_rb, 4.17440e-01_rb, 4.11747e-01_rb, 4.06132e-01_rb, 4.00594e-01_rb, &
     & 3.95131e-01_rb, 3.89743e-01_rb, 3.84428e-01_rb, 3.79185e-01_rb/)
      kao_mo3( 5, :, 8) = (/ &
     & 4.83679e-01_rb, 4.77140e-01_rb, 4.70691e-01_rb, 4.64328e-01_rb, 4.58051e-01_rb, &
     & 4.51859e-01_rb, 4.45751e-01_rb, 4.39726e-01_rb, 4.33781e-01_rb, 4.27918e-01_rb, &
     & 4.22133e-01_rb, 4.16427e-01_rb, 4.10798e-01_rb, 4.05245e-01_rb, 3.99767e-01_rb, &
     & 3.94363e-01_rb, 3.89032e-01_rb, 3.83773e-01_rb, 3.78585e-01_rb/)
      kao_mo3( 6, :, 8) = (/ &
     & 4.72120e-01_rb, 4.65834e-01_rb, 4.59630e-01_rb, 4.53510e-01_rb, 4.47471e-01_rb, &
     & 4.41513e-01_rb, 4.35633e-01_rb, 4.29833e-01_rb, 4.24109e-01_rb, 4.18461e-01_rb, &
     & 4.12889e-01_rb, 4.07391e-01_rb, 4.01966e-01_rb, 3.96614e-01_rb, 3.91332e-01_rb, &
     & 3.86122e-01_rb, 3.80980e-01_rb, 3.75907e-01_rb, 3.70901e-01_rb/)
      kao_mo3( 7, :, 8) = (/ &
     & 4.58683e-01_rb, 4.52758e-01_rb, 4.46909e-01_rb, 4.41135e-01_rb, 4.35437e-01_rb, &
     & 4.29812e-01_rb, 4.24259e-01_rb, 4.18779e-01_rb, 4.13369e-01_rb, 4.08029e-01_rb, &
     & 4.02758e-01_rb, 3.97555e-01_rb, 3.92419e-01_rb, 3.87350e-01_rb, 3.82346e-01_rb, &
     & 3.77406e-01_rb, 3.72531e-01_rb, 3.67719e-01_rb, 3.62968e-01_rb/)
      kao_mo3( 8, :, 8) = (/ &
     & 4.56091e-01_rb, 4.50481e-01_rb, 4.44940e-01_rb, 4.39467e-01_rb, 4.34062e-01_rb, &
     & 4.28722e-01_rb, 4.23449e-01_rb, 4.18240e-01_rb, 4.13096e-01_rb, 4.08015e-01_rb, &
     & 4.02996e-01_rb, 3.98039e-01_rb, 3.93143e-01_rb, 3.88307e-01_rb, 3.83531e-01_rb, &
     & 3.78813e-01_rb, 3.74154e-01_rb, 3.69552e-01_rb, 3.65006e-01_rb/)
      kao_mo3( 9, :, 8) = (/ &
     & 9.11213e-01_rb, 9.03270e-01_rb, 8.95396e-01_rb, 8.87591e-01_rb, 8.79855e-01_rb, &
     & 8.72185e-01_rb, 8.64583e-01_rb, 8.57046e-01_rb, 8.49576e-01_rb, 8.42170e-01_rb, &
     & 8.34829e-01_rb, 8.27552e-01_rb, 8.20339e-01_rb, 8.13188e-01_rb, 8.06100e-01_rb, &
     & 7.99073e-01_rb, 7.92108e-01_rb, 7.85204e-01_rb, 7.78359e-01_rb/)
      kao_mo3( 1, :, 9) = (/ &
     & 5.56194e-01_rb, 5.48595e-01_rb, 5.41100e-01_rb, 5.33707e-01_rb, 5.26415e-01_rb, &
     & 5.19223e-01_rb, 5.12129e-01_rb, 5.05132e-01_rb, 4.98231e-01_rb, 4.91424e-01_rb, &
     & 4.84710e-01_rb, 4.78087e-01_rb, 4.71556e-01_rb, 4.65113e-01_rb, 4.58758e-01_rb, &
     & 4.52491e-01_rb, 4.46309e-01_rb, 4.40211e-01_rb, 4.34197e-01_rb/)
      kao_mo3( 2, :, 9) = (/ &
     & 5.56174e-01_rb, 5.48575e-01_rb, 5.41079e-01_rb, 5.33687e-01_rb, 5.26395e-01_rb, &
     & 5.19203e-01_rb, 5.12109e-01_rb, 5.05112e-01_rb, 4.98211e-01_rb, 4.91404e-01_rb, &
     & 4.84690e-01_rb, 4.78068e-01_rb, 4.71536e-01_rb, 4.65093e-01_rb, 4.58739e-01_rb, &
     & 4.52471e-01_rb, 4.46289e-01_rb, 4.40191e-01_rb, 4.34177e-01_rb/)
      kao_mo3( 3, :, 9) = (/ &
     & 5.55996e-01_rb, 5.48403e-01_rb, 5.40913e-01_rb, 5.33526e-01_rb, 5.26239e-01_rb, &
     & 5.19052e-01_rb, 5.11963e-01_rb, 5.04971e-01_rb, 4.98074e-01_rb, 4.91272e-01_rb, &
     & 4.84562e-01_rb, 4.77944e-01_rb, 4.71417e-01_rb, 4.64978e-01_rb, 4.58628e-01_rb, &
     & 4.52364e-01_rb, 4.46186e-01_rb, 4.40092e-01_rb, 4.34081e-01_rb/)
      kao_mo3( 4, :, 9) = (/ &
     & 5.55859e-01_rb, 5.48271e-01_rb, 5.40786e-01_rb, 5.33404e-01_rb, 5.26123e-01_rb, &
     & 5.18941e-01_rb, 5.11856e-01_rb, 5.04869e-01_rb, 4.97977e-01_rb, 4.91179e-01_rb, &
     & 4.84474e-01_rb, 4.77861e-01_rb, 4.71337e-01_rb, 4.64903e-01_rb, 4.58557e-01_rb, &
     & 4.52297e-01_rb, 4.46123e-01_rb, 4.40033e-01_rb, 4.34026e-01_rb/)
      kao_mo3( 5, :, 9) = (/ &
     & 5.54550e-01_rb, 5.46921e-01_rb, 5.39397e-01_rb, 5.31976e-01_rb, 5.24657e-01_rb, &
     & 5.17439e-01_rb, 5.10320e-01_rb, 5.03300e-01_rb, 4.96376e-01_rb, 4.89547e-01_rb, &
     & 4.82812e-01_rb, 4.76170e-01_rb, 4.69619e-01_rb, 4.63158e-01_rb, 4.56786e-01_rb, &
     & 4.50502e-01_rb, 4.44304e-01_rb, 4.38192e-01_rb, 4.32163e-01_rb/)
      kao_mo3( 6, :, 9) = (/ &
     & 5.53514e-01_rb, 5.45883e-01_rb, 5.38358e-01_rb, 5.30937e-01_rb, 5.23618e-01_rb, &
     & 5.16399e-01_rb, 5.09280e-01_rb, 5.02260e-01_rb, 4.95336e-01_rb, 4.88507e-01_rb, &
     & 4.81773e-01_rb, 4.75132e-01_rb, 4.68582e-01_rb, 4.62122e-01_rb, 4.55752e-01_rb, &
     & 4.49469e-01_rb, 4.43273e-01_rb, 4.37162e-01_rb, 4.31136e-01_rb/)
      kao_mo3( 7, :, 9) = (/ &
     & 5.49865e-01_rb, 5.42303e-01_rb, 5.34846e-01_rb, 5.27491e-01_rb, 5.20237e-01_rb, &
     & 5.13084e-01_rb, 5.06028e-01_rb, 4.99070e-01_rb, 4.92207e-01_rb, 4.85438e-01_rb, &
     & 4.78763e-01_rb, 4.72179e-01_rb, 4.65686e-01_rb, 4.59282e-01_rb, 4.52967e-01_rb, &
     & 4.46738e-01_rb, 4.40595e-01_rb, 4.34536e-01_rb, 4.28561e-01_rb/)
      kao_mo3( 8, :, 9) = (/ &
     & 5.25435e-01_rb, 5.18437e-01_rb, 5.11533e-01_rb, 5.04721e-01_rb, 4.97999e-01_rb, &
     & 4.91367e-01_rb, 4.84823e-01_rb, 4.78366e-01_rb, 4.71996e-01_rb, 4.65710e-01_rb, &
     & 4.59508e-01_rb, 4.53388e-01_rb, 4.47350e-01_rb, 4.41393e-01_rb, 4.35515e-01_rb, &
     & 4.29715e-01_rb, 4.23992e-01_rb, 4.18345e-01_rb, 4.12774e-01_rb/)
      kao_mo3( 9, :, 9) = (/ &
     & 3.48228e-01_rb, 3.45949e-01_rb, 3.43686e-01_rb, 3.41437e-01_rb, 3.39203e-01_rb, &
     & 3.36983e-01_rb, 3.34778e-01_rb, 3.32588e-01_rb, 3.30412e-01_rb, 3.28250e-01_rb, &
     & 3.26102e-01_rb, 3.23968e-01_rb, 3.21848e-01_rb, 3.19742e-01_rb, 3.17650e-01_rb, &
     & 3.15572e-01_rb, 3.13507e-01_rb, 3.11456e-01_rb, 3.09418e-01_rb/)
      kao_mo3( 1, :,10) = (/ &
     & 8.34107e-01_rb, 8.27276e-01_rb, 8.20501e-01_rb, 8.13781e-01_rb, 8.07117e-01_rb, &
     & 8.00507e-01_rb, 7.93951e-01_rb, 7.87449e-01_rb, 7.81000e-01_rb, 7.74604e-01_rb, &
     & 7.68260e-01_rb, 7.61968e-01_rb, 7.55728e-01_rb, 7.49539e-01_rb, 7.43400e-01_rb, &
     & 7.37312e-01_rb, 7.31274e-01_rb, 7.25285e-01_rb, 7.19345e-01_rb/)
      kao_mo3( 2, :,10) = (/ &
     & 8.32838e-01_rb, 8.26022e-01_rb, 8.19263e-01_rb, 8.12558e-01_rb, 8.05908e-01_rb, &
     & 7.99313e-01_rb, 7.92772e-01_rb, 7.86284e-01_rb, 7.79849e-01_rb, 7.73467e-01_rb, &
     & 7.67137e-01_rb, 7.60859e-01_rb, 7.54633e-01_rb, 7.48457e-01_rb, 7.42332e-01_rb, &
     & 7.36257e-01_rb, 7.30232e-01_rb, 7.24256e-01_rb, 7.18329e-01_rb/)
      kao_mo3( 3, :,10) = (/ &
     & 8.31167e-01_rb, 8.24361e-01_rb, 8.17611e-01_rb, 8.10916e-01_rb, 8.04276e-01_rb, &
     & 7.97691e-01_rb, 7.91159e-01_rb, 7.84681e-01_rb, 7.78256e-01_rb, 7.71883e-01_rb, &
     & 7.65563e-01_rb, 7.59294e-01_rb, 7.53077e-01_rb, 7.46910e-01_rb, 7.40795e-01_rb, &
     & 7.34729e-01_rb, 7.28713e-01_rb, 7.22746e-01_rb, 7.16828e-01_rb/)
      kao_mo3( 4, :,10) = (/ &
     & 8.29026e-01_rb, 8.22246e-01_rb, 8.15521e-01_rb, 8.08851e-01_rb, 8.02236e-01_rb, &
     & 7.95675e-01_rb, 7.89167e-01_rb, 7.82713e-01_rb, 7.76312e-01_rb, 7.69962e-01_rb, &
     & 7.63665e-01_rb, 7.57419e-01_rb, 7.51225e-01_rb, 7.45081e-01_rb, 7.38987e-01_rb, &
     & 7.32943e-01_rb, 7.26949e-01_rb, 7.21003e-01_rb, 7.15107e-01_rb/)
      kao_mo3( 5, :,10) = (/ &
     & 8.26226e-01_rb, 8.19471e-01_rb, 8.12771e-01_rb, 8.06126e-01_rb, 7.99536e-01_rb, &
     & 7.92999e-01_rb, 7.86515e-01_rb, 7.80085e-01_rb, 7.73707e-01_rb, 7.67382e-01_rb, &
     & 7.61108e-01_rb, 7.54885e-01_rb, 7.48714e-01_rb, 7.42592e-01_rb, 7.36521e-01_rb, &
     & 7.30500e-01_rb, 7.24527e-01_rb, 7.18604e-01_rb, 7.12729e-01_rb/)
      kao_mo3( 6, :,10) = (/ &
     & 8.33246e-01_rb, 8.26510e-01_rb, 8.19828e-01_rb, 8.13200e-01_rb, 8.06626e-01_rb, &
     & 8.00105e-01_rb, 7.93637e-01_rb, 7.87221e-01_rb, 7.80856e-01_rb, 7.74544e-01_rb, &
     & 7.68282e-01_rb, 7.62071e-01_rb, 7.55910e-01_rb, 7.49799e-01_rb, 7.43737e-01_rb, &
     & 7.37725e-01_rb, 7.31760e-01_rb, 7.25845e-01_rb, 7.19977e-01_rb/)
      kao_mo3( 7, :,10) = (/ &
     & 8.45693e-01_rb, 8.38967e-01_rb, 8.32295e-01_rb, 8.25675e-01_rb, 8.19108e-01_rb, &
     & 8.12594e-01_rb, 8.06131e-01_rb, 7.99719e-01_rb, 7.93359e-01_rb, 7.87049e-01_rb, &
     & 7.80789e-01_rb, 7.74579e-01_rb, 7.68419e-01_rb, 7.62307e-01_rb, 7.56244e-01_rb, &
     & 7.50230e-01_rb, 7.44263e-01_rb, 7.38343e-01_rb, 7.32471e-01_rb/)
      kao_mo3( 8, :,10) = (/ &
     & 8.32139e-01_rb, 8.25565e-01_rb, 8.19044e-01_rb, 8.12574e-01_rb, 8.06156e-01_rb, &
     & 7.99788e-01_rb, 7.93470e-01_rb, 7.87202e-01_rb, 7.80984e-01_rb, 7.74815e-01_rb, &
     & 7.68694e-01_rb, 7.62622e-01_rb, 7.56598e-01_rb, 7.50622e-01_rb, 7.44692e-01_rb, &
     & 7.38810e-01_rb, 7.32974e-01_rb, 7.27184e-01_rb, 7.21440e-01_rb/)
      kao_mo3( 9, :,10) = (/ &
     & 2.34258e-01_rb, 2.35247e-01_rb, 2.36239e-01_rb, 2.37236e-01_rb, 2.38237e-01_rb, &
     & 2.39242e-01_rb, 2.40252e-01_rb, 2.41265e-01_rb, 2.42283e-01_rb, 2.43306e-01_rb, &
     & 2.44332e-01_rb, 2.45363e-01_rb, 2.46398e-01_rb, 2.47438e-01_rb, 2.48482e-01_rb, &
     & 2.49531e-01_rb, 2.50583e-01_rb, 2.51641e-01_rb, 2.52702e-01_rb/)
      kao_mo3( 1, :,11) = (/ &
     & 8.31308e-01_rb, 8.22153e-01_rb, 8.13098e-01_rb, 8.04143e-01_rb, 7.95287e-01_rb, &
     & 7.86528e-01_rb, 7.77866e-01_rb, 7.69299e-01_rb, 7.60827e-01_rb, 7.52448e-01_rb, &
     & 7.44161e-01_rb, 7.35965e-01_rb, 7.27860e-01_rb, 7.19844e-01_rb, 7.11916e-01_rb, &
     & 7.04075e-01_rb, 6.96321e-01_rb, 6.88652e-01_rb, 6.81068e-01_rb/)
      kao_mo3( 2, :,11) = (/ &
     & 8.31577e-01_rb, 8.22400e-01_rb, 8.13324e-01_rb, 8.04349e-01_rb, 7.95472e-01_rb, &
     & 7.86693e-01_rb, 7.78011e-01_rb, 7.69425e-01_rb, 7.60934e-01_rb, 7.52537e-01_rb, &
     & 7.44232e-01_rb, 7.36019e-01_rb, 7.27896e-01_rb, 7.19863e-01_rb, 7.11919e-01_rb, &
     & 7.04062e-01_rb, 6.96292e-01_rb, 6.88608e-01_rb, 6.81009e-01_rb/)
      kao_mo3( 3, :,11) = (/ &
     & 8.31578e-01_rb, 8.22422e-01_rb, 8.13368e-01_rb, 8.04413e-01_rb, 7.95557e-01_rb, &
     & 7.86798e-01_rb, 7.78136e-01_rb, 7.69569e-01_rb, 7.61097e-01_rb, 7.52717e-01_rb, &
     & 7.44430e-01_rb, 7.36235e-01_rb, 7.28129e-01_rb, 7.20113e-01_rb, 7.12185e-01_rb, &
     & 7.04344e-01_rb, 6.96589e-01_rb, 6.88920e-01_rb, 6.81336e-01_rb/)
      kao_mo3( 4, :,11) = (/ &
     & 8.31261e-01_rb, 8.22111e-01_rb, 8.13062e-01_rb, 8.04112e-01_rb, 7.95261e-01_rb, &
     & 7.86507e-01_rb, 7.77850e-01_rb, 7.69288e-01_rb, 7.60820e-01_rb, 7.52445e-01_rb, &
     & 7.44163e-01_rb, 7.35971e-01_rb, 7.27870e-01_rb, 7.19858e-01_rb, 7.11935e-01_rb, &
     & 7.04098e-01_rb, 6.96348e-01_rb, 6.88683e-01_rb, 6.81102e-01_rb/)
      kao_mo3( 5, :,11) = (/ &
     & 8.31565e-01_rb, 8.22404e-01_rb, 8.13344e-01_rb, 8.04384e-01_rb, 7.95523e-01_rb, &
     & 7.86760e-01_rb, 7.78092e-01_rb, 7.69521e-01_rb, 7.61044e-01_rb, 7.52660e-01_rb, &
     & 7.44368e-01_rb, 7.36168e-01_rb, 7.28058e-01_rb, 7.20038e-01_rb, 7.12106e-01_rb, &
     & 7.04261e-01_rb, 6.96503e-01_rb, 6.88830e-01_rb, 6.81242e-01_rb/)
      kao_mo3( 6, :,11) = (/ &
     & 8.17636e-01_rb, 8.08497e-01_rb, 7.99461e-01_rb, 7.90525e-01_rb, 7.81690e-01_rb, &
     & 7.72953e-01_rb, 7.64314e-01_rb, 7.55771e-01_rb, 7.47324e-01_rb, 7.38971e-01_rb, &
     & 7.30712e-01_rb, 7.22545e-01_rb, 7.14469e-01_rb, 7.06483e-01_rb, 6.98587e-01_rb, &
     & 6.90779e-01_rb, 6.83058e-01_rb, 6.75424e-01_rb, 6.67875e-01_rb/)
      kao_mo3( 7, :,11) = (/ &
     & 7.95247e-01_rb, 7.86140e-01_rb, 7.77137e-01_rb, 7.68238e-01_rb, 7.59440e-01_rb, &
     & 7.50743e-01_rb, 7.42145e-01_rb, 7.33646e-01_rb, 7.25245e-01_rb, 7.16939e-01_rb, &
     & 7.08729e-01_rb, 7.00612e-01_rb, 6.92589e-01_rb, 6.84658e-01_rb, 6.76817e-01_rb, &
     & 6.69066e-01_rb, 6.61404e-01_rb, 6.53830e-01_rb, 6.46342e-01_rb/)
      kao_mo3( 8, :,11) = (/ &
     & 7.63069e-01_rb, 7.54006e-01_rb, 7.45051e-01_rb, 7.36202e-01_rb, 7.27458e-01_rb, &
     & 7.18818e-01_rb, 7.10281e-01_rb, 7.01845e-01_rb, 6.93509e-01_rb, 6.85272e-01_rb, &
     & 6.77133e-01_rb, 6.69091e-01_rb, 6.61144e-01_rb, 6.53292e-01_rb, 6.45533e-01_rb, &
     & 6.37866e-01_rb, 6.30290e-01_rb, 6.22804e-01_rb, 6.15407e-01_rb/)
      kao_mo3( 9, :,11) = (/ &
     & 2.03255e-01_rb, 2.03004e-01_rb, 2.02753e-01_rb, 2.02502e-01_rb, 2.02252e-01_rb, &
     & 2.02001e-01_rb, 2.01752e-01_rb, 2.01502e-01_rb, 2.01253e-01_rb, 2.01004e-01_rb, &
     & 2.00755e-01_rb, 2.00507e-01_rb, 2.00259e-01_rb, 2.00011e-01_rb, 1.99764e-01_rb, &
     & 1.99517e-01_rb, 1.99270e-01_rb, 1.99024e-01_rb, 1.98777e-01_rb/)
      kao_mo3( 1, :,12) = (/ &
     & 4.13201e-01_rb, 4.05258e-01_rb, 3.97468e-01_rb, 3.89828e-01_rb, 3.82334e-01_rb, &
     & 3.74985e-01_rb, 3.67777e-01_rb, 3.60707e-01_rb, 3.53774e-01_rb, 3.46973e-01_rb, &
     & 3.40303e-01_rb, 3.33762e-01_rb, 3.27346e-01_rb, 3.21054e-01_rb, 3.14882e-01_rb, &
     & 3.08829e-01_rb, 3.02893e-01_rb, 2.97071e-01_rb, 2.91360e-01_rb/)
      kao_mo3( 2, :,12) = (/ &
     & 4.12835e-01_rb, 4.04897e-01_rb, 3.97112e-01_rb, 3.89477e-01_rb, 3.81988e-01_rb, &
     & 3.74644e-01_rb, 3.67440e-01_rb, 3.60376e-01_rb, 3.53447e-01_rb, 3.46651e-01_rb, &
     & 3.39986e-01_rb, 3.33449e-01_rb, 3.27038e-01_rb, 3.20750e-01_rb, 3.14582e-01_rb, &
     & 3.08534e-01_rb, 3.02602e-01_rb, 2.96784e-01_rb, 2.91077e-01_rb/)
      kao_mo3( 3, :,12) = (/ &
     & 4.13023e-01_rb, 4.05079e-01_rb, 3.97289e-01_rb, 3.89648e-01_rb, 3.82155e-01_rb, &
     & 3.74805e-01_rb, 3.67597e-01_rb, 3.60527e-01_rb, 3.53594e-01_rb, 3.46793e-01_rb, &
     & 3.40124e-01_rb, 3.33583e-01_rb, 3.27167e-01_rb, 3.20875e-01_rb, 3.14704e-01_rb, &
     & 3.08652e-01_rb, 3.02716e-01_rb, 2.96894e-01_rb, 2.91184e-01_rb/)
      kao_mo3( 4, :,12) = (/ &
     & 4.13397e-01_rb, 4.05437e-01_rb, 3.97630e-01_rb, 3.89973e-01_rb, 3.82463e-01_rb, &
     & 3.75099e-01_rb, 3.67876e-01_rb, 3.60792e-01_rb, 3.53844e-01_rb, 3.47031e-01_rb, &
     & 3.40348e-01_rb, 3.33794e-01_rb, 3.27367e-01_rb, 3.21063e-01_rb, 3.14880e-01_rb, &
     & 3.08817e-01_rb, 3.02870e-01_rb, 2.97038e-01_rb, 2.91318e-01_rb/)
      kao_mo3( 5, :,12) = (/ &
     & 4.13043e-01_rb, 4.05106e-01_rb, 3.97321e-01_rb, 3.89686e-01_rb, 3.82198e-01_rb, &
     & 3.74854e-01_rb, 3.67651e-01_rb, 3.60586e-01_rb, 3.53657e-01_rb, 3.46861e-01_rb, &
     & 3.40195e-01_rb, 3.33658e-01_rb, 3.27246e-01_rb, 3.20958e-01_rb, 3.14790e-01_rb, &
     & 3.08741e-01_rb, 3.02808e-01_rb, 2.96990e-01_rb, 2.91283e-01_rb/)
      kao_mo3( 6, :,12) = (/ &
     & 4.13151e-01_rb, 4.05202e-01_rb, 3.97406e-01_rb, 3.89760e-01_rb, 3.82261e-01_rb, &
     & 3.74906e-01_rb, 3.67693e-01_rb, 3.60619e-01_rb, 3.53680e-01_rb, 3.46876e-01_rb, &
     & 3.40202e-01_rb, 3.33656e-01_rb, 3.27237e-01_rb, 3.20941e-01_rb, 3.14766e-01_rb, &
     & 3.08710e-01_rb, 3.02770e-01_rb, 2.96945e-01_rb, 2.91232e-01_rb/)
      kao_mo3( 7, :,12) = (/ &
     & 4.13052e-01_rb, 4.05109e-01_rb, 3.97319e-01_rb, 3.89678e-01_rb, 3.82185e-01_rb, &
     & 3.74835e-01_rb, 3.67627e-01_rb, 3.60557e-01_rb, 3.53624e-01_rb, 3.46823e-01_rb, &
     & 3.40154e-01_rb, 3.33612e-01_rb, 3.27197e-01_rb, 3.20905e-01_rb, 3.14734e-01_rb, &
     & 3.08681e-01_rb, 3.02745e-01_rb, 2.96923e-01_rb, 2.91213e-01_rb/)
      kao_mo3( 8, :,12) = (/ &
     & 4.13152e-01_rb, 4.05209e-01_rb, 3.97418e-01_rb, 3.89778e-01_rb, 3.82284e-01_rb, &
     & 3.74935e-01_rb, 3.67727e-01_rb, 3.60657e-01_rb, 3.53723e-01_rb, 3.46923e-01_rb, &
     & 3.40253e-01_rb, 3.33712e-01_rb, 3.27296e-01_rb, 3.21004e-01_rb, 3.14833e-01_rb, &
     & 3.08780e-01_rb, 3.02844e-01_rb, 2.97021e-01_rb, 2.91311e-01_rb/)
      kao_mo3( 9, :,12) = (/ &
     & 1.31008e-01_rb, 1.30607e-01_rb, 1.30208e-01_rb, 1.29810e-01_rb, 1.29413e-01_rb, &
     & 1.29017e-01_rb, 1.28623e-01_rb, 1.28229e-01_rb, 1.27837e-01_rb, 1.27446e-01_rb, &
     & 1.27056e-01_rb, 1.26668e-01_rb, 1.26280e-01_rb, 1.25894e-01_rb, 1.25509e-01_rb, &
     & 1.25125e-01_rb, 1.24743e-01_rb, 1.24361e-01_rb, 1.23981e-01_rb/)
      kao_mo3( 1, :,13) = (/ &
     & 4.66826e-01_rb, 4.71437e-01_rb, 4.76094e-01_rb, 4.80798e-01_rb, 4.85547e-01_rb, &
     & 4.90344e-01_rb, 4.95187e-01_rb, 5.00079e-01_rb, 5.05019e-01_rb, 5.10008e-01_rb, &
     & 5.15046e-01_rb, 5.20134e-01_rb, 5.25272e-01_rb, 5.30461e-01_rb, 5.35701e-01_rb, &
     & 5.40993e-01_rb, 5.46338e-01_rb, 5.51735e-01_rb, 5.57185e-01_rb/)
      kao_mo3( 2, :,13) = (/ &
     & 4.66579e-01_rb, 4.71199e-01_rb, 4.75865e-01_rb, 4.80577e-01_rb, 4.85336e-01_rb, &
     & 4.90141e-01_rb, 4.94995e-01_rb, 4.99896e-01_rb, 5.04846e-01_rb, 5.09845e-01_rb, &
     & 5.14893e-01_rb, 5.19992e-01_rb, 5.25141e-01_rb, 5.30340e-01_rb, 5.35592e-01_rb, &
     & 5.40895e-01_rb, 5.46251e-01_rb, 5.51660e-01_rb, 5.57122e-01_rb/)
      kao_mo3( 3, :,13) = (/ &
     & 4.66956e-01_rb, 4.71567e-01_rb, 4.76224e-01_rb, 4.80927e-01_rb, 4.85677e-01_rb, &
     & 4.90474e-01_rb, 4.95318e-01_rb, 5.00209e-01_rb, 5.05149e-01_rb, 5.10138e-01_rb, &
     & 5.15176e-01_rb, 5.20264e-01_rb, 5.25402e-01_rb, 5.30591e-01_rb, 5.35831e-01_rb, &
     & 5.41123e-01_rb, 5.46467e-01_rb, 5.51864e-01_rb, 5.57314e-01_rb/)
      kao_mo3( 4, :,13) = (/ &
     & 4.66456e-01_rb, 4.71080e-01_rb, 4.75750e-01_rb, 4.80467e-01_rb, 4.85230e-01_rb, &
     & 4.90040e-01_rb, 4.94898e-01_rb, 4.99804e-01_rb, 5.04759e-01_rb, 5.09763e-01_rb, &
     & 5.14817e-01_rb, 5.19920e-01_rb, 5.25075e-01_rb, 5.30280e-01_rb, 5.35537e-01_rb, &
     & 5.40846e-01_rb, 5.46208e-01_rb, 5.51622e-01_rb, 5.57091e-01_rb/)
      kao_mo3( 5, :,13) = (/ &
     & 4.66853e-01_rb, 4.71456e-01_rb, 4.76104e-01_rb, 4.80798e-01_rb, 4.85539e-01_rb, &
     & 4.90326e-01_rb, 4.95160e-01_rb, 5.00042e-01_rb, 5.04973e-01_rb, 5.09952e-01_rb, &
     & 5.14979e-01_rb, 5.20057e-01_rb, 5.25185e-01_rb, 5.30363e-01_rb, 5.35592e-01_rb, &
     & 5.40873e-01_rb, 5.46205e-01_rb, 5.51591e-01_rb, 5.57029e-01_rb/)
      kao_mo3( 6, :,13) = (/ &
     & 4.66832e-01_rb, 4.71448e-01_rb, 4.76110e-01_rb, 4.80817e-01_rb, 4.85571e-01_rb, &
     & 4.90372e-01_rb, 4.95221e-01_rb, 5.00118e-01_rb, 5.05063e-01_rb, 5.10056e-01_rb, &
     & 5.15100e-01_rb, 5.20193e-01_rb, 5.25336e-01_rb, 5.30531e-01_rb, 5.35776e-01_rb, &
     & 5.41074e-01_rb, 5.46424e-01_rb, 5.51826e-01_rb, 5.57283e-01_rb/)
      kao_mo3( 7, :,13) = (/ &
     & 4.66679e-01_rb, 4.71299e-01_rb, 4.75965e-01_rb, 4.80677e-01_rb, 4.85436e-01_rb, &
     & 4.90241e-01_rb, 4.95095e-01_rb, 4.99996e-01_rb, 5.04946e-01_rb, 5.09945e-01_rb, &
     & 5.14993e-01_rb, 5.20092e-01_rb, 5.25240e-01_rb, 5.30440e-01_rb, 5.35692e-01_rb, &
     & 5.40995e-01_rb, 5.46351e-01_rb, 5.51759e-01_rb, 5.57222e-01_rb/)
      kao_mo3( 8, :,13) = (/ &
     & 4.66982e-01_rb, 4.71598e-01_rb, 4.76260e-01_rb, 4.80967e-01_rb, 4.85721e-01_rb, &
     & 4.90522e-01_rb, 4.95371e-01_rb, 5.00268e-01_rb, 5.05213e-01_rb, 5.10206e-01_rb, &
     & 5.15250e-01_rb, 5.20343e-01_rb, 5.25486e-01_rb, 5.30680e-01_rb, 5.35926e-01_rb, &
     & 5.41223e-01_rb, 5.46573e-01_rb, 5.51976e-01_rb, 5.57432e-01_rb/)
      kao_mo3( 9, :,13) = (/ &
     & 1.13709e-01_rb, 1.13141e-01_rb, 1.12576e-01_rb, 1.12013e-01_rb, 1.11453e-01_rb, &
     & 1.10897e-01_rb, 1.10342e-01_rb, 1.09791e-01_rb, 1.09242e-01_rb, 1.08696e-01_rb, &
     & 1.08153e-01_rb, 1.07613e-01_rb, 1.07075e-01_rb, 1.06540e-01_rb, 1.06007e-01_rb, &
     & 1.05478e-01_rb, 1.04951e-01_rb, 1.04426e-01_rb, 1.03904e-01_rb/)
      kao_mo3( 1, :,14) = (/ &
     & 5.67608e-01_rb, 5.55796e-01_rb, 5.44230e-01_rb, 5.32904e-01_rb, 5.21814e-01_rb, &
     & 5.10955e-01_rb, 5.00322e-01_rb, 4.89910e-01_rb, 4.79714e-01_rb, 4.69731e-01_rb, &
     & 4.59956e-01_rb, 4.50384e-01_rb, 4.41011e-01_rb, 4.31834e-01_rb, 4.22847e-01_rb, &
     & 4.14048e-01_rb, 4.05431e-01_rb, 3.96994e-01_rb, 3.88732e-01_rb/)
      kao_mo3( 2, :,14) = (/ &
     & 5.67766e-01_rb, 5.55948e-01_rb, 5.44376e-01_rb, 5.33045e-01_rb, 5.21950e-01_rb, &
     & 5.11086e-01_rb, 5.00448e-01_rb, 4.90031e-01_rb, 4.79831e-01_rb, 4.69844e-01_rb, &
     & 4.60064e-01_rb, 4.50488e-01_rb, 4.41111e-01_rb, 4.31930e-01_rb, 4.22939e-01_rb, &
     & 4.14136e-01_rb, 4.05516e-01_rb, 3.97075e-01_rb, 3.88810e-01_rb/)
      kao_mo3( 3, :,14) = (/ &
     & 5.67460e-01_rb, 5.55647e-01_rb, 5.44080e-01_rb, 5.32754e-01_rb, 5.21664e-01_rb, &
     & 5.10805e-01_rb, 5.00172e-01_rb, 4.89760e-01_rb, 4.79564e-01_rb, 4.69582e-01_rb, &
     & 4.59806e-01_rb, 4.50235e-01_rb, 4.40862e-01_rb, 4.31685e-01_rb, 4.22699e-01_rb, &
     & 4.13900e-01_rb, 4.05284e-01_rb, 3.96847e-01_rb, 3.88586e-01_rb/)
      kao_mo3( 4, :,14) = (/ &
     & 5.67925e-01_rb, 5.56107e-01_rb, 5.44536e-01_rb, 5.33205e-01_rb, 5.22110e-01_rb, &
     & 5.11246e-01_rb, 5.00608e-01_rb, 4.90191e-01_rb, 4.79991e-01_rb, 4.70004e-01_rb, &
     & 4.60224e-01_rb, 4.50647e-01_rb, 4.41270e-01_rb, 4.32088e-01_rb, 4.23097e-01_rb, &
     & 4.14293e-01_rb, 4.05673e-01_rb, 3.97231e-01_rb, 3.88966e-01_rb/)
      kao_mo3( 5, :,14) = (/ &
     & 5.67520e-01_rb, 5.55733e-01_rb, 5.44190e-01_rb, 5.32887e-01_rb, 5.21818e-01_rb, &
     & 5.10980e-01_rb, 5.00366e-01_rb, 4.89974e-01_rb, 4.79797e-01_rb, 4.69831e-01_rb, &
     & 4.60072e-01_rb, 4.50516e-01_rb, 4.41159e-01_rb, 4.31996e-01_rb, 4.23023e-01_rb, &
     & 4.14236e-01_rb, 4.05633e-01_rb, 3.97207e-01_rb, 3.88957e-01_rb/)
      kao_mo3( 6, :,14) = (/ &
     & 5.67549e-01_rb, 5.55749e-01_rb, 5.44195e-01_rb, 5.32880e-01_rb, 5.21801e-01_rb, &
     & 5.10952e-01_rb, 5.00329e-01_rb, 4.89927e-01_rb, 4.79740e-01_rb, 4.69766e-01_rb, &
     & 4.59999e-01_rb, 4.50435e-01_rb, 4.41070e-01_rb, 4.31900e-01_rb, 4.22920e-01_rb, &
     & 4.14127e-01_rb, 4.05517e-01_rb, 3.97086e-01_rb, 3.88830e-01_rb/)
      kao_mo3( 7, :,14) = (/ &
     & 5.67727e-01_rb, 5.55909e-01_rb, 5.44337e-01_rb, 5.33005e-01_rb, 5.21910e-01_rb, &
     & 5.11046e-01_rb, 5.00408e-01_rb, 4.89991e-01_rb, 4.79791e-01_rb, 4.69804e-01_rb, &
     & 4.60024e-01_rb, 4.50448e-01_rb, 4.41071e-01_rb, 4.31890e-01_rb, 4.22900e-01_rb, &
     & 4.14096e-01_rb, 4.05476e-01_rb, 3.97036e-01_rb, 3.88771e-01_rb/)
      kao_mo3( 8, :,14) = (/ &
     & 5.67795e-01_rb, 5.55965e-01_rb, 5.44381e-01_rb, 5.33039e-01_rb, 5.21933e-01_rb, &
     & 5.11058e-01_rb, 5.00410e-01_rb, 4.89984e-01_rb, 4.79775e-01_rb, 4.69779e-01_rb, &
     & 4.59991e-01_rb, 4.50407e-01_rb, 4.41023e-01_rb, 4.31834e-01_rb, 4.22836e-01_rb, &
     & 4.14026e-01_rb, 4.05400e-01_rb, 3.96953e-01_rb, 3.88683e-01_rb/)
      kao_mo3( 9, :,14) = (/ &
     & 1.32957e-01_rb, 1.31737e-01_rb, 1.30528e-01_rb, 1.29330e-01_rb, 1.28143e-01_rb, &
     & 1.26967e-01_rb, 1.25802e-01_rb, 1.24648e-01_rb, 1.23504e-01_rb, 1.22370e-01_rb, &
     & 1.21247e-01_rb, 1.20135e-01_rb, 1.19032e-01_rb, 1.17940e-01_rb, 1.16857e-01_rb, &
     & 1.15785e-01_rb, 1.14722e-01_rb, 1.13669e-01_rb, 1.12626e-01_rb/)
      kao_mo3( 1, :,15) = (/ &
     & 1.51281e-01_rb, 1.53439e-01_rb, 1.55628e-01_rb, 1.57848e-01_rb, 1.60100e-01_rb, &
     & 1.62384e-01_rb, 1.64700e-01_rb, 1.67049e-01_rb, 1.69432e-01_rb, 1.71849e-01_rb, &
     & 1.74301e-01_rb, 1.76787e-01_rb, 1.79309e-01_rb, 1.81866e-01_rb, 1.84461e-01_rb, &
     & 1.87092e-01_rb, 1.89761e-01_rb, 1.92468e-01_rb, 1.95213e-01_rb/)
      kao_mo3( 2, :,15) = (/ &
     & 1.51431e-01_rb, 1.53589e-01_rb, 1.55778e-01_rb, 1.57998e-01_rb, 1.60250e-01_rb, &
     & 1.62534e-01_rb, 1.64850e-01_rb, 1.67199e-01_rb, 1.69582e-01_rb, 1.71999e-01_rb, &
     & 1.74450e-01_rb, 1.76937e-01_rb, 1.79458e-01_rb, 1.82016e-01_rb, 1.84610e-01_rb, &
     & 1.87241e-01_rb, 1.89909e-01_rb, 1.92616e-01_rb, 1.95361e-01_rb/)
      kao_mo3( 3, :,15) = (/ &
     & 1.51299e-01_rb, 1.53461e-01_rb, 1.55654e-01_rb, 1.57878e-01_rb, 1.60134e-01_rb, &
     & 1.62422e-01_rb, 1.64744e-01_rb, 1.67098e-01_rb, 1.69486e-01_rb, 1.71908e-01_rb, &
     & 1.74364e-01_rb, 1.76856e-01_rb, 1.79383e-01_rb, 1.81947e-01_rb, 1.84547e-01_rb, &
     & 1.87184e-01_rb, 1.89859e-01_rb, 1.92572e-01_rb, 1.95324e-01_rb/)
      kao_mo3( 4, :,15) = (/ &
     & 1.51281e-01_rb, 1.53439e-01_rb, 1.55628e-01_rb, 1.57848e-01_rb, 1.60100e-01_rb, &
     & 1.62384e-01_rb, 1.64700e-01_rb, 1.67049e-01_rb, 1.69432e-01_rb, 1.71849e-01_rb, &
     & 1.74301e-01_rb, 1.76787e-01_rb, 1.79309e-01_rb, 1.81866e-01_rb, 1.84461e-01_rb, &
     & 1.87092e-01_rb, 1.89761e-01_rb, 1.92468e-01_rb, 1.95213e-01_rb/)
      kao_mo3( 5, :,15) = (/ &
     & 1.51281e-01_rb, 1.53439e-01_rb, 1.55628e-01_rb, 1.57848e-01_rb, 1.60100e-01_rb, &
     & 1.62384e-01_rb, 1.64700e-01_rb, 1.67049e-01_rb, 1.69432e-01_rb, 1.71849e-01_rb, &
     & 1.74301e-01_rb, 1.76787e-01_rb, 1.79309e-01_rb, 1.81866e-01_rb, 1.84461e-01_rb, &
     & 1.87092e-01_rb, 1.89761e-01_rb, 1.92468e-01_rb, 1.95213e-01_rb/)
      kao_mo3( 6, :,15) = (/ &
     & 1.51299e-01_rb, 1.53461e-01_rb, 1.55654e-01_rb, 1.57878e-01_rb, 1.60134e-01_rb, &
     & 1.62422e-01_rb, 1.64744e-01_rb, 1.67098e-01_rb, 1.69486e-01_rb, 1.71908e-01_rb, &
     & 1.74364e-01_rb, 1.76856e-01_rb, 1.79383e-01_rb, 1.81947e-01_rb, 1.84547e-01_rb, &
     & 1.87184e-01_rb, 1.89859e-01_rb, 1.92572e-01_rb, 1.95324e-01_rb/)
      kao_mo3( 7, :,15) = (/ &
     & 1.51299e-01_rb, 1.53461e-01_rb, 1.55654e-01_rb, 1.57878e-01_rb, 1.60134e-01_rb, &
     & 1.62422e-01_rb, 1.64744e-01_rb, 1.67098e-01_rb, 1.69486e-01_rb, 1.71908e-01_rb, &
     & 1.74364e-01_rb, 1.76856e-01_rb, 1.79383e-01_rb, 1.81947e-01_rb, 1.84547e-01_rb, &
     & 1.87184e-01_rb, 1.89859e-01_rb, 1.92572e-01_rb, 1.95324e-01_rb/)
      kao_mo3( 8, :,15) = (/ &
     & 1.51281e-01_rb, 1.53439e-01_rb, 1.55628e-01_rb, 1.57848e-01_rb, 1.60100e-01_rb, &
     & 1.62384e-01_rb, 1.64700e-01_rb, 1.67049e-01_rb, 1.69432e-01_rb, 1.71849e-01_rb, &
     & 1.74301e-01_rb, 1.76787e-01_rb, 1.79309e-01_rb, 1.81866e-01_rb, 1.84461e-01_rb, &
     & 1.87092e-01_rb, 1.89761e-01_rb, 1.92468e-01_rb, 1.95213e-01_rb/)
      kao_mo3( 9, :,15) = (/ &
     & 2.44180e-01_rb, 2.35686e-01_rb, 2.27487e-01_rb, 2.19574e-01_rb, 2.11935e-01_rb, &
     & 2.04563e-01_rb, 1.97447e-01_rb, 1.90578e-01_rb, 1.83949e-01_rb, 1.77550e-01_rb, &
     & 1.71373e-01_rb, 1.65412e-01_rb, 1.59658e-01_rb, 1.54104e-01_rb, 1.48743e-01_rb, &
     & 1.43569e-01_rb, 1.38574e-01_rb, 1.33754e-01_rb, 1.29101e-01_rb/)
      kao_mo3( 1, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 2, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 3, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 4, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 5, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 6, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 7, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 8, :,16) = (/ &
     & 1.02934e-01_rb, 1.04369e-01_rb, 1.05825e-01_rb, 1.07300e-01_rb, 1.08797e-01_rb, &
     & 1.10314e-01_rb, 1.11852e-01_rb, 1.13412e-01_rb, 1.14993e-01_rb, 1.16597e-01_rb, &
     & 1.18223e-01_rb, 1.19871e-01_rb, 1.21543e-01_rb, 1.23238e-01_rb, 1.24956e-01_rb, &
     & 1.26699e-01_rb, 1.28466e-01_rb, 1.30257e-01_rb, 1.32073e-01_rb/)
      kao_mo3( 9, :,16) = (/ &
     & 3.91531e-01_rb, 3.78978e-01_rb, 3.66827e-01_rb, 3.55067e-01_rb, 3.43683e-01_rb, &
     & 3.32664e-01_rb, 3.21999e-01_rb, 3.11675e-01_rb, 3.01683e-01_rb, 2.92011e-01_rb, &
     & 2.82648e-01_rb, 2.73586e-01_rb, 2.64815e-01_rb, 2.56325e-01_rb, 2.48107e-01_rb, &
     & 2.40152e-01_rb, 2.32453e-01_rb, 2.25000e-01_rb, 2.17787e-01_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &1.0689e-05_rb,1.6987e-05_rb,1.8993e-05_rb,3.4470e-05_rb,4.0873e-05_rb,4.8275e-05_rb, &
     &6.1178e-05_rb,6.4035e-05_rb,6.6253e-05_rb,7.8914e-05_rb,8.1640e-05_rb,7.9738e-05_rb, &
     &7.8492e-05_rb,9.1565e-05_rb,1.0262e-04_rb,1.0368e-04_rb/)
      forrefo(2,:) = (/ &
     &1.1194e-05_rb,1.6128e-05_rb,1.7213e-05_rb,2.6845e-05_rb,4.1361e-05_rb,5.1508e-05_rb, &
     &6.8245e-05_rb,7.4063e-05_rb,7.6273e-05_rb,8.4061e-05_rb,8.2492e-05_rb,8.1720e-05_rb, &
     &7.7626e-05_rb,1.0096e-04_rb,1.0519e-04_rb,1.0631e-04_rb/)
      forrefo(3,:) = (/ &
     &1.0891e-05_rb,1.4933e-05_rb,1.7964e-05_rb,2.2577e-05_rb,4.4290e-05_rb,5.4675e-05_rb, &
     &7.2494e-05_rb,7.8410e-05_rb,7.6948e-05_rb,7.5742e-05_rb,7.7654e-05_rb,8.2760e-05_rb, &
     &7.8443e-05_rb,9.8384e-05_rb,1.0634e-04_rb,1.0838e-04_rb/)
      forrefo(4,:) = (/ &
     &1.1316e-05_rb,1.5470e-05_rb,2.1246e-05_rb,3.3349e-05_rb,4.8704e-05_rb,5.6424e-05_rb, &
     &5.8569e-05_rb,5.8780e-05_rb,6.0358e-05_rb,6.1586e-05_rb,6.4281e-05_rb,6.9333e-05_rb, &
     &7.2763e-05_rb,7.2675e-05_rb,7.3754e-05_rb,1.0131e-04_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 1.27686e-01_rb, 1.09347e-01_rb, 9.36410e-02_rb, 8.01912e-02_rb, 6.86732e-02_rb, &
     & 5.88096e-02_rb, 5.03627e-02_rb, 4.31290e-02_rb, 3.69343e-02_rb, 3.16294e-02_rb/)
      selfrefo(:, 2) = (/ &
     & 1.40051e-01_rb, 1.20785e-01_rb, 1.04170e-01_rb, 8.98402e-02_rb, 7.74816e-02_rb, &
     & 6.68231e-02_rb, 5.76308e-02_rb, 4.97030e-02_rb, 4.28658e-02_rb, 3.69691e-02_rb/)
      selfrefo(:, 3) = (/ &
     & 1.42322e-01_rb, 1.22872e-01_rb, 1.06080e-01_rb, 9.15829e-02_rb, 7.90671e-02_rb, &
     & 6.82616e-02_rb, 5.89329e-02_rb, 5.08790e-02_rb, 4.39258e-02_rb, 3.79228e-02_rb/)
      selfrefo(:, 4) = (/ &
     & 1.53244e-01_rb, 1.33057e-01_rb, 1.15530e-01_rb, 1.00311e-01_rb, 8.70977e-02_rb, &
     & 7.56244e-02_rb, 6.56626e-02_rb, 5.70130e-02_rb, 4.95028e-02_rb, 4.29819e-02_rb/)
      selfrefo(:, 5) = (/ &
     & 1.71011e-01_rb, 1.46680e-01_rb, 1.25810e-01_rb, 1.07910e-01_rb, 9.25563e-02_rb, &
     & 7.93874e-02_rb, 6.80922e-02_rb, 5.84040e-02_rb, 5.00943e-02_rb, 4.29669e-02_rb/)
      selfrefo(:, 6) = (/ &
     & 1.76012e-01_rb, 1.51010e-01_rb, 1.29560e-01_rb, 1.11157e-01_rb, 9.53672e-02_rb, &
     & 8.18207e-02_rb, 7.01984e-02_rb, 6.02270e-02_rb, 5.16720e-02_rb, 4.43322e-02_rb/)
      selfrefo(:, 7) = (/ &
     & 1.85600e-01_rb, 1.59051e-01_rb, 1.36300e-01_rb, 1.16803e-01_rb, 1.00095e-01_rb, &
     & 8.57776e-02_rb, 7.35077e-02_rb, 6.29930e-02_rb, 5.39823e-02_rb, 4.62606e-02_rb/)
      selfrefo(:, 8) = (/ &
     & 1.88931e-01_rb, 1.61727e-01_rb, 1.38440e-01_rb, 1.18506e-01_rb, 1.01442e-01_rb, &
     & 8.68356e-02_rb, 7.43321e-02_rb, 6.36290e-02_rb, 5.44670e-02_rb, 4.66243e-02_rb/)
      selfrefo(:, 9) = (/ &
     & 1.91122e-01_rb, 1.63407e-01_rb, 1.39710e-01_rb, 1.19450e-01_rb, 1.02128e-01_rb, &
     & 8.73176e-02_rb, 7.46552e-02_rb, 6.38290e-02_rb, 5.45728e-02_rb, 4.66589e-02_rb/)
      selfrefo(:,10) = (/ &
     & 1.91334e-01_rb, 1.64872e-01_rb, 1.42070e-01_rb, 1.22421e-01_rb, 1.05490e-01_rb, &
     & 9.09008e-02_rb, 7.83291e-02_rb, 6.74960e-02_rb, 5.81612e-02_rb, 5.01174e-02_rb/)
      selfrefo(:,11) = (/ &
     & 1.89858e-01_rb, 1.63934e-01_rb, 1.41550e-01_rb, 1.22222e-01_rb, 1.05534e-01_rb, &
     & 9.11237e-02_rb, 7.86814e-02_rb, 6.79380e-02_rb, 5.86615e-02_rb, 5.06517e-02_rb/)
      selfrefo(:,12) = (/ &
     & 1.89783e-01_rb, 1.63757e-01_rb, 1.41300e-01_rb, 1.21923e-01_rb, 1.05203e-01_rb, &
     & 9.07760e-02_rb, 7.83274e-02_rb, 6.75860e-02_rb, 5.83176e-02_rb, 5.03202e-02_rb/)
      selfrefo(:,13) = (/ &
     & 1.87534e-01_rb, 1.62016e-01_rb, 1.39970e-01_rb, 1.20924e-01_rb, 1.04470e-01_rb, &
     & 9.02541e-02_rb, 7.79730e-02_rb, 6.73630e-02_rb, 5.81967e-02_rb, 5.02778e-02_rb/)
      selfrefo(:,14) = (/ &
     & 1.99128e-01_rb, 1.71410e-01_rb, 1.47550e-01_rb, 1.27011e-01_rb, 1.09332e-01_rb, &
     & 9.41131e-02_rb, 8.10128e-02_rb, 6.97360e-02_rb, 6.00289e-02_rb, 5.16731e-02_rb/)
      selfrefo(:,15) = (/ &
     & 1.99460e-01_rb, 1.72342e-01_rb, 1.48910e-01_rb, 1.28664e-01_rb, 1.11171e-01_rb, &
     & 9.60560e-02_rb, 8.29962e-02_rb, 7.17120e-02_rb, 6.19620e-02_rb, 5.35376e-02_rb/)
      selfrefo(:,16) = (/ &
     & 1.99906e-01_rb, 1.72737e-01_rb, 1.49260e-01_rb, 1.28974e-01_rb, 1.11445e-01_rb, &
     & 9.62982e-02_rb, 8.32102e-02_rb, 7.19010e-02_rb, 6.21288e-02_rb, 5.36848e-02_rb/)

      end subroutine lw_kgb05

! **************************************************************************
      subroutine lw_kgb06
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg06, only : fracrefao, kao, kao_mco2, selfrefo, forrefo, &
                            cfc11adjo, cfc12o

      implicit none
      save

! Planck fraction mapping level : P = 473.4280 mb, T = 259.83 K
      fracrefao(:) = (/ &
     &   1.4353e-01_rb,1.4774e-01_rb,1.4467e-01_rb,1.3785e-01_rb,1.2376e-01_rb,1.0214e-01_rb, &
     &   8.1984e-02_rb,6.1152e-02_rb,4.0987e-02_rb,4.5067e-03_rb,4.0020e-03_rb,3.1772e-03_rb, &
     &   2.3458e-03_rb,1.5025e-03_rb,5.7415e-04_rb,8.2970e-05_rb/)

! Minor gas mapping level:
!     lower - co2, p = 706.2720 mb, t = 294.2 k
!     upper - cfc11, cfc12

!      cfc11(:) = (/ &
!     &    0.,      0.,      26.5435, 108.850, &
!     &    58.7804, 54.0875, 41.1065, 35.6120, &
!     &    41.2328, 47.7402, 79.1026, 64.3005, &
!     &    108.206, 141.617, 186.565, 58.4782/)
! Original cfc11 is multiplied by 1.385 to account for the 1060-1107 cm-1 band.

      cfc11adjo(:) = (/ &
     &   0._rb,      0._rb,      36.7627_rb, 150.757_rb, &
     &   81.4109_rb, 74.9112_rb, 56.9325_rb, 49.3226_rb, &
     &   57.1074_rb, 66.1202_rb, 109.557_rb, 89.0562_rb, & 
     &   149.865_rb, 196.140_rb, 258.393_rb, 80.9923_rb/)
      cfc12o(:) = (/ &
     &   62.8368_rb, 43.2626_rb, 26.7549_rb, 22.2487_rb, &
     &   23.5029_rb, 34.8323_rb, 26.2335_rb, 23.2306_rb, &
     &   18.4062_rb, 13.9534_rb, 22.6268_rb, 24.2604_rb, &
     &   30.0088_rb, 26.3634_rb, 15.8237_rb, 57.5050_rb/)

!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels > ~100mb and temperatures.  The first
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the corresponding TREF for this  pressure level, 
!     JT = 2 refers to the temperatureTREF-15, JT = 1 is for TREF-30, 
!     JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  The second 
!     index, JP, runs from 1 to 13 and refers to the corresponding 
!     pressure level in PREF (e.g. JP = 1 is for a pressure of 1053.63 mb).  
!     The third index, IG, goes from 1 to 16, and tells us which 
!     g-interval the absorption coefficients are for.

      kao(:, 1, 1) = (/ &
     &1.5108e-05_rb,1.6223e-05_rb,1.5899e-05_rb,1.5254e-05_rb,1.4889e-05_rb/)
      kao(:, 2, 1) = (/ &
     &1.0690e-05_rb,1.0800e-05_rb,1.0568e-05_rb,1.0057e-05_rb,9.4945e-06_rb/)
      kao(:, 3, 1) = (/ &
     &6.1072e-06_rb,5.8741e-06_rb,5.6796e-06_rb,5.3664e-06_rb,5.4934e-06_rb/)
      kao(:, 4, 1) = (/ &
     &3.0372e-06_rb,2.9964e-06_rb,2.6541e-06_rb,2.8791e-06_rb,3.9638e-06_rb/)
      kao(:, 5, 1) = (/ &
     &1.4777e-06_rb,1.4916e-06_rb,1.4819e-06_rb,2.2264e-06_rb,3.3268e-06_rb/)
      kao(:, 6, 1) = (/ &
     &7.4936e-07_rb,7.2240e-07_rb,1.1518e-06_rb,1.7724e-06_rb,2.8203e-06_rb/)
      kao(:, 7, 1) = (/ &
     &3.4904e-07_rb,5.1177e-07_rb,8.8489e-07_rb,1.5219e-06_rb,2.2871e-06_rb/)
      kao(:, 8, 1) = (/ &
     &2.1072e-07_rb,4.5257e-07_rb,7.4214e-07_rb,1.2628e-06_rb,1.9840e-06_rb/)
      kao(:, 9, 1) = (/ &
     &2.1779e-07_rb,4.8639e-07_rb,8.1133e-07_rb,1.3719e-06_rb,2.1663e-06_rb/)
      kao(:,10, 1) = (/ &
     &2.5134e-07_rb,5.9583e-07_rb,1.0449e-06_rb,1.5899e-06_rb,2.7441e-06_rb/)
      kao(:,11, 1) = (/ &
     &2.9178e-07_rb,6.5749e-07_rb,1.0958e-06_rb,1.7834e-06_rb,3.1413e-06_rb/)
      kao(:,12, 1) = (/ &
     &2.7268e-07_rb,6.1470e-07_rb,1.0270e-06_rb,1.7566e-06_rb,3.0441e-06_rb/)
      kao(:,13, 1) = (/ &
     &2.2995e-07_rb,5.2185e-07_rb,8.7916e-07_rb,1.4999e-06_rb,2.5740e-06_rb/)
      kao(:, 1, 2) = (/ &
     &2.1301e-05_rb,2.3060e-05_rb,2.1909e-05_rb,2.0972e-05_rb,2.1772e-05_rb/)
      kao(:, 2, 2) = (/ &
     &1.4382e-05_rb,1.4147e-05_rb,1.4339e-05_rb,1.3679e-05_rb,1.6973e-05_rb/)
      kao(:, 3, 2) = (/ &
     &8.0559e-06_rb,8.4180e-06_rb,7.4096e-06_rb,9.3490e-06_rb,1.0824e-05_rb/)
      kao(:, 4, 2) = (/ &
     &4.3614e-06_rb,3.9657e-06_rb,5.1299e-06_rb,6.1754e-06_rb,8.2925e-06_rb/)
      kao(:, 5, 2) = (/ &
     &2.2350e-06_rb,2.3231e-06_rb,3.3297e-06_rb,4.8612e-06_rb,6.5883e-06_rb/)
      kao(:, 6, 2) = (/ &
     &1.0863e-06_rb,1.7510e-06_rb,2.4185e-06_rb,3.5854e-06_rb,5.1511e-06_rb/)
      kao(:, 7, 2) = (/ &
     &7.6749e-07_rb,1.2024e-06_rb,1.9782e-06_rb,2.7223e-06_rb,4.2604e-06_rb/)
      kao(:, 8, 2) = (/ &
     &6.8738e-07_rb,9.7505e-07_rb,1.6738e-06_rb,2.4371e-06_rb,3.9386e-06_rb/)
      kao(:, 9, 2) = (/ &
     &7.3668e-07_rb,1.0167e-06_rb,1.9026e-06_rb,3.0451e-06_rb,5.0060e-06_rb/)
      kao(:,10, 2) = (/ &
     &9.8863e-07_rb,1.5373e-06_rb,2.5876e-06_rb,4.8975e-06_rb,7.8206e-06_rb/)
      kao(:,11, 2) = (/ &
     &1.0934e-06_rb,1.6488e-06_rb,2.7961e-06_rb,5.2441e-06_rb,8.5410e-06_rb/)
      kao(:,12, 2) = (/ &
     &1.0277e-06_rb,1.5126e-06_rb,2.6342e-06_rb,4.9122e-06_rb,8.0730e-06_rb/)
      kao(:,13, 2) = (/ &
     &8.7381e-07_rb,1.2665e-06_rb,2.2835e-06_rb,4.2048e-06_rb,6.9189e-06_rb/)
      kao(:, 1, 3) = (/ &
     &2.3993e-05_rb,2.2308e-05_rb,2.5893e-05_rb,3.3883e-05_rb,3.8280e-05_rb/)
      kao(:, 2, 3) = (/ &
     &1.4011e-05_rb,1.4296e-05_rb,2.1223e-05_rb,2.4417e-05_rb,2.7881e-05_rb/)
      kao(:, 3, 3) = (/ &
     &7.3965e-06_rb,9.9180e-06_rb,1.3735e-05_rb,1.4536e-05_rb,1.8056e-05_rb/)
      kao(:, 4, 3) = (/ &
     &4.5365e-06_rb,7.0513e-06_rb,7.2165e-06_rb,9.8585e-06_rb,1.1890e-05_rb/)
      kao(:, 5, 3) = (/ &
     &3.8953e-06_rb,4.3626e-06_rb,5.3805e-06_rb,6.6118e-06_rb,9.5204e-06_rb/)
      kao(:, 6, 3) = (/ &
     &2.4692e-06_rb,2.6620e-06_rb,3.5833e-06_rb,5.1748e-06_rb,7.8116e-06_rb/)
      kao(:, 7, 3) = (/ &
     &1.4531e-06_rb,2.0753e-06_rb,2.7794e-06_rb,4.8340e-06_rb,7.5049e-06_rb/)
      kao(:, 8, 3) = (/ &
     &8.9161e-07_rb,1.6554e-06_rb,2.7945e-06_rb,4.7401e-06_rb,7.3008e-06_rb/)
      kao(:, 9, 3) = (/ &
     &1.1556e-06_rb,2.3050e-06_rb,3.6767e-06_rb,6.4276e-06_rb,1.0393e-05_rb/)
      kao(:,10, 3) = (/ &
     &1.3936e-06_rb,2.7867e-06_rb,4.8066e-06_rb,7.9343e-06_rb,1.4027e-05_rb/)
      kao(:,11, 3) = (/ &
     &1.4469e-06_rb,3.0966e-06_rb,5.3467e-06_rb,9.2496e-06_rb,1.6488e-05_rb/)
      kao(:,12, 3) = (/ &
     &1.3797e-06_rb,2.9577e-06_rb,5.1784e-06_rb,9.2835e-06_rb,1.6169e-05_rb/)
      kao(:,13, 3) = (/ &
     &1.1793e-06_rb,2.5194e-06_rb,4.4903e-06_rb,8.2145e-06_rb,1.4209e-05_rb/)
      kao(:, 1, 4) = (/ &
     &4.3371e-05_rb,4.2284e-05_rb,4.2569e-05_rb,4.5333e-05_rb,4.9887e-05_rb/)
      kao(:, 2, 4) = (/ &
     &2.7836e-05_rb,3.0742e-05_rb,2.5159e-05_rb,3.1058e-05_rb,3.3502e-05_rb/)
      kao(:, 3, 4) = (/ &
     &1.5959e-05_rb,1.5689e-05_rb,1.8607e-05_rb,1.9311e-05_rb,2.2949e-05_rb/)
      kao(:, 4, 4) = (/ &
     &9.4357e-06_rb,8.9581e-06_rb,1.1706e-05_rb,1.2462e-05_rb,1.9504e-05_rb/)
      kao(:, 5, 4) = (/ &
     &4.6991e-06_rb,5.9565e-06_rb,7.1131e-06_rb,1.0656e-05_rb,1.6961e-05_rb/)
      kao(:, 6, 4) = (/ &
     &2.9104e-06_rb,3.9977e-06_rb,5.8385e-06_rb,9.8483e-06_rb,1.4910e-05_rb/)
      kao(:, 7, 4) = (/ &
     &2.0404e-06_rb,2.7360e-06_rb,5.0537e-06_rb,8.2979e-06_rb,1.2550e-05_rb/)
      kao(:, 8, 4) = (/ &
     &1.7124e-06_rb,2.7023e-06_rb,4.6383e-06_rb,7.6524e-06_rb,1.2059e-05_rb/)
      kao(:, 9, 4) = (/ &
     &1.7855e-06_rb,3.1326e-06_rb,5.8535e-06_rb,9.7627e-06_rb,1.5447e-05_rb/)
      kao(:,10, 4) = (/ &
     &2.2188e-06_rb,4.3098e-06_rb,8.7192e-06_rb,1.5363e-05_rb,2.4174e-05_rb/)
      kao(:,11, 4) = (/ &
     &2.5395e-06_rb,4.8780e-06_rb,1.0119e-05_rb,1.6942e-05_rb,2.6266e-05_rb/)
      kao(:,12, 4) = (/ &
     &2.4951e-06_rb,4.8859e-06_rb,9.7649e-06_rb,1.6137e-05_rb,2.5091e-05_rb/)
      kao(:,13, 4) = (/ &
     &2.2608e-06_rb,4.4391e-06_rb,8.5101e-06_rb,1.3874e-05_rb,2.1718e-05_rb/)
      kao(:, 1, 5) = (/ &
     &3.5802e-05_rb,4.9569e-05_rb,5.6081e-05_rb,5.4456e-05_rb,6.1620e-05_rb/)
      kao(:, 2, 5) = (/ &
     &2.7431e-05_rb,3.7092e-05_rb,4.0464e-05_rb,4.0603e-05_rb,5.2992e-05_rb/)
      kao(:, 3, 5) = (/ &
     &1.6511e-05_rb,2.1182e-05_rb,1.9952e-05_rb,2.9271e-05_rb,4.8230e-05_rb/)
      kao(:, 4, 5) = (/ &
     &1.1211e-05_rb,1.2922e-05_rb,1.4441e-05_rb,2.5935e-05_rb,3.8342e-05_rb/)
      kao(:, 5, 5) = (/ &
     &7.1937e-06_rb,7.6149e-06_rb,1.2471e-05_rb,2.1334e-05_rb,3.0465e-05_rb/)
      kao(:, 6, 5) = (/ &
     &4.6613e-06_rb,5.5104e-06_rb,1.0664e-05_rb,1.6922e-05_rb,2.4802e-05_rb/)
      kao(:, 7, 5) = (/ &
     &2.6908e-06_rb,5.1362e-06_rb,9.2865e-06_rb,1.4719e-05_rb,2.2004e-05_rb/)
      kao(:, 8, 5) = (/ &
     &2.4192e-06_rb,4.7977e-06_rb,8.6605e-06_rb,1.4122e-05_rb,2.1533e-05_rb/)
      kao(:, 9, 5) = (/ &
     &2.6505e-06_rb,5.2138e-06_rb,9.6090e-06_rb,1.6493e-05_rb,2.6941e-05_rb/)
      kao(:,10, 5) = (/ &
     &3.8793e-06_rb,7.0191e-06_rb,1.2677e-05_rb,2.2456e-05_rb,3.8574e-05_rb/)
      kao(:,11, 5) = (/ &
     &4.3385e-06_rb,8.1555e-06_rb,1.4661e-05_rb,2.7034e-05_rb,4.5538e-05_rb/)
      kao(:,12, 5) = (/ &
     &4.4806e-06_rb,8.0941e-06_rb,1.5170e-05_rb,2.7140e-05_rb,4.7747e-05_rb/)
      kao(:,13, 5) = (/ &
     &4.0859e-06_rb,7.4306e-06_rb,1.3915e-05_rb,2.5201e-05_rb,4.3638e-05_rb/)
      kao(:, 1, 6) = (/ &
     &6.3502e-05_rb,6.1626e-05_rb,6.7874e-05_rb,7.9926e-05_rb,1.4827e-04_rb/)
      kao(:, 2, 6) = (/ &
     &4.2546e-05_rb,3.3547e-05_rb,4.0025e-05_rb,7.0944e-05_rb,1.2242e-04_rb/)
      kao(:, 3, 6) = (/ &
     &2.4414e-05_rb,2.4466e-05_rb,3.6681e-05_rb,6.7397e-05_rb,9.7673e-05_rb/)
      kao(:, 4, 6) = (/ &
     &1.1878e-05_rb,1.6683e-05_rb,3.4310e-05_rb,5.3734e-05_rb,7.4953e-05_rb/)
      kao(:, 5, 6) = (/ &
     &8.0437e-06_rb,1.5133e-05_rb,2.7743e-05_rb,4.0147e-05_rb,5.6233e-05_rb/)
      kao(:, 6, 6) = (/ &
     &6.1550e-06_rb,1.2316e-05_rb,1.9945e-05_rb,3.0529e-05_rb,4.5113e-05_rb/)
      kao(:, 7, 6) = (/ &
     &4.8891e-06_rb,9.6377e-06_rb,1.6338e-05_rb,2.4863e-05_rb,3.7137e-05_rb/)
      kao(:, 8, 6) = (/ &
     &4.3692e-06_rb,8.7526e-06_rb,1.4669e-05_rb,2.2987e-05_rb,3.5150e-05_rb/)
      kao(:, 9, 6) = (/ &
     &5.3838e-06_rb,1.0727e-05_rb,1.8376e-05_rb,3.0485e-05_rb,4.8603e-05_rb/)
      kao(:,10, 6) = (/ &
     &6.7708e-06_rb,1.4529e-05_rb,2.6810e-05_rb,4.7617e-05_rb,7.8536e-05_rb/)
      kao(:,11, 6) = (/ &
     &8.3666e-06_rb,1.7190e-05_rb,3.3330e-05_rb,5.7837e-05_rb,9.5886e-05_rb/)
      kao(:,12, 6) = (/ &
     &8.6264e-06_rb,1.8503e-05_rb,3.4797e-05_rb,6.1435e-05_rb,1.0046e-04_rb/)
      kao(:,13, 6) = (/ &
     &8.2669e-06_rb,1.8055e-05_rb,3.3739e-05_rb,5.7805e-05_rb,9.4677e-05_rb/)
      kao(:, 1, 7) = (/ &
     &5.0894e-05_rb,6.7019e-05_rb,1.1702e-04_rb,2.4649e-04_rb,3.5046e-04_rb/)
      kao(:, 2, 7) = (/ &
     &4.3304e-05_rb,4.9708e-05_rb,1.0701e-04_rb,1.8228e-04_rb,2.5635e-04_rb/)
      kao(:, 3, 7) = (/ &
     &2.6855e-05_rb,4.5972e-05_rb,9.9426e-05_rb,1.4867e-04_rb,2.0722e-04_rb/)
      kao(:, 4, 7) = (/ &
     &2.0223e-05_rb,4.9973e-05_rb,8.2293e-05_rb,1.1822e-04_rb,1.6484e-04_rb/)
      kao(:, 5, 7) = (/ &
     &1.8923e-05_rb,4.1445e-05_rb,6.4003e-05_rb,9.2899e-05_rb,1.3171e-04_rb/)
      kao(:, 6, 7) = (/ &
     &1.5326e-05_rb,3.0718e-05_rb,4.7457e-05_rb,6.8025e-05_rb,9.5982e-05_rb/)
      kao(:, 7, 7) = (/ &
     &1.2448e-05_rb,2.1715e-05_rb,3.2987e-05_rb,4.9690e-05_rb,7.5010e-05_rb/)
      kao(:, 8, 7) = (/ &
     &1.0066e-05_rb,1.7102e-05_rb,2.7164e-05_rb,4.3976e-05_rb,6.9078e-05_rb/)
      kao(:, 9, 7) = (/ &
     &1.1685e-05_rb,2.0037e-05_rb,3.5620e-05_rb,5.8499e-05_rb,9.0997e-05_rb/)
      kao(:,10, 7) = (/ &
     &1.5861e-05_rb,2.8659e-05_rb,5.0802e-05_rb,8.4995e-05_rb,1.3698e-04_rb/)
      kao(:,11, 7) = (/ &
     &2.1379e-05_rb,3.8980e-05_rb,6.8341e-05_rb,1.1084e-04_rb,1.6745e-04_rb/)
      kao(:,12, 7) = (/ &
     &2.2487e-05_rb,4.0976e-05_rb,7.1749e-05_rb,1.1325e-04_rb,1.7051e-04_rb/)
      kao(:,13, 7) = (/ &
     &2.0952e-05_rb,3.7773e-05_rb,6.5636e-05_rb,1.0928e-04_rb,1.6488e-04_rb/)
      kao(:, 1, 8) = (/ &
     &1.0722e-04_rb,2.7983e-04_rb,4.9965e-04_rb,7.0430e-04_rb,9.5168e-04_rb/)
      kao(:, 2, 8) = (/ &
     &9.2566e-05_rb,2.4429e-04_rb,3.9143e-04_rb,5.5037e-04_rb,7.4482e-04_rb/)
      kao(:, 3, 8) = (/ &
     &8.2611e-05_rb,1.7009e-04_rb,2.5121e-04_rb,3.5806e-04_rb,4.9588e-04_rb/)
      kao(:, 4, 8) = (/ &
     &7.9199e-05_rb,1.3718e-04_rb,2.0767e-04_rb,3.0111e-04_rb,4.2301e-04_rb/)
      kao(:, 5, 8) = (/ &
     &6.6965e-05_rb,1.0994e-04_rb,1.6780e-04_rb,2.4484e-04_rb,3.4718e-04_rb/)
      kao(:, 6, 8) = (/ &
     &5.1557e-05_rb,8.4788e-05_rb,1.3259e-04_rb,1.9727e-04_rb,2.8537e-04_rb/)
      kao(:, 7, 8) = (/ &
     &3.8653e-05_rb,6.5013e-05_rb,1.0364e-04_rb,1.5866e-04_rb,2.2830e-04_rb/)
      kao(:, 8, 8) = (/ &
     &2.8949e-05_rb,4.9620e-05_rb,8.2718e-05_rb,1.3000e-04_rb,1.9256e-04_rb/)
      kao(:, 9, 8) = (/ &
     &2.6449e-05_rb,4.6951e-05_rb,7.7311e-05_rb,1.2263e-04_rb,1.9501e-04_rb/)
      kao(:,10, 8) = (/ &
     &4.1706e-05_rb,7.2947e-05_rb,1.1853e-04_rb,1.7821e-04_rb,2.7161e-04_rb/)
      kao(:,11, 8) = (/ &
     &4.6073e-05_rb,7.9291e-05_rb,1.2399e-04_rb,2.0469e-04_rb,3.0165e-04_rb/)
      kao(:,12, 8) = (/ &
     &4.9634e-05_rb,8.1819e-05_rb,1.2723e-04_rb,2.1479e-04_rb,3.2406e-04_rb/)
      kao(:,13, 8) = (/ &
     &4.8914e-05_rb,8.2845e-05_rb,1.3095e-04_rb,2.1611e-04_rb,3.1389e-04_rb/)
      kao(:, 1, 9) = (/ &
     &7.8561e-04_rb,1.1643e-03_rb,1.6394e-03_rb,2.2582e-03_rb,3.0560e-03_rb/)
      kao(:, 2, 9) = (/ &
     &6.6025e-04_rb,9.8571e-04_rb,1.4034e-03_rb,1.9479e-03_rb,2.6634e-03_rb/)
      kao(:, 3, 9) = (/ &
     &5.1307e-04_rb,7.6204e-04_rb,1.0925e-03_rb,1.5312e-03_rb,2.1163e-03_rb/)
      kao(:, 4, 9) = (/ &
     &3.4762e-04_rb,5.0219e-04_rb,7.2255e-04_rb,1.0302e-03_rb,1.4514e-03_rb/)
      kao(:, 5, 9) = (/ &
     &2.1712e-04_rb,3.4419e-04_rb,5.2965e-04_rb,7.8732e-04_rb,1.1464e-03_rb/)
      kao(:, 6, 9) = (/ &
     &1.8112e-04_rb,2.9769e-04_rb,4.6916e-04_rb,7.0360e-04_rb,1.0307e-03_rb/)
      kao(:, 7, 9) = (/ &
     &1.5449e-04_rb,2.5935e-04_rb,4.1089e-04_rb,6.2682e-04_rb,9.3587e-04_rb/)
      kao(:, 8, 9) = (/ &
     &1.4056e-04_rb,2.4062e-04_rb,3.8562e-04_rb,5.9348e-04_rb,8.8749e-04_rb/)
      kao(:, 9, 9) = (/ &
     &1.4311e-04_rb,2.5683e-04_rb,4.3997e-04_rb,6.9753e-04_rb,1.0500e-03_rb/)
      kao(:,10, 9) = (/ &
     &1.3733e-04_rb,2.3478e-04_rb,3.4047e-04_rb,4.8048e-04_rb,6.5263e-04_rb/)
      kao(:,11, 9) = (/ &
     &1.3199e-04_rb,1.9086e-04_rb,2.6427e-04_rb,3.3652e-04_rb,4.6218e-04_rb/)
      kao(:,12, 9) = (/ &
     &1.2421e-04_rb,1.8123e-04_rb,2.4759e-04_rb,3.0949e-04_rb,4.1038e-04_rb/)
      kao(:,13, 9) = (/ &
     &1.2135e-04_rb,1.9021e-04_rb,2.6678e-04_rb,3.4185e-04_rb,4.8239e-04_rb/)
      kao(:, 1,10) = (/ &
     &2.0319e-03_rb,2.8954e-03_rb,4.0208e-03_rb,5.4666e-03_rb,7.2307e-03_rb/)
      kao(:, 2,10) = (/ &
     &1.8594e-03_rb,2.6055e-03_rb,3.5896e-03_rb,4.9149e-03_rb,6.5643e-03_rb/)
      kao(:, 3,10) = (/ &
     &1.4858e-03_rb,2.1438e-03_rb,3.0067e-03_rb,4.1237e-03_rb,5.5345e-03_rb/)
      kao(:, 4,10) = (/ &
     &1.1237e-03_rb,1.6648e-03_rb,2.4078e-03_rb,3.3725e-03_rb,4.5628e-03_rb/)
      kao(:, 5,10) = (/ &
     &8.5681e-04_rb,1.0371e-03_rb,1.2665e-03_rb,1.6242e-03_rb,2.0962e-03_rb/)
      kao(:, 6,10) = (/ &
     &4.1878e-04_rb,6.5245e-04_rb,8.8673e-04_rb,1.4273e-03_rb,2.1165e-03_rb/)
      kao(:, 7,10) = (/ &
     &4.0657e-04_rb,6.7784e-04_rb,1.0299e-03_rb,1.5982e-03_rb,2.4416e-03_rb/)
      kao(:, 8,10) = (/ &
     &3.7189e-04_rb,6.5876e-04_rb,1.1259e-03_rb,1.8690e-03_rb,2.8514e-03_rb/)
      kao(:, 9,10) = (/ &
     &4.4113e-04_rb,8.4598e-04_rb,1.1991e-03_rb,1.8860e-03_rb,2.5835e-03_rb/)
      kao(:,10,10) = (/ &
     &1.8203e-04_rb,4.6909e-06_rb,8.8196e-06_rb,8.3149e-05_rb,1.6489e-04_rb/)
      kao(:,11,10) = (/ &
     &2.4843e-06_rb,5.0211e-06_rb,4.0342e-05_rb,1.0733e-04_rb,1.3735e-04_rb/)
      kao(:,12,10) = (/ &
     &2.2109e-06_rb,4.6775e-06_rb,6.3563e-05_rb,8.8385e-05_rb,1.1814e-04_rb/)
      kao(:,13,10) = (/ &
     &1.0306e-04_rb,2.9167e-05_rb,5.8184e-05_rb,8.2616e-05_rb,1.2141e-04_rb/)
      kao(:, 1,11) = (/ &
     &2.6851e-03_rb,3.8621e-03_rb,5.3296e-03_rb,7.1411e-03_rb,9.4750e-03_rb/)
      kao(:, 2,11) = (/ &
     &2.4213e-03_rb,3.4979e-03_rb,4.8929e-03_rb,6.6432e-03_rb,8.8031e-03_rb/)
      kao(:, 3,11) = (/ &
     &2.0817e-03_rb,2.9708e-03_rb,4.1510e-03_rb,5.7232e-03_rb,7.7624e-03_rb/)
      kao(:, 4,11) = (/ &
     &1.6627e-03_rb,2.4521e-03_rb,3.4576e-03_rb,4.7721e-03_rb,6.5543e-03_rb/)
      kao(:, 5,11) = (/ &
     &1.2664e-03_rb,1.9577e-03_rb,2.8702e-03_rb,4.0767e-03_rb,5.5807e-03_rb/)
      kao(:, 6,11) = (/ &
     &6.5451e-04_rb,7.6752e-04_rb,1.2109e-03_rb,1.8526e-03_rb,2.8298e-03_rb/)
      kao(:, 7,11) = (/ &
     &4.8621e-04_rb,7.7329e-04_rb,1.3520e-03_rb,2.1779e-03_rb,3.1034e-03_rb/)
      kao(:, 8,11) = (/ &
     &4.3573e-04_rb,8.1759e-04_rb,1.5409e-03_rb,2.1798e-03_rb,3.1170e-03_rb/)
      kao(:, 9,11) = (/ &
     &5.3220e-04_rb,6.5242e-04_rb,1.0365e-03_rb,8.6351e-04_rb,8.5932e-04_rb/)
      kao(:,10,11) = (/ &
     &4.1711e-06_rb,8.9963e-06_rb,1.7277e-05_rb,3.1054e-05_rb,5.1955e-05_rb/)
      kao(:,11,11) = (/ &
     &4.2641e-06_rb,9.2786e-06_rb,1.7917e-05_rb,3.2294e-05_rb,1.1099e-04_rb/)
      kao(:,12,11) = (/ &
     &4.4244e-06_rb,9.4799e-06_rb,1.8618e-05_rb,7.5019e-05_rb,1.5478e-04_rb/)
      kao(:,13,11) = (/ &
     &4.4439e-06_rb,9.5554e-06_rb,3.7663e-05_rb,9.5587e-05_rb,1.4453e-04_rb/)
      kao(:, 1,12) = (/ &
     &3.5676e-03_rb,4.9715e-03_rb,6.8023e-03_rb,9.2475e-03_rb,1.2425e-02_rb/)
      kao(:, 2,12) = (/ &
     &3.3730e-03_rb,4.7958e-03_rb,6.6635e-03_rb,9.0736e-03_rb,1.2194e-02_rb/)
      kao(:, 3,12) = (/ &
     &2.8824e-03_rb,4.1856e-03_rb,5.9756e-03_rb,8.2464e-03_rb,1.1067e-02_rb/)
      kao(:, 4,12) = (/ &
     &2.4288e-03_rb,3.5141e-03_rb,5.0677e-03_rb,7.1261e-03_rb,9.7127e-03_rb/)
      kao(:, 5,12) = (/ &
     &1.9807e-03_rb,2.9284e-03_rb,4.2016e-03_rb,5.9142e-03_rb,8.1484e-03_rb/)
      kao(:, 6,12) = (/ &
     &1.4728e-03_rb,2.2453e-03_rb,3.0533e-03_rb,3.8340e-03_rb,4.6395e-03_rb/)
      kao(:, 7,12) = (/ &
     &5.6245e-04_rb,9.9265e-04_rb,1.6138e-03_rb,2.6180e-03_rb,3.5983e-03_rb/)
      kao(:, 8,12) = (/ &
     &5.6232e-04_rb,1.0725e-03_rb,1.5736e-03_rb,2.7015e-03_rb,4.5325e-03_rb/)
      kao(:, 9,12) = (/ &
     &3.8503e-04_rb,4.5325e-04_rb,3.4069e-05_rb,5.9217e-05_rb,9.6016e-05_rb/)
      kao(:,10,12) = (/ &
     &6.7276e-06_rb,1.4849e-05_rb,2.8992e-05_rb,5.2371e-05_rb,8.7139e-05_rb/)
      kao(:,11,12) = (/ &
     &7.7381e-06_rb,1.7002e-05_rb,3.3523e-05_rb,6.0232e-05_rb,1.0004e-04_rb/)
      kao(:,12,12) = (/ &
     &8.5790e-06_rb,1.9107e-05_rb,3.7746e-05_rb,6.8157e-05_rb,1.1211e-04_rb/)
      kao(:,13,12) = (/ &
     &9.3399e-06_rb,2.1875e-05_rb,4.3407e-05_rb,7.7729e-05_rb,1.5956e-04_rb/)
      kao(:, 1,13) = (/ &
     &4.3788e-03_rb,6.3181e-03_rb,8.9543e-03_rb,1.2364e-02_rb,1.6648e-02_rb/)
      kao(:, 2,13) = (/ &
     &4.4160e-03_rb,6.2758e-03_rb,8.7769e-03_rb,1.2121e-02_rb,1.6531e-02_rb/)
      kao(:, 3,13) = (/ &
     &3.9967e-03_rb,5.8737e-03_rb,8.1369e-03_rb,1.1188e-02_rb,1.5344e-02_rb/)
      kao(:, 4,13) = (/ &
     &3.4004e-03_rb,5.1547e-03_rb,7.3735e-03_rb,1.0182e-02_rb,1.3939e-02_rb/)
      kao(:, 5,13) = (/ &
     &2.9021e-03_rb,4.3823e-03_rb,6.4495e-03_rb,9.1509e-03_rb,1.2669e-02_rb/)
      kao(:, 6,13) = (/ &
     &2.4396e-03_rb,3.6407e-03_rb,5.2497e-03_rb,7.5439e-03_rb,1.0675e-02_rb/)
      kao(:, 7,13) = (/ &
     &1.5932e-03_rb,1.8719e-03_rb,2.2033e-03_rb,2.5091e-03_rb,4.1046e-03_rb/)
      kao(:, 8,13) = (/ &
     &8.1439e-04_rb,1.2125e-03_rb,1.8190e-03_rb,2.8987e-03_rb,3.6148e-03_rb/)
      kao(:, 9,13) = (/ &
     &2.9962e-04_rb,2.5888e-05_rb,4.6085e-05_rb,7.7045e-05_rb,1.2285e-04_rb/)
      kao(:,10,13) = (/ &
     &9.2091e-06_rb,1.8817e-05_rb,3.5290e-05_rb,6.2611e-05_rb,1.0050e-04_rb/)
      kao(:,11,13) = (/ &
     &8.8922e-06_rb,1.8844e-05_rb,3.7039e-05_rb,6.5193e-05_rb,1.0998e-04_rb/)
      kao(:,12,13) = (/ &
     &9.1227e-06_rb,2.0282e-05_rb,3.9304e-05_rb,7.0536e-05_rb,1.1697e-04_rb/)
      kao(:,13,13) = (/ &
     &9.3384e-06_rb,2.0460e-05_rb,4.0361e-05_rb,7.2805e-05_rb,1.1815e-04_rb/)
      kao(:, 1,14) = (/ &
     &5.8176e-03_rb,8.5593e-03_rb,1.2130e-02_rb,1.6770e-02_rb,2.2682e-02_rb/)
      kao(:, 2,14) = (/ &
     &5.7661e-03_rb,8.5728e-03_rb,1.2312e-02_rb,1.7131e-02_rb,2.3220e-02_rb/)
      kao(:, 3,14) = (/ &
     &5.4299e-03_rb,7.9187e-03_rb,1.1613e-02_rb,1.6480e-02_rb,2.2670e-02_rb/)
      kao(:, 4,14) = (/ &
     &4.9805e-03_rb,7.1731e-03_rb,1.0470e-02_rb,1.5114e-02_rb,2.1203e-02_rb/)
      kao(:, 5,14) = (/ &
     &4.4098e-03_rb,6.5055e-03_rb,9.4518e-03_rb,1.3613e-02_rb,1.9313e-02_rb/)
      kao(:, 6,14) = (/ &
     &3.7762e-03_rb,5.6862e-03_rb,8.4288e-03_rb,1.2169e-02_rb,1.7260e-02_rb/)
      kao(:, 7,14) = (/ &
     &3.2464e-03_rb,4.8124e-03_rb,6.8092e-03_rb,9.1900e-03_rb,1.1370e-02_rb/)
      kao(:, 8,14) = (/ &
     &1.1521e-03_rb,1.1423e-03_rb,1.7687e-03_rb,1.5659e-03_rb,2.2329e-03_rb/)
      kao(:, 9,14) = (/ &
     &1.9075e-05_rb,4.4531e-05_rb,8.9053e-05_rb,1.1868e-04_rb,1.6905e-04_rb/)
      kao(:,10,14) = (/ &
     &1.1271e-05_rb,2.8156e-05_rb,6.0244e-05_rb,8.6340e-05_rb,1.3123e-04_rb/)
      kao(:,11,14) = (/ &
     &1.1591e-05_rb,2.8030e-05_rb,4.8484e-05_rb,7.4281e-05_rb,1.1022e-04_rb/)
      kao(:,12,14) = (/ &
     &1.1274e-05_rb,2.5700e-05_rb,4.0234e-05_rb,6.2656e-05_rb,9.8261e-05_rb/)
      kao(:,13,14) = (/ &
     &1.0130e-05_rb,1.8686e-05_rb,3.2466e-05_rb,5.4260e-05_rb,8.5980e-05_rb/)
      kao(:, 1,15) = (/ &
     &8.0623e-03_rb,1.1623e-02_rb,1.6754e-02_rb,2.3990e-02_rb,3.4103e-02_rb/)
      kao(:, 2,15) = (/ &
     &8.1053e-03_rb,1.1698e-02_rb,1.6865e-02_rb,2.3848e-02_rb,3.3543e-02_rb/)
      kao(:, 3,15) = (/ &
     &7.5058e-03_rb,1.0852e-02_rb,1.5814e-02_rb,2.2160e-02_rb,3.0813e-02_rb/)
      kao(:, 4,15) = (/ &
     &6.7816e-03_rb,9.9478e-03_rb,1.4382e-02_rb,2.0576e-02_rb,2.8388e-02_rb/)
      kao(:, 5,15) = (/ &
     &6.3211e-03_rb,9.1146e-03_rb,1.3301e-02_rb,1.9331e-02_rb,2.6874e-02_rb/)
      kao(:, 6,15) = (/ &
     &5.8531e-03_rb,8.4323e-03_rb,1.2188e-02_rb,1.7787e-02_rb,2.5518e-02_rb/)
      kao(:, 7,15) = (/ &
     &5.3028e-03_rb,7.8385e-03_rb,1.1331e-02_rb,1.6285e-02_rb,2.3804e-02_rb/)
      kao(:, 8,15) = (/ &
     &4.7139e-03_rb,4.9648e-03_rb,2.7936e-03_rb,2.1248e-03_rb,1.7982e-03_rb/)
      kao(:, 9,15) = (/ &
     &6.9507e-07_rb,1.4412e-06_rb,2.8345e-06_rb,1.0911e-04_rb,2.4370e-04_rb/)
      kao(:,10,15) = (/ &
     &3.6501e-07_rb,8.3446e-07_rb,1.6843e-06_rb,6.4065e-05_rb,1.5410e-04_rb/)
      kao(:,11,15) = (/ &
     &2.8886e-07_rb,6.6813e-07_rb,2.2629e-05_rb,7.8818e-05_rb,1.6163e-04_rb/)
      kao(:,12,15) = (/ &
     &2.6037e-07_rb,5.5485e-07_rb,3.1734e-05_rb,8.0123e-05_rb,1.4669e-04_rb/)
      kao(:,13,15) = (/ &
     &2.3446e-07_rb,1.2097e-05_rb,3.4961e-05_rb,7.1280e-05_rb,1.2611e-04_rb/)
      kao(:, 1,16) = (/ &
     &8.7570e-03_rb,1.2357e-02_rb,1.9959e-02_rb,3.2636e-02_rb,5.0897e-02_rb/)
      kao(:, 2,16) = (/ &
     &9.0825e-03_rb,1.2808e-02_rb,1.9776e-02_rb,3.3132e-02_rb,5.2704e-02_rb/)
      kao(:, 3,16) = (/ &
     &8.8201e-03_rb,1.2594e-02_rb,1.7588e-02_rb,3.0181e-02_rb,4.9544e-02_rb/)
      kao(:, 4,16) = (/ &
     &8.3170e-03_rb,1.2140e-02_rb,1.6975e-02_rb,2.5787e-02_rb,4.3638e-02_rb/)
      kao(:, 5,16) = (/ &
     &7.7605e-03_rb,1.1646e-02_rb,1.6641e-02_rb,2.2927e-02_rb,3.7740e-02_rb/)
      kao(:, 6,16) = (/ &
     &7.0356e-03_rb,1.0930e-02_rb,1.6057e-02_rb,2.2523e-02_rb,3.2226e-02_rb/)
      kao(:, 7,16) = (/ &
     &6.7105e-03_rb,1.0123e-02_rb,1.5354e-02_rb,2.2098e-02_rb,3.0452e-02_rb/)
      kao(:, 8,16) = (/ &
     &6.7590e-03_rb,9.3669e-03_rb,1.4563e-02_rb,1.3873e-02_rb,1.5872e-03_rb/)
      kao(:, 9,16) = (/ &
     &5.9690e-07_rb,1.4246e-06_rb,2.9975e-06_rb,4.2222e-06_rb,7.0710e-06_rb/)
      kao(:,10,16) = (/ &
     &3.1073e-07_rb,7.7259e-07_rb,1.8451e-06_rb,3.5923e-06_rb,5.6864e-06_rb/)
      kao(:,11,16) = (/ &
     &2.5196e-07_rb,6.5326e-07_rb,1.5669e-06_rb,3.1861e-06_rb,5.6813e-06_rb/)
      kao(:,12,16) = (/ &
     &1.8633e-07_rb,5.5942e-07_rb,1.3020e-06_rb,2.7030e-06_rb,4.5791e-06_rb/)
      kao(:,13,16) = (/ &
     &1.4070e-07_rb,4.5125e-07_rb,1.0269e-06_rb,2.1236e-06_rb,3.8826e-06_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kao_mco2(:, 1) = (/ &
     & 1.45661e-05_rb, 1.73337e-05_rb, 2.06273e-05_rb, 2.45466e-05_rb, 2.92105e-05_rb, &
     & 3.47607e-05_rb, 4.13654e-05_rb, 4.92251e-05_rb, 5.85781e-05_rb, 6.97083e-05_rb, &
     & 8.29533e-05_rb, 9.87149e-05_rb, 1.17471e-04_rb, 1.39792e-04_rb, 1.66353e-04_rb, &
     & 1.97961e-04_rb, 2.35574e-04_rb, 2.80335e-04_rb, 3.33600e-04_rb/)
      kao_mco2(:, 2) = (/ &
     & 9.96332e-06_rb, 1.21229e-05_rb, 1.47506e-05_rb, 1.79478e-05_rb, 2.18381e-05_rb, &
     & 2.65716e-05_rb, 3.23310e-05_rb, 3.93389e-05_rb, 4.78658e-05_rb, 5.82408e-05_rb, &
     & 7.08647e-05_rb, 8.62250e-05_rb, 1.04914e-04_rb, 1.27655e-04_rb, 1.55325e-04_rb, &
     & 1.88992e-04_rb, 2.29957e-04_rb, 2.79801e-04_rb, 3.40448e-04_rb/)
      kao_mco2(:, 3) = (/ &
     & 1.14968e-05_rb, 1.39890e-05_rb, 1.70215e-05_rb, 2.07115e-05_rb, 2.52013e-05_rb, &
     & 3.06644e-05_rb, 3.73118e-05_rb, 4.54002e-05_rb, 5.52420e-05_rb, 6.72173e-05_rb, &
     & 8.17887e-05_rb, 9.95188e-05_rb, 1.21092e-04_rb, 1.47343e-04_rb, 1.79283e-04_rb, &
     & 2.18148e-04_rb, 2.65438e-04_rb, 3.22980e-04_rb, 3.92995e-04_rb/)
      kao_mco2(:, 4) = (/ &
     & 1.02186e-05_rb, 1.23232e-05_rb, 1.48613e-05_rb, 1.79222e-05_rb, 2.16134e-05_rb, &
     & 2.60649e-05_rb, 3.14332e-05_rb, 3.79071e-05_rb, 4.57145e-05_rb, 5.51297e-05_rb, &
     & 6.64843e-05_rb, 8.01773e-05_rb, 9.66905e-05_rb, 1.16605e-04_rb, 1.40621e-04_rb, &
     & 1.69583e-04_rb, 2.04510e-04_rb, 2.46631e-04_rb, 2.97426e-04_rb/)
      kao_mco2(:, 5) = (/ &
     & 1.03469e-05_rb, 1.24680e-05_rb, 1.50239e-05_rb, 1.81037e-05_rb, 2.18149e-05_rb, &
     & 2.62869e-05_rb, 3.16756e-05_rb, 3.81690e-05_rb, 4.59935e-05_rb, 5.54220e-05_rb, &
     & 6.67833e-05_rb, 8.04737e-05_rb, 9.69704e-05_rb, 1.16849e-04_rb, 1.40803e-04_rb, &
     & 1.69667e-04_rb, 2.04448e-04_rb, 2.46359e-04_rb, 2.96861e-04_rb/)
      kao_mco2(:, 6) = (/ &
     & 1.71660e-05_rb, 2.07334e-05_rb, 2.50420e-05_rb, 3.02461e-05_rb, 3.65317e-05_rb, &
     & 4.41235e-05_rb, 5.32930e-05_rb, 6.43680e-05_rb, 7.77446e-05_rb, 9.39010e-05_rb, &
     & 1.13415e-04_rb, 1.36984e-04_rb, 1.65451e-04_rb, 1.99835e-04_rb, 2.41363e-04_rb, &
     & 2.91522e-04_rb, 3.52104e-04_rb, 4.25276e-04_rb, 5.13654e-04_rb/)
      kao_mco2(:, 7) = (/ &
     & 4.78803e-05_rb, 5.79395e-05_rb, 7.01119e-05_rb, 8.48418e-05_rb, 1.02666e-04_rb, &
     & 1.24235e-04_rb, 1.50336e-04_rb, 1.81920e-04_rb, 2.20139e-04_rb, 2.66388e-04_rb, &
     & 3.22354e-04_rb, 3.90077e-04_rb, 4.72028e-04_rb, 5.71197e-04_rb, 6.91199e-04_rb, &
     & 8.36413e-04_rb, 1.01214e-03_rb, 1.22477e-03_rb, 1.48209e-03_rb/)
      kao_mco2(:, 8) = (/ &
     & 1.27954e-04_rb, 1.55281e-04_rb, 1.88445e-04_rb, 2.28692e-04_rb, 2.77534e-04_rb, &
     & 3.36808e-04_rb, 4.08741e-04_rb, 4.96037e-04_rb, 6.01977e-04_rb, 7.30542e-04_rb, &
     & 8.86566e-04_rb, 1.07591e-03_rb, 1.30570e-03_rb, 1.58456e-03_rb, 1.92298e-03_rb, &
     & 2.33367e-03_rb, 2.83208e-03_rb, 3.43694e-03_rb, 4.17097e-03_rb/)
      kao_mco2(:, 9) = (/ &
     & 2.93792e-05_rb, 3.55109e-05_rb, 4.29223e-05_rb, 5.18805e-05_rb, 6.27083e-05_rb, &
     & 7.57960e-05_rb, 9.16151e-05_rb, 1.10736e-04_rb, 1.33847e-04_rb, 1.61782e-04_rb, &
     & 1.95547e-04_rb, 2.36359e-04_rb, 2.85689e-04_rb, 3.45315e-04_rb, 4.17384e-04_rb, &
     & 5.04495e-04_rb, 6.09787e-04_rb, 7.37054e-04_rb, 8.90882e-04_rb/)
      kao_mco2(:,10) = (/ &
     & 5.08569e-05_rb, 6.24700e-05_rb, 7.67350e-05_rb, 9.42574e-05_rb, 1.15781e-04_rb, &
     & 1.42220e-04_rb, 1.74695e-04_rb, 2.14587e-04_rb, 2.63588e-04_rb, 3.23778e-04_rb, &
     & 3.97712e-04_rb, 4.88530e-04_rb, 6.00085e-04_rb, 7.37114e-04_rb, 9.05433e-04_rb, &
     & 1.11219e-03_rb, 1.36616e-03_rb, 1.67812e-03_rb, 2.06131e-03_rb/)
      kao_mco2(:,11) = (/ &
     & 4.82546e-06_rb, 6.21462e-06_rb, 8.00369e-06_rb, 1.03078e-05_rb, 1.32752e-05_rb, &
     & 1.70969e-05_rb, 2.20188e-05_rb, 2.83575e-05_rb, 3.65211e-05_rb, 4.70348e-05_rb, &
     & 6.05753e-05_rb, 7.80138e-05_rb, 1.00472e-04_rb, 1.29397e-04_rb, 1.66647e-04_rb, &
     & 2.14622e-04_rb, 2.76407e-04_rb, 3.55980e-04_rb, 4.58459e-04_rb/)
      kao_mco2(:,12) = (/ &
     & 2.41346e-06_rb, 2.96282e-06_rb, 3.63723e-06_rb, 4.46516e-06_rb, 5.48153e-06_rb, &
     & 6.72926e-06_rb, 8.26100e-06_rb, 1.01414e-05_rb, 1.24498e-05_rb, 1.52837e-05_rb, &
     & 1.87627e-05_rb, 2.30335e-05_rb, 2.82765e-05_rb, 3.47129e-05_rb, 4.26144e-05_rb, &
     & 5.23144e-05_rb, 6.42225e-05_rb, 7.88410e-05_rb, 9.67871e-05_rb/)
      kao_mco2(:,13) = (/ &
     & 2.76412e-06_rb, 3.46195e-06_rb, 4.33596e-06_rb, 5.43062e-06_rb, 6.80164e-06_rb, &
     & 8.51879e-06_rb, 1.06695e-05_rb, 1.33631e-05_rb, 1.67367e-05_rb, 2.09621e-05_rb, &
     & 2.62542e-05_rb, 3.28824e-05_rb, 4.11839e-05_rb, 5.15813e-05_rb, 6.46035e-05_rb, &
     & 8.09134e-05_rb, 1.01341e-04_rb, 1.26925e-04_rb, 1.58969e-04_rb/)
      kao_mco2(:,14) = (/ &
     & 1.25126e-06_rb, 1.54971e-06_rb, 1.91935e-06_rb, 2.37715e-06_rb, 2.94416e-06_rb, &
     & 3.64640e-06_rb, 4.51615e-06_rb, 5.59335e-06_rb, 6.92749e-06_rb, 8.57985e-06_rb, &
     & 1.06263e-05_rb, 1.31610e-05_rb, 1.63001e-05_rb, 2.01881e-05_rb, 2.50034e-05_rb, &
     & 3.09672e-05_rb, 3.83536e-05_rb, 4.75018e-05_rb, 5.88319e-05_rb/)
      kao_mco2(:,15) = (/ &
     & 1.59748e-06_rb, 2.08378e-06_rb, 2.71812e-06_rb, 3.54557e-06_rb, 4.62491e-06_rb, &
     & 6.03282e-06_rb, 7.86932e-06_rb, 1.02649e-05_rb, 1.33897e-05_rb, 1.74658e-05_rb, &
     & 2.27827e-05_rb, 2.97182e-05_rb, 3.87649e-05_rb, 5.05657e-05_rb, 6.59589e-05_rb, &
     & 8.60380e-05_rb, 1.12230e-04_rb, 1.46394e-04_rb, 1.90959e-04_rb/)
      kao_mco2(:,16) = (/ &
     & 1.68148e-06_rb, 2.17133e-06_rb, 2.80388e-06_rb, 3.62071e-06_rb, 4.67549e-06_rb, &
     & 6.03756e-06_rb, 7.79642e-06_rb, 1.00677e-05_rb, 1.30006e-05_rb, 1.67879e-05_rb, &
     & 2.16786e-05_rb, 2.79941e-05_rb, 3.61493e-05_rb, 4.66803e-05_rb, 6.02792e-05_rb, &
     & 7.78398e-05_rb, 1.00516e-04_rb, 1.29799e-04_rb, 1.67612e-04_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &3.2710e-07_rb,5.2119e-07_rb,8.4740e-07_rb,1.6908e-06_rb,2.3433e-06_rb,4.4129e-06_rb, &
     &3.8930e-06_rb,2.3338e-06_rb,2.4115e-06_rb,2.4271e-06_rb,2.4836e-06_rb,2.6470e-06_rb, &
     &2.9559e-06_rb,2.3940e-06_rb,2.9711e-06_rb,2.9511e-06_rb/)
      forrefo(2,:) = (/ &
     &6.5125e-07_rb,1.2128e-06_rb,1.7249e-06_rb,2.7126e-06_rb,3.1780e-06_rb,2.1444e-06_rb, &
     &1.8265e-06_rb,1.7385e-06_rb,1.4574e-06_rb,1.6135e-06_rb,2.4966e-06_rb,2.8127e-06_rb, &
     &2.5229e-06_rb,2.3251e-06_rb,2.5353e-06_rb,3.0200e-06_rb/)
      forrefo(3,:) = (/ &
     &1.4969e-06_rb,1.8516e-06_rb,2.5791e-06_rb,2.7846e-06_rb,1.9789e-06_rb,1.6688e-06_rb, &
     &1.1037e-06_rb,9.9065e-07_rb,1.1557e-06_rb,7.0847e-07_rb,5.7758e-07_rb,4.0425e-07_rb, &
     &3.2427e-07_rb,3.2267e-07_rb,3.1444e-07_rb,2.6046e-07_rb/)
      forrefo(4,:) = (/ &
     &1.7567e-06_rb,1.6891e-06_rb,2.1003e-06_rb,2.0957e-06_rb,2.3664e-06_rb,2.1538e-06_rb, &
     &1.5275e-06_rb,1.0487e-06_rb,8.7390e-07_rb,7.9360e-07_rb,7.7778e-07_rb,8.1445e-07_rb, &
     &8.2121e-07_rb,5.4395e-07_rb,3.1273e-07_rb,3.1848e-07_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 7.73921e-02_rb, 6.45225e-02_rb, 5.37930e-02_rb, 4.48477e-02_rb, 3.73900e-02_rb, &
     & 3.11723e-02_rb, 2.59887e-02_rb, 2.16670e-02_rb, 1.80640e-02_rb, 1.50601e-02_rb/)
      selfrefo(:, 2) = (/ &
     & 8.47756e-02_rb, 7.10616e-02_rb, 5.95660e-02_rb, 4.99301e-02_rb, 4.18529e-02_rb, &
     & 3.50824e-02_rb, 2.94072e-02_rb, 2.46500e-02_rb, 2.06624e-02_rb, 1.73199e-02_rb/)
      selfrefo(:, 3) = (/ &
     & 8.84829e-02_rb, 7.46093e-02_rb, 6.29110e-02_rb, 5.30469e-02_rb, 4.47295e-02_rb, &
     & 3.77161e-02_rb, 3.18025e-02_rb, 2.68160e-02_rb, 2.26114e-02_rb, 1.90661e-02_rb/)
      selfrefo(:, 4) = (/ &
     & 9.27003e-02_rb, 7.88864e-02_rb, 6.71310e-02_rb, 5.71273e-02_rb, 4.86144e-02_rb, &
     & 4.13700e-02_rb, 3.52052e-02_rb, 2.99590e-02_rb, 2.54946e-02_rb, 2.16955e-02_rb/)
      selfrefo(:, 5) = (/ &
     & 9.14315e-02_rb, 7.85661e-02_rb, 6.75110e-02_rb, 5.80115e-02_rb, 4.98487e-02_rb, &
     & 4.28344e-02_rb, 3.68072e-02_rb, 3.16280e-02_rb, 2.71776e-02_rb, 2.33534e-02_rb/)
      selfrefo(:, 6) = (/ &
     & 7.72984e-02_rb, 6.91044e-02_rb, 6.17790e-02_rb, 5.52301e-02_rb, 4.93755e-02_rb, &
     & 4.41414e-02_rb, 3.94622e-02_rb, 3.52790e-02_rb, 3.15392e-02_rb, 2.81959e-02_rb/)
      selfrefo(:, 7) = (/ &
     & 7.46998e-02_rb, 6.66597e-02_rb, 5.94850e-02_rb, 5.30825e-02_rb, 4.73691e-02_rb, &
     & 4.22707e-02_rb, 3.77210e-02_rb, 3.36610e-02_rb, 3.00380e-02_rb, 2.68049e-02_rb/)
      selfrefo(:, 8) = (/ &
     & 7.59386e-02_rb, 6.66263e-02_rb, 5.84560e-02_rb, 5.12876e-02_rb, 4.49982e-02_rb, &
     & 3.94801e-02_rb, 3.46387e-02_rb, 3.03910e-02_rb, 2.66642e-02_rb, 2.33944e-02_rb/)
      selfrefo(:, 9) = (/ &
     & 7.26921e-02_rb, 6.43261e-02_rb, 5.69230e-02_rb, 5.03719e-02_rb, 4.45747e-02_rb, &
     & 3.94447e-02_rb, 3.49051e-02_rb, 3.08880e-02_rb, 2.73332e-02_rb, 2.41875e-02_rb/)
      selfrefo(:,10) = (/ &
     & 7.43684e-02_rb, 6.58735e-02_rb, 5.83490e-02_rb, 5.16840e-02_rb, 4.57803e-02_rb, &
     & 4.05509e-02_rb, 3.59189e-02_rb, 3.18160e-02_rb, 2.81818e-02_rb, 2.49626e-02_rb/)
      selfrefo(:,11) = (/ &
     & 8.97599e-02_rb, 7.73727e-02_rb, 6.66950e-02_rb, 5.74908e-02_rb, 4.95569e-02_rb, &
     & 4.27179e-02_rb, 3.68227e-02_rb, 3.17410e-02_rb, 2.73606e-02_rb, 2.35848e-02_rb/)
      selfrefo(:,12) = (/ &
     & 9.12262e-02_rb, 7.84848e-02_rb, 6.75230e-02_rb, 5.80922e-02_rb, 4.99786e-02_rb, &
     & 4.29982e-02_rb, 3.69927e-02_rb, 3.18260e-02_rb, 2.73809e-02_rb, 2.35567e-02_rb/)
      selfrefo(:,13) = (/ &
     & 9.03254e-02_rb, 7.83291e-02_rb, 6.79260e-02_rb, 5.89046e-02_rb, 5.10813e-02_rb, &
     & 4.42970e-02_rb, 3.84139e-02_rb, 3.33120e-02_rb, 2.88877e-02_rb, 2.50511e-02_rb/)
      selfrefo(:,14) = (/ &
     & 9.22803e-02_rb, 7.94172e-02_rb, 6.83470e-02_rb, 5.88199e-02_rb, 5.06209e-02_rb, &
     & 4.35647e-02_rb, 3.74921e-02_rb, 3.22660e-02_rb, 2.77684e-02_rb, 2.38977e-02_rb/)
      selfrefo(:,15) = (/ &
     & 9.36819e-02_rb, 8.10810e-02_rb, 7.01750e-02_rb, 6.07359e-02_rb, 5.25665e-02_rb, &
     & 4.54959e-02_rb, 3.93764e-02_rb, 3.40800e-02_rb, 2.94960e-02_rb, 2.55286e-02_rb/)
      selfrefo(:,16) = (/ &
     & 1.00195e-01_rb, 8.58713e-02_rb, 7.35950e-02_rb, 6.30737e-02_rb, 5.40566e-02_rb, &
     & 4.63286e-02_rb, 3.97054e-02_rb, 3.40290e-02_rb, 2.91641e-02_rb, 2.49948e-02_rb/)

      end subroutine lw_kgb06

! **************************************************************************
      subroutine lw_kgb07
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg07, only : fracrefao, fracrefbo, kao, kbo, kao_mco2, &
                            kbo_mco2, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P = 706.27 mb, T = 278.94 K
      fracrefao(:, 1) = (/ &
        1.6312e-01_rb,1.4949e-01_rb,1.4305e-01_rb,1.3161e-01_rb,1.1684e-01_rb,9.9900e-02_rb, &
        8.0912e-02_rb,6.0203e-02_rb,4.0149e-02_rb,4.3365e-03_rb,3.5844e-03_rb,2.8019e-03_rb, &
        2.0756e-03_rb,1.3449e-03_rb,5.0492e-04_rb,7.1194e-05_rb/)
      fracrefao(:, 2) = (/ &
        1.6329e-01_rb,1.4989e-01_rb,1.4328e-01_rb,1.3101e-01_rb,1.1691e-01_rb,9.9754e-02_rb, &
        8.0956e-02_rb,5.9912e-02_rb,4.0271e-02_rb,4.3298e-03_rb,3.5626e-03_rb,2.8421e-03_rb, &
        2.1031e-03_rb,1.3360e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)
      fracrefao(:, 3) = (/ &
        1.6236e-01_rb,1.5081e-01_rb,1.4341e-01_rb,1.3083e-01_rb,1.1684e-01_rb,9.9701e-02_rb, &
        8.0956e-02_rb,5.9884e-02_rb,4.0245e-02_rb,4.3837e-03_rb,3.6683e-03_rb,2.9250e-03_rb, &
        2.0969e-03_rb,1.3320e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)
      fracrefao(:, 4) = (/ &
        1.6096e-01_rb,1.5183e-01_rb,1.4354e-01_rb,1.3081e-01_rb,1.1687e-01_rb,9.9619e-02_rb, &
        8.0947e-02_rb,5.9899e-02_rb,4.0416e-02_rb,4.4389e-03_rb,3.7280e-03_rb,2.9548e-03_rb, &
        2.0977e-03_rb,1.3305e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)
      fracrefao(:, 5) = (/ &
        1.5661e-01_rb,1.5478e-01_rb,1.4414e-01_rb,1.3097e-01_rb,1.1695e-01_rb,9.9823e-02_rb, &
        8.0750e-02_rb,6.0100e-02_rb,4.0741e-02_rb,4.4598e-03_rb,3.7366e-03_rb,2.9521e-03_rb, &
        2.0980e-03_rb,1.3297e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)
      fracrefao(:, 6) = (/ &
        1.4879e-01_rb,1.5853e-01_rb,1.4586e-01_rb,1.3162e-01_rb,1.1729e-01_rb,1.0031e-01_rb, &
        8.0908e-02_rb,6.0460e-02_rb,4.1100e-02_rb,4.4578e-03_rb,3.7388e-03_rb,2.9508e-03_rb, &
        2.0986e-03_rb,1.3288e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)
      fracrefao(:, 7) = (/ &
        1.4117e-01_rb,1.4838e-01_rb,1.4807e-01_rb,1.3759e-01_rb,1.2218e-01_rb,1.0228e-01_rb, &
        8.2130e-02_rb,6.1546e-02_rb,4.1522e-02_rb,4.4577e-03_rb,3.7428e-03_rb,2.9475e-03_rb, &
        2.0997e-03_rb,1.3277e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)
      fracrefao(:, 8) = (/ &
        1.4018e-01_rb,1.4207e-01_rb,1.3919e-01_rb,1.3332e-01_rb,1.2325e-01_rb,1.0915e-01_rb, &
        9.0280e-02_rb,6.5554e-02_rb,4.1852e-02_rb,4.4707e-03_rb,3.7572e-03_rb,2.9364e-03_rb, &
        2.1023e-03_rb,1.3249e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)
      fracrefao(:, 9) = (/ &
        1.4863e-01_rb,1.4926e-01_rb,1.4740e-01_rb,1.3558e-01_rb,1.1999e-01_rb,1.0044e-01_rb, &
        8.1927e-02_rb,6.0989e-02_rb,4.0665e-02_rb,4.4481e-03_rb,3.7369e-03_rb,2.9482e-03_rb, &
        2.0976e-03_rb,1.3281e-03_rb,4.8965e-04_rb,6.8900e-05_rb/)

! Planck fraction mapping level : P=95.58 mbar, T= 215.70 K
      fracrefbo(:) = (/ &
        1.5872e-01_rb,1.5443e-01_rb,1.4413e-01_rb,1.3147e-01_rb,1.1634e-01_rb,9.8914e-02_rb, &
        8.0236e-02_rb,6.0197e-02_rb,4.0624e-02_rb,4.4225e-03_rb,3.6688e-03_rb,2.9074e-03_rb, &
        2.0862e-03_rb,1.3039e-03_rb,4.8561e-04_rb,6.8854e-05_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &2.0715e-06_rb,2.0394e-06_rb,5.0240e-06_rb,1.0614e-05_rb,1.1983e-05_rb,1.5554e-05_rb, &
     &1.7831e-05_rb,2.6227e-05_rb,1.2374e-05_rb/)
      kao(:, 2, 1, 1) = (/ &
     &2.6968e-06_rb,2.7207e-06_rb,3.6723e-06_rb,1.1698e-05_rb,1.3601e-05_rb,1.5350e-05_rb, &
     &2.0818e-05_rb,2.8393e-05_rb,1.1509e-05_rb/)
      kao(:, 3, 1, 1) = (/ &
     &3.4058e-06_rb,3.5757e-06_rb,3.8081e-06_rb,7.3554e-06_rb,1.4372e-05_rb,1.6368e-05_rb, &
     &1.9019e-05_rb,3.0058e-05_rb,1.1256e-05_rb/)
      kao(:, 4, 1, 1) = (/ &
     &4.2245e-06_rb,4.8570e-06_rb,4.7347e-06_rb,6.0087e-06_rb,1.4347e-05_rb,1.6902e-05_rb, &
     &1.9725e-05_rb,2.9609e-05_rb,1.0323e-05_rb/)
      kao(:, 5, 1, 1) = (/ &
     &5.1348e-06_rb,6.1189e-06_rb,5.9979e-06_rb,6.4031e-06_rb,9.6268e-06_rb,1.7276e-05_rb, &
     &1.9533e-05_rb,2.5577e-05_rb,1.1200e-05_rb/)
      kao(:, 1, 2, 1) = (/ &
     &2.7400e-06_rb,2.6398e-06_rb,3.5715e-06_rb,8.9653e-06_rb,1.1704e-05_rb,1.2732e-05_rb, &
     &1.5109e-05_rb,1.8326e-05_rb,7.6584e-06_rb/)
      kao(:, 2, 2, 1) = (/ &
     &3.7023e-06_rb,3.5996e-06_rb,3.5466e-06_rb,6.4138e-06_rb,1.4301e-05_rb,1.4484e-05_rb, &
     &1.7298e-05_rb,2.1003e-05_rb,7.8450e-06_rb/)
      kao(:, 3, 2, 1) = (/ &
     &4.8529e-06_rb,4.7902e-06_rb,4.4795e-06_rb,5.2052e-06_rb,1.0079e-05_rb,1.6425e-05_rb, &
     &1.7110e-05_rb,2.3533e-05_rb,8.0830e-06_rb/)
      kao(:, 4, 2, 1) = (/ &
     &6.0952e-06_rb,6.1266e-06_rb,5.7794e-06_rb,5.6603e-06_rb,7.5131e-06_rb,1.6775e-05_rb, &
     &1.7862e-05_rb,2.1237e-05_rb,6.9720e-06_rb/)
      kao(:, 5, 2, 1) = (/ &
     &7.4621e-06_rb,7.9816e-06_rb,7.3755e-06_rb,7.0507e-06_rb,7.2782e-06_rb,1.2172e-05_rb, &
     &1.8558e-05_rb,2.1490e-05_rb,7.0903e-06_rb/)
      kao(:, 1, 3, 1) = (/ &
     &4.8603e-06_rb,4.7726e-06_rb,4.1802e-06_rb,5.2554e-06_rb,9.0303e-06_rb,1.4041e-05_rb, &
     &1.3465e-05_rb,1.4273e-05_rb,4.5115e-06_rb/)
      kao(:, 2, 3, 1) = (/ &
     &6.5045e-06_rb,6.5033e-06_rb,5.8343e-06_rb,5.3408e-06_rb,7.3167e-06_rb,1.4520e-05_rb, &
     &1.5602e-05_rb,1.7982e-05_rb,4.5301e-06_rb/)
      kao(:, 3, 3, 1) = (/ &
     &8.5550e-06_rb,8.5301e-06_rb,7.7170e-06_rb,6.8750e-06_rb,6.6921e-06_rb,1.0610e-05_rb, &
     &1.8653e-05_rb,1.7976e-05_rb,3.9894e-06_rb/)
      kao(:, 4, 3, 1) = (/ &
     &1.0940e-05_rb,1.0943e-05_rb,9.9861e-06_rb,8.8481e-06_rb,7.8291e-06_rb,8.3918e-06_rb, &
     &1.7829e-05_rb,1.8992e-05_rb,3.8512e-06_rb/)
      kao(:, 5, 3, 1) = (/ &
     &1.3936e-05_rb,1.3811e-05_rb,1.2489e-05_rb,1.1153e-05_rb,9.8326e-06_rb,8.7685e-06_rb, &
     &1.2193e-05_rb,1.9703e-05_rb,4.2059e-06_rb/)
      kao(:, 1, 4, 1) = (/ &
     &8.9960e-06_rb,8.6210e-06_rb,7.6632e-06_rb,6.4630e-06_rb,6.4239e-06_rb,9.4367e-06_rb, &
     &1.6258e-05_rb,1.3756e-05_rb,2.4163e-06_rb/)
      kao(:, 2, 4, 1) = (/ &
     &1.2134e-05_rb,1.1807e-05_rb,1.0673e-05_rb,9.1029e-06_rb,7.7016e-06_rb,8.4861e-06_rb, &
     &1.4567e-05_rb,1.5036e-05_rb,2.1658e-06_rb/)
      kao(:, 3, 4, 1) = (/ &
     &1.6015e-05_rb,1.5688e-05_rb,1.4339e-05_rb,1.2455e-05_rb,1.0313e-05_rb,8.5777e-06_rb, &
     &1.1331e-05_rb,1.8116e-05_rb,2.2633e-06_rb/)
      kao(:, 4, 4, 1) = (/ &
     &2.0597e-05_rb,2.0155e-05_rb,1.8480e-05_rb,1.6165e-05_rb,1.3589e-05_rb,1.0837e-05_rb, &
     &9.4849e-06_rb,2.1131e-05_rb,2.4289e-06_rb/)
      kao(:, 5, 4, 1) = (/ &
     &2.5729e-05_rb,2.5473e-05_rb,2.3255e-05_rb,2.0429e-05_rb,1.7213e-05_rb,1.3828e-05_rb, &
     &1.0618e-05_rb,1.5874e-05_rb,3.2874e-06_rb/)
      kao(:, 1, 5, 1) = (/ &
     &1.6376e-05_rb,1.4981e-05_rb,1.3148e-05_rb,1.1171e-05_rb,9.2339e-06_rb,7.8648e-06_rb, &
     &1.1215e-05_rb,1.5160e-05_rb,1.0907e-06_rb/)
      kao(:, 2, 5, 1) = (/ &
     &2.2074e-05_rb,2.0407e-05_rb,1.8083e-05_rb,1.5475e-05_rb,1.2746e-05_rb,1.0050e-05_rb, &
     &1.0049e-05_rb,1.8902e-05_rb,1.3007e-06_rb/)
      kao(:, 3, 5, 1) = (/ &
     &2.9048e-05_rb,2.7094e-05_rb,2.4278e-05_rb,2.1033e-05_rb,1.7417e-05_rb,1.3618e-05_rb, &
     &1.0309e-05_rb,1.5056e-05_rb,1.4150e-06_rb/)
      kao(:, 4, 5, 1) = (/ &
     &3.7623e-05_rb,3.5233e-05_rb,3.1800e-05_rb,2.7623e-05_rb,2.3164e-05_rb,1.8269e-05_rb, &
     &1.3014e-05_rb,1.1951e-05_rb,2.0520e-06_rb/)
      kao(:, 5, 5, 1) = (/ &
     &4.7365e-05_rb,4.4556e-05_rb,4.0255e-05_rb,3.5279e-05_rb,2.9803e-05_rb,2.3695e-05_rb, &
     &1.6824e-05_rb,1.0864e-05_rb,3.0931e-06_rb/)
      kao(:, 1, 6, 1) = (/ &
     &2.9059e-05_rb,2.6117e-05_rb,2.2727e-05_rb,1.9184e-05_rb,1.5516e-05_rb,1.1998e-05_rb, &
     &9.5851e-06_rb,1.6016e-05_rb,6.6366e-07_rb/)
      kao(:, 2, 6, 1) = (/ &
     &3.9281e-05_rb,3.5568e-05_rb,3.1182e-05_rb,2.6432e-05_rb,2.1505e-05_rb,1.6402e-05_rb, &
     &1.1717e-05_rb,1.2828e-05_rb,7.2868e-07_rb/)
      kao(:, 3, 6, 1) = (/ &
     &5.2094e-05_rb,4.7264e-05_rb,4.1432e-05_rb,3.5398e-05_rb,2.9041e-05_rb,2.2416e-05_rb, &
     &1.5568e-05_rb,1.1439e-05_rb,1.0703e-06_rb/)
      kao(:, 4, 6, 1) = (/ &
     &6.7466e-05_rb,6.1301e-05_rb,5.3945e-05_rb,4.6275e-05_rb,3.8268e-05_rb,2.9782e-05_rb, &
     &2.0926e-05_rb,1.2149e-05_rb,1.6636e-06_rb/)
      kao(:, 5, 6, 1) = (/ &
     &8.5711e-05_rb,7.8253e-05_rb,6.9113e-05_rb,5.9588e-05_rb,4.9429e-05_rb,3.8720e-05_rb, &
     &2.7357e-05_rb,1.5302e-05_rb,2.4859e-06_rb/)
      kao(:, 1, 7, 1) = (/ &
     &5.4651e-05_rb,4.8396e-05_rb,4.1917e-05_rb,3.5300e-05_rb,2.8526e-05_rb,2.1678e-05_rb, &
     &1.5027e-05_rb,1.1124e-05_rb,3.7210e-07_rb/)
      kao(:, 2, 7, 1) = (/ &
     &7.4826e-05_rb,6.6392e-05_rb,5.7621e-05_rb,4.8629e-05_rb,3.9371e-05_rb,2.9959e-05_rb, &
     &2.0404e-05_rb,1.1926e-05_rb,4.6517e-07_rb/)
      kao(:, 3, 7, 1) = (/ &
     &9.9886e-05_rb,8.9008e-05_rb,7.7375e-05_rb,6.5427e-05_rb,5.3175e-05_rb,4.0667e-05_rb, &
     &2.7777e-05_rb,1.4838e-05_rb,7.6733e-07_rb/)
      kao(:, 4, 7, 1) = (/ &
     &1.3028e-04_rb,1.1642e-04_rb,1.0145e-04_rb,8.5974e-05_rb,7.0058e-05_rb,5.3698e-05_rb, &
     &3.6993e-05_rb,1.9660e-05_rb,1.2756e-06_rb/)
      kao(:, 5, 7, 1) = (/ &
     &1.6675e-04_rb,1.4925e-04_rb,1.3030e-04_rb,1.1056e-04_rb,9.0256e-05_rb,6.9412e-05_rb, &
     &4.8077e-05_rb,2.5971e-05_rb,2.0598e-06_rb/)
      kao(:, 1, 8, 1) = (/ &
     &1.3360e-04_rb,1.1739e-04_rb,1.0102e-04_rb,8.4552e-05_rb,6.8007e-05_rb,5.1398e-05_rb, &
     &3.4619e-05_rb,1.8035e-05_rb,1.9024e-07_rb/)
      kao(:, 2, 8, 1) = (/ &
     &1.8185e-04_rb,1.6006e-04_rb,1.3792e-04_rb,1.1564e-04_rb,9.3245e-05_rb,7.0615e-05_rb, &
     &4.7676e-05_rb,2.4529e-05_rb,3.0949e-07_rb/)
      kao(:, 3, 8, 1) = (/ &
     &2.4405e-04_rb,2.1504e-04_rb,1.8547e-04_rb,1.5570e-04_rb,1.2559e-04_rb,9.5220e-05_rb, &
     &6.4517e-05_rb,3.3270e-05_rb,5.6654e-07_rb/)
      kao(:, 4, 8, 1) = (/ &
     &3.2056e-04_rb,2.8296e-04_rb,2.4441e-04_rb,2.0538e-04_rb,1.6592e-04_rb,1.2607e-04_rb, &
     &8.5723e-05_rb,4.4370e-05_rb,1.0126e-06_rb/)
      kao(:, 5, 8, 1) = (/ &
     &4.1126e-04_rb,3.6352e-04_rb,3.1426e-04_rb,2.6419e-04_rb,2.1364e-04_rb,1.6265e-04_rb, &
     &1.1079e-04_rb,5.7674e-05_rb,1.7042e-06_rb/)
      kao(:, 1, 9, 1) = (/ &
     &6.2463e-04_rb,5.4698e-04_rb,4.6923e-04_rb,3.9140e-04_rb,3.1352e-04_rb,2.3558e-04_rb, &
     &1.5756e-04_rb,7.9405e-05_rb,1.4148e-07_rb/)
      kao(:, 2, 9, 1) = (/ &
     &8.5834e-04_rb,7.5200e-04_rb,6.4533e-04_rb,5.3850e-04_rb,4.3155e-04_rb,3.2449e-04_rb, &
     &2.1732e-04_rb,1.0986e-04_rb,2.9809e-07_rb/)
      kao(:, 3, 9, 1) = (/ &
     &1.1576e-03_rb,1.0147e-03_rb,8.7098e-04_rb,7.2712e-04_rb,5.8298e-04_rb,4.3871e-04_rb, &
     &2.9418e-04_rb,1.4894e-04_rb,5.9602e-07_rb/)
      kao(:, 4, 9, 1) = (/ &
     &1.5307e-03_rb,1.3423e-03_rb,1.1527e-03_rb,9.6254e-04_rb,7.7214e-04_rb,5.8148e-04_rb, &
     &3.9028e-04_rb,1.9773e-04_rb,1.1255e-06_rb/)
      kao(:, 5, 9, 1) = (/ &
     &1.9847e-03_rb,1.7410e-03_rb,1.4953e-03_rb,1.2490e-03_rb,1.0025e-03_rb,7.5537e-04_rb, &
     &5.0679e-04_rb,2.5732e-04_rb,1.9162e-06_rb/)
      kao(:, 1,10, 1) = (/ &
     &3.6169e-03_rb,3.1652e-03_rb,2.7133e-03_rb,2.2615e-03_rb,1.8097e-03_rb,1.3577e-03_rb, &
     &9.0574e-04_rb,4.5358e-04_rb,1.4845e-07_rb/)
      kao(:, 2,10, 1) = (/ &
     &4.9699e-03_rb,4.3495e-03_rb,3.7290e-03_rb,3.1084e-03_rb,2.4876e-03_rb,1.8667e-03_rb, &
     &1.2456e-03_rb,6.2420e-04_rb,3.2434e-07_rb/)
      kao(:, 3,10, 1) = (/ &
     &6.7347e-03_rb,5.8945e-03_rb,5.0543e-03_rb,4.2138e-03_rb,3.3728e-03_rb,2.5314e-03_rb, &
     &1.6895e-03_rb,8.4729e-04_rb,6.8830e-07_rb/)
      kao(:, 4,10, 1) = (/ &
     &8.9371e-03_rb,7.8231e-03_rb,6.7088e-03_rb,5.5938e-03_rb,4.4779e-03_rb,3.3612e-03_rb, &
     &2.2441e-03_rb,1.1262e-03_rb,1.3449e-06_rb/)
      kao(:, 5,10, 1) = (/ &
     &1.1613e-02_rb,1.0166e-02_rb,8.7190e-03_rb,7.2700e-03_rb,5.8199e-03_rb,4.3690e-03_rb, &
     &2.9176e-03_rb,1.4653e-03_rb,2.4729e-06_rb/)
      kao(:, 1,11, 1) = (/ &
     &8.2249e-03_rb,7.1970e-03_rb,6.1693e-03_rb,5.1415e-03_rb,4.1138e-03_rb,3.0859e-03_rb, &
     &2.0579e-03_rb,1.0298e-03_rb,1.4864e-07_rb/)
      kao(:, 2,11, 1) = (/ &
     &1.1270e-02_rb,9.8625e-03_rb,8.4545e-03_rb,7.0466e-03_rb,5.6386e-03_rb,4.2301e-03_rb, &
     &2.8215e-03_rb,1.4124e-03_rb,3.4574e-07_rb/)
      kao(:, 3,11, 1) = (/ &
     &1.5223e-02_rb,1.3322e-02_rb,1.1421e-02_rb,9.5200e-03_rb,7.6187e-03_rb,5.7167e-03_rb, &
     &3.8140e-03_rb,1.9100e-03_rb,7.2590e-07_rb/)
      kao(:, 4,11, 1) = (/ &
     &2.0093e-02_rb,1.7585e-02_rb,1.5077e-02_rb,1.2568e-02_rb,1.0059e-02_rb,7.5477e-03_rb, &
     &5.0358e-03_rb,2.5228e-03_rb,1.4168e-06_rb/)
      kao(:, 5,11, 1) = (/ &
     &2.5957e-02_rb,2.2717e-02_rb,1.9478e-02_rb,1.6238e-02_rb,1.2998e-02_rb,9.7533e-03_rb, &
     &6.5080e-03_rb,3.2606e-03_rb,2.6860e-06_rb/)
      kao(:, 1,12, 1) = (/ &
     &1.1827e-02_rb,1.0349e-02_rb,8.8708e-03_rb,7.3929e-03_rb,5.9148e-03_rb,4.4367e-03_rb, &
     &2.9587e-03_rb,1.4802e-03_rb,1.3931e-07_rb/)
      kao(:, 2,12, 1) = (/ &
     &1.6211e-02_rb,1.4185e-02_rb,1.2160e-02_rb,1.0134e-02_rb,8.1087e-03_rb,6.0831e-03_rb, &
     &4.0569e-03_rb,2.0301e-03_rb,3.2171e-07_rb/)
      kao(:, 3,12, 1) = (/ &
     &2.1828e-02_rb,1.9101e-02_rb,1.6374e-02_rb,1.3648e-02_rb,1.0921e-02_rb,8.1937e-03_rb, &
     &5.4651e-03_rb,2.7355e-03_rb,6.8081e-07_rb/)
      kao(:, 4,12, 1) = (/ &
     &2.8730e-02_rb,2.5142e-02_rb,2.1554e-02_rb,1.7967e-02_rb,1.4379e-02_rb,1.0788e-02_rb, &
     &7.1961e-03_rb,3.6027e-03_rb,1.3459e-06_rb/)
      kao(:, 5,12, 1) = (/ &
     &3.6999e-02_rb,3.2380e-02_rb,2.7760e-02_rb,2.3140e-02_rb,1.8520e-02_rb,1.3896e-02_rb, &
     &9.2700e-03_rb,4.6424e-03_rb,2.5690e-06_rb/)
      kao(:, 1,13, 1) = (/ &
     &1.3632e-02_rb,1.1928e-02_rb,1.0225e-02_rb,8.5212e-03_rb,6.8176e-03_rb,5.1139e-03_rb, &
     &3.4100e-03_rb,1.7059e-03_rb,1.1695e-07_rb/)
      kao(:, 2,13, 1) = (/ &
     &1.8706e-02_rb,1.6369e-02_rb,1.4031e-02_rb,1.1694e-02_rb,9.3569e-03_rb,7.0195e-03_rb, &
     &4.6811e-03_rb,2.3422e-03_rb,2.7119e-07_rb/)
      kao(:, 3,13, 1) = (/ &
     &2.5132e-02_rb,2.1993e-02_rb,1.8852e-02_rb,1.5713e-02_rb,1.2573e-02_rb,9.4331e-03_rb, &
     &6.2911e-03_rb,3.1482e-03_rb,5.7144e-07_rb/)
      kao(:, 4,13, 1) = (/ &
     &3.3041e-02_rb,2.8914e-02_rb,2.4787e-02_rb,2.0660e-02_rb,1.6533e-02_rb,1.2406e-02_rb, &
     &8.2739e-03_rb,4.1414e-03_rb,1.1335e-06_rb/)
      kao(:, 5,13, 1) = (/ &
     &4.2440e-02_rb,3.7140e-02_rb,3.1841e-02_rb,2.6541e-02_rb,2.1241e-02_rb,1.5937e-02_rb, &
     &1.0630e-02_rb,5.3218e-03_rb,2.1676e-06_rb/)
      kao(:, 1, 1, 2) = (/ &
     &4.1413e-06_rb,5.7568e-06_rb,5.6875e-06_rb,2.2265e-05_rb,2.6421e-05_rb,2.7053e-05_rb, &
     &2.9360e-05_rb,4.0970e-05_rb,1.4958e-05_rb/)
      kao(:, 2, 1, 2) = (/ &
     &5.1991e-06_rb,7.4465e-06_rb,7.4944e-06_rb,1.1514e-05_rb,2.6560e-05_rb,2.7448e-05_rb, &
     &2.9314e-05_rb,3.8310e-05_rb,1.7668e-05_rb/)
      kao(:, 3, 1, 2) = (/ &
     &6.3145e-06_rb,9.3150e-06_rb,9.9659e-06_rb,1.0415e-05_rb,2.4448e-05_rb,2.8503e-05_rb, &
     &3.1282e-05_rb,4.1769e-05_rb,1.5395e-05_rb/)
      kao(:, 4, 1, 2) = (/ &
     &7.5825e-06_rb,1.1197e-05_rb,1.2631e-05_rb,1.2849e-05_rb,1.2505e-05_rb,2.7705e-05_rb, &
     &3.1429e-05_rb,4.1459e-05_rb,1.9577e-05_rb/)
      kao(:, 5, 1, 2) = (/ &
     &9.0691e-06_rb,1.3548e-05_rb,1.5444e-05_rb,1.5939e-05_rb,1.4634e-05_rb,2.1478e-05_rb, &
     &3.0373e-05_rb,4.6504e-05_rb,1.5074e-05_rb/)
      kao(:, 1, 2, 2) = (/ &
     &6.1165e-06_rb,7.4934e-06_rb,6.8625e-06_rb,7.6121e-06_rb,3.0817e-05_rb,3.2183e-05_rb, &
     &3.0483e-05_rb,3.3496e-05_rb,1.0690e-05_rb/)
      kao(:, 2, 2, 2) = (/ &
     &7.6538e-06_rb,9.6280e-06_rb,9.7301e-06_rb,8.1097e-06_rb,1.6826e-05_rb,3.2430e-05_rb, &
     &3.0532e-05_rb,3.1851e-05_rb,9.7120e-06_rb/)
      kao(:, 3, 2, 2) = (/ &
     &9.2709e-06_rb,1.2057e-05_rb,1.2568e-05_rb,1.1762e-05_rb,1.2621e-05_rb,3.1032e-05_rb, &
     &3.2269e-05_rb,3.2750e-05_rb,1.1309e-05_rb/)
      kao(:, 4, 2, 2) = (/ &
     &1.1033e-05_rb,1.5040e-05_rb,1.5728e-05_rb,1.5466e-05_rb,1.4259e-05_rb,1.7169e-05_rb, &
     &3.1754e-05_rb,3.4497e-05_rb,1.2084e-05_rb/)
      kao(:, 5, 2, 2) = (/ &
     &1.2896e-05_rb,1.7847e-05_rb,1.9251e-05_rb,1.9193e-05_rb,1.8160e-05_rb,1.5443e-05_rb, &
     &2.9856e-05_rb,3.4316e-05_rb,1.0107e-05_rb/)
      kao(:, 1, 3, 2) = (/ &
     &1.2491e-05_rb,1.2769e-05_rb,1.2392e-05_rb,1.0198e-05_rb,9.0495e-06_rb,2.4479e-05_rb, &
     &4.2929e-05_rb,3.4687e-05_rb,5.7323e-06_rb/)
      kao(:, 2, 3, 2) = (/ &
     &1.5715e-05_rb,1.6378e-05_rb,1.5890e-05_rb,1.4699e-05_rb,1.1686e-05_rb,1.2801e-05_rb, &
     &4.3476e-05_rb,3.5317e-05_rb,5.8828e-06_rb/)
      kao(:, 3, 3, 2) = (/ &
     &1.9226e-05_rb,2.0527e-05_rb,2.0237e-05_rb,1.8901e-05_rb,1.6806e-05_rb,1.2767e-05_rb, &
     &3.2801e-05_rb,3.5650e-05_rb,7.0460e-06_rb/)
      kao(:, 4, 3, 2) = (/ &
     &2.3182e-05_rb,2.5284e-05_rb,2.5183e-05_rb,2.3943e-05_rb,2.1697e-05_rb,1.8156e-05_rb, &
     &1.6231e-05_rb,3.6132e-05_rb,6.5848e-06_rb/)
      kao(:, 5, 3, 2) = (/ &
     &2.7146e-05_rb,3.0591e-05_rb,3.0801e-05_rb,2.9528e-05_rb,2.6930e-05_rb,2.3450e-05_rb, &
     &1.7771e-05_rb,3.6010e-05_rb,8.7258e-06_rb/)
      kao(:, 1, 4, 2) = (/ &
     &2.6397e-05_rb,2.4798e-05_rb,2.2448e-05_rb,1.9931e-05_rb,1.6167e-05_rb,1.1187e-05_rb, &
     &1.9967e-05_rb,5.0094e-05_rb,2.6889e-06_rb/)
      kao(:, 2, 4, 2) = (/ &
     &3.3110e-05_rb,3.1403e-05_rb,2.8621e-05_rb,2.5685e-05_rb,2.2077e-05_rb,1.6320e-05_rb, &
     &1.2528e-05_rb,5.0766e-05_rb,3.5011e-06_rb/)
      kao(:, 3, 4, 2) = (/ &
     &4.1079e-05_rb,3.9262e-05_rb,3.5800e-05_rb,3.2065e-05_rb,2.7986e-05_rb,2.3135e-05_rb, &
     &1.5357e-05_rb,4.2293e-05_rb,3.6570e-06_rb/)
      kao(:, 4, 4, 2) = (/ &
     &4.9814e-05_rb,4.8178e-05_rb,4.4142e-05_rb,4.0008e-05_rb,3.5086e-05_rb,2.9575e-05_rb, &
     &2.2305e-05_rb,2.4338e-05_rb,5.5271e-06_rb/)
      kao(:, 5, 4, 2) = (/ &
     &5.8999e-05_rb,5.7911e-05_rb,5.3941e-05_rb,4.9131e-05_rb,4.3508e-05_rb,3.6745e-05_rb, &
     &2.8595e-05_rb,1.7749e-05_rb,7.9465e-06_rb/)
      kao(:, 1, 5, 2) = (/ &
     &5.2146e-05_rb,4.7415e-05_rb,4.1895e-05_rb,3.6100e-05_rb,2.9831e-05_rb,2.2853e-05_rb, &
     &1.2797e-05_rb,3.5615e-05_rb,1.8756e-06_rb/)
      kao(:, 2, 5, 2) = (/ &
     &6.6386e-05_rb,6.0667e-05_rb,5.3871e-05_rb,4.6587e-05_rb,3.8801e-05_rb,3.0645e-05_rb, &
     &1.9749e-05_rb,1.7747e-05_rb,1.8651e-06_rb/)
      kao(:, 3, 5, 2) = (/ &
     &8.2249e-05_rb,7.5613e-05_rb,6.7372e-05_rb,5.8338e-05_rb,4.8872e-05_rb,3.9037e-05_rb, &
     &2.7917e-05_rb,1.3785e-05_rb,2.9415e-06_rb/)
      kao(:, 4, 5, 2) = (/ &
     &1.0042e-04_rb,9.2832e-05_rb,8.2527e-05_rb,7.1569e-05_rb,6.0116e-05_rb,4.8290e-05_rb, &
     &3.5743e-05_rb,1.9373e-05_rb,4.5717e-06_rb/)
      kao(:, 5, 5, 2) = (/ &
     &1.2062e-04_rb,1.1245e-04_rb,1.0036e-04_rb,8.7051e-05_rb,7.3237e-05_rb,5.8949e-05_rb, &
     &4.4093e-05_rb,2.6901e-05_rb,6.6085e-06_rb/)
      kao(:, 1, 6, 2) = (/ &
     &9.7868e-05_rb,8.7352e-05_rb,7.6231e-05_rb,6.4624e-05_rb,5.2909e-05_rb,4.0689e-05_rb, &
     &2.7103e-05_rb,1.2458e-05_rb,1.0529e-06_rb/)
      kao(:, 2, 6, 2) = (/ &
     &1.2683e-04_rb,1.1378e-04_rb,9.9028e-05_rb,8.4157e-05_rb,6.8945e-05_rb,5.3462e-05_rb, &
     &3.6917e-05_rb,1.5991e-05_rb,1.3630e-06_rb/)
      kao(:, 3, 6, 2) = (/ &
     &1.5959e-04_rb,1.4323e-04_rb,1.2528e-04_rb,1.0669e-04_rb,8.7707e-05_rb,6.8030e-05_rb, &
     &4.7823e-05_rb,2.4263e-05_rb,2.3097e-06_rb/)
      kao(:, 4, 6, 2) = (/ &
     &1.9648e-04_rb,1.7730e-04_rb,1.5536e-04_rb,1.3239e-04_rb,1.0876e-04_rb,8.4595e-05_rb, &
     &5.9848e-05_rb,3.3068e-05_rb,3.7113e-06_rb/)
      kao(:, 5, 6, 2) = (/ &
     &2.3620e-04_rb,2.1365e-04_rb,1.8738e-04_rb,1.5973e-04_rb,1.3145e-04_rb,1.0268e-04_rb, &
     &7.2957e-05_rb,4.1703e-05_rb,5.6387e-06_rb/)
      kao(:, 1, 7, 2) = (/ &
     &1.9683e-04_rb,1.7373e-04_rb,1.5000e-04_rb,1.2604e-04_rb,1.0197e-04_rb,7.7797e-05_rb, &
     &5.2829e-05_rb,2.4870e-05_rb,6.2713e-07_rb/)
      kao(:, 2, 7, 2) = (/ &
     &2.5760e-04_rb,2.2793e-04_rb,1.9707e-04_rb,1.6593e-04_rb,1.3495e-04_rb,1.0291e-04_rb, &
     &7.0455e-05_rb,3.6046e-05_rb,1.0898e-06_rb/)
      kao(:, 3, 7, 2) = (/ &
     &3.2772e-04_rb,2.9045e-04_rb,2.5191e-04_rb,2.1251e-04_rb,1.7232e-04_rb,1.3160e-04_rb, &
     &9.0425e-05_rb,4.7870e-05_rb,1.9368e-06_rb/)
      kao(:, 4, 7, 2) = (/ &
     &4.0755e-04_rb,3.6253e-04_rb,3.1479e-04_rb,2.6553e-04_rb,2.1537e-04_rb,1.6486e-04_rb, &
     &1.1337e-04_rb,6.0602e-05_rb,3.1700e-06_rb/)
      kao(:, 5, 7, 2) = (/ &
     &4.9614e-04_rb,4.4343e-04_rb,3.8479e-04_rb,3.2448e-04_rb,2.6368e-04_rb,2.0203e-04_rb, &
     &1.3939e-04_rb,7.4791e-05_rb,4.9446e-06_rb/)
      kao(:, 1, 8, 2) = (/ &
     &5.0097e-04_rb,4.3939e-04_rb,3.7796e-04_rb,3.1605e-04_rb,2.5384e-04_rb,1.9144e-04_rb, &
     &1.2886e-04_rb,6.5770e-05_rb,4.5473e-07_rb/)
      kao(:, 2, 8, 2) = (/ &
     &6.6297e-04_rb,5.8193e-04_rb,5.0094e-04_rb,4.1901e-04_rb,3.3675e-04_rb,2.5431e-04_rb, &
     &1.7173e-04_rb,8.8413e-05_rb,9.2809e-07_rb/)
      kao(:, 3, 8, 2) = (/ &
     &8.5051e-04_rb,7.4751e-04_rb,6.4368e-04_rb,5.3885e-04_rb,4.3363e-04_rb,3.2808e-04_rb, &
     &2.2228e-04_rb,1.1512e-04_rb,1.7364e-06_rb/)
      kao(:, 4, 8, 2) = (/ &
     &1.0693e-03_rb,9.4111e-04_rb,8.1096e-04_rb,6.7960e-04_rb,5.4760e-04_rb,4.1579e-04_rb, &
     &2.8187e-04_rb,1.4563e-04_rb,2.9681e-06_rb/)
      kao(:, 5, 8, 2) = (/ &
     &1.3123e-03_rb,1.1563e-03_rb,9.9722e-04_rb,8.3695e-04_rb,6.7599e-04_rb,5.1257e-04_rb, &
     &3.4753e-04_rb,1.7994e-04_rb,4.7447e-06_rb/)
      kao(:, 1, 9, 2) = (/ &
     &2.4740e-03_rb,2.1654e-03_rb,1.8568e-03_rb,1.5484e-03_rb,1.2400e-03_rb,9.3161e-04_rb, &
     &6.2283e-04_rb,3.1298e-04_rb,4.6685e-07_rb/)
      kao(:, 2, 9, 2) = (/ &
     &3.2850e-03_rb,2.8753e-03_rb,2.4661e-03_rb,2.0570e-03_rb,1.6482e-03_rb,1.2389e-03_rb, &
     &8.2844e-04_rb,4.1698e-04_rb,9.7197e-07_rb/)
      kao(:, 3, 9, 2) = (/ &
     &4.2422e-03_rb,3.7138e-03_rb,3.1860e-03_rb,2.6588e-03_rb,2.1312e-03_rb,1.6022e-03_rb, &
     &1.0719e-03_rb,5.4052e-04_rb,1.8767e-06_rb/)
      kao(:, 4, 9, 2) = (/ &
     &5.3553e-03_rb,4.6891e-03_rb,4.0243e-03_rb,3.3598e-03_rb,2.6939e-03_rb,2.0261e-03_rb, &
     &1.3568e-03_rb,6.8626e-04_rb,3.3456e-06_rb/)
      kao(:, 5, 9, 2) = (/ &
     &6.6220e-03_rb,5.7995e-03_rb,4.9792e-03_rb,4.1587e-03_rb,3.3345e-03_rb,2.5085e-03_rb, &
     &1.6821e-03_rb,8.5317e-04_rb,5.6245e-06_rb/)
      kao(:, 1,10, 2) = (/ &
     &1.5050e-02_rb,1.3169e-02_rb,1.1289e-02_rb,9.4076e-03_rb,7.5270e-03_rb,5.6463e-03_rb, &
     &3.7658e-03_rb,1.8853e-03_rb,4.7266e-07_rb/)
      kao(:, 2,10, 2) = (/ &
     &1.9971e-02_rb,1.7476e-02_rb,1.4981e-02_rb,1.2485e-02_rb,9.9900e-03_rb,7.4947e-03_rb, &
     &4.9999e-03_rb,2.5053e-03_rb,1.0606e-06_rb/)
      kao(:, 3,10, 2) = (/ &
     &2.5996e-02_rb,2.2748e-02_rb,1.9499e-02_rb,1.6252e-02_rb,1.3005e-02_rb,9.7575e-03_rb, &
     &6.5102e-03_rb,3.2633e-03_rb,2.1957e-06_rb/)
      kao(:, 4,10, 2) = (/ &
     &3.3067e-02_rb,2.8935e-02_rb,2.4804e-02_rb,2.0673e-02_rb,1.6543e-02_rb,1.2414e-02_rb, &
     &8.2853e-03_rb,4.1549e-03_rb,4.2322e-06_rb/)
      kao(:, 5,10, 2) = (/ &
     &4.1113e-02_rb,3.5978e-02_rb,3.0842e-02_rb,2.5708e-02_rb,2.0575e-02_rb,1.5443e-02_rb, &
     &1.0311e-02_rb,5.1716e-03_rb,7.6651e-06_rb/)
      kao(:, 1,11, 2) = (/ &
     &3.5447e-02_rb,3.1017e-02_rb,2.6586e-02_rb,2.2156e-02_rb,1.7726e-02_rb,1.3295e-02_rb, &
     &8.8649e-03_rb,4.4348e-03_rb,5.0606e-07_rb/)
      kao(:, 2,11, 2) = (/ &
     &4.6742e-02_rb,4.0900e-02_rb,3.5058e-02_rb,2.9216e-02_rb,2.3374e-02_rb,1.7532e-02_rb, &
     &1.1691e-02_rb,5.8504e-03_rb,1.1095e-06_rb/)
      kao(:, 3,11, 2) = (/ &
     &6.0406e-02_rb,5.2857e-02_rb,4.5306e-02_rb,3.7757e-02_rb,3.0208e-02_rb,2.2660e-02_rb, &
     &1.5111e-02_rb,7.5645e-03_rb,2.3165e-06_rb/)
      kao(:, 4,11, 2) = (/ &
     &7.6519e-02_rb,6.6955e-02_rb,5.7393e-02_rb,4.7830e-02_rb,3.8268e-02_rb,2.8708e-02_rb, &
     &1.9148e-02_rb,9.5895e-03_rb,4.5048e-06_rb/)
      kao(:, 5,11, 2) = (/ &
     &9.5074e-02_rb,8.3196e-02_rb,7.1315e-02_rb,5.9435e-02_rb,4.7553e-02_rb,3.5677e-02_rb, &
     &2.3800e-02_rb,1.1926e-02_rb,8.0597e-06_rb/)
      kao(:, 1,12, 2) = (/ &
     &5.2692e-02_rb,4.6104e-02_rb,3.9518e-02_rb,3.2933e-02_rb,2.6346e-02_rb,1.9761e-02_rb, &
     &1.3176e-02_rb,6.5900e-03_rb,4.7314e-07_rb/)
      kao(:, 2,12, 2) = (/ &
     &6.9235e-02_rb,6.0581e-02_rb,5.1926e-02_rb,4.3273e-02_rb,3.4620e-02_rb,2.5967e-02_rb, &
     &1.7314e-02_rb,8.6616e-03_rb,1.0517e-06_rb/)
      kao(:, 3,12, 2) = (/ &
     &8.9222e-02_rb,7.8070e-02_rb,6.6918e-02_rb,5.5767e-02_rb,4.4616e-02_rb,3.3465e-02_rb, &
     &2.2315e-02_rb,1.1167e-02_rb,2.1820e-06_rb/)
      kao(:, 4,12, 2) = (/ &
     &1.1266e-01_rb,9.8579e-02_rb,8.4496e-02_rb,7.0418e-02_rb,5.6336e-02_rb,4.2258e-02_rb, &
     &2.8182e-02_rb,1.4107e-02_rb,4.2014e-06_rb/)
      kao(:, 5,12, 2) = (/ &
     &1.3942e-01_rb,1.2199e-01_rb,1.0457e-01_rb,8.7148e-02_rb,6.9722e-02_rb,5.2304e-02_rb, &
     &3.4887e-02_rb,1.7469e-02_rb,7.4819e-06_rb/)
      kao(:, 1,13, 2) = (/ &
     &6.2876e-02_rb,5.5018e-02_rb,4.7159e-02_rb,3.9299e-02_rb,3.1440e-02_rb,2.3581e-02_rb, &
     &1.5722e-02_rb,7.8632e-03_rb,4.0954e-07_rb/)
      kao(:, 2,13, 2) = (/ &
     &8.2400e-02_rb,7.2100e-02_rb,6.1801e-02_rb,5.1502e-02_rb,4.1203e-02_rb,3.0903e-02_rb, &
     &2.0605e-02_rb,1.0307e-02_rb,9.1322e-07_rb/)
      kao(:, 3,13, 2) = (/ &
     &1.0586e-01_rb,9.2633e-02_rb,7.9402e-02_rb,6.6169e-02_rb,5.2938e-02_rb,3.9706e-02_rb, &
     &2.6476e-02_rb,1.3248e-02_rb,1.8983e-06_rb/)
      kao(:, 4,13, 2) = (/ &
     &1.3330e-01_rb,1.1664e-01_rb,9.9979e-02_rb,8.3320e-02_rb,6.6657e-02_rb,4.9998e-02_rb, &
     &3.3342e-02_rb,1.6688e-02_rb,3.6054e-06_rb/)
      kao(:, 5,13, 2) = (/ &
     &1.6451e-01_rb,1.4395e-01_rb,1.2339e-01_rb,1.0283e-01_rb,8.2267e-02_rb,6.1710e-02_rb, &
     &4.1158e-02_rb,2.0605e-02_rb,6.1726e-06_rb/)
      kao(:, 1, 1, 3) = (/ &
     &1.0158e-05_rb,1.3198e-05_rb,1.2933e-05_rb,1.7710e-05_rb,3.0455e-05_rb,3.0829e-05_rb, &
     &3.6460e-05_rb,4.7058e-05_rb,1.7479e-05_rb/)
      kao(:, 2, 1, 3) = (/ &
     &1.1438e-05_rb,1.5570e-05_rb,1.6943e-05_rb,1.4755e-05_rb,2.6165e-05_rb,3.0134e-05_rb, &
     &3.4781e-05_rb,5.6113e-05_rb,2.1722e-05_rb/)
      kao(:, 3, 1, 3) = (/ &
     &1.2956e-05_rb,1.8129e-05_rb,1.9971e-05_rb,1.8485e-05_rb,1.6628e-05_rb,3.1264e-05_rb, &
     &3.4010e-05_rb,5.1158e-05_rb,2.4518e-05_rb/)
      kao(:, 4, 1, 3) = (/ &
     &1.4615e-05_rb,2.0830e-05_rb,2.2660e-05_rb,2.2625e-05_rb,1.9967e-05_rb,2.4812e-05_rb, &
     &3.9103e-05_rb,5.0423e-05_rb,1.7758e-05_rb/)
      kao(:, 5, 1, 3) = (/ &
     &1.6017e-05_rb,2.3480e-05_rb,2.5498e-05_rb,2.6186e-05_rb,2.5773e-05_rb,1.9360e-05_rb, &
     &3.6373e-05_rb,5.8002e-05_rb,2.1777e-05_rb/)
      kao(:, 1, 2, 3) = (/ &
     &1.6325e-05_rb,1.8415e-05_rb,1.8317e-05_rb,1.4470e-05_rb,2.4482e-05_rb,3.9161e-05_rb, &
     &3.6427e-05_rb,3.6776e-05_rb,1.2226e-05_rb/)
      kao(:, 2, 2, 3) = (/ &
     &1.8436e-05_rb,2.1492e-05_rb,2.2101e-05_rb,2.0928e-05_rb,1.7486e-05_rb,3.4569e-05_rb, &
     &3.4173e-05_rb,4.1432e-05_rb,1.6945e-05_rb/)
      kao(:, 3, 2, 3) = (/ &
     &2.0725e-05_rb,2.4812e-05_rb,2.5608e-05_rb,2.5315e-05_rb,2.0861e-05_rb,2.2865e-05_rb, &
     &3.3858e-05_rb,3.7106e-05_rb,1.4100e-05_rb/)
      kao(:, 4, 2, 3) = (/ &
     &2.3257e-05_rb,2.7888e-05_rb,2.9222e-05_rb,2.9285e-05_rb,2.7055e-05_rb,2.2581e-05_rb, &
     &3.3589e-05_rb,3.8281e-05_rb,1.2084e-05_rb/)
      kao(:, 5, 2, 3) = (/ &
     &2.5809e-05_rb,3.1443e-05_rb,3.3009e-05_rb,3.2988e-05_rb,3.1499e-05_rb,2.8005e-05_rb, &
     &2.1824e-05_rb,4.6522e-05_rb,1.8788e-05_rb/)
      kao(:, 1, 3, 3) = (/ &
     &3.6153e-05_rb,3.6160e-05_rb,3.3889e-05_rb,3.0515e-05_rb,2.3824e-05_rb,2.0510e-05_rb, &
     &5.5046e-05_rb,4.3679e-05_rb,8.4874e-06_rb/)
      kao(:, 2, 3, 3) = (/ &
     &4.1441e-05_rb,4.2015e-05_rb,3.9744e-05_rb,3.6451e-05_rb,3.1872e-05_rb,2.2157e-05_rb, &
     &3.7036e-05_rb,4.0290e-05_rb,7.7192e-06_rb/)
      kao(:, 3, 3, 3) = (/ &
     &4.6667e-05_rb,4.8191e-05_rb,4.5785e-05_rb,4.2457e-05_rb,3.7818e-05_rb,3.1272e-05_rb, &
     &2.3354e-05_rb,3.9936e-05_rb,6.8348e-06_rb/)
      kao(:, 4, 3, 3) = (/ &
     &5.1877e-05_rb,5.4060e-05_rb,5.1719e-05_rb,4.8126e-05_rb,4.3652e-05_rb,3.8051e-05_rb, &
     &2.7576e-05_rb,4.1113e-05_rb,1.0881e-05_rb/)
      kao(:, 5, 3, 3) = (/ &
     &5.7673e-05_rb,6.0087e-05_rb,5.7843e-05_rb,5.4035e-05_rb,4.9732e-05_rb,4.3859e-05_rb, &
     &3.4661e-05_rb,3.3926e-05_rb,1.5564e-05_rb/)
      kao(:, 1, 4, 3) = (/ &
     &8.1569e-05_rb,7.6506e-05_rb,6.9052e-05_rb,6.0598e-05_rb,5.1130e-05_rb,3.9208e-05_rb, &
     &2.1647e-05_rb,7.1331e-05_rb,4.7464e-06_rb/)
      kao(:, 2, 4, 3) = (/ &
     &9.5409e-05_rb,8.9513e-05_rb,8.1016e-05_rb,7.1228e-05_rb,6.0322e-05_rb,4.8270e-05_rb, &
     &3.1172e-05_rb,4.9123e-05_rb,3.8536e-06_rb/)
      kao(:, 3, 4, 3) = (/ &
     &1.0835e-04_rb,1.0286e-04_rb,9.3383e-05_rb,8.2438e-05_rb,7.0089e-05_rb,5.7019e-05_rb, &
     &4.1480e-05_rb,3.2272e-05_rb,5.8347e-06_rb/)
      kao(:, 4, 4, 3) = (/ &
     &1.2088e-04_rb,1.1587e-04_rb,1.0589e-04_rb,9.3297e-05_rb,7.9862e-05_rb,6.5329e-05_rb, &
     &4.9231e-05_rb,2.5421e-05_rb,9.1036e-06_rb/)
      kao(:, 5, 4, 3) = (/ &
     &1.3349e-04_rb,1.2852e-04_rb,1.1738e-04_rb,1.0384e-04_rb,8.9286e-05_rb,7.3614e-05_rb, &
     &5.6906e-05_rb,3.4260e-05_rb,1.3290e-05_rb/)
      kao(:, 1, 5, 3) = (/ &
     &1.7345e-04_rb,1.5727e-04_rb,1.3841e-04_rb,1.1872e-04_rb,9.8241e-05_rb,7.6833e-05_rb, &
     &5.2762e-05_rb,2.6555e-05_rb,1.9880e-06_rb/)
      kao(:, 2, 5, 3) = (/ &
     &2.0342e-04_rb,1.8508e-04_rb,1.6349e-04_rb,1.4064e-04_rb,1.1678e-04_rb,9.1597e-05_rb, &
     &6.4678e-05_rb,2.5634e-05_rb,3.0971e-06_rb/)
      kao(:, 3, 5, 3) = (/ &
     &2.3455e-04_rb,2.1397e-04_rb,1.8954e-04_rb,1.6295e-04_rb,1.3528e-04_rb,1.0632e-04_rb, &
     &7.5971e-05_rb,4.0302e-05_rb,5.0300e-06_rb/)
      kao(:, 4, 5, 3) = (/ &
     &2.6426e-04_rb,2.4201e-04_rb,2.1476e-04_rb,1.8503e-04_rb,1.5431e-04_rb,1.2132e-04_rb, &
     &8.7057e-05_rb,4.9691e-05_rb,7.6275e-06_rb/)
      kao(:, 5, 5, 3) = (/ &
     &2.9276e-04_rb,2.6878e-04_rb,2.3851e-04_rb,2.0652e-04_rb,1.7242e-04_rb,1.3630e-04_rb, &
     &9.8498e-05_rb,5.7853e-05_rb,1.1134e-05_rb/)
      kao(:, 1, 6, 3) = (/ &
     &3.5315e-04_rb,3.1458e-04_rb,2.7325e-04_rb,2.3096e-04_rb,1.8808e-04_rb,1.4453e-04_rb, &
     &9.9918e-05_rb,4.6331e-05_rb,1.2347e-06_rb/)
      kao(:, 2, 6, 3) = (/ &
     &4.1518e-04_rb,3.7021e-04_rb,3.2222e-04_rb,2.7318e-04_rb,2.2345e-04_rb,1.7257e-04_rb, &
     &1.2006e-04_rb,6.3507e-05_rb,2.6453e-06_rb/)
      kao(:, 3, 6, 3) = (/ &
     &4.8020e-04_rb,4.2942e-04_rb,3.7435e-04_rb,3.1847e-04_rb,2.6105e-04_rb,2.0180e-04_rb, &
     &1.4068e-04_rb,7.6253e-05_rb,4.2798e-06_rb/)
      kao(:, 4, 6, 3) = (/ &
     &5.4571e-04_rb,4.8797e-04_rb,4.2639e-04_rb,3.6432e-04_rb,2.9826e-04_rb,2.3028e-04_rb, &
     &1.6074e-04_rb,8.8155e-05_rb,6.6580e-06_rb/)
      kao(:, 5, 6, 3) = (/ &
     &6.1051e-04_rb,5.4685e-04_rb,4.8080e-04_rb,4.0919e-04_rb,3.3486e-04_rb,2.5913e-04_rb, &
     &1.8080e-04_rb,9.9384e-05_rb,9.9722e-06_rb/)
      kao(:, 1, 7, 3) = (/ &
     &7.5693e-04_rb,6.6624e-04_rb,5.7513e-04_rb,4.8344e-04_rb,3.9061e-04_rb,2.9600e-04_rb, &
     &2.0095e-04_rb,1.0427e-04_rb,1.0591e-06_rb/)
      kao(:, 2, 7, 3) = (/ &
     &8.9608e-04_rb,7.8973e-04_rb,6.8311e-04_rb,5.7440e-04_rb,4.6329e-04_rb,3.5236e-04_rb, &
     &2.4053e-04_rb,1.2653e-04_rb,2.0677e-06_rb/)
      kao(:, 3, 7, 3) = (/ &
     &1.0434e-03_rb,9.2160e-04_rb,7.9776e-04_rb,6.7036e-04_rb,5.4238e-04_rb,4.1393e-04_rb, &
     &2.8414e-04_rb,1.5011e-04_rb,3.4509e-06_rb/)
      kao(:, 4, 7, 3) = (/ &
     &1.1912e-03_rb,1.0536e-03_rb,9.1065e-04_rb,7.6644e-04_rb,6.2208e-04_rb,4.7601e-04_rb, &
     &3.2680e-04_rb,1.7285e-04_rb,5.5547e-06_rb/)
      kao(:, 5, 7, 3) = (/ &
     &1.3354e-03_rb,1.1809e-03_rb,1.0220e-03_rb,8.6241e-04_rb,7.0147e-04_rb,5.3735e-04_rb, &
     &3.6893e-04_rb,1.9541e-04_rb,8.3477e-06_rb/)
      kao(:, 1, 8, 3) = (/ &
     &2.0322e-03_rb,1.7810e-03_rb,1.5293e-03_rb,1.2777e-03_rb,1.0262e-03_rb,7.7431e-04_rb, &
     &5.2105e-04_rb,2.6538e-04_rb,9.6103e-07_rb/)
      kao(:, 2, 8, 3) = (/ &
     &2.4399e-03_rb,2.1393e-03_rb,1.8380e-03_rb,1.5373e-03_rb,1.2364e-03_rb,9.3375e-04_rb, &
     &6.2913e-04_rb,3.2110e-04_rb,1.7921e-06_rb/)
      kao(:, 3, 8, 3) = (/ &
     &2.8537e-03_rb,2.5030e-03_rb,2.1528e-03_rb,1.8027e-03_rb,1.4507e-03_rb,1.0966e-03_rb, &
     &7.3847e-04_rb,3.7883e-04_rb,3.0446e-06_rb/)
      kao(:, 4, 8, 3) = (/ &
     &3.2833e-03_rb,2.8808e-03_rb,2.4800e-03_rb,2.0779e-03_rb,1.6729e-03_rb,1.2622e-03_rb, &
     &8.5179e-04_rb,4.3974e-04_rb,4.8671e-06_rb/)
      kao(:, 5, 8, 3) = (/ &
     &3.6992e-03_rb,3.2483e-03_rb,2.7990e-03_rb,2.3454e-03_rb,1.8857e-03_rb,1.4259e-03_rb, &
     &9.6561e-04_rb,4.9917e-04_rb,7.5866e-06_rb/)
      kao(:, 1, 9, 3) = (/ &
     &1.0280e-02_rb,8.9965e-03_rb,7.7134e-03_rb,6.4300e-03_rb,5.1466e-03_rb,3.8628e-03_rb, &
     &2.5792e-03_rb,1.2964e-03_rb,9.2162e-07_rb/)
      kao(:, 2, 9, 3) = (/ &
     &1.2564e-02_rb,1.0997e-02_rb,9.4289e-03_rb,7.8609e-03_rb,6.2929e-03_rb,4.7248e-03_rb, &
     &3.1577e-03_rb,1.5902e-03_rb,1.8412e-06_rb/)
      kao(:, 3, 9, 3) = (/ &
     &1.4876e-02_rb,1.3021e-02_rb,1.1166e-02_rb,9.3099e-03_rb,7.4544e-03_rb,5.6000e-03_rb, &
     &3.7464e-03_rb,1.8890e-03_rb,3.6551e-06_rb/)
      kao(:, 4, 9, 3) = (/ &
     &1.7233e-02_rb,1.5085e-02_rb,1.2936e-02_rb,1.0788e-02_rb,8.6402e-03_rb,6.4946e-03_rb, &
     &4.3480e-03_rb,2.1937e-03_rb,6.4505e-06_rb/)
      kao(:, 5, 9, 3) = (/ &
     &1.9529e-02_rb,1.7097e-02_rb,1.4665e-02_rb,1.2232e-02_rb,9.8022e-03_rb,7.3739e-03_rb, &
     &4.9380e-03_rb,2.4934e-03_rb,1.0329e-05_rb/)
      kao(:, 1,10, 3) = (/ &
     &6.2934e-02_rb,5.5068e-02_rb,4.7203e-02_rb,3.9337e-02_rb,3.1472e-02_rb,2.3606e-02_rb, &
     &1.5740e-02_rb,7.8743e-03_rb,1.2141e-06_rb/)
      kao(:, 2,10, 3) = (/ &
     &7.7985e-02_rb,6.8239e-02_rb,5.8492e-02_rb,4.8746e-02_rb,3.8999e-02_rb,2.9252e-02_rb, &
     &1.9507e-02_rb,9.7597e-03_rb,2.5517e-06_rb/)
      kao(:, 3,10, 3) = (/ &
     &9.3312e-02_rb,8.1650e-02_rb,6.9991e-02_rb,5.8328e-02_rb,4.6667e-02_rb,3.5007e-02_rb, &
     &2.3345e-02_rb,1.1684e-02_rb,4.6909e-06_rb/)
      kao(:, 4,10, 3) = (/ &
     &1.0852e-01_rb,9.4962e-02_rb,8.1401e-02_rb,6.7840e-02_rb,5.4279e-02_rb,4.0718e-02_rb, &
     &2.7157e-02_rb,1.3598e-02_rb,7.7426e-06_rb/)
      kao(:, 5,10, 3) = (/ &
     &1.2387e-01_rb,1.0840e-01_rb,9.2920e-02_rb,7.7443e-02_rb,6.1965e-02_rb,4.6488e-02_rb, &
     &3.1009e-02_rb,1.5537e-02_rb,1.2033e-05_rb/)
      kao(:, 1,11, 3) = (/ &
     &1.4431e-01_rb,1.2627e-01_rb,1.0823e-01_rb,9.0195e-02_rb,7.2157e-02_rb,5.4120e-02_rb, &
     &3.6082e-02_rb,1.8045e-02_rb,1.2885e-06_rb/)
      kao(:, 2,11, 3) = (/ &
     &1.7767e-01_rb,1.5546e-01_rb,1.3326e-01_rb,1.1105e-01_rb,8.8842e-02_rb,6.6637e-02_rb, &
     &4.4429e-02_rb,2.2220e-02_rb,2.6746e-06_rb/)
      kao(:, 3,11, 3) = (/ &
     &2.1192e-01_rb,1.8543e-01_rb,1.5895e-01_rb,1.3246e-01_rb,1.0597e-01_rb,7.9483e-02_rb, &
     &5.2995e-02_rb,2.6507e-02_rb,4.8840e-06_rb/)
      kao(:, 4,11, 3) = (/ &
     &2.4571e-01_rb,2.1501e-01_rb,1.8430e-01_rb,1.5358e-01_rb,1.2287e-01_rb,9.2163e-02_rb, &
     &6.1452e-02_rb,3.0740e-02_rb,7.9723e-06_rb/)
      kao(:, 5,11, 3) = (/ &
     &2.7981e-01_rb,2.4483e-01_rb,2.0987e-01_rb,1.7490e-01_rb,1.3993e-01_rb,1.0496e-01_rb, &
     &6.9988e-02_rb,3.5019e-02_rb,1.2223e-05_rb/)
      kao(:, 1,12, 3) = (/ &
     &2.0829e-01_rb,1.8226e-01_rb,1.5622e-01_rb,1.3018e-01_rb,1.0415e-01_rb,7.8112e-02_rb, &
     &5.2077e-02_rb,2.6041e-02_rb,1.2025e-06_rb/)
      kao(:, 2,12, 3) = (/ &
     &2.5587e-01_rb,2.2388e-01_rb,1.9190e-01_rb,1.5992e-01_rb,1.2794e-01_rb,9.5954e-02_rb, &
     &6.3973e-02_rb,3.1992e-02_rb,2.4634e-06_rb/)
      kao(:, 3,12, 3) = (/ &
     &3.0401e-01_rb,2.6600e-01_rb,2.2801e-01_rb,1.9001e-01_rb,1.5201e-01_rb,1.1401e-01_rb, &
     &7.6014e-02_rb,3.8015e-02_rb,4.4929e-06_rb/)
      kao(:, 4,12, 3) = (/ &
     &3.5185e-01_rb,3.0787e-01_rb,2.6390e-01_rb,2.1992e-01_rb,1.7594e-01_rb,1.3196e-01_rb, &
     &8.7984e-02_rb,4.4005e-02_rb,7.2358e-06_rb/)
      kao(:, 5,12, 3) = (/ &
     &3.9955e-01_rb,3.4961e-01_rb,2.9967e-01_rb,2.4973e-01_rb,1.9980e-01_rb,1.4985e-01_rb, &
     &9.9920e-02_rb,4.9982e-02_rb,1.1171e-05_rb/)
      kao(:, 1,13, 3) = (/ &
     &2.4192e-01_rb,2.1167e-01_rb,1.8143e-01_rb,1.5120e-01_rb,1.2096e-01_rb,9.0718e-02_rb, &
     &6.0481e-02_rb,3.0243e-02_rb,1.0138e-06_rb/)
      kao(:, 2,13, 3) = (/ &
     &2.9636e-01_rb,2.5932e-01_rb,2.2227e-01_rb,1.8523e-01_rb,1.4819e-01_rb,1.1114e-01_rb, &
     &7.4096e-02_rb,3.7053e-02_rb,2.0810e-06_rb/)
      kao(:, 3,13, 3) = (/ &
     &3.5109e-01_rb,3.0721e-01_rb,2.6332e-01_rb,2.1944e-01_rb,1.7555e-01_rb,1.3167e-01_rb, &
     &8.7782e-02_rb,4.3898e-02_rb,3.6752e-06_rb/)
      kao(:, 4,13, 3) = (/ &
     &4.0523e-01_rb,3.5457e-01_rb,3.0393e-01_rb,2.5327e-01_rb,2.0263e-01_rb,1.5197e-01_rb, &
     &1.0132e-01_rb,5.0676e-02_rb,6.1123e-06_rb/)
      kao(:, 5,13, 3) = (/ &
     &4.5914e-01_rb,4.0175e-01_rb,3.4437e-01_rb,2.8698e-01_rb,2.2959e-01_rb,1.7221e-01_rb, &
     &1.1482e-01_rb,5.7432e-02_rb,9.7404e-06_rb/)
      kao(:, 1, 1, 4) = (/ &
     &2.1350e-05_rb,2.4552e-05_rb,2.3723e-05_rb,1.5777e-05_rb,2.7169e-05_rb,3.2921e-05_rb, &
     &3.9368e-05_rb,6.8629e-05_rb,2.6626e-05_rb/)
      kao(:, 2, 1, 4) = (/ &
     &2.2878e-05_rb,2.6833e-05_rb,2.6737e-05_rb,2.0377e-05_rb,2.2142e-05_rb,3.1966e-05_rb, &
     &4.1566e-05_rb,6.1320e-05_rb,2.3858e-05_rb/)
      kao(:, 3, 1, 4) = (/ &
     &2.3851e-05_rb,2.8963e-05_rb,2.9322e-05_rb,2.7907e-05_rb,1.9726e-05_rb,2.5598e-05_rb, &
     &4.2650e-05_rb,6.7498e-05_rb,2.1250e-05_rb/)
      kao(:, 4, 1, 4) = (/ &
     &2.4297e-05_rb,3.1053e-05_rb,3.2431e-05_rb,3.1771e-05_rb,2.7799e-05_rb,2.0892e-05_rb, &
     &3.7144e-05_rb,7.1944e-05_rb,2.3747e-05_rb/)
      kao(:, 5, 1, 4) = (/ &
     &2.4515e-05_rb,3.3133e-05_rb,3.5554e-05_rb,3.5376e-05_rb,3.4171e-05_rb,2.9240e-05_rb, &
     &2.7955e-05_rb,6.5059e-05_rb,3.8779e-05_rb/)
      kao(:, 1, 2, 4) = (/ &
     &3.5233e-05_rb,3.6967e-05_rb,3.5192e-05_rb,3.0346e-05_rb,1.9517e-05_rb,3.6227e-05_rb, &
     &3.6486e-05_rb,4.4755e-05_rb,1.7210e-05_rb/)
      kao(:, 2, 2, 4) = (/ &
     &3.7659e-05_rb,4.0104e-05_rb,3.8633e-05_rb,3.5461e-05_rb,2.4007e-05_rb,2.9205e-05_rb, &
     &3.6839e-05_rb,4.4274e-05_rb,1.6582e-05_rb/)
      kao(:, 3, 2, 4) = (/ &
     &3.9567e-05_rb,4.2994e-05_rb,4.2107e-05_rb,3.9324e-05_rb,3.3808e-05_rb,2.0621e-05_rb, &
     &3.2387e-05_rb,5.0199e-05_rb,1.2696e-05_rb/)
      kao(:, 4, 2, 4) = (/ &
     &4.0697e-05_rb,4.5929e-05_rb,4.5740e-05_rb,4.2992e-05_rb,3.9709e-05_rb,2.8967e-05_rb, &
     &2.4180e-05_rb,4.8120e-05_rb,2.0582e-05_rb/)
      kao(:, 5, 2, 4) = (/ &
     &4.1514e-05_rb,4.8565e-05_rb,4.9023e-05_rb,4.7662e-05_rb,4.4382e-05_rb,3.8389e-05_rb, &
     &2.5686e-05_rb,4.1842e-05_rb,3.1906e-05_rb/)
      kao(:, 1, 3, 4) = (/ &
     &8.1562e-05_rb,7.8380e-05_rb,7.1455e-05_rb,6.3207e-05_rb,5.2782e-05_rb,3.3109e-05_rb, &
     &4.1621e-05_rb,4.2434e-05_rb,8.8242e-06_rb/)
      kao(:, 2, 3, 4) = (/ &
     &8.6593e-05_rb,8.4470e-05_rb,7.7554e-05_rb,6.8896e-05_rb,5.8997e-05_rb,4.4824e-05_rb, &
     &3.0840e-05_rb,4.1510e-05_rb,8.5693e-06_rb/)
      kao(:, 3, 3, 4) = (/ &
     &9.1353e-05_rb,8.9996e-05_rb,8.3114e-05_rb,7.4541e-05_rb,6.4935e-05_rb,5.3159e-05_rb, &
     &2.8537e-05_rb,3.9838e-05_rb,1.1226e-05_rb/)
      kao(:, 4, 3, 4) = (/ &
     &9.5365e-05_rb,9.5648e-05_rb,8.9016e-05_rb,8.0719e-05_rb,7.0818e-05_rb,5.8632e-05_rb, &
     &4.0977e-05_rb,3.3245e-05_rb,1.8214e-05_rb/)
      kao(:, 5, 3, 4) = (/ &
     &9.7542e-05_rb,1.0001e-04_rb,9.4766e-05_rb,8.7247e-05_rb,7.6489e-05_rb,6.4569e-05_rb, &
     &5.1322e-05_rb,2.4346e-05_rb,2.6190e-05_rb/)
      kao(:, 1, 4, 4) = (/ &
     &1.9526e-04_rb,1.7866e-04_rb,1.5808e-04_rb,1.3593e-04_rb,1.1271e-04_rb,8.7918e-05_rb, &
     &5.1266e-05_rb,5.4774e-05_rb,4.6025e-06_rb/)
      kao(:, 2, 4, 4) = (/ &
     &2.0650e-04_rb,1.9062e-04_rb,1.6960e-04_rb,1.4698e-04_rb,1.2286e-04_rb,9.7273e-05_rb, &
     &6.7543e-05_rb,4.4150e-05_rb,5.6277e-06_rb/)
      kao(:, 3, 4, 4) = (/ &
     &2.1739e-04_rb,2.0276e-04_rb,1.8135e-04_rb,1.5734e-04_rb,1.3233e-04_rb,1.0548e-04_rb, &
     &7.6274e-05_rb,3.0228e-05_rb,9.8992e-06_rb/)
      kao(:, 4, 4, 4) = (/ &
     &2.2749e-04_rb,2.1410e-04_rb,1.9235e-04_rb,1.6801e-04_rb,1.4178e-04_rb,1.1383e-04_rb, &
     &8.3889e-05_rb,3.8831e-05_rb,1.4832e-05_rb/)
      kao(:, 5, 4, 4) = (/ &
     &2.3672e-04_rb,2.2535e-04_rb,2.0298e-04_rb,1.7817e-04_rb,1.5157e-04_rb,1.2285e-04_rb, &
     &9.1248e-05_rb,5.4055e-05_rb,2.0953e-05_rb/)
      kao(:, 1, 5, 4) = (/ &
     &4.3901e-04_rb,3.9096e-04_rb,3.4083e-04_rb,2.8855e-04_rb,2.3566e-04_rb,1.8135e-04_rb, &
     &1.2520e-04_rb,3.9791e-05_rb,2.8698e-06_rb/)
      kao(:, 2, 5, 4) = (/ &
     &4.6450e-04_rb,4.1602e-04_rb,3.6285e-04_rb,3.0879e-04_rb,2.5330e-04_rb,1.9642e-04_rb, &
     &1.3685e-04_rb,6.9096e-05_rb,4.7432e-06_rb/)
      kao(:, 3, 5, 4) = (/ &
     &4.8882e-04_rb,4.4052e-04_rb,3.8581e-04_rb,3.3048e-04_rb,2.7253e-04_rb,2.1160e-04_rb, &
     &1.4899e-04_rb,8.1167e-05_rb,8.1879e-06_rb/)
      kao(:, 4, 5, 4) = (/ &
     &5.1005e-04_rb,4.6303e-04_rb,4.0950e-04_rb,3.5246e-04_rb,2.9060e-04_rb,2.2675e-04_rb, &
     &1.6042e-04_rb,8.9994e-05_rb,1.2389e-05_rb/)
      kao(:, 5, 5, 4) = (/ &
     &5.2999e-04_rb,4.8642e-04_rb,4.3340e-04_rb,3.7157e-04_rb,3.0721e-04_rb,2.4050e-04_rb, &
     &1.7156e-04_rb,9.8341e-05_rb,1.7831e-05_rb/)
      kao(:, 1, 6, 4) = (/ &
     &9.3945e-04_rb,8.2623e-04_rb,7.1331e-04_rb,5.9949e-04_rb,4.8483e-04_rb,3.6836e-04_rb, &
     &2.5086e-04_rb,1.2938e-04_rb,2.1494e-06_rb/)
      kao(:, 2, 6, 4) = (/ &
     &1.0016e-03_rb,8.8499e-04_rb,7.6750e-04_rb,6.4657e-04_rb,5.2304e-04_rb,3.9902e-04_rb, &
     &2.7391e-04_rb,1.4413e-04_rb,3.8318e-06_rb/)
      kao(:, 3, 6, 4) = (/ &
     &1.0547e-03_rb,9.3464e-04_rb,8.1142e-04_rb,6.8349e-04_rb,5.5479e-04_rb,4.2609e-04_rb, &
     &2.9394e-04_rb,1.5646e-04_rb,6.4475e-06_rb/)
      kao(:, 4, 6, 4) = (/ &
     &1.1032e-03_rb,9.8345e-04_rb,8.5527e-04_rb,7.2211e-04_rb,5.9000e-04_rb,4.5529e-04_rb, &
     &3.1446e-04_rb,1.6904e-04_rb,9.9612e-06_rb/)
      kao(:, 5, 6, 4) = (/ &
     &1.1467e-03_rb,1.0269e-03_rb,8.9466e-04_rb,7.6180e-04_rb,6.2654e-04_rb,4.8317e-04_rb, &
     &3.3526e-04_rb,1.8155e-04_rb,1.4635e-05_rb/)
      kao(:, 1, 7, 4) = (/ &
     &2.0968e-03_rb,1.8382e-03_rb,1.5799e-03_rb,1.3200e-03_rb,1.0602e-03_rb,8.0055e-04_rb, &
     &5.3931e-04_rb,2.7613e-04_rb,1.7495e-06_rb/)
      kao(:, 2, 7, 4) = (/ &
     &2.2692e-03_rb,1.9926e-03_rb,1.7142e-03_rb,1.4356e-03_rb,1.1561e-03_rb,8.7560e-04_rb, &
     &5.9192e-04_rb,3.0494e-04_rb,3.2823e-06_rb/)
      kao(:, 3, 7, 4) = (/ &
     &2.3957e-03_rb,2.1065e-03_rb,1.8155e-03_rb,1.5242e-03_rb,1.2302e-03_rb,9.3213e-04_rb, &
     &6.3098e-04_rb,3.2845e-04_rb,5.4452e-06_rb/)
      kao(:, 4, 7, 4) = (/ &
     &2.5154e-03_rb,2.2153e-03_rb,1.9155e-03_rb,1.6120e-03_rb,1.3009e-03_rb,9.8717e-04_rb, &
     &6.7313e-04_rb,3.5215e-04_rb,8.4591e-06_rb/)
      kao(:, 5, 7, 4) = (/ &
     &2.6221e-03_rb,2.3149e-03_rb,2.0078e-03_rb,1.6896e-03_rb,1.3663e-03_rb,1.0418e-03_rb, &
     &7.1380e-04_rb,3.7558e-04_rb,1.3208e-05_rb/)
      kao(:, 1, 8, 4) = (/ &
     &5.9687e-03_rb,5.2249e-03_rb,4.4818e-03_rb,3.7390e-03_rb,2.9959e-03_rb,2.2513e-03_rb, &
     &1.5068e-03_rb,7.6107e-04_rb,1.5072e-06_rb/)
      kao(:, 2, 8, 4) = (/ &
     &6.4060e-03_rb,5.6103e-03_rb,4.8155e-03_rb,4.0184e-03_rb,3.2203e-03_rb,2.4232e-03_rb, &
     &1.6246e-03_rb,8.2361e-04_rb,2.9177e-06_rb/)
      kao(:, 3, 8, 4) = (/ &
     &6.8363e-03_rb,5.9919e-03_rb,5.1443e-03_rb,4.2950e-03_rb,3.4467e-03_rb,2.5980e-03_rb, &
     &1.7471e-03_rb,8.8934e-04_rb,5.0920e-06_rb/)
      kao(:, 4, 8, 4) = (/ &
     &7.1796e-03_rb,6.2954e-03_rb,5.4063e-03_rb,4.5177e-03_rb,3.6307e-03_rb,2.7430e-03_rb, &
     &1.8488e-03_rb,9.4284e-04_rb,8.3290e-06_rb/)
      kao(:, 5, 8, 4) = (/ &
     &7.5276e-03_rb,6.6017e-03_rb,5.6732e-03_rb,4.7473e-03_rb,3.8229e-03_rb,2.8925e-03_rb, &
     &1.9515e-03_rb,1.0013e-03_rb,1.2816e-05_rb/)
      kao(:, 1, 9, 4) = (/ &
     &3.2487e-02_rb,2.8428e-02_rb,2.4369e-02_rb,2.0309e-02_rb,1.6251e-02_rb,1.2192e-02_rb, &
     &8.1342e-03_rb,4.0755e-03_rb,1.9489e-06_rb/)
      kao(:, 2, 9, 4) = (/ &
     &3.4271e-02_rb,2.9992e-02_rb,2.5712e-02_rb,2.1432e-02_rb,1.7152e-02_rb,1.2873e-02_rb, &
     &8.5913e-03_rb,4.3059e-03_rb,3.6526e-06_rb/)
      kao(:, 3, 9, 4) = (/ &
     &3.6903e-02_rb,3.2297e-02_rb,2.7692e-02_rb,2.3087e-02_rb,1.8480e-02_rb,1.3870e-02_rb, &
     &9.2583e-03_rb,4.6485e-03_rb,5.8762e-06_rb/)
      kao(:, 4, 9, 4) = (/ &
     &3.8979e-02_rb,3.4119e-02_rb,2.9257e-02_rb,2.4392e-02_rb,1.9526e-02_rb,1.4659e-02_rb, &
     &9.7912e-03_rb,4.9264e-03_rb,9.0883e-06_rb/)
      kao(:, 5, 9, 4) = (/ &
     &4.0909e-02_rb,3.5808e-02_rb,3.0705e-02_rb,2.5602e-02_rb,2.0497e-02_rb,1.5392e-02_rb, &
     &1.0293e-02_rb,5.1880e-03_rb,1.4132e-05_rb/)
      kao(:, 1,10, 4) = (/ &
     &2.1003e-01_rb,1.8378e-01_rb,1.5753e-01_rb,1.3127e-01_rb,1.0502e-01_rb,7.8770e-02_rb, &
     &5.2516e-02_rb,2.6264e-02_rb,2.2708e-06_rb/)
      kao(:, 2,10, 4) = (/ &
     &2.2303e-01_rb,1.9515e-01_rb,1.6728e-01_rb,1.3940e-01_rb,1.1153e-01_rb,8.3651e-02_rb, &
     &5.5777e-02_rb,2.7901e-02_rb,4.3388e-06_rb/)
      kao(:, 3,10, 4) = (/ &
     &2.3715e-01_rb,2.0752e-01_rb,1.7788e-01_rb,1.4824e-01_rb,1.1861e-01_rb,8.8964e-02_rb, &
     &5.9326e-02_rb,2.9681e-02_rb,8.0389e-06_rb/)
      kao(:, 4,10, 4) = (/ &
     &2.5270e-01_rb,2.2113e-01_rb,1.8955e-01_rb,1.5796e-01_rb,1.2638e-01_rb,9.4802e-02_rb, &
     &6.3220e-02_rb,3.1633e-02_rb,1.4122e-05_rb/)
      kao(:, 5,10, 4) = (/ &
     &2.6544e-01_rb,2.3227e-01_rb,1.9911e-01_rb,1.6593e-01_rb,1.3276e-01_rb,9.9586e-02_rb, &
     &6.6411e-02_rb,3.3237e-02_rb,2.0958e-05_rb/)
      kao(:, 1,11, 4) = (/ &
     &4.6663e-01_rb,4.0831e-01_rb,3.4998e-01_rb,2.9165e-01_rb,2.3332e-01_rb,1.7500e-01_rb, &
     &1.1667e-01_rb,5.8341e-02_rb,2.3420e-06_rb/)
      kao(:, 2,11, 4) = (/ &
     &4.9719e-01_rb,4.3506e-01_rb,3.7290e-01_rb,3.1076e-01_rb,2.4862e-01_rb,1.8647e-01_rb, &
     &1.2432e-01_rb,6.2174e-02_rb,4.5807e-06_rb/)
      kao(:, 3,11, 4) = (/ &
     &5.2774e-01_rb,4.6176e-01_rb,3.9581e-01_rb,3.2984e-01_rb,2.6389e-01_rb,1.9793e-01_rb, &
     &1.3196e-01_rb,6.5997e-02_rb,8.8811e-06_rb/)
      kao(:, 4,11, 4) = (/ &
     &5.6221e-01_rb,4.9195e-01_rb,4.2167e-01_rb,3.5141e-01_rb,2.8114e-01_rb,2.1086e-01_rb, &
     &1.4059e-01_rb,7.0317e-02_rb,1.5538e-05_rb/)
      kao(:, 5,11, 4) = (/ &
     &5.9121e-01_rb,5.1732e-01_rb,4.4343e-01_rb,3.6953e-01_rb,2.9564e-01_rb,2.2174e-01_rb, &
     &1.4785e-01_rb,7.3949e-02_rb,2.4433e-05_rb/)
      kao(:, 1,12, 4) = (/ &
     &6.5249e-01_rb,5.7093e-01_rb,4.8937e-01_rb,4.0781e-01_rb,3.2625e-01_rb,2.4469e-01_rb, &
     &1.6313e-01_rb,8.1572e-02_rb,2.2330e-06_rb/)
      kao(:, 2,12, 4) = (/ &
     &6.9534e-01_rb,6.0842e-01_rb,5.2151e-01_rb,4.3459e-01_rb,3.4768e-01_rb,2.6076e-01_rb, &
     &1.7385e-01_rb,8.6935e-02_rb,4.3967e-06_rb/)
      kao(:, 3,12, 4) = (/ &
     &7.4143e-01_rb,6.4875e-01_rb,5.5607e-01_rb,4.6339e-01_rb,3.7072e-01_rb,2.7805e-01_rb, &
     &1.8537e-01_rb,9.2701e-02_rb,8.3925e-06_rb/)
      kao(:, 4,12, 4) = (/ &
     &7.8904e-01_rb,6.9042e-01_rb,5.9179e-01_rb,4.9316e-01_rb,3.9453e-01_rb,2.9591e-01_rb, &
     &1.9728e-01_rb,9.8661e-02_rb,1.5232e-05_rb/)
      kao(:, 5,12, 4) = (/ &
     &8.3131e-01_rb,7.2741e-01_rb,6.2350e-01_rb,5.1959e-01_rb,4.1567e-01_rb,3.1177e-01_rb, &
     &2.0786e-01_rb,1.0395e-01_rb,2.3060e-05_rb/)
      kao(:, 1,13, 4) = (/ &
     &7.3322e-01_rb,6.4157e-01_rb,5.4992e-01_rb,4.5827e-01_rb,3.6662e-01_rb,2.7497e-01_rb, &
     &1.8332e-01_rb,9.1663e-02_rb,1.8044e-06_rb/)
      kao(:, 2,13, 4) = (/ &
     &7.8224e-01_rb,6.8446e-01_rb,5.8668e-01_rb,4.8890e-01_rb,3.9113e-01_rb,2.9335e-01_rb, &
     &1.9557e-01_rb,9.7794e-02_rb,3.6622e-06_rb/)
      kao(:, 3,13, 4) = (/ &
     &8.3769e-01_rb,7.3298e-01_rb,6.2826e-01_rb,5.2356e-01_rb,4.1885e-01_rb,3.1414e-01_rb, &
     &2.0944e-01_rb,1.0473e-01_rb,6.8549e-06_rb/)
      kao(:, 4,13, 4) = (/ &
     &8.9145e-01_rb,7.8000e-01_rb,6.6858e-01_rb,5.5715e-01_rb,4.4572e-01_rb,3.3430e-01_rb, &
     &2.2288e-01_rb,1.1145e-01_rb,1.1699e-05_rb/)
      kao(:, 5,13, 4) = (/ &
     &9.4256e-01_rb,8.2475e-01_rb,7.0694e-01_rb,5.8912e-01_rb,4.7129e-01_rb,3.5348e-01_rb, &
     &2.3566e-01_rb,1.1785e-01_rb,1.9250e-05_rb/)
      kao(:, 1, 1, 5) = (/ &
     &3.1890e-05_rb,3.3428e-05_rb,3.1339e-05_rb,1.9062e-05_rb,2.1359e-05_rb,2.6190e-05_rb, &
     &4.2364e-05_rb,7.2281e-05_rb,2.8801e-05_rb/)
      kao(:, 2, 1, 5) = (/ &
     &3.2241e-05_rb,3.5034e-05_rb,3.4311e-05_rb,2.9337e-05_rb,1.6956e-05_rb,2.7153e-05_rb, &
     &4.1595e-05_rb,7.5807e-05_rb,2.1642e-05_rb/)
      kao(:, 3, 1, 5) = (/ &
     &3.2685e-05_rb,3.7104e-05_rb,3.7239e-05_rb,3.5440e-05_rb,2.4683e-05_rb,2.0584e-05_rb, &
     &3.9467e-05_rb,7.0498e-05_rb,2.5433e-05_rb/)
      kao(:, 4, 1, 5) = (/ &
     &3.3108e-05_rb,3.9233e-05_rb,3.9843e-05_rb,4.0632e-05_rb,3.9351e-05_rb,2.7492e-05_rb, &
     &2.7372e-05_rb,6.6136e-05_rb,4.2940e-05_rb/)
      kao(:, 5, 1, 5) = (/ &
     &3.3102e-05_rb,4.1126e-05_rb,4.3706e-05_rb,4.7600e-05_rb,5.0514e-05_rb,4.9463e-05_rb, &
     &3.6482e-05_rb,4.9147e-05_rb,6.6970e-05_rb/)
      kao(:, 1, 2, 5) = (/ &
     &5.3936e-05_rb,5.3705e-05_rb,4.9534e-05_rb,4.3483e-05_rb,2.4468e-05_rb,2.7314e-05_rb, &
     &3.0874e-05_rb,5.5614e-05_rb,1.6658e-05_rb/)
      kao(:, 2, 2, 5) = (/ &
     &5.4860e-05_rb,5.5845e-05_rb,5.2280e-05_rb,4.7572e-05_rb,3.6459e-05_rb,1.9946e-05_rb, &
     &3.1643e-05_rb,4.8991e-05_rb,1.3490e-05_rb/)
      kao(:, 3, 2, 5) = (/ &
     &5.5709e-05_rb,5.8014e-05_rb,5.5306e-05_rb,5.1404e-05_rb,4.5520e-05_rb,2.7920e-05_rb, &
     &2.4697e-05_rb,4.8298e-05_rb,2.2442e-05_rb/)
      kao(:, 4, 2, 5) = (/ &
     &5.6594e-05_rb,6.0421e-05_rb,5.8710e-05_rb,5.5553e-05_rb,5.0533e-05_rb,4.1849e-05_rb, &
     &2.3926e-05_rb,4.4332e-05_rb,3.8077e-05_rb/)
      kao(:, 5, 2, 5) = (/ &
     &5.6989e-05_rb,6.3301e-05_rb,6.2702e-05_rb,5.9556e-05_rb,5.7075e-05_rb,5.4328e-05_rb, &
     &4.2106e-05_rb,3.0435e-05_rb,5.5304e-05_rb/)
      kao(:, 1, 3, 5) = (/ &
     &1.3054e-04_rb,1.2243e-04_rb,1.0960e-04_rb,9.5396e-05_rb,7.9466e-05_rb,5.1293e-05_rb, &
     &3.9755e-05_rb,3.4475e-05_rb,8.0698e-06_rb/)
      kao(:, 2, 3, 5) = (/ &
     &1.3254e-04_rb,1.2631e-04_rb,1.1422e-04_rb,1.0017e-04_rb,8.4724e-05_rb,6.6970e-05_rb, &
     &3.1985e-05_rb,3.1748e-05_rb,1.1668e-05_rb/)
      kao(:, 3, 3, 5) = (/ &
     &1.3451e-04_rb,1.3069e-04_rb,1.1849e-04_rb,1.0449e-04_rb,8.9441e-05_rb,7.3066e-05_rb, &
     &4.0433e-05_rb,3.2069e-05_rb,2.1119e-05_rb/)
      kao(:, 4, 3, 5) = (/ &
     &1.3620e-04_rb,1.3418e-04_rb,1.2287e-04_rb,1.0958e-04_rb,9.5020e-05_rb,7.9300e-05_rb, &
     &5.8519e-05_rb,2.5240e-05_rb,3.2095e-05_rb/)
      kao(:, 5, 3, 5) = (/ &
     &1.3805e-04_rb,1.3888e-04_rb,1.2760e-04_rb,1.1481e-04_rb,1.0157e-04_rb,8.6256e-05_rb, &
     &6.7417e-05_rb,3.0660e-05_rb,4.5147e-05_rb/)
      kao(:, 1, 4, 5) = (/ &
     &3.2379e-04_rb,2.9036e-04_rb,2.5461e-04_rb,2.1829e-04_rb,1.8055e-04_rb,1.4064e-04_rb, &
     &9.6199e-05_rb,5.2600e-05_rb,5.3338e-06_rb/)
      kao(:, 2, 4, 5) = (/ &
     &3.2979e-04_rb,2.9999e-04_rb,2.6566e-04_rb,2.2924e-04_rb,1.9018e-04_rb,1.4831e-04_rb, &
     &1.0429e-04_rb,3.7208e-05_rb,1.1382e-05_rb/)
      kao(:, 3, 4, 5) = (/ &
     &3.3647e-04_rb,3.0773e-04_rb,2.7537e-04_rb,2.3841e-04_rb,1.9818e-04_rb,1.5534e-04_rb, &
     &1.1107e-04_rb,4.2940e-05_rb,1.8774e-05_rb/)
      kao(:, 4, 4, 5) = (/ &
     &3.4222e-04_rb,3.1790e-04_rb,2.8515e-04_rb,2.4672e-04_rb,2.0544e-04_rb,1.6267e-04_rb, &
     &1.1806e-04_rb,6.5823e-05_rb,2.7569e-05_rb/)
      kao(:, 5, 4, 5) = (/ &
     &3.4598e-04_rb,3.2526e-04_rb,2.9297e-04_rb,2.5514e-04_rb,2.1384e-04_rb,1.7093e-04_rb, &
     &1.2620e-04_rb,7.6156e-05_rb,3.8829e-05_rb/)
      kao(:, 1, 5, 5) = (/ &
     &7.4853e-04_rb,6.5989e-04_rb,5.7237e-04_rb,4.8339e-04_rb,3.9258e-04_rb,3.0112e-04_rb, &
     &2.0775e-04_rb,1.0511e-04_rb,4.9046e-06_rb/)
      kao(:, 2, 5, 5) = (/ &
     &7.7296e-04_rb,6.8526e-04_rb,5.9860e-04_rb,5.0629e-04_rb,4.1334e-04_rb,3.1913e-04_rb, &
     &2.2288e-04_rb,1.1662e-04_rb,9.7426e-06_rb/)
      kao(:, 3, 5, 5) = (/ &
     &7.8512e-04_rb,7.0191e-04_rb,6.1328e-04_rb,5.2156e-04_rb,4.2917e-04_rb,3.3432e-04_rb, &
     &2.3232e-04_rb,1.2535e-04_rb,1.4908e-05_rb/)
      kao(:, 4, 5, 5) = (/ &
     &8.0065e-04_rb,7.1779e-04_rb,6.2814e-04_rb,5.3808e-04_rb,4.4471e-04_rb,3.4601e-04_rb, &
     &2.4241e-04_rb,1.3326e-04_rb,2.1852e-05_rb/)
      kao(:, 5, 5, 5) = (/ &
     &8.1368e-04_rb,7.3211e-04_rb,6.4458e-04_rb,5.5662e-04_rb,4.6149e-04_rb,3.6022e-04_rb, &
     &2.5323e-04_rb,1.4142e-04_rb,3.1532e-05_rb/)
      kao(:, 1, 6, 5) = (/ &
     &1.6528e-03_rb,1.4497e-03_rb,1.2477e-03_rb,1.0459e-03_rb,8.4296e-04_rb,6.4013e-04_rb, &
     &4.3541e-04_rb,2.2882e-04_rb,4.1025e-06_rb/)
      kao(:, 2, 6, 5) = (/ &
     &1.7167e-03_rb,1.5070e-03_rb,1.2980e-03_rb,1.0901e-03_rb,8.8256e-04_rb,6.7196e-04_rb, &
     &4.5785e-04_rb,2.4158e-04_rb,7.1575e-06_rb/)
      kao(:, 3, 6, 5) = (/ &
     &1.7542e-03_rb,1.5443e-03_rb,1.3374e-03_rb,1.1305e-03_rb,9.1589e-04_rb,6.9839e-04_rb, &
     &4.8070e-04_rb,2.5576e-04_rb,1.1170e-05_rb/)
      kao(:, 4, 6, 5) = (/ &
     &1.7835e-03_rb,1.5745e-03_rb,1.3686e-03_rb,1.1552e-03_rb,9.3898e-04_rb,7.2100e-04_rb, &
     &4.9974e-04_rb,2.6756e-04_rb,1.6778e-05_rb/)
      kao(:, 5, 6, 5) = (/ &
     &1.8173e-03_rb,1.6109e-03_rb,1.3997e-03_rb,1.1836e-03_rb,9.6533e-04_rb,7.4477e-04_rb, &
     &5.1869e-04_rb,2.7889e-04_rb,2.4339e-05_rb/)
      kao(:, 1, 7, 5) = (/ &
     &3.9726e-03_rb,3.4772e-03_rb,2.9832e-03_rb,2.4912e-03_rb,1.9978e-03_rb,1.5048e-03_rb, &
     &1.0114e-03_rb,5.1606e-04_rb,3.0815e-06_rb/)
      kao(:, 2, 7, 5) = (/ &
     &3.9885e-03_rb,3.4924e-03_rb,2.9992e-03_rb,2.5047e-03_rb,2.0117e-03_rb,1.5168e-03_rb, &
     &1.0225e-03_rb,5.2471e-04_rb,5.4224e-06_rb/)
      kao(:, 3, 7, 5) = (/ &
     &4.1133e-03_rb,3.6053e-03_rb,3.0965e-03_rb,2.5896e-03_rb,2.0823e-03_rb,1.5758e-03_rb, &
     &1.0680e-03_rb,5.5159e-04_rb,9.0931e-06_rb/)
      kao(:, 4, 7, 5) = (/ &
     &4.1858e-03_rb,3.6723e-03_rb,3.1582e-03_rb,2.6449e-03_rb,2.1343e-03_rb,1.6217e-03_rb, &
     &1.1005e-03_rb,5.7513e-04_rb,1.3973e-05_rb/)
      kao(:, 5, 7, 5) = (/ &
     &4.2673e-03_rb,3.7471e-03_rb,3.2256e-03_rb,2.7082e-03_rb,2.1908e-03_rb,1.6652e-03_rb, &
     &1.1349e-03_rb,5.9686e-04_rb,1.9907e-05_rb/)
      kao(:, 1, 8, 5) = (/ &
     &1.1789e-02_rb,1.0316e-02_rb,8.8432e-03_rb,7.3701e-03_rb,5.8982e-03_rb,4.4284e-03_rb, &
     &2.9575e-03_rb,1.4868e-03_rb,2.6362e-06_rb/)
      kao(:, 2, 8, 5) = (/ &
     &1.1815e-02_rb,1.0339e-02_rb,8.8633e-03_rb,7.3911e-03_rb,5.9198e-03_rb,4.4454e-03_rb, &
     &2.9722e-03_rb,1.4993e-03_rb,4.7391e-06_rb/)
      kao(:, 3, 8, 5) = (/ &
     &1.2063e-02_rb,1.0556e-02_rb,9.0547e-03_rb,7.5533e-03_rb,6.0485e-03_rb,4.5441e-03_rb, &
     &3.0419e-03_rb,1.5395e-03_rb,7.9686e-06_rb/)
      kao(:, 4, 8, 5) = (/ &
     &1.2374e-02_rb,1.0835e-02_rb,9.2985e-03_rb,7.7571e-03_rb,6.2150e-03_rb,4.6751e-03_rb, &
     &3.1349e-03_rb,1.5981e-03_rb,1.2907e-05_rb/)
      kao(:, 5, 8, 5) = (/ &
     &1.2586e-02_rb,1.1028e-02_rb,9.4648e-03_rb,7.8998e-03_rb,6.3362e-03_rb,4.7714e-03_rb, &
     &3.2105e-03_rb,1.6437e-03_rb,1.9633e-05_rb/)
      kao(:, 1, 9, 5) = (/ &
     &6.5443e-02_rb,5.7264e-02_rb,4.9084e-02_rb,4.0904e-02_rb,3.2723e-02_rb,2.4543e-02_rb, &
     &1.6364e-02_rb,8.1848e-03_rb,3.1432e-06_rb/)
      kao(:, 2, 9, 5) = (/ &
     &6.6880e-02_rb,5.8521e-02_rb,5.0162e-02_rb,4.1802e-02_rb,3.3443e-02_rb,2.5083e-02_rb, &
     &1.6728e-02_rb,8.3763e-03_rb,5.7297e-06_rb/)
      kao(:, 3, 9, 5) = (/ &
     &6.6965e-02_rb,5.8596e-02_rb,5.0227e-02_rb,4.1857e-02_rb,3.3491e-02_rb,2.5128e-02_rb, &
     &1.6767e-02_rb,8.3954e-03_rb,1.0132e-05_rb/)
      kao(:, 4, 9, 5) = (/ &
     &6.9095e-02_rb,6.0459e-02_rb,5.1827e-02_rb,4.3201e-02_rb,3.4575e-02_rb,2.5948e-02_rb, &
     &1.7311e-02_rb,8.6749e-03_rb,1.6592e-05_rb/)
      kao(:, 5, 9, 5) = (/ &
     &7.0723e-02_rb,6.1894e-02_rb,5.3070e-02_rb,4.4247e-02_rb,3.5417e-02_rb,2.6580e-02_rb, &
     &1.7739e-02_rb,8.9030e-03_rb,2.5109e-05_rb/)
      kao(:, 1,10, 5) = (/ &
     &4.3117e-01_rb,3.7727e-01_rb,3.2338e-01_rb,2.6948e-01_rb,2.1559e-01_rb,1.6169e-01_rb, &
     &1.0779e-01_rb,5.3898e-02_rb,3.8321e-06_rb/)
      kao(:, 2,10, 5) = (/ &
     &4.4275e-01_rb,3.8740e-01_rb,3.3206e-01_rb,2.7672e-01_rb,2.2138e-01_rb,1.6603e-01_rb, &
     &1.1069e-01_rb,5.5347e-02_rb,7.9189e-06_rb/)
      kao(:, 3,10, 5) = (/ &
     &4.5038e-01_rb,3.9409e-01_rb,3.3779e-01_rb,2.8149e-01_rb,2.2519e-01_rb,1.6890e-01_rb, &
     &1.1260e-01_rb,5.6313e-02_rb,1.3890e-05_rb/)
      kao(:, 4,10, 5) = (/ &
     &4.5731e-01_rb,4.0015e-01_rb,3.4299e-01_rb,2.8583e-01_rb,2.2868e-01_rb,1.7152e-01_rb, &
     &1.1436e-01_rb,5.7212e-02_rb,2.2042e-05_rb/)
      kao(:, 5,10, 5) = (/ &
     &4.7192e-01_rb,4.1294e-01_rb,3.5396e-01_rb,2.9499e-01_rb,2.3600e-01_rb,1.7703e-01_rb, &
     &1.1805e-01_rb,5.9063e-02_rb,3.5819e-05_rb/)
      kao(:, 1,11, 5) = (/ &
     &9.6082e-01_rb,8.4073e-01_rb,7.2062e-01_rb,6.0052e-01_rb,4.8042e-01_rb,3.6032e-01_rb, &
     &2.4021e-01_rb,1.2011e-01_rb,4.1252e-06_rb/)
      kao(:, 2,11, 5) = (/ &
     &9.8883e-01_rb,8.6524e-01_rb,7.4164e-01_rb,6.1802e-01_rb,4.9442e-01_rb,3.7082e-01_rb, &
     &2.4722e-01_rb,1.2361e-01_rb,8.2890e-06_rb/)
      kao(:, 3,11, 5) = (/ &
     &1.0099e+00_rb,8.8371e-01_rb,7.5745e-01_rb,6.3122e-01_rb,5.0498e-01_rb,3.7874e-01_rb, &
     &2.5251e-01_rb,1.2627e-01_rb,1.3752e-05_rb/)
      kao(:, 4,11, 5) = (/ &
     &1.0275e+00_rb,8.9904e-01_rb,7.7063e-01_rb,6.4220e-01_rb,5.1377e-01_rb,3.8534e-01_rb, &
     &2.5691e-01_rb,1.2849e-01_rb,2.1878e-05_rb/)
      kao(:, 5,11, 5) = (/ &
     &1.0619e+00_rb,9.2921e-01_rb,7.9651e-01_rb,6.6374e-01_rb,5.3103e-01_rb,3.9829e-01_rb, &
     &2.6556e-01_rb,1.3282e-01_rb,3.3798e-05_rb/)
      kao(:, 1,12, 5) = (/ &
     &1.3448e+00_rb,1.1767e+00_rb,1.0086e+00_rb,8.4049e-01_rb,6.7240e-01_rb,5.0429e-01_rb, &
     &3.3620e-01_rb,1.6810e-01_rb,3.8444e-06_rb/)
      kao(:, 2,12, 5) = (/ &
     &1.3870e+00_rb,1.2136e+00_rb,1.0403e+00_rb,8.6689e-01_rb,6.9351e-01_rb,5.2014e-01_rb, &
     &3.4675e-01_rb,1.7338e-01_rb,8.1656e-06_rb/)
      kao(:, 3,12, 5) = (/ &
     &1.4199e+00_rb,1.2424e+00_rb,1.0649e+00_rb,8.8747e-01_rb,7.0998e-01_rb,5.3249e-01_rb, &
     &3.5500e-01_rb,1.7752e-01_rb,1.3518e-05_rb/)
      kao(:, 4,12, 5) = (/ &
     &1.4553e+00_rb,1.2734e+00_rb,1.0915e+00_rb,9.0958e-01_rb,7.2769e-01_rb,5.4578e-01_rb, &
     &3.6386e-01_rb,1.8196e-01_rb,2.0674e-05_rb/)
      kao(:, 5,12, 5) = (/ &
     &1.5049e+00_rb,1.3168e+00_rb,1.1287e+00_rb,9.4058e-01_rb,7.5249e-01_rb,5.6439e-01_rb, &
     &3.7628e-01_rb,1.8817e-01_rb,3.3221e-05_rb/)
      kao(:, 1,13, 5) = (/ &
     &1.5112e+00_rb,1.3223e+00_rb,1.1334e+00_rb,9.4452e-01_rb,7.5561e-01_rb,5.6670e-01_rb, &
     &3.7781e-01_rb,1.8891e-01_rb,3.4136e-06_rb/)
      kao(:, 2,13, 5) = (/ &
     &1.5630e+00_rb,1.3677e+00_rb,1.1723e+00_rb,9.7690e-01_rb,7.8153e-01_rb,5.8614e-01_rb, &
     &3.9077e-01_rb,1.9539e-01_rb,7.2314e-06_rb/)
      kao(:, 3,13, 5) = (/ &
     &1.6029e+00_rb,1.4025e+00_rb,1.2022e+00_rb,1.0018e+00_rb,8.0146e-01_rb,6.0110e-01_rb, &
     &4.0074e-01_rb,2.0039e-01_rb,1.2933e-05_rb/)
      kao(:, 4,13, 5) = (/ &
     &1.6548e+00_rb,1.4480e+00_rb,1.2411e+00_rb,1.0343e+00_rb,8.2742e-01_rb,6.2059e-01_rb, &
     &4.1374e-01_rb,2.0689e-01_rb,2.1420e-05_rb/)
      kao(:, 5,13, 5) = (/ &
     &1.7129e+00_rb,1.4988e+00_rb,1.2846e+00_rb,1.0706e+00_rb,8.5645e-01_rb,6.4235e-01_rb, &
     &4.2827e-01_rb,2.1415e-01_rb,3.1482e-05_rb/)
      kao(:, 1, 1, 6) = (/ &
     &4.0608e-05_rb,4.2185e-05_rb,3.9730e-05_rb,2.6630e-05_rb,1.5870e-05_rb,2.1720e-05_rb, &
     &4.0183e-05_rb,5.7933e-05_rb,1.9230e-05_rb/)
      kao(:, 2, 1, 6) = (/ &
     &4.0560e-05_rb,4.3924e-05_rb,4.2843e-05_rb,4.0587e-05_rb,1.9599e-05_rb,1.9434e-05_rb, &
     &2.9187e-05_rb,8.1729e-05_rb,2.4180e-05_rb/)
      kao(:, 3, 1, 6) = (/ &
     &4.0596e-05_rb,4.5223e-05_rb,4.7233e-05_rb,4.8206e-05_rb,4.1815e-05_rb,2.4282e-05_rb, &
     &2.7535e-05_rb,5.3025e-05_rb,4.5769e-05_rb/)
      kao(:, 4, 1, 6) = (/ &
     &4.0424e-05_rb,4.7527e-05_rb,5.3798e-05_rb,5.7119e-05_rb,5.7658e-05_rb,5.2268e-05_rb, &
     &3.6355e-05_rb,4.4941e-05_rb,8.6910e-05_rb/)
      kao(:, 5, 1, 6) = (/ &
     &4.0234e-05_rb,5.1685e-05_rb,6.0703e-05_rb,6.7173e-05_rb,7.3473e-05_rb,7.8264e-05_rb, &
     &7.7542e-05_rb,6.1561e-05_rb,1.1878e-04_rb/)
      kao(:, 1, 2, 6) = (/ &
     &7.1743e-05_rb,6.9776e-05_rb,6.4069e-05_rb,5.6253e-05_rb,3.2492e-05_rb,1.5299e-05_rb, &
     &2.5979e-05_rb,5.8422e-05_rb,1.3265e-05_rb/)
      kao(:, 2, 2, 6) = (/ &
     &7.1061e-05_rb,7.2097e-05_rb,6.7274e-05_rb,6.0734e-05_rb,5.1280e-05_rb,2.1520e-05_rb, &
     &1.9911e-05_rb,4.4529e-05_rb,1.9523e-05_rb/)
      kao(:, 3, 2, 6) = (/ &
     &7.0841e-05_rb,7.4370e-05_rb,7.0760e-05_rb,6.5655e-05_rb,6.0306e-05_rb,4.1020e-05_rb, &
     &2.4466e-05_rb,3.0981e-05_rb,4.7368e-05_rb/)
      kao(:, 4, 2, 6) = (/ &
     &7.0995e-05_rb,7.6358e-05_rb,7.4085e-05_rb,7.2263e-05_rb,6.9627e-05_rb,6.3412e-05_rb, &
     &3.9010e-05_rb,3.3220e-05_rb,7.3941e-05_rb/)
      kao(:, 5, 2, 6) = (/ &
     &7.1418e-05_rb,7.8720e-05_rb,7.9350e-05_rb,8.0487e-05_rb,8.0635e-05_rb,7.8967e-05_rb, &
     &7.4070e-05_rb,4.7509e-05_rb,9.9451e-05_rb/)
      kao(:, 1, 3, 6) = (/ &
     &1.7804e-04_rb,1.6309e-04_rb,1.4603e-04_rb,1.2640e-04_rb,1.0532e-04_rb,7.7851e-05_rb, &
     &3.0401e-05_rb,3.2808e-05_rb,1.0278e-05_rb/)
      kao(:, 2, 3, 6) = (/ &
     &1.7807e-04_rb,1.6656e-04_rb,1.5059e-04_rb,1.3159e-04_rb,1.1024e-04_rb,8.6966e-05_rb, &
     &3.8390e-05_rb,2.7178e-05_rb,2.3712e-05_rb/)
      kao(:, 3, 3, 6) = (/ &
     &1.7838e-04_rb,1.6972e-04_rb,1.5616e-04_rb,1.3747e-04_rb,1.1728e-04_rb,9.5070e-05_rb, &
     &6.7563e-05_rb,2.2396e-05_rb,4.2459e-05_rb/)
      kao(:, 4, 3, 6) = (/ &
     &1.7857e-04_rb,1.7475e-04_rb,1.6100e-04_rb,1.4311e-04_rb,1.2411e-04_rb,1.0321e-04_rb, &
     &8.1061e-05_rb,2.7084e-05_rb,5.9922e-05_rb/)
      kao(:, 5, 3, 6) = (/ &
     &1.7913e-04_rb,1.7932e-04_rb,1.6663e-04_rb,1.4905e-04_rb,1.3079e-04_rb,1.1234e-04_rb, &
     &9.4176e-05_rb,5.9304e-05_rb,8.0703e-05_rb/)
      kao(:, 1, 4, 6) = (/ &
     &4.5849e-04_rb,4.0719e-04_rb,3.5630e-04_rb,3.0212e-04_rb,2.4723e-04_rb,1.9208e-04_rb, &
     &1.3224e-04_rb,4.0670e-05_rb,1.0786e-05_rb/)
      kao(:, 2, 4, 6) = (/ &
     &4.6326e-04_rb,4.1501e-04_rb,3.6371e-04_rb,3.1046e-04_rb,2.5726e-04_rb,2.0145e-04_rb, &
     &1.4032e-04_rb,4.2661e-05_rb,2.2724e-05_rb/)
      kao(:, 3, 4, 6) = (/ &
     &4.5999e-04_rb,4.1805e-04_rb,3.6892e-04_rb,3.1975e-04_rb,2.6673e-04_rb,2.0947e-04_rb, &
     &1.4924e-04_rb,6.8954e-05_rb,3.3787e-05_rb/)
      kao(:, 4, 4, 6) = (/ &
     &4.6003e-04_rb,4.1972e-04_rb,3.7753e-04_rb,3.3032e-04_rb,2.7779e-04_rb,2.1909e-04_rb, &
     &1.5785e-04_rb,9.1573e-05_rb,4.7343e-05_rb/)
      kao(:, 5, 4, 6) = (/ &
     &4.6253e-04_rb,4.2854e-04_rb,3.8917e-04_rb,3.4007e-04_rb,2.8599e-04_rb,2.2760e-04_rb, &
     &1.6713e-04_rb,1.0392e-04_rb,6.4507e-05_rb/)
      kao(:, 1, 5, 6) = (/ &
     &1.1046e-03_rb,9.7210e-04_rb,8.3818e-04_rb,7.0474e-04_rb,5.7182e-04_rb,4.3731e-04_rb, &
     &3.0078e-04_rb,1.5849e-04_rb,1.0669e-05_rb/)
      kao(:, 2, 5, 6) = (/ &
     &1.1184e-03_rb,9.8522e-04_rb,8.5052e-04_rb,7.1956e-04_rb,5.8448e-04_rb,4.4708e-04_rb, &
     &3.0943e-04_rb,1.6582e-04_rb,1.7856e-05_rb/)
      kao(:, 3, 5, 6) = (/ &
     &1.1214e-03_rb,9.8952e-04_rb,8.6298e-04_rb,7.3058e-04_rb,5.9526e-04_rb,4.5967e-04_rb, &
     &3.2343e-04_rb,1.7504e-04_rb,2.7384e-05_rb/)
      kao(:, 4, 5, 6) = (/ &
     &1.1214e-03_rb,9.9887e-04_rb,8.7233e-04_rb,7.4058e-04_rb,6.1074e-04_rb,4.7706e-04_rb, &
     &3.3627e-04_rb,1.8500e-04_rb,4.0071e-05_rb/)
      kao(:, 5, 5, 6) = (/ &
     &1.1239e-03_rb,1.0058e-03_rb,8.8047e-04_rb,7.5602e-04_rb,6.2667e-04_rb,4.9193e-04_rb, &
     &3.4864e-04_rb,1.9526e-04_rb,5.6262e-05_rb/)
      kao(:, 1, 6, 6) = (/ &
     &2.6585e-03_rb,2.3289e-03_rb,2.0017e-03_rb,1.6725e-03_rb,1.3436e-03_rb,1.0134e-03_rb, &
     &6.8348e-04_rb,3.5116e-04_rb,8.2810e-06_rb/)
      kao(:, 2, 6, 6) = (/ &
     &2.5863e-03_rb,2.2683e-03_rb,1.9501e-03_rb,1.6316e-03_rb,1.3110e-03_rb,9.9174e-04_rb, &
     &6.7274e-04_rb,3.5111e-04_rb,1.4039e-05_rb/)
      kao(:, 3, 6, 6) = (/ &
     &2.6176e-03_rb,2.2991e-03_rb,1.9771e-03_rb,1.6537e-03_rb,1.3367e-03_rb,1.0179e-03_rb, &
     &6.9352e-04_rb,3.6693e-04_rb,2.2683e-05_rb/)
      kao(:, 4, 6, 6) = (/ &
     &2.6217e-03_rb,2.3046e-03_rb,1.9871e-03_rb,1.6757e-03_rb,1.3567e-03_rb,1.0347e-03_rb, &
     &7.1195e-04_rb,3.8176e-04_rb,3.4748e-05_rb/)
      kao(:, 5, 6, 6) = (/ &
     &2.6329e-03_rb,2.3195e-03_rb,2.0094e-03_rb,1.6960e-03_rb,1.3768e-03_rb,1.0589e-03_rb, &
     &7.3230e-04_rb,3.9783e-04_rb,4.9695e-05_rb/)
      kao(:, 1, 7, 6) = (/ &
     &6.5036e-03_rb,5.6927e-03_rb,4.8817e-03_rb,4.0715e-03_rb,3.2636e-03_rb,2.4531e-03_rb, &
     &1.6426e-03_rb,8.2916e-04_rb,6.2936e-06_rb/)
      kao(:, 2, 7, 6) = (/ &
     &6.4462e-03_rb,5.6438e-03_rb,4.8427e-03_rb,4.0443e-03_rb,3.2425e-03_rb,2.4409e-03_rb, &
     &1.6364e-03_rb,8.3288e-04_rb,1.1136e-05_rb/)
      kao(:, 3, 7, 6) = (/ &
     &6.4103e-03_rb,5.6141e-03_rb,4.8233e-03_rb,4.0268e-03_rb,3.2317e-03_rb,2.4336e-03_rb, &
     &1.6355e-03_rb,8.3675e-04_rb,1.7991e-05_rb/)
      kao(:, 4, 7, 6) = (/ &
     &6.4801e-03_rb,5.6797e-03_rb,4.8784e-03_rb,4.0772e-03_rb,3.2734e-03_rb,2.4699e-03_rb, &
     &1.6705e-03_rb,8.6234e-04_rb,2.8124e-05_rb/)
      kao(:, 5, 7, 6) = (/ &
     &6.4899e-03_rb,5.6954e-03_rb,4.8999e-03_rb,4.1019e-03_rb,3.3007e-03_rb,2.5022e-03_rb, &
     &1.6963e-03_rb,8.8568e-04_rb,4.2294e-05_rb/)
      kao(:, 1, 8, 6) = (/ &
     &1.9938e-02_rb,1.7447e-02_rb,1.4956e-02_rb,1.2465e-02_rb,9.9746e-03_rb,7.4836e-03_rb, &
     &4.9963e-03_rb,2.5067e-03_rb,5.1161e-06_rb/)
      kao(:, 2, 8, 6) = (/ &
     &2.0028e-02_rb,1.7527e-02_rb,1.5026e-02_rb,1.2525e-02_rb,1.0025e-02_rb,7.5295e-03_rb, &
     &5.0298e-03_rb,2.5282e-03_rb,9.0850e-06_rb/)
      kao(:, 3, 8, 6) = (/ &
     &1.9800e-02_rb,1.7330e-02_rb,1.4859e-02_rb,1.2391e-02_rb,9.9275e-03_rb,7.4581e-03_rb, &
     &4.9868e-03_rb,2.5105e-03_rb,1.5352e-05_rb/)
      kao(:, 4, 8, 6) = (/ &
     &1.9943e-02_rb,1.7456e-02_rb,1.4973e-02_rb,1.2495e-02_rb,1.0008e-02_rb,7.5213e-03_rb, &
     &5.0327e-03_rb,2.5396e-03_rb,2.4408e-05_rb/)
      kao(:, 5, 8, 6) = (/ &
     &2.0100e-02_rb,1.7597e-02_rb,1.5105e-02_rb,1.2604e-02_rb,1.0099e-02_rb,7.5963e-03_rb, &
     &5.0871e-03_rb,2.5804e-03_rb,3.7067e-05_rb/)
      kao(:, 1, 9, 6) = (/ &
     &1.1579e-01_rb,1.0132e-01_rb,8.6843e-02_rb,7.2372e-02_rb,5.7899e-02_rb,4.3426e-02_rb, &
     &2.8954e-02_rb,1.4481e-02_rb,4.8758e-06_rb/)
      kao(:, 2, 9, 6) = (/ &
     &1.1661e-01_rb,1.0203e-01_rb,8.7460e-02_rb,7.2886e-02_rb,5.8311e-02_rb,4.3737e-02_rb, &
     &2.9163e-02_rb,1.4591e-02_rb,1.0307e-05_rb/)
      kao(:, 3, 9, 6) = (/ &
     &1.1709e-01_rb,1.0246e-01_rb,8.7825e-02_rb,7.3192e-02_rb,5.8559e-02_rb,4.3925e-02_rb, &
     &2.9293e-02_rb,1.4674e-02_rb,1.8955e-05_rb/)
      kao(:, 4, 9, 6) = (/ &
     &1.1566e-01_rb,1.0121e-01_rb,8.6757e-02_rb,7.2304e-02_rb,5.7852e-02_rb,4.3402e-02_rb, &
     &2.8967e-02_rb,1.4512e-02_rb,3.1114e-05_rb/)
      kao(:, 5, 9, 6) = (/ &
     &1.1720e-01_rb,1.0255e-01_rb,8.7909e-02_rb,7.3266e-02_rb,5.8634e-02_rb,4.4011e-02_rb, &
     &2.9372e-02_rb,1.4720e-02_rb,4.7731e-05_rb/)
      kao(:, 1,10, 6) = (/ &
     &7.9704e-01_rb,6.9741e-01_rb,5.9778e-01_rb,4.9815e-01_rb,3.9852e-01_rb,2.9889e-01_rb, &
     &1.9926e-01_rb,9.9635e-02_rb,6.0881e-06_rb/)
      kao(:, 2,10, 6) = (/ &
     &8.0524e-01_rb,7.0458e-01_rb,6.0393e-01_rb,5.0328e-01_rb,4.0263e-01_rb,3.0197e-01_rb, &
     &2.0132e-01_rb,1.0067e-01_rb,9.9592e-06_rb/)
      kao(:, 3,10, 6) = (/ &
     &8.0944e-01_rb,7.0827e-01_rb,6.0708e-01_rb,5.0591e-01_rb,4.0473e-01_rb,3.0356e-01_rb, &
     &2.0238e-01_rb,1.0120e-01_rb,1.7097e-05_rb/)
      kao(:, 4,10, 6) = (/ &
     &8.0782e-01_rb,7.0685e-01_rb,6.0589e-01_rb,5.0491e-01_rb,4.0393e-01_rb,3.0295e-01_rb, &
     &2.0198e-01_rb,1.0100e-01_rb,2.9862e-05_rb/)
      kao(:, 5,10, 6) = (/ &
     &8.0293e-01_rb,7.0255e-01_rb,6.0220e-01_rb,5.0184e-01_rb,4.0149e-01_rb,3.0112e-01_rb, &
     &2.0077e-01_rb,1.0043e-01_rb,4.8674e-05_rb/)
      kao(:, 1,11, 6) = (/ &
     &1.8320e+00_rb,1.6031e+00_rb,1.3740e+00_rb,1.1450e+00_rb,9.1602e-01_rb,6.8703e-01_rb, &
     &4.5802e-01_rb,2.2901e-01_rb,6.4794e-06_rb/)
      kao(:, 2,11, 6) = (/ &
     &1.8510e+00_rb,1.6196e+00_rb,1.3883e+00_rb,1.1569e+00_rb,9.2553e-01_rb,6.9415e-01_rb, &
     &4.6277e-01_rb,2.3139e-01_rb,1.1218e-05_rb/)
      kao(:, 3,11, 6) = (/ &
     &1.8631e+00_rb,1.6303e+00_rb,1.3974e+00_rb,1.1645e+00_rb,9.3160e-01_rb,6.9871e-01_rb, &
     &4.6580e-01_rb,2.3291e-01_rb,1.9228e-05_rb/)
      kao(:, 4,11, 6) = (/ &
     &1.8640e+00_rb,1.6310e+00_rb,1.3980e+00_rb,1.1650e+00_rb,9.3201e-01_rb,6.9904e-01_rb, &
     &4.6603e-01_rb,2.3304e-01_rb,3.3633e-05_rb/)
      kao(:, 5,11, 6) = (/ &
     &1.8480e+00_rb,1.6170e+00_rb,1.3860e+00_rb,1.1550e+00_rb,9.2402e-01_rb,6.9305e-01_rb, &
     &4.6206e-01_rb,2.3108e-01_rb,5.6086e-05_rb/)
      kao(:, 1,12, 6) = (/ &
     &2.6276e+00_rb,2.2991e+00_rb,1.9707e+00_rb,1.6422e+00_rb,1.3138e+00_rb,9.8536e-01_rb, &
     &6.5690e-01_rb,3.2845e-01_rb,6.1969e-06_rb/)
      kao(:, 2,12, 6) = (/ &
     &2.6587e+00_rb,2.3264e+00_rb,1.9940e+00_rb,1.6617e+00_rb,1.3293e+00_rb,9.9701e-01_rb, &
     &6.6468e-01_rb,3.3234e-01_rb,9.9191e-06_rb/)
      kao(:, 3,12, 6) = (/ &
     &2.6771e+00_rb,2.3425e+00_rb,2.0078e+00_rb,1.6732e+00_rb,1.3386e+00_rb,1.0039e+00_rb, &
     &6.6929e-01_rb,3.3465e-01_rb,1.7326e-05_rb/)
      kao(:, 4,12, 6) = (/ &
     &2.6786e+00_rb,2.3438e+00_rb,2.0090e+00_rb,1.6742e+00_rb,1.3393e+00_rb,1.0045e+00_rb, &
     &6.6968e-01_rb,3.3487e-01_rb,3.0600e-05_rb/)
      kao(:, 5,12, 6) = (/ &
     &2.6738e+00_rb,2.3396e+00_rb,2.0054e+00_rb,1.6711e+00_rb,1.3369e+00_rb,1.0027e+00_rb, &
     &6.6852e-01_rb,3.3432e-01_rb,4.9574e-05_rb/)
      kao(:, 1,13, 6) = (/ &
     &3.0141e+00_rb,2.6373e+00_rb,2.2606e+00_rb,1.8838e+00_rb,1.5070e+00_rb,1.1303e+00_rb, &
     &7.5353e-01_rb,3.7676e-01_rb,5.4676e-06_rb/)
      kao(:, 2,13, 6) = (/ &
     &3.0533e+00_rb,2.6716e+00_rb,2.2900e+00_rb,1.9083e+00_rb,1.5266e+00_rb,1.1450e+00_rb, &
     &7.6332e-01_rb,3.8166e-01_rb,9.0049e-06_rb/)
      kao(:, 3,13, 6) = (/ &
     &3.0797e+00_rb,2.6947e+00_rb,2.3097e+00_rb,1.9248e+00_rb,1.5398e+00_rb,1.1549e+00_rb, &
     &7.6993e-01_rb,3.8499e-01_rb,1.5493e-05_rb/)
      kao(:, 4,13, 6) = (/ &
     &3.0841e+00_rb,2.6986e+00_rb,2.3131e+00_rb,1.9276e+00_rb,1.5421e+00_rb,1.1566e+00_rb, &
     &7.7107e-01_rb,3.8557e-01_rb,2.5494e-05_rb/)
      kao(:, 5,13, 6) = (/ &
     &3.0968e+00_rb,2.7097e+00_rb,2.3226e+00_rb,1.9355e+00_rb,1.5484e+00_rb,1.1613e+00_rb, &
     &7.7428e-01_rb,3.8723e-01_rb,4.2043e-05_rb/)
      kao(:, 1, 1, 7) = (/ &
     &4.9351e-05_rb,5.1634e-05_rb,5.0550e-05_rb,4.1436e-05_rb,1.6376e-05_rb,2.1065e-05_rb, &
     &3.5627e-05_rb,5.1301e-05_rb,2.4192e-05_rb/)
      kao(:, 2, 1, 7) = (/ &
     &4.8474e-05_rb,5.5540e-05_rb,5.6343e-05_rb,5.4221e-05_rb,4.2170e-05_rb,2.4823e-05_rb, &
     &3.1731e-05_rb,5.9201e-05_rb,4.8733e-05_rb/)
      kao(:, 3, 1, 7) = (/ &
     &4.7492e-05_rb,6.0199e-05_rb,6.3032e-05_rb,6.6317e-05_rb,6.8647e-05_rb,6.4541e-05_rb, &
     &4.0308e-05_rb,5.5421e-05_rb,1.0385e-04_rb/)
      kao(:, 4, 1, 7) = (/ &
     &4.5731e-05_rb,6.4480e-05_rb,7.0656e-05_rb,8.0927e-05_rb,9.1209e-05_rb,1.0812e-04_rb, &
     &1.1177e-04_rb,6.7578e-05_rb,1.5053e-04_rb/)
      kao(:, 5, 1, 7) = (/ &
     &4.5536e-05_rb,6.8791e-05_rb,8.4361e-05_rb,1.0008e-04_rb,1.1558e-04_rb,1.4310e-04_rb, &
     &1.8138e-04_rb,1.7860e-04_rb,2.0339e-04_rb/)
      kao(:, 1, 2, 7) = (/ &
     &8.7853e-05_rb,8.5975e-05_rb,8.0217e-05_rb,7.2158e-05_rb,5.1923e-05_rb,1.7714e-05_rb, &
     &2.3551e-05_rb,3.4001e-05_rb,1.8843e-05_rb/)
      kao(:, 2, 2, 7) = (/ &
     &8.7406e-05_rb,8.8950e-05_rb,8.5976e-05_rb,7.9247e-05_rb,6.9532e-05_rb,3.6395e-05_rb, &
     &2.2706e-05_rb,4.1033e-05_rb,5.5544e-05_rb/)
      kao(:, 3, 2, 7) = (/ &
     &8.7368e-05_rb,9.3020e-05_rb,9.1442e-05_rb,8.8073e-05_rb,8.2295e-05_rb,7.2901e-05_rb, &
     &4.3354e-05_rb,4.6618e-05_rb,9.4748e-05_rb/)
      kao(:, 4, 2, 7) = (/ &
     &8.6439e-05_rb,9.8041e-05_rb,9.9373e-05_rb,9.8643e-05_rb,9.7447e-05_rb,9.7462e-05_rb, &
     &9.8905e-05_rb,6.1372e-05_rb,1.3473e-04_rb/)
      kao(:, 5, 2, 7) = (/ &
     &8.4149e-05_rb,1.0269e-04_rb,1.0851e-04_rb,1.1232e-04_rb,1.1694e-04_rb,1.2384e-04_rb, &
     &1.3615e-04_rb,1.5254e-04_rb,1.8188e-04_rb/)
      kao(:, 1, 3, 7) = (/ &
     &2.2417e-04_rb,2.0404e-04_rb,1.8235e-04_rb,1.5974e-04_rb,1.3536e-04_rb,1.0733e-04_rb, &
     &2.8889e-05_rb,2.3292e-05_rb,2.7610e-05_rb/)
      kao(:, 2, 3, 7) = (/ &
     &2.2507e-04_rb,2.0905e-04_rb,1.8945e-04_rb,1.6889e-04_rb,1.4442e-04_rb,1.1619e-04_rb, &
     &5.9496e-05_rb,2.4524e-05_rb,5.6641e-05_rb/)
      kao(:, 3, 3, 7) = (/ &
     &2.2338e-04_rb,2.1298e-04_rb,1.9588e-04_rb,1.7647e-04_rb,1.5321e-04_rb,1.2706e-04_rb, &
     &9.4765e-05_rb,2.7601e-05_rb,8.4458e-05_rb/)
      kao(:, 4, 3, 7) = (/ &
     &2.2342e-04_rb,2.1728e-04_rb,2.0413e-04_rb,1.8620e-04_rb,1.6349e-04_rb,1.4113e-04_rb, &
     &1.1467e-04_rb,6.7006e-05_rb,1.1844e-04_rb/)
      kao(:, 5, 3, 7) = (/ &
     &2.2238e-04_rb,2.2192e-04_rb,2.1227e-04_rb,1.9647e-04_rb,1.7779e-04_rb,1.5802e-04_rb, &
     &1.3658e-04_rb,1.1946e-04_rb,1.6048e-04_rb/)
      kao(:, 1, 4, 7) = (/ &
     &5.9406e-04_rb,5.2735e-04_rb,4.5854e-04_rb,3.9146e-04_rb,3.2173e-04_rb,2.5115e-04_rb, &
     &1.8204e-04_rb,3.3380e-05_rb,2.9857e-05_rb/)
      kao(:, 2, 4, 7) = (/ &
     &6.0105e-04_rb,5.3531e-04_rb,4.6978e-04_rb,4.0022e-04_rb,3.3097e-04_rb,2.6221e-04_rb, &
     &1.8773e-04_rb,5.6092e-05_rb,4.7642e-05_rb/)
      kao(:, 3, 4, 7) = (/ &
     &5.9920e-04_rb,5.3954e-04_rb,4.7635e-04_rb,4.1122e-04_rb,3.4497e-04_rb,2.7525e-04_rb, &
     &1.9878e-04_rb,1.0920e-04_rb,7.1351e-05_rb/)
      kao(:, 4, 4, 7) = (/ &
     &5.9595e-04_rb,5.4459e-04_rb,4.8204e-04_rb,4.2190e-04_rb,3.5656e-04_rb,2.8691e-04_rb, &
     &2.1194e-04_rb,1.2834e-04_rb,1.0247e-04_rb/)
      kao(:, 5, 4, 7) = (/ &
     &5.9682e-04_rb,5.4845e-04_rb,4.9542e-04_rb,4.3507e-04_rb,3.7084e-04_rb,3.0256e-04_rb, &
     &2.2629e-04_rb,1.4673e-04_rb,1.4061e-04_rb/)
      kao(:, 1, 5, 7) = (/ &
     &1.5636e-03_rb,1.3738e-03_rb,1.1854e-03_rb,9.9627e-04_rb,8.0358e-04_rb,6.1134e-04_rb, &
     &4.1794e-04_rb,2.1940e-04_rb,2.4302e-05_rb/)
      kao(:, 2, 5, 7) = (/ &
     &1.5046e-03_rb,1.3266e-03_rb,1.1472e-03_rb,9.6320e-04_rb,7.8248e-04_rb,5.9833e-04_rb, &
     &4.1211e-04_rb,2.2595e-04_rb,3.8463e-05_rb/)
      kao(:, 3, 5, 7) = (/ &
     &1.5168e-03_rb,1.3431e-03_rb,1.1615e-03_rb,9.8412e-04_rb,8.0210e-04_rb,6.1684e-04_rb, &
     &4.2935e-04_rb,2.3832e-04_rb,5.7958e-05_rb/)
      kao(:, 4, 5, 7) = (/ &
     &1.5026e-03_rb,1.3336e-03_rb,1.1649e-03_rb,9.9050e-04_rb,8.0890e-04_rb,6.2936e-04_rb, &
     &4.4480e-04_rb,2.5053e-04_rb,8.4195e-05_rb/)
      kao(:, 5, 5, 7) = (/ &
     &1.4982e-03_rb,1.3418e-03_rb,1.1752e-03_rb,9.9791e-04_rb,8.2403e-04_rb,6.4480e-04_rb, &
     &4.6309e-04_rb,2.6575e-04_rb,1.1855e-04_rb/)
      kao(:, 1, 6, 7) = (/ &
     &3.8022e-03_rb,3.3297e-03_rb,2.8587e-03_rb,2.3909e-03_rb,1.9206e-03_rb,1.4508e-03_rb, &
     &9.7686e-04_rb,5.0103e-04_rb,1.8374e-05_rb/)
      kao(:, 2, 6, 7) = (/ &
     &3.7188e-03_rb,3.2606e-03_rb,2.8078e-03_rb,2.3509e-03_rb,1.8941e-03_rb,1.4317e-03_rb, &
     &9.6949e-04_rb,5.0170e-04_rb,2.9781e-05_rb/)
      kao(:, 3, 6, 7) = (/ &
     &3.6568e-03_rb,3.2124e-03_rb,2.7660e-03_rb,2.3181e-03_rb,1.8667e-03_rb,1.4148e-03_rb, &
     &9.6121e-04_rb,5.0473e-04_rb,4.5442e-05_rb/)
      kao(:, 4, 6, 7) = (/ &
     &3.6655e-03_rb,3.2274e-03_rb,2.7830e-03_rb,2.3318e-03_rb,1.8859e-03_rb,1.4380e-03_rb, &
     &9.8239e-04_rb,5.2260e-04_rb,6.7780e-05_rb/)
      kao(:, 5, 6, 7) = (/ &
     &3.6236e-03_rb,3.1965e-03_rb,2.7643e-03_rb,2.3328e-03_rb,1.8928e-03_rb,1.4444e-03_rb, &
     &9.9870e-04_rb,5.3910e-04_rb,9.8293e-05_rb/)
      kao(:, 1, 7, 7) = (/ &
     &9.7756e-03_rb,8.5557e-03_rb,7.3357e-03_rb,6.1160e-03_rb,4.8978e-03_rb,3.6826e-03_rb, &
     &2.4642e-03_rb,1.2466e-03_rb,1.4263e-05_rb/)
      kao(:, 2, 7, 7) = (/ &
     &9.5966e-03_rb,8.4005e-03_rb,7.2050e-03_rb,6.0128e-03_rb,4.8216e-03_rb,3.6274e-03_rb, &
     &2.4345e-03_rb,1.2337e-03_rb,2.4094e-05_rb/)
      kao(:, 3, 7, 7) = (/ &
     &9.2939e-03_rb,8.1388e-03_rb,6.9869e-03_rb,5.8375e-03_rb,4.6843e-03_rb,3.5321e-03_rb, &
     &2.3720e-03_rb,1.2113e-03_rb,3.8518e-05_rb/)
      kao(:, 4, 7, 7) = (/ &
     &9.3185e-03_rb,8.1650e-03_rb,7.0200e-03_rb,5.8680e-03_rb,4.7180e-03_rb,3.5579e-03_rb, &
     &2.3933e-03_rb,1.2260e-03_rb,6.0065e-05_rb/)
      kao(:, 5, 7, 7) = (/ &
     &9.2942e-03_rb,8.1544e-03_rb,7.0105e-03_rb,5.8689e-03_rb,4.7186e-03_rb,3.5646e-03_rb, &
     &2.4113e-03_rb,1.2505e-03_rb,8.8114e-05_rb/)
      kao(:, 1, 8, 7) = (/ &
     &3.1428e-02_rb,2.7501e-02_rb,2.3574e-02_rb,1.9646e-02_rb,1.5719e-02_rb,1.1792e-02_rb, &
     &7.8652e-03_rb,3.9445e-03_rb,1.1179e-05_rb/)
      kao(:, 2, 8, 7) = (/ &
     &3.0933e-02_rb,2.7068e-02_rb,2.3204e-02_rb,1.9341e-02_rb,1.5477e-02_rb,1.1614e-02_rb, &
     &7.7579e-03_rb,3.8961e-03_rb,2.0761e-05_rb/)
      kao(:, 3, 8, 7) = (/ &
     &3.0367e-02_rb,2.6575e-02_rb,2.2784e-02_rb,1.8992e-02_rb,1.5203e-02_rb,1.1421e-02_rb, &
     &7.6337e-03_rb,3.8475e-03_rb,3.6107e-05_rb/)
      kao(:, 4, 8, 7) = (/ &
     &2.9365e-02_rb,2.5702e-02_rb,2.2039e-02_rb,1.8381e-02_rb,1.4731e-02_rb,1.1072e-02_rb, &
     &7.4163e-03_rb,3.7447e-03_rb,5.6666e-05_rb/)
      kao(:, 5, 8, 7) = (/ &
     &2.9621e-02_rb,2.5930e-02_rb,2.2243e-02_rb,1.8569e-02_rb,1.4884e-02_rb,1.1199e-02_rb, &
     &7.5061e-03_rb,3.7957e-03_rb,8.4702e-05_rb/)
      kao(:, 1, 9, 7) = (/ &
     &1.9031e-01_rb,1.6652e-01_rb,1.4273e-01_rb,1.1894e-01_rb,9.5159e-02_rb,7.1370e-02_rb, &
     &4.7583e-02_rb,2.3794e-02_rb,1.1825e-05_rb/)
      kao(:, 2, 9, 7) = (/ &
     &1.8790e-01_rb,1.6442e-01_rb,1.4093e-01_rb,1.1744e-01_rb,9.3958e-02_rb,7.0471e-02_rb, &
     &4.6985e-02_rb,2.3500e-02_rb,2.0873e-05_rb/)
      kao(:, 3, 9, 7) = (/ &
     &1.8549e-01_rb,1.6231e-01_rb,1.3912e-01_rb,1.1594e-01_rb,9.2757e-02_rb,6.9574e-02_rb, &
     &4.6391e-02_rb,2.3213e-02_rb,3.5554e-05_rb/)
      kao(:, 4, 9, 7) = (/ &
     &1.8255e-01_rb,1.5974e-01_rb,1.3693e-01_rb,1.1411e-01_rb,9.1298e-02_rb,6.8486e-02_rb, &
     &4.5672e-02_rb,2.2886e-02_rb,5.8671e-05_rb/)
      kao(:, 5, 9, 7) = (/ &
     &1.7785e-01_rb,1.5563e-01_rb,1.3341e-01_rb,1.1118e-01_rb,8.8960e-02_rb,6.6737e-02_rb, &
     &4.4547e-02_rb,2.2336e-02_rb,9.5234e-05_rb/)
      kao(:, 1,10, 7) = (/ &
     &1.3607e+00_rb,1.1905e+00_rb,1.0205e+00_rb,8.5040e-01_rb,6.8031e-01_rb,5.1024e-01_rb, &
     &3.4016e-01_rb,1.7008e-01_rb,8.4759e-06_rb/)
      kao(:, 2,10, 7) = (/ &
     &1.3482e+00_rb,1.1796e+00_rb,1.0111e+00_rb,8.4260e-01_rb,6.7408e-01_rb,5.0556e-01_rb, &
     &3.3705e-01_rb,1.6853e-01_rb,1.7938e-05_rb/)
      kao(:, 3,10, 7) = (/ &
     &1.3360e+00_rb,1.1690e+00_rb,1.0020e+00_rb,8.3498e-01_rb,6.6798e-01_rb,5.0099e-01_rb, &
     &3.3400e-01_rb,1.6701e-01_rb,3.3302e-05_rb/)
      kao(:, 4,10, 7) = (/ &
     &1.3243e+00_rb,1.1588e+00_rb,9.9325e-01_rb,8.2771e-01_rb,6.6217e-01_rb,4.9664e-01_rb, &
     &3.3111e-01_rb,1.6557e-01_rb,5.5097e-05_rb/)
      kao(:, 5,10, 7) = (/ &
     &1.3047e+00_rb,1.1416e+00_rb,9.7851e-01_rb,8.1543e-01_rb,6.5236e-01_rb,4.8929e-01_rb, &
     &3.2621e-01_rb,1.6314e-01_rb,9.0393e-05_rb/)
      kao(:, 1,11, 7) = (/ &
     &3.2276e+00_rb,2.8242e+00_rb,2.4208e+00_rb,2.0173e+00_rb,1.6138e+00_rb,1.2104e+00_rb, &
     &8.0693e-01_rb,4.0346e-01_rb,7.3681e-06_rb/)
      kao(:, 2,11, 7) = (/ &
     &3.2117e+00_rb,2.8103e+00_rb,2.4088e+00_rb,2.0073e+00_rb,1.6059e+00_rb,1.2044e+00_rb, &
     &8.0294e-01_rb,4.0147e-01_rb,1.5281e-05_rb/)
      kao(:, 3,11, 7) = (/ &
     &3.1949e+00_rb,2.7955e+00_rb,2.3962e+00_rb,1.9968e+00_rb,1.5975e+00_rb,1.1981e+00_rb, &
     &7.9874e-01_rb,3.9938e-01_rb,2.9166e-05_rb/)
      kao(:, 4,11, 7) = (/ &
     &3.1739e+00_rb,2.7771e+00_rb,2.3804e+00_rb,1.9837e+00_rb,1.5870e+00_rb,1.1902e+00_rb, &
     &7.9350e-01_rb,3.9677e-01_rb,4.9026e-05_rb/)
      kao(:, 5,11, 7) = (/ &
     &3.1466e+00_rb,2.7532e+00_rb,2.3599e+00_rb,1.9666e+00_rb,1.5733e+00_rb,1.1800e+00_rb, &
     &7.8668e-01_rb,3.9338e-01_rb,8.3128e-05_rb/)
      kao(:, 1,12, 7) = (/ &
     &4.8071e+00_rb,4.2062e+00_rb,3.6053e+00_rb,3.0044e+00_rb,2.4035e+00_rb,1.8027e+00_rb, &
     &1.2018e+00_rb,6.0089e-01_rb,7.8311e-06_rb/)
      kao(:, 2,12, 7) = (/ &
     &4.7982e+00_rb,4.1985e+00_rb,3.5987e+00_rb,2.9989e+00_rb,2.3991e+00_rb,1.7993e+00_rb, &
     &1.1996e+00_rb,5.9978e-01_rb,1.6500e-05_rb/)
      kao(:, 3,12, 7) = (/ &
     &4.7853e+00_rb,4.1872e+00_rb,3.5890e+00_rb,2.9909e+00_rb,2.3927e+00_rb,1.7945e+00_rb, &
     &1.1963e+00_rb,5.9818e-01_rb,3.1398e-05_rb/)
      kao(:, 4,12, 7) = (/ &
     &4.7636e+00_rb,4.1682e+00_rb,3.5727e+00_rb,2.9773e+00_rb,2.3818e+00_rb,1.7864e+00_rb, &
     &1.1909e+00_rb,5.9548e-01_rb,5.6315e-05_rb/)
      kao(:, 5,12, 7) = (/ &
     &4.7234e+00_rb,4.1330e+00_rb,3.5426e+00_rb,2.9521e+00_rb,2.3618e+00_rb,1.7713e+00_rb, &
     &1.1809e+00_rb,5.9047e-01_rb,9.4104e-05_rb/)
      kao(:, 1,13, 7) = (/ &
     &5.7320e+00_rb,5.0155e+00_rb,4.2989e+00_rb,3.5824e+00_rb,2.8660e+00_rb,2.1495e+00_rb, &
     &1.4330e+00_rb,7.1649e-01_rb,9.5542e-06_rb/)
      kao(:, 2,13, 7) = (/ &
     &5.7353e+00_rb,5.0184e+00_rb,4.3015e+00_rb,3.5845e+00_rb,2.8676e+00_rb,2.1507e+00_rb, &
     &1.4338e+00_rb,7.1691e-01_rb,1.9109e-05_rb/)
      kao(:, 3,13, 7) = (/ &
     &5.7278e+00_rb,5.0119e+00_rb,4.2959e+00_rb,3.5799e+00_rb,2.8639e+00_rb,2.1479e+00_rb, &
     &1.4320e+00_rb,7.1599e-01_rb,3.6436e-05_rb/)
      kao(:, 4,13, 7) = (/ &
     &5.7108e+00_rb,4.9969e+00_rb,4.2830e+00_rb,3.5692e+00_rb,2.8554e+00_rb,2.1415e+00_rb, &
     &1.4277e+00_rb,7.1386e-01_rb,6.4763e-05_rb/)
      kao(:, 5,13, 7) = (/ &
     &5.6635e+00_rb,4.9556e+00_rb,4.2476e+00_rb,3.5397e+00_rb,2.8317e+00_rb,2.1238e+00_rb, &
     &1.4159e+00_rb,7.0796e-01_rb,1.0153e-04_rb/)
      kao(:, 1, 1, 8) = (/ &
     &5.6440e-05_rb,6.3815e-05_rb,6.7040e-05_rb,7.2656e-05_rb,5.7075e-05_rb,3.5497e-05_rb, &
     &4.7197e-05_rb,3.9064e-05_rb,8.4705e-05_rb/)
      kao(:, 2, 1, 8) = (/ &
     &5.6956e-05_rb,6.9440e-05_rb,7.5002e-05_rb,9.1724e-05_rb,1.1786e-04_rb,1.1581e-04_rb, &
     &7.2124e-05_rb,6.3862e-05_rb,2.0332e-04_rb/)
      kao(:, 3, 1, 8) = (/ &
     &5.4856e-05_rb,7.4835e-05_rb,8.9381e-05_rb,1.1688e-04_rb,1.6480e-04_rb,2.0403e-04_rb, &
     &2.1723e-04_rb,1.2068e-04_rb,2.9982e-04_rb/)
      kao(:, 4, 1, 8) = (/ &
     &5.2749e-05_rb,8.4849e-05_rb,1.1205e-04_rb,1.5064e-04_rb,2.1741e-04_rb,2.8178e-04_rb, &
     &3.3958e-04_rb,3.6475e-04_rb,4.0572e-04_rb/)
      kao(:, 5, 1, 8) = (/ &
     &4.5645e-05_rb,1.0085e-04_rb,1.3954e-04_rb,1.9411e-04_rb,2.8393e-04_rb,3.7289e-04_rb, &
     &4.5784e-04_rb,5.5744e-04_rb,5.3928e-04_rb/)
      kao(:, 1, 2, 8) = (/ &
     &1.0540e-04_rb,1.0513e-04_rb,1.0042e-04_rb,9.6258e-05_rb,8.8970e-05_rb,4.3184e-05_rb, &
     &2.9692e-05_rb,3.3913e-05_rb,8.0284e-05_rb/)
      kao(:, 2, 2, 8) = (/ &
     &1.0559e-04_rb,1.0940e-04_rb,1.0770e-04_rb,1.0381e-04_rb,1.0664e-04_rb,1.1502e-04_rb, &
     &8.1785e-05_rb,5.3525e-05_rb,1.4202e-04_rb/)
      kao(:, 3, 2, 8) = (/ &
     &1.0218e-04_rb,1.1568e-04_rb,1.1773e-04_rb,1.2016e-04_rb,1.3039e-04_rb,1.6512e-04_rb, &
     &1.8884e-04_rb,1.2031e-04_rb,2.0363e-04_rb/)
      kao(:, 4, 2, 8) = (/ &
     &9.8487e-05_rb,1.1922e-04_rb,1.3281e-04_rb,1.4606e-04_rb,1.6549e-04_rb,2.1784e-04_rb, &
     &2.7427e-04_rb,3.0595e-04_rb,2.8029e-04_rb/)
      kao(:, 5, 2, 8) = (/ &
     &9.4987e-05_rb,1.2846e-04_rb,1.5591e-04_rb,1.8268e-04_rb,2.1442e-04_rb,2.8547e-04_rb, &
     &3.6877e-04_rb,4.4657e-04_rb,3.7746e-04_rb/)
      kao(:, 1, 3, 8) = (/ &
     &2.7647e-04_rb,2.5246e-04_rb,2.2770e-04_rb,2.0041e-04_rb,1.7472e-04_rb,1.4681e-04_rb, &
     &4.5677e-05_rb,2.9185e-05_rb,7.9704e-05_rb/)
      kao(:, 2, 3, 8) = (/ &
     &2.7846e-04_rb,2.6060e-04_rb,2.3622e-04_rb,2.0968e-04_rb,1.8322e-04_rb,1.5414e-04_rb, &
     &1.2348e-04_rb,3.8961e-05_rb,1.2336e-04_rb/)
      kao(:, 3, 3, 8) = (/ &
     &2.7763e-04_rb,2.6427e-04_rb,2.4553e-04_rb,2.2406e-04_rb,1.9856e-04_rb,1.7101e-04_rb, &
     &1.5714e-04_rb,1.3645e-04_rb,1.7764e-04_rb/)
      kao(:, 4, 3, 8) = (/ &
     &2.7073e-04_rb,2.6684e-04_rb,2.5482e-04_rb,2.3763e-04_rb,2.1876e-04_rb,2.0158e-04_rb, &
     &1.9981e-04_rb,2.4173e-04_rb,2.4735e-04_rb/)
      kao(:, 5, 3, 8) = (/ &
     &2.6027e-04_rb,2.7336e-04_rb,2.6560e-04_rb,2.5755e-04_rb,2.5236e-04_rb,2.4811e-04_rb, &
     &2.5900e-04_rb,3.3153e-04_rb,3.3717e-04_rb/)
      kao(:, 1, 4, 8) = (/ &
     &7.9130e-04_rb,7.0063e-04_rb,6.1298e-04_rb,5.1987e-04_rb,4.2755e-04_rb,3.3326e-04_rb, &
     &2.3363e-04_rb,3.7051e-05_rb,7.4199e-05_rb/)
      kao(:, 2, 4, 8) = (/ &
     &7.5980e-04_rb,6.7845e-04_rb,5.9445e-04_rb,5.1095e-04_rb,4.1981e-04_rb,3.2923e-04_rb, &
     &2.4168e-04_rb,1.3135e-04_rb,1.1471e-04_rb/)
      kao(:, 3, 4, 8) = (/ &
     &7.6644e-04_rb,6.8984e-04_rb,6.0948e-04_rb,5.2391e-04_rb,4.3682e-04_rb,3.5056e-04_rb, &
     &2.5756e-04_rb,1.6433e-04_rb,1.6687e-04_rb/)
      kao(:, 4, 4, 8) = (/ &
     &7.5595e-04_rb,6.9178e-04_rb,6.1629e-04_rb,5.3601e-04_rb,4.5434e-04_rb,3.7215e-04_rb, &
     &2.8231e-04_rb,1.9566e-04_rb,2.3395e-04_rb/)
      kao(:, 5, 4, 8) = (/ &
     &7.4067e-04_rb,6.8825e-04_rb,6.1664e-04_rb,5.5301e-04_rb,4.7442e-04_rb,3.9621e-04_rb, &
     &3.1512e-04_rb,2.4572e-04_rb,3.2210e-04_rb/)
      kao(:, 1, 5, 8) = (/ &
     &2.0703e-03_rb,1.8139e-03_rb,1.5621e-03_rb,1.3106e-03_rb,1.0600e-03_rb,8.0652e-04_rb, &
     &5.5133e-04_rb,2.9069e-04_rb,6.3670e-05_rb/)
      kao(:, 2, 5, 8) = (/ &
     &1.9818e-03_rb,1.7441e-03_rb,1.5082e-03_rb,1.2728e-03_rb,1.0326e-03_rb,7.9434e-04_rb, &
     &5.4918e-04_rb,2.9978e-04_rb,1.0147e-04_rb/)
      kao(:, 3, 5, 8) = (/ &
     &1.9776e-03_rb,1.7438e-03_rb,1.5149e-03_rb,1.2743e-03_rb,1.0380e-03_rb,7.9656e-04_rb, &
     &5.5670e-04_rb,3.0633e-04_rb,1.5212e-04_rb/)
      kao(:, 4, 5, 8) = (/ &
     &1.9765e-03_rb,1.7561e-03_rb,1.5287e-03_rb,1.2973e-03_rb,1.0616e-03_rb,8.2154e-04_rb, &
     &5.8167e-04_rb,3.3115e-04_rb,2.1510e-04_rb/)
      kao(:, 5, 5, 8) = (/ &
     &1.9526e-03_rb,1.7391e-03_rb,1.5270e-03_rb,1.3031e-03_rb,1.0804e-03_rb,8.4755e-04_rb, &
     &6.0189e-04_rb,3.6102e-04_rb,2.9241e-04_rb/)
      kao(:, 1, 6, 8) = (/ &
     &5.2193e-03_rb,4.5684e-03_rb,3.9176e-03_rb,3.2705e-03_rb,2.6232e-03_rb,1.9766e-03_rb, &
     &1.3309e-03_rb,6.8212e-04_rb,5.1398e-05_rb/)
      kao(:, 2, 6, 8) = (/ &
     &5.0808e-03_rb,4.4486e-03_rb,3.8217e-03_rb,3.1970e-03_rb,2.5720e-03_rb,1.9485e-03_rb, &
     &1.3187e-03_rb,6.8335e-04_rb,8.5267e-05_rb/)
      kao(:, 3, 6, 8) = (/ &
     &4.8228e-03_rb,4.2284e-03_rb,3.6430e-03_rb,3.0578e-03_rb,2.4711e-03_rb,1.8742e-03_rb, &
     &1.2774e-03_rb,6.7264e-04_rb,1.3245e-04_rb/)
      kao(:, 4, 6, 8) = (/ &
     &4.8943e-03_rb,4.3003e-03_rb,3.7080e-03_rb,3.1199e-03_rb,2.5213e-03_rb,1.9154e-03_rb, &
     &1.3062e-03_rb,6.9055e-04_rb,1.9661e-04_rb/)
      kao(:, 5, 6, 8) = (/ &
     &4.8824e-03_rb,4.3025e-03_rb,3.7232e-03_rb,3.1333e-03_rb,2.5429e-03_rb,1.9512e-03_rb, &
     &1.3415e-03_rb,7.2251e-04_rb,2.7802e-04_rb/)
      kao(:, 1, 7, 8) = (/ &
     &1.3875e-02_rb,1.2141e-02_rb,1.0408e-02_rb,8.6749e-03_rb,6.9418e-03_rb,5.2118e-03_rb, &
     &3.4839e-03_rb,1.7560e-03_rb,4.0537e-05_rb/)
      kao(:, 2, 7, 8) = (/ &
     &1.3633e-02_rb,1.1931e-02_rb,1.0229e-02_rb,8.5279e-03_rb,6.8346e-03_rb,5.1400e-03_rb, &
     &3.4444e-03_rb,1.7477e-03_rb,6.8687e-05_rb/)
      kao(:, 3, 7, 8) = (/ &
     &1.3328e-02_rb,1.1666e-02_rb,1.0007e-02_rb,8.3587e-03_rb,6.7074e-03_rb,5.0533e-03_rb, &
     &3.4014e-03_rb,1.7372e-03_rb,1.1189e-04_rb/)
      kao(:, 4, 7, 8) = (/ &
     &1.2636e-02_rb,1.1066e-02_rb,9.5081e-03_rb,7.9486e-03_rb,6.3850e-03_rb,4.8258e-03_rb, &
     &3.2564e-03_rb,1.6750e-03_rb,1.7143e-04_rb/)
      kao(:, 5, 7, 8) = (/ &
     &1.2752e-02_rb,1.1174e-02_rb,9.6135e-03_rb,8.0399e-03_rb,6.4759e-03_rb,4.9028e-03_rb, &
     &3.3183e-03_rb,1.7153e-03_rb,2.5158e-04_rb/)
      kao(:, 1, 8, 8) = (/ &
     &4.6547e-02_rb,4.0730e-02_rb,3.4913e-02_rb,2.9095e-02_rb,2.3277e-02_rb,1.7460e-02_rb, &
     &1.1642e-02_rb,5.8282e-03_rb,3.2784e-05_rb/)
      kao(:, 2, 8, 8) = (/ &
     &4.5721e-02_rb,4.0008e-02_rb,3.4294e-02_rb,2.8581e-02_rb,2.2868e-02_rb,1.7154e-02_rb, &
     &1.1445e-02_rb,5.7448e-03_rb,5.6960e-05_rb/)
      kao(:, 3, 8, 8) = (/ &
     &4.4866e-02_rb,3.9261e-02_rb,3.3657e-02_rb,2.8053e-02_rb,2.2449e-02_rb,1.6850e-02_rb, &
     &1.1263e-02_rb,5.6659e-03_rb,9.3969e-05_rb/)
      kao(:, 4, 8, 8) = (/ &
     &4.3949e-02_rb,3.8462e-02_rb,3.2975e-02_rb,2.7489e-02_rb,2.2013e-02_rb,1.6548e-02_rb, &
     &1.1065e-02_rb,5.5874e-03_rb,1.4907e-04_rb/)
      kao(:, 5, 8, 8) = (/ &
     &4.1860e-02_rb,3.6641e-02_rb,3.1420e-02_rb,2.6211e-02_rb,2.1011e-02_rb,1.5788e-02_rb, &
     &1.0584e-02_rb,5.3726e-03_rb,2.2697e-04_rb/)
      kao(:, 1, 9, 8) = (/ &
     &2.9682e-01_rb,2.5971e-01_rb,2.2262e-01_rb,1.8551e-01_rb,1.4841e-01_rb,1.1131e-01_rb, &
     &7.4208e-02_rb,3.7107e-02_rb,2.8713e-05_rb/)
      kao(:, 2, 9, 8) = (/ &
     &2.9173e-01_rb,2.5526e-01_rb,2.1881e-01_rb,1.8234e-01_rb,1.4587e-01_rb,1.0941e-01_rb, &
     &7.2941e-02_rb,3.6477e-02_rb,5.3026e-05_rb/)
      kao(:, 3, 9, 8) = (/ &
     &2.8647e-01_rb,2.5066e-01_rb,2.1486e-01_rb,1.7905e-01_rb,1.4325e-01_rb,1.0744e-01_rb, &
     &7.1634e-02_rb,3.5829e-02_rb,9.1273e-05_rb/)
      kao(:, 4, 9, 8) = (/ &
     &2.8100e-01_rb,2.4588e-01_rb,2.1076e-01_rb,1.7564e-01_rb,1.4052e-01_rb,1.0540e-01_rb, &
     &7.0282e-02_rb,3.5177e-02_rb,1.4937e-04_rb/)
      kao(:, 5, 9, 8) = (/ &
     &2.7385e-01_rb,2.3963e-01_rb,2.0540e-01_rb,1.7118e-01_rb,1.3696e-01_rb,1.0274e-01_rb, &
     &6.8516e-02_rb,3.4350e-02_rb,2.3011e-04_rb/)
      kao(:, 1,10, 8) = (/ &
     &2.2461e+00_rb,1.9654e+00_rb,1.6846e+00_rb,1.4038e+00_rb,1.1231e+00_rb,8.4230e-01_rb, &
     &5.6154e-01_rb,2.8077e-01_rb,2.2240e-05_rb/)
      kao(:, 2,10, 8) = (/ &
     &2.2106e+00_rb,1.9343e+00_rb,1.6579e+00_rb,1.3816e+00_rb,1.1053e+00_rb,8.2897e-01_rb, &
     &5.5265e-01_rb,2.7633e-01_rb,4.4575e-05_rb/)
      kao(:, 3,10, 8) = (/ &
     &2.1737e+00_rb,1.9020e+00_rb,1.6303e+00_rb,1.3586e+00_rb,1.0869e+00_rb,8.1516e-01_rb, &
     &5.4345e-01_rb,2.7173e-01_rb,8.8587e-05_rb/)
      kao(:, 4,10, 8) = (/ &
     &2.1348e+00_rb,1.8680e+00_rb,1.6011e+00_rb,1.3342e+00_rb,1.0674e+00_rb,8.0056e-01_rb, &
     &5.3373e-01_rb,2.6688e-01_rb,1.5800e-04_rb/)
      kao(:, 5,10, 8) = (/ &
     &2.0952e+00_rb,1.8333e+00_rb,1.5714e+00_rb,1.3095e+00_rb,1.0476e+00_rb,7.8572e-01_rb, &
     &5.2383e-01_rb,2.6194e-01_rb,2.5854e-04_rb/)
      kao(:, 1,11, 8) = (/ &
     &5.6263e+00_rb,4.9230e+00_rb,4.2197e+00_rb,3.5164e+00_rb,2.8132e+00_rb,2.1099e+00_rb, &
     &1.4066e+00_rb,7.0330e-01_rb,2.4215e-05_rb/)
      kao(:, 2,11, 8) = (/ &
     &5.5442e+00_rb,4.8512e+00_rb,4.1582e+00_rb,3.4652e+00_rb,2.7721e+00_rb,2.0791e+00_rb, &
     &1.3861e+00_rb,6.9304e-01_rb,5.0673e-05_rb/)
      kao(:, 3,11, 8) = (/ &
     &5.4573e+00_rb,4.7752e+00_rb,4.0930e+00_rb,3.4108e+00_rb,2.7286e+00_rb,2.0465e+00_rb, &
     &1.3643e+00_rb,6.8218e-01_rb,9.6231e-05_rb/)
      kao(:, 4,11, 8) = (/ &
     &5.3719e+00_rb,4.7004e+00_rb,4.0290e+00_rb,3.3575e+00_rb,2.6860e+00_rb,2.0145e+00_rb, &
     &1.3430e+00_rb,6.7153e-01_rb,1.7007e-04_rb/)
      kao(:, 5,11, 8) = (/ &
     &5.2804e+00_rb,4.6204e+00_rb,3.9603e+00_rb,3.3002e+00_rb,2.6402e+00_rb,1.9802e+00_rb, &
     &1.3201e+00_rb,6.6009e-01_rb,2.7665e-04_rb/)
      kao(:, 1,12, 8) = (/ &
     &8.8343e+00_rb,7.7300e+00_rb,6.6256e+00_rb,5.5214e+00_rb,4.4171e+00_rb,3.3128e+00_rb, &
     &2.2085e+00_rb,1.1043e+00_rb,2.5372e-05_rb/)
      kao(:, 2,12, 8) = (/ &
     &8.7174e+00_rb,7.6278e+00_rb,6.5381e+00_rb,5.4484e+00_rb,4.3587e+00_rb,3.2690e+00_rb, &
     &2.1794e+00_rb,1.0897e+00_rb,5.2642e-05_rb/)
      kao(:, 3,12, 8) = (/ &
     &8.5962e+00_rb,7.5216e+00_rb,6.4471e+00_rb,5.3726e+00_rb,4.2981e+00_rb,3.2236e+00_rb, &
     &2.1490e+00_rb,1.0745e+00_rb,9.9934e-05_rb/)
      kao(:, 4,12, 8) = (/ &
     &8.4730e+00_rb,7.4139e+00_rb,6.3547e+00_rb,5.2956e+00_rb,4.2365e+00_rb,3.1774e+00_rb, &
     &2.1183e+00_rb,1.0591e+00_rb,1.7486e-04_rb/)
      kao(:, 5,12, 8) = (/ &
     &8.3427e+00_rb,7.2999e+00_rb,6.2571e+00_rb,5.2142e+00_rb,4.1714e+00_rb,3.1285e+00_rb, &
     &2.0857e+00_rb,1.0429e+00_rb,2.8225e-04_rb/)
      kao(:, 1,13, 8) = (/ &
     &1.1088e+01_rb,9.7019e+00_rb,8.3159e+00_rb,6.9299e+00_rb,5.5439e+00_rb,4.1579e+00_rb, &
     &2.7720e+00_rb,1.3860e+00_rb,2.2843e-05_rb/)
      kao(:, 2,13, 8) = (/ &
     &1.0963e+01_rb,9.5924e+00_rb,8.2220e+00_rb,6.8518e+00_rb,5.4814e+00_rb,4.1110e+00_rb, &
     &2.7407e+00_rb,1.3703e+00_rb,4.7912e-05_rb/)
      kao(:, 3,13, 8) = (/ &
     &1.0836e+01_rb,9.4815e+00_rb,8.1269e+00_rb,6.7725e+00_rb,5.4179e+00_rb,4.0635e+00_rb, &
     &2.7090e+00_rb,1.3545e+00_rb,9.1850e-05_rb/)
      kao(:, 4,13, 8) = (/ &
     &1.0705e+01_rb,9.3666e+00_rb,8.0284e+00_rb,6.6904e+00_rb,5.3523e+00_rb,4.0142e+00_rb, &
     &2.6761e+00_rb,1.3381e+00_rb,1.6522e-04_rb/)
      kao(:, 5,13, 8) = (/ &
     &1.0568e+01_rb,9.2475e+00_rb,7.9263e+00_rb,6.6053e+00_rb,5.2842e+00_rb,3.9632e+00_rb, &
     &2.6421e+00_rb,1.3211e+00_rb,2.8269e-04_rb/)
      kao(:, 1, 1, 9) = (/ &
     &6.2917e-05_rb,8.4558e-05_rb,1.3686e-04_rb,1.9740e-04_rb,2.4967e-04_rb,2.7515e-04_rb, &
     &1.6338e-04_rb,4.8223e-05_rb,4.6728e-04_rb/)
      kao(:, 2, 1, 9) = (/ &
     &5.3089e-05_rb,9.0420e-05_rb,1.8251e-04_rb,2.7670e-04_rb,3.5886e-04_rb,4.3834e-04_rb, &
     &4.9477e-04_rb,2.3929e-04_rb,6.8730e-04_rb/)
      kao(:, 3, 1, 9) = (/ &
     &4.3039e-05_rb,1.0946e-04_rb,2.4429e-04_rb,3.7961e-04_rb,4.9689e-04_rb,6.1288e-04_rb, &
     &7.2274e-04_rb,7.7240e-04_rb,9.6431e-04_rb/)
      kao(:, 4, 1, 9) = (/ &
     &3.4069e-05_rb,1.3830e-04_rb,3.2177e-04_rb,5.0878e-04_rb,6.6994e-04_rb,8.2945e-04_rb, &
     &9.8413e-04_rb,1.1125e-03_rb,1.3109e-03_rb/)
      kao(:, 5, 1, 9) = (/ &
     &3.0399e-05_rb,1.7548e-04_rb,4.1760e-04_rb,6.6731e-04_rb,8.8167e-04_rb,1.0941e-03_rb, &
     &1.3015e-03_rb,1.4854e-03_rb,1.7348e-03_rb/)
      kao(:, 1, 2, 9) = (/ &
     &1.1900e-04_rb,1.2836e-04_rb,1.4231e-04_rb,1.8014e-04_rb,2.2526e-04_rb,2.6329e-04_rb, &
     &2.2981e-04_rb,4.7447e-05_rb,3.9923e-04_rb/)
      kao(:, 2, 2, 9) = (/ &
     &1.1384e-04_rb,1.2703e-04_rb,1.6170e-04_rb,2.3947e-04_rb,3.1945e-04_rb,3.8888e-04_rb, &
     &4.4936e-04_rb,3.6063e-04_rb,5.9020e-04_rb/)
      kao(:, 3, 2, 9) = (/ &
     &9.6344e-05_rb,1.3540e-04_rb,2.0202e-04_rb,3.2063e-04_rb,4.4367e-04_rb,5.4566e-04_rb, &
     &6.4489e-04_rb,7.2550e-04_rb,8.3580e-04_rb/)
      kao(:, 4, 2, 9) = (/ &
     &7.7835e-05_rb,1.6099e-04_rb,2.5994e-04_rb,4.2610e-04_rb,6.0189e-04_rb,7.4414e-04_rb, &
     &8.8531e-04_rb,1.0210e-03_rb,1.1481e-03_rb/)
      kao(:, 5, 2, 9) = (/ &
     &6.0959e-05_rb,1.9625e-04_rb,3.3515e-04_rb,5.6029e-04_rb,7.9618e-04_rb,9.9092e-04_rb, &
     &1.1822e-03_rb,1.3661e-03_rb,1.5393e-03_rb/)
      kao(:, 1, 3, 9) = (/ &
     &3.4384e-04_rb,3.1901e-04_rb,2.9656e-04_rb,2.7161e-04_rb,2.4956e-04_rb,2.4575e-04_rb, &
     &2.4892e-04_rb,1.2874e-04_rb,2.5500e-04_rb/)
      kao(:, 2, 3, 9) = (/ &
     &3.2651e-04_rb,2.9987e-04_rb,2.8407e-04_rb,2.6822e-04_rb,2.8382e-04_rb,3.2698e-04_rb, &
     &3.7690e-04_rb,3.8984e-04_rb,3.8097e-04_rb/)
      kao(:, 3, 3, 9) = (/ &
     &3.1133e-04_rb,2.9825e-04_rb,2.8892e-04_rb,2.8983e-04_rb,3.4857e-04_rb,4.4151e-04_rb, &
     &5.3596e-04_rb,6.0662e-04_rb,5.5422e-04_rb/)
      kao(:, 4, 3, 9) = (/ &
     &2.6650e-04_rb,2.9032e-04_rb,3.1626e-04_rb,3.4521e-04_rb,4.5449e-04_rb,5.9386e-04_rb, &
     &7.4052e-04_rb,8.5104e-04_rb,7.8265e-04_rb/)
      kao(:, 5, 3, 9) = (/ &
     &2.2412e-04_rb,2.9039e-04_rb,3.7283e-04_rb,4.3223e-04_rb,5.9364e-04_rb,7.9041e-04_rb, &
     &9.9696e-04_rb,1.1553e-03_rb,1.0705e-03_rb/)
      kao(:, 1, 4, 9) = (/ &
     &1.0105e-03_rb,8.9198e-04_rb,7.7732e-04_rb,6.6567e-04_rb,5.5452e-04_rb,4.4086e-04_rb, &
     &3.2869e-04_rb,1.9418e-04_rb,1.7212e-04_rb/)
      kao(:, 2, 4, 9) = (/ &
     &9.2169e-04_rb,8.2421e-04_rb,7.2924e-04_rb,6.3857e-04_rb,5.4763e-04_rb,4.5198e-04_rb, &
     &3.7244e-04_rb,3.3779e-04_rb,2.7365e-04_rb/)
      kao(:, 3, 4, 9) = (/ &
     &9.1340e-04_rb,8.1831e-04_rb,7.2882e-04_rb,6.3523e-04_rb,5.3088e-04_rb,4.5658e-04_rb, &
     &4.4489e-04_rb,4.8577e-04_rb,4.2302e-04_rb/)
      kao(:, 4, 4, 9) = (/ &
     &8.9251e-04_rb,8.0269e-04_rb,7.3471e-04_rb,6.4402e-04_rb,5.5905e-04_rb,5.1386e-04_rb, &
     &5.6577e-04_rb,6.8086e-04_rb,6.2344e-04_rb/)
      kao(:, 5, 4, 9) = (/ &
     &7.9475e-04_rb,7.3797e-04_rb,7.1362e-04_rb,6.7001e-04_rb,6.3957e-04_rb,6.1730e-04_rb, &
     &7.4587e-04_rb,9.2826e-04_rb,8.7756e-04_rb/)
      kao(:, 1, 5, 9) = (/ &
     &2.6935e-03_rb,2.3593e-03_rb,2.0323e-03_rb,1.7047e-03_rb,1.3798e-03_rb,1.0522e-03_rb, &
     &7.2780e-04_rb,3.9954e-04_rb,1.6653e-04_rb/)
      kao(:, 2, 5, 9) = (/ &
     &2.6191e-03_rb,2.2983e-03_rb,1.9871e-03_rb,1.6780e-03_rb,1.3636e-03_rb,1.0535e-03_rb, &
     &7.4346e-04_rb,4.3038e-04_rb,2.6682e-04_rb/)
      kao(:, 3, 5, 9) = (/ &
     &2.3650e-03_rb,2.0866e-03_rb,1.8132e-03_rb,1.5443e-03_rb,1.2810e-03_rb,1.0093e-03_rb, &
     &7.2606e-04_rb,4.8378e-04_rb,4.1282e-04_rb/)
      kao(:, 4, 5, 9) = (/ &
     &2.4014e-03_rb,2.1168e-03_rb,1.8401e-03_rb,1.5793e-03_rb,1.3157e-03_rb,1.0190e-03_rb, &
     &7.3809e-04_rb,5.7085e-04_rb,6.1502e-04_rb/)
      kao(:, 5, 5, 9) = (/ &
     &2.3689e-03_rb,2.1027e-03_rb,1.8524e-03_rb,1.6218e-03_rb,1.3262e-03_rb,1.0531e-03_rb, &
     &8.0904e-04_rb,7.1684e-04_rb,8.8985e-04_rb/)
      kao(:, 1, 6, 9) = (/ &
     &6.9308e-03_rb,6.0665e-03_rb,5.2022e-03_rb,4.3390e-03_rb,3.4845e-03_rb,2.6255e-03_rb, &
     &1.7705e-03_rb,9.1297e-04_rb,1.4507e-04_rb/)
      kao(:, 2, 6, 9) = (/ &
     &6.7707e-03_rb,5.9271e-03_rb,5.0849e-03_rb,4.2542e-03_rb,3.4202e-03_rb,2.5899e-03_rb, &
     &1.7563e-03_rb,9.2722e-04_rb,2.4197e-04_rb/)
      kao(:, 3, 6, 9) = (/ &
     &6.6111e-03_rb,5.7877e-03_rb,4.9821e-03_rb,4.1734e-03_rb,3.3664e-03_rb,2.5641e-03_rb, &
     &1.7626e-03_rb,9.4198e-04_rb,3.8026e-04_rb/)
      kao(:, 4, 6, 9) = (/ &
     &6.0438e-03_rb,5.3003e-03_rb,4.5745e-03_rb,3.8441e-03_rb,3.1111e-03_rb,2.3922e-03_rb, &
     &1.6672e-03_rb,9.3430e-04_rb,5.5692e-04_rb/)
      kao(:, 5, 6, 9) = (/ &
     &6.1335e-03_rb,5.3879e-03_rb,4.6501e-03_rb,3.9141e-03_rb,3.1883e-03_rb,2.4602e-03_rb, &
     &1.7143e-03_rb,9.6080e-04_rb,7.9260e-04_rb/)
      kao(:, 1, 7, 9) = (/ &
     &1.9047e-02_rb,1.6668e-02_rb,1.4288e-02_rb,1.1909e-02_rb,9.5299e-03_rb,7.1507e-03_rb, &
     &4.7825e-03_rb,2.4097e-03_rb,1.1330e-04_rb/)
      kao(:, 2, 7, 9) = (/ &
     &1.8614e-02_rb,1.6290e-02_rb,1.3965e-02_rb,1.1641e-02_rb,9.3171e-03_rb,7.0086e-03_rb, &
     &4.6932e-03_rb,2.3827e-03_rb,1.9275e-04_rb/)
      kao(:, 3, 7, 9) = (/ &
     &1.8191e-02_rb,1.5921e-02_rb,1.3650e-02_rb,1.1382e-02_rb,9.1322e-03_rb,6.8769e-03_rb, &
     &4.6246e-03_rb,2.3704e-03_rb,3.0881e-04_rb/)
      kao(:, 4, 7, 9) = (/ &
     &1.7657e-02_rb,1.5455e-02_rb,1.3252e-02_rb,1.1078e-02_rb,8.8961e-03_rb,6.7169e-03_rb, &
     &4.5330e-03_rb,2.3676e-03_rb,4.7256e-04_rb/)
      kao(:, 5, 7, 9) = (/ &
     &1.6605e-02_rb,1.4537e-02_rb,1.2495e-02_rb,1.0448e-02_rb,8.3949e-03_rb,6.3348e-03_rb, &
     &4.3112e-03_rb,2.2623e-03_rb,7.0061e-04_rb/)
      kao(:, 1, 8, 9) = (/ &
     &6.6522e-02_rb,5.8208e-02_rb,4.9894e-02_rb,4.1580e-02_rb,3.3266e-02_rb,2.4952e-02_rb, &
     &1.6638e-02_rb,8.3237e-03_rb,8.8556e-05_rb/)
      kao(:, 2, 8, 9) = (/ &
     &6.5331e-02_rb,5.7166e-02_rb,4.9002e-02_rb,4.0837e-02_rb,3.2672e-02_rb,2.4508e-02_rb, &
     &1.6344e-02_rb,8.1975e-03_rb,1.5988e-04_rb/)
      kao(:, 3, 8, 9) = (/ &
     &6.4113e-02_rb,5.6102e-02_rb,4.8090e-02_rb,4.0078e-02_rb,3.2067e-02_rb,2.4056e-02_rb, &
     &1.6062e-02_rb,8.0735e-03_rb,2.6393e-04_rb/)
      kao(:, 4, 8, 9) = (/ &
     &6.2675e-02_rb,5.4844e-02_rb,4.7014e-02_rb,3.9184e-02_rb,3.1353e-02_rb,2.3542e-02_rb, &
     &1.5747e-02_rb,7.9493e-03_rb,4.0578e-04_rb/)
      kao(:, 5, 8, 9) = (/ &
     &6.0829e-02_rb,5.3229e-02_rb,4.5632e-02_rb,3.8033e-02_rb,3.0464e-02_rb,2.2925e-02_rb, &
     &1.5354e-02_rb,7.7739e-03_rb,6.0582e-04_rb/)
      kao(:, 1, 9, 9) = (/ &
     &4.4562e-01_rb,3.8992e-01_rb,3.3422e-01_rb,2.7852e-01_rb,2.2281e-01_rb,1.6711e-01_rb, &
     &1.1141e-01_rb,5.5708e-02_rb,6.4708e-05_rb/)
      kao(:, 2, 9, 9) = (/ &
     &4.3840e-01_rb,3.8360e-01_rb,3.2881e-01_rb,2.7401e-01_rb,2.1921e-01_rb,1.6441e-01_rb, &
     &1.0961e-01_rb,5.4811e-02_rb,1.2246e-04_rb/)
      kao(:, 3, 9, 9) = (/ &
     &4.3101e-01_rb,3.7714e-01_rb,3.2326e-01_rb,2.6939e-01_rb,2.1551e-01_rb,1.6164e-01_rb, &
     &1.0777e-01_rb,5.3892e-02_rb,2.1404e-04_rb/)
      kao(:, 4, 9, 9) = (/ &
     &4.2341e-01_rb,3.7048e-01_rb,3.1756e-01_rb,2.6464e-01_rb,2.1171e-01_rb,1.5879e-01_rb, &
     &1.0587e-01_rb,5.2950e-02_rb,3.4919e-04_rb/)
      kao(:, 5, 9, 9) = (/ &
     &4.1490e-01_rb,3.6305e-01_rb,3.1119e-01_rb,2.5933e-01_rb,2.0748e-01_rb,1.5561e-01_rb, &
     &1.0376e-01_rb,5.1933e-02_rb,5.3978e-04_rb/)
      kao(:, 1,10, 9) = (/ &
     &3.5739e+00_rb,3.1272e+00_rb,2.6804e+00_rb,2.2337e+00_rb,1.7870e+00_rb,1.3402e+00_rb, &
     &8.9349e-01_rb,4.4675e-01_rb,3.6923e-05_rb/)
      kao(:, 2,10, 9) = (/ &
     &3.5173e+00_rb,3.0777e+00_rb,2.6381e+00_rb,2.1983e+00_rb,1.7587e+00_rb,1.3190e+00_rb, &
     &8.7936e-01_rb,4.3968e-01_rb,7.4734e-05_rb/)
      kao(:, 3,10, 9) = (/ &
     &3.4601e+00_rb,3.0276e+00_rb,2.5951e+00_rb,2.1626e+00_rb,1.7301e+00_rb,1.2976e+00_rb, &
     &8.6506e-01_rb,4.3253e-01_rb,1.2645e-04_rb/)
      kao(:, 4,10, 9) = (/ &
     &3.4013e+00_rb,2.9762e+00_rb,2.5510e+00_rb,2.1258e+00_rb,1.7007e+00_rb,1.2755e+00_rb, &
     &8.5034e-01_rb,4.2518e-01_rb,2.0434e-04_rb/)
      kao(:, 5,10, 9) = (/ &
     &3.3408e+00_rb,2.9231e+00_rb,2.5056e+00_rb,2.0880e+00_rb,1.6704e+00_rb,1.2528e+00_rb, &
     &8.3521e-01_rb,4.1763e-01_rb,3.0930e-04_rb/)
      kao(:, 1,11, 9) = (/ &
     &9.5423e+00_rb,8.3495e+00_rb,7.1567e+00_rb,5.9639e+00_rb,4.7712e+00_rb,3.5784e+00_rb, &
     &2.3856e+00_rb,1.1928e+00_rb,3.3494e-05_rb/)
      kao(:, 2,11, 9) = (/ &
     &9.3929e+00_rb,8.2188e+00_rb,7.0447e+00_rb,5.8706e+00_rb,4.6965e+00_rb,3.5224e+00_rb, &
     &2.3483e+00_rb,1.1741e+00_rb,6.4875e-05_rb/)
      kao(:, 3,11, 9) = (/ &
     &9.2404e+00_rb,8.0853e+00_rb,6.9303e+00_rb,5.7752e+00_rb,4.6202e+00_rb,3.4651e+00_rb, &
     &2.3101e+00_rb,1.1551e+00_rb,1.1378e-04_rb/)
      kao(:, 4,11, 9) = (/ &
     &9.0863e+00_rb,7.9506e+00_rb,6.8148e+00_rb,5.6790e+00_rb,4.5432e+00_rb,3.4074e+00_rb, &
     &2.2716e+00_rb,1.1358e+00_rb,1.8195e-04_rb/)
      kao(:, 5,11, 9) = (/ &
     &8.9334e+00_rb,7.8167e+00_rb,6.7001e+00_rb,5.5834e+00_rb,4.4668e+00_rb,3.3501e+00_rb, &
     &2.2334e+00_rb,1.1167e+00_rb,2.6874e-04_rb/)
      kao(:, 1,12, 9) = (/ &
     &1.6018e+01_rb,1.4016e+01_rb,1.2013e+01_rb,1.0011e+01_rb,8.0089e+00_rb,6.0065e+00_rb, &
     &4.0043e+00_rb,2.0022e+00_rb,3.3227e-05_rb/)
      kao(:, 2,12, 9) = (/ &
     &1.5778e+01_rb,1.3806e+01_rb,1.1834e+01_rb,9.8614e+00_rb,7.8892e+00_rb,5.9169e+00_rb, &
     &3.9446e+00_rb,1.9723e+00_rb,6.4843e-05_rb/)
      kao(:, 3,12, 9) = (/ &
     &1.5536e+01_rb,1.3594e+01_rb,1.1652e+01_rb,9.7100e+00_rb,7.7679e+00_rb,5.8260e+00_rb, &
     &3.8840e+00_rb,1.9420e+00_rb,1.1290e-04_rb/)
      kao(:, 4,12, 9) = (/ &
     &1.5292e+01_rb,1.3380e+01_rb,1.1469e+01_rb,9.5575e+00_rb,7.6460e+00_rb,5.7344e+00_rb, &
     &3.8230e+00_rb,1.9115e+00_rb,1.7636e-04_rb/)
      kao(:, 5,12, 9) = (/ &
     &1.5044e+01_rb,1.3163e+01_rb,1.1283e+01_rb,9.4022e+00_rb,7.5218e+00_rb,5.6414e+00_rb, &
     &3.7609e+00_rb,1.8805e+00_rb,2.6723e-04_rb/)
      kao(:, 1,13, 9) = (/ &
     &2.1597e+01_rb,1.8897e+01_rb,1.6197e+01_rb,1.3498e+01_rb,1.0798e+01_rb,8.0986e+00_rb, &
     &5.3991e+00_rb,2.6995e+00_rb,3.8904e-05_rb/)
      kao(:, 2,13, 9) = (/ &
     &2.1295e+01_rb,1.8633e+01_rb,1.5971e+01_rb,1.3309e+01_rb,1.0647e+01_rb,7.9855e+00_rb, &
     &5.3237e+00_rb,2.6618e+00_rb,7.5857e-05_rb/)
      kao(:, 3,13, 9) = (/ &
     &2.0989e+01_rb,1.8366e+01_rb,1.5742e+01_rb,1.3118e+01_rb,1.0494e+01_rb,7.8710e+00_rb, &
     &5.2472e+00_rb,2.6236e+00_rb,1.3015e-04_rb/)
      kao(:, 4,13, 9) = (/ &
     &2.0668e+01_rb,1.8085e+01_rb,1.5501e+01_rb,1.2918e+01_rb,1.0334e+01_rb,7.7506e+00_rb, &
     &5.1670e+00_rb,2.5835e+00_rb,2.0228e-04_rb/)
      kao(:, 5,13, 9) = (/ &
     &2.0333e+01_rb,1.7791e+01_rb,1.5249e+01_rb,1.2708e+01_rb,1.0166e+01_rb,7.6247e+00_rb, &
     &5.0831e+00_rb,2.5416e+00_rb,2.9840e-04_rb/)
      kao(:, 1, 1,10) = (/ &
     &5.6592e-05_rb,1.3380e-04_rb,2.8259e-04_rb,4.0725e-04_rb,5.2722e-04_rb,6.3771e-04_rb, &
     &7.3618e-04_rb,6.8809e-05_rb,1.0230e-03_rb/)
      kao(:, 2, 1,10) = (/ &
     &3.4000e-05_rb,1.7762e-04_rb,3.9282e-04_rb,5.7189e-04_rb,7.4744e-04_rb,9.1578e-04_rb, &
     &1.0739e-03_rb,1.1530e-03_rb,1.4652e-03_rb/)
      kao(:, 3, 1,10) = (/ &
     &3.1822e-05_rb,2.4313e-04_rb,5.2738e-04_rb,7.7595e-04_rb,1.0231e-03_rb,1.2640e-03_rb, &
     &1.5000e-03_rb,1.6967e-03_rb,2.0181e-03_rb/)
      kao(:, 4, 1,10) = (/ &
     &3.1887e-05_rb,3.1730e-04_rb,7.0142e-04_rb,1.0376e-03_rb,1.3716e-03_rb,1.7032e-03_rb, &
     &2.0256e-03_rb,2.3355e-03_rb,2.7159e-03_rb/)
      kao(:, 5, 1,10) = (/ &
     &3.2974e-05_rb,4.0500e-04_rb,9.2096e-04_rb,1.3701e-03_rb,1.8162e-03_rb,2.2594e-03_rb, &
     &2.6867e-03_rb,3.0973e-03_rb,3.6043e-03_rb/)
      kao(:, 1, 2,10) = (/ &
     &1.2846e-04_rb,1.5522e-04_rb,2.7655e-04_rb,3.8379e-04_rb,4.9079e-04_rb,5.9529e-04_rb, &
     &6.9141e-04_rb,5.6796e-04_rb,9.2767e-04_rb/)
      kao(:, 2, 2,10) = (/ &
     &6.5990e-05_rb,1.6552e-04_rb,3.7419e-04_rb,5.3648e-04_rb,6.9689e-04_rb,8.5375e-04_rb, &
     &1.0031e-03_rb,1.1095e-03_rb,1.3393e-03_rb/)
      kao(:, 3, 2,10) = (/ &
     &5.6435e-05_rb,1.5534e-04_rb,5.0363e-04_rb,7.3034e-04_rb,9.5369e-04_rb,1.1711e-03_rb, &
     &1.3827e-03_rb,1.5608e-03_rb,1.8461e-03_rb/)
      kao(:, 4, 2,10) = (/ &
     &5.4174e-05_rb,1.7810e-04_rb,6.6905e-04_rb,9.7686e-04_rb,1.2783e-03_rb,1.5737e-03_rb, &
     &1.8611e-03_rb,2.1178e-03_rb,2.4942e-03_rb/)
      kao(:, 5, 2,10) = (/ &
     &5.2370e-05_rb,1.9597e-04_rb,8.6044e-04_rb,1.2706e-03_rb,1.6729e-03_rb,2.0703e-03_rb, &
     &2.4603e-03_rb,2.8342e-03_rb,3.2918e-03_rb/)
      kao(:, 1, 3,10) = (/ &
     &4.4469e-04_rb,4.0346e-04_rb,3.7124e-04_rb,3.9072e-04_rb,4.6265e-04_rb,5.3518e-04_rb, &
     &6.0670e-04_rb,6.5723e-04_rb,7.6901e-04_rb/)
      kao(:, 2, 3,10) = (/ &
     &2.7821e-04_rb,2.8769e-04_rb,3.4636e-04_rb,4.5772e-04_rb,6.3733e-04_rb,7.5575e-04_rb, &
     &8.7290e-04_rb,9.7779e-04_rb,1.1248e-03_rb/)
      kao(:, 3, 3,10) = (/ &
     &1.6712e-04_rb,2.0135e-04_rb,3.1664e-04_rb,6.0259e-04_rb,8.6177e-04_rb,1.0373e-03_rb, &
     &1.2106e-03_rb,1.3752e-03_rb,1.5723e-03_rb/)
      kao(:, 4, 3,10) = (/ &
     &1.5303e-04_rb,2.3452e-04_rb,3.6629e-04_rb,7.9270e-04_rb,1.1535e-03_rb,1.4036e-03_rb, &
     &1.6498e-03_rb,1.8820e-03_rb,2.1596e-03_rb/)
      kao(:, 5, 3,10) = (/ &
     &1.4546e-04_rb,2.9515e-04_rb,4.4927e-04_rb,1.0579e-03_rb,1.5163e-03_rb,1.8612e-03_rb, &
     &2.1985e-03_rb,2.5140e-03_rb,2.9016e-03_rb/)
      kao(:, 1, 4,10) = (/ &
     &1.2138e-03_rb,1.0643e-03_rb,9.4070e-04_rb,8.1504e-04_rb,7.0358e-04_rb,5.9316e-04_rb, &
     &5.5104e-04_rb,5.6438e-04_rb,3.2078e-04_rb/)
      kao(:, 2, 4,10) = (/ &
     &1.1514e-03_rb,1.0125e-03_rb,9.4071e-04_rb,8.3600e-04_rb,7.4220e-04_rb,6.5907e-04_rb, &
     &7.7143e-04_rb,8.3267e-04_rb,4.7891e-04_rb/)
      kao(:, 3, 4,10) = (/ &
     &8.3276e-04_rb,7.0376e-04_rb,7.1208e-04_rb,6.8052e-04_rb,7.6194e-04_rb,8.9019e-04_rb, &
     &1.0682e-03_rb,1.1778e-03_rb,6.1825e-04_rb/)
      kao(:, 4, 4,10) = (/ &
     &6.2678e-04_rb,5.3178e-04_rb,5.1712e-04_rb,6.2090e-04_rb,7.5690e-04_rb,1.1872e-03_rb, &
     &1.4563e-03_rb,1.6302e-03_rb,8.5537e-04_rb/)
      kao(:, 5, 4,10) = (/ &
     &4.2987e-04_rb,5.2033e-04_rb,6.3756e-04_rb,7.6564e-04_rb,7.8218e-04_rb,1.6224e-03_rb, &
     &1.9482e-03_rb,2.2056e-03_rb,1.2455e-03_rb/)
      kao(:, 1, 5,10) = (/ &
     &3.3420e-03_rb,2.9247e-03_rb,2.5126e-03_rb,2.0946e-03_rb,1.6924e-03_rb,1.3030e-03_rb, &
     &9.3410e-04_rb,5.8986e-04_rb,3.1969e-04_rb/)
      kao(:, 2, 5,10) = (/ &
     &3.1398e-03_rb,2.7488e-03_rb,2.3818e-03_rb,2.0259e-03_rb,1.6962e-03_rb,1.3524e-03_rb, &
     &9.7217e-04_rb,7.2894e-04_rb,5.2320e-04_rb/)
      kao(:, 3, 5,10) = (/ &
     &3.0330e-03_rb,2.6759e-03_rb,2.3370e-03_rb,2.0014e-03_rb,1.6490e-03_rb,1.4032e-03_rb, &
     &1.0593e-03_rb,1.0067e-03_rb,7.5418e-04_rb/)
      kao(:, 4, 5,10) = (/ &
     &2.5775e-03_rb,2.3167e-03_rb,2.0477e-03_rb,1.6916e-03_rb,1.3663e-03_rb,1.1599e-03_rb, &
     &1.0841e-03_rb,1.3957e-03_rb,9.6520e-04_rb/)
      kao(:, 5, 5,10) = (/ &
     &2.1866e-03_rb,2.0017e-03_rb,1.5507e-03_rb,1.2425e-03_rb,1.2248e-03_rb,1.1086e-03_rb, &
     &1.2483e-03_rb,1.9173e-03_rb,1.1788e-03_rb/)
      kao(:, 1, 6,10) = (/ &
     &8.8838e-03_rb,7.7745e-03_rb,6.6646e-03_rb,5.5555e-03_rb,4.4614e-03_rb,3.3623e-03_rb, &
     &2.2796e-03_rb,1.1799e-03_rb,1.7827e-04_rb/)
      kao(:, 2, 6,10) = (/ &
     &8.6572e-03_rb,7.5770e-03_rb,6.4955e-03_rb,5.4321e-03_rb,4.3605e-03_rb,3.3104e-03_rb, &
     &2.2427e-03_rb,1.2382e-03_rb,2.5800e-04_rb/)
      kao(:, 3, 6,10) = (/ &
     &8.3335e-03_rb,7.2931e-03_rb,6.2759e-03_rb,5.2475e-03_rb,4.2339e-03_rb,3.2059e-03_rb, &
     &2.2047e-03_rb,1.3240e-03_rb,4.2156e-04_rb/)
      kao(:, 4, 6,10) = (/ &
     &7.5549e-03_rb,6.6060e-03_rb,5.7369e-03_rb,4.8380e-03_rb,4.0096e-03_rb,3.1204e-03_rb, &
     &2.3001e-03_rb,1.3603e-03_rb,7.8178e-04_rb/)
      kao(:, 5, 6,10) = (/ &
     &6.8684e-03_rb,6.0886e-03_rb,5.1989e-03_rb,4.4067e-03_rb,3.6681e-03_rb,2.8099e-03_rb, &
     &1.8970e-03_rb,1.4204e-03_rb,1.3480e-03_rb/)
      kao(:, 1, 7,10) = (/ &
     &2.4586e-02_rb,2.1513e-02_rb,1.8440e-02_rb,1.5369e-02_rb,1.2296e-02_rb,9.2230e-03_rb, &
     &6.1633e-03_rb,3.1137e-03_rb,1.3933e-04_rb/)
      kao(:, 2, 7,10) = (/ &
     &2.4170e-02_rb,2.1150e-02_rb,1.8129e-02_rb,1.5109e-02_rb,1.2088e-02_rb,9.0886e-03_rb, &
     &6.0878e-03_rb,3.1114e-03_rb,2.3190e-04_rb/)
      kao(:, 3, 7,10) = (/ &
     &2.3662e-02_rb,2.0705e-02_rb,1.7748e-02_rb,1.4791e-02_rb,1.1861e-02_rb,8.9271e-03_rb, &
     &6.0215e-03_rb,3.0844e-03_rb,3.6298e-04_rb/)
      kao(:, 4, 7,10) = (/ &
     &2.2469e-02_rb,1.9660e-02_rb,1.6853e-02_rb,1.4086e-02_rb,1.1306e-02_rb,8.5420e-03_rb, &
     &5.7631e-03_rb,3.0626e-03_rb,5.4029e-04_rb/)
      kao(:, 5, 7,10) = (/ &
     &1.9809e-02_rb,1.7349e-02_rb,1.4886e-02_rb,1.2460e-02_rb,1.0092e-02_rb,7.8522e-03_rb, &
     &5.3378e-03_rb,3.0999e-03_rb,7.7423e-04_rb/)
      kao(:, 1, 8,10) = (/ &
     &8.6508e-02_rb,7.5696e-02_rb,6.4882e-02_rb,5.4068e-02_rb,4.3255e-02_rb,3.2443e-02_rb, &
     &2.1630e-02_rb,1.0817e-02_rb,1.6191e-04_rb/)
      kao(:, 2, 8,10) = (/ &
     &8.4528e-02_rb,7.3960e-02_rb,6.3397e-02_rb,5.2832e-02_rb,4.2266e-02_rb,3.1701e-02_rb, &
     &2.1136e-02_rb,1.0597e-02_rb,3.0221e-04_rb/)
      kao(:, 3, 8,10) = (/ &
     &8.2999e-02_rb,7.2627e-02_rb,6.2253e-02_rb,5.1878e-02_rb,4.1506e-02_rb,3.1132e-02_rb, &
     &2.0757e-02_rb,1.0444e-02_rb,5.5391e-04_rb/)
      kao(:, 4, 8,10) = (/ &
     &8.1486e-02_rb,7.1301e-02_rb,6.1120e-02_rb,5.0928e-02_rb,4.0746e-02_rb,3.0564e-02_rb, &
     &2.0452e-02_rb,1.0346e-02_rb,1.0152e-03_rb/)
      kao(:, 5, 8,10) = (/ &
     &7.7129e-02_rb,6.7490e-02_rb,5.7855e-02_rb,4.8214e-02_rb,3.8581e-02_rb,2.9014e-02_rb, &
     &1.9443e-02_rb,9.9515e-03_rb,1.5121e-03_rb/)
      kao(:, 1, 9,10) = (/ &
     &5.8674e-01_rb,5.1340e-01_rb,4.4007e-01_rb,3.6671e-01_rb,2.9337e-01_rb,2.2003e-01_rb, &
     &1.4669e-01_rb,7.3347e-02_rb,1.3954e-04_rb/)
      kao(:, 2, 9,10) = (/ &
     &5.7350e-01_rb,5.0184e-01_rb,4.3015e-01_rb,3.5845e-01_rb,2.8676e-01_rb,2.1507e-01_rb, &
     &1.4339e-01_rb,7.1694e-02_rb,3.0044e-04_rb/)
      kao(:, 3, 9,10) = (/ &
     &5.6021e-01_rb,4.9017e-01_rb,4.2014e-01_rb,3.5012e-01_rb,2.8010e-01_rb,2.1008e-01_rb, &
     &1.4005e-01_rb,7.0031e-02_rb,5.5636e-04_rb/)
      kao(:, 4, 9,10) = (/ &
     &5.4802e-01_rb,4.7953e-01_rb,4.1105e-01_rb,3.4251e-01_rb,2.7403e-01_rb,2.0552e-01_rb, &
     &1.3702e-01_rb,6.8512e-02_rb,9.3561e-04_rb/)
      kao(:, 5, 9,10) = (/ &
     &5.3709e-01_rb,4.6991e-01_rb,4.0280e-01_rb,3.3568e-01_rb,2.6852e-01_rb,2.0142e-01_rb, &
     &1.3428e-01_rb,6.7143e-02_rb,1.4485e-03_rb/)
      kao(:, 1,10,10) = (/ &
     &4.7861e+00_rb,4.1878e+00_rb,3.5896e+00_rb,2.9913e+00_rb,2.3930e+00_rb,1.7948e+00_rb, &
     &1.1965e+00_rb,5.9827e-01_rb,3.2704e-06_rb/)
      kao(:, 2,10,10) = (/ &
     &4.6903e+00_rb,4.1040e+00_rb,3.5178e+00_rb,2.9315e+00_rb,2.3452e+00_rb,1.7589e+00_rb, &
     &1.1726e+00_rb,5.8632e-01_rb,5.9185e-06_rb/)
      kao(:, 3,10,10) = (/ &
     &4.5984e+00_rb,4.0237e+00_rb,3.4487e+00_rb,2.8740e+00_rb,2.2992e+00_rb,1.7244e+00_rb, &
     &1.1496e+00_rb,5.7479e-01_rb,9.9252e-06_rb/)
      kao(:, 4,10,10) = (/ &
     &4.5067e+00_rb,3.9434e+00_rb,3.3802e+00_rb,2.8168e+00_rb,2.2535e+00_rb,1.6901e+00_rb, &
     &1.1267e+00_rb,5.6337e-01_rb,1.5639e-05_rb/)
      kao(:, 5,10,10) = (/ &
     &4.4320e+00_rb,3.8780e+00_rb,3.3239e+00_rb,2.7699e+00_rb,2.2160e+00_rb,1.6620e+00_rb, &
     &1.1080e+00_rb,5.5401e-01_rb,2.3260e-05_rb/)
      kao(:, 1,11,10) = (/ &
     &1.3118e+01_rb,1.1478e+01_rb,9.8381e+00_rb,8.1983e+00_rb,6.5588e+00_rb,4.9192e+00_rb, &
     &3.2794e+00_rb,1.6397e+00_rb,2.2734e-06_rb/)
      kao(:, 2,11,10) = (/ &
     &1.2878e+01_rb,1.1268e+01_rb,9.6587e+00_rb,8.0488e+00_rb,6.4391e+00_rb,4.8293e+00_rb, &
     &3.2196e+00_rb,1.6098e+00_rb,4.2460e-06_rb/)
      kao(:, 3,11,10) = (/ &
     &1.2677e+01_rb,1.1092e+01_rb,9.5076e+00_rb,7.9230e+00_rb,6.3383e+00_rb,4.7539e+00_rb, &
     &3.1692e+00_rb,1.5846e+00_rb,7.1509e-06_rb/)
      kao(:, 4,11,10) = (/ &
     &1.2452e+01_rb,1.0896e+01_rb,9.3395e+00_rb,7.7831e+00_rb,6.2263e+00_rb,4.6699e+00_rb, &
     &3.1133e+00_rb,1.5566e+00_rb,1.2244e-05_rb/)
      kao(:, 5,11,10) = (/ &
     &1.2241e+01_rb,1.0711e+01_rb,9.1810e+00_rb,7.6509e+00_rb,6.1207e+00_rb,4.5903e+00_rb, &
     &3.0604e+00_rb,1.5302e+00_rb,2.0967e-05_rb/)
      kao(:, 1,12,10) = (/ &
     &2.2864e+01_rb,2.0005e+01_rb,1.7148e+01_rb,1.4290e+01_rb,1.1432e+01_rb,8.5738e+00_rb, &
     &5.7157e+00_rb,2.8580e+00_rb,1.8184e-06_rb/)
      kao(:, 2,12,10) = (/ &
     &2.2487e+01_rb,1.9676e+01_rb,1.6864e+01_rb,1.4054e+01_rb,1.1243e+01_rb,8.4321e+00_rb, &
     &5.6215e+00_rb,2.8107e+00_rb,3.3852e-06_rb/)
      kao(:, 3,12,10) = (/ &
     &2.2126e+01_rb,1.9360e+01_rb,1.6594e+01_rb,1.3829e+01_rb,1.1063e+01_rb,8.2969e+00_rb, &
     &5.5315e+00_rb,2.7657e+00_rb,6.4086e-06_rb/)
      kao(:, 4,12,10) = (/ &
     &2.1772e+01_rb,1.9050e+01_rb,1.6328e+01_rb,1.3607e+01_rb,1.0886e+01_rb,8.1644e+00_rb, &
     &5.4429e+00_rb,2.7215e+00_rb,1.1402e-05_rb/)
      kao(:, 5,12,10) = (/ &
     &2.1416e+01_rb,1.8740e+01_rb,1.6063e+01_rb,1.3386e+01_rb,1.0708e+01_rb,8.0313e+00_rb, &
     &5.3540e+00_rb,2.6771e+00_rb,1.6645e-05_rb/)
      kao(:, 1,13,10) = (/ &
     &3.2272e+01_rb,2.8237e+01_rb,2.4203e+01_rb,2.0170e+01_rb,1.6136e+01_rb,1.2101e+01_rb, &
     &8.0677e+00_rb,4.0338e+00_rb,1.4317e-06_rb/)
      kao(:, 2,13,10) = (/ &
     &3.1783e+01_rb,2.7810e+01_rb,2.3837e+01_rb,1.9864e+01_rb,1.5891e+01_rb,1.1919e+01_rb, &
     &7.9455e+00_rb,3.9728e+00_rb,3.0046e-06_rb/)
      kao(:, 3,13,10) = (/ &
     &3.1267e+01_rb,2.7359e+01_rb,2.3450e+01_rb,1.9542e+01_rb,1.5633e+01_rb,1.1725e+01_rb, &
     &7.8169e+00_rb,3.9083e+00_rb,5.7473e-06_rb/)
      kao(:, 4,13,10) = (/ &
     &3.0805e+01_rb,2.6955e+01_rb,2.3103e+01_rb,1.9253e+01_rb,1.5402e+01_rb,1.1552e+01_rb, &
     &7.7014e+00_rb,3.8506e+00_rb,8.5901e-06_rb/)
      kao(:, 5,13,10) = (/ &
     &3.0347e+01_rb,2.6554e+01_rb,2.2760e+01_rb,1.8967e+01_rb,1.5174e+01_rb,1.1380e+01_rb, &
     &7.5867e+00_rb,3.7933e+00_rb,1.2367e-05_rb/)
      kao(:, 1, 1,11) = (/ &
     &4.7098e-05_rb,1.8036e-04_rb,3.4819e-04_rb,5.0638e-04_rb,6.6181e-04_rb,8.1065e-04_rb, &
     &9.3902e-04_rb,7.7167e-04_rb,1.3022e-03_rb/)
      kao(:, 2, 1,11) = (/ &
     &3.0033e-05_rb,2.6523e-04_rb,5.0795e-04_rb,7.4803e-04_rb,9.8321e-04_rb,1.2080e-03_rb, &
     &1.4174e-03_rb,1.6176e-03_rb,1.9428e-03_rb/)
      kao(:, 3, 1,11) = (/ &
     &3.0544e-05_rb,3.7128e-04_rb,7.2058e-04_rb,1.0671e-03_rb,1.4084e-03_rb,1.7391e-03_rb, &
     &2.0379e-03_rb,2.3032e-03_rb,2.7938e-03_rb/)
      kao(:, 4, 1,11) = (/ &
     &3.0472e-05_rb,4.9757e-04_rb,9.8168e-04_rb,1.4606e-03_rb,1.9374e-03_rb,2.4057e-03_rb, &
     &2.8449e-03_rb,3.1636e-03_rb,3.8517e-03_rb/)
      kao(:, 5, 1,11) = (/ &
     &2.4091e-05_rb,6.5444e-04_rb,1.2934e-03_rb,1.9295e-03_rb,2.5676e-03_rb,3.2003e-03_rb, &
     &3.8144e-03_rb,4.2439e-03_rb,5.1123e-03_rb/)
      kao(:, 1, 2,11) = (/ &
     &1.3816e-04_rb,1.6153e-04_rb,3.3849e-04_rb,4.7695e-04_rb,6.1250e-04_rb,7.4207e-04_rb, &
     &8.5768e-04_rb,9.1861e-04_rb,1.1708e-03_rb/)
      kao(:, 2, 2,11) = (/ &
     &6.1066e-05_rb,1.7974e-04_rb,4.7651e-04_rb,6.8520e-04_rb,8.9139e-04_rb,1.0930e-03_rb, &
     &1.2848e-03_rb,1.4278e-03_rb,1.7346e-03_rb/)
      kao(:, 3, 2,11) = (/ &
     &5.7831e-05_rb,2.0028e-04_rb,6.5612e-04_rb,9.5859e-04_rb,1.2605e-03_rb,1.5625e-03_rb, &
     &1.8554e-03_rb,2.0988e-03_rb,2.4798e-03_rb/)
      kao(:, 4, 2,11) = (/ &
     &5.4516e-05_rb,2.3064e-04_rb,8.8062e-04_rb,1.3017e-03_rb,1.7223e-03_rb,2.1436e-03_rb, &
     &2.5645e-03_rb,2.9565e-03_rb,3.4038e-03_rb/)
      kao(:, 5, 2,11) = (/ &
     &5.5309e-05_rb,3.0448e-04_rb,1.1705e-03_rb,1.7340e-03_rb,2.2966e-03_rb,2.8601e-03_rb, &
     &3.4280e-03_rb,3.9900e-03_rb,4.5458e-03_rb/)
      kao(:, 1, 3,11) = (/ &
     &4.5525e-04_rb,4.1659e-04_rb,4.0041e-04_rb,4.3594e-04_rb,5.6841e-04_rb,6.6824e-04_rb, &
     &7.6559e-04_rb,8.3275e-04_rb,9.8673e-04_rb/)
      kao(:, 2, 3,11) = (/ &
     &2.7564e-04_rb,3.1143e-04_rb,4.0133e-04_rb,6.4539e-04_rb,8.1010e-04_rb,9.7483e-04_rb, &
     &1.1326e-03_rb,1.2606e-03_rb,1.4783e-03_rb/)
      kao(:, 3, 3,11) = (/ &
     &1.5755e-04_rb,2.0262e-04_rb,4.4233e-04_rb,8.8058e-04_rb,1.1240e-03_rb,1.3655e-03_rb, &
     &1.6023e-03_rb,1.8183e-03_rb,2.1060e-03_rb/)
      kao(:, 4, 3,11) = (/ &
     &1.5225e-04_rb,2.1360e-04_rb,4.5750e-04_rb,1.1722e-03_rb,1.5129e-03_rb,1.8525e-03_rb, &
     &2.1885e-03_rb,2.5141e-03_rb,2.8848e-03_rb/)
      kao(:, 5, 3,11) = (/ &
     &1.4157e-04_rb,2.5829e-04_rb,4.6589e-04_rb,1.5429e-03_rb,2.0085e-03_rb,2.4737e-03_rb, &
     &2.9344e-03_rb,3.3883e-03_rb,3.8756e-03_rb/)
      kao(:, 1, 4,11) = (/ &
     &1.2968e-03_rb,1.1627e-03_rb,1.0295e-03_rb,8.8852e-04_rb,7.5100e-04_rb,6.8878e-04_rb, &
     &7.0217e-04_rb,7.2855e-04_rb,6.6909e-04_rb/)
      kao(:, 2, 4,11) = (/ &
     &1.2901e-03_rb,1.1618e-03_rb,9.9278e-04_rb,8.6686e-04_rb,8.2710e-04_rb,9.1628e-04_rb, &
     &1.0053e-03_rb,1.0921e-03_rb,9.3719e-04_rb/)
      kao(:, 3, 4,11) = (/ &
     &8.4696e-04_rb,8.0992e-04_rb,6.6835e-04_rb,7.5797e-04_rb,8.8170e-04_rb,1.2538e-03_rb, &
     &1.4194e-03_rb,1.5777e-03_rb,1.2930e-03_rb/)
      kao(:, 4, 4,11) = (/ &
     &4.0946e-04_rb,5.3575e-04_rb,5.3156e-04_rb,6.5607e-04_rb,1.0924e-03_rb,1.6886e-03_rb, &
     &1.9410e-03_rb,2.1799e-03_rb,1.6795e-03_rb/)
      kao(:, 5, 4,11) = (/ &
     &4.1161e-04_rb,5.7102e-04_rb,5.8841e-04_rb,6.9122e-04_rb,1.4036e-03_rb,2.2467e-03_rb, &
     &2.6025e-03_rb,2.9419e-03_rb,2.1668e-03_rb/)
      kao(:, 1, 5,11) = (/ &
     &3.5249e-03_rb,3.0877e-03_rb,2.6707e-03_rb,2.2562e-03_rb,1.8574e-03_rb,1.4440e-03_rb, &
     &1.0135e-03_rb,7.0090e-04_rb,2.4879e-04_rb/)
      kao(:, 2, 5,11) = (/ &
     &3.4987e-03_rb,3.0660e-03_rb,2.6585e-03_rb,2.2616e-03_rb,1.8230e-03_rb,1.3947e-03_rb, &
     &1.1338e-03_rb,9.8875e-04_rb,4.4035e-04_rb/)
      kao(:, 3, 5,11) = (/ &
     &3.2338e-03_rb,2.8640e-03_rb,2.5128e-03_rb,2.2224e-03_rb,1.8369e-03_rb,1.4590e-03_rb, &
     &1.3054e-03_rb,1.4040e-03_rb,6.0764e-04_rb/)
      kao(:, 4, 5,11) = (/ &
     &2.1959e-03_rb,1.9312e-03_rb,1.7650e-03_rb,1.5181e-03_rb,1.3319e-03_rb,1.3218e-03_rb, &
     &1.7587e-03_rb,1.9394e-03_rb,9.4777e-04_rb/)
      kao(:, 5, 5,11) = (/ &
     &1.2468e-03_rb,1.0861e-03_rb,1.1609e-03_rb,1.1515e-03_rb,1.2470e-03_rb,1.3220e-03_rb, &
     &2.3659e-03_rb,2.6308e-03_rb,1.2356e-03_rb/)
      kao(:, 1, 6,11) = (/ &
     &9.5223e-03_rb,8.3335e-03_rb,7.1442e-03_rb,5.9544e-03_rb,4.7779e-03_rb,3.5984e-03_rb, &
     &2.4247e-03_rb,1.2737e-03_rb,1.4692e-04_rb/)
      kao(:, 2, 6,11) = (/ &
     &9.2485e-03_rb,8.0943e-03_rb,6.9406e-03_rb,5.8161e-03_rb,4.6851e-03_rb,3.5677e-03_rb, &
     &2.4748e-03_rb,1.3469e-03_rb,2.5178e-04_rb/)
      kao(:, 3, 6,11) = (/ &
     &8.7789e-03_rb,7.6827e-03_rb,6.6150e-03_rb,5.5437e-03_rb,4.4673e-03_rb,3.4787e-03_rb, &
     &2.4528e-03_rb,1.4847e-03_rb,3.7920e-04_rb/)
      kao(:, 4, 6,11) = (/ &
     &8.2164e-03_rb,7.2018e-03_rb,6.2465e-03_rb,5.3375e-03_rb,4.3691e-03_rb,3.4064e-03_rb, &
     &2.4151e-03_rb,1.7544e-03_rb,5.7438e-04_rb/)
      kao(:, 5, 6,11) = (/ &
     &6.8576e-03_rb,6.0356e-03_rb,5.1755e-03_rb,4.2553e-03_rb,3.2890e-03_rb,2.6043e-03_rb, &
     &1.9585e-03_rb,2.3328e-03_rb,8.0859e-04_rb/)
      kao(:, 1, 7,11) = (/ &
     &2.6634e-02_rb,2.3306e-02_rb,1.9979e-02_rb,1.6651e-02_rb,1.3321e-02_rb,9.9924e-03_rb, &
     &6.6684e-03_rb,3.3709e-03_rb,1.8920e-04_rb/)
      kao(:, 2, 7,11) = (/ &
     &2.5938e-02_rb,2.2696e-02_rb,1.9456e-02_rb,1.6215e-02_rb,1.2974e-02_rb,9.7488e-03_rb, &
     &6.5437e-03_rb,3.3574e-03_rb,3.7520e-04_rb/)
      kao(:, 3, 7,11) = (/ &
     &2.5453e-02_rb,2.2274e-02_rb,1.9096e-02_rb,1.5918e-02_rb,1.2777e-02_rb,9.6455e-03_rb, &
     &6.5081e-03_rb,3.3802e-03_rb,5.3158e-04_rb/)
      kao(:, 4, 7,11) = (/ &
     &2.4271e-02_rb,2.1241e-02_rb,1.8214e-02_rb,1.5233e-02_rb,1.2290e-02_rb,9.3599e-03_rb, &
     &6.4912e-03_rb,3.4387e-03_rb,8.0667e-04_rb/)
      kao(:, 5, 7,11) = (/ &
     &2.2422e-02_rb,1.9613e-02_rb,1.6806e-02_rb,1.4145e-02_rb,1.1493e-02_rb,8.8108e-03_rb, &
     &6.0692e-03_rb,3.4048e-03_rb,1.1367e-03_rb/)
      kao(:, 1, 8,11) = (/ &
     &9.5097e-02_rb,8.3206e-02_rb,7.1322e-02_rb,5.9437e-02_rb,4.7550e-02_rb,3.5664e-02_rb, &
     &2.3777e-02_rb,1.1891e-02_rb,8.8895e-05_rb/)
      kao(:, 2, 8,11) = (/ &
     &9.2657e-02_rb,8.1078e-02_rb,6.9498e-02_rb,5.7915e-02_rb,4.6333e-02_rb,3.4754e-02_rb, &
     &2.3172e-02_rb,1.1611e-02_rb,1.6093e-04_rb/)
      kao(:, 3, 8,11) = (/ &
     &8.9553e-02_rb,7.8360e-02_rb,6.7165e-02_rb,5.5974e-02_rb,4.4782e-02_rb,3.3592e-02_rb, &
     &2.2399e-02_rb,1.1297e-02_rb,2.7559e-04_rb/)
      kao(:, 4, 8,11) = (/ &
     &8.7450e-02_rb,7.6527e-02_rb,6.5593e-02_rb,5.4667e-02_rb,4.3739e-02_rb,3.2808e-02_rb, &
     &2.1983e-02_rb,1.1121e-02_rb,4.3678e-04_rb/)
      kao(:, 5, 8,11) = (/ &
     &8.6068e-02_rb,7.5315e-02_rb,6.4565e-02_rb,5.3811e-02_rb,4.3053e-02_rb,3.2463e-02_rb, &
     &2.1814e-02_rb,1.1203e-02_rb,7.8846e-04_rb/)
      kao(:, 1, 9,11) = (/ &
     &6.4279e-01_rb,5.6245e-01_rb,4.8207e-01_rb,4.0173e-01_rb,3.2138e-01_rb,2.4105e-01_rb, &
     &1.6070e-01_rb,8.0350e-02_rb,1.4861e-06_rb/)
      kao(:, 2, 9,11) = (/ &
     &6.2663e-01_rb,5.4828e-01_rb,4.6997e-01_rb,3.9164e-01_rb,3.1331e-01_rb,2.3498e-01_rb, &
     &1.5666e-01_rb,7.8331e-02_rb,2.7002e-06_rb/)
      kao(:, 3, 9,11) = (/ &
     &6.1151e-01_rb,5.3505e-01_rb,4.5861e-01_rb,3.8220e-01_rb,3.0575e-01_rb,2.2931e-01_rb, &
     &1.5288e-01_rb,7.6443e-02_rb,4.4491e-06_rb/)
      kao(:, 4, 9,11) = (/ &
     &5.9675e-01_rb,5.2214e-01_rb,4.4753e-01_rb,3.7296e-01_rb,2.9837e-01_rb,2.2379e-01_rb, &
     &1.4920e-01_rb,7.4612e-02_rb,6.8192e-06_rb/)
      kao(:, 5, 9,11) = (/ &
     &5.8474e-01_rb,5.1167e-01_rb,4.3855e-01_rb,3.6547e-01_rb,2.9239e-01_rb,2.1930e-01_rb, &
     &1.4621e-01_rb,7.3117e-02_rb,1.0152e-05_rb/)
      kao(:, 1,10,11) = (/ &
     &5.2427e+00_rb,4.5875e+00_rb,3.9320e+00_rb,3.2767e+00_rb,2.6214e+00_rb,1.9660e+00_rb, &
     &1.3107e+00_rb,6.5538e-01_rb,3.2405e-06_rb/)
      kao(:, 2,10,11) = (/ &
     &5.1308e+00_rb,4.4894e+00_rb,3.8480e+00_rb,3.2067e+00_rb,2.5654e+00_rb,1.9241e+00_rb, &
     &1.2827e+00_rb,6.4136e-01_rb,6.0207e-06_rb/)
      kao(:, 3,10,11) = (/ &
     &5.0278e+00_rb,4.3993e+00_rb,3.7708e+00_rb,3.1424e+00_rb,2.5139e+00_rb,1.8854e+00_rb, &
     &1.2569e+00_rb,6.2847e-01_rb,1.0181e-05_rb/)
      kao(:, 4,10,11) = (/ &
     &4.9276e+00_rb,4.3117e+00_rb,3.6956e+00_rb,3.0798e+00_rb,2.4638e+00_rb,1.8478e+00_rb, &
     &1.2319e+00_rb,6.1596e-01_rb,1.5690e-05_rb/)
      kao(:, 5,10,11) = (/ &
     &4.8175e+00_rb,4.2154e+00_rb,3.6129e+00_rb,3.0109e+00_rb,2.4088e+00_rb,1.8066e+00_rb, &
     &1.2044e+00_rb,6.0226e-01_rb,2.2851e-05_rb/)
      kao(:, 1,11,11) = (/ &
     &1.4444e+01_rb,1.2639e+01_rb,1.0833e+01_rb,9.0277e+00_rb,7.2223e+00_rb,5.4167e+00_rb, &
     &3.6111e+00_rb,1.8056e+00_rb,3.3928e-06_rb/)
      kao(:, 2,11,11) = (/ &
     &1.4191e+01_rb,1.2417e+01_rb,1.0643e+01_rb,8.8695e+00_rb,7.0956e+00_rb,5.3217e+00_rb, &
     &3.5476e+00_rb,1.7739e+00_rb,6.3163e-06_rb/)
      kao(:, 3,11,11) = (/ &
     &1.3919e+01_rb,1.2179e+01_rb,1.0439e+01_rb,8.6994e+00_rb,6.9594e+00_rb,5.2197e+00_rb, &
     &3.4797e+00_rb,1.7399e+00_rb,1.0603e-05_rb/)
      kao(:, 4,11,11) = (/ &
     &1.3652e+01_rb,1.1946e+01_rb,1.0239e+01_rb,8.5326e+00_rb,6.8259e+00_rb,5.1193e+00_rb, &
     &3.4129e+00_rb,1.7065e+00_rb,1.6659e-05_rb/)
      kao(:, 5,11,11) = (/ &
     &1.3348e+01_rb,1.1680e+01_rb,1.0011e+01_rb,8.3424e+00_rb,6.6742e+00_rb,5.0056e+00_rb, &
     &3.3371e+00_rb,1.6685e+00_rb,2.4500e-05_rb/)
      kao(:, 1,12,11) = (/ &
     &2.5285e+01_rb,2.2125e+01_rb,1.8964e+01_rb,1.5803e+01_rb,1.2642e+01_rb,9.4819e+00_rb, &
     &6.3210e+00_rb,3.1605e+00_rb,2.1167e-06_rb/)
      kao(:, 2,12,11) = (/ &
     &2.4880e+01_rb,2.1769e+01_rb,1.8659e+01_rb,1.5550e+01_rb,1.2440e+01_rb,9.3299e+00_rb, &
     &6.2198e+00_rb,3.1099e+00_rb,3.8897e-06_rb/)
      kao(:, 3,12,11) = (/ &
     &2.4433e+01_rb,2.1379e+01_rb,1.8325e+01_rb,1.5271e+01_rb,1.2217e+01_rb,9.1622e+00_rb, &
     &6.1081e+00_rb,3.0541e+00_rb,6.6786e-06_rb/)
      kao(:, 4,12,11) = (/ &
     &2.3948e+01_rb,2.0955e+01_rb,1.7960e+01_rb,1.4967e+01_rb,1.1974e+01_rb,8.9803e+00_rb, &
     &5.9870e+00_rb,2.9934e+00_rb,1.0649e-05_rb/)
      kao(:, 5,12,11) = (/ &
     &2.3474e+01_rb,2.0540e+01_rb,1.7606e+01_rb,1.4671e+01_rb,1.1737e+01_rb,8.8029e+00_rb, &
     &5.8685e+00_rb,2.9342e+00_rb,1.7444e-05_rb/)
      kao(:, 1,13,11) = (/ &
     &3.6004e+01_rb,3.1505e+01_rb,2.7003e+01_rb,2.2503e+01_rb,1.8003e+01_rb,1.3502e+01_rb, &
     &9.0009e+00_rb,4.5004e+00_rb,1.4918e-06_rb/)
      kao(:, 2,13,11) = (/ &
     &3.5389e+01_rb,3.0965e+01_rb,2.6541e+01_rb,2.2118e+01_rb,1.7694e+01_rb,1.3271e+01_rb, &
     &8.8469e+00_rb,4.4235e+00_rb,2.8535e-06_rb/)
      kao(:, 3,13,11) = (/ &
     &3.4779e+01_rb,3.0432e+01_rb,2.6084e+01_rb,2.1737e+01_rb,1.7389e+01_rb,1.3042e+01_rb, &
     &8.6948e+00_rb,4.3474e+00_rb,4.8793e-06_rb/)
      kao(:, 4,13,11) = (/ &
     &3.4139e+01_rb,2.9872e+01_rb,2.5605e+01_rb,2.1337e+01_rb,1.7070e+01_rb,1.2802e+01_rb, &
     &8.5348e+00_rb,4.2675e+00_rb,8.8607e-06_rb/)
      kao(:, 5,13,11) = (/ &
     &3.3563e+01_rb,2.9368e+01_rb,2.5171e+01_rb,2.0976e+01_rb,1.6781e+01_rb,1.2586e+01_rb, &
     &8.3906e+00_rb,4.1953e+00_rb,1.4420e-05_rb/)
      kao(:, 1, 1,12) = (/ &
     &6.4900e-05_rb,2.8719e-04_rb,5.6307e-04_rb,8.4351e-04_rb,1.1291e-03_rb,1.4211e-03_rb, &
     &1.7201e-03_rb,1.9852e-03_rb,2.2469e-03_rb/)
      kao(:, 2, 1,12) = (/ &
     &3.4635e-05_rb,4.0542e-04_rb,7.9731e-04_rb,1.1929e-03_rb,1.5968e-03_rb,2.0148e-03_rb, &
     &2.4518e-03_rb,2.8750e-03_rb,3.1817e-03_rb/)
      kao(:, 3, 1,12) = (/ &
     &3.0386e-05_rb,5.5544e-04_rb,1.0947e-03_rb,1.6361e-03_rb,2.1835e-03_rb,2.7444e-03_rb, &
     &3.3456e-03_rb,3.9572e-03_rb,4.3509e-03_rb/)
      kao(:, 4, 1,12) = (/ &
     &2.1896e-05_rb,7.4746e-04_rb,1.4741e-03_rb,2.1992e-03_rb,2.9238e-03_rb,3.6527e-03_rb, &
     &4.4093e-03_rb,5.2454e-03_rb,5.8288e-03_rb/)
      kao(:, 5, 1,12) = (/ &
     &2.2978e-05_rb,9.8728e-04_rb,1.9484e-03_rb,2.9034e-03_rb,3.8494e-03_rb,4.7859e-03_rb, &
     &5.7216e-03_rb,6.7414e-03_rb,7.6777e-03_rb/)
      kao(:, 1, 2,12) = (/ &
     &1.5749e-04_rb,2.0866e-04_rb,4.6787e-04_rb,6.8742e-04_rb,9.1071e-04_rb,1.1382e-03_rb, &
     &1.3780e-03_rb,1.6236e-03_rb,1.7971e-03_rb/)
      kao(:, 2, 2,12) = (/ &
     &6.4613e-05_rb,2.9516e-04_rb,6.7832e-04_rb,1.0033e-03_rb,1.3310e-03_rb,1.6630e-03_rb, &
     &2.0071e-03_rb,2.4028e-03_rb,2.6327e-03_rb/)
      kao(:, 3, 2,12) = (/ &
     &5.7078e-05_rb,4.5933e-04_rb,9.6118e-04_rb,1.4236e-03_rb,1.8857e-03_rb,2.3486e-03_rb, &
     &2.8168e-03_rb,3.3472e-03_rb,3.7377e-03_rb/)
      kao(:, 4, 2,12) = (/ &
     &4.9262e-05_rb,6.5877e-04_rb,1.3301e-03_rb,1.9726e-03_rb,2.6111e-03_rb,3.2436e-03_rb, &
     &3.8662e-03_rb,4.5052e-03_rb,5.1847e-03_rb/)
      kao(:, 5, 2,12) = (/ &
     &4.0132e-05_rb,8.9682e-04_rb,1.7993e-03_rb,2.6705e-03_rb,3.5334e-03_rb,4.3806e-03_rb, &
     &5.1960e-03_rb,5.9592e-03_rb,7.0262e-03_rb/)
      kao(:, 1, 3,12) = (/ &
     &4.4536e-04_rb,4.2140e-04_rb,4.1326e-04_rb,5.8784e-04_rb,7.3376e-04_rb,8.8410e-04_rb, &
     &1.0353e-03_rb,1.1868e-03_rb,1.3506e-03_rb/)
      kao(:, 2, 3,12) = (/ &
     &3.6360e-04_rb,3.5667e-04_rb,4.9101e-04_rb,8.4210e-04_rb,1.0791e-03_rb,1.3181e-03_rb, &
     &1.5590e-03_rb,1.8085e-03_rb,2.0516e-03_rb/)
      kao(:, 3, 3,12) = (/ &
     &1.5077e-04_rb,2.9297e-04_rb,7.2917e-04_rb,1.2081e-03_rb,1.5723e-03_rb,1.9339e-03_rb, &
     &2.2902e-03_rb,2.6377e-03_rb,3.0400e-03_rb/)
      kao(:, 4, 3,12) = (/ &
     &1.4163e-04_rb,2.2426e-04_rb,1.0557e-03_rb,1.7084e-03_rb,2.2382e-03_rb,2.7621e-03_rb, &
     &3.2708e-03_rb,3.7457e-03_rb,4.3687e-03_rb/)
      kao(:, 5, 3,12) = (/ &
     &1.3019e-04_rb,2.3338e-04_rb,1.4920e-03_rb,2.3388e-03_rb,3.0800e-03_rb,3.8155e-03_rb, &
     &4.5352e-03_rb,5.1815e-03_rb,6.0536e-03_rb/)
      kao(:, 1, 4,12) = (/ &
     &1.4099e-03_rb,1.2519e-03_rb,1.0992e-03_rb,9.6157e-04_rb,8.5060e-04_rb,8.2135e-04_rb, &
     &8.8574e-04_rb,9.4943e-04_rb,1.0571e-03_rb/)
      kao(:, 2, 4,12) = (/ &
     &1.2160e-03_rb,1.0633e-03_rb,1.0308e-03_rb,9.5807e-04_rb,9.2808e-04_rb,1.1772e-03_rb, &
     &1.3218e-03_rb,1.4545e-03_rb,1.6420e-03_rb/)
      kao(:, 3, 4,12) = (/ &
     &7.7631e-04_rb,7.4153e-04_rb,7.5882e-04_rb,8.7236e-04_rb,1.3771e-03_rb,1.6770e-03_rb, &
     &1.9300e-03_rb,2.1670e-03_rb,2.4676e-03_rb/)
      kao(:, 4, 4,12) = (/ &
     &4.5228e-04_rb,5.1764e-04_rb,4.9825e-04_rb,1.0410e-03_rb,1.9642e-03_rb,2.3602e-03_rb, &
     &2.7572e-03_rb,3.1444e-03_rb,3.5784e-03_rb/)
      kao(:, 5, 4,12) = (/ &
     &3.7162e-04_rb,5.4517e-04_rb,4.7653e-04_rb,1.0804e-03_rb,2.6516e-03_rb,3.2347e-03_rb, &
     &3.8223e-03_rb,4.4076e-03_rb,4.9248e-03_rb/)
      kao(:, 1, 5,12) = (/ &
     &3.8990e-03_rb,3.4146e-03_rb,2.9418e-03_rb,2.4792e-03_rb,1.9894e-03_rb,1.5917e-03_rb, &
     &1.1813e-03_rb,8.6801e-04_rb,2.1028e-04_rb/)
      kao(:, 2, 5,12) = (/ &
     &3.6199e-03_rb,3.1680e-03_rb,2.6938e-03_rb,2.3090e-03_rb,1.9956e-03_rb,1.6463e-03_rb, &
     &1.2997e-03_rb,1.3006e-03_rb,2.4494e-04_rb/)
      kao(:, 3, 5,12) = (/ &
     &3.4415e-03_rb,3.0220e-03_rb,2.5761e-03_rb,2.2036e-03_rb,1.8689e-03_rb,1.5859e-03_rb, &
     &1.7898e-03_rb,1.9074e-03_rb,5.0679e-04_rb/)
      kao(:, 4, 5,12) = (/ &
     &2.1716e-03_rb,1.9079e-03_rb,1.6133e-03_rb,1.3356e-03_rb,1.5050e-03_rb,1.8145e-03_rb, &
     &2.4880e-03_rb,2.7223e-03_rb,8.8637e-04_rb/)
      kao(:, 5, 5,12) = (/ &
     &1.3077e-03_rb,1.2440e-03_rb,1.1158e-03_rb,9.0020e-04_rb,1.2713e-03_rb,2.3698e-03_rb, &
     &3.3646e-03_rb,3.7560e-03_rb,1.3375e-03_rb/)
      kao(:, 1, 6,12) = (/ &
     &1.0196e-02_rb,8.9244e-03_rb,7.6531e-03_rb,6.3818e-03_rb,5.1267e-03_rb,3.9081e-03_rb, &
     &2.6537e-03_rb,1.4661e-03_rb,1.6889e-04_rb/)
      kao(:, 2, 6,12) = (/ &
     &1.0111e-02_rb,8.8532e-03_rb,7.5941e-03_rb,6.3593e-03_rb,5.1154e-03_rb,3.8577e-03_rb, &
     &2.6627e-03_rb,1.5273e-03_rb,2.6786e-04_rb/)
      kao(:, 3, 6,12) = (/ &
     &9.5574e-03_rb,8.3745e-03_rb,7.2096e-03_rb,6.0967e-03_rb,4.9927e-03_rb,3.8753e-03_rb, &
     &2.6940e-03_rb,1.7929e-03_rb,4.4721e-04_rb/)
      kao(:, 4, 6,12) = (/ &
     &9.1448e-03_rb,8.0065e-03_rb,6.8180e-03_rb,5.6454e-03_rb,4.5888e-03_rb,3.5047e-03_rb, &
     &2.6334e-03_rb,2.4842e-03_rb,6.5846e-04_rb/)
      kao(:, 5, 6,12) = (/ &
     &5.7075e-03_rb,5.0049e-03_rb,4.5271e-03_rb,4.0365e-03_rb,3.0612e-03_rb,2.5106e-03_rb, &
     &2.5238e-03_rb,3.3792e-03_rb,9.8239e-04_rb/)
      kao(:, 1, 7,12) = (/ &
     &2.9179e-02_rb,2.5534e-02_rb,2.1889e-02_rb,1.8245e-02_rb,1.4599e-02_rb,1.0955e-02_rb, &
     &7.3101e-03_rb,3.7189e-03_rb,1.6629e-04_rb/)
      kao(:, 2, 7,12) = (/ &
     &2.8331e-02_rb,2.4795e-02_rb,2.1258e-02_rb,1.7722e-02_rb,1.4184e-02_rb,1.0647e-02_rb, &
     &7.1746e-03_rb,3.6882e-03_rb,3.2759e-04_rb/)
      kao(:, 3, 7,12) = (/ &
     &2.7584e-02_rb,2.4141e-02_rb,2.0699e-02_rb,1.7251e-02_rb,1.3817e-02_rb,1.0419e-02_rb, &
     &6.9883e-03_rb,3.6331e-03_rb,7.5693e-04_rb/)
      kao(:, 4, 7,12) = (/ &
     &2.7092e-02_rb,2.3710e-02_rb,2.0332e-02_rb,1.6950e-02_rb,1.3669e-02_rb,1.0331e-02_rb, &
     &6.9978e-03_rb,3.6886e-03_rb,1.3272e-03_rb/)
      kao(:, 5, 7,12) = (/ &
     &2.4668e-02_rb,2.1578e-02_rb,1.8509e-02_rb,1.5691e-02_rb,1.2680e-02_rb,9.7141e-03_rb, &
     &6.7022e-03_rb,3.7141e-03_rb,1.7778e-03_rb/)
      kao(:, 1, 8,12) = (/ &
     &1.0361e-01_rb,9.0667e-02_rb,7.7713e-02_rb,6.4765e-02_rb,5.1815e-02_rb,3.8864e-02_rb, &
     &2.5914e-02_rb,1.2965e-02_rb,1.1677e-04_rb/)
      kao(:, 2, 8,12) = (/ &
     &1.0115e-01_rb,8.8507e-02_rb,7.5864e-02_rb,6.3225e-02_rb,5.0584e-02_rb,3.7943e-02_rb, &
     &2.5302e-02_rb,1.2662e-02_rb,2.2624e-04_rb/)
      kao(:, 3, 8,12) = (/ &
     &9.8784e-02_rb,8.6438e-02_rb,7.4094e-02_rb,6.1749e-02_rb,4.9403e-02_rb,3.7059e-02_rb, &
     &2.4715e-02_rb,1.2461e-02_rb,3.7810e-04_rb/)
      kao(:, 4, 8,12) = (/ &
     &9.5779e-02_rb,8.3802e-02_rb,7.1837e-02_rb,5.9866e-02_rb,4.7899e-02_rb,3.5935e-02_rb, &
     &2.4068e-02_rb,1.2158e-02_rb,6.1406e-04_rb/)
      kao(:, 5, 8,12) = (/ &
     &9.3500e-02_rb,8.1816e-02_rb,7.0121e-02_rb,5.8448e-02_rb,4.6756e-02_rb,3.5152e-02_rb, &
     &2.3515e-02_rb,1.2003e-02_rb,9.1223e-04_rb/)
      kao(:, 1, 9,12) = (/ &
     &7.0616e-01_rb,6.1791e-01_rb,5.2965e-01_rb,4.4136e-01_rb,3.5309e-01_rb,2.6483e-01_rb, &
     &1.7655e-01_rb,8.8282e-02_rb,1.5290e-06_rb/)
      kao(:, 2, 9,12) = (/ &
     &6.9151e-01_rb,6.0507e-01_rb,5.1861e-01_rb,4.3218e-01_rb,3.4576e-01_rb,2.5930e-01_rb, &
     &1.7288e-01_rb,8.6445e-02_rb,2.6717e-06_rb/)
      kao(:, 3, 9,12) = (/ &
     &6.7650e-01_rb,5.9194e-01_rb,5.0740e-01_rb,4.2284e-01_rb,3.3827e-01_rb,2.5371e-01_rb, &
     &1.6914e-01_rb,8.4579e-02_rb,4.4922e-06_rb/)
      kao(:, 4, 9,12) = (/ &
     &6.5888e-01_rb,5.7654e-01_rb,4.9417e-01_rb,4.1183e-01_rb,3.2946e-01_rb,2.4710e-01_rb, &
     &1.6473e-01_rb,8.2371e-02_rb,7.3887e-06_rb/)
      kao(:, 5, 9,12) = (/ &
     &6.4040e-01_rb,5.6034e-01_rb,4.8031e-01_rb,4.0028e-01_rb,3.2019e-01_rb,2.4016e-01_rb, &
     &1.6010e-01_rb,8.0057e-02_rb,1.1597e-05_rb/)
      kao(:, 1,10,12) = (/ &
     &5.7734e+00_rb,5.0518e+00_rb,4.3301e+00_rb,3.6085e+00_rb,2.8866e+00_rb,2.1651e+00_rb, &
     &1.4434e+00_rb,7.2167e-01_rb,9.1748e-07_rb/)
      kao(:, 2,10,12) = (/ &
     &5.6550e+00_rb,4.9481e+00_rb,4.2414e+00_rb,3.5344e+00_rb,2.8275e+00_rb,2.1206e+00_rb, &
     &1.4138e+00_rb,7.0688e-01_rb,1.6728e-06_rb/)
      kao(:, 3,10,12) = (/ &
     &5.5293e+00_rb,4.8379e+00_rb,4.1470e+00_rb,3.4558e+00_rb,2.7646e+00_rb,2.0735e+00_rb, &
     &1.3823e+00_rb,6.9113e-01_rb,2.9526e-06_rb/)
      kao(:, 4,10,12) = (/ &
     &5.4054e+00_rb,4.7297e+00_rb,4.0540e+00_rb,3.3783e+00_rb,2.7027e+00_rb,2.0270e+00_rb, &
     &1.3514e+00_rb,6.7568e-01_rb,4.8767e-06_rb/)
      kao(:, 5,10,12) = (/ &
     &5.2733e+00_rb,4.6145e+00_rb,3.9550e+00_rb,3.2959e+00_rb,2.6366e+00_rb,1.9775e+00_rb, &
     &1.3184e+00_rb,6.5919e-01_rb,7.7655e-06_rb/)
      kao(:, 1,11,12) = (/ &
     &1.5785e+01_rb,1.3812e+01_rb,1.1839e+01_rb,9.8659e+00_rb,7.8925e+00_rb,5.9194e+00_rb, &
     &3.9463e+00_rb,1.9732e+00_rb,7.4549e-07_rb/)
      kao(:, 2,11,12) = (/ &
     &1.5479e+01_rb,1.3544e+01_rb,1.1609e+01_rb,9.6738e+00_rb,7.7393e+00_rb,5.8045e+00_rb, &
     &3.8695e+00_rb,1.9348e+00_rb,1.4083e-06_rb/)
      kao(:, 3,11,12) = (/ &
     &1.5180e+01_rb,1.3283e+01_rb,1.1385e+01_rb,9.4881e+00_rb,7.5902e+00_rb,5.6927e+00_rb, &
     &3.7951e+00_rb,1.8975e+00_rb,2.4492e-06_rb/)
      kao(:, 4,11,12) = (/ &
     &1.4875e+01_rb,1.3016e+01_rb,1.1157e+01_rb,9.2972e+00_rb,7.4378e+00_rb,5.5784e+00_rb, &
     &3.7189e+00_rb,1.8594e+00_rb,4.0481e-06_rb/)
      kao(:, 5,11,12) = (/ &
     &1.4584e+01_rb,1.2761e+01_rb,1.0938e+01_rb,9.1150e+00_rb,7.2921e+00_rb,5.4689e+00_rb, &
     &3.6460e+00_rb,1.8230e+00_rb,6.1324e-06_rb/)
      kao(:, 1,12,12) = (/ &
     &2.7851e+01_rb,2.4369e+01_rb,2.0888e+01_rb,1.7407e+01_rb,1.3925e+01_rb,1.0444e+01_rb, &
     &6.9625e+00_rb,3.4813e+00_rb,1.7967e-06_rb/)
      kao(:, 2,12,12) = (/ &
     &2.7381e+01_rb,2.3958e+01_rb,2.0536e+01_rb,1.7113e+01_rb,1.3691e+01_rb,1.0268e+01_rb, &
     &6.8453e+00_rb,3.4226e+00_rb,3.3551e-06_rb/)
      kao(:, 3,12,12) = (/ &
     &2.6898e+01_rb,2.3535e+01_rb,2.0173e+01_rb,1.6811e+01_rb,1.3449e+01_rb,1.0086e+01_rb, &
     &6.7243e+00_rb,3.3621e+00_rb,5.7999e-06_rb/)
      kao(:, 4,12,12) = (/ &
     &2.6422e+01_rb,2.3120e+01_rb,1.9816e+01_rb,1.6513e+01_rb,1.3211e+01_rb,9.9083e+00_rb, &
     &6.6055e+00_rb,3.3027e+00_rb,8.8648e-06_rb/)
      kao(:, 5,12,12) = (/ &
     &2.5893e+01_rb,2.2657e+01_rb,1.9420e+01_rb,1.6183e+01_rb,1.2946e+01_rb,9.7098e+00_rb, &
     &6.4732e+00_rb,3.2366e+00_rb,1.3030e-05_rb/)
      kao(:, 1,13,12) = (/ &
     &4.0056e+01_rb,3.5049e+01_rb,3.0042e+01_rb,2.5035e+01_rb,2.0028e+01_rb,1.5021e+01_rb, &
     &1.0014e+01_rb,5.0068e+00_rb,1.9339e-06_rb/)
      kao(:, 2,13,12) = (/ &
     &3.9543e+01_rb,3.4599e+01_rb,2.9657e+01_rb,2.4713e+01_rb,1.9771e+01_rb,1.4828e+01_rb, &
     &9.8854e+00_rb,4.9428e+00_rb,3.6424e-06_rb/)
      kao(:, 3,13,12) = (/ &
     &3.8932e+01_rb,3.4065e+01_rb,2.9198e+01_rb,2.4333e+01_rb,1.9465e+01_rb,1.4599e+01_rb, &
     &9.7330e+00_rb,4.8663e+00_rb,6.0957e-06_rb/)
      kao(:, 4,13,12) = (/ &
     &3.8278e+01_rb,3.3493e+01_rb,2.8708e+01_rb,2.3923e+01_rb,1.9139e+01_rb,1.4354e+01_rb, &
     &9.5693e+00_rb,4.7847e+00_rb,9.5853e-06_rb/)
      kao(:, 5,13,12) = (/ &
     &3.7502e+01_rb,3.2815e+01_rb,2.8127e+01_rb,2.3438e+01_rb,1.8751e+01_rb,1.4063e+01_rb, &
     &9.3751e+00_rb,4.6877e+00_rb,1.4124e-05_rb/)
      kao(:, 1, 1,13) = (/ &
     &7.4486e-05_rb,5.1116e-04_rb,1.0025e-03_rb,1.4842e-03_rb,1.9493e-03_rb,2.3797e-03_rb, &
     &2.7258e-03_rb,2.8231e-03_rb,3.8853e-03_rb/)
      kao(:, 2, 1,13) = (/ &
     &6.5327e-05_rb,7.3706e-04_rb,1.4529e-03_rb,2.1556e-03_rb,2.8349e-03_rb,3.4671e-03_rb, &
     &3.9764e-03_rb,4.1729e-03_rb,5.6550e-03_rb/)
      kao(:, 3, 1,13) = (/ &
     &2.8267e-05_rb,1.0173e-03_rb,2.0120e-03_rb,2.9905e-03_rb,3.9385e-03_rb,4.8257e-03_rb, &
     &5.5630e-03_rb,5.8315e-03_rb,7.8638e-03_rb/)
      kao(:, 4, 1,13) = (/ &
     &2.3738e-05_rb,1.3547e-03_rb,2.6849e-03_rb,3.9940e-03_rb,5.2660e-03_rb,6.4652e-03_rb, &
     &7.4857e-03_rb,7.8673e-03_rb,1.0519e-02_rb/)
      kao(:, 5, 1,13) = (/ &
     &2.6320e-05_rb,1.7490e-03_rb,3.4713e-03_rb,5.1691e-03_rb,6.8225e-03_rb,8.3917e-03_rb, &
     &9.7490e-03_rb,1.0316e-02_rb,1.3632e-02_rb/)
      kao(:, 1, 2,13) = (/ &
     &1.3584e-04_rb,4.6683e-04_rb,9.0905e-04_rb,1.3477e-03_rb,1.7804e-03_rb,2.1997e-03_rb, &
     &2.5843e-03_rb,2.8459e-03_rb,3.5348e-03_rb/)
      kao(:, 2, 2,13) = (/ &
     &1.2342e-04_rb,6.7608e-04_rb,1.3278e-03_rb,1.9757e-03_rb,2.6172e-03_rb,3.2432e-03_rb, &
     &3.8268e-03_rb,4.2250e-03_rb,5.2098e-03_rb/)
      kao(:, 3, 2,13) = (/ &
     &5.7073e-05_rb,9.3916e-04_rb,1.8537e-03_rb,2.7640e-03_rb,3.6667e-03_rb,4.5531e-03_rb, &
     &5.3892e-03_rb,5.9774e-03_rb,7.3088e-03_rb/)
      kao(:, 4, 2,13) = (/ &
     &4.3007e-05_rb,1.2575e-03_rb,2.4910e-03_rb,3.7207e-03_rb,4.9408e-03_rb,6.1442e-03_rb, &
     &7.2881e-03_rb,8.1201e-03_rb,9.8567e-03_rb/)
      kao(:, 5, 2,13) = (/ &
     &4.4050e-05_rb,1.6325e-03_rb,3.2411e-03_rb,4.8448e-03_rb,6.4428e-03_rb,8.0196e-03_rb, &
     &9.5284e-03_rb,1.0661e-02_rb,1.2860e-02_rb/)
      kao(:, 1, 3,13) = (/ &
     &4.8847e-04_rb,4.8646e-04_rb,7.2624e-04_rb,1.0874e-03_rb,1.4317e-03_rb,1.7747e-03_rb, &
     &2.1141e-03_rb,2.4304e-03_rb,2.8058e-03_rb/)
      kao(:, 2, 3,13) = (/ &
     &3.4302e-04_rb,4.5845e-04_rb,1.0966e-03_rb,1.6182e-03_rb,2.1396e-03_rb,2.6609e-03_rb, &
     &3.1797e-03_rb,3.6788e-03_rb,4.2203e-03_rb/)
      kao(:, 3, 3,13) = (/ &
     &2.3118e-04_rb,5.8673e-04_rb,1.5488e-03_rb,2.2966e-03_rb,3.0451e-03_rb,3.7949e-03_rb, &
     &4.5473e-03_rb,5.2834e-03_rb,6.0295e-03_rb/)
      kao(:, 4, 3,13) = (/ &
     &1.2745e-04_rb,7.5679e-04_rb,2.1131e-03_rb,3.1366e-03_rb,4.1628e-03_rb,5.1929e-03_rb, &
     &6.2318e-03_rb,7.2646e-03_rb,8.2577e-03_rb/)
      kao(:, 5, 3,13) = (/ &
     &1.0163e-04_rb,9.0005e-04_rb,2.8013e-03_rb,4.1651e-03_rb,5.5261e-03_rb,6.8877e-03_rb, &
     &8.2517e-03_rb,9.6360e-03_rb,1.0972e-02_rb/)
      kao(:, 1, 4,13) = (/ &
     &1.4481e-03_rb,1.2897e-03_rb,1.1542e-03_rb,1.0925e-03_rb,1.1503e-03_rb,1.3566e-03_rb, &
     &1.5821e-03_rb,1.8266e-03_rb,2.0462e-03_rb/)
      kao(:, 2, 4,13) = (/ &
     &1.3895e-03_rb,1.2964e-03_rb,1.0801e-03_rb,1.1824e-03_rb,1.6797e-03_rb,2.0441e-03_rb, &
     &2.4259e-03_rb,2.8265e-03_rb,3.1666e-03_rb/)
      kao(:, 3, 4,13) = (/ &
     &8.4970e-04_rb,9.1322e-04_rb,9.4827e-04_rb,1.6805e-03_rb,2.4256e-03_rb,2.9871e-03_rb, &
     &3.5538e-03_rb,4.1410e-03_rb,4.6574e-03_rb/)
      kao(:, 4, 4,13) = (/ &
     &4.1870e-04_rb,6.5230e-04_rb,1.1618e-03_rb,2.4565e-03_rb,3.3994e-03_rb,4.2007e-03_rb, &
     &5.0069e-03_rb,5.8294e-03_rb,6.5858e-03_rb/)
      kao(:, 5, 4,13) = (/ &
     &3.3416e-04_rb,5.3110e-04_rb,1.2880e-03_rb,3.5329e-03_rb,4.6400e-03_rb,5.7434e-03_rb, &
     &6.8465e-03_rb,7.9498e-03_rb,9.0475e-03_rb/)
      kao(:, 1, 5,13) = (/ &
     &4.2095e-03_rb,3.6897e-03_rb,3.1731e-03_rb,2.7426e-03_rb,2.2812e-03_rb,1.8053e-03_rb, &
     &1.3867e-03_rb,1.3663e-03_rb,1.0788e-03_rb/)
      kao(:, 2, 5,13) = (/ &
     &4.0034e-03_rb,3.5161e-03_rb,3.1982e-03_rb,2.7135e-03_rb,2.2188e-03_rb,1.8636e-03_rb, &
     &1.9269e-03_rb,2.1141e-03_rb,1.6636e-03_rb/)
      kao(:, 3, 5,13) = (/ &
     &3.5828e-03_rb,3.2123e-03_rb,2.7997e-03_rb,2.3571e-03_rb,1.9942e-03_rb,2.3633e-03_rb, &
     &2.8135e-03_rb,3.1780e-03_rb,2.3977e-03_rb/)
      kao(:, 4, 5,13) = (/ &
     &2.0578e-03_rb,1.9948e-03_rb,1.9739e-03_rb,1.8494e-03_rb,2.1664e-03_rb,3.4722e-03_rb, &
     &4.0379e-03_rb,4.6191e-03_rb,3.3853e-03_rb/)
      kao(:, 5, 5,13) = (/ &
     &8.8300e-04_rb,9.5901e-04_rb,1.3118e-03_rb,1.4619e-03_rb,2.8921e-03_rb,4.8304e-03_rb, &
     &5.6699e-03_rb,6.5165e-03_rb,5.0451e-03_rb/)
      kao(:, 1, 6,13) = (/ &
     &1.1518e-02_rb,1.0082e-02_rb,8.6450e-03_rb,7.2094e-03_rb,5.7738e-03_rb,4.4142e-03_rb, &
     &2.9967e-03_rb,1.6414e-03_rb,2.3528e-04_rb/)
      kao(:, 2, 6,13) = (/ &
     &1.0779e-02_rb,9.4346e-03_rb,8.0897e-03_rb,6.7606e-03_rb,5.5435e-03_rb,4.2583e-03_rb, &
     &3.0675e-03_rb,1.8475e-03_rb,3.7683e-04_rb/)
      kao(:, 3, 6,13) = (/ &
     &1.0564e-02_rb,9.2493e-03_rb,7.9306e-03_rb,6.7005e-03_rb,5.4724e-03_rb,4.2002e-03_rb, &
     &3.0432e-03_rb,2.5995e-03_rb,5.6137e-04_rb/)
      kao(:, 4, 6,13) = (/ &
     &8.9592e-03_rb,7.8426e-03_rb,6.9339e-03_rb,5.9610e-03_rb,4.9577e-03_rb,4.0115e-03_rb, &
     &3.3973e-03_rb,3.7509e-03_rb,7.8787e-04_rb/)
      kao(:, 5, 6,13) = (/ &
     &5.6084e-03_rb,4.9031e-03_rb,4.1060e-03_rb,3.7154e-03_rb,3.3523e-03_rb,3.1522e-03_rb, &
     &4.6655e-03_rb,5.3564e-03_rb,1.1263e-03_rb/)
      kao(:, 1, 7,13) = (/ &
     &3.2240e-02_rb,2.8212e-02_rb,2.4183e-02_rb,2.0155e-02_rb,1.6127e-02_rb,1.2099e-02_rb, &
     &8.0718e-03_rb,4.1324e-03_rb,2.8875e-04_rb/)
      kao(:, 2, 7,13) = (/ &
     &3.1311e-02_rb,2.7399e-02_rb,2.3486e-02_rb,1.9574e-02_rb,1.5661e-02_rb,1.1749e-02_rb, &
     &7.9670e-03_rb,4.1256e-03_rb,4.9392e-04_rb/)
      kao(:, 3, 7,13) = (/ &
     &2.9762e-02_rb,2.6043e-02_rb,2.2324e-02_rb,1.8611e-02_rb,1.4888e-02_rb,1.1369e-02_rb, &
     &7.7126e-03_rb,4.2395e-03_rb,7.8171e-04_rb/)
      kao(:, 4, 7,13) = (/ &
     &2.8871e-02_rb,2.5265e-02_rb,2.1655e-02_rb,1.8050e-02_rb,1.4581e-02_rb,1.1027e-02_rb, &
     &7.5640e-03_rb,4.2437e-03_rb,1.1828e-03_rb/)
      kao(:, 5, 7,13) = (/ &
     &2.6386e-02_rb,2.3095e-02_rb,1.9798e-02_rb,1.6677e-02_rb,1.3324e-02_rb,1.0191e-02_rb, &
     &7.1310e-03_rb,4.8298e-03_rb,2.2511e-03_rb/)
      kao(:, 1, 8,13) = (/ &
     &1.1497e-01_rb,1.0060e-01_rb,8.6231e-02_rb,7.1859e-02_rb,5.7486e-02_rb,4.3117e-02_rb, &
     &2.8748e-02_rb,1.4377e-02_rb,1.6465e-04_rb/)
      kao(:, 2, 8,13) = (/ &
     &1.1189e-01_rb,9.7901e-02_rb,8.3916e-02_rb,6.9933e-02_rb,5.5946e-02_rb,4.1961e-02_rb, &
     &2.7977e-02_rb,1.3991e-02_rb,2.9555e-04_rb/)
      kao(:, 3, 8,13) = (/ &
     &1.0913e-01_rb,9.5494e-02_rb,8.1850e-02_rb,6.8211e-02_rb,5.4570e-02_rb,4.0928e-02_rb, &
     &2.7290e-02_rb,1.3843e-02_rb,5.2136e-04_rb/)
      kao(:, 4, 8,13) = (/ &
     &1.0534e-01_rb,9.2180e-02_rb,7.9010e-02_rb,6.5847e-02_rb,5.2678e-02_rb,3.9513e-02_rb, &
     &2.6523e-02_rb,1.3514e-02_rb,8.6919e-04_rb/)
      kao(:, 5, 8,13) = (/ &
     &1.0155e-01_rb,8.8866e-02_rb,7.6173e-02_rb,6.3476e-02_rb,5.0785e-02_rb,3.8190e-02_rb, &
     &2.5826e-02_rb,1.3344e-02_rb,1.3651e-03_rb/)
      kao(:, 1, 9,13) = (/ &
     &7.8551e-01_rb,6.8732e-01_rb,5.8909e-01_rb,4.9092e-01_rb,3.9275e-01_rb,2.9457e-01_rb, &
     &1.9638e-01_rb,9.8187e-02_rb,1.8629e-06_rb/)
      kao(:, 2, 9,13) = (/ &
     &7.6466e-01_rb,6.6906e-01_rb,5.7351e-01_rb,4.7791e-01_rb,3.8233e-01_rb,2.8675e-01_rb, &
     &1.9117e-01_rb,9.5586e-02_rb,3.1593e-06_rb/)
      kao(:, 3, 9,13) = (/ &
     &7.4558e-01_rb,6.5237e-01_rb,5.5922e-01_rb,4.6601e-01_rb,3.7282e-01_rb,2.7960e-01_rb, &
     &1.8640e-01_rb,9.3207e-02_rb,5.1342e-06_rb/)
      kao(:, 4, 9,13) = (/ &
     &7.2909e-01_rb,6.3798e-01_rb,5.4684e-01_rb,4.5571e-01_rb,3.6456e-01_rb,2.7343e-01_rb, &
     &1.8229e-01_rb,9.1144e-02_rb,7.8192e-06_rb/)
      kao(:, 5, 9,13) = (/ &
     &7.0726e-01_rb,6.1887e-01_rb,5.3047e-01_rb,4.4203e-01_rb,3.5366e-01_rb,2.6523e-01_rb, &
     &1.7683e-01_rb,8.8423e-02_rb,1.1206e-05_rb/)
      kao(:, 1,10,13) = (/ &
     &6.4269e+00_rb,5.6236e+00_rb,4.8204e+00_rb,4.0168e+00_rb,3.2134e+00_rb,2.4100e+00_rb, &
     &1.6066e+00_rb,8.0335e-01_rb,1.1293e-06_rb/)
      kao(:, 2,10,13) = (/ &
     &6.2640e+00_rb,5.4811e+00_rb,4.6981e+00_rb,3.9150e+00_rb,3.1320e+00_rb,2.3491e+00_rb, &
     &1.5660e+00_rb,7.8302e-01_rb,2.0871e-06_rb/)
      kao(:, 3,10,13) = (/ &
     &6.1220e+00_rb,5.3567e+00_rb,4.5912e+00_rb,3.8262e+00_rb,3.0609e+00_rb,2.2959e+00_rb, &
     &1.5304e+00_rb,7.6518e-01_rb,3.5513e-06_rb/)
      kao(:, 4,10,13) = (/ &
     &5.9998e+00_rb,5.2499e+00_rb,4.4998e+00_rb,3.7498e+00_rb,2.9997e+00_rb,2.2499e+00_rb, &
     &1.5000e+00_rb,7.4994e-01_rb,5.6768e-06_rb/)
      kao(:, 5,10,13) = (/ &
     &5.8812e+00_rb,5.1459e+00_rb,4.4109e+00_rb,3.6757e+00_rb,2.9406e+00_rb,2.2054e+00_rb, &
     &1.4703e+00_rb,7.3514e-01_rb,8.5300e-06_rb/)
      kao(:, 1,11,13) = (/ &
     &1.7678e+01_rb,1.5468e+01_rb,1.3258e+01_rb,1.1049e+01_rb,8.8386e+00_rb,6.6292e+00_rb, &
     &4.4195e+00_rb,2.2097e+00_rb,9.5807e-07_rb/)
      kao(:, 2,11,13) = (/ &
     &1.7256e+01_rb,1.5100e+01_rb,1.2942e+01_rb,1.0785e+01_rb,8.6284e+00_rb,6.4710e+00_rb, &
     &4.3143e+00_rb,2.1571e+00_rb,1.8271e-06_rb/)
      kao(:, 3,11,13) = (/ &
     &1.6917e+01_rb,1.4802e+01_rb,1.2687e+01_rb,1.0572e+01_rb,8.4580e+00_rb,6.3436e+00_rb, &
     &4.2290e+00_rb,2.1146e+00_rb,3.1322e-06_rb/)
      kao(:, 4,11,13) = (/ &
     &1.6624e+01_rb,1.4546e+01_rb,1.2468e+01_rb,1.0390e+01_rb,8.3118e+00_rb,6.2340e+00_rb, &
     &4.1559e+00_rb,2.0780e+00_rb,4.9941e-06_rb/)
      kao(:, 5,11,13) = (/ &
     &1.6312e+01_rb,1.4273e+01_rb,1.2234e+01_rb,1.0195e+01_rb,8.1562e+00_rb,6.1169e+00_rb, &
     &4.0780e+00_rb,2.0391e+00_rb,7.8208e-06_rb/)
      kao(:, 1,12,13) = (/ &
     &3.1083e+01_rb,2.7197e+01_rb,2.3312e+01_rb,1.9426e+01_rb,1.5541e+01_rb,1.1655e+01_rb, &
     &7.7704e+00_rb,3.8852e+00_rb,7.4323e-07_rb/)
      kao(:, 2,12,13) = (/ &
     &3.0431e+01_rb,2.6627e+01_rb,2.2823e+01_rb,1.9019e+01_rb,1.5215e+01_rb,1.1411e+01_rb, &
     &7.6077e+00_rb,3.8037e+00_rb,1.4468e-06_rb/)
      kao(:, 3,12,13) = (/ &
     &2.9924e+01_rb,2.6183e+01_rb,2.2443e+01_rb,1.8702e+01_rb,1.4961e+01_rb,1.1221e+01_rb, &
     &7.4808e+00_rb,3.7405e+00_rb,2.6259e-06_rb/)
      kao(:, 4,12,13) = (/ &
     &2.9439e+01_rb,2.5759e+01_rb,2.2079e+01_rb,1.8399e+01_rb,1.4719e+01_rb,1.1040e+01_rb, &
     &7.3599e+00_rb,3.6798e+00_rb,4.5448e-06_rb/)
      kao(:, 5,12,13) = (/ &
     &2.8940e+01_rb,2.5323e+01_rb,2.1705e+01_rb,1.8088e+01_rb,1.4470e+01_rb,1.0853e+01_rb, &
     &7.2348e+00_rb,3.6174e+00_rb,7.3875e-06_rb/)
      kao(:, 1,13,13) = (/ &
     &4.5024e+01_rb,3.9395e+01_rb,3.3766e+01_rb,2.8140e+01_rb,2.2511e+01_rb,1.6883e+01_rb, &
     &1.1256e+01_rb,5.6279e+00_rb,6.0376e-07_rb/)
      kao(:, 2,13,13) = (/ &
     &4.4143e+01_rb,3.8625e+01_rb,3.3107e+01_rb,2.7589e+01_rb,2.2071e+01_rb,1.6554e+01_rb, &
     &1.1035e+01_rb,5.5177e+00_rb,1.1756e-06_rb/)
      kao(:, 3,13,13) = (/ &
     &4.3457e+01_rb,3.8026e+01_rb,3.2594e+01_rb,2.7161e+01_rb,2.1728e+01_rb,1.6297e+01_rb, &
     &1.0864e+01_rb,5.4320e+00_rb,2.1118e-06_rb/)
      kao(:, 4,13,13) = (/ &
     &4.2787e+01_rb,3.7437e+01_rb,3.2089e+01_rb,2.6742e+01_rb,2.1393e+01_rb,1.6044e+01_rb, &
     &1.0696e+01_rb,5.3482e+00_rb,3.5390e-06_rb/)
      kao(:, 5,13,13) = (/ &
     &4.2071e+01_rb,3.6811e+01_rb,3.1552e+01_rb,2.6294e+01_rb,2.1034e+01_rb,1.5776e+01_rb, &
     &1.0517e+01_rb,5.2587e+00_rb,5.6752e-06_rb/)
      kao(:, 1, 1,14) = (/ &
     &8.0615e-05_rb,9.4684e-04_rb,1.8810e-03_rb,2.8058e-03_rb,3.7171e-03_rb,4.6066e-03_rb, &
     &5.4471e-03_rb,6.0480e-03_rb,7.4265e-03_rb/)
      kao(:, 2, 1,14) = (/ &
     &7.2907e-05_rb,1.3385e-03_rb,2.6578e-03_rb,3.9621e-03_rb,5.2462e-03_rb,6.4943e-03_rb, &
     &7.6854e-03_rb,8.5660e-03_rb,1.0483e-02_rb/)
      kao(:, 3, 1,14) = (/ &
     &6.3747e-05_rb,1.8288e-03_rb,3.6296e-03_rb,5.4086e-03_rb,7.1575e-03_rb,8.8508e-03_rb, &
     &1.0431e-02_rb,1.1658e-02_rb,1.4300e-02_rb/)
      kao(:, 4, 1,14) = (/ &
     &3.9260e-05_rb,2.4240e-03_rb,4.8114e-03_rb,7.1688e-03_rb,9.4794e-03_rb,1.1704e-02_rb, &
     &1.3748e-02_rb,1.5334e-02_rb,1.8941e-02_rb/)
      kao(:, 5, 1,14) = (/ &
     &2.4568e-05_rb,3.1272e-03_rb,6.2078e-03_rb,9.2494e-03_rb,1.2223e-02_rb,1.5067e-02_rb, &
     &1.7657e-02_rb,1.9583e-02_rb,2.4427e-02_rb/)
      kao(:, 1, 2,14) = (/ &
     &1.5596e-04_rb,8.5463e-04_rb,1.6898e-03_rb,2.5200e-03_rb,3.3406e-03_rb,4.1439e-03_rb, &
     &4.9117e-03_rb,5.5191e-03_rb,6.6708e-03_rb/)
      kao(:, 2, 2,14) = (/ &
     &1.3504e-04_rb,1.2362e-03_rb,2.4448e-03_rb,3.6435e-03_rb,4.8236e-03_rb,5.9777e-03_rb, &
     &7.0641e-03_rb,7.9635e-03_rb,9.6287e-03_rb/)
      kao(:, 3, 2,14) = (/ &
     &1.1284e-04_rb,1.7193e-03_rb,3.4047e-03_rb,5.0732e-03_rb,6.7143e-03_rb,8.3092e-03_rb, &
     &9.8022e-03_rb,1.1012e-02_rb,1.3405e-02_rb/)
      kao(:, 4, 2,14) = (/ &
     &7.1068e-05_rb,2.3136e-03_rb,4.5842e-03_rb,6.8306e-03_rb,9.0376e-03_rb,1.1180e-02_rb, &
     &1.3169e-02_rb,1.4761e-02_rb,1.8047e-02_rb/)
      kao(:, 5, 2,14) = (/ &
     &4.9683e-05_rb,3.0234e-03_rb,5.9956e-03_rb,8.9376e-03_rb,1.1824e-02_rb,1.4613e-02_rb, &
     &1.7201e-02_rb,1.9221e-02_rb,2.3615e-02_rb/)
      kao(:, 1, 3,14) = (/ &
     &4.8559e-04_rb,6.3562e-04_rb,1.4148e-03_rb,2.0956e-03_rb,2.7735e-03_rb,3.4411e-03_rb, &
     &4.0892e-03_rb,4.6631e-03_rb,5.5086e-03_rb/)
      kao(:, 2, 3,14) = (/ &
     &3.9874e-04_rb,1.0860e-03_rb,2.1105e-03_rb,3.1322e-03_rb,4.1464e-03_rb,5.1442e-03_rb, &
     &6.1073e-03_rb,6.9472e-03_rb,8.2419e-03_rb/)
      kao(:, 3, 3,14) = (/ &
     &3.0553e-04_rb,1.5424e-03_rb,3.0141e-03_rb,4.4802e-03_rb,5.9320e-03_rb,7.3631e-03_rb, &
     &8.7355e-03_rb,9.9159e-03_rb,1.1803e-02_rb/)
      kao(:, 4, 3,14) = (/ &
     &2.0300e-04_rb,2.1134e-03_rb,4.1489e-03_rb,6.1729e-03_rb,8.1791e-03_rb,1.0152e-02_rb, &
     &1.2037e-02_rb,1.3637e-02_rb,1.6288e-02_rb/)
      kao(:, 5, 3,14) = (/ &
     &1.2261e-04_rb,2.8083e-03_rb,5.5298e-03_rb,8.2360e-03_rb,1.0915e-02_rb,1.3546e-02_rb, &
     &1.6061e-02_rb,1.8163e-02_rb,2.1753e-02_rb/)
      kao(:, 1, 4,14) = (/ &
     &1.6340e-03_rb,1.4799e-03_rb,1.3812e-03_rb,1.7164e-03_rb,2.2393e-03_rb,2.7610e-03_rb, &
     &3.2750e-03_rb,3.7562e-03_rb,4.3569e-03_rb/)
      kao(:, 2, 4,14) = (/ &
     &1.3493e-03_rb,1.1314e-03_rb,1.6943e-03_rb,2.6409e-03_rb,3.4636e-03_rb,4.2822e-03_rb, &
     &5.0857e-03_rb,5.8323e-03_rb,6.7806e-03_rb/)
      kao(:, 3, 4,14) = (/ &
     &9.9538e-04_rb,9.6503e-04_rb,2.6483e-03_rb,3.8772e-03_rb,5.1039e-03_rb,6.3200e-03_rb, &
     &7.5138e-03_rb,8.6138e-03_rb,1.0038e-02_rb/)
      kao(:, 4, 4,14) = (/ &
     &7.1686e-04_rb,9.6594e-04_rb,3.7129e-03_rb,5.4649e-03_rb,7.2111e-03_rb,8.9436e-03_rb, &
     &1.0638e-02_rb,1.2197e-02_rb,1.4232e-02_rb/)
      kao(:, 5, 4,14) = (/ &
     &3.7667e-04_rb,1.0457e-03_rb,5.0357e-03_rb,7.4391e-03_rb,9.8310e-03_rb,1.2205e-02_rb, &
     &1.4525e-02_rb,1.6653e-02_rb,1.9453e-02_rb/)
      kao(:, 1, 5,14) = (/ &
     &4.8186e-03_rb,4.2167e-03_rb,3.6150e-03_rb,3.0747e-03_rb,2.5291e-03_rb,2.2936e-03_rb, &
     &2.5986e-03_rb,2.9578e-03_rb,3.3675e-03_rb/)
      kao(:, 2, 5,14) = (/ &
     &4.3147e-03_rb,3.7763e-03_rb,3.3322e-03_rb,2.8723e-03_rb,2.9945e-03_rb,3.5591e-03_rb, &
     &4.1803e-03_rb,4.7800e-03_rb,5.4630e-03_rb/)
      kao(:, 3, 5,14) = (/ &
     &3.5596e-03_rb,3.1147e-03_rb,2.7981e-03_rb,3.1802e-03_rb,4.4145e-03_rb,5.3933e-03_rb, &
     &6.3651e-03_rb,7.3014e-03_rb,8.3701e-03_rb/)
      kao(:, 4, 5,14) = (/ &
     &2.3785e-03_rb,2.2217e-03_rb,2.2338e-03_rb,4.4885e-03_rb,6.3608e-03_rb,7.8120e-03_rb, &
     &9.2509e-03_rb,1.0634e-02_rb,1.2219e-02_rb/)
      kao(:, 5, 5,14) = (/ &
     &1.3285e-03_rb,1.6004e-03_rb,2.5745e-03_rb,6.3438e-03_rb,8.8283e-03_rb,1.0885e-02_rb, &
     &1.2921e-02_rb,1.4873e-02_rb,1.7112e-02_rb/)
      kao(:, 1, 6,14) = (/ &
     &1.2819e-02_rb,1.1217e-02_rb,9.6163e-03_rb,8.0152e-03_rb,6.4138e-03_rb,4.8948e-03_rb, &
     &3.3434e-03_rb,2.3020e-03_rb,2.0077e-03_rb/)
      kao(:, 2, 6,14) = (/ &
     &1.2485e-02_rb,1.0926e-02_rb,9.3667e-03_rb,7.8075e-03_rb,6.3840e-03_rb,4.8805e-03_rb, &
     &3.7061e-03_rb,3.8075e-03_rb,3.4020e-03_rb/)
      kao(:, 3, 6,14) = (/ &
     &1.1272e-02_rb,9.8625e-03_rb,8.4554e-03_rb,7.3206e-03_rb,6.0114e-03_rb,5.1792e-03_rb, &
     &5.3340e-03_rb,6.0074e-03_rb,5.3316e-03_rb/)
      kao(:, 4, 6,14) = (/ &
     &1.0208e-02_rb,8.9332e-03_rb,7.9437e-03_rb,6.6798e-03_rb,5.8609e-03_rb,6.4855e-03_rb, &
     &7.9228e-03_rb,9.0010e-03_rb,7.9567e-03_rb/)
      kao(:, 5, 6,14) = (/ &
     &6.0521e-03_rb,5.2891e-03_rb,4.8269e-03_rb,4.6206e-03_rb,6.6176e-03_rb,9.3039e-03_rb, &
     &1.1293e-02_rb,1.2903e-02_rb,1.1196e-02_rb/)
      kao(:, 1, 7,14) = (/ &
     &3.6443e-02_rb,3.1891e-02_rb,2.7336e-02_rb,2.2781e-02_rb,1.8226e-02_rb,1.3673e-02_rb, &
     &9.1182e-03_rb,4.6764e-03_rb,1.4855e-04_rb/)
      kao(:, 2, 7,14) = (/ &
     &3.5471e-02_rb,3.1039e-02_rb,2.6606e-02_rb,2.2173e-02_rb,1.7742e-02_rb,1.3309e-02_rb, &
     &9.0490e-03_rb,4.6704e-03_rb,2.5095e-04_rb/)
      kao(:, 3, 7,14) = (/ &
     &3.4456e-02_rb,3.0149e-02_rb,2.5845e-02_rb,2.1540e-02_rb,1.7235e-02_rb,1.3185e-02_rb, &
     &8.9735e-03_rb,5.4794e-03_rb,4.0737e-04_rb/)
      kao(:, 4, 7,14) = (/ &
     &3.1652e-02_rb,2.7700e-02_rb,2.3744e-02_rb,1.9792e-02_rb,1.6225e-02_rb,1.2450e-02_rb, &
     &9.1363e-03_rb,7.6445e-03_rb,6.2917e-04_rb/)
      kao(:, 5, 7,14) = (/ &
     &2.9864e-02_rb,2.6135e-02_rb,2.2399e-02_rb,1.8923e-02_rb,1.5530e-02_rb,1.2143e-02_rb, &
     &1.0151e-02_rb,1.1058e-02_rb,9.1130e-04_rb/)
      kao(:, 1, 8,14) = (/ &
     &1.3223e-01_rb,1.1571e-01_rb,9.9174e-02_rb,8.2651e-02_rb,6.6122e-02_rb,4.9592e-02_rb, &
     &3.3063e-02_rb,1.6534e-02_rb,2.3921e-04_rb/)
      kao(:, 2, 8,14) = (/ &
     &1.2869e-01_rb,1.1260e-01_rb,9.6519e-02_rb,8.0434e-02_rb,6.4349e-02_rb,4.8263e-02_rb, &
     &3.2179e-02_rb,1.6093e-02_rb,4.2965e-04_rb/)
      kao(:, 3, 8,14) = (/ &
     &1.2518e-01_rb,1.0953e-01_rb,9.3886e-02_rb,7.8243e-02_rb,6.2597e-02_rb,4.6952e-02_rb, &
     &3.1306e-02_rb,1.5992e-02_rb,7.4452e-04_rb/)
      kao(:, 4, 8,14) = (/ &
     &1.2188e-01_rb,1.0664e-01_rb,9.1414e-02_rb,7.6180e-02_rb,6.0947e-02_rb,4.5714e-02_rb, &
     &3.0548e-02_rb,1.5885e-02_rb,1.1502e-03_rb/)
      kao(:, 5, 8,14) = (/ &
     &1.1492e-01_rb,1.0055e-01_rb,8.6192e-02_rb,7.1832e-02_rb,5.7467e-02_rb,4.3105e-02_rb, &
     &2.9610e-02_rb,1.5934e-02_rb,1.8646e-03_rb/)
      kao(:, 1, 9,14) = (/ &
     &9.2387e-01_rb,8.0839e-01_rb,6.9293e-01_rb,5.7740e-01_rb,4.6193e-01_rb,3.4646e-01_rb, &
     &2.3098e-01_rb,1.1549e-01_rb,2.2371e-06_rb/)
      kao(:, 2, 9,14) = (/ &
     &9.0122e-01_rb,7.8856e-01_rb,6.7591e-01_rb,5.6327e-01_rb,4.5061e-01_rb,3.3795e-01_rb, &
     &2.2531e-01_rb,1.1266e-01_rb,3.5853e-06_rb/)
      kao(:, 3, 9,14) = (/ &
     &8.7866e-01_rb,7.6881e-01_rb,6.5900e-01_rb,5.4913e-01_rb,4.3933e-01_rb,3.2948e-01_rb, &
     &2.1967e-01_rb,1.0984e-01_rb,5.0567e-06_rb/)
      kao(:, 4, 9,14) = (/ &
     &8.5611e-01_rb,7.4910e-01_rb,6.4207e-01_rb,5.3506e-01_rb,4.2806e-01_rb,3.2105e-01_rb, &
     &2.1404e-01_rb,1.0702e-01_rb,6.9838e-06_rb/)
      kao(:, 5, 9,14) = (/ &
     &8.3494e-01_rb,7.3059e-01_rb,6.2623e-01_rb,5.2185e-01_rb,4.1749e-01_rb,3.1312e-01_rb, &
     &2.0874e-01_rb,1.0439e-01_rb,9.6651e-06_rb/)
      kao(:, 1,10,14) = (/ &
     &7.7336e+00_rb,6.7669e+00_rb,5.8002e+00_rb,4.8337e+00_rb,3.8667e+00_rb,2.9002e+00_rb, &
     &1.9334e+00_rb,9.6674e-01_rb,1.4759e-06_rb/)
      kao(:, 2,10,14) = (/ &
     &7.5671e+00_rb,6.6210e+00_rb,5.6751e+00_rb,4.7295e+00_rb,3.7835e+00_rb,2.8377e+00_rb, &
     &1.8919e+00_rb,9.4588e-01_rb,2.4483e-06_rb/)
      kao(:, 3,10,14) = (/ &
     &7.3934e+00_rb,6.4694e+00_rb,5.5449e+00_rb,4.6209e+00_rb,3.6964e+00_rb,2.7725e+00_rb, &
     &1.8483e+00_rb,9.2413e-01_rb,3.6428e-06_rb/)
      kao(:, 4,10,14) = (/ &
     &7.2085e+00_rb,6.3072e+00_rb,5.4063e+00_rb,4.5053e+00_rb,3.6041e+00_rb,2.7031e+00_rb, &
     &1.8022e+00_rb,9.0111e-01_rb,5.1643e-06_rb/)
      kao(:, 5,10,14) = (/ &
     &7.0344e+00_rb,6.1551e+00_rb,5.2759e+00_rb,4.3968e+00_rb,3.5172e+00_rb,2.6378e+00_rb, &
     &1.7586e+00_rb,8.7933e-01_rb,7.2619e-06_rb/)
      kao(:, 1,11,14) = (/ &
     &2.1626e+01_rb,1.8923e+01_rb,1.6220e+01_rb,1.3516e+01_rb,1.0813e+01_rb,8.1097e+00_rb, &
     &5.4065e+00_rb,2.7033e+00_rb,1.2148e-06_rb/)
      kao(:, 2,11,14) = (/ &
     &2.1170e+01_rb,1.8523e+01_rb,1.5877e+01_rb,1.3231e+01_rb,1.0585e+01_rb,7.9391e+00_rb, &
     &5.2925e+00_rb,2.6462e+00_rb,1.9377e-06_rb/)
      kao(:, 3,11,14) = (/ &
     &2.0665e+01_rb,1.8082e+01_rb,1.5499e+01_rb,1.2916e+01_rb,1.0333e+01_rb,7.7495e+00_rb, &
     &5.1662e+00_rb,2.5832e+00_rb,2.9701e-06_rb/)
      kao(:, 4,11,14) = (/ &
     &2.0140e+01_rb,1.7623e+01_rb,1.5106e+01_rb,1.2588e+01_rb,1.0071e+01_rb,7.5528e+00_rb, &
     &5.0352e+00_rb,2.5176e+00_rb,4.4558e-06_rb/)
      kao(:, 5,11,14) = (/ &
     &1.9664e+01_rb,1.7206e+01_rb,1.4748e+01_rb,1.2290e+01_rb,9.8317e+00_rb,7.3738e+00_rb, &
     &4.9159e+00_rb,2.4579e+00_rb,6.2661e-06_rb/)
      kao(:, 1,12,14) = (/ &
     &3.8270e+01_rb,3.3486e+01_rb,2.8703e+01_rb,2.3919e+01_rb,1.9135e+01_rb,1.4352e+01_rb, &
     &9.5673e+00_rb,4.7836e+00_rb,1.1282e-06_rb/)
      kao(:, 2,12,14) = (/ &
     &3.7417e+01_rb,3.2741e+01_rb,2.8063e+01_rb,2.3387e+01_rb,1.8709e+01_rb,1.4031e+01_rb, &
     &9.3545e+00_rb,4.6770e+00_rb,1.8527e-06_rb/)
      kao(:, 3,12,14) = (/ &
     &3.6489e+01_rb,3.1928e+01_rb,2.7367e+01_rb,2.2805e+01_rb,1.8244e+01_rb,1.3683e+01_rb, &
     &9.1218e+00_rb,4.5610e+00_rb,2.7902e-06_rb/)
      kao(:, 4,12,14) = (/ &
     &3.5556e+01_rb,3.1112e+01_rb,2.6666e+01_rb,2.2223e+01_rb,1.7778e+01_rb,1.3333e+01_rb, &
     &8.8890e+00_rb,4.4446e+00_rb,3.8795e-06_rb/)
      kao(:, 5,12,14) = (/ &
     &3.4682e+01_rb,3.0344e+01_rb,2.6011e+01_rb,2.1676e+01_rb,1.7341e+01_rb,1.3005e+01_rb, &
     &8.6702e+00_rb,4.3349e+00_rb,5.0987e-06_rb/)
      kao(:, 1,13,14) = (/ &
     &5.4806e+01_rb,4.7955e+01_rb,4.1104e+01_rb,3.4254e+01_rb,2.7403e+01_rb,2.0551e+01_rb, &
     &1.3701e+01_rb,6.8507e+00_rb,9.3782e-07_rb/)
      kao(:, 2,13,14) = (/ &
     &5.3507e+01_rb,4.6818e+01_rb,4.0130e+01_rb,3.3442e+01_rb,2.6753e+01_rb,2.0066e+01_rb, &
     &1.3376e+01_rb,6.6886e+00_rb,1.6696e-06_rb/)
      kao(:, 3,13,14) = (/ &
     &5.2032e+01_rb,4.5528e+01_rb,3.9023e+01_rb,3.2519e+01_rb,2.6015e+01_rb,1.9512e+01_rb, &
     &1.3008e+01_rb,6.5038e+00_rb,2.8850e-06_rb/)
      kao(:, 4,13,14) = (/ &
     &5.0622e+01_rb,4.4293e+01_rb,3.7966e+01_rb,3.1638e+01_rb,2.5310e+01_rb,1.8983e+01_rb, &
     &1.2655e+01_rb,6.3275e+00_rb,4.3456e-06_rb/)
      kao(:, 5,13,14) = (/ &
     &4.9341e+01_rb,4.3174e+01_rb,3.7007e+01_rb,3.0838e+01_rb,2.4670e+01_rb,1.8503e+01_rb, &
     &1.2335e+01_rb,6.1672e+00_rb,6.2672e-06_rb/)
      kao(:, 1, 1,15) = (/ &
     &9.0368e-05_rb,1.7587e-03_rb,3.4692e-03_rb,5.1215e-03_rb,6.6729e-03_rb,8.0447e-03_rb, &
     &9.0360e-03_rb,8.9574e-03_rb,1.3337e-02_rb/)
      kao(:, 2, 1,15) = (/ &
     &8.5673e-05_rb,2.5288e-03_rb,4.9923e-03_rb,7.3709e-03_rb,9.6068e-03_rb,1.1588e-02_rb, &
     &1.3021e-02_rb,1.2915e-02_rb,1.9203e-02_rb/)
      kao(:, 3, 1,15) = (/ &
     &7.5800e-05_rb,3.4920e-03_rb,6.8957e-03_rb,1.0184e-02_rb,1.3280e-02_rb,1.6021e-02_rb, &
     &1.8010e-02_rb,1.7888e-02_rb,2.6548e-02_rb/)
      kao(:, 4, 1,15) = (/ &
     &6.6885e-05_rb,4.6582e-03_rb,9.2038e-03_rb,1.3596e-02_rb,1.7729e-02_rb,2.1393e-02_rb, &
     &2.4064e-02_rb,2.3923e-02_rb,3.5446e-02_rb/)
      kao(:, 5, 1,15) = (/ &
     &2.9277e-05_rb,6.0336e-03_rb,1.1922e-02_rb,1.7611e-02_rb,2.2979e-02_rb,2.7733e-02_rb, &
     &3.1220e-02_rb,3.1058e-02_rb,4.5944e-02_rb/)
      kao(:, 1, 2,15) = (/ &
     &1.6162e-04_rb,1.7658e-03_rb,3.4927e-03_rb,5.1816e-03_rb,6.8087e-03_rb,8.3146e-03_rb, &
     &9.5542e-03_rb,9.9349e-03_rb,1.3602e-02_rb/)
      kao(:, 2, 2,15) = (/ &
     &1.4940e-04_rb,2.5810e-03_rb,5.1093e-03_rb,7.5846e-03_rb,9.9670e-03_rb,1.2178e-02_rb, &
     &1.4000e-02_rb,1.4578e-02_rb,1.9917e-02_rb/)
      kao(:, 3, 2,15) = (/ &
     &1.4212e-04_rb,3.6142e-03_rb,7.1589e-03_rb,1.0631e-02_rb,1.3977e-02_rb,1.7085e-02_rb, &
     &1.9645e-02_rb,2.0472e-02_rb,2.7934e-02_rb/)
      kao(:, 4, 2,15) = (/ &
     &1.2057e-04_rb,4.8773e-03_rb,9.6676e-03_rb,1.4359e-02_rb,1.8882e-02_rb,2.3081e-02_rb, &
     &2.6561e-02_rb,2.7724e-02_rb,3.7741e-02_rb/)
      kao(:, 5, 2,15) = (/ &
     &6.0191e-05_rb,6.3788e-03_rb,1.2648e-02_rb,1.8790e-02_rb,2.4713e-02_rb,3.0224e-02_rb, &
     &3.4797e-02_rb,3.6354e-02_rb,4.9402e-02_rb/)
      kao(:, 1, 3,15) = (/ &
     &4.4246e-04_rb,1.6142e-03_rb,3.1839e-03_rb,4.7378e-03_rb,6.2661e-03_rb,7.7441e-03_rb, &
     &9.0975e-03_rb,1.0015e-02_rb,1.2497e-02_rb/)
      kao(:, 2, 3,15) = (/ &
     &3.8619e-04_rb,2.4195e-03_rb,4.7839e-03_rb,7.1249e-03_rb,9.4281e-03_rb,1.1657e-02_rb, &
     &1.3699e-02_rb,1.5096e-02_rb,1.8815e-02_rb/)
      kao(:, 3, 3,15) = (/ &
     &3.7186e-04_rb,3.4615e-03_rb,6.8531e-03_rb,1.0215e-02_rb,1.3514e-02_rb,1.6722e-02_rb, &
     &1.9664e-02_rb,2.1688e-02_rb,2.6981e-02_rb/)
      kao(:, 4, 3,15) = (/ &
     &3.4136e-04_rb,4.7556e-03_rb,9.4307e-03_rb,1.4062e-02_rb,1.8619e-02_rb,2.3030e-02_rb, &
     &2.7098e-02_rb,2.9910e-02_rb,3.7185e-02_rb/)
      kao(:, 5, 3,15) = (/ &
     &2.7238e-04_rb,6.3154e-03_rb,1.2533e-02_rb,1.8695e-02_rb,2.4762e-02_rb,3.0640e-02_rb, &
     &3.6060e-02_rb,3.9831e-02_rb,4.9464e-02_rb/)
      kao(:, 1, 4,15) = (/ &
     &1.8439e-03_rb,1.6255e-03_rb,2.7662e-03_rb,4.1011e-03_rb,5.4250e-03_rb,6.7307e-03_rb, &
     &7.9903e-03_rb,9.0701e-03_rb,1.0766e-02_rb/)
      kao(:, 2, 4,15) = (/ &
     &1.2780e-03_rb,2.0382e-03_rb,4.2819e-03_rb,6.3618e-03_rb,8.4274e-03_rb,1.0464e-02_rb, &
     &1.2431e-02_rb,1.4123e-02_rb,1.6753e-02_rb/)
      kao(:, 3, 4,15) = (/ &
     &1.0129e-03_rb,3.1935e-03_rb,6.2902e-03_rb,9.3615e-03_rb,1.2411e-02_rb,1.5417e-02_rb, &
     &1.8325e-02_rb,2.0828e-02_rb,2.4703e-02_rb/)
      kao(:, 4, 4,15) = (/ &
     &9.5662e-04_rb,4.4942e-03_rb,8.8414e-03_rb,1.3166e-02_rb,1.7466e-02_rb,2.1716e-02_rb, &
     &2.5826e-02_rb,2.9379e-02_rb,3.4795e-02_rb/)
      kao(:, 5, 4,15) = (/ &
     &7.9817e-04_rb,6.0664e-03_rb,1.1965e-02_rb,1.7841e-02_rb,2.3679e-02_rb,2.9437e-02_rb, &
     &3.5012e-02_rb,3.9861e-02_rb,4.7202e-02_rb/)
      kao(:, 1, 5,15) = (/ &
     &5.1399e-03_rb,4.4963e-03_rb,3.8550e-03_rb,3.7300e-03_rb,4.5601e-03_rb,5.6445e-03_rb, &
     &6.7084e-03_rb,7.7027e-03_rb,8.9288e-03_rb/)
      kao(:, 2, 5,15) = (/ &
     &4.9517e-03_rb,4.3316e-03_rb,4.1416e-03_rb,5.5566e-03_rb,7.3222e-03_rb,9.0762e-03_rb, &
     &1.0805e-02_rb,1.2425e-02_rb,1.4409e-02_rb/)
      kao(:, 3, 5,15) = (/ &
     &3.6157e-03_rb,3.1633e-03_rb,5.3630e-03_rb,8.3885e-03_rb,1.1080e-02_rb,1.3756e-02_rb, &
     &1.6391e-02_rb,1.8868e-02_rb,2.1878e-02_rb/)
      kao(:, 4, 5,15) = (/ &
     &2.5485e-03_rb,3.2448e-03_rb,8.0168e-03_rb,1.2065e-02_rb,1.5959e-02_rb,1.9835e-02_rb, &
     &2.3656e-02_rb,2.7245e-02_rb,3.1588e-02_rb/)
      kao(:, 5, 5,15) = (/ &
     &2.2002e-03_rb,3.7004e-03_rb,1.1229e-02_rb,1.6648e-02_rb,2.2051e-02_rb,2.7414e-02_rb, &
     &3.2714e-02_rb,3.7713e-02_rb,4.3725e-02_rb/)
      kao(:, 1, 6,15) = (/ &
     &1.3785e-02_rb,1.2061e-02_rb,1.0339e-02_rb,8.6158e-03_rb,6.8942e-03_rb,5.5266e-03_rb, &
     &5.3843e-03_rb,6.1361e-03_rb,7.0081e-03_rb/)
      kao(:, 2, 6,15) = (/ &
     &1.3401e-02_rb,1.1726e-02_rb,1.0052e-02_rb,8.3774e-03_rb,7.3256e-03_rb,7.6626e-03_rb, &
     &8.9483e-03_rb,1.0302e-02_rb,1.1787e-02_rb/)
      kao(:, 3, 6,15) = (/ &
     &1.3001e-02_rb,1.1375e-02_rb,9.7516e-03_rb,8.8385e-03_rb,9.7753e-03_rb,1.1803e-02_rb, &
     &1.4015e-02_rb,1.6165e-02_rb,1.8521e-02_rb/)
      kao(:, 4, 6,15) = (/ &
     &1.0311e-02_rb,9.0242e-03_rb,7.7593e-03_rb,9.8758e-03_rb,1.4142e-02_rb,1.7458e-02_rb, &
     &2.0764e-02_rb,2.3987e-02_rb,2.7500e-02_rb/)
      kao(:, 5, 6,15) = (/ &
     &6.1007e-03_rb,5.3334e-03_rb,8.4379e-03_rb,1.3947e-02_rb,1.9967e-02_rb,2.4665e-02_rb, &
     &2.9359e-02_rb,3.3960e-02_rb,3.8947e-02_rb/)
      kao(:, 1, 7,15) = (/ &
     &3.9607e-02_rb,3.4655e-02_rb,2.9704e-02_rb,2.4754e-02_rb,1.9804e-02_rb,1.4853e-02_rb, &
     &9.9024e-03_rb,5.6087e-03_rb,4.7437e-03_rb/)
      kao(:, 2, 7,15) = (/ &
     &3.8786e-02_rb,3.3939e-02_rb,2.9089e-02_rb,2.4244e-02_rb,1.9395e-02_rb,1.4546e-02_rb, &
     &1.0087e-02_rb,8.3483e-03_rb,8.1480e-03_rb/)
      kao(:, 3, 7,15) = (/ &
     &3.7941e-02_rb,3.3203e-02_rb,2.8458e-02_rb,2.3715e-02_rb,1.8973e-02_rb,1.4624e-02_rb, &
     &1.2669e-02_rb,1.3455e-02_rb,1.2971e-02_rb/)
      kao(:, 4, 7,15) = (/ &
     &3.7052e-02_rb,3.2423e-02_rb,2.7792e-02_rb,2.3160e-02_rb,1.8933e-02_rb,1.7157e-02_rb, &
     &1.8083e-02_rb,2.0593e-02_rb,1.9406e-02_rb/)
      kao(:, 5, 7,15) = (/ &
     &3.1592e-02_rb,2.7646e-02_rb,2.3693e-02_rb,2.0299e-02_rb,1.8491e-02_rb,2.1228e-02_rb, &
     &2.6244e-02_rb,2.9985e-02_rb,2.7520e-02_rb/)
      kao(:, 1, 8,15) = (/ &
     &1.4580e-01_rb,1.2758e-01_rb,1.0936e-01_rb,9.1128e-02_rb,7.2910e-02_rb,5.4675e-02_rb, &
     &3.6454e-02_rb,1.8228e-02_rb,7.5823e-04_rb/)
      kao(:, 2, 8,15) = (/ &
     &1.4341e-01_rb,1.2548e-01_rb,1.0755e-01_rb,8.9631e-02_rb,7.1703e-02_rb,5.3780e-02_rb, &
     &3.5854e-02_rb,1.7928e-02_rb,1.3701e-03_rb/)
      kao(:, 3, 8,15) = (/ &
     &1.4083e-01_rb,1.2323e-01_rb,1.0562e-01_rb,8.8019e-02_rb,7.0413e-02_rb,5.2809e-02_rb, &
     &3.5210e-02_rb,1.8039e-02_rb,2.2984e-03_rb/)
      kao(:, 4, 8,15) = (/ &
     &1.3806e-01_rb,1.2080e-01_rb,1.0354e-01_rb,8.6290e-02_rb,6.9028e-02_rb,5.1771e-02_rb, &
     &3.4515e-02_rb,2.0878e-02_rb,3.6182e-03_rb/)
      kao(:, 5, 8,15) = (/ &
     &1.3508e-01_rb,1.1820e-01_rb,1.0132e-01_rb,8.4431e-02_rb,6.7546e-02_rb,5.0657e-02_rb, &
     &3.5330e-02_rb,2.7298e-02_rb,5.3909e-03_rb/)
      kao(:, 1, 9,15) = (/ &
     &1.0374e+00_rb,9.0767e-01_rb,7.7802e-01_rb,6.4832e-01_rb,5.1864e-01_rb,3.8900e-01_rb, &
     &2.5933e-01_rb,1.2966e-01_rb,2.1397e-06_rb/)
      kao(:, 2, 9,15) = (/ &
     &1.0197e+00_rb,8.9224e-01_rb,7.6478e-01_rb,6.3733e-01_rb,5.0988e-01_rb,3.8239e-01_rb, &
     &2.5493e-01_rb,1.2747e-01_rb,3.4826e-06_rb/)
      kao(:, 3, 9,15) = (/ &
     &1.0031e+00_rb,8.7777e-01_rb,7.5240e-01_rb,6.2700e-01_rb,5.0158e-01_rb,3.7619e-01_rb, &
     &2.5081e-01_rb,1.2540e-01_rb,5.6811e-06_rb/)
      kao(:, 4, 9,15) = (/ &
     &9.8558e-01_rb,8.6233e-01_rb,7.3918e-01_rb,6.1597e-01_rb,4.9277e-01_rb,3.6960e-01_rb, &
     &2.4641e-01_rb,1.2320e-01_rb,9.7907e-06_rb/)
      kao(:, 5, 9,15) = (/ &
     &9.6612e-01_rb,8.4542e-01_rb,7.2465e-01_rb,6.0387e-01_rb,4.8310e-01_rb,3.6231e-01_rb, &
     &2.4155e-01_rb,1.2077e-01_rb,1.4737e-05_rb/)
      kao(:, 1,10,15) = (/ &
     &8.9111e+00_rb,7.7969e+00_rb,6.6836e+00_rb,5.5691e+00_rb,4.4554e+00_rb,3.3414e+00_rb, &
     &2.2275e+00_rb,1.1138e+00_rb,1.1507e-06_rb/)
      kao(:, 2,10,15) = (/ &
     &8.7649e+00_rb,7.6684e+00_rb,6.5728e+00_rb,5.4777e+00_rb,4.3822e+00_rb,3.2866e+00_rb, &
     &2.1910e+00_rb,1.0955e+00_rb,1.9584e-06_rb/)
      kao(:, 3,10,15) = (/ &
     &8.6133e+00_rb,7.5361e+00_rb,6.4598e+00_rb,5.3829e+00_rb,4.3064e+00_rb,3.2299e+00_rb, &
     &2.1532e+00_rb,1.0766e+00_rb,3.2367e-06_rb/)
      kao(:, 4,10,15) = (/ &
     &8.4534e+00_rb,7.3969e+00_rb,6.3406e+00_rb,5.2836e+00_rb,4.2270e+00_rb,3.1701e+00_rb, &
     &2.1135e+00_rb,1.0568e+00_rb,5.5316e-06_rb/)
      kao(:, 5,10,15) = (/ &
     &8.2801e+00_rb,7.2444e+00_rb,6.2095e+00_rb,5.1748e+00_rb,4.1398e+00_rb,3.1049e+00_rb, &
     &2.0699e+00_rb,1.0350e+00_rb,8.6177e-06_rb/)
      kao(:, 1,11,15) = (/ &
     &2.5435e+01_rb,2.2258e+01_rb,1.9079e+01_rb,1.5898e+01_rb,1.2719e+01_rb,9.5392e+00_rb, &
     &6.3596e+00_rb,3.1798e+00_rb,8.0677e-07_rb/)
      kao(:, 2,11,15) = (/ &
     &2.5109e+01_rb,2.1970e+01_rb,1.8832e+01_rb,1.5693e+01_rb,1.2554e+01_rb,9.4153e+00_rb, &
     &6.2771e+00_rb,3.1385e+00_rb,1.4119e-06_rb/)
      kao(:, 3,11,15) = (/ &
     &2.4708e+01_rb,2.1620e+01_rb,1.8532e+01_rb,1.5442e+01_rb,1.2355e+01_rb,9.2660e+00_rb, &
     &6.1771e+00_rb,3.0885e+00_rb,2.4283e-06_rb/)
      kao(:, 4,11,15) = (/ &
     &2.4206e+01_rb,2.1181e+01_rb,1.8155e+01_rb,1.5130e+01_rb,1.2103e+01_rb,9.0784e+00_rb, &
     &6.0520e+00_rb,3.0256e+00_rb,3.8467e-06_rb/)
      kao(:, 5,11,15) = (/ &
     &2.3645e+01_rb,2.0690e+01_rb,1.7735e+01_rb,1.4778e+01_rb,1.1822e+01_rb,8.8671e+00_rb, &
     &5.9114e+00_rb,2.9557e+00_rb,6.2945e-06_rb/)
      kao(:, 1,12,15) = (/ &
     &4.5821e+01_rb,4.0093e+01_rb,3.4365e+01_rb,2.8640e+01_rb,2.2910e+01_rb,1.7183e+01_rb, &
     &1.1455e+01_rb,5.7277e+00_rb,5.6905e-07_rb/)
      kao(:, 2,12,15) = (/ &
     &4.5215e+01_rb,3.9561e+01_rb,3.3912e+01_rb,2.8260e+01_rb,2.2608e+01_rb,1.6955e+01_rb, &
     &1.1304e+01_rb,5.6520e+00_rb,1.0466e-06_rb/)
      kao(:, 3,12,15) = (/ &
     &4.4441e+01_rb,3.8886e+01_rb,3.3332e+01_rb,2.7776e+01_rb,2.2220e+01_rb,1.6665e+01_rb, &
     &1.1110e+01_rb,5.5552e+00_rb,1.8066e-06_rb/)
      kao(:, 4,12,15) = (/ &
     &4.3598e+01_rb,3.8150e+01_rb,3.2698e+01_rb,2.7250e+01_rb,2.1798e+01_rb,1.6350e+01_rb, &
     &1.0900e+01_rb,5.4499e+00_rb,2.8751e-06_rb/)
      kao(:, 5,12,15) = (/ &
     &4.2666e+01_rb,3.7332e+01_rb,3.2000e+01_rb,2.6665e+01_rb,2.1333e+01_rb,1.6001e+01_rb, &
     &1.0666e+01_rb,5.3333e+00_rb,4.5303e-06_rb/)
      kao(:, 1,13,15) = (/ &
     &6.6305e+01_rb,5.8017e+01_rb,4.9731e+01_rb,4.1443e+01_rb,3.3153e+01_rb,2.4866e+01_rb, &
     &1.6576e+01_rb,8.2888e+00_rb,6.5868e-07_rb/)
      kao(:, 2,13,15) = (/ &
     &6.5425e+01_rb,5.7245e+01_rb,4.9068e+01_rb,4.0889e+01_rb,3.2711e+01_rb,2.4534e+01_rb, &
     &1.6356e+01_rb,8.1783e+00_rb,1.0770e-06_rb/)
      kao(:, 3,13,15) = (/ &
     &6.4452e+01_rb,5.6397e+01_rb,4.8336e+01_rb,4.0283e+01_rb,3.2227e+01_rb,2.4169e+01_rb, &
     &1.6113e+01_rb,8.0564e+00_rb,1.3521e-06_rb/)
      kao(:, 4,13,15) = (/ &
     &6.3313e+01_rb,5.5397e+01_rb,4.7486e+01_rb,3.9571e+01_rb,3.1655e+01_rb,2.3741e+01_rb, &
     &1.5828e+01_rb,7.9140e+00_rb,2.2152e-06_rb/)
      kao(:, 5,13,15) = (/ &
     &6.1963e+01_rb,5.4219e+01_rb,4.6473e+01_rb,3.8726e+01_rb,3.0982e+01_rb,2.3236e+01_rb, &
     &1.5490e+01_rb,7.7450e+00_rb,3.4066e-06_rb/)
      kao(:, 1, 1,16) = (/ &
     &8.8773e-05_rb,1.9140e-03_rb,3.7690e-03_rb,5.5469e-03_rb,7.2016e-03_rb,8.6351e-03_rb, &
     &9.6093e-03_rb,9.3558e-03_rb,1.4394e-02_rb/)
      kao(:, 2, 1,16) = (/ &
     &8.4216e-05_rb,2.7660e-03_rb,5.4492e-03_rb,8.0218e-03_rb,1.0416e-02_rb,1.2486e-02_rb, &
     &1.3893e-02_rb,1.3527e-02_rb,2.0822e-02_rb/)
      kao(:, 3, 1,16) = (/ &
     &8.0290e-05_rb,3.8383e-03_rb,7.5630e-03_rb,1.1136e-02_rb,1.4457e-02_rb,1.7334e-02_rb, &
     &1.9290e-02_rb,1.8781e-02_rb,2.8902e-02_rb/)
      kao(:, 4, 1,16) = (/ &
     &7.6611e-05_rb,5.1452e-03_rb,1.0139e-02_rb,1.4930e-02_rb,1.9392e-02_rb,2.3248e-02_rb, &
     &2.5870e-02_rb,2.5185e-02_rb,3.8771e-02_rb/)
      kao(:, 5, 1,16) = (/ &
     &7.3163e-05_rb,6.6935e-03_rb,1.3197e-02_rb,1.9435e-02_rb,2.5235e-02_rb,3.0263e-02_rb, &
     &3.3675e-02_rb,3.2771e-02_rb,5.0455e-02_rb/)
      kao(:, 1, 2,16) = (/ &
     &1.5679e-04_rb,1.9888e-03_rb,3.9277e-03_rb,5.8149e-03_rb,7.6126e-03_rb,9.2475e-03_rb, &
     &1.0521e-02_rb,1.0714e-02_rb,1.5210e-02_rb/)
      kao(:, 2, 2,16) = (/ &
     &1.4773e-04_rb,2.9274e-03_rb,5.7862e-03_rb,8.5677e-03_rb,1.1217e-02_rb,1.3627e-02_rb, &
     &1.5504e-02_rb,1.5787e-02_rb,2.2417e-02_rb/)
      kao(:, 3, 2,16) = (/ &
     &1.4025e-04_rb,4.1262e-03_rb,8.1606e-03_rb,1.2085e-02_rb,1.5823e-02_rb,1.9223e-02_rb, &
     &2.1876e-02_rb,2.2273e-02_rb,3.1626e-02_rb/)
      kao(:, 4, 2,16) = (/ &
     &1.3354e-04_rb,5.6053e-03_rb,1.1089e-02_rb,1.6429e-02_rb,2.1516e-02_rb,2.6138e-02_rb, &
     &2.9742e-02_rb,3.0283e-02_rb,4.3010e-02_rb/)
      kao(:, 5, 2,16) = (/ &
     &1.2725e-04_rb,7.3793e-03_rb,1.4605e-02_rb,2.1632e-02_rb,2.8334e-02_rb,3.4423e-02_rb, &
     &3.9175e-02_rb,3.9879e-02_rb,5.6642e-02_rb/)
      kao(:, 1, 3,16) = (/ &
     &4.1725e-04_rb,1.8982e-03_rb,3.7447e-03_rb,5.5658e-03_rb,7.3458e-03_rb,9.0433e-03_rb, &
     &1.0548e-02_rb,1.1397e-02_rb,1.4655e-02_rb/)
      kao(:, 2, 3,16) = (/ &
     &3.9098e-04_rb,2.8734e-03_rb,5.6787e-03_rb,8.4482e-03_rb,1.1153e-02_rb,1.3733e-02_rb, &
     &1.6021e-02_rb,1.7312e-02_rb,2.2262e-02_rb/)
      kao(:, 3, 3,16) = (/ &
     &3.4851e-04_rb,4.1474e-03_rb,8.2100e-03_rb,1.2218e-02_rb,1.6131e-02_rb,1.9869e-02_rb, &
     &2.3184e-02_rb,2.5055e-02_rb,3.2211e-02_rb/)
      kao(:, 4, 3,16) = (/ &
     &3.3110e-04_rb,5.7507e-03_rb,1.1396e-02_rb,1.6965e-02_rb,2.2409e-02_rb,2.7601e-02_rb, &
     &3.2208e-02_rb,3.4575e-02_rb,4.4760e-02_rb/)
      kao(:, 5, 3,16) = (/ &
     &3.1495e-04_rb,7.7054e-03_rb,1.5281e-02_rb,2.2758e-02_rb,3.0062e-02_rb,3.7025e-02_rb, &
     &4.3208e-02_rb,4.6704e-02_rb,6.0059e-02_rb/)
      kao(:, 1, 4,16) = (/ &
     &1.7281e-03_rb,1.7667e-03_rb,3.4371e-03_rb,5.0959e-03_rb,6.7367e-03_rb,8.3430e-03_rb, &
     &9.8600e-03_rb,1.1050e-02_rb,1.3383e-02_rb/)
      kao(:, 2, 4,16) = (/ &
     &1.1628e-03_rb,2.7537e-03_rb,5.3881e-03_rb,8.0069e-03_rb,1.0596e-02_rb,1.3125e-02_rb, &
     &1.5518e-02_rb,1.7399e-02_rb,2.1083e-02_rb/)
      kao(:, 3, 4,16) = (/ &
     &1.1124e-03_rb,4.0786e-03_rb,8.0137e-03_rb,1.1921e-02_rb,1.5788e-02_rb,1.9566e-02_rb, &
     &2.3141e-02_rb,2.5952e-02_rb,3.1447e-02_rb/)
      kao(:, 4, 4,16) = (/ &
     &1.0810e-03_rb,5.7837e-03_rb,1.1394e-02_rb,1.6970e-02_rb,2.2483e-02_rb,2.7876e-02_rb, &
     &3.2974e-02_rb,3.6985e-02_rb,4.4815e-02_rb/)
      kao(:, 5, 4,16) = (/ &
     &9.9376e-04_rb,7.9012e-03_rb,1.5599e-02_rb,2.3249e-02_rb,3.0813e-02_rb,3.8207e-02_rb, &
     &4.5202e-02_rb,5.0705e-02_rb,6.1455e-02_rb/)
      kao(:, 1, 5,16) = (/ &
     &5.4114e-03_rb,4.7354e-03_rb,4.0587e-03_rb,4.6142e-03_rb,6.0695e-03_rb,7.5080e-03_rb, &
     &8.9059e-03_rb,1.0147e-02_rb,1.1928e-02_rb/)
      kao(:, 2, 5,16) = (/ &
     &5.1867e-03_rb,4.5542e-03_rb,5.0981e-03_rb,7.5059e-03_rb,9.9017e-03_rb,1.2267e-02_rb, &
     &1.4567e-02_rb,1.6610e-02_rb,1.9542e-02_rb/)
      kao(:, 3, 5,16) = (/ &
     &3.0269e-03_rb,2.6488e-03_rb,7.7938e-03_rb,1.1517e-02_rb,1.5221e-02_rb,1.8876e-02_rb, &
     &2.2431e-02_rb,2.5594e-02_rb,3.0125e-02_rb/)
      kao(:, 4, 5,16) = (/ &
     &2.9184e-03_rb,2.5534e-03_rb,1.1353e-02_rb,1.6816e-02_rb,2.2248e-02_rb,2.7623e-02_rb, &
     &3.2847e-02_rb,3.7486e-02_rb,4.4122e-02_rb/)
      kao(:, 5, 5,16) = (/ &
     &2.8186e-03_rb,8.1567e-03_rb,1.5865e-02_rb,2.3545e-02_rb,3.1185e-02_rb,3.8732e-02_rb, &
     &4.6076e-02_rb,5.2599e-02_rb,6.1936e-02_rb/)
      kao(:, 1, 6,16) = (/ &
     &1.4590e-02_rb,1.2765e-02_rb,1.0941e-02_rb,9.1186e-03_rb,7.2947e-03_rb,6.5373e-03_rb, &
     &7.7212e-03_rb,8.8306e-03_rb,1.0195e-02_rb/)
      kao(:, 2, 6,16) = (/ &
     &1.4301e-02_rb,1.2513e-02_rb,1.0724e-02_rb,8.9392e-03_rb,9.0428e-03_rb,1.1132e-02_rb, &
     &1.3188e-02_rb,1.5117e-02_rb,1.7482e-02_rb/)
      kao(:, 3, 6,16) = (/ &
     &1.3976e-02_rb,1.2228e-02_rb,1.0485e-02_rb,1.0970e-02_rb,1.4363e-02_rb,1.7735e-02_rb, &
     &2.1052e-02_rb,2.4167e-02_rb,2.7980e-02_rb/)
      kao(:, 4, 6,16) = (/ &
     &7.5839e-03_rb,6.6346e-03_rb,9.3364e-03_rb,1.6444e-02_rb,2.1598e-02_rb,2.6720e-02_rb, &
     &3.1771e-02_rb,3.6510e-02_rb,4.2297e-02_rb/)
      kao(:, 5, 6,16) = (/ &
     &7.3499e-03_rb,6.4297e-03_rb,1.6063e-02_rb,2.3546e-02_rb,3.1010e-02_rb,3.8427e-02_rb, &
     &4.5729e-02_rb,5.2589e-02_rb,6.0961e-02_rb/)
      kao(:, 1, 7,16) = (/ &
     &4.2002e-02_rb,3.6747e-02_rb,3.1501e-02_rb,2.6251e-02_rb,2.1002e-02_rb,1.5754e-02_rb, &
     &1.0500e-02_rb,7.5058e-03_rb,8.4905e-03_rb/)
      kao(:, 2, 7,16) = (/ &
     &4.1348e-02_rb,3.6182e-02_rb,3.1017e-02_rb,2.5842e-02_rb,2.0675e-02_rb,1.5509e-02_rb, &
     &1.1819e-02_rb,1.3466e-02_rb,1.5310e-02_rb/)
      kao(:, 3, 7,16) = (/ &
     &4.0563e-02_rb,3.5489e-02_rb,3.0424e-02_rb,2.5351e-02_rb,2.0286e-02_rb,1.6643e-02_rb, &
     &1.9563e-02_rb,2.2386e-02_rb,2.5539e-02_rb/)
      kao(:, 4, 7,16) = (/ &
     &3.9401e-02_rb,3.4717e-02_rb,2.9752e-02_rb,2.4793e-02_rb,2.1115e-02_rb,2.5798e-02_rb, &
     &3.0449e-02_rb,3.4941e-02_rb,3.9945e-02_rb/)
      kao(:, 5, 7,16) = (/ &
     &2.0373e-02_rb,1.7827e-02_rb,1.5278e-02_rb,1.2733e-02_rb,3.0987e-02_rb,3.8017e-02_rb, &
     &4.4987e-02_rb,5.1736e-02_rb,5.9242e-02_rb/)
      kao(:, 1, 8,16) = (/ &
     &1.5400e-01_rb,1.3474e-01_rb,1.1553e-01_rb,9.6239e-02_rb,7.7005e-02_rb,5.7761e-02_rb, &
     &3.8504e-02_rb,1.9252e-02_rb,6.9298e-03_rb/)
      kao(:, 2, 8,16) = (/ &
     &1.5231e-01_rb,1.3330e-01_rb,1.1425e-01_rb,9.5219e-02_rb,7.6158e-02_rb,5.7117e-02_rb, &
     &3.8082e-02_rb,1.9041e-02_rb,1.3109e-02_rb/)
      kao(:, 3, 8,16) = (/ &
     &1.4996e-01_rb,1.3122e-01_rb,1.1247e-01_rb,9.3714e-02_rb,7.4984e-02_rb,5.6235e-02_rb, &
     &3.7492e-02_rb,2.0794e-02_rb,2.1673e-02_rb/)
      kao(:, 4, 8,16) = (/ &
     &1.4710e-01_rb,1.2871e-01_rb,1.1034e-01_rb,9.1947e-02_rb,7.3562e-02_rb,5.5161e-02_rb, &
     &3.6783e-02_rb,3.3457e-02_rb,3.2749e-02_rb/)
      kao(:, 5, 8,16) = (/ &
     &1.4389e-01_rb,1.2590e-01_rb,1.0792e-01_rb,8.9927e-02_rb,7.1955e-02_rb,5.3965e-02_rb, &
     &4.4963e-02_rb,5.0849e-02_rb,4.4095e-02_rb/)
      kao(:, 1, 9,16) = (/ &
     &1.0859e+00_rb,9.5014e-01_rb,8.1454e-01_rb,6.7879e-01_rb,5.4303e-01_rb,4.0724e-01_rb, &
     &2.7149e-01_rb,1.3574e-01_rb,5.1220e-06_rb/)
      kao(:, 2, 9,16) = (/ &
     &1.0768e+00_rb,9.4216e-01_rb,8.0759e-01_rb,6.7299e-01_rb,5.3840e-01_rb,4.0381e-01_rb, &
     &2.6917e-01_rb,1.3460e-01_rb,1.2021e-05_rb/)
      kao(:, 3, 9,16) = (/ &
     &1.0636e+00_rb,9.3068e-01_rb,7.9788e-01_rb,6.6477e-01_rb,5.3197e-01_rb,3.9888e-01_rb, &
     &2.6599e-01_rb,1.3299e-01_rb,2.0747e-05_rb/)
      kao(:, 4, 9,16) = (/ &
     &1.0462e+00_rb,9.1536e-01_rb,7.8477e-01_rb,6.5394e-01_rb,5.2317e-01_rb,3.9241e-01_rb, &
     &2.6164e-01_rb,1.3081e-01_rb,2.6757e-05_rb/)
      kao(:, 5, 9,16) = (/ &
     &1.0256e+00_rb,8.9712e-01_rb,7.6891e-01_rb,6.4089e-01_rb,5.1277e-01_rb,3.8454e-01_rb, &
     &2.5635e-01_rb,1.2820e-01_rb,3.3237e-05_rb/)
      kao(:, 1,10,16) = (/ &
     &9.2943e+00_rb,8.1330e+00_rb,6.9713e+00_rb,5.8089e+00_rb,4.6478e+00_rb,3.4621e+00_rb, &
     &2.3237e+00_rb,1.1620e+00_rb,2.5199e-06_rb/)
      kao(:, 2,10,16) = (/ &
     &9.1327e+00_rb,7.9905e+00_rb,6.8491e+00_rb,5.7077e+00_rb,4.5656e+00_rb,3.4245e+00_rb, &
     &2.2832e+00_rb,1.1416e+00_rb,6.1820e-06_rb/)
      kao(:, 3,10,16) = (/ &
     &9.0419e+00_rb,7.9114e+00_rb,6.7812e+00_rb,5.6501e+00_rb,4.5208e+00_rb,3.3897e+00_rb, &
     &2.2603e+00_rb,1.1303e+00_rb,1.2253e-05_rb/)
      kao(:, 4,10,16) = (/ &
     &8.9128e+00_rb,7.7986e+00_rb,6.6845e+00_rb,5.5708e+00_rb,4.4561e+00_rb,3.3428e+00_rb, &
     &2.2285e+00_rb,1.1142e+00_rb,1.7974e-05_rb/)
      kao(:, 5,10,16) = (/ &
     &8.7796e+00_rb,7.6831e+00_rb,6.5851e+00_rb,5.4876e+00_rb,4.3892e+00_rb,3.2921e+00_rb, &
     &2.1948e+00_rb,1.0975e+00_rb,2.4232e-05_rb/)
      kao(:, 1,11,16) = (/ &
     &2.7588e+01_rb,2.4141e+01_rb,2.0693e+01_rb,1.7244e+01_rb,1.3795e+01_rb,1.0345e+01_rb, &
     &6.8978e+00_rb,3.4484e+00_rb,2.0595e-06_rb/)
      kao(:, 2,11,16) = (/ &
     &2.6657e+01_rb,2.3323e+01_rb,1.9991e+01_rb,1.6660e+01_rb,1.3327e+01_rb,9.9952e+00_rb, &
     &6.6638e+00_rb,3.3322e+00_rb,4.7312e-06_rb/)
      kao(:, 3,11,16) = (/ &
     &2.5989e+01_rb,2.2740e+01_rb,1.9493e+01_rb,1.6242e+01_rb,1.2994e+01_rb,9.7462e+00_rb, &
     &6.4977e+00_rb,3.2488e+00_rb,8.7718e-06_rb/)
      kao(:, 4,11,16) = (/ &
     &2.5704e+01_rb,2.2492e+01_rb,1.9279e+01_rb,1.6066e+01_rb,1.2854e+01_rb,9.6403e+00_rb, &
     &6.4266e+00_rb,3.2130e+00_rb,1.4050e-05_rb/)
      kao(:, 5,11,16) = (/ &
     &2.5636e+01_rb,2.2431e+01_rb,1.9229e+01_rb,1.6023e+01_rb,1.2819e+01_rb,9.6138e+00_rb, &
     &6.4100e+00_rb,3.2046e+00_rb,1.9441e-05_rb/)
      kao(:, 1,12,16) = (/ &
     &5.2064e+01_rb,4.5551e+01_rb,3.9046e+01_rb,3.2542e+01_rb,2.6032e+01_rb,1.9524e+01_rb, &
     &1.3016e+01_rb,6.5078e+00_rb,1.7091e-06_rb/)
      kao(:, 2,12,16) = (/ &
     &5.0183e+01_rb,4.3912e+01_rb,3.7638e+01_rb,3.1363e+01_rb,2.5091e+01_rb,1.8819e+01_rb, &
     &1.2545e+01_rb,6.2729e+00_rb,3.7298e-06_rb/)
      kao(:, 3,12,16) = (/ &
     &4.8961e+01_rb,4.2840e+01_rb,3.6719e+01_rb,3.0598e+01_rb,2.4478e+01_rb,1.8358e+01_rb, &
     &1.2239e+01_rb,6.1197e+00_rb,6.4267e-06_rb/)
      kao(:, 4,12,16) = (/ &
     &4.8192e+01_rb,4.2165e+01_rb,3.6143e+01_rb,3.0118e+01_rb,2.4096e+01_rb,1.8071e+01_rb, &
     &1.2047e+01_rb,6.0236e+00_rb,1.0438e-05_rb/)
      kao(:, 5,12,16) = (/ &
     &4.7950e+01_rb,4.1951e+01_rb,3.5955e+01_rb,2.9964e+01_rb,2.3968e+01_rb,1.7977e+01_rb, &
     &1.1985e+01_rb,5.9924e+00_rb,1.5532e-05_rb/)
      kao(:, 1,13,16) = (/ &
     &7.9237e+01_rb,6.9320e+01_rb,5.9418e+01_rb,4.9515e+01_rb,3.9615e+01_rb,2.9712e+01_rb, &
     &1.9806e+01_rb,9.9046e+00_rb,1.3997e-06_rb/)
      kao(:, 2,13,16) = (/ &
     &7.6481e+01_rb,6.6913e+01_rb,5.7358e+01_rb,4.7795e+01_rb,3.8238e+01_rb,2.8679e+01_rb, &
     &1.9119e+01_rb,9.5593e+00_rb,2.8864e-06_rb/)
      kao(:, 3,13,16) = (/ &
     &7.4663e+01_rb,6.5336e+01_rb,5.5998e+01_rb,4.6664e+01_rb,3.7331e+01_rb,2.7999e+01_rb, &
     &1.8666e+01_rb,9.3324e+00_rb,4.8512e-06_rb/)
      kao(:, 4,13,16) = (/ &
     &7.3334e+01_rb,6.4170e+01_rb,5.5002e+01_rb,4.5830e+01_rb,3.6668e+01_rb,2.7499e+01_rb, &
     &1.8332e+01_rb,9.1667e+00_rb,7.8505e-06_rb/)
      kao(:, 5,13,16) = (/ &
     &7.2543e+01_rb,6.3471e+01_rb,5.4398e+01_rb,4.5335e+01_rb,3.6267e+01_rb,2.7201e+01_rb, &
     &1.8133e+01_rb,9.0671e+00_rb,1.1235e-05_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &6.2747e-02_rb,8.6102e-02_rb,1.1568e-01_rb,1.5208e-01_rb,1.9535e-01_rb/)
      kbo(:,14, 1) = (/ &
     &5.7861e-02_rb,7.9151e-02_rb,1.0594e-01_rb,1.3846e-01_rb,1.7643e-01_rb/)
      kbo(:,15, 1) = (/ &
     &5.3092e-02_rb,7.2482e-02_rb,9.6781e-02_rb,1.2588e-01_rb,1.5918e-01_rb/)
      kbo(:,16, 1) = (/ &
     &4.8457e-02_rb,6.6047e-02_rb,8.7760e-02_rb,1.1351e-01_rb,1.4288e-01_rb/)
      kbo(:,17, 1) = (/ &
     &4.3912e-02_rb,5.9676e-02_rb,7.8923e-02_rb,1.0151e-01_rb,1.2709e-01_rb/)
      kbo(:,18, 1) = (/ &
     &3.9426e-02_rb,5.3365e-02_rb,7.0341e-02_rb,8.9953e-02_rb,1.1230e-01_rb/)
      kbo(:,19, 1) = (/ &
     &3.5046e-02_rb,4.7312e-02_rb,6.2046e-02_rb,7.9063e-02_rb,9.8334e-02_rb/)
      kbo(:,20, 1) = (/ &
     &3.1115e-02_rb,4.1839e-02_rb,5.4636e-02_rb,6.9407e-02_rb,8.5975e-02_rb/)
      kbo(:,21, 1) = (/ &
     &2.7465e-02_rb,3.6785e-02_rb,4.7830e-02_rb,6.0600e-02_rb,7.4770e-02_rb/)
      kbo(:,22, 1) = (/ &
     &2.4593e-02_rb,3.2747e-02_rb,4.2384e-02_rb,5.3412e-02_rb,6.5545e-02_rb/)
      kbo(:,23, 1) = (/ &
     &2.1937e-02_rb,2.9046e-02_rb,3.7411e-02_rb,4.6873e-02_rb,5.7167e-02_rb/)
      kbo(:,24, 1) = (/ &
     &1.9516e-02_rb,2.5711e-02_rb,3.2910e-02_rb,4.0910e-02_rb,4.9576e-02_rb/)
      kbo(:,25, 1) = (/ &
     &1.7347e-02_rb,2.2706e-02_rb,2.8823e-02_rb,3.5568e-02_rb,4.2803e-02_rb/)
      kbo(:,26, 1) = (/ &
     &1.5431e-02_rb,2.0022e-02_rb,2.5209e-02_rb,3.0861e-02_rb,3.6863e-02_rb/)
      kbo(:,27, 1) = (/ &
     &1.3676e-02_rb,1.7588e-02_rb,2.1954e-02_rb,2.6656e-02_rb,3.1628e-02_rb/)
      kbo(:,28, 1) = (/ &
     &1.2072e-02_rb,1.5389e-02_rb,1.9040e-02_rb,2.2944e-02_rb,2.7092e-02_rb/)
      kbo(:,29, 1) = (/ &
     &1.0638e-02_rb,1.3440e-02_rb,1.6483e-02_rb,1.9764e-02_rb,2.3215e-02_rb/)
      kbo(:,30, 1) = (/ &
     &9.3452e-03_rb,1.1695e-02_rb,1.4256e-02_rb,1.7011e-02_rb,1.9889e-02_rb/)
      kbo(:,31, 1) = (/ &
     &8.1958e-03_rb,1.0179e-02_rb,1.2343e-02_rb,1.4653e-02_rb,1.7063e-02_rb/)
      kbo(:,32, 1) = (/ &
     &7.1787e-03_rb,8.8627e-03_rb,1.0685e-02_rb,1.2626e-02_rb,1.4630e-02_rb/)
      kbo(:,33, 1) = (/ &
     &6.2851e-03_rb,7.7078e-03_rb,9.2476e-03_rb,1.0876e-02_rb,1.2551e-02_rb/)
      kbo(:,34, 1) = (/ &
     &5.4503e-03_rb,6.6513e-03_rb,7.9437e-03_rb,9.3044e-03_rb,1.0718e-02_rb/)
      kbo(:,35, 1) = (/ &
     &4.6286e-03_rb,5.6362e-03_rb,6.7164e-03_rb,7.8605e-03_rb,9.0307e-03_rb/)
      kbo(:,36, 1) = (/ &
     &3.8439e-03_rb,4.6777e-03_rb,5.5784e-03_rb,6.5325e-03_rb,7.4907e-03_rb/)
      kbo(:,37, 1) = (/ &
     &3.0907e-03_rb,3.7733e-03_rb,4.5151e-03_rb,5.2952e-03_rb,6.0875e-03_rb/)
      kbo(:,38, 1) = (/ &
     &2.4803e-03_rb,3.0385e-03_rb,3.6485e-03_rb,4.2895e-03_rb,4.9493e-03_rb/)
      kbo(:,39, 1) = (/ &
     &1.9914e-03_rb,2.4489e-03_rb,2.9506e-03_rb,3.4809e-03_rb,4.0250e-03_rb/)
      kbo(:,40, 1) = (/ &
     &1.5591e-03_rb,1.9289e-03_rb,2.3368e-03_rb,2.7699e-03_rb,3.2181e-03_rb/)
      kbo(:,41, 1) = (/ &
     &1.2156e-03_rb,1.5132e-03_rb,1.8431e-03_rb,2.1975e-03_rb,2.5662e-03_rb/)
      kbo(:,42, 1) = (/ &
     &9.4642e-04_rb,1.1860e-03_rb,1.4531e-03_rb,1.7422e-03_rb,2.0436e-03_rb/)
      kbo(:,43, 1) = (/ &
     &7.2576e-04_rb,9.1670e-04_rb,1.1319e-03_rb,1.3659e-03_rb,1.6136e-03_rb/)
      kbo(:,44, 1) = (/ &
     &5.5191e-04_rb,7.0339e-04_rb,8.7651e-04_rb,1.0654e-03_rb,1.2677e-03_rb/)
      kbo(:,45, 1) = (/ &
     &4.1864e-04_rb,5.3928e-04_rb,6.7793e-04_rb,8.3043e-04_rb,9.9568e-04_rb/)
      kbo(:,46, 1) = (/ &
     &3.1490e-04_rb,4.0992e-04_rb,5.2032e-04_rb,6.4322e-04_rb,7.7812e-04_rb/)
      kbo(:,47, 1) = (/ &
     &2.3273e-04_rb,3.0659e-04_rb,3.9375e-04_rb,4.9208e-04_rb,6.0088e-04_rb/)
      kbo(:,48, 1) = (/ &
     &1.7128e-04_rb,2.2854e-04_rb,2.9725e-04_rb,3.7557e-04_rb,4.6327e-04_rb/)
      kbo(:,49, 1) = (/ &
     &1.2553e-04_rb,1.6987e-04_rb,2.2375e-04_rb,2.8623e-04_rb,3.5677e-04_rb/)
      kbo(:,50, 1) = (/ &
     &9.2117e-05_rb,1.2634e-04_rb,1.6842e-04_rb,2.1802e-04_rb,2.7404e-04_rb/)
      kbo(:,51, 1) = (/ &
     &6.7482e-05_rb,9.3783e-05_rb,1.2646e-04_rb,1.6541e-04_rb,2.1004e-04_rb/)
      kbo(:,52, 1) = (/ &
     &4.9294e-05_rb,6.9377e-05_rb,9.4622e-05_rb,1.2516e-04_rb,1.6083e-04_rb/)
      kbo(:,53, 1) = (/ &
     &3.5898e-05_rb,5.1131e-05_rb,7.0555e-05_rb,9.4518e-05_rb,1.2315e-04_rb/)
      kbo(:,54, 1) = (/ &
     &2.6237e-05_rb,3.7710e-05_rb,5.2593e-05_rb,7.1209e-05_rb,9.3663e-05_rb/)
      kbo(:,55, 1) = (/ &
     &1.9162e-05_rb,2.7753e-05_rb,3.9112e-05_rb,5.3467e-05_rb,7.0840e-05_rb/)
      kbo(:,56, 1) = (/ &
     &1.3954e-05_rb,2.0386e-05_rb,2.8986e-05_rb,3.9982e-05_rb,5.3468e-05_rb/)
      kbo(:,57, 1) = (/ &
     &1.0137e-05_rb,1.4950e-05_rb,2.1433e-05_rb,2.9844e-05_rb,4.0285e-05_rb/)
      kbo(:,58, 1) = (/ &
     &7.3955e-06_rb,1.0988e-05_rb,1.5849e-05_rb,2.2304e-05_rb,3.0355e-05_rb/)
      kbo(:,59, 1) = (/ &
     &5.7756e-06_rb,8.5841e-06_rb,1.2417e-05_rb,1.7454e-05_rb,2.3825e-05_rb/)
      kbo(:,13, 2) = (/ &
     &2.8941e-01_rb,3.7928e-01_rb,4.8728e-01_rb,6.1356e-01_rb,7.5720e-01_rb/)
      kbo(:,14, 2) = (/ &
     &2.7482e-01_rb,3.5907e-01_rb,4.5923e-01_rb,5.7444e-01_rb,7.0479e-01_rb/)
      kbo(:,15, 2) = (/ &
     &2.6012e-01_rb,3.3844e-01_rb,4.3050e-01_rb,5.3565e-01_rb,6.5360e-01_rb/)
      kbo(:,16, 2) = (/ &
     &2.4469e-01_rb,3.1749e-01_rb,4.0174e-01_rb,4.9749e-01_rb,6.0352e-01_rb/)
      kbo(:,17, 2) = (/ &
     &2.2805e-01_rb,2.9482e-01_rb,3.7163e-01_rb,4.5800e-01_rb,5.5300e-01_rb/)
      kbo(:,18, 2) = (/ &
     &2.1066e-01_rb,2.7121e-01_rb,3.4050e-01_rb,4.1816e-01_rb,5.0173e-01_rb/)
      kbo(:,19, 2) = (/ &
     &1.9258e-01_rb,2.4696e-01_rb,3.0899e-01_rb,3.7763e-01_rb,4.5019e-01_rb/)
      kbo(:,20, 2) = (/ &
     &1.7535e-01_rb,2.2386e-01_rb,2.7870e-01_rb,3.3833e-01_rb,4.0121e-01_rb/)
      kbo(:,21, 2) = (/ &
     &1.5823e-01_rb,2.0117e-01_rb,2.4903e-01_rb,3.0059e-01_rb,3.5459e-01_rb/)
      kbo(:,22, 2) = (/ &
     &1.4393e-01_rb,1.8176e-01_rb,2.2349e-01_rb,2.6801e-01_rb,3.1434e-01_rb/)
      kbo(:,23, 2) = (/ &
     &1.2993e-01_rb,1.6307e-01_rb,1.9923e-01_rb,2.3752e-01_rb,2.7733e-01_rb/)
      kbo(:,24, 2) = (/ &
     &1.1668e-01_rb,1.4559e-01_rb,1.7680e-01_rb,2.0966e-01_rb,2.4384e-01_rb/)
      kbo(:,25, 2) = (/ &
     &1.0441e-01_rb,1.2946e-01_rb,1.5637e-01_rb,1.8462e-01_rb,2.1368e-01_rb/)
      kbo(:,26, 2) = (/ &
     &9.3354e-02_rb,1.1505e-01_rb,1.3816e-01_rb,1.6244e-01_rb,1.8706e-01_rb/)
      kbo(:,27, 2) = (/ &
     &8.3171e-02_rb,1.0190e-01_rb,1.2182e-01_rb,1.4246e-01_rb,1.6301e-01_rb/)
      kbo(:,28, 2) = (/ &
     &7.3897e-02_rb,9.0049e-02_rb,1.0712e-01_rb,1.2442e-01_rb,1.4130e-01_rb/)
      kbo(:,29, 2) = (/ &
     &6.5628e-02_rb,7.9501e-02_rb,9.3908e-02_rb,1.0826e-01_rb,1.2190e-01_rb/)
      kbo(:,30, 2) = (/ &
     &5.8144e-02_rb,6.9960e-02_rb,8.1950e-02_rb,9.3660e-02_rb,1.0454e-01_rb/)
      kbo(:,31, 2) = (/ &
     &5.1413e-02_rb,6.1336e-02_rb,7.1221e-02_rb,8.0615e-02_rb,8.9347e-02_rb/)
      kbo(:,32, 2) = (/ &
     &4.5308e-02_rb,5.3538e-02_rb,6.1586e-02_rb,6.9140e-02_rb,7.6180e-02_rb/)
      kbo(:,33, 2) = (/ &
     &3.9776e-02_rb,4.6552e-02_rb,5.3085e-02_rb,5.9155e-02_rb,6.4885e-02_rb/)
      kbo(:,34, 2) = (/ &
     &3.4566e-02_rb,4.0142e-02_rb,4.5387e-02_rb,5.0362e-02_rb,5.5085e-02_rb/)
      kbo(:,35, 2) = (/ &
     &2.9501e-02_rb,3.4063e-02_rb,3.8371e-02_rb,4.2482e-02_rb,4.6355e-02_rb/)
      kbo(:,36, 2) = (/ &
     &2.4731e-02_rb,2.8480e-02_rb,3.2042e-02_rb,3.5445e-02_rb,3.8685e-02_rb/)
      kbo(:,37, 2) = (/ &
     &2.0227e-02_rb,2.3337e-02_rb,2.6311e-02_rb,2.9150e-02_rb,3.1875e-02_rb/)
      kbo(:,38, 2) = (/ &
     &1.6522e-02_rb,1.9118e-02_rb,2.1594e-02_rb,2.3970e-02_rb,2.6280e-02_rb/)
      kbo(:,39, 2) = (/ &
     &1.3506e-02_rb,1.5663e-02_rb,1.7734e-02_rb,1.9723e-02_rb,2.1666e-02_rb/)
      kbo(:,40, 2) = (/ &
     &1.0839e-02_rb,1.2642e-02_rb,1.4374e-02_rb,1.6039e-02_rb,1.7665e-02_rb/)
      kbo(:,41, 2) = (/ &
     &8.6687e-03_rb,1.0169e-02_rb,1.1614e-02_rb,1.3014e-02_rb,1.4376e-02_rb/)
      kbo(:,42, 2) = (/ &
     &6.9249e-03_rb,8.1709e-03_rb,9.3824e-03_rb,1.0556e-02_rb,1.1694e-02_rb/)
      kbo(:,43, 2) = (/ &
     &5.4677e-03_rb,6.5006e-03_rb,7.5154e-03_rb,8.4985e-03_rb,9.4530e-03_rb/)
      kbo(:,44, 2) = (/ &
     &4.2855e-03_rb,5.1437e-03_rb,5.9926e-03_rb,6.8145e-03_rb,7.6201e-03_rb/)
      kbo(:,45, 2) = (/ &
     &3.3515e-03_rb,4.0607e-03_rb,4.7699e-03_rb,5.4599e-03_rb,6.1332e-03_rb/)
      kbo(:,46, 2) = (/ &
     &2.6019e-03_rb,3.1870e-03_rb,3.7777e-03_rb,4.3556e-03_rb,4.9205e-03_rb/)
      kbo(:,47, 2) = (/ &
     &1.9902e-03_rb,2.4722e-03_rb,2.9627e-03_rb,3.4486e-03_rb,3.9210e-03_rb/)
      kbo(:,48, 2) = (/ &
     &1.5153e-03_rb,1.9102e-03_rb,2.3171e-03_rb,2.7244e-03_rb,3.1206e-03_rb/)
      kbo(:,49, 2) = (/ &
     &1.1469e-03_rb,1.4720e-03_rb,1.8085e-03_rb,2.1494e-03_rb,2.4830e-03_rb/)
      kbo(:,50, 2) = (/ &
     &8.6648e-04_rb,1.1333e-03_rb,1.4108e-03_rb,1.6953e-03_rb,1.9769e-03_rb/)
      kbo(:,51, 2) = (/ &
     &6.5125e-04_rb,8.6980e-04_rb,1.0984e-03_rb,1.3342e-03_rb,1.5715e-03_rb/)
      kbo(:,52, 2) = (/ &
     &4.8660e-04_rb,6.6498e-04_rb,8.5250e-04_rb,1.0475e-03_rb,1.2461e-03_rb/)
      kbo(:,53, 2) = (/ &
     &3.6091e-04_rb,5.0560e-04_rb,6.5947e-04_rb,8.2040e-04_rb,9.8603e-04_rb/)
      kbo(:,54, 2) = (/ &
     &2.6798e-04_rb,3.8317e-04_rb,5.0899e-04_rb,6.4130e-04_rb,7.7856e-04_rb/)
      kbo(:,55, 2) = (/ &
     &1.9837e-04_rb,2.8906e-04_rb,3.9148e-04_rb,4.9955e-04_rb,6.1247e-04_rb/)
      kbo(:,56, 2) = (/ &
     &1.4613e-04_rb,2.1685e-04_rb,3.0004e-04_rb,3.8773e-04_rb,4.8096e-04_rb/)
      kbo(:,57, 2) = (/ &
     &1.0691e-04_rb,1.6173e-04_rb,2.2873e-04_rb,3.0001e-04_rb,3.7637e-04_rb/)
      kbo(:,58, 2) = (/ &
     &7.8236e-05_rb,1.2069e-04_rb,1.7421e-04_rb,2.3200e-04_rb,2.9412e-04_rb/)
      kbo(:,59, 2) = (/ &
     &6.1826e-05_rb,9.5990e-05_rb,1.3927e-04_rb,1.8625e-04_rb,2.3654e-04_rb/)
      kbo(:,13, 3) = (/ &
     &1.1135e+00_rb,1.3641e+00_rb,1.6160e+00_rb,1.8652e+00_rb,2.1134e+00_rb/)
      kbo(:,14, 3) = (/ &
     &1.0304e+00_rb,1.2521e+00_rb,1.4746e+00_rb,1.6946e+00_rb,1.9141e+00_rb/)
      kbo(:,15, 3) = (/ &
     &9.5384e-01_rb,1.1492e+00_rb,1.3443e+00_rb,1.5379e+00_rb,1.7345e+00_rb/)
      kbo(:,16, 3) = (/ &
     &8.8007e-01_rb,1.0512e+00_rb,1.2220e+00_rb,1.3952e+00_rb,1.5691e+00_rb/)
      kbo(:,17, 3) = (/ &
     &8.0715e-01_rb,9.5537e-01_rb,1.1063e+00_rb,1.2596e+00_rb,1.4122e+00_rb/)
      kbo(:,18, 3) = (/ &
     &7.3500e-01_rb,8.6436e-01_rb,9.9708e-01_rb,1.1308e+00_rb,1.2638e+00_rb/)
      kbo(:,19, 3) = (/ &
     &6.6442e-01_rb,7.7800e-01_rb,8.9341e-01_rb,1.0094e+00_rb,1.1240e+00_rb/)
      kbo(:,20, 3) = (/ &
     &5.9870e-01_rb,6.9782e-01_rb,7.9815e-01_rb,8.9858e-01_rb,9.9720e-01_rb/)
      kbo(:,21, 3) = (/ &
     &5.3577e-01_rb,6.2165e-01_rb,7.0893e-01_rb,7.9562e-01_rb,8.8013e-01_rb/)
      kbo(:,22, 3) = (/ &
     &4.8056e-01_rb,5.5531e-01_rb,6.3060e-01_rb,7.0436e-01_rb,7.7687e-01_rb/)
      kbo(:,23, 3) = (/ &
     &4.2855e-01_rb,4.9296e-01_rb,5.5690e-01_rb,6.1984e-01_rb,6.8212e-01_rb/)
      kbo(:,24, 3) = (/ &
     &3.8024e-01_rb,4.3497e-01_rb,4.8911e-01_rb,5.4325e-01_rb,5.9676e-01_rb/)
      kbo(:,25, 3) = (/ &
     &3.3570e-01_rb,3.8197e-01_rb,4.2813e-01_rb,4.7450e-01_rb,5.2085e-01_rb/)
      kbo(:,26, 3) = (/ &
     &2.9535e-01_rb,3.3462e-01_rb,3.7428e-01_rb,4.1425e-01_rb,4.5431e-01_rb/)
      kbo(:,27, 3) = (/ &
     &2.5873e-01_rb,2.9239e-01_rb,3.2655e-01_rb,3.6110e-01_rb,3.9601e-01_rb/)
      kbo(:,28, 3) = (/ &
     &2.2605e-01_rb,2.5498e-01_rb,2.8445e-01_rb,3.1460e-01_rb,3.4506e-01_rb/)
      kbo(:,29, 3) = (/ &
     &1.9728e-01_rb,2.2221e-01_rb,2.4794e-01_rb,2.7429e-01_rb,3.0081e-01_rb/)
      kbo(:,30, 3) = (/ &
     &1.7195e-01_rb,1.9369e-01_rb,2.1631e-01_rb,2.3929e-01_rb,2.6198e-01_rb/)
      kbo(:,31, 3) = (/ &
     &1.5004e-01_rb,1.6915e-01_rb,1.8896e-01_rb,2.0886e-01_rb,2.2799e-01_rb/)
      kbo(:,32, 3) = (/ &
     &1.3116e-01_rb,1.4802e-01_rb,1.6521e-01_rb,1.8200e-01_rb,1.9806e-01_rb/)
      kbo(:,33, 3) = (/ &
     &1.1493e-01_rb,1.2958e-01_rb,1.4418e-01_rb,1.5826e-01_rb,1.7170e-01_rb/)
      kbo(:,34, 3) = (/ &
     &1.0026e-01_rb,1.1281e-01_rb,1.2510e-01_rb,1.3692e-01_rb,1.4804e-01_rb/)
      kbo(:,35, 3) = (/ &
     &8.6407e-02_rb,9.7015e-02_rb,1.0743e-01_rb,1.1729e-01_rb,1.2663e-01_rb/)
      kbo(:,36, 3) = (/ &
     &7.3396e-02_rb,8.2443e-02_rb,9.1165e-02_rb,9.9519e-02_rb,1.0753e-01_rb/)
      kbo(:,37, 3) = (/ &
     &6.1173e-02_rb,6.8860e-02_rb,7.6255e-02_rb,8.3361e-02_rb,9.0284e-02_rb/)
      kbo(:,38, 3) = (/ &
     &5.0910e-02_rb,5.7435e-02_rb,6.3715e-02_rb,6.9817e-02_rb,7.5733e-02_rb/)
      kbo(:,39, 3) = (/ &
     &4.2376e-02_rb,4.7913e-02_rb,5.3247e-02_rb,5.8502e-02_rb,6.3619e-02_rb/)
      kbo(:,40, 3) = (/ &
     &3.4748e-02_rb,3.9459e-02_rb,4.3993e-02_rb,4.8510e-02_rb,5.2956e-02_rb/)
      kbo(:,41, 3) = (/ &
     &2.8418e-02_rb,3.2417e-02_rb,3.6282e-02_rb,4.0161e-02_rb,4.3993e-02_rb/)
      kbo(:,42, 3) = (/ &
     &2.3210e-02_rb,2.6614e-02_rb,2.9916e-02_rb,3.3242e-02_rb,3.6540e-02_rb/)
      kbo(:,43, 3) = (/ &
     &1.8779e-02_rb,2.1673e-02_rb,2.4493e-02_rb,2.7336e-02_rb,3.0182e-02_rb/)
      kbo(:,44, 3) = (/ &
     &1.5102e-02_rb,1.7567e-02_rb,1.9972e-02_rb,2.2410e-02_rb,2.4858e-02_rb/)
      kbo(:,45, 3) = (/ &
     &1.2119e-02_rb,1.4212e-02_rb,1.6271e-02_rb,1.8344e-02_rb,2.0451e-02_rb/)
      kbo(:,46, 3) = (/ &
     &9.6639e-03_rb,1.1430e-02_rb,1.3190e-02_rb,1.4958e-02_rb,1.6759e-02_rb/)
      kbo(:,47, 3) = (/ &
     &7.6185e-03_rb,9.1001e-03_rb,1.0599e-02_rb,1.2098e-02_rb,1.3639e-02_rb/)
      kbo(:,48, 3) = (/ &
     &5.9842e-03_rb,7.2206e-03_rb,8.5005e-03_rb,9.7716e-03_rb,1.1086e-02_rb/)
      kbo(:,49, 3) = (/ &
     &4.6860e-03_rb,5.7121e-03_rb,6.7999e-03_rb,7.8773e-03_rb,8.9943e-03_rb/)
      kbo(:,50, 3) = (/ &
     &3.6738e-03_rb,4.5201e-03_rb,5.4386e-03_rb,6.3525e-03_rb,7.2951e-03_rb/)
      kbo(:,51, 3) = (/ &
     &2.8749e-03_rb,3.5695e-03_rb,4.3389e-03_rb,5.1158e-03_rb,5.9116e-03_rb/)
      kbo(:,52, 3) = (/ &
     &2.2412e-03_rb,2.8111e-03_rb,3.4548e-03_rb,4.1149e-03_rb,4.7853e-03_rb/)
      kbo(:,53, 3) = (/ &
     &1.7396e-03_rb,2.2062e-03_rb,2.7406e-03_rb,3.3024e-03_rb,3.8694e-03_rb/)
      kbo(:,54, 3) = (/ &
     &1.3505e-03_rb,1.7325e-03_rb,2.1715e-03_rb,2.6457e-03_rb,3.1233e-03_rb/)
      kbo(:,55, 3) = (/ &
     &1.0458e-03_rb,1.3574e-03_rb,1.7165e-03_rb,2.1138e-03_rb,2.5149e-03_rb/)
      kbo(:,56, 3) = (/ &
     &8.0583e-04_rb,1.0599e-03_rb,1.3530e-03_rb,1.6840e-03_rb,2.0192e-03_rb/)
      kbo(:,57, 3) = (/ &
     &6.1766e-04_rb,8.2479e-04_rb,1.0628e-03_rb,1.3371e-03_rb,1.6185e-03_rb/)
      kbo(:,58, 3) = (/ &
     &4.7327e-04_rb,6.4160e-04_rb,8.3562e-04_rb,1.0608e-03_rb,1.2972e-03_rb/)
      kbo(:,59, 3) = (/ &
     &3.8292e-04_rb,5.2222e-04_rb,6.8284e-04_rb,8.6897e-04_rb,1.0664e-03_rb/)
      kbo(:,13, 4) = (/ &
     &3.3749e+00_rb,3.6005e+00_rb,3.8557e+00_rb,4.1032e+00_rb,4.3385e+00_rb/)
      kbo(:,14, 4) = (/ &
     &2.9858e+00_rb,3.1916e+00_rb,3.4148e+00_rb,3.6458e+00_rb,3.8704e+00_rb/)
      kbo(:,15, 4) = (/ &
     &2.6360e+00_rb,2.8231e+00_rb,3.0273e+00_rb,3.2443e+00_rb,3.4520e+00_rb/)
      kbo(:,16, 4) = (/ &
     &2.3243e+00_rb,2.4974e+00_rb,2.6921e+00_rb,2.8866e+00_rb,3.0804e+00_rb/)
      kbo(:,17, 4) = (/ &
     &2.0501e+00_rb,2.2124e+00_rb,2.3895e+00_rb,2.5647e+00_rb,2.7458e+00_rb/)
      kbo(:,18, 4) = (/ &
     &1.8079e+00_rb,1.9578e+00_rb,2.1149e+00_rb,2.2757e+00_rb,2.4444e+00_rb/)
      kbo(:,19, 4) = (/ &
     &1.5916e+00_rb,1.7262e+00_rb,1.8683e+00_rb,2.0160e+00_rb,2.1705e+00_rb/)
      kbo(:,20, 4) = (/ &
     &1.3988e+00_rb,1.5220e+00_rb,1.6508e+00_rb,1.7854e+00_rb,1.9234e+00_rb/)
      kbo(:,21, 4) = (/ &
     &1.2283e+00_rb,1.3399e+00_rb,1.4559e+00_rb,1.5766e+00_rb,1.6985e+00_rb/)
      kbo(:,22, 4) = (/ &
     &1.0842e+00_rb,1.1844e+00_rb,1.2884e+00_rb,1.3958e+00_rb,1.5035e+00_rb/)
      kbo(:,23, 4) = (/ &
     &9.5535e-01_rb,1.0448e+00_rb,1.1388e+00_rb,1.2338e+00_rb,1.3273e+00_rb/)
      kbo(:,24, 4) = (/ &
     &8.3937e-01_rb,9.2038e-01_rb,1.0041e+00_rb,1.0871e+00_rb,1.1675e+00_rb/)
      kbo(:,25, 4) = (/ &
     &7.3697e-01_rb,8.0962e-01_rb,8.8296e-01_rb,9.5450e-01_rb,1.0251e+00_rb/)
      kbo(:,26, 4) = (/ &
     &6.4730e-01_rb,7.1109e-01_rb,7.7450e-01_rb,8.3674e-01_rb,9.0032e-01_rb/)
      kbo(:,27, 4) = (/ &
     &5.6756e-01_rb,6.2293e-01_rb,6.7780e-01_rb,7.3355e-01_rb,7.9083e-01_rb/)
      kbo(:,28, 4) = (/ &
     &4.9666e-01_rb,5.4463e-01_rb,5.9308e-01_rb,6.4333e-01_rb,6.9525e-01_rb/)
      kbo(:,29, 4) = (/ &
     &4.3427e-01_rb,4.7629e-01_rb,5.1987e-01_rb,5.6530e-01_rb,6.1288e-01_rb/)
      kbo(:,30, 4) = (/ &
     &3.7957e-01_rb,4.1709e-01_rb,4.5651e-01_rb,4.9797e-01_rb,5.4181e-01_rb/)
      kbo(:,31, 4) = (/ &
     &3.3226e-01_rb,3.6617e-01_rb,4.0201e-01_rb,4.4025e-01_rb,4.8083e-01_rb/)
      kbo(:,32, 4) = (/ &
     &2.9152e-01_rb,3.2232e-01_rb,3.5534e-01_rb,3.9102e-01_rb,4.2785e-01_rb/)
      kbo(:,33, 4) = (/ &
     &2.5658e-01_rb,2.8478e-01_rb,3.1566e-01_rb,3.4846e-01_rb,3.8205e-01_rb/)
      kbo(:,34, 4) = (/ &
     &2.2568e-01_rb,2.5200e-01_rb,2.8059e-01_rb,3.1045e-01_rb,3.4133e-01_rb/)
      kbo(:,35, 4) = (/ &
     &1.9721e-01_rb,2.2158e-01_rb,2.4755e-01_rb,2.7492e-01_rb,3.0313e-01_rb/)
      kbo(:,36, 4) = (/ &
     &1.7069e-01_rb,1.9278e-01_rb,2.1640e-01_rb,2.4134e-01_rb,2.6698e-01_rb/)
      kbo(:,37, 4) = (/ &
     &1.4514e-01_rb,1.6486e-01_rb,1.8617e-01_rb,2.0876e-01_rb,2.3216e-01_rb/)
      kbo(:,38, 4) = (/ &
     &1.2328e-01_rb,1.4085e-01_rb,1.6011e-01_rb,1.8057e-01_rb,2.0182e-01_rb/)
      kbo(:,39, 4) = (/ &
     &1.0472e-01_rb,1.2042e-01_rb,1.3775e-01_rb,1.5631e-01_rb,1.7570e-01_rb/)
      kbo(:,40, 4) = (/ &
     &8.7593e-02_rb,1.0140e-01_rb,1.1682e-01_rb,1.3356e-01_rb,1.5106e-01_rb/)
      kbo(:,41, 4) = (/ &
     &7.2991e-02_rb,8.5111e-02_rb,9.8743e-02_rb,1.1375e-01_rb,1.2963e-01_rb/)
      kbo(:,42, 4) = (/ &
     &6.0719e-02_rb,7.1311e-02_rb,8.3382e-02_rb,9.6810e-02_rb,1.1115e-01_rb/)
      kbo(:,43, 4) = (/ &
     &4.9944e-02_rb,5.9103e-02_rb,6.9670e-02_rb,8.1603e-02_rb,9.4469e-02_rb/)
      kbo(:,44, 4) = (/ &
     &4.0785e-02_rb,4.8637e-02_rb,5.7853e-02_rb,6.8379e-02_rb,7.9912e-02_rb/)
      kbo(:,45, 4) = (/ &
     &3.3215e-02_rb,3.9939e-02_rb,4.7926e-02_rb,5.7208e-02_rb,6.7537e-02_rb/)
      kbo(:,46, 4) = (/ &
     &2.6843e-02_rb,3.2560e-02_rb,3.9436e-02_rb,4.7558e-02_rb,5.6745e-02_rb/)
      kbo(:,47, 4) = (/ &
     &2.1428e-02_rb,2.6226e-02_rb,3.2052e-02_rb,3.9057e-02_rb,4.7184e-02_rb/)
      kbo(:,48, 4) = (/ &
     &1.7029e-02_rb,2.1031e-02_rb,2.5943e-02_rb,3.1986e-02_rb,3.9099e-02_rb/)
      kbo(:,49, 4) = (/ &
     &1.3477e-02_rb,1.6802e-02_rb,2.0926e-02_rb,2.6109e-02_rb,3.2294e-02_rb/)
      kbo(:,50, 4) = (/ &
     &1.0673e-02_rb,1.3423e-02_rb,1.6872e-02_rb,2.1279e-02_rb,2.6629e-02_rb/)
      kbo(:,51, 4) = (/ &
     &8.4370e-03_rb,1.0700e-02_rb,1.3568e-02_rb,1.7289e-02_rb,2.1889e-02_rb/)
      kbo(:,52, 4) = (/ &
     &6.6453e-03_rb,8.4949e-03_rb,1.0870e-02_rb,1.3987e-02_rb,1.7931e-02_rb/)
      kbo(:,53, 4) = (/ &
     &5.2146e-03_rb,6.7176e-03_rb,8.6804e-03_rb,1.1275e-02_rb,1.4637e-02_rb/)
      kbo(:,54, 4) = (/ &
     &4.1029e-03_rb,5.3246e-03_rb,6.9354e-03_rb,9.0873e-03_rb,1.1925e-02_rb/)
      kbo(:,55, 4) = (/ &
     &3.2267e-03_rb,4.2135e-03_rb,5.5289e-03_rb,7.2992e-03_rb,9.6811e-03_rb/)
      kbo(:,56, 4) = (/ &
     &2.5269e-03_rb,3.3219e-03_rb,4.3912e-03_rb,5.8434e-03_rb,7.8299e-03_rb/)
      kbo(:,57, 4) = (/ &
     &1.9715e-03_rb,2.6095e-03_rb,3.4757e-03_rb,4.6583e-03_rb,6.3020e-03_rb/)
      kbo(:,58, 4) = (/ &
     &1.5393e-03_rb,2.0522e-03_rb,2.7523e-03_rb,3.7145e-03_rb,5.0709e-03_rb/)
      kbo(:,59, 4) = (/ &
     &1.2601e-03_rb,1.6926e-03_rb,2.2864e-03_rb,3.1046e-03_rb,4.2734e-03_rb/)
      kbo(:,13, 5) = (/ &
     &6.9559e+00_rb,7.1944e+00_rb,7.3778e+00_rb,7.6168e+00_rb,7.8840e+00_rb/)
      kbo(:,14, 5) = (/ &
     &6.1197e+00_rb,6.3419e+00_rb,6.5357e+00_rb,6.7535e+00_rb,7.0053e+00_rb/)
      kbo(:,15, 5) = (/ &
     &5.3579e+00_rb,5.5744e+00_rb,5.7766e+00_rb,5.9767e+00_rb,6.2167e+00_rb/)
      kbo(:,16, 5) = (/ &
     &4.6816e+00_rb,4.8936e+00_rb,5.0862e+00_rb,5.2967e+00_rb,5.5327e+00_rb/)
      kbo(:,17, 5) = (/ &
     &4.0899e+00_rb,4.2917e+00_rb,4.4812e+00_rb,4.7025e+00_rb,4.9201e+00_rb/)
      kbo(:,18, 5) = (/ &
     &3.5771e+00_rb,3.7678e+00_rb,3.9593e+00_rb,4.1715e+00_rb,4.3722e+00_rb/)
      kbo(:,19, 5) = (/ &
     &3.1364e+00_rb,3.3176e+00_rb,3.5013e+00_rb,3.6935e+00_rb,3.8830e+00_rb/)
      kbo(:,20, 5) = (/ &
     &2.7593e+00_rb,2.9230e+00_rb,3.0933e+00_rb,3.2687e+00_rb,3.4499e+00_rb/)
      kbo(:,21, 5) = (/ &
     &2.4256e+00_rb,2.5724e+00_rb,2.7298e+00_rb,2.8928e+00_rb,3.0638e+00_rb/)
      kbo(:,22, 5) = (/ &
     &2.1368e+00_rb,2.2729e+00_rb,2.4183e+00_rb,2.5689e+00_rb,2.7252e+00_rb/)
      kbo(:,23, 5) = (/ &
     &1.8817e+00_rb,2.0092e+00_rb,2.1410e+00_rb,2.2786e+00_rb,2.4217e+00_rb/)
      kbo(:,24, 5) = (/ &
     &1.6588e+00_rb,1.7749e+00_rb,1.8947e+00_rb,2.0208e+00_rb,2.1527e+00_rb/)
      kbo(:,25, 5) = (/ &
     &1.4616e+00_rb,1.5668e+00_rb,1.6777e+00_rb,1.7944e+00_rb,1.9128e+00_rb/)
      kbo(:,26, 5) = (/ &
     &1.2875e+00_rb,1.3850e+00_rb,1.4881e+00_rb,1.5943e+00_rb,1.7005e+00_rb/)
      kbo(:,27, 5) = (/ &
     &1.1342e+00_rb,1.2250e+00_rb,1.3195e+00_rb,1.4155e+00_rb,1.5121e+00_rb/)
      kbo(:,28, 5) = (/ &
     &1.0000e+00_rb,1.0834e+00_rb,1.1694e+00_rb,1.2569e+00_rb,1.3467e+00_rb/)
      kbo(:,29, 5) = (/ &
     &8.8272e-01_rb,9.5878e-01_rb,1.0375e+00_rb,1.1189e+00_rb,1.2039e+00_rb/)
      kbo(:,30, 5) = (/ &
     &7.8028e-01_rb,8.4986e-01_rb,9.2253e-01_rb,9.9927e-01_rb,1.0809e+00_rb/)
      kbo(:,31, 5) = (/ &
     &6.9115e-01_rb,7.5536e-01_rb,8.2388e-01_rb,8.9787e-01_rb,9.7590e-01_rb/)
      kbo(:,32, 5) = (/ &
     &6.1412e-01_rb,6.7446e-01_rb,7.4048e-01_rb,8.1095e-01_rb,8.8670e-01_rb/)
      kbo(:,33, 5) = (/ &
     &5.4810e-01_rb,6.0636e-01_rb,6.6911e-01_rb,7.3734e-01_rb,8.1091e-01_rb/)
      kbo(:,34, 5) = (/ &
     &4.9071e-01_rb,5.4601e-01_rb,6.0648e-01_rb,6.7271e-01_rb,7.4441e-01_rb/)
      kbo(:,35, 5) = (/ &
     &4.3693e-01_rb,4.8943e-01_rb,5.4769e-01_rb,6.1149e-01_rb,6.8198e-01_rb/)
      kbo(:,36, 5) = (/ &
     &3.8551e-01_rb,4.3547e-01_rb,4.9121e-01_rb,5.5301e-01_rb,6.2109e-01_rb/)
      kbo(:,37, 5) = (/ &
     &3.3480e-01_rb,3.8150e-01_rb,4.3408e-01_rb,4.9278e-01_rb,5.5808e-01_rb/)
      kbo(:,38, 5) = (/ &
     &2.9081e-01_rb,3.3450e-01_rb,3.8398e-01_rb,4.3987e-01_rb,5.0248e-01_rb/)
      kbo(:,39, 5) = (/ &
     &2.5305e-01_rb,2.9380e-01_rb,3.4050e-01_rb,3.9390e-01_rb,4.5364e-01_rb/)
      kbo(:,40, 5) = (/ &
     &2.1671e-01_rb,2.5418e-01_rb,2.9763e-01_rb,3.4779e-01_rb,4.0458e-01_rb/)
      kbo(:,41, 5) = (/ &
     &1.8493e-01_rb,2.1926e-01_rb,2.5954e-01_rb,3.0662e-01_rb,3.6037e-01_rb/)
      kbo(:,42, 5) = (/ &
     &1.5752e-01_rb,1.8888e-01_rb,2.2625e-01_rb,2.7031e-01_rb,3.2124e-01_rb/)
      kbo(:,43, 5) = (/ &
     &1.3255e-01_rb,1.6078e-01_rb,1.9491e-01_rb,2.3576e-01_rb,2.8350e-01_rb/)
      kbo(:,44, 5) = (/ &
     &1.1061e-01_rb,1.3578e-01_rb,1.6666e-01_rb,2.0425e-01_rb,2.4865e-01_rb/)
      kbo(:,45, 5) = (/ &
     &9.1951e-02_rb,1.1424e-01_rb,1.4202e-01_rb,1.7639e-01_rb,2.1757e-01_rb/)
      kbo(:,46, 5) = (/ &
     &7.5781e-02_rb,9.5221e-02_rb,1.1993e-01_rb,1.5099e-01_rb,1.8901e-01_rb/)
      kbo(:,47, 5) = (/ &
     &6.1487e-02_rb,7.8164e-02_rb,9.9731e-02_rb,1.2737e-01_rb,1.6179e-01_rb/)
      kbo(:,48, 5) = (/ &
     &4.9580e-02_rb,6.3770e-02_rb,8.2434e-02_rb,1.0684e-01_rb,1.3786e-01_rb/)
      kbo(:,49, 5) = (/ &
     &3.9738e-02_rb,5.1709e-02_rb,6.7733e-02_rb,8.9073e-02_rb,1.1686e-01_rb/)
      kbo(:,50, 5) = (/ &
     &3.1862e-02_rb,4.1943e-02_rb,5.5637e-02_rb,7.4270e-02_rb,9.9031e-02_rb/)
      kbo(:,51, 5) = (/ &
     &2.5469e-02_rb,3.3911e-02_rb,4.5561e-02_rb,6.1725e-02_rb,8.3692e-02_rb/)
      kbo(:,52, 5) = (/ &
     &2.0254e-02_rb,2.7266e-02_rb,3.7097e-02_rb,5.1018e-02_rb,7.0332e-02_rb/)
      kbo(:,53, 5) = (/ &
     &1.6008e-02_rb,2.1779e-02_rb,3.0012e-02_rb,4.1907e-02_rb,5.8779e-02_rb/)
      kbo(:,54, 5) = (/ &
     &1.2684e-02_rb,1.7422e-02_rb,2.4313e-02_rb,3.4417e-02_rb,4.9062e-02_rb/)
      kbo(:,55, 5) = (/ &
     &1.0038e-02_rb,1.3916e-02_rb,1.9644e-02_rb,2.8164e-02_rb,4.0795e-02_rb/)
      kbo(:,56, 5) = (/ &
     &7.9079e-03_rb,1.1053e-02_rb,1.5781e-02_rb,2.2925e-02_rb,3.3750e-02_rb/)
      kbo(:,57, 5) = (/ &
     &6.2010e-03_rb,8.7295e-03_rb,1.2598e-02_rb,1.8552e-02_rb,2.7742e-02_rb/)
      kbo(:,58, 5) = (/ &
     &4.8646e-03_rb,6.8966e-03_rb,1.0058e-02_rb,1.5005e-02_rb,2.2774e-02_rb/)
      kbo(:,59, 5) = (/ &
     &4.0331e-03_rb,5.7872e-03_rb,8.5542e-03_rb,1.2975e-02_rb,2.0014e-02_rb/)
      kbo(:,13, 6) = (/ &
     &1.3874e+01_rb,1.4054e+01_rb,1.4175e+01_rb,1.4196e+01_rb,1.4254e+01_rb/)
      kbo(:,14, 6) = (/ &
     &1.2387e+01_rb,1.2564e+01_rb,1.2698e+01_rb,1.2762e+01_rb,1.2781e+01_rb/)
      kbo(:,15, 6) = (/ &
     &1.0973e+01_rb,1.1145e+01_rb,1.1274e+01_rb,1.1365e+01_rb,1.1424e+01_rb/)
      kbo(:,16, 6) = (/ &
     &9.6521e+00_rb,9.8053e+00_rb,9.9370e+00_rb,1.0047e+01_rb,1.0150e+01_rb/)
      kbo(:,17, 6) = (/ &
     &8.4222e+00_rb,8.5719e+00_rb,8.7195e+00_rb,8.8488e+00_rb,9.0159e+00_rb/)
      kbo(:,18, 6) = (/ &
     &7.3090e+00_rb,7.4668e+00_rb,7.6322e+00_rb,7.7932e+00_rb,8.0086e+00_rb/)
      kbo(:,19, 6) = (/ &
     &6.3279e+00_rb,6.4985e+00_rb,6.6813e+00_rb,6.8756e+00_rb,7.1176e+00_rb/)
      kbo(:,20, 6) = (/ &
     &5.4862e+00_rb,5.6719e+00_rb,5.8682e+00_rb,6.0850e+00_rb,6.3337e+00_rb/)
      kbo(:,21, 6) = (/ &
     &4.7753e+00_rb,4.9682e+00_rb,5.1665e+00_rb,5.3943e+00_rb,5.6406e+00_rb/)
      kbo(:,22, 6) = (/ &
     &4.1854e+00_rb,4.3703e+00_rb,4.5713e+00_rb,4.7990e+00_rb,5.0439e+00_rb/)
      kbo(:,23, 6) = (/ &
     &3.6767e+00_rb,3.8508e+00_rb,4.0542e+00_rb,4.2776e+00_rb,4.5191e+00_rb/)
      kbo(:,24, 6) = (/ &
     &3.2355e+00_rb,3.4077e+00_rb,3.6072e+00_rb,3.8245e+00_rb,4.0534e+00_rb/)
      kbo(:,25, 6) = (/ &
     &2.8580e+00_rb,3.0303e+00_rb,3.2220e+00_rb,3.4257e+00_rb,3.6444e+00_rb/)
      kbo(:,26, 6) = (/ &
     &2.5405e+00_rb,2.7078e+00_rb,2.8872e+00_rb,3.0816e+00_rb,3.2880e+00_rb/)
      kbo(:,27, 6) = (/ &
     &2.2677e+00_rb,2.4258e+00_rb,2.5973e+00_rb,2.7818e+00_rb,2.9766e+00_rb/)
      kbo(:,28, 6) = (/ &
     &2.0273e+00_rb,2.1803e+00_rb,2.3462e+00_rb,2.5211e+00_rb,2.7040e+00_rb/)
      kbo(:,29, 6) = (/ &
     &1.8190e+00_rb,1.9689e+00_rb,2.1279e+00_rb,2.2941e+00_rb,2.4662e+00_rb/)
      kbo(:,30, 6) = (/ &
     &1.6397e+00_rb,1.7844e+00_rb,1.9371e+00_rb,2.0958e+00_rb,2.2590e+00_rb/)
      kbo(:,31, 6) = (/ &
     &1.4852e+00_rb,1.6249e+00_rb,1.7714e+00_rb,1.9226e+00_rb,2.0813e+00_rb/)
      kbo(:,32, 6) = (/ &
     &1.3517e+00_rb,1.4862e+00_rb,1.6264e+00_rb,1.7745e+00_rb,1.9335e+00_rb/)
      kbo(:,33, 6) = (/ &
     &1.2364e+00_rb,1.3651e+00_rb,1.5027e+00_rb,1.6513e+00_rb,1.8098e+00_rb/)
      kbo(:,34, 6) = (/ &
     &1.1317e+00_rb,1.2571e+00_rb,1.3947e+00_rb,1.5426e+00_rb,1.7001e+00_rb/)
      kbo(:,35, 6) = (/ &
     &1.0305e+00_rb,1.1540e+00_rb,1.2902e+00_rb,1.4368e+00_rb,1.5937e+00_rb/)
      kbo(:,36, 6) = (/ &
     &9.3137e-01_rb,1.0526e+00_rb,1.1856e+00_rb,1.3299e+00_rb,1.4860e+00_rb/)
      kbo(:,37, 6) = (/ &
     &8.2931e-01_rb,9.4564e-01_rb,1.0739e+00_rb,1.2152e+00_rb,1.3682e+00_rb/)
      kbo(:,38, 6) = (/ &
     &7.3906e-01_rb,8.5034e-01_rb,9.7478e-01_rb,1.1125e+00_rb,1.2627e+00_rb/)
      kbo(:,39, 6) = (/ &
     &6.6006e-01_rb,7.6663e-01_rb,8.8732e-01_rb,1.0214e+00_rb,1.1704e+00_rb/)
      kbo(:,40, 6) = (/ &
     &5.8074e-01_rb,6.8151e-01_rb,7.9698e-01_rb,9.2673e-01_rb,1.0719e+00_rb/)
      kbo(:,41, 6) = (/ &
     &5.0970e-01_rb,6.0467e-01_rb,7.1469e-01_rb,8.3962e-01_rb,9.8097e-01_rb/)
      kbo(:,42, 6) = (/ &
     &4.4691e-01_rb,5.3653e-01_rb,6.4112e-01_rb,7.6136e-01_rb,8.9846e-01_rb/)
      kbo(:,43, 6) = (/ &
     &3.8720e-01_rb,4.7067e-01_rb,5.6921e-01_rb,6.8346e-01_rb,8.1541e-01_rb/)
      kbo(:,44, 6) = (/ &
     &3.3270e-01_rb,4.0984e-01_rb,5.0209e-01_rb,6.1017e-01_rb,7.3637e-01_rb/)
      kbo(:,45, 6) = (/ &
     &2.8475e-01_rb,3.5595e-01_rb,4.4190e-01_rb,5.4378e-01_rb,6.6410e-01_rb/)
      kbo(:,46, 6) = (/ &
     &2.4137e-01_rb,3.0640e-01_rb,3.8581e-01_rb,4.8130e-01_rb,5.9507e-01_rb/)
      kbo(:,47, 6) = (/ &
     &2.0121e-01_rb,2.5952e-01_rb,3.3199e-01_rb,4.2038e-01_rb,5.2702e-01_rb/)
      kbo(:,48, 6) = (/ &
     &1.6644e-01_rb,2.1836e-01_rb,2.8412e-01_rb,3.6558e-01_rb,4.6520e-01_rb/)
      kbo(:,49, 6) = (/ &
     &1.3664e-01_rb,1.8247e-01_rb,2.4166e-01_rb,3.1637e-01_rb,4.0912e-01_rb/)
      kbo(:,50, 6) = (/ &
     &1.1203e-01_rb,1.5228e-01_rb,2.0551e-01_rb,2.7399e-01_rb,3.6023e-01_rb/)
      kbo(:,51, 6) = (/ &
     &9.1424e-02_rb,1.2652e-01_rb,1.7413e-01_rb,2.3648e-01_rb,3.1642e-01_rb/)
      kbo(:,52, 6) = (/ &
     &7.4033e-02_rb,1.0433e-01_rb,1.4653e-01_rb,2.0289e-01_rb,2.7670e-01_rb/)
      kbo(:,53, 6) = (/ &
     &5.9468e-02_rb,8.5397e-02_rb,1.2237e-01_rb,1.7301e-01_rb,2.4059e-01_rb/)
      kbo(:,54, 6) = (/ &
     &4.7831e-02_rb,6.9932e-02_rb,1.0222e-01_rb,1.4759e-01_rb,2.0954e-01_rb/)
      kbo(:,55, 6) = (/ &
     &3.8375e-02_rb,5.7032e-02_rb,8.5062e-02_rb,1.2546e-01_rb,1.8204e-01_rb/)
      kbo(:,56, 6) = (/ &
     &3.0600e-02_rb,4.6150e-02_rb,7.0262e-02_rb,1.0594e-01_rb,1.5707e-01_rb/)
      kbo(:,57, 6) = (/ &
     &2.4221e-02_rb,3.7043e-02_rb,5.7569e-02_rb,8.8786e-02_rb,1.3465e-01_rb/)
      kbo(:,58, 6) = (/ &
     &1.9170e-02_rb,2.9713e-02_rb,4.7101e-02_rb,7.4302e-02_rb,1.1534e-01_rb/)
      kbo(:,59, 6) = (/ &
     &1.6314e-02_rb,2.5760e-02_rb,4.1528e-02_rb,6.6912e-02_rb,1.0596e-01_rb/)
      kbo(:,13, 7) = (/ &
     &2.6383e+01_rb,2.6399e+01_rb,2.6364e+01_rb,2.6286e+01_rb,2.6068e+01_rb/)
      kbo(:,14, 7) = (/ &
     &2.4391e+01_rb,2.4435e+01_rb,2.4445e+01_rb,2.4411e+01_rb,2.4344e+01_rb/)
      kbo(:,15, 7) = (/ &
     &2.2267e+01_rb,2.2343e+01_rb,2.2391e+01_rb,2.2420e+01_rb,2.2406e+01_rb/)
      kbo(:,16, 7) = (/ &
     &2.0101e+01_rb,2.0222e+01_rb,2.0311e+01_rb,2.0370e+01_rb,2.0373e+01_rb/)
      kbo(:,17, 7) = (/ &
     &1.7987e+01_rb,1.8130e+01_rb,1.8242e+01_rb,1.8316e+01_rb,1.8338e+01_rb/)
      kbo(:,18, 7) = (/ &
     &1.5956e+01_rb,1.6114e+01_rb,1.6238e+01_rb,1.6337e+01_rb,1.6400e+01_rb/)
      kbo(:,19, 7) = (/ &
     &1.4050e+01_rb,1.4210e+01_rb,1.4354e+01_rb,1.4495e+01_rb,1.4623e+01_rb/)
      kbo(:,20, 7) = (/ &
     &1.2287e+01_rb,1.2462e+01_rb,1.2644e+01_rb,1.2837e+01_rb,1.3036e+01_rb/)
      kbo(:,21, 7) = (/ &
     &1.0697e+01_rb,1.0904e+01_rb,1.1136e+01_rb,1.1376e+01_rb,1.1647e+01_rb/)
      kbo(:,22, 7) = (/ &
     &9.3216e+00_rb,9.5704e+00_rb,9.8418e+00_rb,1.0134e+01_rb,1.0464e+01_rb/)
      kbo(:,23, 7) = (/ &
     &8.1552e+00_rb,8.4381e+00_rb,8.7358e+00_rb,9.0546e+00_rb,9.4274e+00_rb/)
      kbo(:,24, 7) = (/ &
     &7.1742e+00_rb,7.4714e+00_rb,7.7800e+00_rb,8.1293e+00_rb,8.5227e+00_rb/)
      kbo(:,25, 7) = (/ &
     &6.3453e+00_rb,6.6371e+00_rb,6.9572e+00_rb,7.3300e+00_rb,7.7371e+00_rb/)
      kbo(:,26, 7) = (/ &
     &5.6325e+00_rb,5.9213e+00_rb,6.2619e+00_rb,6.6436e+00_rb,7.0622e+00_rb/)
      kbo(:,27, 7) = (/ &
     &5.0177e+00_rb,5.3180e+00_rb,5.6685e+00_rb,6.0556e+00_rb,6.4791e+00_rb/)
      kbo(:,28, 7) = (/ &
     &4.5006e+00_rb,4.8114e+00_rb,5.1630e+00_rb,5.5512e+00_rb,5.9725e+00_rb/)
      kbo(:,29, 7) = (/ &
     &4.0763e+00_rb,4.3869e+00_rb,4.7357e+00_rb,5.1215e+00_rb,5.5415e+00_rb/)
      kbo(:,30, 7) = (/ &
     &3.7197e+00_rb,4.0279e+00_rb,4.3734e+00_rb,4.7560e+00_rb,5.1758e+00_rb/)
      kbo(:,31, 7) = (/ &
     &3.4217e+00_rb,3.7276e+00_rb,4.0700e+00_rb,4.4513e+00_rb,4.8709e+00_rb/)
      kbo(:,32, 7) = (/ &
     &3.1725e+00_rb,3.4760e+00_rb,3.8179e+00_rb,4.1999e+00_rb,4.6169e+00_rb/)
      kbo(:,33, 7) = (/ &
     &2.9640e+00_rb,3.2682e+00_rb,3.6118e+00_rb,3.9912e+00_rb,4.4110e+00_rb/)
      kbo(:,34, 7) = (/ &
     &2.7815e+00_rb,3.0881e+00_rb,3.4286e+00_rb,3.8090e+00_rb,4.2366e+00_rb/)
      kbo(:,35, 7) = (/ &
     &2.6027e+00_rb,2.9073e+00_rb,3.2445e+00_rb,3.6269e+00_rb,4.0627e+00_rb/)
      kbo(:,36, 7) = (/ &
     &2.4195e+00_rb,2.7195e+00_rb,3.0535e+00_rb,3.4362e+00_rb,3.8766e+00_rb/)
      kbo(:,37, 7) = (/ &
     &2.2155e+00_rb,2.5099e+00_rb,2.8373e+00_rb,3.2150e+00_rb,3.6504e+00_rb/)
      kbo(:,38, 7) = (/ &
     &2.0317e+00_rb,2.3214e+00_rb,2.6457e+00_rb,3.0187e+00_rb,3.4497e+00_rb/)
      kbo(:,39, 7) = (/ &
     &1.8675e+00_rb,2.1535e+00_rb,2.4775e+00_rb,2.8467e+00_rb,3.2732e+00_rb/)
      kbo(:,40, 7) = (/ &
     &1.6933e+00_rb,1.9726e+00_rb,2.2917e+00_rb,2.6527e+00_rb,3.0690e+00_rb/)
      kbo(:,41, 7) = (/ &
     &1.5321e+00_rb,1.8045e+00_rb,2.1184e+00_rb,2.4731e+00_rb,2.8788e+00_rb/)
      kbo(:,42, 7) = (/ &
     &1.3858e+00_rb,1.6512e+00_rb,1.9598e+00_rb,2.3096e+00_rb,2.7061e+00_rb/)
      kbo(:,43, 7) = (/ &
     &1.2395e+00_rb,1.4953e+00_rb,1.7959e+00_rb,2.1389e+00_rb,2.5250e+00_rb/)
      kbo(:,44, 7) = (/ &
     &1.1006e+00_rb,1.3456e+00_rb,1.6365e+00_rb,1.9712e+00_rb,2.3479e+00_rb/)
      kbo(:,45, 7) = (/ &
     &9.7411e-01_rb,1.2078e+00_rb,1.4892e+00_rb,1.8150e+00_rb,2.1832e+00_rb/)
      kbo(:,46, 7) = (/ &
     &8.5489e-01_rb,1.0759e+00_rb,1.3456e+00_rb,1.6612e+00_rb,2.0197e+00_rb/)
      kbo(:,47, 7) = (/ &
     &7.3930e-01_rb,9.4494e-01_rb,1.2000e+00_rb,1.5026e+00_rb,1.8496e+00_rb/)
      kbo(:,48, 7) = (/ &
     &6.3547e-01_rb,8.2535e-01_rb,1.0653e+00_rb,1.3544e+00_rb,1.6889e+00_rb/)
      kbo(:,49, 7) = (/ &
     &5.4241e-01_rb,7.1674e-01_rb,9.4139e-01_rb,1.2160e+00_rb,1.5375e+00_rb/)
      kbo(:,50, 7) = (/ &
     &4.6304e-01_rb,6.2300e-01_rb,8.3217e-01_rb,1.0925e+00_rb,1.4012e+00_rb/)
      kbo(:,51, 7) = (/ &
     &3.9371e-01_rb,5.3995e-01_rb,7.3373e-01_rb,9.7980e-01_rb,1.2748e+00_rb/)
      kbo(:,52, 7) = (/ &
     &3.3220e-01_rb,4.6493e-01_rb,6.4396e-01_rb,8.7508e-01_rb,1.1558e+00_rb/)
      kbo(:,53, 7) = (/ &
     &2.7778e-01_rb,3.9741e-01_rb,5.6181e-01_rb,7.7785e-01_rb,1.0434e+00_rb/)
      kbo(:,54, 7) = (/ &
     &2.3274e-01_rb,3.4050e-01_rb,4.9090e-01_rb,6.9261e-01_rb,9.4387e-01_rb/)
      kbo(:,55, 7) = (/ &
     &1.9435e-01_rb,2.9105e-01_rb,4.2805e-01_rb,6.1562e-01_rb,8.5235e-01_rb/)
      kbo(:,56, 7) = (/ &
     &1.6083e-01_rb,2.4693e-01_rb,3.7158e-01_rb,5.4415e-01_rb,7.6669e-01_rb/)
      kbo(:,57, 7) = (/ &
     &1.3183e-01_rb,2.0770e-01_rb,3.2034e-01_rb,4.7818e-01_rb,6.8649e-01_rb/)
      kbo(:,58, 7) = (/ &
     &1.0786e-01_rb,1.7445e-01_rb,2.7618e-01_rb,4.2058e-01_rb,6.1481e-01_rb/)
      kbo(:,59, 7) = (/ &
     &9.5778e-02_rb,1.5879e-01_rb,2.5699e-01_rb,3.9684e-01_rb,5.8283e-01_rb/)
      kbo(:,13, 8) = (/ &
     &5.1036e+01_rb,5.0460e+01_rb,4.9876e+01_rb,4.9272e+01_rb,4.8645e+01_rb/)
      kbo(:,14, 8) = (/ &
     &4.9659e+01_rb,4.9240e+01_rb,4.8789e+01_rb,4.8317e+01_rb,4.7813e+01_rb/)
      kbo(:,15, 8) = (/ &
     &4.7836e+01_rb,4.7533e+01_rb,4.7179e+01_rb,4.6807e+01_rb,4.6408e+01_rb/)
      kbo(:,16, 8) = (/ &
     &4.5523e+01_rb,4.5307e+01_rb,4.5074e+01_rb,4.4798e+01_rb,4.4497e+01_rb/)
      kbo(:,17, 8) = (/ &
     &4.2751e+01_rb,4.2659e+01_rb,4.2523e+01_rb,4.2351e+01_rb,4.2128e+01_rb/)
      kbo(:,18, 8) = (/ &
     &3.9675e+01_rb,3.9678e+01_rb,3.9613e+01_rb,3.9512e+01_rb,3.9377e+01_rb/)
      kbo(:,19, 8) = (/ &
     &3.6385e+01_rb,3.6442e+01_rb,3.6467e+01_rb,3.6453e+01_rb,3.6385e+01_rb/)
      kbo(:,20, 8) = (/ &
     &3.2988e+01_rb,3.3119e+01_rb,3.3207e+01_rb,3.3253e+01_rb,3.3266e+01_rb/)
      kbo(:,21, 8) = (/ &
     &2.9629e+01_rb,2.9807e+01_rb,2.9950e+01_rb,3.0074e+01_rb,3.0195e+01_rb/)
      kbo(:,22, 8) = (/ &
     &2.6389e+01_rb,2.6615e+01_rb,2.6833e+01_rb,2.7054e+01_rb,2.7281e+01_rb/)
      kbo(:,23, 8) = (/ &
     &2.3355e+01_rb,2.3644e+01_rb,2.3948e+01_rb,2.4296e+01_rb,2.4660e+01_rb/)
      kbo(:,24, 8) = (/ &
     &2.0595e+01_rb,2.0963e+01_rb,2.1391e+01_rb,2.1861e+01_rb,2.2380e+01_rb/)
      kbo(:,25, 8) = (/ &
     &1.8159e+01_rb,1.8637e+01_rb,1.9183e+01_rb,1.9778e+01_rb,2.0439e+01_rb/)
      kbo(:,26, 8) = (/ &
     &1.6089e+01_rb,1.6677e+01_rb,1.7319e+01_rb,1.8032e+01_rb,1.8814e+01_rb/)
      kbo(:,27, 8) = (/ &
     &1.4363e+01_rb,1.5028e+01_rb,1.5757e+01_rb,1.6564e+01_rb,1.7443e+01_rb/)
      kbo(:,28, 8) = (/ &
     &1.2926e+01_rb,1.3648e+01_rb,1.4446e+01_rb,1.5323e+01_rb,1.6326e+01_rb/)
      kbo(:,29, 8) = (/ &
     &1.1723e+01_rb,1.2493e+01_rb,1.3349e+01_rb,1.4336e+01_rb,1.5419e+01_rb/)
      kbo(:,30, 8) = (/ &
     &1.0720e+01_rb,1.1535e+01_rb,1.2485e+01_rb,1.3543e+01_rb,1.4696e+01_rb/)
      kbo(:,31, 8) = (/ &
     &9.8953e+00_rb,1.0794e+01_rb,1.1807e+01_rb,1.2928e+01_rb,1.4151e+01_rb/)
      kbo(:,32, 8) = (/ &
     &9.2630e+00_rb,1.0213e+01_rb,1.1281e+01_rb,1.2468e+01_rb,1.3783e+01_rb/)
      kbo(:,33, 8) = (/ &
     &8.7693e+00_rb,9.7682e+00_rb,1.0900e+01_rb,1.2171e+01_rb,1.3564e+01_rb/)
      kbo(:,34, 8) = (/ &
     &8.3550e+00_rb,9.4057e+00_rb,1.0612e+01_rb,1.1954e+01_rb,1.3416e+01_rb/)
      kbo(:,35, 8) = (/ &
     &7.9503e+00_rb,9.0506e+00_rb,1.0311e+01_rb,1.1711e+01_rb,1.3230e+01_rb/)
      kbo(:,36, 8) = (/ &
     &7.5260e+00_rb,8.6513e+00_rb,9.9500e+00_rb,1.1393e+01_rb,1.2954e+01_rb/)
      kbo(:,37, 8) = (/ &
     &7.0166e+00_rb,8.1408e+00_rb,9.4476e+00_rb,1.0907e+01_rb,1.2489e+01_rb/)
      kbo(:,38, 8) = (/ &
     &6.5631e+00_rb,7.6815e+00_rb,8.9895e+00_rb,1.0461e+01_rb,1.2064e+01_rb/)
      kbo(:,39, 8) = (/ &
     &6.1678e+00_rb,7.2805e+00_rb,8.5861e+00_rb,1.0065e+01_rb,1.1676e+01_rb/)
      kbo(:,40, 8) = (/ &
     &5.7155e+00_rb,6.7987e+00_rb,8.0775e+00_rb,9.5392e+00_rb,1.1142e+01_rb/)
      kbo(:,41, 8) = (/ &
     &5.2949e+00_rb,6.3470e+00_rb,7.5914e+00_rb,9.0258e+00_rb,1.0614e+01_rb/)
      kbo(:,42, 8) = (/ &
     &4.9139e+00_rb,5.9322e+00_rb,7.1403e+00_rb,8.5407e+00_rb,1.0110e+01_rb/)
      kbo(:,43, 8) = (/ &
     &4.5194e+00_rb,5.4902e+00_rb,6.6498e+00_rb,8.0033e+00_rb,9.5351e+00_rb/)
      kbo(:,44, 8) = (/ &
     &4.1399e+00_rb,5.0576e+00_rb,6.1633e+00_rb,7.4614e+00_rb,8.9447e+00_rb/)
      kbo(:,45, 8) = (/ &
     &3.7947e+00_rb,4.6596e+00_rb,5.7095e+00_rb,6.9501e+00_rb,8.3804e+00_rb/)
      kbo(:,46, 8) = (/ &
     &3.4645e+00_rb,4.2731e+00_rb,5.2631e+00_rb,6.4408e+00_rb,7.8107e+00_rb/)
      kbo(:,47, 8) = (/ &
     &3.1333e+00_rb,3.8817e+00_rb,4.8032e+00_rb,5.9109e+00_rb,7.2077e+00_rb/)
      kbo(:,48, 8) = (/ &
     &2.8272e+00_rb,3.5252e+00_rb,4.3790e+00_rb,5.4161e+00_rb,6.6403e+00_rb/)
      kbo(:,49, 8) = (/ &
     &2.5419e+00_rb,3.2000e+00_rb,3.9894e+00_rb,4.9550e+00_rb,6.1077e+00_rb/)
      kbo(:,50, 8) = (/ &
     &2.2876e+00_rb,2.9151e+00_rb,3.6507e+00_rb,4.5513e+00_rb,5.6358e+00_rb/)
      kbo(:,51, 8) = (/ &
     &2.0539e+00_rb,2.6537e+00_rb,3.3480e+00_rb,4.1860e+00_rb,5.2042e+00_rb/)
      kbo(:,52, 8) = (/ &
     &1.8359e+00_rb,2.4075e+00_rb,3.0676e+00_rb,3.8482e+00_rb,4.8005e+00_rb/)
      kbo(:,53, 8) = (/ &
     &1.6315e+00_rb,2.1744e+00_rb,2.8043e+00_rb,3.5342e+00_rb,4.4242e+00_rb/)
      kbo(:,54, 8) = (/ &
     &1.4536e+00_rb,1.9689e+00_rb,2.5734e+00_rb,3.2651e+00_rb,4.0977e+00_rb/)
      kbo(:,55, 8) = (/ &
     &1.2924e+00_rb,1.7818e+00_rb,2.3616e+00_rb,3.0234e+00_rb,3.8040e+00_rb/)
      kbo(:,56, 8) = (/ &
     &1.1428e+00_rb,1.6042e+00_rb,2.1584e+00_rb,2.7950e+00_rb,3.5303e+00_rb/)
      kbo(:,57, 8) = (/ &
     &1.0034e+00_rb,1.4376e+00_rb,1.9645e+00_rb,2.5766e+00_rb,3.2746e+00_rb/)
      kbo(:,58, 8) = (/ &
     &8.7989e-01_rb,1.2882e+00_rb,1.7895e+00_rb,2.3775e+00_rb,3.0470e+00_rb/)
      kbo(:,59, 8) = (/ &
     &8.2607e-01_rb,1.2253e+00_rb,1.7145e+00_rb,2.2929e+00_rb,2.9546e+00_rb/)
      kbo(:,13, 9) = (/ &
     &9.9406e+01_rb,9.8017e+01_rb,9.6610e+01_rb,9.5133e+01_rb,9.3589e+01_rb/)
      kbo(:,14, 9) = (/ &
     &1.0426e+02_rb,1.0289e+02_rb,1.0145e+02_rb,9.9955e+01_rb,9.8418e+01_rb/)
      kbo(:,15, 9) = (/ &
     &1.0831e+02_rb,1.0700e+02_rb,1.0565e+02_rb,1.0422e+02_rb,1.0271e+02_rb/)
      kbo(:,16, 9) = (/ &
     &1.1144e+02_rb,1.1027e+02_rb,1.0899e+02_rb,1.0766e+02_rb,1.0626e+02_rb/)
      kbo(:,17, 9) = (/ &
     &1.1357e+02_rb,1.1252e+02_rb,1.1140e+02_rb,1.1022e+02_rb,1.0897e+02_rb/)
      kbo(:,18, 9) = (/ &
     &1.1451e+02_rb,1.1366e+02_rb,1.1275e+02_rb,1.1172e+02_rb,1.1059e+02_rb/)
      kbo(:,19, 9) = (/ &
     &1.1419e+02_rb,1.1358e+02_rb,1.1283e+02_rb,1.1199e+02_rb,1.1108e+02_rb/)
      kbo(:,20, 9) = (/ &
     &1.1253e+02_rb,1.1217e+02_rb,1.1170e+02_rb,1.1113e+02_rb,1.1043e+02_rb/)
      kbo(:,21, 9) = (/ &
     &1.0964e+02_rb,1.0959e+02_rb,1.0941e+02_rb,1.0911e+02_rb,1.0868e+02_rb/)
      kbo(:,22, 9) = (/ &
     &1.0576e+02_rb,1.0601e+02_rb,1.0614e+02_rb,1.0616e+02_rb,1.0606e+02_rb/)
      kbo(:,23, 9) = (/ &
     &1.0113e+02_rb,1.0172e+02_rb,1.0220e+02_rb,1.0257e+02_rb,1.0280e+02_rb/)
      kbo(:,24, 9) = (/ &
     &9.6005e+01_rb,9.6976e+01_rb,9.7826e+01_rb,9.8561e+01_rb,9.9182e+01_rb/)
      kbo(:,25, 9) = (/ &
     &9.0658e+01_rb,9.2026e+01_rb,9.3302e+01_rb,9.4462e+01_rb,9.5504e+01_rb/)
      kbo(:,26, 9) = (/ &
     &8.5398e+01_rb,8.7188e+01_rb,8.8895e+01_rb,9.0506e+01_rb,9.1994e+01_rb/)
      kbo(:,27, 9) = (/ &
     &8.0425e+01_rb,8.2648e+01_rb,8.4817e+01_rb,8.6900e+01_rb,8.8861e+01_rb/)
      kbo(:,28, 9) = (/ &
     &7.5902e+01_rb,7.8597e+01_rb,8.1250e+01_rb,8.3817e+01_rb,8.6189e+01_rb/)
      kbo(:,29, 9) = (/ &
     &7.2021e+01_rb,7.5193e+01_rb,7.8323e+01_rb,8.1284e+01_rb,8.4079e+01_rb/)
      kbo(:,30, 9) = (/ &
     &6.8846e+01_rb,7.2476e+01_rb,7.5986e+01_rb,7.9352e+01_rb,8.2517e+01_rb/)
      kbo(:,31, 9) = (/ &
     &6.6391e+01_rb,7.0389e+01_rb,7.4280e+01_rb,7.7994e+01_rb,8.1462e+01_rb/)
      kbo(:,32, 9) = (/ &
     &6.4577e+01_rb,6.8942e+01_rb,7.3166e+01_rb,7.7157e+01_rb,8.0829e+01_rb/)
      kbo(:,33, 9) = (/ &
     &6.3392e+01_rb,6.8071e+01_rb,7.2548e+01_rb,7.6730e+01_rb,8.0569e+01_rb/)
      kbo(:,34, 9) = (/ &
     &6.2591e+01_rb,6.7517e+01_rb,7.2182e+01_rb,7.6523e+01_rb,8.0479e+01_rb/)
      kbo(:,35, 9) = (/ &
     &6.1727e+01_rb,6.6854e+01_rb,7.1692e+01_rb,7.6175e+01_rb,8.0232e+01_rb/)
      kbo(:,36, 9) = (/ &
     &6.0594e+01_rb,6.5914e+01_rb,7.0919e+01_rb,7.5536e+01_rb,7.9714e+01_rb/)
      kbo(:,37, 9) = (/ &
     &5.8846e+01_rb,6.4364e+01_rb,6.9558e+01_rb,7.4349e+01_rb,7.8713e+01_rb/)
      kbo(:,38, 9) = (/ &
     &5.7194e+01_rb,6.2887e+01_rb,6.8247e+01_rb,7.3195e+01_rb,7.8049e+01_rb/)
      kbo(:,39, 9) = (/ &
     &5.5665e+01_rb,6.1507e+01_rb,6.7008e+01_rb,7.2147e+01_rb,7.7547e+01_rb/)
      kbo(:,40, 9) = (/ &
     &5.3621e+01_rb,5.9617e+01_rb,6.5286e+01_rb,7.0695e+01_rb,7.6340e+01_rb/)
      kbo(:,41, 9) = (/ &
     &5.1554e+01_rb,5.7685e+01_rb,6.3510e+01_rb,6.9201e+01_rb,7.4982e+01_rb/)
      kbo(:,42, 9) = (/ &
     &4.9528e+01_rb,5.5776e+01_rb,6.1743e+01_rb,6.7740e+01_rb,7.3659e+01_rb/)
      kbo(:,43, 9) = (/ &
     &4.7178e+01_rb,5.3540e+01_rb,5.9647e+01_rb,6.5886e+01_rb,7.2015e+01_rb/)
      kbo(:,44, 9) = (/ &
     &4.4688e+01_rb,5.1145e+01_rb,5.7387e+01_rb,6.3891e+01_rb,7.0203e+01_rb/)
      kbo(:,45, 9) = (/ &
     &4.2221e+01_rb,4.8754e+01_rb,5.5108e+01_rb,6.1908e+01_rb,6.8361e+01_rb/)
      kbo(:,46, 9) = (/ &
     &3.9637e+01_rb,4.6220e+01_rb,5.2693e+01_rb,5.9708e+01_rb,6.6363e+01_rb/)
      kbo(:,47, 9) = (/ &
     &3.6803e+01_rb,4.3409e+01_rb,4.9987e+01_rb,5.7153e+01_rb,6.3987e+01_rb/)
      kbo(:,48, 9) = (/ &
     &3.4015e+01_rb,4.0603e+01_rb,4.7276e+01_rb,5.4594e+01_rb,6.1608e+01_rb/)
      kbo(:,49, 9) = (/ &
     &3.1288e+01_rb,3.7812e+01_rb,4.4636e+01_rb,5.2065e+01_rb,5.9201e+01_rb/)
      kbo(:,50, 9) = (/ &
     &2.8795e+01_rb,3.5220e+01_rb,4.2061e+01_rb,4.9552e+01_rb,5.6813e+01_rb/)
      kbo(:,51, 9) = (/ &
     &2.6450e+01_rb,3.2748e+01_rb,3.9528e+01_rb,4.7065e+01_rb,5.4424e+01_rb/)
      kbo(:,52, 9) = (/ &
     &2.4182e+01_rb,3.0329e+01_rb,3.7052e+01_rb,4.4584e+01_rb,5.2041e+01_rb/)
      kbo(:,53, 9) = (/ &
     &2.1996e+01_rb,2.7965e+01_rb,3.4636e+01_rb,4.2155e+01_rb,4.9659e+01_rb/)
      kbo(:,54, 9) = (/ &
     &2.0069e+01_rb,2.5858e+01_rb,3.2251e+01_rb,3.9700e+01_rb,4.7245e+01_rb/)
      kbo(:,55, 9) = (/ &
     &1.8302e+01_rb,2.3898e+01_rb,3.0045e+01_rb,3.7230e+01_rb,4.4781e+01_rb/)
      kbo(:,56, 9) = (/ &
     &1.6618e+01_rb,2.2006e+01_rb,2.7997e+01_rb,3.4787e+01_rb,4.2319e+01_rb/)
      kbo(:,57, 9) = (/ &
     &1.5017e+01_rb,2.0180e+01_rb,2.5997e+01_rb,3.2401e+01_rb,3.9855e+01_rb/)
      kbo(:,58, 9) = (/ &
     &1.3581e+01_rb,1.8514e+01_rb,2.4151e+01_rb,3.0332e+01_rb,3.7487e+01_rb/)
      kbo(:,59, 9) = (/ &
     &1.3022e+01_rb,1.7859e+01_rb,2.3425e+01_rb,2.9551e+01_rb,3.6092e+01_rb/)
      kbo(:,13,10) = (/ &
     &1.4854e+02_rb,1.4629e+02_rb,1.4392e+02_rb,1.4179e+02_rb,1.3968e+02_rb/)
      kbo(:,14,10) = (/ &
     &1.6449e+02_rb,1.6201e+02_rb,1.5966e+02_rb,1.5740e+02_rb,1.5475e+02_rb/)
      kbo(:,15,10) = (/ &
     &1.8113e+02_rb,1.7883e+02_rb,1.7633e+02_rb,1.7356e+02_rb,1.7082e+02_rb/)
      kbo(:,16,10) = (/ &
     &1.9869e+02_rb,1.9617e+02_rb,1.9337e+02_rb,1.9057e+02_rb,1.8767e+02_rb/)
      kbo(:,17,10) = (/ &
     &2.1579e+02_rb,2.1310e+02_rb,2.1009e+02_rb,2.0678e+02_rb,2.0344e+02_rb/)
      kbo(:,18,10) = (/ &
     &2.3175e+02_rb,2.2862e+02_rb,2.2543e+02_rb,2.2232e+02_rb,2.1916e+02_rb/)
      kbo(:,19,10) = (/ &
     &2.4595e+02_rb,2.4325e+02_rb,2.4040e+02_rb,2.3730e+02_rb,2.3384e+02_rb/)
      kbo(:,20,10) = (/ &
     &2.5917e+02_rb,2.5646e+02_rb,2.5347e+02_rb,2.5026e+02_rb,2.4697e+02_rb/)
      kbo(:,21,10) = (/ &
     &2.7002e+02_rb,2.6751e+02_rb,2.6472e+02_rb,2.6184e+02_rb,2.5881e+02_rb/)
      kbo(:,22,10) = (/ &
     &2.7788e+02_rb,2.7617e+02_rb,2.7411e+02_rb,2.7166e+02_rb,2.6869e+02_rb/)
      kbo(:,23,10) = (/ &
     &2.8310e+02_rb,2.8234e+02_rb,2.8114e+02_rb,2.7929e+02_rb,2.7687e+02_rb/)
      kbo(:,24,10) = (/ &
     &2.8635e+02_rb,2.8652e+02_rb,2.8615e+02_rb,2.8511e+02_rb,2.8345e+02_rb/)
      kbo(:,25,10) = (/ &
     &2.8819e+02_rb,2.8935e+02_rb,2.8976e+02_rb,2.8954e+02_rb,2.8857e+02_rb/)
      kbo(:,26,10) = (/ &
     &2.8894e+02_rb,2.9109e+02_rb,2.9250e+02_rb,2.9307e+02_rb,2.9272e+02_rb/)
      kbo(:,27,10) = (/ &
     &2.8895e+02_rb,2.9239e+02_rb,2.9473e+02_rb,2.9595e+02_rb,2.9616e+02_rb/)
      kbo(:,28,10) = (/ &
     &2.8893e+02_rb,2.9347e+02_rb,2.9660e+02_rb,2.9845e+02_rb,2.9908e+02_rb/)
      kbo(:,29,10) = (/ &
     &2.8896e+02_rb,2.9449e+02_rb,2.9830e+02_rb,3.0067e+02_rb,3.0167e+02_rb/)
      kbo(:,30,10) = (/ &
     &2.8928e+02_rb,2.9562e+02_rb,2.9995e+02_rb,3.0268e+02_rb,3.0395e+02_rb/)
      kbo(:,31,10) = (/ &
     &2.9009e+02_rb,2.9691e+02_rb,3.0163e+02_rb,3.0456e+02_rb,3.0602e+02_rb/)
      kbo(:,32,10) = (/ &
     &2.9131e+02_rb,2.9842e+02_rb,3.0331e+02_rb,3.0639e+02_rb,3.0785e+02_rb/)
      kbo(:,33,10) = (/ &
     &2.9285e+02_rb,3.0006e+02_rb,3.0500e+02_rb,3.0808e+02_rb,3.0942e+02_rb/)
      kbo(:,34,10) = (/ &
     &2.9430e+02_rb,3.0156e+02_rb,3.0650e+02_rb,3.0954e+02_rb,3.1081e+02_rb/)
      kbo(:,35,10) = (/ &
     &2.9506e+02_rb,3.0251e+02_rb,3.0756e+02_rb,3.1064e+02_rb,3.1189e+02_rb/)
      kbo(:,36,10) = (/ &
     &2.9495e+02_rb,3.0278e+02_rb,3.0811e+02_rb,3.1136e+02_rb,3.1273e+02_rb/)
      kbo(:,37,10) = (/ &
     &2.9340e+02_rb,3.0194e+02_rb,3.0787e+02_rb,3.1159e+02_rb,3.1314e+02_rb/)
      kbo(:,38,10) = (/ &
     &2.9167e+02_rb,3.0090e+02_rb,3.0743e+02_rb,3.1157e+02_rb,3.1035e+02_rb/)
      kbo(:,39,10) = (/ &
     &2.8984e+02_rb,2.9975e+02_rb,3.0683e+02_rb,3.1096e+02_rb,3.0638e+02_rb/)
      kbo(:,40,10) = (/ &
     &2.8667e+02_rb,2.9751e+02_rb,3.0546e+02_rb,3.0937e+02_rb,3.0473e+02_rb/)
      kbo(:,41,10) = (/ &
     &2.8308e+02_rb,2.9489e+02_rb,3.0373e+02_rb,3.0736e+02_rb,3.0363e+02_rb/)
      kbo(:,42,10) = (/ &
     &2.7921e+02_rb,2.9200e+02_rb,3.0174e+02_rb,3.0479e+02_rb,3.0187e+02_rb/)
      kbo(:,43,10) = (/ &
     &2.7420e+02_rb,2.8820e+02_rb,2.9899e+02_rb,3.0230e+02_rb,2.9999e+02_rb/)
      kbo(:,44,10) = (/ &
     &2.6840e+02_rb,2.8368e+02_rb,2.9562e+02_rb,2.9904e+02_rb,2.9872e+02_rb/)
      kbo(:,45,10) = (/ &
     &2.6218e+02_rb,2.7870e+02_rb,2.9185e+02_rb,2.9503e+02_rb,2.9798e+02_rb/)
      kbo(:,46,10) = (/ &
     &2.5505e+02_rb,2.7298e+02_rb,2.8727e+02_rb,2.9090e+02_rb,2.9720e+02_rb/)
      kbo(:,47,10) = (/ &
     &2.4654e+02_rb,2.6603e+02_rb,2.8160e+02_rb,2.8645e+02_rb,2.9469e+02_rb/)
      kbo(:,48,10) = (/ &
     &2.3740e+02_rb,2.5847e+02_rb,2.7519e+02_rb,2.8106e+02_rb,2.9248e+02_rb/)
      kbo(:,49,10) = (/ &
     &2.2763e+02_rb,2.5025e+02_rb,2.6743e+02_rb,2.7456e+02_rb,2.9049e+02_rb/)
      kbo(:,50,10) = (/ &
     &2.1791e+02_rb,2.4202e+02_rb,2.6048e+02_rb,2.6898e+02_rb,2.8731e+02_rb/)
      kbo(:,51,10) = (/ &
     &2.0797e+02_rb,2.3350e+02_rb,2.5355e+02_rb,2.6334e+02_rb,2.8310e+02_rb/)
      kbo(:,52,10) = (/ &
     &1.9757e+02_rb,2.2449e+02_rb,2.4576e+02_rb,2.5715e+02_rb,2.7915e+02_rb/)
      kbo(:,53,10) = (/ &
     &1.8669e+02_rb,2.1495e+02_rb,2.3714e+02_rb,2.5067e+02_rb,2.7496e+02_rb/)
      kbo(:,54,10) = (/ &
     &1.7638e+02_rb,2.0577e+02_rb,2.3068e+02_rb,2.4491e+02_rb,2.6874e+02_rb/)
      kbo(:,55,10) = (/ &
     &1.6621e+02_rb,1.9659e+02_rb,2.2369e+02_rb,2.4095e+02_rb,2.6031e+02_rb/)
      kbo(:,56,10) = (/ &
     &1.5581e+02_rb,1.8709e+02_rb,2.1533e+02_rb,2.3658e+02_rb,2.5187e+02_rb/)
      kbo(:,57,10) = (/ &
     &1.4523e+02_rb,1.7728e+02_rb,2.0662e+02_rb,2.3157e+02_rb,2.4581e+02_rb/)
      kbo(:,58,10) = (/ &
     &1.3508e+02_rb,1.6774e+02_rb,1.9802e+02_rb,2.2497e+02_rb,2.4252e+02_rb/)
      kbo(:,59,10) = (/ &
     &1.3094e+02_rb,1.6382e+02_rb,1.9446e+02_rb,2.2185e+02_rb,2.4525e+02_rb/)
      kbo(:,13,11) = (/ &
     &1.6572e+02_rb,1.6289e+02_rb,1.6008e+02_rb,1.5714e+02_rb,1.5448e+02_rb/)
      kbo(:,14,11) = (/ &
     &1.8540e+02_rb,1.8266e+02_rb,1.7967e+02_rb,1.7665e+02_rb,1.7387e+02_rb/)
      kbo(:,15,11) = (/ &
     &2.0814e+02_rb,2.0518e+02_rb,2.0210e+02_rb,1.9898e+02_rb,1.9569e+02_rb/)
      kbo(:,16,11) = (/ &
     &2.3282e+02_rb,2.2978e+02_rb,2.2643e+02_rb,2.2243e+02_rb,2.1800e+02_rb/)
      kbo(:,17,11) = (/ &
     &2.5856e+02_rb,2.5501e+02_rb,2.5085e+02_rb,2.4632e+02_rb,2.4166e+02_rb/)
      kbo(:,18,11) = (/ &
     &2.8422e+02_rb,2.8031e+02_rb,2.7577e+02_rb,2.7086e+02_rb,2.6564e+02_rb/)
      kbo(:,19,11) = (/ &
     &3.0919e+02_rb,3.0467e+02_rb,2.9960e+02_rb,2.9432e+02_rb,2.8897e+02_rb/)
      kbo(:,20,11) = (/ &
     &3.3207e+02_rb,3.2741e+02_rb,3.2245e+02_rb,3.1728e+02_rb,3.1181e+02_rb/)
      kbo(:,21,11) = (/ &
     &3.5360e+02_rb,3.4929e+02_rb,3.4457e+02_rb,3.3936e+02_rb,3.3368e+02_rb/)
      kbo(:,22,11) = (/ &
     &3.7366e+02_rb,3.6938e+02_rb,3.6457e+02_rb,3.5924e+02_rb,3.5347e+02_rb/)
      kbo(:,23,11) = (/ &
     &3.9151e+02_rb,3.8745e+02_rb,3.8282e+02_rb,3.7760e+02_rb,3.7188e+02_rb/)
      kbo(:,24,11) = (/ &
     &4.0699e+02_rb,4.0355e+02_rb,3.9930e+02_rb,3.9428e+02_rb,3.8859e+02_rb/)
      kbo(:,25,11) = (/ &
     &4.2010e+02_rb,4.1762e+02_rb,4.1382e+02_rb,4.0900e+02_rb,4.0333e+02_rb/)
      kbo(:,26,11) = (/ &
     &4.3097e+02_rb,4.2949e+02_rb,4.2634e+02_rb,4.2177e+02_rb,4.1610e+02_rb/)
      kbo(:,27,11) = (/ &
     &4.4020e+02_rb,4.3958e+02_rb,4.3704e+02_rb,4.3277e+02_rb,4.2718e+02_rb/)
      kbo(:,28,11) = (/ &
     &4.4818e+02_rb,4.4821e+02_rb,4.4611e+02_rb,4.4212e+02_rb,4.3652e+02_rb/)
      kbo(:,29,11) = (/ &
     &4.5504e+02_rb,4.5548e+02_rb,4.5376e+02_rb,4.4984e+02_rb,4.4415e+02_rb/)
      kbo(:,30,11) = (/ &
     &4.6091e+02_rb,4.6162e+02_rb,4.6010e+02_rb,4.5616e+02_rb,4.5025e+02_rb/)
      kbo(:,31,11) = (/ &
     &4.6598e+02_rb,4.6680e+02_rb,4.6528e+02_rb,4.6118e+02_rb,4.5498e+02_rb/)
      kbo(:,32,11) = (/ &
     &4.7040e+02_rb,4.7120e+02_rb,4.6944e+02_rb,4.6509e+02_rb,4.5862e+02_rb/)
      kbo(:,33,11) = (/ &
     &4.7430e+02_rb,4.7493e+02_rb,4.7283e+02_rb,4.6810e+02_rb,4.6122e+02_rb/)
      kbo(:,34,11) = (/ &
     &4.7770e+02_rb,4.7802e+02_rb,4.7557e+02_rb,4.7044e+02_rb,4.6319e+02_rb/)
      kbo(:,35,11) = (/ &
     &4.8041e+02_rb,4.8064e+02_rb,4.7804e+02_rb,4.7275e+02_rb,4.6531e+02_rb/)
      kbo(:,36,11) = (/ &
     &4.8253e+02_rb,4.8296e+02_rb,4.8047e+02_rb,4.7521e+02_rb,4.6778e+02_rb/)
      kbo(:,37,11) = (/ &
     &4.8393e+02_rb,4.8502e+02_rb,4.8301e+02_rb,4.7821e+02_rb,4.7102e+02_rb/)
      kbo(:,38,11) = (/ &
     &4.8486e+02_rb,4.8663e+02_rb,4.8514e+02_rb,4.8082e+02_rb,4.7394e+02_rb/)
      kbo(:,39,11) = (/ &
     &4.8541e+02_rb,4.8785e+02_rb,4.8691e+02_rb,4.8302e+02_rb,4.7647e+02_rb/)
      kbo(:,40,11) = (/ &
     &4.8510e+02_rb,4.8864e+02_rb,4.8854e+02_rb,4.8540e+02_rb,4.7945e+02_rb/)
      kbo(:,41,11) = (/ &
     &4.8418e+02_rb,4.8891e+02_rb,4.8975e+02_rb,4.8740e+02_rb,4.8219e+02_rb/)
      kbo(:,42,11) = (/ &
     &4.8279e+02_rb,4.8873e+02_rb,4.9057e+02_rb,4.8904e+02_rb,4.8458e+02_rb/)
      kbo(:,43,11) = (/ &
     &4.8041e+02_rb,4.8782e+02_rb,4.9100e+02_rb,4.9049e+02_rb,4.8682e+02_rb/)
      kbo(:,44,11) = (/ &
     &4.7714e+02_rb,4.8618e+02_rb,4.9081e+02_rb,4.9150e+02_rb,4.8756e+02_rb/)
      kbo(:,45,11) = (/ &
     &4.7308e+02_rb,4.8395e+02_rb,4.9007e+02_rb,4.9204e+02_rb,4.8671e+02_rb/)
      kbo(:,46,11) = (/ &
     &4.6783e+02_rb,4.8086e+02_rb,4.8862e+02_rb,4.9205e+02_rb,4.8480e+02_rb/)
      kbo(:,47,11) = (/ &
     &4.6080e+02_rb,4.7644e+02_rb,4.8623e+02_rb,4.9139e+02_rb,4.8469e+02_rb/)
      kbo(:,48,11) = (/ &
     &4.5259e+02_rb,4.7102e+02_rb,4.8306e+02_rb,4.9003e+02_rb,4.8230e+02_rb/)
      kbo(:,49,11) = (/ &
     &4.4320e+02_rb,4.6445e+02_rb,4.7901e+02_rb,4.8786e+02_rb,4.7807e+02_rb/)
      kbo(:,50,11) = (/ &
     &4.3321e+02_rb,4.5728e+02_rb,4.7435e+02_rb,4.8518e+02_rb,4.7550e+02_rb/)
      kbo(:,51,11) = (/ &
     &4.2248e+02_rb,4.4931e+02_rb,4.6891e+02_rb,4.8193e+02_rb,4.7366e+02_rb/)
      kbo(:,52,11) = (/ &
     &4.1065e+02_rb,4.4034e+02_rb,4.6257e+02_rb,4.7793e+02_rb,4.7004e+02_rb/)
      kbo(:,53,11) = (/ &
     &3.9765e+02_rb,4.3037e+02_rb,4.5526e+02_rb,4.7213e+02_rb,4.6510e+02_rb/)
      kbo(:,54,11) = (/ &
     &3.8467e+02_rb,4.2027e+02_rb,4.4777e+02_rb,4.6790e+02_rb,4.6484e+02_rb/)
      kbo(:,55,11) = (/ &
     &3.7132e+02_rb,4.0974e+02_rb,4.3973e+02_rb,4.6220e+02_rb,4.6820e+02_rb/)
      kbo(:,56,11) = (/ &
     &3.5710e+02_rb,3.9837e+02_rb,4.3097e+02_rb,4.5580e+02_rb,4.7038e+02_rb/)
      kbo(:,57,11) = (/ &
     &3.4209e+02_rb,3.8602e+02_rb,4.2139e+02_rb,4.4866e+02_rb,4.6860e+02_rb/)
      kbo(:,58,11) = (/ &
     &3.2702e+02_rb,3.7354e+02_rb,4.1153e+02_rb,4.4115e+02_rb,4.6327e+02_rb/)
      kbo(:,59,11) = (/ &
     &3.2074e+02_rb,3.6827e+02_rb,4.0735e+02_rb,4.3797e+02_rb,4.6099e+02_rb/)
      kbo(:,13,12) = (/ &
     &1.8437e+02_rb,1.8201e+02_rb,1.7920e+02_rb,1.7619e+02_rb,1.7262e+02_rb/)
      kbo(:,14,12) = (/ &
     &2.0920e+02_rb,2.0655e+02_rb,2.0368e+02_rb,2.0039e+02_rb,1.9660e+02_rb/)
      kbo(:,15,12) = (/ &
     &2.3807e+02_rb,2.3483e+02_rb,2.3126e+02_rb,2.2723e+02_rb,2.2261e+02_rb/)
      kbo(:,16,12) = (/ &
     &2.7050e+02_rb,2.6622e+02_rb,2.6148e+02_rb,2.5668e+02_rb,2.5203e+02_rb/)
      kbo(:,17,12) = (/ &
     &3.0550e+02_rb,3.0047e+02_rb,2.9528e+02_rb,2.8984e+02_rb,2.8414e+02_rb/)
      kbo(:,18,12) = (/ &
     &3.4308e+02_rb,3.3718e+02_rb,3.3104e+02_rb,3.2458e+02_rb,3.1787e+02_rb/)
      kbo(:,19,12) = (/ &
     &3.8177e+02_rb,3.7532e+02_rb,3.6857e+02_rb,3.6127e+02_rb,3.5363e+02_rb/)
      kbo(:,20,12) = (/ &
     &4.2186e+02_rb,4.1506e+02_rb,4.0728e+02_rb,3.9905e+02_rb,3.9052e+02_rb/)
      kbo(:,21,12) = (/ &
     &4.6246e+02_rb,4.5454e+02_rb,4.4606e+02_rb,4.3704e+02_rb,4.2747e+02_rb/)
      kbo(:,22,12) = (/ &
     &5.0135e+02_rb,4.9292e+02_rb,4.8363e+02_rb,4.7369e+02_rb,4.6338e+02_rb/)
      kbo(:,23,12) = (/ &
     &5.3861e+02_rb,5.2961e+02_rb,5.1962e+02_rb,5.0894e+02_rb,4.9767e+02_rb/)
      kbo(:,24,12) = (/ &
     &5.7338e+02_rb,5.6389e+02_rb,5.5338e+02_rb,5.4189e+02_rb,5.2932e+02_rb/)
      kbo(:,25,12) = (/ &
     &6.0539e+02_rb,5.9528e+02_rb,5.8404e+02_rb,5.7158e+02_rb,5.5798e+02_rb/)
      kbo(:,26,12) = (/ &
     &6.3403e+02_rb,6.2334e+02_rb,6.1112e+02_rb,5.9770e+02_rb,5.8310e+02_rb/)
      kbo(:,27,12) = (/ &
     &6.5936e+02_rb,6.4795e+02_rb,6.3483e+02_rb,6.2036e+02_rb,6.0455e+02_rb/)
      kbo(:,28,12) = (/ &
     &6.8125e+02_rb,6.6919e+02_rb,6.5513e+02_rb,6.3951e+02_rb,6.2260e+02_rb/)
      kbo(:,29,12) = (/ &
     &6.9998e+02_rb,6.8712e+02_rb,6.7207e+02_rb,6.5530e+02_rb,6.3734e+02_rb/)
      kbo(:,30,12) = (/ &
     &7.1583e+02_rb,7.0209e+02_rb,6.8590e+02_rb,6.6805e+02_rb,6.4904e+02_rb/)
      kbo(:,31,12) = (/ &
     &7.2876e+02_rb,7.1404e+02_rb,6.9677e+02_rb,6.7790e+02_rb,6.5793e+02_rb/)
      kbo(:,32,12) = (/ &
     &7.3900e+02_rb,7.2324e+02_rb,7.0493e+02_rb,6.8513e+02_rb,6.6427e+02_rb/)
      kbo(:,33,12) = (/ &
     &7.4681e+02_rb,7.2991e+02_rb,7.1072e+02_rb,6.9001e+02_rb,6.6847e+02_rb/)
      kbo(:,34,12) = (/ &
     &7.5291e+02_rb,7.3522e+02_rb,7.1519e+02_rb,6.9374e+02_rb,6.7158e+02_rb/)
      kbo(:,35,12) = (/ &
     &7.5888e+02_rb,7.4060e+02_rb,7.1995e+02_rb,6.9804e+02_rb,6.7534e+02_rb/)
      kbo(:,36,12) = (/ &
     &7.6530e+02_rb,7.4674e+02_rb,7.2578e+02_rb,7.0348e+02_rb,6.8047e+02_rb/)
      kbo(:,37,12) = (/ &
     &7.7324e+02_rb,7.5477e+02_rb,7.3385e+02_rb,7.1142e+02_rb,6.8821e+02_rb/)
      kbo(:,38,12) = (/ &
     &7.8031e+02_rb,7.6209e+02_rb,7.4126e+02_rb,7.1874e+02_rb,6.9540e+02_rb/)
      kbo(:,39,12) = (/ &
     &7.8648e+02_rb,7.6857e+02_rb,7.4796e+02_rb,7.2542e+02_rb,7.0198e+02_rb/)
      kbo(:,40,12) = (/ &
     &7.9353e+02_rb,7.7637e+02_rb,7.5617e+02_rb,7.3391e+02_rb,7.1045e+02_rb/)
      kbo(:,41,12) = (/ &
     &7.9989e+02_rb,7.8377e+02_rb,7.6410e+02_rb,7.4214e+02_rb,7.1879e+02_rb/)
      kbo(:,42,12) = (/ &
     &8.0558e+02_rb,7.9048e+02_rb,7.7152e+02_rb,7.4994e+02_rb,7.2677e+02_rb/)
      kbo(:,43,12) = (/ &
     &8.1116e+02_rb,7.9747e+02_rb,7.7955e+02_rb,7.5862e+02_rb,7.3574e+02_rb/)
      kbo(:,44,12) = (/ &
     &8.1607e+02_rb,8.0410e+02_rb,7.8744e+02_rb,7.6731e+02_rb,7.4497e+02_rb/)
      kbo(:,45,12) = (/ &
     &8.1994e+02_rb,8.0997e+02_rb,7.9477e+02_rb,7.7561e+02_rb,7.5385e+02_rb/)
      kbo(:,46,12) = (/ &
     &8.2292e+02_rb,8.1527e+02_rb,8.0168e+02_rb,7.8384e+02_rb,7.6285e+02_rb/)
      kbo(:,47,12) = (/ &
     &8.2480e+02_rb,8.1993e+02_rb,8.0858e+02_rb,7.9229e+02_rb,7.7235e+02_rb/)
      kbo(:,48,12) = (/ &
     &8.2522e+02_rb,8.2345e+02_rb,8.1455e+02_rb,8.0002e+02_rb,7.8135e+02_rb/)
      kbo(:,49,12) = (/ &
     &8.2404e+02_rb,8.2558e+02_rb,8.1941e+02_rb,8.0697e+02_rb,7.8982e+02_rb/)
      kbo(:,50,12) = (/ &
     &8.2131e+02_rb,8.2647e+02_rb,8.2301e+02_rb,8.1277e+02_rb,7.9722e+02_rb/)
      kbo(:,51,12) = (/ &
     &8.1699e+02_rb,8.2604e+02_rb,8.2550e+02_rb,8.1759e+02_rb,8.0380e+02_rb/)
      kbo(:,52,12) = (/ &
     &8.1118e+02_rb,8.2427e+02_rb,8.2691e+02_rb,8.2151e+02_rb,8.0969e+02_rb/)
      kbo(:,53,12) = (/ &
     &8.0369e+02_rb,8.2099e+02_rb,8.2725e+02_rb,8.2455e+02_rb,8.1493e+02_rb/)
      kbo(:,54,12) = (/ &
     &7.9519e+02_rb,8.1662e+02_rb,8.2647e+02_rb,8.2649e+02_rb,8.1904e+02_rb/)
      kbo(:,55,12) = (/ &
     &7.8525e+02_rb,8.1122e+02_rb,8.2466e+02_rb,8.2753e+02_rb,8.2231e+02_rb/)
      kbo(:,56,12) = (/ &
     &7.7348e+02_rb,8.0460e+02_rb,8.2163e+02_rb,8.2776e+02_rb,8.2487e+02_rb/)
      kbo(:,57,12) = (/ &
     &7.5991e+02_rb,7.9652e+02_rb,8.1752e+02_rb,8.2701e+02_rb,8.2675e+02_rb/)
      kbo(:,58,12) = (/ &
     &7.4511e+02_rb,7.8730e+02_rb,8.1251e+02_rb,8.2536e+02_rb,8.2776e+02_rb/)
      kbo(:,59,12) = (/ &
     &7.3866e+02_rb,7.8320e+02_rb,8.1026e+02_rb,8.2449e+02_rb,8.2800e+02_rb/)
      kbo(:,13,13) = (/ &
     &2.0724e+02_rb,2.0318e+02_rb,2.0003e+02_rb,1.9694e+02_rb,1.9365e+02_rb/)
      kbo(:,14,13) = (/ &
     &2.3785e+02_rb,2.3358e+02_rb,2.2976e+02_rb,2.2561e+02_rb,2.2123e+02_rb/)
      kbo(:,15,13) = (/ &
     &2.7371e+02_rb,2.6880e+02_rb,2.6365e+02_rb,2.5854e+02_rb,2.5349e+02_rb/)
      kbo(:,16,13) = (/ &
     &3.1341e+02_rb,3.0766e+02_rb,3.0218e+02_rb,2.9651e+02_rb,2.9050e+02_rb/)
      kbo(:,17,13) = (/ &
     &3.5823e+02_rb,3.5167e+02_rb,3.4512e+02_rb,3.3848e+02_rb,3.3144e+02_rb/)
      kbo(:,18,13) = (/ &
     &4.0795e+02_rb,4.0075e+02_rb,3.9354e+02_rb,3.8585e+02_rb,3.7775e+02_rb/)
      kbo(:,19,13) = (/ &
     &4.6454e+02_rb,4.5636e+02_rb,4.4761e+02_rb,4.3852e+02_rb,4.2883e+02_rb/)
      kbo(:,20,13) = (/ &
     &5.2670e+02_rb,5.1674e+02_rb,5.0627e+02_rb,4.9500e+02_rb,4.8317e+02_rb/)
      kbo(:,21,13) = (/ &
     &5.9234e+02_rb,5.8071e+02_rb,5.6797e+02_rb,5.5430e+02_rb,5.3997e+02_rb/)
      kbo(:,22,13) = (/ &
     &6.5952e+02_rb,6.4514e+02_rb,6.2980e+02_rb,6.1347e+02_rb,5.9629e+02_rb/)
      kbo(:,23,13) = (/ &
     &7.2668e+02_rb,7.0946e+02_rb,6.9096e+02_rb,6.7135e+02_rb,6.5090e+02_rb/)
      kbo(:,24,13) = (/ &
     &7.9179e+02_rb,7.7144e+02_rb,7.4939e+02_rb,7.2625e+02_rb,7.0273e+02_rb/)
      kbo(:,25,13) = (/ &
     &8.5340e+02_rb,8.2954e+02_rb,8.0385e+02_rb,7.7737e+02_rb,7.5044e+02_rb/)
      kbo(:,26,13) = (/ &
     &9.1011e+02_rb,8.8231e+02_rb,8.5326e+02_rb,8.2331e+02_rb,7.9319e+02_rb/)
      kbo(:,27,13) = (/ &
     &9.6106e+02_rb,9.2973e+02_rb,8.9712e+02_rb,8.6391e+02_rb,8.3062e+02_rb/)
      kbo(:,28,13) = (/ &
     &1.0059e+03_rb,9.7091e+02_rb,9.3507e+02_rb,8.9869e+02_rb,8.6233e+02_rb/)
      kbo(:,29,13) = (/ &
     &1.0437e+03_rb,1.0055e+03_rb,9.6641e+02_rb,9.2715e+02_rb,8.8802e+02_rb/)
      kbo(:,30,13) = (/ &
     &1.0747e+03_rb,1.0334e+03_rb,9.9156e+02_rb,9.4965e+02_rb,9.0810e+02_rb/)
      kbo(:,31,13) = (/ &
     &1.0990e+03_rb,1.0548e+03_rb,1.0106e+03_rb,9.6642e+02_rb,9.2291e+02_rb/)
      kbo(:,32,13) = (/ &
     &1.1170e+03_rb,1.0705e+03_rb,1.0242e+03_rb,9.7810e+02_rb,9.3290e+02_rb/)
      kbo(:,33,13) = (/ &
     &1.1295e+03_rb,1.0810e+03_rb,1.0329e+03_rb,9.8524e+02_rb,9.3872e+02_rb/)
      kbo(:,34,13) = (/ &
     &1.1390e+03_rb,1.0889e+03_rb,1.0392e+03_rb,9.9037e+02_rb,9.4276e+02_rb/)
      kbo(:,35,13) = (/ &
     &1.1495e+03_rb,1.0980e+03_rb,1.0471e+03_rb,9.9723e+02_rb,9.4859e+02_rb/)
      kbo(:,36,13) = (/ &
     &1.1627e+03_rb,1.1100e+03_rb,1.0580e+03_rb,1.0071e+03_rb,9.5754e+02_rb/)
      kbo(:,37,13) = (/ &
     &1.1817e+03_rb,1.1279e+03_rb,1.0749e+03_rb,1.0229e+03_rb,9.7232e+02_rb/)
      kbo(:,38,13) = (/ &
     &1.1995e+03_rb,1.1446e+03_rb,1.0907e+03_rb,1.0378e+03_rb,9.8626e+02_rb/)
      kbo(:,39,13) = (/ &
     &1.2156e+03_rb,1.1601e+03_rb,1.1053e+03_rb,1.0516e+03_rb,9.9926e+02_rb/)
      kbo(:,40,13) = (/ &
     &1.2364e+03_rb,1.1800e+03_rb,1.1244e+03_rb,1.0699e+03_rb,1.0167e+03_rb/)
      kbo(:,41,13) = (/ &
     &1.2568e+03_rb,1.1999e+03_rb,1.1435e+03_rb,1.0883e+03_rb,1.0343e+03_rb/)
      kbo(:,42,13) = (/ &
     &1.2764e+03_rb,1.2191e+03_rb,1.1621e+03_rb,1.1062e+03_rb,1.0515e+03_rb/)
      kbo(:,43,13) = (/ &
     &1.2983e+03_rb,1.2411e+03_rb,1.1834e+03_rb,1.1268e+03_rb,1.0715e+03_rb/)
      kbo(:,44,13) = (/ &
     &1.3210e+03_rb,1.2639e+03_rb,1.2059e+03_rb,1.1485e+03_rb,1.0925e+03_rb/)
      kbo(:,45,13) = (/ &
     &1.3430e+03_rb,1.2862e+03_rb,1.2280e+03_rb,1.1702e+03_rb,1.1134e+03_rb/)
      kbo(:,46,13) = (/ &
     &1.3658e+03_rb,1.3092e+03_rb,1.2511e+03_rb,1.1929e+03_rb,1.1355e+03_rb/)
      kbo(:,47,13) = (/ &
     &1.3902e+03_rb,1.3342e+03_rb,1.2766e+03_rb,1.2180e+03_rb,1.1600e+03_rb/)
      kbo(:,48,13) = (/ &
     &1.4137e+03_rb,1.3587e+03_rb,1.3015e+03_rb,1.2430e+03_rb,1.1845e+03_rb/)
      kbo(:,49,13) = (/ &
     &1.4360e+03_rb,1.3827e+03_rb,1.3260e+03_rb,1.2679e+03_rb,1.2090e+03_rb/)
      kbo(:,50,13) = (/ &
     &1.4561e+03_rb,1.4046e+03_rb,1.3487e+03_rb,1.2910e+03_rb,1.2321e+03_rb/)
      kbo(:,51,13) = (/ &
     &1.4745e+03_rb,1.4251e+03_rb,1.3704e+03_rb,1.3131e+03_rb,1.2546e+03_rb/)
      kbo(:,52,13) = (/ &
     &1.4914e+03_rb,1.4448e+03_rb,1.3917e+03_rb,1.3349e+03_rb,1.2767e+03_rb/)
      kbo(:,53,13) = (/ &
     &1.5066e+03_rb,1.4635e+03_rb,1.4124e+03_rb,1.3565e+03_rb,1.2987e+03_rb/)
      kbo(:,54,13) = (/ &
     &1.5189e+03_rb,1.4798e+03_rb,1.4307e+03_rb,1.3761e+03_rb,1.3187e+03_rb/)
      kbo(:,55,13) = (/ &
     &1.5296e+03_rb,1.4942e+03_rb,1.4476e+03_rb,1.3946e+03_rb,1.3376e+03_rb/)
      kbo(:,56,13) = (/ &
     &1.5385e+03_rb,1.5073e+03_rb,1.4637e+03_rb,1.4124e+03_rb,1.3564e+03_rb/)
      kbo(:,57,13) = (/ &
     &1.5454e+03_rb,1.5190e+03_rb,1.4793e+03_rb,1.4298e+03_rb,1.3750e+03_rb/)
      kbo(:,58,13) = (/ &
     &1.5501e+03_rb,1.5291e+03_rb,1.4929e+03_rb,1.4457e+03_rb,1.3924e+03_rb/)
      kbo(:,59,13) = (/ &
     &1.5515e+03_rb,1.5328e+03_rb,1.4982e+03_rb,1.4521e+03_rb,1.3993e+03_rb/)
      kbo(:,13,14) = (/ &
     &2.5226e+02_rb,2.4628e+02_rb,2.3949e+02_rb,2.3300e+02_rb,2.2711e+02_rb/)
      kbo(:,14,14) = (/ &
     &2.8391e+02_rb,2.7613e+02_rb,2.6825e+02_rb,2.6100e+02_rb,2.5461e+02_rb/)
      kbo(:,15,14) = (/ &
     &3.2054e+02_rb,3.1155e+02_rb,3.0330e+02_rb,2.9588e+02_rb,2.8901e+02_rb/)
      kbo(:,16,14) = (/ &
     &3.6563e+02_rb,3.5616e+02_rb,3.4734e+02_rb,3.3914e+02_rb,3.3106e+02_rb/)
      kbo(:,17,14) = (/ &
     &4.2096e+02_rb,4.1081e+02_rb,4.0125e+02_rb,3.9214e+02_rb,3.8323e+02_rb/)
      kbo(:,18,14) = (/ &
     &4.8740e+02_rb,4.7626e+02_rb,4.6528e+02_rb,4.5466e+02_rb,4.4412e+02_rb/)
      kbo(:,19,14) = (/ &
     &5.6419e+02_rb,5.5097e+02_rb,5.3827e+02_rb,5.2557e+02_rb,5.1276e+02_rb/)
      kbo(:,20,14) = (/ &
     &6.5011e+02_rb,6.3415e+02_rb,6.1868e+02_rb,6.0321e+02_rb,5.8733e+02_rb/)
      kbo(:,21,14) = (/ &
     &7.4435e+02_rb,7.2484e+02_rb,7.0550e+02_rb,6.8621e+02_rb,6.6643e+02_rb/)
      kbo(:,22,14) = (/ &
     &8.4421e+02_rb,8.2013e+02_rb,7.9603e+02_rb,7.7185e+02_rb,7.4730e+02_rb/)
      kbo(:,23,14) = (/ &
     &9.4857e+02_rb,9.1895e+02_rb,8.8918e+02_rb,8.5916e+02_rb,8.2913e+02_rb/)
      kbo(:,24,14) = (/ &
     &1.0549e+03_rb,1.0183e+03_rb,9.8186e+02_rb,9.4534e+02_rb,9.0882e+02_rb/)
      kbo(:,25,14) = (/ &
     &1.1590e+03_rb,1.1145e+03_rb,1.0708e+03_rb,1.0270e+03_rb,9.8371e+02_rb/)
      kbo(:,26,14) = (/ &
     &1.2562e+03_rb,1.2038e+03_rb,1.1521e+03_rb,1.1010e+03_rb,1.0508e+03_rb/)
      kbo(:,27,14) = (/ &
     &1.3447e+03_rb,1.2836e+03_rb,1.2238e+03_rb,1.1655e+03_rb,1.1088e+03_rb/)
      kbo(:,28,14) = (/ &
     &1.4217e+03_rb,1.3523e+03_rb,1.2847e+03_rb,1.2199e+03_rb,1.1573e+03_rb/)
      kbo(:,29,14) = (/ &
     &1.4859e+03_rb,1.4088e+03_rb,1.3346e+03_rb,1.2639e+03_rb,1.1961e+03_rb/)
      kbo(:,30,14) = (/ &
     &1.5380e+03_rb,1.4539e+03_rb,1.3740e+03_rb,1.2981e+03_rb,1.2259e+03_rb/)
      kbo(:,31,14) = (/ &
     &1.5776e+03_rb,1.4880e+03_rb,1.4031e+03_rb,1.3231e+03_rb,1.2472e+03_rb/)
      kbo(:,32,14) = (/ &
     &1.6062e+03_rb,1.5119e+03_rb,1.4231e+03_rb,1.3398e+03_rb,1.2610e+03_rb/)
      kbo(:,33,14) = (/ &
     &1.6248e+03_rb,1.5270e+03_rb,1.4353e+03_rb,1.3494e+03_rb,1.2686e+03_rb/)
      kbo(:,34,14) = (/ &
     &1.6386e+03_rb,1.5378e+03_rb,1.4439e+03_rb,1.3560e+03_rb,1.2737e+03_rb/)
      kbo(:,35,14) = (/ &
     &1.6557e+03_rb,1.5522e+03_rb,1.4559e+03_rb,1.3661e+03_rb,1.2821e+03_rb/)
      kbo(:,36,14) = (/ &
     &1.6790e+03_rb,1.5727e+03_rb,1.4740e+03_rb,1.3819e+03_rb,1.2962e+03_rb/)
      kbo(:,37,14) = (/ &
     &1.7154e+03_rb,1.6055e+03_rb,1.5035e+03_rb,1.4086e+03_rb,1.3204e+03_rb/)
      kbo(:,38,14) = (/ &
     &1.7499e+03_rb,1.6367e+03_rb,1.5316e+03_rb,1.4342e+03_rb,1.3436e+03_rb/)
      kbo(:,39,14) = (/ &
     &1.7821e+03_rb,1.6659e+03_rb,1.5579e+03_rb,1.4580e+03_rb,1.3653e+03_rb/)
      kbo(:,40,14) = (/ &
     &1.8250e+03_rb,1.7051e+03_rb,1.5937e+03_rb,1.4906e+03_rb,1.3951e+03_rb/)
      kbo(:,41,14) = (/ &
     &1.8687e+03_rb,1.7449e+03_rb,1.6302e+03_rb,1.5239e+03_rb,1.4257e+03_rb/)
      kbo(:,42,14) = (/ &
     &1.9118e+03_rb,1.7843e+03_rb,1.6662e+03_rb,1.5570e+03_rb,1.4560e+03_rb/)
      kbo(:,43,14) = (/ &
     &1.9625e+03_rb,1.8305e+03_rb,1.7087e+03_rb,1.5959e+03_rb,1.4917e+03_rb/)
      kbo(:,44,14) = (/ &
     &2.0169e+03_rb,1.8802e+03_rb,1.7544e+03_rb,1.6379e+03_rb,1.5302e+03_rb/)
      kbo(:,45,14) = (/ &
     &2.0719e+03_rb,1.9306e+03_rb,1.8005e+03_rb,1.6804e+03_rb,1.5693e+03_rb/)
      kbo(:,46,14) = (/ &
     &2.1310e+03_rb,1.9850e+03_rb,1.8502e+03_rb,1.7262e+03_rb,1.6114e+03_rb/)
      kbo(:,47,14) = (/ &
     &2.1977e+03_rb,2.0465e+03_rb,1.9066e+03_rb,1.7781e+03_rb,1.6592e+03_rb/)
      kbo(:,48,14) = (/ &
     &2.2663e+03_rb,2.1096e+03_rb,1.9646e+03_rb,1.8312e+03_rb,1.7082e+03_rb/)
      kbo(:,49,14) = (/ &
     &2.3369e+03_rb,2.1743e+03_rb,2.0243e+03_rb,1.8859e+03_rb,1.7586e+03_rb/)
      kbo(:,50,14) = (/ &
     &2.4050e+03_rb,2.2366e+03_rb,2.0818e+03_rb,1.9387e+03_rb,1.8071e+03_rb/)
      kbo(:,51,14) = (/ &
     &2.4725e+03_rb,2.2988e+03_rb,2.1390e+03_rb,1.9914e+03_rb,1.8554e+03_rb/)
      kbo(:,52,14) = (/ &
     &2.5415e+03_rb,2.3629e+03_rb,2.1974e+03_rb,2.0453e+03_rb,1.9050e+03_rb/)
      kbo(:,53,14) = (/ &
     &2.6121e+03_rb,2.4283e+03_rb,2.2576e+03_rb,2.1009e+03_rb,1.9561e+03_rb/)
      kbo(:,54,14) = (/ &
     &2.6782e+03_rb,2.4899e+03_rb,2.3146e+03_rb,2.1531e+03_rb,2.0043e+03_rb/)
      kbo(:,55,14) = (/ &
     &2.7428e+03_rb,2.5502e+03_rb,2.3706e+03_rb,2.2044e+03_rb,2.0516e+03_rb/)
      kbo(:,56,14) = (/ &
     &2.8087e+03_rb,2.6116e+03_rb,2.4275e+03_rb,2.2567e+03_rb,2.0999e+03_rb/)
      kbo(:,57,14) = (/ &
     &2.8759e+03_rb,2.6742e+03_rb,2.4859e+03_rb,2.3106e+03_rb,2.1494e+03_rb/)
      kbo(:,58,14) = (/ &
     &2.9406e+03_rb,2.7347e+03_rb,2.5423e+03_rb,2.3630e+03_rb,2.1974e+03_rb/)
      kbo(:,59,14) = (/ &
     &2.9672e+03_rb,2.7595e+03_rb,2.5656e+03_rb,2.3846e+03_rb,2.2171e+03_rb/)
      kbo(:,13,15) = (/ &
     &3.0519e+02_rb,3.0114e+02_rb,2.9666e+02_rb,2.9142e+02_rb,2.8521e+02_rb/)
      kbo(:,14,15) = (/ &
     &3.4479e+02_rb,3.4114e+02_rb,3.3631e+02_rb,3.3085e+02_rb,3.2433e+02_rb/)
      kbo(:,15,15) = (/ &
     &3.8993e+02_rb,3.8674e+02_rb,3.8200e+02_rb,3.7573e+02_rb,3.6825e+02_rb/)
      kbo(:,16,15) = (/ &
     &4.4412e+02_rb,4.3994e+02_rb,4.3428e+02_rb,4.2699e+02_rb,4.1897e+02_rb/)
      kbo(:,17,15) = (/ &
     &5.0918e+02_rb,5.0358e+02_rb,4.9594e+02_rb,4.8667e+02_rb,4.7654e+02_rb/)
      kbo(:,18,15) = (/ &
     &5.8800e+02_rb,5.7963e+02_rb,5.6899e+02_rb,5.5650e+02_rb,5.4319e+02_rb/)
      kbo(:,19,15) = (/ &
     &6.8137e+02_rb,6.6869e+02_rb,6.5385e+02_rb,6.3714e+02_rb,6.1975e+02_rb/)
      kbo(:,20,15) = (/ &
     &7.8942e+02_rb,7.7129e+02_rb,7.5076e+02_rb,7.2873e+02_rb,7.0666e+02_rb/)
      kbo(:,21,15) = (/ &
     &9.1222e+02_rb,8.8683e+02_rb,8.5955e+02_rb,8.3129e+02_rb,8.0321e+02_rb/)
      kbo(:,22,15) = (/ &
     &1.0462e+03_rb,1.0122e+03_rb,9.7665e+02_rb,9.4080e+02_rb,9.0511e+02_rb/)
      kbo(:,23,15) = (/ &
     &1.1896e+03_rb,1.1445e+03_rb,1.0990e+03_rb,1.0540e+03_rb,1.0094e+03_rb/)
      kbo(:,24,15) = (/ &
     &1.3365e+03_rb,1.2790e+03_rb,1.2222e+03_rb,1.1664e+03_rb,1.1117e+03_rb/)
      kbo(:,25,15) = (/ &
     &1.4807e+03_rb,1.4091e+03_rb,1.3391e+03_rb,1.2717e+03_rb,1.2073e+03_rb/)
      kbo(:,26,15) = (/ &
     &1.6156e+03_rb,1.5289e+03_rb,1.4461e+03_rb,1.3673e+03_rb,1.2929e+03_rb/)
      kbo(:,27,15) = (/ &
     &1.7387e+03_rb,1.6377e+03_rb,1.5427e+03_rb,1.4535e+03_rb,1.3699e+03_rb/)
      kbo(:,28,15) = (/ &
     &1.8481e+03_rb,1.7337e+03_rb,1.6274e+03_rb,1.5282e+03_rb,1.4358e+03_rb/)
      kbo(:,29,15) = (/ &
     &1.9406e+03_rb,1.8143e+03_rb,1.6975e+03_rb,1.5890e+03_rb,1.4891e+03_rb/)
      kbo(:,30,15) = (/ &
     &2.0157e+03_rb,1.8789e+03_rb,1.7529e+03_rb,1.6367e+03_rb,1.5303e+03_rb/)
      kbo(:,31,15) = (/ &
     &2.0726e+03_rb,1.9271e+03_rb,1.7935e+03_rb,1.6714e+03_rb,1.5596e+03_rb/)
      kbo(:,32,15) = (/ &
     &2.1128e+03_rb,1.9603e+03_rb,1.8210e+03_rb,1.6943e+03_rb,1.5785e+03_rb/)
      kbo(:,33,15) = (/ &
     &2.1379e+03_rb,1.9802e+03_rb,1.8368e+03_rb,1.7069e+03_rb,1.5884e+03_rb/)
      kbo(:,34,15) = (/ &
     &2.1559e+03_rb,1.9943e+03_rb,1.8478e+03_rb,1.7154e+03_rb,1.5948e+03_rb/)
      kbo(:,35,15) = (/ &
     &2.1801e+03_rb,2.0143e+03_rb,1.8645e+03_rb,1.7293e+03_rb,1.6063e+03_rb/)
      kbo(:,36,15) = (/ &
     &2.2153e+03_rb,2.0448e+03_rb,1.8906e+03_rb,1.7519e+03_rb,1.6260e+03_rb/)
      kbo(:,37,15) = (/ &
     &2.2726e+03_rb,2.0949e+03_rb,1.9348e+03_rb,1.7908e+03_rb,1.6606e+03_rb/)
      kbo(:,38,15) = (/ &
     &2.3276e+03_rb,2.1431e+03_rb,1.9773e+03_rb,1.8285e+03_rb,1.6940e+03_rb/)
      kbo(:,39,15) = (/ &
     &2.3794e+03_rb,2.1884e+03_rb,2.0174e+03_rb,1.8638e+03_rb,1.7255e+03_rb/)
      kbo(:,40,15) = (/ &
     &2.4500e+03_rb,2.2506e+03_rb,2.0727e+03_rb,1.9128e+03_rb,1.7693e+03_rb/)
      kbo(:,41,15) = (/ &
     &2.5230e+03_rb,2.3148e+03_rb,2.1296e+03_rb,1.9633e+03_rb,1.8145e+03_rb/)
      kbo(:,42,15) = (/ &
     &2.5961e+03_rb,2.3790e+03_rb,2.1863e+03_rb,2.0139e+03_rb,1.8595e+03_rb/)
      kbo(:,43,15) = (/ &
     &2.6832e+03_rb,2.4556e+03_rb,2.2539e+03_rb,2.0744e+03_rb,1.9133e+03_rb/)
      kbo(:,44,15) = (/ &
     &2.7787e+03_rb,2.5394e+03_rb,2.3280e+03_rb,2.1403e+03_rb,1.9719e+03_rb/)
      kbo(:,45,15) = (/ &
     &2.8773e+03_rb,2.6257e+03_rb,2.4039e+03_rb,2.2077e+03_rb,2.0320e+03_rb/)
      kbo(:,46,15) = (/ &
     &2.9856e+03_rb,2.7201e+03_rb,2.4871e+03_rb,2.2811e+03_rb,2.0978e+03_rb/)
      kbo(:,47,15) = (/ &
     &3.1115e+03_rb,2.8292e+03_rb,2.5830e+03_rb,2.3659e+03_rb,2.1733e+03_rb/)
      kbo(:,48,15) = (/ &
     &3.2432e+03_rb,2.9440e+03_rb,2.6832e+03_rb,2.4542e+03_rb,2.2517e+03_rb/)
      kbo(:,49,15) = (/ &
     &3.3819e+03_rb,3.0648e+03_rb,2.7882e+03_rb,2.5466e+03_rb,2.3333e+03_rb/)
      kbo(:,50,15) = (/ &
     &3.5188e+03_rb,3.1840e+03_rb,2.8918e+03_rb,2.6374e+03_rb,2.4134e+03_rb/)
      kbo(:,51,15) = (/ &
     &3.6589e+03_rb,3.3051e+03_rb,2.9974e+03_rb,2.7291e+03_rb,2.4943e+03_rb/)
      kbo(:,52,15) = (/ &
     &3.8062e+03_rb,3.4317e+03_rb,3.1077e+03_rb,2.8252e+03_rb,2.5787e+03_rb/)
      kbo(:,53,15) = (/ &
     &3.9623e+03_rb,3.5652e+03_rb,3.2237e+03_rb,2.9261e+03_rb,2.6671e+03_rb/)
      kbo(:,54,15) = (/ &
     &4.1133e+03_rb,3.6940e+03_rb,3.3349e+03_rb,3.0230e+03_rb,2.7515e+03_rb/)
      kbo(:,55,15) = (/ &
     &4.2651e+03_rb,3.8236e+03_rb,3.4464e+03_rb,3.1202e+03_rb,2.8359e+03_rb/)
      kbo(:,56,15) = (/ &
     &4.4246e+03_rb,3.9598e+03_rb,3.5625e+03_rb,3.2212e+03_rb,2.9236e+03_rb/)
      kbo(:,57,15) = (/ &
     &4.5926e+03_rb,4.1026e+03_rb,3.6846e+03_rb,3.3264e+03_rb,3.0154e+03_rb/)
      kbo(:,58,15) = (/ &
     &4.7612e+03_rb,4.2446e+03_rb,3.8056e+03_rb,3.4306e+03_rb,3.1063e+03_rb/)
      kbo(:,59,15) = (/ &
     &4.8322e+03_rb,4.3040e+03_rb,3.8565e+03_rb,3.4742e+03_rb,3.1440e+03_rb/)
      kbo(:,13,16) = (/ &
     &3.6471e+02_rb,3.5203e+02_rb,3.4366e+02_rb,3.3754e+02_rb,3.3391e+02_rb/)
      kbo(:,14,16) = (/ &
     &4.3205e+02_rb,4.1910e+02_rb,4.0949e+02_rb,4.0181e+02_rb,3.9611e+02_rb/)
      kbo(:,15,16) = (/ &
     &5.1076e+02_rb,4.9691e+02_rb,4.8535e+02_rb,4.7583e+02_rb,4.6772e+02_rb/)
      kbo(:,16,16) = (/ &
     &6.0124e+02_rb,5.8532e+02_rb,5.7134e+02_rb,5.5921e+02_rb,5.4786e+02_rb/)
      kbo(:,17,16) = (/ &
     &7.0293e+02_rb,6.8353e+02_rb,6.6658e+02_rb,6.5132e+02_rb,6.3843e+02_rb/)
      kbo(:,18,16) = (/ &
     &8.1350e+02_rb,7.9054e+02_rb,7.7108e+02_rb,7.5418e+02_rb,7.3825e+02_rb/)
      kbo(:,19,16) = (/ &
     &9.3232e+02_rb,9.0753e+02_rb,8.8580e+02_rb,8.6501e+02_rb,8.4456e+02_rb/)
      kbo(:,20,16) = (/ &
     &1.0608e+03_rb,1.0329e+03_rb,1.0063e+03_rb,9.8009e+02_rb,9.5395e+02_rb/)
      kbo(:,21,16) = (/ &
     &1.1958e+03_rb,1.1622e+03_rb,1.1292e+03_rb,1.0962e+03_rb,1.0633e+03_rb/)
      kbo(:,22,16) = (/ &
     &1.3311e+03_rb,1.2897e+03_rb,1.2482e+03_rb,1.2072e+03_rb,1.1676e+03_rb/)
      kbo(:,23,16) = (/ &
     &1.4683e+03_rb,1.4174e+03_rb,1.3670e+03_rb,1.3170e+03_rb,1.2708e+03_rb/)
      kbo(:,24,16) = (/ &
     &1.6081e+03_rb,1.5439e+03_rb,1.4829e+03_rb,1.4263e+03_rb,1.3726e+03_rb/)
      kbo(:,25,16) = (/ &
     &1.7506e+03_rb,1.6769e+03_rb,1.6089e+03_rb,1.5421e+03_rb,1.4771e+03_rb/)
      kbo(:,26,16) = (/ &
     &1.9057e+03_rb,1.8182e+03_rb,1.7345e+03_rb,1.6548e+03_rb,1.5773e+03_rb/)
      kbo(:,27,16) = (/ &
     &2.0569e+03_rb,1.9520e+03_rb,1.8523e+03_rb,1.7577e+03_rb,1.6669e+03_rb/)
      kbo(:,28,16) = (/ &
     &2.1937e+03_rb,2.0714e+03_rb,1.9558e+03_rb,1.8469e+03_rb,1.7437e+03_rb/)
      kbo(:,29,16) = (/ &
     &2.3109e+03_rb,2.1712e+03_rb,2.0410e+03_rb,1.9197e+03_rb,1.8067e+03_rb/)
      kbo(:,30,16) = (/ &
     &2.4059e+03_rb,2.2511e+03_rb,2.1086e+03_rb,1.9768e+03_rb,1.8554e+03_rb/)
      kbo(:,31,16) = (/ &
     &2.4785e+03_rb,2.3110e+03_rb,2.1586e+03_rb,2.0181e+03_rb,1.8908e+03_rb/)
      kbo(:,32,16) = (/ &
     &2.5298e+03_rb,2.3528e+03_rb,2.1923e+03_rb,2.0454e+03_rb,1.9136e+03_rb/)
      kbo(:,33,16) = (/ &
     &2.5622e+03_rb,2.3783e+03_rb,2.2119e+03_rb,2.0610e+03_rb,1.9258e+03_rb/)
      kbo(:,34,16) = (/ &
     &2.5858e+03_rb,2.3964e+03_rb,2.2256e+03_rb,2.0716e+03_rb,1.9341e+03_rb/)
      kbo(:,35,16) = (/ &
     &2.6161e+03_rb,2.4209e+03_rb,2.2454e+03_rb,2.0882e+03_rb,1.9479e+03_rb/)
      kbo(:,36,16) = (/ &
     &2.6591e+03_rb,2.4573e+03_rb,2.2765e+03_rb,2.1149e+03_rb,1.9709e+03_rb/)
      kbo(:,37,16) = (/ &
     &2.7282e+03_rb,2.5168e+03_rb,2.3287e+03_rb,2.1606e+03_rb,2.0108e+03_rb/)
      kbo(:,38,16) = (/ &
     &2.7948e+03_rb,2.5745e+03_rb,2.3791e+03_rb,2.2045e+03_rb,2.0494e+03_rb/)
      kbo(:,39,16) = (/ &
     &2.8577e+03_rb,2.6290e+03_rb,2.4266e+03_rb,2.2455e+03_rb,2.0858e+03_rb/)
      kbo(:,40,16) = (/ &
     &2.9439e+03_rb,2.7038e+03_rb,2.4920e+03_rb,2.3032e+03_rb,2.1363e+03_rb/)
      kbo(:,41,16) = (/ &
     &3.0330e+03_rb,2.7810e+03_rb,2.5596e+03_rb,2.3632e+03_rb,2.1888e+03_rb/)
      kbo(:,42,16) = (/ &
     &3.1221e+03_rb,2.8586e+03_rb,2.6274e+03_rb,2.4230e+03_rb,2.2411e+03_rb/)
      kbo(:,43,16) = (/ &
     &3.2294e+03_rb,2.9518e+03_rb,2.6848e+03_rb,2.4944e+03_rb,2.3041e+03_rb/)
      kbo(:,44,16) = (/ &
     &3.3475e+03_rb,3.0539e+03_rb,2.7976e+03_rb,2.5728e+03_rb,2.3737e+03_rb/)
      kbo(:,45,16) = (/ &
     &3.4703e+03_rb,3.1592e+03_rb,2.8896e+03_rb,2.6532e+03_rb,2.4448e+03_rb/)
      kbo(:,46,16) = (/ &
     &3.6062e+03_rb,3.2757e+03_rb,2.9905e+03_rb,2.7415e+03_rb,2.5226e+03_rb/)
      kbo(:,47,16) = (/ &
     &3.7659e+03_rb,3.4109e+03_rb,3.1077e+03_rb,2.8434e+03_rb,2.6124e+03_rb/)
      kbo(:,48,16) = (/ &
     &3.9357e+03_rb,3.5542e+03_rb,3.2307e+03_rb,2.9507e+03_rb,2.7060e+03_rb/)
      kbo(:,49,16) = (/ &
     &4.1183e+03_rb,3.7067e+03_rb,3.3602e+03_rb,3.0634e+03_rb,2.8046e+03_rb/)
      kbo(:,50,16) = (/ &
     &4.3018e+03_rb,3.8593e+03_rb,3.4893e+03_rb,3.1743e+03_rb,2.9010e+03_rb/)
      kbo(:,51,16) = (/ &
     &4.4921e+03_rb,4.0168e+03_rb,3.6208e+03_rb,3.2876e+03_rb,3.0000e+03_rb/)
      kbo(:,52,16) = (/ &
     &4.6953e+03_rb,4.1848e+03_rb,3.7620e+03_rb,3.4065e+03_rb,3.1029e+03_rb/)
      kbo(:,53,16) = (/ &
     &4.9149e+03_rb,4.3644e+03_rb,3.9107e+03_rb,3.5325e+03_rb,3.2110e+03_rb/)
      kbo(:,54,16) = (/ &
     &5.1330e+03_rb,4.5400e+03_rb,4.0560e+03_rb,3.6544e+03_rb,3.3151e+03_rb/)
      kbo(:,55,16) = (/ &
     &5.3615e+03_rb,4.7192e+03_rb,4.2041e+03_rb,3.7779e+03_rb,3.4198e+03_rb/)
      kbo(:,56,16) = (/ &
     &5.6061e+03_rb,4.9107e+03_rb,4.3608e+03_rb,3.9074e+03_rb,3.5296e+03_rb/)
      kbo(:,57,16) = (/ &
     &5.8683e+03_rb,5.1170e+03_rb,4.5267e+03_rb,4.0448e+03_rb,3.6452e+03_rb/)
      kbo(:,58,16) = (/ &
     &6.1353e+03_rb,5.3291e+03_rb,4.6942e+03_rb,4.1832e+03_rb,3.7603e+03_rb/)
      kbo(:,59,16) = (/ &
     &6.2495e+03_rb,5.4200e+03_rb,4.7647e+03_rb,4.2413e+03_rb,3.8086e+03_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The third index 
!     runs over the g-channel (1 to 16).

      kao_mco2( 1, :, 1) = (/ &
     & 7.38630e-06_rb, 8.97432e-06_rb, 1.09037e-05_rb, 1.32480e-05_rb, 1.60963e-05_rb, &
     & 1.95569e-05_rb, 2.37615e-05_rb, 2.88701e-05_rb, 3.50770e-05_rb, 4.26184e-05_rb, &
     & 5.17811e-05_rb, 6.29138e-05_rb, 7.64400e-05_rb, 9.28742e-05_rb, 1.12842e-04_rb, &
     & 1.37102e-04_rb, 1.66578e-04_rb, 2.02392e-04_rb, 2.45905e-04_rb/)
      kao_mco2( 2, :, 1) = (/ &
     & 7.03916e-06_rb, 8.58785e-06_rb, 1.04773e-05_rb, 1.27824e-05_rb, 1.55947e-05_rb, &
     & 1.90257e-05_rb, 2.32115e-05_rb, 2.83183e-05_rb, 3.45487e-05_rb, 4.21498e-05_rb, &
     & 5.14233e-05_rb, 6.27370e-05_rb, 7.65398e-05_rb, 9.33794e-05_rb, 1.13924e-04_rb, &
     & 1.38989e-04_rb, 1.69568e-04_rb, 2.06874e-04_rb, 2.52389e-04_rb/)
      kao_mco2( 3, :, 1) = (/ &
     & 7.80015e-06_rb, 9.48520e-06_rb, 1.15343e-05_rb, 1.40260e-05_rb, 1.70560e-05_rb, &
     & 2.07405e-05_rb, 2.52211e-05_rb, 3.06695e-05_rb, 3.72950e-05_rb, 4.53517e-05_rb, &
     & 5.51489e-05_rb, 6.70626e-05_rb, 8.15499e-05_rb, 9.91670e-05_rb, 1.20590e-04_rb, &
     & 1.46640e-04_rb, 1.78319e-04_rb, 2.16841e-04_rb, 2.63684e-04_rb/)
      kao_mco2( 4, :, 1) = (/ &
     & 9.24267e-06_rb, 1.11747e-05_rb, 1.35105e-05_rb, 1.63346e-05_rb, 1.97490e-05_rb, &
     & 2.38771e-05_rb, 2.88682e-05_rb, 3.49025e-05_rb, 4.21981e-05_rb, 5.10188e-05_rb, &
     & 6.16832e-05_rb, 7.45768e-05_rb, 9.01656e-05_rb, 1.09013e-04_rb, 1.31800e-04_rb, &
     & 1.59350e-04_rb, 1.92659e-04_rb, 2.32930e-04_rb, 2.81619e-04_rb/)
      kao_mco2( 5, :, 1) = (/ &
     & 1.59506e-05_rb, 1.90078e-05_rb, 2.26509e-05_rb, 2.69923e-05_rb, 3.21658e-05_rb, &
     & 3.83309e-05_rb, 4.56777e-05_rb, 5.44325e-05_rb, 6.48654e-05_rb, 7.72978e-05_rb, &
     & 9.21132e-05_rb, 1.09768e-04_rb, 1.30807e-04_rb, 1.55878e-04_rb, 1.85755e-04_rb, &
     & 2.21357e-04_rb, 2.63784e-04_rb, 3.14342e-04_rb, 3.74591e-04_rb/)
      kao_mco2( 6, :, 1) = (/ &
     & 3.53189e-05_rb, 4.14789e-05_rb, 4.87131e-05_rb, 5.72092e-05_rb, 6.71870e-05_rb, &
     & 7.89050e-05_rb, 9.26667e-05_rb, 1.08829e-04_rb, 1.27809e-04_rb, 1.50100e-04_rb, &
     & 1.76279e-04_rb, 2.07024e-04_rb, 2.43131e-04_rb, 2.85535e-04_rb, 3.35335e-04_rb, &
     & 3.93821e-04_rb, 4.62507e-04_rb, 5.43172e-04_rb, 6.37906e-04_rb/)
      kao_mco2( 7, :, 1) = (/ &
     & 6.63273e-05_rb, 7.76356e-05_rb, 9.08718e-05_rb, 1.06365e-04_rb, 1.24499e-04_rb, &
     & 1.45725e-04_rb, 1.70570e-04_rb, 1.99651e-04_rb, 2.33689e-04_rb, 2.73531e-04_rb, &
     & 3.20166e-04_rb, 3.74752e-04_rb, 4.38644e-04_rb, 5.13429e-04_rb, 6.00964e-04_rb, &
     & 7.03424e-04_rb, 8.23352e-04_rb, 9.63726e-04_rb, 1.12803e-03_rb/)
      kao_mco2( 8, :, 1) = (/ &
     & 9.01134e-05_rb, 1.05517e-04_rb, 1.23553e-04_rb, 1.44673e-04_rb, 1.69402e-04_rb, &
     & 1.98359e-04_rb, 2.32265e-04_rb, 2.71967e-04_rb, 3.18456e-04_rb, 3.72890e-04_rb, &
     & 4.36630e-04_rb, 5.11265e-04_rb, 5.98657e-04_rb, 7.00989e-04_rb, 8.20811e-04_rb, &
     & 9.61116e-04_rb, 1.12540e-03_rb, 1.31777e-03_rb, 1.54302e-03_rb/)
      kao_mco2( 9, :, 1) = (/ &
     & 1.14205e-05_rb, 1.36364e-05_rb, 1.62823e-05_rb, 1.94416e-05_rb, 2.32139e-05_rb, &
     & 2.77181e-05_rb, 3.30963e-05_rb, 3.95181e-05_rb, 4.71858e-05_rb, 5.63414e-05_rb, &
     & 6.72734e-05_rb, 8.03266e-05_rb, 9.59124e-05_rb, 1.14523e-04_rb, 1.36743e-04_rb, &
     & 1.63276e-04_rb, 1.94957e-04_rb, 2.32784e-04_rb, 2.77952e-04_rb/)
      kao_mco2( 1, :, 2) = (/ &
     & 2.01754e-05_rb, 2.40506e-05_rb, 2.86701e-05_rb, 3.41769e-05_rb, 4.07414e-05_rb, &
     & 4.85668e-05_rb, 5.78953e-05_rb, 6.90155e-05_rb, 8.22717e-05_rb, 9.80739e-05_rb, &
     & 1.16912e-04_rb, 1.39367e-04_rb, 1.66136e-04_rb, 1.98047e-04_rb, 2.36087e-04_rb, &
     & 2.81433e-04_rb, 3.35489e-04_rb, 3.99928e-04_rb, 4.76744e-04_rb/)
      kao_mco2( 2, :, 2) = (/ &
     & 2.08613e-05_rb, 2.48759e-05_rb, 2.96631e-05_rb, 3.53716e-05_rb, 4.21786e-05_rb, &
     & 5.02955e-05_rb, 5.99746e-05_rb, 7.15163e-05_rb, 8.52791e-05_rb, 1.01690e-04_rb, &
     & 1.21260e-04_rb, 1.44596e-04_rb, 1.72422e-04_rb, 2.05604e-04_rb, 2.45171e-04_rb, &
     & 2.92352e-04_rb, 3.48613e-04_rb, 4.15702e-04_rb, 4.95700e-04_rb/)
      kao_mco2( 3, :, 2) = (/ &
     & 2.06879e-05_rb, 2.47009e-05_rb, 2.94924e-05_rb, 3.52133e-05_rb, 4.20439e-05_rb, &
     & 5.01995e-05_rb, 5.99372e-05_rb, 7.15637e-05_rb, 8.54456e-05_rb, 1.02020e-04_rb, &
     & 1.21810e-04_rb, 1.45439e-04_rb, 1.73651e-04_rb, 2.07335e-04_rb, 2.47554e-04_rb, &
     & 2.95574e-04_rb, 3.52909e-04_rb, 4.21366e-04_rb, 5.03102e-04_rb/)
      kao_mco2( 4, :, 2) = (/ &
     & 2.12700e-05_rb, 2.54064e-05_rb, 3.03472e-05_rb, 3.62490e-05_rb, 4.32984e-05_rb, &
     & 5.17188e-05_rb, 6.17767e-05_rb, 7.37906e-05_rb, 8.81410e-05_rb, 1.05282e-04_rb, &
     & 1.25757e-04_rb, 1.50213e-04_rb, 1.79425e-04_rb, 2.14319e-04_rb, 2.55998e-04_rb, &
     & 3.05782e-04_rb, 3.65249e-04_rb, 4.36280e-04_rb, 5.21125e-04_rb/)
      kao_mco2( 5, :, 2) = (/ &
     & 1.88144e-05_rb, 2.25220e-05_rb, 2.69602e-05_rb, 3.22730e-05_rb, 3.86328e-05_rb, &
     & 4.62458e-05_rb, 5.53591e-05_rb, 6.62682e-05_rb, 7.93271e-05_rb, 9.49594e-05_rb, &
     & 1.13672e-04_rb, 1.36073e-04_rb, 1.62887e-04_rb, 1.94986e-04_rb, 2.33410e-04_rb, &
     & 2.79406e-04_rb, 3.34467e-04_rb, 4.00377e-04_rb, 4.79275e-04_rb/)
      kao_mco2( 6, :, 2) = (/ &
     & 1.20964e-05_rb, 1.46021e-05_rb, 1.76268e-05_rb, 2.12780e-05_rb, 2.56856e-05_rb, &
     & 3.10062e-05_rb, 3.74289e-05_rb, 4.51820e-05_rb, 5.45411e-05_rb, 6.58388e-05_rb, &
     & 7.94769e-05_rb, 9.59399e-05_rb, 1.15813e-04_rb, 1.39803e-04_rb, 1.68762e-04_rb, &
     & 2.03720e-04_rb, 2.45919e-04_rb, 2.96859e-04_rb, 3.58350e-04_rb/)
      kao_mco2( 7, :, 2) = (/ &
     & 3.07117e-05_rb, 3.64441e-05_rb, 4.32465e-05_rb, 5.13186e-05_rb, 6.08974e-05_rb, &
     & 7.22642e-05_rb, 8.57525e-05_rb, 1.01758e-04_rb, 1.20752e-04_rb, 1.43291e-04_rb, &
     & 1.70037e-04_rb, 2.01775e-04_rb, 2.39436e-04_rb, 2.84128e-04_rb, 3.37161e-04_rb, &
     & 4.00094e-04_rb, 4.74773e-04_rb, 5.63391e-04_rb, 6.68549e-04_rb/)
      kao_mco2( 8, :, 2) = (/ &
     & 9.34077e-05_rb, 1.10481e-04_rb, 1.30675e-04_rb, 1.54559e-04_rb, 1.82810e-04_rb, &
     & 2.16224e-04_rb, 2.55745e-04_rb, 3.02491e-04_rb, 3.57780e-04_rb, 4.23175e-04_rb, &
     & 5.00523e-04_rb, 5.92009e-04_rb, 7.00217e-04_rb, 8.28203e-04_rb, 9.79582e-04_rb, &
     & 1.15863e-03_rb, 1.37041e-03_rb, 1.62089e-03_rb, 1.91716e-03_rb/)
      kao_mco2( 9, :, 2) = (/ &
     & 1.15325e-05_rb, 1.37935e-05_rb, 1.64978e-05_rb, 1.97322e-05_rb, 2.36007e-05_rb, &
     & 2.82277e-05_rb, 3.37618e-05_rb, 4.03808e-05_rb, 4.82976e-05_rb, 5.77664e-05_rb, &
     & 6.90916e-05_rb, 8.26372e-05_rb, 9.88384e-05_rb, 1.18216e-04_rb, 1.41392e-04_rb, &
     & 1.69113e-04_rb, 2.02267e-04_rb, 2.41922e-04_rb, 2.89352e-04_rb/)
      kao_mco2( 1, :, 3) = (/ &
     & 2.56142e-05_rb, 3.05385e-05_rb, 3.64096e-05_rb, 4.34093e-05_rb, 5.17547e-05_rb, &
     & 6.17045e-05_rb, 7.35672e-05_rb, 8.77104e-05_rb, 1.04573e-04_rb, 1.24677e-04_rb, &
     & 1.48646e-04_rb, 1.77223e-04_rb, 2.11294e-04_rb, 2.51915e-04_rb, 3.00346e-04_rb, &
     & 3.58087e-04_rb, 4.26929e-04_rb, 5.09006e-04_rb, 6.06862e-04_rb/)
      kao_mco2( 2, :, 3) = (/ &
     & 2.49802e-05_rb, 2.98040e-05_rb, 3.55593e-05_rb, 4.24259e-05_rb, 5.06186e-05_rb, &
     & 6.03932e-05_rb, 7.20554e-05_rb, 8.59696e-05_rb, 1.02571e-04_rb, 1.22377e-04_rb, &
     & 1.46009e-04_rb, 1.74204e-04_rb, 2.07844e-04_rb, 2.47979e-04_rb, 2.95865e-04_rb, &
     & 3.52998e-04_rb, 4.21163e-04_rb, 5.02491e-04_rb, 5.99524e-04_rb/)
      kao_mco2( 3, :, 3) = (/ &
     & 2.54644e-05_rb, 3.03959e-05_rb, 3.62825e-05_rb, 4.33091e-05_rb, 5.16965e-05_rb, &
     & 6.17083e-05_rb, 7.36589e-05_rb, 8.79240e-05_rb, 1.04952e-04_rb, 1.25277e-04_rb, &
     & 1.49539e-04_rb, 1.78499e-04_rb, 2.13068e-04_rb, 2.54331e-04_rb, 3.03586e-04_rb, &
     & 3.62380e-04_rb, 4.32560e-04_rb, 5.16331e-04_rb, 6.16326e-04_rb/)
      kao_mco2( 4, :, 3) = (/ &
     & 2.55054e-05_rb, 3.04699e-05_rb, 3.64007e-05_rb, 4.34859e-05_rb, 5.19501e-05_rb, &
     & 6.20619e-05_rb, 7.41418e-05_rb, 8.85731e-05_rb, 1.05813e-04_rb, 1.26409e-04_rb, &
     & 1.51014e-04_rb, 1.80408e-04_rb, 2.15523e-04_rb, 2.57474e-04_rb, 3.07589e-04_rb, &
     & 3.67459e-04_rb, 4.38983e-04_rb, 5.24428e-04_rb, 6.26505e-04_rb/)
      kao_mco2( 5, :, 3) = (/ &
     & 2.48615e-05_rb, 2.97398e-05_rb, 3.55754e-05_rb, 4.25560e-05_rb, 5.09064e-05_rb, &
     & 6.08952e-05_rb, 7.28441e-05_rb, 8.71375e-05_rb, 1.04236e-04_rb, 1.24689e-04_rb, &
     & 1.49155e-04_rb, 1.78423e-04_rb, 2.13433e-04_rb, 2.55313e-04_rb, 3.05410e-04_rb, &
     & 3.65338e-04_rb, 4.37024e-04_rb, 5.22777e-04_rb, 6.25356e-04_rb/)
      kao_mco2( 6, :, 3) = (/ &
     & 2.09074e-05_rb, 2.50891e-05_rb, 3.01072e-05_rb, 3.61290e-05_rb, 4.33553e-05_rb, &
     & 5.20269e-05_rb, 6.24329e-05_rb, 7.49202e-05_rb, 8.99051e-05_rb, 1.07887e-04_rb, &
     & 1.29466e-04_rb, 1.55361e-04_rb, 1.86435e-04_rb, 2.23724e-04_rb, 2.68471e-04_rb, &
     & 3.22169e-04_rb, 3.86607e-04_rb, 4.63933e-04_rb, 5.56725e-04_rb/)
      kao_mco2( 7, :, 3) = (/ &
     & 1.25163e-05_rb, 1.51688e-05_rb, 1.83835e-05_rb, 2.22795e-05_rb, 2.70011e-05_rb, &
     & 3.27234e-05_rb, 3.96583e-05_rb, 4.80630e-05_rb, 5.82488e-05_rb, 7.05933e-05_rb, &
     & 8.55539e-05_rb, 1.03685e-04_rb, 1.25659e-04_rb, 1.52289e-04_rb, 1.84563e-04_rb, &
     & 2.23677e-04_rb, 2.71081e-04_rb, 3.28530e-04_rb, 3.98154e-04_rb/)
      kao_mco2( 8, :, 3) = (/ &
     & 1.00408e-04_rb, 1.20081e-04_rb, 1.43608e-04_rb, 1.71745e-04_rb, 2.05395e-04_rb, &
     & 2.45637e-04_rb, 2.93765e-04_rb, 3.51322e-04_rb, 4.20156e-04_rb, 5.02476e-04_rb, &
     & 6.00926e-04_rb, 7.18665e-04_rb, 8.59472e-04_rb, 1.02787e-03_rb, 1.22926e-03_rb, &
     & 1.47010e-03_rb, 1.75814e-03_rb, 2.10261e-03_rb, 2.51457e-03_rb/)
      kao_mco2( 9, :, 3) = (/ &
     & 8.50402e-06_rb, 1.02737e-05_rb, 1.24116e-05_rb, 1.49945e-05_rb, 1.81148e-05_rb, &
     & 2.18844e-05_rb, 2.64385e-05_rb, 3.19403e-05_rb, 3.85871e-05_rb, 4.66169e-05_rb, &
     & 5.63178e-05_rb, 6.80375e-05_rb, 8.21959e-05_rb, 9.93008e-05_rb, 1.19965e-04_rb, &
     & 1.44930e-04_rb, 1.75089e-04_rb, 2.11525e-04_rb, 2.55543e-04_rb/)
      kao_mco2( 1, :, 4) = (/ &
     & 2.68659e-05_rb, 3.20986e-05_rb, 3.83506e-05_rb, 4.58203e-05_rb, 5.47450e-05_rb, &
     & 6.54078e-05_rb, 7.81476e-05_rb, 9.33687e-05_rb, 1.11555e-04_rb, 1.33282e-04_rb, &
     & 1.59242e-04_rb, 1.90259e-04_rb, 2.27316e-04_rb, 2.71592e-04_rb, 3.24490e-04_rb, &
     & 3.87693e-04_rb, 4.63206e-04_rb, 5.53426e-04_rb, 6.61218e-04_rb/)
      kao_mco2( 2, :, 4) = (/ &
     & 2.74827e-05_rb, 3.28460e-05_rb, 3.92560e-05_rb, 4.69169e-05_rb, 5.60728e-05_rb, &
     & 6.70155e-05_rb, 8.00937e-05_rb, 9.57241e-05_rb, 1.14405e-04_rb, 1.36731e-04_rb, &
     & 1.63415e-04_rb, 1.95305e-04_rb, 2.33419e-04_rb, 2.78972e-04_rb, 3.33413e-04_rb, &
     & 3.98480e-04_rb, 4.76244e-04_rb, 5.69184e-04_rb, 6.80261e-04_rb/)
      kao_mco2( 3, :, 4) = (/ &
     & 2.84702e-05_rb, 3.40189e-05_rb, 4.06490e-05_rb, 4.85713e-05_rb, 5.80375e-05_rb, &
     & 6.93487e-05_rb, 8.28644e-05_rb, 9.90142e-05_rb, 1.18312e-04_rb, 1.41370e-04_rb, &
     & 1.68922e-04_rb, 2.01844e-04_rb, 2.41182e-04_rb, 2.88188e-04_rb, 3.44354e-04_rb, &
     & 4.11466e-04_rb, 4.91659e-04_rb, 5.87481e-04_rb, 7.01977e-04_rb/)
      kao_mco2( 4, :, 4) = (/ &
     & 2.92293e-05_rb, 3.49243e-05_rb, 4.17289e-05_rb, 4.98593e-05_rb, 5.95738e-05_rb, &
     & 7.11810e-05_rb, 8.50498e-05_rb, 1.01621e-04_rb, 1.21420e-04_rb, 1.45078e-04_rb, &
     & 1.73344e-04_rb, 2.07119e-04_rb, 2.47473e-04_rb, 2.95690e-04_rb, 3.53302e-04_rb, &
     & 4.22139e-04_rb, 5.04388e-04_rb, 6.02662e-04_rb, 7.20083e-04_rb/)
      kao_mco2( 5, :, 4) = (/ &
     & 2.88531e-05_rb, 3.45646e-05_rb, 4.14067e-05_rb, 4.96033e-05_rb, 5.94224e-05_rb, &
     & 7.11851e-05_rb, 8.52764e-05_rb, 1.02157e-04_rb, 1.22379e-04_rb, 1.46604e-04_rb, &
     & 1.75625e-04_rb, 2.10391e-04_rb, 2.52038e-04_rb, 3.01929e-04_rb, 3.61697e-04_rb, &
     & 4.33295e-04_rb, 5.19067e-04_rb, 6.21818e-04_rb, 7.44908e-04_rb/)
      kao_mco2( 6, :, 4) = (/ &
     & 2.79869e-05_rb, 3.36885e-05_rb, 4.05516e-05_rb, 4.88130e-05_rb, 5.87574e-05_rb, &
     & 7.07278e-05_rb, 8.51368e-05_rb, 1.02481e-04_rb, 1.23359e-04_rb, 1.48490e-04_rb, &
     & 1.78742e-04_rb, 2.15156e-04_rb, 2.58988e-04_rb, 3.11751e-04_rb, 3.75262e-04_rb, &
     & 4.51712e-04_rb, 5.43737e-04_rb, 6.54510e-04_rb, 7.87849e-04_rb/)
      kao_mco2( 7, :, 4) = (/ &
     & 1.45797e-05_rb, 1.78204e-05_rb, 2.17815e-05_rb, 2.66230e-05_rb, 3.25407e-05_rb, &
     & 3.97737e-05_rb, 4.86145e-05_rb, 5.94203e-05_rb, 7.26281e-05_rb, 8.87715e-05_rb, &
     & 1.08503e-04_rb, 1.32621e-04_rb, 1.62100e-04_rb, 1.98130e-04_rb, 2.42170e-04_rb, &
     & 2.95999e-04_rb, 3.61792e-04_rb, 4.42210e-04_rb, 5.40503e-04_rb/)
      kao_mco2( 8, :, 4) = (/ &
     & 6.32607e-05_rb, 7.63420e-05_rb, 9.21282e-05_rb, 1.11179e-04_rb, 1.34169e-04_rb, &
     & 1.61913e-04_rb, 1.95393e-04_rb, 2.35797e-04_rb, 2.84557e-04_rb, 3.43398e-04_rb, &
     & 4.14407e-04_rb, 5.00100e-04_rb, 6.03512e-04_rb, 7.28308e-04_rb, 8.78909e-04_rb, &
     & 1.06065e-03_rb, 1.27998e-03_rb, 1.54466e-03_rb, 1.86407e-03_rb/)
      kao_mco2( 9, :, 4) = (/ &
     & 1.52296e-05_rb, 1.84301e-05_rb, 2.23032e-05_rb, 2.69902e-05_rb, 3.26622e-05_rb, &
     & 3.95261e-05_rb, 4.78324e-05_rb, 5.78844e-05_rb, 7.00487e-05_rb, 8.47694e-05_rb, &
     & 1.02584e-04_rb, 1.24142e-04_rb, 1.50230e-04_rb, 1.81800e-04_rb, 2.20005e-04_rb, &
     & 2.66239e-04_rb, 3.22190e-04_rb, 3.89897e-04_rb, 4.71833e-04_rb/)
      kao_mco2( 1, :, 5) = (/ &
     & 3.43213e-05_rb, 4.11301e-05_rb, 4.92896e-05_rb, 5.90679e-05_rb, 7.07860e-05_rb, &
     & 8.48288e-05_rb, 1.01657e-04_rb, 1.21825e-04_rb, 1.45993e-04_rb, 1.74955e-04_rb, &
     & 2.09663e-04_rb, 2.51257e-04_rb, 3.01103e-04_rb, 3.60837e-04_rb, 4.32421e-04_rb, &
     & 5.18206e-04_rb, 6.21010e-04_rb, 7.44208e-04_rb, 8.91846e-04_rb/)
      kao_mco2( 2, :, 5) = (/ &
     & 3.14792e-05_rb, 3.79075e-05_rb, 4.56485e-05_rb, 5.49703e-05_rb, 6.61956e-05_rb, &
     & 7.97133e-05_rb, 9.59914e-05_rb, 1.15594e-04_rb, 1.39199e-04_rb, 1.67624e-04_rb, &
     & 2.01854e-04_rb, 2.43075e-04_rb, 2.92712e-04_rb, 3.52487e-04_rb, 4.24467e-04_rb, &
     & 5.11147e-04_rb, 6.15527e-04_rb, 7.41222e-04_rb, 8.92585e-04_rb/)
      kao_mco2( 3, :, 5) = (/ &
     & 3.21655e-05_rb, 3.87990e-05_rb, 4.68006e-05_rb, 5.64523e-05_rb, 6.80945e-05_rb, &
     & 8.21377e-05_rb, 9.90770e-05_rb, 1.19510e-04_rb, 1.44156e-04_rb, 1.73886e-04_rb, &
     & 2.09746e-04_rb, 2.53002e-04_rb, 3.05179e-04_rb, 3.68117e-04_rb, 4.44033e-04_rb, &
     & 5.35607e-04_rb, 6.46066e-04_rb, 7.79304e-04_rb, 9.40020e-04_rb/)
      kao_mco2( 4, :, 5) = (/ &
     & 3.22870e-05_rb, 3.89864e-05_rb, 4.70759e-05_rb, 5.68439e-05_rb, 6.86388e-05_rb, &
     & 8.28810e-05_rb, 1.00078e-04_rb, 1.20844e-04_rb, 1.45919e-04_rb, 1.76196e-04_rb, &
     & 2.12756e-04_rb, 2.56902e-04_rb, 3.10207e-04_rb, 3.74574e-04_rb, 4.52296e-04_rb, &
     & 5.46146e-04_rb, 6.59468e-04_rb, 7.96304e-04_rb, 9.61533e-04_rb/)
      kao_mco2( 5, :, 5) = (/ &
     & 3.31190e-05_rb, 3.99528e-05_rb, 4.81967e-05_rb, 5.81417e-05_rb, 7.01387e-05_rb, &
     & 8.46111e-05_rb, 1.02070e-04_rb, 1.23131e-04_rb, 1.48538e-04_rb, 1.79187e-04_rb, &
     & 2.16161e-04_rb, 2.60764e-04_rb, 3.14570e-04_rb, 3.79479e-04_rb, 4.57781e-04_rb, &
     & 5.52240e-04_rb, 6.66190e-04_rb, 8.03652e-04_rb, 9.69477e-04_rb/)
      kao_mco2( 6, :, 5) = (/ &
     & 3.31287e-05_rb, 3.99772e-05_rb, 4.82413e-05_rb, 5.82139e-05_rb, 7.02480e-05_rb, &
     & 8.47698e-05_rb, 1.02294e-04_rb, 1.23440e-04_rb, 1.48958e-04_rb, 1.79750e-04_rb, &
     & 2.16909e-04_rb, 2.61749e-04_rb, 3.15858e-04_rb, 3.81153e-04_rb, 4.59945e-04_rb, &
     & 5.55026e-04_rb, 6.69762e-04_rb, 8.08216e-04_rb, 9.75292e-04_rb/)
      kao_mco2( 7, :, 5) = (/ &
     & 3.35235e-05_rb, 4.02832e-05_rb, 4.84061e-05_rb, 5.81668e-05_rb, 6.98958e-05_rb, &
     & 8.39898e-05_rb, 1.00926e-04_rb, 1.21277e-04_rb, 1.45731e-04_rb, 1.75117e-04_rb, &
     & 2.10428e-04_rb, 2.52860e-04_rb, 3.03847e-04_rb, 3.65116e-04_rb, 4.38739e-04_rb, &
     & 5.27208e-04_rb, 6.33516e-04_rb, 7.61260e-04_rb, 9.14762e-04_rb/)
      kao_mco2( 8, :, 5) = (/ &
     & 3.57666e-05_rb, 4.27511e-05_rb, 5.10995e-05_rb, 6.10783e-05_rb, 7.30057e-05_rb, &
     & 8.72622e-05_rb, 1.04303e-04_rb, 1.24671e-04_rb, 1.49017e-04_rb, 1.78117e-04_rb, &
     & 2.12900e-04_rb, 2.54475e-04_rb, 3.04169e-04_rb, 3.63567e-04_rb, 4.34565e-04_rb, &
     & 5.19427e-04_rb, 6.20861e-04_rb, 7.42103e-04_rb, 8.87020e-04_rb/)
      kao_mco2( 9, :, 5) = (/ &
     & 2.96349e-05_rb, 3.55202e-05_rb, 4.25743e-05_rb, 5.10292e-05_rb, 6.11633e-05_rb, &
     & 7.33099e-05_rb, 8.78687e-05_rb, 1.05319e-04_rb, 1.26234e-04_rb, 1.51304e-04_rb, &
     & 1.81352e-04_rb, 2.17367e-04_rb, 2.60534e-04_rb, 3.12275e-04_rb, 3.74290e-04_rb, &
     & 4.48622e-04_rb, 5.37715e-04_rb, 6.44502e-04_rb, 7.72495e-04_rb/)
      kao_mco2( 1, :, 6) = (/ &
     & 4.14659e-05_rb, 4.98693e-05_rb, 5.99757e-05_rb, 7.21302e-05_rb, 8.67479e-05_rb, &
     & 1.04328e-04_rb, 1.25471e-04_rb, 1.50899e-04_rb, 1.81479e-04_rb, 2.18257e-04_rb, &
     & 2.62489e-04_rb, 3.15685e-04_rb, 3.79660e-04_rb, 4.56601e-04_rb, 5.49135e-04_rb, &
     & 6.60422e-04_rb, 7.94261e-04_rb, 9.55224e-04_rb, 1.14881e-03_rb/)
      kao_mco2( 2, :, 6) = (/ &
     & 4.25940e-05_rb, 5.11162e-05_rb, 6.13434e-05_rb, 7.36168e-05_rb, 8.83459e-05_rb, &
     & 1.06022e-04_rb, 1.27235e-04_rb, 1.52691e-04_rb, 1.83241e-04_rb, 2.19904e-04_rb, &
     & 2.63902e-04_rb, 3.16703e-04_rb, 3.80068e-04_rb, 4.56111e-04_rb, 5.47368e-04_rb, &
     & 6.56885e-04_rb, 7.88313e-04_rb, 9.46036e-04_rb, 1.13532e-03_rb/)
      kao_mco2( 3, :, 6) = (/ &
     & 4.44940e-05_rb, 5.32922e-05_rb, 6.38303e-05_rb, 7.64522e-05_rb, 9.15700e-05_rb, &
     & 1.09677e-04_rb, 1.31365e-04_rb, 1.57341e-04_rb, 1.88454e-04_rb, 2.25719e-04_rb, &
     & 2.70353e-04_rb, 3.23813e-04_rb, 3.87844e-04_rb, 4.64537e-04_rb, 5.56395e-04_rb, &
     & 6.66418e-04_rb, 7.98196e-04_rb, 9.56032e-04_rb, 1.14508e-03_rb/)
      kao_mco2( 4, :, 6) = (/ &
     & 4.83402e-05_rb, 5.78065e-05_rb, 6.91265e-05_rb, 8.26633e-05_rb, 9.88510e-05_rb, &
     & 1.18209e-04_rb, 1.41357e-04_rb, 1.69038e-04_rb, 2.02140e-04_rb, 2.41725e-04_rb, &
     & 2.89061e-04_rb, 3.45667e-04_rb, 4.13357e-04_rb, 4.94303e-04_rb, 5.91101e-04_rb, &
     & 7.06854e-04_rb, 8.45275e-04_rb, 1.01080e-03_rb, 1.20874e-03_rb/)
      kao_mco2( 5, :, 6) = (/ &
     & 5.14797e-05_rb, 6.15328e-05_rb, 7.35491e-05_rb, 8.79120e-05_rb, 1.05080e-04_rb, &
     & 1.25600e-04_rb, 1.50128e-04_rb, 1.79445e-04_rb, 2.14487e-04_rb, 2.56373e-04_rb, &
     & 3.06438e-04_rb, 3.66281e-04_rb, 4.37809e-04_rb, 5.23305e-04_rb, 6.25498e-04_rb, &
     & 7.47647e-04_rb, 8.93650e-04_rb, 1.06816e-03_rb, 1.27676e-03_rb/)
      kao_mco2( 6, :, 6) = (/ &
     & 5.71481e-05_rb, 6.83156e-05_rb, 8.16652e-05_rb, 9.76237e-05_rb, 1.16701e-04_rb, &
     & 1.39505e-04_rb, 1.66766e-04_rb, 1.99354e-04_rb, 2.38311e-04_rb, 2.84879e-04_rb, &
     & 3.40548e-04_rb, 4.07096e-04_rb, 4.86647e-04_rb, 5.81744e-04_rb, 6.95424e-04_rb, &
     & 8.31318e-04_rb, 9.93769e-04_rb, 1.18796e-03_rb, 1.42010e-03_rb/)
      kao_mco2( 7, :, 6) = (/ &
     & 5.69513e-05_rb, 6.84420e-05_rb, 8.22512e-05_rb, 9.88466e-05_rb, 1.18790e-04_rb, &
     & 1.42758e-04_rb, 1.71562e-04_rb, 2.06177e-04_rb, 2.47776e-04_rb, 2.97768e-04_rb, &
     & 3.57848e-04_rb, 4.30049e-04_rb, 5.16817e-04_rb, 6.21093e-04_rb, 7.46407e-04_rb, &
     & 8.97006e-04_rb, 1.07799e-03_rb, 1.29549e-03_rb, 1.55687e-03_rb/)
      kao_mco2( 8, :, 6) = (/ &
     & 4.39361e-06_rb, 5.50076e-06_rb, 6.88690e-06_rb, 8.62235e-06_rb, 1.07951e-05_rb, &
     & 1.35154e-05_rb, 1.69212e-05_rb, 2.11851e-05_rb, 2.65236e-05_rb, 3.32074e-05_rb, &
     & 4.15754e-05_rb, 5.20520e-05_rb, 6.51687e-05_rb, 8.15907e-05_rb, 1.02151e-04_rb, &
     & 1.27892e-04_rb, 1.60120e-04_rb, 2.00469e-04_rb, 2.50985e-04_rb/)
      kao_mco2( 9, :, 6) = (/ &
     & 5.75515e-05_rb, 6.86850e-05_rb, 8.19722e-05_rb, 9.78298e-05_rb, 1.16755e-04_rb, &
     & 1.39342e-04_rb, 1.66297e-04_rb, 1.98468e-04_rb, 2.36862e-04_rb, 2.82683e-04_rb, &
     & 3.37369e-04_rb, 4.02633e-04_rb, 4.80523e-04_rb, 5.73481e-04_rb, 6.84422e-04_rb, &
     & 8.16824e-04_rb, 9.74841e-04_rb, 1.16342e-03_rb, 1.38849e-03_rb/)
      kao_mco2( 1, :, 7) = (/ &
     & 6.84544e-05_rb, 8.16461e-05_rb, 9.73799e-05_rb, 1.16146e-04_rb, 1.38528e-04_rb, &
     & 1.65223e-04_rb, 1.97063e-04_rb, 2.35039e-04_rb, 2.80333e-04_rb, 3.34355e-04_rb, &
     & 3.98788e-04_rb, 4.75637e-04_rb, 5.67296e-04_rb, 6.76618e-04_rb, 8.07008e-04_rb, &
     & 9.62525e-04_rb, 1.14801e-03_rb, 1.36924e-03_rb, 1.63310e-03_rb/)
      kao_mco2( 2, :, 7) = (/ &
     & 6.88332e-05_rb, 8.21719e-05_rb, 9.80955e-05_rb, 1.17105e-04_rb, 1.39798e-04_rb, &
     & 1.66888e-04_rb, 1.99229e-04_rb, 2.37836e-04_rb, 2.83925e-04_rb, 3.38944e-04_rb, &
     & 4.04627e-04_rb, 4.83037e-04_rb, 5.76641e-04_rb, 6.88385e-04_rb, 8.21782e-04_rb, &
     & 9.81031e-04_rb, 1.17114e-03_rb, 1.39809e-03_rb, 1.66901e-03_rb/)
      kao_mco2( 3, :, 7) = (/ &
     & 7.49899e-05_rb, 8.94606e-05_rb, 1.06724e-04_rb, 1.27318e-04_rb, 1.51887e-04_rb, &
     & 1.81196e-04_rb, 2.16161e-04_rb, 2.57873e-04_rb, 3.07635e-04_rb, 3.66999e-04_rb, &
     & 4.37818e-04_rb, 5.22304e-04_rb, 6.23092e-04_rb, 7.43330e-04_rb, 8.86769e-04_rb, &
     & 1.05789e-03_rb, 1.26203e-03_rb, 1.50556e-03_rb, 1.79608e-03_rb/)
      kao_mco2( 4, :, 7) = (/ &
     & 8.26801e-05_rb, 9.85802e-05_rb, 1.17538e-04_rb, 1.40141e-04_rb, 1.67092e-04_rb, &
     & 1.99225e-04_rb, 2.37537e-04_rb, 2.83217e-04_rb, 3.37682e-04_rb, 4.02621e-04_rb, &
     & 4.80048e-04_rb, 5.72365e-04_rb, 6.82435e-04_rb, 8.13673e-04_rb, 9.70148e-04_rb, &
     & 1.15671e-03_rb, 1.37916e-03_rb, 1.64438e-03_rb, 1.96061e-03_rb/)
      kao_mco2( 5, :, 7) = (/ &
     & 9.29561e-05_rb, 1.10845e-04_rb, 1.32176e-04_rb, 1.57612e-04_rb, 1.87944e-04_rb, &
     & 2.24112e-04_rb, 2.67241e-04_rb, 3.18669e-04_rb, 3.79995e-04_rb, 4.53121e-04_rb, &
     & 5.40321e-04_rb, 6.44302e-04_rb, 7.68293e-04_rb, 9.16146e-04_rb, 1.09245e-03_rb, &
     & 1.30268e-03_rb, 1.55338e-03_rb, 1.85231e-03_rb, 2.20877e-03_rb/)
      kao_mco2( 6, :, 7) = (/ &
     & 1.09700e-04_rb, 1.30879e-04_rb, 1.56148e-04_rb, 1.86294e-04_rb, 2.22261e-04_rb, &
     & 2.65172e-04_rb, 3.16367e-04_rb, 3.77446e-04_rb, 4.50317e-04_rb, 5.37257e-04_rb, &
     & 6.40983e-04_rb, 7.64734e-04_rb, 9.12376e-04_rb, 1.08852e-03_rb, 1.29868e-03_rb, &
     & 1.54941e-03_rb, 1.84854e-03_rb, 2.20543e-03_rb, 2.63122e-03_rb/)
      kao_mco2( 7, :, 7) = (/ &
     & 1.43457e-04_rb, 1.71554e-04_rb, 2.05153e-04_rb, 2.45332e-04_rb, 2.93381e-04_rb, &
     & 3.50840e-04_rb, 4.19552e-04_rb, 5.01722e-04_rb, 5.99985e-04_rb, 7.17492e-04_rb, &
     & 8.58014e-04_rb, 1.02606e-03_rb, 1.22701e-03_rb, 1.46732e-03_rb, 1.75470e-03_rb, &
     & 2.09836e-03_rb, 2.50933e-03_rb, 3.00078e-03_rb, 3.58849e-03_rb/)
      kao_mco2( 8, :, 7) = (/ &
     & 1.52152e-05_rb, 1.89421e-05_rb, 2.35819e-05_rb, 2.93582e-05_rb, 3.65494e-05_rb, &
     & 4.55021e-05_rb, 5.66476e-05_rb, 7.05233e-05_rb, 8.77978e-05_rb, 1.09304e-04_rb, &
     & 1.36077e-04_rb, 1.69409e-04_rb, 2.10905e-04_rb, 2.62565e-04_rb, 3.26880e-04_rb, &
     & 4.06948e-04_rb, 5.06629e-04_rb, 6.30726e-04_rb, 7.85219e-04_rb/)
      kao_mco2( 9, :, 7) = (/ &
     & 1.15683e-04_rb, 1.37544e-04_rb, 1.63535e-04_rb, 1.94438e-04_rb, 2.31180e-04_rb, &
     & 2.74866e-04_rb, 3.26807e-04_rb, 3.88563e-04_rb, 4.61989e-04_rb, 5.49289e-04_rb, &
     & 6.53088e-04_rb, 7.76501e-04_rb, 9.23234e-04_rb, 1.09770e-03_rb, 1.30512e-03_rb, &
     & 1.55175e-03_rb, 1.84498e-03_rb, 2.19362e-03_rb, 2.60815e-03_rb/)
      kao_mco2( 1, :, 8) = (/ &
     & 1.18154e-04_rb, 1.40516e-04_rb, 1.67111e-04_rb, 1.98739e-04_rb, 2.36353e-04_rb, &
     & 2.81086e-04_rb, 3.34285e-04_rb, 3.97553e-04_rb, 4.72796e-04_rb, 5.62278e-04_rb, &
     & 6.68697e-04_rb, 7.95257e-04_rb, 9.45770e-04_rb, 1.12477e-03_rb, 1.33765e-03_rb, &
     & 1.59081e-03_rb, 1.89190e-03_rb, 2.24996e-03_rb, 2.67580e-03_rb/)
      kao_mco2( 2, :, 8) = (/ &
     & 1.40874e-04_rb, 1.67009e-04_rb, 1.97993e-04_rb, 2.34726e-04_rb, 2.78273e-04_rb, &
     & 3.29899e-04_rb, 3.91102e-04_rb, 4.63661e-04_rb, 5.49680e-04_rb, 6.51659e-04_rb, &
     & 7.72556e-04_rb, 9.15884e-04_rb, 1.08580e-03_rb, 1.28724e-03_rb, 1.52605e-03_rb, &
     & 1.80917e-03_rb, 2.14482e-03_rb, 2.54273e-03_rb, 3.01446e-03_rb/)
      kao_mco2( 3, :, 8) = (/ &
     & 1.55092e-04_rb, 1.84132e-04_rb, 2.18609e-04_rb, 2.59542e-04_rb, 3.08140e-04_rb, &
     & 3.65837e-04_rb, 4.34337e-04_rb, 5.15664e-04_rb, 6.12219e-04_rb, 7.26853e-04_rb, &
     & 8.62952e-04_rb, 1.02453e-03_rb, 1.21637e-03_rb, 1.44413e-03_rb, 1.71453e-03_rb, &
     & 2.03557e-03_rb, 2.41671e-03_rb, 2.86923e-03_rb, 3.40647e-03_rb/)
      kao_mco2( 4, :, 8) = (/ &
     & 1.80666e-04_rb, 2.14521e-04_rb, 2.54721e-04_rb, 3.02454e-04_rb, 3.59131e-04_rb, &
     & 4.26429e-04_rb, 5.06339e-04_rb, 6.01223e-04_rb, 7.13887e-04_rb, 8.47663e-04_rb, &
     & 1.00651e-03_rb, 1.19512e-03_rb, 1.41908e-03_rb, 1.68500e-03_rb, 2.00076e-03_rb, &
     & 2.37568e-03_rb, 2.82087e-03_rb, 3.34947e-03_rb, 3.97714e-03_rb/)
      kao_mco2( 5, :, 8) = (/ &
     & 2.21554e-04_rb, 2.63265e-04_rb, 3.12829e-04_rb, 3.71724e-04_rb, 4.41707e-04_rb, &
     & 5.24865e-04_rb, 6.23679e-04_rb, 7.41096e-04_rb, 8.80619e-04_rb, 1.04641e-03_rb, &
     & 1.24341e-03_rb, 1.47750e-03_rb, 1.75567e-03_rb, 2.08620e-03_rb, 2.47896e-03_rb, &
     & 2.94566e-03_rb, 3.50023e-03_rb, 4.15920e-03_rb, 4.94224e-03_rb/)
      kao_mco2( 6, :, 8) = (/ &
     & 2.78997e-04_rb, 3.32548e-04_rb, 3.96378e-04_rb, 4.72460e-04_rb, 5.63146e-04_rb, &
     & 6.71238e-04_rb, 8.00077e-04_rb, 9.53647e-04_rb, 1.13669e-03_rb, 1.35487e-03_rb, &
     & 1.61493e-03_rb, 1.92491e-03_rb, 2.29438e-03_rb, 2.73477e-03_rb, 3.25969e-03_rb, &
     & 3.88537e-03_rb, 4.63114e-03_rb, 5.52005e-03_rb, 6.57958e-03_rb/)
      kao_mco2( 7, :, 8) = (/ &
     & 2.84939e-04_rb, 3.40606e-04_rb, 4.07149e-04_rb, 4.86691e-04_rb, 5.81774e-04_rb, &
     & 6.95432e-04_rb, 8.31295e-04_rb, 9.93700e-04_rb, 1.18783e-03_rb, 1.41989e-03_rb, &
     & 1.69729e-03_rb, 2.02888e-03_rb, 2.42526e-03_rb, 2.89907e-03_rb, 3.46544e-03_rb, &
     & 4.14246e-03_rb, 4.95176e-03_rb, 5.91915e-03_rb, 7.07554e-03_rb/)
      kao_mco2( 8, :, 8) = (/ &
     & 5.30764e-05_rb, 6.47812e-05_rb, 7.90673e-05_rb, 9.65039e-05_rb, 1.17786e-04_rb, &
     & 1.43761e-04_rb, 1.75464e-04_rb, 2.14159e-04_rb, 2.61387e-04_rb, 3.19030e-04_rb, &
     & 3.89385e-04_rb, 4.75255e-04_rb, 5.80062e-04_rb, 7.07982e-04_rb, 8.64111e-04_rb, &
     & 1.05467e-03_rb, 1.28726e-03_rb, 1.57113e-03_rb, 1.91761e-03_rb/)
      kao_mco2( 9, :, 8) = (/ &
     & 2.76806e-04_rb, 3.29639e-04_rb, 3.92556e-04_rb, 4.67481e-04_rb, 5.56708e-04_rb, &
     & 6.62964e-04_rb, 7.89501e-04_rb, 9.40190e-04_rb, 1.11964e-03_rb, 1.33334e-03_rb, &
     & 1.58783e-03_rb, 1.89089e-03_rb, 2.25180e-03_rb, 2.68159e-03_rb, 3.19341e-03_rb, &
     & 3.80293e-03_rb, 4.52878e-03_rb, 5.39316e-03_rb, 6.42253e-03_rb/)
      kao_mco2( 1, :, 9) = (/ &
     & 3.30614e-04_rb, 3.93289e-04_rb, 4.67844e-04_rb, 5.56534e-04_rb, 6.62036e-04_rb, &
     & 7.87539e-04_rb, 9.36833e-04_rb, 1.11443e-03_rb, 1.32569e-03_rb, 1.57700e-03_rb, &
     & 1.87596e-03_rb, 2.23158e-03_rb, 2.65463e-03_rb, 3.15787e-03_rb, 3.75650e-03_rb, &
     & 4.46862e-03_rb, 5.31575e-03_rb, 6.32345e-03_rb, 7.52219e-03_rb/)
      kao_mco2( 2, :, 9) = (/ &
     & 3.78453e-04_rb, 4.50735e-04_rb, 5.36824e-04_rb, 6.39355e-04_rb, 7.61469e-04_rb, &
     & 9.06906e-04_rb, 1.08012e-03_rb, 1.28642e-03_rb, 1.53212e-03_rb, 1.82475e-03_rb, &
     & 2.17326e-03_rb, 2.58835e-03_rb, 3.08271e-03_rb, 3.67149e-03_rb, 4.37273e-03_rb, &
     & 5.20790e-03_rb, 6.20259e-03_rb, 7.38725e-03_rb, 8.79818e-03_rb/)
      kao_mco2( 3, :, 9) = (/ &
     & 4.57576e-04_rb, 5.45512e-04_rb, 6.50348e-04_rb, 7.75330e-04_rb, 9.24332e-04_rb, &
     & 1.10197e-03_rb, 1.31374e-03_rb, 1.56621e-03_rb, 1.86721e-03_rb, 2.22604e-03_rb, &
     & 2.65384e-03_rb, 3.16385e-03_rb, 3.77187e-03_rb, 4.49675e-03_rb, 5.36092e-03_rb, &
     & 6.39117e-03_rb, 7.61942e-03_rb, 9.08370e-03_rb, 1.08294e-02_rb/)
      kao_mco2( 4, :, 9) = (/ &
     & 5.18277e-04_rb, 6.18764e-04_rb, 7.38735e-04_rb, 8.81967e-04_rb, 1.05297e-03_rb, &
     & 1.25713e-03_rb, 1.50087e-03_rb, 1.79187e-03_rb, 2.13929e-03_rb, 2.55407e-03_rb, &
     & 3.04928e-03_rb, 3.64050e-03_rb, 4.34635e-03_rb, 5.18905e-03_rb, 6.19514e-03_rb, &
     & 7.39631e-03_rb, 8.83036e-03_rb, 1.05425e-02_rb, 1.25865e-02_rb/)
      kao_mco2( 5, :, 9) = (/ &
     & 4.45365e-04_rb, 5.32106e-04_rb, 6.35742e-04_rb, 7.59563e-04_rb, 9.07500e-04_rb, &
     & 1.08425e-03_rb, 1.29542e-03_rb, 1.54773e-03_rb, 1.84917e-03_rb, 2.20933e-03_rb, &
     & 2.63963e-03_rb, 3.15374e-03_rb, 3.76797e-03_rb, 4.50184e-03_rb, 5.37865e-03_rb, &
     & 6.42622e-03_rb, 7.67783e-03_rb, 9.17320e-03_rb, 1.09598e-02_rb/)
      kao_mco2( 6, :, 9) = (/ &
     & 2.87301e-04_rb, 3.43009e-04_rb, 4.09519e-04_rb, 4.88926e-04_rb, 5.83730e-04_rb, &
     & 6.96916e-04_rb, 8.32050e-04_rb, 9.93386e-04_rb, 1.18601e-03_rb, 1.41597e-03_rb, &
     & 1.69053e-03_rb, 2.01833e-03_rb, 2.40969e-03_rb, 2.87693e-03_rb, 3.43478e-03_rb, &
     & 4.10079e-03_rb, 4.89594e-03_rb, 5.84527e-03_rb, 6.97867e-03_rb/)
      kao_mco2( 7, :, 9) = (/ &
     & 1.10743e-04_rb, 1.32286e-04_rb, 1.58020e-04_rb, 1.88760e-04_rb, 2.25480e-04_rb, &
     & 2.69342e-04_rb, 3.21738e-04_rb, 3.84326e-04_rb, 4.59090e-04_rb, 5.48397e-04_rb, &
     & 6.55078e-04_rb, 7.82511e-04_rb, 9.34734e-04_rb, 1.11657e-03_rb, 1.33378e-03_rb, &
     & 1.59324e-03_rb, 1.90318e-03_rb, 2.27340e-03_rb, 2.71565e-03_rb/)
      kao_mco2( 8, :, 9) = (/ &
     & 8.63177e-05_rb, 1.03067e-04_rb, 1.23066e-04_rb, 1.46946e-04_rb, 1.75459e-04_rb, &
     & 2.09505e-04_rb, 2.50158e-04_rb, 2.98698e-04_rb, 3.56658e-04_rb, 4.25864e-04_rb, &
     & 5.08498e-04_rb, 6.07168e-04_rb, 7.24982e-04_rb, 8.65658e-04_rb, 1.03363e-03_rb, &
     & 1.23420e-03_rb, 1.47368e-03_rb, 1.75963e-03_rb, 2.10107e-03_rb/)
      kao_mco2( 9, :, 9) = (/ &
     & 4.52715e-04_rb, 5.41540e-04_rb, 6.47792e-04_rb, 7.74892e-04_rb, 9.26929e-04_rb, &
     & 1.10880e-03_rb, 1.32635e-03_rb, 1.58658e-03_rb, 1.89787e-03_rb, 2.27024e-03_rb, &
     & 2.71568e-03_rb, 3.24850e-03_rb, 3.88587e-03_rb, 4.64830e-03_rb, 5.56031e-03_rb, &
     & 6.65127e-03_rb, 7.95627e-03_rb, 9.51732e-03_rb, 1.13847e-02_rb/)
      kao_mco2( 1, :,10) = (/ &
     & 9.10418e-04_rb, 1.08631e-03_rb, 1.29619e-03_rb, 1.54662e-03_rb, 1.84543e-03_rb, &
     & 2.20198e-03_rb, 2.62741e-03_rb, 3.13503e-03_rb, 3.74073e-03_rb, 4.46344e-03_rb, &
     & 5.32580e-03_rb, 6.35476e-03_rb, 7.58251e-03_rb, 9.04748e-03_rb, 1.07955e-02_rb, &
     & 1.28812e-02_rb, 1.53699e-02_rb, 1.83394e-02_rb, 2.18826e-02_rb/)
      kao_mco2( 2, :,10) = (/ &
     & 9.06680e-04_rb, 1.08622e-03_rb, 1.30130e-03_rb, 1.55898e-03_rb, 1.86768e-03_rb, &
     & 2.23750e-03_rb, 2.68056e-03_rb, 3.21135e-03_rb, 3.84724e-03_rb, 4.60905e-03_rb, &
     & 5.52171e-03_rb, 6.61508e-03_rb, 7.92496e-03_rb, 9.49421e-03_rb, 1.13742e-02_rb, &
     & 1.36265e-02_rb, 1.63247e-02_rb, 1.95572e-02_rb, 2.34298e-02_rb/)
      kao_mco2( 3, :,10) = (/ &
     & 8.17976e-04_rb, 9.79458e-04_rb, 1.17282e-03_rb, 1.40435e-03_rb, 1.68160e-03_rb, &
     & 2.01357e-03_rb, 2.41108e-03_rb, 2.88707e-03_rb, 3.45703e-03_rb, 4.13950e-03_rb, &
     & 4.95671e-03_rb, 5.93525e-03_rb, 7.10696e-03_rb, 8.51000e-03_rb, 1.01900e-02_rb, &
     & 1.22017e-02_rb, 1.46105e-02_rb, 1.74949e-02_rb, 2.09486e-02_rb/)
      kao_mco2( 4, :,10) = (/ &
     & 3.70314e-04_rb, 4.41440e-04_rb, 5.26226e-04_rb, 6.27298e-04_rb, 7.47782e-04_rb, &
     & 8.91407e-04_rb, 1.06262e-03_rb, 1.26671e-03_rb, 1.51001e-03_rb, 1.80003e-03_rb, &
     & 2.14576e-03_rb, 2.55789e-03_rb, 3.04918e-03_rb, 3.63483e-03_rb, 4.33297e-03_rb, &
     & 5.16520e-03_rb, 6.15727e-03_rb, 7.33988e-03_rb, 8.74963e-03_rb/)
      kao_mco2( 5, :,10) = (/ &
     & 1.00859e-04_rb, 1.19692e-04_rb, 1.42041e-04_rb, 1.68563e-04_rb, 2.00038e-04_rb, &
     & 2.37389e-04_rb, 2.81715e-04_rb, 3.34318e-04_rb, 3.96742e-04_rb, 4.70823e-04_rb, &
     & 5.58736e-04_rb, 6.63065e-04_rb, 7.86874e-04_rb, 9.33801e-04_rb, 1.10816e-03_rb, &
     & 1.31508e-03_rb, 1.56064e-03_rb, 1.85204e-03_rb, 2.19786e-03_rb/)
      kao_mco2( 6, :,10) = (/ &
     & 9.24477e-05_rb, 1.09659e-04_rb, 1.30074e-04_rb, 1.54290e-04_rb, 1.83015e-04_rb, &
     & 2.17087e-04_rb, 2.57503e-04_rb, 3.05442e-04_rb, 3.62307e-04_rb, 4.29759e-04_rb, &
     & 5.09768e-04_rb, 6.04672e-04_rb, 7.17245e-04_rb, 8.50776e-04_rb, 1.00917e-03_rb, &
     & 1.19704e-03_rb, 1.41990e-03_rb, 1.68425e-03_rb, 1.99780e-03_rb/)
      kao_mco2( 7, :,10) = (/ &
     & 8.42943e-05_rb, 1.00044e-04_rb, 1.18735e-04_rb, 1.40919e-04_rb, 1.67248e-04_rb, &
     & 1.98496e-04_rb, 2.35582e-04_rb, 2.79597e-04_rb, 3.31836e-04_rb, 3.93835e-04_rb, &
     & 4.67418e-04_rb, 5.54748e-04_rb, 6.58395e-04_rb, 7.81407e-04_rb, 9.27402e-04_rb, &
     & 1.10067e-03_rb, 1.30632e-03_rb, 1.55039e-03_rb, 1.84005e-03_rb/)
      kao_mco2( 8, :,10) = (/ &
     & 6.86464e-05_rb, 8.18163e-05_rb, 9.75129e-05_rb, 1.16221e-04_rb, 1.38518e-04_rb, &
     & 1.65093e-04_rb, 1.96767e-04_rb, 2.34517e-04_rb, 2.79509e-04_rb, 3.33133e-04_rb, &
     & 3.97046e-04_rb, 4.73220e-04_rb, 5.64008e-04_rb, 6.72214e-04_rb, 8.01179e-04_rb, &
     & 9.54887e-04_rb, 1.13808e-03_rb, 1.35643e-03_rb, 1.61666e-03_rb/)
      kao_mco2( 9, :,10) = (/ &
     & 1.03095e-04_rb, 1.21985e-04_rb, 1.44335e-04_rb, 1.70781e-04_rb, 2.02072e-04_rb, &
     & 2.39096e-04_rb, 2.82904e-04_rb, 3.34739e-04_rb, 3.96070e-04_rb, 4.68639e-04_rb, &
     & 5.54505e-04_rb, 6.56103e-04_rb, 7.76316e-04_rb, 9.18556e-04_rb, 1.08686e-03_rb, &
     & 1.28599e-03_rb, 1.52162e-03_rb, 1.80041e-03_rb, 2.13029e-03_rb/)
      kao_mco2( 1, :,11) = (/ &
     & 1.01275e-03_rb, 1.21433e-03_rb, 1.45605e-03_rb, 1.74587e-03_rb, 2.09339e-03_rb, &
     & 2.51007e-03_rb, 3.00970e-03_rb, 3.60878e-03_rb, 4.32711e-03_rb, 5.18842e-03_rb, &
     & 6.22117e-03_rb, 7.45950e-03_rb, 8.94430e-03_rb, 1.07247e-02_rb, 1.28594e-02_rb, &
     & 1.54191e-02_rb, 1.84882e-02_rb, 2.21683e-02_rb, 2.65809e-02_rb/)
      kao_mco2( 2, :,11) = (/ &
     & 1.06856e-03_rb, 1.27885e-03_rb, 1.53052e-03_rb, 1.83171e-03_rb, 2.19218e-03_rb, &
     & 2.62359e-03_rb, 3.13990e-03_rb, 3.75781e-03_rb, 4.49732e-03_rb, 5.38236e-03_rb, &
     & 6.44158e-03_rb, 7.70924e-03_rb, 9.22637e-03_rb, 1.10421e-02_rb, 1.32151e-02_rb, &
     & 1.58157e-02_rb, 1.89281e-02_rb, 2.26531e-02_rb, 2.71110e-02_rb/)
      kao_mco2( 3, :,11) = (/ &
     & 7.34896e-04_rb, 8.77863e-04_rb, 1.04864e-03_rb, 1.25265e-03_rb, 1.49634e-03_rb, &
     & 1.78744e-03_rb, 2.13516e-03_rb, 2.55054e-03_rb, 3.04672e-03_rb, 3.63943e-03_rb, &
     & 4.34745e-03_rb, 5.19321e-03_rb, 6.20349e-03_rb, 7.41032e-03_rb, 8.85192e-03_rb, &
     & 1.05740e-02_rb, 1.26311e-02_rb, 1.50883e-02_rb, 1.80236e-02_rb/)
      kao_mco2( 4, :,11) = (/ &
     & 5.89491e-05_rb, 7.12560e-05_rb, 8.61322e-05_rb, 1.04114e-04_rb, 1.25850e-04_rb, &
     & 1.52124e-04_rb, 1.83883e-04_rb, 2.22272e-04_rb, 2.68676e-04_rb, 3.24768e-04_rb, &
     & 3.92571e-04_rb, 4.74528e-04_rb, 5.73595e-04_rb, 6.93346e-04_rb, 8.38096e-04_rb, &
     & 1.01307e-03_rb, 1.22457e-03_rb, 1.48022e-03_rb, 1.78924e-03_rb/)
      kao_mco2( 5, :,11) = (/ &
     & 5.32400e-05_rb, 6.45465e-05_rb, 7.82542e-05_rb, 9.48731e-05_rb, 1.15021e-04_rb, &
     & 1.39448e-04_rb, 1.69063e-04_rb, 2.04966e-04_rb, 2.48495e-04_rb, 3.01268e-04_rb, &
     & 3.65248e-04_rb, 4.42816e-04_rb, 5.36856e-04_rb, 6.50868e-04_rb, 7.89092e-04_rb, &
     & 9.56672e-04_rb, 1.15984e-03_rb, 1.40615e-03_rb, 1.70478e-03_rb/)
      kao_mco2( 6, :,11) = (/ &
     & 5.31408e-05_rb, 6.42409e-05_rb, 7.76597e-05_rb, 9.38814e-05_rb, 1.13491e-04_rb, &
     & 1.37198e-04_rb, 1.65856e-04_rb, 2.00500e-04_rb, 2.42381e-04_rb, 2.93010e-04_rb, &
     & 3.54214e-04_rb, 4.28203e-04_rb, 5.17647e-04_rb, 6.25774e-04_rb, 7.56486e-04_rb, &
     & 9.14503e-04_rb, 1.10553e-03_rb, 1.33645e-03_rb, 1.61561e-03_rb/)
      kao_mco2( 7, :,11) = (/ &
     & 5.24517e-05_rb, 6.32485e-05_rb, 7.62676e-05_rb, 9.19667e-05_rb, 1.10897e-04_rb, &
     & 1.33725e-04_rb, 1.61251e-04_rb, 1.94443e-04_rb, 2.34467e-04_rb, 2.82730e-04_rb, &
     & 3.40928e-04_rb, 4.11106e-04_rb, 4.95728e-04_rb, 5.97770e-04_rb, 7.20816e-04_rb, &
     & 8.69190e-04_rb, 1.04811e-03_rb, 1.26385e-03_rb, 1.52400e-03_rb/)
      kao_mco2( 8, :,11) = (/ &
     & 5.01768e-05_rb, 6.02217e-05_rb, 7.22774e-05_rb, 8.67466e-05_rb, 1.04112e-04_rb, &
     & 1.24955e-04_rb, 1.49969e-04_rb, 1.79991e-04_rb, 2.16024e-04_rb, 2.59270e-04_rb, &
     & 3.11173e-04_rb, 3.73467e-04_rb, 4.48231e-04_rb, 5.37962e-04_rb, 6.45656e-04_rb, &
     & 7.74910e-04_rb, 9.30039e-04_rb, 1.11622e-03_rb, 1.33968e-03_rb/)
      kao_mco2( 9, :,11) = (/ &
     & 5.46391e-05_rb, 6.58765e-05_rb, 7.94252e-05_rb, 9.57603e-05_rb, 1.15455e-04_rb, &
     & 1.39200e-04_rb, 1.67829e-04_rb, 2.02346e-04_rb, 2.43962e-04_rb, 2.94137e-04_rb, &
     & 3.54632e-04_rb, 4.27568e-04_rb, 5.15504e-04_rb, 6.21526e-04_rb, 7.49353e-04_rb, &
     & 9.03471e-04_rb, 1.08929e-03_rb, 1.31331e-03_rb, 1.58342e-03_rb/)
      kao_mco2( 1, :,12) = (/ &
     & 1.18469e-03_rb, 1.41755e-03_rb, 1.69619e-03_rb, 2.02959e-03_rb, 2.42854e-03_rb, &
     & 2.90589e-03_rb, 3.47708e-03_rb, 4.16055e-03_rb, 4.97836e-03_rb, 5.95691e-03_rb, &
     & 7.12782e-03_rb, 8.52889e-03_rb, 1.02053e-02_rb, 1.22113e-02_rb, 1.46116e-02_rb, &
     & 1.74837e-02_rb, 2.09204e-02_rb, 2.50325e-02_rb, 2.99530e-02_rb/)
      kao_mco2( 2, :,12) = (/ &
     & 1.09092e-03_rb, 1.30288e-03_rb, 1.55602e-03_rb, 1.85834e-03_rb, 2.21940e-03_rb, &
     & 2.65061e-03_rb, 3.16560e-03_rb, 3.78064e-03_rb, 4.51519e-03_rb, 5.39245e-03_rb, &
     & 6.44016e-03_rb, 7.69143e-03_rb, 9.18580e-03_rb, 1.09705e-02_rb, 1.31020e-02_rb, &
     & 1.56476e-02_rb, 1.86878e-02_rb, 2.23187e-02_rb, 2.66550e-02_rb/)
      kao_mco2( 3, :,12) = (/ &
     & 3.97521e-04_rb, 4.74103e-04_rb, 5.65438e-04_rb, 6.74369e-04_rb, 8.04285e-04_rb, &
     & 9.59228e-04_rb, 1.14402e-03_rb, 1.36442e-03_rb, 1.62727e-03_rb, 1.94076e-03_rb, &
     & 2.31464e-03_rb, 2.76055e-03_rb, 3.29237e-03_rb, 3.92663e-03_rb, 4.68309e-03_rb, &
     & 5.58528e-03_rb, 6.66128e-03_rb, 7.94456e-03_rb, 9.47505e-03_rb/)
      kao_mco2( 4, :,12) = (/ &
     & 7.18557e-05_rb, 8.56230e-05_rb, 1.02028e-04_rb, 1.21576e-04_rb, 1.44870e-04_rb, &
     & 1.72626e-04_rb, 2.05701e-04_rb, 2.45112e-04_rb, 2.92075e-04_rb, 3.48035e-04_rb, &
     & 4.14718e-04_rb, 4.94176e-04_rb, 5.88858e-04_rb, 7.01682e-04_rb, 8.36121e-04_rb, &
     & 9.96319e-04_rb, 1.18721e-03_rb, 1.41467e-03_rb, 1.68572e-03_rb/)
      kao_mco2( 5, :,12) = (/ &
     & 7.33026e-05_rb, 8.69077e-05_rb, 1.03038e-04_rb, 1.22162e-04_rb, 1.44836e-04_rb, &
     & 1.71717e-04_rb, 2.03588e-04_rb, 2.41375e-04_rb, 2.86175e-04_rb, 3.39289e-04_rb, &
     & 4.02262e-04_rb, 4.76923e-04_rb, 5.65440e-04_rb, 6.70387e-04_rb, 7.94812e-04_rb, &
     & 9.42331e-04_rb, 1.11723e-03_rb, 1.32459e-03_rb, 1.57044e-03_rb/)
      kao_mco2( 6, :,12) = (/ &
     & 7.44053e-05_rb, 8.82167e-05_rb, 1.04592e-04_rb, 1.24007e-04_rb, 1.47025e-04_rb, &
     & 1.74317e-04_rb, 2.06674e-04_rb, 2.45038e-04_rb, 2.90523e-04_rb, 3.44451e-04_rb, &
     & 4.08389e-04_rb, 4.84196e-04_rb, 5.74074e-04_rb, 6.80637e-04_rb, 8.06979e-04_rb, &
     & 9.56774e-04_rb, 1.13437e-03_rb, 1.34494e-03_rb, 1.59459e-03_rb/)
      kao_mco2( 7, :,12) = (/ &
     & 7.68762e-05_rb, 9.11305e-05_rb, 1.08028e-04_rb, 1.28058e-04_rb, 1.51802e-04_rb, &
     & 1.79949e-04_rb, 2.13315e-04_rb, 2.52868e-04_rb, 2.99754e-04_rb, 3.55334e-04_rb, &
     & 4.21220e-04_rb, 4.99322e-04_rb, 5.91905e-04_rb, 7.01656e-04_rb, 8.31756e-04_rb, &
     & 9.85979e-04_rb, 1.16880e-03_rb, 1.38551e-03_rb, 1.64241e-03_rb/)
      kao_mco2( 8, :,12) = (/ &
     & 8.45996e-05_rb, 1.00214e-04_rb, 1.18711e-04_rb, 1.40622e-04_rb, 1.66577e-04_rb, &
     & 1.97323e-04_rb, 2.33743e-04_rb, 2.76885e-04_rb, 3.27991e-04_rb, 3.88529e-04_rb, &
     & 4.60241e-04_rb, 5.45189e-04_rb, 6.45816e-04_rb, 7.65016e-04_rb, 9.06216e-04_rb, &
     & 1.07348e-03_rb, 1.27161e-03_rb, 1.50632e-03_rb, 1.78434e-03_rb/)
      kao_mco2( 9, :,12) = (/ &
     & 7.73583e-05_rb, 9.16767e-05_rb, 1.08645e-04_rb, 1.28755e-04_rb, 1.52586e-04_rb, &
     & 1.80829e-04_rb, 2.14299e-04_rb, 2.53964e-04_rb, 3.00970e-04_rb, 3.56678e-04_rb, &
     & 4.22696e-04_rb, 5.00934e-04_rb, 5.93652e-04_rb, 7.03533e-04_rb, 8.33751e-04_rb, &
     & 9.88072e-04_rb, 1.17096e-03_rb, 1.38769e-03_rb, 1.64454e-03_rb/)
      kao_mco2( 1, :,13) = (/ &
     & 1.20952e-03_rb, 1.44504e-03_rb, 1.72642e-03_rb, 2.06260e-03_rb, 2.46423e-03_rb, &
     & 2.94407e-03_rb, 3.51735e-03_rb, 4.20226e-03_rb, 5.02053e-03_rb, 5.99814e-03_rb, &
     & 7.16612e-03_rb, 8.56153e-03_rb, 1.02287e-02_rb, 1.22204e-02_rb, 1.46000e-02_rb, &
     & 1.74430e-02_rb, 2.08395e-02_rb, 2.48974e-02_rb, 2.97455e-02_rb/)
      kao_mco2( 2, :,13) = (/ &
     & 8.47667e-04_rb, 1.01027e-03_rb, 1.20407e-03_rb, 1.43505e-03_rb, 1.71034e-03_rb, &
     & 2.03843e-03_rb, 2.42946e-03_rb, 2.89550e-03_rb, 3.45094e-03_rb, 4.11293e-03_rb, &
     & 4.90192e-03_rb, 5.84225e-03_rb, 6.96296e-03_rb, 8.29866e-03_rb, 9.89058e-03_rb, &
     & 1.17879e-02_rb, 1.40492e-02_rb, 1.67442e-02_rb, 1.99562e-02_rb/)
      kao_mco2( 3, :,13) = (/ &
     & 1.45612e-04_rb, 1.71739e-04_rb, 2.02554e-04_rb, 2.38897e-04_rb, 2.81762e-04_rb, &
     & 3.32318e-04_rb, 3.91945e-04_rb, 4.62271e-04_rb, 5.45215e-04_rb, 6.43041e-04_rb, &
     & 7.58421e-04_rb, 8.94503e-04_rb, 1.05500e-03_rb, 1.24430e-03_rb, 1.46756e-03_rb, &
     & 1.73088e-03_rb, 2.04145e-03_rb, 2.40774e-03_rb, 2.83975e-03_rb/)
      kao_mco2( 4, :,13) = (/ &
     & 1.40167e-04_rb, 1.65266e-04_rb, 1.94858e-04_rb, 2.29750e-04_rb, 2.70889e-04_rb, &
     & 3.19394e-04_rb, 3.76585e-04_rb, 4.44016e-04_rb, 5.23522e-04_rb, 6.17264e-04_rb, &
     & 7.27791e-04_rb, 8.58110e-04_rb, 1.01176e-03_rb, 1.19293e-03_rb, 1.40654e-03_rb, &
     & 1.65839e-03_rb, 1.95534e-03_rb, 2.30547e-03_rb, 2.71828e-03_rb/)
      kao_mco2( 5, :,13) = (/ &
     & 1.37406e-04_rb, 1.61990e-04_rb, 1.90973e-04_rb, 2.25141e-04_rb, 2.65423e-04_rb, &
     & 3.12911e-04_rb, 3.68896e-04_rb, 4.34898e-04_rb, 5.12709e-04_rb, 6.04442e-04_rb, &
     & 7.12587e-04_rb, 8.40082e-04_rb, 9.90387e-04_rb, 1.16758e-03_rb, 1.37648e-03_rb, &
     & 1.62276e-03_rb, 1.91310e-03_rb, 2.25539e-03_rb, 2.65892e-03_rb/)
      kao_mco2( 6, :,13) = (/ &
     & 1.35356e-04_rb, 1.59577e-04_rb, 1.88132e-04_rb, 2.21797e-04_rb, 2.61485e-04_rb, &
     & 3.08276e-04_rb, 3.63440e-04_rb, 4.28475e-04_rb, 5.05147e-04_rb, 5.95539e-04_rb, &
     & 7.02106e-04_rb, 8.27743e-04_rb, 9.75861e-04_rb, 1.15048e-03_rb, 1.35635e-03_rb, &
     & 1.59906e-03_rb, 1.88520e-03_rb, 2.22255e-03_rb, 2.62025e-03_rb/)
      kao_mco2( 7, :,13) = (/ &
     & 1.33359e-04_rb, 1.57252e-04_rb, 1.85424e-04_rb, 2.18645e-04_rb, 2.57817e-04_rb, &
     & 3.04007e-04_rb, 3.58472e-04_rb, 4.22695e-04_rb, 4.98425e-04_rb, 5.87722e-04_rb, &
     & 6.93017e-04_rb, 8.17177e-04_rb, 9.63581e-04_rb, 1.13621e-03_rb, 1.33978e-03_rb, &
     & 1.57981e-03_rb, 1.86284e-03_rb, 2.19659e-03_rb, 2.59012e-03_rb/)
      kao_mco2( 8, :,13) = (/ &
     & 1.29667e-04_rb, 1.53001e-04_rb, 1.80534e-04_rb, 2.13022e-04_rb, 2.51356e-04_rb, &
     & 2.96589e-04_rb, 3.49961e-04_rb, 4.12938e-04_rb, 4.87249e-04_rb, 5.74931e-04_rb, &
     & 6.78393e-04_rb, 8.00473e-04_rb, 9.44521e-04_rb, 1.11449e-03_rb, 1.31505e-03_rb, &
     & 1.55170e-03_rb, 1.83094e-03_rb, 2.16042e-03_rb, 2.54920e-03_rb/)
      kao_mco2( 9, :,13) = (/ &
     & 1.37892e-04_rb, 1.62557e-04_rb, 1.91635e-04_rb, 2.25914e-04_rb, 2.66324e-04_rb, &
     & 3.13963e-04_rb, 3.70124e-04_rb, 4.36330e-04_rb, 5.14379e-04_rb, 6.06389e-04_rb, &
     & 7.14858e-04_rb, 8.42730e-04_rb, 9.93473e-04_rb, 1.17118e-03_rb, 1.38068e-03_rb, &
     & 1.62765e-03_rb, 1.91880e-03_rb, 2.26202e-03_rb, 2.66665e-03_rb/)
      kao_mco2( 1, :,14) = (/ &
     & 1.28098e-03_rb, 1.52939e-03_rb, 1.82597e-03_rb, 2.18007e-03_rb, 2.60284e-03_rb, &
     & 3.10759e-03_rb, 3.71022e-03_rb, 4.42972e-03_rb, 5.28874e-03_rb, 6.31435e-03_rb, &
     & 7.53885e-03_rb, 9.00081e-03_rb, 1.07463e-02_rb, 1.28302e-02_rb, 1.53183e-02_rb, &
     & 1.82889e-02_rb, 2.18355e-02_rb, 2.60699e-02_rb, 3.11255e-02_rb/)
      kao_mco2( 2, :,14) = (/ &
     & 1.27275e-04_rb, 1.48842e-04_rb, 1.74064e-04_rb, 2.03561e-04_rb, 2.38055e-04_rb, &
     & 2.78395e-04_rb, 3.25570e-04_rb, 3.80740e-04_rb, 4.45259e-04_rb, 5.20710e-04_rb, &
     & 6.08947e-04_rb, 7.12137e-04_rb, 8.32812e-04_rb, 9.73937e-04_rb, 1.13898e-03_rb, &
     & 1.33198e-03_rb, 1.55769e-03_rb, 1.82165e-03_rb, 2.13034e-03_rb/)
      kao_mco2( 3, :,14) = (/ &
     & 1.27744e-04_rb, 1.49255e-04_rb, 1.74389e-04_rb, 2.03755e-04_rb, 2.38066e-04_rb, &
     & 2.78155e-04_rb, 3.24995e-04_rb, 3.79722e-04_rb, 4.43666e-04_rb, 5.18376e-04_rb, &
     & 6.05668e-04_rb, 7.07660e-04_rb, 8.26826e-04_rb, 9.66059e-04_rb, 1.12874e-03_rb, &
     & 1.31881e-03_rb, 1.54089e-03_rb, 1.80037e-03_rb, 2.10354e-03_rb/)
      kao_mco2( 4, :,14) = (/ &
     & 1.28543e-04_rb, 1.50136e-04_rb, 1.75357e-04_rb, 2.04814e-04_rb, 2.39219e-04_rb, &
     & 2.79404e-04_rb, 3.26339e-04_rb, 3.81159e-04_rb, 4.45188e-04_rb, 5.19972e-04_rb, &
     & 6.07319e-04_rb, 7.09339e-04_rb, 8.28496e-04_rb, 9.67670e-04_rb, 1.13022e-03_rb, &
     & 1.32008e-03_rb, 1.54184e-03_rb, 1.80084e-03_rb, 2.10335e-03_rb/)
      kao_mco2( 5, :,14) = (/ &
     & 1.29218e-04_rb, 1.50897e-04_rb, 1.76214e-04_rb, 2.05778e-04_rb, 2.40302e-04_rb, &
     & 2.80618e-04_rb, 3.27698e-04_rb, 3.82678e-04_rb, 4.46881e-04_rb, 5.21855e-04_rb, &
     & 6.09409e-04_rb, 7.11652e-04_rb, 8.31048e-04_rb, 9.70475e-04_rb, 1.13330e-03_rb, &
     & 1.32343e-03_rb, 1.54547e-03_rb, 1.80476e-03_rb, 2.10755e-03_rb/)
      kao_mco2( 6, :,14) = (/ &
     & 1.30502e-04_rb, 1.52368e-04_rb, 1.77898e-04_rb, 2.07706e-04_rb, 2.42508e-04_rb, &
     & 2.83141e-04_rb, 3.30583e-04_rb, 3.85974e-04_rb, 4.50646e-04_rb, 5.26153e-04_rb, &
     & 6.14313e-04_rb, 7.17244e-04_rb, 8.37422e-04_rb, 9.77736e-04_rb, 1.14156e-03_rb, &
     & 1.33283e-03_rb, 1.55616e-03_rb, 1.81690e-03_rb, 2.12133e-03_rb/)
      kao_mco2( 7, :,14) = (/ &
     & 1.32820e-04_rb, 1.55041e-04_rb, 1.80980e-04_rb, 2.11259e-04_rb, 2.46604e-04_rb, &
     & 2.87862e-04_rb, 3.36022e-04_rb, 3.92240e-04_rb, 4.57864e-04_rb, 5.34467e-04_rb, &
     & 6.23886e-04_rb, 7.28265e-04_rb, 8.50107e-04_rb, 9.92334e-04_rb, 1.15836e-03_rb, &
     & 1.35215e-03_rb, 1.57838e-03_rb, 1.84244e-03_rb, 2.15069e-03_rb/)
      kao_mco2( 8, :,14) = (/ &
     & 1.40203e-04_rb, 1.63590e-04_rb, 1.90879e-04_rb, 2.22720e-04_rb, 2.59872e-04_rb, &
     & 3.03221e-04_rb, 3.53801e-04_rb, 4.12819e-04_rb, 4.81681e-04_rb, 5.62031e-04_rb, &
     & 6.55783e-04_rb, 7.65175e-04_rb, 8.92814e-04_rb, 1.04174e-03_rb, 1.21552e-03_rb, &
     & 1.41828e-03_rb, 1.65486e-03_rb, 1.93091e-03_rb, 2.25301e-03_rb/)
      kao_mco2( 9, :,14) = (/ &
     & 1.30642e-04_rb, 1.52513e-04_rb, 1.78046e-04_rb, 2.07853e-04_rb, 2.42651e-04_rb, &
     & 2.83275e-04_rb, 3.30699e-04_rb, 3.86063e-04_rb, 4.50696e-04_rb, 5.26149e-04_rb, &
     & 6.14234e-04_rb, 7.17066e-04_rb, 8.37113e-04_rb, 9.77259e-04_rb, 1.14087e-03_rb, &
     & 1.33186e-03_rb, 1.55484e-03_rb, 1.81514e-03_rb, 2.11902e-03_rb/)
      kao_mco2( 1, :,15) = (/ &
     & 1.37603e-03_rb, 1.64035e-03_rb, 1.95543e-03_rb, 2.33105e-03_rb, 2.77881e-03_rb, &
     & 3.31257e-03_rb, 3.94887e-03_rb, 4.70739e-03_rb, 5.61162e-03_rb, 6.68952e-03_rb, &
     & 7.97449e-03_rb, 9.50627e-03_rb, 1.13323e-02_rb, 1.35091e-02_rb, 1.61039e-02_rb, &
     & 1.91973e-02_rb, 2.28848e-02_rb, 2.72806e-02_rb, 3.25208e-02_rb/)
      kao_mco2( 2, :,15) = (/ &
     & 1.67843e-04_rb, 1.93707e-04_rb, 2.23557e-04_rb, 2.58007e-04_rb, 2.97765e-04_rb, &
     & 3.43650e-04_rb, 3.96606e-04_rb, 4.57722e-04_rb, 5.28256e-04_rb, 6.09659e-04_rb, &
     & 7.03606e-04_rb, 8.12031e-04_rb, 9.37163e-04_rb, 1.08158e-03_rb, 1.24825e-03_rb, &
     & 1.44060e-03_rb, 1.66259e-03_rb, 1.91880e-03_rb, 2.21448e-03_rb/)
      kao_mco2( 3, :,15) = (/ &
     & 1.67595e-04_rb, 1.93410e-04_rb, 2.23200e-04_rb, 2.57579e-04_rb, 2.97253e-04_rb, &
     & 3.43039e-04_rb, 3.95876e-04_rb, 4.56852e-04_rb, 5.27220e-04_rb, 6.08426e-04_rb, &
     & 7.02141e-04_rb, 8.10291e-04_rb, 9.35098e-04_rb, 1.07913e-03_rb, 1.24534e-03_rb, &
     & 1.43716e-03_rb, 1.65853e-03_rb, 1.91398e-03_rb, 2.20879e-03_rb/)
      kao_mco2( 4, :,15) = (/ &
     & 1.67354e-04_rb, 1.93130e-04_rb, 2.22877e-04_rb, 2.57206e-04_rb, 2.96823e-04_rb, &
     & 3.42541e-04_rb, 3.95301e-04_rb, 4.56187e-04_rb, 5.26452e-04_rb, 6.07539e-04_rb, &
     & 7.01116e-04_rb, 8.09106e-04_rb, 9.33728e-04_rb, 1.07755e-03_rb, 1.24352e-03_rb, &
     & 1.43505e-03_rb, 1.65608e-03_rb, 1.91116e-03_rb, 2.20553e-03_rb/)
      kao_mco2( 5, :,15) = (/ &
     & 1.67437e-04_rb, 1.93232e-04_rb, 2.23002e-04_rb, 2.57358e-04_rb, 2.97006e-04_rb, &
     & 3.42763e-04_rb, 3.95570e-04_rb, 4.56511e-04_rb, 5.26842e-04_rb, 6.08007e-04_rb, &
     & 7.01677e-04_rb, 8.09778e-04_rb, 9.34533e-04_rb, 1.07851e-03_rb, 1.24466e-03_rb, &
     & 1.43642e-03_rb, 1.65771e-03_rb, 1.91310e-03_rb, 2.20783e-03_rb/)
      kao_mco2( 6, :,15) = (/ &
     & 1.67267e-04_rb, 1.93027e-04_rb, 2.22753e-04_rb, 2.57057e-04_rb, 2.96645e-04_rb, &
     & 3.42328e-04_rb, 3.95047e-04_rb, 4.55885e-04_rb, 5.26092e-04_rb, 6.07110e-04_rb, &
     & 7.00606e-04_rb, 8.08500e-04_rb, 9.33010e-04_rb, 1.07669e-03_rb, 1.24251e-03_rb, &
     & 1.43385e-03_rb, 1.65467e-03_rb, 1.90949e-03_rb, 2.20355e-03_rb/)
      kao_mco2( 7, :,15) = (/ &
     & 1.67354e-04_rb, 1.93130e-04_rb, 2.22877e-04_rb, 2.57206e-04_rb, 2.96823e-04_rb, &
     & 3.42541e-04_rb, 3.95301e-04_rb, 4.56187e-04_rb, 5.26452e-04_rb, 6.07539e-04_rb, &
     & 7.01116e-04_rb, 8.09106e-04_rb, 9.33728e-04_rb, 1.07755e-03_rb, 1.24352e-03_rb, &
     & 1.43505e-03_rb, 1.65608e-03_rb, 1.91116e-03_rb, 2.20553e-03_rb/)
      kao_mco2( 8, :,15) = (/ &
     & 1.67276e-04_rb, 1.93038e-04_rb, 2.22769e-04_rb, 2.57079e-04_rb, 2.96673e-04_rb, &
     & 3.42365e-04_rb, 3.95094e-04_rb, 4.55944e-04_rb, 5.26166e-04_rb, 6.07203e-04_rb, &
     & 7.00722e-04_rb, 8.08643e-04_rb, 9.33186e-04_rb, 1.07691e-03_rb, 1.24277e-03_rb, &
     & 1.43417e-03_rb, 1.65506e-03_rb, 1.90996e-03_rb, 2.20412e-03_rb/)
      kao_mco2( 9, :,15) = (/ &
     & 1.67437e-04_rb, 1.93232e-04_rb, 2.23002e-04_rb, 2.57358e-04_rb, 2.97006e-04_rb, &
     & 3.42763e-04_rb, 3.95570e-04_rb, 4.56511e-04_rb, 5.26842e-04_rb, 6.08007e-04_rb, &
     & 7.01677e-04_rb, 8.09778e-04_rb, 9.34533e-04_rb, 1.07851e-03_rb, 1.24466e-03_rb, &
     & 1.43642e-03_rb, 1.65771e-03_rb, 1.91310e-03_rb, 2.20783e-03_rb/)
      kao_mco2( 1, :,16) = (/ &
     & 1.42104e-03_rb, 1.69791e-03_rb, 2.02872e-03_rb, 2.42399e-03_rb, 2.89626e-03_rb, &
     & 3.46055e-03_rb, 4.13478e-03_rb, 4.94038e-03_rb, 5.90294e-03_rb, 7.05303e-03_rb, &
     & 8.42720e-03_rb, 1.00691e-02_rb, 1.20309e-02_rb, 1.43749e-02_rb, 1.71757e-02_rb, &
     & 2.05221e-02_rb, 2.45205e-02_rb, 2.92979e-02_rb, 3.50061e-02_rb/)
      kao_mco2( 2, :,16) = (/ &
     & 1.63777e-04_rb, 1.88736e-04_rb, 2.17498e-04_rb, 2.50643e-04_rb, 2.88839e-04_rb, &
     & 3.32857e-04_rb, 3.83582e-04_rb, 4.42037e-04_rb, 5.09401e-04_rb, 5.87030e-04_rb, &
     & 6.76490e-04_rb, 7.79583e-04_rb, 8.98386e-04_rb, 1.03530e-03_rb, 1.19307e-03_rb, &
     & 1.37488e-03_rb, 1.58441e-03_rb, 1.82586e-03_rb, 2.10411e-03_rb/)
      kao_mco2( 3, :,16) = (/ &
     & 1.63679e-04_rb, 1.88621e-04_rb, 2.17365e-04_rb, 2.50489e-04_rb, 2.88661e-04_rb, &
     & 3.32650e-04_rb, 3.83342e-04_rb, 4.41759e-04_rb, 5.09079e-04_rb, 5.86657e-04_rb, &
     & 6.76057e-04_rb, 7.79080e-04_rb, 8.97804e-04_rb, 1.03462e-03_rb, 1.19228e-03_rb, &
     & 1.37397e-03_rb, 1.58335e-03_rb, 1.82464e-03_rb, 2.10269e-03_rb/)
      kao_mco2( 4, :,16) = (/ &
     & 1.63679e-04_rb, 1.88621e-04_rb, 2.17365e-04_rb, 2.50489e-04_rb, 2.88661e-04_rb, &
     & 3.32650e-04_rb, 3.83342e-04_rb, 4.41759e-04_rb, 5.09079e-04_rb, 5.86657e-04_rb, &
     & 6.76057e-04_rb, 7.79080e-04_rb, 8.97804e-04_rb, 1.03462e-03_rb, 1.19228e-03_rb, &
     & 1.37397e-03_rb, 1.58335e-03_rb, 1.82464e-03_rb, 2.10269e-03_rb/)
      kao_mco2( 5, :,16) = (/ &
     & 1.63586e-04_rb, 1.88513e-04_rb, 2.17239e-04_rb, 2.50343e-04_rb, 2.88490e-04_rb, &
     & 3.32451e-04_rb, 3.83111e-04_rb, 4.41490e-04_rb, 5.08766e-04_rb, 5.86292e-04_rb, &
     & 6.75633e-04_rb, 7.78588e-04_rb, 8.97231e-04_rb, 1.03395e-03_rb, 1.19151e-03_rb, &
     & 1.37307e-03_rb, 1.58231e-03_rb, 1.82342e-03_rb, 2.10128e-03_rb/)
      kao_mco2( 6, :,16) = (/ &
     & 1.63679e-04_rb, 1.88621e-04_rb, 2.17365e-04_rb, 2.50489e-04_rb, 2.88661e-04_rb, &
     & 3.32650e-04_rb, 3.83342e-04_rb, 4.41759e-04_rb, 5.09079e-04_rb, 5.86657e-04_rb, &
     & 6.76057e-04_rb, 7.79080e-04_rb, 8.97804e-04_rb, 1.03462e-03_rb, 1.19228e-03_rb, &
     & 1.37397e-03_rb, 1.58335e-03_rb, 1.82464e-03_rb, 2.10269e-03_rb/)
      kao_mco2( 7, :,16) = (/ &
     & 1.63679e-04_rb, 1.88621e-04_rb, 2.17365e-04_rb, 2.50489e-04_rb, 2.88661e-04_rb, &
     & 3.32650e-04_rb, 3.83342e-04_rb, 4.41759e-04_rb, 5.09079e-04_rb, 5.86657e-04_rb, &
     & 6.76057e-04_rb, 7.79080e-04_rb, 8.97804e-04_rb, 1.03462e-03_rb, 1.19228e-03_rb, &
     & 1.37397e-03_rb, 1.58335e-03_rb, 1.82464e-03_rb, 2.10269e-03_rb/)
      kao_mco2( 8, :,16) = (/ &
     & 1.63479e-04_rb, 1.88391e-04_rb, 2.17098e-04_rb, 2.50180e-04_rb, 2.88303e-04_rb, &
     & 3.32236e-04_rb, 3.82863e-04_rb, 4.41205e-04_rb, 5.08437e-04_rb, 5.85914e-04_rb, &
     & 6.75198e-04_rb, 7.78087e-04_rb, 8.96654e-04_rb, 1.03329e-03_rb, 1.19074e-03_rb, &
     & 1.37219e-03_rb, 1.58129e-03_rb, 1.82226e-03_rb, 2.09994e-03_rb/)
      kao_mco2( 9, :,16) = (/ &
     & 1.63586e-04_rb, 1.88513e-04_rb, 2.17239e-04_rb, 2.50343e-04_rb, 2.88490e-04_rb, &
     & 3.32451e-04_rb, 3.83111e-04_rb, 4.41490e-04_rb, 5.08766e-04_rb, 5.86292e-04_rb, &
     & 6.75633e-04_rb, 7.78588e-04_rb, 8.97231e-04_rb, 1.03395e-03_rb, 1.19151e-03_rb, &
     & 1.37307e-03_rb, 1.58231e-03_rb, 1.82342e-03_rb, 2.10128e-03_rb/)

!     The array KBO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level above 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kbo_mco2(:, 1) = (/ &
     & 3.72069e-06_rb, 4.81866e-06_rb, 6.24064e-06_rb, 8.08226e-06_rb, 1.04673e-05_rb, &
     & 1.35562e-05_rb, 1.75567e-05_rb, 2.27376e-05_rb, 2.94475e-05_rb, 3.81375e-05_rb, &
     & 4.93918e-05_rb, 6.39674e-05_rb, 8.28441e-05_rb, 1.07291e-04_rb, 1.38953e-04_rb, &
     & 1.79958e-04_rb, 2.33064e-04_rb, 3.01840e-04_rb, 3.90913e-04_rb/)
      kbo_mco2(:, 2) = (/ &
     & 8.14357e-06_rb, 1.06031e-05_rb, 1.38056e-05_rb, 1.79752e-05_rb, 2.34041e-05_rb, &
     & 3.04728e-05_rb, 3.96763e-05_rb, 5.16596e-05_rb, 6.72622e-05_rb, 8.75770e-05_rb, &
     & 1.14027e-04_rb, 1.48467e-04_rb, 1.93307e-04_rb, 2.51691e-04_rb, 3.27708e-04_rb, &
     & 4.26685e-04_rb, 5.55555e-04_rb, 7.23346e-04_rb, 9.41814e-04_rb/)
      kbo_mco2(:, 3) = (/ &
     & 1.09367e-05_rb, 1.42063e-05_rb, 1.84533e-05_rb, 2.39701e-05_rb, 3.11362e-05_rb, &
     & 4.04446e-05_rb, 5.25358e-05_rb, 6.82417e-05_rb, 8.86432e-05_rb, 1.15144e-04_rb, &
     & 1.49567e-04_rb, 1.94281e-04_rb, 2.52363e-04_rb, 3.27809e-04_rb, 4.25810e-04_rb, &
     & 5.53109e-04_rb, 7.18466e-04_rb, 9.33256e-04_rb, 1.21226e-03_rb/)
      kbo_mco2(:, 4) = (/ &
     & 1.76192e-05_rb, 2.27752e-05_rb, 2.94401e-05_rb, 3.80553e-05_rb, 4.91916e-05_rb, &
     & 6.35867e-05_rb, 8.21944e-05_rb, 1.06247e-04_rb, 1.37339e-04_rb, 1.77529e-04_rb, &
     & 2.29480e-04_rb, 2.96635e-04_rb, 3.83440e-04_rb, 4.95648e-04_rb, 6.40691e-04_rb, &
     & 8.28180e-04_rb, 1.07054e-03_rb, 1.38381e-03_rb, 1.78876e-03_rb/)
      kbo_mco2(:, 5) = (/ &
     & 3.72142e-05_rb, 4.78603e-05_rb, 6.15520e-05_rb, 7.91605e-05_rb, 1.01806e-04_rb, &
     & 1.30931e-04_rb, 1.68387e-04_rb, 2.16558e-04_rb, 2.78510e-04_rb, 3.58185e-04_rb, &
     & 4.60653e-04_rb, 5.92435e-04_rb, 7.61915e-04_rb, 9.79881e-04_rb, 1.26020e-03_rb, &
     & 1.62071e-03_rb, 2.08436e-03_rb, 2.68064e-03_rb, 3.44751e-03_rb/)
      kbo_mco2(:, 6) = (/ &
     & 7.74131e-05_rb, 9.98876e-05_rb, 1.28887e-04_rb, 1.66305e-04_rb, 2.14587e-04_rb, &
     & 2.76886e-04_rb, 3.57272e-04_rb, 4.60994e-04_rb, 5.94831e-04_rb, 7.67521e-04_rb, &
     & 9.90348e-04_rb, 1.27787e-03_rb, 1.64886e-03_rb, 2.12755e-03_rb, 2.74522e-03_rb, &
     & 3.54221e-03_rb, 4.57059e-03_rb, 5.89752e-03_rb, 7.60968e-03_rb/)
      kbo_mco2(:, 7) = (/ &
     & 1.32294e-04_rb, 1.70977e-04_rb, 2.20973e-04_rb, 2.85587e-04_rb, 3.69095e-04_rb, &
     & 4.77022e-04_rb, 6.16507e-04_rb, 7.96779e-04_rb, 1.02976e-03_rb, 1.33088e-03_rb, &
     & 1.72004e-03_rb, 2.22299e-03_rb, 2.87301e-03_rb, 3.71310e-03_rb, 4.79884e-03_rb, &
     & 6.20207e-03_rb, 8.01561e-03_rb, 1.03594e-02_rb, 1.33886e-02_rb/)
      kbo_mco2(:, 8) = (/ &
     & 3.59868e-05_rb, 4.63611e-05_rb, 5.97261e-05_rb, 7.69439e-05_rb, 9.91253e-05_rb, &
     & 1.27701e-04_rb, 1.64515e-04_rb, 2.11941e-04_rb, 2.73040e-04_rb, 3.51752e-04_rb, &
     & 4.53155e-04_rb, 5.83790e-04_rb, 7.52085e-04_rb, 9.68897e-04_rb, 1.24821e-03_rb, &
     & 1.60804e-03_rb, 2.07161e-03_rb, 2.66882e-03_rb, 3.43818e-03_rb/)
      kbo_mco2(:, 9) = (/ &
     & 5.09543e-05_rb, 6.60510e-05_rb, 8.56205e-05_rb, 1.10988e-04_rb, 1.43872e-04_rb, &
     & 1.86498e-04_rb, 2.41753e-04_rb, 3.13380e-04_rb, 4.06228e-04_rb, 5.26585e-04_rb, &
     & 6.82601e-04_rb, 8.84842e-04_rb, 1.14700e-03_rb, 1.48684e-03_rb, 1.92735e-03_rb, &
     & 2.49839e-03_rb, 3.23861e-03_rb, 4.19814e-03_rb, 5.44196e-03_rb/)
      kbo_mco2(:,10) = (/ &
     & 2.08253e-05_rb, 2.64900e-05_rb, 3.36954e-05_rb, 4.28609e-05_rb, 5.45194e-05_rb, &
     & 6.93491e-05_rb, 8.82125e-05_rb, 1.12207e-04_rb, 1.42728e-04_rb, 1.81551e-04_rb, &
     & 2.30935e-04_rb, 2.93751e-04_rb, 3.73653e-04_rb, 4.75290e-04_rb, 6.04572e-04_rb, &
     & 7.69021e-04_rb, 9.78201e-04_rb, 1.24428e-03_rb, 1.58273e-03_rb/)
      kbo_mco2(:,11) = (/ &
     & 2.08953e-05_rb, 2.65543e-05_rb, 3.37459e-05_rb, 4.28852e-05_rb, 5.44996e-05_rb, &
     & 6.92595e-05_rb, 8.80169e-05_rb, 1.11854e-04_rb, 1.42147e-04_rb, 1.80644e-04_rb, &
     & 2.29568e-04_rb, 2.91741e-04_rb, 3.70752e-04_rb, 4.71161e-04_rb, 5.98764e-04_rb, &
     & 7.60925e-04_rb, 9.67005e-04_rb, 1.22889e-03_rb, 1.56171e-03_rb/)
      kbo_mco2(:,12) = (/ &
     & 2.65295e-05_rb, 3.36318e-05_rb, 4.26356e-05_rb, 5.40498e-05_rb, 6.85198e-05_rb, &
     & 8.68636e-05_rb, 1.10118e-04_rb, 1.39599e-04_rb, 1.76972e-04_rb, 2.24350e-04_rb, &
     & 2.84412e-04_rb, 3.60553e-04_rb, 4.57079e-04_rb, 5.79446e-04_rb, 7.34572e-04_rb, &
     & 9.31230e-04_rb, 1.18053e-03_rb, 1.49658e-03_rb, 1.89724e-03_rb/)
      kbo_mco2(:,13) = (/ &
     & 3.45358e-05_rb, 4.36743e-05_rb, 5.52309e-05_rb, 6.98455e-05_rb, 8.83273e-05_rb, &
     & 1.11700e-04_rb, 1.41256e-04_rb, 1.78634e-04_rb, 2.25902e-04_rb, 2.85678e-04_rb, &
     & 3.61271e-04_rb, 4.56867e-04_rb, 5.77758e-04_rb, 7.30639e-04_rb, 9.23973e-04_rb, &
     & 1.16847e-03_rb, 1.47765e-03_rb, 1.86865e-03_rb, 2.36311e-03_rb/)
      kbo_mco2(:,14) = (/ &
     & 3.99721e-05_rb, 5.12343e-05_rb, 6.56698e-05_rb, 8.41725e-05_rb, 1.07888e-04_rb, &
     & 1.38286e-04_rb, 1.77249e-04_rb, 2.27190e-04_rb, 2.91201e-04_rb, 3.73248e-04_rb, &
     & 4.78412e-04_rb, 6.13207e-04_rb, 7.85980e-04_rb, 1.00743e-03_rb, 1.29128e-03_rb, &
     & 1.65510e-03_rb, 2.12144e-03_rb, 2.71916e-03_rb, 3.48529e-03_rb/)
      kbo_mco2(:,15) = (/ &
     & 8.51533e-06_rb, 1.23021e-05_rb, 1.77730e-05_rb, 2.56767e-05_rb, 3.70953e-05_rb, &
     & 5.35918e-05_rb, 7.74243e-05_rb, 1.11855e-04_rb, 1.61598e-04_rb, 2.33461e-04_rb, &
     & 3.37283e-04_rb, 4.87275e-04_rb, 7.03968e-04_rb, 1.01703e-03_rb, 1.46930e-03_rb, &
     & 2.12271e-03_rb, 3.06670e-03_rb, 4.43047e-03_rb, 6.40072e-03_rb/)
      kbo_mco2(:,16) = (/ &
     & 2.93050e-06_rb, 3.65298e-06_rb, 4.55358e-06_rb, 5.67622e-06_rb, 7.07564e-06_rb, &
     & 8.82006e-06_rb, 1.09945e-05_rb, 1.37051e-05_rb, 1.70840e-05_rb, 2.12959e-05_rb, &
     & 2.65461e-05_rb, 3.30908e-05_rb, 4.12490e-05_rb, 5.14185e-05_rb, 6.40952e-05_rb, &
     & 7.98972e-05_rb, 9.95951e-05_rb, 1.24149e-04_rb, 1.54757e-04_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296_rb,260_rb,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &2.0677e-07_rb,2.0363e-07_rb,2.0583e-07_rb,2.0547e-07_rb,2.0267e-07_rb,2.0154e-07_rb, &
     &2.0190e-07_rb,2.0103e-07_rb,1.9869e-07_rb,1.9663e-07_rb,1.9701e-07_rb,2.0103e-07_rb, &
     &2.0527e-07_rb,2.0206e-07_rb,2.0364e-07_rb,2.0364e-07_rb/)
      forrefo(2,:) = (/ &
     &2.2427e-07_rb,2.1489e-07_rb,2.0453e-07_rb,1.9710e-07_rb,1.9650e-07_rb,1.9738e-07_rb, &
     &1.9767e-07_rb,1.9769e-07_rb,1.9940e-07_rb,1.9846e-07_rb,1.9898e-07_rb,1.9853e-07_rb, &
     &2.0000e-07_rb,2.0517e-07_rb,2.0482e-07_rb,2.0482e-07_rb/)
      forrefo(3,:) = (/ &
     &2.2672e-07_rb,2.1706e-07_rb,2.0571e-07_rb,1.9747e-07_rb,1.9706e-07_rb,1.9698e-07_rb, &
     &1.9781e-07_rb,1.9774e-07_rb,1.9724e-07_rb,1.9714e-07_rb,1.9751e-07_rb,1.9758e-07_rb, &
     &1.9840e-07_rb,1.9968e-07_rb,1.9931e-07_rb,1.9880e-07_rb/)
      forrefo(4,:) = (/ &
     &2.2191e-07_rb,2.0899e-07_rb,2.0265e-07_rb,2.0101e-07_rb,2.0034e-07_rb,2.0021e-07_rb, &
     &1.9987e-07_rb,1.9978e-07_rb,1.9902e-07_rb,1.9742e-07_rb,1.9672e-07_rb,1.9615e-07_rb, &
     &1.9576e-07_rb,1.9540e-07_rb,1.9588e-07_rb,1.9590e-07_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 5.18832e-02_rb, 4.28690e-02_rb, 3.54210e-02_rb, 2.92670e-02_rb, 2.41822e-02_rb, &
     & 1.99808e-02_rb, 1.65093e-02_rb, 1.36410e-02_rb, 1.12710e-02_rb, 9.31280e-03_rb/)
      selfrefo(:, 2) = (/ &
     & 4.36030e-02_rb, 3.78379e-02_rb, 3.28350e-02_rb, 2.84936e-02_rb, 2.47262e-02_rb, &
     & 2.14569e-02_rb, 1.86199e-02_rb, 1.61580e-02_rb, 1.40216e-02_rb, 1.21677e-02_rb/)
      selfrefo(:, 3) = (/ &
     & 4.26492e-02_rb, 3.71443e-02_rb, 3.23500e-02_rb, 2.81745e-02_rb, 2.45379e-02_rb, &
     & 2.13707e-02_rb, 1.86124e-02_rb, 1.62100e-02_rb, 1.41177e-02_rb, 1.22955e-02_rb/)
      selfrefo(:, 4) = (/ &
     & 4.03591e-02_rb, 3.54614e-02_rb, 3.11580e-02_rb, 2.73769e-02_rb, 2.40546e-02_rb, &
     & 2.11355e-02_rb, 1.85706e-02_rb, 1.63170e-02_rb, 1.43369e-02_rb, 1.25970e-02_rb/)
      selfrefo(:, 5) = (/ &
     & 3.94512e-02_rb, 3.46232e-02_rb, 3.03860e-02_rb, 2.66674e-02_rb, 2.34038e-02_rb, &
     & 2.05397e-02_rb, 1.80260e-02_rb, 1.58200e-02_rb, 1.38839e-02_rb, 1.21848e-02_rb/)
      selfrefo(:, 6) = (/ &
     & 3.90567e-02_rb, 3.40694e-02_rb, 2.97190e-02_rb, 2.59241e-02_rb, 2.26138e-02_rb, &
     & 1.97261e-02_rb, 1.72072e-02_rb, 1.50100e-02_rb, 1.30933e-02_rb, 1.14214e-02_rb/)
      selfrefo(:, 7) = (/ &
     & 3.85397e-02_rb, 3.36462e-02_rb, 2.93740e-02_rb, 2.56443e-02_rb, 2.23881e-02_rb, &
     & 1.95454e-02_rb, 1.70636e-02_rb, 1.48970e-02_rb, 1.30055e-02_rb, 1.13541e-02_rb/)
      selfrefo(:, 8) = (/ &
     & 3.79692e-02_rb, 3.31360e-02_rb, 2.89180e-02_rb, 2.52369e-02_rb, 2.20245e-02_rb, &
     & 1.92209e-02_rb, 1.67742e-02_rb, 1.46390e-02_rb, 1.27756e-02_rb, 1.11493e-02_rb/)
      selfrefo(:, 9) = (/ &
     & 3.68819e-02_rb, 3.22827e-02_rb, 2.82570e-02_rb, 2.47333e-02_rb, 2.16490e-02_rb, &
     & 1.89494e-02_rb, 1.65863e-02_rb, 1.45180e-02_rb, 1.27076e-02_rb, 1.11229e-02_rb/)
      selfrefo(:,10) = (/ &
     & 3.65157e-02_rb, 3.20121e-02_rb, 2.80640e-02_rb, 2.46028e-02_rb, 2.15685e-02_rb, &
     & 1.89084e-02_rb, 1.65764e-02_rb, 1.45320e-02_rb, 1.27397e-02_rb, 1.11685e-02_rb/)
      selfrefo(:,11) = (/ &
     & 3.59917e-02_rb, 3.16727e-02_rb, 2.78720e-02_rb, 2.45274e-02_rb, 2.15841e-02_rb, &
     & 1.89940e-02_rb, 1.67148e-02_rb, 1.47090e-02_rb, 1.29439e-02_rb, 1.13907e-02_rb/)
      selfrefo(:,12) = (/ &
     & 3.66963e-02_rb, 3.20483e-02_rb, 2.79890e-02_rb, 2.44439e-02_rb, 2.13478e-02_rb, &
     & 1.86438e-02_rb, 1.62824e-02_rb, 1.42200e-02_rb, 1.24189e-02_rb, 1.08459e-02_rb/)
      selfrefo(:,13) = (/ &
     & 3.66422e-02_rb, 3.19026e-02_rb, 2.77760e-02_rb, 2.41832e-02_rb, 2.10551e-02_rb, &
     & 1.83317e-02_rb, 1.59605e-02_rb, 1.38960e-02_rb, 1.20986e-02_rb, 1.05336e-02_rb/)
      selfrefo(:,14) = (/ &
     & 3.81260e-02_rb, 3.29322e-02_rb, 2.84460e-02_rb, 2.45709e-02_rb, 2.12237e-02_rb, &
     & 1.83325e-02_rb, 1.58352e-02_rb, 1.36780e-02_rb, 1.18147e-02_rb, 1.02052e-02_rb/)
      selfrefo(:,15) = (/ &
     & 3.51264e-02_rb, 3.05081e-02_rb, 2.64970e-02_rb, 2.30133e-02_rb, 1.99876e-02_rb, &
     & 1.73597e-02_rb, 1.50773e-02_rb, 1.30950e-02_rb, 1.13733e-02_rb, 9.87800e-03_rb/)
      selfrefo(:,16) = (/ &
     & 3.51264e-02_rb, 3.05081e-02_rb, 2.64970e-02_rb, 2.30133e-02_rb, 1.99876e-02_rb, &
     & 1.73597e-02_rb, 1.50773e-02_rb, 1.30950e-02_rb, 1.13733e-02_rb, 9.87800e-03_rb/)

      end subroutine lw_kgb07

! **************************************************************************
      subroutine lw_kgb08
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg08, only : fracrefao, fracrefbo, kao, kao_mco2, kao_mn2o, &
                            kao_mo3, kbo, kbo_mco2, kbo_mn2o, selfrefo, forrefo, &
                            cfc12o, cfc22adjo

      implicit none
      save

! Planck fraction mapping level : P=473.4280 mb, T = 259.83 K
      fracrefao(:) = (/ &
        1.6004e-01_rb,1.5437e-01_rb,1.4502e-01_rb,1.3084e-01_rb,1.1523e-01_rb,9.7743e-02_rb, &
        8.0376e-02_rb,6.0261e-02_rb,4.1111e-02_rb,4.4772e-03_rb,3.6511e-03_rb,2.9154e-03_rb, &
        2.1184e-03_rb,1.3048e-03_rb,4.6637e-04_rb,6.5624e-05_rb/)

! Planck fraction mapping level : P=95.5835 mb, T= 215.7 K
      fracrefbo(:) = (/ &
        1.4987e-01_rb,1.4665e-01_rb,1.4154e-01_rb,1.3200e-01_rb,1.1902e-01_rb,1.0352e-01_rb, &
        8.4939e-02_rb,6.4105e-02_rb,4.3190e-02_rb,4.5129e-03_rb,3.7656e-03_rb,2.8733e-03_rb, &
        2.0947e-03_rb,1.3201e-03_rb,5.1832e-04_rb,7.7473e-05_rb/)

! Minor gas mapping level:
!     lower - co2, p = 1053.63 mb, t = 294.2 k
!     lower - o3,  p = 317.348 mb, t = 240.77 k
!     lower - n2o, p = 706.2720 mb, t= 278.94 k
!     lower - cfc12,cfc11
!     upper - co2, p = 35.1632 mb, t = 223.28 k
!     upper - n2o, p = 8.716e-2 mb, t = 226.03 k

      cfc12o(:) = (/ &
        85.4027_rb, 89.4696_rb, 74.0959_rb, 67.7480_rb, &
        61.2444_rb, 59.9073_rb, 60.8296_rb, 63.0998_rb, &
        59.6110_rb, 64.0735_rb, 57.2622_rb, 58.9721_rb, &
        43.5505_rb, 26.1192_rb, 32.7023_rb, 32.8667_rb/)
! Original CFC22 is multiplied by 1.485 to account for the 780-850 cm-1 
! and 1290-1335 cm-1 bands.
      cfc22adjo(:) = (/ &
        135.335_rb, 89.6642_rb, 76.2375_rb, 65.9748_rb, &
        63.1164_rb, 60.2935_rb, 64.0299_rb, 75.4264_rb, &
        51.3018_rb, 7.07911_rb, 5.86928_rb, 0.398693_rb, &
        2.82885_rb, 9.12751_rb, 6.28271_rb, 0._rb/)

!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels > ~100mb and temperatures.  The first
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the corresponding TREF for this  pressure level, 
!     JT = 2 refers to the temperatureTREF-15, JT = 1 is for TREF-30, 
!     JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  The second 
!     index, JP, runs from 1 to 13 and refers to the corresponding 
!     pressure level in PREF (e.g. JP = 1 is for a pressure of 1053.63 mb).  
!     The third index, IG, goes from 1 to 16, and tells us which 
!     g-interval the absorption coefficients are for.

      kao(:, 1, 1) = (/ &
     &1.4664e-05_rb,1.7114e-05_rb,1.8876e-05_rb,2.2301e-05_rb,2.6622e-05_rb/)
      kao(:, 2, 1) = (/ &
     &1.0571e-05_rb,1.1718e-05_rb,1.4060e-05_rb,1.7193e-05_rb,2.1557e-05_rb/)
      kao(:, 3, 1) = (/ &
     &6.6138e-06_rb,8.1523e-06_rb,1.0695e-05_rb,1.3782e-05_rb,1.7449e-05_rb/)
      kao(:, 4, 1) = (/ &
     &5.1390e-06_rb,7.0055e-06_rb,9.4447e-06_rb,1.2362e-05_rb,1.5431e-05_rb/)
      kao(:, 5, 1) = (/ &
     &4.7765e-06_rb,6.4140e-06_rb,8.4242e-06_rb,1.0772e-05_rb,1.3788e-05_rb/)
      kao(:, 6, 1) = (/ &
     &4.4818e-06_rb,6.1945e-06_rb,8.3186e-06_rb,1.0638e-05_rb,1.3072e-05_rb/)
      kao(:, 7, 1) = (/ &
     &4.1781e-06_rb,5.8651e-06_rb,8.2034e-06_rb,1.0966e-05_rb,1.3592e-05_rb/)
      kao(:, 8, 1) = (/ &
     &4.6559e-06_rb,6.5570e-06_rb,8.8606e-06_rb,1.1636e-05_rb,1.4449e-05_rb/)
      kao(:, 9, 1) = (/ &
     &6.3964e-06_rb,9.3461e-06_rb,1.3202e-05_rb,1.7140e-05_rb,2.2124e-05_rb/)
      kao(:,10, 1) = (/ &
     &1.0453e-05_rb,1.6266e-05_rb,2.2571e-05_rb,3.2675e-05_rb,4.3832e-05_rb/)
      kao(:,11, 1) = (/ &
     &1.9225e-05_rb,2.5219e-05_rb,3.3439e-05_rb,4.2567e-05_rb,5.0999e-05_rb/)
      kao(:,12, 1) = (/ &
     &1.8581e-05_rb,2.6464e-05_rb,3.4892e-05_rb,4.5782e-05_rb,5.8008e-05_rb/)
      kao(:,13, 1) = (/ &
     &2.3252e-05_rb,2.6601e-05_rb,3.1497e-05_rb,4.1728e-05_rb,5.3564e-05_rb/)
      kao(:, 1, 2) = (/ &
     &2.4622e-05_rb,2.2121e-05_rb,2.5151e-05_rb,3.1972e-05_rb,4.1114e-05_rb/)
      kao(:, 2, 2) = (/ &
     &1.5738e-05_rb,1.6913e-05_rb,2.0782e-05_rb,2.6880e-05_rb,3.4084e-05_rb/)
      kao(:, 3, 2) = (/ &
     &9.8655e-06_rb,1.2753e-05_rb,1.6558e-05_rb,2.1755e-05_rb,2.8126e-05_rb/)
      kao(:, 4, 2) = (/ &
     &7.9265e-06_rb,1.0229e-05_rb,1.3524e-05_rb,1.7603e-05_rb,2.3192e-05_rb/)
      kao(:, 5, 2) = (/ &
     &7.0537e-06_rb,9.6733e-06_rb,1.2570e-05_rb,1.6079e-05_rb,2.0384e-05_rb/)
      kao(:, 6, 2) = (/ &
     &6.5277e-06_rb,9.1084e-06_rb,1.2443e-05_rb,1.5914e-05_rb,2.0339e-05_rb/)
      kao(:, 7, 2) = (/ &
     &7.4442e-06_rb,1.0146e-05_rb,1.2791e-05_rb,1.6212e-05_rb,2.1044e-05_rb/)
      kao(:, 8, 2) = (/ &
     &7.4223e-06_rb,1.0854e-05_rb,1.4643e-05_rb,1.9376e-05_rb,2.5625e-05_rb/)
      kao(:, 9, 2) = (/ &
     &1.4157e-05_rb,1.8510e-05_rb,2.5064e-05_rb,3.4631e-05_rb,4.4655e-05_rb/)
      kao(:,10, 2) = (/ &
     &3.4785e-05_rb,4.4053e-05_rb,5.4155e-05_rb,6.0764e-05_rb,5.9887e-05_rb/)
      kao(:,11, 2) = (/ &
     &4.4199e-05_rb,5.6024e-05_rb,6.6252e-05_rb,8.3322e-05_rb,1.0437e-04_rb/)
      kao(:,12, 2) = (/ &
     &4.8012e-05_rb,6.2560e-05_rb,7.6674e-05_rb,8.6164e-05_rb,1.0081e-04_rb/)
      kao(:,13, 2) = (/ &
     &3.9220e-05_rb,5.8614e-05_rb,7.6090e-05_rb,8.9710e-05_rb,9.6705e-05_rb/)
      kao(:, 1, 3) = (/ &
     &2.7517e-05_rb,3.2252e-05_rb,4.5774e-05_rb,5.9187e-05_rb,7.3901e-05_rb/)
      kao(:, 2, 3) = (/ &
     &1.9414e-05_rb,2.5349e-05_rb,3.4494e-05_rb,4.4078e-05_rb,5.4475e-05_rb/)
      kao(:, 3, 3) = (/ &
     &1.4431e-05_rb,1.9862e-05_rb,2.6815e-05_rb,3.5045e-05_rb,4.4510e-05_rb/)
      kao(:, 4, 3) = (/ &
     &1.1163e-05_rb,1.5886e-05_rb,2.1047e-05_rb,2.7352e-05_rb,3.5151e-05_rb/)
      kao(:, 5, 3) = (/ &
     &1.0004e-05_rb,1.3946e-05_rb,1.9070e-05_rb,2.4940e-05_rb,3.2122e-05_rb/)
      kao(:, 6, 3) = (/ &
     &9.8796e-06_rb,1.2389e-05_rb,1.6574e-05_rb,2.2677e-05_rb,2.9114e-05_rb/)
      kao(:, 7, 3) = (/ &
     &9.6705e-06_rb,1.3203e-05_rb,1.8009e-05_rb,2.4248e-05_rb,2.9926e-05_rb/)
      kao(:, 8, 3) = (/ &
     &1.2684e-05_rb,1.5798e-05_rb,2.0751e-05_rb,2.7135e-05_rb,3.4302e-05_rb/)
      kao(:, 9, 3) = (/ &
     &2.3748e-05_rb,2.8117e-05_rb,3.3728e-05_rb,4.1366e-05_rb,5.3787e-05_rb/)
      kao(:,10, 3) = (/ &
     &4.9449e-05_rb,6.0387e-05_rb,8.1388e-05_rb,1.0134e-04_rb,1.3311e-04_rb/)
      kao(:,11, 3) = (/ &
     &6.5933e-05_rb,8.9755e-05_rb,1.2191e-04_rb,1.3795e-04_rb,1.5355e-04_rb/)
      kao(:,12, 3) = (/ &
     &6.2437e-05_rb,8.0247e-05_rb,1.0990e-04_rb,1.3683e-04_rb,1.6418e-04_rb/)
      kao(:,13, 3) = (/ &
     &5.9989e-05_rb,6.7831e-05_rb,9.4332e-05_rb,1.1542e-04_rb,1.5040e-04_rb/)
      kao(:, 1, 4) = (/ &
     &3.6489e-05_rb,5.9568e-05_rb,7.8043e-05_rb,9.8599e-05_rb,1.2261e-04_rb/)
      kao(:, 2, 4) = (/ &
     &3.0751e-05_rb,4.5700e-05_rb,5.9720e-05_rb,7.6230e-05_rb,9.5698e-05_rb/)
      kao(:, 3, 4) = (/ &
     &2.6862e-05_rb,3.6448e-05_rb,4.6703e-05_rb,5.7936e-05_rb,7.1216e-05_rb/)
      kao(:, 4, 4) = (/ &
     &2.0754e-05_rb,2.7900e-05_rb,3.6677e-05_rb,4.8008e-05_rb,6.1630e-05_rb/)
      kao(:, 5, 4) = (/ &
     &1.6362e-05_rb,2.1862e-05_rb,2.9666e-05_rb,3.8981e-05_rb,5.0305e-05_rb/)
      kao(:, 6, 4) = (/ &
     &1.4991e-05_rb,2.0957e-05_rb,2.6223e-05_rb,3.2254e-05_rb,4.2632e-05_rb/)
      kao(:, 7, 4) = (/ &
     &1.3833e-05_rb,1.9399e-05_rb,2.5656e-05_rb,3.2001e-05_rb,4.1685e-05_rb/)
      kao(:, 8, 4) = (/ &
     &1.6855e-05_rb,2.2852e-05_rb,3.0265e-05_rb,3.5778e-05_rb,4.6073e-05_rb/)
      kao(:, 9, 4) = (/ &
     &3.3370e-05_rb,4.7904e-05_rb,5.8243e-05_rb,6.4094e-05_rb,7.6700e-05_rb/)
      kao(:,10, 4) = (/ &
     &6.8238e-05_rb,1.0340e-04_rb,1.2739e-04_rb,1.5464e-04_rb,1.7523e-04_rb/)
      kao(:,11, 4) = (/ &
     &6.1230e-05_rb,9.6227e-05_rb,1.3743e-04_rb,1.9846e-04_rb,2.3838e-04_rb/)
      kao(:,12, 4) = (/ &
     &6.8823e-05_rb,1.0715e-04_rb,1.3058e-04_rb,1.9049e-04_rb,2.2996e-04_rb/)
      kao(:,13, 4) = (/ &
     &6.5444e-05_rb,1.1659e-04_rb,1.2685e-04_rb,1.7368e-04_rb,2.0128e-04_rb/)
      kao(:, 1, 5) = (/ &
     &7.8999e-05_rb,1.0665e-04_rb,1.3776e-04_rb,1.7432e-04_rb,2.1687e-04_rb/)
      kao(:, 2, 5) = (/ &
     &6.1797e-05_rb,8.3347e-05_rb,1.0862e-04_rb,1.3836e-04_rb,1.7490e-04_rb/)
      kao(:, 3, 5) = (/ &
     &4.7325e-05_rb,6.2800e-05_rb,8.1194e-05_rb,1.0315e-04_rb,1.3026e-04_rb/)
      kao(:, 4, 5) = (/ &
     &3.8433e-05_rb,5.0865e-05_rb,6.5549e-05_rb,8.2585e-05_rb,1.0247e-04_rb/)
      kao(:, 5, 5) = (/ &
     &3.0177e-05_rb,3.9082e-05_rb,4.9288e-05_rb,6.5674e-05_rb,8.5494e-05_rb/)
      kao(:, 6, 5) = (/ &
     &2.4712e-05_rb,3.2374e-05_rb,4.1224e-05_rb,5.5035e-05_rb,7.0157e-05_rb/)
      kao(:, 7, 5) = (/ &
     &2.2530e-05_rb,2.6200e-05_rb,3.5121e-05_rb,4.6306e-05_rb,5.9652e-05_rb/)
      kao(:, 8, 5) = (/ &
     &2.8297e-05_rb,3.6551e-05_rb,4.4329e-05_rb,5.9550e-05_rb,7.1792e-05_rb/)
      kao(:, 9, 5) = (/ &
     &5.6002e-05_rb,7.4631e-05_rb,8.8607e-05_rb,1.1720e-04_rb,1.2997e-04_rb/)
      kao(:,10, 5) = (/ &
     &8.1020e-05_rb,1.1580e-04_rb,1.6197e-04_rb,2.3825e-04_rb,2.8441e-04_rb/)
      kao(:,11, 5) = (/ &
     &9.6146e-05_rb,1.4644e-04_rb,1.7805e-04_rb,2.2438e-04_rb,3.1753e-04_rb/)
      kao(:,12, 5) = (/ &
     &9.5441e-05_rb,1.4982e-04_rb,2.4222e-04_rb,2.7134e-04_rb,3.3168e-04_rb/)
      kao(:,13, 5) = (/ &
     &9.5058e-05_rb,1.4349e-04_rb,2.4835e-04_rb,2.9526e-04_rb,3.7011e-04_rb/)
      kao(:, 1, 6) = (/ &
     &1.5625e-04_rb,2.0681e-04_rb,2.6299e-04_rb,3.2669e-04_rb,4.0038e-04_rb/)
      kao(:, 2, 6) = (/ &
     &1.1702e-04_rb,1.5581e-04_rb,2.0084e-04_rb,2.5419e-04_rb,3.1669e-04_rb/)
      kao(:, 3, 6) = (/ &
     &8.6159e-05_rb,1.1329e-04_rb,1.4769e-04_rb,1.9151e-04_rb,2.4537e-04_rb/)
      kao(:, 4, 6) = (/ &
     &7.1655e-05_rb,9.4348e-05_rb,1.2251e-04_rb,1.5312e-04_rb,1.8840e-04_rb/)
      kao(:, 5, 6) = (/ &
     &5.9776e-05_rb,8.0388e-05_rb,1.0504e-04_rb,1.3009e-04_rb,1.5686e-04_rb/)
      kao(:, 6, 6) = (/ &
     &4.1587e-05_rb,5.6800e-05_rb,7.7481e-05_rb,9.9930e-05_rb,1.3028e-04_rb/)
      kao(:, 7, 6) = (/ &
     &3.7625e-05_rb,5.2950e-05_rb,6.5572e-05_rb,8.3943e-05_rb,1.0947e-04_rb/)
      kao(:, 8, 6) = (/ &
     &4.0934e-05_rb,5.1706e-05_rb,6.6726e-05_rb,8.2858e-05_rb,1.0389e-04_rb/)
      kao(:, 9, 6) = (/ &
     &8.3177e-05_rb,1.0693e-04_rb,1.2771e-04_rb,1.5344e-04_rb,1.9338e-04_rb/)
      kao(:,10, 6) = (/ &
     &1.2590e-04_rb,1.6860e-04_rb,2.4332e-04_rb,2.8123e-04_rb,4.3002e-04_rb/)
      kao(:,11, 6) = (/ &
     &1.5778e-04_rb,2.1103e-04_rb,2.9773e-04_rb,4.5434e-04_rb,5.2006e-04_rb/)
      kao(:,12, 6) = (/ &
     &1.7734e-04_rb,2.3169e-04_rb,2.6921e-04_rb,4.5377e-04_rb,6.4453e-04_rb/)
      kao(:,13, 6) = (/ &
     &1.7874e-04_rb,2.2968e-04_rb,2.8292e-04_rb,4.5303e-04_rb,6.1897e-04_rb/)
      kao(:, 1, 7) = (/ &
     &3.7144e-04_rb,4.7830e-04_rb,6.0203e-04_rb,7.4238e-04_rb,8.9847e-04_rb/)
      kao(:, 2, 7) = (/ &
     &2.8002e-04_rb,3.6431e-04_rb,4.6140e-04_rb,5.7047e-04_rb,6.9195e-04_rb/)
      kao(:, 3, 7) = (/ &
     &1.8338e-04_rb,2.5018e-04_rb,3.2929e-04_rb,4.2096e-04_rb,5.2583e-04_rb/)
      kao(:, 4, 7) = (/ &
     &1.3365e-04_rb,1.7565e-04_rb,2.2370e-04_rb,2.9770e-04_rb,3.9171e-04_rb/)
      kao(:, 5, 7) = (/ &
     &1.1609e-04_rb,1.5410e-04_rb,1.9723e-04_rb,2.4620e-04_rb,3.0830e-04_rb/)
      kao(:, 6, 7) = (/ &
     &1.0480e-04_rb,1.3862e-04_rb,1.7991e-04_rb,2.2372e-04_rb,2.7406e-04_rb/)
      kao(:, 7, 7) = (/ &
     &8.7767e-05_rb,1.1689e-04_rb,1.5270e-04_rb,1.9631e-04_rb,2.3773e-04_rb/)
      kao(:, 8, 7) = (/ &
     &9.8366e-05_rb,1.1738e-04_rb,1.4725e-04_rb,1.8494e-04_rb,2.3601e-04_rb/)
      kao(:, 9, 7) = (/ &
     &1.5443e-04_rb,1.9009e-04_rb,2.4404e-04_rb,2.8078e-04_rb,3.3602e-04_rb/)
      kao(:,10, 7) = (/ &
     &1.5274e-04_rb,2.6878e-04_rb,3.7659e-04_rb,5.3377e-04_rb,5.6306e-04_rb/)
      kao(:,11, 7) = (/ &
     &2.0214e-04_rb,2.6794e-04_rb,4.2457e-04_rb,5.7376e-04_rb,8.8285e-04_rb/)
      kao(:,12, 7) = (/ &
     &2.5054e-04_rb,3.1232e-04_rb,4.4996e-04_rb,6.0765e-04_rb,8.8689e-04_rb/)
      kao(:,13, 7) = (/ &
     &2.7318e-04_rb,3.6929e-04_rb,4.5362e-04_rb,6.1364e-04_rb,9.2141e-04_rb/)
      kao(:, 1, 8) = (/ &
     &1.1751e-03_rb,1.5113e-03_rb,1.8892e-03_rb,2.3109e-03_rb,2.7776e-03_rb/)
      kao(:, 2, 8) = (/ &
     &8.9844e-04_rb,1.1675e-03_rb,1.4748e-03_rb,1.8223e-03_rb,2.2105e-03_rb/)
      kao(:, 3, 8) = (/ &
     &6.3916e-04_rb,8.4337e-04_rb,1.0777e-03_rb,1.3420e-03_rb,1.6361e-03_rb/)
      kao(:, 4, 8) = (/ &
     &3.9704e-04_rb,5.5491e-04_rb,7.4493e-04_rb,9.5633e-04_rb,1.1972e-03_rb/)
      kao(:, 5, 8) = (/ &
     &2.8104e-04_rb,3.5275e-04_rb,4.8572e-04_rb,6.7373e-04_rb,9.0464e-04_rb/)
      kao(:, 6, 8) = (/ &
     &2.6538e-04_rb,3.4221e-04_rb,4.3816e-04_rb,5.5889e-04_rb,6.9563e-04_rb/)
      kao(:, 7, 8) = (/ &
     &2.6802e-04_rb,3.3443e-04_rb,4.3228e-04_rb,5.4299e-04_rb,6.7937e-04_rb/)
      kao(:, 8, 8) = (/ &
     &2.6587e-04_rb,3.6538e-04_rb,4.4243e-04_rb,5.3612e-04_rb,6.5384e-04_rb/)
      kao(:, 9, 8) = (/ &
     &3.1669e-04_rb,4.3173e-04_rb,5.9018e-04_rb,7.8164e-04_rb,9.3453e-04_rb/)
      kao(:,10, 8) = (/ &
     &2.7923e-04_rb,3.5605e-04_rb,4.7230e-04_rb,7.5710e-04_rb,1.3303e-03_rb/)
      kao(:,11, 8) = (/ &
     &3.9189e-04_rb,4.5454e-04_rb,6.3718e-04_rb,5.9520e-04_rb,8.3511e-04_rb/)
      kao(:,12, 8) = (/ &
     &2.6981e-04_rb,5.3035e-04_rb,6.9787e-04_rb,8.5120e-04_rb,7.8888e-04_rb/)
      kao(:,13, 8) = (/ &
     &2.5533e-04_rb,5.3884e-04_rb,7.3128e-04_rb,1.0359e-03_rb,9.5286e-04_rb/)
      kao(:, 1, 9) = (/ &
     &4.1658e-03_rb,5.4654e-03_rb,6.9658e-03_rb,8.6722e-03_rb,1.0572e-02_rb/)
      kao(:, 2, 9) = (/ &
     &3.6223e-03_rb,4.7934e-03_rb,6.1539e-03_rb,7.6963e-03_rb,9.3961e-03_rb/)
      kao(:, 3, 9) = (/ &
     &2.8899e-03_rb,3.8874e-03_rb,5.0544e-03_rb,6.3815e-03_rb,7.8638e-03_rb/)
      kao(:, 4, 9) = (/ &
     &2.1989e-03_rb,3.0197e-03_rb,3.9971e-03_rb,5.1085e-03_rb,6.3356e-03_rb/)
      kao(:, 5, 9) = (/ &
     &1.5192e-03_rb,2.2278e-03_rb,3.0194e-03_rb,3.9009e-03_rb,4.8746e-03_rb/)
      kao(:, 6, 9) = (/ &
     &8.7921e-04_rb,1.3751e-03_rb,1.9853e-03_rb,2.7076e-03_rb,3.5697e-03_rb/)
      kao(:, 7, 9) = (/ &
     &8.1169e-04_rb,1.0620e-03_rb,1.3025e-03_rb,1.8213e-03_rb,2.5733e-03_rb/)
      kao(:, 8, 9) = (/ &
     &8.9592e-04_rb,1.1349e-03_rb,1.5068e-03_rb,1.9574e-03_rb,2.3526e-03_rb/)
      kao(:, 9, 9) = (/ &
     &6.4937e-04_rb,9.7395e-04_rb,1.5824e-03_rb,2.3319e-03_rb,3.1640e-03_rb/)
      kao(:,10, 9) = (/ &
     &5.2173e-04_rb,1.2542e-03_rb,1.3511e-03_rb,1.8295e-03_rb,2.2102e-03_rb/)
      kao(:,11, 9) = (/ &
     &3.8299e-05_rb,7.6837e-04_rb,1.6073e-03_rb,2.1707e-03_rb,2.6859e-03_rb/)
      kao(:,12, 9) = (/ &
     &5.7389e-05_rb,4.8395e-04_rb,1.3815e-03_rb,2.5956e-03_rb,2.8856e-03_rb/)
      kao(:,13, 9) = (/ &
     &6.6437e-05_rb,4.1020e-04_rb,1.3561e-03_rb,2.3769e-03_rb,2.6400e-03_rb/)
      kao(:, 1,10) = (/ &
     &8.5531e-03_rb,1.1311e-02_rb,1.4946e-02_rb,1.9199e-02_rb,2.4070e-02_rb/)
      kao(:, 2,10) = (/ &
     &7.9512e-03_rb,1.0579e-02_rb,1.3947e-02_rb,1.7937e-02_rb,2.2682e-02_rb/)
      kao(:, 3,10) = (/ &
     &6.8586e-03_rb,9.3196e-03_rb,1.2459e-02_rb,1.6108e-02_rb,2.0278e-02_rb/)
      kao(:, 4,10) = (/ &
     &5.7104e-03_rb,7.9133e-03_rb,1.0642e-02_rb,1.3936e-02_rb,1.7830e-02_rb/)
      kao(:, 5,10) = (/ &
     &4.5965e-03_rb,6.4891e-03_rb,8.9433e-03_rb,1.1923e-02_rb,1.5372e-02_rb/)
      kao(:, 6,10) = (/ &
     &3.5770e-03_rb,5.1433e-03_rb,7.2049e-03_rb,9.7750e-03_rb,1.2529e-02_rb/)
      kao(:, 7,10) = (/ &
     &1.4544e-03_rb,1.6129e-03_rb,4.3574e-03_rb,7.0808e-03_rb,9.3333e-03_rb/)
      kao(:, 8,10) = (/ &
     &1.6681e-03_rb,2.3313e-03_rb,2.9075e-03_rb,3.3382e-03_rb,4.3787e-03_rb/)
      kao(:, 9,10) = (/ &
     &8.5765e-04_rb,1.7252e-03_rb,1.7039e-03_rb,3.3311e-03_rb,4.8997e-03_rb/)
      kao(:,10,10) = (/ &
     &2.2034e-05_rb,3.2730e-05_rb,2.9477e-03_rb,2.5954e-03_rb,5.2191e-03_rb/)
      kao(:,11,10) = (/ &
     &1.5700e-05_rb,2.2697e-05_rb,3.1811e-05_rb,9.1022e-03_rb,5.0873e-03_rb/)
      kao(:,12,10) = (/ &
     &1.1747e-05_rb,1.7581e-05_rb,2.4222e-05_rb,3.4764e-05_rb,8.4666e-03_rb/)
      kao(:,13,10) = (/ &
     &8.0435e-06_rb,1.2386e-05_rb,1.8547e-05_rb,2.6164e-05_rb,8.2788e-03_rb/)
      kao(:, 1,11) = (/ &
     &1.1747e-02_rb,1.5703e-02_rb,2.0198e-02_rb,2.5614e-02_rb,3.2111e-02_rb/)
      kao(:, 2,11) = (/ &
     &1.0674e-02_rb,1.4371e-02_rb,1.8733e-02_rb,2.4132e-02_rb,3.0707e-02_rb/)
      kao(:, 3,11) = (/ &
     &9.1890e-03_rb,1.2390e-02_rb,1.6367e-02_rb,2.1591e-02_rb,2.7893e-02_rb/)
      kao(:, 4,11) = (/ &
     &7.8825e-03_rb,1.0755e-02_rb,1.4391e-02_rb,1.9071e-02_rb,2.4702e-02_rb/)
      kao(:, 5,11) = (/ &
     &6.5907e-03_rb,9.1636e-03_rb,1.2468e-02_rb,1.6782e-02_rb,2.1901e-02_rb/)
      kao(:, 6,11) = (/ &
     &5.2625e-03_rb,7.5549e-03_rb,1.0521e-02_rb,1.4321e-02_rb,1.8906e-02_rb/)
      kao(:, 7,11) = (/ &
     &1.3421e-03_rb,5.0016e-03_rb,8.2742e-03_rb,1.0825e-02_rb,1.4096e-02_rb/)
      kao(:, 8,11) = (/ &
     &1.1288e-03_rb,2.7904e-03_rb,2.9048e-03_rb,3.8915e-03_rb,1.0764e-02_rb/)
      kao(:, 9,11) = (/ &
     &1.1212e-03_rb,3.3992e-03_rb,2.3130e-03_rb,2.5518e-03_rb,6.4495e-03_rb/)
      kao(:,10,11) = (/ &
     &2.4789e-05_rb,3.8028e-05_rb,7.9034e-03_rb,4.9082e-03_rb,4.7487e-03_rb/)
      kao(:,11,11) = (/ &
     &1.5630e-05_rb,2.4780e-05_rb,3.4663e-05_rb,4.3353e-05_rb,1.1174e-02_rb/)
      kao(:,12,11) = (/ &
     &1.2516e-05_rb,1.8590e-05_rb,2.5768e-05_rb,3.2997e-05_rb,7.6429e-03_rb/)
      kao(:,13,11) = (/ &
     &9.1474e-06_rb,1.3280e-05_rb,1.7796e-05_rb,2.4600e-05_rb,8.5245e-03_rb/)
      kao(:, 1,12) = (/ &
     &1.9137e-02_rb,2.4205e-02_rb,3.0237e-02_rb,3.7229e-02_rb,4.5273e-02_rb/)
      kao(:, 2,12) = (/ &
     &1.6304e-02_rb,2.1414e-02_rb,2.7665e-02_rb,3.5151e-02_rb,4.3535e-02_rb/)
      kao(:, 3,12) = (/ &
     &1.3446e-02_rb,1.8468e-02_rb,2.4699e-02_rb,3.1802e-02_rb,4.0159e-02_rb/)
      kao(:, 4,12) = (/ &
     &1.0988e-02_rb,1.5565e-02_rb,2.1262e-02_rb,2.7914e-02_rb,3.5968e-02_rb/)
      kao(:, 5,12) = (/ &
     &9.1659e-03_rb,1.3233e-02_rb,1.8175e-02_rb,2.4059e-02_rb,3.1596e-02_rb/)
      kao(:, 6,12) = (/ &
     &7.6758e-03_rb,1.1199e-02_rb,1.5520e-02_rb,2.0906e-02_rb,2.7754e-02_rb/)
      kao(:, 7,12) = (/ &
     &5.4756e-03_rb,9.3901e-03_rb,1.3292e-02_rb,1.8129e-02_rb,2.3738e-02_rb/)
      kao(:, 8,12) = (/ &
     &8.4306e-04_rb,3.0332e-03_rb,3.5332e-03_rb,1.1602e-02_rb,1.7968e-02_rb/)
      kao(:, 9,12) = (/ &
     &1.4946e-03_rb,1.5749e-03_rb,3.2158e-03_rb,3.1867e-03_rb,3.5737e-03_rb/)
      kao(:,10,12) = (/ &
     &2.3105e-05_rb,3.7657e-05_rb,5.4692e-05_rb,1.0942e-02_rb,7.6646e-03_rb/)
      kao(:,11,12) = (/ &
     &1.6278e-05_rb,2.6849e-05_rb,3.8655e-05_rb,5.4498e-05_rb,7.7401e-03_rb/)
      kao(:,12,12) = (/ &
     &1.1713e-05_rb,1.9330e-05_rb,2.9159e-05_rb,3.9386e-05_rb,5.6995e-05_rb/)
      kao(:,13,12) = (/ &
     &9.2970e-06_rb,1.5197e-05_rb,2.2881e-05_rb,3.3051e-05_rb,4.6248e-05_rb/)
      kao(:, 1,13) = (/ &
     &3.6366e-02_rb,4.6887e-02_rb,5.8513e-02_rb,7.0986e-02_rb,8.4117e-02_rb/)
      kao(:, 2,13) = (/ &
     &3.3990e-02_rb,4.3857e-02_rb,5.4708e-02_rb,6.6354e-02_rb,7.9156e-02_rb/)
      kao(:, 3,13) = (/ &
     &2.8482e-02_rb,3.7147e-02_rb,4.6806e-02_rb,5.7889e-02_rb,7.0408e-02_rb/)
      kao(:, 4,13) = (/ &
     &2.2450e-02_rb,2.9975e-02_rb,3.8835e-02_rb,4.9297e-02_rb,6.1450e-02_rb/)
      kao(:, 5,13) = (/ &
     &1.7322e-02_rb,2.3874e-02_rb,3.2063e-02_rb,4.2032e-02_rb,5.3868e-02_rb/)
      kao(:, 6,13) = (/ &
     &1.3022e-02_rb,1.8695e-02_rb,2.6114e-02_rb,3.5380e-02_rb,4.6717e-02_rb/)
      kao(:, 7,13) = (/ &
     &9.9635e-03_rb,1.4798e-02_rb,2.1337e-02_rb,2.9901e-02_rb,4.0578e-02_rb/)
      kao(:, 8,13) = (/ &
     &9.7470e-04_rb,2.7616e-03_rb,1.5907e-02_rb,2.2415e-02_rb,2.7492e-02_rb/)
      kao(:, 9,13) = (/ &
     &1.9362e-03_rb,1.8145e-03_rb,4.1737e-03_rb,3.7651e-03_rb,6.5913e-03_rb/)
      kao(:,10,13) = (/ &
     &2.8915e-05_rb,4.8621e-05_rb,6.9201e-05_rb,8.4840e-03_rb,1.1790e-02_rb/)
      kao(:,11,13) = (/ &
     &2.3309e-05_rb,3.4553e-05_rb,5.2552e-05_rb,7.2505e-05_rb,8.3347e-05_rb/)
      kao(:,12,13) = (/ &
     &1.5870e-05_rb,2.3577e-05_rb,3.4029e-05_rb,5.3277e-05_rb,6.5207e-05_rb/)
      kao(:,13,13) = (/ &
     &1.1931e-05_rb,1.9929e-05_rb,2.7393e-05_rb,3.2822e-05_rb,4.4265e-05_rb/)
      kao(:, 1,14) = (/ &
     &6.9682e-02_rb,8.7076e-02_rb,1.0644e-01_rb,1.2772e-01_rb,1.5052e-01_rb/)
      kao(:, 2,14) = (/ &
     &6.7443e-02_rb,8.6048e-02_rb,1.0697e-01_rb,1.3011e-01_rb,1.5516e-01_rb/)
      kao(:, 3,14) = (/ &
     &6.1305e-02_rb,8.0158e-02_rb,1.0175e-01_rb,1.2582e-01_rb,1.5220e-01_rb/)
      kao(:, 4,14) = (/ &
     &5.3357e-02_rb,7.1652e-02_rb,9.2935e-02_rb,1.1706e-01_rb,1.4376e-01_rb/)
      kao(:, 5,14) = (/ &
     &4.5060e-02_rb,6.2229e-02_rb,8.2609e-02_rb,1.0608e-01_rb,1.3238e-01_rb/)
      kao(:, 6,14) = (/ &
     &3.6438e-02_rb,5.1966e-02_rb,7.0824e-02_rb,9.2958e-02_rb,1.1819e-01_rb/)
      kao(:, 7,14) = (/ &
     &2.8491e-02_rb,4.2112e-02_rb,5.9112e-02_rb,7.9508e-02_rb,1.0325e-01_rb/)
      kao(:, 8,14) = (/ &
     &1.8236e-02_rb,3.3304e-02_rb,4.8438e-02_rb,6.6896e-02_rb,8.8683e-02_rb/)
      kao(:, 9,14) = (/ &
     &2.4537e-03_rb,2.3453e-03_rb,8.8067e-03_rb,2.0762e-02_rb,4.5513e-02_rb/)
      kao(:,10,14) = (/ &
     &1.4330e-05_rb,1.7518e-05_rb,3.1868e-05_rb,4.9025e-05_rb,3.3153e-02_rb/)
      kao(:,11,14) = (/ &
     &1.4087e-05_rb,2.3580e-05_rb,3.6793e-05_rb,4.1734e-05_rb,4.2976e-05_rb/)
      kao(:,12,14) = (/ &
     &1.3087e-05_rb,2.2036e-05_rb,3.4770e-05_rb,3.7226e-05_rb,3.8663e-05_rb/)
      kao(:,13,14) = (/ &
     &6.3004e-06_rb,7.4647e-06_rb,1.3256e-05_rb,1.9422e-05_rb,2.7358e-05_rb/)
      kao(:, 1,15) = (/ &
     &1.0457e-01_rb,1.3337e-01_rb,1.6543e-01_rb,2.0044e-01_rb,2.3788e-01_rb/)
      kao(:, 2,15) = (/ &
     &1.1420e-01_rb,1.4734e-01_rb,1.8451e-01_rb,2.2524e-01_rb,2.6907e-01_rb/)
      kao(:, 3,15) = (/ &
     &1.1631e-01_rb,1.5291e-01_rb,1.9437e-01_rb,2.4033e-01_rb,2.9013e-01_rb/)
      kao(:, 4,15) = (/ &
     &1.1302e-01_rb,1.5183e-01_rb,1.9657e-01_rb,2.4665e-01_rb,3.0148e-01_rb/)
      kao(:, 5,15) = (/ &
     &1.0669e-01_rb,1.4681e-01_rb,1.9375e-01_rb,2.4698e-01_rb,3.0586e-01_rb/)
      kao(:, 6,15) = (/ &
     &9.6727e-02_rb,1.3681e-01_rb,1.8458e-01_rb,2.3961e-01_rb,3.0104e-01_rb/)
      kao(:, 7,15) = (/ &
     &8.4990e-02_rb,1.2388e-01_rb,1.7124e-01_rb,2.2660e-01_rb,2.8964e-01_rb/)
      kao(:, 8,15) = (/ &
     &7.2581e-02_rb,1.0932e-01_rb,1.5506e-01_rb,2.1052e-01_rb,2.7593e-01_rb/)
      kao(:, 9,15) = (/ &
     &2.1552e-02_rb,6.6962e-02_rb,1.2166e-01_rb,1.8985e-01_rb,2.2620e-01_rb/)
      kao(:,10,15) = (/ &
     &8.5757e-06_rb,7.9319e-06_rb,7.5384e-06_rb,5.2149e-06_rb,3.5776e-06_rb/)
      kao(:,11,15) = (/ &
     &7.3855e-06_rb,7.4681e-06_rb,7.1804e-06_rb,6.5771e-06_rb,3.3890e-06_rb/)
      kao(:,12,15) = (/ &
     &5.9318e-06_rb,6.7082e-06_rb,7.4828e-06_rb,6.0815e-06_rb,3.2500e-06_rb/)
      kao(:,13,15) = (/ &
     &7.9734e-06_rb,1.0467e-05_rb,1.8551e-05_rb,3.3018e-05_rb,1.4396e-05_rb/)
      kao(:, 1,16) = (/ &
     &1.0940e-01_rb,1.4000e-01_rb,1.7421e-01_rb,2.1173e-01_rb,2.5214e-01_rb/)
      kao(:, 2,16) = (/ &
     &1.2207e-01_rb,1.5824e-01_rb,1.9908e-01_rb,2.4418e-01_rb,2.9310e-01_rb/)
      kao(:, 3,16) = (/ &
     &1.2803e-01_rb,1.6941e-01_rb,2.1687e-01_rb,2.6992e-01_rb,3.2810e-01_rb/)
      kao(:, 4,16) = (/ &
     &1.2912e-01_rb,1.7512e-01_rb,2.2885e-01_rb,2.8989e-01_rb,3.5760e-01_rb/)
      kao(:, 5,16) = (/ &
     &1.2782e-01_rb,1.7809e-01_rb,2.3798e-01_rb,3.0717e-01_rb,3.8500e-01_rb/)
      kao(:, 6,16) = (/ &
     &1.2288e-01_rb,1.7665e-01_rb,2.4226e-01_rb,3.1945e-01_rb,4.0769e-01_rb/)
      kao(:, 7,16) = (/ &
     &1.1601e-01_rb,1.7265e-01_rb,2.4356e-01_rb,3.2874e-01_rb,4.2783e-01_rb/)
      kao(:, 8,16) = (/ &
     &1.0802e-01_rb,1.6690e-01_rb,2.4265e-01_rb,3.3582e-01_rb,4.4617e-01_rb/)
      kao(:, 9,16) = (/ &
     &9.8584e-02_rb,1.5878e-01_rb,2.3864e-01_rb,3.3924e-01_rb,4.6085e-01_rb/)
      kao(:,10,16) = (/ &
     &2.5370e-06_rb,3.6996e-06_rb,5.1180e-06_rb,9.0911e-06_rb,1.0521e-05_rb/)
      kao(:,11,16) = (/ &
     &2.1410e-06_rb,3.1393e-06_rb,2.7559e-06_rb,9.6933e-06_rb,1.1052e-05_rb/)
      kao(:,12,16) = (/ &
     &1.7911e-06_rb,2.0648e-06_rb,1.9476e-06_rb,1.0250e-05_rb,1.1569e-05_rb/)
      kao(:,13,16) = (/ &
     &6.8424e-07_rb,2.1307e-05_rb,3.0235e-05_rb,1.0419e-05_rb,1.1877e-05_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &2.1590e-02_rb,2.2747e-02_rb,2.3755e-02_rb,2.4762e-02_rb,2.5739e-02_rb/)
      kbo(:,14, 1) = (/ &
     &1.5891e-02_rb,1.6865e-02_rb,1.7785e-02_rb,1.8588e-02_rb,1.9375e-02_rb/)
      kbo(:,15, 1) = (/ &
     &1.2008e-02_rb,1.2891e-02_rb,1.3700e-02_rb,1.4369e-02_rb,1.4970e-02_rb/)
      kbo(:,16, 1) = (/ &
     &9.5594e-03_rb,1.0266e-02_rb,1.0893e-02_rb,1.1429e-02_rb,1.1863e-02_rb/)
      kbo(:,17, 1) = (/ &
     &7.6979e-03_rb,8.2617e-03_rb,8.7527e-03_rb,9.1620e-03_rb,9.5161e-03_rb/)
      kbo(:,18, 1) = (/ &
     &6.2689e-03_rb,6.7130e-03_rb,7.1029e-03_rb,7.4199e-03_rb,7.7032e-03_rb/)
      kbo(:,19, 1) = (/ &
     &5.1124e-03_rb,5.4669e-03_rb,5.7636e-03_rb,6.0175e-03_rb,6.2439e-03_rb/)
      kbo(:,20, 1) = (/ &
     &4.2074e-03_rb,4.4879e-03_rb,4.7226e-03_rb,4.9240e-03_rb,5.1066e-03_rb/)
      kbo(:,21, 1) = (/ &
     &3.4689e-03_rb,3.6905e-03_rb,3.8771e-03_rb,4.0402e-03_rb,4.1874e-03_rb/)
      kbo(:,22, 1) = (/ &
     &2.8675e-03_rb,3.0422e-03_rb,3.1923e-03_rb,3.3247e-03_rb,3.4419e-03_rb/)
      kbo(:,23, 1) = (/ &
     &2.3673e-03_rb,2.5068e-03_rb,2.6269e-03_rb,2.7338e-03_rb,2.8272e-03_rb/)
      kbo(:,24, 1) = (/ &
     &1.9512e-03_rb,2.0620e-03_rb,2.1580e-03_rb,2.2436e-03_rb,2.3178e-03_rb/)
      kbo(:,25, 1) = (/ &
     &1.6104e-03_rb,1.6991e-03_rb,1.7756e-03_rb,1.8445e-03_rb,1.9031e-03_rb/)
      kbo(:,26, 1) = (/ &
     &1.3324e-03_rb,1.4030e-03_rb,1.4641e-03_rb,1.5181e-03_rb,1.5647e-03_rb/)
      kbo(:,27, 1) = (/ &
     &1.1030e-03_rb,1.1588e-03_rb,1.2079e-03_rb,1.2506e-03_rb,1.2885e-03_rb/)
      kbo(:,28, 1) = (/ &
     &9.1277e-04_rb,9.5694e-04_rb,9.9621e-04_rb,1.0308e-03_rb,1.0603e-03_rb/)
      kbo(:,29, 1) = (/ &
     &7.5709e-04_rb,7.9257e-04_rb,8.2415e-04_rb,8.5205e-04_rb,8.7500e-04_rb/)
      kbo(:,30, 1) = (/ &
     &6.2805e-04_rb,6.5667e-04_rb,6.8205e-04_rb,7.0403e-04_rb,7.2224e-04_rb/)
      kbo(:,31, 1) = (/ &
     &5.2190e-04_rb,5.4469e-04_rb,5.6521e-04_rb,5.8274e-04_rb,5.9713e-04_rb/)
      kbo(:,32, 1) = (/ &
     &4.3349e-04_rb,4.5214e-04_rb,4.6842e-04_rb,4.8237e-04_rb,4.9367e-04_rb/)
      kbo(:,33, 1) = (/ &
     &3.5999e-04_rb,3.7522e-04_rb,3.8828e-04_rb,3.9917e-04_rb,4.0831e-04_rb/)
      kbo(:,34, 1) = (/ &
     &2.9885e-04_rb,3.1108e-04_rb,3.2123e-04_rb,3.3020e-04_rb,3.3735e-04_rb/)
      kbo(:,35, 1) = (/ &
     &2.4686e-04_rb,2.5649e-04_rb,2.6489e-04_rb,2.7217e-04_rb,2.7798e-04_rb/)
      kbo(:,36, 1) = (/ &
     &2.0249e-04_rb,2.1050e-04_rb,2.1752e-04_rb,2.2349e-04_rb,2.2845e-04_rb/)
      kbo(:,37, 1) = (/ &
     &1.6556e-04_rb,1.7221e-04_rb,1.7821e-04_rb,1.8323e-04_rb,1.8756e-04_rb/)
      kbo(:,38, 1) = (/ &
     &1.3530e-04_rb,1.4093e-04_rb,1.4599e-04_rb,1.5028e-04_rb,1.5406e-04_rb/)
      kbo(:,39, 1) = (/ &
     &1.1062e-04_rb,1.1539e-04_rb,1.1961e-04_rb,1.2331e-04_rb,1.2663e-04_rb/)
      kbo(:,40, 1) = (/ &
     &8.9884e-05_rb,9.3980e-05_rb,9.7546e-05_rb,1.0075e-04_rb,1.0361e-04_rb/)
      kbo(:,41, 1) = (/ &
     &7.2964e-05_rb,7.6433e-05_rb,7.9468e-05_rb,8.2219e-05_rb,8.4723e-05_rb/)
      kbo(:,42, 1) = (/ &
     &5.9182e-05_rb,6.2146e-05_rb,6.4721e-05_rb,6.7085e-05_rb,6.9267e-05_rb/)
      kbo(:,43, 1) = (/ &
     &4.7864e-05_rb,5.0382e-05_rb,5.2591e-05_rb,5.4634e-05_rb,5.6528e-05_rb/)
      kbo(:,44, 1) = (/ &
     &3.8628e-05_rb,4.0783e-05_rb,4.2701e-05_rb,4.4436e-05_rb,4.6114e-05_rb/)
      kbo(:,45, 1) = (/ &
     &3.1127e-05_rb,3.3002e-05_rb,3.4654e-05_rb,3.6151e-05_rb,3.7566e-05_rb/)
      kbo(:,46, 1) = (/ &
     &2.5026e-05_rb,2.6649e-05_rb,2.8068e-05_rb,2.9367e-05_rb,3.0563e-05_rb/)
      kbo(:,47, 1) = (/ &
     &2.0040e-05_rb,2.1423e-05_rb,2.2656e-05_rb,2.3781e-05_rb,2.4816e-05_rb/)
      kbo(:,48, 1) = (/ &
     &1.6036e-05_rb,1.7209e-05_rb,1.8283e-05_rb,1.9267e-05_rb,2.0138e-05_rb/)
      kbo(:,49, 1) = (/ &
     &1.2828e-05_rb,1.3830e-05_rb,1.4752e-05_rb,1.5602e-05_rb,1.6354e-05_rb/)
      kbo(:,50, 1) = (/ &
     &1.0257e-05_rb,1.1106e-05_rb,1.1884e-05_rb,1.2616e-05_rb,1.3265e-05_rb/)
      kbo(:,51, 1) = (/ &
     &8.1934e-06_rb,8.9110e-06_rb,9.5669e-06_rb,1.0186e-05_rb,1.0748e-05_rb/)
      kbo(:,52, 1) = (/ &
     &6.5467e-06_rb,7.1424e-06_rb,7.7065e-06_rb,8.2286e-06_rb,8.7123e-06_rb/)
      kbo(:,53, 1) = (/ &
     &5.2332e-06_rb,5.7328e-06_rb,6.2119e-06_rb,6.6510e-06_rb,7.0700e-06_rb/)
      kbo(:,54, 1) = (/ &
     &4.1577e-06_rb,4.5779e-06_rb,4.9804e-06_rb,5.3498e-06_rb,5.7020e-06_rb/)
      kbo(:,55, 1) = (/ &
     &3.2878e-06_rb,3.6403e-06_rb,3.9741e-06_rb,4.2805e-06_rb,4.5760e-06_rb/)
      kbo(:,56, 1) = (/ &
     &2.5965e-06_rb,2.8911e-06_rb,3.1663e-06_rb,3.4265e-06_rb,3.6666e-06_rb/)
      kbo(:,57, 1) = (/ &
     &2.0466e-06_rb,2.2908e-06_rb,2.5217e-06_rb,2.7403e-06_rb,2.9373e-06_rb/)
      kbo(:,58, 1) = (/ &
     &1.6093e-06_rb,1.8128e-06_rb,2.0059e-06_rb,2.1882e-06_rb,2.3529e-06_rb/)
      kbo(:,59, 1) = (/ &
     &1.2776e-06_rb,1.4459e-06_rb,1.6042e-06_rb,1.7524e-06_rb,1.8906e-06_rb/)
      kbo(:,13, 2) = (/ &
     &3.8014e-02_rb,3.9737e-02_rb,4.1825e-02_rb,4.3879e-02_rb,4.5802e-02_rb/)
      kbo(:,14, 2) = (/ &
     &3.0759e-02_rb,3.2195e-02_rb,3.3860e-02_rb,3.5735e-02_rb,3.7445e-02_rb/)
      kbo(:,15, 2) = (/ &
     &2.4737e-02_rb,2.6065e-02_rb,2.7499e-02_rb,2.8985e-02_rb,3.0345e-02_rb/)
      kbo(:,16, 2) = (/ &
     &2.0386e-02_rb,2.1604e-02_rb,2.2870e-02_rb,2.4062e-02_rb,2.5169e-02_rb/)
      kbo(:,17, 2) = (/ &
     &1.6973e-02_rb,1.8030e-02_rb,1.9046e-02_rb,1.9999e-02_rb,2.0836e-02_rb/)
      kbo(:,18, 2) = (/ &
     &1.4179e-02_rb,1.5045e-02_rb,1.5843e-02_rb,1.6598e-02_rb,1.7207e-02_rb/)
      kbo(:,19, 2) = (/ &
     &1.1784e-02_rb,1.2488e-02_rb,1.3131e-02_rb,1.3688e-02_rb,1.4139e-02_rb/)
      kbo(:,20, 2) = (/ &
     &9.8208e-03_rb,1.0377e-02_rb,1.0872e-02_rb,1.1296e-02_rb,1.1641e-02_rb/)
      kbo(:,21, 2) = (/ &
     &8.1632e-03_rb,8.5996e-03_rb,8.9808e-03_rb,9.3040e-03_rb,9.5696e-03_rb/)
      kbo(:,22, 2) = (/ &
     &6.7970e-03_rb,7.1290e-03_rb,7.4204e-03_rb,7.6665e-03_rb,7.8692e-03_rb/)
      kbo(:,23, 2) = (/ &
     &5.6384e-03_rb,5.8917e-03_rb,6.1228e-03_rb,6.3098e-03_rb,6.4698e-03_rb/)
      kbo(:,24, 2) = (/ &
     &4.6602e-03_rb,4.8595e-03_rb,5.0396e-03_rb,5.1884e-03_rb,5.3131e-03_rb/)
      kbo(:,25, 2) = (/ &
     &3.8492e-03_rb,4.0101e-03_rb,4.1507e-03_rb,4.2656e-03_rb,4.3666e-03_rb/)
      kbo(:,26, 2) = (/ &
     &3.1817e-03_rb,3.3089e-03_rb,3.4202e-03_rb,3.5130e-03_rb,3.5943e-03_rb/)
      kbo(:,27, 2) = (/ &
     &2.6312e-03_rb,2.7340e-03_rb,2.8215e-03_rb,2.8952e-03_rb,2.9585e-03_rb/)
      kbo(:,28, 2) = (/ &
     &2.1766e-03_rb,2.2592e-03_rb,2.3277e-03_rb,2.3874e-03_rb,2.4369e-03_rb/)
      kbo(:,29, 2) = (/ &
     &1.8050e-03_rb,1.8690e-03_rb,1.9243e-03_rb,1.9721e-03_rb,2.0134e-03_rb/)
      kbo(:,30, 2) = (/ &
     &1.4965e-03_rb,1.5474e-03_rb,1.5922e-03_rb,1.6307e-03_rb,1.6644e-03_rb/)
      kbo(:,31, 2) = (/ &
     &1.2416e-03_rb,1.2835e-03_rb,1.3200e-03_rb,1.3519e-03_rb,1.3783e-03_rb/)
      kbo(:,32, 2) = (/ &
     &1.0308e-03_rb,1.0652e-03_rb,1.0950e-03_rb,1.1207e-03_rb,1.1427e-03_rb/)
      kbo(:,33, 2) = (/ &
     &8.5698e-04_rb,8.8430e-04_rb,9.0863e-04_rb,9.2966e-04_rb,9.4803e-04_rb/)
      kbo(:,34, 2) = (/ &
     &7.1109e-04_rb,7.3332e-04_rb,7.5331e-04_rb,7.7074e-04_rb,7.8601e-04_rb/)
      kbo(:,35, 2) = (/ &
     &5.8729e-04_rb,6.0603e-04_rb,6.2268e-04_rb,6.3737e-04_rb,6.5045e-04_rb/)
      kbo(:,36, 2) = (/ &
     &4.8310e-04_rb,4.9861e-04_rb,5.1251e-04_rb,5.2508e-04_rb,5.3615e-04_rb/)
      kbo(:,37, 2) = (/ &
     &3.9625e-04_rb,4.0955e-04_rb,4.2151e-04_rb,4.3225e-04_rb,4.4172e-04_rb/)
      kbo(:,38, 2) = (/ &
     &3.2503e-04_rb,3.3643e-04_rb,3.4664e-04_rb,3.5587e-04_rb,3.6445e-04_rb/)
      kbo(:,39, 2) = (/ &
     &2.6664e-04_rb,2.7639e-04_rb,2.8514e-04_rb,2.9332e-04_rb,3.0098e-04_rb/)
      kbo(:,40, 2) = (/ &
     &2.1759e-04_rb,2.2608e-04_rb,2.3348e-04_rb,2.4050e-04_rb,2.4712e-04_rb/)
      kbo(:,41, 2) = (/ &
     &1.7739e-04_rb,1.8462e-04_rb,1.9097e-04_rb,1.9703e-04_rb,2.0268e-04_rb/)
      kbo(:,42, 2) = (/ &
     &1.4453e-04_rb,1.5070e-04_rb,1.5612e-04_rb,1.6130e-04_rb,1.6626e-04_rb/)
      kbo(:,43, 2) = (/ &
     &1.1750e-04_rb,1.2276e-04_rb,1.2736e-04_rb,1.3185e-04_rb,1.3616e-04_rb/)
      kbo(:,44, 2) = (/ &
     &9.5367e-05_rb,9.9857e-05_rb,1.0380e-04_rb,1.0768e-04_rb,1.1140e-04_rb/)
      kbo(:,45, 2) = (/ &
     &7.7428e-05_rb,8.1152e-05_rb,8.4582e-05_rb,8.7898e-05_rb,9.1175e-05_rb/)
      kbo(:,46, 2) = (/ &
     &6.2722e-05_rb,6.5854e-05_rb,6.8828e-05_rb,7.1656e-05_rb,7.4494e-05_rb/)
      kbo(:,47, 2) = (/ &
     &5.0620e-05_rb,5.3291e-05_rb,5.5834e-05_rb,5.8234e-05_rb,6.0703e-05_rb/)
      kbo(:,48, 2) = (/ &
     &4.0839e-05_rb,4.3112e-05_rb,4.5285e-05_rb,4.7326e-05_rb,4.9468e-05_rb/)
      kbo(:,49, 2) = (/ &
     &3.2956e-05_rb,3.4871e-05_rb,3.6734e-05_rb,3.8477e-05_rb,4.0339e-05_rb/)
      kbo(:,50, 2) = (/ &
     &2.6578e-05_rb,2.8186e-05_rb,2.9765e-05_rb,3.1246e-05_rb,3.2810e-05_rb/)
      kbo(:,51, 2) = (/ &
     &2.1419e-05_rb,2.2753e-05_rb,2.4086e-05_rb,2.5342e-05_rb,2.6653e-05_rb/)
      kbo(:,52, 2) = (/ &
     &1.7265e-05_rb,1.8376e-05_rb,1.9492e-05_rb,2.0556e-05_rb,2.1651e-05_rb/)
      kbo(:,53, 2) = (/ &
     &1.3928e-05_rb,1.4844e-05_rb,1.5782e-05_rb,1.6689e-05_rb,1.7614e-05_rb/)
      kbo(:,54, 2) = (/ &
     &1.1205e-05_rb,1.1963e-05_rb,1.2726e-05_rb,1.3478e-05_rb,1.4240e-05_rb/)
      kbo(:,55, 2) = (/ &
     &8.9889e-06_rb,9.6190e-06_rb,1.0235e-05_rb,1.0868e-05_rb,1.1479e-05_rb/)
      kbo(:,56, 2) = (/ &
     &7.2032e-06_rb,7.7304e-06_rb,8.2333e-06_rb,8.7486e-06_rb,9.2547e-06_rb/)
      kbo(:,57, 2) = (/ &
     &5.7621e-06_rb,6.2097e-06_rb,6.6195e-06_rb,7.0372e-06_rb,7.4615e-06_rb/)
      kbo(:,58, 2) = (/ &
     &4.6073e-06_rb,4.9865e-06_rb,5.3240e-06_rb,5.6615e-06_rb,6.0083e-06_rb/)
      kbo(:,59, 2) = (/ &
     &3.7094e-06_rb,4.0178e-06_rb,4.3021e-06_rb,4.5755e-06_rb,4.8477e-06_rb/)
      kbo(:,13, 3) = (/ &
     &6.3405e-02_rb,6.5846e-02_rb,6.7902e-02_rb,7.0344e-02_rb,7.3087e-02_rb/)
      kbo(:,14, 3) = (/ &
     &5.1252e-02_rb,5.3361e-02_rb,5.5658e-02_rb,5.8048e-02_rb,6.0738e-02_rb/)
      kbo(:,15, 3) = (/ &
     &4.2340e-02_rb,4.4418e-02_rb,4.6657e-02_rb,4.8923e-02_rb,5.1338e-02_rb/)
      kbo(:,16, 3) = (/ &
     &3.5507e-02_rb,3.7339e-02_rb,3.9465e-02_rb,4.1652e-02_rb,4.3669e-02_rb/)
      kbo(:,17, 3) = (/ &
     &2.9824e-02_rb,3.1523e-02_rb,3.3435e-02_rb,3.5273e-02_rb,3.6953e-02_rb/)
      kbo(:,18, 3) = (/ &
     &2.5125e-02_rb,2.6683e-02_rb,2.8323e-02_rb,2.9797e-02_rb,3.1126e-02_rb/)
      kbo(:,19, 3) = (/ &
     &2.1223e-02_rb,2.2544e-02_rb,2.3820e-02_rb,2.4985e-02_rb,2.5990e-02_rb/)
      kbo(:,20, 3) = (/ &
     &1.7915e-02_rb,1.8986e-02_rb,1.9994e-02_rb,2.0861e-02_rb,2.1602e-02_rb/)
      kbo(:,21, 3) = (/ &
     &1.5062e-02_rb,1.5916e-02_rb,1.6665e-02_rb,1.7312e-02_rb,1.7872e-02_rb/)
      kbo(:,22, 3) = (/ &
     &1.2635e-02_rb,1.3292e-02_rb,1.3846e-02_rb,1.4329e-02_rb,1.4769e-02_rb/)
      kbo(:,23, 3) = (/ &
     &1.0554e-02_rb,1.1051e-02_rb,1.1464e-02_rb,1.1841e-02_rb,1.2180e-02_rb/)
      kbo(:,24, 3) = (/ &
     &8.7770e-03_rb,9.1488e-03_rb,9.4668e-03_rb,9.7625e-03_rb,1.0031e-02_rb/)
      kbo(:,25, 3) = (/ &
     &7.2756e-03_rb,7.5544e-03_rb,7.8035e-03_rb,8.0431e-03_rb,8.2495e-03_rb/)
      kbo(:,26, 3) = (/ &
     &6.0226e-03_rb,6.2378e-03_rb,6.4382e-03_rb,6.6253e-03_rb,6.7857e-03_rb/)
      kbo(:,27, 3) = (/ &
     &4.9811e-03_rb,5.1501e-03_rb,5.3115e-03_rb,5.4587e-03_rb,5.5851e-03_rb/)
      kbo(:,28, 3) = (/ &
     &4.1172e-03_rb,4.2566e-03_rb,4.3852e-03_rb,4.4995e-03_rb,4.6022e-03_rb/)
      kbo(:,29, 3) = (/ &
     &3.4101e-03_rb,3.5232e-03_rb,3.6273e-03_rb,3.7181e-03_rb,3.8009e-03_rb/)
      kbo(:,30, 3) = (/ &
     &2.8248e-03_rb,2.9164e-03_rb,2.9992e-03_rb,3.0754e-03_rb,3.1419e-03_rb/)
      kbo(:,31, 3) = (/ &
     &2.3450e-03_rb,2.4195e-03_rb,2.4879e-03_rb,2.5493e-03_rb,2.6070e-03_rb/)
      kbo(:,32, 3) = (/ &
     &1.9491e-03_rb,2.0095e-03_rb,2.0670e-03_rb,2.1190e-03_rb,2.1683e-03_rb/)
      kbo(:,33, 3) = (/ &
     &1.6195e-03_rb,1.6713e-03_rb,1.7189e-03_rb,1.7649e-03_rb,1.8032e-03_rb/)
      kbo(:,34, 3) = (/ &
     &1.3454e-03_rb,1.3883e-03_rb,1.4290e-03_rb,1.4663e-03_rb,1.4976e-03_rb/)
      kbo(:,35, 3) = (/ &
     &1.1151e-03_rb,1.1504e-03_rb,1.1846e-03_rb,1.2147e-03_rb,1.2425e-03_rb/)
      kbo(:,36, 3) = (/ &
     &9.1872e-04_rb,9.4904e-04_rb,9.7800e-04_rb,1.0033e-03_rb,1.0278e-03_rb/)
      kbo(:,37, 3) = (/ &
     &7.5556e-04_rb,7.8161e-04_rb,8.0630e-04_rb,8.2845e-04_rb,8.4914e-04_rb/)
      kbo(:,38, 3) = (/ &
     &6.2192e-04_rb,6.4458e-04_rb,6.6572e-04_rb,6.8474e-04_rb,7.0313e-04_rb/)
      kbo(:,39, 3) = (/ &
     &5.1238e-04_rb,5.3172e-04_rb,5.4973e-04_rb,5.6636e-04_rb,5.8235e-04_rb/)
      kbo(:,40, 3) = (/ &
     &4.1982e-04_rb,4.3621e-04_rb,4.5139e-04_rb,4.6596e-04_rb,4.7961e-04_rb/)
      kbo(:,41, 3) = (/ &
     &3.4388e-04_rb,3.5746e-04_rb,3.7054e-04_rb,3.8291e-04_rb,3.9470e-04_rb/)
      kbo(:,42, 3) = (/ &
     &2.8144e-04_rb,2.9295e-04_rb,3.0424e-04_rb,3.1457e-04_rb,3.2459e-04_rb/)
      kbo(:,43, 3) = (/ &
     &2.2990e-04_rb,2.3955e-04_rb,2.4928e-04_rb,2.5827e-04_rb,2.6672e-04_rb/)
      kbo(:,44, 3) = (/ &
     &1.8744e-04_rb,1.9582e-04_rb,2.0413e-04_rb,2.1176e-04_rb,2.1912e-04_rb/)
      kbo(:,45, 3) = (/ &
     &1.5285e-04_rb,1.6000e-04_rb,1.6711e-04_rb,1.7363e-04_rb,1.7993e-04_rb/)
      kbo(:,46, 3) = (/ &
     &1.2448e-04_rb,1.3059e-04_rb,1.3656e-04_rb,1.4216e-04_rb,1.4758e-04_rb/)
      kbo(:,47, 3) = (/ &
     &1.0103e-04_rb,1.0628e-04_rb,1.1130e-04_rb,1.1611e-04_rb,1.2074e-04_rb/)
      kbo(:,48, 3) = (/ &
     &8.1983e-05_rb,8.6504e-05_rb,9.0728e-05_rb,9.4862e-05_rb,9.8826e-05_rb/)
      kbo(:,49, 3) = (/ &
     &6.6563e-05_rb,7.0429e-05_rb,7.4014e-05_rb,7.7542e-05_rb,8.0976e-05_rb/)
      kbo(:,50, 3) = (/ &
     &5.3952e-05_rb,5.7235e-05_rb,6.0247e-05_rb,6.3258e-05_rb,6.6182e-05_rb/)
      kbo(:,51, 3) = (/ &
     &4.3663e-05_rb,4.6470e-05_rb,4.9001e-05_rb,5.1519e-05_rb,5.4013e-05_rb/)
      kbo(:,52, 3) = (/ &
     &3.5323e-05_rb,3.7705e-05_rb,3.9846e-05_rb,4.1973e-05_rb,4.4086e-05_rb/)
      kbo(:,53, 3) = (/ &
     &2.8583e-05_rb,3.0614e-05_rb,3.2428e-05_rb,3.4214e-05_rb,3.6024e-05_rb/)
      kbo(:,54, 3) = (/ &
     &2.3052e-05_rb,2.4741e-05_rb,2.6291e-05_rb,2.7754e-05_rb,2.9233e-05_rb/)
      kbo(:,55, 3) = (/ &
     &1.8532e-05_rb,1.9924e-05_rb,2.1226e-05_rb,2.2443e-05_rb,2.3644e-05_rb/)
      kbo(:,56, 3) = (/ &
     &1.4892e-05_rb,1.6026e-05_rb,1.7132e-05_rb,1.8148e-05_rb,1.9127e-05_rb/)
      kbo(:,57, 3) = (/ &
     &1.1957e-05_rb,1.2880e-05_rb,1.3809e-05_rb,1.4669e-05_rb,1.5458e-05_rb/)
      kbo(:,58, 3) = (/ &
     &9.6000e-06_rb,1.0349e-05_rb,1.1127e-05_rb,1.1847e-05_rb,1.2497e-05_rb/)
      kbo(:,59, 3) = (/ &
     &7.7452e-06_rb,8.3627e-06_rb,8.9895e-06_rb,9.5703e-06_rb,1.0120e-05_rb/)
      kbo(:,13, 4) = (/ &
     &9.9915e-02_rb,1.0385e-01_rb,1.0749e-01_rb,1.1095e-01_rb,1.1480e-01_rb/)
      kbo(:,14, 4) = (/ &
     &8.4174e-02_rb,8.7314e-02_rb,9.0833e-02_rb,9.4040e-02_rb,9.7483e-02_rb/)
      kbo(:,15, 4) = (/ &
     &7.1798e-02_rb,7.3772e-02_rb,7.6218e-02_rb,7.9303e-02_rb,8.2911e-02_rb/)
      kbo(:,16, 4) = (/ &
     &6.0987e-02_rb,6.2861e-02_rb,6.5027e-02_rb,6.7869e-02_rb,7.1492e-02_rb/)
      kbo(:,17, 4) = (/ &
     &5.1988e-02_rb,5.3759e-02_rb,5.5853e-02_rb,5.8672e-02_rb,6.1980e-02_rb/)
      kbo(:,18, 4) = (/ &
     &4.4171e-02_rb,4.5935e-02_rb,4.8095e-02_rb,5.0759e-02_rb,5.3616e-02_rb/)
      kbo(:,19, 4) = (/ &
     &3.7411e-02_rb,3.9215e-02_rb,4.1337e-02_rb,4.3631e-02_rb,4.5910e-02_rb/)
      kbo(:,20, 4) = (/ &
     &3.1780e-02_rb,3.3510e-02_rb,3.5386e-02_rb,3.7312e-02_rb,3.9095e-02_rb/)
      kbo(:,21, 4) = (/ &
     &2.6954e-02_rb,2.8513e-02_rb,3.0099e-02_rb,3.1617e-02_rb,3.2915e-02_rb/)
      kbo(:,22, 4) = (/ &
     &2.2888e-02_rb,2.4194e-02_rb,2.5477e-02_rb,2.6596e-02_rb,2.7511e-02_rb/)
      kbo(:,23, 4) = (/ &
     &1.9346e-02_rb,2.0404e-02_rb,2.1382e-02_rb,2.2165e-02_rb,2.2830e-02_rb/)
      kbo(:,24, 4) = (/ &
     &1.6247e-02_rb,1.7083e-02_rb,1.7781e-02_rb,1.8353e-02_rb,1.8851e-02_rb/)
      kbo(:,25, 4) = (/ &
     &1.3597e-02_rb,1.4221e-02_rb,1.4742e-02_rb,1.5163e-02_rb,1.5551e-02_rb/)
      kbo(:,26, 4) = (/ &
     &1.1347e-02_rb,1.1808e-02_rb,1.2193e-02_rb,1.2519e-02_rb,1.2827e-02_rb/)
      kbo(:,27, 4) = (/ &
     &9.4352e-03_rb,9.7820e-03_rb,1.0071e-02_rb,1.0331e-02_rb,1.0583e-02_rb/)
      kbo(:,28, 4) = (/ &
     &7.8281e-03_rb,8.0857e-03_rb,8.3110e-03_rb,8.5276e-03_rb,8.7304e-03_rb/)
      kbo(:,29, 4) = (/ &
     &6.4877e-03_rb,6.6881e-03_rb,6.8753e-03_rb,7.0489e-03_rb,7.2105e-03_rb/)
      kbo(:,30, 4) = (/ &
     &5.3741e-03_rb,5.5372e-03_rb,5.6910e-03_rb,5.8358e-03_rb,5.9652e-03_rb/)
      kbo(:,31, 4) = (/ &
     &4.4624e-03_rb,4.5974e-03_rb,4.7269e-03_rb,4.8416e-03_rb,4.9477e-03_rb/)
      kbo(:,32, 4) = (/ &
     &3.7110e-03_rb,3.8251e-03_rb,3.9302e-03_rb,4.0264e-03_rb,4.1112e-03_rb/)
      kbo(:,33, 4) = (/ &
     &3.0918e-03_rb,3.1864e-03_rb,3.2713e-03_rb,3.3504e-03_rb,3.4219e-03_rb/)
      kbo(:,34, 4) = (/ &
     &2.5734e-03_rb,2.6501e-03_rb,2.7238e-03_rb,2.7897e-03_rb,2.8499e-03_rb/)
      kbo(:,35, 4) = (/ &
     &2.1361e-03_rb,2.2030e-03_rb,2.2659e-03_rb,2.3224e-03_rb,2.3696e-03_rb/)
      kbo(:,36, 4) = (/ &
     &1.7673e-03_rb,1.8262e-03_rb,1.8805e-03_rb,1.9284e-03_rb,1.9688e-03_rb/)
      kbo(:,37, 4) = (/ &
     &1.4584e-03_rb,1.5086e-03_rb,1.5546e-03_rb,1.5960e-03_rb,1.6338e-03_rb/)
      kbo(:,38, 4) = (/ &
     &1.2034e-03_rb,1.2453e-03_rb,1.2856e-03_rb,1.3227e-03_rb,1.3572e-03_rb/)
      kbo(:,39, 4) = (/ &
     &9.9454e-04_rb,1.0299e-03_rb,1.0651e-03_rb,1.0982e-03_rb,1.1293e-03_rb/)
      kbo(:,40, 4) = (/ &
     &8.1766e-04_rb,8.4814e-04_rb,8.7922e-04_rb,9.0780e-04_rb,9.3604e-04_rb/)
      kbo(:,41, 4) = (/ &
     &6.7161e-04_rb,6.9882e-04_rb,7.2604e-04_rb,7.5179e-04_rb,7.7573e-04_rb/)
      kbo(:,42, 4) = (/ &
     &5.5233e-04_rb,5.7642e-04_rb,5.9983e-04_rb,6.2278e-04_rb,6.4351e-04_rb/)
      kbo(:,43, 4) = (/ &
     &4.5350e-04_rb,4.7458e-04_rb,4.9522e-04_rb,5.1500e-04_rb,5.3326e-04_rb/)
      kbo(:,44, 4) = (/ &
     &3.7202e-04_rb,3.9025e-04_rb,4.0848e-04_rb,4.2568e-04_rb,4.4143e-04_rb/)
      kbo(:,45, 4) = (/ &
     &3.0534e-04_rb,3.2126e-04_rb,3.3706e-04_rb,3.5201e-04_rb,3.6606e-04_rb/)
      kbo(:,46, 4) = (/ &
     &2.5026e-04_rb,2.6416e-04_rb,2.7768e-04_rb,2.9081e-04_rb,3.0319e-04_rb/)
      kbo(:,47, 4) = (/ &
     &2.0428e-04_rb,2.1616e-04_rb,2.2794e-04_rb,2.3946e-04_rb,2.5046e-04_rb/)
      kbo(:,48, 4) = (/ &
     &1.6667e-04_rb,1.7701e-04_rb,1.8729e-04_rb,1.9740e-04_rb,2.0693e-04_rb/)
      kbo(:,49, 4) = (/ &
     &1.3611e-04_rb,1.4498e-04_rb,1.5402e-04_rb,1.6290e-04_rb,1.7121e-04_rb/)
      kbo(:,50, 4) = (/ &
     &1.1088e-04_rb,1.1847e-04_rb,1.2631e-04_rb,1.3390e-04_rb,1.4119e-04_rb/)
      kbo(:,51, 4) = (/ &
     &9.0216e-05_rb,9.6607e-05_rb,1.0331e-04_rb,1.0986e-04_rb,1.1615e-04_rb/)
      kbo(:,52, 4) = (/ &
     &7.3423e-05_rb,7.8834e-05_rb,8.4566e-05_rb,9.0143e-05_rb,9.5562e-05_rb/)
      kbo(:,53, 4) = (/ &
     &5.9754e-05_rb,6.4377e-05_rb,6.9227e-05_rb,7.4054e-05_rb,7.8692e-05_rb/)
      kbo(:,54, 4) = (/ &
     &4.8298e-05_rb,5.2166e-05_rb,5.6219e-05_rb,6.0246e-05_rb,6.4263e-05_rb/)
      kbo(:,55, 4) = (/ &
     &3.8831e-05_rb,4.2066e-05_rb,4.5431e-05_rb,4.8757e-05_rb,5.2064e-05_rb/)
      kbo(:,56, 4) = (/ &
     &3.1190e-05_rb,3.3925e-05_rb,3.6680e-05_rb,3.9437e-05_rb,4.2169e-05_rb/)
      kbo(:,57, 4) = (/ &
     &2.5026e-05_rb,2.7326e-05_rb,2.9588e-05_rb,3.1864e-05_rb,3.4154e-05_rb/)
      kbo(:,58, 4) = (/ &
     &2.0044e-05_rb,2.1985e-05_rb,2.3836e-05_rb,2.5731e-05_rb,2.7590e-05_rb/)
      kbo(:,59, 4) = (/ &
     &1.6187e-05_rb,1.7736e-05_rb,1.9214e-05_rb,2.0762e-05_rb,2.2244e-05_rb/)
      kbo(:,13, 5) = (/ &
     &1.6183e-01_rb,1.6388e-01_rb,1.6830e-01_rb,1.7476e-01_rb,1.8120e-01_rb/)
      kbo(:,14, 5) = (/ &
     &1.4136e-01_rb,1.4390e-01_rb,1.4614e-01_rb,1.5033e-01_rb,1.5561e-01_rb/)
      kbo(:,15, 5) = (/ &
     &1.2277e-01_rb,1.2594e-01_rb,1.2862e-01_rb,1.3140e-01_rb,1.3513e-01_rb/)
      kbo(:,16, 5) = (/ &
     &1.0716e-01_rb,1.1019e-01_rb,1.1295e-01_rb,1.1569e-01_rb,1.1878e-01_rb/)
      kbo(:,17, 5) = (/ &
     &9.2722e-02_rb,9.5517e-02_rb,9.8213e-02_rb,1.0105e-01_rb,1.0426e-01_rb/)
      kbo(:,18, 5) = (/ &
     &7.9838e-02_rb,8.2414e-02_rb,8.5010e-02_rb,8.8074e-02_rb,9.1598e-02_rb/)
      kbo(:,19, 5) = (/ &
     &6.8325e-02_rb,7.0740e-02_rb,7.3430e-02_rb,7.6659e-02_rb,8.0269e-02_rb/)
      kbo(:,20, 5) = (/ &
     &5.8275e-02_rb,6.0721e-02_rb,6.3630e-02_rb,6.6772e-02_rb,7.0098e-02_rb/)
      kbo(:,21, 5) = (/ &
     &4.9672e-02_rb,5.2190e-02_rb,5.4967e-02_rb,5.7853e-02_rb,6.0822e-02_rb/)
      kbo(:,22, 5) = (/ &
     &4.2534e-02_rb,4.4900e-02_rb,4.7398e-02_rb,4.9961e-02_rb,5.2416e-02_rb/)
      kbo(:,23, 5) = (/ &
     &3.6361e-02_rb,3.8477e-02_rb,4.0633e-02_rb,4.2770e-02_rb,4.4639e-02_rb/)
      kbo(:,24, 5) = (/ &
     &3.0984e-02_rb,3.2792e-02_rb,3.4594e-02_rb,3.6198e-02_rb,3.7541e-02_rb/)
      kbo(:,25, 5) = (/ &
     &2.6322e-02_rb,2.7818e-02_rb,2.9192e-02_rb,3.0375e-02_rb,3.1335e-02_rb/)
      kbo(:,26, 5) = (/ &
     &2.2285e-02_rb,2.3473e-02_rb,2.4485e-02_rb,2.5360e-02_rb,2.6084e-02_rb/)
      kbo(:,27, 5) = (/ &
     &1.8777e-02_rb,1.9670e-02_rb,2.0440e-02_rb,2.1104e-02_rb,2.1661e-02_rb/)
      kbo(:,28, 5) = (/ &
     &1.5739e-02_rb,1.6410e-02_rb,1.7009e-02_rb,1.7502e-02_rb,1.7957e-02_rb/)
      kbo(:,29, 5) = (/ &
     &1.3157e-02_rb,1.3689e-02_rb,1.4135e-02_rb,1.4550e-02_rb,1.4908e-02_rb/)
      kbo(:,30, 5) = (/ &
     &1.0985e-02_rb,1.1390e-02_rb,1.1749e-02_rb,1.2076e-02_rb,1.2372e-02_rb/)
      kbo(:,31, 5) = (/ &
     &9.1582e-03_rb,9.4742e-03_rb,9.7675e-03_rb,1.0044e-02_rb,1.0289e-02_rb/)
      kbo(:,32, 5) = (/ &
     &7.6339e-03_rb,7.8923e-03_rb,8.1271e-03_rb,8.3548e-03_rb,8.5695e-03_rb/)
      kbo(:,33, 5) = (/ &
     &6.3715e-03_rb,6.5845e-03_rb,6.7878e-03_rb,6.9736e-03_rb,7.1535e-03_rb/)
      kbo(:,34, 5) = (/ &
     &5.3224e-03_rb,5.5025e-03_rb,5.6652e-03_rb,5.8262e-03_rb,5.9749e-03_rb/)
      kbo(:,35, 5) = (/ &
     &4.4348e-03_rb,4.5841e-03_rb,4.7248e-03_rb,4.8589e-03_rb,4.9851e-03_rb/)
      kbo(:,36, 5) = (/ &
     &3.6880e-03_rb,3.8127e-03_rb,3.9338e-03_rb,4.0484e-03_rb,4.1576e-03_rb/)
      kbo(:,37, 5) = (/ &
     &3.0534e-03_rb,3.1615e-03_rb,3.2693e-03_rb,3.3693e-03_rb,3.4657e-03_rb/)
      kbo(:,38, 5) = (/ &
     &2.5332e-03_rb,2.6300e-03_rb,2.7225e-03_rb,2.8131e-03_rb,2.8939e-03_rb/)
      kbo(:,39, 5) = (/ &
     &2.1084e-03_rb,2.1961e-03_rb,2.2783e-03_rb,2.3583e-03_rb,2.4286e-03_rb/)
      kbo(:,40, 5) = (/ &
     &1.7458e-03_rb,1.8240e-03_rb,1.8981e-03_rb,1.9682e-03_rb,2.0283e-03_rb/)
      kbo(:,41, 5) = (/ &
     &1.4453e-03_rb,1.5145e-03_rb,1.5781e-03_rb,1.6391e-03_rb,1.6964e-03_rb/)
      kbo(:,42, 5) = (/ &
     &1.1954e-03_rb,1.2559e-03_rb,1.3113e-03_rb,1.3662e-03_rb,1.4193e-03_rb/)
      kbo(:,43, 5) = (/ &
     &9.8711e-04_rb,1.0388e-03_rb,1.0881e-03_rb,1.1376e-03_rb,1.1854e-03_rb/)
      kbo(:,44, 5) = (/ &
     &8.1670e-04_rb,8.6083e-04_rb,9.0273e-04_rb,9.4724e-04_rb,9.9183e-04_rb/)
      kbo(:,45, 5) = (/ &
     &6.7571e-04_rb,7.1462e-04_rb,7.5340e-04_rb,7.9241e-04_rb,8.3252e-04_rb/)
      kbo(:,46, 5) = (/ &
     &5.5868e-04_rb,5.9210e-04_rb,6.2784e-04_rb,6.6315e-04_rb,6.9901e-04_rb/)
      kbo(:,47, 5) = (/ &
     &4.6002e-04_rb,4.8924e-04_rb,5.2084e-04_rb,5.5250e-04_rb,5.8555e-04_rb/)
      kbo(:,48, 5) = (/ &
     &3.7900e-04_rb,4.0478e-04_rb,4.3312e-04_rb,4.6138e-04_rb,4.9219e-04_rb/)
      kbo(:,49, 5) = (/ &
     &3.1223e-04_rb,3.3515e-04_rb,3.6106e-04_rb,3.8737e-04_rb,4.1475e-04_rb/)
      kbo(:,50, 5) = (/ &
     &2.5631e-04_rb,2.7625e-04_rb,2.9941e-04_rb,3.2338e-04_rb,3.4759e-04_rb/)
      kbo(:,51, 5) = (/ &
     &2.0956e-04_rb,2.2726e-04_rb,2.4758e-04_rb,2.6894e-04_rb,2.9020e-04_rb/)
      kbo(:,52, 5) = (/ &
     &1.7113e-04_rb,1.8689e-04_rb,2.0470e-04_rb,2.2361e-04_rb,2.4260e-04_rb/)
      kbo(:,53, 5) = (/ &
     &1.3975e-04_rb,1.5381e-04_rb,1.6939e-04_rb,1.8611e-04_rb,2.0352e-04_rb/)
      kbo(:,54, 5) = (/ &
     &1.1323e-04_rb,1.2519e-04_rb,1.3824e-04_rb,1.5269e-04_rb,1.6768e-04_rb/)
      kbo(:,55, 5) = (/ &
     &9.1211e-05_rb,1.0111e-04_rb,1.1195e-04_rb,1.2407e-04_rb,1.3663e-04_rb/)
      kbo(:,56, 5) = (/ &
     &7.3375e-05_rb,8.1461e-05_rb,9.0533e-05_rb,1.0060e-04_rb,1.1119e-04_rb/)
      kbo(:,57, 5) = (/ &
     &5.8946e-05_rb,6.5535e-05_rb,7.3109e-05_rb,8.1511e-05_rb,9.0294e-05_rb/)
      kbo(:,58, 5) = (/ &
     &4.7323e-05_rb,5.2682e-05_rb,5.8966e-05_rb,6.5820e-05_rb,7.3137e-05_rb/)
      kbo(:,59, 5) = (/ &
     &3.8250e-05_rb,4.2565e-05_rb,4.7446e-05_rb,5.2866e-05_rb,5.8710e-05_rb/)
      kbo(:,13, 6) = (/ &
     &2.7197e-01_rb,2.7739e-01_rb,2.8096e-01_rb,2.8231e-01_rb,2.8660e-01_rb/)
      kbo(:,14, 6) = (/ &
     &2.4407e-01_rb,2.4904e-01_rb,2.5343e-01_rb,2.5569e-01_rb,2.5783e-01_rb/)
      kbo(:,15, 6) = (/ &
     &2.1616e-01_rb,2.2099e-01_rb,2.2537e-01_rb,2.2864e-01_rb,2.3140e-01_rb/)
      kbo(:,16, 6) = (/ &
     &1.9106e-01_rb,1.9585e-01_rb,1.9991e-01_rb,2.0367e-01_rb,2.0768e-01_rb/)
      kbo(:,17, 6) = (/ &
     &1.6891e-01_rb,1.7330e-01_rb,1.7742e-01_rb,1.8116e-01_rb,1.8548e-01_rb/)
      kbo(:,18, 6) = (/ &
     &1.4873e-01_rb,1.5292e-01_rb,1.5684e-01_rb,1.6048e-01_rb,1.6482e-01_rb/)
      kbo(:,19, 6) = (/ &
     &1.3023e-01_rb,1.3403e-01_rb,1.3773e-01_rb,1.4155e-01_rb,1.4625e-01_rb/)
      kbo(:,20, 6) = (/ &
     &1.1343e-01_rb,1.1684e-01_rb,1.2043e-01_rb,1.2478e-01_rb,1.2990e-01_rb/)
      kbo(:,21, 6) = (/ &
     &9.8160e-02_rb,1.0135e-01_rb,1.0525e-01_rb,1.1007e-01_rb,1.1526e-01_rb/)
      kbo(:,22, 6) = (/ &
     &8.4655e-02_rb,8.8120e-02_rb,9.2429e-02_rb,9.7184e-02_rb,1.0215e-01_rb/)
      kbo(:,23, 6) = (/ &
     &7.3019e-02_rb,7.6775e-02_rb,8.1076e-02_rb,8.5515e-02_rb,9.0104e-02_rb/)
      kbo(:,24, 6) = (/ &
     &6.3162e-02_rb,6.6861e-02_rb,7.0760e-02_rb,7.4872e-02_rb,7.8956e-02_rb/)
      kbo(:,25, 6) = (/ &
     &5.4670e-02_rb,5.8059e-02_rb,6.1565e-02_rb,6.5142e-02_rb,6.8514e-02_rb/)
      kbo(:,26, 6) = (/ &
     &4.7238e-02_rb,5.0243e-02_rb,5.3330e-02_rb,5.6237e-02_rb,5.8783e-02_rb/)
      kbo(:,27, 6) = (/ &
     &4.0718e-02_rb,4.3335e-02_rb,4.5897e-02_rb,4.8107e-02_rb,4.9970e-02_rb/)
      kbo(:,28, 6) = (/ &
     &3.4992e-02_rb,3.7210e-02_rb,3.9178e-02_rb,4.0837e-02_rb,4.2251e-02_rb/)
      kbo(:,29, 6) = (/ &
     &2.9986e-02_rb,3.1757e-02_rb,3.3246e-02_rb,3.4515e-02_rb,3.5697e-02_rb/)
      kbo(:,30, 6) = (/ &
     &2.5598e-02_rb,2.6947e-02_rb,2.8105e-02_rb,2.9142e-02_rb,3.0127e-02_rb/)
      kbo(:,31, 6) = (/ &
     &2.1742e-02_rb,2.2800e-02_rb,2.3733e-02_rb,2.4625e-02_rb,2.5488e-02_rb/)
      kbo(:,32, 6) = (/ &
     &1.8409e-02_rb,1.9261e-02_rb,2.0062e-02_rb,2.0837e-02_rb,2.1556e-02_rb/)
      kbo(:,33, 6) = (/ &
     &1.5564e-02_rb,1.6264e-02_rb,1.6973e-02_rb,1.7637e-02_rb,1.8281e-02_rb/)
      kbo(:,34, 6) = (/ &
     &1.3120e-02_rb,1.3750e-02_rb,1.4364e-02_rb,1.4950e-02_rb,1.5574e-02_rb/)
      kbo(:,35, 6) = (/ &
     &1.1037e-02_rb,1.1597e-02_rb,1.2120e-02_rb,1.2671e-02_rb,1.3247e-02_rb/)
      kbo(:,36, 6) = (/ &
     &9.2368e-03_rb,9.7140e-03_rb,1.0180e-02_rb,1.0677e-02_rb,1.1200e-02_rb/)
      kbo(:,37, 6) = (/ &
     &7.7240e-03_rb,8.1368e-03_rb,8.5470e-03_rb,8.9958e-03_rb,9.4504e-03_rb/)
      kbo(:,38, 6) = (/ &
     &6.4758e-03_rb,6.8295e-03_rb,7.1958e-03_rb,7.5972e-03_rb,8.0066e-03_rb/)
      kbo(:,39, 6) = (/ &
     &5.4501e-03_rb,5.7576e-03_rb,6.0862e-03_rb,6.4345e-03_rb,6.8065e-03_rb/)
      kbo(:,40, 6) = (/ &
     &4.5611e-03_rb,4.8311e-03_rb,5.1160e-03_rb,5.4160e-03_rb,5.7404e-03_rb/)
      kbo(:,41, 6) = (/ &
     &3.8156e-03_rb,4.0605e-03_rb,4.3157e-03_rb,4.5812e-03_rb,4.8631e-03_rb/)
      kbo(:,42, 6) = (/ &
     &3.1991e-03_rb,3.4174e-03_rb,3.6479e-03_rb,3.8959e-03_rb,4.1438e-03_rb/)
      kbo(:,43, 6) = (/ &
     &2.6778e-03_rb,2.8754e-03_rb,3.0830e-03_rb,3.3106e-03_rb,3.5418e-03_rb/)
      kbo(:,44, 6) = (/ &
     &2.2384e-03_rb,2.4221e-03_rb,2.6165e-03_rb,2.8279e-03_rb,3.0365e-03_rb/)
      kbo(:,45, 6) = (/ &
     &1.8762e-03_rb,2.0454e-03_rb,2.2217e-03_rb,2.4182e-03_rb,2.6139e-03_rb/)
      kbo(:,46, 6) = (/ &
     &1.5669e-03_rb,1.7239e-03_rb,1.8868e-03_rb,2.0694e-03_rb,2.2548e-03_rb/)
      kbo(:,47, 6) = (/ &
     &1.2965e-03_rb,1.4378e-03_rb,1.5876e-03_rb,1.7568e-03_rb,1.9326e-03_rb/)
      kbo(:,48, 6) = (/ &
     &1.0774e-03_rb,1.2001e-03_rb,1.3389e-03_rb,1.4948e-03_rb,1.6630e-03_rb/)
      kbo(:,49, 6) = (/ &
     &8.9852e-04_rb,1.0105e-03_rb,1.1309e-03_rb,1.2754e-03_rb,1.4391e-03_rb/)
      kbo(:,50, 6) = (/ &
     &7.4445e-04_rb,8.4643e-04_rb,9.5631e-04_rb,1.0809e-03_rb,1.2318e-03_rb/)
      kbo(:,51, 6) = (/ &
     &6.1558e-04_rb,7.0452e-04_rb,8.0429e-04_rb,9.1653e-04_rb,1.0518e-03_rb/)
      kbo(:,52, 6) = (/ &
     &5.0970e-04_rb,5.8893e-04_rb,6.7706e-04_rb,7.7959e-04_rb,9.0341e-04_rb/)
      kbo(:,53, 6) = (/ &
     &4.2152e-04_rb,4.9282e-04_rb,5.7291e-04_rb,6.6638e-04_rb,7.8149e-04_rb/)
      kbo(:,54, 6) = (/ &
     &3.4337e-04_rb,4.0415e-04_rb,4.7438e-04_rb,5.5660e-04_rb,6.5618e-04_rb/)
      kbo(:,55, 6) = (/ &
     &2.7643e-04_rb,3.2706e-04_rb,3.8585e-04_rb,4.5635e-04_rb,5.3989e-04_rb/)
      kbo(:,56, 6) = (/ &
     &2.2200e-04_rb,2.6388e-04_rb,3.1337e-04_rb,3.7328e-04_rb,4.4370e-04_rb/)
      kbo(:,57, 6) = (/ &
     &1.7748e-04_rb,2.1244e-04_rb,2.5395e-04_rb,3.0395e-04_rb,3.6371e-04_rb/)
      kbo(:,58, 6) = (/ &
     &1.4157e-04_rb,1.7062e-04_rb,2.0500e-04_rb,2.4610e-04_rb,2.9686e-04_rb/)
      kbo(:,59, 6) = (/ &
     &1.1414e-04_rb,1.3738e-04_rb,1.6469e-04_rb,1.9701e-04_rb,2.3735e-04_rb/)
      kbo(:,13, 7) = (/ &
     &4.6955e-01_rb,4.7919e-01_rb,4.8757e-01_rb,4.9335e-01_rb,4.9501e-01_rb/)
      kbo(:,14, 7) = (/ &
     &4.3446e-01_rb,4.4412e-01_rb,4.5217e-01_rb,4.5833e-01_rb,4.6211e-01_rb/)
      kbo(:,15, 7) = (/ &
     &3.9856e-01_rb,4.0695e-01_rb,4.1403e-01_rb,4.2050e-01_rb,4.2565e-01_rb/)
      kbo(:,16, 7) = (/ &
     &3.6468e-01_rb,3.7159e-01_rb,3.7817e-01_rb,3.8407e-01_rb,3.8859e-01_rb/)
      kbo(:,17, 7) = (/ &
     &3.3137e-01_rb,3.3763e-01_rb,3.4358e-01_rb,3.4927e-01_rb,3.5406e-01_rb/)
      kbo(:,18, 7) = (/ &
     &2.9961e-01_rb,3.0527e-01_rb,3.1117e-01_rb,3.1677e-01_rb,3.2160e-01_rb/)
      kbo(:,19, 7) = (/ &
     &2.6897e-01_rb,2.7470e-01_rb,2.8068e-01_rb,2.8609e-01_rb,2.9127e-01_rb/)
      kbo(:,20, 7) = (/ &
     &2.4079e-01_rb,2.4662e-01_rb,2.5228e-01_rb,2.5749e-01_rb,2.6294e-01_rb/)
      kbo(:,21, 7) = (/ &
     &2.1419e-01_rb,2.1985e-01_rb,2.2509e-01_rb,2.3029e-01_rb,2.3688e-01_rb/)
      kbo(:,22, 7) = (/ &
     &1.8948e-01_rb,1.9472e-01_rb,1.9983e-01_rb,2.0584e-01_rb,2.1394e-01_rb/)
      kbo(:,23, 7) = (/ &
     &1.6639e-01_rb,1.7145e-01_rb,1.7716e-01_rb,1.8457e-01_rb,1.9389e-01_rb/)
      kbo(:,24, 7) = (/ &
     &1.4538e-01_rb,1.5085e-01_rb,1.5765e-01_rb,1.6633e-01_rb,1.7588e-01_rb/)
      kbo(:,25, 7) = (/ &
     &1.2705e-01_rb,1.3331e-01_rb,1.4121e-01_rb,1.5025e-01_rb,1.5987e-01_rb/)
      kbo(:,26, 7) = (/ &
     &1.1163e-01_rb,1.1872e-01_rb,1.2704e-01_rb,1.3598e-01_rb,1.4549e-01_rb/)
      kbo(:,27, 7) = (/ &
     &9.8775e-02_rb,1.0620e-01_rb,1.1436e-01_rb,1.2321e-01_rb,1.3236e-01_rb/)
      kbo(:,28, 7) = (/ &
     &8.7819e-02_rb,9.5125e-02_rb,1.0304e-01_rb,1.1161e-01_rb,1.2007e-01_rb/)
      kbo(:,29, 7) = (/ &
     &7.8356e-02_rb,8.5376e-02_rb,9.3117e-02_rb,1.0101e-01_rb,1.0879e-01_rb/)
      kbo(:,30, 7) = (/ &
     &7.0020e-02_rb,7.6849e-02_rb,8.4078e-02_rb,9.1302e-02_rb,9.8416e-02_rb/)
      kbo(:,31, 7) = (/ &
     &6.2819e-02_rb,6.9270e-02_rb,7.5879e-02_rb,8.2483e-02_rb,8.8967e-02_rb/)
      kbo(:,32, 7) = (/ &
     &5.6483e-02_rb,6.2430e-02_rb,6.8481e-02_rb,7.4539e-02_rb,8.0648e-02_rb/)
      kbo(:,33, 7) = (/ &
     &5.0860e-02_rb,5.6316e-02_rb,6.1885e-02_rb,6.7612e-02_rb,7.3808e-02_rb/)
      kbo(:,34, 7) = (/ &
     &4.5682e-02_rb,5.0745e-02_rb,5.5935e-02_rb,6.1551e-02_rb,6.7601e-02_rb/)
      kbo(:,35, 7) = (/ &
     &4.0692e-02_rb,4.5316e-02_rb,5.0264e-02_rb,5.5834e-02_rb,6.1807e-02_rb/)
      kbo(:,36, 7) = (/ &
     &3.5794e-02_rb,4.0089e-02_rb,4.4914e-02_rb,5.0304e-02_rb,5.6176e-02_rb/)
      kbo(:,37, 7) = (/ &
     &3.0961e-02_rb,3.4978e-02_rb,3.9573e-02_rb,4.4775e-02_rb,5.0561e-02_rb/)
      kbo(:,38, 7) = (/ &
     &2.6776e-02_rb,3.0559e-02_rb,3.4968e-02_rb,3.9997e-02_rb,4.5685e-02_rb/)
      kbo(:,39, 7) = (/ &
     &2.3185e-02_rb,2.6765e-02_rb,3.1022e-02_rb,3.5959e-02_rb,4.1519e-02_rb/)
      kbo(:,40, 7) = (/ &
     &1.9763e-02_rb,2.3065e-02_rb,2.7085e-02_rb,3.1775e-02_rb,3.7215e-02_rb/)
      kbo(:,41, 7) = (/ &
     &1.6812e-02_rb,1.9830e-02_rb,2.3604e-02_rb,2.8085e-02_rb,3.3399e-02_rb/)
      kbo(:,42, 7) = (/ &
     &1.4318e-02_rb,1.7069e-02_rb,2.0605e-02_rb,2.4882e-02_rb,3.0046e-02_rb/)
      kbo(:,43, 7) = (/ &
     &1.2083e-02_rb,1.4567e-02_rb,1.7802e-02_rb,2.1853e-02_rb,2.6807e-02_rb/)
      kbo(:,44, 7) = (/ &
     &1.0170e-02_rb,1.2372e-02_rb,1.5314e-02_rb,1.9108e-02_rb,2.3821e-02_rb/)
      kbo(:,45, 7) = (/ &
     &8.6090e-03_rb,1.0534e-02_rb,1.3191e-02_rb,1.6726e-02_rb,2.1197e-02_rb/)
      kbo(:,46, 7) = (/ &
     &7.3022e-03_rb,8.9907e-03_rb,1.1353e-02_rb,1.4581e-02_rb,1.8771e-02_rb/)
      kbo(:,47, 7) = (/ &
     &6.1213e-03_rb,7.6431e-03_rb,9.7321e-03_rb,1.2583e-02_rb,1.6455e-02_rb/)
      kbo(:,48, 7) = (/ &
     &5.1186e-03_rb,6.5188e-03_rb,8.4138e-03_rb,1.0987e-02_rb,1.4502e-02_rb/)
      kbo(:,49, 7) = (/ &
     &4.2957e-03_rb,5.5597e-03_rb,7.3442e-03_rb,9.7283e-03_rb,1.2936e-02_rb/)
      kbo(:,50, 7) = (/ &
     &3.6145e-03_rb,4.7398e-03_rb,6.3555e-03_rb,8.6287e-03_rb,1.1590e-02_rb/)
      kbo(:,51, 7) = (/ &
     &3.0102e-03_rb,4.0430e-03_rb,5.4860e-03_rb,7.5896e-03_rb,1.0447e-02_rb/)
      kbo(:,52, 7) = (/ &
     &2.4958e-03_rb,3.4406e-03_rb,4.7896e-03_rb,6.7274e-03_rb,9.4519e-03_rb/)
      kbo(:,53, 7) = (/ &
     &2.0685e-03_rb,2.9229e-03_rb,4.2030e-03_rb,6.0566e-03_rb,8.6137e-03_rb/)
      kbo(:,54, 7) = (/ &
     &1.6899e-03_rb,2.4174e-03_rb,3.5607e-03_rb,5.2595e-03_rb,7.6384e-03_rb/)
      kbo(:,55, 7) = (/ &
     &1.3559e-03_rb,1.9570e-03_rb,2.9412e-03_rb,4.4253e-03_rb,6.5827e-03_rb/)
      kbo(:,56, 7) = (/ &
     &1.0806e-03_rb,1.5745e-03_rb,2.4094e-03_rb,3.7108e-03_rb,5.6277e-03_rb/)
      kbo(:,57, 7) = (/ &
     &8.5457e-04_rb,1.2632e-03_rb,1.9536e-03_rb,3.0874e-03_rb,4.7870e-03_rb/)
      kbo(:,58, 7) = (/ &
     &6.7302e-04_rb,1.0065e-03_rb,1.5736e-03_rb,2.5427e-03_rb,4.0334e-03_rb/)
      kbo(:,59, 7) = (/ &
     &5.4147e-04_rb,8.1396e-04_rb,1.2891e-03_rb,2.1042e-03_rb,3.3560e-03_rb/)
      kbo(:,13, 8) = (/ &
     &8.0034e-01_rb,8.1640e-01_rb,8.3288e-01_rb,8.4808e-01_rb,8.6148e-01_rb/)
      kbo(:,14, 8) = (/ &
     &7.8054e-01_rb,7.9433e-01_rb,8.0919e-01_rb,8.2411e-01_rb,8.3905e-01_rb/)
      kbo(:,15, 8) = (/ &
     &7.5909e-01_rb,7.7248e-01_rb,7.8592e-01_rb,7.9872e-01_rb,8.1170e-01_rb/)
      kbo(:,16, 8) = (/ &
     &7.3580e-01_rb,7.4804e-01_rb,7.6085e-01_rb,7.7248e-01_rb,7.8295e-01_rb/)
      kbo(:,17, 8) = (/ &
     &7.0630e-01_rb,7.1844e-01_rb,7.2965e-01_rb,7.3912e-01_rb,7.4743e-01_rb/)
      kbo(:,18, 8) = (/ &
     &6.7101e-01_rb,6.8279e-01_rb,6.9322e-01_rb,7.0102e-01_rb,7.0887e-01_rb/)
      kbo(:,19, 8) = (/ &
     &6.2981e-01_rb,6.4055e-01_rb,6.4979e-01_rb,6.5773e-01_rb,6.6515e-01_rb/)
      kbo(:,20, 8) = (/ &
     &5.8544e-01_rb,5.9608e-01_rb,6.0511e-01_rb,6.1342e-01_rb,6.2187e-01_rb/)
      kbo(:,21, 8) = (/ &
     &5.3944e-01_rb,5.5063e-01_rb,5.6005e-01_rb,5.6946e-01_rb,5.7856e-01_rb/)
      kbo(:,22, 8) = (/ &
     &4.9436e-01_rb,5.0585e-01_rb,5.1644e-01_rb,5.2703e-01_rb,5.3664e-01_rb/)
      kbo(:,23, 8) = (/ &
     &4.5102e-01_rb,4.6300e-01_rb,4.7455e-01_rb,4.8579e-01_rb,4.9663e-01_rb/)
      kbo(:,24, 8) = (/ &
     &4.0950e-01_rb,4.2198e-01_rb,4.3426e-01_rb,4.4606e-01_rb,4.5946e-01_rb/)
      kbo(:,25, 8) = (/ &
     &3.7091e-01_rb,3.8365e-01_rb,3.9643e-01_rb,4.1000e-01_rb,4.2576e-01_rb/)
      kbo(:,26, 8) = (/ &
     &3.3534e-01_rb,3.4839e-01_rb,3.6241e-01_rb,3.7838e-01_rb,3.9777e-01_rb/)
      kbo(:,27, 8) = (/ &
     &3.0274e-01_rb,3.1698e-01_rb,3.3298e-01_rb,3.5138e-01_rb,3.7450e-01_rb/)
      kbo(:,28, 8) = (/ &
     &2.7401e-01_rb,2.8968e-01_rb,3.0792e-01_rb,3.3011e-01_rb,3.5579e-01_rb/)
      kbo(:,29, 8) = (/ &
     &2.4984e-01_rb,2.6749e-01_rb,2.8805e-01_rb,3.1339e-01_rb,3.4148e-01_rb/)
      kbo(:,30, 8) = (/ &
     &2.2976e-01_rb,2.4923e-01_rb,2.7317e-01_rb,3.0058e-01_rb,3.3054e-01_rb/)
      kbo(:,31, 8) = (/ &
     &2.1378e-01_rb,2.3592e-01_rb,2.6234e-01_rb,2.9177e-01_rb,3.2355e-01_rb/)
      kbo(:,32, 8) = (/ &
     &2.0184e-01_rb,2.2666e-01_rb,2.5501e-01_rb,2.8631e-01_rb,3.1911e-01_rb/)
      kbo(:,33, 8) = (/ &
     &1.9381e-01_rb,2.2050e-01_rb,2.5091e-01_rb,2.8341e-01_rb,3.1739e-01_rb/)
      kbo(:,34, 8) = (/ &
     &1.8744e-01_rb,2.1594e-01_rb,2.4774e-01_rb,2.8137e-01_rb,3.1632e-01_rb/)
      kbo(:,35, 8) = (/ &
     &1.8056e-01_rb,2.1069e-01_rb,2.4353e-01_rb,2.7798e-01_rb,3.1405e-01_rb/)
      kbo(:,36, 8) = (/ &
     &1.7277e-01_rb,2.0381e-01_rb,2.3716e-01_rb,2.7253e-01_rb,3.0956e-01_rb/)
      kbo(:,37, 8) = (/ &
     &1.6168e-01_rb,1.9289e-01_rb,2.2650e-01_rb,2.6234e-01_rb,2.9987e-01_rb/)
      kbo(:,38, 8) = (/ &
     &1.5148e-01_rb,1.8272e-01_rb,2.1663e-01_rb,2.5277e-01_rb,2.9084e-01_rb/)
      kbo(:,39, 8) = (/ &
     &1.4247e-01_rb,1.7365e-01_rb,2.0770e-01_rb,2.4421e-01_rb,2.8306e-01_rb/)
      kbo(:,40, 8) = (/ &
     &1.3121e-01_rb,1.6190e-01_rb,1.9569e-01_rb,2.3227e-01_rb,2.7141e-01_rb/)
      kbo(:,41, 8) = (/ &
     &1.2044e-01_rb,1.5044e-01_rb,1.8388e-01_rb,2.2048e-01_rb,2.5951e-01_rb/)
      kbo(:,42, 8) = (/ &
     &1.1043e-01_rb,1.3967e-01_rb,1.7269e-01_rb,2.0919e-01_rb,2.4811e-01_rb/)
      kbo(:,43, 8) = (/ &
     &9.9679e-02_rb,1.2792e-01_rb,1.6034e-01_rb,1.9622e-01_rb,2.3472e-01_rb/)
      kbo(:,44, 8) = (/ &
     &8.9034e-02_rb,1.1631e-01_rb,1.4764e-01_rb,1.8285e-01_rb,2.2093e-01_rb/)
      kbo(:,45, 8) = (/ &
     &7.9054e-02_rb,1.0536e-01_rb,1.3566e-01_rb,1.6992e-01_rb,2.0745e-01_rb/)
      kbo(:,46, 8) = (/ &
     &6.9318e-02_rb,9.4501e-02_rb,1.2377e-01_rb,1.5667e-01_rb,1.9349e-01_rb/)
      kbo(:,47, 8) = (/ &
     &5.9551e-02_rb,8.3032e-02_rb,1.1103e-01_rb,1.4286e-01_rb,1.7816e-01_rb/)
      kbo(:,48, 8) = (/ &
     &5.0798e-02_rb,7.2525e-02_rb,9.9035e-02_rb,1.2969e-01_rb,1.6375e-01_rb/)
      kbo(:,49, 8) = (/ &
     &4.3152e-02_rb,6.3003e-02_rb,8.7833e-02_rb,1.1716e-01_rb,1.5024e-01_rb/)
      kbo(:,50, 8) = (/ &
     &3.6793e-02_rb,5.4813e-02_rb,7.7929e-02_rb,1.0576e-01_rb,1.3787e-01_rb/)
      kbo(:,51, 8) = (/ &
     &3.1426e-02_rb,4.7509e-02_rb,6.8953e-02_rb,9.5149e-02_rb,1.2582e-01_rb/)
      kbo(:,52, 8) = (/ &
     &2.6907e-02_rb,4.1306e-02_rb,6.0841e-02_rb,8.5477e-02_rb,1.1483e-01_rb/)
      kbo(:,53, 8) = (/ &
     &2.3168e-02_rb,3.6049e-02_rb,5.3816e-02_rb,7.6669e-02_rb,1.0462e-01_rb/)
      kbo(:,54, 8) = (/ &
     &1.9633e-02_rb,3.1217e-02_rb,4.7423e-02_rb,6.8592e-02_rb,9.4967e-02_rb/)
      kbo(:,55, 8) = (/ &
     &1.6386e-02_rb,2.6769e-02_rb,4.1346e-02_rb,6.0965e-02_rb,8.5476e-02_rb/)
      kbo(:,56, 8) = (/ &
     &1.3620e-02_rb,2.2714e-02_rb,3.5826e-02_rb,5.3847e-02_rb,7.6755e-02_rb/)
      kbo(:,57, 8) = (/ &
     &1.1197e-02_rb,1.9124e-02_rb,3.0904e-02_rb,4.7282e-02_rb,6.8602e-02_rb/)
      kbo(:,58, 8) = (/ &
     &9.1481e-03_rb,1.6046e-02_rb,2.6605e-02_rb,4.1371e-02_rb,6.1207e-02_rb/)
      kbo(:,59, 8) = (/ &
     &7.9051e-03_rb,1.4193e-02_rb,2.3774e-02_rb,3.7742e-02_rb,5.6658e-02_rb/)
      kbo(:,13, 9) = (/ &
     &1.3350e+00_rb,1.3653e+00_rb,1.3981e+00_rb,1.4396e+00_rb,1.4753e+00_rb/)
      kbo(:,14, 9) = (/ &
     &1.4178e+00_rb,1.4520e+00_rb,1.4843e+00_rb,1.5206e+00_rb,1.5583e+00_rb/)
      kbo(:,15, 9) = (/ &
     &1.4732e+00_rb,1.5083e+00_rb,1.5396e+00_rb,1.5755e+00_rb,1.6045e+00_rb/)
      kbo(:,16, 9) = (/ &
     &1.5430e+00_rb,1.5830e+00_rb,1.6160e+00_rb,1.6547e+00_rb,1.6902e+00_rb/)
      kbo(:,17, 9) = (/ &
     &1.6452e+00_rb,1.6845e+00_rb,1.7176e+00_rb,1.7564e+00_rb,1.7915e+00_rb/)
      kbo(:,18, 9) = (/ &
     &1.7229e+00_rb,1.7605e+00_rb,1.7922e+00_rb,1.8300e+00_rb,1.8621e+00_rb/)
      kbo(:,19, 9) = (/ &
     &1.7775e+00_rb,1.8140e+00_rb,1.8454e+00_rb,1.8798e+00_rb,1.9094e+00_rb/)
      kbo(:,20, 9) = (/ &
     &1.8038e+00_rb,1.8386e+00_rb,1.8699e+00_rb,1.9034e+00_rb,1.9311e+00_rb/)
      kbo(:,21, 9) = (/ &
     &1.8050e+00_rb,1.8386e+00_rb,1.8712e+00_rb,1.9046e+00_rb,1.9320e+00_rb/)
      kbo(:,22, 9) = (/ &
     &1.7869e+00_rb,1.8213e+00_rb,1.8557e+00_rb,1.8890e+00_rb,1.9178e+00_rb/)
      kbo(:,23, 9) = (/ &
     &1.7516e+00_rb,1.7874e+00_rb,1.8248e+00_rb,1.8592e+00_rb,1.8901e+00_rb/)
      kbo(:,24, 9) = (/ &
     &1.7051e+00_rb,1.7440e+00_rb,1.7841e+00_rb,1.8220e+00_rb,1.8561e+00_rb/)
      kbo(:,25, 9) = (/ &
     &1.6513e+00_rb,1.6948e+00_rb,1.7396e+00_rb,1.7805e+00_rb,1.8194e+00_rb/)
      kbo(:,26, 9) = (/ &
     &1.5955e+00_rb,1.6454e+00_rb,1.6945e+00_rb,1.7402e+00_rb,1.7819e+00_rb/)
      kbo(:,27, 9) = (/ &
     &1.5421e+00_rb,1.5985e+00_rb,1.6517e+00_rb,1.7028e+00_rb,1.7485e+00_rb/)
      kbo(:,28, 9) = (/ &
     &1.4940e+00_rb,1.5568e+00_rb,1.6151e+00_rb,1.6694e+00_rb,1.7213e+00_rb/)
      kbo(:,29, 9) = (/ &
     &1.4533e+00_rb,1.5206e+00_rb,1.5846e+00_rb,1.6444e+00_rb,1.7003e+00_rb/)
      kbo(:,30, 9) = (/ &
     &1.4208e+00_rb,1.4930e+00_rb,1.5618e+00_rb,1.6263e+00_rb,1.6869e+00_rb/)
      kbo(:,31, 9) = (/ &
     &1.3961e+00_rb,1.4744e+00_rb,1.5474e+00_rb,1.6161e+00_rb,1.6805e+00_rb/)
      kbo(:,32, 9) = (/ &
     &1.3821e+00_rb,1.4633e+00_rb,1.5401e+00_rb,1.6122e+00_rb,1.6817e+00_rb/)
      kbo(:,33, 9) = (/ &
     &1.3749e+00_rb,1.4598e+00_rb,1.5393e+00_rb,1.6160e+00_rb,1.6953e+00_rb/)
      kbo(:,34, 9) = (/ &
     &1.3713e+00_rb,1.4588e+00_rb,1.5429e+00_rb,1.6218e+00_rb,1.7156e+00_rb/)
      kbo(:,35, 9) = (/ &
     &1.3644e+00_rb,1.4561e+00_rb,1.5422e+00_rb,1.6338e+00_rb,1.7288e+00_rb/)
      kbo(:,36, 9) = (/ &
     &1.3516e+00_rb,1.4466e+00_rb,1.5348e+00_rb,1.6360e+00_rb,1.7340e+00_rb/)
      kbo(:,37, 9) = (/ &
     &1.3275e+00_rb,1.4250e+00_rb,1.5169e+00_rb,1.6237e+00_rb,1.7238e+00_rb/)
      kbo(:,38, 9) = (/ &
     &1.3040e+00_rb,1.4043e+00_rb,1.5021e+00_rb,1.6114e+00_rb,1.7135e+00_rb/)
      kbo(:,39, 9) = (/ &
     &1.2823e+00_rb,1.3849e+00_rb,1.4910e+00_rb,1.6008e+00_rb,1.7062e+00_rb/)
      kbo(:,40, 9) = (/ &
     &1.2494e+00_rb,1.3552e+00_rb,1.4636e+00_rb,1.5759e+00_rb,1.6836e+00_rb/)
      kbo(:,41, 9) = (/ &
     &1.2158e+00_rb,1.3244e+00_rb,1.4340e+00_rb,1.5489e+00_rb,1.6593e+00_rb/)
      kbo(:,42, 9) = (/ &
     &1.1826e+00_rb,1.2939e+00_rb,1.4045e+00_rb,1.5221e+00_rb,1.6360e+00_rb/)
      kbo(:,43, 9) = (/ &
     &1.1430e+00_rb,1.2570e+00_rb,1.3680e+00_rb,1.4889e+00_rb,1.6061e+00_rb/)
      kbo(:,44, 9) = (/ &
     &1.1001e+00_rb,1.2164e+00_rb,1.3282e+00_rb,1.4525e+00_rb,1.5734e+00_rb/)
      kbo(:,45, 9) = (/ &
     &1.0569e+00_rb,1.1761e+00_rb,1.2894e+00_rb,1.4161e+00_rb,1.5408e+00_rb/)
      kbo(:,46, 9) = (/ &
     &1.0113e+00_rb,1.1324e+00_rb,1.2480e+00_rb,1.3769e+00_rb,1.5047e+00_rb/)
      kbo(:,47, 9) = (/ &
     &9.5935e-01_rb,1.0831e+00_rb,1.2009e+00_rb,1.3301e+00_rb,1.4611e+00_rb/)
      kbo(:,48, 9) = (/ &
     &9.0670e-01_rb,1.0329e+00_rb,1.1535e+00_rb,1.2830e+00_rb,1.4185e+00_rb/)
      kbo(:,49, 9) = (/ &
     &8.5414e-01_rb,9.8225e-01_rb,1.1056e+00_rb,1.2370e+00_rb,1.3774e+00_rb/)
      kbo(:,50, 9) = (/ &
     &8.0339e-01_rb,9.3369e-01_rb,1.0592e+00_rb,1.1907e+00_rb,1.3340e+00_rb/)
      kbo(:,51, 9) = (/ &
     &7.5351e-01_rb,8.8571e-01_rb,1.0133e+00_rb,1.1450e+00_rb,1.2894e+00_rb/)
      kbo(:,52, 9) = (/ &
     &7.0363e-01_rb,8.3718e-01_rb,9.6715e-01_rb,1.0991e+00_rb,1.2446e+00_rb/)
      kbo(:,53, 9) = (/ &
     &6.5356e-01_rb,7.8813e-01_rb,9.1966e-01_rb,1.0529e+00_rb,1.2015e+00_rb/)
      kbo(:,54, 9) = (/ &
     &6.0842e-01_rb,7.4258e-01_rb,8.7513e-01_rb,1.0061e+00_rb,1.1543e+00_rb/)
      kbo(:,55, 9) = (/ &
     &5.6587e-01_rb,6.9834e-01_rb,8.3229e-01_rb,9.6266e-01_rb,1.1037e+00_rb/)
      kbo(:,56, 9) = (/ &
     &5.2358e-01_rb,6.5442e-01_rb,7.8939e-01_rb,9.2104e-01_rb,1.0541e+00_rb/)
      kbo(:,57, 9) = (/ &
     &4.8183e-01_rb,6.1174e-01_rb,7.4558e-01_rb,8.7862e-01_rb,1.0077e+00_rb/)
      kbo(:,58, 9) = (/ &
     &4.4300e-01_rb,5.7151e-01_rb,7.0375e-01_rb,8.3825e-01_rb,9.6862e-01_rb/)
      kbo(:,59, 9) = (/ &
     &4.2745e-01_rb,5.5483e-01_rb,6.8739e-01_rb,8.2045e-01_rb,9.5225e-01_rb/)
      kbo(:,13,10) = (/ &
     &1.9893e+00_rb,1.9563e+00_rb,1.9174e+00_rb,1.9426e+00_rb,2.0503e+00_rb/)
      kbo(:,14,10) = (/ &
     &1.9716e+00_rb,2.0101e+00_rb,2.0332e+00_rb,2.0825e+00_rb,2.0990e+00_rb/)
      kbo(:,15,10) = (/ &
     &2.2404e+00_rb,2.3114e+00_rb,2.3474e+00_rb,2.4414e+00_rb,2.5235e+00_rb/)
      kbo(:,16,10) = (/ &
     &2.6948e+00_rb,2.7693e+00_rb,2.8500e+00_rb,2.9277e+00_rb,3.0098e+00_rb/)
      kbo(:,17,10) = (/ &
     &2.9786e+00_rb,3.0578e+00_rb,3.1508e+00_rb,3.2081e+00_rb,3.2791e+00_rb/)
      kbo(:,18,10) = (/ &
     &3.2445e+00_rb,3.3300e+00_rb,3.4200e+00_rb,3.4641e+00_rb,3.5430e+00_rb/)
      kbo(:,19,10) = (/ &
     &3.4975e+00_rb,3.5857e+00_rb,3.6668e+00_rb,3.7210e+00_rb,3.8007e+00_rb/)
      kbo(:,20,10) = (/ &
     &3.7569e+00_rb,3.8465e+00_rb,3.9165e+00_rb,3.9790e+00_rb,4.0533e+00_rb/)
      kbo(:,21,10) = (/ &
     &4.0221e+00_rb,4.1092e+00_rb,4.1701e+00_rb,4.2261e+00_rb,4.2948e+00_rb/)
      kbo(:,22,10) = (/ &
     &4.2735e+00_rb,4.3579e+00_rb,4.4003e+00_rb,4.4698e+00_rb,4.5287e+00_rb/)
      kbo(:,23,10) = (/ &
     &4.4889e+00_rb,4.5713e+00_rb,4.6026e+00_rb,4.6888e+00_rb,4.7442e+00_rb/)
      kbo(:,24,10) = (/ &
     &4.6627e+00_rb,4.7429e+00_rb,4.7936e+00_rb,4.8738e+00_rb,4.9305e+00_rb/)
      kbo(:,25,10) = (/ &
     &4.8010e+00_rb,4.8792e+00_rb,4.9473e+00_rb,5.0350e+00_rb,5.0940e+00_rb/)
      kbo(:,26,10) = (/ &
     &4.9124e+00_rb,4.9871e+00_rb,5.0797e+00_rb,5.1628e+00_rb,5.2299e+00_rb/)
      kbo(:,27,10) = (/ &
     &4.9985e+00_rb,5.0753e+00_rb,5.2023e+00_rb,5.2822e+00_rb,5.3506e+00_rb/)
      kbo(:,28,10) = (/ &
     &5.0601e+00_rb,5.1633e+00_rb,5.2946e+00_rb,5.3826e+00_rb,5.4390e+00_rb/)
      kbo(:,29,10) = (/ &
     &5.1149e+00_rb,5.2551e+00_rb,5.3769e+00_rb,5.4560e+00_rb,5.5331e+00_rb/)
      kbo(:,30,10) = (/ &
     &5.1829e+00_rb,5.3405e+00_rb,5.4468e+00_rb,5.5366e+00_rb,5.6267e+00_rb/)
      kbo(:,31,10) = (/ &
     &5.2708e+00_rb,5.3990e+00_rb,5.5145e+00_rb,5.6271e+00_rb,5.7226e+00_rb/)
      kbo(:,32,10) = (/ &
     &5.3210e+00_rb,5.4615e+00_rb,5.6064e+00_rb,5.7149e+00_rb,5.7961e+00_rb/)
      kbo(:,33,10) = (/ &
     &5.3784e+00_rb,5.5514e+00_rb,5.6898e+00_rb,5.7825e+00_rb,5.7881e+00_rb/)
      kbo(:,34,10) = (/ &
     &5.4610e+00_rb,5.6264e+00_rb,5.7421e+00_rb,5.8351e+00_rb,5.7469e+00_rb/)
      kbo(:,35,10) = (/ &
     &5.5182e+00_rb,5.6646e+00_rb,5.7805e+00_rb,5.8049e+00_rb,5.8731e+00_rb/)
      kbo(:,36,10) = (/ &
     &5.5474e+00_rb,5.6825e+00_rb,5.8317e+00_rb,5.7746e+00_rb,5.9436e+00_rb/)
      kbo(:,37,10) = (/ &
     &5.5356e+00_rb,5.6848e+00_rb,5.8413e+00_rb,5.7830e+00_rb,5.9888e+00_rb/)
      kbo(:,38,10) = (/ &
     &5.5240e+00_rb,5.6924e+00_rb,5.8161e+00_rb,5.8447e+00_rb,6.0305e+00_rb/)
      kbo(:,39,10) = (/ &
     &5.5062e+00_rb,5.7035e+00_rb,5.7750e+00_rb,5.8907e+00_rb,6.0598e+00_rb/)
      kbo(:,40,10) = (/ &
     &5.4721e+00_rb,5.6790e+00_rb,5.7621e+00_rb,5.8808e+00_rb,6.0578e+00_rb/)
      kbo(:,41,10) = (/ &
     &5.4259e+00_rb,5.6462e+00_rb,5.7509e+00_rb,5.8640e+00_rb,6.0494e+00_rb/)
      kbo(:,42,10) = (/ &
     &5.3746e+00_rb,5.6111e+00_rb,5.7387e+00_rb,5.8493e+00_rb,6.0309e+00_rb/)
      kbo(:,43,10) = (/ &
     &5.3001e+00_rb,5.5493e+00_rb,5.7206e+00_rb,5.8145e+00_rb,6.0040e+00_rb/)
      kbo(:,44,10) = (/ &
     &5.2173e+00_rb,5.4929e+00_rb,5.6988e+00_rb,5.7584e+00_rb,5.9757e+00_rb/)
      kbo(:,45,10) = (/ &
     &5.1348e+00_rb,5.4235e+00_rb,5.6596e+00_rb,5.7148e+00_rb,5.9430e+00_rb/)
      kbo(:,46,10) = (/ &
     &5.0314e+00_rb,5.3386e+00_rb,5.5941e+00_rb,5.6594e+00_rb,5.8994e+00_rb/)
      kbo(:,47,10) = (/ &
     &4.9050e+00_rb,5.2288e+00_rb,5.5164e+00_rb,5.5882e+00_rb,5.8407e+00_rb/)
      kbo(:,48,10) = (/ &
     &4.7768e+00_rb,5.1235e+00_rb,5.4274e+00_rb,5.5412e+00_rb,5.7765e+00_rb/)
      kbo(:,49,10) = (/ &
     &4.6322e+00_rb,5.0058e+00_rb,5.3313e+00_rb,5.4722e+00_rb,5.7006e+00_rb/)
      kbo(:,50,10) = (/ &
     &4.4926e+00_rb,4.8780e+00_rb,5.2241e+00_rb,5.4133e+00_rb,5.6277e+00_rb/)
      kbo(:,51,10) = (/ &
     &4.3523e+00_rb,4.7488e+00_rb,5.1238e+00_rb,5.3514e+00_rb,5.5587e+00_rb/)
      kbo(:,52,10) = (/ &
     &4.2056e+00_rb,4.6099e+00_rb,5.0074e+00_rb,5.2732e+00_rb,5.4803e+00_rb/)
      kbo(:,53,10) = (/ &
     &4.0530e+00_rb,4.4698e+00_rb,4.8873e+00_rb,5.1842e+00_rb,5.3844e+00_rb/)
      kbo(:,54,10) = (/ &
     &3.8976e+00_rb,4.3362e+00_rb,4.7616e+00_rb,5.1168e+00_rb,5.2766e+00_rb/)
      kbo(:,55,10) = (/ &
     &3.7298e+00_rb,4.2020e+00_rb,4.6200e+00_rb,5.0133e+00_rb,5.2077e+00_rb/)
      kbo(:,56,10) = (/ &
     &3.5605e+00_rb,4.0625e+00_rb,4.4800e+00_rb,4.8943e+00_rb,5.1989e+00_rb/)
      kbo(:,57,10) = (/ &
     &3.3848e+00_rb,3.9107e+00_rb,4.3536e+00_rb,4.7626e+00_rb,5.1398e+00_rb/)
      kbo(:,58,10) = (/ &
     &3.2413e+00_rb,3.7494e+00_rb,4.2254e+00_rb,4.6312e+00_rb,5.0237e+00_rb/)
      kbo(:,59,10) = (/ &
     &3.1874e+00_rb,3.6712e+00_rb,4.1570e+00_rb,4.5764e+00_rb,4.9431e+00_rb/)
      kbo(:,13,11) = (/ &
     &2.0655e+00_rb,2.2522e+00_rb,2.3749e+00_rb,2.3793e+00_rb,2.3141e+00_rb/)
      kbo(:,14,11) = (/ &
     &2.3056e+00_rb,2.3392e+00_rb,2.3483e+00_rb,2.4161e+00_rb,2.4783e+00_rb/)
      kbo(:,15,11) = (/ &
     &2.5044e+00_rb,2.5910e+00_rb,2.7153e+00_rb,2.7838e+00_rb,2.9462e+00_rb/)
      kbo(:,16,11) = (/ &
     &3.2130e+00_rb,3.2960e+00_rb,3.3708e+00_rb,3.3957e+00_rb,3.4936e+00_rb/)
      kbo(:,17,11) = (/ &
     &3.5929e+00_rb,3.6854e+00_rb,3.7568e+00_rb,3.8084e+00_rb,3.9070e+00_rb/)
      kbo(:,18,11) = (/ &
     &3.9502e+00_rb,4.0564e+00_rb,4.1260e+00_rb,4.2219e+00_rb,4.3335e+00_rb/)
      kbo(:,19,11) = (/ &
     &4.3215e+00_rb,4.4423e+00_rb,4.5361e+00_rb,4.6179e+00_rb,4.7405e+00_rb/)
      kbo(:,20,11) = (/ &
     &4.7059e+00_rb,4.8304e+00_rb,4.9342e+00_rb,4.9958e+00_rb,5.1320e+00_rb/)
      kbo(:,21,11) = (/ &
     &5.0845e+00_rb,5.2089e+00_rb,5.3085e+00_rb,5.3635e+00_rb,5.4973e+00_rb/)
      kbo(:,22,11) = (/ &
     &5.4678e+00_rb,5.5866e+00_rb,5.6718e+00_rb,5.7157e+00_rb,5.8446e+00_rb/)
      kbo(:,23,11) = (/ &
     &5.8472e+00_rb,5.9542e+00_rb,6.0224e+00_rb,6.0657e+00_rb,6.1751e+00_rb/)
      kbo(:,24,11) = (/ &
     &6.2102e+00_rb,6.3055e+00_rb,6.3477e+00_rb,6.4025e+00_rb,6.4818e+00_rb/)
      kbo(:,25,11) = (/ &
     &6.5425e+00_rb,6.6276e+00_rb,6.6464e+00_rb,6.7138e+00_rb,6.7776e+00_rb/)
      kbo(:,26,11) = (/ &
     &6.8397e+00_rb,6.9134e+00_rb,6.9153e+00_rb,7.0099e+00_rb,7.0590e+00_rb/)
      kbo(:,27,11) = (/ &
     &7.1001e+00_rb,7.1597e+00_rb,7.1676e+00_rb,7.2592e+00_rb,7.2956e+00_rb/)
      kbo(:,28,11) = (/ &
     &7.3310e+00_rb,7.3578e+00_rb,7.4192e+00_rb,7.4714e+00_rb,7.5147e+00_rb/)
      kbo(:,29,11) = (/ &
     &7.5220e+00_rb,7.5568e+00_rb,7.6448e+00_rb,7.6786e+00_rb,7.7106e+00_rb/)
      kbo(:,30,11) = (/ &
     &7.6639e+00_rb,7.7470e+00_rb,7.8152e+00_rb,7.8571e+00_rb,7.8626e+00_rb/)
      kbo(:,31,11) = (/ &
     &7.8082e+00_rb,7.9292e+00_rb,7.9711e+00_rb,7.9864e+00_rb,7.9914e+00_rb/)
      kbo(:,32,11) = (/ &
     &7.9725e+00_rb,8.0661e+00_rb,8.0888e+00_rb,8.1052e+00_rb,8.1459e+00_rb/)
      kbo(:,33,11) = (/ &
     &8.1196e+00_rb,8.1648e+00_rb,8.2204e+00_rb,8.2561e+00_rb,8.2714e+00_rb/)
      kbo(:,34,11) = (/ &
     &8.1982e+00_rb,8.2822e+00_rb,8.3431e+00_rb,8.3695e+00_rb,8.3494e+00_rb/)
      kbo(:,35,11) = (/ &
     &8.2893e+00_rb,8.3914e+00_rb,8.4459e+00_rb,8.4367e+00_rb,8.2201e+00_rb/)
      kbo(:,36,11) = (/ &
     &8.3709e+00_rb,8.4754e+00_rb,8.5081e+00_rb,8.5002e+00_rb,8.2056e+00_rb/)
      kbo(:,37,11) = (/ &
     &8.4255e+00_rb,8.5299e+00_rb,8.5562e+00_rb,8.5457e+00_rb,8.3297e+00_rb/)
      kbo(:,38,11) = (/ &
     &8.4692e+00_rb,8.5729e+00_rb,8.6048e+00_rb,8.5329e+00_rb,8.4574e+00_rb/)
      kbo(:,39,11) = (/ &
     &8.5044e+00_rb,8.6011e+00_rb,8.6529e+00_rb,8.5036e+00_rb,8.5412e+00_rb/)
      kbo(:,40,11) = (/ &
     &8.5034e+00_rb,8.6122e+00_rb,8.6861e+00_rb,8.5500e+00_rb,8.5878e+00_rb/)
      kbo(:,41,11) = (/ &
     &8.4921e+00_rb,8.6152e+00_rb,8.7146e+00_rb,8.6037e+00_rb,8.6322e+00_rb/)
      kbo(:,42,11) = (/ &
     &8.4716e+00_rb,8.6012e+00_rb,8.7281e+00_rb,8.6344e+00_rb,8.6768e+00_rb/)
      kbo(:,43,11) = (/ &
     &8.4399e+00_rb,8.6022e+00_rb,8.7354e+00_rb,8.6857e+00_rb,8.7108e+00_rb/)
      kbo(:,44,11) = (/ &
     &8.4042e+00_rb,8.5707e+00_rb,8.7247e+00_rb,8.7306e+00_rb,8.7301e+00_rb/)
      kbo(:,45,11) = (/ &
     &8.3422e+00_rb,8.5367e+00_rb,8.7092e+00_rb,8.7655e+00_rb,8.7531e+00_rb/)
      kbo(:,46,11) = (/ &
     &8.2614e+00_rb,8.4925e+00_rb,8.6796e+00_rb,8.7911e+00_rb,8.7636e+00_rb/)
      kbo(:,47,11) = (/ &
     &8.1773e+00_rb,8.4296e+00_rb,8.6376e+00_rb,8.8172e+00_rb,8.7465e+00_rb/)
      kbo(:,48,11) = (/ &
     &8.0681e+00_rb,8.3512e+00_rb,8.5867e+00_rb,8.7946e+00_rb,8.7119e+00_rb/)
      kbo(:,49,11) = (/ &
     &7.9551e+00_rb,8.2607e+00_rb,8.5307e+00_rb,8.7608e+00_rb,8.6937e+00_rb/)
      kbo(:,50,11) = (/ &
     &7.8252e+00_rb,8.1751e+00_rb,8.4616e+00_rb,8.7182e+00_rb,8.6487e+00_rb/)
      kbo(:,51,11) = (/ &
     &7.6779e+00_rb,8.0748e+00_rb,8.3664e+00_rb,8.6630e+00_rb,8.6216e+00_rb/)
      kbo(:,52,11) = (/ &
     &7.5020e+00_rb,7.9591e+00_rb,8.2751e+00_rb,8.6024e+00_rb,8.6131e+00_rb/)
      kbo(:,53,11) = (/ &
     &7.3138e+00_rb,7.8339e+00_rb,8.1806e+00_rb,8.5263e+00_rb,8.5994e+00_rb/)
      kbo(:,54,11) = (/ &
     &7.1204e+00_rb,7.6772e+00_rb,8.0838e+00_rb,8.4459e+00_rb,8.6417e+00_rb/)
      kbo(:,55,11) = (/ &
     &6.9256e+00_rb,7.5251e+00_rb,7.9750e+00_rb,8.3203e+00_rb,8.6369e+00_rb/)
      kbo(:,56,11) = (/ &
     &6.7619e+00_rb,7.3247e+00_rb,7.8477e+00_rb,8.2055e+00_rb,8.5418e+00_rb/)
      kbo(:,57,11) = (/ &
     &6.5989e+00_rb,7.1193e+00_rb,7.6865e+00_rb,8.1073e+00_rb,8.4236e+00_rb/)
      kbo(:,58,11) = (/ &
     &6.3964e+00_rb,6.9666e+00_rb,7.5354e+00_rb,7.9999e+00_rb,8.3228e+00_rb/)
      kbo(:,59,11) = (/ &
     &6.3130e+00_rb,6.9330e+00_rb,7.4209e+00_rb,7.9087e+00_rb,8.2712e+00_rb/)
      kbo(:,13,12) = (/ &
     &2.2326e+00_rb,2.2492e+00_rb,2.2166e+00_rb,2.2179e+00_rb,2.3939e+00_rb/)
      kbo(:,14,12) = (/ &
     &2.6624e+00_rb,2.8068e+00_rb,2.8766e+00_rb,2.8509e+00_rb,2.8616e+00_rb/)
      kbo(:,15,12) = (/ &
     &3.7151e+00_rb,3.8338e+00_rb,3.8710e+00_rb,3.8938e+00_rb,4.0387e+00_rb/)
      kbo(:,16,12) = (/ &
     &4.1478e+00_rb,4.2453e+00_rb,4.2845e+00_rb,4.3058e+00_rb,4.4171e+00_rb/)
      kbo(:,17,12) = (/ &
     &4.4594e+00_rb,4.5666e+00_rb,4.6335e+00_rb,4.6600e+00_rb,4.7776e+00_rb/)
      kbo(:,18,12) = (/ &
     &4.8949e+00_rb,5.0141e+00_rb,5.1198e+00_rb,5.1461e+00_rb,5.2514e+00_rb/)
      kbo(:,19,12) = (/ &
     &5.4166e+00_rb,5.5448e+00_rb,5.6541e+00_rb,5.7098e+00_rb,5.7608e+00_rb/)
      kbo(:,20,12) = (/ &
     &5.9687e+00_rb,6.1117e+00_rb,6.2318e+00_rb,6.2991e+00_rb,6.3382e+00_rb/)
      kbo(:,21,12) = (/ &
     &6.5357e+00_rb,6.6891e+00_rb,6.8185e+00_rb,6.8922e+00_rb,6.9339e+00_rb/)
      kbo(:,22,12) = (/ &
     &7.1247e+00_rb,7.2763e+00_rb,7.4121e+00_rb,7.4770e+00_rb,7.5498e+00_rb/)
      kbo(:,23,12) = (/ &
     &7.7039e+00_rb,7.8572e+00_rb,7.9884e+00_rb,8.0284e+00_rb,8.1432e+00_rb/)
      kbo(:,24,12) = (/ &
     &8.2700e+00_rb,8.4098e+00_rb,8.5280e+00_rb,8.5448e+00_rb,8.6892e+00_rb/)
      kbo(:,25,12) = (/ &
     &8.8099e+00_rb,8.9417e+00_rb,9.0327e+00_rb,9.0480e+00_rb,9.1843e+00_rb/)
      kbo(:,26,12) = (/ &
     &9.3282e+00_rb,9.4425e+00_rb,9.5016e+00_rb,9.4918e+00_rb,9.6320e+00_rb/)
      kbo(:,27,12) = (/ &
     &9.8120e+00_rb,9.9020e+00_rb,9.8963e+00_rb,9.9235e+00_rb,1.0029e+01_rb/)
      kbo(:,28,12) = (/ &
     &1.0246e+01_rb,1.0311e+01_rb,1.0256e+01_rb,1.0345e+01_rb,1.0401e+01_rb/)
      kbo(:,29,12) = (/ &
     &1.0631e+01_rb,1.0621e+01_rb,1.0577e+01_rb,1.0691e+01_rb,1.0719e+01_rb/)
      kbo(:,30,12) = (/ &
     &1.0962e+01_rb,1.0883e+01_rb,1.0936e+01_rb,1.0968e+01_rb,1.0995e+01_rb/)
      kbo(:,31,12) = (/ &
     &1.1183e+01_rb,1.1144e+01_rb,1.1216e+01_rb,1.1241e+01_rb,1.1250e+01_rb/)
      kbo(:,32,12) = (/ &
     &1.1390e+01_rb,1.1439e+01_rb,1.1434e+01_rb,1.1471e+01_rb,1.1430e+01_rb/)
      kbo(:,33,12) = (/ &
     &1.1588e+01_rb,1.1624e+01_rb,1.1599e+01_rb,1.1614e+01_rb,1.1630e+01_rb/)
      kbo(:,34,12) = (/ &
     &1.1823e+01_rb,1.1780e+01_rb,1.1748e+01_rb,1.1805e+01_rb,1.1825e+01_rb/)
      kbo(:,35,12) = (/ &
     &1.1935e+01_rb,1.1885e+01_rb,1.1934e+01_rb,1.1981e+01_rb,1.2011e+01_rb/)
      kbo(:,36,12) = (/ &
     &1.2055e+01_rb,1.2055e+01_rb,1.2092e+01_rb,1.2129e+01_rb,1.2083e+01_rb/)
      kbo(:,37,12) = (/ &
     &1.2143e+01_rb,1.2189e+01_rb,1.2217e+01_rb,1.2211e+01_rb,1.2038e+01_rb/)
      kbo(:,38,12) = (/ &
     &1.2240e+01_rb,1.2287e+01_rb,1.2320e+01_rb,1.2293e+01_rb,1.1958e+01_rb/)
      kbo(:,39,12) = (/ &
     &1.2384e+01_rb,1.2401e+01_rb,1.2395e+01_rb,1.2388e+01_rb,1.1943e+01_rb/)
      kbo(:,40,12) = (/ &
     &1.2464e+01_rb,1.2479e+01_rb,1.2477e+01_rb,1.2446e+01_rb,1.2032e+01_rb/)
      kbo(:,41,12) = (/ &
     &1.2528e+01_rb,1.2533e+01_rb,1.2534e+01_rb,1.2503e+01_rb,1.2103e+01_rb/)
      kbo(:,42,12) = (/ &
     &1.2585e+01_rb,1.2609e+01_rb,1.2591e+01_rb,1.2562e+01_rb,1.2170e+01_rb/)
      kbo(:,43,12) = (/ &
     &1.2648e+01_rb,1.2666e+01_rb,1.2643e+01_rb,1.2594e+01_rb,1.2252e+01_rb/)
      kbo(:,44,12) = (/ &
     &1.2682e+01_rb,1.2728e+01_rb,1.2701e+01_rb,1.2655e+01_rb,1.2340e+01_rb/)
      kbo(:,45,12) = (/ &
     &1.2706e+01_rb,1.2763e+01_rb,1.2739e+01_rb,1.2702e+01_rb,1.2417e+01_rb/)
      kbo(:,46,12) = (/ &
     &1.2744e+01_rb,1.2798e+01_rb,1.2795e+01_rb,1.2773e+01_rb,1.2516e+01_rb/)
      kbo(:,47,12) = (/ &
     &1.2744e+01_rb,1.2852e+01_rb,1.2846e+01_rb,1.2816e+01_rb,1.2652e+01_rb/)
      kbo(:,48,12) = (/ &
     &1.2714e+01_rb,1.2860e+01_rb,1.2886e+01_rb,1.2860e+01_rb,1.2772e+01_rb/)
      kbo(:,49,12) = (/ &
     &1.2667e+01_rb,1.2862e+01_rb,1.2927e+01_rb,1.2926e+01_rb,1.2873e+01_rb/)
      kbo(:,50,12) = (/ &
     &1.2568e+01_rb,1.2858e+01_rb,1.2953e+01_rb,1.2955e+01_rb,1.2993e+01_rb/)
      kbo(:,51,12) = (/ &
     &1.2506e+01_rb,1.2803e+01_rb,1.2982e+01_rb,1.2983e+01_rb,1.3071e+01_rb/)
      kbo(:,52,12) = (/ &
     &1.2444e+01_rb,1.2786e+01_rb,1.2969e+01_rb,1.3005e+01_rb,1.3112e+01_rb/)
      kbo(:,53,12) = (/ &
     &1.2352e+01_rb,1.2737e+01_rb,1.2961e+01_rb,1.3024e+01_rb,1.3150e+01_rb/)
      kbo(:,54,12) = (/ &
     &1.2273e+01_rb,1.2640e+01_rb,1.2911e+01_rb,1.3015e+01_rb,1.3144e+01_rb/)
      kbo(:,55,12) = (/ &
     &1.2241e+01_rb,1.2456e+01_rb,1.2830e+01_rb,1.3015e+01_rb,1.3103e+01_rb/)
      kbo(:,56,12) = (/ &
     &1.2129e+01_rb,1.2403e+01_rb,1.2744e+01_rb,1.2955e+01_rb,1.3054e+01_rb/)
      kbo(:,57,12) = (/ &
     &1.1983e+01_rb,1.2327e+01_rb,1.2603e+01_rb,1.2910e+01_rb,1.3056e+01_rb/)
      kbo(:,58,12) = (/ &
     &1.1825e+01_rb,1.2269e+01_rb,1.2506e+01_rb,1.2817e+01_rb,1.3023e+01_rb/)
      kbo(:,59,12) = (/ &
     &1.1756e+01_rb,1.2243e+01_rb,1.2496e+01_rb,1.2736e+01_rb,1.2923e+01_rb/)
      kbo(:,13,13) = (/ &
     &2.4382e+00_rb,2.5269e+00_rb,2.6487e+00_rb,2.6115e+00_rb,2.6014e+00_rb/)
      kbo(:,14,13) = (/ &
     &3.1530e+00_rb,3.3624e+00_rb,3.5863e+00_rb,3.8586e+00_rb,4.4708e+00_rb/)
      kbo(:,15,13) = (/ &
     &5.2015e+00_rb,5.3405e+00_rb,5.5020e+00_rb,5.5114e+00_rb,5.5124e+00_rb/)
      kbo(:,16,13) = (/ &
     &5.7225e+00_rb,5.8966e+00_rb,6.0571e+00_rb,6.0559e+00_rb,6.0668e+00_rb/)
      kbo(:,17,13) = (/ &
     &6.2716e+00_rb,6.4422e+00_rb,6.5990e+00_rb,6.6435e+00_rb,6.5757e+00_rb/)
      kbo(:,18,13) = (/ &
     &6.7715e+00_rb,6.9371e+00_rb,7.0939e+00_rb,7.1586e+00_rb,7.0596e+00_rb/)
      kbo(:,19,13) = (/ &
     &7.2728e+00_rb,7.4403e+00_rb,7.6008e+00_rb,7.7026e+00_rb,7.6789e+00_rb/)
      kbo(:,20,13) = (/ &
     &7.8570e+00_rb,8.0286e+00_rb,8.1992e+00_rb,8.3169e+00_rb,8.3122e+00_rb/)
      kbo(:,21,13) = (/ &
     &8.5574e+00_rb,8.7300e+00_rb,8.9004e+00_rb,9.0231e+00_rb,9.0274e+00_rb/)
      kbo(:,22,13) = (/ &
     &9.3806e+00_rb,9.5518e+00_rb,9.7137e+00_rb,9.8169e+00_rb,9.7877e+00_rb/)
      kbo(:,23,13) = (/ &
     &1.0275e+01_rb,1.0446e+01_rb,1.0597e+01_rb,1.0672e+01_rb,1.0640e+01_rb/)
      kbo(:,24,13) = (/ &
     &1.1189e+01_rb,1.1360e+01_rb,1.1491e+01_rb,1.1535e+01_rb,1.1479e+01_rb/)
      kbo(:,25,13) = (/ &
     &1.2097e+01_rb,1.2245e+01_rb,1.2356e+01_rb,1.2332e+01_rb,1.2284e+01_rb/)
      kbo(:,26,13) = (/ &
     &1.2961e+01_rb,1.3082e+01_rb,1.3167e+01_rb,1.3081e+01_rb,1.3047e+01_rb/)
      kbo(:,27,13) = (/ &
     &1.3759e+01_rb,1.3850e+01_rb,1.3907e+01_rb,1.3774e+01_rb,1.3731e+01_rb/)
      kbo(:,28,13) = (/ &
     &1.4487e+01_rb,1.4544e+01_rb,1.4510e+01_rb,1.4345e+01_rb,1.4410e+01_rb/)
      kbo(:,29,13) = (/ &
     &1.5131e+01_rb,1.5152e+01_rb,1.4989e+01_rb,1.4893e+01_rb,1.4931e+01_rb/)
      kbo(:,30,13) = (/ &
     &1.5691e+01_rb,1.5628e+01_rb,1.5428e+01_rb,1.5458e+01_rb,1.5409e+01_rb/)
      kbo(:,31,13) = (/ &
     &1.6170e+01_rb,1.5957e+01_rb,1.5859e+01_rb,1.5855e+01_rb,1.5824e+01_rb/)
      kbo(:,32,13) = (/ &
     &1.6473e+01_rb,1.6259e+01_rb,1.6330e+01_rb,1.6208e+01_rb,1.6186e+01_rb/)
      kbo(:,33,13) = (/ &
     &1.6723e+01_rb,1.6704e+01_rb,1.6654e+01_rb,1.6563e+01_rb,1.6454e+01_rb/)
      kbo(:,34,13) = (/ &
     &1.6982e+01_rb,1.6991e+01_rb,1.6942e+01_rb,1.6797e+01_rb,1.6670e+01_rb/)
      kbo(:,35,13) = (/ &
     &1.7322e+01_rb,1.7270e+01_rb,1.7135e+01_rb,1.6953e+01_rb,1.6879e+01_rb/)
      kbo(:,36,13) = (/ &
     &1.7562e+01_rb,1.7451e+01_rb,1.7282e+01_rb,1.7228e+01_rb,1.7169e+01_rb/)
      kbo(:,37,13) = (/ &
     &1.7777e+01_rb,1.7615e+01_rb,1.7471e+01_rb,1.7494e+01_rb,1.7411e+01_rb/)
      kbo(:,38,13) = (/ &
     &1.7932e+01_rb,1.7773e+01_rb,1.7733e+01_rb,1.7665e+01_rb,1.7616e+01_rb/)
      kbo(:,39,13) = (/ &
     &1.7993e+01_rb,1.7881e+01_rb,1.7968e+01_rb,1.7898e+01_rb,1.7772e+01_rb/)
      kbo(:,40,13) = (/ &
     &1.8148e+01_rb,1.8027e+01_rb,1.8107e+01_rb,1.8066e+01_rb,1.7907e+01_rb/)
      kbo(:,41,13) = (/ &
     &1.8273e+01_rb,1.8183e+01_rb,1.8251e+01_rb,1.8205e+01_rb,1.8057e+01_rb/)
      kbo(:,42,13) = (/ &
     &1.8408e+01_rb,1.8324e+01_rb,1.8378e+01_rb,1.8322e+01_rb,1.8177e+01_rb/)
      kbo(:,43,13) = (/ &
     &1.8505e+01_rb,1.8424e+01_rb,1.8497e+01_rb,1.8450e+01_rb,1.8293e+01_rb/)
      kbo(:,44,13) = (/ &
     &1.8604e+01_rb,1.8490e+01_rb,1.8578e+01_rb,1.8585e+01_rb,1.8430e+01_rb/)
      kbo(:,45,13) = (/ &
     &1.8704e+01_rb,1.8600e+01_rb,1.8704e+01_rb,1.8731e+01_rb,1.8566e+01_rb/)
      kbo(:,46,13) = (/ &
     &1.8778e+01_rb,1.8729e+01_rb,1.8813e+01_rb,1.8830e+01_rb,1.8670e+01_rb/)
      kbo(:,47,13) = (/ &
     &1.8850e+01_rb,1.8781e+01_rb,1.8910e+01_rb,1.8941e+01_rb,1.8764e+01_rb/)
      kbo(:,48,13) = (/ &
     &1.8986e+01_rb,1.8857e+01_rb,1.8976e+01_rb,1.9005e+01_rb,1.8895e+01_rb/)
      kbo(:,49,13) = (/ &
     &1.9095e+01_rb,1.9011e+01_rb,1.9061e+01_rb,1.9086e+01_rb,1.9025e+01_rb/)
      kbo(:,50,13) = (/ &
     &1.9280e+01_rb,1.9083e+01_rb,1.9157e+01_rb,1.9191e+01_rb,1.9123e+01_rb/)
      kbo(:,51,13) = (/ &
     &1.9385e+01_rb,1.9151e+01_rb,1.9215e+01_rb,1.9259e+01_rb,1.9196e+01_rb/)
      kbo(:,52,13) = (/ &
     &1.9488e+01_rb,1.9188e+01_rb,1.9260e+01_rb,1.9328e+01_rb,1.9259e+01_rb/)
      kbo(:,53,13) = (/ &
     &1.9657e+01_rb,1.9231e+01_rb,1.9381e+01_rb,1.9391e+01_rb,1.9331e+01_rb/)
      kbo(:,54,13) = (/ &
     &1.9810e+01_rb,1.9362e+01_rb,1.9357e+01_rb,1.9443e+01_rb,1.9402e+01_rb/)
      kbo(:,55,13) = (/ &
     &1.9885e+01_rb,1.9515e+01_rb,1.9337e+01_rb,1.9415e+01_rb,1.9400e+01_rb/)
      kbo(:,56,13) = (/ &
     &1.9952e+01_rb,1.9629e+01_rb,1.9325e+01_rb,1.9383e+01_rb,1.9431e+01_rb/)
      kbo(:,57,13) = (/ &
     &2.0005e+01_rb,1.9822e+01_rb,1.9466e+01_rb,1.9235e+01_rb,1.9419e+01_rb/)
      kbo(:,58,13) = (/ &
     &2.0044e+01_rb,1.9891e+01_rb,1.9520e+01_rb,1.9352e+01_rb,1.9344e+01_rb/)
      kbo(:,59,13) = (/ &
     &2.0058e+01_rb,1.9920e+01_rb,1.9627e+01_rb,1.9319e+01_rb,1.9225e+01_rb/)
      kbo(:,13,14) = (/ &
     &3.1216e+00_rb,3.1914e+00_rb,3.0899e+00_rb,3.3224e+00_rb,4.2010e+00_rb/)
      kbo(:,14,14) = (/ &
     &7.0173e+00_rb,7.2031e+00_rb,7.3581e+00_rb,7.0289e+00_rb,6.9115e+00_rb/)
      kbo(:,15,14) = (/ &
     &7.9333e+00_rb,8.1347e+00_rb,8.2980e+00_rb,8.2914e+00_rb,7.9259e+00_rb/)
      kbo(:,16,14) = (/ &
     &8.8725e+00_rb,9.0683e+00_rb,9.2568e+00_rb,9.4307e+00_rb,9.0182e+00_rb/)
      kbo(:,17,14) = (/ &
     &9.8363e+00_rb,1.0075e+01_rb,1.0298e+01_rb,1.0509e+01_rb,1.0358e+01_rb/)
      kbo(:,18,14) = (/ &
     &1.0886e+01_rb,1.1174e+01_rb,1.1435e+01_rb,1.1674e+01_rb,1.1694e+01_rb/)
      kbo(:,19,14) = (/ &
     &1.2011e+01_rb,1.2333e+01_rb,1.2632e+01_rb,1.2897e+01_rb,1.3012e+01_rb/)
      kbo(:,20,14) = (/ &
     &1.3190e+01_rb,1.3540e+01_rb,1.3861e+01_rb,1.4163e+01_rb,1.4313e+01_rb/)
      kbo(:,21,14) = (/ &
     &1.4385e+01_rb,1.4764e+01_rb,1.5119e+01_rb,1.5437e+01_rb,1.5593e+01_rb/)
      kbo(:,22,14) = (/ &
     &1.5599e+01_rb,1.6013e+01_rb,1.6388e+01_rb,1.6714e+01_rb,1.6844e+01_rb/)
      kbo(:,23,14) = (/ &
     &1.6855e+01_rb,1.7274e+01_rb,1.7644e+01_rb,1.7957e+01_rb,1.7983e+01_rb/)
      kbo(:,24,14) = (/ &
     &1.8152e+01_rb,1.8541e+01_rb,1.8882e+01_rb,1.9172e+01_rb,1.9130e+01_rb/)
      kbo(:,25,14) = (/ &
     &1.9440e+01_rb,1.9782e+01_rb,2.0084e+01_rb,2.0333e+01_rb,2.0194e+01_rb/)
      kbo(:,26,14) = (/ &
     &2.0688e+01_rb,2.0983e+01_rb,2.1232e+01_rb,2.1433e+01_rb,2.1144e+01_rb/)
      kbo(:,27,14) = (/ &
     &2.1870e+01_rb,2.2106e+01_rb,2.2296e+01_rb,2.2344e+01_rb,2.1989e+01_rb/)
      kbo(:,28,14) = (/ &
     &2.2959e+01_rb,2.3128e+01_rb,2.3255e+01_rb,2.3127e+01_rb,2.2540e+01_rb/)
      kbo(:,29,14) = (/ &
     &2.3936e+01_rb,2.4040e+01_rb,2.4108e+01_rb,2.3686e+01_rb,2.3118e+01_rb/)
      kbo(:,30,14) = (/ &
     &2.4794e+01_rb,2.4831e+01_rb,2.4631e+01_rb,2.3967e+01_rb,2.3807e+01_rb/)
      kbo(:,31,14) = (/ &
     &2.5525e+01_rb,2.5458e+01_rb,2.4921e+01_rb,2.4395e+01_rb,2.4482e+01_rb/)
      kbo(:,32,14) = (/ &
     &2.6140e+01_rb,2.5757e+01_rb,2.4988e+01_rb,2.5111e+01_rb,2.5399e+01_rb/)
      kbo(:,33,14) = (/ &
     &2.6507e+01_rb,2.5703e+01_rb,2.5535e+01_rb,2.5869e+01_rb,2.6121e+01_rb/)
      kbo(:,34,14) = (/ &
     &2.6569e+01_rb,2.5953e+01_rb,2.6151e+01_rb,2.6361e+01_rb,2.6801e+01_rb/)
      kbo(:,35,14) = (/ &
     &2.6562e+01_rb,2.6504e+01_rb,2.6810e+01_rb,2.7029e+01_rb,2.7434e+01_rb/)
      kbo(:,36,14) = (/ &
     &2.6827e+01_rb,2.6824e+01_rb,2.7232e+01_rb,2.7537e+01_rb,2.7695e+01_rb/)
      kbo(:,37,14) = (/ &
     &2.7090e+01_rb,2.7286e+01_rb,2.7649e+01_rb,2.7765e+01_rb,2.8180e+01_rb/)
      kbo(:,38,14) = (/ &
     &2.7488e+01_rb,2.7623e+01_rb,2.7893e+01_rb,2.8192e+01_rb,2.8556e+01_rb/)
      kbo(:,39,14) = (/ &
     &2.7778e+01_rb,2.8122e+01_rb,2.8064e+01_rb,2.8480e+01_rb,2.8841e+01_rb/)
      kbo(:,40,14) = (/ &
     &2.8002e+01_rb,2.8307e+01_rb,2.8286e+01_rb,2.8653e+01_rb,2.9020e+01_rb/)
      kbo(:,41,14) = (/ &
     &2.8205e+01_rb,2.8457e+01_rb,2.8430e+01_rb,2.8865e+01_rb,2.9164e+01_rb/)
      kbo(:,42,14) = (/ &
     &2.8421e+01_rb,2.8563e+01_rb,2.8567e+01_rb,2.9033e+01_rb,2.9360e+01_rb/)
      kbo(:,43,14) = (/ &
     &2.8605e+01_rb,2.8649e+01_rb,2.8746e+01_rb,2.9195e+01_rb,2.9582e+01_rb/)
      kbo(:,44,14) = (/ &
     &2.8719e+01_rb,2.8847e+01_rb,2.8915e+01_rb,2.9326e+01_rb,2.9712e+01_rb/)
      kbo(:,45,14) = (/ &
     &2.9008e+01_rb,2.9000e+01_rb,2.9085e+01_rb,2.9440e+01_rb,2.9882e+01_rb/)
      kbo(:,46,14) = (/ &
     &2.9283e+01_rb,2.9066e+01_rb,2.9214e+01_rb,2.9505e+01_rb,3.0075e+01_rb/)
      kbo(:,47,14) = (/ &
     &2.9505e+01_rb,2.9215e+01_rb,2.9192e+01_rb,2.9549e+01_rb,3.0231e+01_rb/)
      kbo(:,48,14) = (/ &
     &2.9847e+01_rb,2.9465e+01_rb,2.9377e+01_rb,2.9796e+01_rb,3.0530e+01_rb/)
      kbo(:,49,14) = (/ &
     &3.0209e+01_rb,2.9590e+01_rb,2.9613e+01_rb,2.9947e+01_rb,3.0891e+01_rb/)
      kbo(:,50,14) = (/ &
     &3.0555e+01_rb,2.9682e+01_rb,2.9721e+01_rb,3.0034e+01_rb,3.0987e+01_rb/)
      kbo(:,51,14) = (/ &
     &3.0932e+01_rb,2.9941e+01_rb,2.9791e+01_rb,3.0042e+01_rb,3.0973e+01_rb/)
      kbo(:,52,14) = (/ &
     &3.1304e+01_rb,3.0128e+01_rb,2.9896e+01_rb,3.0085e+01_rb,3.1235e+01_rb/)
      kbo(:,53,14) = (/ &
     &3.1593e+01_rb,3.0463e+01_rb,3.0008e+01_rb,3.0297e+01_rb,3.1448e+01_rb/)
      kbo(:,54,14) = (/ &
     &3.1880e+01_rb,3.0666e+01_rb,3.0079e+01_rb,3.0245e+01_rb,3.1244e+01_rb/)
      kbo(:,55,14) = (/ &
     &3.2179e+01_rb,3.1215e+01_rb,3.0197e+01_rb,3.0244e+01_rb,3.0630e+01_rb/)
      kbo(:,56,14) = (/ &
     &3.2496e+01_rb,3.1618e+01_rb,3.0405e+01_rb,3.0157e+01_rb,3.0368e+01_rb/)
      kbo(:,57,14) = (/ &
     &3.2846e+01_rb,3.1885e+01_rb,3.0675e+01_rb,3.0336e+01_rb,3.0326e+01_rb/)
      kbo(:,58,14) = (/ &
     &3.3194e+01_rb,3.2160e+01_rb,3.1205e+01_rb,3.0229e+01_rb,3.0346e+01_rb/)
      kbo(:,59,14) = (/ &
     &3.3342e+01_rb,3.2279e+01_rb,3.1453e+01_rb,3.0328e+01_rb,3.0152e+01_rb/)
      kbo(:,13,15) = (/ &
     &4.0030e+00_rb,4.7414e+00_rb,5.8560e+00_rb,6.7438e+00_rb,3.2808e+00_rb/)
      kbo(:,14,15) = (/ &
     &9.0899e+00_rb,9.4664e+00_rb,9.7823e+00_rb,1.0059e+01_rb,8.9794e+00_rb/)
      kbo(:,15,15) = (/ &
     &1.0857e+01_rb,1.1261e+01_rb,1.1617e+01_rb,1.1923e+01_rb,1.1563e+01_rb/)
      kbo(:,16,15) = (/ &
     &1.2889e+01_rb,1.3334e+01_rb,1.3725e+01_rb,1.4064e+01_rb,1.4341e+01_rb/)
      kbo(:,17,15) = (/ &
     &1.5204e+01_rb,1.5687e+01_rb,1.6115e+01_rb,1.6474e+01_rb,1.6771e+01_rb/)
      kbo(:,18,15) = (/ &
     &1.7808e+01_rb,1.8322e+01_rb,1.8770e+01_rb,1.9149e+01_rb,1.9438e+01_rb/)
      kbo(:,19,15) = (/ &
     &2.0675e+01_rb,2.1216e+01_rb,2.1671e+01_rb,2.2040e+01_rb,2.2294e+01_rb/)
      kbo(:,20,15) = (/ &
     &2.3784e+01_rb,2.4335e+01_rb,2.4780e+01_rb,2.5092e+01_rb,2.5318e+01_rb/)
      kbo(:,21,15) = (/ &
     &2.7074e+01_rb,2.7616e+01_rb,2.7992e+01_rb,2.8260e+01_rb,2.8435e+01_rb/)
      kbo(:,22,15) = (/ &
     &3.0512e+01_rb,3.0967e+01_rb,3.1281e+01_rb,3.1482e+01_rb,3.1588e+01_rb/)
      kbo(:,23,15) = (/ &
     &3.3946e+01_rb,3.4307e+01_rb,3.4554e+01_rb,3.4683e+01_rb,3.4702e+01_rb/)
      kbo(:,24,15) = (/ &
     &3.7353e+01_rb,3.7642e+01_rb,3.7797e+01_rb,3.7830e+01_rb,3.7737e+01_rb/)
      kbo(:,25,15) = (/ &
     &4.0725e+01_rb,4.0910e+01_rb,4.0948e+01_rb,4.0847e+01_rb,4.0612e+01_rb/)
      kbo(:,26,15) = (/ &
     &4.3991e+01_rb,4.4018e+01_rb,4.3898e+01_rb,4.3642e+01_rb,4.3255e+01_rb/)
      kbo(:,27,15) = (/ &
     &4.7033e+01_rb,4.6887e+01_rb,4.6594e+01_rb,4.6165e+01_rb,4.5614e+01_rb/)
      kbo(:,28,15) = (/ &
     &4.9783e+01_rb,4.9454e+01_rb,4.8983e+01_rb,4.8387e+01_rb,4.7672e+01_rb/)
      kbo(:,29,15) = (/ &
     &5.2209e+01_rb,5.1689e+01_rb,5.1048e+01_rb,5.0282e+01_rb,4.9143e+01_rb/)
      kbo(:,30,15) = (/ &
     &5.4285e+01_rb,5.3590e+01_rb,5.2778e+01_rb,5.1862e+01_rb,4.9504e+01_rb/)
      kbo(:,31,15) = (/ &
     &5.6017e+01_rb,5.5158e+01_rb,5.4192e+01_rb,5.2520e+01_rb,4.8958e+01_rb/)
      kbo(:,32,15) = (/ &
     &5.7422e+01_rb,5.6414e+01_rb,5.5126e+01_rb,5.1643e+01_rb,4.7729e+01_rb/)
      kbo(:,33,15) = (/ &
     &5.8538e+01_rb,5.7396e+01_rb,5.4438e+01_rb,4.9797e+01_rb,4.8468e+01_rb/)
      kbo(:,34,15) = (/ &
     &5.9425e+01_rb,5.7428e+01_rb,5.3101e+01_rb,5.0744e+01_rb,5.0414e+01_rb/)
      kbo(:,35,15) = (/ &
     &6.0220e+01_rb,5.6550e+01_rb,5.2246e+01_rb,5.2031e+01_rb,5.2160e+01_rb/)
      kbo(:,36,15) = (/ &
     &6.0380e+01_rb,5.6323e+01_rb,5.3290e+01_rb,5.3091e+01_rb,5.1837e+01_rb/)
      kbo(:,37,15) = (/ &
     &6.1064e+01_rb,5.6288e+01_rb,5.3962e+01_rb,5.4330e+01_rb,5.1190e+01_rb/)
      kbo(:,38,15) = (/ &
     &6.1244e+01_rb,5.6422e+01_rb,5.4974e+01_rb,5.5783e+01_rb,5.0823e+01_rb/)
      kbo(:,39,15) = (/ &
     &6.1491e+01_rb,5.6521e+01_rb,5.6495e+01_rb,5.7139e+01_rb,5.0363e+01_rb/)
      kbo(:,40,15) = (/ &
     &6.2285e+01_rb,5.7310e+01_rb,5.7143e+01_rb,5.8068e+01_rb,5.1166e+01_rb/)
      kbo(:,41,15) = (/ &
     &6.3214e+01_rb,5.7786e+01_rb,5.7889e+01_rb,5.8807e+01_rb,5.2057e+01_rb/)
      kbo(:,42,15) = (/ &
     &6.3921e+01_rb,5.8540e+01_rb,5.8564e+01_rb,5.9508e+01_rb,5.2732e+01_rb/)
      kbo(:,43,15) = (/ &
     &6.4936e+01_rb,5.9750e+01_rb,5.9252e+01_rb,6.0188e+01_rb,5.3641e+01_rb/)
      kbo(:,44,15) = (/ &
     &6.6180e+01_rb,6.0810e+01_rb,5.9653e+01_rb,6.1132e+01_rb,5.4737e+01_rb/)
      kbo(:,45,15) = (/ &
     &6.6949e+01_rb,6.1798e+01_rb,5.9966e+01_rb,6.1660e+01_rb,5.5582e+01_rb/)
      kbo(:,46,15) = (/ &
     &6.7824e+01_rb,6.2979e+01_rb,6.0686e+01_rb,6.2546e+01_rb,5.6565e+01_rb/)
      kbo(:,47,15) = (/ &
     &6.9039e+01_rb,6.4342e+01_rb,6.1803e+01_rb,6.2906e+01_rb,5.8086e+01_rb/)
      kbo(:,48,15) = (/ &
     &6.9761e+01_rb,6.5409e+01_rb,6.2421e+01_rb,6.3565e+01_rb,5.8945e+01_rb/)
      kbo(:,49,15) = (/ &
     &7.0431e+01_rb,6.6423e+01_rb,6.3050e+01_rb,6.4501e+01_rb,5.9296e+01_rb/)
      kbo(:,50,15) = (/ &
     &7.1030e+01_rb,6.7738e+01_rb,6.3115e+01_rb,6.5100e+01_rb,6.0669e+01_rb/)
      kbo(:,51,15) = (/ &
     &7.1594e+01_rb,6.8849e+01_rb,6.3996e+01_rb,6.5429e+01_rb,6.2495e+01_rb/)
      kbo(:,52,15) = (/ &
     &7.2143e+01_rb,7.0005e+01_rb,6.4776e+01_rb,6.5575e+01_rb,6.3548e+01_rb/)
      kbo(:,53,15) = (/ &
     &7.2670e+01_rb,7.0732e+01_rb,6.5046e+01_rb,6.6263e+01_rb,6.4505e+01_rb/)
      kbo(:,54,15) = (/ &
     &7.3127e+01_rb,7.1823e+01_rb,6.6914e+01_rb,6.6275e+01_rb,6.7029e+01_rb/)
      kbo(:,55,15) = (/ &
     &7.3553e+01_rb,7.2289e+01_rb,6.8950e+01_rb,6.5529e+01_rb,6.7722e+01_rb/)
      kbo(:,56,15) = (/ &
     &7.3983e+01_rb,7.2738e+01_rb,7.0656e+01_rb,6.5646e+01_rb,6.6961e+01_rb/)
      kbo(:,57,15) = (/ &
     &7.4388e+01_rb,7.3166e+01_rb,7.1848e+01_rb,6.6976e+01_rb,6.6137e+01_rb/)
      kbo(:,58,15) = (/ &
     &7.4771e+01_rb,7.3567e+01_rb,7.2279e+01_rb,6.9020e+01_rb,6.5380e+01_rb/)
      kbo(:,59,15) = (/ &
     &7.4949e+01_rb,7.3730e+01_rb,7.2460e+01_rb,7.0691e+01_rb,6.5332e+01_rb/)
      kbo(:,13,16) = (/ &
     &5.0420e+00_rb,8.6337e+00_rb,8.8692e+00_rb,1.2195e+00_rb,9.3776e-01_rb/)
      kbo(:,14,16) = (/ &
     &1.0108e+01_rb,1.0449e+01_rb,1.0740e+01_rb,1.1004e+01_rb,9.1915e-01_rb/)
      kbo(:,15,16) = (/ &
     &1.2227e+01_rb,1.2648e+01_rb,1.3007e+01_rb,1.3327e+01_rb,1.3667e+01_rb/)
      kbo(:,16,16) = (/ &
     &1.4789e+01_rb,1.5299e+01_rb,1.5728e+01_rb,1.6109e+01_rb,1.6498e+01_rb/)
      kbo(:,17,16) = (/ &
     &1.7862e+01_rb,1.8466e+01_rb,1.8964e+01_rb,1.9401e+01_rb,1.9824e+01_rb/)
      kbo(:,18,16) = (/ &
     &2.1506e+01_rb,2.2192e+01_rb,2.2762e+01_rb,2.3238e+01_rb,2.3665e+01_rb/)
      kbo(:,19,16) = (/ &
     &2.5786e+01_rb,2.6523e+01_rb,2.7142e+01_rb,2.7627e+01_rb,2.8030e+01_rb/)
      kbo(:,20,16) = (/ &
     &3.0723e+01_rb,3.1489e+01_rb,3.2109e+01_rb,3.2556e+01_rb,3.2877e+01_rb/)
      kbo(:,21,16) = (/ &
     &3.6293e+01_rb,3.7031e+01_rb,3.7583e+01_rb,3.7941e+01_rb,3.8130e+01_rb/)
      kbo(:,22,16) = (/ &
     &4.2493e+01_rb,4.3104e+01_rb,4.3485e+01_rb,4.3668e+01_rb,4.3655e+01_rb/)
      kbo(:,23,16) = (/ &
     &4.9099e+01_rb,4.9493e+01_rb,4.9618e+01_rb,4.9538e+01_rb,4.9251e+01_rb/)
      kbo(:,24,16) = (/ &
     &5.5931e+01_rb,5.5983e+01_rb,5.5792e+01_rb,5.5378e+01_rb,5.4757e+01_rb/)
      kbo(:,25,16) = (/ &
     &6.2744e+01_rb,6.2387e+01_rb,6.1774e+01_rb,6.0982e+01_rb,5.9987e+01_rb/)
      kbo(:,26,16) = (/ &
     &6.9294e+01_rb,6.8455e+01_rb,6.7400e+01_rb,6.6158e+01_rb,6.4771e+01_rb/)
      kbo(:,27,16) = (/ &
     &7.5369e+01_rb,7.4018e+01_rb,7.2501e+01_rb,7.0820e+01_rb,6.9030e+01_rb/)
      kbo(:,28,16) = (/ &
     &8.0847e+01_rb,7.8971e+01_rb,7.6973e+01_rb,7.4879e+01_rb,7.2699e+01_rb/)
      kbo(:,29,16) = (/ &
     &8.5588e+01_rb,8.3223e+01_rb,8.0777e+01_rb,7.8292e+01_rb,7.5762e+01_rb/)
      kbo(:,30,16) = (/ &
     &8.9575e+01_rb,8.6758e+01_rb,8.3320e+01_rb,8.1073e+01_rb,7.8238e+01_rb/)
      kbo(:,31,16) = (/ &
     &9.2801e+01_rb,8.9596e+01_rb,8.6404e+01_rb,8.3251e+01_rb,8.0171e+01_rb/)
      kbo(:,32,16) = (/ &
     &9.5328e+01_rb,9.1774e+01_rb,8.8298e+01_rb,8.4902e+01_rb,7.6985e+01_rb/)
      kbo(:,33,16) = (/ &
     &9.7221e+01_rb,9.3383e+01_rb,8.9667e+01_rb,8.6065e+01_rb,5.8850e+01_rb/)
      kbo(:,34,16) = (/ &
     &9.8688e+01_rb,9.4629e+01_rb,9.0722e+01_rb,7.0430e+01_rb,3.0605e+01_rb/)
      kbo(:,35,16) = (/ &
     &1.0011e+02_rb,9.5862e+01_rb,8.7207e+01_rb,5.0589e+01_rb,4.4567e+00_rb/)
      kbo(:,36,16) = (/ &
     &1.0168e+02_rb,9.7257e+01_rb,7.6588e+01_rb,3.3945e+01_rb,3.3821e-01_rb/)
      kbo(:,37,16) = (/ &
     &1.0369e+02_rb,9.9078e+01_rb,7.2492e+01_rb,2.6526e+01_rb,3.3765e-01_rb/)
      kbo(:,38,16) = (/ &
     &1.0555e+02_rb,1.0077e+02_rb,6.5594e+01_rb,1.3779e+01_rb,3.3740e-01_rb/)
      kbo(:,39,16) = (/ &
     &1.0726e+02_rb,9.7866e+01_rb,5.5055e+01_rb,3.4196e-01_rb,3.3714e-01_rb/)
      kbo(:,40,16) = (/ &
     &1.0936e+02_rb,1.0037e+02_rb,5.6740e+01_rb,3.4227e-01_rb,3.3798e-01_rb/)
      kbo(:,41,16) = (/ &
     &1.1147e+02_rb,1.0552e+02_rb,5.9119e+01_rb,1.0312e+00_rb,3.3873e-01_rb/)
      kbo(:,42,16) = (/ &
     &1.1349e+02_rb,1.0806e+02_rb,6.1522e+01_rb,2.4025e+00_rb,3.3958e-01_rb/)
      kbo(:,43,16) = (/ &
     &1.1581e+02_rb,1.1021e+02_rb,6.4818e+01_rb,5.9686e+00_rb,3.4048e-01_rb/)
      kbo(:,44,16) = (/ &
     &1.1830e+02_rb,1.1248e+02_rb,7.1730e+01_rb,8.2150e+00_rb,3.4135e-01_rb/)
      kbo(:,45,16) = (/ &
     &1.2075e+02_rb,1.1473e+02_rb,7.7444e+01_rb,1.2041e+01_rb,3.1303e-01_rb/)
      kbo(:,46,16) = (/ &
     &1.2335e+02_rb,1.1716e+02_rb,8.1949e+01_rb,1.5245e+01_rb,3.1351e-01_rb/)
      kbo(:,47,16) = (/ &
     &1.2629e+02_rb,1.1987e+02_rb,8.8206e+01_rb,2.5272e+01_rb,3.1369e-01_rb/)
      kbo(:,48,16) = (/ &
     &1.2931e+02_rb,1.2262e+02_rb,9.4686e+01_rb,2.9706e+01_rb,3.1394e-01_rb/)
      kbo(:,49,16) = (/ &
     &1.3237e+02_rb,1.2544e+02_rb,9.8342e+01_rb,3.1959e+01_rb,3.1361e-01_rb/)
      kbo(:,50,16) = (/ &
     &1.3530e+02_rb,1.2816e+02_rb,1.0881e+02_rb,3.8226e+01_rb,3.1326e-01_rb/)
      kbo(:,51,16) = (/ &
     &1.3828e+02_rb,1.3089e+02_rb,1.1491e+02_rb,4.8666e+01_rb,3.1193e-01_rb/)
      kbo(:,52,16) = (/ &
     &1.4134e+02_rb,1.3361e+02_rb,1.2195e+02_rb,5.9393e+01_rb,3.1051e-01_rb/)
      kbo(:,53,16) = (/ &
     &1.4442e+02_rb,1.3646e+02_rb,1.2918e+02_rb,6.3097e+01_rb,3.0922e-01_rb/)
      kbo(:,54,16) = (/ &
     &1.4741e+02_rb,1.3916e+02_rb,1.3165e+02_rb,7.8318e+01_rb,3.0699e-01_rb/)
      kbo(:,55,16) = (/ &
     &1.5037e+02_rb,1.4182e+02_rb,1.3403e+02_rb,1.0259e+02_rb,2.6611e+01_rb/)
      kbo(:,56,16) = (/ &
     &1.5341e+02_rb,1.4449e+02_rb,1.3650e+02_rb,1.2291e+02_rb,5.4887e+01_rb/)
      kbo(:,57,16) = (/ &
     &1.5661e+02_rb,1.4731e+02_rb,1.3904e+02_rb,1.3152e+02_rb,7.9860e+01_rb/)
      kbo(:,58,16) = (/ &
     &1.5976e+02_rb,1.5007e+02_rb,1.4153e+02_rb,1.3378e+02_rb,1.0477e+02_rb/)
      kbo(:,59,16) = (/ &
     &1.6110e+02_rb,1.5124e+02_rb,1.4255e+02_rb,1.3470e+02_rb,1.2755e+02_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kao_mco2(:, 1) = (/ &
     & 8.88964e-07_rb, 1.13087e-06_rb, 1.43861e-06_rb, 1.83010e-06_rb, 2.32811e-06_rb, &
     & 2.96165e-06_rb, 3.76760e-06_rb, 4.79286e-06_rb, 6.09712e-06_rb, 7.75630e-06_rb, &
     & 9.86699e-06_rb, 1.25521e-05_rb, 1.59678e-05_rb, 2.03130e-05_rb, 2.58407e-05_rb, &
     & 3.28727e-05_rb, 4.18182e-05_rb, 5.31980e-05_rb, 6.76745e-05_rb/)
      kao_mco2(:, 2) = (/ &
     & 1.10492e-05_rb, 1.35003e-05_rb, 1.64952e-05_rb, 2.01545e-05_rb, 2.46256e-05_rb, &
     & 3.00885e-05_rb, 3.67632e-05_rb, 4.49188e-05_rb, 5.48835e-05_rb, 6.70588e-05_rb, &
     & 8.19351e-05_rb, 1.00111e-04_rb, 1.22320e-04_rb, 1.49455e-04_rb, 1.82610e-04_rb, &
     & 2.23121e-04_rb, 2.72618e-04_rb, 3.33095e-04_rb, 4.06988e-04_rb/)
      kao_mco2(:, 3) = (/ &
     & 1.51034e-05_rb, 1.81249e-05_rb, 2.17508e-05_rb, 2.61020e-05_rb, 3.13238e-05_rb, &
     & 3.75901e-05_rb, 4.51101e-05_rb, 5.41344e-05_rb, 6.49640e-05_rb, 7.79601e-05_rb, &
     & 9.35562e-05_rb, 1.12272e-04_rb, 1.34732e-04_rb, 1.61686e-04_rb, 1.94031e-04_rb, &
     & 2.32847e-04_rb, 2.79429e-04_rb, 3.35329e-04_rb, 4.02411e-04_rb/)
      kao_mco2(:, 4) = (/ &
     & 1.57088e-05_rb, 1.89537e-05_rb, 2.28688e-05_rb, 2.75928e-05_rb, 3.32924e-05_rb, &
     & 4.01695e-05_rb, 4.84671e-05_rb, 5.84787e-05_rb, 7.05584e-05_rb, 8.51332e-05_rb, &
     & 1.02719e-04_rb, 1.23937e-04_rb, 1.49538e-04_rb, 1.80427e-04_rb, 2.17697e-04_rb, &
     & 2.62666e-04_rb, 3.16923e-04_rb, 3.82388e-04_rb, 4.61376e-04_rb/)
      kao_mco2(:, 5) = (/ &
     & 3.09299e-05_rb, 3.73196e-05_rb, 4.50294e-05_rb, 5.43320e-05_rb, 6.55563e-05_rb, &
     & 7.90995e-05_rb, 9.54405e-05_rb, 1.15157e-04_rb, 1.38948e-04_rb, 1.67652e-04_rb, &
     & 2.02288e-04_rb, 2.44078e-04_rb, 2.94501e-04_rb, 3.55342e-04_rb, 4.28751e-04_rb, &
     & 5.17327e-04_rb, 6.24200e-04_rb, 7.53153e-04_rb, 9.08745e-04_rb/)
      kao_mco2(:, 6) = (/ &
     & 1.98653e-05_rb, 2.38878e-05_rb, 2.87248e-05_rb, 3.45413e-05_rb, 4.15355e-05_rb, &
     & 4.99459e-05_rb, 6.00593e-05_rb, 7.22206e-05_rb, 8.68445e-05_rb, 1.04429e-04_rb, &
     & 1.25575e-04_rb, 1.51003e-04_rb, 1.81579e-04_rb, 2.18346e-04_rb, 2.62559e-04_rb, &
     & 3.15724e-04_rb, 3.79654e-04_rb, 4.56529e-04_rb, 5.48971e-04_rb/)
      kao_mco2(:, 7) = (/ &
     & 1.54276e-06_rb, 1.90144e-06_rb, 2.34351e-06_rb, 2.88836e-06_rb, 3.55989e-06_rb, &
     & 4.38754e-06_rb, 5.40761e-06_rb, 6.66485e-06_rb, 8.21439e-06_rb, 1.01242e-05_rb, &
     & 1.24780e-05_rb, 1.53790e-05_rb, 1.89546e-05_rb, 2.33614e-05_rb, 2.87928e-05_rb, &
     & 3.54869e-05_rb, 4.37374e-05_rb, 5.39060e-05_rb, 6.64388e-05_rb/)
      kao_mco2(:, 8) = (/ &
     & 1.66907e-06_rb, 2.11106e-06_rb, 2.67008e-06_rb, 3.37714e-06_rb, 4.27143e-06_rb, &
     & 5.40254e-06_rb, 6.83318e-06_rb, 8.64266e-06_rb, 1.09313e-05_rb, 1.38260e-05_rb, &
     & 1.74872e-05_rb, 2.21180e-05_rb, 2.79750e-05_rb, 3.53830e-05_rb, 4.47527e-05_rb, &
     & 5.66036e-05_rb, 7.15927e-05_rb, 9.05509e-05_rb, 1.14529e-04_rb/)
      kao_mco2(:, 9) = (/ &
     & 1.22817e-06_rb, 1.56416e-06_rb, 1.99206e-06_rb, 2.53703e-06_rb, 3.23108e-06_rb, &
     & 4.11501e-06_rb, 5.24074e-06_rb, 6.67445e-06_rb, 8.50037e-06_rb, 1.08258e-05_rb, &
     & 1.37874e-05_rb, 1.75592e-05_rb, 2.23629e-05_rb, 2.84807e-05_rb, 3.62721e-05_rb, &
     & 4.61950e-05_rb, 5.88325e-05_rb, 7.49272e-05_rb, 9.54249e-05_rb/)
      kao_mco2(:,10) = (/ &
     & 3.45943e-08_rb, 3.84726e-08_rb, 4.27856e-08_rb, 4.75821e-08_rb, 5.29164e-08_rb, &
     & 5.88487e-08_rb, 6.54460e-08_rb, 7.27829e-08_rb, 8.09423e-08_rb, 9.00164e-08_rb, &
     & 1.00108e-07_rb, 1.11331e-07_rb, 1.23811e-07_rb, 1.37691e-07_rb, 1.53128e-07_rb, &
     & 1.70294e-07_rb, 1.89385e-07_rb, 2.10616e-07_rb, 2.34228e-07_rb/)
      kao_mco2(:,11) = (/ &
     & 2.89971e-08_rb, 3.35110e-08_rb, 3.87275e-08_rb, 4.47561e-08_rb, 5.17230e-08_rb, &
     & 5.97745e-08_rb, 6.90794e-08_rb, 7.98327e-08_rb, 9.22599e-08_rb, 1.06622e-07_rb, &
     & 1.23219e-07_rb, 1.42400e-07_rb, 1.64567e-07_rb, 1.90184e-07_rb, 2.19789e-07_rb, &
     & 2.54003e-07_rb, 2.93542e-07_rb, 3.39237e-07_rb, 3.92044e-07_rb/)
      kao_mco2(:,12) = (/ &
     & 2.51330e-08_rb, 2.96783e-08_rb, 3.50457e-08_rb, 4.13837e-08_rb, 4.88679e-08_rb, &
     & 5.77056e-08_rb, 6.81416e-08_rb, 8.04650e-08_rb, 9.50171e-08_rb, 1.12201e-07_rb, &
     & 1.32492e-07_rb, 1.56454e-07_rb, 1.84748e-07_rb, 2.18160e-07_rb, 2.57614e-07_rb, &
     & 3.04203e-07_rb, 3.59218e-07_rb, 4.24182e-07_rb, 5.00895e-07_rb/)
      kao_mco2(:,13) = (/ &
     & 1.16966e-07_rb, 1.13960e-07_rb, 1.11032e-07_rb, 1.08179e-07_rb, 1.05400e-07_rb, &
     & 1.02691e-07_rb, 1.00053e-07_rb, 9.74820e-08_rb, 9.49772e-08_rb, 9.25368e-08_rb, &
     & 9.01591e-08_rb, 8.78425e-08_rb, 8.55854e-08_rb, 8.33863e-08_rb, 8.12437e-08_rb, &
     & 7.91562e-08_rb, 7.71223e-08_rb, 7.51407e-08_rb, 7.32100e-08_rb/)
      kao_mco2(:,14) = (/ &
     & 9.17853e-08_rb, 8.94322e-08_rb, 8.71395e-08_rb, 8.49055e-08_rb, 8.27289e-08_rb, &
     & 8.06080e-08_rb, 7.85415e-08_rb, 7.65279e-08_rb, 7.45660e-08_rb, 7.26544e-08_rb, &
     & 7.07918e-08_rb, 6.89770e-08_rb, 6.72086e-08_rb, 6.54856e-08_rb, 6.38068e-08_rb, &
     & 6.21710e-08_rb, 6.05772e-08_rb, 5.90242e-08_rb, 5.75110e-08_rb/)
      kao_mco2(:,15) = (/ &
     & 8.34607e-08_rb, 8.13236e-08_rb, 7.92413e-08_rb, 7.72122e-08_rb, 7.52351e-08_rb, &
     & 7.33087e-08_rb, 7.14315e-08_rb, 6.96025e-08_rb, 6.78202e-08_rb, 6.60837e-08_rb, &
     & 6.43915e-08_rb, 6.27427e-08_rb, 6.11361e-08_rb, 5.95707e-08_rb, 5.80453e-08_rb, &
     & 5.65590e-08_rb, 5.51108e-08_rb, 5.36996e-08_rb, 5.23246e-08_rb/)
      kao_mco2(:,16) = (/ &
     & 8.34607e-08_rb, 8.13236e-08_rb, 7.92413e-08_rb, 7.72122e-08_rb, 7.52351e-08_rb, &
     & 7.33087e-08_rb, 7.14315e-08_rb, 6.96025e-08_rb, 6.78202e-08_rb, 6.60837e-08_rb, &
     & 6.43915e-08_rb, 6.27427e-08_rb, 6.11361e-08_rb, 5.95707e-08_rb, 5.80453e-08_rb, &
     & 5.65590e-08_rb, 5.51108e-08_rb, 5.36996e-08_rb, 5.23246e-08_rb/)
      kao_mo3(:, 1) = (/ &
     & 1.18276e-01_rb, 1.18009e-01_rb, 1.17742e-01_rb, 1.17476e-01_rb, 1.17210e-01_rb, &
     & 1.16945e-01_rb, 1.16681e-01_rb, 1.16417e-01_rb, 1.16153e-01_rb, 1.15891e-01_rb, &
     & 1.15629e-01_rb, 1.15367e-01_rb, 1.15106e-01_rb, 1.14846e-01_rb, 1.14586e-01_rb, &
     & 1.14327e-01_rb, 1.14069e-01_rb, 1.13811e-01_rb, 1.13553e-01_rb/)
      kao_mo3(:, 2) = (/ &
     & 1.83777e-01_rb, 1.84268e-01_rb, 1.84761e-01_rb, 1.85255e-01_rb, 1.85751e-01_rb, &
     & 1.86248e-01_rb, 1.86746e-01_rb, 1.87245e-01_rb, 1.87746e-01_rb, 1.88248e-01_rb, &
     & 1.88752e-01_rb, 1.89257e-01_rb, 1.89763e-01_rb, 1.90270e-01_rb, 1.90779e-01_rb, &
     & 1.91290e-01_rb, 1.91801e-01_rb, 1.92314e-01_rb, 1.92829e-01_rb/)
      kao_mo3(:, 3) = (/ &
     & 2.33414e-01_rb, 2.34511e-01_rb, 2.35614e-01_rb, 2.36722e-01_rb, 2.37836e-01_rb, &
     & 2.38954e-01_rb, 2.40078e-01_rb, 2.41207e-01_rb, 2.42342e-01_rb, 2.43481e-01_rb, &
     & 2.44626e-01_rb, 2.45777e-01_rb, 2.46933e-01_rb, 2.48094e-01_rb, 2.49261e-01_rb, &
     & 2.50433e-01_rb, 2.51611e-01_rb, 2.52794e-01_rb, 2.53983e-01_rb/)
      kao_mo3(:, 4) = (/ &
     & 2.84906e-01_rb, 2.87358e-01_rb, 2.89832e-01_rb, 2.92328e-01_rb, 2.94844e-01_rb, &
     & 2.97383e-01_rb, 2.99943e-01_rb, 3.02525e-01_rb, 3.05130e-01_rb, 3.07757e-01_rb, &
     & 3.10406e-01_rb, 3.13078e-01_rb, 3.15774e-01_rb, 3.18492e-01_rb, 3.21234e-01_rb, &
     & 3.24000e-01_rb, 3.26789e-01_rb, 3.29603e-01_rb, 3.32440e-01_rb/)
      kao_mo3(:, 5) = (/ &
     & 3.40508e-01_rb, 3.44095e-01_rb, 3.47720e-01_rb, 3.51383e-01_rb, 3.55084e-01_rb, &
     & 3.58824e-01_rb, 3.62604e-01_rb, 3.66424e-01_rb, 3.70284e-01_rb, 3.74184e-01_rb, &
     & 3.78126e-01_rb, 3.82109e-01_rb, 3.86134e-01_rb, 3.90202e-01_rb, 3.94312e-01_rb, &
     & 3.98466e-01_rb, 4.02663e-01_rb, 4.06905e-01_rb, 4.11191e-01_rb/)
      kao_mo3(:, 6) = (/ &
     & 3.78368e-01_rb, 3.83690e-01_rb, 3.89086e-01_rb, 3.94558e-01_rb, 4.00107e-01_rb, &
     & 4.05735e-01_rb, 4.11441e-01_rb, 4.17227e-01_rb, 4.23095e-01_rb, 4.29046e-01_rb, &
     & 4.35080e-01_rb, 4.41199e-01_rb, 4.47404e-01_rb, 4.53697e-01_rb, 4.60078e-01_rb, &
     & 4.66548e-01_rb, 4.73110e-01_rb, 4.79764e-01_rb, 4.86511e-01_rb/)
      kao_mo3(:, 7) = (/ &
     & 4.51965e-01_rb, 4.58461e-01_rb, 4.65051e-01_rb, 4.71735e-01_rb, 4.78516e-01_rb, &
     & 4.85394e-01_rb, 4.92371e-01_rb, 4.99448e-01_rb, 5.06627e-01_rb, 5.13909e-01_rb, &
     & 5.21296e-01_rb, 5.28789e-01_rb, 5.36390e-01_rb, 5.44100e-01_rb, 5.51920e-01_rb, &
     & 5.59854e-01_rb, 5.67901e-01_rb, 5.76064e-01_rb, 5.84344e-01_rb/)
      kao_mo3(:, 8) = (/ &
     & 3.00557e-01_rb, 3.03974e-01_rb, 3.07430e-01_rb, 3.10925e-01_rb, 3.14460e-01_rb, &
     & 3.18035e-01_rb, 3.21651e-01_rb, 3.25307e-01_rb, 3.29006e-01_rb, 3.32746e-01_rb, &
     & 3.36529e-01_rb, 3.40355e-01_rb, 3.44224e-01_rb, 3.48137e-01_rb, 3.52095e-01_rb, &
     & 3.56098e-01_rb, 3.60146e-01_rb, 3.64241e-01_rb, 3.68381e-01_rb/)
      kao_mo3(:, 9) = (/ &
     & 2.10042e-01_rb, 2.12905e-01_rb, 2.15806e-01_rb, 2.18748e-01_rb, 2.21729e-01_rb, &
     & 2.24751e-01_rb, 2.27814e-01_rb, 2.30919e-01_rb, 2.34066e-01_rb, 2.37256e-01_rb, &
     & 2.40489e-01_rb, 2.43767e-01_rb, 2.47089e-01_rb, 2.50457e-01_rb, 2.53870e-01_rb, &
     & 2.57330e-01_rb, 2.60837e-01_rb, 2.64392e-01_rb, 2.67996e-01_rb/)
      kao_mo3(:,10) = (/ &
     & 2.09288e-01_rb, 2.11759e-01_rb, 2.14259e-01_rb, 2.16789e-01_rb, 2.19349e-01_rb, &
     & 2.21939e-01_rb, 2.24559e-01_rb, 2.27210e-01_rb, 2.29893e-01_rb, 2.32607e-01_rb, &
     & 2.35354e-01_rb, 2.38133e-01_rb, 2.40944e-01_rb, 2.43789e-01_rb, 2.46667e-01_rb, &
     & 2.49580e-01_rb, 2.52527e-01_rb, 2.55508e-01_rb, 2.58525e-01_rb/)
      kao_mo3(:,11) = (/ &
     & 2.28947e-01_rb, 2.30609e-01_rb, 2.32283e-01_rb, 2.33969e-01_rb, 2.35667e-01_rb, &
     & 2.37378e-01_rb, 2.39101e-01_rb, 2.40836e-01_rb, 2.42584e-01_rb, 2.44345e-01_rb, &
     & 2.46118e-01_rb, 2.47905e-01_rb, 2.49704e-01_rb, 2.51516e-01_rb, 2.53342e-01_rb, &
     & 2.55181e-01_rb, 2.57033e-01_rb, 2.58899e-01_rb, 2.60778e-01_rb/)
      kao_mo3(:,12) = (/ &
     & 2.57263e-01_rb, 2.58272e-01_rb, 2.59285e-01_rb, 2.60302e-01_rb, 2.61323e-01_rb, &
     & 2.62347e-01_rb, 2.63376e-01_rb, 2.64409e-01_rb, 2.65446e-01_rb, 2.66487e-01_rb, &
     & 2.67532e-01_rb, 2.68581e-01_rb, 2.69635e-01_rb, 2.70692e-01_rb, 2.71753e-01_rb, &
     & 2.72819e-01_rb, 2.73889e-01_rb, 2.74963e-01_rb, 2.76042e-01_rb/)
      kao_mo3(:,13) = (/ &
     & 2.43322e-01_rb, 2.45918e-01_rb, 2.48541e-01_rb, 2.51192e-01_rb, 2.53872e-01_rb, &
     & 2.56580e-01_rb, 2.59317e-01_rb, 2.62083e-01_rb, 2.64879e-01_rb, 2.67704e-01_rb, &
     & 2.70560e-01_rb, 2.73446e-01_rb, 2.76363e-01_rb, 2.79311e-01_rb, 2.82290e-01_rb, &
     & 2.85302e-01_rb, 2.88345e-01_rb, 2.91421e-01_rb, 2.94529e-01_rb/)
      kao_mo3(:,14) = (/ &
     & 2.10568e-01_rb, 2.16529e-01_rb, 2.22660e-01_rb, 2.28964e-01_rb, 2.35446e-01_rb, &
     & 2.42113e-01_rb, 2.48967e-01_rb, 2.56016e-01_rb, 2.63265e-01_rb, 2.70719e-01_rb, &
     & 2.78383e-01_rb, 2.86265e-01_rb, 2.94370e-01_rb, 3.02704e-01_rb, 3.11275e-01_rb, &
     & 3.20088e-01_rb, 3.29150e-01_rb, 3.38470e-01_rb, 3.48052e-01_rb/)
      kao_mo3(:,15) = (/ &
     & 2.60406e-02_rb, 2.78779e-02_rb, 2.98448e-02_rb, 3.19505e-02_rb, 3.42048e-02_rb, &
     & 3.66181e-02_rb, 3.92017e-02_rb, 4.19675e-02_rb, 4.49285e-02_rb, 4.80985e-02_rb, &
     & 5.14920e-02_rb, 5.51250e-02_rb, 5.90143e-02_rb, 6.31781e-02_rb, 6.76356e-02_rb, &
     & 7.24076e-02_rb, 7.75163e-02_rb, 8.29854e-02_rb, 8.88404e-02_rb/)
      kao_mo3(:,16) = (/ &
     & 2.31483e-02_rb, 2.46840e-02_rb, 2.63217e-02_rb, 2.80681e-02_rb, 2.99302e-02_rb, &
     & 3.19160e-02_rb, 3.40335e-02_rb, 3.62914e-02_rb, 3.86992e-02_rb, 4.12668e-02_rb, &
     & 4.40046e-02_rb, 4.69242e-02_rb, 5.00374e-02_rb, 5.33571e-02_rb, 5.68971e-02_rb, &
     & 6.06720e-02_rb, 6.46974e-02_rb, 6.89897e-02_rb, 7.35669e-02_rb/)
      kao_mn2o(:, 1) = (/ &
     & 3.02276e-02_rb, 3.10321e-02_rb, 3.18580e-02_rb, 3.27059e-02_rb, 3.35764e-02_rb, &
     & 3.44700e-02_rb, 3.53875e-02_rb, 3.63293e-02_rb, 3.72962e-02_rb, 3.82889e-02_rb, &
     & 3.93079e-02_rb, 4.03541e-02_rb, 4.14281e-02_rb, 4.25307e-02_rb, 4.36627e-02_rb, &
     & 4.48248e-02_rb, 4.60178e-02_rb, 4.72425e-02_rb, 4.84999e-02_rb/)
      kao_mn2o(:, 2) = (/ &
     & 6.10132e-02_rb, 6.17435e-02_rb, 6.24825e-02_rb, 6.32304e-02_rb, 6.39872e-02_rb, &
     & 6.47531e-02_rb, 6.55281e-02_rb, 6.63124e-02_rb, 6.71061e-02_rb, 6.79093e-02_rb, &
     & 6.87221e-02_rb, 6.95446e-02_rb, 7.03770e-02_rb, 7.12194e-02_rb, 7.20718e-02_rb, &
     & 7.29344e-02_rb, 7.38074e-02_rb, 7.46908e-02_rb, 7.55848e-02_rb/)
      kao_mn2o(:, 3) = (/ &
     & 1.04479e-01_rb, 1.05566e-01_rb, 1.06664e-01_rb, 1.07774e-01_rb, 1.08895e-01_rb, &
     & 1.10028e-01_rb, 1.11173e-01_rb, 1.12329e-01_rb, 1.13498e-01_rb, 1.14679e-01_rb, &
     & 1.15872e-01_rb, 1.17077e-01_rb, 1.18295e-01_rb, 1.19526e-01_rb, 1.20770e-01_rb, &
     & 1.22026e-01_rb, 1.23296e-01_rb, 1.24578e-01_rb, 1.25875e-01_rb/)
      kao_mn2o(:, 4) = (/ &
     & 2.07260e-01_rb, 2.08126e-01_rb, 2.08996e-01_rb, 2.09869e-01_rb, 2.10746e-01_rb, &
     & 2.11627e-01_rb, 2.12511e-01_rb, 2.13399e-01_rb, 2.14291e-01_rb, 2.15187e-01_rb, &
     & 2.16086e-01_rb, 2.16989e-01_rb, 2.17896e-01_rb, 2.18807e-01_rb, 2.19721e-01_rb, &
     & 2.20640e-01_rb, 2.21562e-01_rb, 2.22488e-01_rb, 2.23418e-01_rb/)
      kao_mn2o(:, 5) = (/ &
     & 3.71566e-01_rb, 3.71353e-01_rb, 3.71141e-01_rb, 3.70928e-01_rb, 3.70716e-01_rb, &
     & 3.70504e-01_rb, 3.70292e-01_rb, 3.70080e-01_rb, 3.69869e-01_rb, 3.69657e-01_rb, &
     & 3.69446e-01_rb, 3.69234e-01_rb, 3.69023e-01_rb, 3.68812e-01_rb, 3.68601e-01_rb, &
     & 3.68390e-01_rb, 3.68179e-01_rb, 3.67969e-01_rb, 3.67758e-01_rb/)
      kao_mn2o(:, 6) = (/ &
     & 5.28092e-01_rb, 5.27262e-01_rb, 5.26433e-01_rb, 5.25605e-01_rb, 5.24779e-01_rb, &
     & 5.23954e-01_rb, 5.23130e-01_rb, 5.22307e-01_rb, 5.21486e-01_rb, 5.20666e-01_rb, &
     & 5.19847e-01_rb, 5.19030e-01_rb, 5.18214e-01_rb, 5.17399e-01_rb, 5.16586e-01_rb, &
     & 5.15773e-01_rb, 5.14962e-01_rb, 5.14153e-01_rb, 5.13344e-01_rb/)
      kao_mn2o(:, 7) = (/ &
     & 3.88140e-01_rb, 3.87956e-01_rb, 3.87773e-01_rb, 3.87590e-01_rb, 3.87407e-01_rb, &
     & 3.87224e-01_rb, 3.87041e-01_rb, 3.86858e-01_rb, 3.86675e-01_rb, 3.86492e-01_rb, &
     & 3.86310e-01_rb, 3.86127e-01_rb, 3.85945e-01_rb, 3.85763e-01_rb, 3.85580e-01_rb, &
     & 3.85398e-01_rb, 3.85216e-01_rb, 3.85034e-01_rb, 3.84852e-01_rb/)
      kao_mn2o(:, 8) = (/ &
     & 3.12991e-01_rb, 3.12246e-01_rb, 3.11504e-01_rb, 3.10763e-01_rb, 3.10024e-01_rb, &
     & 3.09287e-01_rb, 3.08552e-01_rb, 3.07818e-01_rb, 3.07086e-01_rb, 3.06356e-01_rb, &
     & 3.05628e-01_rb, 3.04901e-01_rb, 3.04176e-01_rb, 3.03453e-01_rb, 3.02732e-01_rb, &
     & 3.02012e-01_rb, 3.01294e-01_rb, 3.00577e-01_rb, 2.99863e-01_rb/)
      kao_mn2o(:, 9) = (/ &
     & 4.11761e-01_rb, 4.11309e-01_rb, 4.10858e-01_rb, 4.10407e-01_rb, 4.09957e-01_rb, &
     & 4.09507e-01_rb, 4.09057e-01_rb, 4.08608e-01_rb, 4.08160e-01_rb, 4.07712e-01_rb, &
     & 4.07265e-01_rb, 4.06818e-01_rb, 4.06371e-01_rb, 4.05925e-01_rb, 4.05480e-01_rb, &
     & 4.05035e-01_rb, 4.04590e-01_rb, 4.04146e-01_rb, 4.03703e-01_rb/)
      kao_mn2o(:,10) = (/ &
     & 2.84648e-01_rb, 2.87025e-01_rb, 2.89421e-01_rb, 2.91838e-01_rb, 2.94275e-01_rb, &
     & 2.96732e-01_rb, 2.99210e-01_rb, 3.01708e-01_rb, 3.04227e-01_rb, 3.06768e-01_rb, &
     & 3.09329e-01_rb, 3.11912e-01_rb, 3.14517e-01_rb, 3.17143e-01_rb, 3.19791e-01_rb, &
     & 3.22461e-01_rb, 3.25154e-01_rb, 3.27869e-01_rb, 3.30606e-01_rb/)
      kao_mn2o(:,11) = (/ &
     & 2.75090e-01_rb, 2.79370e-01_rb, 2.83716e-01_rb, 2.88129e-01_rb, 2.92611e-01_rb, &
     & 2.97163e-01_rb, 3.01786e-01_rb, 3.06480e-01_rb, 3.11248e-01_rb, 3.16090e-01_rb, &
     & 3.21007e-01_rb, 3.26001e-01_rb, 3.31072e-01_rb, 3.36222e-01_rb, 3.41452e-01_rb, &
     & 3.46764e-01_rb, 3.52158e-01_rb, 3.57636e-01_rb, 3.63200e-01_rb/)
      kao_mn2o(:,12) = (/ &
     & 1.67753e-01_rb, 1.71386e-01_rb, 1.75098e-01_rb, 1.78890e-01_rb, 1.82765e-01_rb, &
     & 1.86723e-01_rb, 1.90767e-01_rb, 1.94899e-01_rb, 1.99120e-01_rb, 2.03433e-01_rb, &
     & 2.07839e-01_rb, 2.12340e-01_rb, 2.16939e-01_rb, 2.21638e-01_rb, 2.26438e-01_rb, &
     & 2.31342e-01_rb, 2.36353e-01_rb, 2.41472e-01_rb, 2.46701e-01_rb/)
      kao_mn2o(:,13) = (/ &
     & 1.40543e-01_rb, 1.42049e-01_rb, 1.43571e-01_rb, 1.45109e-01_rb, 1.46663e-01_rb, &
     & 1.48234e-01_rb, 1.49822e-01_rb, 1.51427e-01_rb, 1.53049e-01_rb, 1.54689e-01_rb, &
     & 1.56346e-01_rb, 1.58021e-01_rb, 1.59713e-01_rb, 1.61424e-01_rb, 1.63153e-01_rb, &
     & 1.64901e-01_rb, 1.66668e-01_rb, 1.68453e-01_rb, 1.70258e-01_rb/)
      kao_mn2o(:,14) = (/ &
     & 1.51530e-01_rb, 1.50944e-01_rb, 1.50360e-01_rb, 1.49779e-01_rb, 1.49199e-01_rb, &
     & 1.48622e-01_rb, 1.48047e-01_rb, 1.47474e-01_rb, 1.46903e-01_rb, 1.46335e-01_rb, &
     & 1.45769e-01_rb, 1.45205e-01_rb, 1.44643e-01_rb, 1.44083e-01_rb, 1.43526e-01_rb, &
     & 1.42971e-01_rb, 1.42418e-01_rb, 1.41867e-01_rb, 1.41318e-01_rb/)
      kao_mn2o(:,15) = (/ &
     & 2.20492e-01_rb, 2.16479e-01_rb, 2.12539e-01_rb, 2.08671e-01_rb, 2.04873e-01_rb, &
     & 2.01145e-01_rb, 1.97484e-01_rb, 1.93890e-01_rb, 1.90361e-01_rb, 1.86897e-01_rb, &
     & 1.83495e-01_rb, 1.80156e-01_rb, 1.76877e-01_rb, 1.73658e-01_rb, 1.70497e-01_rb, &
     & 1.67394e-01_rb, 1.64348e-01_rb, 1.61356e-01_rb, 1.58420e-01_rb/)
      kao_mn2o(:,16) = (/ &
     & 2.19848e-01_rb, 2.15847e-01_rb, 2.11919e-01_rb, 2.08062e-01_rb, 2.04275e-01_rb, &
     & 2.00558e-01_rb, 1.96908e-01_rb, 1.93324e-01_rb, 1.89806e-01_rb, 1.86351e-01_rb, &
     & 1.82960e-01_rb, 1.79630e-01_rb, 1.76361e-01_rb, 1.73151e-01_rb, 1.70000e-01_rb, &
     & 1.66906e-01_rb, 1.63868e-01_rb, 1.60886e-01_rb, 1.57958e-01_rb/)

!     The array KBO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level above 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kbo_mco2(:, 1) = (/ &
     & 4.74280e-08_rb, 6.62724e-08_rb, 9.26042e-08_rb, 1.29398e-07_rb, 1.80812e-07_rb, &
     & 2.52653e-07_rb, 3.53039e-07_rb, 4.93310e-07_rb, 6.89316e-07_rb, 9.63198e-07_rb, &
     & 1.34590e-06_rb, 1.88067e-06_rb, 2.62790e-06_rb, 3.67204e-06_rb, 5.13104e-06_rb, &
     & 7.16974e-06_rb, 1.00185e-05_rb, 1.39991e-05_rb, 1.95613e-05_rb/)
      kbo_mco2(:, 2) = (/ &
     & 1.14872e-07_rb, 1.63356e-07_rb, 2.32304e-07_rb, 3.30352e-07_rb, 4.69783e-07_rb, &
     & 6.68064e-07_rb, 9.50033e-07_rb, 1.35101e-06_rb, 1.92123e-06_rb, 2.73213e-06_rb, &
     & 3.88527e-06_rb, 5.52513e-06_rb, 7.85711e-06_rb, 1.11734e-05_rb, 1.58893e-05_rb, &
     & 2.25957e-05_rb, 3.21326e-05_rb, 4.56948e-05_rb, 6.49811e-05_rb/)
      kbo_mco2(:, 3) = (/ &
     & 3.30676e-07_rb, 4.76313e-07_rb, 6.86094e-07_rb, 9.88267e-07_rb, 1.42353e-06_rb, &
     & 2.05048e-06_rb, 2.95356e-06_rb, 4.25439e-06_rb, 6.12813e-06_rb, 8.82711e-06_rb, &
     & 1.27148e-05_rb, 1.83147e-05_rb, 2.63810e-05_rb, 3.79998e-05_rb, 5.47359e-05_rb, &
     & 7.88430e-05_rb, 1.13568e-04_rb, 1.63585e-04_rb, 2.35632e-04_rb/)
      kbo_mco2(:, 4) = (/ &
     & 6.58642e-07_rb, 9.52761e-07_rb, 1.37822e-06_rb, 1.99368e-06_rb, 2.88396e-06_rb, &
     & 4.17181e-06_rb, 6.03475e-06_rb, 8.72960e-06_rb, 1.26279e-05_rb, 1.82669e-05_rb, &
     & 2.64241e-05_rb, 3.82239e-05_rb, 5.52929e-05_rb, 7.99844e-05_rb, 1.15702e-04_rb, &
     & 1.67369e-04_rb, 2.42109e-04_rb, 3.50223e-04_rb, 5.06617e-04_rb/)
      kbo_mco2(:, 5) = (/ &
     & 1.26418e-06_rb, 1.82095e-06_rb, 2.62292e-06_rb, 3.77810e-06_rb, 5.44204e-06_rb, &
     & 7.83881e-06_rb, 1.12911e-05_rb, 1.62640e-05_rb, 2.34269e-05_rb, 3.37445e-05_rb, &
     & 4.86061e-05_rb, 7.00131e-05_rb, 1.00848e-04_rb, 1.45263e-04_rb, 2.09239e-04_rb, &
     & 3.01392e-04_rb, 4.34131e-04_rb, 6.25329e-04_rb, 9.00733e-04_rb/)
      kbo_mco2(:, 6) = (/ &
     & 2.38529e-06_rb, 3.43110e-06_rb, 4.93545e-06_rb, 7.09937e-06_rb, 1.02120e-05_rb, &
     & 1.46895e-05_rb, 2.11300e-05_rb, 3.03943e-05_rb, 4.37205e-05_rb, 6.28894e-05_rb, &
     & 9.04630e-05_rb, 1.30126e-04_rb, 1.87179e-04_rb, 2.69247e-04_rb, 3.87296e-04_rb, &
     & 5.57104e-04_rb, 8.01364e-04_rb, 1.15272e-03_rb, 1.65812e-03_rb/)
      kbo_mco2(:, 7) = (/ &
     & 5.41398e-06_rb, 7.54295e-06_rb, 1.05091e-05_rb, 1.46417e-05_rb, 2.03993e-05_rb, &
     & 2.84211e-05_rb, 3.95973e-05_rb, 5.51683e-05_rb, 7.68626e-05_rb, 1.07088e-04_rb, &
     & 1.49199e-04_rb, 2.07869e-04_rb, 2.89610e-04_rb, 4.03496e-04_rb, 5.62165e-04_rb, &
     & 7.83229e-04_rb, 1.09122e-03_rb, 1.52033e-03_rb, 2.11818e-03_rb/)
      kbo_mco2(:, 8) = (/ &
     & 1.09995e-05_rb, 1.54018e-05_rb, 2.15660e-05_rb, 3.01973e-05_rb, 4.22831e-05_rb, &
     & 5.92059e-05_rb, 8.29017e-05_rb, 1.16081e-04_rb, 1.62540e-04_rb, 2.27592e-04_rb, &
     & 3.18681e-04_rb, 4.46226e-04_rb, 6.24817e-04_rb, 8.74886e-04_rb, 1.22504e-03_rb, &
     & 1.71533e-03_rb, 2.40185e-03_rb, 3.36313e-03_rb, 4.70915e-03_rb/)
      kbo_mco2(:, 9) = (/ &
     & 3.29051e-05_rb, 4.59996e-05_rb, 6.43050e-05_rb, 8.98950e-05_rb, 1.25668e-04_rb, &
     & 1.75678e-04_rb, 2.45588e-04_rb, 3.43319e-04_rb, 4.79942e-04_rb, 6.70933e-04_rb, &
     & 9.37930e-04_rb, 1.31118e-03_rb, 1.83295e-03_rb, 2.56237e-03_rb, 3.58206e-03_rb, &
     & 5.00753e-03_rb, 7.00027e-03_rb, 9.78599e-03_rb, 1.36803e-02_rb/)
      kbo_mco2(:,10) = (/ &
     & 1.95126e-05_rb, 2.65944e-05_rb, 3.62463e-05_rb, 4.94013e-05_rb, 6.73305e-05_rb, &
     & 9.17669e-05_rb, 1.25072e-04_rb, 1.70465e-04_rb, 2.32332e-04_rb, 3.16652e-04_rb, &
     & 4.31575e-04_rb, 5.88208e-04_rb, 8.01687e-04_rb, 1.09264e-03_rb, 1.48920e-03_rb, &
     & 2.02968e-03_rb, 2.76631e-03_rb, 3.77029e-03_rb, 5.13865e-03_rb/)
      kbo_mco2(:,11) = (/ &
     & 8.67271e-05_rb, 1.19228e-04_rb, 1.63908e-04_rb, 2.25332e-04_rb, 3.09774e-04_rb, &
     & 4.25860e-04_rb, 5.85450e-04_rb, 8.04845e-04_rb, 1.10646e-03_rb, 1.52110e-03_rb, &
     & 2.09112e-03_rb, 2.87476e-03_rb, 3.95207e-03_rb, 5.43309e-03_rb, 7.46911e-03_rb, &
     & 1.02681e-02_rb, 1.41161e-02_rb, 1.94060e-02_rb, 2.66783e-02_rb/)
      kbo_mco2(:,12) = (/ &
     & 3.79194e-07_rb, 5.51419e-07_rb, 8.01866e-07_rb, 1.16606e-06_rb, 1.69567e-06_rb, &
     & 2.46582e-06_rb, 3.58577e-06_rb, 5.21438e-06_rb, 7.58268e-06_rb, 1.10266e-05_rb, &
     & 1.60348e-05_rb, 2.33176e-05_rb, 3.39081e-05_rb, 4.93087e-05_rb, 7.17040e-05_rb, &
     & 1.04271e-04_rb, 1.51630e-04_rb, 2.20498e-04_rb, 3.20644e-04_rb/)
      kbo_mco2(:,13) = (/ &
     & 1.72555e-07_rb, 2.29952e-07_rb, 3.06441e-07_rb, 4.08373e-07_rb, 5.44209e-07_rb, &
     & 7.25229e-07_rb, 9.66461e-07_rb, 1.28793e-06_rb, 1.71634e-06_rb, 2.28724e-06_rb, &
     & 3.04805e-06_rb, 4.06192e-06_rb, 5.41303e-06_rb, 7.21356e-06_rb, 9.61299e-06_rb, &
     & 1.28106e-05_rb, 1.70717e-05_rb, 2.27503e-05_rb, 3.03177e-05_rb/)
      kbo_mco2(:,14) = (/ &
     & 7.42245e-09_rb, 7.17780e-09_rb, 6.94122e-09_rb, 6.71243e-09_rb, 6.49118e-09_rb, &
     & 6.27723e-09_rb, 6.07032e-09_rb, 5.87024e-09_rb, 5.67675e-09_rb, 5.48964e-09_rb, &
     & 5.30870e-09_rb, 5.13372e-09_rb, 4.96451e-09_rb, 4.80087e-09_rb, 4.64263e-09_rb, &
     & 4.48961e-09_rb, 4.34163e-09_rb, 4.19852e-09_rb, 4.06014e-09_rb/)
      kbo_mco2(:,15) = (/ &
     & 7.41847e-09_rb, 7.17332e-09_rb, 6.93627e-09_rb, 6.70705e-09_rb, 6.48541e-09_rb, &
     & 6.27109e-09_rb, 6.06386e-09_rb, 5.86347e-09_rb, 5.66970e-09_rb, 5.48234e-09_rb, &
     & 5.30117e-09_rb, 5.12599e-09_rb, 4.95659e-09_rb, 4.79280e-09_rb, 4.63441e-09_rb, &
     & 4.48126e-09_rb, 4.33317e-09_rb, 4.18998e-09_rb, 4.05152e-09_rb/)
      kbo_mco2(:,16) = (/ &
     & 7.42855e-09_rb, 7.18278e-09_rb, 6.94513e-09_rb, 6.71535e-09_rb, 6.49317e-09_rb, &
     & 6.27834e-09_rb, 6.07062e-09_rb, 5.86977e-09_rb, 5.67557e-09_rb, 5.48779e-09_rb, &
     & 5.30622e-09_rb, 5.13066e-09_rb, 4.96091e-09_rb, 4.79678e-09_rb, 4.63808e-09_rb, &
     & 4.48462e-09_rb, 4.33625e-09_rb, 4.19278e-09_rb, 4.05406e-09_rb/)
      kbo_mn2o(:, 1) = (/ &
     & 2.49055e-04_rb, 2.53574e-04_rb, 2.58175e-04_rb, 2.62860e-04_rb, 2.67629e-04_rb, &
     & 2.72485e-04_rb, 2.77429e-04_rb, 2.82463e-04_rb, 2.87588e-04_rb, 2.92806e-04_rb, &
     & 2.98119e-04_rb, 3.03528e-04_rb, 3.09036e-04_rb, 3.14643e-04_rb, 3.20352e-04_rb, &
     & 3.26165e-04_rb, 3.32083e-04_rb, 3.38109e-04_rb, 3.44243e-04_rb/)
      kbo_mn2o(:, 2) = (/ &
     & 3.79251e-04_rb, 4.04353e-04_rb, 4.31117e-04_rb, 4.59652e-04_rb, 4.90075e-04_rb, &
     & 5.22513e-04_rb, 5.57097e-04_rb, 5.93970e-04_rb, 6.33284e-04_rb, 6.75200e-04_rb, &
     & 7.19890e-04_rb, 7.67539e-04_rb, 8.18340e-04_rb, 8.72505e-04_rb, 9.30255e-04_rb, &
     & 9.91827e-04_rb, 1.05747e-03_rb, 1.12747e-03_rb, 1.20209e-03_rb/)
      kbo_mn2o(:, 3) = (/ &
     & 7.61140e-04_rb, 8.36483e-04_rb, 9.19284e-04_rb, 1.01028e-03_rb, 1.11029e-03_rb, &
     & 1.22019e-03_rb, 1.34098e-03_rb, 1.47372e-03_rb, 1.61959e-03_rb, 1.77991e-03_rb, &
     & 1.95610e-03_rb, 2.14973e-03_rb, 2.36253e-03_rb, 2.59639e-03_rb, 2.85340e-03_rb, &
     & 3.13585e-03_rb, 3.44626e-03_rb, 3.78740e-03_rb, 4.16230e-03_rb/)
      kbo_mn2o(:, 4) = (/ &
     & 2.01074e-03_rb, 2.26915e-03_rb, 2.56077e-03_rb, 2.88987e-03_rb, 3.26126e-03_rb, &
     & 3.68038e-03_rb, 4.15337e-03_rb, 4.68714e-03_rb, 5.28951e-03_rb, 5.96929e-03_rb, &
     & 6.73643e-03_rb, 7.60217e-03_rb, 8.57916e-03_rb, 9.68172e-03_rb, 1.09260e-02_rb, &
     & 1.23301e-02_rb, 1.39147e-02_rb, 1.57030e-02_rb, 1.77211e-02_rb/)
      kbo_mn2o(:, 5) = (/ &
     & 7.43302e-03_rb, 8.32582e-03_rb, 9.32585e-03_rb, 1.04460e-02_rb, 1.17007e-02_rb, &
     & 1.31061e-02_rb, 1.46803e-02_rb, 1.64436e-02_rb, 1.84186e-02_rb, 2.06309e-02_rb, &
     & 2.31090e-02_rb, 2.58846e-02_rb, 2.89937e-02_rb, 3.24762e-02_rb, 3.63769e-02_rb, &
     & 4.07463e-02_rb, 4.56404e-02_rb, 5.11223e-02_rb, 5.72627e-02_rb/)
      kbo_mn2o(:, 6) = (/ &
     & 2.71911e-02_rb, 2.94258e-02_rb, 3.18441e-02_rb, 3.44612e-02_rb, 3.72933e-02_rb, &
     & 4.03582e-02_rb, 4.36750e-02_rb, 4.72644e-02_rb, 5.11487e-02_rb, 5.53523e-02_rb, &
     & 5.99014e-02_rb, 6.48243e-02_rb, 7.01518e-02_rb, 7.59172e-02_rb, 8.21563e-02_rb, &
     & 8.89082e-02_rb, 9.62150e-02_rb, 1.04122e-01_rb, 1.12679e-01_rb/)
      kbo_mn2o(:, 7) = (/ &
     & 1.63331e-01_rb, 1.80469e-01_rb, 1.99406e-01_rb, 2.20330e-01_rb, 2.43449e-01_rb, &
     & 2.68995e-01_rb, 2.97221e-01_rb, 3.28408e-01_rb, 3.62869e-01_rb, 4.00945e-01_rb, &
     & 4.43017e-01_rb, 4.89503e-01_rb, 5.40867e-01_rb, 5.97621e-01_rb, 6.60330e-01_rb, &
     & 7.29619e-01_rb, 8.06179e-01_rb, 8.90772e-01_rb, 9.84242e-01_rb/)
      kbo_mn2o(:, 8) = (/ &
     & 1.32648e+00_rb, 1.33515e+00_rb, 1.34387e+00_rb, 1.35265e+00_rb, 1.36149e+00_rb, &
     & 1.37038e+00_rb, 1.37933e+00_rb, 1.38835e+00_rb, 1.39742e+00_rb, 1.40655e+00_rb, &
     & 1.41574e+00_rb, 1.42499e+00_rb, 1.43429e+00_rb, 1.44367e+00_rb, 1.45310e+00_rb, &
     & 1.46259e+00_rb, 1.47215e+00_rb, 1.48176e+00_rb, 1.49144e+00_rb/)
      kbo_mn2o(:, 9) = (/ &
     & 3.12620e+00_rb, 3.03118e+00_rb, 2.93905e+00_rb, 2.84972e+00_rb, 2.76310e+00_rb, &
     & 2.67911e+00_rb, 2.59768e+00_rb, 2.51873e+00_rb, 2.44217e+00_rb, 2.36794e+00_rb, &
     & 2.29596e+00_rb, 2.22618e+00_rb, 2.15851e+00_rb, 2.09290e+00_rb, 2.02929e+00_rb, &
     & 1.96761e+00_rb, 1.90780e+00_rb, 1.84982e+00_rb, 1.79359e+00_rb/)
      kbo_mn2o(:,10) = (/ &
     & 1.60677e-02_rb, 1.82485e-02_rb, 2.07254e-02_rb, 2.35384e-02_rb, 2.67332e-02_rb, &
     & 3.03617e-02_rb, 3.44827e-02_rb, 3.91629e-02_rb, 4.44785e-02_rb, 5.05154e-02_rb, &
     & 5.73718e-02_rb, 6.51589e-02_rb, 7.40027e-02_rb, 8.40470e-02_rb, 9.54546e-02_rb, &
     & 1.08411e-01_rb, 1.23125e-01_rb, 1.39836e-01_rb, 1.58816e-01_rb/)
      kbo_mn2o(:,11) = (/ &
     & 1.55287e-02_rb, 1.78265e-02_rb, 2.04642e-02_rb, 2.34922e-02_rb, 2.69683e-02_rb, &
     & 3.09588e-02_rb, 3.55397e-02_rb, 4.07984e-02_rb, 4.68352e-02_rb, 5.37653e-02_rb, &
     & 6.17208e-02_rb, 7.08535e-02_rb, 8.13375e-02_rb, 9.33728e-02_rb, 1.07189e-01_rb, &
     & 1.23049e-01_rb, 1.41257e-01_rb, 1.62158e-01_rb, 1.86152e-01_rb/)
      kbo_mn2o(:,12) = (/ &
     & 7.13719e-03_rb, 8.18879e-03_rb, 9.39535e-03_rb, 1.07797e-02_rb, 1.23680e-02_rb, &
     & 1.41903e-02_rb, 1.62811e-02_rb, 1.86800e-02_rb, 2.14324e-02_rb, 2.45902e-02_rb, &
     & 2.82134e-02_rb, 3.23704e-02_rb, 3.71400e-02_rb, 4.26122e-02_rb, 4.88908e-02_rb, &
     & 5.60945e-02_rb, 6.43596e-02_rb, 7.38424e-02_rb, 8.47224e-02_rb/)
      kbo_mn2o(:,13) = (/ &
     & 9.28813e-03_rb, 1.06108e-02_rb, 1.21218e-02_rb, 1.38480e-02_rb, 1.58199e-02_rb, &
     & 1.80727e-02_rb, 2.06463e-02_rb, 2.35864e-02_rb, 2.69452e-02_rb, 3.07822e-02_rb, &
     & 3.51657e-02_rb, 4.01734e-02_rb, 4.58941e-02_rb, 5.24296e-02_rb, 5.98956e-02_rb, &
     & 6.84249e-02_rb, 7.81688e-02_rb, 8.93002e-02_rb, 1.02017e-01_rb/)
      kbo_mn2o(:,14) = (/ &
     & 2.17205e-02_rb, 2.51661e-02_rb, 2.91581e-02_rb, 3.37835e-02_rb, 3.91425e-02_rb, &
     & 4.53517e-02_rb, 5.25458e-02_rb, 6.08811e-02_rb, 7.05387e-02_rb, 8.17282e-02_rb, &
     & 9.46927e-02_rb, 1.09714e-01_rb, 1.27118e-01_rb, 1.47282e-01_rb, 1.70645e-01_rb, &
     & 1.97715e-01_rb, 2.29078e-01_rb, 2.65417e-01_rb, 3.07520e-01_rb/)
      kbo_mn2o(:,15) = (/ &
     & 4.89156e-02_rb, 5.70504e-02_rb, 6.65379e-02_rb, 7.76033e-02_rb, 9.05089e-02_rb, &
     & 1.05561e-01_rb, 1.23116e-01_rb, 1.43590e-01_rb, 1.67469e-01_rb, 1.95320e-01_rb, &
     & 2.27802e-01_rb, 2.65686e-01_rb, 3.09869e-01_rb, 3.61401e-01_rb, 4.21503e-01_rb, &
     & 4.91600e-01_rb, 5.73354e-01_rb, 6.68703e-01_rb, 7.79910e-01_rb/)
      kbo_mn2o(:,16) = (/ &
     & 1.13156e-02_rb, 1.46199e-02_rb, 1.88891e-02_rb, 2.44050e-02_rb, 3.15316e-02_rb, &
     & 4.07393e-02_rb, 5.26358e-02_rb, 6.80061e-02_rb, 8.78649e-02_rb, 1.13523e-01_rb, &
     & 1.46673e-01_rb, 1.89504e-01_rb, 2.44841e-01_rb, 3.16338e-01_rb, 4.08713e-01_rb, &
     & 5.28064e-01_rb, 6.82266e-01_rb, 8.81496e-01_rb, 1.13891e+00_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &4.8166e-07_rb,3.7500e-07_rb,4.8978e-07_rb,5.9624e-07_rb,6.3742e-07_rb,7.5551e-07_rb, &
     &7.7706e-07_rb,6.8681e-07_rb,7.5212e-07_rb,8.0956e-07_rb,7.8117e-07_rb,7.4835e-07_rb, &
     &9.4118e-07_rb,1.2585e-06_rb,1.4976e-06_rb,1.4976e-06_rb/)
      forrefo(2,:) = (/ &
     &3.1320e-07_rb,4.0764e-07_rb,4.7468e-07_rb,5.9976e-07_rb,7.3324e-07_rb,8.1488e-07_rb, &
     &7.6442e-07_rb,8.2007e-07_rb,7.7721e-07_rb,7.6377e-07_rb,8.0327e-07_rb,7.1881e-07_rb, &
     &8.2148e-07_rb,1.0203e-06_rb,1.5033e-06_rb,1.5032e-06_rb/)
      forrefo(3,:) = (/ &
     &4.1831e-07_rb,5.5043e-07_rb,5.7783e-07_rb,6.1294e-07_rb,6.3396e-07_rb,6.2292e-07_rb, &
     &6.1719e-07_rb,6.4183e-07_rb,7.6180e-07_rb,9.5477e-07_rb,9.5901e-07_rb,1.0207e-06_rb, &
     &1.0387e-06_rb,1.1305e-06_rb,1.3602e-06_rb,1.5063e-06_rb/)
      forrefo(4,:) = (/ &
     &8.5878e-07_rb,6.0921e-07_rb,5.5773e-07_rb,5.3374e-07_rb,5.0495e-07_rb,4.9844e-07_rb, &
     &5.1536e-07_rb,5.2908e-07_rb,4.7977e-07_rb,5.3177e-07_rb,4.9266e-07_rb,4.5403e-07_rb, &
     &3.9695e-07_rb,3.4792e-07_rb,3.4912e-07_rb,3.4102e-07_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 3.16029e-02_rb, 2.74633e-02_rb, 2.38660e-02_rb, 2.07399e-02_rb, 1.80232e-02_rb, &
     & 1.56624e-02_rb, 1.36108e-02_rb, 1.18280e-02_rb, 1.02787e-02_rb, 8.93231e-03_rb/)
      selfrefo(:, 2) = (/ &
     & 3.10422e-02_rb, 2.71312e-02_rb, 2.37130e-02_rb, 2.07254e-02_rb, 1.81142e-02_rb, &
     & 1.58320e-02_rb, 1.38374e-02_rb, 1.20940e-02_rb, 1.05703e-02_rb, 9.23854e-03_rb/)
      selfrefo(:, 3) = (/ &
     & 3.08657e-02_rb, 2.69431e-02_rb, 2.35190e-02_rb, 2.05301e-02_rb, 1.79210e-02_rb, &
     & 1.56435e-02_rb, 1.36554e-02_rb, 1.19200e-02_rb, 1.04051e-02_rb, 9.08279e-03_rb/)
      selfrefo(:, 4) = (/ &
     & 3.02668e-02_rb, 2.64686e-02_rb, 2.31470e-02_rb, 2.02422e-02_rb, 1.77020e-02_rb, &
     & 1.54806e-02_rb, 1.35379e-02_rb, 1.18390e-02_rb, 1.03533e-02_rb, 9.05406e-03_rb/)
      selfrefo(:, 5) = (/ &
     & 2.98317e-02_rb, 2.61491e-02_rb, 2.29210e-02_rb, 2.00914e-02_rb, 1.76112e-02_rb, &
     & 1.54371e-02_rb, 1.35314e-02_rb, 1.18610e-02_rb, 1.03968e-02_rb, 9.11332e-03_rb/)
      selfrefo(:, 6) = (/ &
     & 2.95545e-02_rb, 2.59083e-02_rb, 2.27120e-02_rb, 1.99100e-02_rb, 1.74537e-02_rb, &
     & 1.53004e-02_rb, 1.34128e-02_rb, 1.17580e-02_rb, 1.03074e-02_rb, 9.03576e-03_rb/)
      selfrefo(:, 7) = (/ &
     & 2.97352e-02_rb, 2.60320e-02_rb, 2.27900e-02_rb, 1.99517e-02_rb, 1.74670e-02_rb, &
     & 1.52916e-02_rb, 1.33872e-02_rb, 1.17200e-02_rb, 1.02604e-02_rb, 8.98258e-03_rb/)
      selfrefo(:, 8) = (/ &
     & 2.96543e-02_rb, 2.59760e-02_rb, 2.27540e-02_rb, 1.99316e-02_rb, 1.74593e-02_rb, &
     & 1.52937e-02_rb, 1.33967e-02_rb, 1.17350e-02_rb, 1.02794e-02_rb, 9.00437e-03_rb/)
      selfrefo(:, 9) = (/ &
     & 2.97998e-02_rb, 2.60786e-02_rb, 2.28220e-02_rb, 1.99721e-02_rb, 1.74781e-02_rb, &
     & 1.52955e-02_rb, 1.33855e-02_rb, 1.17140e-02_rb, 1.02512e-02_rb, 8.97110e-03_rb/)
      selfrefo(:,10) = (/ &
     & 2.98826e-02_rb, 2.61096e-02_rb, 2.28130e-02_rb, 1.99326e-02_rb, 1.74159e-02_rb, &
     & 1.52170e-02_rb, 1.32957e-02_rb, 1.16170e-02_rb, 1.01502e-02_rb, 8.86867e-03_rb/)
      selfrefo(:,11) = (/ &
     & 2.94710e-02_rb, 2.58147e-02_rb, 2.26120e-02_rb, 1.98066e-02_rb, 1.73493e-02_rb, &
     & 1.51969e-02_rb, 1.33115e-02_rb, 1.16600e-02_rb, 1.02134e-02_rb, 8.94628e-03_rb/)
      selfrefo(:,12) = (/ &
     & 2.96297e-02_rb, 2.59544e-02_rb, 2.27350e-02_rb, 1.99149e-02_rb, 1.74446e-02_rb, &
     & 1.52808e-02_rb, 1.33853e-02_rb, 1.17250e-02_rb, 1.02706e-02_rb, 8.99663e-03_rb/)
      selfrefo(:,13) = (/ &
     & 2.96272e-02_rb, 2.59013e-02_rb, 2.26440e-02_rb, 1.97963e-02_rb, 1.73067e-02_rb, &
     & 1.51302e-02_rb, 1.32275e-02_rb, 1.15640e-02_rb, 1.01097e-02_rb, 8.83833e-03_rb/)
      selfrefo(:,14) = (/ &
     & 2.89906e-02_rb, 2.53971e-02_rb, 2.22490e-02_rb, 1.94911e-02_rb, 1.70751e-02_rb, &
     & 1.49585e-02_rb, 1.31044e-02_rb, 1.14800e-02_rb, 1.00570e-02_rb, 8.81038e-03_rb/)
      selfrefo(:,15) = (/ &
     & 2.80884e-02_rb, 2.46987e-02_rb, 2.17180e-02_rb, 1.90970e-02_rb, 1.67924e-02_rb, &
     & 1.47659e-02_rb, 1.29839e-02_rb, 1.14170e-02_rb, 1.00392e-02_rb, 8.82765e-03_rb/)
      selfrefo(:,16) = (/ &
     & 2.80884e-02_rb, 2.46987e-02_rb, 2.17180e-02_rb, 1.90970e-02_rb, 1.67924e-02_rb, &
     & 1.47659e-02_rb, 1.29839e-02_rb, 1.14170e-02_rb, 1.00392e-02_rb, 8.82765e-03_rb/)

      end subroutine lw_kgb08

! **************************************************************************
      subroutine lw_kgb09
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg09, only : fracrefao, fracrefbo, kao, kbo, kao_mn2o, &
                            kbo_mn2o, selfrefo, forrefo

      implicit none
      save

! Planck fractions mapping level : P=212.7250 mb, T = 223.06 K
      fracrefao(:, 1) = (/ &
     &  1.8129e-01_rb,1.6119e-01_rb,1.3308e-01_rb,1.2342e-01_rb,1.1259e-01_rb,9.7580e-02_rb, &
     &  7.9176e-02_rb,5.8541e-02_rb,3.9084e-02_rb,4.2419e-03_rb,3.4314e-03_rb,2.6935e-03_rb, &
     &  1.9404e-03_rb,1.2218e-03_rb,4.5263e-04_rb,6.0909e-05_rb/)
      fracrefao(:, 2) = (/ &
     &  1.9665e-01_rb,1.5640e-01_rb,1.3101e-01_rb,1.2153e-01_rb,1.1037e-01_rb,9.6043e-02_rb, &
     &  7.7856e-02_rb,5.7547e-02_rb,3.8670e-02_rb,4.1955e-03_rb,3.4104e-03_rb,2.6781e-03_rb, &
     &  1.9245e-03_rb,1.2093e-03_rb,4.4113e-04_rb,6.0913e-05_rb/)
      fracrefao(:, 3) = (/ &
     &  2.0273e-01_rb,1.5506e-01_rb,1.3044e-01_rb,1.2043e-01_rb,1.0952e-01_rb,9.5384e-02_rb, &
     &  7.7157e-02_rb,5.7176e-02_rb,3.8379e-02_rb,4.1584e-03_rb,3.3836e-03_rb,2.6412e-03_rb, &
     &  1.8865e-03_rb,1.1791e-03_rb,4.2094e-04_rb,4.7410e-05_rb/)
      fracrefao(:, 4) = (/ &
     &  2.0272e-01_rb,1.5963e-01_rb,1.2913e-01_rb,1.2060e-01_rb,1.0820e-01_rb,9.4685e-02_rb, &
     &  7.6544e-02_rb,5.6851e-02_rb,3.8155e-02_rb,4.0913e-03_rb,3.3442e-03_rb,2.6054e-03_rb, &
     &  1.8875e-03_rb,1.1263e-03_rb,3.7743e-04_rb,4.7410e-05_rb/)
      fracrefao(:, 5) = (/ &
     &  2.0280e-01_rb,1.6353e-01_rb,1.2910e-01_rb,1.1968e-01_rb,1.0725e-01_rb,9.4112e-02_rb, &
     &  7.5828e-02_rb,5.6526e-02_rb,3.7972e-02_rb,4.0205e-03_rb,3.3063e-03_rb,2.5681e-03_rb, &
     &  1.8386e-03_rb,1.0757e-03_rb,3.5301e-04_rb,4.7410e-05_rb/)
      fracrefao(:, 6) = (/ &
     &  2.0294e-01_rb,1.6840e-01_rb,1.2852e-01_rb,1.1813e-01_rb,1.0724e-01_rb,9.2946e-02_rb, &
     &  7.5029e-02_rb,5.6158e-02_rb,3.7744e-02_rb,3.9632e-03_rb,3.2434e-03_rb,2.5275e-03_rb, &
     &  1.7558e-03_rb,1.0080e-03_rb,3.5301e-04_rb,4.7410e-05_rb/)
      fracrefao(:, 7) = (/ &
     &  2.0313e-01_rb,1.7390e-01_rb,1.2864e-01_rb,1.1689e-01_rb,1.0601e-01_rb,9.1791e-02_rb, &
     &  7.4224e-02_rb,5.5500e-02_rb,3.7374e-02_rb,3.9214e-03_rb,3.1984e-03_rb,2.4162e-03_rb, &
     &  1.6394e-03_rb,9.7275e-04_rb,3.5299e-04_rb,4.7410e-05_rb/)
      fracrefao(:, 8) = (/ &
     &  2.0332e-01_rb,1.7800e-01_rb,1.3286e-01_rb,1.1555e-01_rb,1.0407e-01_rb,9.0475e-02_rb, &
     &  7.2452e-02_rb,5.4566e-02_rb,3.6677e-02_rb,3.7889e-03_rb,3.0351e-03_rb,2.2587e-03_rb, &
     &  1.5764e-03_rb,9.7270e-04_rb,3.5300e-04_rb,4.7410e-05_rb/)
      fracrefao(:, 9) = (/ &
     &  1.9624e-01_rb,1.6519e-01_rb,1.3663e-01_rb,1.1535e-01_rb,1.0719e-01_rb,9.4156e-02_rb, &
     &  7.6745e-02_rb,5.6987e-02_rb,3.8135e-02_rb,4.1626e-03_rb,3.4243e-03_rb,2.7116e-03_rb, &
     &  1.7095e-03_rb,9.7271e-04_rb,3.5299e-04_rb,4.7410e-05_rb/)

! Planck fraction mapping level : p=3.20e-2 mb, t = 197.92 k
      fracrefbo(:) = (/ &
     &  2.0914e-01_rb,1.5077e-01_rb,1.2878e-01_rb,1.1856e-01_rb,1.0695e-01_rb,9.3048e-02_rb, &
     &  7.7645e-02_rb,6.0785e-02_rb,4.0642e-02_rb,4.0499e-03_rb,3.3931e-03_rb,2.6363e-03_rb, &
     &  1.9151e-03_rb,1.1963e-03_rb,4.3471e-04_rb,5.1421e-05_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &2.1746e-06_rb,2.0220e-05_rb,3.5039e-05_rb,4.9564e-05_rb,6.4547e-05_rb,8.0645e-05_rb, &
     &1.0050e-04_rb,1.3591e-04_rb,1.1422e-04_rb/)
      kao(:, 2, 1, 1) = (/ &
     &2.4567e-06_rb,2.4642e-05_rb,4.2577e-05_rb,6.0179e-05_rb,7.8156e-05_rb,9.7521e-05_rb, &
     &1.2325e-04_rb,1.6438e-04_rb,1.3834e-04_rb/)
      kao(:, 3, 1, 1) = (/ &
     &2.7778e-06_rb,2.9755e-05_rb,5.1233e-05_rb,7.2538e-05_rb,9.4256e-05_rb,1.1716e-04_rb, &
     &1.4583e-04_rb,1.9818e-04_rb,1.6722e-04_rb/)
      kao(:, 4, 1, 1) = (/ &
     &3.1364e-06_rb,3.5376e-05_rb,6.1122e-05_rb,8.6709e-05_rb,1.1261e-04_rb,1.4025e-04_rb, &
     &1.7411e-04_rb,2.3326e-04_rb,2.0043e-04_rb/)
      kao(:, 5, 1, 1) = (/ &
     &3.4866e-06_rb,4.1773e-05_rb,7.2421e-05_rb,1.0284e-04_rb,1.3368e-04_rb,1.6658e-04_rb, &
     &2.0645e-04_rb,2.7379e-04_rb,2.3933e-04_rb/)
      kao(:, 1, 2, 1) = (/ &
     &2.7658e-06_rb,1.8665e-05_rb,2.9579e-05_rb,4.0556e-05_rb,5.1475e-05_rb,6.3012e-05_rb, &
     &7.6030e-05_rb,9.6554e-05_rb,8.6790e-05_rb/)
      kao(:, 2, 2, 1) = (/ &
     &3.1474e-06_rb,2.2675e-05_rb,3.6322e-05_rb,4.9569e-05_rb,6.2848e-05_rb,7.6725e-05_rb, &
     &9.2975e-05_rb,1.1920e-04_rb,1.0628e-04_rb/)
      kao(:, 3, 2, 1) = (/ &
     &3.5619e-06_rb,2.7222e-05_rb,4.3763e-05_rb,5.9895e-05_rb,7.6025e-05_rb,9.2855e-05_rb, &
     &1.1166e-04_rb,1.4410e-04_rb,1.2840e-04_rb/)
      kao(:, 4, 2, 1) = (/ &
     &4.0332e-06_rb,3.2194e-05_rb,5.2284e-05_rb,7.1664e-05_rb,9.1201e-05_rb,1.1132e-04_rb, &
     &1.3395e-04_rb,1.6948e-04_rb,1.5482e-04_rb/)
      kao(:, 5, 2, 1) = (/ &
     &4.5426e-06_rb,3.7892e-05_rb,6.1930e-05_rb,8.5200e-05_rb,1.0845e-04_rb,1.3257e-04_rb, &
     &1.5957e-04_rb,2.0150e-04_rb,1.8531e-04_rb/)
      kao(:, 1, 3, 1) = (/ &
     &4.6515e-06_rb,1.9518e-05_rb,2.8024e-05_rb,3.5437e-05_rb,4.2060e-05_rb,4.8949e-05_rb, &
     &5.6458e-05_rb,6.6193e-05_rb,6.4702e-05_rb/)
      kao(:, 2, 3, 1) = (/ &
     &5.3085e-06_rb,2.3661e-05_rb,3.4250e-05_rb,4.3649e-05_rb,5.1882e-05_rb,6.0429e-05_rb, &
     &6.9536e-05_rb,8.2456e-05_rb,7.9400e-05_rb/)
      kao(:, 3, 3, 1) = (/ &
     &6.0401e-06_rb,2.8338e-05_rb,4.1344e-05_rb,5.2675e-05_rb,6.2996e-05_rb,7.3527e-05_rb, &
     &8.4697e-05_rb,9.9499e-05_rb,9.6161e-05_rb/)
      kao(:, 4, 3, 1) = (/ &
     &6.8636e-06_rb,3.3625e-05_rb,4.9393e-05_rb,6.2729e-05_rb,7.5705e-05_rb,8.8632e-05_rb, &
     &1.0218e-04_rb,1.1965e-04_rb,1.1595e-04_rb/)
      kao(:, 5, 3, 1) = (/ &
     &7.7592e-06_rb,3.9481e-05_rb,5.8243e-05_rb,7.4429e-05_rb,9.0184e-05_rb,1.0593e-04_rb, &
     &1.2226e-04_rb,1.4329e-04_rb,1.3957e-04_rb/)
      kao(:, 1, 4, 1) = (/ &
     &7.8254e-06_rb,2.2162e-05_rb,2.8768e-05_rb,3.4092e-05_rb,3.8273e-05_rb,4.1617e-05_rb, &
     &4.4585e-05_rb,4.7843e-05_rb,5.0090e-05_rb/)
      kao(:, 2, 4, 1) = (/ &
     &9.0107e-06_rb,2.6852e-05_rb,3.5234e-05_rb,4.2063e-05_rb,4.7365e-05_rb,5.1787e-05_rb, &
     &5.5276e-05_rb,5.9864e-05_rb,6.1930e-05_rb/)
      kao(:, 3, 4, 1) = (/ &
     &1.0349e-05_rb,3.2172e-05_rb,4.2565e-05_rb,5.1061e-05_rb,5.7654e-05_rb,6.3243e-05_rb, &
     &6.7675e-05_rb,7.3397e-05_rb,7.5548e-05_rb/)
      kao(:, 4, 4, 1) = (/ &
     &1.1906e-05_rb,3.8190e-05_rb,5.0960e-05_rb,6.1161e-05_rb,6.9192e-05_rb,7.6085e-05_rb, &
     &8.1960e-05_rb,8.9085e-05_rb,9.1511e-05_rb/)
      kao(:, 5, 4, 1) = (/ &
     &1.3610e-05_rb,4.4853e-05_rb,6.0373e-05_rb,7.2336e-05_rb,8.2116e-05_rb,9.0436e-05_rb, &
     &9.8270e-05_rb,1.0717e-04_rb,1.0994e-04_rb/)
      kao(:, 1, 5, 1) = (/ &
     &1.2215e-05_rb,2.6429e-05_rb,3.0893e-05_rb,3.4176e-05_rb,3.6538e-05_rb,3.7893e-05_rb, &
     &3.8292e-05_rb,3.7482e-05_rb,3.9876e-05_rb/)
      kao(:, 2, 5, 1) = (/ &
     &1.4086e-05_rb,3.1907e-05_rb,3.7874e-05_rb,4.2237e-05_rb,4.5377e-05_rb,4.7376e-05_rb, &
     &4.7976e-05_rb,4.7239e-05_rb,4.9622e-05_rb/)
      kao(:, 3, 5, 1) = (/ &
     &1.6264e-05_rb,3.8161e-05_rb,4.5845e-05_rb,5.1471e-05_rb,5.5412e-05_rb,5.8145e-05_rb, &
     &5.8993e-05_rb,5.8204e-05_rb,6.0963e-05_rb/)
      kao(:, 4, 5, 1) = (/ &
     &1.8858e-05_rb,4.5142e-05_rb,5.4763e-05_rb,6.1781e-05_rb,6.6831e-05_rb,7.0144e-05_rb, &
     &7.1439e-05_rb,7.0642e-05_rb,7.3664e-05_rb/)
      kao(:, 5, 5, 1) = (/ &
     &2.1682e-05_rb,5.2995e-05_rb,6.4832e-05_rb,7.3555e-05_rb,7.9689e-05_rb,8.3689e-05_rb, &
     &8.5451e-05_rb,8.5137e-05_rb,8.8330e-05_rb/)
      kao(:, 1, 6, 1) = (/ &
     &1.7737e-05_rb,3.1507e-05_rb,3.4689e-05_rb,3.5923e-05_rb,3.6042e-05_rb,3.5547e-05_rb, &
     &3.3855e-05_rb,3.0889e-05_rb,3.3707e-05_rb/)
      kao(:, 2, 6, 1) = (/ &
     &2.0662e-05_rb,3.8270e-05_rb,4.2282e-05_rb,4.3931e-05_rb,4.4914e-05_rb,4.4463e-05_rb, &
     &4.2744e-05_rb,3.9347e-05_rb,4.1797e-05_rb/)
      kao(:, 3, 6, 1) = (/ &
     &2.4128e-05_rb,4.5613e-05_rb,5.0898e-05_rb,5.3590e-05_rb,5.4968e-05_rb,5.4808e-05_rb, &
     &5.3000e-05_rb,4.8943e-05_rb,5.1147e-05_rb/)
      kao(:, 4, 6, 1) = (/ &
     &2.7999e-05_rb,5.3907e-05_rb,6.0955e-05_rb,6.4423e-05_rb,6.6442e-05_rb,6.6605e-05_rb, &
     &6.4604e-05_rb,5.9878e-05_rb,6.1516e-05_rb/)
      kao(:, 5, 6, 1) = (/ &
     &3.2152e-05_rb,6.3564e-05_rb,7.2012e-05_rb,7.6692e-05_rb,7.9463e-05_rb,7.9823e-05_rb, &
     &7.7573e-05_rb,7.2281e-05_rb,7.4081e-05_rb/)
      kao(:, 1, 7, 1) = (/ &
     &2.6880e-05_rb,4.1451e-05_rb,4.2153e-05_rb,4.1795e-05_rb,4.0071e-05_rb,3.7010e-05_rb, &
     &3.2935e-05_rb,2.7157e-05_rb,3.0165e-05_rb/)
      kao(:, 2, 7, 1) = (/ &
     &3.1867e-05_rb,5.0449e-05_rb,5.1950e-05_rb,5.1350e-05_rb,4.9502e-05_rb,4.6117e-05_rb, &
     &4.1536e-05_rb,3.4782e-05_rb,3.7420e-05_rb/)
      kao(:, 3, 7, 1) = (/ &
     &3.7274e-05_rb,6.0537e-05_rb,6.2610e-05_rb,6.2280e-05_rb,6.0355e-05_rb,5.6848e-05_rb, &
     &5.1457e-05_rb,4.3658e-05_rb,4.5586e-05_rb/)
      kao(:, 4, 7, 1) = (/ &
     &4.3315e-05_rb,7.1931e-05_rb,7.4511e-05_rb,7.5005e-05_rb,7.2731e-05_rb,6.8916e-05_rb, &
     &6.2904e-05_rb,5.3755e-05_rb,5.5264e-05_rb/)
      kao(:, 5, 7, 1) = (/ &
     &4.9907e-05_rb,8.4683e-05_rb,8.8542e-05_rb,8.9292e-05_rb,8.6631e-05_rb,8.2549e-05_rb, &
     &7.6077e-05_rb,6.5242e-05_rb,6.5892e-05_rb/)
      kao(:, 1, 8, 1) = (/ &
     &4.7627e-05_rb,6.4864e-05_rb,6.3521e-05_rb,5.9018e-05_rb,5.3108e-05_rb,4.6571e-05_rb, &
     &3.8650e-05_rb,2.8473e-05_rb,2.9135e-05_rb/)
      kao(:, 2, 8, 1) = (/ &
     &5.7767e-05_rb,7.9036e-05_rb,7.7751e-05_rb,7.3012e-05_rb,6.6279e-05_rb,5.8148e-05_rb, &
     &4.8459e-05_rb,3.6303e-05_rb,3.6552e-05_rb/)
      kao(:, 3, 8, 1) = (/ &
     &6.8331e-05_rb,9.5161e-05_rb,9.3980e-05_rb,8.8741e-05_rb,8.0901e-05_rb,7.1279e-05_rb, &
     &5.9682e-05_rb,4.5314e-05_rb,4.4112e-05_rb/)
      kao(:, 4, 8, 1) = (/ &
     &7.9894e-05_rb,1.1320e-04_rb,1.1263e-04_rb,1.0646e-04_rb,9.7413e-05_rb,8.6626e-05_rb, &
     &7.2540e-05_rb,5.5667e-05_rb,5.2996e-05_rb/)
      kao(:, 5, 8, 1) = (/ &
     &9.2972e-05_rb,1.3437e-04_rb,1.3320e-04_rb,1.2651e-04_rb,1.1648e-04_rb,1.0389e-04_rb, &
     &8.7249e-05_rb,6.7590e-05_rb,6.3371e-05_rb/)
      kao(:, 1, 9, 1) = (/ &
     &1.5799e-04_rb,1.8833e-04_rb,1.7550e-04_rb,1.5612e-04_rb,1.3380e-04_rb,1.0918e-04_rb, &
     &8.2063e-05_rb,5.1241e-05_rb,3.5341e-05_rb/)
      kao(:, 2, 9, 1) = (/ &
     &1.9443e-04_rb,2.2819e-04_rb,2.1471e-04_rb,1.9244e-04_rb,1.6566e-04_rb,1.3587e-04_rb, &
     &1.0320e-04_rb,6.4536e-05_rb,4.5195e-05_rb/)
      kao(:, 3, 9, 1) = (/ &
     &2.3622e-04_rb,2.7685e-04_rb,2.6057e-04_rb,2.3445e-04_rb,2.0276e-04_rb,1.6671e-04_rb, &
     &1.2651e-04_rb,7.9797e-05_rb,5.6882e-05_rb/)
      kao(:, 4, 9, 1) = (/ &
     &2.7828e-04_rb,3.3139e-04_rb,3.1210e-04_rb,2.8146e-04_rb,2.4413e-04_rb,2.0071e-04_rb, &
     &1.5211e-04_rb,9.7273e-05_rb,6.9081e-05_rb/)
      kao(:, 5, 9, 1) = (/ &
     &3.2528e-04_rb,3.9307e-04_rb,3.6944e-04_rb,3.3441e-04_rb,2.9052e-04_rb,2.3944e-04_rb, &
     &1.8220e-04_rb,1.1689e-04_rb,8.4202e-05_rb/)
      kao(:, 1,10, 1) = (/ &
     &5.9822e-04_rb,6.2372e-04_rb,5.8089e-04_rb,5.1261e-04_rb,4.3322e-04_rb,3.4273e-04_rb, &
     &2.4200e-04_rb,1.3558e-04_rb,7.2135e-05_rb/)
      kao(:, 2,10, 1) = (/ &
     &7.4373e-04_rb,7.8308e-04_rb,7.2223e-04_rb,6.3443e-04_rb,5.3223e-04_rb,4.2429e-04_rb, &
     &3.0242e-04_rb,1.7107e-04_rb,9.5831e-05_rb/)
      kao(:, 3,10, 1) = (/ &
     &9.1691e-04_rb,9.5621e-04_rb,8.7625e-04_rb,7.6950e-04_rb,6.5144e-04_rb,5.1701e-04_rb, &
     &3.7121e-04_rb,2.1199e-04_rb,1.2423e-04_rb/)
      kao(:, 4,10, 1) = (/ &
     &1.1043e-03_rb,1.1414e-03_rb,1.0539e-03_rb,9.3161e-04_rb,7.8672e-04_rb,6.2626e-04_rb, &
     &4.5160e-04_rb,2.5892e-04_rb,1.5666e-04_rb/)
      kao(:, 5,10, 1) = (/ &
     &1.3045e-03_rb,1.3552e-03_rb,1.2556e-03_rb,1.1172e-03_rb,9.4316e-04_rb,7.4718e-04_rb, &
     &5.4008e-04_rb,3.1196e-04_rb,1.9599e-04_rb/)
      kao(:, 1,11, 1) = (/ &
     &8.7712e-04_rb,8.8837e-04_rb,8.1430e-04_rb,7.2064e-04_rb,6.0791e-04_rb,4.8638e-04_rb, &
     &3.4855e-04_rb,1.8904e-04_rb,9.5086e-05_rb/)
      kao(:, 2,11, 1) = (/ &
     &1.1025e-03_rb,1.1220e-03_rb,1.0310e-03_rb,9.0820e-04_rb,7.5807e-04_rb,5.9864e-04_rb, &
     &4.3064e-04_rb,2.3702e-04_rb,1.2669e-04_rb/)
      kao(:, 3,11, 1) = (/ &
     &1.3684e-03_rb,1.3746e-03_rb,1.2594e-03_rb,1.1058e-03_rb,9.3187e-04_rb,7.3748e-04_rb, &
     &5.2465e-04_rb,2.9520e-04_rb,1.6181e-04_rb/)
      kao(:, 4,11, 1) = (/ &
     &1.6471e-03_rb,1.6623e-03_rb,1.5206e-03_rb,1.3382e-03_rb,1.1315e-03_rb,8.9471e-04_rb, &
     &6.4214e-04_rb,3.6310e-04_rb,2.0489e-04_rb/)
      kao(:, 5,11, 1) = (/ &
     &1.9688e-03_rb,1.9772e-03_rb,1.8138e-03_rb,1.6028e-03_rb,1.3619e-03_rb,1.0826e-03_rb, &
     &7.7699e-04_rb,4.3404e-04_rb,2.5459e-04_rb/)
      kao(:, 1,12, 1) = (/ &
     &9.4409e-04_rb,9.5030e-04_rb,8.6958e-04_rb,7.6723e-04_rb,6.4799e-04_rb,5.1804e-04_rb, &
     &3.7162e-04_rb,2.0179e-04_rb,1.0181e-04_rb/)
      kao(:, 2,12, 1) = (/ &
     &1.2024e-03_rb,1.2024e-03_rb,1.1035e-03_rb,9.7105e-04_rb,8.1245e-04_rb,6.4379e-04_rb, &
     &4.6123e-04_rb,2.5624e-04_rb,1.3746e-04_rb/)
      kao(:, 3,12, 1) = (/ &
     &1.5009e-03_rb,1.4895e-03_rb,1.3527e-03_rb,1.1889e-03_rb,1.0016e-03_rb,7.9376e-04_rb, &
     &5.6671e-04_rb,3.1943e-04_rb,1.7568e-04_rb/)
      kao(:, 4,12, 1) = (/ &
     &1.8192e-03_rb,1.8117e-03_rb,1.6422e-03_rb,1.4410e-03_rb,1.2151e-03_rb,9.6812e-04_rb, &
     &6.9546e-04_rb,3.9137e-04_rb,2.1782e-04_rb/)
      kao(:, 5,12, 1) = (/ &
     &2.1766e-03_rb,2.1635e-03_rb,1.9700e-03_rb,1.7314e-03_rb,1.4651e-03_rb,1.1736e-03_rb, &
     &8.4593e-04_rb,4.7032e-04_rb,2.5768e-04_rb/)
      kao(:, 1,13, 1) = (/ &
     &8.2069e-04_rb,8.2445e-04_rb,7.5154e-04_rb,6.6188e-04_rb,5.6185e-04_rb,4.5083e-04_rb, &
     &3.2275e-04_rb,1.7851e-04_rb,9.1099e-05_rb/)
      kao(:, 2,13, 1) = (/ &
     &1.0473e-03_rb,1.0502e-03_rb,9.5966e-04_rb,8.4257e-04_rb,7.0741e-04_rb,5.6233e-04_rb, &
     &4.0370e-04_rb,2.2679e-04_rb,1.2172e-04_rb/)
      kao(:, 3,13, 1) = (/ &
     &1.3133e-03_rb,1.3024e-03_rb,1.1846e-03_rb,1.0375e-03_rb,8.7436e-04_rb,6.9645e-04_rb, &
     &5.0292e-04_rb,2.8463e-04_rb,1.5549e-04_rb/)
      kao(:, 4,13, 1) = (/ &
     &1.6064e-03_rb,1.5873e-03_rb,1.4461e-03_rb,1.2677e-03_rb,1.0715e-03_rb,8.5901e-04_rb, &
     &6.1842e-04_rb,3.4781e-04_rb,1.9357e-04_rb/)
      kao(:, 5,13, 1) = (/ &
     &1.9305e-03_rb,1.9051e-03_rb,1.7364e-03_rb,1.5314e-03_rb,1.2972e-03_rb,1.0440e-03_rb, &
     &7.5312e-04_rb,4.2210e-04_rb,2.3145e-04_rb/)
      kao(:, 1, 1, 2) = (/ &
     &8.1346e-06_rb,1.0615e-04_rb,1.6451e-04_rb,2.1843e-04_rb,2.7217e-04_rb,3.3271e-04_rb, &
     &4.0424e-04_rb,5.2737e-04_rb,4.6457e-04_rb/)
      kao(:, 2, 1, 2) = (/ &
     &9.5357e-06_rb,1.2245e-04_rb,1.9203e-04_rb,2.5686e-04_rb,3.2147e-04_rb,3.9091e-04_rb, &
     &4.7632e-04_rb,6.1736e-04_rb,5.3890e-04_rb/)
      kao(:, 3, 1, 2) = (/ &
     &1.0894e-05_rb,1.4027e-04_rb,2.2328e-04_rb,3.0098e-04_rb,3.7740e-04_rb,4.5873e-04_rb, &
     &5.6256e-04_rb,7.2836e-04_rb,6.2820e-04_rb/)
      kao(:, 4, 1, 2) = (/ &
     &1.2218e-05_rb,1.6125e-04_rb,2.5837e-04_rb,3.4988e-04_rb,4.3991e-04_rb,5.3396e-04_rb, &
     &6.5212e-04_rb,8.5783e-04_rb,7.3258e-04_rb/)
      kao(:, 5, 1, 2) = (/ &
     &1.3559e-05_rb,1.8418e-04_rb,2.9709e-04_rb,4.0563e-04_rb,5.0913e-04_rb,6.2018e-04_rb, &
     &7.5464e-04_rb,9.9267e-04_rb,8.5248e-04_rb/)
      kao(:, 1, 2, 2) = (/ &
     &1.0648e-05_rb,1.0205e-04_rb,1.5218e-04_rb,1.9460e-04_rb,2.3469e-04_rb,2.7445e-04_rb, &
     &3.2374e-04_rb,3.9687e-04_rb,3.7021e-04_rb/)
      kao(:, 2, 2, 2) = (/ &
     &1.2479e-05_rb,1.1864e-04_rb,1.7693e-04_rb,2.2906e-04_rb,2.7817e-04_rb,3.2635e-04_rb, &
     &3.8100e-04_rb,4.6812e-04_rb,4.2972e-04_rb/)
      kao(:, 3, 2, 2) = (/ &
     &1.4325e-05_rb,1.3615e-04_rb,2.0613e-04_rb,2.6822e-04_rb,3.2742e-04_rb,3.8532e-04_rb, &
     &4.4992e-04_rb,5.5581e-04_rb,5.0382e-04_rb/)
      kao(:, 4, 2, 2) = (/ &
     &1.6131e-05_rb,1.5522e-04_rb,2.3823e-04_rb,3.1216e-04_rb,3.8163e-04_rb,4.4987e-04_rb, &
     &5.2423e-04_rb,6.5095e-04_rb,5.8901e-04_rb/)
      kao(:, 5, 2, 2) = (/ &
     &1.7840e-05_rb,1.7738e-04_rb,2.7413e-04_rb,3.6145e-04_rb,4.4307e-04_rb,5.2217e-04_rb, &
     &6.0968e-04_rb,7.4929e-04_rb,6.8588e-04_rb/)
      kao(:, 1, 3, 2) = (/ &
     &1.7975e-05_rb,1.1236e-04_rb,1.5383e-04_rb,1.8734e-04_rb,2.1612e-04_rb,2.4014e-04_rb, &
     &2.6058e-04_rb,2.9343e-04_rb,2.9752e-04_rb/)
      kao(:, 2, 3, 2) = (/ &
     &2.1208e-05_rb,1.3084e-04_rb,1.8013e-04_rb,2.2080e-04_rb,2.5543e-04_rb,2.8432e-04_rb, &
     &3.1199e-04_rb,3.4846e-04_rb,3.5196e-04_rb/)
      kao(:, 3, 3, 2) = (/ &
     &2.4558e-05_rb,1.5021e-04_rb,2.0878e-04_rb,2.5693e-04_rb,2.9945e-04_rb,3.3619e-04_rb, &
     &3.7048e-04_rb,4.1334e-04_rb,4.1350e-04_rb/)
      kao(:, 4, 3, 2) = (/ &
     &2.7720e-05_rb,1.7111e-04_rb,2.4012e-04_rb,2.9904e-04_rb,3.4910e-04_rb,3.9367e-04_rb, &
     &4.3560e-04_rb,4.8351e-04_rb,4.8248e-04_rb/)
      kao(:, 5, 3, 2) = (/ &
     &3.0839e-05_rb,1.9412e-04_rb,2.7570e-04_rb,3.4584e-04_rb,4.0483e-04_rb,4.5917e-04_rb, &
     &5.0628e-04_rb,5.6356e-04_rb,5.6016e-04_rb/)
      kao(:, 1, 4, 2) = (/ &
     &3.0257e-05_rb,1.3348e-04_rb,1.6797e-04_rb,1.9240e-04_rb,2.0954e-04_rb,2.2222e-04_rb, &
     &2.2898e-04_rb,2.3143e-04_rb,2.5081e-04_rb/)
      kao(:, 2, 4, 2) = (/ &
     &3.6110e-05_rb,1.5427e-04_rb,1.9683e-04_rb,2.2498e-04_rb,2.4900e-04_rb,2.6495e-04_rb, &
     &2.7383e-04_rb,2.7831e-04_rb,3.0130e-04_rb/)
      kao(:, 3, 4, 2) = (/ &
     &4.2020e-05_rb,1.7731e-04_rb,2.2733e-04_rb,2.6268e-04_rb,2.9200e-04_rb,3.1270e-04_rb, &
     &3.2430e-04_rb,3.3243e-04_rb,3.4977e-04_rb/)
      kao(:, 4, 4, 2) = (/ &
     &4.7704e-05_rb,2.0254e-04_rb,2.6080e-04_rb,3.0470e-04_rb,3.4066e-04_rb,3.6571e-04_rb, &
     &3.8177e-04_rb,3.9329e-04_rb,4.0928e-04_rb/)
      kao(:, 5, 4, 2) = (/ &
     &5.3448e-05_rb,2.2883e-04_rb,2.9767e-04_rb,3.4976e-04_rb,3.9269e-04_rb,4.2565e-04_rb, &
     &4.4636e-04_rb,4.5989e-04_rb,4.7277e-04_rb/)
      kao(:, 1, 5, 2) = (/ &
     &4.6988e-05_rb,1.6787e-04_rb,1.9191e-04_rb,2.0648e-04_rb,2.1525e-04_rb,2.1596e-04_rb, &
     &2.0906e-04_rb,1.9576e-04_rb,2.0610e-04_rb/)
      kao(:, 2, 5, 2) = (/ &
     &5.6510e-05_rb,1.9317e-04_rb,2.2448e-04_rb,2.4373e-04_rb,2.5210e-04_rb,2.5521e-04_rb, &
     &2.5032e-04_rb,2.3737e-04_rb,2.5015e-04_rb/)
      kao(:, 3, 5, 2) = (/ &
     &6.6097e-05_rb,2.2111e-04_rb,2.5901e-04_rb,2.8182e-04_rb,2.9480e-04_rb,2.9992e-04_rb, &
     &2.9852e-04_rb,2.8340e-04_rb,3.0111e-04_rb/)
      kao(:, 4, 5, 2) = (/ &
     &7.5373e-05_rb,2.5095e-04_rb,2.9694e-04_rb,3.2434e-04_rb,3.4231e-04_rb,3.5125e-04_rb, &
     &3.5172e-04_rb,3.3625e-04_rb,3.5720e-04_rb/)
      kao(:, 5, 5, 2) = (/ &
     &8.4653e-05_rb,2.8271e-04_rb,3.3633e-04_rb,3.7103e-04_rb,3.9540e-04_rb,4.0854e-04_rb, &
     &4.1017e-04_rb,3.9523e-04_rb,4.0911e-04_rb/)
      kao(:, 1, 6, 2) = (/ &
     &6.8194e-05_rb,2.1119e-04_rb,2.2511e-04_rb,2.2905e-04_rb,2.2761e-04_rb,2.1982e-04_rb, &
     &2.0183e-04_rb,1.7113e-04_rb,1.7129e-04_rb/)
      kao(:, 2, 6, 2) = (/ &
     &8.3032e-05_rb,2.4138e-04_rb,2.6267e-04_rb,2.7055e-04_rb,2.6995e-04_rb,2.6005e-04_rb, &
     &2.3793e-04_rb,2.0771e-04_rb,2.1052e-04_rb/)
      kao(:, 3, 6, 2) = (/ &
     &9.7316e-05_rb,2.7435e-04_rb,3.0334e-04_rb,3.1415e-04_rb,3.1605e-04_rb,3.0269e-04_rb, &
     &2.8123e-04_rb,2.4990e-04_rb,2.5531e-04_rb/)
      kao(:, 4, 6, 2) = (/ &
     &1.1184e-04_rb,3.0984e-04_rb,3.4680e-04_rb,3.6228e-04_rb,3.6303e-04_rb,3.5165e-04_rb, &
     &3.2989e-04_rb,2.9654e-04_rb,3.0715e-04_rb/)
      kao(:, 5, 6, 2) = (/ &
     &1.2632e-04_rb,3.4683e-04_rb,3.9318e-04_rb,4.1205e-04_rb,4.1568e-04_rb,4.0572e-04_rb, &
     &3.8584e-04_rb,3.4794e-04_rb,3.6334e-04_rb/)
      kao(:, 1, 7, 2) = (/ &
     &1.0382e-04_rb,2.6395e-04_rb,2.8989e-04_rb,2.8076e-04_rb,2.6344e-04_rb,2.4071e-04_rb, &
     &2.1106e-04_rb,1.6791e-04_rb,1.4516e-04_rb/)
      kao(:, 2, 7, 2) = (/ &
     &1.2713e-04_rb,3.0635e-04_rb,3.3550e-04_rb,3.2897e-04_rb,3.1196e-04_rb,2.8824e-04_rb, &
     &2.5288e-04_rb,2.0031e-04_rb,1.8143e-04_rb/)
      kao(:, 3, 7, 2) = (/ &
     &1.5126e-04_rb,3.5047e-04_rb,3.8510e-04_rb,3.8068e-04_rb,3.6466e-04_rb,3.3889e-04_rb, &
     &2.9600e-04_rb,2.3790e-04_rb,2.2323e-04_rb/)
      kao(:, 4, 7, 2) = (/ &
     &1.7453e-04_rb,3.9749e-04_rb,4.3569e-04_rb,4.3708e-04_rb,4.2166e-04_rb,3.9085e-04_rb, &
     &3.4474e-04_rb,2.8074e-04_rb,2.7137e-04_rb/)
      kao(:, 5, 7, 2) = (/ &
     &1.9815e-04_rb,4.4700e-04_rb,4.9020e-04_rb,4.9608e-04_rb,4.8316e-04_rb,4.4845e-04_rb, &
     &4.0007e-04_rb,3.2811e-04_rb,3.2529e-04_rb/)
      kao(:, 1, 8, 2) = (/ &
     &1.8568e-04_rb,3.7522e-04_rb,4.0441e-04_rb,4.0400e-04_rb,3.7101e-04_rb,3.2090e-04_rb, &
     &2.6014e-04_rb,1.8741e-04_rb,1.4093e-04_rb/)
      kao(:, 2, 8, 2) = (/ &
     &2.2706e-04_rb,4.4100e-04_rb,4.7331e-04_rb,4.6941e-04_rb,4.3330e-04_rb,3.7933e-04_rb, &
     &3.1116e-04_rb,2.2753e-04_rb,1.7550e-04_rb/)
      kao(:, 3, 8, 2) = (/ &
     &2.7313e-04_rb,5.0681e-04_rb,5.4665e-04_rb,5.4041e-04_rb,5.0328e-04_rb,4.4299e-04_rb, &
     &3.6626e-04_rb,2.6706e-04_rb,2.1512e-04_rb/)
      kao(:, 4, 8, 2) = (/ &
     &3.2050e-04_rb,5.7784e-04_rb,6.2469e-04_rb,6.1766e-04_rb,5.7799e-04_rb,5.1310e-04_rb, &
     &4.2712e-04_rb,3.1237e-04_rb,2.5864e-04_rb/)
      kao(:, 5, 8, 2) = (/ &
     &3.6492e-04_rb,6.4972e-04_rb,7.0632e-04_rb,6.9926e-04_rb,6.5563e-04_rb,5.8535e-04_rb, &
     &4.8972e-04_rb,3.6487e-04_rb,3.1085e-04_rb/)
      kao(:, 1, 9, 2) = (/ &
     &6.2599e-04_rb,8.6654e-04_rb,9.0634e-04_rb,8.8937e-04_rb,8.1140e-04_rb,7.1027e-04_rb, &
     &5.7167e-04_rb,3.5735e-04_rb,2.0982e-04_rb/)
      kao(:, 2, 9, 2) = (/ &
     &7.7627e-04_rb,1.0704e-03_rb,1.1043e-03_rb,1.0586e-03_rb,9.6032e-04_rb,8.3995e-04_rb, &
     &6.6547e-04_rb,4.2636e-04_rb,2.5832e-04_rb/)
      kao(:, 3, 9, 2) = (/ &
     &9.3794e-04_rb,1.2619e-03_rb,1.2949e-03_rb,1.2284e-03_rb,1.1156e-03_rb,9.7182e-04_rb, &
     &7.6979e-04_rb,5.0109e-04_rb,3.1973e-04_rb/)
      kao(:, 4, 9, 2) = (/ &
     &1.1114e-03_rb,1.4537e-03_rb,1.4722e-03_rb,1.4030e-03_rb,1.2802e-03_rb,1.1177e-03_rb, &
     &8.8445e-04_rb,5.8120e-04_rb,3.8922e-04_rb/)
      kao(:, 5, 9, 2) = (/ &
     &1.2779e-03_rb,1.6415e-03_rb,1.6560e-03_rb,1.5834e-03_rb,1.4519e-03_rb,1.2713e-03_rb, &
     &1.0105e-03_rb,6.6729e-04_rb,4.6624e-04_rb/)
      kao(:, 1,10, 2) = (/ &
     &2.4553e-03_rb,2.6165e-03_rb,2.4993e-03_rb,2.2847e-03_rb,2.0267e-03_rb,1.7115e-03_rb, &
     &1.3397e-03_rb,8.6368e-04_rb,3.6576e-04_rb/)
      kao(:, 2,10, 2) = (/ &
     &3.0620e-03_rb,3.2511e-03_rb,3.1049e-03_rb,2.8553e-03_rb,2.5348e-03_rb,2.1265e-03_rb, &
     &1.6314e-03_rb,1.0269e-03_rb,4.5803e-04_rb/)
      kao(:, 3,10, 2) = (/ &
     &3.7364e-03_rb,3.9627e-03_rb,3.7660e-03_rb,3.4224e-03_rb,3.0121e-03_rb,2.5166e-03_rb, &
     &1.9135e-03_rb,1.1990e-03_rb,5.6685e-04_rb/)
      kao(:, 4,10, 2) = (/ &
     &4.4530e-03_rb,4.6554e-03_rb,4.3836e-03_rb,3.9682e-03_rb,3.4829e-03_rb,2.8809e-03_rb, &
     &2.1972e-03_rb,1.3815e-03_rb,6.9188e-04_rb/)
      kao(:, 5,10, 2) = (/ &
     &5.1575e-03_rb,5.3264e-03_rb,4.9909e-03_rb,4.5087e-03_rb,3.9257e-03_rb,3.2602e-03_rb, &
     &2.4888e-03_rb,1.5715e-03_rb,8.2270e-04_rb/)
      kao(:, 1,11, 2) = (/ &
     &3.7833e-03_rb,3.8112e-03_rb,3.5566e-03_rb,3.1989e-03_rb,2.7686e-03_rb,2.2723e-03_rb, &
     &1.7364e-03_rb,1.0753e-03_rb,4.0590e-04_rb/)
      kao(:, 2,11, 2) = (/ &
     &4.6966e-03_rb,4.7207e-03_rb,4.4156e-03_rb,3.9736e-03_rb,3.4554e-03_rb,2.8577e-03_rb, &
     &2.1338e-03_rb,1.2945e-03_rb,5.0517e-04_rb/)
      kao(:, 3,11, 2) = (/ &
     &5.7189e-03_rb,5.7590e-03_rb,5.3345e-03_rb,4.7708e-03_rb,4.1084e-03_rb,3.3729e-03_rb, &
     &2.5008e-03_rb,1.5078e-03_rb,6.3337e-04_rb/)
      kao(:, 4,11, 2) = (/ &
     &6.7827e-03_rb,6.7469e-03_rb,6.2278e-03_rb,5.5367e-03_rb,4.7380e-03_rb,3.8594e-03_rb, &
     &2.8648e-03_rb,1.7294e-03_rb,7.7415e-04_rb/)
      kao(:, 5,11, 2) = (/ &
     &7.8200e-03_rb,7.7382e-03_rb,7.0901e-03_rb,6.2644e-03_rb,5.3533e-03_rb,4.3510e-03_rb, &
     &3.2356e-03_rb,1.9722e-03_rb,9.3312e-04_rb/)
      kao(:, 1,12, 2) = (/ &
     &4.2783e-03_rb,4.2041e-03_rb,3.8742e-03_rb,3.4634e-03_rb,2.9720e-03_rb,2.4393e-03_rb, &
     &1.8351e-03_rb,1.1175e-03_rb,3.9253e-04_rb/)
      kao(:, 2,12, 2) = (/ &
     &5.3337e-03_rb,5.2652e-03_rb,4.8301e-03_rb,4.3189e-03_rb,3.7223e-03_rb,3.0228e-03_rb, &
     &2.2408e-03_rb,1.3443e-03_rb,4.8795e-04_rb/)
      kao(:, 3,12, 2) = (/ &
     &6.5143e-03_rb,6.3579e-03_rb,5.8237e-03_rb,5.1812e-03_rb,4.4118e-03_rb,3.5721e-03_rb, &
     &2.6281e-03_rb,1.5684e-03_rb,6.0394e-04_rb/)
      kao(:, 4,12, 2) = (/ &
     &7.6468e-03_rb,7.4276e-03_rb,6.7625e-03_rb,6.0120e-03_rb,5.0805e-03_rb,4.0892e-03_rb, &
     &3.0024e-03_rb,1.8032e-03_rb,7.4856e-04_rb/)
      kao(:, 5,12, 2) = (/ &
     &8.7322e-03_rb,8.4845e-03_rb,7.6878e-03_rb,6.7875e-03_rb,5.7064e-03_rb,4.6095e-03_rb, &
     &3.4027e-03_rb,2.0497e-03_rb,9.3030e-04_rb/)
      kao(:, 1,13, 2) = (/ &
     &3.8383e-03_rb,3.7685e-03_rb,3.4781e-03_rb,3.1000e-03_rb,2.6661e-03_rb,2.1777e-03_rb, &
     &1.6207e-03_rb,9.7395e-04_rb,3.4434e-04_rb/)
      kao(:, 2,13, 2) = (/ &
     &4.7991e-03_rb,4.7287e-03_rb,4.3604e-03_rb,3.8801e-03_rb,3.3054e-03_rb,2.6658e-03_rb, &
     &1.9755e-03_rb,1.1692e-03_rb,4.3018e-04_rb/)
      kao(:, 3,13, 2) = (/ &
     &5.8240e-03_rb,5.6632e-03_rb,5.1895e-03_rb,4.6114e-03_rb,3.9052e-03_rb,3.1338e-03_rb, &
     &2.3066e-03_rb,1.3724e-03_rb,5.3490e-04_rb/)
      kao(:, 4,13, 2) = (/ &
     &6.7942e-03_rb,6.6067e-03_rb,6.0062e-03_rb,5.2927e-03_rb,4.4607e-03_rb,3.5870e-03_rb, &
     &2.6413e-03_rb,1.5766e-03_rb,6.6330e-04_rb/)
      kao(:, 5,13, 2) = (/ &
     &7.7135e-03_rb,7.4856e-03_rb,6.7623e-03_rb,5.9543e-03_rb,5.0216e-03_rb,4.0477e-03_rb, &
     &2.9856e-03_rb,1.7918e-03_rb,8.1116e-04_rb/)
      kao(:, 1, 1, 3) = (/ &
     &2.4793e-05_rb,3.0799e-04_rb,4.6290e-04_rb,5.9325e-04_rb,7.1737e-04_rb,8.2193e-04_rb, &
     &9.4048e-04_rb,1.1399e-03_rb,1.0252e-03_rb/)
      kao(:, 2, 1, 3) = (/ &
     &2.6296e-05_rb,3.5350e-04_rb,5.3963e-04_rb,6.9573e-04_rb,8.4910e-04_rb,9.9005e-04_rb, &
     &1.1353e-03_rb,1.3914e-03_rb,1.2872e-03_rb/)
      kao(:, 3, 1, 3) = (/ &
     &2.7819e-05_rb,4.0345e-04_rb,6.2337e-04_rb,8.1130e-04_rb,9.9260e-04_rb,1.1727e-03_rb, &
     &1.3740e-03_rb,1.6958e-03_rb,1.5768e-03_rb/)
      kao(:, 4, 1, 3) = (/ &
     &2.9505e-05_rb,4.5829e-04_rb,7.1308e-04_rb,9.3786e-04_rb,1.1508e-03_rb,1.3827e-03_rb, &
     &1.6466e-03_rb,2.0604e-03_rb,1.9074e-03_rb/)
      kao(:, 5, 1, 3) = (/ &
     &3.1281e-05_rb,5.1795e-04_rb,8.1473e-04_rb,1.0770e-03_rb,1.3366e-03_rb,1.6108e-03_rb, &
     &1.9547e-03_rb,2.4904e-03_rb,2.2803e-03_rb/)
      kao(:, 1, 2, 3) = (/ &
     &3.2793e-05_rb,3.0694e-04_rb,4.5221e-04_rb,5.6459e-04_rb,6.5806e-04_rb,7.4456e-04_rb, &
     &8.0925e-04_rb,9.1019e-04_rb,8.6945e-04_rb/)
      kao(:, 2, 2, 3) = (/ &
     &3.4666e-05_rb,3.5123e-04_rb,5.2466e-04_rb,6.5968e-04_rb,7.7040e-04_rb,8.8249e-04_rb, &
     &9.7906e-04_rb,1.1056e-03_rb,1.0654e-03_rb/)
      kao(:, 3, 2, 3) = (/ &
     &3.6731e-05_rb,4.0004e-04_rb,6.0292e-04_rb,7.6491e-04_rb,9.0091e-04_rb,1.0289e-03_rb, &
     &1.1668e-03_rb,1.3438e-03_rb,1.2804e-03_rb/)
      kao(:, 4, 2, 3) = (/ &
     &3.8918e-05_rb,4.5420e-04_rb,6.8900e-04_rb,8.8273e-04_rb,1.0463e-03_rb,1.2043e-03_rb, &
     &1.3845e-03_rb,1.6322e-03_rb,1.5338e-03_rb/)
      kao(:, 5, 2, 3) = (/ &
     &4.1070e-05_rb,5.1139e-04_rb,7.8412e-04_rb,1.0082e-03_rb,1.2070e-03_rb,1.4003e-03_rb, &
     &1.6106e-03_rb,1.9566e-03_rb,1.8338e-03_rb/)
      kao(:, 1, 3, 3) = (/ &
     &5.7419e-05_rb,3.4417e-04_rb,4.7657e-04_rb,5.8011e-04_rb,6.5776e-04_rb,7.1017e-04_rb, &
     &7.4352e-04_rb,7.5165e-04_rb,7.1905e-04_rb/)
      kao(:, 2, 3, 3) = (/ &
     &6.1098e-05_rb,3.8894e-04_rb,5.5050e-04_rb,6.7283e-04_rb,7.6814e-04_rb,8.3562e-04_rb, &
     &8.7563e-04_rb,9.0812e-04_rb,8.7884e-04_rb/)
      kao(:, 3, 3, 3) = (/ &
     &6.4950e-05_rb,4.3994e-04_rb,6.3159e-04_rb,7.7584e-04_rb,8.9433e-04_rb,9.7562e-04_rb, &
     &1.0278e-03_rb,1.0860e-03_rb,1.0655e-03_rb/)
      kao(:, 4, 3, 3) = (/ &
     &6.9137e-05_rb,4.9606e-04_rb,7.1905e-04_rb,8.8979e-04_rb,1.0295e-03_rb,1.1315e-03_rb, &
     &1.2033e-03_rb,1.2952e-03_rb,1.2839e-03_rb/)
      kao(:, 5, 3, 3) = (/ &
     &7.3375e-05_rb,5.5536e-04_rb,8.1436e-04_rb,1.0155e-03_rb,1.1795e-03_rb,1.3051e-03_rb, &
     &1.4046e-03_rb,1.5168e-03_rb,1.5296e-03_rb/)
      kao(:, 1, 4, 3) = (/ &
     &1.0086e-04_rb,4.2454e-04_rb,5.2985e-04_rb,6.0848e-04_rb,6.7121e-04_rb,7.0723e-04_rb, &
     &7.1237e-04_rb,6.8193e-04_rb,6.1640e-04_rb/)
      kao(:, 2, 4, 3) = (/ &
     &1.0794e-04_rb,4.6886e-04_rb,6.0099e-04_rb,7.0397e-04_rb,7.8290e-04_rb,8.2924e-04_rb, &
     &8.4042e-04_rb,8.0656e-04_rb,7.3923e-04_rb/)
      kao(:, 3, 4, 3) = (/ &
     &1.1564e-04_rb,5.1937e-04_rb,6.8316e-04_rb,8.0764e-04_rb,9.0488e-04_rb,9.5975e-04_rb, &
     &9.8680e-04_rb,9.5115e-04_rb,8.9690e-04_rb/)
      kao(:, 4, 4, 3) = (/ &
     &1.2339e-04_rb,5.7610e-04_rb,7.7301e-04_rb,9.2354e-04_rb,1.0350e-03_rb,1.1092e-03_rb, &
     &1.1493e-03_rb,1.1169e-03_rb,1.0760e-03_rb/)
      kao(:, 5, 4, 3) = (/ &
     &1.3105e-04_rb,6.3905e-04_rb,8.6996e-04_rb,1.0475e-03_rb,1.1809e-03_rb,1.2763e-03_rb, &
     &1.3289e-03_rb,1.3078e-03_rb,1.2869e-03_rb/)
      kao(:, 1, 5, 3) = (/ &
     &1.6100e-04_rb,5.0634e-04_rb,6.1668e-04_rb,6.6467e-04_rb,6.8745e-04_rb,6.9758e-04_rb, &
     &6.8538e-04_rb,6.2098e-04_rb,6.1721e-04_rb/)
      kao(:, 2, 5, 3) = (/ &
     &1.7365e-04_rb,5.5505e-04_rb,6.8253e-04_rb,7.5603e-04_rb,7.9631e-04_rb,8.1600e-04_rb, &
     &8.0888e-04_rb,7.3807e-04_rb,7.1431e-04_rb/)
      kao(:, 3, 5, 3) = (/ &
     &1.8648e-04_rb,6.0992e-04_rb,7.6141e-04_rb,8.5901e-04_rb,9.1428e-04_rb,9.4703e-04_rb, &
     &9.4315e-04_rb,8.7432e-04_rb,8.2918e-04_rb/)
      kao(:, 4, 5, 3) = (/ &
     &1.9988e-04_rb,6.7074e-04_rb,8.4873e-04_rb,9.7496e-04_rb,1.0437e-03_rb,1.0922e-03_rb, &
     &1.0916e-03_rb,1.0269e-03_rb,9.7489e-04_rb/)
      kao(:, 5, 5, 3) = (/ &
     &2.1403e-04_rb,7.3152e-04_rb,9.4776e-04_rb,1.0959e-03_rb,1.1876e-03_rb,1.2477e-03_rb, &
     &1.2592e-03_rb,1.1994e-03_rb,1.1556e-03_rb/)
      kao(:, 1, 6, 3) = (/ &
     &2.3782e-04_rb,5.8473e-04_rb,7.0057e-04_rb,7.3727e-04_rb,7.3866e-04_rb,7.0622e-04_rb, &
     &6.5578e-04_rb,5.7152e-04_rb,6.4834e-04_rb/)
      kao(:, 2, 6, 3) = (/ &
     &2.5744e-04_rb,6.5190e-04_rb,7.6744e-04_rb,8.2296e-04_rb,8.3569e-04_rb,8.1871e-04_rb, &
     &7.7021e-04_rb,6.8232e-04_rb,7.1639e-04_rb/)
      kao(:, 3, 6, 3) = (/ &
     &2.7664e-04_rb,7.1488e-04_rb,8.5237e-04_rb,9.1762e-04_rb,9.4622e-04_rb,9.4392e-04_rb, &
     &8.9795e-04_rb,8.0363e-04_rb,8.0674e-04_rb/)
      kao(:, 4, 6, 3) = (/ &
     &2.9757e-04_rb,7.7689e-04_rb,9.3669e-04_rb,1.0212e-03_rb,1.0750e-03_rb,1.0793e-03_rb, &
     &1.0394e-03_rb,9.3920e-04_rb,9.2478e-04_rb/)
      kao(:, 5, 6, 3) = (/ &
     &3.2013e-04_rb,8.4833e-04_rb,1.0249e-03_rb,1.1443e-03_rb,1.2150e-03_rb,1.2313e-03_rb, &
     &1.1962e-03_rb,1.0926e-03_rb,1.0558e-03_rb/)
      kao(:, 1, 7, 3) = (/ &
     &3.7226e-04_rb,7.2122e-04_rb,8.2273e-04_rb,8.7424e-04_rb,8.4905e-04_rb,7.9504e-04_rb, &
     &6.8600e-04_rb,5.4748e-04_rb,6.4757e-04_rb/)
      kao(:, 2, 7, 3) = (/ &
     &4.0471e-04_rb,8.0580e-04_rb,9.2241e-04_rb,9.6290e-04_rb,9.4856e-04_rb,8.9436e-04_rb, &
     &7.9088e-04_rb,6.5024e-04_rb,7.6906e-04_rb/)
      kao(:, 3, 7, 3) = (/ &
     &4.3485e-04_rb,8.8728e-04_rb,1.0201e-03_rb,1.0664e-03_rb,1.0598e-03_rb,1.0046e-03_rb, &
     &9.1434e-04_rb,7.6471e-04_rb,8.8345e-04_rb/)
      kao(:, 4, 7, 3) = (/ &
     &4.6779e-04_rb,9.6940e-04_rb,1.1214e-03_rb,1.1802e-03_rb,1.1771e-03_rb,1.1317e-03_rb, &
     &1.0506e-03_rb,8.9487e-04_rb,9.7870e-04_rb/)
      kao(:, 5, 7, 3) = (/ &
     &5.0564e-04_rb,1.0493e-03_rb,1.2285e-03_rb,1.2989e-03_rb,1.3103e-03_rb,1.2822e-03_rb, &
     &1.2004e-03_rb,1.0399e-03_rb,1.0827e-03_rb/)
      kao(:, 1, 8, 3) = (/ &
     &6.8834e-04_rb,1.0497e-03_rb,1.1354e-03_rb,1.1403e-03_rb,1.1091e-03_rb,1.0282e-03_rb, &
     &8.6922e-04_rb,6.2179e-04_rb,6.4394e-04_rb/)
      kao(:, 2, 8, 3) = (/ &
     &7.5262e-04_rb,1.1719e-03_rb,1.2711e-03_rb,1.2847e-03_rb,1.2498e-03_rb,1.1356e-03_rb, &
     &9.7177e-04_rb,7.1700e-04_rb,7.5449e-04_rb/)
      kao(:, 3, 8, 3) = (/ &
     &8.1196e-04_rb,1.2925e-03_rb,1.4075e-03_rb,1.4221e-03_rb,1.3886e-03_rb,1.2652e-03_rb, &
     &1.0898e-03_rb,8.2946e-04_rb,8.8164e-04_rb/)
      kao(:, 4, 8, 3) = (/ &
     &8.7357e-04_rb,1.4119e-03_rb,1.5462e-03_rb,1.5662e-03_rb,1.5234e-03_rb,1.4066e-03_rb, &
     &1.2187e-03_rb,9.5583e-04_rb,1.0263e-03_rb/)
      kao(:, 5, 8, 3) = (/ &
     &9.4776e-04_rb,1.5351e-03_rb,1.6779e-03_rb,1.7191e-03_rb,1.6760e-03_rb,1.5541e-03_rb, &
     &1.3711e-03_rb,1.0942e-03_rb,1.1866e-03_rb/)
      kao(:, 1, 9, 3) = (/ &
     &2.4030e-03_rb,2.8116e-03_rb,2.7490e-03_rb,2.5782e-03_rb,2.3739e-03_rb,2.0451e-03_rb, &
     &1.6352e-03_rb,1.1525e-03_rb,7.8397e-04_rb/)
      kao(:, 2, 9, 3) = (/ &
     &2.6406e-03_rb,3.0780e-03_rb,3.0311e-03_rb,2.8880e-03_rb,2.6562e-03_rb,2.3012e-03_rb, &
     &1.8579e-03_rb,1.2942e-03_rb,9.5305e-04_rb/)
      kao(:, 3, 9, 3) = (/ &
     &2.8588e-03_rb,3.3826e-03_rb,3.3331e-03_rb,3.2003e-03_rb,2.9491e-03_rb,2.5645e-03_rb, &
     &2.0822e-03_rb,1.4320e-03_rb,1.1371e-03_rb/)
      kao(:, 4, 9, 3) = (/ &
     &3.0751e-03_rb,3.6837e-03_rb,3.6521e-03_rb,3.5172e-03_rb,3.2330e-03_rb,2.8266e-03_rb, &
     &2.3022e-03_rb,1.5958e-03_rb,1.3270e-03_rb/)
      kao(:, 5, 9, 3) = (/ &
     &3.3401e-03_rb,4.0139e-03_rb,3.9712e-03_rb,3.8387e-03_rb,3.5273e-03_rb,3.0908e-03_rb, &
     &2.5332e-03_rb,1.7734e-03_rb,1.5421e-03_rb/)
      kao(:, 1,10, 3) = (/ &
     &9.4661e-03_rb,9.3672e-03_rb,8.6815e-03_rb,7.7670e-03_rb,6.6533e-03_rb,5.4267e-03_rb, &
     &4.0804e-03_rb,2.5558e-03_rb,1.0683e-03_rb/)
      kao(:, 2,10, 3) = (/ &
     &1.0403e-02_rb,1.0310e-02_rb,9.5215e-03_rb,8.5122e-03_rb,7.3124e-03_rb,5.9813e-03_rb, &
     &4.5509e-03_rb,2.8739e-03_rb,1.2946e-03_rb/)
      kao(:, 3,10, 3) = (/ &
     &1.1289e-02_rb,1.1218e-02_rb,1.0349e-02_rb,9.3127e-03_rb,8.0182e-03_rb,6.6138e-03_rb, &
     &5.0551e-03_rb,3.2072e-03_rb,1.5622e-03_rb/)
      kao(:, 4,10, 3) = (/ &
     &1.2130e-02_rb,1.2221e-02_rb,1.1320e-02_rb,1.0189e-02_rb,8.7979e-03_rb,7.2832e-03_rb, &
     &5.5873e-03_rb,3.5427e-03_rb,1.8389e-03_rb/)
      kao(:, 5,10, 3) = (/ &
     &1.3100e-02_rb,1.3300e-02_rb,1.2340e-02_rb,1.1131e-02_rb,9.6189e-03_rb,7.9428e-03_rb, &
     &6.1291e-03_rb,3.8793e-03_rb,2.1440e-03_rb/)
      kao(:, 1,11, 3) = (/ &
     &1.3487e-02_rb,1.2984e-02_rb,1.1782e-02_rb,1.0398e-02_rb,8.8336e-03_rb,7.1334e-03_rb, &
     &5.2262e-03_rb,3.1590e-03_rb,1.1845e-03_rb/)
      kao(:, 2,11, 3) = (/ &
     &1.4804e-02_rb,1.4185e-02_rb,1.2823e-02_rb,1.1346e-02_rb,9.6530e-03_rb,7.7631e-03_rb, &
     &5.7673e-03_rb,3.5383e-03_rb,1.4275e-03_rb/)
      kao(:, 3,11, 3) = (/ &
     &1.5980e-02_rb,1.5367e-02_rb,1.3971e-02_rb,1.2366e-02_rb,1.0542e-02_rb,8.5565e-03_rb, &
     &6.3780e-03_rb,3.9351e-03_rb,1.7235e-03_rb/)
      kao(:, 4,11, 3) = (/ &
     &1.7219e-02_rb,1.6745e-02_rb,1.5253e-02_rb,1.3475e-02_rb,1.1557e-02_rb,9.4243e-03_rb, &
     &6.9880e-03_rb,4.3338e-03_rb,2.0121e-03_rb/)
      kao(:, 5,11, 3) = (/ &
     &1.8659e-02_rb,1.8178e-02_rb,1.6582e-02_rb,1.4739e-02_rb,1.2667e-02_rb,1.0254e-02_rb, &
     &7.6524e-03_rb,4.7481e-03_rb,2.3669e-03_rb/)
      kao(:, 1,12, 3) = (/ &
     &1.4430e-02_rb,1.3711e-02_rb,1.2344e-02_rb,1.0793e-02_rb,9.1312e-03_rb,7.2788e-03_rb, &
     &5.2885e-03_rb,3.1556e-03_rb,1.1392e-03_rb/)
      kao(:, 2,12, 3) = (/ &
     &1.5757e-02_rb,1.4929e-02_rb,1.3431e-02_rb,1.1729e-02_rb,9.9240e-03_rb,7.9738e-03_rb, &
     &5.8467e-03_rb,3.5123e-03_rb,1.3911e-03_rb/)
      kao(:, 3,12, 3) = (/ &
     &1.6948e-02_rb,1.6169e-02_rb,1.4634e-02_rb,1.2830e-02_rb,1.0856e-02_rb,8.7512e-03_rb, &
     &6.4418e-03_rb,3.8765e-03_rb,1.6585e-03_rb/)
      kao(:, 4,12, 3) = (/ &
     &1.8340e-02_rb,1.7651e-02_rb,1.6013e-02_rb,1.4019e-02_rb,1.1962e-02_rb,9.6662e-03_rb, &
     &7.0623e-03_rb,4.2621e-03_rb,1.9806e-03_rb/)
      kao(:, 5,12, 3) = (/ &
     &1.9917e-02_rb,1.9194e-02_rb,1.7450e-02_rb,1.5375e-02_rb,1.3136e-02_rb,1.0559e-02_rb, &
     &7.7061e-03_rb,4.6735e-03_rb,2.2886e-03_rb/)
      kao(:, 1,13, 3) = (/ &
     &1.2378e-02_rb,1.1740e-02_rb,1.0520e-02_rb,9.1794e-03_rb,7.7313e-03_rb,6.1547e-03_rb, &
     &4.4837e-03_rb,2.6655e-03_rb,9.8041e-04_rb/)
      kao(:, 2,13, 3) = (/ &
     &1.3400e-02_rb,1.2704e-02_rb,1.1442e-02_rb,9.9607e-03_rb,8.4166e-03_rb,6.7824e-03_rb, &
     &4.9464e-03_rb,2.9635e-03_rb,1.2027e-03_rb/)
      kao(:, 3,13, 3) = (/ &
     &1.4376e-02_rb,1.3828e-02_rb,1.2497e-02_rb,1.0944e-02_rb,9.2894e-03_rb,7.4859e-03_rb, &
     &5.4627e-03_rb,3.2639e-03_rb,1.4455e-03_rb/)
      kao(:, 4,13, 3) = (/ &
     &1.5663e-02_rb,1.5087e-02_rb,1.3675e-02_rb,1.2023e-02_rb,1.0248e-02_rb,8.2522e-03_rb, &
     &5.9994e-03_rb,3.5980e-03_rb,1.7132e-03_rb/)
      kao(:, 5,13, 3) = (/ &
     &1.7084e-02_rb,1.6538e-02_rb,1.5027e-02_rb,1.3180e-02_rb,1.1220e-02_rb,9.0513e-03_rb, &
     &6.5567e-03_rb,3.9396e-03_rb,2.0154e-03_rb/)
      kao(:, 1, 1, 4) = (/ &
     &4.5432e-05_rb,6.4085e-04_rb,1.0470e-03_rb,1.4404e-03_rb,1.8333e-03_rb,2.2857e-03_rb, &
     &2.8540e-03_rb,3.7474e-03_rb,3.3078e-03_rb/)
      kao(:, 2, 1, 4) = (/ &
     &4.6775e-05_rb,7.3260e-04_rb,1.2196e-03_rb,1.6906e-03_rb,2.1700e-03_rb,2.7279e-03_rb, &
     &3.3928e-03_rb,4.5435e-03_rb,4.0031e-03_rb/)
      kao(:, 3, 1, 4) = (/ &
     &4.7907e-05_rb,8.4030e-04_rb,1.4168e-03_rb,1.9771e-03_rb,2.5623e-03_rb,3.2199e-03_rb, &
     &4.0380e-03_rb,5.3656e-03_rb,4.7474e-03_rb/)
      kao(:, 4, 1, 4) = (/ &
     &4.8733e-05_rb,9.6171e-04_rb,1.6416e-03_rb,2.3126e-03_rb,3.0275e-03_rb,3.7946e-03_rb, &
     &4.7571e-03_rb,6.3684e-03_rb,5.6396e-03_rb/)
      kao(:, 5, 1, 4) = (/ &
     &4.9568e-05_rb,1.0960e-03_rb,1.8926e-03_rb,2.6893e-03_rb,3.5365e-03_rb,4.4677e-03_rb, &
     &5.5986e-03_rb,7.4848e-03_rb,6.6760e-03_rb/)
      kao(:, 1, 2, 4) = (/ &
     &5.9644e-05_rb,6.5165e-04_rb,9.7095e-04_rb,1.2735e-03_rb,1.5693e-03_rb,1.8723e-03_rb, &
     &2.2435e-03_rb,2.8303e-03_rb,2.5518e-03_rb/)
      kao(:, 2, 2, 4) = (/ &
     &6.1550e-05_rb,7.3433e-04_rb,1.1155e-03_rb,1.4862e-03_rb,1.8480e-03_rb,2.2144e-03_rb, &
     &2.6782e-03_rb,3.3851e-03_rb,3.1028e-03_rb/)
      kao(:, 3, 2, 4) = (/ &
     &6.3424e-05_rb,8.2814e-04_rb,1.2862e-03_rb,1.7299e-03_rb,2.1667e-03_rb,2.6220e-03_rb, &
     &3.1654e-03_rb,4.0290e-03_rb,3.7219e-03_rb/)
      kao(:, 4, 2, 4) = (/ &
     &6.4618e-05_rb,9.2770e-04_rb,1.4812e-03_rb,2.0000e-03_rb,2.5281e-03_rb,3.0889e-03_rb, &
     &3.7301e-03_rb,4.7467e-03_rb,4.4551e-03_rb/)
      kao(:, 5, 2, 4) = (/ &
     &6.6495e-05_rb,1.0413e-03_rb,1.6912e-03_rb,2.3096e-03_rb,2.9324e-03_rb,3.6063e-03_rb, &
     &4.3915e-03_rb,5.5831e-03_rb,5.2779e-03_rb/)
      kao(:, 1, 3, 4) = (/ &
     &1.0462e-04_rb,7.2939e-04_rb,1.0510e-03_rb,1.2708e-03_rb,1.4590e-03_rb,1.6268e-03_rb, &
     &1.8071e-03_rb,2.0571e-03_rb,1.9983e-03_rb/)
      kao(:, 2, 3, 4) = (/ &
     &1.0853e-04_rb,8.1591e-04_rb,1.1915e-03_rb,1.4605e-03_rb,1.6878e-03_rb,1.9079e-03_rb, &
     &2.1403e-03_rb,2.4652e-03_rb,2.3859e-03_rb/)
      kao(:, 3, 3, 4) = (/ &
     &1.1161e-04_rb,9.0807e-04_rb,1.3452e-03_rb,1.6696e-03_rb,1.9436e-03_rb,2.2298e-03_rb, &
     &2.5245e-03_rb,2.9256e-03_rb,2.8236e-03_rb/)
      kao(:, 4, 3, 4) = (/ &
     &1.1463e-04_rb,1.0096e-03_rb,1.5126e-03_rb,1.9029e-03_rb,2.2460e-03_rb,2.5957e-03_rb, &
     &2.9637e-03_rb,3.4539e-03_rb,3.3286e-03_rb/)
      kao(:, 5, 3, 4) = (/ &
     &1.1771e-04_rb,1.1227e-03_rb,1.6998e-03_rb,2.1624e-03_rb,2.5867e-03_rb,3.0031e-03_rb, &
     &3.4507e-03_rb,4.0691e-03_rb,3.9216e-03_rb/)
      kao(:, 1, 4, 4) = (/ &
     &1.8980e-04_rb,8.1690e-04_rb,1.1496e-03_rb,1.3743e-03_rb,1.5174e-03_rb,1.5997e-03_rb, &
     &1.6364e-03_rb,1.6486e-03_rb,1.6867e-03_rb/)
      kao(:, 2, 4, 4) = (/ &
     &1.9681e-04_rb,9.1682e-04_rb,1.2947e-03_rb,1.5538e-03_rb,1.7351e-03_rb,1.8526e-03_rb, &
     &1.9065e-03_rb,1.9583e-03_rb,2.0215e-03_rb/)
      kao(:, 3, 4, 4) = (/ &
     &2.0208e-04_rb,1.0146e-03_rb,1.4514e-03_rb,1.7516e-03_rb,1.9841e-03_rb,2.1383e-03_rb, &
     &2.2138e-03_rb,2.3082e-03_rb,2.3998e-03_rb/)
      kao(:, 4, 4, 4) = (/ &
     &2.0762e-04_rb,1.1226e-03_rb,1.6211e-03_rb,1.9728e-03_rb,2.2623e-03_rb,2.4506e-03_rb, &
     &2.5563e-03_rb,2.7166e-03_rb,2.8343e-03_rb/)
      kao(:, 5, 4, 4) = (/ &
     &2.1385e-04_rb,1.2397e-03_rb,1.8026e-03_rb,2.2223e-03_rb,2.5659e-03_rb,2.7894e-03_rb, &
     &2.9545e-03_rb,3.1771e-03_rb,3.3076e-03_rb/)
      kao(:, 1, 5, 4) = (/ &
     &3.1379e-04_rb,9.3426e-04_rb,1.2166e-03_rb,1.4285e-03_rb,1.5654e-03_rb,1.6404e-03_rb, &
     &1.5965e-03_rb,1.4828e-03_rb,1.3604e-03_rb/)
      kao(:, 2, 5, 4) = (/ &
     &3.2699e-04_rb,1.0358e-03_rb,1.3783e-03_rb,1.6163e-03_rb,1.7786e-03_rb,1.8763e-03_rb, &
     &1.8530e-03_rb,1.7399e-03_rb,1.6555e-03_rb/)
      kao(:, 3, 5, 4) = (/ &
     &3.3749e-04_rb,1.1387e-03_rb,1.5374e-03_rb,1.8184e-03_rb,2.0178e-03_rb,2.1215e-03_rb, &
     &2.1453e-03_rb,2.0239e-03_rb,1.9794e-03_rb/)
      kao(:, 4, 5, 4) = (/ &
     &3.4807e-04_rb,1.2529e-03_rb,1.7131e-03_rb,2.0397e-03_rb,2.2808e-03_rb,2.4051e-03_rb, &
     &2.4718e-03_rb,2.3502e-03_rb,2.3300e-03_rb/)
      kao(:, 5, 5, 4) = (/ &
     &3.5954e-04_rb,1.3777e-03_rb,1.8982e-03_rb,2.2796e-03_rb,2.5586e-03_rb,2.7323e-03_rb, &
     &2.8236e-03_rb,2.7161e-03_rb,2.7237e-03_rb/)
      kao(:, 1, 6, 4) = (/ &
     &4.7800e-04_rb,1.0890e-03_rb,1.3133e-03_rb,1.4652e-03_rb,1.5696e-03_rb,1.6077e-03_rb, &
     &1.5816e-03_rb,1.3782e-03_rb,1.1543e-03_rb/)
      kao(:, 2, 6, 4) = (/ &
     &5.0135e-04_rb,1.1692e-03_rb,1.4789e-03_rb,1.6725e-03_rb,1.7871e-03_rb,1.8348e-03_rb, &
     &1.8106e-03_rb,1.6080e-03_rb,1.4185e-03_rb/)
      kao(:, 3, 6, 4) = (/ &
     &5.2250e-04_rb,1.2684e-03_rb,1.6447e-03_rb,1.8893e-03_rb,2.0173e-03_rb,2.0845e-03_rb, &
     &2.0641e-03_rb,1.8764e-03_rb,1.6970e-03_rb/)
      kao(:, 4, 6, 4) = (/ &
     &5.4010e-04_rb,1.3890e-03_rb,1.8350e-03_rb,2.1143e-03_rb,2.2731e-03_rb,2.3591e-03_rb, &
     &2.3476e-03_rb,2.1762e-03_rb,2.0061e-03_rb/)
      kao(:, 5, 6, 4) = (/ &
     &5.6086e-04_rb,1.5152e-03_rb,2.0323e-03_rb,2.3522e-03_rb,2.5434e-03_rb,2.6585e-03_rb, &
     &2.6646e-03_rb,2.5043e-03_rb,2.3580e-03_rb/)
      kao(:, 1, 7, 4) = (/ &
     &7.6472e-04_rb,1.4103e-03_rb,1.5811e-03_rb,1.6521e-03_rb,1.6700e-03_rb,1.6355e-03_rb, &
     &1.5716e-03_rb,1.3655e-03_rb,1.1953e-03_rb/)
      kao(:, 2, 7, 4) = (/ &
     &8.0405e-04_rb,1.4820e-03_rb,1.7182e-03_rb,1.8432e-03_rb,1.9040e-03_rb,1.8839e-03_rb, &
     &1.8004e-03_rb,1.5836e-03_rb,1.3245e-03_rb/)
      kao(:, 3, 7, 4) = (/ &
     &8.3976e-04_rb,1.5925e-03_rb,1.8706e-03_rb,2.0547e-03_rb,2.1511e-03_rb,2.1580e-03_rb, &
     &2.0497e-03_rb,1.8151e-03_rb,1.5136e-03_rb/)
      kao(:, 4, 7, 4) = (/ &
     &8.7606e-04_rb,1.7044e-03_rb,2.0552e-03_rb,2.2845e-03_rb,2.4212e-03_rb,2.4433e-03_rb, &
     &2.3312e-03_rb,2.0671e-03_rb,1.7685e-03_rb/)
      kao(:, 5, 7, 4) = (/ &
     &9.1065e-04_rb,1.8368e-03_rb,2.2563e-03_rb,2.5472e-03_rb,2.7054e-03_rb,2.7384e-03_rb, &
     &2.6284e-03_rb,2.3513e-03_rb,2.0612e-03_rb/)
      kao(:, 1, 8, 4) = (/ &
     &1.4362e-03_rb,2.1402e-03_rb,2.2733e-03_rb,2.2579e-03_rb,2.1547e-03_rb,1.9686e-03_rb, &
     &1.7196e-03_rb,1.4177e-03_rb,1.4603e-03_rb/)
      kao(:, 2, 8, 4) = (/ &
     &1.5196e-03_rb,2.2635e-03_rb,2.4111e-03_rb,2.4495e-03_rb,2.3642e-03_rb,2.2258e-03_rb, &
     &1.9891e-03_rb,1.6444e-03_rb,1.7173e-03_rb/)
      kao(:, 3, 8, 4) = (/ &
     &1.5863e-03_rb,2.3882e-03_rb,2.5869e-03_rb,2.6588e-03_rb,2.5952e-03_rb,2.5025e-03_rb, &
     &2.2780e-03_rb,1.8819e-03_rb,1.8908e-03_rb/)
      kao(:, 4, 8, 4) = (/ &
     &1.6588e-03_rb,2.5177e-03_rb,2.7792e-03_rb,2.8954e-03_rb,2.8908e-03_rb,2.7990e-03_rb, &
     &2.5915e-03_rb,2.1558e-03_rb,2.0333e-03_rb/)
      kao(:, 5, 8, 4) = (/ &
     &1.7367e-03_rb,2.6935e-03_rb,3.0017e-03_rb,3.1619e-03_rb,3.2065e-03_rb,3.1297e-03_rb, &
     &2.9147e-03_rb,2.4525e-03_rb,2.1765e-03_rb/)
      kao(:, 1, 9, 4) = (/ &
     &5.1377e-03_rb,5.6879e-03_rb,5.6413e-03_rb,5.3776e-03_rb,4.8224e-03_rb,4.1689e-03_rb, &
     &3.3325e-03_rb,2.2481e-03_rb,2.0122e-03_rb/)
      kao(:, 2, 9, 4) = (/ &
     &5.4511e-03_rb,6.1548e-03_rb,6.1332e-03_rb,5.7185e-03_rb,5.1593e-03_rb,4.4551e-03_rb, &
     &3.6302e-03_rb,2.5234e-03_rb,2.3673e-03_rb/)
      kao(:, 3, 9, 4) = (/ &
     &5.7014e-03_rb,6.5729e-03_rb,6.5250e-03_rb,6.0830e-03_rb,5.4507e-03_rb,4.7941e-03_rb, &
     &3.9451e-03_rb,2.8561e-03_rb,2.7249e-03_rb/)
      kao(:, 4, 9, 4) = (/ &
     &5.9876e-03_rb,6.9491e-03_rb,6.9266e-03_rb,6.3994e-03_rb,5.8665e-03_rb,5.2094e-03_rb, &
     &4.3177e-03_rb,3.2100e-03_rb,3.0655e-03_rb/)
      kao(:, 5, 9, 4) = (/ &
     &6.2578e-03_rb,7.3161e-03_rb,7.3395e-03_rb,6.8572e-03_rb,6.3066e-03_rb,5.6391e-03_rb, &
     &4.7410e-03_rb,3.6035e-03_rb,3.3864e-03_rb/)
      kao(:, 1,10, 4) = (/ &
     &2.0796e-02_rb,2.0019e-02_rb,1.8165e-02_rb,1.6068e-02_rb,1.3873e-02_rb,1.1404e-02_rb, &
     &8.6779e-03_rb,5.2784e-03_rb,2.8413e-03_rb/)
      kao(:, 2,10, 4) = (/ &
     &2.2188e-02_rb,2.1487e-02_rb,1.9617e-02_rb,1.7448e-02_rb,1.5124e-02_rb,1.2499e-02_rb, &
     &9.2630e-03_rb,5.6664e-03_rb,3.3963e-03_rb/)
      kao(:, 3,10, 4) = (/ &
     &2.3219e-02_rb,2.2843e-02_rb,2.1086e-02_rb,1.8777e-02_rb,1.6130e-02_rb,1.3209e-02_rb, &
     &9.9059e-03_rb,6.0802e-03_rb,4.0019e-03_rb/)
      kao(:, 4,10, 4) = (/ &
     &2.4435e-02_rb,2.4082e-02_rb,2.2303e-02_rb,1.9851e-02_rb,1.7023e-02_rb,1.4121e-02_rb, &
     &1.0410e-02_rb,6.5843e-03_rb,4.6477e-03_rb/)
      kao(:, 5,10, 4) = (/ &
     &2.5609e-02_rb,2.5343e-02_rb,2.3560e-02_rb,2.0930e-02_rb,1.8042e-02_rb,1.4937e-02_rb, &
     &1.1163e-02_rb,7.1400e-03_rb,5.2598e-03_rb/)
      kao(:, 1,11, 4) = (/ &
     &3.0218e-02_rb,2.8464e-02_rb,2.5566e-02_rb,2.2189e-02_rb,1.8763e-02_rb,1.5165e-02_rb, &
     &1.1286e-02_rb,6.7271e-03_rb,3.0443e-03_rb/)
      kao(:, 2,11, 4) = (/ &
     &3.2017e-02_rb,3.0288e-02_rb,2.7351e-02_rb,2.3960e-02_rb,2.0417e-02_rb,1.6571e-02_rb, &
     &1.2201e-02_rb,7.2159e-03_rb,3.6072e-03_rb/)
      kao(:, 3,11, 4) = (/ &
     &3.3713e-02_rb,3.2141e-02_rb,2.9172e-02_rb,2.5698e-02_rb,2.1768e-02_rb,1.7546e-02_rb, &
     &1.2989e-02_rb,7.6734e-03_rb,4.2021e-03_rb/)
      kao(:, 4,11, 4) = (/ &
     &3.5523e-02_rb,3.3941e-02_rb,3.0802e-02_rb,2.7264e-02_rb,2.3091e-02_rb,1.8512e-02_rb, &
     &1.3848e-02_rb,8.2207e-03_rb,4.9261e-03_rb/)
      kao(:, 5,11, 4) = (/ &
     &3.7048e-02_rb,3.5659e-02_rb,3.2500e-02_rb,2.8766e-02_rb,2.4380e-02_rb,1.9724e-02_rb, &
     &1.4735e-02_rb,8.8773e-03_rb,5.6292e-03_rb/)
      kao(:, 1,12, 4) = (/ &
     &3.2170e-02_rb,2.9965e-02_rb,2.6774e-02_rb,2.3265e-02_rb,1.9604e-02_rb,1.5767e-02_rb, &
     &1.1665e-02_rb,6.8628e-03_rb,2.9693e-03_rb/)
      kao(:, 2,12, 4) = (/ &
     &3.3877e-02_rb,3.1857e-02_rb,2.8686e-02_rb,2.5070e-02_rb,2.1306e-02_rb,1.7043e-02_rb, &
     &1.2460e-02_rb,7.3567e-03_rb,3.5157e-03_rb/)
      kao(:, 3,12, 4) = (/ &
     &3.5821e-02_rb,3.3840e-02_rb,3.0497e-02_rb,2.6646e-02_rb,2.2619e-02_rb,1.8108e-02_rb, &
     &1.3276e-02_rb,7.8561e-03_rb,4.1559e-03_rb/)
      kao(:, 4,12, 4) = (/ &
     &3.7503e-02_rb,3.5506e-02_rb,3.2126e-02_rb,2.8288e-02_rb,2.3902e-02_rb,1.9181e-02_rb, &
     &1.4142e-02_rb,8.3990e-03_rb,4.8088e-03_rb/)
      kao(:, 5,12, 4) = (/ &
     &3.9218e-02_rb,3.7289e-02_rb,3.3764e-02_rb,2.9747e-02_rb,2.5253e-02_rb,2.0345e-02_rb, &
     &1.5175e-02_rb,9.0569e-03_rb,5.5254e-03_rb/)
      kao(:, 1,13, 4) = (/ &
     &2.7021e-02_rb,2.5240e-02_rb,2.2638e-02_rb,1.9764e-02_rb,1.6669e-02_rb,1.3389e-02_rb, &
     &9.8031e-03_rb,5.7609e-03_rb,2.5391e-03_rb/)
      kao(:, 2,13, 4) = (/ &
     &2.8550e-02_rb,2.6888e-02_rb,2.4164e-02_rb,2.1146e-02_rb,1.7859e-02_rb,1.4211e-02_rb, &
     &1.0456e-02_rb,6.1648e-03_rb,3.0359e-03_rb/)
      kao(:, 3,13, 4) = (/ &
     &3.0206e-02_rb,2.8336e-02_rb,2.5560e-02_rb,2.2362e-02_rb,1.8953e-02_rb,1.5151e-02_rb, &
     &1.1086e-02_rb,6.5998e-03_rb,3.5704e-03_rb/)
      kao(:, 4,13, 4) = (/ &
     &3.1495e-02_rb,2.9878e-02_rb,2.6989e-02_rb,2.3626e-02_rb,1.9993e-02_rb,1.6052e-02_rb, &
     &1.1891e-02_rb,7.0977e-03_rb,4.1349e-03_rb/)
      kao(:, 5,13, 4) = (/ &
     &3.2969e-02_rb,3.1259e-02_rb,2.8282e-02_rb,2.5005e-02_rb,2.1232e-02_rb,1.7061e-02_rb, &
     &1.2808e-02_rb,7.6578e-03_rb,4.7180e-03_rb/)
      kao(:, 1, 1, 5) = (/ &
     &7.7208e-05_rb,1.2983e-03_rb,2.1990e-03_rb,3.0698e-03_rb,3.9335e-03_rb,4.8939e-03_rb, &
     &6.0507e-03_rb,7.9864e-03_rb,7.3950e-03_rb/)
      kao(:, 2, 1, 5) = (/ &
     &7.7960e-05_rb,1.4953e-03_rb,2.5372e-03_rb,3.5852e-03_rb,4.6877e-03_rb,5.9141e-03_rb, &
     &7.4453e-03_rb,9.6664e-03_rb,8.9362e-03_rb/)
      kao(:, 3, 1, 5) = (/ &
     &7.8616e-05_rb,1.7027e-03_rb,2.9428e-03_rb,4.1895e-03_rb,5.5241e-03_rb,7.0220e-03_rb, &
     &8.8894e-03_rb,1.1837e-02_rb,1.0698e-02_rb/)
      kao(:, 4, 1, 5) = (/ &
     &7.9284e-05_rb,1.9233e-03_rb,3.3889e-03_rb,4.8861e-03_rb,6.4864e-03_rb,8.2886e-03_rb, &
     &1.0489e-02_rb,1.4031e-02_rb,1.2687e-02_rb/)
      kao(:, 5, 1, 5) = (/ &
     &8.0311e-05_rb,2.1623e-03_rb,3.8730e-03_rb,5.6513e-03_rb,7.5603e-03_rb,9.7064e-03_rb, &
     &1.2306e-02_rb,1.6419e-02_rb,1.4857e-02_rb/)
      kao(:, 1, 2, 5) = (/ &
     &1.0510e-04_rb,1.2010e-03_rb,1.9708e-03_rb,2.6863e-03_rb,3.3801e-03_rb,4.0401e-03_rb, &
     &4.8065e-03_rb,5.9839e-03_rb,5.8720e-03_rb/)
      kao(:, 2, 2, 5) = (/ &
     &1.0618e-04_rb,1.3719e-03_rb,2.2892e-03_rb,3.1040e-03_rb,3.9264e-03_rb,4.7961e-03_rb, &
     &5.8356e-03_rb,7.3351e-03_rb,7.0925e-03_rb/)
      kao(:, 3, 2, 5) = (/ &
     &1.0689e-04_rb,1.5661e-03_rb,2.6266e-03_rb,3.5943e-03_rb,4.5681e-03_rb,5.6180e-03_rb, &
     &6.8759e-03_rb,8.8640e-03_rb,8.4509e-03_rb/)
      kao(:, 4, 2, 5) = (/ &
     &1.0861e-04_rb,1.7752e-03_rb,2.9853e-03_rb,4.1453e-03_rb,5.3192e-03_rb,6.5973e-03_rb, &
     &8.1274e-03_rb,1.0456e-02_rb,1.0031e-02_rb/)
      kao(:, 5, 2, 5) = (/ &
     &1.1004e-04_rb,1.9945e-03_rb,3.3831e-03_rb,4.7368e-03_rb,6.1454e-03_rb,7.6846e-03_rb, &
     &9.5320e-03_rb,1.2312e-02_rb,1.1757e-02_rb/)
      kao(:, 1, 3, 5) = (/ &
     &1.9166e-04_rb,1.3172e-03_rb,1.9278e-03_rb,2.4677e-03_rb,2.9707e-03_rb,3.4895e-03_rb, &
     &3.9440e-03_rb,4.4484e-03_rb,4.3497e-03_rb/)
      kao(:, 2, 3, 5) = (/ &
     &1.9336e-04_rb,1.4888e-03_rb,2.1997e-03_rb,2.8500e-03_rb,3.4656e-03_rb,4.0405e-03_rb, &
     &4.6494e-03_rb,5.3886e-03_rb,5.2876e-03_rb/)
      kao(:, 3, 3, 5) = (/ &
     &1.9552e-04_rb,1.6719e-03_rb,2.4978e-03_rb,3.2794e-03_rb,4.0171e-03_rb,4.6883e-03_rb, &
     &5.3810e-03_rb,6.3663e-03_rb,6.3594e-03_rb/)
      kao(:, 4, 3, 5) = (/ &
     &1.9793e-04_rb,1.8618e-03_rb,2.8193e-03_rb,3.7421e-03_rb,4.6002e-03_rb,5.4246e-03_rb, &
     &6.2790e-03_rb,7.4939e-03_rb,7.6260e-03_rb/)
      kao(:, 5, 3, 5) = (/ &
     &2.0202e-04_rb,2.0622e-03_rb,3.1772e-03_rb,4.2408e-03_rb,5.2345e-03_rb,6.2239e-03_rb, &
     &7.2922e-03_rb,8.8196e-03_rb,9.0035e-03_rb/)
      kao(:, 1, 4, 5) = (/ &
     &3.5080e-04_rb,1.5106e-03_rb,2.1048e-03_rb,2.5530e-03_rb,2.8923e-03_rb,3.1426e-03_rb, &
     &3.3832e-03_rb,3.6360e-03_rb,3.6099e-03_rb/)
      kao(:, 2, 4, 5) = (/ &
     &3.5494e-04_rb,1.6898e-03_rb,2.3828e-03_rb,2.9160e-03_rb,3.3056e-03_rb,3.6304e-03_rb, &
     &3.9520e-03_rb,4.2994e-03_rb,4.2882e-03_rb/)
      kao(:, 3, 4, 5) = (/ &
     &3.5978e-04_rb,1.8836e-03_rb,2.6877e-03_rb,3.3008e-03_rb,3.7593e-03_rb,4.1815e-03_rb, &
     &4.6053e-03_rb,4.9917e-03_rb,5.0514e-03_rb/)
      kao(:, 4, 4, 5) = (/ &
     &3.6746e-04_rb,2.0763e-03_rb,3.0082e-03_rb,3.7203e-03_rb,4.2549e-03_rb,4.7934e-03_rb, &
     &5.3242e-03_rb,5.8169e-03_rb,5.8760e-03_rb/)
      kao(:, 5, 4, 5) = (/ &
     &3.7911e-04_rb,2.2831e-03_rb,3.3520e-03_rb,4.1619e-03_rb,4.7913e-03_rb,5.4605e-03_rb, &
     &6.1003e-03_rb,6.7476e-03_rb,6.8032e-03_rb/)
      kao(:, 1, 5, 5) = (/ &
     &5.8701e-04_rb,1.7294e-03_rb,2.3007e-03_rb,2.6940e-03_rb,2.9671e-03_rb,3.1134e-03_rb, &
     &3.1768e-03_rb,3.0490e-03_rb,3.1049e-03_rb/)
      kao(:, 2, 5, 5) = (/ &
     &5.9308e-04_rb,1.9111e-03_rb,2.5835e-03_rb,3.0407e-03_rb,3.3851e-03_rb,3.5845e-03_rb, &
     &3.6445e-03_rb,3.5746e-03_rb,3.6940e-03_rb/)
      kao(:, 3, 5, 5) = (/ &
     &6.0356e-04_rb,2.1186e-03_rb,2.8868e-03_rb,3.4242e-03_rb,3.8486e-03_rb,4.1112e-03_rb, &
     &4.1722e-03_rb,4.1940e-03_rb,4.3605e-03_rb/)
      kao(:, 4, 5, 5) = (/ &
     &6.1818e-04_rb,2.3278e-03_rb,3.1936e-03_rb,3.8321e-03_rb,4.3352e-03_rb,4.6666e-03_rb, &
     &4.7552e-03_rb,4.8779e-03_rb,5.1110e-03_rb/)
      kao(:, 5, 5, 5) = (/ &
     &6.3535e-04_rb,2.5512e-03_rb,3.5322e-03_rb,4.2882e-03_rb,4.8679e-03_rb,5.2358e-03_rb, &
     &5.3921e-03_rb,5.6199e-03_rb,5.9412e-03_rb/)
      kao(:, 1, 6, 5) = (/ &
     &9.2532e-04_rb,2.0015e-03_rb,2.5277e-03_rb,2.8421e-03_rb,3.0437e-03_rb,3.1018e-03_rb, &
     &3.0296e-03_rb,2.8221e-03_rb,2.6040e-03_rb/)
      kao(:, 2, 6, 5) = (/ &
     &9.2928e-04_rb,2.2000e-03_rb,2.8005e-03_rb,3.1777e-03_rb,3.4468e-03_rb,3.5577e-03_rb, &
     &3.5029e-03_rb,3.2686e-03_rb,3.0871e-03_rb/)
      kao(:, 3, 6, 5) = (/ &
     &9.4818e-04_rb,2.4342e-03_rb,3.0892e-03_rb,3.5513e-03_rb,3.8786e-03_rb,4.0485e-03_rb, &
     &4.0367e-03_rb,3.7564e-03_rb,3.6084e-03_rb/)
      kao(:, 4, 6, 5) = (/ &
     &9.7245e-04_rb,2.6688e-03_rb,3.4060e-03_rb,3.9527e-03_rb,4.3416e-03_rb,4.5754e-03_rb, &
     &4.6140e-03_rb,4.3161e-03_rb,4.2427e-03_rb/)
      kao(:, 5, 6, 5) = (/ &
     &9.9339e-04_rb,2.9158e-03_rb,3.7426e-03_rb,4.3630e-03_rb,4.8503e-03_rb,5.1436e-03_rb, &
     &5.2166e-03_rb,4.9082e-03_rb,4.9378e-03_rb/)
      kao(:, 1, 7, 5) = (/ &
     &1.5302e-03_rb,2.5414e-03_rb,2.9540e-03_rb,3.1974e-03_rb,3.3103e-03_rb,3.2792e-03_rb, &
     &3.0626e-03_rb,2.6477e-03_rb,2.2296e-03_rb/)
      kao(:, 2, 7, 5) = (/ &
     &1.5437e-03_rb,2.8069e-03_rb,3.2725e-03_rb,3.5730e-03_rb,3.6894e-03_rb,3.7000e-03_rb, &
     &3.5141e-03_rb,3.0866e-03_rb,2.7517e-03_rb/)
      kao(:, 3, 7, 5) = (/ &
     &1.5828e-03_rb,3.0663e-03_rb,3.6143e-03_rb,3.9451e-03_rb,4.1195e-03_rb,4.1576e-03_rb, &
     &4.0163e-03_rb,3.5822e-03_rb,3.3032e-03_rb/)
      kao(:, 4, 7, 5) = (/ &
     &1.6196e-03_rb,3.3257e-03_rb,3.9578e-03_rb,4.3834e-03_rb,4.5853e-03_rb,4.6606e-03_rb, &
     &4.5385e-03_rb,4.1261e-03_rb,3.9000e-03_rb/)
      kao(:, 5, 7, 5) = (/ &
     &1.6660e-03_rb,3.6179e-03_rb,4.3451e-03_rb,4.8226e-03_rb,5.0668e-03_rb,5.1930e-03_rb, &
     &5.1180e-03_rb,4.7030e-03_rb,4.5474e-03_rb/)
      kao(:, 1, 8, 5) = (/ &
     &2.9602e-03_rb,3.9304e-03_rb,4.1580e-03_rb,4.2228e-03_rb,4.1165e-03_rb,3.9147e-03_rb, &
     &3.5216e-03_rb,2.8265e-03_rb,2.2242e-03_rb/)
      kao(:, 2, 8, 5) = (/ &
     &2.9996e-03_rb,4.2668e-03_rb,4.6028e-03_rb,4.6806e-03_rb,4.6212e-03_rb,4.3859e-03_rb, &
     &3.9542e-03_rb,3.2447e-03_rb,2.5165e-03_rb/)
      kao(:, 3, 8, 5) = (/ &
     &3.0790e-03_rb,4.6440e-03_rb,5.0256e-03_rb,5.1447e-03_rb,5.1348e-03_rb,4.8892e-03_rb, &
     &4.4430e-03_rb,3.7287e-03_rb,2.9386e-03_rb/)
      kao(:, 4, 8, 5) = (/ &
     &3.1467e-03_rb,5.0515e-03_rb,5.4655e-03_rb,5.6295e-03_rb,5.6440e-03_rb,5.4532e-03_rb, &
     &4.9689e-03_rb,4.2347e-03_rb,3.5150e-03_rb/)
      kao(:, 5, 8, 5) = (/ &
     &3.2473e-03_rb,5.4319e-03_rb,5.9099e-03_rb,6.1513e-03_rb,6.2421e-03_rb,6.0606e-03_rb, &
     &5.5483e-03_rb,4.7752e-03_rb,4.1476e-03_rb/)
      kao(:, 1, 9, 5) = (/ &
     &1.0715e-02_rb,1.1382e-02_rb,1.0899e-02_rb,1.0007e-02_rb,9.0437e-03_rb,7.6976e-03_rb, &
     &6.2825e-03_rb,4.4971e-03_rb,3.7652e-03_rb/)
      kao(:, 2, 9, 5) = (/ &
     &1.0909e-02_rb,1.1862e-02_rb,1.1397e-02_rb,1.0777e-02_rb,9.8877e-03_rb,8.5937e-03_rb, &
     &7.0755e-03_rb,5.0938e-03_rb,4.2507e-03_rb/)
      kao(:, 3, 9, 5) = (/ &
     &1.1246e-02_rb,1.2418e-02_rb,1.2293e-02_rb,1.1701e-02_rb,1.0880e-02_rb,9.4889e-03_rb, &
     &7.8244e-03_rb,5.6718e-03_rb,4.5213e-03_rb/)
      kao(:, 4, 9, 5) = (/ &
     &1.1484e-02_rb,1.3245e-02_rb,1.3252e-02_rb,1.2816e-02_rb,1.1850e-02_rb,1.0328e-02_rb, &
     &8.6168e-03_rb,6.3257e-03_rb,4.7999e-03_rb/)
      kao(:, 5, 9, 5) = (/ &
     &1.1883e-02_rb,1.4169e-02_rb,1.4256e-02_rb,1.3856e-02_rb,1.2835e-02_rb,1.1196e-02_rb, &
     &9.4286e-03_rb,7.0084e-03_rb,5.1834e-03_rb/)
      kao(:, 1,10, 5) = (/ &
     &4.3348e-02_rb,4.0646e-02_rb,3.6539e-02_rb,3.2078e-02_rb,2.7151e-02_rb,2.1841e-02_rb, &
     &1.6185e-02_rb,9.9201e-03_rb,6.7193e-03_rb/)
      kao(:, 2,10, 5) = (/ &
     &4.4149e-02_rb,4.2066e-02_rb,3.8091e-02_rb,3.3609e-02_rb,2.8552e-02_rb,2.2983e-02_rb, &
     &1.7376e-02_rb,1.1009e-02_rb,7.5436e-03_rb/)
      kao(:, 3,10, 5) = (/ &
     &4.5674e-02_rb,4.3639e-02_rb,3.9624e-02_rb,3.4947e-02_rb,3.0188e-02_rb,2.4887e-02_rb, &
     &1.8903e-02_rb,1.2177e-02_rb,8.3556e-03_rb/)
      kao(:, 4,10, 5) = (/ &
     &4.6947e-02_rb,4.5302e-02_rb,4.1477e-02_rb,3.7249e-02_rb,3.2588e-02_rb,2.6746e-02_rb, &
     &2.0815e-02_rb,1.3320e-02_rb,9.0276e-03_rb/)
      kao(:, 5,10, 5) = (/ &
     &4.8633e-02_rb,4.7885e-02_rb,4.4096e-02_rb,3.9864e-02_rb,3.4957e-02_rb,2.9104e-02_rb, &
     &2.2653e-02_rb,1.4428e-02_rb,9.7333e-03_rb/)
      kao(:, 1,11, 5) = (/ &
     &6.1692e-02_rb,5.6780e-02_rb,5.0232e-02_rb,4.3693e-02_rb,3.6664e-02_rb,2.9101e-02_rb, &
     &2.1139e-02_rb,1.2500e-02_rb,7.3091e-03_rb/)
      kao(:, 2,11, 5) = (/ &
     &6.3633e-02_rb,5.9268e-02_rb,5.2865e-02_rb,4.5871e-02_rb,3.8419e-02_rb,3.0609e-02_rb, &
     &2.2496e-02_rb,1.3660e-02_rb,8.2073e-03_rb/)
      kao(:, 3,11, 5) = (/ &
     &6.5598e-02_rb,6.1281e-02_rb,5.4828e-02_rb,4.7713e-02_rb,4.0401e-02_rb,3.2790e-02_rb, &
     &2.4296e-02_rb,1.5126e-02_rb,8.8706e-03_rb/)
      kao(:, 4,11, 5) = (/ &
     &6.7575e-02_rb,6.3884e-02_rb,5.7739e-02_rb,5.0715e-02_rb,4.3385e-02_rb,3.5358e-02_rb, &
     &2.6343e-02_rb,1.6652e-02_rb,9.5120e-03_rb/)
      kao(:, 5,11, 5) = (/ &
     &7.0953e-02_rb,6.7728e-02_rb,6.1195e-02_rb,5.3939e-02_rb,4.6283e-02_rb,3.8093e-02_rb, &
     &2.8648e-02_rb,1.8051e-02_rb,1.0472e-02_rb/)
      kao(:, 1,12, 5) = (/ &
     &6.6289e-02_rb,6.0781e-02_rb,5.3636e-02_rb,4.6304e-02_rb,3.8429e-02_rb,3.0261e-02_rb, &
     &2.1777e-02_rb,1.2752e-02_rb,6.7746e-03_rb/)
      kao(:, 2,12, 5) = (/ &
     &6.8551e-02_rb,6.3174e-02_rb,5.6063e-02_rb,4.8348e-02_rb,4.0059e-02_rb,3.1785e-02_rb, &
     &2.3315e-02_rb,1.3856e-02_rb,7.6622e-03_rb/)
      kao(:, 3,12, 5) = (/ &
     &7.0441e-02_rb,6.5327e-02_rb,5.8258e-02_rb,5.0611e-02_rb,4.2587e-02_rb,3.4170e-02_rb, &
     &2.5237e-02_rb,1.5295e-02_rb,8.3950e-03_rb/)
      kao(:, 4,12, 5) = (/ &
     &7.3259e-02_rb,6.8842e-02_rb,6.1685e-02_rb,5.3758e-02_rb,4.5436e-02_rb,3.6706e-02_rb, &
     &2.7377e-02_rb,1.6771e-02_rb,8.9439e-03_rb/)
      kao(:, 5,12, 5) = (/ &
     &7.7467e-02_rb,7.3081e-02_rb,6.5449e-02_rb,5.7418e-02_rb,4.8677e-02_rb,3.9671e-02_rb, &
     &2.9550e-02_rb,1.8202e-02_rb,9.6319e-03_rb/)
      kao(:, 1,13, 5) = (/ &
     &5.6671e-02_rb,5.1982e-02_rb,4.5878e-02_rb,3.9500e-02_rb,3.2807e-02_rb,2.5894e-02_rb, &
     &1.8717e-02_rb,1.0942e-02_rb,5.7523e-03_rb/)
      kao(:, 2,13, 5) = (/ &
     &5.8577e-02_rb,5.4011e-02_rb,4.7896e-02_rb,4.1459e-02_rb,3.4661e-02_rb,2.7671e-02_rb, &
     &2.0186e-02_rb,1.2025e-02_rb,6.4664e-03_rb/)
      kao(:, 3,13, 5) = (/ &
     &6.0296e-02_rb,5.6294e-02_rb,5.0261e-02_rb,4.3702e-02_rb,3.6751e-02_rb,2.9579e-02_rb, &
     &2.1985e-02_rb,1.3253e-02_rb,7.0517e-03_rb/)
      kao(:, 4,13, 5) = (/ &
     &6.3403e-02_rb,5.9492e-02_rb,5.3202e-02_rb,4.6643e-02_rb,3.9435e-02_rb,3.1951e-02_rb, &
     &2.3841e-02_rb,1.4493e-02_rb,7.4965e-03_rb/)
      kao(:, 5,13, 5) = (/ &
     &6.7016e-02_rb,6.3341e-02_rb,5.7105e-02_rb,5.0058e-02_rb,4.2357e-02_rb,3.4571e-02_rb, &
     &2.5674e-02_rb,1.5753e-02_rb,8.1075e-03_rb/)
      kao(:, 1, 1, 6) = (/ &
     &1.5042e-04_rb,2.4198e-03_rb,4.3016e-03_rb,6.2979e-03_rb,8.5168e-03_rb,1.0968e-02_rb, &
     &1.3929e-02_rb,1.8584e-02_rb,1.6834e-02_rb/)
      kao(:, 2, 1, 6) = (/ &
     &1.5402e-04_rb,2.7874e-03_rb,5.0906e-03_rb,7.5405e-03_rb,1.0172e-02_rb,1.3049e-02_rb, &
     &1.6523e-02_rb,2.2172e-02_rb,2.0134e-02_rb/)
      kao(:, 3, 1, 6) = (/ &
     &1.5658e-04_rb,3.2116e-03_rb,5.9652e-03_rb,8.9274e-03_rb,1.2070e-02_rb,1.5491e-02_rb, &
     &1.9553e-02_rb,2.6056e-02_rb,2.3919e-02_rb/)
      kao(:, 4, 1, 6) = (/ &
     &1.5960e-04_rb,3.6836e-03_rb,6.9617e-03_rb,1.0460e-02_rb,1.4154e-02_rb,1.8205e-02_rb, &
     &2.3055e-02_rb,3.0555e-02_rb,2.8089e-02_rb/)
      kao(:, 5, 1, 6) = (/ &
     &1.6165e-04_rb,4.2037e-03_rb,8.0817e-03_rb,1.2177e-02_rb,1.6484e-02_rb,2.1191e-02_rb, &
     &2.6875e-02_rb,3.5769e-02_rb,3.2739e-02_rb/)
      kao(:, 1, 2, 6) = (/ &
     &2.0998e-04_rb,2.2589e-03_rb,3.7899e-03_rb,5.2410e-03_rb,6.7744e-03_rb,8.5703e-03_rb, &
     &1.0697e-02_rb,1.3881e-02_rb,1.3198e-02_rb/)
      kao(:, 2, 2, 6) = (/ &
     &2.1456e-04_rb,2.6007e-03_rb,4.3946e-03_rb,6.1987e-03_rb,8.1361e-03_rb,1.0272e-02_rb, &
     &1.2733e-02_rb,1.6541e-02_rb,1.5875e-02_rb/)
      kao(:, 3, 2, 6) = (/ &
     &2.1842e-04_rb,2.9560e-03_rb,5.0887e-03_rb,7.2752e-03_rb,9.6552e-03_rb,1.2252e-02_rb, &
     &1.5232e-02_rb,1.9582e-02_rb,1.8968e-02_rb/)
      kao(:, 4, 2, 6) = (/ &
     &2.2236e-04_rb,3.3459e-03_rb,5.8736e-03_rb,8.5067e-03_rb,1.1349e-02_rb,1.4414e-02_rb, &
     &1.7964e-02_rb,2.3187e-02_rb,2.2355e-02_rb/)
      kao(:, 5, 2, 6) = (/ &
     &2.2547e-04_rb,3.7713e-03_rb,6.7501e-03_rb,9.9039e-03_rb,1.3264e-02_rb,1.6870e-02_rb, &
     &2.1032e-02_rb,2.7154e-02_rb,2.6176e-02_rb/)
      kao(:, 1, 3, 6) = (/ &
     &3.8749e-04_rb,2.3567e-03_rb,3.5784e-03_rb,4.7302e-03_rb,5.7992e-03_rb,6.7584e-03_rb, &
     &7.8929e-03_rb,9.6810e-03_rb,9.8950e-03_rb/)
      kao(:, 2, 3, 6) = (/ &
     &3.9603e-04_rb,2.6464e-03_rb,4.1366e-03_rb,5.5007e-03_rb,6.7822e-03_rb,8.0470e-03_rb, &
     &9.4913e-03_rb,1.1619e-02_rb,1.2036e-02_rb/)
      kao(:, 3, 3, 6) = (/ &
     &4.0456e-04_rb,2.9559e-03_rb,4.7389e-03_rb,6.3508e-03_rb,7.8799e-03_rb,9.4907e-03_rb, &
     &1.1392e-02_rb,1.3956e-02_rb,1.4457e-02_rb/)
      kao(:, 4, 3, 6) = (/ &
     &4.1360e-04_rb,3.3032e-03_rb,5.4024e-03_rb,7.2987e-03_rb,9.1447e-03_rb,1.1131e-02_rb, &
     &1.3466e-02_rb,1.6568e-02_rb,1.7113e-02_rb/)
      kao(:, 5, 3, 6) = (/ &
     &4.1860e-04_rb,3.6941e-03_rb,6.1025e-03_rb,8.3274e-03_rb,1.0570e-02_rb,1.3013e-02_rb, &
     &1.5813e-02_rb,1.9496e-02_rb,2.0111e-02_rb/)
      kao(:, 1, 4, 6) = (/ &
     &7.1991e-04_rb,2.8586e-03_rb,3.8493e-03_rb,4.6115e-03_rb,5.2750e-03_rb,5.9823e-03_rb, &
     &6.5939e-03_rb,7.1204e-03_rb,6.9950e-03_rb/)
      kao(:, 2, 4, 6) = (/ &
     &7.3932e-04_rb,3.1546e-03_rb,4.3292e-03_rb,5.2487e-03_rb,6.1632e-03_rb,7.0322e-03_rb, &
     &7.8025e-03_rb,8.5583e-03_rb,8.4949e-03_rb/)
      kao(:, 3, 4, 6) = (/ &
     &7.5829e-04_rb,3.4592e-03_rb,4.8355e-03_rb,5.9709e-03_rb,7.1395e-03_rb,8.1909e-03_rb, &
     &9.1159e-03_rb,1.0292e-02_rb,1.0335e-02_rb/)
      kao(:, 4, 4, 6) = (/ &
     &7.6959e-04_rb,3.8025e-03_rb,5.3880e-03_rb,6.7853e-03_rb,8.1872e-03_rb,9.4673e-03_rb, &
     &1.0631e-02_rb,1.2218e-02_rb,1.2516e-02_rb/)
      kao(:, 5, 4, 6) = (/ &
     &7.7964e-04_rb,4.1796e-03_rb,5.9947e-03_rb,7.6582e-03_rb,9.3446e-03_rb,1.0850e-02_rb, &
     &1.2339e-02_rb,1.4388e-02_rb,1.5009e-02_rb/)
      kao(:, 1, 5, 6) = (/ &
     &1.2241e-03_rb,3.4685e-03_rb,4.4046e-03_rb,5.0844e-03_rb,5.4871e-03_rb,5.7179e-03_rb, &
     &5.7620e-03_rb,5.9818e-03_rb,6.1472e-03_rb/)
      kao(:, 2, 5, 6) = (/ &
     &1.2601e-03_rb,3.8071e-03_rb,4.8995e-03_rb,5.7159e-03_rb,6.2114e-03_rb,6.5597e-03_rb, &
     &6.8164e-03_rb,7.1370e-03_rb,7.3501e-03_rb/)
      kao(:, 3, 5, 6) = (/ &
     &1.2927e-03_rb,4.1224e-03_rb,5.4389e-03_rb,6.3889e-03_rb,6.9938e-03_rb,7.4901e-03_rb, &
     &7.9768e-03_rb,8.3697e-03_rb,8.6918e-03_rb/)
      kao(:, 4, 5, 6) = (/ &
     &1.3168e-03_rb,4.4980e-03_rb,6.0112e-03_rb,7.1194e-03_rb,7.8654e-03_rb,8.4979e-03_rb, &
     &9.2325e-03_rb,9.7616e-03_rb,1.0181e-02_rb/)
      kao(:, 5, 5, 6) = (/ &
     &1.3499e-03_rb,4.8864e-03_rb,6.6201e-03_rb,7.8699e-03_rb,8.7970e-03_rb,9.6632e-03_rb, &
     &1.0606e-02_rb,1.1344e-02_rb,1.1812e-02_rb/)
      kao(:, 1, 6, 6) = (/ &
     &1.9359e-03_rb,4.1954e-03_rb,5.0050e-03_rb,5.5462e-03_rb,5.8539e-03_rb,5.9443e-03_rb, &
     &5.7417e-03_rb,5.1473e-03_rb,5.2472e-03_rb/)
      kao(:, 2, 6, 6) = (/ &
     &2.0045e-03_rb,4.5957e-03_rb,5.5374e-03_rb,6.1876e-03_rb,6.5964e-03_rb,6.7501e-03_rb, &
     &6.5907e-03_rb,6.0581e-03_rb,6.2872e-03_rb/)
      kao(:, 3, 6, 6) = (/ &
     &2.0656e-03_rb,4.9538e-03_rb,6.0748e-03_rb,6.8772e-03_rb,7.3778e-03_rb,7.6157e-03_rb, &
     &7.5054e-03_rb,7.1291e-03_rb,7.4717e-03_rb/)
      kao(:, 4, 6, 6) = (/ &
     &2.1235e-03_rb,5.3117e-03_rb,6.6418e-03_rb,7.6042e-03_rb,8.2374e-03_rb,8.5589e-03_rb, &
     &8.5173e-03_rb,8.3153e-03_rb,8.7345e-03_rb/)
      kao(:, 5, 6, 6) = (/ &
     &2.1708e-03_rb,5.7022e-03_rb,7.2758e-03_rb,8.4071e-03_rb,9.1623e-03_rb,9.5678e-03_rb, &
     &9.5860e-03_rb,9.6197e-03_rb,1.0118e-02_rb/)
      kao(:, 1, 7, 6) = (/ &
     &3.2088e-03_rb,5.3976e-03_rb,6.1152e-03_rb,6.4273e-03_rb,6.5565e-03_rb,6.4169e-03_rb, &
     &6.0483e-03_rb,5.1718e-03_rb,4.3840e-03_rb/)
      kao(:, 2, 7, 6) = (/ &
     &3.3329e-03_rb,5.8617e-03_rb,6.7065e-03_rb,7.1293e-03_rb,7.3332e-03_rb,7.2483e-03_rb, &
     &6.8991e-03_rb,6.0070e-03_rb,5.1893e-03_rb/)
      kao(:, 3, 7, 6) = (/ &
     &3.4526e-03_rb,6.3215e-03_rb,7.2907e-03_rb,7.8484e-03_rb,8.1105e-03_rb,8.1244e-03_rb, &
     &7.7961e-03_rb,6.9012e-03_rb,6.1115e-03_rb/)
      kao(:, 4, 7, 6) = (/ &
     &3.5398e-03_rb,6.7841e-03_rb,7.9069e-03_rb,8.5448e-03_rb,8.9613e-03_rb,9.0608e-03_rb, &
     &8.7760e-03_rb,7.8768e-03_rb,7.1030e-03_rb/)
      kao(:, 5, 7, 6) = (/ &
     &3.6419e-03_rb,7.2100e-03_rb,8.5218e-03_rb,9.3036e-03_rb,9.8959e-03_rb,1.0073e-02_rb, &
     &9.8505e-03_rb,8.9292e-03_rb,8.2308e-03_rb/)
      kao(:, 1, 8, 6) = (/ &
     &6.2734e-03_rb,8.1929e-03_rb,8.7347e-03_rb,8.8396e-03_rb,8.5498e-03_rb,7.9153e-03_rb, &
     &7.0346e-03_rb,5.6808e-03_rb,4.1552e-03_rb/)
      kao(:, 2, 8, 6) = (/ &
     &6.5441e-03_rb,8.8580e-03_rb,9.5108e-03_rb,9.7024e-03_rb,9.3653e-03_rb,8.8364e-03_rb, &
     &7.9480e-03_rb,6.5304e-03_rb,4.9813e-03_rb/)
      kao(:, 3, 8, 6) = (/ &
     &6.7938e-03_rb,9.5212e-03_rb,1.0312e-02_rb,1.0573e-02_rb,1.0260e-02_rb,9.7376e-03_rb, &
     &8.8728e-03_rb,7.4306e-03_rb,5.9661e-03_rb/)
      kao(:, 4, 8, 6) = (/ &
     &6.9763e-03_rb,1.0160e-02_rb,1.1107e-02_rb,1.1420e-02_rb,1.1193e-02_rb,1.0684e-02_rb, &
     &9.8581e-03_rb,8.3990e-03_rb,6.9644e-03_rb/)
      kao(:, 5, 8, 6) = (/ &
     &7.1912e-03_rb,1.0832e-02_rb,1.1979e-02_rb,1.2319e-02_rb,1.2084e-02_rb,1.1646e-02_rb, &
     &1.0942e-02_rb,9.4605e-03_rb,8.0079e-03_rb/)
      kao(:, 1, 9, 6) = (/ &
     &2.3423e-02_rb,2.3874e-02_rb,2.2570e-02_rb,2.0907e-02_rb,1.8747e-02_rb,1.6279e-02_rb, &
     &1.3268e-02_rb,9.1967e-03_rb,5.4472e-03_rb/)
      kao(:, 2, 9, 6) = (/ &
     &2.4372e-02_rb,2.5487e-02_rb,2.4526e-02_rb,2.2806e-02_rb,2.0430e-02_rb,1.7721e-02_rb, &
     &1.4547e-02_rb,1.0257e-02_rb,5.8874e-03_rb/)
      kao(:, 3, 9, 6) = (/ &
     &2.5322e-02_rb,2.7228e-02_rb,2.6228e-02_rb,2.4576e-02_rb,2.2164e-02_rb,1.9365e-02_rb, &
     &1.5921e-02_rb,1.1389e-02_rb,6.8168e-03_rb/)
      kao(:, 4, 9, 6) = (/ &
     &2.6144e-02_rb,2.8709e-02_rb,2.7958e-02_rb,2.6366e-02_rb,2.3791e-02_rb,2.0979e-02_rb, &
     &1.7361e-02_rb,1.2542e-02_rb,7.8623e-03_rb/)
      kao(:, 5, 9, 6) = (/ &
     &2.7002e-02_rb,3.0246e-02_rb,2.9564e-02_rb,2.8028e-02_rb,2.5537e-02_rb,2.2731e-02_rb, &
     &1.8860e-02_rb,1.3722e-02_rb,9.0318e-03_rb/)
      kao(:, 1,10, 6) = (/ &
     &9.6759e-02_rb,8.9920e-02_rb,8.0281e-02_rb,6.9298e-02_rb,5.8160e-02_rb,4.6685e-02_rb, &
     &3.4489e-02_rb,2.1137e-02_rb,9.1317e-03_rb/)
      kao(:, 2,10, 6) = (/ &
     &1.0130e-01_rb,9.4928e-02_rb,8.5063e-02_rb,7.4190e-02_rb,6.2751e-02_rb,5.0819e-02_rb, &
     &3.7815e-02_rb,2.3107e-02_rb,9.7753e-03_rb/)
      kao(:, 3,10, 6) = (/ &
     &1.0500e-01_rb,9.9899e-02_rb,9.0245e-02_rb,7.9682e-02_rb,6.7667e-02_rb,5.4638e-02_rb, &
     &4.0873e-02_rb,2.5040e-02_rb,1.0330e-02_rb/)
      kao(:, 4,10, 6) = (/ &
     &1.0896e-01_rb,1.0463e-01_rb,9.5171e-02_rb,8.3946e-02_rb,7.1420e-02_rb,5.8144e-02_rb, &
     &4.3799e-02_rb,2.7205e-02_rb,1.1084e-02_rb/)
      kao(:, 5,10, 6) = (/ &
     &1.1285e-01_rb,1.0942e-01_rb,9.9931e-02_rb,8.8439e-02_rb,7.5783e-02_rb,6.1708e-02_rb, &
     &4.6484e-02_rb,2.9531e-02_rb,1.2060e-02_rb/)
      kao(:, 1,11, 6) = (/ &
     &1.4204e-01_rb,1.2997e-01_rb,1.1520e-01_rb,9.8769e-02_rb,8.1420e-02_rb,6.4217e-02_rb, &
     &4.6399e-02_rb,2.7414e-02_rb,9.3511e-03_rb/)
      kao(:, 2,11, 6) = (/ &
     &1.4781e-01_rb,1.3669e-01_rb,1.2188e-01_rb,1.0492e-01_rb,8.7668e-02_rb,6.9566e-02_rb, &
     &5.0549e-02_rb,3.0014e-02_rb,1.0330e-02_rb/)
      kao(:, 3,11, 6) = (/ &
     &1.5380e-01_rb,1.4353e-01_rb,1.2793e-01_rb,1.1113e-01_rb,9.3325e-02_rb,7.4108e-02_rb, &
     &5.4029e-02_rb,3.2498e-02_rb,1.1533e-02_rb/)
      kao(:, 4,11, 6) = (/ &
     &1.6004e-01_rb,1.5059e-01_rb,1.3460e-01_rb,1.1704e-01_rb,9.8404e-02_rb,7.8709e-02_rb, &
     &5.7847e-02_rb,3.4798e-02_rb,1.2510e-02_rb/)
      kao(:, 5,11, 6) = (/ &
     &1.6529e-01_rb,1.5606e-01_rb,1.4056e-01_rb,1.2310e-01_rb,1.0396e-01_rb,8.3240e-02_rb, &
     &6.1355e-02_rb,3.7415e-02_rb,1.3283e-02_rb/)
      kao(:, 1,12, 6) = (/ &
     &1.5238e-01_rb,1.3897e-01_rb,1.2274e-01_rb,1.0509e-01_rb,8.6816e-02_rb,6.8213e-02_rb, &
     &4.8813e-02_rb,2.8303e-02_rb,8.9094e-03_rb/)
      kao(:, 2,12, 6) = (/ &
     &1.5887e-01_rb,1.4598e-01_rb,1.2939e-01_rb,1.1123e-01_rb,9.2497e-02_rb,7.3141e-02_rb, &
     &5.2574e-02_rb,3.0889e-02_rb,9.4910e-03_rb/)
      kao(:, 3,12, 6) = (/ &
     &1.6583e-01_rb,1.5402e-01_rb,1.3664e-01_rb,1.1788e-01_rb,9.8276e-02_rb,7.7826e-02_rb, &
     &5.6134e-02_rb,3.3329e-02_rb,1.0354e-02_rb/)
      kao(:, 4,12, 6) = (/ &
     &1.7232e-01_rb,1.6056e-01_rb,1.4294e-01_rb,1.2398e-01_rb,1.0387e-01_rb,8.2524e-02_rb, &
     &5.9981e-02_rb,3.5800e-02_rb,1.1471e-02_rb/)
      kao(:, 5,12, 6) = (/ &
     &1.7822e-01_rb,1.6713e-01_rb,1.4985e-01_rb,1.3045e-01_rb,1.0965e-01_rb,8.7208e-02_rb, &
     &6.3531e-02_rb,3.8400e-02_rb,1.2648e-02_rb/)
      kao(:, 1,13, 6) = (/ &
     &1.2851e-01_rb,1.1760e-01_rb,1.0432e-01_rb,8.9471e-02_rb,7.3952e-02_rb,5.7965e-02_rb, &
     &4.1571e-02_rb,2.4244e-02_rb,7.5271e-03_rb/)
      kao(:, 2,13, 6) = (/ &
     &1.3461e-01_rb,1.2431e-01_rb,1.1020e-01_rb,9.4715e-02_rb,7.8713e-02_rb,6.2191e-02_rb, &
     &4.4720e-02_rb,2.6349e-02_rb,7.9374e-03_rb/)
      kao(:, 3,13, 6) = (/ &
     &1.4091e-01_rb,1.3079e-01_rb,1.1585e-01_rb,1.0015e-01_rb,8.3554e-02_rb,6.6181e-02_rb, &
     &4.7816e-02_rb,2.8551e-02_rb,8.7113e-03_rb/)
      kao(:, 4,13, 6) = (/ &
     &1.4657e-01_rb,1.3673e-01_rb,1.2189e-01_rb,1.0573e-01_rb,8.8680e-02_rb,7.0418e-02_rb, &
     &5.0990e-02_rb,3.0632e-02_rb,9.8158e-03_rb/)
      kao(:, 5,13, 6) = (/ &
     &1.5197e-01_rb,1.4281e-01_rb,1.2778e-01_rb,1.1111e-01_rb,9.3402e-02_rb,7.4099e-02_rb, &
     &5.4192e-02_rb,3.2952e-02_rb,1.0865e-02_rb/)
      kao(:, 1, 1, 7) = (/ &
     &2.9775e-04_rb,4.9903e-03_rb,9.5966e-03_rb,1.4458e-02_rb,1.9550e-02_rb,2.5066e-02_rb, &
     &3.1548e-02_rb,4.0907e-02_rb,3.8920e-02_rb/)
      kao(:, 2, 1, 7) = (/ &
     &2.9974e-04_rb,5.8708e-03_rb,1.1484e-02_rb,1.7340e-02_rb,2.3486e-02_rb,3.0156e-02_rb, &
     &3.8022e-02_rb,4.9522e-02_rb,4.6798e-02_rb/)
      kao(:, 3, 1, 7) = (/ &
     &3.0249e-04_rb,6.8379e-03_rb,1.3523e-02_rb,2.0453e-02_rb,2.7734e-02_rb,3.5680e-02_rb, &
     &4.5116e-02_rb,5.8960e-02_rb,5.5301e-02_rb/)
      kao(:, 4, 1, 7) = (/ &
     &3.0505e-04_rb,7.9161e-03_rb,1.5750e-02_rb,2.3843e-02_rb,3.2353e-02_rb,4.1634e-02_rb, &
     &5.2683e-02_rb,6.9279e-02_rb,6.4544e-02_rb/)
      kao(:, 5, 1, 7) = (/ &
     &3.0784e-04_rb,9.1145e-03_rb,1.8178e-02_rb,2.7529e-02_rb,3.7353e-02_rb,4.8074e-02_rb, &
     &6.0824e-02_rb,8.0052e-02_rb,7.4551e-02_rb/)
      kao(:, 1, 2, 7) = (/ &
     &4.4044e-04_rb,4.5745e-03_rb,7.9442e-03_rb,1.1658e-02_rb,1.5612e-02_rb,1.9809e-02_rb, &
     &2.4586e-02_rb,3.1373e-02_rb,3.0910e-02_rb/)
      kao(:, 2, 2, 7) = (/ &
     &4.4502e-04_rb,5.2457e-03_rb,9.4321e-03_rb,1.4009e-02_rb,1.8804e-02_rb,2.3915e-02_rb, &
     &2.9756e-02_rb,3.8088e-02_rb,3.7317e-02_rb/)
      kao(:, 3, 2, 7) = (/ &
     &4.4893e-04_rb,5.9898e-03_rb,1.1101e-02_rb,1.6604e-02_rb,2.2319e-02_rb,2.8413e-02_rb, &
     &3.5386e-02_rb,4.5425e-02_rb,4.4358e-02_rb/)
      kao(:, 4, 2, 7) = (/ &
     &4.5303e-04_rb,6.8346e-03_rb,1.2966e-02_rb,1.9451e-02_rb,2.6168e-02_rb,3.3333e-02_rb, &
     &4.1534e-02_rb,5.3372e-02_rb,5.2063e-02_rb/)
      kao(:, 5, 2, 7) = (/ &
     &4.5653e-04_rb,7.7989e-03_rb,1.5045e-02_rb,2.2592e-02_rb,3.0407e-02_rb,3.8717e-02_rb, &
     &4.8217e-02_rb,6.1953e-02_rb,6.0541e-02_rb/)
      kao(:, 1, 3, 7) = (/ &
     &8.7011e-04_rb,4.7519e-03_rb,7.4191e-03_rb,9.7724e-03_rb,1.2242e-02_rb,1.5076e-02_rb, &
     &1.8293e-02_rb,2.2402e-02_rb,2.3454e-02_rb/)
      kao(:, 2, 3, 7) = (/ &
     &8.7761e-04_rb,5.4004e-03_rb,8.5169e-03_rb,1.1449e-02_rb,1.4656e-02_rb,1.8245e-02_rb, &
     &2.2203e-02_rb,2.7303e-02_rb,2.8468e-02_rb/)
      kao(:, 3, 3, 7) = (/ &
     &8.8555e-04_rb,6.0960e-03_rb,9.7399e-03_rb,1.3383e-02_rb,1.7420e-02_rb,2.1805e-02_rb, &
     &2.6578e-02_rb,3.2747e-02_rb,3.4102e-02_rb/)
      kao(:, 4, 3, 7) = (/ &
     &8.8837e-04_rb,6.8303e-03_rb,1.1116e-02_rb,1.5598e-02_rb,2.0532e-02_rb,2.5757e-02_rb, &
     &3.1445e-02_rb,3.8773e-02_rb,4.0387e-02_rb/)
      kao(:, 5, 3, 7) = (/ &
     &8.9568e-04_rb,7.6159e-03_rb,1.2685e-02_rb,1.8136e-02_rb,2.4011e-02_rb,3.0157e-02_rb, &
     &3.6817e-02_rb,4.5383e-02_rb,4.7355e-02_rb/)
      kao(:, 1, 4, 7) = (/ &
     &1.7216e-03_rb,5.6014e-03_rb,7.5736e-03_rb,9.4674e-03_rb,1.1191e-02_rb,1.2535e-02_rb, &
     &1.3987e-02_rb,1.6268e-02_rb,1.7296e-02_rb/)
      kao(:, 2, 4, 7) = (/ &
     &1.7298e-03_rb,6.1879e-03_rb,8.6832e-03_rb,1.0978e-02_rb,1.2974e-02_rb,1.4749e-02_rb, &
     &1.6870e-02_rb,1.9909e-02_rb,2.1431e-02_rb/)
      kao(:, 3, 4, 7) = (/ &
     &1.7341e-03_rb,6.8367e-03_rb,9.8584e-03_rb,1.2616e-02_rb,1.4944e-02_rb,1.7324e-02_rb, &
     &2.0237e-02_rb,2.4035e-02_rb,2.5998e-02_rb/)
      kao(:, 4, 4, 7) = (/ &
     &1.7534e-03_rb,7.5128e-03_rb,1.1127e-02_rb,1.4362e-02_rb,1.7213e-02_rb,2.0310e-02_rb, &
     &2.4053e-02_rb,2.8665e-02_rb,3.1074e-02_rb/)
      kao(:, 5, 4, 7) = (/ &
     &1.7641e-03_rb,8.2760e-03_rb,1.2507e-02_rb,1.6268e-02_rb,1.9802e-02_rb,2.3773e-02_rb, &
     &2.8361e-02_rb,3.3846e-02_rb,3.6727e-02_rb/)
      kao(:, 1, 5, 7) = (/ &
     &3.0539e-03_rb,7.3205e-03_rb,8.9564e-03_rb,9.9240e-03_rb,1.0675e-02_rb,1.1625e-02_rb, &
     &1.2424e-02_rb,1.2649e-02_rb,1.2475e-02_rb/)
      kao(:, 2, 5, 7) = (/ &
     &3.0697e-03_rb,8.0642e-03_rb,9.9550e-03_rb,1.1166e-02_rb,1.2428e-02_rb,1.3587e-02_rb, &
     &1.4545e-02_rb,1.5222e-02_rb,1.4936e-02_rb/)
      kao(:, 3, 5, 7) = (/ &
     &3.0978e-03_rb,8.8320e-03_rb,1.1001e-02_rb,1.2560e-02_rb,1.4306e-02_rb,1.5802e-02_rb, &
     &1.6906e-02_rb,1.8302e-02_rb,1.7957e-02_rb/)
      kao(:, 4, 5, 7) = (/ &
     &3.1191e-03_rb,9.5288e-03_rb,1.2130e-02_rb,1.4148e-02_rb,1.6348e-02_rb,1.8245e-02_rb, &
     &1.9668e-02_rb,2.1849e-02_rb,2.2010e-02_rb/)
      kao(:, 5, 5, 7) = (/ &
     &3.1173e-03_rb,1.0269e-02_rb,1.3281e-02_rb,1.5937e-02_rb,1.8604e-02_rb,2.0861e-02_rb, &
     &2.2868e-02_rb,2.5903e-02_rb,2.6735e-02_rb/)
      kao(:, 1, 6, 7) = (/ &
     &4.9571e-03_rb,9.1596e-03_rb,1.0920e-02_rb,1.1683e-02_rb,1.1947e-02_rb,1.1808e-02_rb, &
     &1.1212e-02_rb,1.1074e-02_rb,1.1113e-02_rb/)
      kao(:, 2, 6, 7) = (/ &
     &5.0027e-03_rb,9.9969e-03_rb,1.2157e-02_rb,1.3082e-02_rb,1.3449e-02_rb,1.3422e-02_rb, &
     &1.3156e-02_rb,1.3134e-02_rb,1.3279e-02_rb/)
      kao(:, 3, 6, 7) = (/ &
     &5.0299e-03_rb,1.0940e-02_rb,1.3400e-02_rb,1.4481e-02_rb,1.5090e-02_rb,1.5137e-02_rb, &
     &1.5334e-02_rb,1.5370e-02_rb,1.5787e-02_rb/)
      kao(:, 4, 6, 7) = (/ &
     &5.0401e-03_rb,1.1872e-02_rb,1.4616e-02_rb,1.5947e-02_rb,1.6778e-02_rb,1.7087e-02_rb, &
     &1.7734e-02_rb,1.7868e-02_rb,1.8548e-02_rb/)
      kao(:, 5, 6, 7) = (/ &
     &5.0929e-03_rb,1.2799e-02_rb,1.5866e-02_rb,1.7551e-02_rb,1.8584e-02_rb,1.9414e-02_rb, &
     &2.0410e-02_rb,2.0816e-02_rb,2.1713e-02_rb/)
      kao(:, 1, 7, 7) = (/ &
     &8.4314e-03_rb,1.2463e-02_rb,1.3843e-02_rb,1.4407e-02_rb,1.4329e-02_rb,1.3643e-02_rb, &
     &1.2423e-02_rb,1.0398e-02_rb,1.0049e-02_rb/)
      kao(:, 2, 7, 7) = (/ &
     &8.4877e-03_rb,1.3426e-02_rb,1.5237e-02_rb,1.6099e-02_rb,1.6143e-02_rb,1.5465e-02_rb, &
     &1.4143e-02_rb,1.1980e-02_rb,1.1912e-02_rb/)
      kao(:, 3, 7, 7) = (/ &
     &8.5061e-03_rb,1.4450e-02_rb,1.6739e-02_rb,1.7865e-02_rb,1.8006e-02_rb,1.7376e-02_rb, &
     &1.5998e-02_rb,1.3868e-02_rb,1.3962e-02_rb/)
      kao(:, 4, 7, 7) = (/ &
     &8.5919e-03_rb,1.5490e-02_rb,1.8250e-02_rb,1.9632e-02_rb,1.9933e-02_rb,1.9343e-02_rb, &
     &1.7978e-02_rb,1.6120e-02_rb,1.6346e-02_rb/)
      kao(:, 5, 7, 7) = (/ &
     &8.6244e-03_rb,1.6546e-02_rb,1.9762e-02_rb,2.1475e-02_rb,2.1907e-02_rb,2.1414e-02_rb, &
     &2.0099e-02_rb,1.8617e-02_rb,1.9113e-02_rb/)
      kao(:, 1, 8, 7) = (/ &
     &1.6801e-02_rb,2.0166e-02_rb,2.0707e-02_rb,2.0359e-02_rb,1.9438e-02_rb,1.7889e-02_rb, &
     &1.5556e-02_rb,1.2093e-02_rb,8.8687e-03_rb/)
      kao(:, 2, 8, 7) = (/ &
     &1.6841e-02_rb,2.1258e-02_rb,2.2460e-02_rb,2.2381e-02_rb,2.1605e-02_rb,2.0124e-02_rb, &
     &1.7632e-02_rb,1.3928e-02_rb,1.0513e-02_rb/)
      kao(:, 3, 8, 7) = (/ &
     &1.6945e-02_rb,2.2389e-02_rb,2.4138e-02_rb,2.4525e-02_rb,2.3968e-02_rb,2.2498e-02_rb, &
     &1.9836e-02_rb,1.5792e-02_rb,1.2224e-02_rb/)
      kao(:, 4, 8, 7) = (/ &
     &1.7081e-02_rb,2.3610e-02_rb,2.5903e-02_rb,2.6682e-02_rb,2.6351e-02_rb,2.4884e-02_rb, &
     &2.2197e-02_rb,1.7790e-02_rb,1.4133e-02_rb/)
      kao(:, 5, 8, 7) = (/ &
     &1.7107e-02_rb,2.4831e-02_rb,2.7681e-02_rb,2.8814e-02_rb,2.8766e-02_rb,2.7453e-02_rb, &
     &2.4610e-02_rb,1.9914e-02_rb,1.6383e-02_rb/)
      kao(:, 1, 9, 7) = (/ &
     &6.2717e-02_rb,6.2735e-02_rb,5.8603e-02_rb,5.2748e-02_rb,4.6263e-02_rb,3.9117e-02_rb, &
     &3.0759e-02_rb,2.1087e-02_rb,9.9200e-03_rb/)
      kao(:, 2, 9, 7) = (/ &
     &6.3094e-02_rb,6.4670e-02_rb,6.0602e-02_rb,5.5392e-02_rb,4.9420e-02_rb,4.2558e-02_rb, &
     &3.4002e-02_rb,2.3733e-02_rb,1.2138e-02_rb/)
      kao(:, 3, 9, 7) = (/ &
     &6.3563e-02_rb,6.6269e-02_rb,6.3058e-02_rb,5.8357e-02_rb,5.2632e-02_rb,4.5743e-02_rb, &
     &3.7261e-02_rb,2.6594e-02_rb,1.4716e-02_rb/)
      kao(:, 4, 9, 7) = (/ &
     &6.4005e-02_rb,6.8005e-02_rb,6.5556e-02_rb,6.1466e-02_rb,5.6121e-02_rb,4.9302e-02_rb, &
     &4.0642e-02_rb,2.9533e-02_rb,1.7598e-02_rb/)
      kao(:, 5, 9, 7) = (/ &
     &6.4303e-02_rb,6.9477e-02_rb,6.8109e-02_rb,6.4600e-02_rb,5.9691e-02_rb,5.2874e-02_rb, &
     &4.4056e-02_rb,3.2670e-02_rb,2.0612e-02_rb/)
      kao(:, 1,10, 7) = (/ &
     &2.6283e-01_rb,2.3848e-01_rb,2.1237e-01_rb,1.8399e-01_rb,1.5364e-01_rb,1.2096e-01_rb, &
     &8.6686e-02_rb,5.0921e-02_rb,1.2662e-02_rb/)
      kao(:, 2,10, 7) = (/ &
     &2.6495e-01_rb,2.4348e-01_rb,2.1796e-01_rb,1.8933e-01_rb,1.5813e-01_rb,1.2513e-01_rb, &
     &9.1228e-02_rb,5.5191e-02_rb,1.6120e-02_rb/)
      kao(:, 3,10, 7) = (/ &
     &2.6701e-01_rb,2.4762e-01_rb,2.2286e-01_rb,1.9379e-01_rb,1.6250e-01_rb,1.3046e-01_rb, &
     &9.6273e-02_rb,5.9643e-02_rb,2.0041e-02_rb/)
      kao(:, 4,10, 7) = (/ &
     &2.6827e-01_rb,2.5124e-01_rb,2.2683e-01_rb,1.9843e-01_rb,1.6799e-01_rb,1.3618e-01_rb, &
     &1.0159e-01_rb,6.3939e-02_rb,2.4035e-02_rb/)
      kao(:, 5,10, 7) = (/ &
     &2.7234e-01_rb,2.5705e-01_rb,2.3264e-01_rb,2.0401e-01_rb,1.7374e-01_rb,1.4157e-01_rb, &
     &1.0689e-01_rb,6.8300e-02_rb,2.8241e-02_rb/)
      kao(:, 1,11, 7) = (/ &
     &3.8435e-01_rb,3.4486e-01_rb,3.0408e-01_rb,2.6025e-01_rb,2.1577e-01_rb,1.6840e-01_rb, &
     &1.1827e-01_rb,6.6862e-02_rb,1.4067e-02_rb/)
      kao(:, 2,11, 7) = (/ &
     &3.8808e-01_rb,3.5079e-01_rb,3.0947e-01_rb,2.6705e-01_rb,2.2115e-01_rb,1.7271e-01_rb, &
     &1.2306e-01_rb,7.1102e-02_rb,1.7282e-02_rb/)
      kao(:, 3,11, 7) = (/ &
     &3.9133e-01_rb,3.5683e-01_rb,3.1692e-01_rb,2.7320e-01_rb,2.2652e-01_rb,1.7854e-01_rb, &
     &1.2887e-01_rb,7.5471e-02_rb,2.0650e-02_rb/)
      kao(:, 4,11, 7) = (/ &
     &3.9688e-01_rb,3.6404e-01_rb,3.2547e-01_rb,2.8118e-01_rb,2.3421e-01_rb,1.8549e-01_rb, &
     &1.3490e-01_rb,8.0175e-02_rb,2.4385e-02_rb/)
      kao(:, 5,11, 7) = (/ &
     &4.0366e-01_rb,3.7211e-01_rb,3.3310e-01_rb,2.8891e-01_rb,2.4168e-01_rb,1.9277e-01_rb, &
     &1.4118e-01_rb,8.5255e-02_rb,2.8315e-02_rb/)
      kao(:, 1,12, 7) = (/ &
     &4.1788e-01_rb,3.7417e-01_rb,3.2848e-01_rb,2.8026e-01_rb,2.3093e-01_rb,1.7900e-01_rb, &
     &1.2516e-01_rb,6.9829e-02_rb,1.3077e-02_rb/)
      kao(:, 2,12, 7) = (/ &
     &4.2346e-01_rb,3.8197e-01_rb,3.3568e-01_rb,2.8761e-01_rb,2.3734e-01_rb,1.8458e-01_rb, &
     &1.3037e-01_rb,7.4069e-02_rb,1.6501e-02_rb/)
      kao(:, 3,12, 7) = (/ &
     &4.3061e-01_rb,3.9035e-01_rb,3.4485e-01_rb,2.9676e-01_rb,2.4502e-01_rb,1.9175e-01_rb, &
     &1.3672e-01_rb,7.8434e-02_rb,1.9942e-02_rb/)
      kao(:, 4,12, 7) = (/ &
     &4.3991e-01_rb,4.0142e-01_rb,3.5621e-01_rb,3.0583e-01_rb,2.5354e-01_rb,1.9941e-01_rb, &
     &1.4294e-01_rb,8.2867e-02_rb,2.3156e-02_rb/)
      kao(:, 5,12, 7) = (/ &
     &4.5018e-01_rb,4.1383e-01_rb,3.6847e-01_rb,3.1728e-01_rb,2.6385e-01_rb,2.0769e-01_rb, &
     &1.4984e-01_rb,8.7550e-02_rb,2.5906e-02_rb/)
      kao(:, 1,13, 7) = (/ &
     &3.5954e-01_rb,3.2293e-01_rb,2.8290e-01_rb,2.4149e-01_rb,1.9855e-01_rb,1.5392e-01_rb, &
     &1.0799e-01_rb,6.0538e-02_rb,1.1582e-02_rb/)
      kao(:, 2,13, 7) = (/ &
     &3.6620e-01_rb,3.3095e-01_rb,2.9118e-01_rb,2.4962e-01_rb,2.0579e-01_rb,1.6035e-01_rb, &
     &1.1345e-01_rb,6.4561e-02_rb,1.4555e-02_rb/)
      kao(:, 3,13, 7) = (/ &
     &3.7497e-01_rb,3.4139e-01_rb,3.0205e-01_rb,2.5902e-01_rb,2.1397e-01_rb,1.6755e-01_rb, &
     &1.1907e-01_rb,6.8274e-02_rb,1.7516e-02_rb/)
      kao(:, 4,13, 7) = (/ &
     &3.8491e-01_rb,3.5293e-01_rb,3.1344e-01_rb,2.6951e-01_rb,2.2338e-01_rb,1.7537e-01_rb, &
     &1.2550e-01_rb,7.2213e-02_rb,1.9685e-02_rb/)
      kao(:, 5,13, 7) = (/ &
     &3.9456e-01_rb,3.6430e-01_rb,3.2370e-01_rb,2.7885e-01_rb,2.3209e-01_rb,1.8326e-01_rb, &
     &1.3235e-01_rb,7.6929e-02_rb,2.1105e-02_rb/)
      kao(:, 1, 1, 8) = (/ &
     &6.2097e-04_rb,1.4117e-02_rb,2.8277e-02_rb,4.2775e-02_rb,5.7848e-02_rb,7.3960e-02_rb, &
     &9.2407e-02_rb,1.1864e-01_rb,1.1554e-01_rb/)
      kao(:, 2, 1, 8) = (/ &
     &6.1974e-04_rb,1.6905e-02_rb,3.3889e-02_rb,5.1276e-02_rb,6.9348e-02_rb,8.8703e-02_rb, &
     &1.1084e-01_rb,1.4177e-01_rb,1.3853e-01_rb/)
      kao(:, 3, 1, 8) = (/ &
     &6.1587e-04_rb,1.9900e-02_rb,3.9925e-02_rb,6.0422e-02_rb,8.1729e-02_rb,1.0451e-01_rb, &
     &1.3061e-01_rb,1.6696e-01_rb,1.6328e-01_rb/)
      kao(:, 4, 1, 8) = (/ &
     &6.1184e-04_rb,2.3086e-02_rb,4.6344e-02_rb,7.0163e-02_rb,9.4901e-02_rb,1.2137e-01_rb, &
     &1.5165e-01_rb,1.9370e-01_rb,1.8962e-01_rb/)
      kao(:, 5, 1, 8) = (/ &
     &6.0950e-04_rb,2.6468e-02_rb,5.3164e-02_rb,8.0478e-02_rb,1.0886e-01_rb,1.3922e-01_rb, &
     &1.7398e-01_rb,2.2192e-01_rb,2.1753e-01_rb/)
      kao(:, 1, 2, 8) = (/ &
     &9.4214e-04_rb,1.1864e-02_rb,2.3368e-02_rb,3.5168e-02_rb,4.7308e-02_rb,6.0060e-02_rb, &
     &7.4293e-02_rb,9.3548e-02_rb,9.4381e-02_rb/)
      kao(:, 2, 2, 8) = (/ &
     &9.3967e-04_rb,1.4205e-02_rb,2.8221e-02_rb,4.2491e-02_rb,5.7165e-02_rb,7.2616e-02_rb, &
     &8.9835e-02_rb,1.1322e-01_rb,1.1408e-01_rb/)
      kao(:, 3, 2, 8) = (/ &
     &9.3666e-04_rb,1.6793e-02_rb,3.3453e-02_rb,5.0403e-02_rb,6.7823e-02_rb,8.6143e-02_rb, &
     &1.0657e-01_rb,1.3437e-01_rb,1.3538e-01_rb/)
      kao(:, 4, 2, 8) = (/ &
     &9.3490e-04_rb,1.9589e-02_rb,3.9087e-02_rb,5.8901e-02_rb,7.9281e-02_rb,1.0068e-01_rb, &
     &1.2458e-01_rb,1.5697e-01_rb,1.5829e-01_rb/)
      kao(:, 5, 2, 8) = (/ &
     &9.3255e-04_rb,2.2561e-02_rb,4.5087e-02_rb,6.7983e-02_rb,9.1508e-02_rb,1.1621e-01_rb, &
     &1.4375e-01_rb,1.8108e-01_rb,1.8274e-01_rb/)
      kao(:, 1, 3, 8) = (/ &
     &1.9614e-03_rb,1.1299e-02_rb,1.9005e-02_rb,2.7782e-02_rb,3.7000e-02_rb,4.6522e-02_rb, &
     &5.6657e-02_rb,6.9080e-02_rb,7.3508e-02_rb/)
      kao(:, 2, 3, 8) = (/ &
     &1.9548e-03_rb,1.2955e-02_rb,2.2891e-02_rb,3.3903e-02_rb,4.5254e-02_rb,5.6922e-02_rb, &
     &6.9355e-02_rb,8.4555e-02_rb,9.0001e-02_rb/)
      kao(:, 3, 3, 8) = (/ &
     &1.9534e-03_rb,1.4833e-02_rb,2.7240e-02_rb,4.0599e-02_rb,5.4239e-02_rb,6.8264e-02_rb, &
     &8.3194e-02_rb,1.0146e-01_rb,1.0796e-01_rb/)
      kao(:, 4, 3, 8) = (/ &
     &1.9479e-03_rb,1.6939e-02_rb,3.1981e-02_rb,4.7805e-02_rb,6.3918e-02_rb,8.0501e-02_rb, &
     &9.8164e-02_rb,1.1980e-01_rb,1.2731e-01_rb/)
      kao(:, 5, 3, 8) = (/ &
     &1.9437e-03_rb,1.9215e-02_rb,3.7014e-02_rb,5.5432e-02_rb,7.4159e-02_rb,9.3422e-02_rb, &
     &1.1404e-01_rb,1.3937e-01_rb,1.4780e-01_rb/)
      kao(:, 1, 4, 8) = (/ &
     &4.1141e-03_rb,1.2873e-02_rb,1.8872e-02_rb,2.3795e-02_rb,2.9310e-02_rb,3.5813e-02_rb, &
     &4.3001e-02_rb,5.1220e-02_rb,5.6278e-02_rb/)
      kao(:, 2, 4, 8) = (/ &
     &4.1183e-03_rb,1.4575e-02_rb,2.1599e-02_rb,2.8253e-02_rb,3.5801e-02_rb,4.4309e-02_rb, &
     &5.3363e-02_rb,6.3628e-02_rb,6.9867e-02_rb/)
      kao(:, 3, 4, 8) = (/ &
     &4.1349e-03_rb,1.6355e-02_rb,2.4672e-02_rb,3.3305e-02_rb,4.3074e-02_rb,5.3625e-02_rb, &
     &6.4673e-02_rb,7.7228e-02_rb,8.4695e-02_rb/)
      kao(:, 4, 4, 8) = (/ &
     &4.1308e-03_rb,1.8173e-02_rb,2.8034e-02_rb,3.8865e-02_rb,5.0949e-02_rb,6.3606e-02_rb, &
     &7.6798e-02_rb,9.1846e-02_rb,1.0056e-01_rb/)
      kao(:, 5, 4, 8) = (/ &
     &4.1264e-03_rb,1.9905e-02_rb,3.1676e-02_rb,4.4946e-02_rb,5.9333e-02_rb,7.4191e-02_rb, &
     &8.9640e-02_rb,1.0735e-01_rb,1.1739e-01_rb/)
      kao(:, 1, 5, 8) = (/ &
     &7.8459e-03_rb,1.5273e-02_rb,2.0315e-02_rb,2.4490e-02_rb,2.7833e-02_rb,3.0201e-02_rb, &
     &3.3378e-02_rb,3.8411e-02_rb,4.1001e-02_rb/)
      kao(:, 2, 5, 8) = (/ &
     &7.8747e-03_rb,1.6898e-02_rb,2.3176e-02_rb,2.8487e-02_rb,3.2405e-02_rb,3.6383e-02_rb, &
     &4.1437e-02_rb,4.8316e-02_rb,5.2769e-02_rb/)
      kao(:, 3, 5, 8) = (/ &
     &7.8992e-03_rb,1.8661e-02_rb,2.6253e-02_rb,3.2541e-02_rb,3.7550e-02_rb,4.3266e-02_rb, &
     &5.0422e-02_rb,5.9136e-02_rb,6.5527e-02_rb/)
      kao(:, 4, 5, 8) = (/ &
     &7.8989e-03_rb,2.0558e-02_rb,2.9408e-02_rb,3.6576e-02_rb,4.3219e-02_rb,5.0946e-02_rb, &
     &6.0207e-02_rb,7.0821e-02_rb,7.8616e-02_rb/)
      kao(:, 5, 5, 8) = (/ &
     &7.9229e-03_rb,2.2512e-02_rb,3.2599e-02_rb,4.0913e-02_rb,4.9454e-02_rb,5.9390e-02_rb, &
     &7.0729e-02_rb,8.3313e-02_rb,9.2573e-02_rb/)
      kao(:, 1, 6, 8) = (/ &
     &1.3755e-02_rb,2.1264e-02_rb,2.4103e-02_rb,2.5712e-02_rb,2.7798e-02_rb,2.9236e-02_rb, &
     &2.9846e-02_rb,2.9593e-02_rb,2.7586e-02_rb/)
      kao(:, 2, 6, 8) = (/ &
     &1.3782e-02_rb,2.2999e-02_rb,2.6490e-02_rb,2.9514e-02_rb,3.2454e-02_rb,3.4496e-02_rb, &
     &3.5566e-02_rb,3.7000e-02_rb,3.5132e-02_rb/)
      kao(:, 3, 6, 8) = (/ &
     &1.3863e-02_rb,2.4665e-02_rb,2.9042e-02_rb,3.3641e-02_rb,3.7254e-02_rb,4.0147e-02_rb, &
     &4.1998e-02_rb,4.5403e-02_rb,4.4921e-02_rb/)
      kao(:, 4, 6, 8) = (/ &
     &1.3921e-02_rb,2.6309e-02_rb,3.1937e-02_rb,3.7874e-02_rb,4.2332e-02_rb,4.5980e-02_rb, &
     &4.9161e-02_rb,5.4719e-02_rb,5.5713e-02_rb/)
      kao(:, 5, 6, 8) = (/ &
     &1.4010e-02_rb,2.7955e-02_rb,3.5119e-02_rb,4.2034e-02_rb,4.7545e-02_rb,5.2148e-02_rb, &
     &5.7163e-02_rb,6.4833e-02_rb,6.7480e-02_rb/)
      kao(:, 1, 7, 8) = (/ &
     &2.4762e-02_rb,3.1775e-02_rb,3.3749e-02_rb,3.4176e-02_rb,3.3323e-02_rb,3.1060e-02_rb, &
     &2.9254e-02_rb,2.6750e-02_rb,2.6287e-02_rb/)
      kao(:, 2, 7, 8) = (/ &
     &2.4951e-02_rb,3.3870e-02_rb,3.6729e-02_rb,3.7700e-02_rb,3.7125e-02_rb,3.5645e-02_rb, &
     &3.4725e-02_rb,3.2382e-02_rb,3.2363e-02_rb/)
      kao(:, 3, 7, 8) = (/ &
     &2.5173e-02_rb,3.5837e-02_rb,3.9660e-02_rb,4.1206e-02_rb,4.1346e-02_rb,4.1029e-02_rb, &
     &4.0471e-02_rb,3.8518e-02_rb,3.8913e-02_rb/)
      kao(:, 4, 7, 8) = (/ &
     &2.5363e-02_rb,3.7886e-02_rb,4.2599e-02_rb,4.5028e-02_rb,4.5616e-02_rb,4.6713e-02_rb, &
     &4.6613e-02_rb,4.5204e-02_rb,4.5916e-02_rb/)
      kao(:, 5, 7, 8) = (/ &
     &2.5554e-02_rb,3.9913e-02_rb,4.5725e-02_rb,4.8757e-02_rb,5.0398e-02_rb,5.2532e-02_rb, &
     &5.3118e-02_rb,5.2718e-02_rb,5.3341e-02_rb/)
      kao(:, 1, 8, 8) = (/ &
     &5.1772e-02_rb,5.6742e-02_rb,5.6311e-02_rb,5.3676e-02_rb,4.9761e-02_rb,4.4432e-02_rb, &
     &3.7642e-02_rb,2.8159e-02_rb,2.5366e-02_rb/)
      kao(:, 2, 8, 8) = (/ &
     &5.2310e-02_rb,5.9935e-02_rb,6.0103e-02_rb,5.8243e-02_rb,5.4861e-02_rb,4.9530e-02_rb, &
     &4.2560e-02_rb,3.2942e-02_rb,3.0796e-02_rb/)
      kao(:, 3, 8, 8) = (/ &
     &5.2803e-02_rb,6.3037e-02_rb,6.4134e-02_rb,6.2766e-02_rb,5.9645e-02_rb,5.4655e-02_rb, &
     &4.7864e-02_rb,3.8688e-02_rb,3.7143e-02_rb/)
      kao(:, 4, 8, 8) = (/ &
     &5.3460e-02_rb,6.5918e-02_rb,6.8003e-02_rb,6.7271e-02_rb,6.4711e-02_rb,6.0132e-02_rb, &
     &5.3215e-02_rb,4.4724e-02_rb,4.4005e-02_rb/)
      kao(:, 5, 8, 8) = (/ &
     &5.3710e-02_rb,6.8699e-02_rb,7.1633e-02_rb,7.2036e-02_rb,6.9887e-02_rb,6.5520e-02_rb, &
     &5.8694e-02_rb,5.1143e-02_rb,5.1305e-02_rb/)
      kao(:, 1, 9, 8) = (/ &
     &2.0198e-01_rb,1.8901e-01_rb,1.7403e-01_rb,1.5566e-01_rb,1.3460e-01_rb,1.1050e-01_rb, &
     &8.4423e-02_rb,5.5049e-02_rb,2.7409e-02_rb/)
      kao(:, 2, 9, 8) = (/ &
     &2.0495e-01_rb,1.9639e-01_rb,1.8291e-01_rb,1.6519e-01_rb,1.4305e-01_rb,1.1881e-01_rb, &
     &9.2245e-02_rb,6.1787e-02_rb,3.3663e-02_rb/)
      kao(:, 3, 9, 8) = (/ &
     &2.0775e-01_rb,2.0359e-01_rb,1.9173e-01_rb,1.7370e-01_rb,1.5212e-01_rb,1.2752e-01_rb, &
     &1.0028e-01_rb,6.8520e-02_rb,3.9556e-02_rb/)
      kao(:, 4, 9, 8) = (/ &
     &2.0971e-01_rb,2.1033e-01_rb,1.9959e-01_rb,1.8173e-01_rb,1.6052e-01_rb,1.3619e-01_rb, &
     &1.0838e-01_rb,7.5430e-02_rb,4.6210e-02_rb/)
      kao(:, 5, 9, 8) = (/ &
     &2.1122e-01_rb,2.1661e-01_rb,2.0701e-01_rb,1.8977e-01_rb,1.6868e-01_rb,1.4451e-01_rb, &
     &1.1695e-01_rb,8.2426e-02_rb,5.2611e-02_rb/)
      kao(:, 1,10, 8) = (/ &
     &8.7179e-01_rb,7.7305e-01_rb,6.7676e-01_rb,5.7800e-01_rb,4.7858e-01_rb,3.7509e-01_rb, &
     &2.6780e-01_rb,1.5257e-01_rb,3.4825e-02_rb/)
      kao(:, 2,10, 8) = (/ &
     &8.8690e-01_rb,7.9154e-01_rb,6.9682e-01_rb,6.0089e-01_rb,5.0085e-01_rb,3.9677e-01_rb, &
     &2.8510e-01_rb,1.6381e-01_rb,4.0840e-02_rb/)
      kao(:, 3,10, 8) = (/ &
     &8.9846e-01_rb,8.0873e-01_rb,7.1712e-01_rb,6.2211e-01_rb,5.2235e-01_rb,4.1529e-01_rb, &
     &3.0033e-01_rb,1.7576e-01_rb,4.8253e-02_rb/)
      kao(:, 4,10, 8) = (/ &
     &9.1202e-01_rb,8.2713e-01_rb,7.3854e-01_rb,6.4399e-01_rb,5.4276e-01_rb,4.3255e-01_rb, &
     &3.1463e-01_rb,1.8727e-01_rb,5.7567e-02_rb/)
      kao(:, 5,10, 8) = (/ &
     &9.1292e-01_rb,8.3520e-01_rb,7.5311e-01_rb,6.6058e-01_rb,5.5851e-01_rb,4.4868e-01_rb, &
     &3.2919e-01_rb,1.9841e-01_rb,6.5659e-02_rb/)
      kao(:, 1,11, 8) = (/ &
     &1.3103e+00_rb,1.1565e+00_rb,1.0072e+00_rb,8.5492e-01_rb,7.0080e-01_rb,5.4405e-01_rb, &
     &3.8288e-01_rb,2.1296e-01_rb,3.6932e-02_rb/)
      kao(:, 2,11, 8) = (/ &
     &1.3299e+00_rb,1.1796e+00_rb,1.0327e+00_rb,8.8125e-01_rb,7.2693e-01_rb,5.6857e-01_rb, &
     &4.0335e-01_rb,2.2633e-01_rb,4.3190e-02_rb/)
      kao(:, 3,11, 8) = (/ &
     &1.3530e+00_rb,1.2064e+00_rb,1.0600e+00_rb,9.1131e-01_rb,7.5598e-01_rb,5.9411e-01_rb, &
     &4.2265e-01_rb,2.3978e-01_rb,5.2762e-02_rb/)
      kao(:, 4,11, 8) = (/ &
     &1.3587e+00_rb,1.2195e+00_rb,1.0781e+00_rb,9.3079e-01_rb,7.7634e-01_rb,6.1327e-01_rb, &
     &4.3892e-01_rb,2.5297e-01_rb,6.4865e-02_rb/)
      kao(:, 5,11, 8) = (/ &
     &1.3600e+00_rb,1.2284e+00_rb,1.0908e+00_rb,9.4457e-01_rb,7.9211e-01_rb,6.2699e-01_rb, &
     &4.5262e-01_rb,2.6437e-01_rb,7.6848e-02_rb/)
      kao(:, 1,12, 8) = (/ &
     &1.4407e+00_rb,1.2713e+00_rb,1.1065e+00_rb,9.3835e-01_rb,7.6825e-01_rb,5.9432e-01_rb, &
     &4.1638e-01_rb,2.2888e-01_rb,3.4685e-02_rb/)
      kao(:, 2,12, 8) = (/ &
     &1.4696e+00_rb,1.3029e+00_rb,1.1378e+00_rb,9.6981e-01_rb,7.9767e-01_rb,6.2103e-01_rb, &
     &4.3779e-01_rb,2.4254e-01_rb,4.2654e-02_rb/)
      kao(:, 3,12, 8) = (/ &
     &1.4810e+00_rb,1.3211e+00_rb,1.1578e+00_rb,9.9148e-01_rb,8.2013e-01_rb,6.4181e-01_rb, &
     &4.5466e-01_rb,2.5578e-01_rb,5.2496e-02_rb/)
      kao(:, 4,12, 8) = (/ &
     &1.4858e+00_rb,1.3306e+00_rb,1.1727e+00_rb,1.0074e+00_rb,8.3618e-01_rb,6.5672e-01_rb, &
     &4.6743e-01_rb,2.6737e-01_rb,6.3361e-02_rb/)
      kao(:, 5,12, 8) = (/ &
     &1.4815e+00_rb,1.3335e+00_rb,1.1804e+00_rb,1.0190e+00_rb,8.4985e-01_rb,6.7121e-01_rb, &
     &4.8121e-01_rb,2.7853e-01_rb,7.5291e-02_rb/)
      kao(:, 1,13, 8) = (/ &
     &1.2616e+00_rb,1.1163e+00_rb,9.7204e-01_rb,8.2547e-01_rb,6.7669e-01_rb,5.2391e-01_rb, &
     &3.6673e-01_rb,2.0106e-01_rb,2.9782e-02_rb/)
      kao(:, 2,13, 8) = (/ &
     &1.2777e+00_rb,1.1375e+00_rb,9.9345e-01_rb,8.4804e-01_rb,6.9806e-01_rb,5.4388e-01_rb, &
     &3.8298e-01_rb,2.1265e-01_rb,3.7539e-02_rb/)
      kao(:, 3,13, 8) = (/ &
     &1.2857e+00_rb,1.1501e+00_rb,1.0105e+00_rb,8.6538e-01_rb,7.1567e-01_rb,5.5928e-01_rb, &
     &3.9596e-01_rb,2.2292e-01_rb,4.5370e-02_rb/)
      kao(:, 4,13, 8) = (/ &
     &1.2877e+00_rb,1.1566e+00_rb,1.0204e+00_rb,8.7739e-01_rb,7.2832e-01_rb,5.7170e-01_rb, &
     &4.0727e-01_rb,2.3325e-01_rb,5.5134e-02_rb/)
      kao(:, 5,13, 8) = (/ &
     &1.2810e+00_rb,1.1580e+00_rb,1.0267e+00_rb,8.8778e-01_rb,7.3941e-01_rb,5.8252e-01_rb, &
     &4.1760e-01_rb,2.4248e-01_rb,6.6222e-02_rb/)
      kao(:, 1, 1, 9) = (/ &
     &1.6209e-03_rb,5.7589e-02_rb,1.1558e-01_rb,1.7436e-01_rb,2.3438e-01_rb,2.9678e-01_rb, &
     &3.6435e-01_rb,4.4557e-01_rb,4.6867e-01_rb/)
      kao(:, 2, 1, 9) = (/ &
     &1.6160e-03_rb,6.8254e-02_rb,1.3700e-01_rb,2.0665e-01_rb,2.7776e-01_rb,3.5160e-01_rb, &
     &4.3147e-01_rb,5.2804e-01_rb,5.5543e-01_rb/)
      kao(:, 3, 1, 9) = (/ &
     &1.6156e-03_rb,7.9739e-02_rb,1.6004e-01_rb,2.4140e-01_rb,3.2446e-01_rb,4.1061e-01_rb, &
     &5.0337e-01_rb,6.1528e-01_rb,6.4882e-01_rb/)
      kao(:, 4, 1, 9) = (/ &
     &1.6118e-03_rb,9.1878e-02_rb,1.8447e-01_rb,2.7828e-01_rb,3.7409e-01_rb,4.7339e-01_rb, &
     &5.8011e-01_rb,7.0748e-01_rb,7.4809e-01_rb/)
      kao(:, 5, 1, 9) = (/ &
     &1.6027e-03_rb,1.0449e-01_rb,2.0983e-01_rb,3.1665e-01_rb,4.2579e-01_rb,5.3908e-01_rb, &
     &6.6087e-01_rb,8.0485e-01_rb,8.5148e-01_rb/)
      kao(:, 1, 2, 9) = (/ &
     &2.5385e-03_rb,4.9947e-02_rb,1.0010e-01_rb,1.5075e-01_rb,2.0224e-01_rb,2.5529e-01_rb, &
     &3.1202e-01_rb,3.8059e-01_rb,4.0436e-01_rb/)
      kao(:, 2, 2, 9) = (/ &
     &2.5420e-03_rb,5.9613e-02_rb,1.1949e-01_rb,1.7998e-01_rb,2.4152e-01_rb,3.0496e-01_rb, &
     &3.7260e-01_rb,4.5413e-01_rb,4.8291e-01_rb/)
      kao(:, 3, 2, 9) = (/ &
     &2.5425e-03_rb,6.9977e-02_rb,1.4031e-01_rb,2.1138e-01_rb,2.8370e-01_rb,3.5840e-01_rb, &
     &4.3825e-01_rb,5.3358e-01_rb,5.6726e-01_rb/)
      kao(:, 4, 2, 9) = (/ &
     &2.5349e-03_rb,8.0938e-02_rb,1.6229e-01_rb,2.4456e-01_rb,3.2834e-01_rb,4.1497e-01_rb, &
     &5.0769e-01_rb,6.1900e-01_rb,6.5654e-01_rb/)
      kao(:, 5, 2, 9) = (/ &
     &2.5236e-03_rb,9.2377e-02_rb,1.8522e-01_rb,2.7915e-01_rb,3.7480e-01_rb,4.7381e-01_rb, &
     &5.8018e-01_rb,7.0850e-01_rb,7.4944e-01_rb/)
      kao(:, 1, 3, 9) = (/ &
     &5.2884e-03_rb,4.1358e-02_rb,8.2480e-02_rb,1.2391e-01_rb,1.6574e-01_rb,2.0832e-01_rb, &
     &2.5278e-01_rb,3.0394e-01_rb,3.3126e-01_rb/)
      kao(:, 2, 3, 9) = (/ &
     &5.3098e-03_rb,4.9746e-02_rb,9.9365e-02_rb,1.4928e-01_rb,1.9972e-01_rb,2.5111e-01_rb, &
     &3.0480e-01_rb,3.6676e-01_rb,3.9919e-01_rb/)
      kao(:, 3, 3, 9) = (/ &
     &5.3133e-03_rb,5.8864e-02_rb,1.1765e-01_rb,1.7679e-01_rb,2.3653e-01_rb,2.9742e-01_rb, &
     &3.6115e-01_rb,4.3493e-01_rb,4.7279e-01_rb/)
      kao(:, 4, 3, 9) = (/ &
     &5.3184e-03_rb,6.8570e-02_rb,1.3708e-01_rb,2.0601e-01_rb,2.7562e-01_rb,3.4665e-01_rb, &
     &4.2100e-01_rb,5.0744e-01_rb,5.5094e-01_rb/)
      kao(:, 5, 3, 9) = (/ &
     &5.2937e-03_rb,7.8913e-02_rb,1.5776e-01_rb,2.3709e-01_rb,3.1721e-01_rb,3.9889e-01_rb, &
     &4.8431e-01_rb,5.8366e-01_rb,6.3409e-01_rb/)
      kao(:, 1, 4, 9) = (/ &
     &1.1189e-02_rb,3.7631e-02_rb,6.6855e-02_rb,9.9420e-02_rb,1.3264e-01_rb,1.6622e-01_rb, &
     &2.0064e-01_rb,2.3828e-01_rb,2.6479e-01_rb/)
      kao(:, 2, 4, 9) = (/ &
     &1.1235e-02_rb,4.3836e-02_rb,8.1298e-02_rb,1.2151e-01_rb,1.6212e-01_rb,2.0319e-01_rb, &
     &2.4526e-01_rb,2.9131e-01_rb,3.2370e-01_rb/)
      kao(:, 3, 4, 9) = (/ &
     &1.1256e-02_rb,5.0846e-02_rb,9.7157e-02_rb,1.4551e-01_rb,1.9420e-01_rb,2.4340e-01_rb, &
     &2.9384e-01_rb,3.4897e-01_rb,3.8783e-01_rb/)
      kao(:, 4, 4, 9) = (/ &
     &1.1233e-02_rb,5.8695e-02_rb,1.1439e-01_rb,1.7147e-01_rb,2.2883e-01_rb,2.8678e-01_rb, &
     &3.4608e-01_rb,4.1077e-01_rb,4.5705e-01_rb/)
      kao(:, 5, 4, 9) = (/ &
     &1.1193e-02_rb,6.7408e-02_rb,1.3290e-01_rb,1.9926e-01_rb,2.6591e-01_rb,3.3325e-01_rb, &
     &4.0214e-01_rb,4.7714e-01_rb,5.3115e-01_rb/)
      kao(:, 1, 5, 9) = (/ &
     &2.1456e-02_rb,4.5196e-02_rb,6.1958e-02_rb,8.2153e-02_rb,1.0545e-01_rb,1.3112e-01_rb, &
     &1.5772e-01_rb,1.8584e-01_rb,2.0874e-01_rb/)
      kao(:, 2, 5, 9) = (/ &
     &2.1627e-02_rb,4.9543e-02_rb,7.2420e-02_rb,9.9628e-02_rb,1.3042e-01_rb,1.6274e-01_rb, &
     &1.9580e-01_rb,2.3075e-01_rb,2.5926e-01_rb/)
      kao(:, 3, 5, 9) = (/ &
     &2.1648e-02_rb,5.4430e-02_rb,8.4381e-02_rb,1.1963e-01_rb,1.5816e-01_rb,1.9769e-01_rb, &
     &2.3786e-01_rb,2.8024e-01_rb,3.1505e-01_rb/)
      kao(:, 4, 5, 9) = (/ &
     &2.1655e-02_rb,5.9909e-02_rb,9.8020e-02_rb,1.4216e-01_rb,1.8871e-01_rb,2.3596e-01_rb, &
     &2.8391e-01_rb,3.3439e-01_rb,3.7619e-01_rb/)
      kao(:, 5, 5, 9) = (/ &
     &2.1575e-02_rb,6.6071e-02_rb,1.1341e-01_rb,1.6675e-01_rb,2.2186e-01_rb,2.7746e-01_rb, &
     &3.3388e-01_rb,3.9317e-01_rb,4.4246e-01_rb/)
      kao(:, 1, 6, 9) = (/ &
     &3.8314e-02_rb,5.9481e-02_rb,7.1457e-02_rb,8.1684e-02_rb,9.1784e-02_rb,1.0487e-01_rb, &
     &1.2171e-01_rb,1.4213e-01_rb,1.5860e-01_rb/)
      kao(:, 2, 6, 9) = (/ &
     &3.8611e-02_rb,6.4337e-02_rb,7.9461e-02_rb,9.3850e-02_rb,1.0993e-01_rb,1.2967e-01_rb, &
     &1.5327e-01_rb,1.7946e-01_rb,2.0255e-01_rb/)
      kao(:, 3, 6, 9) = (/ &
     &3.8670e-02_rb,6.9241e-02_rb,8.8613e-02_rb,1.0808e-01_rb,1.3134e-01_rb,1.5836e-01_rb, &
     &1.8890e-01_rb,2.2142e-01_rb,2.5002e-01_rb/)
      kao(:, 4, 6, 9) = (/ &
     &3.8675e-02_rb,7.4319e-02_rb,9.8760e-02_rb,1.2484e-01_rb,1.5591e-01_rb,1.9105e-01_rb, &
     &2.2866e-01_rb,2.6805e-01_rb,3.0304e-01_rb/)
      kao(:, 5, 6, 9) = (/ &
     &3.8508e-02_rb,7.9526e-02_rb,1.0987e-01_rb,1.4404e-01_rb,1.8357e-01_rb,2.2696e-01_rb, &
     &2.7218e-01_rb,3.1912e-01_rb,3.6102e-01_rb/)
      kao(:, 1, 7, 9) = (/ &
     &7.2084e-02_rb,8.8512e-02_rb,9.7172e-02_rb,1.0090e-01_rb,1.0168e-01_rb,1.0216e-01_rb, &
     &1.0271e-01_rb,1.0842e-01_rb,1.0501e-01_rb/)
      kao(:, 2, 7, 9) = (/ &
     &7.2564e-02_rb,9.4711e-02_rb,1.0583e-01_rb,1.1196e-01_rb,1.1550e-01_rb,1.2008e-01_rb, &
     &1.2583e-01_rb,1.3849e-01_rb,1.4021e-01_rb/)
      kao(:, 3, 7, 9) = (/ &
     &7.2730e-02_rb,1.0095e-01_rb,1.1490e-01_rb,1.2380e-01_rb,1.3141e-01_rb,1.4065e-01_rb, &
     &1.5344e-01_rb,1.7335e-01_rb,1.8125e-01_rb/)
      kao(:, 4, 7, 9) = (/ &
     &7.2941e-02_rb,1.0706e-01_rb,1.2422e-01_rb,1.3654e-01_rb,1.5005e-01_rb,1.6497e-01_rb, &
     &1.8541e-01_rb,2.1284e-01_rb,2.2809e-01_rb/)
      kao(:, 5, 7, 9) = (/ &
     &7.3026e-02_rb,1.1297e-01_rb,1.3379e-01_rb,1.5116e-01_rb,1.7068e-01_rb,1.9286e-01_rb, &
     &2.2139e-01_rb,2.5637e-01_rb,2.7993e-01_rb/)
      kao(:, 1, 8, 9) = (/ &
     &1.6179e-01_rb,1.6443e-01_rb,1.6310e-01_rb,1.5765e-01_rb,1.4868e-01_rb,1.3533e-01_rb, &
     &1.1770e-01_rb,9.8380e-02_rb,8.1836e-02_rb/)
      kao(:, 2, 8, 9) = (/ &
     &1.6423e-01_rb,1.7218e-01_rb,1.7516e-01_rb,1.7171e-01_rb,1.6374e-01_rb,1.5153e-01_rb, &
     &1.3534e-01_rb,1.1999e-01_rb,1.0061e-01_rb/)
      kao(:, 3, 8, 9) = (/ &
     &1.6654e-01_rb,1.7965e-01_rb,1.8707e-01_rb,1.8587e-01_rb,1.7956e-01_rb,1.6926e-01_rb, &
     &1.5537e-01_rb,1.4544e-01_rb,1.2284e-01_rb/)
      kao(:, 4, 8, 9) = (/ &
     &1.6762e-01_rb,1.8741e-01_rb,1.9894e-01_rb,2.0043e-01_rb,1.9638e-01_rb,1.8821e-01_rb, &
     &1.7907e-01_rb,1.7570e-01_rb,1.5659e-01_rb/)
      kao(:, 5, 8, 9) = (/ &
     &1.6808e-01_rb,1.9570e-01_rb,2.1084e-01_rb,2.1493e-01_rb,2.1406e-01_rb,2.0908e-01_rb, &
     &2.0616e-01_rb,2.1009e-01_rb,1.9730e-01_rb/)
      kao(:, 1, 9, 9) = (/ &
     &7.0428e-01_rb,6.3434e-01_rb,5.6644e-01_rb,4.9559e-01_rb,4.2070e-01_rb,3.4143e-01_rb, &
     &2.6063e-01_rb,1.7246e-01_rb,1.1446e-01_rb/)
      kao(:, 2, 9, 9) = (/ &
     &7.1533e-01_rb,6.5148e-01_rb,5.8487e-01_rb,5.1694e-01_rb,4.4379e-01_rb,3.6693e-01_rb, &
     &2.8562e-01_rb,1.9355e-01_rb,1.3717e-01_rb/)
      kao(:, 3, 9, 9) = (/ &
     &7.2105e-01_rb,6.6501e-01_rb,6.0595e-01_rb,5.4025e-01_rb,4.6705e-01_rb,3.9267e-01_rb, &
     &3.1101e-01_rb,2.1648e-01_rb,1.6351e-01_rb/)
      kao(:, 4, 9, 9) = (/ &
     &7.2262e-01_rb,6.7430e-01_rb,6.2376e-01_rb,5.6249e-01_rb,4.9295e-01_rb,4.1839e-01_rb, &
     &3.3764e-01_rb,2.4119e-01_rb,1.9114e-01_rb/)
      kao(:, 5, 9, 9) = (/ &
     &7.2252e-01_rb,6.8406e-01_rb,6.4078e-01_rb,5.8400e-01_rb,5.1738e-01_rb,4.4614e-01_rb, &
     &3.6505e-01_rb,2.6763e-01_rb,2.2362e-01_rb/)
      kao(:, 1,10, 9) = (/ &
     &3.3754e+00_rb,2.9579e+00_rb,2.5567e+00_rb,2.1598e+00_rb,1.7623e+00_rb,1.3527e+00_rb, &
     &9.4464e-01_rb,5.2414e-01_rb,1.6022e-01_rb/)
      kao(:, 2,10, 9) = (/ &
     &3.4121e+00_rb,2.9947e+00_rb,2.5993e+00_rb,2.2033e+00_rb,1.8038e+00_rb,1.3952e+00_rb, &
     &9.8851e-01_rb,5.5797e-01_rb,1.9922e-01_rb/)
      kao(:, 3,10, 9) = (/ &
     &3.4354e+00_rb,3.0219e+00_rb,2.6345e+00_rb,2.2476e+00_rb,1.8449e+00_rb,1.4451e+00_rb, &
     &1.0317e+00_rb,5.9234e-01_rb,2.4180e-01_rb/)
      kao(:, 4,10, 9) = (/ &
     &3.4369e+00_rb,3.0367e+00_rb,2.6577e+00_rb,2.2740e+00_rb,1.8821e+00_rb,1.4871e+00_rb, &
     &1.0728e+00_rb,6.2615e-01_rb,2.8291e-01_rb/)
      kao(:, 5,10, 9) = (/ &
     &3.4461e+00_rb,3.0546e+00_rb,2.6839e+00_rb,2.3057e+00_rb,1.9262e+00_rb,1.5287e+00_rb, &
     &1.1134e+00_rb,6.5955e-01_rb,3.3010e-01_rb/)
      kao(:, 1,11, 9) = (/ &
     &5.5731e+00_rb,4.8785e+00_rb,4.1979e+00_rb,3.5322e+00_rb,2.8619e+00_rb,2.1883e+00_rb, &
     &1.5051e+00_rb,8.1479e-01_rb,1.7197e-01_rb/)
      kao(:, 2,11, 9) = (/ &
     &5.6124e+00_rb,4.9158e+00_rb,4.2449e+00_rb,3.5793e+00_rb,2.9170e+00_rb,2.2388e+00_rb, &
     &1.5568e+00_rb,8.5687e-01_rb,2.2531e-01_rb/)
      kao(:, 3,11, 9) = (/ &
     &5.6253e+00_rb,4.9334e+00_rb,4.2755e+00_rb,3.6196e+00_rb,2.9577e+00_rb,2.2787e+00_rb, &
     &1.6022e+00_rb,8.9680e-01_rb,2.6360e-01_rb/)
      kao(:, 4,11, 9) = (/ &
     &5.6351e+00_rb,4.9513e+00_rb,4.3021e+00_rb,3.6565e+00_rb,2.9923e+00_rb,2.3311e+00_rb, &
     &1.6513e+00_rb,9.3728e-01_rb,3.0681e-01_rb/)
      kao(:, 5,11, 9) = (/ &
     &5.6511e+00_rb,4.9808e+00_rb,4.3475e+00_rb,3.7008e+00_rb,3.0483e+00_rb,2.3866e+00_rb, &
     &1.7012e+00_rb,9.7887e-01_rb,3.4951e-01_rb/)
      kao(:, 1,12, 9) = (/ &
     &6.6833e+00_rb,5.8503e+00_rb,5.0320e+00_rb,4.2291e+00_rb,3.4220e+00_rb,2.6133e+00_rb, &
     &1.7900e+00_rb,9.6260e-01_rb,1.7445e-01_rb/)
      kao(:, 2,12, 9) = (/ &
     &6.6977e+00_rb,5.8665e+00_rb,5.0629e+00_rb,4.2649e+00_rb,3.4722e+00_rb,2.6560e+00_rb, &
     &1.8378e+00_rb,1.0034e+00_rb,2.1910e-01_rb/)
      kao(:, 3,12, 9) = (/ &
     &6.7112e+00_rb,5.8830e+00_rb,5.0943e+00_rb,4.3071e+00_rb,3.5122e+00_rb,2.7000e+00_rb, &
     &1.8863e+00_rb,1.0455e+00_rb,2.5420e-01_rb/)
      kao(:, 4,12, 9) = (/ &
     &6.7306e+00_rb,5.9135e+00_rb,5.1333e+00_rb,4.3624e+00_rb,3.5635e+00_rb,2.7641e+00_rb, &
     &1.9469e+00_rb,1.0906e+00_rb,2.9618e-01_rb/)
      kao(:, 5,12, 9) = (/ &
     &6.7198e+00_rb,5.9182e+00_rb,5.1572e+00_rb,4.3874e+00_rb,3.6068e+00_rb,2.8158e+00_rb, &
     &1.9979e+00_rb,1.1364e+00_rb,3.4172e-01_rb/)
      kao(:, 1,13, 9) = (/ &
     &6.2228e+00_rb,5.4483e+00_rb,4.6948e+00_rb,3.9489e+00_rb,3.2053e+00_rb,2.4477e+00_rb, &
     &1.6826e+00_rb,9.0899e-01_rb,1.6340e-01_rb/)
      kao(:, 2,13, 9) = (/ &
     &6.2479e+00_rb,5.4750e+00_rb,4.7365e+00_rb,3.9942e+00_rb,3.2532e+00_rb,2.4916e+00_rb, &
     &1.7319e+00_rb,9.4656e-01_rb,1.9164e-01_rb/)
      kao(:, 3,13, 9) = (/ &
     &6.2720e+00_rb,5.5063e+00_rb,4.7748e+00_rb,4.0523e+00_rb,3.3026e+00_rb,2.5508e+00_rb, &
     &1.7859e+00_rb,9.9108e-01_rb,2.3050e-01_rb/)
      kao(:, 4,13, 9) = (/ &
     &6.2663e+00_rb,5.5165e+00_rb,4.8016e+00_rb,4.0858e+00_rb,3.3448e+00_rb,2.6010e+00_rb, &
     &1.8364e+00_rb,1.0337e+00_rb,2.6420e-01_rb/)
      kao(:, 5,13, 9) = (/ &
     &6.2869e+00_rb,5.5494e+00_rb,4.8492e+00_rb,4.1318e+00_rb,3.4036e+00_rb,2.6598e+00_rb, &
     &1.8865e+00_rb,1.0759e+00_rb,2.9687e-01_rb/)
      kao(:, 1, 1,10) = (/ &
     &2.7731e-03_rb,1.5625e-01_rb,3.1254e-01_rb,4.6910e-01_rb,6.2583e-01_rb,7.8249e-01_rb, &
     &9.3706e-01_rb,1.0845e+00_rb,1.2516e+00_rb/)
      kao(:, 2, 1,10) = (/ &
     &2.6994e-03_rb,1.8576e-01_rb,3.7148e-01_rb,5.5698e-01_rb,7.4226e-01_rb,9.2683e-01_rb, &
     &1.1083e+00_rb,1.2874e+00_rb,1.4844e+00_rb/)
      kao(:, 3, 1,10) = (/ &
     &2.5977e-03_rb,2.1688e-01_rb,4.3344e-01_rb,6.4957e-01_rb,8.6528e-01_rb,1.0791e+00_rb, &
     &1.2881e+00_rb,1.4959e+00_rb,1.7305e+00_rb/)
      kao(:, 4, 1,10) = (/ &
     &2.4994e-03_rb,2.5005e-01_rb,4.9954e-01_rb,7.4789e-01_rb,9.9444e-01_rb,1.2373e+00_rb, &
     &1.4746e+00_rb,1.7076e+00_rb,1.9888e+00_rb/)
      kao(:, 5, 1,10) = (/ &
     &2.4443e-03_rb,2.8623e-01_rb,5.7132e-01_rb,8.5454e-01_rb,1.1344e+00_rb,1.4080e+00_rb, &
     &1.6698e+00_rb,1.9238e+00_rb,2.2687e+00_rb/)
      kao(:, 1, 2,10) = (/ &
     &4.6033e-03_rb,1.4235e-01_rb,2.8494e-01_rb,4.2818e-01_rb,5.7234e-01_rb,7.1833e-01_rb, &
     &8.6574e-01_rb,1.0139e+00_rb,1.1445e+00_rb/)
      kao(:, 2, 2,10) = (/ &
     &4.4279e-03_rb,1.7116e-01_rb,3.4236e-01_rb,5.1392e-01_rb,6.8612e-01_rb,8.5990e-01_rb, &
     &1.0377e+00_rb,1.2137e+00_rb,1.3721e+00_rb/)
      kao(:, 3, 2,10) = (/ &
     &4.2606e-03_rb,2.0315e-01_rb,4.0616e-01_rb,6.0913e-01_rb,8.1201e-01_rb,1.0152e+00_rb, &
     &1.2192e+00_rb,1.4256e+00_rb,1.6238e+00_rb/)
      kao(:, 4, 2,10) = (/ &
     &4.1205e-03_rb,2.3773e-01_rb,4.7526e-01_rb,7.1252e-01_rb,9.4929e-01_rb,1.1850e+00_rb, &
     &1.4175e+00_rb,1.6433e+00_rb,1.8984e+00_rb/)
      kao(:, 5, 2,10) = (/ &
     &4.0494e-03_rb,2.7547e-01_rb,5.5055e-01_rb,8.2539e-01_rb,1.0993e+00_rb,1.3710e+00_rb, &
     &1.6368e+00_rb,1.8772e+00_rb,2.1984e+00_rb/)
      kao(:, 1, 3,10) = (/ &
     &1.0161e-02_rb,1.2214e-01_rb,2.4424e-01_rb,3.6667e-01_rb,4.8955e-01_rb,6.1355e-01_rb, &
     &7.4009e-01_rb,8.7555e-01_rb,9.7887e-01_rb/)
      kao(:, 2, 3,10) = (/ &
     &9.8188e-03_rb,1.5023e-01_rb,3.0029e-01_rb,4.5055e-01_rb,6.0115e-01_rb,7.5236e-01_rb, &
     &9.0544e-01_rb,1.0662e+00_rb,1.2020e+00_rb/)
      kao(:, 3, 3,10) = (/ &
     &9.4977e-03_rb,1.8146e-01_rb,3.6272e-01_rb,5.4423e-01_rb,7.2598e-01_rb,9.0829e-01_rb, &
     &1.0912e+00_rb,1.2774e+00_rb,1.4517e+00_rb/)
      kao(:, 4, 3,10) = (/ &
     &9.2658e-03_rb,2.1540e-01_rb,4.3072e-01_rb,6.4627e-01_rb,8.6225e-01_rb,1.0787e+00_rb, &
     &1.2961e+00_rb,1.5118e+00_rb,1.7242e+00_rb/)
      kao(:, 5, 3,10) = (/ &
     &9.2390e-03_rb,2.5102e-01_rb,5.0193e-01_rb,7.5312e-01_rb,1.0052e+00_rb,1.2582e+00_rb, &
     &1.5132e+00_rb,1.7682e+00_rb,2.0100e+00_rb/)
      kao(:, 1, 4,10) = (/ &
     &2.2805e-02_rb,1.0395e-01_rb,2.0752e-01_rb,3.1119e-01_rb,4.1502e-01_rb,5.1909e-01_rb, &
     &6.2381e-01_rb,7.3100e-01_rb,8.2961e-01_rb/)
      kao(:, 2, 4,10) = (/ &
     &2.2222e-02_rb,1.2866e-01_rb,2.5687e-01_rb,3.8520e-01_rb,5.1380e-01_rb,6.4273e-01_rb, &
     &7.7289e-01_rb,9.0648e-01_rb,1.0271e+00_rb/)
      kao(:, 3, 4,10) = (/ &
     &2.1664e-02_rb,1.5642e-01_rb,3.1238e-01_rb,4.6876e-01_rb,6.2523e-01_rb,7.8254e-01_rb, &
     &9.4140e-01_rb,1.1052e+00_rb,1.2499e+00_rb/)
      kao(:, 4, 4,10) = (/ &
     &2.1586e-02_rb,1.8661e-01_rb,3.7283e-01_rb,5.5940e-01_rb,7.4664e-01_rb,9.3462e-01_rb, &
     &1.1258e+00_rb,1.3254e+00_rb,1.4927e+00_rb/)
      kao(:, 5, 4,10) = (/ &
     &2.1462e-02_rb,2.1863e-01_rb,4.3680e-01_rb,6.5519e-01_rb,8.7456e-01_rb,1.0957e+00_rb, &
     &1.3203e+00_rb,1.5569e+00_rb,1.7484e+00_rb/)
      kao(:, 1, 5,10) = (/ &
     &4.7130e-02_rb,9.3953e-02_rb,1.7393e-01_rb,2.6053e-01_rb,3.4731e-01_rb,4.3431e-01_rb, &
     &5.2182e-01_rb,6.1134e-01_rb,6.9380e-01_rb/)
      kao(:, 2, 5,10) = (/ &
     &4.5557e-02_rb,1.1245e-01_rb,2.1773e-01_rb,3.2619e-01_rb,4.3487e-01_rb,5.4373e-01_rb, &
     &6.5319e-01_rb,7.6497e-01_rb,8.6883e-01_rb/)
      kao(:, 3, 5,10) = (/ &
     &4.5179e-02_rb,1.3419e-01_rb,2.6606e-01_rb,3.9876e-01_rb,5.3167e-01_rb,6.6490e-01_rb, &
     &7.9898e-01_rb,9.3637e-01_rb,1.0624e+00_rb/)
      kao(:, 4, 5,10) = (/ &
     &4.5058e-02_rb,1.5943e-01_rb,3.1786e-01_rb,4.7639e-01_rb,6.3534e-01_rb,7.9462e-01_rb, &
     &9.5569e-01_rb,1.1218e+00_rb,1.2696e+00_rb/)
      kao(:, 5, 5,10) = (/ &
     &4.5124e-02_rb,1.8659e-01_rb,3.7199e-01_rb,5.5748e-01_rb,7.4317e-01_rb,9.2965e-01_rb, &
     &1.1179e+00_rb,1.3144e+00_rb,1.4843e+00_rb/)
      kao(:, 1, 6,10) = (/ &
     &9.1822e-02_rb,1.2595e-01_rb,1.5650e-01_rb,2.1453e-01_rb,2.8383e-01_rb,3.5468e-01_rb, &
     &4.2569e-01_rb,4.9790e-01_rb,5.6600e-01_rb/)
      kao(:, 2, 6,10) = (/ &
     &9.0834e-02_rb,1.3514e-01_rb,1.8905e-01_rb,2.7114e-01_rb,3.6116e-01_rb,4.5123e-01_rb, &
     &5.4153e-01_rb,6.3330e-01_rb,7.2013e-01_rb/)
      kao(:, 3, 6,10) = (/ &
     &9.0336e-02_rb,1.4401e-01_rb,2.2730e-01_rb,3.3603e-01_rb,4.4738e-01_rb,5.5884e-01_rb, &
     &6.7092e-01_rb,7.8481e-01_rb,8.9194e-01_rb/)
      kao(:, 4, 6,10) = (/ &
     &9.0439e-02_rb,1.5770e-01_rb,2.7084e-01_rb,4.0445e-01_rb,5.3847e-01_rb,6.7263e-01_rb, &
     &8.0760e-01_rb,9.4541e-01_rb,1.0734e+00_rb/)
      kao(:, 5, 6,10) = (/ &
     &8.9534e-02_rb,1.7535e-01_rb,3.1878e-01_rb,4.7665e-01_rb,6.3445e-01_rb,7.9249e-01_rb, &
     &9.5149e-01_rb,1.1135e+00_rb,1.2646e+00_rb/)
      kao(:, 1, 7,10) = (/ &
     &1.8704e-01_rb,2.0782e-01_rb,2.1870e-01_rb,2.2895e-01_rb,2.4473e-01_rb,2.8566e-01_rb, &
     &3.3903e-01_rb,3.9559e-01_rb,4.4999e-01_rb/)
      kao(:, 2, 7,10) = (/ &
     &1.8532e-01_rb,2.1563e-01_rb,2.3939e-01_rb,2.5636e-01_rb,3.0272e-01_rb,3.6581e-01_rb, &
     &4.3811e-01_rb,5.1140e-01_rb,5.8167e-01_rb/)
      kao(:, 3, 7,10) = (/ &
     &1.8680e-01_rb,2.2377e-01_rb,2.5927e-01_rb,2.9855e-01_rb,3.7097e-01_rb,4.5865e-01_rb, &
     &5.4953e-01_rb,6.4162e-01_rb,7.2947e-01_rb/)
      kao(:, 4, 7,10) = (/ &
     &1.8310e-01_rb,2.3512e-01_rb,2.8042e-01_rb,3.5020e-01_rb,4.4840e-01_rb,5.5852e-01_rb, &
     &6.6925e-01_rb,7.8169e-01_rb,8.8834e-01_rb/)
      kao(:, 5, 7,10) = (/ &
     &1.7882e-01_rb,2.4826e-01_rb,3.0849e-01_rb,4.0772e-01_rb,5.3530e-01_rb,6.6727e-01_rb, &
     &7.9969e-01_rb,9.3357e-01_rb,1.0615e+00_rb/)
      kao(:, 1, 8,10) = (/ &
     &4.1112e-01_rb,4.0974e-01_rb,4.1065e-01_rb,3.9316e-01_rb,3.5874e-01_rb,3.2789e-01_rb, &
     &2.9581e-01_rb,3.0764e-01_rb,1.9137e-01_rb/)
      kao(:, 2, 8,10) = (/ &
     &4.0366e-01_rb,4.2457e-01_rb,4.2635e-01_rb,4.1703e-01_rb,3.9720e-01_rb,3.6880e-01_rb, &
     &3.6253e-01_rb,4.0327e-01_rb,3.4592e-01_rb/)
      kao(:, 3, 8,10) = (/ &
     &3.8607e-01_rb,4.4207e-01_rb,4.4571e-01_rb,4.4555e-01_rb,4.3797e-01_rb,4.1894e-01_rb, &
     &4.4727e-01_rb,5.1245e-01_rb,5.5117e-01_rb/)
      kao(:, 4, 8,10) = (/ &
     &3.7227e-01_rb,4.5775e-01_rb,4.6855e-01_rb,4.7950e-01_rb,4.7625e-01_rb,4.8559e-01_rb, &
     &5.4513e-01_rb,6.3288e-01_rb,7.1148e-01_rb/)
      kao(:, 5, 8,10) = (/ &
     &3.6637e-01_rb,4.6773e-01_rb,4.9188e-01_rb,5.1329e-01_rb,5.2260e-01_rb,5.7072e-01_rb, &
     &6.6113e-01_rb,7.6991e-01_rb,8.6824e-01_rb/)
      kao(:, 1, 9,10) = (/ &
     &1.5096e+00_rb,1.3467e+00_rb,1.2191e+00_rb,1.0959e+00_rb,9.8221e-01_rb,8.6433e-01_rb, &
     &6.8915e-01_rb,4.5155e-01_rb,2.5455e-01_rb/)
      kao(:, 2, 9,10) = (/ &
     &1.4729e+00_rb,1.3283e+00_rb,1.2484e+00_rb,1.1515e+00_rb,1.0570e+00_rb,9.2603e-01_rb, &
     &7.4690e-01_rb,5.0470e-01_rb,3.4249e-01_rb/)
      kao(:, 3, 9,10) = (/ &
     &1.4533e+00_rb,1.3348e+00_rb,1.2453e+00_rb,1.1987e+00_rb,1.1351e+00_rb,9.9543e-01_rb, &
     &8.1023e-01_rb,5.6196e-01_rb,4.0591e-01_rb/)
      kao(:, 4, 9,10) = (/ &
     &1.4744e+00_rb,1.3676e+00_rb,1.2850e+00_rb,1.2602e+00_rb,1.1957e+00_rb,1.0697e+00_rb, &
     &8.6704e-01_rb,6.2711e-01_rb,4.8211e-01_rb/)
      kao(:, 5, 9,10) = (/ &
     &1.4999e+00_rb,1.4091e+00_rb,1.3355e+00_rb,1.3264e+00_rb,1.2721e+00_rb,1.1339e+00_rb, &
     &9.2252e-01_rb,7.1025e-01_rb,5.3091e-01_rb/)
      kao(:, 1,10,10) = (/ &
     &7.2829e+00_rb,6.3687e+00_rb,5.4628e+00_rb,4.5827e+00_rb,3.6816e+00_rb,2.8784e+00_rb, &
     &1.9461e+00_rb,1.1228e+00_rb,2.9615e-01_rb/)
      kao(:, 2,10,10) = (/ &
     &7.2567e+00_rb,6.3523e+00_rb,5.5013e+00_rb,4.6178e+00_rb,3.8060e+00_rb,2.8925e+00_rb, &
     &1.9877e+00_rb,1.2045e+00_rb,5.0840e-01_rb/)
      kao(:, 3,10,10) = (/ &
     &7.3526e+00_rb,6.4384e+00_rb,5.5471e+00_rb,4.6686e+00_rb,3.8188e+00_rb,2.9106e+00_rb, &
     &2.0584e+00_rb,1.3182e+00_rb,5.8969e-01_rb/)
      kao(:, 4,10,10) = (/ &
     &7.6517e+00_rb,6.6965e+00_rb,5.8098e+00_rb,4.9129e+00_rb,3.9609e+00_rb,3.0608e+00_rb, &
     &2.1952e+00_rb,1.4642e+00_rb,7.2061e-01_rb/)
      kao(:, 5,10,10) = (/ &
     &7.6682e+00_rb,6.7495e+00_rb,5.8665e+00_rb,4.9992e+00_rb,4.0971e+00_rb,3.1895e+00_rb, &
     &2.3244e+00_rb,1.6171e+00_rb,8.0277e-01_rb/)
      kao(:, 1,11,10) = (/ &
     &1.2863e+01_rb,1.1253e+01_rb,9.6455e+00_rb,8.0559e+00_rb,6.4984e+00_rb,4.9709e+00_rb, &
     &3.3831e+00_rb,1.7991e+00_rb,3.6644e-01_rb/)
      kao(:, 2,11,10) = (/ &
     &1.3151e+01_rb,1.1511e+01_rb,9.8627e+00_rb,8.2765e+00_rb,6.6703e+00_rb,5.1195e+00_rb, &
     &3.4235e+00_rb,1.8440e+00_rb,4.6729e-01_rb/)
      kao(:, 3,11,10) = (/ &
     &1.3385e+01_rb,1.1711e+01_rb,1.0081e+01_rb,8.4279e+00_rb,6.8536e+00_rb,5.2620e+00_rb, &
     &3.6394e+00_rb,1.9452e+00_rb,7.7574e-01_rb/)
      kao(:, 4,11,10) = (/ &
     &1.3506e+01_rb,1.1818e+01_rb,1.0183e+01_rb,8.6029e+00_rb,7.0409e+00_rb,5.3665e+00_rb, &
     &3.7651e+00_rb,2.0591e+00_rb,8.8231e-01_rb/)
      kao(:, 5,11,10) = (/ &
     &1.3392e+01_rb,1.1719e+01_rb,1.0115e+01_rb,8.6680e+00_rb,7.0525e+00_rb,5.5043e+00_rb, &
     &3.9029e+00_rb,2.2010e+00_rb,9.9688e-01_rb/)
      kao(:, 1,12,10) = (/ &
     &1.6670e+01_rb,1.4588e+01_rb,1.2501e+01_rb,1.0421e+01_rb,8.3941e+00_rb,6.4270e+00_rb, &
     &4.3660e+00_rb,2.2934e+00_rb,3.9405e-01_rb/)
      kao(:, 2,12,10) = (/ &
     &1.7105e+01_rb,1.4960e+01_rb,1.2826e+01_rb,1.0751e+01_rb,8.6199e+00_rb,6.6739e+00_rb, &
     &4.5154e+00_rb,2.4126e+00_rb,5.4759e-01_rb/)
      kao(:, 3,12,10) = (/ &
     &1.7307e+01_rb,1.5147e+01_rb,1.3016e+01_rb,1.0902e+01_rb,8.9354e+00_rb,6.8220e+00_rb, &
     &4.7181e+00_rb,2.5631e+00_rb,8.3787e-01_rb/)
      kao(:, 4,12,10) = (/ &
     &1.7279e+01_rb,1.5121e+01_rb,1.3042e+01_rb,1.0948e+01_rb,8.9655e+00_rb,6.8546e+00_rb, &
     &4.7965e+00_rb,2.6658e+00_rb,9.5310e-01_rb/)
      kao(:, 5,12,10) = (/ &
     &1.7622e+01_rb,1.5423e+01_rb,1.3295e+01_rb,1.1391e+01_rb,9.1951e+00_rb,7.1021e+00_rb, &
     &4.9630e+00_rb,2.7853e+00_rb,1.0381e+00_rb/)
      kao(:, 1,13,10) = (/ &
     &1.7285e+01_rb,1.5124e+01_rb,1.2963e+01_rb,1.0843e+01_rb,8.6958e+00_rb,6.7399e+00_rb, &
     &4.5324e+00_rb,2.3893e+00_rb,3.8223e-01_rb/)
      kao(:, 2,13,10) = (/ &
     &1.7394e+01_rb,1.5220e+01_rb,1.3045e+01_rb,1.0933e+01_rb,8.8783e+00_rb,6.8319e+00_rb, &
     &4.6535e+00_rb,2.5278e+00_rb,6.6322e-01_rb/)
      kao(:, 3,13,10) = (/ &
     &1.7534e+01_rb,1.5350e+01_rb,1.3222e+01_rb,1.1038e+01_rb,9.1270e+00_rb,6.8978e+00_rb, &
     &4.7818e+00_rb,2.6214e+00_rb,7.3123e-01_rb/)
      kao(:, 4,13,10) = (/ &
     &1.7891e+01_rb,1.5660e+01_rb,1.3479e+01_rb,1.1400e+01_rb,9.3159e+00_rb,7.0998e+00_rb, &
     &4.9577e+00_rb,2.7354e+00_rb,8.9654e-01_rb/)
      kao(:, 5,13,10) = (/ &
     &1.8019e+01_rb,1.5767e+01_rb,1.3610e+01_rb,1.1614e+01_rb,9.4166e+00_rb,7.3122e+00_rb, &
     &5.1530e+00_rb,2.8785e+00_rb,1.0692e+00_rb/)
      kao(:, 1, 1,11) = (/ &
     &3.1711e-03_rb,2.2841e-01_rb,4.5620e-01_rb,6.8299e-01_rb,9.0802e-01_rb,1.1291e+00_rb, &
     &1.3398e+00_rb,1.4993e+00_rb,1.8159e+00_rb/)
      kao(:, 2, 1,11) = (/ &
     &3.0704e-03_rb,2.7128e-01_rb,5.4234e-01_rb,8.1284e-01_rb,1.0822e+00_rb,1.3468e+00_rb, &
     &1.5998e+00_rb,1.7945e+00_rb,2.1643e+00_rb/)
      kao(:, 3, 1,11) = (/ &
     &2.9866e-03_rb,3.1478e-01_rb,6.2947e-01_rb,9.4408e-01_rb,1.2581e+00_rb,1.5709e+00_rb, &
     &1.8741e+00_rb,2.1028e+00_rb,2.5161e+00_rb/)
      kao(:, 4, 1,11) = (/ &
     &2.9163e-03_rb,3.5938e-01_rb,7.1835e-01_rb,1.0774e+00_rb,1.4362e+00_rb,1.7946e+00_rb, &
     &2.1451e+00_rb,2.4217e+00_rb,2.8722e+00_rb/)
      kao(:, 5, 1,11) = (/ &
     &2.8034e-03_rb,4.0521e-01_rb,8.0970e-01_rb,1.2136e+00_rb,1.6171e+00_rb,2.0187e+00_rb, &
     &2.4125e+00_rb,2.7343e+00_rb,3.2340e+00_rb/)
      kao(:, 1, 2,11) = (/ &
     &5.2383e-03_rb,2.1282e-01_rb,4.2542e-01_rb,6.3727e-01_rb,8.4863e-01_rb,1.0581e+00_rb, &
     &1.2660e+00_rb,1.4631e+00_rb,1.6971e+00_rb/)
      kao(:, 2, 2,11) = (/ &
     &5.1027e-03_rb,2.5414e-01_rb,5.0811e-01_rb,7.6215e-01_rb,1.0157e+00_rb,1.2690e+00_rb, &
     &1.5182e+00_rb,1.7540e+00_rb,2.0313e+00_rb/)
      kao(:, 3, 2,11) = (/ &
     &4.9821e-03_rb,2.9639e-01_rb,5.9276e-01_rb,8.8910e-01_rb,1.1857e+00_rb,1.4825e+00_rb, &
     &1.7783e+00_rb,2.0622e+00_rb,2.3711e+00_rb/)
      kao(:, 4, 2,11) = (/ &
     &4.8330e-03_rb,3.4110e-01_rb,6.8181e-01_rb,1.0221e+00_rb,1.3622e+00_rb,1.7021e+00_rb, &
     &2.0428e+00_rb,2.3773e+00_rb,2.7241e+00_rb/)
      kao(:, 5, 2,11) = (/ &
     &4.6466e-03_rb,3.8845e-01_rb,7.7604e-01_rb,1.1627e+00_rb,1.5479e+00_rb,1.9317e+00_rb, &
     &2.3125e+00_rb,2.6917e+00_rb,3.0955e+00_rb/)
      kao(:, 1, 3,11) = (/ &
     &1.1968e-02_rb,1.8925e-01_rb,3.7814e-01_rb,5.6720e-01_rb,7.5640e-01_rb,9.4598e-01_rb, &
     &1.1354e+00_rb,1.3192e+00_rb,1.5123e+00_rb/)
      kao(:, 2, 3,11) = (/ &
     &1.1738e-02_rb,2.2858e-01_rb,4.5676e-01_rb,6.8520e-01_rb,9.1426e-01_rb,1.1439e+00_rb, &
     &1.3747e+00_rb,1.6035e+00_rb,1.8280e+00_rb/)
      kao(:, 3, 3,11) = (/ &
     &1.1441e-02_rb,2.7028e-01_rb,5.3999e-01_rb,8.0979e-01_rb,1.0799e+00_rb,1.3506e+00_rb, &
     &1.6224e+00_rb,1.8949e+00_rb,2.1592e+00_rb/)
      kao(:, 4, 3,11) = (/ &
     &1.1066e-02_rb,3.1556e-01_rb,6.3024e-01_rb,9.4495e-01_rb,1.2595e+00_rb,1.5736e+00_rb, &
     &1.8868e+00_rb,2.1970e+00_rb,2.5183e+00_rb/)
      kao(:, 5, 3,11) = (/ &
     &1.0627e-02_rb,3.6442e-01_rb,7.2807e-01_rb,1.0914e+00_rb,1.4538e+00_rb,1.8154e+00_rb, &
     &2.1726e+00_rb,2.5191e+00_rb,2.9067e+00_rb/)
      kao(:, 1, 4,11) = (/ &
     &2.8048e-02_rb,1.6151e-01_rb,3.2202e-01_rb,4.8262e-01_rb,6.4375e-01_rb,8.0530e-01_rb, &
     &9.6854e-01_rb,1.1364e+00_rb,1.2863e+00_rb/)
      kao(:, 2, 4,11) = (/ &
     &2.7460e-02_rb,1.9867e-01_rb,3.9620e-01_rb,5.9396e-01_rb,7.9208e-01_rb,9.9102e-01_rb, &
     &1.1917e+00_rb,1.3983e+00_rb,1.5828e+00_rb/)
      kao(:, 3, 4,11) = (/ &
     &2.6672e-02_rb,2.3899e-01_rb,4.7658e-01_rb,7.1429e-01_rb,9.5231e-01_rb,1.1913e+00_rb, &
     &1.4318e+00_rb,1.6791e+00_rb,1.9031e+00_rb/)
      kao(:, 4, 4,11) = (/ &
     &2.5665e-02_rb,2.8345e-01_rb,5.6525e-01_rb,8.4715e-01_rb,1.1294e+00_rb,1.4117e+00_rb, &
     &1.6956e+00_rb,1.9833e+00_rb,2.2571e+00_rb/)
      kao(:, 5, 4,11) = (/ &
     &2.4958e-02_rb,3.3271e-01_rb,6.6383e-01_rb,9.9466e-01_rb,1.3259e+00_rb,1.6572e+00_rb, &
     &1.9874e+00_rb,2.3183e+00_rb,2.6500e+00_rb/)
      kao(:, 1, 5,11) = (/ &
     &5.9740e-02_rb,1.3717e-01_rb,2.6992e-01_rb,4.0387e-01_rb,5.3789e-01_rb,6.7238e-01_rb, &
     &8.0752e-01_rb,9.4549e-01_rb,1.0734e+00_rb/)
      kao(:, 2, 5,11) = (/ &
     &5.8323e-02_rb,1.6942e-01_rb,3.3625e-01_rb,5.0313e-01_rb,6.7036e-01_rb,8.3790e-01_rb, &
     &1.0063e+00_rb,1.1790e+00_rb,1.3380e+00_rb/)
      kao(:, 3, 5,11) = (/ &
     &5.6700e-02_rb,2.0744e-01_rb,4.1183e-01_rb,6.1629e-01_rb,8.2117e-01_rb,1.0263e+00_rb, &
     &1.2326e+00_rb,1.4427e+00_rb,1.6392e+00_rb/)
      kao(:, 4, 5,11) = (/ &
     &5.4845e-02_rb,2.5070e-01_rb,4.9802e-01_rb,7.4525e-01_rb,9.9296e-01_rb,1.2407e+00_rb, &
     &1.4895e+00_rb,1.7409e+00_rb,1.9825e+00_rb/)
      kao(:, 5, 5,11) = (/ &
     &5.3384e-02_rb,2.9836e-01_rb,5.9305e-01_rb,8.8824e-01_rb,1.1834e+00_rb,1.4792e+00_rb, &
     &1.7758e+00_rb,2.0754e+00_rb,2.3639e+00_rb/)
      kao(:, 1, 6,11) = (/ &
     &1.1443e-01_rb,1.5330e-01_rb,2.2814e-01_rb,3.3358e-01_rb,4.4343e-01_rb,5.5347e-01_rb, &
     &6.6399e-01_rb,7.7608e-01_rb,8.8294e-01_rb/)
      kao(:, 2, 6,11) = (/ &
     &1.1139e-01_rb,1.6753e-01_rb,2.8410e-01_rb,4.2322e-01_rb,5.6271e-01_rb,7.0257e-01_rb, &
     &8.4275e-01_rb,9.8447e-01_rb,1.1213e+00_rb/)
      kao(:, 3, 6,11) = (/ &
     &1.0816e-01_rb,1.9391e-01_rb,3.5154e-01_rb,5.2463e-01_rb,6.9799e-01_rb,8.7181e-01_rb, &
     &1.0460e+00_rb,1.2220e+00_rb,1.3917e+00_rb/)
      kao(:, 4, 6,11) = (/ &
     &1.0514e-01_rb,2.2522e-01_rb,4.2775e-01_rb,6.3893e-01_rb,8.5037e-01_rb,1.0626e+00_rb, &
     &1.2753e+00_rb,1.4901e+00_rb,1.6964e+00_rb/)
      kao(:, 5, 6,11) = (/ &
     &1.0414e-01_rb,2.6189e-01_rb,5.1110e-01_rb,7.6424e-01_rb,1.0178e+00_rb,1.2716e+00_rb, &
     &1.5264e+00_rb,1.7854e+00_rb,2.0311e+00_rb/)
      kao(:, 1, 7,11) = (/ &
     &2.3680e-01_rb,2.5474e-01_rb,2.7467e-01_rb,2.9968e-01_rb,3.6549e-01_rb,4.4931e-01_rb, &
     &5.3835e-01_rb,6.2811e-01_rb,7.1559e-01_rb/)
      kao(:, 2, 7,11) = (/ &
     &2.3126e-01_rb,2.6667e-01_rb,2.9742e-01_rb,3.6770e-01_rb,4.6828e-01_rb,5.8355e-01_rb, &
     &6.9932e-01_rb,8.1593e-01_rb,9.2966e-01_rb/)
      kao(:, 3, 7,11) = (/ &
     &2.2333e-01_rb,2.8145e-01_rb,3.3561e-01_rb,4.4803e-01_rb,5.8828e-01_rb,7.3378e-01_rb, &
     &8.7969e-01_rb,1.0266e+00_rb,1.1700e+00_rb/)
      kao(:, 4, 7,11) = (/ &
     &2.2006e-01_rb,2.9370e-01_rb,3.8943e-01_rb,5.4388e-01_rb,7.2193e-01_rb,9.0087e-01_rb, &
     &1.0804e+00_rb,1.2609e+00_rb,1.4372e+00_rb/)
      kao(:, 5, 7,11) = (/ &
     &2.1836e-01_rb,3.0927e-01_rb,4.5013e-01_rb,6.5145e-01_rb,8.6638e-01_rb,1.0814e+00_rb, &
     &1.2967e+00_rb,1.5147e+00_rb,1.7257e+00_rb/)
      kao(:, 1, 8,11) = (/ &
     &5.7508e-01_rb,5.5228e-01_rb,5.1832e-01_rb,4.8920e-01_rb,4.5962e-01_rb,4.2790e-01_rb, &
     &4.4544e-01_rb,5.0496e-01_rb,5.4406e-01_rb/)
      kao(:, 2, 8,11) = (/ &
     &5.5570e-01_rb,5.5086e-01_rb,5.3954e-01_rb,5.2836e-01_rb,5.0571e-01_rb,5.1612e-01_rb, &
     &5.7597e-01_rb,6.6763e-01_rb,7.6063e-01_rb/)
      kao(:, 3, 8,11) = (/ &
     &5.4644e-01_rb,5.5917e-01_rb,5.6849e-01_rb,5.6772e-01_rb,5.6845e-01_rb,6.3045e-01_rb, &
     &7.3261e-01_rb,8.5320e-01_rb,9.7237e-01_rb/)
      kao(:, 4, 8,11) = (/ &
     &5.4298e-01_rb,5.6882e-01_rb,5.9693e-01_rb,6.1038e-01_rb,6.5823e-01_rb,7.6720e-01_rb, &
     &9.1091e-01_rb,1.0615e+00_rb,1.2099e+00_rb/)
      kao(:, 5, 8,11) = (/ &
     &5.3663e-01_rb,5.8485e-01_rb,6.2913e-01_rb,6.6732e-01_rb,7.6736e-01_rb,9.2195e-01_rb, &
     &1.1025e+00_rb,1.2850e+00_rb,1.4649e+00_rb/)
      kao(:, 1, 9,11) = (/ &
     &2.5279e+00_rb,2.2431e+00_rb,2.0106e+00_rb,1.7600e+00_rb,1.4790e+00_rb,1.2051e+00_rb, &
     &8.9590e-01_rb,5.8517e-01_rb,3.0795e-01_rb/)
      kao(:, 2, 9,11) = (/ &
     &2.4601e+00_rb,2.1917e+00_rb,2.0103e+00_rb,1.7592e+00_rb,1.5280e+00_rb,1.2575e+00_rb, &
     &9.5967e-01_rb,6.7082e-01_rb,3.9930e-01_rb/)
      kao(:, 3, 9,11) = (/ &
     &2.4168e+00_rb,2.1757e+00_rb,2.0177e+00_rb,1.7994e+00_rb,1.5699e+00_rb,1.3186e+00_rb, &
     &1.0425e+00_rb,7.7727e-01_rb,5.2484e-01_rb/)
      kao(:, 4, 9,11) = (/ &
     &2.3770e+00_rb,2.1985e+00_rb,2.0410e+00_rb,1.8443e+00_rb,1.6274e+00_rb,1.3869e+00_rb, &
     &1.1288e+00_rb,9.2337e-01_rb,5.9212e-01_rb/)
      kao(:, 5, 9,11) = (/ &
     &2.3400e+00_rb,2.1850e+00_rb,2.0960e+00_rb,1.9025e+00_rb,1.6878e+00_rb,1.4602e+00_rb, &
     &1.2280e+00_rb,1.0954e+00_rb,7.2073e-01_rb/)
      kao(:, 1,10,11) = (/ &
     &1.1112e+01_rb,9.7248e+00_rb,8.3361e+00_rb,7.0270e+00_rb,5.7440e+00_rb,4.5096e+00_rb, &
     &3.2779e+00_rb,1.8950e+00_rb,4.0940e-01_rb/)
      kao(:, 2,10,11) = (/ &
     &1.0924e+01_rb,9.5564e+00_rb,8.1917e+00_rb,6.9650e+00_rb,5.7085e+00_rb,4.6298e+00_rb, &
     &3.3486e+00_rb,2.0156e+00_rb,4.6752e-01_rb/)
      kao(:, 3,10,11) = (/ &
     &1.0832e+01_rb,9.4781e+00_rb,8.2229e+00_rb,6.9872e+00_rb,5.8796e+00_rb,4.6954e+00_rb, &
     &3.5108e+00_rb,2.1192e+00_rb,8.0168e-01_rb/)
      kao(:, 4,10,11) = (/ &
     &1.0490e+01_rb,9.1768e+00_rb,8.0065e+00_rb,6.9497e+00_rb,5.9506e+00_rb,4.7816e+00_rb, &
     &3.6391e+00_rb,2.2227e+00_rb,9.3825e-01_rb/)
      kao(:, 5,10,11) = (/ &
     &1.0399e+01_rb,9.1010e+00_rb,8.0219e+00_rb,7.0325e+00_rb,5.9506e+00_rb,4.9243e+00_rb, &
     &3.8048e+00_rb,2.3499e+00_rb,1.1716e+00_rb/)
      kao(:, 1,11,11) = (/ &
     &1.6964e+01_rb,1.4849e+01_rb,1.2726e+01_rb,1.0607e+01_rb,8.5722e+00_rb,6.5304e+00_rb, &
     &4.5723e+00_rb,2.5353e+00_rb,5.7715e-01_rb/)
      kao(:, 2,11,11) = (/ &
     &1.6824e+01_rb,1.4725e+01_rb,1.2623e+01_rb,1.0586e+01_rb,8.5709e+00_rb,6.6756e+00_rb, &
     &4.7183e+00_rb,2.7595e+00_rb,6.2192e-01_rb/)
      kao(:, 3,11,11) = (/ &
     &1.6755e+01_rb,1.4665e+01_rb,1.2572e+01_rb,1.0610e+01_rb,8.6759e+00_rb,6.8661e+00_rb, &
     &4.7973e+00_rb,2.9494e+00_rb,7.1889e-01_rb/)
      kao(:, 4,11,11) = (/ &
     &1.6922e+01_rb,1.4802e+01_rb,1.2769e+01_rb,1.0738e+01_rb,8.9005e+00_rb,6.9130e+00_rb, &
     &5.0022e+00_rb,3.1663e+00_rb,1.1906e+00_rb/)
      kao(:, 5,11,11) = (/ &
     &1.7198e+01_rb,1.5045e+01_rb,1.3001e+01_rb,1.1038e+01_rb,9.0834e+00_rb,7.0679e+00_rb, &
     &5.2222e+00_rb,3.3529e+00_rb,1.3885e+00_rb/)
      kao(:, 1,12,11) = (/ &
     &2.1802e+01_rb,1.9077e+01_rb,1.6353e+01_rb,1.3626e+01_rb,1.0986e+01_rb,8.2719e+00_rb, &
     &5.7215e+00_rb,2.9889e+00_rb,6.5084e-01_rb/)
      kao(:, 2,12,11) = (/ &
     &2.2052e+01_rb,1.9310e+01_rb,1.6546e+01_rb,1.3830e+01_rb,1.1139e+01_rb,8.5492e+00_rb, &
     &5.8655e+00_rb,3.1860e+00_rb,7.2046e-01_rb/)
      kao(:, 3,12,11) = (/ &
     &2.2451e+01_rb,1.9654e+01_rb,1.6841e+01_rb,1.4123e+01_rb,1.1335e+01_rb,8.7814e+00_rb, &
     &5.9237e+00_rb,3.3098e+00_rb,8.3218e-01_rb/)
      kao(:, 4,12,11) = (/ &
     &2.3090e+01_rb,2.0216e+01_rb,1.7353e+01_rb,1.4511e+01_rb,1.1935e+01_rb,9.0228e+00_rb, &
     &6.2499e+00_rb,3.5354e+00_rb,1.3462e+00_rb/)
      kao(:, 5,12,11) = (/ &
     &2.3297e+01_rb,2.0383e+01_rb,1.7601e+01_rb,1.4664e+01_rb,1.2080e+01_rb,9.1910e+00_rb, &
     &6.4263e+00_rb,3.7093e+00_rb,1.5753e+00_rb/)
      kao(:, 1,13,11) = (/ &
     &2.3137e+01_rb,2.0253e+01_rb,1.7360e+01_rb,1.4461e+01_rb,1.1660e+01_rb,8.7711e+00_rb, &
     &6.0766e+00_rb,3.2004e+00_rb,6.2333e-01_rb/)
      kao(:, 2,13,11) = (/ &
     &2.3705e+01_rb,2.0739e+01_rb,1.7789e+01_rb,1.4919e+01_rb,1.1958e+01_rb,9.2720e+00_rb, &
     &6.2579e+00_rb,3.3419e+00_rb,6.6494e-01_rb/)
      kao(:, 3,13,11) = (/ &
     &2.4274e+01_rb,2.1226e+01_rb,1.8190e+01_rb,1.5260e+01_rb,1.2326e+01_rb,9.5663e+00_rb, &
     &6.4788e+00_rb,3.4909e+00_rb,1.1201e+00_rb/)
      kao(:, 4,13,11) = (/ &
     &2.4386e+01_rb,2.1333e+01_rb,1.8377e+01_rb,1.5357e+01_rb,1.2667e+01_rb,9.6781e+00_rb, &
     &6.7508e+00_rb,3.6779e+00_rb,1.3420e+00_rb/)
      kao(:, 5,13,11) = (/ &
     &2.4394e+01_rb,2.1348e+01_rb,1.8396e+01_rb,1.5507e+01_rb,1.2756e+01_rb,9.7415e+00_rb, &
     &6.8863e+00_rb,3.8176e+00_rb,1.6819e+00_rb/)
      kao(:, 1, 1,12) = (/ &
     &3.7292e-03_rb,3.2906e-01_rb,6.5580e-01_rb,9.7933e-01_rb,1.2975e+00_rb,1.6059e+00_rb, &
     &1.8953e+00_rb,2.1342e+00_rb,2.5948e+00_rb/)
      kao(:, 2, 1,12) = (/ &
     &3.6338e-03_rb,3.9331e-01_rb,7.8289e-01_rb,1.1672e+00_rb,1.5430e+00_rb,1.9051e+00_rb, &
     &2.2331e+00_rb,2.4741e+00_rb,3.0857e+00_rb/)
      kao(:, 3, 1,12) = (/ &
     &3.5564e-03_rb,4.6323e-01_rb,9.2190e-01_rb,1.3739e+00_rb,1.8144e+00_rb,2.2323e+00_rb, &
     &2.6037e+00_rb,2.8683e+00_rb,3.6286e+00_rb/)
      kao(:, 4, 1,12) = (/ &
     &3.4829e-03_rb,5.3356e-01_rb,1.0626e+00_rb,1.5845e+00_rb,2.0933e+00_rb,2.5784e+00_rb, &
     &3.0158e+00_rb,3.3179e+00_rb,4.1863e+00_rb/)
      kao(:, 5, 1,12) = (/ &
     &3.4329e-03_rb,6.0324e-01_rb,1.2029e+00_rb,1.7965e+00_rb,2.3794e+00_rb,2.9402e+00_rb, &
     &3.4530e+00_rb,3.8265e+00_rb,4.7585e+00_rb/)
      kao(:, 1, 2,12) = (/ &
     &6.2080e-03_rb,3.1766e-01_rb,6.3367e-01_rb,9.4784e-01_rb,1.2588e+00_rb,1.5639e+00_rb, &
     &1.8558e+00_rb,2.1035e+00_rb,2.5172e+00_rb/)
      kao(:, 2, 2,12) = (/ &
     &6.0635e-03_rb,3.8413e-01_rb,7.6634e-01_rb,1.1462e+00_rb,1.5215e+00_rb,1.8884e+00_rb, &
     &2.2355e+00_rb,2.5176e+00_rb,3.0427e+00_rb/)
      kao(:, 3, 2,12) = (/ &
     &5.9382e-03_rb,4.5423e-01_rb,9.0644e-01_rb,1.3562e+00_rb,1.8011e+00_rb,2.2367e+00_rb, &
     &2.6480e+00_rb,2.9724e+00_rb,3.6017e+00_rb/)
      kao(:, 4, 2,12) = (/ &
     &5.8323e-03_rb,5.2519e-01_rb,1.0487e+00_rb,1.5700e+00_rb,2.0875e+00_rb,2.5951e+00_rb, &
     &3.0747e+00_rb,3.4387e+00_rb,4.1744e+00_rb/)
      kao(:, 5, 2,12) = (/ &
     &5.7338e-03_rb,5.9386e-01_rb,1.1863e+00_rb,1.7768e+00_rb,2.3632e+00_rb,2.9403e+00_rb, &
     &3.4914e+00_rb,3.9302e+00_rb,4.7260e+00_rb/)
      kao(:, 1, 3,12) = (/ &
     &1.4176e-02_rb,2.8836e-01_rb,5.7586e-01_rb,8.6297e-01_rb,1.1493e+00_rb,1.4343e+00_rb, &
     &1.7160e+00_rb,1.9825e+00_rb,2.2980e+00_rb/)
      kao(:, 2, 3,12) = (/ &
     &1.3806e-02_rb,3.5186e-01_rb,7.0276e-01_rb,1.0533e+00_rb,1.4030e+00_rb,1.7510e+00_rb, &
     &2.0945e+00_rb,2.4186e+00_rb,2.8053e+00_rb/)
      kao(:, 3, 3,12) = (/ &
     &1.3514e-02_rb,4.1890e-01_rb,8.3697e-01_rb,1.2547e+00_rb,1.6718e+00_rb,2.0870e+00_rb, &
     &2.4990e+00_rb,2.8908e+00_rb,3.3428e+00_rb/)
      kao(:, 4, 3,12) = (/ &
     &1.3249e-02_rb,4.8707e-01_rb,9.7331e-01_rb,1.4597e+00_rb,1.9459e+00_rb,2.4316e+00_rb, &
     &2.9156e+00_rb,3.3828e+00_rb,3.8910e+00_rb/)
      kao(:, 5, 3,12) = (/ &
     &1.3015e-02_rb,5.5537e-01_rb,1.1100e+00_rb,1.6644e+00_rb,2.2195e+00_rb,2.7745e+00_rb, &
     &3.3300e+00_rb,3.8734e+00_rb,4.4382e+00_rb/)
      kao(:, 1, 4,12) = (/ &
     &3.3184e-02_rb,2.5339e-01_rb,5.0565e-01_rb,7.5787e-01_rb,1.0100e+00_rb,1.2620e+00_rb, &
     &1.5135e+00_rb,1.7634e+00_rb,2.0190e+00_rb/)
      kao(:, 2, 4,12) = (/ &
     &3.2427e-02_rb,3.1318e-01_rb,6.2514e-01_rb,9.3708e-01_rb,1.2489e+00_rb,1.5601e+00_rb, &
     &1.8704e+00_rb,2.1781e+00_rb,2.4966e+00_rb/)
      kao(:, 3, 4,12) = (/ &
     &3.1776e-02_rb,3.7781e-01_rb,7.5434e-01_rb,1.1309e+00_rb,1.5072e+00_rb,1.8833e+00_rb, &
     &2.2580e+00_rb,2.6266e+00_rb,3.0132e+00_rb/)
      kao(:, 4, 4,12) = (/ &
     &3.1130e-02_rb,4.4383e-01_rb,8.8640e-01_rb,1.3292e+00_rb,1.7718e+00_rb,2.2142e+00_rb, &
     &2.6572e+00_rb,3.0981e+00_rb,3.5423e+00_rb/)
      kao(:, 5, 4,12) = (/ &
     &3.0303e-02_rb,5.1080e-01_rb,1.0203e+00_rb,1.5301e+00_rb,2.0399e+00_rb,2.5495e+00_rb, &
     &3.0593e+00_rb,3.5718e+00_rb,4.0785e+00_rb/)
      kao(:, 1, 5,12) = (/ &
     &7.0197e-02_rb,2.2033e-01_rb,4.3908e-01_rb,6.5775e-01_rb,8.7643e-01_rb,1.0950e+00_rb, &
     &1.3134e+00_rb,1.5311e+00_rb,1.7514e+00_rb/)
      kao(:, 2, 5,12) = (/ &
     &6.8848e-02_rb,2.7720e-01_rb,5.5265e-01_rb,8.2808e-01_rb,1.1036e+00_rb,1.3787e+00_rb, &
     &1.6538e+00_rb,1.9275e+00_rb,2.2056e+00_rb/)
      kao(:, 3, 5,12) = (/ &
     &6.6971e-02_rb,3.3853e-01_rb,6.7513e-01_rb,1.0119e+00_rb,1.3485e+00_rb,1.6854e+00_rb, &
     &2.0224e+00_rb,2.3592e+00_rb,2.6952e+00_rb/)
      kao(:, 4, 5,12) = (/ &
     &6.5365e-02_rb,4.0161e-01_rb,8.0132e-01_rb,1.2011e+00_rb,1.6012e+00_rb,2.0009e+00_rb, &
     &2.4008e+00_rb,2.8038e+00_rb,3.2005e+00_rb/)
      kao(:, 5, 5,12) = (/ &
     &6.3431e-02_rb,4.6828e-01_rb,9.3456e-01_rb,1.4005e+00_rb,1.8671e+00_rb,2.3331e+00_rb, &
     &2.7981e+00_rb,3.2640e+00_rb,3.7323e+00_rb/)
      kao(:, 1, 6,12) = (/ &
     &1.3973e-01_rb,2.0662e-01_rb,3.6962e-01_rb,5.5318e-01_rb,7.3681e-01_rb,9.2064e-01_rb, &
     &1.1045e+00_rb,1.2891e+00_rb,1.4713e+00_rb/)
      kao(:, 2, 6,12) = (/ &
     &1.3671e-01_rb,2.4877e-01_rb,4.7403e-01_rb,7.0978e-01_rb,9.4572e-01_rb,1.1814e+00_rb, &
     &1.4179e+00_rb,1.6551e+00_rb,1.8888e+00_rb/)
      kao(:, 3, 6,12) = (/ &
     &1.3368e-01_rb,2.9917e-01_rb,5.8968e-01_rb,8.8312e-01_rb,1.1767e+00_rb,1.4704e+00_rb, &
     &1.7646e+00_rb,2.0602e+00_rb,2.3507e+00_rb/)
      kao(:, 4, 6,12) = (/ &
     &1.2994e-01_rb,3.5894e-01_rb,7.1459e-01_rb,1.0704e+00_rb,1.4263e+00_rb,1.7824e+00_rb, &
     &2.1385e+00_rb,2.4959e+00_rb,2.8498e+00_rb/)
      kao(:, 5, 6,12) = (/ &
     &1.2525e-01_rb,4.2650e-01_rb,8.4975e-01_rb,1.2729e+00_rb,1.6963e+00_rb,2.1200e+00_rb, &
     &2.5435e+00_rb,2.9674e+00_rb,3.3897e+00_rb/)
      kao(:, 1, 7,12) = (/ &
     &2.8974e-01_rb,3.1610e-01_rb,3.5823e-01_rb,4.6930e-01_rb,6.1329e-01_rb,7.6583e-01_rb, &
     &9.1850e-01_rb,1.0715e+00_rb,1.2231e+00_rb/)
      kao(:, 2, 7,12) = (/ &
     &2.8480e-01_rb,3.2946e-01_rb,4.3038e-01_rb,6.0019e-01_rb,7.9796e-01_rb,9.9665e-01_rb, &
     &1.1955e+00_rb,1.3949e+00_rb,1.5921e+00_rb/)
      kao(:, 3, 7,12) = (/ &
     &2.7871e-01_rb,3.5067e-01_rb,5.1857e-01_rb,7.5801e-01_rb,1.0094e+00_rb,1.2608e+00_rb, &
     &1.5125e+00_rb,1.7653e+00_rb,2.0145e+00_rb/)
      kao(:, 4, 7,12) = (/ &
     &2.7300e-01_rb,3.8461e-01_rb,6.2621e-01_rb,9.3301e-01_rb,1.2423e+00_rb,1.5518e+00_rb, &
     &1.8620e+00_rb,2.1734e+00_rb,2.4799e+00_rb/)
      kao(:, 5, 7,12) = (/ &
     &2.6591e-01_rb,4.2736e-01_rb,7.5270e-01_rb,1.1266e+00_rb,1.5003e+00_rb,1.8747e+00_rb, &
     &2.2489e+00_rb,2.6247e+00_rb,2.9958e+00_rb/)
      kao(:, 1, 8,12) = (/ &
     &7.1395e-01_rb,6.8533e-01_rb,6.5866e-01_rb,6.2646e-01_rb,6.1056e-01_rb,6.5589e-01_rb, &
     &7.5232e-01_rb,8.7691e-01_rb,1.0005e+00_rb/)
      kao(:, 2, 8,12) = (/ &
     &7.0494e-01_rb,6.9398e-01_rb,6.9094e-01_rb,6.7668e-01_rb,7.2746e-01_rb,8.4153e-01_rb, &
     &1.0006e+00_rb,1.1668e+00_rb,1.3319e+00_rb/)
      kao(:, 3, 8,12) = (/ &
     &6.9397e-01_rb,7.0598e-01_rb,7.2266e-01_rb,7.6185e-01_rb,8.8476e-01_rb,1.0705e+00_rb, &
     &1.2831e+00_rb,1.4961e+00_rb,1.7076e+00_rb/)
      kao(:, 4, 8,12) = (/ &
     &6.8087e-01_rb,7.2362e-01_rb,7.6721e-01_rb,8.7767e-01_rb,1.0787e+00_rb,1.3359e+00_rb, &
     &1.6015e+00_rb,1.8678e+00_rb,2.1324e+00_rb/)
      kao(:, 5, 8,12) = (/ &
     &6.7139e-01_rb,7.3688e-01_rb,8.3187e-01_rb,1.0246e+00_rb,1.3106e+00_rb,1.6352e+00_rb, &
     &1.9607e+00_rb,2.2865e+00_rb,2.6110e+00_rb/)
      kao(:, 1, 9,12) = (/ &
     &3.4095e+00_rb,2.9940e+00_rb,2.6084e+00_rb,2.2712e+00_rb,1.8987e+00_rb,1.5323e+00_rb, &
     &1.1724e+00_rb,8.1780e-01_rb,4.0421e-01_rb/)
      kao(:, 2, 9,12) = (/ &
     &3.3693e+00_rb,2.9832e+00_rb,2.6354e+00_rb,2.2978e+00_rb,1.9462e+00_rb,1.6166e+00_rb, &
     &1.2713e+00_rb,1.0128e+00_rb,5.1858e-01_rb/)
      kao(:, 3, 9,12) = (/ &
     &3.3273e+00_rb,2.9606e+00_rb,2.6476e+00_rb,2.3138e+00_rb,2.0240e+00_rb,1.7033e+00_rb, &
     &1.3935e+00_rb,1.2814e+00_rb,6.1493e-01_rb/)
      kao(:, 4, 9,12) = (/ &
     &3.2577e+00_rb,2.9148e+00_rb,2.6437e+00_rb,2.3667e+00_rb,2.0968e+00_rb,1.8097e+00_rb, &
     &1.5817e+00_rb,1.6061e+00_rb,8.7818e-01_rb/)
      kao(:, 5, 9,12) = (/ &
     &3.1999e+00_rb,2.9208e+00_rb,2.6366e+00_rb,2.4121e+00_rb,2.1879e+00_rb,1.9394e+00_rb, &
     &1.8270e+00_rb,1.9776e+00_rb,1.4315e+00_rb/)
      kao(:, 1,10,12) = (/ &
     &1.8080e+01_rb,1.5820e+01_rb,1.3560e+01_rb,1.1300e+01_rb,9.1208e+00_rb,6.9174e+00_rb, &
     &4.8050e+00_rb,2.5816e+00_rb,5.9225e-01_rb/)
      kao(:, 2,10,12) = (/ &
     &1.7863e+01_rb,1.5633e+01_rb,1.3401e+01_rb,1.1233e+01_rb,9.0719e+00_rb,6.9705e+00_rb, &
     &4.8769e+00_rb,2.6813e+00_rb,6.3411e-01_rb/)
      kao(:, 3,10,12) = (/ &
     &1.7446e+01_rb,1.5265e+01_rb,1.3103e+01_rb,1.1057e+01_rb,9.0250e+00_rb,7.0448e+00_rb, &
     &4.8922e+00_rb,2.7949e+00_rb,7.0806e-01_rb/)
      kao(:, 4,10,12) = (/ &
     &1.7175e+01_rb,1.5031e+01_rb,1.2961e+01_rb,1.0925e+01_rb,8.9744e+00_rb,7.0129e+00_rb, &
     &4.9823e+00_rb,2.9347e+00_rb,1.2270e+00_rb/)
      kao(:, 5,10,12) = (/ &
     &1.6980e+01_rb,1.4865e+01_rb,1.2847e+01_rb,1.0927e+01_rb,9.0016e+00_rb,7.0557e+00_rb, &
     &5.0460e+00_rb,3.0782e+00_rb,1.3161e+00_rb/)
      kao(:, 1,11,12) = (/ &
     &3.1621e+01_rb,2.7664e+01_rb,2.3716e+01_rb,1.9762e+01_rb,1.5836e+01_rb,1.2028e+01_rb, &
     &8.3404e+00_rb,4.4951e+00_rb,7.7226e-01_rb/)
      kao(:, 2,11,12) = (/ &
     &3.1089e+01_rb,2.7197e+01_rb,2.3314e+01_rb,1.9432e+01_rb,1.5688e+01_rb,1.1946e+01_rb, &
     &8.4568e+00_rb,4.5500e+00_rb,1.2396e+00_rb/)
      kao(:, 3,11,12) = (/ &
     &3.0576e+01_rb,2.6758e+01_rb,2.2933e+01_rb,1.9212e+01_rb,1.5533e+01_rb,1.1953e+01_rb, &
     &8.4697e+00_rb,4.6829e+00_rb,1.0564e+00_rb/)
      kao(:, 4,11,12) = (/ &
     &3.0097e+01_rb,2.6346e+01_rb,2.2583e+01_rb,1.9026e+01_rb,1.5507e+01_rb,1.2181e+01_rb, &
     &8.5232e+00_rb,4.7855e+00_rb,1.0613e+00_rb/)
      kao(:, 5,11,12) = (/ &
     &2.9719e+01_rb,2.6007e+01_rb,2.2377e+01_rb,1.8843e+01_rb,1.5650e+01_rb,1.2244e+01_rb, &
     &8.6646e+00_rb,4.9270e+00_rb,1.7788e+00_rb/)
      kao(:, 1,12,12) = (/ &
     &3.8089e+01_rb,3.3321e+01_rb,2.8563e+01_rb,2.3805e+01_rb,1.9048e+01_rb,1.4479e+01_rb, &
     &1.0042e+01_rb,5.5700e+00_rb,1.0139e+00_rb/)
      kao(:, 2,12,12) = (/ &
     &3.7254e+01_rb,3.2590e+01_rb,2.7941e+01_rb,2.3286e+01_rb,1.8814e+01_rb,1.4266e+01_rb, &
     &1.0187e+01_rb,5.6505e+00_rb,1.3358e+00_rb/)
      kao(:, 3,12,12) = (/ &
     &3.6753e+01_rb,3.2149e+01_rb,2.7565e+01_rb,2.3058e+01_rb,1.8681e+01_rb,1.4515e+01_rb, &
     &1.0441e+01_rb,5.8743e+00_rb,1.3962e+00_rb/)
      kao(:, 4,12,12) = (/ &
     &3.6095e+01_rb,3.1568e+01_rb,2.7073e+01_rb,2.2830e+01_rb,1.8483e+01_rb,1.4754e+01_rb, &
     &1.0446e+01_rb,6.0858e+00_rb,1.2630e+00_rb/)
      kao(:, 5,12,12) = (/ &
     &3.5605e+01_rb,3.1165e+01_rb,2.6709e+01_rb,2.2579e+01_rb,1.8648e+01_rb,1.4780e+01_rb, &
     &1.0721e+01_rb,6.3554e+00_rb,1.9400e+00_rb/)
      kao(:, 1,13,12) = (/ &
     &3.6373e+01_rb,3.1828e+01_rb,2.7282e+01_rb,2.2746e+01_rb,1.8235e+01_rb,1.3858e+01_rb, &
     &9.7116e+00_rb,5.3620e+00_rb,1.4114e+00_rb/)
      kao(:, 2,13,12) = (/ &
     &3.6095e+01_rb,3.1587e+01_rb,2.7062e+01_rb,2.2560e+01_rb,1.8254e+01_rb,1.3864e+01_rb, &
     &1.0010e+01_rb,5.5792e+00_rb,1.4567e+00_rb/)
      kao(:, 3,13,12) = (/ &
     &3.5583e+01_rb,3.1140e+01_rb,2.6700e+01_rb,2.2381e+01_rb,1.8087e+01_rb,1.4135e+01_rb, &
     &1.0160e+01_rb,5.8997e+00_rb,1.1528e+00_rb/)
      kao(:, 4,13,12) = (/ &
     &3.5474e+01_rb,3.1045e+01_rb,2.6613e+01_rb,2.2429e+01_rb,1.8206e+01_rb,1.4545e+01_rb, &
     &1.0311e+01_rb,6.1832e+00_rb,1.6292e+00_rb/)
      kao(:, 5,13,12) = (/ &
     &3.5565e+01_rb,3.1124e+01_rb,2.6773e+01_rb,2.2531e+01_rb,1.8670e+01_rb,1.4802e+01_rb, &
     &1.0686e+01_rb,6.5577e+00_rb,2.3163e+00_rb/)
      kao(:, 1, 1,13) = (/ &
     &4.6813e-03_rb,5.0961e-01_rb,1.0176e+00_rb,1.5220e+00_rb,2.0224e+00_rb,2.5138e+00_rb, &
     &2.9825e+00_rb,3.3723e+00_rb,4.0446e+00_rb/)
      kao(:, 2, 1,13) = (/ &
     &4.5378e-03_rb,5.9150e-01_rb,1.1812e+00_rb,1.7694e+00_rb,2.3540e+00_rb,2.9329e+00_rb, &
     &3.4999e+00_rb,3.9983e+00_rb,4.7077e+00_rb/)
      kao(:, 3, 1,13) = (/ &
     &4.3993e-03_rb,6.8031e-01_rb,1.3585e+00_rb,2.0344e+00_rb,2.7071e+00_rb,3.3760e+00_rb, &
     &4.0375e+00_rb,4.6330e+00_rb,5.4138e+00_rb/)
      kao(:, 4, 1,13) = (/ &
     &4.2648e-03_rb,7.8088e-01_rb,1.5583e+00_rb,2.3313e+00_rb,3.0997e+00_rb,3.8593e+00_rb, &
     &4.5971e+00_rb,5.2655e+00_rb,6.1991e+00_rb/)
      kao(:, 5, 1,13) = (/ &
     &4.1347e-03_rb,8.9242e-01_rb,1.7790e+00_rb,2.6579e+00_rb,3.5250e+00_rb,4.3745e+00_rb, &
     &5.1887e+00_rb,5.9002e+00_rb,7.0497e+00_rb/)
      kao(:, 1, 2,13) = (/ &
     &7.6228e-03_rb,4.9032e-01_rb,9.7924e-01_rb,1.4667e+00_rb,1.9517e+00_rb,2.4315e+00_rb, &
     &2.8986e+00_rb,3.3149e+00_rb,3.9031e+00_rb/)
      kao(:, 2, 2,13) = (/ &
     &7.3845e-03_rb,5.7215e-01_rb,1.1423e+00_rb,1.7099e+00_rb,2.2735e+00_rb,2.8309e+00_rb, &
     &3.3762e+00_rb,3.8853e+00_rb,4.5466e+00_rb/)
      kao(:, 3, 2,13) = (/ &
     &7.1668e-03_rb,6.6400e-01_rb,1.3252e+00_rb,1.9825e+00_rb,2.6343e+00_rb,3.2753e+00_rb, &
     &3.8969e+00_rb,4.4792e+00_rb,5.2681e+00_rb/)
      kao(:, 4, 2,13) = (/ &
     &6.9674e-03_rb,7.6745e-01_rb,1.5306e+00_rb,2.2881e+00_rb,3.0362e+00_rb,3.7701e+00_rb, &
     &4.4777e+00_rb,5.1450e+00_rb,6.0719e+00_rb/)
      kao(:, 5, 2,13) = (/ &
     &6.7870e-03_rb,8.8445e-01_rb,1.7639e+00_rb,2.6357e+00_rb,3.4964e+00_rb,4.3377e+00_rb, &
     &5.1390e+00_rb,5.8595e+00_rb,6.9924e+00_rb/)
      kao(:, 1, 3,13) = (/ &
     &1.6838e-02_rb,4.6207e-01_rb,9.2280e-01_rb,1.3820e+00_rb,1.8387e+00_rb,2.2901e+00_rb, &
     &2.7306e+00_rb,3.1427e+00_rb,3.6769e+00_rb/)
      kao(:, 2, 3,13) = (/ &
     &1.6384e-02_rb,5.4636e-01_rb,1.0908e+00_rb,1.6331e+00_rb,2.1721e+00_rb,2.7041e+00_rb, &
     &3.2216e+00_rb,3.6963e+00_rb,4.3436e+00_rb/)
      kao(:, 3, 3,13) = (/ &
     &1.5944e-02_rb,6.4109e-01_rb,1.2798e+00_rb,1.9157e+00_rb,2.5468e+00_rb,3.1693e+00_rb, &
     &3.7717e+00_rb,4.3145e+00_rb,5.0931e+00_rb/)
      kao(:, 4, 3,13) = (/ &
     &1.5558e-02_rb,7.4877e-01_rb,1.4944e+00_rb,2.2360e+00_rb,2.9707e+00_rb,3.6929e+00_rb, &
     &4.3862e+00_rb,5.0013e+00_rb,5.9409e+00_rb/)
      kao(:, 5, 3,13) = (/ &
     &1.5161e-02_rb,8.6996e-01_rb,1.7361e+00_rb,2.5972e+00_rb,3.4498e+00_rb,4.2854e+00_rb, &
     &5.0840e+00_rb,5.7719e+00_rb,6.8989e+00_rb/)
      kao(:, 1, 4,13) = (/ &
     &3.7812e-02_rb,4.2471e-01_rb,8.4847e-01_rb,1.2717e+00_rb,1.6944e+00_rb,2.1155e+00_rb, &
     &2.5337e+00_rb,2.9379e+00_rb,3.3881e+00_rb/)
      kao(:, 2, 4,13) = (/ &
     &3.6812e-02_rb,5.1516e-01_rb,1.0290e+00_rb,1.5422e+00_rb,2.0538e+00_rb,2.5636e+00_rb, &
     &3.0667e+00_rb,3.5462e+00_rb,4.1066e+00_rb/)
      kao(:, 3, 4,13) = (/ &
     &3.5849e-02_rb,6.1666e-01_rb,1.2316e+00_rb,1.8451e+00_rb,2.4569e+00_rb,3.0640e+00_rb, &
     &3.6618e+00_rb,4.2232e+00_rb,4.9129e+00_rb/)
      kao(:, 4, 4,13) = (/ &
     &3.5079e-02_rb,7.3193e-01_rb,1.4618e+00_rb,2.1899e+00_rb,2.9146e+00_rb,3.6330e+00_rb, &
     &4.3356e+00_rb,4.9789e+00_rb,5.8283e+00_rb/)
      kao(:, 5, 4,13) = (/ &
     &3.4393e-02_rb,8.5645e-01_rb,1.7103e+00_rb,2.5622e+00_rb,3.4100e+00_rb,4.2503e+00_rb, &
     &5.0712e+00_rb,5.8174e+00_rb,6.8190e+00_rb/)
      kao(:, 1, 5,13) = (/ &
     &8.0627e-02_rb,3.8028e-01_rb,7.5923e-01_rb,1.1381e+00_rb,1.5165e+00_rb,1.8947e+00_rb, &
     &2.2720e+00_rb,2.6453e+00_rb,3.0318e+00_rb/)
      kao(:, 2, 5,13) = (/ &
     &7.8562e-02_rb,4.7262e-01_rb,9.4356e-01_rb,1.4144e+00_rb,1.8847e+00_rb,2.3543e+00_rb, &
     &2.8214e+00_rb,3.2802e+00_rb,3.7680e+00_rb/)
      kao(:, 3, 5,13) = (/ &
     &7.6982e-02_rb,5.7810e-01_rb,1.1544e+00_rb,1.7303e+00_rb,2.3053e+00_rb,2.8784e+00_rb, &
     &3.4478e+00_rb,4.0008e+00_rb,4.6091e+00_rb/)
      kao(:, 4, 5,13) = (/ &
     &7.5657e-02_rb,6.9735e-01_rb,1.3928e+00_rb,2.0875e+00_rb,2.7814e+00_rb,3.4728e+00_rb, &
     &4.1587e+00_rb,4.8230e+00_rb,5.5613e+00_rb/)
      kao(:, 5, 5,13) = (/ &
     &7.4333e-02_rb,8.2506e-01_rb,1.6482e+00_rb,2.4702e+00_rb,3.2916e+00_rb,4.1103e+00_rb, &
     &4.9228e+00_rb,5.7104e+00_rb,6.5816e+00_rb/)
      kao(:, 1, 6,13) = (/ &
     &1.6425e-01_rb,3.3608e-01_rb,6.6609e-01_rb,9.9817e-01_rb,1.3302e+00_rb,1.6619e+00_rb, &
     &1.9931e+00_rb,2.3215e+00_rb,2.6587e+00_rb/)
      kao(:, 2, 6,13) = (/ &
     &1.6074e-01_rb,4.2522e-01_rb,8.4831e-01_rb,1.2715e+00_rb,1.6941e+00_rb,2.1166e+00_rb, &
     &2.5378e+00_rb,2.9543e+00_rb,3.3863e+00_rb/)
      kao(:, 3, 6,13) = (/ &
     &1.5834e-01_rb,5.2989e-01_rb,1.0574e+00_rb,1.5848e+00_rb,2.1117e+00_rb,2.6382e+00_rb, &
     &3.1634e+00_rb,3.6820e+00_rb,4.2213e+00_rb/)
      kao(:, 4, 6,13) = (/ &
     &1.5607e-01_rb,6.4460e-01_rb,1.2868e+00_rb,1.9290e+00_rb,2.5707e+00_rb,3.2115e+00_rb, &
     &3.8511e+00_rb,4.4833e+00_rb,5.1392e+00_rb/)
      kao(:, 5, 6,13) = (/ &
     &1.5296e-01_rb,7.6868e-01_rb,1.5350e+00_rb,2.3008e+00_rb,3.0663e+00_rb,3.8313e+00_rb, &
     &4.5942e+00_rb,5.3490e+00_rb,6.1304e+00_rb/)
      kao(:, 1, 7,13) = (/ &
     &3.5667e-01_rb,3.9890e-01_rb,5.8897e-01_rb,8.6077e-01_rb,1.1469e+00_rb,1.4325e+00_rb, &
     &1.7184e+00_rb,2.0036e+00_rb,2.2913e+00_rb/)
      kao(:, 2, 7,13) = (/ &
     &3.5161e-01_rb,4.4967e-01_rb,7.5040e-01_rb,1.1221e+00_rb,1.4950e+00_rb,1.8682e+00_rb, &
     &2.2405e+00_rb,2.6119e+00_rb,2.9872e+00_rb/)
      kao(:, 3, 7,13) = (/ &
     &3.4606e-01_rb,5.1859e-01_rb,9.4635e-01_rb,1.4181e+00_rb,1.8897e+00_rb,2.3609e+00_rb, &
     &2.8323e+00_rb,3.3018e+00_rb,3.7764e+00_rb/)
      kao(:, 4, 7,13) = (/ &
     &3.3825e-01_rb,6.0265e-01_rb,1.1668e+00_rb,1.7485e+00_rb,2.3304e+00_rb,2.9114e+00_rb, &
     &3.4925e+00_rb,4.0716e+00_rb,4.6575e+00_rb/)
      kao(:, 5, 7,13) = (/ &
     &3.3043e-01_rb,7.1091e-01_rb,1.4088e+00_rb,2.1112e+00_rb,2.8136e+00_rb,3.5161e+00_rb, &
     &4.2182e+00_rb,4.9170e+00_rb,5.6239e+00_rb/)
      kao(:, 1, 8,13) = (/ &
     &9.2067e-01_rb,8.6182e-01_rb,8.2952e-01_rb,8.7491e-01_rb,1.0086e+00_rb,1.2181e+00_rb, &
     &1.4607e+00_rb,1.7034e+00_rb,1.9462e+00_rb/)
      kao(:, 2, 8,13) = (/ &
     &9.0671e-01_rb,8.7567e-01_rb,8.9610e-01_rb,1.0491e+00_rb,1.2996e+00_rb,1.6153e+00_rb, &
     &1.9371e+00_rb,2.2592e+00_rb,2.5816e+00_rb/)
      kao(:, 3, 8,13) = (/ &
     &8.8973e-01_rb,8.8439e-01_rb,1.0044e+00_rb,1.2766e+00_rb,1.6623e+00_rb,2.0766e+00_rb, &
     &2.4910e+00_rb,2.9049e+00_rb,3.3193e+00_rb/)
      kao(:, 4, 8,13) = (/ &
     &8.6846e-01_rb,9.1357e-01_rb,1.1469e+00_rb,1.5675e+00_rb,2.0797e+00_rb,2.5979e+00_rb, &
     &3.1162e+00_rb,3.6352e+00_rb,4.1539e+00_rb/)
      kao(:, 5, 8,13) = (/ &
     &8.4664e-01_rb,9.6627e-01_rb,1.3262e+00_rb,1.9064e+00_rb,2.5399e+00_rb,3.1734e+00_rb, &
     &3.8073e+00_rb,4.4405e+00_rb,5.0740e+00_rb/)
      kao(:, 1, 9,13) = (/ &
     &4.4067e+00_rb,3.8562e+00_rb,3.3799e+00_rb,2.9279e+00_rb,2.4898e+00_rb,2.0023e+00_rb, &
     &1.6061e+00_rb,1.4556e+00_rb,5.3966e-01_rb/)
      kao(:, 2, 9,13) = (/ &
     &4.3584e+00_rb,3.8431e+00_rb,3.3719e+00_rb,2.9985e+00_rb,2.5395e+00_rb,2.1329e+00_rb, &
     &1.8824e+00_rb,1.9290e+00_rb,1.2534e+00_rb/)
      kao(:, 3, 9,13) = (/ &
     &4.2715e+00_rb,3.7950e+00_rb,3.4185e+00_rb,3.0378e+00_rb,2.6388e+00_rb,2.3497e+00_rb, &
     &2.2667e+00_rb,2.5092e+00_rb,2.3205e+00_rb/)
      kao(:, 4, 9,13) = (/ &
     &4.1934e+00_rb,3.7482e+00_rb,3.4519e+00_rb,3.0862e+00_rb,2.7969e+00_rb,2.6493e+00_rb, &
     &2.7789e+00_rb,3.1851e+00_rb,3.5246e+00_rb/)
      kao(:, 5, 9,13) = (/ &
     &4.0917e+00_rb,3.7105e+00_rb,3.4634e+00_rb,3.1979e+00_rb,3.0093e+00_rb,3.0525e+00_rb, &
     &3.4012e+00_rb,3.9555e+00_rb,4.5178e+00_rb/)
      kao(:, 1,10,13) = (/ &
     &2.3719e+01_rb,2.0753e+01_rb,1.7788e+01_rb,1.4825e+01_rb,1.1907e+01_rb,9.0621e+00_rb, &
     &6.2541e+00_rb,3.4363e+00_rb,9.1814e-01_rb/)
      kao(:, 2,10,13) = (/ &
     &2.3432e+01_rb,2.0501e+01_rb,1.7571e+01_rb,1.4658e+01_rb,1.1890e+01_rb,9.0622e+00_rb, &
     &6.4024e+00_rb,3.5910e+00_rb,1.1158e+00_rb/)
      kao(:, 3,10,13) = (/ &
     &2.3069e+01_rb,2.0187e+01_rb,1.7300e+01_rb,1.4541e+01_rb,1.1793e+01_rb,9.1570e+00_rb, &
     &6.5099e+00_rb,3.8249e+00_rb,9.3188e-01_rb/)
      kao(:, 4,10,13) = (/ &
     &2.2641e+01_rb,1.9809e+01_rb,1.6998e+01_rb,1.4367e+01_rb,1.1776e+01_rb,9.2770e+00_rb, &
     &6.6180e+00_rb,4.0885e+00_rb,9.9189e-01_rb/)
      kao(:, 5,10,13) = (/ &
     &2.2220e+01_rb,1.9436e+01_rb,1.6770e+01_rb,1.4187e+01_rb,1.1792e+01_rb,9.2910e+00_rb, &
     &6.8771e+00_rb,4.4865e+00_rb,1.9399e+00_rb/)
      kao(:, 1,11,13) = (/ &
     &4.4319e+01_rb,3.8780e+01_rb,3.3240e+01_rb,2.7701e+01_rb,2.2160e+01_rb,1.6704e+01_rb, &
     &1.1307e+01_rb,6.0563e+00_rb,5.8625e-01_rb/)
      kao(:, 2,11,13) = (/ &
     &4.3809e+01_rb,3.8330e+01_rb,3.2852e+01_rb,2.7376e+01_rb,2.1936e+01_rb,1.6626e+01_rb, &
     &1.1378e+01_rb,6.1989e+00_rb,8.1730e-01_rb/)
      kao(:, 3,11,13) = (/ &
     &4.3202e+01_rb,3.7799e+01_rb,3.2400e+01_rb,2.7000e+01_rb,2.1788e+01_rb,1.6575e+01_rb, &
     &1.1586e+01_rb,6.3577e+00_rb,1.8296e+00_rb/)
      kao(:, 4,11,13) = (/ &
     &4.2458e+01_rb,3.7147e+01_rb,3.1841e+01_rb,2.6621e+01_rb,2.1500e+01_rb,1.6577e+01_rb, &
     &1.1663e+01_rb,6.6208e+00_rb,1.8735e+00_rb/)
      kao(:, 5,11,13) = (/ &
     &4.1634e+01_rb,3.6429e+01_rb,3.1221e+01_rb,2.6233e+01_rb,2.1246e+01_rb,1.6580e+01_rb, &
     &1.1685e+01_rb,6.9092e+00_rb,1.7032e+00_rb/)
      kao(:, 1,12,13) = (/ &
     &6.1904e+01_rb,5.4167e+01_rb,4.6429e+01_rb,3.8690e+01_rb,3.0952e+01_rb,2.3248e+01_rb, &
     &1.5661e+01_rb,8.2582e+00_rb,4.3930e-01_rb/)
      kao(:, 2,12,13) = (/ &
     &6.1270e+01_rb,5.3611e+01_rb,4.5953e+01_rb,3.8296e+01_rb,3.0637e+01_rb,2.3190e+01_rb, &
     &1.5723e+01_rb,8.4338e+00_rb,1.2396e+00_rb/)
      kao(:, 3,12,13) = (/ &
     &6.0345e+01_rb,5.2802e+01_rb,4.5261e+01_rb,3.7714e+01_rb,3.0326e+01_rb,2.2898e+01_rb, &
     &1.5824e+01_rb,8.5563e+00_rb,2.5106e+00_rb/)
      kao(:, 4,12,13) = (/ &
     &5.9306e+01_rb,5.1895e+01_rb,4.4474e+01_rb,3.7089e+01_rb,2.9895e+01_rb,2.2756e+01_rb, &
     &1.5929e+01_rb,8.7873e+00_rb,2.2974e+00_rb/)
      kao(:, 5,12,13) = (/ &
     &5.8419e+01_rb,5.1111e+01_rb,4.3811e+01_rb,3.6683e+01_rb,2.9523e+01_rb,2.2821e+01_rb, &
     &1.5911e+01_rb,9.0327e+00_rb,2.2973e+00_rb/)
      kao(:, 1,13,13) = (/ &
     &6.8544e+01_rb,5.9975e+01_rb,5.1408e+01_rb,4.2840e+01_rb,3.4272e+01_rb,2.5775e+01_rb, &
     &1.7340e+01_rb,9.2235e+00_rb,4.0824e-01_rb/)
      kao(:, 2,13,13) = (/ &
     &6.7698e+01_rb,5.9233e+01_rb,5.0770e+01_rb,4.2312e+01_rb,3.3875e+01_rb,2.5602e+01_rb, &
     &1.7402e+01_rb,9.3485e+00_rb,1.7747e+00_rb/)
      kao(:, 3,13,13) = (/ &
     &6.6803e+01_rb,5.8453e+01_rb,5.0103e+01_rb,4.1750e+01_rb,3.3601e+01_rb,2.5372e+01_rb, &
     &1.7686e+01_rb,9.4660e+00_rb,2.9931e+00_rb/)
      kao(:, 4,13,13) = (/ &
     &6.5956e+01_rb,5.7706e+01_rb,4.9463e+01_rb,4.1288e+01_rb,3.3259e+01_rb,2.5336e+01_rb, &
     &1.7750e+01_rb,9.7483e+00_rb,2.7732e+00_rb/)
      kao(:, 5,13,13) = (/ &
     &6.5110e+01_rb,5.6969e+01_rb,4.8829e+01_rb,4.0948e+01_rb,3.2989e+01_rb,2.5559e+01_rb, &
     &1.7705e+01_rb,9.9706e+00_rb,2.2584e+00_rb/)
      kao(:, 1, 1,14) = (/ &
     &5.3617e-03_rb,8.8862e-01_rb,1.7657e+00_rb,2.6246e+00_rb,3.4547e+00_rb,4.2326e+00_rb, &
     &4.9022e+00_rb,5.2848e+00_rb,6.9093e+00_rb/)
      kao(:, 2, 1,14) = (/ &
     &5.2510e-03_rb,1.0670e+00_rb,2.1191e+00_rb,3.1492e+00_rb,4.1423e+00_rb,5.0696e+00_rb, &
     &5.8593e+00_rb,6.2778e+00_rb,8.2844e+00_rb/)
      kao(:, 3, 1,14) = (/ &
     &5.1410e-03_rb,1.2574e+00_rb,2.4970e+00_rb,3.7095e+00_rb,4.8778e+00_rb,5.9654e+00_rb, &
     &6.8831e+00_rb,7.3419e+00_rb,9.7553e+00_rb/)
      kao(:, 4, 1,14) = (/ &
     &5.0266e-03_rb,1.4566e+00_rb,2.8932e+00_rb,4.2987e+00_rb,5.6515e+00_rb,6.9086e+00_rb, &
     &7.9640e+00_rb,8.4651e+00_rb,1.1303e+01_rb/)
      kao(:, 5, 1,14) = (/ &
     &4.9104e-03_rb,1.6562e+00_rb,3.2920e+00_rb,4.8956e+00_rb,6.4440e+00_rb,7.8868e+00_rb, &
     &9.0871e+00_rb,9.6375e+00_rb,1.2888e+01_rb/)
      kao(:, 1, 2,14) = (/ &
     &8.7449e-03_rb,9.0334e-01_rb,1.7988e+00_rb,2.6821e+00_rb,3.5450e+00_rb,4.3691e+00_rb, &
     &5.1072e+00_rb,5.5794e+00_rb,7.0898e+00_rb/)
      kao(:, 2, 2,14) = (/ &
     &8.5771e-03_rb,1.0973e+00_rb,2.1846e+00_rb,3.2569e+00_rb,4.3031e+00_rb,5.3011e+00_rb, &
     &6.1907e+00_rb,6.7403e+00_rb,8.6059e+00_rb/)
      kao(:, 3, 2,14) = (/ &
     &8.3757e-03_rb,1.3007e+00_rb,2.5910e+00_rb,3.8649e+00_rb,5.1100e+00_rb,6.3010e+00_rb, &
     &7.3563e+00_rb,7.9915e+00_rb,1.0220e+01_rb/)
      kao(:, 4, 2,14) = (/ &
     &8.1726e-03_rb,1.5076e+00_rb,3.0039e+00_rb,4.4835e+00_rb,5.9326e+00_rb,7.3236e+00_rb, &
     &8.5740e+00_rb,9.3177e+00_rb,1.1865e+01_rb/)
      kao(:, 5, 2,14) = (/ &
     &7.9706e-03_rb,1.7155e+00_rb,3.4194e+00_rb,5.1046e+00_rb,6.7592e+00_rb,8.3528e+00_rb, &
     &9.7973e+00_rb,1.0700e+01_rb,1.3518e+01_rb/)
      kao(:, 1, 3,14) = (/ &
     &1.9310e-02_rb,8.7468e-01_rb,1.7453e+00_rb,2.6103e+00_rb,3.4663e+00_rb,4.3037e+00_rb, &
     &5.0955e+00_rb,5.7297e+00_rb,6.9321e+00_rb/)
      kao(:, 2, 3,14) = (/ &
     &1.8969e-02_rb,1.0738e+00_rb,2.1432e+00_rb,3.2076e+00_rb,4.2608e+00_rb,5.2941e+00_rb, &
     &6.2765e+00_rb,7.0620e+00_rb,8.5210e+00_rb/)
      kao(:, 3, 3,14) = (/ &
     &1.8605e-02_rb,1.2813e+00_rb,2.5582e+00_rb,3.8286e+00_rb,5.0883e+00_rb,6.3262e+00_rb, &
     &7.5114e+00_rb,8.4846e+00_rb,1.0176e+01_rb/)
      kao(:, 4, 3,14) = (/ &
     &1.8213e-02_rb,1.4944e+00_rb,2.9835e+00_rb,4.4663e+00_rb,5.9372e+00_rb,7.3859e+00_rb, &
     &8.7760e+00_rb,9.9435e+00_rb,1.1874e+01_rb/)
      kao(:, 5, 3,14) = (/ &
     &1.7876e-02_rb,1.7091e+00_rb,3.4131e+00_rb,5.1104e+00_rb,6.7954e+00_rb,8.4548e+00_rb, &
     &1.0056e+01_rb,1.1421e+01_rb,1.3590e+01_rb/)
      kao(:, 1, 4,14) = (/ &
     &4.4344e-02_rb,8.3370e-01_rb,1.6650e+00_rb,2.4941e+00_rb,3.3185e+00_rb,4.1344e+00_rb, &
     &4.9297e+00_rb,5.6430e+00_rb,6.6364e+00_rb/)
      kao(:, 2, 4,14) = (/ &
     &4.3566e-02_rb,1.0283e+00_rb,2.0545e+00_rb,3.0778e+00_rb,4.0971e+00_rb,5.1067e+00_rb, &
     &6.0943e+00_rb,6.9950e+00_rb,8.1935e+00_rb/)
      kao(:, 3, 4,14) = (/ &
     &4.2932e-02_rb,1.2330e+00_rb,2.4636e+00_rb,3.6915e+00_rb,4.9148e+00_rb,6.1292e+00_rb, &
     &7.3214e+00_rb,8.4254e+00_rb,9.8287e+00_rb/)
      kao(:, 4, 4,14) = (/ &
     &4.2251e-02_rb,1.4427e+00_rb,2.8831e+00_rb,4.3208e+00_rb,5.7549e+00_rb,7.1799e+00_rb, &
     &8.5827e+00_rb,9.9037e+00_rb,1.1509e+01_rb/)
      kao(:, 5, 4,14) = (/ &
     &4.1551e-02_rb,1.6617e+00_rb,3.3208e+00_rb,4.9772e+00_rb,6.6290e+00_rb,8.2714e+00_rb, &
     &9.8934e+00_rb,1.1428e+01_rb,1.3257e+01_rb/)
      kao(:, 1, 5,14) = (/ &
     &9.4754e-02_rb,7.8821e-01_rb,1.5748e+00_rb,2.3603e+00_rb,3.1437e+00_rb,3.9237e+00_rb, &
     &4.6947e+00_rb,5.4302e+00_rb,6.2865e+00_rb/)
      kao(:, 2, 5,14) = (/ &
     &9.3575e-02_rb,9.7845e-01_rb,1.9550e+00_rb,2.9310e+00_rb,3.9047e+00_rb,4.8748e+00_rb, &
     &5.8356e+00_rb,6.7595e+00_rb,7.8084e+00_rb/)
      kao(:, 3, 5,14) = (/ &
     &9.2188e-02_rb,1.1782e+00_rb,2.3547e+00_rb,3.5302e+00_rb,4.7038e+00_rb,5.8743e+00_rb, &
     &7.0357e+00_rb,8.1610e+00_rb,9.4064e+00_rb/)
      kao(:, 4, 5,14) = (/ &
     &9.0816e-02_rb,1.3863e+00_rb,2.7710e+00_rb,4.1546e+00_rb,5.5362e+00_rb,6.9136e+00_rb, &
     &8.2813e+00_rb,9.6122e+00_rb,1.1071e+01_rb/)
      kao(:, 5, 5,14) = (/ &
     &8.9641e-02_rb,1.6059e+00_rb,3.2095e+00_rb,4.8121e+00_rb,6.4122e+00_rb,8.0077e+00_rb, &
     &9.5924e+00_rb,1.1136e+01_rb,1.2823e+01_rb/)
      kao(:, 1, 6,14) = (/ &
     &1.9317e-01_rb,7.2809e-01_rb,1.4547e+00_rb,2.1811e+00_rb,2.9065e+00_rb,3.6305e+00_rb, &
     &4.3507e+00_rb,5.0577e+00_rb,5.8117e+00_rb/)
      kao(:, 2, 6,14) = (/ &
     &1.9031e-01_rb,9.0988e-01_rb,1.8182e+00_rb,2.7260e+00_rb,3.6329e+00_rb,4.5385e+00_rb, &
     &5.4408e+00_rb,6.3293e+00_rb,7.2643e+00_rb/)
      kao(:, 3, 6,14) = (/ &
     &1.8758e-01_rb,1.1038e+00_rb,2.2061e+00_rb,3.3074e+00_rb,4.4083e+00_rb,5.5073e+00_rb, &
     &6.6028e+00_rb,7.6822e+00_rb,8.8149e+00_rb/)
      kao(:, 4, 6,14) = (/ &
     &1.8516e-01_rb,1.3141e+00_rb,2.6258e+00_rb,3.9372e+00_rb,5.2485e+00_rb,6.5558e+00_rb, &
     &7.8598e+00_rb,9.1449e+00_rb,1.0495e+01_rb/)
      kao(:, 5, 6,14) = (/ &
     &1.8433e-01_rb,1.5378e+00_rb,3.0732e+00_rb,4.6078e+00_rb,6.1419e+00_rb,7.6735e+00_rb, &
     &9.1990e+00_rb,1.0701e+01_rb,1.2282e+01_rb/)
      kao(:, 1, 7,14) = (/ &
     &4.2545e-01_rb,7.0197e-01_rb,1.3229e+00_rb,1.9833e+00_rb,2.6434e+00_rb,3.3025e+00_rb, &
     &3.9608e+00_rb,4.6133e+00_rb,5.2849e+00_rb/)
      kao(:, 2, 7,14) = (/ &
     &4.1807e-01_rb,8.4884e-01_rb,1.6721e+00_rb,2.5066e+00_rb,3.3410e+00_rb,4.1750e+00_rb, &
     &5.0067e+00_rb,5.8326e+00_rb,6.6798e+00_rb/)
      kao(:, 3, 7,14) = (/ &
     &4.1332e-01_rb,1.0301e+00_rb,2.0577e+00_rb,3.0848e+00_rb,4.1118e+00_rb,5.1378e+00_rb, &
     &6.1618e+00_rb,7.1777e+00_rb,8.2211e+00_rb/)
      kao(:, 4, 7,14) = (/ &
     &4.0936e-01_rb,1.2414e+00_rb,2.4798e+00_rb,3.7180e+00_rb,4.9556e+00_rb,6.1921e+00_rb, &
     &7.4267e+00_rb,8.6520e+00_rb,9.9083e+00_rb/)
      kao(:, 5, 7,14) = (/ &
     &4.0685e-01_rb,1.4675e+00_rb,2.9318e+00_rb,4.3961e+00_rb,5.8593e+00_rb,7.3214e+00_rb, &
     &8.7804e+00_rb,1.0230e+01_rb,1.1715e+01_rb/)
      kao(:, 1, 8,14) = (/ &
     &1.1195e+00_rb,1.0895e+00_rb,1.3598e+00_rb,1.7846e+00_rb,2.3466e+00_rb,2.9325e+00_rb, &
     &3.5182e+00_rb,4.1012e+00_rb,4.6898e+00_rb/)
      kao(:, 2, 8,14) = (/ &
     &1.1037e+00_rb,1.1788e+00_rb,1.6176e+00_rb,2.2927e+00_rb,3.0553e+00_rb,3.8181e+00_rb, &
     &4.5804e+00_rb,5.3406e+00_rb,6.1069e+00_rb/)
      kao(:, 3, 8,14) = (/ &
     &1.0884e+00_rb,1.2851e+00_rb,1.9342e+00_rb,2.8650e+00_rb,3.8186e+00_rb,4.7719e+00_rb, &
     &5.7236e+00_rb,6.6739e+00_rb,7.6330e+00_rb/)
      kao(:, 4, 8,14) = (/ &
     &1.0777e+00_rb,1.3879e+00_rb,2.3316e+00_rb,3.4927e+00_rb,4.6551e+00_rb,5.8169e+00_rb, &
     &6.9783e+00_rb,8.1364e+00_rb,9.3055e+00_rb/)
      kao(:, 5, 8,14) = (/ &
     &1.0697e+00_rb,1.5195e+00_rb,2.7843e+00_rb,4.1737e+00_rb,5.5632e+00_rb,6.9521e+00_rb, &
     &8.3391e+00_rb,9.7222e+00_rb,1.1121e+01_rb/)
      kao(:, 1, 9,14) = (/ &
     &5.6190e+00_rb,4.9167e+00_rb,4.3250e+00_rb,3.7358e+00_rb,3.4047e+00_rb,3.1579e+00_rb, &
     &3.1500e+00_rb,3.5459e+00_rb,3.9362e+00_rb/)
      kao(:, 2, 9,14) = (/ &
     &5.5255e+00_rb,4.8418e+00_rb,4.3146e+00_rb,3.9154e+00_rb,3.7623e+00_rb,3.7282e+00_rb, &
     &4.0980e+00_rb,4.7612e+00_rb,5.4406e+00_rb/)
      kao(:, 3, 9,14) = (/ &
     &5.4430e+00_rb,4.8357e+00_rb,4.3573e+00_rb,4.2040e+00_rb,4.1594e+00_rb,4.5007e+00_rb, &
     &5.2453e+00_rb,6.1166e+00_rb,6.9893e+00_rb/)
      kao(:, 4, 9,14) = (/ &
     &5.3842e+00_rb,4.8388e+00_rb,4.5334e+00_rb,4.4707e+00_rb,4.7073e+00_rb,5.4536e+00_rb, &
     &6.5005e+00_rb,7.5810e+00_rb,8.6617e+00_rb/)
      kao(:, 5, 9,14) = (/ &
     &5.3385e+00_rb,4.8392e+00_rb,4.7032e+00_rb,4.7787e+00_rb,5.4197e+00_rb,6.5502e+00_rb, &
     &7.8564e+00_rb,9.1615e+00_rb,1.0470e+01_rb/)
      kao(:, 1,10,14) = (/ &
     &3.1503e+01_rb,2.7564e+01_rb,2.3627e+01_rb,1.9689e+01_rb,1.5752e+01_rb,1.2029e+01_rb, &
     &8.2850e+00_rb,5.0089e+00_rb,2.6421e-01_rb/)
      kao(:, 2,10,14) = (/ &
     &3.1041e+01_rb,2.7162e+01_rb,2.3282e+01_rb,1.9402e+01_rb,1.5696e+01_rb,1.2052e+01_rb, &
     &8.5363e+00_rb,5.5767e+00_rb,1.2004e+00_rb/)
      kao(:, 3,10,14) = (/ &
     &3.0653e+01_rb,2.6822e+01_rb,2.2991e+01_rb,1.9210e+01_rb,1.5684e+01_rb,1.2101e+01_rb, &
     &9.0343e+00_rb,6.3229e+00_rb,1.6811e+00_rb/)
      kao(:, 4,10,14) = (/ &
     &3.0330e+01_rb,2.6541e+01_rb,2.2749e+01_rb,1.9185e+01_rb,1.5678e+01_rb,1.2425e+01_rb, &
     &9.5207e+00_rb,7.4497e+00_rb,1.5881e+00_rb/)
      kao(:, 5,10,14) = (/ &
     &2.9986e+01_rb,2.6238e+01_rb,2.2506e+01_rb,1.9078e+01_rb,1.5796e+01_rb,1.2833e+01_rb, &
     &1.0036e+01_rb,8.8467e+00_rb,2.8996e+00_rb/)
      kao(:, 1,11,14) = (/ &
     &6.1029e+01_rb,5.3402e+01_rb,4.5771e+01_rb,3.8145e+01_rb,3.0515e+01_rb,2.2887e+01_rb, &
     &1.5544e+01_rb,8.4710e+00_rb,5.1903e-01_rb/)
      kao(:, 2,11,14) = (/ &
     &6.0319e+01_rb,5.2777e+01_rb,4.5238e+01_rb,3.7700e+01_rb,3.0160e+01_rb,2.2857e+01_rb, &
     &1.5622e+01_rb,8.8835e+00_rb,1.0450e+00_rb/)
      kao(:, 3,11,14) = (/ &
     &5.9632e+01_rb,5.2177e+01_rb,4.4723e+01_rb,3.7270e+01_rb,2.9909e+01_rb,2.2818e+01_rb, &
     &1.5829e+01_rb,9.3768e+00_rb,1.3014e+00_rb/)
      kao(:, 4,11,14) = (/ &
     &5.9020e+01_rb,5.1642e+01_rb,4.4266e+01_rb,3.6888e+01_rb,2.9847e+01_rb,2.2764e+01_rb, &
     &1.6262e+01_rb,9.9702e+00_rb,2.2647e+00_rb/)
      kao(:, 5,11,14) = (/ &
     &5.8283e+01_rb,5.0998e+01_rb,4.3714e+01_rb,3.6531e+01_rb,2.9620e+01_rb,2.3044e+01_rb, &
     &1.6667e+01_rb,1.0858e+01_rb,2.1604e+00_rb/)
      kao(:, 1,12,14) = (/ &
     &8.6818e+01_rb,7.5961e+01_rb,6.5105e+01_rb,5.4264e+01_rb,4.3410e+01_rb,3.2557e+01_rb, &
     &2.1958e+01_rb,1.1756e+01_rb,1.0818e+00_rb/)
      kao(:, 2,12,14) = (/ &
     &8.5898e+01_rb,7.5160e+01_rb,6.4423e+01_rb,5.3685e+01_rb,4.2947e+01_rb,3.2329e+01_rb, &
     &2.1951e+01_rb,1.2183e+01_rb,6.2703e-01_rb/)
      kao(:, 3,12,14) = (/ &
     &8.4970e+01_rb,7.4354e+01_rb,6.3731e+01_rb,5.3109e+01_rb,4.2485e+01_rb,3.2310e+01_rb, &
     &2.2149e+01_rb,1.2602e+01_rb,1.0531e+00_rb/)
      kao(:, 4,12,14) = (/ &
     &8.4043e+01_rb,7.3537e+01_rb,6.3034e+01_rb,5.2533e+01_rb,4.2288e+01_rb,3.2140e+01_rb, &
     &2.2696e+01_rb,1.3012e+01_rb,4.3852e+00_rb/)
      kao(:, 5,12,14) = (/ &
     &8.3130e+01_rb,7.2745e+01_rb,6.2351e+01_rb,5.1964e+01_rb,4.2084e+01_rb,3.2159e+01_rb, &
     &2.3067e+01_rb,1.3670e+01_rb,3.1697e+00_rb/)
      kao(:, 1,13,14) = (/ &
     &9.9062e+01_rb,8.6679e+01_rb,7.4295e+01_rb,6.1914e+01_rb,4.9531e+01_rb,3.7148e+01_rb, &
     &2.5053e+01_rb,1.3336e+01_rb,1.0355e+00_rb/)
      kao(:, 2,13,14) = (/ &
     &9.8005e+01_rb,8.5755e+01_rb,7.3503e+01_rb,6.1254e+01_rb,4.9004e+01_rb,3.6913e+01_rb, &
     &2.4991e+01_rb,1.3937e+01_rb,6.5467e-01_rb/)
      kao(:, 3,13,14) = (/ &
     &9.6910e+01_rb,8.4797e+01_rb,7.2683e+01_rb,6.0573e+01_rb,4.8483e+01_rb,3.6814e+01_rb, &
     &2.5110e+01_rb,1.4339e+01_rb,1.8215e+00_rb/)
      kao(:, 4,13,14) = (/ &
     &9.5837e+01_rb,8.3857e+01_rb,7.1882e+01_rb,5.9896e+01_rb,4.8229e+01_rb,3.6584e+01_rb, &
     &2.5752e+01_rb,1.4680e+01_rb,4.3881e+00_rb/)
      kao(:, 5,13,14) = (/ &
     &9.4386e+01_rb,8.2585e+01_rb,7.0788e+01_rb,5.9024e+01_rb,4.7765e+01_rb,3.6452e+01_rb, &
     &2.6308e+01_rb,1.5330e+01_rb,4.3734e+00_rb/)
      kao(:, 1, 1,15) = (/ &
     &6.2662e-03_rb,2.0747e+00_rb,4.1170e+00_rb,6.1089e+00_rb,8.0146e+00_rb,9.7584e+00_rb, &
     &1.1145e+01_rb,1.1422e+01_rb,1.6029e+01_rb/)
      kao(:, 2, 1,15) = (/ &
     &6.1351e-03_rb,2.4271e+00_rb,4.8178e+00_rb,7.1504e+00_rb,9.3877e+00_rb,1.1443e+01_rb, &
     &1.3088e+01_rb,1.3461e+01_rb,1.8775e+01_rb/)
      kao(:, 3, 1,15) = (/ &
     &6.0097e-03_rb,2.7826e+00_rb,5.5248e+00_rb,8.2041e+00_rb,1.0775e+01_rb,1.3145e+01_rb, &
     &1.5060e+01_rb,1.5543e+01_rb,2.1551e+01_rb/)
      kao(:, 4, 1,15) = (/ &
     &5.9020e-03_rb,3.1403e+00_rb,6.2337e+00_rb,9.2578e+00_rb,1.2161e+01_rb,1.4850e+01_rb, &
     &1.7039e+01_rb,1.7642e+01_rb,2.4321e+01_rb/)
      kao(:, 5, 1,15) = (/ &
     &5.8030e-03_rb,3.5098e+00_rb,6.9636e+00_rb,1.0331e+01_rb,1.3560e+01_rb,1.6533e+01_rb, &
     &1.8991e+01_rb,1.9724e+01_rb,2.7120e+01_rb/)
      kao(:, 1, 2,15) = (/ &
     &1.0266e-02_rb,2.1405e+00_rb,4.2622e+00_rb,6.3547e+00_rb,8.3970e+00_rb,1.0344e+01_rb, &
     &1.2064e+01_rb,1.2985e+01_rb,1.6794e+01_rb/)
      kao(:, 2, 2,15) = (/ &
     &1.0014e-02_rb,2.5163e+00_rb,5.0119e+00_rb,7.4752e+00_rb,9.8815e+00_rb,1.2172e+01_rb, &
     &1.4218e+01_rb,1.5356e+01_rb,1.9763e+01_rb/)
      kao(:, 3, 2,15) = (/ &
     &9.8638e-03_rb,2.9079e+00_rb,5.7890e+00_rb,8.6292e+00_rb,1.1401e+01_rb,1.4046e+01_rb, &
     &1.6408e+01_rb,1.7772e+01_rb,2.2802e+01_rb/)
      kao(:, 4, 2,15) = (/ &
     &9.7234e-03_rb,3.3191e+00_rb,6.6057e+00_rb,9.8408e+00_rb,1.2995e+01_rb,1.5989e+01_rb, &
     &1.8633e+01_rb,2.0195e+01_rb,2.5989e+01_rb/)
      kao(:, 5, 2,15) = (/ &
     &9.5893e-03_rb,3.7435e+00_rb,7.4476e+00_rb,1.1096e+01_rb,1.4644e+01_rb,1.8006e+01_rb, &
     &2.0946e+01_rb,2.2600e+01_rb,2.9288e+01_rb/)
      kao(:, 1, 3,15) = (/ &
     &2.3937e-02_rb,2.0884e+00_rb,4.1689e+00_rb,6.2403e+00_rb,8.2897e+00_rb,1.0301e+01_rb, &
     &1.2219e+01_rb,1.3769e+01_rb,1.6579e+01_rb/)
      kao(:, 2, 3,15) = (/ &
     &2.3289e-02_rb,2.4959e+00_rb,4.9798e+00_rb,7.4513e+00_rb,9.8967e+00_rb,1.2286e+01_rb, &
     &1.4557e+01_rb,1.6412e+01_rb,1.9793e+01_rb/)
      kao(:, 3, 3,15) = (/ &
     &2.2745e-02_rb,2.9294e+00_rb,5.8465e+00_rb,8.7448e+00_rb,1.1610e+01_rb,1.4408e+01_rb, &
     &1.7054e+01_rb,1.9158e+01_rb,2.3219e+01_rb/)
      kao(:, 4, 3,15) = (/ &
     &2.2330e-02_rb,3.3848e+00_rb,6.7550e+00_rb,1.0101e+01_rb,1.3409e+01_rb,1.6637e+01_rb, &
     &1.9679e+01_rb,2.2057e+01_rb,2.6817e+01_rb/)
      kao(:, 5, 3,15) = (/ &
     &2.1962e-02_rb,3.8549e+00_rb,7.6925e+00_rb,1.1504e+01_rb,1.5265e+01_rb,1.8935e+01_rb, &
     &2.2399e+01_rb,2.5067e+01_rb,3.0529e+01_rb/)
      kao(:, 1, 4,15) = (/ &
     &5.8511e-02_rb,1.9940e+00_rb,3.9844e+00_rb,5.9687e+00_rb,7.9437e+00_rb,9.9016e+00_rb, &
     &1.1811e+01_rb,1.3553e+01_rb,1.5887e+01_rb/)
      kao(:, 2, 4,15) = (/ &
     &5.6982e-02_rb,2.4233e+00_rb,4.8420e+00_rb,7.2530e+00_rb,9.6512e+00_rb,1.2029e+01_rb, &
     &1.4341e+01_rb,1.6438e+01_rb,1.9302e+01_rb/)
      kao(:, 3, 4,15) = (/ &
     &5.5673e-02_rb,2.8886e+00_rb,5.7711e+00_rb,8.6444e+00_rb,1.1501e+01_rb,1.4329e+01_rb, &
     &1.7084e+01_rb,1.9562e+01_rb,2.3002e+01_rb/)
      kao(:, 4, 4,15) = (/ &
     &5.4580e-02_rb,3.3869e+00_rb,6.7661e+00_rb,1.0134e+01_rb,1.3482e+01_rb,1.6795e+01_rb, &
     &2.0015e+01_rb,2.2895e+01_rb,2.6964e+01_rb/)
      kao(:, 5, 4,15) = (/ &
     &5.3615e-02_rb,3.9077e+00_rb,7.8058e+00_rb,1.1691e+01_rb,1.5555e+01_rb,1.9373e+01_rb, &
     &2.3083e+01_rb,2.6384e+01_rb,3.1109e+01_rb/)
      kao(:, 1, 5,15) = (/ &
     &1.2989e-01_rb,1.8900e+00_rb,3.7783e+00_rb,5.6637e+00_rb,7.5463e+00_rb,9.4177e+00_rb, &
     &1.1267e+01_rb,1.3028e+01_rb,1.5093e+01_rb/)
      kao(:, 2, 5,15) = (/ &
     &1.2665e-01_rb,2.3422e+00_rb,4.6829e+00_rb,7.0179e+00_rb,9.3476e+00_rb,1.1662e+01_rb, &
     &1.3953e+01_rb,1.6125e+01_rb,1.8695e+01_rb/)
      kao(:, 3, 5,15) = (/ &
     &1.2397e-01_rb,2.8405e+00_rb,5.6772e+00_rb,8.5098e+00_rb,1.1334e+01_rb,1.4142e+01_rb, &
     &1.6912e+01_rb,1.9527e+01_rb,2.2668e+01_rb/)
      kao(:, 4, 5,15) = (/ &
     &1.2164e-01_rb,3.3767e+00_rb,6.7490e+00_rb,1.0116e+01_rb,1.3472e+01_rb,1.6807e+01_rb, &
     &2.0097e+01_rb,2.3197e+01_rb,2.6943e+01_rb/)
      kao(:, 5, 5,15) = (/ &
     &1.1945e-01_rb,3.9432e+00_rb,7.8814e+00_rb,1.1812e+01_rb,1.5732e+01_rb,1.9624e+01_rb, &
     &2.3463e+01_rb,2.7077e+01_rb,3.1463e+01_rb/)
      kao(:, 1, 6,15) = (/ &
     &2.7269e-01_rb,1.7592e+00_rb,3.5166e+00_rb,5.2733e+00_rb,7.0272e+00_rb,8.7771e+00_rb, &
     &1.0515e+01_rb,1.2209e+01_rb,1.4054e+01_rb/)
      kao(:, 2, 6,15) = (/ &
     &2.6586e-01_rb,2.2331e+00_rb,4.4644e+00_rb,6.6948e+00_rb,8.9210e+00_rb,1.1141e+01_rb, &
     &1.3343e+01_rb,1.5486e+01_rb,1.7841e+01_rb/)
      kao(:, 3, 6,15) = (/ &
     &2.5987e-01_rb,2.7625e+00_rb,5.5224e+00_rb,8.2799e+00_rb,1.1033e+01_rb,1.3778e+01_rb, &
     &1.6504e+01_rb,1.9146e+01_rb,2.2064e+01_rb/)
      kao(:, 4, 6,15) = (/ &
     &2.5470e-01_rb,3.3380e+00_rb,6.6713e+00_rb,1.0004e+01_rb,1.3331e+01_rb,1.6646e+01_rb, &
     &1.9934e+01_rb,2.3121e+01_rb,2.6661e+01_rb/)
      kao(:, 5, 6,15) = (/ &
     &2.4980e-01_rb,3.9509e+00_rb,7.8982e+00_rb,1.1841e+01_rb,1.5778e+01_rb,1.9702e+01_rb, &
     &2.3595e+01_rb,2.7362e+01_rb,3.1554e+01_rb/)
      kao(:, 1, 7,15) = (/ &
     &6.0751e-01_rb,1.6155e+00_rb,3.2295e+00_rb,4.8433e+00_rb,6.4556e+00_rb,8.0666e+00_rb, &
     &9.6721e+00_rb,1.1255e+01_rb,1.2910e+01_rb/)
      kao(:, 2, 7,15) = (/ &
     &5.9214e-01_rb,2.1056e+00_rb,4.2107e+00_rb,6.3119e+00_rb,8.4131e+00_rb,1.0511e+01_rb, &
     &1.2602e+01_rb,1.4667e+01_rb,1.6825e+01_rb/)
      kao(:, 3, 7,15) = (/ &
     &5.7904e-01_rb,2.6613e+00_rb,5.3198e+00_rb,7.9772e+00_rb,1.0632e+01_rb,1.3285e+01_rb, &
     &1.5925e+01_rb,1.8528e+01_rb,2.1262e+01_rb/)
      kao(:, 4, 7,15) = (/ &
     &5.6819e-01_rb,3.2710e+00_rb,6.5412e+00_rb,9.8097e+00_rb,1.3074e+01_rb,1.6330e+01_rb, &
     &1.9581e+01_rb,2.2775e+01_rb,2.6146e+01_rb/)
      kao(:, 5, 7,15) = (/ &
     &5.5940e-01_rb,3.9287e+00_rb,7.8537e+00_rb,1.1777e+01_rb,1.5697e+01_rb,1.9610e+01_rb, &
     &2.3510e+01_rb,2.7346e+01_rb,3.1392e+01_rb/)
      kao(:, 1, 8,15) = (/ &
     &1.5944e+00_rb,1.8253e+00_rb,2.9741e+00_rb,4.4602e+00_rb,5.9464e+00_rb,7.4314e+00_rb, &
     &8.9145e+00_rb,1.0387e+01_rb,1.1892e+01_rb/)
      kao(:, 2, 8,15) = (/ &
     &1.5557e+00_rb,2.1022e+00_rb,3.9278e+00_rb,5.8889e+00_rb,7.8512e+00_rb,9.8106e+00_rb, &
     &1.1770e+01_rb,1.3716e+01_rb,1.5700e+01_rb/)
      kao(:, 3, 8,15) = (/ &
     &1.5271e+00_rb,2.5583e+00_rb,5.0719e+00_rb,7.6057e+00_rb,1.0139e+01_rb,1.2670e+01_rb, &
     &1.5200e+01_rb,1.7713e+01_rb,2.0275e+01_rb/)
      kao(:, 4, 8,15) = (/ &
     &1.5040e+00_rb,3.1773e+00_rb,6.3513e+00_rb,9.5214e+00_rb,1.2692e+01_rb,1.5861e+01_rb, &
     &1.9025e+01_rb,2.2168e+01_rb,2.5379e+01_rb/)
      kao(:, 5, 8,15) = (/ &
     &1.4829e+00_rb,3.8723e+00_rb,7.7385e+00_rb,1.1605e+01_rb,1.5469e+01_rb,1.9331e+01_rb, &
     &2.3189e+01_rb,2.7019e+01_rb,3.0932e+01_rb/)
      kao(:, 1, 9,15) = (/ &
     &7.9603e+00_rb,6.9650e+00_rb,6.1670e+00_rb,5.9409e+00_rb,5.9163e+00_rb,6.8445e+00_rb, &
     &8.2104e+00_rb,9.5758e+00_rb,1.0949e+01_rb/)
      kao(:, 2, 9,15) = (/ &
     &7.7491e+00_rb,6.7807e+00_rb,6.5143e+00_rb,6.5222e+00_rb,7.4209e+00_rb,9.1698e+00_rb, &
     &1.1001e+01_rb,1.2830e+01_rb,1.4668e+01_rb/)
      kao(:, 3, 9,15) = (/ &
     &7.6260e+00_rb,6.7058e+00_rb,6.8127e+00_rb,7.5772e+00_rb,9.5702e+00_rb,1.1961e+01_rb, &
     &1.4350e+01_rb,1.6735e+01_rb,1.9129e+01_rb/)
      kao(:, 4, 9,15) = (/ &
     &7.5222e+00_rb,6.8189e+00_rb,7.2464e+00_rb,9.2266e+00_rb,1.2156e+01_rb,1.5192e+01_rb, &
     &1.8221e+01_rb,2.1251e+01_rb,2.4297e+01_rb/)
      kao(:, 5, 9,15) = (/ &
     &7.4213e+00_rb,6.9850e+00_rb,8.1141e+00_rb,1.1288e+01_rb,1.5041e+01_rb,1.8797e+01_rb, &
     &2.2550e+01_rb,2.6298e+01_rb,3.0063e+01_rb/)
      kao(:, 1,10,15) = (/ &
     &4.4970e+01_rb,3.9351e+01_rb,3.3728e+01_rb,2.8106e+01_rb,2.2484e+01_rb,1.7001e+01_rb, &
     &1.2662e+01_rb,9.3833e+00_rb,3.0844e+00_rb/)
      kao(:, 2,10,15) = (/ &
     &4.3703e+01_rb,3.8241e+01_rb,3.2779e+01_rb,2.7313e+01_rb,2.1862e+01_rb,1.7553e+01_rb, &
     &1.3621e+01_rb,1.2273e+01_rb,6.8989e+00_rb/)
      kao(:, 3,10,15) = (/ &
     &4.2828e+01_rb,3.7472e+01_rb,3.2122e+01_rb,2.6765e+01_rb,2.2087e+01_rb,1.8341e+01_rb, &
     &1.5372e+01_rb,1.6156e+01_rb,1.4584e+01_rb/)
      kao(:, 4,10,15) = (/ &
     &4.2192e+01_rb,3.6921e+01_rb,3.1646e+01_rb,2.6524e+01_rb,2.2776e+01_rb,1.9260e+01_rb, &
     &1.8346e+01_rb,2.0694e+01_rb,2.3643e+01_rb/)
      kao(:, 5,10,15) = (/ &
     &4.1609e+01_rb,3.6411e+01_rb,3.1210e+01_rb,2.6835e+01_rb,2.3359e+01_rb,2.1143e+01_rb, &
     &2.2256e+01_rb,2.5773e+01_rb,2.9447e+01_rb/)
      kao(:, 1,11,15) = (/ &
     &8.7878e+01_rb,7.6892e+01_rb,6.5913e+01_rb,5.4929e+01_rb,4.3942e+01_rb,3.2958e+01_rb, &
     &2.2278e+01_rb,1.3146e+01_rb,1.2934e+00_rb/)
      kao(:, 2,11,15) = (/ &
     &8.5528e+01_rb,7.4837e+01_rb,6.4148e+01_rb,5.3454e+01_rb,4.2765e+01_rb,3.2087e+01_rb, &
     &2.2860e+01_rb,1.5064e+01_rb,9.2095e-01_rb/)
      kao(:, 3,11,15) = (/ &
     &8.3842e+01_rb,7.3371e+01_rb,6.2884e+01_rb,5.2405e+01_rb,4.1923e+01_rb,3.2241e+01_rb, &
     &2.3615e+01_rb,1.8250e+01_rb,5.0549e+00_rb/)
      kao(:, 4,11,15) = (/ &
     &8.2422e+01_rb,7.2123e+01_rb,6.1819e+01_rb,5.1515e+01_rb,4.1404e+01_rb,3.2935e+01_rb, &
     &2.5033e+01_rb,2.2612e+01_rb,1.1342e+01_rb/)
      kao(:, 5,11,15) = (/ &
     &8.1155e+01_rb,7.1009e+01_rb,6.0866e+01_rb,5.0722e+01_rb,4.1664e+01_rb,3.3361e+01_rb, &
     &2.7672e+01_rb,2.7836e+01_rb,2.1588e+01_rb/)
      kao(:, 1,12,15) = (/ &
     &1.2764e+02_rb,1.1168e+02_rb,9.5717e+01_rb,7.9766e+01_rb,6.3821e+01_rb,4.7862e+01_rb, &
     &3.1927e+01_rb,1.7517e+01_rb,5.0733e-04_rb/)
      kao(:, 2,12,15) = (/ &
     &1.2458e+02_rb,1.0901e+02_rb,9.3437e+01_rb,7.7867e+01_rb,6.2287e+01_rb,4.6718e+01_rb, &
     &3.2041e+01_rb,1.8889e+01_rb,1.2295e+00_rb/)
      kao(:, 3,12,15) = (/ &
     &1.2214e+02_rb,1.0687e+02_rb,9.1606e+01_rb,7.6332e+01_rb,6.1069e+01_rb,4.6041e+01_rb, &
     &3.2605e+01_rb,2.1597e+01_rb,1.0764e+00_rb/)
      kao(:, 4,12,15) = (/ &
     &1.1993e+02_rb,1.0494e+02_rb,8.9942e+01_rb,7.4954e+01_rb,5.9961e+01_rb,4.6289e+01_rb, &
     &3.3082e+01_rb,2.5581e+01_rb,1.5268e+00_rb/)
      kao(:, 5,12,15) = (/ &
     &1.1793e+02_rb,1.0318e+02_rb,8.8443e+01_rb,7.3700e+01_rb,5.9323e+01_rb,4.6666e+01_rb, &
     &3.5017e+01_rb,3.0597e+01_rb,1.3997e+01_rb/)
      kao(:, 1,13,15) = (/ &
     &1.4863e+02_rb,1.3005e+02_rb,1.1147e+02_rb,9.2892e+01_rb,7.4315e+01_rb,5.5737e+01_rb, &
     &3.7199e+01_rb,2.0080e+01_rb,6.4806e-01_rb/)
      kao(:, 2,13,15) = (/ &
     &1.4520e+02_rb,1.2704e+02_rb,1.0889e+02_rb,9.0753e+01_rb,7.2597e+01_rb,5.4452e+01_rb, &
     &3.7281e+01_rb,2.1314e+01_rb,1.2348e+00_rb/)
      kao(:, 3,13,15) = (/ &
     &1.4229e+02_rb,1.2450e+02_rb,1.0672e+02_rb,8.8931e+01_rb,7.1150e+01_rb,5.3675e+01_rb, &
     &3.7660e+01_rb,2.4186e+01_rb,1.1194e+00_rb/)
      kao(:, 4,13,15) = (/ &
     &1.3965e+02_rb,1.2219e+02_rb,1.0474e+02_rb,8.7285e+01_rb,6.9825e+01_rb,5.3800e+01_rb, &
     &3.7958e+01_rb,2.8295e+01_rb,2.6393e+00_rb/)
      kao(:, 5,13,15) = (/ &
     &1.3732e+02_rb,1.2017e+02_rb,1.0299e+02_rb,8.5829e+01_rb,6.9074e+01_rb,5.3874e+01_rb, &
     &3.9693e+01_rb,3.3322e+01_rb,1.2822e+01_rb/)
      kao(:, 1, 1,16) = (/ &
     &6.7483e-03_rb,2.7477e+00_rb,5.4216e+00_rb,7.9838e+00_rb,1.0362e+01_rb,1.2414e+01_rb, &
     &1.3792e+01_rb,1.3364e+01_rb,2.0724e+01_rb/)
      kao(:, 2, 1,16) = (/ &
     &6.6276e-03_rb,3.2705e+00_rb,6.4539e+00_rb,9.5049e+00_rb,1.2338e+01_rb,1.4784e+01_rb, &
     &1.6428e+01_rb,1.5929e+01_rb,2.4675e+01_rb/)
      kao(:, 3, 1,16) = (/ &
     &6.5036e-03_rb,3.8140e+00_rb,7.5283e+00_rb,1.1085e+01_rb,1.4395e+01_rb,1.7249e+01_rb, &
     &1.9172e+01_rb,1.8603e+01_rb,2.8790e+01_rb/)
      kao(:, 4, 1,16) = (/ &
     &6.3721e-03_rb,4.3730e+00_rb,8.6292e+00_rb,1.2711e+01_rb,1.6503e+01_rb,1.9784e+01_rb, &
     &2.1993e+01_rb,2.1351e+01_rb,3.3006e+01_rb/)
      kao(:, 5, 1,16) = (/ &
     &6.2363e-03_rb,4.9396e+00_rb,9.7465e+00_rb,1.4357e+01_rb,1.8648e+01_rb,2.2353e+01_rb, &
     &2.4860e+01_rb,2.4146e+01_rb,3.7295e+01_rb/)
      kao(:, 1, 2,16) = (/ &
     &1.1399e-02_rb,3.0700e+00_rb,6.0865e+00_rb,9.0222e+00_rb,1.1819e+01_rb,1.4357e+01_rb, &
     &1.6335e+01_rb,1.6615e+01_rb,2.3637e+01_rb/)
      kao(:, 2, 2,16) = (/ &
     &1.1191e-02_rb,3.6855e+00_rb,7.3071e+00_rb,1.0831e+01_rb,1.4189e+01_rb,1.7243e+01_rb, &
     &1.9624e+01_rb,1.9971e+01_rb,2.8378e+01_rb/)
      kao(:, 3, 2,16) = (/ &
     &1.0967e-02_rb,4.3287e+00_rb,8.5840e+00_rb,1.2722e+01_rb,1.6670e+01_rb,2.0258e+01_rb, &
     &2.3062e+01_rb,2.3488e+01_rb,3.3340e+01_rb/)
      kao(:, 4, 2,16) = (/ &
     &1.0738e-02_rb,4.9902e+00_rb,9.8948e+00_rb,1.4671e+01_rb,1.9224e+01_rb,2.3370e+01_rb, &
     &2.6613e+01_rb,2.7035e+01_rb,3.8446e+01_rb/)
      kao(:, 5, 2,16) = (/ &
     &1.0463e-02_rb,5.6638e+00_rb,1.1230e+01_rb,1.6648e+01_rb,2.1823e+01_rb,2.6530e+01_rb, &
     &3.0222e+01_rb,3.0819e+01_rb,4.3644e+01_rb/)
      kao(:, 1, 3,16) = (/ &
     &2.6306e-02_rb,3.2694e+00_rb,6.5125e+00_rb,9.7138e+00_rb,1.2840e+01_rb,1.5828e+01_rb, &
     &1.8484e+01_rb,2.0009e+01_rb,2.5680e+01_rb/)
      kao(:, 2, 3,16) = (/ &
     &2.5884e-02_rb,3.9785e+00_rb,7.9277e+00_rb,1.1819e+01_rb,1.5631e+01_rb,1.9201e+01_rb, &
     &2.2505e+01_rb,2.4372e+01_rb,3.1262e+01_rb/)
      kao(:, 3, 3,16) = (/ &
     &2.5346e-02_rb,4.7273e+00_rb,9.4145e+00_rb,1.4042e+01_rb,1.8571e+01_rb,2.2894e+01_rb, &
     &2.6751e+01_rb,2.8974e+01_rb,3.7022e+01_rb/)
      kao(:, 4, 3,16) = (/ &
     &2.4784e-02_rb,5.5013e+00_rb,1.0958e+01_rb,1.6345e+01_rb,2.1611e+01_rb,2.6651e+01_rb, &
     &3.1147e+01_rb,3.3763e+01_rb,4.3221e+01_rb/)
      kao(:, 5, 3,16) = (/ &
     &2.4200e-02_rb,6.2942e+00_rb,1.2535e+01_rb,1.8701e+01_rb,2.4733e+01_rb,3.0502e+01_rb, &
     &3.5641e+01_rb,3.8658e+01_rb,4.9465e+01_rb/)
      kao(:, 1, 4,16) = (/ &
     &6.3011e-02_rb,3.3909e+00_rb,6.7686e+00_rb,1.0126e+01_rb,1.3449e+01_rb,1.6701e+01_rb, &
     &1.9789e+01_rb,2.2242e+01_rb,2.6898e+01_rb/)
      kao(:, 2, 4,16) = (/ &
     &6.1738e-02_rb,4.1953e+00_rb,8.3750e+00_rb,1.2529e+01_rb,1.6640e+01_rb,2.0669e+01_rb, &
     &2.4483e+01_rb,2.7534e+01_rb,3.3280e+01_rb/)
      kao(:, 3, 4,16) = (/ &
     &6.0385e-02_rb,5.0522e+00_rb,1.0085e+01_rb,1.5088e+01_rb,2.0041e+01_rb,2.4891e+01_rb, &
     &2.9498e+01_rb,3.3182e+01_rb,4.0082e+01_rb/)
      kao(:, 4, 4,16) = (/ &
     &5.8960e-02_rb,5.9475e+00_rb,1.1873e+01_rb,1.7764e+01_rb,2.3588e+01_rb,2.9299e+01_rb, &
     &3.4723e+01_rb,3.9079e+01_rb,4.7175e+01_rb/)
      kao(:, 5, 4,16) = (/ &
     &5.7513e-02_rb,6.8674e+00_rb,1.3704e+01_rb,2.0508e+01_rb,2.7243e+01_rb,3.3832e+01_rb, &
     &4.0104e+01_rb,4.5142e+01_rb,5.4486e+01_rb/)
      kao(:, 1, 5,16) = (/ &
     &1.5311e-01_rb,3.4647e+00_rb,6.9218e+00_rb,1.0370e+01_rb,1.3799e+01_rb,1.7194e+01_rb, &
     &2.0501e+01_rb,2.3473e+01_rb,2.7599e+01_rb/)
      kao(:, 2, 5,16) = (/ &
     &1.4808e-01_rb,4.3642e+00_rb,8.7211e+00_rb,1.3066e+01_rb,1.7386e+01_rb,2.1656e+01_rb, &
     &2.5836e+01_rb,2.9576e+01_rb,3.4772e+01_rb/)
      kao(:, 3, 5,16) = (/ &
     &1.4324e-01_rb,5.3326e+00_rb,1.0656e+01_rb,1.5962e+01_rb,2.1244e+01_rb,2.6472e+01_rb, &
     &3.1566e+01_rb,3.6158e+01_rb,4.2487e+01_rb/)
      kao(:, 4, 5,16) = (/ &
     &1.3867e-01_rb,6.3343e+00_rb,1.2694e+01_rb,1.9018e+01_rb,2.5310e+01_rb,3.1544e+01_rb, &
     &3.7609e+01_rb,4.3076e+01_rb,5.0619e+01_rb/)
      kao(:, 5, 5,16) = (/ &
     &1.3442e-01_rb,7.4110e+00_rb,1.4809e+01_rb,2.2187e+01_rb,2.9523e+01_rb,3.6788e+01_rb, &
     &4.3874e+01_rb,5.0262e+01_rb,5.9046e+01_rb/)
      kao(:, 1, 6,16) = (/ &
     &3.3965e-01_rb,3.4550e+00_rb,6.9058e+00_rb,1.0352e+01_rb,1.3789e+01_rb,1.7206e+01_rb, &
     &2.0578e+01_rb,2.3766e+01_rb,2.7578e+01_rb/)
      kao(:, 2, 6,16) = (/ &
     &3.2990e-01_rb,4.4451e+00_rb,8.8565e+00_rb,1.3318e+01_rb,1.7740e+01_rb,2.2139e+01_rb, &
     &2.6477e+01_rb,3.0582e+01_rb,3.5479e+01_rb/)
      kao(:, 3, 6,16) = (/ &
     &3.2021e-01_rb,5.5240e+00_rb,1.1042e+01_rb,1.6554e+01_rb,2.2048e+01_rb,2.7511e+01_rb, &
     &3.2908e+01_rb,3.8025e+01_rb,4.4095e+01_rb/)
      kao(:, 4, 6,16) = (/ &
     &3.1092e-01_rb,6.6752e+00_rb,1.3341e+01_rb,2.0002e+01_rb,2.6641e+01_rb,3.3245e+01_rb, &
     &3.9761e+01_rb,4.5941e+01_rb,5.3282e+01_rb/)
      kao(:, 5, 6,16) = (/ &
     &3.0227e-01_rb,7.8771e+00_rb,1.5743e+01_rb,2.3602e+01_rb,3.1434e+01_rb,3.9236e+01_rb, &
     &4.6923e+01_rb,5.4228e+01_rb,6.2865e+01_rb/)
      kao(:, 1, 7,16) = (/ &
     &7.9672e-01_rb,3.3862e+00_rb,6.7703e+00_rb,1.0151e+01_rb,1.3528e+01_rb,1.6895e+01_rb, &
     &2.0243e+01_rb,2.3498e+01_rb,2.7056e+01_rb/)
      kao(:, 2, 7,16) = (/ &
     &7.7399e-01_rb,4.4592e+00_rb,8.9171e+00_rb,1.3369e+01_rb,1.7816e+01_rb,2.2252e+01_rb, &
     &2.6655e+01_rb,3.0954e+01_rb,3.5514e+01_rb/)
      kao(:, 3, 7,16) = (/ &
     &7.5170e-01_rb,5.6480e+00_rb,1.1293e+01_rb,1.6933e+01_rb,2.2566e+01_rb,2.8186e+01_rb, &
     &3.3769e+01_rb,3.9205e+01_rb,4.5131e+01_rb/)
      kao(:, 4, 7,16) = (/ &
     &7.3106e-01_rb,6.9294e+00_rb,1.3857e+01_rb,2.0779e+01_rb,2.7691e+01_rb,3.4583e+01_rb, &
     &4.1435e+01_rb,4.8102e+01_rb,5.5380e+01_rb/)
      kao(:, 5, 7,16) = (/ &
     &7.1147e-01_rb,8.2841e+00_rb,1.6563e+01_rb,2.4830e+01_rb,3.3089e+01_rb,4.1334e+01_rb, &
     &4.9522e+01_rb,5.7496e+01_rb,6.6175e+01_rb/)
      kao(:, 1, 8,16) = (/ &
     &2.2037e+00_rb,3.2722e+00_rb,6.5418e+00_rb,9.8122e+00_rb,1.3082e+01_rb,1.6344e+01_rb, &
     &1.9599e+01_rb,2.2817e+01_rb,2.6163e+01_rb/)
      kao(:, 2, 8,16) = (/ &
     &2.1416e+00_rb,4.4204e+00_rb,8.8385e+00_rb,1.3255e+01_rb,1.7675e+01_rb,2.2079e+01_rb, &
     &2.6477e+01_rb,3.0833e+01_rb,3.5348e+01_rb/)
      kao(:, 3, 8,16) = (/ &
     &2.0842e+00_rb,5.7151e+00_rb,1.1428e+01_rb,1.7137e+01_rb,2.2842e+01_rb,2.8548e+01_rb, &
     &3.4230e+01_rb,3.9856e+01_rb,4.5681e+01_rb/)
      kao(:, 4, 8,16) = (/ &
     &2.0305e+00_rb,7.1274e+00_rb,1.4255e+01_rb,2.1373e+01_rb,2.8489e+01_rb,3.5598e+01_rb, &
     &4.2693e+01_rb,4.9704e+01_rb,5.6975e+01_rb/)
      kao(:, 5, 8,16) = (/ &
     &1.9781e+00_rb,8.6369e+00_rb,1.7269e+01_rb,2.5895e+01_rb,3.4524e+01_rb,4.3140e+01_rb, &
     &5.1725e+01_rb,6.0225e+01_rb,6.9043e+01_rb/)
      kao(:, 1, 9,16) = (/ &
     &1.1509e+01_rb,1.0070e+01_rb,8.6316e+00_rb,9.3146e+00_rb,1.2418e+01_rb,1.5525e+01_rb, &
     &1.8622e+01_rb,2.1717e+01_rb,2.4834e+01_rb/)
      kao(:, 2, 9,16) = (/ &
     &1.1216e+01_rb,9.8124e+00_rb,8.9628e+00_rb,1.2947e+01_rb,1.7258e+01_rb,2.1566e+01_rb, &
     &2.5882e+01_rb,3.0171e+01_rb,3.4512e+01_rb/)
      kao(:, 3, 9,16) = (/ &
     &1.0936e+01_rb,9.5703e+00_rb,1.1414e+01_rb,1.7115e+01_rb,2.2811e+01_rb,2.8515e+01_rb, &
     &3.4212e+01_rb,3.9896e+01_rb,4.5613e+01_rb/)
      kao(:, 4, 9,16) = (/ &
     &1.0665e+01_rb,9.3327e+00_rb,1.4497e+01_rb,2.1741e+01_rb,2.8981e+01_rb,3.6218e+01_rb, &
     &4.3300e+01_rb,5.0677e+01_rb,5.7950e+01_rb/)
      kao(:, 5, 9,16) = (/ &
     &1.0398e+01_rb,9.5016e+00_rb,1.7820e+01_rb,2.6724e+01_rb,3.5623e+01_rb,4.4518e+01_rb, &
     &5.3410e+01_rb,6.2271e+01_rb,7.1226e+01_rb/)
      kao(:, 1,10,16) = (/ &
     &6.7305e+01_rb,5.8689e+01_rb,5.0477e+01_rb,4.2062e+01_rb,3.3650e+01_rb,2.5240e+01_rb, &
     &1.8527e+01_rb,2.0871e+01_rb,2.3854e+01_rb/)
      kao(:, 2,10,16) = (/ &
     &6.5751e+01_rb,5.7530e+01_rb,4.9313e+01_rb,4.1093e+01_rb,3.2876e+01_rb,2.4802e+01_rb, &
     &2.5524e+01_rb,2.9774e+01_rb,3.4038e+01_rb/)
      kao(:, 3,10,16) = (/ &
     &6.4198e+01_rb,5.6172e+01_rb,4.8149e+01_rb,4.0130e+01_rb,3.2100e+01_rb,2.8998e+01_rb, &
     &3.4448e+01_rb,4.0179e+01_rb,4.5916e+01_rb/)
      kao(:, 4,10,16) = (/ &
     &6.2664e+01_rb,5.4833e+01_rb,4.6832e+01_rb,3.9166e+01_rb,3.2607e+01_rb,3.7058e+01_rb, &
     &4.4476e+01_rb,5.1875e+01_rb,5.9284e+01_rb/)
      kao(:, 5,10,16) = (/ &
     &6.1101e+01_rb,5.3467e+01_rb,4.5664e+01_rb,3.8190e+01_rb,3.7365e+01_rb,4.6149e+01_rb, &
     &5.5367e+01_rb,6.4570e+01_rb,7.3787e+01_rb/)
      kao(:, 1,11,16) = (/ &
     &1.3576e+02_rb,1.1879e+02_rb,1.0182e+02_rb,8.4849e+01_rb,6.7875e+01_rb,5.0907e+01_rb, &
     &3.3938e+01_rb,2.3799e+01_rb,9.0162e+00_rb/)
      kao(:, 2,11,16) = (/ &
     &1.3280e+02_rb,1.1619e+02_rb,9.9587e+01_rb,8.3000e+01_rb,6.6392e+01_rb,4.9800e+01_rb, &
     &3.3906e+01_rb,3.3859e+01_rb,3.7265e+01_rb/)
      kao(:, 3,11,16) = (/ &
     &1.2969e+02_rb,1.1350e+02_rb,9.7289e+01_rb,8.1076e+01_rb,6.4844e+01_rb,4.8642e+01_rb, &
     &3.9970e+01_rb,4.5587e+01_rb,5.2094e+01_rb/)
      kao(:, 4,11,16) = (/ &
     &1.2653e+02_rb,1.1071e+02_rb,9.4885e+01_rb,7.9077e+01_rb,6.3265e+01_rb,4.8850e+01_rb, &
     &5.0310e+01_rb,5.8678e+01_rb,6.7071e+01_rb/)
      kao(:, 5,11,16) = (/ &
     &1.2325e+02_rb,1.0784e+02_rb,9.2433e+01_rb,7.7029e+01_rb,6.1622e+01_rb,5.4553e+01_rb, &
     &6.2511e+01_rb,7.2895e+01_rb,8.3291e+01_rb/)
      kao(:, 1,12,16) = (/ &
     &2.0298e+02_rb,1.7762e+02_rb,1.5224e+02_rb,1.2687e+02_rb,1.0147e+02_rb,7.6126e+01_rb, &
     &5.0748e+01_rb,2.9045e+01_rb,9.6314e-05_rb/)
      kao(:, 2,12,16) = (/ &
     &1.9850e+02_rb,1.7368e+02_rb,1.4888e+02_rb,1.2405e+02_rb,9.9250e+01_rb,7.4435e+01_rb, &
     &4.9625e+01_rb,3.8234e+01_rb,2.3705e+01_rb/)
      kao(:, 3,12,16) = (/ &
     &1.9369e+02_rb,1.6949e+02_rb,1.4528e+02_rb,1.2103e+02_rb,9.6845e+01_rb,7.2640e+01_rb, &
     &5.1319e+01_rb,5.1242e+01_rb,5.0529e+01_rb/)
      kao(:, 4,12,16) = (/ &
     &1.8865e+02_rb,1.6508e+02_rb,1.4148e+02_rb,1.1791e+02_rb,9.4323e+01_rb,7.0746e+01_rb, &
     &5.9155e+01_rb,6.5776e+01_rb,7.5148e+01_rb/)
      kao(:, 5,12,16) = (/ &
     &1.8345e+02_rb,1.6051e+02_rb,1.3757e+02_rb,1.1465e+02_rb,9.1715e+01_rb,7.1673e+01_rb, &
     &7.0340e+01_rb,8.1440e+01_rb,9.3046e+01_rb/)
      kao(:, 1,13,16) = (/ &
     &2.4179e+02_rb,2.1156e+02_rb,1.8133e+02_rb,1.5111e+02_rb,1.2089e+02_rb,9.0670e+01_rb, &
     &6.0447e+01_rb,3.4181e+01_rb,7.5740e-05_rb/)
      kao(:, 2,13,16) = (/ &
     &2.3602e+02_rb,2.0653e+02_rb,1.7702e+02_rb,1.4755e+02_rb,1.1802e+02_rb,8.8512e+01_rb, &
     &5.9018e+01_rb,4.3484e+01_rb,2.5520e+01_rb/)
      kao(:, 3,13,16) = (/ &
     &2.2984e+02_rb,2.0112e+02_rb,1.7238e+02_rb,1.4366e+02_rb,1.1493e+02_rb,8.6216e+01_rb, &
     &6.0956e+01_rb,5.6909e+01_rb,5.1259e+01_rb/)
      kao(:, 4,13,16) = (/ &
     &2.2335e+02_rb,1.9547e+02_rb,1.6749e+02_rb,1.3960e+02_rb,1.1169e+02_rb,8.3756e+01_rb, &
     &6.8596e+01_rb,7.2790e+01_rb,7.7355e+01_rb/)
      kao(:, 5,13,16) = (/ &
     &2.1661e+02_rb,1.8954e+02_rb,1.6247e+02_rb,1.3539e+02_rb,1.0832e+02_rb,8.5058e+01_rb, &
     &7.9741e+01_rb,8.9915e+01_rb,1.0226e+02_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &2.3382e-03_rb,2.8918e-03_rb,3.5228e-03_rb,4.2513e-03_rb,5.0992e-03_rb/)
      kbo(:,14, 1) = (/ &
     &2.0993e-03_rb,2.6087e-03_rb,3.2027e-03_rb,3.9155e-03_rb,4.7075e-03_rb/)
      kbo(:,15, 1) = (/ &
     &1.9068e-03_rb,2.4224e-03_rb,2.9993e-03_rb,3.6366e-03_rb,4.4490e-03_rb/)
      kbo(:,16, 1) = (/ &
     &1.7630e-03_rb,2.2267e-03_rb,2.7514e-03_rb,3.4114e-03_rb,4.1614e-03_rb/)
      kbo(:,17, 1) = (/ &
     &1.6519e-03_rb,2.0886e-03_rb,2.6010e-03_rb,3.2457e-03_rb,3.9066e-03_rb/)
      kbo(:,18, 1) = (/ &
     &1.5561e-03_rb,1.9733e-03_rb,2.5429e-03_rb,3.1325e-03_rb,3.6764e-03_rb/)
      kbo(:,19, 1) = (/ &
     &1.4778e-03_rb,1.9477e-03_rb,2.4734e-03_rb,2.9725e-03_rb,3.4413e-03_rb/)
      kbo(:,20, 1) = (/ &
     &1.3764e-03_rb,1.8206e-03_rb,2.2794e-03_rb,2.6918e-03_rb,3.0807e-03_rb/)
      kbo(:,21, 1) = (/ &
     &1.2547e-03_rb,1.6614e-03_rb,2.0387e-03_rb,2.3912e-03_rb,2.7222e-03_rb/)
      kbo(:,22, 1) = (/ &
     &1.1692e-03_rb,1.5034e-03_rb,1.8117e-03_rb,2.1136e-03_rb,2.3959e-03_rb/)
      kbo(:,23, 1) = (/ &
     &1.0852e-03_rb,1.3578e-03_rb,1.6152e-03_rb,1.8580e-03_rb,2.0983e-03_rb/)
      kbo(:,24, 1) = (/ &
     &9.9196e-04_rb,1.2216e-03_rb,1.4332e-03_rb,1.6336e-03_rb,1.8396e-03_rb/)
      kbo(:,25, 1) = (/ &
     &9.0621e-04_rb,1.0934e-03_rb,1.2579e-03_rb,1.4326e-03_rb,1.5972e-03_rb/)
      kbo(:,26, 1) = (/ &
     &8.1196e-04_rb,9.6544e-04_rb,1.0989e-03_rb,1.2439e-03_rb,1.3772e-03_rb/)
      kbo(:,27, 1) = (/ &
     &7.2402e-04_rb,8.4382e-04_rb,9.5455e-04_rb,1.0692e-03_rb,1.1790e-03_rb/)
      kbo(:,28, 1) = (/ &
     &6.3816e-04_rb,7.3173e-04_rb,8.2333e-04_rb,9.1530e-04_rb,1.0008e-03_rb/)
      kbo(:,29, 1) = (/ &
     &5.5611e-04_rb,6.2561e-04_rb,7.0457e-04_rb,7.7239e-04_rb,8.4074e-04_rb/)
      kbo(:,30, 1) = (/ &
     &4.7758e-04_rb,5.3585e-04_rb,5.9602e-04_rb,6.4848e-04_rb,7.0299e-04_rb/)
      kbo(:,31, 1) = (/ &
     &4.0233e-04_rb,4.5494e-04_rb,5.0095e-04_rb,5.4165e-04_rb,5.8359e-04_rb/)
      kbo(:,32, 1) = (/ &
     &3.4147e-04_rb,3.8327e-04_rb,4.2049e-04_rb,4.5512e-04_rb,4.8721e-04_rb/)
      kbo(:,33, 1) = (/ &
     &2.9210e-04_rb,3.2465e-04_rb,3.5276e-04_rb,3.8153e-04_rb,4.1121e-04_rb/)
      kbo(:,34, 1) = (/ &
     &2.4930e-04_rb,2.7584e-04_rb,2.9920e-04_rb,3.2247e-04_rb,3.4759e-04_rb/)
      kbo(:,35, 1) = (/ &
     &2.1208e-04_rb,2.3419e-04_rb,2.5253e-04_rb,2.7252e-04_rb,2.9279e-04_rb/)
      kbo(:,36, 1) = (/ &
     &1.7811e-04_rb,1.9459e-04_rb,2.1195e-04_rb,2.2908e-04_rb,2.4619e-04_rb/)
      kbo(:,37, 1) = (/ &
     &1.4252e-04_rb,1.5802e-04_rb,1.7124e-04_rb,1.8557e-04_rb,1.9967e-04_rb/)
      kbo(:,38, 1) = (/ &
     &1.1316e-04_rb,1.2691e-04_rb,1.3875e-04_rb,1.4940e-04_rb,1.6149e-04_rb/)
      kbo(:,39, 1) = (/ &
     &8.9734e-05_rb,1.0111e-04_rb,1.1110e-04_rb,1.2104e-04_rb,1.3018e-04_rb/)
      kbo(:,40, 1) = (/ &
     &6.9862e-05_rb,7.9338e-05_rb,8.7351e-05_rb,9.5361e-05_rb,1.0350e-04_rb/)
      kbo(:,41, 1) = (/ &
     &5.3903e-05_rb,6.1377e-05_rb,6.8596e-05_rb,7.4910e-05_rb,8.1525e-05_rb/)
      kbo(:,42, 1) = (/ &
     &4.1353e-05_rb,4.7327e-05_rb,5.3464e-05_rb,5.8873e-05_rb,6.3813e-05_rb/)
      kbo(:,43, 1) = (/ &
     &3.1177e-05_rb,3.6021e-05_rb,4.0748e-05_rb,4.5236e-05_rb,4.9388e-05_rb/)
      kbo(:,44, 1) = (/ &
     &2.3155e-05_rb,2.7105e-05_rb,3.0750e-05_rb,3.4487e-05_rb,3.7902e-05_rb/)
      kbo(:,45, 1) = (/ &
     &1.6939e-05_rb,2.0019e-05_rb,2.3152e-05_rb,2.6034e-05_rb,2.8825e-05_rb/)
      kbo(:,46, 1) = (/ &
     &1.2188e-05_rb,1.4774e-05_rb,1.7220e-05_rb,1.9590e-05_rb,2.1820e-05_rb/)
      kbo(:,47, 1) = (/ &
     &8.5598e-06_rb,1.0765e-05_rb,1.2719e-05_rb,1.4628e-05_rb,1.6428e-05_rb/)
      kbo(:,48, 1) = (/ &
     &5.9262e-06_rb,7.6349e-06_rb,9.2601e-06_rb,1.0811e-05_rb,1.2306e-05_rb/)
      kbo(:,49, 1) = (/ &
     &3.9332e-06_rb,5.2612e-06_rb,6.6345e-06_rb,7.8390e-06_rb,9.0160e-06_rb/)
      kbo(:,50, 1) = (/ &
     &2.6915e-06_rb,3.7062e-06_rb,4.7528e-06_rb,5.7591e-06_rb,6.6828e-06_rb/)
      kbo(:,51, 1) = (/ &
     &1.9031e-06_rb,2.5633e-06_rb,3.3817e-06_rb,4.2212e-06_rb,4.9973e-06_rb/)
      kbo(:,52, 1) = (/ &
     &1.3022e-06_rb,1.7876e-06_rb,2.4129e-06_rb,3.0582e-06_rb,3.6770e-06_rb/)
      kbo(:,53, 1) = (/ &
     &8.8514e-07_rb,1.2531e-06_rb,1.6651e-06_rb,2.1701e-06_rb,2.6821e-06_rb/)
      kbo(:,54, 1) = (/ &
     &6.3174e-07_rb,9.0490e-07_rb,1.2164e-06_rb,1.6231e-06_rb,2.0201e-06_rb/)
      kbo(:,55, 1) = (/ &
     &4.6092e-07_rb,6.6386e-07_rb,9.1477e-07_rb,1.2278e-06_rb,1.5504e-06_rb/)
      kbo(:,56, 1) = (/ &
     &3.3312e-07_rb,4.8579e-07_rb,6.8774e-07_rb,9.1495e-07_rb,1.1856e-06_rb/)
      kbo(:,57, 1) = (/ &
     &2.4004e-07_rb,3.5567e-07_rb,5.1418e-07_rb,6.8280e-07_rb,9.0960e-07_rb/)
      kbo(:,58, 1) = (/ &
     &1.7424e-07_rb,2.6290e-07_rb,3.8304e-07_rb,5.1880e-07_rb,6.9948e-07_rb/)
      kbo(:,59, 1) = (/ &
     &1.4545e-07_rb,2.2249e-07_rb,3.3056e-07_rb,4.4487e-07_rb,6.0780e-07_rb/)
      kbo(:,13, 2) = (/ &
     &1.0289e-02_rb,1.3009e-02_rb,1.5474e-02_rb,1.7839e-02_rb,2.0097e-02_rb/)
      kbo(:,14, 2) = (/ &
     &9.5725e-03_rb,1.1818e-02_rb,1.3904e-02_rb,1.5918e-02_rb,1.7958e-02_rb/)
      kbo(:,15, 2) = (/ &
     &8.9675e-03_rb,1.0842e-02_rb,1.2642e-02_rb,1.4399e-02_rb,1.6048e-02_rb/)
      kbo(:,16, 2) = (/ &
     &8.1677e-03_rb,9.7202e-03_rb,1.1280e-02_rb,1.2681e-02_rb,1.4168e-02_rb/)
      kbo(:,17, 2) = (/ &
     &7.4338e-03_rb,8.8205e-03_rb,1.0076e-02_rb,1.1262e-02_rb,1.2428e-02_rb/)
      kbo(:,18, 2) = (/ &
     &6.8634e-03_rb,8.0133e-03_rb,8.9513e-03_rb,9.9366e-03_rb,1.1014e-02_rb/)
      kbo(:,19, 2) = (/ &
     &6.3552e-03_rb,7.2047e-03_rb,8.0129e-03_rb,8.8966e-03_rb,9.8381e-03_rb/)
      kbo(:,20, 2) = (/ &
     &5.6397e-03_rb,6.2755e-03_rb,6.9887e-03_rb,7.7846e-03_rb,8.5391e-03_rb/)
      kbo(:,21, 2) = (/ &
     &4.8925e-03_rb,5.4088e-03_rb,6.0399e-03_rb,6.7004e-03_rb,7.3289e-03_rb/)
      kbo(:,22, 2) = (/ &
     &4.2004e-03_rb,4.6864e-03_rb,5.2247e-03_rb,5.7527e-03_rb,6.2510e-03_rb/)
      kbo(:,23, 2) = (/ &
     &3.5982e-03_rb,4.0432e-03_rb,4.4892e-03_rb,4.9192e-03_rb,5.3260e-03_rb/)
      kbo(:,24, 2) = (/ &
     &3.1341e-03_rb,3.4942e-03_rb,3.8506e-03_rb,4.1837e-03_rb,4.5083e-03_rb/)
      kbo(:,25, 2) = (/ &
     &2.7078e-03_rb,2.9853e-03_rb,3.2644e-03_rb,3.5527e-03_rb,3.8354e-03_rb/)
      kbo(:,26, 2) = (/ &
     &2.3166e-03_rb,2.5349e-03_rb,2.7760e-03_rb,3.0099e-03_rb,3.2441e-03_rb/)
      kbo(:,27, 2) = (/ &
     &1.9581e-03_rb,2.1476e-03_rb,2.3442e-03_rb,2.5333e-03_rb,2.7284e-03_rb/)
      kbo(:,28, 2) = (/ &
     &1.6566e-03_rb,1.8064e-03_rb,1.9708e-03_rb,2.1221e-03_rb,2.2915e-03_rb/)
      kbo(:,29, 2) = (/ &
     &1.3865e-03_rb,1.5225e-03_rb,1.6510e-03_rb,1.7817e-03_rb,1.9308e-03_rb/)
      kbo(:,30, 2) = (/ &
     &1.1671e-03_rb,1.2777e-03_rb,1.3825e-03_rb,1.4960e-03_rb,1.6203e-03_rb/)
      kbo(:,31, 2) = (/ &
     &9.8586e-04_rb,1.0724e-03_rb,1.1604e-03_rb,1.2563e-03_rb,1.3588e-03_rb/)
      kbo(:,32, 2) = (/ &
     &8.3378e-04_rb,9.0396e-04_rb,9.7429e-04_rb,1.0568e-03_rb,1.1470e-03_rb/)
      kbo(:,33, 2) = (/ &
     &7.0369e-04_rb,7.5618e-04_rb,8.2196e-04_rb,8.9196e-04_rb,9.6493e-04_rb/)
      kbo(:,34, 2) = (/ &
     &5.9334e-04_rb,6.3847e-04_rb,6.9888e-04_rb,7.5675e-04_rb,8.1741e-04_rb/)
      kbo(:,35, 2) = (/ &
     &4.9419e-04_rb,5.3430e-04_rb,5.8625e-04_rb,6.3690e-04_rb,6.8685e-04_rb/)
      kbo(:,36, 2) = (/ &
     &4.1196e-04_rb,4.5151e-04_rb,4.9203e-04_rb,5.3162e-04_rb,5.7140e-04_rb/)
      kbo(:,37, 2) = (/ &
     &3.3813e-04_rb,3.6860e-04_rb,4.0277e-04_rb,4.3459e-04_rb,4.6722e-04_rb/)
      kbo(:,38, 2) = (/ &
     &2.7817e-04_rb,3.0151e-04_rb,3.2943e-04_rb,3.5593e-04_rb,3.8227e-04_rb/)
      kbo(:,39, 2) = (/ &
     &2.2867e-04_rb,2.4762e-04_rb,2.6968e-04_rb,2.9008e-04_rb,3.1209e-04_rb/)
      kbo(:,40, 2) = (/ &
     &1.8342e-04_rb,1.9773e-04_rb,2.1631e-04_rb,2.3254e-04_rb,2.4976e-04_rb/)
      kbo(:,41, 2) = (/ &
     &1.4676e-04_rb,1.5810e-04_rb,1.7216e-04_rb,1.8594e-04_rb,1.9960e-04_rb/)
      kbo(:,42, 2) = (/ &
     &1.1746e-04_rb,1.2665e-04_rb,1.3672e-04_rb,1.4836e-04_rb,1.5881e-04_rb/)
      kbo(:,43, 2) = (/ &
     &9.3086e-05_rb,1.0013e-04_rb,1.0780e-04_rb,1.1711e-04_rb,1.2583e-04_rb/)
      kbo(:,44, 2) = (/ &
     &7.3232e-05_rb,7.8805e-05_rb,8.4882e-05_rb,9.2011e-05_rb,9.9424e-05_rb/)
      kbo(:,45, 2) = (/ &
     &5.7257e-05_rb,6.2283e-05_rb,6.6827e-05_rb,7.2104e-05_rb,7.8039e-05_rb/)
      kbo(:,46, 2) = (/ &
     &4.5024e-05_rb,4.8935e-05_rb,5.2600e-05_rb,5.6600e-05_rb,6.1280e-05_rb/)
      kbo(:,47, 2) = (/ &
     &3.5357e-05_rb,3.8098e-05_rb,4.1339e-05_rb,4.4405e-05_rb,4.7940e-05_rb/)
      kbo(:,48, 2) = (/ &
     &2.7815e-05_rb,2.9967e-05_rb,3.2437e-05_rb,3.4873e-05_rb,3.7402e-05_rb/)
      kbo(:,49, 2) = (/ &
     &2.1939e-05_rb,2.3492e-05_rb,2.5305e-05_rb,2.7332e-05_rb,2.9323e-05_rb/)
      kbo(:,50, 2) = (/ &
     &1.7237e-05_rb,1.8528e-05_rb,1.9953e-05_rb,2.1525e-05_rb,2.3159e-05_rb/)
      kbo(:,51, 2) = (/ &
     &1.3436e-05_rb,1.4671e-05_rb,1.5802e-05_rb,1.6957e-05_rb,1.8296e-05_rb/)
      kbo(:,52, 2) = (/ &
     &1.0450e-05_rb,1.1538e-05_rb,1.2438e-05_rb,1.3421e-05_rb,1.4433e-05_rb/)
      kbo(:,53, 2) = (/ &
     &8.1106e-06_rb,9.0282e-06_rb,9.8120e-06_rb,1.0593e-05_rb,1.1339e-05_rb/)
      kbo(:,54, 2) = (/ &
     &6.3166e-06_rb,7.0785e-06_rb,7.7924e-06_rb,8.4016e-06_rb,8.9926e-06_rb/)
      kbo(:,55, 2) = (/ &
     &4.9361e-06_rb,5.5830e-06_rb,6.1858e-06_rb,6.7021e-06_rb,7.1744e-06_rb/)
      kbo(:,56, 2) = (/ &
     &3.8472e-06_rb,4.3596e-06_rb,4.8727e-06_rb,5.3362e-06_rb,5.7261e-06_rb/)
      kbo(:,57, 2) = (/ &
     &2.9833e-06_rb,3.4045e-06_rb,3.8019e-06_rb,4.2184e-06_rb,4.5533e-06_rb/)
      kbo(:,58, 2) = (/ &
     &2.2932e-06_rb,2.6742e-06_rb,3.0024e-06_rb,3.3291e-06_rb,3.6047e-06_rb/)
      kbo(:,59, 2) = (/ &
     &1.8323e-06_rb,2.1714e-06_rb,2.4292e-06_rb,2.6999e-06_rb,2.9218e-06_rb/)
      kbo(:,13, 3) = (/ &
     &3.0891e-02_rb,3.3118e-02_rb,3.6308e-02_rb,3.9588e-02_rb,4.3089e-02_rb/)
      kbo(:,14, 3) = (/ &
     &2.6329e-02_rb,2.8653e-02_rb,3.1325e-02_rb,3.4211e-02_rb,3.7000e-02_rb/)
      kbo(:,15, 3) = (/ &
     &2.2620e-02_rb,2.4702e-02_rb,2.7101e-02_rb,2.9342e-02_rb,3.1645e-02_rb/)
      kbo(:,16, 3) = (/ &
     &1.9521e-02_rb,2.1493e-02_rb,2.3534e-02_rb,2.5338e-02_rb,2.7120e-02_rb/)
      kbo(:,17, 3) = (/ &
     &1.7199e-02_rb,1.8861e-02_rb,2.0377e-02_rb,2.1918e-02_rb,2.3545e-02_rb/)
      kbo(:,18, 3) = (/ &
     &1.5181e-02_rb,1.6445e-02_rb,1.7796e-02_rb,1.9171e-02_rb,2.0759e-02_rb/)
      kbo(:,19, 3) = (/ &
     &1.3401e-02_rb,1.4529e-02_rb,1.5713e-02_rb,1.7031e-02_rb,1.8531e-02_rb/)
      kbo(:,20, 3) = (/ &
     &1.1566e-02_rb,1.2595e-02_rb,1.3616e-02_rb,1.4797e-02_rb,1.6195e-02_rb/)
      kbo(:,21, 3) = (/ &
     &9.9790e-03_rb,1.0856e-02_rb,1.1739e-02_rb,1.2840e-02_rb,1.4039e-02_rb/)
      kbo(:,22, 3) = (/ &
     &8.5571e-03_rb,9.3189e-03_rb,1.0148e-02_rb,1.1149e-02_rb,1.2271e-02_rb/)
      kbo(:,23, 3) = (/ &
     &7.3578e-03_rb,7.9960e-03_rb,8.8066e-03_rb,9.6774e-03_rb,1.0719e-02_rb/)
      kbo(:,24, 3) = (/ &
     &6.2710e-03_rb,6.8510e-03_rb,7.5829e-03_rb,8.4285e-03_rb,9.3410e-03_rb/)
      kbo(:,25, 3) = (/ &
     &5.3189e-03_rb,5.9223e-03_rb,6.5735e-03_rb,7.3163e-03_rb,8.1167e-03_rb/)
      kbo(:,26, 3) = (/ &
     &4.5763e-03_rb,5.1227e-03_rb,5.7207e-03_rb,6.3561e-03_rb,7.0425e-03_rb/)
      kbo(:,27, 3) = (/ &
     &3.9329e-03_rb,4.4226e-03_rb,4.9281e-03_rb,5.4554e-03_rb,6.0474e-03_rb/)
      kbo(:,28, 3) = (/ &
     &3.3818e-03_rb,3.7926e-03_rb,4.2066e-03_rb,4.6531e-03_rb,5.1585e-03_rb/)
      kbo(:,29, 3) = (/ &
     &2.9035e-03_rb,3.2271e-03_rb,3.5855e-03_rb,3.9705e-03_rb,4.3799e-03_rb/)
      kbo(:,30, 3) = (/ &
     &2.4837e-03_rb,2.7647e-03_rb,3.0488e-03_rb,3.3703e-03_rb,3.7029e-03_rb/)
      kbo(:,31, 3) = (/ &
     &2.0927e-03_rb,2.3283e-03_rb,2.5712e-03_rb,2.8545e-03_rb,3.1157e-03_rb/)
      kbo(:,32, 3) = (/ &
     &1.7660e-03_rb,1.9568e-03_rb,2.1676e-03_rb,2.3923e-03_rb,2.6159e-03_rb/)
      kbo(:,33, 3) = (/ &
     &1.4871e-03_rb,1.6499e-03_rb,1.8213e-03_rb,2.0066e-03_rb,2.1979e-03_rb/)
      kbo(:,34, 3) = (/ &
     &1.2670e-03_rb,1.4012e-03_rb,1.5485e-03_rb,1.7073e-03_rb,1.8735e-03_rb/)
      kbo(:,35, 3) = (/ &
     &1.0672e-03_rb,1.1813e-03_rb,1.3131e-03_rb,1.4449e-03_rb,1.5924e-03_rb/)
      kbo(:,36, 3) = (/ &
     &8.9185e-04_rb,9.8733e-04_rb,1.0955e-03_rb,1.2168e-03_rb,1.3441e-03_rb/)
      kbo(:,37, 3) = (/ &
     &7.2820e-04_rb,8.0537e-04_rb,8.9528e-04_rb,9.9910e-04_rb,1.1050e-03_rb/)
      kbo(:,38, 3) = (/ &
     &5.9186e-04_rb,6.5591e-04_rb,7.2922e-04_rb,8.1759e-04_rb,9.0441e-04_rb/)
      kbo(:,39, 3) = (/ &
     &4.8327e-04_rb,5.3477e-04_rb,5.9527e-04_rb,6.6884e-04_rb,7.4026e-04_rb/)
      kbo(:,40, 3) = (/ &
     &3.8671e-04_rb,4.2869e-04_rb,4.7628e-04_rb,5.3686e-04_rb,5.9502e-04_rb/)
      kbo(:,41, 3) = (/ &
     &3.0862e-04_rb,3.4166e-04_rb,3.8000e-04_rb,4.2745e-04_rb,4.7580e-04_rb/)
      kbo(:,42, 3) = (/ &
     &2.4577e-04_rb,2.7168e-04_rb,3.0221e-04_rb,3.3942e-04_rb,3.8025e-04_rb/)
      kbo(:,43, 3) = (/ &
     &1.9416e-04_rb,2.1479e-04_rb,2.3878e-04_rb,2.6702e-04_rb,3.0069e-04_rb/)
      kbo(:,44, 3) = (/ &
     &1.5374e-04_rb,1.6929e-04_rb,1.8776e-04_rb,2.0950e-04_rb,2.3556e-04_rb/)
      kbo(:,45, 3) = (/ &
     &1.2163e-04_rb,1.3301e-04_rb,1.4695e-04_rb,1.6433e-04_rb,1.8435e-04_rb/)
      kbo(:,46, 3) = (/ &
     &9.4928e-05_rb,1.0426e-04_rb,1.1546e-04_rb,1.2841e-04_rb,1.4410e-04_rb/)
      kbo(:,47, 3) = (/ &
     &7.3995e-05_rb,8.2130e-05_rb,9.0212e-05_rb,1.0006e-04_rb,1.1186e-04_rb/)
      kbo(:,48, 3) = (/ &
     &5.7893e-05_rb,6.3962e-05_rb,7.0352e-05_rb,7.8032e-05_rb,8.7072e-05_rb/)
      kbo(:,49, 3) = (/ &
     &4.5051e-05_rb,4.9908e-05_rb,5.5306e-05_rb,6.0837e-05_rb,6.7927e-05_rb/)
      kbo(:,50, 3) = (/ &
     &3.5303e-05_rb,3.9326e-05_rb,4.3324e-05_rb,4.7812e-05_rb,5.3164e-05_rb/)
      kbo(:,51, 3) = (/ &
     &2.7771e-05_rb,3.0856e-05_rb,3.4054e-05_rb,3.7804e-05_rb,4.1758e-05_rb/)
      kbo(:,52, 3) = (/ &
     &2.1938e-05_rb,2.4227e-05_rb,2.6919e-05_rb,2.9733e-05_rb,3.2981e-05_rb/)
      kbo(:,53, 3) = (/ &
     &1.7187e-05_rb,1.9050e-05_rb,2.1224e-05_rb,2.3374e-05_rb,2.5986e-05_rb/)
      kbo(:,54, 3) = (/ &
     &1.3551e-05_rb,1.5084e-05_rb,1.6671e-05_rb,1.8493e-05_rb,2.0639e-05_rb/)
      kbo(:,55, 3) = (/ &
     &1.0736e-05_rb,1.1998e-05_rb,1.3230e-05_rb,1.4714e-05_rb,1.6387e-05_rb/)
      kbo(:,56, 3) = (/ &
     &8.4780e-06_rb,9.5613e-06_rb,1.0535e-05_rb,1.1722e-05_rb,1.2981e-05_rb/)
      kbo(:,57, 3) = (/ &
     &6.6892e-06_rb,7.5486e-06_rb,8.3874e-06_rb,9.3040e-06_rb,1.0324e-05_rb/)
      kbo(:,58, 3) = (/ &
     &5.3282e-06_rb,5.9530e-06_rb,6.6841e-06_rb,7.4078e-06_rb,8.2298e-06_rb/)
      kbo(:,59, 3) = (/ &
     &4.3459e-06_rb,4.8321e-06_rb,5.4567e-06_rb,6.0369e-06_rb,6.7261e-06_rb/)
      kbo(:,13, 4) = (/ &
     &6.7009e-02_rb,7.0746e-02_rb,7.4150e-02_rb,7.7569e-02_rb,8.1680e-02_rb/)
      kbo(:,14, 4) = (/ &
     &5.5943e-02_rb,5.8697e-02_rb,6.1678e-02_rb,6.4914e-02_rb,6.9379e-02_rb/)
      kbo(:,15, 4) = (/ &
     &4.6843e-02_rb,4.9217e-02_rb,5.1833e-02_rb,5.5783e-02_rb,5.9767e-02_rb/)
      kbo(:,16, 4) = (/ &
     &3.9239e-02_rb,4.1262e-02_rb,4.4032e-02_rb,4.7642e-02_rb,5.1925e-02_rb/)
      kbo(:,17, 4) = (/ &
     &3.3107e-02_rb,3.5295e-02_rb,3.8541e-02_rb,4.2220e-02_rb,4.6357e-02_rb/)
      kbo(:,18, 4) = (/ &
     &2.8797e-02_rb,3.1620e-02_rb,3.4631e-02_rb,3.8034e-02_rb,4.1725e-02_rb/)
      kbo(:,19, 4) = (/ &
     &2.5628e-02_rb,2.8281e-02_rb,3.1139e-02_rb,3.4358e-02_rb,3.7925e-02_rb/)
      kbo(:,20, 4) = (/ &
     &2.2579e-02_rb,2.4906e-02_rb,2.7554e-02_rb,3.0345e-02_rb,3.3488e-02_rb/)
      kbo(:,21, 4) = (/ &
     &1.9528e-02_rb,2.1738e-02_rb,2.4107e-02_rb,2.6707e-02_rb,2.9475e-02_rb/)
      kbo(:,22, 4) = (/ &
     &1.7101e-02_rb,1.8986e-02_rb,2.1039e-02_rb,2.3296e-02_rb,2.5855e-02_rb/)
      kbo(:,23, 4) = (/ &
     &1.4866e-02_rb,1.6502e-02_rb,1.8337e-02_rb,2.0414e-02_rb,2.2548e-02_rb/)
      kbo(:,24, 4) = (/ &
     &1.2821e-02_rb,1.4288e-02_rb,1.5941e-02_rb,1.7747e-02_rb,1.9679e-02_rb/)
      kbo(:,25, 4) = (/ &
     &1.1061e-02_rb,1.2369e-02_rb,1.3872e-02_rb,1.5379e-02_rb,1.7059e-02_rb/)
      kbo(:,26, 4) = (/ &
     &9.5354e-03_rb,1.0723e-02_rb,1.2012e-02_rb,1.3430e-02_rb,1.4839e-02_rb/)
      kbo(:,27, 4) = (/ &
     &8.1975e-03_rb,9.2673e-03_rb,1.0387e-02_rb,1.1650e-02_rb,1.2827e-02_rb/)
      kbo(:,28, 4) = (/ &
     &7.0231e-03_rb,7.9702e-03_rb,8.9828e-03_rb,1.0032e-02_rb,1.1053e-02_rb/)
      kbo(:,29, 4) = (/ &
     &6.0200e-03_rb,6.8421e-03_rb,7.7670e-03_rb,8.6094e-03_rb,9.5249e-03_rb/)
      kbo(:,30, 4) = (/ &
     &5.1558e-03_rb,5.8823e-03_rb,6.6735e-03_rb,7.4143e-03_rb,8.2222e-03_rb/)
      kbo(:,31, 4) = (/ &
     &4.4508e-03_rb,5.1031e-03_rb,5.7395e-03_rb,6.3990e-03_rb,7.0930e-03_rb/)
      kbo(:,32, 4) = (/ &
     &3.8687e-03_rb,4.4226e-03_rb,4.9634e-03_rb,5.5499e-03_rb,6.1447e-03_rb/)
      kbo(:,33, 4) = (/ &
     &3.3860e-03_rb,3.8414e-03_rb,4.3170e-03_rb,4.8129e-03_rb,5.3113e-03_rb/)
      kbo(:,34, 4) = (/ &
     &2.9667e-03_rb,3.3572e-03_rb,3.7642e-03_rb,4.1882e-03_rb,4.5973e-03_rb/)
      kbo(:,35, 4) = (/ &
     &2.5854e-03_rb,2.9160e-03_rb,3.2542e-03_rb,3.6120e-03_rb,3.9574e-03_rb/)
      kbo(:,36, 4) = (/ &
     &2.2132e-03_rb,2.5004e-03_rb,2.7859e-03_rb,3.0722e-03_rb,3.3673e-03_rb/)
      kbo(:,37, 4) = (/ &
     &1.8397e-03_rb,2.0798e-03_rb,2.3185e-03_rb,2.5525e-03_rb,2.8046e-03_rb/)
      kbo(:,38, 4) = (/ &
     &1.5241e-03_rb,1.7240e-03_rb,1.9250e-03_rb,2.1190e-03_rb,2.3334e-03_rb/)
      kbo(:,39, 4) = (/ &
     &1.2578e-03_rb,1.4258e-03_rb,1.5974e-03_rb,1.7602e-03_rb,1.9414e-03_rb/)
      kbo(:,40, 4) = (/ &
     &1.0136e-03_rb,1.1511e-03_rb,1.2925e-03_rb,1.4274e-03_rb,1.5799e-03_rb/)
      kbo(:,41, 4) = (/ &
     &8.1255e-04_rb,9.2485e-04_rb,1.0421e-03_rb,1.1560e-03_rb,1.2823e-03_rb/)
      kbo(:,42, 4) = (/ &
     &6.5076e-04_rb,7.4076e-04_rb,8.3891e-04_rb,9.3536e-04_rb,1.0398e-03_rb/)
      kbo(:,43, 4) = (/ &
     &5.1457e-04_rb,5.8675e-04_rb,6.6625e-04_rb,7.5025e-04_rb,8.3332e-04_rb/)
      kbo(:,44, 4) = (/ &
     &4.0130e-04_rb,4.6112e-04_rb,5.2500e-04_rb,5.9353e-04_rb,6.6396e-04_rb/)
      kbo(:,45, 4) = (/ &
     &3.1174e-04_rb,3.6176e-04_rb,4.1281e-04_rb,4.6943e-04_rb,5.2749e-04_rb/)
      kbo(:,46, 4) = (/ &
     &2.4198e-04_rb,2.8347e-04_rb,3.2327e-04_rb,3.6940e-04_rb,4.1847e-04_rb/)
      kbo(:,47, 4) = (/ &
     &1.8653e-04_rb,2.1946e-04_rb,2.5297e-04_rb,2.9033e-04_rb,3.3117e-04_rb/)
      kbo(:,48, 4) = (/ &
     &1.4273e-04_rb,1.6990e-04_rb,1.9780e-04_rb,2.2703e-04_rb,2.6076e-04_rb/)
      kbo(:,49, 4) = (/ &
     &1.1020e-04_rb,1.3076e-04_rb,1.5369e-04_rb,1.7736e-04_rb,2.0453e-04_rb/)
      kbo(:,50, 4) = (/ &
     &8.5840e-05_rb,1.0098e-04_rb,1.2003e-04_rb,1.3937e-04_rb,1.6133e-04_rb/)
      kbo(:,51, 4) = (/ &
     &6.7375e-05_rb,7.8667e-05_rb,9.3691e-05_rb,1.0967e-04_rb,1.2714e-04_rb/)
      kbo(:,52, 4) = (/ &
     &5.2333e-05_rb,6.1321e-05_rb,7.2765e-05_rb,8.6008e-05_rb,9.9915e-05_rb/)
      kbo(:,53, 4) = (/ &
     &4.0658e-05_rb,4.7664e-05_rb,5.6514e-05_rb,6.7300e-05_rb,7.8701e-05_rb/)
      kbo(:,54, 4) = (/ &
     &3.1857e-05_rb,3.7478e-05_rb,4.4306e-05_rb,5.3025e-05_rb,6.2299e-05_rb/)
      kbo(:,55, 4) = (/ &
     &2.4955e-05_rb,2.9435e-05_rb,3.4823e-05_rb,4.1578e-05_rb,4.9419e-05_rb/)
      kbo(:,56, 4) = (/ &
     &1.9655e-05_rb,2.3009e-05_rb,2.7169e-05_rb,3.2570e-05_rb,3.9005e-05_rb/)
      kbo(:,57, 4) = (/ &
     &1.5476e-05_rb,1.8100e-05_rb,2.1340e-05_rb,2.5521e-05_rb,3.0667e-05_rb/)
      kbo(:,58, 4) = (/ &
     &1.2154e-05_rb,1.4292e-05_rb,1.6827e-05_rb,2.0050e-05_rb,2.4167e-05_rb/)
      kbo(:,59, 4) = (/ &
     &9.8901e-06_rb,1.1647e-05_rb,1.3760e-05_rb,1.6454e-05_rb,1.9831e-05_rb/)
      kbo(:,13, 5) = (/ &
     &1.3634e-01_rb,1.4252e-01_rb,1.5001e-01_rb,1.6028e-01_rb,1.7134e-01_rb/)
      kbo(:,14, 5) = (/ &
     &1.1274e-01_rb,1.1839e-01_rb,1.2582e-01_rb,1.3453e-01_rb,1.4378e-01_rb/)
      kbo(:,15, 5) = (/ &
     &9.3550e-02_rb,9.9682e-02_rb,1.0681e-01_rb,1.1432e-01_rb,1.2335e-01_rb/)
      kbo(:,16, 5) = (/ &
     &7.9704e-02_rb,8.5926e-02_rb,9.2006e-02_rb,9.9216e-02_rb,1.0669e-01_rb/)
      kbo(:,17, 5) = (/ &
     &6.9562e-02_rb,7.4399e-02_rb,8.0031e-02_rb,8.6090e-02_rb,9.3210e-02_rb/)
      kbo(:,18, 5) = (/ &
     &6.1028e-02_rb,6.5373e-02_rb,7.1097e-02_rb,7.7143e-02_rb,8.4153e-02_rb/)
      kbo(:,19, 5) = (/ &
     &5.4138e-02_rb,5.9145e-02_rb,6.4366e-02_rb,7.0461e-02_rb,7.6822e-02_rb/)
      kbo(:,20, 5) = (/ &
     &4.7698e-02_rb,5.2419e-02_rb,5.7657e-02_rb,6.3398e-02_rb,6.9523e-02_rb/)
      kbo(:,21, 5) = (/ &
     &4.2126e-02_rb,4.6323e-02_rb,5.1164e-02_rb,5.6007e-02_rb,6.1597e-02_rb/)
      kbo(:,22, 5) = (/ &
     &3.6873e-02_rb,4.0763e-02_rb,4.4958e-02_rb,4.9556e-02_rb,5.4204e-02_rb/)
      kbo(:,23, 5) = (/ &
     &3.2201e-02_rb,3.5761e-02_rb,3.9530e-02_rb,4.3670e-02_rb,4.7920e-02_rb/)
      kbo(:,24, 5) = (/ &
     &2.8239e-02_rb,3.1305e-02_rb,3.4844e-02_rb,3.8415e-02_rb,4.2157e-02_rb/)
      kbo(:,25, 5) = (/ &
     &2.4669e-02_rb,2.7511e-02_rb,3.0568e-02_rb,3.3769e-02_rb,3.6890e-02_rb/)
      kbo(:,26, 5) = (/ &
     &2.1582e-02_rb,2.4173e-02_rb,2.6811e-02_rb,2.9496e-02_rb,3.2330e-02_rb/)
      kbo(:,27, 5) = (/ &
     &1.8849e-02_rb,2.1094e-02_rb,2.3446e-02_rb,2.5755e-02_rb,2.8374e-02_rb/)
      kbo(:,28, 5) = (/ &
     &1.6379e-02_rb,1.8330e-02_rb,2.0367e-02_rb,2.2542e-02_rb,2.4931e-02_rb/)
      kbo(:,29, 5) = (/ &
     &1.4280e-02_rb,1.5993e-02_rb,1.7723e-02_rb,1.9736e-02_rb,2.1895e-02_rb/)
      kbo(:,30, 5) = (/ &
     &1.2451e-02_rb,1.3902e-02_rb,1.5477e-02_rb,1.7322e-02_rb,1.9234e-02_rb/)
      kbo(:,31, 5) = (/ &
     &1.0840e-02_rb,1.2122e-02_rb,1.3593e-02_rb,1.5159e-02_rb,1.6921e-02_rb/)
      kbo(:,32, 5) = (/ &
     &9.4753e-03_rb,1.0651e-02_rb,1.1992e-02_rb,1.3397e-02_rb,1.4887e-02_rb/)
      kbo(:,33, 5) = (/ &
     &8.3358e-03_rb,9.4249e-03_rb,1.0592e-02_rb,1.1871e-02_rb,1.3208e-02_rb/)
      kbo(:,34, 5) = (/ &
     &7.3799e-03_rb,8.3765e-03_rb,9.4456e-03_rb,1.0582e-02_rb,1.1863e-02_rb/)
      kbo(:,35, 5) = (/ &
     &6.4631e-03_rb,7.3628e-03_rb,8.3364e-03_rb,9.3419e-03_rb,1.0526e-02_rb/)
      kbo(:,36, 5) = (/ &
     &5.5926e-03_rb,6.3888e-03_rb,7.2585e-03_rb,8.2192e-03_rb,9.2462e-03_rb/)
      kbo(:,37, 5) = (/ &
     &4.7015e-03_rb,5.3951e-03_rb,6.1680e-03_rb,7.0229e-03_rb,7.9329e-03_rb/)
      kbo(:,38, 5) = (/ &
     &3.9517e-03_rb,4.5644e-03_rb,5.2429e-03_rb,6.0141e-03_rb,6.8184e-03_rb/)
      kbo(:,39, 5) = (/ &
     &3.3266e-03_rb,3.8629e-03_rb,4.4575e-03_rb,5.1473e-03_rb,5.8646e-03_rb/)
      kbo(:,40, 5) = (/ &
     &2.7281e-03_rb,3.1818e-03_rb,3.6932e-03_rb,4.2742e-03_rb,4.8976e-03_rb/)
      kbo(:,41, 5) = (/ &
     &2.2270e-03_rb,2.6113e-03_rb,3.0386e-03_rb,3.5308e-03_rb,4.0601e-03_rb/)
      kbo(:,42, 5) = (/ &
     &1.8093e-03_rb,2.1403e-03_rb,2.4947e-03_rb,2.9136e-03_rb,3.3566e-03_rb/)
      kbo(:,43, 5) = (/ &
     &1.4510e-03_rb,1.7247e-03_rb,2.0294e-03_rb,2.3716e-03_rb,2.7407e-03_rb/)
      kbo(:,44, 5) = (/ &
     &1.1565e-03_rb,1.3809e-03_rb,1.6356e-03_rb,1.9170e-03_rb,2.2199e-03_rb/)
      kbo(:,45, 5) = (/ &
     &9.2074e-04_rb,1.1011e-03_rb,1.3110e-03_rb,1.5432e-03_rb,1.7974e-03_rb/)
      kbo(:,46, 5) = (/ &
     &7.2799e-04_rb,8.7147e-04_rb,1.0468e-03_rb,1.2375e-03_rb,1.4475e-03_rb/)
      kbo(:,47, 5) = (/ &
     &5.6825e-04_rb,6.8576e-04_rb,8.2827e-04_rb,9.8300e-04_rb,1.1595e-03_rb/)
      kbo(:,48, 5) = (/ &
     &4.4362e-04_rb,5.3785e-04_rb,6.5132e-04_rb,7.7947e-04_rb,9.2729e-04_rb/)
      kbo(:,49, 5) = (/ &
     &3.4360e-04_rb,4.2004e-04_rb,5.1060e-04_rb,6.1673e-04_rb,7.3913e-04_rb/)
      kbo(:,50, 5) = (/ &
     &2.6639e-04_rb,3.2974e-04_rb,4.0310e-04_rb,4.8991e-04_rb,5.9182e-04_rb/)
      kbo(:,51, 5) = (/ &
     &2.0705e-04_rb,2.5803e-04_rb,3.1790e-04_rb,3.8886e-04_rb,4.7401e-04_rb/)
      kbo(:,52, 5) = (/ &
     &1.6104e-04_rb,2.0156e-04_rb,2.4998e-04_rb,3.0713e-04_rb,3.7759e-04_rb/)
      kbo(:,53, 5) = (/ &
     &1.2422e-04_rb,1.5670e-04_rb,1.9587e-04_rb,2.4211e-04_rb,2.9925e-04_rb/)
      kbo(:,54, 5) = (/ &
     &9.6443e-05_rb,1.2239e-04_rb,1.5459e-04_rb,1.9203e-04_rb,2.3866e-04_rb/)
      kbo(:,55, 5) = (/ &
     &7.5345e-05_rb,9.6163e-05_rb,1.2185e-04_rb,1.5278e-04_rb,1.9092e-04_rb/)
      kbo(:,56, 5) = (/ &
     &5.8479e-05_rb,7.5388e-05_rb,9.6163e-05_rb,1.2109e-04_rb,1.5224e-04_rb/)
      kbo(:,57, 5) = (/ &
     &4.5173e-05_rb,5.8728e-05_rb,7.5432e-05_rb,9.5735e-05_rb,1.2084e-04_rb/)
      kbo(:,58, 5) = (/ &
     &3.5110e-05_rb,4.5659e-05_rb,5.9108e-05_rb,7.5684e-05_rb,9.5780e-05_rb/)
      kbo(:,59, 5) = (/ &
     &2.8669e-05_rb,3.7452e-05_rb,4.8577e-05_rb,6.2553e-05_rb,7.9913e-05_rb/)
      kbo(:,13, 6) = (/ &
     &3.1943e-01_rb,3.3627e-01_rb,3.5152e-01_rb,3.6856e-01_rb,3.8489e-01_rb/)
      kbo(:,14, 6) = (/ &
     &2.6955e-01_rb,2.8315e-01_rb,2.9631e-01_rb,3.1035e-01_rb,3.2480e-01_rb/)
      kbo(:,15, 6) = (/ &
     &2.2744e-01_rb,2.3792e-01_rb,2.4935e-01_rb,2.6127e-01_rb,2.7359e-01_rb/)
      kbo(:,16, 6) = (/ &
     &1.9190e-01_rb,2.0128e-01_rb,2.1209e-01_rb,2.2354e-01_rb,2.3634e-01_rb/)
      kbo(:,17, 6) = (/ &
     &1.6550e-01_rb,1.7572e-01_rb,1.8575e-01_rb,1.9639e-01_rb,2.0864e-01_rb/)
      kbo(:,18, 6) = (/ &
     &1.4664e-01_rb,1.5533e-01_rb,1.6464e-01_rb,1.7496e-01_rb,1.8532e-01_rb/)
      kbo(:,19, 6) = (/ &
     &1.3086e-01_rb,1.3857e-01_rb,1.4754e-01_rb,1.5676e-01_rb,1.6804e-01_rb/)
      kbo(:,20, 6) = (/ &
     &1.1476e-01_rb,1.2256e-01_rb,1.3048e-01_rb,1.3999e-01_rb,1.5052e-01_rb/)
      kbo(:,21, 6) = (/ &
     &1.0042e-01_rb,1.0755e-01_rb,1.1552e-01_rb,1.2457e-01_rb,1.3541e-01_rb/)
      kbo(:,22, 6) = (/ &
     &8.7811e-02_rb,9.4376e-02_rb,1.0240e-01_rb,1.1139e-01_rb,1.2190e-01_rb/)
      kbo(:,23, 6) = (/ &
     &7.6746e-02_rb,8.3440e-02_rb,9.0958e-02_rb,9.9436e-02_rb,1.0837e-01_rb/)
      kbo(:,24, 6) = (/ &
     &6.7020e-02_rb,7.3597e-02_rb,8.0186e-02_rb,8.7655e-02_rb,9.5847e-02_rb/)
      kbo(:,25, 6) = (/ &
     &5.8852e-02_rb,6.4342e-02_rb,7.0651e-02_rb,7.7553e-02_rb,8.5595e-02_rb/)
      kbo(:,26, 6) = (/ &
     &5.1501e-02_rb,5.6677e-02_rb,6.2671e-02_rb,6.9309e-02_rb,7.6862e-02_rb/)
      kbo(:,27, 6) = (/ &
     &4.5068e-02_rb,5.0089e-02_rb,5.5571e-02_rb,6.2056e-02_rb,6.8532e-02_rb/)
      kbo(:,28, 6) = (/ &
     &3.9606e-02_rb,4.4193e-02_rb,4.9408e-02_rb,5.5057e-02_rb,6.0956e-02_rb/)
      kbo(:,29, 6) = (/ &
     &3.4989e-02_rb,3.9176e-02_rb,4.4032e-02_rb,4.9174e-02_rb,5.4655e-02_rb/)
      kbo(:,30, 6) = (/ &
     &3.0943e-02_rb,3.4971e-02_rb,3.9398e-02_rb,4.4027e-02_rb,4.9163e-02_rb/)
      kbo(:,31, 6) = (/ &
     &2.7553e-02_rb,3.1287e-02_rb,3.5351e-02_rb,3.9782e-02_rb,4.4558e-02_rb/)
      kbo(:,32, 6) = (/ &
     &2.4710e-02_rb,2.8155e-02_rb,3.1869e-02_rb,3.6099e-02_rb,4.0666e-02_rb/)
      kbo(:,33, 6) = (/ &
     &2.2173e-02_rb,2.5362e-02_rb,2.8990e-02_rb,3.2965e-02_rb,3.7297e-02_rb/)
      kbo(:,34, 6) = (/ &
     &2.0107e-02_rb,2.3109e-02_rb,2.6574e-02_rb,3.0332e-02_rb,3.4357e-02_rb/)
      kbo(:,35, 6) = (/ &
     &1.8022e-02_rb,2.0877e-02_rb,2.4132e-02_rb,2.7710e-02_rb,3.1520e-02_rb/)
      kbo(:,36, 6) = (/ &
     &1.5963e-02_rb,1.8659e-02_rb,2.1660e-02_rb,2.4929e-02_rb,2.8560e-02_rb/)
      kbo(:,37, 6) = (/ &
     &1.3764e-02_rb,1.6188e-02_rb,1.8916e-02_rb,2.1932e-02_rb,2.5294e-02_rb/)
      kbo(:,38, 6) = (/ &
     &1.1849e-02_rb,1.4028e-02_rb,1.6494e-02_rb,1.9269e-02_rb,2.2340e-02_rb/)
      kbo(:,39, 6) = (/ &
     &1.0207e-02_rb,1.2194e-02_rb,1.4401e-02_rb,1.6941e-02_rb,1.9796e-02_rb/)
      kbo(:,40, 6) = (/ &
     &8.5573e-03_rb,1.0315e-02_rb,1.2260e-02_rb,1.4559e-02_rb,1.7141e-02_rb/)
      kbo(:,41, 6) = (/ &
     &7.1290e-03_rb,8.6738e-03_rb,1.0403e-02_rb,1.2445e-02_rb,1.4801e-02_rb/)
      kbo(:,42, 6) = (/ &
     &5.9179e-03_rb,7.2709e-03_rb,8.8122e-03_rb,1.0627e-02_rb,1.2787e-02_rb/)
      kbo(:,43, 6) = (/ &
     &4.8318e-03_rb,5.9892e-03_rb,7.3334e-03_rb,8.9514e-03_rb,1.0874e-02_rb/)
      kbo(:,44, 6) = (/ &
     &3.8999e-03_rb,4.8767e-03_rb,6.0501e-03_rb,7.4816e-03_rb,9.1794e-03_rb/)
      kbo(:,45, 6) = (/ &
     &3.1360e-03_rb,3.9601e-03_rb,4.9837e-03_rb,6.2240e-03_rb,7.7311e-03_rb/)
      kbo(:,46, 6) = (/ &
     &2.5027e-03_rb,3.1827e-03_rb,4.0540e-03_rb,5.1329e-03_rb,6.4480e-03_rb/)
      kbo(:,47, 6) = (/ &
     &1.9756e-03_rb,2.5273e-03_rb,3.2610e-03_rb,4.1749e-03_rb,5.3122e-03_rb/)
      kbo(:,48, 6) = (/ &
     &1.5473e-03_rb,1.9960e-03_rb,2.6043e-03_rb,3.3772e-03_rb,4.3437e-03_rb/)
      kbo(:,49, 6) = (/ &
     &1.2068e-03_rb,1.5706e-03_rb,2.0674e-03_rb,2.7091e-03_rb,3.5307e-03_rb/)
      kbo(:,50, 6) = (/ &
     &9.4521e-04_rb,1.2396e-03_rb,1.6445e-03_rb,2.1801e-03_rb,2.8792e-03_rb/)
      kbo(:,51, 6) = (/ &
     &7.3804e-04_rb,9.7746e-04_rb,1.3076e-03_rb,1.7542e-03_rb,2.3409e-03_rb/)
      kbo(:,52, 6) = (/ &
     &5.7206e-04_rb,7.6763e-04_rb,1.0338e-03_rb,1.4015e-03_rb,1.8909e-03_rb/)
      kbo(:,53, 6) = (/ &
     &4.4225e-04_rb,5.9805e-04_rb,8.1248e-04_rb,1.1113e-03_rb,1.5170e-03_rb/)
      kbo(:,54, 6) = (/ &
     &3.4467e-04_rb,4.7010e-04_rb,6.4272e-04_rb,8.8848e-04_rb,1.2276e-03_rb/)
      kbo(:,55, 6) = (/ &
     &2.6885e-04_rb,3.6925e-04_rb,5.1051e-04_rb,7.1238e-04_rb,9.9424e-04_rb/)
      kbo(:,56, 6) = (/ &
     &2.0891e-04_rb,2.8866e-04_rb,4.0323e-04_rb,5.6854e-04_rb,8.0318e-04_rb/)
      kbo(:,57, 6) = (/ &
     &1.6206e-04_rb,2.2457e-04_rb,3.1663e-04_rb,4.5154e-04_rb,6.4436e-04_rb/)
      kbo(:,58, 6) = (/ &
     &1.2565e-04_rb,1.7534e-04_rb,2.4919e-04_rb,3.5849e-04_rb,5.1835e-04_rb/)
      kbo(:,59, 6) = (/ &
     &1.0370e-04_rb,1.4590e-04_rb,2.0972e-04_rb,3.0572e-04_rb,4.4824e-04_rb/)
      kbo(:,13, 7) = (/ &
     &8.6885e-01_rb,8.9426e-01_rb,9.2071e-01_rb,9.5081e-01_rb,9.7593e-01_rb/)
      kbo(:,14, 7) = (/ &
     &7.3422e-01_rb,7.5692e-01_rb,7.8528e-01_rb,8.1141e-01_rb,8.3349e-01_rb/)
      kbo(:,15, 7) = (/ &
     &6.1192e-01_rb,6.3447e-01_rb,6.5837e-01_rb,6.7854e-01_rb,7.0269e-01_rb/)
      kbo(:,16, 7) = (/ &
     &5.0884e-01_rb,5.2842e-01_rb,5.4670e-01_rb,5.6627e-01_rb,5.8867e-01_rb/)
      kbo(:,17, 7) = (/ &
     &4.3640e-01_rb,4.5419e-01_rb,4.7288e-01_rb,4.9197e-01_rb,5.1336e-01_rb/)
      kbo(:,18, 7) = (/ &
     &3.8407e-01_rb,4.0203e-01_rb,4.1993e-01_rb,4.4018e-01_rb,4.6362e-01_rb/)
      kbo(:,19, 7) = (/ &
     &3.4228e-01_rb,3.5912e-01_rb,3.7740e-01_rb,3.9888e-01_rb,4.2103e-01_rb/)
      kbo(:,20, 7) = (/ &
     &3.0199e-01_rb,3.1730e-01_rb,3.3484e-01_rb,3.5351e-01_rb,3.7675e-01_rb/)
      kbo(:,21, 7) = (/ &
     &2.6553e-01_rb,2.8042e-01_rb,2.9603e-01_rb,3.1609e-01_rb,3.3882e-01_rb/)
      kbo(:,22, 7) = (/ &
     &2.3320e-01_rb,2.4726e-01_rb,2.6434e-01_rb,2.8368e-01_rb,3.0736e-01_rb/)
      kbo(:,23, 7) = (/ &
     &2.0578e-01_rb,2.1941e-01_rb,2.3579e-01_rb,2.5629e-01_rb,2.8224e-01_rb/)
      kbo(:,24, 7) = (/ &
     &1.8045e-01_rb,1.9362e-01_rb,2.1135e-01_rb,2.3376e-01_rb,2.5793e-01_rb/)
      kbo(:,25, 7) = (/ &
     &1.5885e-01_rb,1.7311e-01_rb,1.9174e-01_rb,2.1249e-01_rb,2.3261e-01_rb/)
      kbo(:,26, 7) = (/ &
     &1.4101e-01_rb,1.5639e-01_rb,1.7418e-01_rb,1.9114e-01_rb,2.1048e-01_rb/)
      kbo(:,27, 7) = (/ &
     &1.2599e-01_rb,1.4076e-01_rb,1.5557e-01_rb,1.7183e-01_rb,1.9023e-01_rb/)
      kbo(:,28, 7) = (/ &
     &1.1218e-01_rb,1.2546e-01_rb,1.3903e-01_rb,1.5482e-01_rb,1.7215e-01_rb/)
      kbo(:,29, 7) = (/ &
     &1.0072e-01_rb,1.1254e-01_rb,1.2583e-01_rb,1.4062e-01_rb,1.5650e-01_rb/)
      kbo(:,30, 7) = (/ &
     &9.0713e-02_rb,1.0192e-01_rb,1.1464e-01_rb,1.2854e-01_rb,1.4398e-01_rb/)
      kbo(:,31, 7) = (/ &
     &8.2277e-02_rb,9.3097e-02_rb,1.0524e-01_rb,1.1856e-01_rb,1.3355e-01_rb/)
      kbo(:,32, 7) = (/ &
     &7.5303e-02_rb,8.5763e-02_rb,9.7401e-02_rb,1.1054e-01_rb,1.2526e-01_rb/)
      kbo(:,33, 7) = (/ &
     &6.9506e-02_rb,7.9549e-02_rb,9.0902e-02_rb,1.0398e-01_rb,1.1862e-01_rb/)
      kbo(:,34, 7) = (/ &
     &6.4727e-02_rb,7.4554e-02_rb,8.5918e-02_rb,9.8930e-02_rb,1.1374e-01_rb/)
      kbo(:,35, 7) = (/ &
     &5.9573e-02_rb,6.9480e-02_rb,8.0602e-02_rb,9.3549e-02_rb,1.0847e-01_rb/)
      kbo(:,36, 7) = (/ &
     &5.4200e-02_rb,6.3825e-02_rb,7.4842e-02_rb,8.7689e-02_rb,1.0224e-01_rb/)
      kbo(:,37, 7) = (/ &
     &4.7941e-02_rb,5.7083e-02_rb,6.7560e-02_rb,8.0041e-02_rb,9.3919e-02_rb/)
      kbo(:,38, 7) = (/ &
     &4.2423e-02_rb,5.1057e-02_rb,6.0983e-02_rb,7.2873e-02_rb,8.6484e-02_rb/)
      kbo(:,39, 7) = (/ &
     &3.7589e-02_rb,4.5606e-02_rb,5.5155e-02_rb,6.6511e-02_rb,7.9703e-02_rb/)
      kbo(:,40, 7) = (/ &
     &3.2287e-02_rb,3.9605e-02_rb,4.8489e-02_rb,5.9153e-02_rb,7.1379e-02_rb/)
      kbo(:,41, 7) = (/ &
     &2.7611e-02_rb,3.4241e-02_rb,4.2425e-02_rb,5.2312e-02_rb,6.3597e-02_rb/)
      kbo(:,42, 7) = (/ &
     &2.3526e-02_rb,2.9544e-02_rb,3.7050e-02_rb,4.6066e-02_rb,5.6624e-02_rb/)
      kbo(:,43, 7) = (/ &
     &1.9724e-02_rb,2.5077e-02_rb,3.1787e-02_rb,3.9922e-02_rb,4.9656e-02_rb/)
      kbo(:,44, 7) = (/ &
     &1.6360e-02_rb,2.1054e-02_rb,2.6994e-02_rb,3.4254e-02_rb,4.3106e-02_rb/)
      kbo(:,45, 7) = (/ &
     &1.3479e-02_rb,1.7563e-02_rb,2.2810e-02_rb,2.9308e-02_rb,3.7331e-02_rb/)
      kbo(:,46, 7) = (/ &
     &1.0982e-02_rb,1.4524e-02_rb,1.9109e-02_rb,2.4872e-02_rb,3.2032e-02_rb/)
      kbo(:,47, 7) = (/ &
     &8.8182e-03_rb,1.1815e-02_rb,1.5755e-02_rb,2.0785e-02_rb,2.7117e-02_rb/)
      kbo(:,48, 7) = (/ &
     &7.0297e-03_rb,9.5228e-03_rb,1.2883e-02_rb,1.7248e-02_rb,2.2808e-02_rb/)
      kbo(:,49, 7) = (/ &
     &5.5570e-03_rb,7.6141e-03_rb,1.0446e-02_rb,1.4193e-02_rb,1.9030e-02_rb/)
      kbo(:,50, 7) = (/ &
     &4.4018e-03_rb,6.0988e-03_rb,8.5078e-03_rb,1.1727e-02_rb,1.5958e-02_rb/)
      kbo(:,51, 7) = (/ &
     &3.4813e-03_rb,4.8790e-03_rb,6.9087e-03_rb,9.6740e-03_rb,1.3370e-02_rb/)
      kbo(:,52, 7) = (/ &
     &2.7259e-03_rb,3.8645e-03_rb,5.5503e-03_rb,7.9313e-03_rb,1.1133e-02_rb/)
      kbo(:,53, 7) = (/ &
     &2.1153e-03_rb,3.0336e-03_rb,4.4195e-03_rb,6.4300e-03_rb,9.1714e-03_rb/)
      kbo(:,54, 7) = (/ &
     &1.6649e-03_rb,2.4099e-03_rb,3.5602e-03_rb,5.2651e-03_rb,7.6558e-03_rb/)
      kbo(:,55, 7) = (/ &
     &1.3120e-03_rb,1.9223e-03_rb,2.8746e-03_rb,4.3204e-03_rb,6.4144e-03_rb/)
      kbo(:,56, 7) = (/ &
     &1.0280e-03_rb,1.5213e-03_rb,2.3034e-03_rb,3.5254e-03_rb,5.3355e-03_rb/)
      kbo(:,57, 7) = (/ &
     &8.0119e-04_rb,1.1961e-03_rb,1.8353e-03_rb,2.8565e-03_rb,4.4054e-03_rb/)
      kbo(:,58, 7) = (/ &
     &6.2596e-04_rb,9.4223e-04_rb,1.4648e-03_rb,2.3194e-03_rb,3.6416e-03_rb/)
      kbo(:,59, 7) = (/ &
     &5.2923e-04_rb,8.1079e-04_rb,1.2847e-03_rb,2.0681e-03_rb,3.3065e-03_rb/)
      kbo(:,13, 8) = (/ &
     &3.0137e+00_rb,3.0721e+00_rb,3.1135e+00_rb,3.1353e+00_rb,3.1526e+00_rb/)
      kbo(:,14, 8) = (/ &
     &2.5881e+00_rb,2.6331e+00_rb,2.6669e+00_rb,2.6887e+00_rb,2.7106e+00_rb/)
      kbo(:,15, 8) = (/ &
     &2.2017e+00_rb,2.2477e+00_rb,2.2747e+00_rb,2.3022e+00_rb,2.3199e+00_rb/)
      kbo(:,16, 8) = (/ &
     &1.8745e+00_rb,1.9119e+00_rb,1.9407e+00_rb,1.9650e+00_rb,1.9910e+00_rb/)
      kbo(:,17, 8) = (/ &
     &1.5865e+00_rb,1.6181e+00_rb,1.6438e+00_rb,1.6744e+00_rb,1.7044e+00_rb/)
      kbo(:,18, 8) = (/ &
     &1.3469e+00_rb,1.3737e+00_rb,1.4061e+00_rb,1.4422e+00_rb,1.4784e+00_rb/)
      kbo(:,19, 8) = (/ &
     &1.1589e+00_rb,1.1936e+00_rb,1.2348e+00_rb,1.2728e+00_rb,1.3146e+00_rb/)
      kbo(:,20, 8) = (/ &
     &1.0242e+00_rb,1.0660e+00_rb,1.1056e+00_rb,1.1466e+00_rb,1.1869e+00_rb/)
      kbo(:,21, 8) = (/ &
     &9.1121e-01_rb,9.5273e-01_rb,9.9427e-01_rb,1.0332e+00_rb,1.0710e+00_rb/)
      kbo(:,22, 8) = (/ &
     &8.0647e-01_rb,8.4440e-01_rb,8.8114e-01_rb,9.1866e-01_rb,9.5785e-01_rb/)
      kbo(:,23, 8) = (/ &
     &7.0834e-01_rb,7.4542e-01_rb,7.7998e-01_rb,8.1759e-01_rb,8.5850e-01_rb/)
      kbo(:,24, 8) = (/ &
     &6.2177e-01_rb,6.5425e-01_rb,6.8799e-01_rb,7.2751e-01_rb,7.7799e-01_rb/)
      kbo(:,25, 8) = (/ &
     &5.4589e-01_rb,5.7729e-01_rb,6.1309e-01_rb,6.5804e-01_rb,7.1412e-01_rb/)
      kbo(:,26, 8) = (/ &
     &4.8128e-01_rb,5.1418e-01_rb,5.5324e-01_rb,6.0347e-01_rb,6.6308e-01_rb/)
      kbo(:,27, 8) = (/ &
     &4.2682e-01_rb,4.6055e-01_rb,5.0524e-01_rb,5.5765e-01_rb,6.2135e-01_rb/)
      kbo(:,28, 8) = (/ &
     &3.8006e-01_rb,4.1721e-01_rb,4.6337e-01_rb,5.1919e-01_rb,5.8322e-01_rb/)
      kbo(:,29, 8) = (/ &
     &3.4196e-01_rb,3.8125e-01_rb,4.2943e-01_rb,4.8588e-01_rb,5.5221e-01_rb/)
      kbo(:,30, 8) = (/ &
     &3.1093e-01_rb,3.5114e-01_rb,4.0038e-01_rb,4.5863e-01_rb,5.2276e-01_rb/)
      kbo(:,31, 8) = (/ &
     &2.8616e-01_rb,3.2668e-01_rb,3.7633e-01_rb,4.3472e-01_rb,4.9941e-01_rb/)
      kbo(:,32, 8) = (/ &
     &2.6688e-01_rb,3.0820e-01_rb,3.5883e-01_rb,4.1523e-01_rb,4.8206e-01_rb/)
      kbo(:,33, 8) = (/ &
     &2.5149e-01_rb,2.9538e-01_rb,3.4473e-01_rb,4.0327e-01_rb,4.6953e-01_rb/)
      kbo(:,34, 8) = (/ &
     &2.4069e-01_rb,2.8516e-01_rb,3.3563e-01_rb,3.9421e-01_rb,4.6040e-01_rb/)
      kbo(:,35, 8) = (/ &
     &2.2923e-01_rb,2.7328e-01_rb,3.2510e-01_rb,3.8300e-01_rb,4.4899e-01_rb/)
      kbo(:,36, 8) = (/ &
     &2.1677e-01_rb,2.6037e-01_rb,3.1183e-01_rb,3.6883e-01_rb,4.3509e-01_rb/)
      kbo(:,37, 8) = (/ &
     &1.9848e-01_rb,2.3986e-01_rb,2.8965e-01_rb,3.4421e-01_rb,4.0854e-01_rb/)
      kbo(:,38, 8) = (/ &
     &1.8143e-01_rb,2.2127e-01_rb,2.6901e-01_rb,3.2212e-01_rb,3.8468e-01_rb/)
      kbo(:,39, 8) = (/ &
     &1.6606e-01_rb,2.0455e-01_rb,2.5024e-01_rb,3.0203e-01_rb,3.6363e-01_rb/)
      kbo(:,40, 8) = (/ &
     &1.4741e-01_rb,1.8322e-01_rb,2.2603e-01_rb,2.7520e-01_rb,3.3417e-01_rb/)
      kbo(:,41, 8) = (/ &
     &1.3011e-01_rb,1.6308e-01_rb,2.0319e-01_rb,2.4965e-01_rb,3.0588e-01_rb/)
      kbo(:,42, 8) = (/ &
     &1.1484e-01_rb,1.4491e-01_rb,1.8233e-01_rb,2.2638e-01_rb,2.7964e-01_rb/)
      kbo(:,43, 8) = (/ &
     &9.9279e-02_rb,1.2633e-01_rb,1.6039e-01_rb,2.0179e-01_rb,2.5143e-01_rb/)
      kbo(:,44, 8) = (/ &
     &8.4904e-02_rb,1.0929e-01_rb,1.3953e-01_rb,1.7766e-01_rb,2.2379e-01_rb/)
      kbo(:,45, 8) = (/ &
     &7.2367e-02_rb,9.4015e-02_rb,1.2097e-01_rb,1.5526e-01_rb,1.9786e-01_rb/)
      kbo(:,46, 8) = (/ &
     &6.1012e-02_rb,8.0269e-02_rb,1.0419e-01_rb,1.3470e-01_rb,1.7331e-01_rb/)
      kbo(:,47, 8) = (/ &
     &5.0418e-02_rb,6.7581e-02_rb,8.8667e-02_rb,1.1556e-01_rb,1.5023e-01_rb/)
      kbo(:,48, 8) = (/ &
     &4.1555e-02_rb,5.6345e-02_rb,7.4999e-02_rb,9.8811e-02_rb,1.2934e-01_rb/)
      kbo(:,49, 8) = (/ &
     &3.3904e-02_rb,4.6392e-02_rb,6.2939e-02_rb,8.3873e-02_rb,1.1077e-01_rb/)
      kbo(:,50, 8) = (/ &
     &2.7924e-02_rb,3.8644e-02_rb,5.3103e-02_rb,7.1817e-02_rb,9.5860e-02_rb/)
      kbo(:,51, 8) = (/ &
     &2.3027e-02_rb,3.2254e-02_rb,4.4770e-02_rb,6.1630e-02_rb,8.3189e-02_rb/)
      kbo(:,52, 8) = (/ &
     &1.8860e-02_rb,2.6783e-02_rb,3.7563e-02_rb,5.2481e-02_rb,7.1941e-02_rb/)
      kbo(:,53, 8) = (/ &
     &1.5315e-02_rb,2.2045e-02_rb,3.1363e-02_rb,4.4231e-02_rb,6.1710e-02_rb/)
      kbo(:,54, 8) = (/ &
     &1.2583e-02_rb,1.8403e-02_rb,2.6586e-02_rb,3.7901e-02_rb,5.3727e-02_rb/)
      kbo(:,55, 8) = (/ &
     &1.0382e-02_rb,1.5404e-02_rb,2.2584e-02_rb,3.2684e-02_rb,4.7053e-02_rb/)
      kbo(:,56, 8) = (/ &
     &8.4944e-03_rb,1.2799e-02_rb,1.9133e-02_rb,2.8106e-02_rb,4.1010e-02_rb/)
      kbo(:,57, 8) = (/ &
     &6.8852e-03_rb,1.0598e-02_rb,1.6081e-02_rb,2.3991e-02_rb,3.5498e-02_rb/)
      kbo(:,58, 8) = (/ &
     &5.5895e-03_rb,8.7768e-03_rb,1.3535e-02_rb,2.0567e-02_rb,3.0872e-02_rb/)
      kbo(:,59, 8) = (/ &
     &4.9705e-03_rb,7.9600e-03_rb,1.2473e-02_rb,1.9319e-02_rb,2.9463e-02_rb/)
      kbo(:,13, 9) = (/ &
     &1.4590e+01_rb,1.4670e+01_rb,1.4841e+01_rb,1.4854e+01_rb,1.4880e+01_rb/)
      kbo(:,14, 9) = (/ &
     &1.3239e+01_rb,1.3368e+01_rb,1.3423e+01_rb,1.3479e+01_rb,1.3588e+01_rb/)
      kbo(:,15, 9) = (/ &
     &1.1801e+01_rb,1.1873e+01_rb,1.1983e+01_rb,1.2092e+01_rb,1.2194e+01_rb/)
      kbo(:,16, 9) = (/ &
     &1.0397e+01_rb,1.0519e+01_rb,1.0636e+01_rb,1.0761e+01_rb,1.0910e+01_rb/)
      kbo(:,17, 9) = (/ &
     &9.1941e+00_rb,9.3273e+00_rb,9.4699e+00_rb,9.6116e+00_rb,9.7497e+00_rb/)
      kbo(:,18, 9) = (/ &
     &8.2105e+00_rb,8.3532e+00_rb,8.4897e+00_rb,8.6223e+00_rb,8.7373e+00_rb/)
      kbo(:,19, 9) = (/ &
     &7.2827e+00_rb,7.4240e+00_rb,7.5562e+00_rb,7.6727e+00_rb,7.7914e+00_rb/)
      kbo(:,20, 9) = (/ &
     &6.3721e+00_rb,6.4943e+00_rb,6.6028e+00_rb,6.7186e+00_rb,6.8391e+00_rb/)
      kbo(:,21, 9) = (/ &
     &5.5515e+00_rb,5.6468e+00_rb,5.7503e+00_rb,5.8563e+00_rb,5.9656e+00_rb/)
      kbo(:,22, 9) = (/ &
     &4.8000e+00_rb,4.8972e+00_rb,4.9944e+00_rb,5.1084e+00_rb,5.2408e+00_rb/)
      kbo(:,23, 9) = (/ &
     &4.1624e+00_rb,4.2656e+00_rb,4.3811e+00_rb,4.5085e+00_rb,4.6380e+00_rb/)
      kbo(:,24, 9) = (/ &
     &3.6468e+00_rb,3.7664e+00_rb,3.9092e+00_rb,4.0407e+00_rb,4.1687e+00_rb/)
      kbo(:,25, 9) = (/ &
     &3.2292e+00_rb,3.3800e+00_rb,3.5189e+00_rb,3.6529e+00_rb,3.8087e+00_rb/)
      kbo(:,26, 9) = (/ &
     &2.9129e+00_rb,3.0593e+00_rb,3.1926e+00_rb,3.3487e+00_rb,3.5171e+00_rb/)
      kbo(:,27, 9) = (/ &
     &2.6651e+00_rb,2.7860e+00_rb,2.9417e+00_rb,3.1050e+00_rb,3.2838e+00_rb/)
      kbo(:,28, 9) = (/ &
     &2.4547e+00_rb,2.5852e+00_rb,2.7243e+00_rb,2.8973e+00_rb,3.0879e+00_rb/)
      kbo(:,29, 9) = (/ &
     &2.2775e+00_rb,2.4144e+00_rb,2.5520e+00_rb,2.7166e+00_rb,2.9171e+00_rb/)
      kbo(:,30, 9) = (/ &
     &2.1281e+00_rb,2.2766e+00_rb,2.4266e+00_rb,2.5792e+00_rb,2.7778e+00_rb/)
      kbo(:,31, 9) = (/ &
     &1.9982e+00_rb,2.1596e+00_rb,2.3215e+00_rb,2.4758e+00_rb,2.6690e+00_rb/)
      kbo(:,32, 9) = (/ &
     &1.8974e+00_rb,2.0696e+00_rb,2.2391e+00_rb,2.4103e+00_rb,2.6183e+00_rb/)
      kbo(:,33, 9) = (/ &
     &1.8191e+00_rb,1.9994e+00_rb,2.1809e+00_rb,2.3775e+00_rb,2.6019e+00_rb/)
      kbo(:,34, 9) = (/ &
     &1.7674e+00_rb,1.9617e+00_rb,2.1590e+00_rb,2.3767e+00_rb,2.6194e+00_rb/)
      kbo(:,35, 9) = (/ &
     &1.7160e+00_rb,1.9253e+00_rb,2.1379e+00_rb,2.3725e+00_rb,2.6356e+00_rb/)
      kbo(:,36, 9) = (/ &
     &1.6660e+00_rb,1.8913e+00_rb,2.1169e+00_rb,2.3615e+00_rb,2.6378e+00_rb/)
      kbo(:,37, 9) = (/ &
     &1.5784e+00_rb,1.8090e+00_rb,2.0343e+00_rb,2.2860e+00_rb,2.5707e+00_rb/)
      kbo(:,38, 9) = (/ &
     &1.4967e+00_rb,1.7299e+00_rb,1.9619e+00_rb,2.2122e+00_rb,2.4970e+00_rb/)
      kbo(:,39, 9) = (/ &
     &1.4275e+00_rb,1.6566e+00_rb,1.8885e+00_rb,2.1422e+00_rb,2.4238e+00_rb/)
      kbo(:,40, 9) = (/ &
     &1.3319e+00_rb,1.5527e+00_rb,1.7867e+00_rb,2.0342e+00_rb,2.3084e+00_rb/)
      kbo(:,41, 9) = (/ &
     &1.2324e+00_rb,1.4488e+00_rb,1.6798e+00_rb,1.9260e+00_rb,2.1920e+00_rb/)
      kbo(:,42, 9) = (/ &
     &1.1393e+00_rb,1.3560e+00_rb,1.5786e+00_rb,1.8237e+00_rb,2.0822e+00_rb/)
      kbo(:,43, 9) = (/ &
     &1.0393e+00_rb,1.2480e+00_rb,1.4586e+00_rb,1.7011e+00_rb,1.9564e+00_rb/)
      kbo(:,44, 9) = (/ &
     &9.4004e-01_rb,1.1318e+00_rb,1.3401e+00_rb,1.5753e+00_rb,1.8301e+00_rb/)
      kbo(:,45, 9) = (/ &
     &8.4677e-01_rb,1.0287e+00_rb,1.2277e+00_rb,1.4538e+00_rb,1.7017e+00_rb/)
      kbo(:,46, 9) = (/ &
     &7.5668e-01_rb,9.2772e-01_rb,1.1175e+00_rb,1.3351e+00_rb,1.5740e+00_rb/)
      kbo(:,47, 9) = (/ &
     &6.6952e-01_rb,8.3230e-01_rb,1.0096e+00_rb,1.2100e+00_rb,1.4418e+00_rb/)
      kbo(:,48, 9) = (/ &
     &5.9179e-01_rb,7.4047e-01_rb,9.0922e-01_rb,1.0966e+00_rb,1.3183e+00_rb/)
      kbo(:,49, 9) = (/ &
     &5.2110e-01_rb,6.5712e-01_rb,8.1623e-01_rb,9.9075e-01_rb,1.1944e+00_rb/)
      kbo(:,50, 9) = (/ &
     &4.6158e-01_rb,5.8589e-01_rb,7.3680e-01_rb,8.9883e-01_rb,1.0896e+00_rb/)
      kbo(:,51, 9) = (/ &
     &4.0970e-01_rb,5.2471e-01_rb,6.6511e-01_rb,8.1893e-01_rb,1.0001e+00_rb/)
      kbo(:,52, 9) = (/ &
     &3.6376e-01_rb,4.6797e-01_rb,5.9787e-01_rb,7.4696e-01_rb,9.1589e-01_rb/)
      kbo(:,53, 9) = (/ &
     &3.2051e-01_rb,4.1628e-01_rb,5.3506e-01_rb,6.7993e-01_rb,8.3743e-01_rb/)
      kbo(:,54, 9) = (/ &
     &2.8466e-01_rb,3.7491e-01_rb,4.8412e-01_rb,6.2077e-01_rb,7.6925e-01_rb/)
      kbo(:,55, 9) = (/ &
     &2.5367e-01_rb,3.3870e-01_rb,4.3869e-01_rb,5.6524e-01_rb,7.0875e-01_rb/)
      kbo(:,56, 9) = (/ &
     &2.2562e-01_rb,3.0476e-01_rb,3.9766e-01_rb,5.1473e-01_rb,6.5170e-01_rb/)
      kbo(:,57, 9) = (/ &
     &1.9827e-01_rb,2.7217e-01_rb,3.6127e-01_rb,4.6695e-01_rb,5.9679e-01_rb/)
      kbo(:,58, 9) = (/ &
     &1.7439e-01_rb,2.4383e-01_rb,3.2816e-01_rb,4.2523e-01_rb,5.4584e-01_rb/)
      kbo(:,59, 9) = (/ &
     &1.6511e-01_rb,2.3377e-01_rb,3.1581e-01_rb,4.1157e-01_rb,5.2975e-01_rb/)
      kbo(:,13,10) = (/ &
     &4.0695e+01_rb,4.0469e+01_rb,4.0155e+01_rb,4.1544e+01_rb,4.2682e+01_rb/)
      kbo(:,14,10) = (/ &
     &3.9304e+01_rb,3.9060e+01_rb,4.0807e+01_rb,4.1587e+01_rb,4.0953e+01_rb/)
      kbo(:,15,10) = (/ &
     &3.7657e+01_rb,3.9117e+01_rb,3.9307e+01_rb,3.8957e+01_rb,3.9710e+01_rb/)
      kbo(:,16,10) = (/ &
     &3.6278e+01_rb,3.6420e+01_rb,3.6566e+01_rb,3.7322e+01_rb,3.7268e+01_rb/)
      kbo(:,17,10) = (/ &
     &3.3662e+01_rb,3.3973e+01_rb,3.4667e+01_rb,3.4884e+01_rb,3.5251e+01_rb/)
      kbo(:,18,10) = (/ &
     &3.1195e+01_rb,3.1946e+01_rb,3.2272e+01_rb,3.2818e+01_rb,3.3898e+01_rb/)
      kbo(:,19,10) = (/ &
     &2.9346e+01_rb,2.9678e+01_rb,3.0231e+01_rb,3.1581e+01_rb,3.2542e+01_rb/)
      kbo(:,20,10) = (/ &
     &2.6569e+01_rb,2.7189e+01_rb,2.8571e+01_rb,2.9702e+01_rb,3.0461e+01_rb/)
      kbo(:,21,10) = (/ &
     &2.3800e+01_rb,2.5133e+01_rb,2.6296e+01_rb,2.7333e+01_rb,2.8348e+01_rb/)
      kbo(:,22,10) = (/ &
     &2.1748e+01_rb,2.2899e+01_rb,2.3947e+01_rb,2.5006e+01_rb,2.6299e+01_rb/)
      kbo(:,23,10) = (/ &
     &1.9685e+01_rb,2.0476e+01_rb,2.1391e+01_rb,2.2690e+01_rb,2.3900e+01_rb/)
      kbo(:,24,10) = (/ &
     &1.7032e+01_rb,1.7826e+01_rb,1.8674e+01_rb,1.9703e+01_rb,2.1087e+01_rb/)
      kbo(:,25,10) = (/ &
     &1.4880e+01_rb,1.5564e+01_rb,1.6390e+01_rb,1.7502e+01_rb,1.8671e+01_rb/)
      kbo(:,26,10) = (/ &
     &1.3363e+01_rb,1.4020e+01_rb,1.5087e+01_rb,1.6178e+01_rb,1.7289e+01_rb/)
      kbo(:,27,10) = (/ &
     &1.1727e+01_rb,1.2959e+01_rb,1.3942e+01_rb,1.5094e+01_rb,1.6463e+01_rb/)
      kbo(:,28,10) = (/ &
     &1.0659e+01_rb,1.1764e+01_rb,1.3018e+01_rb,1.4300e+01_rb,1.5936e+01_rb/)
      kbo(:,29,10) = (/ &
     &9.6581e+00_rb,1.0907e+01_rb,1.2325e+01_rb,1.4028e+01_rb,1.5807e+01_rb/)
      kbo(:,30,10) = (/ &
     &8.9612e+00_rb,1.0145e+01_rb,1.1747e+01_rb,1.3771e+01_rb,1.5726e+01_rb/)
      kbo(:,31,10) = (/ &
     &8.6124e+00_rb,9.7350e+00_rb,1.1421e+01_rb,1.3475e+01_rb,1.5366e+01_rb/)
      kbo(:,32,10) = (/ &
     &8.5911e+00_rb,9.4800e+00_rb,1.1115e+01_rb,1.3010e+01_rb,1.4984e+01_rb/)
      kbo(:,33,10) = (/ &
     &8.6709e+00_rb,9.3343e+00_rb,1.0809e+01_rb,1.2622e+01_rb,1.4676e+01_rb/)
      kbo(:,34,10) = (/ &
     &8.8793e+00_rb,9.5541e+00_rb,1.0639e+01_rb,1.2461e+01_rb,1.4700e+01_rb/)
      kbo(:,35,10) = (/ &
     &9.1036e+00_rb,9.7617e+00_rb,1.0550e+01_rb,1.2303e+01_rb,1.4597e+01_rb/)
      kbo(:,36,10) = (/ &
     &9.1500e+00_rb,9.8472e+00_rb,1.0675e+01_rb,1.1920e+01_rb,1.4033e+01_rb/)
      kbo(:,37,10) = (/ &
     &8.9913e+00_rb,9.7513e+00_rb,1.0626e+01_rb,1.1848e+01_rb,1.3311e+01_rb/)
      kbo(:,38,10) = (/ &
     &8.8541e+00_rb,9.6133e+00_rb,1.0479e+01_rb,1.1805e+01_rb,1.2978e+01_rb/)
      kbo(:,39,10) = (/ &
     &8.6355e+00_rb,9.4791e+00_rb,1.0441e+01_rb,1.1774e+01_rb,1.2974e+01_rb/)
      kbo(:,40,10) = (/ &
     &8.2572e+00_rb,9.1900e+00_rb,1.0103e+01_rb,1.1340e+01_rb,1.2612e+01_rb/)
      kbo(:,41,10) = (/ &
     &7.9700e+00_rb,8.8564e+00_rb,9.7227e+00_rb,1.0892e+01_rb,1.2191e+01_rb/)
      kbo(:,42,10) = (/ &
     &7.7269e+00_rb,8.5147e+00_rb,9.3572e+00_rb,1.0449e+01_rb,1.1769e+01_rb/)
      kbo(:,43,10) = (/ &
     &7.3457e+00_rb,8.1022e+00_rb,8.9870e+00_rb,9.9755e+00_rb,1.1267e+01_rb/)
      kbo(:,44,10) = (/ &
     &6.8559e+00_rb,7.7825e+00_rb,8.6085e+00_rb,9.5618e+00_rb,1.0705e+01_rb/)
      kbo(:,45,10) = (/ &
     &6.3680e+00_rb,7.3808e+00_rb,8.2404e+00_rb,9.1444e+00_rb,1.0228e+01_rb/)
      kbo(:,46,10) = (/ &
     &5.9178e+00_rb,6.9688e+00_rb,7.8931e+00_rb,8.8310e+00_rb,9.8114e+00_rb/)
      kbo(:,47,10) = (/ &
     &5.4530e+00_rb,6.4924e+00_rb,7.5375e+00_rb,8.5314e+00_rb,9.4745e+00_rb/)
      kbo(:,48,10) = (/ &
     &4.9630e+00_rb,6.0598e+00_rb,7.0862e+00_rb,8.1895e+00_rb,9.1462e+00_rb/)
      kbo(:,49,10) = (/ &
     &4.4814e+00_rb,5.6082e+00_rb,6.6461e+00_rb,7.8385e+00_rb,8.8792e+00_rb/)
      kbo(:,50,10) = (/ &
     &4.0666e+00_rb,5.1497e+00_rb,6.2742e+00_rb,7.4367e+00_rb,8.5539e+00_rb/)
      kbo(:,51,10) = (/ &
     &3.6813e+00_rb,4.7093e+00_rb,5.9040e+00_rb,7.0217e+00_rb,8.2062e+00_rb/)
      kbo(:,52,10) = (/ &
     &3.3213e+00_rb,4.2985e+00_rb,5.4430e+00_rb,6.5658e+00_rb,7.7730e+00_rb/)
      kbo(:,53,10) = (/ &
     &2.9730e+00_rb,3.8940e+00_rb,5.0012e+00_rb,6.1367e+00_rb,7.2957e+00_rb/)
      kbo(:,54,10) = (/ &
     &2.6494e+00_rb,3.5088e+00_rb,4.5590e+00_rb,5.7479e+00_rb,6.8609e+00_rb/)
      kbo(:,55,10) = (/ &
     &2.3517e+00_rb,3.1542e+00_rb,4.1525e+00_rb,5.3587e+00_rb,6.4637e+00_rb/)
      kbo(:,56,10) = (/ &
     &2.0870e+00_rb,2.8232e+00_rb,3.7684e+00_rb,4.9288e+00_rb,6.0659e+00_rb/)
      kbo(:,57,10) = (/ &
     &1.8716e+00_rb,2.5211e+00_rb,3.3862e+00_rb,4.4963e+00_rb,5.6364e+00_rb/)
      kbo(:,58,10) = (/ &
     &1.6996e+00_rb,2.2494e+00_rb,3.0457e+00_rb,4.0946e+00_rb,5.2813e+00_rb/)
      kbo(:,59,10) = (/ &
     &1.6125e+00_rb,2.1057e+00_rb,2.8675e+00_rb,3.8607e+00_rb,5.0173e+00_rb/)
      kbo(:,13,11) = (/ &
     &5.4280e+01_rb,5.5937e+01_rb,5.6472e+01_rb,5.6083e+01_rb,5.6645e+01_rb/)
      kbo(:,14,11) = (/ &
     &5.6104e+01_rb,5.7569e+01_rb,5.6325e+01_rb,5.6970e+01_rb,5.8322e+01_rb/)
      kbo(:,15,11) = (/ &
     &5.6157e+01_rb,5.5505e+01_rb,5.6534e+01_rb,5.8680e+01_rb,5.8495e+01_rb/)
      kbo(:,16,11) = (/ &
     &5.4215e+01_rb,5.5590e+01_rb,5.7381e+01_rb,5.6844e+01_rb,5.6502e+01_rb/)
      kbo(:,17,11) = (/ &
     &5.3917e+01_rb,5.5209e+01_rb,5.4751e+01_rb,5.5176e+01_rb,5.5535e+01_rb/)
      kbo(:,18,11) = (/ &
     &5.2925e+01_rb,5.3017e+01_rb,5.3849e+01_rb,5.4671e+01_rb,5.5470e+01_rb/)
      kbo(:,19,11) = (/ &
     &5.0412e+01_rb,5.1474e+01_rb,5.2761e+01_rb,5.3623e+01_rb,5.4522e+01_rb/)
      kbo(:,20,11) = (/ &
     &4.7799e+01_rb,4.9437e+01_rb,5.0594e+01_rb,5.1515e+01_rb,5.3379e+01_rb/)
      kbo(:,21,11) = (/ &
     &4.5510e+01_rb,4.6838e+01_rb,4.7923e+01_rb,4.9803e+01_rb,5.2087e+01_rb/)
      kbo(:,22,11) = (/ &
     &4.2197e+01_rb,4.3528e+01_rb,4.5703e+01_rb,4.8296e+01_rb,5.0603e+01_rb/)
      kbo(:,23,11) = (/ &
     &3.8776e+01_rb,4.1167e+01_rb,4.4015e+01_rb,4.6341e+01_rb,4.9195e+01_rb/)
      kbo(:,24,11) = (/ &
     &3.6077e+01_rb,3.8799e+01_rb,4.1478e+01_rb,4.4468e+01_rb,4.7105e+01_rb/)
      kbo(:,25,11) = (/ &
     &3.3423e+01_rb,3.6288e+01_rb,3.9007e+01_rb,4.1836e+01_rb,4.5174e+01_rb/)
      kbo(:,26,11) = (/ &
     &3.0575e+01_rb,3.3241e+01_rb,3.5927e+01_rb,3.9133e+01_rb,4.2785e+01_rb/)
      kbo(:,27,11) = (/ &
     &2.7764e+01_rb,3.0312e+01_rb,3.3054e+01_rb,3.6826e+01_rb,4.0509e+01_rb/)
      kbo(:,28,11) = (/ &
     &2.5413e+01_rb,2.8045e+01_rb,3.1419e+01_rb,3.5118e+01_rb,3.8747e+01_rb/)
      kbo(:,29,11) = (/ &
     &2.4096e+01_rb,2.6980e+01_rb,3.0700e+01_rb,3.4243e+01_rb,3.7699e+01_rb/)
      kbo(:,30,11) = (/ &
     &2.3140e+01_rb,2.6541e+01_rb,2.9953e+01_rb,3.3565e+01_rb,3.7298e+01_rb/)
      kbo(:,31,11) = (/ &
     &2.2574e+01_rb,2.6228e+01_rb,2.9925e+01_rb,3.3653e+01_rb,3.8032e+01_rb/)
      kbo(:,32,11) = (/ &
     &2.2078e+01_rb,2.6231e+01_rb,3.0164e+01_rb,3.4342e+01_rb,3.9067e+01_rb/)
      kbo(:,33,11) = (/ &
     &2.1802e+01_rb,2.6392e+01_rb,3.0637e+01_rb,3.5356e+01_rb,4.0173e+01_rb/)
      kbo(:,34,11) = (/ &
     &2.1886e+01_rb,2.6523e+01_rb,3.1557e+01_rb,3.6504e+01_rb,4.1183e+01_rb/)
      kbo(:,35,11) = (/ &
     &2.1287e+01_rb,2.6137e+01_rb,3.1875e+01_rb,3.7051e+01_rb,4.1397e+01_rb/)
      kbo(:,36,11) = (/ &
     &2.0396e+01_rb,2.5293e+01_rb,3.0920e+01_rb,3.6387e+01_rb,4.1546e+01_rb/)
      kbo(:,37,11) = (/ &
     &1.9398e+01_rb,2.4195e+01_rb,2.9734e+01_rb,3.5138e+01_rb,4.1598e+01_rb/)
      kbo(:,38,11) = (/ &
     &1.8580e+01_rb,2.3161e+01_rb,2.8626e+01_rb,3.3912e+01_rb,4.1158e+01_rb/)
      kbo(:,39,11) = (/ &
     &1.8482e+01_rb,2.2205e+01_rb,2.7504e+01_rb,3.3104e+01_rb,4.0527e+01_rb/)
      kbo(:,40,11) = (/ &
     &1.7984e+01_rb,2.0750e+01_rb,2.5770e+01_rb,3.1669e+01_rb,3.8818e+01_rb/)
      kbo(:,41,11) = (/ &
     &1.7355e+01_rb,1.9936e+01_rb,2.4013e+01_rb,3.0152e+01_rb,3.7102e+01_rb/)
      kbo(:,42,11) = (/ &
     &1.6681e+01_rb,1.9220e+01_rb,2.2251e+01_rb,2.8625e+01_rb,3.5420e+01_rb/)
      kbo(:,43,11) = (/ &
     &1.5870e+01_rb,1.8435e+01_rb,2.1070e+01_rb,2.6668e+01_rb,3.3375e+01_rb/)
      kbo(:,44,11) = (/ &
     &1.5165e+01_rb,1.7568e+01_rb,2.0082e+01_rb,2.4204e+01_rb,3.1050e+01_rb/)
      kbo(:,45,11) = (/ &
     &1.4453e+01_rb,1.6684e+01_rb,1.9148e+01_rb,2.2404e+01_rb,2.8741e+01_rb/)
      kbo(:,46,11) = (/ &
     &1.3674e+01_rb,1.5793e+01_rb,1.8236e+01_rb,2.1064e+01_rb,2.6017e+01_rb/)
      kbo(:,47,11) = (/ &
     &1.2730e+01_rb,1.4847e+01_rb,1.7129e+01_rb,1.9769e+01_rb,2.3301e+01_rb/)
      kbo(:,48,11) = (/ &
     &1.1966e+01_rb,1.3921e+01_rb,1.6062e+01_rb,1.8570e+01_rb,2.1519e+01_rb/)
      kbo(:,49,11) = (/ &
     &1.1256e+01_rb,1.2947e+01_rb,1.5051e+01_rb,1.7441e+01_rb,2.0164e+01_rb/)
      kbo(:,50,11) = (/ &
     &1.0630e+01_rb,1.2231e+01_rb,1.4125e+01_rb,1.6384e+01_rb,1.9019e+01_rb/)
      kbo(:,51,11) = (/ &
     &9.8987e+00_rb,1.1569e+01_rb,1.3215e+01_rb,1.5489e+01_rb,1.7851e+01_rb/)
      kbo(:,52,11) = (/ &
     &9.1901e+00_rb,1.0969e+01_rb,1.2500e+01_rb,1.4560e+01_rb,1.6807e+01_rb/)
      kbo(:,53,11) = (/ &
     &8.5518e+00_rb,1.0279e+01_rb,1.1856e+01_rb,1.3672e+01_rb,1.5980e+01_rb/)
      kbo(:,54,11) = (/ &
     &7.8898e+00_rb,9.6237e+00_rb,1.1322e+01_rb,1.2850e+01_rb,1.5109e+01_rb/)
      kbo(:,55,11) = (/ &
     &7.2910e+00_rb,9.0794e+00_rb,1.0802e+01_rb,1.2222e+01_rb,1.4316e+01_rb/)
      kbo(:,56,11) = (/ &
     &6.6231e+00_rb,8.4503e+00_rb,1.0147e+01_rb,1.1641e+01_rb,1.3475e+01_rb/)
      kbo(:,57,11) = (/ &
     &5.9797e+00_rb,7.8956e+00_rb,9.5656e+00_rb,1.1172e+01_rb,1.2770e+01_rb/)
      kbo(:,58,11) = (/ &
     &5.3792e+00_rb,7.3070e+00_rb,9.0406e+00_rb,1.0661e+01_rb,1.2099e+01_rb/)
      kbo(:,59,11) = (/ &
     &5.1187e+00_rb,7.0446e+00_rb,8.7908e+00_rb,1.0434e+01_rb,1.1861e+01_rb/)
      kbo(:,13,12) = (/ &
     &8.5594e+01_rb,8.5081e+01_rb,8.4548e+01_rb,8.3781e+01_rb,8.2265e+01_rb/)
      kbo(:,14,12) = (/ &
     &8.5160e+01_rb,8.5106e+01_rb,8.5561e+01_rb,8.4404e+01_rb,8.6474e+01_rb/)
      kbo(:,15,12) = (/ &
     &8.6225e+01_rb,8.6934e+01_rb,8.6406e+01_rb,8.7005e+01_rb,8.9359e+01_rb/)
      kbo(:,16,12) = (/ &
     &8.5430e+01_rb,8.5513e+01_rb,8.6639e+01_rb,9.0490e+01_rb,9.3290e+01_rb/)
      kbo(:,17,12) = (/ &
     &8.2443e+01_rb,8.4907e+01_rb,8.8384e+01_rb,9.0076e+01_rb,9.1263e+01_rb/)
      kbo(:,18,12) = (/ &
     &8.5149e+01_rb,8.7540e+01_rb,8.9291e+01_rb,9.0823e+01_rb,9.2775e+01_rb/)
      kbo(:,19,12) = (/ &
     &8.8628e+01_rb,9.0598e+01_rb,9.2322e+01_rb,9.4441e+01_rb,9.6854e+01_rb/)
      kbo(:,20,12) = (/ &
     &8.9356e+01_rb,9.1085e+01_rb,9.4127e+01_rb,9.6914e+01_rb,9.9051e+01_rb/)
      kbo(:,21,12) = (/ &
     &8.7608e+01_rb,9.1521e+01_rb,9.5537e+01_rb,9.7630e+01_rb,9.9135e+01_rb/)
      kbo(:,22,12) = (/ &
     &8.6703e+01_rb,9.1359e+01_rb,9.3725e+01_rb,9.6097e+01_rb,9.8620e+01_rb/)
      kbo(:,23,12) = (/ &
     &8.4986e+01_rb,8.8658e+01_rb,9.1547e+01_rb,9.4897e+01_rb,9.8884e+01_rb/)
      kbo(:,24,12) = (/ &
     &8.1861e+01_rb,8.5495e+01_rb,8.9569e+01_rb,9.4335e+01_rb,9.9177e+01_rb/)
      kbo(:,25,12) = (/ &
     &7.8144e+01_rb,8.2621e+01_rb,8.8221e+01_rb,9.4555e+01_rb,9.9082e+01_rb/)
      kbo(:,26,12) = (/ &
     &7.5558e+01_rb,8.1550e+01_rb,8.8515e+01_rb,9.4800e+01_rb,1.0018e+02_rb/)
      kbo(:,27,12) = (/ &
     &7.4385e+01_rb,8.1401e+01_rb,8.8755e+01_rb,9.4867e+01_rb,9.9863e+01_rb/)
      kbo(:,28,12) = (/ &
     &7.2786e+01_rb,8.0630e+01_rb,8.7375e+01_rb,9.3177e+01_rb,9.7577e+01_rb/)
      kbo(:,29,12) = (/ &
     &7.2494e+01_rb,7.9832e+01_rb,8.6004e+01_rb,9.1472e+01_rb,9.7317e+01_rb/)
      kbo(:,30,12) = (/ &
     &7.1849e+01_rb,7.8701e+01_rb,8.4991e+01_rb,9.0891e+01_rb,9.7338e+01_rb/)
      kbo(:,31,12) = (/ &
     &7.1241e+01_rb,7.7805e+01_rb,8.4010e+01_rb,9.1171e+01_rb,1.0059e+02_rb/)
      kbo(:,32,12) = (/ &
     &7.0698e+01_rb,7.7293e+01_rb,8.4547e+01_rb,9.6001e+01_rb,1.0686e+02_rb/)
      kbo(:,33,12) = (/ &
     &7.0154e+01_rb,7.8613e+01_rb,9.0857e+01_rb,1.0245e+02_rb,1.1312e+02_rb/)
      kbo(:,34,12) = (/ &
     &7.1031e+01_rb,8.3561e+01_rb,9.6187e+01_rb,1.0815e+02_rb,1.1901e+02_rb/)
      kbo(:,35,12) = (/ &
     &7.5071e+01_rb,8.8162e+01_rb,1.0114e+02_rb,1.1296e+02_rb,1.2569e+02_rb/)
      kbo(:,36,12) = (/ &
     &7.8029e+01_rb,9.1692e+01_rb,1.0504e+02_rb,1.1849e+02_rb,1.3079e+02_rb/)
      kbo(:,37,12) = (/ &
     &7.7450e+01_rb,9.1080e+01_rb,1.0499e+02_rb,1.1872e+02_rb,1.3083e+02_rb/)
      kbo(:,38,12) = (/ &
     &7.6390e+01_rb,9.0295e+01_rb,1.0451e+02_rb,1.1896e+02_rb,1.3094e+02_rb/)
      kbo(:,39,12) = (/ &
     &7.4846e+01_rb,8.9967e+01_rb,1.0468e+02_rb,1.1884e+02_rb,1.3087e+02_rb/)
      kbo(:,40,12) = (/ &
     &7.0636e+01_rb,8.6765e+01_rb,1.0145e+02_rb,1.1588e+02_rb,1.2805e+02_rb/)
      kbo(:,41,12) = (/ &
     &6.6305e+01_rb,8.2647e+01_rb,9.8224e+01_rb,1.1219e+02_rb,1.2504e+02_rb/)
      kbo(:,42,12) = (/ &
     &6.2198e+01_rb,7.8258e+01_rb,9.4811e+01_rb,1.0862e+02_rb,1.2155e+02_rb/)
      kbo(:,43,12) = (/ &
     &5.7303e+01_rb,7.2832e+01_rb,8.9604e+01_rb,1.0357e+02_rb,1.1674e+02_rb/)
      kbo(:,44,12) = (/ &
     &5.2689e+01_rb,6.6907e+01_rb,8.3558e+01_rb,9.8406e+01_rb,1.1125e+02_rb/)
      kbo(:,45,12) = (/ &
     &4.8182e+01_rb,6.1264e+01_rb,7.7478e+01_rb,9.3228e+01_rb,1.0590e+02_rb/)
      kbo(:,46,12) = (/ &
     &4.3386e+01_rb,5.6258e+01_rb,7.0871e+01_rb,8.6833e+01_rb,1.0102e+02_rb/)
      kbo(:,47,12) = (/ &
     &3.8704e+01_rb,5.0818e+01_rb,6.4039e+01_rb,7.9666e+01_rb,9.5229e+01_rb/)
      kbo(:,48,12) = (/ &
     &3.3968e+01_rb,4.5364e+01_rb,5.8393e+01_rb,7.2512e+01_rb,8.8396e+01_rb/)
      kbo(:,49,12) = (/ &
     &2.9879e+01_rb,4.0583e+01_rb,5.2774e+01_rb,6.5752e+01_rb,8.1056e+01_rb/)
      kbo(:,50,12) = (/ &
     &2.7602e+01_rb,3.6039e+01_rb,4.7664e+01_rb,6.0460e+01_rb,7.4304e+01_rb/)
      kbo(:,51,12) = (/ &
     &2.6051e+01_rb,3.1889e+01_rb,4.2974e+01_rb,5.5102e+01_rb,6.8169e+01_rb/)
      kbo(:,52,12) = (/ &
     &2.4438e+01_rb,2.8876e+01_rb,3.8469e+01_rb,5.0102e+01_rb,6.2912e+01_rb/)
      kbo(:,53,12) = (/ &
     &2.2776e+01_rb,2.7295e+01_rb,3.4271e+01_rb,4.5285e+01_rb,5.7384e+01_rb/)
      kbo(:,54,12) = (/ &
     &2.1249e+01_rb,2.5821e+01_rb,3.0686e+01_rb,4.1216e+01_rb,5.2669e+01_rb/)
      kbo(:,55,12) = (/ &
     &2.0155e+01_rb,2.4296e+01_rb,2.8676e+01_rb,3.7338e+01_rb,4.8393e+01_rb/)
      kbo(:,56,12) = (/ &
     &1.8949e+01_rb,2.2808e+01_rb,2.7284e+01_rb,3.3732e+01_rb,4.4387e+01_rb/)
      kbo(:,57,12) = (/ &
     &1.7651e+01_rb,2.1401e+01_rb,2.5896e+01_rb,3.0538e+01_rb,4.0399e+01_rb/)
      kbo(:,58,12) = (/ &
     &1.6454e+01_rb,2.0257e+01_rb,2.4400e+01_rb,2.8791e+01_rb,3.6888e+01_rb/)
      kbo(:,59,12) = (/ &
     &1.6078e+01_rb,1.9650e+01_rb,2.3686e+01_rb,2.8130e+01_rb,3.5337e+01_rb/)
      kbo(:,13,13) = (/ &
     &1.6179e+02_rb,1.5851e+02_rb,1.5620e+02_rb,1.5532e+02_rb,1.5316e+02_rb/)
      kbo(:,14,13) = (/ &
     &1.7476e+02_rb,1.7156e+02_rb,1.7068e+02_rb,1.6952e+02_rb,1.6413e+02_rb/)
      kbo(:,15,13) = (/ &
     &1.8520e+02_rb,1.8372e+02_rb,1.8384e+02_rb,1.8019e+02_rb,1.7522e+02_rb/)
      kbo(:,16,13) = (/ &
     &1.9454e+02_rb,1.9542e+02_rb,1.9248e+02_rb,1.8773e+02_rb,1.8813e+02_rb/)
      kbo(:,17,13) = (/ &
     &2.0028e+02_rb,1.9793e+02_rb,1.9596e+02_rb,1.9785e+02_rb,2.0205e+02_rb/)
      kbo(:,18,13) = (/ &
     &1.9037e+02_rb,1.9170e+02_rb,1.9555e+02_rb,2.0164e+02_rb,2.0309e+02_rb/)
      kbo(:,19,13) = (/ &
     &1.7958e+02_rb,1.8623e+02_rb,1.9196e+02_rb,1.9543e+02_rb,1.9670e+02_rb/)
      kbo(:,20,13) = (/ &
     &1.7423e+02_rb,1.8217e+02_rb,1.8721e+02_rb,1.9064e+02_rb,1.9763e+02_rb/)
      kbo(:,21,13) = (/ &
     &1.7205e+02_rb,1.7751e+02_rb,1.8259e+02_rb,1.9102e+02_rb,1.9985e+02_rb/)
      kbo(:,22,13) = (/ &
     &1.7310e+02_rb,1.7837e+02_rb,1.8681e+02_rb,1.9582e+02_rb,2.0409e+02_rb/)
      kbo(:,23,13) = (/ &
     &1.7691e+02_rb,1.8432e+02_rb,1.9387e+02_rb,2.0103e+02_rb,2.1026e+02_rb/)
      kbo(:,24,13) = (/ &
     &1.8088e+02_rb,1.8942e+02_rb,1.9891e+02_rb,2.0699e+02_rb,2.1646e+02_rb/)
      kbo(:,25,13) = (/ &
     &1.8492e+02_rb,1.9434e+02_rb,2.0408e+02_rb,2.1437e+02_rb,2.2277e+02_rb/)
      kbo(:,26,13) = (/ &
     &1.8625e+02_rb,1.9837e+02_rb,2.0945e+02_rb,2.1941e+02_rb,2.3087e+02_rb/)
      kbo(:,27,13) = (/ &
     &1.9010e+02_rb,2.0364e+02_rb,2.1615e+02_rb,2.2715e+02_rb,2.4079e+02_rb/)
      kbo(:,28,13) = (/ &
     &1.9712e+02_rb,2.1090e+02_rb,2.2463e+02_rb,2.3844e+02_rb,2.5439e+02_rb/)
      kbo(:,29,13) = (/ &
     &2.0453e+02_rb,2.1990e+02_rb,2.3570e+02_rb,2.5241e+02_rb,2.6509e+02_rb/)
      kbo(:,30,13) = (/ &
     &2.1308e+02_rb,2.3113e+02_rb,2.4908e+02_rb,2.6461e+02_rb,2.7735e+02_rb/)
      kbo(:,31,13) = (/ &
     &2.2395e+02_rb,2.4239e+02_rb,2.6296e+02_rb,2.7710e+02_rb,2.8754e+02_rb/)
      kbo(:,32,13) = (/ &
     &2.3649e+02_rb,2.5891e+02_rb,2.7696e+02_rb,2.8809e+02_rb,2.9409e+02_rb/)
      kbo(:,33,13) = (/ &
     &2.5295e+02_rb,2.7352e+02_rb,2.8553e+02_rb,2.9364e+02_rb,3.0036e+02_rb/)
      kbo(:,34,13) = (/ &
     &2.6621e+02_rb,2.8028e+02_rb,2.9209e+02_rb,2.9925e+02_rb,3.0905e+02_rb/)
      kbo(:,35,13) = (/ &
     &2.7274e+02_rb,2.8720e+02_rb,2.9678e+02_rb,3.0598e+02_rb,3.1573e+02_rb/)
      kbo(:,36,13) = (/ &
     &2.7668e+02_rb,2.9126e+02_rb,3.0131e+02_rb,3.1293e+02_rb,3.2456e+02_rb/)
      kbo(:,37,13) = (/ &
     &2.7566e+02_rb,2.9125e+02_rb,3.0241e+02_rb,3.1513e+02_rb,3.2636e+02_rb/)
      kbo(:,38,13) = (/ &
     &2.7524e+02_rb,2.9197e+02_rb,3.0411e+02_rb,3.1688e+02_rb,3.2854e+02_rb/)
      kbo(:,39,13) = (/ &
     &2.7502e+02_rb,2.9241e+02_rb,3.0488e+02_rb,3.1825e+02_rb,3.3122e+02_rb/)
      kbo(:,40,13) = (/ &
     &2.7103e+02_rb,2.8939e+02_rb,3.0276e+02_rb,3.1510e+02_rb,3.2909e+02_rb/)
      kbo(:,41,13) = (/ &
     &2.6612e+02_rb,2.8502e+02_rb,2.9924e+02_rb,3.1164e+02_rb,3.2628e+02_rb/)
      kbo(:,42,13) = (/ &
     &2.6020e+02_rb,2.7999e+02_rb,2.9554e+02_rb,3.0769e+02_rb,3.2142e+02_rb/)
      kbo(:,43,13) = (/ &
     &2.5242e+02_rb,2.7289e+02_rb,2.9079e+02_rb,3.0394e+02_rb,3.1657e+02_rb/)
      kbo(:,44,13) = (/ &
     &2.4141e+02_rb,2.6492e+02_rb,2.8389e+02_rb,2.9851e+02_rb,3.1003e+02_rb/)
      kbo(:,45,13) = (/ &
     &2.3199e+02_rb,2.5697e+02_rb,2.7534e+02_rb,2.9238e+02_rb,3.0488e+02_rb/)
      kbo(:,46,13) = (/ &
     &2.2099e+02_rb,2.4628e+02_rb,2.6862e+02_rb,2.8625e+02_rb,2.9891e+02_rb/)
      kbo(:,47,13) = (/ &
     &2.0813e+02_rb,2.3515e+02_rb,2.5936e+02_rb,2.7698e+02_rb,2.9245e+02_rb/)
      kbo(:,48,13) = (/ &
     &1.9611e+02_rb,2.2305e+02_rb,2.4767e+02_rb,2.6948e+02_rb,2.8622e+02_rb/)
      kbo(:,49,13) = (/ &
     &1.8313e+02_rb,2.1047e+02_rb,2.3654e+02_rb,2.6025e+02_rb,2.7775e+02_rb/)
      kbo(:,50,13) = (/ &
     &1.6903e+02_rb,1.9938e+02_rb,2.2519e+02_rb,2.4907e+02_rb,2.7015e+02_rb/)
      kbo(:,51,13) = (/ &
     &1.5421e+02_rb,1.8800e+02_rb,2.1428e+02_rb,2.3876e+02_rb,2.6215e+02_rb/)
      kbo(:,52,13) = (/ &
     &1.4056e+02_rb,1.7488e+02_rb,2.0358e+02_rb,2.2873e+02_rb,2.5153e+02_rb/)
      kbo(:,53,13) = (/ &
     &1.2774e+02_rb,1.6089e+02_rb,1.9159e+02_rb,2.1758e+02_rb,2.4063e+02_rb/)
      kbo(:,54,13) = (/ &
     &1.1505e+02_rb,1.4830e+02_rb,1.8148e+02_rb,2.0706e+02_rb,2.3185e+02_rb/)
      kbo(:,55,13) = (/ &
     &1.0455e+02_rb,1.3680e+02_rb,1.7009e+02_rb,1.9749e+02_rb,2.2149e+02_rb/)
      kbo(:,56,13) = (/ &
     &9.5380e+01_rb,1.2458e+02_rb,1.5763e+02_rb,1.8850e+02_rb,2.1165e+02_rb/)
      kbo(:,57,13) = (/ &
     &8.6328e+01_rb,1.1370e+02_rb,1.4580e+02_rb,1.7843e+02_rb,2.0341e+02_rb/)
      kbo(:,58,13) = (/ &
     &7.7503e+01_rb,1.0463e+02_rb,1.3405e+02_rb,1.6684e+02_rb,1.9463e+02_rb/)
      kbo(:,59,13) = (/ &
     &7.3613e+01_rb,1.0087e+02_rb,1.2946e+02_rb,1.6169e+02_rb,1.9043e+02_rb/)
      kbo(:,13,14) = (/ &
     &2.3384e+02_rb,2.3134e+02_rb,2.2678e+02_rb,2.2105e+02_rb,2.2130e+02_rb/)
      kbo(:,14,14) = (/ &
     &2.6370e+02_rb,2.6091e+02_rb,2.5467e+02_rb,2.5118e+02_rb,2.4954e+02_rb/)
      kbo(:,15,14) = (/ &
     &2.9591e+02_rb,2.9159e+02_rb,2.8464e+02_rb,2.8328e+02_rb,2.7984e+02_rb/)
      kbo(:,16,14) = (/ &
     &3.2944e+02_rb,3.2156e+02_rb,3.1816e+02_rb,3.1521e+02_rb,3.0449e+02_rb/)
      kbo(:,17,14) = (/ &
     &3.5907e+02_rb,3.5209e+02_rb,3.5084e+02_rb,3.4076e+02_rb,3.2553e+02_rb/)
      kbo(:,18,14) = (/ &
     &3.8932e+02_rb,3.8700e+02_rb,3.7610e+02_rb,3.6121e+02_rb,3.5570e+02_rb/)
      kbo(:,19,14) = (/ &
     &4.2093e+02_rb,4.0885e+02_rb,3.9722e+02_rb,3.9095e+02_rb,3.9197e+02_rb/)
      kbo(:,20,14) = (/ &
     &4.4653e+02_rb,4.3247e+02_rb,4.2391e+02_rb,4.2574e+02_rb,4.2089e+02_rb/)
      kbo(:,21,14) = (/ &
     &4.6745e+02_rb,4.5974e+02_rb,4.5782e+02_rb,4.5591e+02_rb,4.4616e+02_rb/)
      kbo(:,22,14) = (/ &
     &4.8396e+02_rb,4.8410e+02_rb,4.8532e+02_rb,4.7987e+02_rb,4.6662e+02_rb/)
      kbo(:,23,14) = (/ &
     &5.0568e+02_rb,5.0696e+02_rb,5.0168e+02_rb,4.9917e+02_rb,4.8477e+02_rb/)
      kbo(:,24,14) = (/ &
     &5.2372e+02_rb,5.2925e+02_rb,5.2568e+02_rb,5.1576e+02_rb,5.0916e+02_rb/)
      kbo(:,25,14) = (/ &
     &5.4773e+02_rb,5.4681e+02_rb,5.4261e+02_rb,5.3467e+02_rb,5.2926e+02_rb/)
      kbo(:,26,14) = (/ &
     &5.6992e+02_rb,5.6525e+02_rb,5.5986e+02_rb,5.5383e+02_rb,5.4189e+02_rb/)
      kbo(:,27,14) = (/ &
     &5.8413e+02_rb,5.7753e+02_rb,5.7399e+02_rb,5.6915e+02_rb,5.5630e+02_rb/)
      kbo(:,28,14) = (/ &
     &5.9410e+02_rb,5.9489e+02_rb,5.8870e+02_rb,5.8276e+02_rb,5.7225e+02_rb/)
      kbo(:,29,14) = (/ &
     &6.0586e+02_rb,6.0400e+02_rb,6.0319e+02_rb,5.9066e+02_rb,5.8348e+02_rb/)
      kbo(:,30,14) = (/ &
     &6.2023e+02_rb,6.2112e+02_rb,6.1269e+02_rb,6.0507e+02_rb,5.8831e+02_rb/)
      kbo(:,31,14) = (/ &
     &6.3249e+02_rb,6.3678e+02_rb,6.2186e+02_rb,6.1145e+02_rb,5.9841e+02_rb/)
      kbo(:,32,14) = (/ &
     &6.5129e+02_rb,6.4085e+02_rb,6.3065e+02_rb,6.1664e+02_rb,6.0677e+02_rb/)
      kbo(:,33,14) = (/ &
     &6.5648e+02_rb,6.4719e+02_rb,6.3483e+02_rb,6.2799e+02_rb,6.1689e+02_rb/)
      kbo(:,34,14) = (/ &
     &6.6261e+02_rb,6.5214e+02_rb,6.4542e+02_rb,6.3852e+02_rb,6.1993e+02_rb/)
      kbo(:,35,14) = (/ &
     &6.7078e+02_rb,6.6309e+02_rb,6.5953e+02_rb,6.4635e+02_rb,6.1863e+02_rb/)
      kbo(:,36,14) = (/ &
     &6.8510e+02_rb,6.7944e+02_rb,6.7467e+02_rb,6.5682e+02_rb,6.1696e+02_rb/)
      kbo(:,37,14) = (/ &
     &6.9465e+02_rb,6.8814e+02_rb,6.8342e+02_rb,6.6410e+02_rb,6.2485e+02_rb/)
      kbo(:,38,14) = (/ &
     &7.0151e+02_rb,6.9441e+02_rb,6.9017e+02_rb,6.7086e+02_rb,6.3205e+02_rb/)
      kbo(:,39,14) = (/ &
     &7.0868e+02_rb,7.0183e+02_rb,6.9685e+02_rb,6.7866e+02_rb,6.3921e+02_rb/)
      kbo(:,40,14) = (/ &
     &7.1075e+02_rb,7.0438e+02_rb,7.0305e+02_rb,6.8866e+02_rb,6.5105e+02_rb/)
      kbo(:,41,14) = (/ &
     &7.0971e+02_rb,7.0837e+02_rb,7.0822e+02_rb,6.9421e+02_rb,6.5990e+02_rb/)
      kbo(:,42,14) = (/ &
     &7.1142e+02_rb,7.1113e+02_rb,7.1018e+02_rb,6.9720e+02_rb,6.7342e+02_rb/)
      kbo(:,43,14) = (/ &
     &7.1363e+02_rb,7.1120e+02_rb,7.0210e+02_rb,7.0055e+02_rb,6.8488e+02_rb/)
      kbo(:,44,14) = (/ &
     &7.1148e+02_rb,7.0646e+02_rb,7.0565e+02_rb,7.0398e+02_rb,6.8877e+02_rb/)
      kbo(:,45,14) = (/ &
     &7.0895e+02_rb,7.0958e+02_rb,7.0605e+02_rb,6.9798e+02_rb,6.9240e+02_rb/)
      kbo(:,46,14) = (/ &
     &6.9950e+02_rb,7.0866e+02_rb,7.0226e+02_rb,6.9819e+02_rb,6.9658e+02_rb/)
      kbo(:,47,14) = (/ &
     &6.8721e+02_rb,7.0244e+02_rb,7.0188e+02_rb,6.9806e+02_rb,6.9161e+02_rb/)
      kbo(:,48,14) = (/ &
     &6.8035e+02_rb,6.9565e+02_rb,6.9976e+02_rb,6.9387e+02_rb,6.8913e+02_rb/)
      kbo(:,49,14) = (/ &
     &6.6852e+02_rb,6.8533e+02_rb,6.9809e+02_rb,6.9470e+02_rb,6.8647e+02_rb/)
      kbo(:,50,14) = (/ &
     &6.5815e+02_rb,6.7824e+02_rb,6.9156e+02_rb,6.9473e+02_rb,6.8677e+02_rb/)
      kbo(:,51,14) = (/ &
     &6.5633e+02_rb,6.6741e+02_rb,6.8334e+02_rb,6.9427e+02_rb,6.8718e+02_rb/)
      kbo(:,52,14) = (/ &
     &6.4799e+02_rb,6.6007e+02_rb,6.7742e+02_rb,6.8641e+02_rb,6.8886e+02_rb/)
      kbo(:,53,14) = (/ &
     &6.3478e+02_rb,6.5848e+02_rb,6.6595e+02_rb,6.8004e+02_rb,6.8977e+02_rb/)
      kbo(:,54,14) = (/ &
     &6.3075e+02_rb,6.5140e+02_rb,6.6048e+02_rb,6.7510e+02_rb,6.7898e+02_rb/)
      kbo(:,55,14) = (/ &
     &6.2053e+02_rb,6.3982e+02_rb,6.5909e+02_rb,6.6252e+02_rb,6.7620e+02_rb/)
      kbo(:,56,14) = (/ &
     &6.0488e+02_rb,6.3692e+02_rb,6.5472e+02_rb,6.5961e+02_rb,6.7144e+02_rb/)
      kbo(:,57,14) = (/ &
     &5.9240e+02_rb,6.2947e+02_rb,6.4320e+02_rb,6.5881e+02_rb,6.5824e+02_rb/)
      kbo(:,58,14) = (/ &
     &5.7619e+02_rb,6.1537e+02_rb,6.4199e+02_rb,6.5527e+02_rb,6.5777e+02_rb/)
      kbo(:,59,14) = (/ &
     &5.7161e+02_rb,6.1151e+02_rb,6.3977e+02_rb,6.4718e+02_rb,6.5762e+02_rb/)
      kbo(:,13,15) = (/ &
     &3.5083e+02_rb,3.4274e+02_rb,3.3588e+02_rb,3.2964e+02_rb,3.0980e+02_rb/)
      kbo(:,14,15) = (/ &
     &4.0382e+02_rb,3.9443e+02_rb,3.8616e+02_rb,3.7379e+02_rb,3.5621e+02_rb/)
      kbo(:,15,15) = (/ &
     &4.6271e+02_rb,4.5170e+02_rb,4.4204e+02_rb,4.2053e+02_rb,4.0520e+02_rb/)
      kbo(:,16,15) = (/ &
     &5.2768e+02_rb,5.1495e+02_rb,4.9504e+02_rb,4.7377e+02_rb,4.6833e+02_rb/)
      kbo(:,17,15) = (/ &
     &5.9879e+02_rb,5.8378e+02_rb,5.5161e+02_rb,5.3753e+02_rb,5.3799e+02_rb/)
      kbo(:,18,15) = (/ &
     &6.7489e+02_rb,6.3850e+02_rb,6.1688e+02_rb,6.0987e+02_rb,5.7677e+02_rb/)
      kbo(:,19,15) = (/ &
     &7.3779e+02_rb,7.0513e+02_rb,6.9510e+02_rb,6.5830e+02_rb,6.0487e+02_rb/)
      kbo(:,20,15) = (/ &
     &8.1079e+02_rb,7.9490e+02_rb,7.5998e+02_rb,6.9897e+02_rb,6.4843e+02_rb/)
      kbo(:,21,15) = (/ &
     &8.8426e+02_rb,8.6003e+02_rb,8.0304e+02_rb,7.3384e+02_rb,6.9837e+02_rb/)
      kbo(:,22,15) = (/ &
     &9.6812e+02_rb,9.1088e+02_rb,8.3391e+02_rb,7.8330e+02_rb,7.5669e+02_rb/)
      kbo(:,23,15) = (/ &
     &1.0128e+03_rb,9.4218e+02_rb,8.8720e+02_rb,8.2530e+02_rb,7.8833e+02_rb/)
      kbo(:,24,15) = (/ &
     &1.0622e+03_rb,9.7878e+02_rb,9.1791e+02_rb,8.7765e+02_rb,7.9731e+02_rb/)
      kbo(:,25,15) = (/ &
     &1.0853e+03_rb,1.0245e+03_rb,9.5328e+02_rb,8.9340e+02_rb,8.1721e+02_rb/)
      kbo(:,26,15) = (/ &
     &1.1200e+03_rb,1.0458e+03_rb,9.9235e+02_rb,9.0775e+02_rb,8.4493e+02_rb/)
      kbo(:,27,15) = (/ &
     &1.1556e+03_rb,1.0812e+03_rb,1.0090e+03_rb,9.2343e+02_rb,8.1758e+02_rb/)
      kbo(:,28,15) = (/ &
     &1.1865e+03_rb,1.1049e+03_rb,1.0073e+03_rb,9.1620e+02_rb,7.7173e+02_rb/)
      kbo(:,29,15) = (/ &
     &1.1906e+03_rb,1.1065e+03_rb,1.0028e+03_rb,8.7527e+02_rb,7.2650e+02_rb/)
      kbo(:,30,15) = (/ &
     &1.1992e+03_rb,1.0831e+03_rb,9.6830e+02_rb,8.1150e+02_rb,6.8040e+02_rb/)
      kbo(:,31,15) = (/ &
     &1.1794e+03_rb,1.0618e+03_rb,9.0601e+02_rb,7.5400e+02_rb,6.0274e+02_rb/)
      kbo(:,32,15) = (/ &
     &1.1491e+03_rb,9.9644e+02_rb,8.3044e+02_rb,6.7287e+02_rb,5.1890e+02_rb/)
      kbo(:,33,15) = (/ &
     &1.0978e+03_rb,9.1760e+02_rb,7.5228e+02_rb,5.8187e+02_rb,4.2649e+02_rb/)
      kbo(:,34,15) = (/ &
     &1.0237e+03_rb,8.4870e+02_rb,6.6311e+02_rb,4.9111e+02_rb,3.3958e+02_rb/)
      kbo(:,35,15) = (/ &
     &9.6010e+02_rb,7.7009e+02_rb,5.7949e+02_rb,4.1414e+02_rb,2.7668e+02_rb/)
      kbo(:,36,15) = (/ &
     &9.0171e+02_rb,7.0086e+02_rb,5.0886e+02_rb,3.3823e+02_rb,2.2240e+02_rb/)
      kbo(:,37,15) = (/ &
     &9.0464e+02_rb,7.0259e+02_rb,5.0334e+02_rb,3.3120e+02_rb,2.1250e+02_rb/)
      kbo(:,38,15) = (/ &
     &9.1272e+02_rb,7.0777e+02_rb,5.0236e+02_rb,3.2627e+02_rb,2.0281e+02_rb/)
      kbo(:,39,15) = (/ &
     &9.1592e+02_rb,7.0699e+02_rb,5.0029e+02_rb,3.1832e+02_rb,1.8984e+02_rb/)
      kbo(:,40,15) = (/ &
     &9.7111e+02_rb,7.5888e+02_rb,5.4218e+02_rb,3.5289e+02_rb,2.1710e+02_rb/)
      kbo(:,41,15) = (/ &
     &1.0392e+03_rb,8.1334e+02_rb,5.9329e+02_rb,4.0507e+02_rb,2.5679e+02_rb/)
      kbo(:,42,15) = (/ &
     &1.1018e+03_rb,8.7332e+02_rb,6.5360e+02_rb,4.6461e+02_rb,2.9410e+02_rb/)
      kbo(:,43,15) = (/ &
     &1.1780e+03_rb,9.5750e+02_rb,7.5295e+02_rb,5.3640e+02_rb,3.4968e+02_rb/)
      kbo(:,44,15) = (/ &
     &1.2774e+03_rb,1.0610e+03_rb,8.3382e+02_rb,6.1903e+02_rb,4.3881e+02_rb/)
      kbo(:,45,15) = (/ &
     &1.3696e+03_rb,1.1411e+03_rb,9.2825e+02_rb,7.2390e+02_rb,5.2087e+02_rb/)
      kbo(:,46,15) = (/ &
     &1.4458e+03_rb,1.2392e+03_rb,1.0284e+03_rb,8.1462e+02_rb,6.0446e+02_rb/)
      kbo(:,47,15) = (/ &
     &1.5185e+03_rb,1.3558e+03_rb,1.1323e+03_rb,9.2353e+02_rb,7.1923e+02_rb/)
      kbo(:,48,15) = (/ &
     &1.5480e+03_rb,1.4278e+03_rb,1.2433e+03_rb,1.0338e+03_rb,8.2416e+02_rb/)
      kbo(:,49,15) = (/ &
     &1.6321e+03_rb,1.4959e+03_rb,1.3391e+03_rb,1.1340e+03_rb,9.3754e+02_rb/)
      kbo(:,50,15) = (/ &
     &1.7049e+03_rb,1.5268e+03_rb,1.4037e+03_rb,1.2337e+03_rb,1.0324e+03_rb/)
      kbo(:,51,15) = (/ &
     &1.7473e+03_rb,1.6024e+03_rb,1.4662e+03_rb,1.3130e+03_rb,1.1235e+03_rb/)
      kbo(:,52,15) = (/ &
     &1.7742e+03_rb,1.6697e+03_rb,1.4942e+03_rb,1.3753e+03_rb,1.2160e+03_rb/)
      kbo(:,53,15) = (/ &
     &1.8257e+03_rb,1.7098e+03_rb,1.5772e+03_rb,1.4297e+03_rb,1.2866e+03_rb/)
      kbo(:,54,15) = (/ &
     &1.8238e+03_rb,1.7301e+03_rb,1.6290e+03_rb,1.4640e+03_rb,1.3508e+03_rb/)
      kbo(:,55,15) = (/ &
     &1.8598e+03_rb,1.7789e+03_rb,1.6666e+03_rb,1.5389e+03_rb,1.3930e+03_rb/)
      kbo(:,56,15) = (/ &
     &1.9305e+03_rb,1.7834e+03_rb,1.6800e+03_rb,1.5831e+03_rb,1.4252e+03_rb/)
      kbo(:,57,15) = (/ &
     &1.9902e+03_rb,1.8099e+03_rb,1.7296e+03_rb,1.6197e+03_rb,1.4994e+03_rb/)
      kbo(:,58,15) = (/ &
     &2.0569e+03_rb,1.8785e+03_rb,1.7252e+03_rb,1.6299e+03_rb,1.5366e+03_rb/)
      kbo(:,59,15) = (/ &
     &2.0792e+03_rb,1.9024e+03_rb,1.7290e+03_rb,1.6559e+03_rb,1.5493e+03_rb/)
      kbo(:,13,16) = (/ &
     &5.7074e+02_rb,5.5713e+02_rb,5.4253e+02_rb,5.2721e+02_rb,5.1131e+02_rb/)
      kbo(:,14,16) = (/ &
     &6.6774e+02_rb,6.5027e+02_rb,6.3161e+02_rb,6.1261e+02_rb,5.9377e+02_rb/)
      kbo(:,15,16) = (/ &
     &7.7763e+02_rb,7.5535e+02_rb,7.3247e+02_rb,7.0914e+02_rb,6.8554e+02_rb/)
      kbo(:,16,16) = (/ &
     &9.0056e+02_rb,8.7306e+02_rb,8.4456e+02_rb,8.1516e+02_rb,7.0895e+02_rb/)
      kbo(:,17,16) = (/ &
     &1.0366e+03_rb,1.0015e+03_rb,9.6519e+02_rb,8.8128e+02_rb,7.1380e+02_rb/)
      kbo(:,18,16) = (/ &
     &1.1817e+03_rb,1.1369e+03_rb,1.0597e+03_rb,8.5644e+02_rb,7.3707e+02_rb/)
      kbo(:,19,16) = (/ &
     &1.3316e+03_rb,1.2750e+03_rb,1.0321e+03_rb,8.8674e+02_rb,7.7129e+02_rb/)
      kbo(:,20,16) = (/ &
     &1.4803e+03_rb,1.2710e+03_rb,1.0855e+03_rb,9.4408e+02_rb,7.2356e+02_rb/)
      kbo(:,21,16) = (/ &
     &1.6247e+03_rb,1.3272e+03_rb,1.1530e+03_rb,9.8338e+02_rb,6.9198e+02_rb/)
      kbo(:,22,16) = (/ &
     &1.6008e+03_rb,1.3778e+03_rb,1.2081e+03_rb,9.0370e+02_rb,5.5641e+02_rb/)
      kbo(:,23,16) = (/ &
     &1.6342e+03_rb,1.4243e+03_rb,1.1304e+03_rb,8.5528e+02_rb,4.9240e+02_rb/)
      kbo(:,24,16) = (/ &
     &1.6690e+03_rb,1.4128e+03_rb,1.0773e+03_rb,6.8055e+02_rb,4.3204e+02_rb/)
      kbo(:,25,16) = (/ &
     &1.6932e+03_rb,1.3171e+03_rb,9.9569e+02_rb,5.6924e+02_rb,2.7405e+02_rb/)
      kbo(:,26,16) = (/ &
     &1.5997e+03_rb,1.2565e+03_rb,7.2147e+02_rb,4.1508e+02_rb,1.6363e+00_rb/)
      kbo(:,27,16) = (/ &
     &1.4709e+03_rb,1.0620e+03_rb,5.5367e+02_rb,1.6801e+02_rb,1.7711e+00_rb/)
      kbo(:,28,16) = (/ &
     &1.2868e+03_rb,7.4225e+02_rb,4.1091e+02_rb,1.3979e+00_rb,1.9138e+00_rb/)
      kbo(:,29,16) = (/ &
     &1.1046e+03_rb,5.7327e+02_rb,1.1435e+02_rb,1.5227e+00_rb,2.0690e+00_rb/)
      kbo(:,30,16) = (/ &
     &7.5951e+02_rb,3.1070e+02_rb,1.1800e+00_rb,1.6572e+00_rb,2.2234e+00_rb/)
      kbo(:,31,16) = (/ &
     &5.2269e+02_rb,8.9357e-01_rb,1.2993e+00_rb,1.7936e+00_rb,2.3615e+00_rb/)
      kbo(:,32,16) = (/ &
     &1.3504e+02_rb,9.8754e-01_rb,1.4263e+00_rb,1.9479e+00_rb,2.5792e+00_rb/)
      kbo(:,33,16) = (/ &
     &7.3408e-01_rb,1.0937e+00_rb,1.5601e+00_rb,2.1018e+00_rb,2.7689e+00_rb/)
      kbo(:,34,16) = (/ &
     &8.0973e-01_rb,1.1916e+00_rb,1.6719e+00_rb,2.2462e+00_rb,2.9376e+00_rb/)
      kbo(:,35,16) = (/ &
     &8.5624e-01_rb,1.2515e+00_rb,1.7457e+00_rb,2.3340e+00_rb,3.0393e+00_rb/)
      kbo(:,36,16) = (/ &
     &8.6582e-01_rb,1.2640e+00_rb,1.7620e+00_rb,2.3537e+00_rb,3.0624e+00_rb/)
      kbo(:,37,16) = (/ &
     &8.2392e-01_rb,1.2109e+00_rb,1.6970e+00_rb,2.2619e+00_rb,2.9651e+00_rb/)
      kbo(:,38,16) = (/ &
     &7.7820e-01_rb,1.1515e+00_rb,1.6284e+00_rb,2.1818e+00_rb,2.8537e+00_rb/)
      kbo(:,39,16) = (/ &
     &7.3137e-01_rb,1.0905e+00_rb,1.5568e+00_rb,2.1026e+00_rb,2.7617e+00_rb/)
      kbo(:,40,16) = (/ &
     &6.6113e-01_rb,9.9827e-01_rb,1.4324e+00_rb,1.9599e+00_rb,2.5782e+00_rb/)
      kbo(:,41,16) = (/ &
     &5.8979e-01_rb,9.0432e-01_rb,1.3165e+00_rb,1.8211e+00_rb,2.4166e+00_rb/)
      kbo(:,42,16) = (/ &
     &5.2189e-01_rb,8.1262e-01_rb,1.2033e+00_rb,1.6872e+00_rb,2.2584e+00_rb/)
      kbo(:,43,16) = (/ &
     &4.5012e-01_rb,7.1522e-01_rb,1.0693e+00_rb,1.5212e+00_rb,2.0670e+00_rb/)
      kbo(:,44,16) = (/ &
     &3.7946e-01_rb,6.1667e-01_rb,9.4312e-01_rb,1.3624e+00_rb,1.8770e+00_rb/)
      kbo(:,45,16) = (/ &
     &3.1558e-01_rb,5.2623e-01_rb,8.0787e-01_rb,1.2032e+00_rb,1.6863e+00_rb/)
      kbo(:,46,16) = (/ &
     &2.9590e+02_rb,4.4343e-01_rb,7.0619e-01_rb,1.0578e+00_rb,1.5068e+00_rb/)
      kbo(:,47,16) = (/ &
     &7.2659e+02_rb,3.5873e-01_rb,5.9101e-01_rb,9.0599e-01_rb,1.3156e+00_rb/)
      kbo(:,48,16) = (/ &
     &1.3181e+03_rb,3.4672e+02_rb,4.8767e-01_rb,7.6683e-01_rb,1.1370e+00_rb/)
      kbo(:,49,16) = (/ &
     &1.6047e+03_rb,7.6641e+02_rb,7.0071e+01_rb,6.4471e-01_rb,9.8037e-01_rb/)
      kbo(:,50,16) = (/ &
     &1.8848e+03_rb,1.3103e+03_rb,4.1613e+02_rb,5.4100e-01_rb,8.3899e-01_rb/)
      kbo(:,51,16) = (/ &
     &2.2044e+03_rb,1.5848e+03_rb,7.6808e+02_rb,1.1316e+02_rb,7.2276e-01_rb/)
      kbo(:,52,16) = (/ &
     &2.7162e+03_rb,1.8452e+03_rb,1.2957e+03_rb,4.5829e+02_rb,6.1559e-01_rb/)
      kbo(:,53,16) = (/ &
     &3.1107e+03_rb,2.1559e+03_rb,1.5529e+03_rb,8.4689e+02_rb,1.7464e+02_rb/)
      kbo(:,54,16) = (/ &
     &3.6955e+03_rb,2.6522e+03_rb,1.8196e+03_rb,1.2866e+03_rb,5.0615e+02_rb/)
      kbo(:,55,16) = (/ &
     &4.0343e+03_rb,2.9873e+03_rb,2.0751e+03_rb,1.5293e+03_rb,8.5669e+02_rb/)
      kbo(:,56,16) = (/ &
     &4.1921e+03_rb,3.4914e+03_rb,2.5653e+03_rb,1.7745e+03_rb,1.2881e+03_rb/)
      kbo(:,57,16) = (/ &
     &4.3609e+03_rb,3.8712e+03_rb,2.9041e+03_rb,2.0379e+03_rb,1.5211e+03_rb/)
      kbo(:,58,16) = (/ &
     &4.5313e+03_rb,4.0098e+03_rb,3.4243e+03_rb,2.5213e+03_rb,1.7627e+03_rb/)
      kbo(:,59,16) = (/ &
     &4.6034e+03_rb,4.0679e+03_rb,3.6299e+03_rb,2.7016e+03_rb,1.9029e+03_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The third index 
!     runs over the g-channel (1 to 16).

      kao_mn2o( 1, :, 1) = (/ &
     & 5.41078e-02_rb, 5.59051e-02_rb, 5.77620e-02_rb, 5.96805e-02_rb, 6.16628e-02_rb, &
     & 6.37110e-02_rb, 6.58272e-02_rb, 6.80137e-02_rb, 7.02728e-02_rb, 7.26069e-02_rb, &
     & 7.50185e-02_rb, 7.75103e-02_rb, 8.00848e-02_rb, 8.27449e-02_rb, 8.54933e-02_rb, &
     & 8.83330e-02_rb, 9.12670e-02_rb, 9.42984e-02_rb, 9.74306e-02_rb/)
      kao_mn2o( 2, :, 1) = (/ &
     & 1.19602e-01_rb, 1.22963e-01_rb, 1.26417e-01_rb, 1.29969e-01_rb, 1.33621e-01_rb, &
     & 1.37375e-01_rb, 1.41235e-01_rb, 1.45203e-01_rb, 1.49283e-01_rb, 1.53477e-01_rb, &
     & 1.57789e-01_rb, 1.62223e-01_rb, 1.66780e-01_rb, 1.71466e-01_rb, 1.76284e-01_rb, &
     & 1.81237e-01_rb, 1.86329e-01_rb, 1.91564e-01_rb, 1.96946e-01_rb/)
      kao_mn2o( 3, :, 1) = (/ &
     & 1.49614e-01_rb, 1.53427e-01_rb, 1.57337e-01_rb, 1.61346e-01_rb, 1.65457e-01_rb, &
     & 1.69674e-01_rb, 1.73997e-01_rb, 1.78431e-01_rb, 1.82978e-01_rb, 1.87641e-01_rb, &
     & 1.92422e-01_rb, 1.97325e-01_rb, 2.02354e-01_rb, 2.07510e-01_rb, 2.12798e-01_rb, &
     & 2.18221e-01_rb, 2.23781e-01_rb, 2.29484e-01_rb, 2.35331e-01_rb/)
      kao_mn2o( 4, :, 1) = (/ &
     & 1.80029e-01_rb, 1.84202e-01_rb, 1.88472e-01_rb, 1.92841e-01_rb, 1.97311e-01_rb, &
     & 2.01884e-01_rb, 2.06564e-01_rb, 2.11352e-01_rb, 2.16252e-01_rb, 2.21264e-01_rb, &
     & 2.26393e-01_rb, 2.31641e-01_rb, 2.37010e-01_rb, 2.42504e-01_rb, 2.48126e-01_rb, &
     & 2.53877e-01_rb, 2.59762e-01_rb, 2.65783e-01_rb, 2.71944e-01_rb/)
      kao_mn2o( 5, :, 1) = (/ &
     & 2.08279e-01_rb, 2.13029e-01_rb, 2.17888e-01_rb, 2.22858e-01_rb, 2.27941e-01_rb, &
     & 2.33140e-01_rb, 2.38458e-01_rb, 2.43897e-01_rb, 2.49460e-01_rb, 2.55150e-01_rb, &
     & 2.60969e-01_rb, 2.66922e-01_rb, 2.73010e-01_rb, 2.79237e-01_rb, 2.85606e-01_rb, &
     & 2.92120e-01_rb, 2.98783e-01_rb, 3.05598e-01_rb, 3.12568e-01_rb/)
      kao_mn2o( 6, :, 1) = (/ &
     & 2.17336e-01_rb, 2.22571e-01_rb, 2.27931e-01_rb, 2.33421e-01_rb, 2.39043e-01_rb, &
     & 2.44801e-01_rb, 2.50697e-01_rb, 2.56735e-01_rb, 2.62918e-01_rb, 2.69251e-01_rb, &
     & 2.75735e-01_rb, 2.82377e-01_rb, 2.89178e-01_rb, 2.96142e-01_rb, 3.03275e-01_rb, &
     & 3.10579e-01_rb, 3.18060e-01_rb, 3.25720e-01_rb, 3.33565e-01_rb/)
      kao_mn2o( 7, :, 1) = (/ &
     & 2.23903e-01_rb, 2.29349e-01_rb, 2.34926e-01_rb, 2.40640e-01_rb, 2.46493e-01_rb, &
     & 2.52488e-01_rb, 2.58628e-01_rb, 2.64918e-01_rb, 2.71361e-01_rb, 2.77961e-01_rb, &
     & 2.84721e-01_rb, 2.91646e-01_rb, 2.98739e-01_rb, 3.06005e-01_rb, 3.13447e-01_rb, &
     & 3.21070e-01_rb, 3.28879e-01_rb, 3.36877e-01_rb, 3.45071e-01_rb/)
      kao_mn2o( 8, :, 1) = (/ &
     & 2.23400e-01_rb, 2.28843e-01_rb, 2.34418e-01_rb, 2.40130e-01_rb, 2.45980e-01_rb, &
     & 2.51973e-01_rb, 2.58112e-01_rb, 2.64401e-01_rb, 2.70843e-01_rb, 2.77442e-01_rb, &
     & 2.84202e-01_rb, 2.91126e-01_rb, 2.98219e-01_rb, 3.05485e-01_rb, 3.12928e-01_rb, &
     & 3.20552e-01_rb, 3.28362e-01_rb, 3.36362e-01_rb, 3.44557e-01_rb/)
      kao_mn2o( 9, :, 1) = (/ &
     & 1.89279e-01_rb, 1.94423e-01_rb, 1.99707e-01_rb, 2.05135e-01_rb, 2.10710e-01_rb, &
     & 2.16437e-01_rb, 2.22319e-01_rb, 2.28361e-01_rb, 2.34568e-01_rb, 2.40943e-01_rb, &
     & 2.47492e-01_rb, 2.54218e-01_rb, 2.61127e-01_rb, 2.68224e-01_rb, 2.75514e-01_rb, &
     & 2.83002e-01_rb, 2.90694e-01_rb, 2.98594e-01_rb, 3.06709e-01_rb/)
      kao_mn2o( 1, :, 2) = (/ &
     & 9.46669e-02_rb, 9.77137e-02_rb, 1.00858e-01_rb, 1.04104e-01_rb, 1.07455e-01_rb, &
     & 1.10913e-01_rb, 1.14483e-01_rb, 1.18167e-01_rb, 1.21971e-01_rb, 1.25896e-01_rb, &
     & 1.29948e-01_rb, 1.34130e-01_rb, 1.38447e-01_rb, 1.42903e-01_rb, 1.47502e-01_rb, &
     & 1.52249e-01_rb, 1.57149e-01_rb, 1.62207e-01_rb, 1.67427e-01_rb/)
      kao_mn2o( 2, :, 2) = (/ &
     & 5.11901e-01_rb, 5.24950e-01_rb, 5.38331e-01_rb, 5.52053e-01_rb, 5.66125e-01_rb, &
     & 5.80556e-01_rb, 5.95354e-01_rb, 6.10530e-01_rb, 6.26093e-01_rb, 6.42052e-01_rb, &
     & 6.58418e-01_rb, 6.75202e-01_rb, 6.92413e-01_rb, 7.10062e-01_rb, 7.28162e-01_rb, &
     & 7.46723e-01_rb, 7.65758e-01_rb, 7.85277e-01_rb, 8.05294e-01_rb/)
      kao_mn2o( 3, :, 2) = (/ &
     & 8.32946e-01_rb, 8.45780e-01_rb, 8.58813e-01_rb, 8.72046e-01_rb, 8.85482e-01_rb, &
     & 8.99126e-01_rb, 9.12980e-01_rb, 9.27048e-01_rb, 9.41332e-01_rb, 9.55837e-01_rb, &
     & 9.70565e-01_rb, 9.85520e-01_rb, 1.00070e+00_rb, 1.01612e+00_rb, 1.03178e+00_rb, &
     & 1.04768e+00_rb, 1.06382e+00_rb, 1.08021e+00_rb, 1.09686e+00_rb/)
      kao_mn2o( 4, :, 2) = (/ &
     & 1.04032e+00_rb, 1.05475e+00_rb, 1.06937e+00_rb, 1.08419e+00_rb, 1.09922e+00_rb, &
     & 1.11446e+00_rb, 1.12991e+00_rb, 1.14557e+00_rb, 1.16145e+00_rb, 1.17755e+00_rb, &
     & 1.19387e+00_rb, 1.21042e+00_rb, 1.22720e+00_rb, 1.24421e+00_rb, 1.26146e+00_rb, &
     & 1.27895e+00_rb, 1.29668e+00_rb, 1.31465e+00_rb, 1.33287e+00_rb/)
      kao_mn2o( 5, :, 2) = (/ &
     & 1.22685e+00_rb, 1.24267e+00_rb, 1.25870e+00_rb, 1.27493e+00_rb, 1.29137e+00_rb, &
     & 1.30803e+00_rb, 1.32490e+00_rb, 1.34199e+00_rb, 1.35930e+00_rb, 1.37683e+00_rb, &
     & 1.39459e+00_rb, 1.41257e+00_rb, 1.43079e+00_rb, 1.44925e+00_rb, 1.46794e+00_rb, &
     & 1.48687e+00_rb, 1.50605e+00_rb, 1.52547e+00_rb, 1.54515e+00_rb/)
      kao_mn2o( 6, :, 2) = (/ &
     & 1.53781e+00_rb, 1.55206e+00_rb, 1.56645e+00_rb, 1.58097e+00_rb, 1.59563e+00_rb, &
     & 1.61042e+00_rb, 1.62535e+00_rb, 1.64042e+00_rb, 1.65562e+00_rb, 1.67097e+00_rb, &
     & 1.68646e+00_rb, 1.70210e+00_rb, 1.71788e+00_rb, 1.73380e+00_rb, 1.74987e+00_rb, &
     & 1.76610e+00_rb, 1.78247e+00_rb, 1.79899e+00_rb, 1.81567e+00_rb/)
      kao_mn2o( 7, :, 2) = (/ &
     & 1.90476e+00_rb, 1.91858e+00_rb, 1.93251e+00_rb, 1.94653e+00_rb, 1.96065e+00_rb, &
     & 1.97488e+00_rb, 1.98921e+00_rb, 2.00365e+00_rb, 2.01819e+00_rb, 2.03283e+00_rb, &
     & 2.04758e+00_rb, 2.06244e+00_rb, 2.07741e+00_rb, 2.09248e+00_rb, 2.10767e+00_rb, &
     & 2.12296e+00_rb, 2.13837e+00_rb, 2.15388e+00_rb, 2.16951e+00_rb/)
      kao_mn2o( 8, :, 2) = (/ &
     & 2.38211e+00_rb, 2.39819e+00_rb, 2.41438e+00_rb, 2.43068e+00_rb, 2.44709e+00_rb, &
     & 2.46361e+00_rb, 2.48024e+00_rb, 2.49699e+00_rb, 2.51384e+00_rb, 2.53082e+00_rb, &
     & 2.54790e+00_rb, 2.56510e+00_rb, 2.58242e+00_rb, 2.59985e+00_rb, 2.61741e+00_rb, &
     & 2.63508e+00_rb, 2.65287e+00_rb, 2.67078e+00_rb, 2.68881e+00_rb/)
      kao_mn2o( 9, :, 2) = (/ &
     & 1.26464e+00_rb, 1.28107e+00_rb, 1.29772e+00_rb, 1.31458e+00_rb, 1.33166e+00_rb, &
     & 1.34896e+00_rb, 1.36649e+00_rb, 1.38424e+00_rb, 1.40223e+00_rb, 1.42044e+00_rb, &
     & 1.43890e+00_rb, 1.45759e+00_rb, 1.47653e+00_rb, 1.49572e+00_rb, 1.51515e+00_rb, &
     & 1.53483e+00_rb, 1.55478e+00_rb, 1.57498e+00_rb, 1.59544e+00_rb/)
      kao_mn2o( 1, :, 3) = (/ &
     & 1.50082e-01_rb, 1.59095e-01_rb, 1.68650e-01_rb, 1.78779e-01_rb, 1.89516e-01_rb, &
     & 2.00898e-01_rb, 2.12963e-01_rb, 2.25753e-01_rb, 2.39311e-01_rb, 2.53683e-01_rb, &
     & 2.68919e-01_rb, 2.85069e-01_rb, 3.02190e-01_rb, 3.20339e-01_rb, 3.39577e-01_rb, &
     & 3.59971e-01_rb, 3.81590e-01_rb, 4.04507e-01_rb, 4.28801e-01_rb/)
      kao_mn2o( 2, :, 3) = (/ &
     & 3.09551e+00_rb, 3.09780e+00_rb, 3.10008e+00_rb, 3.10237e+00_rb, 3.10466e+00_rb, &
     & 3.10695e+00_rb, 3.10925e+00_rb, 3.11154e+00_rb, 3.11384e+00_rb, 3.11614e+00_rb, &
     & 3.11844e+00_rb, 3.12074e+00_rb, 3.12305e+00_rb, 3.12535e+00_rb, 3.12766e+00_rb, &
     & 3.12997e+00_rb, 3.13228e+00_rb, 3.13459e+00_rb, 3.13691e+00_rb/)
      kao_mn2o( 3, :, 3) = (/ &
     & 4.42661e+00_rb, 4.40858e+00_rb, 4.39062e+00_rb, 4.37274e+00_rb, 4.35493e+00_rb, &
     & 4.33719e+00_rb, 4.31953e+00_rb, 4.30193e+00_rb, 4.28441e+00_rb, 4.26696e+00_rb, &
     & 4.24958e+00_rb, 4.23227e+00_rb, 4.21503e+00_rb, 4.19787e+00_rb, 4.18077e+00_rb, &
     & 4.16374e+00_rb, 4.14678e+00_rb, 4.12989e+00_rb, 4.11307e+00_rb/)
      kao_mn2o( 4, :, 3) = (/ &
     & 5.77864e+00_rb, 5.74085e+00_rb, 5.70331e+00_rb, 5.66602e+00_rb, 5.62897e+00_rb, &
     & 5.59216e+00_rb, 5.55559e+00_rb, 5.51926e+00_rb, 5.48317e+00_rb, 5.44731e+00_rb, &
     & 5.41169e+00_rb, 5.37631e+00_rb, 5.34115e+00_rb, 5.30622e+00_rb, 5.27152e+00_rb, &
     & 5.23705e+00_rb, 5.20281e+00_rb, 5.16879e+00_rb, 5.13499e+00_rb/)
      kao_mn2o( 5, :, 3) = (/ &
     & 7.17294e+00_rb, 7.10890e+00_rb, 7.04542e+00_rb, 6.98251e+00_rb, 6.92017e+00_rb, &
     & 6.85838e+00_rb, 6.79714e+00_rb, 6.73645e+00_rb, 6.67630e+00_rb, 6.61669e+00_rb, &
     & 6.55761e+00_rb, 6.49905e+00_rb, 6.44102e+00_rb, 6.38351e+00_rb, 6.32651e+00_rb, &
     & 6.27003e+00_rb, 6.21404e+00_rb, 6.15856e+00_rb, 6.10357e+00_rb/)
      kao_mn2o( 6, :, 3) = (/ &
     & 9.05082e+00_rb, 8.96132e+00_rb, 8.87271e+00_rb, 8.78498e+00_rb, 8.69811e+00_rb, &
     & 8.61210e+00_rb, 8.52694e+00_rb, 8.44262e+00_rb, 8.35914e+00_rb, 8.27648e+00_rb, &
     & 8.19464e+00_rb, 8.11361e+00_rb, 8.03338e+00_rb, 7.95394e+00_rb, 7.87529e+00_rb, &
     & 7.79742e+00_rb, 7.72032e+00_rb, 7.64398e+00_rb, 7.56839e+00_rb/)
      kao_mn2o( 7, :, 3) = (/ &
     & 1.18749e+01_rb, 1.17648e+01_rb, 1.16556e+01_rb, 1.15475e+01_rb, 1.14403e+01_rb, &
     & 1.13342e+01_rb, 1.12290e+01_rb, 1.11248e+01_rb, 1.10216e+01_rb, 1.09194e+01_rb, &
     & 1.08180e+01_rb, 1.07177e+01_rb, 1.06182e+01_rb, 1.05197e+01_rb, 1.04221e+01_rb, &
     & 1.03254e+01_rb, 1.02296e+01_rb, 1.01347e+01_rb, 1.00407e+01_rb/)
      kao_mn2o( 8, :, 3) = (/ &
     & 1.41428e+01_rb, 1.40323e+01_rb, 1.39227e+01_rb, 1.38139e+01_rb, 1.37060e+01_rb, &
     & 1.35989e+01_rb, 1.34927e+01_rb, 1.33873e+01_rb, 1.32827e+01_rb, 1.31790e+01_rb, &
     & 1.30760e+01_rb, 1.29738e+01_rb, 1.28725e+01_rb, 1.27719e+01_rb, 1.26722e+01_rb, &
     & 1.25732e+01_rb, 1.24750e+01_rb, 1.23775e+01_rb, 1.22808e+01_rb/)
      kao_mn2o( 9, :, 3) = (/ &
     & 7.34993e+00_rb, 7.29335e+00_rb, 7.23720e+00_rb, 7.18149e+00_rb, 7.12620e+00_rb, &
     & 7.07134e+00_rb, 7.01690e+00_rb, 6.96288e+00_rb, 6.90928e+00_rb, 6.85609e+00_rb, &
     & 6.80331e+00_rb, 6.75094e+00_rb, 6.69897e+00_rb, 6.64739e+00_rb, 6.59622e+00_rb, &
     & 6.54544e+00_rb, 6.49505e+00_rb, 6.44505e+00_rb, 6.39543e+00_rb/)
      kao_mn2o( 1, :, 4) = (/ &
     & 6.11248e-01_rb, 6.37225e-01_rb, 6.64306e-01_rb, 6.92538e-01_rb, 7.21970e-01_rb, &
     & 7.52653e-01_rb, 7.84639e-01_rb, 8.17985e-01_rb, 8.52749e-01_rb, 8.88989e-01_rb, &
     & 9.26770e-01_rb, 9.66157e-01_rb, 1.00722e+00_rb, 1.05002e+00_rb, 1.09465e+00_rb, &
     & 1.14117e+00_rb, 1.18967e+00_rb, 1.24022e+00_rb, 1.29293e+00_rb/)
      kao_mn2o( 2, :, 4) = (/ &
     & 5.07253e+00_rb, 5.05299e+00_rb, 5.03353e+00_rb, 5.01414e+00_rb, 4.99483e+00_rb, &
     & 4.97559e+00_rb, 4.95642e+00_rb, 4.93733e+00_rb, 4.91831e+00_rb, 4.89937e+00_rb, &
     & 4.88050e+00_rb, 4.86170e+00_rb, 4.84297e+00_rb, 4.82432e+00_rb, 4.80573e+00_rb, &
     & 4.78722e+00_rb, 4.76878e+00_rb, 4.75042e+00_rb, 4.73212e+00_rb/)
      kao_mn2o( 3, :, 4) = (/ &
     & 7.45829e+00_rb, 7.42266e+00_rb, 7.38719e+00_rb, 7.35190e+00_rb, 7.31677e+00_rb, &
     & 7.28181e+00_rb, 7.24702e+00_rb, 7.21240e+00_rb, 7.17794e+00_rb, 7.14364e+00_rb, &
     & 7.10951e+00_rb, 7.07554e+00_rb, 7.04173e+00_rb, 7.00809e+00_rb, 6.97461e+00_rb, &
     & 6.94128e+00_rb, 6.90812e+00_rb, 6.87511e+00_rb, 6.84226e+00_rb/)
      kao_mn2o( 4, :, 4) = (/ &
     & 9.58893e+00_rb, 9.54796e+00_rb, 9.50716e+00_rb, 9.46654e+00_rb, 9.42609e+00_rb, &
     & 9.38581e+00_rb, 9.34571e+00_rb, 9.30578e+00_rb, 9.26602e+00_rb, 9.22642e+00_rb, &
     & 9.18700e+00_rb, 9.14775e+00_rb, 9.10866e+00_rb, 9.06974e+00_rb, 9.03099e+00_rb, &
     & 8.99240e+00_rb, 8.95398e+00_rb, 8.91572e+00_rb, 8.87762e+00_rb/)
      kao_mn2o( 5, :, 4) = (/ &
     & 1.16344e+01_rb, 1.16012e+01_rb, 1.15681e+01_rb, 1.15351e+01_rb, 1.15022e+01_rb, &
     & 1.14694e+01_rb, 1.14366e+01_rb, 1.14040e+01_rb, 1.13715e+01_rb, 1.13390e+01_rb, &
     & 1.13067e+01_rb, 1.12744e+01_rb, 1.12422e+01_rb, 1.12102e+01_rb, 1.11782e+01_rb, &
     & 1.11463e+01_rb, 1.11145e+01_rb, 1.10828e+01_rb, 1.10511e+01_rb/)
      kao_mn2o( 6, :, 4) = (/ &
     & 1.12460e+01_rb, 1.12402e+01_rb, 1.12344e+01_rb, 1.12286e+01_rb, 1.12228e+01_rb, &
     & 1.12170e+01_rb, 1.12112e+01_rb, 1.12055e+01_rb, 1.11997e+01_rb, 1.11939e+01_rb, &
     & 1.11882e+01_rb, 1.11824e+01_rb, 1.11766e+01_rb, 1.11709e+01_rb, 1.11651e+01_rb, &
     & 1.11594e+01_rb, 1.11536e+01_rb, 1.11479e+01_rb, 1.11421e+01_rb/)
      kao_mn2o( 7, :, 4) = (/ &
     & 8.89265e+00_rb, 8.91419e+00_rb, 8.93578e+00_rb, 8.95743e+00_rb, 8.97913e+00_rb, &
     & 9.00088e+00_rb, 9.02268e+00_rb, 9.04454e+00_rb, 9.06645e+00_rb, 9.08841e+00_rb, &
     & 9.11043e+00_rb, 9.13250e+00_rb, 9.15462e+00_rb, 9.17680e+00_rb, 9.19903e+00_rb, &
     & 9.22131e+00_rb, 9.24365e+00_rb, 9.26604e+00_rb, 9.28849e+00_rb/)
      kao_mn2o( 8, :, 4) = (/ &
     & 6.83933e+00_rb, 6.86688e+00_rb, 6.89453e+00_rb, 6.92230e+00_rb, 6.95018e+00_rb, &
     & 6.97817e+00_rb, 7.00627e+00_rb, 7.03449e+00_rb, 7.06282e+00_rb, 7.09126e+00_rb, &
     & 7.11982e+00_rb, 7.14850e+00_rb, 7.17729e+00_rb, 7.20619e+00_rb, 7.23521e+00_rb, &
     & 7.26435e+00_rb, 7.29361e+00_rb, 7.32298e+00_rb, 7.35248e+00_rb/)
      kao_mn2o( 9, :, 4) = (/ &
     & 1.10637e+01_rb, 1.10232e+01_rb, 1.09829e+01_rb, 1.09427e+01_rb, 1.09026e+01_rb, &
     & 1.08627e+01_rb, 1.08230e+01_rb, 1.07833e+01_rb, 1.07439e+01_rb, 1.07046e+01_rb, &
     & 1.06654e+01_rb, 1.06263e+01_rb, 1.05875e+01_rb, 1.05487e+01_rb, 1.05101e+01_rb, &
     & 1.04716e+01_rb, 1.04333e+01_rb, 1.03951e+01_rb, 1.03571e+01_rb/)
      kao_mn2o( 1, :, 5) = (/ &
     & 2.53460e+00_rb, 2.56050e+00_rb, 2.58667e+00_rb, 2.61310e+00_rb, 2.63980e+00_rb, &
     & 2.66678e+00_rb, 2.69403e+00_rb, 2.72156e+00_rb, 2.74937e+00_rb, 2.77746e+00_rb, &
     & 2.80585e+00_rb, 2.83452e+00_rb, 2.86348e+00_rb, 2.89275e+00_rb, 2.92231e+00_rb, &
     & 2.95217e+00_rb, 2.98234e+00_rb, 3.01281e+00_rb, 3.04360e+00_rb/)
      kao_mn2o( 2, :, 5) = (/ &
     & 7.45650e+00_rb, 7.44283e+00_rb, 7.42919e+00_rb, 7.41557e+00_rb, 7.40198e+00_rb, &
     & 7.38841e+00_rb, 7.37487e+00_rb, 7.36135e+00_rb, 7.34786e+00_rb, 7.33439e+00_rb, &
     & 7.32095e+00_rb, 7.30753e+00_rb, 7.29413e+00_rb, 7.28076e+00_rb, 7.26742e+00_rb, &
     & 7.25410e+00_rb, 7.24080e+00_rb, 7.22753e+00_rb, 7.21428e+00_rb/)
      kao_mn2o( 3, :, 5) = (/ &
     & 1.06311e+01_rb, 1.06110e+01_rb, 1.05909e+01_rb, 1.05709e+01_rb, 1.05509e+01_rb, &
     & 1.05310e+01_rb, 1.05111e+01_rb, 1.04912e+01_rb, 1.04713e+01_rb, 1.04516e+01_rb, &
     & 1.04318e+01_rb, 1.04121e+01_rb, 1.03924e+01_rb, 1.03727e+01_rb, 1.03531e+01_rb, &
     & 1.03336e+01_rb, 1.03140e+01_rb, 1.02945e+01_rb, 1.02751e+01_rb/)
      kao_mn2o( 4, :, 5) = (/ &
     & 1.03924e+01_rb, 1.03895e+01_rb, 1.03867e+01_rb, 1.03838e+01_rb, 1.03809e+01_rb, &
     & 1.03780e+01_rb, 1.03751e+01_rb, 1.03722e+01_rb, 1.03693e+01_rb, 1.03665e+01_rb, &
     & 1.03636e+01_rb, 1.03607e+01_rb, 1.03578e+01_rb, 1.03549e+01_rb, 1.03521e+01_rb, &
     & 1.03492e+01_rb, 1.03463e+01_rb, 1.03434e+01_rb, 1.03406e+01_rb/)
      kao_mn2o( 5, :, 5) = (/ &
     & 7.82277e+00_rb, 7.83872e+00_rb, 7.85471e+00_rb, 7.87073e+00_rb, 7.88678e+00_rb, &
     & 7.90287e+00_rb, 7.91899e+00_rb, 7.93514e+00_rb, 7.95132e+00_rb, 7.96754e+00_rb, &
     & 7.98379e+00_rb, 8.00008e+00_rb, 8.01639e+00_rb, 8.03274e+00_rb, 8.04913e+00_rb, &
     & 8.06555e+00_rb, 8.08200e+00_rb, 8.09848e+00_rb, 8.11500e+00_rb/)
      kao_mn2o( 6, :, 5) = (/ &
     & 6.05225e+00_rb, 6.06883e+00_rb, 6.08545e+00_rb, 6.10212e+00_rb, 6.11883e+00_rb, &
     & 6.13559e+00_rb, 6.15240e+00_rb, 6.16925e+00_rb, 6.18615e+00_rb, 6.20309e+00_rb, &
     & 6.22008e+00_rb, 6.23712e+00_rb, 6.25420e+00_rb, 6.27133e+00_rb, 6.28851e+00_rb, &
     & 6.30574e+00_rb, 6.32301e+00_rb, 6.34033e+00_rb, 6.35769e+00_rb/)
      kao_mn2o( 7, :, 5) = (/ &
     & 5.24135e+00_rb, 5.25696e+00_rb, 5.27261e+00_rb, 5.28831e+00_rb, 5.30405e+00_rb, &
     & 5.31984e+00_rb, 5.33568e+00_rb, 5.35157e+00_rb, 5.36750e+00_rb, 5.38348e+00_rb, &
     & 5.39951e+00_rb, 5.41558e+00_rb, 5.43171e+00_rb, 5.44788e+00_rb, 5.46410e+00_rb, &
     & 5.48037e+00_rb, 5.49668e+00_rb, 5.51305e+00_rb, 5.52946e+00_rb/)
      kao_mn2o( 8, :, 5) = (/ &
     & 4.40240e+00_rb, 4.40915e+00_rb, 4.41591e+00_rb, 4.42268e+00_rb, 4.42946e+00_rb, &
     & 4.43625e+00_rb, 4.44305e+00_rb, 4.44986e+00_rb, 4.45668e+00_rb, 4.46351e+00_rb, &
     & 4.47035e+00_rb, 4.47720e+00_rb, 4.48407e+00_rb, 4.49094e+00_rb, 4.49782e+00_rb, &
     & 4.50472e+00_rb, 4.51162e+00_rb, 4.51854e+00_rb, 4.52547e+00_rb/)
      kao_mn2o( 9, :, 5) = (/ &
     & 8.56554e+00_rb, 8.59185e+00_rb, 8.61824e+00_rb, 8.64470e+00_rb, 8.67125e+00_rb, &
     & 8.69788e+00_rb, 8.72460e+00_rb, 8.75139e+00_rb, 8.77827e+00_rb, 8.80523e+00_rb, &
     & 8.83227e+00_rb, 8.85939e+00_rb, 8.88660e+00_rb, 8.91389e+00_rb, 8.94127e+00_rb, &
     & 8.96873e+00_rb, 8.99627e+00_rb, 9.02390e+00_rb, 9.05161e+00_rb/)
      kao_mn2o( 1, :, 6) = (/ &
     & 5.78695e+00_rb, 5.78939e+00_rb, 5.79182e+00_rb, 5.79426e+00_rb, 5.79670e+00_rb, &
     & 5.79914e+00_rb, 5.80158e+00_rb, 5.80403e+00_rb, 5.80647e+00_rb, 5.80892e+00_rb, &
     & 5.81136e+00_rb, 5.81381e+00_rb, 5.81626e+00_rb, 5.81870e+00_rb, 5.82115e+00_rb, &
     & 5.82361e+00_rb, 5.82606e+00_rb, 5.82851e+00_rb, 5.83096e+00_rb/)
      kao_mn2o( 2, :, 6) = (/ &
     & 1.22893e+01_rb, 1.22556e+01_rb, 1.22221e+01_rb, 1.21886e+01_rb, 1.21552e+01_rb, &
     & 1.21220e+01_rb, 1.20888e+01_rb, 1.20557e+01_rb, 1.20227e+01_rb, 1.19898e+01_rb, &
     & 1.19569e+01_rb, 1.19242e+01_rb, 1.18915e+01_rb, 1.18590e+01_rb, 1.18265e+01_rb, &
     & 1.17941e+01_rb, 1.17618e+01_rb, 1.17296e+01_rb, 1.16975e+01_rb/)
      kao_mn2o( 3, :, 6) = (/ &
     & 7.93118e+00_rb, 7.94590e+00_rb, 7.96065e+00_rb, 7.97542e+00_rb, 7.99022e+00_rb, &
     & 8.00505e+00_rb, 8.01990e+00_rb, 8.03478e+00_rb, 8.04970e+00_rb, 8.06463e+00_rb, &
     & 8.07960e+00_rb, 8.09459e+00_rb, 8.10961e+00_rb, 8.12466e+00_rb, 8.13974e+00_rb, &
     & 8.15485e+00_rb, 8.16998e+00_rb, 8.18514e+00_rb, 8.20033e+00_rb/)
      kao_mn2o( 4, :, 6) = (/ &
     & 4.08899e+00_rb, 4.11435e+00_rb, 4.13988e+00_rb, 4.16556e+00_rb, 4.19140e+00_rb, &
     & 4.21740e+00_rb, 4.24357e+00_rb, 4.26989e+00_rb, 4.29638e+00_rb, 4.32304e+00_rb, &
     & 4.34985e+00_rb, 4.37684e+00_rb, 4.40399e+00_rb, 4.43131e+00_rb, 4.45880e+00_rb, &
     & 4.48646e+00_rb, 4.51430e+00_rb, 4.54230e+00_rb, 4.57048e+00_rb/)
      kao_mn2o( 5, :, 6) = (/ &
     & 2.61358e+00_rb, 2.64029e+00_rb, 2.66728e+00_rb, 2.69454e+00_rb, 2.72209e+00_rb, &
     & 2.74991e+00_rb, 2.77802e+00_rb, 2.80641e+00_rb, 2.83510e+00_rb, 2.86408e+00_rb, &
     & 2.89335e+00_rb, 2.92293e+00_rb, 2.95280e+00_rb, 2.98299e+00_rb, 3.01348e+00_rb, &
     & 3.04428e+00_rb, 3.07540e+00_rb, 3.10683e+00_rb, 3.13859e+00_rb/)
      kao_mn2o( 6, :, 6) = (/ &
     & 2.40720e+00_rb, 2.43430e+00_rb, 2.46169e+00_rb, 2.48940e+00_rb, 2.51741e+00_rb, &
     & 2.54575e+00_rb, 2.57440e+00_rb, 2.60337e+00_rb, 2.63267e+00_rb, 2.66230e+00_rb, &
     & 2.69226e+00_rb, 2.72256e+00_rb, 2.75320e+00_rb, 2.78419e+00_rb, 2.81552e+00_rb, &
     & 2.84721e+00_rb, 2.87925e+00_rb, 2.91166e+00_rb, 2.94443e+00_rb/)
      kao_mn2o( 7, :, 6) = (/ &
     & 1.99607e+00_rb, 2.01725e+00_rb, 2.03865e+00_rb, 2.06028e+00_rb, 2.08214e+00_rb, &
     & 2.10423e+00_rb, 2.12655e+00_rb, 2.14912e+00_rb, 2.17192e+00_rb, 2.19496e+00_rb, &
     & 2.21825e+00_rb, 2.24179e+00_rb, 2.26557e+00_rb, 2.28961e+00_rb, 2.31390e+00_rb, &
     & 2.33845e+00_rb, 2.36326e+00_rb, 2.38834e+00_rb, 2.41368e+00_rb/)
      kao_mn2o( 8, :, 6) = (/ &
     & 1.94150e+00_rb, 1.96398e+00_rb, 1.98671e+00_rb, 2.00971e+00_rb, 2.03298e+00_rb, &
     & 2.05651e+00_rb, 2.08032e+00_rb, 2.10440e+00_rb, 2.12876e+00_rb, 2.15341e+00_rb, &
     & 2.17834e+00_rb, 2.20355e+00_rb, 2.22906e+00_rb, 2.25487e+00_rb, 2.28097e+00_rb, &
     & 2.30738e+00_rb, 2.33409e+00_rb, 2.36111e+00_rb, 2.38844e+00_rb/)
      kao_mn2o( 9, :, 6) = (/ &
     & 2.47259e+00_rb, 2.48950e+00_rb, 2.50653e+00_rb, 2.52367e+00_rb, 2.54093e+00_rb, &
     & 2.55831e+00_rb, 2.57581e+00_rb, 2.59343e+00_rb, 2.61117e+00_rb, 2.62903e+00_rb, &
     & 2.64701e+00_rb, 2.66511e+00_rb, 2.68334e+00_rb, 2.70169e+00_rb, 2.72017e+00_rb, &
     & 2.73878e+00_rb, 2.75751e+00_rb, 2.77637e+00_rb, 2.79536e+00_rb/)
      kao_mn2o( 1, :, 7) = (/ &
     & 1.23417e+01_rb, 1.22618e+01_rb, 1.21823e+01_rb, 1.21034e+01_rb, 1.20250e+01_rb, &
     & 1.19471e+01_rb, 1.18697e+01_rb, 1.17928e+01_rb, 1.17164e+01_rb, 1.16405e+01_rb, &
     & 1.15651e+01_rb, 1.14901e+01_rb, 1.14157e+01_rb, 1.13417e+01_rb, 1.12683e+01_rb, &
     & 1.11952e+01_rb, 1.11227e+01_rb, 1.10507e+01_rb, 1.09791e+01_rb/)
      kao_mn2o( 2, :, 7) = (/ &
     & 9.30957e+00_rb, 9.32775e+00_rb, 9.34597e+00_rb, 9.36421e+00_rb, 9.38250e+00_rb, &
     & 9.40082e+00_rb, 9.41918e+00_rb, 9.43757e+00_rb, 9.45600e+00_rb, 9.47446e+00_rb, &
     & 9.49296e+00_rb, 9.51150e+00_rb, 9.53007e+00_rb, 9.54868e+00_rb, 9.56732e+00_rb, &
     & 9.58601e+00_rb, 9.60472e+00_rb, 9.62348e+00_rb, 9.64227e+00_rb/)
      kao_mn2o( 3, :, 7) = (/ &
     & 4.15867e+00_rb, 4.19254e+00_rb, 4.22668e+00_rb, 4.26110e+00_rb, 4.29581e+00_rb, &
     & 4.33079e+00_rb, 4.36606e+00_rb, 4.40162e+00_rb, 4.43747e+00_rb, 4.47360e+00_rb, &
     & 4.51004e+00_rb, 4.54677e+00_rb, 4.58380e+00_rb, 4.62113e+00_rb, 4.65876e+00_rb, &
     & 4.69670e+00_rb, 4.73495e+00_rb, 4.77351e+00_rb, 4.81239e+00_rb/)
      kao_mn2o( 4, :, 7) = (/ &
     & 3.55634e+00_rb, 3.59382e+00_rb, 3.63169e+00_rb, 3.66996e+00_rb, 3.70863e+00_rb, &
     & 3.74771e+00_rb, 3.78720e+00_rb, 3.82711e+00_rb, 3.86744e+00_rb, 3.90820e+00_rb, &
     & 3.94938e+00_rb, 3.99100e+00_rb, 4.03305e+00_rb, 4.07555e+00_rb, 4.11850e+00_rb, &
     & 4.16190e+00_rb, 4.20576e+00_rb, 4.25008e+00_rb, 4.29486e+00_rb/)
      kao_mn2o( 5, :, 7) = (/ &
     & 3.09468e+00_rb, 3.12655e+00_rb, 3.15876e+00_rb, 3.19129e+00_rb, 3.22416e+00_rb, &
     & 3.25737e+00_rb, 3.29092e+00_rb, 3.32482e+00_rb, 3.35907e+00_rb, 3.39366e+00_rb, &
     & 3.42862e+00_rb, 3.46393e+00_rb, 3.49961e+00_rb, 3.53566e+00_rb, 3.57208e+00_rb, &
     & 3.60887e+00_rb, 3.64604e+00_rb, 3.68360e+00_rb, 3.72154e+00_rb/)
      kao_mn2o( 6, :, 7) = (/ &
     & 2.75473e+00_rb, 2.78356e+00_rb, 2.81268e+00_rb, 2.84211e+00_rb, 2.87185e+00_rb, &
     & 2.90190e+00_rb, 2.93227e+00_rb, 2.96295e+00_rb, 2.99395e+00_rb, 3.02528e+00_rb, &
     & 3.05694e+00_rb, 3.08892e+00_rb, 3.12125e+00_rb, 3.15391e+00_rb, 3.18691e+00_rb, &
     & 3.22025e+00_rb, 3.25395e+00_rb, 3.28800e+00_rb, 3.32240e+00_rb/)
      kao_mn2o( 7, :, 7) = (/ &
     & 2.68587e+00_rb, 2.71431e+00_rb, 2.74306e+00_rb, 2.77211e+00_rb, 2.80146e+00_rb, &
     & 2.83113e+00_rb, 2.86111e+00_rb, 2.89141e+00_rb, 2.92203e+00_rb, 2.95298e+00_rb, &
     & 2.98425e+00_rb, 3.01585e+00_rb, 3.04779e+00_rb, 3.08006e+00_rb, 3.11268e+00_rb, &
     & 3.14564e+00_rb, 3.17896e+00_rb, 3.21262e+00_rb, 3.24664e+00_rb/)
      kao_mn2o( 8, :, 7) = (/ &
     & 2.54778e+00_rb, 2.57461e+00_rb, 2.60173e+00_rb, 2.62914e+00_rb, 2.65683e+00_rb, &
     & 2.68482e+00_rb, 2.71310e+00_rb, 2.74168e+00_rb, 2.77056e+00_rb, 2.79974e+00_rb, &
     & 2.82923e+00_rb, 2.85903e+00_rb, 2.88915e+00_rb, 2.91958e+00_rb, 2.95033e+00_rb, &
     & 2.98141e+00_rb, 3.01282e+00_rb, 3.04455e+00_rb, 3.07662e+00_rb/)
      kao_mn2o( 9, :, 7) = (/ &
     & 2.78137e+00_rb, 2.80957e+00_rb, 2.83805e+00_rb, 2.86682e+00_rb, 2.89589e+00_rb, &
     & 2.92525e+00_rb, 2.95491e+00_rb, 2.98486e+00_rb, 3.01512e+00_rb, 3.04569e+00_rb, &
     & 3.07657e+00_rb, 3.10776e+00_rb, 3.13927e+00_rb, 3.17110e+00_rb, 3.20324e+00_rb, &
     & 3.23572e+00_rb, 3.26852e+00_rb, 3.30166e+00_rb, 3.33513e+00_rb/)
      kao_mn2o( 1, :, 8) = (/ &
     & 2.28384e+01_rb, 2.27450e+01_rb, 2.26519e+01_rb, 2.25593e+01_rb, 2.24670e+01_rb, &
     & 2.23751e+01_rb, 2.22835e+01_rb, 2.21924e+01_rb, 2.21016e+01_rb, 2.20112e+01_rb, &
     & 2.19211e+01_rb, 2.18314e+01_rb, 2.17421e+01_rb, 2.16532e+01_rb, 2.15646e+01_rb, &
     & 2.14764e+01_rb, 2.13885e+01_rb, 2.13010e+01_rb, 2.12139e+01_rb/)
      kao_mn2o( 2, :, 8) = (/ &
     & 4.48608e+00_rb, 4.52259e+00_rb, 4.55939e+00_rb, 4.59649e+00_rb, 4.63389e+00_rb, &
     & 4.67160e+00_rb, 4.70961e+00_rb, 4.74793e+00_rb, 4.78656e+00_rb, 4.82551e+00_rb, &
     & 4.86478e+00_rb, 4.90436e+00_rb, 4.94427e+00_rb, 4.98450e+00_rb, 5.02506e+00_rb, &
     & 5.06595e+00_rb, 5.10717e+00_rb, 5.14873e+00_rb, 5.19062e+00_rb/)
      kao_mn2o( 3, :, 8) = (/ &
     & 3.69928e+00_rb, 3.72584e+00_rb, 3.75259e+00_rb, 3.77953e+00_rb, 3.80666e+00_rb, &
     & 3.83399e+00_rb, 3.86152e+00_rb, 3.88924e+00_rb, 3.91717e+00_rb, 3.94529e+00_rb, &
     & 3.97361e+00_rb, 4.00214e+00_rb, 4.03088e+00_rb, 4.05982e+00_rb, 4.08896e+00_rb, &
     & 4.11832e+00_rb, 4.14789e+00_rb, 4.17767e+00_rb, 4.20766e+00_rb/)
      kao_mn2o( 4, :, 8) = (/ &
     & 3.17856e+00_rb, 3.19596e+00_rb, 3.21345e+00_rb, 3.23104e+00_rb, 3.24872e+00_rb, &
     & 3.26650e+00_rb, 3.28437e+00_rb, 3.30235e+00_rb, 3.32042e+00_rb, 3.33859e+00_rb, &
     & 3.35686e+00_rb, 3.37523e+00_rb, 3.39370e+00_rb, 3.41227e+00_rb, 3.43095e+00_rb, &
     & 3.44972e+00_rb, 3.46860e+00_rb, 3.48758e+00_rb, 3.50667e+00_rb/)
      kao_mn2o( 5, :, 8) = (/ &
     & 3.16549e+00_rb, 3.18288e+00_rb, 3.20037e+00_rb, 3.21795e+00_rb, 3.23563e+00_rb, &
     & 3.25340e+00_rb, 3.27128e+00_rb, 3.28925e+00_rb, 3.30732e+00_rb, 3.32549e+00_rb, &
     & 3.34376e+00_rb, 3.36213e+00_rb, 3.38060e+00_rb, 3.39917e+00_rb, 3.41785e+00_rb, &
     & 3.43662e+00_rb, 3.45551e+00_rb, 3.47449e+00_rb, 3.49358e+00_rb/)
      kao_mn2o( 6, :, 8) = (/ &
     & 3.16612e+00_rb, 3.18355e+00_rb, 3.20108e+00_rb, 3.21870e+00_rb, 3.23643e+00_rb, &
     & 3.25425e+00_rb, 3.27217e+00_rb, 3.29018e+00_rb, 3.30830e+00_rb, 3.32652e+00_rb, &
     & 3.34483e+00_rb, 3.36325e+00_rb, 3.38177e+00_rb, 3.40039e+00_rb, 3.41911e+00_rb, &
     & 3.43794e+00_rb, 3.45687e+00_rb, 3.47591e+00_rb, 3.49505e+00_rb/)
      kao_mn2o( 7, :, 8) = (/ &
     & 3.19644e+00_rb, 3.21419e+00_rb, 3.23203e+00_rb, 3.24996e+00_rb, 3.26800e+00_rb, &
     & 3.28614e+00_rb, 3.30438e+00_rb, 3.32272e+00_rb, 3.34116e+00_rb, 3.35970e+00_rb, &
     & 3.37835e+00_rb, 3.39710e+00_rb, 3.41596e+00_rb, 3.43492e+00_rb, 3.45398e+00_rb, &
     & 3.47315e+00_rb, 3.49243e+00_rb, 3.51181e+00_rb, 3.53130e+00_rb/)
      kao_mn2o( 8, :, 8) = (/ &
     & 3.35759e+00_rb, 3.37775e+00_rb, 3.39804e+00_rb, 3.41845e+00_rb, 3.43899e+00_rb, &
     & 3.45964e+00_rb, 3.48042e+00_rb, 3.50133e+00_rb, 3.52236e+00_rb, 3.54351e+00_rb, &
     & 3.56480e+00_rb, 3.58621e+00_rb, 3.60775e+00_rb, 3.62942e+00_rb, 3.65122e+00_rb, &
     & 3.67315e+00_rb, 3.69521e+00_rb, 3.71741e+00_rb, 3.73974e+00_rb/)
      kao_mn2o( 9, :, 8) = (/ &
     & 3.17378e+00_rb, 3.19135e+00_rb, 3.20901e+00_rb, 3.22677e+00_rb, 3.24462e+00_rb, &
     & 3.26258e+00_rb, 3.28063e+00_rb, 3.29879e+00_rb, 3.31704e+00_rb, 3.33540e+00_rb, &
     & 3.35386e+00_rb, 3.37242e+00_rb, 3.39108e+00_rb, 3.40984e+00_rb, 3.42871e+00_rb, &
     & 3.44769e+00_rb, 3.46677e+00_rb, 3.48595e+00_rb, 3.50524e+00_rb/)
      kao_mn2o( 1, :, 9) = (/ &
     & 2.09106e+01_rb, 2.08779e+01_rb, 2.08452e+01_rb, 2.08126e+01_rb, 2.07800e+01_rb, &
     & 2.07475e+01_rb, 2.07150e+01_rb, 2.06826e+01_rb, 2.06502e+01_rb, 2.06179e+01_rb, &
     & 2.05856e+01_rb, 2.05534e+01_rb, 2.05213e+01_rb, 2.04891e+01_rb, 2.04571e+01_rb, &
     & 2.04251e+01_rb, 2.03931e+01_rb, 2.03612e+01_rb, 2.03293e+01_rb/)
      kao_mn2o( 2, :, 9) = (/ &
     & 2.60494e+00_rb, 2.62757e+00_rb, 2.65040e+00_rb, 2.67343e+00_rb, 2.69665e+00_rb, &
     & 2.72008e+00_rb, 2.74372e+00_rb, 2.76756e+00_rb, 2.79160e+00_rb, 2.81586e+00_rb, &
     & 2.84032e+00_rb, 2.86500e+00_rb, 2.88989e+00_rb, 2.91500e+00_rb, 2.94033e+00_rb, &
     & 2.96588e+00_rb, 2.99164e+00_rb, 3.01764e+00_rb, 3.04386e+00_rb/)
      kao_mn2o( 3, :, 9) = (/ &
     & 2.42238e+00_rb, 2.44514e+00_rb, 2.46811e+00_rb, 2.49130e+00_rb, 2.51471e+00_rb, &
     & 2.53834e+00_rb, 2.56219e+00_rb, 2.58626e+00_rb, 2.61056e+00_rb, 2.63509e+00_rb, &
     & 2.65985e+00_rb, 2.68484e+00_rb, 2.71007e+00_rb, 2.73554e+00_rb, 2.76124e+00_rb, &
     & 2.78718e+00_rb, 2.81337e+00_rb, 2.83981e+00_rb, 2.86649e+00_rb/)
      kao_mn2o( 4, :, 9) = (/ &
     & 2.33681e+00_rb, 2.35961e+00_rb, 2.38263e+00_rb, 2.40588e+00_rb, 2.42935e+00_rb, &
     & 2.45305e+00_rb, 2.47699e+00_rb, 2.50115e+00_rb, 2.52556e+00_rb, 2.55020e+00_rb, &
     & 2.57508e+00_rb, 2.60021e+00_rb, 2.62558e+00_rb, 2.65119e+00_rb, 2.67706e+00_rb, &
     & 2.70318e+00_rb, 2.72955e+00_rb, 2.75619e+00_rb, 2.78308e+00_rb/)
      kao_mn2o( 5, :, 9) = (/ &
     & 2.26420e+00_rb, 2.28696e+00_rb, 2.30996e+00_rb, 2.33319e+00_rb, 2.35665e+00_rb, &
     & 2.38035e+00_rb, 2.40429e+00_rb, 2.42846e+00_rb, 2.45288e+00_rb, 2.47755e+00_rb, &
     & 2.50246e+00_rb, 2.52763e+00_rb, 2.55304e+00_rb, 2.57871e+00_rb, 2.60465e+00_rb, &
     & 2.63084e+00_rb, 2.65729e+00_rb, 2.68401e+00_rb, 2.71100e+00_rb/)
      kao_mn2o( 6, :, 9) = (/ &
     & 2.19628e+00_rb, 2.21902e+00_rb, 2.24199e+00_rb, 2.26520e+00_rb, 2.28865e+00_rb, &
     & 2.31234e+00_rb, 2.33628e+00_rb, 2.36047e+00_rb, 2.38491e+00_rb, 2.40959e+00_rb, &
     & 2.43454e+00_rb, 2.45974e+00_rb, 2.48521e+00_rb, 2.51094e+00_rb, 2.53693e+00_rb, &
     & 2.56319e+00_rb, 2.58973e+00_rb, 2.61654e+00_rb, 2.64363e+00_rb/)
      kao_mn2o( 7, :, 9) = (/ &
     & 2.07829e+00_rb, 2.10090e+00_rb, 2.12375e+00_rb, 2.14685e+00_rb, 2.17021e+00_rb, &
     & 2.19381e+00_rb, 2.21767e+00_rb, 2.24180e+00_rb, 2.26618e+00_rb, 2.29083e+00_rb, &
     & 2.31575e+00_rb, 2.34094e+00_rb, 2.36640e+00_rb, 2.39214e+00_rb, 2.41816e+00_rb, &
     & 2.44446e+00_rb, 2.47105e+00_rb, 2.49793e+00_rb, 2.52510e+00_rb/)
      kao_mn2o( 8, :, 9) = (/ &
     & 1.68230e+00_rb, 1.70305e+00_rb, 1.72404e+00_rb, 1.74530e+00_rb, 1.76681e+00_rb, &
     & 1.78860e+00_rb, 1.81065e+00_rb, 1.83297e+00_rb, 1.85557e+00_rb, 1.87845e+00_rb, &
     & 1.90161e+00_rb, 1.92505e+00_rb, 1.94878e+00_rb, 1.97281e+00_rb, 1.99713e+00_rb, &
     & 2.02176e+00_rb, 2.04668e+00_rb, 2.07191e+00_rb, 2.09746e+00_rb/)
      kao_mn2o( 9, :, 9) = (/ &
     & 2.23224e+00_rb, 2.25486e+00_rb, 2.27771e+00_rb, 2.30079e+00_rb, 2.32411e+00_rb, &
     & 2.34766e+00_rb, 2.37145e+00_rb, 2.39548e+00_rb, 2.41975e+00_rb, 2.44427e+00_rb, &
     & 2.46904e+00_rb, 2.49406e+00_rb, 2.51933e+00_rb, 2.54486e+00_rb, 2.57065e+00_rb, &
     & 2.59670e+00_rb, 2.62301e+00_rb, 2.64959e+00_rb, 2.67644e+00_rb/)
      kao_mn2o( 1, :,10) = (/ &
     & 1.30711e+01_rb, 1.31853e+01_rb, 1.33004e+01_rb, 1.34166e+01_rb, 1.35339e+01_rb, &
     & 1.36521e+01_rb, 1.37714e+01_rb, 1.38917e+01_rb, 1.40130e+01_rb, 1.41355e+01_rb, &
     & 1.42590e+01_rb, 1.43835e+01_rb, 1.45092e+01_rb, 1.46360e+01_rb, 1.47638e+01_rb, &
     & 1.48928e+01_rb, 1.50229e+01_rb, 1.51542e+01_rb, 1.52866e+01_rb/)
      kao_mn2o( 2, :,10) = (/ &
     & 2.71206e-01_rb, 2.90148e-01_rb, 3.10413e-01_rb, 3.32093e-01_rb, 3.55287e-01_rb, &
     & 3.80102e-01_rb, 4.06649e-01_rb, 4.35051e-01_rb, 4.65436e-01_rb, 4.97943e-01_rb, &
     & 5.32721e-01_rb, 5.69928e-01_rb, 6.09733e-01_rb, 6.52319e-01_rb, 6.97879e-01_rb, &
     & 7.46621e-01_rb, 7.98767e-01_rb, 8.54555e-01_rb, 9.14239e-01_rb/)
      kao_mn2o( 3, :,10) = (/ &
     & 2.65609e-01_rb, 2.84236e-01_rb, 3.04170e-01_rb, 3.25501e-01_rb, 3.48329e-01_rb, &
     & 3.72758e-01_rb, 3.98900e-01_rb, 4.26875e-01_rb, 4.56812e-01_rb, 4.88849e-01_rb, &
     & 5.23132e-01_rb, 5.59820e-01_rb, 5.99080e-01_rb, 6.41095e-01_rb, 6.86055e-01_rb, &
     & 7.34169e-01_rb, 7.85657e-01_rb, 8.40756e-01_rb, 8.99718e-01_rb/)
      kao_mn2o( 4, :,10) = (/ &
     & 2.55277e-01_rb, 2.73370e-01_rb, 2.92745e-01_rb, 3.13494e-01_rb, 3.35714e-01_rb, &
     & 3.59508e-01_rb, 3.84989e-01_rb, 4.12275e-01_rb, 4.41496e-01_rb, 4.72788e-01_rb, &
     & 5.06298e-01_rb, 5.42183e-01_rb, 5.80611e-01_rb, 6.21763e-01_rb, 6.65831e-01_rb, &
     & 7.13023e-01_rb, 7.63560e-01_rb, 8.17678e-01_rb, 8.75633e-01_rb/)
      kao_mn2o( 5, :,10) = (/ &
     & 2.41481e-01_rb, 2.58840e-01_rb, 2.77446e-01_rb, 2.97390e-01_rb, 3.18768e-01_rb, &
     & 3.41682e-01_rb, 3.66244e-01_rb, 3.92571e-01_rb, 4.20791e-01_rb, 4.51039e-01_rb, &
     & 4.83461e-01_rb, 5.18215e-01_rb, 5.55466e-01_rb, 5.95396e-01_rb, 6.38195e-01_rb, &
     & 6.84071e-01_rb, 7.33245e-01_rb, 7.85954e-01_rb, 8.42452e-01_rb/)
      kao_mn2o( 6, :,10) = (/ &
     & 2.37173e-01_rb, 2.54360e-01_rb, 2.72792e-01_rb, 2.92559e-01_rb, 3.13759e-01_rb, &
     & 3.36495e-01_rb, 3.60878e-01_rb, 3.87029e-01_rb, 4.15074e-01_rb, 4.45152e-01_rb, &
     & 4.77409e-01_rb, 5.12004e-01_rb, 5.49105e-01_rb, 5.88895e-01_rb, 6.31569e-01_rb, &
     & 6.77334e-01_rb, 7.26416e-01_rb, 7.79055e-01_rb, 8.35508e-01_rb/)
      kao_mn2o( 7, :,10) = (/ &
     & 2.27414e-01_rb, 2.44231e-01_rb, 2.62292e-01_rb, 2.81689e-01_rb, 3.02520e-01_rb, &
     & 3.24891e-01_rb, 3.48917e-01_rb, 3.74720e-01_rb, 4.02430e-01_rb, 4.32190e-01_rb, &
     & 4.64151e-01_rb, 4.98475e-01_rb, 5.35337e-01_rb, 5.74926e-01_rb, 6.17442e-01_rb, &
     & 6.63102e-01_rb, 7.12138e-01_rb, 7.64801e-01_rb, 8.21358e-01_rb/)
      kao_mn2o( 8, :,10) = (/ &
     & 1.77234e-01_rb, 1.92029e-01_rb, 2.08060e-01_rb, 2.25429e-01_rb, 2.44248e-01_rb, &
     & 2.64638e-01_rb, 2.86730e-01_rb, 3.10667e-01_rb, 3.36601e-01_rb, 3.64701e-01_rb, &
     & 3.95147e-01_rb, 4.28134e-01_rb, 4.63875e-01_rb, 5.02600e-01_rb, 5.44557e-01_rb, &
     & 5.90017e-01_rb, 6.39272e-01_rb, 6.92639e-01_rb, 7.50461e-01_rb/)
      kao_mn2o( 9, :,10) = (/ &
     & 2.41727e-01_rb, 2.59094e-01_rb, 2.77710e-01_rb, 2.97662e-01_rb, 3.19049e-01_rb, &
     & 3.41972e-01_rb, 3.66541e-01_rb, 3.92877e-01_rb, 4.21104e-01_rb, 4.51359e-01_rb, &
     & 4.83788e-01_rb, 5.18547e-01_rb, 5.55804e-01_rb, 5.95737e-01_rb, 6.38539e-01_rb, &
     & 6.84417e-01_rb, 7.33590e-01_rb, 7.86297e-01_rb, 8.42790e-01_rb/)
      kao_mn2o( 1, :,11) = (/ &
     & 6.65287e+00_rb, 6.69137e+00_rb, 6.73008e+00_rb, 6.76903e+00_rb, 6.80820e+00_rb, &
     & 6.84759e+00_rb, 6.88721e+00_rb, 6.92707e+00_rb, 6.96715e+00_rb, 7.00746e+00_rb, &
     & 7.04801e+00_rb, 7.08879e+00_rb, 7.12981e+00_rb, 7.17107e+00_rb, 7.21256e+00_rb, &
     & 7.25430e+00_rb, 7.29628e+00_rb, 7.33850e+00_rb, 7.38096e+00_rb/)
      kao_mn2o( 2, :,11) = (/ &
     & 2.06252e-01_rb, 2.27731e-01_rb, 2.51447e-01_rb, 2.77633e-01_rb, 3.06546e-01_rb, &
     & 3.38470e-01_rb, 3.73719e-01_rb, 4.12638e-01_rb, 4.55611e-01_rb, 5.03058e-01_rb, &
     & 5.55447e-01_rb, 6.13292e-01_rb, 6.77160e-01_rb, 7.47680e-01_rb, 8.25544e-01_rb, &
     & 9.11517e-01_rb, 1.00644e+00_rb, 1.11125e+00_rb, 1.22698e+00_rb/)
      kao_mn2o( 3, :,11) = (/ &
     & 2.05840e-01_rb, 2.27279e-01_rb, 2.50952e-01_rb, 2.77090e-01_rb, 3.05950e-01_rb, &
     & 3.37816e-01_rb, 3.73002e-01_rb, 4.11852e-01_rb, 4.54748e-01_rb, 5.02113e-01_rb, &
     & 5.54411e-01_rb, 6.12155e-01_rb, 6.75915e-01_rb, 7.46315e-01_rb, 8.24047e-01_rb, &
     & 9.09876e-01_rb, 1.00465e+00_rb, 1.10928e+00_rb, 1.22482e+00_rb/)
      kao_mn2o( 4, :,11) = (/ &
     & 2.04702e-01_rb, 2.26031e-01_rb, 2.49582e-01_rb, 2.75587e-01_rb, 3.04301e-01_rb, &
     & 3.36008e-01_rb, 3.71018e-01_rb, 4.09675e-01_rb, 4.52361e-01_rb, 4.99495e-01_rb, &
     & 5.51539e-01_rb, 6.09007e-01_rb, 6.72461e-01_rb, 7.42528e-01_rb, 8.19895e-01_rb, &
     & 9.05323e-01_rb, 9.99653e-01_rb, 1.10381e+00_rb, 1.21882e+00_rb/)
      kao_mn2o( 5, :,11) = (/ &
     & 2.03481e-01_rb, 2.24689e-01_rb, 2.48108e-01_rb, 2.73967e-01_rb, 3.02522e-01_rb, &
     & 3.34053e-01_rb, 3.68871e-01_rb, 4.07317e-01_rb, 4.49771e-01_rb, 4.96649e-01_rb, &
     & 5.48414e-01_rb, 6.05574e-01_rb, 6.68691e-01_rb, 7.38387e-01_rb, 8.15347e-01_rb, &
     & 9.00329e-01_rb, 9.94168e-01_rb, 1.09779e+00_rb, 1.21221e+00_rb/)
      kao_mn2o( 6, :,11) = (/ &
     & 2.01513e-01_rb, 2.22529e-01_rb, 2.45738e-01_rb, 2.71367e-01_rb, 2.99670e-01_rb, &
     & 3.30924e-01_rb, 3.65437e-01_rb, 4.03550e-01_rb, 4.45639e-01_rb, 4.92117e-01_rb, &
     & 5.43442e-01_rb, 6.00120e-01_rb, 6.62710e-01_rb, 7.31827e-01_rb, 8.08153e-01_rb, &
     & 8.92439e-01_rb, 9.85516e-01_rb, 1.08830e+00_rb, 1.20180e+00_rb/)
      kao_mn2o( 7, :,11) = (/ &
     & 1.97136e-01_rb, 2.17723e-01_rb, 2.40459e-01_rb, 2.65570e-01_rb, 2.93304e-01_rb, &
     & 3.23933e-01_rb, 3.57762e-01_rb, 3.95122e-01_rb, 4.36385e-01_rb, 4.81956e-01_rb, &
     & 5.32287e-01_rb, 5.87873e-01_rb, 6.49264e-01_rb, 7.17067e-01_rb, 7.91950e-01_rb, &
     & 8.74653e-01_rb, 9.65992e-01_rb, 1.06687e+00_rb, 1.17828e+00_rb/)
      kao_mn2o( 8, :,11) = (/ &
     & 1.79518e-01_rb, 1.98371e-01_rb, 2.19204e-01_rb, 2.42224e-01_rb, 2.67662e-01_rb, &
     & 2.95772e-01_rb, 3.26833e-01_rb, 3.61157e-01_rb, 3.99085e-01_rb, 4.40996e-01_rb, &
     & 4.87309e-01_rb, 5.38486e-01_rb, 5.95037e-01_rb, 6.57526e-01_rb, 7.26579e-01_rb, &
     & 8.02883e-01_rb, 8.87201e-01_rb, 9.80373e-01_rb, 1.08333e+00_rb/)
      kao_mn2o( 9, :,11) = (/ &
     & 2.03481e-01_rb, 2.24689e-01_rb, 2.48108e-01_rb, 2.73967e-01_rb, 3.02522e-01_rb, &
     & 3.34053e-01_rb, 3.68871e-01_rb, 4.07317e-01_rb, 4.49771e-01_rb, 4.96649e-01_rb, &
     & 5.48414e-01_rb, 6.05574e-01_rb, 6.68691e-01_rb, 7.38387e-01_rb, 8.15347e-01_rb, &
     & 9.00329e-01_rb, 9.94168e-01_rb, 1.09779e+00_rb, 1.21221e+00_rb/)
      kao_mn2o( 1, :,12) = (/ &
     & 5.89636e+00_rb, 5.95081e+00_rb, 6.00576e+00_rb, 6.06121e+00_rb, 6.11718e+00_rb, &
     & 6.17366e+00_rb, 6.23067e+00_rb, 6.28820e+00_rb, 6.34627e+00_rb, 6.40487e+00_rb, &
     & 6.46401e+00_rb, 6.52369e+00_rb, 6.58393e+00_rb, 6.64472e+00_rb, 6.70608e+00_rb, &
     & 6.76800e+00_rb, 6.83050e+00_rb, 6.89357e+00_rb, 6.95722e+00_rb/)
      kao_mn2o( 2, :,12) = (/ &
     & 7.18699e-05_rb, 9.48140e-05_rb, 1.25083e-04_rb, 1.65015e-04_rb, 2.17695e-04_rb, &
     & 2.87193e-04_rb, 3.78877e-04_rb, 4.99831e-04_rb, 6.59400e-04_rb, 8.69909e-04_rb, &
     & 1.14762e-03_rb, 1.51400e-03_rb, 1.99733e-03_rb, 2.63497e-03_rb, 3.47616e-03_rb, &
     & 4.58591e-03_rb, 6.04993e-03_rb, 7.98133e-03_rb, 1.05293e-02_rb/)
      kao_mn2o( 3, :,12) = (/ &
     & 7.20868e-05_rb, 9.50993e-05_rb, 1.25458e-04_rb, 1.65508e-04_rb, 2.18344e-04_rb, &
     & 2.88046e-04_rb, 3.80000e-04_rb, 5.01307e-04_rb, 6.61341e-04_rb, 8.72462e-04_rb, &
     & 1.15098e-03_rb, 1.51841e-03_rb, 2.00313e-03_rb, 2.64260e-03_rb, 3.48620e-03_rb, &
     & 4.59911e-03_rb, 6.06729e-03_rb, 8.00416e-03_rb, 1.05593e-02_rb/)
      kao_mn2o( 4, :,12) = (/ &
     & 7.21734e-05_rb, 9.52161e-05_rb, 1.25616e-04_rb, 1.65721e-04_rb, 2.18630e-04_rb, &
     & 2.88432e-04_rb, 3.80519e-04_rb, 5.02007e-04_rb, 6.62282e-04_rb, 8.73727e-04_rb, &
     & 1.15268e-03_rb, 1.52070e-03_rb, 2.00621e-03_rb, 2.64673e-03_rb, 3.49174e-03_rb, &
     & 4.60655e-03_rb, 6.07727e-03_rb, 8.01755e-03_rb, 1.05773e-02_rb/)
      kao_mn2o( 5, :,12) = (/ &
     & 7.22599e-05_rb, 9.53329e-05_rb, 1.25773e-04_rb, 1.65933e-04_rb, 2.18916e-04_rb, &
     & 2.88818e-04_rb, 3.81038e-04_rb, 5.02706e-04_rb, 6.63223e-04_rb, 8.74992e-04_rb, &
     & 1.15438e-03_rb, 1.52298e-03_rb, 2.00928e-03_rb, 2.65085e-03_rb, 3.49728e-03_rb, &
     & 4.61398e-03_rb, 6.08725e-03_rb, 8.03094e-03_rb, 1.05952e-02_rb/)
      kao_mn2o( 6, :,12) = (/ &
     & 7.29962e-05_rb, 9.63091e-05_rb, 1.27067e-04_rb, 1.67649e-04_rb, 2.21191e-04_rb, &
     & 2.91833e-04_rb, 3.85036e-04_rb, 5.08005e-04_rb, 6.70247e-04_rb, 8.84304e-04_rb, &
     & 1.16672e-03_rb, 1.53934e-03_rb, 2.03096e-03_rb, 2.67959e-03_rb, 3.53537e-03_rb, &
     & 4.66447e-03_rb, 6.15417e-03_rb, 8.11962e-03_rb, 1.07128e-02_rb/)
      kao_mn2o( 7, :,12) = (/ &
     & 7.47398e-05_rb, 9.86139e-05_rb, 1.30114e-04_rb, 1.71677e-04_rb, 2.26516e-04_rb, &
     & 2.98872e-04_rb, 3.94340e-04_rb, 5.20305e-04_rb, 6.86506e-04_rb, 9.05797e-04_rb, &
     & 1.19514e-03_rb, 1.57690e-03_rb, 2.08061e-03_rb, 2.74522e-03_rb, 3.62213e-03_rb, &
     & 4.77915e-03_rb, 6.30576e-03_rb, 8.32001e-03_rb, 1.09777e-02_rb/)
      kao_mn2o( 8, :,12) = (/ &
     & 7.57487e-05_rb, 9.99802e-05_rb, 1.31963e-04_rb, 1.74177e-04_rb, 2.29896e-04_rb, &
     & 3.03438e-04_rb, 4.00506e-04_rb, 5.28625e-04_rb, 6.97729e-04_rb, 9.20927e-04_rb, &
     & 1.21553e-03_rb, 1.60437e-03_rb, 2.11759e-03_rb, 2.79499e-03_rb, 3.68909e-03_rb, &
     & 4.86921e-03_rb, 6.42684e-03_rb, 8.48274e-03_rb, 1.11963e-02_rb/)
      kao_mn2o( 9, :,12) = (/ &
     & 7.22467e-05_rb, 9.53177e-05_rb, 1.25756e-04_rb, 1.65915e-04_rb, 2.18898e-04_rb, &
     & 2.88800e-04_rb, 3.81024e-04_rb, 5.02700e-04_rb, 6.63231e-04_rb, 8.75024e-04_rb, &
     & 1.15445e-03_rb, 1.52311e-03_rb, 2.00950e-03_rb, 2.65121e-03_rb, 3.49784e-03_rb, &
     & 4.61483e-03_rb, 6.08852e-03_rb, 8.03280e-03_rb, 1.05980e-02_rb/)
      kao_mn2o( 1, :,13) = (/ &
     & 1.14265e+01_rb, 1.16380e+01_rb, 1.18534e+01_rb, 1.20728e+01_rb, 1.22962e+01_rb, &
     & 1.25238e+01_rb, 1.27556e+01_rb, 1.29917e+01_rb, 1.32322e+01_rb, 1.34771e+01_rb, &
     & 1.37265e+01_rb, 1.39806e+01_rb, 1.42394e+01_rb, 1.45029e+01_rb, 1.47714e+01_rb, &
     & 1.50448e+01_rb, 1.53232e+01_rb, 1.56068e+01_rb, 1.58957e+01_rb/)
      kao_mn2o( 2, :,13) = (/ &
     & 7.97796e-05_rb, 1.05659e-04_rb, 1.39932e-04_rb, 1.85324e-04_rb, 2.45439e-04_rb, &
     & 3.25054e-04_rb, 4.30496e-04_rb, 5.70140e-04_rb, 7.55082e-04_rb, 1.00002e-03_rb, &
     & 1.32440e-03_rb, 1.75401e-03_rb, 2.32298e-03_rb, 3.07651e-03_rb, 4.07447e-03_rb, &
     & 5.39614e-03_rb, 7.14655e-03_rb, 9.46475e-03_rb, 1.25349e-02_rb/)
      kao_mn2o( 3, :,13) = (/ &
     & 7.95035e-05_rb, 1.05293e-04_rb, 1.39449e-04_rb, 1.84684e-04_rb, 2.44592e-04_rb, &
     & 3.23934e-04_rb, 4.29013e-04_rb, 5.68178e-04_rb, 7.52486e-04_rb, 9.96580e-04_rb, &
     & 1.31985e-03_rb, 1.74800e-03_rb, 2.31502e-03_rb, 3.06597e-03_rb, 4.06052e-03_rb, &
     & 5.37770e-03_rb, 7.12214e-03_rb, 9.43244e-03_rb, 1.24922e-02_rb/)
      kao_mn2o( 4, :,13) = (/ &
     & 7.92339e-05_rb, 1.04938e-04_rb, 1.38980e-04_rb, 1.84065e-04_rb, 2.43776e-04_rb, &
     & 3.22857e-04_rb, 4.27593e-04_rb, 5.66305e-04_rb, 7.50016e-04_rb, 9.93322e-04_rb, &
     & 1.31556e-03_rb, 1.74233e-03_rb, 2.30754e-03_rb, 3.05612e-03_rb, 4.04752e-03_rb, &
     & 5.36055e-03_rb, 7.09953e-03_rb, 9.40262e-03_rb, 1.24528e-02_rb/)
      kao_mn2o( 5, :,13) = (/ &
     & 7.90000e-05_rb, 1.04627e-04_rb, 1.38566e-04_rb, 1.83516e-04_rb, 2.43046e-04_rb, &
     & 3.21887e-04_rb, 4.26303e-04_rb, 5.64591e-04_rb, 7.47738e-04_rb, 9.90295e-04_rb, &
     & 1.31154e-03_rb, 1.73698e-03_rb, 2.30044e-03_rb, 3.04667e-03_rb, 4.03498e-03_rb, &
     & 5.34388e-03_rb, 7.07737e-03_rb, 9.37318e-03_rb, 1.24137e-02_rb/)
      kao_mn2o( 6, :,13) = (/ &
     & 7.76004e-05_rb, 1.02776e-04_rb, 1.36118e-04_rb, 1.80278e-04_rb, 2.38764e-04_rb, &
     & 3.16224e-04_rb, 4.18814e-04_rb, 5.54686e-04_rb, 7.34638e-04_rb, 9.72970e-04_rb, &
     & 1.28862e-03_rb, 1.70668e-03_rb, 2.26036e-03_rb, 2.99367e-03_rb, 3.96488e-03_rb, &
     & 5.25118e-03_rb, 6.95477e-03_rb, 9.21105e-03_rb, 1.21993e-02_rb/)
      kao_mn2o( 7, :,13) = (/ &
     & 7.52813e-05_rb, 9.97094e-05_rb, 1.32064e-04_rb, 1.74918e-04_rb, 2.31677e-04_rb, &
     & 3.06854e-04_rb, 4.06426e-04_rb, 5.38308e-04_rb, 7.12984e-04_rb, 9.44341e-04_rb, &
     & 1.25077e-03_rb, 1.65664e-03_rb, 2.19420e-03_rb, 2.90620e-03_rb, 3.84923e-03_rb, &
     & 5.09828e-03_rb, 6.75263e-03_rb, 8.94379e-03_rb, 1.18460e-02_rb/)
      kao_mn2o( 8, :,13) = (/ &
     & 6.87436e-05_rb, 9.10605e-05_rb, 1.20622e-04_rb, 1.59781e-04_rb, 2.11653e-04_rb, &
     & 2.80364e-04_rb, 3.71381e-04_rb, 4.91946e-04_rb, 6.51651e-04_rb, 8.63203e-04_rb, &
     & 1.14343e-03_rb, 1.51464e-03_rb, 2.00635e-03_rb, 2.65769e-03_rb, 3.52048e-03_rb, &
     & 4.66337e-03_rb, 6.17729e-03_rb, 8.18269e-03_rb, 1.08391e-02_rb/)
      kao_mn2o( 9, :,13) = (/ &
     & 7.90357e-05_rb, 1.04671e-04_rb, 1.38622e-04_rb, 1.83585e-04_rb, 2.43132e-04_rb, &
     & 3.21994e-04_rb, 4.26434e-04_rb, 5.64750e-04_rb, 7.47931e-04_rb, 9.90526e-04_rb, &
     & 1.31181e-03_rb, 1.73730e-03_rb, 2.30081e-03_rb, 3.04709e-03_rb, 4.03543e-03_rb, &
     & 5.34435e-03_rb, 7.07782e-03_rb, 9.37355e-03_rb, 1.24139e-02_rb/)
      kao_mn2o( 1, :,14) = (/ &
     & 1.61373e+01_rb, 1.64784e+01_rb, 1.68266e+01_rb, 1.71822e+01_rb, 1.75454e+01_rb, &
     & 1.79162e+01_rb, 1.82948e+01_rb, 1.86814e+01_rb, 1.90762e+01_rb, 1.94794e+01_rb, &
     & 1.98911e+01_rb, 2.03114e+01_rb, 2.07407e+01_rb, 2.11790e+01_rb, 2.16266e+01_rb, &
     & 2.20836e+01_rb, 2.25504e+01_rb, 2.30269e+01_rb, 2.35136e+01_rb/)
      kao_mn2o( 2, :,14) = (/ &
     & 6.92866e-10_rb, 9.24655e-10_rb, 1.23398e-09_rb, 1.64680e-09_rb, 2.19771e-09_rb, &
     & 2.93292e-09_rb, 3.91409e-09_rb, 5.22349e-09_rb, 6.97093e-09_rb, 9.30295e-09_rb, &
     & 1.24151e-08_rb, 1.65684e-08_rb, 2.21111e-08_rb, 2.95081e-08_rb, 3.93796e-08_rb, &
     & 5.25535e-08_rb, 7.01346e-08_rb, 9.35970e-08_rb, 1.24908e-07_rb/)
      kao_mn2o( 3, :,14) = (/ &
     & 6.94564e-10_rb, 9.26928e-10_rb, 1.23703e-09_rb, 1.65088e-09_rb, 2.20317e-09_rb, &
     & 2.94024e-09_rb, 3.92389e-09_rb, 5.23661e-09_rb, 6.98851e-09_rb, 9.32650e-09_rb, &
     & 1.24467e-08_rb, 1.66107e-08_rb, 2.21677e-08_rb, 2.95839e-08_rb, 3.94811e-08_rb, &
     & 5.26894e-08_rb, 7.03165e-08_rb, 9.38407e-08_rb, 1.25235e-07_rb/)
      kao_mn2o( 4, :,14) = (/ &
     & 6.98644e-10_rb, 9.32310e-10_rb, 1.24413e-09_rb, 1.66023e-09_rb, 2.21551e-09_rb, &
     & 2.95649e-09_rb, 3.94531e-09_rb, 5.26484e-09_rb, 7.02570e-09_rb, 9.37548e-09_rb, &
     & 1.25112e-08_rb, 1.66956e-08_rb, 2.22795e-08_rb, 2.97311e-08_rb, 3.96748e-08_rb, &
     & 5.29443e-08_rb, 7.06518e-08_rb, 9.42817e-08_rb, 1.25815e-07_rb/)
      kao_mn2o( 5, :,14) = (/ &
     & 7.03261e-10_rb, 9.38472e-10_rb, 1.25235e-09_rb, 1.67121e-09_rb, 2.23016e-09_rb, &
     & 2.97605e-09_rb, 3.97141e-09_rb, 5.29968e-09_rb, 7.07220e-09_rb, 9.43754e-09_rb, &
     & 1.25940e-08_rb, 1.68062e-08_rb, 2.24271e-08_rb, 2.99280e-08_rb, 3.99376e-08_rb, &
     & 5.32951e-08_rb, 7.11200e-08_rb, 9.49066e-08_rb, 1.26649e-07_rb/)
      kao_mn2o( 6, :,14) = (/ &
     & 7.12478e-10_rb, 9.50674e-10_rb, 1.26850e-09_rb, 1.69259e-09_rb, 2.25845e-09_rb, &
     & 3.01350e-09_rb, 4.02096e-09_rb, 5.36525e-09_rb, 7.15896e-09_rb, 9.55233e-09_rb, &
     & 1.27459e-08_rb, 1.70071e-08_rb, 2.26928e-08_rb, 3.02795e-08_rb, 4.04025e-08_rb, &
     & 5.39099e-08_rb, 7.19330e-08_rb, 9.59815e-08_rb, 1.28070e-07_rb/)
      kao_mn2o( 7, :,14) = (/ &
     & 7.28994e-10_rb, 9.72644e-10_rb, 1.29773e-09_rb, 1.73147e-09_rb, 2.31017e-09_rb, &
     & 3.08230e-09_rb, 4.11249e-09_rb, 5.48700e-09_rb, 7.32092e-09_rb, 9.76777e-09_rb, &
     & 1.30324e-08_rb, 1.73883e-08_rb, 2.31999e-08_rb, 3.09540e-08_rb, 4.12996e-08_rb, &
     & 5.51032e-08_rb, 7.35203e-08_rb, 9.80928e-08_rb, 1.30878e-07_rb/)
      kao_mn2o( 8, :,14) = (/ &
     & 7.87604e-10_rb, 1.05043e-09_rb, 1.40097e-09_rb, 1.86848e-09_rb, 2.49201e-09_rb, &
     & 3.32360e-09_rb, 4.43271e-09_rb, 5.91194e-09_rb, 7.88479e-09_rb, 1.05160e-08_rb, &
     & 1.40253e-08_rb, 1.87056e-08_rb, 2.49478e-08_rb, 3.32730e-08_rb, 4.43764e-08_rb, &
     & 5.91851e-08_rb, 7.89356e-08_rb, 1.05277e-07_rb, 1.40408e-07_rb/)
      kao_mn2o( 9, :,14) = (/ &
     & 7.03261e-10_rb, 9.38472e-10_rb, 1.25235e-09_rb, 1.67121e-09_rb, 2.23016e-09_rb, &
     & 2.97605e-09_rb, 3.97141e-09_rb, 5.29968e-09_rb, 7.07220e-09_rb, 9.43754e-09_rb, &
     & 1.25940e-08_rb, 1.68062e-08_rb, 2.24271e-08_rb, 2.99280e-08_rb, 3.99376e-08_rb, &
     & 5.32951e-08_rb, 7.11200e-08_rb, 9.49066e-08_rb, 1.26649e-07_rb/)
      kao_mn2o( 1, :,15) = (/ &
     & 2.14029e+01_rb, 2.16782e+01_rb, 2.19571e+01_rb, 2.22396e+01_rb, 2.25257e+01_rb, &
     & 2.28155e+01_rb, 2.31090e+01_rb, 2.34063e+01_rb, 2.37074e+01_rb, 2.40124e+01_rb, &
     & 2.43213e+01_rb, 2.46342e+01_rb, 2.49511e+01_rb, 2.52721e+01_rb, 2.55972e+01_rb, &
     & 2.59265e+01_rb, 2.62600e+01_rb, 2.65979e+01_rb, 2.69400e+01_rb/)
      kao_mn2o( 2, :,15) = (/ &
     & 5.68659e-10_rb, 7.55629e-10_rb, 1.00407e-09_rb, 1.33421e-09_rb, 1.77288e-09_rb, &
     & 2.35579e-09_rb, 3.13036e-09_rb, 4.15960e-09_rb, 5.52724e-09_rb, 7.34455e-09_rb, &
     & 9.75939e-09_rb, 1.29682e-08_rb, 1.72320e-08_rb, 2.28978e-08_rb, 3.04264e-08_rb, &
     & 4.04304e-08_rb, 5.37236e-08_rb, 7.13875e-08_rb, 9.48591e-08_rb/)
      kao_mn2o( 3, :,15) = (/ &
     & 5.59573e-10_rb, 7.43558e-10_rb, 9.88035e-10_rb, 1.31290e-09_rb, 1.74457e-09_rb, &
     & 2.31817e-09_rb, 3.08037e-09_rb, 4.09318e-09_rb, 5.43900e-09_rb, 7.22730e-09_rb, &
     & 9.60360e-09_rb, 1.27612e-08_rb, 1.69570e-08_rb, 2.25324e-08_rb, 2.99409e-08_rb, &
     & 3.97853e-08_rb, 5.28665e-08_rb, 7.02486e-08_rb, 9.33459e-08_rb/)
      kao_mn2o( 4, :,15) = (/ &
     & 5.50488e-10_rb, 7.31486e-10_rb, 9.71996e-10_rb, 1.29158e-09_rb, 1.71625e-09_rb, &
     & 2.28055e-09_rb, 3.03039e-09_rb, 4.02676e-09_rb, 5.35075e-09_rb, 7.11005e-09_rb, &
     & 9.44781e-09_rb, 1.25542e-08_rb, 1.66820e-08_rb, 2.21670e-08_rb, 2.94554e-08_rb, &
     & 3.91402e-08_rb, 5.20093e-08_rb, 6.91098e-08_rb, 9.18327e-08_rb/)
      kao_mn2o( 5, :,15) = (/ &
     & 5.34010e-10_rb, 7.09574e-10_rb, 9.42858e-10_rb, 1.25284e-09_rb, 1.66473e-09_rb, &
     & 2.21203e-09_rb, 2.93927e-09_rb, 3.90560e-09_rb, 5.18963e-09_rb, 6.89580e-09_rb, &
     & 9.16290e-09_rb, 1.21754e-08_rb, 1.61782e-08_rb, 2.14970e-08_rb, 2.85645e-08_rb, &
     & 3.79555e-08_rb, 5.04340e-08_rb, 6.70149e-08_rb, 8.90470e-08_rb/)
      kao_mn2o( 6, :,15) = (/ &
     & 5.08144e-10_rb, 6.75221e-10_rb, 8.97231e-10_rb, 1.19224e-09_rb, 1.58424e-09_rb, &
     & 2.10513e-09_rb, 2.79729e-09_rb, 3.71703e-09_rb, 4.93919e-09_rb, 6.56317e-09_rb, &
     & 8.72112e-09_rb, 1.15886e-08_rb, 1.53989e-08_rb, 2.04620e-08_rb, 2.71898e-08_rb, &
     & 3.61297e-08_rb, 4.80091e-08_rb, 6.37943e-08_rb, 8.47696e-08_rb/)
      kao_mn2o( 7, :,15) = (/ &
     & 4.56716e-10_rb, 6.06884e-10_rb, 8.06427e-10_rb, 1.07158e-09_rb, 1.42391e-09_rb, &
     & 1.89210e-09_rb, 2.51422e-09_rb, 3.34089e-09_rb, 4.43938e-09_rb, 5.89904e-09_rb, &
     & 7.83864e-09_rb, 1.04160e-08_rb, 1.38408e-08_rb, 1.83916e-08_rb, 2.44387e-08_rb, &
     & 3.24742e-08_rb, 4.31517e-08_rb, 5.73399e-08_rb, 7.61932e-08_rb/)
      kao_mn2o( 8, :,15) = (/ &
     & 2.78366e-10_rb, 3.69881e-10_rb, 4.91482e-10_rb, 6.53061e-10_rb, 8.67760e-10_rb, &
     & 1.15304e-09_rb, 1.53211e-09_rb, 2.03581e-09_rb, 2.70510e-09_rb, 3.59441e-09_rb, &
     & 4.77611e-09_rb, 6.34629e-09_rb, 8.43268e-09_rb, 1.12050e-08_rb, 1.48887e-08_rb, &
     & 1.97835e-08_rb, 2.62875e-08_rb, 3.49296e-08_rb, 4.64130e-08_rb/)
      kao_mn2o( 9, :,15) = (/ &
     & 5.34010e-10_rb, 7.09574e-10_rb, 9.42858e-10_rb, 1.25284e-09_rb, 1.66473e-09_rb, &
     & 2.21203e-09_rb, 2.93927e-09_rb, 3.90560e-09_rb, 5.18963e-09_rb, 6.89580e-09_rb, &
     & 9.16290e-09_rb, 1.21754e-08_rb, 1.61782e-08_rb, 2.14970e-08_rb, 2.85645e-08_rb, &
     & 3.79555e-08_rb, 5.04340e-08_rb, 6.70149e-08_rb, 8.90470e-08_rb/)
      kao_mn2o( 1, :,16) = (/ &
     & 2.90784e+01_rb, 2.93787e+01_rb, 2.96820e+01_rb, 2.99885e+01_rb, 3.02982e+01_rb, &
     & 3.06110e+01_rb, 3.09271e+01_rb, 3.12464e+01_rb, 3.15690e+01_rb, 3.18950e+01_rb, &
     & 3.22243e+01_rb, 3.25571e+01_rb, 3.28932e+01_rb, 3.32329e+01_rb, 3.35760e+01_rb, &
     & 3.39227e+01_rb, 3.42730e+01_rb, 3.46269e+01_rb, 3.49844e+01_rb/)
      kao_mn2o( 2, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)
      kao_mn2o( 3, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)
      kao_mn2o( 4, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)
      kao_mn2o( 5, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)
      kao_mn2o( 6, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)
      kao_mn2o( 7, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)
      kao_mn2o( 8, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)
      kao_mn2o( 9, :,16) = (/ &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, &
     & 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb, 0.00000e+00_rb/)

!     The array KBO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level above 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kbo_mn2o(:, 1) = (/ &
     & 8.42688e-03_rb, 8.96787e-03_rb, 9.54358e-03_rb, 1.01563e-02_rb, 1.08083e-02_rb, &
     & 1.15021e-02_rb, 1.22405e-02_rb, 1.30263e-02_rb, 1.38626e-02_rb, 1.47525e-02_rb, &
     & 1.56996e-02_rb, 1.67075e-02_rb, 1.77800e-02_rb, 1.89215e-02_rb, 2.01362e-02_rb, &
     & 2.14289e-02_rb, 2.28045e-02_rb, 2.42685e-02_rb, 2.58265e-02_rb/)
      kbo_mn2o(:, 2) = (/ &
     & 2.24976e-02_rb, 2.38935e-02_rb, 2.53762e-02_rb, 2.69508e-02_rb, 2.86231e-02_rb, &
     & 3.03991e-02_rb, 3.22854e-02_rb, 3.42887e-02_rb, 3.64163e-02_rb, 3.86760e-02_rb, &
     & 4.10759e-02_rb, 4.36246e-02_rb, 4.63315e-02_rb, 4.92064e-02_rb, 5.22597e-02_rb, &
     & 5.55024e-02_rb, 5.89464e-02_rb, 6.26040e-02_rb, 6.64886e-02_rb/)
      kbo_mn2o(:, 3) = (/ &
     & 5.93542e-02_rb, 6.37312e-02_rb, 6.84310e-02_rb, 7.34774e-02_rb, 7.88960e-02_rb, &
     & 8.47141e-02_rb, 9.09613e-02_rb, 9.76692e-02_rb, 1.04872e-01_rb, 1.12605e-01_rb, &
     & 1.20910e-01_rb, 1.29826e-01_rb, 1.39400e-01_rb, 1.49680e-01_rb, 1.60718e-01_rb, &
     & 1.72570e-01_rb, 1.85296e-01_rb, 1.98961e-01_rb, 2.13633e-01_rb/)
      kbo_mn2o(:, 4) = (/ &
     & 1.98022e-01_rb, 2.05895e-01_rb, 2.14082e-01_rb, 2.22594e-01_rb, 2.31445e-01_rb, &
     & 2.40647e-01_rb, 2.50216e-01_rb, 2.60164e-01_rb, 2.70509e-01_rb, 2.81265e-01_rb, &
     & 2.92448e-01_rb, 3.04076e-01_rb, 3.16167e-01_rb, 3.28738e-01_rb, 3.41809e-01_rb, &
     & 3.55400e-01_rb, 3.69531e-01_rb, 3.84224e-01_rb, 3.99501e-01_rb/)
      kbo_mn2o(:, 5) = (/ &
     & 6.41413e-01_rb, 6.46239e-01_rb, 6.51101e-01_rb, 6.56000e-01_rb, 6.60936e-01_rb, &
     & 6.65910e-01_rb, 6.70920e-01_rb, 6.75968e-01_rb, 6.81054e-01_rb, 6.86179e-01_rb, &
     & 6.91342e-01_rb, 6.96544e-01_rb, 7.01785e-01_rb, 7.07065e-01_rb, 7.12385e-01_rb, &
     & 7.17746e-01_rb, 7.23146e-01_rb, 7.28587e-01_rb, 7.34070e-01_rb/)
      kbo_mn2o(:, 6) = (/ &
     & 1.47906e+00_rb, 1.48768e+00_rb, 1.49635e+00_rb, 1.50507e+00_rb, 1.51384e+00_rb, &
     & 1.52267e+00_rb, 1.53154e+00_rb, 1.54047e+00_rb, 1.54944e+00_rb, 1.55847e+00_rb, &
     & 1.56755e+00_rb, 1.57669e+00_rb, 1.58588e+00_rb, 1.59512e+00_rb, 1.60442e+00_rb, &
     & 1.61377e+00_rb, 1.62317e+00_rb, 1.63263e+00_rb, 1.64215e+00_rb/)
      kbo_mn2o(:, 7) = (/ &
     & 3.53152e+00_rb, 3.55492e+00_rb, 3.57848e+00_rb, 3.60219e+00_rb, 3.62606e+00_rb, &
     & 3.65008e+00_rb, 3.67427e+00_rb, 3.69862e+00_rb, 3.72313e+00_rb, 3.74780e+00_rb, &
     & 3.77263e+00_rb, 3.79763e+00_rb, 3.82279e+00_rb, 3.84812e+00_rb, 3.87362e+00_rb, &
     & 3.89929e+00_rb, 3.92513e+00_rb, 3.95114e+00_rb, 3.97732e+00_rb/)
      kbo_mn2o(:, 8) = (/ &
     & 9.06783e+00_rb, 9.04597e+00_rb, 9.02415e+00_rb, 9.00239e+00_rb, 8.98069e+00_rb, &
     & 8.95903e+00_rb, 8.93743e+00_rb, 8.91588e+00_rb, 8.89438e+00_rb, 8.87293e+00_rb, &
     & 8.85154e+00_rb, 8.83020e+00_rb, 8.80890e+00_rb, 8.78766e+00_rb, 8.76647e+00_rb, &
     & 8.74533e+00_rb, 8.72425e+00_rb, 8.70321e+00_rb, 8.68223e+00_rb/)
      kbo_mn2o(:, 9) = (/ &
     & 3.88220e+01_rb, 3.85805e+01_rb, 3.83405e+01_rb, 3.81019e+01_rb, 3.78649e+01_rb, &
     & 3.76293e+01_rb, 3.73952e+01_rb, 3.71625e+01_rb, 3.69313e+01_rb, 3.67016e+01_rb, &
     & 3.64732e+01_rb, 3.62463e+01_rb, 3.60208e+01_rb, 3.57967e+01_rb, 3.55740e+01_rb, &
     & 3.53527e+01_rb, 3.51327e+01_rb, 3.49142e+01_rb, 3.46970e+01_rb/)
      kbo_mn2o(:, 10) = (/ &
     & 1.14211e+02_rb, 1.13955e+02_rb, 1.13700e+02_rb, 1.13445e+02_rb, 1.13191e+02_rb, &
     & 1.12938e+02_rb, 1.12685e+02_rb, 1.12433e+02_rb, 1.12181e+02_rb, 1.11930e+02_rb, &
     & 1.11679e+02_rb, 1.11429e+02_rb, 1.11180e+02_rb, 1.10931e+02_rb, 1.10682e+02_rb, &
     & 1.10434e+02_rb, 1.10187e+02_rb, 1.09940e+02_rb, 1.09694e+02_rb/)
      kbo_mn2o(:, 11) = (/ &
     & 1.60513e+02_rb, 1.60857e+02_rb, 1.61201e+02_rb, 1.61547e+02_rb, 1.61893e+02_rb, &
     & 1.62240e+02_rb, 1.62587e+02_rb, 1.62936e+02_rb, 1.63285e+02_rb, 1.63635e+02_rb, &
     & 1.63985e+02_rb, 1.64337e+02_rb, 1.64689e+02_rb, 1.65041e+02_rb, 1.65395e+02_rb, &
     & 1.65749e+02_rb, 1.66105e+02_rb, 1.66460e+02_rb, 1.66817e+02_rb/)
      kbo_mn2o(:, 12) = (/ &
     & 1.71473e+02_rb, 1.72766e+02_rb, 1.74068e+02_rb, 1.75381e+02_rb, 1.76703e+02_rb, &
     & 1.78035e+02_rb, 1.79377e+02_rb, 1.80729e+02_rb, 1.82091e+02_rb, 1.83464e+02_rb, &
     & 1.84847e+02_rb, 1.86240e+02_rb, 1.87644e+02_rb, 1.89059e+02_rb, 1.90484e+02_rb, &
     & 1.91920e+02_rb, 1.93367e+02_rb, 1.94824e+02_rb, 1.96293e+02_rb/)
      kbo_mn2o(:, 13) = (/ &
     & 2.71287e+01_rb, 2.75538e+01_rb, 2.79856e+01_rb, 2.84241e+01_rb, 2.88695e+01_rb, &
     & 2.93219e+01_rb, 2.97814e+01_rb, 3.02480e+01_rb, 3.07220e+01_rb, 3.12035e+01_rb, &
     & 3.16924e+01_rb, 3.21890e+01_rb, 3.26934e+01_rb, 3.32058e+01_rb, 3.37261e+01_rb, &
     & 3.42546e+01_rb, 3.47914e+01_rb, 3.53365e+01_rb, 3.58903e+01_rb/)
      kbo_mn2o(:, 14) = (/ &
     & 1.70389e+01_rb, 1.70899e+01_rb, 1.71411e+01_rb, 1.71924e+01_rb, 1.72439e+01_rb, &
     & 1.72955e+01_rb, 1.73473e+01_rb, 1.73992e+01_rb, 1.74513e+01_rb, 1.75035e+01_rb, &
     & 1.75559e+01_rb, 1.76085e+01_rb, 1.76612e+01_rb, 1.77141e+01_rb, 1.77671e+01_rb, &
     & 1.78203e+01_rb, 1.78736e+01_rb, 1.79271e+01_rb, 1.79808e+01_rb/)
      kbo_mn2o(:, 15) = (/ &
     & 2.49725e+00_rb, 2.66861e+00_rb, 2.85174e+00_rb, 3.04743e+00_rb, 3.25655e+00_rb, &
     & 3.48003e+00_rb, 3.71883e+00_rb, 3.97403e+00_rb, 4.24673e+00_rb, 4.53815e+00_rb, &
     & 4.84957e+00_rb, 5.18236e+00_rb, 5.53798e+00_rb, 5.91801e+00_rb, 6.32412e+00_rb, &
     & 6.75809e+00_rb, 7.22185e+00_rb, 7.71742e+00_rb, 8.24701e+00_rb/)
      kbo_mn2o(:, 16) = (/ &
     & 1.82935e-03_rb, 2.58912e-03_rb, 3.66444e-03_rb, 5.18637e-03_rb, 7.34039e-03_rb, &
     & 1.03890e-02_rb, 1.47038e-02_rb, 2.08106e-02_rb, 2.94538e-02_rb, 4.16865e-02_rb, &
     & 5.89999e-02_rb, 8.35040e-02_rb, 1.18185e-01_rb, 1.67270e-01_rb, 2.36741e-01_rb, &
     & 3.35065e-01_rb, 4.74225e-01_rb, 6.71180e-01_rb, 9.49936e-01_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &7.5352e-06_rb,2.9812e-05_rb,1.4497e-04_rb,4.4006e-04_rb,1.0492e-03_rb,1.9676e-03_rb, &
     &1.9989e-03_rb,1.9099e-03_rb,2.2121e-03_rb,2.4491e-03_rb,2.9573e-03_rb,2.6344e-03_rb, &
     &3.0629e-03_rb,3.3547e-03_rb,5.0643e-03_rb,5.0642e-03_rb/)
      forrefo(2,:) = (/ &
     &6.6070e-06_rb,4.8618e-05_rb,3.1112e-04_rb,8.4235e-04_rb,1.4179e-03_rb,1.4315e-03_rb, &
     &1.4685e-03_rb,1.6554e-03_rb,2.1171e-03_rb,2.3545e-03_rb,2.5165e-03_rb,2.7680e-03_rb, &
     &2.6985e-03_rb,3.5345e-03_rb,4.2924e-03_rb,5.0712e-03_rb/)
      forrefo(3,:) = (/ &
     &6.5962e-06_rb,7.2595e-04_rb,1.3429e-03_rb,1.1675e-03_rb,9.8384e-04_rb,8.8787e-04_rb, &
     &8.7557e-04_rb,8.0589e-04_rb,7.7024e-04_rb,8.7518e-04_rb,9.5213e-04_rb,9.0849e-04_rb, &
     &1.2596e-03_rb,2.5106e-03_rb,3.9471e-03_rb,5.0742e-03_rb/)
      forrefo(4,:) = (/ &
     &3.6217e-04_rb,1.0709e-03_rb,1.0628e-03_rb,8.5640e-04_rb,8.9332e-04_rb,8.3372e-04_rb, &
     &7.8539e-04_rb,8.2828e-04_rb,8.3329e-04_rb,8.5118e-04_rb,8.2878e-04_rb,6.8570e-04_rb, &
     &6.3815e-04_rb,8.0648e-04_rb,2.3236e-03_rb,4.0321e-03_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 2.83453e-02_rb, 2.51439e-02_rb, 2.23040e-02_rb, 1.97849e-02_rb, 1.75503e-02_rb, &
     & 1.55681e-02_rb, 1.38097e-02_rb, 1.22500e-02_rb, 1.08664e-02_rb, 9.63912e-03_rb/)
      selfrefo(:, 2) = (/ &
     & 3.05185e-02_rb, 2.72374e-02_rb, 2.43090e-02_rb, 2.16955e-02_rb, 1.93629e-02_rb, &
     & 1.72811e-02_rb, 1.54232e-02_rb, 1.37650e-02_rb, 1.22851e-02_rb, 1.09643e-02_rb/)
      selfrefo(:, 3) = (/ &
     & 4.23833e-02_rb, 3.76250e-02_rb, 3.34010e-02_rb, 2.96512e-02_rb, 2.63223e-02_rb, &
     & 2.33672e-02_rb, 2.07439e-02_rb, 1.84150e-02_rb, 1.63476e-02_rb, 1.45123e-02_rb/)
      selfrefo(:, 4) = (/ &
     & 5.76481e-02_rb, 5.13686e-02_rb, 4.57730e-02_rb, 4.07870e-02_rb, 3.63441e-02_rb, &
     & 3.23851e-02_rb, 2.88574e-02_rb, 2.57140e-02_rb, 2.29130e-02_rb, 2.04171e-02_rb/)
      selfrefo(:, 5) = (/ &
     & 6.92255e-02_rb, 6.33521e-02_rb, 5.79770e-02_rb, 5.30580e-02_rb, 4.85563e-02_rb, &
     & 4.44365e-02_rb, 4.06663e-02_rb, 3.72160e-02_rb, 3.40584e-02_rb, 3.11687e-02_rb/)
      selfrefo(:, 6) = (/ &
     & 6.07694e-02_rb, 5.94182e-02_rb, 5.80970e-02_rb, 5.68052e-02_rb, 5.55422e-02_rb, &
     & 5.43072e-02_rb, 5.30997e-02_rb, 5.19190e-02_rb, 5.07646e-02_rb, 4.96358e-02_rb/)
      selfrefo(:, 7) = (/ &
     & 6.23749e-02_rb, 6.07744e-02_rb, 5.92150e-02_rb, 5.76956e-02_rb, 5.62152e-02_rb, &
     & 5.47728e-02_rb, 5.33674e-02_rb, 5.19980e-02_rb, 5.06638e-02_rb, 4.93638e-02_rb/)
      selfrefo(:, 8) = (/ &
     & 6.90744e-02_rb, 6.61811e-02_rb, 6.34090e-02_rb, 6.07530e-02_rb, 5.82083e-02_rb, &
     & 5.57702e-02_rb, 5.34342e-02_rb, 5.11960e-02_rb, 4.90516e-02_rb, 4.69970e-02_rb/)
      selfrefo(:, 9) = (/ &
     & 8.08992e-02_rb, 7.68876e-02_rb, 7.30750e-02_rb, 6.94514e-02_rb, 6.60075e-02_rb, &
     & 6.27344e-02_rb, 5.96236e-02_rb, 5.66670e-02_rb, 5.38570e-02_rb, 5.11864e-02_rb/)
      selfrefo(:,10) = (/ &
     & 8.70197e-02_rb, 8.27485e-02_rb, 7.86870e-02_rb, 7.48248e-02_rb, 7.11522e-02_rb, &
     & 6.76599e-02_rb, 6.43389e-02_rb, 6.11810e-02_rb, 5.81781e-02_rb, 5.53225e-02_rb/)
      selfrefo(:,11) = (/ &
     & 8.84776e-02_rb, 8.54262e-02_rb, 8.24800e-02_rb, 7.96354e-02_rb, 7.68890e-02_rb, &
     & 7.42373e-02_rb, 7.16770e-02_rb, 6.92050e-02_rb, 6.68183e-02_rb, 6.45139e-02_rb/)
      selfrefo(:,12) = (/ &
     & 9.82552e-02_rb, 9.25696e-02_rb, 8.72130e-02_rb, 8.21664e-02_rb, 7.74118e-02_rb, &
     & 7.29323e-02_rb, 6.87121e-02_rb, 6.47360e-02_rb, 6.09900e-02_rb, 5.74608e-02_rb/)
      selfrefo(:,13) = (/ &
     & 9.32447e-02_rb, 8.96818e-02_rb, 8.62550e-02_rb, 8.29592e-02_rb, 7.97893e-02_rb, &
     & 7.67405e-02_rb, 7.38082e-02_rb, 7.09880e-02_rb, 6.82755e-02_rb, 6.56667e-02_rb/)
      selfrefo(:,14) = (/ &
     & 1.15363e-01_rb, 1.08593e-01_rb, 1.02220e-01_rb, 9.62210e-02_rb, 9.05741e-02_rb, &
     & 8.52585e-02_rb, 8.02549e-02_rb, 7.55450e-02_rb, 7.11115e-02_rb, 6.69382e-02_rb/)
      selfrefo(:,15) = (/ &
     & 1.23179e-01_rb, 1.19247e-01_rb, 1.15440e-01_rb, 1.11755e-01_rb, 1.08187e-01_rb, &
     & 1.04734e-01_rb, 1.01391e-01_rb, 9.81540e-02_rb, 9.50207e-02_rb, 9.19875e-02_rb/)
      selfrefo(:,16) = (/ &
     & 1.44104e-01_rb, 1.36412e-01_rb, 1.29130e-01_rb, 1.22237e-01_rb, 1.15712e-01_rb, &
     & 1.09535e-01_rb, 1.03688e-01_rb, 9.81530e-02_rb, 9.29135e-02_rb, 8.79537e-02_rb/)

      end subroutine lw_kgb09

! **************************************************************************
      subroutine lw_kgb10
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg10, only : fracrefao, fracrefbo, kao, kbo, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P = 212.7250, T = 223.06 K
      fracrefao(:) = (/ &
     &  1.6909e-01_rb, 1.5419e-01_rb, 1.3999e-01_rb, 1.2637e-01_rb, &
     &  1.1429e-01_rb, 9.9676e-02_rb, 8.0093e-02_rb, 6.0283e-02_rb, &
     &  4.1077e-02_rb, 4.4857e-03_rb, 3.6545e-03_rb, 2.9243e-03_rb, &
     &  2.0407e-03_rb, 1.2891e-03_rb, 4.8767e-04_rb, 6.7748e-05_rb/)

! Planck fraction mapping level : P = 95.58350 mb, T = 215.70 K
      fracrefbo(:) = (/ &
     &  1.7391e-01_rb, 1.5680e-01_rb, 1.4419e-01_rb, 1.2672e-01_rb, &
     &  1.0708e-01_rb, 9.7034e-02_rb, 7.8545e-02_rb, 5.9784e-02_rb, &
     &  4.0879e-02_rb, 4.4704e-03_rb, 3.7150e-03_rb, 2.9038e-03_rb, &
     &  2.1454e-03_rb, 1.2802e-03_rb, 4.8328e-04_rb, 6.7378e-05_rb/)

!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels > ~100mb and temperatures.  The first
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the corresponding TREF for this  pressure level, 
!     JT = 2 refers to the temperatureTREF-15, JT = 1 is for TREF-30, 
!     JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  The second 
!     index, JP, runs from 1 to 13 and refers to the corresponding 
!     pressure level in PREF (e.g. JP = 1 is for a pressure of 1053.63 mb).  
!     The third index, IG, goes from 1 to 16, and tells us which 
!     g-interval the absorption coefficients are for.

      kao(:, 1, 1) = (/ &
     &2.7213e-02_rb,2.9029e-02_rb,3.0838e-02_rb,3.2767e-02_rb,3.4630e-02_rb/)
      kao(:, 2, 1) = (/ &
     &2.1383e-02_rb,2.2832e-02_rb,2.4365e-02_rb,2.5925e-02_rb,2.7376e-02_rb/)
      kao(:, 3, 1) = (/ &
     &1.6478e-02_rb,1.7675e-02_rb,1.8942e-02_rb,2.0177e-02_rb,2.1374e-02_rb/)
      kao(:, 4, 1) = (/ &
     &1.2848e-02_rb,1.3809e-02_rb,1.4825e-02_rb,1.5852e-02_rb,1.6799e-02_rb/)
      kao(:, 5, 1) = (/ &
     &1.0029e-02_rb,1.0847e-02_rb,1.1686e-02_rb,1.2512e-02_rb,1.3297e-02_rb/)
      kao(:, 6, 1) = (/ &
     &7.8313e-03_rb,8.5460e-03_rb,9.2025e-03_rb,9.9079e-03_rb,1.0554e-02_rb/)
      kao(:, 7, 1) = (/ &
     &6.1234e-03_rb,6.6846e-03_rb,7.2818e-03_rb,7.8451e-03_rb,8.4144e-03_rb/)
      kao(:, 8, 1) = (/ &
     &4.8747e-03_rb,5.2881e-03_rb,5.7474e-03_rb,6.2355e-03_rb,6.7085e-03_rb/)
      kao(:, 9, 1) = (/ &
     &4.1059e-03_rb,4.5848e-03_rb,4.9152e-03_rb,5.2401e-03_rb,5.5908e-03_rb/)
      kao(:,10, 1) = (/ &
     &3.5412e-03_rb,4.0565e-03_rb,4.5689e-03_rb,5.1092e-03_rb,5.6716e-03_rb/)
      kao(:,11, 1) = (/ &
     &3.0492e-03_rb,3.6436e-03_rb,4.0799e-03_rb,4.5260e-03_rb,4.9802e-03_rb/)
      kao(:,12, 1) = (/ &
     &2.5821e-03_rb,3.0995e-03_rb,3.5069e-03_rb,3.8699e-03_rb,4.2575e-03_rb/)
      kao(:,13, 1) = (/ &
     &2.1558e-03_rb,2.5890e-03_rb,2.9127e-03_rb,3.2139e-03_rb,3.5455e-03_rb/)
      kao(:, 1, 2) = (/ &
     &5.2889e-02_rb,5.6315e-02_rb,5.9927e-02_rb,6.3408e-02_rb,6.6329e-02_rb/)
      kao(:, 2, 2) = (/ &
     &4.1932e-02_rb,4.4934e-02_rb,4.8030e-02_rb,5.0855e-02_rb,5.3372e-02_rb/)
      kao(:, 3, 2) = (/ &
     &3.2754e-02_rb,3.5198e-02_rb,3.7786e-02_rb,4.0294e-02_rb,4.2438e-02_rb/)
      kao(:, 4, 2) = (/ &
     &2.5838e-02_rb,2.7680e-02_rb,2.9873e-02_rb,3.1963e-02_rb,3.3931e-02_rb/)
      kao(:, 5, 2) = (/ &
     &2.0528e-02_rb,2.2079e-02_rb,2.3740e-02_rb,2.5501e-02_rb,2.7219e-02_rb/)
      kao(:, 6, 2) = (/ &
     &1.6350e-02_rb,1.7491e-02_rb,1.8902e-02_rb,2.0280e-02_rb,2.1774e-02_rb/)
      kao(:, 7, 2) = (/ &
     &1.2732e-02_rb,1.3953e-02_rb,1.5005e-02_rb,1.6145e-02_rb,1.7323e-02_rb/)
      kao(:, 8, 2) = (/ &
     &9.7464e-03_rb,1.1159e-02_rb,1.2150e-02_rb,1.2940e-02_rb,1.3856e-02_rb/)
      kao(:, 9, 2) = (/ &
     &7.5337e-03_rb,8.5370e-03_rb,9.6841e-03_rb,1.0825e-02_rb,1.1894e-02_rb/)
      kao(:,10, 2) = (/ &
     &7.1476e-03_rb,7.8468e-03_rb,8.6104e-03_rb,9.4234e-03_rb,1.0165e-02_rb/)
      kao(:,11, 2) = (/ &
     &7.4726e-03_rb,7.8619e-03_rb,8.4110e-03_rb,8.8097e-03_rb,9.3621e-03_rb/)
      kao(:,12, 2) = (/ &
     &6.8212e-03_rb,7.3104e-03_rb,7.6521e-03_rb,8.0570e-03_rb,8.3782e-03_rb/)
      kao(:,13, 2) = (/ &
     &5.7626e-03_rb,6.1115e-03_rb,6.3578e-03_rb,6.6833e-03_rb,6.9834e-03_rb/)
      kao(:, 1, 3) = (/ &
     &9.2909e-02_rb,9.6713e-02_rb,9.9436e-02_rb,1.0242e-01_rb,1.0613e-01_rb/)
      kao(:, 2, 3) = (/ &
     &7.4548e-02_rb,7.7785e-02_rb,8.0363e-02_rb,8.2840e-02_rb,8.6028e-02_rb/)
      kao(:, 3, 3) = (/ &
     &5.8714e-02_rb,6.1823e-02_rb,6.4284e-02_rb,6.6702e-02_rb,6.9231e-02_rb/)
      kao(:, 4, 3) = (/ &
     &4.6349e-02_rb,4.9440e-02_rb,5.1743e-02_rb,5.3890e-02_rb,5.5994e-02_rb/)
      kao(:, 5, 3) = (/ &
     &3.6507e-02_rb,3.9309e-02_rb,4.1637e-02_rb,4.3586e-02_rb,4.5365e-02_rb/)
      kao(:, 6, 3) = (/ &
     &2.8427e-02_rb,3.1177e-02_rb,3.3328e-02_rb,3.5180e-02_rb,3.6733e-02_rb/)
      kao(:, 7, 3) = (/ &
     &2.2397e-02_rb,2.4640e-02_rb,2.6638e-02_rb,2.8311e-02_rb,2.9747e-02_rb/)
      kao(:, 8, 3) = (/ &
     &1.7861e-02_rb,1.9252e-02_rb,2.1033e-02_rb,2.2648e-02_rb,2.3985e-02_rb/)
      kao(:, 9, 3) = (/ &
     &1.4398e-02_rb,1.5884e-02_rb,1.6962e-02_rb,1.7935e-02_rb,1.8840e-02_rb/)
      kao(:,10, 3) = (/ &
     &1.3336e-02_rb,1.5085e-02_rb,1.6848e-02_rb,1.8578e-02_rb,1.9024e-02_rb/)
      kao(:,11, 3) = (/ &
     &1.0996e-02_rb,1.2680e-02_rb,1.4233e-02_rb,1.6184e-02_rb,1.7835e-02_rb/)
      kao(:,12, 3) = (/ &
     &9.6066e-03_rb,1.0608e-02_rb,1.2101e-02_rb,1.3713e-02_rb,1.5338e-02_rb/)
      kao(:,13, 3) = (/ &
     &8.0007e-03_rb,8.9344e-03_rb,1.0260e-02_rb,1.1651e-02_rb,1.2914e-02_rb/)
      kao(:, 1, 4) = (/ &
     &1.4098e-01_rb,1.4735e-01_rb,1.5390e-01_rb,1.6007e-01_rb,1.6623e-01_rb/)
      kao(:, 2, 4) = (/ &
     &1.1373e-01_rb,1.1930e-01_rb,1.2491e-01_rb,1.3057e-01_rb,1.3593e-01_rb/)
      kao(:, 3, 4) = (/ &
     &9.0856e-02_rb,9.5503e-02_rb,1.0012e-01_rb,1.0479e-01_rb,1.0971e-01_rb/)
      kao(:, 4, 4) = (/ &
     &7.2695e-02_rb,7.6746e-02_rb,8.0662e-02_rb,8.4622e-02_rb,8.8881e-02_rb/)
      kao(:, 5, 4) = (/ &
     &5.8402e-02_rb,6.1962e-02_rb,6.5265e-02_rb,6.8694e-02_rb,7.2371e-02_rb/)
      kao(:, 6, 4) = (/ &
     &4.7100e-02_rb,5.0100e-02_rb,5.2955e-02_rb,5.5897e-02_rb,5.9012e-02_rb/)
      kao(:, 7, 4) = (/ &
     &3.7714e-02_rb,4.0303e-02_rb,4.2858e-02_rb,4.5396e-02_rb,4.8032e-02_rb/)
      kao(:, 8, 4) = (/ &
     &2.9938e-02_rb,3.2418e-02_rb,3.4599e-02_rb,3.6801e-02_rb,3.9055e-02_rb/)
      kao(:, 9, 4) = (/ &
     &2.3570e-02_rb,2.5475e-02_rb,2.7580e-02_rb,2.9598e-02_rb,3.1549e-02_rb/)
      kao(:,10, 4) = (/ &
     &2.3291e-02_rb,2.1658e-02_rb,2.1418e-02_rb,2.1490e-02_rb,2.3536e-02_rb/)
      kao(:,11, 4) = (/ &
     &2.1808e-02_rb,2.2150e-02_rb,2.2833e-02_rb,2.0354e-02_rb,1.9688e-02_rb/)
      kao(:,12, 4) = (/ &
     &1.9246e-02_rb,1.9438e-02_rb,1.9839e-02_rb,2.0164e-02_rb,1.7716e-02_rb/)
      kao(:,13, 4) = (/ &
     &1.6236e-02_rb,1.6164e-02_rb,1.6551e-02_rb,1.6579e-02_rb,1.4849e-02_rb/)
      kao(:, 1, 5) = (/ &
     &2.5886e-01_rb,2.7100e-01_rb,2.8220e-01_rb,2.9245e-01_rb,3.0214e-01_rb/)
      kao(:, 2, 5) = (/ &
     &2.0909e-01_rb,2.1918e-01_rb,2.2861e-01_rb,2.3740e-01_rb,2.4610e-01_rb/)
      kao(:, 3, 5) = (/ &
     &1.6622e-01_rb,1.7497e-01_rb,1.8311e-01_rb,1.9088e-01_rb,1.9860e-01_rb/)
      kao(:, 4, 5) = (/ &
     &1.3301e-01_rb,1.4069e-01_rb,1.4792e-01_rb,1.5478e-01_rb,1.6132e-01_rb/)
      kao(:, 5, 5) = (/ &
     &1.0688e-01_rb,1.1355e-01_rb,1.1989e-01_rb,1.2584e-01_rb,1.3154e-01_rb/)
      kao(:, 6, 5) = (/ &
     &8.5419e-02_rb,9.1274e-02_rb,9.6848e-02_rb,1.0202e-01_rb,1.0704e-01_rb/)
      kao(:, 7, 5) = (/ &
     &6.8100e-02_rb,7.3262e-02_rb,7.8047e-02_rb,8.2594e-02_rb,8.7027e-02_rb/)
      kao(:, 8, 5) = (/ &
     &5.4252e-02_rb,5.8815e-02_rb,6.2989e-02_rb,6.6937e-02_rb,7.0783e-02_rb/)
      kao(:, 9, 5) = (/ &
     &4.3117e-02_rb,4.7224e-02_rb,5.0916e-02_rb,5.4328e-02_rb,5.7817e-02_rb/)
      kao(:,10, 5) = (/ &
     &2.9151e-02_rb,3.5667e-02_rb,4.0092e-02_rb,4.3888e-02_rb,4.6870e-02_rb/)
      kao(:,11, 5) = (/ &
     &2.3941e-02_rb,2.7450e-02_rb,2.9750e-02_rb,3.5348e-02_rb,3.9440e-02_rb/)
      kao(:,12, 5) = (/ &
     &2.0570e-02_rb,2.3238e-02_rb,2.5236e-02_rb,2.7327e-02_rb,3.2816e-02_rb/)
      kao(:,13, 5) = (/ &
     &1.7253e-02_rb,1.9755e-02_rb,2.1401e-02_rb,2.3591e-02_rb,2.8114e-02_rb/)
      kao(:, 1, 6) = (/ &
     &5.6993e-01_rb,6.0360e-01_rb,6.3489e-01_rb,6.6230e-01_rb,6.8659e-01_rb/)
      kao(:, 2, 6) = (/ &
     &4.6502e-01_rb,4.9401e-01_rb,5.2007e-01_rb,5.4304e-01_rb,5.6306e-01_rb/)
      kao(:, 3, 6) = (/ &
     &3.7271e-01_rb,3.9757e-01_rb,4.1967e-01_rb,4.3927e-01_rb,4.5631e-01_rb/)
      kao(:, 4, 6) = (/ &
     &2.9874e-01_rb,3.1990e-01_rb,3.3890e-01_rb,3.5594e-01_rb,3.7139e-01_rb/)
      kao(:, 5, 6) = (/ &
     &2.3859e-01_rb,2.5711e-01_rb,2.7375e-01_rb,2.8893e-01_rb,3.0280e-01_rb/)
      kao(:, 6, 6) = (/ &
     &1.8917e-01_rb,2.0549e-01_rb,2.2017e-01_rb,2.3369e-01_rb,2.4611e-01_rb/)
      kao(:, 7, 6) = (/ &
     &1.4938e-01_rb,1.6367e-01_rb,1.7681e-01_rb,1.8887e-01_rb,1.9989e-01_rb/)
      kao(:, 8, 6) = (/ &
     &1.1758e-01_rb,1.3011e-01_rb,1.4157e-01_rb,1.5226e-01_rb,1.6197e-01_rb/)
      kao(:, 9, 6) = (/ &
     &9.1609e-02_rb,1.0230e-01_rb,1.1226e-01_rb,1.2154e-01_rb,1.2990e-01_rb/)
      kao(:,10, 6) = (/ &
     &7.2629e-02_rb,8.2527e-02_rb,9.1786e-02_rb,1.0065e-01_rb,1.0832e-01_rb/)
      kao(:,11, 6) = (/ &
     &6.0173e-02_rb,6.9095e-02_rb,7.8339e-02_rb,8.6340e-02_rb,9.2996e-02_rb/)
      kao(:,12, 6) = (/ &
     &5.1074e-02_rb,5.9514e-02_rb,6.6969e-02_rb,7.3447e-02_rb,7.9277e-02_rb/)
      kao(:,13, 6) = (/ &
     &4.3826e-02_rb,5.0848e-02_rb,5.7125e-02_rb,6.2916e-02_rb,6.7915e-02_rb/)
      kao(:, 1, 7) = (/ &
     &1.4129e+00_rb,1.4750e+00_rb,1.5277e+00_rb,1.5727e+00_rb,1.6117e+00_rb/)
      kao(:, 2, 7) = (/ &
     &1.1749e+00_rb,1.2287e+00_rb,1.2750e+00_rb,1.3159e+00_rb,1.3523e+00_rb/)
      kao(:, 3, 7) = (/ &
     &9.5348e-01_rb,1.0008e+00_rb,1.0433e+00_rb,1.0822e+00_rb,1.1168e+00_rb/)
      kao(:, 4, 7) = (/ &
     &7.6995e-01_rb,8.1381e-01_rb,8.5421e-01_rb,8.9012e-01_rb,9.2111e-01_rb/)
      kao(:, 5, 7) = (/ &
     &6.2211e-01_rb,6.6255e-01_rb,6.9934e-01_rb,7.3149e-01_rb,7.5879e-01_rb/)
      kao(:, 6, 7) = (/ &
     &5.0056e-01_rb,5.3742e-01_rb,5.7038e-01_rb,5.9882e-01_rb,6.2305e-01_rb/)
      kao(:, 7, 7) = (/ &
     &4.0115e-01_rb,4.3352e-01_rb,4.6244e-01_rb,4.8765e-01_rb,5.0920e-01_rb/)
      kao(:, 8, 7) = (/ &
     &3.1902e-01_rb,3.4754e-01_rb,3.7305e-01_rb,3.9544e-01_rb,4.1482e-01_rb/)
      kao(:, 9, 7) = (/ &
     &2.5207e-01_rb,2.7718e-01_rb,2.9962e-01_rb,3.1933e-01_rb,3.3666e-01_rb/)
      kao(:,10, 7) = (/ &
     &1.9498e-01_rb,2.1588e-01_rb,2.3464e-01_rb,2.5127e-01_rb,2.6668e-01_rb/)
      kao(:,11, 7) = (/ &
     &1.6583e-01_rb,1.8332e-01_rb,1.9835e-01_rb,2.1191e-01_rb,2.2523e-01_rb/)
      kao(:,12, 7) = (/ &
     &1.3827e-01_rb,1.5202e-01_rb,1.6547e-01_rb,1.7854e-01_rb,1.8991e-01_rb/)
      kao(:,13, 7) = (/ &
     &1.1534e-01_rb,1.2733e-01_rb,1.3962e-01_rb,1.4987e-01_rb,1.6008e-01_rb/)
      kao(:, 1, 8) = (/ &
     &3.5920e+00_rb,3.7800e+00_rb,3.9524e+00_rb,4.1109e+00_rb,4.2529e+00_rb/)
      kao(:, 2, 8) = (/ &
     &3.2111e+00_rb,3.3886e+00_rb,3.5521e+00_rb,3.6948e+00_rb,3.8225e+00_rb/)
      kao(:, 3, 8) = (/ &
     &2.7787e+00_rb,2.9477e+00_rb,3.0955e+00_rb,3.2271e+00_rb,3.3442e+00_rb/)
      kao(:, 4, 8) = (/ &
     &2.3588e+00_rb,2.5119e+00_rb,2.6472e+00_rb,2.7686e+00_rb,2.8775e+00_rb/)
      kao(:, 5, 8) = (/ &
     &1.9705e+00_rb,2.1082e+00_rb,2.2326e+00_rb,2.3452e+00_rb,2.4470e+00_rb/)
      kao(:, 6, 8) = (/ &
     &1.6168e+00_rb,1.7418e+00_rb,1.8565e+00_rb,1.9616e+00_rb,2.0551e+00_rb/)
      kao(:, 7, 8) = (/ &
     &1.3089e+00_rb,1.4235e+00_rb,1.5288e+00_rb,1.6233e+00_rb,1.7079e+00_rb/)
      kao(:, 8, 8) = (/ &
     &1.0505e+00_rb,1.1539e+00_rb,1.2478e+00_rb,1.3319e+00_rb,1.4095e+00_rb/)
      kao(:, 9, 8) = (/ &
     &8.3583e-01_rb,9.2738e-01_rb,1.0101e+00_rb,1.0861e+00_rb,1.1567e+00_rb/)
      kao(:,10, 8) = (/ &
     &6.6552e-01_rb,7.4516e-01_rb,8.1795e-01_rb,8.8505e-01_rb,9.4596e-01_rb/)
      kao(:,11, 8) = (/ &
     &5.5048e-01_rb,6.1499e-01_rb,6.7571e-01_rb,7.3145e-01_rb,7.8064e-01_rb/)
      kao(:,12, 8) = (/ &
     &4.5804e-01_rb,5.1260e-01_rb,5.6222e-01_rb,6.0624e-01_rb,6.4709e-01_rb/)
      kao(:,13, 8) = (/ &
     &3.8513e-01_rb,4.3049e-01_rb,4.7109e-01_rb,5.0795e-01_rb,5.3980e-01_rb/)
      kao(:, 1, 9) = (/ &
     &1.0443e+01_rb,1.1048e+01_rb,1.1589e+01_rb,1.2078e+01_rb,1.2523e+01_rb/)
      kao(:, 2, 9) = (/ &
     &1.0305e+01_rb,1.0938e+01_rb,1.1513e+01_rb,1.2045e+01_rb,1.2531e+01_rb/)
      kao(:, 3, 9) = (/ &
     &9.8576e+00_rb,1.0529e+01_rb,1.1150e+01_rb,1.1712e+01_rb,1.2221e+01_rb/)
      kao(:, 4, 9) = (/ &
     &9.2371e+00_rb,9.9233e+00_rb,1.0551e+01_rb,1.1124e+01_rb,1.1647e+01_rb/)
      kao(:, 5, 9) = (/ &
     &8.4700e+00_rb,9.1602e+00_rb,9.7951e+00_rb,1.0380e+01_rb,1.0911e+01_rb/)
      kao(:, 6, 9) = (/ &
     &7.5880e+00_rb,8.2763e+00_rb,8.9154e+00_rb,9.5007e+00_rb,1.0024e+01_rb/)
      kao(:, 7, 9) = (/ &
     &6.6790e+00_rb,7.3503e+00_rb,7.9748e+00_rb,8.5357e+00_rb,9.0412e+00_rb/)
      kao(:, 8, 9) = (/ &
     &5.7712e+00_rb,6.4120e+00_rb,7.0009e+00_rb,7.5325e+00_rb,8.0088e+00_rb/)
      kao(:, 9, 9) = (/ &
     &4.8989e+00_rb,5.4909e+00_rb,6.0328e+00_rb,6.5241e+00_rb,6.9680e+00_rb/)
      kao(:,10, 9) = (/ &
     &4.1164e+00_rb,4.6491e+00_rb,5.1402e+00_rb,5.5886e+00_rb,6.0036e+00_rb/)
      kao(:,11, 9) = (/ &
     &3.6151e+00_rb,4.0753e+00_rb,4.5022e+00_rb,4.8994e+00_rb,5.2546e+00_rb/)
      kao(:,12, 9) = (/ &
     &3.1350e+00_rb,3.5329e+00_rb,3.9073e+00_rb,4.2435e+00_rb,4.5415e+00_rb/)
      kao(:,13, 9) = (/ &
     &2.6929e+00_rb,3.0365e+00_rb,3.3480e+00_rb,3.6295e+00_rb,3.8842e+00_rb/)
      kao(:, 1,10) = (/ &
     &1.9924e+01_rb,2.0919e+01_rb,2.2000e+01_rb,2.2972e+01_rb,2.3773e+01_rb/)
      kao(:, 2,10) = (/ &
     &2.0689e+01_rb,2.1909e+01_rb,2.3034e+01_rb,2.3987e+01_rb,2.4788e+01_rb/)
      kao(:, 3,10) = (/ &
     &2.0776e+01_rb,2.2053e+01_rb,2.3211e+01_rb,2.4276e+01_rb,2.5292e+01_rb/)
      kao(:, 4,10) = (/ &
     &2.0236e+01_rb,2.1839e+01_rb,2.3315e+01_rb,2.4662e+01_rb,2.5879e+01_rb/)
      kao(:, 5,10) = (/ &
     &1.9987e+01_rb,2.1808e+01_rb,2.3390e+01_rb,2.4727e+01_rb,2.5958e+01_rb/)
      kao(:, 6,10) = (/ &
     &1.9523e+01_rb,2.1285e+01_rb,2.2798e+01_rb,2.4193e+01_rb,2.5643e+01_rb/)
      kao(:, 7,10) = (/ &
     &1.8326e+01_rb,2.0057e+01_rb,2.1638e+01_rb,2.3310e+01_rb,2.4996e+01_rb/)
      kao(:, 8,10) = (/ &
     &1.6812e+01_rb,1.8592e+01_rb,2.0423e+01_rb,2.2320e+01_rb,2.4092e+01_rb/)
      kao(:, 9,10) = (/ &
     &1.5126e+01_rb,1.7059e+01_rb,1.9069e+01_rb,2.1023e+01_rb,2.2713e+01_rb/)
      kao(:,10,10) = (/ &
     &1.3603e+01_rb,1.5643e+01_rb,1.7659e+01_rb,1.9476e+01_rb,2.1017e+01_rb/)
      kao(:,11,10) = (/ &
     &1.3136e+01_rb,1.5065e+01_rb,1.6788e+01_rb,1.8298e+01_rb,1.9745e+01_rb/)
      kao(:,12,10) = (/ &
     &1.2291e+01_rb,1.3925e+01_rb,1.5389e+01_rb,1.6819e+01_rb,1.8254e+01_rb/)
      kao(:,13,10) = (/ &
     &1.1103e+01_rb,1.2502e+01_rb,1.3897e+01_rb,1.5277e+01_rb,1.6528e+01_rb/)
      kao(:, 1,11) = (/ &
     &2.4296e+01_rb,2.5293e+01_rb,2.6167e+01_rb,2.7129e+01_rb,2.8181e+01_rb/)
      kao(:, 2,11) = (/ &
     &2.5960e+01_rb,2.7094e+01_rb,2.8248e+01_rb,2.9459e+01_rb,3.0485e+01_rb/)
      kao(:, 3,11) = (/ &
     &2.6865e+01_rb,2.8307e+01_rb,2.9751e+01_rb,3.1102e+01_rb,3.2279e+01_rb/)
      kao(:, 4,11) = (/ &
     &2.7354e+01_rb,2.8995e+01_rb,3.0628e+01_rb,3.2100e+01_rb,3.3421e+01_rb/)
      kao(:, 5,11) = (/ &
     &2.6969e+01_rb,2.8892e+01_rb,3.0767e+01_rb,3.2621e+01_rb,3.4273e+01_rb/)
      kao(:, 6,11) = (/ &
     &2.6094e+01_rb,2.8469e+01_rb,3.0801e+01_rb,3.2980e+01_rb,3.4919e+01_rb/)
      kao(:, 7,11) = (/ &
     &2.5366e+01_rb,2.8149e+01_rb,3.0864e+01_rb,3.3154e+01_rb,3.5069e+01_rb/)
      kao(:, 8,11) = (/ &
     &2.4570e+01_rb,2.7568e+01_rb,3.0172e+01_rb,3.2348e+01_rb,3.4412e+01_rb/)
      kao(:, 9,11) = (/ &
     &2.3141e+01_rb,2.6027e+01_rb,2.8564e+01_rb,3.0892e+01_rb,3.3292e+01_rb/)
      kao(:,10,11) = (/ &
     &2.1375e+01_rb,2.4163e+01_rb,2.6748e+01_rb,2.9381e+01_rb,3.2113e+01_rb/)
      kao(:,11,11) = (/ &
     &2.0563e+01_rb,2.3343e+01_rb,2.6200e+01_rb,2.9082e+01_rb,3.1737e+01_rb/)
      kao(:,12,11) = (/ &
     &1.9759e+01_rb,2.2669e+01_rb,2.5592e+01_rb,2.8225e+01_rb,3.0484e+01_rb/)
      kao(:,13,11) = (/ &
     &1.9005e+01_rb,2.1823e+01_rb,2.4335e+01_rb,2.6560e+01_rb,2.8649e+01_rb/)
      kao(:, 1,12) = (/ &
     &2.8554e+01_rb,3.0109e+01_rb,3.1534e+01_rb,3.2820e+01_rb,3.3912e+01_rb/)
      kao(:, 2,12) = (/ &
     &3.1883e+01_rb,3.3547e+01_rb,3.5069e+01_rb,3.6425e+01_rb,3.7803e+01_rb/)
      kao(:, 3,12) = (/ &
     &3.5025e+01_rb,3.6824e+01_rb,3.8424e+01_rb,3.9924e+01_rb,4.1396e+01_rb/)
      kao(:, 4,12) = (/ &
     &3.7112e+01_rb,3.9231e+01_rb,4.1147e+01_rb,4.2932e+01_rb,4.4646e+01_rb/)
      kao(:, 5,12) = (/ &
     &3.8447e+01_rb,4.0861e+01_rb,4.3125e+01_rb,4.5199e+01_rb,4.7297e+01_rb/)
      kao(:, 6,12) = (/ &
     &3.8994e+01_rb,4.1741e+01_rb,4.4237e+01_rb,4.6637e+01_rb,4.8930e+01_rb/)
      kao(:, 7,12) = (/ &
     &3.8693e+01_rb,4.1674e+01_rb,4.4392e+01_rb,4.7287e+01_rb,5.0041e+01_rb/)
      kao(:, 8,12) = (/ &
     &3.7231e+01_rb,4.0534e+01_rb,4.3968e+01_rb,4.7486e+01_rb,5.0745e+01_rb/)
      kao(:, 9,12) = (/ &
     &3.5314e+01_rb,3.9308e+01_rb,4.3458e+01_rb,4.7418e+01_rb,5.1164e+01_rb/)
      kao(:,10,12) = (/ &
     &3.3674e+01_rb,3.8340e+01_rb,4.2937e+01_rb,4.7301e+01_rb,5.1085e+01_rb/)
      kao(:,11,12) = (/ &
     &3.4306e+01_rb,3.9228e+01_rb,4.3781e+01_rb,4.7771e+01_rb,5.1424e+01_rb/)
      kao(:,12,12) = (/ &
     &3.4432e+01_rb,3.9064e+01_rb,4.3274e+01_rb,4.7292e+01_rb,5.1289e+01_rb/)
      kao(:,13,12) = (/ &
     &3.3504e+01_rb,3.7950e+01_rb,4.2337e+01_rb,4.6726e+01_rb,5.1022e+01_rb/)
      kao(:, 1,13) = (/ &
     &3.3083e+01_rb,3.5231e+01_rb,3.7169e+01_rb,3.8902e+01_rb,4.0496e+01_rb/)
      kao(:, 2,13) = (/ &
     &3.7816e+01_rb,4.0283e+01_rb,4.2419e+01_rb,4.4328e+01_rb,4.6116e+01_rb/)
      kao(:, 3,13) = (/ &
     &4.2916e+01_rb,4.5668e+01_rb,4.8087e+01_rb,5.0367e+01_rb,5.2460e+01_rb/)
      kao(:, 4,13) = (/ &
     &4.7941e+01_rb,5.0937e+01_rb,5.3613e+01_rb,5.6174e+01_rb,5.8534e+01_rb/)
      kao(:, 5,13) = (/ &
     &5.2708e+01_rb,5.6040e+01_rb,5.9163e+01_rb,6.2033e+01_rb,6.4593e+01_rb/)
      kao(:, 6,13) = (/ &
     &5.6670e+01_rb,6.0546e+01_rb,6.4072e+01_rb,6.7245e+01_rb,7.0176e+01_rb/)
      kao(:, 7,13) = (/ &
     &5.9263e+01_rb,6.3946e+01_rb,6.8113e+01_rb,7.1753e+01_rb,7.5271e+01_rb/)
      kao(:, 8,13) = (/ &
     &6.0990e+01_rb,6.6317e+01_rb,7.1009e+01_rb,7.5310e+01_rb,7.9458e+01_rb/)
      kao(:, 9,13) = (/ &
     &6.1500e+01_rb,6.7310e+01_rb,7.2473e+01_rb,7.7444e+01_rb,8.1955e+01_rb/)
      kao(:,10,13) = (/ &
     &6.1095e+01_rb,6.7188e+01_rb,7.2950e+01_rb,7.8366e+01_rb,8.3528e+01_rb/)
      kao(:,11,13) = (/ &
     &6.2204e+01_rb,6.8688e+01_rb,7.5122e+01_rb,8.1365e+01_rb,8.7577e+01_rb/)
      kao(:,12,13) = (/ &
     &6.2610e+01_rb,7.0078e+01_rb,7.7313e+01_rb,8.4495e+01_rb,9.1274e+01_rb/)
      kao(:,13,13) = (/ &
     &6.3252e+01_rb,7.1343e+01_rb,7.9418e+01_rb,8.6956e+01_rb,9.3893e+01_rb/)
      kao(:, 1,14) = (/ &
     &4.3344e+01_rb,4.5304e+01_rb,4.7200e+01_rb,4.8977e+01_rb,5.0634e+01_rb/)
      kao(:, 2,14) = (/ &
     &4.8672e+01_rb,5.1219e+01_rb,5.3748e+01_rb,5.6207e+01_rb,5.8584e+01_rb/)
      kao(:, 3,14) = (/ &
     &5.3835e+01_rb,5.7385e+01_rb,6.0811e+01_rb,6.4094e+01_rb,6.7183e+01_rb/)
      kao(:, 4,14) = (/ &
     &5.9051e+01_rb,6.3463e+01_rb,6.7847e+01_rb,7.2047e+01_rb,7.6005e+01_rb/)
      kao(:, 5,14) = (/ &
     &6.4910e+01_rb,7.0372e+01_rb,7.5734e+01_rb,8.0867e+01_rb,8.5466e+01_rb/)
      kao(:, 6,14) = (/ &
     &7.1177e+01_rb,7.7830e+01_rb,8.4454e+01_rb,9.0502e+01_rb,9.5851e+01_rb/)
      kao(:, 7,14) = (/ &
     &7.8645e+01_rb,8.6401e+01_rb,9.3877e+01_rb,1.0067e+02_rb,1.0680e+02_rb/)
      kao(:, 8,14) = (/ &
     &8.6837e+01_rb,9.5972e+01_rb,1.0455e+02_rb,1.1218e+02_rb,1.1889e+02_rb/)
      kao(:, 9,14) = (/ &
     &9.4588e+01_rb,1.0523e+02_rb,1.1528e+02_rb,1.2405e+02_rb,1.3172e+02_rb/)
      kao(:,10,14) = (/ &
     &1.0185e+02_rb,1.1428e+02_rb,1.2585e+02_rb,1.3577e+02_rb,1.4474e+02_rb/)
      kao(:,11,14) = (/ &
     &1.1416e+02_rb,1.2817e+02_rb,1.4064e+02_rb,1.5135e+02_rb,1.6079e+02_rb/)
      kao(:,12,14) = (/ &
     &1.2658e+02_rb,1.4157e+02_rb,1.5496e+02_rb,1.6630e+02_rb,1.7621e+02_rb/)
      kao(:,13,14) = (/ &
     &1.3763e+02_rb,1.5352e+02_rb,1.6709e+02_rb,1.7925e+02_rb,1.9013e+02_rb/)
      kao(:, 1,15) = (/ &
     &5.4748e+01_rb,5.6924e+01_rb,5.8795e+01_rb,6.0375e+01_rb,6.1716e+01_rb/)
      kao(:, 2,15) = (/ &
     &6.5566e+01_rb,6.8287e+01_rb,7.0585e+01_rb,7.2551e+01_rb,7.4160e+01_rb/)
      kao(:, 3,15) = (/ &
     &7.7583e+01_rb,8.1027e+01_rb,8.3928e+01_rb,8.6358e+01_rb,8.8397e+01_rb/)
      kao(:, 4,15) = (/ &
     &8.9871e+01_rb,9.4173e+01_rb,9.7817e+01_rb,1.0084e+02_rb,1.0339e+02_rb/)
      kao(:, 5,15) = (/ &
     &1.0246e+02_rb,1.0776e+02_rb,1.1220e+02_rb,1.1592e+02_rb,1.1961e+02_rb/)
      kao(:, 6,15) = (/ &
     &1.1487e+02_rb,1.2133e+02_rb,1.2665e+02_rb,1.3188e+02_rb,1.3707e+02_rb/)
      kao(:, 7,15) = (/ &
     &1.2683e+02_rb,1.3446e+02_rb,1.4160e+02_rb,1.4872e+02_rb,1.5558e+02_rb/)
      kao(:, 8,15) = (/ &
     &1.3795e+02_rb,1.4756e+02_rb,1.5674e+02_rb,1.6606e+02_rb,1.7523e+02_rb/)
      kao(:, 9,15) = (/ &
     &1.4965e+02_rb,1.6170e+02_rb,1.7284e+02_rb,1.8422e+02_rb,1.9597e+02_rb/)
      kao(:,10,15) = (/ &
     &1.6376e+02_rb,1.7796e+02_rb,1.9116e+02_rb,2.0528e+02_rb,2.1939e+02_rb/)
      kao(:,11,15) = (/ &
     &1.8752e+02_rb,2.0335e+02_rb,2.1913e+02_rb,2.3662e+02_rb,2.5359e+02_rb/)
      kao(:,12,15) = (/ &
     &2.1430e+02_rb,2.3299e+02_rb,2.5241e+02_rb,2.7277e+02_rb,2.9286e+02_rb/)
      kao(:,13,15) = (/ &
     &2.4469e+02_rb,2.6735e+02_rb,2.9101e+02_rb,3.1438e+02_rb,3.3674e+02_rb/)
      kao(:, 1,16) = (/ &
     &5.6182e+01_rb,5.8534e+01_rb,6.0580e+01_rb,6.2336e+01_rb,6.3846e+01_rb/)
      kao(:, 2,16) = (/ &
     &6.7714e+01_rb,7.1253e+01_rb,7.3886e+01_rb,7.6167e+01_rb,7.8129e+01_rb/)
      kao(:, 3,16) = (/ &
     &8.2239e+01_rb,8.6274e+01_rb,8.9810e+01_rb,9.2865e+01_rb,9.5500e+01_rb/)
      kao(:, 4,16) = (/ &
     &9.7623e+01_rb,1.0299e+02_rb,1.0769e+02_rb,1.1178e+02_rb,1.1532e+02_rb/)
      kao(:, 5,16) = (/ &
     &1.1485e+02_rb,1.2193e+02_rb,1.2815e+02_rb,1.3359e+02_rb,1.3829e+02_rb/)
      kao(:, 6,16) = (/ &
     &1.3405e+02_rb,1.4336e+02_rb,1.5157e+02_rb,1.5875e+02_rb,1.6501e+02_rb/)
      kao(:, 7,16) = (/ &
     &1.5562e+02_rb,1.6780e+02_rb,1.7860e+02_rb,1.8805e+02_rb,1.9632e+02_rb/)
      kao(:, 8,16) = (/ &
     &1.7987e+02_rb,1.9571e+02_rb,2.0979e+02_rb,2.2214e+02_rb,2.3298e+02_rb/)
      kao(:, 9,16) = (/ &
     &2.0674e+02_rb,2.2717e+02_rb,2.4538e+02_rb,2.6145e+02_rb,2.7552e+02_rb/)
      kao(:,10,16) = (/ &
     &2.3822e+02_rb,2.6423e+02_rb,2.8739e+02_rb,3.0794e+02_rb,3.2592e+02_rb/)
      kao(:,11,16) = (/ &
     &2.8871e+02_rb,3.1996e+02_rb,3.4774e+02_rb,3.7226e+02_rb,3.9354e+02_rb/)
      kao(:,12,16) = (/ &
     &3.4916e+02_rb,3.8637e+02_rb,4.1932e+02_rb,4.4825e+02_rb,4.7321e+02_rb/)
      kao(:,13,16) = (/ &
     &4.2045e+02_rb,4.6429e+02_rb,5.0303e+02_rb,5.3671e+02_rb,5.6546e+02_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &2.1558e-03_rb,2.5890e-03_rb,2.9127e-03_rb,3.2139e-03_rb,3.5455e-03_rb/)
      kbo(:,14, 1) = (/ &
     &1.8304e-03_rb,2.1809e-03_rb,2.4298e-03_rb,2.6878e-03_rb,2.9625e-03_rb/)
      kbo(:,15, 1) = (/ &
     &1.5486e-03_rb,1.7962e-03_rb,2.0058e-03_rb,2.2286e-03_rb,2.4705e-03_rb/)
      kbo(:,16, 1) = (/ &
     &1.3025e-03_rb,1.4833e-03_rb,1.6606e-03_rb,1.8503e-03_rb,2.0485e-03_rb/)
      kbo(:,17, 1) = (/ &
     &1.0810e-03_rb,1.2227e-03_rb,1.3725e-03_rb,1.5304e-03_rb,1.6938e-03_rb/)
      kbo(:,18, 1) = (/ &
     &8.7942e-04_rb,1.0015e-03_rb,1.1266e-03_rb,1.2596e-03_rb,1.3951e-03_rb/)
      kbo(:,19, 1) = (/ &
     &7.2280e-04_rb,8.2373e-04_rb,9.2739e-04_rb,1.0381e-03_rb,1.1467e-03_rb/)
      kbo(:,20, 1) = (/ &
     &6.0184e-04_rb,6.8491e-04_rb,7.6954e-04_rb,8.5989e-04_rb,9.4913e-04_rb/)
      kbo(:,21, 1) = (/ &
     &5.0117e-04_rb,5.6823e-04_rb,6.3855e-04_rb,7.1320e-04_rb,7.8654e-04_rb/)
      kbo(:,22, 1) = (/ &
     &4.1938e-04_rb,4.7543e-04_rb,5.3375e-04_rb,5.9543e-04_rb,6.5571e-04_rb/)
      kbo(:,23, 1) = (/ &
     &3.5199e-04_rb,3.9855e-04_rb,4.4679e-04_rb,4.9714e-04_rb,5.4750e-04_rb/)
      kbo(:,24, 1) = (/ &
     &2.9639e-04_rb,3.3452e-04_rb,3.7432e-04_rb,4.1615e-04_rb,4.5706e-04_rb/)
      kbo(:,25, 1) = (/ &
     &2.4906e-04_rb,2.8117e-04_rb,3.1412e-04_rb,3.4839e-04_rb,3.7317e-04_rb/)
      kbo(:,26, 1) = (/ &
     &2.1029e-04_rb,2.3678e-04_rb,2.6421e-04_rb,2.9238e-04_rb,3.0575e-04_rb/)
      kbo(:,27, 1) = (/ &
     &1.7727e-04_rb,1.9942e-04_rb,2.2208e-04_rb,2.4556e-04_rb,2.5262e-04_rb/)
      kbo(:,28, 1) = (/ &
     &1.4969e-04_rb,1.6790e-04_rb,1.8692e-04_rb,2.0183e-04_rb,2.0915e-04_rb/)
      kbo(:,29, 1) = (/ &
     &1.2629e-04_rb,1.4151e-04_rb,1.5706e-04_rb,1.6548e-04_rb,1.7223e-04_rb/)
      kbo(:,30, 1) = (/ &
     &1.0651e-04_rb,1.1909e-04_rb,1.3186e-04_rb,1.3639e-04_rb,1.4198e-04_rb/)
      kbo(:,31, 1) = (/ &
     &8.9698e-05_rb,1.0002e-04_rb,1.0798e-04_rb,1.1271e-04_rb,1.1661e-04_rb/)
      kbo(:,32, 1) = (/ &
     &7.5536e-05_rb,8.4037e-05_rb,8.8553e-05_rb,9.2337e-05_rb,9.5468e-05_rb/)
      kbo(:,33, 1) = (/ &
     &6.3563e-05_rb,7.0507e-05_rb,7.3023e-05_rb,7.6153e-05_rb,7.8142e-05_rb/)
      kbo(:,34, 1) = (/ &
     &5.3365e-05_rb,5.7820e-05_rb,6.0490e-05_rb,6.2653e-05_rb,6.4307e-05_rb/)
      kbo(:,35, 1) = (/ &
     &4.4295e-05_rb,4.7505e-05_rb,4.9568e-05_rb,5.1436e-05_rb,5.2756e-05_rb/)
      kbo(:,36, 1) = (/ &
     &3.6359e-05_rb,3.8955e-05_rb,4.0612e-05_rb,4.2071e-05_rb,4.3157e-05_rb/)
      kbo(:,37, 1) = (/ &
     &2.9443e-05_rb,3.1950e-05_rb,3.3272e-05_rb,3.4376e-05_rb,3.5259e-05_rb/)
      kbo(:,38, 1) = (/ &
     &2.3815e-05_rb,2.6274e-05_rb,2.7165e-05_rb,2.8049e-05_rb,2.8811e-05_rb/)
      kbo(:,39, 1) = (/ &
     &1.9245e-05_rb,2.1354e-05_rb,2.2111e-05_rb,2.2930e-05_rb,2.3532e-05_rb/)
      kbo(:,40, 1) = (/ &
     &1.5405e-05_rb,1.7147e-05_rb,1.8049e-05_rb,1.8756e-05_rb,1.9282e-05_rb/)
      kbo(:,41, 1) = (/ &
     &1.2319e-05_rb,1.3748e-05_rb,1.4753e-05_rb,1.5289e-05_rb,1.5770e-05_rb/)
      kbo(:,42, 1) = (/ &
     &9.8357e-06_rb,1.1012e-05_rb,1.2183e-05_rb,1.2499e-05_rb,1.2877e-05_rb/)
      kbo(:,43, 1) = (/ &
     &7.7892e-06_rb,8.7858e-06_rb,9.7457e-06_rb,1.0234e-05_rb,1.0599e-05_rb/)
      kbo(:,44, 1) = (/ &
     &6.1605e-06_rb,6.9805e-06_rb,7.7675e-06_rb,8.4197e-06_rb,8.6950e-06_rb/)
      kbo(:,45, 1) = (/ &
     &4.8809e-06_rb,5.5274e-06_rb,6.1865e-06_rb,6.8268e-06_rb,7.0906e-06_rb/)
      kbo(:,46, 1) = (/ &
     &3.8470e-06_rb,4.3665e-06_rb,4.9029e-06_rb,5.4349e-06_rb,5.8089e-06_rb/)
      kbo(:,47, 1) = (/ &
     &3.0311e-06_rb,3.4294e-06_rb,3.8630e-06_rb,4.3044e-06_rb,4.7453e-06_rb/)
      kbo(:,48, 1) = (/ &
     &2.3694e-06_rb,2.7049e-06_rb,3.0473e-06_rb,3.4008e-06_rb,3.7674e-06_rb/)
      kbo(:,49, 1) = (/ &
     &1.8467e-06_rb,2.1296e-06_rb,2.4020e-06_rb,2.6917e-06_rb,2.9837e-06_rb/)
      kbo(:,50, 1) = (/ &
     &1.4542e-06_rb,1.6765e-06_rb,1.8996e-06_rb,2.1320e-06_rb,2.3692e-06_rb/)
      kbo(:,51, 1) = (/ &
     &1.1539e-06_rb,1.3178e-06_rb,1.5055e-06_rb,1.6911e-06_rb,1.8864e-06_rb/)
      kbo(:,52, 1) = (/ &
     &8.7493e-07_rb,1.0411e-06_rb,1.1898e-06_rb,1.3435e-06_rb,1.4983e-06_rb/)
      kbo(:,53, 1) = (/ &
     &6.6072e-07_rb,8.3282e-07_rb,9.4298e-07_rb,1.0653e-06_rb,1.1935e-06_rb/)
      kbo(:,54, 1) = (/ &
     &5.0251e-07_rb,6.4316e-07_rb,7.5237e-07_rb,8.5108e-07_rb,9.5630e-07_rb/)
      kbo(:,55, 1) = (/ &
     &3.8195e-07_rb,4.9281e-07_rb,6.0554e-07_rb,6.8279e-07_rb,7.6458e-07_rb/)
      kbo(:,56, 1) = (/ &
     &2.8943e-07_rb,3.7779e-07_rb,4.8048e-07_rb,5.4688e-07_rb,6.1650e-07_rb/)
      kbo(:,57, 1) = (/ &
     &2.1868e-07_rb,2.8870e-07_rb,3.7076e-07_rb,4.4677e-07_rb,4.9616e-07_rb/)
      kbo(:,58, 1) = (/ &
     &1.6598e-07_rb,2.2133e-07_rb,2.8693e-07_rb,3.5995e-07_rb,4.0195e-07_rb/)
      kbo(:,59, 1) = (/ &
     &1.3312e-07_rb,1.7804e-07_rb,2.3134e-07_rb,2.9072e-07_rb,3.3304e-07_rb/)
      kbo(:,13, 2) = (/ &
     &5.7626e-03_rb,6.1115e-03_rb,6.3578e-03_rb,6.6833e-03_rb,6.9834e-03_rb/)
      kbo(:,14, 2) = (/ &
     &4.7365e-03_rb,4.9915e-03_rb,5.2669e-03_rb,5.5245e-03_rb,5.7996e-03_rb/)
      kbo(:,15, 2) = (/ &
     &3.8570e-03_rb,4.0469e-03_rb,4.3101e-03_rb,4.5195e-03_rb,4.7607e-03_rb/)
      kbo(:,16, 2) = (/ &
     &3.0836e-03_rb,3.2692e-03_rb,3.4748e-03_rb,3.6812e-03_rb,3.8941e-03_rb/)
      kbo(:,17, 2) = (/ &
     &2.4665e-03_rb,2.6482e-03_rb,2.8018e-03_rb,2.9854e-03_rb,3.1785e-03_rb/)
      kbo(:,18, 2) = (/ &
     &1.9741e-03_rb,2.1245e-03_rb,2.2564e-03_rb,2.4081e-03_rb,2.5780e-03_rb/)
      kbo(:,19, 2) = (/ &
     &1.5958e-03_rb,1.7126e-03_rb,1.8351e-03_rb,1.9560e-03_rb,2.0570e-03_rb/)
      kbo(:,20, 2) = (/ &
     &1.3038e-03_rb,1.3978e-03_rb,1.5013e-03_rb,1.6102e-03_rb,1.6734e-03_rb/)
      kbo(:,21, 2) = (/ &
     &1.0706e-03_rb,1.1492e-03_rb,1.2352e-03_rb,1.3269e-03_rb,1.3755e-03_rb/)
      kbo(:,22, 2) = (/ &
     &8.8260e-04_rb,9.4987e-04_rb,1.0209e-03_rb,1.0834e-03_rb,1.1278e-03_rb/)
      kbo(:,23, 2) = (/ &
     &7.2583e-04_rb,7.8218e-04_rb,8.4637e-04_rb,8.8814e-04_rb,9.2896e-04_rb/)
      kbo(:,24, 2) = (/ &
     &6.0040e-04_rb,6.4872e-04_rb,7.0437e-04_rb,7.3258e-04_rb,7.6936e-04_rb/)
      kbo(:,25, 2) = (/ &
     &4.9798e-04_rb,5.3936e-04_rb,5.8024e-04_rb,6.0451e-04_rb,6.4613e-04_rb/)
      kbo(:,26, 2) = (/ &
     &4.1436e-04_rb,4.5078e-04_rb,4.7645e-04_rb,5.0082e-04_rb,5.4367e-04_rb/)
      kbo(:,27, 2) = (/ &
     &3.4567e-04_rb,3.7682e-04_rb,3.9546e-04_rb,4.1621e-04_rb,4.5625e-04_rb/)
      kbo(:,28, 2) = (/ &
     &2.8902e-04_rb,3.1546e-04_rb,3.2798e-04_rb,3.5075e-04_rb,3.8306e-04_rb/)
      kbo(:,29, 2) = (/ &
     &2.4193e-04_rb,2.5865e-04_rb,2.7208e-04_rb,2.9507e-04_rb,3.2156e-04_rb/)
      kbo(:,30, 2) = (/ &
     &2.0282e-04_rb,2.1375e-04_rb,2.2606e-04_rb,2.4794e-04_rb,2.6904e-04_rb/)
      kbo(:,31, 2) = (/ &
     &1.6986e-04_rb,1.7726e-04_rb,1.9036e-04_rb,2.0754e-04_rb,2.2529e-04_rb/)
      kbo(:,32, 2) = (/ &
     &1.3922e-04_rb,1.4692e-04_rb,1.5999e-04_rb,1.7445e-04_rb,1.8931e-04_rb/)
      kbo(:,33, 2) = (/ &
     &1.1520e-04_rb,1.2220e-04_rb,1.3453e-04_rb,1.4590e-04_rb,1.5875e-04_rb/)
      kbo(:,34, 2) = (/ &
     &9.5764e-05_rb,1.0295e-04_rb,1.1231e-04_rb,1.2221e-04_rb,1.3235e-04_rb/)
      kbo(:,35, 2) = (/ &
     &7.8951e-05_rb,8.5467e-05_rb,9.3381e-05_rb,1.0154e-04_rb,1.0962e-04_rb/)
      kbo(:,36, 2) = (/ &
     &6.4805e-05_rb,7.0212e-05_rb,7.6766e-05_rb,8.3544e-05_rb,9.0139e-05_rb/)
      kbo(:,37, 2) = (/ &
     &5.2918e-05_rb,5.7064e-05_rb,6.2426e-05_rb,6.8102e-05_rb,7.3562e-05_rb/)
      kbo(:,38, 2) = (/ &
     &4.3224e-05_rb,4.6216e-05_rb,5.0772e-05_rb,5.5504e-05_rb,6.0016e-05_rb/)
      kbo(:,39, 2) = (/ &
     &3.5291e-05_rb,3.7592e-05_rb,4.1355e-05_rb,4.5145e-05_rb,4.8932e-05_rb/)
      kbo(:,40, 2) = (/ &
     &2.8761e-05_rb,3.0515e-05_rb,3.3449e-05_rb,3.6463e-05_rb,3.9654e-05_rb/)
      kbo(:,41, 2) = (/ &
     &2.3487e-05_rb,2.4801e-05_rb,2.6922e-05_rb,2.9459e-05_rb,3.2147e-05_rb/)
      kbo(:,42, 2) = (/ &
     &1.9314e-05_rb,2.0243e-05_rb,2.1564e-05_rb,2.3753e-05_rb,2.5964e-05_rb/)
      kbo(:,43, 2) = (/ &
     &1.5553e-05_rb,1.6470e-05_rb,1.7479e-05_rb,1.9035e-05_rb,2.0784e-05_rb/)
      kbo(:,44, 2) = (/ &
     &1.2457e-05_rb,1.3468e-05_rb,1.4190e-05_rb,1.5199e-05_rb,1.6632e-05_rb/)
      kbo(:,45, 2) = (/ &
     &9.9734e-06_rb,1.1008e-05_rb,1.1581e-05_rb,1.2254e-05_rb,1.3346e-05_rb/)
      kbo(:,46, 2) = (/ &
     &7.9698e-06_rb,8.8088e-06_rb,9.4445e-06_rb,9.9340e-06_rb,1.0690e-05_rb/)
      kbo(:,47, 2) = (/ &
     &6.3565e-06_rb,7.0249e-06_rb,7.7396e-06_rb,8.1033e-06_rb,8.5527e-06_rb/)
      kbo(:,48, 2) = (/ &
     &5.1256e-06_rb,5.5929e-06_rb,6.1735e-06_rb,6.6021e-06_rb,6.9371e-06_rb/)
      kbo(:,49, 2) = (/ &
     &4.0784e-06_rb,4.4718e-06_rb,4.9212e-06_rb,5.4190e-06_rb,5.6740e-06_rb/)
      kbo(:,50, 2) = (/ &
     &3.3297e-06_rb,3.6155e-06_rb,3.9436e-06_rb,4.3292e-06_rb,4.6291e-06_rb/)
      kbo(:,51, 2) = (/ &
     &2.7486e-06_rb,2.9107e-06_rb,3.1698e-06_rb,3.4767e-06_rb,3.8139e-06_rb/)
      kbo(:,52, 2) = (/ &
     &2.2530e-06_rb,2.4006e-06_rb,2.5759e-06_rb,2.7925e-06_rb,3.0627e-06_rb/)
      kbo(:,53, 2) = (/ &
     &1.8678e-06_rb,1.9765e-06_rb,2.1052e-06_rb,2.2503e-06_rb,2.4591e-06_rb/)
      kbo(:,54, 2) = (/ &
     &1.5340e-06_rb,1.6385e-06_rb,1.7450e-06_rb,1.8671e-06_rb,1.9805e-06_rb/)
      kbo(:,55, 2) = (/ &
     &1.2873e-06_rb,1.3532e-06_rb,1.4432e-06_rb,1.5328e-06_rb,1.6236e-06_rb/)
      kbo(:,56, 2) = (/ &
     &1.0668e-06_rb,1.1355e-06_rb,1.1865e-06_rb,1.2884e-06_rb,1.3667e-06_rb/)
      kbo(:,57, 2) = (/ &
     &8.4592e-07_rb,9.6149e-07_rb,9.8845e-07_rb,1.0536e-06_rb,1.1286e-06_rb/)
      kbo(:,58, 2) = (/ &
     &6.5721e-07_rb,8.0847e-07_rb,8.4916e-07_rb,8.7196e-07_rb,9.4267e-07_rb/)
      kbo(:,59, 2) = (/ &
     &5.2809e-07_rb,6.6890e-07_rb,7.2431e-07_rb,7.4028e-07_rb,7.8416e-07_rb/)
      kbo(:,13, 3) = (/ &
     &8.0007e-03_rb,8.9344e-03_rb,1.0260e-02_rb,1.1651e-02_rb,1.2914e-02_rb/)
      kbo(:,14, 3) = (/ &
     &6.8039e-03_rb,7.6829e-03_rb,8.7921e-03_rb,9.9141e-03_rb,1.0757e-02_rb/)
      kbo(:,15, 3) = (/ &
     &5.7029e-03_rb,6.5489e-03_rb,7.4475e-03_rb,8.3471e-03_rb,8.8091e-03_rb/)
      kbo(:,16, 3) = (/ &
     &4.8074e-03_rb,5.5085e-03_rb,6.2455e-03_rb,6.9353e-03_rb,7.1997e-03_rb/)
      kbo(:,17, 3) = (/ &
     &4.0213e-03_rb,4.6076e-03_rb,5.2117e-03_rb,5.6731e-03_rb,5.8805e-03_rb/)
      kbo(:,18, 3) = (/ &
     &3.3461e-03_rb,3.8362e-03_rb,4.3242e-03_rb,4.6154e-03_rb,4.7757e-03_rb/)
      kbo(:,19, 3) = (/ &
     &2.7996e-03_rb,3.2075e-03_rb,3.5883e-03_rb,3.7732e-03_rb,3.8872e-03_rb/)
      kbo(:,20, 3) = (/ &
     &2.3589e-03_rb,2.6931e-03_rb,2.9843e-03_rb,3.1004e-03_rb,3.1743e-03_rb/)
      kbo(:,21, 3) = (/ &
     &1.9734e-03_rb,2.2564e-03_rb,2.4531e-03_rb,2.5572e-03_rb,2.5979e-03_rb/)
      kbo(:,22, 3) = (/ &
     &1.6777e-03_rb,1.9014e-03_rb,2.0201e-03_rb,2.0877e-03_rb,2.1290e-03_rb/)
      kbo(:,23, 3) = (/ &
     &1.4255e-03_rb,1.6043e-03_rb,1.6675e-03_rb,1.7121e-03_rb,1.7219e-03_rb/)
      kbo(:,24, 3) = (/ &
     &1.2066e-03_rb,1.3233e-03_rb,1.3782e-03_rb,1.4109e-03_rb,1.3842e-03_rb/)
      kbo(:,25, 3) = (/ &
     &1.0218e-03_rb,1.0936e-03_rb,1.1313e-03_rb,1.1590e-03_rb,1.1132e-03_rb/)
      kbo(:,26, 3) = (/ &
     &8.6686e-04_rb,9.0770e-04_rb,9.3474e-04_rb,9.3738e-04_rb,9.0394e-04_rb/)
      kbo(:,27, 3) = (/ &
     &7.2332e-04_rb,7.5480e-04_rb,7.7347e-04_rb,7.5530e-04_rb,7.3995e-04_rb/)
      kbo(:,28, 3) = (/ &
     &5.9945e-04_rb,6.1747e-04_rb,6.3797e-04_rb,6.1191e-04_rb,6.1018e-04_rb/)
      kbo(:,29, 3) = (/ &
     &4.9636e-04_rb,5.1141e-04_rb,5.1979e-04_rb,4.9718e-04_rb,5.0501e-04_rb/)
      kbo(:,30, 3) = (/ &
     &4.1287e-04_rb,4.2400e-04_rb,4.1392e-04_rb,4.0737e-04_rb,4.1659e-04_rb/)
      kbo(:,31, 3) = (/ &
     &3.3517e-04_rb,3.4906e-04_rb,3.3288e-04_rb,3.3660e-04_rb,3.4335e-04_rb/)
      kbo(:,32, 3) = (/ &
     &2.7784e-04_rb,2.7984e-04_rb,2.7143e-04_rb,2.7785e-04_rb,2.8417e-04_rb/)
      kbo(:,33, 3) = (/ &
     &2.3052e-04_rb,2.2367e-04_rb,2.2307e-04_rb,2.2883e-04_rb,2.3495e-04_rb/)
      kbo(:,34, 3) = (/ &
     &1.9073e-04_rb,1.8192e-04_rb,1.8515e-04_rb,1.8942e-04_rb,1.9507e-04_rb/)
      kbo(:,35, 3) = (/ &
     &1.5683e-04_rb,1.4914e-04_rb,1.5288e-04_rb,1.5663e-04_rb,1.6156e-04_rb/)
      kbo(:,36, 3) = (/ &
     &1.2835e-04_rb,1.2262e-04_rb,1.2564e-04_rb,1.2896e-04_rb,1.3319e-04_rb/)
      kbo(:,37, 3) = (/ &
     &1.0651e-04_rb,1.0086e-04_rb,1.0335e-04_rb,1.0615e-04_rb,1.0970e-04_rb/)
      kbo(:,38, 3) = (/ &
     &8.7372e-05_rb,8.2916e-05_rb,8.4948e-05_rb,8.7192e-05_rb,9.0174e-05_rb/)
      kbo(:,39, 3) = (/ &
     &7.1580e-05_rb,6.8441e-05_rb,6.9733e-05_rb,7.1507e-05_rb,7.4030e-05_rb/)
      kbo(:,40, 3) = (/ &
     &5.8422e-05_rb,5.7750e-05_rb,5.6983e-05_rb,5.8663e-05_rb,6.0682e-05_rb/)
      kbo(:,41, 3) = (/ &
     &4.7623e-05_rb,4.8587e-05_rb,4.6749e-05_rb,4.8145e-05_rb,4.9685e-05_rb/)
      kbo(:,42, 3) = (/ &
     &3.8729e-05_rb,4.0813e-05_rb,3.8776e-05_rb,3.9518e-05_rb,4.0826e-05_rb/)
      kbo(:,43, 3) = (/ &
     &3.2019e-05_rb,3.3273e-05_rb,3.2653e-05_rb,3.2753e-05_rb,3.3527e-05_rb/)
      kbo(:,44, 3) = (/ &
     &2.6412e-05_rb,2.7141e-05_rb,2.7698e-05_rb,2.6940e-05_rb,2.7582e-05_rb/)
      kbo(:,45, 3) = (/ &
     &2.1266e-05_rb,2.2156e-05_rb,2.3306e-05_rb,2.2397e-05_rb,2.2801e-05_rb/)
      kbo(:,46, 3) = (/ &
     &1.7333e-05_rb,1.8301e-05_rb,1.8885e-05_rb,1.9036e-05_rb,1.8747e-05_rb/)
      kbo(:,47, 3) = (/ &
     &1.4070e-05_rb,1.4972e-05_rb,1.5353e-05_rb,1.6172e-05_rb,1.5451e-05_rb/)
      kbo(:,48, 3) = (/ &
     &1.0908e-05_rb,1.2143e-05_rb,1.2701e-05_rb,1.3131e-05_rb,1.3135e-05_rb/)
      kbo(:,49, 3) = (/ &
     &8.5257e-06_rb,9.9029e-06_rb,1.0615e-05_rb,1.0623e-05_rb,1.1229e-05_rb/)
      kbo(:,50, 3) = (/ &
     &6.5831e-06_rb,7.8068e-06_rb,8.5689e-06_rb,8.8823e-06_rb,9.1397e-06_rb/)
      kbo(:,51, 3) = (/ &
     &5.0124e-06_rb,6.1123e-06_rb,6.9735e-06_rb,7.5152e-06_rb,7.4209e-06_rb/)
      kbo(:,52, 3) = (/ &
     &3.8557e-06_rb,4.7153e-06_rb,5.6513e-06_rb,6.0740e-06_rb,6.2588e-06_rb/)
      kbo(:,53, 3) = (/ &
     &2.9713e-06_rb,3.6139e-06_rb,4.3749e-06_rb,4.9165e-06_rb,5.3050e-06_rb/)
      kbo(:,54, 3) = (/ &
     &2.2856e-06_rb,2.7967e-06_rb,3.4018e-06_rb,4.0149e-06_rb,4.3280e-06_rb/)
      kbo(:,55, 3) = (/ &
     &1.7432e-06_rb,2.1952e-06_rb,2.6416e-06_rb,3.2063e-06_rb,3.5207e-06_rb/)
      kbo(:,56, 3) = (/ &
     &1.3460e-06_rb,1.6981e-06_rb,2.0681e-06_rb,2.4690e-06_rb,2.8558e-06_rb/)
      kbo(:,57, 3) = (/ &
     &1.0794e-06_rb,1.2900e-06_rb,1.6260e-06_rb,1.9447e-06_rb,2.3391e-06_rb/)
      kbo(:,58, 3) = (/ &
     &9.2042e-07_rb,9.9682e-07_rb,1.2594e-06_rb,1.5341e-06_rb,1.8245e-06_rb/)
      kbo(:,59, 3) = (/ &
     &7.5487e-07_rb,8.2111e-07_rb,1.0075e-06_rb,1.2341e-06_rb,1.4799e-06_rb/)
      kbo(:,13, 4) = (/ &
     &1.6236e-02_rb,1.6164e-02_rb,1.6551e-02_rb,1.6579e-02_rb,1.4849e-02_rb/)
      kbo(:,14, 4) = (/ &
     &1.3333e-02_rb,1.3358e-02_rb,1.3723e-02_rb,1.3164e-02_rb,1.2310e-02_rb/)
      kbo(:,15, 4) = (/ &
     &1.0791e-02_rb,1.1005e-02_rb,1.1347e-02_rb,1.0262e-02_rb,1.0198e-02_rb/)
      kbo(:,16, 4) = (/ &
     &8.7740e-03_rb,9.0551e-03_rb,8.8988e-03_rb,8.1326e-03_rb,8.4246e-03_rb/)
      kbo(:,17, 4) = (/ &
     &7.1855e-03_rb,7.4271e-03_rb,6.8507e-03_rb,6.6124e-03_rb,6.9447e-03_rb/)
      kbo(:,18, 4) = (/ &
     &5.8832e-03_rb,5.8794e-03_rb,5.3061e-03_rb,5.4156e-03_rb,5.7271e-03_rb/)
      kbo(:,19, 4) = (/ &
     &4.7612e-03_rb,4.5173e-03_rb,4.2495e-03_rb,4.4758e-03_rb,4.7797e-03_rb/)
      kbo(:,20, 4) = (/ &
     &3.8929e-03_rb,3.5371e-03_rb,3.4837e-03_rb,3.7081e-03_rb,4.0335e-03_rb/)
      kbo(:,21, 4) = (/ &
     &3.1210e-03_rb,2.8158e-03_rb,2.9040e-03_rb,3.0717e-03_rb,3.3855e-03_rb/)
      kbo(:,22, 4) = (/ &
     &2.4122e-03_rb,2.2915e-03_rb,2.4348e-03_rb,2.6102e-03_rb,2.8568e-03_rb/)
      kbo(:,23, 4) = (/ &
     &1.8931e-03_rb,1.8847e-03_rb,2.0226e-03_rb,2.2045e-03_rb,2.4293e-03_rb/)
      kbo(:,24, 4) = (/ &
     &1.5231e-03_rb,1.5878e-03_rb,1.6899e-03_rb,1.8552e-03_rb,2.0775e-03_rb/)
      kbo(:,25, 4) = (/ &
     &1.2458e-03_rb,1.3309e-03_rb,1.4317e-03_rb,1.5678e-03_rb,1.7779e-03_rb/)
      kbo(:,26, 4) = (/ &
     &1.0318e-03_rb,1.1118e-03_rb,1.2146e-03_rb,1.3449e-03_rb,1.5065e-03_rb/)
      kbo(:,27, 4) = (/ &
     &8.6665e-04_rb,9.3057e-04_rb,1.0235e-03_rb,1.1557e-03_rb,1.2745e-03_rb/)
      kbo(:,28, 4) = (/ &
     &7.2935e-04_rb,7.9123e-04_rb,8.6656e-04_rb,9.8388e-04_rb,1.0757e-03_rb/)
      kbo(:,29, 4) = (/ &
     &6.1286e-04_rb,6.7240e-04_rb,7.4124e-04_rb,8.3872e-04_rb,9.0911e-04_rb/)
      kbo(:,30, 4) = (/ &
     &5.1400e-04_rb,5.6684e-04_rb,6.4255e-04_rb,7.1220e-04_rb,7.7258e-04_rb/)
      kbo(:,31, 4) = (/ &
     &4.4005e-04_rb,4.8076e-04_rb,5.5017e-04_rb,6.0255e-04_rb,6.5647e-04_rb/)
      kbo(:,32, 4) = (/ &
     &3.7263e-04_rb,4.1679e-04_rb,4.6901e-04_rb,5.1118e-04_rb,5.5709e-04_rb/)
      kbo(:,33, 4) = (/ &
     &3.1452e-04_rb,3.5853e-04_rb,3.9810e-04_rb,4.3526e-04_rb,4.7464e-04_rb/)
      kbo(:,34, 4) = (/ &
     &2.6538e-04_rb,3.0537e-04_rb,3.3607e-04_rb,3.6889e-04_rb,4.0236e-04_rb/)
      kbo(:,35, 4) = (/ &
     &2.2314e-04_rb,2.5674e-04_rb,2.8212e-04_rb,3.1017e-04_rb,3.3804e-04_rb/)
      kbo(:,36, 4) = (/ &
     &1.8558e-04_rb,2.1307e-04_rb,2.3484e-04_rb,2.5807e-04_rb,2.8146e-04_rb/)
      kbo(:,37, 4) = (/ &
     &1.5092e-04_rb,1.7515e-04_rb,1.9371e-04_rb,2.1363e-04_rb,2.3312e-04_rb/)
      kbo(:,38, 4) = (/ &
     &1.2396e-04_rb,1.4379e-04_rb,1.5969e-04_rb,1.7662e-04_rb,1.9284e-04_rb/)
      kbo(:,39, 4) = (/ &
     &1.0157e-04_rb,1.1773e-04_rb,1.3175e-04_rb,1.4587e-04_rb,1.5934e-04_rb/)
      kbo(:,40, 4) = (/ &
     &8.2605e-05_rb,9.4273e-05_rb,1.0813e-04_rb,1.1977e-04_rb,1.3107e-04_rb/)
      kbo(:,41, 4) = (/ &
     &6.6939e-05_rb,7.5416e-05_rb,8.8332e-05_rb,9.8121e-05_rb,1.0774e-04_rb/)
      kbo(:,42, 4) = (/ &
     &5.4049e-05_rb,6.0030e-05_rb,7.1511e-05_rb,8.0199e-05_rb,8.8261e-05_rb/)
      kbo(:,43, 4) = (/ &
     &4.3092e-05_rb,4.8470e-05_rb,5.6902e-05_rb,6.4800e-05_rb,7.1905e-05_rb/)
      kbo(:,44, 4) = (/ &
     &3.4303e-05_rb,3.8888e-05_rb,4.4709e-05_rb,5.2317e-05_rb,5.8294e-05_rb/)
      kbo(:,45, 4) = (/ &
     &2.7706e-05_rb,3.1192e-05_rb,3.5138e-05_rb,4.1880e-05_rb,4.7037e-05_rb/)
      kbo(:,46, 4) = (/ &
     &2.2020e-05_rb,2.4816e-05_rb,2.8235e-05_rb,3.2847e-05_rb,3.7855e-05_rb/)
      kbo(:,47, 4) = (/ &
     &1.7397e-05_rb,1.9571e-05_rb,2.2339e-05_rb,2.5403e-05_rb,3.0154e-05_rb/)
      kbo(:,48, 4) = (/ &
     &1.4464e-05_rb,1.5535e-05_rb,1.7628e-05_rb,2.0160e-05_rb,2.3523e-05_rb/)
      kbo(:,49, 4) = (/ &
     &1.2203e-05_rb,1.2334e-05_rb,1.3809e-05_rb,1.5955e-05_rb,1.8189e-05_rb/)
      kbo(:,50, 4) = (/ &
     &1.0868e-05_rb,1.0166e-05_rb,1.1038e-05_rb,1.2542e-05_rb,1.4482e-05_rb/)
      kbo(:,51, 4) = (/ &
     &9.5445e-06_rb,8.4914e-06_rb,8.8728e-06_rb,9.7788e-06_rb,1.1531e-05_rb/)
      kbo(:,52, 4) = (/ &
     &7.6675e-06_rb,7.5593e-06_rb,7.1200e-06_rb,7.9511e-06_rb,9.0030e-06_rb/)
      kbo(:,53, 4) = (/ &
     &6.1578e-06_rb,6.8621e-06_rb,5.9549e-06_rb,6.4007e-06_rb,7.0103e-06_rb/)
      kbo(:,54, 4) = (/ &
     &4.9970e-06_rb,5.5372e-06_rb,5.2950e-06_rb,5.1133e-06_rb,5.6754e-06_rb/)
      kbo(:,55, 4) = (/ &
     &4.1058e-06_rb,4.4463e-06_rb,4.8381e-06_rb,4.2110e-06_rb,4.6398e-06_rb/)
      kbo(:,56, 4) = (/ &
     &3.3825e-06_rb,3.6285e-06_rb,4.0286e-06_rb,3.7391e-06_rb,3.7193e-06_rb/)
      kbo(:,57, 4) = (/ &
     &2.7087e-06_rb,2.9631e-06_rb,3.2500e-06_rb,3.4185e-06_rb,3.0256e-06_rb/)
      kbo(:,58, 4) = (/ &
     &2.0759e-06_rb,2.4918e-06_rb,2.6378e-06_rb,2.9253e-06_rb,2.6729e-06_rb/)
      kbo(:,59, 4) = (/ &
     &1.6942e-06_rb,2.0927e-06_rb,2.1992e-06_rb,2.4373e-06_rb,2.3395e-06_rb/)
      kbo(:,13, 5) = (/ &
     &1.7253e-02_rb,1.9755e-02_rb,2.1401e-02_rb,2.3591e-02_rb,2.8114e-02_rb/)
      kbo(:,14, 5) = (/ &
     &1.4836e-02_rb,1.6934e-02_rb,1.8341e-02_rb,2.1005e-02_rb,2.4456e-02_rb/)
      kbo(:,15, 5) = (/ &
     &1.2786e-02_rb,1.4254e-02_rb,1.5589e-02_rb,1.8700e-02_rb,2.1272e-02_rb/)
      kbo(:,16, 5) = (/ &
     &1.0892e-02_rb,1.2056e-02_rb,1.3761e-02_rb,1.6336e-02_rb,1.8224e-02_rb/)
      kbo(:,17, 5) = (/ &
     &9.1405e-03_rb,1.0167e-02_rb,1.2171e-02_rb,1.4009e-02_rb,1.5426e-02_rb/)
      kbo(:,18, 5) = (/ &
     &7.6420e-03_rb,8.7762e-03_rb,1.0582e-02_rb,1.1875e-02_rb,1.2969e-02_rb/)
      kbo(:,19, 5) = (/ &
     &6.4312e-03_rb,7.6685e-03_rb,8.9635e-03_rb,9.9226e-03_rb,1.0844e-02_rb/)
      kbo(:,20, 5) = (/ &
     &5.4039e-03_rb,6.6521e-03_rb,7.5702e-03_rb,8.3653e-03_rb,9.0934e-03_rb/)
      kbo(:,21, 5) = (/ &
     &4.6584e-03_rb,5.7137e-03_rb,6.4063e-03_rb,7.0462e-03_rb,7.6384e-03_rb/)
      kbo(:,22, 5) = (/ &
     &4.1485e-03_rb,4.9033e-03_rb,5.4200e-03_rb,5.9610e-03_rb,6.4648e-03_rb/)
      kbo(:,23, 5) = (/ &
     &3.6403e-03_rb,4.1705e-03_rb,4.6055e-03_rb,5.0423e-03_rb,5.4667e-03_rb/)
      kbo(:,24, 5) = (/ &
     &3.1466e-03_rb,3.5285e-03_rb,3.8988e-03_rb,4.2642e-03_rb,4.6134e-03_rb/)
      kbo(:,25, 5) = (/ &
     &2.6968e-03_rb,2.9919e-03_rb,3.3029e-03_rb,3.6014e-03_rb,3.8905e-03_rb/)
      kbo(:,26, 5) = (/ &
     &2.2919e-03_rb,2.5427e-03_rb,2.7966e-03_rb,3.0460e-03_rb,3.3030e-03_rb/)
      kbo(:,27, 5) = (/ &
     &1.9487e-03_rb,2.1617e-03_rb,2.3708e-03_rb,2.5760e-03_rb,2.8001e-03_rb/)
      kbo(:,28, 5) = (/ &
     &1.6604e-03_rb,1.8339e-03_rb,2.0064e-03_rb,2.1858e-03_rb,2.3683e-03_rb/)
      kbo(:,29, 5) = (/ &
     &1.4167e-03_rb,1.5573e-03_rb,1.7005e-03_rb,1.8546e-03_rb,2.0015e-03_rb/)
      kbo(:,30, 5) = (/ &
     &1.2043e-03_rb,1.3228e-03_rb,1.4435e-03_rb,1.5699e-03_rb,1.6896e-03_rb/)
      kbo(:,31, 5) = (/ &
     &1.0225e-03_rb,1.1229e-03_rb,1.2273e-03_rb,1.3283e-03_rb,1.4296e-03_rb/)
      kbo(:,32, 5) = (/ &
     &8.6992e-04_rb,9.5293e-04_rb,1.0416e-03_rb,1.1243e-03_rb,1.2104e-03_rb/)
      kbo(:,33, 5) = (/ &
     &7.3968e-04_rb,8.1278e-04_rb,8.8420e-04_rb,9.5333e-04_rb,1.0276e-03_rb/)
      kbo(:,34, 5) = (/ &
     &6.2724e-04_rb,6.8945e-04_rb,7.4802e-04_rb,8.0740e-04_rb,8.7396e-04_rb/)
      kbo(:,35, 5) = (/ &
     &5.2729e-04_rb,5.7995e-04_rb,6.2910e-04_rb,6.7983e-04_rb,7.3853e-04_rb/)
      kbo(:,36, 5) = (/ &
     &4.3895e-04_rb,4.8339e-04_rb,5.2479e-04_rb,5.6936e-04_rb,6.1891e-04_rb/)
      kbo(:,37, 5) = (/ &
     &3.6323e-04_rb,4.0071e-04_rb,4.3613e-04_rb,4.7458e-04_rb,5.1673e-04_rb/)
      kbo(:,38, 5) = (/ &
     &3.0001e-04_rb,3.3215e-04_rb,3.6263e-04_rb,3.9587e-04_rb,4.3165e-04_rb/)
      kbo(:,39, 5) = (/ &
     &2.4808e-04_rb,2.7543e-04_rb,3.0133e-04_rb,3.3050e-04_rb,3.6200e-04_rb/)
      kbo(:,40, 5) = (/ &
     &2.0392e-04_rb,2.2766e-04_rb,2.5038e-04_rb,2.7569e-04_rb,3.0329e-04_rb/)
      kbo(:,41, 5) = (/ &
     &1.6718e-04_rb,1.8754e-04_rb,2.0771e-04_rb,2.3036e-04_rb,2.5452e-04_rb/)
      kbo(:,42, 5) = (/ &
     &1.3676e-04_rb,1.5432e-04_rb,1.7210e-04_rb,1.9216e-04_rb,2.1391e-04_rb/)
      kbo(:,43, 5) = (/ &
     &1.1103e-04_rb,1.2612e-04_rb,1.4186e-04_rb,1.5938e-04_rb,1.7898e-04_rb/)
      kbo(:,44, 5) = (/ &
     &8.9549e-05_rb,1.0259e-04_rb,1.1643e-04_rb,1.3169e-04_rb,1.4912e-04_rb/)
      kbo(:,45, 5) = (/ &
     &7.2088e-05_rb,8.3098e-05_rb,9.5307e-05_rb,1.0860e-04_rb,1.2373e-04_rb/)
      kbo(:,46, 5) = (/ &
     &5.7589e-05_rb,6.6833e-05_rb,7.7497e-05_rb,8.9096e-05_rb,1.0212e-04_rb/)
      kbo(:,47, 5) = (/ &
     &4.5404e-05_rb,5.3366e-05_rb,6.2424e-05_rb,7.2413e-05_rb,8.3476e-05_rb/)
      kbo(:,48, 5) = (/ &
     &3.5296e-05_rb,4.2388e-05_rb,4.9972e-05_rb,5.8618e-05_rb,6.7991e-05_rb/)
      kbo(:,49, 5) = (/ &
     &2.7057e-05_rb,3.3322e-05_rb,3.9733e-05_rb,4.7217e-05_rb,5.5093e-05_rb/)
      kbo(:,50, 5) = (/ &
     &2.0028e-05_rb,2.6063e-05_rb,3.1674e-05_rb,3.7934e-05_rb,4.4693e-05_rb/)
      kbo(:,51, 5) = (/ &
     &1.4762e-05_rb,2.0261e-05_rb,2.5067e-05_rb,3.0345e-05_rb,3.6144e-05_rb/)
      kbo(:,52, 5) = (/ &
     &1.1471e-05_rb,1.5106e-05_rb,1.9721e-05_rb,2.4107e-05_rb,2.9127e-05_rb/)
      kbo(:,53, 5) = (/ &
     &8.8078e-06_rb,1.0927e-05_rb,1.5370e-05_rb,1.9101e-05_rb,2.3328e-05_rb/)
      kbo(:,54, 5) = (/ &
     &6.7922e-06_rb,8.5333e-06_rb,1.1608e-05_rb,1.5114e-05_rb,1.8676e-05_rb/)
      kbo(:,55, 5) = (/ &
     &5.2087e-06_rb,6.6736e-06_rb,8.5401e-06_rb,1.1909e-05_rb,1.4884e-05_rb/)
      kbo(:,56, 5) = (/ &
     &3.9778e-06_rb,5.1458e-06_rb,6.5614e-06_rb,9.1281e-06_rb,1.1849e-05_rb/)
      kbo(:,57, 5) = (/ &
     &3.4293e-06_rb,3.9957e-06_rb,5.1202e-06_rb,6.7753e-06_rb,9.3431e-06_rb/)
      kbo(:,58, 5) = (/ &
     &3.0812e-06_rb,3.0418e-06_rb,3.9938e-06_rb,5.1720e-06_rb,7.2446e-06_rb/)
      kbo(:,59, 5) = (/ &
     &2.6710e-06_rb,2.4462e-06_rb,3.2530e-06_rb,4.2384e-06_rb,5.8592e-06_rb/)
      kbo(:,13, 6) = (/ &
     &4.3826e-02_rb,5.0848e-02_rb,5.7125e-02_rb,6.2916e-02_rb,6.7915e-02_rb/)
      kbo(:,14, 6) = (/ &
     &3.8242e-02_rb,4.3851e-02_rb,4.9242e-02_rb,5.3907e-02_rb,5.8385e-02_rb/)
      kbo(:,15, 6) = (/ &
     &3.3170e-02_rb,3.7873e-02_rb,4.2212e-02_rb,4.6203e-02_rb,5.0021e-02_rb/)
      kbo(:,16, 6) = (/ &
     &2.8430e-02_rb,3.2381e-02_rb,3.6071e-02_rb,3.9408e-02_rb,4.2554e-02_rb/)
      kbo(:,17, 6) = (/ &
     &2.4252e-02_rb,2.7453e-02_rb,3.0545e-02_rb,3.3265e-02_rb,3.5923e-02_rb/)
      kbo(:,18, 6) = (/ &
     &2.0461e-02_rb,2.3166e-02_rb,2.5616e-02_rb,2.7856e-02_rb,3.0048e-02_rb/)
      kbo(:,19, 6) = (/ &
     &1.7273e-02_rb,1.9462e-02_rb,2.1473e-02_rb,2.3367e-02_rb,2.5163e-02_rb/)
      kbo(:,20, 6) = (/ &
     &1.4713e-02_rb,1.6450e-02_rb,1.8126e-02_rb,1.9702e-02_rb,2.1262e-02_rb/)
      kbo(:,21, 6) = (/ &
     &1.2468e-02_rb,1.3887e-02_rb,1.5268e-02_rb,1.6611e-02_rb,1.7949e-02_rb/)
      kbo(:,22, 6) = (/ &
     &1.0608e-02_rb,1.1782e-02_rb,1.2961e-02_rb,1.4100e-02_rb,1.5205e-02_rb/)
      kbo(:,23, 6) = (/ &
     &8.9996e-03_rb,9.9872e-03_rb,1.1001e-02_rb,1.1992e-02_rb,1.2875e-02_rb/)
      kbo(:,24, 6) = (/ &
     &7.6639e-03_rb,8.5225e-03_rb,9.3703e-03_rb,1.0166e-02_rb,1.0905e-02_rb/)
      kbo(:,25, 6) = (/ &
     &6.5402e-03_rb,7.2570e-03_rb,7.9762e-03_rb,8.6328e-03_rb,9.2482e-03_rb/)
      kbo(:,26, 6) = (/ &
     &5.5998e-03_rb,6.2109e-03_rb,6.7992e-03_rb,7.3298e-03_rb,7.8244e-03_rb/)
      kbo(:,27, 6) = (/ &
     &4.7932e-03_rb,5.3061e-03_rb,5.7794e-03_rb,6.2257e-03_rb,6.6438e-03_rb/)
      kbo(:,28, 6) = (/ &
     &4.0951e-03_rb,4.5312e-03_rb,4.9233e-03_rb,5.2892e-03_rb,5.6437e-03_rb/)
      kbo(:,29, 6) = (/ &
     &3.5062e-03_rb,3.8638e-03_rb,4.1941e-03_rb,4.4995e-03_rb,4.7944e-03_rb/)
      kbo(:,30, 6) = (/ &
     &2.9997e-03_rb,3.2915e-03_rb,3.5642e-03_rb,3.8238e-03_rb,4.0827e-03_rb/)
      kbo(:,31, 6) = (/ &
     &2.5609e-03_rb,2.8021e-03_rb,3.0278e-03_rb,3.2471e-03_rb,3.4700e-03_rb/)
      kbo(:,32, 6) = (/ &
     &2.1838e-03_rb,2.3823e-03_rb,2.5746e-03_rb,2.7615e-03_rb,2.9515e-03_rb/)
      kbo(:,33, 6) = (/ &
     &1.8610e-03_rb,2.0261e-03_rb,2.1868e-03_rb,2.3490e-03_rb,2.5125e-03_rb/)
      kbo(:,34, 6) = (/ &
     &1.5812e-03_rb,1.7190e-03_rb,1.8561e-03_rb,1.9953e-03_rb,2.1347e-03_rb/)
      kbo(:,35, 6) = (/ &
     &1.3323e-03_rb,1.4487e-03_rb,1.5664e-03_rb,1.6864e-03_rb,1.8025e-03_rb/)
      kbo(:,36, 6) = (/ &
     &1.1120e-03_rb,1.2101e-03_rb,1.3111e-03_rb,1.4134e-03_rb,1.5124e-03_rb/)
      kbo(:,37, 6) = (/ &
     &9.2067e-04_rb,1.0045e-03_rb,1.0903e-03_rb,1.1790e-03_rb,1.2650e-03_rb/)
      kbo(:,38, 6) = (/ &
     &7.6186e-04_rb,8.3301e-04_rb,9.0690e-04_rb,9.8204e-04_rb,1.0582e-03_rb/)
      kbo(:,39, 6) = (/ &
     &6.3121e-04_rb,6.9074e-04_rb,7.5442e-04_rb,8.1836e-04_rb,8.8377e-04_rb/)
      kbo(:,40, 6) = (/ &
     &5.2096e-04_rb,5.7198e-04_rb,6.2426e-04_rb,6.7904e-04_rb,7.3548e-04_rb/)
      kbo(:,41, 6) = (/ &
     &4.2992e-04_rb,4.7365e-04_rb,5.1846e-04_rb,5.6371e-04_rb,6.1228e-04_rb/)
      kbo(:,42, 6) = (/ &
     &3.5500e-04_rb,3.9262e-04_rb,4.3159e-04_rb,4.7009e-04_rb,5.1120e-04_rb/)
      kbo(:,43, 6) = (/ &
     &2.9285e-04_rb,3.2562e-04_rb,3.5900e-04_rb,3.9210e-04_rb,4.2694e-04_rb/)
      kbo(:,44, 6) = (/ &
     &2.4061e-04_rb,2.6923e-04_rb,2.9805e-04_rb,3.2732e-04_rb,3.5760e-04_rb/)
      kbo(:,45, 6) = (/ &
     &1.9754e-04_rb,2.2270e-04_rb,2.4799e-04_rb,2.7345e-04_rb,3.0033e-04_rb/)
      kbo(:,46, 6) = (/ &
     &1.6118e-04_rb,1.8331e-04_rb,2.0573e-04_rb,2.2834e-04_rb,2.5255e-04_rb/)
      kbo(:,47, 6) = (/ &
     &1.3001e-04_rb,1.4919e-04_rb,1.6905e-04_rb,1.8953e-04_rb,2.1172e-04_rb/)
      kbo(:,48, 6) = (/ &
     &1.0430e-04_rb,1.2103e-04_rb,1.3853e-04_rb,1.5665e-04_rb,1.7689e-04_rb/)
      kbo(:,49, 6) = (/ &
     &8.3158e-05_rb,9.7679e-05_rb,1.1288e-04_rb,1.2898e-04_rb,1.4689e-04_rb/)
      kbo(:,50, 6) = (/ &
     &6.6182e-05_rb,7.8799e-05_rb,9.1873e-05_rb,1.0601e-04_rb,1.2184e-04_rb/)
      kbo(:,51, 6) = (/ &
     &5.2460e-05_rb,6.3349e-05_rb,7.4559e-05_rb,8.7033e-05_rb,1.0081e-04_rb/)
      kbo(:,52, 6) = (/ &
     &4.1391e-05_rb,5.0671e-05_rb,6.0237e-05_rb,7.1015e-05_rb,8.3290e-05_rb/)
      kbo(:,53, 6) = (/ &
     &3.2451e-05_rb,4.0299e-05_rb,4.8474e-05_rb,5.7732e-05_rb,6.8499e-05_rb/)
      kbo(:,54, 6) = (/ &
     &2.5414e-05_rb,3.2075e-05_rb,3.9061e-05_rb,4.7059e-05_rb,5.6458e-05_rb/)
      kbo(:,55, 6) = (/ &
     &1.9791e-05_rb,2.5430e-05_rb,3.1447e-05_rb,3.8320e-05_rb,4.6433e-05_rb/)
      kbo(:,56, 6) = (/ &
     &1.5282e-05_rb,2.0059e-05_rb,2.5203e-05_rb,3.1033e-05_rb,3.8063e-05_rb/)
      kbo(:,57, 6) = (/ &
     &1.1377e-05_rb,1.5690e-05_rb,2.0076e-05_rb,2.5035e-05_rb,3.1091e-05_rb/)
      kbo(:,58, 6) = (/ &
     &8.3098e-06_rb,1.2203e-05_rb,1.5952e-05_rb,2.0176e-05_rb,2.5366e-05_rb/)
      kbo(:,59, 6) = (/ &
     &6.5651e-06_rb,9.9766e-06_rb,1.3186e-05_rb,1.6915e-05_rb,2.1496e-05_rb/)
      kbo(:,13, 7) = (/ &
     &1.1534e-01_rb,1.2733e-01_rb,1.3962e-01_rb,1.4987e-01_rb,1.6008e-01_rb/)
      kbo(:,14, 7) = (/ &
     &9.8557e-02_rb,1.0921e-01_rb,1.1846e-01_rb,1.2756e-01_rb,1.3545e-01_rb/)
      kbo(:,15, 7) = (/ &
     &8.4816e-02_rb,9.3361e-02_rb,1.0130e-01_rb,1.0828e-01_rb,1.1452e-01_rb/)
      kbo(:,16, 7) = (/ &
     &7.2792e-02_rb,7.9992e-02_rb,8.6025e-02_rb,9.1745e-02_rb,9.7220e-02_rb/)
      kbo(:,17, 7) = (/ &
     &6.2250e-02_rb,6.8145e-02_rb,7.3215e-02_rb,7.8161e-02_rb,8.2912e-02_rb/)
      kbo(:,18, 7) = (/ &
     &5.2897e-02_rb,5.7617e-02_rb,6.2131e-02_rb,6.6296e-02_rb,7.0468e-02_rb/)
      kbo(:,19, 7) = (/ &
     &4.4630e-02_rb,4.8620e-02_rb,5.2395e-02_rb,5.5983e-02_rb,5.9561e-02_rb/)
      kbo(:,20, 7) = (/ &
     &3.7638e-02_rb,4.1018e-02_rb,4.4192e-02_rb,4.7348e-02_rb,5.0426e-02_rb/)
      kbo(:,21, 7) = (/ &
     &3.1712e-02_rb,3.4570e-02_rb,3.7337e-02_rb,4.0055e-02_rb,4.2699e-02_rb/)
      kbo(:,22, 7) = (/ &
     &2.6951e-02_rb,2.9361e-02_rb,3.1724e-02_rb,3.4058e-02_rb,3.6337e-02_rb/)
      kbo(:,23, 7) = (/ &
     &2.2985e-02_rb,2.5005e-02_rb,2.7002e-02_rb,2.8884e-02_rb,3.0844e-02_rb/)
      kbo(:,24, 7) = (/ &
     &1.9564e-02_rb,2.1275e-02_rb,2.2950e-02_rb,2.4579e-02_rb,2.6269e-02_rb/)
      kbo(:,25, 7) = (/ &
     &1.6678e-02_rb,1.8134e-02_rb,1.9557e-02_rb,2.0984e-02_rb,2.2448e-02_rb/)
      kbo(:,26, 7) = (/ &
     &1.4215e-02_rb,1.5478e-02_rb,1.6731e-02_rb,1.8053e-02_rb,1.9409e-02_rb/)
      kbo(:,27, 7) = (/ &
     &1.2146e-02_rb,1.3259e-02_rb,1.4401e-02_rb,1.5542e-02_rb,1.6732e-02_rb/)
      kbo(:,28, 7) = (/ &
     &1.0451e-02_rb,1.1387e-02_rb,1.2366e-02_rb,1.3378e-02_rb,1.4403e-02_rb/)
      kbo(:,29, 7) = (/ &
     &8.9638e-03_rb,9.7824e-03_rb,1.0625e-02_rb,1.1522e-02_rb,1.2402e-02_rb/)
      kbo(:,30, 7) = (/ &
     &7.7059e-03_rb,8.4110e-03_rb,9.1535e-03_rb,9.9173e-03_rb,1.0684e-02_rb/)
      kbo(:,31, 7) = (/ &
     &6.6159e-03_rb,7.2344e-03_rb,7.8777e-03_rb,8.5358e-03_rb,9.2030e-03_rb/)
      kbo(:,32, 7) = (/ &
     &5.6925e-03_rb,6.2370e-03_rb,6.7880e-03_rb,7.3601e-03_rb,7.9298e-03_rb/)
      kbo(:,33, 7) = (/ &
     &4.9047e-03_rb,5.3755e-03_rb,5.8574e-03_rb,6.3478e-03_rb,6.8444e-03_rb/)
      kbo(:,34, 7) = (/ &
     &4.2261e-03_rb,4.6310e-03_rb,5.0504e-03_rb,5.4765e-03_rb,5.9065e-03_rb/)
      kbo(:,35, 7) = (/ &
     &3.6014e-03_rb,3.9541e-03_rb,4.3177e-03_rb,4.6806e-03_rb,5.0649e-03_rb/)
      kbo(:,36, 7) = (/ &
     &3.0405e-03_rb,3.3438e-03_rb,3.6573e-03_rb,3.9714e-03_rb,4.3136e-03_rb/)
      kbo(:,37, 7) = (/ &
     &2.5476e-03_rb,2.8157e-03_rb,3.0872e-03_rb,3.3632e-03_rb,3.6681e-03_rb/)
      kbo(:,38, 7) = (/ &
     &2.1334e-03_rb,2.3678e-03_rb,2.6052e-03_rb,2.8532e-03_rb,3.1134e-03_rb/)
      kbo(:,39, 7) = (/ &
     &1.7845e-03_rb,1.9869e-03_rb,2.1996e-03_rb,2.4178e-03_rb,2.6484e-03_rb/)
      kbo(:,40, 7) = (/ &
     &1.4840e-03_rb,1.6609e-03_rb,1.8504e-03_rb,2.0474e-03_rb,2.2560e-03_rb/)
      kbo(:,41, 7) = (/ &
     &1.2344e-03_rb,1.3898e-03_rb,1.5559e-03_rb,1.7345e-03_rb,1.9234e-03_rb/)
      kbo(:,42, 7) = (/ &
     &1.0314e-03_rb,1.1653e-03_rb,1.3107e-03_rb,1.4669e-03_rb,1.6353e-03_rb/)
      kbo(:,43, 7) = (/ &
     &8.5631e-04_rb,9.7409e-04_rb,1.1003e-03_rb,1.2355e-03_rb,1.3860e-03_rb/)
      kbo(:,44, 7) = (/ &
     &7.0883e-04_rb,8.1303e-04_rb,9.2536e-04_rb,1.0435e-03_rb,1.1739e-03_rb/)
      kbo(:,45, 7) = (/ &
     &5.8732e-04_rb,6.7556e-04_rb,7.7384e-04_rb,8.8025e-04_rb,9.9590e-04_rb/)
      kbo(:,46, 7) = (/ &
     &4.8484e-04_rb,5.6171e-04_rb,6.4432e-04_rb,7.3847e-04_rb,8.4169e-04_rb/)
      kbo(:,47, 7) = (/ &
     &3.9752e-04_rb,4.6412e-04_rb,5.3807e-04_rb,6.1858e-04_rb,7.0821e-04_rb/)
      kbo(:,48, 7) = (/ &
     &3.2593e-04_rb,3.8380e-04_rb,4.4885e-04_rb,5.1997e-04_rb,5.9487e-04_rb/)
      kbo(:,49, 7) = (/ &
     &2.6581e-04_rb,3.1620e-04_rb,3.7509e-04_rb,4.3658e-04_rb,5.0219e-04_rb/)
      kbo(:,50, 7) = (/ &
     &2.1682e-04_rb,2.6203e-04_rb,3.1361e-04_rb,3.6786e-04_rb,4.2524e-04_rb/)
      kbo(:,51, 7) = (/ &
     &1.7696e-04_rb,2.1726e-04_rb,2.6292e-04_rb,3.1080e-04_rb,3.6141e-04_rb/)
      kbo(:,52, 7) = (/ &
     &1.4350e-04_rb,1.8000e-04_rb,2.2008e-04_rb,2.6275e-04_rb,3.0811e-04_rb/)
      kbo(:,53, 7) = (/ &
     &1.1566e-04_rb,1.4775e-04_rb,1.8326e-04_rb,2.2179e-04_rb,2.6354e-04_rb/)
      kbo(:,54, 7) = (/ &
     &9.3174e-05_rb,1.2130e-04_rb,1.5287e-04_rb,1.8794e-04_rb,2.2723e-04_rb/)
      kbo(:,55, 7) = (/ &
     &7.4812e-05_rb,9.9351e-05_rb,1.2721e-04_rb,1.5976e-04_rb,1.9539e-04_rb/)
      kbo(:,56, 7) = (/ &
     &5.9751e-05_rb,8.0907e-05_rb,1.0532e-04_rb,1.3503e-04_rb,1.6719e-04_rb/)
      kbo(:,57, 7) = (/ &
     &4.7338e-05_rb,6.5345e-05_rb,8.6815e-05_rb,1.1307e-04_rb,1.4234e-04_rb/)
      kbo(:,58, 7) = (/ &
     &3.7388e-05_rb,5.2674e-05_rb,7.1720e-05_rb,9.4757e-05_rb,1.2118e-04_rb/)
      kbo(:,59, 7) = (/ &
     &3.1368e-05_rb,4.4899e-05_rb,6.2069e-05_rb,8.2754e-05_rb,1.0682e-04_rb/)
      kbo(:,13, 8) = (/ &
     &3.8513e-01_rb,4.3049e-01_rb,4.7109e-01_rb,5.0795e-01_rb,5.3980e-01_rb/)
      kbo(:,14, 8) = (/ &
     &3.2769e-01_rb,3.6398e-01_rb,3.9786e-01_rb,4.2776e-01_rb,4.5444e-01_rb/)
      kbo(:,15, 8) = (/ &
     &2.7898e-01_rb,3.0981e-01_rb,3.3796e-01_rb,3.6334e-01_rb,3.8812e-01_rb/)
      kbo(:,16, 8) = (/ &
     &2.3683e-01_rb,2.6231e-01_rb,2.8577e-01_rb,3.0755e-01_rb,3.2800e-01_rb/)
      kbo(:,17, 8) = (/ &
     &2.0036e-01_rb,2.2125e-01_rb,2.4047e-01_rb,2.5907e-01_rb,2.7604e-01_rb/)
      kbo(:,18, 8) = (/ &
     &1.7015e-01_rb,1.8726e-01_rb,2.0338e-01_rb,2.1873e-01_rb,2.3341e-01_rb/)
      kbo(:,19, 8) = (/ &
     &1.4494e-01_rb,1.5899e-01_rb,1.7245e-01_rb,1.8542e-01_rb,1.9788e-01_rb/)
      kbo(:,20, 8) = (/ &
     &1.2385e-01_rb,1.3588e-01_rb,1.4742e-01_rb,1.5823e-01_rb,1.6882e-01_rb/)
      kbo(:,21, 8) = (/ &
     &1.0654e-01_rb,1.1655e-01_rb,1.2609e-01_rb,1.3545e-01_rb,1.4408e-01_rb/)
      kbo(:,22, 8) = (/ &
     &9.1991e-02_rb,1.0041e-01_rb,1.0849e-01_rb,1.1626e-01_rb,1.2371e-01_rb/)
      kbo(:,23, 8) = (/ &
     &7.9046e-02_rb,8.6400e-02_rb,9.3327e-02_rb,9.9889e-02_rb,1.0642e-01_rb/)
      kbo(:,24, 8) = (/ &
     &6.8021e-02_rb,7.4328e-02_rb,8.0300e-02_rb,8.6084e-02_rb,9.1544e-02_rb/)
      kbo(:,25, 8) = (/ &
     &5.8516e-02_rb,6.3854e-02_rb,6.9101e-02_rb,7.4075e-02_rb,7.8861e-02_rb/)
      kbo(:,26, 8) = (/ &
     &5.0443e-02_rb,5.5091e-02_rb,5.9525e-02_rb,6.3760e-02_rb,6.7833e-02_rb/)
      kbo(:,27, 8) = (/ &
     &4.3538e-02_rb,4.7477e-02_rb,5.1180e-02_rb,5.4850e-02_rb,5.8530e-02_rb/)
      kbo(:,28, 8) = (/ &
     &3.7617e-02_rb,4.0916e-02_rb,4.4139e-02_rb,4.7301e-02_rb,5.0513e-02_rb/)
      kbo(:,29, 8) = (/ &
     &3.2546e-02_rb,3.5342e-02_rb,3.8069e-02_rb,4.0845e-02_rb,4.3756e-02_rb/)
      kbo(:,30, 8) = (/ &
     &2.8067e-02_rb,3.0536e-02_rb,3.2922e-02_rb,3.5388e-02_rb,3.8072e-02_rb/)
      kbo(:,31, 8) = (/ &
     &2.4245e-02_rb,2.6387e-02_rb,2.8550e-02_rb,3.0769e-02_rb,3.3007e-02_rb/)
      kbo(:,32, 8) = (/ &
     &2.1010e-02_rb,2.2870e-02_rb,2.4839e-02_rb,2.6760e-02_rb,2.8725e-02_rb/)
      kbo(:,33, 8) = (/ &
     &1.8247e-02_rb,1.9921e-02_rb,2.1633e-02_rb,2.3376e-02_rb,2.5089e-02_rb/)
      kbo(:,34, 8) = (/ &
     &1.5899e-02_rb,1.7453e-02_rb,1.8953e-02_rb,2.0489e-02_rb,2.2006e-02_rb/)
      kbo(:,35, 8) = (/ &
     &1.3799e-02_rb,1.5171e-02_rb,1.6500e-02_rb,1.7816e-02_rb,1.9222e-02_rb/)
      kbo(:,36, 8) = (/ &
     &1.1856e-02_rb,1.3040e-02_rb,1.4223e-02_rb,1.5409e-02_rb,1.6675e-02_rb/)
      kbo(:,37, 8) = (/ &
     &1.0122e-02_rb,1.1172e-02_rb,1.2227e-02_rb,1.3320e-02_rb,1.4426e-02_rb/)
      kbo(:,38, 8) = (/ &
     &8.6297e-03_rb,9.5720e-03_rb,1.0548e-02_rb,1.1499e-02_rb,1.2504e-02_rb/)
      kbo(:,39, 8) = (/ &
     &7.3583e-03_rb,8.2086e-03_rb,9.0807e-03_rb,9.9445e-03_rb,1.0852e-02_rb/)
      kbo(:,40, 8) = (/ &
     &6.2575e-03_rb,7.0294e-03_rb,7.8131e-03_rb,8.5974e-03_rb,9.4311e-03_rb/)
      kbo(:,41, 8) = (/ &
     &5.3221e-03_rb,6.0117e-03_rb,6.7242e-03_rb,7.4367e-03_rb,8.2130e-03_rb/)
      kbo(:,42, 8) = (/ &
     &4.5217e-03_rb,5.1291e-03_rb,5.7774e-03_rb,6.4411e-03_rb,7.1607e-03_rb/)
      kbo(:,43, 8) = (/ &
     &3.8118e-03_rb,4.3626e-03_rb,4.9535e-03_rb,5.5702e-03_rb,6.2186e-03_rb/)
      kbo(:,44, 8) = (/ &
     &3.1928e-03_rb,3.7055e-03_rb,4.2384e-03_rb,4.7956e-03_rb,5.3949e-03_rb/)
      kbo(:,45, 8) = (/ &
     &2.6724e-03_rb,3.1345e-03_rb,3.6292e-03_rb,4.1408e-03_rb,4.6835e-03_rb/)
      kbo(:,46, 8) = (/ &
     &2.2327e-03_rb,2.6446e-03_rb,3.0956e-03_rb,3.5684e-03_rb,4.0653e-03_rb/)
      kbo(:,47, 8) = (/ &
     &1.8490e-03_rb,2.2161e-03_rb,2.6105e-03_rb,3.0391e-03_rb,3.5058e-03_rb/)
      kbo(:,48, 8) = (/ &
     &1.5271e-03_rb,1.8513e-03_rb,2.2029e-03_rb,2.5798e-03_rb,3.0175e-03_rb/)
      kbo(:,49, 8) = (/ &
     &1.2568e-03_rb,1.5444e-03_rb,1.8551e-03_rb,2.1969e-03_rb,2.5931e-03_rb/)
      kbo(:,50, 8) = (/ &
     &1.0411e-03_rb,1.2920e-03_rb,1.5680e-03_rb,1.8789e-03_rb,2.2434e-03_rb/)
      kbo(:,51, 8) = (/ &
     &8.6157e-04_rb,1.0792e-03_rb,1.3252e-03_rb,1.6044e-03_rb,1.9374e-03_rb/)
      kbo(:,52, 8) = (/ &
     &7.1345e-04_rb,8.9803e-04_rb,1.1146e-03_rb,1.3696e-03_rb,1.6714e-03_rb/)
      kbo(:,53, 8) = (/ &
     &5.9120e-04_rb,7.4871e-04_rb,9.4040e-04_rb,1.1662e-03_rb,1.4395e-03_rb/)
      kbo(:,54, 8) = (/ &
     &4.9248e-04_rb,6.3323e-04_rb,7.9771e-04_rb,9.9957e-04_rb,1.2405e-03_rb/)
      kbo(:,55, 8) = (/ &
     &4.1065e-04_rb,5.3488e-04_rb,6.8225e-04_rb,8.6108e-04_rb,1.0736e-03_rb/)
      kbo(:,56, 8) = (/ &
     &3.4182e-04_rb,4.5283e-04_rb,5.8716e-04_rb,7.4149e-04_rb,9.3761e-04_rb/)
      kbo(:,57, 8) = (/ &
     &2.8276e-04_rb,3.8428e-04_rb,5.0468e-04_rb,6.4575e-04_rb,8.2300e-04_rb/)
      kbo(:,58, 8) = (/ &
     &2.3443e-04_rb,3.2611e-04_rb,4.3528e-04_rb,5.6896e-04_rb,7.2980e-04_rb/)
      kbo(:,59, 8) = (/ &
     &2.0666e-04_rb,2.9266e-04_rb,3.9707e-04_rb,5.2680e-04_rb,6.7824e-04_rb/)
      kbo(:,13, 9) = (/ &
     &2.6929e+00_rb,3.0365e+00_rb,3.3480e+00_rb,3.6295e+00_rb,3.8842e+00_rb/)
      kbo(:,14, 9) = (/ &
     &2.3218e+00_rb,2.6065e+00_rb,2.8670e+00_rb,3.1040e+00_rb,3.3162e+00_rb/)
      kbo(:,15, 9) = (/ &
     &1.9883e+00_rb,2.2252e+00_rb,2.4424e+00_rb,2.6373e+00_rb,2.8081e+00_rb/)
      kbo(:,16, 9) = (/ &
     &1.6988e+00_rb,1.8964e+00_rb,2.0747e+00_rb,2.2343e+00_rb,2.3791e+00_rb/)
      kbo(:,17, 9) = (/ &
     &1.4495e+00_rb,1.6116e+00_rb,1.7595e+00_rb,1.8933e+00_rb,2.0138e+00_rb/)
      kbo(:,18, 9) = (/ &
     &1.2318e+00_rb,1.3662e+00_rb,1.4893e+00_rb,1.6003e+00_rb,1.6980e+00_rb/)
      kbo(:,19, 9) = (/ &
     &1.0432e+00_rb,1.1554e+00_rb,1.2571e+00_rb,1.3476e+00_rb,1.4284e+00_rb/)
      kbo(:,20, 9) = (/ &
     &8.8399e-01_rb,9.7584e-01_rb,1.0583e+00_rb,1.1327e+00_rb,1.2000e+00_rb/)
      kbo(:,21, 9) = (/ &
     &7.4719e-01_rb,8.2246e-01_rb,8.9025e-01_rb,9.5182e-01_rb,1.0085e+00_rb/)
      kbo(:,22, 9) = (/ &
     &6.3607e-01_rb,6.9712e-01_rb,7.5292e-01_rb,8.0394e-01_rb,8.5135e-01_rb/)
      kbo(:,23, 9) = (/ &
     &5.4262e-01_rb,5.9143e-01_rb,6.3738e-01_rb,6.8008e-01_rb,7.1995e-01_rb/)
      kbo(:,24, 9) = (/ &
     &4.6285e-01_rb,5.0331e-01_rb,5.4067e-01_rb,5.7573e-01_rb,6.1046e-01_rb/)
      kbo(:,25, 9) = (/ &
     &3.9562e-01_rb,4.2919e-01_rb,4.6040e-01_rb,4.9042e-01_rb,5.1991e-01_rb/)
      kbo(:,26, 9) = (/ &
     &3.4017e-01_rb,3.6788e-01_rb,3.9442e-01_rb,4.2037e-01_rb,4.4731e-01_rb/)
      kbo(:,27, 9) = (/ &
     &2.9399e-01_rb,3.1651e-01_rb,3.3903e-01_rb,3.6222e-01_rb,3.8618e-01_rb/)
      kbo(:,28, 9) = (/ &
     &2.5435e-01_rb,2.7396e-01_rb,2.9328e-01_rb,3.1366e-01_rb,3.3524e-01_rb/)
      kbo(:,29, 9) = (/ &
     &2.2098e-01_rb,2.3815e-01_rb,2.5600e-01_rb,2.7395e-01_rb,2.9332e-01_rb/)
      kbo(:,30, 9) = (/ &
     &1.9332e-01_rb,2.0838e-01_rb,2.2416e-01_rb,2.4057e-01_rb,2.5735e-01_rb/)
      kbo(:,31, 9) = (/ &
     &1.6991e-01_rb,1.8327e-01_rb,1.9737e-01_rb,2.1207e-01_rb,2.2734e-01_rb/)
      kbo(:,32, 9) = (/ &
     &1.4986e-01_rb,1.6194e-01_rb,1.7445e-01_rb,1.8782e-01_rb,2.0152e-01_rb/)
      kbo(:,33, 9) = (/ &
     &1.3279e-01_rb,1.4378e-01_rb,1.5504e-01_rb,1.6697e-01_rb,1.7955e-01_rb/)
      kbo(:,34, 9) = (/ &
     &1.1789e-01_rb,1.2771e-01_rb,1.3816e-01_rb,1.4885e-01_rb,1.6033e-01_rb/)
      kbo(:,35, 9) = (/ &
     &1.0423e-01_rb,1.1314e-01_rb,1.2268e-01_rb,1.3249e-01_rb,1.4303e-01_rb/)
      kbo(:,36, 9) = (/ &
     &9.1569e-02_rb,9.9843e-02_rb,1.0848e-01_rb,1.1752e-01_rb,1.2715e-01_rb/)
      kbo(:,37, 9) = (/ &
     &7.9654e-02_rb,8.7324e-02_rb,9.5396e-02_rb,1.0365e-01_rb,1.1217e-01_rb/)
      kbo(:,38, 9) = (/ &
     &6.9582e-02_rb,7.6500e-02_rb,8.3939e-02_rb,9.1706e-02_rb,9.9308e-02_rb/)
      kbo(:,39, 9) = (/ &
     &6.0817e-02_rb,6.7147e-02_rb,7.4089e-02_rb,8.1389e-02_rb,8.8469e-02_rb/)
      kbo(:,40, 9) = (/ &
     &5.2895e-02_rb,5.8816e-02_rb,6.5088e-02_rb,7.1837e-02_rb,7.8761e-02_rb/)
      kbo(:,41, 9) = (/ &
     &4.5994e-02_rb,5.1507e-02_rb,5.7306e-02_rb,6.3493e-02_rb,7.0017e-02_rb/)
      kbo(:,42, 9) = (/ &
     &3.9933e-02_rb,4.5126e-02_rb,5.0659e-02_rb,5.6440e-02_rb,6.2355e-02_rb/)
      kbo(:,43, 9) = (/ &
     &3.4407e-02_rb,3.9333e-02_rb,4.4505e-02_rb,5.0025e-02_rb,5.5477e-02_rb/)
      kbo(:,44, 9) = (/ &
     &2.9641e-02_rb,3.4179e-02_rb,3.9085e-02_rb,4.4243e-02_rb,4.9585e-02_rb/)
      kbo(:,45, 9) = (/ &
     &2.5534e-02_rb,2.9717e-02_rb,3.4321e-02_rb,3.9331e-02_rb,4.4385e-02_rb/)
      kbo(:,46, 9) = (/ &
     &2.1943e-02_rb,2.5772e-02_rb,3.0011e-02_rb,3.4665e-02_rb,3.9677e-02_rb/)
      kbo(:,47, 9) = (/ &
     &1.8799e-02_rb,2.2299e-02_rb,2.6155e-02_rb,3.0452e-02_rb,3.5108e-02_rb/)
      kbo(:,48, 9) = (/ &
     &1.6134e-02_rb,1.9292e-02_rb,2.2898e-02_rb,2.6798e-02_rb,3.1152e-02_rb/)
      kbo(:,49, 9) = (/ &
     &1.3799e-02_rb,1.6693e-02_rb,1.9932e-02_rb,2.3629e-02_rb,2.7686e-02_rb/)
      kbo(:,50, 9) = (/ &
     &1.1807e-02_rb,1.4439e-02_rb,1.7455e-02_rb,2.0914e-02_rb,2.4717e-02_rb/)
      kbo(:,51, 9) = (/ &
     &1.0105e-02_rb,1.2520e-02_rb,1.5304e-02_rb,1.8588e-02_rb,2.2129e-02_rb/)
      kbo(:,52, 9) = (/ &
     &8.6629e-03_rb,1.0849e-02_rb,1.3445e-02_rb,1.6444e-02_rb,1.9913e-02_rb/)
      kbo(:,53, 9) = (/ &
     &7.4874e-03_rb,9.4257e-03_rb,1.1801e-02_rb,1.4610e-02_rb,1.7870e-02_rb/)
      kbo(:,54, 9) = (/ &
     &6.4970e-03_rb,8.2758e-03_rb,1.0456e-02_rb,1.3093e-02_rb,1.6122e-02_rb/)
      kbo(:,55, 9) = (/ &
     &5.6140e-03_rb,7.3217e-03_rb,9.3276e-03_rb,1.1747e-02_rb,1.4696e-02_rb/)
      kbo(:,56, 9) = (/ &
     &4.8349e-03_rb,6.4792e-03_rb,8.3317e-03_rb,1.0602e-02_rb,1.3343e-02_rb/)
      kbo(:,57, 9) = (/ &
     &4.1640e-03_rb,5.6963e-03_rb,7.4736e-03_rb,9.6066e-03_rb,1.2192e-02_rb/)
      kbo(:,58, 9) = (/ &
     &3.5901e-03_rb,5.0045e-03_rb,6.7274e-03_rb,8.7656e-03_rb,1.1231e-02_rb/)
      kbo(:,59, 9) = (/ &
     &3.3029e-03_rb,4.6798e-03_rb,6.3469e-03_rb,8.3368e-03_rb,1.0817e-02_rb/)
      kbo(:,13,10) = (/ &
     &1.1103e+01_rb,1.2502e+01_rb,1.3897e+01_rb,1.5277e+01_rb,1.6528e+01_rb/)
      kbo(:,14,10) = (/ &
     &9.9415e+00_rb,1.1260e+01_rb,1.2546e+01_rb,1.3697e+01_rb,1.4756e+01_rb/)
      kbo(:,15,10) = (/ &
     &8.8808e+00_rb,1.0035e+01_rb,1.1086e+01_rb,1.2088e+01_rb,1.3034e+01_rb/)
      kbo(:,16,10) = (/ &
     &7.8211e+00_rb,8.7752e+00_rb,9.7015e+00_rb,1.0578e+01_rb,1.1354e+01_rb/)
      kbo(:,17,10) = (/ &
     &6.7833e+00_rb,7.6204e+00_rb,8.4176e+00_rb,9.1234e+00_rb,9.7674e+00_rb/)
      kbo(:,18,10) = (/ &
     &5.8703e+00_rb,6.5862e+00_rb,7.2317e+00_rb,7.8324e+00_rb,8.4128e+00_rb/)
      kbo(:,19,10) = (/ &
     &5.0772e+00_rb,5.6603e+00_rb,6.2160e+00_rb,6.7517e+00_rb,7.2522e+00_rb/)
      kbo(:,20,10) = (/ &
     &4.3641e+00_rb,4.8646e+00_rb,5.3518e+00_rb,5.8113e+00_rb,6.2183e+00_rb/)
      kbo(:,21,10) = (/ &
     &3.7436e+00_rb,4.1787e+00_rb,4.5968e+00_rb,4.9653e+00_rb,5.2958e+00_rb/)
      kbo(:,22,10) = (/ &
     &3.2384e+00_rb,3.6097e+00_rb,3.9430e+00_rb,4.2432e+00_rb,4.5244e+00_rb/)
      kbo(:,23,10) = (/ &
     &2.7953e+00_rb,3.1049e+00_rb,3.3758e+00_rb,3.6302e+00_rb,3.8688e+00_rb/)
      kbo(:,24,10) = (/ &
     &2.4111e+00_rb,2.6600e+00_rb,2.8956e+00_rb,3.1175e+00_rb,3.3165e+00_rb/)
      kbo(:,25,10) = (/ &
     &2.0794e+00_rb,2.2877e+00_rb,2.4858e+00_rb,2.6707e+00_rb,2.8419e+00_rb/)
      kbo(:,26,10) = (/ &
     &1.7863e+00_rb,1.9622e+00_rb,2.1280e+00_rb,2.2830e+00_rb,2.4241e+00_rb/)
      kbo(:,27,10) = (/ &
     &1.5261e+00_rb,1.6805e+00_rb,1.8222e+00_rb,1.9507e+00_rb,2.0763e+00_rb/)
      kbo(:,28,10) = (/ &
     &1.3094e+00_rb,1.4366e+00_rb,1.5593e+00_rb,1.6738e+00_rb,1.7905e+00_rb/)
      kbo(:,29,10) = (/ &
     &1.1331e+00_rb,1.2336e+00_rb,1.3334e+00_rb,1.4402e+00_rb,1.5544e+00_rb/)
      kbo(:,30,10) = (/ &
     &9.7850e-01_rb,1.0622e+00_rb,1.1514e+00_rb,1.2513e+00_rb,1.3634e+00_rb/)
      kbo(:,31,10) = (/ &
     &8.4902e-01_rb,9.2389e-01_rb,1.0050e+00_rb,1.1001e+00_rb,1.2086e+00_rb/)
      kbo(:,32,10) = (/ &
     &7.4130e-01_rb,8.0932e-01_rb,8.8870e-01_rb,9.7905e-01_rb,1.0856e+00_rb/)
      kbo(:,33,10) = (/ &
     &6.5296e-01_rb,7.1655e-01_rb,7.9236e-01_rb,8.8248e-01_rb,9.8250e-01_rb/)
      kbo(:,34,10) = (/ &
     &5.7962e-01_rb,6.3885e-01_rb,7.0985e-01_rb,7.9650e-01_rb,8.9263e-01_rb/)
      kbo(:,35,10) = (/ &
     &5.1372e-01_rb,5.6904e-01_rb,6.3621e-01_rb,7.1633e-01_rb,8.0771e-01_rb/)
      kbo(:,36,10) = (/ &
     &4.5469e-01_rb,5.0394e-01_rb,5.6668e-01_rb,6.4061e-01_rb,7.2625e-01_rb/)
      kbo(:,37,10) = (/ &
     &3.9957e-01_rb,4.4312e-01_rb,4.9317e-01_rb,5.6045e-01_rb,6.4131e-01_rb/)
      kbo(:,38,10) = (/ &
     &3.5188e-01_rb,3.9249e-01_rb,4.3332e-01_rb,4.9135e-01_rb,5.6831e-01_rb/)
      kbo(:,39,10) = (/ &
     &3.1275e-01_rb,3.5014e-01_rb,3.8817e-01_rb,4.3541e-01_rb,5.0377e-01_rb/)
      kbo(:,40,10) = (/ &
     &2.7524e-01_rb,3.0717e-01_rb,3.4405e-01_rb,3.8385e-01_rb,4.3866e-01_rb/)
      kbo(:,41,10) = (/ &
     &2.4392e-01_rb,2.7163e-01_rb,3.0427e-01_rb,3.4320e-01_rb,3.8763e-01_rb/)
      kbo(:,42,10) = (/ &
     &2.1672e-01_rb,2.4359e-01_rb,2.7030e-01_rb,3.0489e-01_rb,3.4816e-01_rb/)
      kbo(:,43,10) = (/ &
     &1.9366e-01_rb,2.1755e-01_rb,2.4331e-01_rb,2.6943e-01_rb,3.1064e-01_rb/)
      kbo(:,44,10) = (/ &
     &1.7412e-01_rb,1.9505e-01_rb,2.1790e-01_rb,2.4485e-01_rb,2.7503e-01_rb/)
      kbo(:,45,10) = (/ &
     &1.5640e-01_rb,1.7593e-01_rb,1.9695e-01_rb,2.2100e-01_rb,2.4918e-01_rb/)
      kbo(:,46,10) = (/ &
     &1.3989e-01_rb,1.5897e-01_rb,1.7897e-01_rb,2.0081e-01_rb,2.2644e-01_rb/)
      kbo(:,47,10) = (/ &
     &1.2271e-01_rb,1.4204e-01_rb,1.6295e-01_rb,1.8322e-01_rb,2.0661e-01_rb/)
      kbo(:,48,10) = (/ &
     &1.0686e-01_rb,1.2583e-01_rb,1.4636e-01_rb,1.6777e-01_rb,1.9011e-01_rb/)
      kbo(:,49,10) = (/ &
     &9.3685e-02_rb,1.1078e-01_rb,1.3130e-01_rb,1.5261e-01_rb,1.7455e-01_rb/)
      kbo(:,50,10) = (/ &
     &8.2656e-02_rb,9.8591e-02_rb,1.1744e-01_rb,1.3916e-01_rb,1.6129e-01_rb/)
      kbo(:,51,10) = (/ &
     &7.2693e-02_rb,8.8183e-02_rb,1.0575e-01_rb,1.2537e-01_rb,1.4856e-01_rb/)
      kbo(:,52,10) = (/ &
     &6.4048e-02_rb,7.9108e-02_rb,9.5503e-02_rb,1.1470e-01_rb,1.3627e-01_rb/)
      kbo(:,53,10) = (/ &
     &5.5496e-02_rb,7.0682e-02_rb,8.5938e-02_rb,1.0406e-01_rb,1.2637e-01_rb/)
      kbo(:,54,10) = (/ &
     &4.9035e-02_rb,6.2876e-02_rb,7.7887e-02_rb,9.5399e-02_rb,1.1721e-01_rb/)
      kbo(:,55,10) = (/ &
     &4.3446e-02_rb,5.6855e-02_rb,7.1213e-02_rb,8.8342e-02_rb,1.0826e-01_rb/)
      kbo(:,56,10) = (/ &
     &3.8859e-02_rb,5.1445e-02_rb,6.5981e-02_rb,8.2010e-02_rb,1.0158e-01_rb/)
      kbo(:,57,10) = (/ &
     &3.5022e-02_rb,4.6988e-02_rb,6.1022e-02_rb,7.6424e-02_rb,9.5735e-02_rb/)
      kbo(:,58,10) = (/ &
     &3.1909e-02_rb,4.3092e-02_rb,5.6695e-02_rb,7.2335e-02_rb,9.0564e-02_rb/)
      kbo(:,59,10) = (/ &
     &3.0556e-02_rb,4.1959e-02_rb,5.5206e-02_rb,7.1568e-02_rb,9.0259e-02_rb/)
      kbo(:,13,11) = (/ &
     &1.9005e+01_rb,2.1823e+01_rb,2.4335e+01_rb,2.6560e+01_rb,2.8649e+01_rb/)
      kbo(:,14,11) = (/ &
     &1.8043e+01_rb,2.0403e+01_rb,2.2563e+01_rb,2.4639e+01_rb,2.6669e+01_rb/)
      kbo(:,15,11) = (/ &
     &1.6503e+01_rb,1.8586e+01_rb,2.0593e+01_rb,2.2552e+01_rb,2.4342e+01_rb/)
      kbo(:,16,11) = (/ &
     &1.4859e+01_rb,1.6766e+01_rb,1.8612e+01_rb,2.0317e+01_rb,2.1884e+01_rb/)
      kbo(:,17,11) = (/ &
     &1.3269e+01_rb,1.4973e+01_rb,1.6543e+01_rb,1.8024e+01_rb,1.9429e+01_rb/)
      kbo(:,18,11) = (/ &
     &1.1718e+01_rb,1.3157e+01_rb,1.4545e+01_rb,1.5849e+01_rb,1.7039e+01_rb/)
      kbo(:,19,11) = (/ &
     &1.0207e+01_rb,1.1484e+01_rb,1.2680e+01_rb,1.3769e+01_rb,1.4775e+01_rb/)
      kbo(:,20,11) = (/ &
     &8.8979e+00_rb,9.9836e+00_rb,1.0976e+01_rb,1.1897e+01_rb,1.2806e+01_rb/)
      kbo(:,21,11) = (/ &
     &7.7345e+00_rb,8.6287e+00_rb,9.4674e+00_rb,1.0308e+01_rb,1.1126e+01_rb/)
      kbo(:,22,11) = (/ &
     &6.7360e+00_rb,7.4922e+00_rb,8.2529e+00_rb,8.9982e+00_rb,9.7172e+00_rb/)
      kbo(:,23,11) = (/ &
     &5.8386e+00_rb,6.5155e+00_rb,7.1951e+00_rb,7.8490e+00_rb,8.4500e+00_rb/)
      kbo(:,24,11) = (/ &
     &5.0807e+00_rb,5.6843e+00_rb,6.2699e+00_rb,6.8121e+00_rb,7.3100e+00_rb/)
      kbo(:,25,11) = (/ &
     &4.4422e+00_rb,4.9643e+00_rb,5.4499e+00_rb,5.8998e+00_rb,6.3372e+00_rb/)
      kbo(:,26,11) = (/ &
     &3.9004e+00_rb,4.3334e+00_rb,4.7344e+00_rb,5.1243e+00_rb,5.5060e+00_rb/)
      kbo(:,27,11) = (/ &
     &3.4204e+00_rb,3.7816e+00_rb,4.1294e+00_rb,4.4714e+00_rb,4.7992e+00_rb/)
      kbo(:,28,11) = (/ &
     &2.9981e+00_rb,3.3076e+00_rb,3.6130e+00_rb,3.9107e+00_rb,4.1979e+00_rb/)
      kbo(:,29,11) = (/ &
     &2.6272e+00_rb,2.9045e+00_rb,3.1705e+00_rb,3.4300e+00_rb,3.6761e+00_rb/)
      kbo(:,30,11) = (/ &
     &2.3072e+00_rb,2.5571e+00_rb,2.7905e+00_rb,3.0127e+00_rb,3.2425e+00_rb/)
      kbo(:,31,11) = (/ &
     &2.0315e+00_rb,2.2496e+00_rb,2.4602e+00_rb,2.6655e+00_rb,2.8882e+00_rb/)
      kbo(:,32,11) = (/ &
     &1.7947e+00_rb,1.9859e+00_rb,2.1751e+00_rb,2.3754e+00_rb,2.5995e+00_rb/)
      kbo(:,33,11) = (/ &
     &1.5917e+00_rb,1.7608e+00_rb,1.9393e+00_rb,2.1339e+00_rb,2.3696e+00_rb/)
      kbo(:,34,11) = (/ &
     &1.4106e+00_rb,1.5681e+00_rb,1.7384e+00_rb,1.9365e+00_rb,2.1773e+00_rb/)
      kbo(:,35,11) = (/ &
     &1.2470e+00_rb,1.3941e+00_rb,1.5593e+00_rb,1.7605e+00_rb,1.9959e+00_rb/)
      kbo(:,36,11) = (/ &
     &1.0975e+00_rb,1.2350e+00_rb,1.3953e+00_rb,1.5917e+00_rb,1.8213e+00_rb/)
      kbo(:,37,11) = (/ &
     &9.5256e-01_rb,1.0776e+00_rb,1.2351e+00_rb,1.4193e+00_rb,1.6407e+00_rb/)
      kbo(:,38,11) = (/ &
     &8.3116e-01_rb,9.4539e-01_rb,1.0963e+00_rb,1.2721e+00_rb,1.4845e+00_rb/)
      kbo(:,39,11) = (/ &
     &7.2978e-01_rb,8.3514e-01_rb,9.7338e-01_rb,1.1448e+00_rb,1.3526e+00_rb/)
      kbo(:,40,11) = (/ &
     &6.3508e-01_rb,7.2958e-01_rb,8.5453e-01_rb,1.0153e+00_rb,1.2157e+00_rb/)
      kbo(:,41,11) = (/ &
     &5.5559e-01_rb,6.3933e-01_rb,7.5266e-01_rb,8.9822e-01_rb,1.0902e+00_rb/)
      kbo(:,42,11) = (/ &
     &4.9132e-01_rb,5.6247e-01_rb,6.6545e-01_rb,8.0063e-01_rb,9.7700e-01_rb/)
      kbo(:,43,11) = (/ &
     &4.3434e-01_rb,4.9358e-01_rb,5.8232e-01_rb,7.0818e-01_rb,8.6923e-01_rb/)
      kbo(:,44,11) = (/ &
     &3.8638e-01_rb,4.3517e-01_rb,5.1097e-01_rb,6.1891e-01_rb,7.7146e-01_rb/)
      kbo(:,45,11) = (/ &
     &3.4515e-01_rb,3.9086e-01_rb,4.5155e-01_rb,5.4453e-01_rb,6.8039e-01_rb/)
      kbo(:,46,11) = (/ &
     &3.1047e-01_rb,3.4972e-01_rb,4.0476e-01_rb,4.8078e-01_rb,5.9879e-01_rb/)
      kbo(:,47,11) = (/ &
     &2.8096e-01_rb,3.1543e-01_rb,3.6034e-01_rb,4.2515e-01_rb,5.2456e-01_rb/)
      kbo(:,48,11) = (/ &
     &2.5443e-01_rb,2.8861e-01_rb,3.2488e-01_rb,3.8074e-01_rb,4.6176e-01_rb/)
      kbo(:,49,11) = (/ &
     &2.3075e-01_rb,2.6370e-01_rb,2.9745e-01_rb,3.4508e-01_rb,4.1364e-01_rb/)
      kbo(:,50,11) = (/ &
     &2.0955e-01_rb,2.4450e-01_rb,2.7626e-01_rb,3.1515e-01_rb,3.7610e-01_rb/)
      kbo(:,51,11) = (/ &
     &1.9162e-01_rb,2.2508e-01_rb,2.5888e-01_rb,2.9434e-01_rb,3.4519e-01_rb/)
      kbo(:,52,11) = (/ &
     &1.7326e-01_rb,2.0725e-01_rb,2.4087e-01_rb,2.7521e-01_rb,3.1976e-01_rb/)
      kbo(:,53,11) = (/ &
     &1.5761e-01_rb,1.9120e-01_rb,2.2695e-01_rb,2.6019e-01_rb,2.9822e-01_rb/)
      kbo(:,54,11) = (/ &
     &1.4365e-01_rb,1.7736e-01_rb,2.1379e-01_rb,2.4744e-01_rb,2.8175e-01_rb/)
      kbo(:,55,11) = (/ &
     &1.3037e-01_rb,1.6532e-01_rb,2.0060e-01_rb,2.3640e-01_rb,2.6972e-01_rb/)
      kbo(:,56,11) = (/ &
     &1.1768e-01_rb,1.5347e-01_rb,1.8925e-01_rb,2.2636e-01_rb,2.6180e-01_rb/)
      kbo(:,57,11) = (/ &
     &1.0584e-01_rb,1.4146e-01_rb,1.7843e-01_rb,2.1611e-01_rb,2.5112e-01_rb/)
      kbo(:,58,11) = (/ &
     &9.5443e-02_rb,1.3188e-01_rb,1.6746e-01_rb,2.0707e-01_rb,2.4328e-01_rb/)
      kbo(:,59,11) = (/ &
     &9.0897e-02_rb,1.2729e-01_rb,1.6511e-01_rb,2.0322e-01_rb,2.4119e-01_rb/)
      kbo(:,13,12) = (/ &
     &3.3504e+01_rb,3.7950e+01_rb,4.2337e+01_rb,4.6726e+01_rb,5.1022e+01_rb/)
      kbo(:,14,12) = (/ &
     &3.2726e+01_rb,3.7291e+01_rb,4.1814e+01_rb,4.6223e+01_rb,5.0163e+01_rb/)
      kbo(:,15,12) = (/ &
     &3.1900e+01_rb,3.6439e+01_rb,4.0782e+01_rb,4.4671e+01_rb,4.8184e+01_rb/)
      kbo(:,16,12) = (/ &
     &3.0776e+01_rb,3.4900e+01_rb,3.8620e+01_rb,4.2065e+01_rb,4.5353e+01_rb/)
      kbo(:,17,12) = (/ &
     &2.8746e+01_rb,3.2300e+01_rb,3.5697e+01_rb,3.8925e+01_rb,4.2020e+01_rb/)
      kbo(:,18,12) = (/ &
     &2.6146e+01_rb,2.9399e+01_rb,3.2494e+01_rb,3.5494e+01_rb,3.8316e+01_rb/)
      kbo(:,19,12) = (/ &
     &2.3524e+01_rb,2.6429e+01_rb,2.9272e+01_rb,3.2000e+01_rb,3.4621e+01_rb/)
      kbo(:,20,12) = (/ &
     &2.1002e+01_rb,2.3641e+01_rb,2.6168e+01_rb,2.8626e+01_rb,3.0966e+01_rb/)
      kbo(:,21,12) = (/ &
     &1.8638e+01_rb,2.0929e+01_rb,2.3211e+01_rb,2.5404e+01_rb,2.7506e+01_rb/)
      kbo(:,22,12) = (/ &
     &1.6511e+01_rb,1.8592e+01_rb,2.0614e+01_rb,2.2575e+01_rb,2.4466e+01_rb/)
      kbo(:,23,12) = (/ &
     &1.4666e+01_rb,1.6512e+01_rb,1.8295e+01_rb,2.0007e+01_rb,2.1707e+01_rb/)
      kbo(:,24,12) = (/ &
     &1.3045e+01_rb,1.4646e+01_rb,1.6188e+01_rb,1.7728e+01_rb,1.9330e+01_rb/)
      kbo(:,25,12) = (/ &
     &1.1583e+01_rb,1.2959e+01_rb,1.4347e+01_rb,1.5799e+01_rb,1.7308e+01_rb/)
      kbo(:,26,12) = (/ &
     &1.0289e+01_rb,1.1524e+01_rb,1.2828e+01_rb,1.4195e+01_rb,1.5608e+01_rb/)
      kbo(:,27,12) = (/ &
     &9.1629e+00_rb,1.0320e+01_rb,1.1545e+01_rb,1.2818e+01_rb,1.4160e+01_rb/)
      kbo(:,28,12) = (/ &
     &8.1951e+00_rb,9.2900e+00_rb,1.0435e+01_rb,1.1638e+01_rb,1.2904e+01_rb/)
      kbo(:,29,12) = (/ &
     &7.3862e+00_rb,8.4074e+00_rb,9.4905e+00_rb,1.0629e+01_rb,1.1848e+01_rb/)
      kbo(:,30,12) = (/ &
     &6.7112e+00_rb,7.6605e+00_rb,8.6762e+00_rb,9.7816e+00_rb,1.0954e+01_rb/)
      kbo(:,31,12) = (/ &
     &6.1419e+00_rb,7.0283e+00_rb,8.0036e+00_rb,9.0662e+00_rb,1.0224e+01_rb/)
      kbo(:,32,12) = (/ &
     &5.6561e+00_rb,6.5080e+00_rb,7.4459e+00_rb,8.4904e+00_rb,9.6547e+00_rb/)
      kbo(:,33,12) = (/ &
     &5.2496e+00_rb,6.0774e+00_rb,7.0019e+00_rb,8.0523e+00_rb,9.2390e+00_rb/)
      kbo(:,34,12) = (/ &
     &4.8878e+00_rb,5.7010e+00_rb,6.6301e+00_rb,7.6985e+00_rb,8.9231e+00_rb/)
      kbo(:,35,12) = (/ &
     &4.5253e+00_rb,5.3274e+00_rb,6.2605e+00_rb,7.3465e+00_rb,8.6116e+00_rb/)
      kbo(:,36,12) = (/ &
     &4.1488e+00_rb,4.9390e+00_rb,5.8698e+00_rb,6.9625e+00_rb,8.2516e+00_rb/)
      kbo(:,37,12) = (/ &
     &3.7312e+00_rb,4.4908e+00_rb,5.4013e+00_rb,6.4804e+00_rb,7.7554e+00_rb/)
      kbo(:,38,12) = (/ &
     &3.3680e+00_rb,4.1012e+00_rb,4.9908e+00_rb,6.0547e+00_rb,7.3121e+00_rb/)
      kbo(:,39,12) = (/ &
     &3.0586e+00_rb,3.7669e+00_rb,4.6363e+00_rb,5.6845e+00_rb,6.9229e+00_rb/)
      kbo(:,40,12) = (/ &
     &2.7296e+00_rb,3.4041e+00_rb,4.2320e+00_rb,5.2462e+00_rb,6.4437e+00_rb/)
      kbo(:,41,12) = (/ &
     &2.4362e+00_rb,3.0740e+00_rb,3.8624e+00_rb,4.8348e+00_rb,5.9904e+00_rb/)
      kbo(:,42,12) = (/ &
     &2.1809e+00_rb,2.7810e+00_rb,3.5324e+00_rb,4.4598e+00_rb,5.5747e+00_rb/)
      kbo(:,43,12) = (/ &
     &1.9255e+00_rb,2.4860e+00_rb,3.1917e+00_rb,4.0645e+00_rb,5.1258e+00_rb/)
      kbo(:,44,12) = (/ &
     &1.6844e+00_rb,2.2090e+00_rb,2.8652e+00_rb,3.6821e+00_rb,4.6820e+00_rb/)
      kbo(:,45,12) = (/ &
     &1.4745e+00_rb,1.9569e+00_rb,2.5696e+00_rb,3.3333e+00_rb,4.2717e+00_rb/)
      kbo(:,46,12) = (/ &
     &1.2799e+00_rb,1.7238e+00_rb,2.2842e+00_rb,2.9993e+00_rb,3.8727e+00_rb/)
      kbo(:,47,12) = (/ &
     &1.0954e+00_rb,1.4926e+00_rb,2.0054e+00_rb,2.6579e+00_rb,3.4668e+00_rb/)
      kbo(:,48,12) = (/ &
     &9.3943e-01_rb,1.2870e+00_rb,1.7558e+00_rb,2.3470e+00_rb,3.0956e+00_rb/)
      kbo(:,49,12) = (/ &
     &8.0789e-01_rb,1.1100e+00_rb,1.5297e+00_rb,2.0665e+00_rb,2.7506e+00_rb/)
      kbo(:,50,12) = (/ &
     &7.0633e-01_rb,9.6173e-01_rb,1.3381e+00_rb,1.8318e+00_rb,2.4528e+00_rb/)
      kbo(:,51,12) = (/ &
     &6.2380e-01_rb,8.4012e-01_rb,1.1696e+00_rb,1.6208e+00_rb,2.1914e+00_rb/)
      kbo(:,52,12) = (/ &
     &5.6356e-01_rb,7.3733e-01_rb,1.0234e+00_rb,1.4290e+00_rb,1.9532e+00_rb/)
      kbo(:,53,12) = (/ &
     &5.0909e-01_rb,6.5068e-01_rb,8.9219e-01_rb,1.2539e+00_rb,1.7345e+00_rb/)
      kbo(:,54,12) = (/ &
     &4.6136e-01_rb,5.8883e-01_rb,7.8810e-01_rb,1.1078e+00_rb,1.5488e+00_rb/)
      kbo(:,55,12) = (/ &
     &4.1950e-01_rb,5.3756e-01_rb,7.0213e-01_rb,9.8100e-01_rb,1.3825e+00_rb/)
      kbo(:,56,12) = (/ &
     &3.8410e-01_rb,4.8996e-01_rb,6.3016e-01_rb,8.6652e-01_rb,1.2252e+00_rb/)
      kbo(:,57,12) = (/ &
     &3.5624e-01_rb,4.4831e-01_rb,5.7215e-01_rb,7.6572e-01_rb,1.0848e+00_rb/)
      kbo(:,58,12) = (/ &
     &3.2868e-01_rb,4.1829e-01_rb,5.2553e-01_rb,6.8368e-01_rb,9.6165e-01_rb/)
      kbo(:,59,12) = (/ &
     &3.1733e-01_rb,4.0674e-01_rb,5.0354e-01_rb,6.5575e-01_rb,9.0743e-01_rb/)
      kbo(:,13,13) = (/ &
     &6.3252e+01_rb,7.1343e+01_rb,7.9418e+01_rb,8.6956e+01_rb,9.3893e+01_rb/)
      kbo(:,14,13) = (/ &
     &6.4432e+01_rb,7.3277e+01_rb,8.1636e+01_rb,8.9410e+01_rb,9.6695e+01_rb/)
      kbo(:,15,13) = (/ &
     &6.5832e+01_rb,7.4648e+01_rb,8.2893e+01_rb,9.0741e+01_rb,9.8138e+01_rb/)
      kbo(:,16,13) = (/ &
     &6.5929e+01_rb,7.4500e+01_rb,8.2870e+01_rb,9.0803e+01_rb,9.8296e+01_rb/)
      kbo(:,17,13) = (/ &
     &6.5097e+01_rb,7.3774e+01_rb,8.2069e+01_rb,9.0039e+01_rb,9.7697e+01_rb/)
      kbo(:,18,13) = (/ &
     &6.3876e+01_rb,7.2341e+01_rb,8.0612e+01_rb,8.8540e+01_rb,9.6103e+01_rb/)
      kbo(:,19,13) = (/ &
     &6.1808e+01_rb,7.0129e+01_rb,7.8103e+01_rb,8.5721e+01_rb,9.2920e+01_rb/)
      kbo(:,20,13) = (/ &
     &5.9337e+01_rb,6.7178e+01_rb,7.4740e+01_rb,8.1921e+01_rb,8.8860e+01_rb/)
      kbo(:,21,13) = (/ &
     &5.6051e+01_rb,6.3423e+01_rb,7.0500e+01_rb,7.7416e+01_rb,8.4103e+01_rb/)
      kbo(:,22,13) = (/ &
     &5.2578e+01_rb,5.9410e+01_rb,6.6220e+01_rb,7.2967e+01_rb,7.9639e+01_rb/)
      kbo(:,23,13) = (/ &
     &4.8800e+01_rb,5.5307e+01_rb,6.1937e+01_rb,6.8633e+01_rb,7.5406e+01_rb/)
      kbo(:,24,13) = (/ &
     &4.5172e+01_rb,5.1522e+01_rb,5.8057e+01_rb,6.4742e+01_rb,7.1436e+01_rb/)
      kbo(:,25,13) = (/ &
     &4.2002e+01_rb,4.8199e+01_rb,5.4646e+01_rb,6.1219e+01_rb,6.7880e+01_rb/)
      kbo(:,26,13) = (/ &
     &3.9325e+01_rb,4.5395e+01_rb,5.1713e+01_rb,5.8235e+01_rb,6.4961e+01_rb/)
      kbo(:,27,13) = (/ &
     &3.7049e+01_rb,4.2989e+01_rb,4.9236e+01_rb,5.5807e+01_rb,6.2667e+01_rb/)
      kbo(:,28,13) = (/ &
     &3.5161e+01_rb,4.0996e+01_rb,4.7272e+01_rb,5.3955e+01_rb,6.1012e+01_rb/)
      kbo(:,29,13) = (/ &
     &3.3638e+01_rb,3.9504e+01_rb,4.5884e+01_rb,5.2759e+01_rb,6.0034e+01_rb/)
      kbo(:,30,13) = (/ &
     &3.2492e+01_rb,3.8479e+01_rb,4.5047e+01_rb,5.2111e+01_rb,5.9617e+01_rb/)
      kbo(:,31,13) = (/ &
     &3.1781e+01_rb,3.7957e+01_rb,4.4719e+01_rb,5.2013e+01_rb,5.9720e+01_rb/)
      kbo(:,32,13) = (/ &
     &3.1516e+01_rb,3.7888e+01_rb,4.4882e+01_rb,5.2382e+01_rb,6.0245e+01_rb/)
      kbo(:,33,13) = (/ &
     &3.1641e+01_rb,3.8231e+01_rb,4.5439e+01_rb,5.3118e+01_rb,6.1113e+01_rb/)
      kbo(:,34,13) = (/ &
     &3.1881e+01_rb,3.8668e+01_rb,4.6052e+01_rb,5.3873e+01_rb,6.1967e+01_rb/)
      kbo(:,35,13) = (/ &
     &3.1765e+01_rb,3.8677e+01_rb,4.6167e+01_rb,5.4068e+01_rb,6.2237e+01_rb/)
      kbo(:,36,13) = (/ &
     &3.1127e+01_rb,3.8083e+01_rb,4.5619e+01_rb,5.3554e+01_rb,6.1754e+01_rb/)
      kbo(:,37,13) = (/ &
     &2.9631e+01_rb,3.6533e+01_rb,4.4019e+01_rb,5.1934e+01_rb,6.0138e+01_rb/)
      kbo(:,38,13) = (/ &
     &2.8216e+01_rb,3.5039e+01_rb,4.2468e+01_rb,5.0350e+01_rb,5.8547e+01_rb/)
      kbo(:,39,13) = (/ &
     &2.6913e+01_rb,3.3654e+01_rb,4.1024e+01_rb,4.8851e+01_rb,5.7030e+01_rb/)
      kbo(:,40,13) = (/ &
     &2.5061e+01_rb,3.1637e+01_rb,3.8879e+01_rb,4.6620e+01_rb,5.4739e+01_rb/)
      kbo(:,41,13) = (/ &
     &2.3239e+01_rb,2.9632e+01_rb,3.6721e+01_rb,4.4349e+01_rb,5.2395e+01_rb/)
      kbo(:,42,13) = (/ &
     &2.1518e+01_rb,2.7714e+01_rb,3.4638e+01_rb,4.2154e+01_rb,5.0111e+01_rb/)
      kbo(:,43,13) = (/ &
     &1.9578e+01_rb,2.5525e+01_rb,3.2232e+01_rb,3.9588e+01_rb,4.7417e+01_rb/)
      kbo(:,44,13) = (/ &
     &1.7631e+01_rb,2.3285e+01_rb,2.9750e+01_rb,3.6914e+01_rb,4.4596e+01_rb/)
      kbo(:,45,13) = (/ &
     &1.5817e+01_rb,2.1168e+01_rb,2.7372e+01_rb,3.4312e+01_rb,4.1842e+01_rb/)
      kbo(:,46,13) = (/ &
     &1.4031e+01_rb,1.9049e+01_rb,2.4960e+01_rb,3.1648e+01_rb,3.8993e+01_rb/)
      kbo(:,47,13) = (/ &
     &1.2204e+01_rb,1.6850e+01_rb,2.2409e+01_rb,2.8797e+01_rb,3.5899e+01_rb/)
      kbo(:,48,13) = (/ &
     &1.0543e+01_rb,1.4813e+01_rb,2.0005e+01_rb,2.6074e+01_rb,3.2906e+01_rb/)
      kbo(:,49,13) = (/ &
     &9.0410e+00_rb,1.2940e+01_rb,1.7763e+01_rb,2.3491e+01_rb,3.0027e+01_rb/)
      kbo(:,50,13) = (/ &
     &7.7723e+00_rb,1.1328e+01_rb,1.5799e+01_rb,2.1188e+01_rb,2.7436e+01_rb/)
      kbo(:,51,13) = (/ &
     &6.6603e+00_rb,9.8956e+00_rb,1.4031e+01_rb,1.9082e+01_rb,2.5030e+01_rb/)
      kbo(:,52,13) = (/ &
     &5.6483e+00_rb,8.5936e+00_rb,1.2389e+01_rb,1.7108e+01_rb,2.2735e+01_rb/)
      kbo(:,53,13) = (/ &
     &4.7548e+00_rb,7.4145e+00_rb,1.0879e+01_rb,1.5253e+01_rb,2.0556e+01_rb/)
      kbo(:,54,13) = (/ &
     &4.0362e+00_rb,6.4182e+00_rb,9.6047e+00_rb,1.3672e+01_rb,1.8664e+01_rb/)
      kbo(:,55,13) = (/ &
     &3.4328e+00_rb,5.5491e+00_rb,8.4816e+00_rb,1.2254e+01_rb,1.6947e+01_rb/)
      kbo(:,56,13) = (/ &
     &2.9068e+00_rb,4.7740e+00_rb,7.4485e+00_rb,1.0936e+01_rb,1.5335e+01_rb/)
      kbo(:,57,13) = (/ &
     &2.4469e+00_rb,4.0838e+00_rb,6.4921e+00_rb,9.7157e+00_rb,1.3818e+01_rb/)
      kbo(:,58,13) = (/ &
     &2.0735e+00_rb,3.4925e+00_rb,5.6637e+00_rb,8.6413e+00_rb,1.2467e+01_rb/)
      kbo(:,59,13) = (/ &
     &1.9355e+00_rb,3.2709e+00_rb,5.3499e+00_rb,8.2221e+00_rb,1.1950e+01_rb/)
      kbo(:,13,14) = (/ &
     &1.3763e+02_rb,1.5352e+02_rb,1.6709e+02_rb,1.7925e+02_rb,1.9013e+02_rb/)
      kbo(:,14,14) = (/ &
     &1.4836e+02_rb,1.6388e+02_rb,1.7793e+02_rb,1.9078e+02_rb,2.0291e+02_rb/)
      kbo(:,15,14) = (/ &
     &1.5609e+02_rb,1.7227e+02_rb,1.8754e+02_rb,2.0210e+02_rb,2.1605e+02_rb/)
      kbo(:,16,14) = (/ &
     &1.6287e+02_rb,1.8058e+02_rb,1.9742e+02_rb,2.1358e+02_rb,2.2879e+02_rb/)
      kbo(:,17,14) = (/ &
     &1.6936e+02_rb,1.8829e+02_rb,2.0654e+02_rb,2.2380e+02_rb,2.3943e+02_rb/)
      kbo(:,18,14) = (/ &
     &1.7409e+02_rb,1.9426e+02_rb,2.1359e+02_rb,2.3157e+02_rb,2.4840e+02_rb/)
      kbo(:,19,14) = (/ &
     &1.7751e+02_rb,1.9877e+02_rb,2.1900e+02_rb,2.3829e+02_rb,2.5666e+02_rb/)
      kbo(:,20,14) = (/ &
     &1.8029e+02_rb,2.0255e+02_rb,2.2414e+02_rb,2.4492e+02_rb,2.6421e+02_rb/)
      kbo(:,21,14) = (/ &
     &1.8288e+02_rb,2.0645e+02_rb,2.2930e+02_rb,2.5080e+02_rb,2.7109e+02_rb/)
      kbo(:,22,14) = (/ &
     &1.8758e+02_rb,2.1208e+02_rb,2.3543e+02_rb,2.5750e+02_rb,2.7832e+02_rb/)
      kbo(:,23,14) = (/ &
     &1.9239e+02_rb,2.1719e+02_rb,2.4091e+02_rb,2.6350e+02_rb,2.8469e+02_rb/)
      kbo(:,24,14) = (/ &
     &1.9651e+02_rb,2.2173e+02_rb,2.4591e+02_rb,2.6890e+02_rb,2.9066e+02_rb/)
      kbo(:,25,14) = (/ &
     &2.0022e+02_rb,2.2597e+02_rb,2.5071e+02_rb,2.7426e+02_rb,2.9648e+02_rb/)
      kbo(:,26,14) = (/ &
     &2.0417e+02_rb,2.3049e+02_rb,2.5589e+02_rb,2.7996e+02_rb,3.0257e+02_rb/)
      kbo(:,27,14) = (/ &
     &2.0832e+02_rb,2.3518e+02_rb,2.6110e+02_rb,2.8565e+02_rb,3.0854e+02_rb/)
      kbo(:,28,14) = (/ &
     &2.1279e+02_rb,2.4021e+02_rb,2.6653e+02_rb,2.9138e+02_rb,3.1446e+02_rb/)
      kbo(:,29,14) = (/ &
     &2.1796e+02_rb,2.4575e+02_rb,2.7237e+02_rb,2.9736e+02_rb,3.2050e+02_rb/)
      kbo(:,30,14) = (/ &
     &2.2364e+02_rb,2.5171e+02_rb,2.7842e+02_rb,3.0343e+02_rb,3.2655e+02_rb/)
      kbo(:,31,14) = (/ &
     &2.2986e+02_rb,2.5803e+02_rb,2.8476e+02_rb,3.0967e+02_rb,3.3263e+02_rb/)
      kbo(:,32,14) = (/ &
     &2.3644e+02_rb,2.6463e+02_rb,2.9123e+02_rb,3.1599e+02_rb,3.3870e+02_rb/)
      kbo(:,33,14) = (/ &
     &2.4330e+02_rb,2.7142e+02_rb,2.9784e+02_rb,3.2234e+02_rb,3.4468e+02_rb/)
      kbo(:,34,14) = (/ &
     &2.4932e+02_rb,2.7733e+02_rb,3.0352e+02_rb,3.2779e+02_rb,3.4977e+02_rb/)
      kbo(:,35,14) = (/ &
     &2.5266e+02_rb,2.8063e+02_rb,3.0678e+02_rb,3.3096e+02_rb,3.5278e+02_rb/)
      kbo(:,36,14) = (/ &
     &2.5297e+02_rb,2.8101e+02_rb,3.0732e+02_rb,3.3158e+02_rb,3.5352e+02_rb/)
      kbo(:,37,14) = (/ &
     &2.4886e+02_rb,2.7734e+02_rb,3.0396e+02_rb,3.2863e+02_rb,3.5100e+02_rb/)
      kbo(:,38,14) = (/ &
     &2.4449e+02_rb,2.7331e+02_rb,3.0032e+02_rb,3.2535e+02_rb,3.4818e+02_rb/)
      kbo(:,39,14) = (/ &
     &2.4014e+02_rb,2.6925e+02_rb,2.9663e+02_rb,3.2206e+02_rb,3.4524e+02_rb/)
      kbo(:,40,14) = (/ &
     &2.3269e+02_rb,2.6223e+02_rb,2.9013e+02_rb,3.1615e+02_rb,3.3997e+02_rb/)
      kbo(:,41,14) = (/ &
     &2.2477e+02_rb,2.5475e+02_rb,2.8316e+02_rb,3.0972e+02_rb,3.3417e+02_rb/)
      kbo(:,42,14) = (/ &
     &2.1674e+02_rb,2.4709e+02_rb,2.7597e+02_rb,3.0309e+02_rb,3.2816e+02_rb/)
      kbo(:,43,14) = (/ &
     &2.0690e+02_rb,2.3764e+02_rb,2.6708e+02_rb,2.9483e+02_rb,3.2058e+02_rb/)
      kbo(:,44,14) = (/ &
     &1.9620e+02_rb,2.2728e+02_rb,2.5727e+02_rb,2.8565e+02_rb,3.1213e+02_rb/)
      kbo(:,45,14) = (/ &
     &1.8544e+02_rb,2.1679e+02_rb,2.4722e+02_rb,2.7624e+02_rb,3.0339e+02_rb/)
      kbo(:,46,14) = (/ &
     &1.7394e+02_rb,2.0555e+02_rb,2.3639e+02_rb,2.6594e+02_rb,2.9386e+02_rb/)
      kbo(:,47,14) = (/ &
     &1.6112e+02_rb,1.9279e+02_rb,2.2405e+02_rb,2.5421e+02_rb,2.8283e+02_rb/)
      kbo(:,48,14) = (/ &
     &1.4837e+02_rb,1.8000e+02_rb,2.1155e+02_rb,2.4220e+02_rb,2.7155e+02_rb/)
      kbo(:,49,14) = (/ &
     &1.3570e+02_rb,1.6720e+02_rb,1.9889e+02_rb,2.3000e+02_rb,2.5992e+02_rb/)
      kbo(:,50,14) = (/ &
     &1.2401e+02_rb,1.5522e+02_rb,1.8690e+02_rb,2.1834e+02_rb,2.4877e+02_rb/)
      kbo(:,51,14) = (/ &
     &1.1285e+02_rb,1.4368e+02_rb,1.7528e+02_rb,2.0692e+02_rb,2.3777e+02_rb/)
      kbo(:,52,14) = (/ &
     &1.0205e+02_rb,1.3225e+02_rb,1.6371e+02_rb,1.9541e+02_rb,2.2663e+02_rb/)
      kbo(:,53,14) = (/ &
     &9.1561e+01_rb,1.2094e+02_rb,1.5208e+02_rb,1.8381e+02_rb,2.1529e+02_rb/)
      kbo(:,54,14) = (/ &
     &8.2288e+01_rb,1.1082e+02_rb,1.4154e+02_rb,1.7313e+02_rb,2.0480e+02_rb/)
      kbo(:,55,14) = (/ &
     &7.3729e+01_rb,1.0135e+02_rb,1.3152e+02_rb,1.6297e+02_rb,1.9471e+02_rb/)
      kbo(:,56,14) = (/ &
     &6.5504e+01_rb,9.2170e+01_rb,1.2164e+02_rb,1.5283e+02_rb,1.8453e+02_rb/)
      kbo(:,57,14) = (/ &
     &5.7661e+01_rb,8.3295e+01_rb,1.1195e+02_rb,1.4273e+02_rb,1.7435e+02_rb/)
      kbo(:,58,14) = (/ &
     &5.0636e+01_rb,7.5168e+01_rb,1.0296e+02_rb,1.3321e+02_rb,1.6472e+02_rb/)
      kbo(:,59,14) = (/ &
     &4.7932e+01_rb,7.1971e+01_rb,9.9421e+01_rb,1.2944e+02_rb,1.6088e+02_rb/)
      kbo(:,13,15) = (/ &
     &2.4469e+02_rb,2.6735e+02_rb,2.9101e+02_rb,3.1438e+02_rb,3.3674e+02_rb/)
      kbo(:,14,15) = (/ &
     &2.8094e+02_rb,3.0921e+02_rb,3.3701e+02_rb,3.6348e+02_rb,3.8824e+02_rb/)
      kbo(:,15,15) = (/ &
     &3.2313e+02_rb,3.5640e+02_rb,3.8804e+02_rb,4.1754e+02_rb,4.4448e+02_rb/)
      kbo(:,16,15) = (/ &
     &3.6992e+02_rb,4.0761e+02_rb,4.4269e+02_rb,4.7470e+02_rb,5.0364e+02_rb/)
      kbo(:,17,15) = (/ &
     &4.1926e+02_rb,4.6079e+02_rb,4.9879e+02_rb,5.3327e+02_rb,5.6513e+02_rb/)
      kbo(:,18,15) = (/ &
     &4.7156e+02_rb,5.1653e+02_rb,5.5718e+02_rb,5.9454e+02_rb,6.2828e+02_rb/)
      kbo(:,19,15) = (/ &
     &5.2525e+02_rb,5.7308e+02_rb,6.1700e+02_rb,6.5663e+02_rb,6.9127e+02_rb/)
      kbo(:,20,15) = (/ &
     &5.7876e+02_rb,6.2973e+02_rb,6.7595e+02_rb,7.1709e+02_rb,7.5345e+02_rb/)
      kbo(:,21,15) = (/ &
     &6.2988e+02_rb,6.8342e+02_rb,7.3148e+02_rb,7.7494e+02_rb,8.1239e+02_rb/)
      kbo(:,22,15) = (/ &
     &6.8118e+02_rb,7.3681e+02_rb,7.8702e+02_rb,8.3157e+02_rb,8.6949e+02_rb/)
      kbo(:,23,15) = (/ &
     &7.2944e+02_rb,7.8796e+02_rb,8.3973e+02_rb,8.8478e+02_rb,9.2278e+02_rb/)
      kbo(:,24,15) = (/ &
     &7.7652e+02_rb,8.3661e+02_rb,8.8931e+02_rb,9.3457e+02_rb,9.7211e+02_rb/)
      kbo(:,25,15) = (/ &
     &8.2208e+02_rb,8.8322e+02_rb,9.3559e+02_rb,9.8038e+02_rb,1.0170e+03_rb/)
      kbo(:,26,15) = (/ &
     &8.6654e+02_rb,9.2755e+02_rb,9.7898e+02_rb,1.0223e+03_rb,1.0574e+03_rb/)
      kbo(:,27,15) = (/ &
     &9.0828e+02_rb,9.6846e+02_rb,1.0184e+03_rb,1.0599e+03_rb,1.0932e+03_rb/)
      kbo(:,28,15) = (/ &
     &9.4716e+02_rb,1.0058e+03_rb,1.0539e+03_rb,1.0933e+03_rb,1.1247e+03_rb/)
      kbo(:,29,15) = (/ &
     &9.8313e+02_rb,1.0399e+03_rb,1.0858e+03_rb,1.1230e+03_rb,1.1519e+03_rb/)
      kbo(:,30,15) = (/ &
     &1.0160e+03_rb,1.0704e+03_rb,1.1141e+03_rb,1.1489e+03_rb,1.1756e+03_rb/)
      kbo(:,31,15) = (/ &
     &1.0460e+03_rb,1.0956e+03_rb,1.1391e+03_rb,1.1716e+03_rb,1.1960e+03_rb/)
      kbo(:,32,15) = (/ &
     &1.0732e+03_rb,1.1224e+03_rb,1.1612e+03_rb,1.1912e+03_rb,1.2136e+03_rb/)
      kbo(:,33,15) = (/ &
     &1.0978e+03_rb,1.1441e+03_rb,1.1805e+03_rb,1.2081e+03_rb,1.2287e+03_rb/)
      kbo(:,34,15) = (/ &
     &1.1183e+03_rb,1.1622e+03_rb,1.1963e+03_rb,1.2219e+03_rb,1.2409e+03_rb/)
      kbo(:,35,15) = (/ &
     &1.1319e+03_rb,1.1747e+03_rb,1.2074e+03_rb,1.2320e+03_rb,1.2497e+03_rb/)
      kbo(:,36,15) = (/ &
     &1.1393e+03_rb,1.1815e+03_rb,1.2141e+03_rb,1.2383e+03_rb,1.2557e+03_rb/)
      kbo(:,37,15) = (/ &
     &1.1379e+03_rb,1.1816e+03_rb,1.2154e+03_rb,1.2404e+03_rb,1.2585e+03_rb/)
      kbo(:,38,15) = (/ &
     &1.1350e+03_rb,1.1805e+03_rb,1.2153e+03_rb,1.2413e+03_rb,1.2601e+03_rb/)
      kbo(:,39,15) = (/ &
     &1.1312e+03_rb,1.1782e+03_rb,1.2145e+03_rb,1.2415e+03_rb,1.2613e+03_rb/)
      kbo(:,40,15) = (/ &
     &1.1209e+03_rb,1.1709e+03_rb,1.2094e+03_rb,1.2385e+03_rb,1.2597e+03_rb/)
      kbo(:,41,15) = (/ &
     &1.1085e+03_rb,1.1615e+03_rb,1.2028e+03_rb,1.2343e+03_rb,1.2572e+03_rb/)
      kbo(:,42,15) = (/ &
     &1.0950e+03_rb,1.1510e+03_rb,1.1953e+03_rb,1.2290e+03_rb,1.2540e+03_rb/)
      kbo(:,43,15) = (/ &
     &1.0762e+03_rb,1.1364e+03_rb,1.1842e+03_rb,1.2211e+03_rb,1.2485e+03_rb/)
      kbo(:,44,15) = (/ &
     &1.0545e+03_rb,1.1188e+03_rb,1.1706e+03_rb,1.2111e+03_rb,1.2414e+03_rb/)
      kbo(:,45,15) = (/ &
     &1.0306e+03_rb,1.0995e+03_rb,1.1556e+03_rb,1.1997e+03_rb,1.2332e+03_rb/)
      kbo(:,46,15) = (/ &
     &1.0035e+03_rb,1.0771e+03_rb,1.1376e+03_rb,1.1860e+03_rb,1.2231e+03_rb/)
      kbo(:,47,15) = (/ &
     &9.7068e+02_rb,1.0499e+03_rb,1.1155e+03_rb,1.1686e+03_rb,1.2101e+03_rb/)
      kbo(:,48,15) = (/ &
     &9.3560e+02_rb,1.0200e+03_rb,1.0913e+03_rb,1.1493e+03_rb,1.1954e+03_rb/)
      kbo(:,49,15) = (/ &
     &8.9859e+02_rb,9.8816e+02_rb,1.0648e+03_rb,1.1280e+03_rb,1.1788e+03_rb/)
      kbo(:,50,15) = (/ &
     &8.6187e+02_rb,9.5626e+02_rb,1.0379e+03_rb,1.1061e+03_rb,1.1615e+03_rb/)
      kbo(:,51,15) = (/ &
     &8.2436e+02_rb,9.2347e+02_rb,1.0097e+03_rb,1.0831e+03_rb,1.1429e+03_rb/)
      kbo(:,52,15) = (/ &
     &7.8552e+02_rb,8.8919e+02_rb,9.8012e+02_rb,1.0582e+03_rb,1.1227e+03_rb/)
      kbo(:,53,15) = (/ &
     &7.4472e+02_rb,8.5281e+02_rb,9.4852e+02_rb,1.0314e+03_rb,1.1008e+03_rb/)
      kbo(:,54,15) = (/ &
     &7.0603e+02_rb,8.1807e+02_rb,9.1796e+02_rb,1.0053e+03_rb,1.0792e+03_rb/)
      kbo(:,55,15) = (/ &
     &6.6818e+02_rb,7.8362e+02_rb,8.8753e+02_rb,9.7880e+02_rb,1.0572e+03_rb/)
      kbo(:,56,15) = (/ &
     &6.2949e+02_rb,7.4771e+02_rb,8.5573e+02_rb,9.5104e+02_rb,1.0336e+03_rb/)
      kbo(:,57,15) = (/ &
     &5.9013e+02_rb,7.1102e+02_rb,8.2256e+02_rb,9.2198e+02_rb,1.0088e+03_rb/)
      kbo(:,58,15) = (/ &
     &5.5222e+02_rb,6.7509e+02_rb,7.8994e+02_rb,8.9322e+02_rb,9.8385e+02_rb/)
      kbo(:,59,15) = (/ &
     &5.3696e+02_rb,6.6055e+02_rb,7.7661e+02_rb,8.8142e+02_rb,9.7364e+02_rb/)
      kbo(:,13,16) = (/ &
     &4.2045e+02_rb,4.6429e+02_rb,5.0303e+02_rb,5.3671e+02_rb,5.6546e+02_rb/)
      kbo(:,14,16) = (/ &
     &5.0808e+02_rb,5.5889e+02_rb,6.0315e+02_rb,6.4104e+02_rb,6.7319e+02_rb/)
      kbo(:,15,16) = (/ &
     &6.0971e+02_rb,6.6729e+02_rb,7.1666e+02_rb,7.5857e+02_rb,7.9337e+02_rb/)
      kbo(:,16,16) = (/ &
     &7.2443e+02_rb,7.8825e+02_rb,8.4219e+02_rb,8.8727e+02_rb,9.2382e+02_rb/)
      kbo(:,17,16) = (/ &
     &8.5000e+02_rb,9.1908e+02_rb,9.7683e+02_rb,1.0239e+03_rb,1.0613e+03_rb/)
      kbo(:,18,16) = (/ &
     &9.8305e+02_rb,1.0566e+03_rb,1.1167e+03_rb,1.1645e+03_rb,1.2017e+03_rb/)
      kbo(:,19,16) = (/ &
     &1.1204e+03_rb,1.1968e+03_rb,1.2579e+03_rb,1.3058e+03_rb,1.3449e+03_rb/)
      kbo(:,20,16) = (/ &
     &1.2624e+03_rb,1.3419e+03_rb,1.4045e+03_rb,1.4519e+03_rb,1.4914e+03_rb/)
      kbo(:,21,16) = (/ &
     &1.4128e+03_rb,1.4938e+03_rb,1.5557e+03_rb,1.6009e+03_rb,1.6375e+03_rb/)
      kbo(:,22,16) = (/ &
     &1.5738e+03_rb,1.6513e+03_rb,1.7095e+03_rb,1.7503e+03_rb,1.7825e+03_rb/)
      kbo(:,23,16) = (/ &
     &1.7348e+03_rb,1.8057e+03_rb,1.8583e+03_rb,1.8939e+03_rb,1.9201e+03_rb/)
      kbo(:,24,16) = (/ &
     &1.8918e+03_rb,1.9538e+03_rb,1.9990e+03_rb,2.0285e+03_rb,2.0481e+03_rb/)
      kbo(:,25,16) = (/ &
     &2.0394e+03_rb,2.0915e+03_rb,2.1288e+03_rb,2.1514e+03_rb,2.1633e+03_rb/)
      kbo(:,26,16) = (/ &
     &2.1757e+03_rb,2.2166e+03_rb,2.2458e+03_rb,2.2605e+03_rb,2.2652e+03_rb/)
      kbo(:,27,16) = (/ &
     &2.2965e+03_rb,2.3277e+03_rb,2.3485e+03_rb,2.3546e+03_rb,2.3529e+03_rb/)
      kbo(:,28,16) = (/ &
     &2.4019e+03_rb,2.4242e+03_rb,2.4364e+03_rb,2.4355e+03_rb,2.4262e+03_rb/)
      kbo(:,29,16) = (/ &
     &2.4918e+03_rb,2.5053e+03_rb,2.5101e+03_rb,2.5025e+03_rb,2.4871e+03_rb/)
      kbo(:,30,16) = (/ &
     &2.5673e+03_rb,2.5738e+03_rb,2.5706e+03_rb,2.5574e+03_rb,2.5361e+03_rb/)
      kbo(:,31,16) = (/ &
     &2.6301e+03_rb,2.6241e+03_rb,2.6201e+03_rb,2.6003e+03_rb,2.5741e+03_rb/)
      kbo(:,32,16) = (/ &
     &2.6807e+03_rb,2.6744e+03_rb,2.6583e+03_rb,2.6342e+03_rb,2.6035e+03_rb/)
      kbo(:,33,16) = (/ &
     &2.7216e+03_rb,2.7096e+03_rb,2.6883e+03_rb,2.6595e+03_rb,2.6244e+03_rb/)
      kbo(:,34,16) = (/ &
     &2.7530e+03_rb,2.7368e+03_rb,2.7115e+03_rb,2.6793e+03_rb,2.6415e+03_rb/)
      kbo(:,35,16) = (/ &
     &2.7800e+03_rb,2.7599e+03_rb,2.7318e+03_rb,2.6972e+03_rb,2.6568e+03_rb/)
      kbo(:,36,16) = (/ &
     &2.8032e+03_rb,2.7809e+03_rb,2.7511e+03_rb,2.7149e+03_rb,2.6728e+03_rb/)
      kbo(:,37,16) = (/ &
     &2.8255e+03_rb,2.8022e+03_rb,2.7720e+03_rb,2.7352e+03_rb,2.6929e+03_rb/)
      kbo(:,38,16) = (/ &
     &2.8447e+03_rb,2.8207e+03_rb,2.7907e+03_rb,2.7534e+03_rb,2.7108e+03_rb/)
      kbo(:,39,16) = (/ &
     &2.8614e+03_rb,2.8371e+03_rb,2.8072e+03_rb,2.7697e+03_rb,2.7272e+03_rb/)
      kbo(:,40,16) = (/ &
     &2.8775e+03_rb,2.8542e+03_rb,2.8246e+03_rb,2.7882e+03_rb,2.7463e+03_rb/)
      kbo(:,41,16) = (/ &
     &2.8919e+03_rb,2.8698e+03_rb,2.8410e+03_rb,2.8060e+03_rb,2.7644e+03_rb/)
      kbo(:,42,16) = (/ &
     &2.9035e+03_rb,2.8842e+03_rb,2.8555e+03_rb,2.8221e+03_rb,2.7810e+03_rb/)
      kbo(:,43,16) = (/ &
     &2.9142e+03_rb,2.8974e+03_rb,2.8713e+03_rb,2.8381e+03_rb,2.7994e+03_rb/)
      kbo(:,44,16) = (/ &
     &2.9225e+03_rb,2.9100e+03_rb,2.8852e+03_rb,2.8542e+03_rb,2.8174e+03_rb/)
      kbo(:,45,16) = (/ &
     &2.9284e+03_rb,2.9199e+03_rb,2.8988e+03_rb,2.8699e+03_rb,2.8343e+03_rb/)
      kbo(:,46,16) = (/ &
     &2.9312e+03_rb,2.9287e+03_rb,2.9114e+03_rb,2.8839e+03_rb,2.8514e+03_rb/)
      kbo(:,47,16) = (/ &
     &2.9278e+03_rb,2.9353e+03_rb,2.9229e+03_rb,2.8988e+03_rb,2.8673e+03_rb/)
      kbo(:,48,16) = (/ &
     &2.9198e+03_rb,2.9382e+03_rb,2.9322e+03_rb,2.9123e+03_rb,2.8825e+03_rb/)
      kbo(:,49,16) = (/ &
     &2.9055e+03_rb,2.9369e+03_rb,2.9391e+03_rb,2.9238e+03_rb,2.8967e+03_rb/)
      kbo(:,50,16) = (/ &
     &2.8866e+03_rb,2.9312e+03_rb,2.9427e+03_rb,2.9328e+03_rb,2.9100e+03_rb/)
      kbo(:,51,16) = (/ &
     &2.8621e+03_rb,2.9209e+03_rb,2.9426e+03_rb,2.9399e+03_rb,2.9204e+03_rb/)
      kbo(:,52,16) = (/ &
     &2.8313e+03_rb,2.9062e+03_rb,2.9403e+03_rb,2.9447e+03_rb,2.9298e+03_rb/)
      kbo(:,53,16) = (/ &
     &2.7937e+03_rb,2.8849e+03_rb,2.9330e+03_rb,2.9466e+03_rb,2.9378e+03_rb/)
      kbo(:,54,16) = (/ &
     &2.7518e+03_rb,2.8608e+03_rb,2.9213e+03_rb,2.9463e+03_rb,2.9434e+03_rb/)
      kbo(:,55,16) = (/ &
     &2.7066e+03_rb,2.8327e+03_rb,2.9070e+03_rb,2.9427e+03_rb,2.9473e+03_rb/)
      kbo(:,56,16) = (/ &
     &2.6539e+03_rb,2.7985e+03_rb,2.8885e+03_rb,2.9351e+03_rb,2.9485e+03_rb/)
      kbo(:,57,16) = (/ &
     &2.5953e+03_rb,2.7598e+03_rb,2.8651e+03_rb,2.9247e+03_rb,2.9481e+03_rb/)
      kbo(:,58,16) = (/ &
     &2.5319e+03_rb,2.7165e+03_rb,2.8391e+03_rb,2.9117e+03_rb,2.9450e+03_rb/)
      kbo(:,59,16) = (/ &
     &2.5058e+03_rb,2.6981e+03_rb,2.8277e+03_rb,2.9062e+03_rb,2.9427e+03_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &1.0515e-02_rb,1.4860e-02_rb,1.7181e-02_rb,1.6642e-02_rb,1.6644e-02_rb,1.5649e-02_rb, &
     &1.7734e-02_rb,1.7521e-02_rb,1.7868e-02_rb,1.8400e-02_rb,1.9361e-02_rb,2.1487e-02_rb, &
     &2.0192e-02_rb,1.6545e-02_rb,2.0922e-02_rb,2.0922e-02_rb/)
      forrefo(2,:) = (/ &
     &1.0423e-02_rb,1.4593e-02_rb,1.6329e-02_rb,1.7071e-02_rb,1.7252e-02_rb,1.6188e-02_rb, &
     &1.7752e-02_rb,1.7913e-02_rb,1.7551e-02_rb,1.8203e-02_rb,1.7946e-02_rb,1.9828e-02_rb, &
     &2.1566e-02_rb,1.9707e-02_rb,2.0944e-02_rb,2.0944e-02_rb/)
      forrefo(3,:) = (/ &
     &9.2770e-03_rb,1.2818e-02_rb,1.7181e-02_rb,1.7858e-02_rb,1.7888e-02_rb,1.7121e-02_rb, &
     &1.8116e-02_rb,1.8230e-02_rb,1.7719e-02_rb,1.7833e-02_rb,1.8438e-02_rb,1.7995e-02_rb, &
     &2.0895e-02_rb,2.1525e-02_rb,2.0517e-02_rb,2.0954e-02_rb/)
      forrefo(4,:) = (/ &
     &8.3290e-03_rb,1.3483e-02_rb,1.5432e-02_rb,2.0793e-02_rb,1.8404e-02_rb,1.7470e-02_rb, &
     &1.7253e-02_rb,1.7132e-02_rb,1.7119e-02_rb,1.7376e-02_rb,1.7030e-02_rb,1.6847e-02_rb, &
     &1.5562e-02_rb,1.6836e-02_rb,1.8746e-02_rb,2.1233e-02_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 2.41120e-01_rb, 2.27071e-01_rb, 2.13840e-01_rb, 2.01380e-01_rb, 1.89646e-01_rb, &
     & 1.78596e-01_rb, 1.68190e-01_rb, 1.58390e-01_rb, 1.49161e-01_rb, 1.40470e-01_rb/)
      selfrefo(:, 2) = (/ &
     & 3.11156e-01_rb, 2.92249e-01_rb, 2.74490e-01_rb, 2.57810e-01_rb, 2.42144e-01_rb, &
     & 2.27430e-01_rb, 2.13610e-01_rb, 2.00630e-01_rb, 1.88439e-01_rb, 1.76988e-01_rb/)
      selfrefo(:, 3) = (/ &
     & 3.37148e-01_rb, 3.17767e-01_rb, 2.99500e-01_rb, 2.82283e-01_rb, 2.66056e-01_rb, &
     & 2.50762e-01_rb, 2.36347e-01_rb, 2.22760e-01_rb, 2.09955e-01_rb, 1.97885e-01_rb/)
      selfrefo(:, 4) = (/ &
     & 3.57139e-01_rb, 3.32763e-01_rb, 3.10050e-01_rb, 2.88888e-01_rb, 2.69170e-01_rb, &
     & 2.50798e-01_rb, 2.33680e-01_rb, 2.17730e-01_rb, 2.02869e-01_rb, 1.89022e-01_rb/)
      selfrefo(:, 5) = (/ &
     & 3.60626e-01_rb, 3.35433e-01_rb, 3.12000e-01_rb, 2.90204e-01_rb, 2.69931e-01_rb, &
     & 2.51074e-01_rb, 2.33534e-01_rb, 2.17220e-01_rb, 2.02045e-01_rb, 1.87931e-01_rb/)
      selfrefo(:, 6) = (/ &
     & 3.42420e-01_rb, 3.18795e-01_rb, 2.96800e-01_rb, 2.76323e-01_rb, 2.57258e-01_rb, &
     & 2.39509e-01_rb, 2.22985e-01_rb, 2.07600e-01_rb, 1.93277e-01_rb, 1.79942e-01_rb/)
      selfrefo(:, 7) = (/ &
     & 3.65491e-01_rb, 3.41599e-01_rb, 3.19270e-01_rb, 2.98400e-01_rb, 2.78895e-01_rb, &
     & 2.60664e-01_rb, 2.43625e-01_rb, 2.27700e-01_rb, 2.12816e-01_rb, 1.98905e-01_rb/)
      selfrefo(:, 8) = (/ &
     & 3.70354e-01_rb, 3.45005e-01_rb, 3.21390e-01_rb, 2.99392e-01_rb, 2.78899e-01_rb, &
     & 2.59809e-01_rb, 2.42026e-01_rb, 2.25460e-01_rb, 2.10028e-01_rb, 1.95652e-01_rb/)
      selfrefo(:, 9) = (/ &
     & 3.60483e-01_rb, 3.37846e-01_rb, 3.16630e-01_rb, 2.96747e-01_rb, 2.78112e-01_rb, &
     & 2.60648e-01_rb, 2.44280e-01_rb, 2.28940e-01_rb, 2.14563e-01_rb, 2.01090e-01_rb/)
      selfrefo(:,10) = (/ &
     & 3.71845e-01_rb, 3.48164e-01_rb, 3.25990e-01_rb, 3.05229e-01_rb, 2.85790e-01_rb, &
     & 2.67588e-01_rb, 2.50547e-01_rb, 2.34590e-01_rb, 2.19650e-01_rb, 2.05661e-01_rb/)
      selfrefo(:,11) = (/ &
     & 3.60606e-01_rb, 3.40789e-01_rb, 3.22060e-01_rb, 3.04361e-01_rb, 2.87634e-01_rb, &
     & 2.71826e-01_rb, 2.56888e-01_rb, 2.42770e-01_rb, 2.29428e-01_rb, 2.16819e-01_rb/)
      selfrefo(:,12) = (/ &
     & 3.90046e-01_rb, 3.68879e-01_rb, 3.48860e-01_rb, 3.29928e-01_rb, 3.12023e-01_rb, &
     & 2.95089e-01_rb, 2.79075e-01_rb, 2.63930e-01_rb, 2.49607e-01_rb, 2.36061e-01_rb/)
      selfrefo(:,13) = (/ &
     & 4.38542e-01_rb, 4.05139e-01_rb, 3.74280e-01_rb, 3.45771e-01_rb, 3.19434e-01_rb, &
     & 2.95103e-01_rb, 2.72626e-01_rb, 2.51860e-01_rb, 2.32676e-01_rb, 2.14953e-01_rb/)
      selfrefo(:,14) = (/ &
     & 4.19448e-01_rb, 3.81920e-01_rb, 3.47750e-01_rb, 3.16637e-01_rb, 2.88307e-01_rb, &
     & 2.62513e-01_rb, 2.39026e-01_rb, 2.17640e-01_rb, 1.98168e-01_rb, 1.80438e-01_rb/)
      selfrefo(:,15) = (/ &
     & 4.20276e-01_rb, 3.92281e-01_rb, 3.66150e-01_rb, 3.41760e-01_rb, 3.18995e-01_rb, &
     & 2.97746e-01_rb, 2.77912e-01_rb, 2.59400e-01_rb, 2.42121e-01_rb, 2.25993e-01_rb/)
      selfrefo(:,16) = (/ &
     & 4.20276e-01_rb, 3.92281e-01_rb, 3.66150e-01_rb, 3.41760e-01_rb, 3.18995e-01_rb, &
     & 2.97746e-01_rb, 2.77912e-01_rb, 2.59400e-01_rb, 2.42121e-01_rb, 2.25993e-01_rb/)

      end subroutine lw_kgb10

! **************************************************************************
      subroutine lw_kgb11
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg11, only : fracrefao, fracrefbo, kao, kbo, kao_mo2, &
                            kbo_mo2, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P=1053.63 mb, T= 294.2 K
      fracrefao(:) = (/ &
     &  1.4601e-01_rb,1.3824e-01_rb,1.4240e-01_rb,1.3463e-01_rb,1.1948e-01_rb,1.0440e-01_rb, &
     &  8.8667e-02_rb,6.5792e-02_rb,4.3893e-02_rb,4.7941e-03_rb,4.0760e-03_rb,3.3207e-03_rb, &
     &  2.4087e-03_rb,1.3912e-03_rb,4.3482e-04_rb,6.0932e-05_rb/)

! Planck fraction mapping level : P=0.353 mb, T = 262.11 K
      fracrefbo(:) = (/ &
     &  7.2928e-02_rb,1.4900e-01_rb,1.6156e-01_rb,1.5603e-01_rb,1.3934e-01_rb,1.1394e-01_rb, &
     &  8.8783e-02_rb,6.2411e-02_rb,4.0191e-02_rb,4.4587e-03_rb,3.9533e-03_rb,3.0847e-03_rb, &
     &  2.2317e-03_rb,1.4410e-03_rb,5.6722e-04_rb,7.7933e-05_rb/)

!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels > ~100mb and temperatures.  The first
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the corresponding TREF for this  pressure level, 
!     JT = 2 refers to the temperatureTREF-15, JT = 1 is for TREF-30, 
!     JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  The second 
!     index, JP, runs from 1 to 13 and refers to the corresponding 
!     pressure level in PREF (e.g. JP = 1 is for a pressure of 1053.63 mb).  
!     The third index, IG, goes from 1 to 16, and tells us which 
!     g-interval the absorption coefficients are for.

      kao(:, 1, 1) = (/ &
     &4.9423e-02_rb,4.8938e-02_rb,4.8236e-02_rb,4.7630e-02_rb,4.7027e-02_rb/)
      kao(:, 2, 1) = (/ &
     &4.0264e-02_rb,3.9991e-02_rb,3.9414e-02_rb,3.8921e-02_rb,3.8455e-02_rb/)
      kao(:, 3, 1) = (/ &
     &3.2762e-02_rb,3.2612e-02_rb,3.2225e-02_rb,3.1842e-02_rb,3.1448e-02_rb/)
      kao(:, 4, 1) = (/ &
     &2.6929e-02_rb,2.6828e-02_rb,2.6636e-02_rb,2.6311e-02_rb,2.5979e-02_rb/)
      kao(:, 5, 1) = (/ &
     &2.2254e-02_rb,2.2190e-02_rb,2.2139e-02_rb,2.1844e-02_rb,2.1606e-02_rb/)
      kao(:, 6, 1) = (/ &
     &1.8428e-02_rb,1.8406e-02_rb,1.8346e-02_rb,1.8223e-02_rb,1.7986e-02_rb/)
      kao(:, 7, 1) = (/ &
     &1.5302e-02_rb,1.5279e-02_rb,1.5227e-02_rb,1.5173e-02_rb,1.5001e-02_rb/)
      kao(:, 8, 1) = (/ &
     &1.2917e-02_rb,1.2821e-02_rb,1.2762e-02_rb,1.2673e-02_rb,1.2614e-02_rb/)
      kao(:, 9, 1) = (/ &
     &1.4361e-02_rb,1.3355e-02_rb,1.2836e-02_rb,1.2424e-02_rb,1.2069e-02_rb/)
      kao(:,10, 1) = (/ &
     &1.9078e-02_rb,1.9077e-02_rb,1.9301e-02_rb,1.9494e-02_rb,1.9600e-02_rb/)
      kao(:,11, 1) = (/ &
     &1.6651e-02_rb,1.7279e-02_rb,1.7762e-02_rb,1.8212e-02_rb,1.8508e-02_rb/)
      kao(:,12, 1) = (/ &
     &1.4359e-02_rb,1.4912e-02_rb,1.5370e-02_rb,1.5766e-02_rb,1.6068e-02_rb/)
      kao(:,13, 1) = (/ &
     &1.2190e-02_rb,1.2693e-02_rb,1.3086e-02_rb,1.3355e-02_rb,1.3602e-02_rb/)
      kao(:, 1, 2) = (/ &
     &1.3874e-01_rb,1.3507e-01_rb,1.3188e-01_rb,1.2875e-01_rb,1.2573e-01_rb/)
      kao(:, 2, 2) = (/ &
     &1.1449e-01_rb,1.1143e-01_rb,1.0880e-01_rb,1.0626e-01_rb,1.0384e-01_rb/)
      kao(:, 3, 2) = (/ &
     &9.4610e-02_rb,9.2157e-02_rb,8.9899e-02_rb,8.7836e-02_rb,8.5813e-02_rb/)
      kao(:, 4, 2) = (/ &
     &7.8921e-02_rb,7.6917e-02_rb,7.5042e-02_rb,7.3283e-02_rb,7.1619e-02_rb/)
      kao(:, 5, 2) = (/ &
     &6.6025e-02_rb,6.4395e-02_rb,6.2788e-02_rb,6.1406e-02_rb,5.9938e-02_rb/)
      kao(:, 6, 2) = (/ &
     &5.5198e-02_rb,5.3979e-02_rb,5.2652e-02_rb,5.1421e-02_rb,5.0291e-02_rb/)
      kao(:, 7, 2) = (/ &
     &4.5982e-02_rb,4.5142e-02_rb,4.4142e-02_rb,4.3045e-02_rb,4.2121e-02_rb/)
      kao(:, 8, 2) = (/ &
     &3.8128e-02_rb,3.7632e-02_rb,3.6913e-02_rb,3.6127e-02_rb,3.5207e-02_rb/)
      kao(:, 9, 2) = (/ &
     &2.9988e-02_rb,3.0199e-02_rb,2.9924e-02_rb,2.9588e-02_rb,2.9106e-02_rb/)
      kao(:,10, 2) = (/ &
     &4.8557e-02_rb,4.4974e-02_rb,3.7588e-02_rb,3.1508e-02_rb,2.7381e-02_rb/)
      kao(:,11, 2) = (/ &
     &4.6113e-02_rb,4.4271e-02_rb,4.2375e-02_rb,4.0024e-02_rb,3.7665e-02_rb/)
      kao(:,12, 2) = (/ &
     &4.0305e-02_rb,3.9408e-02_rb,3.7504e-02_rb,3.5981e-02_rb,3.4390e-02_rb/)
      kao(:,13, 2) = (/ &
     &3.3779e-02_rb,3.2735e-02_rb,3.1178e-02_rb,2.9814e-02_rb,2.8271e-02_rb/)
      kao(:, 1, 3) = (/ &
     &2.4150e-01_rb,2.3616e-01_rb,2.3111e-01_rb,2.2673e-01_rb,2.2290e-01_rb/)
      kao(:, 2, 3) = (/ &
     &2.0007e-01_rb,1.9568e-01_rb,1.9176e-01_rb,1.8825e-01_rb,1.8518e-01_rb/)
      kao(:, 3, 3) = (/ &
     &1.6573e-01_rb,1.6221e-01_rb,1.5908e-01_rb,1.5616e-01_rb,1.5376e-01_rb/)
      kao(:, 4, 3) = (/ &
     &1.3876e-01_rb,1.3573e-01_rb,1.3298e-01_rb,1.3059e-01_rb,1.2856e-01_rb/)
      kao(:, 5, 3) = (/ &
     &1.1663e-01_rb,1.1399e-01_rb,1.1156e-01_rb,1.0951e-01_rb,1.0785e-01_rb/)
      kao(:, 6, 3) = (/ &
     &9.8259e-02_rb,9.5861e-02_rb,9.3802e-02_rb,9.1948e-02_rb,9.0492e-02_rb/)
      kao(:, 7, 3) = (/ &
     &8.2887e-02_rb,8.0616e-02_rb,7.8766e-02_rb,7.7206e-02_rb,7.5856e-02_rb/)
      kao(:, 8, 3) = (/ &
     &6.9730e-02_rb,6.7730e-02_rb,6.5964e-02_rb,6.4565e-02_rb,6.3437e-02_rb/)
      kao(:, 9, 3) = (/ &
     &5.8101e-02_rb,5.6452e-02_rb,5.5028e-02_rb,5.3674e-02_rb,5.2557e-02_rb/)
      kao(:,10, 3) = (/ &
     &3.7373e-02_rb,3.5041e-02_rb,3.8621e-02_rb,4.1558e-02_rb,4.3012e-02_rb/)
      kao(:,11, 3) = (/ &
     &5.3584e-02_rb,4.0890e-02_rb,3.3882e-02_rb,2.9034e-02_rb,2.6383e-02_rb/)
      kao(:,12, 3) = (/ &
     &5.8428e-02_rb,4.4689e-02_rb,3.5111e-02_rb,2.9172e-02_rb,2.4695e-02_rb/)
      kao(:,13, 3) = (/ &
     &4.7904e-02_rb,3.6965e-02_rb,2.8601e-02_rb,2.3919e-02_rb,2.0680e-02_rb/)
      kao(:, 1, 4) = (/ &
     &4.2548e-01_rb,4.1807e-01_rb,4.1094e-01_rb,4.0423e-01_rb,3.9806e-01_rb/)
      kao(:, 2, 4) = (/ &
     &3.5494e-01_rb,3.4863e-01_rb,3.4235e-01_rb,3.3657e-01_rb,3.3155e-01_rb/)
      kao(:, 3, 4) = (/ &
     &2.9648e-01_rb,2.9099e-01_rb,2.8545e-01_rb,2.8071e-01_rb,2.7648e-01_rb/)
      kao(:, 4, 4) = (/ &
     &2.4939e-01_rb,2.4483e-01_rb,2.4021e-01_rb,2.3626e-01_rb,2.3256e-01_rb/)
      kao(:, 5, 4) = (/ &
     &2.1025e-01_rb,2.0654e-01_rb,2.0285e-01_rb,1.9937e-01_rb,1.9615e-01_rb/)
      kao(:, 6, 4) = (/ &
     &1.7714e-01_rb,1.7417e-01_rb,1.7124e-01_rb,1.6826e-01_rb,1.6547e-01_rb/)
      kao(:, 7, 4) = (/ &
     &1.4884e-01_rb,1.4658e-01_rb,1.4422e-01_rb,1.4179e-01_rb,1.3940e-01_rb/)
      kao(:, 8, 4) = (/ &
     &1.2471e-01_rb,1.2286e-01_rb,1.2117e-01_rb,1.1917e-01_rb,1.1721e-01_rb/)
      kao(:, 9, 4) = (/ &
     &1.0337e-01_rb,1.0218e-01_rb,1.0077e-01_rb,9.9333e-02_rb,9.7869e-02_rb/)
      kao(:,10, 4) = (/ &
     &8.5198e-02_rb,8.3413e-02_rb,8.1099e-02_rb,7.8395e-02_rb,7.6293e-02_rb/)
      kao(:,11, 4) = (/ &
     &6.0820e-02_rb,6.7915e-02_rb,6.7626e-02_rb,6.6844e-02_rb,6.6371e-02_rb/)
      kao(:,12, 4) = (/ &
     &4.0000e-02_rb,4.8190e-02_rb,5.3574e-02_rb,5.5575e-02_rb,5.5151e-02_rb/)
      kao(:,13, 4) = (/ &
     &3.3587e-02_rb,4.0386e-02_rb,4.5135e-02_rb,4.6401e-02_rb,4.5805e-02_rb/)
      kao(:, 1, 5) = (/ &
     &7.7524e-01_rb,7.6536e-01_rb,7.5522e-01_rb,7.4431e-01_rb,7.3311e-01_rb/)
      kao(:, 2, 5) = (/ &
     &6.5185e-01_rb,6.4405e-01_rb,6.3568e-01_rb,6.2684e-01_rb,6.1765e-01_rb/)
      kao(:, 3, 5) = (/ &
     &5.4580e-01_rb,5.3942e-01_rb,5.3289e-01_rb,5.2546e-01_rb,5.1768e-01_rb/)
      kao(:, 4, 5) = (/ &
     &4.5945e-01_rb,4.5416e-01_rb,4.4868e-01_rb,4.4229e-01_rb,4.3592e-01_rb/)
      kao(:, 5, 5) = (/ &
     &3.8777e-01_rb,3.8350e-01_rb,3.7876e-01_rb,3.7352e-01_rb,3.6853e-01_rb/)
      kao(:, 6, 5) = (/ &
     &3.2750e-01_rb,3.2403e-01_rb,3.2009e-01_rb,3.1598e-01_rb,3.1216e-01_rb/)
      kao(:, 7, 5) = (/ &
     &2.7610e-01_rb,2.7333e-01_rb,2.7016e-01_rb,2.6699e-01_rb,2.6402e-01_rb/)
      kao(:, 8, 5) = (/ &
     &2.3237e-01_rb,2.3021e-01_rb,2.2764e-01_rb,2.2511e-01_rb,2.2263e-01_rb/)
      kao(:, 9, 5) = (/ &
     &1.9461e-01_rb,1.9312e-01_rb,1.9116e-01_rb,1.8911e-01_rb,1.8708e-01_rb/)
      kao(:,10, 5) = (/ &
     &1.4772e-01_rb,1.4997e-01_rb,1.5067e-01_rb,1.5164e-01_rb,1.5169e-01_rb/)
      kao(:,11, 5) = (/ &
     &1.1317e-01_rb,1.1322e-01_rb,1.1552e-01_rb,1.1824e-01_rb,1.1886e-01_rb/)
      kao(:,12, 5) = (/ &
     &9.7948e-02_rb,9.4310e-02_rb,9.3527e-02_rb,9.3094e-02_rb,9.5273e-02_rb/)
      kao(:,13, 5) = (/ &
     &8.1443e-02_rb,7.8686e-02_rb,7.8076e-02_rb,7.8298e-02_rb,8.0221e-02_rb/)
      kao(:, 1, 6) = (/ &
     &1.5697e+00_rb,1.5485e+00_rb,1.5270e+00_rb,1.5062e+00_rb,1.4845e+00_rb/)
      kao(:, 2, 6) = (/ &
     &1.3465e+00_rb,1.3286e+00_rb,1.3101e+00_rb,1.2912e+00_rb,1.2719e+00_rb/)
      kao(:, 3, 6) = (/ &
     &1.1474e+00_rb,1.1322e+00_rb,1.1161e+00_rb,1.0997e+00_rb,1.0837e+00_rb/)
      kao(:, 4, 6) = (/ &
     &9.7861e-01_rb,9.6591e-01_rb,9.5236e-01_rb,9.3871e-01_rb,9.2522e-01_rb/)
      kao(:, 5, 6) = (/ &
     &8.3345e-01_rb,8.2282e-01_rb,8.1147e-01_rb,8.0015e-01_rb,7.8882e-01_rb/)
      kao(:, 6, 6) = (/ &
     &7.0768e-01_rb,6.9919e-01_rb,6.8975e-01_rb,6.8018e-01_rb,6.7024e-01_rb/)
      kao(:, 7, 6) = (/ &
     &5.9916e-01_rb,5.9240e-01_rb,5.8444e-01_rb,5.7632e-01_rb,5.6762e-01_rb/)
      kao(:, 8, 6) = (/ &
     &5.0612e-01_rb,5.0051e-01_rb,4.9423e-01_rb,4.8731e-01_rb,4.7989e-01_rb/)
      kao(:, 9, 6) = (/ &
     &4.2622e-01_rb,4.2196e-01_rb,4.1690e-01_rb,4.1124e-01_rb,4.0525e-01_rb/)
      kao(:,10, 6) = (/ &
     &3.5041e-01_rb,3.4937e-01_rb,3.4686e-01_rb,3.4326e-01_rb,3.3900e-01_rb/)
      kao(:,11, 6) = (/ &
     &2.8138e-01_rb,2.8270e-01_rb,2.8314e-01_rb,2.8109e-01_rb,2.7863e-01_rb/)
      kao(:,12, 6) = (/ &
     &2.2569e-01_rb,2.3115e-01_rb,2.3249e-01_rb,2.3233e-01_rb,2.3100e-01_rb/)
      kao(:,13, 6) = (/ &
     &1.8817e-01_rb,1.9206e-01_rb,1.9341e-01_rb,1.9316e-01_rb,1.9216e-01_rb/)
      kao(:, 1, 7) = (/ &
     &3.4196e+00_rb,3.3801e+00_rb,3.3399e+00_rb,3.2946e+00_rb,3.2477e+00_rb/)
      kao(:, 2, 7) = (/ &
     &3.0295e+00_rb,2.9903e+00_rb,2.9469e+00_rb,2.9032e+00_rb,2.8599e+00_rb/)
      kao(:, 3, 7) = (/ &
     &2.6483e+00_rb,2.6096e+00_rb,2.5696e+00_rb,2.5316e+00_rb,2.4936e+00_rb/)
      kao(:, 4, 7) = (/ &
     &2.3044e+00_rb,2.2705e+00_rb,2.2374e+00_rb,2.2055e+00_rb,2.1724e+00_rb/)
      kao(:, 5, 7) = (/ &
     &1.9970e+00_rb,1.9688e+00_rb,1.9416e+00_rb,1.9143e+00_rb,1.8852e+00_rb/)
      kao(:, 6, 7) = (/ &
     &1.7224e+00_rb,1.6992e+00_rb,1.6760e+00_rb,1.6526e+00_rb,1.6276e+00_rb/)
      kao(:, 7, 7) = (/ &
     &1.4770e+00_rb,1.4579e+00_rb,1.4384e+00_rb,1.4185e+00_rb,1.3976e+00_rb/)
      kao(:, 8, 7) = (/ &
     &1.2593e+00_rb,1.2433e+00_rb,1.2274e+00_rb,1.2112e+00_rb,1.1939e+00_rb/)
      kao(:, 9, 7) = (/ &
     &1.0697e+00_rb,1.0563e+00_rb,1.0434e+00_rb,1.0295e+00_rb,1.0150e+00_rb/)
      kao(:,10, 7) = (/ &
     &9.0213e-01_rb,8.9139e-01_rb,8.8119e-01_rb,8.6954e-01_rb,8.5716e-01_rb/)
      kao(:,11, 7) = (/ &
     &7.5061e-01_rb,7.4446e-01_rb,7.3597e-01_rb,7.2638e-01_rb,7.1615e-01_rb/)
      kao(:,12, 7) = (/ &
     &6.2457e-01_rb,6.1924e-01_rb,6.1219e-01_rb,6.0465e-01_rb,5.9633e-01_rb/)
      kao(:,13, 7) = (/ &
     &5.2204e-01_rb,5.1758e-01_rb,5.1182e-01_rb,5.0530e-01_rb,4.9787e-01_rb/)
      kao(:, 1, 8) = (/ &
     &7.9795e+00_rb,7.8457e+00_rb,7.7132e+00_rb,7.5890e+00_rb,7.4645e+00_rb/)
      kao(:, 2, 8) = (/ &
     &7.5076e+00_rb,7.3733e+00_rb,7.2519e+00_rb,7.1316e+00_rb,7.0099e+00_rb/)
      kao(:, 3, 8) = (/ &
     &6.9340e+00_rb,6.8148e+00_rb,6.7036e+00_rb,6.5875e+00_rb,6.4766e+00_rb/)
      kao(:, 4, 8) = (/ &
     &6.3414e+00_rb,6.2366e+00_rb,6.1298e+00_rb,6.0215e+00_rb,5.9183e+00_rb/)
      kao(:, 5, 8) = (/ &
     &5.7373e+00_rb,5.6423e+00_rb,5.5412e+00_rb,5.4426e+00_rb,5.3434e+00_rb/)
      kao(:, 6, 8) = (/ &
     &5.1373e+00_rb,5.0482e+00_rb,4.9577e+00_rb,4.8621e+00_rb,4.7659e+00_rb/)
      kao(:, 7, 8) = (/ &
     &4.5492e+00_rb,4.4684e+00_rb,4.3821e+00_rb,4.2928e+00_rb,4.2067e+00_rb/)
      kao(:, 8, 8) = (/ &
     &3.9884e+00_rb,3.9140e+00_rb,3.8349e+00_rb,3.7563e+00_rb,3.6821e+00_rb/)
      kao(:, 9, 8) = (/ &
     &3.4656e+00_rb,3.3995e+00_rb,3.3300e+00_rb,3.2636e+00_rb,3.2010e+00_rb/)
      kao(:,10, 8) = (/ &
     &2.9841e+00_rb,2.9267e+00_rb,2.8680e+00_rb,2.8124e+00_rb,2.7592e+00_rb/)
      kao(:,11, 8) = (/ &
     &2.5294e+00_rb,2.4795e+00_rb,2.4321e+00_rb,2.3862e+00_rb,2.3418e+00_rb/)
      kao(:,12, 8) = (/ &
     &2.1315e+00_rb,2.0918e+00_rb,2.0527e+00_rb,2.0146e+00_rb,1.9776e+00_rb/)
      kao(:,13, 8) = (/ &
     &1.7932e+00_rb,1.7598e+00_rb,1.7272e+00_rb,1.6957e+00_rb,1.6643e+00_rb/)
      kao(:, 1, 9) = (/ &
     &1.8965e+01_rb,1.8711e+01_rb,1.8469e+01_rb,1.8242e+01_rb,1.8035e+01_rb/)
      kao(:, 2, 9) = (/ &
     &1.9621e+01_rb,1.9373e+01_rb,1.9124e+01_rb,1.8882e+01_rb,1.8648e+01_rb/)
      kao(:, 3, 9) = (/ &
     &2.0111e+01_rb,1.9849e+01_rb,1.9576e+01_rb,1.9310e+01_rb,1.9047e+01_rb/)
      kao(:, 4, 9) = (/ &
     &2.0303e+01_rb,2.0017e+01_rb,1.9726e+01_rb,1.9447e+01_rb,1.9174e+01_rb/)
      kao(:, 5, 9) = (/ &
     &2.0176e+01_rb,1.9882e+01_rb,1.9591e+01_rb,1.9312e+01_rb,1.9035e+01_rb/)
      kao(:, 6, 9) = (/ &
     &1.9747e+01_rb,1.9457e+01_rb,1.9166e+01_rb,1.8888e+01_rb,1.8606e+01_rb/)
      kao(:, 7, 9) = (/ &
     &1.9018e+01_rb,1.8727e+01_rb,1.8446e+01_rb,1.8173e+01_rb,1.7878e+01_rb/)
      kao(:, 8, 9) = (/ &
     &1.8015e+01_rb,1.7734e+01_rb,1.7466e+01_rb,1.7191e+01_rb,1.6898e+01_rb/)
      kao(:, 9, 9) = (/ &
     &1.6785e+01_rb,1.6527e+01_rb,1.6264e+01_rb,1.5992e+01_rb,1.5711e+01_rb/)
      kao(:,10, 9) = (/ &
     &1.5379e+01_rb,1.5144e+01_rb,1.4899e+01_rb,1.4648e+01_rb,1.4396e+01_rb/)
      kao(:,11, 9) = (/ &
     &1.3809e+01_rb,1.3588e+01_rb,1.3370e+01_rb,1.3146e+01_rb,1.2927e+01_rb/)
      kao(:,12, 9) = (/ &
     &1.2243e+01_rb,1.2056e+01_rb,1.1862e+01_rb,1.1669e+01_rb,1.1464e+01_rb/)
      kao(:,13, 9) = (/ &
     &1.0743e+01_rb,1.0581e+01_rb,1.0416e+01_rb,1.0238e+01_rb,1.0057e+01_rb/)
      kao(:, 1,10) = (/ &
     &3.4918e+01_rb,3.4410e+01_rb,3.3938e+01_rb,3.3384e+01_rb,3.2811e+01_rb/)
      kao(:, 2,10) = (/ &
     &3.7421e+01_rb,3.6860e+01_rb,3.6321e+01_rb,3.5822e+01_rb,3.5445e+01_rb/)
      kao(:, 3,10) = (/ &
     &4.0412e+01_rb,3.9722e+01_rb,3.9217e+01_rb,3.8832e+01_rb,3.8460e+01_rb/)
      kao(:, 4,10) = (/ &
     &4.3162e+01_rb,4.2539e+01_rb,4.2112e+01_rb,4.1640e+01_rb,4.1120e+01_rb/)
      kao(:, 5,10) = (/ &
     &4.5609e+01_rb,4.5045e+01_rb,4.4552e+01_rb,4.3899e+01_rb,4.3271e+01_rb/)
      kao(:, 6,10) = (/ &
     &4.7877e+01_rb,4.7338e+01_rb,4.6747e+01_rb,4.6031e+01_rb,4.5492e+01_rb/)
      kao(:, 7,10) = (/ &
     &4.9831e+01_rb,4.9324e+01_rb,4.8717e+01_rb,4.8005e+01_rb,4.7611e+01_rb/)
      kao(:, 8,10) = (/ &
     &5.1031e+01_rb,5.0574e+01_rb,4.9915e+01_rb,4.9244e+01_rb,4.8761e+01_rb/)
      kao(:, 9,10) = (/ &
     &5.1465e+01_rb,5.0984e+01_rb,5.0270e+01_rb,4.9612e+01_rb,4.9032e+01_rb/)
      kao(:,10,10) = (/ &
     &5.0911e+01_rb,5.0267e+01_rb,4.9512e+01_rb,4.8818e+01_rb,4.8092e+01_rb/)
      kao(:,11,10) = (/ &
     &4.8719e+01_rb,4.7980e+01_rb,4.7190e+01_rb,4.6412e+01_rb,4.5593e+01_rb/)
      kao(:,12,10) = (/ &
     &4.5566e+01_rb,4.4779e+01_rb,4.4020e+01_rb,4.3287e+01_rb,4.2695e+01_rb/)
      kao(:,13,10) = (/ &
     &4.1842e+01_rb,4.1126e+01_rb,4.0474e+01_rb,3.9934e+01_rb,3.9455e+01_rb/)
      kao(:, 1,11) = (/ &
     &4.3485e+01_rb,4.3086e+01_rb,4.2575e+01_rb,4.2046e+01_rb,4.1574e+01_rb/)
      kao(:, 2,11) = (/ &
     &4.8017e+01_rb,4.7410e+01_rb,4.6743e+01_rb,4.6112e+01_rb,4.5460e+01_rb/)
      kao(:, 3,11) = (/ &
     &5.1964e+01_rb,5.1351e+01_rb,5.0628e+01_rb,4.9892e+01_rb,4.9112e+01_rb/)
      kao(:, 4,11) = (/ &
     &5.5970e+01_rb,5.5232e+01_rb,5.4425e+01_rb,5.3658e+01_rb,5.2964e+01_rb/)
      kao(:, 5,11) = (/ &
     &6.0234e+01_rb,5.9398e+01_rb,5.8467e+01_rb,5.7720e+01_rb,5.7104e+01_rb/)
      kao(:, 6,11) = (/ &
     &6.4036e+01_rb,6.3174e+01_rb,6.2265e+01_rb,6.1623e+01_rb,6.0928e+01_rb/)
      kao(:, 7,11) = (/ &
     &6.7087e+01_rb,6.6347e+01_rb,6.5603e+01_rb,6.4996e+01_rb,6.4192e+01_rb/)
      kao(:, 8,11) = (/ &
     &6.9922e+01_rb,6.9386e+01_rb,6.8783e+01_rb,6.8159e+01_rb,6.7317e+01_rb/)
      kao(:, 9,11) = (/ &
     &7.2326e+01_rb,7.1863e+01_rb,7.1404e+01_rb,7.0792e+01_rb,6.9972e+01_rb/)
      kao(:,10,11) = (/ &
     &7.3782e+01_rb,7.3486e+01_rb,7.3017e+01_rb,7.2321e+01_rb,7.1486e+01_rb/)
      kao(:,11,11) = (/ &
     &7.4137e+01_rb,7.3779e+01_rb,7.3113e+01_rb,7.2218e+01_rb,7.1354e+01_rb/)
      kao(:,12,11) = (/ &
     &7.3214e+01_rb,7.2506e+01_rb,7.1526e+01_rb,7.0521e+01_rb,6.9446e+01_rb/)
      kao(:,13,11) = (/ &
     &7.0360e+01_rb,6.9342e+01_rb,6.8249e+01_rb,6.7098e+01_rb,6.5980e+01_rb/)
      kao(:, 1,12) = (/ &
     &5.1887e+01_rb,5.1393e+01_rb,5.0969e+01_rb,5.0675e+01_rb,5.0203e+01_rb/)
      kao(:, 2,12) = (/ &
     &5.9704e+01_rb,5.9201e+01_rb,5.8829e+01_rb,5.8469e+01_rb,5.7899e+01_rb/)
      kao(:, 3,12) = (/ &
     &6.7858e+01_rb,6.7238e+01_rb,6.6756e+01_rb,6.6195e+01_rb,6.5626e+01_rb/)
      kao(:, 4,12) = (/ &
     &7.5062e+01_rb,7.4258e+01_rb,7.3522e+01_rb,7.2807e+01_rb,7.2094e+01_rb/)
      kao(:, 5,12) = (/ &
     &8.1152e+01_rb,8.0143e+01_rb,7.9393e+01_rb,7.8647e+01_rb,7.7881e+01_rb/)
      kao(:, 6,12) = (/ &
     &8.7340e+01_rb,8.6150e+01_rb,8.5372e+01_rb,8.4516e+01_rb,8.3657e+01_rb/)
      kao(:, 7,12) = (/ &
     &9.4121e+01_rb,9.2665e+01_rb,9.1695e+01_rb,9.0706e+01_rb,8.9764e+01_rb/)
      kao(:, 8,12) = (/ &
     &1.0086e+02_rb,9.9171e+01_rb,9.7989e+01_rb,9.6851e+01_rb,9.5921e+01_rb/)
      kao(:, 9,12) = (/ &
     &1.0671e+02_rb,1.0483e+02_rb,1.0352e+02_rb,1.0233e+02_rb,1.0145e+02_rb/)
      kao(:,10,12) = (/ &
     &1.1135e+02_rb,1.0957e+02_rb,1.0846e+02_rb,1.0742e+02_rb,1.0652e+02_rb/)
      kao(:,11,12) = (/ &
     &1.1430e+02_rb,1.1336e+02_rb,1.1254e+02_rb,1.1170e+02_rb,1.1063e+02_rb/)
      kao(:,12,12) = (/ &
     &1.1681e+02_rb,1.1638e+02_rb,1.1571e+02_rb,1.1469e+02_rb,1.1347e+02_rb/)
      kao(:,13,12) = (/ &
     &1.1832e+02_rb,1.1787e+02_rb,1.1696e+02_rb,1.1585e+02_rb,1.1448e+02_rb/)
      kao(:, 1,13) = (/ &
     &6.2138e+01_rb,6.1536e+01_rb,6.0957e+01_rb,6.0185e+01_rb,5.9644e+01_rb/)
      kao(:, 2,13) = (/ &
     &7.3104e+01_rb,7.2436e+01_rb,7.1604e+01_rb,7.0616e+01_rb,6.9923e+01_rb/)
      kao(:, 3,13) = (/ &
     &8.5575e+01_rb,8.4690e+01_rb,8.3633e+01_rb,8.2595e+01_rb,8.1733e+01_rb/)
      kao(:, 4,13) = (/ &
     &9.8880e+01_rb,9.7932e+01_rb,9.6845e+01_rb,9.5711e+01_rb,9.4631e+01_rb/)
      kao(:, 5,13) = (/ &
     &1.1273e+02_rb,1.1168e+02_rb,1.1051e+02_rb,1.0912e+02_rb,1.0775e+02_rb/)
      kao(:, 6,13) = (/ &
     &1.2651e+02_rb,1.2538e+02_rb,1.2389e+02_rb,1.2226e+02_rb,1.2074e+02_rb/)
      kao(:, 7,13) = (/ &
     &1.4003e+02_rb,1.3861e+02_rb,1.3671e+02_rb,1.3477e+02_rb,1.3302e+02_rb/)
      kao(:, 8,13) = (/ &
     &1.5267e+02_rb,1.5069e+02_rb,1.4845e+02_rb,1.4633e+02_rb,1.4433e+02_rb/)
      kao(:, 9,13) = (/ &
     &1.6510e+02_rb,1.6280e+02_rb,1.6039e+02_rb,1.5803e+02_rb,1.5572e+02_rb/)
      kao(:,10,13) = (/ &
     &1.7758e+02_rb,1.7506e+02_rb,1.7227e+02_rb,1.6951e+02_rb,1.6718e+02_rb/)
      kao(:,11,13) = (/ &
     &1.8910e+02_rb,1.8565e+02_rb,1.8227e+02_rb,1.7970e+02_rb,1.7779e+02_rb/)
      kao(:,12,13) = (/ &
     &1.9894e+02_rb,1.9500e+02_rb,1.9204e+02_rb,1.8987e+02_rb,1.8782e+02_rb/)
      kao(:,13,13) = (/ &
     &2.0665e+02_rb,2.0326e+02_rb,2.0074e+02_rb,1.9867e+02_rb,1.9680e+02_rb/)
      kao(:, 1,14) = (/ &
     &7.2473e+01_rb,7.1368e+01_rb,7.0428e+01_rb,6.9932e+01_rb,6.9604e+01_rb/)
      kao(:, 2,14) = (/ &
     &8.7789e+01_rb,8.6447e+01_rb,8.5315e+01_rb,8.4536e+01_rb,8.3865e+01_rb/)
      kao(:, 3,14) = (/ &
     &1.0616e+02_rb,1.0473e+02_rb,1.0338e+02_rb,1.0236e+02_rb,1.0132e+02_rb/)
      kao(:, 4,14) = (/ &
     &1.2669e+02_rb,1.2502e+02_rb,1.2342e+02_rb,1.2210e+02_rb,1.2081e+02_rb/)
      kao(:, 5,14) = (/ &
     &1.4951e+02_rb,1.4761e+02_rb,1.4568e+02_rb,1.4409e+02_rb,1.4262e+02_rb/)
      kao(:, 6,14) = (/ &
     &1.7487e+02_rb,1.7259e+02_rb,1.7054e+02_rb,1.6873e+02_rb,1.6691e+02_rb/)
      kao(:, 7,14) = (/ &
     &2.0246e+02_rb,2.0010e+02_rb,1.9793e+02_rb,1.9585e+02_rb,1.9382e+02_rb/)
      kao(:, 8,14) = (/ &
     &2.3272e+02_rb,2.3034e+02_rb,2.2811e+02_rb,2.2588e+02_rb,2.2363e+02_rb/)
      kao(:, 9,14) = (/ &
     &2.6521e+02_rb,2.6298e+02_rb,2.6062e+02_rb,2.5802e+02_rb,2.5525e+02_rb/)
      kao(:,10,14) = (/ &
     &2.9843e+02_rb,2.9609e+02_rb,2.9357e+02_rb,2.9069e+02_rb,2.8721e+02_rb/)
      kao(:,11,14) = (/ &
     &3.3019e+02_rb,3.2789e+02_rb,3.2506e+02_rb,3.2111e+02_rb,3.1667e+02_rb/)
      kao(:,12,14) = (/ &
     &3.6200e+02_rb,3.5928e+02_rb,3.5512e+02_rb,3.5030e+02_rb,3.4598e+02_rb/)
      kao(:,13,14) = (/ &
     &3.9453e+02_rb,3.9054e+02_rb,3.8556e+02_rb,3.8029e+02_rb,3.7518e+02_rb/)
      kao(:, 1,15) = (/ &
     &8.4296e+01_rb,8.4294e+01_rb,8.4592e+01_rb,8.4817e+01_rb,8.4944e+01_rb/)
      kao(:, 2,15) = (/ &
     &1.0316e+02_rb,1.0286e+02_rb,1.0304e+02_rb,1.0333e+02_rb,1.0370e+02_rb/)
      kao(:, 3,15) = (/ &
     &1.2648e+02_rb,1.2563e+02_rb,1.2552e+02_rb,1.2542e+02_rb,1.2564e+02_rb/)
      kao(:, 4,15) = (/ &
     &1.5355e+02_rb,1.5194e+02_rb,1.5117e+02_rb,1.5062e+02_rb,1.5044e+02_rb/)
      kao(:, 5,15) = (/ &
     &1.8607e+02_rb,1.8324e+02_rb,1.8143e+02_rb,1.8019e+02_rb,1.7911e+02_rb/)
      kao(:, 6,15) = (/ &
     &2.2538e+02_rb,2.2140e+02_rb,2.1806e+02_rb,2.1567e+02_rb,2.1369e+02_rb/)
      kao(:, 7,15) = (/ &
     &2.7204e+02_rb,2.6705e+02_rb,2.6220e+02_rb,2.5861e+02_rb,2.5549e+02_rb/)
      kao(:, 8,15) = (/ &
     &3.2681e+02_rb,3.2069e+02_rb,3.1478e+02_rb,3.0961e+02_rb,3.0517e+02_rb/)
      kao(:, 9,15) = (/ &
     &3.9109e+02_rb,3.8311e+02_rb,3.7570e+02_rb,3.6941e+02_rb,3.6369e+02_rb/)
      kao(:,10,15) = (/ &
     &4.6558e+02_rb,4.5519e+02_rb,4.4566e+02_rb,4.3786e+02_rb,4.3114e+02_rb/)
      kao(:,11,15) = (/ &
     &5.4422e+02_rb,5.3170e+02_rb,5.2160e+02_rb,5.1300e+02_rb,5.0502e+02_rb/)
      kao(:,12,15) = (/ &
     &6.3083e+02_rb,6.1809e+02_rb,6.0782e+02_rb,5.9755e+02_rb,5.8778e+02_rb/)
      kao(:,13,15) = (/ &
     &7.2633e+02_rb,7.1401e+02_rb,7.0254e+02_rb,6.9098e+02_rb,6.8020e+02_rb/)
      kao(:, 1,16) = (/ &
     &9.0293e+01_rb,9.1524e+01_rb,9.2499e+01_rb,9.3157e+01_rb,9.3569e+01_rb/)
      kao(:, 2,16) = (/ &
     &1.1228e+02_rb,1.1346e+02_rb,1.1477e+02_rb,1.1575e+02_rb,1.1638e+02_rb/)
      kao(:, 3,16) = (/ &
     &1.3962e+02_rb,1.4080e+02_rb,1.4217e+02_rb,1.4343e+02_rb,1.4444e+02_rb/)
      kao(:, 4,16) = (/ &
     &1.7141e+02_rb,1.7272e+02_rb,1.7425e+02_rb,1.7564e+02_rb,1.7671e+02_rb/)
      kao(:, 5,16) = (/ &
     &2.0680e+02_rb,2.1031e+02_rb,2.1221e+02_rb,2.1383e+02_rb,2.1512e+02_rb/)
      kao(:, 6,16) = (/ &
     &2.5314e+02_rb,2.5478e+02_rb,2.5735e+02_rb,2.5955e+02_rb,2.6116e+02_rb/)
      kao(:, 7,16) = (/ &
     &3.0938e+02_rb,3.0774e+02_rb,3.1113e+02_rb,3.1414e+02_rb,3.1633e+02_rb/)
      kao(:, 8,16) = (/ &
     &3.7909e+02_rb,3.7514e+02_rb,3.7476e+02_rb,3.7901e+02_rb,3.8212e+02_rb/)
      kao(:, 9,16) = (/ &
     &4.6433e+02_rb,4.5795e+02_rb,4.5378e+02_rb,4.5558e+02_rb,4.5979e+02_rb/)
      kao(:,10,16) = (/ &
     &5.6730e+02_rb,5.5767e+02_rb,5.5088e+02_rb,5.4957e+02_rb,5.5150e+02_rb/)
      kao(:,11,16) = (/ &
     &6.8668e+02_rb,6.7309e+02_rb,6.6516e+02_rb,6.6333e+02_rb,6.6322e+02_rb/)
      kao(:,12,16) = (/ &
     &8.3053e+02_rb,8.1176e+02_rb,8.0043e+02_rb,7.9744e+02_rb,7.9485e+02_rb/)
      kao(:,13,16) = (/ &
     &1.0026e+03_rb,9.7775e+02_rb,9.6058e+02_rb,9.5314e+02_rb,9.4719e+02_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &1.2190e-02_rb,1.2693e-02_rb,1.3086e-02_rb,1.3355e-02_rb,1.3602e-02_rb/)
      kbo(:,14, 1) = (/ &
     &1.0639e-02_rb,1.0991e-02_rb,1.1315e-02_rb,1.1564e-02_rb,1.1518e-02_rb/)
      kbo(:,15, 1) = (/ &
     &9.4343e-03_rb,9.6676e-03_rb,9.5938e-03_rb,9.5530e-03_rb,9.4890e-03_rb/)
      kbo(:,16, 1) = (/ &
     &7.9681e-03_rb,7.8419e-03_rb,7.8162e-03_rb,7.7492e-03_rb,7.7652e-03_rb/)
      kbo(:,17, 1) = (/ &
     &6.4068e-03_rb,6.3539e-03_rb,6.3265e-03_rb,6.3294e-03_rb,6.2292e-03_rb/)
      kbo(:,18, 1) = (/ &
     &5.1420e-03_rb,5.1335e-03_rb,5.0780e-03_rb,5.0408e-03_rb,5.0223e-03_rb/)
      kbo(:,19, 1) = (/ &
     &4.1739e-03_rb,4.1404e-03_rb,4.1032e-03_rb,4.0908e-03_rb,4.0562e-03_rb/)
      kbo(:,20, 1) = (/ &
     &3.3887e-03_rb,3.3668e-03_rb,3.3481e-03_rb,3.3359e-03_rb,3.2948e-03_rb/)
      kbo(:,21, 1) = (/ &
     &2.7580e-03_rb,2.7551e-03_rb,2.7356e-03_rb,2.7092e-03_rb,2.6841e-03_rb/)
      kbo(:,22, 1) = (/ &
     &2.2558e-03_rb,2.2435e-03_rb,2.2235e-03_rb,2.2041e-03_rb,2.1870e-03_rb/)
      kbo(:,23, 1) = (/ &
     &1.8426e-03_rb,1.8324e-03_rb,1.8088e-03_rb,1.7975e-03_rb,1.7882e-03_rb/)
      kbo(:,24, 1) = (/ &
     &1.5091e-03_rb,1.4914e-03_rb,1.4770e-03_rb,1.4698e-03_rb,1.4675e-03_rb/)
      kbo(:,25, 1) = (/ &
     &1.2361e-03_rb,1.2170e-03_rb,1.2068e-03_rb,1.2059e-03_rb,1.2054e-03_rb/)
      kbo(:,26, 1) = (/ &
     &1.0050e-03_rb,9.9424e-04_rb,9.8921e-04_rb,9.8950e-04_rb,9.9102e-04_rb/)
      kbo(:,27, 1) = (/ &
     &8.1924e-04_rb,8.1181e-04_rb,8.1111e-04_rb,8.1148e-04_rb,8.1057e-04_rb/)
      kbo(:,28, 1) = (/ &
     &6.6881e-04_rb,6.6398e-04_rb,6.6473e-04_rb,6.6536e-04_rb,6.6057e-04_rb/)
      kbo(:,29, 1) = (/ &
     &5.4452e-04_rb,5.4181e-04_rb,5.4254e-04_rb,5.4342e-04_rb,5.3551e-04_rb/)
      kbo(:,30, 1) = (/ &
     &4.4259e-04_rb,4.4140e-04_rb,4.4207e-04_rb,4.4326e-04_rb,4.2874e-04_rb/)
      kbo(:,31, 1) = (/ &
     &3.5899e-04_rb,3.5826e-04_rb,3.5911e-04_rb,3.5984e-04_rb,3.4298e-04_rb/)
      kbo(:,32, 1) = (/ &
     &2.9138e-04_rb,2.9104e-04_rb,2.9178e-04_rb,2.9237e-04_rb,2.7522e-04_rb/)
      kbo(:,33, 1) = (/ &
     &2.3674e-04_rb,2.3667e-04_rb,2.3708e-04_rb,2.3749e-04_rb,2.2116e-04_rb/)
      kbo(:,34, 1) = (/ &
     &1.9293e-04_rb,1.9295e-04_rb,1.9307e-04_rb,1.9353e-04_rb,1.7944e-04_rb/)
      kbo(:,35, 1) = (/ &
     &1.5741e-04_rb,1.5737e-04_rb,1.5735e-04_rb,1.5778e-04_rb,1.4713e-04_rb/)
      kbo(:,36, 1) = (/ &
     &1.2872e-04_rb,1.2833e-04_rb,1.2856e-04_rb,1.2886e-04_rb,1.2147e-04_rb/)
      kbo(:,37, 1) = (/ &
     &1.0573e-04_rb,1.0521e-04_rb,1.0532e-04_rb,1.0552e-04_rb,1.0153e-04_rb/)
      kbo(:,38, 1) = (/ &
     &8.6912e-05_rb,8.6294e-05_rb,8.6278e-05_rb,8.6393e-05_rb,8.5663e-05_rb/)
      kbo(:,39, 1) = (/ &
     &7.1476e-05_rb,7.0797e-05_rb,7.0693e-05_rb,7.0706e-05_rb,7.0752e-05_rb/)
      kbo(:,40, 1) = (/ &
     &5.8778e-05_rb,5.8375e-05_rb,5.8149e-05_rb,5.8120e-05_rb,5.8038e-05_rb/)
      kbo(:,41, 1) = (/ &
     &4.8208e-05_rb,4.8291e-05_rb,4.7939e-05_rb,4.7839e-05_rb,4.7717e-05_rb/)
      kbo(:,42, 1) = (/ &
     &3.9735e-05_rb,3.9794e-05_rb,3.9588e-05_rb,3.9414e-05_rb,3.9275e-05_rb/)
      kbo(:,43, 1) = (/ &
     &3.3069e-05_rb,3.2750e-05_rb,3.2898e-05_rb,3.2646e-05_rb,3.2434e-05_rb/)
      kbo(:,44, 1) = (/ &
     &2.7160e-05_rb,2.7263e-05_rb,2.7120e-05_rb,2.7165e-05_rb,2.6874e-05_rb/)
      kbo(:,45, 1) = (/ &
     &2.2285e-05_rb,2.2565e-05_rb,2.2508e-05_rb,2.2456e-05_rb,2.2357e-05_rb/)
      kbo(:,46, 1) = (/ &
     &1.8468e-05_rb,1.8529e-05_rb,1.8744e-05_rb,1.8640e-05_rb,1.8584e-05_rb/)
      kbo(:,47, 1) = (/ &
     &1.5267e-05_rb,1.5373e-05_rb,1.5389e-05_rb,1.5590e-05_rb,1.5417e-05_rb/)
      kbo(:,48, 1) = (/ &
     &1.2576e-05_rb,1.2765e-05_rb,1.2771e-05_rb,1.2828e-05_rb,1.2953e-05_rb/)
      kbo(:,49, 1) = (/ &
     &1.0526e-05_rb,1.0503e-05_rb,1.0641e-05_rb,1.0645e-05_rb,1.0671e-05_rb/)
      kbo(:,50, 1) = (/ &
     &8.5146e-06_rb,8.8286e-06_rb,8.7879e-06_rb,8.8870e-06_rb,8.8515e-06_rb/)
      kbo(:,51, 1) = (/ &
     &6.8551e-06_rb,7.1714e-06_rb,7.4262e-06_rb,7.3568e-06_rb,7.4039e-06_rb/)
      kbo(:,52, 1) = (/ &
     &5.5111e-06_rb,5.7854e-06_rb,6.0431e-06_rb,6.2230e-06_rb,6.1473e-06_rb/)
      kbo(:,53, 1) = (/ &
     &4.4382e-06_rb,4.6559e-06_rb,4.8718e-06_rb,5.0815e-06_rb,5.1941e-06_rb/)
      kbo(:,54, 1) = (/ &
     &3.6177e-06_rb,3.7935e-06_rb,3.9613e-06_rb,4.1342e-06_rb,4.2936e-06_rb/)
      kbo(:,55, 1) = (/ &
     &2.9336e-06_rb,3.1004e-06_rb,3.2420e-06_rb,3.3846e-06_rb,3.5194e-06_rb/)
      kbo(:,56, 1) = (/ &
     &2.3777e-06_rb,2.5393e-06_rb,2.6559e-06_rb,2.7737e-06_rb,2.8819e-06_rb/)
      kbo(:,57, 1) = (/ &
     &1.9258e-06_rb,2.0635e-06_rb,2.1875e-06_rb,2.2772e-06_rb,2.3684e-06_rb/)
      kbo(:,58, 1) = (/ &
     &1.5658e-06_rb,1.6813e-06_rb,1.7910e-06_rb,1.8800e-06_rb,1.9530e-06_rb/)
      kbo(:,59, 1) = (/ &
     &1.3188e-06_rb,1.4141e-06_rb,1.4979e-06_rb,1.5764e-06_rb,1.6384e-06_rb/)
      kbo(:,13, 2) = (/ &
     &3.3779e-02_rb,3.2735e-02_rb,3.1178e-02_rb,2.9814e-02_rb,2.8271e-02_rb/)
      kbo(:,14, 2) = (/ &
     &2.7945e-02_rb,2.6846e-02_rb,2.5419e-02_rb,2.4157e-02_rb,2.2814e-02_rb/)
      kbo(:,15, 2) = (/ &
     &2.2748e-02_rb,2.1521e-02_rb,2.0571e-02_rb,1.9559e-02_rb,1.8047e-02_rb/)
      kbo(:,16, 2) = (/ &
     &1.8337e-02_rb,1.7587e-02_rb,1.6750e-02_rb,1.5705e-02_rb,1.3259e-02_rb/)
      kbo(:,17, 2) = (/ &
     &1.5034e-02_rb,1.4343e-02_rb,1.3736e-02_rb,1.1541e-02_rb,9.7400e-03_rb/)
      kbo(:,18, 2) = (/ &
     &1.2164e-02_rb,1.1828e-02_rb,9.9998e-03_rb,8.2911e-03_rb,6.9774e-03_rb/)
      kbo(:,19, 2) = (/ &
     &1.0053e-02_rb,8.7760e-03_rb,7.2944e-03_rb,6.0291e-03_rb,5.2188e-03_rb/)
      kbo(:,20, 2) = (/ &
     &8.0996e-03_rb,6.6628e-03_rb,5.4744e-03_rb,4.5474e-03_rb,4.0343e-03_rb/)
      kbo(:,21, 2) = (/ &
     &6.1685e-03_rb,5.0455e-03_rb,4.1278e-03_rb,3.5537e-03_rb,3.1461e-03_rb/)
      kbo(:,22, 2) = (/ &
     &4.6680e-03_rb,3.7894e-03_rb,3.1347e-03_rb,2.7602e-03_rb,2.4538e-03_rb/)
      kbo(:,23, 2) = (/ &
     &3.5105e-03_rb,2.8430e-03_rb,2.4311e-03_rb,2.1442e-03_rb,1.9116e-03_rb/)
      kbo(:,24, 2) = (/ &
     &2.6394e-03_rb,2.1852e-03_rb,1.8960e-03_rb,1.6702e-03_rb,1.4993e-03_rb/)
      kbo(:,25, 2) = (/ &
     &2.0041e-03_rb,1.6920e-03_rb,1.4784e-03_rb,1.3107e-03_rb,1.1883e-03_rb/)
      kbo(:,26, 2) = (/ &
     &1.5414e-03_rb,1.3189e-03_rb,1.1564e-03_rb,1.0333e-03_rb,9.4710e-04_rb/)
      kbo(:,27, 2) = (/ &
     &1.2025e-03_rb,1.0381e-03_rb,9.1423e-04_rb,8.1996e-04_rb,7.6323e-04_rb/)
      kbo(:,28, 2) = (/ &
     &9.4481e-04_rb,8.2314e-04_rb,7.2601e-04_rb,6.5680e-04_rb,6.2105e-04_rb/)
      kbo(:,29, 2) = (/ &
     &7.4291e-04_rb,6.5138e-04_rb,5.7968e-04_rb,5.2914e-04_rb,5.0508e-04_rb/)
      kbo(:,30, 2) = (/ &
     &5.8796e-04_rb,5.1841e-04_rb,4.6161e-04_rb,4.2521e-04_rb,4.1671e-04_rb/)
      kbo(:,31, 2) = (/ &
     &4.6334e-04_rb,4.1198e-04_rb,3.6828e-04_rb,3.4267e-04_rb,3.4322e-04_rb/)
      kbo(:,32, 2) = (/ &
     &3.6683e-04_rb,3.2718e-04_rb,2.9598e-04_rb,2.7743e-04_rb,2.8106e-04_rb/)
      kbo(:,33, 2) = (/ &
     &2.9220e-04_rb,2.6104e-04_rb,2.3912e-04_rb,2.2529e-04_rb,2.3032e-04_rb/)
      kbo(:,34, 2) = (/ &
     &2.3494e-04_rb,2.1114e-04_rb,1.9458e-04_rb,1.8336e-04_rb,1.8849e-04_rb/)
      kbo(:,35, 2) = (/ &
     &1.9117e-04_rb,1.7220e-04_rb,1.5969e-04_rb,1.5013e-04_rb,1.5359e-04_rb/)
      kbo(:,36, 2) = (/ &
     &1.5747e-04_rb,1.4221e-04_rb,1.3171e-04_rb,1.2370e-04_rb,1.2459e-04_rb/)
      kbo(:,37, 2) = (/ &
     &1.3250e-04_rb,1.1910e-04_rb,1.0968e-04_rb,1.0296e-04_rb,1.0130e-04_rb/)
      kbo(:,38, 2) = (/ &
     &1.1160e-04_rb,9.9715e-05_rb,9.1665e-05_rb,8.5730e-05_rb,8.1933e-05_rb/)
      kbo(:,39, 2) = (/ &
     &9.4135e-05_rb,8.3974e-05_rb,7.6783e-05_rb,7.1718e-05_rb,6.7517e-05_rb/)
      kbo(:,40, 2) = (/ &
     &8.0997e-05_rb,7.2288e-05_rb,6.5235e-05_rb,6.0491e-05_rb,5.6970e-05_rb/)
      kbo(:,41, 2) = (/ &
     &7.0436e-05_rb,6.2097e-05_rb,5.5431e-05_rb,5.1123e-05_rb,4.8054e-05_rb/)
      kbo(:,42, 2) = (/ &
     &6.1655e-05_rb,5.3367e-05_rb,4.7548e-05_rb,4.3416e-05_rb,4.0552e-05_rb/)
      kbo(:,43, 2) = (/ &
     &5.5272e-05_rb,4.6832e-05_rb,4.1549e-05_rb,3.7143e-05_rb,3.4563e-05_rb/)
      kbo(:,44, 2) = (/ &
     &5.0350e-05_rb,4.1911e-05_rb,3.6573e-05_rb,3.2553e-05_rb,2.9621e-05_rb/)
      kbo(:,45, 2) = (/ &
     &4.6680e-05_rb,3.8062e-05_rb,3.2149e-05_rb,2.8703e-05_rb,2.5694e-05_rb/)
      kbo(:,46, 2) = (/ &
     &4.2769e-05_rb,3.5698e-05_rb,2.9139e-05_rb,2.5199e-05_rb,2.2638e-05_rb/)
      kbo(:,47, 2) = (/ &
     &3.6477e-05_rb,3.2822e-05_rb,2.7467e-05_rb,2.2730e-05_rb,2.0075e-05_rb/)
      kbo(:,48, 2) = (/ &
     &3.0609e-05_rb,2.9832e-05_rb,2.5641e-05_rb,2.1246e-05_rb,1.7927e-05_rb/)
      kbo(:,49, 2) = (/ &
     &2.5892e-05_rb,2.4976e-05_rb,2.3819e-05_rb,2.0178e-05_rb,1.6719e-05_rb/)
      kbo(:,50, 2) = (/ &
     &2.2068e-05_rb,2.0897e-05_rb,2.0445e-05_rb,1.8401e-05_rb,1.5648e-05_rb/)
      kbo(:,51, 2) = (/ &
     &1.8894e-05_rb,1.8060e-05_rb,1.7047e-05_rb,1.6809e-05_rb,1.4426e-05_rb/)
      kbo(:,52, 2) = (/ &
     &1.6131e-05_rb,1.5345e-05_rb,1.4613e-05_rb,1.3961e-05_rb,1.3398e-05_rb/)
      kbo(:,53, 2) = (/ &
     &1.3727e-05_rb,1.3222e-05_rb,1.2619e-05_rb,1.1906e-05_rb,1.1467e-05_rb/)
      kbo(:,54, 2) = (/ &
     &1.1644e-05_rb,1.1203e-05_rb,1.0779e-05_rb,1.0373e-05_rb,9.7090e-06_rb/)
      kbo(:,55, 2) = (/ &
     &9.9606e-06_rb,9.5440e-06_rb,9.2216e-06_rb,8.7822e-06_rb,8.3198e-06_rb/)
      kbo(:,56, 2) = (/ &
     &8.4517e-06_rb,8.1696e-06_rb,7.8373e-06_rb,7.5094e-06_rb,7.2266e-06_rb/)
      kbo(:,57, 2) = (/ &
     &7.1634e-06_rb,7.0985e-06_rb,6.7296e-06_rb,6.4794e-06_rb,6.1647e-06_rb/)
      kbo(:,58, 2) = (/ &
     &6.2286e-06_rb,5.9713e-06_rb,5.8323e-06_rb,5.5370e-06_rb,5.3465e-06_rb/)
      kbo(:,59, 2) = (/ &
     &5.3307e-06_rb,5.0367e-06_rb,4.9690e-06_rb,4.6758e-06_rb,4.5062e-06_rb/)
      kbo(:,13, 3) = (/ &
     &4.7904e-02_rb,3.6965e-02_rb,2.8601e-02_rb,2.3919e-02_rb,2.0680e-02_rb/)
      kbo(:,14, 3) = (/ &
     &3.6570e-02_rb,2.7810e-02_rb,2.1848e-02_rb,1.8508e-02_rb,1.6582e-02_rb/)
      kbo(:,15, 3) = (/ &
     &2.6097e-02_rb,1.9628e-02_rb,1.6268e-02_rb,1.4252e-02_rb,1.3801e-02_rb/)
      kbo(:,16, 3) = (/ &
     &1.8293e-02_rb,1.4393e-02_rb,1.2369e-02_rb,1.1568e-02_rb,1.2855e-02_rb/)
      kbo(:,17, 3) = (/ &
     &1.3004e-02_rb,1.0838e-02_rb,9.6882e-03_rb,1.0818e-02_rb,1.1950e-02_rb/)
      kbo(:,18, 3) = (/ &
     &9.6007e-03_rb,8.2004e-03_rb,9.1048e-03_rb,1.0148e-02_rb,1.0662e-02_rb/)
      kbo(:,19, 3) = (/ &
     &7.2730e-03_rb,7.5847e-03_rb,8.4339e-03_rb,9.0734e-03_rb,9.2118e-03_rb/)
      kbo(:,20, 3) = (/ &
     &5.9400e-03_rb,6.7100e-03_rb,7.4249e-03_rb,7.7409e-03_rb,7.7182e-03_rb/)
      kbo(:,21, 3) = (/ &
     &5.2740e-03_rb,5.8871e-03_rb,6.3746e-03_rb,6.4908e-03_rb,6.3895e-03_rb/)
      kbo(:,22, 3) = (/ &
     &4.5973e-03_rb,5.1329e-03_rb,5.3865e-03_rb,5.3473e-03_rb,5.2488e-03_rb/)
      kbo(:,23, 3) = (/ &
     &4.0216e-03_rb,4.3958e-03_rb,4.5011e-03_rb,4.3919e-03_rb,4.3384e-03_rb/)
      kbo(:,24, 3) = (/ &
     &3.5050e-03_rb,3.6950e-03_rb,3.6848e-03_rb,3.6232e-03_rb,3.5889e-03_rb/)
      kbo(:,25, 3) = (/ &
     &3.0079e-03_rb,3.0947e-03_rb,3.0321e-03_rb,2.9863e-03_rb,2.9569e-03_rb/)
      kbo(:,26, 3) = (/ &
     &2.5223e-03_rb,2.5389e-03_rb,2.4899e-03_rb,2.4616e-03_rb,2.4119e-03_rb/)
      kbo(:,27, 3) = (/ &
     &2.0904e-03_rb,2.0720e-03_rb,2.0405e-03_rb,2.0263e-03_rb,1.9704e-03_rb/)
      kbo(:,28, 3) = (/ &
     &1.7240e-03_rb,1.6892e-03_rb,1.6695e-03_rb,1.6561e-03_rb,1.6055e-03_rb/)
      kbo(:,29, 3) = (/ &
     &1.4068e-03_rb,1.3720e-03_rb,1.3607e-03_rb,1.3422e-03_rb,1.3070e-03_rb/)
      kbo(:,30, 3) = (/ &
     &1.1328e-03_rb,1.1145e-03_rb,1.1117e-03_rb,1.0886e-03_rb,1.0628e-03_rb/)
      kbo(:,31, 3) = (/ &
     &9.1589e-04_rb,9.0415e-04_rb,9.0310e-04_rb,8.8069e-04_rb,8.6082e-04_rb/)
      kbo(:,32, 3) = (/ &
     &7.4268e-04_rb,7.3425e-04_rb,7.3091e-04_rb,7.1193e-04_rb,7.0016e-04_rb/)
      kbo(:,33, 3) = (/ &
     &5.9881e-04_rb,5.9536e-04_rb,5.8909e-04_rb,5.7513e-04_rb,5.6984e-04_rb/)
      kbo(:,34, 3) = (/ &
     &4.8522e-04_rb,4.8326e-04_rb,4.7702e-04_rb,4.6806e-04_rb,4.6482e-04_rb/)
      kbo(:,35, 3) = (/ &
     &3.9408e-04_rb,3.9282e-04_rb,3.8758e-04_rb,3.8191e-04_rb,3.7951e-04_rb/)
      kbo(:,36, 3) = (/ &
     &3.2090e-04_rb,3.1998e-04_rb,3.1651e-04_rb,3.1185e-04_rb,3.1064e-04_rb/)
      kbo(:,37, 3) = (/ &
     &2.6225e-04_rb,2.6215e-04_rb,2.6078e-04_rb,2.5651e-04_rb,2.5549e-04_rb/)
      kbo(:,38, 3) = (/ &
     &2.1480e-04_rb,2.1472e-04_rb,2.1476e-04_rb,2.1116e-04_rb,2.0969e-04_rb/)
      kbo(:,39, 3) = (/ &
     &1.7555e-04_rb,1.7537e-04_rb,1.7598e-04_rb,1.7350e-04_rb,1.7249e-04_rb/)
      kbo(:,40, 3) = (/ &
     &1.4506e-04_rb,1.4338e-04_rb,1.4427e-04_rb,1.4359e-04_rb,1.4195e-04_rb/)
      kbo(:,41, 3) = (/ &
     &1.1753e-04_rb,1.1789e-04_rb,1.1842e-04_rb,1.1899e-04_rb,1.1706e-04_rb/)
      kbo(:,42, 3) = (/ &
     &9.4811e-05_rb,9.6651e-05_rb,9.6909e-05_rb,9.7780e-05_rb,9.6573e-05_rb/)
      kbo(:,43, 3) = (/ &
     &7.5161e-05_rb,7.9489e-05_rb,7.9256e-05_rb,8.0096e-05_rb,8.0065e-05_rb/)
      kbo(:,44, 3) = (/ &
     &5.8204e-05_rb,6.3635e-05_rb,6.5251e-05_rb,6.5174e-05_rb,6.6162e-05_rb/)
      kbo(:,45, 3) = (/ &
     &4.3809e-05_rb,5.0393e-05_rb,5.3351e-05_rb,5.3552e-05_rb,5.3892e-05_rb/)
      kbo(:,46, 3) = (/ &
     &3.2980e-05_rb,3.7988e-05_rb,4.2459e-05_rb,4.3951e-05_rb,4.3847e-05_rb/)
      kbo(:,47, 3) = (/ &
     &2.7646e-05_rb,2.8723e-05_rb,3.2811e-05_rb,3.5391e-05_rb,3.6137e-05_rb/)
      kbo(:,48, 3) = (/ &
     &2.4543e-05_rb,2.1924e-05_rb,2.4598e-05_rb,2.7741e-05_rb,2.9339e-05_rb/)
      kbo(:,49, 3) = (/ &
     &2.2466e-05_rb,1.9218e-05_rb,1.8251e-05_rb,2.0876e-05_rb,2.3032e-05_rb/)
      kbo(:,50, 3) = (/ &
     &2.1181e-05_rb,1.7247e-05_rb,1.5253e-05_rb,1.5927e-05_rb,1.7960e-05_rb/)
      kbo(:,51, 3) = (/ &
     &2.0829e-05_rb,1.5682e-05_rb,1.3335e-05_rb,1.2152e-05_rb,1.3683e-05_rb/)
      kbo(:,52, 3) = (/ &
     &2.0195e-05_rb,1.5008e-05_rb,1.2032e-05_rb,1.0543e-05_rb,1.0211e-05_rb/)
      kbo(:,53, 3) = (/ &
     &1.9323e-05_rb,1.4642e-05_rb,1.1065e-05_rb,9.3359e-06_rb,8.4701e-06_rb/)
      kbo(:,54, 3) = (/ &
     &1.7666e-05_rb,1.4231e-05_rb,1.0596e-05_rb,8.3241e-06_rb,7.3928e-06_rb/)
      kbo(:,55, 3) = (/ &
     &1.5462e-05_rb,1.3265e-05_rb,1.0248e-05_rb,7.8075e-06_rb,6.5230e-06_rb/)
      kbo(:,56, 3) = (/ &
     &1.3370e-05_rb,1.2106e-05_rb,9.7995e-06_rb,7.4236e-06_rb,5.8878e-06_rb/)
      kbo(:,57, 3) = (/ &
     &1.1515e-05_rb,1.0653e-05_rb,9.2073e-06_rb,7.1981e-06_rb,5.4819e-06_rb/)
      kbo(:,58, 3) = (/ &
     &1.0034e-05_rb,9.2814e-06_rb,8.4358e-06_rb,6.8772e-06_rb,5.2130e-06_rb/)
      kbo(:,59, 3) = (/ &
     &8.4460e-06_rb,7.8198e-06_rb,7.3452e-06_rb,6.2331e-06_rb,4.7237e-06_rb/)
      kbo(:,13, 4) = (/ &
     &3.3587e-02_rb,4.0386e-02_rb,4.5135e-02_rb,4.6401e-02_rb,4.5805e-02_rb/)
      kbo(:,14, 4) = (/ &
     &3.1030e-02_rb,3.6513e-02_rb,3.9636e-02_rb,3.9291e-02_rb,3.8805e-02_rb/)
      kbo(:,15, 4) = (/ &
     &2.9952e-02_rb,3.3504e-02_rb,3.3526e-02_rb,3.3527e-02_rb,3.2785e-02_rb/)
      kbo(:,16, 4) = (/ &
     &2.7625e-02_rb,2.8337e-02_rb,2.8532e-02_rb,2.8121e-02_rb,2.7038e-02_rb/)
      kbo(:,17, 4) = (/ &
     &2.3797e-02_rb,2.4231e-02_rb,2.3927e-02_rb,2.3137e-02_rb,2.2675e-02_rb/)
      kbo(:,18, 4) = (/ &
     &2.0364e-02_rb,2.0218e-02_rb,1.9689e-02_rb,1.9353e-02_rb,1.9321e-02_rb/)
      kbo(:,19, 4) = (/ &
     &1.7135e-02_rb,1.6671e-02_rb,1.6414e-02_rb,1.6325e-02_rb,1.6362e-02_rb/)
      kbo(:,20, 4) = (/ &
     &1.4048e-02_rb,1.3836e-02_rb,1.3650e-02_rb,1.3725e-02_rb,1.3743e-02_rb/)
      kbo(:,21, 4) = (/ &
     &1.1598e-02_rb,1.1451e-02_rb,1.1386e-02_rb,1.1434e-02_rb,1.1536e-02_rb/)
      kbo(:,22, 4) = (/ &
     &9.5671e-03_rb,9.4431e-03_rb,9.4962e-03_rb,9.5565e-03_rb,9.6638e-03_rb/)
      kbo(:,23, 4) = (/ &
     &7.8733e-03_rb,7.8272e-03_rb,7.8608e-03_rb,8.0045e-03_rb,8.0429e-03_rb/)
      kbo(:,24, 4) = (/ &
     &6.4854e-03_rb,6.5239e-03_rb,6.5932e-03_rb,6.6847e-03_rb,6.6897e-03_rb/)
      kbo(:,25, 4) = (/ &
     &5.3475e-03_rb,5.3959e-03_rb,5.5055e-03_rb,5.5513e-03_rb,5.5516e-03_rb/)
      kbo(:,26, 4) = (/ &
     &4.4370e-03_rb,4.4984e-03_rb,4.5821e-03_rb,4.5978e-03_rb,4.6194e-03_rb/)
      kbo(:,27, 4) = (/ &
     &3.6654e-03_rb,3.7378e-03_rb,3.7830e-03_rb,3.7897e-03_rb,3.8209e-03_rb/)
      kbo(:,28, 4) = (/ &
     &3.0135e-03_rb,3.0852e-03_rb,3.1155e-03_rb,3.1198e-03_rb,3.1538e-03_rb/)
      kbo(:,29, 4) = (/ &
     &2.4690e-03_rb,2.5299e-03_rb,2.5457e-03_rb,2.5581e-03_rb,2.5884e-03_rb/)
      kbo(:,30, 4) = (/ &
     &2.0282e-03_rb,2.0645e-03_rb,2.0716e-03_rb,2.0943e-03_rb,2.1183e-03_rb/)
      kbo(:,31, 4) = (/ &
     &1.6532e-03_rb,1.6753e-03_rb,1.6810e-03_rb,1.7064e-03_rb,1.7271e-03_rb/)
      kbo(:,32, 4) = (/ &
     &1.3440e-03_rb,1.3610e-03_rb,1.3688e-03_rb,1.3929e-03_rb,1.4080e-03_rb/)
      kbo(:,33, 4) = (/ &
     &1.0937e-03_rb,1.1058e-03_rb,1.1183e-03_rb,1.1384e-03_rb,1.1495e-03_rb/)
      kbo(:,34, 4) = (/ &
     &8.9328e-04_rb,9.0340e-04_rb,9.1730e-04_rb,9.3209e-04_rb,9.4284e-04_rb/)
      kbo(:,35, 4) = (/ &
     &7.2895e-04_rb,7.3894e-04_rb,7.5120e-04_rb,7.6322e-04_rb,7.7389e-04_rb/)
      kbo(:,36, 4) = (/ &
     &5.9465e-04_rb,6.0388e-04_rb,6.1360e-04_rb,6.2477e-04_rb,6.3446e-04_rb/)
      kbo(:,37, 4) = (/ &
     &4.8509e-04_rb,4.9426e-04_rb,5.0234e-04_rb,5.1316e-04_rb,5.2248e-04_rb/)
      kbo(:,38, 4) = (/ &
     &3.9518e-04_rb,4.0486e-04_rb,4.1118e-04_rb,4.2126e-04_rb,4.3035e-04_rb/)
      kbo(:,39, 4) = (/ &
     &3.2206e-04_rb,3.3126e-04_rb,3.3692e-04_rb,3.4570e-04_rb,3.5386e-04_rb/)
      kbo(:,40, 4) = (/ &
     &2.6189e-04_rb,2.7140e-04_rb,2.7744e-04_rb,2.8454e-04_rb,2.9309e-04_rb/)
      kbo(:,41, 4) = (/ &
     &2.1513e-04_rb,2.2151e-04_rb,2.2843e-04_rb,2.3405e-04_rb,2.4235e-04_rb/)
      kbo(:,42, 4) = (/ &
     &1.7598e-04_rb,1.8130e-04_rb,1.8775e-04_rb,1.9283e-04_rb,2.0002e-04_rb/)
      kbo(:,43, 4) = (/ &
     &1.4391e-04_rb,1.4742e-04_rb,1.5366e-04_rb,1.5883e-04_rb,1.6416e-04_rb/)
      kbo(:,44, 4) = (/ &
     &1.1869e-04_rb,1.2058e-04_rb,1.2494e-04_rb,1.3042e-04_rb,1.3475e-04_rb/)
      kbo(:,45, 4) = (/ &
     &9.8053e-05_rb,9.8247e-05_rb,1.0155e-04_rb,1.0624e-04_rb,1.1079e-04_rb/)
      kbo(:,46, 4) = (/ &
     &8.0733e-05_rb,8.0988e-05_rb,8.2961e-05_rb,8.6570e-05_rb,9.0733e-05_rb/)
      kbo(:,47, 4) = (/ &
     &6.6646e-05_rb,6.6793e-05_rb,6.7614e-05_rb,7.0248e-05_rb,7.3658e-05_rb/)
      kbo(:,48, 4) = (/ &
     &5.5124e-05_rb,5.5074e-05_rb,5.5667e-05_rb,5.7150e-05_rb,5.9800e-05_rb/)
      kbo(:,49, 4) = (/ &
     &4.4694e-05_rb,4.5221e-05_rb,4.5769e-05_rb,4.6677e-05_rb,4.8625e-05_rb/)
      kbo(:,50, 4) = (/ &
     &3.5617e-05_rb,3.7343e-05_rb,3.7522e-05_rb,3.8384e-05_rb,3.9484e-05_rb/)
      kbo(:,51, 4) = (/ &
     &2.7917e-05_rb,3.0153e-05_rb,3.0833e-05_rb,3.1535e-05_rb,3.2403e-05_rb/)
      kbo(:,52, 4) = (/ &
     &2.1904e-05_rb,2.3837e-05_rb,2.5410e-05_rb,2.5925e-05_rb,2.6605e-05_rb/)
      kbo(:,53, 4) = (/ &
     &1.6636e-05_rb,1.9086e-05_rb,2.0539e-05_rb,2.1236e-05_rb,2.1831e-05_rb/)
      kbo(:,54, 4) = (/ &
     &1.2490e-05_rb,1.4743e-05_rb,1.6336e-05_rb,1.7546e-05_rb,1.7865e-05_rb/)
      kbo(:,55, 4) = (/ &
     &1.0189e-05_rb,1.1568e-05_rb,1.3196e-05_rb,1.4286e-05_rb,1.4740e-05_rb/)
      kbo(:,56, 4) = (/ &
     &9.0688e-06_rb,8.6690e-06_rb,1.0255e-05_rb,1.1493e-05_rb,1.2371e-05_rb/)
      kbo(:,57, 4) = (/ &
     &8.8593e-06_rb,6.9758e-06_rb,8.0033e-06_rb,9.3145e-06_rb,1.0058e-05_rb/)
      kbo(:,58, 4) = (/ &
     &8.9813e-06_rb,6.2718e-06_rb,5.9506e-06_rb,7.1169e-06_rb,8.0866e-06_rb/)
      kbo(:,59, 4) = (/ &
     &8.3077e-06_rb,5.6478e-06_rb,4.7582e-06_rb,5.5908e-06_rb,6.6051e-06_rb/)
      kbo(:,13, 5) = (/ &
     &8.1443e-02_rb,7.8686e-02_rb,7.8076e-02_rb,7.8298e-02_rb,8.0221e-02_rb/)
      kbo(:,14, 5) = (/ &
     &6.8138e-02_rb,6.7426e-02_rb,6.7266e-02_rb,6.8778e-02_rb,7.0029e-02_rb/)
      kbo(:,15, 5) = (/ &
     &5.7566e-02_rb,5.8181e-02_rb,6.0037e-02_rb,6.1077e-02_rb,6.2170e-02_rb/)
      kbo(:,16, 5) = (/ &
     &4.9602e-02_rb,5.1862e-02_rb,5.2830e-02_rb,5.3796e-02_rb,5.4837e-02_rb/)
      kbo(:,17, 5) = (/ &
     &4.4316e-02_rb,4.5504e-02_rb,4.6554e-02_rb,4.7427e-02_rb,4.7730e-02_rb/)
      kbo(:,18, 5) = (/ &
     &3.9127e-02_rb,4.0270e-02_rb,4.1019e-02_rb,4.1264e-02_rb,4.1372e-02_rb/)
      kbo(:,19, 5) = (/ &
     &3.4464e-02_rb,3.5381e-02_rb,3.5731e-02_rb,3.5860e-02_rb,3.6031e-02_rb/)
      kbo(:,20, 5) = (/ &
     &2.9866e-02_rb,3.0433e-02_rb,3.0697e-02_rb,3.0719e-02_rb,3.0767e-02_rb/)
      kbo(:,21, 5) = (/ &
     &2.5526e-02_rb,2.5898e-02_rb,2.6044e-02_rb,2.5989e-02_rb,2.5913e-02_rb/)
      kbo(:,22, 5) = (/ &
     &2.1617e-02_rb,2.1821e-02_rb,2.1825e-02_rb,2.1764e-02_rb,2.1636e-02_rb/)
      kbo(:,23, 5) = (/ &
     &1.8178e-02_rb,1.8256e-02_rb,1.8221e-02_rb,1.8126e-02_rb,1.8025e-02_rb/)
      kbo(:,24, 5) = (/ &
     &1.5221e-02_rb,1.5229e-02_rb,1.5185e-02_rb,1.5079e-02_rb,1.4988e-02_rb/)
      kbo(:,25, 5) = (/ &
     &1.2672e-02_rb,1.2670e-02_rb,1.2609e-02_rb,1.2518e-02_rb,1.2439e-02_rb/)
      kbo(:,26, 5) = (/ &
     &1.0521e-02_rb,1.0503e-02_rb,1.0444e-02_rb,1.0375e-02_rb,1.0305e-02_rb/)
      kbo(:,27, 5) = (/ &
     &8.6769e-03_rb,8.6518e-03_rb,8.6102e-03_rb,8.5574e-03_rb,8.4970e-03_rb/)
      kbo(:,28, 5) = (/ &
     &7.1348e-03_rb,7.1122e-03_rb,7.0812e-03_rb,7.0380e-03_rb,6.9899e-03_rb/)
      kbo(:,29, 5) = (/ &
     &5.8206e-03_rb,5.8023e-03_rb,5.7788e-03_rb,5.7535e-03_rb,5.7174e-03_rb/)
      kbo(:,30, 5) = (/ &
     &4.7381e-03_rb,4.7263e-03_rb,4.7093e-03_rb,4.6856e-03_rb,4.6651e-03_rb/)
      kbo(:,31, 5) = (/ &
     &3.8310e-03_rb,3.8249e-03_rb,3.8179e-03_rb,3.8007e-03_rb,3.7880e-03_rb/)
      kbo(:,32, 5) = (/ &
     &3.1058e-03_rb,3.1004e-03_rb,3.0969e-03_rb,3.0876e-03_rb,3.0842e-03_rb/)
      kbo(:,33, 5) = (/ &
     &2.5170e-03_rb,2.5165e-03_rb,2.5146e-03_rb,2.5120e-03_rb,2.5162e-03_rb/)
      kbo(:,34, 5) = (/ &
     &2.0509e-03_rb,2.0525e-03_rb,2.0528e-03_rb,2.0549e-03_rb,2.0638e-03_rb/)
      kbo(:,35, 5) = (/ &
     &1.6738e-03_rb,1.6767e-03_rb,1.6787e-03_rb,1.6855e-03_rb,1.6962e-03_rb/)
      kbo(:,36, 5) = (/ &
     &1.3688e-03_rb,1.3726e-03_rb,1.3769e-03_rb,1.3854e-03_rb,1.3974e-03_rb/)
      kbo(:,37, 5) = (/ &
     &1.1244e-03_rb,1.1287e-03_rb,1.1344e-03_rb,1.1436e-03_rb,1.1555e-03_rb/)
      kbo(:,38, 5) = (/ &
     &9.2354e-04_rb,9.2789e-04_rb,9.3393e-04_rb,9.4440e-04_rb,9.5553e-04_rb/)
      kbo(:,39, 5) = (/ &
     &7.5825e-04_rb,7.6213e-04_rb,7.6943e-04_rb,7.7957e-04_rb,7.9082e-04_rb/)
      kbo(:,40, 5) = (/ &
     &6.2483e-04_rb,6.2933e-04_rb,6.3674e-04_rb,6.4631e-04_rb,6.5698e-04_rb/)
      kbo(:,41, 5) = (/ &
     &5.1520e-04_rb,5.2035e-04_rb,5.2751e-04_rb,5.3662e-04_rb,5.4646e-04_rb/)
      kbo(:,42, 5) = (/ &
     &4.2480e-04_rb,4.3031e-04_rb,4.3691e-04_rb,4.4575e-04_rb,4.5526e-04_rb/)
      kbo(:,43, 5) = (/ &
     &3.5001e-04_rb,3.5557e-04_rb,3.6220e-04_rb,3.7053e-04_rb,3.7981e-04_rb/)
      kbo(:,44, 5) = (/ &
     &2.8750e-04_rb,2.9363e-04_rb,3.0042e-04_rb,3.0784e-04_rb,3.1671e-04_rb/)
      kbo(:,45, 5) = (/ &
     &2.3558e-04_rb,2.4233e-04_rb,2.4882e-04_rb,2.5588e-04_rb,2.6409e-04_rb/)
      kbo(:,46, 5) = (/ &
     &1.9252e-04_rb,1.9920e-04_rb,2.0547e-04_rb,2.1209e-04_rb,2.1984e-04_rb/)
      kbo(:,47, 5) = (/ &
     &1.5646e-04_rb,1.6321e-04_rb,1.6931e-04_rb,1.7555e-04_rb,1.8250e-04_rb/)
      kbo(:,48, 5) = (/ &
     &1.2570e-04_rb,1.3295e-04_rb,1.3900e-04_rb,1.4479e-04_rb,1.5118e-04_rb/)
      kbo(:,49, 5) = (/ &
     &1.0046e-04_rb,1.0778e-04_rb,1.1356e-04_rb,1.1908e-04_rb,1.2499e-04_rb/)
      kbo(:,50, 5) = (/ &
     &8.0435e-05_rb,8.6489e-05_rb,9.2769e-05_rb,9.7911e-05_rb,1.0325e-04_rb/)
      kbo(:,51, 5) = (/ &
     &6.3458e-05_rb,6.9460e-05_rb,7.5498e-05_rb,8.0303e-05_rb,8.5194e-05_rb/)
      kbo(:,52, 5) = (/ &
     &4.9865e-05_rb,5.5730e-05_rb,6.0501e-05_rb,6.5586e-05_rb,7.0113e-05_rb/)
      kbo(:,53, 5) = (/ &
     &3.9952e-05_rb,4.3532e-05_rb,4.8481e-05_rb,5.3147e-05_rb,5.7399e-05_rb/)
      kbo(:,54, 5) = (/ &
     &3.2880e-05_rb,3.4587e-05_rb,3.8794e-05_rb,4.2722e-05_rb,4.7074e-05_rb/)
      kbo(:,55, 5) = (/ &
     &2.7120e-05_rb,2.7717e-05_rb,3.0553e-05_rb,3.4428e-05_rb,3.8350e-05_rb/)
      kbo(:,56, 5) = (/ &
     &2.2127e-05_rb,2.2850e-05_rb,2.4431e-05_rb,2.7545e-05_rb,3.0669e-05_rb/)
      kbo(:,57, 5) = (/ &
     &1.8252e-05_rb,1.8866e-05_rb,1.9461e-05_rb,2.1577e-05_rb,2.4721e-05_rb/)
      kbo(:,58, 5) = (/ &
     &1.4337e-05_rb,1.5384e-05_rb,1.6084e-05_rb,1.7424e-05_rb,1.9750e-05_rb/)
      kbo(:,59, 5) = (/ &
     &1.1278e-05_rb,1.2651e-05_rb,1.3368e-05_rb,1.4398e-05_rb,1.6235e-05_rb/)
      kbo(:,13, 6) = (/ &
     &1.8817e-01_rb,1.9206e-01_rb,1.9341e-01_rb,1.9316e-01_rb,1.9216e-01_rb/)
      kbo(:,14, 6) = (/ &
     &1.5839e-01_rb,1.6013e-01_rb,1.6098e-01_rb,1.6083e-01_rb,1.6026e-01_rb/)
      kbo(:,15, 6) = (/ &
     &1.3908e-01_rb,1.4022e-01_rb,1.4029e-01_rb,1.3949e-01_rb,1.3862e-01_rb/)
      kbo(:,16, 6) = (/ &
     &1.2016e-01_rb,1.2041e-01_rb,1.1995e-01_rb,1.1931e-01_rb,1.1861e-01_rb/)
      kbo(:,17, 6) = (/ &
     &1.0302e-01_rb,1.0297e-01_rb,1.0263e-01_rb,1.0230e-01_rb,1.0181e-01_rb/)
      kbo(:,18, 6) = (/ &
     &8.8231e-02_rb,8.8161e-02_rb,8.7913e-02_rb,8.7602e-02_rb,8.7188e-02_rb/)
      kbo(:,19, 6) = (/ &
     &7.6358e-02_rb,7.6259e-02_rb,7.5975e-02_rb,7.5746e-02_rb,7.5236e-02_rb/)
      kbo(:,20, 6) = (/ &
     &6.5716e-02_rb,6.5541e-02_rb,6.5322e-02_rb,6.5232e-02_rb,6.4993e-02_rb/)
      kbo(:,21, 6) = (/ &
     &5.5986e-02_rb,5.5897e-02_rb,5.5927e-02_rb,5.6065e-02_rb,5.6062e-02_rb/)
      kbo(:,22, 6) = (/ &
     &4.7367e-02_rb,4.7462e-02_rb,4.7736e-02_rb,4.7820e-02_rb,4.7753e-02_rb/)
      kbo(:,23, 6) = (/ &
     &4.0099e-02_rb,4.0361e-02_rb,4.0481e-02_rb,4.0442e-02_rb,4.0254e-02_rb/)
      kbo(:,24, 6) = (/ &
     &3.3998e-02_rb,3.4135e-02_rb,3.4123e-02_rb,3.4025e-02_rb,3.3829e-02_rb/)
      kbo(:,25, 6) = (/ &
     &2.8658e-02_rb,2.8658e-02_rb,2.8582e-02_rb,2.8475e-02_rb,2.8305e-02_rb/)
      kbo(:,26, 6) = (/ &
     &2.3955e-02_rb,2.3909e-02_rb,2.3836e-02_rb,2.3748e-02_rb,2.3606e-02_rb/)
      kbo(:,27, 6) = (/ &
     &1.9889e-02_rb,1.9852e-02_rb,1.9780e-02_rb,1.9709e-02_rb,1.9615e-02_rb/)
      kbo(:,28, 6) = (/ &
     &1.6444e-02_rb,1.6406e-02_rb,1.6372e-02_rb,1.6318e-02_rb,1.6245e-02_rb/)
      kbo(:,29, 6) = (/ &
     &1.3491e-02_rb,1.3473e-02_rb,1.3456e-02_rb,1.3404e-02_rb,1.3356e-02_rb/)
      kbo(:,30, 6) = (/ &
     &1.1049e-02_rb,1.1039e-02_rb,1.1036e-02_rb,1.1011e-02_rb,1.0973e-02_rb/)
      kbo(:,31, 6) = (/ &
     &9.0143e-03_rb,9.0192e-03_rb,9.0194e-03_rb,9.0134e-03_rb,9.0031e-03_rb/)
      kbo(:,32, 6) = (/ &
     &7.3665e-03_rb,7.3853e-03_rb,7.3885e-03_rb,7.3955e-03_rb,7.4067e-03_rb/)
      kbo(:,33, 6) = (/ &
     &6.0271e-03_rb,6.0462e-03_rb,6.0622e-03_rb,6.0836e-03_rb,6.1045e-03_rb/)
      kbo(:,34, 6) = (/ &
     &4.9546e-03_rb,4.9811e-03_rb,5.0047e-03_rb,5.0339e-03_rb,5.0627e-03_rb/)
      kbo(:,35, 6) = (/ &
     &4.0822e-03_rb,4.1121e-03_rb,4.1424e-03_rb,4.1730e-03_rb,4.2061e-03_rb/)
      kbo(:,36, 6) = (/ &
     &3.3681e-03_rb,3.3993e-03_rb,3.4334e-03_rb,3.4635e-03_rb,3.4987e-03_rb/)
      kbo(:,37, 6) = (/ &
     &2.7875e-03_rb,2.8195e-03_rb,2.8505e-03_rb,2.8836e-03_rb,2.9180e-03_rb/)
      kbo(:,38, 6) = (/ &
     &2.3080e-03_rb,2.3382e-03_rb,2.3688e-03_rb,2.3992e-03_rb,2.4337e-03_rb/)
      kbo(:,39, 6) = (/ &
     &1.9086e-03_rb,1.9391e-03_rb,1.9678e-03_rb,1.9967e-03_rb,2.0294e-03_rb/)
      kbo(:,40, 6) = (/ &
     &1.5841e-03_rb,1.6128e-03_rb,1.6390e-03_rb,1.6667e-03_rb,1.6980e-03_rb/)
      kbo(:,41, 6) = (/ &
     &1.3139e-03_rb,1.3408e-03_rb,1.3651e-03_rb,1.3913e-03_rb,1.4213e-03_rb/)
      kbo(:,42, 6) = (/ &
     &1.0897e-03_rb,1.1135e-03_rb,1.1369e-03_rb,1.1616e-03_rb,1.1887e-03_rb/)
      kbo(:,43, 6) = (/ &
     &9.0311e-04_rb,9.2481e-04_rb,9.4572e-04_rb,9.6914e-04_rb,9.9399e-04_rb/)
      kbo(:,44, 6) = (/ &
     &7.4776e-04_rb,7.6718e-04_rb,7.8637e-04_rb,8.0787e-04_rb,8.3044e-04_rb/)
      kbo(:,45, 6) = (/ &
     &6.1863e-04_rb,6.3626e-04_rb,6.5392e-04_rb,6.7285e-04_rb,6.9425e-04_rb/)
      kbo(:,46, 6) = (/ &
     &5.1188e-04_rb,5.2759e-04_rb,5.4354e-04_rb,5.6054e-04_rb,5.8050e-04_rb/)
      kbo(:,47, 6) = (/ &
     &4.2255e-04_rb,4.3715e-04_rb,4.5173e-04_rb,4.6695e-04_rb,4.8462e-04_rb/)
      kbo(:,48, 6) = (/ &
     &3.4819e-04_rb,3.6156e-04_rb,3.7458e-04_rb,3.8864e-04_rb,4.0475e-04_rb/)
      kbo(:,49, 6) = (/ &
     &2.8602e-04_rb,2.9814e-04_rb,3.1015e-04_rb,3.2283e-04_rb,3.3769e-04_rb/)
      kbo(:,50, 6) = (/ &
     &2.3475e-04_rb,2.4603e-04_rb,2.5676e-04_rb,2.6851e-04_rb,2.8207e-04_rb/)
      kbo(:,51, 6) = (/ &
     &1.9267e-04_rb,2.0293e-04_rb,2.1257e-04_rb,2.2327e-04_rb,2.3584e-04_rb/)
      kbo(:,52, 6) = (/ &
     &1.5736e-04_rb,1.6664e-04_rb,1.7558e-04_rb,1.8519e-04_rb,1.9650e-04_rb/)
      kbo(:,53, 6) = (/ &
     &1.2741e-04_rb,1.3627e-04_rb,1.4450e-04_rb,1.5309e-04_rb,1.6331e-04_rb/)
      kbo(:,54, 6) = (/ &
     &1.0368e-04_rb,1.1186e-04_rb,1.1942e-04_rb,1.2720e-04_rb,1.3642e-04_rb/)
      kbo(:,55, 6) = (/ &
     &8.4018e-05_rb,9.1900e-05_rb,9.8814e-05_rb,1.0595e-04_rb,1.1419e-04_rb/)
      kbo(:,56, 6) = (/ &
     &6.7644e-05_rb,7.5107e-05_rb,8.1537e-05_rb,8.8130e-05_rb,9.5553e-05_rb/)
      kbo(:,57, 6) = (/ &
     &5.3052e-05_rb,6.0800e-05_rb,6.7049e-05_rb,7.3153e-05_rb,7.9865e-05_rb/)
      kbo(:,58, 6) = (/ &
     &4.1297e-05_rb,4.9020e-05_rb,5.4922e-05_rb,6.0589e-05_rb,6.6753e-05_rb/)
      kbo(:,59, 6) = (/ &
     &3.4006e-05_rb,4.0647e-05_rb,4.6127e-05_rb,5.1418e-05_rb,5.7293e-05_rb/)
      kbo(:,13, 7) = (/ &
     &5.2204e-01_rb,5.1758e-01_rb,5.1182e-01_rb,5.0530e-01_rb,4.9787e-01_rb/)
      kbo(:,14, 7) = (/ &
     &4.3225e-01_rb,4.2797e-01_rb,4.2283e-01_rb,4.1695e-01_rb,4.1054e-01_rb/)
      kbo(:,15, 7) = (/ &
     &3.5438e-01_rb,3.5034e-01_rb,3.4544e-01_rb,3.4060e-01_rb,3.3555e-01_rb/)
      kbo(:,16, 7) = (/ &
     &3.0255e-01_rb,2.9825e-01_rb,2.9421e-01_rb,2.8975e-01_rb,2.8561e-01_rb/)
      kbo(:,17, 7) = (/ &
     &2.5944e-01_rb,2.5596e-01_rb,2.5201e-01_rb,2.4828e-01_rb,2.4466e-01_rb/)
      kbo(:,18, 7) = (/ &
     &2.2110e-01_rb,2.1760e-01_rb,2.1435e-01_rb,2.1143e-01_rb,2.0904e-01_rb/)
      kbo(:,19, 7) = (/ &
     &1.8864e-01_rb,1.8589e-01_rb,1.8343e-01_rb,1.8143e-01_rb,1.7990e-01_rb/)
      kbo(:,20, 7) = (/ &
     &1.6121e-01_rb,1.5932e-01_rb,1.5784e-01_rb,1.5634e-01_rb,1.5506e-01_rb/)
      kbo(:,21, 7) = (/ &
     &1.3878e-01_rb,1.3751e-01_rb,1.3632e-01_rb,1.3482e-01_rb,1.3362e-01_rb/)
      kbo(:,22, 7) = (/ &
     &1.1962e-01_rb,1.1845e-01_rb,1.1744e-01_rb,1.1632e-01_rb,1.1539e-01_rb/)
      kbo(:,23, 7) = (/ &
     &1.0245e-01_rb,1.0173e-01_rb,1.0097e-01_rb,1.0038e-01_rb,9.9933e-02_rb/)
      kbo(:,24, 7) = (/ &
     &8.7666e-02_rb,8.7158e-02_rb,8.6968e-02_rb,8.6971e-02_rb,8.7255e-02_rb/)
      kbo(:,25, 7) = (/ &
     &7.4640e-02_rb,7.4734e-02_rb,7.5053e-02_rb,7.5269e-02_rb,7.5390e-02_rb/)
      kbo(:,26, 7) = (/ &
     &6.3735e-02_rb,6.4061e-02_rb,6.4333e-02_rb,6.4421e-02_rb,6.4368e-02_rb/)
      kbo(:,27, 7) = (/ &
     &5.4126e-02_rb,5.4429e-02_rb,5.4564e-02_rb,5.4521e-02_rb,5.4525e-02_rb/)
      kbo(:,28, 7) = (/ &
     &4.5724e-02_rb,4.5897e-02_rb,4.5948e-02_rb,4.5981e-02_rb,4.6016e-02_rb/)
      kbo(:,29, 7) = (/ &
     &3.8172e-02_rb,3.8308e-02_rb,3.8382e-02_rb,3.8489e-02_rb,3.8585e-02_rb/)
      kbo(:,30, 7) = (/ &
     &3.1718e-02_rb,3.1883e-02_rb,3.2020e-02_rb,3.2173e-02_rb,3.2316e-02_rb/)
      kbo(:,31, 7) = (/ &
     &2.6252e-02_rb,2.6436e-02_rb,2.6597e-02_rb,2.6780e-02_rb,2.6950e-02_rb/)
      kbo(:,32, 7) = (/ &
     &2.1754e-02_rb,2.1941e-02_rb,2.2153e-02_rb,2.2326e-02_rb,2.2517e-02_rb/)
      kbo(:,33, 7) = (/ &
     &1.8085e-02_rb,1.8277e-02_rb,1.8461e-02_rb,1.8652e-02_rb,1.8849e-02_rb/)
      kbo(:,34, 7) = (/ &
     &1.5122e-02_rb,1.5298e-02_rb,1.5478e-02_rb,1.5652e-02_rb,1.5844e-02_rb/)
      kbo(:,35, 7) = (/ &
     &1.2625e-02_rb,1.2802e-02_rb,1.2974e-02_rb,1.3152e-02_rb,1.3333e-02_rb/)
      kbo(:,36, 7) = (/ &
     &1.0557e-02_rb,1.0724e-02_rb,1.0878e-02_rb,1.1051e-02_rb,1.1225e-02_rb/)
      kbo(:,37, 7) = (/ &
     &8.8496e-03_rb,9.0081e-03_rb,9.1627e-03_rb,9.3269e-03_rb,9.4871e-03_rb/)
      kbo(:,38, 7) = (/ &
     &7.4071e-03_rb,7.5543e-03_rb,7.6993e-03_rb,7.8541e-03_rb,8.0170e-03_rb/)
      kbo(:,39, 7) = (/ &
     &6.1854e-03_rb,6.3247e-03_rb,6.4677e-03_rb,6.6094e-03_rb,6.7526e-03_rb/)
      kbo(:,40, 7) = (/ &
     &5.1922e-03_rb,5.3234e-03_rb,5.4554e-03_rb,5.5916e-03_rb,5.7266e-03_rb/)
      kbo(:,41, 7) = (/ &
     &4.3585e-03_rb,4.4819e-03_rb,4.6086e-03_rb,4.7343e-03_rb,4.8561e-03_rb/)
      kbo(:,42, 7) = (/ &
     &3.6533e-03_rb,3.7711e-03_rb,3.8861e-03_rb,3.9990e-03_rb,4.1168e-03_rb/)
      kbo(:,43, 7) = (/ &
     &3.0584e-03_rb,3.1667e-03_rb,3.2727e-03_rb,3.3777e-03_rb,3.4881e-03_rb/)
      kbo(:,44, 7) = (/ &
     &2.5534e-03_rb,2.6526e-03_rb,2.7492e-03_rb,2.8491e-03_rb,2.9548e-03_rb/)
      kbo(:,45, 7) = (/ &
     &2.1284e-03_rb,2.2160e-03_rb,2.3074e-03_rb,2.3986e-03_rb,2.4950e-03_rb/)
      kbo(:,46, 7) = (/ &
     &1.7695e-03_rb,1.8482e-03_rb,1.9316e-03_rb,2.0146e-03_rb,2.1010e-03_rb/)
      kbo(:,47, 7) = (/ &
     &1.4673e-03_rb,1.5398e-03_rb,1.6138e-03_rb,1.6894e-03_rb,1.7692e-03_rb/)
      kbo(:,48, 7) = (/ &
     &1.2142e-03_rb,1.2798e-03_rb,1.3450e-03_rb,1.4129e-03_rb,1.4833e-03_rb/)
      kbo(:,49, 7) = (/ &
     &1.0007e-03_rb,1.0578e-03_rb,1.1143e-03_rb,1.1757e-03_rb,1.2387e-03_rb/)
      kbo(:,50, 7) = (/ &
     &8.2795e-04_rb,8.7794e-04_rb,9.2798e-04_rb,9.8346e-04_rb,1.0399e-03_rb/)
      kbo(:,51, 7) = (/ &
     &6.8604e-04_rb,7.3011e-04_rb,7.7427e-04_rb,8.2321e-04_rb,8.7374e-04_rb/)
      kbo(:,52, 7) = (/ &
     &5.6660e-04_rb,6.0603e-04_rb,6.4555e-04_rb,6.8763e-04_rb,7.3282e-04_rb/)
      kbo(:,53, 7) = (/ &
     &4.6575e-04_rb,5.0073e-04_rb,5.3612e-04_rb,5.7296e-04_rb,6.1258e-04_rb/)
      kbo(:,54, 7) = (/ &
     &3.8679e-04_rb,4.1843e-04_rb,4.5022e-04_rb,4.8415e-04_rb,5.2013e-04_rb/)
      kbo(:,55, 7) = (/ &
     &3.2310e-04_rb,3.5117e-04_rb,3.8038e-04_rb,4.1226e-04_rb,4.4542e-04_rb/)
      kbo(:,56, 7) = (/ &
     &2.6895e-04_rb,2.9441e-04_rb,3.2135e-04_rb,3.5045e-04_rb,3.8084e-04_rb/)
      kbo(:,57, 7) = (/ &
     &2.2346e-04_rb,2.4644e-04_rb,2.7077e-04_rb,2.9691e-04_rb,3.2529e-04_rb/)
      kbo(:,58, 7) = (/ &
     &1.8566e-04_rb,2.0641e-04_rb,2.2834e-04_rb,2.5256e-04_rb,2.7894e-04_rb/)
      kbo(:,59, 7) = (/ &
     &1.5949e-04_rb,1.7859e-04_rb,1.9903e-04_rb,2.2176e-04_rb,2.4718e-04_rb/)
      kbo(:,13, 8) = (/ &
     &1.7932e+00_rb,1.7598e+00_rb,1.7272e+00_rb,1.6957e+00_rb,1.6643e+00_rb/)
      kbo(:,14, 8) = (/ &
     &1.5024e+00_rb,1.4748e+00_rb,1.4482e+00_rb,1.4213e+00_rb,1.3943e+00_rb/)
      kbo(:,15, 8) = (/ &
     &1.2493e+00_rb,1.2269e+00_rb,1.2042e+00_rb,1.1821e+00_rb,1.1593e+00_rb/)
      kbo(:,16, 8) = (/ &
     &1.0236e+00_rb,1.0050e+00_rb,9.8683e-01_rb,9.6754e-01_rb,9.4805e-01_rb/)
      kbo(:,17, 8) = (/ &
     &8.3764e-01_rb,8.2167e-01_rb,8.0558e-01_rb,7.8848e-01_rb,7.7231e-01_rb/)
      kbo(:,18, 8) = (/ &
     &7.0049e-01_rb,6.8715e-01_rb,6.7310e-01_rb,6.5885e-01_rb,6.4482e-01_rb/)
      kbo(:,19, 8) = (/ &
     &6.0067e-01_rb,5.8909e-01_rb,5.7714e-01_rb,5.6481e-01_rb,5.5329e-01_rb/)
      kbo(:,20, 8) = (/ &
     &5.1436e-01_rb,5.0417e-01_rb,4.9351e-01_rb,4.8348e-01_rb,4.7495e-01_rb/)
      kbo(:,21, 8) = (/ &
     &4.3696e-01_rb,4.2847e-01_rb,4.2014e-01_rb,4.1292e-01_rb,4.0709e-01_rb/)
      kbo(:,22, 8) = (/ &
     &3.7052e-01_rb,3.6397e-01_rb,3.5765e-01_rb,3.5323e-01_rb,3.4919e-01_rb/)
      kbo(:,23, 8) = (/ &
     &3.1534e-01_rb,3.1049e-01_rb,3.0630e-01_rb,3.0316e-01_rb,3.0035e-01_rb/)
      kbo(:,24, 8) = (/ &
     &2.7112e-01_rb,2.6740e-01_rb,2.6459e-01_rb,2.6223e-01_rb,2.5988e-01_rb/)
      kbo(:,25, 8) = (/ &
     &2.3380e-01_rb,2.3133e-01_rb,2.2943e-01_rb,2.2772e-01_rb,2.2653e-01_rb/)
      kbo(:,26, 8) = (/ &
     &2.0236e-01_rb,2.0088e-01_rb,1.9954e-01_rb,1.9878e-01_rb,1.9870e-01_rb/)
      kbo(:,27, 8) = (/ &
     &1.7470e-01_rb,1.7394e-01_rb,1.7356e-01_rb,1.7393e-01_rb,1.7411e-01_rb/)
      kbo(:,28, 8) = (/ &
     &1.5024e-01_rb,1.5047e-01_rb,1.5098e-01_rb,1.5164e-01_rb,1.5243e-01_rb/)
      kbo(:,29, 8) = (/ &
     &1.2816e-01_rb,1.2887e-01_rb,1.2996e-01_rb,1.3066e-01_rb,1.3197e-01_rb/)
      kbo(:,30, 8) = (/ &
     &1.0887e-01_rb,1.0998e-01_rb,1.1099e-01_rb,1.1227e-01_rb,1.1365e-01_rb/)
      kbo(:,31, 8) = (/ &
     &9.1656e-02_rb,9.2754e-02_rb,9.3840e-02_rb,9.5163e-02_rb,9.6499e-02_rb/)
      kbo(:,32, 8) = (/ &
     &7.7165e-02_rb,7.8228e-02_rb,7.9327e-02_rb,8.0584e-02_rb,8.1883e-02_rb/)
      kbo(:,33, 8) = (/ &
     &6.4796e-02_rb,6.5863e-02_rb,6.6915e-02_rb,6.7986e-02_rb,6.9105e-02_rb/)
      kbo(:,34, 8) = (/ &
     &5.4901e-02_rb,5.5964e-02_rb,5.6945e-02_rb,5.8000e-02_rb,5.9045e-02_rb/)
      kbo(:,35, 8) = (/ &
     &4.6607e-02_rb,4.7594e-02_rb,4.8499e-02_rb,4.9452e-02_rb,5.0386e-02_rb/)
      kbo(:,36, 8) = (/ &
     &3.9577e-02_rb,4.0492e-02_rb,4.1341e-02_rb,4.2242e-02_rb,4.3090e-02_rb/)
      kbo(:,37, 8) = (/ &
     &3.3790e-02_rb,3.4662e-02_rb,3.5486e-02_rb,3.6314e-02_rb,3.7168e-02_rb/)
      kbo(:,38, 8) = (/ &
     &2.8754e-02_rb,2.9559e-02_rb,3.0387e-02_rb,3.1137e-02_rb,3.1929e-02_rb/)
      kbo(:,39, 8) = (/ &
     &2.4394e-02_rb,2.5115e-02_rb,2.5838e-02_rb,2.6544e-02_rb,2.7274e-02_rb/)
      kbo(:,40, 8) = (/ &
     &2.0920e-02_rb,2.1659e-02_rb,2.2398e-02_rb,2.3092e-02_rb,2.3784e-02_rb/)
      kbo(:,41, 8) = (/ &
     &1.7960e-02_rb,1.8686e-02_rb,1.9408e-02_rb,2.0118e-02_rb,2.0798e-02_rb/)
      kbo(:,42, 8) = (/ &
     &1.5382e-02_rb,1.6080e-02_rb,1.6764e-02_rb,1.7451e-02_rb,1.8100e-02_rb/)
      kbo(:,43, 8) = (/ &
     &1.3156e-02_rb,1.3822e-02_rb,1.4465e-02_rb,1.5132e-02_rb,1.5767e-02_rb/)
      kbo(:,44, 8) = (/ &
     &1.1236e-02_rb,1.1847e-02_rb,1.2471e-02_rb,1.3074e-02_rb,1.3675e-02_rb/)
      kbo(:,45, 8) = (/ &
     &9.5475e-03_rb,1.0122e-02_rb,1.0686e-02_rb,1.1243e-02_rb,1.1794e-02_rb/)
      kbo(:,46, 8) = (/ &
     &8.0771e-03_rb,8.6018e-03_rb,9.1254e-03_rb,9.6411e-03_rb,1.0156e-02_rb/)
      kbo(:,47, 8) = (/ &
     &6.8185e-03_rb,7.3016e-03_rb,7.7913e-03_rb,8.2730e-03_rb,8.7590e-03_rb/)
      kbo(:,48, 8) = (/ &
     &5.7132e-03_rb,6.1559e-03_rb,6.5990e-03_rb,7.0467e-03_rb,7.5035e-03_rb/)
      kbo(:,49, 8) = (/ &
     &4.7309e-03_rb,5.1282e-03_rb,5.5334e-03_rb,5.9379e-03_rb,6.3413e-03_rb/)
      kbo(:,50, 8) = (/ &
     &3.9545e-03_rb,4.3092e-03_rb,4.6836e-03_rb,5.0555e-03_rb,5.4339e-03_rb/)
      kbo(:,51, 8) = (/ &
     &3.3094e-03_rb,3.6354e-03_rb,3.9810e-03_rb,4.3245e-03_rb,4.6768e-03_rb/)
      kbo(:,52, 8) = (/ &
     &2.7556e-03_rb,3.0488e-03_rb,3.3581e-03_rb,3.6757e-03_rb,3.9978e-03_rb/)
      kbo(:,53, 8) = (/ &
     &2.2741e-03_rb,2.5328e-03_rb,2.8070e-03_rb,3.0901e-03_rb,3.3837e-03_rb/)
      kbo(:,54, 8) = (/ &
     &1.9244e-03_rb,2.1630e-03_rb,2.4199e-03_rb,2.6919e-03_rb,2.9670e-03_rb/)
      kbo(:,55, 8) = (/ &
     &1.6527e-03_rb,1.8750e-03_rb,2.1244e-03_rb,2.3895e-03_rb,2.6621e-03_rb/)
      kbo(:,56, 8) = (/ &
     &1.4187e-03_rb,1.6262e-03_rb,1.8617e-03_rb,2.1120e-03_rb,2.3834e-03_rb/)
      kbo(:,57, 8) = (/ &
     &1.2123e-03_rb,1.4102e-03_rb,1.6308e-03_rb,1.8701e-03_rb,2.1361e-03_rb/)
      kbo(:,58, 8) = (/ &
     &1.0428e-03_rb,1.2269e-03_rb,1.4357e-03_rb,1.6692e-03_rb,1.9342e-03_rb/)
      kbo(:,59, 8) = (/ &
     &9.5774e-04_rb,1.1485e-03_rb,1.3717e-03_rb,1.6313e-03_rb,1.9270e-03_rb/)
      kbo(:,13, 9) = (/ &
     &1.0743e+01_rb,1.0581e+01_rb,1.0416e+01_rb,1.0238e+01_rb,1.0057e+01_rb/)
      kbo(:,14, 9) = (/ &
     &9.3266e+00_rb,9.1890e+00_rb,9.0401e+00_rb,8.8785e+00_rb,8.7093e+00_rb/)
      kbo(:,15, 9) = (/ &
     &8.0329e+00_rb,7.9105e+00_rb,7.7721e+00_rb,7.6225e+00_rb,7.4769e+00_rb/)
      kbo(:,16, 9) = (/ &
     &6.8686e+00_rb,6.7530e+00_rb,6.6275e+00_rb,6.5036e+00_rb,6.3858e+00_rb/)
      kbo(:,17, 9) = (/ &
     &5.8249e+00_rb,5.7232e+00_rb,5.6209e+00_rb,5.5225e+00_rb,5.4281e+00_rb/)
      kbo(:,18, 9) = (/ &
     &4.8929e+00_rb,4.8094e+00_rb,4.7279e+00_rb,4.6483e+00_rb,4.5724e+00_rb/)
      kbo(:,19, 9) = (/ &
     &4.0706e+00_rb,4.0042e+00_rb,3.9380e+00_rb,3.8751e+00_rb,3.8145e+00_rb/)
      kbo(:,20, 9) = (/ &
     &3.3806e+00_rb,3.3267e+00_rb,3.2749e+00_rb,3.2247e+00_rb,3.1746e+00_rb/)
      kbo(:,21, 9) = (/ &
     &2.8074e+00_rb,2.7644e+00_rb,2.7235e+00_rb,2.6821e+00_rb,2.6407e+00_rb/)
      kbo(:,22, 9) = (/ &
     &2.3283e+00_rb,2.2955e+00_rb,2.2616e+00_rb,2.2266e+00_rb,2.1957e+00_rb/)
      kbo(:,23, 9) = (/ &
     &1.9321e+00_rb,1.9042e+00_rb,1.8758e+00_rb,1.8494e+00_rb,1.8261e+00_rb/)
      kbo(:,24, 9) = (/ &
     &1.6234e+00_rb,1.5933e+00_rb,1.5660e+00_rb,1.5420e+00_rb,1.5232e+00_rb/)
      kbo(:,25, 9) = (/ &
     &1.3980e+00_rb,1.3704e+00_rb,1.3436e+00_rb,1.3210e+00_rb,1.3008e+00_rb/)
      kbo(:,26, 9) = (/ &
     &1.2106e+00_rb,1.1883e+00_rb,1.1672e+00_rb,1.1488e+00_rb,1.1330e+00_rb/)
      kbo(:,27, 9) = (/ &
     &1.0466e+00_rb,1.0297e+00_rb,1.0139e+00_rb,1.0016e+00_rb,9.9022e-01_rb/)
      kbo(:,28, 9) = (/ &
     &9.0567e-01_rb,8.9244e-01_rb,8.8275e-01_rb,8.7361e-01_rb,8.6722e-01_rb/)
      kbo(:,29, 9) = (/ &
     &7.7743e-01_rb,7.6923e-01_rb,7.6234e-01_rb,7.5840e-01_rb,7.5702e-01_rb/)
      kbo(:,30, 9) = (/ &
     &6.6693e-01_rb,6.6196e-01_rb,6.5810e-01_rb,6.5663e-01_rb,6.5785e-01_rb/)
      kbo(:,31, 9) = (/ &
     &5.6777e-01_rb,5.6527e-01_rb,5.6420e-01_rb,5.6593e-01_rb,5.6662e-01_rb/)
      kbo(:,32, 9) = (/ &
     &4.8384e-01_rb,4.8324e-01_rb,4.8562e-01_rb,4.8803e-01_rb,4.8996e-01_rb/)
      kbo(:,33, 9) = (/ &
     &4.1137e-01_rb,4.1406e-01_rb,4.1751e-01_rb,4.2081e-01_rb,4.2550e-01_rb/)
      kbo(:,34, 9) = (/ &
     &3.5416e-01_rb,3.5806e-01_rb,3.6166e-01_rb,3.6664e-01_rb,3.7277e-01_rb/)
      kbo(:,35, 9) = (/ &
     &3.0554e-01_rb,3.0948e-01_rb,3.1424e-01_rb,3.1976e-01_rb,3.2676e-01_rb/)
      kbo(:,36, 9) = (/ &
     &2.6418e-01_rb,2.6871e-01_rb,2.7383e-01_rb,2.8027e-01_rb,2.8707e-01_rb/)
      kbo(:,37, 9) = (/ &
     &2.2969e-01_rb,2.3421e-01_rb,2.3975e-01_rb,2.4624e-01_rb,2.5310e-01_rb/)
      kbo(:,38, 9) = (/ &
     &1.9950e-01_rb,2.0421e-01_rb,2.0978e-01_rb,2.1651e-01_rb,2.2302e-01_rb/)
      kbo(:,39, 9) = (/ &
     &1.7282e-01_rb,1.7783e-01_rb,1.8337e-01_rb,1.9005e-01_rb,1.9644e-01_rb/)
      kbo(:,40, 9) = (/ &
     &1.5139e-01_rb,1.5623e-01_rb,1.6190e-01_rb,1.6820e-01_rb,1.7447e-01_rb/)
      kbo(:,41, 9) = (/ &
     &1.3264e-01_rb,1.3745e-01_rb,1.4301e-01_rb,1.4932e-01_rb,1.5546e-01_rb/)
      kbo(:,42, 9) = (/ &
     &1.1607e-01_rb,1.2085e-01_rb,1.2625e-01_rb,1.3247e-01_rb,1.3847e-01_rb/)
      kbo(:,43, 9) = (/ &
     &1.0129e-01_rb,1.0600e-01_rb,1.1131e-01_rb,1.1735e-01_rb,1.2314e-01_rb/)
      kbo(:,44, 9) = (/ &
     &8.8076e-02_rb,9.2672e-02_rb,9.7764e-02_rb,1.0358e-01_rb,1.0942e-01_rb/)
      kbo(:,45, 9) = (/ &
     &7.6267e-02_rb,8.0788e-02_rb,8.5604e-02_rb,9.1124e-02_rb,9.6796e-02_rb/)
      kbo(:,46, 9) = (/ &
     &6.5896e-02_rb,7.0073e-02_rb,7.4628e-02_rb,7.9860e-02_rb,8.5349e-02_rb/)
      kbo(:,47, 9) = (/ &
     &5.6947e-02_rb,6.0912e-02_rb,6.5102e-02_rb,6.9953e-02_rb,7.5324e-02_rb/)
      kbo(:,48, 9) = (/ &
     &4.8993e-02_rb,5.2552e-02_rb,5.6622e-02_rb,6.1050e-02_rb,6.6050e-02_rb/)
      kbo(:,49, 9) = (/ &
     &4.1828e-02_rb,4.5103e-02_rb,4.8726e-02_rb,5.2827e-02_rb,5.7536e-02_rb/)
      kbo(:,50, 9) = (/ &
     &3.6237e-02_rb,3.9173e-02_rb,4.2523e-02_rb,4.6248e-02_rb,5.0679e-02_rb/)
      kbo(:,51, 9) = (/ &
     &3.1483e-02_rb,3.4194e-02_rb,3.7317e-02_rb,4.0792e-02_rb,4.4937e-02_rb/)
      kbo(:,52, 9) = (/ &
     &2.7288e-02_rb,2.9720e-02_rb,3.2635e-02_rb,3.5833e-02_rb,3.9605e-02_rb/)
      kbo(:,53, 9) = (/ &
     &2.3553e-02_rb,2.5752e-02_rb,2.8298e-02_rb,3.1273e-02_rb,3.4805e-02_rb/)
      kbo(:,54, 9) = (/ &
     &2.0927e-02_rb,2.2955e-02_rb,2.5436e-02_rb,2.8294e-02_rb,3.1547e-02_rb/)
      kbo(:,55, 9) = (/ &
     &1.8850e-02_rb,2.0892e-02_rb,2.3245e-02_rb,2.6068e-02_rb,2.9259e-02_rb/)
      kbo(:,56, 9) = (/ &
     &1.7133e-02_rb,1.9134e-02_rb,2.1378e-02_rb,2.4152e-02_rb,2.7278e-02_rb/)
      kbo(:,57, 9) = (/ &
     &1.5614e-02_rb,1.7616e-02_rb,1.9861e-02_rb,2.2450e-02_rb,2.5658e-02_rb/)
      kbo(:,58, 9) = (/ &
     &1.4386e-02_rb,1.6367e-02_rb,1.8612e-02_rb,2.1235e-02_rb,2.4366e-02_rb/)
      kbo(:,59, 9) = (/ &
     &1.4750e-02_rb,1.6838e-02_rb,1.9250e-02_rb,2.2069e-02_rb,2.5653e-02_rb/)
      kbo(:,13,10) = (/ &
     &4.1842e+01_rb,4.1126e+01_rb,4.0474e+01_rb,3.9934e+01_rb,3.9455e+01_rb/)
      kbo(:,14,10) = (/ &
     &3.7817e+01_rb,3.7244e+01_rb,3.6754e+01_rb,3.6387e+01_rb,3.5986e+01_rb/)
      kbo(:,15,10) = (/ &
     &3.3775e+01_rb,3.3359e+01_rb,3.3057e+01_rb,3.2734e+01_rb,3.2346e+01_rb/)
      kbo(:,16,10) = (/ &
     &2.9866e+01_rb,2.9628e+01_rb,2.9352e+01_rb,2.8982e+01_rb,2.8500e+01_rb/)
      kbo(:,17,10) = (/ &
     &2.6256e+01_rb,2.6026e+01_rb,2.5674e+01_rb,2.5248e+01_rb,2.4813e+01_rb/)
      kbo(:,18,10) = (/ &
     &2.2845e+01_rb,2.2539e+01_rb,2.2184e+01_rb,2.1830e+01_rb,2.1506e+01_rb/)
      kbo(:,19,10) = (/ &
     &1.9624e+01_rb,1.9336e+01_rb,1.9054e+01_rb,1.8789e+01_rb,1.8555e+01_rb/)
      kbo(:,20,10) = (/ &
     &1.6714e+01_rb,1.6495e+01_rb,1.6283e+01_rb,1.6088e+01_rb,1.5920e+01_rb/)
      kbo(:,21,10) = (/ &
     &1.4184e+01_rb,1.4023e+01_rb,1.3853e+01_rb,1.3722e+01_rb,1.3615e+01_rb/)
      kbo(:,22,10) = (/ &
     &1.2010e+01_rb,1.1876e+01_rb,1.1769e+01_rb,1.1693e+01_rb,1.1608e+01_rb/)
      kbo(:,23,10) = (/ &
     &1.0140e+01_rb,1.0060e+01_rb,1.0004e+01_rb,9.9480e+00_rb,9.8828e+00_rb/)
      kbo(:,24,10) = (/ &
     &8.3529e+00_rb,8.3955e+00_rb,8.4004e+00_rb,8.3887e+00_rb,8.3750e+00_rb/)
      kbo(:,25,10) = (/ &
     &6.5990e+00_rb,6.6692e+00_rb,6.7250e+00_rb,6.7763e+00_rb,6.8513e+00_rb/)
      kbo(:,26,10) = (/ &
     &5.2218e+00_rb,5.2429e+00_rb,5.2934e+00_rb,5.3672e+00_rb,5.4447e+00_rb/)
      kbo(:,27,10) = (/ &
     &4.4199e+00_rb,4.3564e+00_rb,4.3290e+00_rb,4.3162e+00_rb,4.3878e+00_rb/)
      kbo(:,28,10) = (/ &
     &3.8059e+00_rb,3.7536e+00_rb,3.6821e+00_rb,3.6546e+00_rb,3.6556e+00_rb/)
      kbo(:,29,10) = (/ &
     &3.2943e+00_rb,3.2284e+00_rb,3.1752e+00_rb,3.1401e+00_rb,3.1165e+00_rb/)
      kbo(:,30,10) = (/ &
     &2.8426e+00_rb,2.7881e+00_rb,2.7489e+00_rb,2.7185e+00_rb,2.7182e+00_rb/)
      kbo(:,31,10) = (/ &
     &2.4366e+00_rb,2.4010e+00_rb,2.3877e+00_rb,2.3684e+00_rb,2.4040e+00_rb/)
      kbo(:,32,10) = (/ &
     &2.1031e+00_rb,2.0888e+00_rb,2.0634e+00_rb,2.0885e+00_rb,2.1581e+00_rb/)
      kbo(:,33,10) = (/ &
     &1.8304e+00_rb,1.8104e+00_rb,1.8138e+00_rb,1.8703e+00_rb,1.9665e+00_rb/)
      kbo(:,34,10) = (/ &
     &1.5963e+00_rb,1.5877e+00_rb,1.6209e+00_rb,1.6809e+00_rb,1.8060e+00_rb/)
      kbo(:,35,10) = (/ &
     &1.3950e+00_rb,1.4093e+00_rb,1.4535e+00_rb,1.5372e+00_rb,1.6748e+00_rb/)
      kbo(:,36,10) = (/ &
     &1.2291e+00_rb,1.2517e+00_rb,1.3081e+00_rb,1.4001e+00_rb,1.5466e+00_rb/)
      kbo(:,37,10) = (/ &
     &1.0841e+00_rb,1.1159e+00_rb,1.1678e+00_rb,1.2585e+00_rb,1.4003e+00_rb/)
      kbo(:,38,10) = (/ &
     &9.5502e-01_rb,9.9217e-01_rb,1.0459e+00_rb,1.1387e+00_rb,1.2834e+00_rb/)
      kbo(:,39,10) = (/ &
     &8.4320e-01_rb,8.8008e-01_rb,9.3921e-01_rb,1.0432e+00_rb,1.1899e+00_rb/)
      kbo(:,40,10) = (/ &
     &7.4742e-01_rb,7.8308e-01_rb,8.3378e-01_rb,9.2859e-01_rb,1.0715e+00_rb/)
      kbo(:,41,10) = (/ &
     &6.6607e-01_rb,6.9838e-01_rb,7.4378e-01_rb,8.2580e-01_rb,9.6253e-01_rb/)
      kbo(:,42,10) = (/ &
     &5.9399e-01_rb,6.2402e-01_rb,6.6818e-01_rb,7.3908e-01_rb,8.7040e-01_rb/)
      kbo(:,43,10) = (/ &
     &5.3134e-01_rb,5.5822e-01_rb,5.9679e-01_rb,6.5834e-01_rb,7.8163e-01_rb/)
      kbo(:,44,10) = (/ &
     &4.7483e-01_rb,4.9813e-01_rb,5.3656e-01_rb,5.8775e-01_rb,7.0057e-01_rb/)
      kbo(:,45,10) = (/ &
     &4.2242e-01_rb,4.4383e-01_rb,4.8053e-01_rb,5.2923e-01_rb,6.3377e-01_rb/)
      kbo(:,46,10) = (/ &
     &3.7426e-01_rb,3.9626e-01_rb,4.2998e-01_rb,4.7693e-01_rb,5.7307e-01_rb/)
      kbo(:,47,10) = (/ &
     &3.3305e-01_rb,3.5232e-01_rb,3.8380e-01_rb,4.2680e-01_rb,5.1056e-01_rb/)
      kbo(:,48,10) = (/ &
     &2.9118e-01_rb,3.1195e-01_rb,3.3996e-01_rb,3.8124e-01_rb,4.5847e-01_rb/)
      kbo(:,49,10) = (/ &
     &2.5186e-01_rb,2.7185e-01_rb,2.9957e-01_rb,3.4007e-01_rb,4.1454e-01_rb/)
      kbo(:,50,10) = (/ &
     &2.1978e-01_rb,2.4030e-01_rb,2.6478e-01_rb,3.0486e-01_rb,3.7227e-01_rb/)
      kbo(:,51,10) = (/ &
     &1.9384e-01_rb,2.1260e-01_rb,2.3536e-01_rb,2.7096e-01_rb,3.3311e-01_rb/)
      kbo(:,52,10) = (/ &
     &1.6918e-01_rb,1.8784e-01_rb,2.0901e-01_rb,2.4060e-01_rb,2.9976e-01_rb/)
      kbo(:,53,10) = (/ &
     &1.4644e-01_rb,1.6376e-01_rb,1.8425e-01_rb,2.1431e-01_rb,2.7054e-01_rb/)
      kbo(:,54,10) = (/ &
     &1.3139e-01_rb,1.4808e-01_rb,1.6611e-01_rb,1.9208e-01_rb,2.4005e-01_rb/)
      kbo(:,55,10) = (/ &
     &1.2172e-01_rb,1.3644e-01_rb,1.5558e-01_rb,1.7831e-01_rb,2.1286e-01_rb/)
      kbo(:,56,10) = (/ &
     &1.1074e-01_rb,1.2717e-01_rb,1.4642e-01_rb,1.6603e-01_rb,1.9730e-01_rb/)
      kbo(:,57,10) = (/ &
     &1.0195e-01_rb,1.1787e-01_rb,1.3707e-01_rb,1.5838e-01_rb,1.8439e-01_rb/)
      kbo(:,58,10) = (/ &
     &9.6243e-02_rb,1.1076e-01_rb,1.2956e-01_rb,1.5169e-01_rb,1.7652e-01_rb/)
      kbo(:,59,10) = (/ &
     &9.7302e-02_rb,1.1627e-01_rb,1.3740e-01_rb,1.6328e-01_rb,1.9053e-01_rb/)
      kbo(:,13,11) = (/ &
     &7.0360e+01_rb,6.9342e+01_rb,6.8249e+01_rb,6.7098e+01_rb,6.5980e+01_rb/)
      kbo(:,14,11) = (/ &
     &6.5791e+01_rb,6.4723e+01_rb,6.3625e+01_rb,6.2598e+01_rb,6.1783e+01_rb/)
      kbo(:,15,11) = (/ &
     &6.0426e+01_rb,5.9395e+01_rb,5.8516e+01_rb,5.7828e+01_rb,5.7219e+01_rb/)
      kbo(:,16,11) = (/ &
     &5.4683e+01_rb,5.3915e+01_rb,5.3328e+01_rb,5.2878e+01_rb,5.2385e+01_rb/)
      kbo(:,17,11) = (/ &
     &4.8953e+01_rb,4.8487e+01_rb,4.8177e+01_rb,4.7782e+01_rb,4.7308e+01_rb/)
      kbo(:,18,11) = (/ &
     &4.3480e+01_rb,4.3274e+01_rb,4.2982e+01_rb,4.2563e+01_rb,4.2021e+01_rb/)
      kbo(:,19,11) = (/ &
     &3.8401e+01_rb,3.8213e+01_rb,3.7862e+01_rb,3.7391e+01_rb,3.6879e+01_rb/)
      kbo(:,20,11) = (/ &
     &3.3602e+01_rb,3.3341e+01_rb,3.2960e+01_rb,3.2565e+01_rb,3.2185e+01_rb/)
      kbo(:,21,11) = (/ &
     &2.9094e+01_rb,2.8806e+01_rb,2.8524e+01_rb,2.8243e+01_rb,2.8023e+01_rb/)
      kbo(:,22,11) = (/ &
     &2.4954e+01_rb,2.4758e+01_rb,2.4575e+01_rb,2.4433e+01_rb,2.4352e+01_rb/)
      kbo(:,23,11) = (/ &
     &2.1326e+01_rb,2.1219e+01_rb,2.1148e+01_rb,2.1108e+01_rb,2.1117e+01_rb/)
      kbo(:,24,11) = (/ &
     &1.8212e+01_rb,1.8167e+01_rb,1.8172e+01_rb,1.8221e+01_rb,1.8288e+01_rb/)
      kbo(:,25,11) = (/ &
     &1.5541e+01_rb,1.5569e+01_rb,1.5649e+01_rb,1.5756e+01_rb,1.5867e+01_rb/)
      kbo(:,26,11) = (/ &
     &1.3185e+01_rb,1.3326e+01_rb,1.3479e+01_rb,1.3624e+01_rb,1.3786e+01_rb/)
      kbo(:,27,11) = (/ &
     &1.0920e+01_rb,1.1211e+01_rb,1.1487e+01_rb,1.1743e+01_rb,1.1957e+01_rb/)
      kbo(:,28,11) = (/ &
     &9.0678e+00_rb,9.3710e+00_rb,9.6834e+00_rb,9.9920e+00_rb,1.0324e+01_rb/)
      kbo(:,29,11) = (/ &
     &7.6881e+00_rb,7.9879e+00_rb,8.2968e+00_rb,8.6379e+00_rb,9.0061e+00_rb/)
      kbo(:,30,11) = (/ &
     &6.6033e+00_rb,6.8954e+00_rb,7.2204e+00_rb,7.5787e+00_rb,7.9480e+00_rb/)
      kbo(:,31,11) = (/ &
     &5.8224e+00_rb,6.1213e+00_rb,6.4394e+00_rb,6.7947e+00_rb,7.1839e+00_rb/)
      kbo(:,32,11) = (/ &
     &5.1825e+00_rb,5.4854e+00_rb,5.8203e+00_rb,6.1672e+00_rb,6.5644e+00_rb/)
      kbo(:,33,11) = (/ &
     &4.6793e+00_rb,4.9853e+00_rb,5.3218e+00_rb,5.6796e+00_rb,6.0723e+00_rb/)
      kbo(:,34,11) = (/ &
     &4.2112e+00_rb,4.5240e+00_rb,4.8672e+00_rb,5.2510e+00_rb,5.6439e+00_rb/)
      kbo(:,35,11) = (/ &
     &3.8061e+00_rb,4.1193e+00_rb,4.4694e+00_rb,4.8571e+00_rb,5.2645e+00_rb/)
      kbo(:,36,11) = (/ &
     &3.4331e+00_rb,3.7452e+00_rb,4.0978e+00_rb,4.4887e+00_rb,4.9173e+00_rb/)
      kbo(:,37,11) = (/ &
     &3.0586e+00_rb,3.3607e+00_rb,3.7172e+00_rb,4.1153e+00_rb,4.5572e+00_rb/)
      kbo(:,38,11) = (/ &
     &2.7517e+00_rb,3.0453e+00_rb,3.3976e+00_rb,3.7947e+00_rb,4.2441e+00_rb/)
      kbo(:,39,11) = (/ &
     &2.5013e+00_rb,2.7908e+00_rb,3.1383e+00_rb,3.5236e+00_rb,3.9765e+00_rb/)
      kbo(:,40,11) = (/ &
     &2.2268e+00_rb,2.5094e+00_rb,2.8534e+00_rb,3.2426e+00_rb,3.6921e+00_rb/)
      kbo(:,41,11) = (/ &
     &1.9832e+00_rb,2.2609e+00_rb,2.6000e+00_rb,2.9899e+00_rb,3.4337e+00_rb/)
      kbo(:,42,11) = (/ &
     &1.7785e+00_rb,2.0479e+00_rb,2.3793e+00_rb,2.7692e+00_rb,3.2048e+00_rb/)
      kbo(:,43,11) = (/ &
     &1.5856e+00_rb,1.8448e+00_rb,2.1667e+00_rb,2.5496e+00_rb,2.9735e+00_rb/)
      kbo(:,44,11) = (/ &
     &1.4123e+00_rb,1.6629e+00_rb,1.9679e+00_rb,2.3444e+00_rb,2.7547e+00_rb/)
      kbo(:,45,11) = (/ &
     &1.2640e+00_rb,1.5067e+00_rb,1.7987e+00_rb,2.1603e+00_rb,2.5558e+00_rb/)
      kbo(:,46,11) = (/ &
     &1.1290e+00_rb,1.3620e+00_rb,1.6422e+00_rb,1.9855e+00_rb,2.3652e+00_rb/)
      kbo(:,47,11) = (/ &
     &9.9161e-01_rb,1.2154e+00_rb,1.4836e+00_rb,1.8087e+00_rb,2.1736e+00_rb/)
      kbo(:,48,11) = (/ &
     &8.7615e-01_rb,1.0875e+00_rb,1.3440e+00_rb,1.6515e+00_rb,1.9976e+00_rb/)
      kbo(:,49,11) = (/ &
     &7.7782e-01_rb,9.7773e-01_rb,1.2220e+00_rb,1.5130e+00_rb,1.8356e+00_rb/)
      kbo(:,50,11) = (/ &
     &6.8345e-01_rb,8.7089e-01_rb,1.1065e+00_rb,1.3830e+00_rb,1.6924e+00_rb/)
      kbo(:,51,11) = (/ &
     &5.9562e-01_rb,7.7251e-01_rb,9.9776e-01_rb,1.2648e+00_rb,1.5621e+00_rb/)
      kbo(:,52,11) = (/ &
     &5.1948e-01_rb,6.8509e-01_rb,8.9806e-01_rb,1.1564e+00_rb,1.4407e+00_rb/)
      kbo(:,53,11) = (/ &
     &4.5430e-01_rb,6.0810e-01_rb,8.0999e-01_rb,1.0551e+00_rb,1.3256e+00_rb/)
      kbo(:,54,11) = (/ &
     &3.8584e-01_rb,5.2960e-01_rb,7.1926e-01_rb,9.5469e-01_rb,1.2208e+00_rb/)
      kbo(:,55,11) = (/ &
     &3.2585e-01_rb,4.5415e-01_rb,6.2813e-01_rb,8.5255e-01_rb,1.1189e+00_rb/)
      kbo(:,56,11) = (/ &
     &2.9347e-01_rb,3.8309e-01_rb,5.4571e-01_rb,7.5695e-01_rb,1.0111e+00_rb/)
      kbo(:,57,11) = (/ &
     &2.6872e-01_rb,3.3263e-01_rb,4.6996e-01_rb,6.6376e-01_rb,9.0688e-01_rb/)
      kbo(:,58,11) = (/ &
     &2.4960e-01_rb,3.0758e-01_rb,4.0207e-01_rb,5.8213e-01_rb,8.0803e-01_rb/)
      kbo(:,59,11) = (/ &
     &2.6183e-01_rb,3.2106e-01_rb,3.9760e-01_rb,5.1091e-01_rb,7.1717e-01_rb/)
      kbo(:,13,12) = (/ &
     &1.1832e+02_rb,1.1787e+02_rb,1.1696e+02_rb,1.1585e+02_rb,1.1448e+02_rb/)
      kbo(:,14,12) = (/ &
     &1.1787e+02_rb,1.1706e+02_rb,1.1585e+02_rb,1.1435e+02_rb,1.1282e+02_rb/)
      kbo(:,15,12) = (/ &
     &1.1482e+02_rb,1.1352e+02_rb,1.1198e+02_rb,1.1040e+02_rb,1.0880e+02_rb/)
      kbo(:,16,12) = (/ &
     &1.0896e+02_rb,1.0749e+02_rb,1.0590e+02_rb,1.0433e+02_rb,1.0304e+02_rb/)
      kbo(:,17,12) = (/ &
     &1.0128e+02_rb,9.9814e+01_rb,9.8422e+01_rb,9.7349e+01_rb,9.6504e+01_rb/)
      kbo(:,18,12) = (/ &
     &9.2677e+01_rb,9.1506e+01_rb,9.0639e+01_rb,9.0100e+01_rb,8.9682e+01_rb/)
      kbo(:,19,12) = (/ &
     &8.3897e+01_rb,8.3222e+01_rb,8.2911e+01_rb,8.2751e+01_rb,8.2502e+01_rb/)
      kbo(:,20,12) = (/ &
     &7.5342e+01_rb,7.5217e+01_rb,7.5315e+01_rb,7.5254e+01_rb,7.5130e+01_rb/)
      kbo(:,21,12) = (/ &
     &6.7351e+01_rb,6.7622e+01_rb,6.7745e+01_rb,6.7796e+01_rb,6.7734e+01_rb/)
      kbo(:,22,12) = (/ &
     &5.9964e+01_rb,6.0293e+01_rb,6.0504e+01_rb,6.0565e+01_rb,6.0581e+01_rb/)
      kbo(:,23,12) = (/ &
     &5.3111e+01_rb,5.3464e+01_rb,5.3681e+01_rb,5.3837e+01_rb,5.4061e+01_rb/)
      kbo(:,24,12) = (/ &
     &4.6795e+01_rb,4.7144e+01_rb,4.7460e+01_rb,4.7828e+01_rb,4.8279e+01_rb/)
      kbo(:,25,12) = (/ &
     &4.1060e+01_rb,4.1497e+01_rb,4.1989e+01_rb,4.2550e+01_rb,4.3265e+01_rb/)
      kbo(:,26,12) = (/ &
     &3.6007e+01_rb,3.6594e+01_rb,3.7261e+01_rb,3.8060e+01_rb,3.9005e+01_rb/)
      kbo(:,27,12) = (/ &
     &3.1657e+01_rb,3.2380e+01_rb,3.3234e+01_rb,3.4259e+01_rb,3.5455e+01_rb/)
      kbo(:,28,12) = (/ &
     &2.7936e+01_rb,2.8816e+01_rb,2.9878e+01_rb,3.1120e+01_rb,3.2499e+01_rb/)
      kbo(:,29,12) = (/ &
     &2.4811e+01_rb,2.5887e+01_rb,2.7138e+01_rb,2.8541e+01_rb,3.0069e+01_rb/)
      kbo(:,30,12) = (/ &
     &2.2271e+01_rb,2.3484e+01_rb,2.4883e+01_rb,2.6431e+01_rb,2.8119e+01_rb/)
      kbo(:,31,12) = (/ &
     &2.0203e+01_rb,2.1534e+01_rb,2.3058e+01_rb,2.4751e+01_rb,2.6602e+01_rb/)
      kbo(:,32,12) = (/ &
     &1.8534e+01_rb,1.9980e+01_rb,2.1635e+01_rb,2.3479e+01_rb,2.5477e+01_rb/)
      kbo(:,33,12) = (/ &
     &1.7225e+01_rb,1.8794e+01_rb,2.0572e+01_rb,2.2559e+01_rb,2.4699e+01_rb/)
      kbo(:,34,12) = (/ &
     &1.6174e+01_rb,1.7857e+01_rb,1.9770e+01_rb,2.1880e+01_rb,2.4147e+01_rb/)
      kbo(:,35,12) = (/ &
     &1.5236e+01_rb,1.7017e+01_rb,1.9034e+01_rb,2.1242e+01_rb,2.3610e+01_rb/)
      kbo(:,36,12) = (/ &
     &1.4322e+01_rb,1.6183e+01_rb,1.8272e+01_rb,2.0552e+01_rb,2.2984e+01_rb/)
      kbo(:,37,12) = (/ &
     &1.3315e+01_rb,1.5210e+01_rb,1.7331e+01_rb,1.9645e+01_rb,2.2115e+01_rb/)
      kbo(:,38,12) = (/ &
     &1.2427e+01_rb,1.4349e+01_rb,1.6493e+01_rb,1.8831e+01_rb,2.1326e+01_rb/)
      kbo(:,39,12) = (/ &
     &1.1655e+01_rb,1.3594e+01_rb,1.5756e+01_rb,1.8111e+01_rb,2.0619e+01_rb/)
      kbo(:,40,12) = (/ &
     &1.0800e+01_rb,1.2718e+01_rb,1.4870e+01_rb,1.7214e+01_rb,1.9711e+01_rb/)
      kbo(:,41,12) = (/ &
     &1.0009e+01_rb,1.1890e+01_rb,1.4023e+01_rb,1.6343e+01_rb,1.8829e+01_rb/)
      kbo(:,42,12) = (/ &
     &9.2911e+00_rb,1.1131e+01_rb,1.3230e+01_rb,1.5526e+01_rb,1.7990e+01_rb/)
      kbo(:,43,12) = (/ &
     &8.5453e+00_rb,1.0322e+01_rb,1.2366e+01_rb,1.4622e+01_rb,1.7049e+01_rb/)
      kbo(:,44,12) = (/ &
     &7.8249e+00_rb,9.5233e+00_rb,1.1499e+01_rb,1.3704e+01_rb,1.6085e+01_rb/)
      kbo(:,45,12) = (/ &
     &7.1656e+00_rb,8.7847e+00_rb,1.0682e+01_rb,1.2826e+01_rb,1.5159e+01_rb/)
      kbo(:,46,12) = (/ &
     &6.5222e+00_rb,8.0621e+00_rb,9.8691e+00_rb,1.1939e+01_rb,1.4215e+01_rb/)
      kbo(:,47,12) = (/ &
     &5.8688e+00_rb,7.3253e+00_rb,9.0292e+00_rb,1.1003e+01_rb,1.3207e+01_rb/)
      kbo(:,48,12) = (/ &
     &5.2686e+00_rb,6.6401e+00_rb,8.2460e+00_rb,1.0116e+01_rb,1.2240e+01_rb/)
      kbo(:,49,12) = (/ &
     &4.7119e+00_rb,6.0011e+00_rb,7.5181e+00_rb,9.2810e+00_rb,1.1309e+01_rb/)
      kbo(:,50,12) = (/ &
     &4.2243e+00_rb,5.4409e+00_rb,6.8771e+00_rb,8.5451e+00_rb,1.0476e+01_rb/)
      kbo(:,51,12) = (/ &
     &3.7894e+00_rb,4.9333e+00_rb,6.2908e+00_rb,7.8717e+00_rb,9.7058e+00_rb/)
      kbo(:,52,12) = (/ &
     &3.3915e+00_rb,4.4569e+00_rb,5.7416e+00_rb,7.2423e+00_rb,8.9799e+00_rb/)
      kbo(:,53,12) = (/ &
     &3.0222e+00_rb,4.0130e+00_rb,5.2245e+00_rb,6.6473e+00_rb,8.2931e+00_rb/)
      kbo(:,54,12) = (/ &
     &2.7077e+00_rb,3.6347e+00_rb,4.7766e+00_rb,6.1280e+00_rb,7.6968e+00_rb/)
      kbo(:,55,12) = (/ &
     &2.4210e+00_rb,3.2970e+00_rb,4.3697e+00_rb,5.6562e+00_rb,7.1563e+00_rb/)
      kbo(:,56,12) = (/ &
     &2.1307e+00_rb,2.9831e+00_rb,3.9864e+00_rb,5.2103e+00_rb,6.6404e+00_rb/)
      kbo(:,57,12) = (/ &
     &1.8552e+00_rb,2.6736e+00_rb,3.6295e+00_rb,4.7868e+00_rb,6.1491e+00_rb/)
      kbo(:,58,12) = (/ &
     &1.6050e+00_rb,2.3734e+00_rb,3.3136e+00_rb,4.4004e+00_rb,5.7055e+00_rb/)
      kbo(:,59,12) = (/ &
     &1.4434e+00_rb,2.1814e+00_rb,3.1118e+00_rb,4.2418e+00_rb,5.5300e+00_rb/)
      kbo(:,13,13) = (/ &
     &2.0665e+02_rb,2.0326e+02_rb,2.0074e+02_rb,1.9867e+02_rb,1.9680e+02_rb/)
      kbo(:,14,13) = (/ &
     &2.1250e+02_rb,2.0998e+02_rb,2.0827e+02_rb,2.0659e+02_rb,2.0468e+02_rb/)
      kbo(:,15,13) = (/ &
     &2.1695e+02_rb,2.1587e+02_rb,2.1446e+02_rb,2.1260e+02_rb,2.1056e+02_rb/)
      kbo(:,16,13) = (/ &
     &2.2062e+02_rb,2.1972e+02_rb,2.1811e+02_rb,2.1619e+02_rb,2.1401e+02_rb/)
      kbo(:,17,13) = (/ &
     &2.2121e+02_rb,2.1996e+02_rb,2.1823e+02_rb,2.1618e+02_rb,2.1385e+02_rb/)
      kbo(:,18,13) = (/ &
     &2.1775e+02_rb,2.1636e+02_rb,2.1445e+02_rb,2.1229e+02_rb,2.1021e+02_rb/)
      kbo(:,19,13) = (/ &
     &2.1064e+02_rb,2.0900e+02_rb,2.0712e+02_rb,2.0551e+02_rb,2.0407e+02_rb/)
      kbo(:,20,13) = (/ &
     &2.0008e+02_rb,1.9874e+02_rb,1.9769e+02_rb,1.9686e+02_rb,1.9621e+02_rb/)
      kbo(:,21,13) = (/ &
     &1.8747e+02_rb,1.8700e+02_rb,1.8691e+02_rb,1.8703e+02_rb,1.8742e+02_rb/)
      kbo(:,22,13) = (/ &
     &1.7415e+02_rb,1.7476e+02_rb,1.7572e+02_rb,1.7701e+02_rb,1.7852e+02_rb/)
      kbo(:,23,13) = (/ &
     &1.6113e+02_rb,1.6295e+02_rb,1.6510e+02_rb,1.6752e+02_rb,1.7009e+02_rb/)
      kbo(:,24,13) = (/ &
     &1.4912e+02_rb,1.5211e+02_rb,1.5536e+02_rb,1.5887e+02_rb,1.6232e+02_rb/)
      kbo(:,25,13) = (/ &
     &1.3846e+02_rb,1.4246e+02_rb,1.4677e+02_rb,1.5111e+02_rb,1.5523e+02_rb/)
      kbo(:,26,13) = (/ &
     &1.2926e+02_rb,1.3422e+02_rb,1.3933e+02_rb,1.4432e+02_rb,1.4910e+02_rb/)
      kbo(:,27,13) = (/ &
     &1.2138e+02_rb,1.2724e+02_rb,1.3302e+02_rb,1.3864e+02_rb,1.4402e+02_rb/)
      kbo(:,28,13) = (/ &
     &1.1492e+02_rb,1.2144e+02_rb,1.2785e+02_rb,1.3411e+02_rb,1.4008e+02_rb/)
      kbo(:,29,13) = (/ &
     &1.0978e+02_rb,1.1690e+02_rb,1.2395e+02_rb,1.3076e+02_rb,1.3730e+02_rb/)
      kbo(:,30,13) = (/ &
     &1.0580e+02_rb,1.1355e+02_rb,1.2118e+02_rb,1.2849e+02_rb,1.3548e+02_rb/)
      kbo(:,31,13) = (/ &
     &1.0304e+02_rb,1.1137e+02_rb,1.1944e+02_rb,1.2716e+02_rb,1.3451e+02_rb/)
      kbo(:,32,13) = (/ &
     &1.0136e+02_rb,1.1014e+02_rb,1.1859e+02_rb,1.2663e+02_rb,1.3416e+02_rb/)
      kbo(:,33,13) = (/ &
     &1.0056e+02_rb,1.0971e+02_rb,1.1845e+02_rb,1.2667e+02_rb,1.3431e+02_rb/)
      kbo(:,34,13) = (/ &
     &1.0020e+02_rb,1.0961e+02_rb,1.1855e+02_rb,1.2688e+02_rb,1.3458e+02_rb/)
      kbo(:,35,13) = (/ &
     &9.9483e+01_rb,1.0912e+02_rb,1.1821e+02_rb,1.2667e+02_rb,1.3443e+02_rb/)
      kbo(:,36,13) = (/ &
     &9.8103e+01_rb,1.0792e+02_rb,1.1721e+02_rb,1.2581e+02_rb,1.3368e+02_rb/)
      kbo(:,37,13) = (/ &
     &9.5455e+01_rb,1.0552e+02_rb,1.1504e+02_rb,1.2386e+02_rb,1.3193e+02_rb/)
      kbo(:,38,13) = (/ &
     &9.2912e+01_rb,1.0319e+02_rb,1.1292e+02_rb,1.2195e+02_rb,1.3022e+02_rb/)
      kbo(:,39,13) = (/ &
     &9.0538e+01_rb,1.0100e+02_rb,1.1091e+02_rb,1.2014e+02_rb,1.2859e+02_rb/)
      kbo(:,40,13) = (/ &
     &8.7195e+01_rb,9.7854e+01_rb,1.0799e+02_rb,1.1748e+02_rb,1.2618e+02_rb/)
      kbo(:,41,13) = (/ &
     &8.3785e+01_rb,9.4631e+01_rb,1.0497e+02_rb,1.1470e+02_rb,1.2366e+02_rb/)
      kbo(:,42,13) = (/ &
     &8.0430e+01_rb,9.1433e+01_rb,1.0197e+02_rb,1.1192e+02_rb,1.2113e+02_rb/)
      kbo(:,43,13) = (/ &
     &7.6477e+01_rb,8.7634e+01_rb,9.8384e+01_rb,1.0856e+02_rb,1.1808e+02_rb/)
      kbo(:,44,13) = (/ &
     &7.2299e+01_rb,8.3562e+01_rb,9.4524e+01_rb,1.0494e+02_rb,1.1474e+02_rb/)
      kbo(:,45,13) = (/ &
     &6.8166e+01_rb,7.9496e+01_rb,9.0629e+01_rb,1.0127e+02_rb,1.1130e+02_rb/)
      kbo(:,46,13) = (/ &
     &6.3834e+01_rb,7.5188e+01_rb,8.6450e+01_rb,9.7312e+01_rb,1.0760e+02_rb/)
      kbo(:,47,13) = (/ &
     &5.9081e+01_rb,7.0413e+01_rb,8.1772e+01_rb,9.2852e+01_rb,1.0340e+02_rb/)
      kbo(:,48,13) = (/ &
     &5.4396e+01_rb,6.5661e+01_rb,7.7060e+01_rb,8.8317e+01_rb,9.9114e+01_rb/)
      kbo(:,49,13) = (/ &
     &4.9801e+01_rb,6.0961e+01_rb,7.2354e+01_rb,8.3709e+01_rb,9.4725e+01_rb/)
      kbo(:,50,13) = (/ &
     &4.5581e+01_rb,5.6585e+01_rb,6.7933e+01_rb,7.9345e+01_rb,9.0545e+01_rb/)
      kbo(:,51,13) = (/ &
     &4.1581e+01_rb,5.2405e+01_rb,6.3662e+01_rb,7.5096e+01_rb,8.6425e+01_rb/)
      kbo(:,52,13) = (/ &
     &3.7719e+01_rb,4.8309e+01_rb,5.9448e+01_rb,7.0850e+01_rb,8.2254e+01_rb/)
      kbo(:,53,13) = (/ &
     &3.4012e+01_rb,4.4282e+01_rb,5.5265e+01_rb,6.6609e+01_rb,7.8050e+01_rb/)
      kbo(:,54,13) = (/ &
     &3.0756e+01_rb,4.0675e+01_rb,5.1476e+01_rb,6.2735e+01_rb,7.4183e+01_rb/)
      kbo(:,55,13) = (/ &
     &2.7771e+01_rb,3.7316e+01_rb,4.7906e+01_rb,5.9042e+01_rb,7.0461e+01_rb/)
      kbo(:,56,13) = (/ &
     &2.4943e+01_rb,3.4093e+01_rb,4.4398e+01_rb,5.5401e+01_rb,6.6760e+01_rb/)
      kbo(:,57,13) = (/ &
     &2.2280e+01_rb,3.0993e+01_rb,4.0957e+01_rb,5.1794e+01_rb,6.3073e+01_rb/)
      kbo(:,58,13) = (/ &
     &1.9914e+01_rb,2.8171e+01_rb,3.7788e+01_rb,4.8428e+01_rb,5.9596e+01_rb/)
      kbo(:,59,13) = (/ &
     &1.9003e+01_rb,2.7062e+01_rb,3.6534e+01_rb,4.7084e+01_rb,5.8201e+01_rb/)
      kbo(:,13,14) = (/ &
     &3.9453e+02_rb,3.9054e+02_rb,3.8556e+02_rb,3.8029e+02_rb,3.7518e+02_rb/)
      kbo(:,14,14) = (/ &
     &4.2588e+02_rb,4.2023e+02_rb,4.1375e+02_rb,4.0776e+02_rb,4.0199e+02_rb/)
      kbo(:,15,14) = (/ &
     &4.5408e+02_rb,4.4627e+02_rb,4.3917e+02_rb,4.3301e+02_rb,4.2714e+02_rb/)
      kbo(:,16,14) = (/ &
     &4.7789e+02_rb,4.6976e+02_rb,4.6328e+02_rb,4.5698e+02_rb,4.5114e+02_rb/)
      kbo(:,17,14) = (/ &
     &5.0033e+02_rb,4.9329e+02_rb,4.8659e+02_rb,4.8030e+02_rb,4.7472e+02_rb/)
      kbo(:,18,14) = (/ &
     &5.2160e+02_rb,5.1479e+02_rb,5.0850e+02_rb,5.0281e+02_rb,4.9755e+02_rb/)
      kbo(:,19,14) = (/ &
     &5.3975e+02_rb,5.3405e+02_rb,5.2880e+02_rb,5.2359e+02_rb,5.1826e+02_rb/)
      kbo(:,20,14) = (/ &
     &5.5471e+02_rb,5.5025e+02_rb,5.4544e+02_rb,5.4054e+02_rb,5.3559e+02_rb/)
      kbo(:,21,14) = (/ &
     &5.6616e+02_rb,5.6246e+02_rb,5.5854e+02_rb,5.5439e+02_rb,5.5012e+02_rb/)
      kbo(:,22,14) = (/ &
     &5.7373e+02_rb,5.7143e+02_rb,5.6842e+02_rb,5.6526e+02_rb,5.6165e+02_rb/)
      kbo(:,23,14) = (/ &
     &5.7822e+02_rb,5.7745e+02_rb,5.7602e+02_rb,5.7378e+02_rb,5.7033e+02_rb/)
      kbo(:,24,14) = (/ &
     &5.8038e+02_rb,5.8144e+02_rb,5.8147e+02_rb,5.7978e+02_rb,5.7697e+02_rb/)
      kbo(:,25,14) = (/ &
     &5.8144e+02_rb,5.8416e+02_rb,5.8500e+02_rb,5.8436e+02_rb,5.8236e+02_rb/)
      kbo(:,26,14) = (/ &
     &5.8189e+02_rb,5.8575e+02_rb,5.8780e+02_rb,5.8818e+02_rb,5.8686e+02_rb/)
      kbo(:,27,14) = (/ &
     &5.8218e+02_rb,5.8719e+02_rb,5.9038e+02_rb,5.9148e+02_rb,5.9059e+02_rb/)
      kbo(:,28,14) = (/ &
     &5.8262e+02_rb,5.8871e+02_rb,5.9261e+02_rb,5.9417e+02_rb,5.9365e+02_rb/)
      kbo(:,29,14) = (/ &
     &5.8351e+02_rb,5.9046e+02_rb,5.9468e+02_rb,5.9643e+02_rb,5.9604e+02_rb/)
      kbo(:,30,14) = (/ &
     &5.8500e+02_rb,5.9236e+02_rb,5.9667e+02_rb,5.9846e+02_rb,5.9808e+02_rb/)
      kbo(:,31,14) = (/ &
     &5.8683e+02_rb,5.9418e+02_rb,5.9850e+02_rb,6.0020e+02_rb,5.9975e+02_rb/)
      kbo(:,32,14) = (/ &
     &5.8889e+02_rb,5.9602e+02_rb,6.0013e+02_rb,6.0165e+02_rb,6.0111e+02_rb/)
      kbo(:,33,14) = (/ &
     &5.9113e+02_rb,5.9792e+02_rb,6.0167e+02_rb,6.0289e+02_rb,6.0219e+02_rb/)
      kbo(:,34,14) = (/ &
     &5.9309e+02_rb,5.9954e+02_rb,6.0296e+02_rb,6.0390e+02_rb,6.0309e+02_rb/)
      kbo(:,35,14) = (/ &
     &5.9425e+02_rb,6.0057e+02_rb,6.0391e+02_rb,6.0481e+02_rb,6.0395e+02_rb/)
      kbo(:,36,14) = (/ &
     &5.9453e+02_rb,6.0102e+02_rb,6.0449e+02_rb,6.0555e+02_rb,6.0478e+02_rb/)
      kbo(:,37,14) = (/ &
     &5.9344e+02_rb,6.0064e+02_rb,6.0466e+02_rb,6.0613e+02_rb,6.0565e+02_rb/)
      kbo(:,38,14) = (/ &
     &5.9210e+02_rb,6.0005e+02_rb,6.0460e+02_rb,6.0657e+02_rb,6.0636e+02_rb/)
      kbo(:,39,14) = (/ &
     &5.9062e+02_rb,5.9933e+02_rb,6.0439e+02_rb,6.0685e+02_rb,6.0693e+02_rb/)
      kbo(:,40,14) = (/ &
     &5.8781e+02_rb,5.9770e+02_rb,6.0363e+02_rb,6.0675e+02_rb,6.0742e+02_rb/)
      kbo(:,41,14) = (/ &
     &5.8441e+02_rb,5.9563e+02_rb,6.0254e+02_rb,6.0641e+02_rb,6.0773e+02_rb/)
      kbo(:,42,14) = (/ &
     &5.8066e+02_rb,5.9320e+02_rb,6.0125e+02_rb,6.0582e+02_rb,6.0784e+02_rb/)
      kbo(:,43,14) = (/ &
     &5.7546e+02_rb,5.8975e+02_rb,5.9921e+02_rb,6.0478e+02_rb,6.0764e+02_rb/)
      kbo(:,44,14) = (/ &
     &5.6919e+02_rb,5.8551e+02_rb,5.9655e+02_rb,6.0334e+02_rb,6.0709e+02_rb/)
      kbo(:,45,14) = (/ &
     &5.6228e+02_rb,5.8057e+02_rb,5.9328e+02_rb,6.0149e+02_rb,6.0621e+02_rb/)
      kbo(:,46,14) = (/ &
     &5.5416e+02_rb,5.7453e+02_rb,5.8925e+02_rb,5.9904e+02_rb,6.0487e+02_rb/)
      kbo(:,47,14) = (/ &
     &5.4413e+02_rb,5.6695e+02_rb,5.8396e+02_rb,5.9568e+02_rb,6.0296e+02_rb/)
      kbo(:,48,14) = (/ &
     &5.3289e+02_rb,5.5838e+02_rb,5.7778e+02_rb,5.9151e+02_rb,6.0055e+02_rb/)
      kbo(:,49,14) = (/ &
     &5.2046e+02_rb,5.4886e+02_rb,5.7056e+02_rb,5.8661e+02_rb,5.9745e+02_rb/)
      kbo(:,50,14) = (/ &
     &5.0763e+02_rb,5.3890e+02_rb,5.6309e+02_rb,5.8123e+02_rb,5.9389e+02_rb/)
      kbo(:,51,14) = (/ &
     &4.9397e+02_rb,5.2829e+02_rb,5.5490e+02_rb,5.7522e+02_rb,5.8985e+02_rb/)
      kbo(:,52,14) = (/ &
     &4.7917e+02_rb,5.1663e+02_rb,5.4597e+02_rb,5.6843e+02_rb,5.8514e+02_rb/)
      kbo(:,53,14) = (/ &
     &4.6310e+02_rb,5.0389e+02_rb,5.3606e+02_rb,5.6086e+02_rb,5.7969e+02_rb/)
      kbo(:,54,14) = (/ &
     &4.4738e+02_rb,4.9113e+02_rb,5.2610e+02_rb,5.5325e+02_rb,5.7401e+02_rb/)
      kbo(:,55,14) = (/ &
     &4.3145e+02_rb,4.7800e+02_rb,5.1576e+02_rb,5.4530e+02_rb,5.6795e+02_rb/)
      kbo(:,56,14) = (/ &
     &4.1442e+02_rb,4.6397e+02_rb,5.0460e+02_rb,5.3665e+02_rb,5.6135e+02_rb/)
      kbo(:,57,14) = (/ &
     &3.9638e+02_rb,4.4904e+02_rb,4.9255e+02_rb,5.2719e+02_rb,5.5411e+02_rb/)
      kbo(:,58,14) = (/ &
     &3.7836e+02_rb,4.3410e+02_rb,4.8026e+02_rb,5.1754e+02_rb,5.4672e+02_rb/)
      kbo(:,59,14) = (/ &
     &3.7094e+02_rb,4.2781e+02_rb,4.7507e+02_rb,5.1348e+02_rb,5.4351e+02_rb/)
      kbo(:,13,15) = (/ &
     &7.2633e+02_rb,7.1401e+02_rb,7.0254e+02_rb,6.9098e+02_rb,6.8020e+02_rb/)
      kbo(:,14,15) = (/ &
     &8.3011e+02_rb,8.1776e+02_rb,8.0585e+02_rb,7.9323e+02_rb,7.8158e+02_rb/)
      kbo(:,15,15) = (/ &
     &9.4371e+02_rb,9.3141e+02_rb,9.1779e+02_rb,9.0358e+02_rb,8.9021e+02_rb/)
      kbo(:,16,15) = (/ &
     &1.0657e+03_rb,1.0512e+03_rb,1.0343e+03_rb,1.0183e+03_rb,1.0024e+03_rb/)
      kbo(:,17,15) = (/ &
     &1.1900e+03_rb,1.1715e+03_rb,1.1530e+03_rb,1.1344e+03_rb,1.1149e+03_rb/)
      kbo(:,18,15) = (/ &
     &1.3144e+03_rb,1.2934e+03_rb,1.2719e+03_rb,1.2491e+03_rb,1.2254e+03_rb/)
      kbo(:,19,15) = (/ &
     &1.4395e+03_rb,1.4148e+03_rb,1.3884e+03_rb,1.3607e+03_rb,1.3337e+03_rb/)
      kbo(:,20,15) = (/ &
     &1.5639e+03_rb,1.5338e+03_rb,1.5021e+03_rb,1.4707e+03_rb,1.4392e+03_rb/)
      kbo(:,21,15) = (/ &
     &1.6839e+03_rb,1.6488e+03_rb,1.6126e+03_rb,1.5763e+03_rb,1.5389e+03_rb/)
      kbo(:,22,15) = (/ &
     &1.7952e+03_rb,1.7545e+03_rb,1.7135e+03_rb,1.6710e+03_rb,1.6285e+03_rb/)
      kbo(:,23,15) = (/ &
     &1.8976e+03_rb,1.8509e+03_rb,1.8033e+03_rb,1.7556e+03_rb,1.7089e+03_rb/)
      kbo(:,24,15) = (/ &
     &1.9890e+03_rb,1.9360e+03_rb,1.8826e+03_rb,1.8303e+03_rb,1.7792e+03_rb/)
      kbo(:,25,15) = (/ &
     &2.0674e+03_rb,2.0090e+03_rb,1.9512e+03_rb,1.8940e+03_rb,1.8389e+03_rb/)
      kbo(:,26,15) = (/ &
     &2.1327e+03_rb,2.0701e+03_rb,2.0075e+03_rb,1.9460e+03_rb,1.8872e+03_rb/)
      kbo(:,27,15) = (/ &
     &2.1871e+03_rb,2.1199e+03_rb,2.0526e+03_rb,1.9878e+03_rb,1.9258e+03_rb/)
      kbo(:,28,15) = (/ &
     &2.2305e+03_rb,2.1590e+03_rb,2.0884e+03_rb,2.0207e+03_rb,1.9559e+03_rb/)
      kbo(:,29,15) = (/ &
     &2.2640e+03_rb,2.1881e+03_rb,2.1149e+03_rb,2.0446e+03_rb,1.9773e+03_rb/)
      kbo(:,30,15) = (/ &
     &2.2867e+03_rb,2.2086e+03_rb,2.1332e+03_rb,2.0610e+03_rb,1.9920e+03_rb/)
      kbo(:,31,15) = (/ &
     &2.3021e+03_rb,2.2217e+03_rb,2.1448e+03_rb,2.0709e+03_rb,2.0000e+03_rb/)
      kbo(:,32,15) = (/ &
     &2.3102e+03_rb,2.2286e+03_rb,2.1502e+03_rb,2.0751e+03_rb,2.0031e+03_rb/)
      kbo(:,33,15) = (/ &
     &2.3126e+03_rb,2.2299e+03_rb,2.1503e+03_rb,2.0747e+03_rb,2.0018e+03_rb/)
      kbo(:,34,15) = (/ &
     &2.3131e+03_rb,2.2295e+03_rb,2.1494e+03_rb,2.0730e+03_rb,1.9993e+03_rb/)
      kbo(:,35,15) = (/ &
     &2.3181e+03_rb,2.2337e+03_rb,2.1530e+03_rb,2.0757e+03_rb,2.0015e+03_rb/)
      kbo(:,36,15) = (/ &
     &2.3296e+03_rb,2.2444e+03_rb,2.1625e+03_rb,2.0844e+03_rb,2.0094e+03_rb/)
      kbo(:,37,15) = (/ &
     &2.3524e+03_rb,2.2655e+03_rb,2.1822e+03_rb,2.1029e+03_rb,2.0270e+03_rb/)
      kbo(:,38,15) = (/ &
     &2.3744e+03_rb,2.2859e+03_rb,2.2015e+03_rb,2.1208e+03_rb,2.0440e+03_rb/)
      kbo(:,39,15) = (/ &
     &2.3947e+03_rb,2.3050e+03_rb,2.2197e+03_rb,2.1376e+03_rb,2.0600e+03_rb/)
      kbo(:,40,15) = (/ &
     &2.4238e+03_rb,2.3323e+03_rb,2.2455e+03_rb,2.1621e+03_rb,2.0831e+03_rb/)
      kbo(:,41,15) = (/ &
     &2.4535e+03_rb,2.3601e+03_rb,2.2718e+03_rb,2.1873e+03_rb,2.1067e+03_rb/)
      kbo(:,42,15) = (/ &
     &2.4827e+03_rb,2.3879e+03_rb,2.2978e+03_rb,2.2120e+03_rb,2.1299e+03_rb/)
      kbo(:,43,15) = (/ &
     &2.5170e+03_rb,2.4207e+03_rb,2.3287e+03_rb,2.2417e+03_rb,2.1580e+03_rb/)
      kbo(:,44,15) = (/ &
     &2.5535e+03_rb,2.4558e+03_rb,2.3619e+03_rb,2.2731e+03_rb,2.1881e+03_rb/)
      kbo(:,45,15) = (/ &
     &2.5895e+03_rb,2.4909e+03_rb,2.3955e+03_rb,2.3046e+03_rb,2.2183e+03_rb/)
      kbo(:,46,15) = (/ &
     &2.6275e+03_rb,2.5286e+03_rb,2.4314e+03_rb,2.3383e+03_rb,2.2507e+03_rb/)
      kbo(:,47,15) = (/ &
     &2.6692e+03_rb,2.5702e+03_rb,2.4716e+03_rb,2.3766e+03_rb,2.2868e+03_rb/)
      kbo(:,48,15) = (/ &
     &2.7104e+03_rb,2.6117e+03_rb,2.5124e+03_rb,2.4158e+03_rb,2.3235e+03_rb/)
      kbo(:,49,15) = (/ &
     &2.7515e+03_rb,2.6527e+03_rb,2.5535e+03_rb,2.4553e+03_rb,2.3608e+03_rb/)
      kbo(:,50,15) = (/ &
     &2.7894e+03_rb,2.6912e+03_rb,2.5919e+03_rb,2.4928e+03_rb,2.3968e+03_rb/)
      kbo(:,51,15) = (/ &
     &2.8258e+03_rb,2.7283e+03_rb,2.6293e+03_rb,2.5297e+03_rb,2.4319e+03_rb/)
      kbo(:,52,15) = (/ &
     &2.8615e+03_rb,2.7650e+03_rb,2.6661e+03_rb,2.5667e+03_rb,2.4680e+03_rb/)
      kbo(:,53,15) = (/ &
     &2.8958e+03_rb,2.8012e+03_rb,2.7030e+03_rb,2.6033e+03_rb,2.5041e+03_rb/)
      kbo(:,54,15) = (/ &
     &2.9257e+03_rb,2.8343e+03_rb,2.7368e+03_rb,2.6377e+03_rb,2.5378e+03_rb/)
      kbo(:,55,15) = (/ &
     &2.9533e+03_rb,2.8653e+03_rb,2.7685e+03_rb,2.6697e+03_rb,2.5700e+03_rb/)
      kbo(:,56,15) = (/ &
     &2.9793e+03_rb,2.8948e+03_rb,2.8002e+03_rb,2.7018e+03_rb,2.6024e+03_rb/)
      kbo(:,57,15) = (/ &
     &3.0042e+03_rb,2.9237e+03_rb,2.8317e+03_rb,2.7340e+03_rb,2.6347e+03_rb/)
      kbo(:,58,15) = (/ &
     &3.0257e+03_rb,2.9495e+03_rb,2.8610e+03_rb,2.7641e+03_rb,2.6651e+03_rb/)
      kbo(:,59,15) = (/ &
     &3.0338e+03_rb,2.9597e+03_rb,2.8725e+03_rb,2.7761e+03_rb,2.6773e+03_rb/)
      kbo(:,13,16) = (/ &
     &1.0026e+03_rb,9.7775e+02_rb,9.6058e+02_rb,9.5314e+02_rb,9.4719e+02_rb/)
      kbo(:,14,16) = (/ &
     &1.2035e+03_rb,1.1711e+03_rb,1.1486e+03_rb,1.1339e+03_rb,1.1211e+03_rb/)
      kbo(:,15,16) = (/ &
     &1.4371e+03_rb,1.3952e+03_rb,1.3661e+03_rb,1.3423e+03_rb,1.3197e+03_rb/)
      kbo(:,16,16) = (/ &
     &1.7031e+03_rb,1.6520e+03_rb,1.6127e+03_rb,1.5775e+03_rb,1.5442e+03_rb/)
      kbo(:,17,16) = (/ &
     &2.0034e+03_rb,1.9408e+03_rb,1.8881e+03_rb,1.8394e+03_rb,1.7917e+03_rb/)
      kbo(:,18,16) = (/ &
     &2.3372e+03_rb,2.2594e+03_rb,2.1891e+03_rb,2.1223e+03_rb,2.0585e+03_rb/)
      kbo(:,19,16) = (/ &
     &2.7003e+03_rb,2.6007e+03_rb,2.5092e+03_rb,2.4222e+03_rb,2.3387e+03_rb/)
      kbo(:,20,16) = (/ &
     &3.0827e+03_rb,2.9575e+03_rb,2.8405e+03_rb,2.7290e+03_rb,2.6233e+03_rb/)
      kbo(:,21,16) = (/ &
     &3.4765e+03_rb,3.3200e+03_rb,3.1728e+03_rb,3.0348e+03_rb,2.9062e+03_rb/)
      kbo(:,22,16) = (/ &
     &3.8557e+03_rb,3.6642e+03_rb,3.4867e+03_rb,3.3227e+03_rb,3.1685e+03_rb/)
      kbo(:,23,16) = (/ &
     &4.2141e+03_rb,3.9881e+03_rb,3.7808e+03_rb,3.5882e+03_rb,3.4103e+03_rb/)
      kbo(:,24,16) = (/ &
     &4.5433e+03_rb,4.2837e+03_rb,4.0442e+03_rb,3.8246e+03_rb,3.6242e+03_rb/)
      kbo(:,25,16) = (/ &
     &4.8340e+03_rb,4.5405e+03_rb,4.2717e+03_rb,4.0284e+03_rb,3.8059e+03_rb/)
      kbo(:,26,16) = (/ &
     &5.0756e+03_rb,4.7511e+03_rb,4.4591e+03_rb,4.1940e+03_rb,3.9524e+03_rb/)
      kbo(:,27,16) = (/ &
     &5.2698e+03_rb,4.9206e+03_rb,4.6082e+03_rb,4.3247e+03_rb,4.0667e+03_rb/)
      kbo(:,28,16) = (/ &
     &5.4203e+03_rb,5.0514e+03_rb,4.7208e+03_rb,4.4225e+03_rb,4.1519e+03_rb/)
      kbo(:,29,16) = (/ &
     &5.5225e+03_rb,5.1430e+03_rb,4.7990e+03_rb,4.4889e+03_rb,4.2086e+03_rb/)
      kbo(:,30,16) = (/ &
     &5.5968e+03_rb,5.2018e+03_rb,4.8479e+03_rb,4.5299e+03_rb,4.2431e+03_rb/)
      kbo(:,31,16) = (/ &
     &5.6319e+03_rb,5.2297e+03_rb,4.8692e+03_rb,4.5471e+03_rb,4.2577e+03_rb/)
      kbo(:,32,16) = (/ &
     &5.6393e+03_rb,5.2323e+03_rb,4.8693e+03_rb,4.5451e+03_rb,4.2544e+03_rb/)
      kbo(:,33,16) = (/ &
     &5.6219e+03_rb,5.2153e+03_rb,4.8522e+03_rb,4.5281e+03_rb,4.2389e+03_rb/)
      kbo(:,34,16) = (/ &
     &5.6014e+03_rb,5.1950e+03_rb,4.8333e+03_rb,4.5105e+03_rb,4.2225e+03_rb/)
      kbo(:,35,16) = (/ &
     &5.6085e+03_rb,5.1993e+03_rb,4.8360e+03_rb,4.5120e+03_rb,4.2238e+03_rb/)
      kbo(:,36,16) = (/ &
     &5.6508e+03_rb,5.2362e+03_rb,4.8672e+03_rb,4.5401e+03_rb,4.2477e+03_rb/)
      kbo(:,37,16) = (/ &
     &5.7523e+03_rb,5.3259e+03_rb,4.9464e+03_rb,4.6095e+03_rb,4.3092e+03_rb/)
      kbo(:,38,16) = (/ &
     &5.8536e+03_rb,5.4145e+03_rb,5.0247e+03_rb,4.6788e+03_rb,4.3703e+03_rb/)
      kbo(:,39,16) = (/ &
     &5.9499e+03_rb,5.4442e+03_rb,5.0495e+03_rb,4.7451e+03_rb,4.4289e+03_rb/)
      kbo(:,40,16) = (/ &
     &6.0943e+03_rb,5.6261e+03_rb,5.2115e+03_rb,4.8437e+03_rb,4.5174e+03_rb/)
      kbo(:,41,16) = (/ &
     &6.2466e+03_rb,5.7599e+03_rb,5.3292e+03_rb,4.9481e+03_rb,4.6100e+03_rb/)
      kbo(:,42,16) = (/ &
     &6.4016e+03_rb,5.8949e+03_rb,5.4490e+03_rb,5.0540e+03_rb,4.7039e+03_rb/)
      kbo(:,43,16) = (/ &
     &6.5921e+03_rb,6.0603e+03_rb,5.5952e+03_rb,5.1830e+03_rb,4.8181e+03_rb/)
      kbo(:,44,16) = (/ &
     &6.8054e+03_rb,6.2437e+03_rb,5.7562e+03_rb,5.3258e+03_rb,4.9439e+03_rb/)
      kbo(:,45,16) = (/ &
     &7.0283e+03_rb,6.4351e+03_rb,5.9235e+03_rb,5.4736e+03_rb,5.0747e+03_rb/)
      kbo(:,46,16) = (/ &
     &7.2775e+03_rb,6.6473e+03_rb,6.1076e+03_rb,5.6361e+03_rb,5.2188e+03_rb/)
      kbo(:,47,16) = (/ &
     &7.5694e+03_rb,6.8970e+03_rb,6.3223e+03_rb,5.8241e+03_rb,5.3852e+03_rb/)
      kbo(:,48,16) = (/ &
     &7.8805e+03_rb,7.1623e+03_rb,6.5487e+03_rb,6.0220e+03_rb,5.5600e+03_rb/)
      kbo(:,49,16) = (/ &
     &8.2112e+03_rb,7.4430e+03_rb,6.7899e+03_rb,6.2293e+03_rb,5.7432e+03_rb/)
      kbo(:,50,16) = (/ &
     &8.5417e+03_rb,7.7234e+03_rb,7.0279e+03_rb,6.4340e+03_rb,5.9223e+03_rb/)
      kbo(:,51,16) = (/ &
     &8.8820e+03_rb,8.0126e+03_rb,7.2736e+03_rb,6.6439e+03_rb,6.1042e+03_rb/)
      kbo(:,52,16) = (/ &
     &9.2420e+03_rb,8.3170e+03_rb,7.5326e+03_rb,6.8651e+03_rb,6.2940e+03_rb/)
      kbo(:,53,16) = (/ &
     &9.6269e+03_rb,8.6414e+03_rb,7.8081e+03_rb,7.0991e+03_rb,6.4953e+03_rb/)
      kbo(:,54,16) = (/ &
     &1.0002e+04_rb,8.9556e+03_rb,8.0739e+03_rb,7.3265e+03_rb,6.6894e+03_rb/)
      kbo(:,55,16) = (/ &
     &1.0382e+04_rb,9.2723e+03_rb,8.3431e+03_rb,7.5548e+03_rb,6.8840e+03_rb/)
      kbo(:,56,16) = (/ &
     &1.0780e+04_rb,9.6091e+03_rb,8.6265e+03_rb,7.7944e+03_rb,7.0899e+03_rb/)
      kbo(:,57,16) = (/ &
     &1.1201e+04_rb,9.9648e+03_rb,8.9241e+03_rb,8.0480e+03_rb,7.3039e+03_rb/)
      kbo(:,58,16) = (/ &
     &1.1621e+04_rb,1.0321e+04_rb,9.2223e+03_rb,8.3005e+03_rb,7.5182e+03_rb/)
      kbo(:,59,16) = (/ &
     &1.1796e+04_rb,1.0468e+04_rb,9.3476e+03_rb,8.4058e+03_rb,7.6078e+03_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kao_mo2(:, 1) = (/ &
     & 2.31723e-06_rb, 2.28697e-06_rb, 2.25710e-06_rb, 2.22762e-06_rb, 2.19852e-06_rb, &
     & 2.16981e-06_rb, 2.14147e-06_rb, 2.11350e-06_rb, 2.08590e-06_rb, 2.05865e-06_rb, &
     & 2.03176e-06_rb, 2.00523e-06_rb, 1.97904e-06_rb, 1.95319e-06_rb, 1.92768e-06_rb, &
     & 1.90250e-06_rb, 1.87765e-06_rb, 1.85313e-06_rb, 1.82893e-06_rb/)
      kao_mo2(:, 2) = (/ &
     & 1.81980e-06_rb, 1.81352e-06_rb, 1.80726e-06_rb, 1.80101e-06_rb, 1.79479e-06_rb, &
     & 1.78860e-06_rb, 1.78242e-06_rb, 1.77626e-06_rb, 1.77013e-06_rb, 1.76402e-06_rb, &
     & 1.75792e-06_rb, 1.75185e-06_rb, 1.74580e-06_rb, 1.73977e-06_rb, 1.73377e-06_rb, &
     & 1.72778e-06_rb, 1.72181e-06_rb, 1.71587e-06_rb, 1.70994e-06_rb/)
      kao_mo2(:, 3) = (/ &
     & 2.26922e-06_rb, 2.25413e-06_rb, 2.23914e-06_rb, 2.22425e-06_rb, 2.20945e-06_rb, &
     & 2.19476e-06_rb, 2.18016e-06_rb, 2.16566e-06_rb, 2.15126e-06_rb, 2.13695e-06_rb, &
     & 2.12274e-06_rb, 2.10862e-06_rb, 2.09459e-06_rb, 2.08066e-06_rb, 2.06683e-06_rb, &
     & 2.05308e-06_rb, 2.03942e-06_rb, 2.02586e-06_rb, 2.01239e-06_rb/)
      kao_mo2(:, 4) = (/ &
     & 2.15555e-06_rb, 2.14539e-06_rb, 2.13527e-06_rb, 2.12520e-06_rb, 2.11517e-06_rb, &
     & 2.10520e-06_rb, 2.09527e-06_rb, 2.08538e-06_rb, 2.07555e-06_rb, 2.06576e-06_rb, &
     & 2.05601e-06_rb, 2.04631e-06_rb, 2.03666e-06_rb, 2.02706e-06_rb, 2.01749e-06_rb, &
     & 2.00798e-06_rb, 1.99851e-06_rb, 1.98908e-06_rb, 1.97970e-06_rb/)
      kao_mo2(:, 5) = (/ &
     & 2.05821e-06_rb, 2.04914e-06_rb, 2.04011e-06_rb, 2.03111e-06_rb, 2.02216e-06_rb, &
     & 2.01324e-06_rb, 2.00437e-06_rb, 1.99553e-06_rb, 1.98673e-06_rb, 1.97798e-06_rb, &
     & 1.96926e-06_rb, 1.96057e-06_rb, 1.95193e-06_rb, 1.94333e-06_rb, 1.93476e-06_rb, &
     & 1.92623e-06_rb, 1.91774e-06_rb, 1.90928e-06_rb, 1.90087e-06_rb/)
      kao_mo2(:, 6) = (/ &
     & 2.20148e-06_rb, 2.18998e-06_rb, 2.17854e-06_rb, 2.16717e-06_rb, 2.15585e-06_rb, &
     & 2.14459e-06_rb, 2.13339e-06_rb, 2.12225e-06_rb, 2.11117e-06_rb, 2.10014e-06_rb, &
     & 2.08918e-06_rb, 2.07827e-06_rb, 2.06741e-06_rb, 2.05662e-06_rb, 2.04588e-06_rb, &
     & 2.03519e-06_rb, 2.02457e-06_rb, 2.01399e-06_rb, 2.00348e-06_rb/)
      kao_mo2(:, 7) = (/ &
     & 2.28960e-06_rb, 2.27651e-06_rb, 2.26349e-06_rb, 2.25054e-06_rb, 2.23767e-06_rb, &
     & 2.22487e-06_rb, 2.21215e-06_rb, 2.19950e-06_rb, 2.18692e-06_rb, 2.17441e-06_rb, &
     & 2.16198e-06_rb, 2.14961e-06_rb, 2.13732e-06_rb, 2.12509e-06_rb, 2.11294e-06_rb, &
     & 2.10085e-06_rb, 2.08884e-06_rb, 2.07689e-06_rb, 2.06501e-06_rb/)
      kao_mo2(:, 8) = (/ &
     & 2.28564e-06_rb, 2.27363e-06_rb, 2.26168e-06_rb, 2.24980e-06_rb, 2.23798e-06_rb, &
     & 2.22622e-06_rb, 2.21452e-06_rb, 2.20288e-06_rb, 2.19131e-06_rb, 2.17980e-06_rb, &
     & 2.16834e-06_rb, 2.15695e-06_rb, 2.14562e-06_rb, 2.13434e-06_rb, 2.12313e-06_rb, &
     & 2.11197e-06_rb, 2.10087e-06_rb, 2.08984e-06_rb, 2.07886e-06_rb/)
      kao_mo2(:, 9) = (/ &
     & 2.28505e-06_rb, 2.27395e-06_rb, 2.26291e-06_rb, 2.25192e-06_rb, 2.24099e-06_rb, &
     & 2.23011e-06_rb, 2.21928e-06_rb, 2.20850e-06_rb, 2.19778e-06_rb, 2.18711e-06_rb, &
     & 2.17649e-06_rb, 2.16592e-06_rb, 2.15540e-06_rb, 2.14494e-06_rb, 2.13452e-06_rb, &
     & 2.12416e-06_rb, 2.11385e-06_rb, 2.10358e-06_rb, 2.09337e-06_rb/)
      kao_mo2(:,10) = (/ &
     & 2.25915e-06_rb, 2.24938e-06_rb, 2.23965e-06_rb, 2.22997e-06_rb, 2.22032e-06_rb, &
     & 2.21072e-06_rb, 2.20116e-06_rb, 2.19164e-06_rb, 2.18216e-06_rb, 2.17272e-06_rb, &
     & 2.16333e-06_rb, 2.15397e-06_rb, 2.14465e-06_rb, 2.13538e-06_rb, 2.12614e-06_rb, &
     & 2.11695e-06_rb, 2.10779e-06_rb, 2.09868e-06_rb, 2.08960e-06_rb/)
      kao_mo2(:,11) = (/ &
     & 2.52025e-06_rb, 2.50423e-06_rb, 2.48831e-06_rb, 2.47249e-06_rb, 2.45677e-06_rb, &
     & 2.44115e-06_rb, 2.42563e-06_rb, 2.41021e-06_rb, 2.39489e-06_rb, 2.37967e-06_rb, &
     & 2.36454e-06_rb, 2.34951e-06_rb, 2.33457e-06_rb, 2.31973e-06_rb, 2.30498e-06_rb, &
     & 2.29033e-06_rb, 2.27577e-06_rb, 2.26130e-06_rb, 2.24692e-06_rb/)
      kao_mo2(:,12) = (/ &
     & 2.52634e-06_rb, 2.51180e-06_rb, 2.49735e-06_rb, 2.48299e-06_rb, 2.46871e-06_rb, &
     & 2.45451e-06_rb, 2.44039e-06_rb, 2.42635e-06_rb, 2.41239e-06_rb, 2.39851e-06_rb, &
     & 2.38472e-06_rb, 2.37100e-06_rb, 2.35736e-06_rb, 2.34380e-06_rb, 2.33032e-06_rb, &
     & 2.31691e-06_rb, 2.30358e-06_rb, 2.29033e-06_rb, 2.27716e-06_rb/)
      kao_mo2(:,13) = (/ &
     & 2.66614e-06_rb, 2.64897e-06_rb, 2.63191e-06_rb, 2.61496e-06_rb, 2.59812e-06_rb, &
     & 2.58138e-06_rb, 2.56476e-06_rb, 2.54824e-06_rb, 2.53183e-06_rb, 2.51552e-06_rb, &
     & 2.49932e-06_rb, 2.48322e-06_rb, 2.46723e-06_rb, 2.45134e-06_rb, 2.43555e-06_rb, &
     & 2.41987e-06_rb, 2.40428e-06_rb, 2.38880e-06_rb, 2.37341e-06_rb/)
      kao_mo2(:,14) = (/ &
     & 2.96755e-06_rb, 2.94803e-06_rb, 2.92864e-06_rb, 2.90937e-06_rb, 2.89023e-06_rb, &
     & 2.87122e-06_rb, 2.85233e-06_rb, 2.83357e-06_rb, 2.81493e-06_rb, 2.79641e-06_rb, &
     & 2.77802e-06_rb, 2.75974e-06_rb, 2.74159e-06_rb, 2.72355e-06_rb, 2.70563e-06_rb, &
     & 2.68784e-06_rb, 2.67015e-06_rb, 2.65259e-06_rb, 2.63514e-06_rb/)
      kao_mo2(:,15) = (/ &
     & 1.30668e-06_rb, 1.31378e-06_rb, 1.32091e-06_rb, 1.32808e-06_rb, 1.33530e-06_rb, &
     & 1.34255e-06_rb, 1.34984e-06_rb, 1.35717e-06_rb, 1.36454e-06_rb, 1.37195e-06_rb, &
     & 1.37941e-06_rb, 1.38690e-06_rb, 1.39443e-06_rb, 1.40200e-06_rb, 1.40962e-06_rb, &
     & 1.41727e-06_rb, 1.42497e-06_rb, 1.43271e-06_rb, 1.44049e-06_rb/)
      kao_mo2(:,16) = (/ &
     & 5.99001e-07_rb, 6.16844e-07_rb, 6.35219e-07_rb, 6.54141e-07_rb, 6.73626e-07_rb, &
     & 6.93692e-07_rb, 7.14356e-07_rb, 7.35635e-07_rb, 7.57548e-07_rb, 7.80114e-07_rb, &
     & 8.03352e-07_rb, 8.27282e-07_rb, 8.51925e-07_rb, 8.77302e-07_rb, 9.03435e-07_rb, &
     & 9.30347e-07_rb, 9.58060e-07_rb, 9.86599e-07_rb, 1.01599e-06_rb/)

!     The array KBO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level above 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kbo_mo2(:, 1) = (/ &
     & 4.97626e-07_rb, 5.05955e-07_rb, 5.14424e-07_rb, 5.23034e-07_rb, 5.31789e-07_rb, &
     & 5.40690e-07_rb, 5.49739e-07_rb, 5.58941e-07_rb, 5.68296e-07_rb, 5.77808e-07_rb, &
     & 5.87479e-07_rb, 5.97312e-07_rb, 6.07310e-07_rb, 6.17475e-07_rb, 6.27810e-07_rb, &
     & 6.38318e-07_rb, 6.49002e-07_rb, 6.59865e-07_rb, 6.70910e-07_rb/)
      kbo_mo2(:, 2) = (/ &
     & 3.10232e-06_rb, 3.06339e-06_rb, 3.02496e-06_rb, 2.98700e-06_rb, 2.94952e-06_rb, &
     & 2.91252e-06_rb, 2.87597e-06_rb, 2.83989e-06_rb, 2.80426e-06_rb, 2.76907e-06_rb, &
     & 2.73433e-06_rb, 2.70002e-06_rb, 2.66614e-06_rb, 2.63269e-06_rb, 2.59966e-06_rb, &
     & 2.56704e-06_rb, 2.53483e-06_rb, 2.50303e-06_rb, 2.47162e-06_rb/)
      kbo_mo2(:, 3) = (/ &
     & 2.91635e-06_rb, 2.88637e-06_rb, 2.85669e-06_rb, 2.82733e-06_rb, 2.79826e-06_rb, &
     & 2.76949e-06_rb, 2.74102e-06_rb, 2.71284e-06_rb, 2.68495e-06_rb, 2.65735e-06_rb, &
     & 2.63003e-06_rb, 2.60299e-06_rb, 2.57623e-06_rb, 2.54975e-06_rb, 2.52353e-06_rb, &
     & 2.49759e-06_rb, 2.47191e-06_rb, 2.44650e-06_rb, 2.42135e-06_rb/)
      kbo_mo2(:, 4) = (/ &
     & 3.15584e-06_rb, 3.11986e-06_rb, 3.08430e-06_rb, 3.04914e-06_rb, 3.01438e-06_rb, &
     & 2.98002e-06_rb, 2.94605e-06_rb, 2.91247e-06_rb, 2.87927e-06_rb, 2.84645e-06_rb, &
     & 2.81400e-06_rb, 2.78192e-06_rb, 2.75021e-06_rb, 2.71886e-06_rb, 2.68787e-06_rb, &
     & 2.65723e-06_rb, 2.62694e-06_rb, 2.59699e-06_rb, 2.56739e-06_rb/)
      kbo_mo2(:, 5) = (/ &
     & 2.52067e-06_rb, 2.50127e-06_rb, 2.48202e-06_rb, 2.46291e-06_rb, 2.44396e-06_rb, &
     & 2.42515e-06_rb, 2.40648e-06_rb, 2.38796e-06_rb, 2.36958e-06_rb, 2.35134e-06_rb, &
     & 2.33324e-06_rb, 2.31529e-06_rb, 2.29747e-06_rb, 2.27978e-06_rb, 2.26224e-06_rb, &
     & 2.24482e-06_rb, 2.22755e-06_rb, 2.21040e-06_rb, 2.19339e-06_rb/)
      kbo_mo2(:, 6) = (/ &
     & 2.37304e-06_rb, 2.36340e-06_rb, 2.35380e-06_rb, 2.34423e-06_rb, 2.33471e-06_rb, &
     & 2.32522e-06_rb, 2.31578e-06_rb, 2.30637e-06_rb, 2.29700e-06_rb, 2.28766e-06_rb, &
     & 2.27837e-06_rb, 2.26911e-06_rb, 2.25989e-06_rb, 2.25071e-06_rb, 2.24157e-06_rb, &
     & 2.23246e-06_rb, 2.22339e-06_rb, 2.21436e-06_rb, 2.20536e-06_rb/)
      kbo_mo2(:, 7) = (/ &
     & 2.56366e-06_rb, 2.56395e-06_rb, 2.56424e-06_rb, 2.56453e-06_rb, 2.56482e-06_rb, &
     & 2.56510e-06_rb, 2.56539e-06_rb, 2.56568e-06_rb, 2.56597e-06_rb, 2.56625e-06_rb, &
     & 2.56654e-06_rb, 2.56683e-06_rb, 2.56712e-06_rb, 2.56741e-06_rb, 2.56769e-06_rb, &
     & 2.56798e-06_rb, 2.56827e-06_rb, 2.56856e-06_rb, 2.56885e-06_rb/)
      kbo_mo2(:, 8) = (/ &
     & 2.54502e-06_rb, 2.55393e-06_rb, 2.56287e-06_rb, 2.57185e-06_rb, 2.58085e-06_rb, &
     & 2.58989e-06_rb, 2.59896e-06_rb, 2.60806e-06_rb, 2.61719e-06_rb, 2.62636e-06_rb, &
     & 2.63555e-06_rb, 2.64478e-06_rb, 2.65404e-06_rb, 2.66334e-06_rb, 2.67266e-06_rb, &
     & 2.68202e-06_rb, 2.69141e-06_rb, 2.70084e-06_rb, 2.71030e-06_rb/)
      kbo_mo2(:, 9) = (/ &
     & 1.84106e-06_rb, 1.83922e-06_rb, 1.83737e-06_rb, 1.83553e-06_rb, 1.83369e-06_rb, &
     & 1.83186e-06_rb, 1.83002e-06_rb, 1.82819e-06_rb, 1.82636e-06_rb, 1.82453e-06_rb, &
     & 1.82270e-06_rb, 1.82087e-06_rb, 1.81905e-06_rb, 1.81723e-06_rb, 1.81541e-06_rb, &
     & 1.81359e-06_rb, 1.81177e-06_rb, 1.80996e-06_rb, 1.80814e-06_rb/)
      kbo_mo2(:,10) = (/ &
     & 1.83886e-06_rb, 1.83632e-06_rb, 1.83379e-06_rb, 1.83126e-06_rb, 1.82874e-06_rb, &
     & 1.82622e-06_rb, 1.82370e-06_rb, 1.82119e-06_rb, 1.81868e-06_rb, 1.81617e-06_rb, &
     & 1.81367e-06_rb, 1.81117e-06_rb, 1.80867e-06_rb, 1.80618e-06_rb, 1.80369e-06_rb, &
     & 1.80120e-06_rb, 1.79872e-06_rb, 1.79624e-06_rb, 1.79377e-06_rb/)
      kbo_mo2(:,11) = (/ &
     & 2.30390e-06_rb, 2.30269e-06_rb, 2.30148e-06_rb, 2.30028e-06_rb, 2.29907e-06_rb, &
     & 2.29787e-06_rb, 2.29667e-06_rb, 2.29546e-06_rb, 2.29426e-06_rb, 2.29306e-06_rb, &
     & 2.29186e-06_rb, 2.29066e-06_rb, 2.28946e-06_rb, 2.28826e-06_rb, 2.28706e-06_rb, &
     & 2.28586e-06_rb, 2.28466e-06_rb, 2.28347e-06_rb, 2.28227e-06_rb/)
      kbo_mo2(:,12) = (/ &
     & 2.38201e-06_rb, 2.36536e-06_rb, 2.34882e-06_rb, 2.33240e-06_rb, 2.31609e-06_rb, &
     & 2.29990e-06_rb, 2.28382e-06_rb, 2.26785e-06_rb, 2.25199e-06_rb, 2.23625e-06_rb, &
     & 2.22061e-06_rb, 2.20508e-06_rb, 2.18967e-06_rb, 2.17436e-06_rb, 2.15915e-06_rb, &
     & 2.14406e-06_rb, 2.12907e-06_rb, 2.11418e-06_rb, 2.09940e-06_rb/)
      kbo_mo2(:,13) = (/ &
     & 2.33326e-06_rb, 2.32549e-06_rb, 2.31775e-06_rb, 2.31003e-06_rb, 2.30234e-06_rb, &
     & 2.29467e-06_rb, 2.28703e-06_rb, 2.27941e-06_rb, 2.27182e-06_rb, 2.26426e-06_rb, &
     & 2.25672e-06_rb, 2.24920e-06_rb, 2.24171e-06_rb, 2.23424e-06_rb, 2.22680e-06_rb, &
     & 2.21939e-06_rb, 2.21200e-06_rb, 2.20463e-06_rb, 2.19729e-06_rb/)
      kbo_mo2(:,14) = (/ &
     & 2.75292e-06_rb, 2.75210e-06_rb, 2.75129e-06_rb, 2.75047e-06_rb, 2.74965e-06_rb, &
     & 2.74883e-06_rb, 2.74801e-06_rb, 2.74720e-06_rb, 2.74638e-06_rb, 2.74556e-06_rb, &
     & 2.74475e-06_rb, 2.74393e-06_rb, 2.74311e-06_rb, 2.74230e-06_rb, 2.74148e-06_rb, &
     & 2.74067e-06_rb, 2.73985e-06_rb, 2.73904e-06_rb, 2.73822e-06_rb/)
      kbo_mo2(:,15) = (/ &
     & 2.55262e-06_rb, 2.53364e-06_rb, 2.51480e-06_rb, 2.49611e-06_rb, 2.47755e-06_rb, &
     & 2.45913e-06_rb, 2.44084e-06_rb, 2.42269e-06_rb, 2.40468e-06_rb, 2.38680e-06_rb, &
     & 2.36906e-06_rb, 2.35144e-06_rb, 2.33396e-06_rb, 2.31660e-06_rb, 2.29938e-06_rb, &
     & 2.28228e-06_rb, 2.26531e-06_rb, 2.24847e-06_rb, 2.23175e-06_rb/)
      kbo_mo2(:,16) = (/ &
     & 3.11382e-06_rb, 3.08751e-06_rb, 3.06141e-06_rb, 3.03554e-06_rb, 3.00989e-06_rb, &
     & 2.98445e-06_rb, 2.95923e-06_rb, 2.93422e-06_rb, 2.90942e-06_rb, 2.88483e-06_rb, &
     & 2.86045e-06_rb, 2.83628e-06_rb, 2.81231e-06_rb, 2.78854e-06_rb, 2.76498e-06_rb, &
     & 2.74161e-06_rb, 2.71844e-06_rb, 2.69547e-06_rb, 2.67269e-06_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &2.8858e-02_rb,3.6879e-02_rb,4.0746e-02_rb,4.2561e-02_rb,4.2740e-02_rb,4.2707e-02_rb, &
     &4.4109e-02_rb,4.4540e-02_rb,4.5206e-02_rb,4.4679e-02_rb,4.5034e-02_rb,4.5364e-02_rb, &
     &4.6790e-02_rb,4.7857e-02_rb,4.8328e-02_rb,4.8084e-02_rb/)
      forrefo(2,:) = (/ &
     &2.7887e-02_rb,3.7376e-02_rb,4.0980e-02_rb,4.2986e-02_rb,4.3054e-02_rb,4.2975e-02_rb, &
     &4.3754e-02_rb,4.4352e-02_rb,4.4723e-02_rb,4.6236e-02_rb,4.5273e-02_rb,4.5360e-02_rb, &
     &4.5332e-02_rb,4.7587e-02_rb,4.7035e-02_rb,5.0267e-02_rb/)
      forrefo(3,:) = (/ &
     &2.5846e-02_rb,3.6753e-02_rb,4.2334e-02_rb,4.3806e-02_rb,4.3848e-02_rb,4.3215e-02_rb, &
     &4.3838e-02_rb,4.4278e-02_rb,4.4658e-02_rb,4.5403e-02_rb,4.5255e-02_rb,4.6347e-02_rb, &
     &4.4722e-02_rb,4.6612e-02_rb,4.6836e-02_rb,4.8720e-02_rb/)
      forrefo(4,:) = (/ &
     &2.8955e-02_rb,3.7608e-02_rb,4.1989e-02_rb,4.4919e-02_rb,4.2803e-02_rb,4.2842e-02_rb, &
     &4.2632e-02_rb,4.1056e-02_rb,4.0086e-02_rb,4.1401e-02_rb,4.2746e-02_rb,4.2142e-02_rb, &
     &4.1871e-02_rb,4.3917e-02_rb,4.5462e-02_rb,4.8359e-02_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 5.96496e-01_rb, 5.49171e-01_rb, 5.05600e-01_rb, 4.65486e-01_rb, 4.28555e-01_rb, &
     & 3.94554e-01_rb, 3.63250e-01_rb, 3.34430e-01_rb, 3.07897e-01_rb, 2.83468e-01_rb/)
      selfrefo(:, 2) = (/ &
     & 7.46455e-01_rb, 6.82459e-01_rb, 6.23950e-01_rb, 5.70457e-01_rb, 5.21550e-01_rb, &
     & 4.76836e-01_rb, 4.35956e-01_rb, 3.98580e-01_rb, 3.64409e-01_rb, 3.33167e-01_rb/)
      selfrefo(:, 3) = (/ &
     & 7.86805e-01_rb, 7.21186e-01_rb, 6.61040e-01_rb, 6.05910e-01_rb, 5.55378e-01_rb, &
     & 5.09059e-01_rb, 4.66605e-01_rb, 4.27690e-01_rb, 3.92021e-01_rb, 3.59327e-01_rb/)
      selfrefo(:, 4) = (/ &
     & 8.11740e-01_rb, 7.44359e-01_rb, 6.82570e-01_rb, 6.25910e-01_rb, 5.73954e-01_rb, &
     & 5.26311e-01_rb, 4.82622e-01_rb, 4.42560e-01_rb, 4.05823e-01_rb, 3.72136e-01_rb/)
      selfrefo(:, 5) = (/ &
     & 8.14870e-01_rb, 7.47200e-01_rb, 6.85150e-01_rb, 6.28253e-01_rb, 5.76081e-01_rb, &
     & 5.28241e-01_rb, 4.84374e-01_rb, 4.44150e-01_rb, 4.07266e-01_rb, 3.73446e-01_rb/)
      selfrefo(:, 6) = (/ &
     & 8.10104e-01_rb, 7.43259e-01_rb, 6.81930e-01_rb, 6.25661e-01_rb, 5.74035e-01_rb, &
     & 5.26669e-01_rb, 4.83212e-01_rb, 4.43340e-01_rb, 4.06758e-01_rb, 3.73195e-01_rb/)
      selfrefo(:, 7) = (/ &
     & 8.13119e-01_rb, 7.48127e-01_rb, 6.88330e-01_rb, 6.33312e-01_rb, 5.82692e-01_rb, &
     & 5.36118e-01_rb, 4.93267e-01_rb, 4.53840e-01_rb, 4.17565e-01_rb, 3.84189e-01_rb/)
      selfrefo(:, 8) = (/ &
     & 8.26137e-01_rb, 7.58984e-01_rb, 6.97290e-01_rb, 6.40611e-01_rb, 5.88539e-01_rb, &
     & 5.40699e-01_rb, 4.96748e-01_rb, 4.56370e-01_rb, 4.19274e-01_rb, 3.85193e-01_rb/)
      selfrefo(:, 9) = (/ &
     & 8.30566e-01_rb, 7.63984e-01_rb, 7.02740e-01_rb, 6.46405e-01_rb, 5.94587e-01_rb, &
     & 5.46922e-01_rb, 5.03079e-01_rb, 4.62750e-01_rb, 4.25654e-01_rb, 3.91532e-01_rb/)
      selfrefo(:,10) = (/ &
     & 8.67471e-01_rb, 7.91575e-01_rb, 7.22320e-01_rb, 6.59124e-01_rb, 6.01457e-01_rb, &
     & 5.48835e-01_rb, 5.00817e-01_rb, 4.57000e-01_rb, 4.17017e-01_rb, 3.80532e-01_rb/)
      selfrefo(:,11) = (/ &
     & 8.51029e-01_rb, 7.79373e-01_rb, 7.13750e-01_rb, 6.53652e-01_rb, 5.98615e-01_rb, &
     & 5.48212e-01_rb, 5.02053e-01_rb, 4.59780e-01_rb, 4.21067e-01_rb, 3.85613e-01_rb/)
      selfrefo(:,12) = (/ &
     & 8.36772e-01_rb, 7.68751e-01_rb, 7.06260e-01_rb, 6.48848e-01_rb, 5.96104e-01_rb, &
     & 5.47647e-01_rb, 5.03129e-01_rb, 4.62230e-01_rb, 4.24655e-01_rb, 3.90136e-01_rb/)
      selfrefo(:,13) = (/ &
     & 8.36551e-01_rb, 7.71089e-01_rb, 7.10750e-01_rb, 6.55133e-01_rb, 6.03867e-01_rb, &
     & 5.56614e-01_rb, 5.13058e-01_rb, 4.72910e-01_rb, 4.35904e-01_rb, 4.01794e-01_rb/)
      selfrefo(:,14) = (/ &
     & 8.84307e-01_rb, 8.11175e-01_rb, 7.44090e-01_rb, 6.82553e-01_rb, 6.26106e-01_rb, &
     & 5.74326e-01_rb, 5.26829e-01_rb, 4.83260e-01_rb, 4.43294e-01_rb, 4.06633e-01_rb/)
      selfrefo(:,15) = (/ &
     & 8.90356e-01_rb, 8.19830e-01_rb, 7.54890e-01_rb, 6.95094e-01_rb, 6.40035e-01_rb, &
     & 5.89337e-01_rb, 5.42655e-01_rb, 4.99670e-01_rb, 4.60090e-01_rb, 4.23646e-01_rb/)
      selfrefo(:,16) = (/ &
     & 9.67549e-01_rb, 8.79393e-01_rb, 7.99270e-01_rb, 7.26447e-01_rb, 6.60259e-01_rb, &
     & 6.00101e-01_rb, 5.45425e-01_rb, 4.95730e-01_rb, 4.50563e-01_rb, 4.09511e-01_rb/)

      end subroutine lw_kgb11

! **************************************************************************
      subroutine lw_kgb12
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg12, only : fracrefao, kao, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P = 174.1640 mbar, T= 215.78 K
      fracrefao(:, 1) = (/ &
     &  1.3984e-01_rb,1.6809e-01_rb,1.8072e-01_rb,1.5400e-01_rb,1.2613e-01_rb,9.6959e-02_rb, &
     &  5.9713e-02_rb,3.8631e-02_rb,2.6937e-02_rb,3.1711e-03_rb,2.3458e-03_rb,1.4653e-03_rb, &
     &  1.0567e-03_rb,6.6504e-04_rb,2.4957e-04_rb,3.5172e-05_rb/)
      fracrefao(:, 2) = (/ &
     &  1.2745e-01_rb,1.6107e-01_rb,1.6568e-01_rb,1.5436e-01_rb,1.3183e-01_rb,1.0166e-01_rb, &
     &  6.4506e-02_rb,4.7756e-02_rb,3.4472e-02_rb,3.7189e-03_rb,2.9349e-03_rb,2.1469e-03_rb, &
     &  1.3746e-03_rb,7.1691e-04_rb,2.8057e-04_rb,5.6242e-05_rb/)
      fracrefao(:, 3) = (/ &
     &  1.2181e-01_rb,1.5404e-01_rb,1.6540e-01_rb,1.5255e-01_rb,1.3736e-01_rb,9.8856e-02_rb, &
     &  6.8927e-02_rb,5.1385e-02_rb,3.7046e-02_rb,4.0302e-03_rb,3.0949e-03_rb,2.3772e-03_rb, &
     &  1.6538e-03_rb,8.9641e-04_rb,4.6991e-04_rb,1.1251e-04_rb/)
      fracrefao(:, 4) = (/ &
     &  1.1794e-01_rb,1.4864e-01_rb,1.6316e-01_rb,1.5341e-01_rb,1.3986e-01_rb,9.6656e-02_rb, &
     &  7.2478e-02_rb,5.5061e-02_rb,3.8886e-02_rb,4.3398e-03_rb,3.3576e-03_rb,2.4891e-03_rb, &
     &  1.7674e-03_rb,1.0764e-03_rb,7.7689e-04_rb,1.1251e-04_rb/)
      fracrefao(:, 5) = (/ &
     &  1.1635e-01_rb,1.4342e-01_rb,1.5924e-01_rb,1.5670e-01_rb,1.3740e-01_rb,9.7087e-02_rb, &
     &  7.6250e-02_rb,5.7802e-02_rb,4.0808e-02_rb,4.4113e-03_rb,3.6035e-03_rb,2.6269e-03_rb, &
     &  1.7586e-03_rb,1.6498e-03_rb,7.7689e-04_rb,1.1251e-04_rb/)
      fracrefao(:, 6) = (/ &
     &  1.1497e-01_rb,1.3751e-01_rb,1.5587e-01_rb,1.5904e-01_rb,1.3140e-01_rb,1.0159e-01_rb, &
     &  7.9729e-02_rb,6.1475e-02_rb,4.2382e-02_rb,4.5291e-03_rb,3.8161e-03_rb,2.7683e-03_rb, &
     &  1.9899e-03_rb,2.0395e-03_rb,7.7720e-04_rb,1.1251e-04_rb/)
      fracrefao(:, 7) = (/ &
     &  1.1331e-01_rb,1.3015e-01_rb,1.5574e-01_rb,1.5489e-01_rb,1.2697e-01_rb,1.0746e-01_rb, &
     &  8.4777e-02_rb,6.5145e-02_rb,4.4293e-02_rb,4.7426e-03_rb,3.8383e-03_rb,2.9065e-03_rb, &
     &  2.8430e-03_rb,2.0401e-03_rb,7.7689e-04_rb,1.1251e-04_rb/)
      fracrefao(:, 8) = (/ &
     &  1.0993e-01_rb,1.2320e-01_rb,1.4893e-01_rb,1.4573e-01_rb,1.3174e-01_rb,1.1149e-01_rb, &
     &  9.3326e-02_rb,6.9942e-02_rb,4.6762e-02_rb,4.9309e-03_rb,3.8583e-03_rb,4.1889e-03_rb, &
     &  3.0415e-03_rb,2.0406e-03_rb,7.7720e-04_rb,1.1251e-04_rb/)
      fracrefao(:, 9) = (/ &
     &  1.2028e-01_rb,1.2091e-01_rb,1.3098e-01_rb,1.3442e-01_rb,1.3574e-01_rb,1.1739e-01_rb, &
     &  9.5343e-02_rb,7.0224e-02_rb,5.3456e-02_rb,6.0206e-03_rb,5.0758e-03_rb,4.1906e-03_rb, &
     &  3.0431e-03_rb,2.0400e-03_rb,7.7689e-04_rb,1.1251e-04_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &1.7511e-07_rb,6.8135e-05_rb,1.1804e-04_rb,1.6931e-04_rb,2.2006e-04_rb,2.8275e-04_rb, &
     &3.5722e-04_rb,4.9530e-04_rb,3.5719e-04_rb/)
      kao(:, 2, 1, 1) = (/ &
     &1.9831e-07_rb,7.7459e-05_rb,1.3459e-04_rb,1.9095e-04_rb,2.5338e-04_rb,3.1724e-04_rb, &
     &4.1347e-04_rb,5.5823e-04_rb,4.1876e-04_rb/)
      kao(:, 3, 1, 1) = (/ &
     &2.2440e-07_rb,8.7653e-05_rb,1.5288e-04_rb,2.1636e-04_rb,2.8321e-04_rb,3.6387e-04_rb, &
     &4.5547e-04_rb,6.4036e-04_rb,4.9001e-04_rb/)
      kao(:, 4, 1, 1) = (/ &
     &2.4907e-07_rb,9.8921e-05_rb,1.7279e-04_rb,2.4495e-04_rb,3.1911e-04_rb,4.0317e-04_rb, &
     &5.1922e-04_rb,7.1060e-04_rb,5.5803e-04_rb/)
      kao(:, 5, 1, 1) = (/ &
     &2.7577e-07_rb,1.1088e-04_rb,1.9448e-04_rb,2.7670e-04_rb,3.6112e-04_rb,4.5291e-04_rb, &
     &5.7232e-04_rb,7.9647e-04_rb,6.3718e-04_rb/)
      kao(:, 1, 2, 1) = (/ &
     &2.1743e-07_rb,5.9697e-05_rb,1.0011e-04_rb,1.3856e-04_rb,1.7837e-04_rb,2.1937e-04_rb, &
     &2.7665e-04_rb,3.5934e-04_rb,2.6865e-04_rb/)
      kao(:, 2, 2, 1) = (/ &
     &2.4689e-07_rb,6.8199e-05_rb,1.1473e-04_rb,1.5826e-04_rb,2.0299e-04_rb,2.5265e-04_rb, &
     &3.0999e-04_rb,4.0941e-04_rb,3.2103e-04_rb/)
      kao(:, 3, 2, 1) = (/ &
     &2.7759e-07_rb,7.7428e-05_rb,1.3086e-04_rb,1.8044e-04_rb,2.2946e-04_rb,2.8434e-04_rb, &
     &3.5412e-04_rb,4.7313e-04_rb,3.7455e-04_rb/)
      kao(:, 4, 2, 1) = (/ &
     &3.1162e-07_rb,8.7312e-05_rb,1.4837e-04_rb,2.0486e-04_rb,2.6086e-04_rb,3.2026e-04_rb, &
     &3.9490e-04_rb,5.1882e-04_rb,4.3142e-04_rb/)
      kao(:, 5, 2, 1) = (/ &
     &3.4483e-07_rb,9.7902e-05_rb,1.6760e-04_rb,2.3163e-04_rb,2.9600e-04_rb,3.6332e-04_rb, &
     &4.4224e-04_rb,5.8485e-04_rb,4.9654e-04_rb/)
      kao(:, 1, 3, 1) = (/ &
     &3.5522e-07_rb,5.4633e-05_rb,8.9953e-05_rb,1.1946e-04_rb,1.4735e-04_rb,1.7547e-04_rb, &
     &2.0438e-04_rb,2.5064e-04_rb,1.9532e-04_rb/)
      kao(:, 2, 3, 1) = (/ &
     &4.0312e-07_rb,6.2965e-05_rb,1.0335e-04_rb,1.3746e-04_rb,1.6817e-04_rb,1.9939e-04_rb, &
     &2.3562e-04_rb,2.9335e-04_rb,2.3482e-04_rb/)
      kao(:, 3, 3, 1) = (/ &
     &4.5344e-07_rb,7.1817e-05_rb,1.1822e-04_rb,1.5709e-04_rb,1.9287e-04_rb,2.2736e-04_rb, &
     &2.6619e-04_rb,3.2521e-04_rb,2.7722e-04_rb/)
      kao(:, 4, 3, 1) = (/ &
     &5.1341e-07_rb,8.1197e-05_rb,1.3399e-04_rb,1.7909e-04_rb,2.2021e-04_rb,2.5936e-04_rb, &
     &3.0089e-04_rb,3.7007e-04_rb,3.2243e-04_rb/)
      kao(:, 5, 3, 1) = (/ &
     &5.7093e-07_rb,9.0935e-05_rb,1.5129e-04_rb,2.0303e-04_rb,2.4981e-04_rb,2.9482e-04_rb, &
     &3.4243e-04_rb,4.0910e-04_rb,3.7408e-04_rb/)
      kao(:, 1, 4, 1) = (/ &
     &5.9029e-07_rb,4.8269e-05_rb,8.1661e-05_rb,1.0687e-04_rb,1.2826e-04_rb,1.4610e-04_rb, &
     &1.6333e-04_rb,1.8670e-04_rb,1.4151e-04_rb/)
      kao(:, 2, 4, 1) = (/ &
     &6.7060e-07_rb,5.6349e-05_rb,9.4916e-05_rb,1.2365e-04_rb,1.4803e-04_rb,1.6776e-04_rb, &
     &1.8719e-04_rb,2.1036e-04_rb,1.7196e-04_rb/)
      kao(:, 3, 4, 1) = (/ &
     &7.5759e-07_rb,6.4869e-05_rb,1.0916e-04_rb,1.4206e-04_rb,1.6962e-04_rb,1.9348e-04_rb, &
     &2.1325e-04_rb,2.4238e-04_rb,2.0611e-04_rb/)
      kao(:, 4, 4, 1) = (/ &
     &8.5937e-07_rb,7.4063e-05_rb,1.2416e-04_rb,1.6207e-04_rb,1.9412e-04_rb,2.2126e-04_rb, &
     &2.4479e-04_rb,2.7146e-04_rb,2.4157e-04_rb/)
      kao(:, 5, 4, 1) = (/ &
     &9.5897e-07_rb,8.3687e-05_rb,1.3973e-04_rb,1.8364e-04_rb,2.2051e-04_rb,2.5234e-04_rb, &
     &2.7911e-04_rb,3.0928e-04_rb,2.8248e-04_rb/)
      kao(:, 1, 5, 1) = (/ &
     &9.1759e-07_rb,4.1947e-05_rb,7.0794e-05_rb,9.4885e-05_rb,1.1243e-04_rb,1.2599e-04_rb, &
     &1.3683e-04_rb,1.4293e-04_rb,1.0336e-04_rb/)
      kao(:, 2, 5, 1) = (/ &
     &1.0425e-06_rb,4.9113e-05_rb,8.3728e-05_rb,1.1140e-04_rb,1.3168e-04_rb,1.4605e-04_rb, &
     &1.5650e-04_rb,1.6563e-04_rb,1.2713e-04_rb/)
      kao(:, 3, 5, 1) = (/ &
     &1.1835e-06_rb,5.7263e-05_rb,9.7394e-05_rb,1.2924e-04_rb,1.5190e-04_rb,1.6881e-04_rb, &
     &1.8096e-04_rb,1.8906e-04_rb,1.5436e-04_rb/)
      kao(:, 4, 5, 1) = (/ &
     &1.3393e-06_rb,6.6178e-05_rb,1.1132e-04_rb,1.4759e-04_rb,1.7411e-04_rb,1.9396e-04_rb, &
     &2.0764e-04_rb,2.1628e-04_rb,1.8190e-04_rb/)
      kao(:, 5, 5, 1) = (/ &
     &1.4988e-06_rb,7.5103e-05_rb,1.2617e-04_rb,1.6721e-04_rb,1.9778e-04_rb,2.2104e-04_rb, &
     &2.3789e-04_rb,2.4796e-04_rb,2.1416e-04_rb/)
      kao(:, 1, 6, 1) = (/ &
     &1.3509e-06_rb,3.6190e-05_rb,6.0186e-05_rb,8.0089e-05_rb,9.7447e-05_rb,1.0817e-04_rb, &
     &1.1477e-04_rb,1.1459e-04_rb,7.5216e-05_rb/)
      kao(:, 2, 6, 1) = (/ &
     &1.5379e-06_rb,4.3048e-05_rb,7.1223e-05_rb,9.5429e-05_rb,1.1555e-04_rb,1.2778e-04_rb, &
     &1.3385e-04_rb,1.3550e-04_rb,9.3747e-05_rb/)
      kao(:, 3, 6, 1) = (/ &
     &1.7505e-06_rb,5.0649e-05_rb,8.3443e-05_rb,1.1225e-04_rb,1.3468e-04_rb,1.4925e-04_rb, &
     &1.5516e-04_rb,1.5522e-04_rb,1.1566e-04_rb/)
      kao(:, 4, 6, 1) = (/ &
     &1.9769e-06_rb,5.8613e-05_rb,9.7176e-05_rb,1.2952e-04_rb,1.5512e-04_rb,1.7221e-04_rb, &
     &1.7952e-04_rb,1.7884e-04_rb,1.3754e-04_rb/)
      kao(:, 5, 6, 1) = (/ &
     &2.2201e-06_rb,6.7356e-05_rb,1.1202e-04_rb,1.4825e-04_rb,1.7722e-04_rb,1.9650e-04_rb, &
     &2.0554e-04_rb,2.0495e-04_rb,1.6355e-04_rb/)
      kao(:, 1, 7, 1) = (/ &
     &2.1304e-06_rb,3.3089e-05_rb,5.1849e-05_rb,6.8730e-05_rb,8.2305e-05_rb,9.3472e-05_rb, &
     &9.9164e-05_rb,9.8631e-05_rb,5.9418e-05_rb/)
      kao(:, 2, 7, 1) = (/ &
     &2.4308e-06_rb,3.9636e-05_rb,6.2051e-05_rb,8.1968e-05_rb,9.8909e-05_rb,1.1256e-04_rb, &
     &1.1810e-04_rb,1.1396e-04_rb,7.5269e-05_rb/)
      kao(:, 3, 7, 1) = (/ &
     &2.7537e-06_rb,4.6821e-05_rb,7.3795e-05_rb,9.6812e-05_rb,1.1738e-04_rb,1.3274e-04_rb, &
     &1.3903e-04_rb,1.3282e-04_rb,9.2641e-05_rb/)
      kao(:, 4, 7, 1) = (/ &
     &3.1212e-06_rb,5.4439e-05_rb,8.6765e-05_rb,1.1374e-04_rb,1.3694e-04_rb,1.5474e-04_rb, &
     &1.6192e-04_rb,1.5375e-04_rb,1.1184e-04_rb/)
      kao(:, 5, 7, 1) = (/ &
     &3.5210e-06_rb,6.2611e-05_rb,9.9782e-05_rb,1.3214e-04_rb,1.5850e-04_rb,1.7752e-04_rb, &
     &1.8622e-04_rb,1.7734e-04_rb,1.3357e-04_rb/)
      kao(:, 1, 8, 1) = (/ &
     &4.1180e-06_rb,3.5356e-05_rb,4.9875e-05_rb,6.2112e-05_rb,7.3272e-05_rb,8.1665e-05_rb, &
     &8.7559e-05_rb,8.6370e-05_rb,4.9881e-05_rb/)
      kao(:, 2, 8, 1) = (/ &
     &4.6880e-06_rb,4.2308e-05_rb,6.0139e-05_rb,7.5108e-05_rb,8.8001e-05_rb,9.9269e-05_rb, &
     &1.0597e-04_rb,1.0409e-04_rb,6.3232e-05_rb/)
      kao(:, 3, 8, 1) = (/ &
     &5.2893e-06_rb,4.9705e-05_rb,7.1733e-05_rb,8.9997e-05_rb,1.0540e-04_rb,1.1807e-04_rb, &
     &1.2729e-04_rb,1.2367e-04_rb,7.9488e-05_rb/)
      kao(:, 4, 8, 1) = (/ &
     &5.9934e-06_rb,5.7629e-05_rb,8.3852e-05_rb,1.0617e-04_rb,1.2505e-04_rb,1.3960e-04_rb, &
     &1.5001e-04_rb,1.4519e-04_rb,9.7332e-05_rb/)
      kao(:, 5, 8, 1) = (/ &
     &6.8016e-06_rb,6.6193e-05_rb,9.7237e-05_rb,1.2349e-04_rb,1.4561e-04_rb,1.6359e-04_rb, &
     &1.7405e-04_rb,1.6782e-04_rb,1.1691e-04_rb/)
      kao(:, 1, 9, 1) = (/ &
     &1.5173e-05_rb,6.0073e-05_rb,7.5522e-05_rb,8.4608e-05_rb,8.9391e-05_rb,9.1045e-05_rb, &
     &9.0015e-05_rb,8.5089e-05_rb,6.2916e-05_rb/)
      kao(:, 2, 9, 1) = (/ &
     &1.7118e-05_rb,7.2165e-05_rb,9.0975e-05_rb,1.0209e-04_rb,1.0823e-04_rb,1.1031e-04_rb, &
     &1.0910e-04_rb,1.0450e-04_rb,8.0378e-05_rb/)
      kao(:, 3, 9, 1) = (/ &
     &1.9333e-05_rb,8.4779e-05_rb,1.0715e-04_rb,1.2042e-04_rb,1.2865e-04_rb,1.3251e-04_rb, &
     &1.3198e-04_rb,1.2575e-04_rb,1.0185e-04_rb/)
      kao(:, 4, 9, 1) = (/ &
     &2.2020e-05_rb,9.7885e-05_rb,1.2402e-04_rb,1.4031e-04_rb,1.5091e-04_rb,1.5615e-04_rb, &
     &1.5775e-04_rb,1.5010e-04_rb,1.2495e-04_rb/)
      kao(:, 5, 9, 1) = (/ &
     &2.4990e-05_rb,1.1102e-04_rb,1.4167e-04_rb,1.6181e-04_rb,1.7511e-04_rb,1.8231e-04_rb, &
     &1.8406e-04_rb,1.7738e-04_rb,1.4891e-04_rb/)
      kao(:, 1,10, 1) = (/ &
     &6.3843e-05_rb,1.3785e-04_rb,1.5635e-04_rb,1.6287e-04_rb,1.6037e-04_rb,1.5248e-04_rb, &
     &1.3606e-04_rb,1.0761e-04_rb,1.0144e-04_rb/)
      kao(:, 2,10, 1) = (/ &
     &7.2156e-05_rb,1.6111e-04_rb,1.8530e-04_rb,1.9419e-04_rb,1.9482e-04_rb,1.8548e-04_rb, &
     &1.6571e-04_rb,1.3139e-04_rb,1.2609e-04_rb/)
      kao(:, 3,10, 1) = (/ &
     &8.1633e-05_rb,1.8623e-04_rb,2.1467e-04_rb,2.2993e-04_rb,2.3146e-04_rb,2.2096e-04_rb, &
     &1.9656e-04_rb,1.5757e-04_rb,1.5350e-04_rb/)
      kao(:, 4,10, 1) = (/ &
     &9.2413e-05_rb,2.1393e-04_rb,2.4898e-04_rb,2.6688e-04_rb,2.6838e-04_rb,2.5715e-04_rb, &
     &2.2901e-04_rb,1.8640e-04_rb,1.8257e-04_rb/)
      kao(:, 5,10, 1) = (/ &
     &1.0426e-04_rb,2.4538e-04_rb,2.8330e-04_rb,3.0298e-04_rb,3.0752e-04_rb,2.9416e-04_rb, &
     &2.6511e-04_rb,2.1835e-04_rb,2.1288e-04_rb/)
      kao(:, 1,11, 1) = (/ &
     &9.9914e-05_rb,1.8204e-04_rb,1.9923e-04_rb,2.0432e-04_rb,1.9733e-04_rb,1.8201e-04_rb, &
     &1.5904e-04_rb,1.2299e-04_rb,1.0910e-04_rb/)
      kao(:, 2,11, 1) = (/ &
     &1.1199e-04_rb,2.1265e-04_rb,2.3389e-04_rb,2.4089e-04_rb,2.3442e-04_rb,2.2069e-04_rb, &
     &1.9371e-04_rb,1.4882e-04_rb,1.3411e-04_rb/)
      kao(:, 3,11, 1) = (/ &
     &1.2687e-04_rb,2.4499e-04_rb,2.7026e-04_rb,2.7909e-04_rb,2.7675e-04_rb,2.6044e-04_rb, &
     &2.2915e-04_rb,1.7718e-04_rb,1.6026e-04_rb/)
      kao(:, 4,11, 1) = (/ &
     &1.4261e-04_rb,2.8101e-04_rb,3.1141e-04_rb,3.2255e-04_rb,3.1948e-04_rb,3.0150e-04_rb, &
     &2.6612e-04_rb,2.0752e-04_rb,1.8601e-04_rb/)
      kao(:, 5,11, 1) = (/ &
     &1.5789e-04_rb,3.1929e-04_rb,3.5513e-04_rb,3.6602e-04_rb,3.6282e-04_rb,3.4410e-04_rb, &
     &3.0577e-04_rb,2.4097e-04_rb,2.1352e-04_rb/)
      kao(:, 1,12, 1) = (/ &
     &1.1122e-04_rb,1.9022e-04_rb,2.0568e-04_rb,2.0798e-04_rb,2.0013e-04_rb,1.8369e-04_rb, &
     &1.5756e-04_rb,1.2037e-04_rb,1.0098e-04_rb/)
      kao(:, 2,12, 1) = (/ &
     &1.2500e-04_rb,2.2156e-04_rb,2.4046e-04_rb,2.4347e-04_rb,2.3764e-04_rb,2.2116e-04_rb, &
     &1.9272e-04_rb,1.4643e-04_rb,1.2390e-04_rb/)
      kao(:, 3,12, 1) = (/ &
     &1.4056e-04_rb,2.5489e-04_rb,2.7764e-04_rb,2.8122e-04_rb,2.7905e-04_rb,2.5886e-04_rb, &
     &2.2636e-04_rb,1.7352e-04_rb,1.4664e-04_rb/)
      kao(:, 4,12, 1) = (/ &
     &1.5858e-04_rb,2.9021e-04_rb,3.1841e-04_rb,3.2551e-04_rb,3.1903e-04_rb,2.9848e-04_rb, &
     &2.6254e-04_rb,2.0265e-04_rb,1.7061e-04_rb/)
      kao(:, 5,12, 1) = (/ &
     &1.7398e-04_rb,3.2750e-04_rb,3.6207e-04_rb,3.6880e-04_rb,3.6058e-04_rb,3.3907e-04_rb, &
     &2.9970e-04_rb,2.3401e-04_rb,1.9575e-04_rb/)
      kao(:, 1,13, 1) = (/ &
     &9.9982e-05_rb,1.6786e-04_rb,1.8157e-04_rb,1.8311e-04_rb,1.7732e-04_rb,1.6257e-04_rb, &
     &1.4107e-04_rb,1.0684e-04_rb,8.8282e-05_rb/)
      kao(:, 2,13, 1) = (/ &
     &1.1208e-04_rb,1.9489e-04_rb,2.1207e-04_rb,2.1420e-04_rb,2.0856e-04_rb,1.9501e-04_rb, &
     &1.6942e-04_rb,1.2896e-04_rb,1.0706e-04_rb/)
      kao(:, 3,13, 1) = (/ &
     &1.2580e-04_rb,2.2390e-04_rb,2.4368e-04_rb,2.4899e-04_rb,2.4426e-04_rb,2.2695e-04_rb, &
     &1.9796e-04_rb,1.5273e-04_rb,1.2657e-04_rb/)
      kao(:, 4,13, 1) = (/ &
     &1.4115e-04_rb,2.5382e-04_rb,2.8079e-04_rb,2.8499e-04_rb,2.7843e-04_rb,2.6072e-04_rb, &
     &2.2903e-04_rb,1.7905e-04_rb,1.4711e-04_rb/)
      kao(:, 5,13, 1) = (/ &
     &1.5465e-04_rb,2.8798e-04_rb,3.1774e-04_rb,3.2258e-04_rb,3.1431e-04_rb,2.9430e-04_rb, &
     &2.6133e-04_rb,2.0499e-04_rb,1.6802e-04_rb/)
      kao(:, 1, 1, 2) = (/ &
     &3.7683e-07_rb,1.9808e-04_rb,3.6685e-04_rb,5.3916e-04_rb,7.1167e-04_rb,8.9309e-04_rb, &
     &1.1251e-03_rb,1.6060e-03_rb,1.3216e-03_rb/)
      kao(:, 2, 1, 2) = (/ &
     &4.2181e-07_rb,2.2081e-04_rb,4.0843e-04_rb,6.0005e-04_rb,7.9854e-04_rb,1.0235e-03_rb, &
     &1.2747e-03_rb,1.7730e-03_rb,1.5072e-03_rb/)
      kao(:, 3, 1, 2) = (/ &
     &4.6205e-07_rb,2.4503e-04_rb,4.5395e-04_rb,6.6471e-04_rb,8.8503e-04_rb,1.1345e-03_rb, &
     &1.4567e-03_rb,1.9804e-03_rb,1.6670e-03_rb/)
      kao(:, 4, 1, 2) = (/ &
     &5.1491e-07_rb,2.7069e-04_rb,5.0378e-04_rb,7.3699e-04_rb,9.7626e-04_rb,1.2446e-03_rb, &
     &1.5952e-03_rb,2.2188e-03_rb,1.8463e-03_rb/)
      kao(:, 5, 1, 2) = (/ &
     &5.5977e-07_rb,2.9608e-04_rb,5.5719e-04_rb,8.1557e-04_rb,1.0806e-03_rb,1.3738e-03_rb, &
     &1.7497e-03_rb,2.4313e-03_rb,2.0562e-03_rb/)
      kao(:, 1, 2, 2) = (/ &
     &4.7123e-07_rb,1.6943e-04_rb,3.0287e-04_rb,4.3680e-04_rb,5.7665e-04_rb,7.0994e-04_rb, &
     &8.5911e-04_rb,1.1308e-03_rb,1.0172e-03_rb/)
      kao(:, 2, 2, 2) = (/ &
     &5.2931e-07_rb,1.8961e-04_rb,3.3976e-04_rb,4.8868e-04_rb,6.4337e-04_rb,8.0862e-04_rb, &
     &9.8935e-04_rb,1.2742e-03_rb,1.1602e-03_rb/)
      kao(:, 3, 2, 2) = (/ &
     &5.8844e-07_rb,2.1120e-04_rb,3.7935e-04_rb,5.4606e-04_rb,7.1722e-04_rb,8.9758e-04_rb, &
     &1.1168e-03_rb,1.4348e-03_rb,1.3002e-03_rb/)
      kao(:, 4, 2, 2) = (/ &
     &6.4390e-07_rb,2.3336e-04_rb,4.2266e-04_rb,6.0899e-04_rb,7.9911e-04_rb,9.9511e-04_rb, &
     &1.2304e-03_rb,1.6357e-03_rb,1.4539e-03_rb/)
      kao(:, 5, 2, 2) = (/ &
     &7.0717e-07_rb,2.5573e-04_rb,4.6939e-04_rb,6.7801e-04_rb,8.8876e-04_rb,1.1052e-03_rb, &
     &1.3595e-03_rb,1.7810e-03_rb,1.6268e-03_rb/)
      kao(:, 1, 3, 2) = (/ &
     &7.7382e-07_rb,1.5592e-04_rb,2.6004e-04_rb,3.5917e-04_rb,4.5597e-04_rb,5.6221e-04_rb, &
     &6.6191e-04_rb,7.8092e-04_rb,7.5320e-04_rb/)
      kao(:, 2, 3, 2) = (/ &
     &8.7904e-07_rb,1.7462e-04_rb,2.9301e-04_rb,4.0553e-04_rb,5.1639e-04_rb,6.3236e-04_rb, &
     &7.5999e-04_rb,8.9592e-04_rb,8.6804e-04_rb/)
      kao(:, 3, 3, 2) = (/ &
     &9.8281e-07_rb,1.9263e-04_rb,3.2870e-04_rb,4.5631e-04_rb,5.8039e-04_rb,7.0848e-04_rb, &
     &8.4867e-04_rb,1.0369e-03_rb,9.8463e-04_rb/)
      kao(:, 4, 3, 2) = (/ &
     &1.0791e-06_rb,2.1196e-04_rb,3.6740e-04_rb,5.1125e-04_rb,6.5128e-04_rb,7.9536e-04_rb, &
     &9.4873e-04_rb,1.1498e-03_rb,1.1137e-03_rb/)
      kao(:, 5, 3, 2) = (/ &
     &1.1811e-06_rb,2.3314e-04_rb,4.0444e-04_rb,5.7096e-04_rb,7.2953e-04_rb,8.9019e-04_rb, &
     &1.0598e-03_rb,1.2765e-03_rb,1.2563e-03_rb/)
      kao(:, 1, 4, 2) = (/ &
     &1.2931e-06_rb,1.5459e-04_rb,2.3712e-04_rb,3.1151e-04_rb,3.8051e-04_rb,4.4732e-04_rb, &
     &5.1939e-04_rb,5.8139e-04_rb,5.5932e-04_rb/)
      kao(:, 2, 4, 2) = (/ &
     &1.4961e-06_rb,1.6924e-04_rb,2.6743e-04_rb,3.5309e-04_rb,4.3277e-04_rb,5.1005e-04_rb, &
     &5.8720e-04_rb,6.8557e-04_rb,6.5449e-04_rb/)
      kao(:, 3, 4, 2) = (/ &
     &1.6687e-06_rb,1.8668e-04_rb,2.9946e-04_rb,3.9815e-04_rb,4.8978e-04_rb,5.7712e-04_rb, &
     &6.6568e-04_rb,7.7293e-04_rb,7.4803e-04_rb/)
      kao(:, 4, 4, 2) = (/ &
     &1.8491e-06_rb,2.0517e-04_rb,3.2982e-04_rb,4.4741e-04_rb,5.5202e-04_rb,6.5205e-04_rb, &
     &7.5198e-04_rb,8.7064e-04_rb,8.5540e-04_rb/)
      kao(:, 5, 4, 2) = (/ &
     &2.0324e-06_rb,2.2162e-04_rb,3.6469e-04_rb,4.9544e-04_rb,6.1968e-04_rb,7.3378e-04_rb, &
     &8.4769e-04_rb,9.7614e-04_rb,9.7197e-04_rb/)
      kao(:, 1, 5, 2) = (/ &
     &2.0065e-06_rb,1.5150e-04_rb,2.2968e-04_rb,2.8386e-04_rb,3.3168e-04_rb,3.7581e-04_rb, &
     &4.1489e-04_rb,4.5675e-04_rb,4.1358e-04_rb/)
      kao(:, 2, 5, 2) = (/ &
     &2.3252e-06_rb,1.6854e-04_rb,2.5480e-04_rb,3.2081e-04_rb,3.7754e-04_rb,4.3026e-04_rb, &
     &4.7606e-04_rb,5.2820e-04_rb,4.9368e-04_rb/)
      kao(:, 3, 5, 2) = (/ &
     &2.6085e-06_rb,1.8331e-04_rb,2.8033e-04_rb,3.6056e-04_rb,4.2825e-04_rb,4.8928e-04_rb, &
     &5.4309e-04_rb,5.9807e-04_rb,5.6993e-04_rb/)
      kao(:, 4, 5, 2) = (/ &
     &2.9132e-06_rb,1.9761e-04_rb,3.1015e-04_rb,3.9882e-04_rb,4.8412e-04_rb,5.5460e-04_rb, &
     &6.1782e-04_rb,6.7980e-04_rb,6.5837e-04_rb/)
      kao(:, 5, 5, 2) = (/ &
     &3.2361e-06_rb,2.1434e-04_rb,3.4118e-04_rb,4.4180e-04_rb,5.3752e-04_rb,6.2588e-04_rb, &
     &6.9930e-04_rb,7.7113e-04_rb,7.5327e-04_rb/)
      kao(:, 1, 6, 2) = (/ &
     &2.9230e-06_rb,1.3806e-04_rb,2.1835e-04_rb,2.7095e-04_rb,3.0194e-04_rb,3.2758e-04_rb, &
     &3.4587e-04_rb,3.6408e-04_rb,3.0468e-04_rb/)
      kao(:, 2, 6, 2) = (/ &
     &3.4021e-06_rb,1.5365e-04_rb,2.4578e-04_rb,3.0400e-04_rb,3.4286e-04_rb,3.7318e-04_rb, &
     &3.9861e-04_rb,4.1438e-04_rb,3.7021e-04_rb/)
      kao(:, 3, 6, 2) = (/ &
     &3.8620e-06_rb,1.7051e-04_rb,2.7334e-04_rb,3.3438e-04_rb,3.8715e-04_rb,4.2432e-04_rb, &
     &4.5662e-04_rb,4.7772e-04_rb,4.3260e-04_rb/)
      kao(:, 4, 6, 2) = (/ &
     &4.2870e-06_rb,1.8887e-04_rb,2.9668e-04_rb,3.7137e-04_rb,4.2886e-04_rb,4.8133e-04_rb, &
     &5.2053e-04_rb,5.4689e-04_rb,5.0258e-04_rb/)
      kao(:, 5, 6, 2) = (/ &
     &4.8423e-06_rb,2.0662e-04_rb,3.1977e-04_rb,4.1089e-04_rb,4.7609e-04_rb,5.3742e-04_rb, &
     &5.9126e-04_rb,6.2337e-04_rb,5.7823e-04_rb/)
      kao(:, 1, 7, 2) = (/ &
     &4.5404e-06_rb,1.3143e-04_rb,2.0727e-04_rb,2.6052e-04_rb,2.9635e-04_rb,3.0743e-04_rb, &
     &3.0825e-04_rb,2.9703e-04_rb,2.2197e-04_rb/)
      kao(:, 2, 7, 2) = (/ &
     &5.2640e-06_rb,1.4772e-04_rb,2.3068e-04_rb,2.9687e-04_rb,3.3184e-04_rb,3.4986e-04_rb, &
     &3.5249e-04_rb,3.4543e-04_rb,2.7330e-04_rb/)
      kao(:, 3, 7, 2) = (/ &
     &6.0466e-06_rb,1.6470e-04_rb,2.5696e-04_rb,3.3234e-04_rb,3.6914e-04_rb,3.9626e-04_rb, &
     &4.0246e-04_rb,3.9902e-04_rb,3.2329e-04_rb/)
      kao(:, 4, 7, 2) = (/ &
     &6.7565e-06_rb,1.8107e-04_rb,2.8421e-04_rb,3.6286e-04_rb,4.1183e-04_rb,4.3876e-04_rb, &
     &4.5748e-04_rb,4.5835e-04_rb,3.7913e-04_rb/)
      kao(:, 5, 7, 2) = (/ &
     &7.6474e-06_rb,1.9867e-04_rb,3.1475e-04_rb,3.9310e-04_rb,4.5438e-04_rb,4.8957e-04_rb, &
     &5.1385e-04_rb,5.2320e-04_rb,4.4016e-04_rb/)
      kao(:, 1, 8, 2) = (/ &
     &8.6520e-06_rb,1.4110e-04_rb,2.0496e-04_rb,2.5993e-04_rb,2.9742e-04_rb,3.1821e-04_rb, &
     &3.1456e-04_rb,2.7649e-04_rb,1.8485e-04_rb/)
      kao(:, 2, 8, 2) = (/ &
     &1.0009e-05_rb,1.5517e-04_rb,2.3206e-04_rb,2.9171e-04_rb,3.4405e-04_rb,3.6414e-04_rb, &
     &3.5939e-04_rb,3.1713e-04_rb,2.2424e-04_rb/)
      kao(:, 3, 8, 2) = (/ &
     &1.1509e-05_rb,1.7205e-04_rb,2.6018e-04_rb,3.2636e-04_rb,3.8244e-04_rb,4.1241e-04_rb, &
     &4.0304e-04_rb,3.6377e-04_rb,2.6821e-04_rb/)
      kao(:, 4, 8, 2) = (/ &
     &1.3117e-05_rb,1.8959e-04_rb,2.8945e-04_rb,3.6544e-04_rb,4.2147e-04_rb,4.5788e-04_rb, &
     &4.4922e-04_rb,4.1527e-04_rb,3.1806e-04_rb/)
      kao(:, 5, 8, 2) = (/ &
     &1.4790e-05_rb,2.0902e-04_rb,3.1840e-04_rb,4.0603e-04_rb,4.6521e-04_rb,4.9945e-04_rb, &
     &5.0220e-04_rb,4.7045e-04_rb,3.7190e-04_rb/)
      kao(:, 1, 9, 2) = (/ &
     &3.0998e-05_rb,2.0637e-04_rb,2.7947e-04_rb,3.3530e-04_rb,3.6480e-04_rb,3.8205e-04_rb, &
     &3.8653e-04_rb,3.4762e-04_rb,1.8242e-04_rb/)
      kao(:, 2, 9, 2) = (/ &
     &3.6096e-05_rb,2.3076e-04_rb,3.1240e-04_rb,3.7284e-04_rb,4.1471e-04_rb,4.3432e-04_rb, &
     &4.4125e-04_rb,4.0372e-04_rb,2.2363e-04_rb/)
      kao(:, 3, 9, 2) = (/ &
     &4.2035e-05_rb,2.5734e-04_rb,3.4822e-04_rb,4.1425e-04_rb,4.6570e-04_rb,4.9153e-04_rb, &
     &4.9534e-04_rb,4.6243e-04_rb,2.6686e-04_rb/)
      kao(:, 4, 9, 2) = (/ &
     &4.8029e-05_rb,2.8335e-04_rb,3.8624e-04_rb,4.5968e-04_rb,5.1473e-04_rb,5.5196e-04_rb, &
     &5.5517e-04_rb,5.1807e-04_rb,3.1577e-04_rb/)
      kao(:, 5, 9, 2) = (/ &
     &5.5331e-05_rb,3.0966e-04_rb,4.2837e-04_rb,5.0793e-04_rb,5.6875e-04_rb,6.0971e-04_rb, &
     &6.2368e-04_rb,5.6511e-04_rb,3.7171e-04_rb/)
      kao(:, 1,10, 2) = (/ &
     &1.2964e-04_rb,4.0632e-04_rb,5.1358e-04_rb,5.5581e-04_rb,5.6980e-04_rb,5.6112e-04_rb, &
     &5.2892e-04_rb,4.3728e-04_rb,2.8865e-04_rb/)
      kao(:, 2,10, 2) = (/ &
     &1.5095e-04_rb,4.5953e-04_rb,5.7199e-04_rb,6.2596e-04_rb,6.3988e-04_rb,6.3023e-04_rb, &
     &5.9124e-04_rb,5.0100e-04_rb,3.5610e-04_rb/)
      kao(:, 3,10, 2) = (/ &
     &1.7573e-04_rb,5.1815e-04_rb,6.3409e-04_rb,6.9698e-04_rb,7.1541e-04_rb,7.0519e-04_rb, &
     &6.6061e-04_rb,5.7112e-04_rb,4.2862e-04_rb/)
      kao(:, 4,10, 2) = (/ &
     &2.0254e-04_rb,5.7408e-04_rb,7.0140e-04_rb,7.6485e-04_rb,7.9509e-04_rb,7.8487e-04_rb, &
     &7.3767e-04_rb,6.4340e-04_rb,5.0196e-04_rb/)
      kao(:, 5,10, 2) = (/ &
     &2.3735e-04_rb,6.3435e-04_rb,7.7477e-04_rb,8.4107e-04_rb,8.7369e-04_rb,8.7290e-04_rb, &
     &8.2035e-04_rb,7.1353e-04_rb,5.7378e-04_rb/)
      kao(:, 1,11, 2) = (/ &
     &2.0611e-04_rb,5.0579e-04_rb,6.0906e-04_rb,6.6598e-04_rb,6.7463e-04_rb,6.4325e-04_rb, &
     &5.7358e-04_rb,4.6594e-04_rb,3.7105e-04_rb/)
      kao(:, 2,11, 2) = (/ &
     &2.4168e-04_rb,5.7457e-04_rb,6.8930e-04_rb,7.4695e-04_rb,7.5788e-04_rb,7.2324e-04_rb, &
     &6.4676e-04_rb,5.2943e-04_rb,4.4283e-04_rb/)
      kao(:, 3,11, 2) = (/ &
     &2.7855e-04_rb,6.5109e-04_rb,7.7336e-04_rb,8.3624e-04_rb,8.3721e-04_rb,8.0652e-04_rb, &
     &7.2648e-04_rb,5.9132e-04_rb,5.1523e-04_rb/)
      kao(:, 4,11, 2) = (/ &
     &3.2934e-04_rb,7.2843e-04_rb,8.5922e-04_rb,9.2211e-04_rb,9.2829e-04_rb,8.9207e-04_rb, &
     &8.1359e-04_rb,6.5969e-04_rb,5.9146e-04_rb/)
      kao(:, 5,11, 2) = (/ &
     &3.7624e-04_rb,8.0764e-04_rb,9.4816e-04_rb,1.0176e-03_rb,1.0267e-03_rb,9.8589e-04_rb, &
     &9.0003e-04_rb,7.3360e-04_rb,6.6373e-04_rb/)
      kao(:, 1,12, 2) = (/ &
     &2.3690e-04_rb,5.2345e-04_rb,6.1583e-04_rb,6.6206e-04_rb,6.7308e-04_rb,6.4411e-04_rb, &
     &5.7108e-04_rb,4.4594e-04_rb,3.6966e-04_rb/)
      kao(:, 2,12, 2) = (/ &
     &2.7636e-04_rb,5.9693e-04_rb,6.9824e-04_rb,7.5086e-04_rb,7.5372e-04_rb,7.2129e-04_rb, &
     &6.4015e-04_rb,5.0417e-04_rb,4.3519e-04_rb/)
      kao(:, 3,12, 2) = (/ &
     &3.2475e-04_rb,6.7274e-04_rb,7.7976e-04_rb,8.4185e-04_rb,8.4051e-04_rb,8.0110e-04_rb, &
     &7.2036e-04_rb,5.6771e-04_rb,5.0173e-04_rb/)
      kao(:, 4,12, 2) = (/ &
     &3.7742e-04_rb,7.5354e-04_rb,8.6958e-04_rb,9.2971e-04_rb,9.3567e-04_rb,8.9034e-04_rb, &
     &7.9942e-04_rb,6.3511e-04_rb,5.6743e-04_rb/)
      kao(:, 5,12, 2) = (/ &
     &4.2656e-04_rb,8.3460e-04_rb,9.6197e-04_rb,1.0276e-03_rb,1.0348e-03_rb,9.8795e-04_rb, &
     &8.8414e-04_rb,7.0917e-04_rb,6.3181e-04_rb/)
      kao(:, 1,13, 2) = (/ &
     &2.1559e-04_rb,4.7034e-04_rb,5.5318e-04_rb,5.8764e-04_rb,5.9246e-04_rb,5.7342e-04_rb, &
     &5.0467e-04_rb,3.9184e-04_rb,3.4198e-04_rb/)
      kao(:, 2,13, 2) = (/ &
     &2.5152e-04_rb,5.3422e-04_rb,6.2195e-04_rb,6.6659e-04_rb,6.7272e-04_rb,6.3853e-04_rb, &
     &5.7068e-04_rb,4.4436e-04_rb,3.9456e-04_rb/)
      kao(:, 3,13, 2) = (/ &
     &3.0302e-04_rb,6.0127e-04_rb,6.9437e-04_rb,7.4686e-04_rb,7.5249e-04_rb,7.1183e-04_rb, &
     &6.4004e-04_rb,5.0162e-04_rb,4.4714e-04_rb/)
      kao(:, 4,13, 2) = (/ &
     &3.4242e-04_rb,6.7081e-04_rb,7.7273e-04_rb,8.2774e-04_rb,8.3533e-04_rb,7.9499e-04_rb, &
     &7.1004e-04_rb,5.6394e-04_rb,5.0105e-04_rb/)
      kao(:, 5,13, 2) = (/ &
     &3.8220e-04_rb,7.4575e-04_rb,8.5383e-04_rb,9.1037e-04_rb,9.2303e-04_rb,8.8587e-04_rb, &
     &7.8559e-04_rb,6.3207e-04_rb,5.5475e-04_rb/)
      kao(:, 1, 1, 3) = (/ &
     &5.3797e-07_rb,4.8188e-04_rb,8.9729e-04_rb,1.3200e-03_rb,1.7770e-03_rb,2.3291e-03_rb, &
     &3.1122e-03_rb,4.4511e-03_rb,3.5150e-03_rb/)
      kao(:, 2, 1, 3) = (/ &
     &6.1262e-07_rb,5.2615e-04_rb,9.8845e-04_rb,1.4517e-03_rb,1.9408e-03_rb,2.5148e-03_rb, &
     &3.2963e-03_rb,4.8584e-03_rb,3.8321e-03_rb/)
      kao(:, 3, 1, 3) = (/ &
     &7.0635e-07_rb,5.6647e-04_rb,1.0753e-03_rb,1.5925e-03_rb,2.1392e-03_rb,2.7441e-03_rb, &
     &3.5300e-03_rb,5.0419e-03_rb,4.2299e-03_rb/)
      kao(:, 4, 1, 3) = (/ &
     &7.6766e-07_rb,6.0727e-04_rb,1.1623e-03_rb,1.7247e-03_rb,2.3255e-03_rb,3.0054e-03_rb, &
     &3.8553e-03_rb,5.3163e-03_rb,4.6081e-03_rb/)
      kao(:, 5, 1, 3) = (/ &
     &8.3851e-07_rb,6.4796e-04_rb,1.2484e-03_rb,1.8617e-03_rb,2.5127e-03_rb,3.2521e-03_rb, &
     &4.1787e-03_rb,5.7407e-03_rb,4.9828e-03_rb/)
      kao(:, 1, 2, 3) = (/ &
     &6.9858e-07_rb,4.1220e-04_rb,7.5871e-04_rb,1.0893e-03_rb,1.4250e-03_rb,1.8069e-03_rb, &
     &2.3218e-03_rb,3.2208e-03_rb,2.7451e-03_rb/)
      kao(:, 2, 2, 3) = (/ &
     &8.1006e-07_rb,4.4690e-04_rb,8.3751e-04_rb,1.2083e-03_rb,1.5781e-03_rb,1.9758e-03_rb, &
     &2.4966e-03_rb,3.4638e-03_rb,3.0268e-03_rb/)
      kao(:, 3, 2, 3) = (/ &
     &9.0719e-07_rb,4.8245e-04_rb,9.0580e-04_rb,1.3167e-03_rb,1.7362e-03_rb,2.1788e-03_rb, &
     &2.7148e-03_rb,3.6613e-03_rb,3.3476e-03_rb/)
      kao(:, 4, 2, 3) = (/ &
     &9.9070e-07_rb,5.1202e-04_rb,9.7859e-04_rb,1.4272e-03_rb,1.8822e-03_rb,2.3784e-03_rb, &
     &2.9761e-03_rb,3.9189e-03_rb,3.6534e-03_rb/)
      kao(:, 5, 2, 3) = (/ &
     &1.0800e-06_rb,5.4986e-04_rb,1.0443e-03_rb,1.5370e-03_rb,2.0346e-03_rb,2.5754e-03_rb, &
     &3.2289e-03_rb,4.2581e-03_rb,3.9665e-03_rb/)
      kao(:, 1, 3, 3) = (/ &
     &1.1956e-06_rb,3.6533e-04_rb,6.4857e-04_rb,9.1797e-04_rb,1.1753e-03_rb,1.4236e-03_rb, &
     &1.7165e-03_rb,2.2362e-03_rb,2.0894e-03_rb/)
      kao(:, 2, 3, 3) = (/ &
     &1.3685e-06_rb,3.9659e-04_rb,7.1084e-04_rb,1.0164e-03_rb,1.3082e-03_rb,1.5859e-03_rb, &
     &1.8841e-03_rb,2.3586e-03_rb,2.3274e-03_rb/)
      kao(:, 3, 3, 3) = (/ &
     &1.5093e-06_rb,4.2397e-04_rb,7.7009e-04_rb,1.1048e-03_rb,1.4344e-03_rb,1.7565e-03_rb, &
     &2.0870e-03_rb,2.5514e-03_rb,2.5907e-03_rb/)
      kao(:, 4, 3, 3) = (/ &
     &1.6732e-06_rb,4.5580e-04_rb,8.2534e-04_rb,1.1953e-03_rb,1.5533e-03_rb,1.9057e-03_rb, &
     &2.2804e-03_rb,2.8031e-03_rb,2.8387e-03_rb/)
      kao(:, 5, 3, 3) = (/ &
     &1.8304e-06_rb,4.8851e-04_rb,8.8340e-04_rb,1.2735e-03_rb,1.6704e-03_rb,2.0642e-03_rb, &
     &2.4727e-03_rb,3.0561e-03_rb,3.0939e-03_rb/)
      kao(:, 1, 4, 3) = (/ &
     &2.0440e-06_rb,3.4190e-04_rb,5.7528e-04_rb,7.8555e-04_rb,9.8576e-04_rb,1.1789e-03_rb, &
     &1.3557e-03_rb,1.6128e-03_rb,1.5891e-03_rb/)
      kao(:, 2, 4, 3) = (/ &
     &2.2803e-06_rb,3.6626e-04_rb,6.2613e-04_rb,8.6621e-04_rb,1.0945e-03_rb,1.3150e-03_rb, &
     &1.5191e-03_rb,1.7412e-03_rb,1.7827e-03_rb/)
      kao(:, 3, 4, 3) = (/ &
     &2.5344e-06_rb,3.9307e-04_rb,6.7511e-04_rb,9.4372e-04_rb,1.1954e-03_rb,1.4474e-03_rb, &
     &1.6882e-03_rb,1.9262e-03_rb,2.0030e-03_rb/)
      kao(:, 4, 4, 3) = (/ &
     &2.8636e-06_rb,4.1974e-04_rb,7.2769e-04_rb,1.0153e-03_rb,1.2977e-03_rb,1.5716e-03_rb, &
     &1.8427e-03_rb,2.1315e-03_rb,2.2084e-03_rb/)
      kao(:, 5, 4, 3) = (/ &
     &3.0668e-06_rb,4.5207e-04_rb,7.8267e-04_rb,1.0885e-03_rb,1.3854e-03_rb,1.6897e-03_rb, &
     &1.9964e-03_rb,2.3147e-03_rb,2.4195e-03_rb/)
      kao(:, 1, 5, 3) = (/ &
     &3.1319e-06_rb,3.2418e-04_rb,5.2433e-04_rb,6.9885e-04_rb,8.4707e-04_rb,9.8232e-04_rb, &
     &1.1112e-03_rb,1.2410e-03_rb,1.2226e-03_rb/)
      kao(:, 2, 5, 3) = (/ &
     &3.5295e-06_rb,3.4929e-04_rb,5.6868e-04_rb,7.6274e-04_rb,9.3852e-04_rb,1.0972e-03_rb, &
     &1.2472e-03_rb,1.3812e-03_rb,1.3798e-03_rb/)
      kao(:, 3, 5, 3) = (/ &
     &3.9745e-06_rb,3.7641e-04_rb,6.1261e-04_rb,8.2611e-04_rb,1.0255e-03_rb,1.2065e-03_rb, &
     &1.3836e-03_rb,1.5454e-03_rb,1.5563e-03_rb/)
      kao(:, 4, 5, 3) = (/ &
     &4.4389e-06_rb,4.0602e-04_rb,6.5760e-04_rb,8.9247e-04_rb,1.1052e-03_rb,1.3151e-03_rb, &
     &1.5074e-03_rb,1.7081e-03_rb,1.7334e-03_rb/)
      kao(:, 5, 5, 3) = (/ &
     &4.7663e-06_rb,4.3568e-04_rb,7.0547e-04_rb,9.6253e-04_rb,1.1901e-03_rb,1.4105e-03_rb, &
     &1.6319e-03_rb,1.8559e-03_rb,1.9044e-03_rb/)
      kao(:, 1, 6, 3) = (/ &
     &4.5500e-06_rb,3.2085e-04_rb,4.8959e-04_rb,6.2597e-04_rb,7.4855e-04_rb,8.4386e-04_rb, &
     &9.2463e-04_rb,9.8196e-04_rb,9.3322e-04_rb/)
      kao(:, 2, 6, 3) = (/ &
     &5.1832e-06_rb,3.4894e-04_rb,5.2646e-04_rb,6.8892e-04_rb,8.2642e-04_rb,9.4419e-04_rb, &
     &1.0344e-03_rb,1.1120e-03_rb,1.0567e-03_rb/)
      kao(:, 3, 6, 3) = (/ &
     &5.7454e-06_rb,3.7624e-04_rb,5.6596e-04_rb,7.4193e-04_rb,8.9866e-04_rb,1.0339e-03_rb, &
     &1.1478e-03_rb,1.2434e-03_rb,1.1989e-03_rb/)
      kao(:, 4, 6, 3) = (/ &
     &6.4183e-06_rb,4.0049e-04_rb,6.1119e-04_rb,7.9917e-04_rb,9.7241e-04_rb,1.1206e-03_rb, &
     &1.2562e-03_rb,1.3721e-03_rb,1.3554e-03_rb/)
      kao(:, 5, 6, 3) = (/ &
     &6.9324e-06_rb,4.2668e-04_rb,6.6038e-04_rb,8.5368e-04_rb,1.0486e-03_rb,1.2116e-03_rb, &
     &1.3599e-03_rb,1.5001e-03_rb,1.5043e-03_rb/)
      kao(:, 1, 7, 3) = (/ &
     &7.2797e-06_rb,3.0677e-04_rb,4.8071e-04_rb,5.9971e-04_rb,6.8742e-04_rb,7.5672e-04_rb, &
     &8.0124e-04_rb,8.1058e-04_rb,7.0953e-04_rb/)
      kao(:, 2, 7, 3) = (/ &
     &8.1177e-06_rb,3.3920e-04_rb,5.2455e-04_rb,6.4489e-04_rb,7.6436e-04_rb,8.4338e-04_rb, &
     &9.0408e-04_rb,9.2147e-04_rb,8.0482e-04_rb/)
      kao(:, 3, 7, 3) = (/ &
     &8.9509e-06_rb,3.6425e-04_rb,5.6797e-04_rb,6.9667e-04_rb,8.2046e-04_rb,9.2590e-04_rb, &
     &9.9874e-04_rb,1.0317e-03_rb,9.2280e-04_rb/)
      kao(:, 4, 7, 3) = (/ &
     &9.9962e-06_rb,3.8975e-04_rb,6.1216e-04_rb,7.5517e-04_rb,8.8215e-04_rb,1.0000e-03_rb, &
     &1.0895e-03_rb,1.1382e-03_rb,1.0492e-03_rb/)
      kao(:, 5, 7, 3) = (/ &
     &1.0669e-05_rb,4.1752e-04_rb,6.5205e-04_rb,8.1765e-04_rb,9.5018e-04_rb,1.0745e-03_rb, &
     &1.1767e-03_rb,1.2498e-03_rb,1.1757e-03_rb/)
      kao(:, 1, 8, 3) = (/ &
     &1.3269e-05_rb,3.0208e-04_rb,4.8542e-04_rb,6.1720e-04_rb,7.0751e-04_rb,7.5832e-04_rb, &
     &7.5956e-04_rb,7.2658e-04_rb,5.2753e-04_rb/)
      kao(:, 2, 8, 3) = (/ &
     &1.5282e-05_rb,3.2807e-04_rb,5.3968e-04_rb,6.8467e-04_rb,7.6733e-04_rb,8.2175e-04_rb, &
     &8.4708e-04_rb,8.2655e-04_rb,6.1209e-04_rb/)
      kao(:, 3, 8, 3) = (/ &
     &1.6919e-05_rb,3.5455e-04_rb,5.8673e-04_rb,7.5006e-04_rb,8.3417e-04_rb,8.8820e-04_rb, &
     &9.3120e-04_rb,9.2583e-04_rb,7.0621e-04_rb/)
      kao(:, 4, 8, 3) = (/ &
     &1.8616e-05_rb,3.8664e-04_rb,6.2653e-04_rb,8.0927e-04_rb,9.0712e-04_rb,9.6236e-04_rb, &
     &1.0056e-03_rb,1.0184e-03_rb,8.0931e-04_rb/)
      kao(:, 5, 8, 3) = (/ &
     &2.0327e-05_rb,4.2321e-04_rb,6.7354e-04_rb,8.6755e-04_rb,9.7906e-04_rb,1.0424e-03_rb, &
     &1.0862e-03_rb,1.0982e-03_rb,9.1316e-04_rb/)
      kao(:, 1, 9, 3) = (/ &
     &5.0114e-05_rb,4.1653e-04_rb,5.9063e-04_rb,7.3539e-04_rb,8.5123e-04_rb,9.2178e-04_rb, &
     &9.4309e-04_rb,8.5391e-04_rb,5.1499e-04_rb/)
      kao(:, 2, 9, 3) = (/ &
     &5.7273e-05_rb,4.5656e-04_rb,6.4069e-04_rb,7.9699e-04_rb,9.4296e-04_rb,1.0365e-03_rb, &
     &1.0446e-03_rb,9.3220e-04_rb,6.0883e-04_rb/)
      kao(:, 3, 9, 3) = (/ &
     &6.4035e-05_rb,4.9861e-04_rb,7.0653e-04_rb,8.6985e-04_rb,1.0116e-03_rb,1.1468e-03_rb, &
     &1.1543e-03_rb,1.0092e-03_rb,7.0553e-04_rb/)
      kao(:, 4, 9, 3) = (/ &
     &7.2188e-05_rb,5.4617e-04_rb,7.8095e-04_rb,9.4772e-04_rb,1.1003e-03_rb,1.2322e-03_rb, &
     &1.2562e-03_rb,1.0988e-03_rb,7.9526e-04_rb/)
      kao(:, 5, 9, 3) = (/ &
     &7.9550e-05_rb,6.0184e-04_rb,8.5464e-04_rb,1.0416e-03_rb,1.1956e-03_rb,1.3190e-03_rb, &
     &1.3528e-03_rb,1.1984e-03_rb,8.7997e-04_rb/)
      kao(:, 1,10, 3) = (/ &
     &2.3306e-04_rb,7.9772e-04_rb,9.8566e-04_rb,1.1183e-03_rb,1.1737e-03_rb,1.1902e-03_rb, &
     &1.1793e-03_rb,1.0698e-03_rb,6.6407e-04_rb/)
      kao(:, 2,10, 3) = (/ &
     &2.6771e-04_rb,8.8993e-04_rb,1.0956e-03_rb,1.2259e-03_rb,1.2971e-03_rb,1.3020e-03_rb, &
     &1.2785e-03_rb,1.2036e-03_rb,7.5718e-04_rb/)
      kao(:, 3,10, 3) = (/ &
     &3.0294e-04_rb,9.8041e-04_rb,1.2123e-03_rb,1.3475e-03_rb,1.4168e-03_rb,1.4273e-03_rb, &
     &1.4006e-03_rb,1.3282e-03_rb,8.5838e-04_rb/)
      kao(:, 4,10, 3) = (/ &
     &3.4186e-04_rb,1.0838e-03_rb,1.3329e-03_rb,1.4855e-03_rb,1.5514e-03_rb,1.5775e-03_rb, &
     &1.5302e-03_rb,1.4297e-03_rb,9.7732e-04_rb/)
      kao(:, 5,10, 3) = (/ &
     &3.7940e-04_rb,1.2006e-03_rb,1.4657e-03_rb,1.6333e-03_rb,1.7070e-03_rb,1.7269e-03_rb, &
     &1.6832e-03_rb,1.5552e-03_rb,1.1108e-03_rb/)
      kao(:, 1,11, 3) = (/ &
     &4.1370e-04_rb,1.0481e-03_rb,1.2382e-03_rb,1.3193e-03_rb,1.3405e-03_rb,1.3128e-03_rb, &
     &1.2136e-03_rb,1.0530e-03_rb,7.9947e-04_rb/)
      kao(:, 2,11, 3) = (/ &
     &4.6551e-04_rb,1.1702e-03_rb,1.3813e-03_rb,1.4636e-03_rb,1.4831e-03_rb,1.4472e-03_rb, &
     &1.3317e-03_rb,1.1447e-03_rb,9.1716e-04_rb/)
      kao(:, 3,11, 3) = (/ &
     &5.2337e-04_rb,1.3114e-03_rb,1.5380e-03_rb,1.6111e-03_rb,1.6417e-03_rb,1.5910e-03_rb, &
     &1.4641e-03_rb,1.2585e-03_rb,1.0575e-03_rb/)
      kao(:, 4,11, 3) = (/ &
     &5.8236e-04_rb,1.4612e-03_rb,1.6981e-03_rb,1.7829e-03_rb,1.8015e-03_rb,1.7452e-03_rb, &
     &1.6069e-03_rb,1.3761e-03_rb,1.2010e-03_rb/)
      kao(:, 5,11, 3) = (/ &
     &6.7131e-04_rb,1.6254e-03_rb,1.8811e-03_rb,1.9666e-03_rb,1.9744e-03_rb,1.9137e-03_rb, &
     &1.7677e-03_rb,1.5105e-03_rb,1.3428e-03_rb/)
      kao(:, 1,12, 3) = (/ &
     &5.0548e-04_rb,1.1217e-03_rb,1.2964e-03_rb,1.3785e-03_rb,1.3692e-03_rb,1.2935e-03_rb, &
     &1.1705e-03_rb,9.6704e-04_rb,8.2680e-04_rb/)
      kao(:, 2,12, 3) = (/ &
     &5.7320e-04_rb,1.2560e-03_rb,1.4612e-03_rb,1.5290e-03_rb,1.5133e-03_rb,1.4345e-03_rb, &
     &1.2992e-03_rb,1.0595e-03_rb,9.5961e-04_rb/)
      kao(:, 3,12, 3) = (/ &
     &6.3853e-04_rb,1.4125e-03_rb,1.6378e-03_rb,1.6918e-03_rb,1.6663e-03_rb,1.5909e-03_rb, &
     &1.4283e-03_rb,1.1648e-03_rb,1.0789e-03_rb/)
      kao(:, 4,12, 3) = (/ &
     &7.1367e-04_rb,1.5888e-03_rb,1.8194e-03_rb,1.8745e-03_rb,1.8358e-03_rb,1.7471e-03_rb, &
     &1.5691e-03_rb,1.2862e-03_rb,1.2039e-03_rb/)
      kao(:, 5,12, 3) = (/ &
     &8.2207e-04_rb,1.7742e-03_rb,2.0103e-03_rb,2.0603e-03_rb,2.0193e-03_rb,1.9194e-03_rb, &
     &1.7244e-03_rb,1.4199e-03_rb,1.3514e-03_rb/)
      kao(:, 1,13, 3) = (/ &
     &5.0420e-04_rb,1.0317e-03_rb,1.1965e-03_rb,1.2627e-03_rb,1.2425e-03_rb,1.1623e-03_rb, &
     &1.0397e-03_rb,8.4556e-04_rb,7.6156e-04_rb/)
      kao(:, 2,13, 3) = (/ &
     &5.6808e-04_rb,1.1670e-03_rb,1.3463e-03_rb,1.4009e-03_rb,1.3701e-03_rb,1.2956e-03_rb, &
     &1.1567e-03_rb,9.3024e-04_rb,8.8050e-04_rb/)
      kao(:, 3,13, 3) = (/ &
     &6.1838e-04_rb,1.3214e-03_rb,1.5180e-03_rb,1.5502e-03_rb,1.5020e-03_rb,1.4361e-03_rb, &
     &1.2780e-03_rb,1.0272e-03_rb,9.9967e-04_rb/)
      kao(:, 4,13, 3) = (/ &
     &7.0527e-04_rb,1.4908e-03_rb,1.6794e-03_rb,1.7102e-03_rb,1.6621e-03_rb,1.5793e-03_rb, &
     &1.4142e-03_rb,1.1355e-03_rb,1.1267e-03_rb/)
      kao(:, 5,13, 3) = (/ &
     &8.1247e-04_rb,1.6340e-03_rb,1.8549e-03_rb,1.8870e-03_rb,1.8374e-03_rb,1.7298e-03_rb, &
     &1.5606e-03_rb,1.2534e-03_rb,1.2510e-03_rb/)
      kao(:, 1, 1, 4) = (/ &
     &7.9349e-07_rb,1.0718e-03_rb,2.1076e-03_rb,3.1773e-03_rb,4.3493e-03_rb,5.6505e-03_rb, &
     &7.2449e-03_rb,1.0124e-02_rb,8.6807e-03_rb/)
      kao(:, 2, 1, 4) = (/ &
     &7.9171e-07_rb,1.1405e-03_rb,2.2617e-03_rb,3.4209e-03_rb,4.6727e-03_rb,6.0666e-03_rb, &
     &7.8396e-03_rb,1.0702e-02_rb,9.3285e-03_rb/)
      kao(:, 3, 1, 4) = (/ &
     &8.3534e-07_rb,1.2022e-03_rb,2.3697e-03_rb,3.6079e-03_rb,4.9651e-03_rb,6.4864e-03_rb, &
     &8.3654e-03_rb,1.1513e-02_rb,9.9193e-03_rb/)
      kao(:, 4, 1, 4) = (/ &
     &9.5267e-07_rb,1.2723e-03_rb,2.4978e-03_rb,3.7943e-03_rb,5.1994e-03_rb,6.8025e-03_rb, &
     &8.8715e-03_rb,1.2190e-02_rb,1.0375e-02_rb/)
      kao(:, 5, 1, 4) = (/ &
     &1.1458e-06_rb,1.3554e-03_rb,2.6484e-03_rb,4.0109e-03_rb,5.4856e-03_rb,7.1578e-03_rb, &
     &9.2726e-03_rb,1.2904e-02_rb,1.0954e-02_rb/)
      kao(:, 1, 2, 4) = (/ &
     &9.1454e-07_rb,9.0539e-04_rb,1.7315e-03_rb,2.5817e-03_rb,3.4611e-03_rb,4.4469e-03_rb, &
     &5.5705e-03_rb,7.4448e-03_rb,6.8782e-03_rb/)
      kao(:, 2, 2, 4) = (/ &
     &9.5187e-07_rb,9.6041e-04_rb,1.8588e-03_rb,2.7755e-03_rb,3.7306e-03_rb,4.7775e-03_rb, &
     &6.0319e-03_rb,7.9894e-03_rb,7.4238e-03_rb/)
      kao(:, 3, 2, 4) = (/ &
     &1.1150e-06_rb,1.0164e-03_rb,1.9573e-03_rb,2.9264e-03_rb,3.9480e-03_rb,5.1214e-03_rb, &
     &6.4611e-03_rb,8.5702e-03_rb,7.8654e-03_rb/)
      kao(:, 4, 2, 4) = (/ &
     &1.3385e-06_rb,1.0876e-03_rb,2.0659e-03_rb,3.0814e-03_rb,4.1574e-03_rb,5.3507e-03_rb, &
     &6.8052e-03_rb,9.1287e-03_rb,8.2842e-03_rb/)
      kao(:, 5, 2, 4) = (/ &
     &1.6022e-06_rb,1.1568e-03_rb,2.2052e-03_rb,3.2703e-03_rb,4.4002e-03_rb,5.6510e-03_rb, &
     &7.1684e-03_rb,9.6207e-03_rb,8.7667e-03_rb/)
      kao(:, 1, 3, 4) = (/ &
     &1.5574e-06_rb,7.6371e-04_rb,1.4483e-03_rb,2.1006e-03_rb,2.7598e-03_rb,3.4620e-03_rb, &
     &4.1959e-03_rb,5.2056e-03_rb,5.3344e-03_rb/)
      kao(:, 2, 3, 4) = (/ &
     &1.7591e-06_rb,8.1114e-04_rb,1.5471e-03_rb,2.2761e-03_rb,2.9860e-03_rb,3.7307e-03_rb, &
     &4.5489e-03_rb,5.7315e-03_rb,5.7929e-03_rb/)
      kao(:, 3, 3, 4) = (/ &
     &2.0958e-06_rb,8.7335e-04_rb,1.6413e-03_rb,2.3988e-03_rb,3.1565e-03_rb,3.9689e-03_rb, &
     &4.8920e-03_rb,6.1466e-03_rb,6.1501e-03_rb/)
      kao(:, 4, 3, 4) = (/ &
     &2.4948e-06_rb,9.3193e-04_rb,1.7550e-03_rb,2.5427e-03_rb,3.3388e-03_rb,4.1838e-03_rb, &
     &5.1384e-03_rb,6.5670e-03_rb,6.5006e-03_rb/)
      kao(:, 5, 3, 4) = (/ &
     &2.9808e-06_rb,9.8734e-04_rb,1.8752e-03_rb,2.7256e-03_rb,3.5562e-03_rb,4.4297e-03_rb, &
     &5.4345e-03_rb,6.8804e-03_rb,6.9100e-03_rb/)
      kao(:, 1, 4, 4) = (/ &
     &2.8339e-06_rb,6.5550e-04_rb,1.2204e-03_rb,1.7505e-03_rb,2.2569e-03_rb,2.7626e-03_rb, &
     &3.2755e-03_rb,3.8524e-03_rb,4.1315e-03_rb/)
      kao(:, 2, 4, 4) = (/ &
     &3.2988e-06_rb,7.0845e-04_rb,1.2966e-03_rb,1.8754e-03_rb,2.4581e-03_rb,3.0020e-03_rb, &
     &3.5639e-03_rb,4.2448e-03_rb,4.5130e-03_rb/)
      kao(:, 3, 4, 4) = (/ &
     &3.9491e-06_rb,7.6198e-04_rb,1.3871e-03_rb,1.9942e-03_rb,2.6023e-03_rb,3.1978e-03_rb, &
     &3.8488e-03_rb,4.5802e-03_rb,4.8443e-03_rb/)
      kao(:, 4, 4, 4) = (/ &
     &4.6425e-06_rb,8.1982e-04_rb,1.4902e-03_rb,2.1380e-03_rb,2.7710e-03_rb,3.3974e-03_rb, &
     &4.0487e-03_rb,4.8659e-03_rb,5.1413e-03_rb/)
      kao(:, 5, 4, 4) = (/ &
     &5.6265e-06_rb,8.8017e-04_rb,1.5882e-03_rb,2.2893e-03_rb,2.9763e-03_rb,3.6352e-03_rb, &
     &4.2916e-03_rb,5.1439e-03_rb,5.4622e-03_rb/)
      kao(:, 1, 5, 4) = (/ &
     &4.8447e-06_rb,5.8594e-04_rb,1.0296e-03_rb,1.4656e-03_rb,1.8696e-03_rb,2.2619e-03_rb, &
     &2.6388e-03_rb,3.0084e-03_rb,3.1950e-03_rb/)
      kao(:, 2, 5, 4) = (/ &
     &5.5680e-06_rb,6.3350e-04_rb,1.1054e-03_rb,1.5640e-03_rb,2.0148e-03_rb,2.4702e-03_rb, &
     &2.8772e-03_rb,3.3130e-03_rb,3.5193e-03_rb/)
      kao(:, 3, 5, 4) = (/ &
     &6.5723e-06_rb,6.8615e-04_rb,1.1977e-03_rb,1.6753e-03_rb,2.1497e-03_rb,2.6254e-03_rb, &
     &3.0969e-03_rb,3.5912e-03_rb,3.8422e-03_rb/)
      kao(:, 4, 5, 4) = (/ &
     &7.8530e-06_rb,7.4277e-04_rb,1.2884e-03_rb,1.8101e-03_rb,2.3101e-03_rb,2.7992e-03_rb, &
     &3.2952e-03_rb,3.8151e-03_rb,4.0689e-03_rb/)
      kao(:, 5, 5, 4) = (/ &
     &9.4527e-06_rb,8.0105e-04_rb,1.3871e-03_rb,1.9371e-03_rb,2.4908e-03_rb,3.0121e-03_rb, &
     &3.5288e-03_rb,4.0532e-03_rb,4.3486e-03_rb/)
      kao(:, 1, 6, 4) = (/ &
     &7.9687e-06_rb,5.4507e-04_rb,8.9513e-04_rb,1.2382e-03_rb,1.5589e-03_rb,1.8510e-03_rb, &
     &2.1398e-03_rb,2.3860e-03_rb,2.4635e-03_rb/)
      kao(:, 2, 6, 4) = (/ &
     &8.8712e-06_rb,5.9098e-04_rb,9.7039e-04_rb,1.3235e-03_rb,1.6755e-03_rb,2.0285e-03_rb, &
     &2.3501e-03_rb,2.6411e-03_rb,2.7555e-03_rb/)
      kao(:, 3, 6, 4) = (/ &
     &1.0300e-05_rb,6.3947e-04_rb,1.0497e-03_rb,1.4366e-03_rb,1.7944e-03_rb,2.1608e-03_rb, &
     &2.5184e-03_rb,2.8809e-03_rb,3.0211e-03_rb/)
      kao(:, 4, 6, 4) = (/ &
     &1.2171e-05_rb,6.9277e-04_rb,1.1370e-03_rb,1.5510e-03_rb,1.9450e-03_rb,2.3217e-03_rb, &
     &2.6890e-03_rb,3.0692e-03_rb,3.2284e-03_rb/)
      kao(:, 5, 6, 4) = (/ &
     &1.4564e-05_rb,7.4600e-04_rb,1.2318e-03_rb,1.6801e-03_rb,2.0948e-03_rb,2.5115e-03_rb, &
     &2.8923e-03_rb,3.2756e-03_rb,3.4483e-03_rb/)
      kao(:, 1, 7, 4) = (/ &
     &1.3302e-05_rb,5.5386e-04_rb,8.3382e-04_rb,1.0993e-03_rb,1.3564e-03_rb,1.5663e-03_rb, &
     &1.7522e-03_rb,1.9212e-03_rb,1.8928e-03_rb/)
      kao(:, 2, 7, 4) = (/ &
     &1.5137e-05_rb,6.0362e-04_rb,9.0303e-04_rb,1.1816e-03_rb,1.4350e-03_rb,1.6949e-03_rb, &
     &1.9391e-03_rb,2.1345e-03_rb,2.1400e-03_rb/)
      kao(:, 3, 7, 4) = (/ &
     &1.7393e-05_rb,6.5556e-04_rb,9.7760e-04_rb,1.2752e-03_rb,1.5638e-03_rb,1.8142e-03_rb, &
     &2.0760e-03_rb,2.3419e-03_rb,2.3618e-03_rb/)
      kao(:, 4, 7, 4) = (/ &
     &2.0190e-05_rb,7.0877e-04_rb,1.0583e-03_rb,1.3832e-03_rb,1.6934e-03_rb,1.9720e-03_rb, &
     &2.2273e-03_rb,2.4896e-03_rb,2.5682e-03_rb/)
      kao(:, 5, 7, 4) = (/ &
     &2.3865e-05_rb,7.6246e-04_rb,1.1476e-03_rb,1.5003e-03_rb,1.8325e-03_rb,2.1363e-03_rb, &
     &2.4216e-03_rb,2.6647e-03_rb,2.7377e-03_rb/)
      kao(:, 1, 8, 4) = (/ &
     &2.8723e-05_rb,5.8785e-04_rb,8.8654e-04_rb,1.0769e-03_rb,1.2579e-03_rb,1.4310e-03_rb, &
     &1.5394e-03_rb,1.5987e-03_rb,1.4479e-03_rb/)
      kao(:, 2, 8, 4) = (/ &
     &3.1951e-05_rb,6.5934e-04_rb,9.7493e-04_rb,1.1672e-03_rb,1.3594e-03_rb,1.5243e-03_rb, &
     &1.6749e-03_rb,1.7779e-03_rb,1.6535e-03_rb/)
      kao(:, 3, 8, 4) = (/ &
     &3.6028e-05_rb,7.3349e-04_rb,1.0597e-03_rb,1.2704e-03_rb,1.4675e-03_rb,1.6499e-03_rb, &
     &1.7965e-03_rb,1.9280e-03_rb,1.8443e-03_rb/)
      kao(:, 4, 8, 4) = (/ &
     &4.1972e-05_rb,7.9901e-04_rb,1.1534e-03_rb,1.3845e-03_rb,1.5898e-03_rb,1.7819e-03_rb, &
     &1.9610e-03_rb,2.0652e-03_rb,2.0278e-03_rb/)
      kao(:, 5, 8, 4) = (/ &
     &4.8818e-05_rb,8.6068e-04_rb,1.2469e-03_rb,1.5029e-03_rb,1.7252e-03_rb,1.9359e-03_rb, &
     &2.1153e-03_rb,2.2493e-03_rb,2.1613e-03_rb/)
      kao(:, 1, 9, 4) = (/ &
     &1.2151e-04_rb,7.9394e-04_rb,1.1551e-03_rb,1.4165e-03_rb,1.6142e-03_rb,1.6912e-03_rb, &
     &1.6602e-03_rb,1.6194e-03_rb,1.1991e-03_rb/)
      kao(:, 2, 9, 4) = (/ &
     &1.3408e-04_rb,8.8305e-04_rb,1.2970e-03_rb,1.6023e-03_rb,1.7938e-03_rb,1.8665e-03_rb, &
     &1.8031e-03_rb,1.7264e-03_rb,1.3714e-03_rb/)
      kao(:, 3, 9, 4) = (/ &
     &1.5040e-04_rb,9.7901e-04_rb,1.4287e-03_rb,1.7867e-03_rb,2.0033e-03_rb,2.0339e-03_rb, &
     &1.9620e-03_rb,1.8611e-03_rb,1.5284e-03_rb/)
      kao(:, 4, 9, 4) = (/ &
     &1.6901e-04_rb,1.0844e-03_rb,1.5638e-03_rb,1.9651e-03_rb,2.1812e-03_rb,2.2146e-03_rb, &
     &2.1459e-03_rb,2.0086e-03_rb,1.6589e-03_rb/)
      kao(:, 5, 9, 4) = (/ &
     &1.9165e-04_rb,1.1941e-03_rb,1.7106e-03_rb,2.1260e-03_rb,2.3624e-03_rb,2.4142e-03_rb, &
     &2.3405e-03_rb,2.1792e-03_rb,1.8085e-03_rb/)
      kao(:, 1,10, 4) = (/ &
     &5.9868e-04_rb,1.5993e-03_rb,1.9364e-03_rb,2.1129e-03_rb,2.2524e-03_rb,2.3246e-03_rb, &
     &2.2594e-03_rb,2.0146e-03_rb,1.3568e-03_rb/)
      kao(:, 2,10, 4) = (/ &
     &6.6752e-04_rb,1.7567e-03_rb,2.1277e-03_rb,2.3435e-03_rb,2.4977e-03_rb,2.6109e-03_rb, &
     &2.5681e-03_rb,2.2608e-03_rb,1.5177e-03_rb/)
      kao(:, 3,10, 4) = (/ &
     &7.4901e-04_rb,1.9228e-03_rb,2.3324e-03_rb,2.5957e-03_rb,2.7706e-03_rb,2.8949e-03_rb, &
     &2.8715e-03_rb,2.4740e-03_rb,1.6681e-03_rb/)
      kao(:, 4,10, 4) = (/ &
     &8.4268e-04_rb,2.1035e-03_rb,2.5651e-03_rb,2.8675e-03_rb,3.0553e-03_rb,3.1760e-03_rb, &
     &3.1745e-03_rb,2.7207e-03_rb,1.8575e-03_rb/)
      kao(:, 5,10, 4) = (/ &
     &9.4245e-04_rb,2.2879e-03_rb,2.8080e-03_rb,3.1494e-03_rb,3.3540e-03_rb,3.4813e-03_rb, &
     &3.4487e-03_rb,2.9536e-03_rb,2.0691e-03_rb/)
      kao(:, 1,11, 4) = (/ &
     &1.0805e-03_rb,2.1148e-03_rb,2.4220e-03_rb,2.5310e-03_rb,2.5389e-03_rb,2.4625e-03_rb, &
     &2.3509e-03_rb,2.0401e-03_rb,1.4417e-03_rb/)
      kao(:, 2,11, 4) = (/ &
     &1.2248e-03_rb,2.3043e-03_rb,2.6421e-03_rb,2.7805e-03_rb,2.8051e-03_rb,2.7290e-03_rb, &
     &2.6298e-03_rb,2.3094e-03_rb,1.6718e-03_rb/)
      kao(:, 3,11, 4) = (/ &
     &1.3919e-03_rb,2.5018e-03_rb,2.8694e-03_rb,3.0522e-03_rb,3.0921e-03_rb,3.0140e-03_rb, &
     &2.9024e-03_rb,2.5763e-03_rb,1.8644e-03_rb/)
      kao(:, 4,11, 4) = (/ &
     &1.5591e-03_rb,2.7193e-03_rb,3.1319e-03_rb,3.3417e-03_rb,3.3992e-03_rb,3.3292e-03_rb, &
     &3.1865e-03_rb,2.8388e-03_rb,2.0955e-03_rb/)
      kao(:, 5,11, 4) = (/ &
     &1.7150e-03_rb,2.9473e-03_rb,3.3980e-03_rb,3.6551e-03_rb,3.7158e-03_rb,3.6708e-03_rb, &
     &3.4907e-03_rb,3.0872e-03_rb,2.3494e-03_rb/)
      kao(:, 1,12, 4) = (/ &
     &1.3897e-03_rb,2.3284e-03_rb,2.5562e-03_rb,2.6114e-03_rb,2.5756e-03_rb,2.4579e-03_rb, &
     &2.2328e-03_rb,1.8907e-03_rb,1.4460e-03_rb/)
      kao(:, 2,12, 4) = (/ &
     &1.5697e-03_rb,2.5462e-03_rb,2.7773e-03_rb,2.8739e-03_rb,2.8515e-03_rb,2.7127e-03_rb, &
     &2.4702e-03_rb,2.1317e-03_rb,1.6710e-03_rb/)
      kao(:, 3,12, 4) = (/ &
     &1.7564e-03_rb,2.7736e-03_rb,3.0368e-03_rb,3.1575e-03_rb,3.1336e-03_rb,2.9932e-03_rb, &
     &2.7316e-03_rb,2.3689e-03_rb,1.9165e-03_rb/)
      kao(:, 4,12, 4) = (/ &
     &1.9373e-03_rb,3.0028e-03_rb,3.3026e-03_rb,3.4473e-03_rb,3.4197e-03_rb,3.2885e-03_rb, &
     &3.0105e-03_rb,2.5975e-03_rb,2.1734e-03_rb/)
      kao(:, 5,12, 4) = (/ &
     &2.1103e-03_rb,3.2861e-03_rb,3.6126e-03_rb,3.7579e-03_rb,3.7472e-03_rb,3.5983e-03_rb, &
     &3.3137e-03_rb,2.8243e-03_rb,2.4241e-03_rb/)
      kao(:, 1,13, 4) = (/ &
     &1.4034e-03_rb,2.2073e-03_rb,2.3423e-03_rb,2.3940e-03_rb,2.3352e-03_rb,2.2289e-03_rb, &
     &1.9861e-03_rb,1.6453e-03_rb,1.3641e-03_rb/)
      kao(:, 2,13, 4) = (/ &
     &1.5468e-03_rb,2.3993e-03_rb,2.5715e-03_rb,2.6353e-03_rb,2.5905e-03_rb,2.4689e-03_rb, &
     &2.2035e-03_rb,1.8516e-03_rb,1.5595e-03_rb/)
      kao(:, 3,13, 4) = (/ &
     &1.7054e-03_rb,2.6002e-03_rb,2.8041e-03_rb,2.9001e-03_rb,2.8726e-03_rb,2.7020e-03_rb, &
     &2.4437e-03_rb,2.0505e-03_rb,1.7798e-03_rb/)
      kao(:, 4,13, 4) = (/ &
     &1.8746e-03_rb,2.8344e-03_rb,3.0758e-03_rb,3.1886e-03_rb,3.1543e-03_rb,2.9591e-03_rb, &
     &2.7010e-03_rb,2.2473e-03_rb,2.0185e-03_rb/)
      kao(:, 5,13, 4) = (/ &
     &2.0493e-03_rb,3.1290e-03_rb,3.3711e-03_rb,3.5047e-03_rb,3.4626e-03_rb,3.2529e-03_rb, &
     &2.9617e-03_rb,2.4466e-03_rb,2.2855e-03_rb/)
      kao(:, 1, 1, 5) = (/ &
     &2.7288e-06_rb,2.0772e-03_rb,4.1339e-03_rb,6.3040e-03_rb,8.6169e-03_rb,1.1216e-02_rb, &
     &1.4425e-02_rb,1.9774e-02_rb,1.7219e-02_rb/)
      kao(:, 2, 1, 5) = (/ &
     &3.1343e-06_rb,2.2547e-03_rb,4.4747e-03_rb,6.8269e-03_rb,9.3305e-03_rb,1.2134e-02_rb, &
     &1.5662e-02_rb,2.1511e-02_rb,1.8646e-02_rb/)
      kao(:, 3, 1, 5) = (/ &
     &3.5230e-06_rb,2.4421e-03_rb,4.8659e-03_rb,7.3919e-03_rb,1.0064e-02_rb,1.3080e-02_rb, &
     &1.6864e-02_rb,2.3200e-02_rb,2.0105e-02_rb/)
      kao(:, 4, 1, 5) = (/ &
     &3.8970e-06_rb,2.6162e-03_rb,5.2335e-03_rb,7.9722e-03_rb,1.0895e-02_rb,1.4129e-02_rb, &
     &1.8093e-02_rb,2.4994e-02_rb,2.1780e-02_rb/)
      kao(:, 5, 1, 5) = (/ &
     &4.2153e-06_rb,2.7842e-03_rb,5.5880e-03_rb,8.5221e-03_rb,1.1648e-02_rb,1.5136e-02_rb, &
     &1.9474e-02_rb,2.6742e-02_rb,2.3280e-02_rb/)
      kao(:, 1, 2, 5) = (/ &
     &3.8170e-06_rb,1.7299e-03_rb,3.3770e-03_rb,5.0561e-03_rb,6.8488e-03_rb,8.8137e-03_rb, &
     &1.1142e-02_rb,1.4709e-02_rb,1.3674e-02_rb/)
      kao(:, 2, 2, 5) = (/ &
     &4.3252e-06_rb,1.8890e-03_rb,3.6509e-03_rb,5.4769e-03_rb,7.4269e-03_rb,9.5439e-03_rb, &
     &1.2081e-02_rb,1.6033e-02_rb,1.4831e-02_rb/)
      kao(:, 3, 2, 5) = (/ &
     &4.7646e-06_rb,2.0435e-03_rb,3.9774e-03_rb,5.9664e-03_rb,8.0622e-03_rb,1.0295e-02_rb, &
     &1.3026e-02_rb,1.7373e-02_rb,1.6093e-02_rb/)
      kao(:, 4, 2, 5) = (/ &
     &5.2162e-06_rb,2.1898e-03_rb,4.2814e-03_rb,6.4462e-03_rb,8.7251e-03_rb,1.1197e-02_rb, &
     &1.4093e-02_rb,1.8642e-02_rb,1.7418e-02_rb/)
      kao(:, 5, 2, 5) = (/ &
     &5.6571e-06_rb,2.3346e-03_rb,4.5674e-03_rb,6.8936e-03_rb,9.3452e-03_rb,1.2004e-02_rb, &
     &1.5146e-02_rb,2.0072e-02_rb,1.8662e-02_rb/)
      kao(:, 1, 3, 5) = (/ &
     &6.7841e-06_rb,1.4874e-03_rb,2.7805e-03_rb,4.0760e-03_rb,5.3909e-03_rb,6.7517e-03_rb, &
     &8.3542e-03_rb,1.0487e-02_rb,1.0633e-02_rb/)
      kao(:, 2, 3, 5) = (/ &
     &7.5652e-06_rb,1.6242e-03_rb,3.0372e-03_rb,4.4125e-03_rb,5.8428e-03_rb,7.3572e-03_rb, &
     &9.0599e-03_rb,1.1418e-02_rb,1.1564e-02_rb/)
      kao(:, 3, 3, 5) = (/ &
     &8.3338e-06_rb,1.7491e-03_rb,3.3027e-03_rb,4.8269e-03_rb,6.3750e-03_rb,7.9902e-03_rb, &
     &9.8070e-03_rb,1.2345e-02_rb,1.2611e-02_rb/)
      kao(:, 4, 3, 5) = (/ &
     &9.1322e-06_rb,1.8750e-03_rb,3.5479e-03_rb,5.2057e-03_rb,6.8969e-03_rb,8.6842e-03_rb, &
     &1.0693e-02_rb,1.3305e-02_rb,1.3678e-02_rb/)
      kao(:, 5, 3, 5) = (/ &
     &9.9531e-06_rb,2.0097e-03_rb,3.7981e-03_rb,5.5735e-03_rb,7.4004e-03_rb,9.3447e-03_rb, &
     &1.1509e-02_rb,1.4423e-02_rb,1.4706e-02_rb/)
      kao(:, 1, 4, 5) = (/ &
     &1.2103e-05_rb,1.3043e-03_rb,2.3537e-03_rb,3.3772e-03_rb,4.3942e-03_rb,5.3862e-03_rb, &
     &6.4402e-03_rb,7.7504e-03_rb,8.3034e-03_rb/)
      kao(:, 2, 4, 5) = (/ &
     &1.3453e-05_rb,1.4203e-03_rb,2.5925e-03_rb,3.7016e-03_rb,4.7682e-03_rb,5.8654e-03_rb, &
     &7.0102e-03_rb,8.4347e-03_rb,9.0709e-03_rb/)
      kao(:, 3, 4, 5) = (/ &
     &1.4828e-05_rb,1.5367e-03_rb,2.8071e-03_rb,4.0356e-03_rb,5.2345e-03_rb,6.4096e-03_rb, &
     &7.5937e-03_rb,9.1611e-03_rb,9.8978e-03_rb/)
      kao(:, 4, 4, 5) = (/ &
     &1.6265e-05_rb,1.6502e-03_rb,3.0181e-03_rb,4.3485e-03_rb,5.6510e-03_rb,6.9455e-03_rb, &
     &8.2885e-03_rb,9.9540e-03_rb,1.0767e-02_rb/)
      kao(:, 5, 4, 5) = (/ &
     &1.7755e-05_rb,1.7615e-03_rb,3.2390e-03_rb,4.6692e-03_rb,6.0695e-03_rb,7.4667e-03_rb, &
     &8.9542e-03_rb,1.0781e-02_rb,1.1635e-02_rb/)
      kao(:, 1, 5, 5) = (/ &
     &2.0332e-05_rb,1.1698e-03_rb,2.0394e-03_rb,2.8422e-03_rb,3.6398e-03_rb,4.4106e-03_rb, &
     &5.1682e-03_rb,5.9629e-03_rb,6.5029e-03_rb/)
      kao(:, 2, 5, 5) = (/ &
     &2.2530e-05_rb,1.2736e-03_rb,2.2339e-03_rb,3.1431e-03_rb,3.9975e-03_rb,4.8072e-03_rb, &
     &5.6451e-03_rb,6.5119e-03_rb,7.1285e-03_rb/)
      kao(:, 3, 5, 5) = (/ &
     &2.4796e-05_rb,1.3790e-03_rb,2.4265e-03_rb,3.4174e-03_rb,4.3749e-03_rb,5.2936e-03_rb, &
     &6.1677e-03_rb,7.0807e-03_rb,7.7613e-03_rb/)
      kao(:, 4, 5, 5) = (/ &
     &2.7075e-05_rb,1.4796e-03_rb,2.6222e-03_rb,3.6840e-03_rb,4.7300e-03_rb,5.7397e-03_rb, &
     &6.7237e-03_rb,7.7301e-03_rb,8.5090e-03_rb/)
      kao(:, 5, 5, 5) = (/ &
     &2.9544e-05_rb,1.5786e-03_rb,2.8146e-03_rb,3.9658e-03_rb,5.0797e-03_rb,6.1834e-03_rb, &
     &7.2517e-03_rb,8.3791e-03_rb,9.2155e-03_rb/)
      kao(:, 1, 6, 5) = (/ &
     &3.1599e-05_rb,1.0623e-03_rb,1.7784e-03_rb,2.4227e-03_rb,3.0276e-03_rb,3.6296e-03_rb, &
     &4.1838e-03_rb,4.7343e-03_rb,5.0477e-03_rb/)
      kao(:, 2, 6, 5) = (/ &
     &3.5232e-05_rb,1.1598e-03_rb,1.9531e-03_rb,2.6761e-03_rb,3.3625e-03_rb,3.9768e-03_rb, &
     &4.5866e-03_rb,5.2016e-03_rb,5.5671e-03_rb/)
      kao(:, 3, 6, 5) = (/ &
     &3.8902e-05_rb,1.2546e-03_rb,2.1314e-03_rb,2.9272e-03_rb,3.6836e-03_rb,4.3880e-03_rb, &
     &5.0585e-03_rb,5.6644e-03_rb,6.0941e-03_rb/)
      kao(:, 4, 6, 5) = (/ &
     &4.2647e-05_rb,1.3467e-03_rb,2.3051e-03_rb,3.1735e-03_rb,3.9922e-03_rb,4.7680e-03_rb, &
     &5.5151e-03_rb,6.2191e-03_rb,6.7036e-03_rb/)
      kao(:, 5, 6, 5) = (/ &
     &4.6653e-05_rb,1.4422e-03_rb,2.4729e-03_rb,3.4216e-03_rb,4.3101e-03_rb,5.1401e-03_rb, &
     &5.9675e-03_rb,6.7430e-03_rb,7.2989e-03_rb/)
      kao(:, 1, 7, 5) = (/ &
     &5.0846e-05_rb,1.0315e-03_rb,1.6111e-03_rb,2.1236e-03_rb,2.5799e-03_rb,3.0310e-03_rb, &
     &3.4624e-03_rb,3.8172e-03_rb,3.8938e-03_rb/)
      kao(:, 2, 7, 5) = (/ &
     &5.6601e-05_rb,1.1149e-03_rb,1.7798e-03_rb,2.3658e-03_rb,2.8898e-03_rb,3.3716e-03_rb, &
     &3.7952e-03_rb,4.1967e-03_rb,4.3621e-03_rb/)
      kao(:, 3, 7, 5) = (/ &
     &6.2823e-05_rb,1.2046e-03_rb,1.9431e-03_rb,2.5998e-03_rb,3.1765e-03_rb,3.7177e-03_rb, &
     &4.2137e-03_rb,4.5955e-03_rb,4.7978e-03_rb/)
      kao(:, 4, 7, 5) = (/ &
     &6.9611e-05_rb,1.2957e-03_rb,2.0957e-03_rb,2.8201e-03_rb,3.4636e-03_rb,4.0605e-03_rb, &
     &4.6007e-03_rb,5.0741e-03_rb,5.2639e-03_rb/)
      kao(:, 5, 7, 5) = (/ &
     &7.6939e-05_rb,1.3859e-03_rb,2.2478e-03_rb,3.0375e-03_rb,3.7523e-03_rb,4.4027e-03_rb, &
     &4.9784e-03_rb,5.5148e-03_rb,5.7724e-03_rb/)
      kao(:, 1, 8, 5) = (/ &
     &9.6301e-05_rb,1.1719e-03_rb,1.6543e-03_rb,2.0770e-03_rb,2.3972e-03_rb,2.6733e-03_rb, &
     &2.9412e-03_rb,3.1694e-03_rb,3.0111e-03_rb/)
      kao(:, 2, 8, 5) = (/ &
     &1.0686e-04_rb,1.2764e-03_rb,1.7930e-03_rb,2.2808e-03_rb,2.6778e-03_rb,3.0239e-03_rb, &
     &3.3001e-03_rb,3.5015e-03_rb,3.3999e-03_rb/)
      kao(:, 3, 8, 5) = (/ &
     &1.1947e-04_rb,1.3739e-03_rb,1.9473e-03_rb,2.4712e-03_rb,2.9582e-03_rb,3.3516e-03_rb, &
     &3.6623e-03_rb,3.8834e-03_rb,3.7468e-03_rb/)
      kao(:, 4, 8, 5) = (/ &
     &1.3232e-04_rb,1.4765e-03_rb,2.1012e-03_rb,2.6685e-03_rb,3.2209e-03_rb,3.6672e-03_rb, &
     &4.0113e-03_rb,4.2720e-03_rb,4.1080e-03_rb/)
      kao(:, 5, 8, 5) = (/ &
     &1.4706e-04_rb,1.5789e-03_rb,2.2554e-03_rb,2.8711e-03_rb,3.4655e-03_rb,3.9725e-03_rb, &
     &4.3795e-03_rb,4.6534e-03_rb,4.5441e-03_rb/)
      kao(:, 1, 9, 5) = (/ &
     &3.5361e-04_rb,1.7799e-03_rb,2.5051e-03_rb,2.9026e-03_rb,3.0684e-03_rb,3.1720e-03_rb, &
     &3.1736e-03_rb,2.9481e-03_rb,2.3479e-03_rb/)
      kao(:, 2, 9, 5) = (/ &
     &3.8911e-04_rb,1.9917e-03_rb,2.7818e-03_rb,3.1870e-03_rb,3.3554e-03_rb,3.4634e-03_rb, &
     &3.5258e-03_rb,3.3674e-03_rb,2.6687e-03_rb/)
      kao(:, 3, 9, 5) = (/ &
     &4.2895e-04_rb,2.2031e-03_rb,3.0511e-03_rb,3.4571e-03_rb,3.6423e-03_rb,3.7656e-03_rb, &
     &3.8544e-03_rb,3.7669e-03_rb,2.9800e-03_rb/)
      kao(:, 4, 9, 5) = (/ &
     &4.7521e-04_rb,2.4131e-03_rb,3.3032e-03_rb,3.7275e-03_rb,3.9589e-03_rb,4.0895e-03_rb, &
     &4.1693e-03_rb,4.1455e-03_rb,3.3334e-03_rb/)
      kao(:, 5, 9, 5) = (/ &
     &5.2696e-04_rb,2.6212e-03_rb,3.5469e-03_rb,4.0100e-03_rb,4.2674e-03_rb,4.4062e-03_rb, &
     &4.4862e-03_rb,4.5098e-03_rb,3.6774e-03_rb/)
      kao(:, 1,10, 5) = (/ &
     &1.5564e-03_rb,3.2260e-03_rb,4.0223e-03_rb,4.5990e-03_rb,4.8864e-03_rb,4.9553e-03_rb, &
     &4.6372e-03_rb,3.8115e-03_rb,2.4163e-03_rb/)
      kao(:, 2,10, 5) = (/ &
     &1.6655e-03_rb,3.5530e-03_rb,4.4984e-03_rb,5.1615e-03_rb,5.5140e-03_rb,5.5636e-03_rb, &
     &5.1458e-03_rb,4.1627e-03_rb,2.8169e-03_rb/)
      kao(:, 3,10, 5) = (/ &
     &1.7970e-03_rb,3.9283e-03_rb,4.9975e-03_rb,5.7194e-03_rb,6.1431e-03_rb,6.1500e-03_rb, &
     &5.6336e-03_rb,4.5688e-03_rb,3.2157e-03_rb/)
      kao(:, 4,10, 5) = (/ &
     &1.9389e-03_rb,4.3419e-03_rb,5.5082e-03_rb,6.3036e-03_rb,6.7602e-03_rb,6.7061e-03_rb, &
     &6.1219e-03_rb,4.9831e-03_rb,3.5478e-03_rb/)
      kao(:, 5,10, 5) = (/ &
     &2.1200e-03_rb,4.8101e-03_rb,6.0597e-03_rb,6.9046e-03_rb,7.3538e-03_rb,7.2522e-03_rb, &
     &6.6229e-03_rb,5.4082e-03_rb,3.8617e-03_rb/)
      kao(:, 1,11, 5) = (/ &
     &2.5844e-03_rb,4.2072e-03_rb,4.7201e-03_rb,5.1141e-03_rb,5.3078e-03_rb,5.2931e-03_rb, &
     &4.9397e-03_rb,3.9983e-03_rb,2.6464e-03_rb/)
      kao(:, 2,11, 5) = (/ &
     &2.7239e-03_rb,4.5852e-03_rb,5.1921e-03_rb,5.6631e-03_rb,5.9270e-03_rb,5.9381e-03_rb, &
     &5.5459e-03_rb,4.4351e-03_rb,2.9283e-03_rb/)
      kao(:, 3,11, 5) = (/ &
     &2.8937e-03_rb,4.9994e-03_rb,5.7378e-03_rb,6.2452e-03_rb,6.5647e-03_rb,6.5969e-03_rb, &
     &6.1297e-03_rb,4.8694e-03_rb,3.2879e-03_rb/)
      kao(:, 4,11, 5) = (/ &
     &3.0973e-03_rb,5.4803e-03_rb,6.2977e-03_rb,6.8750e-03_rb,7.2241e-03_rb,7.2627e-03_rb, &
     &6.7036e-03_rb,5.3184e-03_rb,3.6746e-03_rb/)
      kao(:, 5,11, 5) = (/ &
     &3.3444e-03_rb,6.0139e-03_rb,6.9682e-03_rb,7.5496e-03_rb,7.9438e-03_rb,7.9183e-03_rb, &
     &7.2786e-03_rb,5.7800e-03_rb,4.1120e-03_rb/)
      kao(:, 1,12, 5) = (/ &
     &3.1142e-03_rb,4.6509e-03_rb,5.0639e-03_rb,5.2099e-03_rb,5.2596e-03_rb,5.1306e-03_rb, &
     &4.7167e-03_rb,3.8189e-03_rb,2.7150e-03_rb/)
      kao(:, 2,12, 5) = (/ &
     &3.2756e-03_rb,5.0359e-03_rb,5.5390e-03_rb,5.7321e-03_rb,5.8154e-03_rb,5.7214e-03_rb, &
     &5.2745e-03_rb,4.2400e-03_rb,3.0513e-03_rb/)
      kao(:, 3,12, 5) = (/ &
     &3.4985e-03_rb,5.4685e-03_rb,6.0590e-03_rb,6.2917e-03_rb,6.4404e-03_rb,6.3170e-03_rb, &
     &5.8354e-03_rb,4.6512e-03_rb,3.3763e-03_rb/)
      kao(:, 4,12, 5) = (/ &
     &3.7766e-03_rb,5.9630e-03_rb,6.6546e-03_rb,6.9419e-03_rb,7.0961e-03_rb,6.9336e-03_rb, &
     &6.4037e-03_rb,5.0749e-03_rb,3.7160e-03_rb/)
      kao(:, 5,12, 5) = (/ &
     &4.1085e-03_rb,6.5232e-03_rb,7.2797e-03_rb,7.6482e-03_rb,7.7587e-03_rb,7.5770e-03_rb, &
     &6.9583e-03_rb,5.5150e-03_rb,4.0847e-03_rb/)
      kao(:, 1,13, 5) = (/ &
     &2.9969e-03_rb,4.3888e-03_rb,4.7684e-03_rb,4.8307e-03_rb,4.7938e-03_rb,4.5847e-03_rb, &
     &4.1916e-03_rb,3.3359e-03_rb,2.6072e-03_rb/)
      kao(:, 2,13, 5) = (/ &
     &3.2153e-03_rb,4.7897e-03_rb,5.2331e-03_rb,5.3333e-03_rb,5.2913e-03_rb,5.1054e-03_rb, &
     &4.6633e-03_rb,3.6830e-03_rb,2.9306e-03_rb/)
      kao(:, 3,13, 5) = (/ &
     &3.4754e-03_rb,5.2096e-03_rb,5.7373e-03_rb,5.8606e-03_rb,5.8296e-03_rb,5.6492e-03_rb, &
     &5.1331e-03_rb,4.0274e-03_rb,3.2891e-03_rb/)
      kao(:, 4,13, 5) = (/ &
     &3.7596e-03_rb,5.6812e-03_rb,6.2772e-03_rb,6.4413e-03_rb,6.4105e-03_rb,6.2095e-03_rb, &
     &5.6055e-03_rb,4.3924e-03_rb,3.6856e-03_rb/)
      kao(:, 5,13, 5) = (/ &
     &4.1250e-03_rb,6.2069e-03_rb,6.8482e-03_rb,7.0531e-03_rb,7.0243e-03_rb,6.7656e-03_rb, &
     &6.0849e-03_rb,4.7766e-03_rb,4.1519e-03_rb/)
      kao(:, 1, 1, 6) = (/ &
     &8.6000e-06_rb,4.4651e-03_rb,9.0097e-03_rb,1.3704e-02_rb,1.8645e-02_rb,2.4046e-02_rb, &
     &3.0633e-02_rb,4.1152e-02_rb,3.7281e-02_rb/)
      kao(:, 2, 1, 6) = (/ &
     &9.1677e-06_rb,4.8343e-03_rb,9.7595e-03_rb,1.4853e-02_rb,2.0235e-02_rb,2.6158e-02_rb, &
     &3.3249e-02_rb,4.4739e-02_rb,4.0461e-02_rb/)
      kao(:, 3, 1, 6) = (/ &
     &9.7617e-06_rb,5.2064e-03_rb,1.0520e-02_rb,1.6024e-02_rb,2.1831e-02_rb,2.8229e-02_rb, &
     &3.6006e-02_rb,4.8415e-02_rb,4.3651e-02_rb/)
      kao(:, 4, 1, 6) = (/ &
     &1.0395e-05_rb,5.5866e-03_rb,1.1290e-02_rb,1.7191e-02_rb,2.3426e-02_rb,3.0307e-02_rb, &
     &3.8717e-02_rb,5.2240e-02_rb,4.6842e-02_rb/)
      kao(:, 5, 1, 6) = (/ &
     &1.1075e-05_rb,5.9593e-03_rb,1.2050e-02_rb,1.8360e-02_rb,2.5043e-02_rb,3.2426e-02_rb, &
     &4.1434e-02_rb,5.5912e-02_rb,5.0074e-02_rb/)
      kao(:, 1, 2, 6) = (/ &
     &1.1648e-05_rb,3.7355e-03_rb,7.3892e-03_rb,1.1190e-02_rb,1.5132e-02_rb,1.9317e-02_rb, &
     &2.4213e-02_rb,3.1544e-02_rb,3.0250e-02_rb/)
      kao(:, 2, 2, 6) = (/ &
     &1.2445e-05_rb,4.0410e-03_rb,8.0302e-03_rb,1.2160e-02_rb,1.6448e-02_rb,2.1055e-02_rb, &
     &2.6369e-02_rb,3.4311e-02_rb,3.2880e-02_rb/)
      kao(:, 3, 2, 6) = (/ &
     &1.3310e-05_rb,4.3542e-03_rb,8.6703e-03_rb,1.3133e-02_rb,1.7778e-02_rb,2.2773e-02_rb, &
     &2.8564e-02_rb,3.7228e-02_rb,3.5538e-02_rb/)
      kao(:, 4, 2, 6) = (/ &
     &1.4232e-05_rb,4.6616e-03_rb,9.3074e-03_rb,1.4103e-02_rb,1.9095e-02_rb,2.4467e-02_rb, &
     &3.0730e-02_rb,4.0202e-02_rb,3.8172e-02_rb/)
      kao(:, 5, 2, 6) = (/ &
     &1.5231e-05_rb,4.9691e-03_rb,9.9386e-03_rb,1.5066e-02_rb,2.0413e-02_rb,2.6167e-02_rb, &
     &3.2915e-02_rb,4.3077e-02_rb,4.0807e-02_rb/)
      kao(:, 1, 3, 6) = (/ &
     &2.0796e-05_rb,3.1636e-03_rb,6.0427e-03_rb,8.9479e-03_rb,1.1951e-02_rb,1.5111e-02_rb, &
     &1.8532e-02_rb,2.3072e-02_rb,2.3870e-02_rb/)
      kao(:, 2, 3, 6) = (/ &
     &2.2285e-05_rb,3.4357e-03_rb,6.5684e-03_rb,9.7449e-03_rb,1.3046e-02_rb,1.6505e-02_rb, &
     &2.0281e-02_rb,2.5200e-02_rb,2.6059e-02_rb/)
      kao(:, 3, 3, 6) = (/ &
     &2.3921e-05_rb,3.7148e-03_rb,7.1066e-03_rb,1.0556e-02_rb,1.4151e-02_rb,1.7911e-02_rb, &
     &2.2018e-02_rb,2.7457e-02_rb,2.8268e-02_rb/)
      kao(:, 4, 3, 6) = (/ &
     &2.5678e-05_rb,3.9815e-03_rb,7.6234e-03_rb,1.1355e-02_rb,1.5233e-02_rb,1.9276e-02_rb, &
     &2.3703e-02_rb,2.9635e-02_rb,3.0431e-02_rb/)
      kao(:, 5, 3, 6) = (/ &
     &2.7604e-05_rb,4.2441e-03_rb,8.1373e-03_rb,1.2146e-02_rb,1.6312e-02_rb,2.0656e-02_rb, &
     &2.5417e-02_rb,3.1780e-02_rb,3.2590e-02_rb/)
      kao(:, 1, 4, 6) = (/ &
     &3.7548e-05_rb,2.7525e-03_rb,5.0829e-03_rb,7.3519e-03_rb,9.5921e-03_rb,1.1880e-02_rb, &
     &1.4330e-02_rb,1.7270e-02_rb,1.8770e-02_rb/)
      kao(:, 2, 4, 6) = (/ &
     &4.0479e-05_rb,3.0018e-03_rb,5.5477e-03_rb,8.0335e-03_rb,1.0500e-02_rb,1.3017e-02_rb, &
     &1.5741e-02_rb,1.8989e-02_rb,2.0592e-02_rb/)
      kao(:, 3, 4, 6) = (/ &
     &4.3632e-05_rb,3.2358e-03_rb,6.0288e-03_rb,8.7218e-03_rb,1.1388e-02_rb,1.4152e-02_rb, &
     &1.7154e-02_rb,2.0724e-02_rb,2.2417e-02_rb/)
      kao(:, 4, 4, 6) = (/ &
     &4.6956e-05_rb,3.4641e-03_rb,6.4839e-03_rb,9.3953e-03_rb,1.2286e-02_rb,1.5294e-02_rb, &
     &1.8543e-02_rb,2.2416e-02_rb,2.4245e-02_rb/)
      kao(:, 5, 4, 6) = (/ &
     &5.0658e-05_rb,3.6946e-03_rb,6.9352e-03_rb,1.0060e-02_rb,1.3165e-02_rb,1.6424e-02_rb, &
     &1.9935e-02_rb,2.4115e-02_rb,2.6056e-02_rb/)
      kao(:, 1, 5, 6) = (/ &
     &6.1528e-05_rb,2.4644e-03_rb,4.3512e-03_rb,6.1626e-03_rb,7.9327e-03_rb,9.6496e-03_rb, &
     &1.1347e-02_rb,1.3264e-02_rb,1.4745e-02_rb/)
      kao(:, 2, 5, 6) = (/ &
     &6.6793e-05_rb,2.6844e-03_rb,4.7746e-03_rb,6.7632e-03_rb,8.7202e-03_rb,1.0626e-02_rb, &
     &1.2508e-02_rb,1.4649e-02_rb,1.6279e-02_rb/)
      kao(:, 3, 5, 6) = (/ &
     &7.2459e-05_rb,2.8961e-03_rb,5.1774e-03_rb,7.3853e-03_rb,9.5105e-03_rb,1.1570e-02_rb, &
     &1.3632e-02_rb,1.6051e-02_rb,1.7802e-02_rb/)
      kao(:, 4, 5, 6) = (/ &
     &7.8727e-05_rb,3.1045e-03_rb,5.5659e-03_rb,7.9693e-03_rb,1.0288e-02_rb,1.2541e-02_rb, &
     &1.4785e-02_rb,1.7438e-02_rb,1.9338e-02_rb/)
      kao(:, 5, 5, 6) = (/ &
     &8.5530e-05_rb,3.3182e-03_rb,5.9543e-03_rb,8.5528e-03_rb,1.1059e-02_rb,1.3485e-02_rb, &
     &1.5922e-02_rb,1.8829e-02_rb,2.0873e-02_rb/)
      kao(:, 1, 6, 6) = (/ &
     &9.4265e-05_rb,2.2301e-03_rb,3.8046e-03_rb,5.2488e-03_rb,6.6170e-03_rb,7.9289e-03_rb, &
     &9.1950e-03_rb,1.0423e-02_rb,1.1518e-02_rb/)
      kao(:, 2, 6, 6) = (/ &
     &1.0266e-04_rb,2.4246e-03_rb,4.1792e-03_rb,5.7800e-03_rb,7.2894e-03_rb,8.7747e-03_rb, &
     &1.0200e-02_rb,1.1548e-02_rb,1.2780e-02_rb/)
      kao(:, 3, 6, 6) = (/ &
     &1.1200e-04_rb,2.6239e-03_rb,4.5381e-03_rb,6.2992e-03_rb,7.9916e-03_rb,9.6188e-03_rb, &
     &1.1174e-02_rb,1.2683e-02_rb,1.4059e-02_rb/)
      kao(:, 4, 6, 6) = (/ &
     &1.2224e-04_rb,2.8287e-03_rb,4.8898e-03_rb,6.8010e-03_rb,8.6582e-03_rb,1.0462e-02_rb, &
     &1.2176e-02_rb,1.3803e-02_rb,1.5334e-02_rb/)
      kao(:, 5, 6, 6) = (/ &
     &1.3337e-04_rb,3.0366e-03_rb,5.2437e-03_rb,7.3027e-03_rb,9.3153e-03_rb,1.1279e-02_rb, &
     &1.3144e-02_rb,1.4949e-02_rb,1.6628e-02_rb/)
      kao(:, 1, 7, 6) = (/ &
     &1.5263e-04_rb,2.1146e-03_rb,3.4378e-03_rb,4.6052e-03_rb,5.6815e-03_rb,6.6787e-03_rb, &
     &7.5592e-03_rb,8.3902e-03_rb,8.9861e-03_rb/)
      kao(:, 2, 7, 6) = (/ &
     &1.6683e-04_rb,2.3125e-03_rb,3.7648e-03_rb,5.0866e-03_rb,6.2982e-03_rb,7.4001e-03_rb, &
     &8.4314e-03_rb,9.3717e-03_rb,1.0001e-02_rb/)
      kao(:, 3, 7, 6) = (/ &
     &1.8246e-04_rb,2.5152e-03_rb,4.0947e-03_rb,5.5502e-03_rb,6.9009e-03_rb,8.1426e-03_rb, &
     &9.2748e-03_rb,1.0344e-02_rb,1.1085e-02_rb/)
      kao(:, 4, 7, 6) = (/ &
     &1.9965e-04_rb,2.7190e-03_rb,4.4394e-03_rb,6.0162e-03_rb,7.4822e-03_rb,8.8463e-03_rb, &
     &1.0151e-02_rb,1.1321e-02_rb,1.2163e-02_rb/)
      kao(:, 5, 7, 6) = (/ &
     &2.1839e-04_rb,2.9201e-03_rb,4.7890e-03_rb,6.4854e-03_rb,8.0589e-03_rb,9.5473e-03_rb, &
     &1.0985e-02_rb,1.2296e-02_rb,1.3266e-02_rb/)
      kao(:, 1, 8, 6) = (/ &
     &2.9912e-04_rb,2.3311e-03_rb,3.4065e-03_rb,4.3529e-03_rb,5.1991e-03_rb,5.9115e-03_rb, &
     &6.5345e-03_rb,6.9536e-03_rb,6.9673e-03_rb/)
      kao(:, 2, 8, 6) = (/ &
     &3.2667e-04_rb,2.5198e-03_rb,3.7444e-03_rb,4.8113e-03_rb,5.7475e-03_rb,6.5852e-03_rb, &
     &7.2722e-03_rb,7.8027e-03_rb,7.8100e-03_rb/)
      kao(:, 3, 8, 6) = (/ &
     &3.5783e-04_rb,2.7045e-03_rb,4.0852e-03_rb,5.2736e-03_rb,6.2947e-03_rb,7.2359e-03_rb, &
     &8.0488e-03_rb,8.6314e-03_rb,8.7294e-03_rb/)
      kao(:, 4, 8, 6) = (/ &
     &3.9253e-04_rb,2.8930e-03_rb,4.4286e-03_rb,5.7246e-03_rb,6.8467e-03_rb,7.8891e-03_rb, &
     &8.7826e-03_rb,9.4970e-03_rb,9.6421e-03_rb/)
      kao(:, 5, 8, 6) = (/ &
     &4.3012e-04_rb,3.0887e-03_rb,4.7690e-03_rb,6.1815e-03_rb,7.4286e-03_rb,8.5468e-03_rb, &
     &9.5146e-03_rb,1.0319e-02_rb,1.0565e-02_rb/)
      kao(:, 1, 9, 6) = (/ &
     &1.0976e-03_rb,4.1506e-03_rb,5.1147e-03_rb,5.7794e-03_rb,6.2915e-03_rb,6.5755e-03_rb, &
     &6.7173e-03_rb,6.5721e-03_rb,5.2619e-03_rb/)
      kao(:, 2, 9, 6) = (/ &
     &1.2019e-03_rb,4.4814e-03_rb,5.5166e-03_rb,6.2823e-03_rb,6.8818e-03_rb,7.2627e-03_rb, &
     &7.4725e-03_rb,7.3662e-03_rb,5.9464e-03_rb/)
      kao(:, 3, 9, 6) = (/ &
     &1.3207e-03_rb,4.7927e-03_rb,5.9242e-03_rb,6.7576e-03_rb,7.4599e-03_rb,7.9614e-03_rb, &
     &8.2442e-03_rb,8.1477e-03_rb,6.6764e-03_rb/)
      kao(:, 4, 9, 6) = (/ &
     &1.4520e-03_rb,5.0989e-03_rb,6.3402e-03_rb,7.2519e-03_rb,8.0196e-03_rb,8.6608e-03_rb, &
     &9.0083e-03_rb,8.9373e-03_rb,7.4095e-03_rb/)
      kao(:, 5, 9, 6) = (/ &
     &1.5966e-03_rb,5.4241e-03_rb,6.7693e-03_rb,7.7581e-03_rb,8.6262e-03_rb,9.3645e-03_rb, &
     &9.7704e-03_rb,9.7289e-03_rb,8.1550e-03_rb/)
      kao(:, 1,10, 6) = (/ &
     &4.4394e-03_rb,9.1190e-03_rb,1.0725e-02_rb,1.1220e-02_rb,1.1152e-02_rb,1.0508e-02_rb, &
     &9.4386e-03_rb,7.8107e-03_rb,4.9143e-03_rb/)
      kao(:, 2,10, 6) = (/ &
     &4.8803e-03_rb,1.0187e-02_rb,1.1809e-02_rb,1.2265e-02_rb,1.2072e-02_rb,1.1380e-02_rb, &
     &1.0318e-02_rb,8.6414e-03_rb,5.6112e-03_rb/)
      kao(:, 3,10, 6) = (/ &
     &5.3784e-03_rb,1.1243e-02_rb,1.2866e-02_rb,1.3294e-02_rb,1.2988e-02_rb,1.2282e-02_rb, &
     &1.1181e-02_rb,9.4518e-03_rb,6.2671e-03_rb/)
      kao(:, 4,10, 6) = (/ &
     &5.9406e-03_rb,1.2247e-02_rb,1.3888e-02_rb,1.4276e-02_rb,1.3911e-02_rb,1.3225e-02_rb, &
     &1.2026e-02_rb,1.0238e-02_rb,6.9421e-03_rb/)
      kao(:, 5,10, 6) = (/ &
     &6.5660e-03_rb,1.3194e-02_rb,1.4891e-02_rb,1.5253e-02_rb,1.4902e-02_rb,1.4176e-02_rb, &
     &1.2864e-02_rb,1.1066e-02_rb,7.6281e-03_rb/)
      kao(:, 1,11, 6) = (/ &
     &6.6711e-03_rb,1.1267e-02_rb,1.2748e-02_rb,1.3242e-02_rb,1.2977e-02_rb,1.2030e-02_rb, &
     &1.0523e-02_rb,8.3002e-03_rb,5.3202e-03_rb/)
      kao(:, 2,11, 6) = (/ &
     &7.3092e-03_rb,1.2565e-02_rb,1.4207e-02_rb,1.4653e-02_rb,1.4245e-02_rb,1.3138e-02_rb, &
     &1.1437e-02_rb,9.1124e-03_rb,6.0956e-03_rb/)
      kao(:, 3,11, 6) = (/ &
     &8.0261e-03_rb,1.3900e-02_rb,1.5599e-02_rb,1.6022e-02_rb,1.5510e-02_rb,1.4232e-02_rb, &
     &1.2399e-02_rb,9.8954e-03_rb,6.7920e-03_rb/)
      kao(:, 4,11, 6) = (/ &
     &8.8835e-03_rb,1.5218e-02_rb,1.7018e-02_rb,1.7348e-02_rb,1.6755e-02_rb,1.5328e-02_rb, &
     &1.3427e-02_rb,1.0676e-02_rb,7.4156e-03_rb/)
      kao(:, 5,11, 6) = (/ &
     &9.8592e-03_rb,1.6591e-02_rb,1.8382e-02_rb,1.8693e-02_rb,1.7995e-02_rb,1.6489e-02_rb, &
     &1.4464e-02_rb,1.1488e-02_rb,8.0687e-03_rb/)
      kao(:, 1,12, 6) = (/ &
     &7.8286e-03_rb,1.1844e-02_rb,1.2951e-02_rb,1.3269e-02_rb,1.2819e-02_rb,1.1719e-02_rb, &
     &1.0146e-02_rb,7.7901e-03_rb,5.3706e-03_rb/)
      kao(:, 2,12, 6) = (/ &
     &8.5205e-03_rb,1.3089e-02_rb,1.4355e-02_rb,1.4624e-02_rb,1.4116e-02_rb,1.2895e-02_rb, &
     &1.1103e-02_rb,8.5730e-03_rb,5.9882e-03_rb/)
      kao(:, 3,12, 6) = (/ &
     &9.3309e-03_rb,1.4405e-02_rb,1.5746e-02_rb,1.6010e-02_rb,1.5387e-02_rb,1.4067e-02_rb, &
     &1.2065e-02_rb,9.3726e-03_rb,6.6686e-03_rb/)
      kao(:, 4,12, 6) = (/ &
     &1.0285e-02_rb,1.5780e-02_rb,1.7147e-02_rb,1.7368e-02_rb,1.6677e-02_rb,1.5224e-02_rb, &
     &1.3058e-02_rb,1.0197e-02_rb,7.3827e-03_rb/)
      kao(:, 5,12, 6) = (/ &
     &1.1378e-02_rb,1.7192e-02_rb,1.8621e-02_rb,1.8778e-02_rb,1.8045e-02_rb,1.6433e-02_rb, &
     &1.4105e-02_rb,1.1017e-02_rb,8.1428e-03_rb/)
      kao(:, 1,13, 6) = (/ &
     &7.7075e-03_rb,1.1053e-02_rb,1.1788e-02_rb,1.1798e-02_rb,1.1282e-02_rb,1.0218e-02_rb, &
     &8.7608e-03_rb,6.6974e-03_rb,4.9774e-03_rb/)
      kao(:, 2,13, 6) = (/ &
     &8.3931e-03_rb,1.2074e-02_rb,1.2868e-02_rb,1.2866e-02_rb,1.2320e-02_rb,1.1165e-02_rb, &
     &9.5984e-03_rb,7.4025e-03_rb,5.6029e-03_rb/)
      kao(:, 3,13, 6) = (/ &
     &9.1257e-03_rb,1.3198e-02_rb,1.3994e-02_rb,1.3989e-02_rb,1.3400e-02_rb,1.2194e-02_rb, &
     &1.0455e-02_rb,8.1317e-03_rb,6.2279e-03_rb/)
      kao(:, 4,13, 6) = (/ &
     &9.9594e-03_rb,1.4337e-02_rb,1.5200e-02_rb,1.5161e-02_rb,1.4494e-02_rb,1.3263e-02_rb, &
     &1.1356e-02_rb,8.8525e-03_rb,6.8478e-03_rb/)
      kao(:, 5,13, 6) = (/ &
     &1.0855e-02_rb,1.5527e-02_rb,1.6501e-02_rb,1.6412e-02_rb,1.5661e-02_rb,1.4395e-02_rb, &
     &1.2355e-02_rb,9.5887e-03_rb,7.3951e-03_rb/)
      kao(:, 1, 1, 7) = (/ &
     &2.8482e-05_rb,1.1072e-02_rb,2.2376e-02_rb,3.4052e-02_rb,4.6363e-02_rb,5.9888e-02_rb, &
     &7.6137e-02_rb,1.0147e-01_rb,9.2714e-02_rb/)
      kao(:, 2, 1, 7) = (/ &
     &2.9459e-05_rb,1.1920e-02_rb,2.4092e-02_rb,3.6662e-02_rb,4.9933e-02_rb,6.4531e-02_rb, &
     &8.2136e-02_rb,1.0950e-01_rb,9.9856e-02_rb/)
      kao(:, 3, 1, 7) = (/ &
     &3.0378e-05_rb,1.2748e-02_rb,2.5776e-02_rb,3.9242e-02_rb,5.3450e-02_rb,6.9069e-02_rb, &
     &8.7941e-02_rb,1.1741e-01_rb,1.0689e-01_rb/)
      kao(:, 4, 1, 7) = (/ &
     &3.1242e-05_rb,1.3565e-02_rb,2.7429e-02_rb,4.1762e-02_rb,5.6896e-02_rb,7.3556e-02_rb, &
     &9.3626e-02_rb,1.2500e-01_rb,1.1378e-01_rb/)
      kao(:, 5, 1, 7) = (/ &
     &3.2096e-05_rb,1.4368e-02_rb,2.9038e-02_rb,4.4226e-02_rb,6.0256e-02_rb,7.7920e-02_rb, &
     &9.9246e-02_rb,1.3270e-01_rb,1.2050e-01_rb/)
      kao(:, 1, 2, 7) = (/ &
     &3.8108e-05_rb,9.1980e-03_rb,1.8505e-02_rb,2.8037e-02_rb,3.7923e-02_rb,4.8524e-02_rb, &
     &6.0800e-02_rb,7.8866e-02_rb,7.5829e-02_rb/)
      kao(:, 2, 2, 7) = (/ &
     &3.9453e-05_rb,9.9404e-03_rb,2.0012e-02_rb,3.0307e-02_rb,4.0999e-02_rb,5.2467e-02_rb, &
     &6.5752e-02_rb,8.5496e-02_rb,8.1980e-02_rb/)
      kao(:, 3, 2, 7) = (/ &
     &4.0728e-05_rb,1.0645e-02_rb,2.1444e-02_rb,3.2483e-02_rb,4.3962e-02_rb,5.6282e-02_rb, &
     &7.0607e-02_rb,9.1834e-02_rb,8.7906e-02_rb/)
      kao(:, 4, 2, 7) = (/ &
     &4.1960e-05_rb,1.1349e-02_rb,2.2852e-02_rb,3.4627e-02_rb,4.6872e-02_rb,6.0009e-02_rb, &
     &7.5320e-02_rb,9.8100e-02_rb,9.3725e-02_rb/)
      kao(:, 5, 2, 7) = (/ &
     &4.3152e-05_rb,1.2042e-02_rb,2.4257e-02_rb,3.6759e-02_rb,4.9754e-02_rb,6.3714e-02_rb, &
     &7.9963e-02_rb,1.0426e-01_rb,9.9487e-02_rb/)
      kao(:, 1, 3, 7) = (/ &
     &6.8682e-05_rb,7.6817e-03_rb,1.5032e-02_rb,2.2610e-02_rb,3.0354e-02_rb,3.8397e-02_rb, &
     &4.7144e-02_rb,5.8545e-02_rb,6.0680e-02_rb/)
      kao(:, 2, 3, 7) = (/ &
     &7.1048e-05_rb,8.3077e-03_rb,1.6309e-02_rb,2.4554e-02_rb,3.2973e-02_rb,4.1716e-02_rb, &
     &5.1249e-02_rb,6.3726e-02_rb,6.5916e-02_rb/)
      kao(:, 3, 3, 7) = (/ &
     &7.3516e-05_rb,8.8904e-03_rb,1.7519e-02_rb,2.6389e-02_rb,3.5446e-02_rb,4.4862e-02_rb, &
     &5.5125e-02_rb,6.8637e-02_rb,7.0859e-02_rb/)
      kao(:, 4, 3, 7) = (/ &
     &7.5927e-05_rb,9.4787e-03_rb,1.8742e-02_rb,2.8235e-02_rb,3.7922e-02_rb,4.7994e-02_rb, &
     &5.8964e-02_rb,7.3444e-02_rb,7.5806e-02_rb/)
      kao(:, 5, 3, 7) = (/ &
     &7.8228e-05_rb,1.0059e-02_rb,1.9946e-02_rb,3.0057e-02_rb,4.0372e-02_rb,5.1096e-02_rb, &
     &6.2816e-02_rb,7.8277e-02_rb,8.0701e-02_rb/)
      kao(:, 1, 4, 7) = (/ &
     &1.2728e-04_rb,6.6847e-03_rb,1.2528e-02_rb,1.8338e-02_rb,2.4310e-02_rb,3.0521e-02_rb, &
     &3.7034e-02_rb,4.4629e-02_rb,4.8506e-02_rb/)
      kao(:, 2, 4, 7) = (/ &
     &1.3229e-04_rb,7.2462e-03_rb,1.3606e-02_rb,1.9949e-02_rb,2.6505e-02_rb,3.3306e-02_rb, &
     &4.0432e-02_rb,4.8804e-02_rb,5.2931e-02_rb/)
      kao(:, 3, 4, 7) = (/ &
     &1.3690e-04_rb,7.7950e-03_rb,1.4636e-02_rb,2.1498e-02_rb,2.8618e-02_rb,3.5967e-02_rb, &
     &4.3669e-02_rb,5.2720e-02_rb,5.7159e-02_rb/)
      kao(:, 4, 4, 7) = (/ &
     &1.4141e-04_rb,8.3363e-03_rb,1.5650e-02_rb,2.3036e-02_rb,3.0707e-02_rb,3.8607e-02_rb, &
     &4.6880e-02_rb,5.6623e-02_rb,6.1327e-02_rb/)
      kao(:, 5, 4, 7) = (/ &
     &1.4572e-04_rb,8.8720e-03_rb,1.6649e-02_rb,2.4570e-02_rb,3.2799e-02_rb,4.1227e-02_rb, &
     &5.0069e-02_rb,6.0500e-02_rb,6.5502e-02_rb/)
      kao(:, 1, 5, 7) = (/ &
     &2.1573e-04_rb,5.8827e-03_rb,1.0779e-02_rb,1.5450e-02_rb,1.9971e-02_rb,2.4513e-02_rb, &
     &2.9346e-02_rb,3.4831e-02_rb,3.8718e-02_rb/)
      kao(:, 2, 5, 7) = (/ &
     &2.2440e-04_rb,6.4009e-03_rb,1.1752e-02_rb,1.6862e-02_rb,2.1813e-02_rb,2.6822e-02_rb, &
     &3.2187e-02_rb,3.8239e-02_rb,4.2467e-02_rb/)
      kao(:, 3, 5, 7) = (/ &
     &2.3312e-04_rb,6.9022e-03_rb,1.2696e-02_rb,1.8213e-02_rb,2.3573e-02_rb,2.9063e-02_rb, &
     &3.4955e-02_rb,4.1525e-02_rb,4.6111e-02_rb/)
      kao(:, 4, 5, 7) = (/ &
     &2.4120e-04_rb,7.3983e-03_rb,1.3629e-02_rb,1.9546e-02_rb,2.5293e-02_rb,3.1249e-02_rb, &
     &3.7630e-02_rb,4.4709e-02_rb,4.9634e-02_rb/)
      kao(:, 5, 5, 7) = (/ &
     &2.4900e-04_rb,7.8813e-03_rb,1.4548e-02_rb,2.0857e-02_rb,2.7012e-02_rb,3.3451e-02_rb, &
     &4.0320e-02_rb,4.7915e-02_rb,5.3175e-02_rb/)
      kao(:, 1, 6, 7) = (/ &
     &3.4036e-04_rb,5.3231e-03_rb,9.2729e-03_rb,1.3061e-02_rb,1.6728e-02_rb,2.0215e-02_rb, &
     &2.3560e-02_rb,2.7195e-02_rb,3.0556e-02_rb/)
      kao(:, 2, 6, 7) = (/ &
     &3.5502e-04_rb,5.8134e-03_rb,1.0151e-02_rb,1.4345e-02_rb,1.8389e-02_rb,2.2227e-02_rb, &
     &2.5905e-02_rb,3.0036e-02_rb,3.3756e-02_rb/)
      kao(:, 3, 6, 7) = (/ &
     &3.6934e-04_rb,6.2790e-03_rb,1.1006e-02_rb,1.5584e-02_rb,1.9972e-02_rb,2.4157e-02_rb, &
     &2.8189e-02_rb,3.2811e-02_rb,3.6865e-02_rb/)
      kao(:, 4, 6, 7) = (/ &
     &3.8386e-04_rb,6.7401e-03_rb,1.1853e-02_rb,1.6807e-02_rb,2.1541e-02_rb,2.6034e-02_rb, &
     &3.0406e-02_rb,3.5498e-02_rb,3.9873e-02_rb/)
      kao(:, 5, 6, 7) = (/ &
     &3.9798e-04_rb,7.1899e-03_rb,1.2680e-02_rb,1.7996e-02_rb,2.3081e-02_rb,2.7888e-02_rb, &
     &3.2622e-02_rb,3.8170e-02_rb,4.2864e-02_rb/)
      kao(:, 1, 7, 7) = (/ &
     &5.6321e-04_rb,5.0131e-03_rb,8.3197e-03_rb,1.1342e-02_rb,1.4164e-02_rb,1.6877e-02_rb, &
     &1.9447e-02_rb,2.1713e-02_rb,2.3905e-02_rb/)
      kao(:, 2, 7, 7) = (/ &
     &5.9016e-04_rb,5.4925e-03_rb,9.1570e-03_rb,1.2482e-02_rb,1.5641e-02_rb,1.8693e-02_rb, &
     &2.1556e-02_rb,2.4086e-02_rb,2.6629e-02_rb/)
      kao(:, 3, 7, 7) = (/ &
     &6.1706e-04_rb,5.9641e-03_rb,9.9615e-03_rb,1.3605e-02_rb,1.7087e-02_rb,2.0445e-02_rb, &
     &2.3593e-02_rb,2.6361e-02_rb,2.9262e-02_rb/)
      kao(:, 4, 7, 7) = (/ &
     &6.4250e-04_rb,6.4355e-03_rb,1.0740e-02_rb,1.4689e-02_rb,1.8507e-02_rb,2.2162e-02_rb, &
     &2.5558e-02_rb,2.8565e-02_rb,3.1822e-02_rb/)
      kao(:, 5, 7, 7) = (/ &
     &6.6706e-04_rb,6.9143e-03_rb,1.1505e-02_rb,1.5753e-02_rb,1.9886e-02_rb,2.3836e-02_rb, &
     &2.7497e-02_rb,3.0749e-02_rb,3.4313e-02_rb/)
      kao(:, 1, 8, 7) = (/ &
     &1.1137e-03_rb,5.2798e-03_rb,8.0960e-03_rb,1.0553e-02_rb,1.2781e-02_rb,1.4788e-02_rb, &
     &1.6508e-02_rb,1.7972e-02_rb,1.8634e-02_rb/)
      kao(:, 2, 8, 7) = (/ &
     &1.1746e-03_rb,5.8110e-03_rb,8.9215e-03_rb,1.1680e-02_rb,1.4167e-02_rb,1.6394e-02_rb, &
     &1.8389e-02_rb,2.0091e-02_rb,2.0946e-02_rb/)
      kao(:, 3, 8, 7) = (/ &
     &1.2333e-03_rb,6.3510e-03_rb,9.7459e-03_rb,1.2786e-02_rb,1.5523e-02_rb,1.7978e-02_rb, &
     &2.0210e-02_rb,2.2150e-02_rb,2.3198e-02_rb/)
      kao(:, 4, 8, 7) = (/ &
     &1.2923e-03_rb,6.8806e-03_rb,1.0573e-02_rb,1.3874e-02_rb,1.6833e-02_rb,1.9520e-02_rb, &
     &2.2008e-02_rb,2.4130e-02_rb,2.5397e-02_rb/)
      kao(:, 5, 8, 7) = (/ &
     &1.3493e-03_rb,7.4001e-03_rb,1.1421e-02_rb,1.4955e-02_rb,1.8119e-02_rb,2.1028e-02_rb, &
     &2.3751e-02_rb,2.6082e-02_rb,2.7521e-02_rb/)
      kao(:, 1, 9, 7) = (/ &
     &4.1652e-03_rb,9.1226e-03_rb,1.1391e-02_rb,1.3127e-02_rb,1.4574e-02_rb,1.5712e-02_rb, &
     &1.6412e-02_rb,1.6590e-02_rb,1.4324e-02_rb/)
      kao(:, 2, 9, 7) = (/ &
     &4.4077e-03_rb,9.7658e-03_rb,1.2393e-02_rb,1.4489e-02_rb,1.6135e-02_rb,1.7441e-02_rb, &
     &1.8307e-02_rb,1.8536e-02_rb,1.6186e-02_rb/)
      kao(:, 3, 9, 7) = (/ &
     &4.6510e-03_rb,1.0425e-02_rb,1.3450e-02_rb,1.5897e-02_rb,1.7759e-02_rb,1.9177e-02_rb, &
     &2.0163e-02_rb,2.0445e-02_rb,1.8103e-02_rb/)
      kao(:, 4, 9, 7) = (/ &
     &4.8932e-03_rb,1.1096e-02_rb,1.4490e-02_rb,1.7265e-02_rb,1.9404e-02_rb,2.0920e-02_rb, &
     &2.2008e-02_rb,2.2308e-02_rb,1.9978e-02_rb/)
      kao(:, 5, 9, 7) = (/ &
     &5.1319e-03_rb,1.1758e-02_rb,1.5541e-02_rb,1.8602e-02_rb,2.0988e-02_rb,2.2688e-02_rb, &
     &2.3851e-02_rb,2.4156e-02_rb,2.1799e-02_rb/)
      kao(:, 1,10, 7) = (/ &
     &1.7349e-02_rb,2.4277e-02_rb,2.5467e-02_rb,2.5566e-02_rb,2.4825e-02_rb,2.3373e-02_rb, &
     &2.1223e-02_rb,1.8524e-02_rb,1.1809e-02_rb/)
      kao(:, 2,10, 7) = (/ &
     &1.8441e-02_rb,2.5550e-02_rb,2.7079e-02_rb,2.7402e-02_rb,2.6767e-02_rb,2.5495e-02_rb, &
     &2.3503e-02_rb,2.0675e-02_rb,1.3556e-02_rb/)
      kao(:, 3,10, 7) = (/ &
     &1.9634e-02_rb,2.6955e-02_rb,2.8784e-02_rb,2.9151e-02_rb,2.8790e-02_rb,2.7521e-02_rb, &
     &2.5774e-02_rb,2.2876e-02_rb,1.5334e-02_rb/)
      kao(:, 4,10, 7) = (/ &
     &2.0834e-02_rb,2.8474e-02_rb,3.0492e-02_rb,3.0988e-02_rb,3.0837e-02_rb,2.9673e-02_rb, &
     &2.8036e-02_rb,2.5100e-02_rb,1.7072e-02_rb/)
      kao(:, 5,10, 7) = (/ &
     &2.2004e-02_rb,3.0118e-02_rb,3.2233e-02_rb,3.2923e-02_rb,3.2760e-02_rb,3.1852e-02_rb, &
     &3.0321e-02_rb,2.7280e-02_rb,1.8710e-02_rb/)
      kao(:, 1,11, 7) = (/ &
     &2.5621e-02_rb,3.2655e-02_rb,3.3386e-02_rb,3.2478e-02_rb,3.0535e-02_rb,2.7796e-02_rb, &
     &2.3855e-02_rb,1.8680e-02_rb,1.0565e-02_rb/)
      kao(:, 2,11, 7) = (/ &
     &2.7487e-02_rb,3.4718e-02_rb,3.5449e-02_rb,3.4659e-02_rb,3.2784e-02_rb,2.9958e-02_rb, &
     &2.6042e-02_rb,2.0737e-02_rb,1.2143e-02_rb/)
      kao(:, 3,11, 7) = (/ &
     &2.9493e-02_rb,3.6885e-02_rb,3.7684e-02_rb,3.6984e-02_rb,3.4999e-02_rb,3.2138e-02_rb, &
     &2.8186e-02_rb,2.2834e-02_rb,1.3640e-02_rb/)
      kao(:, 4,11, 7) = (/ &
     &3.1405e-02_rb,3.9361e-02_rb,4.0123e-02_rb,3.9393e-02_rb,3.7354e-02_rb,3.4345e-02_rb, &
     &3.0182e-02_rb,2.4896e-02_rb,1.5233e-02_rb/)
      kao(:, 5,11, 7) = (/ &
     &3.3372e-02_rb,4.1804e-02_rb,4.2639e-02_rb,4.1888e-02_rb,3.9776e-02_rb,3.6559e-02_rb, &
     &3.2377e-02_rb,2.6934e-02_rb,1.6905e-02_rb/)
      kao(:, 1,12, 7) = (/ &
     &2.8107e-02_rb,3.4513e-02_rb,3.4691e-02_rb,3.3440e-02_rb,3.1149e-02_rb,2.8092e-02_rb, &
     &2.3835e-02_rb,1.7987e-02_rb,9.8223e-03_rb/)
      kao(:, 2,12, 7) = (/ &
     &3.0318e-02_rb,3.7007e-02_rb,3.7165e-02_rb,3.5894e-02_rb,3.3524e-02_rb,3.0240e-02_rb, &
     &2.5840e-02_rb,1.9792e-02_rb,1.1332e-02_rb/)
      kao(:, 3,12, 7) = (/ &
     &3.2580e-02_rb,3.9682e-02_rb,3.9790e-02_rb,3.8457e-02_rb,3.5971e-02_rb,3.2486e-02_rb, &
     &2.7883e-02_rb,2.1561e-02_rb,1.2728e-02_rb/)
      kao(:, 4,12, 7) = (/ &
     &3.4838e-02_rb,4.2408e-02_rb,4.2616e-02_rb,4.1228e-02_rb,3.8571e-02_rb,3.4918e-02_rb, &
     &2.9920e-02_rb,2.3363e-02_rb,1.4318e-02_rb/)
      kao(:, 5,12, 7) = (/ &
     &3.7163e-02_rb,4.5231e-02_rb,4.5400e-02_rb,4.4006e-02_rb,4.1203e-02_rb,3.7331e-02_rb, &
     &3.2004e-02_rb,2.5209e-02_rb,1.5927e-02_rb/)
      kao(:, 1,13, 7) = (/ &
     &2.5217e-02_rb,3.0486e-02_rb,3.0548e-02_rb,2.9385e-02_rb,2.7336e-02_rb,2.4524e-02_rb, &
     &2.0890e-02_rb,1.5756e-02_rb,9.4392e-03_rb/)
      kao(:, 2,13, 7) = (/ &
     &2.7202e-02_rb,3.2841e-02_rb,3.2933e-02_rb,3.1772e-02_rb,2.9593e-02_rb,2.6611e-02_rb, &
     &2.2741e-02_rb,1.7329e-02_rb,1.0602e-02_rb/)
      kao(:, 3,13, 7) = (/ &
     &2.9280e-02_rb,3.5351e-02_rb,3.5444e-02_rb,3.4289e-02_rb,3.1993e-02_rb,2.8821e-02_rb, &
     &2.4603e-02_rb,1.8828e-02_rb,1.1938e-02_rb/)
      kao(:, 4,13, 7) = (/ &
     &3.1331e-02_rb,3.7956e-02_rb,3.7975e-02_rb,3.6801e-02_rb,3.4445e-02_rb,3.1086e-02_rb, &
     &2.6513e-02_rb,2.0428e-02_rb,1.3267e-02_rb/)
      kao(:, 5,13, 7) = (/ &
     &3.3613e-02_rb,4.0685e-02_rb,4.0655e-02_rb,3.9388e-02_rb,3.6935e-02_rb,3.3268e-02_rb, &
     &2.8459e-02_rb,2.2050e-02_rb,1.4574e-02_rb/)
      kao(:, 1, 1, 8) = (/ &
     &7.0857e-05_rb,3.1247e-02_rb,6.3050e-02_rb,9.5741e-02_rb,1.2991e-01_rb,1.6688e-01_rb, &
     &2.1008e-01_rb,2.7194e-01_rb,2.5981e-01_rb/)
      kao(:, 2, 1, 8) = (/ &
     &7.3061e-05_rb,3.3759e-02_rb,6.8122e-02_rb,1.0346e-01_rb,1.4038e-01_rb,1.8030e-01_rb, &
     &2.2703e-01_rb,2.9445e-01_rb,2.8076e-01_rb/)
      kao(:, 3, 1, 8) = (/ &
     &7.5439e-05_rb,3.6248e-02_rb,7.3153e-02_rb,1.1109e-01_rb,1.5077e-01_rb,1.9367e-01_rb, &
     &2.4381e-01_rb,3.1652e-01_rb,3.0153e-01_rb/)
      kao(:, 4, 1, 8) = (/ &
     &7.7957e-05_rb,3.8675e-02_rb,7.8063e-02_rb,1.1859e-01_rb,1.6099e-01_rb,2.0691e-01_rb, &
     &2.6053e-01_rb,3.3811e-01_rb,3.2197e-01_rb/)
      kao(:, 5, 1, 8) = (/ &
     &8.0722e-05_rb,4.1119e-02_rb,8.2986e-02_rb,1.2604e-01_rb,1.7112e-01_rb,2.1995e-01_rb, &
     &2.7714e-01_rb,3.5946e-01_rb,3.4222e-01_rb/)
      kao(:, 1, 2, 8) = (/ &
     &9.7224e-05_rb,2.6558e-02_rb,5.3452e-02_rb,8.0855e-02_rb,1.0918e-01_rb,1.3924e-01_rb, &
     &1.7330e-01_rb,2.2109e-01_rb,2.1834e-01_rb/)
      kao(:, 2, 2, 8) = (/ &
     &1.0058e-04_rb,2.8759e-02_rb,5.7870e-02_rb,8.7575e-02_rb,1.1827e-01_rb,1.5089e-01_rb, &
     &1.8793e-01_rb,2.3994e-01_rb,2.3652e-01_rb/)
      kao(:, 3, 2, 8) = (/ &
     &1.0421e-04_rb,3.0964e-02_rb,6.2308e-02_rb,9.4299e-02_rb,1.2733e-01_rb,1.6243e-01_rb, &
     &2.0230e-01_rb,2.5862e-01_rb,2.5463e-01_rb/)
      kao(:, 4, 2, 8) = (/ &
     &1.0798e-04_rb,3.3179e-02_rb,6.6764e-02_rb,1.0101e-01_rb,1.3640e-01_rb,1.7400e-01_rb, &
     &2.1675e-01_rb,2.7715e-01_rb,2.7277e-01_rb/)
      kao(:, 5, 2, 8) = (/ &
     &1.1188e-04_rb,3.5361e-02_rb,7.1160e-02_rb,1.0767e-01_rb,1.4545e-01_rb,1.8554e-01_rb, &
     &2.3113e-01_rb,2.9555e-01_rb,2.9088e-01_rb/)
      kao(:, 1, 3, 8) = (/ &
     &1.7704e-04_rb,2.1958e-02_rb,4.4006e-02_rb,6.6294e-02_rb,8.8988e-02_rb,1.1245e-01_rb, &
     &1.3781e-01_rb,1.7014e-01_rb,1.7794e-01_rb/)
      kao(:, 2, 3, 8) = (/ &
     &1.8401e-04_rb,2.3898e-02_rb,4.7908e-02_rb,7.2164e-02_rb,9.6867e-02_rb,1.2239e-01_rb, &
     &1.4998e-01_rb,1.8528e-01_rb,1.9370e-01_rb/)
      kao(:, 3, 3, 8) = (/ &
     &1.9123e-04_rb,2.5882e-02_rb,5.1886e-02_rb,7.8168e-02_rb,1.0491e-01_rb,1.3257e-01_rb, &
     &1.6241e-01_rb,2.0047e-01_rb,2.0978e-01_rb/)
      kao(:, 4, 3, 8) = (/ &
     &1.9826e-04_rb,2.7841e-02_rb,5.5828e-02_rb,8.4093e-02_rb,1.1289e-01_rb,1.4266e-01_rb, &
     &1.7485e-01_rb,2.1582e-01_rb,2.2574e-01_rb/)
      kao(:, 5, 3, 8) = (/ &
     &2.0558e-04_rb,2.9781e-02_rb,5.9723e-02_rb,8.9997e-02_rb,1.2076e-01_rb,1.5262e-01_rb, &
     &1.8706e-01_rb,2.3095e-01_rb,2.4148e-01_rb/)
      kao(:, 1, 4, 8) = (/ &
     &3.2677e-04_rb,1.8554e-02_rb,3.5926e-02_rb,5.3829e-02_rb,7.1995e-02_rb,9.0469e-02_rb, &
     &1.0973e-01_rb,1.3198e-01_rb,1.4393e-01_rb/)
      kao(:, 2, 4, 8) = (/ &
     &3.3998e-04_rb,2.0214e-02_rb,3.9351e-02_rb,5.9051e-02_rb,7.8970e-02_rb,9.9220e-02_rb, &
     &1.2030e-01_rb,1.4461e-01_rb,1.5787e-01_rb/)
      kao(:, 3, 4, 8) = (/ &
     &3.5423e-04_rb,2.1886e-02_rb,4.2819e-02_rb,6.4313e-02_rb,8.6016e-02_rb,1.0809e-01_rb, &
     &1.3108e-01_rb,1.5763e-01_rb,1.7196e-01_rb/)
      kao(:, 4, 4, 8) = (/ &
     &3.6836e-04_rb,2.3526e-02_rb,4.6236e-02_rb,6.9481e-02_rb,9.2916e-02_rb,1.1677e-01_rb, &
     &1.4164e-01_rb,1.7039e-01_rb,1.8576e-01_rb/)
      kao(:, 5, 4, 8) = (/ &
     &3.8269e-04_rb,2.5159e-02_rb,4.9646e-02_rb,7.4614e-02_rb,9.9776e-02_rb,1.2541e-01_rb, &
     &1.5213e-01_rb,1.8307e-01_rb,1.9947e-01_rb/)
      kao(:, 1, 5, 8) = (/ &
     &5.5618e-04_rb,1.6693e-02_rb,3.0352e-02_rb,4.4004e-02_rb,5.8213e-02_rb,7.2865e-02_rb, &
     &8.7904e-02_rb,1.0425e-01_rb,1.1619e-01_rb/)
      kao(:, 2, 5, 8) = (/ &
     &5.8090e-04_rb,1.8251e-02_rb,3.3255e-02_rb,4.8452e-02_rb,6.4257e-02_rb,8.0492e-02_rb, &
     &9.7115e-02_rb,1.1513e-01_rb,1.2836e-01_rb/)
      kao(:, 3, 5, 8) = (/ &
     &6.0567e-04_rb,1.9782e-02_rb,3.6119e-02_rb,5.2854e-02_rb,7.0252e-02_rb,8.8045e-02_rb, &
     &1.0624e-01_rb,1.2602e-01_rb,1.4037e-01_rb/)
      kao(:, 4, 5, 8) = (/ &
     &6.3036e-04_rb,2.1279e-02_rb,3.8977e-02_rb,5.7257e-02_rb,7.6243e-02_rb,9.5550e-02_rb, &
     &1.1529e-01_rb,1.3680e-01_rb,1.5235e-01_rb/)
      kao(:, 5, 5, 8) = (/ &
     &6.5503e-04_rb,2.2767e-02_rb,4.1832e-02_rb,6.1655e-02_rb,8.2195e-02_rb,1.0302e-01_rb, &
     &1.2434e-01_rb,1.4756e-01_rb,1.6424e-01_rb/)
      kao(:, 1, 6, 8) = (/ &
     &8.8779e-04_rb,1.5219e-02_rb,2.6847e-02_rb,3.7621e-02_rb,4.8063e-02_rb,5.8817e-02_rb, &
     &7.0265e-02_rb,8.2696e-02_rb,9.3190e-02_rb/)
      kao(:, 2, 6, 8) = (/ &
     &9.2874e-04_rb,1.6673e-02_rb,2.9554e-02_rb,4.1458e-02_rb,5.3049e-02_rb,6.5160e-02_rb, &
     &7.8074e-02_rb,9.1921e-02_rb,1.0355e-01_rb/)
      kao(:, 3, 6, 8) = (/ &
     &9.6950e-04_rb,1.8111e-02_rb,3.2190e-02_rb,4.5211e-02_rb,5.8015e-02_rb,7.1500e-02_rb, &
     &8.5838e-02_rb,1.0107e-01_rb,1.1382e-01_rb/)
      kao(:, 4, 6, 8) = (/ &
     &1.0101e-03_rb,1.9522e-02_rb,3.4803e-02_rb,4.8905e-02_rb,6.3015e-02_rb,7.7875e-02_rb, &
     &9.3602e-02_rb,1.1021e-01_rb,1.2414e-01_rb/)
      kao(:, 5, 6, 8) = (/ &
     &1.0515e-03_rb,2.0916e-02_rb,3.7389e-02_rb,5.2547e-02_rb,6.7966e-02_rb,8.4205e-02_rb, &
     &1.0130e-01_rb,1.1932e-01_rb,1.3433e-01_rb/)
      kao(:, 1, 7, 8) = (/ &
     &1.4977e-03_rb,1.4733e-02_rb,2.4381e-02_rb,3.3428e-02_rb,4.1931e-02_rb,4.9753e-02_rb, &
     &5.7173e-02_rb,6.5580e-02_rb,7.4312e-02_rb/)
      kao(:, 2, 7, 8) = (/ &
     &1.5637e-03_rb,1.6075e-02_rb,2.6857e-02_rb,3.6980e-02_rb,4.6449e-02_rb,5.5152e-02_rb, &
     &6.3551e-02_rb,7.3261e-02_rb,8.3066e-02_rb/)
      kao(:, 3, 7, 8) = (/ &
     &1.6297e-03_rb,1.7399e-02_rb,2.9305e-02_rb,4.0484e-02_rb,5.0904e-02_rb,6.0470e-02_rb, &
     &6.9956e-02_rb,8.1009e-02_rb,9.1840e-02_rb/)
      kao(:, 4, 7, 8) = (/ &
     &1.7004e-03_rb,1.8688e-02_rb,3.1747e-02_rb,4.3982e-02_rb,5.5347e-02_rb,6.5760e-02_rb, &
     &7.6435e-02_rb,8.8798e-02_rb,1.0066e-01_rb/)
      kao(:, 5, 7, 8) = (/ &
     &1.7747e-03_rb,1.9942e-02_rb,3.4156e-02_rb,4.7424e-02_rb,5.9706e-02_rb,7.0996e-02_rb, &
     &8.2872e-02_rb,9.6510e-02_rb,1.0938e-01_rb/)
      kao(:, 1, 8, 8) = (/ &
     &3.0289e-03_rb,1.6187e-02_rb,2.4623e-02_rb,3.1800e-02_rb,3.8382e-02_rb,4.4473e-02_rb, &
     &4.9956e-02_rb,5.4111e-02_rb,5.8932e-02_rb/)
      kao(:, 2, 8, 8) = (/ &
     &3.1588e-03_rb,1.7601e-02_rb,2.6972e-02_rb,3.5067e-02_rb,4.2580e-02_rb,4.9539e-02_rb, &
     &5.5733e-02_rb,6.0374e-02_rb,6.6309e-02_rb/)
      kao(:, 3, 8, 8) = (/ &
     &3.2918e-03_rb,1.8992e-02_rb,2.9294e-02_rb,3.8315e-02_rb,4.6735e-02_rb,5.4548e-02_rb, &
     &6.1452e-02_rb,6.6677e-02_rb,7.3699e-02_rb/)
      kao(:, 4, 8, 8) = (/ &
     &3.4281e-03_rb,2.0362e-02_rb,3.1558e-02_rb,4.1571e-02_rb,5.0908e-02_rb,5.9537e-02_rb, &
     &6.7138e-02_rb,7.3015e-02_rb,8.1106e-02_rb/)
      kao(:, 5, 8, 8) = (/ &
     &3.5753e-03_rb,2.1737e-02_rb,3.3774e-02_rb,4.4733e-02_rb,5.4993e-02_rb,6.4441e-02_rb, &
     &7.2719e-02_rb,7.9338e-02_rb,8.8474e-02_rb/)
      kao(:, 1, 9, 8) = (/ &
     &1.1613e-02_rb,2.6600e-02_rb,3.4830e-02_rb,4.1045e-02_rb,4.5758e-02_rb,4.9187e-02_rb, &
     &5.0957e-02_rb,5.0881e-02_rb,4.6332e-02_rb/)
      kao(:, 2, 9, 8) = (/ &
     &1.2035e-02_rb,2.8837e-02_rb,3.7970e-02_rb,4.4834e-02_rb,5.0301e-02_rb,5.4274e-02_rb, &
     &5.6564e-02_rb,5.6979e-02_rb,5.2710e-02_rb/)
      kao(:, 3, 9, 8) = (/ &
     &1.2460e-02_rb,3.1118e-02_rb,4.1014e-02_rb,4.8606e-02_rb,5.4716e-02_rb,5.9241e-02_rb, &
     &6.2128e-02_rb,6.3036e-02_rb,5.8975e-02_rb/)
      kao(:, 4, 9, 8) = (/ &
     &1.2980e-02_rb,3.3463e-02_rb,4.4127e-02_rb,5.2378e-02_rb,5.9008e-02_rb,6.4138e-02_rb, &
     &6.7667e-02_rb,6.9075e-02_rb,6.5274e-02_rb/)
      kao(:, 5, 9, 8) = (/ &
     &1.3559e-02_rb,3.5809e-02_rb,4.7263e-02_rb,5.6212e-02_rb,6.3302e-02_rb,6.8912e-02_rb, &
     &7.3069e-02_rb,7.5010e-02_rb,7.1590e-02_rb/)
      kao(:, 1,10, 8) = (/ &
     &4.9669e-02_rb,6.5649e-02_rb,7.1116e-02_rb,7.3534e-02_rb,7.3825e-02_rb,7.2083e-02_rb, &
     &6.7844e-02_rb,5.9498e-02_rb,3.3788e-02_rb/)
      kao(:, 2,10, 8) = (/ &
     &5.1505e-02_rb,7.0624e-02_rb,7.6814e-02_rb,7.9826e-02_rb,8.0618e-02_rb,7.8892e-02_rb, &
     &7.4439e-02_rb,6.5949e-02_rb,3.8529e-02_rb/)
      kao(:, 3,10, 8) = (/ &
     &5.3297e-02_rb,7.5384e-02_rb,8.2789e-02_rb,8.6542e-02_rb,8.7369e-02_rb,8.5820e-02_rb, &
     &8.1089e-02_rb,7.2258e-02_rb,4.3360e-02_rb/)
      kao(:, 4,10, 8) = (/ &
     &5.5068e-02_rb,8.0008e-02_rb,8.8749e-02_rb,9.3221e-02_rb,9.4169e-02_rb,9.2692e-02_rb, &
     &8.7784e-02_rb,7.8437e-02_rb,4.8014e-02_rb/)
      kao(:, 5,10, 8) = (/ &
     &5.7734e-02_rb,8.4667e-02_rb,9.4556e-02_rb,9.9634e-02_rb,1.0118e-01_rb,9.9703e-02_rb, &
     &9.4571e-02_rb,8.4495e-02_rb,5.2667e-02_rb/)
      kao(:, 1,11, 8) = (/ &
     &7.6448e-02_rb,9.0381e-02_rb,9.1555e-02_rb,9.0047e-02_rb,8.6717e-02_rb,8.1217e-02_rb, &
     &7.3241e-02_rb,6.1179e-02_rb,3.1726e-02_rb/)
      kao(:, 2,11, 8) = (/ &
     &7.9612e-02_rb,9.5621e-02_rb,9.8574e-02_rb,9.7505e-02_rb,9.4314e-02_rb,8.8768e-02_rb, &
     &8.0204e-02_rb,6.7360e-02_rb,3.6181e-02_rb/)
      kao(:, 3,11, 8) = (/ &
     &8.2652e-02_rb,1.0183e-01_rb,1.0547e-01_rb,1.0483e-01_rb,1.0201e-01_rb,9.6508e-02_rb, &
     &8.7273e-02_rb,7.3474e-02_rb,4.0817e-02_rb/)
      kao(:, 4,11, 8) = (/ &
     &8.5862e-02_rb,1.0806e-01_rb,1.1212e-01_rb,1.1216e-01_rb,1.0950e-01_rb,1.0402e-01_rb, &
     &9.4626e-02_rb,7.9735e-02_rb,4.5328e-02_rb/)
      kao(:, 5,11, 8) = (/ &
     &8.9572e-02_rb,1.1434e-01_rb,1.1896e-01_rb,1.1939e-01_rb,1.1678e-01_rb,1.1141e-01_rb, &
     &1.0160e-01_rb,8.6030e-02_rb,4.9522e-02_rb/)
      kao(:, 1,12, 8) = (/ &
     &8.7861e-02_rb,9.9993e-02_rb,9.9171e-02_rb,9.4628e-02_rb,8.8704e-02_rb,8.0579e-02_rb, &
     &7.0917e-02_rb,5.7333e-02_rb,2.8393e-02_rb/)
      kao(:, 2,12, 8) = (/ &
     &9.1432e-02_rb,1.0608e-01_rb,1.0583e-01_rb,1.0218e-01_rb,9.6300e-02_rb,8.8064e-02_rb, &
     &7.7816e-02_rb,6.3024e-02_rb,3.2578e-02_rb/)
      kao(:, 3,12, 8) = (/ &
     &9.5110e-02_rb,1.1196e-01_rb,1.1325e-01_rb,1.0996e-01_rb,1.0369e-01_rb,9.5361e-02_rb, &
     &8.4644e-02_rb,6.8902e-02_rb,3.7335e-02_rb/)
      kao(:, 4,12, 8) = (/ &
     &9.9318e-02_rb,1.1859e-01_rb,1.2042e-01_rb,1.1748e-01_rb,1.1108e-01_rb,1.0241e-01_rb, &
     &9.1344e-02_rb,7.4779e-02_rb,4.1207e-02_rb/)
      kao(:, 5,12, 8) = (/ &
     &1.0395e-01_rb,1.2551e-01_rb,1.2774e-01_rb,1.2502e-01_rb,1.1865e-01_rb,1.0948e-01_rb, &
     &9.7810e-02_rb,8.0483e-02_rb,4.4886e-02_rb/)
      kao(:, 1,13, 8) = (/ &
     &7.9847e-02_rb,9.1714e-02_rb,9.0958e-02_rb,8.6422e-02_rb,8.0456e-02_rb,7.2533e-02_rb, &
     &6.2189e-02_rb,4.9649e-02_rb,2.4638e-02_rb/)
      kao(:, 2,13, 8) = (/ &
     &8.3442e-02_rb,9.7407e-02_rb,9.6872e-02_rb,9.3129e-02_rb,8.7376e-02_rb,7.8985e-02_rb, &
     &6.8360e-02_rb,5.4702e-02_rb,2.8608e-02_rb/)
      kao(:, 3,13, 8) = (/ &
     &8.7437e-02_rb,1.0293e-01_rb,1.0345e-01_rb,9.9852e-02_rb,9.3864e-02_rb,8.5454e-02_rb, &
     &7.4382e-02_rb,5.9885e-02_rb,3.2171e-02_rb/)
      kao(:, 4,13, 8) = (/ &
     &9.2235e-02_rb,1.0932e-01_rb,1.1033e-01_rb,1.0673e-01_rb,1.0043e-01_rb,9.1575e-02_rb, &
     &8.0229e-02_rb,6.4806e-02_rb,3.5846e-02_rb/)
      kao(:, 5,13, 8) = (/ &
     &9.7358e-02_rb,1.1603e-01_rb,1.1725e-01_rb,1.1373e-01_rb,1.0722e-01_rb,9.7942e-02_rb, &
     &8.5906e-02_rb,6.9573e-02_rb,3.9453e-02_rb/)
      kao(:, 1, 1, 9) = (/ &
     &2.3734e-04_rb,1.2635e-01_rb,2.5378e-01_rb,3.8280e-01_rb,5.1449e-01_rb,6.5095e-01_rb, &
     &7.9709e-01_rb,9.7059e-01_rb,1.0290e+00_rb/)
      kao(:, 2, 1, 9) = (/ &
     &2.4162e-04_rb,1.3562e-01_rb,2.7235e-01_rb,4.1078e-01_rb,5.5198e-01_rb,6.9838e-01_rb, &
     &8.5534e-01_rb,1.0400e+00_rb,1.1040e+00_rb/)
      kao(:, 3, 1, 9) = (/ &
     &2.4599e-04_rb,1.4459e-01_rb,2.9042e-01_rb,4.3810e-01_rb,5.8884e-01_rb,7.4499e-01_rb, &
     &9.1233e-01_rb,1.1085e+00_rb,1.1777e+00_rb/)
      kao(:, 4, 1, 9) = (/ &
     &2.5041e-04_rb,1.5330e-01_rb,3.0793e-01_rb,4.6450e-01_rb,6.2431e-01_rb,7.9002e-01_rb, &
     &9.6789e-01_rb,1.1761e+00_rb,1.2486e+00_rb/)
      kao(:, 5, 1, 9) = (/ &
     &2.5474e-04_rb,1.6182e-01_rb,3.2498e-01_rb,4.9034e-01_rb,6.5899e-01_rb,8.3368e-01_rb, &
     &1.0214e+00_rb,1.2421e+00_rb,1.3180e+00_rb/)
      kao(:, 1, 2, 9) = (/ &
     &3.4870e-04_rb,1.1450e-01_rb,2.2975e-01_rb,3.4621e-01_rb,4.6468e-01_rb,5.8691e-01_rb, &
     &7.1765e-01_rb,8.7423e-01_rb,9.2933e-01_rb/)
      kao(:, 2, 2, 9) = (/ &
     &3.5536e-04_rb,1.2334e-01_rb,2.4755e-01_rb,3.7300e-01_rb,5.0064e-01_rb,6.3232e-01_rb, &
     &7.7280e-01_rb,9.4059e-01_rb,1.0013e+00_rb/)
      kao(:, 3, 2, 9) = (/ &
     &3.6217e-04_rb,1.3183e-01_rb,2.6460e-01_rb,3.9876e-01_rb,5.3531e-01_rb,6.7635e-01_rb, &
     &8.2730e-01_rb,1.0064e+00_rb,1.0706e+00_rb/)
      kao(:, 4, 2, 9) = (/ &
     &3.6933e-04_rb,1.4017e-01_rb,2.8129e-01_rb,4.2395e-01_rb,5.6909e-01_rb,7.1898e-01_rb, &
     &8.7939e-01_rb,1.0712e+00_rb,1.1382e+00_rb/)
      kao(:, 5, 2, 9) = (/ &
     &3.7836e-04_rb,1.4828e-01_rb,2.9766e-01_rb,4.4860e-01_rb,6.0234e-01_rb,7.6096e-01_rb, &
     &9.3060e-01_rb,1.1332e+00_rb,1.2047e+00_rb/)
      kao(:, 1, 3, 9) = (/ &
     &6.8597e-04_rb,1.0077e-01_rb,2.0189e-01_rb,3.0366e-01_rb,4.0641e-01_rb,5.1122e-01_rb, &
     &6.2085e-01_rb,7.4738e-01_rb,8.1281e-01_rb/)
      kao(:, 2, 3, 9) = (/ &
     &6.9976e-04_rb,1.0900e-01_rb,2.1840e-01_rb,3.2848e-01_rb,4.3972e-01_rb,5.5316e-01_rb, &
     &6.7192e-01_rb,8.0915e-01_rb,8.7943e-01_rb/)
      kao(:, 3, 3, 9) = (/ &
     &7.1420e-04_rb,1.1699e-01_rb,2.3444e-01_rb,3.5259e-01_rb,4.7192e-01_rb,5.9381e-01_rb, &
     &7.2140e-01_rb,8.6946e-01_rb,9.4382e-01_rb/)
      kao(:, 4, 3, 9) = (/ &
     &7.3229e-04_rb,1.2483e-01_rb,2.5014e-01_rb,3.7620e-01_rb,5.0367e-01_rb,6.3374e-01_rb, &
     &7.7001e-01_rb,9.2785e-01_rb,1.0073e+00_rb/)
      kao(:, 5, 3, 9) = (/ &
     &7.5049e-04_rb,1.3237e-01_rb,2.6526e-01_rb,3.9901e-01_rb,5.3417e-01_rb,6.7225e-01_rb, &
     &8.1699e-01_rb,9.8549e-01_rb,1.0683e+00_rb/)
      kao(:, 1, 4, 9) = (/ &
     &1.3742e-03_rb,8.7179e-02_rb,1.7451e-01_rb,2.6209e-01_rb,3.5019e-01_rb,4.3920e-01_rb, &
     &5.3064e-01_rb,6.3091e-01_rb,7.0034e-01_rb/)
      kao(:, 2, 4, 9) = (/ &
     &1.4034e-03_rb,9.4872e-02_rb,1.8987e-01_rb,2.8522e-01_rb,3.8105e-01_rb,4.7796e-01_rb, &
     &5.7741e-01_rb,6.8650e-01_rb,7.6206e-01_rb/)
      kao(:, 3, 4, 9) = (/ &
     &1.4371e-03_rb,1.0231e-01_rb,2.0478e-01_rb,3.0758e-01_rb,4.1096e-01_rb,5.1550e-01_rb, &
     &6.2287e-01_rb,7.4083e-01_rb,8.2187e-01_rb/)
      kao(:, 4, 4, 9) = (/ &
     &1.4753e-03_rb,1.0953e-01_rb,2.1924e-01_rb,3.2932e-01_rb,4.4000e-01_rb,5.5196e-01_rb, &
     &6.6692e-01_rb,7.9351e-01_rb,8.7995e-01_rb/)
      kao(:, 5, 4, 9) = (/ &
     &1.5185e-03_rb,1.1661e-01_rb,2.3342e-01_rb,3.5060e-01_rb,4.6845e-01_rb,5.8764e-01_rb, &
     &7.1008e-01_rb,8.4502e-01_rb,9.3684e-01_rb/)
      kao(:, 1, 5, 9) = (/ &
     &2.5355e-03_rb,7.5685e-02_rb,1.4908e-01_rb,2.2375e-01_rb,2.9863e-01_rb,3.7400e-01_rb, &
     &4.5045e-01_rb,5.3139e-01_rb,5.9720e-01_rb/)
      kao(:, 2, 5, 9) = (/ &
     &2.5953e-03_rb,8.2577e-02_rb,1.6333e-01_rb,2.4515e-01_rb,3.2721e-01_rb,4.0982e-01_rb, &
     &4.9365e-01_rb,5.8247e-01_rb,6.5435e-01_rb/)
      kao(:, 3, 5, 9) = (/ &
     &2.6636e-03_rb,8.9208e-02_rb,1.7705e-01_rb,2.6573e-01_rb,3.5473e-01_rb,4.4427e-01_rb, &
     &5.3522e-01_rb,6.3160e-01_rb,7.0937e-01_rb/)
      kao(:, 4, 5, 9) = (/ &
     &2.7420e-03_rb,9.5746e-02_rb,1.9046e-01_rb,2.8588e-01_rb,3.8160e-01_rb,4.7792e-01_rb, &
     &5.7573e-01_rb,6.7936e-01_rb,7.6310e-01_rb/)
      kao(:, 5, 5, 9) = (/ &
     &2.8238e-03_rb,1.0214e-01_rb,2.0360e-01_rb,3.0562e-01_rb,4.0796e-01_rb,5.1095e-01_rb, &
     &6.1550e-01_rb,7.2630e-01_rb,8.1581e-01_rb/)
      kao(:, 1, 6, 9) = (/ &
     &4.3605e-03_rb,6.9106e-02_rb,1.2775e-01_rb,1.8881e-01_rb,2.5131e-01_rb,3.1447e-01_rb, &
     &3.7819e-01_rb,4.4408e-01_rb,5.0252e-01_rb/)
      kao(:, 2, 6, 9) = (/ &
     &4.4677e-03_rb,7.5202e-02_rb,1.4031e-01_rb,2.0810e-01_rb,2.7735e-01_rb,3.4706e-01_rb, &
     &4.1739e-01_rb,4.9016e-01_rb,5.5457e-01_rb/)
      kao(:, 3, 6, 9) = (/ &
     &4.5887e-03_rb,8.1094e-02_rb,1.5254e-01_rb,2.2684e-01_rb,3.0254e-01_rb,3.7858e-01_rb, &
     &4.5531e-01_rb,5.3473e-01_rb,6.0493e-01_rb/)
      kao(:, 4, 6, 9) = (/ &
     &4.7247e-03_rb,8.6870e-02_rb,1.6450e-01_rb,2.4527e-01_rb,3.2719e-01_rb,4.0945e-01_rb, &
     &4.9240e-01_rb,5.7834e-01_rb,6.5421e-01_rb/)
      kao(:, 5, 6, 9) = (/ &
     &4.8692e-03_rb,9.2597e-02_rb,1.7638e-01_rb,2.6357e-01_rb,3.5163e-01_rb,4.4001e-01_rb, &
     &5.2913e-01_rb,6.2153e-01_rb,7.0308e-01_rb/)
      kao(:, 1, 7, 9) = (/ &
     &7.8126e-03_rb,6.9794e-02_rb,1.1632e-01_rb,1.6292e-01_rb,2.1140e-01_rb,2.6176e-01_rb, &
     &3.1398e-01_rb,3.6765e-01_rb,4.1782e-01_rb/)
      kao(:, 2, 7, 9) = (/ &
     &8.0155e-03_rb,7.6059e-02_rb,1.2742e-01_rb,1.7989e-01_rb,2.3439e-01_rb,2.9098e-01_rb, &
     &3.4941e-01_rb,4.0917e-01_rb,4.6495e-01_rb/)
      kao(:, 3, 7, 9) = (/ &
     &8.2416e-03_rb,8.1970e-02_rb,1.3813e-01_rb,1.9625e-01_rb,2.5659e-01_rb,3.1926e-01_rb, &
     &3.8354e-01_rb,4.4917e-01_rb,5.1030e-01_rb/)
      kao(:, 4, 7, 9) = (/ &
     &8.4921e-03_rb,8.7603e-02_rb,1.4873e-01_rb,2.1252e-01_rb,2.7865e-01_rb,3.4736e-01_rb, &
     &4.1730e-01_rb,4.8866e-01_rb,5.5524e-01_rb/)
      kao(:, 5, 7, 9) = (/ &
     &8.7515e-03_rb,9.3044e-02_rb,1.5925e-01_rb,2.2867e-01_rb,3.0064e-01_rb,3.7532e-01_rb, &
     &4.5091e-01_rb,5.2808e-01_rb,5.9993e-01_rb/)
      kao(:, 1, 8, 9) = (/ &
     &1.6603e-02_rb,7.8794e-02_rb,1.2181e-01_rb,1.5872e-01_rb,1.9167e-01_rb,2.2528e-01_rb, &
     &2.6122e-01_rb,3.0128e-01_rb,3.4340e-01_rb/)
      kao(:, 2, 8, 9) = (/ &
     &1.7055e-02_rb,8.5849e-02_rb,1.3372e-01_rb,1.7446e-01_rb,2.1153e-01_rb,2.5044e-01_rb, &
     &2.9195e-01_rb,3.3805e-01_rb,3.8527e-01_rb/)
      kao(:, 3, 8, 9) = (/ &
     &1.7594e-02_rb,9.2738e-02_rb,1.4493e-01_rb,1.8895e-01_rb,2.3080e-01_rb,2.7486e-01_rb, &
     &3.2173e-01_rb,3.7362e-01_rb,4.2584e-01_rb/)
      kao(:, 4, 8, 9) = (/ &
     &1.8162e-02_rb,9.9290e-02_rb,1.5571e-01_rb,2.0324e-01_rb,2.5013e-01_rb,2.9946e-01_rb, &
     &3.5175e-01_rb,4.0935e-01_rb,4.6655e-01_rb/)
      kao(:, 5, 8, 9) = (/ &
     &1.8749e-02_rb,1.0557e-01_rb,1.6622e-01_rb,2.1745e-01_rb,2.6934e-01_rb,3.2386e-01_rb, &
     &3.8166e-01_rb,4.4484e-01_rb,5.0696e-01_rb/)
      kao(:, 1, 9, 9) = (/ &
     &6.5840e-02_rb,1.3860e-01_rb,1.7838e-01_rb,2.0998e-01_rb,2.3508e-01_rb,2.5392e-01_rb, &
     &2.6499e-01_rb,2.6488e-01_rb,2.7801e-01_rb/)
      kao(:, 2, 9, 9) = (/ &
     &6.7872e-02_rb,1.4849e-01_rb,1.9371e-01_rb,2.2975e-01_rb,2.5869e-01_rb,2.8072e-01_rb, &
     &2.9363e-01_rb,2.9601e-01_rb,3.1493e-01_rb/)
      kao(:, 3, 9, 9) = (/ &
     &7.0225e-02_rb,1.5836e-01_rb,2.0886e-01_rb,2.4918e-01_rb,2.8159e-01_rb,3.0644e-01_rb, &
     &3.2072e-01_rb,3.2657e-01_rb,3.5112e-01_rb/)
      kao(:, 4, 9, 9) = (/ &
     &7.2560e-02_rb,1.6804e-01_rb,2.2356e-01_rb,2.6776e-01_rb,3.0359e-01_rb,3.3128e-01_rb, &
     &3.4695e-01_rb,3.5732e-01_rb,3.8745e-01_rb/)
      kao(:, 5, 9, 9) = (/ &
     &7.5125e-02_rb,1.7750e-01_rb,2.3755e-01_rb,2.8560e-01_rb,3.2501e-01_rb,3.5553e-01_rb, &
     &3.7247e-01_rb,3.8768e-01_rb,4.2319e-01_rb/)
      kao(:, 1,10, 9) = (/ &
     &2.8983e-01_rb,3.6990e-01_rb,4.0037e-01_rb,4.0949e-01_rb,4.0589e-01_rb,3.9081e-01_rb, &
     &3.6404e-01_rb,3.1914e-01_rb,2.2264e-01_rb/)
      kao(:, 2,10, 9) = (/ &
     &2.9917e-01_rb,3.8833e-01_rb,4.2499e-01_rb,4.3869e-01_rb,4.3868e-01_rb,4.2555e-01_rb, &
     &3.9991e-01_rb,3.5416e-01_rb,2.5485e-01_rb/)
      kao(:, 3,10, 9) = (/ &
     &3.0939e-01_rb,4.0713e-01_rb,4.4875e-01_rb,4.6776e-01_rb,4.7103e-01_rb,4.6014e-01_rb, &
     &4.3525e-01_rb,3.8827e-01_rb,2.8738e-01_rb/)
      kao(:, 4,10, 9) = (/ &
     &3.2110e-01_rb,4.2672e-01_rb,4.7284e-01_rb,4.9667e-01_rb,5.0290e-01_rb,4.9348e-01_rb, &
     &4.6953e-01_rb,4.2184e-01_rb,3.1974e-01_rb/)
      kao(:, 5,10, 9) = (/ &
     &3.3272e-01_rb,4.4730e-01_rb,4.9768e-01_rb,5.2532e-01_rb,5.3384e-01_rb,5.2588e-01_rb, &
     &5.0287e-01_rb,4.5423e-01_rb,3.5166e-01_rb/)
      kao(:, 1,11, 9) = (/ &
     &4.4524e-01_rb,5.1679e-01_rb,5.3684e-01_rb,5.3332e-01_rb,5.1092e-01_rb,4.7255e-01_rb, &
     &4.1877e-01_rb,3.4229e-01_rb,1.8048e-01_rb/)
      kao(:, 2,11, 9) = (/ &
     &4.5963e-01_rb,5.4202e-01_rb,5.6658e-01_rb,5.6626e-01_rb,5.4700e-01_rb,5.0983e-01_rb, &
     &4.5605e-01_rb,3.7725e-01_rb,2.0742e-01_rb/)
      kao(:, 3,11, 9) = (/ &
     &4.7588e-01_rb,5.6665e-01_rb,5.9649e-01_rb,5.9968e-01_rb,5.8254e-01_rb,5.4674e-01_rb, &
     &4.9265e-01_rb,4.1176e-01_rb,2.3443e-01_rb/)
      kao(:, 4,11, 9) = (/ &
     &4.9478e-01_rb,5.9201e-01_rb,6.2710e-01_rb,6.3330e-01_rb,6.1823e-01_rb,5.8344e-01_rb, &
     &5.2842e-01_rb,4.4573e-01_rb,2.6065e-01_rb/)
      kao(:, 5,11, 9) = (/ &
     &5.1485e-01_rb,6.2017e-01_rb,6.5811e-01_rb,6.6709e-01_rb,6.5355e-01_rb,6.1972e-01_rb, &
     &5.6393e-01_rb,4.7838e-01_rb,2.8720e-01_rb/)
      kao(:, 1,12, 9) = (/ &
     &5.0543e-01_rb,5.6903e-01_rb,5.8050e-01_rb,5.7018e-01_rb,5.4105e-01_rb,4.9466e-01_rb, &
     &4.2828e-01_rb,3.3843e-01_rb,1.4652e-01_rb/)
      kao(:, 2,12, 9) = (/ &
     &5.2298e-01_rb,5.9603e-01_rb,6.1306e-01_rb,6.0448e-01_rb,5.7710e-01_rb,5.3128e-01_rb, &
     &4.6400e-01_rb,3.7149e-01_rb,1.6865e-01_rb/)
      kao(:, 3,12, 9) = (/ &
     &5.4270e-01_rb,6.2472e-01_rb,6.4493e-01_rb,6.3902e-01_rb,6.1334e-01_rb,5.6783e-01_rb, &
     &4.9936e-01_rb,4.0428e-01_rb,1.8993e-01_rb/)
      kao(:, 4,12, 9) = (/ &
     &5.6576e-01_rb,6.5409e-01_rb,6.7814e-01_rb,6.7406e-01_rb,6.4973e-01_rb,6.0390e-01_rb, &
     &5.3494e-01_rb,4.3628e-01_rb,2.1164e-01_rb/)
      kao(:, 5,12, 9) = (/ &
     &5.9076e-01_rb,6.8665e-01_rb,7.1248e-01_rb,7.0991e-01_rb,6.8560e-01_rb,6.4007e-01_rb, &
     &5.7053e-01_rb,4.6743e-01_rb,2.3320e-01_rb/)
      kao(:, 1,13, 9) = (/ &
     &4.5979e-01_rb,5.1890e-01_rb,5.2855e-01_rb,5.1825e-01_rb,4.9146e-01_rb,4.4960e-01_rb, &
     &3.8971e-01_rb,3.0598e-01_rb,1.2566e-01_rb/)
      kao(:, 2,13, 9) = (/ &
     &4.7640e-01_rb,5.4480e-01_rb,5.5925e-01_rb,5.5059e-01_rb,5.2436e-01_rb,4.8283e-01_rb, &
     &4.2163e-01_rb,3.3565e-01_rb,1.4550e-01_rb/)
      kao(:, 3,13, 9) = (/ &
     &4.9565e-01_rb,5.7250e-01_rb,5.8989e-01_rb,5.8311e-01_rb,5.5799e-01_rb,5.1559e-01_rb, &
     &4.5382e-01_rb,3.6511e-01_rb,1.6523e-01_rb/)
      kao(:, 4,13, 9) = (/ &
     &5.1794e-01_rb,6.0150e-01_rb,6.2192e-01_rb,6.1629e-01_rb,5.9157e-01_rb,5.4908e-01_rb, &
     &4.8624e-01_rb,3.9414e-01_rb,1.8408e-01_rb/)
      kao(:, 5,13, 9) = (/ &
     &5.4127e-01_rb,6.3348e-01_rb,6.5477e-01_rb,6.5028e-01_rb,6.2556e-01_rb,5.8254e-01_rb, &
     &5.1808e-01_rb,4.2264e-01_rb,2.0283e-01_rb/)
      kao(:, 1, 1,10) = (/ &
     &6.5918e-04_rb,3.4888e-01_rb,6.9848e-01_rb,1.0485e+00_rb,1.4002e+00_rb,1.7549e+00_rb, &
     &2.1190e+00_rb,2.5078e+00_rb,2.8004e+00_rb/)
      kao(:, 2, 1,10) = (/ &
     &6.7280e-04_rb,3.7184e-01_rb,7.4416e-01_rb,1.1175e+00_rb,1.4925e+00_rb,1.8689e+00_rb, &
     &2.2475e+00_rb,2.6390e+00_rb,2.9850e+00_rb/)
      kao(:, 3, 1,10) = (/ &
     &6.8523e-04_rb,3.9709e-01_rb,7.9353e-01_rb,1.1893e+00_rb,1.5844e+00_rb,1.9796e+00_rb, &
     &2.3755e+00_rb,2.7728e+00_rb,3.1689e+00_rb/)
      kao(:, 4, 1,10) = (/ &
     &6.9619e-04_rb,4.2236e-01_rb,8.4460e-01_rb,1.2663e+00_rb,1.6869e+00_rb,2.1024e+00_rb, &
     &2.5075e+00_rb,2.8990e+00_rb,3.3737e+00_rb/)
      kao(:, 5, 1,10) = (/ &
     &7.0541e-04_rb,4.4505e-01_rb,8.9002e-01_rb,1.3353e+00_rb,1.7799e+00_rb,2.2235e+00_rb, &
     &2.6534e+00_rb,3.0275e+00_rb,3.5598e+00_rb/)
      kao(:, 1, 2,10) = (/ &
     &1.0168e-03_rb,3.3520e-01_rb,6.7064e-01_rb,1.0063e+00_rb,1.3424e+00_rb,1.6793e+00_rb, &
     &2.0199e+00_rb,2.3746e+00_rb,2.6847e+00_rb/)
      kao(:, 2, 2,10) = (/ &
     &1.0383e-03_rb,3.6090e-01_rb,7.2200e-01_rb,1.0832e+00_rb,1.4445e+00_rb,1.8057e+00_rb, &
     &2.1681e+00_rb,2.5457e+00_rb,2.8891e+00_rb/)
      kao(:, 3, 2,10) = (/ &
     &1.0584e-03_rb,3.8758e-01_rb,7.7508e-01_rb,1.1623e+00_rb,1.5490e+00_rb,1.9350e+00_rb, &
     &2.3176e+00_rb,2.7064e+00_rb,3.0980e+00_rb/)
      kao(:, 4, 2,10) = (/ &
     &1.0746e-03_rb,4.1292e-01_rb,8.2598e-01_rb,1.2389e+00_rb,1.6519e+00_rb,2.0646e+00_rb, &
     &2.4744e+00_rb,2.8678e+00_rb,3.3037e+00_rb/)
      kao(:, 5, 2,10) = (/ &
     &1.0744e-03_rb,4.3717e-01_rb,8.7417e-01_rb,1.3107e+00_rb,1.7471e+00_rb,2.1834e+00_rb, &
     &2.6192e+00_rb,3.0444e+00_rb,3.4941e+00_rb/)
      kao(:, 1, 3,10) = (/ &
     &2.0748e-03_rb,3.1600e-01_rb,6.3217e-01_rb,9.4857e-01_rb,1.2657e+00_rb,1.5838e+00_rb, &
     &1.9039e+00_rb,2.2287e+00_rb,2.5314e+00_rb/)
      kao(:, 2, 3,10) = (/ &
     &2.1217e-03_rb,3.4261e-01_rb,6.8545e-01_rb,1.0284e+00_rb,1.3716e+00_rb,1.7154e+00_rb, &
     &2.0606e+00_rb,2.4138e+00_rb,2.7432e+00_rb/)
      kao(:, 3, 3,10) = (/ &
     &2.1619e-03_rb,3.6999e-01_rb,7.4010e-01_rb,1.1107e+00_rb,1.4817e+00_rb,1.8526e+00_rb, &
     &2.2240e+00_rb,2.5942e+00_rb,2.9634e+00_rb/)
      kao(:, 4, 3,10) = (/ &
     &2.1755e-03_rb,3.9596e-01_rb,7.9208e-01_rb,1.1885e+00_rb,1.5854e+00_rb,1.9831e+00_rb, &
     &2.3825e+00_rb,2.7842e+00_rb,3.1709e+00_rb/)
      kao(:, 5, 3,10) = (/ &
     &2.1880e-03_rb,4.2191e-01_rb,8.4399e-01_rb,1.2664e+00_rb,1.6893e+00_rb,2.1130e+00_rb, &
     &2.5373e+00_rb,2.9615e+00_rb,3.3787e+00_rb/)
      kao(:, 1, 4,10) = (/ &
     &4.2494e-03_rb,2.8992e-01_rb,5.8012e-01_rb,8.7071e-01_rb,1.1620e+00_rb,1.4546e+00_rb, &
     &1.7505e+00_rb,2.0568e+00_rb,2.3239e+00_rb/)
      kao(:, 2, 4,10) = (/ &
     &4.3512e-03_rb,3.1537e-01_rb,6.3099e-01_rb,9.4695e-01_rb,1.2638e+00_rb,1.5819e+00_rb, &
     &1.9039e+00_rb,2.2390e+00_rb,2.5276e+00_rb/)
      kao(:, 3, 4,10) = (/ &
     &4.4136e-03_rb,3.4243e-01_rb,6.8516e-01_rb,1.0282e+00_rb,1.3719e+00_rb,1.7170e+00_rb, &
     &2.0650e+00_rb,2.4245e+00_rb,2.7438e+00_rb/)
      kao(:, 4, 4,10) = (/ &
     &4.4541e-03_rb,3.6913e-01_rb,7.3845e-01_rb,1.1085e+00_rb,1.4792e+00_rb,1.8513e+00_rb, &
     &2.2276e+00_rb,2.6166e+00_rb,2.9583e+00_rb/)
      kao(:, 5, 4,10) = (/ &
     &4.4637e-03_rb,3.9522e-01_rb,7.9055e-01_rb,1.1865e+00_rb,1.5830e+00_rb,1.9815e+00_rb, &
     &2.3838e+00_rb,2.7994e+00_rb,3.1659e+00_rb/)
      kao(:, 1, 5,10) = (/ &
     &7.8944e-03_rb,2.6027e-01_rb,5.2060e-01_rb,7.8129e-01_rb,1.0423e+00_rb,1.3045e+00_rb, &
     &1.5691e+00_rb,1.8427e+00_rb,2.0845e+00_rb/)
      kao(:, 2, 5,10) = (/ &
     &8.0572e-03_rb,2.8413e-01_rb,5.6833e-01_rb,8.5287e-01_rb,1.1378e+00_rb,1.4238e+00_rb, &
     &1.7119e+00_rb,2.0096e+00_rb,2.2754e+00_rb/)
      kao(:, 3, 5,10) = (/ &
     &8.1742e-03_rb,3.1030e-01_rb,6.2054e-01_rb,9.3122e-01_rb,1.2421e+00_rb,1.5541e+00_rb, &
     &1.8685e+00_rb,2.1911e+00_rb,2.4841e+00_rb/)
      kao(:, 4, 5,10) = (/ &
     &8.2414e-03_rb,3.3618e-01_rb,6.7234e-01_rb,1.0088e+00_rb,1.3458e+00_rb,1.6840e+00_rb, &
     &2.0247e+00_rb,2.3758e+00_rb,2.6915e+00_rb/)
      kao(:, 5, 5,10) = (/ &
     &8.3003e-03_rb,3.6135e-01_rb,7.2289e-01_rb,1.0847e+00_rb,1.4472e+00_rb,1.8109e+00_rb, &
     &2.1782e+00_rb,2.5572e+00_rb,2.8943e+00_rb/)
      kao(:, 1, 6,10) = (/ &
     &1.3769e-02_rb,2.2868e-01_rb,4.5729e-01_rb,6.8615e-01_rb,9.1532e-01_rb,1.1447e+00_rb, &
     &1.3758e+00_rb,1.6126e+00_rb,1.8305e+00_rb/)
      kao(:, 2, 6,10) = (/ &
     &1.4077e-02_rb,2.5186e-01_rb,5.0368e-01_rb,7.5558e-01_rb,1.0079e+00_rb,1.2605e+00_rb, &
     &1.5146e+00_rb,1.7737e+00_rb,2.0155e+00_rb/)
      kao(:, 3, 6,10) = (/ &
     &1.4339e-02_rb,2.7680e-01_rb,5.5350e-01_rb,8.3033e-01_rb,1.1075e+00_rb,1.3852e+00_rb, &
     &1.6642e+00_rb,1.9484e+00_rb,2.2148e+00_rb/)
      kao(:, 4, 6,10) = (/ &
     &1.4519e-02_rb,3.0142e-01_rb,6.0270e-01_rb,9.0418e-01_rb,1.2061e+00_rb,1.5083e+00_rb, &
     &1.8129e+00_rb,2.1232e+00_rb,2.4121e+00_rb/)
      kao(:, 5, 6,10) = (/ &
     &1.4692e-02_rb,3.2398e-01_rb,6.4788e-01_rb,9.7210e-01_rb,1.2967e+00_rb,1.6219e+00_rb, &
     &1.9490e+00_rb,2.2841e+00_rb,2.5932e+00_rb/)
      kao(:, 1, 7,10) = (/ &
     &2.5736e-02_rb,2.1730e-01_rb,3.9499e-01_rb,5.9240e-01_rb,7.9003e-01_rb,9.8783e-01_rb, &
     &1.1864e+00_rb,1.3877e+00_rb,1.5798e+00_rb/)
      kao(:, 2, 7,10) = (/ &
     &2.6391e-02_rb,2.3410e-01_rb,4.4100e-01_rb,6.6147e-01_rb,8.8204e-01_rb,1.1029e+00_rb, &
     &1.3242e+00_rb,1.5482e+00_rb,1.7637e+00_rb/)
      kao(:, 3, 7,10) = (/ &
     &2.6951e-02_rb,2.5201e-01_rb,4.8957e-01_rb,7.3433e-01_rb,9.7905e-01_rb,1.2243e+00_rb, &
     &1.4699e+00_rb,1.7185e+00_rb,1.9578e+00_rb/)
      kao(:, 4, 7,10) = (/ &
     &2.7390e-02_rb,2.7020e-01_rb,5.3407e-01_rb,8.0106e-01_rb,1.0682e+00_rb,1.3359e+00_rb, &
     &1.6042e+00_rb,1.8761e+00_rb,2.1361e+00_rb/)
      kao(:, 5, 7,10) = (/ &
     &2.8064e-02_rb,2.8861e-01_rb,5.7611e-01_rb,8.6397e-01_rb,1.1521e+00_rb,1.4408e+00_rb, &
     &1.7300e+00_rb,2.0229e+00_rb,2.3040e+00_rb/)
      kao(:, 1, 8,10) = (/ &
     &5.8578e-02_rb,2.6880e-01_rb,4.0001e-01_rb,5.2061e-01_rb,6.7938e-01_rb,8.4922e-01_rb, &
     &1.0194e+00_rb,1.1905e+00_rb,1.3581e+00_rb/)
      kao(:, 2, 8,10) = (/ &
     &6.0087e-02_rb,2.9139e-01_rb,4.3501e-01_rb,5.7583e-01_rb,7.6491e-01_rb,9.5619e-01_rb, &
     &1.1477e+00_rb,1.3402e+00_rb,1.5292e+00_rb/)
      kao(:, 3, 8,10) = (/ &
     &6.1083e-02_rb,3.1400e-01_rb,4.7010e-01_rb,6.4266e-01_rb,8.5685e-01_rb,1.0710e+00_rb, &
     &1.2856e+00_rb,1.5014e+00_rb,1.7131e+00_rb/)
      kao(:, 4, 8,10) = (/ &
     &6.2072e-02_rb,3.3715e-01_rb,5.0004e-01_rb,7.0498e-01_rb,9.3980e-01_rb,1.1748e+00_rb, &
     &1.4102e+00_rb,1.6468e+00_rb,1.8789e+00_rb/)
      kao(:, 5, 8,10) = (/ &
     &6.3481e-02_rb,3.5691e-01_rb,5.2888e-01_rb,7.6432e-01_rb,1.0191e+00_rb,1.2740e+00_rb, &
     &1.5290e+00_rb,1.7860e+00_rb,2.0375e+00_rb/)
      kao(:, 1, 9,10) = (/ &
     &2.5051e-01_rb,4.8216e-01_rb,6.3956e-01_rb,7.5081e-01_rb,8.3417e-01_rb,8.8639e-01_rb, &
     &9.0389e-01_rb,1.0106e+00_rb,1.1541e+00_rb/)
      kao(:, 2, 9,10) = (/ &
     &2.5603e-01_rb,5.1856e-01_rb,6.9227e-01_rb,8.2007e-01_rb,9.1537e-01_rb,9.7189e-01_rb, &
     &9.9778e-01_rb,1.1454e+00_rb,1.3083e+00_rb/)
      kao(:, 3, 9,10) = (/ &
     &2.6013e-01_rb,5.5198e-01_rb,7.4416e-01_rb,8.8806e-01_rb,9.9547e-01_rb,1.0583e+00_rb, &
     &1.1091e+00_rb,1.2896e+00_rb,1.4729e+00_rb/)
      kao(:, 4, 9,10) = (/ &
     &2.6455e-01_rb,5.8224e-01_rb,7.9653e-01_rb,9.5696e-01_rb,1.0752e+00_rb,1.1310e+00_rb, &
     &1.2203e+00_rb,1.4235e+00_rb,1.6258e+00_rb/)
      kao(:, 5, 9,10) = (/ &
     &2.6779e-01_rb,6.1167e-01_rb,8.4854e-01_rb,1.0208e+00_rb,1.1434e+00_rb,1.1964e+00_rb, &
     &1.3330e+00_rb,1.5553e+00_rb,1.7761e+00_rb/)
      kao(:, 1,10,10) = (/ &
     &1.1877e+00_rb,1.3423e+00_rb,1.4114e+00_rb,1.4638e+00_rb,1.4747e+00_rb,1.4384e+00_rb, &
     &1.3561e+00_rb,1.1884e+00_rb,9.6724e-01_rb/)
      kao(:, 2,10,10) = (/ &
     &1.2130e+00_rb,1.4001e+00_rb,1.4986e+00_rb,1.5677e+00_rb,1.5910e+00_rb,1.5689e+00_rb, &
     &1.4918e+00_rb,1.3127e+00_rb,1.1039e+00_rb/)
      kao(:, 3,10,10) = (/ &
     &1.2346e+00_rb,1.4593e+00_rb,1.5877e+00_rb,1.6627e+00_rb,1.7012e+00_rb,1.6974e+00_rb, &
     &1.6231e+00_rb,1.4376e+00_rb,1.2384e+00_rb/)
      kao(:, 4,10,10) = (/ &
     &1.2516e+00_rb,1.5151e+00_rb,1.6747e+00_rb,1.7501e+00_rb,1.8076e+00_rb,1.8250e+00_rb, &
     &1.7537e+00_rb,1.5512e+00_rb,1.3735e+00_rb/)
      kao(:, 5,10,10) = (/ &
     &1.2661e+00_rb,1.5638e+00_rb,1.7522e+00_rb,1.8374e+00_rb,1.9121e+00_rb,1.9420e+00_rb, &
     &1.8736e+00_rb,1.6582e+00_rb,1.5128e+00_rb/)
      kao(:, 1,11,10) = (/ &
     &1.9480e+00_rb,2.0492e+00_rb,2.0295e+00_rb,1.9735e+00_rb,1.8970e+00_rb,1.7736e+00_rb, &
     &1.5866e+00_rb,1.3339e+00_rb,8.4829e-01_rb/)
      kao(:, 2,11,10) = (/ &
     &1.9908e+00_rb,2.1211e+00_rb,2.1260e+00_rb,2.0949e+00_rb,2.0240e+00_rb,1.9060e+00_rb, &
     &1.7325e+00_rb,1.4683e+00_rb,9.6828e-01_rb/)
      kao(:, 3,11,10) = (/ &
     &2.0285e+00_rb,2.1879e+00_rb,2.2239e+00_rb,2.2116e+00_rb,2.1470e+00_rb,2.0366e+00_rb, &
     &1.8735e+00_rb,1.6014e+00_rb,1.0930e+00_rb/)
      kao(:, 4,11,10) = (/ &
     &2.0556e+00_rb,2.2562e+00_rb,2.3222e+00_rb,2.3233e+00_rb,2.2648e+00_rb,2.1592e+00_rb, &
     &2.0072e+00_rb,1.7228e+00_rb,1.2296e+00_rb/)
      kao(:, 5,11,10) = (/ &
     &2.0825e+00_rb,2.3069e+00_rb,2.4192e+00_rb,2.4269e+00_rb,2.3834e+00_rb,2.2768e+00_rb, &
     &2.1248e+00_rb,1.8445e+00_rb,1.3568e+00_rb/)
      kao(:, 1,12,10) = (/ &
     &2.3362e+00_rb,2.4153e+00_rb,2.3669e+00_rb,2.2577e+00_rb,2.1055e+00_rb,1.9147e+00_rb, &
     &1.6820e+00_rb,1.3486e+00_rb,7.4407e-01_rb/)
      kao(:, 2,12,10) = (/ &
     &2.3893e+00_rb,2.4937e+00_rb,2.4670e+00_rb,2.3773e+00_rb,2.2296e+00_rb,2.0482e+00_rb, &
     &1.8201e+00_rb,1.4800e+00_rb,8.5242e-01_rb/)
      kao(:, 3,12,10) = (/ &
     &2.4357e+00_rb,2.5702e+00_rb,2.5666e+00_rb,2.4887e+00_rb,2.3537e+00_rb,2.1812e+00_rb, &
     &1.9511e+00_rb,1.6049e+00_rb,9.6662e-01_rb/)
      kao(:, 4,12,10) = (/ &
     &2.4652e+00_rb,2.6456e+00_rb,2.6664e+00_rb,2.5981e+00_rb,2.4742e+00_rb,2.3104e+00_rb, &
     &2.0695e+00_rb,1.7245e+00_rb,1.0828e+00_rb/)
      kao(:, 5,12,10) = (/ &
     &2.4945e+00_rb,2.7001e+00_rb,2.7645e+00_rb,2.7014e+00_rb,2.5936e+00_rb,2.4312e+00_rb, &
     &2.1779e+00_rb,1.8427e+00_rb,1.2015e+00_rb/)
      kao(:, 1,13,10) = (/ &
     &2.2054e+00_rb,2.3120e+00_rb,2.2775e+00_rb,2.1807e+00_rb,2.0276e+00_rb,1.8297e+00_rb, &
     &1.5931e+00_rb,1.2531e+00_rb,5.9202e-01_rb/)
      kao(:, 2,13,10) = (/ &
     &2.2559e+00_rb,2.3878e+00_rb,2.3725e+00_rb,2.2888e+00_rb,2.1438e+00_rb,1.9536e+00_rb, &
     &1.7172e+00_rb,1.3694e+00_rb,6.7488e-01_rb/)
      kao(:, 3,13,10) = (/ &
     &2.2951e+00_rb,2.4612e+00_rb,2.4689e+00_rb,2.3923e+00_rb,2.2596e+00_rb,2.0732e+00_rb, &
     &1.8332e+00_rb,1.4777e+00_rb,7.6019e-01_rb/)
      kao(:, 4,13,10) = (/ &
     &2.3235e+00_rb,2.5286e+00_rb,2.5591e+00_rb,2.4947e+00_rb,2.3722e+00_rb,2.1862e+00_rb, &
     &1.9383e+00_rb,1.5857e+00_rb,8.5248e-01_rb/)
      kao(:, 5,13,10) = (/ &
     &2.3616e+00_rb,2.5838e+00_rb,2.6490e+00_rb,2.5951e+00_rb,2.4760e+00_rb,2.2908e+00_rb, &
     &2.0454e+00_rb,1.6895e+00_rb,9.4490e-01_rb/)
      kao(:, 1, 1,11) = (/ &
     &9.3294e-04_rb,5.3309e-01_rb,1.0647e+00_rb,1.5932e+00_rb,2.1160e+00_rb,2.6255e+00_rb, &
     &3.0986e+00_rb,3.3968e+00_rb,4.2319e+00_rb/)
      kao(:, 2, 1,11) = (/ &
     &9.5345e-04_rb,5.6455e-01_rb,1.1277e+00_rb,1.6875e+00_rb,2.2413e+00_rb,2.7825e+00_rb, &
     &3.2884e+00_rb,3.6383e+00_rb,4.4826e+00_rb/)
      kao(:, 3, 1,11) = (/ &
     &9.7241e-04_rb,5.9098e-01_rb,1.1810e+00_rb,1.7692e+00_rb,2.3540e+00_rb,2.9286e+00_rb, &
     &3.4675e+00_rb,3.8516e+00_rb,4.7080e+00_rb/)
      kao(:, 4, 1,11) = (/ &
     &9.8984e-04_rb,6.1800e-01_rb,1.2338e+00_rb,1.8462e+00_rb,2.4536e+00_rb,3.0526e+00_rb, &
     &3.6266e+00_rb,4.0600e+00_rb,4.9071e+00_rb/)
      kao(:, 5, 1,11) = (/ &
     &1.0059e-03_rb,6.4816e-01_rb,1.2933e+00_rb,1.9339e+00_rb,2.5658e+00_rb,3.1822e+00_rb, &
     &3.7668e+00_rb,4.2510e+00_rb,5.1316e+00_rb/)
      kao(:, 1, 2,11) = (/ &
     &1.5074e-03_rb,5.1913e-01_rb,1.0387e+00_rb,1.5585e+00_rb,2.0790e+00_rb,2.5998e+00_rb, &
     &3.1169e+00_rb,3.5984e+00_rb,4.1578e+00_rb/)
      kao(:, 2, 2,11) = (/ &
     &1.5417e-03_rb,5.5254e-01_rb,1.1053e+00_rb,1.6578e+00_rb,2.2101e+00_rb,2.7629e+00_rb, &
     &3.3129e+00_rb,3.8170e+00_rb,4.4201e+00_rb/)
      kao(:, 3, 2,11) = (/ &
     &1.5713e-03_rb,5.8438e-01_rb,1.1685e+00_rb,1.7524e+00_rb,2.3358e+00_rb,2.9174e+00_rb, &
     &3.4950e+00_rb,4.0242e+00_rb,4.6716e+00_rb/)
      kao(:, 4, 2,11) = (/ &
     &1.5983e-03_rb,6.1743e-01_rb,1.2340e+00_rb,1.8490e+00_rb,2.4617e+00_rb,3.0691e+00_rb, &
     &3.6659e+00_rb,4.2202e+00_rb,4.9233e+00_rb/)
      kao(:, 5, 2,11) = (/ &
     &1.6216e-03_rb,6.5216e-01_rb,1.3030e+00_rb,1.9521e+00_rb,2.5969e+00_rb,3.2326e+00_rb, &
     &3.8472e+00_rb,4.4004e+00_rb,5.1936e+00_rb/)
      kao(:, 1, 3,11) = (/ &
     &3.2100e-03_rb,4.8846e-01_rb,9.7712e-01_rb,1.4663e+00_rb,1.9565e+00_rb,2.4483e+00_rb, &
     &2.9445e+00_rb,3.4602e+00_rb,3.9130e+00_rb/)
      kao(:, 2, 3,11) = (/ &
     &3.2832e-03_rb,5.2715e-01_rb,1.0544e+00_rb,1.5820e+00_rb,2.1102e+00_rb,2.6392e+00_rb, &
     &3.1691e+00_rb,3.7028e+00_rb,4.2204e+00_rb/)
      kao(:, 3, 3,11) = (/ &
     &3.3509e-03_rb,5.6319e-01_rb,1.1265e+00_rb,1.6900e+00_rb,2.2533e+00_rb,2.8179e+00_rb, &
     &3.3838e+00_rb,3.9526e+00_rb,4.5066e+00_rb/)
      kao(:, 4, 3,11) = (/ &
     &3.4122e-03_rb,6.0075e-01_rb,1.2012e+00_rb,1.8012e+00_rb,2.4005e+00_rb,2.9988e+00_rb, &
     &3.5947e+00_rb,4.1839e+00_rb,4.8011e+00_rb/)
      kao(:, 5, 3,11) = (/ &
     &3.4694e-03_rb,6.3990e-01_rb,1.2794e+00_rb,1.9184e+00_rb,2.5559e+00_rb,3.1911e+00_rb, &
     &3.8197e+00_rb,4.4273e+00_rb,5.1119e+00_rb/)
      kao(:, 1, 4,11) = (/ &
     &6.8944e-03_rb,4.5524e-01_rb,9.1034e-01_rb,1.3656e+00_rb,1.8210e+00_rb,2.2768e+00_rb, &
     &2.7339e+00_rb,3.1966e+00_rb,3.6419e+00_rb/)
      kao(:, 2, 4,11) = (/ &
     &7.0719e-03_rb,4.9844e-01_rb,9.9694e-01_rb,1.4955e+00_rb,1.9943e+00_rb,2.4932e+00_rb, &
     &2.9929e+00_rb,3.4921e+00_rb,3.9886e+00_rb/)
      kao(:, 3, 4,11) = (/ &
     &7.2326e-03_rb,5.3850e-01_rb,1.0768e+00_rb,1.6152e+00_rb,2.1534e+00_rb,2.6917e+00_rb, &
     &3.2302e+00_rb,3.7700e+00_rb,4.3067e+00_rb/)
      kao(:, 4, 4,11) = (/ &
     &7.3806e-03_rb,5.7980e-01_rb,1.1594e+00_rb,1.7389e+00_rb,2.3180e+00_rb,2.8967e+00_rb, &
     &3.4739e+00_rb,4.0439e+00_rb,4.6359e+00_rb/)
      kao(:, 5, 4,11) = (/ &
     &7.4998e-03_rb,6.1978e-01_rb,1.2395e+00_rb,1.8590e+00_rb,2.4785e+00_rb,3.0979e+00_rb, &
     &3.7146e+00_rb,4.3244e+00_rb,4.9568e+00_rb/)
      kao(:, 1, 5,11) = (/ &
     &1.3442e-02_rb,4.2037e-01_rb,8.4065e-01_rb,1.2612e+00_rb,1.6819e+00_rb,2.1029e+00_rb, &
     &2.5255e+00_rb,2.9520e+00_rb,3.3637e+00_rb/)
      kao(:, 2, 5,11) = (/ &
     &1.3797e-02_rb,4.6626e-01_rb,9.3234e-01_rb,1.3987e+00_rb,1.8650e+00_rb,2.3319e+00_rb, &
     &2.7996e+00_rb,3.2699e+00_rb,3.7300e+00_rb/)
      kao(:, 3, 5,11) = (/ &
     &1.4112e-02_rb,5.0925e-01_rb,1.0185e+00_rb,1.5280e+00_rb,2.0375e+00_rb,2.5478e+00_rb, &
     &3.0591e+00_rb,3.5748e+00_rb,4.0749e+00_rb/)
      kao(:, 4, 5,11) = (/ &
     &1.4385e-02_rb,5.5077e-01_rb,1.1015e+00_rb,1.6526e+00_rb,2.2034e+00_rb,2.7548e+00_rb, &
     &3.3072e+00_rb,3.8625e+00_rb,4.4066e+00_rb/)
      kao(:, 5, 5,11) = (/ &
     &1.4645e-02_rb,5.9271e-01_rb,1.1855e+00_rb,1.7781e+00_rb,2.3710e+00_rb,2.9641e+00_rb, &
     &3.5574e+00_rb,4.1512e+00_rb,4.7419e+00_rb/)
      kao(:, 1, 6,11) = (/ &
     &2.4122e-02_rb,3.8084e-01_rb,7.6147e-01_rb,1.1422e+00_rb,1.5231e+00_rb,1.9045e+00_rb, &
     &2.2870e+00_rb,2.6730e+00_rb,3.0460e+00_rb/)
      kao(:, 2, 6,11) = (/ &
     &2.4797e-02_rb,4.2571e-01_rb,8.5136e-01_rb,1.2771e+00_rb,1.7031e+00_rb,2.1298e+00_rb, &
     &2.5578e+00_rb,2.9908e+00_rb,3.4060e+00_rb/)
      kao(:, 3, 6,11) = (/ &
     &2.5418e-02_rb,4.6778e-01_rb,9.3545e-01_rb,1.4036e+00_rb,1.8720e+00_rb,2.3409e+00_rb, &
     &2.8115e+00_rb,3.2888e+00_rb,3.7438e+00_rb/)
      kao(:, 4, 6,11) = (/ &
     &2.5988e-02_rb,5.1023e-01_rb,1.0205e+00_rb,1.5308e+00_rb,2.0415e+00_rb,2.5527e+00_rb, &
     &3.0658e+00_rb,3.5842e+00_rb,4.0828e+00_rb/)
      kao(:, 5, 6,11) = (/ &
     &2.6439e-02_rb,5.5329e-01_rb,1.1066e+00_rb,1.6600e+00_rb,2.2137e+00_rb,2.7680e+00_rb, &
     &3.3240e+00_rb,3.8852e+00_rb,4.4272e+00_rb/)
      kao(:, 1, 7,11) = (/ &
     &4.4902e-02_rb,3.4110e-01_rb,6.7879e-01_rb,1.0183e+00_rb,1.3577e+00_rb,1.6974e+00_rb, &
     &2.0380e+00_rb,2.3811e+00_rb,2.7150e+00_rb/)
      kao(:, 2, 7,11) = (/ &
     &4.6257e-02_rb,3.7900e-01_rb,7.5783e-01_rb,1.1366e+00_rb,1.5157e+00_rb,1.8951e+00_rb, &
     &2.2756e+00_rb,2.6593e+00_rb,3.0311e+00_rb/)
      kao(:, 3, 7,11) = (/ &
     &4.7417e-02_rb,4.1838e-01_rb,8.3644e-01_rb,1.2549e+00_rb,1.6732e+00_rb,2.0920e+00_rb, &
     &2.5119e+00_rb,2.9363e+00_rb,3.3461e+00_rb/)
      kao(:, 4, 7,11) = (/ &
     &4.8439e-02_rb,4.5979e-01_rb,9.1922e-01_rb,1.3788e+00_rb,1.8387e+00_rb,2.2989e+00_rb, &
     &2.7602e+00_rb,3.2261e+00_rb,3.6771e+00_rb/)
      kao(:, 5, 7,11) = (/ &
     &4.9115e-02_rb,5.0257e-01_rb,1.0048e+00_rb,1.5073e+00_rb,2.0099e+00_rb,2.5129e+00_rb, &
     &3.0174e+00_rb,3.5268e+00_rb,4.0195e+00_rb/)
      kao(:, 1, 8,11) = (/ &
     &9.9346e-02_rb,4.0741e-01_rb,6.1474e-01_rb,8.8954e-01_rb,1.1859e+00_rb,1.4823e+00_rb, &
     &1.7793e+00_rb,2.0775e+00_rb,2.3709e+00_rb/)
      kao(:, 2, 8,11) = (/ &
     &1.0236e-01_rb,4.4079e-01_rb,6.7253e-01_rb,9.9766e-01_rb,1.3301e+00_rb,1.6626e+00_rb, &
     &1.9955e+00_rb,2.3301e+00_rb,2.6593e+00_rb/)
      kao(:, 3, 8,11) = (/ &
     &1.0516e-01_rb,4.6951e-01_rb,7.3794e-01_rb,1.1060e+00_rb,1.4745e+00_rb,1.8432e+00_rb, &
     &2.2125e+00_rb,2.5835e+00_rb,2.9480e+00_rb/)
      kao(:, 4, 8,11) = (/ &
     &1.0772e-01_rb,4.9500e-01_rb,8.1532e-01_rb,1.2227e+00_rb,1.6302e+00_rb,2.0375e+00_rb, &
     &2.4456e+00_rb,2.8562e+00_rb,3.2595e+00_rb/)
      kao(:, 5, 8,11) = (/ &
     &1.0947e-01_rb,5.2130e-01_rb,8.9659e-01_rb,1.3445e+00_rb,1.7925e+00_rb,2.2406e+00_rb, &
     &2.6897e+00_rb,3.1404e+00_rb,3.5840e+00_rb/)
      kao(:, 1, 9,11) = (/ &
     &4.1823e-01_rb,7.9567e-01_rb,1.0382e+00_rb,1.1972e+00_rb,1.2989e+00_rb,1.3526e+00_rb, &
     &1.5261e+00_rb,1.7800e+00_rb,2.0331e+00_rb/)
      kao(:, 2, 9,11) = (/ &
     &4.3170e-01_rb,8.4637e-01_rb,1.1195e+00_rb,1.3007e+00_rb,1.4025e+00_rb,1.4821e+00_rb, &
     &1.7319e+00_rb,2.0202e+00_rb,2.3074e+00_rb/)
      kao(:, 3, 9,11) = (/ &
     &4.4387e-01_rb,8.9320e-01_rb,1.1954e+00_rb,1.3918e+00_rb,1.4965e+00_rb,1.6263e+00_rb, &
     &1.9385e+00_rb,2.2615e+00_rb,2.5830e+00_rb/)
      kao(:, 4, 9,11) = (/ &
     &4.5501e-01_rb,9.4246e-01_rb,1.2607e+00_rb,1.4727e+00_rb,1.5878e+00_rb,1.7989e+00_rb, &
     &2.1575e+00_rb,2.5169e+00_rb,2.8754e+00_rb/)
      kao(:, 5, 9,11) = (/ &
     &4.6515e-01_rb,9.9000e-01_rb,1.3196e+00_rb,1.5528e+00_rb,1.6906e+00_rb,1.9810e+00_rb, &
     &2.3767e+00_rb,2.7728e+00_rb,3.1676e+00_rb/)
      kao(:, 1,10,11) = (/ &
     &1.9805e+00_rb,2.2049e+00_rb,2.3763e+00_rb,2.4479e+00_rb,2.4732e+00_rb,2.4264e+00_rb, &
     &2.2604e+00_rb,1.8802e+00_rb,1.7493e+00_rb/)
      kao(:, 2,10,11) = (/ &
     &2.0472e+00_rb,2.3153e+00_rb,2.5048e+00_rb,2.6141e+00_rb,2.6517e+00_rb,2.6235e+00_rb, &
     &2.4547e+00_rb,2.0470e+00_rb,2.0043e+00_rb/)
      kao(:, 3,10,11) = (/ &
     &2.1094e+00_rb,2.4143e+00_rb,2.6357e+00_rb,2.7700e+00_rb,2.8309e+00_rb,2.7965e+00_rb, &
     &2.6317e+00_rb,2.2035e+00_rb,2.2660e+00_rb/)
      kao(:, 4,10,11) = (/ &
     &2.1647e+00_rb,2.5051e+00_rb,2.7487e+00_rb,2.9276e+00_rb,2.9982e+00_rb,2.9552e+00_rb, &
     &2.7953e+00_rb,2.3731e+00_rb,2.5329e+00_rb/)
      kao(:, 5,10,11) = (/ &
     &2.2155e+00_rb,2.5942e+00_rb,2.8618e+00_rb,3.0775e+00_rb,3.1500e+00_rb,3.1109e+00_rb, &
     &2.9560e+00_rb,2.5540e+00_rb,2.7905e+00_rb/)
      kao(:, 1,11,11) = (/ &
     &3.3272e+00_rb,3.3391e+00_rb,3.4265e+00_rb,3.3299e+00_rb,3.1864e+00_rb,3.0000e+00_rb, &
     &2.7382e+00_rb,2.2548e+00_rb,1.5937e+00_rb/)
      kao(:, 2,11,11) = (/ &
     &3.4245e+00_rb,3.4923e+00_rb,3.5888e+00_rb,3.5207e+00_rb,3.3952e+00_rb,3.2270e+00_rb, &
     &2.9458e+00_rb,2.4552e+00_rb,1.8248e+00_rb/)
      kao(:, 3,11,11) = (/ &
     &3.5147e+00_rb,3.6350e+00_rb,3.7493e+00_rb,3.6979e+00_rb,3.6003e+00_rb,3.4366e+00_rb, &
     &3.1408e+00_rb,2.6446e+00_rb,2.0543e+00_rb/)
      kao(:, 4,11,11) = (/ &
     &3.5973e+00_rb,3.7611e+00_rb,3.8955e+00_rb,3.8697e+00_rb,3.7999e+00_rb,3.6320e+00_rb, &
     &3.3291e+00_rb,2.8219e+00_rb,2.2769e+00_rb/)
      kao(:, 5,11,11) = (/ &
     &3.6658e+00_rb,3.8812e+00_rb,4.0226e+00_rb,4.0479e+00_rb,3.9815e+00_rb,3.8145e+00_rb, &
     &3.5185e+00_rb,2.9782e+00_rb,2.4861e+00_rb/)
      kao(:, 1,12,11) = (/ &
     &4.1187e+00_rb,4.0374e+00_rb,4.0352e+00_rb,3.8356e+00_rb,3.5772e+00_rb,3.2668e+00_rb, &
     &2.8837e+00_rb,2.3539e+00_rb,1.4349e+00_rb/)
      kao(:, 2,12,11) = (/ &
     &4.2212e+00_rb,4.1986e+00_rb,4.2178e+00_rb,4.0426e+00_rb,3.8006e+00_rb,3.4932e+00_rb, &
     &3.0942e+00_rb,2.5554e+00_rb,1.6369e+00_rb/)
      kao(:, 3,12,11) = (/ &
     &4.3153e+00_rb,4.3421e+00_rb,4.3917e+00_rb,4.2429e+00_rb,4.0161e+00_rb,3.6975e+00_rb, &
     &3.3039e+00_rb,2.7454e+00_rb,1.8396e+00_rb/)
      kao(:, 4,12,11) = (/ &
     &4.3996e+00_rb,4.4753e+00_rb,4.5481e+00_rb,4.4345e+00_rb,4.2153e+00_rb,3.8944e+00_rb, &
     &3.5079e+00_rb,2.9264e+00_rb,2.0403e+00_rb/)
      kao(:, 5,12,11) = (/ &
     &4.4754e+00_rb,4.5978e+00_rb,4.6899e+00_rb,4.6220e+00_rb,4.4006e+00_rb,4.0838e+00_rb, &
     &3.7036e+00_rb,3.0926e+00_rb,2.2349e+00_rb/)
      kao(:, 1,13,11) = (/ &
     &4.0592e+00_rb,4.0332e+00_rb,3.9681e+00_rb,3.7590e+00_rb,3.4963e+00_rb,3.1626e+00_rb, &
     &2.7397e+00_rb,2.2238e+00_rb,1.2732e+00_rb/)
      kao(:, 2,13,11) = (/ &
     &4.1472e+00_rb,4.1742e+00_rb,4.1368e+00_rb,3.9606e+00_rb,3.7120e+00_rb,3.3731e+00_rb, &
     &2.9427e+00_rb,2.4115e+00_rb,1.4537e+00_rb/)
      kao(:, 3,13,11) = (/ &
     &4.2292e+00_rb,4.3011e+00_rb,4.2979e+00_rb,4.1507e+00_rb,3.9119e+00_rb,3.5700e+00_rb, &
     &3.1423e+00_rb,2.5914e+00_rb,1.6282e+00_rb/)
      kao(:, 4,13,11) = (/ &
     &4.3001e+00_rb,4.4170e+00_rb,4.4457e+00_rb,4.3341e+00_rb,4.0986e+00_rb,3.7637e+00_rb, &
     &3.3341e+00_rb,2.7592e+00_rb,1.8128e+00_rb/)
      kao(:, 5,13,11) = (/ &
     &4.3627e+00_rb,4.5278e+00_rb,4.5856e+00_rb,4.5011e+00_rb,4.2740e+00_rb,3.9472e+00_rb, &
     &3.5112e+00_rb,2.9185e+00_rb,1.9901e+00_rb/)
      kao(:, 1, 1,12) = (/ &
     &1.2832e-03_rb,7.6911e-01_rb,1.5308e+00_rb,2.2815e+00_rb,3.0131e+00_rb,3.7102e+00_rb, &
     &4.3339e+00_rb,4.8156e+00_rb,6.0262e+00_rb/)
      kao(:, 2, 1,12) = (/ &
     &1.3165e-03_rb,8.3137e-01_rb,1.6545e+00_rb,2.4655e+00_rb,3.2561e+00_rb,4.0092e+00_rb, &
     &4.6800e+00_rb,5.1382e+00_rb,6.5123e+00_rb/)
      kao(:, 3, 1,12) = (/ &
     &1.3465e-03_rb,8.9467e-01_rb,1.7805e+00_rb,2.6534e+00_rb,3.5040e+00_rb,4.3130e+00_rb, &
     &5.0270e+00_rb,5.5081e+00_rb,7.0080e+00_rb/)
      kao(:, 4, 1,12) = (/ &
     &1.3733e-03_rb,9.5420e-01_rb,1.9011e+00_rb,2.8343e+00_rb,3.7449e+00_rb,4.6125e+00_rb, &
     &5.3804e+00_rb,5.8655e+00_rb,7.4899e+00_rb/)
      kao(:, 5, 1,12) = (/ &
     &1.3973e-03_rb,1.0065e+00_rb,2.0056e+00_rb,2.9931e+00_rb,3.9616e+00_rb,4.8908e+00_rb, &
     &5.7173e+00_rb,6.2206e+00_rb,7.9232e+00_rb/)
      kao(:, 1, 2,12) = (/ &
     &2.1567e-03_rb,7.9503e-01_rb,1.5851e+00_rb,2.3675e+00_rb,3.1367e+00_rb,3.8805e+00_rb, &
     &4.5651e+00_rb,5.0566e+00_rb,6.2733e+00_rb/)
      kao(:, 2, 2,12) = (/ &
     &2.2096e-03_rb,8.5765e-01_rb,1.7102e+00_rb,2.5549e+00_rb,3.3862e+00_rb,4.1922e+00_rb, &
     &4.9361e+00_rb,5.4818e+00_rb,6.7723e+00_rb/)
      kao(:, 3, 2,12) = (/ &
     &2.2593e-03_rb,9.2132e-01_rb,1.8371e+00_rb,2.7441e+00_rb,3.6360e+00_rb,4.5001e+00_rb, &
     &5.3023e+00_rb,5.9092e+00_rb,7.2719e+00_rb/)
      kao(:, 4, 2,12) = (/ &
     &2.3029e-03_rb,9.7961e-01_rb,1.9547e+00_rb,2.9227e+00_rb,3.8771e+00_rb,4.8047e+00_rb, &
     &5.6684e+00_rb,6.3211e+00_rb,7.7542e+00_rb/)
      kao(:, 5, 2,12) = (/ &
     &2.3432e-03_rb,1.0294e+00_rb,2.0551e+00_rb,3.0745e+00_rb,4.0831e+00_rb,5.0720e+00_rb, &
     &6.0082e+00_rb,6.7230e+00_rb,8.1662e+00_rb/)
      kao(:, 1, 3,12) = (/ &
     &4.8280e-03_rb,7.9900e-01_rb,1.5962e+00_rb,2.3906e+00_rb,3.1803e+00_rb,3.9604e+00_rb, &
     &4.7157e+00_rb,5.3732e+00_rb,6.3605e+00_rb/)
      kao(:, 2, 3,12) = (/ &
     &4.9522e-03_rb,8.6159e-01_rb,1.7217e+00_rb,2.5790e+00_rb,3.4316e+00_rb,4.2766e+00_rb, &
     &5.0979e+00_rb,5.8264e+00_rb,6.8633e+00_rb/)
      kao(:, 3, 3,12) = (/ &
     &5.0521e-03_rb,9.2450e-01_rb,1.8472e+00_rb,2.7674e+00_rb,3.6829e+00_rb,4.5888e+00_rb, &
     &5.4724e+00_rb,6.2581e+00_rb,7.3659e+00_rb/)
      kao(:, 4, 3,12) = (/ &
     &5.1417e-03_rb,9.8492e-01_rb,1.9684e+00_rb,2.9494e+00_rb,3.9267e+00_rb,4.8950e+00_rb, &
     &5.8398e+00_rb,6.6855e+00_rb,7.8533e+00_rb/)
      kao(:, 5, 3,12) = (/ &
     &5.2128e-03_rb,1.0351e+00_rb,2.0685e+00_rb,3.1013e+00_rb,4.1304e+00_rb,5.1536e+00_rb, &
     &6.1563e+00_rb,7.0696e+00_rb,8.2608e+00_rb/)
      kao(:, 1, 4,12) = (/ &
     &1.0891e-02_rb,7.7727e-01_rb,1.5543e+00_rb,2.3309e+00_rb,3.1067e+00_rb,3.8812e+00_rb, &
     &4.6512e+00_rb,5.4004e+00_rb,6.2133e+00_rb/)
      kao(:, 2, 4,12) = (/ &
     &1.1146e-02_rb,8.4013e-01_rb,1.6801e+00_rb,2.5195e+00_rb,3.3588e+00_rb,4.1965e+00_rb, &
     &5.0315e+00_rb,5.8522e+00_rb,6.7175e+00_rb/)
      kao(:, 3, 4,12) = (/ &
     &1.1369e-02_rb,9.0347e-01_rb,1.8066e+00_rb,2.7094e+00_rb,3.6116e+00_rb,4.5128e+00_rb, &
     &5.4097e+00_rb,6.2868e+00_rb,7.2231e+00_rb/)
      kao(:, 4, 4,12) = (/ &
     &1.1543e-02_rb,9.6345e-01_rb,1.9267e+00_rb,2.8898e+00_rb,3.8526e+00_rb,4.8146e+00_rb, &
     &5.7741e+00_rb,6.7146e+00_rb,7.7051e+00_rb/)
      kao(:, 5, 4,12) = (/ &
     &1.1696e-02_rb,1.0158e+00_rb,2.0314e+00_rb,3.0468e+00_rb,4.0618e+00_rb,5.0772e+00_rb, &
     &6.0909e+00_rb,7.0982e+00_rb,8.1235e+00_rb/)
      kao(:, 1, 5,12) = (/ &
     &2.2302e-02_rb,7.3490e-01_rb,1.4696e+00_rb,2.2046e+00_rb,2.9394e+00_rb,3.6745e+00_rb, &
     &4.4097e+00_rb,5.1450e+00_rb,5.8787e+00_rb/)
      kao(:, 2, 5,12) = (/ &
     &2.2823e-02_rb,7.9768e-01_rb,1.5952e+00_rb,2.3932e+00_rb,3.1913e+00_rb,3.9900e+00_rb, &
     &4.7899e+00_rb,5.5934e+00_rb,6.3823e+00_rb/)
      kao(:, 3, 5,12) = (/ &
     &2.3280e-02_rb,8.5775e-01_rb,1.7153e+00_rb,2.5734e+00_rb,3.4313e+00_rb,4.2902e+00_rb, &
     &5.1502e+00_rb,6.0153e+00_rb,6.8623e+00_rb/)
      kao(:, 4, 5,12) = (/ &
     &2.3683e-02_rb,9.1895e-01_rb,1.8383e+00_rb,2.7574e+00_rb,3.6772e+00_rb,4.5980e+00_rb, &
     &5.5202e+00_rb,6.4486e+00_rb,7.3542e+00_rb/)
      kao(:, 5, 5,12) = (/ &
     &2.3999e-02_rb,9.7537e-01_rb,1.9505e+00_rb,2.9260e+00_rb,3.9018e+00_rb,4.8784e+00_rb, &
     &5.8576e+00_rb,6.8447e+00_rb,7.8034e+00_rb/)
      kao(:, 1, 6,12) = (/ &
     &4.2265e-02_rb,6.7923e-01_rb,1.3583e+00_rb,2.0372e+00_rb,2.7164e+00_rb,3.3965e+00_rb, &
     &4.0771e+00_rb,4.7611e+00_rb,5.4325e+00_rb/)
      kao(:, 2, 6,12) = (/ &
     &4.3312e-02_rb,7.4027e-01_rb,1.4802e+00_rb,2.2203e+00_rb,2.9606e+00_rb,3.7016e+00_rb, &
     &4.4440e+00_rb,5.1934e+00_rb,5.9209e+00_rb/)
      kao(:, 3, 6,12) = (/ &
     &4.4154e-02_rb,8.0193e-01_rb,1.6036e+00_rb,2.4053e+00_rb,3.2071e+00_rb,4.0091e+00_rb, &
     &4.8130e+00_rb,5.6200e+00_rb,6.4137e+00_rb/)
      kao(:, 4, 6,12) = (/ &
     &4.4818e-02_rb,8.6447e-01_rb,1.7287e+00_rb,2.5931e+00_rb,3.4580e+00_rb,4.3234e+00_rb, &
     &5.1892e+00_rb,6.0606e+00_rb,6.9157e+00_rb/)
      kao(:, 5, 6,12) = (/ &
     &4.5427e-02_rb,9.2353e-01_rb,1.8468e+00_rb,2.7701e+00_rb,3.6940e+00_rb,4.6182e+00_rb, &
     &5.5436e+00_rb,6.4744e+00_rb,7.3877e+00_rb/)
      kao(:, 1, 7,12) = (/ &
     &8.2851e-02_rb,6.1813e-01_rb,1.2353e+00_rb,1.8525e+00_rb,2.4701e+00_rb,3.0880e+00_rb, &
     &3.7064e+00_rb,4.3285e+00_rb,4.9392e+00_rb/)
      kao(:, 2, 7,12) = (/ &
     &8.4778e-02_rb,6.8174e-01_rb,1.3624e+00_rb,2.0434e+00_rb,2.7245e+00_rb,3.4060e+00_rb, &
     &4.0883e+00_rb,4.7735e+00_rb,5.4481e+00_rb/)
      kao(:, 3, 7,12) = (/ &
     &8.6620e-02_rb,7.4569e-01_rb,1.4908e+00_rb,2.2359e+00_rb,2.9814e+00_rb,3.7272e+00_rb, &
     &4.4733e+00_rb,5.2220e+00_rb,5.9623e+00_rb/)
      kao(:, 4, 7,12) = (/ &
     &8.8221e-02_rb,8.0972e-01_rb,1.6191e+00_rb,2.4284e+00_rb,3.2380e+00_rb,4.0476e+00_rb, &
     &4.8581e+00_rb,5.6715e+00_rb,6.4756e+00_rb/)
      kao(:, 5, 7,12) = (/ &
     &8.9388e-02_rb,8.6929e-01_rb,1.7383e+00_rb,2.6072e+00_rb,3.4769e+00_rb,4.3464e+00_rb, &
     &5.2172e+00_rb,6.0912e+00_rb,6.9533e+00_rb/)
      kao(:, 1, 8,12) = (/ &
     &1.9209e-01_rb,6.2527e-01_rb,1.1133e+00_rb,1.6689e+00_rb,2.2244e+00_rb,2.7803e+00_rb, &
     &3.3362e+00_rb,3.8938e+00_rb,4.4464e+00_rb/)
      kao(:, 2, 8,12) = (/ &
     &1.9727e-01_rb,6.6108e-01_rb,1.2430e+00_rb,1.8635e+00_rb,2.4844e+00_rb,3.1054e+00_rb, &
     &3.7267e+00_rb,4.3504e+00_rb,4.9671e+00_rb/)
      kao(:, 3, 8,12) = (/ &
     &2.0144e-01_rb,7.0485e-01_rb,1.3718e+00_rb,2.0572e+00_rb,2.7423e+00_rb,3.4280e+00_rb, &
     &4.1141e+00_rb,4.8012e+00_rb,5.4833e+00_rb/)
      kao(:, 4, 8,12) = (/ &
     &2.0484e-01_rb,7.5576e-01_rb,1.4986e+00_rb,2.2473e+00_rb,2.9964e+00_rb,3.7450e+00_rb, &
     &4.4951e+00_rb,5.2469e+00_rb,5.9915e+00_rb/)
      kao(:, 5, 8,12) = (/ &
     &2.0777e-01_rb,8.0793e-01_rb,1.6132e+00_rb,2.4193e+00_rb,3.2252e+00_rb,4.0323e+00_rb, &
     &4.8391e+00_rb,5.6481e+00_rb,6.4494e+00_rb/)
      kao(:, 1, 9,12) = (/ &
     &8.2620e-01_rb,1.3515e+00_rb,1.6396e+00_rb,1.8633e+00_rb,2.0617e+00_rb,2.4827e+00_rb, &
     &2.9771e+00_rb,3.4729e+00_rb,3.9666e+00_rb/)
      kao(:, 2, 9,12) = (/ &
     &8.4850e-01_rb,1.4155e+00_rb,1.7443e+00_rb,1.9664e+00_rb,2.2558e+00_rb,2.7862e+00_rb, &
     &3.3424e+00_rb,3.8990e+00_rb,4.4541e+00_rb/)
      kao(:, 3, 9,12) = (/ &
     &8.6743e-01_rb,1.4840e+00_rb,1.8355e+00_rb,2.0729e+00_rb,2.4805e+00_rb,3.0958e+00_rb, &
     &3.7138e+00_rb,4.3325e+00_rb,4.9497e+00_rb/)
      kao(:, 4, 9,12) = (/ &
     &8.8373e-01_rb,1.5402e+00_rb,1.9288e+00_rb,2.1892e+00_rb,2.7106e+00_rb,3.3871e+00_rb, &
     &4.0635e+00_rb,4.7407e+00_rb,5.4157e+00_rb/)
      kao(:, 5, 9,12) = (/ &
     &8.9622e-01_rb,1.5897e+00_rb,2.0124e+00_rb,2.3107e+00_rb,2.9348e+00_rb,3.6673e+00_rb, &
     &4.4005e+00_rb,5.1335e+00_rb,5.8643e+00_rb/)
      kao(:, 1,10,12) = (/ &
     &3.9173e+00_rb,3.9928e+00_rb,4.3366e+00_rb,4.4280e+00_rb,4.2663e+00_rb,3.9998e+00_rb, &
     &3.6166e+00_rb,3.1308e+00_rb,3.5126e+00_rb/)
      kao(:, 2,10,12) = (/ &
     &4.0212e+00_rb,4.1687e+00_rb,4.5593e+00_rb,4.6368e+00_rb,4.5220e+00_rb,4.2834e+00_rb, &
     &3.8634e+00_rb,3.4894e+00_rb,3.9713e+00_rb/)
      kao(:, 3,10,12) = (/ &
     &4.1147e+00_rb,4.3230e+00_rb,4.7454e+00_rb,4.8522e+00_rb,4.7749e+00_rb,4.5433e+00_rb, &
     &4.0957e+00_rb,3.8708e+00_rb,4.4196e+00_rb/)
      kao(:, 4,10,12) = (/ &
     &4.1890e+00_rb,4.4586e+00_rb,4.9236e+00_rb,5.0494e+00_rb,4.9988e+00_rb,4.8016e+00_rb, &
     &4.3028e+00_rb,4.2486e+00_rb,4.8508e+00_rb/)
      kao(:, 5,10,12) = (/ &
     &4.2486e+00_rb,4.5669e+00_rb,5.0902e+00_rb,5.2222e+00_rb,5.2265e+00_rb,5.0216e+00_rb, &
     &4.5057e+00_rb,4.6204e+00_rb,5.2753e+00_rb/)
      kao(:, 1,11,12) = (/ &
     &6.5008e+00_rb,6.1849e+00_rb,6.2789e+00_rb,6.2685e+00_rb,5.9393e+00_rb,5.4109e+00_rb, &
     &4.7099e+00_rb,3.7046e+00_rb,3.2687e+00_rb/)
      kao(:, 2,11,12) = (/ &
     &6.6711e+00_rb,6.4071e+00_rb,6.5745e+00_rb,6.5690e+00_rb,6.2447e+00_rb,5.7486e+00_rb, &
     &5.0466e+00_rb,3.9652e+00_rb,3.6849e+00_rb/)
      kao(:, 3,11,12) = (/ &
     &6.8141e+00_rb,6.5961e+00_rb,6.8266e+00_rb,6.8425e+00_rb,6.5425e+00_rb,6.0645e+00_rb, &
     &5.3679e+00_rb,4.1951e+00_rb,4.0848e+00_rb/)
      kao(:, 4,11,12) = (/ &
     &6.9309e+00_rb,6.7641e+00_rb,7.0497e+00_rb,7.0951e+00_rb,6.8173e+00_rb,6.3688e+00_rb, &
     &5.6628e+00_rb,4.4427e+00_rb,4.4838e+00_rb/)
      kao(:, 5,11,12) = (/ &
     &7.0334e+00_rb,6.9031e+00_rb,7.2676e+00_rb,7.3125e+00_rb,7.0679e+00_rb,6.6762e+00_rb, &
     &5.9144e+00_rb,4.7231e+00_rb,4.9084e+00_rb/)
      kao(:, 1,12,12) = (/ &
     &8.0356e+00_rb,7.5209e+00_rb,7.4229e+00_rb,7.2434e+00_rb,6.7979e+00_rb,6.1202e+00_rb, &
     &5.2895e+00_rb,4.1070e+00_rb,2.9936e+00_rb/)
      kao(:, 2,12,12) = (/ &
     &8.2347e+00_rb,7.7748e+00_rb,7.7310e+00_rb,7.5713e+00_rb,7.1351e+00_rb,6.4951e+00_rb, &
     &5.6590e+00_rb,4.4043e+00_rb,3.3743e+00_rb/)
      kao(:, 3,12,12) = (/ &
     &8.4084e+00_rb,7.9908e+00_rb,8.0012e+00_rb,7.8714e+00_rb,7.4600e+00_rb,6.8499e+00_rb, &
     &6.0055e+00_rb,4.6706e+00_rb,3.7574e+00_rb/)
      kao(:, 4,12,12) = (/ &
     &8.5546e+00_rb,8.1702e+00_rb,8.2474e+00_rb,8.1496e+00_rb,7.7631e+00_rb,7.1869e+00_rb, &
     &6.3250e+00_rb,4.9362e+00_rb,4.1426e+00_rb/)
      kao(:, 5,12,12) = (/ &
     &8.6676e+00_rb,8.3299e+00_rb,8.4723e+00_rb,8.3950e+00_rb,8.0405e+00_rb,7.5149e+00_rb, &
     &6.6159e+00_rb,5.2129e+00_rb,4.5254e+00_rb/)
      kao(:, 1,13,12) = (/ &
     &7.9759e+00_rb,7.5343e+00_rb,7.4503e+00_rb,7.1563e+00_rb,6.6524e+00_rb,6.0033e+00_rb, &
     &5.2080e+00_rb,4.0643e+00_rb,2.7278e+00_rb/)
      kao(:, 2,13,12) = (/ &
     &8.1794e+00_rb,7.7874e+00_rb,7.7473e+00_rb,7.4642e+00_rb,6.9912e+00_rb,6.3834e+00_rb, &
     &5.5711e+00_rb,4.3572e+00_rb,3.0821e+00_rb/)
      kao(:, 3,13,12) = (/ &
     &8.3457e+00_rb,8.0091e+00_rb,8.0057e+00_rb,7.7578e+00_rb,7.3258e+00_rb,6.7364e+00_rb, &
     &5.9096e+00_rb,4.6439e+00_rb,3.4477e+00_rb/)
      kao(:, 4,13,12) = (/ &
     &8.4812e+00_rb,8.2074e+00_rb,8.2395e+00_rb,8.0326e+00_rb,7.6295e+00_rb,7.0667e+00_rb, &
     &6.2213e+00_rb,4.9334e+00_rb,3.7948e+00_rb/)
      kao(:, 5,13,12) = (/ &
     &8.5914e+00_rb,8.3651e+00_rb,8.4655e+00_rb,8.2768e+00_rb,7.9296e+00_rb,7.3785e+00_rb, &
     &6.5133e+00_rb,5.2165e+00_rb,4.1485e+00_rb/)
      kao(:, 1, 1,13) = (/ &
     &1.8376e-03_rb,1.2493e+00_rb,2.4869e+00_rb,3.7066e+00_rb,4.8962e+00_rb,6.0283e+00_rb, &
     &7.0325e+00_rb,7.5732e+00_rb,9.7924e+00_rb/)
      kao(:, 2, 1,13) = (/ &
     &1.8435e-03_rb,1.3137e+00_rb,2.6151e+00_rb,3.8978e+00_rb,5.1474e+00_rb,6.3345e+00_rb, &
     &7.3693e+00_rb,7.9436e+00_rb,1.0295e+01_rb/)
      kao(:, 3, 1,13) = (/ &
     &1.8457e-03_rb,1.3823e+00_rb,2.7512e+00_rb,4.0980e+00_rb,5.4060e+00_rb,6.6439e+00_rb, &
     &7.7282e+00_rb,8.2817e+00_rb,1.0812e+01_rb/)
      kao(:, 4, 1,13) = (/ &
     &1.8458e-03_rb,1.4480e+00_rb,2.8814e+00_rb,4.2920e+00_rb,5.6663e+00_rb,6.9707e+00_rb, &
     &8.1084e+00_rb,8.6593e+00_rb,1.1333e+01_rb/)
      kao(:, 5, 1,13) = (/ &
     &1.8581e-03_rb,1.5191e+00_rb,3.0214e+00_rb,4.4961e+00_rb,5.9276e+00_rb,7.2804e+00_rb, &
     &8.4721e+00_rb,9.0750e+00_rb,1.1855e+01_rb/)
      kao(:, 1, 2,13) = (/ &
     &3.0081e-03_rb,1.2887e+00_rb,2.5721e+00_rb,3.8463e+00_rb,5.1064e+00_rb,6.3378e+00_rb, &
     &7.4987e+00_rb,8.3933e+00_rb,1.0213e+01_rb/)
      kao(:, 2, 2,13) = (/ &
     &3.0389e-03_rb,1.3599e+00_rb,2.7131e+00_rb,4.0552e+00_rb,5.3783e+00_rb,6.6678e+00_rb, &
     &7.8772e+00_rb,8.7981e+00_rb,1.0756e+01_rb/)
      kao(:, 3, 2,13) = (/ &
     &3.0623e-03_rb,1.4308e+00_rb,2.8555e+00_rb,4.2711e+00_rb,5.6674e+00_rb,7.0255e+00_rb, &
     &8.2810e+00_rb,9.2099e+00_rb,1.1335e+01_rb/)
      kao(:, 4, 2,13) = (/ &
     &3.1073e-03_rb,1.4998e+00_rb,2.9927e+00_rb,4.4738e+00_rb,5.9361e+00_rb,7.3612e+00_rb, &
     &8.6975e+00_rb,9.6506e+00_rb,1.1872e+01_rb/)
      kao(:, 5, 2,13) = (/ &
     &3.1371e-03_rb,1.5763e+00_rb,3.1432e+00_rb,4.6958e+00_rb,6.2242e+00_rb,7.7091e+00_rb, &
     &9.0917e+00_rb,1.0108e+01_rb,1.2448e+01_rb/)
      kao(:, 1, 3,13) = (/ &
     &6.7566e-03_rb,1.3010e+00_rb,2.6001e+00_rb,3.8967e+00_rb,5.1878e+00_rb,6.4676e+00_rb, &
     &7.7183e+00_rb,8.8575e+00_rb,1.0376e+01_rb/)
      kao(:, 2, 3,13) = (/ &
     &6.8655e-03_rb,1.3725e+00_rb,2.7424e+00_rb,4.1083e+00_rb,5.4679e+00_rb,6.8133e+00_rb, &
     &8.1286e+00_rb,9.3210e+00_rb,1.0936e+01_rb/)
      kao(:, 3, 3,13) = (/ &
     &7.0162e-03_rb,1.4463e+00_rb,2.8900e+00_rb,4.3294e+00_rb,5.7613e+00_rb,7.1793e+00_rb, &
     &8.5582e+00_rb,9.7979e+00_rb,1.1523e+01_rb/)
      kao(:, 4, 3,13) = (/ &
     &7.1355e-03_rb,1.5186e+00_rb,3.0341e+00_rb,4.5447e+00_rb,6.0464e+00_rb,7.5309e+00_rb, &
     &8.9783e+00_rb,1.0290e+01_rb,1.2093e+01_rb/)
      kao(:, 5, 3,13) = (/ &
     &7.2326e-03_rb,1.6009e+00_rb,3.1979e+00_rb,4.7883e+00_rb,6.3681e+00_rb,7.9274e+00_rb, &
     &9.4396e+00_rb,1.0789e+01_rb,1.2736e+01_rb/)
      kao(:, 1, 4,13) = (/ &
     &1.5770e-02_rb,1.2974e+00_rb,2.5940e+00_rb,3.8891e+00_rb,5.1823e+00_rb,6.4711e+00_rb, &
     &7.7497e+00_rb,8.9872e+00_rb,1.0364e+01_rb/)
      kao(:, 2, 4,13) = (/ &
     &1.6102e-02_rb,1.3725e+00_rb,2.7438e+00_rb,4.1133e+00_rb,5.4792e+00_rb,6.8384e+00_rb, &
     &8.1824e+00_rb,9.4681e+00_rb,1.0958e+01_rb/)
      kao(:, 3, 4,13) = (/ &
     &1.6461e-02_rb,1.4529e+00_rb,2.9044e+00_rb,4.3540e+00_rb,5.7994e+00_rb,7.2375e+00_rb, &
     &8.6575e+00_rb,1.0008e+01_rb,1.1599e+01_rb/)
      kao(:, 4, 4,13) = (/ &
     &1.6764e-02_rb,1.5336e+00_rb,3.0659e+00_rb,4.5952e+00_rb,6.1201e+00_rb,7.6366e+00_rb, &
     &9.1307e+00_rb,1.0552e+01_rb,1.2240e+01_rb/)
      kao(:, 5, 4,13) = (/ &
     &1.6983e-02_rb,1.6203e+00_rb,3.2389e+00_rb,4.8548e+00_rb,6.4661e+00_rb,8.0668e+00_rb, &
     &9.6455e+00_rb,1.1132e+01_rb,1.2932e+01_rb/)
      kao(:, 1, 5,13) = (/ &
     &3.3958e-02_rb,1.2926e+00_rb,2.5850e+00_rb,3.8765e+00_rb,5.1671e+00_rb,6.4548e+00_rb, &
     &7.7382e+00_rb,9.0001e+00_rb,1.0334e+01_rb/)
      kao(:, 2, 5,13) = (/ &
     &3.4722e-02_rb,1.3756e+00_rb,2.7505e+00_rb,4.1243e+00_rb,5.4969e+00_rb,6.8654e+00_rb, &
     &8.2252e+00_rb,9.5545e+00_rb,1.0994e+01_rb/)
      kao(:, 3, 5,13) = (/ &
     &3.5363e-02_rb,1.4648e+00_rb,2.9290e+00_rb,4.3919e+00_rb,5.8529e+00_rb,7.3099e+00_rb, &
     &8.7568e+00_rb,1.0164e+01_rb,1.1706e+01_rb/)
      kao(:, 4, 5,13) = (/ &
     &3.6012e-02_rb,1.5505e+00_rb,3.1003e+00_rb,4.6491e+00_rb,6.1954e+00_rb,7.7382e+00_rb, &
     &9.2690e+00_rb,1.0761e+01_rb,1.2391e+01_rb/)
      kao(:, 5, 5,13) = (/ &
     &3.6453e-02_rb,1.6361e+00_rb,3.2714e+00_rb,4.9053e+00_rb,6.5374e+00_rb,8.1645e+00_rb, &
     &9.7803e+00_rb,1.1352e+01_rb,1.3075e+01_rb/)
      kao(:, 1, 6,13) = (/ &
     &6.7837e-02_rb,1.2756e+00_rb,2.5511e+00_rb,3.8262e+00_rb,5.1016e+00_rb,6.3755e+00_rb, &
     &7.6486e+00_rb,8.9147e+00_rb,1.0203e+01_rb/)
      kao(:, 2, 6,13) = (/ &
     &6.9368e-02_rb,1.3655e+00_rb,2.7307e+00_rb,4.0953e+00_rb,5.4595e+00_rb,6.8218e+00_rb, &
     &8.1822e+00_rb,9.5288e+00_rb,1.0919e+01_rb/)
      kao(:, 3, 6,13) = (/ &
     &7.0610e-02_rb,1.4598e+00_rb,2.9193e+00_rb,4.3789e+00_rb,5.8375e+00_rb,7.2946e+00_rb, &
     &8.7480e+00_rb,1.0185e+01_rb,1.1675e+01_rb/)
      kao(:, 4, 6,13) = (/ &
     &7.1895e-02_rb,1.5499e+00_rb,3.0992e+00_rb,4.6481e+00_rb,6.1963e+00_rb,7.7431e+00_rb, &
     &9.2863e+00_rb,1.0814e+01_rb,1.2392e+01_rb/)
      kao(:, 5, 6,13) = (/ &
     &7.2713e-02_rb,1.6401e+00_rb,3.2796e+00_rb,4.9184e+00_rb,6.5572e+00_rb,8.1927e+00_rb, &
     &9.8252e+00_rb,1.1440e+01_rb,1.3114e+01_rb/)
      kao(:, 1, 7,13) = (/ &
     &1.4148e-01_rb,1.2306e+00_rb,2.4611e+00_rb,3.6917e+00_rb,4.9218e+00_rb,6.1530e+00_rb, &
     &7.3843e+00_rb,8.6153e+00_rb,9.8436e+00_rb/)
      kao(:, 2, 7,13) = (/ &
     &1.4468e-01_rb,1.3267e+00_rb,2.6534e+00_rb,3.9799e+00_rb,5.3067e+00_rb,6.6328e+00_rb, &
     &7.9589e+00_rb,9.2835e+00_rb,1.0613e+01_rb/)
      kao(:, 3, 7,13) = (/ &
     &1.4723e-01_rb,1.4271e+00_rb,2.8538e+00_rb,4.2801e+00_rb,5.7062e+00_rb,7.1323e+00_rb, &
     &8.5577e+00_rb,9.9797e+00_rb,1.1412e+01_rb/)
      kao(:, 4, 7,13) = (/ &
     &1.4959e-01_rb,1.5234e+00_rb,3.0460e+00_rb,4.5687e+00_rb,6.0905e+00_rb,7.6133e+00_rb, &
     &9.1342e+00_rb,1.0653e+01_rb,1.2180e+01_rb/)
      kao(:, 5, 7,13) = (/ &
     &1.5109e-01_rb,1.6156e+00_rb,3.2301e+00_rb,4.8444e+00_rb,6.4594e+00_rb,8.0732e+00_rb, &
     &9.6859e+00_rb,1.1294e+01_rb,1.2918e+01_rb/)
      kao(:, 1, 8,13) = (/ &
     &3.4983e-01_rb,1.1544e+00_rb,2.3087e+00_rb,3.4632e+00_rb,4.6176e+00_rb,5.7728e+00_rb, &
     &6.9283e+00_rb,8.0852e+00_rb,9.2350e+00_rb/)
      kao(:, 2, 8,13) = (/ &
     &3.5757e-01_rb,1.2554e+00_rb,2.5092e+00_rb,3.7633e+00_rb,5.0176e+00_rb,6.2716e+00_rb, &
     &7.5265e+00_rb,8.7822e+00_rb,1.0034e+01_rb/)
      kao(:, 3, 8,13) = (/ &
     &3.6359e-01_rb,1.3606e+00_rb,2.7192e+00_rb,4.0780e+00_rb,5.4366e+00_rb,6.7958e+00_rb, &
     &8.1543e+00_rb,9.5151e+00_rb,1.0871e+01_rb/)
      kao(:, 4, 8,13) = (/ &
     &3.6947e-01_rb,1.4619e+00_rb,2.9219e+00_rb,4.3820e+00_rb,5.8417e+00_rb,7.3019e+00_rb, &
     &8.7622e+00_rb,1.0223e+01_rb,1.1681e+01_rb/)
      kao(:, 5, 8,13) = (/ &
     &3.7269e-01_rb,1.5629e+00_rb,3.1234e+00_rb,4.6839e+00_rb,6.2445e+00_rb,7.8052e+00_rb, &
     &9.3661e+00_rb,1.0928e+01_rb,1.2487e+01_rb/)
      kao(:, 1, 9,13) = (/ &
     &1.6120e+00_rb,2.1579e+00_rb,2.5856e+00_rb,3.2058e+00_rb,4.2226e+00_rb,5.2767e+00_rb, &
     &6.3317e+00_rb,7.3871e+00_rb,8.4404e+00_rb/)
      kao(:, 2, 9,13) = (/ &
     &1.6481e+00_rb,2.2537e+00_rb,2.6687e+00_rb,3.4901e+00_rb,4.6465e+00_rb,5.8062e+00_rb, &
     &6.9659e+00_rb,8.1266e+00_rb,9.2849e+00_rb/)
      kao(:, 3, 9,13) = (/ &
     &1.6791e+00_rb,2.3216e+00_rb,2.7898e+00_rb,3.8113e+00_rb,5.0781e+00_rb,6.3453e+00_rb, &
     &7.6127e+00_rb,8.8803e+00_rb,1.0146e+01_rb/)
      kao(:, 4, 9,13) = (/ &
     &1.7002e+00_rb,2.3849e+00_rb,2.9226e+00_rb,4.1389e+00_rb,5.5144e+00_rb,6.8902e+00_rb, &
     &8.2665e+00_rb,9.6440e+00_rb,1.1018e+01_rb/)
      kao(:, 5, 9,13) = (/ &
     &1.7139e+00_rb,2.4356e+00_rb,3.0744e+00_rb,4.4589e+00_rb,5.9418e+00_rb,7.4246e+00_rb, &
     &8.9078e+00_rb,1.0391e+01_rb,1.1873e+01_rb/)
      kao(:, 1,10,13) = (/ &
     &8.1900e+00_rb,7.5735e+00_rb,7.5164e+00_rb,7.4868e+00_rb,7.2490e+00_rb,6.5441e+00_rb, &
     &6.0001e+00_rb,6.6902e+00_rb,7.6402e+00_rb/)
      kao(:, 2,10,13) = (/ &
     &8.3686e+00_rb,7.7990e+00_rb,7.8449e+00_rb,7.8651e+00_rb,7.5310e+00_rb,6.7442e+00_rb, &
     &6.5109e+00_rb,7.4533e+00_rb,8.5097e+00_rb/)
      kao(:, 3,10,13) = (/ &
     &8.5094e+00_rb,8.0068e+00_rb,8.1129e+00_rb,8.1601e+00_rb,7.7387e+00_rb,7.0241e+00_rb, &
     &7.1098e+00_rb,8.2483e+00_rb,9.4190e+00_rb/)
      kao(:, 4,10,13) = (/ &
     &8.6083e+00_rb,8.1729e+00_rb,8.3302e+00_rb,8.3900e+00_rb,7.9589e+00_rb,7.3305e+00_rb, &
     &7.7681e+00_rb,9.0471e+00_rb,1.0331e+01_rb/)
      kao(:, 5,10,13) = (/ &
     &8.6642e+00_rb,8.2947e+00_rb,8.5084e+00_rb,8.6017e+00_rb,8.1354e+00_rb,7.7044e+00_rb, &
     &8.4486e+00_rb,9.8464e+00_rb,1.1244e+01_rb/)
      kao(:, 1,11,13) = (/ &
     &1.4422e+01_rb,1.2923e+01_rb,1.2146e+01_rb,1.1467e+01_rb,1.0797e+01_rb,9.7043e+00_rb, &
     &7.9341e+00_rb,6.4523e+00_rb,7.2283e+00_rb/)
      kao(:, 2,11,13) = (/ &
     &1.4693e+01_rb,1.3224e+01_rb,1.2546e+01_rb,1.1961e+01_rb,1.1284e+01_rb,1.0051e+01_rb, &
     &8.2717e+00_rb,7.1710e+00_rb,8.1411e+00_rb/)
      kao(:, 3,11,13) = (/ &
     &1.4865e+01_rb,1.3437e+01_rb,1.2872e+01_rb,1.2352e+01_rb,1.1673e+01_rb,1.0373e+01_rb, &
     &8.6477e+00_rb,7.9776e+00_rb,9.0911e+00_rb/)
      kao(:, 4,11,13) = (/ &
     &1.4985e+01_rb,1.3594e+01_rb,1.3136e+01_rb,1.2689e+01_rb,1.2000e+01_rb,1.0700e+01_rb, &
     &9.0501e+00_rb,8.7877e+00_rb,1.0028e+01_rb/)
      kao(:, 5,11,13) = (/ &
     &1.5049e+01_rb,1.3704e+01_rb,1.3344e+01_rb,1.2975e+01_rb,1.2311e+01_rb,1.0977e+01_rb, &
     &9.5392e+00_rb,9.6092e+00_rb,1.0970e+01_rb/)
      kao(:, 1,12,13) = (/ &
     &1.8498e+01_rb,1.6493e+01_rb,1.5330e+01_rb,1.4296e+01_rb,1.3227e+01_rb,1.1776e+01_rb, &
     &9.6093e+00_rb,6.9105e+00_rb,6.8698e+00_rb/)
      kao(:, 2,12,13) = (/ &
     &1.8757e+01_rb,1.6799e+01_rb,1.5767e+01_rb,1.4839e+01_rb,1.3780e+01_rb,1.2237e+01_rb, &
     &1.0031e+01_rb,7.5179e+00_rb,7.8121e+00_rb/)
      kao(:, 3,12,13) = (/ &
     &1.8947e+01_rb,1.7036e+01_rb,1.6095e+01_rb,1.5249e+01_rb,1.4223e+01_rb,1.2650e+01_rb, &
     &1.0480e+01_rb,8.1785e+00_rb,8.7273e+00_rb/)
      kao(:, 4,12,13) = (/ &
     &1.9063e+01_rb,1.7202e+01_rb,1.6365e+01_rb,1.5574e+01_rb,1.4614e+01_rb,1.3047e+01_rb, &
     &1.0927e+01_rb,8.8781e+00_rb,9.6514e+00_rb/)
      kao(:, 5,12,13) = (/ &
     &1.9121e+01_rb,1.7309e+01_rb,1.6592e+01_rb,1.5868e+01_rb,1.4993e+01_rb,1.3399e+01_rb, &
     &1.1411e+01_rb,9.6204e+00_rb,1.0609e+01_rb/)
      kao(:, 1,13,13) = (/ &
     &1.8681e+01_rb,1.6791e+01_rb,1.5759e+01_rb,1.4851e+01_rb,1.3770e+01_rb,1.2167e+01_rb, &
     &1.0009e+01_rb,7.2580e+00_rb,6.5238e+00_rb/)
      kao(:, 2,13,13) = (/ &
     &1.8892e+01_rb,1.7070e+01_rb,1.6163e+01_rb,1.5373e+01_rb,1.4290e+01_rb,1.2645e+01_rb, &
     &1.0505e+01_rb,7.8149e+00_rb,7.3987e+00_rb/)
      kao(:, 3,13,13) = (/ &
     &1.9028e+01_rb,1.7273e+01_rb,1.6517e+01_rb,1.5784e+01_rb,1.4702e+01_rb,1.3114e+01_rb, &
     &1.1026e+01_rb,8.4153e+00_rb,8.2894e+00_rb/)
      kao(:, 4,13,13) = (/ &
     &1.9122e+01_rb,1.7418e+01_rb,1.6814e+01_rb,1.6131e+01_rb,1.5108e+01_rb,1.3570e+01_rb, &
     &1.1549e+01_rb,9.0457e+00_rb,9.1940e+00_rb/)
      kao(:, 5,13,13) = (/ &
     &1.9151e+01_rb,1.7520e+01_rb,1.7041e+01_rb,1.6440e+01_rb,1.5494e+01_rb,1.4010e+01_rb, &
     &1.2106e+01_rb,9.7104e+00_rb,1.0102e+01_rb/)
      kao(:, 1, 1,14) = (/ &
     &3.3087e-03_rb,1.8715e+00_rb,3.7152e+00_rb,5.5155e+00_rb,7.2487e+00_rb,8.8573e+00_rb, &
     &1.0201e+01_rb,1.0807e+01_rb,1.4497e+01_rb/)
      kao(:, 2, 1,14) = (/ &
     &3.2220e-03_rb,1.9525e+00_rb,3.8750e+00_rb,5.7533e+00_rb,7.5552e+00_rb,9.2212e+00_rb, &
     &1.0608e+01_rb,1.1197e+01_rb,1.5110e+01_rb/)
      kao(:, 3, 1,14) = (/ &
     &3.1339e-03_rb,2.0260e+00_rb,4.0195e+00_rb,5.9653e+00_rb,7.8344e+00_rb,9.5608e+00_rb, &
     &1.0981e+01_rb,1.1562e+01_rb,1.5669e+01_rb/)
      kao(:, 4, 1,14) = (/ &
     &3.0455e-03_rb,2.1060e+00_rb,4.1769e+00_rb,6.1939e+00_rb,8.1222e+00_rb,9.8877e+00_rb, &
     &1.1323e+01_rb,1.1897e+01_rb,1.6244e+01_rb/)
      kao(:, 5, 1,14) = (/ &
     &2.9339e-03_rb,2.1869e+00_rb,4.3369e+00_rb,6.4318e+00_rb,8.4325e+00_rb,1.0262e+01_rb, &
     &1.1718e+01_rb,1.2199e+01_rb,1.6865e+01_rb/)
      kao(:, 1, 2,14) = (/ &
     &5.2878e-03_rb,2.0475e+00_rb,4.0760e+00_rb,6.0751e+00_rb,8.0240e+00_rb,9.8767e+00_rb, &
     &1.1510e+01_rb,1.2463e+01_rb,1.6048e+01_rb/)
      kao(:, 2, 2,14) = (/ &
     &5.1473e-03_rb,2.1393e+00_rb,4.2595e+00_rb,6.3491e+00_rb,8.3874e+00_rb,1.0326e+01_rb, &
     &1.2039e+01_rb,1.3009e+01_rb,1.6775e+01_rb/)
      kao(:, 3, 2,14) = (/ &
     &5.0049e-03_rb,2.2262e+00_rb,4.4297e+00_rb,6.5989e+00_rb,8.7109e+00_rb,1.0721e+01_rb, &
     &1.2503e+01_rb,1.3517e+01_rb,1.7422e+01_rb/)
      kao(:, 4, 2,14) = (/ &
     &4.8227e-03_rb,2.3172e+00_rb,4.6099e+00_rb,6.8638e+00_rb,9.0568e+00_rb,1.1128e+01_rb, &
     &1.2937e+01_rb,1.3964e+01_rb,1.8113e+01_rb/)
      kao(:, 5, 2,14) = (/ &
     &4.6542e-03_rb,2.4084e+00_rb,4.7911e+00_rb,7.1333e+00_rb,9.4086e+00_rb,1.1557e+01_rb, &
     &1.3418e+01_rb,1.4407e+01_rb,1.8817e+01_rb/)
      kao(:, 1, 3,14) = (/ &
     &1.1438e-02_rb,2.1889e+00_rb,4.3692e+00_rb,6.5353e+00_rb,8.6778e+00_rb,1.0773e+01_rb, &
     &1.2758e+01_rb,1.4346e+01_rb,1.7356e+01_rb/)
      kao(:, 2, 3,14) = (/ &
     &1.1139e-02_rb,2.3041e+00_rb,4.5995e+00_rb,6.8793e+00_rb,9.1316e+00_rb,1.1334e+01_rb, &
     &1.3411e+01_rb,1.5052e+01_rb,1.8263e+01_rb/)
      kao(:, 3, 3,14) = (/ &
     &1.0770e-02_rb,2.4104e+00_rb,4.8103e+00_rb,7.1935e+00_rb,9.5485e+00_rb,1.1847e+01_rb, &
     &1.4017e+01_rb,1.5716e+01_rb,1.9097e+01_rb/)
      kao(:, 4, 3,14) = (/ &
     &1.0423e-02_rb,2.5146e+00_rb,5.0181e+00_rb,7.5021e+00_rb,9.9570e+00_rb,1.2351e+01_rb, &
     &1.4597e+01_rb,1.6319e+01_rb,1.9914e+01_rb/)
      kao(:, 5, 3,14) = (/ &
     &1.0104e-02_rb,2.6169e+00_rb,5.2203e+00_rb,7.8059e+00_rb,1.0356e+01_rb,1.2839e+01_rb, &
     &1.5169e+01_rb,1.6937e+01_rb,2.0712e+01_rb/)
      kao(:, 1, 4,14) = (/ &
     &2.5365e-02_rb,2.3050e+00_rb,4.6063e+00_rb,6.9017e+00_rb,9.1853e+00_rb,1.1448e+01_rb, &
     &1.3656e+01_rb,1.5659e+01_rb,1.8370e+01_rb/)
      kao(:, 2, 4,14) = (/ &
     &2.4778e-02_rb,2.4401e+00_rb,4.8764e+00_rb,7.3060e+00_rb,9.7240e+00_rb,1.2119e+01_rb, &
     &1.4458e+01_rb,1.6580e+01_rb,1.9448e+01_rb/)
      kao(:, 3, 4,14) = (/ &
     &2.4097e-02_rb,2.5614e+00_rb,5.1183e+00_rb,7.6688e+00_rb,1.0206e+01_rb,1.2719e+01_rb, &
     &1.5172e+01_rb,1.7403e+01_rb,2.0411e+01_rb/)
      kao(:, 4, 4,14) = (/ &
     &2.3478e-02_rb,2.6774e+00_rb,5.3500e+00_rb,8.0159e+00_rb,1.0665e+01_rb,1.3289e+01_rb, &
     &1.5848e+01_rb,1.8161e+01_rb,2.1330e+01_rb/)
      kao(:, 5, 4,14) = (/ &
     &2.2944e-02_rb,2.7978e+00_rb,5.5896e+00_rb,8.3727e+00_rb,1.1139e+01_rb,1.3876e+01_rb, &
     &1.6535e+01_rb,1.8911e+01_rb,2.2278e+01_rb/)
      kao(:, 1, 5,14) = (/ &
     &5.2400e-02_rb,2.3882e+00_rb,4.7744e+00_rb,7.1579e+00_rb,9.5370e+00_rb,1.1906e+01_rb, &
     &1.4251e+01_rb,1.6502e+01_rb,1.9074e+01_rb/)
      kao(:, 2, 5,14) = (/ &
     &5.1529e-02_rb,2.5417e+00_rb,5.0820e+00_rb,7.6184e+00_rb,1.0150e+01_rb,1.2670e+01_rb, &
     &1.5165e+01_rb,1.7564e+01_rb,2.0300e+01_rb/)
      kao(:, 3, 5,14) = (/ &
     &5.0713e-02_rb,2.6834e+00_rb,5.3644e+00_rb,8.0415e+00_rb,1.0713e+01_rb,1.3375e+01_rb, &
     &1.6005e+01_rb,1.8525e+01_rb,2.1426e+01_rb/)
      kao(:, 4, 5,14) = (/ &
     &4.9736e-02_rb,2.8194e+00_rb,5.6359e+00_rb,8.4488e+00_rb,1.1255e+01_rb,1.4048e+01_rb, &
     &1.6805e+01_rb,1.9440e+01_rb,2.2511e+01_rb/)
      kao(:, 5, 5,14) = (/ &
     &4.8999e-02_rb,2.9595e+00_rb,5.9157e+00_rb,8.8682e+00_rb,1.1813e+01_rb,1.4742e+01_rb, &
     &1.7634e+01_rb,2.0380e+01_rb,2.3625e+01_rb/)
      kao(:, 1, 6,14) = (/ &
     &1.0351e-01_rb,2.4306e+00_rb,4.8601e+00_rb,7.2890e+00_rb,9.7147e+00_rb,1.2135e+01_rb, &
     &1.4543e+01_rb,1.6905e+01_rb,1.9429e+01_rb/)
      kao(:, 2, 6,14) = (/ &
     &1.0255e-01_rb,2.6140e+00_rb,5.2274e+00_rb,7.8383e+00_rb,1.0448e+01_rb,1.3051e+01_rb, &
     &1.5640e+01_rb,1.8174e+01_rb,2.0896e+01_rb/)
      kao(:, 3, 6,14) = (/ &
     &1.0183e-01_rb,2.7755e+00_rb,5.5503e+00_rb,8.3234e+00_rb,1.1093e+01_rb,1.3857e+01_rb, &
     &1.6607e+01_rb,1.9300e+01_rb,2.2186e+01_rb/)
      kao(:, 4, 6,14) = (/ &
     &1.0081e-01_rb,2.9307e+00_rb,5.8597e+00_rb,8.7876e+00_rb,1.1711e+01_rb,1.4627e+01_rb, &
     &1.7527e+01_rb,2.0360e+01_rb,2.3422e+01_rb/)
      kao(:, 5, 6,14) = (/ &
     &1.0025e-01_rb,3.0883e+00_rb,6.1762e+00_rb,9.2605e+00_rb,1.2342e+01_rb,1.5414e+01_rb, &
     &1.8466e+01_rb,2.1444e+01_rb,2.4683e+01_rb/)
      kao(:, 1, 7,14) = (/ &
     &2.1803e-01_rb,2.4631e+00_rb,4.9259e+00_rb,7.3877e+00_rb,9.8477e+00_rb,1.2306e+01_rb, &
     &1.4757e+01_rb,1.7182e+01_rb,1.9695e+01_rb/)
      kao(:, 2, 7,14) = (/ &
     &2.1806e-01_rb,2.6703e+00_rb,5.3400e+00_rb,8.0091e+00_rb,1.0677e+01_rb,1.3341e+01_rb, &
     &1.6000e+01_rb,1.8630e+01_rb,2.1354e+01_rb/)
      kao(:, 3, 7,14) = (/ &
     &2.1815e-01_rb,2.8507e+00_rb,5.7004e+00_rb,8.5504e+00_rb,1.1399e+01_rb,1.4242e+01_rb, &
     &1.7080e+01_rb,1.9888e+01_rb,2.2798e+01_rb/)
      kao(:, 4, 7,14) = (/ &
     &2.1765e-01_rb,3.0257e+00_rb,6.0505e+00_rb,9.0745e+00_rb,1.2096e+01_rb,1.5115e+01_rb, &
     &1.8125e+01_rb,2.1099e+01_rb,2.4193e+01_rb/)
      kao(:, 5, 7,14) = (/ &
     &2.1725e-01_rb,3.2088e+00_rb,6.4163e+00_rb,9.6232e+00_rb,1.2829e+01_rb,1.6028e+01_rb, &
     &1.9219e+01_rb,2.2369e+01_rb,2.5657e+01_rb/)
      kao(:, 1, 8,14) = (/ &
     &5.5719e-01_rb,2.4793e+00_rb,4.9583e+00_rb,7.4367e+00_rb,9.9152e+00_rb,1.2393e+01_rb, &
     &1.4868e+01_rb,1.7332e+01_rb,1.9830e+01_rb/)
      kao(:, 2, 8,14) = (/ &
     &5.6059e-01_rb,2.7128e+00_rb,5.4248e+00_rb,8.1366e+00_rb,1.0849e+01_rb,1.3560e+01_rb, &
     &1.6268e+01_rb,1.8965e+01_rb,2.1697e+01_rb/)
      kao(:, 3, 8,14) = (/ &
     &5.6435e-01_rb,2.9177e+00_rb,5.8349e+00_rb,8.7514e+00_rb,1.1668e+01_rb,1.4583e+01_rb, &
     &1.7497e+01_rb,2.0397e+01_rb,2.3335e+01_rb/)
      kao(:, 4, 8,14) = (/ &
     &5.6354e-01_rb,3.1161e+00_rb,6.2315e+00_rb,9.3470e+00_rb,1.2462e+01_rb,1.5576e+01_rb, &
     &1.8684e+01_rb,2.1779e+01_rb,2.4923e+01_rb/)
      kao(:, 5, 8,14) = (/ &
     &5.6160e-01_rb,3.3171e+00_rb,6.6335e+00_rb,9.9496e+00_rb,1.3266e+01_rb,1.6580e+01_rb, &
     &1.9888e+01_rb,2.3183e+01_rb,2.6532e+01_rb/)
      kao(:, 1, 9,14) = (/ &
     &2.7023e+00_rb,3.1133e+00_rb,4.9392e+00_rb,7.4089e+00_rb,9.8776e+00_rb,1.2348e+01_rb, &
     &1.4816e+01_rb,1.7282e+01_rb,1.9755e+01_rb/)
      kao(:, 2, 9,14) = (/ &
     &2.7329e+00_rb,3.1899e+00_rb,5.4498e+00_rb,8.1745e+00_rb,1.0899e+01_rb,1.3624e+01_rb, &
     &1.6348e+01_rb,1.9070e+01_rb,2.1798e+01_rb/)
      kao(:, 3, 9,14) = (/ &
     &2.7414e+00_rb,3.2928e+00_rb,5.9087e+00_rb,8.8629e+00_rb,1.1816e+01_rb,1.4770e+01_rb, &
     &1.7722e+01_rb,2.0675e+01_rb,2.3632e+01_rb/)
      kao(:, 4, 9,14) = (/ &
     &2.7386e+00_rb,3.4095e+00_rb,6.3408e+00_rb,9.5112e+00_rb,1.2680e+01_rb,1.5850e+01_rb, &
     &1.9019e+01_rb,2.2186e+01_rb,2.5359e+01_rb/)
      kao(:, 5, 9,14) = (/ &
     &2.7253e+00_rb,3.5556e+00_rb,6.7742e+00_rb,1.0160e+01_rb,1.3547e+01_rb,1.6934e+01_rb, &
     &2.0319e+01_rb,2.3701e+01_rb,2.7094e+01_rb/)
      kao(:, 1,10,14) = (/ &
     &1.4620e+01_rb,1.3012e+01_rb,1.2093e+01_rb,1.1252e+01_rb,1.0888e+01_rb,1.2263e+01_rb, &
     &1.4693e+01_rb,1.7142e+01_rb,1.9590e+01_rb/)
      kao(:, 2,10,14) = (/ &
     &1.4753e+01_rb,1.3186e+01_rb,1.2290e+01_rb,1.1490e+01_rb,1.1589e+01_rb,1.3604e+01_rb, &
     &1.6324e+01_rb,1.9044e+01_rb,2.1765e+01_rb/)
      kao(:, 3,10,14) = (/ &
     &1.4775e+01_rb,1.3219e+01_rb,1.2416e+01_rb,1.1687e+01_rb,1.2328e+01_rb,1.4817e+01_rb, &
     &1.7779e+01_rb,2.0743e+01_rb,2.3705e+01_rb/)
      kao(:, 4,10,14) = (/ &
     &1.4749e+01_rb,1.3203e+01_rb,1.2546e+01_rb,1.1974e+01_rb,1.3088e+01_rb,1.5981e+01_rb, &
     &1.9178e+01_rb,2.2373e+01_rb,2.5570e+01_rb/)
      kao(:, 5,10,14) = (/ &
     &1.4700e+01_rb,1.3186e+01_rb,1.2659e+01_rb,1.2331e+01_rb,1.3931e+01_rb,1.7138e+01_rb, &
     &2.0565e+01_rb,2.3991e+01_rb,2.7418e+01_rb/)
      kao(:, 1,11,14) = (/ &
     &2.7410e+01_rb,2.3992e+01_rb,2.1335e+01_rb,1.9158e+01_rb,1.6847e+01_rb,1.4928e+01_rb, &
     &1.5265e+01_rb,1.7677e+01_rb,2.0202e+01_rb/)
      kao(:, 2,11,14) = (/ &
     &2.7494e+01_rb,2.4097e+01_rb,2.1553e+01_rb,1.9434e+01_rb,1.7307e+01_rb,1.5809e+01_rb, &
     &1.6793e+01_rb,1.9551e+01_rb,2.2343e+01_rb/)
      kao(:, 3,11,14) = (/ &
     &2.7540e+01_rb,2.4181e+01_rb,2.1730e+01_rb,1.9742e+01_rb,1.7701e+01_rb,1.6732e+01_rb, &
     &1.8269e+01_rb,2.1306e+01_rb,2.4350e+01_rb/)
      kao(:, 4,11,14) = (/ &
     &2.7514e+01_rb,2.4204e+01_rb,2.1897e+01_rb,2.0034e+01_rb,1.8171e+01_rb,1.7696e+01_rb, &
     &1.9766e+01_rb,2.3053e+01_rb,2.6341e+01_rb/)
      kao(:, 5,11,14) = (/ &
     &2.7458e+01_rb,2.4204e+01_rb,2.2040e+01_rb,2.0324e+01_rb,1.8730e+01_rb,1.8750e+01_rb, &
     &2.1236e+01_rb,2.4764e+01_rb,2.8296e+01_rb/)
      kao(:, 1,12,14) = (/ &
     &3.7739e+01_rb,3.3022e+01_rb,2.9081e+01_rb,2.5731e+01_rb,2.2384e+01_rb,1.9192e+01_rb, &
     &1.6882e+01_rb,1.7924e+01_rb,2.0483e+01_rb/)
      kao(:, 2,12,14) = (/ &
     &3.7888e+01_rb,3.3157e+01_rb,2.9316e+01_rb,2.6118e+01_rb,2.2987e+01_rb,1.9918e+01_rb, &
     &1.8210e+01_rb,1.9805e+01_rb,2.2631e+01_rb/)
      kao(:, 3,12,14) = (/ &
     &3.7945e+01_rb,3.3236e+01_rb,2.9563e+01_rb,2.6509e+01_rb,2.3567e+01_rb,2.0714e+01_rb, &
     &1.9521e+01_rb,2.1672e+01_rb,2.4760e+01_rb/)
      kao(:, 4,12,14) = (/ &
     &3.7956e+01_rb,3.3289e+01_rb,2.9765e+01_rb,2.6932e+01_rb,2.4108e+01_rb,2.1541e+01_rb, &
     &2.0877e+01_rb,2.3478e+01_rb,2.6821e+01_rb/)
      kao(:, 5,12,14) = (/ &
     &3.7967e+01_rb,3.3331e+01_rb,2.9953e+01_rb,2.7347e+01_rb,2.4651e+01_rb,2.2457e+01_rb, &
     &2.2265e+01_rb,2.5248e+01_rb,2.8840e+01_rb/)
      kao(:, 1,13,14) = (/ &
     &4.1396e+01_rb,3.6239e+01_rb,3.2059e+01_rb,2.8493e+01_rb,2.4944e+01_rb,2.1585e+01_rb, &
     &1.8204e+01_rb,1.7791e+01_rb,2.0316e+01_rb/)
      kao(:, 2,13,14) = (/ &
     &4.1543e+01_rb,3.6421e+01_rb,3.2427e+01_rb,2.9033e+01_rb,2.5736e+01_rb,2.2468e+01_rb, &
     &1.9493e+01_rb,1.9781e+01_rb,2.2598e+01_rb/)
      kao(:, 3,13,14) = (/ &
     &4.1678e+01_rb,3.6592e+01_rb,3.2739e+01_rb,2.9584e+01_rb,2.6500e+01_rb,2.3336e+01_rb, &
     &2.0749e+01_rb,2.1701e+01_rb,2.4790e+01_rb/)
      kao(:, 4,13,14) = (/ &
     &4.1767e+01_rb,3.6719e+01_rb,3.3041e+01_rb,3.0106e+01_rb,2.7204e+01_rb,2.4205e+01_rb, &
     &2.2086e+01_rb,2.3604e+01_rb,2.6962e+01_rb/)
      kao(:, 5,13,14) = (/ &
     &4.1856e+01_rb,3.6841e+01_rb,3.3340e+01_rb,3.0667e+01_rb,2.7878e+01_rb,2.5159e+01_rb, &
     &2.3471e+01_rb,2.5553e+01_rb,2.9188e+01_rb/)
      kao(:, 1, 1,15) = (/ &
     &4.1661e-03_rb,3.0613e+00_rb,6.0764e+00_rb,9.0246e+00_rb,1.1861e+01_rb,1.4480e+01_rb, &
     &1.6622e+01_rb,1.7284e+01_rb,2.3722e+01_rb/)
      kao(:, 2, 1,15) = (/ &
     &4.0148e-03_rb,3.1418e+00_rb,6.2320e+00_rb,9.2486e+00_rb,1.2140e+01_rb,1.4819e+01_rb, &
     &1.7035e+01_rb,1.7764e+01_rb,2.4279e+01_rb/)
      kao(:, 3, 1,15) = (/ &
     &3.8658e-03_rb,3.2144e+00_rb,6.3745e+00_rb,9.4527e+00_rb,1.2398e+01_rb,1.5111e+01_rb, &
     &1.7346e+01_rb,1.8143e+01_rb,2.4796e+01_rb/)
      kao(:, 4, 1,15) = (/ &
     &3.7196e-03_rb,3.2826e+00_rb,6.5071e+00_rb,9.6422e+00_rb,1.2633e+01_rb,1.5374e+01_rb, &
     &1.7599e+01_rb,1.8433e+01_rb,2.5267e+01_rb/)
      kao(:, 5, 1,15) = (/ &
     &3.5761e-03_rb,3.3515e+00_rb,6.6395e+00_rb,9.8324e+00_rb,1.2873e+01_rb,1.5639e+01_rb, &
     &1.7844e+01_rb,1.8646e+01_rb,2.5747e+01_rb/)
      kao(:, 1, 2,15) = (/ &
     &6.8577e-03_rb,3.3744e+00_rb,6.7125e+00_rb,9.9952e+00_rb,1.3184e+01_rb,1.6201e+01_rb, &
     &1.8839e+01_rb,2.0375e+01_rb,2.6368e+01_rb/)
      kao(:, 2, 2,15) = (/ &
     &6.5948e-03_rb,3.4888e+00_rb,6.9393e+00_rb,1.0331e+01_rb,1.3623e+01_rb,1.6730e+01_rb, &
     &1.9423e+01_rb,2.0902e+01_rb,2.7246e+01_rb/)
      kao(:, 3, 2,15) = (/ &
     &6.3372e-03_rb,3.5940e+00_rb,7.1483e+00_rb,1.0636e+01_rb,1.4019e+01_rb,1.7199e+01_rb, &
     &1.9946e+01_rb,2.1371e+01_rb,2.8038e+01_rb/)
      kao(:, 4, 2,15) = (/ &
     &6.0860e-03_rb,3.6975e+00_rb,7.3505e+00_rb,1.0936e+01_rb,1.4402e+01_rb,1.7655e+01_rb, &
     &2.0430e+01_rb,2.1802e+01_rb,2.8803e+01_rb/)
      kao(:, 5, 2,15) = (/ &
     &5.8418e-03_rb,3.7949e+00_rb,7.5419e+00_rb,1.1217e+01_rb,1.4770e+01_rb,1.8090e+01_rb, &
     &2.0913e+01_rb,2.2203e+01_rb,2.9539e+01_rb/)
      kao(:, 1, 3,15) = (/ &
     &1.5312e-02_rb,3.7161e+00_rb,7.4117e+00_rb,1.1074e+01_rb,1.4677e+01_rb,1.8180e+01_rb, &
     &2.1427e+01_rb,2.3812e+01_rb,2.9355e+01_rb/)
      kao(:, 2, 3,15) = (/ &
     &1.4697e-02_rb,3.8703e+00_rb,7.7181e+00_rb,1.1532e+01_rb,1.5288e+01_rb,1.8926e+01_rb, &
     &2.2293e+01_rb,2.4742e+01_rb,3.0575e+01_rb/)
      kao(:, 3, 3,15) = (/ &
     &1.4095e-02_rb,4.0069e+00_rb,7.9901e+00_rb,1.1939e+01_rb,1.5823e+01_rb,1.9587e+01_rb, &
     &2.3063e+01_rb,2.5568e+01_rb,3.1646e+01_rb/)
      kao(:, 4, 3,15) = (/ &
     &1.3512e-02_rb,4.1402e+00_rb,8.2563e+00_rb,1.2333e+01_rb,1.6342e+01_rb,2.0226e+01_rb, &
     &2.3806e+01_rb,2.6351e+01_rb,3.2684e+01_rb/)
      kao(:, 5, 3,15) = (/ &
     &1.2949e-02_rb,4.2659e+00_rb,8.5058e+00_rb,1.2707e+01_rb,1.6836e+01_rb,2.0833e+01_rb, &
     &2.4517e+01_rb,2.7109e+01_rb,3.3673e+01_rb/)
      kao(:, 1, 4,15) = (/ &
     &3.5017e-02_rb,4.0833e+00_rb,8.1553e+00_rb,1.2211e+01_rb,1.6236e+01_rb,2.0204e+01_rb, &
     &2.4024e+01_rb,2.7316e+01_rb,3.2472e+01_rb/)
      kao(:, 2, 4,15) = (/ &
     &3.3537e-02_rb,4.2828e+00_rb,8.5541e+00_rb,1.2808e+01_rb,1.7029e+01_rb,2.1189e+01_rb, &
     &2.5196e+01_rb,2.8637e+01_rb,3.4057e+01_rb/)
      kao(:, 3, 4,15) = (/ &
     &3.2113e-02_rb,4.4600e+00_rb,8.9081e+00_rb,1.3336e+01_rb,1.7733e+01_rb,2.2059e+01_rb, &
     &2.6228e+01_rb,2.9796e+01_rb,3.5465e+01_rb/)
      kao(:, 4, 4,15) = (/ &
     &3.0746e-02_rb,4.6307e+00_rb,9.2473e+00_rb,1.3839e+01_rb,1.8403e+01_rb,2.2892e+01_rb, &
     &2.7214e+01_rb,3.0894e+01_rb,3.6805e+01_rb/)
      kao(:, 5, 4,15) = (/ &
     &2.9433e-02_rb,4.7874e+00_rb,9.5611e+00_rb,1.4314e+01_rb,1.9029e+01_rb,2.3669e+01_rb, &
     &2.8132e+01_rb,3.1929e+01_rb,3.8057e+01_rb/)
      kao(:, 1, 5,15) = (/ &
     &7.3896e-02_rb,4.4639e+00_rb,8.9220e+00_rb,1.3370e+01_rb,1.7804e+01_rb,2.2204e+01_rb, &
     &2.6523e+01_rb,3.0537e+01_rb,3.5608e+01_rb/)
      kao(:, 2, 5,15) = (/ &
     &7.0692e-02_rb,4.7138e+00_rb,9.4187e+00_rb,1.4116e+01_rb,1.8797e+01_rb,2.3443e+01_rb, &
     &2.8005e+01_rb,3.2244e+01_rb,3.7595e+01_rb/)
      kao(:, 3, 5,15) = (/ &
     &6.7622e-02_rb,4.9383e+00_rb,9.8707e+00_rb,1.4791e+01_rb,1.9696e+01_rb,2.4560e+01_rb, &
     &2.9335e+01_rb,3.3760e+01_rb,3.9391e+01_rb/)
      kao(:, 4, 5,15) = (/ &
     &6.4677e-02_rb,5.1515e+00_rb,1.0296e+01_rb,1.5428e+01_rb,2.0542e+01_rb,2.5616e+01_rb, &
     &3.0597e+01_rb,3.5203e+01_rb,4.1084e+01_rb/)
      kao(:, 5, 5,15) = (/ &
     &6.1858e-02_rb,5.3461e+00_rb,1.0685e+01_rb,1.6012e+01_rb,2.1318e+01_rb,2.6583e+01_rb, &
     &3.1750e+01_rb,3.6527e+01_rb,4.2635e+01_rb/)
      kao(:, 1, 6,15) = (/ &
     &1.4684e-01_rb,4.8264e+00_rb,9.6500e+00_rb,1.4468e+01_rb,1.9279e+01_rb,2.4068e+01_rb, &
     &2.8816e+01_rb,3.3389e+01_rb,3.8558e+01_rb/)
      kao(:, 2, 6,15) = (/ &
     &1.4028e-01_rb,5.1303e+00_rb,1.0258e+01_rb,1.5380e+01_rb,2.0490e+01_rb,2.5585e+01_rb, &
     &3.0631e+01_rb,3.5492e+01_rb,4.0980e+01_rb/)
      kao(:, 3, 6,15) = (/ &
     &1.3401e-01_rb,5.4150e+00_rb,1.0825e+01_rb,1.6231e+01_rb,2.1624e+01_rb,2.7001e+01_rb, &
     &3.2322e+01_rb,3.7445e+01_rb,4.3249e+01_rb/)
      kao(:, 4, 6,15) = (/ &
     &1.2806e-01_rb,5.6771e+00_rb,1.1351e+01_rb,1.7018e+01_rb,2.2674e+01_rb,2.8310e+01_rb, &
     &3.3887e+01_rb,3.9255e+01_rb,4.5348e+01_rb/)
      kao(:, 5, 6,15) = (/ &
     &1.2235e-01_rb,5.9138e+00_rb,1.1824e+01_rb,1.7725e+01_rb,2.3617e+01_rb,2.9485e+01_rb, &
     &3.5301e+01_rb,4.0890e+01_rb,4.7234e+01_rb/)
      kao(:, 1, 7,15) = (/ &
     &3.0725e-01_rb,5.1535e+00_rb,1.0306e+01_rb,1.5454e+01_rb,2.0602e+01_rb,2.5736e+01_rb, &
     &3.0848e+01_rb,3.5878e+01_rb,4.1204e+01_rb/)
      kao(:, 2, 7,15) = (/ &
     &2.9321e-01_rb,5.5259e+00_rb,1.1050e+01_rb,1.6570e+01_rb,2.2088e+01_rb,2.7592e+01_rb, &
     &3.3073e+01_rb,3.8464e+01_rb,4.4176e+01_rb/)
      kao(:, 3, 7,15) = (/ &
     &2.7993e-01_rb,5.8735e+00_rb,1.1744e+01_rb,1.7612e+01_rb,2.3476e+01_rb,2.9329e+01_rb, &
     &3.5155e+01_rb,4.0875e+01_rb,4.6953e+01_rb/)
      kao(:, 4, 7,15) = (/ &
     &2.6791e-01_rb,6.1876e+00_rb,1.2373e+01_rb,1.8557e+01_rb,2.4734e+01_rb,3.0898e+01_rb, &
     &3.7039e+01_rb,4.3070e+01_rb,4.9468e+01_rb/)
      kao(:, 5, 7,15) = (/ &
     &2.5886e-01_rb,6.4685e+00_rb,1.2934e+01_rb,1.9398e+01_rb,2.5854e+01_rb,3.2302e+01_rb, &
     &3.8716e+01_rb,4.5022e+01_rb,5.1708e+01_rb/)
      kao(:, 1, 8,15) = (/ &
     &7.7279e-01_rb,5.4596e+00_rb,1.0918e+01_rb,1.6376e+01_rb,2.1832e+01_rb,2.7286e+01_rb, &
     &3.2729e+01_rb,3.8130e+01_rb,4.3663e+01_rb/)
      kao(:, 2, 8,15) = (/ &
     &7.3823e-01_rb,5.9033e+00_rb,1.1807e+01_rb,1.7707e+01_rb,2.3607e+01_rb,2.9503e+01_rb, &
     &3.5387e+01_rb,4.1229e+01_rb,4.7214e+01_rb/)
      kao(:, 3, 8,15) = (/ &
     &7.0825e-01_rb,6.3158e+00_rb,1.2632e+01_rb,1.8945e+01_rb,2.5259e+01_rb,3.1562e+01_rb, &
     &3.7862e+01_rb,4.4105e+01_rb,5.0517e+01_rb/)
      kao(:, 4, 8,15) = (/ &
     &6.8827e-01_rb,6.6883e+00_rb,1.3376e+01_rb,2.0062e+01_rb,2.6743e+01_rb,3.3422e+01_rb, &
     &4.0088e+01_rb,4.6704e+01_rb,5.3485e+01_rb/)
      kao(:, 5, 8,15) = (/ &
     &6.7674e-01_rb,7.0269e+00_rb,1.4054e+01_rb,2.1077e+01_rb,2.8099e+01_rb,3.5115e+01_rb, &
     &4.2117e+01_rb,4.9067e+01_rb,5.6197e+01_rb/)
      kao(:, 1, 9,15) = (/ &
     &3.6929e+00_rb,5.7245e+00_rb,1.1448e+01_rb,1.7170e+01_rb,2.2894e+01_rb,2.8616e+01_rb, &
     &3.4335e+01_rb,4.0048e+01_rb,4.5789e+01_rb/)
      kao(:, 2, 9,15) = (/ &
     &3.5483e+00_rb,6.2547e+00_rb,1.2509e+01_rb,1.8761e+01_rb,2.5016e+01_rb,3.1268e+01_rb, &
     &3.7517e+01_rb,4.3757e+01_rb,5.0032e+01_rb/)
      kao(:, 3, 9,15) = (/ &
     &3.4652e+00_rb,6.7430e+00_rb,1.3484e+01_rb,2.0224e+01_rb,2.6965e+01_rb,3.3705e+01_rb, &
     &4.0443e+01_rb,4.7167e+01_rb,5.3930e+01_rb/)
      kao(:, 4, 9,15) = (/ &
     &3.4260e+00_rb,7.1974e+00_rb,1.4393e+01_rb,2.1588e+01_rb,2.8784e+01_rb,3.5977e+01_rb, &
     &4.3167e+01_rb,5.0348e+01_rb,5.7567e+01_rb/)
      kao(:, 5, 9,15) = (/ &
     &3.4123e+00_rb,7.6134e+00_rb,1.5225e+01_rb,2.2838e+01_rb,3.0451e+01_rb,3.8059e+01_rb, &
     &4.5665e+01_rb,5.3256e+01_rb,6.0901e+01_rb/)
      kao(:, 1,10,15) = (/ &
     &1.9879e+01_rb,1.7395e+01_rb,1.5929e+01_rb,1.8251e+01_rb,2.3917e+01_rb,2.9895e+01_rb, &
     &3.5873e+01_rb,4.1851e+01_rb,4.7831e+01_rb/)
      kao(:, 2,10,15) = (/ &
     &1.9461e+01_rb,1.7089e+01_rb,1.6077e+01_rb,1.9823e+01_rb,2.6409e+01_rb,3.3015e+01_rb, &
     &3.9616e+01_rb,4.6210e+01_rb,5.2815e+01_rb/)
      kao(:, 3,10,15) = (/ &
     &1.9336e+01_rb,1.7101e+01_rb,1.6570e+01_rb,2.1574e+01_rb,2.8765e+01_rb,3.5957e+01_rb, &
     &4.3144e+01_rb,5.0330e+01_rb,5.7526e+01_rb/)
      kao(:, 4,10,15) = (/ &
     &1.9333e+01_rb,1.7178e+01_rb,1.7183e+01_rb,2.3203e+01_rb,3.0928e+01_rb,3.8662e+01_rb, &
     &4.6389e+01_rb,5.4117e+01_rb,6.1850e+01_rb/)
      kao(:, 5,10,15) = (/ &
     &1.9454e+01_rb,1.7376e+01_rb,1.7879e+01_rb,2.4671e+01_rb,3.2892e+01_rb,4.1116e+01_rb, &
     &4.9334e+01_rb,5.7555e+01_rb,6.5778e+01_rb/)
      kao(:, 1,11,15) = (/ &
     &3.7918e+01_rb,3.3177e+01_rb,2.9050e+01_rb,2.5728e+01_rb,2.6964e+01_rb,3.2638e+01_rb, &
     &3.9160e+01_rb,4.5686e+01_rb,5.2215e+01_rb/)
      kao(:, 2,11,15) = (/ &
     &3.7982e+01_rb,3.3235e+01_rb,2.9190e+01_rb,2.6661e+01_rb,2.9222e+01_rb,3.6220e+01_rb, &
     &4.3460e+01_rb,5.0704e+01_rb,5.7949e+01_rb/)
      kao(:, 3,11,15) = (/ &
     &3.8191e+01_rb,3.3419e+01_rb,2.9491e+01_rb,2.7784e+01_rb,3.1731e+01_rb,3.9527e+01_rb, &
     &4.7423e+01_rb,5.5330e+01_rb,6.3238e+01_rb/)
      kao(:, 4,11,15) = (/ &
     &3.8572e+01_rb,3.3751e+01_rb,2.9960e+01_rb,2.8966e+01_rb,3.4093e+01_rb,4.2514e+01_rb, &
     &5.1014e+01_rb,5.9516e+01_rb,6.8017e+01_rb/)
      kao(:, 5,11,15) = (/ &
     &3.9093e+01_rb,3.4206e+01_rb,3.0547e+01_rb,3.0263e+01_rb,3.6228e+01_rb,4.5245e+01_rb, &
     &5.4290e+01_rb,6.3320e+01_rb,7.2371e+01_rb/)
      kao(:, 1,12,15) = (/ &
     &5.5475e+01_rb,4.8541e+01_rb,4.2038e+01_rb,3.6129e+01_rb,3.3332e+01_rb,3.5714e+01_rb, &
     &4.2669e+01_rb,4.9771e+01_rb,5.6891e+01_rb/)
      kao(:, 2,12,15) = (/ &
     &5.6059e+01_rb,4.9052e+01_rb,4.2754e+01_rb,3.7078e+01_rb,3.5375e+01_rb,3.9558e+01_rb, &
     &4.7442e+01_rb,5.5347e+01_rb,6.3258e+01_rb/)
      kao(:, 3,12,15) = (/ &
     &5.6760e+01_rb,4.9664e+01_rb,4.3494e+01_rb,3.8346e+01_rb,3.7575e+01_rb,4.3186e+01_rb, &
     &5.1821e+01_rb,6.0453e+01_rb,6.9095e+01_rb/)
      kao(:, 4,12,15) = (/ &
     &5.7648e+01_rb,5.0443e+01_rb,4.4349e+01_rb,3.9649e+01_rb,3.9898e+01_rb,4.6561e+01_rb, &
     &5.5869e+01_rb,6.5177e+01_rb,7.4491e+01_rb/)
      kao(:, 5,12,15) = (/ &
     &5.8564e+01_rb,5.1244e+01_rb,4.5202e+01_rb,4.0926e+01_rb,4.2120e+01_rb,4.9601e+01_rb, &
     &5.9519e+01_rb,6.9433e+01_rb,7.9355e+01_rb/)
      kao(:, 1,13,15) = (/ &
     &6.5843e+01_rb,5.7608e+01_rb,4.9977e+01_rb,4.3124e+01_rb,3.8891e+01_rb,3.9628e+01_rb, &
     &4.6579e+01_rb,5.4339e+01_rb,6.2102e+01_rb/)
      kao(:, 2,13,15) = (/ &
     &6.6934e+01_rb,5.8564e+01_rb,5.1105e+01_rb,4.4471e+01_rb,4.1117e+01_rb,4.3582e+01_rb, &
     &5.1721e+01_rb,6.0334e+01_rb,6.8958e+01_rb/)
      kao(:, 3,13,15) = (/ &
     &6.8030e+01_rb,5.9532e+01_rb,5.2179e+01_rb,4.5978e+01_rb,4.3480e+01_rb,4.7374e+01_rb, &
     &5.6498e+01_rb,6.5896e+01_rb,7.5328e+01_rb/)
      kao(:, 4,13,15) = (/ &
     &6.9202e+01_rb,6.0551e+01_rb,5.3246e+01_rb,4.7483e+01_rb,4.5884e+01_rb,5.0923e+01_rb, &
     &6.0842e+01_rb,7.0980e+01_rb,8.1125e+01_rb/)
      kao(:, 5,13,15) = (/ &
     &7.0366e+01_rb,6.1567e+01_rb,5.4251e+01_rb,4.8884e+01_rb,4.8138e+01_rb,5.4098e+01_rb, &
     &6.4681e+01_rb,7.5454e+01_rb,8.6236e+01_rb/)
      kao(:, 1, 1,16) = (/ &
     &4.2680e-03_rb,4.0094e+00_rb,7.9109e+00_rb,1.1649e+01_rb,1.5123e+01_rb,1.8131e+01_rb, &
     &2.0185e+01_rb,1.9759e+01_rb,3.0246e+01_rb/)
      kao(:, 2, 1,16) = (/ &
     &4.1096e-03_rb,4.1769e+00_rb,8.2411e+00_rb,1.2134e+01_rb,1.5754e+01_rb,1.8883e+01_rb, &
     &2.1013e+01_rb,2.0555e+01_rb,3.1509e+01_rb/)
      kao(:, 3, 1,16) = (/ &
     &3.9552e-03_rb,4.3246e+00_rb,8.5324e+00_rb,1.2562e+01_rb,1.6307e+01_rb,1.9544e+01_rb, &
     &2.1743e+01_rb,2.1252e+01_rb,3.2613e+01_rb/)
      kao(:, 4, 1,16) = (/ &
     &3.8024e-03_rb,4.4537e+00_rb,8.7866e+00_rb,1.2936e+01_rb,1.6790e+01_rb,2.0122e+01_rb, &
     &2.2380e+01_rb,2.1858e+01_rb,3.3579e+01_rb/)
      kao(:, 5, 1,16) = (/ &
     &3.6530e-03_rb,4.5661e+00_rb,9.0077e+00_rb,1.3262e+01_rb,1.7211e+01_rb,2.0624e+01_rb, &
     &2.2936e+01_rb,2.2383e+01_rb,3.4420e+01_rb/)
      kao(:, 1, 2,16) = (/ &
     &7.0518e-03_rb,4.7486e+00_rb,9.4112e+00_rb,1.3945e+01_rb,1.8257e+01_rb,2.2167e+01_rb, &
     &2.5197e+01_rb,2.5646e+01_rb,3.6513e+01_rb/)
      kao(:, 2, 2,16) = (/ &
     &6.7731e-03_rb,4.9653e+00_rb,9.8411e+00_rb,1.4578e+01_rb,1.9087e+01_rb,2.3173e+01_rb, &
     &2.6338e+01_rb,2.6795e+01_rb,3.8174e+01_rb/)
      kao(:, 3, 2,16) = (/ &
     &6.4989e-03_rb,5.1561e+00_rb,1.0219e+01_rb,1.5138e+01_rb,1.9822e+01_rb,2.4059e+01_rb, &
     &2.7344e+01_rb,2.7802e+01_rb,3.9644e+01_rb/)
      kao(:, 4, 2,16) = (/ &
     &6.2358e-03_rb,5.3229e+00_rb,1.0548e+01_rb,1.5628e+01_rb,2.0461e+01_rb,2.4836e+01_rb, &
     &2.8225e+01_rb,2.8690e+01_rb,4.0921e+01_rb/)
      kao(:, 5, 2,16) = (/ &
     &5.9777e-03_rb,5.4686e+00_rb,1.0838e+01_rb,1.6055e+01_rb,2.1017e+01_rb,2.5512e+01_rb, &
     &2.8989e+01_rb,2.9460e+01_rb,4.2034e+01_rb/)
      kao(:, 1, 3,16) = (/ &
     &1.5841e-02_rb,5.5685e+00_rb,1.1089e+01_rb,1.6529e+01_rb,2.1842e+01_rb,2.6900e+01_rb, &
     &3.1365e+01_rb,3.3823e+01_rb,4.3684e+01_rb/)
      kao(:, 2, 3,16) = (/ &
     &1.5180e-02_rb,5.8535e+00_rb,1.1654e+01_rb,1.7376e+01_rb,2.2954e+01_rb,2.8269e+01_rb, &
     &3.2966e+01_rb,3.5548e+01_rb,4.5909e+01_rb/)
      kao(:, 3, 3,16) = (/ &
     &1.4542e-02_rb,6.1037e+00_rb,1.2154e+01_rb,1.8119e+01_rb,2.3939e+01_rb,2.9482e+01_rb, &
     &3.4375e+01_rb,3.7061e+01_rb,4.7877e+01_rb/)
      kao(:, 4, 3,16) = (/ &
     &1.3926e-02_rb,6.3236e+00_rb,1.2592e+01_rb,1.8771e+01_rb,2.4803e+01_rb,3.0545e+01_rb, &
     &3.5612e+01_rb,3.8393e+01_rb,4.9606e+01_rb/)
      kao(:, 5, 3,16) = (/ &
     &1.3332e-02_rb,6.5149e+00_rb,1.2971e+01_rb,1.9341e+01_rb,2.5553e+01_rb,3.1470e+01_rb, &
     &3.6691e+01_rb,3.9549e+01_rb,5.1106e+01_rb/)
      kao(:, 1, 4,16) = (/ &
     &3.6504e-02_rb,6.4847e+00_rb,1.2941e+01_rb,1.9357e+01_rb,2.5697e+01_rb,3.1893e+01_rb, &
     &3.7735e+01_rb,4.2276e+01_rb,5.1393e+01_rb/)
      kao(:, 2, 4,16) = (/ &
     &3.4931e-02_rb,6.8562e+00_rb,1.3684e+01_rb,2.0466e+01_rb,2.7170e+01_rb,3.3719e+01_rb, &
     &3.9896e+01_rb,4.4696e+01_rb,5.4340e+01_rb/)
      kao(:, 3, 4,16) = (/ &
     &3.3404e-02_rb,7.1832e+00_rb,1.4337e+01_rb,2.1441e+01_rb,2.8465e+01_rb,3.5334e+01_rb, &
     &4.1803e+01_rb,4.6838e+01_rb,5.6929e+01_rb/)
      kao(:, 4, 4,16) = (/ &
     &3.1940e-02_rb,7.4729e+00_rb,1.4911e+01_rb,2.2302e+01_rb,2.9607e+01_rb,3.6753e+01_rb, &
     &4.3482e+01_rb,4.8710e+01_rb,5.9213e+01_rb/)
      kao(:, 5, 4,16) = (/ &
     &3.0537e-02_rb,7.7225e+00_rb,1.5413e+01_rb,2.3052e+01_rb,3.0605e+01_rb,3.7982e+01_rb, &
     &4.4943e+01_rb,5.0367e+01_rb,6.1209e+01_rb/)
      kao(:, 1, 5,16) = (/ &
     &7.7459e-02_rb,7.5153e+00_rb,1.5015e+01_rb,2.2490e+01_rb,2.9920e+01_rb,3.7263e+01_rb, &
     &4.4388e+01_rb,5.0697e+01_rb,5.9839e+01_rb/)
      kao(:, 2, 5,16) = (/ &
     &7.3973e-02_rb,7.9956e+00_rb,1.5974e+01_rb,2.3926e+01_rb,3.1831e+01_rb,3.9653e+01_rb, &
     &4.7235e+01_rb,5.3936e+01_rb,6.3661e+01_rb/)
      kao(:, 3, 5,16) = (/ &
     &7.0652e-02_rb,8.4194e+00_rb,1.6824e+01_rb,2.5198e+01_rb,3.3521e+01_rb,4.1754e+01_rb, &
     &4.9742e+01_rb,5.6808e+01_rb,6.7041e+01_rb/)
      kao(:, 4, 5,16) = (/ &
     &6.7486e-02_rb,8.7920e+00_rb,1.7565e+01_rb,2.6310e+01_rb,3.5003e+01_rb,4.3595e+01_rb, &
     &5.1933e+01_rb,5.9317e+01_rb,7.0006e+01_rb/)
      kao(:, 5, 5,16) = (/ &
     &6.4478e-02_rb,9.1159e+00_rb,1.8215e+01_rb,2.7284e+01_rb,3.6299e+01_rb,4.5208e+01_rb, &
     &5.3851e+01_rb,6.1525e+01_rb,7.2598e+01_rb/)
      kao(:, 1, 6,16) = (/ &
     &1.5475e-01_rb,8.6422e+00_rb,1.7275e+01_rb,2.5892e+01_rb,3.4482e+01_rb,4.3028e+01_rb, &
     &5.1423e+01_rb,5.9299e+01_rb,6.8964e+01_rb/)
      kao(:, 2, 6,16) = (/ &
     &1.4760e-01_rb,9.2603e+00_rb,1.8510e+01_rb,2.7740e+01_rb,3.6952e+01_rb,4.6093e+01_rb, &
     &5.5105e+01_rb,6.3549e+01_rb,7.3905e+01_rb/)
      kao(:, 3, 6,16) = (/ &
     &1.4090e-01_rb,9.8052e+00_rb,1.9599e+01_rb,2.9376e+01_rb,3.9123e+01_rb,4.8807e+01_rb, &
     &5.8340e+01_rb,6.7288e+01_rb,7.8246e+01_rb/)
      kao(:, 4, 6,16) = (/ &
     &1.3455e-01_rb,1.0284e+01_rb,2.0553e+01_rb,3.0809e+01_rb,4.1029e+01_rb,5.1188e+01_rb, &
     &6.1191e+01_rb,7.0573e+01_rb,8.2057e+01_rb/)
      kao(:, 5, 6,16) = (/ &
     &1.2855e-01_rb,1.0696e+01_rb,2.1381e+01_rb,3.2059e+01_rb,4.2695e+01_rb,5.3257e+01_rb, &
     &6.3651e+01_rb,7.3440e+01_rb,8.5390e+01_rb/)
      kao(:, 1, 7,16) = (/ &
     &3.2765e-01_rb,9.8708e+00_rb,1.9738e+01_rb,2.9598e+01_rb,3.9436e+01_rb,4.9244e+01_rb, &
     &5.8981e+01_rb,6.8395e+01_rb,7.8872e+01_rb/)
      kao(:, 2, 7,16) = (/ &
     &3.1237e-01_rb,1.0655e+01_rb,2.1308e+01_rb,3.1948e+01_rb,4.2573e+01_rb,5.3154e+01_rb, &
     &6.3663e+01_rb,7.3848e+01_rb,8.5146e+01_rb/)
      kao(:, 3, 7,16) = (/ &
     &2.9817e-01_rb,1.1348e+01_rb,2.2690e+01_rb,3.4027e+01_rb,4.5330e+01_rb,5.6613e+01_rb, &
     &6.7816e+01_rb,7.8654e+01_rb,9.0660e+01_rb/)
      kao(:, 4, 7,16) = (/ &
     &2.8488e-01_rb,1.1956e+01_rb,2.3907e+01_rb,3.5845e+01_rb,4.7755e+01_rb,5.9649e+01_rb, &
     &7.1415e+01_rb,8.2866e+01_rb,9.5510e+01_rb/)
      kao(:, 5, 7,16) = (/ &
     &2.7283e-01_rb,1.2480e+01_rb,2.4950e+01_rb,3.7415e+01_rb,4.9097e+01_rb,6.2266e+01_rb, &
     &7.4578e+01_rb,8.6491e+01_rb,9.9702e+01_rb/)
      kao(:, 1, 8,16) = (/ &
     &8.3606e-01_rb,1.1195e+01_rb,2.2388e+01_rb,3.3574e+01_rb,4.4755e+01_rb,5.5920e+01_rb, &
     &6.7048e+01_rb,7.8029e+01_rb,8.9508e+01_rb/)
      kao(:, 2, 8,16) = (/ &
     &7.9798e-01_rb,1.2177e+01_rb,2.4351e+01_rb,3.6522e+01_rb,4.8685e+01_rb,6.0830e+01_rb, &
     &7.2963e+01_rb,8.4892e+01_rb,9.7369e+01_rb/)
      kao(:, 3, 8,16) = (/ &
     &7.6304e-01_rb,1.3047e+01_rb,2.6090e+01_rb,3.9130e+01_rb,5.2156e+01_rb,6.5163e+01_rb, &
     &7.8140e+01_rb,9.0945e+01_rb,1.0431e+02_rb/)
      kao(:, 4, 8,16) = (/ &
     &7.3296e-01_rb,1.3800e+01_rb,2.7601e+01_rb,4.1393e+01_rb,5.5174e+01_rb,6.8947e+01_rb, &
     &8.2670e+01_rb,9.6216e+01_rb,1.1035e+02_rb/)
      kao(:, 5, 8,16) = (/ &
     &7.2250e-01_rb,1.4453e+01_rb,2.8899e+01_rb,4.3348e+01_rb,5.7784e+01_rb,7.2193e+01_rb, &
     &8.6582e+01_rb,1.0076e+02_rb,1.1557e+02_rb/)
      kao(:, 1, 9,16) = (/ &
     &4.0611e+00_rb,1.2573e+01_rb,2.5149e+01_rb,3.7721e+01_rb,5.0293e+01_rb,6.2854e+01_rb, &
     &7.5422e+01_rb,8.7942e+01_rb,1.0059e+02_rb/)
      kao(:, 2, 9,16) = (/ &
     &3.8919e+00_rb,1.3793e+01_rb,2.7583e+01_rb,4.1373e+01_rb,5.5157e+01_rb,6.8943e+01_rb, &
     &8.2722e+01_rb,9.6458e+01_rb,1.1031e+02_rb/)
      kao(:, 3, 9,16) = (/ &
     &3.7505e+00_rb,1.4863e+01_rb,2.9722e+01_rb,4.4587e+01_rb,5.9447e+01_rb,7.4303e+01_rb, &
     &8.9145e+01_rb,1.0395e+02_rb,1.1889e+02_rb/)
      kao(:, 4, 9,16) = (/ &
     &3.6915e+00_rb,1.5793e+01_rb,3.1587e+01_rb,4.7379e+01_rb,6.3167e+01_rb,7.8955e+01_rb, &
     &9.4727e+01_rb,1.1046e+02_rb,1.2634e+02_rb/)
      kao(:, 5, 9,16) = (/ &
     &3.8118e+00_rb,1.6590e+01_rb,3.3180e+01_rb,4.9767e+01_rb,6.6364e+01_rb,8.2935e+01_rb, &
     &9.9506e+01_rb,1.1603e+02_rb,1.3273e+02_rb/)
      kao(:, 1,10,16) = (/ &
     &2.2252e+01_rb,1.9469e+01_rb,2.8181e+01_rb,4.2271e+01_rb,5.6359e+01_rb,7.0451e+01_rb, &
     &8.4538e+01_rb,9.8612e+01_rb,1.1272e+02_rb/)
      kao(:, 2,10,16) = (/ &
     &2.1525e+01_rb,1.8834e+01_rb,3.1126e+01_rb,4.6690e+01_rb,6.2252e+01_rb,7.7813e+01_rb, &
     &9.3373e+01_rb,1.0893e+02_rb,1.2450e+02_rb/)
      kao(:, 3,10,16) = (/ &
     &2.1146e+01_rb,1.8632e+01_rb,3.3699e+01_rb,5.0550e+01_rb,6.7413e+01_rb,8.4265e+01_rb, &
     &1.0112e+02_rb,1.1795e+02_rb,1.3483e+02_rb/)
      kao(:, 4,10,16) = (/ &
     &2.1709e+01_rb,1.9529e+01_rb,3.5926e+01_rb,5.3893e+01_rb,7.1851e+01_rb,8.9812e+01_rb, &
     &1.0777e+02_rb,1.2572e+02_rb,1.4370e+02_rb/)
      kao(:, 5,10,16) = (/ &
     &2.2565e+01_rb,2.0486e+01_rb,3.7826e+01_rb,5.6738e+01_rb,7.5652e+01_rb,9.4560e+01_rb, &
     &1.1347e+02_rb,1.3237e+02_rb,1.5131e+02_rb/)
      kao(:, 1,11,16) = (/ &
     &4.2772e+01_rb,3.7423e+01_rb,3.4479e+01_rb,4.9359e+01_rb,6.5807e+01_rb,8.2260e+01_rb, &
     &9.8714e+01_rb,1.1516e+02_rb,1.3161e+02_rb/)
      kao(:, 2,11,16) = (/ &
     &4.2480e+01_rb,3.7168e+01_rb,3.6886e+01_rb,5.4309e+01_rb,7.2413e+01_rb,9.0516e+01_rb, &
     &1.0859e+02_rb,1.2671e+02_rb,1.4483e+02_rb/)
      kao(:, 3,11,16) = (/ &
     &4.3695e+01_rb,3.8234e+01_rb,3.9646e+01_rb,5.8582e+01_rb,7.8105e+01_rb,9.7632e+01_rb, &
     &1.1715e+02_rb,1.3667e+02_rb,1.5621e+02_rb/)
      kao(:, 4,11,16) = (/ &
     &4.5497e+01_rb,3.9810e+01_rb,4.2112e+01_rb,6.2236e+01_rb,8.2974e+01_rb,1.0371e+02_rb, &
     &1.2447e+02_rb,1.4520e+02_rb,1.6595e+02_rb/)
      kao(:, 5,11,16) = (/ &
     &4.7319e+01_rb,4.1402e+01_rb,4.4245e+01_rb,6.5288e+01_rb,8.7037e+01_rb,1.0880e+02_rb, &
     &1.3053e+02_rb,1.5228e+02_rb,1.7407e+02_rb/)
      kao(:, 1,12,16) = (/ &
     &6.3760e+01_rb,5.5793e+01_rb,4.7823e+01_rb,5.6928e+01_rb,7.5904e+01_rb,9.4850e+01_rb, &
     &1.1385e+02_rb,1.3279e+02_rb,1.5181e+02_rb/)
      kao(:, 2,12,16) = (/ &
     &6.5427e+01_rb,5.7247e+01_rb,4.9649e+01_rb,6.2349e+01_rb,8.3126e+01_rb,1.0392e+02_rb, &
     &1.2469e+02_rb,1.4546e+02_rb,1.6625e+02_rb/)
      kao(:, 3,12,16) = (/ &
     &6.8202e+01_rb,5.9674e+01_rb,5.2535e+01_rb,6.6975e+01_rb,8.9301e+01_rb,1.1163e+02_rb, &
     &1.3393e+02_rb,1.5627e+02_rb,1.7860e+02_rb/)
      kao(:, 4,12,16) = (/ &
     &7.0922e+01_rb,6.2055e+01_rb,5.5321e+01_rb,7.0905e+01_rb,9.4539e+01_rb,1.1818e+02_rb, &
     &1.4181e+02_rb,1.6544e+02_rb,1.8908e+02_rb/)
      kao(:, 5,12,16) = (/ &
     &7.3649e+01_rb,6.4442e+01_rb,5.7813e+01_rb,7.4307e+01_rb,9.9092e+01_rb,1.2384e+02_rb, &
     &1.4863e+02_rb,1.7339e+02_rb,1.9818e+02_rb/)
      kao(:, 1,13,16) = (/ &
     &7.8782e+01_rb,6.8941e+01_rb,5.9088e+01_rb,6.5861e+01_rb,8.6369e+01_rb,1.0796e+02_rb, &
     &1.2954e+02_rb,1.5113e+02_rb,1.7274e+02_rb/)
      kao(:, 2,13,16) = (/ &
     &8.1731e+01_rb,7.1509e+01_rb,6.1634e+01_rb,7.1635e+01_rb,9.4482e+01_rb,1.1810e+02_rb, &
     &1.4170e+02_rb,1.6534e+02_rb,1.8896e+02_rb/)
      kao(:, 3,13,16) = (/ &
     &8.5240e+01_rb,7.4588e+01_rb,6.4915e+01_rb,7.6648e+01_rb,1.0154e+02_rb,1.2693e+02_rb, &
     &1.5232e+02_rb,1.7766e+02_rb,2.0308e+02_rb/)
      kao(:, 4,13,16) = (/ &
     &8.8622e+01_rb,7.7543e+01_rb,6.7967e+01_rb,8.0867e+01_rb,1.0759e+02_rb,1.3449e+02_rb, &
     &1.6139e+02_rb,1.8828e+02_rb,2.1519e+02_rb/)
      kao(:, 5,13,16) = (/ &
     &9.1737e+01_rb,8.0288e+01_rb,7.0891e+01_rb,8.4742e+01_rb,1.1298e+02_rb,1.4117e+02_rb, &
     &1.6939e+02_rb,1.9762e+02_rb,2.2595e+02_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &1.4739e-04_rb,3.1686e-04_rb,8.5973e-04_rb,1.9039e-03_rb,3.1820e-03_rb,3.6596e-03_rb, &
     &3.8724e-03_rb,3.6785e-03_rb,3.7141e-03_rb,3.7646e-03_rb,4.2955e-03_rb,4.6343e-03_rb, &
     &5.0612e-03_rb,4.0227e-03_rb,4.2966e-03_rb,4.6622e-03_rb/)
      forrefo(2,:) = (/ &
     &1.9397e-04_rb,3.6322e-04_rb,8.9797e-04_rb,2.1001e-03_rb,3.0307e-03_rb,3.5563e-03_rb, &
     &3.8498e-03_rb,3.5741e-03_rb,3.5914e-03_rb,3.7658e-03_rb,3.8895e-03_rb,4.4072e-03_rb, &
     &4.7112e-03_rb,4.2230e-03_rb,4.2666e-03_rb,4.6634e-03_rb/)
      forrefo(3,:) = (/ &
     &3.1506e-04_rb,7.3687e-04_rb,1.9678e-03_rb,2.5531e-03_rb,2.8345e-03_rb,2.7809e-03_rb, &
     &2.9124e-03_rb,2.7125e-03_rb,2.6644e-03_rb,2.4907e-03_rb,2.7032e-03_rb,4.0967e-03_rb, &
     &4.1971e-03_rb,4.4507e-03_rb,4.2293e-03_rb,4.6633e-03_rb/)
      forrefo(4,:) = (/ &
     &8.8196e-04_rb,2.1125e-03_rb,2.8042e-03_rb,2.8891e-03_rb,2.4362e-03_rb,1.8733e-03_rb, &
     &1.4078e-03_rb,1.1987e-03_rb,1.2808e-03_rb,8.9050e-04_rb,9.4375e-04_rb,7.8351e-04_rb, &
     &1.0756e-03_rb,1.6586e-03_rb,1.7511e-03_rb,4.7803e-03_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 2.37879e-02_rb, 2.10719e-02_rb, 1.86660e-02_rb, 1.65348e-02_rb, 1.46469e-02_rb, &
     & 1.29746e-02_rb, 1.14932e-02_rb, 1.01810e-02_rb, 9.01858e-03_rb, 7.98888e-03_rb/)
      selfrefo(:, 2) = (/ &
     & 3.10625e-02_rb, 2.82664e-02_rb, 2.57220e-02_rb, 2.34066e-02_rb, 2.12997e-02_rb, &
     & 1.93824e-02_rb, 1.76377e-02_rb, 1.60500e-02_rb, 1.46053e-02_rb, 1.32906e-02_rb/)
      selfrefo(:, 3) = (/ &
     & 5.19103e-02_rb, 4.80004e-02_rb, 4.43850e-02_rb, 4.10419e-02_rb, 3.79506e-02_rb, &
     & 3.50922e-02_rb, 3.24491e-02_rb, 3.00050e-02_rb, 2.77450e-02_rb, 2.56553e-02_rb/)
      selfrefo(:, 4) = (/ &
     & 9.12444e-02_rb, 8.38675e-02_rb, 7.70870e-02_rb, 7.08547e-02_rb, 6.51263e-02_rb, &
     & 5.98610e-02_rb, 5.50214e-02_rb, 5.05730e-02_rb, 4.64843e-02_rb, 4.27262e-02_rb/)
      selfrefo(:, 5) = (/ &
     & 1.11323e-01_rb, 1.04217e-01_rb, 9.75650e-02_rb, 9.13376e-02_rb, 8.55076e-02_rb, &
     & 8.00498e-02_rb, 7.49403e-02_rb, 7.01570e-02_rb, 6.56790e-02_rb, 6.14868e-02_rb/)
      selfrefo(:, 6) = (/ &
     & 1.25301e-01_rb, 1.16877e-01_rb, 1.09020e-01_rb, 1.01691e-01_rb, 9.48543e-02_rb, &
     & 8.84774e-02_rb, 8.25293e-02_rb, 7.69810e-02_rb, 7.18057e-02_rb, 6.69784e-02_rb/)
      selfrefo(:, 7) = (/ &
     & 1.34063e-01_rb, 1.24662e-01_rb, 1.15920e-01_rb, 1.07791e-01_rb, 1.00232e-01_rb, &
     & 9.32035e-02_rb, 8.66676e-02_rb, 8.05900e-02_rb, 7.49386e-02_rb, 6.96836e-02_rb/)
      selfrefo(:, 8) = (/ &
     & 1.26997e-01_rb, 1.18306e-01_rb, 1.10210e-01_rb, 1.02668e-01_rb, 9.56417e-02_rb, &
     & 8.90964e-02_rb, 8.29991e-02_rb, 7.73190e-02_rb, 7.20276e-02_rb, 6.70984e-02_rb/)
      selfrefo(:, 9) = (/ &
     & 1.28823e-01_rb, 1.20235e-01_rb, 1.12220e-01_rb, 1.04739e-01_rb, 9.77569e-02_rb, &
     & 9.12402e-02_rb, 8.51579e-02_rb, 7.94810e-02_rb, 7.41826e-02_rb, 6.92374e-02_rb/)
      selfrefo(:,10) = (/ &
     & 1.35802e-01_rb, 1.25981e-01_rb, 1.16870e-01_rb, 1.08418e-01_rb, 1.00577e-01_rb, &
     & 9.33034e-02_rb, 8.65557e-02_rb, 8.02960e-02_rb, 7.44890e-02_rb, 6.91020e-02_rb/)
      selfrefo(:,11) = (/ &
     & 1.35475e-01_rb, 1.27572e-01_rb, 1.20130e-01_rb, 1.13122e-01_rb, 1.06523e-01_rb, &
     & 1.00309e-01_rb, 9.44573e-02_rb, 8.89470e-02_rb, 8.37582e-02_rb, 7.88721e-02_rb/)
      selfrefo(:,12) = (/ &
     & 1.51195e-01_rb, 1.41159e-01_rb, 1.31790e-01_rb, 1.23043e-01_rb, 1.14876e-01_rb, &
     & 1.07251e-01_rb, 1.00132e-01_rb, 9.34860e-02_rb, 8.72809e-02_rb, 8.14877e-02_rb/)
      selfrefo(:,13) = (/ &
     & 1.57538e-01_rb, 1.47974e-01_rb, 1.38990e-01_rb, 1.30552e-01_rb, 1.22626e-01_rb, &
     & 1.15181e-01_rb, 1.08188e-01_rb, 1.01620e-01_rb, 9.54505e-02_rb, 8.96556e-02_rb/)
      selfrefo(:,14) = (/ &
     & 1.53567e-01_rb, 1.41564e-01_rb, 1.30500e-01_rb, 1.20300e-01_rb, 1.10898e-01_rb, &
     & 1.02231e-01_rb, 9.42406e-02_rb, 8.68750e-02_rb, 8.00851e-02_rb, 7.38259e-02_rb/)
      selfrefo(:,15) = (/ &
     & 1.53687e-01_rb, 1.42981e-01_rb, 1.33020e-01_rb, 1.23753e-01_rb, 1.15132e-01_rb, &
     & 1.07112e-01_rb, 9.96500e-02_rb, 9.27080e-02_rb, 8.62496e-02_rb, 8.02412e-02_rb/)
      selfrefo(:,16) = (/ &
     & 1.65129e-01_rb, 1.53285e-01_rb, 1.42290e-01_rb, 1.32084e-01_rb, 1.22610e-01_rb, &
     & 1.13815e-01_rb, 1.05651e-01_rb, 9.80730e-02_rb, 9.10384e-02_rb, 8.45083e-02_rb/)

      end subroutine lw_kgb12

! **************************************************************************
      subroutine lw_kgb13
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg13, only : fracrefao, fracrefbo, kao, kao_mco2, kao_mco, &
                            kbo_mo3, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P=473.4280 mb, T = 259.83 K      
      fracrefao(:, 1) = (/ &
     &  1.7534e-01_rb,1.7394e-01_rb,1.6089e-01_rb,1.3782e-01_rb,1.0696e-01_rb,8.5853e-02_rb, &
     &  6.6548e-02_rb,4.9053e-02_rb,3.2064e-02_rb,3.4820e-03_rb,2.8763e-03_rb,2.2204e-03_rb, &
     &  1.5612e-03_rb,9.8572e-04_rb,3.6853e-04_rb,5.1612e-05_rb/)
      fracrefao(:, 2) = (/ &
     &  1.7489e-01_rb,1.7309e-01_rb,1.5981e-01_rb,1.3782e-01_rb,1.0797e-01_rb,8.6367e-02_rb, &
     &  6.7042e-02_rb,4.9257e-02_rb,3.2207e-02_rb,3.4820e-03_rb,2.8767e-03_rb,2.2203e-03_rb, &
     &  1.5613e-03_rb,9.8571e-04_rb,3.6853e-04_rb,5.1612e-05_rb/)
      fracrefao(:, 3) = (/ &
     &  1.7459e-01_rb,1.7259e-01_rb,1.5948e-01_rb,1.3694e-01_rb,1.0815e-01_rb,8.7376e-02_rb, &
     &  6.7339e-02_rb,4.9541e-02_rb,3.2333e-02_rb,3.5019e-03_rb,2.8958e-03_rb,2.2527e-03_rb, &
     &  1.6099e-03_rb,9.8574e-04_rb,3.6853e-04_rb,5.1612e-05_rb/)
      fracrefao(:, 4) = (/ &
     &  1.7391e-01_rb,1.7244e-01_rb,1.5921e-01_rb,1.3644e-01_rb,1.0787e-01_rb,8.7776e-02_rb, &
     &  6.8361e-02_rb,4.9628e-02_rb,3.2578e-02_rb,3.5117e-03_rb,2.9064e-03_rb,2.2571e-03_rb, &
     &  1.6887e-03_rb,1.0045e-03_rb,3.6853e-04_rb,5.1612e-05_rb/)
      fracrefao(:, 5) = (/ &
     &  1.7338e-01_rb,1.7157e-01_rb,1.5957e-01_rb,1.3571e-01_rb,1.0773e-01_rb,8.7966e-02_rb, &
     &  6.9000e-02_rb,5.0300e-02_rb,3.2813e-02_rb,3.5470e-03_rb,2.9425e-03_rb,2.2552e-03_rb, &
     &  1.7038e-03_rb,1.1025e-03_rb,3.6853e-04_rb,5.1612e-05_rb/)
      fracrefao(:, 6) = (/ &
     &  1.7230e-01_rb,1.7082e-01_rb,1.5917e-01_rb,1.3562e-01_rb,1.0806e-01_rb,8.7635e-02_rb, &
     &  6.9815e-02_rb,5.1155e-02_rb,3.3139e-02_rb,3.6264e-03_rb,2.9436e-03_rb,2.3417e-03_rb, &
     &  1.7731e-03_rb,1.1156e-03_rb,4.4533e-04_rb,5.1612e-05_rb/)
      fracrefao(:, 7) = (/ &
     &  1.7073e-01_rb,1.6961e-01_rb,1.5844e-01_rb,1.3594e-01_rb,1.0821e-01_rb,8.7791e-02_rb, &
     &  7.0502e-02_rb,5.1904e-02_rb,3.4107e-02_rb,3.5888e-03_rb,2.9574e-03_rb,2.5851e-03_rb, &
     &  1.9127e-03_rb,1.1537e-03_rb,4.7789e-04_rb,1.0016e-04_rb/)
      fracrefao(:, 8) = (/ &
     &  1.6700e-01_rb,1.6848e-01_rb,1.5628e-01_rb,1.3448e-01_rb,1.1011e-01_rb,8.9016e-02_rb, &
     &  7.1973e-02_rb,5.2798e-02_rb,3.5650e-02_rb,3.8534e-03_rb,3.4142e-03_rb,2.7799e-03_rb, &
     &  2.1288e-03_rb,1.3043e-03_rb,6.2858e-04_rb,1.0016e-04_rb/)
      fracrefao(:, 9) = (/ &
     &  1.6338e-01_rb,1.5565e-01_rb,1.4470e-01_rb,1.3500e-01_rb,1.1909e-01_rb,9.8312e-02_rb, &
     &  7.9023e-02_rb,5.5728e-02_rb,3.6831e-02_rb,3.6569e-03_rb,3.0552e-03_rb,2.3431e-03_rb, &
     &  1.7088e-03_rb,1.1082e-03_rb,3.6829e-04_rb,5.1612e-05_rb/)

! Planck fraction mapping level : P=4.758820 mb, T = 250.85 K
      fracrefbo(:) = (/ &
     &  1.5411e-01_rb,1.3573e-01_rb,1.2527e-01_rb,1.2698e-01_rb,1.2394e-01_rb,1.0876e-01_rb, &
     &  8.9906e-02_rb,6.9551e-02_rb,4.8240e-02_rb,5.2434e-03_rb,4.3630e-03_rb,3.4262e-03_rb, &
     &  2.5124e-03_rb,1.5479e-03_rb,3.7294e-04_rb,5.1050e-05_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &6.4065e-07_rb,1.0419e-05_rb,1.7041e-05_rb,2.3341e-05_rb,2.9288e-05_rb,3.6860e-05_rb, &
     &5.2289e-05_rb,7.6369e-05_rb,2.2034e-05_rb/)
      kao(:, 2, 1, 1) = (/ &
     &7.4425e-07_rb,1.2994e-05_rb,2.1634e-05_rb,2.9437e-05_rb,3.6128e-05_rb,4.4577e-05_rb, &
     &5.5507e-05_rb,7.3010e-05_rb,3.0368e-05_rb/)
      kao(:, 3, 1, 1) = (/ &
     &9.1477e-07_rb,1.6195e-05_rb,2.7037e-05_rb,3.6851e-05_rb,4.4956e-05_rb,5.3696e-05_rb, &
     &6.5423e-05_rb,7.7959e-05_rb,3.9940e-05_rb/)
      kao(:, 4, 1, 1) = (/ &
     &1.0920e-06_rb,2.0021e-05_rb,3.3475e-05_rb,4.5528e-05_rb,5.5351e-05_rb,6.4790e-05_rb, &
     &7.8460e-05_rb,9.0393e-05_rb,5.1295e-05_rb/)
      kao(:, 5, 1, 1) = (/ &
     &1.3489e-06_rb,2.4363e-05_rb,4.1025e-05_rb,5.5638e-05_rb,6.7646e-05_rb,7.8828e-05_rb, &
     &9.2175e-05_rb,1.0262e-04_rb,6.4740e-05_rb/)
      kao(:, 1, 2, 1) = (/ &
     &9.4474e-07_rb,9.5739e-06_rb,1.4537e-05_rb,1.9319e-05_rb,2.3955e-05_rb,2.8125e-05_rb, &
     &3.4867e-05_rb,5.4010e-05_rb,1.9403e-05_rb/)
      kao(:, 2, 2, 1) = (/ &
     &1.1323e-06_rb,1.1739e-05_rb,1.8325e-05_rb,2.4542e-05_rb,3.0201e-05_rb,3.5031e-05_rb, &
     &4.2528e-05_rb,5.9167e-05_rb,2.5395e-05_rb/)
      kao(:, 3, 2, 1) = (/ &
     &1.3451e-06_rb,1.4450e-05_rb,2.3075e-05_rb,3.0847e-05_rb,3.7507e-05_rb,4.3224e-05_rb, &
     &5.0526e-05_rb,6.6079e-05_rb,3.2769e-05_rb/)
      kao(:, 4, 2, 1) = (/ &
     &1.6171e-06_rb,1.7819e-05_rb,2.8678e-05_rb,3.8333e-05_rb,4.6715e-05_rb,5.3613e-05_rb, &
     &6.0668e-05_rb,7.1518e-05_rb,4.1642e-05_rb/)
      kao(:, 5, 2, 1) = (/ &
     &1.9441e-06_rb,2.1692e-05_rb,3.5243e-05_rb,4.7173e-05_rb,5.7398e-05_rb,6.5939e-05_rb, &
     &7.3567e-05_rb,8.2891e-05_rb,5.2140e-05_rb/)
      kao(:, 1, 3, 1) = (/ &
     &1.9997e-06_rb,1.0333e-05_rb,1.4351e-05_rb,1.7071e-05_rb,1.9818e-05_rb,2.2671e-05_rb, &
     &2.5587e-05_rb,3.3080e-05_rb,2.1440e-05_rb/)
      kao(:, 2, 3, 1) = (/ &
     &2.2813e-06_rb,1.2534e-05_rb,1.7541e-05_rb,2.1448e-05_rb,2.5145e-05_rb,2.8879e-05_rb, &
     &3.1889e-05_rb,3.7831e-05_rb,2.6738e-05_rb/)
      kao(:, 3, 3, 1) = (/ &
     &2.7401e-06_rb,1.5206e-05_rb,2.1444e-05_rb,2.7039e-05_rb,3.1929e-05_rb,3.6455e-05_rb, &
     &3.9620e-05_rb,4.6057e-05_rb,3.1366e-05_rb/)
      kao(:, 4, 3, 1) = (/ &
     &3.2871e-06_rb,1.8031e-05_rb,2.6416e-05_rb,3.3815e-05_rb,3.9999e-05_rb,4.5410e-05_rb, &
     &4.9342e-05_rb,5.4203e-05_rb,3.7064e-05_rb/)
      kao(:, 5, 3, 1) = (/ &
     &3.8869e-06_rb,2.1455e-05_rb,3.2368e-05_rb,4.1756e-05_rb,4.9581e-05_rb,5.6093e-05_rb, &
     &6.1027e-05_rb,6.5117e-05_rb,4.4920e-05_rb/)
      kao(:, 1, 4, 1) = (/ &
     &4.3610e-06_rb,1.3316e-05_rb,1.6242e-05_rb,1.8257e-05_rb,1.9350e-05_rb,2.0037e-05_rb, &
     &2.0833e-05_rb,2.3040e-05_rb,2.2366e-05_rb/)
      kao(:, 2, 4, 1) = (/ &
     &5.0348e-06_rb,1.5693e-05_rb,1.9649e-05_rb,2.2560e-05_rb,2.3970e-05_rb,2.5222e-05_rb, &
     &2.6081e-05_rb,2.7399e-05_rb,2.7623e-05_rb/)
      kao(:, 3, 4, 1) = (/ &
     &5.7893e-06_rb,1.8717e-05_rb,2.3542e-05_rb,2.7248e-05_rb,2.9677e-05_rb,3.1779e-05_rb, &
     &3.2973e-05_rb,3.3876e-05_rb,3.4347e-05_rb/)
      kao(:, 4, 4, 1) = (/ &
     &6.7700e-06_rb,2.2355e-05_rb,2.8015e-05_rb,3.2751e-05_rb,3.6706e-05_rb,3.9566e-05_rb, &
     &4.1488e-05_rb,4.1986e-05_rb,4.0984e-05_rb/)
      kao(:, 5, 4, 1) = (/ &
     &8.0207e-06_rb,2.6505e-05_rb,3.3421e-05_rb,3.9357e-05_rb,4.5121e-05_rb,4.9003e-05_rb, &
     &5.1776e-05_rb,5.2148e-05_rb,4.7085e-05_rb/)
      kao(:, 1, 5, 1) = (/ &
     &8.6602e-06_rb,1.8553e-05_rb,2.0839e-05_rb,2.1788e-05_rb,2.1544e-05_rb,2.1077e-05_rb, &
     &1.9473e-05_rb,1.8495e-05_rb,2.3671e-05_rb/)
      kao(:, 2, 5, 1) = (/ &
     &9.9810e-06_rb,2.2031e-05_rb,2.4930e-05_rb,2.6247e-05_rb,2.6462e-05_rb,2.6137e-05_rb, &
     &2.4415e-05_rb,2.2553e-05_rb,2.8177e-05_rb/)
      kao(:, 3, 5, 1) = (/ &
     &1.1619e-05_rb,2.6356e-05_rb,2.9247e-05_rb,3.1562e-05_rb,3.1911e-05_rb,3.1801e-05_rb, &
     &3.0166e-05_rb,2.7978e-05_rb,3.2481e-05_rb/)
      kao(:, 4, 5, 1) = (/ &
     &1.3216e-05_rb,3.1104e-05_rb,3.4868e-05_rb,3.7332e-05_rb,3.8497e-05_rb,3.8279e-05_rb, &
     &3.7615e-05_rb,3.4834e-05_rb,3.8514e-05_rb/)
      kao(:, 5, 5, 1) = (/ &
     &1.5414e-05_rb,3.5706e-05_rb,4.1155e-05_rb,4.4051e-05_rb,4.6023e-05_rb,4.6762e-05_rb, &
     &4.6430e-05_rb,4.3490e-05_rb,4.6117e-05_rb/)
      kao(:, 1, 6, 1) = (/ &
     &1.6458e-05_rb,2.5684e-05_rb,2.8012e-05_rb,2.7844e-05_rb,2.6244e-05_rb,2.4237e-05_rb, &
     &2.0656e-05_rb,1.6798e-05_rb,2.3796e-05_rb/)
      kao(:, 2, 6, 1) = (/ &
     &1.8595e-05_rb,3.0385e-05_rb,3.3443e-05_rb,3.3768e-05_rb,3.2056e-05_rb,2.9483e-05_rb, &
     &2.5851e-05_rb,2.0866e-05_rb,2.8662e-05_rb/)
      kao(:, 3, 6, 1) = (/ &
     &2.1348e-05_rb,3.6540e-05_rb,3.9601e-05_rb,3.9640e-05_rb,3.8210e-05_rb,3.5898e-05_rb, &
     &3.1968e-05_rb,2.5765e-05_rb,3.3968e-05_rb/)
      kao(:, 4, 6, 1) = (/ &
     &2.4741e-05_rb,4.2510e-05_rb,4.6979e-05_rb,4.6425e-05_rb,4.6165e-05_rb,4.2824e-05_rb, &
     &3.8814e-05_rb,3.1863e-05_rb,3.8072e-05_rb/)
      kao(:, 5, 6, 1) = (/ &
     &2.8527e-05_rb,5.0183e-05_rb,5.5064e-05_rb,5.5234e-05_rb,5.4182e-05_rb,5.0966e-05_rb, &
     &4.6758e-05_rb,3.9447e-05_rb,4.4935e-05_rb/)
      kao(:, 1, 7, 1) = (/ &
     &3.2476e-05_rb,3.8679e-05_rb,3.9032e-05_rb,3.7741e-05_rb,3.4552e-05_rb,2.9807e-05_rb, &
     &2.4164e-05_rb,1.7294e-05_rb,2.4102e-05_rb/)
      kao(:, 2, 7, 1) = (/ &
     &3.6570e-05_rb,4.6320e-05_rb,4.7537e-05_rb,4.6067e-05_rb,4.2268e-05_rb,3.6503e-05_rb, &
     &3.0476e-05_rb,2.2134e-05_rb,2.9448e-05_rb/)
      kao(:, 3, 7, 1) = (/ &
     &4.1124e-05_rb,5.4270e-05_rb,5.7210e-05_rb,5.5312e-05_rb,5.2106e-05_rb,4.5231e-05_rb, &
     &3.7737e-05_rb,2.7811e-05_rb,3.4668e-05_rb/)
      kao(:, 4, 7, 1) = (/ &
     &4.8133e-05_rb,6.4794e-05_rb,6.7628e-05_rb,6.6254e-05_rb,6.2299e-05_rb,5.4379e-05_rb, &
     &4.5931e-05_rb,3.4327e-05_rb,4.1743e-05_rb/)
      kao(:, 5, 7, 1) = (/ &
     &5.5532e-05_rb,7.7694e-05_rb,8.0423e-05_rb,7.9418e-05_rb,7.2354e-05_rb,6.4221e-05_rb, &
     &5.4806e-05_rb,4.1684e-05_rb,4.8759e-05_rb/)
      kao(:, 1, 8, 1) = (/ &
     &7.4902e-05_rb,7.7106e-05_rb,7.0826e-05_rb,6.4010e-05_rb,5.5968e-05_rb,4.6986e-05_rb, &
     &3.6409e-05_rb,2.2524e-05_rb,2.8079e-05_rb/)
      kao(:, 2, 8, 1) = (/ &
     &8.5067e-05_rb,8.8842e-05_rb,8.3769e-05_rb,7.6787e-05_rb,6.8134e-05_rb,5.7821e-05_rb, &
     &4.4557e-05_rb,2.8824e-05_rb,3.6133e-05_rb/)
      kao(:, 3, 8, 1) = (/ &
     &9.7879e-05_rb,1.0460e-04_rb,9.9884e-05_rb,9.2458e-05_rb,8.3229e-05_rb,7.0097e-05_rb, &
     &5.5355e-05_rb,3.6496e-05_rb,4.1979e-05_rb/)
      kao(:, 4, 8, 1) = (/ &
     &1.1321e-04_rb,1.2375e-04_rb,1.2003e-04_rb,1.1241e-04_rb,1.0088e-04_rb,8.6080e-05_rb, &
     &6.8531e-05_rb,4.5034e-05_rb,5.0526e-05_rb/)
      kao(:, 5, 8, 1) = (/ &
     &1.3142e-04_rb,1.5020e-04_rb,1.4626e-04_rb,1.3653e-04_rb,1.2235e-04_rb,1.0518e-04_rb, &
     &8.2469e-05_rb,5.5412e-05_rb,6.0484e-05_rb/)
      kao(:, 1, 9, 1) = (/ &
     &3.0190e-04_rb,2.8195e-04_rb,2.5208e-04_rb,2.1662e-04_rb,1.7929e-04_rb,1.4025e-04_rb, &
     &1.0049e-04_rb,5.8291e-05_rb,3.9381e-05_rb/)
      kao(:, 2, 9, 1) = (/ &
     &3.5007e-04_rb,3.2966e-04_rb,2.9418e-04_rb,2.5482e-04_rb,2.1206e-04_rb,1.6775e-04_rb, &
     &1.2208e-04_rb,7.2618e-05_rb,5.1006e-05_rb/)
      kao(:, 3, 9, 1) = (/ &
     &4.1355e-04_rb,3.9069e-04_rb,3.5043e-04_rb,3.0530e-04_rb,2.5611e-04_rb,2.0488e-04_rb, &
     &1.4964e-04_rb,8.8870e-05_rb,6.5960e-05_rb/)
      kao(:, 4, 9, 1) = (/ &
     &4.9093e-04_rb,4.6398e-04_rb,4.1726e-04_rb,3.6373e-04_rb,3.0710e-04_rb,2.4775e-04_rb, &
     &1.8258e-04_rb,1.0881e-04_rb,8.2307e-05_rb/)
      kao(:, 5, 9, 1) = (/ &
     &5.8022e-04_rb,5.5027e-04_rb,5.0137e-04_rb,4.3986e-04_rb,3.7225e-04_rb,2.9865e-04_rb, &
     &2.2079e-04_rb,1.3406e-04_rb,9.5285e-05_rb/)
      kao(:, 1,10, 1) = (/ &
     &1.2883e-03_rb,1.1451e-03_rb,9.9720e-04_rb,8.4603e-04_rb,6.8855e-04_rb,5.2856e-04_rb, &
     &3.6226e-04_rb,1.9198e-04_rb,7.4637e-05_rb/)
      kao(:, 2,10, 1) = (/ &
     &1.5104e-03_rb,1.3471e-03_rb,1.1771e-03_rb,9.9916e-04_rb,8.1528e-04_rb,6.2425e-04_rb, &
     &4.3021e-04_rb,2.3018e-04_rb,8.8724e-05_rb/)
      kao(:, 3,10, 1) = (/ &
     &1.8167e-03_rb,1.6260e-03_rb,1.4233e-03_rb,1.2086e-03_rb,9.8644e-04_rb,7.6173e-04_rb, &
     &5.2737e-04_rb,2.8610e-04_rb,1.0717e-04_rb/)
      kao(:, 4,10, 1) = (/ &
     &2.2177e-03_rb,1.9922e-03_rb,1.7424e-03_rb,1.4817e-03_rb,1.2127e-03_rb,9.3691e-04_rb, &
     &6.5292e-04_rb,3.5708e-04_rb,1.2411e-04_rb/)
      kao(:, 5,10, 1) = (/ &
     &2.7210e-03_rb,2.4399e-03_rb,2.1395e-03_rb,1.8137e-03_rb,1.4859e-03_rb,1.1551e-03_rb, &
     &8.0975e-04_rb,4.4504e-04_rb,1.4851e-04_rb/)
      kao(:, 1,11, 1) = (/ &
     &1.9516e-03_rb,1.7255e-03_rb,1.4956e-03_rb,1.2644e-03_rb,1.0251e-03_rb,7.8257e-04_rb, &
     &5.3359e-04_rb,2.7877e-04_rb,7.8095e-05_rb/)
      kao(:, 2,11, 1) = (/ &
     &2.3604e-03_rb,2.0924e-03_rb,1.8171e-03_rb,1.5363e-03_rb,1.2459e-03_rb,9.5100e-04_rb, &
     &6.5089e-04_rb,3.4345e-04_rb,9.5842e-05_rb/)
      kao(:, 3,11, 1) = (/ &
     &2.8804e-03_rb,2.5577e-03_rb,2.2239e-03_rb,1.8797e-03_rb,1.5233e-03_rb,1.1642e-03_rb, &
     &8.0020e-04_rb,4.2543e-04_rb,1.1860e-04_rb/)
      kao(:, 4,11, 1) = (/ &
     &3.5705e-03_rb,3.1731e-03_rb,2.7557e-03_rb,2.3268e-03_rb,1.8893e-03_rb,1.4468e-03_rb, &
     &9.9612e-04_rb,5.3416e-04_rb,1.4487e-04_rb/)
      kao(:, 5,11, 1) = (/ &
     &4.4488e-03_rb,3.9532e-03_rb,3.4428e-03_rb,2.9093e-03_rb,2.3658e-03_rb,1.8156e-03_rb, &
     &1.2567e-03_rb,6.7979e-04_rb,1.7247e-04_rb/)
      kao(:, 1,12, 1) = (/ &
     &2.1650e-03_rb,1.9116e-03_rb,1.6550e-03_rb,1.3963e-03_rb,1.1297e-03_rb,8.5899e-04_rb, &
     &5.8509e-04_rb,3.0490e-04_rb,7.1706e-05_rb/)
      kao(:, 2,12, 1) = (/ &
     &2.6267e-03_rb,2.3248e-03_rb,2.0167e-03_rb,1.7021e-03_rb,1.3758e-03_rb,1.0485e-03_rb, &
     &7.1504e-04_rb,3.7456e-04_rb,8.9235e-05_rb/)
      kao(:, 3,12, 1) = (/ &
     &3.2534e-03_rb,2.8836e-03_rb,2.5027e-03_rb,2.1051e-03_rb,1.7048e-03_rb,1.2988e-03_rb, &
     &8.8817e-04_rb,4.6846e-04_rb,1.1111e-04_rb/)
      kao(:, 4,12, 1) = (/ &
     &4.0663e-03_rb,3.6075e-03_rb,3.1250e-03_rb,2.6309e-03_rb,2.1307e-03_rb,1.6269e-03_rb, &
     &1.1159e-03_rb,5.9423e-04_rb,1.3807e-04_rb/)
      kao(:, 5,12, 1) = (/ &
     &5.1489e-03_rb,4.5663e-03_rb,3.9639e-03_rb,3.3363e-03_rb,2.7011e-03_rb,2.0639e-03_rb, &
     &1.4193e-03_rb,7.5878e-04_rb,1.6511e-04_rb/)
      kao(:, 1,13, 1) = (/ &
     &1.8892e-03_rb,1.6706e-03_rb,1.4468e-03_rb,1.2178e-03_rb,9.8369e-04_rb,7.4837e-04_rb, &
     &5.0829e-04_rb,2.6445e-04_rb,6.0347e-05_rb/)
      kao(:, 2,13, 1) = (/ &
     &2.3493e-03_rb,2.0800e-03_rb,1.8039e-03_rb,1.5176e-03_rb,1.2269e-03_rb,9.3478e-04_rb, &
     &6.3614e-04_rb,3.3311e-04_rb,7.4952e-05_rb/)
      kao(:, 3,13, 1) = (/ &
     &2.9367e-03_rb,2.6050e-03_rb,2.2571e-03_rb,1.8958e-03_rb,1.5337e-03_rb,1.1694e-03_rb, &
     &7.9916e-04_rb,4.2184e-04_rb,9.2442e-05_rb/)
      kao(:, 4,13, 1) = (/ &
     &3.6657e-03_rb,3.2513e-03_rb,2.8158e-03_rb,2.3704e-03_rb,1.9186e-03_rb,1.4653e-03_rb, &
     &1.0024e-03_rb,5.3341e-04_rb,1.1484e-04_rb/)
      kao(:, 5,13, 1) = (/ &
     &4.6760e-03_rb,4.1516e-03_rb,3.5983e-03_rb,3.0289e-03_rb,2.4503e-03_rb,1.8734e-03_rb, &
     &1.2888e-03_rb,6.8680e-04_rb,1.3777e-04_rb/)
      kao(:, 1, 1, 2) = (/ &
     &1.8748e-06_rb,2.5986e-05_rb,3.9140e-05_rb,5.0515e-05_rb,6.0453e-05_rb,7.0222e-05_rb, &
     &9.2153e-05_rb,1.1527e-04_rb,4.9863e-05_rb/)
      kao(:, 2, 1, 2) = (/ &
     &2.2195e-06_rb,3.2877e-05_rb,5.0201e-05_rb,6.4757e-05_rb,7.7459e-05_rb,8.9731e-05_rb, &
     &1.0523e-04_rb,1.3275e-04_rb,6.4946e-05_rb/)
      kao(:, 3, 1, 2) = (/ &
     &2.6699e-06_rb,4.0948e-05_rb,6.3539e-05_rb,8.1241e-05_rb,9.7151e-05_rb,1.1092e-04_rb, &
     &1.1761e-04_rb,1.3825e-04_rb,8.4658e-05_rb/)
      kao(:, 4, 1, 2) = (/ &
     &3.1333e-06_rb,5.0568e-05_rb,7.8106e-05_rb,9.9691e-05_rb,1.1981e-04_rb,1.3549e-04_rb, &
     &1.3924e-04_rb,1.5344e-04_rb,1.0884e-04_rb/)
      kao(:, 5, 1, 2) = (/ &
     &3.7302e-06_rb,6.1696e-05_rb,9.4521e-05_rb,1.2091e-04_rb,1.4534e-04_rb,1.6166e-04_rb, &
     &1.6745e-04_rb,1.7689e-04_rb,1.3808e-04_rb/)
      kao(:, 1, 2, 2) = (/ &
     &2.7303e-06_rb,2.4541e-05_rb,3.5763e-05_rb,4.4104e-05_rb,5.1595e-05_rb,5.8966e-05_rb, &
     &6.8130e-05_rb,1.0200e-04_rb,3.9223e-05_rb/)
      kao(:, 2, 2, 2) = (/ &
     &3.2091e-06_rb,3.1379e-05_rb,4.5745e-05_rb,5.7229e-05_rb,6.6576e-05_rb,7.4659e-05_rb, &
     &8.1615e-05_rb,9.9977e-05_rb,5.3367e-05_rb/)
      kao(:, 3, 2, 2) = (/ &
     &3.8233e-06_rb,3.9294e-05_rb,5.7741e-05_rb,7.2569e-05_rb,8.4616e-05_rb,9.4873e-05_rb, &
     &1.0240e-04_rb,1.1173e-04_rb,6.9904e-05_rb/)
      kao(:, 4, 2, 2) = (/ &
     &4.4247e-06_rb,4.8330e-05_rb,7.1550e-05_rb,9.0142e-05_rb,1.0471e-04_rb,1.1736e-04_rb, &
     &1.2542e-04_rb,1.2602e-04_rb,8.9761e-05_rb/)
      kao(:, 5, 2, 2) = (/ &
     &5.2245e-06_rb,5.9597e-05_rb,8.7482e-05_rb,1.0983e-04_rb,1.2772e-04_rb,1.4263e-04_rb, &
     &1.4957e-04_rb,1.4619e-04_rb,1.1276e-04_rb/)
      kao(:, 1, 3, 2) = (/ &
     &5.2515e-06_rb,2.5764e-05_rb,3.5421e-05_rb,4.2053e-05_rb,4.7449e-05_rb,5.0608e-05_rb, &
     &5.3790e-05_rb,6.3623e-05_rb,3.2707e-05_rb/)
      kao(:, 2, 3, 2) = (/ &
     &6.1812e-06_rb,3.3267e-05_rb,4.5617e-05_rb,5.4363e-05_rb,6.1462e-05_rb,6.5644e-05_rb, &
     &6.8603e-05_rb,7.2538e-05_rb,4.2974e-05_rb/)
      kao(:, 3, 3, 2) = (/ &
     &7.0789e-06_rb,4.1750e-05_rb,5.7967e-05_rb,6.9149e-05_rb,7.8026e-05_rb,8.4324e-05_rb, &
     &8.7723e-05_rb,8.7036e-05_rb,5.7402e-05_rb/)
      kao(:, 4, 3, 2) = (/ &
     &8.1019e-06_rb,5.2243e-05_rb,7.2254e-05_rb,8.6560e-05_rb,9.8380e-05_rb,1.0586e-04_rb, &
     &1.1006e-04_rb,1.0834e-04_rb,7.4594e-05_rb/)
      kao(:, 5, 3, 2) = (/ &
     &9.5928e-06_rb,6.4752e-05_rb,8.9095e-05_rb,1.0726e-04_rb,1.2114e-04_rb,1.3016e-04_rb, &
     &1.3551e-04_rb,1.3048e-04_rb,9.3901e-05_rb/)
      kao(:, 1, 4, 2) = (/ &
     &1.1020e-05_rb,2.8494e-05_rb,3.6932e-05_rb,4.2975e-05_rb,4.6074e-05_rb,4.6953e-05_rb, &
     &4.6313e-05_rb,4.4542e-05_rb,3.2970e-05_rb/)
      kao(:, 2, 4, 2) = (/ &
     &1.2562e-05_rb,3.7065e-05_rb,4.7812e-05_rb,5.4915e-05_rb,5.9599e-05_rb,6.1605e-05_rb, &
     &6.1355e-05_rb,5.8223e-05_rb,3.9379e-05_rb/)
      kao(:, 3, 4, 2) = (/ &
     &1.4576e-05_rb,4.7351e-05_rb,6.1614e-05_rb,7.0211e-05_rb,7.6184e-05_rb,7.9246e-05_rb, &
     &7.8840e-05_rb,7.4646e-05_rb,4.5622e-05_rb/)
      kao(:, 4, 4, 2) = (/ &
     &1.6280e-05_rb,5.8550e-05_rb,7.7244e-05_rb,8.9750e-05_rb,9.5812e-05_rb,1.0028e-04_rb, &
     &1.0042e-04_rb,9.4563e-05_rb,5.6157e-05_rb/)
      kao(:, 5, 4, 2) = (/ &
     &1.8452e-05_rb,7.1720e-05_rb,9.6110e-05_rb,1.1183e-04_rb,1.1944e-04_rb,1.2546e-04_rb, &
     &1.2521e-04_rb,1.1758e-04_rb,7.2313e-05_rb/)
      kao(:, 1, 5, 2) = (/ &
     &2.1312e-05_rb,3.6806e-05_rb,4.2862e-05_rb,4.6599e-05_rb,4.7270e-05_rb,4.6274e-05_rb, &
     &4.2916e-05_rb,3.6817e-05_rb,3.2593e-05_rb/)
      kao(:, 2, 5, 2) = (/ &
     &2.5202e-05_rb,4.5358e-05_rb,5.3413e-05_rb,5.8866e-05_rb,6.0628e-05_rb,5.9617e-05_rb, &
     &5.6530e-05_rb,4.9579e-05_rb,3.9610e-05_rb/)
      kao(:, 3, 5, 2) = (/ &
     &2.8699e-05_rb,5.5778e-05_rb,6.8120e-05_rb,7.4417e-05_rb,7.7573e-05_rb,7.6799e-05_rb, &
     &7.3323e-05_rb,6.4915e-05_rb,4.8072e-05_rb/)
      kao(:, 4, 5, 2) = (/ &
     &3.2753e-05_rb,6.7844e-05_rb,8.5625e-05_rb,9.3879e-05_rb,9.7934e-05_rb,9.8316e-05_rb, &
     &9.3375e-05_rb,8.3569e-05_rb,5.7445e-05_rb/)
      kao(:, 5, 5, 2) = (/ &
     &3.6908e-05_rb,8.3612e-05_rb,1.0490e-04_rb,1.1632e-04_rb,1.2275e-04_rb,1.2328e-04_rb, &
     &1.1667e-04_rb,1.0539e-04_rb,6.6570e-05_rb/)
      kao(:, 1, 6, 2) = (/ &
     &4.3241e-05_rb,5.7363e-05_rb,5.9383e-05_rb,5.9083e-05_rb,5.6687e-05_rb,5.1929e-05_rb, &
     &4.4854e-05_rb,3.4134e-05_rb,3.3554e-05_rb/)
      kao(:, 2, 6, 2) = (/ &
     &4.8160e-05_rb,6.6362e-05_rb,7.1460e-05_rb,7.1908e-05_rb,6.9540e-05_rb,6.3380e-05_rb, &
     &5.6957e-05_rb,4.4490e-05_rb,3.9435e-05_rb/)
      kao(:, 3, 6, 2) = (/ &
     &5.5150e-05_rb,7.8279e-05_rb,8.5897e-05_rb,8.6929e-05_rb,8.6216e-05_rb,7.9912e-05_rb, &
     &7.1634e-05_rb,5.7727e-05_rb,4.6871e-05_rb/)
      kao(:, 4, 6, 2) = (/ &
     &6.2702e-05_rb,9.3878e-05_rb,1.0189e-04_rb,1.0755e-04_rb,1.0575e-04_rb,1.0053e-04_rb, &
     &8.9365e-05_rb,7.5152e-05_rb,5.7874e-05_rb/)
      kao(:, 5, 6, 2) = (/ &
     &7.1646e-05_rb,1.0994e-04_rb,1.2283e-04_rb,1.3142e-04_rb,1.3048e-04_rb,1.2339e-04_rb, &
     &1.1339e-04_rb,9.4667e-05_rb,6.9560e-05_rb/)
      kao(:, 1, 7, 2) = (/ &
     &9.4217e-05_rb,1.0625e-04_rb,1.0297e-04_rb,9.4341e-05_rb,8.4820e-05_rb,7.4540e-05_rb, &
     &5.9573e-05_rb,4.0162e-05_rb,3.4032e-05_rb/)
      kao(:, 2, 7, 2) = (/ &
     &1.0328e-04_rb,1.1698e-04_rb,1.1273e-04_rb,1.0730e-04_rb,9.8854e-05_rb,8.8068e-05_rb, &
     &7.1680e-05_rb,5.0363e-05_rb,4.1780e-05_rb/)
      kao(:, 3, 7, 2) = (/ &
     &1.1436e-04_rb,1.3466e-04_rb,1.3293e-04_rb,1.2807e-04_rb,1.1742e-04_rb,1.0440e-04_rb, &
     &8.7142e-05_rb,6.2911e-05_rb,5.1346e-05_rb/)
      kao(:, 4, 7, 2) = (/ &
     &1.2805e-04_rb,1.5498e-04_rb,1.5579e-04_rb,1.5032e-04_rb,1.3953e-04_rb,1.2638e-04_rb, &
     &1.0638e-04_rb,7.6925e-05_rb,6.1784e-05_rb/)
      kao(:, 5, 7, 2) = (/ &
     &1.4510e-04_rb,1.7937e-04_rb,1.8186e-04_rb,1.7514e-04_rb,1.6579e-04_rb,1.5232e-04_rb, &
     &1.2822e-04_rb,9.5671e-05_rb,7.0048e-05_rb/)
      kao(:, 1, 8, 2) = (/ &
     &2.4926e-04_rb,2.4392e-04_rb,2.2297e-04_rb,1.9521e-04_rb,1.6651e-04_rb,1.3349e-04_rb, &
     &9.8669e-05_rb,6.3201e-05_rb,3.1807e-05_rb/)
      kao(:, 2, 8, 2) = (/ &
     &2.7454e-04_rb,2.7755e-04_rb,2.5558e-04_rb,2.2780e-04_rb,1.9618e-04_rb,1.5908e-04_rb, &
     &1.2121e-04_rb,7.6667e-05_rb,4.0012e-05_rb/)
      kao(:, 3, 8, 2) = (/ &
     &2.9508e-04_rb,3.0091e-04_rb,2.7929e-04_rb,2.5223e-04_rb,2.1930e-04_rb,1.8562e-04_rb, &
     &1.4312e-04_rb,9.2529e-05_rb,5.6355e-05_rb/)
      kao(:, 4, 8, 2) = (/ &
     &3.2350e-04_rb,3.3697e-04_rb,3.1656e-04_rb,2.8676e-04_rb,2.5417e-04_rb,2.1233e-04_rb, &
     &1.6702e-04_rb,1.1113e-04_rb,6.5692e-05_rb/)
      kao(:, 5, 8, 2) = (/ &
     &3.5413e-04_rb,3.7149e-04_rb,3.5724e-04_rb,3.2794e-04_rb,2.8883e-04_rb,2.4521e-04_rb, &
     &1.9502e-04_rb,1.3265e-04_rb,7.4636e-05_rb/)
      kao(:, 1, 9, 2) = (/ &
     &1.1370e-03_rb,1.0245e-03_rb,9.0233e-04_rb,7.7373e-04_rb,6.3596e-04_rb,4.9782e-04_rb, &
     &3.4706e-04_rb,1.8813e-04_rb,5.4967e-05_rb/)
      kao(:, 2, 9, 2) = (/ &
     &1.2549e-03_rb,1.1327e-03_rb,1.0081e-03_rb,8.6174e-04_rb,7.1233e-04_rb,5.5663e-04_rb, &
     &3.9054e-04_rb,2.1740e-04_rb,6.5863e-05_rb/)
      kao(:, 3, 9, 2) = (/ &
     &1.4193e-03_rb,1.2929e-03_rb,1.1501e-03_rb,9.8954e-04_rb,8.1834e-04_rb,6.4034e-04_rb, &
     &4.5590e-04_rb,2.5841e-04_rb,7.6782e-05_rb/)
      kao(:, 4, 9, 2) = (/ &
     &1.5730e-03_rb,1.4525e-03_rb,1.2939e-03_rb,1.1191e-03_rb,9.2869e-04_rb,7.2912e-04_rb, &
     &5.2003e-04_rb,2.9928e-04_rb,9.3109e-05_rb/)
      kao(:, 5, 9, 2) = (/ &
     &1.7020e-03_rb,1.5888e-03_rb,1.4187e-03_rb,1.2315e-03_rb,1.0334e-03_rb,8.2293e-04_rb, &
     &5.9761e-04_rb,3.4411e-04_rb,1.1836e-04_rb/)
      kao(:, 1,10, 2) = (/ &
     &6.1528e-03_rb,5.4056e-03_rb,4.6635e-03_rb,3.9185e-03_rb,3.1663e-03_rb,2.4050e-03_rb, &
     &1.6372e-03_rb,8.5009e-04_rb,8.0054e-05_rb/)
      kao(:, 2,10, 2) = (/ &
     &6.8550e-03_rb,6.0391e-03_rb,5.2246e-03_rb,4.3821e-03_rb,3.5514e-03_rb,2.7172e-03_rb, &
     &1.8529e-03_rb,9.7224e-04_rb,1.0130e-04_rb/)
      kao(:, 3,10, 2) = (/ &
     &7.6163e-03_rb,6.7254e-03_rb,5.8028e-03_rb,4.8844e-03_rb,3.9710e-03_rb,3.0268e-03_rb, &
     &2.0640e-03_rb,1.0804e-03_rb,1.3327e-04_rb/)
      kao(:, 4,10, 2) = (/ &
     &8.4017e-03_rb,7.4385e-03_rb,6.4324e-03_rb,5.4278e-03_rb,4.4132e-03_rb,3.3677e-03_rb, &
     &2.3075e-03_rb,1.2226e-03_rb,1.6598e-04_rb/)
      kao(:, 5,10, 2) = (/ &
     &9.3911e-03_rb,8.3333e-03_rb,7.2272e-03_rb,6.1232e-03_rb,4.9837e-03_rb,3.8167e-03_rb, &
     &2.6213e-03_rb,1.4063e-03_rb,2.0638e-04_rb/)
      kao(:, 1,11, 2) = (/ &
     &1.1762e-02_rb,1.0306e-02_rb,8.8593e-03_rb,7.4127e-03_rb,5.9627e-03_rb,4.5045e-03_rb, &
     &3.0511e-03_rb,1.5666e-03_rb,8.2025e-05_rb/)
      kao(:, 2,11, 2) = (/ &
     &1.3012e-02_rb,1.1426e-02_rb,9.8347e-03_rb,8.2336e-03_rb,6.6323e-03_rb,5.0275e-03_rb, &
     &3.4043e-03_rb,1.7550e-03_rb,1.0611e-04_rb/)
      kao(:, 3,11, 2) = (/ &
     &1.4612e-02_rb,1.2845e-02_rb,1.1070e-02_rb,9.2725e-03_rb,7.4822e-03_rb,5.6847e-03_rb, &
     &3.8534e-03_rb,1.9989e-03_rb,1.3346e-04_rb/)
      kao(:, 4,11, 2) = (/ &
     &1.6251e-02_rb,1.4313e-02_rb,1.2347e-02_rb,1.0360e-02_rb,8.3777e-03_rb,6.3716e-03_rb, &
     &4.3250e-03_rb,2.2445e-03_rb,1.6083e-04_rb/)
      kao(:, 5,11, 2) = (/ &
     &1.8150e-02_rb,1.6016e-02_rb,1.3813e-02_rb,1.1608e-02_rb,9.3833e-03_rb,7.1360e-03_rb, &
     &4.8421e-03_rb,2.5233e-03_rb,2.0030e-04_rb/)
      kao(:, 1,12, 2) = (/ &
     &1.5059e-02_rb,1.3197e-02_rb,1.1332e-02_rb,9.4687e-03_rb,7.6107e-03_rb,5.7434e-03_rb, &
     &3.8799e-03_rb,1.9799e-03_rb,8.0330e-05_rb/)
      kao(:, 2,12, 2) = (/ &
     &1.6512e-02_rb,1.4476e-02_rb,1.2442e-02_rb,1.0410e-02_rb,8.3743e-03_rb,6.3355e-03_rb, &
     &4.2771e-03_rb,2.1952e-03_rb,9.9323e-05_rb/)
      kao(:, 3,12, 2) = (/ &
     &1.8685e-02_rb,1.6400e-02_rb,1.4116e-02_rb,1.1818e-02_rb,9.5219e-03_rb,7.2188e-03_rb, &
     &4.8730e-03_rb,2.5083e-03_rb,1.2343e-04_rb/)
      kao(:, 4,12, 2) = (/ &
     &2.1153e-02_rb,1.8588e-02_rb,1.6002e-02_rb,1.3413e-02_rb,1.0821e-02_rb,8.2029e-03_rb, &
     &5.5391e-03_rb,2.8500e-03_rb,1.4931e-04_rb/)
      kao(:, 5,12, 2) = (/ &
     &2.3737e-02_rb,2.0901e-02_rb,1.7991e-02_rb,1.5090e-02_rb,1.2172e-02_rb,9.2290e-03_rb, &
     &6.2485e-03_rb,3.2234e-03_rb,1.9022e-04_rb/)
      kao(:, 1,13, 2) = (/ &
     &1.4847e-02_rb,1.3010e-02_rb,1.1175e-02_rb,9.3417e-03_rb,7.5039e-03_rb,5.6608e-03_rb, &
     &3.8195e-03_rb,1.9483e-03_rb,7.0259e-05_rb/)
      kao(:, 2,13, 2) = (/ &
     &1.6739e-02_rb,1.4679e-02_rb,1.2619e-02_rb,1.0555e-02_rb,8.4903e-03_rb,6.4217e-03_rb, &
     &4.3269e-03_rb,2.2129e-03_rb,8.5490e-05_rb/)
      kao(:, 3,13, 2) = (/ &
     &1.8825e-02_rb,1.6518e-02_rb,1.4211e-02_rb,1.1898e-02_rb,9.5859e-03_rb,7.2507e-03_rb, &
     &4.8896e-03_rb,2.5047e-03_rb,1.1226e-04_rb/)
      kao(:, 4,13, 2) = (/ &
     &2.1292e-02_rb,1.8712e-02_rb,1.6107e-02_rb,1.3496e-02_rb,1.0877e-02_rb,8.2331e-03_rb, &
     &5.5639e-03_rb,2.8510e-03_rb,1.4031e-04_rb/)
      kao(:, 5,13, 2) = (/ &
     &2.4158e-02_rb,2.1241e-02_rb,1.8299e-02_rb,1.5344e-02_rb,1.2375e-02_rb,9.3646e-03_rb, &
     &6.3197e-03_rb,3.2533e-03_rb,1.7265e-04_rb/)
      kao(:, 1, 1, 3) = (/ &
     &1.0410e-05_rb,6.1962e-05_rb,8.9498e-05_rb,1.0661e-04_rb,1.2164e-04_rb,1.3377e-04_rb, &
     &1.2941e-04_rb,1.4521e-04_rb,8.1226e-05_rb/)
      kao(:, 2, 1, 3) = (/ &
     &1.1876e-05_rb,7.8368e-05_rb,1.1198e-04_rb,1.3409e-04_rb,1.5187e-04_rb,1.6090e-04_rb, &
     &1.6481e-04_rb,1.6441e-04_rb,1.1141e-04_rb/)
      kao(:, 3, 1, 3) = (/ &
     &1.3686e-05_rb,9.7536e-05_rb,1.3724e-04_rb,1.6641e-04_rb,1.8557e-04_rb,1.9386e-04_rb, &
     &1.9539e-04_rb,1.8991e-04_rb,1.4967e-04_rb/)
      kao(:, 4, 1, 3) = (/ &
     &1.5807e-05_rb,1.1754e-04_rb,1.6722e-04_rb,2.0238e-04_rb,2.2277e-04_rb,2.3084e-04_rb, &
     &2.3103e-04_rb,2.1679e-04_rb,1.9490e-04_rb/)
      kao(:, 5, 1, 3) = (/ &
     &1.8430e-05_rb,1.4041e-04_rb,2.0063e-04_rb,2.4077e-04_rb,2.6088e-04_rb,2.7081e-04_rb, &
     &2.7088e-04_rb,2.6451e-04_rb,2.4633e-04_rb/)
      kao(:, 1, 2, 3) = (/ &
     &1.4574e-05_rb,6.1181e-05_rb,8.5219e-05_rb,1.0233e-04_rb,1.1097e-04_rb,1.1864e-04_rb, &
     &1.2443e-04_rb,1.2339e-04_rb,6.7714e-05_rb/)
      kao(:, 2, 2, 3) = (/ &
     &1.6727e-05_rb,7.7358e-05_rb,1.0881e-04_rb,1.2844e-04_rb,1.4104e-04_rb,1.5016e-04_rb, &
     &1.5142e-04_rb,1.4164e-04_rb,9.0996e-05_rb/)
      kao(:, 3, 2, 3) = (/ &
     &1.9092e-05_rb,9.6319e-05_rb,1.3467e-04_rb,1.5937e-04_rb,1.7543e-04_rb,1.8362e-04_rb, &
     &1.8038e-04_rb,1.5924e-04_rb,1.2122e-04_rb/)
      kao(:, 4, 2, 3) = (/ &
     &2.2128e-05_rb,1.1862e-04_rb,1.6445e-04_rb,1.9397e-04_rb,2.1438e-04_rb,2.2054e-04_rb, &
     &2.1448e-04_rb,2.0102e-04_rb,1.5787e-04_rb/)
      kao(:, 5, 2, 3) = (/ &
     &2.5953e-05_rb,1.4141e-04_rb,1.9710e-04_rb,2.3342e-04_rb,2.5581e-04_rb,2.5978e-04_rb, &
     &2.5390e-04_rb,2.3627e-04_rb,2.0062e-04_rb/)
      kao(:, 1, 3, 3) = (/ &
     &2.6011e-05_rb,6.9300e-05_rb,8.8885e-05_rb,1.0316e-04_rb,1.1255e-04_rb,1.1444e-04_rb, &
     &1.1119e-04_rb,1.0838e-04_rb,5.7064e-05_rb/)
      kao(:, 2, 3, 3) = (/ &
     &2.9960e-05_rb,8.6707e-05_rb,1.1415e-04_rb,1.3351e-04_rb,1.4420e-04_rb,1.4685e-04_rb, &
     &1.4419e-04_rb,1.3258e-04_rb,7.5649e-05_rb/)
      kao(:, 3, 3, 3) = (/ &
     &3.4740e-05_rb,1.0851e-04_rb,1.4450e-04_rb,1.6834e-04_rb,1.8054e-04_rb,1.8416e-04_rb, &
     &1.8038e-04_rb,1.5855e-04_rb,9.9657e-05_rb/)
      kao(:, 4, 3, 3) = (/ &
     &4.0367e-05_rb,1.3338e-04_rb,1.7889e-04_rb,2.0752e-04_rb,2.2266e-04_rb,2.2792e-04_rb, &
     &2.1975e-04_rb,1.8879e-04_rb,1.2950e-04_rb/)
      kao(:, 5, 3, 3) = (/ &
     &4.7325e-05_rb,1.6087e-04_rb,2.1635e-04_rb,2.5031e-04_rb,2.7116e-04_rb,2.7639e-04_rb, &
     &2.6032e-04_rb,2.2480e-04_rb,1.6585e-04_rb/)
      kao(:, 1, 4, 3) = (/ &
     &4.6629e-05_rb,8.8856e-05_rb,1.0307e-04_rb,1.1034e-04_rb,1.1324e-04_rb,1.1518e-04_rb, &
     &1.0941e-04_rb,9.2447e-05_rb,4.9807e-05_rb/)
      kao(:, 2, 4, 3) = (/ &
     &5.3813e-05_rb,1.0914e-04_rb,1.2953e-04_rb,1.4280e-04_rb,1.4973e-04_rb,1.5074e-04_rb, &
     &1.4150e-04_rb,1.2111e-04_rb,6.6358e-05_rb/)
      kao(:, 3, 4, 3) = (/ &
     &6.3364e-05_rb,1.3307e-04_rb,1.6255e-04_rb,1.8292e-04_rb,1.9276e-04_rb,1.9285e-04_rb, &
     &1.8002e-04_rb,1.5407e-04_rb,8.8052e-05_rb/)
      kao(:, 4, 4, 3) = (/ &
     &7.4529e-05_rb,1.6123e-04_rb,2.0145e-04_rb,2.2832e-04_rb,2.4093e-04_rb,2.3982e-04_rb, &
     &2.2562e-04_rb,1.8990e-04_rb,1.1560e-04_rb/)
      kao(:, 5, 4, 3) = (/ &
     &8.8134e-05_rb,1.9717e-04_rb,2.4798e-04_rb,2.7909e-04_rb,2.9532e-04_rb,2.9441e-04_rb, &
     &2.7564e-04_rb,2.2733e-04_rb,1.4686e-04_rb/)
      kao(:, 1, 5, 3) = (/ &
     &7.7279e-05_rb,1.1122e-04_rb,1.2109e-04_rb,1.2443e-04_rb,1.2189e-04_rb,1.1617e-04_rb, &
     &1.0613e-04_rb,8.7521e-05_rb,4.7012e-05_rb/)
      kao(:, 2, 5, 3) = (/ &
     &8.9473e-05_rb,1.3546e-04_rb,1.5195e-04_rb,1.5794e-04_rb,1.5786e-04_rb,1.5329e-04_rb, &
     &1.4127e-04_rb,1.1487e-04_rb,6.0204e-05_rb/)
      kao(:, 3, 5, 3) = (/ &
     &1.0328e-04_rb,1.6610e-04_rb,1.8961e-04_rb,1.9990e-04_rb,2.0229e-04_rb,2.0006e-04_rb, &
     &1.8432e-04_rb,1.4896e-04_rb,7.6087e-05_rb/)
      kao(:, 4, 5, 3) = (/ &
     &1.2119e-04_rb,2.0367e-04_rb,2.3155e-04_rb,2.4851e-04_rb,2.5658e-04_rb,2.5211e-04_rb, &
     &2.3296e-04_rb,1.8864e-04_rb,9.8744e-05_rb/)
      kao(:, 5, 5, 3) = (/ &
     &1.4337e-04_rb,2.4554e-04_rb,2.8278e-04_rb,3.0781e-04_rb,3.1834e-04_rb,3.1139e-04_rb, &
     &2.8860e-04_rb,2.3262e-04_rb,1.2920e-04_rb/)
      kao(:, 1, 6, 3) = (/ &
     &1.1761e-04_rb,1.4538e-04_rb,1.4284e-04_rb,1.3740e-04_rb,1.2980e-04_rb,1.1847e-04_rb, &
     &1.0232e-04_rb,8.1254e-05_rb,4.1748e-05_rb/)
      kao(:, 2, 6, 3) = (/ &
     &1.3703e-04_rb,1.7162e-04_rb,1.7382e-04_rb,1.7289e-04_rb,1.6644e-04_rb,1.5514e-04_rb, &
     &1.3666e-04_rb,1.0938e-04_rb,5.6033e-05_rb/)
      kao(:, 3, 6, 3) = (/ &
     &1.5744e-04_rb,2.0479e-04_rb,2.1458e-04_rb,2.1922e-04_rb,2.1487e-04_rb,2.0072e-04_rb, &
     &1.7976e-04_rb,1.4480e-04_rb,7.3663e-05_rb/)
      kao(:, 4, 6, 3) = (/ &
     &1.7947e-04_rb,2.4802e-04_rb,2.6658e-04_rb,2.7311e-04_rb,2.6613e-04_rb,2.5639e-04_rb, &
     &2.3337e-04_rb,1.8491e-04_rb,9.1670e-05_rb/)
      kao(:, 5, 6, 3) = (/ &
     &2.1213e-04_rb,2.9698e-04_rb,3.2800e-04_rb,3.3255e-04_rb,3.3379e-04_rb,3.2227e-04_rb, &
     &2.9213e-04_rb,2.3295e-04_rb,1.1053e-04_rb/)
      kao(:, 1, 7, 3) = (/ &
     &2.1717e-04_rb,2.2955e-04_rb,2.1661e-04_rb,1.9608e-04_rb,1.6989e-04_rb,1.4299e-04_rb, &
     &1.1566e-04_rb,8.2895e-05_rb,3.7387e-05_rb/)
      kao(:, 2, 7, 3) = (/ &
     &2.4414e-04_rb,2.6643e-04_rb,2.5638e-04_rb,2.3114e-04_rb,2.0334e-04_rb,1.7738e-04_rb, &
     &1.4895e-04_rb,1.0764e-04_rb,5.2222e-05_rb/)
      kao(:, 3, 7, 3) = (/ &
     &2.7286e-04_rb,3.0487e-04_rb,2.9526e-04_rb,2.7261e-04_rb,2.5126e-04_rb,2.2507e-04_rb, &
     &1.8734e-04_rb,1.4022e-04_rb,6.8628e-05_rb/)
      kao(:, 4, 7, 3) = (/ &
     &3.1153e-04_rb,3.5338e-04_rb,3.4554e-04_rb,3.2591e-04_rb,3.0719e-04_rb,2.7679e-04_rb, &
     &2.3565e-04_rb,1.8522e-04_rb,8.6360e-05_rb/)
      kao(:, 5, 7, 3) = (/ &
     &3.5307e-04_rb,4.0595e-04_rb,4.0654e-04_rb,3.9853e-04_rb,3.7853e-04_rb,3.4348e-04_rb, &
     &3.0049e-04_rb,2.3594e-04_rb,1.1315e-04_rb/)
      kao(:, 1, 8, 3) = (/ &
     &5.5525e-04_rb,5.3290e-04_rb,4.9219e-04_rb,4.3834e-04_rb,3.7060e-04_rb,2.9620e-04_rb, &
     &2.1670e-04_rb,1.3044e-04_rb,4.3913e-05_rb/)
      kao(:, 2, 8, 3) = (/ &
     &6.1273e-04_rb,5.9169e-04_rb,5.4415e-04_rb,4.8432e-04_rb,4.1256e-04_rb,3.3431e-04_rb, &
     &2.4554e-04_rb,1.5735e-04_rb,5.6222e-05_rb/)
      kao(:, 3, 8, 3) = (/ &
     &6.6697e-04_rb,6.5930e-04_rb,6.1229e-04_rb,5.4542e-04_rb,4.6824e-04_rb,3.7897e-04_rb, &
     &2.8714e-04_rb,1.8945e-04_rb,7.2182e-05_rb/)
      kao(:, 4, 8, 3) = (/ &
     &7.3789e-04_rb,7.3274e-04_rb,6.8030e-04_rb,6.0993e-04_rb,5.2803e-04_rb,4.3826e-04_rb, &
     &3.4177e-04_rb,2.2879e-04_rb,9.7984e-05_rb/)
      kao(:, 5, 8, 3) = (/ &
     &8.2418e-04_rb,8.2930e-04_rb,7.7540e-04_rb,7.0471e-04_rb,6.2407e-04_rb,5.2518e-04_rb, &
     &4.1114e-04_rb,2.7739e-04_rb,1.2917e-04_rb/)
      kao(:, 1, 9, 3) = (/ &
     &2.9866e-03_rb,2.6446e-03_rb,2.3122e-03_rb,1.9743e-03_rb,1.6235e-03_rb,1.2497e-03_rb, &
     &8.7401e-04_rb,4.7254e-04_rb,6.5815e-05_rb/)
      kao(:, 2, 9, 3) = (/ &
     &3.1867e-03_rb,2.8548e-03_rb,2.5070e-03_rb,2.1512e-03_rb,1.7728e-03_rb,1.3804e-03_rb, &
     &9.6416e-04_rb,5.2342e-04_rb,8.4426e-05_rb/)
      kao(:, 3, 9, 3) = (/ &
     &3.3577e-03_rb,3.0331e-03_rb,2.6681e-03_rb,2.2844e-03_rb,1.8894e-03_rb,1.4824e-03_rb, &
     &1.0437e-03_rb,5.7926e-04_rb,1.1456e-04_rb/)
      kao(:, 4, 9, 3) = (/ &
     &3.5955e-03_rb,3.2669e-03_rb,2.8865e-03_rb,2.4786e-03_rb,2.0664e-03_rb,1.6261e-03_rb, &
     &1.1591e-03_rb,6.6253e-04_rb,1.4708e-04_rb/)
      kao(:, 5, 9, 3) = (/ &
     &3.9554e-03_rb,3.6156e-03_rb,3.2186e-03_rb,2.7765e-03_rb,2.3055e-03_rb,1.8177e-03_rb, &
     &1.3061e-03_rb,7.6692e-04_rb,1.9062e-04_rb/)
      kao(:, 1,10, 3) = (/ &
     &1.6695e-02_rb,1.4628e-02_rb,1.2561e-02_rb,1.0499e-02_rb,8.4474e-03_rb,6.4042e-03_rb, &
     &4.3459e-03_rb,2.2571e-03_rb,5.1038e-05_rb/)
      kao(:, 2,10, 3) = (/ &
     &1.8223e-02_rb,1.5977e-02_rb,1.3728e-02_rb,1.1517e-02_rb,9.2932e-03_rb,7.0542e-03_rb, &
     &4.8075e-03_rb,2.4890e-03_rb,1.1555e-04_rb/)
      kao(:, 3,10, 3) = (/ &
     &2.0043e-02_rb,1.7593e-02_rb,1.5175e-02_rb,1.2748e-02_rb,1.0288e-02_rb,7.8318e-03_rb, &
     &5.3279e-03_rb,2.7782e-03_rb,1.4398e-04_rb/)
      kao(:, 4,10, 3) = (/ &
     &2.1904e-02_rb,1.9249e-02_rb,1.6643e-02_rb,1.3989e-02_rb,1.1315e-02_rb,8.6138e-03_rb, &
     &5.8652e-03_rb,3.0632e-03_rb,1.9715e-04_rb/)
      kao(:, 5,10, 3) = (/ &
     &2.3159e-02_rb,2.0410e-02_rb,1.7664e-02_rb,1.4873e-02_rb,1.2019e-02_rb,9.1728e-03_rb, &
     &6.2874e-03_rb,3.3158e-03_rb,2.7195e-04_rb/)
      kao(:, 1,11, 3) = (/ &
     &3.2603e-02_rb,2.8559e-02_rb,2.4510e-02_rb,2.0449e-02_rb,1.6397e-02_rb,1.2361e-02_rb, &
     &8.3115e-03_rb,4.2614e-03_rb,4.3561e-05_rb/)
      kao(:, 2,11, 3) = (/ &
     &3.5490e-02_rb,3.1085e-02_rb,2.6692e-02_rb,2.2303e-02_rb,1.7919e-02_rb,1.3525e-02_rb, &
     &9.1241e-03_rb,4.6814e-03_rb,1.0680e-04_rb/)
      kao(:, 3,11, 3) = (/ &
     &3.8854e-02_rb,3.4035e-02_rb,2.9244e-02_rb,2.4483e-02_rb,1.9700e-02_rb,1.4916e-02_rb, &
     &1.0095e-02_rb,5.1808e-03_rb,1.8562e-04_rb/)
      kao(:, 4,11, 3) = (/ &
     &4.2305e-02_rb,3.7086e-02_rb,3.1904e-02_rb,2.6732e-02_rb,2.1504e-02_rb,1.6276e-02_rb, &
     &1.1000e-02_rb,5.6671e-03_rb,2.4195e-04_rb/)
      kao(:, 5,11, 3) = (/ &
     &4.6003e-02_rb,4.0358e-02_rb,3.4797e-02_rb,2.9165e-02_rb,2.3531e-02_rb,1.7850e-02_rb, &
     &1.2113e-02_rb,6.2729e-03_rb,3.2815e-04_rb/)
      kao(:, 1,12, 3) = (/ &
     &4.2717e-02_rb,3.7390e-02_rb,3.2076e-02_rb,2.6766e-02_rb,2.1453e-02_rb,1.6148e-02_rb, &
     &1.0833e-02_rb,5.5285e-03_rb,5.3162e-05_rb/)
      kao(:, 2,12, 3) = (/ &
     &4.7716e-02_rb,4.1793e-02_rb,3.5854e-02_rb,2.9930e-02_rb,2.4020e-02_rb,1.8099e-02_rb, &
     &1.2166e-02_rb,6.2010e-03_rb,1.2199e-04_rb/)
      kao(:, 3,12, 3) = (/ &
     &5.2485e-02_rb,4.5967e-02_rb,3.9456e-02_rb,3.2983e-02_rb,2.6496e-02_rb,1.9981e-02_rb, &
     &1.3462e-02_rb,6.8646e-03_rb,1.9551e-04_rb/)
      kao(:, 4,12, 3) = (/ &
     &5.6971e-02_rb,4.9905e-02_rb,4.2910e-02_rb,3.5876e-02_rb,2.8829e-02_rb,2.1772e-02_rb, &
     &1.4678e-02_rb,7.4885e-03_rb,2.4038e-04_rb/)
      kao(:, 5,12, 3) = (/ &
     &6.1845e-02_rb,5.4216e-02_rb,4.6694e-02_rb,3.9065e-02_rb,3.1428e-02_rb,2.3760e-02_rb, &
     &1.6034e-02_rb,8.2201e-03_rb,3.2330e-04_rb/)
      kao(:, 1,13, 3) = (/ &
     &4.4159e-02_rb,3.8661e-02_rb,3.3159e-02_rb,2.7655e-02_rb,2.2159e-02_rb,1.6679e-02_rb, &
     &1.1182e-02_rb,5.6866e-03_rb,7.0844e-05_rb/)
      kao(:, 2,13, 3) = (/ &
     &4.9543e-02_rb,4.3376e-02_rb,3.7217e-02_rb,3.1073e-02_rb,2.4939e-02_rb,1.8777e-02_rb, &
     &1.2622e-02_rb,6.4095e-03_rb,1.2350e-04_rb/)
      kao(:, 3,13, 3) = (/ &
     &5.4720e-02_rb,4.7942e-02_rb,4.1156e-02_rb,3.4403e-02_rb,2.7607e-02_rb,2.0806e-02_rb, &
     &1.3984e-02_rb,7.1021e-03_rb,1.5128e-04_rb/)
      kao(:, 4,13, 3) = (/ &
     &5.9716e-02_rb,5.2321e-02_rb,4.4977e-02_rb,3.7585e-02_rb,3.0190e-02_rb,2.2781e-02_rb, &
     &1.5325e-02_rb,7.8087e-03_rb,2.1269e-04_rb/)
      kao(:, 5,13, 3) = (/ &
     &6.4624e-02_rb,5.6664e-02_rb,4.8723e-02_rb,4.0765e-02_rb,3.2763e-02_rb,2.4744e-02_rb, &
     &1.6652e-02_rb,8.4913e-03_rb,3.0727e-04_rb/)
      kao(:, 1, 1, 4) = (/ &
     &5.7873e-05_rb,1.6488e-04_rb,2.0580e-04_rb,2.2960e-04_rb,2.3556e-04_rb,2.2261e-04_rb, &
     &2.0972e-04_rb,1.9322e-04_rb,1.3088e-04_rb/)
      kao(:, 2, 1, 4) = (/ &
     &6.6249e-05_rb,1.9462e-04_rb,2.4549e-04_rb,2.7463e-04_rb,2.7934e-04_rb,2.6538e-04_rb, &
     &2.3743e-04_rb,2.0925e-04_rb,1.7715e-04_rb/)
      kao(:, 3, 1, 4) = (/ &
     &7.5658e-05_rb,2.2561e-04_rb,2.8843e-04_rb,3.1940e-04_rb,3.2388e-04_rb,3.1216e-04_rb, &
     &2.9036e-04_rb,2.7653e-04_rb,2.3143e-04_rb/)
      kao(:, 4, 1, 4) = (/ &
     &8.5832e-05_rb,2.6205e-04_rb,3.3188e-04_rb,3.6495e-04_rb,3.7173e-04_rb,3.6401e-04_rb, &
     &3.4696e-04_rb,3.4171e-04_rb,2.9590e-04_rb/)
      kao(:, 5, 1, 4) = (/ &
     &9.6691e-05_rb,2.9985e-04_rb,3.7678e-04_rb,4.1538e-04_rb,4.2689e-04_rb,4.2505e-04_rb, &
     &4.1168e-04_rb,4.0784e-04_rb,3.7540e-04_rb/)
      kao(:, 1, 2, 4) = (/ &
     &8.0001e-05_rb,1.7863e-04_rb,2.1664e-04_rb,2.3517e-04_rb,2.4432e-04_rb,2.3529e-04_rb, &
     &2.0358e-04_rb,1.5402e-04_rb,1.1156e-04_rb/)
      kao(:, 2, 2, 4) = (/ &
     &9.2299e-05_rb,2.0953e-04_rb,2.5648e-04_rb,2.8347e-04_rb,2.9289e-04_rb,2.8063e-04_rb, &
     &2.4649e-04_rb,1.9992e-04_rb,1.5181e-04_rb/)
      kao(:, 3, 2, 4) = (/ &
     &1.0437e-04_rb,2.4399e-04_rb,3.0364e-04_rb,3.3466e-04_rb,3.4277e-04_rb,3.2781e-04_rb, &
     &2.9451e-04_rb,2.4303e-04_rb,2.0131e-04_rb/)
      kao(:, 4, 2, 4) = (/ &
     &1.1814e-04_rb,2.8134e-04_rb,3.5342e-04_rb,3.8732e-04_rb,3.9443e-04_rb,3.8031e-04_rb, &
     &3.4829e-04_rb,2.8801e-04_rb,2.6066e-04_rb/)
      kao(:, 5, 2, 4) = (/ &
     &1.3198e-04_rb,3.2404e-04_rb,4.0415e-04_rb,4.4181e-04_rb,4.5121e-04_rb,4.4037e-04_rb, &
     &4.0877e-04_rb,3.5353e-04_rb,3.3227e-04_rb/)
      kao(:, 1, 3, 4) = (/ &
     &1.4443e-04_rb,2.3826e-04_rb,2.7329e-04_rb,2.8523e-04_rb,2.8239e-04_rb,2.6871e-04_rb, &
     &2.4140e-04_rb,1.7625e-04_rb,9.1057e-05_rb/)
      kao(:, 2, 3, 4) = (/ &
     &1.6500e-04_rb,2.7899e-04_rb,3.2197e-04_rb,3.3661e-04_rb,3.4010e-04_rb,3.2744e-04_rb, &
     &2.9069e-04_rb,2.1583e-04_rb,1.2870e-04_rb/)
      kao(:, 3, 3, 4) = (/ &
     &1.8759e-04_rb,3.2667e-04_rb,3.7258e-04_rb,3.9759e-04_rb,4.0496e-04_rb,3.8750e-04_rb, &
     &3.4189e-04_rb,2.6207e-04_rb,1.7564e-04_rb/)
      kao(:, 4, 3, 4) = (/ &
     &2.1411e-04_rb,3.7810e-04_rb,4.3373e-04_rb,4.6557e-04_rb,4.7150e-04_rb,4.5014e-04_rb, &
     &3.9858e-04_rb,3.1356e-04_rb,2.3208e-04_rb/)
      kao(:, 5, 3, 4) = (/ &
     &2.3870e-04_rb,4.2957e-04_rb,5.0123e-04_rb,5.3924e-04_rb,5.3944e-04_rb,5.1553e-04_rb, &
     &4.6343e-04_rb,3.7106e-04_rb,2.9980e-04_rb/)
      kao(:, 1, 4, 4) = (/ &
     &2.6076e-04_rb,3.4439e-04_rb,3.6366e-04_rb,3.6515e-04_rb,3.5256e-04_rb,3.2080e-04_rb, &
     &2.7636e-04_rb,2.0889e-04_rb,8.2321e-05_rb/)
      kao(:, 2, 4, 4) = (/ &
     &2.9760e-04_rb,4.0373e-04_rb,4.3154e-04_rb,4.3412e-04_rb,4.1663e-04_rb,3.8448e-04_rb, &
     &3.3798e-04_rb,2.5481e-04_rb,1.1139e-04_rb/)
      kao(:, 3, 4, 4) = (/ &
     &3.4031e-04_rb,4.7084e-04_rb,5.0815e-04_rb,5.1064e-04_rb,4.9099e-04_rb,4.5954e-04_rb, &
     &4.0644e-04_rb,3.0187e-04_rb,1.5330e-04_rb/)
      kao(:, 4, 4, 4) = (/ &
     &3.8682e-04_rb,5.4745e-04_rb,5.9070e-04_rb,5.9370e-04_rb,5.7774e-04_rb,5.4347e-04_rb, &
     &4.7190e-04_rb,3.5452e-04_rb,2.0604e-04_rb/)
      kao(:, 5, 4, 4) = (/ &
     &4.3959e-04_rb,6.2729e-04_rb,6.7728e-04_rb,6.8652e-04_rb,6.7377e-04_rb,6.2625e-04_rb, &
     &5.4307e-04_rb,4.1535e-04_rb,2.7101e-04_rb/)
      kao(:, 1, 5, 4) = (/ &
     &4.3137e-04_rb,4.9354e-04_rb,4.9901e-04_rb,4.7432e-04_rb,4.3778e-04_rb,3.8485e-04_rb, &
     &3.1938e-04_rb,2.2709e-04_rb,7.3559e-05_rb/)
      kao(:, 2, 5, 4) = (/ &
     &4.9217e-04_rb,5.7735e-04_rb,5.8525e-04_rb,5.6095e-04_rb,5.2221e-04_rb,4.6398e-04_rb, &
     &3.8313e-04_rb,2.8160e-04_rb,1.0296e-04_rb/)
      kao(:, 3, 5, 4) = (/ &
     &5.6924e-04_rb,6.7555e-04_rb,6.8298e-04_rb,6.6362e-04_rb,6.1781e-04_rb,5.4692e-04_rb, &
     &4.5981e-04_rb,3.4088e-04_rb,1.4005e-04_rb/)
      kao(:, 4, 5, 4) = (/ &
     &6.5162e-04_rb,7.8635e-04_rb,7.9830e-04_rb,7.7682e-04_rb,7.2322e-04_rb,6.4526e-04_rb, &
     &5.4580e-04_rb,4.0061e-04_rb,1.8482e-04_rb/)
      kao(:, 5, 5, 4) = (/ &
     &7.5040e-04_rb,9.0883e-04_rb,9.2599e-04_rb,8.9925e-04_rb,8.3896e-04_rb,7.5567e-04_rb, &
     &6.3867e-04_rb,4.6416e-04_rb,2.4367e-04_rb/)
      kao(:, 1, 6, 4) = (/ &
     &6.5259e-04_rb,6.7358e-04_rb,6.6023e-04_rb,6.1431e-04_rb,5.5026e-04_rb,4.6572e-04_rb, &
     &3.6933e-04_rb,2.4724e-04_rb,6.3454e-05_rb/)
      kao(:, 2, 6, 4) = (/ &
     &7.5346e-04_rb,7.9866e-04_rb,7.7727e-04_rb,7.2579e-04_rb,6.4756e-04_rb,5.5655e-04_rb, &
     &4.4395e-04_rb,3.0264e-04_rb,8.7333e-05_rb/)
      kao(:, 3, 6, 4) = (/ &
     &8.8029e-04_rb,9.4301e-04_rb,9.1678e-04_rb,8.5383e-04_rb,7.6672e-04_rb,6.6157e-04_rb, &
     &5.3248e-04_rb,3.6645e-04_rb,1.2006e-04_rb/)
      kao(:, 4, 6, 4) = (/ &
     &1.0324e-03_rb,1.1112e-03_rb,1.0784e-03_rb,1.0064e-03_rb,9.0982e-04_rb,7.8126e-04_rb, &
     &6.2832e-04_rb,4.4008e-04_rb,1.6674e-04_rb/)
      kao(:, 5, 6, 4) = (/ &
     &1.1873e-03_rb,1.2942e-03_rb,1.2623e-03_rb,1.1793e-03_rb,1.0646e-03_rb,9.2296e-04_rb, &
     &7.3952e-04_rb,5.1891e-04_rb,2.2335e-04_rb/)
      kao(:, 1, 7, 4) = (/ &
     &1.0039e-03_rb,9.6265e-04_rb,8.9986e-04_rb,8.1533e-04_rb,7.1416e-04_rb,5.9439e-04_rb, &
     &4.5276e-04_rb,2.8651e-04_rb,6.5221e-05_rb/)
      kao(:, 2, 7, 4) = (/ &
     &1.1698e-03_rb,1.1414e-03_rb,1.0686e-03_rb,9.8012e-04_rb,8.6148e-04_rb,7.1658e-04_rb, &
     &5.4576e-04_rb,3.5076e-04_rb,8.8064e-05_rb/)
      kao(:, 3, 7, 4) = (/ &
     &1.3785e-03_rb,1.3654e-03_rb,1.2898e-03_rb,1.1834e-03_rb,1.0346e-03_rb,8.5792e-04_rb, &
     &6.6115e-04_rb,4.2986e-04_rb,1.2055e-04_rb/)
      kao(:, 4, 7, 4) = (/ &
     &1.6296e-03_rb,1.6337e-03_rb,1.5460e-03_rb,1.4118e-03_rb,1.2372e-03_rb,1.0278e-03_rb, &
     &7.9671e-04_rb,5.1331e-04_rb,1.5903e-04_rb/)
      kao(:, 5, 7, 4) = (/ &
     &1.9001e-03_rb,1.9217e-03_rb,1.8220e-03_rb,1.6571e-03_rb,1.4527e-03_rb,1.2164e-03_rb, &
     &9.4433e-04_rb,6.1289e-04_rb,2.0363e-04_rb/)
      kao(:, 1, 8, 4) = (/ &
     &1.8226e-03_rb,1.6514e-03_rb,1.4839e-03_rb,1.2932e-03_rb,1.0982e-03_rb,8.8723e-04_rb, &
     &6.5582e-04_rb,3.8869e-04_rb,7.0418e-05_rb/)
      kao(:, 2, 8, 4) = (/ &
     &2.1183e-03_rb,1.9484e-03_rb,1.7717e-03_rb,1.5557e-03_rb,1.3178e-03_rb,1.0649e-03_rb, &
     &7.9415e-04_rb,4.7222e-04_rb,9.8173e-05_rb/)
      kao(:, 3, 8, 4) = (/ &
     &2.5295e-03_rb,2.3395e-03_rb,2.1347e-03_rb,1.8817e-03_rb,1.6058e-03_rb,1.3046e-03_rb, &
     &9.6185e-04_rb,5.7415e-04_rb,1.2861e-04_rb/)
      kao(:, 4, 8, 4) = (/ &
     &3.0151e-03_rb,2.8208e-03_rb,2.5722e-03_rb,2.2683e-03_rb,1.9222e-03_rb,1.5620e-03_rb, &
     &1.1535e-03_rb,7.0406e-04_rb,1.7432e-04_rb/)
      kao(:, 5, 8, 4) = (/ &
     &3.5096e-03_rb,3.3177e-03_rb,3.0150e-03_rb,2.6542e-03_rb,2.2606e-03_rb,1.8422e-03_rb, &
     &1.3738e-03_rb,8.5046e-04_rb,2.3260e-04_rb/)
      kao(:, 1, 9, 4) = (/ &
     &7.9941e-03_rb,7.0328e-03_rb,6.0711e-03_rb,5.1105e-03_rb,4.1565e-03_rb,3.2070e-03_rb, &
     &2.2152e-03_rb,1.1971e-03_rb,7.9032e-05_rb/)
      kao(:, 2, 9, 4) = (/ &
     &8.9783e-03_rb,7.9063e-03_rb,6.8488e-03_rb,5.7943e-03_rb,4.7410e-03_rb,3.6528e-03_rb, &
     &2.5467e-03_rb,1.3852e-03_rb,1.1999e-04_rb/)
      kao(:, 3, 9, 4) = (/ &
     &1.0198e-02_rb,9.0183e-03_rb,7.8510e-03_rb,6.6811e-03_rb,5.4839e-03_rb,4.2200e-03_rb, &
     &2.9428e-03_rb,1.6034e-03_rb,1.6821e-04_rb/)
      kao(:, 4, 9, 4) = (/ &
     &1.1749e-02_rb,1.0423e-02_rb,9.0948e-03_rb,7.7411e-03_rb,6.3357e-03_rb,4.8731e-03_rb, &
     &3.3881e-03_rb,1.8608e-03_rb,2.4632e-04_rb/)
      kao(:, 5, 9, 4) = (/ &
     &1.3350e-02_rb,1.1910e-02_rb,1.0410e-02_rb,8.8879e-03_rb,7.2904e-03_rb,5.6383e-03_rb, &
     &3.9275e-03_rb,2.1612e-03_rb,3.3903e-04_rb/)
      kao(:, 1,10, 4) = (/ &
     &4.5801e-02_rb,4.0109e-02_rb,3.4395e-02_rb,2.8682e-02_rb,2.2984e-02_rb,1.7267e-02_rb, &
     &1.1579e-02_rb,5.9235e-03_rb,6.3484e-05_rb/)
      kao(:, 2,10, 4) = (/ &
     &4.9659e-02_rb,4.3500e-02_rb,3.7321e-02_rb,3.1127e-02_rb,2.4948e-02_rb,1.8778e-02_rb, &
     &1.2636e-02_rb,6.5101e-03_rb,9.5466e-05_rb/)
      kao(:, 3,10, 4) = (/ &
     &5.4357e-02_rb,4.7634e-02_rb,4.0874e-02_rb,3.4140e-02_rb,2.7418e-02_rb,2.0676e-02_rb, &
     &1.3967e-02_rb,7.2134e-03_rb,2.1246e-04_rb/)
      kao(:, 4,10, 4) = (/ &
     &6.0291e-02_rb,5.2843e-02_rb,4.5373e-02_rb,3.7958e-02_rb,3.0510e-02_rb,2.3082e-02_rb, &
     &1.5672e-02_rb,8.0960e-03_rb,3.5404e-04_rb/)
      kao(:, 5,10, 4) = (/ &
     &6.7661e-02_rb,5.9365e-02_rb,5.1042e-02_rb,4.2745e-02_rb,3.4451e-02_rb,2.6105e-02_rb, &
     &1.7693e-02_rb,9.1211e-03_rb,4.7192e-04_rb/)
      kao(:, 1,11, 4) = (/ &
     &9.0702e-02_rb,7.9394e-02_rb,6.8068e-02_rb,5.6747e-02_rb,4.5417e-02_rb,3.4107e-02_rb, &
     &2.2797e-02_rb,1.1511e-02_rb,7.5853e-05_rb/)
      kao(:, 2,11, 4) = (/ &
     &9.9244e-02_rb,8.6879e-02_rb,7.4483e-02_rb,6.2100e-02_rb,4.9720e-02_rb,3.7362e-02_rb, &
     &2.5008e-02_rb,1.2672e-02_rb,1.0256e-04_rb/)
      kao(:, 3,11, 4) = (/ &
     &1.0852e-01_rb,9.5023e-02_rb,8.1487e-02_rb,6.7960e-02_rb,5.4445e-02_rb,4.0919e-02_rb, &
     &2.7423e-02_rb,1.3982e-02_rb,1.6509e-04_rb/)
      kao(:, 4,11, 4) = (/ &
     &1.1956e-01_rb,1.0470e-01_rb,8.9828e-02_rb,7.4940e-02_rb,6.0117e-02_rb,4.5252e-02_rb, &
     &3.0437e-02_rb,1.5543e-02_rb,3.1661e-04_rb/)
      kao(:, 5,11, 4) = (/ &
     &1.3039e-01_rb,1.1429e-01_rb,9.8079e-02_rb,8.1967e-02_rb,6.5782e-02_rb,4.9565e-02_rb, &
     &3.3397e-02_rb,1.7070e-02_rb,5.0645e-04_rb/)
      kao(:, 1,12, 4) = (/ &
     &1.2021e-01_rb,1.0519e-01_rb,9.0195e-02_rb,7.5171e-02_rb,6.0154e-02_rb,4.5148e-02_rb, &
     &3.0151e-02_rb,1.5167e-02_rb,6.7632e-05_rb/)
      kao(:, 2,12, 4) = (/ &
     &1.3016e-01_rb,1.1394e-01_rb,9.7701e-02_rb,8.1469e-02_rb,6.5222e-02_rb,4.8981e-02_rb, &
     &3.2777e-02_rb,1.6564e-02_rb,9.6691e-05_rb/)
      kao(:, 3,12, 4) = (/ &
     &1.4257e-01_rb,1.2479e-01_rb,1.0706e-01_rb,8.9271e-02_rb,7.1459e-02_rb,5.3715e-02_rb, &
     &3.5967e-02_rb,1.8266e-02_rb,1.8650e-04_rb/)
      kao(:, 4,12, 4) = (/ &
     &1.5648e-01_rb,1.3701e-01_rb,1.1748e-01_rb,9.8018e-02_rb,7.8554e-02_rb,5.9075e-02_rb, &
     &3.9673e-02_rb,2.0186e-02_rb,3.5387e-04_rb/)
      kao(:, 5,12, 4) = (/ &
     &1.6740e-01_rb,1.4658e-01_rb,1.2570e-01_rb,1.0496e-01_rb,8.4156e-02_rb,6.3336e-02_rb, &
     &4.2553e-02_rb,2.1604e-02_rb,5.3320e-04_rb/)
      kao(:, 1,13, 4) = (/ &
     &1.2573e-01_rb,1.1000e-01_rb,9.4314e-02_rb,7.8615e-02_rb,6.2929e-02_rb,4.7219e-02_rb, &
     &3.1536e-02_rb,1.5865e-02_rb,5.0474e-05_rb/)
      kao(:, 2,13, 4) = (/ &
     &1.3569e-01_rb,1.1873e-01_rb,1.0180e-01_rb,8.4863e-02_rb,6.7917e-02_rb,5.1000e-02_rb, &
     &3.4100e-02_rb,1.7211e-02_rb,1.0366e-04_rb/)
      kao(:, 3,13, 4) = (/ &
     &1.4578e-01_rb,1.2755e-01_rb,1.0936e-01_rb,9.1174e-02_rb,7.3030e-02_rb,5.4868e-02_rb, &
     &3.6725e-02_rb,1.8584e-02_rb,2.4417e-04_rb/)
      kao(:, 4,13, 4) = (/ &
     &1.5300e-01_rb,1.3393e-01_rb,1.1485e-01_rb,9.5797e-02_rb,7.6755e-02_rb,5.7691e-02_rb, &
     &3.8700e-02_rb,1.9588e-02_rb,3.9463e-04_rb/)
      kao(:, 5,13, 4) = (/ &
     &1.5948e-01_rb,1.3959e-01_rb,1.1975e-01_rb,9.9939e-02_rb,8.0103e-02_rb,6.0267e-02_rb, &
     &4.0442e-02_rb,2.0476e-02_rb,5.2489e-04_rb/)
      kao(:, 1, 1, 5) = (/ &
     &3.0730e-04_rb,4.0869e-04_rb,4.2558e-04_rb,4.1836e-04_rb,3.9519e-04_rb,3.6372e-04_rb, &
     &2.8694e-04_rb,2.6584e-04_rb,2.1801e-04_rb/)
      kao(:, 2, 1, 5) = (/ &
     &3.2538e-04_rb,4.3862e-04_rb,4.6062e-04_rb,4.5800e-04_rb,4.4348e-04_rb,4.2125e-04_rb, &
     &3.6505e-04_rb,3.2613e-04_rb,2.9396e-04_rb/)
      kao(:, 3, 1, 5) = (/ &
     &3.3965e-04_rb,4.6778e-04_rb,5.0124e-04_rb,5.0588e-04_rb,5.0627e-04_rb,4.8837e-04_rb, &
     &4.4591e-04_rb,3.9091e-04_rb,3.8601e-04_rb/)
      kao(:, 4, 1, 5) = (/ &
     &3.5305e-04_rb,5.0005e-04_rb,5.4810e-04_rb,5.6545e-04_rb,5.7634e-04_rb,5.6350e-04_rb, &
     &5.3398e-04_rb,4.9322e-04_rb,4.9703e-04_rb/)
      kao(:, 5, 1, 5) = (/ &
     &3.6843e-04_rb,5.3559e-04_rb,6.0074e-04_rb,6.3388e-04_rb,6.5476e-04_rb,6.5121e-04_rb, &
     &6.3615e-04_rb,6.1596e-04_rb,6.2900e-04_rb/)
      kao(:, 1, 2, 5) = (/ &
     &4.2086e-04_rb,5.1417e-04_rb,5.1972e-04_rb,4.9790e-04_rb,4.6126e-04_rb,4.1005e-04_rb, &
     &3.3841e-04_rb,2.3086e-04_rb,1.8396e-04_rb/)
      kao(:, 2, 2, 5) = (/ &
     &4.4142e-04_rb,5.4838e-04_rb,5.5918e-04_rb,5.4271e-04_rb,5.1015e-04_rb,4.6371e-04_rb, &
     &3.9686e-04_rb,2.8420e-04_rb,2.4486e-04_rb/)
      kao(:, 3, 2, 5) = (/ &
     &4.6438e-04_rb,5.8432e-04_rb,6.0102e-04_rb,5.9651e-04_rb,5.6920e-04_rb,5.3041e-04_rb, &
     &4.5976e-04_rb,3.7342e-04_rb,3.2265e-04_rb/)
      kao(:, 4, 2, 5) = (/ &
     &4.8279e-04_rb,6.2418e-04_rb,6.4960e-04_rb,6.5695e-04_rb,6.3988e-04_rb,6.0274e-04_rb, &
     &5.3017e-04_rb,4.6801e-04_rb,4.1702e-04_rb/)
      kao(:, 5, 2, 5) = (/ &
     &4.9938e-04_rb,6.6314e-04_rb,7.0911e-04_rb,7.2552e-04_rb,7.1847e-04_rb,6.8356e-04_rb, &
     &6.1801e-04_rb,5.7242e-04_rb,5.3937e-04_rb/)
      kao(:, 1, 3, 5) = (/ &
     &7.5073e-04_rb,8.1743e-04_rb,7.9414e-04_rb,7.4039e-04_rb,6.6425e-04_rb,5.6902e-04_rb, &
     &4.5211e-04_rb,2.9071e-04_rb,1.7629e-04_rb/)
      kao(:, 2, 3, 5) = (/ &
     &7.9242e-04_rb,8.7156e-04_rb,8.5005e-04_rb,8.0165e-04_rb,7.1947e-04_rb,6.2467e-04_rb, &
     &5.0816e-04_rb,3.5038e-04_rb,2.3642e-04_rb/)
      kao(:, 3, 3, 5) = (/ &
     &8.3043e-04_rb,9.2555e-04_rb,9.1453e-04_rb,8.6093e-04_rb,7.8464e-04_rb,6.9275e-04_rb, &
     &5.7807e-04_rb,4.1288e-04_rb,3.0691e-04_rb/)
      kao(:, 4, 3, 5) = (/ &
     &8.6053e-04_rb,9.7674e-04_rb,9.7765e-04_rb,9.2813e-04_rb,8.5853e-04_rb,7.7377e-04_rb, &
     &6.5702e-04_rb,4.8317e-04_rb,3.9911e-04_rb/)
      kao(:, 5, 3, 5) = (/ &
     &8.9084e-04_rb,1.0351e-03_rb,1.0435e-03_rb,1.0049e-03_rb,9.4814e-04_rb,8.6574e-04_rb, &
     &7.4637e-04_rb,5.6491e-04_rb,5.0649e-04_rb/)
      kao(:, 1, 4, 5) = (/ &
     &1.3691e-03_rb,1.3420e-03_rb,1.2789e-03_rb,1.1571e-03_rb,1.0054e-03_rb,8.3457e-04_rb, &
     &6.3859e-04_rb,4.0761e-04_rb,1.5090e-04_rb/)
      kao(:, 2, 4, 5) = (/ &
     &1.4497e-03_rb,1.4458e-03_rb,1.3693e-03_rb,1.2399e-03_rb,1.0886e-03_rb,9.0944e-04_rb, &
     &6.9922e-04_rb,4.5891e-04_rb,2.1864e-04_rb/)
      kao(:, 3, 4, 5) = (/ &
     &1.5092e-03_rb,1.5288e-03_rb,1.4492e-03_rb,1.3231e-03_rb,1.1702e-03_rb,9.8222e-04_rb, &
     &7.7013e-04_rb,5.2474e-04_rb,3.0407e-04_rb/)
      kao(:, 4, 4, 5) = (/ &
     &1.5602e-03_rb,1.6068e-03_rb,1.5312e-03_rb,1.4087e-03_rb,1.2594e-03_rb,1.0655e-03_rb, &
     &8.5735e-04_rb,5.9578e-04_rb,3.9268e-04_rb/)
      kao(:, 5, 4, 5) = (/ &
     &1.6108e-03_rb,1.6853e-03_rb,1.6164e-03_rb,1.5057e-03_rb,1.3492e-03_rb,1.1682e-03_rb, &
     &9.6058e-04_rb,6.7959e-04_rb,4.9525e-04_rb/)
      kao(:, 1, 5, 5) = (/ &
     &2.3434e-03_rb,2.1631e-03_rb,1.9859e-03_rb,1.7683e-03_rb,1.5141e-03_rb,1.2226e-03_rb, &
     &8.9923e-04_rb,5.4477e-04_rb,1.2575e-04_rb/)
      kao(:, 2, 5, 5) = (/ &
     &2.4752e-03_rb,2.3171e-03_rb,2.1360e-03_rb,1.9016e-03_rb,1.6214e-03_rb,1.3108e-03_rb, &
     &9.7831e-04_rb,5.9705e-04_rb,1.8419e-04_rb/)
      kao(:, 3, 5, 5) = (/ &
     &2.5685e-03_rb,2.4387e-03_rb,2.2507e-03_rb,2.0028e-03_rb,1.7171e-03_rb,1.4030e-03_rb, &
     &1.0542e-03_rb,6.5942e-04_rb,2.6114e-04_rb/)
      kao(:, 4, 5, 5) = (/ &
     &2.6469e-03_rb,2.5385e-03_rb,2.3599e-03_rb,2.1047e-03_rb,1.8139e-03_rb,1.4974e-03_rb, &
     &1.1404e-03_rb,7.3650e-04_rb,3.6173e-04_rb/)
      kao(:, 5, 5, 5) = (/ &
     &2.7111e-03_rb,2.6448e-03_rb,2.4645e-03_rb,2.2107e-03_rb,1.9222e-03_rb,1.6048e-03_rb, &
     &1.2367e-03_rb,8.2822e-04_rb,4.7124e-04_rb/)
      kao(:, 1, 6, 5) = (/ &
     &3.6870e-03_rb,3.3081e-03_rb,2.9535e-03_rb,2.5781e-03_rb,2.1710e-03_rb,1.7353e-03_rb, &
     &1.2527e-03_rb,7.1995e-04_rb,1.0649e-04_rb/)
      kao(:, 2, 6, 5) = (/ &
     &3.8867e-03_rb,3.5260e-03_rb,3.1721e-03_rb,2.7718e-03_rb,2.3392e-03_rb,1.8682e-03_rb, &
     &1.3468e-03_rb,7.8719e-04_rb,1.5668e-04_rb/)
      kao(:, 3, 6, 5) = (/ &
     &4.0284e-03_rb,3.6872e-03_rb,3.3334e-03_rb,2.9169e-03_rb,2.4661e-03_rb,1.9733e-03_rb, &
     &1.4372e-03_rb,8.5750e-04_rb,2.1580e-04_rb/)
      kao(:, 4, 6, 5) = (/ &
     &4.1355e-03_rb,3.8269e-03_rb,3.4723e-03_rb,3.0564e-03_rb,2.5962e-03_rb,2.0858e-03_rb, &
     &1.5388e-03_rb,9.2919e-04_rb,2.8921e-04_rb/)
      kao(:, 5, 6, 5) = (/ &
     &4.2530e-03_rb,3.9771e-03_rb,3.6076e-03_rb,3.1973e-03_rb,2.7175e-03_rb,2.1980e-03_rb, &
     &1.6503e-03_rb,1.0123e-03_rb,3.9933e-04_rb/)
      kao(:, 1, 7, 5) = (/ &
     &5.9303e-03_rb,5.2537e-03_rb,4.6038e-03_rb,3.9568e-03_rb,3.2763e-03_rb,2.5699e-03_rb, &
     &1.8266e-03_rb,1.0238e-03_rb,9.0759e-05_rb/)
      kao(:, 2, 7, 5) = (/ &
     &6.2719e-03_rb,5.5854e-03_rb,4.9264e-03_rb,4.2403e-03_rb,3.5196e-03_rb,2.7643e-03_rb, &
     &1.9714e-03_rb,1.1058e-03_rb,1.3173e-04_rb/)
      kao(:, 3, 7, 5) = (/ &
     &6.5193e-03_rb,5.8407e-03_rb,5.1576e-03_rb,4.4431e-03_rb,3.6909e-03_rb,2.9114e-03_rb, &
     &2.0825e-03_rb,1.1806e-03_rb,1.8536e-04_rb/)
      kao(:, 4, 7, 5) = (/ &
     &6.6989e-03_rb,6.0314e-03_rb,5.3552e-03_rb,4.6281e-03_rb,3.8605e-03_rb,3.0624e-03_rb, &
     &2.1992e-03_rb,1.2697e-03_rb,2.6787e-04_rb/)
      kao(:, 5, 7, 5) = (/ &
     &6.9450e-03_rb,6.3008e-03_rb,5.6152e-03_rb,4.8622e-03_rb,4.0675e-03_rb,3.2291e-03_rb, &
     &2.3204e-03_rb,1.3675e-03_rb,3.5672e-04_rb/)
      kao(:, 1, 8, 5) = (/ &
     &1.1078e-02_rb,9.7294e-03_rb,8.3981e-03_rb,7.0957e-03_rb,5.7917e-03_rb,4.4718e-03_rb, &
     &3.1091e-03_rb,1.6886e-03_rb,1.1294e-04_rb/)
      kao(:, 2, 8, 5) = (/ &
     &1.1717e-02_rb,1.0310e-02_rb,8.9257e-03_rb,7.5752e-03_rb,6.2113e-03_rb,4.8050e-03_rb, &
     &3.3497e-03_rb,1.8274e-03_rb,1.4724e-04_rb/)
      kao(:, 3, 8, 5) = (/ &
     &1.2207e-02_rb,1.0778e-02_rb,9.3683e-03_rb,7.9668e-03_rb,6.5278e-03_rb,5.0484e-03_rb, &
     &3.5310e-03_rb,1.9391e-03_rb,2.0653e-04_rb/)
      kao(:, 4, 8, 5) = (/ &
     &1.2674e-02_rb,1.1230e-02_rb,9.8021e-03_rb,8.3609e-03_rb,6.8807e-03_rb,5.3359e-03_rb, &
     &3.7364e-03_rb,2.0615e-03_rb,2.8343e-04_rb/)
      kao(:, 5, 8, 5) = (/ &
     &1.3265e-02_rb,1.1790e-02_rb,1.0322e-02_rb,8.8148e-03_rb,7.2381e-03_rb,5.6108e-03_rb, &
     &3.9433e-03_rb,2.1912e-03_rb,3.7254e-04_rb/)
      kao(:, 1, 9, 5) = (/ &
     &3.7052e-02_rb,3.2437e-02_rb,2.7835e-02_rb,2.3250e-02_rb,1.8658e-02_rb,1.4080e-02_rb, &
     &9.5558e-03_rb,4.9966e-03_rb,1.9423e-04_rb/)
      kao(:, 2, 9, 5) = (/ &
     &3.9539e-02_rb,3.4640e-02_rb,2.9744e-02_rb,2.4857e-02_rb,1.9981e-02_rb,1.5146e-02_rb, &
     &1.0322e-02_rb,5.4117e-03_rb,2.4661e-04_rb/)
      kao(:, 3, 9, 5) = (/ &
     &4.1918e-02_rb,3.6737e-02_rb,3.1561e-02_rb,2.6402e-02_rb,2.1269e-02_rb,1.6195e-02_rb, &
     &1.1056e-02_rb,5.7963e-03_rb,3.2011e-04_rb/)
      kao(:, 4, 9, 5) = (/ &
     &4.4100e-02_rb,3.8673e-02_rb,3.3273e-02_rb,2.7895e-02_rb,2.2554e-02_rb,1.7214e-02_rb, &
     &1.1779e-02_rb,6.1756e-03_rb,4.1260e-04_rb/)
      kao(:, 5, 9, 5) = (/ &
     &4.6405e-02_rb,4.0709e-02_rb,3.5073e-02_rb,2.9453e-02_rb,2.3852e-02_rb,1.8195e-02_rb, &
     &1.2445e-02_rb,6.5667e-03_rb,5.2938e-04_rb/)
      kao(:, 1,10, 5) = (/ &
     &1.5260e-01_rb,1.3356e-01_rb,1.1452e-01_rb,9.5486e-02_rb,7.6440e-02_rb,5.7459e-02_rb, &
     &3.8411e-02_rb,1.9334e-02_rb,1.8781e-04_rb/)
      kao(:, 2,10, 5) = (/ &
     &1.6192e-01_rb,1.4171e-01_rb,1.2154e-01_rb,1.0139e-01_rb,8.1225e-02_rb,6.1032e-02_rb, &
     &4.0817e-02_rb,2.0639e-02_rb,3.0020e-04_rb/)
      kao(:, 3,10, 5) = (/ &
     &1.6952e-01_rb,1.4839e-01_rb,1.2731e-01_rb,1.0623e-01_rb,8.5090e-02_rb,6.3990e-02_rb, &
     &4.2882e-02_rb,2.1810e-02_rb,3.7223e-04_rb/)
      kao(:, 4,10, 5) = (/ &
     &1.7675e-01_rb,1.5478e-01_rb,1.3285e-01_rb,1.1085e-01_rb,8.8843e-02_rb,6.6852e-02_rb, &
     &4.4829e-02_rb,2.2833e-02_rb,4.6711e-04_rb/)
      kao(:, 5,10, 5) = (/ &
     &1.8239e-01_rb,1.5968e-01_rb,1.3704e-01_rb,1.1434e-01_rb,9.1701e-02_rb,6.9058e-02_rb, &
     &4.6429e-02_rb,2.3733e-02_rb,6.6086e-04_rb/)
      kao(:, 1,11, 5) = (/ &
     &2.7058e-01_rb,2.3683e-01_rb,2.0302e-01_rb,1.6928e-01_rb,1.3552e-01_rb,1.0170e-01_rb, &
     &6.7859e-02_rb,3.4017e-02_rb,1.5001e-04_rb/)
      kao(:, 2,11, 5) = (/ &
     &2.7834e-01_rb,2.4359e-01_rb,2.0889e-01_rb,1.7416e-01_rb,1.3942e-01_rb,1.0462e-01_rb, &
     &6.9857e-02_rb,3.5072e-02_rb,2.9190e-04_rb/)
      kao(:, 3,11, 5) = (/ &
     &2.8582e-01_rb,2.5022e-01_rb,2.1462e-01_rb,1.7895e-01_rb,1.4327e-01_rb,1.0754e-01_rb, &
     &7.1838e-02_rb,3.6157e-02_rb,3.9721e-04_rb/)
      kao(:, 4,11, 5) = (/ &
     &2.8898e-01_rb,2.5301e-01_rb,2.1700e-01_rb,1.8097e-01_rb,1.4487e-01_rb,1.0884e-01_rb, &
     &7.2766e-02_rb,3.6699e-02_rb,5.3597e-04_rb/)
      kao(:, 5,11, 5) = (/ &
     &2.9536e-01_rb,2.5857e-01_rb,2.2177e-01_rb,1.8491e-01_rb,1.4809e-01_rb,1.1136e-01_rb, &
     &7.4481e-02_rb,3.7679e-02_rb,7.1552e-04_rb/)
      kao(:, 1,12, 5) = (/ &
     &3.2683e-01_rb,2.8605e-01_rb,2.4525e-01_rb,2.0441e-01_rb,1.6361e-01_rb,1.2282e-01_rb, &
     &8.1893e-02_rb,4.1016e-02_rb,1.4076e-04_rb/)
      kao(:, 2,12, 5) = (/ &
     &3.3106e-01_rb,2.8971e-01_rb,2.4839e-01_rb,2.0704e-01_rb,1.6570e-01_rb,1.2434e-01_rb, &
     &8.2938e-02_rb,4.1600e-02_rb,2.8033e-04_rb/)
      kao(:, 3,12, 5) = (/ &
     &3.3746e-01_rb,2.9539e-01_rb,2.5326e-01_rb,2.1111e-01_rb,1.6904e-01_rb,1.2682e-01_rb, &
     &8.4697e-02_rb,4.2481e-02_rb,3.8254e-04_rb/)
      kao(:, 4,12, 5) = (/ &
     &3.4047e-01_rb,2.9813e-01_rb,2.5560e-01_rb,2.1310e-01_rb,1.7062e-01_rb,1.2812e-01_rb, &
     &8.5526e-02_rb,4.3047e-02_rb,5.1612e-04_rb/)
      kao(:, 5,12, 5) = (/ &
     &3.4523e-01_rb,3.0226e-01_rb,2.5919e-01_rb,2.1613e-01_rb,1.7310e-01_rb,1.3003e-01_rb, &
     &8.6933e-02_rb,4.3928e-02_rb,7.3625e-04_rb/)
      kao(:, 1,13, 5) = (/ &
     &2.9792e-01_rb,2.6068e-01_rb,2.2349e-01_rb,1.8628e-01_rb,1.4905e-01_rb,1.1181e-01_rb, &
     &7.4561e-02_rb,3.7329e-02_rb,1.5388e-04_rb/)
      kao(:, 2,13, 5) = (/ &
     &3.0081e-01_rb,2.6326e-01_rb,2.2566e-01_rb,1.8811e-01_rb,1.5050e-01_rb,1.1291e-01_rb, &
     &7.5310e-02_rb,3.7768e-02_rb,2.8934e-04_rb/)
      kao(:, 3,13, 5) = (/ &
     &3.0121e-01_rb,2.6362e-01_rb,2.2602e-01_rb,1.8840e-01_rb,1.5077e-01_rb,1.1318e-01_rb, &
     &7.5585e-02_rb,3.7991e-02_rb,3.3166e-04_rb/)
      kao(:, 4,13, 5) = (/ &
     &3.0495e-01_rb,2.6698e-01_rb,2.2888e-01_rb,1.9082e-01_rb,1.5277e-01_rb,1.1476e-01_rb, &
     &7.6636e-02_rb,3.8673e-02_rb,4.6400e-04_rb/)
      kao(:, 5,13, 5) = (/ &
     &3.1093e-01_rb,2.7226e-01_rb,2.3342e-01_rb,1.9467e-01_rb,1.5596e-01_rb,1.1716e-01_rb, &
     &7.8384e-02_rb,3.9602e-02_rb,6.9078e-04_rb/)
      kao(:, 1, 1, 6) = (/ &
     &6.7719e-04_rb,7.1472e-04_rb,7.1797e-04_rb,6.8628e-04_rb,6.3509e-04_rb,5.7059e-04_rb, &
     &5.0987e-04_rb,3.4938e-04_rb,4.0895e-04_rb/)
      kao(:, 2, 1, 6) = (/ &
     &6.7482e-04_rb,7.4520e-04_rb,7.5758e-04_rb,7.4570e-04_rb,7.0620e-04_rb,6.5805e-04_rb, &
     &6.1645e-04_rb,4.7281e-04_rb,5.4801e-04_rb/)
      kao(:, 3, 1, 6) = (/ &
     &6.7924e-04_rb,7.8313e-04_rb,8.0713e-04_rb,8.1739e-04_rb,7.9231e-04_rb,7.6723e-04_rb, &
     &7.3523e-04_rb,6.1861e-04_rb,7.0683e-04_rb/)
      kao(:, 4, 1, 6) = (/ &
     &6.8649e-04_rb,8.2657e-04_rb,8.7129e-04_rb,9.0685e-04_rb,8.9681e-04_rb,8.9801e-04_rb, &
     &8.6860e-04_rb,7.7360e-04_rb,8.9309e-04_rb/)
      kao(:, 5, 1, 6) = (/ &
     &6.9142e-04_rb,8.7359e-04_rb,9.5301e-04_rb,1.0075e-03_rb,1.0203e-03_rb,1.0461e-03_rb, &
     &1.0218e-03_rb,9.5114e-04_rb,1.0965e-03_rb/)
      kao(:, 1, 2, 6) = (/ &
     &9.6029e-04_rb,9.5330e-04_rb,9.1502e-04_rb,8.4442e-04_rb,7.4240e-04_rb,6.3131e-04_rb, &
     &5.2007e-04_rb,3.7829e-04_rb,3.5691e-04_rb/)
      kao(:, 2, 2, 6) = (/ &
     &9.5985e-04_rb,9.8942e-04_rb,9.6445e-04_rb,8.9433e-04_rb,8.0300e-04_rb,7.0134e-04_rb, &
     &6.1315e-04_rb,5.0159e-04_rb,4.8663e-04_rb/)
      kao(:, 3, 2, 6) = (/ &
     &9.6320e-04_rb,1.0310e-03_rb,1.0191e-03_rb,9.5205e-04_rb,8.7761e-04_rb,7.9453e-04_rb, &
     &7.3253e-04_rb,6.2365e-04_rb,6.4547e-04_rb/)
      kao(:, 4, 2, 6) = (/ &
     &9.7399e-04_rb,1.0755e-03_rb,1.0817e-03_rb,1.0333e-03_rb,9.7218e-04_rb,9.1068e-04_rb, &
     &8.7686e-04_rb,7.5862e-04_rb,8.3212e-04_rb/)
      kao(:, 5, 2, 6) = (/ &
     &9.8138e-04_rb,1.1324e-03_rb,1.1507e-03_rb,1.1333e-03_rb,1.0937e-03_rb,1.0509e-03_rb, &
     &1.0310e-03_rb,9.0559e-04_rb,1.0246e-03_rb/)
      kao(:, 1, 3, 6) = (/ &
     &1.8119e-03_rb,1.6817e-03_rb,1.5499e-03_rb,1.3740e-03_rb,1.1743e-03_rb,9.5554e-04_rb, &
     &7.1656e-04_rb,4.7034e-04_rb,2.9777e-04_rb/)
      kao(:, 2, 3, 6) = (/ &
     &1.8107e-03_rb,1.7310e-03_rb,1.6189e-03_rb,1.4391e-03_rb,1.2510e-03_rb,1.0204e-03_rb, &
     &7.8709e-04_rb,5.5271e-04_rb,4.0666e-04_rb/)
      kao(:, 3, 3, 6) = (/ &
     &1.8217e-03_rb,1.7823e-03_rb,1.6795e-03_rb,1.5235e-03_rb,1.3331e-03_rb,1.1033e-03_rb, &
     &8.7727e-04_rb,6.6343e-04_rb,5.5347e-04_rb/)
      kao(:, 4, 3, 6) = (/ &
     &1.8325e-03_rb,1.8451e-03_rb,1.7507e-03_rb,1.6178e-03_rb,1.4289e-03_rb,1.2102e-03_rb, &
     &9.9121e-04_rb,7.9875e-04_rb,7.2923e-04_rb/)
      kao(:, 5, 3, 6) = (/ &
     &1.8499e-03_rb,1.9065e-03_rb,1.8500e-03_rb,1.7218e-03_rb,1.5483e-03_rb,1.3411e-03_rb, &
     &1.1326e-03_rb,9.4539e-04_rb,9.3725e-04_rb/)
      kao(:, 1, 4, 6) = (/ &
     &3.4114e-03_rb,3.0491e-03_rb,2.7140e-03_rb,2.3662e-03_rb,1.9783e-03_rb,1.5658e-03_rb, &
     &1.1300e-03_rb,6.5946e-04_rb,2.6201e-04_rb/)
      kao(:, 2, 4, 6) = (/ &
     &3.4376e-03_rb,3.0989e-03_rb,2.7962e-03_rb,2.4537e-03_rb,2.0666e-03_rb,1.6491e-03_rb, &
     &1.2098e-03_rb,7.2995e-04_rb,3.5441e-04_rb/)
      kao(:, 3, 4, 6) = (/ &
     &3.4485e-03_rb,3.1733e-03_rb,2.8848e-03_rb,2.5421e-03_rb,2.1715e-03_rb,1.7580e-03_rb, &
     &1.3056e-03_rb,8.1745e-04_rb,4.7057e-04_rb/)
      kao(:, 4, 4, 6) = (/ &
     &3.4770e-03_rb,3.2632e-03_rb,3.0027e-03_rb,2.6508e-03_rb,2.2906e-03_rb,1.8839e-03_rb, &
     &1.4242e-03_rb,9.3156e-04_rb,6.1906e-04_rb/)
      kao(:, 5, 4, 6) = (/ &
     &3.5195e-03_rb,3.3661e-03_rb,3.1261e-03_rb,2.7792e-03_rb,2.4277e-03_rb,2.0261e-03_rb, &
     &1.5634e-03_rb,1.0703e-03_rb,8.1465e-04_rb/)
      kao(:, 1, 5, 6) = (/ &
     &5.7502e-03_rb,5.0807e-03_rb,4.4217e-03_rb,3.7897e-03_rb,3.1287e-03_rb,2.4547e-03_rb, &
     &1.7341e-03_rb,9.6239e-04_rb,2.4204e-04_rb/)
      kao(:, 2, 5, 6) = (/ &
     &5.7894e-03_rb,5.1466e-03_rb,4.5231e-03_rb,3.9026e-03_rb,3.2431e-03_rb,2.5561e-03_rb, &
     &1.8209e-03_rb,1.0387e-03_rb,3.3073e-04_rb/)
      kao(:, 3, 5, 6) = (/ &
     &5.8510e-03_rb,5.2408e-03_rb,4.6666e-03_rb,4.0479e-03_rb,3.3863e-03_rb,2.6690e-03_rb, &
     &1.9231e-03_rb,1.1318e-03_rb,4.4023e-04_rb/)
      kao(:, 4, 5, 6) = (/ &
     &5.9488e-03_rb,5.3756e-03_rb,4.8127e-03_rb,4.2175e-03_rb,3.5437e-03_rb,2.8108e-03_rb, &
     &2.0541e-03_rb,1.2413e-03_rb,5.7017e-04_rb/)
      kao(:, 5, 5, 6) = (/ &
     &6.0632e-03_rb,5.5286e-03_rb,4.9850e-03_rb,4.3894e-03_rb,3.7125e-03_rb,2.9737e-03_rb, &
     &2.2202e-03_rb,1.3784e-03_rb,7.3804e-04_rb/)
      kao(:, 1, 6, 6) = (/ &
     &9.0144e-03_rb,7.9241e-03_rb,6.8389e-03_rb,5.7807e-03_rb,4.7224e-03_rb,3.6386e-03_rb, &
     &2.5487e-03_rb,1.3755e-03_rb,2.0268e-04_rb/)
      kao(:, 2, 6, 6) = (/ &
     &9.0405e-03_rb,7.9650e-03_rb,6.9105e-03_rb,5.8762e-03_rb,4.8286e-03_rb,3.7488e-03_rb, &
     &2.6372e-03_rb,1.4443e-03_rb,2.8152e-04_rb/)
      kao(:, 3, 6, 6) = (/ &
     &9.2000e-03_rb,8.1570e-03_rb,7.1089e-03_rb,6.0851e-03_rb,5.0081e-03_rb,3.9064e-03_rb, &
     &2.7603e-03_rb,1.5330e-03_rb,3.9833e-04_rb/)
      kao(:, 4, 6, 6) = (/ &
     &9.3676e-03_rb,8.3397e-03_rb,7.3293e-03_rb,6.2884e-03_rb,5.2172e-03_rb,4.0838e-03_rb, &
     &2.8967e-03_rb,1.6576e-03_rb,5.5504e-04_rb/)
      kao(:, 5, 6, 6) = (/ &
     &9.5404e-03_rb,8.5459e-03_rb,7.5717e-03_rb,6.5076e-03_rb,5.4312e-03_rb,4.2828e-03_rb, &
     &3.0616e-03_rb,1.8170e-03_rb,7.2861e-04_rb/)
      kao(:, 1, 7, 6) = (/ &
     &1.4907e-02_rb,1.3059e-02_rb,1.1219e-02_rb,9.3912e-03_rb,7.6074e-03_rb,5.8006e-03_rb, &
     &3.9637e-03_rb,2.0976e-03_rb,1.8870e-04_rb/)
      kao(:, 2, 7, 6) = (/ &
     &1.4960e-02_rb,1.3123e-02_rb,1.1304e-02_rb,9.5034e-03_rb,7.7189e-03_rb,5.9029e-03_rb, &
     &4.0714e-03_rb,2.1752e-03_rb,2.5278e-04_rb/)
      kao(:, 3, 7, 6) = (/ &
     &1.5077e-02_rb,1.3255e-02_rb,1.1479e-02_rb,9.6943e-03_rb,7.9204e-03_rb,6.0871e-03_rb, &
     &4.2229e-03_rb,2.2784e-03_rb,3.4866e-04_rb/)
      kao(:, 4, 7, 6) = (/ &
     &1.5413e-02_rb,1.3606e-02_rb,1.1805e-02_rb,1.0016e-02_rb,8.1764e-03_rb,6.3063e-03_rb, &
     &4.3884e-03_rb,2.3906e-03_rb,4.7884e-04_rb/)
      kao(:, 5, 7, 6) = (/ &
     &1.5640e-02_rb,1.3874e-02_rb,1.2089e-02_rb,1.0323e-02_rb,8.4660e-03_rb,6.5392e-03_rb, &
     &4.5932e-03_rb,2.5421e-03_rb,6.8588e-04_rb/)
      kao(:, 1, 8, 6) = (/ &
     &2.8559e-02_rb,2.4997e-02_rb,2.1448e-02_rb,1.7898e-02_rb,1.4352e-02_rb,1.0840e-02_rb, &
     &7.3538e-03_rb,3.7976e-03_rb,1.9487e-04_rb/)
      kao(:, 2, 8, 6) = (/ &
     &2.8738e-02_rb,2.5172e-02_rb,2.1609e-02_rb,1.8050e-02_rb,1.4521e-02_rb,1.1028e-02_rb, &
     &7.4791e-03_rb,3.8937e-03_rb,2.5597e-04_rb/)
      kao(:, 3, 8, 6) = (/ &
     &2.9088e-02_rb,2.5505e-02_rb,2.1920e-02_rb,1.8369e-02_rb,1.4844e-02_rb,1.1305e-02_rb, &
     &7.7085e-03_rb,4.0334e-03_rb,3.3647e-04_rb/)
      kao(:, 4, 8, 6) = (/ &
     &2.9471e-02_rb,2.5866e-02_rb,2.2282e-02_rb,1.8723e-02_rb,1.5191e-02_rb,1.1604e-02_rb, &
     &7.9478e-03_rb,4.2092e-03_rb,4.5107e-04_rb/)
      kao(:, 5, 8, 6) = (/ &
     &2.9965e-02_rb,2.6334e-02_rb,2.2760e-02_rb,1.9185e-02_rb,1.5605e-02_rb,1.1940e-02_rb, &
     &8.2260e-03_rb,4.4031e-03_rb,6.3668e-04_rb/)
      kao(:, 1, 9, 6) = (/ &
     &1.0333e-01_rb,9.0409e-02_rb,7.7506e-02_rb,6.4597e-02_rb,5.1717e-02_rb,3.8817e-02_rb, &
     &2.5905e-02_rb,1.3064e-02_rb,2.4712e-04_rb/)
      kao(:, 2, 9, 6) = (/ &
     &1.0373e-01_rb,9.0771e-02_rb,7.7825e-02_rb,6.4896e-02_rb,5.1961e-02_rb,3.9016e-02_rb, &
     &2.6085e-02_rb,1.3262e-02_rb,3.3832e-04_rb/)
      kao(:, 3, 9, 6) = (/ &
     &1.0493e-01_rb,9.1818e-02_rb,7.8786e-02_rb,6.5718e-02_rb,5.2648e-02_rb,3.9574e-02_rb, &
     &2.6569e-02_rb,1.3576e-02_rb,4.4601e-04_rb/)
      kao(:, 4, 9, 6) = (/ &
     &1.0587e-01_rb,9.2691e-02_rb,7.9566e-02_rb,6.6407e-02_rb,5.3246e-02_rb,4.0117e-02_rb, &
     &2.7043e-02_rb,1.3891e-02_rb,5.4826e-04_rb/)
      kao(:, 5, 9, 6) = (/ &
     &1.0759e-01_rb,9.4255e-02_rb,8.0925e-02_rb,6.7610e-02_rb,5.4300e-02_rb,4.1036e-02_rb, &
     &2.7792e-02_rb,1.4265e-02_rb,6.4886e-04_rb/)
      kao(:, 1,10, 6) = (/ &
     &3.9062e-01_rb,3.4177e-01_rb,2.9293e-01_rb,2.4412e-01_rb,1.9530e-01_rb,1.4643e-01_rb, &
     &9.7639e-02_rb,4.8848e-02_rb,1.4547e-04_rb/)
      kao(:, 2,10, 6) = (/ &
     &3.9313e-01_rb,3.4397e-01_rb,2.9486e-01_rb,2.4569e-01_rb,1.9653e-01_rb,1.4745e-01_rb, &
     &9.8352e-02_rb,4.9236e-02_rb,2.6944e-04_rb/)
      kao(:, 3,10, 6) = (/ &
     &3.9636e-01_rb,3.4679e-01_rb,2.9726e-01_rb,2.4770e-01_rb,1.9822e-01_rb,1.4872e-01_rb, &
     &9.9231e-02_rb,4.9728e-02_rb,4.9543e-04_rb/)
      kao(:, 4,10, 6) = (/ &
     &4.0190e-01_rb,3.5169e-01_rb,3.0137e-01_rb,2.5124e-01_rb,2.0112e-01_rb,1.5095e-01_rb, &
     &1.0079e-01_rb,5.0721e-02_rb,7.4715e-04_rb/)
      kao(:, 5,10, 6) = (/ &
     &4.1118e-01_rb,3.5981e-01_rb,3.0844e-01_rb,2.5722e-01_rb,2.0591e-01_rb,1.5462e-01_rb, &
     &1.0334e-01_rb,5.2226e-02_rb,1.0737e-03_rb/)
      kao(:, 1,11, 6) = (/ &
     &5.2843e-01_rb,4.6229e-01_rb,3.9627e-01_rb,3.3017e-01_rb,2.6411e-01_rb,1.9813e-01_rb, &
     &1.3214e-01_rb,6.6158e-02_rb,1.5345e-04_rb/)
      kao(:, 2,11, 6) = (/ &
     &5.2350e-01_rb,4.5807e-01_rb,3.9260e-01_rb,3.2718e-01_rb,2.6177e-01_rb,1.9641e-01_rb, &
     &1.3106e-01_rb,6.5668e-02_rb,2.3814e-04_rb/)
      kao(:, 3,11, 6) = (/ &
     &5.2048e-01_rb,4.5536e-01_rb,3.9029e-01_rb,3.2525e-01_rb,2.6032e-01_rb,1.9539e-01_rb, &
     &1.3041e-01_rb,6.5403e-02_rb,4.8642e-04_rb/)
      kao(:, 4,11, 6) = (/ &
     &5.2442e-01_rb,4.5878e-01_rb,3.9329e-01_rb,3.2791e-01_rb,2.6245e-01_rb,1.9699e-01_rb, &
     &1.3158e-01_rb,6.6140e-02_rb,7.7713e-04_rb/)
      kao(:, 5,11, 6) = (/ &
     &5.3645e-01_rb,4.6936e-01_rb,4.0243e-01_rb,3.3557e-01_rb,2.6862e-01_rb,2.0159e-01_rb, &
     &1.3470e-01_rb,6.7828e-02_rb,1.0827e-03_rb/)
      kao(:, 1,12, 6) = (/ &
     &5.9047e-01_rb,5.1664e-01_rb,4.4278e-01_rb,3.6903e-01_rb,2.9521e-01_rb,2.2135e-01_rb, &
     &1.4776e-01_rb,7.3993e-02_rb,1.4236e-04_rb/)
      kao(:, 2,12, 6) = (/ &
     &5.8585e-01_rb,5.1270e-01_rb,4.3944e-01_rb,3.6623e-01_rb,2.9300e-01_rb,2.1996e-01_rb, &
     &1.4676e-01_rb,7.3476e-02_rb,2.2023e-04_rb/)
      kao(:, 3,12, 6) = (/ &
     &5.7367e-01_rb,5.0194e-01_rb,4.3031e-01_rb,3.5876e-01_rb,2.8718e-01_rb,2.1559e-01_rb, &
     &1.4380e-01_rb,7.2153e-02_rb,4.4743e-04_rb/)
      kao(:, 4,12, 6) = (/ &
     &5.7511e-01_rb,5.0312e-01_rb,4.3146e-01_rb,3.5985e-01_rb,2.8795e-01_rb,2.1609e-01_rb, &
     &1.4436e-01_rb,7.2515e-02_rb,8.0036e-04_rb/)
      kao(:, 5,12, 6) = (/ &
     &5.8792e-01_rb,5.1441e-01_rb,4.4120e-01_rb,3.6789e-01_rb,2.9440e-01_rb,2.2104e-01_rb, &
     &1.4769e-01_rb,7.4288e-02_rb,1.0818e-03_rb/)
      kao(:, 1,13, 6) = (/ &
     &5.2587e-01_rb,4.6012e-01_rb,3.9434e-01_rb,3.2864e-01_rb,2.6295e-01_rb,1.9729e-01_rb, &
     &1.3161e-01_rb,6.5880e-02_rb,1.1855e-04_rb/)
      kao(:, 2,13, 6) = (/ &
     &5.1655e-01_rb,4.5203e-01_rb,3.8739e-01_rb,3.2291e-01_rb,2.5848e-01_rb,1.9395e-01_rb, &
     &1.2940e-01_rb,6.4811e-02_rb,1.8738e-04_rb/)
      kao(:, 3,13, 6) = (/ &
     &5.1454e-01_rb,4.5033e-01_rb,3.8607e-01_rb,3.2190e-01_rb,2.5767e-01_rb,1.9330e-01_rb, &
     &1.2903e-01_rb,6.4719e-02_rb,4.6442e-04_rb/)
      kao(:, 4,13, 6) = (/ &
     &5.2306e-01_rb,4.5763e-01_rb,3.9250e-01_rb,3.2722e-01_rb,2.6190e-01_rb,1.9661e-01_rb, &
     &1.3127e-01_rb,6.5873e-02_rb,7.3146e-04_rb/)
      kao(:, 5,13, 6) = (/ &
     &5.3732e-01_rb,4.7035e-01_rb,4.0337e-01_rb,3.3620e-01_rb,2.6904e-01_rb,2.0206e-01_rb, &
     &1.3501e-01_rb,6.8074e-02_rb,1.0136e-03_rb/)
      kao(:, 1, 1, 7) = (/ &
     &1.2103e-03_rb,1.2175e-03_rb,1.1935e-03_rb,1.1549e-03_rb,1.0791e-03_rb,9.5598e-04_rb, &
     &8.1255e-04_rb,6.1193e-04_rb,6.2180e-04_rb/)
      kao(:, 2, 1, 7) = (/ &
     &1.2178e-03_rb,1.2781e-03_rb,1.2880e-03_rb,1.2578e-03_rb,1.2013e-03_rb,1.0898e-03_rb, &
     &9.4696e-04_rb,8.3333e-04_rb,8.1545e-04_rb/)
      kao(:, 3, 1, 7) = (/ &
     &1.2282e-03_rb,1.3405e-03_rb,1.3881e-03_rb,1.3716e-03_rb,1.3394e-03_rb,1.2381e-03_rb, &
     &1.1125e-03_rb,1.0711e-03_rb,1.0876e-03_rb/)
      kao(:, 4, 1, 7) = (/ &
     &1.2403e-03_rb,1.4084e-03_rb,1.4922e-03_rb,1.5029e-03_rb,1.4913e-03_rb,1.4226e-03_rb, &
     &1.3116e-03_rb,1.3637e-03_rb,1.4095e-03_rb/)
      kao(:, 5, 1, 7) = (/ &
     &1.2587e-03_rb,1.4782e-03_rb,1.5894e-03_rb,1.6533e-03_rb,1.6739e-03_rb,1.6196e-03_rb, &
     &1.5433e-03_rb,1.7053e-03_rb,1.8020e-03_rb/)
      kao(:, 1, 2, 7) = (/ &
     &1.5958e-03_rb,1.5396e-03_rb,1.4781e-03_rb,1.3976e-03_rb,1.2942e-03_rb,1.1273e-03_rb, &
     &9.3307e-04_rb,7.0021e-04_rb,6.0904e-04_rb/)
      kao(:, 2, 2, 7) = (/ &
     &1.6176e-03_rb,1.6176e-03_rb,1.5750e-03_rb,1.5193e-03_rb,1.4296e-03_rb,1.2725e-03_rb, &
     &1.0674e-03_rb,8.4851e-04_rb,7.7411e-04_rb/)
      kao(:, 3, 2, 7) = (/ &
     &1.6471e-03_rb,1.6851e-03_rb,1.6807e-03_rb,1.6583e-03_rb,1.5791e-03_rb,1.4314e-03_rb, &
     &1.2293e-03_rb,1.0317e-03_rb,9.9584e-04_rb/)
      kao(:, 4, 2, 7) = (/ &
     &1.6650e-03_rb,1.7627e-03_rb,1.7970e-03_rb,1.8046e-03_rb,1.7334e-03_rb,1.6113e-03_rb, &
     &1.4228e-03_rb,1.2451e-03_rb,1.2394e-03_rb/)
      kao(:, 5, 2, 7) = (/ &
     &1.7122e-03_rb,1.8558e-03_rb,1.9287e-03_rb,1.9533e-03_rb,1.9037e-03_rb,1.8293e-03_rb, &
     &1.6473e-03_rb,1.5087e-03_rb,1.5795e-03_rb/)
      kao(:, 1, 3, 7) = (/ &
     &2.9351e-03_rb,2.6534e-03_rb,2.4324e-03_rb,2.1954e-03_rb,1.9158e-03_rb,1.6076e-03_rb, &
     &1.2520e-03_rb,8.5789e-04_rb,6.1375e-04_rb/)
      kao(:, 2, 3, 7) = (/ &
     &2.9633e-03_rb,2.7281e-03_rb,2.5348e-03_rb,2.3132e-03_rb,2.0567e-03_rb,1.7614e-03_rb, &
     &1.4143e-03_rb,9.9890e-04_rb,7.9742e-04_rb/)
      kao(:, 3, 3, 7) = (/ &
     &3.0018e-03_rb,2.8266e-03_rb,2.6658e-03_rb,2.4521e-03_rb,2.2182e-03_rb,1.9505e-03_rb, &
     &1.5883e-03_rb,1.1664e-03_rb,9.9661e-04_rb/)
      kao(:, 4, 3, 7) = (/ &
     &3.0730e-03_rb,2.9520e-03_rb,2.8043e-03_rb,2.6204e-03_rb,2.4182e-03_rb,2.1521e-03_rb, &
     &1.7967e-03_rb,1.3650e-03_rb,1.2315e-03_rb/)
      kao(:, 5, 3, 7) = (/ &
     &3.1816e-03_rb,3.1107e-03_rb,2.9552e-03_rb,2.8080e-03_rb,2.6346e-03_rb,2.3544e-03_rb, &
     &2.0373e-03_rb,1.6039e-03_rb,1.5090e-03_rb/)
      kao(:, 1, 4, 7) = (/ &
     &5.7269e-03_rb,5.0918e-03_rb,4.4466e-03_rb,3.8641e-03_rb,3.2580e-03_rb,2.6083e-03_rb, &
     &1.8965e-03_rb,1.1416e-03_rb,5.7204e-04_rb/)
      kao(:, 2, 4, 7) = (/ &
     &5.7492e-03_rb,5.1587e-03_rb,4.5741e-03_rb,4.0239e-03_rb,3.4230e-03_rb,2.7638e-03_rb, &
     &2.0485e-03_rb,1.2804e-03_rb,7.6092e-04_rb/)
      kao(:, 3, 4, 7) = (/ &
     &5.8525e-03_rb,5.2748e-03_rb,4.7562e-03_rb,4.2073e-03_rb,3.6092e-03_rb,2.9384e-03_rb, &
     &2.2265e-03_rb,1.4580e-03_rb,9.9771e-04_rb/)
      kao(:, 4, 4, 7) = (/ &
     &5.9797e-03_rb,5.4326e-03_rb,4.9495e-03_rb,4.4248e-03_rb,3.8014e-03_rb,3.1433e-03_rb, &
     &2.4307e-03_rb,1.6612e-03_rb,1.3149e-03_rb/)
      kao(:, 5, 4, 7) = (/ &
     &6.1647e-03_rb,5.6540e-03_rb,5.1948e-03_rb,4.6848e-03_rb,4.0444e-03_rb,3.3860e-03_rb, &
     &2.6695e-03_rb,1.9106e-03_rb,1.6620e-03_rb/)
      kao(:, 1, 5, 7) = (/ &
     &1.0220e-02_rb,9.0008e-03_rb,7.7868e-03_rb,6.5817e-03_rb,5.4248e-03_rb,4.2290e-03_rb, &
     &3.0009e-03_rb,1.6787e-03_rb,5.3957e-04_rb/)
      kao(:, 2, 5, 7) = (/ &
     &1.0367e-02_rb,9.1668e-03_rb,7.9693e-03_rb,6.7850e-03_rb,5.6408e-03_rb,4.4280e-03_rb, &
     &3.1863e-03_rb,1.8183e-03_rb,7.0339e-04_rb/)
      kao(:, 3, 5, 7) = (/ &
     &1.0581e-02_rb,9.4102e-03_rb,8.2049e-03_rb,7.0786e-03_rb,5.9001e-03_rb,4.6839e-03_rb, &
     &3.3914e-03_rb,1.9878e-03_rb,9.2591e-04_rb/)
      kao(:, 4, 5, 7) = (/ &
     &1.0809e-02_rb,9.6775e-03_rb,8.5132e-03_rb,7.3701e-03_rb,6.2042e-03_rb,4.9938e-03_rb, &
     &3.6238e-03_rb,2.1871e-03_rb,1.1943e-03_rb/)
      kao(:, 5, 5, 7) = (/ &
     &1.1112e-02_rb,1.0026e-02_rb,8.9250e-03_rb,7.7808e-03_rb,6.5932e-03_rb,5.3151e-03_rb, &
     &3.9019e-03_rb,2.4156e-03_rb,1.5529e-03_rb/)
      kao(:, 1, 6, 7) = (/ &
     &1.6720e-02_rb,1.4665e-02_rb,1.2615e-02_rb,1.0575e-02_rb,8.5627e-03_rb,6.5744e-03_rb, &
     &4.5286e-03_rb,2.4830e-03_rb,5.6458e-04_rb/)
      kao(:, 2, 6, 7) = (/ &
     &1.7104e-02_rb,1.5034e-02_rb,1.2975e-02_rb,1.0925e-02_rb,8.9009e-03_rb,6.8822e-03_rb, &
     &4.7999e-03_rb,2.6619e-03_rb,7.2679e-04_rb/)
      kao(:, 3, 6, 7) = (/ &
     &1.7534e-02_rb,1.5438e-02_rb,1.3395e-02_rb,1.1323e-02_rb,9.3166e-03_rb,7.2349e-03_rb, &
     &5.0972e-03_rb,2.8734e-03_rb,9.0147e-04_rb/)
      kao(:, 4, 6, 7) = (/ &
     &1.8091e-02_rb,1.6011e-02_rb,1.3940e-02_rb,1.1869e-02_rb,9.7728e-03_rb,7.6440e-03_rb, &
     &5.4515e-03_rb,3.0818e-03_rb,1.1288e-03_rb/)
      kao(:, 5, 6, 7) = (/ &
     &1.8583e-02_rb,1.6530e-02_rb,1.4432e-02_rb,1.2438e-02_rb,1.0309e-02_rb,8.1409e-03_rb, &
     &5.8462e-03_rb,3.3346e-03_rb,1.4293e-03_rb/)
      kao(:, 1, 7, 7) = (/ &
     &2.7982e-02_rb,2.4501e-02_rb,2.1043e-02_rb,1.7577e-02_rb,1.4110e-02_rb,1.0697e-02_rb, &
     &7.3114e-03_rb,3.8393e-03_rb,5.1721e-04_rb/)
      kao(:, 2, 7, 7) = (/ &
     &2.8730e-02_rb,2.5189e-02_rb,2.1653e-02_rb,1.8120e-02_rb,1.4619e-02_rb,1.1160e-02_rb, &
     &7.6475e-03_rb,4.0900e-03_rb,6.8268e-04_rb/)
      kao(:, 3, 7, 7) = (/ &
     &2.9602e-02_rb,2.5984e-02_rb,2.2367e-02_rb,1.8790e-02_rb,1.5226e-02_rb,1.1688e-02_rb, &
     &8.0684e-03_rb,4.3672e-03_rb,8.7865e-04_rb/)
      kao(:, 4, 7, 7) = (/ &
     &3.0383e-02_rb,2.6722e-02_rb,2.3100e-02_rb,1.9474e-02_rb,1.5890e-02_rb,1.2270e-02_rb, &
     &8.5486e-03_rb,4.6828e-03_rb,1.1058e-03_rb/)
      kao(:, 5, 7, 7) = (/ &
     &3.1027e-02_rb,2.7319e-02_rb,2.3722e-02_rb,2.0052e-02_rb,1.6464e-02_rb,1.2803e-02_rb, &
     &9.0257e-03_rb,5.0413e-03_rb,1.3701e-03_rb/)
      kao(:, 1, 8, 7) = (/ &
     &5.4825e-02_rb,4.7970e-02_rb,4.1147e-02_rb,3.4322e-02_rb,2.7487e-02_rb,2.0656e-02_rb, &
     &1.3854e-02_rb,7.1281e-03_rb,4.3340e-04_rb/)
      kao(:, 2, 8, 7) = (/ &
     &5.6231e-02_rb,4.9217e-02_rb,4.2253e-02_rb,3.5263e-02_rb,2.8269e-02_rb,2.1292e-02_rb, &
     &1.4434e-02_rb,7.4660e-03_rb,6.0870e-04_rb/)
      kao(:, 3, 8, 7) = (/ &
     &5.7926e-02_rb,5.0738e-02_rb,4.3579e-02_rb,3.6411e-02_rb,2.9254e-02_rb,2.2161e-02_rb, &
     &1.5102e-02_rb,7.9134e-03_rb,8.5563e-04_rb/)
      kao(:, 4, 8, 7) = (/ &
     &5.9329e-02_rb,5.2017e-02_rb,4.4709e-02_rb,3.7445e-02_rb,3.0174e-02_rb,2.2939e-02_rb, &
     &1.5732e-02_rb,8.3409e-03_rb,1.1069e-03_rb/)
      kao(:, 5, 8, 7) = (/ &
     &6.0706e-02_rb,5.3284e-02_rb,4.5885e-02_rb,3.8526e-02_rb,3.1172e-02_rb,2.3870e-02_rb, &
     &1.6429e-02_rb,8.7694e-03_rb,1.3835e-03_rb/)
      kao(:, 1, 9, 7) = (/ &
     &2.0429e-01_rb,1.7875e-01_rb,1.5322e-01_rb,1.2768e-01_rb,1.0216e-01_rb,7.6684e-02_rb, &
     &5.1143e-02_rb,2.5629e-02_rb,2.9263e-04_rb/)
      kao(:, 2, 9, 7) = (/ &
     &2.0990e-01_rb,1.8367e-01_rb,1.5743e-01_rb,1.3122e-01_rb,1.0506e-01_rb,7.8849e-02_rb, &
     &5.2652e-02_rb,2.6459e-02_rb,5.6921e-04_rb/)
      kao(:, 3, 9, 7) = (/ &
     &2.1338e-01_rb,1.8672e-01_rb,1.6006e-01_rb,1.3349e-01_rb,1.0689e-01_rb,8.0285e-02_rb, &
     &5.3664e-02_rb,2.7200e-02_rb,9.3758e-04_rb/)
      kao(:, 4, 9, 7) = (/ &
     &2.2002e-01_rb,1.9252e-01_rb,1.6512e-01_rb,1.3776e-01_rb,1.1035e-01_rb,8.2930e-02_rb, &
     &5.5562e-02_rb,2.8342e-02_rb,1.3055e-03_rb/)
      kao(:, 5, 9, 7) = (/ &
     &2.2506e-01_rb,1.9692e-01_rb,1.6901e-01_rb,1.4099e-01_rb,1.1303e-01_rb,8.5078e-02_rb, &
     &5.7190e-02_rb,2.9461e-02_rb,1.8874e-03_rb/)
      kao(:, 1,10, 7) = (/ &
     &8.3776e-01_rb,7.3310e-01_rb,6.2841e-01_rb,5.2361e-01_rb,4.1890e-01_rb,3.1415e-01_rb, &
     &2.0945e-01_rb,1.0479e-01_rb,1.9258e-04_rb/)
      kao(:, 2,10, 7) = (/ &
     &8.5667e-01_rb,7.4964e-01_rb,6.4250e-01_rb,5.3547e-01_rb,4.2837e-01_rb,3.2129e-01_rb, &
     &2.1421e-01_rb,1.0723e-01_rb,3.0052e-04_rb/)
      kao(:, 3,10, 7) = (/ &
     &8.7378e-01_rb,7.6452e-01_rb,6.5535e-01_rb,5.4608e-01_rb,4.3691e-01_rb,3.2776e-01_rb, &
     &2.1870e-01_rb,1.0952e-01_rb,5.4766e-04_rb/)
      kao(:, 4,10, 7) = (/ &
     &8.9911e-01_rb,7.8666e-01_rb,6.7423e-01_rb,5.6186e-01_rb,4.4961e-01_rb,3.3744e-01_rb, &
     &2.2515e-01_rb,1.1291e-01_rb,1.0571e-03_rb/)
      kao(:, 5,10, 7) = (/ &
     &9.2543e-01_rb,8.0983e-01_rb,6.9418e-01_rb,5.7864e-01_rb,4.6314e-01_rb,3.4767e-01_rb, &
     &2.3212e-01_rb,1.1658e-01_rb,1.4779e-03_rb/)
      kao(:, 1,11, 7) = (/ &
     &1.1170e+00_rb,9.7735e-01_rb,8.3780e-01_rb,6.9827e-01_rb,5.5860e-01_rb,4.1894e-01_rb, &
     &2.7927e-01_rb,1.3969e-01_rb,1.8784e-04_rb/)
      kao(:, 2,11, 7) = (/ &
     &1.1469e+00_rb,1.0035e+00_rb,8.6021e-01_rb,7.1681e-01_rb,5.7347e-01_rb,4.3009e-01_rb, &
     &2.8675e-01_rb,1.4352e-01_rb,2.9142e-04_rb/)
      kao(:, 3,11, 7) = (/ &
     &1.1926e+00_rb,1.0436e+00_rb,8.9439e-01_rb,7.4544e-01_rb,5.9622e-01_rb,4.4725e-01_rb, &
     &2.9836e-01_rb,1.4934e-01_rb,4.5203e-04_rb/)
      kao(:, 4,11, 7) = (/ &
     &1.2362e+00_rb,1.0816e+00_rb,9.2703e-01_rb,7.7246e-01_rb,6.1814e-01_rb,4.6378e-01_rb, &
     &3.0936e-01_rb,1.5503e-01_rb,7.8924e-04_rb/)
      kao(:, 5,11, 7) = (/ &
     &1.2847e+00_rb,1.1241e+00_rb,9.6352e-01_rb,8.0300e-01_rb,6.4259e-01_rb,4.8242e-01_rb, &
     &3.2196e-01_rb,1.6156e-01_rb,1.3839e-03_rb/)
      kao(:, 1,12, 7) = (/ &
     &1.0784e+00_rb,9.4374e-01_rb,8.0896e-01_rb,6.7399e-01_rb,5.3931e-01_rb,4.0450e-01_rb, &
     &2.6958e-01_rb,1.3485e-01_rb,1.8250e-04_rb/)
      kao(:, 2,12, 7) = (/ &
     &1.1016e+00_rb,9.6383e-01_rb,8.2624e-01_rb,6.8848e-01_rb,5.5081e-01_rb,4.1305e-01_rb, &
     &2.7543e-01_rb,1.3793e-01_rb,2.7118e-04_rb/)
      kao(:, 3,12, 7) = (/ &
     &1.1494e+00_rb,1.0057e+00_rb,8.6213e-01_rb,7.1839e-01_rb,5.7458e-01_rb,4.3099e-01_rb, &
     &2.8756e-01_rb,1.4395e-01_rb,4.0903e-04_rb/)
      kao(:, 4,12, 7) = (/ &
     &1.1966e+00_rb,1.0468e+00_rb,8.9727e-01_rb,7.4760e-01_rb,5.9830e-01_rb,4.4892e-01_rb, &
     &2.9949e-01_rb,1.5008e-01_rb,6.0710e-04_rb/)
      kao(:, 5,12, 7) = (/ &
     &1.2391e+00_rb,1.0842e+00_rb,9.2930e-01_rb,7.7450e-01_rb,6.1988e-01_rb,4.6531e-01_rb, &
     &3.1051e-01_rb,1.5585e-01_rb,1.1075e-03_rb/)
      kao(:, 1,13, 7) = (/ &
     &9.1609e-01_rb,8.0170e-01_rb,6.8730e-01_rb,5.7285e-01_rb,4.5825e-01_rb,3.4365e-01_rb, &
     &2.2921e-01_rb,1.1478e-01_rb,1.8534e-04_rb/)
      kao(:, 2,13, 7) = (/ &
     &9.2338e-01_rb,8.0782e-01_rb,6.9263e-01_rb,5.7718e-01_rb,4.6175e-01_rb,3.4635e-01_rb, &
     &2.3110e-01_rb,1.1575e-01_rb,2.6985e-04_rb/)
      kao(:, 3,13, 7) = (/ &
     &9.4678e-01_rb,8.2823e-01_rb,7.0984e-01_rb,5.9160e-01_rb,4.7336e-01_rb,3.5526e-01_rb, &
     &2.3691e-01_rb,1.1874e-01_rb,3.9189e-04_rb/)
      kao(:, 4,13, 7) = (/ &
     &9.5798e-01_rb,8.3839e-01_rb,7.1856e-01_rb,5.9888e-01_rb,4.7933e-01_rb,3.5969e-01_rb, &
     &2.4009e-01_rb,1.2046e-01_rb,6.8070e-04_rb/)
      kao(:, 5,13, 7) = (/ &
     &9.7420e-01_rb,8.5221e-01_rb,7.3072e-01_rb,6.0920e-01_rb,4.8778e-01_rb,3.6616e-01_rb, &
     &2.4455e-01_rb,1.2274e-01_rb,1.2004e-03_rb/)
      kao(:, 1, 1, 8) = (/ &
     &2.2086e-03_rb,2.0999e-03_rb,2.0159e-03_rb,1.8719e-03_rb,1.7233e-03_rb,1.5637e-03_rb, &
     &1.3735e-03_rb,1.3147e-03_rb,1.3769e-03_rb/)
      kao(:, 2, 1, 8) = (/ &
     &2.2485e-03_rb,2.2158e-03_rb,2.1547e-03_rb,2.0369e-03_rb,1.8898e-03_rb,1.7374e-03_rb, &
     &1.6006e-03_rb,1.7014e-03_rb,1.7807e-03_rb/)
      kao(:, 3, 1, 8) = (/ &
     &2.2803e-03_rb,2.3414e-03_rb,2.2949e-03_rb,2.2265e-03_rb,2.0601e-03_rb,1.9356e-03_rb, &
     &1.8992e-03_rb,2.1518e-03_rb,2.2509e-03_rb/)
      kao(:, 4, 1, 8) = (/ &
     &2.3178e-03_rb,2.4462e-03_rb,2.4702e-03_rb,2.4013e-03_rb,2.2630e-03_rb,2.1680e-03_rb, &
     &2.2995e-03_rb,2.6824e-03_rb,2.8243e-03_rb/)
      kao(:, 5, 1, 8) = (/ &
     &2.3355e-03_rb,2.5902e-03_rb,2.6514e-03_rb,2.5825e-03_rb,2.4802e-03_rb,2.5028e-03_rb, &
     &2.7971e-03_rb,3.2930e-03_rb,3.4957e-03_rb/)
      kao(:, 1, 2, 8) = (/ &
     &3.2554e-03_rb,3.0029e-03_rb,2.8238e-03_rb,2.5475e-03_rb,2.2552e-03_rb,1.9853e-03_rb, &
     &1.6584e-03_rb,1.2784e-03_rb,1.0936e-03_rb/)
      kao(:, 2, 2, 8) = (/ &
     &3.3021e-03_rb,3.1146e-03_rb,2.9858e-03_rb,2.7302e-03_rb,2.4455e-03_rb,2.1856e-03_rb, &
     &1.8782e-03_rb,1.5376e-03_rb,1.5050e-03_rb/)
      kao(:, 3, 2, 8) = (/ &
     &3.3434e-03_rb,3.2633e-03_rb,3.1581e-03_rb,2.9235e-03_rb,2.6724e-03_rb,2.4236e-03_rb, &
     &2.1164e-03_rb,1.8700e-03_rb,1.9615e-03_rb/)
      kao(:, 4, 2, 8) = (/ &
     &3.3924e-03_rb,3.4464e-03_rb,3.3467e-03_rb,3.1420e-03_rb,2.9417e-03_rb,2.6854e-03_rb, &
     &2.3908e-03_rb,2.2943e-03_rb,2.5456e-03_rb/)
      kao(:, 5, 2, 8) = (/ &
     &3.4124e-03_rb,3.5941e-03_rb,3.5577e-03_rb,3.4139e-03_rb,3.2366e-03_rb,2.9618e-03_rb, &
     &2.7452e-03_rb,2.8336e-03_rb,3.1731e-03_rb/)
      kao(:, 1, 3, 8) = (/ &
     &6.2310e-03_rb,5.6159e-03_rb,4.9769e-03_rb,4.4245e-03_rb,3.8289e-03_rb,3.1465e-03_rb, &
     &2.4651e-03_rb,1.6762e-03_rb,1.0171e-03_rb/)
      kao(:, 2, 3, 8) = (/ &
     &6.3558e-03_rb,5.7885e-03_rb,5.2143e-03_rb,4.7067e-03_rb,4.0833e-03_rb,3.4041e-03_rb, &
     &2.6899e-03_rb,1.9219e-03_rb,1.2943e-03_rb/)
      kao(:, 3, 3, 8) = (/ &
     &6.4584e-03_rb,5.9604e-03_rb,5.4819e-03_rb,4.9958e-03_rb,4.3577e-03_rb,3.6662e-03_rb, &
     &2.9872e-03_rb,2.1872e-03_rb,1.6861e-03_rb/)
      kao(:, 4, 3, 8) = (/ &
     &6.6103e-03_rb,6.1684e-03_rb,5.8142e-03_rb,5.2903e-03_rb,4.6437e-03_rb,3.9929e-03_rb, &
     &3.3051e-03_rb,2.4974e-03_rb,2.2070e-03_rb/)
      kao(:, 5, 3, 8) = (/ &
     &6.6588e-03_rb,6.3708e-03_rb,6.1123e-03_rb,5.6154e-03_rb,4.9949e-03_rb,4.3617e-03_rb, &
     &3.6895e-03_rb,2.8582e-03_rb,2.8798e-03_rb/)
      kao(:, 1, 4, 8) = (/ &
     &1.1807e-02_rb,1.0450e-02_rb,9.1344e-03_rb,7.7844e-03_rb,6.5100e-03_rb,5.2593e-03_rb, &
     &3.9218e-03_rb,2.4635e-03_rb,1.1324e-03_rb/)
      kao(:, 2, 4, 8) = (/ &
     &1.2088e-02_rb,1.0775e-02_rb,9.4615e-03_rb,8.1147e-03_rb,6.9001e-03_rb,5.6187e-03_rb, &
     &4.2536e-03_rb,2.7131e-03_rb,1.3848e-03_rb/)
      kao(:, 3, 4, 8) = (/ &
     &1.2276e-02_rb,1.1057e-02_rb,9.7542e-03_rb,8.5429e-03_rb,7.3199e-03_rb,6.0294e-03_rb, &
     &4.5931e-03_rb,3.0093e-03_rb,1.7357e-03_rb/)
      kao(:, 4, 4, 8) = (/ &
     &1.2505e-02_rb,1.1360e-02_rb,1.0130e-02_rb,9.0368e-03_rb,7.7990e-03_rb,6.4476e-03_rb, &
     &5.0000e-03_rb,3.3597e-03_rb,2.0753e-03_rb/)
      kao(:, 5, 4, 8) = (/ &
     &1.2694e-02_rb,1.1662e-02_rb,1.0517e-02_rb,9.4548e-03_rb,8.2827e-03_rb,6.9233e-03_rb, &
     &5.4172e-03_rb,3.7628e-03_rb,2.5678e-03_rb/)
      kao(:, 1, 5, 8) = (/ &
     &2.0656e-02_rb,1.8160e-02_rb,1.5700e-02_rb,1.3242e-02_rb,1.0772e-02_rb,8.3779e-03_rb, &
     &6.0246e-03_rb,3.5524e-03_rb,1.1630e-03_rb/)
      kao(:, 2, 5, 8) = (/ &
     &2.0910e-02_rb,1.8437e-02_rb,1.6005e-02_rb,1.3585e-02_rb,1.1145e-02_rb,8.8221e-03_rb, &
     &6.4062e-03_rb,3.8753e-03_rb,1.4990e-03_rb/)
      kao(:, 3, 5, 8) = (/ &
     &2.1222e-02_rb,1.8787e-02_rb,1.6416e-02_rb,1.3970e-02_rb,1.1606e-02_rb,9.3057e-03_rb, &
     &6.8725e-03_rb,4.2169e-03_rb,1.9100e-03_rb/)
      kao(:, 4, 5, 8) = (/ &
     &2.1675e-02_rb,1.9271e-02_rb,1.6942e-02_rb,1.4504e-02_rb,1.2206e-02_rb,9.8161e-03_rb, &
     &7.3695e-03_rb,4.6216e-03_rb,2.4296e-03_rb/)
      kao(:, 5, 5, 8) = (/ &
     &2.2102e-02_rb,1.9781e-02_rb,1.7441e-02_rb,1.5076e-02_rb,1.2804e-02_rb,1.0400e-02_rb, &
     &7.9027e-03_rb,5.0748e-03_rb,2.9846e-03_rb/)
      kao(:, 1, 6, 8) = (/ &
     &3.4516e-02_rb,3.0236e-02_rb,2.6032e-02_rb,2.1800e-02_rb,1.7574e-02_rb,1.3389e-02_rb, &
     &9.2847e-03_rb,5.1122e-03_rb,1.0508e-03_rb/)
      kao(:, 2, 6, 8) = (/ &
     &3.4800e-02_rb,3.0544e-02_rb,2.6323e-02_rb,2.2117e-02_rb,1.7921e-02_rb,1.3756e-02_rb, &
     &9.6631e-03_rb,5.4342e-03_rb,1.4451e-03_rb/)
      kao(:, 3, 6, 8) = (/ &
     &3.5127e-02_rb,3.0902e-02_rb,2.6702e-02_rb,2.2553e-02_rb,1.8369e-02_rb,1.4212e-02_rb, &
     &1.0118e-02_rb,5.8263e-03_rb,1.8818e-03_rb/)
      kao(:, 4, 6, 8) = (/ &
     &3.5628e-02_rb,3.1400e-02_rb,2.7233e-02_rb,2.3087e-02_rb,1.8895e-02_rb,1.4840e-02_rb, &
     &1.0658e-02_rb,6.2966e-03_rb,2.4028e-03_rb/)
      kao(:, 5, 6, 8) = (/ &
     &3.6414e-02_rb,3.2167e-02_rb,2.8039e-02_rb,2.3770e-02_rb,1.9619e-02_rb,1.5526e-02_rb, &
     &1.1248e-02_rb,6.8040e-03_rb,2.9828e-03_rb/)
      kao(:, 1, 7, 8) = (/ &
     &6.0465e-02_rb,5.2915e-02_rb,4.5423e-02_rb,3.7947e-02_rb,3.0446e-02_rb,2.2948e-02_rb, &
     &1.5515e-02_rb,8.2239e-03_rb,1.0379e-03_rb/)
      kao(:, 2, 7, 8) = (/ &
     &6.0755e-02_rb,5.3175e-02_rb,4.5748e-02_rb,3.8231e-02_rb,3.0723e-02_rb,2.3271e-02_rb, &
     &1.5903e-02_rb,8.5285e-03_rb,1.3797e-03_rb/)
      kao(:, 3, 7, 8) = (/ &
     &6.1398e-02_rb,5.3805e-02_rb,4.6318e-02_rb,3.8793e-02_rb,3.1274e-02_rb,2.3804e-02_rb, &
     &1.6410e-02_rb,8.9355e-03_rb,1.8226e-03_rb/)
      kao(:, 4, 7, 8) = (/ &
     &6.2566e-02_rb,5.4915e-02_rb,4.7291e-02_rb,3.9712e-02_rb,3.2135e-02_rb,2.4531e-02_rb, &
     &1.7075e-02_rb,9.4211e-03_rb,2.3895e-03_rb/)
      kao(:, 5, 7, 8) = (/ &
     &6.4353e-02_rb,5.6526e-02_rb,4.8732e-02_rb,4.1035e-02_rb,3.3303e-02_rb,2.5613e-02_rb, &
     &1.7950e-02_rb,9.9840e-03_rb,2.9275e-03_rb/)
      kao(:, 1, 8, 8) = (/ &
     &1.2368e-01_rb,1.0822e-01_rb,9.2766e-02_rb,7.7347e-02_rb,6.2005e-02_rb,4.6603e-02_rb, &
     &3.1194e-02_rb,1.5883e-02_rb,8.5274e-04_rb/)
      kao(:, 2, 8, 8) = (/ &
     &1.2468e-01_rb,1.0911e-01_rb,9.3546e-02_rb,7.8108e-02_rb,6.2629e-02_rb,4.7130e-02_rb, &
     &3.1595e-02_rb,1.6348e-02_rb,1.3444e-03_rb/)
      kao(:, 3, 8, 8) = (/ &
     &1.2638e-01_rb,1.1058e-01_rb,9.4937e-02_rb,7.9293e-02_rb,6.3607e-02_rb,4.7918e-02_rb, &
     &3.2304e-02_rb,1.6902e-02_rb,1.7616e-03_rb/)
      kao(:, 4, 8, 8) = (/ &
     &1.2969e-01_rb,1.1353e-01_rb,9.7623e-02_rb,8.1508e-02_rb,6.5432e-02_rb,4.9461e-02_rb, &
     &3.3527e-02_rb,1.7620e-02_rb,2.4617e-03_rb/)
      kao(:, 5, 8, 8) = (/ &
     &1.3392e-01_rb,1.1729e-01_rb,1.0082e-01_rb,8.4317e-02_rb,6.7894e-02_rb,5.1430e-02_rb, &
     &3.4991e-02_rb,1.8580e-02_rb,3.1642e-03_rb/)
      kao(:, 1, 9, 8) = (/ &
     &4.6846e-01_rb,4.0990e-01_rb,3.5136e-01_rb,2.9284e-01_rb,2.3423e-01_rb,1.7564e-01_rb, &
     &1.1723e-01_rb,5.8786e-02_rb,4.0030e-04_rb/)
      kao(:, 2, 9, 8) = (/ &
     &4.7406e-01_rb,4.1475e-01_rb,3.5556e-01_rb,2.9624e-01_rb,2.3703e-01_rb,1.7789e-01_rb, &
     &1.1882e-01_rb,5.9622e-02_rb,6.8004e-04_rb/)
      kao(:, 3, 9, 8) = (/ &
     &4.8684e-01_rb,4.2603e-01_rb,3.6514e-01_rb,3.0430e-01_rb,2.4358e-01_rb,1.8292e-01_rb, &
     &1.2220e-01_rb,6.1402e-02_rb,1.0402e-03_rb/)
      kao(:, 4, 9, 8) = (/ &
     &5.0043e-01_rb,4.3778e-01_rb,3.7520e-01_rb,3.1290e-01_rb,2.5060e-01_rb,1.8834e-01_rb, &
     &1.2594e-01_rb,6.3564e-02_rb,1.7944e-03_rb/)
      kao(:, 5, 9, 8) = (/ &
     &5.2090e-01_rb,4.5588e-01_rb,3.9070e-01_rb,3.2594e-01_rb,2.6129e-01_rb,1.9629e-01_rb, &
     &1.3136e-01_rb,6.6645e-02_rb,2.5620e-03_rb/)
      kao(:, 1,10, 8) = (/ &
     &1.9599e+00_rb,1.7147e+00_rb,1.4700e+00_rb,1.2249e+00_rb,9.7999e-01_rb,7.3502e-01_rb, &
     &4.8995e-01_rb,2.4496e-01_rb,4.5019e-04_rb/)
      kao(:, 2,10, 8) = (/ &
     &2.0004e+00_rb,1.7503e+00_rb,1.5003e+00_rb,1.2502e+00_rb,1.0002e+00_rb,7.5004e-01_rb, &
     &5.0013e-01_rb,2.5017e-01_rb,7.7160e-04_rb/)
      kao(:, 3,10, 8) = (/ &
     &2.0706e+00_rb,1.8118e+00_rb,1.5527e+00_rb,1.2941e+00_rb,1.0353e+00_rb,7.7648e-01_rb, &
     &5.1774e-01_rb,2.5926e-01_rb,1.2278e-03_rb/)
      kao(:, 4,10, 8) = (/ &
     &2.1488e+00_rb,1.8803e+00_rb,1.6119e+00_rb,1.3430e+00_rb,1.0744e+00_rb,8.0582e-01_rb, &
     &5.3747e-01_rb,2.6934e-01_rb,1.1252e-03_rb/)
      kao(:, 5,10, 8) = (/ &
     &2.2415e+00_rb,1.9614e+00_rb,1.6811e+00_rb,1.4008e+00_rb,1.1207e+00_rb,8.4068e-01_rb, &
     &5.6107e-01_rb,2.8130e-01_rb,1.6850e-03_rb/)
      kao(:, 1,11, 8) = (/ &
     &2.8911e+00_rb,2.5300e+00_rb,2.1683e+00_rb,1.8069e+00_rb,1.4455e+00_rb,1.0841e+00_rb, &
     &7.2274e-01_rb,3.6133e-01_rb,4.4874e-04_rb/)
      kao(:, 2,11, 8) = (/ &
     &2.9950e+00_rb,2.6205e+00_rb,2.2462e+00_rb,1.8718e+00_rb,1.4974e+00_rb,1.1232e+00_rb, &
     &7.4873e-01_rb,3.7448e-01_rb,7.8243e-04_rb/)
      kao(:, 3,11, 8) = (/ &
     &3.1150e+00_rb,2.7257e+00_rb,2.3363e+00_rb,1.9467e+00_rb,1.5577e+00_rb,1.1681e+00_rb, &
     &7.7884e-01_rb,3.8967e-01_rb,1.2703e-03_rb/)
      kao(:, 4,11, 8) = (/ &
     &3.2669e+00_rb,2.8585e+00_rb,2.4505e+00_rb,2.0417e+00_rb,1.6337e+00_rb,1.2254e+00_rb, &
     &8.1715e-01_rb,4.0914e-01_rb,1.7634e-03_rb/)
      kao(:, 5,11, 8) = (/ &
     &3.4234e+00_rb,2.9957e+00_rb,2.5672e+00_rb,2.1393e+00_rb,1.7114e+00_rb,1.2836e+00_rb, &
     &8.5639e-01_rb,4.2894e-01_rb,1.6787e-03_rb/)
      kao(:, 1,12, 8) = (/ &
     &3.0768e+00_rb,2.6917e+00_rb,2.3070e+00_rb,1.9231e+00_rb,1.5383e+00_rb,1.1537e+00_rb, &
     &7.6915e-01_rb,3.8462e-01_rb,4.7094e-04_rb/)
      kao(:, 2,12, 8) = (/ &
     &3.2435e+00_rb,2.8382e+00_rb,2.4330e+00_rb,2.0274e+00_rb,1.6219e+00_rb,1.2164e+00_rb, &
     &8.1089e-01_rb,4.0548e-01_rb,8.2872e-04_rb/)
      kao(:, 3,12, 8) = (/ &
     &3.4231e+00_rb,2.9954e+00_rb,2.5672e+00_rb,2.1393e+00_rb,1.7115e+00_rb,1.2835e+00_rb, &
     &8.5578e-01_rb,4.2826e-01_rb,1.3467e-03_rb/)
      kao(:, 4,12, 8) = (/ &
     &3.5915e+00_rb,3.1425e+00_rb,2.6940e+00_rb,2.2448e+00_rb,1.7959e+00_rb,1.3469e+00_rb, &
     &8.9804e-01_rb,4.4980e-01_rb,2.0869e-03_rb/)
      kao(:, 5,12, 8) = (/ &
     &3.8078e+00_rb,3.3318e+00_rb,2.8557e+00_rb,2.3795e+00_rb,1.9039e+00_rb,1.4280e+00_rb, &
     &9.5252e-01_rb,4.7712e-01_rb,1.8673e-03_rb/)
      kao(:, 1,13, 8) = (/ &
     &2.3608e+00_rb,2.0656e+00_rb,1.7703e+00_rb,1.4752e+00_rb,1.1801e+00_rb,8.8512e-01_rb, &
     &5.9005e-01_rb,2.9502e-01_rb,4.6667e-04_rb/)
      kao(:, 2,13, 8) = (/ &
     &2.5500e+00_rb,2.2311e+00_rb,1.9119e+00_rb,1.5932e+00_rb,1.2746e+00_rb,9.5617e-01_rb, &
     &6.3722e-01_rb,3.1884e-01_rb,8.1387e-04_rb/)
      kao(:, 3,13, 8) = (/ &
     &2.7068e+00_rb,2.3688e+00_rb,2.0306e+00_rb,1.6919e+00_rb,1.3534e+00_rb,1.0151e+00_rb, &
     &6.7709e-01_rb,3.3887e-01_rb,1.3055e-03_rb/)
      kao(:, 4,13, 8) = (/ &
     &2.8840e+00_rb,2.5234e+00_rb,2.1627e+00_rb,1.8025e+00_rb,1.4417e+00_rb,1.0815e+00_rb, &
     &7.2137e-01_rb,3.6139e-01_rb,1.9116e-03_rb/)
      kao(:, 5,13, 8) = (/ &
     &3.0939e+00_rb,2.7071e+00_rb,2.3203e+00_rb,1.9334e+00_rb,1.5471e+00_rb,1.1606e+00_rb, &
     &7.7438e-01_rb,3.8815e-01_rb,1.4651e-03_rb/)
      kao(:, 1, 1, 9) = (/ &
     &3.8342e-03_rb,3.6037e-03_rb,3.3390e-03_rb,3.1415e-03_rb,3.0062e-03_rb,3.0639e-03_rb, &
     &3.3790e-03_rb,3.9710e-03_rb,4.6693e-03_rb/)
      kao(:, 2, 1, 9) = (/ &
     &3.8099e-03_rb,3.5245e-03_rb,3.4236e-03_rb,3.3696e-03_rb,3.4435e-03_rb,3.7521e-03_rb, &
     &4.3079e-03_rb,5.1949e-03_rb,5.9521e-03_rb/)
      kao(:, 3, 1, 9) = (/ &
     &3.6846e-03_rb,3.4976e-03_rb,3.5740e-03_rb,3.6729e-03_rb,4.0680e-03_rb,4.6302e-03_rb, &
     &5.4360e-03_rb,6.6553e-03_rb,7.4735e-03_rb/)
      kao(:, 4, 1, 9) = (/ &
     &3.5274e-03_rb,3.5376e-03_rb,3.7030e-03_rb,4.1336e-03_rb,4.8513e-03_rb,5.7165e-03_rb, &
     &6.7340e-03_rb,8.3554e-03_rb,9.2181e-03_rb/)
      kao(:, 5, 1, 9) = (/ &
     &3.3692e-03_rb,3.5348e-03_rb,3.9656e-03_rb,4.7179e-03_rb,5.7827e-03_rb,6.9797e-03_rb, &
     &8.2404e-03_rb,1.0337e-02_rb,1.1151e-02_rb/)
      kao(:, 1, 2, 9) = (/ &
     &6.4244e-03_rb,5.9003e-03_rb,5.2917e-03_rb,4.8738e-03_rb,4.2886e-03_rb,3.6979e-03_rb, &
     &3.3432e-03_rb,3.4356e-03_rb,4.2740e-03_rb/)
      kao(:, 2, 2, 9) = (/ &
     &6.4294e-03_rb,5.9365e-03_rb,5.4378e-03_rb,5.0910e-03_rb,4.5359e-03_rb,4.1466e-03_rb, &
     &4.0476e-03_rb,4.4354e-03_rb,5.4215e-03_rb/)
      kao(:, 3, 2, 9) = (/ &
     &6.3847e-03_rb,6.0135e-03_rb,5.5846e-03_rb,5.2573e-03_rb,4.8954e-03_rb,4.7657e-03_rb, &
     &4.9765e-03_rb,5.6368e-03_rb,6.8419e-03_rb/)
      kao(:, 4, 2, 9) = (/ &
     &6.3412e-03_rb,5.9177e-03_rb,5.7222e-03_rb,5.4979e-03_rb,5.4021e-03_rb,5.5794e-03_rb, &
     &6.1113e-03_rb,7.0879e-03_rb,8.4995e-03_rb/)
      kao(:, 5, 2, 9) = (/ &
     &6.1488e-03_rb,5.8853e-03_rb,5.8235e-03_rb,5.8427e-03_rb,6.0130e-03_rb,6.6088e-03_rb, &
     &7.4819e-03_rb,8.8092e-03_rb,1.0289e-02_rb/)
      kao(:, 1, 3, 9) = (/ &
     &1.4195e-02_rb,1.2643e-02_rb,1.1125e-02_rb,9.5338e-03_rb,8.0743e-03_rb,6.7413e-03_rb, &
     &5.1394e-03_rb,3.4808e-03_rb,3.6417e-03_rb/)
      kao(:, 2, 3, 9) = (/ &
     &1.4156e-02_rb,1.2714e-02_rb,1.1322e-02_rb,9.8573e-03_rb,8.5639e-03_rb,7.1930e-03_rb, &
     &5.4628e-03_rb,4.1430e-03_rb,4.8796e-03_rb/)
      kao(:, 3, 3, 9) = (/ &
     &1.4281e-02_rb,1.3014e-02_rb,1.1636e-02_rb,1.0282e-02_rb,9.0638e-03_rb,7.5306e-03_rb, &
     &5.9490e-03_rb,5.0136e-03_rb,6.2613e-03_rb/)
      kao(:, 4, 3, 9) = (/ &
     &1.4318e-02_rb,1.3194e-02_rb,1.1832e-02_rb,1.0698e-02_rb,9.4111e-03_rb,7.9426e-03_rb, &
     &6.6489e-03_rb,6.1359e-03_rb,7.8675e-03_rb/)
      kao(:, 5, 3, 9) = (/ &
     &1.4241e-02_rb,1.3300e-02_rb,1.2028e-02_rb,1.0934e-02_rb,9.7284e-03_rb,8.5621e-03_rb, &
     &7.5356e-03_rb,7.5232e-03_rb,9.7190e-03_rb/)
      kao(:, 1, 4, 9) = (/ &
     &3.1371e-02_rb,2.7594e-02_rb,2.3918e-02_rb,2.0236e-02_rb,1.6538e-02_rb,1.2862e-02_rb, &
     &9.3415e-03_rb,5.6206e-03_rb,3.0136e-03_rb/)
      kao(:, 2, 4, 9) = (/ &
     &3.1367e-02_rb,2.7706e-02_rb,2.4150e-02_rb,2.0624e-02_rb,1.6974e-02_rb,1.3427e-02_rb, &
     &9.9709e-03_rb,6.1282e-03_rb,4.2995e-03_rb/)
      kao(:, 3, 4, 9) = (/ &
     &3.1583e-02_rb,2.8007e-02_rb,2.4625e-02_rb,2.1088e-02_rb,1.7481e-02_rb,1.4076e-02_rb, &
     &1.0648e-02_rb,6.6037e-03_rb,5.6873e-03_rb/)
      kao(:, 4, 4, 9) = (/ &
     &3.1875e-02_rb,2.8446e-02_rb,2.5140e-02_rb,2.1570e-02_rb,1.8204e-02_rb,1.5013e-02_rb, &
     &1.1353e-02_rb,7.2455e-03_rb,7.5289e-03_rb/)
      kao(:, 5, 4, 9) = (/ &
     &3.2159e-02_rb,2.8870e-02_rb,2.5760e-02_rb,2.2306e-02_rb,1.9019e-02_rb,1.5749e-02_rb, &
     &1.1959e-02_rb,8.0455e-03_rb,9.2604e-03_rb/)
      kao(:, 1, 5, 9) = (/ &
     &6.1996e-02_rb,5.4276e-02_rb,4.6766e-02_rb,3.9233e-02_rb,3.1722e-02_rb,2.4192e-02_rb, &
     &1.6676e-02_rb,9.3526e-03_rb,2.3723e-03_rb/)
      kao(:, 2, 5, 9) = (/ &
     &6.2786e-02_rb,5.5036e-02_rb,4.7544e-02_rb,4.0014e-02_rb,3.2500e-02_rb,2.4919e-02_rb, &
     &1.7414e-02_rb,9.9958e-03_rb,3.1775e-03_rb/)
      kao(:, 3, 5, 9) = (/ &
     &6.3408e-02_rb,5.5719e-02_rb,4.8263e-02_rb,4.0805e-02_rb,3.3340e-02_rb,2.5705e-02_rb, &
     &1.8217e-02_rb,1.0807e-02_rb,4.4457e-03_rb/)
      kao(:, 4, 5, 9) = (/ &
     &6.3958e-02_rb,5.6455e-02_rb,4.8967e-02_rb,4.1701e-02_rb,3.4289e-02_rb,2.6801e-02_rb, &
     &1.9500e-02_rb,1.1614e-02_rb,6.0592e-03_rb/)
      kao(:, 5, 5, 9) = (/ &
     &6.4544e-02_rb,5.7091e-02_rb,4.9896e-02_rb,4.2837e-02_rb,3.5365e-02_rb,2.7903e-02_rb, &
     &2.0589e-02_rb,1.2558e-02_rb,7.7840e-03_rb/)
      kao(:, 1, 6, 9) = (/ &
     &1.1420e-01_rb,9.9928e-02_rb,8.5730e-02_rb,7.1669e-02_rb,5.7641e-02_rb,4.3533e-02_rb, &
     &2.9463e-02_rb,1.5596e-02_rb,1.8870e-03_rb/)
      kao(:, 2, 6, 9) = (/ &
     &1.1463e-01_rb,1.0031e-01_rb,8.6249e-02_rb,7.2249e-02_rb,5.8195e-02_rb,4.4132e-02_rb, &
     &3.0125e-02_rb,1.6255e-02_rb,2.7185e-03_rb/)
      kao(:, 3, 6, 9) = (/ &
     &1.1505e-01_rb,1.0074e-01_rb,8.6836e-02_rb,7.2850e-02_rb,5.8886e-02_rb,4.5056e-02_rb, &
     &3.1103e-02_rb,1.7221e-02_rb,3.7627e-03_rb/)
      kao(:, 4, 6, 9) = (/ &
     &1.1693e-01_rb,1.0252e-01_rb,8.8568e-02_rb,7.4567e-02_rb,6.0606e-02_rb,4.6540e-02_rb, &
     &3.2424e-02_rb,1.8275e-02_rb,5.0267e-03_rb/)
      kao(:, 5, 6, 9) = (/ &
     &1.1806e-01_rb,1.0371e-01_rb,8.9817e-02_rb,7.5865e-02_rb,6.1963e-02_rb,4.7866e-02_rb, &
     &3.3695e-02_rb,1.9500e-02_rb,6.8317e-03_rb/)
      kao(:, 1, 7, 9) = (/ &
     &2.1427e-01_rb,1.8749e-01_rb,1.6070e-01_rb,1.3399e-01_rb,1.0738e-01_rb,8.0867e-02_rb, &
     &5.4264e-02_rb,2.7682e-02_rb,1.2395e-03_rb/)
      kao(:, 2, 7, 9) = (/ &
     &2.1459e-01_rb,1.8775e-01_rb,1.6094e-01_rb,1.3438e-01_rb,1.0794e-01_rb,8.1406e-02_rb, &
     &5.4844e-02_rb,2.8427e-02_rb,2.0965e-03_rb/)
      kao(:, 3, 7, 9) = (/ &
     &2.1615e-01_rb,1.8914e-01_rb,1.6231e-01_rb,1.3573e-01_rb,1.0921e-01_rb,8.2571e-02_rb, &
     &5.5952e-02_rb,2.9582e-02_rb,3.1925e-03_rb/)
      kao(:, 4, 7, 9) = (/ &
     &2.1891e-01_rb,1.9155e-01_rb,1.6470e-01_rb,1.3802e-01_rb,1.1113e-01_rb,8.4390e-02_rb, &
     &5.7587e-02_rb,3.0908e-02_rb,4.5715e-03_rb/)
      kao(:, 5, 7, 9) = (/ &
     &2.2155e-01_rb,1.9400e-01_rb,1.6713e-01_rb,1.4027e-01_rb,1.1333e-01_rb,8.6454e-02_rb, &
     &5.9526e-02_rb,3.2445e-02_rb,6.5617e-03_rb/)
      kao(:, 1, 8, 9) = (/ &
     &4.6547e-01_rb,4.0732e-01_rb,3.4911e-01_rb,2.9100e-01_rb,2.3277e-01_rb,1.7470e-01_rb, &
     &1.1672e-01_rb,5.8764e-02_rb,8.5250e-04_rb/)
      kao(:, 2, 8, 9) = (/ &
     &4.6469e-01_rb,4.0659e-01_rb,3.4854e-01_rb,2.9045e-01_rb,2.3244e-01_rb,1.7466e-01_rb, &
     &1.1705e-01_rb,5.9152e-02_rb,1.3360e-03_rb/)
      kao(:, 3, 8, 9) = (/ &
     &4.6927e-01_rb,4.1062e-01_rb,3.5196e-01_rb,2.9346e-01_rb,2.3515e-01_rb,1.7702e-01_rb, &
     &1.1874e-01_rb,6.0462e-02_rb,2.4435e-03_rb/)
      kao(:, 4, 8, 9) = (/ &
     &4.7393e-01_rb,4.1463e-01_rb,3.5541e-01_rb,2.9668e-01_rb,2.3809e-01_rb,1.7953e-01_rb, &
     &1.2082e-01_rb,6.2242e-02_rb,3.3995e-03_rb/)
      kao(:, 5, 8, 9) = (/ &
     &4.8023e-01_rb,4.2020e-01_rb,3.6048e-01_rb,3.0117e-01_rb,2.4212e-01_rb,1.8271e-01_rb, &
     &1.2350e-01_rb,6.4451e-02_rb,5.1655e-03_rb/)
      kao(:, 1, 9, 9) = (/ &
     &1.9039e+00_rb,1.6660e+00_rb,1.4281e+00_rb,1.1899e+00_rb,9.5193e-01_rb,7.1405e-01_rb, &
     &4.7606e-01_rb,2.3807e-01_rb,5.1784e-04_rb/)
      kao(:, 2, 9, 9) = (/ &
     &1.8960e+00_rb,1.6589e+00_rb,1.4220e+00_rb,1.1850e+00_rb,9.4792e-01_rb,7.1098e-01_rb, &
     &4.7399e-01_rb,2.3745e-01_rb,1.0379e-03_rb/)
      kao(:, 3, 9, 9) = (/ &
     &1.8985e+00_rb,1.6610e+00_rb,1.4239e+00_rb,1.1866e+00_rb,9.4922e-01_rb,7.1187e-01_rb, &
     &4.7501e-01_rb,2.3850e-01_rb,1.9956e-03_rb/)
      kao(:, 4, 9, 9) = (/ &
     &1.9331e+00_rb,1.6922e+00_rb,1.4503e+00_rb,1.2084e+00_rb,9.6666e-01_rb,7.2514e-01_rb, &
     &4.8432e-01_rb,2.4379e-01_rb,3.3200e-03_rb/)
      kao(:, 5, 9, 9) = (/ &
     &1.9563e+00_rb,1.7116e+00_rb,1.4675e+00_rb,1.2227e+00_rb,9.7814e-01_rb,7.3426e-01_rb, &
     &4.9112e-01_rb,2.4753e-01_rb,5.3379e-03_rb/)
      kao(:, 1,10, 9) = (/ &
     &8.5901e+00_rb,7.5167e+00_rb,6.4427e+00_rb,5.3701e+00_rb,4.2955e+00_rb,3.2213e+00_rb, &
     &2.1478e+00_rb,1.0738e+00_rb,2.8232e-05_rb/)
      kao(:, 2,10, 9) = (/ &
     &8.5325e+00_rb,7.4671e+00_rb,6.4018e+00_rb,5.3343e+00_rb,4.2664e+00_rb,3.2001e+00_rb, &
     &2.1333e+00_rb,1.0667e+00_rb,3.4266e-05_rb/)
      kao(:, 3,10, 9) = (/ &
     &8.5180e+00_rb,7.4539e+00_rb,6.3892e+00_rb,5.3241e+00_rb,4.2587e+00_rb,3.1944e+00_rb, &
     &2.1293e+00_rb,1.0648e+00_rb,4.9015e-05_rb/)
      kao(:, 4,10, 9) = (/ &
     &8.6373e+00_rb,7.5561e+00_rb,6.4798e+00_rb,5.3988e+00_rb,4.3190e+00_rb,3.2393e+00_rb, &
     &2.1596e+00_rb,1.0801e+00_rb,1.1052e-03_rb/)
      kao(:, 5,10, 9) = (/ &
     &8.7651e+00_rb,7.6691e+00_rb,6.5729e+00_rb,5.4773e+00_rb,4.3830e+00_rb,3.2869e+00_rb, &
     &2.1915e+00_rb,1.0965e+00_rb,2.0612e-03_rb/)
      kao(:, 1,11, 9) = (/ &
     &1.3293e+01_rb,1.1631e+01_rb,9.9694e+00_rb,8.3080e+00_rb,6.6463e+00_rb,4.9848e+00_rb, &
     &3.3235e+00_rb,1.6616e+00_rb,8.6041e-05_rb/)
      kao(:, 2,11, 9) = (/ &
     &1.3175e+01_rb,1.1527e+01_rb,9.8819e+00_rb,8.2359e+00_rb,6.5871e+00_rb,4.9407e+00_rb, &
     &3.2935e+00_rb,1.6469e+00_rb,8.8405e-05_rb/)
      kao(:, 3,11, 9) = (/ &
     &1.3260e+01_rb,1.1601e+01_rb,9.9453e+00_rb,8.2878e+00_rb,6.6300e+00_rb,4.9725e+00_rb, &
     &3.3155e+00_rb,1.6576e+00_rb,7.6661e-05_rb/)
      kao(:, 4,11, 9) = (/ &
     &1.3414e+01_rb,1.1737e+01_rb,1.0061e+01_rb,8.3841e+00_rb,6.7068e+00_rb,5.0294e+00_rb, &
     &3.3528e+00_rb,1.6766e+00_rb,3.1506e-04_rb/)
      kao(:, 5,11, 9) = (/ &
     &1.3624e+01_rb,1.1922e+01_rb,1.0219e+01_rb,8.5160e+00_rb,6.8128e+00_rb,5.1095e+00_rb, &
     &3.4058e+00_rb,1.7038e+00_rb,1.6944e-03_rb/)
      kao(:, 1,12, 9) = (/ &
     &1.4782e+01_rb,1.2935e+01_rb,1.1089e+01_rb,9.2388e+00_rb,7.3920e+00_rb,5.5445e+00_rb, &
     &3.6962e+00_rb,1.8480e+00_rb,1.3867e-04_rb/)
      kao(:, 2,12, 9) = (/ &
     &1.4730e+01_rb,1.2889e+01_rb,1.1047e+01_rb,9.2076e+00_rb,7.3659e+00_rb,5.5246e+00_rb, &
     &3.6830e+00_rb,1.8415e+00_rb,1.5061e-04_rb/)
      kao(:, 3,12, 9) = (/ &
     &1.4865e+01_rb,1.3005e+01_rb,1.1148e+01_rb,9.2913e+00_rb,7.4319e+00_rb,5.5733e+00_rb, &
     &3.7153e+00_rb,1.8580e+00_rb,1.4666e-04_rb/)
      kao(:, 4,12, 9) = (/ &
     &1.5163e+01_rb,1.3269e+01_rb,1.1374e+01_rb,9.4785e+00_rb,7.5816e+00_rb,5.6866e+00_rb, &
     &3.7915e+00_rb,1.8957e+00_rb,1.4926e-04_rb/)
      kao(:, 5,12, 9) = (/ &
     &1.5423e+01_rb,1.3495e+01_rb,1.1567e+01_rb,9.6394e+00_rb,7.7115e+00_rb,5.7838e+00_rb, &
     &3.8558e+00_rb,1.9287e+00_rb,1.8983e-03_rb/)
      kao(:, 1,13, 9) = (/ &
     &1.2526e+01_rb,1.0960e+01_rb,9.3953e+00_rb,7.8295e+00_rb,6.2636e+00_rb,4.6971e+00_rb, &
     &3.1319e+00_rb,1.5656e+00_rb,2.2960e-04_rb/)
      kao(:, 2,13, 9) = (/ &
     &1.2594e+01_rb,1.1020e+01_rb,9.4466e+00_rb,7.8734e+00_rb,6.2977e+00_rb,4.7229e+00_rb, &
     &3.1489e+00_rb,1.5743e+00_rb,3.0250e-04_rb/)
      kao(:, 3,13, 9) = (/ &
     &1.2808e+01_rb,1.1205e+01_rb,9.6060e+00_rb,8.0068e+00_rb,6.4043e+00_rb,4.8030e+00_rb, &
     &3.2030e+00_rb,1.6008e+00_rb,4.1712e-04_rb/)
      kao(:, 4,13, 9) = (/ &
     &1.3107e+01_rb,1.1466e+01_rb,9.8274e+00_rb,8.1873e+00_rb,6.5536e+00_rb,4.9140e+00_rb, &
     &3.2751e+00_rb,1.6381e+00_rb,6.5679e-04_rb/)
      kao(:, 5,13, 9) = (/ &
     &1.3444e+01_rb,1.1761e+01_rb,1.0081e+01_rb,8.4011e+00_rb,6.7204e+00_rb,5.0404e+00_rb, &
     &3.3592e+00_rb,1.6815e+00_rb,2.8416e-03_rb/)
      kao(:, 1, 1,10) = (/ &
     &4.6973e-03_rb,3.3679e-03_rb,3.0810e-03_rb,4.4179e-03_rb,5.3922e-03_rb,6.8127e-03_rb, &
     &7.5300e-03_rb,9.6520e-03_rb,1.0742e-02_rb/)
      kao(:, 2, 1,10) = (/ &
     &3.3940e-03_rb,3.6347e-03_rb,3.8181e-03_rb,5.7069e-03_rb,7.1993e-03_rb,9.1516e-03_rb, &
     &1.0242e-02_rb,1.2914e-02_rb,1.4284e-02_rb/)
      kao(:, 3, 1,10) = (/ &
     &2.9328e-03_rb,3.5249e-03_rb,5.0061e-03_rb,7.3569e-03_rb,8.9474e-03_rb,1.1546e-02_rb, &
     &1.2963e-02_rb,1.7115e-02_rb,1.7551e-02_rb/)
      kao(:, 4, 1,10) = (/ &
     &2.4964e-03_rb,3.6551e-03_rb,6.6228e-03_rb,8.9461e-03_rb,1.0774e-02_rb,1.3965e-02_rb, &
     &1.7435e-02_rb,2.2332e-02_rb,2.1398e-02_rb/)
      kao(:, 5, 1,10) = (/ &
     &2.1304e-03_rb,4.5161e-03_rb,8.4664e-03_rb,1.0811e-02_rb,1.3640e-02_rb,1.7099e-02_rb, &
     &2.2735e-02_rb,2.8461e-02_rb,2.7259e-02_rb/)
      kao(:, 1, 2,10) = (/ &
     &9.4205e-03_rb,8.2881e-03_rb,6.8055e-03_rb,5.1998e-03_rb,6.0003e-03_rb,6.4194e-03_rb, &
     &7.3653e-03_rb,8.1269e-03_rb,1.1868e-02_rb/)
      kao(:, 2, 2,10) = (/ &
     &8.5761e-03_rb,7.3303e-03_rb,5.6742e-03_rb,5.4200e-03_rb,7.5061e-03_rb,8.7164e-03_rb, &
     &9.8614e-03_rb,1.1132e-02_rb,1.4929e-02_rb/)
      kao(:, 3, 2,10) = (/ &
     &7.2342e-03_rb,5.9377e-03_rb,5.5335e-03_rb,7.0973e-03_rb,9.3181e-03_rb,1.1104e-02_rb, &
     &1.2752e-02_rb,1.5333e-02_rb,1.8200e-02_rb/)
      kao(:, 4, 2,10) = (/ &
     &5.3015e-03_rb,5.6105e-03_rb,6.0212e-03_rb,9.3735e-03_rb,1.1344e-02_rb,1.3845e-02_rb, &
     &1.6350e-02_rb,2.0205e-02_rb,2.2040e-02_rb/)
      kao(:, 5, 2,10) = (/ &
     &4.6868e-03_rb,5.1363e-03_rb,7.4969e-03_rb,1.0079e-02_rb,1.4155e-02_rb,1.6850e-02_rb, &
     &1.9756e-02_rb,2.5412e-02_rb,2.7831e-02_rb/)
      kao(:, 1, 3,10) = (/ &
     &2.2659e-02_rb,2.0083e-02_rb,1.7749e-02_rb,1.5132e-02_rb,1.2304e-02_rb,9.8131e-03_rb, &
     &6.5475e-03_rb,7.6061e-03_rb,9.2024e-03_rb/)
      kao(:, 2, 3,10) = (/ &
     &2.3012e-02_rb,2.0379e-02_rb,1.7548e-02_rb,1.4904e-02_rb,1.1694e-02_rb,9.3796e-03_rb, &
     &9.2412e-03_rb,1.0030e-02_rb,1.2744e-02_rb/)
      kao(:, 3, 3,10) = (/ &
     &2.1396e-02_rb,1.8727e-02_rb,1.6107e-02_rb,1.2456e-02_rb,1.0255e-02_rb,1.0536e-02_rb, &
     &1.1966e-02_rb,1.3095e-02_rb,1.7259e-02_rb/)
      kao(:, 4, 3,10) = (/ &
     &1.9331e-02_rb,1.6488e-02_rb,1.2963e-02_rb,1.0270e-02_rb,1.1515e-02_rb,1.2315e-02_rb, &
     &1.5198e-02_rb,1.6672e-02_rb,2.2500e-02_rb/)
      kao(:, 5, 3,10) = (/ &
     &1.6551e-02_rb,1.3344e-02_rb,1.1001e-02_rb,1.0385e-02_rb,1.2450e-02_rb,1.4894e-02_rb, &
     &1.8912e-02_rb,2.0849e-02_rb,2.4782e-02_rb/)
      kao(:, 1, 4,10) = (/ &
     &5.3443e-02_rb,4.6752e-02_rb,4.0212e-02_rb,3.3727e-02_rb,2.6764e-02_rb,2.0366e-02_rb, &
     &1.4100e-02_rb,8.5626e-03_rb,5.0179e-03_rb/)
      kao(:, 2, 4,10) = (/ &
     &5.5520e-02_rb,4.8788e-02_rb,4.2227e-02_rb,3.5776e-02_rb,2.9164e-02_rb,2.2326e-02_rb, &
     &1.5452e-02_rb,9.1701e-03_rb,6.6031e-03_rb/)
      kao(:, 3, 4,10) = (/ &
     &5.3841e-02_rb,4.7471e-02_rb,4.1338e-02_rb,3.5633e-02_rb,2.9297e-02_rb,2.2785e-02_rb, &
     &1.5793e-02_rb,1.1481e-02_rb,9.9885e-03_rb/)
      kao(:, 4, 4,10) = (/ &
     &5.4923e-02_rb,4.8665e-02_rb,4.2046e-02_rb,3.5458e-02_rb,2.8786e-02_rb,2.1135e-02_rb, &
     &1.3946e-02_rb,1.4729e-02_rb,1.3190e-02_rb/)
      kao(:, 5, 4,10) = (/ &
     &5.1650e-02_rb,4.6045e-02_rb,4.0319e-02_rb,3.3252e-02_rb,2.5245e-02_rb,1.9132e-02_rb, &
     &1.5667e-02_rb,1.8634e-02_rb,1.9277e-02_rb/)
      kao(:, 1, 5,10) = (/ &
     &1.1780e-01_rb,1.0308e-01_rb,8.8391e-02_rb,7.4152e-02_rb,5.9704e-02_rb,4.5204e-02_rb, &
     &3.0807e-02_rb,1.6385e-02_rb,4.0089e-03_rb/)
      kao(:, 2, 5,10) = (/ &
     &1.1261e-01_rb,9.8543e-02_rb,8.5059e-02_rb,7.1216e-02_rb,5.7505e-02_rb,4.3649e-02_rb, &
     &3.0528e-02_rb,1.7529e-02_rb,9.3093e-03_rb/)
      kao(:, 3, 5,10) = (/ &
     &1.1400e-01_rb,9.9742e-02_rb,8.6449e-02_rb,7.3042e-02_rb,5.9615e-02_rb,4.6638e-02_rb, &
     &3.2532e-02_rb,1.8567e-02_rb,1.1184e-02_rb/)
      kao(:, 4, 5,10) = (/ &
     &1.1613e-01_rb,1.0163e-01_rb,8.9121e-02_rb,7.4876e-02_rb,6.0453e-02_rb,4.6196e-02_rb, &
     &3.1216e-02_rb,1.9048e-02_rb,1.2339e-02_rb/)
      kao(:, 5, 5,10) = (/ &
     &1.2016e-01_rb,1.0608e-01_rb,9.2375e-02_rb,7.6465e-02_rb,6.2248e-02_rb,4.8476e-02_rb, &
     &3.3522e-02_rb,1.7446e-02_rb,1.7040e-02_rb/)
      kao(:, 1, 6,10) = (/ &
     &2.3387e-01_rb,2.0464e-01_rb,1.7541e-01_rb,1.4624e-01_rb,1.1769e-01_rb,8.8692e-02_rb, &
     &5.9747e-02_rb,3.0756e-02_rb,2.5605e-03_rb/)
      kao(:, 2, 6,10) = (/ &
     &2.3730e-01_rb,2.0756e-01_rb,1.7772e-01_rb,1.4892e-01_rb,1.1985e-01_rb,9.0758e-02_rb, &
     &6.1778e-02_rb,3.2933e-02_rb,4.6893e-03_rb/)
      kao(:, 3, 6,10) = (/ &
     &2.4203e-01_rb,2.1158e-01_rb,1.8155e-01_rb,1.5250e-01_rb,1.2284e-01_rb,9.2705e-02_rb, &
     &6.3067e-02_rb,3.2776e-02_rb,9.7345e-03_rb/)
      kao(:, 4, 6,10) = (/ &
     &2.3049e-01_rb,2.0160e-01_rb,1.7416e-01_rb,1.4569e-01_rb,1.1768e-01_rb,8.9784e-02_rb, &
     &6.1642e-02_rb,3.4682e-02_rb,1.6936e-02_rb/)
      kao(:, 5, 6,10) = (/ &
     &2.3636e-01_rb,2.0679e-01_rb,1.7932e-01_rb,1.5154e-01_rb,1.2337e-01_rb,9.4475e-02_rb, &
     &6.5584e-02_rb,3.6305e-02_rb,2.2069e-02_rb/)
      kao(:, 1, 7,10) = (/ &
     &4.9443e-01_rb,4.3211e-01_rb,3.7050e-01_rb,3.0887e-01_rb,2.4699e-01_rb,1.8540e-01_rb, &
     &1.2438e-01_rb,6.2986e-02_rb,2.0096e-03_rb/)
      kao(:, 2, 7,10) = (/ &
     &4.9639e-01_rb,4.3473e-01_rb,3.7266e-01_rb,3.1039e-01_rb,2.4852e-01_rb,1.8749e-01_rb, &
     &1.2568e-01_rb,6.4131e-02_rb,3.0764e-03_rb/)
      kao(:, 3, 7,10) = (/ &
     &4.9265e-01_rb,4.3128e-01_rb,3.6987e-01_rb,3.0823e-01_rb,2.4792e-01_rb,1.8671e-01_rb, &
     &1.2562e-01_rb,6.4213e-02_rb,5.4655e-03_rb/)
      kao(:, 4, 7,10) = (/ &
     &4.7703e-01_rb,4.1742e-01_rb,3.5749e-01_rb,2.9850e-01_rb,2.4081e-01_rb,1.8239e-01_rb, &
     &1.2380e-01_rb,6.5802e-02_rb,1.0925e-02_rb/)
      kao(:, 5, 7,10) = (/ &
     &4.8180e-01_rb,4.2216e-01_rb,3.6195e-01_rb,3.0277e-01_rb,2.4364e-01_rb,1.8402e-01_rb, &
     &1.2523e-01_rb,6.7909e-02_rb,1.4296e-02_rb/)
      kao(:, 1, 8,10) = (/ &
     &1.1962e+00_rb,1.0464e+00_rb,8.9717e-01_rb,7.4708e-01_rb,5.9764e-01_rb,4.4825e-01_rb, &
     &2.9884e-01_rb,1.5061e-01_rb,3.8986e-03_rb/)
      kao(:, 2, 8,10) = (/ &
     &1.2041e+00_rb,1.0536e+00_rb,9.0306e-01_rb,7.5220e-01_rb,6.0244e-01_rb,4.5171e-01_rb, &
     &3.0103e-01_rb,1.5197e-01_rb,3.3573e-03_rb/)
      kao(:, 3, 8,10) = (/ &
     &1.1525e+00_rb,1.0084e+00_rb,8.6433e-01_rb,7.2027e-01_rb,5.7631e-01_rb,4.3217e-01_rb, &
     &2.8996e-01_rb,1.4673e-01_rb,3.4045e-03_rb/)
      kao(:, 4, 8,10) = (/ &
     &1.1627e+00_rb,1.0178e+00_rb,8.7243e-01_rb,7.2665e-01_rb,5.8167e-01_rb,4.3823e-01_rb, &
     &2.9244e-01_rb,1.4815e-01_rb,8.5595e-03_rb/)
      kao(:, 5, 8,10) = (/ &
     &1.1683e+00_rb,1.0228e+00_rb,8.7612e-01_rb,7.3014e-01_rb,5.8445e-01_rb,4.4237e-01_rb, &
     &2.9748e-01_rb,1.5239e-01_rb,1.0748e-02_rb/)
      kao(:, 1, 9,10) = (/ &
     &5.3473e+00_rb,4.6810e+00_rb,4.0088e+00_rb,3.3420e+00_rb,2.6745e+00_rb,2.0052e+00_rb, &
     &1.3357e+00_rb,6.6899e-01_rb,9.0646e-06_rb/)
      kao(:, 2, 9,10) = (/ &
     &5.3849e+00_rb,4.7131e+00_rb,4.0386e+00_rb,3.3668e+00_rb,2.6936e+00_rb,2.0203e+00_rb, &
     &1.3467e+00_rb,6.7337e-01_rb,1.6160e-05_rb/)
      kao(:, 3, 9,10) = (/ &
     &5.2604e+00_rb,4.6046e+00_rb,3.9456e+00_rb,3.2881e+00_rb,2.6303e+00_rb,1.9734e+00_rb, &
     &1.3151e+00_rb,6.5758e-01_rb,2.7795e-05_rb/)
      kao(:, 4, 9,10) = (/ &
     &4.9950e+00_rb,4.3651e+00_rb,3.7432e+00_rb,3.1211e+00_rb,2.4979e+00_rb,1.8723e+00_rb, &
     &1.2477e+00_rb,6.2561e-01_rb,4.1978e-05_rb/)
      kao(:, 5, 9,10) = (/ &
     &5.1108e+00_rb,4.4671e+00_rb,3.8297e+00_rb,3.1913e+00_rb,2.5537e+00_rb,1.9170e+00_rb, &
     &1.2756e+00_rb,6.4540e-01_rb,6.6209e-05_rb/)
      kao(:, 1,10,10) = (/ &
     &2.6042e+01_rb,2.2785e+01_rb,1.9540e+01_rb,1.6264e+01_rb,1.3016e+01_rb,9.7654e+00_rb, &
     &6.5078e+00_rb,3.2552e+00_rb,5.7707e-06_rb/)
      kao(:, 2,10,10) = (/ &
     &2.6479e+01_rb,2.3167e+01_rb,1.9837e+01_rb,1.6530e+01_rb,1.3239e+01_rb,9.9265e+00_rb, &
     &6.6195e+00_rb,3.3086e+00_rb,1.0081e-05_rb/)
      kao(:, 3,10,10) = (/ &
     &2.6009e+01_rb,2.2746e+01_rb,1.9506e+01_rb,1.6255e+01_rb,1.3009e+01_rb,9.7479e+00_rb, &
     &6.5045e+00_rb,3.2511e+00_rb,1.6976e-05_rb/)
      kao(:, 4,10,10) = (/ &
     &2.4905e+01_rb,2.1818e+01_rb,1.8660e+01_rb,1.5572e+01_rb,1.2451e+01_rb,9.3433e+00_rb, &
     &6.2220e+00_rb,3.1125e+00_rb,2.8117e-05_rb/)
      kao(:, 5,10,10) = (/ &
     &2.5169e+01_rb,2.2024e+01_rb,1.8885e+01_rb,1.5746e+01_rb,1.2587e+01_rb,9.4388e+00_rb, &
     &6.2934e+00_rb,3.1427e+00_rb,4.4861e-05_rb/)
      kao(:, 1,11,10) = (/ &
     &4.3366e+01_rb,3.7949e+01_rb,3.2540e+01_rb,2.7105e+01_rb,2.1683e+01_rb,1.6263e+01_rb, &
     &1.0837e+01_rb,5.4210e+00_rb,4.9323e-06_rb/)
      kao(:, 2,11,10) = (/ &
     &4.4481e+01_rb,3.8935e+01_rb,3.3362e+01_rb,2.7791e+01_rb,2.2247e+01_rb,1.6680e+01_rb, &
     &1.1125e+01_rb,5.5599e+00_rb,8.9387e-06_rb/)
      kao(:, 3,11,10) = (/ &
     &4.2759e+01_rb,3.7428e+01_rb,3.2070e+01_rb,2.6725e+01_rb,2.1369e+01_rb,1.6026e+01_rb, &
     &1.0686e+01_rb,5.3424e+00_rb,1.3904e-05_rb/)
      kao(:, 4,11,10) = (/ &
     &4.2776e+01_rb,3.7387e+01_rb,3.2051e+01_rb,2.6735e+01_rb,2.1399e+01_rb,1.6039e+01_rb, &
     &1.0699e+01_rb,5.3492e+00_rb,2.3732e-05_rb/)
      kao(:, 5,11,10) = (/ &
     &4.2881e+01_rb,3.7499e+01_rb,3.2160e+01_rb,2.6814e+01_rb,2.1427e+01_rb,1.6080e+01_rb, &
     &1.0724e+01_rb,5.3602e+00_rb,3.9053e-05_rb/)
      kao(:, 1,12,10) = (/ &
     &5.2035e+01_rb,4.5530e+01_rb,3.8991e+01_rb,3.2521e+01_rb,2.6007e+01_rb,1.9502e+01_rb, &
     &1.3003e+01_rb,6.5017e+00_rb,4.1086e-06_rb/)
      kao(:, 2,12,10) = (/ &
     &5.2620e+01_rb,4.6040e+01_rb,3.9465e+01_rb,3.2876e+01_rb,2.6300e+01_rb,1.9726e+01_rb, &
     &1.3150e+01_rb,6.5751e+00_rb,7.6006e-06_rb/)
      kao(:, 3,12,10) = (/ &
     &5.1699e+01_rb,4.5233e+01_rb,3.8772e+01_rb,3.2297e+01_rb,2.5847e+01_rb,1.9404e+01_rb, &
     &1.2928e+01_rb,6.4618e+00_rb,1.3155e-05_rb/)
      kao(:, 4,12,10) = (/ &
     &5.1530e+01_rb,4.5092e+01_rb,3.8615e+01_rb,3.2196e+01_rb,2.5751e+01_rb,1.9320e+01_rb, &
     &1.2877e+01_rb,6.4388e+00_rb,2.2125e-05_rb/)
      kao(:, 5,12,10) = (/ &
     &5.2358e+01_rb,4.5788e+01_rb,3.9246e+01_rb,3.2725e+01_rb,2.6162e+01_rb,1.9634e+01_rb, &
     &1.3083e+01_rb,6.5448e+00_rb,3.4034e-05_rb/)
      kao(:, 1,13,10) = (/ &
     &4.8070e+01_rb,4.2058e+01_rb,3.6028e+01_rb,3.0041e+01_rb,2.4032e+01_rb,1.8020e+01_rb, &
     &1.2016e+01_rb,6.0067e+00_rb,1.1683e-05_rb/)
      kao(:, 2,13,10) = (/ &
     &4.7586e+01_rb,4.1613e+01_rb,3.5689e+01_rb,2.9748e+01_rb,2.3780e+01_rb,1.7835e+01_rb, &
     &1.1890e+01_rb,5.9450e+00_rb,6.6906e-06_rb/)
      kao(:, 3,13,10) = (/ &
     &4.7760e+01_rb,4.1804e+01_rb,3.5818e+01_rb,2.9821e+01_rb,2.3863e+01_rb,1.7898e+01_rb, &
     &1.1923e+01_rb,5.9715e+00_rb,1.1688e-05_rb/)
      kao(:, 4,13,10) = (/ &
     &4.7976e+01_rb,4.1998e+01_rb,3.6011e+01_rb,3.0021e+01_rb,2.3989e+01_rb,1.7987e+01_rb, &
     &1.2008e+01_rb,6.0018e+00_rb,2.0801e-05_rb/)
      kao(:, 5,13,10) = (/ &
     &4.8185e+01_rb,4.2179e+01_rb,3.6131e+01_rb,3.0110e+01_rb,2.4087e+01_rb,1.8066e+01_rb, &
     &1.2054e+01_rb,6.0193e+00_rb,3.3629e-05_rb/)
      kao(:, 1, 1,11) = (/ &
     &2.3144e-03_rb,3.0235e-03_rb,4.2030e-03_rb,4.5486e-03_rb,6.9673e-03_rb,8.5656e-03_rb, &
     &1.0150e-02_rb,1.3548e-02_rb,1.3475e-02_rb/)
      kao(:, 2, 1,11) = (/ &
     &2.0416e-03_rb,3.1999e-03_rb,4.6569e-03_rb,6.4031e-03_rb,9.8267e-03_rb,1.1395e-02_rb, &
     &1.5022e-02_rb,1.8877e-02_rb,1.8904e-02_rb/)
      kao(:, 3, 1,11) = (/ &
     &1.8509e-03_rb,4.2185e-03_rb,5.5775e-03_rb,9.1095e-03_rb,1.2979e-02_rb,1.4639e-02_rb, &
     &2.1829e-02_rb,2.5577e-02_rb,2.5272e-02_rb/)
      kao(:, 4, 1,11) = (/ &
     &1.7995e-03_rb,5.2396e-03_rb,7.4739e-03_rb,1.1646e-02_rb,1.7153e-02_rb,1.9747e-02_rb, &
     &2.8712e-02_rb,3.3739e-02_rb,3.3147e-02_rb/)
      kao(:, 5, 1,11) = (/ &
     &1.8412e-03_rb,6.1417e-03_rb,9.5119e-03_rb,1.5457e-02_rb,2.1034e-02_rb,2.5800e-02_rb, &
     &3.6795e-02_rb,4.3339e-02_rb,4.2100e-02_rb/)
      kao(:, 1, 2,11) = (/ &
     &7.4427e-03_rb,5.5490e-03_rb,4.6664e-03_rb,4.6380e-03_rb,5.2610e-03_rb,8.4068e-03_rb, &
     &1.0505e-02_rb,1.1967e-02_rb,1.0119e-02_rb/)
      kao(:, 2, 2,11) = (/ &
     &5.3180e-03_rb,5.4523e-03_rb,5.6480e-03_rb,5.7960e-03_rb,8.0504e-03_rb,1.1015e-02_rb, &
     &1.3308e-02_rb,1.6901e-02_rb,1.5834e-02_rb/)
      kao(:, 3, 2,11) = (/ &
     &4.1509e-03_rb,5.0420e-03_rb,5.9494e-03_rb,7.8778e-03_rb,1.1077e-02_rb,1.4709e-02_rb, &
     &1.7403e-02_rb,2.2592e-02_rb,2.1710e-02_rb/)
      kao(:, 4, 2,11) = (/ &
     &3.7251e-03_rb,5.8604e-03_rb,7.1881e-03_rb,1.0249e-02_rb,1.4831e-02_rb,1.9015e-02_rb, &
     &2.1299e-02_rb,2.9770e-02_rb,2.9198e-02_rb/)
      kao(:, 5, 2,11) = (/ &
     &3.3170e-03_rb,7.3759e-03_rb,9.2312e-03_rb,1.4285e-02_rb,1.9035e-02_rb,2.3096e-02_rb, &
     &2.6856e-02_rb,3.8325e-02_rb,3.7190e-02_rb/)
      kao(:, 1, 3,11) = (/ &
     &2.3930e-02_rb,2.1175e-02_rb,1.8068e-02_rb,1.3752e-02_rb,9.3232e-03_rb,6.3033e-03_rb, &
     &9.0890e-03_rb,1.0309e-02_rb,9.2920e-03_rb/)
      kao(:, 2, 3,11) = (/ &
     &2.1364e-02_rb,1.8346e-02_rb,1.4408e-02_rb,1.0567e-02_rb,7.3487e-03_rb,8.3255e-03_rb, &
     &1.2320e-02_rb,1.3559e-02_rb,1.1942e-02_rb/)
      kao(:, 3, 3,11) = (/ &
     &1.6897e-02_rb,1.4069e-02_rb,1.1090e-02_rb,9.4072e-03_rb,9.0158e-03_rb,1.2182e-02_rb, &
     &1.6529e-02_rb,1.8032e-02_rb,1.6702e-02_rb/)
      kao(:, 4, 3,11) = (/ &
     &1.1735e-02_rb,1.1062e-02_rb,1.1919e-02_rb,1.0177e-02_rb,1.1885e-02_rb,1.6740e-02_rb, &
     &2.0422e-02_rb,2.2761e-02_rb,2.2381e-02_rb/)
      kao(:, 5, 3,11) = (/ &
     &9.3268e-03_rb,1.0886e-02_rb,1.2245e-02_rb,1.2355e-02_rb,1.5695e-02_rb,2.1667e-02_rb, &
     &2.4460e-02_rb,2.7729e-02_rb,3.0274e-02_rb/)
      kao(:, 1, 4,11) = (/ &
     &5.8030e-02_rb,5.0790e-02_rb,4.4332e-02_rb,3.7503e-02_rb,3.1337e-02_rb,2.3862e-02_rb, &
     &1.5919e-02_rb,8.9924e-03_rb,1.0517e-02_rb/)
      kao(:, 2, 4,11) = (/ &
     &5.8203e-02_rb,5.1238e-02_rb,4.3852e-02_rb,3.6687e-02_rb,2.8842e-02_rb,2.1050e-02_rb, &
     &1.3525e-02_rb,1.1884e-02_rb,1.5506e-02_rb/)
      kao(:, 3, 4,11) = (/ &
     &6.0219e-02_rb,5.3113e-02_rb,4.5845e-02_rb,3.6288e-02_rb,2.6852e-02_rb,1.6648e-02_rb, &
     &1.3537e-02_rb,1.6277e-02_rb,1.8129e-02_rb/)
      kao(:, 4, 4,11) = (/ &
     &4.9871e-02_rb,4.2283e-02_rb,3.5849e-02_rb,2.8922e-02_rb,1.9343e-02_rb,1.4341e-02_rb, &
     &1.8188e-02_rb,2.0990e-02_rb,2.0822e-02_rb/)
      kao(:, 5, 4,11) = (/ &
     &3.8385e-02_rb,3.4058e-02_rb,2.7365e-02_rb,2.1994e-02_rb,1.7140e-02_rb,1.6164e-02_rb, &
     &2.3472e-02_rb,2.6305e-02_rb,2.6373e-02_rb/)
      kao(:, 1, 5,11) = (/ &
     &1.3556e-01_rb,1.1844e-01_rb,1.0158e-01_rb,8.5252e-02_rb,6.8440e-02_rb,5.1875e-02_rb, &
     &3.5448e-02_rb,1.8978e-02_rb,8.8762e-03_rb/)
      kao(:, 2, 5,11) = (/ &
     &1.3807e-01_rb,1.2082e-01_rb,1.0421e-01_rb,8.7432e-02_rb,7.0858e-02_rb,5.4903e-02_rb, &
     &3.7824e-02_rb,2.0068e-02_rb,1.1692e-02_rb/)
      kao(:, 3, 5,11) = (/ &
     &1.3721e-01_rb,1.2013e-01_rb,1.0360e-01_rb,8.7259e-02_rb,7.0575e-02_rb,5.3560e-02_rb, &
     &3.5670e-02_rb,1.8870e-02_rb,1.8216e-02_rb/)
      kao(:, 4, 5,11) = (/ &
     &1.2962e-01_rb,1.1333e-01_rb,9.6679e-02_rb,8.1723e-02_rb,6.6334e-02_rb,5.1939e-02_rb, &
     &3.3737e-02_rb,2.0192e-02_rb,2.7154e-02_rb/)
      kao(:, 5, 5,11) = (/ &
     &1.2340e-01_rb,1.0852e-01_rb,9.2540e-02_rb,7.8313e-02_rb,6.2622e-02_rb,4.1413e-02_rb, &
     &2.5923e-02_rb,2.4326e-02_rb,3.4052e-02_rb/)
      kao(:, 1, 6,11) = (/ &
     &2.8825e-01_rb,2.5222e-01_rb,2.1620e-01_rb,1.8017e-01_rb,1.4417e-01_rb,1.0893e-01_rb, &
     &7.3359e-02_rb,3.7509e-02_rb,5.6670e-03_rb/)
      kao(:, 2, 6,11) = (/ &
     &2.8768e-01_rb,2.5188e-01_rb,2.1600e-01_rb,1.7993e-01_rb,1.4497e-01_rb,1.0976e-01_rb, &
     &7.3838e-02_rb,3.8599e-02_rb,8.8428e-03_rb/)
      kao(:, 3, 6,11) = (/ &
     &2.8241e-01_rb,2.4722e-01_rb,2.1181e-01_rb,1.7724e-01_rb,1.4248e-01_rb,1.0766e-01_rb, &
     &7.3624e-02_rb,3.9950e-02_rb,1.3973e-02_rb/)
      kao(:, 4, 6,11) = (/ &
     &2.8627e-01_rb,2.5067e-01_rb,2.1487e-01_rb,1.8056e-01_rb,1.4619e-01_rb,1.1172e-01_rb, &
     &7.8306e-02_rb,4.0986e-02_rb,2.0403e-02_rb/)
      kao(:, 5, 6,11) = (/ &
     &2.7874e-01_rb,2.4399e-01_rb,2.0989e-01_rb,1.7645e-01_rb,1.4231e-01_rb,1.0969e-01_rb, &
     &7.6704e-02_rb,4.0995e-02_rb,2.4450e-02_rb/)
      kao(:, 1, 7,11) = (/ &
     &6.1169e-01_rb,5.3575e-01_rb,4.5923e-01_rb,3.8252e-01_rb,3.0616e-01_rb,2.2929e-01_rb, &
     &1.5383e-01_rb,7.7531e-02_rb,2.8956e-03_rb/)
      kao(:, 2, 7,11) = (/ &
     &6.1159e-01_rb,5.3511e-01_rb,4.5848e-01_rb,3.8242e-01_rb,3.0567e-01_rb,2.3004e-01_rb, &
     &1.5345e-01_rb,7.7731e-02_rb,6.2629e-03_rb/)
      kao(:, 3, 7,11) = (/ &
     &6.0642e-01_rb,5.3036e-01_rb,4.5410e-01_rb,3.7880e-01_rb,3.0308e-01_rb,2.2893e-01_rb, &
     &1.5450e-01_rb,7.9546e-02_rb,1.1879e-02_rb/)
      kao(:, 4, 7,11) = (/ &
     &6.2322e-01_rb,5.4531e-01_rb,4.6718e-01_rb,3.8974e-01_rb,3.1356e-01_rb,2.3563e-01_rb, &
     &1.5873e-01_rb,8.1521e-02_rb,1.3030e-02_rb/)
      kao(:, 5, 7,11) = (/ &
     &6.1773e-01_rb,5.4019e-01_rb,4.6283e-01_rb,3.8859e-01_rb,3.1161e-01_rb,2.3601e-01_rb, &
     &1.5998e-01_rb,8.3125e-02_rb,1.9458e-02_rb/)
      kao(:, 1, 8,11) = (/ &
     &1.4932e+00_rb,1.3065e+00_rb,1.1199e+00_rb,9.3315e-01_rb,7.4665e-01_rb,5.5991e-01_rb, &
     &3.7327e-01_rb,1.8697e-01_rb,1.1999e-05_rb/)
      kao(:, 2, 8,11) = (/ &
     &1.5196e+00_rb,1.3312e+00_rb,1.1397e+00_rb,9.5010e-01_rb,7.6026e-01_rb,5.7020e-01_rb, &
     &3.8013e-01_rb,1.9174e-01_rb,7.7042e-03_rb/)
      kao(:, 3, 8,11) = (/ &
     &1.5505e+00_rb,1.3567e+00_rb,1.1629e+00_rb,9.6902e-01_rb,7.7525e-01_rb,5.8146e-01_rb, &
     &3.8981e-01_rb,1.9691e-01_rb,1.2006e-02_rb/)
      kao(:, 4, 8,11) = (/ &
     &1.4878e+00_rb,1.3018e+00_rb,1.1160e+00_rb,9.2984e-01_rb,7.4341e-01_rb,5.5771e-01_rb, &
     &3.7622e-01_rb,1.9146e-01_rb,1.2696e-02_rb/)
      kao(:, 5, 8,11) = (/ &
     &1.4729e+00_rb,1.2883e+00_rb,1.1049e+00_rb,9.2129e-01_rb,7.3618e-01_rb,5.5599e-01_rb, &
     &3.7163e-01_rb,1.8805e-01_rb,1.8722e-02_rb/)
      kao(:, 1, 9,11) = (/ &
     &6.8904e+00_rb,6.0266e+00_rb,5.1680e+00_rb,4.3068e+00_rb,3.4452e+00_rb,2.5839e+00_rb, &
     &1.7234e+00_rb,8.6092e-01_rb,6.7224e-06_rb/)
      kao(:, 2, 9,11) = (/ &
     &7.0980e+00_rb,6.2107e+00_rb,5.3294e+00_rb,4.4415e+00_rb,3.5510e+00_rb,2.6633e+00_rb, &
     &1.7755e+00_rb,8.8723e-01_rb,1.2025e-05_rb/)
      kao(:, 3, 9,11) = (/ &
     &7.2688e+00_rb,6.3564e+00_rb,5.4482e+00_rb,4.5401e+00_rb,3.6343e+00_rb,2.7243e+00_rb, &
     &1.8161e+00_rb,9.0754e-01_rb,1.9985e-05_rb/)
      kao(:, 4, 9,11) = (/ &
     &7.1740e+00_rb,6.2782e+00_rb,5.3807e+00_rb,4.4818e+00_rb,3.5855e+00_rb,2.6901e+00_rb, &
     &1.7938e+00_rb,8.9646e-01_rb,3.0947e-05_rb/)
      kao(:, 5, 9,11) = (/ &
     &6.7468e+00_rb,5.9097e+00_rb,5.0601e+00_rb,4.2188e+00_rb,3.3754e+00_rb,2.5313e+00_rb, &
     &1.6873e+00_rb,8.4597e-01_rb,4.6476e-05_rb/)
      kao(:, 1,10,11) = (/ &
     &3.5457e+01_rb,3.1027e+01_rb,2.6582e+01_rb,2.2175e+01_rb,1.7740e+01_rb,1.3297e+01_rb, &
     &8.8648e+00_rb,4.4349e+00_rb,3.8551e-06_rb/)
      kao(:, 2,10,11) = (/ &
     &3.5857e+01_rb,3.1366e+01_rb,2.6891e+01_rb,2.2421e+01_rb,1.7929e+01_rb,1.3446e+01_rb, &
     &8.9645e+00_rb,4.4822e+00_rb,7.1785e-06_rb/)
      kao(:, 3,10,11) = (/ &
     &3.7065e+01_rb,3.2447e+01_rb,2.7799e+01_rb,2.3166e+01_rb,1.8532e+01_rb,1.3906e+01_rb, &
     &9.2664e+00_rb,4.6331e+00_rb,1.2630e-05_rb/)
      kao(:, 4,10,11) = (/ &
     &3.6718e+01_rb,3.2113e+01_rb,2.7539e+01_rb,2.2938e+01_rb,1.8366e+01_rb,1.3764e+01_rb, &
     &9.1841e+00_rb,4.5916e+00_rb,2.1179e-05_rb/)
      kao(:, 5,10,11) = (/ &
     &3.5010e+01_rb,3.0613e+01_rb,2.6257e+01_rb,2.1870e+01_rb,1.7497e+01_rb,1.3129e+01_rb, &
     &8.7481e+00_rb,4.3808e+00_rb,3.3504e-05_rb/)
      kao(:, 1,11,11) = (/ &
     &6.2218e+01_rb,5.4427e+01_rb,4.6646e+01_rb,3.8874e+01_rb,3.1110e+01_rb,2.3334e+01_rb, &
     &1.5561e+01_rb,7.7776e+00_rb,3.5875e-06_rb/)
      kao(:, 2,11,11) = (/ &
     &6.2105e+01_rb,5.4407e+01_rb,4.6579e+01_rb,3.8839e+01_rb,3.1033e+01_rb,2.3318e+01_rb, &
     &1.5536e+01_rb,7.7679e+00_rb,5.9066e-06_rb/)
      kao(:, 3,11,11) = (/ &
     &6.4816e+01_rb,5.6715e+01_rb,4.8615e+01_rb,4.0513e+01_rb,3.2442e+01_rb,2.4317e+01_rb, &
     &1.6205e+01_rb,8.1056e+00_rb,1.0776e-05_rb/)
      kao(:, 4,11,11) = (/ &
     &6.2787e+01_rb,5.4987e+01_rb,4.7112e+01_rb,3.9241e+01_rb,3.1378e+01_rb,2.3555e+01_rb, &
     &1.5697e+01_rb,7.8485e+00_rb,1.7304e-05_rb/)
      kao(:, 5,11,11) = (/ &
     &6.1876e+01_rb,5.4130e+01_rb,4.6407e+01_rb,3.8627e+01_rb,3.0933e+01_rb,2.3204e+01_rb, &
     &1.5469e+01_rb,7.7346e+00_rb,2.7512e-05_rb/)
      kao(:, 1,12,11) = (/ &
     &8.0103e+01_rb,7.0095e+01_rb,6.0103e+01_rb,5.0066e+01_rb,4.0052e+01_rb,3.0040e+01_rb, &
     &2.0026e+01_rb,1.0013e+01_rb,3.2505e-06_rb/)
      kao(:, 2,12,11) = (/ &
     &7.9090e+01_rb,6.9204e+01_rb,5.9318e+01_rb,4.9432e+01_rb,3.9544e+01_rb,2.9659e+01_rb, &
     &1.9773e+01_rb,9.8860e+00_rb,5.5034e-06_rb/)
      kao(:, 3,12,11) = (/ &
     &7.9848e+01_rb,6.9859e+01_rb,5.9882e+01_rb,4.9902e+01_rb,3.9894e+01_rb,2.9946e+01_rb, &
     &1.9960e+01_rb,9.9805e+00_rb,8.6063e-06_rb/)
      kao(:, 4,12,11) = (/ &
     &7.8661e+01_rb,6.8779e+01_rb,5.8977e+01_rb,4.9159e+01_rb,3.9293e+01_rb,2.9497e+01_rb, &
     &1.9663e+01_rb,9.8183e+00_rb,1.4131e-05_rb/)
      kao(:, 5,12,11) = (/ &
     &7.8792e+01_rb,6.8975e+01_rb,5.9124e+01_rb,4.9246e+01_rb,3.9445e+01_rb,2.9569e+01_rb, &
     &1.9707e+01_rb,9.8563e+00_rb,2.3305e-05_rb/)
      kao(:, 1,13,11) = (/ &
     &7.5314e+01_rb,6.5898e+01_rb,5.6507e+01_rb,4.7072e+01_rb,3.7657e+01_rb,2.8254e+01_rb, &
     &1.8828e+01_rb,9.4159e+00_rb,2.7819e-06_rb/)
      kao(:, 2,13,11) = (/ &
     &7.4029e+01_rb,6.4852e+01_rb,5.5523e+01_rb,4.6247e+01_rb,3.7031e+01_rb,2.7774e+01_rb, &
     &1.8528e+01_rb,9.2576e+00_rb,5.0908e-06_rb/)
      kao(:, 3,13,11) = (/ &
     &7.3797e+01_rb,6.4573e+01_rb,5.5347e+01_rb,4.6143e+01_rb,3.6917e+01_rb,2.7689e+01_rb, &
     &1.8458e+01_rb,9.2250e+00_rb,8.2936e-06_rb/)
      kao(:, 4,13,11) = (/ &
     &7.5507e+01_rb,6.6112e+01_rb,5.6628e+01_rb,4.7225e+01_rb,3.7779e+01_rb,2.8348e+01_rb, &
     &1.8877e+01_rb,9.4449e+00_rb,1.2487e-05_rb/)
      kao(:, 5,13,11) = (/ &
     &7.6962e+01_rb,6.7290e+01_rb,5.7729e+01_rb,4.8107e+01_rb,3.8486e+01_rb,2.8863e+01_rb, &
     &1.9221e+01_rb,9.6284e+00_rb,1.8751e-05_rb/)
      kao(:, 1, 1,12) = (/ &
     &1.1046e-03_rb,3.3707e-03_rb,5.0558e-03_rb,8.2647e-03_rb,9.1015e-03_rb,1.0694e-02_rb, &
     &1.8807e-02_rb,2.2647e-02_rb,1.8110e-02_rb/)
      kao(:, 2, 1,12) = (/ &
     &1.0969e-03_rb,4.7708e-03_rb,7.6740e-03_rb,1.0546e-02_rb,1.0900e-02_rb,1.6801e-02_rb, &
     &2.5799e-02_rb,3.1294e-02_rb,2.2312e-02_rb/)
      kao(:, 3, 1,12) = (/ &
     &1.0873e-03_rb,6.1594e-03_rb,1.0535e-02_rb,1.2275e-02_rb,1.5399e-02_rb,2.5780e-02_rb, &
     &3.4278e-02_rb,4.1551e-02_rb,3.1702e-02_rb/)
      kao(:, 4, 1,12) = (/ &
     &1.0681e-03_rb,6.7407e-03_rb,1.3124e-02_rb,1.6579e-02_rb,1.9835e-02_rb,3.5596e-02_rb, &
     &4.4369e-02_rb,5.3472e-02_rb,4.1109e-02_rb/)
      kao(:, 5, 1,12) = (/ &
     &1.0270e-03_rb,8.1418e-03_rb,1.5112e-02_rb,2.0848e-02_rb,2.8658e-02_rb,4.6490e-02_rb, &
     &5.6118e-02_rb,6.7233e-02_rb,5.7448e-02_rb/)
      kao(:, 1, 2,12) = (/ &
     &1.5646e-03_rb,3.6954e-03_rb,5.8279e-03_rb,7.1796e-03_rb,9.8217e-03_rb,9.7524e-03_rb, &
     &1.1081e-02_rb,1.8489e-02_rb,1.8167e-02_rb/)
      kao(:, 2, 2,12) = (/ &
     &1.5547e-03_rb,3.8903e-03_rb,7.0691e-03_rb,1.0316e-02_rb,1.1952e-02_rb,1.3642e-02_rb, &
     &1.6631e-02_rb,2.5775e-02_rb,2.2105e-02_rb/)
      kao(:, 3, 2,12) = (/ &
     &1.5005e-03_rb,5.8120e-03_rb,1.0164e-02_rb,1.2828e-02_rb,1.5616e-02_rb,1.8084e-02_rb, &
     &2.4815e-02_rb,3.4836e-02_rb,2.8922e-02_rb/)
      kao(:, 4, 2,12) = (/ &
     &1.4224e-03_rb,7.6958e-03_rb,1.3586e-02_rb,1.5505e-02_rb,1.9916e-02_rb,2.3661e-02_rb, &
     &3.6496e-02_rb,4.5732e-02_rb,3.8152e-02_rb/)
      kao(:, 5, 2,12) = (/ &
     &1.3736e-03_rb,9.1086e-03_rb,1.6016e-02_rb,2.0127e-02_rb,2.5470e-02_rb,3.1020e-02_rb, &
     &5.0141e-02_rb,5.8654e-02_rb,5.1074e-02_rb/)
      kao(:, 1, 3,12) = (/ &
     &1.0482e-02_rb,8.9646e-03_rb,6.8350e-03_rb,8.2815e-03_rb,8.8992e-03_rb,1.0605e-02_rb, &
     &1.0328e-02_rb,1.1583e-02_rb,1.6044e-02_rb/)
      kao(:, 2, 3,12) = (/ &
     &5.7078e-03_rb,6.1160e-03_rb,7.6830e-03_rb,9.7377e-03_rb,1.2964e-02_rb,1.2587e-02_rb, &
     &1.4124e-02_rb,1.6978e-02_rb,2.4120e-02_rb/)
      kao(:, 3, 3,12) = (/ &
     &3.9502e-03_rb,6.0459e-03_rb,1.0569e-02_rb,1.4120e-02_rb,1.6811e-02_rb,1.6440e-02_rb, &
     &1.9141e-02_rb,2.2713e-02_rb,3.1566e-02_rb/)
      kao(:, 4, 3,12) = (/ &
     &3.2701e-03_rb,7.5815e-03_rb,1.2592e-02_rb,1.9220e-02_rb,1.9128e-02_rb,2.1966e-02_rb, &
     &2.6084e-02_rb,3.3096e-02_rb,3.6118e-02_rb/)
      kao(:, 5, 3,12) = (/ &
     &2.7401e-03_rb,7.8080e-03_rb,1.5492e-02_rb,2.3290e-02_rb,2.4892e-02_rb,2.8848e-02_rb, &
     &3.4321e-02_rb,4.6873e-02_rb,4.7340e-02_rb/)
      kao(:, 1, 4,12) = (/ &
     &6.0672e-02_rb,5.3090e-02_rb,4.4556e-02_rb,3.6202e-02_rb,2.6483e-02_rb,1.8181e-02_rb, &
     &1.2199e-02_rb,1.0993e-02_rb,1.4969e-02_rb/)
      kao(:, 2, 4,12) = (/ &
     &4.3589e-02_rb,3.8144e-02_rb,3.0755e-02_rb,2.3593e-02_rb,1.9730e-02_rb,1.5624e-02_rb, &
     &1.3490e-02_rb,1.5542e-02_rb,2.0057e-02_rb/)
      kao(:, 3, 4,12) = (/ &
     &2.4695e-02_rb,2.2865e-02_rb,1.7074e-02_rb,1.5816e-02_rb,1.6661e-02_rb,2.0463e-02_rb, &
     &1.7348e-02_rb,2.1210e-02_rb,2.8829e-02_rb/)
      kao(:, 4, 4,12) = (/ &
     &1.4306e-02_rb,1.8283e-02_rb,1.4453e-02_rb,1.9081e-02_rb,2.2731e-02_rb,2.4085e-02_rb, &
     &2.3223e-02_rb,2.7809e-02_rb,4.1362e-02_rb/)
      kao(:, 5, 4,12) = (/ &
     &1.0713e-02_rb,1.1979e-02_rb,1.4158e-02_rb,2.2119e-02_rb,2.9428e-02_rb,2.9426e-02_rb, &
     &3.0795e-02_rb,3.5958e-02_rb,5.4444e-02_rb/)
      kao(:, 1, 5,12) = (/ &
     &1.5652e-01_rb,1.3723e-01_rb,1.1755e-01_rb,9.8247e-02_rb,7.8467e-02_rb,5.9349e-02_rb, &
     &3.9820e-02_rb,1.8398e-02_rb,1.1394e-02_rb/)
      kao(:, 2, 5,12) = (/ &
     &1.5350e-01_rb,1.3431e-01_rb,1.1514e-01_rb,9.6381e-02_rb,7.7667e-02_rb,5.6368e-02_rb, &
     &3.3743e-02_rb,1.4780e-02_rb,1.5913e-02_rb/)
      kao(:, 3, 5,12) = (/ &
     &1.3349e-01_rb,1.1672e-01_rb,9.9565e-02_rb,8.1852e-02_rb,6.1169e-02_rb,4.3464e-02_rb, &
     &2.9522e-02_rb,1.7569e-02_rb,2.3571e-02_rb/)
      kao(:, 4, 5,12) = (/ &
     &1.0423e-01_rb,9.1271e-02_rb,7.7253e-02_rb,6.0811e-02_rb,4.8046e-02_rb,3.4464e-02_rb, &
     &2.5349e-02_rb,2.3626e-02_rb,3.4271e-02_rb/)
      kao(:, 5, 5,12) = (/ &
     &6.1168e-02_rb,5.3666e-02_rb,4.5385e-02_rb,3.6131e-02_rb,3.2187e-02_rb,3.6873e-02_rb, &
     &3.0883e-02_rb,3.2443e-02_rb,4.3030e-02_rb/)
      kao(:, 1, 6,12) = (/ &
     &3.3296e-01_rb,2.9114e-01_rb,2.4975e-01_rb,2.0812e-01_rb,1.6650e-01_rb,1.2577e-01_rb, &
     &8.4465e-02_rb,4.3210e-02_rb,7.5744e-03_rb/)
      kao(:, 2, 6,12) = (/ &
     &3.2661e-01_rb,2.8604e-01_rb,2.4502e-01_rb,2.0429e-01_rb,1.6474e-01_rb,1.2400e-01_rb, &
     &8.4046e-02_rb,4.3324e-02_rb,1.2737e-02_rb/)
      kao(:, 3, 6,12) = (/ &
     &3.3274e-01_rb,2.9155e-01_rb,2.4958e-01_rb,2.1022e-01_rb,1.6841e-01_rb,1.2817e-01_rb, &
     &8.5832e-02_rb,4.1156e-02_rb,1.9272e-02_rb/)
      kao(:, 4, 6,12) = (/ &
     &3.2428e-01_rb,2.8378e-01_rb,2.4307e-01_rb,2.0420e-01_rb,1.6451e-01_rb,1.2138e-01_rb, &
     &7.6212e-02_rb,3.3602e-02_rb,2.1893e-02_rb/)
      kao(:, 5, 6,12) = (/ &
     &2.8830e-01_rb,2.5208e-01_rb,2.1788e-01_rb,1.7562e-01_rb,1.3634e-01_rb,9.6797e-02_rb, &
     &6.1659e-02_rb,2.8996e-02_rb,3.3943e-02_rb/)
      kao(:, 1, 7,12) = (/ &
     &7.1633e-01_rb,6.2687e-01_rb,5.3652e-01_rb,4.4712e-01_rb,3.5797e-01_rb,2.6845e-01_rb, &
     &1.7938e-01_rb,9.0924e-02_rb,9.7506e-03_rb/)
      kao(:, 2, 7,12) = (/ &
     &7.4602e-01_rb,6.5233e-01_rb,5.5949e-01_rb,4.6595e-01_rb,3.7278e-01_rb,2.7940e-01_rb, &
     &1.8898e-01_rb,9.6146e-02_rb,1.9089e-02_rb/)
      kao(:, 3, 7,12) = (/ &
     &7.5756e-01_rb,6.6287e-01_rb,5.6888e-01_rb,4.7384e-01_rb,3.7881e-01_rb,2.8626e-01_rb, &
     &1.9222e-01_rb,9.8080e-02_rb,1.5850e-02_rb/)
      kao(:, 4, 7,12) = (/ &
     &7.2210e-01_rb,6.3187e-01_rb,5.4195e-01_rb,4.5134e-01_rb,3.6428e-01_rb,2.7503e-01_rb, &
     &1.8532e-01_rb,9.5998e-02_rb,2.2744e-02_rb/)
      kao(:, 5, 7,12) = (/ &
     &7.1833e-01_rb,6.2908e-01_rb,5.3886e-01_rb,4.4907e-01_rb,3.6079e-01_rb,2.7147e-01_rb, &
     &1.8148e-01_rb,9.1961e-02_rb,3.2788e-02_rb/)
      kao(:, 1, 8,12) = (/ &
     &1.8651e+00_rb,1.6319e+00_rb,1.3988e+00_rb,1.1656e+00_rb,9.3248e-01_rb,6.9936e-01_rb, &
     &4.6627e-01_rb,2.3315e-01_rb,7.1174e-06_rb/)
      kao(:, 2, 8,12) = (/ &
     &1.9142e+00_rb,1.6730e+00_rb,1.4347e+00_rb,1.1964e+00_rb,9.5588e-01_rb,7.1697e-01_rb, &
     &4.7797e-01_rb,2.4089e-01_rb,1.3089e-05_rb/)
      kao(:, 3, 8,12) = (/ &
     &1.9338e+00_rb,1.6922e+00_rb,1.4504e+00_rb,1.2080e+00_rb,9.6694e-01_rb,7.2522e-01_rb, &
     &4.8350e-01_rb,2.4403e-01_rb,6.4382e-03_rb/)
      kao(:, 4, 8,12) = (/ &
     &1.9257e+00_rb,1.6851e+00_rb,1.4444e+00_rb,1.2027e+00_rb,9.6352e-01_rb,7.2222e-01_rb, &
     &4.8580e-01_rb,2.4642e-01_rb,2.0202e-02_rb/)
      kao(:, 5, 8,12) = (/ &
     &1.8578e+00_rb,1.6228e+00_rb,1.3921e+00_rb,1.1585e+00_rb,9.2745e-01_rb,6.9620e-01_rb, &
     &4.7147e-01_rb,2.4294e-01_rb,3.5578e-02_rb/)
      kao(:, 1, 9,12) = (/ &
     &8.9732e+00_rb,7.8574e+00_rb,6.7348e+00_rb,5.6085e+00_rb,4.4901e+00_rb,3.3675e+00_rb, &
     &2.2449e+00_rb,1.1217e+00_rb,5.5128e-06_rb/)
      kao(:, 2, 9,12) = (/ &
     &9.0176e+00_rb,7.8899e+00_rb,6.7555e+00_rb,5.6293e+00_rb,4.5061e+00_rb,3.3798e+00_rb, &
     &2.2531e+00_rb,1.1272e+00_rb,1.0749e-05_rb/)
      kao(:, 3, 9,12) = (/ &
     &9.1462e+00_rb,8.0080e+00_rb,6.8640e+00_rb,5.7226e+00_rb,4.5733e+00_rb,3.4343e+00_rb, &
     &2.2880e+00_rb,1.1452e+00_rb,1.7030e-05_rb/)
      kao(:, 4, 9,12) = (/ &
     &9.3997e+00_rb,8.2308e+00_rb,7.0500e+00_rb,5.8749e+00_rb,4.7035e+00_rb,3.5295e+00_rb, &
     &2.3500e+00_rb,1.1750e+00_rb,2.7289e-05_rb/)
      kao(:, 5, 9,12) = (/ &
     &9.4496e+00_rb,8.2622e+00_rb,7.0818e+00_rb,5.9018e+00_rb,4.7215e+00_rb,3.5410e+00_rb, &
     &2.3622e+00_rb,1.1804e+00_rb,4.0208e-05_rb/)
      kao(:, 1,10,12) = (/ &
     &4.7340e+01_rb,4.1390e+01_rb,3.5503e+01_rb,2.9570e+01_rb,2.3638e+01_rb,1.7753e+01_rb, &
     &1.1835e+01_rb,5.9140e+00_rb,4.0857e-06_rb/)
      kao(:, 2,10,12) = (/ &
     &4.7468e+01_rb,4.1532e+01_rb,3.5600e+01_rb,2.9666e+01_rb,2.3733e+01_rb,1.7801e+01_rb, &
     &1.1867e+01_rb,5.9335e+00_rb,7.3318e-06_rb/)
      kao(:, 3,10,12) = (/ &
     &4.7779e+01_rb,4.1808e+01_rb,3.5836e+01_rb,2.9862e+01_rb,2.3890e+01_rb,1.7931e+01_rb, &
     &1.1954e+01_rb,5.9724e+00_rb,1.2287e-05_rb/)
      kao(:, 4,10,12) = (/ &
     &5.0181e+01_rb,4.3911e+01_rb,3.7609e+01_rb,3.1364e+01_rb,2.5077e+01_rb,1.8819e+01_rb, &
     &1.2546e+01_rb,6.2708e+00_rb,2.0136e-05_rb/)
      kao(:, 5,10,12) = (/ &
     &5.0358e+01_rb,4.4054e+01_rb,3.7768e+01_rb,3.1474e+01_rb,2.5199e+01_rb,1.8885e+01_rb, &
     &1.2590e+01_rb,6.2949e+00_rb,3.0786e-05_rb/)
      kao(:, 1,11,12) = (/ &
     &8.8012e+01_rb,7.7091e+01_rb,6.6061e+01_rb,5.5064e+01_rb,4.4041e+01_rb,3.3007e+01_rb, &
     &2.2019e+01_rb,1.1017e+01_rb,3.6494e-06_rb/)
      kao(:, 2,11,12) = (/ &
     &8.9290e+01_rb,7.8045e+01_rb,6.6917e+01_rb,5.5778e+01_rb,4.4669e+01_rb,3.3448e+01_rb, &
     &2.2295e+01_rb,1.1156e+01_rb,6.7273e-06_rb/)
      kao(:, 3,11,12) = (/ &
     &8.6982e+01_rb,7.6115e+01_rb,6.5187e+01_rb,5.4368e+01_rb,4.3469e+01_rb,3.2619e+01_rb, &
     &2.1748e+01_rb,1.0873e+01_rb,1.1236e-05_rb/)
      kao(:, 4,11,12) = (/ &
     &8.9808e+01_rb,7.8580e+01_rb,6.7351e+01_rb,5.6127e+01_rb,4.4937e+01_rb,3.3677e+01_rb, &
     &2.2469e+01_rb,1.1226e+01_rb,1.9240e-05_rb/)
      kao(:, 5,11,12) = (/ &
     &9.0868e+01_rb,7.9552e+01_rb,6.8146e+01_rb,5.6871e+01_rb,4.5498e+01_rb,3.4074e+01_rb, &
     &2.2716e+01_rb,1.1367e+01_rb,2.9192e-05_rb/)
      kao(:, 1,12,12) = (/ &
     &1.1586e+02_rb,1.0129e+02_rb,8.6824e+01_rb,7.2354e+01_rb,5.7931e+01_rb,4.3416e+01_rb, &
     &2.8943e+01_rb,1.4471e+01_rb,2.9778e-06_rb/)
      kao(:, 2,12,12) = (/ &
     &1.1852e+02_rb,1.0370e+02_rb,8.8887e+01_rb,7.4073e+01_rb,5.9259e+01_rb,4.4444e+01_rb, &
     &2.9629e+01_rb,1.4815e+01_rb,6.0415e-06_rb/)
      kao(:, 3,12,12) = (/ &
     &1.1782e+02_rb,1.0310e+02_rb,8.8376e+01_rb,7.3644e+01_rb,5.8951e+01_rb,4.4160e+01_rb, &
     &2.9459e+01_rb,1.4728e+01_rb,1.0435e-05_rb/)
      kao(:, 4,12,12) = (/ &
     &1.1710e+02_rb,1.0244e+02_rb,8.7885e+01_rb,7.3194e+01_rb,5.8621e+01_rb,4.3876e+01_rb, &
     &2.9251e+01_rb,1.4654e+01_rb,1.8188e-05_rb/)
      kao(:, 5,12,12) = (/ &
     &1.1577e+02_rb,1.0131e+02_rb,8.6825e+01_rb,7.2359e+01_rb,5.7901e+01_rb,4.3424e+01_rb, &
     &2.8943e+01_rb,1.4462e+01_rb,2.8178e-05_rb/)
      kao(:, 1,13,12) = (/ &
     &1.1625e+02_rb,1.0180e+02_rb,8.7251e+01_rb,7.2708e+01_rb,5.8118e+01_rb,4.3593e+01_rb, &
     &2.9061e+01_rb,1.4533e+01_rb,2.6097e-06_rb/)
      kao(:, 2,13,12) = (/ &
     &1.1986e+02_rb,1.0483e+02_rb,8.9927e+01_rb,7.4912e+01_rb,5.9931e+01_rb,4.4948e+01_rb, &
     &2.9950e+01_rb,1.4982e+01_rb,4.5705e-06_rb/)
      kao(:, 3,13,12) = (/ &
     &1.1859e+02_rb,1.0377e+02_rb,8.8940e+01_rb,7.4121e+01_rb,5.9293e+01_rb,4.4471e+01_rb, &
     &2.9646e+01_rb,1.4823e+01_rb,9.6333e-06_rb/)
      kao(:, 4,13,12) = (/ &
     &1.1553e+02_rb,1.0103e+02_rb,8.6643e+01_rb,7.2158e+01_rb,5.7729e+01_rb,4.3335e+01_rb, &
     &2.8881e+01_rb,1.4445e+01_rb,1.6615e-05_rb/)
      kao(:, 5,13,12) = (/ &
     &1.1521e+02_rb,1.0087e+02_rb,8.6350e+01_rb,7.1959e+01_rb,5.7620e+01_rb,4.3214e+01_rb, &
     &2.8812e+01_rb,1.4383e+01_rb,2.5689e-05_rb/)
      kao(:, 1, 1,13) = (/ &
     &1.2352e-03_rb,5.0775e-03_rb,9.0568e-03_rb,9.6608e-03_rb,1.7051e-02_rb,2.9171e-02_rb, &
     &3.4997e-02_rb,4.0847e-02_rb,3.4367e-02_rb/)
      kao(:, 2, 1,13) = (/ &
     &1.2435e-03_rb,6.8803e-03_rb,1.1829e-02_rb,1.3668e-02_rb,2.8015e-02_rb,4.0318e-02_rb, &
     &4.8254e-02_rb,5.5929e-02_rb,5.6166e-02_rb/)
      kao(:, 3, 1,13) = (/ &
     &1.2378e-03_rb,7.6901e-03_rb,1.3370e-02_rb,2.0872e-02_rb,4.0011e-02_rb,5.3853e-02_rb, &
     &6.4341e-02_rb,7.4163e-02_rb,8.0135e-02_rb/)
      kao(:, 4, 1,13) = (/ &
     &1.2280e-03_rb,1.0734e-02_rb,1.6343e-02_rb,3.0629e-02_rb,5.6049e-02_rb,6.9850e-02_rb, &
     &8.3340e-02_rb,9.5619e-02_rb,1.1209e-01_rb/)
      kao(:, 5, 1,13) = (/ &
     &1.2088e-03_rb,1.3761e-02_rb,2.1254e-02_rb,4.2402e-02_rb,7.0900e-02_rb,8.8313e-02_rb, &
     &1.0523e-01_rb,1.2032e-01_rb,1.4179e-01_rb/)
      kao(:, 1, 2,13) = (/ &
     &1.7062e-03_rb,5.0744e-03_rb,8.6767e-03_rb,1.3080e-02_rb,1.1607e-02_rb,1.7056e-02_rb, &
     &2.8742e-02_rb,3.5357e-02_rb,2.3747e-02_rb/)
      kao(:, 2, 2,13) = (/ &
     &1.7222e-03_rb,6.7515e-03_rb,1.2374e-02_rb,1.5331e-02_rb,1.8393e-02_rb,2.7370e-02_rb, &
     &4.2648e-02_rb,4.9678e-02_rb,3.7912e-02_rb/)
      kao(:, 3, 2,13) = (/ &
     &1.7299e-03_rb,8.6001e-03_rb,1.6797e-02_rb,2.0526e-02_rb,2.6294e-02_rb,4.1034e-02_rb, &
     &5.7948e-02_rb,6.7295e-02_rb,5.4721e-02_rb/)
      kao(:, 4, 2,13) = (/ &
     &1.7668e-03_rb,1.0879e-02_rb,2.0064e-02_rb,2.7385e-02_rb,3.4476e-02_rb,5.8386e-02_rb, &
     &7.6246e-02_rb,8.8358e-02_rb,7.0573e-02_rb/)
      kao(:, 5, 2,13) = (/ &
     &1.7596e-03_rb,1.1860e-02_rb,2.3664e-02_rb,3.3939e-02_rb,4.8492e-02_rb,7.9884e-02_rb, &
     &9.7519e-02_rb,1.1290e-01_rb,9.7083e-02_rb/)
      kao(:, 1, 3,13) = (/ &
     &3.1651e-03_rb,5.2961e-03_rb,8.5469e-03_rb,1.1513e-02_rb,1.5558e-02_rb,1.4881e-02_rb, &
     &1.6424e-02_rb,2.5024e-02_rb,2.8147e-02_rb/)
      kao(:, 2, 3,13) = (/ &
     &3.3189e-03_rb,8.3809e-03_rb,1.1489e-02_rb,1.6299e-02_rb,2.2221e-02_rb,2.2841e-02_rb, &
     &2.5024e-02_rb,3.8200e-02_rb,3.8233e-02_rb/)
      kao(:, 3, 3,13) = (/ &
     &3.3701e-03_rb,1.1923e-02_rb,1.5036e-02_rb,2.2373e-02_rb,2.6383e-02_rb,3.1900e-02_rb, &
     &3.2997e-02_rb,5.6522e-02_rb,4.6578e-02_rb/)
      kao(:, 4, 3,13) = (/ &
     &3.3871e-03_rb,1.2781e-02_rb,1.7799e-02_rb,2.9758e-02_rb,3.5615e-02_rb,4.1357e-02_rb, &
     &4.8578e-02_rb,7.5987e-02_rb,6.6261e-02_rb/)
      kao(:, 5, 3,13) = (/ &
     &3.3797e-03_rb,1.4220e-02_rb,2.3964e-02_rb,3.5644e-02_rb,4.5626e-02_rb,4.9180e-02_rb, &
     &6.9386e-02_rb,9.9109e-02_rb,8.8310e-02_rb/)
      kao(:, 1, 4,13) = (/ &
     &1.0343e-02_rb,9.0620e-03_rb,1.2015e-02_rb,1.1324e-02_rb,1.3382e-02_rb,1.5654e-02_rb, &
     &1.8625e-02_rb,1.7070e-02_rb,2.1331e-02_rb/)
      kao(:, 2, 4,13) = (/ &
     &6.4226e-03_rb,5.6266e-03_rb,1.6045e-02_rb,1.6336e-02_rb,1.7909e-02_rb,2.2857e-02_rb, &
     &2.5406e-02_rb,2.4740e-02_rb,3.1419e-02_rb/)
      kao(:, 3, 4,13) = (/ &
     &6.5549e-03_rb,5.7385e-03_rb,1.8741e-02_rb,2.0901e-02_rb,2.5444e-02_rb,3.1318e-02_rb, &
     &3.5488e-02_rb,3.2476e-02_rb,4.5848e-02_rb/)
      kao(:, 4, 4,13) = (/ &
     &6.6407e-03_rb,5.8164e-03_rb,2.0962e-02_rb,2.6844e-02_rb,3.4892e-02_rb,4.1403e-02_rb, &
     &4.7653e-02_rb,4.2426e-02_rb,6.5116e-02_rb/)
      kao(:, 5, 4,13) = (/ &
     &6.8300e-03_rb,1.2571e-02_rb,2.5118e-02_rb,3.2793e-02_rb,4.7122e-02_rb,5.5638e-02_rb, &
     &5.7712e-02_rb,6.2270e-02_rb,8.9700e-02_rb/)
      kao(:, 1, 5,13) = (/ &
     &9.0149e-02_rb,7.8756e-02_rb,6.7503e-02_rb,5.7299e-02_rb,4.6607e-02_rb,3.2042e-02_rb, &
     &2.0392e-02_rb,1.6128e-02_rb,1.9894e-02_rb/)
      kao(:, 2, 5,13) = (/ &
     &4.8191e-02_rb,4.2297e-02_rb,3.6135e-02_rb,3.5890e-02_rb,2.5466e-02_rb,2.2998e-02_rb, &
     &2.3063e-02_rb,2.5066e-02_rb,2.8493e-02_rb/)
      kao(:, 3, 5,13) = (/ &
     &2.0878e-02_rb,1.8444e-02_rb,2.0162e-02_rb,2.6175e-02_rb,2.5330e-02_rb,2.8034e-02_rb, &
     &3.3888e-02_rb,3.5693e-02_rb,3.9583e-02_rb/)
      kao(:, 4, 5,13) = (/ &
     &1.2467e-02_rb,1.0910e-02_rb,2.2261e-02_rb,3.0489e-02_rb,3.2389e-02_rb,3.6918e-02_rb, &
     &4.6126e-02_rb,4.7452e-02_rb,5.4222e-02_rb/)
      kao(:, 5, 5,13) = (/ &
     &1.2730e-02_rb,1.1204e-02_rb,2.9691e-02_rb,3.6357e-02_rb,4.1593e-02_rb,4.9722e-02_rb, &
     &6.1593e-02_rb,5.8372e-02_rb,7.2216e-02_rb/)
      kao(:, 1, 6,13) = (/ &
     &3.3772e-01_rb,2.9577e-01_rb,2.5329e-01_rb,2.1108e-01_rb,1.6887e-01_rb,1.2604e-01_rb, &
     &7.8045e-02_rb,3.5455e-02_rb,1.6952e-02_rb/)
      kao(:, 2, 6,13) = (/ &
     &2.6913e-01_rb,2.3492e-01_rb,2.0161e-01_rb,1.6802e-01_rb,1.3597e-01_rb,9.6507e-02_rb, &
     &6.1842e-02_rb,3.0282e-02_rb,3.2161e-02_rb/)
      kao(:, 3, 6,13) = (/ &
     &1.8198e-01_rb,1.5897e-01_rb,1.3649e-01_rb,1.1355e-01_rb,1.0014e-01_rb,6.6575e-02_rb, &
     &4.2969e-02_rb,3.1778e-02_rb,5.1438e-02_rb/)
      kao(:, 4, 6,13) = (/ &
     &9.5916e-02_rb,8.3628e-02_rb,7.1947e-02_rb,6.5931e-02_rb,5.4714e-02_rb,4.3955e-02_rb, &
     &3.9042e-02_rb,4.5021e-02_rb,5.3865e-02_rb/)
      kao(:, 5, 6,13) = (/ &
     &3.8338e-02_rb,3.3822e-02_rb,2.8989e-02_rb,4.7523e-02_rb,4.4391e-02_rb,4.9375e-02_rb, &
     &5.3458e-02_rb,6.4090e-02_rb,6.5458e-02_rb/)
      kao(:, 1, 7,13) = (/ &
     &8.4065e-01_rb,7.3491e-01_rb,6.3155e-01_rb,5.2581e-01_rb,4.2110e-01_rb,3.1579e-01_rb, &
     &2.1053e-01_rb,1.0597e-01_rb,2.9312e-06_rb/)
      kao(:, 2, 7,13) = (/ &
     &8.3906e-01_rb,7.3475e-01_rb,6.2926e-01_rb,5.2485e-01_rb,4.1990e-01_rb,3.1520e-01_rb, &
     &2.1013e-01_rb,9.8180e-02_rb,5.1556e-06_rb/)
      kao(:, 3, 7,13) = (/ &
     &7.1003e-01_rb,6.2128e-01_rb,5.3203e-01_rb,4.4338e-01_rb,3.5504e-01_rb,2.6905e-01_rb, &
     &1.6926e-01_rb,7.8231e-02_rb,2.0966e-02_rb/)
      kao(:, 4, 7,13) = (/ &
     &5.6430e-01_rb,4.9377e-01_rb,4.2325e-01_rb,3.5272e-01_rb,2.8217e-01_rb,2.1549e-01_rb, &
     &1.3644e-01_rb,6.7355e-02_rb,4.1095e-02_rb/)
      kao(:, 5, 7,13) = (/ &
     &3.6362e-01_rb,3.1751e-01_rb,2.7269e-01_rb,2.2725e-01_rb,1.9431e-01_rb,1.4468e-01_rb, &
     &9.9409e-02_rb,6.2076e-02_rb,6.3795e-02_rb/)
      kao(:, 1, 8,13) = (/ &
     &2.2236e+00_rb,1.9437e+00_rb,1.6676e+00_rb,1.3884e+00_rb,1.1107e+00_rb,8.3381e-01_rb, &
     &5.5588e-01_rb,2.7795e-01_rb,1.5745e-06_rb/)
      kao(:, 2, 8,13) = (/ &
     &2.2078e+00_rb,1.9318e+00_rb,1.6572e+00_rb,1.3798e+00_rb,1.1048e+00_rb,8.2853e-01_rb, &
     &5.5239e-01_rb,2.7779e-01_rb,2.9445e-06_rb/)
      kao(:, 3, 8,13) = (/ &
     &2.2271e+00_rb,1.9469e+00_rb,1.6686e+00_rb,1.3901e+00_rb,1.1125e+00_rb,8.3518e-01_rb, &
     &5.5623e-01_rb,2.8153e-01_rb,5.1096e-06_rb/)
      kao(:, 4, 8,13) = (/ &
     &2.1637e+00_rb,1.8932e+00_rb,1.6227e+00_rb,1.3534e+00_rb,1.0818e+00_rb,8.1139e-01_rb, &
     &5.4287e-01_rb,2.5697e-01_rb,9.9333e-06_rb/)
      kao(:, 5, 8,13) = (/ &
     &1.8021e+00_rb,1.5805e+00_rb,1.3532e+00_rb,1.1288e+00_rb,9.0270e-01_rb,6.7733e-01_rb, &
     &4.5729e-01_rb,2.1399e-01_rb,3.0555e-05_rb/)
      kao(:, 1, 9,13) = (/ &
     &1.0795e+01_rb,9.4461e+00_rb,8.0967e+00_rb,6.7526e+00_rb,5.3928e+00_rb,4.0485e+00_rb, &
     &2.6990e+00_rb,1.3503e+00_rb,8.4365e-07_rb/)
      kao(:, 2, 9,13) = (/ &
     &1.1064e+01_rb,9.6811e+00_rb,8.2980e+00_rb,6.9149e+00_rb,5.5320e+00_rb,4.1492e+00_rb, &
     &2.7660e+00_rb,1.3844e+00_rb,1.6130e-06_rb/)
      kao(:, 3, 9,13) = (/ &
     &1.1399e+01_rb,9.9733e+00_rb,8.5488e+00_rb,7.1204e+00_rb,5.6991e+00_rb,4.2710e+00_rb, &
     &2.8497e+00_rb,1.4241e+00_rb,6.0805e-06_rb/)
      kao(:, 4, 9,13) = (/ &
     &1.1508e+01_rb,1.0061e+01_rb,8.6395e+00_rb,7.1995e+00_rb,5.7491e+00_rb,4.3120e+00_rb, &
     &2.8800e+00_rb,1.4386e+00_rb,1.4128e-05_rb/)
      kao(:, 5, 9,13) = (/ &
     &1.1400e+01_rb,9.9831e+00_rb,8.5576e+00_rb,7.1312e+00_rb,5.7049e+00_rb,4.2787e+00_rb, &
     &2.8526e+00_rb,1.4263e+00_rb,2.6902e-05_rb/)
      kao(:, 1,10,13) = (/ &
     &6.0229e+01_rb,5.2742e+01_rb,4.5172e+01_rb,3.7644e+01_rb,3.0138e+01_rb,2.2585e+01_rb, &
     &1.5057e+01_rb,7.5286e+00_rb,7.1488e-07_rb/)
      kao(:, 2,10,13) = (/ &
     &6.2236e+01_rb,5.4452e+01_rb,4.6675e+01_rb,3.8896e+01_rb,3.1118e+01_rb,2.3338e+01_rb, &
     &1.5559e+01_rb,7.7794e+00_rb,2.2353e-06_rb/)
      kao(:, 3,10,13) = (/ &
     &6.3127e+01_rb,5.5205e+01_rb,4.7321e+01_rb,3.9431e+01_rb,3.1516e+01_rb,2.3656e+01_rb, &
     &1.5745e+01_rb,7.8865e+00_rb,5.3891e-06_rb/)
      kao(:, 4,10,13) = (/ &
     &6.1110e+01_rb,5.3475e+01_rb,4.5823e+01_rb,3.8155e+01_rb,3.0566e+01_rb,2.2893e+01_rb, &
     &1.5262e+01_rb,7.6312e+00_rb,1.1169e-05_rb/)
      kao(:, 5,10,13) = (/ &
     &6.4796e+01_rb,5.6740e+01_rb,4.8597e+01_rb,4.0457e+01_rb,3.2404e+01_rb,2.4299e+01_rb, &
     &1.6199e+01_rb,8.1078e+00_rb,2.0550e-05_rb/)
      kao(:, 1,11,13) = (/ &
     &1.1504e+02_rb,1.0058e+02_rb,8.6129e+01_rb,7.1842e+01_rb,5.7476e+01_rb,4.3140e+01_rb, &
     &2.8738e+01_rb,1.4357e+01_rb,9.3132e-07_rb/)
      kao(:, 2,11,13) = (/ &
     &1.1792e+02_rb,1.0317e+02_rb,8.8507e+01_rb,7.3697e+01_rb,5.8960e+01_rb,4.4221e+01_rb, &
     &2.9502e+01_rb,1.4740e+01_rb,2.4724e-06_rb/)
      kao(:, 3,11,13) = (/ &
     &1.2218e+02_rb,1.0691e+02_rb,9.1705e+01_rb,7.6362e+01_rb,6.1091e+01_rb,4.5817e+01_rb, &
     &3.0544e+01_rb,1.5272e+01_rb,5.4659e-06_rb/)
      kao(:, 4,11,13) = (/ &
     &1.2095e+02_rb,1.0583e+02_rb,9.0715e+01_rb,7.5596e+01_rb,6.0430e+01_rb,4.5404e+01_rb, &
     &3.0213e+01_rb,1.5120e+01_rb,1.0930e-05_rb/)
      kao(:, 5,11,13) = (/ &
     &1.1751e+02_rb,1.0271e+02_rb,8.8136e+01_rb,7.3302e+01_rb,5.8706e+01_rb,4.4021e+01_rb, &
     &2.9378e+01_rb,1.4677e+01_rb,1.9680e-05_rb/)
      kao(:, 1,12,13) = (/ &
     &1.6045e+02_rb,1.4037e+02_rb,1.2031e+02_rb,1.0025e+02_rb,8.0142e+01_rb,6.0153e+01_rb, &
     &4.0104e+01_rb,2.0052e+01_rb,1.7476e-06_rb/)
      kao(:, 2,12,13) = (/ &
     &1.6342e+02_rb,1.4301e+02_rb,1.2246e+02_rb,1.0215e+02_rb,8.1640e+01_rb,6.1227e+01_rb, &
     &4.0821e+01_rb,2.0431e+01_rb,2.4757e-06_rb/)
      kao(:, 3,12,13) = (/ &
     &1.6723e+02_rb,1.4633e+02_rb,1.2542e+02_rb,1.0452e+02_rb,8.3616e+01_rb,6.2709e+01_rb, &
     &4.1810e+01_rb,2.0904e+01_rb,5.2593e-06_rb/)
      kao(:, 4,12,13) = (/ &
     &1.7014e+02_rb,1.4884e+02_rb,1.2747e+02_rb,1.0623e+02_rb,8.4980e+01_rb,6.3783e+01_rb, &
     &4.2524e+01_rb,2.1245e+01_rb,9.8363e-06_rb/)
      kao(:, 5,12,13) = (/ &
     &1.6634e+02_rb,1.4563e+02_rb,1.2476e+02_rb,1.0397e+02_rb,8.3150e+01_rb,6.2327e+01_rb, &
     &4.1610e+01_rb,2.0792e+01_rb,1.5825e-05_rb/)
      kao(:, 1,13,13) = (/ &
     &1.7420e+02_rb,1.5232e+02_rb,1.3056e+02_rb,1.0880e+02_rb,8.7095e+01_rb,6.5324e+01_rb, &
     &4.3550e+01_rb,2.1773e+01_rb,2.0359e-06_rb/)
      kao(:, 2,13,13) = (/ &
     &1.7505e+02_rb,1.5318e+02_rb,1.3125e+02_rb,1.0942e+02_rb,8.7528e+01_rb,6.5649e+01_rb, &
     &4.3765e+01_rb,2.1882e+01_rb,3.6885e-06_rb/)
      kao(:, 3,13,13) = (/ &
     &1.7705e+02_rb,1.5491e+02_rb,1.3279e+02_rb,1.1065e+02_rb,8.8524e+01_rb,6.6391e+01_rb, &
     &4.4262e+01_rb,2.2130e+01_rb,4.3611e-06_rb/)
      kao(:, 4,13,13) = (/ &
     &1.7817e+02_rb,1.5590e+02_rb,1.3363e+02_rb,1.1135e+02_rb,8.9083e+01_rb,6.6758e+01_rb, &
     &4.4540e+01_rb,2.2253e+01_rb,7.0150e-06_rb/)
      kao(:, 5,13,13) = (/ &
     &1.7584e+02_rb,1.5385e+02_rb,1.3199e+02_rb,1.0999e+02_rb,8.7917e+01_rb,6.5939e+01_rb, &
     &4.3996e+01_rb,2.1972e+01_rb,1.1356e-05_rb/)
      kao(:, 1, 1,14) = (/ &
     &1.3576e-03_rb,5.9355e-03_rb,1.2070e-02_rb,3.4616e-02_rb,5.3142e-02_rb,6.5578e-02_rb, &
     &7.6852e-02_rb,8.4718e-02_rb,1.0628e-01_rb/)
      kao(:, 2, 1,14) = (/ &
     &1.4884e-03_rb,6.6850e-03_rb,2.0385e-02_rb,5.2397e-02_rb,7.2846e-02_rb,8.9954e-02_rb, &
     &1.0527e-01_rb,1.1545e-01_rb,1.4569e-01_rb/)
      kao(:, 3, 1,14) = (/ &
     &1.6038e-03_rb,1.0153e-02_rb,3.3679e-02_rb,7.2611e-02_rb,9.6223e-02_rb,1.1901e-01_rb, &
     &1.3945e-01_rb,1.5228e-01_rb,1.9245e-01_rb/)
      kao(:, 4, 1,14) = (/ &
     &1.7044e-03_rb,1.3918e-02_rb,4.9229e-02_rb,9.2925e-02_rb,1.2323e-01_rb,1.5253e-01_rb, &
     &1.7914e-01_rb,1.9539e-01_rb,2.4647e-01_rb/)
      kao(:, 5, 1,14) = (/ &
     &1.8000e-03_rb,1.6377e-02_rb,6.7350e-02_rb,1.1593e-01_rb,1.5369e-01_rb,1.9039e-01_rb, &
     &2.2401e-01_rb,2.4471e-01_rb,3.0739e-01_rb/)
      kao(:, 1, 2,14) = (/ &
     &2.3036e-03_rb,8.1410e-03_rb,1.1112e-02_rb,1.5744e-02_rb,3.9389e-02_rb,5.9885e-02_rb, &
     &7.1149e-02_rb,8.0027e-02_rb,7.8847e-02_rb/)
      kao(:, 2, 2,14) = (/ &
     &2.5134e-03_rb,1.3270e-02_rb,1.6290e-02_rb,2.9767e-02_rb,5.8863e-02_rb,8.3096e-02_rb, &
     &9.8866e-02_rb,1.1160e-01_rb,1.1776e-01_rb/)
      kao(:, 3, 2,14) = (/ &
     &2.6930e-03_rb,1.4292e-02_rb,1.9903e-02_rb,4.6191e-02_rb,8.3472e-02_rb,1.1096e-01_rb, &
     &1.3216e-01_rb,1.4974e-01_rb,1.6719e-01_rb/)
      kao(:, 4, 2,14) = (/ &
     &2.7875e-03_rb,1.4114e-02_rb,2.4277e-02_rb,6.6428e-02_rb,1.1553e-01_rb,1.4377e-01_rb, &
     &1.7103e-01_rb,1.9428e-01_rb,2.3106e-01_rb/)
      kao(:, 5, 2,14) = (/ &
     &2.8719e-03_rb,1.9476e-02_rb,3.5900e-02_rb,9.2156e-02_rb,1.4592e-01_rb,1.8154e-01_rb, &
     &2.1576e-01_rb,2.4502e-01_rb,2.9183e-01_rb/)
      kao(:, 1, 3,14) = (/ &
     &5.1130e-03_rb,4.4754e-03_rb,1.7095e-02_rb,1.4542e-02_rb,2.0610e-02_rb,2.6947e-02_rb, &
     &5.0520e-02_rb,6.8277e-02_rb,4.1188e-02_rb/)
      kao(:, 2, 3,14) = (/ &
     &5.4209e-03_rb,4.7463e-03_rb,2.5747e-02_rb,2.2058e-02_rb,2.5589e-02_rb,4.5664e-02_rb, &
     &7.6140e-02_rb,9.7102e-02_rb,5.5959e-02_rb/)
      kao(:, 3, 3,14) = (/ &
     &5.7296e-03_rb,5.0218e-03_rb,2.3801e-02_rb,3.0343e-02_rb,3.8012e-02_rb,6.9885e-02_rb, &
     &1.1346e-01_rb,1.3263e-01_rb,8.0840e-02_rb/)
      kao(:, 4, 3,14) = (/ &
     &5.9993e-03_rb,9.9078e-03_rb,3.1169e-02_rb,3.4849e-02_rb,5.4434e-02_rb,1.0180e-01_rb, &
     &1.5168e-01_rb,1.7523e-01_rb,1.1385e-01_rb/)
      kao(:, 5, 3,14) = (/ &
     &6.2420e-03_rb,1.8221e-02_rb,3.8314e-02_rb,4.8056e-02_rb,8.0195e-02_rb,1.4525e-01_rb, &
     &1.9539e-01_rb,2.2499e-01_rb,1.6255e-01_rb/)
      kao(:, 1, 4,14) = (/ &
     &1.0992e-02_rb,9.6146e-03_rb,8.2555e-03_rb,1.9598e-02_rb,2.6210e-02_rb,2.3069e-02_rb, &
     &2.4334e-02_rb,4.0918e-02_rb,4.3847e-02_rb/)
      kao(:, 2, 4,14) = (/ &
     &1.1861e-02_rb,1.0380e-02_rb,8.8997e-03_rb,3.1372e-02_rb,3.1071e-02_rb,3.3427e-02_rb, &
     &3.7533e-02_rb,6.6384e-02_rb,5.4303e-02_rb/)
      kao(:, 3, 4,14) = (/ &
     &1.2630e-02_rb,1.1058e-02_rb,1.5158e-02_rb,4.6362e-02_rb,4.1015e-02_rb,4.0980e-02_rb, &
     &5.2909e-02_rb,1.0322e-01_rb,7.5185e-02_rb/)
      kao(:, 4, 4,14) = (/ &
     &1.3324e-02_rb,1.1665e-02_rb,2.8664e-02_rb,4.2673e-02_rb,5.2993e-02_rb,5.8606e-02_rb, &
     &7.8441e-02_rb,1.5172e-01_rb,9.8036e-02_rb/)
      kao(:, 5, 4,14) = (/ &
     &1.3148e-02_rb,1.1477e-02_rb,4.1837e-02_rb,5.5479e-02_rb,6.7238e-02_rb,7.8671e-02_rb, &
     &1.1999e-01_rb,1.9993e-01_rb,1.2716e-01_rb/)
      kao(:, 1, 5,14) = (/ &
     &2.2062e-02_rb,1.9303e-02_rb,1.6547e-02_rb,1.3790e-02_rb,1.9210e-02_rb,3.4370e-02_rb, &
     &2.6360e-02_rb,2.4628e-02_rb,2.1281e-02_rb/)
      kao(:, 2, 5,14) = (/ &
     &2.4016e-02_rb,2.1018e-02_rb,1.8036e-02_rb,1.5017e-02_rb,3.4207e-02_rb,3.8549e-02_rb, &
     &3.6828e-02_rb,3.7924e-02_rb,4.8887e-02_rb/)
      kao(:, 3, 5,14) = (/ &
     &2.4307e-02_rb,2.1199e-02_rb,1.8170e-02_rb,2.0814e-02_rb,5.1440e-02_rb,4.8654e-02_rb, &
     &5.0226e-02_rb,5.5730e-02_rb,8.3554e-02_rb/)
      kao(:, 4, 5,14) = (/ &
     &2.3589e-02_rb,2.0645e-02_rb,1.7700e-02_rb,3.6957e-02_rb,7.2673e-02_rb,6.2587e-02_rb, &
     &6.6284e-02_rb,7.5553e-02_rb,1.2501e-01_rb/)
      kao(:, 5, 5,14) = (/ &
     &2.3359e-02_rb,2.0344e-02_rb,1.7524e-02_rb,5.3828e-02_rb,7.8003e-02_rb,7.9260e-02_rb, &
     &8.5892e-02_rb,1.2047e-01_rb,1.2633e-01_rb/)
      kao(:, 1, 6,14) = (/ &
     &4.0073e-02_rb,3.4956e-02_rb,2.9962e-02_rb,2.4964e-02_rb,1.9975e-02_rb,1.9358e-02_rb, &
     &3.3916e-02_rb,2.3741e-02_rb,4.4372e-06_rb/)
      kao(:, 2, 6,14) = (/ &
     &4.0377e-02_rb,3.5184e-02_rb,3.0162e-02_rb,2.5240e-02_rb,2.0191e-02_rb,3.6273e-02_rb, &
     &4.6417e-02_rb,3.4541e-02_rb,7.2649e-06_rb/)
      kao(:, 3, 6,14) = (/ &
     &4.0665e-02_rb,3.5540e-02_rb,3.0502e-02_rb,2.5425e-02_rb,2.0338e-02_rb,5.3239e-02_rb, &
     &5.4545e-02_rb,5.0382e-02_rb,1.1617e-05_rb/)
      kao(:, 4, 6,14) = (/ &
     &4.0830e-02_rb,3.5726e-02_rb,3.0620e-02_rb,2.5522e-02_rb,4.1685e-02_rb,7.6060e-02_rb, &
     &6.8633e-02_rb,6.7501e-02_rb,4.8322e-02_rb/)
      kao(:, 5, 6,14) = (/ &
     &4.0974e-02_rb,3.5852e-02_rb,3.0742e-02_rb,2.5614e-02_rb,6.3799e-02_rb,1.0670e-01_rb, &
     &8.5712e-02_rb,8.9786e-02_rb,9.4019e-02_rb/)
      kao(:, 1, 7,14) = (/ &
     &3.5883e-01_rb,3.1506e-01_rb,2.6911e-01_rb,2.2518e-01_rb,1.7880e-01_rb,1.3456e-01_rb, &
     &8.9710e-02_rb,5.4606e-02_rb,2.4375e-06_rb/)
      kao(:, 2, 7,14) = (/ &
     &1.1133e-01_rb,9.7402e-02_rb,8.3516e-02_rb,6.9576e-02_rb,5.5665e-02_rb,4.1750e-02_rb, &
     &3.4893e-02_rb,3.9977e-02_rb,4.2861e-06_rb/)
      kao(:, 3, 7,14) = (/ &
     &7.4556e-02_rb,6.5240e-02_rb,5.5921e-02_rb,4.6603e-02_rb,3.7275e-02_rb,2.7967e-02_rb, &
     &5.4564e-02_rb,5.0979e-02_rb,7.1669e-06_rb/)
      kao(:, 4, 7,14) = (/ &
     &7.6453e-02_rb,6.6897e-02_rb,5.7506e-02_rb,4.7791e-02_rb,3.8237e-02_rb,4.2981e-02_rb, &
     &7.9842e-02_rb,6.6791e-02_rb,1.1410e-05_rb/)
      kao(:, 5, 7,14) = (/ &
     &7.6151e-02_rb,6.6633e-02_rb,5.7121e-02_rb,4.7606e-02_rb,3.8095e-02_rb,6.8563e-02_rb, &
     &1.1205e-01_rb,9.1020e-02_rb,1.8778e-05_rb/)
      kao(:, 1, 8,14) = (/ &
     &1.6212e+00_rb,1.4215e+00_rb,1.2160e+00_rb,1.0153e+00_rb,8.1222e-01_rb,6.0796e-01_rb, &
     &4.0531e-01_rb,2.0267e-01_rb,1.3659e-06_rb/)
      kao(:, 2, 8,14) = (/ &
     &1.0949e+00_rb,9.5809e-01_rb,8.2118e-01_rb,6.8434e-01_rb,5.4744e-01_rb,4.1058e-01_rb, &
     &2.7374e-01_rb,1.3687e-01_rb,2.5240e-06_rb/)
      kao(:, 3, 8,14) = (/ &
     &5.4276e-01_rb,4.7782e-01_rb,4.0969e-01_rb,3.4354e-01_rb,2.7305e-01_rb,2.0349e-01_rb, &
     &1.3658e-01_rb,8.0855e-02_rb,4.4900e-06_rb/)
      kao(:, 4, 8,14) = (/ &
     &1.6665e-01_rb,1.4583e-01_rb,1.2498e-01_rb,1.0416e-01_rb,8.3314e-02_rb,6.2486e-02_rb, &
     &4.1670e-02_rb,8.3057e-02_rb,7.7801e-06_rb/)
      kao(:, 5, 8,14) = (/ &
     &1.6956e-01_rb,1.4835e-01_rb,1.2718e-01_rb,1.0598e-01_rb,8.5407e-02_rb,6.3592e-02_rb, &
     &5.5348e-02_rb,1.2090e-01_rb,1.2725e-05_rb/)
      kao(:, 1, 9,14) = (/ &
     &1.1544e+01_rb,1.0088e+01_rb,8.6483e+00_rb,7.2069e+00_rb,5.7730e+00_rb,4.3233e+00_rb, &
     &2.8823e+00_rb,1.4419e+00_rb,7.6692e-07_rb/)
      kao(:, 2, 9,14) = (/ &
     &9.0308e+00_rb,7.9019e+00_rb,6.7729e+00_rb,5.6441e+00_rb,4.5156e+00_rb,3.3865e+00_rb, &
     &2.2577e+00_rb,1.1268e+00_rb,1.4786e-06_rb/)
      kao(:, 3, 9,14) = (/ &
     &6.2830e+00_rb,5.4975e+00_rb,4.7120e+00_rb,3.9269e+00_rb,3.1414e+00_rb,2.3561e+00_rb, &
     &1.5707e+00_rb,7.8536e-01_rb,2.7689e-06_rb/)
      kao(:, 4, 9,14) = (/ &
     &3.6304e+00_rb,3.1764e+00_rb,2.7089e+00_rb,2.2550e+00_rb,1.8152e+00_rb,1.3614e+00_rb, &
     &9.0300e-01_rb,4.5381e-01_rb,4.9167e-06_rb/)
      kao(:, 5, 9,14) = (/ &
     &1.4890e+00_rb,1.3035e+00_rb,1.1168e+00_rb,9.3072e-01_rb,7.4457e-01_rb,5.5869e-01_rb, &
     &3.7227e-01_rb,1.8616e-01_rb,8.2396e-06_rb/)
      kao(:, 1,10,14) = (/ &
     &7.2198e+01_rb,6.3175e+01_rb,5.4150e+01_rb,4.5125e+01_rb,3.6100e+01_rb,2.7075e+01_rb, &
     &1.8050e+01_rb,9.0251e+00_rb,4.5184e-07_rb/)
      kao(:, 2,10,14) = (/ &
     &6.4999e+01_rb,5.6876e+01_rb,4.8750e+01_rb,4.0625e+01_rb,3.2499e+01_rb,2.4375e+01_rb, &
     &1.6250e+01_rb,8.1250e+00_rb,9.0060e-07_rb/)
      kao(:, 3,10,14) = (/ &
     &5.3041e+01_rb,4.6463e+01_rb,3.9827e+01_rb,3.3188e+01_rb,2.6599e+01_rb,1.9891e+01_rb, &
     &1.3300e+01_rb,6.6377e+00_rb,1.7393e-06_rb/)
      kao(:, 4,10,14) = (/ &
     &4.2448e+01_rb,3.7144e+01_rb,3.1913e+01_rb,2.6594e+01_rb,2.1225e+01_rb,1.5957e+01_rb, &
     &1.0638e+01_rb,5.3187e+00_rb,3.1449e-06_rb/)
      kao(:, 5,10,14) = (/ &
     &2.5727e+01_rb,2.2511e+01_rb,1.9295e+01_rb,1.6144e+01_rb,1.2810e+01_rb,9.6477e+00_rb, &
     &6.4321e+00_rb,3.2028e+00_rb,5.3500e-06_rb/)
      kao(:, 1,11,14) = (/ &
     &1.4503e+02_rb,1.2691e+02_rb,1.0890e+02_rb,9.0641e+01_rb,7.2517e+01_rb,5.4390e+01_rb, &
     &3.6259e+01_rb,1.8129e+01_rb,3.6866e-07_rb/)
      kao(:, 2,11,14) = (/ &
     &1.4340e+02_rb,1.2547e+02_rb,1.0755e+02_rb,8.9629e+01_rb,7.1700e+01_rb,5.3764e+01_rb, &
     &3.5851e+01_rb,1.7926e+01_rb,7.4257e-07_rb/)
      kao(:, 3,11,14) = (/ &
     &1.2279e+02_rb,1.0744e+02_rb,9.2097e+01_rb,7.6747e+01_rb,6.1397e+01_rb,4.6046e+01_rb, &
     &3.0699e+01_rb,1.5350e+01_rb,1.4132e-06_rb/)
      kao(:, 4,11,14) = (/ &
     &1.0332e+02_rb,9.0407e+01_rb,7.7495e+01_rb,6.4577e+01_rb,5.1661e+01_rb,3.8673e+01_rb, &
     &2.5832e+01_rb,1.2916e+01_rb,2.5660e-06_rb/)
      kao(:, 5,11,14) = (/ &
     &8.4752e+01_rb,7.4338e+01_rb,6.3562e+01_rb,5.3097e+01_rb,4.2377e+01_rb,3.1859e+01_rb, &
     &2.1189e+01_rb,1.0594e+01_rb,4.3954e-06_rb/)
      kao(:, 1,12,14) = (/ &
     &2.1206e+02_rb,1.8578e+02_rb,1.5923e+02_rb,1.3269e+02_rb,1.0615e+02_rb,7.9613e+01_rb, &
     &5.3076e+01_rb,2.6538e+01_rb,2.9519e-07_rb/)
      kao(:, 2,12,14) = (/ &
     &2.1371e+02_rb,1.8696e+02_rb,1.6019e+02_rb,1.3354e+02_rb,1.0696e+02_rb,8.0220e+01_rb, &
     &5.3478e+01_rb,2.6706e+01_rb,5.9412e-07_rb/)
      kao(:, 3,12,14) = (/ &
     &1.9963e+02_rb,1.7467e+02_rb,1.4971e+02_rb,1.2476e+02_rb,9.9809e+01_rb,7.4861e+01_rb, &
     &4.9906e+01_rb,2.4953e+01_rb,1.1512e-06_rb/)
      kao(:, 4,12,14) = (/ &
     &1.7291e+02_rb,1.5155e+02_rb,1.2989e+02_rb,1.0824e+02_rb,8.6596e+01_rb,6.4945e+01_rb, &
     &4.3297e+01_rb,2.1649e+01_rb,2.0935e-06_rb/)
      kao(:, 5,12,14) = (/ &
     &1.4993e+02_rb,1.3106e+02_rb,1.1245e+02_rb,9.3704e+01_rb,7.4891e+01_rb,5.6224e+01_rb, &
     &3.7445e+01_rb,1.8742e+01_rb,5.3426e-06_rb/)
      kao(:, 1,13,14) = (/ &
     &2.3337e+02_rb,2.0357e+02_rb,1.7476e+02_rb,1.4563e+02_rb,1.1651e+02_rb,8.7377e+01_rb, &
     &5.8250e+01_rb,2.9126e+01_rb,2.3044e-07_rb/)
      kao(:, 2,13,14) = (/ &
     &2.3770e+02_rb,2.0799e+02_rb,1.7827e+02_rb,1.4856e+02_rb,1.1885e+02_rb,8.9136e+01_rb, &
     &5.9426e+01_rb,2.9711e+01_rb,4.7533e-07_rb/)
      kao(:, 3,13,14) = (/ &
     &2.3937e+02_rb,2.0945e+02_rb,1.7953e+02_rb,1.4961e+02_rb,1.1968e+02_rb,8.9763e+01_rb, &
     &5.9841e+01_rb,2.9920e+01_rb,1.2740e-06_rb/)
      kao(:, 4,13,14) = (/ &
     &2.1343e+02_rb,1.8675e+02_rb,1.6007e+02_rb,1.3339e+02_rb,1.0671e+02_rb,8.0033e+01_rb, &
     &5.3356e+01_rb,2.6677e+01_rb,3.7441e-06_rb/)
      kao(:, 5,13,14) = (/ &
     &1.8434e+02_rb,1.6130e+02_rb,1.3825e+02_rb,1.1521e+02_rb,9.2167e+01_rb,6.9126e+01_rb, &
     &4.6084e+01_rb,2.3083e+01_rb,8.0408e-06_rb/)
      kao(:, 1, 1,15) = (/ &
     &4.3757e-04_rb,1.7661e-02_rb,6.5219e-02_rb,9.6736e-02_rb,1.2696e-01_rb,1.5472e-01_rb, &
     &1.7677e-01_rb,1.8124e-01_rb,2.5391e-01_rb/)
      kao(:, 2, 1,15) = (/ &
     &4.7244e-04_rb,2.2315e-02_rb,8.7744e-02_rb,1.3008e-01_rb,1.7055e-01_rb,2.0764e-01_rb, &
     &2.3757e-01_rb,2.4446e-01_rb,3.4110e-01_rb/)
      kao(:, 3, 1,15) = (/ &
     &4.9936e-04_rb,3.5467e-02_rb,1.1452e-01_rb,1.6964e-01_rb,2.2220e-01_rb,2.7019e-01_rb, &
     &3.0873e-01_rb,3.1867e-01_rb,4.4441e-01_rb/)
      kao(:, 4, 1,15) = (/ &
     &5.2088e-04_rb,5.0547e-02_rb,1.4538e-01_rb,2.1535e-01_rb,2.8212e-01_rb,3.4264e-01_rb, &
     &3.9053e-01_rb,4.0360e-01_rb,5.6424e-01_rb/)
      kao(:, 5, 1,15) = (/ &
     &5.3484e-04_rb,7.2405e-02_rb,1.8030e-01_rb,2.6712e-01_rb,3.4967e-01_rb,4.2472e-01_rb, &
     &4.8331e-01_rb,4.9868e-01_rb,6.9933e-01_rb/)
      kao(:, 1, 2,15) = (/ &
     &5.8930e-04_rb,5.2056e-04_rb,3.8823e-02_rb,9.4547e-02_rb,1.2473e-01_rb,1.5329e-01_rb, &
     &1.7814e-01_rb,1.9186e-01_rb,2.4946e-01_rb/)
      kao(:, 2, 2,15) = (/ &
     &6.3170e-04_rb,5.6011e-04_rb,6.0488e-02_rb,1.2953e-01_rb,1.7093e-01_rb,2.0982e-01_rb, &
     &2.4359e-01_rb,2.6142e-01_rb,3.4185e-01_rb/)
      kao(:, 3, 2,15) = (/ &
     &6.6396e-04_rb,1.3947e-02_rb,9.3321e-02_rb,1.7148e-01_rb,2.2617e-01_rb,2.7768e-01_rb, &
     &3.2205e-01_rb,3.4444e-01_rb,4.5233e-01_rb/)
      kao(:, 4, 2,15) = (/ &
     &6.8779e-04_rb,3.2900e-02_rb,1.4029e-01_rb,2.2027e-01_rb,2.9047e-01_rb,3.5662e-01_rb, &
     &4.1356e-01_rb,4.4128e-01_rb,5.8094e-01_rb/)
      kao(:, 5, 2,15) = (/ &
     &7.0403e-04_rb,3.1909e-02_rb,1.8526e-01_rb,2.7582e-01_rb,3.6375e-01_rb,4.4666e-01_rb, &
     &5.1781e-01_rb,5.5205e-01_rb,7.2748e-01_rb/)
      kao(:, 1, 3,15) = (/ &
     &1.0314e-03_rb,9.3537e-04_rb,7.8009e-04_rb,3.5190e-02_rb,7.8660e-02_rb,1.3993e-01_rb, &
     &1.6539e-01_rb,1.8481e-01_rb,1.5700e-01_rb/)
      kao(:, 2, 3,15) = (/ &
     &1.1044e-03_rb,9.7099e-04_rb,3.0472e-03_rb,6.2470e-02_rb,1.3066e-01_rb,1.9656e-01_rb, &
     &2.3229e-01_rb,2.5938e-01_rb,2.6190e-01_rb/)
      kao(:, 3, 3,15) = (/ &
     &1.1610e-03_rb,1.0227e-03_rb,3.8685e-02_rb,9.5707e-02_rb,2.0189e-01_rb,2.6560e-01_rb, &
     &3.1379e-01_rb,3.5039e-01_rb,4.0559e-01_rb/)
      kao(:, 4, 3,15) = (/ &
     &1.1999e-03_rb,1.0620e-03_rb,6.9192e-02_rb,1.4962e-01_rb,2.7976e-01_rb,3.4701e-01_rb, &
     &4.1016e-01_rb,4.5806e-01_rb,5.5953e-01_rb/)
      kao(:, 5, 3,15) = (/ &
     &1.2331e-03_rb,1.0899e-03_rb,6.4034e-02_rb,2.1379e-01_rb,3.5531e-01_rb,4.4075e-01_rb, &
     &5.2094e-01_rb,5.8205e-01_rb,7.1061e-01_rb/)
      kao(:, 1, 4,15) = (/ &
     &1.8101e-03_rb,1.5857e-03_rb,1.3614e-03_rb,1.1362e-03_rb,2.5114e-02_rb,5.7242e-02_rb, &
     &1.1151e-01_rb,1.6445e-01_rb,5.0940e-02_rb/)
      kao(:, 2, 4,15) = (/ &
     &1.9417e-03_rb,1.7009e-03_rb,1.4618e-03_rb,1.2222e-03_rb,6.6639e-02_rb,9.6622e-02_rb, &
     &1.8463e-01_rb,2.3787e-01_rb,1.3472e-01_rb/)
      kao(:, 3, 4,15) = (/ &
     &2.0504e-03_rb,1.7934e-03_rb,1.5465e-03_rb,1.2907e-03_rb,6.6457e-02_rb,1.6526e-01_rb, &
     &2.8062e-01_rb,3.2903e-01_rb,1.3147e-01_rb/)
      kao(:, 4, 4,15) = (/ &
     &2.1280e-03_rb,1.8660e-03_rb,1.6076e-03_rb,5.7880e-02_rb,1.0798e-01_rb,2.4822e-01_rb, &
     &3.8266e-01_rb,4.3856e-01_rb,2.1657e-01_rb/)
      kao(:, 5, 4,15) = (/ &
     &3.4223e-03_rb,3.1028e-03_rb,2.6690e-03_rb,1.0905e-01_rb,1.5760e-01_rb,3.4585e-01_rb, &
     &4.9399e-01_rb,5.6645e-01_rb,3.1389e-01_rb/)
      kao(:, 1, 5,15) = (/ &
     &2.8907e-03_rb,2.5398e-03_rb,2.1780e-03_rb,1.8164e-03_rb,1.4541e-03_rb,1.0940e-03_rb, &
     &7.0483e-02_rb,1.0515e-01_rb,9.1867e-06_rb/)
      kao(:, 2, 5,15) = (/ &
     &3.1254e-03_rb,2.7393e-03_rb,2.3477e-03_rb,1.9606e-03_rb,1.5698e-03_rb,3.9518e-02_rb, &
     &7.4031e-02_rb,1.7523e-01_rb,1.5406e-05_rb/)
      kao(:, 3, 5,15) = (/ &
     &6.7235e-03_rb,5.8882e-03_rb,5.0497e-03_rb,4.2126e-03_rb,3.3747e-03_rb,9.4974e-02_rb, &
     &1.2240e-01_rb,2.6835e-01_rb,2.4302e-05_rb/)
      kao(:, 4, 5,15) = (/ &
     &1.0699e-02_rb,9.3645e-03_rb,8.0383e-03_rb,6.6993e-03_rb,5.3674e-03_rb,1.6163e-01_rb, &
     &2.1236e-01_rb,3.9941e-01_rb,3.6217e-05_rb/)
      kao(:, 5, 5,15) = (/ &
     &1.3717e-02_rb,1.2003e-02_rb,1.0303e-02_rb,8.5901e-03_rb,7.6360e-02_rb,1.2908e-01_rb, &
     &3.1926e-01_rb,5.2644e-01_rb,1.7649e-01_rb/)
      kao(:, 1, 6,15) = (/ &
     &8.1592e-03_rb,7.4369e-03_rb,6.3758e-03_rb,5.3146e-03_rb,4.2520e-03_rb,3.1909e-03_rb, &
     &2.1298e-03_rb,5.0122e-02_rb,5.2510e-06_rb/)
      kao(:, 2, 6,15) = (/ &
     &1.6119e-02_rb,1.4482e-02_rb,1.2421e-02_rb,1.0082e-02_rb,8.0668e-03_rb,6.0513e-03_rb, &
     &2.1472e-02_rb,7.5343e-02_rb,9.3553e-06_rb/)
      kao(:, 3, 6,15) = (/ &
     &2.2594e-02_rb,1.9767e-02_rb,1.6948e-02_rb,1.4132e-02_rb,1.1307e-02_rb,8.4832e-03_rb, &
     &8.7408e-02_rb,1.3678e-01_rb,1.5081e-05_rb/)
      kao(:, 4, 6,15) = (/ &
     &2.8287e-02_rb,2.4765e-02_rb,2.1227e-02_rb,1.7693e-02_rb,1.4154e-02_rb,1.0619e-02_rb, &
     &1.5774e-01_rb,2.2631e-01_rb,2.2824e-05_rb/)
      kao(:, 5, 6,15) = (/ &
     &3.3007e-02_rb,2.8867e-02_rb,2.4778e-02_rb,2.0634e-02_rb,1.6523e-02_rb,1.2392e-02_rb, &
     &2.0991e-01_rb,3.3594e-01_rb,3.3161e-05_rb/)
      kao(:, 1, 7,15) = (/ &
     &3.2923e-02_rb,2.8801e-02_rb,2.4671e-02_rb,2.0232e-02_rb,1.6462e-02_rb,1.2339e-02_rb, &
     &8.2302e-03_rb,4.1175e-03_rb,2.9069e-06_rb/)
      kao(:, 2, 7,15) = (/ &
     &4.4354e-02_rb,3.8819e-02_rb,3.3267e-02_rb,2.7723e-02_rb,2.2187e-02_rb,1.6641e-02_rb, &
     &1.1094e-02_rb,4.4470e-02_rb,5.3667e-06_rb/)
      kao(:, 3, 7,15) = (/ &
     &5.4986e-02_rb,4.8115e-02_rb,4.1243e-02_rb,3.4364e-02_rb,2.7498e-02_rb,2.0642e-02_rb, &
     &1.3901e-02_rb,1.0510e-01_rb,9.1519e-06_rb/)
      kao(:, 4, 7,15) = (/ &
     &6.4798e-02_rb,5.6696e-02_rb,4.8186e-02_rb,4.0508e-02_rb,3.2405e-02_rb,2.4095e-02_rb, &
     &1.6208e-02_rb,1.7836e-01_rb,1.4620e-05_rb/)
      kao(:, 5, 7,15) = (/ &
     &7.2855e-02_rb,6.3755e-02_rb,5.4641e-02_rb,4.5546e-02_rb,3.6442e-02_rb,2.7544e-02_rb, &
     &1.8370e-02_rb,1.8524e-01_rb,2.2125e-05_rb/)
      kao(:, 1, 8,15) = (/ &
     &9.4104e-02_rb,8.2337e-02_rb,7.0574e-02_rb,5.8796e-02_rb,4.7043e-02_rb,3.5284e-02_rb, &
     &2.3531e-02_rb,1.1762e-02_rb,1.5730e-06_rb/)
      kao(:, 2, 8,15) = (/ &
     &1.1684e-01_rb,1.0226e-01_rb,8.7581e-02_rb,7.3021e-02_rb,5.8384e-02_rb,4.3796e-02_rb, &
     &2.9205e-02_rb,1.4602e-02_rb,3.1057e-06_rb/)
      kao(:, 3, 8,15) = (/ &
     &1.3899e-01_rb,1.2165e-01_rb,1.0424e-01_rb,8.6874e-02_rb,6.9512e-02_rb,5.2129e-02_rb, &
     &3.4755e-02_rb,1.7377e-02_rb,5.5778e-06_rb/)
      kao(:, 4, 8,15) = (/ &
     &1.6026e-01_rb,1.4023e-01_rb,1.2021e-01_rb,1.0019e-01_rb,8.0158e-02_rb,6.0094e-02_rb, &
     &4.0057e-02_rb,2.0043e-02_rb,9.3711e-06_rb/)
      kao(:, 5, 8,15) = (/ &
     &1.7464e-01_rb,1.5283e-01_rb,1.3102e-01_rb,1.0918e-01_rb,8.7353e-02_rb,6.5505e-02_rb, &
     &4.3670e-02_rb,2.1848e-02_rb,1.4780e-05_rb/)
      kao(:, 1, 9,15) = (/ &
     &4.3914e-01_rb,3.8426e-01_rb,3.2523e-01_rb,2.7101e-01_rb,2.1679e-01_rb,1.6466e-01_rb, &
     &1.0980e-01_rb,5.4190e-02_rb,8.3394e-07_rb/)
      kao(:, 2, 9,15) = (/ &
     &5.3157e-01_rb,4.6496e-01_rb,3.9866e-01_rb,3.3222e-01_rb,2.6580e-01_rb,1.9932e-01_rb, &
     &1.3289e-01_rb,6.6437e-02_rb,1.7793e-06_rb/)
      kao(:, 3, 9,15) = (/ &
     &6.2792e-01_rb,5.4930e-01_rb,4.7092e-01_rb,3.9242e-01_rb,3.1384e-01_rb,2.3543e-01_rb, &
     &1.5698e-01_rb,7.8487e-02_rb,3.4095e-06_rb/)
      kao(:, 4, 9,15) = (/ &
     &6.9612e-01_rb,6.0916e-01_rb,5.2207e-01_rb,4.4160e-01_rb,3.4808e-01_rb,2.6107e-01_rb, &
     &1.7405e-01_rb,8.7030e-02_rb,6.0449e-06_rb/)
      kao(:, 5, 9,15) = (/ &
     &7.1518e-01_rb,6.2568e-01_rb,5.3633e-01_rb,4.4688e-01_rb,3.5754e-01_rb,2.6806e-01_rb, &
     &1.7876e-01_rb,8.9378e-02_rb,9.9175e-06_rb/)
      kao(:, 1,10,15) = (/ &
     &1.3796e+01_rb,1.2071e+01_rb,1.0346e+01_rb,8.6223e+00_rb,6.8983e+00_rb,5.1733e+00_rb, &
     &3.4490e+00_rb,1.7245e+00_rb,4.7027e-07_rb/)
      kao(:, 2,10,15) = (/ &
     &2.6233e+00_rb,2.2954e+00_rb,1.9673e+00_rb,1.6395e+00_rb,1.3116e+00_rb,9.8356e-01_rb, &
     &6.5573e-01_rb,3.2783e-01_rb,1.0735e-06_rb/)
      kao(:, 3,10,15) = (/ &
     &2.9028e+00_rb,2.5398e+00_rb,2.1774e+00_rb,1.8145e+00_rb,1.4515e+00_rb,1.0885e+00_rb, &
     &7.2570e-01_rb,3.6294e-01_rb,2.1946e-06_rb/)
      kao(:, 4,10,15) = (/ &
     &2.9628e+00_rb,2.5919e+00_rb,2.2220e+00_rb,1.8518e+00_rb,1.4813e+00_rb,1.1109e+00_rb, &
     &7.4065e-01_rb,3.7038e-01_rb,4.0439e-06_rb/)
      kao(:, 5,10,15) = (/ &
     &3.0421e+00_rb,2.6615e+00_rb,2.2818e+00_rb,1.9012e+00_rb,1.5210e+00_rb,1.1408e+00_rb, &
     &7.6043e-01_rb,3.8023e-01_rb,6.8742e-06_rb/)
      kao(:, 1,11,15) = (/ &
     &5.1563e+01_rb,4.5117e+01_rb,3.8674e+01_rb,3.2227e+01_rb,2.5782e+01_rb,1.9336e+01_rb, &
     &1.2891e+01_rb,6.4453e+00_rb,4.0707e-07_rb/)
      kao(:, 2,11,15) = (/ &
     &4.0297e+00_rb,3.5262e+00_rb,3.0227e+00_rb,2.5187e+00_rb,2.0150e+00_rb,1.5474e+00_rb, &
     &1.0073e+00_rb,5.0377e-01_rb,9.5505e-07_rb/)
      kao(:, 3,11,15) = (/ &
     &4.2638e+00_rb,3.7317e+00_rb,3.1981e+00_rb,2.6654e+00_rb,2.1321e+00_rb,1.5993e+00_rb, &
     &1.0662e+00_rb,5.3312e-01_rb,1.9370e-06_rb/)
      kao(:, 4,11,15) = (/ &
     &4.5225e+00_rb,3.9566e+00_rb,3.3892e+00_rb,2.8266e+00_rb,2.2603e+00_rb,1.6955e+00_rb, &
     &1.1303e+00_rb,5.6504e-01_rb,3.5823e-06_rb/)
      kao(:, 5,11,15) = (/ &
     &4.7336e+00_rb,4.1418e+00_rb,3.5502e+00_rb,2.9582e+00_rb,2.3669e+00_rb,1.7755e+00_rb, &
     &1.1834e+00_rb,5.9176e-01_rb,6.1298e-06_rb/)
      kao(:, 1,12,15) = (/ &
     &1.0350e+02_rb,9.0564e+01_rb,7.7625e+01_rb,6.4688e+01_rb,5.1750e+01_rb,3.8814e+01_rb, &
     &2.5874e+01_rb,1.2937e+01_rb,3.8485e-07_rb/)
      kao(:, 2,12,15) = (/ &
     &3.5400e+01_rb,3.0974e+01_rb,2.7218e+01_rb,2.2126e+01_rb,1.7700e+01_rb,1.3275e+01_rb, &
     &8.8500e+00_rb,4.4247e+00_rb,8.6295e-07_rb/)
      kao(:, 3,12,15) = (/ &
     &4.7917e+00_rb,4.1902e+00_rb,3.5927e+00_rb,2.9945e+00_rb,2.3939e+00_rb,1.7963e+00_rb, &
     &1.1970e+00_rb,5.9908e-01_rb,1.7414e-06_rb/)
      kao(:, 4,12,15) = (/ &
     &4.7749e+00_rb,4.1779e+00_rb,3.5811e+00_rb,2.9844e+00_rb,2.3876e+00_rb,1.7905e+00_rb, &
     &1.1938e+00_rb,5.9689e-01_rb,3.1576e-06_rb/)
      kao(:, 5,12,15) = (/ &
     &4.7755e+00_rb,4.1774e+00_rb,3.5815e+00_rb,2.9845e+00_rb,2.3876e+00_rb,1.7910e+00_rb, &
     &1.1938e+00_rb,5.9711e-01_rb,5.4455e-06_rb/)
      kao(:, 1,13,15) = (/ &
     &1.3564e+02_rb,1.2035e+02_rb,1.0244e+02_rb,8.5373e+01_rb,6.8293e+01_rb,5.1221e+01_rb, &
     &3.4147e+01_rb,1.7074e+01_rb,3.5301e-07_rb/)
      kao(:, 2,13,15) = (/ &
     &6.6923e+01_rb,5.8556e+01_rb,5.0190e+01_rb,4.1824e+01_rb,3.3458e+01_rb,2.5094e+01_rb, &
     &1.6730e+01_rb,8.3652e+00_rb,7.8175e-07_rb/)
      kao(:, 3,13,15) = (/ &
     &3.7003e+00_rb,3.2381e+00_rb,2.7755e+00_rb,2.3126e+00_rb,1.8500e+00_rb,1.3876e+00_rb, &
     &9.2513e-01_rb,4.6258e-01_rb,1.5659e-06_rb/)
      kao(:, 4,13,15) = (/ &
     &3.7328e+00_rb,3.2655e+00_rb,2.7985e+00_rb,2.3334e+00_rb,1.8661e+00_rb,1.3995e+00_rb, &
     &9.3310e-01_rb,4.6647e-01_rb,2.9454e-06_rb/)
      kao(:, 5,13,15) = (/ &
     &3.7209e+00_rb,3.2581e+00_rb,2.7929e+00_rb,2.3278e+00_rb,1.8618e+00_rb,1.3966e+00_rb, &
     &9.3071e-01_rb,4.6580e-01_rb,5.0591e-06_rb/)
      kao(:, 1, 1,16) = (/ &
     &3.4803e-04_rb,3.1375e-04_rb,9.0029e-02_rb,1.3230e-01_rb,1.7123e-01_rb,2.0432e-01_rb, &
     &2.2540e-01_rb,2.1561e-01_rb,3.4246e-01_rb/)
      kao(:, 2, 1,16) = (/ &
     &3.6711e-04_rb,6.2185e-02_rb,1.2251e-01_rb,1.8007e-01_rb,2.3310e-01_rb,2.7812e-01_rb, &
     &3.0698e-01_rb,2.9374e-01_rb,4.6619e-01_rb/)
      kao(:, 3, 1,16) = (/ &
     &3.8231e-04_rb,8.1789e-02_rb,1.6116e-01_rb,2.3685e-01_rb,3.0661e-01_rb,3.6587e-01_rb, &
     &4.0384e-01_rb,3.8690e-01_rb,6.1322e-01_rb/)
      kao(:, 4, 1,16) = (/ &
     &3.9398e-04_rb,1.0451e-01_rb,2.0587e-01_rb,3.0259e-01_rb,3.9179e-01_rb,4.6767e-01_rb, &
     &5.1634e-01_rb,4.9462e-01_rb,7.8358e-01_rb/)
      kao(:, 5, 1,16) = (/ &
     &4.0251e-04_rb,1.3022e-01_rb,2.5663e-01_rb,3.7711e-01_rb,4.8832e-01_rb,5.8296e-01_rb, &
     &6.4377e-01_rb,6.1717e-01_rb,9.7664e-01_rb/)
      kao(:, 1, 2,16) = (/ &
     &4.4386e-04_rb,3.9528e-04_rb,9.5606e-02_rb,1.4144e-01_rb,1.8490e-01_rb,2.2388e-01_rb, &
     &2.5317e-01_rb,2.5424e-01_rb,3.6979e-01_rb/)
      kao(:, 2, 2,16) = (/ &
     &4.6963e-04_rb,4.2143e-04_rb,1.3208e-01_rb,1.9543e-01_rb,2.5549e-01_rb,3.0939e-01_rb, &
     &3.5000e-01_rb,3.5157e-01_rb,5.1097e-01_rb/)
      kao(:, 3, 2,16) = (/ &
     &4.9052e-04_rb,4.4450e-04_rb,1.7597e-01_rb,2.6040e-01_rb,3.4044e-01_rb,4.1226e-01_rb, &
     &4.6651e-01_rb,4.6873e-01_rb,6.8087e-01_rb/)
      kao(:, 4, 2,16) = (/ &
     &5.0548e-04_rb,4.6448e-04_rb,2.2723e-01_rb,3.3632e-01_rb,4.3972e-01_rb,5.3255e-01_rb, &
     &6.0266e-01_rb,6.0594e-01_rb,8.7943e-01_rb/)
      kao(:, 5, 2,16) = (/ &
     &5.1704e-04_rb,1.4434e-01_rb,2.8579e-01_rb,4.2309e-01_rb,5.5324e-01_rb,6.7011e-01_rb, &
     &7.5842e-01_rb,7.6291e-01_rb,1.1065e+00_rb/)
      kao(:, 1, 3,16) = (/ &
     &7.4514e-04_rb,6.5703e-04_rb,5.6805e-04_rb,1.3986e-01_rb,1.8472e-01_rb,2.2721e-01_rb, &
     &2.6439e-01_rb,2.8334e-01_rb,3.6943e-01_rb/)
      kao(:, 2, 3,16) = (/ &
     &7.9344e-04_rb,7.0163e-04_rb,6.1002e-04_rb,1.9810e-01_rb,2.6159e-01_rb,3.2183e-01_rb, &
     &3.7436e-01_rb,4.0145e-01_rb,5.2319e-01_rb/)
      kao(:, 3, 3,16) = (/ &
     &8.3288e-04_rb,7.4056e-04_rb,6.4608e-04_rb,2.6931e-01_rb,3.5570e-01_rb,4.3753e-01_rb, &
     &5.0928e-01_rb,5.4624e-01_rb,7.1140e-01_rb/)
      kao(:, 4, 3,16) = (/ &
     &8.6418e-04_rb,7.7079e-04_rb,1.2130e-02_rb,3.5401e-01_rb,4.6743e-01_rb,5.7518e-01_rb, &
     &6.6923e-01_rb,7.1819e-01_rb,9.3487e-01_rb/)
      kao(:, 5, 3,16) = (/ &
     &8.8628e-04_rb,7.9797e-04_rb,3.0313e-01_rb,4.5187e-01_rb,5.9661e-01_rb,7.3417e-01_rb, &
     &8.5463e-01_rb,9.1729e-01_rb,1.1932e+00_rb/)
      kao(:, 1, 4,16) = (/ &
     &1.2636e-03_rb,1.1057e-03_rb,9.5136e-04_rb,7.9620e-04_rb,6.4188e-04_rb,2.1791e-01_rb, &
     &2.5758e-01_rb,2.8791e-01_rb,2.3364e-05_rb/)
      kao(:, 2, 4,16) = (/ &
     &1.3610e-03_rb,1.1957e-03_rb,1.0270e-03_rb,8.6413e-04_rb,7.0025e-04_rb,3.1776e-01_rb, &
     &3.7570e-01_rb,4.1991e-01_rb,3.8771e-05_rb/)
      kao(:, 3, 4,16) = (/ &
     &1.4417e-03_rb,1.2662e-03_rb,1.0946e-03_rb,9.2411e-04_rb,3.5694e-01_rb,4.4267e-01_rb, &
     &5.2346e-01_rb,5.8520e-01_rb,7.1388e-01_rb/)
      kao(:, 4, 4,16) = (/ &
     &1.5012e-03_rb,1.3268e-03_rb,1.1484e-03_rb,9.7089e-04_rb,4.7866e-01_rb,5.9391e-01_rb, &
     &7.0229e-01_rb,7.8545e-01_rb,9.5731e-01_rb/)
      kao(:, 5, 4,16) = (/ &
     &1.5493e-03_rb,1.3712e-03_rb,1.1941e-03_rb,1.0165e-03_rb,6.2163e-01_rb,7.7110e-01_rb, &
     &9.1212e-01_rb,1.0202e+00_rb,1.2432e+00_rb/)
      kao(:, 1, 5,16) = (/ &
     &1.9563e-03_rb,1.7130e-03_rb,1.4716e-03_rb,1.2286e-03_rb,9.8449e-04_rb,7.4210e-04_rb, &
     &5.0011e-04_rb,2.7578e-01_rb,1.4521e-05_rb/)
      kao(:, 2, 5,16) = (/ &
     &2.1251e-03_rb,1.8616e-03_rb,1.5991e-03_rb,1.3382e-03_rb,1.0747e-03_rb,8.1191e-04_rb, &
     &3.6392e-01_rb,4.1523e-01_rb,2.5317e-05_rb/)
      kao(:, 3, 5,16) = (/ &
     &2.2685e-03_rb,1.9902e-03_rb,1.7118e-03_rb,1.4319e-03_rb,1.1550e-03_rb,8.7607e-04_rb, &
     &5.2057e-01_rb,5.9412e-01_rb,4.1189e-05_rb/)
      kao(:, 4, 5,16) = (/ &
     &2.3834e-03_rb,2.0939e-03_rb,1.8039e-03_rb,1.5145e-03_rb,1.2237e-03_rb,9.3428e-04_rb, &
     &7.1378e-01_rb,8.1462e-01_rb,6.3394e-05_rb/)
      kao(:, 5, 5,16) = (/ &
     &2.4764e-03_rb,2.1786e-03_rb,1.8819e-03_rb,1.5832e-03_rb,1.2860e-03_rb,7.9256e-01_rb, &
     &9.4410e-01_rb,1.0778e+00_rb,9.2980e-05_rb/)
      kao(:, 1, 6,16) = (/ &
     &2.8061e-03_rb,2.4583e-03_rb,2.1085e-03_rb,1.7622e-03_rb,1.4067e-03_rb,1.0576e-03_rb, &
     &7.0829e-04_rb,1.6992e-01_rb,8.6758e-06_rb/)
      kao(:, 2, 6,16) = (/ &
     &3.0921e-03_rb,2.7056e-03_rb,2.3212e-03_rb,1.9356e-03_rb,1.5529e-03_rb,1.1677e-03_rb, &
     &7.8418e-04_rb,3.9086e-01_rb,1.5977e-05_rb/)
      kao(:, 3, 6,16) = (/ &
     &3.3322e-03_rb,2.9180e-03_rb,2.5057e-03_rb,2.0938e-03_rb,1.6795e-03_rb,1.2657e-03_rb, &
     &8.5374e-04_rb,5.7620e-01_rb,2.7234e-05_rb/)
      kao(:, 4, 6,16) = (/ &
     &3.5394e-03_rb,3.0979e-03_rb,2.6608e-03_rb,2.2263e-03_rb,1.7897e-03_rb,1.3517e-03_rb, &
     &9.1571e-04_rb,8.0997e-01_rb,4.3592e-05_rb/)
      kao(:, 5, 6,16) = (/ &
     &3.6973e-03_rb,3.2450e-03_rb,2.7879e-03_rb,2.3342e-03_rb,1.8813e-03_rb,1.4280e-03_rb, &
     &2.3591e-01_rb,1.0936e+00_rb,6.6108e-05_rb/)
      kao(:, 1, 7,16) = (/ &
     &4.1566e-03_rb,3.6382e-03_rb,3.1187e-03_rb,2.5984e-03_rb,2.0809e-03_rb,1.5620e-03_rb, &
     &1.0430e-03_rb,5.2415e-04_rb,5.0515e-06_rb/)
      kao(:, 2, 7,16) = (/ &
     &4.6397e-03_rb,4.0602e-03_rb,3.4821e-03_rb,2.9026e-03_rb,2.3244e-03_rb,1.7452e-03_rb, &
     &1.1674e-03_rb,5.8830e-04_rb,9.8982e-06_rb/)
      kao(:, 3, 7,16) = (/ &
     &5.0641e-03_rb,4.4282e-03_rb,3.8004e-03_rb,3.1687e-03_rb,2.5412e-03_rb,1.9080e-03_rb, &
     &1.2777e-03_rb,6.4842e-04_rb,1.7758e-05_rb/)
      kao(:, 4, 7,16) = (/ &
     &5.4224e-03_rb,4.7462e-03_rb,4.0771e-03_rb,3.3992e-03_rb,2.7272e-03_rb,2.0505e-03_rb, &
     &1.3779e-03_rb,7.0354e-04_rb,2.9670e-05_rb/)
      kao(:, 5, 7,16) = (/ &
     &5.7229e-03_rb,5.0101e-03_rb,4.3011e-03_rb,3.5983e-03_rb,2.8833e-03_rb,2.1757e-03_rb, &
     &1.4658e-03_rb,6.1847e-01_rb,4.6639e-05_rb/)
      kao(:, 1, 8,16) = (/ &
     &7.1654e-03_rb,6.2688e-03_rb,5.3745e-03_rb,4.4778e-03_rb,3.5800e-03_rb,2.6883e-03_rb, &
     &1.7931e-03_rb,8.9706e-04_rb,2.8888e-06_rb/)
      kao(:, 2, 8,16) = (/ &
     &8.1399e-03_rb,7.1118e-03_rb,6.0997e-03_rb,5.0831e-03_rb,4.0665e-03_rb,3.0517e-03_rb, &
     &2.0386e-03_rb,1.0214e-03_rb,6.0412e-06_rb/)
      kao(:, 3, 8,16) = (/ &
     &8.9800e-03_rb,7.8609e-03_rb,6.7393e-03_rb,5.6216e-03_rb,4.4978e-03_rb,3.3745e-03_rb, &
     &2.2551e-03_rb,1.1328e-03_rb,1.1454e-05_rb/)
      kao(:, 4, 8,16) = (/ &
     &9.7139e-03_rb,8.5067e-03_rb,7.2942e-03_rb,6.0777e-03_rb,4.8682e-03_rb,3.6593e-03_rb, &
     &2.4437e-03_rb,1.2326e-03_rb,2.0029e-05_rb/)
      kao(:, 5, 8,16) = (/ &
     &1.0337e-02_rb,9.0473e-03_rb,7.7560e-03_rb,6.4703e-03_rb,5.1860e-03_rb,3.8948e-03_rb, &
     &2.6072e-03_rb,1.3207e-03_rb,3.2759e-05_rb/)
      kao(:, 1, 9,16) = (/ &
     &2.3247e-02_rb,2.0336e-02_rb,1.7429e-02_rb,1.4524e-02_rb,1.1625e-02_rb,8.7188e-03_rb, &
     &5.8147e-03_rb,2.9059e-03_rb,1.6015e-06_rb/)
      kao(:, 2, 9,16) = (/ &
     &2.6840e-02_rb,2.3482e-02_rb,2.0132e-02_rb,1.6770e-02_rb,1.3422e-02_rb,1.0063e-02_rb, &
     &6.7117e-03_rb,3.3577e-03_rb,3.6050e-06_rb/)
      kao(:, 3, 9,16) = (/ &
     &3.0078e-02_rb,2.6329e-02_rb,2.2553e-02_rb,1.8793e-02_rb,1.5044e-02_rb,1.1280e-02_rb, &
     &7.5240e-03_rb,3.7656e-03_rb,7.2546e-06_rb/)
      kao(:, 4, 9,16) = (/ &
     &3.2925e-02_rb,2.8810e-02_rb,2.4697e-02_rb,2.0586e-02_rb,3.0583e-02_rb,1.2361e-02_rb, &
     &8.2421e-03_rb,4.1354e-03_rb,1.3329e-05_rb/)
      kao(:, 5, 9,16) = (/ &
     &3.5342e-02_rb,3.0934e-02_rb,2.6503e-02_rb,2.2104e-02_rb,1.7684e-02_rb,1.3262e-02_rb, &
     &8.8536e-03_rb,4.4397e-03_rb,2.2693e-05_rb/)
      kao(:, 1,10,16) = (/ &
     &8.4703e-02_rb,7.4117e-02_rb,6.3535e-02_rb,5.2927e-02_rb,4.2343e-02_rb,3.1773e-02_rb, &
     &2.1175e-02_rb,1.0588e-02_rb,9.2688e-07_rb/)
      kao(:, 2,10,16) = (/ &
     &9.9410e-02_rb,8.7009e-02_rb,7.4556e-02_rb,6.2160e-02_rb,4.9698e-02_rb,3.7265e-02_rb, &
     &2.4850e-02_rb,1.2429e-02_rb,2.2335e-06_rb/)
      kao(:, 3,10,16) = (/ &
     &1.1291e-01_rb,9.8729e-02_rb,8.4692e-02_rb,7.0598e-02_rb,5.6422e-02_rb,4.2355e-02_rb, &
     &2.8232e-02_rb,1.4106e-02_rb,4.7476e-06_rb/)
      kao(:, 4,10,16) = (/ &
     &1.2498e-01_rb,1.0930e-01_rb,9.3695e-02_rb,7.8057e-02_rb,6.2479e-02_rb,4.6859e-02_rb, &
     &3.1244e-02_rb,1.5627e-02_rb,9.1259e-06_rb/)
      kao(:, 5,10,16) = (/ &
     &1.3523e-01_rb,1.1838e-01_rb,1.0149e-01_rb,8.4512e-02_rb,6.7619e-02_rb,5.0737e-02_rb, &
     &3.3842e-02_rb,1.6917e-02_rb,1.6155e-05_rb/)
      kao(:, 1,11,16) = (/ &
     &1.1702e-01_rb,1.0242e-01_rb,8.7781e-02_rb,7.3100e-02_rb,5.8528e-02_rb,4.3847e-02_rb, &
     &2.9250e-02_rb,1.4631e-02_rb,8.3202e-07_rb/)
      kao(:, 2,11,16) = (/ &
     &1.3733e-01_rb,1.2017e-01_rb,1.0300e-01_rb,8.5853e-02_rb,6.8653e-02_rb,5.1492e-02_rb, &
     &3.4322e-02_rb,1.7153e-02_rb,2.0128e-06_rb/)
      kao(:, 3,11,16) = (/ &
     &1.5584e-01_rb,1.3638e-01_rb,1.1685e-01_rb,9.7461e-02_rb,7.7843e-02_rb,5.8372e-02_rb, &
     &3.8976e-02_rb,1.9510e-02_rb,4.3129e-06_rb/)
      kao(:, 4,11,16) = (/ &
     &1.7239e-01_rb,1.5086e-01_rb,1.2928e-01_rb,1.0779e-01_rb,8.6176e-02_rb,6.4725e-02_rb, &
     &4.3097e-02_rb,2.1516e-02_rb,8.3117e-06_rb/)
      kao(:, 5,11,16) = (/ &
     &1.8654e-01_rb,1.6325e-01_rb,1.3989e-01_rb,1.1667e-01_rb,9.3408e-02_rb,7.0060e-02_rb, &
     &4.6775e-02_rb,2.3342e-02_rb,1.4729e-05_rb/)
      kao(:, 1,12,16) = (/ &
     &1.1854e-01_rb,1.0376e-01_rb,8.8946e-02_rb,7.4074e-02_rb,5.9255e-02_rb,4.4504e-02_rb, &
     &2.9679e-02_rb,1.4800e-02_rb,7.4749e-07_rb/)
      kao(:, 2,12,16) = (/ &
     &1.3943e-01_rb,1.2204e-01_rb,1.0463e-01_rb,8.7342e-02_rb,6.9856e-02_rb,5.2356e-02_rb, &
     &3.4894e-02_rb,1.7410e-02_rb,1.8161e-06_rb/)
      kao(:, 3,12,16) = (/ &
     &1.5836e-01_rb,1.3868e-01_rb,1.1885e-01_rb,9.9001e-02_rb,7.9338e-02_rb,5.9413e-02_rb, &
     &3.9707e-02_rb,1.9806e-02_rb,3.8901e-06_rb/)
      kao(:, 4,12,16) = (/ &
     &1.7966e-01_rb,1.5728e-01_rb,1.3478e-01_rb,1.1217e-01_rb,8.9784e-02_rb,6.7270e-02_rb, &
     &4.4900e-02_rb,2.2432e-02_rb,7.2752e-06_rb/)
      kao(:, 5,12,16) = (/ &
     &2.0635e-01_rb,1.8060e-01_rb,1.5479e-01_rb,1.2891e-01_rb,1.0321e-01_rb,7.7324e-02_rb, &
     &5.1591e-02_rb,2.5791e-02_rb,1.2040e-05_rb/)
      kao(:, 1,13,16) = (/ &
     &2.1930e-01_rb,8.1271e-02_rb,6.9650e-02_rb,5.7973e-02_rb,4.6358e-02_rb,3.4782e-02_rb, &
     &2.3204e-02_rb,1.1588e-02_rb,6.6806e-07_rb/)
      kao(:, 2,13,16) = (/ &
     &1.1205e-01_rb,9.8083e-02_rb,8.4056e-02_rb,7.0107e-02_rb,5.6050e-02_rb,4.2050e-02_rb, &
     &2.7998e-02_rb,1.4011e-02_rb,1.5755e-06_rb/)
      kao(:, 3,13,16) = (/ &
     &1.3646e-01_rb,1.1932e-01_rb,1.0236e-01_rb,8.5263e-02_rb,6.8236e-02_rb,5.1131e-02_rb, &
     &3.4091e-02_rb,1.7065e-02_rb,3.1454e-06_rb/)
      kao(:, 4,13,16) = (/ &
     &1.5976e-01_rb,1.3979e-01_rb,1.1981e-01_rb,9.9935e-02_rb,7.9931e-02_rb,5.9857e-02_rb, &
     &3.9898e-02_rb,1.9961e-02_rb,5.5256e-06_rb/)
      kao(:, 5,13,16) = (/ &
     &1.7769e-01_rb,1.5548e-01_rb,1.3326e-01_rb,1.1109e-01_rb,8.8840e-02_rb,6.6629e-02_rb, &
     &4.4447e-02_rb,2.2232e-02_rb,9.1943e-06_rb/)

!     The array KAO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The third index 
!     runs over the g-channel (1 to 16).

      kao_mco2( 1, :, 1) = (/ &
     & 1.09539e-04_rb, 1.17067e-04_rb, 1.25113e-04_rb, 1.33712e-04_rb, 1.42902e-04_rb, &
     & 1.52724e-04_rb, 1.63221e-04_rb, 1.74439e-04_rb, 1.86428e-04_rb, 1.99241e-04_rb, &
     & 2.12934e-04_rb, 2.27569e-04_rb, 2.43210e-04_rb, 2.59926e-04_rb, 2.77790e-04_rb, &
     & 2.96883e-04_rb, 3.17287e-04_rb, 3.39094e-04_rb, 3.62400e-04_rb/)
      kao_mco2( 2, :, 1) = (/ &
     & 1.25202e-04_rb, 1.34718e-04_rb, 1.44957e-04_rb, 1.55974e-04_rb, 1.67829e-04_rb, &
     & 1.80585e-04_rb, 1.94311e-04_rb, 2.09079e-04_rb, 2.24971e-04_rb, 2.42069e-04_rb, &
     & 2.60468e-04_rb, 2.80265e-04_rb, 3.01567e-04_rb, 3.24488e-04_rb, 3.49150e-04_rb, &
     & 3.75688e-04_rb, 4.04242e-04_rb, 4.34966e-04_rb, 4.68026e-04_rb/)
      kao_mco2( 3, :, 1) = (/ &
     & 1.12112e-04_rb, 1.21090e-04_rb, 1.30786e-04_rb, 1.41259e-04_rb, 1.52571e-04_rb, &
     & 1.64788e-04_rb, 1.77984e-04_rb, 1.92237e-04_rb, 2.07631e-04_rb, 2.24257e-04_rb, &
     & 2.42215e-04_rb, 2.61611e-04_rb, 2.82560e-04_rb, 3.05187e-04_rb, 3.29625e-04_rb, &
     & 3.56021e-04_rb, 3.84530e-04_rb, 4.15322e-04_rb, 4.48580e-04_rb/)
      kao_mco2( 4, :, 1) = (/ &
     & 9.74130e-05_rb, 1.05372e-04_rb, 1.13982e-04_rb, 1.23295e-04_rb, 1.33369e-04_rb, &
     & 1.44265e-04_rb, 1.56053e-04_rb, 1.68803e-04_rb, 1.82595e-04_rb, 1.97514e-04_rb, &
     & 2.13652e-04_rb, 2.31109e-04_rb, 2.49992e-04_rb, 2.70418e-04_rb, 2.92512e-04_rb, &
     & 3.16412e-04_rb, 3.42265e-04_rb, 3.70230e-04_rb, 4.00479e-04_rb/)
      kao_mco2( 5, :, 1) = (/ &
     & 8.71018e-05_rb, 9.40759e-05_rb, 1.01608e-04_rb, 1.09744e-04_rb, 1.18531e-04_rb, &
     & 1.28022e-04_rb, 1.38272e-04_rb, 1.49343e-04_rb, 1.61301e-04_rb, 1.74216e-04_rb, &
     & 1.88166e-04_rb, 2.03232e-04_rb, 2.19504e-04_rb, 2.37079e-04_rb, 2.56062e-04_rb, &
     & 2.76565e-04_rb, 2.98709e-04_rb, 3.22626e-04_rb, 3.48458e-04_rb/)
      kao_mco2( 6, :, 1) = (/ &
     & 7.55256e-05_rb, 8.17414e-05_rb, 8.84688e-05_rb, 9.57500e-05_rb, 1.03630e-04_rb, &
     & 1.12159e-04_rb, 1.21390e-04_rb, 1.31381e-04_rb, 1.42193e-04_rb, 1.53896e-04_rb, &
     & 1.66562e-04_rb, 1.80270e-04_rb, 1.95107e-04_rb, 2.11164e-04_rb, 2.28543e-04_rb, &
     & 2.47353e-04_rb, 2.67710e-04_rb, 2.89743e-04_rb, 3.13589e-04_rb/)
      kao_mco2( 7, :, 1) = (/ &
     & 5.31515e-05_rb, 6.06869e-05_rb, 6.92907e-05_rb, 7.91143e-05_rb, 9.03306e-05_rb, &
     & 1.03137e-04_rb, 1.17759e-04_rb, 1.34454e-04_rb, 1.53516e-04_rb, 1.75281e-04_rb, &
     & 2.00131e-04_rb, 2.28504e-04_rb, 2.60900e-04_rb, 2.97888e-04_rb, 3.40121e-04_rb, &
     & 3.88341e-04_rb, 4.43397e-04_rb, 5.06259e-04_rb, 5.78033e-04_rb/)
      kao_mco2( 8, :, 1) = (/ &
     & 2.52471e-04_rb, 2.96005e-04_rb, 3.47045e-04_rb, 4.06886e-04_rb, 4.77045e-04_rb, &
     & 5.59302e-04_rb, 6.55742e-04_rb, 7.68811e-04_rb, 9.01377e-04_rb, 1.05680e-03_rb, &
     & 1.23902e-03_rb, 1.45267e-03_rb, 1.70315e-03_rb, 1.99683e-03_rb, 2.34114e-03_rb, &
     & 2.74482e-03_rb, 3.21811e-03_rb, 3.77300e-03_rb, 4.42358e-03_rb/)
      kao_mco2( 9, :, 1) = (/ &
     & 4.06711e-05_rb, 4.53161e-05_rb, 5.04917e-05_rb, 5.62583e-05_rb, 6.26836e-05_rb, &
     & 6.98427e-05_rb, 7.78194e-05_rb, 8.67071e-05_rb, 9.66100e-05_rb, 1.07644e-04_rb, &
     & 1.19938e-04_rb, 1.33636e-04_rb, 1.48898e-04_rb, 1.65904e-04_rb, 1.84852e-04_rb, &
     & 2.05964e-04_rb, 2.29487e-04_rb, 2.55697e-04_rb, 2.84900e-04_rb/)
      kao_mco2( 1, :, 2) = (/ &
     & 2.01759e-04_rb, 2.15641e-04_rb, 2.30478e-04_rb, 2.46336e-04_rb, 2.63285e-04_rb, &
     & 2.81400e-04_rb, 3.00761e-04_rb, 3.21455e-04_rb, 3.43573e-04_rb, 3.67212e-04_rb, &
     & 3.92477e-04_rb, 4.19482e-04_rb, 4.48344e-04_rb, 4.79192e-04_rb, 5.12162e-04_rb, &
     & 5.47401e-04_rb, 5.85064e-04_rb, 6.25319e-04_rb, 6.68343e-04_rb/)
      kao_mco2( 2, :, 2) = (/ &
     & 2.53461e-04_rb, 2.70916e-04_rb, 2.89574e-04_rb, 3.09516e-04_rb, 3.30832e-04_rb, &
     & 3.53616e-04_rb, 3.77969e-04_rb, 4.03999e-04_rb, 4.31822e-04_rb, 4.61561e-04_rb, &
     & 4.93348e-04_rb, 5.27324e-04_rb, 5.63640e-04_rb, 6.02457e-04_rb, 6.43948e-04_rb, &
     & 6.88295e-04_rb, 7.35697e-04_rb, 7.86364e-04_rb, 8.40519e-04_rb/)
      kao_mco2( 3, :, 2) = (/ &
     & 2.58821e-04_rb, 2.76943e-04_rb, 2.96334e-04_rb, 3.17082e-04_rb, 3.39283e-04_rb, &
     & 3.63038e-04_rb, 3.88457e-04_rb, 4.15655e-04_rb, 4.44758e-04_rb, 4.75899e-04_rb, &
     & 5.09220e-04_rb, 5.44874e-04_rb, 5.83024e-04_rb, 6.23845e-04_rb, 6.67525e-04_rb, &
     & 7.14263e-04_rb, 7.64273e-04_rb, 8.17785e-04_rb, 8.75043e-04_rb/)
      kao_mco2( 4, :, 2) = (/ &
     & 2.46588e-04_rb, 2.64630e-04_rb, 2.83993e-04_rb, 3.04771e-04_rb, 3.27071e-04_rb, &
     & 3.51001e-04_rb, 3.76683e-04_rb, 4.04244e-04_rb, 4.33821e-04_rb, 4.65563e-04_rb, &
     & 4.99627e-04_rb, 5.36183e-04_rb, 5.75414e-04_rb, 6.17515e-04_rb, 6.62697e-04_rb, &
     & 7.11185e-04_rb, 7.63220e-04_rb, 8.19063e-04_rb, 8.78991e-04_rb/)
      kao_mco2( 5, :, 2) = (/ &
     & 2.19140e-04_rb, 2.36464e-04_rb, 2.55158e-04_rb, 2.75330e-04_rb, 2.97097e-04_rb, &
     & 3.20585e-04_rb, 3.45929e-04_rb, 3.73277e-04_rb, 4.02787e-04_rb, 4.34630e-04_rb, &
     & 4.68991e-04_rb, 5.06068e-04_rb, 5.46076e-04_rb, 5.89247e-04_rb, 6.35831e-04_rb, &
     & 6.86097e-04_rb, 7.40338e-04_rb, 7.98867e-04_rb, 8.62022e-04_rb/)
      kao_mco2( 6, :, 2) = (/ &
     & 1.74073e-04_rb, 1.92221e-04_rb, 2.12260e-04_rb, 2.34388e-04_rb, 2.58824e-04_rb, &
     & 2.85807e-04_rb, 3.15603e-04_rb, 3.48505e-04_rb, 3.84837e-04_rb, 4.24957e-04_rb, &
     & 4.69260e-04_rb, 5.18181e-04_rb, 5.72202e-04_rb, 6.31855e-04_rb, 6.97727e-04_rb, &
     & 7.70466e-04_rb, 8.50789e-04_rb, 9.39485e-04_rb, 1.03743e-03_rb/)
      kao_mco2( 7, :, 2) = (/ &
     & 1.74359e-04_rb, 1.99276e-04_rb, 2.27753e-04_rb, 2.60299e-04_rb, 2.97497e-04_rb, &
     & 3.40010e-04_rb, 3.88599e-04_rb, 4.44130e-04_rb, 5.07598e-04_rb, 5.80135e-04_rb, &
     & 6.63039e-04_rb, 7.57789e-04_rb, 8.66079e-04_rb, 9.89845e-04_rb, 1.13130e-03_rb, &
     & 1.29296e-03_rb, 1.47773e-03_rb, 1.68890e-03_rb, 1.93025e-03_rb/)
      kao_mco2( 8, :, 2) = (/ &
     & 1.08215e-03_rb, 1.20760e-03_rb, 1.34759e-03_rb, 1.50382e-03_rb, 1.67815e-03_rb, &
     & 1.87270e-03_rb, 2.08980e-03_rb, 2.33206e-03_rb, 2.60242e-03_rb, 2.90411e-03_rb, &
     & 3.24078e-03_rb, 3.61648e-03_rb, 4.03573e-03_rb, 4.50359e-03_rb, 5.02568e-03_rb, &
     & 5.60830e-03_rb, 6.25846e-03_rb, 6.98399e-03_rb, 7.79363e-03_rb/)
      kao_mco2( 9, :, 2) = (/ &
     & 1.04969e-04_rb, 1.20766e-04_rb, 1.38939e-04_rb, 1.59848e-04_rb, 1.83903e-04_rb, &
     & 2.11578e-04_rb, 2.43418e-04_rb, 2.80049e-04_rb, 3.22193e-04_rb, 3.70678e-04_rb, &
     & 4.26461e-04_rb, 4.90638e-04_rb, 5.64472e-04_rb, 6.49418e-04_rb, 7.47147e-04_rb, &
     & 8.59583e-04_rb, 9.88940e-04_rb, 1.13776e-03_rb, 1.30898e-03_rb/)
      kao_mco2( 1, :, 3) = (/ &
     & 3.72106e-04_rb, 3.96252e-04_rb, 4.21966e-04_rb, 4.49347e-04_rb, 4.78506e-04_rb, &
     & 5.09557e-04_rb, 5.42623e-04_rb, 5.77834e-04_rb, 6.15330e-04_rb, 6.55260e-04_rb, &
     & 6.97781e-04_rb, 7.43060e-04_rb, 7.91278e-04_rb, 8.42626e-04_rb, 8.97304e-04_rb, &
     & 9.55532e-04_rb, 1.01754e-03_rb, 1.08357e-03_rb, 1.15388e-03_rb/)
      kao_mco2( 2, :, 3) = (/ &
     & 4.20563e-04_rb, 4.46162e-04_rb, 4.73319e-04_rb, 5.02130e-04_rb, 5.32693e-04_rb, &
     & 5.65118e-04_rb, 5.99516e-04_rb, 6.36007e-04_rb, 6.74720e-04_rb, 7.15789e-04_rb, &
     & 7.59358e-04_rb, 8.05579e-04_rb, 8.54613e-04_rb, 9.06632e-04_rb, 9.61817e-04_rb, &
     & 1.02036e-03_rb, 1.08247e-03_rb, 1.14836e-03_rb, 1.21826e-03_rb/)
      kao_mco2( 3, :, 3) = (/ &
     & 4.89664e-04_rb, 5.18321e-04_rb, 5.48654e-04_rb, 5.80764e-04_rb, 6.14752e-04_rb, &
     & 6.50729e-04_rb, 6.88812e-04_rb, 7.29124e-04_rb, 7.71795e-04_rb, 8.16963e-04_rb, &
     & 8.64774e-04_rb, 9.15384e-04_rb, 9.68955e-04_rb, 1.02566e-03_rb, 1.08569e-03_rb, &
     & 1.14922e-03_rb, 1.21648e-03_rb, 1.28767e-03_rb, 1.36303e-03_rb/)
      kao_mco2( 4, :, 3) = (/ &
     & 4.61143e-04_rb, 4.92198e-04_rb, 5.25343e-04_rb, 5.60720e-04_rb, 5.98480e-04_rb, &
     & 6.38783e-04_rb, 6.81799e-04_rb, 7.27713e-04_rb, 7.76718e-04_rb, 8.29023e-04_rb, &
     & 8.84851e-04_rb, 9.44438e-04_rb, 1.00804e-03_rb, 1.07592e-03_rb, 1.14837e-03_rb, &
     & 1.22571e-03_rb, 1.30825e-03_rb, 1.39635e-03_rb, 1.49038e-03_rb/)
      kao_mco2( 5, :, 3) = (/ &
     & 4.01988e-04_rb, 4.36672e-04_rb, 4.74349e-04_rb, 5.15278e-04_rb, 5.59737e-04_rb, &
     & 6.08032e-04_rb, 6.60495e-04_rb, 7.17484e-04_rb, 7.79390e-04_rb, 8.46638e-04_rb, &
     & 9.19688e-04_rb, 9.99041e-04_rb, 1.08524e-03_rb, 1.17888e-03_rb, 1.28059e-03_rb, &
     & 1.39109e-03_rb, 1.51111e-03_rb, 1.64149e-03_rb, 1.78313e-03_rb/)
      kao_mco2( 6, :, 3) = (/ &
     & 3.35536e-04_rb, 3.74371e-04_rb, 4.17700e-04_rb, 4.66045e-04_rb, 5.19985e-04_rb, &
     & 5.80169e-04_rb, 6.47318e-04_rb, 7.22238e-04_rb, 8.05831e-04_rb, 8.99098e-04_rb, &
     & 1.00316e-03_rb, 1.11927e-03_rb, 1.24881e-03_rb, 1.39335e-03_rb, 1.55461e-03_rb, &
     & 1.73455e-03_rb, 1.93530e-03_rb, 2.15930e-03_rb, 2.40921e-03_rb/)
      kao_mco2( 7, :, 3) = (/ &
     & 3.24677e-04_rb, 3.75160e-04_rb, 4.33491e-04_rb, 5.00893e-04_rb, 5.78774e-04_rb, &
     & 6.68765e-04_rb, 7.72749e-04_rb, 8.92900e-04_rb, 1.03173e-03_rb, 1.19215e-03_rb, &
     & 1.37751e-03_rb, 1.59170e-03_rb, 1.83918e-03_rb, 2.12515e-03_rb, 2.45558e-03_rb, &
     & 2.83738e-03_rb, 3.27856e-03_rb, 3.78832e-03_rb, 4.37735e-03_rb/)
      kao_mco2( 8, :, 3) = (/ &
     & 2.24656e-03_rb, 2.45550e-03_rb, 2.68386e-03_rb, 2.93347e-03_rb, 3.20629e-03_rb, &
     & 3.50448e-03_rb, 3.83041e-03_rb, 4.18665e-03_rb, 4.57602e-03_rb, 5.00160e-03_rb, &
     & 5.46677e-03_rb, 5.97519e-03_rb, 6.53090e-03_rb, 7.13829e-03_rb, 7.80217e-03_rb, &
     & 8.52780e-03_rb, 9.32091e-03_rb, 1.01878e-02_rb, 1.11353e-02_rb/)
      kao_mco2( 9, :, 3) = (/ &
     & 2.07746e-04_rb, 2.38909e-04_rb, 2.74746e-04_rb, 3.15959e-04_rb, 3.63355e-04_rb, &
     & 4.17860e-04_rb, 4.80541e-04_rb, 5.52625e-04_rb, 6.35521e-04_rb, 7.30852e-04_rb, &
     & 8.40484e-04_rb, 9.66561e-04_rb, 1.11155e-03_rb, 1.27829e-03_rb, 1.47004e-03_rb, &
     & 1.69055e-03_rb, 1.94414e-03_rb, 2.23577e-03_rb, 2.57115e-03_rb/)
      kao_mco2( 1, :, 4) = (/ &
     & 7.26052e-04_rb, 7.62476e-04_rb, 8.00726e-04_rb, 8.40896e-04_rb, 8.83081e-04_rb, &
     & 9.27382e-04_rb, 9.73905e-04_rb, 1.02276e-03_rb, 1.07407e-03_rb, 1.12795e-03_rb, &
     & 1.18454e-03_rb, 1.24396e-03_rb, 1.30637e-03_rb, 1.37190e-03_rb, 1.44073e-03_rb, &
     & 1.51300e-03_rb, 1.58890e-03_rb, 1.66861e-03_rb, 1.75232e-03_rb/)
      kao_mco2( 2, :, 4) = (/ &
     & 4.65815e-04_rb, 5.01167e-04_rb, 5.39203e-04_rb, 5.80126e-04_rb, 6.24154e-04_rb, &
     & 6.71524e-04_rb, 7.22489e-04_rb, 7.77322e-04_rb, 8.36316e-04_rb, 8.99788e-04_rb, &
     & 9.68077e-04_rb, 1.04155e-03_rb, 1.12060e-03_rb, 1.20564e-03_rb, 1.29714e-03_rb, &
     & 1.39559e-03_rb, 1.50151e-03_rb, 1.61546e-03_rb, 1.73807e-03_rb/)
      kao_mco2( 3, :, 4) = (/ &
     & 3.56225e-04_rb, 3.93073e-04_rb, 4.33732e-04_rb, 4.78598e-04_rb, 5.28105e-04_rb, &
     & 5.82732e-04_rb, 6.43010e-04_rb, 7.09524e-04_rb, 7.82918e-04_rb, 8.63903e-04_rb, &
     & 9.53266e-04_rb, 1.05187e-03_rb, 1.16068e-03_rb, 1.28074e-03_rb, 1.41322e-03_rb, &
     & 1.55941e-03_rb, 1.72071e-03_rb, 1.89870e-03_rb, 2.09511e-03_rb/)
      kao_mco2( 4, :, 4) = (/ &
     & 3.37845e-04_rb, 3.79675e-04_rb, 4.26684e-04_rb, 4.79514e-04_rb, 5.38884e-04_rb, &
     & 6.05606e-04_rb, 6.80589e-04_rb, 7.64855e-04_rb, 8.59555e-04_rb, 9.65980e-04_rb, &
     & 1.08558e-03_rb, 1.21999e-03_rb, 1.37105e-03_rb, 1.54080e-03_rb, 1.73157e-03_rb, &
     & 1.94597e-03_rb, 2.18691e-03_rb, 2.45767e-03_rb, 2.76197e-03_rb/)
      kao_mco2( 5, :, 4) = (/ &
     & 3.52456e-04_rb, 4.02782e-04_rb, 4.60294e-04_rb, 5.26017e-04_rb, 6.01126e-04_rb, &
     & 6.86958e-04_rb, 7.85046e-04_rb, 8.97140e-04_rb, 1.02524e-03_rb, 1.17163e-03_rb, &
     & 1.33892e-03_rb, 1.53010e-03_rb, 1.74858e-03_rb, 1.99825e-03_rb, 2.28358e-03_rb, &
     & 2.60964e-03_rb, 2.98226e-03_rb, 3.40809e-03_rb, 3.89471e-03_rb/)
      kao_mco2( 6, :, 4) = (/ &
     & 4.42884e-04_rb, 5.08187e-04_rb, 5.83119e-04_rb, 6.69100e-04_rb, 7.67758e-04_rb, &
     & 8.80963e-04_rb, 1.01086e-03_rb, 1.15991e-03_rb, 1.33094e-03_rb, 1.52718e-03_rb, &
     & 1.75237e-03_rb, 2.01075e-03_rb, 2.30724e-03_rb, 2.64744e-03_rb, 3.03780e-03_rb, &
     & 3.48572e-03_rb, 3.99969e-03_rb, 4.58944e-03_rb, 5.26614e-03_rb/)
      kao_mco2( 7, :, 4) = (/ &
     & 8.09850e-04_rb, 9.09940e-04_rb, 1.02240e-03_rb, 1.14876e-03_rb, 1.29074e-03_rb, &
     & 1.45026e-03_rb, 1.62950e-03_rb, 1.83089e-03_rb, 2.05718e-03_rb, 2.31143e-03_rb, &
     & 2.59710e-03_rb, 2.91808e-03_rb, 3.27873e-03_rb, 3.68395e-03_rb, 4.13926e-03_rb, &
     & 4.65083e-03_rb, 5.22564e-03_rb, 5.87148e-03_rb, 6.59715e-03_rb/)
      kao_mco2( 8, :, 4) = (/ &
     & 3.13265e-03_rb, 3.42454e-03_rb, 3.74362e-03_rb, 4.09243e-03_rb, 4.47375e-03_rb, &
     & 4.89059e-03_rb, 5.34627e-03_rb, 5.84441e-03_rb, 6.38897e-03_rb, 6.98426e-03_rb, &
     & 7.63502e-03_rb, 8.34642e-03_rb, 9.12409e-03_rb, 9.97423e-03_rb, 1.09036e-02_rb, &
     & 1.19195e-02_rb, 1.30301e-02_rb, 1.42442e-02_rb, 1.55714e-02_rb/)
      kao_mco2( 9, :, 4) = (/ &
     & 5.71287e-04_rb, 6.51252e-04_rb, 7.42411e-04_rb, 8.46330e-04_rb, 9.64794e-04_rb, &
     & 1.09984e-03_rb, 1.25379e-03_rb, 1.42929e-03_rb, 1.62935e-03_rb, 1.85742e-03_rb, &
     & 2.11741e-03_rb, 2.41380e-03_rb, 2.75167e-03_rb, 3.13683e-03_rb, 3.57591e-03_rb, &
     & 4.07645e-03_rb, 4.64705e-03_rb, 5.29751e-03_rb, 6.03903e-03_rb/)
      kao_mco2( 1, :, 5) = (/ &
     & 2.92395e-04_rb, 3.32719e-04_rb, 3.78604e-04_rb, 4.30818e-04_rb, 4.90232e-04_rb, &
     & 5.57839e-04_rb, 6.34771e-04_rb, 7.22312e-04_rb, 8.21927e-04_rb, 9.35278e-04_rb, &
     & 1.06426e-03_rb, 1.21104e-03_rb, 1.37805e-03_rb, 1.56810e-03_rb, 1.78435e-03_rb, &
     & 2.03043e-03_rb, 2.31045e-03_rb, 2.62908e-03_rb, 2.99166e-03_rb/)
      kao_mco2( 2, :, 5) = (/ &
     & 3.13069e-04_rb, 3.61343e-04_rb, 4.17061e-04_rb, 4.81371e-04_rb, 5.55597e-04_rb, &
     & 6.41269e-04_rb, 7.40151e-04_rb, 8.54280e-04_rb, 9.86008e-04_rb, 1.13805e-03_rb, &
     & 1.31353e-03_rb, 1.51608e-03_rb, 1.74985e-03_rb, 2.01967e-03_rb, 2.33110e-03_rb, &
     & 2.69055e-03_rb, 3.10543e-03_rb, 3.58427e-03_rb, 4.13696e-03_rb/)
      kao_mco2( 3, :, 5) = (/ &
     & 3.06937e-04_rb, 3.57841e-04_rb, 4.17187e-04_rb, 4.86375e-04_rb, 5.67038e-04_rb, &
     & 6.61078e-04_rb, 7.70714e-04_rb, 8.98532e-04_rb, 1.04755e-03_rb, 1.22128e-03_rb, &
     & 1.42382e-03_rb, 1.65996e-03_rb, 1.93525e-03_rb, 2.25620e-03_rb, 2.63038e-03_rb, &
     & 3.06661e-03_rb, 3.57519e-03_rb, 4.16812e-03_rb, 4.85937e-03_rb/)
      kao_mco2( 4, :, 5) = (/ &
     & 4.06428e-04_rb, 4.72379e-04_rb, 5.49033e-04_rb, 6.38125e-04_rb, 7.41674e-04_rb, &
     & 8.62026e-04_rb, 1.00191e-03_rb, 1.16449e-03_rb, 1.35345e-03_rb, 1.57308e-03_rb, &
     & 1.82834e-03_rb, 2.12503e-03_rb, 2.46986e-03_rb, 2.87064e-03_rb, 3.33647e-03_rb, &
     & 3.87788e-03_rb, 4.50715e-03_rb, 5.23852e-03_rb, 6.08858e-03_rb/)
      kao_mco2( 5, :, 5) = (/ &
     & 6.01967e-04_rb, 6.90414e-04_rb, 7.91856e-04_rb, 9.08204e-04_rb, 1.04165e-03_rb, &
     & 1.19470e-03_rb, 1.37023e-03_rb, 1.57156e-03_rb, 1.80247e-03_rb, 2.06731e-03_rb, &
     & 2.37106e-03_rb, 2.71944e-03_rb, 3.11901e-03_rb, 3.57729e-03_rb, 4.10290e-03_rb, &
     & 4.70574e-03_rb, 5.39716e-03_rb, 6.19017e-03_rb, 7.09969e-03_rb/)
      kao_mco2( 6, :, 5) = (/ &
     & 1.11622e-03_rb, 1.25799e-03_rb, 1.41776e-03_rb, 1.59783e-03_rb, 1.80077e-03_rb, &
     & 2.02947e-03_rb, 2.28723e-03_rb, 2.57773e-03_rb, 2.90512e-03_rb, 3.27408e-03_rb, &
     & 3.68992e-03_rb, 4.15856e-03_rb, 4.68673e-03_rb, 5.28197e-03_rb, 5.95282e-03_rb, &
     & 6.70887e-03_rb, 7.56094e-03_rb, 8.52123e-03_rb, 9.60348e-03_rb/)
      kao_mco2( 7, :, 5) = (/ &
     & 3.63860e-03_rb, 3.96164e-03_rb, 4.31337e-03_rb, 4.69632e-03_rb, 5.11327e-03_rb, &
     & 5.56724e-03_rb, 6.06151e-03_rb, 6.59967e-03_rb, 7.18561e-03_rb, 7.82356e-03_rb, &
     & 8.51816e-03_rb, 9.27443e-03_rb, 1.00978e-02_rb, 1.09943e-02_rb, 1.19705e-02_rb, &
     & 1.30332e-02_rb, 1.41904e-02_rb, 1.54502e-02_rb, 1.68219e-02_rb/)
      kao_mco2( 8, :, 5) = (/ &
     & 5.96957e-03_rb, 6.53049e-03_rb, 7.14412e-03_rb, 7.81541e-03_rb, 8.54977e-03_rb, &
     & 9.35314e-03_rb, 1.02320e-02_rb, 1.11934e-02_rb, 1.22452e-02_rb, 1.33958e-02_rb, &
     & 1.46545e-02_rb, 1.60315e-02_rb, 1.75379e-02_rb, 1.91858e-02_rb, 2.09886e-02_rb, &
     & 2.29608e-02_rb, 2.51182e-02_rb, 2.74784e-02_rb, 3.00604e-02_rb/)
      kao_mco2( 9, :, 5) = (/ &
     & 1.19381e-03_rb, 1.33882e-03_rb, 1.50143e-03_rb, 1.68379e-03_rb, 1.88831e-03_rb, &
     & 2.11767e-03_rb, 2.37488e-03_rb, 2.66333e-03_rb, 2.98683e-03_rb, 3.34961e-03_rb, &
     & 3.75646e-03_rb, 4.21272e-03_rb, 4.72440e-03_rb, 5.29823e-03_rb, 5.94176e-03_rb, &
     & 6.66345e-03_rb, 7.47280e-03_rb, 8.38045e-03_rb, 9.39835e-03_rb/)
      kao_mco2( 1, :, 6) = (/ &
     & 4.12429e-04_rb, 4.84830e-04_rb, 5.69942e-04_rb, 6.69995e-04_rb, 7.87613e-04_rb, &
     & 9.25878e-04_rb, 1.08842e-03_rb, 1.27949e-03_rb, 1.50410e-03_rb, 1.76814e-03_rb, &
     & 2.07854e-03_rb, 2.44343e-03_rb, 2.87237e-03_rb, 3.37662e-03_rb, 3.96938e-03_rb, &
     & 4.66621e-03_rb, 5.48536e-03_rb, 6.44831e-03_rb, 7.58031e-03_rb/)
      kao_mco2( 2, :, 6) = (/ &
     & 6.43498e-04_rb, 7.46132e-04_rb, 8.65134e-04_rb, 1.00312e-03_rb, 1.16311e-03_rb, &
     & 1.34861e-03_rb, 1.56371e-03_rb, 1.81310e-03_rb, 2.10228e-03_rb, 2.43758e-03_rb, &
     & 2.82635e-03_rb, 3.27714e-03_rb, 3.79981e-03_rb, 4.40586e-03_rb, 5.10855e-03_rb, &
     & 5.92333e-03_rb, 6.86806e-03_rb, 7.96346e-03_rb, 9.23357e-03_rb/)
      kao_mco2( 3, :, 6) = (/ &
     & 1.11336e-03_rb, 1.26910e-03_rb, 1.44662e-03_rb, 1.64897e-03_rb, 1.87962e-03_rb, &
     & 2.14254e-03_rb, 2.44224e-03_rb, 2.78385e-03_rb, 3.17325e-03_rb, 3.61712e-03_rb, &
     & 4.12308e-03_rb, 4.69981e-03_rb, 5.35720e-03_rb, 6.10656e-03_rb, 6.96073e-03_rb, &
     & 7.93439e-03_rb, 9.04424e-03_rb, 1.03093e-02_rb, 1.17514e-02_rb/)
      kao_mco2( 4, :, 6) = (/ &
     & 1.87991e-03_rb, 2.10276e-03_rb, 2.35202e-03_rb, 2.63082e-03_rb, 2.94268e-03_rb, &
     & 3.29150e-03_rb, 3.68168e-03_rb, 4.11810e-03_rb, 4.60626e-03_rb, 5.15228e-03_rb, &
     & 5.76303e-03_rb, 6.44617e-03_rb, 7.21030e-03_rb, 8.06500e-03_rb, 9.02102e-03_rb, &
     & 1.00904e-02_rb, 1.12865e-02_rb, 1.26244e-02_rb, 1.41208e-02_rb/)
      kao_mco2( 5, :, 6) = (/ &
     & 3.65848e-03_rb, 4.01372e-03_rb, 4.40346e-03_rb, 4.83104e-03_rb, 5.30015e-03_rb, &
     & 5.81480e-03_rb, 6.37943e-03_rb, 6.99888e-03_rb, 7.67849e-03_rb, 8.42408e-03_rb, &
     & 9.24208e-03_rb, 1.01395e-02_rb, 1.11241e-02_rb, 1.22042e-02_rb, 1.33893e-02_rb, &
     & 1.46894e-02_rb, 1.61158e-02_rb, 1.76806e-02_rb, 1.93975e-02_rb/)
      kao_mco2( 6, :, 6) = (/ &
     & 5.38476e-03_rb, 5.85088e-03_rb, 6.35735e-03_rb, 6.90765e-03_rb, 7.50560e-03_rb, &
     & 8.15530e-03_rb, 8.86124e-03_rb, 9.62829e-03_rb, 1.04617e-02_rb, 1.13673e-02_rb, &
     & 1.23513e-02_rb, 1.34205e-02_rb, 1.45822e-02_rb, 1.58445e-02_rb, 1.72160e-02_rb, &
     & 1.87062e-02_rb, 2.03255e-02_rb, 2.20849e-02_rb, 2.39966e-02_rb/)
      kao_mco2( 7, :, 6) = (/ &
     & 6.27017e-03_rb, 6.84772e-03_rb, 7.47846e-03_rb, 8.16731e-03_rb, 8.91960e-03_rb, &
     & 9.74118e-03_rb, 1.06384e-02_rb, 1.16183e-02_rb, 1.26885e-02_rb, 1.38573e-02_rb, &
     & 1.51336e-02_rb, 1.65276e-02_rb, 1.80500e-02_rb, 1.97125e-02_rb, 2.15283e-02_rb, &
     & 2.35112e-02_rb, 2.56769e-02_rb, 2.80420e-02_rb, 3.06249e-02_rb/)
      kao_mco2( 8, :, 6) = (/ &
     & 9.61932e-03_rb, 1.04802e-02_rb, 1.14182e-02_rb, 1.24401e-02_rb, 1.35534e-02_rb, &
     & 1.47664e-02_rb, 1.60880e-02_rb, 1.75278e-02_rb, 1.90965e-02_rb, 2.08056e-02_rb, &
     & 2.26677e-02_rb, 2.46964e-02_rb, 2.69066e-02_rb, 2.93147e-02_rb, 3.19383e-02_rb, &
     & 3.47967e-02_rb, 3.79110e-02_rb, 4.13039e-02_rb, 4.50005e-02_rb/)
      kao_mco2( 9, :, 6) = (/ &
     & 2.37921e-03_rb, 2.64556e-03_rb, 2.94173e-03_rb, 3.27105e-03_rb, 3.63724e-03_rb, &
     & 4.04442e-03_rb, 4.49718e-03_rb, 5.00064e-03_rb, 5.56045e-03_rb, 6.18293e-03_rb, &
     & 6.87510e-03_rb, 7.64475e-03_rb, 8.50057e-03_rb, 9.45219e-03_rb, 1.05103e-02_rb, &
     & 1.16870e-02_rb, 1.29953e-02_rb, 1.44501e-02_rb, 1.60677e-02_rb/)
      kao_mco2( 1, :, 7) = (/ &
     & 4.64970e-03_rb, 5.13188e-03_rb, 5.66406e-03_rb, 6.25144e-03_rb, 6.89972e-03_rb, &
     & 7.61523e-03_rb, 8.40493e-03_rb, 9.27654e-03_rb, 1.02385e-02_rb, 1.13003e-02_rb, &
     & 1.24721e-02_rb, 1.37655e-02_rb, 1.51930e-02_rb, 1.67685e-02_rb, 1.85075e-02_rb, &
     & 2.04267e-02_rb, 2.25450e-02_rb, 2.48829e-02_rb, 2.74633e-02_rb/)
      kao_mco2( 2, :, 7) = (/ &
     & 6.37148e-03_rb, 6.96805e-03_rb, 7.62046e-03_rb, 8.33397e-03_rb, 9.11428e-03_rb, &
     & 9.96765e-03_rb, 1.09009e-02_rb, 1.19216e-02_rb, 1.30378e-02_rb, 1.42585e-02_rb, &
     & 1.55935e-02_rb, 1.70536e-02_rb, 1.86503e-02_rb, 2.03965e-02_rb, 2.23062e-02_rb, &
     & 2.43948e-02_rb, 2.66789e-02_rb, 2.91768e-02_rb, 3.19086e-02_rb/)
      kao_mco2( 3, :, 7) = (/ &
     & 7.79364e-03_rb, 8.48097e-03_rb, 9.22892e-03_rb, 1.00428e-02_rb, 1.09285e-02_rb, &
     & 1.18923e-02_rb, 1.29411e-02_rb, 1.40825e-02_rb, 1.53244e-02_rb, 1.66759e-02_rb, &
     & 1.81466e-02_rb, 1.97470e-02_rb, 2.14885e-02_rb, 2.33836e-02_rb, 2.54458e-02_rb, &
     & 2.76899e-02_rb, 3.01320e-02_rb, 3.27893e-02_rb, 3.56811e-02_rb/)
      kao_mco2( 4, :, 7) = (/ &
     & 8.70586e-03_rb, 9.48737e-03_rb, 1.03390e-02_rb, 1.12672e-02_rb, 1.22786e-02_rb, &
     & 1.33808e-02_rb, 1.45820e-02_rb, 1.58910e-02_rb, 1.73175e-02_rb, 1.88721e-02_rb, &
     & 2.05662e-02_rb, 2.24124e-02_rb, 2.44243e-02_rb, 2.66169e-02_rb, 2.90062e-02_rb, &
     & 3.16101e-02_rb, 3.44477e-02_rb, 3.75400e-02_rb, 4.09099e-02_rb/)
      kao_mco2( 5, :, 7) = (/ &
     & 9.24510e-03_rb, 1.00865e-02_rb, 1.10045e-02_rb, 1.20061e-02_rb, 1.30988e-02_rb, &
     & 1.42910e-02_rb, 1.55916e-02_rb, 1.70106e-02_rb, 1.85588e-02_rb, 2.02479e-02_rb, &
     & 2.20908e-02_rb, 2.41013e-02_rb, 2.62948e-02_rb, 2.86880e-02_rb, 3.12990e-02_rb, &
     & 3.41476e-02_rb, 3.72555e-02_rb, 4.06462e-02_rb, 4.43455e-02_rb/)
      kao_mco2( 6, :, 7) = (/ &
     & 1.09559e-02_rb, 1.19933e-02_rb, 1.31290e-02_rb, 1.43722e-02_rb, 1.57331e-02_rb, &
     & 1.72229e-02_rb, 1.88537e-02_rb, 2.06390e-02_rb, 2.25933e-02_rb, 2.47327e-02_rb, &
     & 2.70747e-02_rb, 2.96384e-02_rb, 3.24449e-02_rb, 3.55171e-02_rb, 3.88802e-02_rb, &
     & 4.25619e-02_rb, 4.65921e-02_rb, 5.10039e-02_rb, 5.58335e-02_rb/)
      kao_mco2( 7, :, 7) = (/ &
     & 1.36116e-02_rb, 1.48659e-02_rb, 1.62357e-02_rb, 1.77318e-02_rb, 1.93657e-02_rb, &
     & 2.11502e-02_rb, 2.30991e-02_rb, 2.52276e-02_rb, 2.75522e-02_rb, 3.00910e-02_rb, &
     & 3.28638e-02_rb, 3.58921e-02_rb, 3.91995e-02_rb, 4.28116e-02_rb, 4.67565e-02_rb, &
     & 5.10650e-02_rb, 5.57704e-02_rb, 6.09095e-02_rb, 6.65221e-02_rb/)
      kao_mco2( 8, :, 7) = (/ &
     & 1.51783e-02_rb, 1.64551e-02_rb, 1.78392e-02_rb, 1.93399e-02_rb, 2.09667e-02_rb, &
     & 2.27304e-02_rb, 2.46424e-02_rb, 2.67153e-02_rb, 2.89626e-02_rb, 3.13988e-02_rb, &
     & 3.40401e-02_rb, 3.69035e-02_rb, 4.00077e-02_rb, 4.33731e-02_rb, 4.70216e-02_rb, &
     & 5.09770e-02_rb, 5.52651e-02_rb, 5.99139e-02_rb, 6.49538e-02_rb/)
      kao_mco2( 9, :, 7) = (/ &
     & 1.00072e-02_rb, 1.08638e-02_rb, 1.17937e-02_rb, 1.28032e-02_rb, 1.38991e-02_rb, &
     & 1.50888e-02_rb, 1.63803e-02_rb, 1.77824e-02_rb, 1.93045e-02_rb, 2.09568e-02_rb, &
     & 2.27507e-02_rb, 2.46980e-02_rb, 2.68120e-02_rb, 2.91070e-02_rb, 3.15984e-02_rb, &
     & 3.43031e-02_rb, 3.72393e-02_rb, 4.04268e-02_rb, 4.38872e-02_rb/)
      kao_mco2( 1, :, 8) = (/ &
     & 1.59610e-02_rb, 1.74387e-02_rb, 1.90532e-02_rb, 2.08171e-02_rb, 2.27444e-02_rb, &
     & 2.48501e-02_rb, 2.71508e-02_rb, 2.96645e-02_rb, 3.24109e-02_rb, 3.54115e-02_rb, &
     & 3.86900e-02_rb, 4.22720e-02_rb, 4.61856e-02_rb, 5.04616e-02_rb, 5.51334e-02_rb, &
     & 6.02378e-02_rb, 6.58147e-02_rb, 7.19079e-02_rb, 7.85653e-02_rb/)
      kao_mco2( 2, :, 8) = (/ &
     & 1.61961e-02_rb, 1.76986e-02_rb, 1.93405e-02_rb, 2.11348e-02_rb, 2.30955e-02_rb, &
     & 2.52381e-02_rb, 2.75794e-02_rb, 3.01380e-02_rb, 3.29340e-02_rb, 3.59893e-02_rb, &
     & 3.93280e-02_rb, 4.29766e-02_rb, 4.69636e-02_rb, 5.13204e-02_rb, 5.60815e-02_rb, &
     & 6.12843e-02_rb, 6.69697e-02_rb, 7.31826e-02_rb, 7.99718e-02_rb/)
      kao_mco2( 3, :, 8) = (/ &
     & 1.72034e-02_rb, 1.88241e-02_rb, 2.05974e-02_rb, 2.25377e-02_rb, 2.46609e-02_rb, &
     & 2.69841e-02_rb, 2.95261e-02_rb, 3.23076e-02_rb, 3.53511e-02_rb, 3.86813e-02_rb, &
     & 4.23253e-02_rb, 4.63126e-02_rb, 5.06754e-02_rb, 5.54493e-02_rb, 6.06728e-02_rb, &
     & 6.63885e-02_rb, 7.26426e-02_rb, 7.94859e-02_rb, 8.69738e-02_rb/)
      kao_mco2( 4, :, 8) = (/ &
     & 1.79777e-02_rb, 1.96517e-02_rb, 2.14815e-02_rb, 2.34817e-02_rb, 2.56682e-02_rb, &
     & 2.80583e-02_rb, 3.06709e-02_rb, 3.35268e-02_rb, 3.66486e-02_rb, 4.00611e-02_rb, &
     & 4.37914e-02_rb, 4.78690e-02_rb, 5.23262e-02_rb, 5.71985e-02_rb, 6.25245e-02_rb, &
     & 6.83464e-02_rb, 7.47104e-02_rb, 8.16670e-02_rb, 8.92713e-02_rb/)
      kao_mco2( 5, :, 8) = (/ &
     & 2.02540e-02_rb, 2.21214e-02_rb, 2.41610e-02_rb, 2.63887e-02_rb, 2.88218e-02_rb, &
     & 3.14792e-02_rb, 3.43816e-02_rb, 3.75516e-02_rb, 4.10139e-02_rb, 4.47954e-02_rb, &
     & 4.89256e-02_rb, 5.34366e-02_rb, 5.83635e-02_rb, 6.37447e-02_rb, 6.96220e-02_rb, &
     & 7.60413e-02_rb, 8.30523e-02_rb, 9.07098e-02_rb, 9.90734e-02_rb/)
      kao_mco2( 6, :, 8) = (/ &
     & 2.19009e-02_rb, 2.38517e-02_rb, 2.59762e-02_rb, 2.82899e-02_rb, 3.08097e-02_rb, &
     & 3.35540e-02_rb, 3.65427e-02_rb, 3.97976e-02_rb, 4.33424e-02_rb, 4.72030e-02_rb, &
     & 5.14074e-02_rb, 5.59863e-02_rb, 6.09731e-02_rb, 6.64040e-02_rb, 7.23187e-02_rb, &
     & 7.87603e-02_rb, 8.57755e-02_rb, 9.34157e-02_rb, 1.01736e-01_rb/)
      kao_mco2( 7, :, 8) = (/ &
     & 2.52383e-02_rb, 2.73978e-02_rb, 2.97421e-02_rb, 3.22869e-02_rb, 3.50496e-02_rb, &
     & 3.80486e-02_rb, 4.13042e-02_rb, 4.48383e-02_rb, 4.86749e-02_rb, 5.28397e-02_rb, &
     & 5.73610e-02_rb, 6.22690e-02_rb, 6.75970e-02_rb, 7.33810e-02_rb, 7.96598e-02_rb, &
     & 8.64758e-02_rb, 9.38751e-02_rb, 1.01907e-01_rb, 1.10627e-01_rb/)
      kao_mco2( 8, :, 8) = (/ &
     & 3.36506e-02_rb, 3.59288e-02_rb, 3.83613e-02_rb, 4.09584e-02_rb, 4.37313e-02_rb, &
     & 4.66920e-02_rb, 4.98531e-02_rb, 5.32283e-02_rb, 5.68319e-02_rb, 6.06795e-02_rb, &
     & 6.47876e-02_rb, 6.91739e-02_rb, 7.38570e-02_rb, 7.88573e-02_rb, 8.41960e-02_rb, &
     & 8.98962e-02_rb, 9.59824e-02_rb, 1.02481e-01_rb, 1.09419e-01_rb/)
      kao_mco2( 9, :, 8) = (/ &
     & 2.15151e-02_rb, 2.34420e-02_rb, 2.55415e-02_rb, 2.78291e-02_rb, 3.03215e-02_rb, &
     & 3.30372e-02_rb, 3.59961e-02_rb, 3.92200e-02_rb, 4.27326e-02_rb, 4.65598e-02_rb, &
     & 5.07299e-02_rb, 5.52734e-02_rb, 6.02238e-02_rb, 6.56176e-02_rb, 7.14944e-02_rb, &
     & 7.78977e-02_rb, 8.48744e-02_rb, 9.24759e-02_rb, 1.00758e-01_rb/)
      kao_mco2( 1, :, 9) = (/ &
     & 3.34296e-02_rb, 3.64437e-02_rb, 3.97294e-02_rb, 4.33114e-02_rb, 4.72164e-02_rb, &
     & 5.14734e-02_rb, 5.61143e-02_rb, 6.11735e-02_rb, 6.66890e-02_rb, 7.27016e-02_rb, &
     & 7.92564e-02_rb, 8.64022e-02_rb, 9.41922e-02_rb, 1.02685e-01_rb, 1.11943e-01_rb, &
     & 1.22035e-01_rb, 1.33038e-01_rb, 1.45033e-01_rb, 1.58109e-01_rb/)
      kao_mco2( 2, :, 9) = (/ &
     & 3.73946e-02_rb, 4.07543e-02_rb, 4.44160e-02_rb, 4.84066e-02_rb, 5.27558e-02_rb, &
     & 5.74958e-02_rb, 6.26616e-02_rb, 6.82915e-02_rb, 7.44273e-02_rb, 8.11144e-02_rb, &
     & 8.84023e-02_rb, 9.63449e-02_rb, 1.05001e-01_rb, 1.14435e-01_rb, 1.24717e-01_rb, &
     & 1.35922e-01_rb, 1.48135e-01_rb, 1.61444e-01_rb, 1.75949e-01_rb/)
      kao_mco2( 3, :, 9) = (/ &
     & 4.24539e-02_rb, 4.61192e-02_rb, 5.01010e-02_rb, 5.44265e-02_rb, 5.91255e-02_rb, &
     & 6.42302e-02_rb, 6.97756e-02_rb, 7.57998e-02_rb, 8.23442e-02_rb, 8.94535e-02_rb, &
     & 9.71766e-02_rb, 1.05566e-01_rb, 1.14681e-01_rb, 1.24582e-01_rb, 1.35338e-01_rb, &
     & 1.47022e-01_rb, 1.59716e-01_rb, 1.73505e-01_rb, 1.88485e-01_rb/)
      kao_mco2( 4, :, 9) = (/ &
     & 5.30296e-02_rb, 5.73416e-02_rb, 6.20043e-02_rb, 6.70462e-02_rb, 7.24980e-02_rb, &
     & 7.83931e-02_rb, 8.47676e-02_rb, 9.16604e-02_rb, 9.91137e-02_rb, 1.07173e-01_rb, &
     & 1.15888e-01_rb, 1.25311e-01_rb, 1.35501e-01_rb, 1.46519e-01_rb, 1.58433e-01_rb, &
     & 1.71316e-01_rb, 1.85246e-01_rb, 2.00309e-01_rb, 2.16597e-01_rb/)
      kao_mco2( 5, :, 9) = (/ &
     & 6.26111e-02_rb, 6.74018e-02_rb, 7.25591e-02_rb, 7.81111e-02_rb, 8.40878e-02_rb, &
     & 9.05218e-02_rb, 9.74482e-02_rb, 1.04904e-01_rb, 1.12931e-01_rb, 1.21572e-01_rb, &
     & 1.30875e-01_rb, 1.40889e-01_rb, 1.51669e-01_rb, 1.63274e-01_rb, 1.75767e-01_rb, &
     & 1.89216e-01_rb, 2.03694e-01_rb, 2.19279e-01_rb, 2.36058e-01_rb/)
      kao_mco2( 6, :, 9) = (/ &
     & 7.59080e-02_rb, 8.13446e-02_rb, 8.71706e-02_rb, 9.34139e-02_rb, 1.00104e-01_rb, &
     & 1.07274e-01_rb, 1.14957e-01_rb, 1.23190e-01_rb, 1.32013e-01_rb, 1.41468e-01_rb, &
     & 1.51600e-01_rb, 1.62458e-01_rb, 1.74094e-01_rb, 1.86562e-01_rb, 1.99924e-01_rb, &
     & 2.14243e-01_rb, 2.29587e-01_rb, 2.46031e-01_rb, 2.63652e-01_rb/)
      kao_mco2( 7, :, 9) = (/ &
     & 8.81942e-02_rb, 9.39942e-02_rb, 1.00176e-01_rb, 1.06763e-01_rb, 1.13784e-01_rb, &
     & 1.21267e-01_rb, 1.29242e-01_rb, 1.37742e-01_rb, 1.46800e-01_rb, 1.56454e-01_rb, &
     & 1.66743e-01_rb, 1.77708e-01_rb, 1.89395e-01_rb, 2.01850e-01_rb, 2.15124e-01_rb, &
     & 2.29272e-01_rb, 2.44349e-01_rb, 2.60418e-01_rb, 2.77544e-01_rb/)
      kao_mco2( 8, :, 9) = (/ &
     & 6.28535e-02_rb, 6.69314e-02_rb, 7.12740e-02_rb, 7.58982e-02_rb, 8.08225e-02_rb, &
     & 8.60662e-02_rb, 9.16502e-02_rb, 9.75965e-02_rb, 1.03929e-01_rb, 1.10671e-01_rb, &
     & 1.17852e-01_rb, 1.25498e-01_rb, 1.33640e-01_rb, 1.42311e-01_rb, 1.51544e-01_rb, &
     & 1.61376e-01_rb, 1.71846e-01_rb, 1.82996e-01_rb, 1.94868e-01_rb/)
      kao_mco2( 9, :, 9) = (/ &
     & 6.39196e-02_rb, 6.86702e-02_rb, 7.37738e-02_rb, 7.92568e-02_rb, 8.51473e-02_rb, &
     & 9.14756e-02_rb, 9.82742e-02_rb, 1.05578e-01_rb, 1.13425e-01_rb, 1.21855e-01_rb, &
     & 1.30911e-01_rb, 1.40641e-01_rb, 1.51093e-01_rb, 1.62323e-01_rb, 1.74387e-01_rb, &
     & 1.87348e-01_rb, 2.01272e-01_rb, 2.16231e-01_rb, 2.32301e-01_rb/)
      kao_mco2( 1, :,10) = (/ &
     & 9.44086e-02_rb, 1.02788e-01_rb, 1.11911e-01_rb, 1.21844e-01_rb, 1.32659e-01_rb, &
     & 1.44434e-01_rb, 1.57253e-01_rb, 1.71211e-01_rb, 1.86407e-01_rb, 2.02952e-01_rb, &
     & 2.20966e-01_rb, 2.40578e-01_rb, 2.61932e-01_rb, 2.85180e-01_rb, 3.10492e-01_rb, &
     & 3.38051e-01_rb, 3.68056e-01_rb, 4.00723e-01_rb, 4.36291e-01_rb/)
      kao_mco2( 2, :,10) = (/ &
     & 1.29528e-01_rb, 1.39646e-01_rb, 1.50554e-01_rb, 1.62315e-01_rb, 1.74994e-01_rb, &
     & 1.88664e-01_rb, 2.03401e-01_rb, 2.19290e-01_rb, 2.36419e-01_rb, 2.54887e-01_rb, &
     & 2.74798e-01_rb, 2.96263e-01_rb, 3.19406e-01_rb, 3.44356e-01_rb, 3.71255e-01_rb, &
     & 4.00256e-01_rb, 4.31522e-01_rb, 4.65230e-01_rb, 5.01571e-01_rb/)
      kao_mco2( 3, :,10) = (/ &
     & 1.52325e-01_rb, 1.62991e-01_rb, 1.74404e-01_rb, 1.86616e-01_rb, 1.99684e-01_rb, &
     & 2.13666e-01_rb, 2.28628e-01_rb, 2.44637e-01_rb, 2.61767e-01_rb, 2.80096e-01_rb, &
     & 2.99710e-01_rb, 3.20696e-01_rb, 3.43152e-01_rb, 3.67181e-01_rb, 3.92892e-01_rb, &
     & 4.20403e-01_rb, 4.49841e-01_rb, 4.81340e-01_rb, 5.15045e-01_rb/)
      kao_mco2( 4, :,10) = (/ &
     & 1.59763e-01_rb, 1.70378e-01_rb, 1.81698e-01_rb, 1.93770e-01_rb, 2.06644e-01_rb, &
     & 2.20373e-01_rb, 2.35015e-01_rb, 2.50629e-01_rb, 2.67281e-01_rb, 2.85039e-01_rb, &
     & 3.03977e-01_rb, 3.24174e-01_rb, 3.45712e-01_rb, 3.68681e-01_rb, 3.93176e-01_rb, &
     & 4.19299e-01_rb, 4.47157e-01_rb, 4.76866e-01_rb, 5.08549e-01_rb/)
      kao_mco2( 5, :,10) = (/ &
     & 1.79202e-01_rb, 1.91125e-01_rb, 2.03840e-01_rb, 2.17402e-01_rb, 2.31866e-01_rb, &
     & 2.47292e-01_rb, 2.63744e-01_rb, 2.81291e-01_rb, 3.00005e-01_rb, 3.19964e-01_rb, &
     & 3.41251e-01_rb, 3.63955e-01_rb, 3.88169e-01_rb, 4.13994e-01_rb, 4.41537e-01_rb, &
     & 4.70912e-01_rb, 5.02242e-01_rb, 5.35656e-01_rb, 5.71293e-01_rb/)
      kao_mco2( 6, :,10) = (/ &
     & 1.66628e-01_rb, 1.76984e-01_rb, 1.87984e-01_rb, 1.99668e-01_rb, 2.12078e-01_rb, &
     & 2.25259e-01_rb, 2.39259e-01_rb, 2.54129e-01_rb, 2.69924e-01_rb, 2.86700e-01_rb, &
     & 3.04519e-01_rb, 3.23446e-01_rb, 3.43549e-01_rb, 3.64901e-01_rb, 3.87580e-01_rb, &
     & 4.11669e-01_rb, 4.37255e-01_rb, 4.64431e-01_rb, 4.93297e-01_rb/)
      kao_mco2( 7, :,10) = (/ &
     & 2.03980e-01_rb, 2.17141e-01_rb, 2.31152e-01_rb, 2.46067e-01_rb, 2.61945e-01_rb, &
     & 2.78847e-01_rb, 2.96839e-01_rb, 3.15993e-01_rb, 3.36382e-01_rb, 3.58087e-01_rb, &
     & 3.81193e-01_rb, 4.05789e-01_rb, 4.31972e-01_rb, 4.59845e-01_rb, 4.89517e-01_rb, &
     & 5.21103e-01_rb, 5.54727e-01_rb, 5.90520e-01_rb, 6.28623e-01_rb/)
      kao_mco2( 8, :,10) = (/ &
     & 1.96161e-04_rb, 2.07177e-04_rb, 2.18812e-04_rb, 2.31101e-04_rb, 2.44079e-04_rb, &
     & 2.57787e-04_rb, 2.72264e-04_rb, 2.87554e-04_rb, 3.03703e-04_rb, 3.20758e-04_rb, &
     & 3.38772e-04_rb, 3.57797e-04_rb, 3.77891e-04_rb, 3.99113e-04_rb, 4.21527e-04_rb, &
     & 4.45200e-04_rb, 4.70202e-04_rb, 4.96608e-04_rb, 5.24498e-04_rb/)
      kao_mco2( 9, :,10) = (/ &
     & 1.76275e-01_rb, 1.88091e-01_rb, 2.00699e-01_rb, 2.14152e-01_rb, 2.28507e-01_rb, &
     & 2.43824e-01_rb, 2.60168e-01_rb, 2.77607e-01_rb, 2.96216e-01_rb, 3.16071e-01_rb, &
     & 3.37258e-01_rb, 3.59865e-01_rb, 3.83987e-01_rb, 4.09726e-01_rb, 4.37190e-01_rb, &
     & 4.66495e-01_rb, 4.97765e-01_rb, 5.31131e-01_rb, 5.66733e-01_rb/)
      kao_mco2( 1, :,11) = (/ &
     & 1.99797e-01_rb, 2.14154e-01_rb, 2.29543e-01_rb, 2.46038e-01_rb, 2.63718e-01_rb, &
     & 2.82669e-01_rb, 3.02981e-01_rb, 3.24753e-01_rb, 3.48090e-01_rb, 3.73104e-01_rb, &
     & 3.99915e-01_rb, 4.28652e-01_rb, 4.59455e-01_rb, 4.92471e-01_rb, 5.27859e-01_rb, &
     & 5.65791e-01_rb, 6.06448e-01_rb, 6.50027e-01_rb, 6.96738e-01_rb/)
      kao_mco2( 2, :,11) = (/ &
     & 2.20638e-01_rb, 2.35685e-01_rb, 2.51759e-01_rb, 2.68929e-01_rb, 2.87271e-01_rb, &
     & 3.06863e-01_rb, 3.27791e-01_rb, 3.50146e-01_rb, 3.74026e-01_rb, 3.99535e-01_rb, &
     & 4.26784e-01_rb, 4.55891e-01_rb, 4.86983e-01_rb, 5.20195e-01_rb, 5.55673e-01_rb, &
     & 5.93570e-01_rb, 6.34052e-01_rb, 6.77294e-01_rb, 7.23486e-01_rb/)
      kao_mco2( 3, :,11) = (/ &
     & 2.62988e-01_rb, 2.80924e-01_rb, 3.00085e-01_rb, 3.20552e-01_rb, 3.42414e-01_rb, &
     & 3.65768e-01_rb, 3.90715e-01_rb, 4.17363e-01_rb, 4.45829e-01_rb, 4.76237e-01_rb, &
     & 5.08718e-01_rb, 5.43414e-01_rb, 5.80477e-01_rb, 6.20068e-01_rb, 6.62359e-01_rb, &
     & 7.07535e-01_rb, 7.55791e-01_rb, 8.07339e-01_rb, 8.62403e-01_rb/)
      kao_mco2( 4, :,11) = (/ &
     & 2.43674e-01_rb, 2.59946e-01_rb, 2.77304e-01_rb, 2.95821e-01_rb, 3.15575e-01_rb, &
     & 3.36647e-01_rb, 3.59127e-01_rb, 3.83108e-01_rb, 4.08691e-01_rb, 4.35981e-01_rb, &
     & 4.65094e-01_rb, 4.96152e-01_rb, 5.29282e-01_rb, 5.64626e-01_rb, 6.02329e-01_rb, &
     & 6.42550e-01_rb, 6.85457e-01_rb, 7.31229e-01_rb, 7.80057e-01_rb/)
      kao_mco2( 5, :,11) = (/ &
     & 2.23323e-01_rb, 2.37553e-01_rb, 2.52689e-01_rb, 2.68791e-01_rb, 2.85918e-01_rb, &
     & 3.04136e-01_rb, 3.23515e-01_rb, 3.44129e-01_rb, 3.66057e-01_rb, 3.89381e-01_rb, &
     & 4.14192e-01_rb, 4.40584e-01_rb, 4.68657e-01_rb, 4.98520e-01_rb, 5.30285e-01_rb, &
     & 5.64074e-01_rb, 6.00016e-01_rb, 6.38248e-01_rb, 6.78917e-01_rb/)
      kao_mco2( 6, :,11) = (/ &
     & 2.83716e-01_rb, 3.02622e-01_rb, 3.22788e-01_rb, 3.44298e-01_rb, 3.67241e-01_rb, &
     & 3.91713e-01_rb, 4.17816e-01_rb, 4.45658e-01_rb, 4.75356e-01_rb, 5.07033e-01_rb, &
     & 5.40820e-01_rb, 5.76859e-01_rb, 6.15300e-01_rb, 6.56302e-01_rb, 7.00037e-01_rb, &
     & 7.46686e-01_rb, 7.96443e-01_rb, 8.49516e-01_rb, 9.06126e-01_rb/)
      kao_mco2( 7, :,11) = (/ &
     & 1.00497e-03_rb, 1.06500e-03_rb, 1.12863e-03_rb, 1.19606e-03_rb, 1.26751e-03_rb, &
     & 1.34323e-03_rb, 1.42348e-03_rb, 1.50852e-03_rb, 1.59864e-03_rb, 1.69414e-03_rb, &
     & 1.79535e-03_rb, 1.90261e-03_rb, 2.01628e-03_rb, 2.13673e-03_rb, 2.26438e-03_rb, &
     & 2.39966e-03_rb, 2.54302e-03_rb, 2.69494e-03_rb, 2.85594e-03_rb/)
      kao_mco2( 8, :,11) = (/ &
     & 3.22623e-04_rb, 3.39937e-04_rb, 3.58181e-04_rb, 3.77404e-04_rb, 3.97658e-04_rb, &
     & 4.19000e-04_rb, 4.41487e-04_rb, 4.65180e-04_rb, 4.90146e-04_rb, 5.16451e-04_rb, &
     & 5.44167e-04_rb, 5.73372e-04_rb, 6.04143e-04_rb, 6.36567e-04_rb, 6.70730e-04_rb, &
     & 7.06726e-04_rb, 7.44655e-04_rb, 7.84619e-04_rb, 8.26727e-04_rb/)
      kao_mco2( 9, :,11) = (/ &
     & 2.23872e-01_rb, 2.38360e-01_rb, 2.53786e-01_rb, 2.70210e-01_rb, 2.87697e-01_rb, &
     & 3.06316e-01_rb, 3.26140e-01_rb, 3.47247e-01_rb, 3.69720e-01_rb, 3.93647e-01_rb, &
     & 4.19122e-01_rb, 4.46246e-01_rb, 4.75126e-01_rb, 5.05874e-01_rb, 5.38613e-01_rb, &
     & 5.73470e-01_rb, 6.10583e-01_rb, 6.50098e-01_rb, 6.92170e-01_rb/)
      kao_mco2( 1, :,12) = (/ &
     & 3.52418e-01_rb, 3.76085e-01_rb, 4.01341e-01_rb, 4.28293e-01_rb, 4.57055e-01_rb, &
     & 4.87749e-01_rb, 5.20504e-01_rb, 5.55458e-01_rb, 5.92760e-01_rb, 6.32567e-01_rb, &
     & 6.75047e-01_rb, 7.20380e-01_rb, 7.68757e-01_rb, 8.20383e-01_rb, 8.75476e-01_rb, &
     & 9.34268e-01_rb, 9.97009e-01_rb, 1.06396e+00_rb, 1.13541e+00_rb/)
      kao_mco2( 2, :,12) = (/ &
     & 3.38812e-01_rb, 3.61001e-01_rb, 3.84645e-01_rb, 4.09836e-01_rb, 4.36678e-01_rb, &
     & 4.65278e-01_rb, 4.95750e-01_rb, 5.28219e-01_rb, 5.62814e-01_rb, 5.99674e-01_rb, &
     & 6.38949e-01_rb, 6.80796e-01_rb, 7.25384e-01_rb, 7.72892e-01_rb, 8.23511e-01_rb, &
     & 8.77446e-01_rb, 9.34913e-01_rb, 9.96144e-01_rb, 1.06138e+00_rb/)
      kao_mco2( 3, :,12) = (/ &
     & 3.44644e-01_rb, 3.66671e-01_rb, 3.90105e-01_rb, 4.15038e-01_rb, 4.41564e-01_rb, &
     & 4.69785e-01_rb, 4.99810e-01_rb, 5.31754e-01_rb, 5.65740e-01_rb, 6.01897e-01_rb, &
     & 6.40366e-01_rb, 6.81293e-01_rb, 7.24836e-01_rb, 7.71162e-01_rb, 8.20448e-01_rb, &
     & 8.72885e-01_rb, 9.28673e-01_rb, 9.88027e-01_rb, 1.05117e+00_rb/)
      kao_mco2( 4, :,12) = (/ &
     & 4.20358e-01_rb, 4.47809e-01_rb, 4.77053e-01_rb, 5.08207e-01_rb, 5.41395e-01_rb, &
     & 5.76750e-01_rb, 6.14414e-01_rb, 6.54538e-01_rb, 6.97282e-01_rb, 7.42818e-01_rb, &
     & 7.91327e-01_rb, 8.43004e-01_rb, 8.98056e-01_rb, 9.56703e-01_rb, 1.01918e+00_rb, &
     & 1.08574e+00_rb, 1.15664e+00_rb, 1.23217e+00_rb, 1.31264e+00_rb/)
      kao_mco2( 5, :,12) = (/ &
     & 4.42756e-01_rb, 4.72000e-01_rb, 5.03174e-01_rb, 5.36408e-01_rb, 5.71837e-01_rb, &
     & 6.09606e-01_rb, 6.49870e-01_rb, 6.92793e-01_rb, 7.38551e-01_rb, 7.87331e-01_rb, &
     & 8.39333e-01_rb, 8.94770e-01_rb, 9.53868e-01_rb, 1.01687e+00_rb, 1.08403e+00_rb, &
     & 1.15563e+00_rb, 1.23196e+00_rb, 1.31333e+00_rb, 1.40007e+00_rb/)
      kao_mco2( 6, :,12) = (/ &
     & 1.53662e-01_rb, 1.63104e-01_rb, 1.73126e-01_rb, 1.83764e-01_rb, 1.95055e-01_rb, &
     & 2.07040e-01_rb, 2.19762e-01_rb, 2.33265e-01_rb, 2.47598e-01_rb, 2.62811e-01_rb, &
     & 2.78960e-01_rb, 2.96100e-01_rb, 3.14294e-01_rb, 3.33606e-01_rb, 3.54104e-01_rb, &
     & 3.75862e-01_rb, 3.98956e-01_rb, 4.23470e-01_rb, 4.49490e-01_rb/)
      kao_mco2( 7, :,12) = (/ &
     & 5.41472e-04_rb, 5.65116e-04_rb, 5.89793e-04_rb, 6.15547e-04_rb, 6.42426e-04_rb, &
     & 6.70479e-04_rb, 6.99757e-04_rb, 7.30313e-04_rb, 7.62203e-04_rb, 7.95486e-04_rb, &
     & 8.30223e-04_rb, 8.66476e-04_rb, 9.04312e-04_rb, 9.43801e-04_rb, 9.85013e-04_rb, &
     & 1.02803e-03_rb, 1.07292e-03_rb, 1.11977e-03_rb, 1.16866e-03_rb/)
      kao_mco2( 8, :,12) = (/ &
     & 5.94251e-04_rb, 6.17650e-04_rb, 6.41969e-04_rb, 6.67246e-04_rb, 6.93518e-04_rb, &
     & 7.20824e-04_rb, 7.49206e-04_rb, 7.78705e-04_rb, 8.09366e-04_rb, 8.41234e-04_rb, &
     & 8.74356e-04_rb, 9.08783e-04_rb, 9.44566e-04_rb, 9.81757e-04_rb, 1.02041e-03_rb, &
     & 1.06059e-03_rb, 1.10235e-03_rb, 1.14575e-03_rb, 1.19087e-03_rb/)
      kao_mco2( 9, :,12) = (/ &
     & 4.21683e-01_rb, 4.49025e-01_rb, 4.78140e-01_rb, 5.09142e-01_rb, 5.42155e-01_rb, &
     & 5.77309e-01_rb, 6.14741e-01_rb, 6.54601e-01_rb, 6.97046e-01_rb, 7.42242e-01_rb, &
     & 7.90369e-01_rb, 8.41617e-01_rb, 8.96188e-01_rb, 9.54297e-01_rb, 1.01617e+00_rb, &
     & 1.08206e+00_rb, 1.15222e+00_rb, 1.22693e+00_rb, 1.30649e+00_rb/)
      kao_mco2( 1, :,13) = (/ &
     & 5.61805e-01_rb, 5.98988e-01_rb, 6.38631e-01_rb, 6.80898e-01_rb, 7.25962e-01_rb, &
     & 7.74009e-01_rb, 8.25236e-01_rb, 8.79853e-01_rb, 9.38085e-01_rb, 1.00017e+00_rb, &
     & 1.06637e+00_rb, 1.13694e+00_rb, 1.21219e+00_rb, 1.29242e+00_rb, 1.37795e+00_rb, &
     & 1.46915e+00_rb, 1.56638e+00_rb, 1.67005e+00_rb, 1.78058e+00_rb/)
      kao_mco2( 2, :,13) = (/ &
     & 5.55938e-01_rb, 5.91800e-01_rb, 6.29976e-01_rb, 6.70615e-01_rb, 7.13876e-01_rb, &
     & 7.59927e-01_rb, 8.08949e-01_rb, 8.61133e-01_rb, 9.16683e-01_rb, 9.75817e-01_rb, &
     & 1.03877e+00_rb, 1.10577e+00_rb, 1.17711e+00_rb, 1.25304e+00_rb, 1.33387e+00_rb, &
     & 1.41992e+00_rb, 1.51152e+00_rb, 1.60902e+00_rb, 1.71282e+00_rb/)
      kao_mco2( 3, :,13) = (/ &
     & 5.94615e-01_rb, 6.33277e-01_rb, 6.74453e-01_rb, 7.18307e-01_rb, 7.65012e-01_rb, &
     & 8.14753e-01_rb, 8.67729e-01_rb, 9.24149e-01_rb, 9.84238e-01_rb, 1.04823e+00_rb, &
     & 1.11639e+00_rb, 1.18898e+00_rb, 1.26629e+00_rb, 1.34862e+00_rb, 1.43631e+00_rb, &
     & 1.52970e+00_rb, 1.62916e+00_rb, 1.73509e+00_rb, 1.84791e+00_rb/)
      kao_mco2( 4, :,13) = (/ &
     & 5.48973e-01_rb, 5.84145e-01_rb, 6.21570e-01_rb, 6.61394e-01_rb, 7.03768e-01_rb, &
     & 7.48858e-01_rb, 7.96836e-01_rb, 8.47889e-01_rb, 9.02212e-01_rb, 9.60015e-01_rb, &
     & 1.02152e+00_rb, 1.08697e+00_rb, 1.15661e+00_rb, 1.23071e+00_rb, 1.30956e+00_rb, &
     & 1.39347e+00_rb, 1.48274e+00_rb, 1.57774e+00_rb, 1.67883e+00_rb/)
      kao_mco2( 5, :,13) = (/ &
     & 1.49742e-01_rb, 1.59049e-01_rb, 1.68934e-01_rb, 1.79434e-01_rb, 1.90586e-01_rb, &
     & 2.02432e-01_rb, 2.15013e-01_rb, 2.28377e-01_rb, 2.42571e-01_rb, 2.57648e-01_rb, &
     & 2.73661e-01_rb, 2.90670e-01_rb, 3.08736e-01_rb, 3.27925e-01_rb, 3.48307e-01_rb, &
     & 3.69955e-01_rb, 3.92949e-01_rb, 4.17372e-01_rb, 4.43312e-01_rb/)
      kao_mco2( 6, :,13) = (/ &
     & 8.81777e-04_rb, 9.16690e-04_rb, 9.52985e-04_rb, 9.90718e-04_rb, 1.02994e-03_rb, &
     & 1.07072e-03_rb, 1.11312e-03_rb, 1.15719e-03_rb, 1.20301e-03_rb, 1.25064e-03_rb, &
     & 1.30016e-03_rb, 1.35163e-03_rb, 1.40515e-03_rb, 1.46079e-03_rb, 1.51862e-03_rb, &
     & 1.57875e-03_rb, 1.64126e-03_rb, 1.70624e-03_rb, 1.77380e-03_rb/)
      kao_mco2( 7, :,13) = (/ &
     & 8.84366e-04_rb, 9.20446e-04_rb, 9.57999e-04_rb, 9.97083e-04_rb, 1.03776e-03_rb, &
     & 1.08010e-03_rb, 1.12417e-03_rb, 1.17003e-03_rb, 1.21777e-03_rb, 1.26745e-03_rb, &
     & 1.31916e-03_rb, 1.37298e-03_rb, 1.42899e-03_rb, 1.48729e-03_rb, 1.54797e-03_rb, &
     & 1.61113e-03_rb, 1.67686e-03_rb, 1.74527e-03_rb, 1.81647e-03_rb/)
      kao_mco2( 8, :,13) = (/ &
     & 8.92597e-04_rb, 9.33069e-04_rb, 9.75377e-04_rb, 1.01960e-03_rb, 1.06583e-03_rb, &
     & 1.11416e-03_rb, 1.16468e-03_rb, 1.21749e-03_rb, 1.27269e-03_rb, 1.33040e-03_rb, &
     & 1.39073e-03_rb, 1.45378e-03_rb, 1.51970e-03_rb, 1.58861e-03_rb, 1.66064e-03_rb, &
     & 1.73594e-03_rb, 1.81465e-03_rb, 1.89693e-03_rb, 1.98294e-03_rb/)
      kao_mco2( 9, :,13) = (/ &
     & 1.46280e-01_rb, 1.55378e-01_rb, 1.65043e-01_rb, 1.75308e-01_rb, 1.86212e-01_rb, &
     & 1.97794e-01_rb, 2.10097e-01_rb, 2.23164e-01_rb, 2.37045e-01_rb, 2.51788e-01_rb, &
     & 2.67449e-01_rb, 2.84084e-01_rb, 3.01754e-01_rb, 3.20522e-01_rb, 3.40458e-01_rb, &
     & 3.61634e-01_rb, 3.84127e-01_rb, 4.08020e-01_rb, 4.33398e-01_rb/)
      kao_mco2( 1, :,14) = (/ &
     & 9.20236e-01_rb, 9.80010e-01_rb, 1.04367e+00_rb, 1.11146e+00_rb, 1.18366e+00_rb, &
     & 1.26054e+00_rb, 1.34242e+00_rb, 1.42962e+00_rb, 1.52248e+00_rb, 1.62137e+00_rb, &
     & 1.72669e+00_rb, 1.83885e+00_rb, 1.95829e+00_rb, 2.08549e+00_rb, 2.22096e+00_rb, &
     & 2.36522e+00_rb, 2.51886e+00_rb, 2.68247e+00_rb, 2.85671e+00_rb/)
      kao_mco2( 2, :,14) = (/ &
     & 8.39823e-01_rb, 8.95471e-01_rb, 9.54806e-01_rb, 1.01807e+00_rb, 1.08553e+00_rb, &
     & 1.15746e+00_rb, 1.23416e+00_rb, 1.31593e+00_rb, 1.40313e+00_rb, 1.49610e+00_rb, &
     & 1.59523e+00_rb, 1.70094e+00_rb, 1.81364e+00_rb, 1.93382e+00_rb, 2.06195e+00_rb, &
     & 2.19858e+00_rb, 2.34426e+00_rb, 2.49960e+00_rb, 2.66522e+00_rb/)
      kao_mco2( 3, :,14) = (/ &
     & 5.39252e-01_rb, 5.73971e-01_rb, 6.10925e-01_rb, 6.50259e-01_rb, 6.92125e-01_rb, &
     & 7.36686e-01_rb, 7.84117e-01_rb, 8.34601e-01_rb, 8.88336e-01_rb, 9.45530e-01_rb, &
     & 1.00641e+00_rb, 1.07120e+00_rb, 1.14017e+00_rb, 1.21358e+00_rb, 1.29171e+00_rb, &
     & 1.37488e+00_rb, 1.46340e+00_rb, 1.55762e+00_rb, 1.65790e+00_rb/)
      kao_mco2( 4, :,14) = (/ &
     & 1.14837e-03_rb, 1.19701e-03_rb, 1.24770e-03_rb, 1.30055e-03_rb, 1.35563e-03_rb, &
     & 1.41305e-03_rb, 1.47289e-03_rb, 1.53528e-03_rb, 1.60030e-03_rb, 1.66808e-03_rb, &
     & 1.73873e-03_rb, 1.81237e-03_rb, 1.88913e-03_rb, 1.96914e-03_rb, 2.05254e-03_rb, &
     & 2.13947e-03_rb, 2.23009e-03_rb, 2.32454e-03_rb, 2.42299e-03_rb/)
      kao_mco2( 5, :,14) = (/ &
     & 1.14611e-03_rb, 1.19424e-03_rb, 1.24440e-03_rb, 1.29666e-03_rb, 1.35111e-03_rb, &
     & 1.40786e-03_rb, 1.46698e-03_rb, 1.52859e-03_rb, 1.59279e-03_rb, 1.65968e-03_rb, &
     & 1.72938e-03_rb, 1.80201e-03_rb, 1.87769e-03_rb, 1.95655e-03_rb, 2.03872e-03_rb, &
     & 2.12434e-03_rb, 2.21355e-03_rb, 2.30651e-03_rb, 2.40338e-03_rb/)
      kao_mco2( 6, :,14) = (/ &
     & 1.14203e-03_rb, 1.18930e-03_rb, 1.23852e-03_rb, 1.28979e-03_rb, 1.34317e-03_rb, &
     & 1.39877e-03_rb, 1.45666e-03_rb, 1.51695e-03_rb, 1.57974e-03_rb, 1.64513e-03_rb, &
     & 1.71322e-03_rb, 1.78413e-03_rb, 1.85798e-03_rb, 1.93488e-03_rb, 2.01497e-03_rb, &
     & 2.09837e-03_rb, 2.18522e-03_rb, 2.27567e-03_rb, 2.36986e-03_rb/)
      kao_mco2( 7, :,14) = (/ &
     & 1.11217e-03_rb, 1.15727e-03_rb, 1.20421e-03_rb, 1.25305e-03_rb, 1.30386e-03_rb, &
     & 1.35674e-03_rb, 1.41177e-03_rb, 1.46902e-03_rb, 1.52860e-03_rb, 1.59059e-03_rb, &
     & 1.65510e-03_rb, 1.72222e-03_rb, 1.79207e-03_rb, 1.86475e-03_rb, 1.94037e-03_rb, &
     & 2.01907e-03_rb, 2.10095e-03_rb, 2.18616e-03_rb, 2.27482e-03_rb/)
      kao_mco2( 8, :,14) = (/ &
     & 1.21596e-03_rb, 1.25817e-03_rb, 1.30183e-03_rb, 1.34702e-03_rb, 1.39377e-03_rb, &
     & 1.44214e-03_rb, 1.49219e-03_rb, 1.54398e-03_rb, 1.59757e-03_rb, 1.65302e-03_rb, &
     & 1.71039e-03_rb, 1.76975e-03_rb, 1.83117e-03_rb, 1.89473e-03_rb, 1.96049e-03_rb, &
     & 2.02853e-03_rb, 2.09893e-03_rb, 2.17178e-03_rb, 2.24716e-03_rb/)
      kao_mco2( 9, :,14) = (/ &
     & 1.14611e-03_rb, 1.19424e-03_rb, 1.24440e-03_rb, 1.29666e-03_rb, 1.35111e-03_rb, &
     & 1.40786e-03_rb, 1.46698e-03_rb, 1.52859e-03_rb, 1.59279e-03_rb, 1.65968e-03_rb, &
     & 1.72938e-03_rb, 1.80201e-03_rb, 1.87769e-03_rb, 1.95655e-03_rb, 2.03872e-03_rb, &
     & 2.12434e-03_rb, 2.21355e-03_rb, 2.30651e-03_rb, 2.40338e-03_rb/)
      kao_mco2( 1, :,15) = (/ &
     & 1.29470e+00_rb, 1.37848e+00_rb, 1.46768e+00_rb, 1.56266e+00_rb, 1.66378e+00_rb, &
     & 1.77145e+00_rb, 1.88609e+00_rb, 2.00814e+00_rb, 2.13809e+00_rb, 2.27645e+00_rb, &
     & 2.42376e+00_rb, 2.58061e+00_rb, 2.74761e+00_rb, 2.92541e+00_rb, 3.11472e+00_rb, &
     & 3.31628e+00_rb, 3.53088e+00_rb, 3.75938e+00_rb, 4.00265e+00_rb/)
      kao_mco2( 2, :,15) = (/ &
     & 7.23701e-01_rb, 7.68508e-01_rb, 8.16089e-01_rb, 8.66616e-01_rb, 9.20272e-01_rb, &
     & 9.77250e-01_rb, 1.03775e+00_rb, 1.10201e+00_rb, 1.17024e+00_rb, 1.24269e+00_rb, &
     & 1.31963e+00_rb, 1.40133e+00_rb, 1.48809e+00_rb, 1.58023e+00_rb, 1.67807e+00_rb, &
     & 1.78196e+00_rb, 1.89229e+00_rb, 2.00945e+00_rb, 2.13386e+00_rb/)
      kao_mco2( 3, :,15) = (/ &
     & 1.81684e-03_rb, 1.85424e-03_rb, 1.89241e-03_rb, 1.93137e-03_rb, 1.97114e-03_rb, &
     & 2.01172e-03_rb, 2.05313e-03_rb, 2.09540e-03_rb, 2.13854e-03_rb, 2.18257e-03_rb, &
     & 2.22750e-03_rb, 2.27336e-03_rb, 2.32016e-03_rb, 2.36793e-03_rb, 2.41668e-03_rb, &
     & 2.46643e-03_rb, 2.51721e-03_rb, 2.56903e-03_rb, 2.62192e-03_rb/)
      kao_mco2( 4, :,15) = (/ &
     & 1.84644e-03_rb, 1.88437e-03_rb, 1.92309e-03_rb, 1.96260e-03_rb, 2.00293e-03_rb, &
     & 2.04408e-03_rb, 2.08608e-03_rb, 2.12894e-03_rb, 2.17268e-03_rb, 2.21732e-03_rb, &
     & 2.26288e-03_rb, 2.30938e-03_rb, 2.35683e-03_rb, 2.40525e-03_rb, 2.45467e-03_rb, &
     & 2.50510e-03_rb, 2.55658e-03_rb, 2.60910e-03_rb, 2.66271e-03_rb/)
      kao_mco2( 5, :,15) = (/ &
     & 1.88579e-03_rb, 1.92454e-03_rb, 1.96408e-03_rb, 2.00443e-03_rb, 2.04561e-03_rb, &
     & 2.08764e-03_rb, 2.13054e-03_rb, 2.17431e-03_rb, 2.21898e-03_rb, 2.26457e-03_rb, &
     & 2.31110e-03_rb, 2.35858e-03_rb, 2.40704e-03_rb, 2.45650e-03_rb, 2.50697e-03_rb, &
     & 2.55848e-03_rb, 2.61104e-03_rb, 2.66469e-03_rb, 2.71943e-03_rb/)
      kao_mco2( 6, :,15) = (/ &
     & 1.95322e-03_rb, 1.99316e-03_rb, 2.03391e-03_rb, 2.07549e-03_rb, 2.11793e-03_rb, &
     & 2.16123e-03_rb, 2.20542e-03_rb, 2.25051e-03_rb, 2.29652e-03_rb, 2.34347e-03_rb, &
     & 2.39139e-03_rb, 2.44028e-03_rb, 2.49017e-03_rb, 2.54109e-03_rb, 2.59304e-03_rb, &
     & 2.64605e-03_rb, 2.70015e-03_rb, 2.75536e-03_rb, 2.81169e-03_rb/)
      kao_mco2( 7, :,15) = (/ &
     & 2.13640e-03_rb, 2.17976e-03_rb, 2.22400e-03_rb, 2.26914e-03_rb, 2.31520e-03_rb, &
     & 2.36219e-03_rb, 2.41013e-03_rb, 2.45905e-03_rb, 2.50896e-03_rb, 2.55988e-03_rb, &
     & 2.61184e-03_rb, 2.66485e-03_rb, 2.71893e-03_rb, 2.77412e-03_rb, 2.83042e-03_rb, &
     & 2.88787e-03_rb, 2.94648e-03_rb, 3.00629e-03_rb, 3.06730e-03_rb/)
      kao_mco2( 8, :,15) = (/ &
     & 2.17014e-03_rb, 2.21411e-03_rb, 2.25897e-03_rb, 2.30474e-03_rb, 2.35143e-03_rb, &
     & 2.39908e-03_rb, 2.44769e-03_rb, 2.49728e-03_rb, 2.54788e-03_rb, 2.59950e-03_rb, &
     & 2.65217e-03_rb, 2.70591e-03_rb, 2.76073e-03_rb, 2.81667e-03_rb, 2.87374e-03_rb, &
     & 2.93197e-03_rb, 2.99137e-03_rb, 3.05198e-03_rb, 3.11382e-03_rb/)
      kao_mco2( 9, :,15) = (/ &
     & 1.88579e-03_rb, 1.92454e-03_rb, 1.96408e-03_rb, 2.00443e-03_rb, 2.04561e-03_rb, &
     & 2.08764e-03_rb, 2.13054e-03_rb, 2.17431e-03_rb, 2.21898e-03_rb, 2.26457e-03_rb, &
     & 2.31110e-03_rb, 2.35858e-03_rb, 2.40704e-03_rb, 2.45650e-03_rb, 2.50697e-03_rb, &
     & 2.55848e-03_rb, 2.61104e-03_rb, 2.66469e-03_rb, 2.71943e-03_rb/)
      kao_mco2( 1, :,16) = (/ &
     & 1.48989e+00_rb, 1.58377e+00_rb, 1.68356e+00_rb, 1.78964e+00_rb, 1.90241e+00_rb, &
     & 2.02228e+00_rb, 2.14971e+00_rb, 2.28516e+00_rb, 2.42915e+00_rb, 2.58221e+00_rb, &
     & 2.74492e+00_rb, 2.91788e+00_rb, 3.10174e+00_rb, 3.29718e+00_rb, 3.50494e+00_rb, &
     & 3.72578e+00_rb, 3.96055e+00_rb, 4.21010e+00_rb, 4.47538e+00_rb/)
      kao_mco2( 2, :,16) = (/ &
     & 2.10609e-03_rb, 2.14759e-03_rb, 2.18992e-03_rb, 2.23307e-03_rb, 2.27708e-03_rb, &
     & 2.32196e-03_rb, 2.36771e-03_rb, 2.41438e-03_rb, 2.46196e-03_rb, 2.51047e-03_rb, &
     & 2.55995e-03_rb, 2.61040e-03_rb, 2.66184e-03_rb, 2.71430e-03_rb, 2.76779e-03_rb, &
     & 2.82234e-03_rb, 2.87796e-03_rb, 2.93467e-03_rb, 2.99251e-03_rb/)
      kao_mco2( 3, :,16) = (/ &
     & 2.10609e-03_rb, 2.14759e-03_rb, 2.18992e-03_rb, 2.23307e-03_rb, 2.27708e-03_rb, &
     & 2.32196e-03_rb, 2.36771e-03_rb, 2.41438e-03_rb, 2.46196e-03_rb, 2.51047e-03_rb, &
     & 2.55995e-03_rb, 2.61040e-03_rb, 2.66184e-03_rb, 2.71430e-03_rb, 2.76779e-03_rb, &
     & 2.82234e-03_rb, 2.87796e-03_rb, 2.93467e-03_rb, 2.99251e-03_rb/)
      kao_mco2( 4, :,16) = (/ &
     & 2.10609e-03_rb, 2.14759e-03_rb, 2.18992e-03_rb, 2.23307e-03_rb, 2.27708e-03_rb, &
     & 2.32196e-03_rb, 2.36771e-03_rb, 2.41438e-03_rb, 2.46196e-03_rb, 2.51047e-03_rb, &
     & 2.55995e-03_rb, 2.61040e-03_rb, 2.66184e-03_rb, 2.71430e-03_rb, 2.76779e-03_rb, &
     & 2.82234e-03_rb, 2.87796e-03_rb, 2.93467e-03_rb, 2.99251e-03_rb/)
      kao_mco2( 5, :,16) = (/ &
     & 2.10609e-03_rb, 2.14759e-03_rb, 2.18992e-03_rb, 2.23307e-03_rb, 2.27708e-03_rb, &
     & 2.32196e-03_rb, 2.36771e-03_rb, 2.41438e-03_rb, 2.46196e-03_rb, 2.51047e-03_rb, &
     & 2.55995e-03_rb, 2.61040e-03_rb, 2.66184e-03_rb, 2.71430e-03_rb, 2.76779e-03_rb, &
     & 2.82234e-03_rb, 2.87796e-03_rb, 2.93467e-03_rb, 2.99251e-03_rb/)
      kao_mco2( 6, :,16) = (/ &
     & 2.10609e-03_rb, 2.14759e-03_rb, 2.18992e-03_rb, 2.23307e-03_rb, 2.27708e-03_rb, &
     & 2.32196e-03_rb, 2.36771e-03_rb, 2.41438e-03_rb, 2.46196e-03_rb, 2.51047e-03_rb, &
     & 2.55995e-03_rb, 2.61040e-03_rb, 2.66184e-03_rb, 2.71430e-03_rb, 2.76779e-03_rb, &
     & 2.82234e-03_rb, 2.87796e-03_rb, 2.93467e-03_rb, 2.99251e-03_rb/)
      kao_mco2( 7, :,16) = (/ &
     & 2.09970e-03_rb, 2.14101e-03_rb, 2.18312e-03_rb, 2.22606e-03_rb, 2.26985e-03_rb, &
     & 2.31450e-03_rb, 2.36003e-03_rb, 2.40645e-03_rb, 2.45379e-03_rb, 2.50205e-03_rb, &
     & 2.55127e-03_rb, 2.60146e-03_rb, 2.65263e-03_rb, 2.70481e-03_rb, 2.75801e-03_rb, &
     & 2.81226e-03_rb, 2.86758e-03_rb, 2.92399e-03_rb, 2.98150e-03_rb/)
      kao_mco2( 8, :,16) = (/ &
     & 2.09970e-03_rb, 2.14101e-03_rb, 2.18312e-03_rb, 2.22606e-03_rb, 2.26985e-03_rb, &
     & 2.31450e-03_rb, 2.36003e-03_rb, 2.40645e-03_rb, 2.45379e-03_rb, 2.50205e-03_rb, &
     & 2.55127e-03_rb, 2.60146e-03_rb, 2.65263e-03_rb, 2.70481e-03_rb, 2.75801e-03_rb, &
     & 2.81226e-03_rb, 2.86758e-03_rb, 2.92399e-03_rb, 2.98150e-03_rb/)
      kao_mco2( 9, :,16) = (/ &
     & 2.10609e-03_rb, 2.14759e-03_rb, 2.18992e-03_rb, 2.23307e-03_rb, 2.27708e-03_rb, &
     & 2.32196e-03_rb, 2.36771e-03_rb, 2.41438e-03_rb, 2.46196e-03_rb, 2.51047e-03_rb, &
     & 2.55995e-03_rb, 2.61040e-03_rb, 2.66184e-03_rb, 2.71430e-03_rb, 2.76779e-03_rb, &
     & 2.82234e-03_rb, 2.87796e-03_rb, 2.93467e-03_rb, 2.99251e-03_rb/)

      kao_mco( 1, :, 1) = (/ &
     & 4.58355e-01_rb, 4.47074e-01_rb, 4.36070e-01_rb, 4.25337e-01_rb, 4.14868e-01_rb, &
     & 4.04657e-01_rb, 3.94697e-01_rb, 3.84982e-01_rb, 3.75506e-01_rb, 3.66264e-01_rb, &
     & 3.57249e-01_rb, 3.48456e-01_rb, 3.39879e-01_rb, 3.31514e-01_rb, 3.23354e-01_rb, &
     & 3.15395e-01_rb, 3.07632e-01_rb, 3.00061e-01_rb, 2.92675e-01_rb/)
      kao_mco( 2, :, 1) = (/ &
     & 7.03080e-01_rb, 6.84132e-01_rb, 6.65696e-01_rb, 6.47756e-01_rb, 6.30300e-01_rb, &
     & 6.13314e-01_rb, 5.96786e-01_rb, 5.80703e-01_rb, 5.65053e-01_rb, 5.49826e-01_rb, &
     & 5.35009e-01_rb, 5.20591e-01_rb, 5.06561e-01_rb, 4.92910e-01_rb, 4.79627e-01_rb, &
     & 4.66701e-01_rb, 4.54124e-01_rb, 4.41886e-01_rb, 4.29978e-01_rb/)
      kao_mco( 3, :, 1) = (/ &
     & 8.53018e-01_rb, 8.29537e-01_rb, 8.06703e-01_rb, 7.84497e-01_rb, 7.62903e-01_rb, &
     & 7.41903e-01_rb, 7.21481e-01_rb, 7.01621e-01_rb, 6.82307e-01_rb, 6.63526e-01_rb, &
     & 6.45261e-01_rb, 6.27499e-01_rb, 6.10226e-01_rb, 5.93429e-01_rb, 5.77094e-01_rb, &
     & 5.61208e-01_rb, 5.45760e-01_rb, 5.30737e-01_rb, 5.16128e-01_rb/)
      kao_mco( 4, :, 1) = (/ &
     & 9.58866e-01_rb, 9.31881e-01_rb, 9.05654e-01_rb, 8.80166e-01_rb, 8.55395e-01_rb, &
     & 8.31321e-01_rb, 8.07925e-01_rb, 7.85187e-01_rb, 7.63089e-01_rb, 7.41613e-01_rb, &
     & 7.20742e-01_rb, 7.00457e-01_rb, 6.80744e-01_rb, 6.61586e-01_rb, 6.42966e-01_rb, &
     & 6.24871e-01_rb, 6.07285e-01_rb, 5.90194e-01_rb, 5.73584e-01_rb/)
      kao_mco( 5, :, 1) = (/ &
     & 1.07140e+00_rb, 1.04056e+00_rb, 1.01061e+00_rb, 9.81521e-01_rb, 9.53269e-01_rb, &
     & 9.25829e-01_rb, 8.99180e-01_rb, 8.73297e-01_rb, 8.48160e-01_rb, 8.23746e-01_rb, &
     & 8.00035e-01_rb, 7.77006e-01_rb, 7.54641e-01_rb, 7.32919e-01_rb, 7.11822e-01_rb, &
     & 6.91333e-01_rb, 6.71433e-01_rb, 6.52106e-01_rb, 6.33336e-01_rb/)
      kao_mco( 6, :, 1) = (/ &
     & 1.21046e+00_rb, 1.17478e+00_rb, 1.14015e+00_rb, 1.10655e+00_rb, 1.07393e+00_rb, &
     & 1.04228e+00_rb, 1.01156e+00_rb, 9.81740e-01_rb, 9.52803e-01_rb, 9.24720e-01_rb, &
     & 8.97463e-01_rb, 8.71011e-01_rb, 8.45338e-01_rb, 8.20422e-01_rb, 7.96240e-01_rb, &
     & 7.72771e-01_rb, 7.49993e-01_rb, 7.27887e-01_rb, 7.06433e-01_rb/)
      kao_mco( 7, :, 1) = (/ &
     & 1.57730e+00_rb, 1.52919e+00_rb, 1.48255e+00_rb, 1.43733e+00_rb, 1.39349e+00_rb, &
     & 1.35099e+00_rb, 1.30978e+00_rb, 1.26983e+00_rb, 1.23110e+00_rb, 1.19355e+00_rb, &
     & 1.15715e+00_rb, 1.12186e+00_rb, 1.08764e+00_rb, 1.05446e+00_rb, 1.02230e+00_rb, &
     & 9.91121e-01_rb, 9.60890e-01_rb, 9.31583e-01_rb, 9.03169e-01_rb/)
      kao_mco( 8, :, 1) = (/ &
     & 2.43678e+00_rb, 2.36595e+00_rb, 2.29719e+00_rb, 2.23042e+00_rb, 2.16560e+00_rb, &
     & 2.10266e+00_rb, 2.04154e+00_rb, 1.98221e+00_rb, 1.92460e+00_rb, 1.86866e+00_rb, &
     & 1.81435e+00_rb, 1.76162e+00_rb, 1.71042e+00_rb, 1.66070e+00_rb, 1.61244e+00_rb, &
     & 1.56557e+00_rb, 1.52007e+00_rb, 1.47589e+00_rb, 1.43300e+00_rb/)
      kao_mco( 9, :, 1) = (/ &
     & 9.66296e-01_rb, 9.39903e-01_rb, 9.14232e-01_rb, 8.89262e-01_rb, 8.64973e-01_rb, &
     & 8.41348e-01_rb, 8.18369e-01_rb, 7.96017e-01_rb, 7.74275e-01_rb, 7.53128e-01_rb, &
     & 7.32558e-01_rb, 7.12549e-01_rb, 6.93088e-01_rb, 6.74157e-01_rb, 6.55744e-01_rb, &
     & 6.37834e-01_rb, 6.20413e-01_rb, 6.03468e-01_rb, 5.86985e-01_rb/)
      kao_mco( 1, :, 2) = (/ &
     & 1.15047e+00_rb, 1.12127e+00_rb, 1.09281e+00_rb, 1.06507e+00_rb, 1.03804e+00_rb, &
     & 1.01169e+00_rb, 9.86010e-01_rb, 9.60983e-01_rb, 9.36591e-01_rb, 9.12818e-01_rb, &
     & 8.89649e-01_rb, 8.67067e-01_rb, 8.45059e-01_rb, 8.23610e-01_rb, 8.02705e-01_rb, &
     & 7.82330e-01_rb, 7.62473e-01_rb, 7.43119e-01_rb, 7.24257e-01_rb/)
      kao_mco( 2, :, 2) = (/ &
     & 1.43243e+00_rb, 1.39430e+00_rb, 1.35719e+00_rb, 1.32106e+00_rb, 1.28590e+00_rb, &
     & 1.25167e+00_rb, 1.21836e+00_rb, 1.18593e+00_rb, 1.15436e+00_rb, 1.12364e+00_rb, &
     & 1.09373e+00_rb, 1.06462e+00_rb, 1.03628e+00_rb, 1.00870e+00_rb, 9.81848e-01_rb, &
     & 9.55714e-01_rb, 9.30275e-01_rb, 9.05514e-01_rb, 8.81412e-01_rb/)
      kao_mco( 3, :, 2) = (/ &
     & 1.61389e+00_rb, 1.56911e+00_rb, 1.52556e+00_rb, 1.48323e+00_rb, 1.44207e+00_rb, &
     & 1.40205e+00_rb, 1.36314e+00_rb, 1.32531e+00_rb, 1.28854e+00_rb, 1.25278e+00_rb, &
     & 1.21801e+00_rb, 1.18421e+00_rb, 1.15135e+00_rb, 1.11940e+00_rb, 1.08834e+00_rb, &
     & 1.05814e+00_rb, 1.02877e+00_rb, 1.00022e+00_rb, 9.72466e-01_rb/)
      kao_mco( 4, :, 2) = (/ &
     & 1.78458e+00_rb, 1.73440e+00_rb, 1.68564e+00_rb, 1.63825e+00_rb, 1.59219e+00_rb, &
     & 1.54742e+00_rb, 1.50391e+00_rb, 1.46163e+00_rb, 1.42053e+00_rb, 1.38059e+00_rb, &
     & 1.34178e+00_rb, 1.30405e+00_rb, 1.26739e+00_rb, 1.23175e+00_rb, 1.19712e+00_rb, &
     & 1.16346e+00_rb, 1.13075e+00_rb, 1.09896e+00_rb, 1.06806e+00_rb/)
      kao_mco( 5, :, 2) = (/ &
     & 1.92622e+00_rb, 1.87172e+00_rb, 1.81876e+00_rb, 1.76730e+00_rb, 1.71730e+00_rb, &
     & 1.66871e+00_rb, 1.62150e+00_rb, 1.57562e+00_rb, 1.53104e+00_rb, 1.48772e+00_rb, &
     & 1.44563e+00_rb, 1.40473e+00_rb, 1.36498e+00_rb, 1.32636e+00_rb, 1.28883e+00_rb, &
     & 1.25237e+00_rb, 1.21693e+00_rb, 1.18250e+00_rb, 1.14905e+00_rb/)
      kao_mco( 6, :, 2) = (/ &
     & 2.23194e+00_rb, 2.16782e+00_rb, 2.10554e+00_rb, 2.04505e+00_rb, 1.98630e+00_rb, &
     & 1.92924e+00_rb, 1.87381e+00_rb, 1.81998e+00_rb, 1.76770e+00_rb, 1.71691e+00_rb, &
     & 1.66759e+00_rb, 1.61968e+00_rb, 1.57315e+00_rb, 1.52796e+00_rb, 1.48406e+00_rb, &
     & 1.44143e+00_rb, 1.40002e+00_rb, 1.35980e+00_rb, 1.32073e+00_rb/)
      kao_mco( 7, :, 2) = (/ &
     & 2.64692e+00_rb, 2.57290e+00_rb, 2.50096e+00_rb, 2.43103e+00_rb, 2.36305e+00_rb, &
     & 2.29697e+00_rb, 2.23275e+00_rb, 2.17031e+00_rb, 2.10963e+00_rb, 2.05064e+00_rb, &
     & 1.99330e+00_rb, 1.93756e+00_rb, 1.88338e+00_rb, 1.83072e+00_rb, 1.77953e+00_rb, &
     & 1.72977e+00_rb, 1.68140e+00_rb, 1.63438e+00_rb, 1.58868e+00_rb/)
      kao_mco( 8, :, 2) = (/ &
     & 2.86812e+00_rb, 2.80121e+00_rb, 2.73586e+00_rb, 2.67204e+00_rb, 2.60970e+00_rb, &
     & 2.54882e+00_rb, 2.48936e+00_rb, 2.43129e+00_rb, 2.37457e+00_rb, 2.31917e+00_rb, &
     & 2.26507e+00_rb, 2.21223e+00_rb, 2.16062e+00_rb, 2.11022e+00_rb, 2.06099e+00_rb, &
     & 2.01291e+00_rb, 1.96595e+00_rb, 1.92009e+00_rb, 1.87529e+00_rb/)
      kao_mco( 9, :, 2) = (/ &
     & 1.25243e+00_rb, 1.22790e+00_rb, 1.20385e+00_rb, 1.18027e+00_rb, 1.15716e+00_rb, &
     & 1.13449e+00_rb, 1.11227e+00_rb, 1.09049e+00_rb, 1.06913e+00_rb, 1.04819e+00_rb, &
     & 1.02766e+00_rb, 1.00754e+00_rb, 9.87804e-01_rb, 9.68457e-01_rb, 9.49490e-01_rb, &
     & 9.30894e-01_rb, 9.12662e-01_rb, 8.94787e-01_rb, 8.77263e-01_rb/)
      kao_mco( 1, :, 3) = (/ &
     & 2.55598e+00_rb, 2.48729e+00_rb, 2.42045e+00_rb, 2.35541e+00_rb, 2.29211e+00_rb, &
     & 2.23052e+00_rb, 2.17058e+00_rb, 2.11225e+00_rb, 2.05549e+00_rb, 2.00025e+00_rb, &
     & 1.94650e+00_rb, 1.89419e+00_rb, 1.84329e+00_rb, 1.79376e+00_rb, 1.74555e+00_rb, &
     & 1.69865e+00_rb, 1.65300e+00_rb, 1.60858e+00_rb, 1.56535e+00_rb/)
      kao_mco( 2, :, 3) = (/ &
     & 2.93113e+00_rb, 2.85257e+00_rb, 2.77612e+00_rb, 2.70172e+00_rb, 2.62932e+00_rb, &
     & 2.55885e+00_rb, 2.49028e+00_rb, 2.42354e+00_rb, 2.35859e+00_rb, 2.29538e+00_rb, &
     & 2.23386e+00_rb, 2.17400e+00_rb, 2.11573e+00_rb, 2.05903e+00_rb, 2.00385e+00_rb, &
     & 1.95015e+00_rb, 1.89788e+00_rb, 1.84702e+00_rb, 1.79752e+00_rb/)
      kao_mco( 3, :, 3) = (/ &
     & 3.26626e+00_rb, 3.18025e+00_rb, 3.09651e+00_rb, 3.01497e+00_rb, 2.93558e+00_rb, &
     & 2.85828e+00_rb, 2.78302e+00_rb, 2.70973e+00_rb, 2.63838e+00_rb, 2.56891e+00_rb, &
     & 2.50126e+00_rb, 2.43540e+00_rb, 2.37127e+00_rb, 2.30883e+00_rb, 2.24803e+00_rb, &
     & 2.18883e+00_rb, 2.13120e+00_rb, 2.07508e+00_rb, 2.02044e+00_rb/)
      kao_mco( 4, :, 3) = (/ &
     & 3.65895e+00_rb, 3.56418e+00_rb, 3.47187e+00_rb, 3.38194e+00_rb, 3.29435e+00_rb, &
     & 3.20903e+00_rb, 3.12591e+00_rb, 3.04495e+00_rb, 2.96608e+00_rb, 2.88926e+00_rb, &
     & 2.81443e+00_rb, 2.74153e+00_rb, 2.67053e+00_rb, 2.60136e+00_rb, 2.53398e+00_rb, &
     & 2.46835e+00_rb, 2.40442e+00_rb, 2.34214e+00_rb, 2.28148e+00_rb/)
      kao_mco( 5, :, 3) = (/ &
     & 4.13692e+00_rb, 4.03459e+00_rb, 3.93479e+00_rb, 3.83746e+00_rb, 3.74254e+00_rb, &
     & 3.64997e+00_rb, 3.55968e+00_rb, 3.47163e+00_rb, 3.38576e+00_rb, 3.30201e+00_rb, &
     & 3.22034e+00_rb, 3.14068e+00_rb, 3.06299e+00_rb, 2.98723e+00_rb, 2.91334e+00_rb, &
     & 2.84128e+00_rb, 2.77100e+00_rb, 2.70246e+00_rb, 2.63561e+00_rb/)
      kao_mco( 6, :, 3) = (/ &
     & 4.42856e+00_rb, 4.32480e+00_rb, 4.22348e+00_rb, 4.12453e+00_rb, 4.02790e+00_rb, &
     & 3.93353e+00_rb, 3.84137e+00_rb, 3.75137e+00_rb, 3.66348e+00_rb, 3.57765e+00_rb, &
     & 3.49383e+00_rb, 3.41198e+00_rb, 3.33204e+00_rb, 3.25397e+00_rb, 3.17774e+00_rb, &
     & 3.10329e+00_rb, 3.03058e+00_rb, 2.95958e+00_rb, 2.89024e+00_rb/)
      kao_mco( 7, :, 3) = (/ &
     & 4.31306e+00_rb, 4.21750e+00_rb, 4.12406e+00_rb, 4.03268e+00_rb, 3.94333e+00_rb, &
     & 3.85596e+00_rb, 3.77053e+00_rb, 3.68699e+00_rb, 3.60530e+00_rb, 3.52542e+00_rb, &
     & 3.44731e+00_rb, 3.37093e+00_rb, 3.29624e+00_rb, 3.22321e+00_rb, 3.15179e+00_rb, &
     & 3.08196e+00_rb, 3.01368e+00_rb, 2.94691e+00_rb, 2.88161e+00_rb/)
      kao_mco( 8, :, 3) = (/ &
     & 4.38922e+00_rb, 4.32180e+00_rb, 4.25543e+00_rb, 4.19007e+00_rb, 4.12571e+00_rb, &
     & 4.06234e+00_rb, 3.99995e+00_rb, 3.93851e+00_rb, 3.87802e+00_rb, 3.81846e+00_rb, &
     & 3.75981e+00_rb, 3.70206e+00_rb, 3.64520e+00_rb, 3.58922e+00_rb, 3.53409e+00_rb, &
     & 3.47981e+00_rb, 3.42636e+00_rb, 3.37374e+00_rb, 3.32192e+00_rb/)
      kao_mco( 9, :, 3) = (/ &
     & 1.56810e+00_rb, 1.54211e+00_rb, 1.51654e+00_rb, 1.49139e+00_rb, 1.46667e+00_rb, &
     & 1.44235e+00_rb, 1.41844e+00_rb, 1.39492e+00_rb, 1.37179e+00_rb, 1.34905e+00_rb, &
     & 1.32668e+00_rb, 1.30469e+00_rb, 1.28306e+00_rb, 1.26178e+00_rb, 1.24086e+00_rb, &
     & 1.22029e+00_rb, 1.20006e+00_rb, 1.18016e+00_rb, 1.16059e+00_rb/)
      kao_mco( 1, :, 4) = (/ &
     & 6.58275e+00_rb, 6.43026e+00_rb, 6.28130e+00_rb, 6.13579e+00_rb, 5.99365e+00_rb, &
     & 5.85481e+00_rb, 5.71918e+00_rb, 5.58669e+00_rb, 5.45727e+00_rb, 5.33085e+00_rb, &
     & 5.20736e+00_rb, 5.08673e+00_rb, 4.96889e+00_rb, 4.85379e+00_rb, 4.74135e+00_rb, &
     & 4.63151e+00_rb, 4.52422e+00_rb, 4.41942e+00_rb, 4.31704e+00_rb/)
      kao_mco( 2, :, 4) = (/ &
     & 6.59883e+00_rb, 6.45139e+00_rb, 6.30725e+00_rb, 6.16633e+00_rb, 6.02855e+00_rb, &
     & 5.89386e+00_rb, 5.76217e+00_rb, 5.63342e+00_rb, 5.50756e+00_rb, 5.38450e+00_rb, &
     & 5.26419e+00_rb, 5.14657e+00_rb, 5.03158e+00_rb, 4.91916e+00_rb, 4.80925e+00_rb, &
     & 4.70180e+00_rb, 4.59675e+00_rb, 4.49404e+00_rb, 4.39363e+00_rb/)
      kao_mco( 3, :, 4) = (/ &
     & 6.58521e+00_rb, 6.44452e+00_rb, 6.30683e+00_rb, 6.17209e+00_rb, 6.04023e+00_rb, &
     & 5.91118e+00_rb, 5.78489e+00_rb, 5.66130e+00_rb, 5.54034e+00_rb, 5.42198e+00_rb, &
     & 5.30614e+00_rb, 5.19277e+00_rb, 5.08183e+00_rb, 4.97326e+00_rb, 4.86701e+00_rb, &
     & 4.76303e+00_rb, 4.66127e+00_rb, 4.56168e+00_rb, 4.46422e+00_rb/)
      kao_mco( 4, :, 4) = (/ &
     & 6.33742e+00_rb, 6.20870e+00_rb, 6.08260e+00_rb, 5.95905e+00_rb, 5.83802e+00_rb, &
     & 5.71945e+00_rb, 5.60328e+00_rb, 5.48947e+00_rb, 5.37798e+00_rb, 5.26875e+00_rb, &
     & 5.16174e+00_rb, 5.05690e+00_rb, 4.95419e+00_rb, 4.85357e+00_rb, 4.75499e+00_rb, &
     & 4.65841e+00_rb, 4.56379e+00_rb, 4.47110e+00_rb, 4.38029e+00_rb/)
      kao_mco( 5, :, 4) = (/ &
     & 5.99732e+00_rb, 5.88459e+00_rb, 5.77398e+00_rb, 5.66545e+00_rb, 5.55896e+00_rb, &
     & 5.45447e+00_rb, 5.35194e+00_rb, 5.25134e+00_rb, 5.15263e+00_rb, 5.05578e+00_rb, &
     & 4.96075e+00_rb, 4.86750e+00_rb, 4.77601e+00_rb, 4.68623e+00_rb, 4.59815e+00_rb, &
     & 4.51171e+00_rb, 4.42691e+00_rb, 4.34370e+00_rb, 4.26205e+00_rb/)
      kao_mco( 6, :, 4) = (/ &
     & 5.74529e+00_rb, 5.65249e+00_rb, 5.56119e+00_rb, 5.47136e+00_rb, 5.38299e+00_rb, &
     & 5.29604e+00_rb, 5.21049e+00_rb, 5.12633e+00_rb, 5.04353e+00_rb, 4.96206e+00_rb, &
     & 4.88191e+00_rb, 4.80306e+00_rb, 4.72547e+00_rb, 4.64915e+00_rb, 4.57405e+00_rb, &
     & 4.50017e+00_rb, 4.42748e+00_rb, 4.35596e+00_rb, 4.28560e+00_rb/)
      kao_mco( 7, :, 4) = (/ &
     & 5.87251e+00_rb, 5.79956e+00_rb, 5.72753e+00_rb, 5.65638e+00_rb, 5.58613e+00_rb, &
     & 5.51674e+00_rb, 5.44822e+00_rb, 5.38054e+00_rb, 5.31371e+00_rb, 5.24771e+00_rb, &
     & 5.18253e+00_rb, 5.11815e+00_rb, 5.05458e+00_rb, 4.99180e+00_rb, 4.92979e+00_rb, &
     & 4.86856e+00_rb, 4.80809e+00_rb, 4.74837e+00_rb, 4.68939e+00_rb/)
      kao_mco( 8, :, 4) = (/ &
     & 5.68503e+00_rb, 5.62827e+00_rb, 5.57207e+00_rb, 5.51644e+00_rb, 5.46136e+00_rb, &
     & 5.40684e+00_rb, 5.35285e+00_rb, 5.29941e+00_rb, 5.24650e+00_rb, 5.19412e+00_rb, &
     & 5.14226e+00_rb, 5.09092e+00_rb, 5.04009e+00_rb, 4.98977e+00_rb, 4.93995e+00_rb, &
     & 4.89063e+00_rb, 4.84180e+00_rb, 4.79346e+00_rb, 4.74560e+00_rb/)
      kao_mco( 9, :, 4) = (/ &
     & 2.69278e+00_rb, 2.65058e+00_rb, 2.60903e+00_rb, 2.56814e+00_rb, 2.52789e+00_rb, &
     & 2.48827e+00_rb, 2.44927e+00_rb, 2.41088e+00_rb, 2.37310e+00_rb, 2.33590e+00_rb, &
     & 2.29929e+00_rb, 2.26325e+00_rb, 2.22778e+00_rb, 2.19286e+00_rb, 2.15849e+00_rb, &
     & 2.12466e+00_rb, 2.09136e+00_rb, 2.05859e+00_rb, 2.02632e+00_rb/)
      kao_mco( 1, :, 5) = (/ &
     & 9.12231e+00_rb, 9.00052e+00_rb, 8.88036e+00_rb, 8.76180e+00_rb, 8.64482e+00_rb, &
     & 8.52941e+00_rb, 8.41553e+00_rb, 8.30318e+00_rb, 8.19233e+00_rb, 8.08295e+00_rb, &
     & 7.97504e+00_rb, 7.86857e+00_rb, 7.76352e+00_rb, 7.65987e+00_rb, 7.55760e+00_rb, &
     & 7.45671e+00_rb, 7.35715e+00_rb, 7.25893e+00_rb, 7.16202e+00_rb/)
      kao_mco( 2, :, 5) = (/ &
     & 8.37315e+00_rb, 8.27808e+00_rb, 8.18410e+00_rb, 8.09118e+00_rb, 7.99931e+00_rb, &
     & 7.90849e+00_rb, 7.81871e+00_rb, 7.72994e+00_rb, 7.64217e+00_rb, 7.55541e+00_rb, &
     & 7.46963e+00_rb, 7.38482e+00_rb, 7.30098e+00_rb, 7.21809e+00_rb, 7.13614e+00_rb, &
     & 7.05512e+00_rb, 6.97502e+00_rb, 6.89582e+00_rb, 6.81753e+00_rb/)
      kao_mco( 3, :, 5) = (/ &
     & 8.14557e+00_rb, 8.06533e+00_rb, 7.98587e+00_rb, 7.90720e+00_rb, 7.82930e+00_rb, &
     & 7.75217e+00_rb, 7.67580e+00_rb, 7.60018e+00_rb, 7.52530e+00_rb, 7.45117e+00_rb, &
     & 7.37776e+00_rb, 7.30508e+00_rb, 7.23311e+00_rb, 7.16186e+00_rb, 7.09130e+00_rb, &
     & 7.02144e+00_rb, 6.95227e+00_rb, 6.88378e+00_rb, 6.81596e+00_rb/)
      kao_mco( 4, :, 5) = (/ &
     & 8.18046e+00_rb, 8.11056e+00_rb, 8.04126e+00_rb, 7.97256e+00_rb, 7.90444e+00_rb, &
     & 7.83690e+00_rb, 7.76994e+00_rb, 7.70355e+00_rb, 7.63773e+00_rb, 7.57247e+00_rb, &
     & 7.50777e+00_rb, 7.44362e+00_rb, 7.38002e+00_rb, 7.31697e+00_rb, 7.25445e+00_rb, &
     & 7.19247e+00_rb, 7.13101e+00_rb, 7.07008e+00_rb, 7.00968e+00_rb/)
      kao_mco( 5, :, 5) = (/ &
     & 8.30092e+00_rb, 8.23529e+00_rb, 8.17019e+00_rb, 8.10559e+00_rb, 8.04151e+00_rb, &
     & 7.97794e+00_rb, 7.91487e+00_rb, 7.85230e+00_rb, 7.79022e+00_rb, 7.72863e+00_rb, &
     & 7.66753e+00_rb, 7.60691e+00_rb, 7.54678e+00_rb, 7.48711e+00_rb, 7.42792e+00_rb, &
     & 7.36920e+00_rb, 7.31094e+00_rb, 7.25314e+00_rb, 7.19580e+00_rb/)
      kao_mco( 6, :, 5) = (/ &
     & 8.30014e+00_rb, 8.24466e+00_rb, 8.18955e+00_rb, 8.13481e+00_rb, 8.08044e+00_rb, &
     & 8.02642e+00_rb, 7.97277e+00_rb, 7.91948e+00_rb, 7.86655e+00_rb, 7.81396e+00_rb, &
     & 7.76173e+00_rb, 7.70985e+00_rb, 7.65832e+00_rb, 7.60713e+00_rb, 7.55628e+00_rb, &
     & 7.50577e+00_rb, 7.45560e+00_rb, 7.40577e+00_rb, 7.35627e+00_rb/)
      kao_mco( 7, :, 5) = (/ &
     & 7.95931e+00_rb, 7.93958e+00_rb, 7.91989e+00_rb, 7.90025e+00_rb, 7.88066e+00_rb, &
     & 7.86112e+00_rb, 7.84163e+00_rb, 7.82219e+00_rb, 7.80279e+00_rb, 7.78344e+00_rb, &
     & 7.76414e+00_rb, 7.74489e+00_rb, 7.72568e+00_rb, 7.70653e+00_rb, 7.68742e+00_rb, &
     & 7.66836e+00_rb, 7.64934e+00_rb, 7.63038e+00_rb, 7.61146e+00_rb/)
      kao_mco( 8, :, 5) = (/ &
     & 9.32576e+00_rb, 9.31747e+00_rb, 9.30919e+00_rb, 9.30092e+00_rb, 9.29265e+00_rb, &
     & 9.28439e+00_rb, 9.27613e+00_rb, 9.26789e+00_rb, 9.25965e+00_rb, 9.25142e+00_rb, &
     & 9.24320e+00_rb, 9.23498e+00_rb, 9.22677e+00_rb, 9.21857e+00_rb, 9.21038e+00_rb, &
     & 9.20219e+00_rb, 9.19401e+00_rb, 9.18584e+00_rb, 9.17767e+00_rb/)
      kao_mco( 9, :, 5) = (/ &
     & 4.13116e+00_rb, 4.08426e+00_rb, 4.03788e+00_rb, 3.99204e+00_rb, 3.94671e+00_rb, &
     & 3.90190e+00_rb, 3.85760e+00_rb, 3.81380e+00_rb, 3.77049e+00_rb, 3.72768e+00_rb, &
     & 3.68536e+00_rb, 3.64351e+00_rb, 3.60214e+00_rb, 3.56124e+00_rb, 3.52081e+00_rb, &
     & 3.48083e+00_rb, 3.44131e+00_rb, 3.40224e+00_rb, 3.36361e+00_rb/)
      kao_mco( 1, :, 6) = (/ &
     & 1.21200e+01_rb, 1.21580e+01_rb, 1.21961e+01_rb, 1.22344e+01_rb, 1.22728e+01_rb, &
     & 1.23113e+01_rb, 1.23499e+01_rb, 1.23886e+01_rb, 1.24275e+01_rb, 1.24664e+01_rb, &
     & 1.25056e+01_rb, 1.25448e+01_rb, 1.25841e+01_rb, 1.26236e+01_rb, 1.26632e+01_rb, &
     & 1.27029e+01_rb, 1.27428e+01_rb, 1.27827e+01_rb, 1.28228e+01_rb/)
      kao_mco( 2, :, 6) = (/ &
     & 1.25231e+01_rb, 1.25625e+01_rb, 1.26020e+01_rb, 1.26417e+01_rb, 1.26815e+01_rb, &
     & 1.27214e+01_rb, 1.27614e+01_rb, 1.28015e+01_rb, 1.28418e+01_rb, 1.28822e+01_rb, &
     & 1.29228e+01_rb, 1.29634e+01_rb, 1.30042e+01_rb, 1.30451e+01_rb, 1.30862e+01_rb, &
     & 1.31274e+01_rb, 1.31687e+01_rb, 1.32101e+01_rb, 1.32517e+01_rb/)
      kao_mco( 3, :, 6) = (/ &
     & 1.27566e+01_rb, 1.27983e+01_rb, 1.28401e+01_rb, 1.28820e+01_rb, 1.29241e+01_rb, &
     & 1.29663e+01_rb, 1.30087e+01_rb, 1.30512e+01_rb, 1.30938e+01_rb, 1.31366e+01_rb, &
     & 1.31795e+01_rb, 1.32225e+01_rb, 1.32657e+01_rb, 1.33090e+01_rb, 1.33525e+01_rb, &
     & 1.33961e+01_rb, 1.34399e+01_rb, 1.34838e+01_rb, 1.35278e+01_rb/)
      kao_mco( 4, :, 6) = (/ &
     & 1.27132e+01_rb, 1.27454e+01_rb, 1.27777e+01_rb, 1.28101e+01_rb, 1.28425e+01_rb, &
     & 1.28750e+01_rb, 1.29077e+01_rb, 1.29403e+01_rb, 1.29731e+01_rb, 1.30060e+01_rb, &
     & 1.30389e+01_rb, 1.30720e+01_rb, 1.31051e+01_rb, 1.31383e+01_rb, 1.31716e+01_rb, &
     & 1.32049e+01_rb, 1.32384e+01_rb, 1.32719e+01_rb, 1.33055e+01_rb/)
      kao_mco( 5, :, 6) = (/ &
     & 1.33151e+01_rb, 1.33523e+01_rb, 1.33896e+01_rb, 1.34271e+01_rb, 1.34646e+01_rb, &
     & 1.35022e+01_rb, 1.35400e+01_rb, 1.35779e+01_rb, 1.36158e+01_rb, 1.36539e+01_rb, &
     & 1.36921e+01_rb, 1.37303e+01_rb, 1.37687e+01_rb, 1.38072e+01_rb, 1.38458e+01_rb, &
     & 1.38846e+01_rb, 1.39234e+01_rb, 1.39623e+01_rb, 1.40013e+01_rb/)
      kao_mco( 6, :, 6) = (/ &
     & 1.41448e+01_rb, 1.41902e+01_rb, 1.42357e+01_rb, 1.42814e+01_rb, 1.43272e+01_rb, &
     & 1.43732e+01_rb, 1.44194e+01_rb, 1.44656e+01_rb, 1.45121e+01_rb, 1.45586e+01_rb, &
     & 1.46054e+01_rb, 1.46522e+01_rb, 1.46993e+01_rb, 1.47464e+01_rb, 1.47938e+01_rb, &
     & 1.48413e+01_rb, 1.48889e+01_rb, 1.49367e+01_rb, 1.49846e+01_rb/)
      kao_mco( 7, :, 6) = (/ &
     & 1.56578e+01_rb, 1.56938e+01_rb, 1.57299e+01_rb, 1.57661e+01_rb, 1.58024e+01_rb, &
     & 1.58388e+01_rb, 1.58752e+01_rb, 1.59117e+01_rb, 1.59484e+01_rb, 1.59851e+01_rb, &
     & 1.60218e+01_rb, 1.60587e+01_rb, 1.60957e+01_rb, 1.61327e+01_rb, 1.61698e+01_rb, &
     & 1.62070e+01_rb, 1.62443e+01_rb, 1.62817e+01_rb, 1.63192e+01_rb/)
      kao_mco( 8, :, 6) = (/ &
     & 1.73627e+01_rb, 1.74761e+01_rb, 1.75903e+01_rb, 1.77052e+01_rb, 1.78208e+01_rb, &
     & 1.79373e+01_rb, 1.80544e+01_rb, 1.81724e+01_rb, 1.82911e+01_rb, 1.84106e+01_rb, &
     & 1.85309e+01_rb, 1.86519e+01_rb, 1.87738e+01_rb, 1.88964e+01_rb, 1.90198e+01_rb, &
     & 1.91441e+01_rb, 1.92692e+01_rb, 1.93950e+01_rb, 1.95217e+01_rb/)
      kao_mco( 9, :, 6) = (/ &
     & 7.16326e+00_rb, 7.12921e+00_rb, 7.09531e+00_rb, 7.06158e+00_rb, 7.02800e+00_rb, &
     & 6.99459e+00_rb, 6.96133e+00_rb, 6.92824e+00_rb, 6.89530e+00_rb, 6.86252e+00_rb, &
     & 6.82989e+00_rb, 6.79742e+00_rb, 6.76510e+00_rb, 6.73293e+00_rb, 6.70092e+00_rb, &
     & 6.66906e+00_rb, 6.63736e+00_rb, 6.60580e+00_rb, 6.57439e+00_rb/)
      kao_mco( 1, :, 7) = (/ &
     & 2.09288e+01_rb, 2.10487e+01_rb, 2.11692e+01_rb, 2.12904e+01_rb, 2.14124e+01_rb, &
     & 2.15350e+01_rb, 2.16583e+01_rb, 2.17823e+01_rb, 2.19070e+01_rb, 2.20325e+01_rb, &
     & 2.21587e+01_rb, 2.22855e+01_rb, 2.24132e+01_rb, 2.25415e+01_rb, 2.26706e+01_rb, &
     & 2.28004e+01_rb, 2.29310e+01_rb, 2.30623e+01_rb, 2.31943e+01_rb/)
      kao_mco( 2, :, 7) = (/ &
     & 2.08159e+01_rb, 2.09509e+01_rb, 2.10867e+01_rb, 2.12234e+01_rb, 2.13610e+01_rb, &
     & 2.14994e+01_rb, 2.16388e+01_rb, 2.17791e+01_rb, 2.19202e+01_rb, 2.20623e+01_rb, &
     & 2.22053e+01_rb, 2.23493e+01_rb, 2.24942e+01_rb, 2.26400e+01_rb, 2.27867e+01_rb, &
     & 2.29345e+01_rb, 2.30831e+01_rb, 2.32328e+01_rb, 2.33834e+01_rb/)
      kao_mco( 3, :, 7) = (/ &
     & 2.10827e+01_rb, 2.12409e+01_rb, 2.14002e+01_rb, 2.15608e+01_rb, 2.17225e+01_rb, &
     & 2.18855e+01_rb, 2.20497e+01_rb, 2.22151e+01_rb, 2.23818e+01_rb, 2.25497e+01_rb, &
     & 2.27189e+01_rb, 2.28893e+01_rb, 2.30611e+01_rb, 2.32341e+01_rb, 2.34084e+01_rb, &
     & 2.35840e+01_rb, 2.37609e+01_rb, 2.39392e+01_rb, 2.41188e+01_rb/)
      kao_mco( 4, :, 7) = (/ &
     & 2.13866e+01_rb, 2.15772e+01_rb, 2.17694e+01_rb, 2.19634e+01_rb, 2.21590e+01_rb, &
     & 2.23565e+01_rb, 2.25556e+01_rb, 2.27566e+01_rb, 2.29594e+01_rb, 2.31639e+01_rb, &
     & 2.33703e+01_rb, 2.35785e+01_rb, 2.37886e+01_rb, 2.40005e+01_rb, 2.42144e+01_rb, &
     & 2.44301e+01_rb, 2.46477e+01_rb, 2.48673e+01_rb, 2.50889e+01_rb/)
      kao_mco( 5, :, 7) = (/ &
     & 1.93714e+01_rb, 1.95595e+01_rb, 1.97493e+01_rb, 1.99410e+01_rb, 2.01345e+01_rb, &
     & 2.03300e+01_rb, 2.05273e+01_rb, 2.07265e+01_rb, 2.09277e+01_rb, 2.11308e+01_rb, &
     & 2.13359e+01_rb, 2.15430e+01_rb, 2.17521e+01_rb, 2.19632e+01_rb, 2.21764e+01_rb, &
     & 2.23917e+01_rb, 2.26090e+01_rb, 2.28284e+01_rb, 2.30500e+01_rb/)
      kao_mco( 6, :, 7) = (/ &
     & 1.70418e+01_rb, 1.72109e+01_rb, 1.73817e+01_rb, 1.75541e+01_rb, 1.77283e+01_rb, &
     & 1.79041e+01_rb, 1.80818e+01_rb, 1.82612e+01_rb, 1.84423e+01_rb, 1.86253e+01_rb, &
     & 1.88101e+01_rb, 1.89967e+01_rb, 1.91852e+01_rb, 1.93755e+01_rb, 1.95678e+01_rb, &
     & 1.97619e+01_rb, 1.99580e+01_rb, 2.01560e+01_rb, 2.03560e+01_rb/)
      kao_mco( 7, :, 7) = (/ &
     & 1.31735e+01_rb, 1.32921e+01_rb, 1.34118e+01_rb, 1.35326e+01_rb, 1.36545e+01_rb, &
     & 1.37775e+01_rb, 1.39015e+01_rb, 1.40267e+01_rb, 1.41531e+01_rb, 1.42805e+01_rb, &
     & 1.44091e+01_rb, 1.45389e+01_rb, 1.46698e+01_rb, 1.48019e+01_rb, 1.49353e+01_rb, &
     & 1.50698e+01_rb, 1.52055e+01_rb, 1.53424e+01_rb, 1.54806e+01_rb/)
      kao_mco( 8, :, 7) = (/ &
     & 4.97361e+00_rb, 4.96550e+00_rb, 4.95740e+00_rb, 4.94931e+00_rb, 4.94124e+00_rb, &
     & 4.93318e+00_rb, 4.92513e+00_rb, 4.91709e+00_rb, 4.90907e+00_rb, 4.90106e+00_rb, &
     & 4.89307e+00_rb, 4.88509e+00_rb, 4.87712e+00_rb, 4.86916e+00_rb, 4.86122e+00_rb, &
     & 4.85329e+00_rb, 4.84537e+00_rb, 4.83747e+00_rb, 4.82958e+00_rb/)
      kao_mco( 9, :, 7) = (/ &
     & 1.76121e+01_rb, 1.75887e+01_rb, 1.75653e+01_rb, 1.75420e+01_rb, 1.75187e+01_rb, &
     & 1.74955e+01_rb, 1.74722e+01_rb, 1.74490e+01_rb, 1.74259e+01_rb, 1.74027e+01_rb, &
     & 1.73796e+01_rb, 1.73566e+01_rb, 1.73335e+01_rb, 1.73105e+01_rb, 1.72875e+01_rb, &
     & 1.72646e+01_rb, 1.72416e+01_rb, 1.72188e+01_rb, 1.71959e+01_rb/)
      kao_mco( 1, :, 8) = (/ &
     & 5.99126e+00_rb, 6.08386e+00_rb, 6.17790e+00_rb, 6.27339e+00_rb, 6.37035e+00_rb, &
     & 6.46881e+00_rb, 6.56880e+00_rb, 6.67033e+00_rb, 6.77343e+00_rb, 6.87812e+00_rb, &
     & 6.98443e+00_rb, 7.09238e+00_rb, 7.20201e+00_rb, 7.31332e+00_rb, 7.42636e+00_rb, &
     & 7.54115e+00_rb, 7.65771e+00_rb, 7.77607e+00_rb, 7.89626e+00_rb/)
      kao_mco( 2, :, 8) = (/ &
     & 4.71621e+00_rb, 4.78830e+00_rb, 4.86149e+00_rb, 4.93580e+00_rb, 5.01124e+00_rb, &
     & 5.08784e+00_rb, 5.16561e+00_rb, 5.24456e+00_rb, 5.32473e+00_rb, 5.40612e+00_rb, &
     & 5.48875e+00_rb, 5.57264e+00_rb, 5.65782e+00_rb, 5.74430e+00_rb, 5.83211e+00_rb, &
     & 5.92125e+00_rb, 6.01176e+00_rb, 6.10365e+00_rb, 6.19694e+00_rb/)
      kao_mco( 3, :, 8) = (/ &
     & 2.77067e+00_rb, 2.81437e+00_rb, 2.85876e+00_rb, 2.90385e+00_rb, 2.94965e+00_rb, &
     & 2.99617e+00_rb, 3.04343e+00_rb, 3.09143e+00_rb, 3.14019e+00_rb, 3.18972e+00_rb, &
     & 3.24003e+00_rb, 3.29114e+00_rb, 3.34305e+00_rb, 3.39578e+00_rb, 3.44934e+00_rb, &
     & 3.50374e+00_rb, 3.55901e+00_rb, 3.61514e+00_rb, 3.67216e+00_rb/)
      kao_mco( 4, :, 8) = (/ &
     & 1.22388e+00_rb, 1.24248e+00_rb, 1.26136e+00_rb, 1.28053e+00_rb, 1.29999e+00_rb, &
     & 1.31974e+00_rb, 1.33979e+00_rb, 1.36015e+00_rb, 1.38082e+00_rb, 1.40181e+00_rb, &
     & 1.42311e+00_rb, 1.44473e+00_rb, 1.46669e+00_rb, 1.48897e+00_rb, 1.51160e+00_rb, &
     & 1.53457e+00_rb, 1.55789e+00_rb, 1.58156e+00_rb, 1.60559e+00_rb/)
      kao_mco( 5, :, 8) = (/ &
     & 1.41479e+00_rb, 1.43540e+00_rb, 1.45631e+00_rb, 1.47752e+00_rb, 1.49904e+00_rb, &
     & 1.52088e+00_rb, 1.54303e+00_rb, 1.56550e+00_rb, 1.58831e+00_rb, 1.61144e+00_rb, &
     & 1.63491e+00_rb, 1.65872e+00_rb, 1.68288e+00_rb, 1.70740e+00_rb, 1.73227e+00_rb, &
     & 1.75750e+00_rb, 1.78310e+00_rb, 1.80907e+00_rb, 1.83542e+00_rb/)
      kao_mco( 6, :, 8) = (/ &
     & 1.43154e+00_rb, 1.46074e+00_rb, 1.49053e+00_rb, 1.52093e+00_rb, 1.55196e+00_rb, &
     & 1.58361e+00_rb, 1.61591e+00_rb, 1.64887e+00_rb, 1.68250e+00_rb, 1.71682e+00_rb, &
     & 1.75184e+00_rb, 1.78757e+00_rb, 1.82403e+00_rb, 1.86123e+00_rb, 1.89920e+00_rb, &
     & 1.93793e+00_rb, 1.97746e+00_rb, 2.01779e+00_rb, 2.05895e+00_rb/)
      kao_mco( 7, :, 8) = (/ &
     & 2.49358e+00_rb, 2.56028e+00_rb, 2.62875e+00_rb, 2.69906e+00_rb, 2.77124e+00_rb, &
     & 2.84536e+00_rb, 2.92146e+00_rb, 2.99960e+00_rb, 3.07982e+00_rb, 3.16219e+00_rb, &
     & 3.24677e+00_rb, 3.33360e+00_rb, 3.42276e+00_rb, 3.51430e+00_rb, 3.60829e+00_rb, &
     & 3.70480e+00_rb, 3.80388e+00_rb, 3.90562e+00_rb, 4.01007e+00_rb/)
      kao_mco( 8, :, 8) = (/ &
     & 4.32513e+00_rb, 4.39903e+00_rb, 4.47420e+00_rb, 4.55065e+00_rb, 4.62841e+00_rb, &
     & 4.70750e+00_rb, 4.78794e+00_rb, 4.86975e+00_rb, 4.95296e+00_rb, 5.03759e+00_rb, &
     & 5.12367e+00_rb, 5.21122e+00_rb, 5.30027e+00_rb, 5.39084e+00_rb, 5.48295e+00_rb, &
     & 5.57664e+00_rb, 5.67193e+00_rb, 5.76885e+00_rb, 5.86743e+00_rb/)
      kao_mco( 9, :, 8) = (/ &
     & 3.35160e+01_rb, 3.36789e+01_rb, 3.38425e+01_rb, 3.40069e+01_rb, 3.41722e+01_rb, &
     & 3.43382e+01_rb, 3.45050e+01_rb, 3.46727e+01_rb, 3.48412e+01_rb, 3.50105e+01_rb, &
     & 3.51806e+01_rb, 3.53515e+01_rb, 3.55233e+01_rb, 3.56959e+01_rb, 3.58693e+01_rb, &
     & 3.60436e+01_rb, 3.62187e+01_rb, 3.63947e+01_rb, 3.65715e+01_rb/)
      kao_mco( 1, :, 9) = (/ &
     & 8.68159e-01_rb, 9.13680e-01_rb, 9.61587e-01_rb, 1.01201e+00_rb, 1.06507e+00_rb, &
     & 1.12091e+00_rb, 1.17969e+00_rb, 1.24154e+00_rb, 1.30664e+00_rb, 1.37515e+00_rb, &
     & 1.44726e+00_rb, 1.52314e+00_rb, 1.60300e+00_rb, 1.68705e+00_rb, 1.77551e+00_rb, &
     & 1.86861e+00_rb, 1.96658e+00_rb, 2.06970e+00_rb, 2.17822e+00_rb/)
      kao_mco( 2, :, 9) = (/ &
     & 9.04391e-01_rb, 9.49669e-01_rb, 9.97214e-01_rb, 1.04714e+00_rb, 1.09956e+00_rb, &
     & 1.15461e+00_rb, 1.21242e+00_rb, 1.27312e+00_rb, 1.33685e+00_rb, 1.40378e+00_rb, &
     & 1.47406e+00_rb, 1.54786e+00_rb, 1.62535e+00_rb, 1.70673e+00_rb, 1.79217e+00_rb, &
     & 1.88190e+00_rb, 1.97611e+00_rb, 2.07505e+00_rb, 2.17893e+00_rb/)
      kao_mco( 3, :, 9) = (/ &
     & 9.67479e-01_rb, 1.01312e+00_rb, 1.06092e+00_rb, 1.11098e+00_rb, 1.16339e+00_rb, &
     & 1.21828e+00_rb, 1.27576e+00_rb, 1.33595e+00_rb, 1.39898e+00_rb, 1.46499e+00_rb, &
     & 1.53411e+00_rb, 1.60649e+00_rb, 1.68228e+00_rb, 1.76165e+00_rb, 1.84476e+00_rb, &
     & 1.93180e+00_rb, 2.02294e+00_rb, 2.11839e+00_rb, 2.21833e+00_rb/)
      kao_mco( 4, :, 9) = (/ &
     & 1.05240e+00_rb, 1.09817e+00_rb, 1.14592e+00_rb, 1.19576e+00_rb, 1.24776e+00_rb, &
     & 1.30202e+00_rb, 1.35864e+00_rb, 1.41772e+00_rb, 1.47937e+00_rb, 1.54371e+00_rb, &
     & 1.61084e+00_rb, 1.68089e+00_rb, 1.75398e+00_rb, 1.83026e+00_rb, 1.90985e+00_rb, &
     & 1.99290e+00_rb, 2.07957e+00_rb, 2.17000e+00_rb, 2.26437e+00_rb/)
      kao_mco( 5, :, 9) = (/ &
     & 1.25800e+00_rb, 1.30557e+00_rb, 1.35494e+00_rb, 1.40618e+00_rb, 1.45935e+00_rb, &
     & 1.51454e+00_rb, 1.57181e+00_rb, 1.63125e+00_rb, 1.69293e+00_rb, 1.75695e+00_rb, &
     & 1.82339e+00_rb, 1.89234e+00_rb, 1.96389e+00_rb, 2.03816e+00_rb, 2.11523e+00_rb, &
     & 2.19522e+00_rb, 2.27823e+00_rb, 2.36438e+00_rb, 2.45378e+00_rb/)
      kao_mco( 6, :, 9) = (/ &
     & 1.76509e+00_rb, 1.80550e+00_rb, 1.84683e+00_rb, 1.88911e+00_rb, 1.93235e+00_rb, &
     & 1.97658e+00_rb, 2.02183e+00_rb, 2.06811e+00_rb, 2.11546e+00_rb, 2.16388e+00_rb, &
     & 2.21342e+00_rb, 2.26408e+00_rb, 2.31591e+00_rb, 2.36893e+00_rb, 2.42315e+00_rb, &
     & 2.47862e+00_rb, 2.53536e+00_rb, 2.59340e+00_rb, 2.65277e+00_rb/)
      kao_mco( 7, :, 9) = (/ &
     & 2.03543e+00_rb, 2.05285e+00_rb, 2.07042e+00_rb, 2.08815e+00_rb, 2.10602e+00_rb, &
     & 2.12405e+00_rb, 2.14223e+00_rb, 2.16057e+00_rb, 2.17907e+00_rb, 2.19772e+00_rb, &
     & 2.21654e+00_rb, 2.23551e+00_rb, 2.25465e+00_rb, 2.27395e+00_rb, 2.29342e+00_rb, &
     & 2.31305e+00_rb, 2.33285e+00_rb, 2.35282e+00_rb, 2.37296e+00_rb/)
      kao_mco( 8, :, 9) = (/ &
     & 3.18883e+00_rb, 3.20538e+00_rb, 3.22200e+00_rb, 3.23872e+00_rb, 3.25552e+00_rb, &
     & 3.27241e+00_rb, 3.28939e+00_rb, 3.30645e+00_rb, 3.32360e+00_rb, 3.34085e+00_rb, &
     & 3.35818e+00_rb, 3.37560e+00_rb, 3.39311e+00_rb, 3.41071e+00_rb, 3.42841e+00_rb, &
     & 3.44619e+00_rb, 3.46407e+00_rb, 3.48204e+00_rb, 3.50011e+00_rb/)
      kao_mco( 9, :, 9) = (/ &
     & 3.97585e+00_rb, 3.96333e+00_rb, 3.95084e+00_rb, 3.93839e+00_rb, 3.92598e+00_rb, &
     & 3.91361e+00_rb, 3.90128e+00_rb, 3.88899e+00_rb, 3.87674e+00_rb, 3.86452e+00_rb, &
     & 3.85235e+00_rb, 3.84021e+00_rb, 3.82811e+00_rb, 3.81605e+00_rb, 3.80402e+00_rb, &
     & 3.79204e+00_rb, 3.78009e+00_rb, 3.76818e+00_rb, 3.75631e+00_rb/)
      kao_mco( 1, :,10) = (/ &
     & 8.62646e-01_rb, 9.35164e-01_rb, 1.01378e+00_rb, 1.09900e+00_rb, 1.19139e+00_rb, &
     & 1.29154e+00_rb, 1.40011e+00_rb, 1.51781e+00_rb, 1.64541e+00_rb, 1.78373e+00_rb, &
     & 1.93367e+00_rb, 2.09623e+00_rb, 2.27245e+00_rb, 2.46348e+00_rb, 2.67057e+00_rb, &
     & 2.89507e+00_rb, 3.13844e+00_rb, 3.40227e+00_rb, 3.68828e+00_rb/)
      kao_mco( 2, :,10) = (/ &
     & 8.04693e-01_rb, 8.72167e-01_rb, 9.45298e-01_rb, 1.02456e+00_rb, 1.11047e+00_rb, &
     & 1.20358e+00_rb, 1.30450e+00_rb, 1.41389e+00_rb, 1.53244e+00_rb, 1.66094e+00_rb, &
     & 1.80021e+00_rb, 1.95115e+00_rb, 2.11476e+00_rb, 2.29208e+00_rb, 2.48427e+00_rb, &
     & 2.69258e+00_rb, 2.91835e+00_rb, 3.16305e+00_rb, 3.42827e+00_rb/)
      kao_mco( 3, :,10) = (/ &
     & 7.66566e-01_rb, 8.24651e-01_rb, 8.87137e-01_rb, 9.54358e-01_rb, 1.02667e+00_rb, &
     & 1.10447e+00_rb, 1.18815e+00_rb, 1.27818e+00_rb, 1.37503e+00_rb, 1.47922e+00_rb, &
     & 1.59131e+00_rb, 1.71189e+00_rb, 1.84160e+00_rb, 1.98114e+00_rb, 2.13126e+00_rb, &
     & 2.29275e+00_rb, 2.46648e+00_rb, 2.65337e+00_rb, 2.85442e+00_rb/)
      kao_mco( 4, :,10) = (/ &
     & 5.40305e-01_rb, 5.77106e-01_rb, 6.16414e-01_rb, 6.58400e-01_rb, 7.03245e-01_rb, &
     & 7.51145e-01_rb, 8.02307e-01_rb, 8.56954e-01_rb, 9.15323e-01_rb, 9.77668e-01_rb, &
     & 1.04426e+00_rb, 1.11539e+00_rb, 1.19136e+00_rb, 1.27250e+00_rb, 1.35918e+00_rb, &
     & 1.45175e+00_rb, 1.55064e+00_rb, 1.65625e+00_rb, 1.76906e+00_rb/)
      kao_mco( 5, :,10) = (/ &
     & 8.22474e-01_rb, 8.02911e-01_rb, 7.83814e-01_rb, 7.65171e-01_rb, 7.46971e-01_rb, &
     & 7.29204e-01_rb, 7.11860e-01_rb, 6.94928e-01_rb, 6.78399e-01_rb, 6.62263e-01_rb, &
     & 6.46511e-01_rb, 6.31133e-01_rb, 6.16122e-01_rb, 6.01467e-01_rb, 5.87161e-01_rb, &
     & 5.73195e-01_rb, 5.59562e-01_rb, 5.46252e-01_rb, 5.33260e-01_rb/)
      kao_mco( 6, :,10) = (/ &
     & 1.28162e+00_rb, 1.25110e+00_rb, 1.22131e+00_rb, 1.19223e+00_rb, 1.16384e+00_rb, &
     & 1.13613e+00_rb, 1.10908e+00_rb, 1.08267e+00_rb, 1.05689e+00_rb, 1.03173e+00_rb, &
     & 1.00716e+00_rb, 9.83184e-01_rb, 9.59774e-01_rb, 9.36921e-01_rb, 9.14613e-01_rb, &
     & 8.92836e-01_rb, 8.71577e-01_rb, 8.50825e-01_rb, 8.30567e-01_rb/)
      kao_mco( 7, :,10) = (/ &
     & 1.92679e+00_rb, 1.90551e+00_rb, 1.88446e+00_rb, 1.86365e+00_rb, 1.84307e+00_rb, &
     & 1.82271e+00_rb, 1.80258e+00_rb, 1.78267e+00_rb, 1.76298e+00_rb, 1.74351e+00_rb, &
     & 1.72425e+00_rb, 1.70520e+00_rb, 1.68637e+00_rb, 1.66774e+00_rb, 1.64932e+00_rb, &
     & 1.63111e+00_rb, 1.61309e+00_rb, 1.59527e+00_rb, 1.57765e+00_rb/)
      kao_mco( 8, :,10) = (/ &
     & 4.66485e+00_rb, 4.60869e+00_rb, 4.55320e+00_rb, 4.49838e+00_rb, 4.44423e+00_rb, &
     & 4.39072e+00_rb, 4.33786e+00_rb, 4.28563e+00_rb, 4.23404e+00_rb, 4.18306e+00_rb, &
     & 4.13270e+00_rb, 4.08295e+00_rb, 4.03379e+00_rb, 3.98523e+00_rb, 3.93725e+00_rb, &
     & 3.88985e+00_rb, 3.84301e+00_rb, 3.79675e+00_rb, 3.75104e+00_rb/)
      kao_mco( 9, :,10) = (/ &
     & 1.41505e+00_rb, 1.37820e+00_rb, 1.34232e+00_rb, 1.30736e+00_rb, 1.27332e+00_rb, &
     & 1.24016e+00_rb, 1.20786e+00_rb, 1.17641e+00_rb, 1.14578e+00_rb, 1.11594e+00_rb, &
     & 1.08688e+00_rb, 1.05858e+00_rb, 1.03101e+00_rb, 1.00416e+00_rb, 9.78015e-01_rb, &
     & 9.52547e-01_rb, 9.27742e-01_rb, 9.03583e-01_rb, 8.80053e-01_rb/)
      kao_mco( 1, :,11) = (/ &
     & 3.40468e-03_rb, 4.05994e-03_rb, 4.84130e-03_rb, 5.77305e-03_rb, 6.88412e-03_rb, &
     & 8.20902e-03_rb, 9.78890e-03_rb, 1.16728e-02_rb, 1.39194e-02_rb, 1.65983e-02_rb, &
     & 1.97927e-02_rb, 2.36020e-02_rb, 2.81444e-02_rb, 3.35610e-02_rb, 4.00200e-02_rb, &
     & 4.77222e-02_rb, 5.69067e-02_rb, 6.78588e-02_rb, 8.09187e-02_rb/)
      kao_mco( 2, :,11) = (/ &
     & 3.85021e-02_rb, 4.02208e-02_rb, 4.20162e-02_rb, 4.38918e-02_rb, 4.58512e-02_rb, &
     & 4.78980e-02_rb, 5.00361e-02_rb, 5.22697e-02_rb, 5.46031e-02_rb, 5.70405e-02_rb, &
     & 5.95868e-02_rb, 6.22468e-02_rb, 6.50254e-02_rb, 6.79282e-02_rb, 7.09605e-02_rb, &
     & 7.41282e-02_rb, 7.74372e-02_rb, 8.08940e-02_rb, 8.45051e-02_rb/)
      kao_mco( 3, :,11) = (/ &
     & 5.24852e-01_rb, 5.10480e-01_rb, 4.96501e-01_rb, 4.82905e-01_rb, 4.69681e-01_rb, &
     & 4.56820e-01_rb, 4.44310e-01_rb, 4.32143e-01_rb, 4.20310e-01_rb, 4.08800e-01_rb, &
     & 3.97606e-01_rb, 3.86718e-01_rb, 3.76128e-01_rb, 3.65828e-01_rb, 3.55810e-01_rb, &
     & 3.46067e-01_rb, 3.36590e-01_rb, 3.27373e-01_rb, 3.18409e-01_rb/)
      kao_mco( 4, :,11) = (/ &
     & 5.86290e-01_rb, 5.70241e-01_rb, 5.54632e-01_rb, 5.39450e-01_rb, 5.24683e-01_rb, &
     & 5.10321e-01_rb, 4.96352e-01_rb, 4.82765e-01_rb, 4.69550e-01_rb, 4.56697e-01_rb, &
     & 4.44196e-01_rb, 4.32036e-01_rb, 4.20210e-01_rb, 4.08708e-01_rb, 3.97520e-01_rb, &
     & 3.86638e-01_rb, 3.76055e-01_rb, 3.65761e-01_rb, 3.55749e-01_rb/)
      kao_mco( 5, :,11) = (/ &
     & 1.66977e+00_rb, 1.61807e+00_rb, 1.56798e+00_rb, 1.51943e+00_rb, 1.47239e+00_rb, &
     & 1.42681e+00_rb, 1.38264e+00_rb, 1.33983e+00_rb, 1.29835e+00_rb, 1.25815e+00_rb, &
     & 1.21920e+00_rb, 1.18146e+00_rb, 1.14488e+00_rb, 1.10943e+00_rb, 1.07509e+00_rb, &
     & 1.04180e+00_rb, 1.00955e+00_rb, 9.78295e-01_rb, 9.48008e-01_rb/)
      kao_mco( 6, :,11) = (/ &
     & 1.96627e+00_rb, 1.90948e+00_rb, 1.85432e+00_rb, 1.80076e+00_rb, 1.74875e+00_rb, &
     & 1.69823e+00_rb, 1.64918e+00_rb, 1.60155e+00_rb, 1.55529e+00_rb, 1.51036e+00_rb, &
     & 1.46674e+00_rb, 1.42437e+00_rb, 1.38323e+00_rb, 1.34328e+00_rb, 1.30448e+00_rb, &
     & 1.26680e+00_rb, 1.23021e+00_rb, 1.19467e+00_rb, 1.16016e+00_rb/)
      kao_mco( 7, :,11) = (/ &
     & 1.67574e+00_rb, 1.63510e+00_rb, 1.59544e+00_rb, 1.55674e+00_rb, 1.51898e+00_rb, &
     & 1.48213e+00_rb, 1.44618e+00_rb, 1.41111e+00_rb, 1.37688e+00_rb, 1.34348e+00_rb, &
     & 1.31090e+00_rb, 1.27910e+00_rb, 1.24808e+00_rb, 1.21780e+00_rb, 1.18826e+00_rb, &
     & 1.15944e+00_rb, 1.13132e+00_rb, 1.10388e+00_rb, 1.07710e+00_rb/)
      kao_mco( 8, :,11) = (/ &
     & 2.00764e+00_rb, 1.96233e+00_rb, 1.91803e+00_rb, 1.87474e+00_rb, 1.83242e+00_rb, &
     & 1.79106e+00_rb, 1.75063e+00_rb, 1.71111e+00_rb, 1.67249e+00_rb, 1.63474e+00_rb, &
     & 1.59784e+00_rb, 1.56177e+00_rb, 1.52652e+00_rb, 1.49206e+00_rb, 1.45838e+00_rb, &
     & 1.42546e+00_rb, 1.39329e+00_rb, 1.36184e+00_rb, 1.33110e+00_rb/)
      kao_mco( 9, :,11) = (/ &
     & 1.83026e+00_rb, 1.77349e+00_rb, 1.71849e+00_rb, 1.66519e+00_rb, 1.61355e+00_rb, &
     & 1.56350e+00_rb, 1.51501e+00_rb, 1.46803e+00_rb, 1.42250e+00_rb, 1.37838e+00_rb, &
     & 1.33563e+00_rb, 1.29421e+00_rb, 1.25407e+00_rb, 1.21517e+00_rb, 1.17749e+00_rb, &
     & 1.14097e+00_rb, 1.10558e+00_rb, 1.07129e+00_rb, 1.03807e+00_rb/)
      kao_mco( 1, :,12) = (/ &
     & 3.90309e-04_rb, 4.81310e-04_rb, 5.93528e-04_rb, 7.31909e-04_rb, 9.02554e-04_rb, &
     & 1.11299e-03_rb, 1.37248e-03_rb, 1.69247e-03_rb, 2.08708e-03_rb, 2.57368e-03_rb, &
     & 3.17374e-03_rb, 3.91370e-03_rb, 4.82617e-03_rb, 5.95140e-03_rb, 7.33897e-03_rb, &
     & 9.05007e-03_rb, 1.11601e-02_rb, 1.37621e-02_rb, 1.69707e-02_rb/)
      kao_mco( 2, :,12) = (/ &
     & 9.80585e-02_rb, 9.77457e-02_rb, 9.74339e-02_rb, 9.71231e-02_rb, 9.68132e-02_rb, &
     & 9.65044e-02_rb, 9.61965e-02_rb, 9.58897e-02_rb, 9.55838e-02_rb, 9.52789e-02_rb, &
     & 9.49749e-02_rb, 9.46719e-02_rb, 9.43699e-02_rb, 9.40689e-02_rb, 9.37688e-02_rb, &
     & 9.34697e-02_rb, 9.31715e-02_rb, 9.28743e-02_rb, 9.25780e-02_rb/)
      kao_mco( 3, :,12) = (/ &
     & 3.15258e-01_rb, 3.09936e-01_rb, 3.04704e-01_rb, 2.99560e-01_rb, 2.94503e-01_rb, &
     & 2.89532e-01_rb, 2.84645e-01_rb, 2.79840e-01_rb, 2.75116e-01_rb, 2.70472e-01_rb, &
     & 2.65906e-01_rb, 2.61417e-01_rb, 2.57004e-01_rb, 2.52666e-01_rb, 2.48401e-01_rb, &
     & 2.44207e-01_rb, 2.40085e-01_rb, 2.36032e-01_rb, 2.32048e-01_rb/)
      kao_mco( 4, :,12) = (/ &
     & 9.74407e-01_rb, 9.46900e-01_rb, 9.20170e-01_rb, 8.94195e-01_rb, 8.68952e-01_rb, &
     & 8.44422e-01_rb, 8.20585e-01_rb, 7.97421e-01_rb, 7.74910e-01_rb, 7.53035e-01_rb, &
     & 7.31777e-01_rb, 7.11120e-01_rb, 6.91046e-01_rb, 6.71538e-01_rb, 6.52581e-01_rb, &
     & 6.34159e-01_rb, 6.16257e-01_rb, 5.98861e-01_rb, 5.81956e-01_rb/)
      kao_mco( 5, :,12) = (/ &
     & 1.04234e+00_rb, 1.01364e+00_rb, 9.85726e-01_rb, 9.58581e-01_rb, 9.32184e-01_rb, &
     & 9.06514e-01_rb, 8.81551e-01_rb, 8.57275e-01_rb, 8.33668e-01_rb, 8.10710e-01_rb, &
     & 7.88385e-01_rb, 7.66675e-01_rb, 7.45563e-01_rb, 7.25032e-01_rb, 7.05066e-01_rb, &
     & 6.85650e-01_rb, 6.66769e-01_rb, 6.48408e-01_rb, 6.30552e-01_rb/)
      kao_mco( 6, :,12) = (/ &
     & 1.79052e+00_rb, 1.73725e+00_rb, 1.68557e+00_rb, 1.63543e+00_rb, 1.58678e+00_rb, &
     & 1.53957e+00_rb, 1.49377e+00_rb, 1.44933e+00_rb, 1.40622e+00_rb, 1.36439e+00_rb, &
     & 1.32380e+00_rb, 1.28442e+00_rb, 1.24621e+00_rb, 1.20913e+00_rb, 1.17316e+00_rb, &
     & 1.13826e+00_rb, 1.10440e+00_rb, 1.07155e+00_rb, 1.03967e+00_rb/)
      kao_mco( 7, :,12) = (/ &
     & 2.99551e+00_rb, 2.90366e+00_rb, 2.81462e+00_rb, 2.72831e+00_rb, 2.64464e+00_rb, &
     & 2.56355e+00_rb, 2.48494e+00_rb, 2.40874e+00_rb, 2.33487e+00_rb, 2.26328e+00_rb, &
     & 2.19387e+00_rb, 2.12660e+00_rb, 2.06139e+00_rb, 1.99818e+00_rb, 1.93690e+00_rb, &
     & 1.87751e+00_rb, 1.81993e+00_rb, 1.76413e+00_rb, 1.71003e+00_rb/)
      kao_mco( 8, :,12) = (/ &
     & 2.89665e+00_rb, 2.81184e+00_rb, 2.72951e+00_rb, 2.64960e+00_rb, 2.57202e+00_rb, &
     & 2.49672e+00_rb, 2.42362e+00_rb, 2.35266e+00_rb, 2.28378e+00_rb, 2.21692e+00_rb, &
     & 2.15201e+00_rb, 2.08900e+00_rb, 2.02784e+00_rb, 1.96847e+00_rb, 1.91084e+00_rb, &
     & 1.85489e+00_rb, 1.80059e+00_rb, 1.74787e+00_rb, 1.69669e+00_rb/)
      kao_mco( 9, :,12) = (/ &
     & 1.03145e+00_rb, 1.00335e+00_rb, 9.76014e-01_rb, 9.49426e-01_rb, 9.23561e-01_rb, &
     & 8.98402e-01_rb, 8.73927e-01_rb, 8.50120e-01_rb, 8.26961e-01_rb, 8.04433e-01_rb, &
     & 7.82518e-01_rb, 7.61201e-01_rb, 7.40464e-01_rb, 7.20293e-01_rb, 7.00670e-01_rb, &
     & 6.81583e-01_rb, 6.63015e-01_rb, 6.44953e-01_rb, 6.27383e-01_rb/)
      kao_mco( 1, :,13) = (/ &
     & 5.27769e-04_rb, 6.65449e-04_rb, 8.39047e-04_rb, 1.05793e-03_rb, 1.33392e-03_rb, &
     & 1.68190e-03_rb, 2.12066e-03_rb, 2.67388e-03_rb, 3.37143e-03_rb, 4.25094e-03_rb, &
     & 5.35990e-03_rb, 6.75816e-03_rb, 8.52117e-03_rb, 1.07441e-02_rb, 1.35470e-02_rb, &
     & 1.70810e-02_rb, 2.15370e-02_rb, 2.71554e-02_rb, 3.42395e-02_rb/)
      kao_mco( 2, :,13) = (/ &
     & 1.08329e-01_rb, 1.10179e-01_rb, 1.12060e-01_rb, 1.13974e-01_rb, 1.15920e-01_rb, &
     & 1.17899e-01_rb, 1.19913e-01_rb, 1.21960e-01_rb, 1.24043e-01_rb, 1.26161e-01_rb, &
     & 1.28316e-01_rb, 1.30507e-01_rb, 1.32735e-01_rb, 1.35002e-01_rb, 1.37307e-01_rb, &
     & 1.39652e-01_rb, 1.42037e-01_rb, 1.44462e-01_rb, 1.46929e-01_rb/)
      kao_mco( 3, :,13) = (/ &
     & 1.95992e-01_rb, 1.94515e-01_rb, 1.93049e-01_rb, 1.91594e-01_rb, 1.90150e-01_rb, &
     & 1.88717e-01_rb, 1.87294e-01_rb, 1.85882e-01_rb, 1.84481e-01_rb, 1.83091e-01_rb, &
     & 1.81711e-01_rb, 1.80341e-01_rb, 1.78982e-01_rb, 1.77633e-01_rb, 1.76294e-01_rb, &
     & 1.74965e-01_rb, 1.73646e-01_rb, 1.72337e-01_rb, 1.71038e-01_rb/)
      kao_mco( 4, :,13) = (/ &
     & 4.49766e-01_rb, 4.42749e-01_rb, 4.35841e-01_rb, 4.29042e-01_rb, 4.22348e-01_rb, &
     & 4.15759e-01_rb, 4.09272e-01_rb, 4.02887e-01_rb, 3.96601e-01_rb, 3.90414e-01_rb, &
     & 3.84323e-01_rb, 3.78327e-01_rb, 3.72424e-01_rb, 3.66614e-01_rb, 3.60894e-01_rb, &
     & 3.55264e-01_rb, 3.49721e-01_rb, 3.44265e-01_rb, 3.38894e-01_rb/)
      kao_mco( 5, :,13) = (/ &
     & 1.07498e+00_rb, 1.04736e+00_rb, 1.02045e+00_rb, 9.94232e-01_rb, 9.68686e-01_rb, &
     & 9.43797e-01_rb, 9.19547e-01_rb, 8.95920e-01_rb, 8.72900e-01_rb, 8.50471e-01_rb, &
     & 8.28619e-01_rb, 8.07329e-01_rb, 7.86585e-01_rb, 7.66374e-01_rb, 7.46683e-01_rb, &
     & 7.27498e-01_rb, 7.08805e-01_rb, 6.90593e-01_rb, 6.72849e-01_rb/)
      kao_mco( 6, :,13) = (/ &
     & 1.66569e+00_rb, 1.61740e+00_rb, 1.57052e+00_rb, 1.52500e+00_rb, 1.48080e+00_rb, &
     & 1.43787e+00_rb, 1.39620e+00_rb, 1.35573e+00_rb, 1.31643e+00_rb, 1.27827e+00_rb, &
     & 1.24122e+00_rb, 1.20524e+00_rb, 1.17031e+00_rb, 1.13638e+00_rb, 1.10344e+00_rb, &
     & 1.07146e+00_rb, 1.04040e+00_rb, 1.01025e+00_rb, 9.80963e-01_rb/)
      kao_mco( 7, :,13) = (/ &
     & 1.52948e+00_rb, 1.48763e+00_rb, 1.44693e+00_rb, 1.40735e+00_rb, 1.36884e+00_rb, &
     & 1.33139e+00_rb, 1.29497e+00_rb, 1.25954e+00_rb, 1.22508e+00_rb, 1.19156e+00_rb, &
     & 1.15896e+00_rb, 1.12725e+00_rb, 1.09641e+00_rb, 1.06641e+00_rb, 1.03724e+00_rb, &
     & 1.00886e+00_rb, 9.81259e-01_rb, 9.54412e-01_rb, 9.28301e-01_rb/)
      kao_mco( 8, :,13) = (/ &
     & 3.81027e+00_rb, 3.69422e+00_rb, 3.58170e+00_rb, 3.47261e+00_rb, 3.36684e+00_rb, &
     & 3.26429e+00_rb, 3.16486e+00_rb, 3.06847e+00_rb, 2.97501e+00_rb, 2.88439e+00_rb, &
     & 2.79654e+00_rb, 2.71136e+00_rb, 2.62878e+00_rb, 2.54871e+00_rb, 2.47108e+00_rb, &
     & 2.39581e+00_rb, 2.32284e+00_rb, 2.25209e+00_rb, 2.18350e+00_rb/)
      kao_mco( 9, :,13) = (/ &
     & 1.12516e+00_rb, 1.09531e+00_rb, 1.06625e+00_rb, 1.03796e+00_rb, 1.01042e+00_rb, &
     & 9.83616e-01_rb, 9.57520e-01_rb, 9.32117e-01_rb, 9.07387e-01_rb, 8.83314e-01_rb, &
     & 8.59880e-01_rb, 8.37067e-01_rb, 8.14859e-01_rb, 7.93241e-01_rb, 7.72196e-01_rb, &
     & 7.51709e-01_rb, 7.31766e-01_rb, 7.12352e-01_rb, 6.93454e-01_rb/)
      kao_mco( 1, :,14) = (/ &
     & 4.79165e-04_rb, 6.26966e-04_rb, 8.20356e-04_rb, 1.07340e-03_rb, 1.40450e-03_rb, &
     & 1.83772e-03_rb, 2.40457e-03_rb, 3.14628e-03_rb, 4.11676e-03_rb, 5.38660e-03_rb, &
     & 7.04813e-03_rb, 9.22216e-03_rb, 1.20668e-02_rb, 1.57889e-02_rb, 2.06590e-02_rb, &
     & 2.70314e-02_rb, 3.53694e-02_rb, 4.62792e-02_rb, 6.05542e-02_rb/)
      kao_mco( 2, :,14) = (/ &
     & 4.75367e-04_rb, 6.21987e-04_rb, 8.13830e-04_rb, 1.06484e-03_rb, 1.39328e-03_rb, &
     & 1.82302e-03_rb, 2.38530e-03_rb, 3.12101e-03_rb, 4.08365e-03_rb, 5.34318e-03_rb, &
     & 6.99121e-03_rb, 9.14756e-03_rb, 1.19690e-02_rb, 1.56607e-02_rb, 2.04909e-02_rb, &
     & 2.68111e-02_rb, 3.50806e-02_rb, 4.59007e-02_rb, 6.00580e-02_rb/)
      kao_mco( 3, :,14) = (/ &
     & 1.39594e-01_rb, 1.42407e-01_rb, 1.45276e-01_rb, 1.48202e-01_rb, 1.51188e-01_rb, &
     & 1.54234e-01_rb, 1.57342e-01_rb, 1.60512e-01_rb, 1.63745e-01_rb, 1.67044e-01_rb, &
     & 1.70410e-01_rb, 1.73843e-01_rb, 1.77345e-01_rb, 1.80918e-01_rb, 1.84563e-01_rb, &
     & 1.88282e-01_rb, 1.92075e-01_rb, 1.95945e-01_rb, 1.99892e-01_rb/)
      kao_mco( 4, :,14) = (/ &
     & 2.73418e-01_rb, 2.71322e-01_rb, 2.69242e-01_rb, 2.67177e-01_rb, 2.65129e-01_rb, &
     & 2.63096e-01_rb, 2.61079e-01_rb, 2.59077e-01_rb, 2.57091e-01_rb, 2.55120e-01_rb, &
     & 2.53164e-01_rb, 2.51223e-01_rb, 2.49297e-01_rb, 2.47386e-01_rb, 2.45489e-01_rb, &
     & 2.43607e-01_rb, 2.41739e-01_rb, 2.39886e-01_rb, 2.38047e-01_rb/)
      kao_mco( 5, :,14) = (/ &
     & 5.01880e-01_rb, 4.95066e-01_rb, 4.88344e-01_rb, 4.81713e-01_rb, 4.75173e-01_rb, &
     & 4.68721e-01_rb, 4.62357e-01_rb, 4.56079e-01_rb, 4.49887e-01_rb, 4.43778e-01_rb, &
     & 4.37753e-01_rb, 4.31809e-01_rb, 4.25946e-01_rb, 4.20163e-01_rb, 4.14458e-01_rb, &
     & 4.08830e-01_rb, 4.03279e-01_rb, 3.97804e-01_rb, 3.92403e-01_rb/)
      kao_mco( 6, :,14) = (/ &
     & 9.46125e-01_rb, 9.29642e-01_rb, 9.13447e-01_rb, 8.97533e-01_rb, 8.81897e-01_rb, &
     & 8.66533e-01_rb, 8.51437e-01_rb, 8.36603e-01_rb, 8.22029e-01_rb, 8.07708e-01_rb, &
     & 7.93636e-01_rb, 7.79810e-01_rb, 7.66225e-01_rb, 7.52876e-01_rb, 7.39760e-01_rb, &
     & 7.26872e-01_rb, 7.14209e-01_rb, 7.01766e-01_rb, 6.89540e-01_rb/)
      kao_mco( 7, :,14) = (/ &
     & 2.47697e+00_rb, 2.41183e+00_rb, 2.34840e+00_rb, 2.28664e+00_rb, 2.22650e+00_rb, &
     & 2.16795e+00_rb, 2.11093e+00_rb, 2.05541e+00_rb, 2.00136e+00_rb, 1.94872e+00_rb, &
     & 1.89747e+00_rb, 1.84757e+00_rb, 1.79898e+00_rb, 1.75167e+00_rb, 1.70560e+00_rb, &
     & 1.66074e+00_rb, 1.61707e+00_rb, 1.57454e+00_rb, 1.53313e+00_rb/)
      kao_mco( 8, :,14) = (/ &
     & 2.19323e+00_rb, 2.13926e+00_rb, 2.08662e+00_rb, 2.03528e+00_rb, 1.98519e+00_rb, &
     & 1.93635e+00_rb, 1.88870e+00_rb, 1.84222e+00_rb, 1.79689e+00_rb, 1.75268e+00_rb, &
     & 1.70955e+00_rb, 1.66748e+00_rb, 1.62645e+00_rb, 1.58643e+00_rb, 1.54739e+00_rb, &
     & 1.50932e+00_rb, 1.47218e+00_rb, 1.43595e+00_rb, 1.40062e+00_rb/)
      kao_mco( 9, :,14) = (/ &
     & 5.68351e-01_rb, 5.60360e-01_rb, 5.52481e-01_rb, 5.44712e-01_rb, 5.37053e-01_rb, &
     & 5.29502e-01_rb, 5.22057e-01_rb, 5.14716e-01_rb, 5.07479e-01_rb, 5.00343e-01_rb, &
     & 4.93308e-01_rb, 4.86372e-01_rb, 4.79533e-01_rb, 4.72790e-01_rb, 4.66142e-01_rb, &
     & 4.59588e-01_rb, 4.53126e-01_rb, 4.46754e-01_rb, 4.40473e-01_rb/)
      kao_mco( 1, :,15) = (/ &
     & 7.87937e-04_rb, 1.01733e-03_rb, 1.31351e-03_rb, 1.69591e-03_rb, 2.18964e-03_rb, &
     & 2.82712e-03_rb, 3.65018e-03_rb, 4.71286e-03_rb, 6.08493e-03_rb, 7.85644e-03_rb, &
     & 1.01437e-02_rb, 1.30969e-02_rb, 1.69098e-02_rb, 2.18327e-02_rb, 2.81889e-02_rb, &
     & 3.63956e-02_rb, 4.69915e-02_rb, 6.06722e-02_rb, 7.83357e-02_rb/)
      kao_mco( 2, :,15) = (/ &
     & 7.87937e-04_rb, 1.01733e-03_rb, 1.31351e-03_rb, 1.69591e-03_rb, 2.18964e-03_rb, &
     & 2.82712e-03_rb, 3.65018e-03_rb, 4.71286e-03_rb, 6.08493e-03_rb, 7.85644e-03_rb, &
     & 1.01437e-02_rb, 1.30969e-02_rb, 1.69098e-02_rb, 2.18327e-02_rb, 2.81889e-02_rb, &
     & 3.63956e-02_rb, 4.69915e-02_rb, 6.06722e-02_rb, 7.83357e-02_rb/)
      kao_mco( 3, :,15) = (/ &
     & 1.97281e-01_rb, 2.00714e-01_rb, 2.04206e-01_rb, 2.07760e-01_rb, 2.11375e-01_rb, &
     & 2.15053e-01_rb, 2.18795e-01_rb, 2.22603e-01_rb, 2.26476e-01_rb, 2.30417e-01_rb, &
     & 2.34426e-01_rb, 2.38506e-01_rb, 2.42656e-01_rb, 2.46878e-01_rb, 2.51174e-01_rb, &
     & 2.55545e-01_rb, 2.59992e-01_rb, 2.64516e-01_rb, 2.69119e-01_rb/)
      kao_mco( 4, :,15) = (/ &
     & 4.47509e-01_rb, 4.52222e-01_rb, 4.56985e-01_rb, 4.61799e-01_rb, 4.66663e-01_rb, &
     & 4.71578e-01_rb, 4.76545e-01_rb, 4.81565e-01_rb, 4.86637e-01_rb, 4.91763e-01_rb, &
     & 4.96942e-01_rb, 5.02177e-01_rb, 5.07466e-01_rb, 5.12811e-01_rb, 5.18213e-01_rb, &
     & 5.23671e-01_rb, 5.29187e-01_rb, 5.34761e-01_rb, 5.40393e-01_rb/)
      kao_mco( 5, :,15) = (/ &
     & 1.02732e+00_rb, 1.02091e+00_rb, 1.01453e+00_rb, 1.00820e+00_rb, 1.00191e+00_rb, &
     & 9.95660e-01_rb, 9.89447e-01_rb, 9.83273e-01_rb, 9.77137e-01_rb, 9.71039e-01_rb, &
     & 9.64980e-01_rb, 9.58958e-01_rb, 9.52974e-01_rb, 9.47027e-01_rb, 9.41118e-01_rb, &
     & 9.35245e-01_rb, 9.29409e-01_rb, 9.23609e-01_rb, 9.17846e-01_rb/)
      kao_mco( 6, :,15) = (/ &
     & 1.13766e+00_rb, 1.12944e+00_rb, 1.12128e+00_rb, 1.11318e+00_rb, 1.10514e+00_rb, &
     & 1.09715e+00_rb, 1.08923e+00_rb, 1.08136e+00_rb, 1.07355e+00_rb, 1.06579e+00_rb, &
     & 1.05809e+00_rb, 1.05045e+00_rb, 1.04286e+00_rb, 1.03532e+00_rb, 1.02784e+00_rb, &
     & 1.02042e+00_rb, 1.01305e+00_rb, 1.00573e+00_rb, 9.98462e-01_rb/)
      kao_mco( 7, :,15) = (/ &
     & 1.13268e+00_rb, 1.12496e+00_rb, 1.11730e+00_rb, 1.10969e+00_rb, 1.10213e+00_rb, &
     & 1.09462e+00_rb, 1.08717e+00_rb, 1.07976e+00_rb, 1.07241e+00_rb, 1.06510e+00_rb, &
     & 1.05785e+00_rb, 1.05064e+00_rb, 1.04349e+00_rb, 1.03638e+00_rb, 1.02932e+00_rb, &
     & 1.02231e+00_rb, 1.01535e+00_rb, 1.00843e+00_rb, 1.00156e+00_rb/)
      kao_mco( 8, :,15) = (/ &
     & 1.11982e+00_rb, 1.11343e+00_rb, 1.10707e+00_rb, 1.10075e+00_rb, 1.09447e+00_rb, &
     & 1.08822e+00_rb, 1.08201e+00_rb, 1.07583e+00_rb, 1.06969e+00_rb, 1.06358e+00_rb, &
     & 1.05751e+00_rb, 1.05147e+00_rb, 1.04547e+00_rb, 1.03950e+00_rb, 1.03356e+00_rb, &
     & 1.02766e+00_rb, 1.02180e+00_rb, 1.01596e+00_rb, 1.01016e+00_rb/)
      kao_mco( 9, :,15) = (/ &
     & 1.03561e+00_rb, 1.02902e+00_rb, 1.02246e+00_rb, 1.01595e+00_rb, 1.00948e+00_rb, &
     & 1.00305e+00_rb, 9.96667e-01_rb, 9.90320e-01_rb, 9.84013e-01_rb, 9.77747e-01_rb, &
     & 9.71520e-01_rb, 9.65334e-01_rb, 9.59186e-01_rb, 9.53078e-01_rb, 9.47008e-01_rb, &
     & 9.40978e-01_rb, 9.34985e-01_rb, 9.29031e-01_rb, 9.23115e-01_rb/)
      kao_mco( 1, :,16) = (/ &
     & 1.22217e-03_rb, 1.56836e-03_rb, 2.01261e-03_rb, 2.58271e-03_rb, 3.31429e-03_rb, &
     & 4.25310e-03_rb, 5.45784e-03_rb, 7.00383e-03_rb, 8.98775e-03_rb, 1.15336e-02_rb, &
     & 1.48007e-02_rb, 1.89931e-02_rb, 2.43731e-02_rb, 3.12771e-02_rb, 4.01367e-02_rb, &
     & 5.15059e-02_rb, 6.60956e-02_rb, 8.48179e-02_rb, 1.08843e-01_rb/)
      kao_mco( 2, :,16) = (/ &
     & 1.22217e-03_rb, 1.56836e-03_rb, 2.01261e-03_rb, 2.58271e-03_rb, 3.31429e-03_rb, &
     & 4.25310e-03_rb, 5.45784e-03_rb, 7.00383e-03_rb, 8.98775e-03_rb, 1.15336e-02_rb, &
     & 1.48007e-02_rb, 1.89931e-02_rb, 2.43731e-02_rb, 3.12771e-02_rb, 4.01367e-02_rb, &
     & 5.15059e-02_rb, 6.60956e-02_rb, 8.48179e-02_rb, 1.08843e-01_rb/)
      kao_mco( 3, :,16) = (/ &
     & 1.22217e-03_rb, 1.56836e-03_rb, 2.01261e-03_rb, 2.58271e-03_rb, 3.31429e-03_rb, &
     & 4.25310e-03_rb, 5.45784e-03_rb, 7.00383e-03_rb, 8.98775e-03_rb, 1.15336e-02_rb, &
     & 1.48007e-02_rb, 1.89931e-02_rb, 2.43731e-02_rb, 3.12771e-02_rb, 4.01367e-02_rb, &
     & 5.15059e-02_rb, 6.60956e-02_rb, 8.48179e-02_rb, 1.08843e-01_rb/)
      kao_mco( 4, :,16) = (/ &
     & 1.01221e+00_rb, 1.01660e+00_rb, 1.02101e+00_rb, 1.02544e+00_rb, 1.02989e+00_rb, &
     & 1.03436e+00_rb, 1.03884e+00_rb, 1.04335e+00_rb, 1.04788e+00_rb, 1.05243e+00_rb, &
     & 1.05699e+00_rb, 1.06158e+00_rb, 1.06619e+00_rb, 1.07081e+00_rb, 1.07546e+00_rb, &
     & 1.08012e+00_rb, 1.08481e+00_rb, 1.08952e+00_rb, 1.09425e+00_rb/)
      kao_mco( 5, :,16) = (/ &
     & 1.01221e+00_rb, 1.01660e+00_rb, 1.02101e+00_rb, 1.02544e+00_rb, 1.02989e+00_rb, &
     & 1.03436e+00_rb, 1.03884e+00_rb, 1.04335e+00_rb, 1.04788e+00_rb, 1.05243e+00_rb, &
     & 1.05699e+00_rb, 1.06158e+00_rb, 1.06619e+00_rb, 1.07081e+00_rb, 1.07546e+00_rb, &
     & 1.08012e+00_rb, 1.08481e+00_rb, 1.08952e+00_rb, 1.09425e+00_rb/)
      kao_mco( 6, :,16) = (/ &
     & 1.01221e+00_rb, 1.01660e+00_rb, 1.02101e+00_rb, 1.02544e+00_rb, 1.02989e+00_rb, &
     & 1.03436e+00_rb, 1.03884e+00_rb, 1.04335e+00_rb, 1.04788e+00_rb, 1.05243e+00_rb, &
     & 1.05699e+00_rb, 1.06158e+00_rb, 1.06619e+00_rb, 1.07081e+00_rb, 1.07546e+00_rb, &
     & 1.08012e+00_rb, 1.08481e+00_rb, 1.08952e+00_rb, 1.09425e+00_rb/)
      kao_mco( 7, :,16) = (/ &
     & 1.01221e+00_rb, 1.01660e+00_rb, 1.02101e+00_rb, 1.02544e+00_rb, 1.02989e+00_rb, &
     & 1.03436e+00_rb, 1.03884e+00_rb, 1.04335e+00_rb, 1.04788e+00_rb, 1.05243e+00_rb, &
     & 1.05699e+00_rb, 1.06158e+00_rb, 1.06619e+00_rb, 1.07081e+00_rb, 1.07546e+00_rb, &
     & 1.08012e+00_rb, 1.08481e+00_rb, 1.08952e+00_rb, 1.09425e+00_rb/)
      kao_mco( 8, :,16) = (/ &
     & 1.01221e+00_rb, 1.01660e+00_rb, 1.02101e+00_rb, 1.02544e+00_rb, 1.02989e+00_rb, &
     & 1.03436e+00_rb, 1.03884e+00_rb, 1.04335e+00_rb, 1.04788e+00_rb, 1.05243e+00_rb, &
     & 1.05699e+00_rb, 1.06158e+00_rb, 1.06619e+00_rb, 1.07081e+00_rb, 1.07546e+00_rb, &
     & 1.08012e+00_rb, 1.08481e+00_rb, 1.08952e+00_rb, 1.09425e+00_rb/)
      kao_mco( 9, :,16) = (/ &
     & 1.01221e+00_rb, 1.01660e+00_rb, 1.02101e+00_rb, 1.02544e+00_rb, 1.02989e+00_rb, &
     & 1.03436e+00_rb, 1.03884e+00_rb, 1.04335e+00_rb, 1.04788e+00_rb, 1.05243e+00_rb, &
     & 1.05699e+00_rb, 1.06158e+00_rb, 1.06619e+00_rb, 1.07081e+00_rb, 1.07546e+00_rb, &
     & 1.08012e+00_rb, 1.08481e+00_rb, 1.08952e+00_rb, 1.09425e+00_rb/)

!     The array KBO_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level above 100~ mb.   The first index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The second index 
!     runs over the g-channel (1 to 16).

      kbo_mo3(:, 1) = (/ &
     & 1.07596e-02_rb, 1.12146e-02_rb, 1.16887e-02_rb, 1.21830e-02_rb, 1.26981e-02_rb, &
     & 1.32350e-02_rb, 1.37946e-02_rb, 1.43779e-02_rb, 1.49858e-02_rb, 1.56194e-02_rb, &
     & 1.62799e-02_rb, 1.69682e-02_rb, 1.76857e-02_rb, 1.84334e-02_rb, 1.92129e-02_rb, &
     & 2.00252e-02_rb, 2.08719e-02_rb, 2.17544e-02_rb, 2.26743e-02_rb/)
      kbo_mo3(:, 2) = (/ &
     & 9.48276e-02_rb, 9.66591e-02_rb, 9.85260e-02_rb, 1.00429e-01_rb, 1.02369e-01_rb, &
     & 1.04346e-01_rb, 1.06361e-01_rb, 1.08416e-01_rb, 1.10510e-01_rb, 1.12644e-01_rb, &
     & 1.14820e-01_rb, 1.17037e-01_rb, 1.19298e-01_rb, 1.21602e-01_rb, 1.23951e-01_rb, &
     & 1.26345e-01_rb, 1.28785e-01_rb, 1.31273e-01_rb, 1.33808e-01_rb/)
      kbo_mo3(:, 3) = (/ &
     & 3.54721e-01_rb, 3.55779e-01_rb, 3.56841e-01_rb, 3.57906e-01_rb, 3.58973e-01_rb, &
     & 3.60044e-01_rb, 3.61119e-01_rb, 3.62196e-01_rb, 3.63277e-01_rb, 3.64360e-01_rb, &
     & 3.65448e-01_rb, 3.66538e-01_rb, 3.67631e-01_rb, 3.68728e-01_rb, 3.69828e-01_rb, &
     & 3.70932e-01_rb, 3.72038e-01_rb, 3.73148e-01_rb, 3.74262e-01_rb/)
      kbo_mo3(:, 4) = (/ &
     & 6.46454e-01_rb, 6.43823e-01_rb, 6.41202e-01_rb, 6.38593e-01_rb, 6.35994e-01_rb, &
     & 6.33405e-01_rb, 6.30827e-01_rb, 6.28260e-01_rb, 6.25703e-01_rb, 6.23156e-01_rb, &
     & 6.20620e-01_rb, 6.18094e-01_rb, 6.15578e-01_rb, 6.13073e-01_rb, 6.10578e-01_rb, &
     & 6.08093e-01_rb, 6.05618e-01_rb, 6.03153e-01_rb, 6.00698e-01_rb/)
      kbo_mo3(:, 5) = (/ &
     & 9.29832e-01_rb, 9.22877e-01_rb, 9.15975e-01_rb, 9.09124e-01_rb, 9.02324e-01_rb, &
     & 8.95576e-01_rb, 8.88877e-01_rb, 8.82229e-01_rb, 8.75631e-01_rb, 8.69082e-01_rb, &
     & 8.62582e-01_rb, 8.56130e-01_rb, 8.49727e-01_rb, 8.43372e-01_rb, 8.37064e-01_rb, &
     & 8.30803e-01_rb, 8.24589e-01_rb, 8.18422e-01_rb, 8.12301e-01_rb/)
      kbo_mo3(:, 6) = (/ &
     & 1.43531e+00_rb, 1.42616e+00_rb, 1.41706e+00_rb, 1.40802e+00_rb, 1.39903e+00_rb, &
     & 1.39010e+00_rb, 1.38124e+00_rb, 1.37242e+00_rb, 1.36367e+00_rb, 1.35496e+00_rb, &
     & 1.34632e+00_rb, 1.33773e+00_rb, 1.32919e+00_rb, 1.32071e+00_rb, 1.31229e+00_rb, &
     & 1.30391e+00_rb, 1.29559e+00_rb, 1.28733e+00_rb, 1.27911e+00_rb/)
      kbo_mo3(:, 7) = (/ &
     & 2.68664e+00_rb, 2.67196e+00_rb, 2.65736e+00_rb, 2.64284e+00_rb, 2.62840e+00_rb, &
     & 2.61404e+00_rb, 2.59975e+00_rb, 2.58555e+00_rb, 2.57142e+00_rb, 2.55737e+00_rb, &
     & 2.54340e+00_rb, 2.52950e+00_rb, 2.51568e+00_rb, 2.50193e+00_rb, 2.48826e+00_rb, &
     & 2.47466e+00_rb, 2.46114e+00_rb, 2.44769e+00_rb, 2.43432e+00_rb/)
      kbo_mo3(:, 8) = (/ &
     & 2.45343e+00_rb, 2.43442e+00_rb, 2.41556e+00_rb, 2.39684e+00_rb, 2.37827e+00_rb, &
     & 2.35984e+00_rb, 2.34156e+00_rb, 2.32342e+00_rb, 2.30541e+00_rb, 2.28755e+00_rb, &
     & 2.26983e+00_rb, 2.25224e+00_rb, 2.23479e+00_rb, 2.21747e+00_rb, 2.20029e+00_rb, &
     & 2.18324e+00_rb, 2.16633e+00_rb, 2.14954e+00_rb, 2.13289e+00_rb/)
      kbo_mo3(:, 9) = (/ &
     & 1.55879e-01_rb, 1.55998e-01_rb, 1.56118e-01_rb, 1.56238e-01_rb, 1.56358e-01_rb, &
     & 1.56478e-01_rb, 1.56599e-01_rb, 1.56719e-01_rb, 1.56840e-01_rb, 1.56960e-01_rb, &
     & 1.57081e-01_rb, 1.57201e-01_rb, 1.57322e-01_rb, 1.57443e-01_rb, 1.57564e-01_rb, &
     & 1.57685e-01_rb, 1.57806e-01_rb, 1.57928e-01_rb, 1.58049e-01_rb/)
      kbo_mo3(:,10) = (/ &
     & 8.75149e-03_rb, 8.88794e-03_rb, 9.02651e-03_rb, 9.16725e-03_rb, 9.31018e-03_rb, &
     & 9.45534e-03_rb, 9.60276e-03_rb, 9.75248e-03_rb, 9.90454e-03_rb, 1.00590e-02_rb, &
     & 1.02158e-02_rb, 1.03751e-02_rb, 1.05368e-02_rb, 1.07011e-02_rb, 1.08680e-02_rb, &
     & 1.10374e-02_rb, 1.12095e-02_rb, 1.13843e-02_rb, 1.15618e-02_rb/)
      kbo_mo3(:,11) = (/ &
     & 8.83874e-03_rb, 8.97926e-03_rb, 9.12201e-03_rb, 9.26703e-03_rb, 9.41436e-03_rb, &
     & 9.56403e-03_rb, 9.71608e-03_rb, 9.87055e-03_rb, 1.00275e-02_rb, 1.01869e-02_rb, &
     & 1.03488e-02_rb, 1.05134e-02_rb, 1.06805e-02_rb, 1.08503e-02_rb, 1.10228e-02_rb, &
     & 1.11980e-02_rb, 1.13761e-02_rb, 1.15569e-02_rb, 1.17407e-02_rb/)
      kbo_mo3(:,12) = (/ &
     & 9.59461e-03_rb, 9.70417e-03_rb, 9.81498e-03_rb, 9.92705e-03_rb, 1.00404e-02_rb, &
     & 1.01550e-02_rb, 1.02710e-02_rb, 1.03883e-02_rb, 1.05069e-02_rb, 1.06269e-02_rb, &
     & 1.07482e-02_rb, 1.08709e-02_rb, 1.09951e-02_rb, 1.11206e-02_rb, 1.12476e-02_rb, &
     & 1.13760e-02_rb, 1.15059e-02_rb, 1.16373e-02_rb, 1.17702e-02_rb/)
      kbo_mo3(:,13) = (/ &
     & 1.13077e-02_rb, 1.14079e-02_rb, 1.15089e-02_rb, 1.16109e-02_rb, 1.17138e-02_rb, &
     & 1.18176e-02_rb, 1.19223e-02_rb, 1.20279e-02_rb, 1.21344e-02_rb, 1.22419e-02_rb, &
     & 1.23504e-02_rb, 1.24598e-02_rb, 1.25702e-02_rb, 1.26816e-02_rb, 1.27939e-02_rb, &
     & 1.29073e-02_rb, 1.30216e-02_rb, 1.31370e-02_rb, 1.32534e-02_rb/)
      kbo_mo3(:,14) = (/ &
     & 6.74844e-03_rb, 6.82637e-03_rb, 6.90519e-03_rb, 6.98493e-03_rb, 7.06558e-03_rb, &
     & 7.14717e-03_rb, 7.22970e-03_rb, 7.31318e-03_rb, 7.39762e-03_rb, 7.48304e-03_rb, &
     & 7.56945e-03_rb, 7.65686e-03_rb, 7.74527e-03_rb, 7.83470e-03_rb, 7.92517e-03_rb, &
     & 8.01668e-03_rb, 8.10925e-03_rb, 8.20289e-03_rb, 8.29761e-03_rb/)
      kbo_mo3(:,15) = (/ &
     & 7.94595e-03_rb, 8.00015e-03_rb, 8.05472e-03_rb, 8.10966e-03_rb, 8.16497e-03_rb, &
     & 8.22067e-03_rb, 8.27674e-03_rb, 8.33320e-03_rb, 8.39004e-03_rb, 8.44727e-03_rb, &
     & 8.50489e-03_rb, 8.56290e-03_rb, 8.62130e-03_rb, 8.68011e-03_rb, 8.73932e-03_rb, &
     & 8.79893e-03_rb, 8.85895e-03_rb, 8.91937e-03_rb, 8.98021e-03_rb/)
      kbo_mo3(:,16) = (/ &
     & 1.85967e-03_rb, 1.86082e-03_rb, 1.86197e-03_rb, 1.86312e-03_rb, 1.86428e-03_rb, &
     & 1.86543e-03_rb, 1.86658e-03_rb, 1.86774e-03_rb, 1.86889e-03_rb, 1.87005e-03_rb, &
     & 1.87121e-03_rb, 1.87236e-03_rb, 1.87352e-03_rb, 1.87468e-03_rb, 1.87584e-03_rb, &
     & 1.87700e-03_rb, 1.87816e-03_rb, 1.87932e-03_rb, 1.88049e-03_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &1.6586e-05_rb,1.9995e-05_rb,1.8582e-05_rb,1.3988e-05_rb,1.3650e-05_rb,1.1079e-05_rb, &
     &9.5855e-06_rb,8.4062e-06_rb,1.3558e-05_rb,1.8620e-05_rb,2.2652e-05_rb,1.7883e-05_rb, &
     &2.6241e-05_rb,3.1171e-05_rb,3.9386e-05_rb,4.4415e-05_rb/)
      forrefo(2,:) = (/ &
     &2.0730e-05_rb,2.3258e-05_rb,2.1543e-05_rb,1.5660e-05_rb,9.7872e-06_rb,8.1078e-06_rb, &
     &7.0246e-06_rb,6.0428e-06_rb,4.8793e-06_rb,4.4937e-06_rb,4.7078e-06_rb,4.6898e-06_rb, &
     &6.9481e-06_rb,8.6269e-06_rb,3.1761e-06_rb,3.1440e-06_rb/)
      forrefo(3,:) = (/ &
     &1.5737e-05_rb,2.2501e-05_rb,2.3520e-05_rb,2.0288e-05_rb,1.2083e-05_rb,6.8256e-06_rb, &
     &6.0637e-06_rb,5.5434e-06_rb,4.3888e-06_rb,3.8435e-06_rb,3.8477e-06_rb,3.8314e-06_rb, &
     &3.8251e-06_rb,3.3637e-06_rb,3.1950e-06_rb,3.1440e-06_rb/)
      forrefo(4,:) = (/ &
     &1.1400e-05_rb,7.9751e-06_rb,8.8659e-06_rb,1.5884e-05_rb,1.9118e-05_rb,1.9429e-05_rb, &
     &2.0532e-05_rb,2.2155e-05_rb,2.3894e-05_rb,2.2984e-05_rb,2.3731e-05_rb,2.4538e-05_rb, &
     &2.6697e-05_rb,1.9329e-05_rb,3.3306e-06_rb,3.2018e-06_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 9.62275e-03_rb, 8.29909e-03_rb, 7.15750e-03_rb, 6.17294e-03_rb, 5.32382e-03_rb, &
     & 4.59150e-03_rb, 3.95991e-03_rb, 3.41520e-03_rb, 2.94542e-03_rb, 2.54026e-03_rb/)
      selfrefo(:, 2) = (/ &
     & 9.76664e-03_rb, 8.47783e-03_rb, 7.35910e-03_rb, 6.38799e-03_rb, 5.54504e-03_rb, &
     & 4.81331e-03_rb, 4.17815e-03_rb, 3.62680e-03_rb, 3.14821e-03_rb, 2.73277e-03_rb/)
      selfrefo(:, 3) = (/ &
     & 9.53856e-03_rb, 8.23750e-03_rb, 7.11390e-03_rb, 6.14356e-03_rb, 5.30558e-03_rb, &
     & 4.58190e-03_rb, 3.95693e-03_rb, 3.41720e-03_rb, 2.95109e-03_rb, 2.54856e-03_rb/)
      selfrefo(:, 4) = (/ &
     & 8.47621e-03_rb, 7.29518e-03_rb, 6.27870e-03_rb, 5.40385e-03_rb, 4.65091e-03_rb, &
     & 4.00287e-03_rb, 3.44513e-03_rb, 2.96510e-03_rb, 2.55196e-03_rb, 2.19638e-03_rb/)
      selfrefo(:, 5) = (/ &
     & 6.71258e-03_rb, 5.95346e-03_rb, 5.28020e-03_rb, 4.68307e-03_rb, 4.15348e-03_rb, &
     & 3.68377e-03_rb, 3.26718e-03_rb, 2.89770e-03_rb, 2.57000e-03_rb, 2.27937e-03_rb/)
      selfrefo(:, 6) = (/ &
     & 6.29140e-03_rb, 5.55557e-03_rb, 4.90580e-03_rb, 4.33203e-03_rb, 3.82536e-03_rb, &
     & 3.37795e-03_rb, 2.98287e-03_rb, 2.63400e-03_rb, 2.32593e-03_rb, 2.05389e-03_rb/)
      selfrefo(:, 7) = (/ &
     & 6.00229e-03_rb, 5.28180e-03_rb, 4.64780e-03_rb, 4.08990e-03_rb, 3.59897e-03_rb, &
     & 3.16696e-03_rb, 2.78682e-03_rb, 2.45230e-03_rb, 2.15794e-03_rb, 1.89891e-03_rb/)
      selfrefo(:, 8) = (/ &
     & 5.78892e-03_rb, 5.07191e-03_rb, 4.44370e-03_rb, 3.89330e-03_rb, 3.41108e-03_rb, &
     & 2.98858e-03_rb, 2.61842e-03_rb, 2.29410e-03_rb, 2.00995e-03_rb, 1.76100e-03_rb/)
      selfrefo(:, 9) = (/ &
     & 4.96186e-03_rb, 4.56767e-03_rb, 4.20480e-03_rb, 3.87076e-03_rb, 3.56325e-03_rb, &
     & 3.28017e-03_rb, 3.01959e-03_rb, 2.77970e-03_rb, 2.55887e-03_rb, 2.35559e-03_rb/)
      selfrefo(:,10) = (/ &
     & 4.56849e-03_rb, 4.35527e-03_rb, 4.15200e-03_rb, 3.95822e-03_rb, 3.77348e-03_rb, &
     & 3.59736e-03_rb, 3.42946e-03_rb, 3.26940e-03_rb, 3.11681e-03_rb, 2.97134e-03_rb/)
      selfrefo(:,11) = (/ &
     & 4.47310e-03_rb, 4.32453e-03_rb, 4.18090e-03_rb, 4.04204e-03_rb, 3.90779e-03_rb, &
     & 3.77799e-03_rb, 3.65251e-03_rb, 3.53120e-03_rb, 3.41392e-03_rb, 3.30053e-03_rb/)
      selfrefo(:,12) = (/ &
     & 4.46459e-03_rb, 4.24031e-03_rb, 4.02730e-03_rb, 3.82499e-03_rb, 3.63284e-03_rb, &
     & 3.45035e-03_rb, 3.27702e-03_rb, 3.11240e-03_rb, 2.95605e-03_rb, 2.80755e-03_rb/)
      selfrefo(:,13) = (/ &
     & 4.43961e-03_rb, 4.35658e-03_rb, 4.27510e-03_rb, 4.19514e-03_rb, 4.11669e-03_rb, &
     & 4.03969e-03_rb, 3.96414e-03_rb, 3.89000e-03_rb, 3.81725e-03_rb, 3.74585e-03_rb/)
      selfrefo(:,14) = (/ &
     & 4.40512e-03_rb, 4.41515e-03_rb, 4.42520e-03_rb, 4.43527e-03_rb, 4.44537e-03_rb, &
     & 4.45549e-03_rb, 4.46563e-03_rb, 4.47580e-03_rb, 4.48599e-03_rb, 4.49620e-03_rb/)
      selfrefo(:,15) = (/ &
     & 3.21965e-03_rb, 3.42479e-03_rb, 3.64300e-03_rb, 3.87512e-03_rb, 4.12202e-03_rb, &
     & 4.38466e-03_rb, 4.66403e-03_rb, 4.96120e-03_rb, 5.27731e-03_rb, 5.61355e-03_rb/)
      selfrefo(:,16) = (/ &
     & 3.11402e-03_rb, 3.35870e-03_rb, 3.62260e-03_rb, 3.90724e-03_rb, 4.21424e-03_rb, &
     & 4.54536e-03_rb, 4.90250e-03_rb, 5.28770e-03_rb, 5.70317e-03_rb, 6.15128e-03_rb/)

      end subroutine lw_kgb13

! **************************************************************************
      subroutine lw_kgb14
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg14, only : fracrefao, fracrefbo, kao, kbo, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P = 142.5940 mb, T = 215.70 K
      fracrefao(:) = (/ &
     &  1.9360e-01_rb, 1.7276e-01_rb, 1.4811e-01_rb, 1.2238e-01_rb, &
     &  1.0242e-01_rb, 8.6830e-02_rb, 7.1890e-02_rb, 5.4030e-02_rb, &
     &  3.5075e-02_rb, 3.8052e-03_rb, 3.1458e-03_rb, 2.4873e-03_rb, &
     &  1.8182e-03_rb, 1.1563e-03_rb, 4.3251e-04_rb, 5.7744e-05_rb/)

! Planck fraction mapping level : P = 4.758820mb, T = 250.85 K
      fracrefbo(:) = (/ &
     &  1.8599e-01_rb, 1.6646e-01_rb, 1.4264e-01_rb, 1.2231e-01_rb, &
     &  1.0603e-01_rb, 9.2014e-02_rb, 7.5287e-02_rb, 5.6758e-02_rb, &
     &  3.8386e-02_rb, 4.2139e-03_rb, 3.5399e-03_rb, 2.7381e-03_rb, &
     &  1.9202e-03_rb, 1.2083e-03_rb, 4.5395e-04_rb, 6.2699e-05_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1) = (/ &
     &3.5183e-01_rb,3.7165e-01_rb,3.9536e-01_rb,4.2364e-01_rb,4.5645e-01_rb/)
      kao(:, 2, 1) = (/ &
     &2.9958e-01_rb,3.1568e-01_rb,3.3580e-01_rb,3.6030e-01_rb,3.8890e-01_rb/)
      kao(:, 3, 1) = (/ &
     &2.5105e-01_rb,2.6327e-01_rb,2.7913e-01_rb,2.9918e-01_rb,3.2278e-01_rb/)
      kao(:, 4, 1) = (/ &
     &2.0894e-01_rb,2.1786e-01_rb,2.3000e-01_rb,2.4588e-01_rb,2.6517e-01_rb/)
      kao(:, 5, 1) = (/ &
     &1.7343e-01_rb,1.7993e-01_rb,1.8907e-01_rb,2.0143e-01_rb,2.1685e-01_rb/)
      kao(:, 6, 1) = (/ &
     &1.4374e-01_rb,1.4829e-01_rb,1.5503e-01_rb,1.6436e-01_rb,1.7642e-01_rb/)
      kao(:, 7, 1) = (/ &
     &1.1916e-01_rb,1.2232e-01_rb,1.2721e-01_rb,1.3426e-01_rb,1.4361e-01_rb/)
      kao(:, 8, 1) = (/ &
     &9.8898e-02_rb,1.0120e-01_rb,1.0467e-01_rb,1.0982e-01_rb,1.1705e-01_rb/)
      kao(:, 9, 1) = (/ &
     &8.2212e-02_rb,8.3740e-02_rb,8.6244e-02_rb,9.0121e-02_rb,9.5602e-02_rb/)
      kao(:,10, 1) = (/ &
     &6.8330e-02_rb,6.9351e-02_rb,7.1210e-02_rb,7.4137e-02_rb,7.8349e-02_rb/)
      kao(:,11, 1) = (/ &
     &5.6910e-02_rb,5.7856e-02_rb,5.9557e-02_rb,6.2211e-02_rb,6.6055e-02_rb/)
      kao(:,12, 1) = (/ &
     &4.7279e-02_rb,4.8187e-02_rb,4.9716e-02_rb,5.2169e-02_rb,5.5693e-02_rb/)
      kao(:,13, 1) = (/ &
     &3.9228e-02_rb,4.0055e-02_rb,4.1480e-02_rb,4.3734e-02_rb,4.6826e-02_rb/)
      kao(:, 1, 2) = (/ &
     &1.2827e+00_rb,1.3853e+00_rb,1.5108e+00_rb,1.6643e+00_rb,1.8505e+00_rb/)
      kao(:, 2, 2) = (/ &
     &1.1042e+00_rb,1.1988e+00_rb,1.3140e+00_rb,1.4542e+00_rb,1.6229e+00_rb/)
      kao(:, 3, 2) = (/ &
     &9.2790e-01_rb,1.0121e+00_rb,1.1133e+00_rb,1.2349e+00_rb,1.3807e+00_rb/)
      kao(:, 4, 2) = (/ &
     &7.7001e-01_rb,8.4339e-01_rb,9.3172e-01_rb,1.0366e+00_rb,1.1604e+00_rb/)
      kao(:, 5, 2) = (/ &
     &6.3550e-01_rb,6.9915e-01_rb,7.7552e-01_rb,8.6518e-01_rb,9.7050e-01_rb/)
      kao(:, 6, 2) = (/ &
     &5.2058e-01_rb,5.7460e-01_rb,6.3982e-01_rb,7.1578e-01_rb,8.0435e-01_rb/)
      kao(:, 7, 2) = (/ &
     &4.2451e-01_rb,4.7028e-01_rb,5.2464e-01_rb,5.8830e-01_rb,6.6179e-01_rb/)
      kao(:, 8, 2) = (/ &
     &3.4539e-01_rb,3.8222e-01_rb,4.2607e-01_rb,4.7789e-01_rb,5.3786e-01_rb/)
      kao(:, 9, 2) = (/ &
     &2.7941e-01_rb,3.0849e-01_rb,3.4320e-01_rb,3.8420e-01_rb,4.3240e-01_rb/)
      kao(:,10, 2) = (/ &
     &2.2666e-01_rb,2.5024e-01_rb,2.7777e-01_rb,3.1051e-01_rb,3.4934e-01_rb/)
      kao(:,11, 2) = (/ &
     &1.9193e-01_rb,2.1218e-01_rb,2.3589e-01_rb,2.6422e-01_rb,2.9804e-01_rb/)
      kao(:,12, 2) = (/ &
     &1.6216e-01_rb,1.7954e-01_rb,2.0015e-01_rb,2.2464e-01_rb,2.5376e-01_rb/)
      kao(:,13, 2) = (/ &
     &1.3654e-01_rb,1.5163e-01_rb,1.6956e-01_rb,1.9071e-01_rb,2.1617e-01_rb/)
      kao(:, 1, 3) = (/ &
     &5.3312e+00_rb,5.9534e+00_rb,6.6227e+00_rb,7.3398e+00_rb,8.1099e+00_rb/)
      kao(:, 2, 3) = (/ &
     &4.6807e+00_rb,5.2433e+00_rb,5.8458e+00_rb,6.4988e+00_rb,7.1971e+00_rb/)
      kao(:, 3, 3) = (/ &
     &3.9284e+00_rb,4.4200e+00_rb,4.9505e+00_rb,5.5235e+00_rb,6.1336e+00_rb/)
      kao(:, 4, 3) = (/ &
     &3.2160e+00_rb,3.6333e+00_rb,4.0896e+00_rb,4.5842e+00_rb,5.1138e+00_rb/)
      kao(:, 5, 3) = (/ &
     &2.5935e+00_rb,2.9443e+00_rb,3.3309e+00_rb,3.7529e+00_rb,4.2079e+00_rb/)
      kao(:, 6, 3) = (/ &
     &2.0555e+00_rb,2.3468e+00_rb,2.6695e+00_rb,3.0246e+00_rb,3.4114e+00_rb/)
      kao(:, 7, 3) = (/ &
     &1.6122e+00_rb,1.8522e+00_rb,2.1211e+00_rb,2.4178e+00_rb,2.7436e+00_rb/)
      kao(:, 8, 3) = (/ &
     &1.2572e+00_rb,1.4549e+00_rb,1.6787e+00_rb,1.9278e+00_rb,2.2011e+00_rb/)
      kao(:, 9, 3) = (/ &
     &9.7554e-01_rb,1.1374e+00_rb,1.3224e+00_rb,1.5307e+00_rb,1.7599e+00_rb/)
      kao(:,10, 3) = (/ &
     &7.6412e-01_rb,8.9744e-01_rb,1.0509e+00_rb,1.2253e+00_rb,1.4183e+00_rb/)
      kao(:,11, 3) = (/ &
     &6.4606e-01_rb,7.6360e-01_rb,8.9925e-01_rb,1.0524e+00_rb,1.2194e+00_rb/)
      kao(:,12, 3) = (/ &
     &5.4819e-01_rb,6.5159e-01_rb,7.6939e-01_rb,8.9987e-01_rb,1.0405e+00_rb/)
      kao(:,13, 3) = (/ &
     &4.6584e-01_rb,5.5439e-01_rb,6.5378e-01_rb,7.6270e-01_rb,8.8077e-01_rb/)
      kao(:, 1, 4) = (/ &
     &1.6140e+01_rb,1.7514e+01_rb,1.8771e+01_rb,1.9886e+01_rb,2.0829e+01_rb/)
      kao(:, 2, 4) = (/ &
     &1.4035e+01_rb,1.5167e+01_rb,1.6185e+01_rb,1.7079e+01_rb,1.7864e+01_rb/)
      kao(:, 3, 4) = (/ &
     &1.2090e+01_rb,1.3039e+01_rb,1.3889e+01_rb,1.4653e+01_rb,1.5339e+01_rb/)
      kao(:, 4, 4) = (/ &
     &1.0243e+01_rb,1.1063e+01_rb,1.1803e+01_rb,1.2472e+01_rb,1.3075e+01_rb/)
      kao(:, 5, 4) = (/ &
     &8.5658e+00_rb,9.2747e+00_rb,9.9250e+00_rb,1.0511e+01_rb,1.1040e+01_rb/)
      kao(:, 6, 4) = (/ &
     &7.0689e+00_rb,7.6839e+00_rb,8.2525e+00_rb,8.7702e+00_rb,9.2411e+00_rb/)
      kao(:, 7, 4) = (/ &
     &5.7728e+00_rb,6.3036e+00_rb,6.7993e+00_rb,7.2580e+00_rb,7.6760e+00_rb/)
      kao(:, 8, 4) = (/ &
     &4.6719e+00_rb,5.1262e+00_rb,5.5542e+00_rb,5.9566e+00_rb,6.3304e+00_rb/)
      kao(:, 9, 4) = (/ &
     &3.7449e+00_rb,4.1314e+00_rb,4.4992e+00_rb,4.8512e+00_rb,5.1834e+00_rb/)
      kao(:,10, 4) = (/ &
     &2.9983e+00_rb,3.3274e+00_rb,3.6424e+00_rb,3.9480e+00_rb,4.2418e+00_rb/)
      kao(:,11, 4) = (/ &
     &2.5136e+00_rb,2.7891e+00_rb,3.0545e+00_rb,3.3182e+00_rb,3.5747e+00_rb/)
      kao(:,12, 4) = (/ &
     &2.1004e+00_rb,2.3314e+00_rb,2.5605e+00_rb,2.7929e+00_rb,3.0174e+00_rb/)
      kao(:,13, 4) = (/ &
     &1.7524e+00_rb,1.9492e+00_rb,2.1512e+00_rb,2.3564e+00_rb,2.5478e+00_rb/)
      kao(:, 1, 5) = (/ &
     &3.4724e+01_rb,3.5203e+01_rb,3.5615e+01_rb,3.5967e+01_rb,3.6300e+01_rb/)
      kao(:, 2, 5) = (/ &
     &2.9704e+01_rb,3.0154e+01_rb,3.0541e+01_rb,3.0880e+01_rb,3.1155e+01_rb/)
      kao(:, 3, 5) = (/ &
     &2.5203e+01_rb,2.5623e+01_rb,2.5990e+01_rb,2.6275e+01_rb,2.6504e+01_rb/)
      kao(:, 4, 5) = (/ &
     &2.1552e+01_rb,2.1924e+01_rb,2.2232e+01_rb,2.2450e+01_rb,2.2639e+01_rb/)
      kao(:, 5, 5) = (/ &
     &1.8513e+01_rb,1.8836e+01_rb,1.9081e+01_rb,1.9246e+01_rb,1.9410e+01_rb/)
      kao(:, 6, 5) = (/ &
     &1.5765e+01_rb,1.6046e+01_rb,1.6242e+01_rb,1.6370e+01_rb,1.6509e+01_rb/)
      kao(:, 7, 5) = (/ &
     &1.3333e+01_rb,1.3583e+01_rb,1.3745e+01_rb,1.3853e+01_rb,1.3971e+01_rb/)
      kao(:, 8, 5) = (/ &
     &1.1230e+01_rb,1.1452e+01_rb,1.1589e+01_rb,1.1683e+01_rb,1.1782e+01_rb/)
      kao(:, 9, 5) = (/ &
     &9.4196e+00_rb,9.6109e+00_rb,9.7356e+00_rb,9.8175e+00_rb,9.8960e+00_rb/)
      kao(:,10, 5) = (/ &
     &7.8743e+00_rb,8.0366e+00_rb,8.1439e+00_rb,8.2124e+00_rb,8.2768e+00_rb/)
      kao(:,11, 5) = (/ &
     &6.6265e+00_rb,6.7386e+00_rb,6.8161e+00_rb,6.8681e+00_rb,6.9329e+00_rb/)
      kao(:,12, 5) = (/ &
     &5.5390e+00_rb,5.6241e+00_rb,5.6817e+00_rb,5.7305e+00_rb,5.8031e+00_rb/)
      kao(:,13, 5) = (/ &
     &4.6094e+00_rb,4.6772e+00_rb,4.7246e+00_rb,4.7737e+00_rb,4.8549e+00_rb/)
      kao(:, 1, 6) = (/ &
     &5.5082e+01_rb,5.5287e+01_rb,5.5714e+01_rb,5.6227e+01_rb,5.6768e+01_rb/)
      kao(:, 2, 6) = (/ &
     &4.8272e+01_rb,4.8698e+01_rb,4.9143e+01_rb,4.9607e+01_rb,5.0132e+01_rb/)
      kao(:, 3, 6) = (/ &
     &4.2021e+01_rb,4.2370e+01_rb,4.2729e+01_rb,4.3163e+01_rb,4.3674e+01_rb/)
      kao(:, 4, 6) = (/ &
     &3.6214e+01_rb,3.6443e+01_rb,3.6742e+01_rb,3.7158e+01_rb,3.7640e+01_rb/)
      kao(:, 5, 6) = (/ &
     &3.1082e+01_rb,3.1249e+01_rb,3.1507e+01_rb,3.1893e+01_rb,3.2320e+01_rb/)
      kao(:, 6, 6) = (/ &
     &2.6864e+01_rb,2.6986e+01_rb,2.7204e+01_rb,2.7526e+01_rb,2.7858e+01_rb/)
      kao(:, 7, 6) = (/ &
     &2.3146e+01_rb,2.3231e+01_rb,2.3407e+01_rb,2.3654e+01_rb,2.3926e+01_rb/)
      kao(:, 8, 6) = (/ &
     &1.9819e+01_rb,1.9868e+01_rb,1.9994e+01_rb,2.0178e+01_rb,2.0400e+01_rb/)
      kao(:, 9, 6) = (/ &
     &1.6892e+01_rb,1.6913e+01_rb,1.7001e+01_rb,1.7132e+01_rb,1.7310e+01_rb/)
      kao(:,10, 6) = (/ &
     &1.4365e+01_rb,1.4367e+01_rb,1.4429e+01_rb,1.4521e+01_rb,1.4660e+01_rb/)
      kao(:,11, 6) = (/ &
     &1.2162e+01_rb,1.2177e+01_rb,1.2223e+01_rb,1.2309e+01_rb,1.2439e+01_rb/)
      kao(:,12, 6) = (/ &
     &1.0257e+01_rb,1.0260e+01_rb,1.0295e+01_rb,1.0375e+01_rb,1.0502e+01_rb/)
      kao(:,13, 6) = (/ &
     &8.6109e+00_rb,8.6045e+00_rb,8.6391e+00_rb,8.7159e+00_rb,8.8472e+00_rb/)
      kao(:, 1, 7) = (/ &
     &9.6537e+01_rb,9.5488e+01_rb,9.4812e+01_rb,9.4687e+01_rb,9.5057e+01_rb/)
      kao(:, 2, 7) = (/ &
     &8.5306e+01_rb,8.4772e+01_rb,8.4928e+01_rb,8.5670e+01_rb,8.6805e+01_rb/)
      kao(:, 3, 7) = (/ &
     &7.4847e+01_rb,7.4994e+01_rb,7.5806e+01_rb,7.7131e+01_rb,7.8782e+01_rb/)
      kao(:, 4, 7) = (/ &
     &6.5341e+01_rb,6.6016e+01_rb,6.7305e+01_rb,6.9026e+01_rb,7.0761e+01_rb/)
      kao(:, 5, 7) = (/ &
     &5.6784e+01_rb,5.7825e+01_rb,5.9429e+01_rb,6.1058e+01_rb,6.2599e+01_rb/)
      kao(:, 6, 7) = (/ &
     &4.9097e+01_rb,5.0352e+01_rb,5.1865e+01_rb,5.3258e+01_rb,5.4642e+01_rb/)
      kao(:, 7, 7) = (/ &
     &4.2608e+01_rb,4.3857e+01_rb,4.5074e+01_rb,4.6282e+01_rb,4.7459e+01_rb/)
      kao(:, 8, 7) = (/ &
     &3.7209e+01_rb,3.8259e+01_rb,3.9241e+01_rb,4.0246e+01_rb,4.1244e+01_rb/)
      kao(:, 9, 7) = (/ &
     &3.2353e+01_rb,3.3212e+01_rb,3.4023e+01_rb,3.4877e+01_rb,3.5739e+01_rb/)
      kao(:,10, 7) = (/ &
     &2.7960e+01_rb,2.8627e+01_rb,2.9290e+01_rb,3.0022e+01_rb,3.0747e+01_rb/)
      kao(:,11, 7) = (/ &
     &2.4264e+01_rb,2.4777e+01_rb,2.5355e+01_rb,2.5965e+01_rb,2.6544e+01_rb/)
      kao(:,12, 7) = (/ &
     &2.0930e+01_rb,2.1372e+01_rb,2.1863e+01_rb,2.2353e+01_rb,2.2822e+01_rb/)
      kao(:,13, 7) = (/ &
     &1.7993e+01_rb,1.8373e+01_rb,1.8772e+01_rb,1.9157e+01_rb,1.9544e+01_rb/)
      kao(:, 1, 8) = (/ &
     &2.1856e+02_rb,2.1566e+02_rb,2.1284e+02_rb,2.0997e+02_rb,2.0718e+02_rb/)
      kao(:, 2, 8) = (/ &
     &2.0300e+02_rb,2.0022e+02_rb,1.9750e+02_rb,1.9481e+02_rb,1.9235e+02_rb/)
      kao(:, 3, 8) = (/ &
     &1.8584e+02_rb,1.8317e+02_rb,1.8063e+02_rb,1.7831e+02_rb,1.7621e+02_rb/)
      kao(:, 4, 8) = (/ &
     &1.6761e+02_rb,1.6519e+02_rb,1.6298e+02_rb,1.6101e+02_rb,1.5981e+02_rb/)
      kao(:, 5, 8) = (/ &
     &1.4928e+02_rb,1.4711e+02_rb,1.4522e+02_rb,1.4425e+02_rb,1.4420e+02_rb/)
      kao(:, 6, 8) = (/ &
     &1.3122e+02_rb,1.2933e+02_rb,1.2822e+02_rb,1.2825e+02_rb,1.2918e+02_rb/)
      kao(:, 7, 8) = (/ &
     &1.1410e+02_rb,1.1264e+02_rb,1.1239e+02_rb,1.1324e+02_rb,1.1505e+02_rb/)
      kao(:, 8, 8) = (/ &
     &9.8348e+01_rb,9.7493e+01_rb,9.7940e+01_rb,9.9442e+01_rb,1.0191e+02_rb/)
      kao(:, 9, 8) = (/ &
     &8.4647e+01_rb,8.4268e+01_rb,8.5204e+01_rb,8.7179e+01_rb,9.0023e+01_rb/)
      kao(:,10, 8) = (/ &
     &7.3021e+01_rb,7.3118e+01_rb,7.4447e+01_rb,7.6754e+01_rb,7.9414e+01_rb/)
      kao(:,11, 8) = (/ &
     &6.3300e+01_rb,6.4312e+01_rb,6.6319e+01_rb,6.8546e+01_rb,7.0686e+01_rb/)
      kao(:,12, 8) = (/ &
     &5.5501e+01_rb,5.7093e+01_rb,5.8983e+01_rb,6.0787e+01_rb,6.2564e+01_rb/)
      kao(:,13, 8) = (/ &
     &4.8706e+01_rb,5.0287e+01_rb,5.1813e+01_rb,5.3328e+01_rb,5.4791e+01_rb/)
      kao(:, 1, 9) = (/ &
     &5.5934e+02_rb,5.5574e+02_rb,5.5144e+02_rb,5.4673e+02_rb,5.4155e+02_rb/)
      kao(:, 2, 9) = (/ &
     &5.7946e+02_rb,5.7510e+02_rb,5.7015e+02_rb,5.6485e+02_rb,5.5885e+02_rb/)
      kao(:, 3, 9) = (/ &
     &5.9076e+02_rb,5.8577e+02_rb,5.8036e+02_rb,5.7428e+02_rb,5.6790e+02_rb/)
      kao(:, 4, 9) = (/ &
     &5.9122e+02_rb,5.8578e+02_rb,5.7964e+02_rb,5.7332e+02_rb,5.6635e+02_rb/)
      kao(:, 5, 9) = (/ &
     &5.8028e+02_rb,5.7411e+02_rb,5.6769e+02_rb,5.6068e+02_rb,5.5353e+02_rb/)
      kao(:, 6, 9) = (/ &
     &5.5962e+02_rb,5.5294e+02_rb,5.4600e+02_rb,5.3888e+02_rb,5.3142e+02_rb/)
      kao(:, 7, 9) = (/ &
     &5.2985e+02_rb,5.2302e+02_rb,5.1578e+02_rb,5.0845e+02_rb,5.0105e+02_rb/)
      kao(:, 8, 9) = (/ &
     &4.9277e+02_rb,4.8554e+02_rb,4.7838e+02_rb,4.7112e+02_rb,4.6381e+02_rb/)
      kao(:, 9, 9) = (/ &
     &4.5092e+02_rb,4.4373e+02_rb,4.3650e+02_rb,4.2952e+02_rb,4.2278e+02_rb/)
      kao(:,10, 9) = (/ &
     &4.0542e+02_rb,3.9841e+02_rb,3.9162e+02_rb,3.8505e+02_rb,3.7968e+02_rb/)
      kao(:,11, 9) = (/ &
     &3.5669e+02_rb,3.5010e+02_rb,3.4395e+02_rb,3.3923e+02_rb,3.3643e+02_rb/)
      kao(:,12, 9) = (/ &
     &3.0992e+02_rb,3.0411e+02_rb,2.9975e+02_rb,2.9742e+02_rb,2.9692e+02_rb/)
      kao(:,13, 9) = (/ &
     &2.6703e+02_rb,2.6279e+02_rb,2.6062e+02_rb,2.6039e+02_rb,2.6194e+02_rb/)
      kao(:, 1,10) = (/ &
     &9.5667e+02_rb,9.5651e+02_rb,9.5375e+02_rb,9.4993e+02_rb,9.4458e+02_rb/)
      kao(:, 2,10) = (/ &
     &1.0737e+03_rb,1.0712e+03_rb,1.0675e+03_rb,1.0621e+03_rb,1.0554e+03_rb/)
      kao(:, 3,10) = (/ &
     &1.1862e+03_rb,1.1837e+03_rb,1.1793e+03_rb,1.1732e+03_rb,1.1657e+03_rb/)
      kao(:, 4,10) = (/ &
     &1.2923e+03_rb,1.2885e+03_rb,1.2837e+03_rb,1.2762e+03_rb,1.2683e+03_rb/)
      kao(:, 5,10) = (/ &
     &1.3860e+03_rb,1.3831e+03_rb,1.3760e+03_rb,1.3692e+03_rb,1.3583e+03_rb/)
      kao(:, 6,10) = (/ &
     &1.4574e+03_rb,1.4515e+03_rb,1.4451e+03_rb,1.4344e+03_rb,1.4246e+03_rb/)
      kao(:, 7,10) = (/ &
     &1.5032e+03_rb,1.4948e+03_rb,1.4847e+03_rb,1.4737e+03_rb,1.4593e+03_rb/)
      kao(:, 8,10) = (/ &
     &1.5178e+03_rb,1.5093e+03_rb,1.4967e+03_rb,1.4828e+03_rb,1.4689e+03_rb/)
      kao(:, 9,10) = (/ &
     &1.5009e+03_rb,1.4874e+03_rb,1.4751e+03_rb,1.4596e+03_rb,1.4422e+03_rb/)
      kao(:,10,10) = (/ &
     &1.4526e+03_rb,1.4382e+03_rb,1.4207e+03_rb,1.4042e+03_rb,1.3871e+03_rb/)
      kao(:,11,10) = (/ &
     &1.3669e+03_rb,1.3492e+03_rb,1.3313e+03_rb,1.3144e+03_rb,1.2952e+03_rb/)
      kao(:,12,10) = (/ &
     &1.2561e+03_rb,1.2377e+03_rb,1.2201e+03_rb,1.2021e+03_rb,1.1826e+03_rb/)
      kao(:,13,10) = (/ &
     &1.1367e+03_rb,1.1187e+03_rb,1.1007e+03_rb,1.0822e+03_rb,1.0635e+03_rb/)
      kao(:, 1,11) = (/ &
     &1.0788e+03_rb,1.0784e+03_rb,1.0786e+03_rb,1.0769e+03_rb,1.0720e+03_rb/)
      kao(:, 2,11) = (/ &
     &1.2360e+03_rb,1.2387e+03_rb,1.2382e+03_rb,1.2346e+03_rb,1.2287e+03_rb/)
      kao(:, 3,11) = (/ &
     &1.4028e+03_rb,1.4050e+03_rb,1.4041e+03_rb,1.4005e+03_rb,1.3939e+03_rb/)
      kao(:, 4,11) = (/ &
     &1.5761e+03_rb,1.5770e+03_rb,1.5755e+03_rb,1.5691e+03_rb,1.5622e+03_rb/)
      kao(:, 5,11) = (/ &
     &1.7429e+03_rb,1.7443e+03_rb,1.7409e+03_rb,1.7348e+03_rb,1.7262e+03_rb/)
      kao(:, 6,11) = (/ &
     &1.9004e+03_rb,1.9007e+03_rb,1.8945e+03_rb,1.8893e+03_rb,1.8776e+03_rb/)
      kao(:, 7,11) = (/ &
     &2.0382e+03_rb,2.0344e+03_rb,2.0314e+03_rb,2.0213e+03_rb,2.0121e+03_rb/)
      kao(:, 8,11) = (/ &
     &2.1442e+03_rb,2.1390e+03_rb,2.1319e+03_rb,2.1222e+03_rb,2.1076e+03_rb/)
      kao(:, 9,11) = (/ &
     &2.2113e+03_rb,2.2063e+03_rb,2.1931e+03_rb,2.1796e+03_rb,2.1631e+03_rb/)
      kao(:,10,11) = (/ &
     &2.2360e+03_rb,2.2231e+03_rb,2.2114e+03_rb,2.1930e+03_rb,2.1737e+03_rb/)
      kao(:,11,11) = (/ &
     &2.2031e+03_rb,2.1887e+03_rb,2.1696e+03_rb,2.1476e+03_rb,2.1259e+03_rb/)
      kao(:,12,11) = (/ &
     &2.1236e+03_rb,2.1030e+03_rb,2.0791e+03_rb,2.0554e+03_rb,2.0303e+03_rb/)
      kao(:,13,11) = (/ &
     &1.9976e+03_rb,1.9732e+03_rb,1.9489e+03_rb,1.9244e+03_rb,1.8976e+03_rb/)
      kao(:, 1,12) = (/ &
     &1.1934e+03_rb,1.1955e+03_rb,1.1958e+03_rb,1.1936e+03_rb,1.1920e+03_rb/)
      kao(:, 2,12) = (/ &
     &1.3945e+03_rb,1.3970e+03_rb,1.3984e+03_rb,1.3987e+03_rb,1.3979e+03_rb/)
      kao(:, 3,12) = (/ &
     &1.6215e+03_rb,1.6249e+03_rb,1.6252e+03_rb,1.6264e+03_rb,1.6242e+03_rb/)
      kao(:, 4,12) = (/ &
     &1.8681e+03_rb,1.8729e+03_rb,1.8755e+03_rb,1.8766e+03_rb,1.8729e+03_rb/)
      kao(:, 5,12) = (/ &
     &2.1302e+03_rb,2.1346e+03_rb,2.1412e+03_rb,2.1410e+03_rb,2.1383e+03_rb/)
      kao(:, 6,12) = (/ &
     &2.4029e+03_rb,2.4115e+03_rb,2.4156e+03_rb,2.4143e+03_rb,2.4101e+03_rb/)
      kao(:, 7,12) = (/ &
     &2.6748e+03_rb,2.6869e+03_rb,2.6879e+03_rb,2.6884e+03_rb,2.6793e+03_rb/)
      kao(:, 8,12) = (/ &
     &2.9380e+03_rb,2.9477e+03_rb,2.9496e+03_rb,2.9469e+03_rb,2.9380e+03_rb/)
      kao(:, 9,12) = (/ &
     &3.1781e+03_rb,3.1822e+03_rb,3.1866e+03_rb,3.1790e+03_rb,3.1694e+03_rb/)
      kao(:,10,12) = (/ &
     &3.3770e+03_rb,3.3808e+03_rb,3.3765e+03_rb,3.3686e+03_rb,3.3530e+03_rb/)
      kao(:,11,12) = (/ &
     &3.5189e+03_rb,3.5130e+03_rb,3.5038e+03_rb,3.4874e+03_rb,3.4673e+03_rb/)
      kao(:,12,12) = (/ &
     &3.5908e+03_rb,3.5789e+03_rb,3.5609e+03_rb,3.5388e+03_rb,3.5103e+03_rb/)
      kao(:,13,12) = (/ &
     &3.5806e+03_rb,3.5582e+03_rb,3.5338e+03_rb,3.5034e+03_rb,3.4725e+03_rb/)
      kao(:, 1,13) = (/ &
     &1.2919e+03_rb,1.2953e+03_rb,1.2979e+03_rb,1.2980e+03_rb,1.2956e+03_rb/)
      kao(:, 2,13) = (/ &
     &1.5383e+03_rb,1.5420e+03_rb,1.5447e+03_rb,1.5440e+03_rb,1.5420e+03_rb/)
      kao(:, 3,13) = (/ &
     &1.8220e+03_rb,1.8287e+03_rb,1.8322e+03_rb,1.8325e+03_rb,1.8308e+03_rb/)
      kao(:, 4,13) = (/ &
     &2.1443e+03_rb,2.1550e+03_rb,2.1609e+03_rb,2.1623e+03_rb,2.1617e+03_rb/)
      kao(:, 5,13) = (/ &
     &2.5123e+03_rb,2.5251e+03_rb,2.5330e+03_rb,2.5368e+03_rb,2.5365e+03_rb/)
      kao(:, 6,13) = (/ &
     &2.9214e+03_rb,2.9360e+03_rb,2.9475e+03_rb,2.9537e+03_rb,2.9558e+03_rb/)
      kao(:, 7,13) = (/ &
     &3.3684e+03_rb,3.3871e+03_rb,3.4024e+03_rb,3.4082e+03_rb,3.4125e+03_rb/)
      kao(:, 8,13) = (/ &
     &3.8472e+03_rb,3.8701e+03_rb,3.8882e+03_rb,3.8980e+03_rb,3.9031e+03_rb/)
      kao(:, 9,13) = (/ &
     &4.3463e+03_rb,4.3744e+03_rb,4.3907e+03_rb,4.4045e+03_rb,4.4088e+03_rb/)
      kao(:,10,13) = (/ &
     &4.8548e+03_rb,4.8861e+03_rb,4.9050e+03_rb,4.9166e+03_rb,4.9195e+03_rb/)
      kao(:,11,13) = (/ &
     &5.3685e+03_rb,5.3944e+03_rb,5.4093e+03_rb,5.4165e+03_rb,5.4085e+03_rb/)
      kao(:,12,13) = (/ &
     &5.8390e+03_rb,5.8545e+03_rb,5.8642e+03_rb,5.8596e+03_rb,5.8502e+03_rb/)
      kao(:,13,13) = (/ &
     &6.2371e+03_rb,6.2518e+03_rb,6.2489e+03_rb,6.2389e+03_rb,6.2163e+03_rb/)
      kao(:, 1,14) = (/ &
     &1.3744e+03_rb,1.3784e+03_rb,1.3801e+03_rb,1.3822e+03_rb,1.3833e+03_rb/)
      kao(:, 2,14) = (/ &
     &1.6525e+03_rb,1.6579e+03_rb,1.6603e+03_rb,1.6628e+03_rb,1.6635e+03_rb/)
      kao(:, 3,14) = (/ &
     &1.9810e+03_rb,1.9908e+03_rb,1.9968e+03_rb,2.0006e+03_rb,2.0007e+03_rb/)
      kao(:, 4,14) = (/ &
     &2.3728e+03_rb,2.3844e+03_rb,2.3938e+03_rb,2.4019e+03_rb,2.4042e+03_rb/)
      kao(:, 5,14) = (/ &
     &2.8373e+03_rb,2.8532e+03_rb,2.8650e+03_rb,2.8745e+03_rb,2.8783e+03_rb/)
      kao(:, 6,14) = (/ &
     &3.3748e+03_rb,3.3991e+03_rb,3.4179e+03_rb,3.4320e+03_rb,3.4378e+03_rb/)
      kao(:, 7,14) = (/ &
     &3.9998e+03_rb,4.0298e+03_rb,4.0539e+03_rb,4.0746e+03_rb,4.0866e+03_rb/)
      kao(:, 8,14) = (/ &
     &4.7196e+03_rb,4.7615e+03_rb,4.7914e+03_rb,4.8146e+03_rb,4.8273e+03_rb/)
      kao(:, 9,14) = (/ &
     &5.5304e+03_rb,5.5847e+03_rb,5.6275e+03_rb,5.6623e+03_rb,5.6782e+03_rb/)
      kao(:,10,14) = (/ &
     &6.4513e+03_rb,6.5110e+03_rb,6.5637e+03_rb,6.6030e+03_rb,6.6279e+03_rb/)
      kao(:,11,14) = (/ &
     &7.5126e+03_rb,7.5759e+03_rb,7.6236e+03_rb,7.6577e+03_rb,7.6817e+03_rb/)
      kao(:,12,14) = (/ &
     &8.6584e+03_rb,8.7304e+03_rb,8.7796e+03_rb,8.8102e+03_rb,8.8235e+03_rb/)
      kao(:,13,14) = (/ &
     &9.8812e+03_rb,9.9499e+03_rb,9.9974e+03_rb,1.0022e+04_rb,1.0029e+04_rb/)
      kao(:, 1,15) = (/ &
     &1.4528e+03_rb,1.4581e+03_rb,1.4601e+03_rb,1.4582e+03_rb,1.4558e+03_rb/)
      kao(:, 2,15) = (/ &
     &1.7546e+03_rb,1.7631e+03_rb,1.7666e+03_rb,1.7671e+03_rb,1.7653e+03_rb/)
      kao(:, 3,15) = (/ &
     &2.1167e+03_rb,2.1275e+03_rb,2.1344e+03_rb,2.1393e+03_rb,2.1396e+03_rb/)
      kao(:, 4,15) = (/ &
     &2.5536e+03_rb,2.5679e+03_rb,2.5752e+03_rb,2.5829e+03_rb,2.5878e+03_rb/)
      kao(:, 5,15) = (/ &
     &3.0751e+03_rb,3.0964e+03_rb,3.1097e+03_rb,3.1178e+03_rb,3.1252e+03_rb/)
      kao(:, 6,15) = (/ &
     &3.6981e+03_rb,3.7293e+03_rb,3.7501e+03_rb,3.7620e+03_rb,3.7738e+03_rb/)
      kao(:, 7,15) = (/ &
     &4.4451e+03_rb,4.4852e+03_rb,4.5165e+03_rb,4.5355e+03_rb,4.5519e+03_rb/)
      kao(:, 8,15) = (/ &
     &5.3398e+03_rb,5.3890e+03_rb,5.4280e+03_rb,5.4575e+03_rb,5.4832e+03_rb/)
      kao(:, 9,15) = (/ &
     &6.4016e+03_rb,6.4666e+03_rb,6.5191e+03_rb,6.5509e+03_rb,6.5855e+03_rb/)
      kao(:,10,15) = (/ &
     &7.6539e+03_rb,7.7451e+03_rb,7.8138e+03_rb,7.8602e+03_rb,7.8961e+03_rb/)
      kao(:,11,15) = (/ &
     &9.1780e+03_rb,9.2834e+03_rb,9.3652e+03_rb,9.4242e+03_rb,9.4602e+03_rb/)
      kao(:,12,15) = (/ &
     &1.0986e+04_rb,1.1091e+04_rb,1.1175e+04_rb,1.1241e+04_rb,1.1280e+04_rb/)
      kao(:,13,15) = (/ &
     &1.3095e+04_rb,1.3203e+04_rb,1.3280e+04_rb,1.3335e+04_rb,1.3364e+04_rb/)
      kao(:, 1,16) = (/ &
     &1.4758e+03_rb,1.4811e+03_rb,1.4822e+03_rb,1.4873e+03_rb,1.4919e+03_rb/)
      kao(:, 2,16) = (/ &
     &1.7931e+03_rb,1.7988e+03_rb,1.8023e+03_rb,1.8054e+03_rb,1.8115e+03_rb/)
      kao(:, 3,16) = (/ &
     &2.1801e+03_rb,2.1858e+03_rb,2.1915e+03_rb,2.1950e+03_rb,2.1996e+03_rb/)
      kao(:, 4,16) = (/ &
     &2.6497e+03_rb,2.6575e+03_rb,2.6648e+03_rb,2.6696e+03_rb,2.6752e+03_rb/)
      kao(:, 5,16) = (/ &
     &3.2162e+03_rb,3.2298e+03_rb,3.2415e+03_rb,3.2482e+03_rb,3.2542e+03_rb/)
      kao(:, 6,16) = (/ &
     &3.8965e+03_rb,3.9207e+03_rb,3.9398e+03_rb,3.9520e+03_rb,3.9556e+03_rb/)
      kao(:, 7,16) = (/ &
     &4.7145e+03_rb,4.7502e+03_rb,4.7821e+03_rb,4.8036e+03_rb,4.8122e+03_rb/)
      kao(:, 8,16) = (/ &
     &5.7091e+03_rb,5.7519e+03_rb,5.7946e+03_rb,5.8300e+03_rb,5.8477e+03_rb/)
      kao(:, 9,16) = (/ &
     &6.9081e+03_rb,6.9691e+03_rb,7.0079e+03_rb,7.0618e+03_rb,7.0936e+03_rb/)
      kao(:,10,16) = (/ &
     &8.3531e+03_rb,8.4377e+03_rb,8.4862e+03_rb,8.5388e+03_rb,8.5857e+03_rb/)
      kao(:,11,16) = (/ &
     &1.0140e+04_rb,1.0233e+04_rb,1.0291e+04_rb,1.0344e+04_rb,1.0381e+04_rb/)
      kao(:,12,16) = (/ &
     &1.2278e+04_rb,1.2376e+04_rb,1.2452e+04_rb,1.2503e+04_rb,1.2519e+04_rb/)
      kao(:,13,16) = (/ &
     &1.4816e+04_rb,1.4930e+04_rb,1.5014e+04_rb,1.5052e+04_rb,1.5044e+04_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &3.9228e-02_rb,4.0055e-02_rb,4.1480e-02_rb,4.3734e-02_rb,4.6826e-02_rb/)
      kbo(:,14, 1) = (/ &
     &3.2541e-02_rb,3.3340e-02_rb,3.4715e-02_rb,3.6769e-02_rb,3.9564e-02_rb/)
      kbo(:,15, 1) = (/ &
     &2.6969e-02_rb,2.7753e-02_rb,2.9038e-02_rb,3.0896e-02_rb,3.3356e-02_rb/)
      kbo(:,16, 1) = (/ &
     &2.2342e-02_rb,2.3099e-02_rb,2.4270e-02_rb,2.5925e-02_rb,2.8078e-02_rb/)
      kbo(:,17, 1) = (/ &
     &1.8513e-02_rb,1.9222e-02_rb,2.0267e-02_rb,2.1721e-02_rb,2.3597e-02_rb/)
      kbo(:,18, 1) = (/ &
     &1.5353e-02_rb,1.5989e-02_rb,1.6916e-02_rb,1.8172e-02_rb,1.9812e-02_rb/)
      kbo(:,19, 1) = (/ &
     &1.2730e-02_rb,1.3292e-02_rb,1.4104e-02_rb,1.5198e-02_rb,1.6633e-02_rb/)
      kbo(:,20, 1) = (/ &
     &1.0558e-02_rb,1.1059e-02_rb,1.1764e-02_rb,1.2725e-02_rb,1.3995e-02_rb/)
      kbo(:,21, 1) = (/ &
     &8.7555e-03_rb,9.1943e-03_rb,9.8141e-03_rb,1.0663e-02_rb,1.1783e-02_rb/)
      kbo(:,22, 1) = (/ &
     &7.2788e-03_rb,7.6721e-03_rb,8.2312e-03_rb,8.9953e-03_rb,1.0002e-02_rb/)
      kbo(:,23, 1) = (/ &
     &6.0534e-03_rb,6.4084e-03_rb,6.9154e-03_rb,7.6052e-03_rb,8.5112e-03_rb/)
      kbo(:,24, 1) = (/ &
     &5.0381e-03_rb,5.3630e-03_rb,5.8249e-03_rb,6.4486e-03_rb,7.2625e-03_rb/)
      kbo(:,25, 1) = (/ &
     &4.2012e-03_rb,4.4994e-03_rb,4.9231e-03_rb,5.4862e-03_rb,6.2185e-03_rb/)
      kbo(:,26, 1) = (/ &
     &3.5140e-03_rb,3.7917e-03_rb,4.1803e-03_rb,4.6907e-03_rb,5.3531e-03_rb/)
      kbo(:,27, 1) = (/ &
     &2.9461e-03_rb,3.2058e-03_rb,3.5595e-03_rb,4.0238e-03_rb,4.6243e-03_rb/)
      kbo(:,28, 1) = (/ &
     &2.4782e-03_rb,2.7200e-03_rb,3.0408e-03_rb,3.4639e-03_rb,4.0087e-03_rb/)
      kbo(:,29, 1) = (/ &
     &2.0935e-03_rb,2.3161e-03_rb,2.6092e-03_rb,2.9949e-03_rb,3.4943e-03_rb/)
      kbo(:,30, 1) = (/ &
     &1.7747e-03_rb,1.9777e-03_rb,2.2475e-03_rb,2.5999e-03_rb,3.0591e-03_rb/)
      kbo(:,31, 1) = (/ &
     &1.5106e-03_rb,1.6973e-03_rb,1.9445e-03_rb,2.2691e-03_rb,2.6924e-03_rb/)
      kbo(:,32, 1) = (/ &
     &1.2905e-03_rb,1.4629e-03_rb,1.6903e-03_rb,1.9914e-03_rb,2.3817e-03_rb/)
      kbo(:,33, 1) = (/ &
     &1.1070e-03_rb,1.2664e-03_rb,1.4773e-03_rb,1.7562e-03_rb,2.1183e-03_rb/)
      kbo(:,34, 1) = (/ &
     &9.4969e-04_rb,1.0951e-03_rb,1.2897e-03_rb,1.5465e-03_rb,1.8794e-03_rb/)
      kbo(:,35, 1) = (/ &
     &8.0650e-04_rb,9.3657e-04_rb,1.1113e-03_rb,1.3423e-03_rb,1.6411e-03_rb/)
      kbo(:,36, 1) = (/ &
     &6.7527e-04_rb,7.8828e-04_rb,9.4082e-04_rb,1.1424e-03_rb,1.4040e-03_rb/)
      kbo(:,37, 1) = (/ &
     &5.5374e-04_rb,6.4781e-04_rb,7.7522e-04_rb,9.4463e-04_rb,1.1654e-03_rb/)
      kbo(:,38, 1) = (/ &
     &4.5356e-04_rb,5.3144e-04_rb,6.3765e-04_rb,7.7978e-04_rb,9.6558e-04_rb/)
      kbo(:,39, 1) = (/ &
     &3.7155e-04_rb,4.3614e-04_rb,5.2472e-04_rb,6.4387e-04_rb,8.0047e-04_rb/)
      kbo(:,40, 1) = (/ &
     &2.9980e-04_rb,3.5165e-04_rb,4.2320e-04_rb,5.2024e-04_rb,6.4874e-04_rb/)
      kbo(:,41, 1) = (/ &
     &2.4145e-04_rb,2.8281e-04_rb,3.4028e-04_rb,4.1898e-04_rb,5.2376e-04_rb/)
      kbo(:,42, 1) = (/ &
     &1.9444e-04_rb,2.2740e-04_rb,2.7348e-04_rb,3.3721e-04_rb,4.2264e-04_rb/)
      kbo(:,43, 1) = (/ &
     &1.5533e-04_rb,1.8108e-04_rb,2.1726e-04_rb,2.6787e-04_rb,3.3649e-04_rb/)
      kbo(:,44, 1) = (/ &
     &1.2372e-04_rb,1.4362e-04_rb,1.7176e-04_rb,2.1151e-04_rb,2.6597e-04_rb/)
      kbo(:,45, 1) = (/ &
     &9.8615e-05_rb,1.1395e-04_rb,1.3575e-04_rb,1.6686e-04_rb,2.0989e-04_rb/)
      kbo(:,46, 1) = (/ &
     &7.8410e-05_rb,9.0146e-05_rb,1.0684e-04_rb,1.3092e-04_rb,1.6455e-04_rb/)
      kbo(:,47, 1) = (/ &
     &6.2111e-05_rb,7.0925e-05_rb,8.3491e-05_rb,1.0171e-04_rb,1.2752e-04_rb/)
      kbo(:,48, 1) = (/ &
     &4.9292e-05_rb,5.5878e-05_rb,6.5308e-05_rb,7.9022e-05_rb,9.8758e-05_rb/)
      kbo(:,49, 1) = (/ &
     &3.9189e-05_rb,4.4083e-05_rb,5.1134e-05_rb,6.1433e-05_rb,7.6433e-05_rb/)
      kbo(:,50, 1) = (/ &
     &3.1297e-05_rb,3.4944e-05_rb,4.0262e-05_rb,4.8024e-05_rb,5.9468e-05_rb/)
      kbo(:,51, 1) = (/ &
     &2.5040e-05_rb,2.7793e-05_rb,3.1794e-05_rb,3.7665e-05_rb,4.6363e-05_rb/)
      kbo(:,52, 1) = (/ &
     &2.0024e-05_rb,2.2137e-05_rb,2.5151e-05_rb,2.9572e-05_rb,3.6157e-05_rb/)
      kbo(:,53, 1) = (/ &
     &1.5989e-05_rb,1.7662e-05_rb,1.9908e-05_rb,2.3247e-05_rb,2.8202e-05_rb/)
      kbo(:,54, 1) = (/ &
     &1.2802e-05_rb,1.4140e-05_rb,1.5857e-05_rb,1.8384e-05_rb,2.2160e-05_rb/)
      kbo(:,55, 1) = (/ &
     &1.0251e-05_rb,1.1335e-05_rb,1.2664e-05_rb,1.4589e-05_rb,1.7471e-05_rb/)
      kbo(:,56, 1) = (/ &
     &8.1863e-06_rb,9.0810e-06_rb,1.0125e-05_rb,1.1592e-05_rb,1.3791e-05_rb/)
      kbo(:,57, 1) = (/ &
     &6.5183e-06_rb,7.2717e-06_rb,8.0938e-06_rb,9.2169e-06_rb,1.0895e-05_rb/)
      kbo(:,58, 1) = (/ &
     &5.1879e-06_rb,5.8228e-06_rb,6.4835e-06_rb,7.3485e-06_rb,8.6418e-06_rb/)
      kbo(:,59, 1) = (/ &
     &4.2168e-06_rb,4.7498e-06_rb,5.3015e-06_rb,6.0144e-06_rb,7.0853e-06_rb/)
      kbo(:,13, 2) = (/ &
     &1.3654e-01_rb,1.5163e-01_rb,1.6956e-01_rb,1.9071e-01_rb,2.1617e-01_rb/)
      kbo(:,14, 2) = (/ &
     &1.1559e-01_rb,1.2885e-01_rb,1.4454e-01_rb,1.6328e-01_rb,1.8606e-01_rb/)
      kbo(:,15, 2) = (/ &
     &9.7693e-02_rb,1.0924e-01_rb,1.2296e-01_rb,1.3968e-01_rb,1.6004e-01_rb/)
      kbo(:,16, 2) = (/ &
     &8.2470e-02_rb,9.2497e-02_rb,1.0461e-01_rb,1.1951e-01_rb,1.3759e-01_rb/)
      kbo(:,17, 2) = (/ &
     &6.9485e-02_rb,7.8236e-02_rb,8.9019e-02_rb,1.0217e-01_rb,1.1819e-01_rb/)
      kbo(:,18, 2) = (/ &
     &5.8434e-02_rb,6.6177e-02_rb,7.5740e-02_rb,8.7380e-02_rb,1.0143e-01_rb/)
      kbo(:,19, 2) = (/ &
     &4.9105e-02_rb,5.5950e-02_rb,6.4367e-02_rb,7.4577e-02_rb,8.6834e-02_rb/)
      kbo(:,20, 2) = (/ &
     &4.1358e-02_rb,4.7392e-02_rb,5.4755e-02_rb,6.3671e-02_rb,7.4385e-02_rb/)
      kbo(:,21, 2) = (/ &
     &3.4862e-02_rb,4.0138e-02_rb,4.6538e-02_rb,5.4275e-02_rb,6.3577e-02_rb/)
      kbo(:,22, 2) = (/ &
     &2.9678e-02_rb,3.4317e-02_rb,3.9913e-02_rb,4.6700e-02_rb,5.4863e-02_rb/)
      kbo(:,23, 2) = (/ &
     &2.5302e-02_rb,2.9357e-02_rb,3.4270e-02_rb,4.0209e-02_rb,4.7364e-02_rb/)
      kbo(:,24, 2) = (/ &
     &2.1615e-02_rb,2.5159e-02_rb,2.9473e-02_rb,3.4693e-02_rb,4.0992e-02_rb/)
      kbo(:,25, 2) = (/ &
     &1.8506e-02_rb,2.1620e-02_rb,2.5407e-02_rb,3.0032e-02_rb,3.5645e-02_rb/)
      kbo(:,26, 2) = (/ &
     &1.5911e-02_rb,1.8663e-02_rb,2.2016e-02_rb,2.6153e-02_rb,3.1176e-02_rb/)
      kbo(:,27, 2) = (/ &
     &1.3710e-02_rb,1.6139e-02_rb,1.9144e-02_rb,2.2842e-02_rb,2.7378e-02_rb/)
      kbo(:,28, 2) = (/ &
     &1.1834e-02_rb,1.3997e-02_rb,1.6695e-02_rb,2.0027e-02_rb,2.4111e-02_rb/)
      kbo(:,29, 2) = (/ &
     &1.0249e-02_rb,1.2190e-02_rb,1.4630e-02_rb,1.7647e-02_rb,2.1313e-02_rb/)
      kbo(:,30, 2) = (/ &
     &8.9036e-03_rb,1.0661e-02_rb,1.2868e-02_rb,1.5600e-02_rb,1.8888e-02_rb/)
      kbo(:,31, 2) = (/ &
     &7.7687e-03_rb,9.3699e-03_rb,1.1378e-02_rb,1.3839e-02_rb,1.6787e-02_rb/)
      kbo(:,32, 2) = (/ &
     &6.8140e-03_rb,8.2759e-03_rb,1.0101e-02_rb,1.2319e-02_rb,1.4958e-02_rb/)
      kbo(:,33, 2) = (/ &
     &6.0058e-03_rb,7.3417e-03_rb,8.9915e-03_rb,1.0990e-02_rb,1.3357e-02_rb/)
      kbo(:,34, 2) = (/ &
     &5.2812e-03_rb,6.4864e-03_rb,7.9715e-03_rb,9.7608e-03_rb,1.1880e-02_rb/)
      kbo(:,35, 2) = (/ &
     &4.5664e-03_rb,5.6316e-03_rb,6.9402e-03_rb,8.5171e-03_rb,1.0395e-02_rb/)
      kbo(:,36, 2) = (/ &
     &3.8650e-03_rb,4.7853e-03_rb,5.9182e-03_rb,7.2844e-03_rb,8.9146e-03_rb/)
      kbo(:,37, 2) = (/ &
     &3.1698e-03_rb,3.9391e-03_rb,4.8941e-03_rb,6.0480e-03_rb,7.4275e-03_rb/)
      kbo(:,38, 2) = (/ &
     &2.5939e-03_rb,3.2378e-03_rb,4.0393e-03_rb,5.0137e-03_rb,6.1786e-03_rb/)
      kbo(:,39, 2) = (/ &
     &2.1228e-03_rb,2.6601e-03_rb,3.3343e-03_rb,4.1558e-03_rb,5.1416e-03_rb/)
      kbo(:,40, 2) = (/ &
     &1.6967e-03_rb,2.1355e-03_rb,2.6888e-03_rb,3.3677e-03_rb,4.1883e-03_rb/)
      kbo(:,41, 2) = (/ &
     &1.3505e-03_rb,1.7067e-03_rb,2.1592e-03_rb,2.7184e-03_rb,3.3988e-03_rb/)
      kbo(:,42, 2) = (/ &
     &1.0736e-03_rb,1.3626e-03_rb,1.7319e-03_rb,2.1915e-03_rb,2.7553e-03_rb/)
      kbo(:,43, 2) = (/ &
     &8.4090e-04_rb,1.0714e-03_rb,1.3684e-03_rb,1.7414e-03_rb,2.2023e-03_rb/)
      kbo(:,44, 2) = (/ &
     &6.5390e-04_rb,8.3586e-04_rb,1.0728e-03_rb,1.3727e-03_rb,1.7466e-03_rb/)
      kbo(:,45, 2) = (/ &
     &5.0782e-04_rb,6.5105e-04_rb,8.3889e-04_rb,1.0793e-03_rb,1.3823e-03_rb/)
      kbo(:,46, 2) = (/ &
     &3.9179e-04_rb,5.0302e-04_rb,6.5132e-04_rb,8.4212e-04_rb,1.0854e-03_rb/)
      kbo(:,47, 2) = (/ &
     &2.9882e-04_rb,3.8363e-04_rb,4.9873e-04_rb,6.4829e-04_rb,8.4103e-04_rb/)
      kbo(:,48, 2) = (/ &
     &2.2778e-04_rb,2.9209e-04_rb,3.8081e-04_rb,4.9759e-04_rb,6.4942e-04_rb/)
      kbo(:,49, 2) = (/ &
     &1.7348e-04_rb,2.2201e-04_rb,2.8994e-04_rb,3.8059e-04_rb,4.9968e-04_rb/)
      kbo(:,50, 2) = (/ &
     &1.3291e-04_rb,1.6970e-04_rb,2.2169e-04_rb,2.9250e-04_rb,3.8590e-04_rb/)
      kbo(:,51, 2) = (/ &
     &1.0207e-04_rb,1.2991e-04_rb,1.6972e-04_rb,2.2476e-04_rb,2.9809e-04_rb/)
      kbo(:,52, 2) = (/ &
     &7.8428e-05_rb,9.9370e-05_rb,1.2969e-04_rb,1.7236e-04_rb,2.2955e-04_rb/)
      kbo(:,53, 2) = (/ &
     &6.0320e-05_rb,7.5906e-05_rb,9.8948e-05_rb,1.3170e-04_rb,1.7631e-04_rb/)
      kbo(:,54, 2) = (/ &
     &4.6701e-05_rb,5.8457e-05_rb,7.5987e-05_rb,1.0130e-04_rb,1.3626e-04_rb/)
      kbo(:,55, 2) = (/ &
     &3.6311e-05_rb,4.5177e-05_rb,5.8536e-05_rb,7.8091e-05_rb,1.0548e-04_rb/)
      kbo(:,56, 2) = (/ &
     &2.8265e-05_rb,3.4933e-05_rb,4.5042e-05_rb,6.0092e-05_rb,8.1465e-05_rb/)
      kbo(:,57, 2) = (/ &
     &2.2041e-05_rb,2.7013e-05_rb,3.4636e-05_rb,4.6160e-05_rb,6.2788e-05_rb/)
      kbo(:,58, 2) = (/ &
     &1.7267e-05_rb,2.0998e-05_rb,2.6754e-05_rb,3.5587e-05_rb,4.8495e-05_rb/)
      kbo(:,59, 2) = (/ &
     &1.3975e-05_rb,1.7004e-05_rb,2.1693e-05_rb,2.8956e-05_rb,3.9630e-05_rb/)
      kbo(:,13, 3) = (/ &
     &4.6584e-01_rb,5.5439e-01_rb,6.5378e-01_rb,7.6270e-01_rb,8.8077e-01_rb/)
      kbo(:,14, 3) = (/ &
     &4.0124e-01_rb,4.7657e-01_rb,5.6034e-01_rb,6.5177e-01_rb,7.5174e-01_rb/)
      kbo(:,15, 3) = (/ &
     &3.4513e-01_rb,4.0912e-01_rb,4.7944e-01_rb,5.5671e-01_rb,6.4124e-01_rb/)
      kbo(:,16, 3) = (/ &
     &2.9666e-01_rb,3.5062e-01_rb,4.0982e-01_rb,4.7509e-01_rb,5.4644e-01_rb/)
      kbo(:,17, 3) = (/ &
     &2.5400e-01_rb,2.9955e-01_rb,3.4952e-01_rb,4.0464e-01_rb,4.6504e-01_rb/)
      kbo(:,18, 3) = (/ &
     &2.1648e-01_rb,2.5489e-01_rb,2.9720e-01_rb,3.4406e-01_rb,3.9550e-01_rb/)
      kbo(:,19, 3) = (/ &
     &1.8385e-01_rb,2.1623e-01_rb,2.5204e-01_rb,2.9205e-01_rb,3.3599e-01_rb/)
      kbo(:,20, 3) = (/ &
     &1.5627e-01_rb,1.8359e-01_rb,2.1416e-01_rb,2.4849e-01_rb,2.8606e-01_rb/)
      kbo(:,21, 3) = (/ &
     &1.3276e-01_rb,1.5592e-01_rb,1.8216e-01_rb,2.1168e-01_rb,2.4388e-01_rb/)
      kbo(:,22, 3) = (/ &
     &1.1395e-01_rb,1.3395e-01_rb,1.5675e-01_rb,1.8217e-01_rb,2.0967e-01_rb/)
      kbo(:,23, 3) = (/ &
     &9.7815e-02_rb,1.1526e-01_rb,1.3514e-01_rb,1.5691e-01_rb,1.8033e-01_rb/)
      kbo(:,24, 3) = (/ &
     &8.4102e-02_rb,9.9408e-02_rb,1.1660e-01_rb,1.3523e-01_rb,1.5511e-01_rb/)
      kbo(:,25, 3) = (/ &
     &7.2584e-02_rb,8.5921e-02_rb,1.0065e-01_rb,1.1653e-01_rb,1.3347e-01_rb/)
      kbo(:,26, 3) = (/ &
     &6.2935e-02_rb,7.4476e-02_rb,8.7054e-02_rb,1.0059e-01_rb,1.1498e-01_rb/)
      kbo(:,27, 3) = (/ &
     &5.4628e-02_rb,6.4547e-02_rb,7.5255e-02_rb,8.6846e-02_rb,9.9043e-02_rb/)
      kbo(:,28, 3) = (/ &
     &4.7438e-02_rb,5.5913e-02_rb,6.5081e-02_rb,7.4969e-02_rb,8.5391e-02_rb/)
      kbo(:,29, 3) = (/ &
     &4.1247e-02_rb,4.8483e-02_rb,5.6377e-02_rb,6.4788e-02_rb,7.3775e-02_rb/)
      kbo(:,30, 3) = (/ &
     &3.5874e-02_rb,4.2080e-02_rb,4.8862e-02_rb,5.6085e-02_rb,6.3787e-02_rb/)
      kbo(:,31, 3) = (/ &
     &3.1224e-02_rb,3.6582e-02_rb,4.2396e-02_rb,4.8680e-02_rb,5.5240e-02_rb/)
      kbo(:,32, 3) = (/ &
     &2.7211e-02_rb,3.1854e-02_rb,3.6880e-02_rb,4.2315e-02_rb,4.7892e-02_rb/)
      kbo(:,33, 3) = (/ &
     &2.3744e-02_rb,2.7755e-02_rb,3.2147e-02_rb,3.6823e-02_rb,4.1518e-02_rb/)
      kbo(:,34, 3) = (/ &
     &2.0626e-02_rb,2.4111e-02_rb,2.7927e-02_rb,3.1884e-02_rb,3.5825e-02_rb/)
      kbo(:,35, 3) = (/ &
     &1.7661e-02_rb,2.0671e-02_rb,2.3952e-02_rb,2.7283e-02_rb,3.0580e-02_rb/)
      kbo(:,36, 3) = (/ &
     &1.4872e-02_rb,1.7453e-02_rb,2.0237e-02_rb,2.3045e-02_rb,2.5826e-02_rb/)
      kbo(:,37, 3) = (/ &
     &1.2231e-02_rb,1.4407e-02_rb,1.6754e-02_rb,1.9119e-02_rb,2.1470e-02_rb/)
      kbo(:,38, 3) = (/ &
     &1.0046e-02_rb,1.1883e-02_rb,1.3859e-02_rb,1.5852e-02_rb,1.7844e-02_rb/)
      kbo(:,39, 3) = (/ &
     &8.2562e-03_rb,9.8067e-03_rb,1.1470e-02_rb,1.3152e-02_rb,1.4841e-02_rb/)
      kbo(:,40, 3) = (/ &
     &6.6596e-03_rb,7.9555e-03_rb,9.3542e-03_rb,1.0774e-02_rb,1.2201e-02_rb/)
      kbo(:,41, 3) = (/ &
     &5.3569e-03_rb,6.4354e-03_rb,7.6095e-03_rb,8.8054e-03_rb,1.0012e-02_rb/)
      kbo(:,42, 3) = (/ &
     &4.3009e-03_rb,5.1973e-03_rb,6.1851e-03_rb,7.1918e-03_rb,8.2134e-03_rb/)
      kbo(:,43, 3) = (/ &
     &3.4093e-03_rb,4.1490e-03_rb,4.9732e-03_rb,5.8239e-03_rb,6.6848e-03_rb/)
      kbo(:,44, 3) = (/ &
     &2.6823e-03_rb,3.2897e-03_rb,3.9739e-03_rb,4.6907e-03_rb,5.4167e-03_rb/)
      kbo(:,45, 3) = (/ &
     &2.1046e-03_rb,2.6003e-03_rb,3.1662e-03_rb,3.7704e-03_rb,4.3820e-03_rb/)
      kbo(:,46, 3) = (/ &
     &1.6388e-03_rb,2.0403e-03_rb,2.5048e-03_rb,3.0127e-03_rb,3.5286e-03_rb/)
      kbo(:,47, 3) = (/ &
     &1.2595e-03_rb,1.5807e-03_rb,1.9580e-03_rb,2.3819e-03_rb,2.8168e-03_rb/)
      kbo(:,48, 3) = (/ &
     &9.6423e-04_rb,1.2197e-03_rb,1.5249e-03_rb,1.8746e-03_rb,2.2418e-03_rb/)
      kbo(:,49, 3) = (/ &
     &7.3527e-04_rb,9.3718e-04_rb,1.1821e-03_rb,1.4684e-03_rb,1.7777e-03_rb/)
      kbo(:,50, 3) = (/ &
     &5.6217e-04_rb,7.2217e-04_rb,9.1835e-04_rb,1.1512e-03_rb,1.4112e-03_rb/)
      kbo(:,51, 3) = (/ &
     &4.2967e-04_rb,5.5583e-04_rb,7.1300e-04_rb,9.0190e-04_rb,1.1184e-03_rb/)
      kbo(:,52, 3) = (/ &
     &3.2742e-04_rb,4.2631e-04_rb,5.5137e-04_rb,7.0406e-04_rb,8.8313e-04_rb/)
      kbo(:,53, 3) = (/ &
     &2.4868e-04_rb,3.2582e-04_rb,4.2465e-04_rb,5.4760e-04_rb,6.9409e-04_rb/)
      kbo(:,54, 3) = (/ &
     &1.8974e-04_rb,2.5027e-04_rb,3.2861e-04_rb,4.2756e-04_rb,5.4716e-04_rb/)
      kbo(:,55, 3) = (/ &
     &1.4494e-04_rb,1.9239e-04_rb,2.5442e-04_rb,3.3395e-04_rb,4.3136e-04_rb/)
      kbo(:,56, 3) = (/ &
     &1.1043e-04_rb,1.4756e-04_rb,1.9639e-04_rb,2.5993e-04_rb,3.3900e-04_rb/)
      kbo(:,57, 3) = (/ &
     &8.3928e-05_rb,1.1287e-04_rb,1.5113e-04_rb,2.0155e-04_rb,2.6555e-04_rb/)
      kbo(:,58, 3) = (/ &
     &6.4018e-05_rb,8.6477e-05_rb,1.1648e-04_rb,1.5666e-04_rb,2.0830e-04_rb/)
      kbo(:,59, 3) = (/ &
     &5.1497e-05_rb,6.9896e-05_rb,9.4663e-05_rb,1.2810e-04_rb,1.7150e-04_rb/)
      kbo(:,13, 4) = (/ &
     &1.7524e+00_rb,1.9492e+00_rb,2.1512e+00_rb,2.3564e+00_rb,2.5478e+00_rb/)
      kbo(:,14, 4) = (/ &
     &1.4772e+00_rb,1.6494e+00_rb,1.8280e+00_rb,2.0044e+00_rb,2.1600e+00_rb/)
      kbo(:,15, 4) = (/ &
     &1.2495e+00_rb,1.4008e+00_rb,1.5559e+00_rb,1.6974e+00_rb,1.8203e+00_rb/)
      kbo(:,16, 4) = (/ &
     &1.0605e+00_rb,1.1902e+00_rb,1.3172e+00_rb,1.4293e+00_rb,1.5280e+00_rb/)
      kbo(:,17, 4) = (/ &
     &8.9958e-01_rb,1.0068e+00_rb,1.1092e+00_rb,1.1993e+00_rb,1.2798e+00_rb/)
      kbo(:,18, 4) = (/ &
     &7.6135e-01_rb,8.4970e-01_rb,9.3198e-01_rb,1.0052e+00_rb,1.0709e+00_rb/)
      kbo(:,19, 4) = (/ &
     &6.4340e-01_rb,7.1592e-01_rb,7.8258e-01_rb,8.4177e-01_rb,8.9530e-01_rb/)
      kbo(:,20, 4) = (/ &
     &5.4418e-01_rb,6.0357e-01_rb,6.5718e-01_rb,7.0498e-01_rb,7.4918e-01_rb/)
      kbo(:,21, 4) = (/ &
     &4.5911e-01_rb,5.0742e-01_rb,5.5063e-01_rb,5.8988e-01_rb,6.2662e-01_rb/)
      kbo(:,22, 4) = (/ &
     &3.8906e-01_rb,4.2787e-01_rb,4.6285e-01_rb,4.9521e-01_rb,5.2615e-01_rb/)
      kbo(:,23, 4) = (/ &
     &3.2912e-01_rb,3.6040e-01_rb,3.8895e-01_rb,4.1593e-01_rb,4.4229e-01_rb/)
      kbo(:,24, 4) = (/ &
     &2.7830e-01_rb,3.0371e-01_rb,3.2729e-01_rb,3.5001e-01_rb,3.7213e-01_rb/)
      kbo(:,25, 4) = (/ &
     &2.3534e-01_rb,2.5620e-01_rb,2.7595e-01_rb,2.9506e-01_rb,3.1323e-01_rb/)
      kbo(:,26, 4) = (/ &
     &1.9937e-01_rb,2.1667e-01_rb,2.3330e-01_rb,2.4915e-01_rb,2.6395e-01_rb/)
      kbo(:,27, 4) = (/ &
     &1.6896e-01_rb,1.8346e-01_rb,1.9732e-01_rb,2.1038e-01_rb,2.2245e-01_rb/)
      kbo(:,28, 4) = (/ &
     &1.4324e-01_rb,1.5539e-01_rb,1.6686e-01_rb,1.7760e-01_rb,1.8754e-01_rb/)
      kbo(:,29, 4) = (/ &
     &1.2158e-01_rb,1.3162e-01_rb,1.4109e-01_rb,1.4990e-01_rb,1.5824e-01_rb/)
      kbo(:,30, 4) = (/ &
     &1.0319e-01_rb,1.1149e-01_rb,1.1927e-01_rb,1.2651e-01_rb,1.3374e-01_rb/)
      kbo(:,31, 4) = (/ &
     &8.7596e-02_rb,9.4445e-02_rb,1.0083e-01_rb,1.0683e-01_rb,1.1324e-01_rb/)
      kbo(:,32, 4) = (/ &
     &7.4371e-02_rb,7.9986e-02_rb,8.5218e-02_rb,9.0404e-02_rb,9.6151e-02_rb/)
      kbo(:,33, 4) = (/ &
     &6.3127e-02_rb,6.7754e-02_rb,7.2100e-02_rb,7.6670e-02_rb,8.1947e-02_rb/)
      kbo(:,34, 4) = (/ &
     &5.3448e-02_rb,5.7257e-02_rb,6.0976e-02_rb,6.5117e-02_rb,6.9920e-02_rb/)
      kbo(:,35, 4) = (/ &
     &4.4958e-02_rb,4.8132e-02_rb,5.1386e-02_rb,5.5110e-02_rb,5.9396e-02_rb/)
      kbo(:,36, 4) = (/ &
     &3.7535e-02_rb,4.0225e-02_rb,4.3084e-02_rb,4.6370e-02_rb,5.0142e-02_rb/)
      kbo(:,37, 4) = (/ &
     &3.1000e-02_rb,3.3296e-02_rb,3.5751e-02_rb,3.8593e-02_rb,4.1835e-02_rb/)
      kbo(:,38, 4) = (/ &
     &2.5584e-02_rb,2.7542e-02_rb,2.9661e-02_rb,3.2102e-02_rb,3.4896e-02_rb/)
      kbo(:,39, 4) = (/ &
     &2.1117e-02_rb,2.2790e-02_rb,2.4613e-02_rb,2.6708e-02_rb,2.9115e-02_rb/)
      kbo(:,40, 4) = (/ &
     &1.7285e-02_rb,1.8708e-02_rb,2.0253e-02_rb,2.2025e-02_rb,2.4072e-02_rb/)
      kbo(:,41, 4) = (/ &
     &1.4124e-02_rb,1.5332e-02_rb,1.6640e-02_rb,1.8138e-02_rb,1.9869e-02_rb/)
      kbo(:,42, 4) = (/ &
     &1.1535e-02_rb,1.2562e-02_rb,1.3665e-02_rb,1.4929e-02_rb,1.6393e-02_rb/)
      kbo(:,43, 4) = (/ &
     &9.3628e-03_rb,1.0238e-02_rb,1.1164e-02_rb,1.2221e-02_rb,1.3452e-02_rb/)
      kbo(:,44, 4) = (/ &
     &7.5734e-03_rb,8.3195e-03_rb,9.0965e-03_rb,9.9764e-03_rb,1.1001e-02_rb/)
      kbo(:,45, 4) = (/ &
     &6.1205e-03_rb,6.7536e-03_rb,7.4083e-03_rb,8.1381e-03_rb,8.9936e-03_rb/)
      kbo(:,46, 4) = (/ &
     &4.9262e-03_rb,5.4662e-03_rb,6.0165e-03_rb,6.6206e-03_rb,7.3272e-03_rb/)
      kbo(:,47, 4) = (/ &
     &3.9391e-03_rb,4.3970e-03_rb,4.8610e-03_rb,5.3580e-03_rb,5.9369e-03_rb/)
      kbo(:,48, 4) = (/ &
     &3.1403e-03_rb,3.5287e-03_rb,3.9210e-03_rb,4.3323e-03_rb,4.8039e-03_rb/)
      kbo(:,49, 4) = (/ &
     &2.4956e-03_rb,2.8254e-03_rb,3.1568e-03_rb,3.4981e-03_rb,3.8839e-03_rb/)
      kbo(:,50, 4) = (/ &
     &1.9828e-03_rb,2.2636e-03_rb,2.5424e-03_rb,2.8288e-03_rb,3.1441e-03_rb/)
      kbo(:,51, 4) = (/ &
     &1.5714e-03_rb,1.8113e-03_rb,2.0459e-03_rb,2.2868e-03_rb,2.5462e-03_rb/)
      kbo(:,52, 4) = (/ &
     &1.2395e-03_rb,1.4453e-03_rb,1.6428e-03_rb,1.8457e-03_rb,2.0598e-03_rb/)
      kbo(:,53, 4) = (/ &
     &9.7290e-04_rb,1.1497e-03_rb,1.3161e-03_rb,1.4871e-03_rb,1.6649e-03_rb/)
      kbo(:,54, 4) = (/ &
     &7.6445e-04_rb,9.1516e-04_rb,1.0560e-03_rb,1.2000e-03_rb,1.3484e-03_rb/)
      kbo(:,55, 4) = (/ &
     &5.9991e-04_rb,7.2766e-04_rb,8.4765e-04_rb,9.6806e-04_rb,1.0927e-03_rb/)
      kbo(:,56, 4) = (/ &
     &4.6862e-04_rb,5.7638e-04_rb,6.7857e-04_rb,7.7940e-04_rb,8.8453e-04_rb/)
      kbo(:,57, 4) = (/ &
     &3.6453e-04_rb,4.5447e-04_rb,5.4199e-04_rb,6.2645e-04_rb,7.1474e-04_rb/)
      kbo(:,58, 4) = (/ &
     &2.8389e-04_rb,3.5818e-04_rb,4.3272e-04_rb,5.0377e-04_rb,5.7789e-04_rb/)
      kbo(:,59, 4) = (/ &
     &2.3060e-04_rb,2.9282e-04_rb,3.5542e-04_rb,4.1534e-04_rb,4.7810e-04_rb/)
      kbo(:,13, 5) = (/ &
     &4.6094e+00_rb,4.6772e+00_rb,4.7246e+00_rb,4.7737e+00_rb,4.8549e+00_rb/)
      kbo(:,14, 5) = (/ &
     &3.8278e+00_rb,3.8806e+00_rb,3.9251e+00_rb,3.9810e+00_rb,4.0718e+00_rb/)
      kbo(:,15, 5) = (/ &
     &3.1722e+00_rb,3.2168e+00_rb,3.2634e+00_rb,3.3308e+00_rb,3.4210e+00_rb/)
      kbo(:,16, 5) = (/ &
     &2.6249e+00_rb,2.6705e+00_rb,2.7237e+00_rb,2.7913e+00_rb,2.8656e+00_rb/)
      kbo(:,17, 5) = (/ &
     &2.1764e+00_rb,2.2249e+00_rb,2.2741e+00_rb,2.3304e+00_rb,2.3919e+00_rb/)
      kbo(:,18, 5) = (/ &
     &1.8098e+00_rb,1.8520e+00_rb,1.8935e+00_rb,1.9392e+00_rb,1.9922e+00_rb/)
      kbo(:,19, 5) = (/ &
     &1.5044e+00_rb,1.5380e+00_rb,1.5723e+00_rb,1.6122e+00_rb,1.6588e+00_rb/)
      kbo(:,20, 5) = (/ &
     &1.2493e+00_rb,1.2764e+00_rb,1.3059e+00_rb,1.3405e+00_rb,1.3817e+00_rb/)
      kbo(:,21, 5) = (/ &
     &1.0373e+00_rb,1.0597e+00_rb,1.0855e+00_rb,1.1148e+00_rb,1.1513e+00_rb/)
      kbo(:,22, 5) = (/ &
     &8.6282e-01_rb,8.8188e-01_rb,9.0343e-01_rb,9.2979e-01_rb,9.6231e-01_rb/)
      kbo(:,23, 5) = (/ &
     &7.1704e-01_rb,7.3333e-01_rb,7.5232e-01_rb,7.7633e-01_rb,8.0551e-01_rb/)
      kbo(:,24, 5) = (/ &
     &5.9518e-01_rb,6.0940e-01_rb,6.2643e-01_rb,6.4862e-01_rb,6.7508e-01_rb/)
      kbo(:,25, 5) = (/ &
     &4.9422e-01_rb,5.0656e-01_rb,5.2248e-01_rb,5.4283e-01_rb,5.6739e-01_rb/)
      kbo(:,26, 5) = (/ &
     &4.1037e-01_rb,4.2180e-01_rb,4.3687e-01_rb,4.5588e-01_rb,4.7822e-01_rb/)
      kbo(:,27, 5) = (/ &
     &3.4124e-01_rb,3.5223e-01_rb,3.6648e-01_rb,3.8368e-01_rb,4.0379e-01_rb/)
      kbo(:,28, 5) = (/ &
     &2.8464e-01_rb,2.9501e-01_rb,3.0821e-01_rb,3.2362e-01_rb,3.4157e-01_rb/)
      kbo(:,29, 5) = (/ &
     &2.3808e-01_rb,2.4801e-01_rb,2.5984e-01_rb,2.7366e-01_rb,2.8965e-01_rb/)
      kbo(:,30, 5) = (/ &
     &1.9967e-01_rb,2.0890e-01_rb,2.1955e-01_rb,2.3198e-01_rb,2.4657e-01_rb/)
      kbo(:,31, 5) = (/ &
     &1.6792e-01_rb,1.7624e-01_rb,1.8593e-01_rb,1.9729e-01_rb,2.1071e-01_rb/)
      kbo(:,32, 5) = (/ &
     &1.4147e-01_rb,1.4902e-01_rb,1.5786e-01_rb,1.6833e-01_rb,1.8092e-01_rb/)
      kbo(:,33, 5) = (/ &
     &1.1942e-01_rb,1.2630e-01_rb,1.3442e-01_rb,1.4414e-01_rb,1.5600e-01_rb/)
      kbo(:,34, 5) = (/ &
     &1.0087e-01_rb,1.0707e-01_rb,1.1451e-01_rb,1.2366e-01_rb,1.3474e-01_rb/)
      kbo(:,35, 5) = (/ &
     &8.4860e-02_rb,9.0419e-02_rb,9.7193e-02_rb,1.0562e-01_rb,1.1587e-01_rb/)
      kbo(:,36, 5) = (/ &
     &7.0999e-02_rb,7.5907e-02_rb,8.1988e-02_rb,8.9567e-02_rb,9.8842e-02_rb/)
      kbo(:,37, 5) = (/ &
     &5.8886e-02_rb,6.3102e-02_rb,6.8391e-02_rb,7.5008e-02_rb,8.3160e-02_rb/)
      kbo(:,38, 5) = (/ &
     &4.8832e-02_rb,5.2452e-02_rb,5.7040e-02_rb,6.2826e-02_rb,7.0009e-02_rb/)
      kbo(:,39, 5) = (/ &
     &4.0501e-02_rb,4.3616e-02_rb,4.7608e-02_rb,5.2697e-02_rb,5.9016e-02_rb/)
      kbo(:,40, 5) = (/ &
     &3.3361e-02_rb,3.5983e-02_rb,3.9374e-02_rb,4.3750e-02_rb,4.9196e-02_rb/)
      kbo(:,41, 5) = (/ &
     &2.7445e-02_rb,2.9649e-02_rb,3.2516e-02_rb,3.6256e-02_rb,4.0930e-02_rb/)
      kbo(:,42, 5) = (/ &
     &2.2572e-02_rb,2.4412e-02_rb,2.6829e-02_rb,3.0022e-02_rb,3.4040e-02_rb/)
      kbo(:,43, 5) = (/ &
     &1.8476e-02_rb,1.9991e-02_rb,2.1997e-02_rb,2.4673e-02_rb,2.8080e-02_rb/)
      kbo(:,44, 5) = (/ &
     &1.5092e-02_rb,1.6330e-02_rb,1.7973e-02_rb,2.0194e-02_rb,2.3058e-02_rb/)
      kbo(:,45, 5) = (/ &
     &1.2321e-02_rb,1.3334e-02_rb,1.4674e-02_rb,1.6507e-02_rb,1.8911e-02_rb/)
      kbo(:,46, 5) = (/ &
     &1.0036e-02_rb,1.0861e-02_rb,1.1940e-02_rb,1.3440e-02_rb,1.5434e-02_rb/)
      kbo(:,47, 5) = (/ &
     &8.1439e-03_rb,8.8117e-03_rb,9.6681e-03_rb,1.0871e-02_rb,1.2500e-02_rb/)
      kbo(:,48, 5) = (/ &
     &6.6059e-03_rb,7.1462e-03_rb,7.8259e-03_rb,8.7835e-03_rb,1.0106e-02_rb/)
      kbo(:,49, 5) = (/ &
     &5.3548e-03_rb,5.7913e-03_rb,6.3343e-03_rb,7.0935e-03_rb,8.1581e-03_rb/)
      kbo(:,50, 5) = (/ &
     &4.3482e-03_rb,4.7014e-03_rb,5.1411e-03_rb,5.7429e-03_rb,6.6034e-03_rb/)
      kbo(:,51, 5) = (/ &
     &3.5331e-03_rb,3.8185e-03_rb,4.1765e-03_rb,4.6543e-03_rb,5.3466e-03_rb/)
      kbo(:,52, 5) = (/ &
     &2.8695e-03_rb,3.1007e-03_rb,3.3920e-03_rb,3.7708e-03_rb,4.3257e-03_rb/)
      kbo(:,53, 5) = (/ &
     &2.3291e-03_rb,2.5165e-03_rb,2.7524e-03_rb,3.0532e-03_rb,3.4935e-03_rb/)
      kbo(:,54, 5) = (/ &
     &1.8925e-03_rb,2.0468e-03_rb,2.2382e-03_rb,2.4801e-03_rb,2.8323e-03_rb/)
      kbo(:,55, 5) = (/ &
     &1.5385e-03_rb,1.6657e-03_rb,1.8210e-03_rb,2.0171e-03_rb,2.2988e-03_rb/)
      kbo(:,56, 5) = (/ &
     &1.2499e-03_rb,1.3548e-03_rb,1.4808e-03_rb,1.6398e-03_rb,1.8634e-03_rb/)
      kbo(:,57, 5) = (/ &
     &1.0138e-03_rb,1.1010e-03_rb,1.2032e-03_rb,1.3320e-03_rb,1.5093e-03_rb/)
      kbo(:,58, 5) = (/ &
     &8.2159e-04_rb,8.9516e-04_rb,9.7844e-04_rb,1.0832e-03_rb,1.2245e-03_rb/)
      kbo(:,59, 5) = (/ &
     &6.7629e-04_rb,7.3875e-04_rb,8.0921e-04_rb,8.9812e-04_rb,1.0190e-03_rb/)
      kbo(:,13, 6) = (/ &
     &8.6109e+00_rb,8.6045e+00_rb,8.6391e+00_rb,8.7159e+00_rb,8.8472e+00_rb/)
      kbo(:,14, 6) = (/ &
     &7.2012e+00_rb,7.1992e+00_rb,7.2347e+00_rb,7.3193e+00_rb,7.4652e+00_rb/)
      kbo(:,15, 6) = (/ &
     &6.0036e+00_rb,6.0084e+00_rb,6.0525e+00_rb,6.1531e+00_rb,6.3134e+00_rb/)
      kbo(:,16, 6) = (/ &
     &4.9860e+00_rb,4.9990e+00_rb,5.0571e+00_rb,5.1776e+00_rb,5.3476e+00_rb/)
      kbo(:,17, 6) = (/ &
     &4.1286e+00_rb,4.1532e+00_rb,4.2350e+00_rb,4.3671e+00_rb,4.5277e+00_rb/)
      kbo(:,18, 6) = (/ &
     &3.4143e+00_rb,3.4596e+00_rb,3.5560e+00_rb,3.6799e+00_rb,3.8169e+00_rb/)
      kbo(:,19, 6) = (/ &
     &2.8306e+00_rb,2.8919e+00_rb,2.9820e+00_rb,3.0877e+00_rb,3.2050e+00_rb/)
      kbo(:,20, 6) = (/ &
     &2.3552e+00_rb,2.4158e+00_rb,2.4951e+00_rb,2.5846e+00_rb,2.6876e+00_rb/)
      kbo(:,21, 6) = (/ &
     &1.9600e+00_rb,2.0149e+00_rb,2.0826e+00_rb,2.1623e+00_rb,2.2526e+00_rb/)
      kbo(:,22, 6) = (/ &
     &1.6326e+00_rb,1.6825e+00_rb,1.7432e+00_rb,1.8140e+00_rb,1.8956e+00_rb/)
      kbo(:,23, 6) = (/ &
     &1.3616e+00_rb,1.4065e+00_rb,1.4611e+00_rb,1.5245e+00_rb,1.5974e+00_rb/)
      kbo(:,24, 6) = (/ &
     &1.1379e+00_rb,1.1784e+00_rb,1.2266e+00_rb,1.2828e+00_rb,1.3489e+00_rb/)
      kbo(:,25, 6) = (/ &
     &9.5263e-01_rb,9.8858e-01_rb,1.0309e+00_rb,1.0818e+00_rb,1.1428e+00_rb/)
      kbo(:,26, 6) = (/ &
     &7.9793e-01_rb,8.2975e-01_rb,8.6871e-01_rb,9.1620e-01_rb,9.7213e-01_rb/)
      kbo(:,27, 6) = (/ &
     &6.6853e-01_rb,6.9732e-01_rb,7.3341e-01_rb,7.7799e-01_rb,8.3032e-01_rb/)
      kbo(:,28, 6) = (/ &
     &5.6032e-01_rb,5.8734e-01_rb,6.2174e-01_rb,6.6339e-01_rb,7.1179e-01_rb/)
      kbo(:,29, 6) = (/ &
     &4.7080e-01_rb,4.9684e-01_rb,5.2958e-01_rb,5.6837e-01_rb,6.1280e-01_rb/)
      kbo(:,30, 6) = (/ &
     &3.9732e-01_rb,4.2229e-01_rb,4.5298e-01_rb,4.8884e-01_rb,5.2914e-01_rb/)
      kbo(:,31, 6) = (/ &
     &3.3719e-01_rb,3.6099e-01_rb,3.8930e-01_rb,4.2211e-01_rb,4.5851e-01_rb/)
      kbo(:,32, 6) = (/ &
     &2.8780e-01_rb,3.1000e-01_rb,3.3607e-01_rb,3.6581e-01_rb,3.9865e-01_rb/)
      kbo(:,33, 6) = (/ &
     &2.4698e-01_rb,2.6737e-01_rb,2.9124e-01_rb,3.1811e-01_rb,3.4839e-01_rb/)
      kbo(:,34, 6) = (/ &
     &2.1208e-01_rb,2.3079e-01_rb,2.5250e-01_rb,2.7695e-01_rb,3.0517e-01_rb/)
      kbo(:,35, 6) = (/ &
     &1.8116e-01_rb,1.9828e-01_rb,2.1775e-01_rb,2.3997e-01_rb,2.6607e-01_rb/)
      kbo(:,36, 6) = (/ &
     &1.5361e-01_rb,1.6893e-01_rb,1.8633e-01_rb,2.0647e-01_rb,2.3023e-01_rb/)
      kbo(:,37, 6) = (/ &
     &1.2853e-01_rb,1.4194e-01_rb,1.5730e-01_rb,1.7520e-01_rb,1.9644e-01_rb/)
      kbo(:,38, 6) = (/ &
     &1.0750e-01_rb,1.1924e-01_rb,1.3275e-01_rb,1.4862e-01_rb,1.6766e-01_rb/)
      kbo(:,39, 6) = (/ &
     &8.9946e-02_rb,1.0018e-01_rb,1.1208e-01_rb,1.2616e-01_rb,1.4325e-01_rb/)
      kbo(:,40, 6) = (/ &
     &7.4468e-02_rb,8.3253e-02_rb,9.3534e-02_rb,1.0578e-01_rb,1.2083e-01_rb/)
      kbo(:,41, 6) = (/ &
     &6.1560e-02_rb,6.9060e-02_rb,7.7885e-02_rb,8.8512e-02_rb,1.0168e-01_rb/)
      kbo(:,42, 6) = (/ &
     &5.0884e-02_rb,5.7270e-02_rb,6.4836e-02_rb,7.4017e-02_rb,8.5530e-02_rb/)
      kbo(:,43, 6) = (/ &
     &4.1741e-02_rb,4.7136e-02_rb,5.3555e-02_rb,6.1413e-02_rb,7.1280e-02_rb/)
      kbo(:,44, 6) = (/ &
     &3.4114e-02_rb,3.8637e-02_rb,4.4075e-02_rb,5.0720e-02_rb,5.9134e-02_rb/)
      kbo(:,45, 6) = (/ &
     &2.7843e-02_rb,3.1631e-02_rb,3.6225e-02_rb,4.1848e-02_rb,4.9013e-02_rb/)
      kbo(:,46, 6) = (/ &
     &2.2624e-02_rb,2.5773e-02_rb,2.9633e-02_rb,3.4369e-02_rb,4.0405e-02_rb/)
      kbo(:,47, 6) = (/ &
     &1.8260e-02_rb,2.0843e-02_rb,2.4043e-02_rb,2.7985e-02_rb,3.3023e-02_rb/)
      kbo(:,48, 6) = (/ &
     &1.4711e-02_rb,1.6820e-02_rb,1.9466e-02_rb,2.2727e-02_rb,2.6914e-02_rb/)
      kbo(:,49, 6) = (/ &
     &1.1827e-02_rb,1.3542e-02_rb,1.5709e-02_rb,1.8410e-02_rb,2.1875e-02_rb/)
      kbo(:,50, 6) = (/ &
     &9.5219e-03_rb,1.0923e-02_rb,1.2699e-02_rb,1.4938e-02_rb,1.7808e-02_rb/)
      kbo(:,51, 6) = (/ &
     &7.6687e-03_rb,8.8049e-03_rb,1.0259e-02_rb,1.2113e-02_rb,1.4495e-02_rb/)
      kbo(:,52, 6) = (/ &
     &6.1732e-03_rb,7.0853e-03_rb,8.2719e-03_rb,9.7966e-03_rb,1.1770e-02_rb/)
      kbo(:,53, 6) = (/ &
     &4.9698e-03_rb,5.6903e-03_rb,6.6530e-03_rb,7.9081e-03_rb,9.5294e-03_rb/)
      kbo(:,54, 6) = (/ &
     &4.0149e-03_rb,4.5833e-03_rb,5.3685e-03_rb,6.4021e-03_rb,7.7450e-03_rb/)
      kbo(:,55, 6) = (/ &
     &3.2497e-03_rb,3.6951e-03_rb,4.3359e-03_rb,5.1870e-03_rb,6.3011e-03_rb/)
      kbo(:,56, 6) = (/ &
     &2.6325e-03_rb,2.9773e-03_rb,3.4964e-03_rb,4.1937e-03_rb,5.1170e-03_rb/)
      kbo(:,57, 6) = (/ &
     &2.1354e-03_rb,2.3977e-03_rb,2.8151e-03_rb,3.3845e-03_rb,4.1468e-03_rb/)
      kbo(:,58, 6) = (/ &
     &1.7364e-03_rb,1.9358e-03_rb,2.2698e-03_rb,2.7348e-03_rb,3.3639e-03_rb/)
      kbo(:,59, 6) = (/ &
     &1.4324e-03_rb,1.6023e-03_rb,1.8879e-03_rb,2.2888e-03_rb,2.8370e-03_rb/)
      kbo(:,13, 7) = (/ &
     &1.7993e+01_rb,1.8373e+01_rb,1.8772e+01_rb,1.9157e+01_rb,1.9544e+01_rb/)
      kbo(:,14, 7) = (/ &
     &1.5395e+01_rb,1.5713e+01_rb,1.6031e+01_rb,1.6348e+01_rb,1.6675e+01_rb/)
      kbo(:,15, 7) = (/ &
     &1.3087e+01_rb,1.3328e+01_rb,1.3586e+01_rb,1.3851e+01_rb,1.4166e+01_rb/)
      kbo(:,16, 7) = (/ &
     &1.1053e+01_rb,1.1254e+01_rb,1.1469e+01_rb,1.1711e+01_rb,1.2037e+01_rb/)
      kbo(:,17, 7) = (/ &
     &9.3076e+00_rb,9.4744e+00_rb,9.6627e+00_rb,9.9088e+00_rb,1.0257e+01_rb/)
      kbo(:,18, 7) = (/ &
     &7.8193e+00_rb,7.9665e+00_rb,8.1534e+00_rb,8.4211e+00_rb,8.7715e+00_rb/)
      kbo(:,19, 7) = (/ &
     &6.5448e+00_rb,6.6861e+00_rb,6.8916e+00_rb,7.1741e+00_rb,7.4962e+00_rb/)
      kbo(:,20, 7) = (/ &
     &5.4651e+00_rb,5.6202e+00_rb,5.8422e+00_rb,6.1124e+00_rb,6.3952e+00_rb/)
      kbo(:,21, 7) = (/ &
     &4.5716e+00_rb,4.7392e+00_rb,4.9589e+00_rb,5.1951e+00_rb,5.4343e+00_rb/)
      kbo(:,22, 7) = (/ &
     &3.8421e+00_rb,4.0181e+00_rb,4.2111e+00_rb,4.4111e+00_rb,4.6135e+00_rb/)
      kbo(:,23, 7) = (/ &
     &3.2420e+00_rb,3.4014e+00_rb,3.5676e+00_rb,3.7366e+00_rb,3.9163e+00_rb/)
      kbo(:,24, 7) = (/ &
     &2.7372e+00_rb,2.8745e+00_rb,3.0185e+00_rb,3.1680e+00_rb,3.3279e+00_rb/)
      kbo(:,25, 7) = (/ &
     &2.3109e+00_rb,2.4310e+00_rb,2.5564e+00_rb,2.6909e+00_rb,2.8371e+00_rb/)
      kbo(:,26, 7) = (/ &
     &1.9553e+00_rb,2.0603e+00_rb,2.1726e+00_rb,2.2928e+00_rb,2.4305e+00_rb/)
      kbo(:,27, 7) = (/ &
     &1.6578e+00_rb,1.7514e+00_rb,1.8510e+00_rb,1.9617e+00_rb,2.0943e+00_rb/)
      kbo(:,28, 7) = (/ &
     &1.4096e+00_rb,1.4926e+00_rb,1.5811e+00_rb,1.6863e+00_rb,1.8139e+00_rb/)
      kbo(:,29, 7) = (/ &
     &1.2014e+00_rb,1.2743e+00_rb,1.3570e+00_rb,1.4576e+00_rb,1.5798e+00_rb/)
      kbo(:,30, 7) = (/ &
     &1.0255e+00_rb,1.0913e+00_rb,1.1701e+00_rb,1.2677e+00_rb,1.3837e+00_rb/)
      kbo(:,31, 7) = (/ &
     &8.7693e-01_rb,9.3909e-01_rb,1.0166e+00_rb,1.1099e+00_rb,1.2195e+00_rb/)
      kbo(:,32, 7) = (/ &
     &7.5259e-01_rb,8.1353e-01_rb,8.8887e-01_rb,9.7843e-01_rb,1.0818e+00_rb/)
      kbo(:,33, 7) = (/ &
     &6.4947e-01_rb,7.0941e-01_rb,7.8262e-01_rb,8.6809e-01_rb,9.6577e-01_rb/)
      kbo(:,34, 7) = (/ &
     &5.6287e-01_rb,6.2118e-01_rb,6.9115e-01_rb,7.7236e-01_rb,8.6471e-01_rb/)
      kbo(:,35, 7) = (/ &
     &4.8662e-01_rb,5.4191e-01_rb,6.0756e-01_rb,6.8389e-01_rb,7.7035e-01_rb/)
      kbo(:,36, 7) = (/ &
     &4.1799e-01_rb,4.6899e-01_rb,5.2971e-01_rb,6.0005e-01_rb,6.8040e-01_rb/)
      kbo(:,37, 7) = (/ &
     &3.5425e-01_rb,3.9998e-01_rb,4.5446e-01_rb,5.1797e-01_rb,5.9123e-01_rb/)
      kbo(:,38, 7) = (/ &
     &3.0041e-01_rb,3.4120e-01_rb,3.9013e-01_rb,4.4734e-01_rb,5.1414e-01_rb/)
      kbo(:,39, 7) = (/ &
     &2.5510e-01_rb,2.9152e-01_rb,3.3528e-01_rb,3.8698e-01_rb,4.4801e-01_rb/)
      kbo(:,40, 7) = (/ &
     &2.1385e-01_rb,2.4566e-01_rb,2.8431e-01_rb,3.3018e-01_rb,3.8482e-01_rb/)
      kbo(:,41, 7) = (/ &
     &1.7890e-01_rb,2.0671e-01_rb,2.4058e-01_rb,2.8118e-01_rb,3.2987e-01_rb/)
      kbo(:,42, 7) = (/ &
     &1.4939e-01_rb,1.7381e-01_rb,2.0357e-01_rb,2.3938e-01_rb,2.8282e-01_rb/)
      kbo(:,43, 7) = (/ &
     &1.2354e-01_rb,1.4458e-01_rb,1.7057e-01_rb,2.0193e-01_rb,2.4014e-01_rb/)
      kbo(:,44, 7) = (/ &
     &1.0153e-01_rb,1.1950e-01_rb,1.4202e-01_rb,1.6945e-01_rb,2.0287e-01_rb/)
      kbo(:,45, 7) = (/ &
     &8.3299e-02_rb,9.8554e-02_rb,1.1794e-01_rb,1.4189e-01_rb,1.7119e-01_rb/)
      kbo(:,46, 7) = (/ &
     &6.7950e-02_rb,8.0732e-02_rb,9.7254e-02_rb,1.1796e-01_rb,1.4359e-01_rb/)
      kbo(:,47, 7) = (/ &
     &5.4969e-02_rb,6.5444e-02_rb,7.9285e-02_rb,9.6924e-02_rb,1.1901e-01_rb/)
      kbo(:,48, 7) = (/ &
     &4.4384e-02_rb,5.2919e-02_rb,6.4422e-02_rb,7.9306e-02_rb,9.8242e-02_rb/)
      kbo(:,49, 7) = (/ &
     &3.5781e-02_rb,4.2676e-02_rb,5.2185e-02_rb,6.4673e-02_rb,8.0797e-02_rb/)
      kbo(:,50, 7) = (/ &
     &2.8901e-02_rb,3.4506e-02_rb,4.2382e-02_rb,5.2852e-02_rb,6.6579e-02_rb/)
      kbo(:,51, 7) = (/ &
     &2.3369e-02_rb,2.7878e-02_rb,3.4389e-02_rb,4.3175e-02_rb,5.4833e-02_rb/)
      kbo(:,52, 7) = (/ &
     &1.8862e-02_rb,2.2473e-02_rb,2.7804e-02_rb,3.5169e-02_rb,4.5009e-02_rb/)
      kbo(:,53, 7) = (/ &
     &1.5202e-02_rb,1.8063e-02_rb,2.2412e-02_rb,2.8532e-02_rb,3.6823e-02_rb/)
      kbo(:,54, 7) = (/ &
     &1.2294e-02_rb,1.4584e-02_rb,1.8116e-02_rb,2.3236e-02_rb,3.0266e-02_rb/)
      kbo(:,55, 7) = (/ &
     &9.9456e-03_rb,1.1792e-02_rb,1.4659e-02_rb,1.8922e-02_rb,2.4874e-02_rb/)
      kbo(:,56, 7) = (/ &
     &8.0324e-03_rb,9.5190e-03_rb,1.1834e-02_rb,1.5356e-02_rb,2.0377e-02_rb/)
      kbo(:,57, 7) = (/ &
     &6.4764e-03_rb,7.6756e-03_rb,9.5255e-03_rb,1.2420e-02_rb,1.6628e-02_rb/)
      kbo(:,58, 7) = (/ &
     &5.2278e-03_rb,6.1981e-03_rb,7.6844e-03_rb,1.0060e-02_rb,1.3591e-02_rb/)
      kbo(:,59, 7) = (/ &
     &4.3384e-03_rb,5.1762e-03_rb,6.4759e-03_rb,8.5723e-03_rb,1.1730e-02_rb/)
      kbo(:,13, 8) = (/ &
     &4.8706e+01_rb,5.0287e+01_rb,5.1813e+01_rb,5.3328e+01_rb,5.4791e+01_rb/)
      kbo(:,14, 8) = (/ &
     &4.2745e+01_rb,4.3995e+01_rb,4.5243e+01_rb,4.6475e+01_rb,4.7668e+01_rb/)
      kbo(:,15, 8) = (/ &
     &3.7251e+01_rb,3.8260e+01_rb,3.9278e+01_rb,4.0277e+01_rb,4.1241e+01_rb/)
      kbo(:,16, 8) = (/ &
     &3.2307e+01_rb,3.3121e+01_rb,3.3949e+01_rb,3.4768e+01_rb,3.5567e+01_rb/)
      kbo(:,17, 8) = (/ &
     &2.7892e+01_rb,2.8569e+01_rb,2.9248e+01_rb,2.9919e+01_rb,3.0586e+01_rb/)
      kbo(:,18, 8) = (/ &
     &2.3896e+01_rb,2.4456e+01_rb,2.5013e+01_rb,2.5581e+01_rb,2.6214e+01_rb/)
      kbo(:,19, 8) = (/ &
     &2.0345e+01_rb,2.0796e+01_rb,2.1262e+01_rb,2.1777e+01_rb,2.2440e+01_rb/)
      kbo(:,20, 8) = (/ &
     &1.7240e+01_rb,1.7611e+01_rb,1.8026e+01_rb,1.8548e+01_rb,1.9267e+01_rb/)
      kbo(:,21, 8) = (/ &
     &1.4559e+01_rb,1.4881e+01_rb,1.5281e+01_rb,1.5850e+01_rb,1.6627e+01_rb/)
      kbo(:,22, 8) = (/ &
     &1.2297e+01_rb,1.2604e+01_rb,1.3048e+01_rb,1.3680e+01_rb,1.4474e+01_rb/)
      kbo(:,23, 8) = (/ &
     &1.0392e+01_rb,1.0731e+01_rb,1.1231e+01_rb,1.1887e+01_rb,1.2635e+01_rb/)
      kbo(:,24, 8) = (/ &
     &8.8024e+00_rb,9.1906e+00_rb,9.7265e+00_rb,1.0368e+01_rb,1.1047e+01_rb/)
      kbo(:,25, 8) = (/ &
     &7.4985e+00_rb,7.9218e+00_rb,8.4606e+00_rb,9.0520e+00_rb,9.6698e+00_rb/)
      kbo(:,26, 8) = (/ &
     &6.4322e+00_rb,6.8736e+00_rb,7.3794e+00_rb,7.9154e+00_rb,8.4712e+00_rb/)
      kbo(:,27, 8) = (/ &
     &5.5575e+00_rb,5.9810e+00_rb,6.4388e+00_rb,6.9229e+00_rb,7.4216e+00_rb/)
      kbo(:,28, 8) = (/ &
     &4.8206e+00_rb,5.2066e+00_rb,5.6224e+00_rb,6.0597e+00_rb,6.5182e+00_rb/)
      kbo(:,29, 8) = (/ &
     &4.1909e+00_rb,4.5439e+00_rb,4.9224e+00_rb,5.3216e+00_rb,5.7541e+00_rb/)
      kbo(:,30, 8) = (/ &
     &3.6537e+00_rb,3.9765e+00_rb,4.3215e+00_rb,4.6952e+00_rb,5.1087e+00_rb/)
      kbo(:,31, 8) = (/ &
     &3.1984e+00_rb,3.4930e+00_rb,3.8129e+00_rb,4.1678e+00_rb,4.5716e+00_rb/)
      kbo(:,32, 8) = (/ &
     &2.8116e+00_rb,3.0829e+00_rb,3.3853e+00_rb,3.7289e+00_rb,4.1280e+00_rb/)
      kbo(:,33, 8) = (/ &
     &2.4834e+00_rb,2.7385e+00_rb,3.0281e+00_rb,3.3657e+00_rb,3.7662e+00_rb/)
      kbo(:,34, 8) = (/ &
     &2.1985e+00_rb,2.4407e+00_rb,2.7219e+00_rb,3.0561e+00_rb,3.4576e+00_rb/)
      kbo(:,35, 8) = (/ &
     &1.9383e+00_rb,2.1683e+00_rb,2.4400e+00_rb,2.7681e+00_rb,3.1668e+00_rb/)
      kbo(:,36, 8) = (/ &
     &1.6974e+00_rb,1.9136e+00_rb,2.1729e+00_rb,2.4903e+00_rb,2.8775e+00_rb/)
      kbo(:,37, 8) = (/ &
     &1.4652e+00_rb,1.6633e+00_rb,1.9048e+00_rb,2.2035e+00_rb,2.5698e+00_rb/)
      kbo(:,38, 8) = (/ &
     &1.2656e+00_rb,1.4474e+00_rb,1.6717e+00_rb,1.9523e+00_rb,2.2987e+00_rb/)
      kbo(:,39, 8) = (/ &
     &1.0950e+00_rb,1.2617e+00_rb,1.4708e+00_rb,1.7354e+00_rb,2.0626e+00_rb/)
      kbo(:,40, 8) = (/ &
     &9.3521e-01_rb,1.0851e+00_rb,1.2754e+00_rb,1.5183e+00_rb,1.8212e+00_rb/)
      kbo(:,41, 8) = (/ &
     &7.9768e-01_rb,9.3182e-01_rb,1.1039e+00_rb,1.3257e+00_rb,1.6049e+00_rb/)
      kbo(:,42, 8) = (/ &
     &6.8000e-01_rb,8.0060e-01_rb,9.5561e-01_rb,1.1575e+00_rb,1.4146e+00_rb/)
      kbo(:,43, 8) = (/ &
     &5.7402e-01_rb,6.8098e-01_rb,8.1874e-01_rb,9.9948e-01_rb,1.2325e+00_rb/)
      kbo(:,44, 8) = (/ &
     &4.8186e-01_rb,5.7592e-01_rb,6.9798e-01_rb,8.5786e-01_rb,1.0670e+00_rb/)
      kbo(:,45, 8) = (/ &
     &4.0392e-01_rb,4.8633e-01_rb,5.9393e-01_rb,7.3592e-01_rb,9.2240e-01_rb/)
      kbo(:,46, 8) = (/ &
     &3.3669e-01_rb,4.0818e-01_rb,5.0211e-01_rb,6.2723e-01_rb,7.9252e-01_rb/)
      kbo(:,47, 8) = (/ &
     &2.7781e-01_rb,3.3888e-01_rb,4.1961e-01_rb,5.2781e-01_rb,6.7290e-01_rb/)
      kbo(:,48, 8) = (/ &
     &2.2877e-01_rb,2.8059e-01_rb,3.4952e-01_rb,4.4268e-01_rb,5.6946e-01_rb/)
      kbo(:,49, 8) = (/ &
     &1.8802e-01_rb,2.3180e-01_rb,2.9026e-01_rb,3.7003e-01_rb,4.7989e-01_rb/)
      kbo(:,50, 8) = (/ &
     &1.5486e-01_rb,1.9224e-01_rb,2.4194e-01_rb,3.1045e-01_rb,4.0570e-01_rb/)
      kbo(:,51, 8) = (/ &
     &1.2726e-01_rb,1.5950e-01_rb,2.0189e-01_rb,2.6070e-01_rb,3.4311e-01_rb/)
      kbo(:,52, 8) = (/ &
     &1.0409e-01_rb,1.3185e-01_rb,1.6823e-01_rb,2.1834e-01_rb,2.8934e-01_rb/)
      kbo(:,53, 8) = (/ &
     &8.4711e-02_rb,1.0844e-01_rb,1.3980e-01_rb,1.8253e-01_rb,2.4332e-01_rb/)
      kbo(:,54, 8) = (/ &
     &6.9167e-02_rb,8.9444e-02_rb,1.1665e-01_rb,1.5372e-01_rb,2.0609e-01_rb/)
      kbo(:,55, 8) = (/ &
     &5.6448e-02_rb,7.3743e-02_rb,9.7299e-02_rb,1.2965e-01_rb,1.7511e-01_rb/)
      kbo(:,56, 8) = (/ &
     &4.5936e-02_rb,6.0554e-02_rb,8.0824e-02_rb,1.0896e-01_rb,1.4860e-01_rb/)
      kbo(:,57, 8) = (/ &
     &3.7232e-02_rb,4.9532e-02_rb,6.6834e-02_rb,9.1174e-02_rb,1.2579e-01_rb/)
      kbo(:,58, 8) = (/ &
     &3.0170e-02_rb,4.0563e-02_rb,5.5307e-02_rb,7.6347e-02_rb,1.0664e-01_rb/)
      kbo(:,59, 8) = (/ &
     &2.5677e-02_rb,3.5025e-02_rb,4.8385e-02_rb,6.7734e-02_rb,9.6011e-02_rb/)
      kbo(:,13, 9) = (/ &
     &2.6703e+02_rb,2.6279e+02_rb,2.6062e+02_rb,2.6039e+02_rb,2.6194e+02_rb/)
      kbo(:,14, 9) = (/ &
     &2.2898e+02_rb,2.2692e+02_rb,2.2683e+02_rb,2.2864e+02_rb,2.3216e+02_rb/)
      kbo(:,15, 9) = (/ &
     &1.9675e+02_rb,1.9661e+02_rb,1.9840e+02_rb,2.0200e+02_rb,2.0726e+02_rb/)
      kbo(:,16, 9) = (/ &
     &1.6984e+02_rb,1.7136e+02_rb,1.7476e+02_rb,1.7992e+02_rb,1.8656e+02_rb/)
      kbo(:,17, 9) = (/ &
     &1.4758e+02_rb,1.5050e+02_rb,1.5527e+02_rb,1.6140e+02_rb,1.6749e+02_rb/)
      kbo(:,18, 9) = (/ &
     &1.2946e+02_rb,1.3358e+02_rb,1.3896e+02_rb,1.4424e+02_rb,1.4936e+02_rb/)
      kbo(:,19, 9) = (/ &
     &1.1448e+02_rb,1.1916e+02_rb,1.2382e+02_rb,1.2832e+02_rb,1.3257e+02_rb/)
      kbo(:,20, 9) = (/ &
     &1.0147e+02_rb,1.0559e+02_rb,1.0961e+02_rb,1.1348e+02_rb,1.1720e+02_rb/)
      kbo(:,21, 9) = (/ &
     &8.9412e+01_rb,9.2958e+01_rb,9.6430e+01_rb,9.9808e+01_rb,1.0309e+02_rb/)
      kbo(:,22, 9) = (/ &
     &7.8582e+01_rb,8.1638e+01_rb,8.4651e+01_rb,8.7620e+01_rb,9.0634e+01_rb/)
      kbo(:,23, 9) = (/ &
     &6.8855e+01_rb,7.1486e+01_rb,7.4134e+01_rb,7.6820e+01_rb,7.9730e+01_rb/)
      kbo(:,24, 9) = (/ &
     &6.0268e+01_rb,6.2580e+01_rb,6.4940e+01_rb,6.7498e+01_rb,7.0383e+01_rb/)
      kbo(:,25, 9) = (/ &
     &5.2817e+01_rb,5.4857e+01_rb,5.7070e+01_rb,5.9583e+01_rb,6.2456e+01_rb/)
      kbo(:,26, 9) = (/ &
     &4.6400e+01_rb,4.8304e+01_rb,5.0443e+01_rb,5.2928e+01_rb,5.5843e+01_rb/)
      kbo(:,27, 9) = (/ &
     &4.0882e+01_rb,4.2740e+01_rb,4.4869e+01_rb,4.7387e+01_rb,5.0354e+01_rb/)
      kbo(:,28, 9) = (/ &
     &3.6127e+01_rb,3.8035e+01_rb,4.0229e+01_rb,4.2798e+01_rb,4.5828e+01_rb/)
      kbo(:,29, 9) = (/ &
     &3.2099e+01_rb,3.4067e+01_rb,3.6357e+01_rb,3.9031e+01_rb,4.2139e+01_rb/)
      kbo(:,30, 9) = (/ &
     &2.8701e+01_rb,3.0741e+01_rb,3.3127e+01_rb,3.5914e+01_rb,3.9160e+01_rb/)
      kbo(:,31, 9) = (/ &
     &2.5860e+01_rb,2.7970e+01_rb,3.0458e+01_rb,3.3377e+01_rb,3.6776e+01_rb/)
      kbo(:,32, 9) = (/ &
     &2.3508e+01_rb,2.5694e+01_rb,2.8287e+01_rb,3.1347e+01_rb,3.4892e+01_rb/)
      kbo(:,33, 9) = (/ &
     &2.1570e+01_rb,2.3849e+01_rb,2.6553e+01_rb,2.9752e+01_rb,3.3419e+01_rb/)
      kbo(:,34, 9) = (/ &
     &1.9918e+01_rb,2.2276e+01_rb,2.5098e+01_rb,2.8399e+01_rb,3.2162e+01_rb/)
      kbo(:,35, 9) = (/ &
     &1.8364e+01_rb,2.0767e+01_rb,2.3651e+01_rb,2.7001e+01_rb,3.0813e+01_rb/)
      kbo(:,36, 9) = (/ &
     &1.6826e+01_rb,1.9227e+01_rb,2.2113e+01_rb,2.5466e+01_rb,2.9284e+01_rb/)
      kbo(:,37, 9) = (/ &
     &1.5183e+01_rb,1.7515e+01_rb,2.0322e+01_rb,2.3610e+01_rb,2.7381e+01_rb/)
      kbo(:,38, 9) = (/ &
     &1.3727e+01_rb,1.5985e+01_rb,1.8713e+01_rb,2.1934e+01_rb,2.5649e+01_rb/)
      kbo(:,39, 9) = (/ &
     &1.2450e+01_rb,1.4635e+01_rb,1.7282e+01_rb,2.0444e+01_rb,2.4108e+01_rb/)
      kbo(:,40, 9) = (/ &
     &1.1136e+01_rb,1.3200e+01_rb,1.5726e+01_rb,1.8770e+01_rb,2.2337e+01_rb/)
      kbo(:,41, 9) = (/ &
     &9.9523e+00_rb,1.1888e+01_rb,1.4289e+01_rb,1.7214e+01_rb,2.0673e+01_rb/)
      kbo(:,42, 9) = (/ &
     &8.8997e+00_rb,1.0713e+01_rb,1.2995e+01_rb,1.5799e+01_rb,1.9151e+01_rb/)
      kbo(:,43, 9) = (/ &
     &7.8779e+00_rb,9.5542e+00_rb,1.1690e+01_rb,1.4348e+01_rb,1.7554e+01_rb/)
      kbo(:,44, 9) = (/ &
     &6.9290e+00_rb,8.4792e+00_rb,1.0459e+01_rb,1.2952e+01_rb,1.5997e+01_rb/)
      kbo(:,45, 9) = (/ &
     &6.0796e+00_rb,7.5168e+00_rb,9.3520e+00_rb,1.1681e+01_rb,1.4566e+01_rb/)
      kbo(:,46, 9) = (/ &
     &5.2960e+00_rb,6.6184e+00_rb,8.3160e+00_rb,1.0473e+01_rb,1.3177e+01_rb/)
      kbo(:,47, 9) = (/ &
     &4.5500e+00_rb,5.7498e+00_rb,7.3069e+00_rb,9.2819e+00_rb,1.1781e+01_rb/)
      kbo(:,48, 9) = (/ &
     &3.8931e+00_rb,4.9768e+00_rb,6.3975e+00_rb,8.2101e+00_rb,1.0505e+01_rb/)
      kbo(:,49, 9) = (/ &
     &3.3134e+00_rb,4.2874e+00_rb,5.5792e+00_rb,7.2463e+00_rb,9.3480e+00_rb/)
      kbo(:,50, 9) = (/ &
     &2.8246e+00_rb,3.7000e+00_rb,4.8781e+00_rb,6.4148e+00_rb,8.3545e+00_rb/)
      kbo(:,51, 9) = (/ &
     &2.4042e+00_rb,3.1908e+00_rb,4.2622e+00_rb,5.6755e+00_rb,7.4755e+00_rb/)
      kbo(:,52, 9) = (/ &
     &2.0363e+00_rb,2.7388e+00_rb,3.7098e+00_rb,5.0044e+00_rb,6.6735e+00_rb/)
      kbo(:,53, 9) = (/ &
     &1.7163e+00_rb,2.3398e+00_rb,3.2134e+00_rb,4.3950e+00_rb,5.9331e+00_rb/)
      kbo(:,54, 9) = (/ &
     &1.4526e+00_rb,2.0075e+00_rb,2.7999e+00_rb,3.8816e+00_rb,5.3040e+00_rb/)
      kbo(:,55, 9) = (/ &
     &1.2300e+00_rb,1.7228e+00_rb,2.4419e+00_rb,3.4323e+00_rb,4.7483e+00_rb/)
      kbo(:,56, 9) = (/ &
     &1.0375e+00_rb,1.4731e+00_rb,2.1205e+00_rb,3.0254e+00_rb,4.2381e+00_rb/)
      kbo(:,57, 9) = (/ &
     &8.7139e-01_rb,1.2530e+00_rb,1.8332e+00_rb,2.6576e+00_rb,3.7724e+00_rb/)
      kbo(:,58, 9) = (/ &
     &7.3413e-01_rb,1.0690e+00_rb,1.5879e+00_rb,2.3382e+00_rb,3.3642e+00_rb/)
      kbo(:,59, 9) = (/ &
     &6.5989e-01_rb,9.7719e-01_rb,1.4752e+00_rb,2.2003e+00_rb,3.1955e+00_rb/)
      kbo(:,13,10) = (/ &
     &1.1367e+03_rb,1.1187e+03_rb,1.1007e+03_rb,1.0822e+03_rb,1.0635e+03_rb/)
      kbo(:,14,10) = (/ &
     &1.0074e+03_rb,9.9018e+02_rb,9.7308e+02_rb,9.5574e+02_rb,9.3889e+02_rb/)
      kbo(:,15,10) = (/ &
     &8.7706e+02_rb,8.6124e+02_rb,8.4557e+02_rb,8.3018e+02_rb,8.1545e+02_rb/)
      kbo(:,16,10) = (/ &
     &7.5406e+02_rb,7.4004e+02_rb,7.2614e+02_rb,7.1298e+02_rb,7.0106e+02_rb/)
      kbo(:,17,10) = (/ &
     &6.4217e+02_rb,6.3016e+02_rb,6.1851e+02_rb,6.1057e+02_rb,6.1770e+02_rb/)
      kbo(:,18,10) = (/ &
     &5.4317e+02_rb,5.3308e+02_rb,5.2858e+02_rb,5.4063e+02_rb,5.6728e+02_rb/)
      kbo(:,19,10) = (/ &
     &4.6024e+02_rb,4.5629e+02_rb,4.6936e+02_rb,4.9569e+02_rb,5.2404e+02_rb/)
      kbo(:,20,10) = (/ &
     &3.9833e+02_rb,4.0962e+02_rb,4.3282e+02_rb,4.5686e+02_rb,4.8096e+02_rb/)
      kbo(:,21,10) = (/ &
     &3.5836e+02_rb,3.7926e+02_rb,4.0031e+02_rb,4.2094e+02_rb,4.4134e+02_rb/)
      kbo(:,22,10) = (/ &
     &3.2968e+02_rb,3.4978e+02_rb,3.6939e+02_rb,3.8833e+02_rb,4.0603e+02_rb/)
      kbo(:,23,10) = (/ &
     &3.0114e+02_rb,3.1980e+02_rb,3.3812e+02_rb,3.5559e+02_rb,3.7182e+02_rb/)
      kbo(:,24,10) = (/ &
     &2.7330e+02_rb,2.9068e+02_rb,3.0752e+02_rb,3.2347e+02_rb,3.3875e+02_rb/)
      kbo(:,25,10) = (/ &
     &2.4688e+02_rb,2.6295e+02_rb,2.7833e+02_rb,2.9325e+02_rb,3.0804e+02_rb/)
      kbo(:,26,10) = (/ &
     &2.2243e+02_rb,2.3718e+02_rb,2.5149e+02_rb,2.6591e+02_rb,2.8021e+02_rb/)
      kbo(:,27,10) = (/ &
     &2.0056e+02_rb,2.1372e+02_rb,2.2739e+02_rb,2.4128e+02_rb,2.5540e+02_rb/)
      kbo(:,28,10) = (/ &
     &1.8160e+02_rb,1.9338e+02_rb,2.0609e+02_rb,2.1959e+02_rb,2.3363e+02_rb/)
      kbo(:,29,10) = (/ &
     &1.6537e+02_rb,1.7620e+02_rb,1.8800e+02_rb,2.0075e+02_rb,2.1484e+02_rb/)
      kbo(:,30,10) = (/ &
     &1.5182e+02_rb,1.6163e+02_rb,1.7256e+02_rb,1.8487e+02_rb,1.9853e+02_rb/)
      kbo(:,31,10) = (/ &
     &1.4022e+02_rb,1.4937e+02_rb,1.5973e+02_rb,1.7160e+02_rb,1.8530e+02_rb/)
      kbo(:,32,10) = (/ &
     &1.3043e+02_rb,1.3908e+02_rb,1.4919e+02_rb,1.6089e+02_rb,1.7519e+02_rb/)
      kbo(:,33,10) = (/ &
     &1.2206e+02_rb,1.3058e+02_rb,1.4055e+02_rb,1.5267e+02_rb,1.6777e+02_rb/)
      kbo(:,34,10) = (/ &
     &1.1470e+02_rb,1.2330e+02_rb,1.3342e+02_rb,1.4627e+02_rb,1.6240e+02_rb/)
      kbo(:,35,10) = (/ &
     &1.0771e+02_rb,1.1657e+02_rb,1.2696e+02_rb,1.4044e+02_rb,1.5769e+02_rb/)
      kbo(:,36,10) = (/ &
     &1.0083e+02_rb,1.1016e+02_rb,1.2074e+02_rb,1.3465e+02_rb,1.5261e+02_rb/)
      kbo(:,37,10) = (/ &
     &9.3496e+01_rb,1.0322e+02_rb,1.1406e+02_rb,1.2781e+02_rb,1.4573e+02_rb/)
      kbo(:,38,10) = (/ &
     &8.6889e+01_rb,9.7016e+01_rb,1.0824e+02_rb,1.2185e+02_rb,1.3969e+02_rb/)
      kbo(:,39,10) = (/ &
     &8.1021e+01_rb,9.1487e+01_rb,1.0329e+02_rb,1.1682e+02_rb,1.3445e+02_rb/)
      kbo(:,40,10) = (/ &
     &7.4776e+01_rb,8.5570e+01_rb,9.7630e+01_rb,1.1098e+02_rb,1.2786e+02_rb/)
      kbo(:,41,10) = (/ &
     &6.8905e+01_rb,7.9963e+01_rb,9.2277e+01_rb,1.0563e+02_rb,1.2163e+02_rb/)
      kbo(:,42,10) = (/ &
     &6.3556e+01_rb,7.4784e+01_rb,8.7205e+01_rb,1.0072e+02_rb,1.1596e+02_rb/)
      kbo(:,43,10) = (/ &
     &5.8129e+01_rb,6.9316e+01_rb,8.1730e+01_rb,9.5263e+01_rb,1.0987e+02_rb/)
      kbo(:,44,10) = (/ &
     &5.2913e+01_rb,6.3890e+01_rb,7.6248e+01_rb,8.9686e+01_rb,1.0397e+02_rb/)
      kbo(:,45,10) = (/ &
     &4.8208e+01_rb,5.8780e+01_rb,7.0988e+01_rb,8.4262e+01_rb,9.8407e+01_rb/)
      kbo(:,46,10) = (/ &
     &4.3761e+01_rb,5.3798e+01_rb,6.5649e+01_rb,7.8725e+01_rb,9.2702e+01_rb/)
      kbo(:,47,10) = (/ &
     &3.9326e+01_rb,4.8763e+01_rb,6.0030e+01_rb,7.2829e+01_rb,8.6540e+01_rb/)
      kbo(:,48,10) = (/ &
     &3.5364e+01_rb,4.4166e+01_rb,5.4802e+01_rb,6.7118e+01_rb,8.0555e+01_rb/)
      kbo(:,49,10) = (/ &
     &3.1766e+01_rb,3.9895e+01_rb,4.9874e+01_rb,6.1599e+01_rb,7.4720e+01_rb/)
      kbo(:,50,10) = (/ &
     &2.8694e+01_rb,3.6225e+01_rb,4.5586e+01_rb,5.6696e+01_rb,6.9417e+01_rb/)
      kbo(:,51,10) = (/ &
     &2.5920e+01_rb,3.2954e+01_rb,4.1705e+01_rb,5.2192e+01_rb,6.4368e+01_rb/)
      kbo(:,52,10) = (/ &
     &2.3265e+01_rb,2.9945e+01_rb,3.8066e+01_rb,4.7953e+01_rb,5.9548e+01_rb/)
      kbo(:,53,10) = (/ &
     &2.0700e+01_rb,2.7205e+01_rb,3.4731e+01_rb,4.4004e+01_rb,5.4983e+01_rb/)
      kbo(:,54,10) = (/ &
     &1.8407e+01_rb,2.4804e+01_rb,3.1883e+01_rb,4.0571e+01_rb,5.0997e+01_rb/)
      kbo(:,55,10) = (/ &
     &1.6304e+01_rb,2.2551e+01_rb,2.9350e+01_rb,3.7469e+01_rb,4.7350e+01_rb/)
      kbo(:,56,10) = (/ &
     &1.4323e+01_rb,2.0360e+01_rb,2.7019e+01_rb,3.4608e+01_rb,4.3950e+01_rb/)
      kbo(:,57,10) = (/ &
     &1.2463e+01_rb,1.8241e+01_rb,2.4765e+01_rb,3.1934e+01_rb,4.0687e+01_rb/)
      kbo(:,58,10) = (/ &
     &1.0795e+01_rb,1.6294e+01_rb,2.2666e+01_rb,2.9555e+01_rb,3.7776e+01_rb/)
      kbo(:,59,10) = (/ &
     &1.0106e+01_rb,1.5507e+01_rb,2.1812e+01_rb,2.8629e+01_rb,3.6645e+01_rb/)
      kbo(:,13,11) = (/ &
     &1.9976e+03_rb,1.9732e+03_rb,1.9489e+03_rb,1.9244e+03_rb,1.8976e+03_rb/)
      kbo(:,14,11) = (/ &
     &1.8361e+03_rb,1.8109e+03_rb,1.7859e+03_rb,1.7607e+03_rb,1.7342e+03_rb/)
      kbo(:,15,11) = (/ &
     &1.6588e+03_rb,1.6335e+03_rb,1.6090e+03_rb,1.5838e+03_rb,1.5592e+03_rb/)
      kbo(:,16,11) = (/ &
     &1.4709e+03_rb,1.4475e+03_rb,1.4246e+03_rb,1.4016e+03_rb,1.3798e+03_rb/)
      kbo(:,17,11) = (/ &
     &1.2816e+03_rb,1.2604e+03_rb,1.2396e+03_rb,1.2204e+03_rb,1.2023e+03_rb/)
      kbo(:,18,11) = (/ &
     &1.1028e+03_rb,1.0839e+03_rb,1.0668e+03_rb,1.0513e+03_rb,1.0382e+03_rb/)
      kbo(:,19,11) = (/ &
     &9.4023e+02_rb,9.2525e+02_rb,9.1156e+02_rb,9.0352e+02_rb,9.1240e+02_rb/)
      kbo(:,20,11) = (/ &
     &7.9708e+02_rb,7.8540e+02_rb,7.8133e+02_rb,7.9678e+02_rb,8.2988e+02_rb/)
      kbo(:,21,11) = (/ &
     &6.7587e+02_rb,6.7233e+02_rb,6.8973e+02_rb,7.2606e+02_rb,7.7277e+02_rb/)
      kbo(:,22,11) = (/ &
     &5.8284e+02_rb,5.9841e+02_rb,6.3319e+02_rb,6.7525e+02_rb,7.1925e+02_rb/)
      kbo(:,23,11) = (/ &
     &5.2309e+02_rb,5.5425e+02_rb,5.9038e+02_rb,6.2826e+02_rb,6.6879e+02_rb/)
      kbo(:,24,11) = (/ &
     &4.8568e+02_rb,5.1820e+02_rb,5.5078e+02_rb,5.8566e+02_rb,6.2252e+02_rb/)
      kbo(:,25,11) = (/ &
     &4.5215e+02_rb,4.8337e+02_rb,5.1498e+02_rb,5.4697e+02_rb,5.8166e+02_rb/)
      kbo(:,26,11) = (/ &
     &4.2013e+02_rb,4.5049e+02_rb,4.8168e+02_rb,5.1282e+02_rb,5.4620e+02_rb/)
      kbo(:,27,11) = (/ &
     &3.8959e+02_rb,4.1991e+02_rb,4.5056e+02_rb,4.8229e+02_rb,5.1534e+02_rb/)
      kbo(:,28,11) = (/ &
     &3.6165e+02_rb,3.9145e+02_rb,4.2248e+02_rb,4.5510e+02_rb,4.8890e+02_rb/)
      kbo(:,29,11) = (/ &
     &3.3655e+02_rb,3.6613e+02_rb,3.9802e+02_rb,4.3186e+02_rb,4.6748e+02_rb/)
      kbo(:,30,11) = (/ &
     &3.1395e+02_rb,3.4455e+02_rb,3.7776e+02_rb,4.1303e+02_rb,4.5043e+02_rb/)
      kbo(:,31,11) = (/ &
     &2.9496e+02_rb,3.2692e+02_rb,3.6155e+02_rb,3.9895e+02_rb,4.3848e+02_rb/)
      kbo(:,32,11) = (/ &
     &2.7981e+02_rb,3.1316e+02_rb,3.4973e+02_rb,3.8935e+02_rb,4.3105e+02_rb/)
      kbo(:,33,11) = (/ &
     &2.6816e+02_rb,3.0318e+02_rb,3.4211e+02_rb,3.8404e+02_rb,4.2836e+02_rb/)
      kbo(:,34,11) = (/ &
     &2.5884e+02_rb,2.9576e+02_rb,3.3707e+02_rb,3.8116e+02_rb,4.2770e+02_rb/)
      kbo(:,35,11) = (/ &
     &2.4952e+02_rb,2.8807e+02_rb,3.3125e+02_rb,3.7732e+02_rb,4.2524e+02_rb/)
      kbo(:,36,11) = (/ &
     &2.3943e+02_rb,2.7867e+02_rb,3.2325e+02_rb,3.7058e+02_rb,4.1944e+02_rb/)
      kbo(:,37,11) = (/ &
     &2.2613e+02_rb,2.6501e+02_rb,3.0984e+02_rb,3.5803e+02_rb,4.0758e+02_rb/)
      kbo(:,38,11) = (/ &
     &2.1436e+02_rb,2.5281e+02_rb,2.9747e+02_rb,3.4610e+02_rb,3.9628e+02_rb/)
      kbo(:,39,11) = (/ &
     &2.0421e+02_rb,2.4224e+02_rb,2.8638e+02_rb,3.3523e+02_rb,3.8565e+02_rb/)
      kbo(:,40,11) = (/ &
     &1.9196e+02_rb,2.2865e+02_rb,2.7155e+02_rb,3.1991e+02_rb,3.7045e+02_rb/)
      kbo(:,41,11) = (/ &
     &1.8048e+02_rb,2.1574e+02_rb,2.5717e+02_rb,3.0447e+02_rb,3.5504e+02_rb/)
      kbo(:,42,11) = (/ &
     &1.7002e+02_rb,2.0375e+02_rb,2.4381e+02_rb,2.8960e+02_rb,3.3999e+02_rb/)
      kbo(:,43,11) = (/ &
     &1.5882e+02_rb,1.9068e+02_rb,2.2888e+02_rb,2.7297e+02_rb,3.2243e+02_rb/)
      kbo(:,44,11) = (/ &
     &1.4807e+02_rb,1.7774e+02_rb,2.1381e+02_rb,2.5595e+02_rb,3.0384e+02_rb/)
      kbo(:,45,11) = (/ &
     &1.3832e+02_rb,1.6584e+02_rb,1.9975e+02_rb,2.3997e+02_rb,2.8600e+02_rb/)
      kbo(:,46,11) = (/ &
     &1.2854e+02_rb,1.5423e+02_rb,1.8595e+02_rb,2.2400e+02_rb,2.6785e+02_rb/)
      kbo(:,47,11) = (/ &
     &1.1798e+02_rb,1.4267e+02_rb,1.7172e+02_rb,2.0717e+02_rb,2.4874e+02_rb/)
      kbo(:,48,11) = (/ &
     &1.0754e+02_rb,1.3197e+02_rb,1.5865e+02_rb,1.9157e+02_rb,2.3076e+02_rb/)
      kbo(:,49,11) = (/ &
     &9.7426e+01_rb,1.2165e+02_rb,1.4680e+02_rb,1.7709e+02_rb,2.1383e+02_rb/)
      kbo(:,50,11) = (/ &
     &8.8058e+01_rb,1.1199e+02_rb,1.3670e+02_rb,1.6456e+02_rb,1.9880e+02_rb/)
      kbo(:,51,11) = (/ &
     &7.9382e+01_rb,1.0270e+02_rb,1.2731e+02_rb,1.5331e+02_rb,1.8527e+02_rb/)
      kbo(:,52,11) = (/ &
     &7.1270e+01_rb,9.3720e+01_rb,1.1802e+02_rb,1.4309e+02_rb,1.7263e+02_rb/)
      kbo(:,53,11) = (/ &
     &6.3724e+01_rb,8.4883e+01_rb,1.0881e+02_rb,1.3358e+02_rb,1.6088e+02_rb/)
      kbo(:,54,11) = (/ &
     &5.7330e+01_rb,7.7159e+01_rb,1.0045e+02_rb,1.2510e+02_rb,1.5088e+02_rb/)
      kbo(:,55,11) = (/ &
     &5.1611e+01_rb,7.0208e+01_rb,9.2617e+01_rb,1.1701e+02_rb,1.4209e+02_rb/)
      kbo(:,56,11) = (/ &
     &4.6383e+01_rb,6.3679e+01_rb,8.4927e+01_rb,1.0898e+02_rb,1.3380e+02_rb/)
      kbo(:,57,11) = (/ &
     &4.1604e+01_rb,5.7613e+01_rb,7.7613e+01_rb,1.0100e+02_rb,1.2575e+02_rb/)
      kbo(:,58,11) = (/ &
     &3.7495e+01_rb,5.2208e+01_rb,7.1056e+01_rb,9.3655e+01_rb,1.1815e+02_rb/)
      kbo(:,59,11) = (/ &
     &3.5885e+01_rb,5.0106e+01_rb,6.8487e+01_rb,9.0651e+01_rb,1.1506e+02_rb/)
      kbo(:,13,12) = (/ &
     &3.5806e+03_rb,3.5582e+03_rb,3.5338e+03_rb,3.5034e+03_rb,3.4725e+03_rb/)
      kbo(:,14,12) = (/ &
     &3.4862e+03_rb,3.4597e+03_rb,3.4292e+03_rb,3.3949e+03_rb,3.3614e+03_rb/)
      kbo(:,15,12) = (/ &
     &3.3163e+03_rb,3.2855e+03_rb,3.2498e+03_rb,3.2159e+03_rb,3.1795e+03_rb/)
      kbo(:,16,12) = (/ &
     &3.0905e+03_rb,3.0565e+03_rb,3.0217e+03_rb,2.9883e+03_rb,2.9514e+03_rb/)
      kbo(:,17,12) = (/ &
     &2.8252e+03_rb,2.7909e+03_rb,2.7597e+03_rb,2.7275e+03_rb,2.6935e+03_rb/)
      kbo(:,18,12) = (/ &
     &2.5391e+03_rb,2.5095e+03_rb,2.4810e+03_rb,2.4508e+03_rb,2.4233e+03_rb/)
      kbo(:,19,12) = (/ &
     &2.2475e+03_rb,2.2231e+03_rb,2.1980e+03_rb,2.1744e+03_rb,2.1532e+03_rb/)
      kbo(:,20,12) = (/ &
     &1.9592e+03_rb,1.9385e+03_rb,1.9191e+03_rb,1.9031e+03_rb,1.8880e+03_rb/)
      kbo(:,21,12) = (/ &
     &1.6882e+03_rb,1.6730e+03_rb,1.6603e+03_rb,1.6503e+03_rb,1.6513e+03_rb/)
      kbo(:,22,12) = (/ &
     &1.4424e+03_rb,1.4327e+03_rb,1.4274e+03_rb,1.4386e+03_rb,1.4720e+03_rb/)
      kbo(:,23,12) = (/ &
     &1.2274e+03_rb,1.2245e+03_rb,1.2417e+03_rb,1.2835e+03_rb,1.3444e+03_rb/)
      kbo(:,24,12) = (/ &
     &1.0462e+03_rb,1.0639e+03_rb,1.1088e+03_rb,1.1752e+03_rb,1.2597e+03_rb/)
      kbo(:,25,12) = (/ &
     &9.1114e+02_rb,9.5200e+02_rb,1.0177e+03_rb,1.1057e+03_rb,1.2094e+03_rb/)
      kbo(:,26,12) = (/ &
     &8.2003e+02_rb,8.7985e+02_rb,9.6324e+02_rb,1.0645e+03_rb,1.1739e+03_rb/)
      kbo(:,27,12) = (/ &
     &7.6267e+02_rb,8.3792e+02_rb,9.2956e+02_rb,1.0323e+03_rb,1.1455e+03_rb/)
      kbo(:,28,12) = (/ &
     &7.2963e+02_rb,8.0994e+02_rb,9.0250e+02_rb,1.0075e+03_rb,1.1258e+03_rb/)
      kbo(:,29,12) = (/ &
     &7.0594e+02_rb,7.8863e+02_rb,8.8342e+02_rb,9.9282e+02_rb,1.1166e+03_rb/)
      kbo(:,30,12) = (/ &
     &6.8865e+02_rb,7.7319e+02_rb,8.7237e+02_rb,9.8744e+02_rb,1.1171e+03_rb/)
      kbo(:,31,12) = (/ &
     &6.7685e+02_rb,7.6521e+02_rb,8.7052e+02_rb,9.9144e+02_rb,1.1267e+03_rb/)
      kbo(:,32,12) = (/ &
     &6.7111e+02_rb,7.6562e+02_rb,8.7695e+02_rb,1.0037e+03_rb,1.1434e+03_rb/)
      kbo(:,33,12) = (/ &
     &6.7259e+02_rb,7.7346e+02_rb,8.9020e+02_rb,1.0220e+03_rb,1.1658e+03_rb/)
      kbo(:,34,12) = (/ &
     &6.7713e+02_rb,7.8296e+02_rb,9.0470e+02_rb,1.0405e+03_rb,1.1875e+03_rb/)
      kbo(:,35,12) = (/ &
     &6.7665e+02_rb,7.8599e+02_rb,9.1053e+02_rb,1.0483e+03_rb,1.1976e+03_rb/)
      kbo(:,36,12) = (/ &
     &6.6854e+02_rb,7.7898e+02_rb,9.0439e+02_rb,1.0428e+03_rb,1.1928e+03_rb/)
      kbo(:,37,12) = (/ &
     &6.4709e+02_rb,7.5625e+02_rb,8.8001e+02_rb,1.0171e+03_rb,1.1660e+03_rb/)
      kbo(:,38,12) = (/ &
     &6.2682e+02_rb,7.3409e+02_rb,8.5611e+02_rb,9.9182e+02_rb,1.1392e+03_rb/)
      kbo(:,39,12) = (/ &
     &6.0784e+02_rb,7.1328e+02_rb,8.3356e+02_rb,9.6784e+02_rb,1.1136e+03_rb/)
      kbo(:,40,12) = (/ &
     &5.7971e+02_rb,6.8247e+02_rb,7.9953e+02_rb,9.3053e+02_rb,1.0739e+03_rb/)
      kbo(:,41,12) = (/ &
     &5.5088e+02_rb,6.5173e+02_rb,7.6511e+02_rb,8.9294e+02_rb,1.0334e+03_rb/)
      kbo(:,42,12) = (/ &
     &5.2250e+02_rb,6.2258e+02_rb,7.3222e+02_rb,8.5651e+02_rb,9.9385e+02_rb/)
      kbo(:,43,12) = (/ &
     &4.8933e+02_rb,5.8876e+02_rb,6.9433e+02_rb,8.1420e+02_rb,9.4769e+02_rb/)
      kbo(:,44,12) = (/ &
     &4.5428e+02_rb,5.5295e+02_rb,6.5546e+02_rb,7.7054e+02_rb,8.9971e+02_rb/)
      kbo(:,45,12) = (/ &
     &4.2007e+02_rb,5.1771e+02_rb,6.1870e+02_rb,7.2869e+02_rb,8.5325e+02_rb/)
      kbo(:,46,12) = (/ &
     &3.8522e+02_rb,4.8086e+02_rb,5.8089e+02_rb,6.8626e+02_rb,8.0577e+02_rb/)
      kbo(:,47,12) = (/ &
     &3.4877e+02_rb,4.4033e+02_rb,5.3933e+02_rb,6.4157e+02_rb,7.5519e+02_rb/)
      kbo(:,48,12) = (/ &
     &3.1493e+02_rb,4.0108e+02_rb,4.9823e+02_rb,5.9910e+02_rb,7.0691e+02_rb/)
      kbo(:,49,12) = (/ &
     &2.8364e+02_rb,3.6388e+02_rb,4.5779e+02_rb,5.5756e+02_rb,6.6121e+02_rb/)
      kbo(:,50,12) = (/ &
     &2.5678e+02_rb,3.3121e+02_rb,4.2060e+02_rb,5.1905e+02_rb,6.2075e+02_rb/)
      kbo(:,51,12) = (/ &
     &2.3307e+02_rb,3.0175e+02_rb,3.8585e+02_rb,4.8220e+02_rb,5.8283e+02_rb/)
      kbo(:,52,12) = (/ &
     &2.1162e+02_rb,2.7441e+02_rb,3.5299e+02_rb,4.4577e+02_rb,5.4547e+02_rb/)
      kbo(:,53,12) = (/ &
     &1.9234e+02_rb,2.4923e+02_rb,3.2216e+02_rb,4.1019e+02_rb,5.0837e+02_rb/)
      kbo(:,54,12) = (/ &
     &1.7628e+02_rb,2.2813e+02_rb,2.9582e+02_rb,3.7887e+02_rb,4.7480e+02_rb/)
      kbo(:,55,12) = (/ &
     &1.6140e+02_rb,2.0966e+02_rb,2.7210e+02_rb,3.5029e+02_rb,4.4296e+02_rb/)
      kbo(:,56,12) = (/ &
     &1.4678e+02_rb,1.9288e+02_rb,2.5016e+02_rb,3.2346e+02_rb,4.1177e+02_rb/)
      kbo(:,57,12) = (/ &
     &1.3255e+02_rb,1.7758e+02_rb,2.2997e+02_rb,2.9828e+02_rb,3.8188e+02_rb/)
      kbo(:,58,12) = (/ &
     &1.1943e+02_rb,1.6353e+02_rb,2.1228e+02_rb,2.7566e+02_rb,3.5477e+02_rb/)
      kbo(:,59,12) = (/ &
     &1.1423e+02_rb,1.5792e+02_rb,2.0564e+02_rb,2.6702e+02_rb,3.4426e+02_rb/)
      kbo(:,13,13) = (/ &
     &6.2371e+03_rb,6.2518e+03_rb,6.2489e+03_rb,6.2389e+03_rb,6.2163e+03_rb/)
      kbo(:,14,13) = (/ &
     &6.5479e+03_rb,6.5460e+03_rb,6.5322e+03_rb,6.5131e+03_rb,6.4799e+03_rb/)
      kbo(:,15,13) = (/ &
     &6.7275e+03_rb,6.7134e+03_rb,6.6981e+03_rb,6.6679e+03_rb,6.6340e+03_rb/)
      kbo(:,16,13) = (/ &
     &6.7706e+03_rb,6.7549e+03_rb,6.7279e+03_rb,6.6968e+03_rb,6.6585e+03_rb/)
      kbo(:,17,13) = (/ &
     &6.6836e+03_rb,6.6601e+03_rb,6.6362e+03_rb,6.6043e+03_rb,6.5760e+03_rb/)
      kbo(:,18,13) = (/ &
     &6.4651e+03_rb,6.4504e+03_rb,6.4295e+03_rb,6.4100e+03_rb,6.3855e+03_rb/)
      kbo(:,19,13) = (/ &
     &6.1476e+03_rb,6.1374e+03_rb,6.1314e+03_rb,6.1247e+03_rb,6.1206e+03_rb/)
      kbo(:,20,13) = (/ &
     &5.7657e+03_rb,5.7701e+03_rb,5.7793e+03_rb,5.7885e+03_rb,5.8038e+03_rb/)
      kbo(:,21,13) = (/ &
     &5.3372e+03_rb,5.3657e+03_rb,5.3946e+03_rb,5.4291e+03_rb,5.4659e+03_rb/)
      kbo(:,22,13) = (/ &
     &4.8964e+03_rb,4.9428e+03_rb,4.9998e+03_rb,5.0621e+03_rb,5.1258e+03_rb/)
      kbo(:,23,13) = (/ &
     &4.4608e+03_rb,4.5344e+03_rb,4.6177e+03_rb,4.7044e+03_rb,4.8008e+03_rb/)
      kbo(:,24,13) = (/ &
     &4.0572e+03_rb,4.1535e+03_rb,4.2613e+03_rb,4.3798e+03_rb,4.5026e+03_rb/)
      kbo(:,25,13) = (/ &
     &3.6891e+03_rb,3.8125e+03_rb,3.9482e+03_rb,4.0905e+03_rb,4.2411e+03_rb/)
      kbo(:,26,13) = (/ &
     &3.3690e+03_rb,3.5206e+03_rb,3.6781e+03_rb,3.8529e+03_rb,4.0416e+03_rb/)
      kbo(:,27,13) = (/ &
     &3.0960e+03_rb,3.2709e+03_rb,3.4667e+03_rb,3.6805e+03_rb,3.9027e+03_rb/)
      kbo(:,28,13) = (/ &
     &2.8685e+03_rb,3.0838e+03_rb,3.3203e+03_rb,3.5644e+03_rb,3.8131e+03_rb/)
      kbo(:,29,13) = (/ &
     &2.7079e+03_rb,2.9620e+03_rb,3.2275e+03_rb,3.4951e+03_rb,3.7635e+03_rb/)
      kbo(:,30,13) = (/ &
     &2.6061e+03_rb,2.8858e+03_rb,3.1752e+03_rb,3.4611e+03_rb,3.7451e+03_rb/)
      kbo(:,31,13) = (/ &
     &2.5477e+03_rb,2.8507e+03_rb,3.1575e+03_rb,3.4577e+03_rb,3.7506e+03_rb/)
      kbo(:,32,13) = (/ &
     &2.5270e+03_rb,2.8474e+03_rb,3.1664e+03_rb,3.4757e+03_rb,3.7742e+03_rb/)
      kbo(:,33,13) = (/ &
     &2.5348e+03_rb,2.8669e+03_rb,3.1934e+03_rb,3.5095e+03_rb,3.8093e+03_rb/)
      kbo(:,34,13) = (/ &
     &2.5519e+03_rb,2.8931e+03_rb,3.2251e+03_rb,3.5432e+03_rb,3.8451e+03_rb/)
      kbo(:,35,13) = (/ &
     &2.5512e+03_rb,2.8985e+03_rb,3.2338e+03_rb,3.5558e+03_rb,3.8584e+03_rb/)
      kbo(:,36,13) = (/ &
     &2.5221e+03_rb,2.8745e+03_rb,3.2143e+03_rb,3.5395e+03_rb,3.8450e+03_rb/)
      kbo(:,37,13) = (/ &
     &2.4460e+03_rb,2.8025e+03_rb,3.1490e+03_rb,3.4791e+03_rb,3.7899e+03_rb/)
      kbo(:,38,13) = (/ &
     &2.3708e+03_rb,2.7313e+03_rb,3.0831e+03_rb,3.4179e+03_rb,3.7342e+03_rb/)
      kbo(:,39,13) = (/ &
     &2.3003e+03_rb,2.6642e+03_rb,3.0201e+03_rb,3.3586e+03_rb,3.6803e+03_rb/)
      kbo(:,40,13) = (/ &
     &2.1969e+03_rb,2.5616e+03_rb,2.9219e+03_rb,3.2671e+03_rb,3.5954e+03_rb/)
      kbo(:,41,13) = (/ &
     &2.0924e+03_rb,2.4560e+03_rb,2.8204e+03_rb,3.1716e+03_rb,3.5055e+03_rb/)
      kbo(:,42,13) = (/ &
     &1.9915e+03_rb,2.3511e+03_rb,2.7188e+03_rb,3.0759e+03_rb,3.4151e+03_rb/)
      kbo(:,43,13) = (/ &
     &1.8747e+03_rb,2.2279e+03_rb,2.5969e+03_rb,2.9589e+03_rb,3.3053e+03_rb/)
      kbo(:,44,13) = (/ &
     &1.7544e+03_rb,2.0986e+03_rb,2.4653e+03_rb,2.8326e+03_rb,3.1860e+03_rb/)
      kbo(:,45,13) = (/ &
     &1.6386e+03_rb,1.9731e+03_rb,2.3348e+03_rb,2.7048e+03_rb,3.0644e+03_rb/)
      kbo(:,46,13) = (/ &
     &1.5205e+03_rb,1.8444e+03_rb,2.1974e+03_rb,2.5688e+03_rb,2.9332e+03_rb/)
      kbo(:,47,13) = (/ &
     &1.3958e+03_rb,1.7058e+03_rb,2.0486e+03_rb,2.4160e+03_rb,2.7851e+03_rb/)
      kbo(:,48,13) = (/ &
     &1.2783e+03_rb,1.5734e+03_rb,1.9036e+03_rb,2.2627e+03_rb,2.6347e+03_rb/)
      kbo(:,49,13) = (/ &
     &1.1678e+03_rb,1.4470e+03_rb,1.7643e+03_rb,2.1130e+03_rb,2.4832e+03_rb/)
      kbo(:,50,13) = (/ &
     &1.0712e+03_rb,1.3344e+03_rb,1.6384e+03_rb,1.9762e+03_rb,2.3408e+03_rb/)
      kbo(:,51,13) = (/ &
     &9.8353e+02_rb,1.2311e+03_rb,1.5209e+03_rb,1.8472e+03_rb,2.2032e+03_rb/)
      kbo(:,52,13) = (/ &
     &9.0254e+02_rb,1.1339e+03_rb,1.4093e+03_rb,1.7232e+03_rb,2.0693e+03_rb/)
      kbo(:,53,13) = (/ &
     &8.2760e+02_rb,1.0429e+03_rb,1.3025e+03_rb,1.6032e+03_rb,1.9381e+03_rb/)
      kbo(:,54,13) = (/ &
     &7.6424e+02_rb,9.6451e+02_rb,1.2096e+03_rb,1.4971e+03_rb,1.8216e+03_rb/)
      kbo(:,55,13) = (/ &
     &7.1080e+02_rb,8.9446e+02_rb,1.1253e+03_rb,1.4000e+03_rb,1.7133e+03_rb/)
      kbo(:,56,13) = (/ &
     &6.6381e+02_rb,8.2901e+02_rb,1.0458e+03_rb,1.3069e+03_rb,1.6083e+03_rb/)
      kbo(:,57,13) = (/ &
     &6.2290e+02_rb,7.6867e+02_rb,9.7122e+02_rb,1.2178e+03_rb,1.5070e+03_rb/)
      kbo(:,58,13) = (/ &
     &5.8709e+02_rb,7.1760e+02_rb,9.0453e+02_rb,1.1378e+03_rb,1.4149e+03_rb/)
      kbo(:,59,13) = (/ &
     &5.7268e+02_rb,6.9849e+02_rb,8.7854e+02_rb,1.1067e+03_rb,1.3783e+03_rb/)
      kbo(:,13,14) = (/ &
     &9.8812e+03_rb,9.9499e+03_rb,9.9974e+03_rb,1.0022e+04_rb,1.0029e+04_rb/)
      kbo(:,14,14) = (/ &
     &1.1156e+04_rb,1.1216e+04_rb,1.1254e+04_rb,1.1270e+04_rb,1.1265e+04_rb/)
      kbo(:,15,14) = (/ &
     &1.2449e+04_rb,1.2496e+04_rb,1.2517e+04_rb,1.2519e+04_rb,1.2490e+04_rb/)
      kbo(:,16,14) = (/ &
     &1.3700e+04_rb,1.3729e+04_rb,1.3741e+04_rb,1.3718e+04_rb,1.3680e+04_rb/)
      kbo(:,17,14) = (/ &
     &1.4861e+04_rb,1.4895e+04_rb,1.4882e+04_rb,1.4853e+04_rb,1.4786e+04_rb/)
      kbo(:,18,14) = (/ &
     &1.5910e+04_rb,1.5917e+04_rb,1.5910e+04_rb,1.5867e+04_rb,1.5794e+04_rb/)
      kbo(:,19,14) = (/ &
     &1.6801e+04_rb,1.6817e+04_rb,1.6802e+04_rb,1.6758e+04_rb,1.6676e+04_rb/)
      kbo(:,20,14) = (/ &
     &1.7524e+04_rb,1.7556e+04_rb,1.7546e+04_rb,1.7509e+04_rb,1.7437e+04_rb/)
      kbo(:,21,14) = (/ &
     &1.8096e+04_rb,1.8141e+04_rb,1.8145e+04_rb,1.8122e+04_rb,1.8064e+04_rb/)
      kbo(:,22,14) = (/ &
     &1.8520e+04_rb,1.8595e+04_rb,1.8633e+04_rb,1.8624e+04_rb,1.8566e+04_rb/)
      kbo(:,23,14) = (/ &
     &1.8825e+04_rb,1.8946e+04_rb,1.9018e+04_rb,1.9030e+04_rb,1.8984e+04_rb/)
      kbo(:,24,14) = (/ &
     &1.9042e+04_rb,1.9220e+04_rb,1.9316e+04_rb,1.9349e+04_rb,1.9324e+04_rb/)
      kbo(:,25,14) = (/ &
     &1.9210e+04_rb,1.9418e+04_rb,1.9542e+04_rb,1.9606e+04_rb,1.9602e+04_rb/)
      kbo(:,26,14) = (/ &
     &1.9338e+04_rb,1.9577e+04_rb,1.9740e+04_rb,1.9821e+04_rb,1.9828e+04_rb/)
      kbo(:,27,14) = (/ &
     &1.9451e+04_rb,1.9723e+04_rb,1.9903e+04_rb,2.0002e+04_rb,2.0015e+04_rb/)
      kbo(:,28,14) = (/ &
     &1.9563e+04_rb,1.9853e+04_rb,2.0047e+04_rb,2.0156e+04_rb,2.0162e+04_rb/)
      kbo(:,29,14) = (/ &
     &1.9675e+04_rb,1.9975e+04_rb,2.0176e+04_rb,2.0280e+04_rb,2.0284e+04_rb/)
      kbo(:,30,14) = (/ &
     &1.9781e+04_rb,2.0092e+04_rb,2.0291e+04_rb,2.0384e+04_rb,2.0380e+04_rb/)
      kbo(:,31,14) = (/ &
     &1.9898e+04_rb,2.0205e+04_rb,2.0389e+04_rb,2.0467e+04_rb,2.0456e+04_rb/)
      kbo(:,32,14) = (/ &
     &2.0014e+04_rb,2.0306e+04_rb,2.0476e+04_rb,2.0543e+04_rb,2.0514e+04_rb/)
      kbo(:,33,14) = (/ &
     &2.0125e+04_rb,2.0400e+04_rb,2.0544e+04_rb,2.0600e+04_rb,2.0558e+04_rb/)
      kbo(:,34,14) = (/ &
     &2.0217e+04_rb,2.0476e+04_rb,2.0611e+04_rb,2.0645e+04_rb,2.0591e+04_rb/)
      kbo(:,35,14) = (/ &
     &2.0276e+04_rb,2.0526e+04_rb,2.0654e+04_rb,2.0682e+04_rb,2.0620e+04_rb/)
      kbo(:,36,14) = (/ &
     &2.0294e+04_rb,2.0553e+04_rb,2.0682e+04_rb,2.0712e+04_rb,2.0656e+04_rb/)
      kbo(:,37,14) = (/ &
     &2.0260e+04_rb,2.0540e+04_rb,2.0692e+04_rb,2.0741e+04_rb,2.0704e+04_rb/)
      kbo(:,38,14) = (/ &
     &2.0217e+04_rb,2.0522e+04_rb,2.0695e+04_rb,2.0763e+04_rb,2.0734e+04_rb/)
      kbo(:,39,14) = (/ &
     &2.0167e+04_rb,2.0500e+04_rb,2.0691e+04_rb,2.0777e+04_rb,2.0762e+04_rb/)
      kbo(:,40,14) = (/ &
     &2.0068e+04_rb,2.0438e+04_rb,2.0666e+04_rb,2.0778e+04_rb,2.0786e+04_rb/)
      kbo(:,41,14) = (/ &
     &1.9949e+04_rb,2.0361e+04_rb,2.0631e+04_rb,2.0768e+04_rb,2.0804e+04_rb/)
      kbo(:,42,14) = (/ &
     &1.9819e+04_rb,2.0274e+04_rb,2.0579e+04_rb,2.0750e+04_rb,2.0814e+04_rb/)
      kbo(:,43,14) = (/ &
     &1.9645e+04_rb,2.0149e+04_rb,2.0503e+04_rb,2.0721e+04_rb,2.0814e+04_rb/)
      kbo(:,44,14) = (/ &
     &1.9429e+04_rb,1.9994e+04_rb,2.0396e+04_rb,2.0663e+04_rb,2.0798e+04_rb/)
      kbo(:,45,14) = (/ &
     &1.9189e+04_rb,1.9824e+04_rb,2.0285e+04_rb,2.0595e+04_rb,2.0770e+04_rb/)
      kbo(:,46,14) = (/ &
     &1.8912e+04_rb,1.9621e+04_rb,2.0139e+04_rb,2.0501e+04_rb,2.0722e+04_rb/)
      kbo(:,47,14) = (/ &
     &1.8574e+04_rb,1.9355e+04_rb,1.9947e+04_rb,2.0373e+04_rb,2.0652e+04_rb/)
      kbo(:,48,14) = (/ &
     &1.8199e+04_rb,1.9060e+04_rb,1.9733e+04_rb,2.0223e+04_rb,2.0560e+04_rb/)
      kbo(:,49,14) = (/ &
     &1.7788e+04_rb,1.8737e+04_rb,1.9487e+04_rb,2.0042e+04_rb,2.0440e+04_rb/)
      kbo(:,50,14) = (/ &
     &1.7372e+04_rb,1.8402e+04_rb,1.9222e+04_rb,1.9853e+04_rb,2.0312e+04_rb/)
      kbo(:,51,14) = (/ &
     &1.6932e+04_rb,1.8048e+04_rb,1.8944e+04_rb,1.9653e+04_rb,2.0163e+04_rb/)
      kbo(:,52,14) = (/ &
     &1.6460e+04_rb,1.7668e+04_rb,1.8640e+04_rb,1.9416e+04_rb,1.9992e+04_rb/)
      kbo(:,53,14) = (/ &
     &1.5959e+04_rb,1.7252e+04_rb,1.8310e+04_rb,1.9147e+04_rb,1.9806e+04_rb/)
      kbo(:,54,14) = (/ &
     &1.5461e+04_rb,1.6846e+04_rb,1.7980e+04_rb,1.8888e+04_rb,1.9606e+04_rb/)
      kbo(:,55,14) = (/ &
     &1.4966e+04_rb,1.6424e+04_rb,1.7640e+04_rb,1.8623e+04_rb,1.9399e+04_rb/)
      kbo(:,56,14) = (/ &
     &1.4452e+04_rb,1.5984e+04_rb,1.7276e+04_rb,1.8327e+04_rb,1.9166e+04_rb/)
      kbo(:,57,14) = (/ &
     &1.3910e+04_rb,1.5518e+04_rb,1.6891e+04_rb,1.8015e+04_rb,1.8917e+04_rb/)
      kbo(:,58,14) = (/ &
     &1.3384e+04_rb,1.5053e+04_rb,1.6496e+04_rb,1.7699e+04_rb,1.8667e+04_rb/)
      kbo(:,59,14) = (/ &
     &1.3166e+04_rb,1.4859e+04_rb,1.6332e+04_rb,1.7567e+04_rb,1.8563e+04_rb/)
      kbo(:,13,15) = (/ &
     &1.3095e+04_rb,1.3203e+04_rb,1.3280e+04_rb,1.3335e+04_rb,1.3364e+04_rb/)
      kbo(:,14,15) = (/ &
     &1.5533e+04_rb,1.5637e+04_rb,1.5702e+04_rb,1.5732e+04_rb,1.5730e+04_rb/)
      kbo(:,15,15) = (/ &
     &1.8283e+04_rb,1.8379e+04_rb,1.8421e+04_rb,1.8416e+04_rb,1.8371e+04_rb/)
      kbo(:,16,15) = (/ &
     &2.1357e+04_rb,2.1409e+04_rb,2.1406e+04_rb,2.1356e+04_rb,2.1240e+04_rb/)
      kbo(:,17,15) = (/ &
     &2.4722e+04_rb,2.4699e+04_rb,2.4632e+04_rb,2.4482e+04_rb,2.4282e+04_rb/)
      kbo(:,18,15) = (/ &
     &2.8315e+04_rb,2.8214e+04_rb,2.8019e+04_rb,2.7755e+04_rb,2.7441e+04_rb/)
      kbo(:,19,15) = (/ &
     &3.2071e+04_rb,3.1837e+04_rb,3.1501e+04_rb,3.1092e+04_rb,3.0622e+04_rb/)
      kbo(:,20,15) = (/ &
     &3.5875e+04_rb,3.5466e+04_rb,3.4981e+04_rb,3.4394e+04_rb,3.3745e+04_rb/)
      kbo(:,21,15) = (/ &
     &3.9627e+04_rb,3.9030e+04_rb,3.8357e+04_rb,3.7580e+04_rb,3.6746e+04_rb/)
      kbo(:,22,15) = (/ &
     &4.3205e+04_rb,4.2400e+04_rb,4.1478e+04_rb,4.0494e+04_rb,3.9468e+04_rb/)
      kbo(:,23,15) = (/ &
     &4.6532e+04_rb,4.5460e+04_rb,4.4309e+04_rb,4.3126e+04_rb,4.1899e+04_rb/)
      kbo(:,24,15) = (/ &
     &4.9513e+04_rb,4.8176e+04_rb,4.6831e+04_rb,4.5435e+04_rb,4.4005e+04_rb/)
      kbo(:,25,15) = (/ &
     &5.2106e+04_rb,5.0560e+04_rb,4.9001e+04_rb,4.7400e+04_rb,4.5767e+04_rb/)
      kbo(:,26,15) = (/ &
     &5.4289e+04_rb,5.2537e+04_rb,5.0767e+04_rb,4.8979e+04_rb,4.7180e+04_rb/)
      kbo(:,27,15) = (/ &
     &5.6074e+04_rb,5.4129e+04_rb,5.2176e+04_rb,5.0219e+04_rb,4.8292e+04_rb/)
      kbo(:,28,15) = (/ &
     &5.7491e+04_rb,5.5358e+04_rb,5.3246e+04_rb,5.1171e+04_rb,4.9128e+04_rb/)
      kbo(:,29,15) = (/ &
     &5.8538e+04_rb,5.6261e+04_rb,5.4015e+04_rb,5.1836e+04_rb,4.9710e+04_rb/)
      kbo(:,30,15) = (/ &
     &5.9277e+04_rb,5.6882e+04_rb,5.4538e+04_rb,5.2280e+04_rb,5.0075e+04_rb/)
      kbo(:,31,15) = (/ &
     &5.9738e+04_rb,5.7244e+04_rb,5.4845e+04_rb,5.2509e+04_rb,5.0252e+04_rb/)
      kbo(:,32,15) = (/ &
     &5.9975e+04_rb,5.7405e+04_rb,5.4958e+04_rb,5.2573e+04_rb,5.0275e+04_rb/)
      kbo(:,33,15) = (/ &
     &6.0007e+04_rb,5.7410e+04_rb,5.4914e+04_rb,5.2499e+04_rb,5.0167e+04_rb/)
      kbo(:,34,15) = (/ &
     &5.9998e+04_rb,5.7359e+04_rb,5.4840e+04_rb,5.2398e+04_rb,5.0041e+04_rb/)
      kbo(:,35,15) = (/ &
     &6.0132e+04_rb,5.7462e+04_rb,5.4907e+04_rb,5.2438e+04_rb,5.0062e+04_rb/)
      kbo(:,36,15) = (/ &
     &6.0467e+04_rb,5.7764e+04_rb,5.5174e+04_rb,5.2679e+04_rb,5.0279e+04_rb/)
      kbo(:,37,15) = (/ &
     &6.1171e+04_rb,5.8407e+04_rb,5.5781e+04_rb,5.3245e+04_rb,5.0803e+04_rb/)
      kbo(:,38,15) = (/ &
     &6.1845e+04_rb,5.9045e+04_rb,5.6369e+04_rb,5.3796e+04_rb,5.1321e+04_rb/)
      kbo(:,39,15) = (/ &
     &6.2474e+04_rb,5.9622e+04_rb,5.6925e+04_rb,5.4319e+04_rb,5.1811e+04_rb/)
      kbo(:,40,15) = (/ &
     &6.3380e+04_rb,6.0485e+04_rb,5.7724e+04_rb,5.5082e+04_rb,5.2540e+04_rb/)
      kbo(:,41,15) = (/ &
     &6.4308e+04_rb,6.1360e+04_rb,5.8546e+04_rb,5.5868e+04_rb,5.3286e+04_rb/)
      kbo(:,42,15) = (/ &
     &6.5214e+04_rb,6.2221e+04_rb,5.9366e+04_rb,5.6643e+04_rb,5.4029e+04_rb/)
      kbo(:,43,15) = (/ &
     &6.6280e+04_rb,6.3259e+04_rb,6.0335e+04_rb,5.7567e+04_rb,5.4915e+04_rb/)
      kbo(:,44,15) = (/ &
     &6.7443e+04_rb,6.4360e+04_rb,6.1383e+04_rb,5.8557e+04_rb,5.5857e+04_rb/)
      kbo(:,45,15) = (/ &
     &6.8616e+04_rb,6.5461e+04_rb,6.2445e+04_rb,5.9557e+04_rb,5.6818e+04_rb/)
      kbo(:,46,15) = (/ &
     &6.9855e+04_rb,6.6628e+04_rb,6.3578e+04_rb,6.0631e+04_rb,5.7835e+04_rb/)
      kbo(:,47,15) = (/ &
     &7.1240e+04_rb,6.7969e+04_rb,6.4839e+04_rb,6.1843e+04_rb,5.8987e+04_rb/)
      kbo(:,48,15) = (/ &
     &7.2637e+04_rb,6.9326e+04_rb,6.6121e+04_rb,6.3073e+04_rb,6.0145e+04_rb/)
      kbo(:,49,15) = (/ &
     &7.4017e+04_rb,7.0688e+04_rb,6.7429e+04_rb,6.4324e+04_rb,6.1340e+04_rb/)
      kbo(:,50,15) = (/ &
     &7.5349e+04_rb,7.1972e+04_rb,6.8674e+04_rb,6.5502e+04_rb,6.2467e+04_rb/)
      kbo(:,51,15) = (/ &
     &7.6637e+04_rb,7.3219e+04_rb,6.9913e+04_rb,6.6663e+04_rb,6.3587e+04_rb/)
      kbo(:,52,15) = (/ &
     &7.7955e+04_rb,7.4483e+04_rb,7.1122e+04_rb,6.7846e+04_rb,6.4721e+04_rb/)
      kbo(:,53,15) = (/ &
     &7.9280e+04_rb,7.5753e+04_rb,7.2364e+04_rb,6.9053e+04_rb,6.5858e+04_rb/)
      kbo(:,54,15) = (/ &
     &8.0495e+04_rb,7.6933e+04_rb,7.3504e+04_rb,7.0171e+04_rb,6.6914e+04_rb/)
      kbo(:,55,15) = (/ &
     &8.1662e+04_rb,7.8089e+04_rb,7.4612e+04_rb,7.1233e+04_rb,6.7959e+04_rb/)
      kbo(:,56,15) = (/ &
     &8.2843e+04_rb,7.9230e+04_rb,7.5715e+04_rb,7.2316e+04_rb,6.9008e+04_rb/)
      kbo(:,57,15) = (/ &
     &8.4005e+04_rb,8.0398e+04_rb,7.6842e+04_rb,7.3404e+04_rb,7.0077e+04_rb/)
      kbo(:,58,15) = (/ &
     &8.5078e+04_rb,8.1502e+04_rb,7.7919e+04_rb,7.4439e+04_rb,7.1083e+04_rb/)
      kbo(:,59,15) = (/ &
     &8.5505e+04_rb,8.1958e+04_rb,7.8361e+04_rb,7.4870e+04_rb,7.1482e+04_rb/)
      kbo(:,13,16) = (/ &
     &1.4816e+04_rb,1.4930e+04_rb,1.5014e+04_rb,1.5052e+04_rb,1.5044e+04_rb/)
      kbo(:,14,16) = (/ &
     &1.7836e+04_rb,1.7951e+04_rb,1.8019e+04_rb,1.8017e+04_rb,1.7984e+04_rb/)
      kbo(:,15,16) = (/ &
     &2.1381e+04_rb,2.1462e+04_rb,2.1481e+04_rb,2.1430e+04_rb,2.1342e+04_rb/)
      kbo(:,16,16) = (/ &
     &2.5477e+04_rb,2.5475e+04_rb,2.5403e+04_rb,2.5275e+04_rb,2.5089e+04_rb/)
      kbo(:,17,16) = (/ &
     &3.0124e+04_rb,3.0002e+04_rb,2.9773e+04_rb,2.9519e+04_rb,2.9186e+04_rb/)
      kbo(:,18,16) = (/ &
     &3.5286e+04_rb,3.4965e+04_rb,3.4530e+04_rb,3.4092e+04_rb,3.3540e+04_rb/)
      kbo(:,19,16) = (/ &
     &4.0852e+04_rb,4.0241e+04_rb,3.9575e+04_rb,3.8871e+04_rb,3.8057e+04_rb/)
      kbo(:,20,16) = (/ &
     &4.6699e+04_rb,4.5739e+04_rb,4.4741e+04_rb,4.3700e+04_rb,4.2576e+04_rb/)
      kbo(:,21,16) = (/ &
     &5.2654e+04_rb,5.1275e+04_rb,4.9864e+04_rb,4.8428e+04_rb,4.6961e+04_rb/)
      kbo(:,22,16) = (/ &
     &5.8403e+04_rb,5.6551e+04_rb,5.4684e+04_rb,5.2822e+04_rb,5.0976e+04_rb/)
      kbo(:,23,16) = (/ &
     &6.3816e+04_rb,6.1463e+04_rb,5.9121e+04_rb,5.6821e+04_rb,5.4590e+04_rb/)
      kbo(:,24,16) = (/ &
     &6.8713e+04_rb,6.5863e+04_rb,6.3058e+04_rb,6.0345e+04_rb,5.7751e+04_rb/)
      kbo(:,25,16) = (/ &
     &7.2989e+04_rb,6.9656e+04_rb,6.6424e+04_rb,6.3329e+04_rb,6.0404e+04_rb/)
      kbo(:,26,16) = (/ &
     &7.6570e+04_rb,7.2775e+04_rb,6.9149e+04_rb,6.5716e+04_rb,6.2522e+04_rb/)
      kbo(:,27,16) = (/ &
     &7.9459e+04_rb,7.5274e+04_rb,7.1313e+04_rb,6.7589e+04_rb,6.4169e+04_rb/)
      kbo(:,28,16) = (/ &
     &8.1683e+04_rb,7.7174e+04_rb,7.2947e+04_rb,6.8995e+04_rb,6.5376e+04_rb/)
      kbo(:,29,16) = (/ &
     &8.3285e+04_rb,7.8510e+04_rb,7.4071e+04_rb,6.9957e+04_rb,6.6188e+04_rb/)
      kbo(:,30,16) = (/ &
     &8.4346e+04_rb,7.9371e+04_rb,7.4777e+04_rb,7.0557e+04_rb,6.6673e+04_rb/)
      kbo(:,31,16) = (/ &
     &8.4928e+04_rb,7.9835e+04_rb,7.5096e+04_rb,7.0806e+04_rb,6.6861e+04_rb/)
      kbo(:,32,16) = (/ &
     &8.5101e+04_rb,7.9916e+04_rb,7.5117e+04_rb,7.0792e+04_rb,6.6791e+04_rb/)
      kbo(:,33,16) = (/ &
     &8.4949e+04_rb,7.9717e+04_rb,7.4898e+04_rb,7.0557e+04_rb,6.6529e+04_rb/)
      kbo(:,34,16) = (/ &
     &8.4743e+04_rb,7.9470e+04_rb,7.4656e+04_rb,7.0309e+04_rb,6.6256e+04_rb/)
      kbo(:,35,16) = (/ &
     &8.4858e+04_rb,7.9542e+04_rb,7.4709e+04_rb,7.0325e+04_rb,6.6240e+04_rb/)
      kbo(:,36,16) = (/ &
     &8.5440e+04_rb,8.0039e+04_rb,7.5130e+04_rb,7.0697e+04_rb,6.6564e+04_rb/)
      kbo(:,37,16) = (/ &
     &8.6760e+04_rb,8.1217e+04_rb,7.6179e+04_rb,7.1644e+04_rb,6.7431e+04_rb/)
      kbo(:,38,16) = (/ &
     &8.8049e+04_rb,8.2388e+04_rb,7.7214e+04_rb,7.2579e+04_rb,6.8294e+04_rb/)
      kbo(:,39,16) = (/ &
     &8.9280e+04_rb,8.3525e+04_rb,7.8218e+04_rb,7.3474e+04_rb,6.9111e+04_rb/)
      kbo(:,40,16) = (/ &
     &9.1121e+04_rb,8.5172e+04_rb,7.9710e+04_rb,7.4807e+04_rb,7.0344e+04_rb/)
      kbo(:,41,16) = (/ &
     &9.3033e+04_rb,8.6893e+04_rb,8.1281e+04_rb,7.6203e+04_rb,7.1626e+04_rb/)
      kbo(:,42,16) = (/ &
     &9.4965e+04_rb,8.8622e+04_rb,8.0415e+04_rb,7.7606e+04_rb,7.2910e+04_rb/)
      kbo(:,43,16) = (/ &
     &9.7303e+04_rb,9.0720e+04_rb,8.4775e+04_rb,7.9330e+04_rb,7.4454e+04_rb/)
      kbo(:,44,16) = (/ &
     &9.9859e+04_rb,9.3029e+04_rb,8.6867e+04_rb,8.1230e+04_rb,7.6153e+04_rb/)
      kbo(:,45,16) = (/ &
     &1.0249e+05_rb,9.5410e+04_rb,8.8999e+04_rb,8.3177e+04_rb,7.7890e+04_rb/)
      kbo(:,46,16) = (/ &
     &1.0536e+05_rb,9.7993e+04_rb,9.1330e+04_rb,8.5316e+04_rb,7.9796e+04_rb/)
      kbo(:,47,16) = (/ &
     &1.0871e+05_rb,1.0095e+05_rb,9.4014e+04_rb,8.7743e+04_rb,8.2024e+04_rb/)
      kbo(:,48,16) = (/ &
     &1.1228e+05_rb,1.0405e+05_rb,9.6823e+04_rb,9.0264e+04_rb,8.4322e+04_rb/)
      kbo(:,49,16) = (/ &
     &1.1602e+05_rb,1.0727e+05_rb,9.9709e+04_rb,9.2873e+04_rb,8.6711e+04_rb/)
      kbo(:,50,16) = (/ &
     &1.1975e+05_rb,1.1051e+05_rb,1.0253e+05_rb,9.5425e+04_rb,8.8999e+04_rb/)
      kbo(:,51,16) = (/ &
     &1.2354e+05_rb,1.1380e+05_rb,1.0536e+05_rb,9.7987e+04_rb,9.1299e+04_rb/)
      kbo(:,52,16) = (/ &
     &1.2745e+05_rb,1.1723e+05_rb,1.0833e+05_rb,1.0062e+05_rb,9.3694e+04_rb/)
      kbo(:,53,16) = (/ &
     &1.3160e+05_rb,1.2088e+05_rb,1.1148e+05_rb,1.0337e+05_rb,9.6183e+04_rb/)
      kbo(:,54,16) = (/ &
     &1.3555e+05_rb,1.2436e+05_rb,1.1452e+05_rb,1.0598e+05_rb,9.8537e+04_rb/)
      kbo(:,55,16) = (/ &
     &1.3946e+05_rb,1.2781e+05_rb,1.1754e+05_rb,1.0859e+05_rb,1.0085e+05_rb/)
      kbo(:,56,16) = (/ &
     &1.4362e+05_rb,1.3143e+05_rb,1.2072e+05_rb,1.1136e+05_rb,1.0326e+05_rb/)
      kbo(:,57,16) = (/ &
     &1.4797e+05_rb,1.3519e+05_rb,1.2403e+05_rb,1.1423e+05_rb,1.0573e+05_rb/)
      kbo(:,58,16) = (/ &
     &1.5241e+05_rb,1.3886e+05_rb,1.2729e+05_rb,1.1707e+05_rb,1.0818e+05_rb/)
      kbo(:,59,16) = (/ &
     &1.5431e+05_rb,1.4041e+05_rb,1.2862e+05_rb,1.1825e+05_rb,1.0919e+05_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &2.7075e-06_rb,2.2609e-06_rb,1.5633e-06_rb,8.7484e-07_rb,5.5470e-07_rb,4.8456e-07_rb, &
     &4.7463e-07_rb,4.6154e-07_rb,4.4425e-07_rb,4.2960e-07_rb,4.2626e-07_rb,4.1715e-07_rb, &
     &4.2607e-07_rb,3.6616e-07_rb,2.6366e-07_rb,2.6029e-07_rb/)
      forrefo(2,:) = (/ &
     &2.6759e-06_rb,2.2237e-06_rb,1.4466e-06_rb,9.3032e-07_rb,6.4927e-07_rb,5.4809e-07_rb, &
     &4.9504e-07_rb,4.6305e-07_rb,4.4873e-07_rb,4.2146e-07_rb,4.2176e-07_rb,4.2812e-07_rb, &
     &4.0529e-07_rb,4.0969e-07_rb,2.9442e-07_rb,2.6821e-07_rb/)
      forrefo(3,:) = (/ &
     &2.6608e-06_rb,2.1140e-06_rb,1.4838e-06_rb,9.2083e-07_rb,6.3350e-07_rb,5.7195e-07_rb, &
     &6.2253e-07_rb,5.1783e-07_rb,4.4749e-07_rb,4.3261e-07_rb,4.2553e-07_rb,4.2175e-07_rb, &
     &4.1085e-07_rb,4.0358e-07_rb,3.5340e-07_rb,2.7191e-07_rb/)
      forrefo(4,:) = (/ &
     &2.6412e-06_rb,1.9814e-06_rb,1.2672e-06_rb,8.1129e-07_rb,7.1447e-07_rb,7.5026e-07_rb, &
     &7.4386e-07_rb,7.2759e-07_rb,7.3583e-07_rb,7.6493e-07_rb,8.8959e-07_rb,7.5534e-07_rb, &
     &5.3734e-07_rb,4.5572e-07_rb,4.1676e-07_rb,3.6198e-07_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 4.67262e-03_rb, 3.95211e-03_rb, 3.34270e-03_rb, 2.82726e-03_rb, 2.39130e-03_rb, &
     & 2.02256e-03_rb, 1.71069e-03_rb, 1.44690e-03_rb, 1.22379e-03_rb, 1.03508e-03_rb/)
      selfrefo(:, 2) = (/ &
     & 4.42593e-03_rb, 3.73338e-03_rb, 3.14920e-03_rb, 2.65643e-03_rb, 2.24076e-03_rb, &
     & 1.89014e-03_rb, 1.59438e-03_rb, 1.34490e-03_rb, 1.13446e-03_rb, 9.56943e-04_rb/)
      selfrefo(:, 3) = (/ &
     & 3.96072e-03_rb, 3.33789e-03_rb, 2.81300e-03_rb, 2.37065e-03_rb, 1.99786e-03_rb, &
     & 1.68369e-03_rb, 1.41893e-03_rb, 1.19580e-03_rb, 1.00776e-03_rb, 8.49286e-04_rb/)
      selfrefo(:, 4) = (/ &
     & 3.71833e-03_rb, 3.10030e-03_rb, 2.58500e-03_rb, 2.15535e-03_rb, 1.79711e-03_rb, &
     & 1.49841e-03_rb, 1.24936e-03_rb, 1.04170e-03_rb, 8.68558e-04_rb, 7.24195e-04_rb/)
      selfrefo(:, 5) = (/ &
     & 3.55755e-03_rb, 2.95355e-03_rb, 2.45210e-03_rb, 2.03578e-03_rb, 1.69015e-03_rb, &
     & 1.40320e-03_rb, 1.16497e-03_rb, 9.67180e-04_rb, 8.02973e-04_rb, 6.66646e-04_rb/)
      selfrefo(:, 6) = (/ &
     & 3.47601e-03_rb, 2.88628e-03_rb, 2.39660e-03_rb, 1.99000e-03_rb, 1.65238e-03_rb, &
     & 1.37204e-03_rb, 1.13927e-03_rb, 9.45980e-04_rb, 7.85487e-04_rb, 6.52224e-04_rb/)
      selfrefo(:, 7) = (/ &
     & 3.44479e-03_rb, 2.86224e-03_rb, 2.37820e-03_rb, 1.97602e-03_rb, 1.64185e-03_rb, &
     & 1.36420e-03_rb, 1.13350e-03_rb, 9.41810e-04_rb, 7.82539e-04_rb, 6.50204e-04_rb/)
      selfrefo(:, 8) = (/ &
     & 3.40154e-03_rb, 2.82953e-03_rb, 2.35370e-03_rb, 1.95789e-03_rb, 1.62864e-03_rb, &
     & 1.35476e-03_rb, 1.12694e-03_rb, 9.37430e-04_rb, 7.79788e-04_rb, 6.48655e-04_rb/)
      selfrefo(:, 9) = (/ &
     & 3.39380e-03_rb, 2.82288e-03_rb, 2.34800e-03_rb, 1.95301e-03_rb, 1.62446e-03_rb, &
     & 1.35119e-03_rb, 1.12389e-03_rb, 9.34820e-04_rb, 7.77560e-04_rb, 6.46755e-04_rb/)
      selfrefo(:,10) = (/ &
     & 3.37185e-03_rb, 2.80654e-03_rb, 2.33600e-03_rb, 1.94435e-03_rb, 1.61837e-03_rb, &
     & 1.34704e-03_rb, 1.12120e-03_rb, 9.33220e-04_rb, 7.76759e-04_rb, 6.46530e-04_rb/)
      selfrefo(:,11) = (/ &
     & 3.37924e-03_rb, 2.81172e-03_rb, 2.33950e-03_rb, 1.94659e-03_rb, 1.61967e-03_rb, &
     & 1.34765e-03_rb, 1.12132e-03_rb, 9.33000e-04_rb, 7.76306e-04_rb, 6.45930e-04_rb/)
      selfrefo(:,12) = (/ &
     & 3.39658e-03_rb, 2.82289e-03_rb, 2.34610e-03_rb, 1.94984e-03_rb, 1.62051e-03_rb, &
     & 1.34680e-03_rb, 1.11933e-03_rb, 9.30270e-04_rb, 7.73146e-04_rb, 6.42561e-04_rb/)
      selfrefo(:,13) = (/ &
     & 3.36070e-03_rb, 2.79913e-03_rb, 2.33140e-03_rb, 1.94183e-03_rb, 1.61735e-03_rb, &
     & 1.34709e-03_rb, 1.12199e-03_rb, 9.34510e-04_rb, 7.78354e-04_rb, 6.48292e-04_rb/)
      selfrefo(:,14) = (/ &
     & 3.40428e-03_rb, 2.81994e-03_rb, 2.33590e-03_rb, 1.93495e-03_rb, 1.60282e-03_rb, &
     & 1.32770e-03_rb, 1.09980e-03_rb, 9.11020e-04_rb, 7.54645e-04_rb, 6.25111e-04_rb/)
      selfrefo(:,15) = (/ &
     & 3.27075e-03_rb, 2.70783e-03_rb, 2.24180e-03_rb, 1.85597e-03_rb, 1.53655e-03_rb, &
     & 1.27210e-03_rb, 1.05317e-03_rb, 8.71910e-04_rb, 7.21849e-04_rb, 5.97615e-04_rb/)
      selfrefo(:,16) = (/ &
     & 3.23123e-03_rb, 2.67891e-03_rb, 2.22100e-03_rb, 1.84136e-03_rb, 1.52661e-03_rb, &
     & 1.26567e-03_rb, 1.04932e-03_rb, 8.69960e-04_rb, 7.21256e-04_rb, 5.97970e-04_rb/)

      end subroutine lw_kgb14

! **************************************************************************
      subroutine lw_kgb15
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg15, only : fracrefao, kao, kao_mn2, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level : P = 1053. mb, T = 294.2 K
      fracrefao(:, 1) = (/ &
     &  1.0689e-01_rb,1.1563e-01_rb,1.2447e-01_rb,1.2921e-01_rb,1.2840e-01_rb,1.2113e-01_rb, &
     &  1.0643e-01_rb,8.4987e-02_rb,6.0142e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 2) = (/ &
     &  1.0782e-01_rb,1.1637e-01_rb,1.2290e-01_rb,1.2911e-01_rb,1.2841e-01_rb,1.2113e-01_rb, &
     &  1.0643e-01_rb,8.4987e-02_rb,6.0142e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 3) = (/ &
     &  1.0858e-01_rb,1.1860e-01_rb,1.2237e-01_rb,1.2665e-01_rb,1.2841e-01_rb,1.2111e-01_rb, &
     &  1.0642e-01_rb,8.4987e-02_rb,6.0142e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 4) = (/ &
     &  1.1022e-01_rb,1.1965e-01_rb,1.2334e-01_rb,1.2383e-01_rb,1.2761e-01_rb,1.2109e-01_rb, &
     &  1.0642e-01_rb,8.4987e-02_rb,6.0142e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 5) = (/ &
     &  1.1342e-01_rb,1.2069e-01_rb,1.2360e-01_rb,1.2447e-01_rb,1.2340e-01_rb,1.2020e-01_rb, &
     &  1.0639e-01_rb,8.4987e-02_rb,6.0142e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 6) = (/ &
     &  1.1771e-01_rb,1.2280e-01_rb,1.2177e-01_rb,1.2672e-01_rb,1.2398e-01_rb,1.1787e-01_rb, &
     &  1.0131e-01_rb,8.4987e-02_rb,6.0142e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 7) = (/ &
     &  1.2320e-01_rb,1.2491e-01_rb,1.2001e-01_rb,1.2936e-01_rb,1.2653e-01_rb,1.1929e-01_rb, &
     &  9.8955e-02_rb,7.4887e-02_rb,6.0142e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 8) = (/ &
     &  1.3105e-01_rb,1.2563e-01_rb,1.3055e-01_rb,1.2854e-01_rb,1.3402e-01_rb,1.1571e-01_rb, &
     &  9.4876e-02_rb,6.0459e-02_rb,5.6457e-02_rb,6.6798e-03_rb,5.5293e-03_rb,4.3700e-03_rb, &
     &  3.2061e-03_rb,2.0476e-03_rb,7.7366e-04_rb,1.0897e-04_rb/)
      fracrefao(:, 9) = (/ &
     &  1.1375e-01_rb,1.2090e-01_rb,1.2348e-01_rb,1.2458e-01_rb,1.2406e-01_rb,1.1921e-01_rb, &
     &  1.0802e-01_rb,8.6613e-02_rb,5.8125e-02_rb,6.2984e-03_rb,5.2359e-03_rb,4.0641e-03_rb, &
     &  2.9379e-03_rb,1.9001e-03_rb,7.2646e-04_rb,1.0553e-04_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &3.4242e-02_rb,1.7071e-01_rb,2.6138e-01_rb,3.1721e-01_rb,3.2432e-01_rb,2.6831e-01_rb, &
     &2.0528e-01_rb,1.1088e-01_rb,5.8968e-01_rb/)
      kao(:, 2, 1, 1) = (/ &
     &3.3652e-02_rb,1.6779e-01_rb,2.5725e-01_rb,3.0829e-01_rb,3.1520e-01_rb,2.6815e-01_rb, &
     &2.0103e-01_rb,1.1213e-01_rb,5.6496e-01_rb/)
      kao(:, 3, 1, 1) = (/ &
     &3.2936e-02_rb,1.6580e-01_rb,2.5340e-01_rb,3.0468e-01_rb,3.0583e-01_rb,2.6658e-01_rb, &
     &1.9781e-01_rb,1.1388e-01_rb,5.5053e-01_rb/)
      kao(:, 4, 1, 1) = (/ &
     &3.2004e-02_rb,1.6475e-01_rb,2.5029e-01_rb,3.0171e-01_rb,3.0308e-01_rb,2.6842e-01_rb, &
     &1.9857e-01_rb,1.1861e-01_rb,5.4961e-01_rb/)
      kao(:, 5, 1, 1) = (/ &
     &3.0990e-02_rb,1.6410e-01_rb,2.4670e-01_rb,2.9476e-01_rb,3.0044e-01_rb,2.7387e-01_rb, &
     &1.9493e-01_rb,1.2451e-01_rb,5.3996e-01_rb/)
      kao(:, 1, 2, 1) = (/ &
     &3.0564e-02_rb,1.4940e-01_rb,2.2694e-01_rb,2.6721e-01_rb,2.6456e-01_rb,2.1891e-01_rb, &
     &1.6459e-01_rb,9.1113e-02_rb,4.6587e-01_rb/)
      kao(:, 2, 2, 1) = (/ &
     &2.9826e-02_rb,1.4880e-01_rb,2.2345e-01_rb,2.6474e-01_rb,2.5889e-01_rb,2.2136e-01_rb, &
     &1.6243e-01_rb,9.1934e-02_rb,4.6056e-01_rb/)
      kao(:, 3, 2, 1) = (/ &
     &2.9292e-02_rb,1.4800e-01_rb,2.2058e-01_rb,2.6235e-01_rb,2.5684e-01_rb,2.2295e-01_rb, &
     &1.6276e-01_rb,9.5842e-02_rb,4.5687e-01_rb/)
      kao(:, 4, 2, 1) = (/ &
     &2.8631e-02_rb,1.4795e-01_rb,2.1809e-01_rb,2.5690e-01_rb,2.5387e-01_rb,2.2588e-01_rb, &
     &1.6201e-01_rb,9.9403e-02_rb,4.5029e-01_rb/)
      kao(:, 5, 2, 1) = (/ &
     &2.7916e-02_rb,1.4649e-01_rb,2.1539e-01_rb,2.5025e-01_rb,2.5060e-01_rb,2.2852e-01_rb, &
     &1.6118e-01_rb,1.0330e-01_rb,4.4510e-01_rb/)
      kao(:, 1, 3, 1) = (/ &
     &2.7060e-02_rb,1.3249e-01_rb,1.9563e-01_rb,2.2645e-01_rb,2.1616e-01_rb,1.7969e-01_rb, &
     &1.3284e-01_rb,7.4965e-02_rb,3.7244e-01_rb/)
      kao(:, 2, 3, 1) = (/ &
     &2.6723e-02_rb,1.3208e-01_rb,1.9386e-01_rb,2.2490e-01_rb,2.1534e-01_rb,1.8332e-01_rb, &
     &1.3372e-01_rb,7.6721e-02_rb,3.7364e-01_rb/)
      kao(:, 3, 3, 1) = (/ &
     &2.6384e-02_rb,1.3168e-01_rb,1.9203e-01_rb,2.2240e-01_rb,2.1507e-01_rb,1.8752e-01_rb, &
     &1.3386e-01_rb,7.9482e-02_rb,3.7891e-01_rb/)
      kao(:, 4, 3, 1) = (/ &
     &2.5689e-02_rb,1.3020e-01_rb,1.8980e-01_rb,2.1868e-01_rb,2.1152e-01_rb,1.8912e-01_rb, &
     &1.3349e-01_rb,8.2411e-02_rb,3.7480e-01_rb/)
      kao(:, 5, 3, 1) = (/ &
     &2.4939e-02_rb,1.2901e-01_rb,1.8570e-01_rb,2.1295e-01_rb,2.0942e-01_rb,1.8777e-01_rb, &
     &1.3349e-01_rb,8.5566e-02_rb,3.6973e-01_rb/)
      kao(:, 1, 4, 1) = (/ &
     &2.4242e-02_rb,1.1699e-01_rb,1.6835e-01_rb,1.9257e-01_rb,1.7909e-01_rb,1.4906e-01_rb, &
     &1.0978e-01_rb,6.3298e-02_rb,3.0317e-01_rb/)
      kao(:, 2, 4, 1) = (/ &
     &2.3651e-02_rb,1.1679e-01_rb,1.6721e-01_rb,1.9147e-01_rb,1.7987e-01_rb,1.5405e-01_rb, &
     &1.1069e-01_rb,6.4435e-02_rb,3.0873e-01_rb/)
      kao(:, 3, 4, 1) = (/ &
     &2.2665e-02_rb,1.1583e-01_rb,1.6543e-01_rb,1.8831e-01_rb,1.8066e-01_rb,1.5730e-01_rb, &
     &1.1107e-01_rb,6.6643e-02_rb,3.1154e-01_rb/)
      kao(:, 4, 4, 1) = (/ &
     &2.1814e-02_rb,1.1462e-01_rb,1.6249e-01_rb,1.8491e-01_rb,1.7772e-01_rb,1.5750e-01_rb, &
     &1.1124e-01_rb,6.9078e-02_rb,3.1214e-01_rb/)
      kao(:, 5, 4, 1) = (/ &
     &2.1129e-02_rb,1.1306e-01_rb,1.5907e-01_rb,1.7919e-01_rb,1.7465e-01_rb,1.5650e-01_rb, &
     &1.1133e-01_rb,7.1551e-02_rb,3.0812e-01_rb/)
      kao(:, 1, 5, 1) = (/ &
     &2.0560e-02_rb,1.0249e-01_rb,1.4447e-01_rb,1.6390e-01_rb,1.4827e-01_rb,1.2487e-01_rb, &
     &9.0642e-02_rb,5.3488e-02_rb,2.4754e-01_rb/)
      kao(:, 2, 5, 1) = (/ &
     &1.9799e-02_rb,1.0201e-01_rb,1.4357e-01_rb,1.6295e-01_rb,1.5070e-01_rb,1.2910e-01_rb, &
     &9.1763e-02_rb,5.4394e-02_rb,2.5641e-01_rb/)
      kao(:, 3, 5, 1) = (/ &
     &1.9183e-02_rb,1.0090e-01_rb,1.4151e-01_rb,1.5954e-01_rb,1.5069e-01_rb,1.3095e-01_rb, &
     &9.2718e-02_rb,5.6321e-02_rb,2.5912e-01_rb/)
      kao(:, 4, 5, 1) = (/ &
     &1.8512e-02_rb,9.9552e-02_rb,1.3834e-01_rb,1.5497e-01_rb,1.4881e-01_rb,1.3077e-01_rb, &
     &9.3091e-02_rb,5.8675e-02_rb,2.5974e-01_rb/)
      kao(:, 5, 5, 1) = (/ &
     &1.7786e-02_rb,9.7242e-02_rb,1.3532e-01_rb,1.5076e-01_rb,1.4713e-01_rb,1.3062e-01_rb, &
     &9.3284e-02_rb,6.0479e-02_rb,2.5760e-01_rb/)
      kao(:, 1, 6, 1) = (/ &
     &1.7311e-02_rb,8.8922e-02_rb,1.2346e-01_rb,1.3907e-01_rb,1.2282e-01_rb,1.0362e-01_rb, &
     &7.4353e-02_rb,4.4575e-02_rb,2.0465e-01_rb/)
      kao(:, 2, 6, 1) = (/ &
     &1.6759e-02_rb,8.7881e-02_rb,1.2272e-01_rb,1.3747e-01_rb,1.2636e-01_rb,1.0747e-01_rb, &
     &7.5966e-02_rb,4.6065e-02_rb,2.1117e-01_rb/)
      kao(:, 3, 6, 1) = (/ &
     &1.6118e-02_rb,8.6674e-02_rb,1.1960e-01_rb,1.3378e-01_rb,1.2578e-01_rb,1.0890e-01_rb, &
     &7.7032e-02_rb,4.7370e-02_rb,2.1535e-01_rb/)
      kao(:, 4, 6, 1) = (/ &
     &1.5604e-02_rb,8.5004e-02_rb,1.1715e-01_rb,1.3060e-01_rb,1.2464e-01_rb,1.0936e-01_rb, &
     &7.8068e-02_rb,4.9979e-02_rb,2.1513e-01_rb/)
      kao(:, 5, 6, 1) = (/ &
     &1.5010e-02_rb,8.2612e-02_rb,1.1457e-01_rb,1.2694e-01_rb,1.2379e-01_rb,1.0886e-01_rb, &
     &7.8577e-02_rb,5.1634e-02_rb,2.1475e-01_rb/)
      kao(:, 1, 7, 1) = (/ &
     &1.4769e-02_rb,7.6038e-02_rb,1.0565e-01_rb,1.1761e-01_rb,1.0346e-01_rb,8.6214e-02_rb, &
     &6.1132e-02_rb,3.6999e-02_rb,1.7055e-01_rb/)
      kao(:, 2, 7, 1) = (/ &
     &1.4212e-02_rb,7.5191e-02_rb,1.0412e-01_rb,1.1565e-01_rb,1.0591e-01_rb,8.9750e-02_rb, &
     &6.2857e-02_rb,3.8703e-02_rb,1.7619e-01_rb/)
      kao(:, 3, 7, 1) = (/ &
     &1.3756e-02_rb,7.4145e-02_rb,1.0177e-01_rb,1.1260e-01_rb,1.0586e-01_rb,9.1675e-02_rb, &
     &6.4846e-02_rb,4.0069e-02_rb,1.8020e-01_rb/)
      kao(:, 4, 7, 1) = (/ &
     &1.3258e-02_rb,7.2484e-02_rb,9.9439e-02_rb,1.0966e-01_rb,1.0522e-01_rb,9.2290e-02_rb, &
     &6.5394e-02_rb,4.2345e-02_rb,1.8038e-01_rb/)
      kao(:, 5, 7, 1) = (/ &
     &1.2766e-02_rb,7.0689e-02_rb,9.6896e-02_rb,1.0710e-01_rb,1.0411e-01_rb,9.1721e-02_rb, &
     &6.6375e-02_rb,4.4166e-02_rb,1.8031e-01_rb/)
      kao(:, 1, 8, 1) = (/ &
     &1.2774e-02_rb,6.5331e-02_rb,9.0505e-02_rb,1.0027e-01_rb,8.8579e-02_rb,7.2873e-02_rb, &
     &5.1439e-02_rb,3.0866e-02_rb,1.4484e-01_rb/)
      kao(:, 2, 8, 1) = (/ &
     &1.2386e-02_rb,6.4820e-02_rb,8.9216e-02_rb,9.8573e-02_rb,9.0271e-02_rb,7.6134e-02_rb, &
     &5.3003e-02_rb,3.2488e-02_rb,1.4948e-01_rb/)
      kao(:, 3, 8, 1) = (/ &
     &1.1934e-02_rb,6.3787e-02_rb,8.6950e-02_rb,9.5946e-02_rb,9.0138e-02_rb,7.8085e-02_rb, &
     &5.5347e-02_rb,3.4134e-02_rb,1.5219e-01_rb/)
      kao(:, 4, 8, 1) = (/ &
     &1.1544e-02_rb,6.2588e-02_rb,8.5063e-02_rb,9.3318e-02_rb,8.9759e-02_rb,7.8769e-02_rb, &
     &5.5958e-02_rb,3.6086e-02_rb,1.5324e-01_rb/)
      kao(:, 5, 8, 1) = (/ &
     &1.1084e-02_rb,6.0720e-02_rb,8.2631e-02_rb,9.1723e-02_rb,8.9027e-02_rb,7.8819e-02_rb, &
     &5.6823e-02_rb,3.7768e-02_rb,1.5367e-01_rb/)
      kao(:, 1, 9, 1) = (/ &
     &1.1061e-02_rb,5.5946e-02_rb,7.7372e-02_rb,8.4825e-02_rb,7.4671e-02_rb,6.0881e-02_rb, &
     &4.3320e-02_rb,2.5693e-02_rb,1.2149e-01_rb/)
      kao(:, 2, 9, 1) = (/ &
     &1.0663e-02_rb,5.5517e-02_rb,7.6169e-02_rb,8.3555e-02_rb,7.6380e-02_rb,6.3832e-02_rb, &
     &4.4653e-02_rb,2.7044e-02_rb,1.2592e-01_rb/)
      kao(:, 3, 9, 1) = (/ &
     &1.0298e-02_rb,5.4609e-02_rb,7.4061e-02_rb,8.1380e-02_rb,7.6283e-02_rb,6.5805e-02_rb, &
     &4.6544e-02_rb,2.8652e-02_rb,1.2784e-01_rb/)
      kao(:, 4, 9, 1) = (/ &
     &9.9642e-03_rb,5.3590e-02_rb,7.2457e-02_rb,7.9291e-02_rb,7.6297e-02_rb,6.6701e-02_rb, &
     &4.7215e-02_rb,3.0429e-02_rb,1.2985e-01_rb/)
      kao(:, 5, 9, 1) = (/ &
     &9.5838e-03_rb,5.1774e-02_rb,7.0525e-02_rb,7.7952e-02_rb,7.5761e-02_rb,6.7035e-02_rb, &
     &4.8099e-02_rb,3.1771e-02_rb,1.2981e-01_rb/)
      kao(:, 1,10, 1) = (/ &
     &9.5065e-03_rb,4.7743e-02_rb,6.5906e-02_rb,7.1569e-02_rb,6.3088e-02_rb,5.1008e-02_rb, &
     &3.6270e-02_rb,2.1647e-02_rb,1.0151e-01_rb/)
      kao(:, 2,10, 1) = (/ &
     &9.1696e-03_rb,4.7417e-02_rb,6.4599e-02_rb,7.0371e-02_rb,6.4348e-02_rb,5.3431e-02_rb, &
     &3.7744e-02_rb,2.2620e-02_rb,1.0603e-01_rb/)
      kao(:, 3,10, 1) = (/ &
     &8.8668e-03_rb,4.6740e-02_rb,6.2818e-02_rb,6.8749e-02_rb,6.4307e-02_rb,5.5442e-02_rb, &
     &3.9323e-02_rb,2.3889e-02_rb,1.0716e-01_rb/)
      kao(:, 4,10, 1) = (/ &
     &8.5593e-03_rb,4.5273e-02_rb,6.1475e-02_rb,6.7236e-02_rb,6.4539e-02_rb,5.6295e-02_rb, &
     &4.0023e-02_rb,2.5561e-02_rb,1.0908e-01_rb/)
      kao(:, 5,10, 1) = (/ &
     &8.2765e-03_rb,4.3918e-02_rb,5.9902e-02_rb,6.5936e-02_rb,6.4146e-02_rb,5.6753e-02_rb, &
     &4.0609e-02_rb,2.6739e-02_rb,1.0938e-01_rb/)
      kao(:, 1,11, 1) = (/ &
     &8.0135e-03_rb,4.0574e-02_rb,5.5523e-02_rb,5.9842e-02_rb,5.3590e-02_rb,4.3205e-02_rb, &
     &3.0578e-02_rb,1.8553e-02_rb,8.6419e-02_rb/)
      kao(:, 2,11, 1) = (/ &
     &7.7438e-03_rb,4.0051e-02_rb,5.3954e-02_rb,5.8561e-02_rb,5.4366e-02_rb,4.5533e-02_rb, &
     &3.1980e-02_rb,1.9368e-02_rb,8.9461e-02_rb/)
      kao(:, 3,11, 1) = (/ &
     &7.4839e-03_rb,3.9067e-02_rb,5.2580e-02_rb,5.7160e-02_rb,5.4156e-02_rb,4.6927e-02_rb, &
     &3.3356e-02_rb,2.0535e-02_rb,9.0297e-02_rb/)
      kao(:, 4,11, 1) = (/ &
     &7.2468e-03_rb,3.7809e-02_rb,5.1257e-02_rb,5.6080e-02_rb,5.4294e-02_rb,4.7574e-02_rb, &
     &3.3980e-02_rb,2.1720e-02_rb,9.1828e-02_rb/)
      kao(:, 5,11, 1) = (/ &
     &6.9859e-03_rb,3.6737e-02_rb,5.0287e-02_rb,5.5092e-02_rb,5.3946e-02_rb,4.8082e-02_rb, &
     &3.4738e-02_rb,2.2682e-02_rb,9.1913e-02_rb/)
      kao(:, 1,12, 1) = (/ &
     &6.8723e-03_rb,3.4461e-02_rb,4.6618e-02_rb,5.0325e-02_rb,4.5843e-02_rb,3.7154e-02_rb, &
     &2.6350e-02_rb,1.5945e-02_rb,7.4796e-02_rb/)
      kao(:, 2,12, 1) = (/ &
     &6.6342e-03_rb,3.3814e-02_rb,4.5285e-02_rb,4.9258e-02_rb,4.6140e-02_rb,3.8916e-02_rb, &
     &2.7614e-02_rb,1.6684e-02_rb,7.6431e-02_rb/)
      kao(:, 3,12, 1) = (/ &
     &6.4256e-03_rb,3.2718e-02_rb,4.4302e-02_rb,4.8163e-02_rb,4.5963e-02_rb,4.0038e-02_rb, &
     &2.8672e-02_rb,1.7798e-02_rb,7.6864e-02_rb/)
      kao(:, 4,12, 1) = (/ &
     &6.1980e-03_rb,3.1806e-02_rb,4.3232e-02_rb,4.7321e-02_rb,4.6023e-02_rb,4.0771e-02_rb, &
     &2.9167e-02_rb,1.8567e-02_rb,7.7874e-02_rb/)
      kao(:, 5,12, 1) = (/ &
     &5.9663e-03_rb,3.0964e-02_rb,4.2379e-02_rb,4.6375e-02_rb,4.5692e-02_rb,4.1306e-02_rb, &
     &2.9920e-02_rb,1.9375e-02_rb,7.7903e-02_rb/)
      kao(:, 1,13, 1) = (/ &
     &6.1312e-03_rb,2.9682e-02_rb,3.9975e-02_rb,4.3642e-02_rb,4.0252e-02_rb,3.3035e-02_rb, &
     &2.3558e-02_rb,1.4050e-02_rb,6.6647e-02_rb/)
      kao(:, 2,13, 1) = (/ &
     &5.9440e-03_rb,2.8920e-02_rb,3.8788e-02_rb,4.2829e-02_rb,4.0420e-02_rb,3.4105e-02_rb, &
     &2.4570e-02_rb,1.4699e-02_rb,6.7206e-02_rb/)
      kao(:, 3,13, 1) = (/ &
     &5.7437e-03_rb,2.7994e-02_rb,3.7951e-02_rb,4.1862e-02_rb,4.0303e-02_rb,3.5364e-02_rb, &
     &2.5449e-02_rb,1.5507e-02_rb,6.7644e-02_rb/)
      kao(:, 4,13, 1) = (/ &
     &5.5406e-03_rb,2.7263e-02_rb,3.7056e-02_rb,4.0900e-02_rb,3.9818e-02_rb,3.6193e-02_rb, &
     &2.6077e-02_rb,1.6255e-02_rb,6.7402e-02_rb/)
      kao(:, 5,13, 1) = (/ &
     &5.3457e-03_rb,2.6652e-02_rb,3.6336e-02_rb,3.9995e-02_rb,3.9467e-02_rb,3.6352e-02_rb, &
     &2.6575e-02_rb,1.6998e-02_rb,6.7304e-02_rb/)
      kao(:, 1, 1, 2) = (/ &
     &4.1184e-02_rb,2.7179e-01_rb,3.3427e-01_rb,3.7214e-01_rb,4.0092e-01_rb,3.9640e-01_rb, &
     &3.3752e-01_rb,3.1662e-01_rb,7.0395e-01_rb/)
      kao(:, 2, 1, 2) = (/ &
     &3.9613e-02_rb,2.5448e-01_rb,3.1529e-01_rb,3.6420e-01_rb,3.9285e-01_rb,3.8292e-01_rb, &
     &3.4622e-01_rb,3.3570e-01_rb,7.0351e-01_rb/)
      kao(:, 3, 1, 2) = (/ &
     &3.8482e-02_rb,2.4018e-01_rb,3.0435e-01_rb,3.6080e-01_rb,3.8815e-01_rb,3.7286e-01_rb, &
     &3.5535e-01_rb,3.5400e-01_rb,6.9477e-01_rb/)
      kao(:, 4, 1, 2) = (/ &
     &3.7848e-02_rb,2.2963e-01_rb,2.9537e-01_rb,3.6193e-01_rb,3.8220e-01_rb,3.7308e-01_rb, &
     &3.6939e-01_rb,3.6570e-01_rb,6.8368e-01_rb/)
      kao(:, 5, 1, 2) = (/ &
     &3.7774e-02_rb,2.2394e-01_rb,2.9359e-01_rb,3.5744e-01_rb,3.7874e-01_rb,3.6962e-01_rb, &
     &3.8607e-01_rb,3.7579e-01_rb,6.9280e-01_rb/)
      kao(:, 1, 2, 2) = (/ &
     &3.5631e-02_rb,2.2069e-01_rb,2.7258e-01_rb,3.1550e-01_rb,3.3342e-01_rb,3.2657e-01_rb, &
     &2.8024e-01_rb,2.7588e-01_rb,5.9197e-01_rb/)
      kao(:, 2, 2, 2) = (/ &
     &3.4562e-02_rb,2.0891e-01_rb,2.6384e-01_rb,3.1412e-01_rb,3.2954e-01_rb,3.2211e-01_rb, &
     &2.9490e-01_rb,2.9533e-01_rb,5.8323e-01_rb/)
      kao(:, 3, 2, 2) = (/ &
     &3.3571e-02_rb,1.9802e-01_rb,2.5784e-01_rb,3.1122e-01_rb,3.2677e-01_rb,3.1912e-01_rb, &
     &3.0590e-01_rb,3.0715e-01_rb,5.7853e-01_rb/)
      kao(:, 4, 2, 2) = (/ &
     &3.2736e-02_rb,1.9123e-01_rb,2.5333e-01_rb,3.0890e-01_rb,3.2624e-01_rb,3.1676e-01_rb, &
     &3.1863e-01_rb,3.1850e-01_rb,5.8754e-01_rb/)
      kao(:, 5, 2, 2) = (/ &
     &3.2264e-02_rb,1.8835e-01_rb,2.5452e-01_rb,3.0891e-01_rb,3.2749e-01_rb,3.1271e-01_rb, &
     &3.2651e-01_rb,3.2839e-01_rb,5.9068e-01_rb/)
      kao(:, 1, 3, 2) = (/ &
     &3.1584e-02_rb,1.8152e-01_rb,2.2653e-01_rb,2.6984e-01_rb,2.8266e-01_rb,2.7419e-01_rb, &
     &2.3718e-01_rb,2.3355e-01_rb,4.9551e-01_rb/)
      kao(:, 2, 3, 2) = (/ &
     &3.0265e-02_rb,1.7457e-01_rb,2.2109e-01_rb,2.6703e-01_rb,2.7958e-01_rb,2.7087e-01_rb, &
     &2.4754e-01_rb,2.5014e-01_rb,4.9470e-01_rb/)
      kao(:, 3, 3, 2) = (/ &
     &2.9150e-02_rb,1.6735e-01_rb,2.1902e-01_rb,2.6391e-01_rb,2.7849e-01_rb,2.6534e-01_rb, &
     &2.5781e-01_rb,2.6293e-01_rb,4.9430e-01_rb/)
      kao(:, 4, 3, 2) = (/ &
     &2.8592e-02_rb,1.6364e-01_rb,2.1995e-01_rb,2.6189e-01_rb,2.7866e-01_rb,2.6352e-01_rb, &
     &2.7044e-01_rb,2.7363e-01_rb,4.9527e-01_rb/)
      kao(:, 5, 3, 2) = (/ &
     &2.8231e-02_rb,1.6046e-01_rb,2.1887e-01_rb,2.6443e-01_rb,2.7635e-01_rb,2.6884e-01_rb, &
     &2.7778e-01_rb,2.8066e-01_rb,4.9153e-01_rb/)
      kao(:, 1, 4, 2) = (/ &
     &2.7586e-02_rb,1.5275e-01_rb,1.9293e-01_rb,2.2899e-01_rb,2.4107e-01_rb,2.3244e-01_rb, &
     &2.0047e-01_rb,1.9534e-01_rb,4.2244e-01_rb/)
      kao(:, 2, 4, 2) = (/ &
     &2.6887e-02_rb,1.4829e-01_rb,1.9195e-01_rb,2.2560e-01_rb,2.3907e-01_rb,2.2550e-01_rb, &
     &2.1032e-01_rb,2.1041e-01_rb,4.2198e-01_rb/)
      kao(:, 3, 4, 2) = (/ &
     &2.6805e-02_rb,1.4416e-01_rb,1.9016e-01_rb,2.2531e-01_rb,2.3789e-01_rb,2.2485e-01_rb, &
     &2.1879e-01_rb,2.2259e-01_rb,4.1901e-01_rb/)
      kao(:, 4, 4, 2) = (/ &
     &2.6734e-02_rb,1.4087e-01_rb,1.9109e-01_rb,2.2675e-01_rb,2.3622e-01_rb,2.2628e-01_rb, &
     &2.2901e-01_rb,2.3158e-01_rb,4.1030e-01_rb/)
      kao(:, 5, 4, 2) = (/ &
     &2.6617e-02_rb,1.3796e-01_rb,1.9377e-01_rb,2.2557e-01_rb,2.3342e-01_rb,2.2649e-01_rb, &
     &2.3523e-01_rb,2.3617e-01_rb,4.1160e-01_rb/)
      kao(:, 1, 5, 2) = (/ &
     &2.5332e-02_rb,1.2963e-01_rb,1.6655e-01_rb,1.9042e-01_rb,2.0895e-01_rb,1.9510e-01_rb, &
     &1.6916e-01_rb,1.6222e-01_rb,3.6316e-01_rb/)
      kao(:, 2, 5, 2) = (/ &
     &2.5181e-02_rb,1.2537e-01_rb,1.6592e-01_rb,1.8998e-01_rb,2.0621e-01_rb,1.9136e-01_rb, &
     &1.7764e-01_rb,1.7625e-01_rb,3.5695e-01_rb/)
      kao(:, 3, 5, 2) = (/ &
     &2.4972e-02_rb,1.2219e-01_rb,1.6618e-01_rb,1.9129e-01_rb,2.0217e-01_rb,1.9130e-01_rb, &
     &1.8535e-01_rb,1.8719e-01_rb,3.4788e-01_rb/)
      kao(:, 4, 5, 2) = (/ &
     &2.4749e-02_rb,1.2096e-01_rb,1.6848e-01_rb,1.9087e-01_rb,1.9911e-01_rb,1.9130e-01_rb, &
     &1.9341e-01_rb,1.9372e-01_rb,3.4412e-01_rb/)
      kao(:, 5, 5, 2) = (/ &
     &2.4660e-02_rb,1.2126e-01_rb,1.7164e-01_rb,1.8946e-01_rb,1.9462e-01_rb,1.9216e-01_rb, &
     &1.9926e-01_rb,1.9935e-01_rb,3.3902e-01_rb/)
      kao(:, 1, 6, 2) = (/ &
     &2.3422e-02_rb,1.0907e-01_rb,1.4314e-01_rb,1.6055e-01_rb,1.7919e-01_rb,1.6497e-01_rb, &
     &1.4423e-01_rb,1.3518e-01_rb,3.0537e-01_rb/)
      kao(:, 2, 6, 2) = (/ &
     &2.3193e-02_rb,1.0714e-01_rb,1.4280e-01_rb,1.6151e-01_rb,1.7408e-01_rb,1.6236e-01_rb, &
     &1.4991e-01_rb,1.4676e-01_rb,2.9795e-01_rb/)
      kao(:, 3, 6, 2) = (/ &
     &2.2992e-02_rb,1.0582e-01_rb,1.4541e-01_rb,1.6228e-01_rb,1.7059e-01_rb,1.6204e-01_rb, &
     &1.5617e-01_rb,1.5704e-01_rb,2.9013e-01_rb/)
      kao(:, 4, 6, 2) = (/ &
     &2.2904e-02_rb,1.0595e-01_rb,1.4661e-01_rb,1.6015e-01_rb,1.6718e-01_rb,1.6268e-01_rb, &
     &1.6254e-01_rb,1.6133e-01_rb,2.8558e-01_rb/)
      kao(:, 5, 6, 2) = (/ &
     &2.3037e-02_rb,1.0659e-01_rb,1.4720e-01_rb,1.5884e-01_rb,1.6394e-01_rb,1.6537e-01_rb, &
     &1.6790e-01_rb,1.6640e-01_rb,2.8291e-01_rb/)
      kao(:, 1, 7, 2) = (/ &
     &2.1778e-02_rb,9.4871e-02_rb,1.2415e-01_rb,1.3737e-01_rb,1.5258e-01_rb,1.4160e-01_rb, &
     &1.2347e-01_rb,1.1302e-01_rb,2.5408e-01_rb/)
      kao(:, 2, 7, 2) = (/ &
     &2.1522e-02_rb,9.3423e-02_rb,1.2417e-01_rb,1.3839e-01_rb,1.4741e-01_rb,1.4006e-01_rb, &
     &1.2799e-01_rb,1.2268e-01_rb,2.4943e-01_rb/)
      kao(:, 3, 7, 2) = (/ &
     &2.1257e-02_rb,9.3282e-02_rb,1.2533e-01_rb,1.3843e-01_rb,1.4474e-01_rb,1.3911e-01_rb, &
     &1.3239e-01_rb,1.3095e-01_rb,2.4208e-01_rb/)
      kao(:, 4, 7, 2) = (/ &
     &2.0841e-02_rb,9.3161e-02_rb,1.2672e-01_rb,1.3698e-01_rb,1.4106e-01_rb,1.4019e-01_rb, &
     &1.3855e-01_rb,1.3550e-01_rb,2.3874e-01_rb/)
      kao(:, 5, 7, 2) = (/ &
     &2.0486e-02_rb,9.4079e-02_rb,1.2713e-01_rb,1.3508e-01_rb,1.3852e-01_rb,1.4171e-01_rb, &
     &1.4291e-01_rb,1.3980e-01_rb,2.3641e-01_rb/)
      kao(:, 1, 8, 2) = (/ &
     &2.0219e-02_rb,8.3245e-02_rb,1.0821e-01_rb,1.1854e-01_rb,1.3123e-01_rb,1.2323e-01_rb, &
     &1.0617e-01_rb,9.4794e-02_rb,2.1425e-01_rb/)
      kao(:, 2, 8, 2) = (/ &
     &1.9540e-02_rb,8.2217e-02_rb,1.0811e-01_rb,1.1985e-01_rb,1.2621e-01_rb,1.2208e-01_rb, &
     &1.0997e-01_rb,1.0316e-01_rb,2.0956e-01_rb/)
      kao(:, 3, 8, 2) = (/ &
     &1.9115e-02_rb,8.1811e-02_rb,1.0961e-01_rb,1.1959e-01_rb,1.2388e-01_rb,1.2129e-01_rb, &
     &1.1337e-01_rb,1.1053e-01_rb,2.0624e-01_rb/)
      kao(:, 4, 8, 2) = (/ &
     &1.8733e-02_rb,8.1851e-02_rb,1.1044e-01_rb,1.1828e-01_rb,1.2090e-01_rb,1.2160e-01_rb, &
     &1.1850e-01_rb,1.1463e-01_rb,2.0160e-01_rb/)
      kao(:, 5, 8, 2) = (/ &
     &1.8489e-02_rb,8.3310e-02_rb,1.0940e-01_rb,1.1607e-01_rb,1.1923e-01_rb,1.2205e-01_rb, &
     &1.2223e-01_rb,1.1845e-01_rb,2.0014e-01_rb/)
      kao(:, 1, 9, 2) = (/ &
     &1.7934e-02_rb,7.2499e-02_rb,9.3347e-02_rb,1.0259e-01_rb,1.1263e-01_rb,1.0693e-01_rb, &
     &9.0284e-02_rb,7.9546e-02_rb,1.8180e-01_rb/)
      kao(:, 2, 9, 2) = (/ &
     &1.7445e-02_rb,7.1879e-02_rb,9.3312e-02_rb,1.0334e-01_rb,1.0867e-01_rb,1.0654e-01_rb, &
     &9.3871e-02_rb,8.7011e-02_rb,1.7650e-01_rb/)
      kao(:, 3, 9, 2) = (/ &
     &1.7054e-02_rb,7.1164e-02_rb,9.4558e-02_rb,1.0223e-01_rb,1.0632e-01_rb,1.0559e-01_rb, &
     &9.7124e-02_rb,9.3354e-02_rb,1.7444e-01_rb/)
      kao(:, 4, 9, 2) = (/ &
     &1.6642e-02_rb,7.2006e-02_rb,9.4764e-02_rb,1.0087e-01_rb,1.0340e-01_rb,1.0507e-01_rb, &
     &1.0155e-01_rb,9.7191e-02_rb,1.7012e-01_rb/)
      kao(:, 5, 9, 2) = (/ &
     &1.6243e-02_rb,7.2866e-02_rb,9.2285e-02_rb,9.9097e-02_rb,1.0185e-01_rb,1.0516e-01_rb, &
     &1.0490e-01_rb,1.0097e-01_rb,1.7041e-01_rb/)
      kao(:, 1,10, 2) = (/ &
     &1.5979e-02_rb,6.3035e-02_rb,8.0318e-02_rb,8.8320e-02_rb,9.6214e-02_rb,9.2269e-02_rb, &
     &7.7109e-02_rb,6.7263e-02_rb,1.5451e-01_rb/)
      kao(:, 2,10, 2) = (/ &
     &1.5470e-02_rb,6.2313e-02_rb,8.0559e-02_rb,8.8699e-02_rb,9.3120e-02_rb,9.2382e-02_rb, &
     &7.9788e-02_rb,7.3810e-02_rb,1.5001e-01_rb/)
      kao(:, 3,10, 2) = (/ &
     &1.4968e-02_rb,6.2321e-02_rb,8.1515e-02_rb,8.7265e-02_rb,9.0898e-02_rb,9.1273e-02_rb, &
     &8.3098e-02_rb,7.9462e-02_rb,1.4837e-01_rb/)
      kao(:, 4,10, 2) = (/ &
     &1.4648e-02_rb,6.3201e-02_rb,8.0459e-02_rb,8.6353e-02_rb,8.8769e-02_rb,9.0582e-02_rb, &
     &8.6861e-02_rb,8.2886e-02_rb,1.4494e-01_rb/)
      kao(:, 5,10, 2) = (/ &
     &1.4327e-02_rb,6.4058e-02_rb,7.8092e-02_rb,8.4556e-02_rb,8.7537e-02_rb,9.0662e-02_rb, &
     &8.9892e-02_rb,8.5887e-02_rb,1.4505e-01_rb/)
      kao(:, 1,11, 2) = (/ &
     &1.3861e-02_rb,5.4196e-02_rb,6.8747e-02_rb,7.5984e-02_rb,8.0767e-02_rb,7.9801e-02_rb, &
     &6.6830e-02_rb,5.9302e-02_rb,1.2963e-01_rb/)
      kao(:, 2,11, 2) = (/ &
     &1.3347e-02_rb,5.4113e-02_rb,6.9722e-02_rb,7.5263e-02_rb,7.8466e-02_rb,7.8575e-02_rb, &
     &6.9140e-02_rb,6.4556e-02_rb,1.2675e-01_rb/)
      kao(:, 3,11, 2) = (/ &
     &1.3016e-02_rb,5.4555e-02_rb,6.9179e-02_rb,7.4613e-02_rb,7.7291e-02_rb,7.7531e-02_rb, &
     &7.2200e-02_rb,6.9022e-02_rb,1.2482e-01_rb/)
      kao(:, 4,11, 2) = (/ &
     &1.2671e-02_rb,5.5408e-02_rb,6.7230e-02_rb,7.3198e-02_rb,7.5526e-02_rb,7.7122e-02_rb, &
     &7.5177e-02_rb,7.1296e-02_rb,1.2336e-01_rb/)
      kao(:, 5,11, 2) = (/ &
     &1.2351e-02_rb,5.6031e-02_rb,6.5093e-02_rb,7.1581e-02_rb,7.4883e-02_rb,7.6822e-02_rb, &
     &7.7284e-02_rb,7.3771e-02_rb,1.2246e-01_rb/)
      kao(:, 1,12, 2) = (/ &
     &1.2126e-02_rb,4.7285e-02_rb,5.9933e-02_rb,6.5129e-02_rb,6.8290e-02_rb,6.7884e-02_rb, &
     &5.7878e-02_rb,5.1782e-02_rb,1.0839e-01_rb/)
      kao(:, 2,12, 2) = (/ &
     &1.1745e-02_rb,4.7398e-02_rb,5.9988e-02_rb,6.4477e-02_rb,6.6821e-02_rb,6.6892e-02_rb, &
     &6.0270e-02_rb,5.6137e-02_rb,1.0621e-01_rb/)
      kao(:, 3,12, 2) = (/ &
     &1.1385e-02_rb,4.8238e-02_rb,5.8218e-02_rb,6.3614e-02_rb,6.5819e-02_rb,6.5904e-02_rb, &
     &6.2567e-02_rb,5.9036e-02_rb,1.0557e-01_rb/)
      kao(:, 4,12, 2) = (/ &
     &1.1087e-02_rb,4.8582e-02_rb,5.6782e-02_rb,6.2244e-02_rb,6.4582e-02_rb,6.5133e-02_rb, &
     &6.4669e-02_rb,6.1446e-02_rb,1.0435e-01_rb/)
      kao(:, 5,12, 2) = (/ &
     &1.0909e-02_rb,4.7898e-02_rb,5.5031e-02_rb,6.0882e-02_rb,6.4057e-02_rb,6.5202e-02_rb, &
     &6.6350e-02_rb,6.3114e-02_rb,1.0415e-01_rb/)
      kao(:, 1,13, 2) = (/ &
     &1.1050e-02_rb,4.2216e-02_rb,5.3216e-02_rb,5.6607e-02_rb,5.8614e-02_rb,5.8762e-02_rb, &
     &5.1111e-02_rb,4.5278e-02_rb,9.1112e-02_rb/)
      kao(:, 2,13, 2) = (/ &
     &1.0650e-02_rb,4.2640e-02_rb,5.2374e-02_rb,5.5676e-02_rb,5.7428e-02_rb,5.7903e-02_rb, &
     &5.3022e-02_rb,4.8756e-02_rb,9.0299e-02_rb/)
      kao(:, 3,13, 2) = (/ &
     &1.0375e-02_rb,4.2997e-02_rb,5.0849e-02_rb,5.4963e-02_rb,5.6516e-02_rb,5.6613e-02_rb, &
     &5.4398e-02_rb,5.1351e-02_rb,8.9757e-02_rb/)
      kao(:, 4,13, 2) = (/ &
     &1.0186e-02_rb,4.2550e-02_rb,4.9650e-02_rb,5.4126e-02_rb,5.6087e-02_rb,5.6100e-02_rb, &
     &5.5740e-02_rb,5.3111e-02_rb,8.9898e-02_rb/)
      kao(:, 5,13, 2) = (/ &
     &9.9803e-03_rb,4.1596e-02_rb,4.8435e-02_rb,5.3214e-02_rb,5.5733e-02_rb,5.6166e-02_rb, &
     &5.6820e-02_rb,5.4773e-02_rb,8.9718e-02_rb/)
      kao(:, 1, 1, 3) = (/ &
     &1.0349e-01_rb,2.2333e-01_rb,4.3448e-01_rb,4.7188e-01_rb,4.9049e-01_rb,5.6379e-01_rb, &
     &6.8188e-01_rb,5.5213e-01_rb,8.7331e-01_rb/)
      kao(:, 2, 1, 3) = (/ &
     &9.8776e-02_rb,2.3587e-01_rb,4.2090e-01_rb,4.7099e-01_rb,4.8532e-01_rb,5.6756e-01_rb, &
     &6.8304e-01_rb,5.3972e-01_rb,8.6585e-01_rb/)
      kao(:, 3, 1, 3) = (/ &
     &9.4379e-02_rb,2.4459e-01_rb,4.0703e-01_rb,4.4771e-01_rb,4.8054e-01_rb,5.8051e-01_rb, &
     &6.8126e-01_rb,5.2715e-01_rb,8.6189e-01_rb/)
      kao(:, 4, 1, 3) = (/ &
     &8.9966e-02_rb,2.4911e-01_rb,4.0891e-01_rb,4.3132e-01_rb,4.8073e-01_rb,5.8214e-01_rb, &
     &6.7058e-01_rb,5.2176e-01_rb,8.6463e-01_rb/)
      kao(:, 5, 1, 3) = (/ &
     &8.6080e-02_rb,2.4561e-01_rb,4.0823e-01_rb,4.3495e-01_rb,4.8341e-01_rb,5.8583e-01_rb, &
     &6.6494e-01_rb,5.1295e-01_rb,8.6781e-01_rb/)
      kao(:, 1, 2, 3) = (/ &
     &9.1121e-02_rb,2.7597e-01_rb,3.6090e-01_rb,3.8864e-01_rb,4.2468e-01_rb,5.0778e-01_rb, &
     &6.0614e-01_rb,4.7263e-01_rb,7.5147e-01_rb/)
      kao(:, 2, 2, 3) = (/ &
     &8.6968e-02_rb,2.8112e-01_rb,3.5384e-01_rb,3.7604e-01_rb,4.2522e-01_rb,5.1397e-01_rb, &
     &5.9893e-01_rb,4.6387e-01_rb,7.5402e-01_rb/)
      kao(:, 3, 2, 3) = (/ &
     &8.3177e-02_rb,2.8458e-01_rb,3.4657e-01_rb,3.6832e-01_rb,4.2349e-01_rb,5.1886e-01_rb, &
     &5.9537e-01_rb,4.5576e-01_rb,7.6481e-01_rb/)
      kao(:, 4, 2, 3) = (/ &
     &7.9593e-02_rb,2.8362e-01_rb,3.4405e-01_rb,3.6602e-01_rb,4.2737e-01_rb,5.2263e-01_rb, &
     &5.9169e-01_rb,4.4715e-01_rb,7.7154e-01_rb/)
      kao(:, 5, 2, 3) = (/ &
     &7.6305e-02_rb,2.7901e-01_rb,3.3923e-01_rb,3.6563e-01_rb,4.2905e-01_rb,5.2631e-01_rb, &
     &5.9188e-01_rb,4.4184e-01_rb,7.8075e-01_rb/)
      kao(:, 1, 3, 3) = (/ &
     &8.1532e-02_rb,2.6557e-01_rb,3.0691e-01_rb,3.2639e-01_rb,3.6998e-01_rb,4.4624e-01_rb, &
     &5.2995e-01_rb,4.1015e-01_rb,6.5448e-01_rb/)
      kao(:, 2, 3, 3) = (/ &
     &7.8061e-02_rb,2.5705e-01_rb,3.0144e-01_rb,3.1922e-01_rb,3.7157e-01_rb,4.5007e-01_rb, &
     &5.2671e-01_rb,4.0238e-01_rb,6.6049e-01_rb/)
      kao(:, 3, 3, 3) = (/ &
     &7.4871e-02_rb,2.4964e-01_rb,2.9703e-01_rb,3.1658e-01_rb,3.7272e-01_rb,4.5628e-01_rb, &
     &5.2424e-01_rb,3.9540e-01_rb,6.6589e-01_rb/)
      kao(:, 4, 3, 3) = (/ &
     &7.1934e-02_rb,2.4252e-01_rb,2.8855e-01_rb,3.1667e-01_rb,3.7723e-01_rb,4.6275e-01_rb, &
     &5.1485e-01_rb,3.8931e-01_rb,6.7492e-01_rb/)
      kao(:, 5, 3, 3) = (/ &
     &6.9067e-02_rb,2.3451e-01_rb,2.8461e-01_rb,3.1625e-01_rb,3.8425e-01_rb,4.6414e-01_rb, &
     &5.0872e-01_rb,3.8671e-01_rb,6.9966e-01_rb/)
      kao(:, 1, 4, 3) = (/ &
     &7.4166e-02_rb,2.1898e-01_rb,2.6310e-01_rb,2.7673e-01_rb,3.2117e-01_rb,3.8906e-01_rb, &
     &4.5677e-01_rb,3.5535e-01_rb,5.6605e-01_rb/)
      kao(:, 2, 4, 3) = (/ &
     &7.1027e-02_rb,2.1158e-01_rb,2.5634e-01_rb,2.7577e-01_rb,3.2463e-01_rb,3.9708e-01_rb, &
     &4.5460e-01_rb,3.5060e-01_rb,5.7431e-01_rb/)
      kao(:, 3, 4, 3) = (/ &
     &6.8100e-02_rb,2.0832e-01_rb,2.5127e-01_rb,2.7640e-01_rb,3.2722e-01_rb,4.0091e-01_rb, &
     &4.5112e-01_rb,3.4506e-01_rb,5.8564e-01_rb/)
      kao(:, 4, 4, 3) = (/ &
     &6.5366e-02_rb,2.0407e-01_rb,2.4599e-01_rb,2.7530e-01_rb,3.3554e-01_rb,4.0555e-01_rb, &
     &4.3614e-01_rb,3.4106e-01_rb,6.0722e-01_rb/)
      kao(:, 5, 4, 3) = (/ &
     &6.2828e-02_rb,2.0183e-01_rb,2.4016e-01_rb,2.8032e-01_rb,3.4415e-01_rb,4.1256e-01_rb, &
     &4.2811e-01_rb,3.3888e-01_rb,6.2274e-01_rb/)
      kao(:, 1, 5, 3) = (/ &
     &6.8085e-02_rb,1.8443e-01_rb,2.2193e-01_rb,2.4125e-01_rb,2.7858e-01_rb,3.3878e-01_rb, &
     &3.9224e-01_rb,3.0775e-01_rb,4.8825e-01_rb/)
      kao(:, 2, 5, 3) = (/ &
     &6.5024e-02_rb,1.8062e-01_rb,2.1594e-01_rb,2.3959e-01_rb,2.8255e-01_rb,3.4709e-01_rb, &
     &3.8693e-01_rb,3.0387e-01_rb,4.9773e-01_rb/)
      kao(:, 3, 5, 3) = (/ &
     &6.2309e-02_rb,1.7709e-01_rb,2.1233e-01_rb,2.4099e-01_rb,2.9052e-01_rb,3.5084e-01_rb, &
     &3.7550e-01_rb,2.9937e-01_rb,5.1983e-01_rb/)
      kao(:, 4, 5, 3) = (/ &
     &6.0049e-02_rb,1.7340e-01_rb,2.0891e-01_rb,2.4567e-01_rb,2.9963e-01_rb,3.5573e-01_rb, &
     &3.6423e-01_rb,2.9642e-01_rb,5.3792e-01_rb/)
      kao(:, 5, 5, 3) = (/ &
     &5.7860e-02_rb,1.7101e-01_rb,2.0517e-01_rb,2.5155e-01_rb,3.0986e-01_rb,3.5542e-01_rb, &
     &3.5482e-01_rb,2.9214e-01_rb,5.5424e-01_rb/)
      kao(:, 1, 6, 3) = (/ &
     &6.1553e-02_rb,1.6486e-01_rb,1.8851e-01_rb,2.1013e-01_rb,2.4069e-01_rb,2.9423e-01_rb, &
     &3.2998e-01_rb,2.6499e-01_rb,4.1520e-01_rb/)
      kao(:, 2, 6, 3) = (/ &
     &5.8962e-02_rb,1.6161e-01_rb,1.8469e-01_rb,2.1043e-01_rb,2.4572e-01_rb,3.0023e-01_rb, &
     &3.2384e-01_rb,2.6279e-01_rb,4.2954e-01_rb/)
      kao(:, 3, 6, 3) = (/ &
     &5.6865e-02_rb,1.5845e-01_rb,1.8137e-01_rb,2.1320e-01_rb,2.5567e-01_rb,3.0422e-01_rb, &
     &3.1399e-01_rb,2.5823e-01_rb,4.4811e-01_rb/)
      kao(:, 4, 6, 3) = (/ &
     &5.4727e-02_rb,1.5470e-01_rb,1.8033e-01_rb,2.2018e-01_rb,2.6466e-01_rb,3.0350e-01_rb, &
     &3.0450e-01_rb,2.5479e-01_rb,4.6359e-01_rb/)
      kao(:, 5, 6, 3) = (/ &
     &5.2654e-02_rb,1.5114e-01_rb,1.8009e-01_rb,2.2727e-01_rb,2.7074e-01_rb,3.0288e-01_rb, &
     &2.9697e-01_rb,2.5146e-01_rb,4.7630e-01_rb/)
      kao(:, 1, 7, 3) = (/ &
     &5.4386e-02_rb,1.4374e-01_rb,1.6223e-01_rb,1.8219e-01_rb,2.0984e-01_rb,2.5288e-01_rb, &
     &2.7777e-01_rb,2.2929e-01_rb,3.5662e-01_rb/)
      kao(:, 2, 7, 3) = (/ &
     &5.2357e-02_rb,1.4042e-01_rb,1.5995e-01_rb,1.8335e-01_rb,2.1810e-01_rb,2.5710e-01_rb, &
     &2.7180e-01_rb,2.2723e-01_rb,3.7033e-01_rb/)
      kao(:, 3, 7, 3) = (/ &
     &5.0433e-02_rb,1.3664e-01_rb,1.5916e-01_rb,1.8872e-01_rb,2.2495e-01_rb,2.5913e-01_rb, &
     &2.6518e-01_rb,2.2296e-01_rb,3.8796e-01_rb/)
      kao(:, 4, 7, 3) = (/ &
     &4.8970e-02_rb,1.3435e-01_rb,1.5882e-01_rb,1.9514e-01_rb,2.3146e-01_rb,2.5861e-01_rb, &
     &2.5643e-01_rb,2.1963e-01_rb,4.0045e-01_rb/)
      kao(:, 5, 7, 3) = (/ &
     &4.7677e-02_rb,1.3141e-01_rb,1.5996e-01_rb,2.0198e-01_rb,2.3377e-01_rb,2.5698e-01_rb, &
     &2.5063e-01_rb,2.1548e-01_rb,4.0471e-01_rb/)
      kao(:, 1, 8, 3) = (/ &
     &4.9561e-02_rb,1.2585e-01_rb,1.4205e-01_rb,1.6041e-01_rb,1.8455e-01_rb,2.1813e-01_rb, &
     &2.3719e-01_rb,1.9955e-01_rb,3.0456e-01_rb/)
      kao(:, 2, 8, 3) = (/ &
     &4.7879e-02_rb,1.2232e-01_rb,1.4089e-01_rb,1.6222e-01_rb,1.9194e-01_rb,2.2092e-01_rb, &
     &2.3163e-01_rb,1.9730e-01_rb,3.1885e-01_rb/)
      kao(:, 3, 8, 3) = (/ &
     &4.6426e-02_rb,1.1957e-01_rb,1.4043e-01_rb,1.6717e-01_rb,1.9752e-01_rb,2.2121e-01_rb, &
     &2.2638e-01_rb,1.9261e-01_rb,3.3187e-01_rb/)
      kao(:, 4, 8, 3) = (/ &
     &4.5268e-02_rb,1.1738e-01_rb,1.4149e-01_rb,1.7294e-01_rb,2.0208e-01_rb,2.2061e-01_rb, &
     &2.1864e-01_rb,1.8926e-01_rb,3.4074e-01_rb/)
      kao(:, 5, 8, 3) = (/ &
     &4.3956e-02_rb,1.1566e-01_rb,1.4446e-01_rb,1.7951e-01_rb,2.0361e-01_rb,2.1938e-01_rb, &
     &2.1462e-01_rb,1.8542e-01_rb,3.4583e-01_rb/)
      kao(:, 1, 9, 3) = (/ &
     &4.5250e-02_rb,1.0911e-01_rb,1.2308e-01_rb,1.4028e-01_rb,1.6172e-01_rb,1.8816e-01_rb, &
     &2.0368e-01_rb,1.7299e-01_rb,2.5859e-01_rb/)
      kao(:, 2, 9, 3) = (/ &
     &4.3924e-02_rb,1.0618e-01_rb,1.2262e-01_rb,1.4258e-01_rb,1.6714e-01_rb,1.8918e-01_rb, &
     &1.9807e-01_rb,1.7073e-01_rb,2.7235e-01_rb/)
      kao(:, 3, 9, 3) = (/ &
     &4.2608e-02_rb,1.0467e-01_rb,1.2300e-01_rb,1.4745e-01_rb,1.7253e-01_rb,1.8886e-01_rb, &
     &1.9316e-01_rb,1.6589e-01_rb,2.8262e-01_rb/)
      kao(:, 4, 9, 3) = (/ &
     &4.1423e-02_rb,1.0229e-01_rb,1.2419e-01_rb,1.5295e-01_rb,1.7546e-01_rb,1.8853e-01_rb, &
     &1.8673e-01_rb,1.6267e-01_rb,2.8816e-01_rb/)
      kao(:, 5, 9, 3) = (/ &
     &4.0021e-02_rb,1.0144e-01_rb,1.2853e-01_rb,1.5736e-01_rb,1.7632e-01_rb,1.8748e-01_rb, &
     &1.8330e-01_rb,1.5887e-01_rb,2.9011e-01_rb/)
      kao(:, 1,10, 3) = (/ &
     &4.1171e-02_rb,9.3658e-02_rb,1.0622e-01_rb,1.2188e-01_rb,1.4076e-01_rb,1.6261e-01_rb, &
     &1.7423e-01_rb,1.4922e-01_rb,2.1862e-01_rb/)
      kao(:, 2,10, 3) = (/ &
     &3.9662e-02_rb,9.1742e-02_rb,1.0641e-01_rb,1.2434e-01_rb,1.4583e-01_rb,1.6228e-01_rb, &
     &1.6976e-01_rb,1.4678e-01_rb,2.3073e-01_rb/)
      kao(:, 3,10, 3) = (/ &
     &3.8151e-02_rb,8.9581e-02_rb,1.0765e-01_rb,1.2937e-01_rb,1.4971e-01_rb,1.6177e-01_rb, &
     &1.6479e-01_rb,1.4256e-01_rb,2.3771e-01_rb/)
      kao(:, 4,10, 3) = (/ &
     &3.6829e-02_rb,8.8190e-02_rb,1.0980e-01_rb,1.3385e-01_rb,1.5088e-01_rb,1.6121e-01_rb, &
     &1.5970e-01_rb,1.3919e-01_rb,2.4389e-01_rb/)
      kao(:, 5,10, 3) = (/ &
     &3.5799e-02_rb,8.7081e-02_rb,1.1397e-01_rb,1.3613e-01_rb,1.5068e-01_rb,1.6032e-01_rb, &
     &1.5668e-01_rb,1.3612e-01_rb,2.4347e-01_rb/)
      kao(:, 1,11, 3) = (/ &
     &3.5950e-02_rb,7.9375e-02_rb,9.2025e-02_rb,1.0686e-01_rb,1.2416e-01_rb,1.3928e-01_rb, &
     &1.4761e-01_rb,1.2730e-01_rb,1.9139e-01_rb/)
      kao(:, 2,11, 3) = (/ &
     &3.4807e-02_rb,7.7447e-02_rb,9.2655e-02_rb,1.1006e-01_rb,1.2736e-01_rb,1.3892e-01_rb, &
     &1.4381e-01_rb,1.2426e-01_rb,1.9723e-01_rb/)
      kao(:, 3,11, 3) = (/ &
     &3.3668e-02_rb,7.6215e-02_rb,9.4748e-02_rb,1.1430e-01_rb,1.2842e-01_rb,1.3852e-01_rb, &
     &1.3892e-01_rb,1.2007e-01_rb,2.0316e-01_rb/)
      kao(:, 4,11, 3) = (/ &
     &3.2675e-02_rb,7.5405e-02_rb,9.7335e-02_rb,1.1502e-01_rb,1.2820e-01_rb,1.3796e-01_rb, &
     &1.3569e-01_rb,1.1793e-01_rb,2.0391e-01_rb/)
      kao(:, 5,11, 3) = (/ &
     &3.1803e-02_rb,7.5385e-02_rb,1.0078e-01_rb,1.1505e-01_rb,1.2792e-01_rb,1.3782e-01_rb, &
     &1.3291e-01_rb,1.1573e-01_rb,2.0592e-01_rb/)
      kao(:, 1,12, 3) = (/ &
     &3.2107e-02_rb,6.7817e-02_rb,8.0190e-02_rb,9.4418e-02_rb,1.0883e-01_rb,1.2016e-01_rb, &
     &1.2478e-01_rb,1.0916e-01_rb,1.6437e-01_rb/)
      kao(:, 2,12, 3) = (/ &
     &3.0865e-02_rb,6.6495e-02_rb,8.1339e-02_rb,9.7570e-02_rb,1.0958e-01_rb,1.1971e-01_rb, &
     &1.2159e-01_rb,1.0575e-01_rb,1.6979e-01_rb/)
      kao(:, 3,12, 3) = (/ &
     &2.9814e-02_rb,6.5982e-02_rb,8.4322e-02_rb,9.8160e-02_rb,1.0983e-01_rb,1.1899e-01_rb, &
     &1.1775e-01_rb,1.0260e-01_rb,1.7251e-01_rb/)
      kao(:, 4,12, 3) = (/ &
     &2.8719e-02_rb,6.6210e-02_rb,8.5962e-02_rb,9.8372e-02_rb,1.0963e-01_rb,1.1883e-01_rb, &
     &1.1590e-01_rb,1.0037e-01_rb,1.7377e-01_rb/)
      kao(:, 5,12, 3) = (/ &
     &2.7641e-02_rb,6.7001e-02_rb,8.6676e-02_rb,9.9268e-02_rb,1.1059e-01_rb,1.1880e-01_rb, &
     &1.1301e-01_rb,9.8900e-02_rb,1.7602e-01_rb/)
      kao(:, 1,13, 3) = (/ &
     &2.9283e-02_rb,6.0178e-02_rb,7.1238e-02_rb,8.4877e-02_rb,9.5368e-02_rb,1.0416e-01_rb, &
     &1.0619e-01_rb,9.4038e-02_rb,1.4273e-01_rb/)
      kao(:, 2,13, 3) = (/ &
     &2.8137e-02_rb,5.9367e-02_rb,7.3469e-02_rb,8.5617e-02_rb,9.5728e-02_rb,1.0364e-01_rb, &
     &1.0343e-01_rb,9.0972e-02_rb,1.4741e-01_rb/)
      kao(:, 3,13, 3) = (/ &
     &2.7056e-02_rb,5.9346e-02_rb,7.4992e-02_rb,8.5630e-02_rb,9.5820e-02_rb,1.0325e-01_rb, &
     &1.0105e-01_rb,8.7823e-02_rb,1.4877e-01_rb/)
      kao(:, 4,13, 3) = (/ &
     &2.6017e-02_rb,6.0117e-02_rb,7.5313e-02_rb,8.5867e-02_rb,9.6184e-02_rb,1.0321e-01_rb, &
     &9.9567e-02_rb,8.6122e-02_rb,1.5028e-01_rb/)
      kao(:, 5,13, 3) = (/ &
     &2.5188e-02_rb,6.1586e-02_rb,7.5405e-02_rb,8.7449e-02_rb,9.7275e-02_rb,1.0347e-01_rb, &
     &9.7619e-02_rb,8.4749e-02_rb,1.5271e-01_rb/)
      kao(:, 1, 1, 4) = (/ &
     &2.0140e-01_rb,2.2296e-01_rb,3.8441e-01_rb,7.9237e-01_rb,8.6626e-01_rb,8.2263e-01_rb, &
     &7.4305e-01_rb,7.5768e-01_rb,1.5145e+00_rb/)
      kao(:, 2, 1, 4) = (/ &
     &1.9181e-01_rb,2.1723e-01_rb,4.1174e-01_rb,7.7988e-01_rb,8.4188e-01_rb,7.8979e-01_rb, &
     &7.0362e-01_rb,7.1757e-01_rb,1.4753e+00_rb/)
      kao(:, 3, 1, 4) = (/ &
     &1.8358e-01_rb,2.1296e-01_rb,4.3167e-01_rb,7.6375e-01_rb,8.2127e-01_rb,7.5942e-01_rb, &
     &6.7535e-01_rb,6.8460e-01_rb,1.4372e+00_rb/)
      kao(:, 4, 1, 4) = (/ &
     &1.7731e-01_rb,2.0956e-01_rb,4.3260e-01_rb,7.5378e-01_rb,8.0524e-01_rb,7.2961e-01_rb, &
     &6.5477e-01_rb,6.4648e-01_rb,1.3999e+00_rb/)
      kao(:, 5, 1, 4) = (/ &
     &1.7164e-01_rb,2.1055e-01_rb,4.2755e-01_rb,7.3334e-01_rb,7.8436e-01_rb,7.1000e-01_rb, &
     &6.4297e-01_rb,6.0884e-01_rb,1.3521e+00_rb/)
      kao(:, 1, 2, 4) = (/ &
     &1.7302e-01_rb,2.0244e-01_rb,5.3076e-01_rb,6.9942e-01_rb,7.3274e-01_rb,6.9098e-01_rb, &
     &6.5285e-01_rb,6.3638e-01_rb,1.2559e+00_rb/)
      kao(:, 2, 2, 4) = (/ &
     &1.6489e-01_rb,1.9797e-01_rb,5.4053e-01_rb,6.8204e-01_rb,7.0789e-01_rb,6.6109e-01_rb, &
     &6.3152e-01_rb,6.0382e-01_rb,1.2010e+00_rb/)
      kao(:, 3, 2, 4) = (/ &
     &1.5760e-01_rb,1.9570e-01_rb,5.4478e-01_rb,6.6415e-01_rb,6.8272e-01_rb,6.3617e-01_rb, &
     &6.0950e-01_rb,5.7193e-01_rb,1.1640e+00_rb/)
      kao(:, 4, 2, 4) = (/ &
     &1.5184e-01_rb,1.9435e-01_rb,5.4381e-01_rb,6.4780e-01_rb,6.6147e-01_rb,6.1692e-01_rb, &
     &5.8732e-01_rb,5.4197e-01_rb,1.1223e+00_rb/)
      kao(:, 5, 2, 4) = (/ &
     &1.4698e-01_rb,1.9520e-01_rb,5.4107e-01_rb,6.3896e-01_rb,6.4395e-01_rb,6.0782e-01_rb, &
     &5.7401e-01_rb,5.1623e-01_rb,1.0839e+00_rb/)
      kao(:, 1, 3, 4) = (/ &
     &1.5095e-01_rb,2.3961e-01_rb,5.3891e-01_rb,5.9274e-01_rb,6.0360e-01_rb,5.9975e-01_rb, &
     &5.7583e-01_rb,5.3639e-01_rb,1.0254e+00_rb/)
      kao(:, 2, 3, 4) = (/ &
     &1.4378e-01_rb,2.4579e-01_rb,5.2840e-01_rb,5.8072e-01_rb,5.8574e-01_rb,5.7325e-01_rb, &
     &5.5457e-01_rb,5.1150e-01_rb,9.9368e-01_rb/)
      kao(:, 3, 3, 4) = (/ &
     &1.3744e-01_rb,2.5218e-01_rb,5.1071e-01_rb,5.6679e-01_rb,5.6782e-01_rb,5.5647e-01_rb, &
     &5.3781e-01_rb,4.7984e-01_rb,9.5622e-01_rb/)
      kao(:, 4, 3, 4) = (/ &
     &1.3182e-01_rb,2.5642e-01_rb,4.9891e-01_rb,5.5373e-01_rb,5.5212e-01_rb,5.4201e-01_rb, &
     &5.2720e-01_rb,4.5497e-01_rb,9.3692e-01_rb/)
      kao(:, 5, 3, 4) = (/ &
     &1.2736e-01_rb,2.6156e-01_rb,4.9042e-01_rb,5.4609e-01_rb,5.4094e-01_rb,5.2788e-01_rb, &
     &5.2063e-01_rb,4.2896e-01_rb,9.1393e-01_rb/)
      kao(:, 1, 4, 4) = (/ &
     &1.3296e-01_rb,3.0808e-01_rb,4.5560e-01_rb,5.0161e-01_rb,5.1973e-01_rb,5.1660e-01_rb, &
     &5.1162e-01_rb,4.5889e-01_rb,8.7778e-01_rb/)
      kao(:, 2, 4, 4) = (/ &
     &1.2659e-01_rb,3.1156e-01_rb,4.4164e-01_rb,4.8779e-01_rb,5.0041e-01_rb,4.9812e-01_rb, &
     &4.9015e-01_rb,4.3243e-01_rb,8.4542e-01_rb/)
      kao(:, 3, 4, 4) = (/ &
     &1.2094e-01_rb,3.1188e-01_rb,4.3105e-01_rb,4.7578e-01_rb,4.8617e-01_rb,4.8333e-01_rb, &
     &4.7842e-01_rb,4.0299e-01_rb,8.1678e-01_rb/)
      kao(:, 4, 4, 4) = (/ &
     &1.1593e-01_rb,3.1351e-01_rb,4.2237e-01_rb,4.6451e-01_rb,4.7350e-01_rb,4.7079e-01_rb, &
     &4.7610e-01_rb,3.7858e-01_rb,7.9667e-01_rb/)
      kao(:, 5, 4, 4) = (/ &
     &1.1150e-01_rb,3.1306e-01_rb,4.1390e-01_rb,4.5362e-01_rb,4.6429e-01_rb,4.6167e-01_rb, &
     &4.7081e-01_rb,3.5772e-01_rb,7.7552e-01_rb/)
      kao(:, 1, 5, 4) = (/ &
     &1.1755e-01_rb,3.2807e-01_rb,3.8548e-01_rb,4.2521e-01_rb,4.4554e-01_rb,4.5203e-01_rb, &
     &4.4500e-01_rb,3.9225e-01_rb,7.4478e-01_rb/)
      kao(:, 2, 5, 4) = (/ &
     &1.1209e-01_rb,3.1909e-01_rb,3.7705e-01_rb,4.1235e-01_rb,4.2996e-01_rb,4.3302e-01_rb, &
     &4.3279e-01_rb,3.6494e-01_rb,7.1780e-01_rb/)
      kao(:, 3, 5, 4) = (/ &
     &1.0716e-01_rb,3.1181e-01_rb,3.6570e-01_rb,3.9834e-01_rb,4.1575e-01_rb,4.2162e-01_rb, &
     &4.2981e-01_rb,3.4051e-01_rb,6.9974e-01_rb/)
      kao(:, 4, 5, 4) = (/ &
     &1.0275e-01_rb,3.0301e-01_rb,3.5750e-01_rb,3.8990e-01_rb,4.0553e-01_rb,4.1539e-01_rb, &
     &4.2536e-01_rb,3.1843e-01_rb,6.8036e-01_rb/)
      kao(:, 5, 5, 4) = (/ &
     &9.8998e-02_rb,2.9446e-01_rb,3.5111e-01_rb,3.8417e-01_rb,3.9923e-01_rb,4.1474e-01_rb, &
     &4.2083e-01_rb,3.0243e-01_rb,6.6644e-01_rb/)
      kao(:, 1, 6, 4) = (/ &
     &1.0705e-01_rb,2.7318e-01_rb,3.3208e-01_rb,3.6143e-01_rb,3.8591e-01_rb,3.9313e-01_rb, &
     &3.9284e-01_rb,3.3741e-01_rb,6.4786e-01_rb/)
      kao(:, 2, 6, 4) = (/ &
     &1.0199e-01_rb,2.6545e-01_rb,3.2251e-01_rb,3.5040e-01_rb,3.7450e-01_rb,3.7890e-01_rb, &
     &3.8585e-01_rb,3.0993e-01_rb,6.2347e-01_rb/)
      kao(:, 3, 6, 4) = (/ &
     &9.7437e-02_rb,2.5576e-01_rb,3.1499e-01_rb,3.4107e-01_rb,3.6300e-01_rb,3.7247e-01_rb, &
     &3.8009e-01_rb,2.8815e-01_rb,6.0580e-01_rb/)
      kao(:, 4, 6, 4) = (/ &
     &9.3482e-02_rb,2.4922e-01_rb,3.0800e-01_rb,3.3313e-01_rb,3.5414e-01_rb,3.7096e-01_rb, &
     &3.7706e-01_rb,2.7157e-01_rb,5.9892e-01_rb/)
      kao(:, 5, 6, 4) = (/ &
     &9.0031e-02_rb,2.4481e-01_rb,3.0199e-01_rb,3.2745e-01_rb,3.5204e-01_rb,3.7018e-01_rb, &
     &3.6552e-01_rb,2.5507e-01_rb,5.9364e-01_rb/)
      kao(:, 1, 7, 4) = (/ &
     &1.0201e-01_rb,2.3487e-01_rb,2.8528e-01_rb,3.1378e-01_rb,3.3635e-01_rb,3.4598e-01_rb, &
     &3.4770e-01_rb,2.9069e-01_rb,5.6026e-01_rb/)
      kao(:, 2, 7, 4) = (/ &
     &9.7333e-02_rb,2.2730e-01_rb,2.7830e-01_rb,3.0507e-01_rb,3.2187e-01_rb,3.3481e-01_rb, &
     &3.4172e-01_rb,2.6797e-01_rb,5.4227e-01_rb/)
      kao(:, 3, 7, 4) = (/ &
     &9.3243e-02_rb,2.2136e-01_rb,2.6944e-01_rb,2.9618e-01_rb,3.1454e-01_rb,3.2973e-01_rb, &
     &3.3451e-01_rb,2.4821e-01_rb,5.2884e-01_rb/)
      kao(:, 4, 7, 4) = (/ &
     &8.9660e-02_rb,2.1638e-01_rb,2.6334e-01_rb,2.9056e-01_rb,3.1245e-01_rb,3.2791e-01_rb, &
     &3.2519e-01_rb,2.3192e-01_rb,5.2407e-01_rb/)
      kao(:, 5, 7, 4) = (/ &
     &8.6492e-02_rb,2.1156e-01_rb,2.5984e-01_rb,2.8528e-01_rb,3.1573e-01_rb,3.2789e-01_rb, &
     &3.1681e-01_rb,2.1806e-01_rb,5.2688e-01_rb/)
      kao(:, 1, 8, 4) = (/ &
     &1.0039e-01_rb,2.1049e-01_rb,2.5117e-01_rb,2.7733e-01_rb,2.9294e-01_rb,3.0451e-01_rb, &
     &3.0671e-01_rb,2.5418e-01_rb,4.9108e-01_rb/)
      kao(:, 2, 8, 4) = (/ &
     &9.6148e-02_rb,2.0539e-01_rb,2.4350e-01_rb,2.6931e-01_rb,2.8451e-01_rb,2.9582e-01_rb, &
     &3.0170e-01_rb,2.3605e-01_rb,4.7511e-01_rb/)
      kao(:, 3, 8, 4) = (/ &
     &9.2145e-02_rb,2.0009e-01_rb,2.3646e-01_rb,2.6177e-01_rb,2.7896e-01_rb,2.9276e-01_rb, &
     &2.9264e-01_rb,2.1892e-01_rb,4.6537e-01_rb/)
      kao(:, 4, 8, 4) = (/ &
     &8.8569e-02_rb,1.9568e-01_rb,2.3097e-01_rb,2.5688e-01_rb,2.7814e-01_rb,2.9214e-01_rb, &
     &2.8481e-01_rb,2.0263e-01_rb,4.6603e-01_rb/)
      kao(:, 5, 8, 4) = (/ &
     &8.5629e-02_rb,1.9094e-01_rb,2.2714e-01_rb,2.5259e-01_rb,2.7953e-01_rb,2.8904e-01_rb, &
     &2.7433e-01_rb,1.9066e-01_rb,4.6777e-01_rb/)
      kao(:, 1, 9, 4) = (/ &
     &9.8021e-02_rb,1.9151e-01_rb,2.2309e-01_rb,2.4564e-01_rb,2.5865e-01_rb,2.6783e-01_rb, &
     &2.6894e-01_rb,2.2292e-01_rb,4.2854e-01_rb/)
      kao(:, 2, 9, 4) = (/ &
     &9.3022e-02_rb,1.8527e-01_rb,2.1592e-01_rb,2.3651e-01_rb,2.5064e-01_rb,2.6055e-01_rb, &
     &2.6499e-01_rb,2.0617e-01_rb,4.1580e-01_rb/)
      kao(:, 3, 9, 4) = (/ &
     &8.8595e-02_rb,1.8097e-01_rb,2.1073e-01_rb,2.3156e-01_rb,2.4703e-01_rb,2.5851e-01_rb, &
     &2.5441e-01_rb,1.9146e-01_rb,4.1005e-01_rb/)
      kao(:, 4, 9, 4) = (/ &
     &8.4486e-02_rb,1.7670e-01_rb,2.0686e-01_rb,2.2734e-01_rb,2.4622e-01_rb,2.5730e-01_rb, &
     &2.4711e-01_rb,1.7700e-01_rb,4.1054e-01_rb/)
      kao(:, 5, 9, 4) = (/ &
     &8.1546e-02_rb,1.7284e-01_rb,2.0523e-01_rb,2.2674e-01_rb,2.4921e-01_rb,2.5266e-01_rb, &
     &2.3614e-01_rb,1.6619e-01_rb,4.1551e-01_rb/)
      kao(:, 1,10, 4) = (/ &
     &9.1049e-02_rb,1.7362e-01_rb,1.9833e-01_rb,2.1764e-01_rb,2.2798e-01_rb,2.3471e-01_rb, &
     &2.3646e-01_rb,1.9422e-01_rb,3.7343e-01_rb/)
      kao(:, 2,10, 4) = (/ &
     &8.6996e-02_rb,1.6791e-01_rb,1.9240e-01_rb,2.1075e-01_rb,2.2237e-01_rb,2.2936e-01_rb, &
     &2.3178e-01_rb,1.7985e-01_rb,3.6356e-01_rb/)
      kao(:, 3,10, 4) = (/ &
     &8.2800e-02_rb,1.6395e-01_rb,1.8777e-01_rb,2.0552e-01_rb,2.1940e-01_rb,2.2862e-01_rb, &
     &2.2169e-01_rb,1.6614e-01_rb,3.6046e-01_rb/)
      kao(:, 4,10, 4) = (/ &
     &7.8811e-02_rb,1.5975e-01_rb,1.8561e-01_rb,2.0375e-01_rb,2.1988e-01_rb,2.2450e-01_rb, &
     &2.1345e-01_rb,1.5492e-01_rb,3.6096e-01_rb/)
      kao(:, 5,10, 4) = (/ &
     &7.5529e-02_rb,1.5650e-01_rb,1.8353e-01_rb,2.0505e-01_rb,2.1950e-01_rb,2.1739e-01_rb, &
     &2.0454e-01_rb,1.4507e-01_rb,3.6055e-01_rb/)
      kao(:, 1,11, 4) = (/ &
     &7.7672e-02_rb,1.5145e-01_rb,1.7382e-01_rb,1.8990e-01_rb,1.9990e-01_rb,2.0432e-01_rb, &
     &2.0547e-01_rb,1.6386e-01_rb,3.1843e-01_rb/)
      kao(:, 2,11, 4) = (/ &
     &7.3723e-02_rb,1.4611e-01_rb,1.6807e-01_rb,1.8489e-01_rb,1.9610e-01_rb,2.0254e-01_rb, &
     &1.9720e-01_rb,1.5217e-01_rb,3.1488e-01_rb/)
      kao(:, 3,11, 4) = (/ &
     &7.0457e-02_rb,1.4249e-01_rb,1.6412e-01_rb,1.8266e-01_rb,1.9585e-01_rb,1.9778e-01_rb, &
     &1.8906e-01_rb,1.4085e-01_rb,3.1421e-01_rb/)
      kao(:, 4,11, 4) = (/ &
     &6.7094e-02_rb,1.3863e-01_rb,1.6386e-01_rb,1.8321e-01_rb,1.9415e-01_rb,1.9186e-01_rb, &
     &1.8023e-01_rb,1.3105e-01_rb,3.1088e-01_rb/)
      kao(:, 5,11, 4) = (/ &
     &6.4100e-02_rb,1.3524e-01_rb,1.6357e-01_rb,1.8497e-01_rb,1.9137e-01_rb,1.8498e-01_rb, &
     &1.7319e-01_rb,1.2321e-01_rb,3.0324e-01_rb/)
      kao(:, 1,12, 4) = (/ &
     &6.6765e-02_rb,1.2994e-01_rb,1.4944e-01_rb,1.6445e-01_rb,1.7587e-01_rb,1.8101e-01_rb, &
     &1.7769e-01_rb,1.3984e-01_rb,2.7673e-01_rb/)
      kao(:, 2,12, 4) = (/ &
     &6.3023e-02_rb,1.2593e-01_rb,1.4588e-01_rb,1.6162e-01_rb,1.7387e-01_rb,1.7653e-01_rb, &
     &1.6990e-01_rb,1.2980e-01_rb,2.7397e-01_rb/)
      kao(:, 3,12, 4) = (/ &
     &6.0078e-02_rb,1.2173e-01_rb,1.4433e-01_rb,1.6136e-01_rb,1.7141e-01_rb,1.7100e-01_rb, &
     &1.6190e-01_rb,1.2017e-01_rb,2.6912e-01_rb/)
      kao(:, 4,12, 4) = (/ &
     &5.7591e-02_rb,1.1797e-01_rb,1.4432e-01_rb,1.6200e-01_rb,1.6903e-01_rb,1.6549e-01_rb, &
     &1.5510e-01_rb,1.1177e-01_rb,2.6329e-01_rb/)
      kao(:, 5,12, 4) = (/ &
     &5.5506e-02_rb,1.1583e-01_rb,1.4556e-01_rb,1.5779e-01_rb,1.6302e-01_rb,1.5857e-01_rb, &
     &1.4926e-01_rb,1.0509e-01_rb,2.5258e-01_rb/)
      kao(:, 1,13, 4) = (/ &
     &5.9522e-02_rb,1.1440e-01_rb,1.3268e-01_rb,1.4518e-01_rb,1.5668e-01_rb,1.6016e-01_rb, &
     &1.5672e-01_rb,1.2200e-01_rb,2.4269e-01_rb/)
      kao(:, 2,13, 4) = (/ &
     &5.6603e-02_rb,1.1049e-01_rb,1.3023e-01_rb,1.4451e-01_rb,1.5437e-01_rb,1.5452e-01_rb, &
     &1.4924e-01_rb,1.1346e-01_rb,2.3750e-01_rb/)
      kao(:, 3,13, 4) = (/ &
     &5.4101e-02_rb,1.0639e-01_rb,1.2967e-01_rb,1.4482e-01_rb,1.5052e-01_rb,1.4869e-01_rb, &
     &1.4191e-01_rb,1.0569e-01_rb,2.3366e-01_rb/)
      kao(:, 4,13, 4) = (/ &
     &5.1959e-02_rb,1.0421e-01_rb,1.3053e-01_rb,1.4062e-01_rb,1.4577e-01_rb,1.4358e-01_rb, &
     &1.3605e-01_rb,9.8273e-02_rb,2.2442e-01_rb/)
      kao(:, 5,13, 4) = (/ &
     &4.9821e-02_rb,1.0307e-01_rb,1.3162e-01_rb,1.3605e-01_rb,1.4048e-01_rb,1.3772e-01_rb, &
     &1.3142e-01_rb,9.2354e-02_rb,2.1552e-01_rb/)
      kao(:, 1, 1, 5) = (/ &
     &4.4662e-01_rb,4.5491e-01_rb,4.6518e-01_rb,4.8073e-01_rb,9.6301e-01_rb,1.0662e+00_rb, &
     &9.6837e-01_rb,6.1516e-01_rb,1.5492e+00_rb/)
      kao(:, 2, 1, 5) = (/ &
     &4.3070e-01_rb,4.3901e-01_rb,4.4894e-01_rb,4.8843e-01_rb,9.7366e-01_rb,1.0185e+00_rb, &
     &9.1919e-01_rb,5.9289e-01_rb,1.4372e+00_rb/)
      kao(:, 3, 1, 5) = (/ &
     &4.1920e-01_rb,4.2623e-01_rb,4.3584e-01_rb,5.1853e-01_rb,9.4077e-01_rb,9.6967e-01_rb, &
     &8.7962e-01_rb,5.7716e-01_rb,1.3511e+00_rb/)
      kao(:, 4, 1, 5) = (/ &
     &4.1320e-01_rb,4.1951e-01_rb,4.2768e-01_rb,5.3113e-01_rb,8.9548e-01_rb,9.3086e-01_rb, &
     &8.2656e-01_rb,5.7871e-01_rb,1.2540e+00_rb/)
      kao(:, 5, 1, 5) = (/ &
     &4.1049e-01_rb,4.1583e-01_rb,4.2551e-01_rb,5.4820e-01_rb,8.6249e-01_rb,8.9543e-01_rb, &
     &7.7827e-01_rb,5.9312e-01_rb,1.1774e+00_rb/)
      kao(:, 1, 2, 5) = (/ &
     &3.7419e-01_rb,3.9315e-01_rb,4.1387e-01_rb,7.3684e-01_rb,9.3729e-01_rb,9.5798e-01_rb, &
     &8.1977e-01_rb,5.4534e-01_rb,1.3830e+00_rb/)
      kao(:, 2, 2, 5) = (/ &
     &3.6046e-01_rb,3.7870e-01_rb,3.9949e-01_rb,7.5022e-01_rb,8.9119e-01_rb,9.0575e-01_rb, &
     &7.6682e-01_rb,5.2408e-01_rb,1.3014e+00_rb/)
      kao(:, 3, 2, 5) = (/ &
     &3.5058e-01_rb,3.6732e-01_rb,3.9129e-01_rb,7.6077e-01_rb,8.5537e-01_rb,8.6853e-01_rb, &
     &7.2131e-01_rb,5.2368e-01_rb,1.1992e+00_rb/)
      kao(:, 4, 2, 5) = (/ &
     &3.4363e-01_rb,3.5974e-01_rb,3.8593e-01_rb,7.6592e-01_rb,8.1963e-01_rb,8.3241e-01_rb, &
     &6.8939e-01_rb,5.2630e-01_rb,1.1280e+00_rb/)
      kao(:, 5, 2, 5) = (/ &
     &3.4117e-01_rb,3.5633e-01_rb,3.8040e-01_rb,7.4257e-01_rb,7.9365e-01_rb,7.8846e-01_rb, &
     &6.5921e-01_rb,5.2675e-01_rb,1.0716e+00_rb/)
      kao(:, 1, 3, 5) = (/ &
     &3.1617e-01_rb,3.4533e-01_rb,4.9004e-01_rb,8.0662e-01_rb,8.6490e-01_rb,8.2831e-01_rb, &
     &6.9369e-01_rb,4.9053e-01_rb,1.2685e+00_rb/)
      kao(:, 2, 3, 5) = (/ &
     &3.0386e-01_rb,3.3185e-01_rb,4.9271e-01_rb,7.6499e-01_rb,8.1967e-01_rb,7.8960e-01_rb, &
     &6.4565e-01_rb,4.7244e-01_rb,1.1699e+00_rb/)
      kao(:, 3, 3, 5) = (/ &
     &2.9464e-01_rb,3.2055e-01_rb,5.0043e-01_rb,7.2785e-01_rb,7.8235e-01_rb,7.5123e-01_rb, &
     &6.1372e-01_rb,4.7097e-01_rb,1.0909e+00_rb/)
      kao(:, 4, 3, 5) = (/ &
     &2.8765e-01_rb,3.1339e-01_rb,5.0754e-01_rb,6.9697e-01_rb,7.5208e-01_rb,7.1843e-01_rb, &
     &5.8571e-01_rb,4.6400e-01_rb,1.0036e+00_rb/)
      kao(:, 5, 3, 5) = (/ &
     &2.8344e-01_rb,3.0751e-01_rb,5.1242e-01_rb,6.6855e-01_rb,7.1995e-01_rb,6.9218e-01_rb, &
     &5.7184e-01_rb,4.7451e-01_rb,9.2127e-01_rb/)
      kao(:, 1, 4, 5) = (/ &
     &2.6775e-01_rb,3.0604e-01_rb,6.4660e-01_rb,7.3076e-01_rb,7.6521e-01_rb,7.2923e-01_rb, &
     &5.8618e-01_rb,4.3353e-01_rb,1.1015e+00_rb/)
      kao(:, 2, 4, 5) = (/ &
     &2.5632e-01_rb,2.9306e-01_rb,6.2522e-01_rb,6.9215e-01_rb,7.2674e-01_rb,6.8433e-01_rb, &
     &5.5230e-01_rb,4.1918e-01_rb,1.0081e+00_rb/)
      kao(:, 3, 4, 5) = (/ &
     &2.4780e-01_rb,2.8292e-01_rb,5.9390e-01_rb,6.5413e-01_rb,6.8773e-01_rb,6.5028e-01_rb, &
     &5.2160e-01_rb,4.1724e-01_rb,9.3062e-01_rb/)
      kao(:, 4, 4, 5) = (/ &
     &2.4100e-01_rb,2.7551e-01_rb,5.7041e-01_rb,6.2793e-01_rb,6.5534e-01_rb,6.2452e-01_rb, &
     &5.0866e-01_rb,4.1719e-01_rb,8.4917e-01_rb/)
      kao(:, 5, 4, 5) = (/ &
     &2.3658e-01_rb,2.6967e-01_rb,5.4813e-01_rb,6.0747e-01_rb,6.2636e-01_rb,5.9897e-01_rb, &
     &4.9899e-01_rb,4.2737e-01_rb,7.8635e-01_rb/)
      kao(:, 1, 5, 5) = (/ &
     &2.2714e-01_rb,3.0892e-01_rb,5.8868e-01_rb,6.5760e-01_rb,6.7083e-01_rb,6.2555e-01_rb, &
     &5.1259e-01_rb,3.8131e-01_rb,9.5049e-01_rb/)
      kao(:, 2, 5, 5) = (/ &
     &2.1666e-01_rb,3.0798e-01_rb,5.5062e-01_rb,6.2165e-01_rb,6.2939e-01_rb,5.9031e-01_rb, &
     &4.8040e-01_rb,3.7074e-01_rb,8.6898e-01_rb/)
      kao(:, 3, 5, 5) = (/ &
     &2.0871e-01_rb,3.0702e-01_rb,5.2691e-01_rb,5.8794e-01_rb,5.9599e-01_rb,5.6236e-01_rb, &
     &4.6150e-01_rb,3.6395e-01_rb,7.8288e-01_rb/)
      kao(:, 4, 5, 5) = (/ &
     &2.0276e-01_rb,3.0943e-01_rb,5.0433e-01_rb,5.5988e-01_rb,5.6518e-01_rb,5.3494e-01_rb, &
     &4.4833e-01_rb,3.6773e-01_rb,7.2419e-01_rb/)
      kao(:, 5, 5, 5) = (/ &
     &1.9892e-01_rb,3.1215e-01_rb,4.8278e-01_rb,5.3676e-01_rb,5.4170e-01_rb,5.0924e-01_rb, &
     &4.4109e-01_rb,3.7855e-01_rb,6.8358e-01_rb/)
      kao(:, 1, 6, 5) = (/ &
     &1.9298e-01_rb,3.8310e-01_rb,5.2601e-01_rb,5.8117e-01_rb,5.8505e-01_rb,5.4459e-01_rb, &
     &4.4513e-01_rb,3.3639e-01_rb,8.1804e-01_rb/)
      kao(:, 2, 6, 5) = (/ &
     &1.8380e-01_rb,3.7978e-01_rb,4.9784e-01_rb,5.4325e-01_rb,5.4503e-01_rb,5.1006e-01_rb, &
     &4.1750e-01_rb,3.2606e-01_rb,7.4136e-01_rb/)
      kao(:, 3, 6, 5) = (/ &
     &1.7694e-01_rb,3.8180e-01_rb,4.7027e-01_rb,5.1699e-01_rb,5.1079e-01_rb,4.8235e-01_rb, &
     &4.0208e-01_rb,3.2052e-01_rb,6.7777e-01_rb/)
      kao(:, 4, 6, 5) = (/ &
     &1.7133e-01_rb,3.7179e-01_rb,4.4558e-01_rb,4.8630e-01_rb,4.9002e-01_rb,4.5737e-01_rb, &
     &3.9155e-01_rb,3.2262e-01_rb,6.2593e-01_rb/)
      kao(:, 5, 6, 5) = (/ &
     &1.6754e-01_rb,3.5443e-01_rb,4.2444e-01_rb,4.6383e-01_rb,4.6954e-01_rb,4.4064e-01_rb, &
     &3.8904e-01_rb,3.3286e-01_rb,5.7664e-01_rb/)
      kao(:, 1, 7, 5) = (/ &
     &1.6680e-01_rb,3.8673e-01_rb,4.6977e-01_rb,5.0982e-01_rb,5.0645e-01_rb,4.7419e-01_rb, &
     &3.9067e-01_rb,2.9795e-01_rb,7.1500e-01_rb/)
      kao(:, 2, 7, 5) = (/ &
     &1.5848e-01_rb,3.6483e-01_rb,4.3657e-01_rb,4.7719e-01_rb,4.7339e-01_rb,4.4276e-01_rb, &
     &3.6591e-01_rb,2.8456e-01_rb,6.4266e-01_rb/)
      kao(:, 3, 7, 5) = (/ &
     &1.5206e-01_rb,3.4607e-01_rb,4.1294e-01_rb,4.4755e-01_rb,4.4747e-01_rb,4.1880e-01_rb, &
     &3.5274e-01_rb,2.8027e-01_rb,5.8922e-01_rb/)
      kao(:, 4, 7, 5) = (/ &
     &1.4707e-01_rb,3.2827e-01_rb,3.9251e-01_rb,4.2303e-01_rb,4.2524e-01_rb,3.9890e-01_rb, &
     &3.4920e-01_rb,2.8325e-01_rb,5.4340e-01_rb/)
      kao(:, 5, 7, 5) = (/ &
     &1.4361e-01_rb,3.1545e-01_rb,3.7348e-01_rb,4.0710e-01_rb,4.0646e-01_rb,3.8591e-01_rb, &
     &3.3880e-01_rb,2.9285e-01_rb,5.0986e-01_rb/)
      kao(:, 1, 8, 5) = (/ &
     &1.4820e-01_rb,3.5091e-01_rb,4.2013e-01_rb,4.5147e-01_rb,4.5056e-01_rb,4.1729e-01_rb, &
     &3.4786e-01_rb,2.6599e-01_rb,6.3121e-01_rb/)
      kao(:, 2, 8, 5) = (/ &
     &1.4070e-01_rb,3.2895e-01_rb,3.9148e-01_rb,4.1739e-01_rb,4.1786e-01_rb,3.8937e-01_rb, &
     &3.2642e-01_rb,2.5048e-01_rb,5.6833e-01_rb/)
      kao(:, 3, 8, 5) = (/ &
     &1.3491e-01_rb,3.1064e-01_rb,3.6928e-01_rb,3.9231e-01_rb,3.9615e-01_rb,3.7023e-01_rb, &
     &3.1619e-01_rb,2.4413e-01_rb,5.2283e-01_rb/)
      kao(:, 4, 8, 5) = (/ &
     &1.3040e-01_rb,2.9486e-01_rb,3.4865e-01_rb,3.7267e-01_rb,3.7585e-01_rb,3.5308e-01_rb, &
     &3.1289e-01_rb,2.4763e-01_rb,4.8158e-01_rb/)
      kao(:, 5, 8, 5) = (/ &
     &1.2713e-01_rb,2.8372e-01_rb,3.3299e-01_rb,3.5768e-01_rb,3.6293e-01_rb,3.4468e-01_rb, &
     &2.9924e-01_rb,2.5414e-01_rb,4.5058e-01_rb/)
      kao(:, 1, 9, 5) = (/ &
     &1.3333e-01_rb,3.1732e-01_rb,3.7806e-01_rb,3.9822e-01_rb,3.9576e-01_rb,3.6830e-01_rb, &
     &3.0959e-01_rb,2.3751e-01_rb,5.5734e-01_rb/)
      kao(:, 2, 9, 5) = (/ &
     &1.2728e-01_rb,2.9745e-01_rb,3.4973e-01_rb,3.7018e-01_rb,3.6774e-01_rb,3.4366e-01_rb, &
     &2.8994e-01_rb,2.2165e-01_rb,5.0172e-01_rb/)
      kao(:, 3, 9, 5) = (/ &
     &1.2260e-01_rb,2.7734e-01_rb,3.2656e-01_rb,3.4735e-01_rb,3.4508e-01_rb,3.2798e-01_rb, &
     &2.8353e-01_rb,2.1398e-01_rb,4.5989e-01_rb/)
      kao(:, 4, 9, 5) = (/ &
     &1.1943e-01_rb,2.6308e-01_rb,3.0660e-01_rb,3.2980e-01_rb,3.3173e-01_rb,3.1278e-01_rb, &
     &2.7390e-01_rb,2.1485e-01_rb,4.2942e-01_rb/)
      kao(:, 5, 9, 5) = (/ &
     &1.1701e-01_rb,2.4928e-01_rb,2.9183e-01_rb,3.1474e-01_rb,3.2039e-01_rb,3.0863e-01_rb, &
     &2.6433e-01_rb,2.1871e-01_rb,3.9836e-01_rb/)
      kao(:, 1,10, 5) = (/ &
     &1.2488e-01_rb,2.8649e-01_rb,3.3436e-01_rb,3.5060e-01_rb,3.4715e-01_rb,3.2381e-01_rb, &
     &2.7352e-01_rb,2.0897e-01_rb,4.9029e-01_rb/)
      kao(:, 2,10, 5) = (/ &
     &1.1908e-01_rb,2.6494e-01_rb,3.0883e-01_rb,3.2464e-01_rb,3.2171e-01_rb,3.0303e-01_rb, &
     &2.5647e-01_rb,1.9520e-01_rb,4.4008e-01_rb/)
      kao(:, 3,10, 5) = (/ &
     &1.1556e-01_rb,2.4763e-01_rb,2.8674e-01_rb,3.0514e-01_rb,3.0302e-01_rb,2.8868e-01_rb, &
     &2.5222e-01_rb,1.8782e-01_rb,4.0842e-01_rb/)
      kao(:, 4,10, 5) = (/ &
     &1.1322e-01_rb,2.3334e-01_rb,2.7126e-01_rb,2.8846e-01_rb,2.9340e-01_rb,2.7863e-01_rb, &
     &2.4033e-01_rb,1.8521e-01_rb,3.8163e-01_rb/)
      kao(:, 5,10, 5) = (/ &
     &1.1126e-01_rb,2.2285e-01_rb,2.5915e-01_rb,2.7811e-01_rb,2.8680e-01_rb,2.7376e-01_rb, &
     &2.3030e-01_rb,1.8823e-01_rb,3.5861e-01_rb/)
      kao(:, 1,11, 5) = (/ &
     &1.2133e-01_rb,2.4900e-01_rb,2.8640e-01_rb,2.9703e-01_rb,2.9301e-01_rb,2.7475e-01_rb, &
     &2.3250e-01_rb,1.7899e-01_rb,4.0785e-01_rb/)
      kao(:, 2,11, 5) = (/ &
     &1.1707e-01_rb,2.3382e-01_rb,2.6584e-01_rb,2.7776e-01_rb,2.7455e-01_rb,2.6096e-01_rb, &
     &2.2631e-01_rb,1.6847e-01_rb,3.7573e-01_rb/)
      kao(:, 3,11, 5) = (/ &
     &1.1375e-01_rb,2.1845e-01_rb,2.5115e-01_rb,2.6117e-01_rb,2.6335e-01_rb,2.5113e-01_rb, &
     &2.1653e-01_rb,1.6132e-01_rb,3.4922e-01_rb/)
      kao(:, 4,11, 5) = (/ &
     &1.1166e-01_rb,2.0864e-01_rb,2.3761e-01_rb,2.5163e-01_rb,2.5840e-01_rb,2.4419e-01_rb, &
     &2.0592e-01_rb,1.6130e-01_rb,3.3156e-01_rb/)
      kao(:, 5,11, 5) = (/ &
     &1.1026e-01_rb,2.0025e-01_rb,2.2827e-01_rb,2.4711e-01_rb,2.5459e-01_rb,2.3449e-01_rb, &
     &1.9737e-01_rb,1.6301e-01_rb,3.1155e-01_rb/)
      kao(:, 1,12, 5) = (/ &
     &1.2068e-01_rb,2.2498e-01_rb,2.5241e-01_rb,2.6093e-01_rb,2.5316e-01_rb,2.3830e-01_rb, &
     &2.0580e-01_rb,1.5434e-01_rb,3.4650e-01_rb/)
      kao(:, 2,12, 5) = (/ &
     &1.1732e-01_rb,2.1063e-01_rb,2.3754e-01_rb,2.4422e-01_rb,2.4130e-01_rb,2.2913e-01_rb, &
     &1.9581e-01_rb,1.4538e-01_rb,3.2419e-01_rb/)
      kao(:, 3,12, 5) = (/ &
     &1.1291e-01_rb,2.0050e-01_rb,2.2356e-01_rb,2.3528e-01_rb,2.3734e-01_rb,2.2099e-01_rb, &
     &1.8732e-01_rb,1.4169e-01_rb,3.0809e-01_rb/)
      kao(:, 4,12, 5) = (/ &
     &1.0801e-01_rb,1.9213e-01_rb,2.1514e-01_rb,2.2964e-01_rb,2.3229e-01_rb,2.1151e-01_rb, &
     &1.7615e-01_rb,1.4140e-01_rb,2.8807e-01_rb/)
      kao(:, 5,12, 5) = (/ &
     &1.0409e-01_rb,1.8478e-01_rb,2.0856e-01_rb,2.2937e-01_rb,2.2388e-01_rb,2.0038e-01_rb, &
     &1.7052e-01_rb,1.4327e-01_rb,2.6150e-01_rb/)
      kao(:, 1,13, 5) = (/ &
     &1.1514e-01_rb,2.0822e-01_rb,2.3178e-01_rb,2.3757e-01_rb,2.2988e-01_rb,2.1559e-01_rb, &
     &1.8505e-01_rb,1.3755e-01_rb,3.0255e-01_rb/)
      kao(:, 2,13, 5) = (/ &
     &1.0856e-01_rb,1.9378e-01_rb,2.1572e-01_rb,2.2466e-01_rb,2.2140e-01_rb,2.1071e-01_rb, &
     &1.7476e-01_rb,1.2903e-01_rb,2.8912e-01_rb/)
      kao(:, 3,13, 5) = (/ &
     &1.0257e-01_rb,1.8315e-01_rb,2.0306e-01_rb,2.1501e-01_rb,2.1837e-01_rb,1.9973e-01_rb, &
     &1.6546e-01_rb,1.2594e-01_rb,2.7440e-01_rb/)
      kao(:, 4,13, 5) = (/ &
     &9.7502e-02_rb,1.7382e-01_rb,1.9450e-01_rb,2.1448e-01_rb,2.0917e-01_rb,1.8735e-01_rb, &
     &1.5671e-01_rb,1.2605e-01_rb,2.4875e-01_rb/)
      kao(:, 5,13, 5) = (/ &
     &9.4048e-02_rb,1.6507e-01_rb,1.8951e-01_rb,2.1087e-01_rb,1.9884e-01_rb,1.7732e-01_rb, &
     &1.5188e-01_rb,1.2757e-01_rb,2.1998e-01_rb/)
      kao(:, 1, 1, 6) = (/ &
     &1.2242e+00_rb,1.1159e+00_rb,1.0120e+00_rb,9.1389e-01_rb,8.1856e-01_rb,1.1483e+00_rb, &
     &9.8709e-01_rb,9.2159e-01_rb,4.7502e-01_rb/)
      kao(:, 2, 1, 6) = (/ &
     &1.1928e+00_rb,1.0903e+00_rb,9.9311e-01_rb,9.0065e-01_rb,8.3223e-01_rb,1.1145e+00_rb, &
     &9.5738e-01_rb,9.2153e-01_rb,6.9449e-01_rb/)
      kao(:, 3, 1, 6) = (/ &
     &1.1808e+00_rb,1.0824e+00_rb,9.8824e-01_rb,8.9828e-01_rb,8.9586e-01_rb,1.0864e+00_rb, &
     &9.3281e-01_rb,9.2795e-01_rb,8.4213e-01_rb/)
      kao(:, 4, 1, 6) = (/ &
     &1.1721e+00_rb,1.0760e+00_rb,9.8481e-01_rb,8.9679e-01_rb,9.5583e-01_rb,1.0646e+00_rb, &
     &9.4941e-01_rb,9.2047e-01_rb,8.5863e-01_rb/)
      kao(:, 5, 1, 6) = (/ &
     &1.1790e+00_rb,1.0814e+00_rb,9.8996e-01_rb,8.9927e-01_rb,1.0035e+00_rb,1.0346e+00_rb, &
     &9.5167e-01_rb,9.2657e-01_rb,7.2509e-01_rb/)
      kao(:, 1, 2, 6) = (/ &
     &1.0264e+00_rb,9.4284e-01_rb,8.6709e-01_rb,7.9559e-01_rb,1.1080e+00_rb,1.0254e+00_rb, &
     &9.2224e-01_rb,8.7122e-01_rb,1.0512e+00_rb/)
      kao(:, 2, 2, 6) = (/ &
     &1.0018e+00_rb,9.2359e-01_rb,8.5299e-01_rb,7.8672e-01_rb,1.0674e+00_rb,9.8417e-01_rb, &
     &8.9700e-01_rb,8.6779e-01_rb,9.4293e-01_rb/)
      kao(:, 3, 2, 6) = (/ &
     &9.8988e-01_rb,9.1596e-01_rb,8.4833e-01_rb,7.8816e-01_rb,1.0499e+00_rb,9.5536e-01_rb, &
     &8.7579e-01_rb,8.5648e-01_rb,8.1445e-01_rb/)
      kao(:, 4, 2, 6) = (/ &
     &9.8822e-01_rb,9.1536e-01_rb,8.4663e-01_rb,7.9377e-01_rb,1.0292e+00_rb,9.3843e-01_rb, &
     &8.7227e-01_rb,8.6174e-01_rb,7.0730e-01_rb/)
      kao(:, 5, 2, 6) = (/ &
     &9.8800e-01_rb,9.1393e-01_rb,8.4855e-01_rb,8.2484e-01_rb,1.0072e+00_rb,9.4807e-01_rb, &
     &8.8827e-01_rb,8.7216e-01_rb,5.9331e-01_rb/)
      kao(:, 1, 3, 6) = (/ &
     &8.6794e-01_rb,8.0430e-01_rb,7.5031e-01_rb,8.9541e-01_rb,1.0214e+00_rb,9.4865e-01_rb, &
     &8.5821e-01_rb,7.9700e-01_rb,8.7719e-01_rb/)
      kao(:, 2, 3, 6) = (/ &
     &8.4313e-01_rb,7.8532e-01_rb,7.3608e-01_rb,9.3906e-01_rb,9.7060e-01_rb,9.0816e-01_rb, &
     &8.3425e-01_rb,7.9621e-01_rb,7.6329e-01_rb/)
      kao(:, 3, 3, 6) = (/ &
     &8.3157e-01_rb,7.7858e-01_rb,7.3266e-01_rb,9.5972e-01_rb,9.4116e-01_rb,8.8096e-01_rb, &
     &8.0848e-01_rb,7.9477e-01_rb,6.5966e-01_rb/)
      kao(:, 4, 3, 6) = (/ &
     &8.2987e-01_rb,7.7678e-01_rb,7.3040e-01_rb,9.3971e-01_rb,9.0923e-01_rb,8.7002e-01_rb, &
     &8.0556e-01_rb,8.1217e-01_rb,6.0022e-01_rb/)
      kao(:, 5, 3, 6) = (/ &
     &8.2093e-01_rb,7.6968e-01_rb,7.2610e-01_rb,9.2404e-01_rb,8.9670e-01_rb,8.5391e-01_rb, &
     &7.9373e-01_rb,8.1610e-01_rb,5.3117e-01_rb/)
      kao(:, 1, 4, 6) = (/ &
     &7.3504e-01_rb,6.8896e-01_rb,6.5474e-01_rb,9.5125e-01_rb,9.3510e-01_rb,8.8191e-01_rb, &
     &7.8339e-01_rb,7.2709e-01_rb,7.9136e-01_rb/)
      kao(:, 2, 4, 6) = (/ &
     &7.1205e-01_rb,6.7167e-01_rb,6.7088e-01_rb,9.0689e-01_rb,8.7886e-01_rb,8.4133e-01_rb, &
     &7.6196e-01_rb,7.2232e-01_rb,6.9067e-01_rb/)
      kao(:, 3, 4, 6) = (/ &
     &7.0055e-01_rb,6.6424e-01_rb,7.0492e-01_rb,8.8094e-01_rb,8.5026e-01_rb,8.0832e-01_rb, &
     &7.4494e-01_rb,7.3074e-01_rb,6.1263e-01_rb/)
      kao(:, 4, 4, 6) = (/ &
     &6.9588e-01_rb,6.6015e-01_rb,7.3043e-01_rb,8.5040e-01_rb,8.2745e-01_rb,7.8458e-01_rb, &
     &7.3728e-01_rb,7.3901e-01_rb,5.3700e-01_rb/)
      kao(:, 5, 4, 6) = (/ &
     &6.8558e-01_rb,6.5206e-01_rb,7.5371e-01_rb,8.2365e-01_rb,8.2063e-01_rb,7.6670e-01_rb, &
     &7.3328e-01_rb,7.4717e-01_rb,4.4797e-01_rb/)
      kao(:, 1, 5, 6) = (/ &
     &6.2240e-01_rb,5.9202e-01_rb,7.9818e-01_rb,8.6221e-01_rb,8.4556e-01_rb,7.9840e-01_rb, &
     &7.0353e-01_rb,6.4834e-01_rb,7.3196e-01_rb/)
      kao(:, 2, 5, 6) = (/ &
     &6.0152e-01_rb,5.7633e-01_rb,8.0789e-01_rb,8.1513e-01_rb,8.0789e-01_rb,7.5099e-01_rb, &
     &6.7710e-01_rb,6.4945e-01_rb,6.3067e-01_rb/)
      kao(:, 3, 5, 6) = (/ &
     &5.8999e-01_rb,5.6942e-01_rb,7.6969e-01_rb,7.8647e-01_rb,7.7674e-01_rb,7.1468e-01_rb, &
     &6.6534e-01_rb,6.5860e-01_rb,5.4173e-01_rb/)
      kao(:, 4, 5, 6) = (/ &
     &5.8241e-01_rb,5.6326e-01_rb,7.5015e-01_rb,7.6096e-01_rb,7.5392e-01_rb,6.9429e-01_rb, &
     &6.6094e-01_rb,6.6621e-01_rb,4.4433e-01_rb/)
      kao(:, 5, 5, 6) = (/ &
     &5.7098e-01_rb,5.5310e-01_rb,7.2743e-01_rb,7.3939e-01_rb,7.3730e-01_rb,6.8982e-01_rb, &
     &6.6680e-01_rb,6.6632e-01_rb,3.5896e-01_rb/)
      kao(:, 1, 6, 6) = (/ &
     &5.2885e-01_rb,5.1244e-01_rb,7.5494e-01_rb,7.8724e-01_rb,7.7080e-01_rb,7.0256e-01_rb, &
     &6.2568e-01_rb,5.6883e-01_rb,6.4732e-01_rb/)
      kao(:, 2, 6, 6) = (/ &
     &5.0860e-01_rb,4.9747e-01_rb,7.1101e-01_rb,7.4294e-01_rb,7.2488e-01_rb,6.6139e-01_rb, &
     &6.0894e-01_rb,5.7630e-01_rb,5.5132e-01_rb/)
      kao(:, 3, 6, 6) = (/ &
     &4.9680e-01_rb,4.8858e-01_rb,6.8601e-01_rb,7.0262e-01_rb,6.9083e-01_rb,6.2832e-01_rb, &
     &5.9581e-01_rb,5.8177e-01_rb,4.5843e-01_rb/)
      kao(:, 4, 6, 6) = (/ &
     &4.8828e-01_rb,4.9338e-01_rb,6.6185e-01_rb,6.8834e-01_rb,6.6433e-01_rb,6.1518e-01_rb, &
     &5.9405e-01_rb,5.8713e-01_rb,3.7286e-01_rb/)
      kao(:, 5, 6, 6) = (/ &
     &4.7758e-01_rb,5.0393e-01_rb,6.4306e-01_rb,6.6695e-01_rb,6.4365e-01_rb,6.0629e-01_rb, &
     &5.9623e-01_rb,5.8994e-01_rb,3.0878e-01_rb/)
      kao(:, 1, 7, 6) = (/ &
     &4.5544e-01_rb,5.1675e-01_rb,6.9040e-01_rb,7.2442e-01_rb,6.9583e-01_rb,6.2374e-01_rb, &
     &5.5330e-01_rb,4.9819e-01_rb,5.7066e-01_rb/)
      kao(:, 2, 7, 6) = (/ &
     &4.3526e-01_rb,5.2433e-01_rb,6.5887e-01_rb,6.7132e-01_rb,6.4888e-01_rb,5.8229e-01_rb, &
     &5.3592e-01_rb,5.0387e-01_rb,4.7263e-01_rb/)
      kao(:, 3, 7, 6) = (/ &
     &4.2386e-01_rb,5.3503e-01_rb,6.2067e-01_rb,6.3646e-01_rb,6.0983e-01_rb,5.5722e-01_rb, &
     &5.2827e-01_rb,5.1003e-01_rb,3.8891e-01_rb/)
      kao(:, 4, 7, 6) = (/ &
     &4.1478e-01_rb,5.3714e-01_rb,5.9921e-01_rb,6.0957e-01_rb,5.8334e-01_rb,5.4159e-01_rb, &
     &5.2418e-01_rb,5.1278e-01_rb,3.1885e-01_rb/)
      kao(:, 5, 7, 6) = (/ &
     &4.0467e-01_rb,5.2009e-01_rb,5.7701e-01_rb,5.8575e-01_rb,5.6859e-01_rb,5.4155e-01_rb, &
     &5.3930e-01_rb,5.1710e-01_rb,2.6235e-01_rb/)
      kao(:, 1, 8, 6) = (/ &
     &4.0091e-01_rb,5.6115e-01_rb,6.4335e-01_rb,6.5229e-01_rb,6.2013e-01_rb,5.5832e-01_rb, &
     &4.9333e-01_rb,4.3362e-01_rb,5.1143e-01_rb/)
      kao(:, 2, 8, 6) = (/ &
     &3.8169e-01_rb,5.2953e-01_rb,5.9829e-01_rb,6.0811e-01_rb,5.7734e-01_rb,5.1950e-01_rb, &
     &4.7393e-01_rb,4.3986e-01_rb,4.2322e-01_rb/)
      kao(:, 3, 8, 6) = (/ &
     &3.7078e-01_rb,5.0436e-01_rb,5.6958e-01_rb,5.7197e-01_rb,5.3831e-01_rb,4.9299e-01_rb, &
     &4.6737e-01_rb,4.4557e-01_rb,3.4628e-01_rb/)
      kao(:, 4, 8, 6) = (/ &
     &3.6161e-01_rb,4.8989e-01_rb,5.4426e-01_rb,5.4657e-01_rb,5.1938e-01_rb,4.8189e-01_rb, &
     &4.6346e-01_rb,4.4852e-01_rb,2.8491e-01_rb/)
      kao(:, 5, 8, 6) = (/ &
     &3.5235e-01_rb,4.6785e-01_rb,5.2499e-01_rb,5.2434e-01_rb,5.0210e-01_rb,4.7954e-01_rb, &
     &4.8680e-01_rb,4.5258e-01_rb,2.3809e-01_rb/)
      kao(:, 1, 9, 6) = (/ &
     &3.5188e-01_rb,5.0784e-01_rb,5.8680e-01_rb,5.8599e-01_rb,5.5453e-01_rb,4.9355e-01_rb, &
     &4.3434e-01_rb,3.7345e-01_rb,4.5760e-01_rb/)
      kao(:, 2, 9, 6) = (/ &
     &3.3384e-01_rb,4.7725e-01_rb,5.4527e-01_rb,5.4039e-01_rb,5.1277e-01_rb,4.6053e-01_rb, &
     &4.1752e-01_rb,3.8054e-01_rb,3.7664e-01_rb/)
      kao(:, 3, 9, 6) = (/ &
     &3.2337e-01_rb,4.6079e-01_rb,5.0889e-01_rb,5.0540e-01_rb,4.8167e-01_rb,4.3259e-01_rb, &
     &4.1005e-01_rb,3.8447e-01_rb,3.0502e-01_rb/)
      kao(:, 4, 9, 6) = (/ &
     &3.1489e-01_rb,4.4224e-01_rb,4.8906e-01_rb,4.8098e-01_rb,4.5962e-01_rb,4.2889e-01_rb, &
     &4.1407e-01_rb,3.8891e-01_rb,2.4901e-01_rb/)
      kao(:, 5, 9, 6) = (/ &
     &3.0731e-01_rb,4.2604e-01_rb,4.6542e-01_rb,4.6457e-01_rb,4.4425e-01_rb,4.2325e-01_rb, &
     &4.3000e-01_rb,3.9500e-01_rb,2.1365e-01_rb/)
      kao(:, 1,10, 6) = (/ &
     &3.0872e-01_rb,4.6438e-01_rb,5.2605e-01_rb,5.1940e-01_rb,4.8758e-01_rb,4.3435e-01_rb, &
     &3.8048e-01_rb,3.2415e-01_rb,4.0329e-01_rb/)
      kao(:, 2,10, 6) = (/ &
     &2.9295e-01_rb,4.4115e-01_rb,4.8348e-01_rb,4.7740e-01_rb,4.4909e-01_rb,4.0221e-01_rb, &
     &3.6624e-01_rb,3.2721e-01_rb,3.3011e-01_rb/)
      kao(:, 3,10, 6) = (/ &
     &2.8328e-01_rb,4.1772e-01_rb,4.5515e-01_rb,4.4741e-01_rb,4.2330e-01_rb,3.8012e-01_rb, &
     &3.5851e-01_rb,3.3085e-01_rb,2.6425e-01_rb/)
      kao(:, 4,10, 6) = (/ &
     &2.7562e-01_rb,4.0009e-01_rb,4.2827e-01_rb,4.2695e-01_rb,4.0250e-01_rb,3.7844e-01_rb, &
     &3.6844e-01_rb,3.3695e-01_rb,2.1886e-01_rb/)
      kao(:, 5,10, 6) = (/ &
     &2.6849e-01_rb,3.8403e-01_rb,4.1066e-01_rb,4.0861e-01_rb,3.9231e-01_rb,3.7926e-01_rb, &
     &3.7696e-01_rb,3.4332e-01_rb,1.9262e-01_rb/)
      kao(:, 1,11, 6) = (/ &
     &2.6466e-01_rb,4.1355e-01_rb,4.4454e-01_rb,4.4042e-01_rb,4.1233e-01_rb,3.6752e-01_rb, &
     &3.2678e-01_rb,2.7894e-01_rb,3.2749e-01_rb/)
      kao(:, 2,11, 6) = (/ &
     &2.5291e-01_rb,3.8781e-01_rb,4.1430e-01_rb,4.0659e-01_rb,3.8277e-01_rb,3.4153e-01_rb, &
     &3.1495e-01_rb,2.7988e-01_rb,2.6257e-01_rb/)
      kao(:, 3,11, 6) = (/ &
     &2.4541e-01_rb,3.7100e-01_rb,3.8849e-01_rb,3.8315e-01_rb,3.6055e-01_rb,3.3569e-01_rb, &
     &3.1896e-01_rb,2.8851e-01_rb,2.1720e-01_rb/)
      kao(:, 4,11, 6) = (/ &
     &2.3761e-01_rb,3.4992e-01_rb,3.7138e-01_rb,3.6645e-01_rb,3.4895e-01_rb,3.3415e-01_rb, &
     &3.2576e-01_rb,2.9375e-01_rb,1.9101e-01_rb/)
      kao(:, 5,11, 6) = (/ &
     &2.3136e-01_rb,3.3422e-01_rb,3.5550e-01_rb,3.5462e-01_rb,3.4596e-01_rb,3.4410e-01_rb, &
     &3.3336e-01_rb,3.0010e-01_rb,1.7683e-01_rb/)
      kao(:, 1,12, 6) = (/ &
     &2.3099e-01_rb,3.6467e-01_rb,3.8654e-01_rb,3.7740e-01_rb,3.5454e-01_rb,3.1341e-01_rb, &
     &2.8207e-01_rb,2.4141e-01_rb,2.7098e-01_rb/)
      kao(:, 2,12, 6) = (/ &
     &2.2244e-01_rb,3.4190e-01_rb,3.5753e-01_rb,3.5048e-01_rb,3.3109e-01_rb,2.9786e-01_rb, &
     &2.7909e-01_rb,2.4469e-01_rb,2.1730e-01_rb/)
      kao(:, 3,12, 6) = (/ &
     &2.1694e-01_rb,3.2183e-01_rb,3.3982e-01_rb,3.3193e-01_rb,3.1521e-01_rb,2.9823e-01_rb, &
     &2.8101e-01_rb,2.4988e-01_rb,1.8539e-01_rb/)
      kao(:, 4,12, 6) = (/ &
     &2.1365e-01_rb,3.0582e-01_rb,3.2214e-01_rb,3.1915e-01_rb,3.0963e-01_rb,3.0101e-01_rb, &
     &2.9010e-01_rb,2.5511e-01_rb,1.7246e-01_rb/)
      kao(:, 5,12, 6) = (/ &
     &2.1126e-01_rb,2.9440e-01_rb,3.1065e-01_rb,3.1395e-01_rb,3.1604e-01_rb,3.0993e-01_rb, &
     &2.9537e-01_rb,2.6162e-01_rb,1.5291e-01_rb/)
      kao(:, 1,13, 6) = (/ &
     &2.2386e-01_rb,3.3097e-01_rb,3.4162e-01_rb,3.3327e-01_rb,3.1314e-01_rb,2.7728e-01_rb, &
     &2.5014e-01_rb,2.1154e-01_rb,2.3428e-01_rb/)
      kao(:, 2,13, 6) = (/ &
     &2.2040e-01_rb,3.1261e-01_rb,3.2237e-01_rb,3.1373e-01_rb,2.9682e-01_rb,2.6651e-01_rb, &
     &2.5032e-01_rb,2.1702e-01_rb,1.9039e-01_rb/)
      kao(:, 3,13, 6) = (/ &
     &2.1610e-01_rb,2.9730e-01_rb,3.0955e-01_rb,3.0133e-01_rb,2.8651e-01_rb,2.7201e-01_rb, &
     &2.5430e-01_rb,2.2027e-01_rb,1.6665e-01_rb/)
      kao(:, 4,13, 6) = (/ &
     &2.1312e-01_rb,2.8629e-01_rb,2.9748e-01_rb,2.9400e-01_rb,2.8967e-01_rb,2.7599e-01_rb, &
     &2.5853e-01_rb,2.2523e-01_rb,1.5373e-01_rb/)
      kao(:, 5,13, 6) = (/ &
     &2.0806e-01_rb,2.7757e-01_rb,2.8967e-01_rb,2.9354e-01_rb,2.9376e-01_rb,2.8253e-01_rb, &
     &2.6218e-01_rb,2.3101e-01_rb,1.3574e-01_rb/)
      kao(:, 1, 1, 7) = (/ &
     &3.5613e+00_rb,3.1188e+00_rb,2.6764e+00_rb,2.2340e+00_rb,1.7917e+00_rb,1.6352e+00_rb, &
     &1.8092e+00_rb,1.6379e+00_rb,2.7539e-02_rb/)
      kao(:, 2, 1, 7) = (/ &
     &3.4002e+00_rb,2.9785e+00_rb,2.5569e+00_rb,2.1349e+00_rb,1.7133e+00_rb,1.7200e+00_rb, &
     &1.7738e+00_rb,1.6325e+00_rb,3.6313e-02_rb/)
      kao(:, 3, 1, 7) = (/ &
     &3.2557e+00_rb,2.8529e+00_rb,2.4503e+00_rb,2.0478e+00_rb,1.6479e+00_rb,1.7947e+00_rb, &
     &1.7613e+00_rb,1.6123e+00_rb,7.9598e-02_rb/)
      kao(:, 4, 1, 7) = (/ &
     &3.1418e+00_rb,2.7541e+00_rb,2.3673e+00_rb,1.9817e+00_rb,1.6011e+00_rb,1.8327e+00_rb, &
     &1.7198e+00_rb,1.6148e+00_rb,2.1944e-01_rb/)
      kao(:, 5, 1, 7) = (/ &
     &3.1204e+00_rb,2.7380e+00_rb,2.3562e+00_rb,1.9785e+00_rb,1.6099e+00_rb,1.8773e+00_rb, &
     &1.7180e+00_rb,1.6127e+00_rb,2.9776e-01_rb/)
      kao(:, 1, 2, 7) = (/ &
     &2.9874e+00_rb,2.6163e+00_rb,2.2453e+00_rb,1.8746e+00_rb,1.5312e+00_rb,1.8525e+00_rb, &
     &1.7071e+00_rb,1.5893e+00_rb,5.1776e-01_rb/)
      kao(:, 2, 2, 7) = (/ &
     &2.8505e+00_rb,2.4972e+00_rb,2.1440e+00_rb,1.7909e+00_rb,1.5966e+00_rb,1.8306e+00_rb, &
     &1.6863e+00_rb,1.5842e+00_rb,4.3879e-01_rb/)
      kao(:, 3, 2, 7) = (/ &
     &2.7350e+00_rb,2.3969e+00_rb,2.0589e+00_rb,1.7214e+00_rb,1.6370e+00_rb,1.7877e+00_rb, &
     &1.6805e+00_rb,1.5918e+00_rb,4.1523e-01_rb/)
      kao(:, 4, 2, 7) = (/ &
     &2.6409e+00_rb,2.3156e+00_rb,1.9935e+00_rb,1.6727e+00_rb,1.6830e+00_rb,1.7575e+00_rb, &
     &1.6634e+00_rb,1.5980e+00_rb,2.9373e-01_rb/)
      kao(:, 5, 2, 7) = (/ &
     &2.5886e+00_rb,2.2740e+00_rb,1.9607e+00_rb,1.6497e+00_rb,1.7391e+00_rb,1.7232e+00_rb, &
     &1.6344e+00_rb,1.6105e+00_rb,1.8509e-01_rb/)
      kao(:, 1, 3, 7) = (/ &
     &2.5285e+00_rb,2.2147e+00_rb,1.9010e+00_rb,1.5874e+00_rb,1.8079e+00_rb,1.7347e+00_rb, &
     &1.6267e+00_rb,1.5173e+00_rb,4.9543e-01_rb/)
      kao(:, 2, 3, 7) = (/ &
     &2.4109e+00_rb,2.1124e+00_rb,1.8139e+00_rb,1.5156e+00_rb,1.7831e+00_rb,1.7011e+00_rb, &
     &1.6143e+00_rb,1.5141e+00_rb,4.4268e-01_rb/)
      kao(:, 3, 3, 7) = (/ &
     &2.3122e+00_rb,2.0266e+00_rb,1.7413e+00_rb,1.4891e+00_rb,1.7442e+00_rb,1.6741e+00_rb, &
     &1.6075e+00_rb,1.5297e+00_rb,3.4522e-01_rb/)
      kao(:, 4, 3, 7) = (/ &
     &2.2338e+00_rb,1.9602e+00_rb,1.6889e+00_rb,1.5208e+00_rb,1.7228e+00_rb,1.6317e+00_rb, &
     &1.5933e+00_rb,1.5434e+00_rb,2.1394e-01_rb/)
      kao(:, 5, 3, 7) = (/ &
     &2.1901e+00_rb,1.9250e+00_rb,1.6600e+00_rb,1.5510e+00_rb,1.6972e+00_rb,1.6258e+00_rb, &
     &1.5808e+00_rb,1.5686e+00_rb,1.5097e-01_rb/)
      kao(:, 1, 4, 7) = (/ &
     &2.1490e+00_rb,1.8824e+00_rb,1.6159e+00_rb,1.5783e+00_rb,1.6916e+00_rb,1.6097e+00_rb, &
     &1.5676e+00_rb,1.4069e+00_rb,5.0896e-01_rb/)
      kao(:, 2, 4, 7) = (/ &
     &2.0429e+00_rb,1.7900e+00_rb,1.5373e+00_rb,1.6130e+00_rb,1.6613e+00_rb,1.5828e+00_rb, &
     &1.5388e+00_rb,1.4176e+00_rb,3.7374e-01_rb/)
      kao(:, 3, 4, 7) = (/ &
     &1.9560e+00_rb,1.7146e+00_rb,1.4748e+00_rb,1.6326e+00_rb,1.6129e+00_rb,1.5608e+00_rb, &
     &1.5146e+00_rb,1.4336e+00_rb,2.6187e-01_rb/)
      kao(:, 4, 4, 7) = (/ &
     &1.8914e+00_rb,1.6603e+00_rb,1.4322e+00_rb,1.6118e+00_rb,1.5843e+00_rb,1.5418e+00_rb, &
     &1.4880e+00_rb,1.4608e+00_rb,1.8007e-01_rb/)
      kao(:, 5, 4, 7) = (/ &
     &1.8569e+00_rb,1.6328e+00_rb,1.4097e+00_rb,1.6022e+00_rb,1.5587e+00_rb,1.5446e+00_rb, &
     &1.4805e+00_rb,1.4912e+00_rb,1.1749e-01_rb/)
      kao(:, 1, 5, 7) = (/ &
     &1.8276e+00_rb,1.6010e+00_rb,1.3745e+00_rb,1.6030e+00_rb,1.5630e+00_rb,1.5166e+00_rb, &
     &1.4457e+00_rb,1.2855e+00_rb,4.3144e-01_rb/)
      kao(:, 2, 5, 7) = (/ &
     &1.7325e+00_rb,1.5182e+00_rb,1.3289e+00_rb,1.5483e+00_rb,1.5172e+00_rb,1.4962e+00_rb, &
     &1.4204e+00_rb,1.2974e+00_rb,3.2532e-01_rb/)
      kao(:, 3, 5, 7) = (/ &
     &1.6556e+00_rb,1.4514e+00_rb,1.3491e+00_rb,1.5022e+00_rb,1.4839e+00_rb,1.4801e+00_rb, &
     &1.3899e+00_rb,1.3214e+00_rb,2.2874e-01_rb/)
      kao(:, 4, 5, 7) = (/ &
     &1.6008e+00_rb,1.4058e+00_rb,1.3614e+00_rb,1.4817e+00_rb,1.4667e+00_rb,1.4542e+00_rb, &
     &1.3728e+00_rb,1.3543e+00_rb,1.4774e-01_rb/)
      kao(:, 5, 5, 7) = (/ &
     &1.5722e+00_rb,1.3835e+00_rb,1.3910e+00_rb,1.4609e+00_rb,1.4510e+00_rb,1.4341e+00_rb, &
     &1.3580e+00_rb,1.3940e+00_rb,9.9947e-02_rb/)
      kao(:, 1, 6, 7) = (/ &
     &1.5600e+00_rb,1.3665e+00_rb,1.3891e+00_rb,1.4520e+00_rb,1.4361e+00_rb,1.4207e+00_rb, &
     &1.3105e+00_rb,1.1513e+00_rb,3.8692e-01_rb/)
      kao(:, 2, 6, 7) = (/ &
     &1.4739e+00_rb,1.2917e+00_rb,1.3972e+00_rb,1.4114e+00_rb,1.4085e+00_rb,1.3840e+00_rb, &
     &1.2763e+00_rb,1.1653e+00_rb,2.7607e-01_rb/)
      kao(:, 3, 6, 7) = (/ &
     &1.4042e+00_rb,1.2317e+00_rb,1.3535e+00_rb,1.3731e+00_rb,1.3834e+00_rb,1.3540e+00_rb, &
     &1.2537e+00_rb,1.1954e+00_rb,1.8549e-01_rb/)
      kao(:, 4, 6, 7) = (/ &
     &1.3553e+00_rb,1.1909e+00_rb,1.3235e+00_rb,1.3411e+00_rb,1.3606e+00_rb,1.3282e+00_rb, &
     &1.2356e+00_rb,1.2339e+00_rb,1.3113e-01_rb/)
      kao(:, 5, 6, 7) = (/ &
     &1.3290e+00_rb,1.1708e+00_rb,1.3052e+00_rb,1.3291e+00_rb,1.3435e+00_rb,1.3084e+00_rb, &
     &1.2394e+00_rb,1.2789e+00_rb,8.5104e-02_rb/)
      kao(:, 1, 7, 7) = (/ &
     &1.3486e+00_rb,1.1821e+00_rb,1.3320e+00_rb,1.3296e+00_rb,1.3530e+00_rb,1.2921e+00_rb, &
     &1.1792e+00_rb,1.0146e+00_rb,3.3967e-01_rb/)
      kao(:, 2, 7, 7) = (/ &
     &1.2698e+00_rb,1.1140e+00_rb,1.2658e+00_rb,1.2934e+00_rb,1.3084e+00_rb,1.2591e+00_rb, &
     &1.1496e+00_rb,1.0323e+00_rb,2.4089e-01_rb/)
      kao(:, 3, 7, 7) = (/ &
     &1.2066e+00_rb,1.0597e+00_rb,1.2265e+00_rb,1.2606e+00_rb,1.2717e+00_rb,1.2202e+00_rb, &
     &1.1236e+00_rb,1.0630e+00_rb,1.6653e-01_rb/)
      kao(:, 4, 7, 7) = (/ &
     &1.1624e+00_rb,1.0337e+00_rb,1.1938e+00_rb,1.2388e+00_rb,1.2436e+00_rb,1.1958e+00_rb, &
     &1.1143e+00_rb,1.1071e+00_rb,1.1640e-01_rb/)
      kao(:, 5, 7, 7) = (/ &
     &1.1373e+00_rb,1.0419e+00_rb,1.1824e+00_rb,1.2289e+00_rb,1.2206e+00_rb,1.1715e+00_rb, &
     &1.1114e+00_rb,1.1517e+00_rb,7.1520e-02_rb/)
      kao(:, 1, 8, 7) = (/ &
     &1.1959e+00_rb,1.0587e+00_rb,1.2168e+00_rb,1.2562e+00_rb,1.2533e+00_rb,1.1755e+00_rb, &
     &1.0535e+00_rb,8.9257e-01_rb,2.9562e-01_rb/)
      kao(:, 2, 8, 7) = (/ &
     &1.1212e+00_rb,1.0393e+00_rb,1.1676e+00_rb,1.2170e+00_rb,1.2001e+00_rb,1.1361e+00_rb, &
     &1.0253e+00_rb,9.0430e-01_rb,2.1449e-01_rb/)
      kao(:, 3, 8, 7) = (/ &
     &1.0601e+00_rb,1.0297e+00_rb,1.1266e+00_rb,1.1809e+00_rb,1.1623e+00_rb,1.1020e+00_rb, &
     &1.0023e+00_rb,9.3591e-01_rb,1.5382e-01_rb/)
      kao(:, 4, 8, 7) = (/ &
     &1.0187e+00_rb,1.0227e+00_rb,1.0991e+00_rb,1.1460e+00_rb,1.1243e+00_rb,1.0751e+00_rb, &
     &9.9421e-01_rb,9.7855e-01_rb,1.0944e-01_rb/)
      kao(:, 5, 8, 7) = (/ &
     &9.9531e-01_rb,1.0219e+00_rb,1.0832e+00_rb,1.1243e+00_rb,1.1050e+00_rb,1.0609e+00_rb, &
     &9.9136e-01_rb,1.0252e+00_rb,7.0614e-02_rb/)
      kao(:, 1, 9, 7) = (/ &
     &1.0539e+00_rb,1.0559e+00_rb,1.1142e+00_rb,1.1729e+00_rb,1.1295e+00_rb,1.0580e+00_rb, &
     &9.3382e-01_rb,7.7194e-01_rb,2.7489e-01_rb/)
      kao(:, 2, 9, 7) = (/ &
     &9.8392e-01_rb,1.0263e+00_rb,1.0713e+00_rb,1.1148e+00_rb,1.0796e+00_rb,1.0137e+00_rb, &
     &9.0260e-01_rb,7.8234e-01_rb,1.9796e-01_rb/)
      kao(:, 3, 9, 7) = (/ &
     &9.2807e-01_rb,9.7104e-01_rb,1.0401e+00_rb,1.0698e+00_rb,1.0381e+00_rb,9.8279e-01_rb, &
     &8.8087e-01_rb,8.1424e-01_rb,1.4396e-01_rb/)
      kao(:, 4, 9, 7) = (/ &
     &8.9022e-01_rb,9.3422e-01_rb,1.0109e+00_rb,1.0366e+00_rb,1.0048e+00_rb,9.5511e-01_rb, &
     &8.7539e-01_rb,8.5432e-01_rb,1.0430e-01_rb/)
      kao(:, 5, 9, 7) = (/ &
     &8.6877e-01_rb,9.2050e-01_rb,1.0000e+00_rb,1.0116e+00_rb,9.8826e-01_rb,9.4009e-01_rb, &
     &8.7922e-01_rb,9.0156e-01_rb,6.8077e-02_rb/)
      kao(:, 1,10, 7) = (/ &
     &9.2527e-01_rb,9.7705e-01_rb,1.0325e+00_rb,1.0578e+00_rb,1.0149e+00_rb,9.3478e-01_rb, &
     &8.1724e-01_rb,6.6415e-01_rb,2.3891e-01_rb/)
      kao(:, 2,10, 7) = (/ &
     &8.6097e-01_rb,9.1106e-01_rb,9.9506e-01_rb,1.0007e+00_rb,9.6610e-01_rb,8.9639e-01_rb, &
     &7.8861e-01_rb,6.7596e-01_rb,1.7815e-01_rb/)
      kao(:, 3,10, 7) = (/ &
     &8.1181e-01_rb,8.7070e-01_rb,9.5033e-01_rb,9.5476e-01_rb,9.2168e-01_rb,8.6431e-01_rb, &
     &7.7032e-01_rb,7.0318e-01_rb,1.3199e-01_rb/)
      kao(:, 4,10, 7) = (/ &
     &7.7887e-01_rb,8.4520e-01_rb,9.2163e-01_rb,9.1956e-01_rb,8.9130e-01_rb,8.3659e-01_rb, &
     &7.6501e-01_rb,7.4145e-01_rb,9.1831e-02_rb/)
      kao(:, 5,10, 7) = (/ &
     &7.5933e-01_rb,8.3227e-01_rb,9.0273e-01_rb,9.0188e-01_rb,8.7623e-01_rb,8.2773e-01_rb, &
     &7.7824e-01_rb,7.8260e-01_rb,6.3171e-02_rb/)
      kao(:, 1,11, 7) = (/ &
     &7.8226e-01_rb,8.4719e-01_rb,9.2612e-01_rb,9.1570e-01_rb,8.7421e-01_rb,8.0479e-01_rb, &
     &6.9799e-01_rb,5.7039e-01_rb,1.8643e-01_rb/)
      kao(:, 2,11, 7) = (/ &
     &7.3229e-01_rb,8.0376e-01_rb,8.7472e-01_rb,8.6879e-01_rb,8.2958e-01_rb,7.6891e-01_rb, &
     &6.7602e-01_rb,5.8714e-01_rb,1.4000e-01_rb/)
      kao(:, 3,11, 7) = (/ &
     &6.9599e-01_rb,7.7478e-01_rb,8.4133e-01_rb,8.3314e-01_rb,7.9919e-01_rb,7.4119e-01_rb, &
     &6.6696e-01_rb,6.1671e-01_rb,1.0181e-01_rb/)
      kao(:, 4,11, 7) = (/ &
     &6.7489e-01_rb,7.6185e-01_rb,8.1367e-01_rb,8.1128e-01_rb,7.7952e-01_rb,7.3228e-01_rb, &
     &6.7358e-01_rb,6.5125e-01_rb,7.0225e-02_rb/)
      kao(:, 5,11, 7) = (/ &
     &6.5905e-01_rb,7.5391e-01_rb,7.9725e-01_rb,7.9623e-01_rb,7.7236e-01_rb,7.3477e-01_rb, &
     &6.9321e-01_rb,6.9245e-01_rb,4.7555e-02_rb/)
      kao(:, 1,12, 7) = (/ &
     &6.7862e-01_rb,7.6075e-01_rb,8.1212e-01_rb,7.9974e-01_rb,7.5616e-01_rb,6.9320e-01_rb, &
     &5.9851e-01_rb,4.9309e-01_rb,1.4949e-01_rb/)
      kao(:, 2,12, 7) = (/ &
     &6.3807e-01_rb,7.3022e-01_rb,7.7190e-01_rb,7.6245e-01_rb,7.2390e-01_rb,6.6782e-01_rb, &
     &5.8563e-01_rb,5.1153e-01_rb,1.1607e-01_rb/)
      kao(:, 3,12, 7) = (/ &
     &6.1294e-01_rb,7.1220e-01_rb,7.4125e-01_rb,7.3342e-01_rb,6.9810e-01_rb,6.4969e-01_rb, &
     &5.8866e-01_rb,5.3877e-01_rb,8.5989e-02_rb/)
      kao(:, 4,12, 7) = (/ &
     &5.9734e-01_rb,7.0046e-01_rb,7.2548e-01_rb,7.1635e-01_rb,6.8868e-01_rb,6.5109e-01_rb, &
     &5.9747e-01_rb,5.7263e-01_rb,5.7849e-02_rb/)
      kao(:, 5,12, 7) = (/ &
     &5.8148e-01_rb,6.8644e-01_rb,7.1487e-01_rb,7.0660e-01_rb,6.8790e-01_rb,6.6368e-01_rb, &
     &6.1903e-01_rb,6.0994e-01_rb,3.8734e-02_rb/)
      kao(:, 1,13, 7) = (/ &
     &6.1552e-01_rb,7.1356e-01_rb,7.3721e-01_rb,7.1646e-01_rb,6.7473e-01_rb,6.1507e-01_rb, &
     &5.2767e-01_rb,4.3311e-01_rb,1.2482e-01_rb/)
      kao(:, 2,13, 7) = (/ &
     &5.8292e-01_rb,6.8409e-01_rb,7.0235e-01_rb,6.8503e-01_rb,6.4503e-01_rb,5.9514e-01_rb, &
     &5.2204e-01_rb,4.4742e-01_rb,9.8236e-02_rb/)
      kao(:, 3,13, 7) = (/ &
     &5.6436e-01_rb,6.6497e-01_rb,6.8075e-01_rb,6.6573e-01_rb,6.3039e-01_rb,5.8718e-01_rb, &
     &5.2651e-01_rb,4.7434e-01_rb,7.2297e-02_rb/)
      kao(:, 4,13, 7) = (/ &
     &5.4811e-01_rb,6.4832e-01_rb,6.6742e-01_rb,6.5348e-01_rb,6.2977e-01_rb,5.9747e-01_rb, &
     &5.3863e-01_rb,5.0502e-01_rb,5.7494e-02_rb/)
      kao(:, 5,13, 7) = (/ &
     &5.3739e-01_rb,6.3448e-01_rb,6.5434e-01_rb,6.4659e-01_rb,6.3825e-01_rb,6.0640e-01_rb, &
     &5.6100e-01_rb,5.4046e-01_rb,3.2048e-02_rb/)
      kao(:, 1, 1, 8) = (/ &
     &1.1660e+01_rb,1.0203e+01_rb,8.7459e+00_rb,7.2889e+00_rb,5.8318e+00_rb,4.3748e+00_rb, &
     &3.7766e+00_rb,3.7739e+00_rb,2.3608e-03_rb/)
      kao(:, 2, 1, 8) = (/ &
     &1.1346e+01_rb,9.9282e+00_rb,8.5104e+00_rb,7.0926e+00_rb,5.6748e+00_rb,4.2570e+00_rb, &
     &3.8721e+00_rb,3.9057e+00_rb,3.1824e-03_rb/)
      kao(:, 3, 1, 8) = (/ &
     &1.1194e+01_rb,9.7951e+00_rb,8.3963e+00_rb,6.9976e+00_rb,5.5988e+00_rb,4.2000e+00_rb, &
     &3.9469e+00_rb,4.0712e+00_rb,4.2520e-03_rb/)
      kao(:, 4, 1, 8) = (/ &
     &1.1228e+01_rb,9.8250e+00_rb,8.4220e+00_rb,7.0189e+00_rb,5.6159e+00_rb,4.2457e+00_rb, &
     &4.0615e+00_rb,4.2140e+00_rb,1.1766e-02_rb/)
      kao(:, 5, 1, 8) = (/ &
     &1.1405e+01_rb,9.9798e+00_rb,8.5545e+00_rb,7.1289e+00_rb,5.7036e+00_rb,4.3747e+00_rb, &
     &4.1857e+00_rb,4.3557e+00_rb,1.5766e-01_rb/)
      kao(:, 1, 2, 8) = (/ &
     &9.6985e+00_rb,8.4867e+00_rb,7.2748e+00_rb,6.0630e+00_rb,4.8511e+00_rb,3.9524e+00_rb, &
     &3.9347e+00_rb,3.7679e+00_rb,7.6708e-02_rb/)
      kao(:, 2, 2, 8) = (/ &
     &9.4032e+00_rb,8.2283e+00_rb,7.0533e+00_rb,5.8784e+00_rb,4.7035e+00_rb,3.9843e+00_rb, &
     &4.0089e+00_rb,3.9489e+00_rb,9.1596e-02_rb/)
      kao(:, 3, 2, 8) = (/ &
     &9.2359e+00_rb,8.0819e+00_rb,6.9278e+00_rb,5.7738e+00_rb,4.6198e+00_rb,4.0563e+00_rb, &
     &4.1174e+00_rb,4.1185e+00_rb,7.3144e-02_rb/)
      kao(:, 4, 2, 8) = (/ &
     &9.2178e+00_rb,8.0660e+00_rb,6.9143e+00_rb,5.7625e+00_rb,4.6107e+00_rb,4.1515e+00_rb, &
     &4.2394e+00_rb,4.2904e+00_rb,1.1674e-01_rb/)
      kao(:, 5, 2, 8) = (/ &
     &9.3781e+00_rb,8.2055e+00_rb,7.0329e+00_rb,5.8605e+00_rb,4.6883e+00_rb,4.2988e+00_rb, &
     &4.3503e+00_rb,4.4505e+00_rb,1.2134e-01_rb/)
      kao(:, 1, 3, 8) = (/ &
     &8.1381e+00_rb,7.1213e+00_rb,6.1045e+00_rb,5.0877e+00_rb,4.0880e+00_rb,4.0519e+00_rb, &
     &3.8139e+00_rb,3.6880e+00_rb,8.7204e-02_rb/)
      kao(:, 2, 3, 8) = (/ &
     &7.8478e+00_rb,6.8673e+00_rb,5.8868e+00_rb,4.9063e+00_rb,4.0625e+00_rb,4.0788e+00_rb, &
     &3.9290e+00_rb,3.8689e+00_rb,6.5887e-02_rb/)
      kao(:, 3, 3, 8) = (/ &
     &7.6555e+00_rb,6.6990e+00_rb,5.7426e+00_rb,4.7861e+00_rb,4.0779e+00_rb,4.1101e+00_rb, &
     &4.0765e+00_rb,4.0574e+00_rb,7.1216e-02_rb/)
      kao(:, 4, 3, 8) = (/ &
     &7.5783e+00_rb,6.6314e+00_rb,5.6846e+00_rb,4.7378e+00_rb,4.1353e+00_rb,4.1781e+00_rb, &
     &4.2207e+00_rb,4.2405e+00_rb,8.4339e-02_rb/)
      kao(:, 5, 3, 8) = (/ &
     &7.6543e+00_rb,6.6980e+00_rb,5.7416e+00_rb,4.7853e+00_rb,4.2544e+00_rb,4.2669e+00_rb, &
     &4.4018e+00_rb,4.4237e+00_rb,7.8612e-02_rb/)
      kao(:, 1, 4, 8) = (/ &
     &6.8589e+00_rb,6.0020e+00_rb,5.1451e+00_rb,4.2882e+00_rb,3.9706e+00_rb,3.8613e+00_rb, &
     &3.5947e+00_rb,3.5300e+00_rb,6.1710e-02_rb/)
      kao(:, 2, 4, 8) = (/ &
     &6.5794e+00_rb,5.7574e+00_rb,4.9355e+00_rb,4.1135e+00_rb,3.9575e+00_rb,3.8878e+00_rb, &
     &3.7477e+00_rb,3.7212e+00_rb,6.0817e-02_rb/)
      kao(:, 3, 4, 8) = (/ &
     &6.3760e+00_rb,5.5795e+00_rb,4.7829e+00_rb,3.9985e+00_rb,3.9783e+00_rb,3.9606e+00_rb, &
     &3.9214e+00_rb,3.9164e+00_rb,6.3574e-02_rb/)
      kao(:, 4, 4, 8) = (/ &
     &6.2600e+00_rb,5.4779e+00_rb,4.6959e+00_rb,4.0045e+00_rb,4.0114e+00_rb,4.0769e+00_rb, &
     &4.1164e+00_rb,4.1060e+00_rb,7.8132e-02_rb/)
      kao(:, 5, 4, 8) = (/ &
     &6.2608e+00_rb,5.4787e+00_rb,4.6966e+00_rb,4.0716e+00_rb,4.0797e+00_rb,4.1992e+00_rb, &
     &4.3140e+00_rb,4.3043e+00_rb,7.5353e-02_rb/)
      kao(:, 1, 5, 8) = (/ &
     &5.7927e+00_rb,5.0691e+00_rb,4.3455e+00_rb,3.8105e+00_rb,3.8237e+00_rb,3.5526e+00_rb, &
     &3.3728e+00_rb,3.3373e+00_rb,4.9756e-02_rb/)
      kao(:, 2, 5, 8) = (/ &
     &5.5277e+00_rb,4.8372e+00_rb,4.1467e+00_rb,3.7691e+00_rb,3.7793e+00_rb,3.6108e+00_rb, &
     &3.5410e+00_rb,3.5220e+00_rb,4.2289e-02_rb/)
      kao(:, 3, 5, 8) = (/ &
     &5.3258e+00_rb,4.6606e+00_rb,3.9953e+00_rb,3.7485e+00_rb,3.7814e+00_rb,3.7044e+00_rb, &
     &3.7302e+00_rb,3.7158e+00_rb,4.7217e-02_rb/)
      kao(:, 4, 5, 8) = (/ &
     &5.1933e+00_rb,4.5445e+00_rb,3.8958e+00_rb,3.7349e+00_rb,3.8276e+00_rb,3.8482e+00_rb, &
     &3.9315e+00_rb,3.9070e+00_rb,5.2805e-02_rb/)
      kao(:, 5, 5, 8) = (/ &
     &5.1489e+00_rb,4.5057e+00_rb,3.8627e+00_rb,3.7753e+00_rb,3.9160e+00_rb,4.0236e+00_rb, &
     &4.1558e+00_rb,4.1173e+00_rb,7.3119e-02_rb/)
      kao(:, 1, 6, 8) = (/ &
     &4.9153e+00_rb,4.3014e+00_rb,3.6874e+00_rb,3.6742e+00_rb,3.4664e+00_rb,3.2352e+00_rb, &
     &3.1382e+00_rb,3.1290e+00_rb,5.3656e-02_rb/)
      kao(:, 2, 6, 8) = (/ &
     &4.6666e+00_rb,4.0837e+00_rb,3.5075e+00_rb,3.5947e+00_rb,3.4390e+00_rb,3.3186e+00_rb, &
     &3.2995e+00_rb,3.2957e+00_rb,3.0716e-02_rb/)
      kao(:, 3, 6, 8) = (/ &
     &4.4714e+00_rb,3.9130e+00_rb,3.4299e+00_rb,3.5379e+00_rb,3.4620e+00_rb,3.4453e+00_rb, &
     &3.4927e+00_rb,3.4789e+00_rb,2.2748e-02_rb/)
      kao(:, 4, 6, 8) = (/ &
     &4.3327e+00_rb,3.7916e+00_rb,3.3931e+00_rb,3.5287e+00_rb,3.5342e+00_rb,3.5995e+00_rb, &
     &3.6934e+00_rb,3.6650e+00_rb,4.0419e-02_rb/)
      kao(:, 5, 6, 8) = (/ &
     &4.2621e+00_rb,3.7298e+00_rb,3.3976e+00_rb,3.5656e+00_rb,3.6513e+00_rb,3.7826e+00_rb, &
     &3.9184e+00_rb,3.8666e+00_rb,1.0762e-01_rb/)
      kao(:, 1, 7, 8) = (/ &
     &4.2315e+00_rb,3.7030e+00_rb,3.3451e+00_rb,3.3493e+00_rb,3.1177e+00_rb,2.9742e+00_rb, &
     &2.9072e+00_rb,2.9068e+00_rb,5.8460e-02_rb/)
      kao(:, 2, 7, 8) = (/ &
     &3.9985e+00_rb,3.4984e+00_rb,3.2669e+00_rb,3.2554e+00_rb,3.1149e+00_rb,3.0526e+00_rb, &
     &3.0595e+00_rb,3.0700e+00_rb,2.7402e-02_rb/)
      kao(:, 3, 7, 8) = (/ &
     &3.8099e+00_rb,3.3338e+00_rb,3.2097e+00_rb,3.2104e+00_rb,3.1587e+00_rb,3.1808e+00_rb, &
     &3.2348e+00_rb,3.2401e+00_rb,1.9004e-02_rb/)
      kao(:, 4, 7, 8) = (/ &
     &3.6700e+00_rb,3.2116e+00_rb,3.1648e+00_rb,3.2052e+00_rb,3.2433e+00_rb,3.3349e+00_rb, &
     &3.4286e+00_rb,3.4160e+00_rb,4.2481e-02_rb/)
      kao(:, 5, 7, 8) = (/ &
     &3.5860e+00_rb,3.1383e+00_rb,3.1473e+00_rb,3.2492e+00_rb,3.3786e+00_rb,3.5157e+00_rb, &
     &3.6429e+00_rb,3.6020e+00_rb,1.5676e-01_rb/)
      kao(:, 1, 8, 8) = (/ &
     &3.7309e+00_rb,3.2652e+00_rb,3.2085e+00_rb,3.0458e+00_rb,2.8667e+00_rb,2.7650e+00_rb, &
     &2.6802e+00_rb,2.6374e+00_rb,6.7482e-02_rb/)
      kao(:, 2, 8, 8) = (/ &
     &3.5088e+00_rb,3.0710e+00_rb,3.0883e+00_rb,2.9531e+00_rb,2.8706e+00_rb,2.8363e+00_rb, &
     &2.8330e+00_rb,2.8092e+00_rb,3.1671e-02_rb/)
      kao(:, 3, 8, 8) = (/ &
     &3.3283e+00_rb,2.9131e+00_rb,2.9801e+00_rb,2.9151e+00_rb,2.9077e+00_rb,2.9395e+00_rb, &
     &2.9913e+00_rb,2.9792e+00_rb,1.9457e-02_rb/)
      kao(:, 4, 8, 8) = (/ &
     &3.1890e+00_rb,2.7912e+00_rb,2.9223e+00_rb,2.9235e+00_rb,2.9935e+00_rb,3.0761e+00_rb, &
     &3.1767e+00_rb,3.1591e+00_rb,4.5502e-02_rb/)
      kao(:, 5, 8, 8) = (/ &
     &3.0966e+00_rb,2.7276e+00_rb,2.9000e+00_rb,2.9842e+00_rb,3.1139e+00_rb,3.2433e+00_rb, &
     &3.3703e+00_rb,3.3444e+00_rb,1.7959e-01_rb/)
      kao(:, 1, 9, 8) = (/ &
     &3.2795e+00_rb,2.8728e+00_rb,2.9171e+00_rb,2.7516e+00_rb,2.6513e+00_rb,2.5188e+00_rb, &
     &2.4080e+00_rb,2.3392e+00_rb,6.9869e-02_rb/)
      kao(:, 2, 9, 8) = (/ &
     &3.0734e+00_rb,2.7076e+00_rb,2.7821e+00_rb,2.6844e+00_rb,2.6368e+00_rb,2.5852e+00_rb, &
     &2.5517e+00_rb,2.5045e+00_rb,4.1359e-02_rb/)
      kao(:, 3, 9, 8) = (/ &
     &2.9006e+00_rb,2.6099e+00_rb,2.6833e+00_rb,2.6568e+00_rb,2.6662e+00_rb,2.6837e+00_rb, &
     &2.7125e+00_rb,2.6712e+00_rb,2.1538e-02_rb/)
      kao(:, 4, 9, 8) = (/ &
     &2.7639e+00_rb,2.5377e+00_rb,2.6242e+00_rb,2.6595e+00_rb,2.7323e+00_rb,2.8096e+00_rb, &
     &2.8901e+00_rb,2.8466e+00_rb,5.4396e-02_rb/)
      kao(:, 5, 9, 8) = (/ &
     &2.6669e+00_rb,2.4856e+00_rb,2.6031e+00_rb,2.7098e+00_rb,2.8344e+00_rb,2.9696e+00_rb, &
     &3.0834e+00_rb,3.0331e+00_rb,2.0077e-01_rb/)
      kao(:, 1,10, 8) = (/ &
     &2.8722e+00_rb,2.6105e+00_rb,2.6085e+00_rb,2.5091e+00_rb,2.4056e+00_rb,2.2547e+00_rb, &
     &2.1348e+00_rb,2.0413e+00_rb,7.1950e-02_rb/)
      kao(:, 2,10, 8) = (/ &
     &2.6764e+00_rb,2.5038e+00_rb,2.4795e+00_rb,2.4482e+00_rb,2.3806e+00_rb,2.3128e+00_rb, &
     &2.2556e+00_rb,2.1912e+00_rb,3.7979e-02_rb/)
      kao(:, 3,10, 8) = (/ &
     &2.5139e+00_rb,2.4083e+00_rb,2.3949e+00_rb,2.4105e+00_rb,2.4135e+00_rb,2.4077e+00_rb, &
     &2.4119e+00_rb,2.3558e+00_rb,2.2438e-02_rb/)
      kao(:, 4,10, 8) = (/ &
     &2.3840e+00_rb,2.3185e+00_rb,2.3481e+00_rb,2.4127e+00_rb,2.4770e+00_rb,2.5357e+00_rb, &
     &2.5771e+00_rb,2.5231e+00_rb,7.0033e-02_rb/)
      kao(:, 5,10, 8) = (/ &
     &2.2922e+00_rb,2.2525e+00_rb,2.3358e+00_rb,2.4514e+00_rb,2.5719e+00_rb,2.6836e+00_rb, &
     &2.7691e+00_rb,2.7055e+00_rb,2.1431e-01_rb/)
      kao(:, 1,11, 8) = (/ &
     &2.4203e+00_rb,2.3366e+00_rb,2.2726e+00_rb,2.2388e+00_rb,2.1243e+00_rb,2.0012e+00_rb, &
     &1.8959e+00_rb,1.8076e+00_rb,5.5769e-02_rb/)
      kao(:, 2,11, 8) = (/ &
     &2.2551e+00_rb,2.2021e+00_rb,2.1879e+00_rb,2.1841e+00_rb,2.1218e+00_rb,2.0691e+00_rb, &
     &2.0223e+00_rb,1.9543e+00_rb,2.8668e-02_rb/)
      kao(:, 3,11, 8) = (/ &
     &2.1166e+00_rb,2.0926e+00_rb,2.1229e+00_rb,2.1590e+00_rb,2.1592e+00_rb,2.1682e+00_rb, &
     &2.1642e+00_rb,2.1075e+00_rb,3.1850e-02_rb/)
      kao(:, 4,11, 8) = (/ &
     &2.0122e+00_rb,2.0159e+00_rb,2.0901e+00_rb,2.1693e+00_rb,2.2337e+00_rb,2.2891e+00_rb, &
     &2.3331e+00_rb,2.2748e+00_rb,1.4071e-01_rb/)
      kao(:, 5,11, 8) = (/ &
     &1.9514e+00_rb,1.9763e+00_rb,2.0989e+00_rb,2.2225e+00_rb,2.3388e+00_rb,2.4447e+00_rb, &
     &2.5215e+00_rb,2.4588e+00_rb,2.8723e-01_rb/)
      kao(:, 1,12, 8) = (/ &
     &2.0683e+00_rb,2.0370e+00_rb,2.0375e+00_rb,1.9798e+00_rb,1.8792e+00_rb,1.7675e+00_rb, &
     &1.6700e+00_rb,1.5868e+00_rb,4.4568e-02_rb/)
      kao(:, 2,12, 8) = (/ &
     &1.9287e+00_rb,1.9214e+00_rb,1.9599e+00_rb,1.9353e+00_rb,1.8869e+00_rb,1.8278e+00_rb, &
     &1.7894e+00_rb,1.7244e+00_rb,2.3573e-02_rb/)
      kao(:, 3,12, 8) = (/ &
     &1.8200e+00_rb,1.8347e+00_rb,1.9076e+00_rb,1.9217e+00_rb,1.9295e+00_rb,1.9250e+00_rb, &
     &1.9263e+00_rb,1.8738e+00_rb,5.2475e-02_rb/)
      kao(:, 4,12, 8) = (/ &
     &1.7432e+00_rb,1.7845e+00_rb,1.8845e+00_rb,1.9488e+00_rb,2.0009e+00_rb,2.0498e+00_rb, &
     &2.0942e+00_rb,2.0382e+00_rb,1.8765e-01_rb/)
      kao(:, 5,12, 8) = (/ &
     &1.7053e+00_rb,1.7760e+00_rb,1.9065e+00_rb,2.0215e+00_rb,2.1153e+00_rb,2.2054e+00_rb, &
     &2.2781e+00_rb,2.2210e+00_rb,3.6587e-01_rb/)
      kao(:, 1,13, 8) = (/ &
     &1.8562e+00_rb,1.8459e+00_rb,1.8637e+00_rb,1.7957e+00_rb,1.6878e+00_rb,1.5797e+00_rb, &
     &1.4757e+00_rb,1.3906e+00_rb,3.6132e-02_rb/)
      kao(:, 2,13, 8) = (/ &
     &1.7399e+00_rb,1.7611e+00_rb,1.7971e+00_rb,1.7529e+00_rb,1.6955e+00_rb,1.6354e+00_rb, &
     &1.5829e+00_rb,1.5250e+00_rb,2.1418e-02_rb/)
      kao(:, 3,13, 8) = (/ &
     &1.6467e+00_rb,1.6945e+00_rb,1.7550e+00_rb,1.7487e+00_rb,1.7380e+00_rb,1.7251e+00_rb, &
     &1.7178e+00_rb,1.6689e+00_rb,6.4309e-02_rb/)
      kao(:, 4,13, 8) = (/ &
     &1.5930e+00_rb,1.6654e+00_rb,1.7520e+00_rb,1.7797e+00_rb,1.8113e+00_rb,1.8460e+00_rb, &
     &1.8831e+00_rb,1.8288e+00_rb,2.0946e-01_rb/)
      kao(:, 5,13, 8) = (/ &
     &1.5695e+00_rb,1.6755e+00_rb,1.7906e+00_rb,1.8612e+00_rb,1.9314e+00_rb,2.0101e+00_rb, &
     &2.0671e+00_rb,2.0074e+00_rb,3.8278e-01_rb/)
      kao(:, 1, 1, 9) = (/ &
     &4.3448e+02_rb,3.8017e+02_rb,3.2586e+02_rb,2.7155e+02_rb,2.1724e+02_rb,1.6293e+02_rb, &
     &1.0862e+02_rb,5.5107e+01_rb,2.7465e-03_rb/)
      kao(:, 2, 1, 9) = (/ &
     &5.8321e+02_rb,5.1031e+02_rb,4.3741e+02_rb,3.6451e+02_rb,2.9161e+02_rb,2.1870e+02_rb, &
     &1.4580e+02_rb,7.3550e+01_rb,3.5822e-03_rb/)
      kao(:, 3, 1, 9) = (/ &
     &7.6101e+02_rb,6.6588e+02_rb,5.7076e+02_rb,4.7563e+02_rb,3.8051e+02_rb,2.8538e+02_rb, &
     &1.9025e+02_rb,9.5593e+01_rb,5.6366e-03_rb/)
      kao(:, 4, 1, 9) = (/ &
     &9.6683e+02_rb,8.4598e+02_rb,7.2512e+02_rb,6.0427e+02_rb,4.8342e+02_rb,3.6256e+02_rb, &
     &2.4171e+02_rb,1.2116e+02_rb,6.3697e-02_rb/)
      kao(:, 5, 1, 9) = (/ &
     &1.1977e+03_rb,1.0480e+03_rb,8.9827e+02_rb,7.4856e+02_rb,5.9885e+02_rb,4.4914e+02_rb, &
     &2.9942e+02_rb,1.4986e+02_rb,2.8050e-01_rb/)
      kao(:, 1, 2, 9) = (/ &
     &3.3233e+02_rb,2.9079e+02_rb,2.4925e+02_rb,2.0770e+02_rb,1.6616e+02_rb,1.2462e+02_rb, &
     &8.3297e+01_rb,4.3020e+01_rb,1.2769e-02_rb/)
      kao(:, 2, 2, 9) = (/ &
     &4.5139e+02_rb,3.9497e+02_rb,3.3854e+02_rb,2.8212e+02_rb,2.2570e+02_rb,1.6927e+02_rb, &
     &1.1303e+02_rb,5.7664e+01_rb,6.8183e-02_rb/)
      kao(:, 3, 2, 9) = (/ &
     &5.9515e+02_rb,5.2076e+02_rb,4.4637e+02_rb,3.7197e+02_rb,2.9758e+02_rb,2.2318e+02_rb, &
     &1.4890e+02_rb,7.5394e+01_rb,1.7070e-01_rb/)
      kao(:, 4, 2, 9) = (/ &
     &7.6179e+02_rb,6.6657e+02_rb,5.7134e+02_rb,4.7612e+02_rb,3.8090e+02_rb,2.8567e+02_rb, &
     &1.9048e+02_rb,9.5968e+01_rb,4.9948e-01_rb/)
      kao(:, 5, 2, 9) = (/ &
     &9.5206e+02_rb,8.3305e+02_rb,7.1404e+02_rb,5.9504e+02_rb,4.7603e+02_rb,3.5702e+02_rb, &
     &2.3802e+02_rb,1.1952e+02_rb,1.1527e+00_rb/)
      kao(:, 1, 3, 9) = (/ &
     &2.3507e+02_rb,2.0568e+02_rb,1.7630e+02_rb,1.4692e+02_rb,1.1753e+02_rb,8.8151e+01_rb, &
     &5.9624e+01_rb,3.1706e+01_rb,5.0754e-02_rb/)
      kao(:, 2, 3, 9) = (/ &
     &3.2531e+02_rb,2.8465e+02_rb,2.4398e+02_rb,2.0332e+02_rb,1.6266e+02_rb,1.2199e+02_rb, &
     &8.2067e+01_rb,4.2715e+01_rb,9.9945e-02_rb/)
      kao(:, 3, 3, 9) = (/ &
     &4.3545e+02_rb,3.8102e+02_rb,3.2659e+02_rb,2.7216e+02_rb,2.1773e+02_rb,1.6330e+02_rb, &
     &1.0943e+02_rb,5.6175e+01_rb,4.3033e-01_rb/)
      kao(:, 4, 3, 9) = (/ &
     &5.6623e+02_rb,4.9545e+02_rb,4.2468e+02_rb,3.5390e+02_rb,2.8312e+02_rb,2.1234e+02_rb, &
     &1.4196e+02_rb,7.2208e+01_rb,1.0055e+00_rb/)
      kao(:, 5, 3, 9) = (/ &
     &7.1637e+02_rb,6.2682e+02_rb,5.3728e+02_rb,4.4773e+02_rb,3.5819e+02_rb,2.6864e+02_rb, &
     &1.7932e+02_rb,9.0659e+01_rb,1.5764e+00_rb/)
      kao(:, 1, 4, 9) = (/ &
     &1.6080e+02_rb,1.4070e+02_rb,1.2060e+02_rb,1.0050e+02_rb,8.0399e+01_rb,6.0779e+01_rb, &
     &4.1842e+01_rb,2.3410e+01_rb,9.5747e-02_rb/)
      kao(:, 2, 4, 9) = (/ &
     &2.2661e+02_rb,1.9829e+02_rb,1.6996e+02_rb,1.4163e+02_rb,1.1331e+02_rb,8.5452e+01_rb, &
     &5.8120e+01_rb,3.1330e+01_rb,8.2203e-01_rb/)
      kao(:, 3, 4, 9) = (/ &
     &3.0940e+02_rb,2.7072e+02_rb,2.3205e+02_rb,1.9338e+02_rb,1.5470e+02_rb,1.1642e+02_rb, &
     &7.8610e+01_rb,4.1331e+01_rb,1.4967e+00_rb/)
      kao(:, 4, 4, 9) = (/ &
     &4.0889e+02_rb,3.5778e+02_rb,3.0667e+02_rb,2.5556e+02_rb,2.0445e+02_rb,1.5361e+02_rb, &
     &1.0323e+02_rb,5.3416e+01_rb,2.0722e+00_rb/)
      kao(:, 5, 4, 9) = (/ &
     &5.2581e+02_rb,4.6008e+02_rb,3.9436e+02_rb,3.2863e+02_rb,2.6291e+02_rb,1.9734e+02_rb, &
     &1.3221e+02_rb,6.7657e+01_rb,2.6260e+00_rb/)
      kao(:, 1, 5, 9) = (/ &
     &1.0850e+02_rb,9.4934e+01_rb,8.1372e+01_rb,6.7810e+01_rb,5.4459e+01_rb,4.1884e+01_rb, &
     &2.9671e+01_rb,1.7923e+01_rb,1.1913e+00_rb/)
      kao(:, 2, 5, 9) = (/ &
     &1.5572e+02_rb,1.3625e+02_rb,1.1679e+02_rb,9.7324e+01_rb,7.8121e+01_rb,5.9526e+01_rb, &
     &4.1277e+01_rb,2.3510e+01_rb,1.9137e+00_rb/)
      kao(:, 3, 5, 9) = (/ &
     &2.1676e+02_rb,1.8967e+02_rb,1.6257e+02_rb,1.3548e+02_rb,1.0864e+02_rb,8.2293e+01_rb, &
     &5.6282e+01_rb,3.0781e+01_rb,2.5565e+00_rb/)
      kao(:, 4, 5, 9) = (/ &
     &2.9183e+02_rb,2.5535e+02_rb,2.1887e+02_rb,1.8239e+02_rb,1.4612e+02_rb,1.1027e+02_rb, &
     &7.4761e+01_rb,3.9788e+01_rb,3.1260e+00_rb/)
      kao(:, 5, 5, 9) = (/ &
     &3.8115e+02_rb,3.3351e+02_rb,2.8587e+02_rb,2.3822e+02_rb,1.9070e+02_rb,1.4356e+02_rb, &
     &9.6772e+01_rb,5.0545e+01_rb,3.5402e+00_rb/)
      kao(:, 1, 6, 9) = (/ &
     &7.1661e+01_rb,6.2704e+01_rb,5.3746e+01_rb,4.4797e+01_rb,3.6644e+01_rb,2.8875e+01_rb, &
     &2.1420e+01_rb,1.4378e+01_rb,2.0773e+00_rb/)
      kao(:, 2, 6, 9) = (/ &
     &1.0465e+02_rb,9.1565e+01_rb,7.8484e+01_rb,6.5455e+01_rb,5.3153e+01_rb,4.1163e+01_rb, &
     &2.9464e+01_rb,1.8199e+01_rb,2.7911e+00_rb/)
      kao(:, 3, 6, 9) = (/ &
     &1.4858e+02_rb,1.3001e+02_rb,1.1144e+02_rb,9.2961e+01_rb,7.5079e+01_rb,5.7480e+01_rb, &
     &4.0178e+01_rb,2.3340e+01_rb,3.3860e+00_rb/)
      kao(:, 4, 6, 9) = (/ &
     &2.0400e+02_rb,1.7850e+02_rb,1.5300e+02_rb,1.2759e+02_rb,1.0268e+02_rb,7.8054e+01_rb, &
     &5.3729e+01_rb,2.9889e+01_rb,3.8146e+00_rb/)
      kao(:, 5, 6, 9) = (/ &
     &2.7127e+02_rb,2.3736e+02_rb,2.0345e+02_rb,1.6959e+02_rb,1.3617e+02_rb,1.0303e+02_rb, &
     &7.0191e+01_rb,3.7883e+01_rb,4.1027e+00_rb/)
      kao(:, 1, 7, 9) = (/ &
     &4.7664e+01_rb,4.1706e+01_rb,3.5748e+01_rb,3.0192e+01_rb,2.5283e+01_rb,2.0687e+01_rb, &
     &1.6356e+01_rb,1.2414e+01_rb,2.7141e+00_rb/)
      kao(:, 2, 7, 9) = (/ &
     &7.0406e+01_rb,6.1605e+01_rb,5.2805e+01_rb,4.4485e+01_rb,3.6675e+01_rb,2.9135e+01_rb, &
     &2.1846e+01_rb,1.4960e+01_rb,3.3740e+00_rb/)
      kao(:, 3, 7, 9) = (/ &
     &1.0189e+02_rb,8.9150e+01_rb,7.6414e+01_rb,6.4182e+01_rb,5.2360e+01_rb,4.0780e+01_rb, &
     &2.9464e+01_rb,1.8559e+01_rb,3.8705e+00_rb/)
      kao(:, 4, 7, 9) = (/ &
     &1.4268e+02_rb,1.2484e+02_rb,1.0701e+02_rb,8.9652e+01_rb,7.2635e+01_rb,5.5858e+01_rb, &
     &3.9357e+01_rb,2.3287e+01_rb,4.2268e+00_rb/)
      kao(:, 5, 7, 9) = (/ &
     &1.9337e+02_rb,1.6920e+02_rb,1.4503e+02_rb,1.2126e+02_rb,9.7796e+01_rb,7.4601e+01_rb, &
     &5.1680e+01_rb,2.9228e+01_rb,4.3853e+00_rb/)
      kao(:, 1, 8, 9) = (/ &
     &3.2619e+01_rb,2.8542e+01_rb,2.4489e+01_rb,2.1251e+01_rb,1.8391e+01_rb,1.5792e+01_rb, &
     &1.3493e+01_rb,1.1316e+01_rb,3.0747e+00_rb/)
      kao(:, 2, 8, 9) = (/ &
     &4.8375e+01_rb,4.2328e+01_rb,3.6375e+01_rb,3.1177e+01_rb,2.6286e+01_rb,2.1635e+01_rb, &
     &1.7234e+01_rb,1.3233e+01_rb,3.6551e+00_rb/)
      kao(:, 3, 8, 9) = (/ &
     &7.1088e+01_rb,6.2202e+01_rb,5.3468e+01_rb,4.5389e+01_rb,3.7595e+01_rb,3.0015e+01_rb, &
     &2.2675e+01_rb,1.5741e+01_rb,4.0898e+00_rb/)
      kao(:, 4, 8, 9) = (/ &
     &1.0151e+02_rb,8.8821e+01_rb,7.6303e+01_rb,6.4370e+01_rb,5.2682e+01_rb,4.1218e+01_rb, &
     &2.9982e+01_rb,1.9170e+01_rb,4.3948e+00_rb/)
      kao(:, 5, 8, 9) = (/ &
     &1.4013e+02_rb,1.2261e+02_rb,1.0526e+02_rb,8.8413e+01_rb,7.1810e+01_rb,5.5432e+01_rb, &
     &3.9307e+01_rb,2.3602e+01_rb,4.4795e+00_rb/)
      kao(:, 1, 9, 9) = (/ &
     &2.2374e+01_rb,1.9574e+01_rb,1.7087e+01_rb,1.5322e+01_rb,1.3873e+01_rb,1.2730e+01_rb, &
     &1.1784e+01_rb,1.0057e+01_rb,3.2920e+00_rb/)
      kao(:, 2, 9, 9) = (/ &
     &3.3006e+01_rb,2.8880e+01_rb,2.5155e+01_rb,2.2057e+01_rb,1.9232e+01_rb,1.6658e+01_rb, &
     &1.4329e+01_rb,1.1850e+01_rb,3.8043e+00_rb/)
      kao(:, 3, 9, 9) = (/ &
     &4.8965e+01_rb,4.2845e+01_rb,3.7181e+01_rb,3.2051e+01_rb,2.7172e+01_rb,2.2508e+01_rb, &
     &1.8078e+01_rb,1.3961e+01_rb,4.2059e+00_rb/)
      kao(:, 4, 9, 9) = (/ &
     &7.1218e+01_rb,6.2316e+01_rb,5.3886e+01_rb,4.5928e+01_rb,3.8193e+01_rb,3.0649e+01_rb, &
     &2.3347e+01_rb,1.6446e+01_rb,4.4539e+00_rb/)
      kao(:, 5, 9, 9) = (/ &
     &1.0013e+02_rb,8.7618e+01_rb,7.5558e+01_rb,6.3912e+01_rb,5.2481e+01_rb,4.1239e+01_rb, &
     &3.0241e+01_rb,1.9662e+01_rb,4.4837e+00_rb/)
      kao(:, 1,10, 9) = (/ &
     &1.6016e+01_rb,1.4014e+01_rb,1.2654e+01_rb,1.1832e+01_rb,1.1333e+01_rb,1.1079e+01_rb, &
     &1.0430e+01_rb,8.9198e+00_rb,3.4165e+00_rb/)
      kao(:, 2,10, 9) = (/ &
     &2.3316e+01_rb,2.0402e+01_rb,1.8216e+01_rb,1.6474e+01_rb,1.5033e+01_rb,1.3817e+01_rb, &
     &1.2667e+01_rb,1.0521e+01_rb,3.8593e+00_rb/)
      kao(:, 3,10, 9) = (/ &
     &3.4676e+01_rb,3.0345e+01_rb,2.6796e+01_rb,2.3607e+01_rb,2.0669e+01_rb,1.7940e+01_rb, &
     &1.5414e+01_rb,1.2426e+01_rb,4.1980e+00_rb/)
      kao(:, 4,10, 9) = (/ &
     &5.1075e+01_rb,4.4715e+01_rb,3.9109e+01_rb,3.3826e+01_rb,2.8757e+01_rb,2.3883e+01_rb, &
     &1.9235e+01_rb,1.4676e+01_rb,4.3830e+00_rb/)
      kao(:, 5,10, 9) = (/ &
     &7.2971e+01_rb,6.3894e+01_rb,5.5508e+01_rb,4.7426e+01_rb,3.9536e+01_rb,3.1844e+01_rb, &
     &2.4374e+01_rb,1.7279e+01_rb,4.3810e+00_rb/)
      kao(:, 1,11, 9) = (/ &
     &1.3577e+01_rb,1.1946e+01_rb,1.1209e+01_rb,1.0837e+01_rb,1.0794e+01_rb,1.0607e+01_rb, &
     &9.9046e+00_rb,8.4158e+00_rb,3.5771e+00_rb/)
      kao(:, 2,11, 9) = (/ &
     &1.9792e+01_rb,1.7445e+01_rb,1.5940e+01_rb,1.4785e+01_rb,1.3920e+01_rb,1.3199e+01_rb, &
     &1.2097e+01_rb,9.9522e+00_rb,3.9185e+00_rb/)
      kao(:, 3,11, 9) = (/ &
     &2.9476e+01_rb,2.5957e+01_rb,2.3237e+01_rb,2.0843e+01_rb,1.8688e+01_rb,1.6684e+01_rb, &
     &1.4791e+01_rb,1.1766e+01_rb,4.1553e+00_rb/)
      kao(:, 4,11, 9) = (/ &
     &4.3413e+01_rb,3.8176e+01_rb,3.3695e+01_rb,2.9502e+01_rb,2.5515e+01_rb,2.1698e+01_rb, &
     &1.8077e+01_rb,1.3908e+01_rb,4.1942e+00_rb/)
      kao(:, 5,11, 9) = (/ &
     &6.2007e+01_rb,5.4458e+01_rb,4.7606e+01_rb,4.1020e+01_rb,3.4622e+01_rb,2.8398e+01_rb, &
     &2.2392e+01_rb,1.6392e+01_rb,4.1692e+00_rb/)
      kao(:, 1,12, 9) = (/ &
     &1.1680e+01_rb,1.0479e+01_rb,1.0122e+01_rb,1.0167e+01_rb,1.0289e+01_rb,1.0072e+01_rb, &
     &9.3417e+00_rb,7.9079e+00_rb,3.5200e+00_rb/)
      kao(:, 2,12, 9) = (/ &
     &1.7053e+01_rb,1.5231e+01_rb,1.4215e+01_rb,1.3566e+01_rb,1.3123e+01_rb,1.2618e+01_rb, &
     &1.1431e+01_rb,9.3416e+00_rb,3.7828e+00_rb/)
      kao(:, 3,12, 9) = (/ &
     &2.5404e+01_rb,2.2573e+01_rb,2.0511e+01_rb,1.8777e+01_rb,1.7216e+01_rb,1.5812e+01_rb, &
     &1.4031e+01_rb,1.1064e+01_rb,3.9374e+00_rb/)
      kao(:, 4,12, 9) = (/ &
     &3.7431e+01_rb,3.3112e+01_rb,2.9529e+01_rb,2.6224e+01_rb,2.3088e+01_rb,2.0100e+01_rb, &
     &1.7184e+01_rb,1.3093e+01_rb,3.9253e+00_rb/)
      kao(:, 5,12, 9) = (/ &
     &5.3474e+01_rb,4.7145e+01_rb,4.1510e+01_rb,3.6119e+01_rb,3.0908e+01_rb,2.5843e+01_rb, &
     &2.0970e+01_rb,1.5444e+01_rb,3.9004e+00_rb/)
      kao(:, 1,13, 9) = (/ &
     &1.0438e+01_rb,9.5597e+00_rb,9.5087e+00_rb,9.7911e+00_rb,9.8820e+00_rb,9.5875e+00_rb, &
     &8.8692e+00_rb,7.4811e+00_rb,3.3069e+00_rb/)
      kao(:, 2,13, 9) = (/ &
     &1.5260e+01_rb,1.3818e+01_rb,1.3178e+01_rb,1.2874e+01_rb,1.2679e+01_rb,1.2091e+01_rb, &
     &1.0875e+01_rb,8.8312e+00_rb,3.5228e+00_rb/)
      kao(:, 3,13, 9) = (/ &
     &2.2789e+01_rb,2.0434e+01_rb,1.8837e+01_rb,1.7547e+01_rb,1.6403e+01_rb,1.5263e+01_rb, &
     &1.3389e+01_rb,1.0463e+01_rb,3.6351e+00_rb/)
      kao(:, 4,13, 9) = (/ &
     &3.3586e+01_rb,2.9891e+01_rb,2.6912e+01_rb,2.4218e+01_rb,2.1653e+01_rb,1.9218e+01_rb, &
     &1.6457e+01_rb,1.2402e+01_rb,3.6180e+00_rb/)
      kao(:, 5,13, 9) = (/ &
     &4.7989e+01_rb,4.2477e+01_rb,3.7650e+01_rb,3.3082e+01_rb,2.8647e+01_rb,2.4351e+01_rb, &
     &2.0129e+01_rb,1.4659e+01_rb,3.6174e+00_rb/)
      kao(:, 1, 1,10) = (/ &
     &2.6405e+03_rb,2.3104e+03_rb,1.9803e+03_rb,1.6503e+03_rb,1.3202e+03_rb,9.9017e+02_rb, &
     &6.6011e+02_rb,3.3006e+02_rb,2.0071e-03_rb/)
      kao(:, 2, 1,10) = (/ &
     &3.3430e+03_rb,2.9251e+03_rb,2.5072e+03_rb,2.0894e+03_rb,1.6715e+03_rb,1.2536e+03_rb, &
     &8.3575e+02_rb,4.1787e+02_rb,3.1756e-03_rb/)
      kao(:, 3, 1,10) = (/ &
     &4.1007e+03_rb,3.5881e+03_rb,3.0756e+03_rb,2.5630e+03_rb,2.0504e+03_rb,1.5378e+03_rb, &
     &1.0252e+03_rb,5.1259e+02_rb,1.5257e-02_rb/)
      kao(:, 4, 1,10) = (/ &
     &4.9110e+03_rb,4.2972e+03_rb,3.6833e+03_rb,3.0694e+03_rb,2.4555e+03_rb,1.8416e+03_rb, &
     &1.2278e+03_rb,6.1388e+02_rb,1.9623e-01_rb/)
      kao(:, 5, 1,10) = (/ &
     &5.7642e+03_rb,5.0437e+03_rb,4.3232e+03_rb,3.6026e+03_rb,2.8821e+03_rb,2.1616e+03_rb, &
     &1.4411e+03_rb,7.2053e+02_rb,3.1630e-01_rb/)
      kao(:, 1, 2,10) = (/ &
     &2.0643e+03_rb,1.8062e+03_rb,1.5482e+03_rb,1.2902e+03_rb,1.0321e+03_rb,7.7410e+02_rb, &
     &5.1607e+02_rb,2.5804e+02_rb,7.9610e-02_rb/)
      kao(:, 2, 2,10) = (/ &
     &2.6419e+03_rb,2.3117e+03_rb,1.9815e+03_rb,1.6512e+03_rb,1.3210e+03_rb,9.9073e+02_rb, &
     &6.6049e+02_rb,3.3024e+02_rb,3.1756e-01_rb/)
      kao(:, 3, 2,10) = (/ &
     &3.2679e+03_rb,2.8594e+03_rb,2.4509e+03_rb,2.0424e+03_rb,1.6340e+03_rb,1.2255e+03_rb, &
     &8.1698e+02_rb,4.0849e+02_rb,1.6019e+00_rb/)
      kao(:, 4, 2,10) = (/ &
     &3.9535e+03_rb,3.4593e+03_rb,2.9652e+03_rb,2.4710e+03_rb,1.9768e+03_rb,1.4826e+03_rb, &
     &9.8838e+02_rb,4.9419e+02_rb,2.9545e+00_rb/)
      kao(:, 5, 2,10) = (/ &
     &4.6801e+03_rb,4.0951e+03_rb,3.5101e+03_rb,2.9251e+03_rb,2.3401e+03_rb,1.7551e+03_rb, &
     &1.1700e+03_rb,5.8502e+02_rb,4.1449e+00_rb/)
      kao(:, 1, 3,10) = (/ &
     &1.5076e+03_rb,1.3191e+03_rb,1.1307e+03_rb,9.4224e+02_rb,7.5379e+02_rb,5.6534e+02_rb, &
     &3.7689e+02_rb,1.8845e+02_rb,5.0945e-01_rb/)
      kao(:, 2, 3,10) = (/ &
     &1.9623e+03_rb,1.7170e+03_rb,1.4718e+03_rb,1.2265e+03_rb,9.8117e+02_rb,7.3588e+02_rb, &
     &4.9059e+02_rb,2.4529e+02_rb,2.4679e+00_rb/)
      kao(:, 3, 3,10) = (/ &
     &2.4731e+03_rb,2.1640e+03_rb,1.8549e+03_rb,1.5457e+03_rb,1.2366e+03_rb,9.2743e+02_rb, &
     &6.1829e+02_rb,3.0914e+02_rb,5.8082e+00_rb/)
      kao(:, 4, 3,10) = (/ &
     &3.0330e+03_rb,2.6538e+03_rb,2.2747e+03_rb,1.8956e+03_rb,1.5165e+03_rb,1.1374e+03_rb, &
     &7.5824e+02_rb,3.7912e+02_rb,6.6677e+00_rb/)
      kao(:, 5, 3,10) = (/ &
     &3.6345e+03_rb,3.1802e+03_rb,2.7259e+03_rb,2.2715e+03_rb,1.8172e+03_rb,1.3629e+03_rb, &
     &9.0862e+02_rb,4.5431e+02_rb,6.6949e+00_rb/)
      kao(:, 1, 4,10) = (/ &
     &1.0627e+03_rb,9.2986e+02_rb,7.9702e+02_rb,6.6419e+02_rb,5.3135e+02_rb,3.9851e+02_rb, &
     &2.6568e+02_rb,1.3284e+02_rb,8.6599e+00_rb/)
      kao(:, 2, 4,10) = (/ &
     &1.4166e+03_rb,1.2395e+03_rb,1.0624e+03_rb,8.8537e+02_rb,7.0829e+02_rb,5.3122e+02_rb, &
     &3.5415e+02_rb,1.7707e+02_rb,9.4746e+00_rb/)
      kao(:, 3, 4,10) = (/ &
     &1.8198e+03_rb,1.5923e+03_rb,1.3648e+03_rb,1.1374e+03_rb,9.0989e+02_rb,6.8241e+02_rb, &
     &4.5494e+02_rb,2.2747e+02_rb,9.4272e+00_rb/)
      kao(:, 4, 4,10) = (/ &
     &2.2704e+03_rb,1.9866e+03_rb,1.7028e+03_rb,1.4190e+03_rb,1.1352e+03_rb,8.5140e+02_rb, &
     &5.6760e+02_rb,2.8380e+02_rb,9.3833e+00_rb/)
      kao(:, 5, 4,10) = (/ &
     &2.7615e+03_rb,2.4164e+03_rb,2.0712e+03_rb,1.7260e+03_rb,1.3808e+03_rb,1.0356e+03_rb, &
     &6.9039e+02_rb,3.4519e+02_rb,9.3214e+00_rb/)
      kao(:, 1, 5,10) = (/ &
     &7.3556e+02_rb,6.4362e+02_rb,5.5167e+02_rb,4.5973e+02_rb,3.6778e+02_rb,2.7584e+02_rb, &
     &1.8389e+02_rb,9.1946e+01_rb,1.0345e+01_rb/)
      kao(:, 2, 5,10) = (/ &
     &1.0082e+03_rb,8.8219e+02_rb,7.5616e+02_rb,6.3013e+02_rb,5.0411e+02_rb,3.7808e+02_rb, &
     &2.5205e+02_rb,1.2603e+02_rb,1.0318e+01_rb/)
      kao(:, 3, 5,10) = (/ &
     &1.3203e+03_rb,1.1552e+03_rb,9.9020e+02_rb,8.2516e+02_rb,6.6013e+02_rb,4.9510e+02_rb, &
     &3.3007e+02_rb,1.6503e+02_rb,1.0269e+01_rb/)
      kao(:, 4, 5,10) = (/ &
     &1.6790e+03_rb,1.4691e+03_rb,1.2593e+03_rb,1.0494e+03_rb,8.3951e+02_rb,6.2963e+02_rb, &
     &4.1976e+02_rb,2.0988e+02_rb,1.0211e+01_rb/)
      kao(:, 5, 5,10) = (/ &
     &2.0720e+03_rb,1.8130e+03_rb,1.5540e+03_rb,1.2950e+03_rb,1.0360e+03_rb,7.7699e+02_rb, &
     &5.1799e+02_rb,2.5900e+02_rb,1.0128e+01_rb/)
      kao(:, 1, 6,10) = (/ &
     &4.9443e+02_rb,4.3263e+02_rb,3.7082e+02_rb,3.0902e+02_rb,2.4721e+02_rb,1.8541e+02_rb, &
     &1.2361e+02_rb,6.1804e+01_rb,1.1088e+01_rb/)
      kao(:, 2, 6,10) = (/ &
     &6.9842e+02_rb,6.1111e+02_rb,5.2381e+02_rb,4.3651e+02_rb,3.4921e+02_rb,2.6191e+02_rb, &
     &1.7460e+02_rb,8.7302e+01_rb,1.1041e+01_rb/)
      kao(:, 3, 6,10) = (/ &
     &9.3821e+02_rb,8.2094e+02_rb,7.0366e+02_rb,5.8638e+02_rb,4.6911e+02_rb,3.5183e+02_rb, &
     &2.3455e+02_rb,1.1728e+02_rb,1.0988e+01_rb/)
      kao(:, 4, 6,10) = (/ &
     &1.2155e+03_rb,1.0636e+03_rb,9.1165e+02_rb,7.5971e+02_rb,6.0777e+02_rb,4.5582e+02_rb, &
     &3.0388e+02_rb,1.5194e+02_rb,1.0913e+01_rb/)
      kao(:, 5, 6,10) = (/ &
     &1.5286e+03_rb,1.3375e+03_rb,1.1464e+03_rb,9.5536e+02_rb,7.6429e+02_rb,5.7321e+02_rb, &
     &3.8214e+02_rb,1.9107e+02_rb,1.0808e+01_rb/)
      kao(:, 1, 7,10) = (/ &
     &3.2977e+02_rb,2.8854e+02_rb,2.4732e+02_rb,2.0610e+02_rb,1.6488e+02_rb,1.2366e+02_rb, &
     &8.2441e+01_rb,4.1221e+01_rb,1.1566e+01_rb/)
      kao(:, 2, 7,10) = (/ &
     &4.8209e+02_rb,4.2183e+02_rb,3.6157e+02_rb,3.0131e+02_rb,2.4104e+02_rb,1.8078e+02_rb, &
     &1.2052e+02_rb,6.0261e+01_rb,1.1564e+01_rb/)
      kao(:, 3, 7,10) = (/ &
     &6.6530e+02_rb,5.8214e+02_rb,4.9898e+02_rb,4.1581e+02_rb,3.3265e+02_rb,2.4949e+02_rb, &
     &1.6632e+02_rb,8.3163e+01_rb,1.1535e+01_rb/)
      kao(:, 4, 7,10) = (/ &
     &8.8049e+02_rb,7.7043e+02_rb,6.6036e+02_rb,5.5030e+02_rb,4.4024e+02_rb,3.3018e+02_rb, &
     &2.2012e+02_rb,1.1006e+02_rb,1.1437e+01_rb/)
      kao(:, 5, 7,10) = (/ &
     &1.1278e+03_rb,9.8684e+02_rb,8.4586e+02_rb,7.0488e+02_rb,5.6391e+02_rb,4.2293e+02_rb, &
     &2.8195e+02_rb,1.4098e+02_rb,1.1316e+01_rb/)
      kao(:, 1, 8,10) = (/ &
     &2.2204e+02_rb,1.9428e+02_rb,1.6653e+02_rb,1.3877e+02_rb,1.1102e+02_rb,8.3264e+01_rb, &
     &5.5510e+01_rb,3.0059e+01_rb,1.1861e+01_rb/)
      kao(:, 2, 8,10) = (/ &
     &3.3638e+02_rb,2.9433e+02_rb,2.5228e+02_rb,2.1024e+02_rb,1.6819e+02_rb,1.2614e+02_rb, &
     &8.4095e+01_rb,4.2076e+01_rb,1.1785e+01_rb/)
      kao(:, 3, 8,10) = (/ &
     &4.7771e+02_rb,4.1800e+02_rb,3.5828e+02_rb,2.9857e+02_rb,2.3886e+02_rb,1.7914e+02_rb, &
     &1.1943e+02_rb,5.9714e+01_rb,1.1693e+01_rb/)
      kao(:, 4, 8,10) = (/ &
     &6.4744e+02_rb,5.6651e+02_rb,4.8558e+02_rb,4.0465e+02_rb,3.2372e+02_rb,2.4279e+02_rb, &
     &1.6186e+02_rb,8.0931e+01_rb,1.1572e+01_rb/)
      kao(:, 5, 8,10) = (/ &
     &8.4463e+02_rb,7.3905e+02_rb,6.3347e+02_rb,5.2789e+02_rb,4.2232e+02_rb,3.1674e+02_rb, &
     &2.1116e+02_rb,1.0558e+02_rb,1.1447e+01_rb/)
      kao(:, 1, 9,10) = (/ &
     &1.4633e+02_rb,1.2804e+02_rb,1.0975e+02_rb,9.1455e+01_rb,7.3164e+01_rb,5.4873e+01_rb, &
     &3.7228e+01_rb,2.6999e+01_rb,1.1829e+01_rb/)
      kao(:, 2, 9,10) = (/ &
     &2.2961e+02_rb,2.0091e+02_rb,1.7220e+02_rb,1.4350e+02_rb,1.1480e+02_rb,8.6102e+01_rb, &
     &5.7402e+01_rb,3.3291e+01_rb,1.1753e+01_rb/)
      kao(:, 3, 9,10) = (/ &
     &3.3705e+02_rb,2.9492e+02_rb,2.5279e+02_rb,2.1066e+02_rb,1.6852e+02_rb,1.2639e+02_rb, &
     &8.4263e+01_rb,4.2726e+01_rb,1.1656e+01_rb/)
      kao(:, 4, 9,10) = (/ &
     &4.6791e+02_rb,4.0942e+02_rb,3.5094e+02_rb,2.9245e+02_rb,2.3396e+02_rb,1.7547e+02_rb, &
     &1.1698e+02_rb,5.8490e+01_rb,1.1546e+01_rb/)
      kao(:, 5, 9,10) = (/ &
     &6.2380e+02_rb,5.4582e+02_rb,4.6785e+02_rb,3.8987e+02_rb,3.1190e+02_rb,2.3392e+02_rb, &
     &1.5595e+02_rb,7.7975e+01_rb,1.1440e+01_rb/)
      kao(:, 1,10,10) = (/ &
     &9.9121e+01_rb,8.6731e+01_rb,7.4340e+01_rb,6.1950e+01_rb,4.9560e+01_rb,3.7719e+01_rb, &
     &3.0913e+01_rb,2.3571e+01_rb,1.2025e+01_rb/)
      kao(:, 2,10,10) = (/ &
     &1.5969e+02_rb,1.3973e+02_rb,1.1977e+02_rb,9.9804e+01_rb,7.9844e+01_rb,5.9883e+01_rb, &
     &4.1267e+01_rb,2.9611e+01_rb,1.1959e+01_rb/)
      kao(:, 3,10,10) = (/ &
     &2.4182e+02_rb,2.1160e+02_rb,1.8137e+02_rb,1.5114e+02_rb,1.2091e+02_rb,9.0684e+01_rb, &
     &6.0456e+01_rb,3.6780e+01_rb,1.1864e+01_rb/)
      kao(:, 4,10,10) = (/ &
     &3.4325e+02_rb,3.0034e+02_rb,2.5744e+02_rb,2.1453e+02_rb,1.7163e+02_rb,1.2872e+02_rb, &
     &8.5813e+01_rb,4.5301e+01_rb,1.1768e+01_rb/)
      kao(:, 5,10,10) = (/ &
     &4.6649e+02_rb,4.0818e+02_rb,3.4987e+02_rb,2.9156e+02_rb,2.3325e+02_rb,1.7493e+02_rb, &
     &1.1662e+02_rb,5.8448e+01_rb,1.1666e+01_rb/)
      kao(:, 1,11,10) = (/ &
     &8.4453e+01_rb,7.3896e+01_rb,6.3339e+01_rb,5.2783e+01_rb,4.2282e+01_rb,3.5008e+01_rb, &
     &3.0468e+01_rb,2.2795e+01_rb,1.2480e+01_rb/)
      kao(:, 2,11,10) = (/ &
     &1.3606e+02_rb,1.1905e+02_rb,1.0204e+02_rb,8.5035e+01_rb,6.8028e+01_rb,5.1171e+01_rb, &
     &3.9405e+01_rb,2.8346e+01_rb,1.2418e+01_rb/)
      kao(:, 3,11,10) = (/ &
     &2.0562e+02_rb,1.7991e+02_rb,1.5421e+02_rb,1.2851e+02_rb,1.0281e+02_rb,7.7106e+01_rb, &
     &5.2236e+01_rb,3.5283e+01_rb,1.2333e+01_rb/)
      kao(:, 4,11,10) = (/ &
     &2.9197e+02_rb,2.5547e+02_rb,2.1898e+02_rb,1.8248e+02_rb,1.4598e+02_rb,1.0949e+02_rb, &
     &7.2992e+01_rb,4.3383e+01_rb,1.2243e+01_rb/)
      kao(:, 5,11,10) = (/ &
     &3.9645e+02_rb,3.4690e+02_rb,2.9734e+02_rb,2.4778e+02_rb,1.9823e+02_rb,1.4867e+02_rb, &
     &9.9113e+01_rb,5.2639e+01_rb,1.2132e+01_rb/)
      kao(:, 1,12,10) = (/ &
     &7.3008e+01_rb,6.3882e+01_rb,5.4756e+01_rb,4.5630e+01_rb,3.8097e+01_rb,3.4292e+01_rb, &
     &2.9157e+01_rb,2.1820e+01_rb,1.2784e+01_rb/)
      kao(:, 2,12,10) = (/ &
     &1.1777e+02_rb,1.0304e+02_rb,8.8324e+01_rb,7.3604e+01_rb,5.8883e+01_rb,4.6283e+01_rb, &
     &3.8218e+01_rb,2.7110e+01_rb,1.2693e+01_rb/)
      kao(:, 3,12,10) = (/ &
     &1.7754e+02_rb,1.5535e+02_rb,1.3316e+02_rb,1.1096e+02_rb,8.8771e+01_rb,6.6578e+01_rb, &
     &4.9180e+01_rb,3.3603e+01_rb,1.2566e+01_rb/)
      kao(:, 4,12,10) = (/ &
     &2.5172e+02_rb,2.2026e+02_rb,1.8879e+02_rb,1.5733e+02_rb,1.2586e+02_rb,9.4396e+01_rb, &
     &6.3855e+01_rb,4.1285e+01_rb,1.2431e+01_rb/)
      kao(:, 5,12,10) = (/ &
     &3.4203e+02_rb,2.9928e+02_rb,2.5652e+02_rb,2.1377e+02_rb,1.7102e+02_rb,1.2826e+02_rb, &
     &8.5508e+01_rb,5.0087e+01_rb,1.2252e+01_rb/)
      kao(:, 1,13,10) = (/ &
     &6.5744e+01_rb,5.7526e+01_rb,4.9308e+01_rb,4.1412e+01_rb,3.7107e+01_rb,3.3686e+01_rb, &
     &2.7959e+01_rb,2.0954e+01_rb,1.2543e+01_rb/)
      kao(:, 2,13,10) = (/ &
     &1.0592e+02_rb,9.2679e+01_rb,7.9440e+01_rb,6.6200e+01_rb,5.3492e+01_rb,4.5296e+01_rb, &
     &3.7012e+01_rb,2.6073e+01_rb,1.2385e+01_rb/)
      kao(:, 3,13,10) = (/ &
     &1.5952e+02_rb,1.3958e+02_rb,1.1964e+02_rb,9.9699e+01_rb,7.9759e+01_rb,6.0961e+01_rb, &
     &4.7985e+01_rb,3.2430e+01_rb,1.2250e+01_rb/)
      kao(:, 4,13,10) = (/ &
     &2.2626e+02_rb,1.9798e+02_rb,1.6969e+02_rb,1.4141e+02_rb,1.1313e+02_rb,8.4848e+01_rb, &
     &6.0965e+01_rb,3.9929e+01_rb,1.2100e+01_rb/)
      kao(:, 5,13,10) = (/ &
     &3.0736e+02_rb,2.6894e+02_rb,2.3052e+02_rb,1.9210e+02_rb,1.5368e+02_rb,1.1526e+02_rb, &
     &7.7678e+01_rb,4.8494e+01_rb,1.1980e+01_rb/)
      kao(:, 1, 1,11) = (/ &
     &3.9010e+03_rb,3.4134e+03_rb,2.9257e+03_rb,2.4381e+03_rb,1.9505e+03_rb,1.4629e+03_rb, &
     &9.7525e+02_rb,4.8762e+02_rb,2.5641e-03_rb/)
      kao(:, 2, 1,11) = (/ &
     &4.8941e+03_rb,4.2823e+03_rb,3.6706e+03_rb,3.0588e+03_rb,2.4471e+03_rb,1.8353e+03_rb, &
     &1.2235e+03_rb,6.1176e+02_rb,4.7147e-03_rb/)
      kao(:, 3, 1,11) = (/ &
     &6.0244e+03_rb,5.2713e+03_rb,4.5183e+03_rb,3.7652e+03_rb,3.0122e+03_rb,2.2591e+03_rb, &
     &1.5061e+03_rb,7.5305e+02_rb,1.8229e-02_rb/)
      kao(:, 4, 1,11) = (/ &
     &7.2726e+03_rb,6.3635e+03_rb,5.4544e+03_rb,4.5454e+03_rb,3.6363e+03_rb,2.7272e+03_rb, &
     &1.8181e+03_rb,9.0907e+02_rb,4.1403e-02_rb/)
      kao(:, 5, 1,11) = (/ &
     &8.6200e+03_rb,7.5425e+03_rb,6.4650e+03_rb,5.3875e+03_rb,4.3100e+03_rb,3.2325e+03_rb, &
     &2.1550e+03_rb,1.0775e+03_rb,7.0391e-01_rb/)
      kao(:, 1, 2,11) = (/ &
     &3.0926e+03_rb,2.7060e+03_rb,2.3194e+03_rb,1.9329e+03_rb,1.5463e+03_rb,1.1597e+03_rb, &
     &7.7315e+02_rb,3.8657e+02_rb,1.4758e-01_rb/)
      kao(:, 2, 2,11) = (/ &
     &3.9512e+03_rb,3.4573e+03_rb,2.9634e+03_rb,2.4695e+03_rb,1.9756e+03_rb,1.4817e+03_rb, &
     &9.8779e+02_rb,4.9390e+02_rb,7.5464e-01_rb/)
      kao(:, 3, 2,11) = (/ &
     &4.9225e+03_rb,4.3072e+03_rb,3.6919e+03_rb,3.0766e+03_rb,2.4613e+03_rb,1.8460e+03_rb, &
     &1.2306e+03_rb,6.1532e+02_rb,2.2469e+00_rb/)
      kao(:, 4, 2,11) = (/ &
     &6.0042e+03_rb,5.2537e+03_rb,4.5032e+03_rb,3.7526e+03_rb,3.0021e+03_rb,2.2516e+03_rb, &
     &1.5011e+03_rb,7.5053e+02_rb,4.5134e+00_rb/)
      kao(:, 5, 2,11) = (/ &
     &7.1983e+03_rb,6.2985e+03_rb,5.3987e+03_rb,4.4989e+03_rb,3.5991e+03_rb,2.6994e+03_rb, &
     &1.7996e+03_rb,8.9979e+02_rb,5.4294e+00_rb/)
      kao(:, 1, 3,11) = (/ &
     &2.3009e+03_rb,2.0133e+03_rb,1.7257e+03_rb,1.4381e+03_rb,1.1505e+03_rb,8.6285e+02_rb, &
     &5.7523e+02_rb,2.8762e+02_rb,4.0343e+00_rb/)
      kao(:, 2, 3,11) = (/ &
     &2.9916e+03_rb,2.6177e+03_rb,2.2437e+03_rb,1.8698e+03_rb,1.4958e+03_rb,1.1219e+03_rb, &
     &7.4790e+02_rb,3.7395e+02_rb,9.4960e+00_rb/)
      kao(:, 3, 3,11) = (/ &
     &3.7957e+03_rb,3.3213e+03_rb,2.8468e+03_rb,2.3723e+03_rb,1.8979e+03_rb,1.4234e+03_rb, &
     &9.4893e+02_rb,4.7447e+02_rb,9.8961e+00_rb/)
      kao(:, 4, 3,11) = (/ &
     &4.7090e+03_rb,4.1204e+03_rb,3.5318e+03_rb,2.9432e+03_rb,2.3545e+03_rb,1.7659e+03_rb, &
     &1.1773e+03_rb,5.8863e+02_rb,9.8448e+00_rb/)
      kao(:, 5, 3,11) = (/ &
     &5.7119e+03_rb,4.9979e+03_rb,4.2839e+03_rb,3.5699e+03_rb,2.8560e+03_rb,2.1420e+03_rb, &
     &1.4280e+03_rb,7.1399e+02_rb,9.5434e+00_rb/)
      kao(:, 1, 4,11) = (/ &
     &1.6459e+03_rb,1.4402e+03_rb,1.2345e+03_rb,1.0287e+03_rb,8.2297e+02_rb,6.1723e+02_rb, &
     &4.1148e+02_rb,2.0574e+02_rb,1.1261e+01_rb/)
      kao(:, 2, 4,11) = (/ &
     &2.1944e+03_rb,1.9201e+03_rb,1.6458e+03_rb,1.3715e+03_rb,1.0972e+03_rb,8.2292e+02_rb, &
     &5.4861e+02_rb,2.7431e+02_rb,1.1246e+01_rb/)
      kao(:, 3, 4,11) = (/ &
     &2.8415e+03_rb,2.4863e+03_rb,2.1311e+03_rb,1.7759e+03_rb,1.4207e+03_rb,1.0656e+03_rb, &
     &7.1037e+02_rb,3.5519e+02_rb,1.1203e+01_rb/)
      kao(:, 4, 4,11) = (/ &
     &3.5819e+03_rb,3.1342e+03_rb,2.6865e+03_rb,2.2387e+03_rb,1.7910e+03_rb,1.3432e+03_rb, &
     &8.9549e+02_rb,4.4774e+02_rb,1.1137e+01_rb/)
      kao(:, 5, 4,11) = (/ &
     &4.4164e+03_rb,3.8643e+03_rb,3.3123e+03_rb,2.7602e+03_rb,2.2082e+03_rb,1.6561e+03_rb, &
     &1.1041e+03_rb,5.5205e+02_rb,1.1062e+01_rb/)
      kao(:, 1, 5,11) = (/ &
     &1.1573e+03_rb,1.0126e+03_rb,8.6797e+02_rb,7.2331e+02_rb,5.7865e+02_rb,4.3399e+02_rb, &
     &2.8932e+02_rb,1.4466e+02_rb,1.2627e+01_rb/)
      kao(:, 2, 5,11) = (/ &
     &1.5804e+03_rb,1.3829e+03_rb,1.1853e+03_rb,9.8775e+02_rb,7.9020e+02_rb,5.9265e+02_rb, &
     &3.9510e+02_rb,1.9755e+02_rb,1.2599e+01_rb/)
      kao(:, 3, 5,11) = (/ &
     &2.0873e+03_rb,1.8264e+03_rb,1.5655e+03_rb,1.3046e+03_rb,1.0437e+03_rb,7.8274e+02_rb, &
     &5.2183e+02_rb,2.6092e+02_rb,1.2536e+01_rb/)
      kao(:, 4, 5,11) = (/ &
     &2.6827e+03_rb,2.3473e+03_rb,2.0120e+03_rb,1.6767e+03_rb,1.3413e+03_rb,1.0060e+03_rb, &
     &6.7067e+02_rb,3.3534e+02_rb,1.2467e+01_rb/)
      kao(:, 5, 5,11) = (/ &
     &3.3592e+03_rb,2.9393e+03_rb,2.5194e+03_rb,2.0995e+03_rb,1.6796e+03_rb,1.2597e+03_rb, &
     &8.3980e+02_rb,4.1990e+02_rb,1.2393e+01_rb/)
      kao(:, 1, 6,11) = (/ &
     &7.9180e+02_rb,6.9282e+02_rb,5.9385e+02_rb,4.9487e+02_rb,3.9590e+02_rb,2.9692e+02_rb, &
     &1.9795e+02_rb,9.8975e+01_rb,1.3948e+01_rb/)
      kao(:, 2, 6,11) = (/ &
     &1.1067e+03_rb,9.6838e+02_rb,8.3004e+02_rb,6.9170e+02_rb,5.5336e+02_rb,4.1502e+02_rb, &
     &2.7668e+02_rb,1.3834e+02_rb,1.3917e+01_rb/)
      kao(:, 3, 6,11) = (/ &
     &1.4976e+03_rb,1.3104e+03_rb,1.1232e+03_rb,9.3600e+02_rb,7.4880e+02_rb,5.6160e+02_rb, &
     &3.7440e+02_rb,1.8720e+02_rb,1.3860e+01_rb/)
      kao(:, 4, 6,11) = (/ &
     &1.9619e+03_rb,1.7167e+03_rb,1.4715e+03_rb,1.2262e+03_rb,9.8097e+02_rb,7.3573e+02_rb, &
     &4.9049e+02_rb,2.4524e+02_rb,1.3785e+01_rb/)
      kao(:, 5, 6,11) = (/ &
     &2.4991e+03_rb,2.1867e+03_rb,1.8743e+03_rb,1.5619e+03_rb,1.2496e+03_rb,9.3716e+02_rb, &
     &6.2477e+02_rb,3.1239e+02_rb,1.3707e+01_rb/)
      kao(:, 1, 7,11) = (/ &
     &5.3794e+02_rb,4.7070e+02_rb,4.0346e+02_rb,3.3621e+02_rb,2.6897e+02_rb,2.0173e+02_rb, &
     &1.3449e+02_rb,6.7243e+01_rb,1.5200e+01_rb/)
      kao(:, 2, 7,11) = (/ &
     &7.7168e+02_rb,6.7522e+02_rb,5.7876e+02_rb,4.8230e+02_rb,3.8584e+02_rb,2.8938e+02_rb, &
     &1.9292e+02_rb,9.6460e+01_rb,1.5161e+01_rb/)
      kao(:, 3, 7,11) = (/ &
     &1.0690e+03_rb,9.3537e+02_rb,8.0174e+02_rb,6.6812e+02_rb,5.3449e+02_rb,4.0087e+02_rb, &
     &2.6725e+02_rb,1.3362e+02_rb,1.5097e+01_rb/)
      kao(:, 4, 7,11) = (/ &
     &1.4303e+03_rb,1.2515e+03_rb,1.0727e+03_rb,8.9396e+02_rb,7.1516e+02_rb,5.3637e+02_rb, &
     &3.5758e+02_rb,1.7879e+02_rb,1.5017e+01_rb/)
      kao(:, 5, 7,11) = (/ &
     &1.8576e+03_rb,1.6254e+03_rb,1.3932e+03_rb,1.1610e+03_rb,9.2880e+02_rb,6.9660e+02_rb, &
     &4.6440e+02_rb,2.3220e+02_rb,1.4920e+01_rb/)
      kao(:, 1, 8,11) = (/ &
     &3.6857e+02_rb,3.2250e+02_rb,2.7643e+02_rb,2.3036e+02_rb,1.8428e+02_rb,1.3821e+02_rb, &
     &9.2143e+01_rb,4.6072e+01_rb,1.6297e+01_rb/)
      kao(:, 2, 8,11) = (/ &
     &5.4427e+02_rb,4.7624e+02_rb,4.0820e+02_rb,3.4017e+02_rb,2.7214e+02_rb,2.0410e+02_rb, &
     &1.3607e+02_rb,6.8035e+01_rb,1.6238e+01_rb/)
      kao(:, 3, 8,11) = (/ &
     &7.7191e+02_rb,6.7542e+02_rb,5.7893e+02_rb,4.8244e+02_rb,3.8595e+02_rb,2.8947e+02_rb, &
     &1.9298e+02_rb,9.6489e+01_rb,1.6155e+01_rb/)
      kao(:, 4, 8,11) = (/ &
     &1.0564e+03_rb,9.2435e+02_rb,7.9230e+02_rb,6.6025e+02_rb,5.2820e+02_rb,3.9615e+02_rb, &
     &2.6410e+02_rb,1.3205e+02_rb,1.6062e+01_rb/)
      kao(:, 5, 8,11) = (/ &
     &1.3972e+03_rb,1.2225e+03_rb,1.0479e+03_rb,8.7322e+02_rb,6.9858e+02_rb,5.2393e+02_rb, &
     &3.4929e+02_rb,1.7465e+02_rb,1.5943e+01_rb/)
      kao(:, 1, 9,11) = (/ &
     &2.4734e+02_rb,2.1642e+02_rb,1.8550e+02_rb,1.5459e+02_rb,1.2367e+02_rb,9.2751e+01_rb, &
     &6.1834e+01_rb,3.4329e+01_rb,1.7139e+01_rb/)
      kao(:, 2, 9,11) = (/ &
     &3.7646e+02_rb,3.2941e+02_rb,2.8235e+02_rb,2.3529e+02_rb,1.8823e+02_rb,1.4117e+02_rb, &
     &9.4116e+01_rb,4.7119e+01_rb,1.7064e+01_rb/)
      kao(:, 3, 9,11) = (/ &
     &5.4702e+02_rb,4.7864e+02_rb,4.1027e+02_rb,3.4189e+02_rb,2.7351e+02_rb,2.0513e+02_rb, &
     &1.3676e+02_rb,6.8378e+01_rb,1.6972e+01_rb/)
      kao(:, 4, 9,11) = (/ &
     &7.6564e+02_rb,6.6994e+02_rb,5.7423e+02_rb,4.7853e+02_rb,3.8282e+02_rb,2.8712e+02_rb, &
     &1.9141e+02_rb,9.5706e+01_rb,1.6856e+01_rb/)
      kao(:, 5, 9,11) = (/ &
     &1.0333e+03_rb,9.0412e+02_rb,7.7496e+02_rb,6.4580e+02_rb,5.1664e+02_rb,3.8748e+02_rb, &
     &2.5832e+02_rb,1.2916e+02_rb,1.6713e+01_rb/)
      kao(:, 1,10,11) = (/ &
     &1.6975e+02_rb,1.4853e+02_rb,1.2731e+02_rb,1.0609e+02_rb,8.4874e+01_rb,6.3656e+01_rb, &
     &4.3000e+01_rb,3.1831e+01_rb,1.7572e+01_rb/)
      kao(:, 2,10,11) = (/ &
     &2.6544e+02_rb,2.3226e+02_rb,1.9908e+02_rb,1.6590e+02_rb,1.3272e+02_rb,9.9541e+01_rb, &
     &6.6361e+01_rb,3.9122e+01_rb,1.7485e+01_rb/)
      kao(:, 3,10,11) = (/ &
     &3.9471e+02_rb,3.4537e+02_rb,2.9603e+02_rb,2.4669e+02_rb,1.9735e+02_rb,1.4801e+02_rb, &
     &9.8676e+01_rb,5.0382e+01_rb,1.7381e+01_rb/)
      kao(:, 4,10,11) = (/ &
     &5.6315e+02_rb,4.9275e+02_rb,4.2236e+02_rb,3.5197e+02_rb,2.8157e+02_rb,2.1118e+02_rb, &
     &1.4079e+02_rb,7.0394e+01_rb,1.7247e+01_rb/)
      kao(:, 5,10,11) = (/ &
     &7.7410e+02_rb,6.7734e+02_rb,5.8057e+02_rb,4.8381e+02_rb,3.8705e+02_rb,2.9029e+02_rb, &
     &1.9352e+02_rb,9.6763e+01_rb,1.7092e+01_rb/)
      kao(:, 1,11,11) = (/ &
     &1.4476e+02_rb,1.2667e+02_rb,1.0857e+02_rb,9.0475e+01_rb,7.2380e+01_rb,5.4285e+01_rb, &
     &4.0590e+01_rb,3.2171e+01_rb,1.7597e+01_rb/)
      kao(:, 2,11,11) = (/ &
     &2.2628e+02_rb,1.9800e+02_rb,1.6971e+02_rb,1.4143e+02_rb,1.1314e+02_rb,8.4856e+01_rb, &
     &5.6719e+01_rb,3.9479e+01_rb,1.7480e+01_rb/)
      kao(:, 3,11,11) = (/ &
     &3.3682e+02_rb,2.9472e+02_rb,2.5262e+02_rb,2.1051e+02_rb,1.6841e+02_rb,1.2631e+02_rb, &
     &8.4204e+01_rb,4.8355e+01_rb,1.7349e+01_rb/)
      kao(:, 4,11,11) = (/ &
     &4.8164e+02_rb,4.2144e+02_rb,3.6123e+02_rb,3.0103e+02_rb,2.4082e+02_rb,1.8062e+02_rb, &
     &1.2041e+02_rb,6.1572e+01_rb,1.7186e+01_rb/)
      kao(:, 5,11,11) = (/ &
     &6.6267e+02_rb,5.7984e+02_rb,4.9701e+02_rb,4.1417e+02_rb,3.3134e+02_rb,2.4850e+02_rb, &
     &1.6567e+02_rb,8.2835e+01_rb,1.7015e+01_rb/)
      kao(:, 1,12,11) = (/ &
     &1.2510e+02_rb,1.0946e+02_rb,9.3823e+01_rb,7.8186e+01_rb,6.2549e+01_rb,4.8100e+01_rb, &
     &4.0318e+01_rb,3.2221e+01_rb,1.7708e+01_rb/)
      kao(:, 2,12,11) = (/ &
     &1.9563e+02_rb,1.7117e+02_rb,1.4672e+02_rb,1.2227e+02_rb,9.7813e+01_rb,7.3360e+01_rb, &
     &5.2529e+01_rb,3.9562e+01_rb,1.7612e+01_rb/)
      kao(:, 3,12,11) = (/ &
     &2.9201e+02_rb,2.5551e+02_rb,2.1901e+02_rb,1.8251e+02_rb,1.4600e+02_rb,1.0950e+02_rb, &
     &7.3154e+01_rb,4.8447e+01_rb,1.7481e+01_rb/)
      kao(:, 4,12,11) = (/ &
     &4.1774e+02_rb,3.6552e+02_rb,3.1331e+02_rb,2.6109e+02_rb,2.0887e+02_rb,1.5665e+02_rb, &
     &1.0444e+02_rb,5.8915e+01_rb,1.7339e+01_rb/)
      kao(:, 5,12,11) = (/ &
     &5.7504e+02_rb,5.0316e+02_rb,4.3128e+02_rb,3.5940e+02_rb,2.8752e+02_rb,2.1564e+02_rb, &
     &1.4376e+02_rb,7.3682e+01_rb,1.7186e+01_rb/)
      kao(:, 1,13,11) = (/ &
     &1.1264e+02_rb,9.8561e+01_rb,8.4481e+01_rb,7.0401e+01_rb,5.6432e+01_rb,4.7061e+01_rb, &
     &4.0409e+01_rb,3.2253e+01_rb,1.8302e+01_rb/)
      kao(:, 2,13,11) = (/ &
     &1.7621e+02_rb,1.5419e+02_rb,1.3216e+02_rb,1.1013e+02_rb,8.8108e+01_rb,6.6456e+01_rb, &
     &5.2592e+01_rb,3.9667e+01_rb,1.8194e+01_rb/)
      kao(:, 3,13,11) = (/ &
     &2.6303e+02_rb,2.3015e+02_rb,1.9727e+02_rb,1.6440e+02_rb,1.3152e+02_rb,9.8638e+01_rb, &
     &6.8547e+01_rb,4.8772e+01_rb,1.8071e+01_rb/)
      kao(:, 4,13,11) = (/ &
     &3.7673e+02_rb,3.2964e+02_rb,2.8255e+02_rb,2.3546e+02_rb,1.8837e+02_rb,1.4128e+02_rb, &
     &9.4251e+01_rb,5.9423e+01_rb,1.7926e+01_rb/)
      kao(:, 5,13,11) = (/ &
     &5.1904e+02_rb,4.5416e+02_rb,3.8928e+02_rb,3.2440e+02_rb,2.5952e+02_rb,1.9464e+02_rb, &
     &1.2976e+02_rb,7.1698e+01_rb,1.7708e+01_rb/)
      kao(:, 1, 1,12) = (/ &
     &6.1566e+03_rb,5.3871e+03_rb,4.6175e+03_rb,3.8479e+03_rb,3.0783e+03_rb,2.3087e+03_rb, &
     &1.5392e+03_rb,7.6958e+02_rb,3.6622e-03_rb/)
      kao(:, 2, 1,12) = (/ &
     &7.8640e+03_rb,6.8810e+03_rb,5.8980e+03_rb,4.9150e+03_rb,3.9320e+03_rb,2.9490e+03_rb, &
     &1.9660e+03_rb,9.8301e+02_rb,3.6025e-03_rb/)
      kao(:, 3, 1,12) = (/ &
     &9.7949e+03_rb,8.5705e+03_rb,7.3462e+03_rb,6.1218e+03_rb,4.8974e+03_rb,3.6731e+03_rb, &
     &2.4487e+03_rb,1.2244e+03_rb,3.3010e-02_rb/)
      kao(:, 4, 1,12) = (/ &
     &1.1959e+04_rb,1.0464e+04_rb,8.9695e+03_rb,7.4746e+03_rb,5.9797e+03_rb,4.4847e+03_rb, &
     &2.9898e+03_rb,1.4949e+03_rb,6.0612e-02_rb/)
      kao(:, 5, 1,12) = (/ &
     &1.4386e+04_rb,1.2588e+04_rb,1.0790e+04_rb,8.9913e+03_rb,7.1931e+03_rb,5.3948e+03_rb, &
     &3.5965e+03_rb,1.7983e+03_rb,7.6915e-02_rb/)
      kao(:, 1, 2,12) = (/ &
     &5.0811e+03_rb,4.4460e+03_rb,3.8109e+03_rb,3.1757e+03_rb,2.5406e+03_rb,1.9054e+03_rb, &
     &1.2703e+03_rb,6.3514e+02_rb,2.7963e-01_rb/)
      kao(:, 2, 2,12) = (/ &
     &6.5891e+03_rb,5.7655e+03_rb,4.9418e+03_rb,4.1182e+03_rb,3.2946e+03_rb,2.4709e+03_rb, &
     &1.6473e+03_rb,8.2364e+02_rb,2.3002e+00_rb/)
      kao(:, 3, 2,12) = (/ &
     &8.3666e+03_rb,7.3208e+03_rb,6.2750e+03_rb,5.2291e+03_rb,4.1833e+03_rb,3.1375e+03_rb, &
     &2.0917e+03_rb,1.0458e+03_rb,4.9790e+00_rb/)
      kao(:, 4, 2,12) = (/ &
     &1.0356e+04_rb,9.0616e+03_rb,7.7671e+03_rb,6.4726e+03_rb,5.1781e+03_rb,3.8836e+03_rb, &
     &2.5890e+03_rb,1.2945e+03_rb,6.0169e+00_rb/)
      kao(:, 5, 2,12) = (/ &
     &1.2531e+04_rb,1.0965e+04_rb,9.3984e+03_rb,7.8320e+03_rb,6.2656e+03_rb,4.6992e+03_rb, &
     &3.1328e+03_rb,1.5664e+03_rb,6.8160e+00_rb/)
      kao(:, 1, 3,12) = (/ &
     &3.8913e+03_rb,3.4049e+03_rb,2.9185e+03_rb,2.4321e+03_rb,1.9457e+03_rb,1.4593e+03_rb, &
     &9.7284e+02_rb,4.8642e+02_rb,1.1253e+01_rb/)
      kao(:, 2, 3,12) = (/ &
     &5.1870e+03_rb,4.5386e+03_rb,3.8902e+03_rb,3.2418e+03_rb,2.5935e+03_rb,1.9451e+03_rb, &
     &1.2967e+03_rb,6.4837e+02_rb,1.1246e+01_rb/)
      kao(:, 3, 3,12) = (/ &
     &6.6906e+03_rb,5.8543e+03_rb,5.0180e+03_rb,4.1816e+03_rb,3.3453e+03_rb,2.5090e+03_rb, &
     &1.6727e+03_rb,8.3633e+02_rb,1.1235e+01_rb/)
      kao(:, 4, 3,12) = (/ &
     &8.4124e+03_rb,7.3608e+03_rb,6.3093e+03_rb,5.2577e+03_rb,4.2062e+03_rb,3.1546e+03_rb, &
     &2.1031e+03_rb,1.0515e+03_rb,1.1206e+01_rb/)
      kao(:, 5, 3,12) = (/ &
     &1.0381e+04_rb,9.0837e+03_rb,7.7860e+03_rb,6.4883e+03_rb,5.1907e+03_rb,3.8930e+03_rb, &
     &2.5953e+03_rb,1.2977e+03_rb,1.1154e+01_rb/)
      kao(:, 1, 4,12) = (/ &
     &2.8697e+03_rb,2.5110e+03_rb,2.1523e+03_rb,1.7936e+03_rb,1.4349e+03_rb,1.0761e+03_rb, &
     &7.1743e+02_rb,3.5872e+02_rb,1.3022e+01_rb/)
      kao(:, 2, 4,12) = (/ &
     &3.9101e+03_rb,3.4213e+03_rb,2.9326e+03_rb,2.4438e+03_rb,1.9551e+03_rb,1.4663e+03_rb, &
     &9.7753e+02_rb,4.8877e+02_rb,1.3025e+01_rb/)
      kao(:, 3, 4,12) = (/ &
     &5.1573e+03_rb,4.5126e+03_rb,3.8680e+03_rb,3.2233e+03_rb,2.5786e+03_rb,1.9340e+03_rb, &
     &1.2893e+03_rb,6.4466e+02_rb,1.3008e+01_rb/)
      kao(:, 4, 4,12) = (/ &
     &6.6403e+03_rb,5.8103e+03_rb,4.9802e+03_rb,4.1502e+03_rb,3.3201e+03_rb,2.4901e+03_rb, &
     &1.6601e+03_rb,8.3004e+02_rb,1.2981e+01_rb/)
      kao(:, 5, 4,12) = (/ &
     &8.3023e+03_rb,7.2645e+03_rb,6.2267e+03_rb,5.1889e+03_rb,4.1511e+03_rb,3.1134e+03_rb, &
     &2.0756e+03_rb,1.0378e+03_rb,1.2927e+01_rb/)
      kao(:, 1, 5,12) = (/ &
     &2.0574e+03_rb,1.8002e+03_rb,1.5431e+03_rb,1.2859e+03_rb,1.0287e+03_rb,7.7153e+02_rb, &
     &5.1435e+02_rb,2.5718e+02_rb,1.4941e+01_rb/)
      kao(:, 2, 5,12) = (/ &
     &2.8796e+03_rb,2.5197e+03_rb,2.1597e+03_rb,1.7998e+03_rb,1.4398e+03_rb,1.0799e+03_rb, &
     &7.1991e+02_rb,3.5996e+02_rb,1.4958e+01_rb/)
      kao(:, 3, 5,12) = (/ &
     &3.9044e+03_rb,3.4163e+03_rb,2.9283e+03_rb,2.4402e+03_rb,1.9522e+03_rb,1.4641e+03_rb, &
     &9.7610e+02_rb,4.8805e+02_rb,1.4965e+01_rb/)
      kao(:, 4, 5,12) = (/ &
     &5.1045e+03_rb,4.4665e+03_rb,3.8284e+03_rb,3.1903e+03_rb,2.5523e+03_rb,1.9142e+03_rb, &
     &1.2761e+03_rb,6.3807e+02_rb,1.4927e+01_rb/)
      kao(:, 5, 5,12) = (/ &
     &6.5127e+03_rb,5.6986e+03_rb,4.8846e+03_rb,4.0705e+03_rb,3.2564e+03_rb,2.4423e+03_rb, &
     &1.6282e+03_rb,8.1409e+02_rb,1.4857e+01_rb/)
      kao(:, 1, 6,12) = (/ &
     &1.4218e+03_rb,1.2441e+03_rb,1.0663e+03_rb,8.8861e+02_rb,7.1089e+02_rb,5.3316e+02_rb, &
     &3.5544e+02_rb,1.7772e+02_rb,1.7034e+01_rb/)
      kao(:, 2, 6,12) = (/ &
     &2.0585e+03_rb,1.8012e+03_rb,1.5439e+03_rb,1.2865e+03_rb,1.0292e+03_rb,7.7193e+02_rb, &
     &5.1462e+02_rb,2.5731e+02_rb,1.7052e+01_rb/)
      kao(:, 3, 6,12) = (/ &
     &2.8529e+03_rb,2.4963e+03_rb,2.1397e+03_rb,1.7831e+03_rb,1.4265e+03_rb,1.0699e+03_rb, &
     &7.1324e+02_rb,3.5662e+02_rb,1.7043e+01_rb/)
      kao(:, 4, 6,12) = (/ &
     &3.8237e+03_rb,3.3458e+03_rb,2.8678e+03_rb,2.3898e+03_rb,1.9119e+03_rb,1.4339e+03_rb, &
     &9.5593e+02_rb,4.7797e+02_rb,1.6999e+01_rb/)
      kao(:, 5, 6,12) = (/ &
     &4.9757e+03_rb,4.3537e+03_rb,3.7318e+03_rb,3.1098e+03_rb,2.4878e+03_rb,1.8659e+03_rb, &
     &1.2439e+03_rb,6.2196e+02_rb,1.6923e+01_rb/)
      kao(:, 1, 7,12) = (/ &
     &9.7219e+02_rb,8.5066e+02_rb,7.2914e+02_rb,6.0761e+02_rb,4.8609e+02_rb,3.6457e+02_rb, &
     &2.4305e+02_rb,1.2152e+02_rb,1.9204e+01_rb/)
      kao(:, 2, 7,12) = (/ &
     &1.4501e+03_rb,1.2688e+03_rb,1.0876e+03_rb,9.0631e+02_rb,7.2505e+02_rb,5.4378e+02_rb, &
     &3.6252e+02_rb,1.8126e+02_rb,1.9225e+01_rb/)
      kao(:, 3, 7,12) = (/ &
     &2.0671e+03_rb,1.8087e+03_rb,1.5503e+03_rb,1.2919e+03_rb,1.0335e+03_rb,7.7515e+02_rb, &
     &5.1676e+02_rb,2.5838e+02_rb,1.9209e+01_rb/)
      kao(:, 4, 7,12) = (/ &
     &2.8414e+03_rb,2.4862e+03_rb,2.1311e+03_rb,1.7759e+03_rb,1.4207e+03_rb,1.0655e+03_rb, &
     &7.1035e+02_rb,3.5518e+02_rb,1.9156e+01_rb/)
      kao(:, 5, 7,12) = (/ &
     &3.7638e+03_rb,3.2933e+03_rb,2.8228e+03_rb,2.3524e+03_rb,1.8819e+03_rb,1.4114e+03_rb, &
     &9.4094e+02_rb,4.7047e+02_rb,1.9066e+01_rb/)
      kao(:, 1, 8,12) = (/ &
     &6.6611e+02_rb,5.8284e+02_rb,4.9958e+02_rb,4.1632e+02_rb,3.3305e+02_rb,2.4979e+02_rb, &
     &1.6653e+02_rb,8.3265e+01_rb,2.1374e+01_rb/)
      kao(:, 2, 8,12) = (/ &
     &1.0262e+03_rb,8.9794e+02_rb,7.6966e+02_rb,6.4139e+02_rb,5.1311e+02_rb,3.8483e+02_rb, &
     &2.5656e+02_rb,1.2828e+02_rb,2.1403e+01_rb/)
      kao(:, 3, 8,12) = (/ &
     &1.5100e+03_rb,1.3213e+03_rb,1.1325e+03_rb,9.4376e+02_rb,7.5501e+02_rb,5.6626e+02_rb, &
     &3.7751e+02_rb,1.8875e+02_rb,2.1385e+01_rb/)
      kao(:, 4, 8,12) = (/ &
     &2.1206e+03_rb,1.8555e+03_rb,1.5904e+03_rb,1.3254e+03_rb,1.0603e+03_rb,7.9522e+02_rb, &
     &5.3015e+02_rb,2.6507e+02_rb,2.1320e+01_rb/)
      kao(:, 5, 8,12) = (/ &
     &2.8794e+03_rb,2.5195e+03_rb,2.1596e+03_rb,1.7996e+03_rb,1.4397e+03_rb,1.0798e+03_rb, &
     &7.1985e+02_rb,3.5993e+02_rb,2.1216e+01_rb/)
      kao(:, 1, 9,12) = (/ &
     &4.4380e+02_rb,3.8832e+02_rb,3.3285e+02_rb,2.7737e+02_rb,2.2190e+02_rb,1.6642e+02_rb, &
     &1.1095e+02_rb,5.5475e+01_rb,2.3490e+01_rb/)
      kao(:, 2, 9,12) = (/ &
     &7.0869e+02_rb,6.2011e+02_rb,5.3152e+02_rb,4.4293e+02_rb,3.5435e+02_rb,2.6576e+02_rb, &
     &1.7717e+02_rb,8.8587e+01_rb,2.3505e+01_rb/)
      kao(:, 3, 9,12) = (/ &
     &1.0731e+03_rb,9.3894e+02_rb,8.0481e+02_rb,6.7067e+02_rb,5.3654e+02_rb,4.0240e+02_rb, &
     &2.6827e+02_rb,1.3414e+02_rb,2.3465e+01_rb/)
      kao(:, 4, 9,12) = (/ &
     &1.5496e+03_rb,1.3559e+03_rb,1.1622e+03_rb,9.6851e+02_rb,7.7481e+02_rb,5.8111e+02_rb, &
     &3.8741e+02_rb,1.9370e+02_rb,2.3379e+01_rb/)
      kao(:, 5, 9,12) = (/ &
     &2.1497e+03_rb,1.8810e+03_rb,1.6123e+03_rb,1.3436e+03_rb,1.0749e+03_rb,8.0615e+02_rb, &
     &5.3743e+02_rb,2.6872e+02_rb,2.3258e+01_rb/)
      kao(:, 1,10,12) = (/ &
     &3.0213e+02_rb,2.6436e+02_rb,2.2659e+02_rb,1.8883e+02_rb,1.5106e+02_rb,1.1330e+02_rb, &
     &7.5532e+01_rb,4.3106e+01_rb,2.5395e+01_rb/)
      kao(:, 2,10,12) = (/ &
     &4.9804e+02_rb,4.3578e+02_rb,3.7353e+02_rb,3.1127e+02_rb,2.4902e+02_rb,1.8676e+02_rb, &
     &1.2451e+02_rb,6.2505e+01_rb,2.5385e+01_rb/)
      kao(:, 3,10,12) = (/ &
     &7.7418e+02_rb,6.7741e+02_rb,5.8063e+02_rb,4.8386e+02_rb,3.8709e+02_rb,2.9032e+02_rb, &
     &1.9354e+02_rb,9.6773e+01_rb,2.5325e+01_rb/)
      kao(:, 4,10,12) = (/ &
     &1.1469e+03_rb,1.0035e+03_rb,8.6015e+02_rb,7.1679e+02_rb,5.7343e+02_rb,4.3007e+02_rb, &
     &2.8671e+02_rb,1.4336e+02_rb,2.5217e+01_rb/)
      kao(:, 5,10,12) = (/ &
     &1.6217e+03_rb,1.4190e+03_rb,1.2162e+03_rb,1.0135e+03_rb,8.1083e+02_rb,6.0812e+02_rb, &
     &4.0541e+02_rb,2.0271e+02_rb,2.5082e+01_rb/)
      kao(:, 1,11,12) = (/ &
     &2.6097e+02_rb,2.2835e+02_rb,1.9573e+02_rb,1.6311e+02_rb,1.3048e+02_rb,9.7863e+01_rb, &
     &6.5361e+01_rb,4.5364e+01_rb,2.6989e+01_rb/)
      kao(:, 2,11,12) = (/ &
     &4.3087e+02_rb,3.7701e+02_rb,3.2315e+02_rb,2.6929e+02_rb,2.1543e+02_rb,1.6158e+02_rb, &
     &1.0772e+02_rb,5.7754e+01_rb,2.6926e+01_rb/)
      kao(:, 3,11,12) = (/ &
     &6.7272e+02_rb,5.8863e+02_rb,5.0454e+02_rb,4.2045e+02_rb,3.3636e+02_rb,2.5227e+02_rb, &
     &1.6818e+02_rb,8.4156e+01_rb,2.6813e+01_rb/)
      kao(:, 4,11,12) = (/ &
     &9.9607e+02_rb,8.7156e+02_rb,7.4705e+02_rb,6.2254e+02_rb,4.9803e+02_rb,3.7353e+02_rb, &
     &2.4902e+02_rb,1.2451e+02_rb,2.6666e+01_rb/)
      kao(:, 5,11,12) = (/ &
     &1.4127e+03_rb,1.2361e+03_rb,1.0595e+03_rb,8.8291e+02_rb,7.0633e+02_rb,5.2975e+02_rb, &
     &3.5316e+02_rb,1.7658e+02_rb,2.6473e+01_rb/)
      kao(:, 1,12,12) = (/ &
     &2.2830e+02_rb,1.9976e+02_rb,1.7123e+02_rb,1.4269e+02_rb,1.1415e+02_rb,8.5613e+01_rb, &
     &6.0099e+01_rb,4.8249e+01_rb,2.8107e+01_rb/)
      kao(:, 2,12,12) = (/ &
     &3.7815e+02_rb,3.3089e+02_rb,2.8362e+02_rb,2.3635e+02_rb,1.8908e+02_rb,1.4181e+02_rb, &
     &9.4540e+01_rb,5.8332e+01_rb,2.7966e+01_rb/)
      kao(:, 3,12,12) = (/ &
     &5.9040e+02_rb,5.1660e+02_rb,4.4280e+02_rb,3.6900e+02_rb,2.9520e+02_rb,2.2140e+02_rb, &
     &1.4760e+02_rb,7.6746e+01_rb,2.7793e+01_rb/)
      kao(:, 4,12,12) = (/ &
     &8.7660e+02_rb,7.6702e+02_rb,6.5745e+02_rb,5.4787e+02_rb,4.3830e+02_rb,3.2873e+02_rb, &
     &2.1915e+02_rb,1.0959e+02_rb,2.7580e+01_rb/)
      kao(:, 5,12,12) = (/ &
     &1.2432e+03_rb,1.0878e+03_rb,9.3242e+02_rb,7.7702e+02_rb,6.2161e+02_rb,4.6621e+02_rb, &
     &3.1081e+02_rb,1.5540e+02_rb,2.7330e+01_rb/)
      kao(:, 1,13,12) = (/ &
     &2.0770e+02_rb,1.8173e+02_rb,1.5577e+02_rb,1.2981e+02_rb,1.0385e+02_rb,7.8129e+01_rb, &
     &6.0586e+01_rb,5.1072e+01_rb,2.8383e+01_rb/)
      kao(:, 2,13,12) = (/ &
     &3.4420e+02_rb,3.0118e+02_rb,2.5815e+02_rb,2.1513e+02_rb,1.7210e+02_rb,1.2908e+02_rb, &
     &8.7185e+01_rb,6.1802e+01_rb,2.8222e+01_rb/)
      kao(:, 3,13,12) = (/ &
     &5.3871e+02_rb,4.7137e+02_rb,4.0403e+02_rb,3.3670e+02_rb,2.6936e+02_rb,2.0202e+02_rb, &
     &1.3468e+02_rb,7.6219e+01_rb,2.8010e+01_rb/)
      kao(:, 4,13,12) = (/ &
     &7.9982e+02_rb,6.9985e+02_rb,5.9987e+02_rb,4.9989e+02_rb,3.9991e+02_rb,2.9994e+02_rb, &
     &1.9996e+02_rb,1.0196e+02_rb,2.7782e+01_rb/)
      kao(:, 5,13,12) = (/ &
     &1.1353e+03_rb,9.9341e+02_rb,8.5150e+02_rb,7.0958e+02_rb,5.6767e+02_rb,4.2575e+02_rb, &
     &2.8384e+02_rb,1.4192e+02_rb,2.7521e+01_rb/)
      kao(:, 1, 1,13) = (/ &
     &1.0877e+04_rb,9.5175e+03_rb,8.1579e+03_rb,6.7982e+03_rb,5.4386e+03_rb,4.0789e+03_rb, &
     &2.7193e+03_rb,1.3596e+03_rb,2.7375e-03_rb/)
      kao(:, 2, 1,13) = (/ &
     &1.4140e+04_rb,1.2372e+04_rb,1.0605e+04_rb,8.8373e+03_rb,7.0699e+03_rb,5.3024e+03_rb, &
     &3.5349e+03_rb,1.7675e+03_rb,2.6515e-03_rb/)
      kao(:, 3, 1,13) = (/ &
     &1.7904e+04_rb,1.5666e+04_rb,1.3428e+04_rb,1.1190e+04_rb,8.9519e+03_rb,6.7139e+03_rb, &
     &4.4759e+03_rb,2.2380e+03_rb,3.8993e-02_rb/)
      kao(:, 4, 1,13) = (/ &
     &2.2059e+04_rb,1.9302e+04_rb,1.6544e+04_rb,1.3787e+04_rb,1.1029e+04_rb,8.2721e+03_rb, &
     &5.5147e+03_rb,2.7574e+03_rb,9.2576e-02_rb/)
      kao(:, 5, 1,13) = (/ &
     &2.6544e+04_rb,2.3226e+04_rb,1.9908e+04_rb,1.6590e+04_rb,1.3272e+04_rb,9.9541e+03_rb, &
     &6.6361e+03_rb,3.3180e+03_rb,1.0695e-01_rb/)
      kao(:, 1, 2,13) = (/ &
     &9.6500e+03_rb,8.4437e+03_rb,7.2375e+03_rb,6.0312e+03_rb,4.8250e+03_rb,3.6187e+03_rb, &
     &2.4125e+03_rb,1.2062e+03_rb,3.2200e-01_rb/)
      kao(:, 2, 2,13) = (/ &
     &1.2735e+04_rb,1.1143e+04_rb,9.5512e+03_rb,7.9593e+03_rb,6.3674e+03_rb,4.7756e+03_rb, &
     &3.1837e+03_rb,1.5919e+03_rb,5.1280e+00_rb/)
      kao(:, 3, 2,13) = (/ &
     &1.6244e+04_rb,1.4213e+04_rb,1.2183e+04_rb,1.0152e+04_rb,8.1218e+03_rb,6.0913e+03_rb, &
     &4.0609e+03_rb,2.0304e+03_rb,7.4252e+00_rb/)
      kao(:, 4, 2,13) = (/ &
     &2.0224e+04_rb,1.7696e+04_rb,1.5168e+04_rb,1.2640e+04_rb,1.0112e+04_rb,7.5839e+03_rb, &
     &5.0559e+03_rb,2.5280e+03_rb,7.7282e+00_rb/)
      kao(:, 5, 2,13) = (/ &
     &2.4645e+04_rb,2.1565e+04_rb,1.8484e+04_rb,1.5403e+04_rb,1.2323e+04_rb,9.2420e+03_rb, &
     &6.1614e+03_rb,3.0807e+03_rb,7.1576e+00_rb/)
      kao(:, 1, 3,13) = (/ &
     &7.8601e+03_rb,6.8776e+03_rb,5.8951e+03_rb,4.9126e+03_rb,3.9300e+03_rb,2.9475e+03_rb, &
     &1.9650e+03_rb,9.8251e+02_rb,1.2409e+01_rb/)
      kao(:, 2, 3,13) = (/ &
     &1.0573e+04_rb,9.2512e+03_rb,7.9296e+03_rb,6.6080e+03_rb,5.2864e+03_rb,3.9648e+03_rb, &
     &2.6432e+03_rb,1.3216e+03_rb,1.2422e+01_rb/)
      kao(:, 3, 3,13) = (/ &
     &1.3794e+04_rb,1.2070e+04_rb,1.0346e+04_rb,8.6214e+03_rb,6.8971e+03_rb,5.1729e+03_rb, &
     &3.4486e+03_rb,1.7243e+03_rb,1.2413e+01_rb/)
      kao(:, 4, 3,13) = (/ &
     &1.7502e+04_rb,1.5315e+04_rb,1.3127e+04_rb,1.0939e+04_rb,8.7512e+03_rb,6.5634e+03_rb, &
     &4.3756e+03_rb,2.1878e+03_rb,1.2394e+01_rb/)
      kao(:, 5, 3,13) = (/ &
     &2.1662e+04_rb,1.8954e+04_rb,1.6247e+04_rb,1.3539e+04_rb,1.0831e+04_rb,8.1233e+03_rb, &
     &5.4155e+03_rb,2.7078e+03_rb,1.2358e+01_rb/)
      kao(:, 1, 4,13) = (/ &
     &6.0906e+03_rb,5.3293e+03_rb,4.5680e+03_rb,3.8067e+03_rb,3.0453e+03_rb,2.2840e+03_rb, &
     &1.5227e+03_rb,7.6133e+02_rb,1.4627e+01_rb/)
      kao(:, 2, 4,13) = (/ &
     &8.4475e+03_rb,7.3915e+03_rb,6.3356e+03_rb,5.2797e+03_rb,4.2237e+03_rb,3.1678e+03_rb, &
     &2.1119e+03_rb,1.0559e+03_rb,1.4656e+01_rb/)
      kao(:, 3, 4,13) = (/ &
     &1.1299e+04_rb,9.8868e+03_rb,8.4744e+03_rb,7.0620e+03_rb,5.6496e+03_rb,4.2372e+03_rb, &
     &2.8248e+03_rb,1.4124e+03_rb,1.4666e+01_rb/)
      kao(:, 4, 4,13) = (/ &
     &1.4605e+04_rb,1.2779e+04_rb,1.0954e+04_rb,9.1282e+03_rb,7.3025e+03_rb,5.4769e+03_rb, &
     &3.6513e+03_rb,1.8256e+03_rb,1.4645e+01_rb/)
      kao(:, 5, 4,13) = (/ &
     &1.8360e+04_rb,1.6065e+04_rb,1.3770e+04_rb,1.1475e+04_rb,9.1801e+03_rb,6.8851e+03_rb, &
     &4.5900e+03_rb,2.2950e+03_rb,1.4604e+01_rb/)
      kao(:, 1, 5,13) = (/ &
     &4.6024e+03_rb,4.0271e+03_rb,3.4518e+03_rb,2.8765e+03_rb,2.3012e+03_rb,1.7259e+03_rb, &
     &1.1506e+03_rb,5.7529e+02_rb,1.7191e+01_rb/)
      kao(:, 2, 5,13) = (/ &
     &6.5855e+03_rb,5.7623e+03_rb,4.9391e+03_rb,4.1159e+03_rb,3.2927e+03_rb,2.4695e+03_rb, &
     &1.6464e+03_rb,8.2318e+02_rb,1.7235e+01_rb/)
      kao(:, 3, 5,13) = (/ &
     &8.9962e+03_rb,7.8717e+03_rb,6.7471e+03_rb,5.6226e+03_rb,4.4981e+03_rb,3.3736e+03_rb, &
     &2.2490e+03_rb,1.1245e+03_rb,1.7249e+01_rb/)
      kao(:, 4, 5,13) = (/ &
     &1.1873e+04_rb,1.0388e+04_rb,8.9044e+03_rb,7.4203e+03_rb,5.9363e+03_rb,4.4522e+03_rb, &
     &2.9681e+03_rb,1.4841e+03_rb,1.7236e+01_rb/)
      kao(:, 5, 5,13) = (/ &
     &1.5203e+04_rb,1.3303e+04_rb,1.1402e+04_rb,9.5020e+03_rb,7.6016e+03_rb,5.7012e+03_rb, &
     &3.8008e+03_rb,1.9004e+03_rb,1.7202e+01_rb/)
      kao(:, 1, 6,13) = (/ &
     &3.3402e+03_rb,2.9227e+03_rb,2.5052e+03_rb,2.0876e+03_rb,1.6701e+03_rb,1.2526e+03_rb, &
     &8.3506e+02_rb,4.1753e+02_rb,2.0086e+01_rb/)
      kao(:, 2, 6,13) = (/ &
     &4.9170e+03_rb,4.3024e+03_rb,3.6877e+03_rb,3.0731e+03_rb,2.4585e+03_rb,1.8439e+03_rb, &
     &1.2292e+03_rb,6.1462e+02_rb,2.0160e+01_rb/)
      kao(:, 3, 6,13) = (/ &
     &6.9136e+03_rb,6.0494e+03_rb,5.1852e+03_rb,4.3210e+03_rb,3.4568e+03_rb,2.5926e+03_rb, &
     &1.7284e+03_rb,8.6420e+02_rb,2.0194e+01_rb/)
      kao(:, 4, 6,13) = (/ &
     &9.3499e+03_rb,8.1811e+03_rb,7.0124e+03_rb,5.8437e+03_rb,4.6749e+03_rb,3.5062e+03_rb, &
     &2.3375e+03_rb,1.1687e+03_rb,2.0186e+01_rb/)
      kao(:, 5, 6,13) = (/ &
     &1.2234e+04_rb,1.0705e+04_rb,9.1755e+03_rb,7.6462e+03_rb,6.1170e+03_rb,4.5877e+03_rb, &
     &3.0585e+03_rb,1.5292e+03_rb,2.0149e+01_rb/)
      kao(:, 1, 7,13) = (/ &
     &2.3634e+03_rb,2.0680e+03_rb,1.7726e+03_rb,1.4771e+03_rb,1.1817e+03_rb,8.8628e+02_rb, &
     &5.9086e+02_rb,2.9543e+02_rb,2.3318e+01_rb/)
      kao(:, 2, 7,13) = (/ &
     &3.6109e+03_rb,3.1596e+03_rb,2.7082e+03_rb,2.2568e+03_rb,1.8055e+03_rb,1.3541e+03_rb, &
     &9.0273e+02_rb,4.5136e+02_rb,2.3426e+01_rb/)
      kao(:, 3, 7,13) = (/ &
     &5.2430e+03_rb,4.5876e+03_rb,3.9322e+03_rb,3.2769e+03_rb,2.6215e+03_rb,1.9661e+03_rb, &
     &1.3107e+03_rb,6.5537e+02_rb,2.3476e+01_rb/)
      kao(:, 4, 7,13) = (/ &
     &7.2754e+03_rb,6.3660e+03_rb,5.4565e+03_rb,4.5471e+03_rb,3.6377e+03_rb,2.7283e+03_rb, &
     &1.8188e+03_rb,9.0942e+02_rb,2.3476e+01_rb/)
      kao(:, 5, 7,13) = (/ &
     &9.7107e+03_rb,8.4968e+03_rb,7.2830e+03_rb,6.0691e+03_rb,4.8553e+03_rb,3.6415e+03_rb, &
     &2.4276e+03_rb,1.2138e+03_rb,2.3445e+01_rb/)
      kao(:, 1, 8,13) = (/ &
     &1.6675e+03_rb,1.4591e+03_rb,1.2506e+03_rb,1.0422e+03_rb,8.3375e+02_rb,6.2531e+02_rb, &
     &4.1687e+02_rb,2.0844e+02_rb,2.6870e+01_rb/)
      kao(:, 2, 8,13) = (/ &
     &2.6544e+03_rb,2.3226e+03_rb,1.9908e+03_rb,1.6590e+03_rb,1.3272e+03_rb,9.9538e+02_rb, &
     &6.6359e+02_rb,3.3179e+02_rb,2.6998e+01_rb/)
      kao(:, 3, 8,13) = (/ &
     &3.9767e+03_rb,3.4797e+03_rb,2.9826e+03_rb,2.4855e+03_rb,1.9884e+03_rb,1.4913e+03_rb, &
     &9.9419e+02_rb,4.9709e+02_rb,2.7066e+01_rb/)
      kao(:, 4, 8,13) = (/ &
     &5.6620e+03_rb,4.9542e+03_rb,4.2465e+03_rb,3.5387e+03_rb,2.8310e+03_rb,2.1232e+03_rb, &
     &1.4155e+03_rb,7.0775e+02_rb,2.7083e+01_rb/)
      kao(:, 5, 8,13) = (/ &
     &7.7334e+03_rb,6.7667e+03_rb,5.8000e+03_rb,4.8334e+03_rb,3.8667e+03_rb,2.9000e+03_rb, &
     &1.9333e+03_rb,9.6667e+02_rb,2.7049e+01_rb/)
      kao(:, 1, 9,13) = (/ &
     &1.1358e+03_rb,9.9378e+02_rb,8.5181e+02_rb,7.0984e+02_rb,5.6787e+02_rb,4.2591e+02_rb, &
     &2.8394e+02_rb,1.4197e+02_rb,3.0678e+01_rb/)
      kao(:, 2, 9,13) = (/ &
     &1.8861e+03_rb,1.6503e+03_rb,1.4146e+03_rb,1.1788e+03_rb,9.4305e+02_rb,7.0729e+02_rb, &
     &4.7153e+02_rb,2.3576e+02_rb,3.0840e+01_rb/)
      kao(:, 3, 9,13) = (/ &
     &2.9209e+03_rb,2.5558e+03_rb,2.1907e+03_rb,1.8256e+03_rb,1.4605e+03_rb,1.0953e+03_rb, &
     &7.3023e+02_rb,3.6511e+02_rb,3.0929e+01_rb/)
      kao(:, 4, 9,13) = (/ &
     &4.2847e+03_rb,3.7491e+03_rb,3.2135e+03_rb,2.6779e+03_rb,2.1424e+03_rb,1.6068e+03_rb, &
     &1.0712e+03_rb,5.3559e+02_rb,3.0953e+01_rb/)
      kao(:, 5, 9,13) = (/ &
     &6.0053e+03_rb,5.2546e+03_rb,4.5040e+03_rb,3.7533e+03_rb,3.0026e+03_rb,2.2520e+03_rb, &
     &1.5013e+03_rb,7.5066e+02_rb,3.0913e+01_rb/)
      kao(:, 1,10,13) = (/ &
     &7.8733e+02_rb,6.8892e+02_rb,5.9050e+02_rb,4.9208e+02_rb,3.9366e+02_rb,2.9525e+02_rb, &
     &1.9683e+02_rb,9.8416e+01_rb,3.4699e+01_rb/)
      kao(:, 2,10,13) = (/ &
     &1.3543e+03_rb,1.1850e+03_rb,1.0157e+03_rb,8.4641e+02_rb,6.7713e+02_rb,5.0785e+02_rb, &
     &3.3856e+02_rb,1.6928e+02_rb,3.4872e+01_rb/)
      kao(:, 3,10,13) = (/ &
     &2.1695e+03_rb,1.8983e+03_rb,1.6271e+03_rb,1.3559e+03_rb,1.0848e+03_rb,8.1356e+02_rb, &
     &5.4238e+02_rb,2.7119e+02_rb,3.4962e+01_rb/)
      kao(:, 4,10,13) = (/ &
     &3.2703e+03_rb,2.8615e+03_rb,2.4527e+03_rb,2.0439e+03_rb,1.6351e+03_rb,1.2263e+03_rb, &
     &8.1756e+02_rb,4.0878e+02_rb,3.4974e+01_rb/)
      kao(:, 5,10,13) = (/ &
     &4.6791e+03_rb,4.0942e+03_rb,3.5093e+03_rb,2.9244e+03_rb,2.3395e+03_rb,1.7546e+03_rb, &
     &1.1698e+03_rb,5.8488e+02_rb,3.4915e+01_rb/)
      kao(:, 1,11,13) = (/ &
     &7.0254e+02_rb,6.1472e+02_rb,5.2690e+02_rb,4.3908e+02_rb,3.5127e+02_rb,2.6345e+02_rb, &
     &1.7563e+02_rb,8.8441e+01_rb,3.8867e+01_rb/)
      kao(:, 2,11,13) = (/ &
     &1.2148e+03_rb,1.0630e+03_rb,9.1111e+02_rb,7.5926e+02_rb,6.0740e+02_rb,4.5555e+02_rb, &
     &3.0370e+02_rb,1.5185e+02_rb,3.8999e+01_rb/)
      kao(:, 3,11,13) = (/ &
     &1.9474e+03_rb,1.7040e+03_rb,1.4606e+03_rb,1.2171e+03_rb,9.7371e+02_rb,7.3028e+02_rb, &
     &4.8685e+02_rb,2.4343e+02_rb,3.9046e+01_rb/)
      kao(:, 4,11,13) = (/ &
     &2.9354e+03_rb,2.5684e+03_rb,2.2015e+03_rb,1.8346e+03_rb,1.4677e+03_rb,1.1008e+03_rb, &
     &7.3384e+02_rb,3.6692e+02_rb,3.9009e+01_rb/)
      kao(:, 5,11,13) = (/ &
     &4.2118e+03_rb,3.6853e+03_rb,3.1589e+03_rb,2.6324e+03_rb,2.1059e+03_rb,1.5794e+03_rb, &
     &1.0529e+03_rb,5.2647e+02_rb,3.8898e+01_rb/)
      kao(:, 1,12,13) = (/ &
     &6.3490e+02_rb,5.5553e+02_rb,4.7617e+02_rb,3.9681e+02_rb,3.1745e+02_rb,2.3809e+02_rb, &
     &1.5872e+02_rb,8.3817e+01_rb,4.2920e+01_rb/)
      kao(:, 2,12,13) = (/ &
     &1.0964e+03_rb,9.5936e+02_rb,8.2231e+02_rb,6.8526e+02_rb,5.4821e+02_rb,4.1116e+02_rb, &
     &2.7410e+02_rb,1.3705e+02_rb,4.3006e+01_rb/)
      kao(:, 3,12,13) = (/ &
     &1.7602e+03_rb,1.5402e+03_rb,1.3202e+03_rb,1.1001e+03_rb,8.8010e+02_rb,6.6008e+02_rb, &
     &4.4005e+02_rb,2.2003e+02_rb,4.2996e+01_rb/)
      kao(:, 4,12,13) = (/ &
     &2.6586e+03_rb,2.3263e+03_rb,1.9940e+03_rb,1.6616e+03_rb,1.3293e+03_rb,9.9698e+02_rb, &
     &6.6465e+02_rb,3.3233e+02_rb,4.2901e+01_rb/)
      kao(:, 5,12,13) = (/ &
     &3.8091e+03_rb,3.3329e+03_rb,2.8568e+03_rb,2.3807e+03_rb,1.9045e+03_rb,1.4284e+03_rb, &
     &9.5227e+02_rb,4.7613e+02_rb,4.2723e+01_rb/)
      kao(:, 1,13,13) = (/ &
     &5.9158e+02_rb,5.1763e+02_rb,4.4368e+02_rb,3.6974e+02_rb,2.9579e+02_rb,2.2184e+02_rb, &
     &1.4790e+02_rb,8.4889e+01_rb,4.6678e+01_rb/)
      kao(:, 2,13,13) = (/ &
     &1.0244e+03_rb,8.9631e+02_rb,7.6827e+02_rb,6.4022e+02_rb,5.1218e+02_rb,3.8414e+02_rb, &
     &2.5609e+02_rb,1.2940e+02_rb,4.6683e+01_rb/)
      kao(:, 3,13,13) = (/ &
     &1.6452e+03_rb,1.4395e+03_rb,1.2339e+03_rb,1.0282e+03_rb,8.2259e+02_rb,6.1694e+02_rb, &
     &4.1130e+02_rb,2.0565e+02_rb,4.6592e+01_rb/)
      kao(:, 4,13,13) = (/ &
     &2.4841e+03_rb,2.1736e+03_rb,1.8631e+03_rb,1.5526e+03_rb,1.2421e+03_rb,9.3154e+02_rb, &
     &6.2103e+02_rb,3.1052e+02_rb,4.6412e+01_rb/)
      kao(:, 5,13,13) = (/ &
     &3.5669e+03_rb,3.1210e+03_rb,2.6752e+03_rb,2.2293e+03_rb,1.7834e+03_rb,1.3376e+03_rb, &
     &8.9172e+02_rb,4.4586e+02_rb,4.6169e+01_rb/)
      kao(:, 1, 1,14) = (/ &
     &2.0814e+04_rb,1.8212e+04_rb,1.5610e+04_rb,1.3008e+04_rb,1.0407e+04_rb,7.8051e+03_rb, &
     &5.2034e+03_rb,2.6017e+03_rb,4.7014e-03_rb/)
      kao(:, 2, 1,14) = (/ &
     &2.6951e+04_rb,2.3582e+04_rb,2.0213e+04_rb,1.6844e+04_rb,1.3475e+04_rb,1.0107e+04_rb, &
     &6.7377e+03_rb,3.3688e+03_rb,6.0711e-03_rb/)
      kao(:, 3, 1,14) = (/ &
     &3.3875e+04_rb,2.9641e+04_rb,2.5406e+04_rb,2.1172e+04_rb,1.6938e+04_rb,1.2703e+04_rb, &
     &8.4688e+03_rb,4.2344e+03_rb,2.5176e-02_rb/)
      kao(:, 4, 1,14) = (/ &
     &4.1606e+04_rb,3.6405e+04_rb,3.1205e+04_rb,2.6004e+04_rb,2.0803e+04_rb,1.5602e+04_rb, &
     &1.0402e+04_rb,5.2008e+03_rb,1.0468e-01_rb/)
      kao(:, 5, 1,14) = (/ &
     &5.0069e+04_rb,4.3810e+04_rb,3.7551e+04_rb,3.1293e+04_rb,2.5034e+04_rb,1.8776e+04_rb, &
     &1.2517e+04_rb,6.2586e+03_rb,1.0717e-01_rb/)
      kao(:, 1, 2,14) = (/ &
     &1.9736e+04_rb,1.7269e+04_rb,1.4802e+04_rb,1.2335e+04_rb,9.8681e+03_rb,7.4011e+03_rb, &
     &4.9341e+03_rb,2.4670e+03_rb,1.4424e-01_rb/)
      kao(:, 2, 2,14) = (/ &
     &2.5938e+04_rb,2.2696e+04_rb,1.9454e+04_rb,1.6211e+04_rb,1.2969e+04_rb,9.7268e+03_rb, &
     &6.4845e+03_rb,3.2423e+03_rb,5.4893e+00_rb/)
      kao(:, 3, 2,14) = (/ &
     &3.3117e+04_rb,2.8978e+04_rb,2.4838e+04_rb,2.0698e+04_rb,1.6559e+04_rb,1.2419e+04_rb, &
     &8.2793e+03_rb,4.1397e+03_rb,6.5583e+00_rb/)
      kao(:, 4, 2,14) = (/ &
     &4.1192e+04_rb,3.6043e+04_rb,3.0894e+04_rb,2.5745e+04_rb,2.0596e+04_rb,1.5447e+04_rb, &
     &1.0298e+04_rb,5.1490e+03_rb,7.2955e+00_rb/)
      kao(:, 5, 2,14) = (/ &
     &5.0084e+04_rb,4.3824e+04_rb,3.7563e+04_rb,3.1303e+04_rb,2.5042e+04_rb,1.8782e+04_rb, &
     &1.2521e+04_rb,6.2605e+03_rb,4.5976e+00_rb/)
      kao(:, 1, 3,14) = (/ &
     &1.7352e+04_rb,1.5183e+04_rb,1.3014e+04_rb,1.0845e+04_rb,8.6759e+03_rb,6.5069e+03_rb, &
     &4.3379e+03_rb,2.1690e+03_rb,1.3353e+01_rb/)
      kao(:, 2, 3,14) = (/ &
     &2.3378e+04_rb,2.0456e+04_rb,1.7533e+04_rb,1.4611e+04_rb,1.1689e+04_rb,8.7667e+03_rb, &
     &5.8445e+03_rb,2.9222e+03_rb,1.3389e+01_rb/)
      kao(:, 3, 3,14) = (/ &
     &3.0448e+04_rb,2.6642e+04_rb,2.2836e+04_rb,1.9030e+04_rb,1.5224e+04_rb,1.1418e+04_rb, &
     &7.6119e+03_rb,3.8060e+03_rb,1.3397e+01_rb/)
      kao(:, 4, 3,14) = (/ &
     &3.8507e+04_rb,3.3693e+04_rb,2.8880e+04_rb,2.4067e+04_rb,1.9253e+04_rb,1.4440e+04_rb, &
     &9.6266e+03_rb,4.8133e+03_rb,1.3385e+01_rb/)
      kao(:, 5, 3,14) = (/ &
     &4.7469e+04_rb,4.1535e+04_rb,3.5602e+04_rb,2.9668e+04_rb,2.3734e+04_rb,1.7801e+04_rb, &
     &1.1867e+04_rb,5.9336e+03_rb,1.3346e+01_rb/)
      kao(:, 1, 4,14) = (/ &
     &1.4631e+04_rb,1.2802e+04_rb,1.0973e+04_rb,9.1445e+03_rb,7.3156e+03_rb,5.4867e+03_rb, &
     &3.6578e+03_rb,1.8289e+03_rb,1.5943e+01_rb/)
      kao(:, 2, 4,14) = (/ &
     &2.0262e+04_rb,1.7729e+04_rb,1.5197e+04_rb,1.2664e+04_rb,1.0131e+04_rb,7.5983e+03_rb, &
     &5.0655e+03_rb,2.5328e+03_rb,1.6007e+01_rb/)
      kao(:, 3, 4,14) = (/ &
     &2.6988e+04_rb,2.3615e+04_rb,2.0241e+04_rb,1.6868e+04_rb,1.3494e+04_rb,1.0121e+04_rb, &
     &6.7470e+03_rb,3.3735e+03_rb,1.6031e+01_rb/)
      kao(:, 4, 4,14) = (/ &
     &3.4853e+04_rb,3.0497e+04_rb,2.6140e+04_rb,2.1783e+04_rb,1.7427e+04_rb,1.3070e+04_rb, &
     &8.7133e+03_rb,4.3567e+03_rb,1.6028e+01_rb/)
      kao(:, 5, 4,14) = (/ &
     &4.3821e+04_rb,3.8343e+04_rb,3.2866e+04_rb,2.7388e+04_rb,2.1910e+04_rb,1.6433e+04_rb, &
     &1.0955e+04_rb,5.4776e+03_rb,1.6009e+01_rb/)
      kao(:, 1, 5,14) = (/ &
     &1.2010e+04_rb,1.0509e+04_rb,9.0077e+03_rb,7.5064e+03_rb,6.0051e+03_rb,4.5038e+03_rb, &
     &3.0026e+03_rb,1.5013e+03_rb,1.9009e+01_rb/)
      kao(:, 2, 5,14) = (/ &
     &1.7127e+04_rb,1.4986e+04_rb,1.2846e+04_rb,1.0705e+04_rb,8.5637e+03_rb,6.4228e+03_rb, &
     &4.2818e+03_rb,2.1409e+03_rb,1.9111e+01_rb/)
      kao(:, 3, 5,14) = (/ &
     &2.3449e+04_rb,2.0517e+04_rb,1.7586e+04_rb,1.4655e+04_rb,1.1724e+04_rb,8.7932e+03_rb, &
     &5.8621e+03_rb,2.9311e+03_rb,1.9171e+01_rb/)
      kao(:, 4, 5,14) = (/ &
     &3.0924e+04_rb,2.7059e+04_rb,2.3193e+04_rb,1.9328e+04_rb,1.5462e+04_rb,1.1597e+04_rb, &
     &7.7310e+03_rb,3.8655e+03_rb,1.9184e+01_rb/)
      kao(:, 5, 5,14) = (/ &
     &3.9495e+04_rb,3.4558e+04_rb,2.9621e+04_rb,2.4684e+04_rb,1.9748e+04_rb,1.4811e+04_rb, &
     &9.8738e+03_rb,4.9369e+03_rb,1.9172e+01_rb/)
      kao(:, 1, 6,14) = (/ &
     &9.4628e+03_rb,8.2800e+03_rb,7.0971e+03_rb,5.9142e+03_rb,4.7314e+03_rb,3.5485e+03_rb, &
     &2.3657e+03_rb,1.1828e+03_rb,2.2644e+01_rb/)
      kao(:, 2, 6,14) = (/ &
     &1.3981e+04_rb,1.2233e+04_rb,1.0486e+04_rb,8.7381e+03_rb,6.9905e+03_rb,5.2429e+03_rb, &
     &3.4952e+03_rb,1.7476e+03_rb,2.2781e+01_rb/)
      kao(:, 3, 6,14) = (/ &
     &1.9648e+04_rb,1.7192e+04_rb,1.4736e+04_rb,1.2280e+04_rb,9.8239e+03_rb,7.3679e+03_rb, &
     &4.9119e+03_rb,2.4560e+03_rb,2.2869e+01_rb/)
      kao(:, 4, 6,14) = (/ &
     &2.6483e+04_rb,2.3173e+04_rb,1.9862e+04_rb,1.6552e+04_rb,1.3242e+04_rb,9.9312e+03_rb, &
     &6.6208e+03_rb,3.3104e+03_rb,2.2915e+01_rb/)
      kao(:, 5, 6,14) = (/ &
     &3.4470e+04_rb,3.0161e+04_rb,2.5853e+04_rb,2.1544e+04_rb,1.7235e+04_rb,1.2926e+04_rb, &
     &8.6175e+03_rb,4.3088e+03_rb,2.2923e+01_rb/)
      kao(:, 1, 7,14) = (/ &
     &7.3078e+03_rb,6.3943e+03_rb,5.4808e+03_rb,4.5673e+03_rb,3.6539e+03_rb,2.7404e+03_rb, &
     &1.8269e+03_rb,9.1346e+02_rb,2.6896e+01_rb/)
      kao(:, 2, 7,14) = (/ &
     &1.1178e+04_rb,9.7811e+03_rb,8.3838e+03_rb,6.9865e+03_rb,5.5892e+03_rb,4.1919e+03_rb, &
     &2.7946e+03_rb,1.3973e+03_rb,2.7090e+01_rb/)
      kao(:, 3, 7,14) = (/ &
     &1.6168e+04_rb,1.4147e+04_rb,1.2126e+04_rb,1.0105e+04_rb,8.0839e+03_rb,6.0629e+03_rb, &
     &4.0420e+03_rb,2.0210e+03_rb,2.7221e+01_rb/)
      kao(:, 4, 7,14) = (/ &
     &2.2360e+04_rb,1.9565e+04_rb,1.6770e+04_rb,1.3975e+04_rb,1.1180e+04_rb,8.3848e+03_rb, &
     &5.5899e+03_rb,2.7949e+03_rb,2.7298e+01_rb/)
      kao(:, 5, 7,14) = (/ &
     &2.9827e+04_rb,2.6099e+04_rb,2.2370e+04_rb,1.8642e+04_rb,1.4913e+04_rb,1.1185e+04_rb, &
     &7.4567e+03_rb,3.7283e+03_rb,2.7315e+01_rb/)
      kao(:, 1, 8,14) = (/ &
     &5.5945e+03_rb,4.8952e+03_rb,4.1959e+03_rb,3.4966e+03_rb,2.7972e+03_rb,2.0979e+03_rb, &
     &1.3986e+03_rb,6.9931e+02_rb,3.1842e+01_rb/)
      kao(:, 2, 8,14) = (/ &
     &8.8907e+03_rb,7.7794e+03_rb,6.6680e+03_rb,5.5567e+03_rb,4.4454e+03_rb,3.3340e+03_rb, &
     &2.2227e+03_rb,1.1113e+03_rb,3.2101e+01_rb/)
      kao(:, 3, 8,14) = (/ &
     &1.3296e+04_rb,1.1634e+04_rb,9.9724e+03_rb,8.3103e+03_rb,6.6483e+03_rb,4.9862e+03_rb, &
     &3.3241e+03_rb,1.6621e+03_rb,3.2282e+01_rb/)
      kao(:, 4, 8,14) = (/ &
     &1.8955e+04_rb,1.6586e+04_rb,1.4216e+04_rb,1.1847e+04_rb,9.4776e+03_rb,7.1082e+03_rb, &
     &4.7388e+03_rb,2.3694e+03_rb,3.2388e+01_rb/)
      kao(:, 5, 8,14) = (/ &
     &2.5906e+04_rb,2.2668e+04_rb,1.9430e+04_rb,1.6191e+04_rb,1.2953e+04_rb,9.7149e+03_rb, &
     &6.4766e+03_rb,3.2383e+03_rb,3.2425e+01_rb/)
      kao(:, 1, 9,14) = (/ &
     &4.1067e+03_rb,3.5934e+03_rb,3.0800e+03_rb,2.5667e+03_rb,2.0533e+03_rb,1.5400e+03_rb, &
     &1.0267e+03_rb,5.1334e+02_rb,3.7521e+01_rb/)
      kao(:, 2, 9,14) = (/ &
     &6.8230e+03_rb,5.9701e+03_rb,5.1172e+03_rb,4.2644e+03_rb,3.4115e+03_rb,2.5586e+03_rb, &
     &1.7057e+03_rb,8.5287e+02_rb,3.7865e+01_rb/)
      kao(:, 3, 9,14) = (/ &
     &1.0614e+04_rb,9.2875e+03_rb,7.9607e+03_rb,6.6339e+03_rb,5.3071e+03_rb,3.9804e+03_rb, &
     &2.6536e+03_rb,1.3268e+03_rb,3.8104e+01_rb/)
      kao(:, 4, 9,14) = (/ &
     &1.5585e+04_rb,1.3637e+04_rb,1.1689e+04_rb,9.7408e+03_rb,7.7926e+03_rb,5.8445e+03_rb, &
     &3.8963e+03_rb,1.9481e+03_rb,3.8251e+01_rb/)
      kao(:, 5, 9,14) = (/ &
     &2.1772e+04_rb,1.9051e+04_rb,1.6329e+04_rb,1.3608e+04_rb,1.0886e+04_rb,8.1646e+03_rb, &
     &5.4430e+03_rb,2.7215e+03_rb,3.8308e+01_rb/)
      kao(:, 1,10,14) = (/ &
     &3.0498e+03_rb,2.6686e+03_rb,2.2874e+03_rb,1.9061e+03_rb,1.5249e+03_rb,1.1437e+03_rb, &
     &7.6245e+02_rb,3.8122e+02_rb,4.4006e+01_rb/)
      kao(:, 2,10,14) = (/ &
     &5.3046e+03_rb,4.6415e+03_rb,3.9785e+03_rb,3.3154e+03_rb,2.6523e+03_rb,1.9892e+03_rb, &
     &1.3261e+03_rb,6.6307e+02_rb,4.4440e+01_rb/)
      kao(:, 3,10,14) = (/ &
     &8.5201e+03_rb,7.4551e+03_rb,6.3901e+03_rb,5.3250e+03_rb,4.2600e+03_rb,3.1950e+03_rb, &
     &2.1300e+03_rb,1.0650e+03_rb,4.4746e+01_rb/)
      kao(:, 4,10,14) = (/ &
     &1.2818e+04_rb,1.1216e+04_rb,9.6138e+03_rb,8.0115e+03_rb,6.4092e+03_rb,4.8069e+03_rb, &
     &3.2046e+03_rb,1.6023e+03_rb,4.4928e+01_rb/)
      kao(:, 5,10,14) = (/ &
     &1.8340e+04_rb,1.6048e+04_rb,1.3755e+04_rb,1.1463e+04_rb,9.1701e+03_rb,6.8776e+03_rb, &
     &4.5851e+03_rb,2.2925e+03_rb,4.5005e+01_rb/)
      kao(:, 1,11,14) = (/ &
     &2.9434e+03_rb,2.5755e+03_rb,2.2076e+03_rb,1.8396e+03_rb,1.4717e+03_rb,1.1038e+03_rb, &
     &7.3585e+02_rb,3.6793e+02_rb,5.1570e+01_rb/)
      kao(:, 2,11,14) = (/ &
     &5.1056e+03_rb,4.4674e+03_rb,3.8292e+03_rb,3.1910e+03_rb,2.5528e+03_rb,1.9146e+03_rb, &
     &1.2764e+03_rb,6.3819e+02_rb,5.2012e+01_rb/)
      kao(:, 3,11,14) = (/ &
     &8.1976e+03_rb,7.1728e+03_rb,6.1481e+03_rb,5.1234e+03_rb,4.0987e+03_rb,3.0741e+03_rb, &
     &2.0494e+03_rb,1.0247e+03_rb,5.2312e+01_rb/)
      kao(:, 4,11,14) = (/ &
     &1.2377e+04_rb,1.0830e+04_rb,9.2826e+03_rb,7.7355e+03_rb,6.1884e+03_rb,4.6413e+03_rb, &
     &3.0942e+03_rb,1.5471e+03_rb,5.2478e+01_rb/)
      kao(:, 5,11,14) = (/ &
     &1.7764e+04_rb,1.5544e+04_rb,1.3323e+04_rb,1.1103e+04_rb,8.8820e+03_rb,6.6616e+03_rb, &
     &4.4410e+03_rb,2.2205e+03_rb,5.2507e+01_rb/)
      kao(:, 1,12,14) = (/ &
     &2.8315e+03_rb,2.4776e+03_rb,2.1237e+03_rb,1.7697e+03_rb,1.4158e+03_rb,1.0618e+03_rb, &
     &7.0789e+02_rb,3.5394e+02_rb,5.9960e+01_rb/)
      kao(:, 2,12,14) = (/ &
     &4.9224e+03_rb,4.3071e+03_rb,3.6918e+03_rb,3.0765e+03_rb,2.4612e+03_rb,1.8459e+03_rb, &
     &1.2306e+03_rb,6.1530e+02_rb,6.0405e+01_rb/)
      kao(:, 3,12,14) = (/ &
     &7.9376e+03_rb,6.9454e+03_rb,5.9532e+03_rb,4.9610e+03_rb,3.9688e+03_rb,2.9766e+03_rb, &
     &1.9844e+03_rb,9.9220e+02_rb,6.0667e+01_rb/)
      kao(:, 4,12,14) = (/ &
     &1.2004e+04_rb,1.0504e+04_rb,9.0032e+03_rb,7.5027e+03_rb,6.0021e+03_rb,4.5016e+03_rb, &
     &3.0011e+03_rb,1.5005e+03_rb,6.0769e+01_rb/)
      kao(:, 5,12,14) = (/ &
     &1.7211e+04_rb,1.5059e+04_rb,1.2908e+04_rb,1.0757e+04_rb,8.6054e+03_rb,6.4541e+03_rb, &
     &4.3027e+03_rb,2.1514e+03_rb,6.0741e+01_rb/)
      kao(:, 1,13,14) = (/ &
     &2.8060e+03_rb,2.4552e+03_rb,2.1045e+03_rb,1.7537e+03_rb,1.4030e+03_rb,1.0522e+03_rb, &
     &7.0149e+02_rb,3.5075e+02_rb,6.9076e+01_rb/)
      kao(:, 2,13,14) = (/ &
     &4.9007e+03_rb,4.2881e+03_rb,3.6756e+03_rb,3.0630e+03_rb,2.4504e+03_rb,1.8378e+03_rb, &
     &1.2252e+03_rb,6.1260e+02_rb,6.9478e+01_rb/)
      kao(:, 3,13,14) = (/ &
     &7.8921e+03_rb,6.9056e+03_rb,5.9191e+03_rb,4.9326e+03_rb,3.9460e+03_rb,2.9595e+03_rb, &
     &1.9730e+03_rb,9.8652e+02_rb,6.9694e+01_rb/)
      kao(:, 4,13,14) = (/ &
     &1.1933e+04_rb,1.0441e+04_rb,8.9495e+03_rb,7.4579e+03_rb,5.9663e+03_rb,4.4748e+03_rb, &
     &2.9832e+03_rb,1.4916e+03_rb,6.9728e+01_rb/)
      kao(:, 5,13,14) = (/ &
     &1.7152e+04_rb,1.5008e+04_rb,1.2864e+04_rb,1.0720e+04_rb,8.5759e+03_rb,6.4320e+03_rb, &
     &4.2880e+03_rb,2.1440e+03_rb,6.9601e+01_rb/)
      kao(:, 1, 1,15) = (/ &
     &3.5423e+04_rb,3.0995e+04_rb,2.6567e+04_rb,2.2139e+04_rb,1.7712e+04_rb,1.3284e+04_rb, &
     &8.8558e+03_rb,4.4279e+03_rb,7.1610e-03_rb/)
      kao(:, 2, 1,15) = (/ &
     &4.5820e+04_rb,4.0093e+04_rb,3.4365e+04_rb,2.8638e+04_rb,2.2910e+04_rb,1.7183e+04_rb, &
     &1.1455e+04_rb,5.7276e+03_rb,7.1091e-03_rb/)
      kao(:, 3, 1,15) = (/ &
     &5.7623e+04_rb,5.0421e+04_rb,4.3218e+04_rb,3.6015e+04_rb,2.8812e+04_rb,2.1609e+04_rb, &
     &1.4406e+04_rb,7.2029e+03_rb,7.8150e-03_rb/)
      kao(:, 4, 1,15) = (/ &
     &7.0726e+04_rb,6.1885e+04_rb,5.3044e+04_rb,4.4204e+04_rb,3.5363e+04_rb,2.6522e+04_rb, &
     &1.7681e+04_rb,8.8407e+03_rb,4.0966e-02_rb/)
      kao(:, 5, 1,15) = (/ &
     &8.4947e+04_rb,7.4329e+04_rb,6.3711e+04_rb,5.3092e+04_rb,4.2474e+04_rb,3.1855e+04_rb, &
     &2.1237e+04_rb,1.0618e+04_rb,1.7079e-01_rb/)
      kao(:, 1, 2,15) = (/ &
     &3.6296e+04_rb,3.1759e+04_rb,2.7222e+04_rb,2.2685e+04_rb,1.8148e+04_rb,1.3611e+04_rb, &
     &9.0740e+03_rb,4.5370e+03_rb,7.6264e-03_rb/)
      kao(:, 2, 2,15) = (/ &
     &4.7676e+04_rb,4.1717e+04_rb,3.5757e+04_rb,2.9798e+04_rb,2.3838e+04_rb,1.7879e+04_rb, &
     &1.1919e+04_rb,5.9595e+03_rb,7.8530e+00_rb/)
      kao(:, 3, 2,15) = (/ &
     &6.0722e+04_rb,5.3132e+04_rb,4.5542e+04_rb,3.7951e+04_rb,3.0361e+04_rb,2.2771e+04_rb, &
     &1.5181e+04_rb,7.5903e+03_rb,1.1682e+01_rb/)
      kao(:, 4, 2,15) = (/ &
     &7.5305e+04_rb,6.5892e+04_rb,5.6479e+04_rb,4.7065e+04_rb,3.7652e+04_rb,2.8239e+04_rb, &
     &1.8826e+04_rb,9.4131e+03_rb,7.5532e+00_rb/)
      kao(:, 5, 2,15) = (/ &
     &9.1241e+04_rb,7.9835e+04_rb,6.8430e+04_rb,5.7025e+04_rb,4.5620e+04_rb,3.4215e+04_rb, &
     &2.2810e+04_rb,1.1405e+04_rb,1.0957e+00_rb/)
      kao(:, 1, 3,15) = (/ &
     &3.4715e+04_rb,3.0376e+04_rb,2.6036e+04_rb,2.1697e+04_rb,1.7358e+04_rb,1.3018e+04_rb, &
     &8.6788e+03_rb,4.3394e+03_rb,1.4232e+01_rb/)
      kao(:, 2, 3,15) = (/ &
     &4.6613e+04_rb,4.0787e+04_rb,3.4960e+04_rb,2.9133e+04_rb,2.3307e+04_rb,1.7480e+04_rb, &
     &1.1653e+04_rb,5.8267e+03_rb,1.4235e+01_rb/)
      kao(:, 3, 3,15) = (/ &
     &6.0451e+04_rb,5.2894e+04_rb,4.5338e+04_rb,3.7782e+04_rb,3.0225e+04_rb,2.2669e+04_rb, &
     &1.5113e+04_rb,7.5563e+03_rb,1.4209e+01_rb/)
      kao(:, 4, 3,15) = (/ &
     &7.6104e+04_rb,6.6591e+04_rb,5.7078e+04_rb,4.7565e+04_rb,3.8052e+04_rb,2.8539e+04_rb, &
     &1.9026e+04_rb,9.5130e+03_rb,1.4121e+01_rb/)
      kao(:, 5, 3,15) = (/ &
     &9.3385e+04_rb,8.1712e+04_rb,7.0039e+04_rb,5.8365e+04_rb,4.6692e+04_rb,3.5019e+04_rb, &
     &2.3346e+04_rb,1.1673e+04_rb,1.4063e+01_rb/)
      kao(:, 1, 4,15) = (/ &
     &3.1974e+04_rb,2.7977e+04_rb,2.3981e+04_rb,1.9984e+04_rb,1.5987e+04_rb,1.1990e+04_rb, &
     &7.9936e+03_rb,3.9968e+03_rb,1.7130e+01_rb/)
      kao(:, 2, 4,15) = (/ &
     &4.4033e+04_rb,3.8529e+04_rb,3.3025e+04_rb,2.7521e+04_rb,2.2017e+04_rb,1.6512e+04_rb, &
     &1.1008e+04_rb,5.5042e+03_rb,1.7154e+01_rb/)
      kao(:, 3, 4,15) = (/ &
     &5.8299e+04_rb,5.1012e+04_rb,4.3725e+04_rb,3.6437e+04_rb,2.9150e+04_rb,2.1862e+04_rb, &
     &1.4575e+04_rb,7.2874e+03_rb,1.7147e+01_rb/)
      kao(:, 4, 4,15) = (/ &
     &7.4658e+04_rb,6.5325e+04_rb,5.5993e+04_rb,4.6661e+04_rb,3.7329e+04_rb,2.7997e+04_rb, &
     &1.8664e+04_rb,9.3322e+03_rb,1.7125e+01_rb/)
      kao(:, 5, 4,15) = (/ &
     &9.3061e+04_rb,8.1428e+04_rb,6.9796e+04_rb,5.8163e+04_rb,4.6530e+04_rb,3.4898e+04_rb, &
     &2.3265e+04_rb,1.1633e+04_rb,1.7053e+01_rb/)
      kao(:, 1, 5,15) = (/ &
     &2.8780e+04_rb,2.5183e+04_rb,2.1585e+04_rb,1.7988e+04_rb,1.4390e+04_rb,1.0793e+04_rb, &
     &7.1950e+03_rb,3.5975e+03_rb,2.0610e+01_rb/)
      kao(:, 2, 5,15) = (/ &
     &4.0696e+04_rb,3.5609e+04_rb,3.0522e+04_rb,2.5435e+04_rb,2.0348e+04_rb,1.5261e+04_rb, &
     &1.0174e+04_rb,5.0871e+03_rb,2.0673e+01_rb/)
      kao(:, 3, 5,15) = (/ &
     &5.5132e+04_rb,4.8240e+04_rb,4.1349e+04_rb,3.4457e+04_rb,2.7566e+04_rb,2.0674e+04_rb, &
     &1.3783e+04_rb,6.8915e+03_rb,2.0688e+01_rb/)
      kao(:, 4, 5,15) = (/ &
     &7.2202e+04_rb,6.3177e+04_rb,5.4152e+04_rb,4.5126e+04_rb,3.6101e+04_rb,2.7076e+04_rb, &
     &1.8051e+04_rb,9.0253e+03_rb,2.0678e+01_rb/)
      kao(:, 5, 5,15) = (/ &
     &9.1794e+04_rb,8.0320e+04_rb,6.8846e+04_rb,5.7371e+04_rb,4.5897e+04_rb,3.4423e+04_rb, &
     &2.2949e+04_rb,1.1474e+04_rb,2.0627e+01_rb/)
      kao(:, 1, 6,15) = (/ &
     &2.4928e+04_rb,2.1812e+04_rb,1.8696e+04_rb,1.5580e+04_rb,1.2464e+04_rb,9.3479e+03_rb, &
     &6.2319e+03_rb,3.1160e+03_rb,2.4743e+01_rb/)
      kao(:, 2, 6,15) = (/ &
     &3.6459e+04_rb,3.1902e+04_rb,2.7344e+04_rb,2.2787e+04_rb,1.8230e+04_rb,1.3672e+04_rb, &
     &9.1148e+03_rb,4.5574e+03_rb,2.4868e+01_rb/)
      kao(:, 3, 6,15) = (/ &
     &5.0921e+04_rb,4.4556e+04_rb,3.8191e+04_rb,3.1826e+04_rb,2.5461e+04_rb,1.9095e+04_rb, &
     &1.2730e+04_rb,6.3651e+03_rb,2.4935e+01_rb/)
      kao(:, 4, 6,15) = (/ &
     &6.8337e+04_rb,5.9795e+04_rb,5.1253e+04_rb,4.2711e+04_rb,3.4168e+04_rb,2.5626e+04_rb, &
     &1.7084e+04_rb,8.5421e+03_rb,2.4952e+01_rb/)
      kao(:, 5, 6,15) = (/ &
     &8.8665e+04_rb,7.7582e+04_rb,6.6499e+04_rb,5.5416e+04_rb,4.4333e+04_rb,3.3249e+04_rb, &
     &2.2166e+04_rb,1.1083e+04_rb,2.4917e+01_rb/)
      kao(:, 1, 7,15) = (/ &
     &2.1322e+04_rb,1.8657e+04_rb,1.5992e+04_rb,1.3326e+04_rb,1.0661e+04_rb,7.9957e+03_rb, &
     &5.3305e+03_rb,2.6652e+03_rb,2.9665e+01_rb/)
      kao(:, 2, 7,15) = (/ &
     &3.2413e+04_rb,2.8361e+04_rb,2.4310e+04_rb,2.0258e+04_rb,1.6206e+04_rb,1.2155e+04_rb, &
     &8.1031e+03_rb,4.0516e+03_rb,2.9861e+01_rb/)
      kao(:, 3, 7,15) = (/ &
     &4.6696e+04_rb,4.0859e+04_rb,3.5022e+04_rb,2.9185e+04_rb,2.3348e+04_rb,1.7511e+04_rb, &
     &1.1674e+04_rb,5.8369e+03_rb,3.0001e+01_rb/)
      kao(:, 4, 7,15) = (/ &
     &6.4261e+04_rb,5.6228e+04_rb,4.8195e+04_rb,4.0163e+04_rb,3.2130e+04_rb,2.4098e+04_rb, &
     &1.6065e+04_rb,8.0325e+03_rb,3.0074e+01_rb/)
      kao(:, 5, 7,15) = (/ &
     &8.5103e+04_rb,7.4465e+04_rb,6.3827e+04_rb,5.3189e+04_rb,4.2551e+04_rb,3.1914e+04_rb, &
     &2.1276e+04_rb,1.0638e+04_rb,3.0077e+01_rb/)
      kao(:, 1, 8,15) = (/ &
     &1.8276e+04_rb,1.5991e+04_rb,1.3707e+04_rb,1.1422e+04_rb,9.1380e+03_rb,6.8535e+03_rb, &
     &4.5690e+03_rb,2.2845e+03_rb,3.5525e+01_rb/)
      kao(:, 2, 8,15) = (/ &
     &2.8919e+04_rb,2.5304e+04_rb,2.1689e+04_rb,1.8074e+04_rb,1.4459e+04_rb,1.0845e+04_rb, &
     &7.2298e+03_rb,3.6149e+03_rb,3.5844e+01_rb/)
      kao(:, 3, 8,15) = (/ &
     &4.2999e+04_rb,3.7624e+04_rb,3.2249e+04_rb,2.6874e+04_rb,2.1500e+04_rb,1.6125e+04_rb, &
     &1.0750e+04_rb,5.3749e+03_rb,3.6058e+01_rb/)
      kao(:, 4, 8,15) = (/ &
     &6.0712e+04_rb,5.3123e+04_rb,4.5534e+04_rb,3.7945e+04_rb,3.0356e+04_rb,2.2767e+04_rb, &
     &1.5178e+04_rb,7.5890e+03_rb,3.6184e+01_rb/)
      kao(:, 5, 8,15) = (/ &
     &8.2146e+04_rb,7.1878e+04_rb,6.1610e+04_rb,5.1341e+04_rb,4.1073e+04_rb,3.0805e+04_rb, &
     &2.0537e+04_rb,1.0268e+04_rb,3.6243e+01_rb/)
      kao(:, 1, 9,15) = (/ &
     &1.5122e+04_rb,1.3232e+04_rb,1.1341e+04_rb,9.4512e+03_rb,7.5610e+03_rb,5.6707e+03_rb, &
     &3.7805e+03_rb,1.8902e+03_rb,4.2513e+01_rb/)
      kao(:, 2, 9,15) = (/ &
     &2.4972e+04_rb,2.1851e+04_rb,1.8729e+04_rb,1.5608e+04_rb,1.2486e+04_rb,9.3646e+03_rb, &
     &6.2431e+03_rb,3.1215e+03_rb,4.2950e+01_rb/)
      kao(:, 3, 9,15) = (/ &
     &3.8412e+04_rb,3.3611e+04_rb,2.8809e+04_rb,2.4007e+04_rb,1.9206e+04_rb,1.4404e+04_rb, &
     &9.6030e+03_rb,4.8015e+03_rb,4.3278e+01_rb/)
      kao(:, 4, 9,15) = (/ &
     &5.5843e+04_rb,4.8863e+04_rb,4.1882e+04_rb,3.4902e+04_rb,2.7921e+04_rb,2.0941e+04_rb, &
     &1.3961e+04_rb,6.9804e+03_rb,4.3480e+01_rb/)
      kao(:, 5, 9,15) = (/ &
     &7.7611e+04_rb,6.7910e+04_rb,5.8208e+04_rb,4.8507e+04_rb,3.8805e+04_rb,2.9104e+04_rb, &
     &1.9403e+04_rb,9.7013e+03_rb,4.3587e+01_rb/)
      kao(:, 1,10,15) = (/ &
     &1.2684e+04_rb,1.1099e+04_rb,9.5132e+03_rb,7.9276e+03_rb,6.3421e+03_rb,4.7566e+03_rb, &
     &3.1710e+03_rb,1.5855e+03_rb,5.0840e+01_rb/)
      kao(:, 2,10,15) = (/ &
     &2.1787e+04_rb,1.9064e+04_rb,1.6340e+04_rb,1.3617e+04_rb,1.0893e+04_rb,8.1700e+03_rb, &
     &5.4467e+03_rb,2.7233e+03_rb,5.1430e+01_rb/)
      kao(:, 3,10,15) = (/ &
     &3.4684e+04_rb,3.0349e+04_rb,2.6013e+04_rb,2.1678e+04_rb,1.7342e+04_rb,1.3007e+04_rb, &
     &8.6710e+03_rb,4.3355e+03_rb,5.1863e+01_rb/)
      kao(:, 4,10,15) = (/ &
     &5.1947e+04_rb,4.5454e+04_rb,3.8960e+04_rb,3.2467e+04_rb,2.5974e+04_rb,1.9480e+04_rb, &
     &1.2987e+04_rb,6.4934e+03_rb,5.2160e+01_rb/)
      kao(:, 5,10,15) = (/ &
     &7.3912e+04_rb,6.4673e+04_rb,5.5434e+04_rb,4.6195e+04_rb,3.6956e+04_rb,2.7717e+04_rb, &
     &1.8478e+04_rb,9.2389e+03_rb,5.2321e+01_rb/)
      kao(:, 1,11,15) = (/ &
     &1.3711e+04_rb,1.1997e+04_rb,1.0283e+04_rb,8.5694e+03_rb,6.8555e+03_rb,5.1416e+03_rb, &
     &3.4277e+03_rb,1.7139e+03_rb,6.1054e+01_rb/)
      kao(:, 2,11,15) = (/ &
     &2.3669e+04_rb,2.0711e+04_rb,1.7752e+04_rb,1.4793e+04_rb,1.1835e+04_rb,8.8760e+03_rb, &
     &5.9173e+03_rb,2.9586e+03_rb,6.1742e+01_rb/)
      kao(:, 3,11,15) = (/ &
     &3.7832e+04_rb,3.3103e+04_rb,2.8374e+04_rb,2.3645e+04_rb,1.8916e+04_rb,1.4187e+04_rb, &
     &9.4579e+03_rb,4.7289e+03_rb,6.2215e+01_rb/)
      kao(:, 4,11,15) = (/ &
     &5.6757e+04_rb,4.9662e+04_rb,4.2568e+04_rb,3.5473e+04_rb,2.8378e+04_rb,2.1284e+04_rb, &
     &1.4189e+04_rb,7.0945e+03_rb,6.2524e+01_rb/)
      kao(:, 5,11,15) = (/ &
     &8.0798e+04_rb,7.0698e+04_rb,6.0598e+04_rb,5.0499e+04_rb,4.0399e+04_rb,3.0299e+04_rb, &
     &2.0199e+04_rb,1.0100e+04_rb,6.2689e+01_rb/)
      kao(:, 1,12,15) = (/ &
     &1.4989e+04_rb,1.3115e+04_rb,1.1241e+04_rb,9.3679e+03_rb,7.4943e+03_rb,5.6207e+03_rb, &
     &3.7471e+03_rb,1.8736e+03_rb,7.3162e+01_rb/)
      kao(:, 2,12,15) = (/ &
     &2.5949e+04_rb,2.2705e+04_rb,1.9461e+04_rb,1.6218e+04_rb,1.2974e+04_rb,9.7307e+03_rb, &
     &6.4871e+03_rb,3.2436e+03_rb,7.3885e+01_rb/)
      kao(:, 3,12,15) = (/ &
     &4.1513e+04_rb,3.6324e+04_rb,3.1134e+04_rb,2.5945e+04_rb,2.0756e+04_rb,1.5567e+04_rb, &
     &1.0378e+04_rb,5.1891e+03_rb,7.4405e+01_rb/)
      kao(:, 4,12,15) = (/ &
     &6.2315e+04_rb,5.4526e+04_rb,4.6737e+04_rb,3.8947e+04_rb,3.1158e+04_rb,2.3368e+04_rb, &
     &1.5579e+04_rb,7.7894e+03_rb,7.4701e+01_rb/)
      kao(:, 5,12,15) = (/ &
     &8.8861e+04_rb,7.7754e+04_rb,6.6646e+04_rb,5.5538e+04_rb,4.4431e+04_rb,3.3323e+04_rb, &
     &2.2215e+04_rb,1.1108e+04_rb,7.4795e+01_rb/)
      kao(:, 1,13,15) = (/ &
     &1.6915e+04_rb,1.4801e+04_rb,1.2687e+04_rb,1.0572e+04_rb,8.4577e+03_rb,6.3433e+03_rb, &
     &4.2289e+03_rb,2.1144e+03_rb,8.7334e+01_rb/)
      kao(:, 2,13,15) = (/ &
     &2.9284e+04_rb,2.5624e+04_rb,2.1963e+04_rb,1.8303e+04_rb,1.4642e+04_rb,1.0982e+04_rb, &
     &7.3211e+03_rb,3.6605e+03_rb,8.8104e+01_rb/)
      kao(:, 3,13,15) = (/ &
     &4.6892e+04_rb,4.1031e+04_rb,3.5169e+04_rb,2.9308e+04_rb,2.3446e+04_rb,1.7585e+04_rb, &
     &1.1723e+04_rb,5.8616e+03_rb,8.8585e+01_rb/)
      kao(:, 4,13,15) = (/ &
     &7.0448e+04_rb,6.1642e+04_rb,5.2836e+04_rb,4.4030e+04_rb,3.5224e+04_rb,2.6418e+04_rb, &
     &1.7612e+04_rb,8.8061e+03_rb,8.8802e+01_rb/)
      kao(:, 5,13,15) = (/ &
     &1.0039e+05_rb,8.7843e+04_rb,7.5294e+04_rb,6.2745e+04_rb,5.0196e+04_rb,3.7647e+04_rb, &
     &2.5098e+04_rb,1.2549e+04_rb,8.8791e+01_rb/)
      kao(:, 1, 1,16) = (/ &
     &4.3683e+04_rb,3.8223e+04_rb,3.2762e+04_rb,2.7302e+04_rb,2.1842e+04_rb,1.6381e+04_rb, &
     &1.0921e+04_rb,5.4604e+03_rb,7.6073e-03_rb/)
      kao(:, 2, 1,16) = (/ &
     &5.6813e+04_rb,4.9712e+04_rb,4.2610e+04_rb,3.5508e+04_rb,2.8407e+04_rb,2.1305e+04_rb, &
     &1.4203e+04_rb,7.1017e+03_rb,7.6403e-03_rb/)
      kao(:, 3, 1,16) = (/ &
     &7.1746e+04_rb,6.2777e+04_rb,5.3809e+04_rb,4.4841e+04_rb,3.5873e+04_rb,2.6905e+04_rb, &
     &1.7936e+04_rb,8.9682e+03_rb,3.7278e-03_rb/)
      kao(:, 4, 1,16) = (/ &
     &8.8334e+04_rb,7.7292e+04_rb,6.6250e+04_rb,5.5209e+04_rb,4.4167e+04_rb,3.3125e+04_rb, &
     &2.2083e+04_rb,1.1042e+04_rb,3.7292e-03_rb/)
      kao(:, 5, 1,16) = (/ &
     &1.0639e+05_rb,9.3091e+04_rb,7.9793e+04_rb,6.6494e+04_rb,5.3195e+04_rb,3.9896e+04_rb, &
     &2.6598e+04_rb,1.3299e+04_rb,7.8826e-01_rb/)
      kao(:, 1, 2,16) = (/ &
     &4.6897e+04_rb,4.1035e+04_rb,3.5173e+04_rb,2.9311e+04_rb,2.3449e+04_rb,1.7586e+04_rb, &
     &1.1724e+04_rb,5.8622e+03_rb,8.2408e-03_rb/)
      kao(:, 2, 2,16) = (/ &
     &6.1887e+04_rb,5.4151e+04_rb,4.6415e+04_rb,3.8679e+04_rb,3.0943e+04_rb,2.3208e+04_rb, &
     &1.5472e+04_rb,7.7359e+03_rb,1.2063e+01_rb/)
      kao(:, 3, 2,16) = (/ &
     &7.9108e+04_rb,6.9219e+04_rb,5.9331e+04_rb,4.9442e+04_rb,3.9554e+04_rb,2.9666e+04_rb, &
     &1.9777e+04_rb,9.8885e+03_rb,1.1743e+01_rb/)
      kao(:, 4, 2,16) = (/ &
     &9.8415e+04_rb,8.6113e+04_rb,7.3811e+04_rb,6.1509e+04_rb,4.9207e+04_rb,3.6906e+04_rb, &
     &2.4604e+04_rb,1.2302e+04_rb,3.5693e-03_rb/)
      kao(:, 5, 2,16) = (/ &
     &1.1961e+05_rb,1.0466e+05_rb,8.9707e+04_rb,7.4755e+04_rb,5.9804e+04_rb,4.4853e+04_rb, &
     &2.9902e+04_rb,1.4951e+04_rb,5.8824e-01_rb/)
      kao(:, 1, 3,16) = (/ &
     &4.7187e+04_rb,4.1289e+04_rb,3.5390e+04_rb,2.9492e+04_rb,2.3594e+04_rb,1.7695e+04_rb, &
     &1.1797e+04_rb,5.8984e+03_rb,1.4554e+01_rb/)
      kao(:, 2, 3,16) = (/ &
     &6.3662e+04_rb,5.5704e+04_rb,4.7746e+04_rb,3.9789e+04_rb,3.1831e+04_rb,2.3873e+04_rb, &
     &1.5915e+04_rb,7.9577e+03_rb,1.4573e+01_rb/)
      kao(:, 3, 3,16) = (/ &
     &8.2931e+04_rb,7.2565e+04_rb,6.2198e+04_rb,5.1832e+04_rb,4.1465e+04_rb,3.1099e+04_rb, &
     &2.0733e+04_rb,1.0366e+04_rb,1.4578e+01_rb/)
      kao(:, 4, 3,16) = (/ &
     &1.0484e+05_rb,9.1735e+04_rb,7.8630e+04_rb,6.5525e+04_rb,5.2420e+04_rb,3.9315e+04_rb, &
     &2.6210e+04_rb,1.3105e+04_rb,1.4538e+01_rb/)
      kao(:, 5, 3,16) = (/ &
     &1.2920e+05_rb,1.1305e+05_rb,9.6898e+04_rb,8.0748e+04_rb,6.4599e+04_rb,4.8449e+04_rb, &
     &3.2299e+04_rb,1.6150e+04_rb,1.4475e+01_rb/)
      kao(:, 1, 4,16) = (/ &
     &4.6077e+04_rb,4.0317e+04_rb,3.4558e+04_rb,2.8798e+04_rb,2.3038e+04_rb,1.7279e+04_rb, &
     &1.1519e+04_rb,5.7596e+03_rb,1.7584e+01_rb/)
      kao(:, 2, 4,16) = (/ &
     &6.3824e+04_rb,5.5846e+04_rb,4.7868e+04_rb,3.9890e+04_rb,3.1912e+04_rb,2.3934e+04_rb, &
     &1.5956e+04_rb,7.9779e+03_rb,1.7639e+01_rb/)
      kao(:, 3, 4,16) = (/ &
     &8.4999e+04_rb,7.4374e+04_rb,6.3749e+04_rb,5.3124e+04_rb,4.2499e+04_rb,3.1874e+04_rb, &
     &2.1250e+04_rb,1.0625e+04_rb,1.7665e+01_rb/)
      kao(:, 4, 4,16) = (/ &
     &1.0949e+05_rb,9.5804e+04_rb,8.2118e+04_rb,6.8432e+04_rb,5.4745e+04_rb,4.1059e+04_rb, &
     &2.7373e+04_rb,1.3686e+04_rb,1.7655e+01_rb/)
      kao(:, 5, 4,16) = (/ &
     &1.3706e+05_rb,1.1993e+05_rb,1.0280e+05_rb,8.5665e+04_rb,6.8532e+04_rb,5.1399e+04_rb, &
     &3.4266e+04_rb,1.7133e+04_rb,1.7601e+01_rb/)
      kao(:, 1, 5,16) = (/ &
     &4.4408e+04_rb,3.8857e+04_rb,3.3306e+04_rb,2.7755e+04_rb,2.2204e+04_rb,1.6653e+04_rb, &
     &1.1102e+04_rb,5.5509e+03_rb,2.1257e+01_rb/)
      kao(:, 2, 5,16) = (/ &
     &6.3274e+04_rb,5.5364e+04_rb,4.7455e+04_rb,3.9546e+04_rb,3.1637e+04_rb,2.3728e+04_rb, &
     &1.5818e+04_rb,7.9092e+03_rb,2.1367e+01_rb/)
      kao(:, 3, 5,16) = (/ &
     &8.6266e+04_rb,7.5483e+04_rb,6.4700e+04_rb,5.3916e+04_rb,4.3133e+04_rb,3.2350e+04_rb, &
     &2.1567e+04_rb,1.0783e+04_rb,2.1428e+01_rb/)
      kao(:, 4, 5,16) = (/ &
     &1.1336e+05_rb,9.9194e+04_rb,8.5023e+04_rb,7.0853e+04_rb,5.6682e+04_rb,4.2512e+04_rb, &
     &2.8341e+04_rb,1.4171e+04_rb,2.1442e+01_rb/)
      kao(:, 5, 5,16) = (/ &
     &1.4437e+05_rb,1.2632e+05_rb,1.0827e+05_rb,9.0229e+04_rb,7.2183e+04_rb,5.4137e+04_rb, &
     &3.6091e+04_rb,1.8046e+04_rb,2.1411e+01_rb/)
      kao(:, 1, 6,16) = (/ &
     &4.1622e+04_rb,3.6419e+04_rb,3.1217e+04_rb,2.6014e+04_rb,2.0811e+04_rb,1.5608e+04_rb, &
     &1.0406e+04_rb,5.2028e+03_rb,2.5704e+01_rb/)
      kao(:, 2, 6,16) = (/ &
     &6.1272e+04_rb,5.3613e+04_rb,4.5954e+04_rb,3.8295e+04_rb,3.0636e+04_rb,2.2977e+04_rb, &
     &1.5318e+04_rb,7.6590e+03_rb,2.5880e+01_rb/)
      kao(:, 3, 6,16) = (/ &
     &8.5823e+04_rb,7.5095e+04_rb,6.4367e+04_rb,5.3639e+04_rb,4.2911e+04_rb,3.2183e+04_rb, &
     &2.1456e+04_rb,1.0728e+04_rb,2.5988e+01_rb/)
      kao(:, 4, 6,16) = (/ &
     &1.1534e+05_rb,1.0092e+05_rb,8.6501e+04_rb,7.2084e+04_rb,5.7667e+04_rb,4.3251e+04_rb, &
     &2.8834e+04_rb,1.4417e+04_rb,2.6050e+01_rb/)
      kao(:, 5, 6,16) = (/ &
     &1.4970e+05_rb,1.3098e+05_rb,1.1227e+05_rb,9.3560e+04_rb,7.4848e+04_rb,5.6136e+04_rb, &
     &3.7424e+04_rb,1.8712e+04_rb,2.6033e+01_rb/)
      kao(:, 1, 7,16) = (/ &
     &3.8762e+04_rb,3.3917e+04_rb,2.9071e+04_rb,2.4226e+04_rb,1.9381e+04_rb,1.4536e+04_rb, &
     &9.6904e+03_rb,4.8452e+03_rb,3.1098e+01_rb/)
      kao(:, 2, 7,16) = (/ &
     &5.9114e+04_rb,5.1725e+04_rb,4.4335e+04_rb,3.6946e+04_rb,2.9557e+04_rb,2.2168e+04_rb, &
     &1.4778e+04_rb,7.3892e+03_rb,3.1343e+01_rb/)
      kao(:, 3, 7,16) = (/ &
     &8.5273e+04_rb,7.4614e+04_rb,6.3955e+04_rb,5.3296e+04_rb,4.2636e+04_rb,3.1977e+04_rb, &
     &2.1318e+04_rb,1.0659e+04_rb,3.1509e+01_rb/)
      kao(:, 4, 7,16) = (/ &
     &1.1738e+05_rb,1.0271e+05_rb,8.8034e+04_rb,7.3362e+04_rb,5.8689e+04_rb,4.4017e+04_rb, &
     &2.9345e+04_rb,1.4672e+04_rb,3.1622e+01_rb/)
      kao(:, 5, 7,16) = (/ &
     &1.5542e+05_rb,1.3599e+05_rb,1.1657e+05_rb,9.7138e+04_rb,7.7710e+04_rb,5.8283e+04_rb, &
     &3.8855e+04_rb,1.9427e+04_rb,3.1655e+01_rb/)
      kao(:, 1, 8,16) = (/ &
     &3.6377e+04_rb,3.1830e+04_rb,2.7279e+04_rb,2.2736e+04_rb,1.8189e+04_rb,1.3641e+04_rb, &
     &9.0943e+03_rb,4.5472e+03_rb,3.7610e+01_rb/)
      kao(:, 2, 8,16) = (/ &
     &5.7649e+04_rb,5.0443e+04_rb,4.3237e+04_rb,3.6031e+04_rb,2.8825e+04_rb,2.1619e+04_rb, &
     &1.4412e+04_rb,7.2062e+03_rb,3.7934e+01_rb/)
      kao(:, 3, 8,16) = (/ &
     &8.5747e+04_rb,7.5029e+04_rb,6.4311e+04_rb,5.3592e+04_rb,4.2874e+04_rb,3.2155e+04_rb, &
     &2.1437e+04_rb,1.0718e+04_rb,3.8193e+01_rb/)
      kao(:, 4, 8,16) = (/ &
     &1.2106e+05_rb,1.0593e+05_rb,9.0796e+04_rb,7.5663e+04_rb,6.0531e+04_rb,4.5398e+04_rb, &
     &3.0265e+04_rb,1.5133e+04_rb,3.8356e+01_rb/)
      kao(:, 5, 8,16) = (/ &
     &1.6369e+05_rb,1.4323e+05_rb,1.2277e+05_rb,1.0231e+05_rb,8.1844e+04_rb,6.1383e+04_rb, &
     &4.0922e+04_rb,2.0461e+04_rb,3.8437e+01_rb/)
      kao(:, 1, 9,16) = (/ &
     &3.3210e+04_rb,2.9058e+04_rb,2.4907e+04_rb,2.0756e+04_rb,1.6605e+04_rb,1.2454e+04_rb, &
     &8.3024e+03_rb,4.1512e+03_rb,4.5425e+01_rb/)
      kao(:, 2, 9,16) = (/ &
     &5.4859e+04_rb,4.8001e+04_rb,4.1144e+04_rb,3.4287e+04_rb,2.7429e+04_rb,2.0572e+04_rb, &
     &1.3715e+04_rb,6.8573e+03_rb,4.5889e+01_rb/)
      kao(:, 3, 9,16) = (/ &
     &8.4365e+04_rb,7.3820e+04_rb,6.3274e+04_rb,5.2728e+04_rb,4.2183e+04_rb,3.1637e+04_rb, &
     &2.1091e+04_rb,1.0546e+04_rb,4.6246e+01_rb/)
      kao(:, 4, 9,16) = (/ &
     &1.2241e+05_rb,1.0711e+05_rb,9.1805e+04_rb,7.6504e+04_rb,6.1203e+04_rb,4.5902e+04_rb, &
     &3.0602e+04_rb,1.5301e+04_rb,4.6486e+01_rb/)
      kao(:, 5, 9,16) = (/ &
     &1.6922e+05_rb,1.4806e+05_rb,1.2691e+05_rb,1.0576e+05_rb,8.4608e+04_rb,6.3456e+04_rb, &
     &4.2304e+04_rb,2.1152e+04_rb,4.6625e+01_rb/)
      kao(:, 1,10,16) = (/ &
     &3.0979e+04_rb,2.7106e+04_rb,2.3234e+04_rb,1.9362e+04_rb,1.5489e+04_rb,1.1617e+04_rb, &
     &7.7447e+03_rb,3.8723e+03_rb,5.4886e+01_rb/)
      kao(:, 2,10,16) = (/ &
     &5.3196e+04_rb,4.6547e+04_rb,3.9897e+04_rb,3.3248e+04_rb,2.6598e+04_rb,1.9948e+04_rb, &
     &1.3299e+04_rb,6.6495e+03_rb,5.5476e+01_rb/)
      kao(:, 3,10,16) = (/ &
     &8.4341e+04_rb,7.3798e+04_rb,6.3256e+04_rb,5.2713e+04_rb,4.2170e+04_rb,3.1628e+04_rb, &
     &2.1085e+04_rb,1.0332e+04_rb,5.5974e+01_rb/)
      kao(:, 4,10,16) = (/ &
     &1.2535e+05_rb,1.0968e+05_rb,9.4014e+04_rb,7.8345e+04_rb,6.2676e+04_rb,4.7007e+04_rb, &
     &3.1338e+04_rb,1.5669e+04_rb,5.6279e+01_rb/)
      kao(:, 5,10,16) = (/ &
     &1.7674e+05_rb,1.5464e+05_rb,1.3255e+05_rb,1.1046e+05_rb,8.8368e+04_rb,6.6276e+04_rb, &
     &4.4184e+04_rb,2.2092e+04_rb,5.6482e+01_rb/)
      kao(:, 1,11,16) = (/ &
     &3.7488e+04_rb,3.2802e+04_rb,2.8116e+04_rb,2.3430e+04_rb,1.8744e+04_rb,1.4058e+04_rb, &
     &9.3718e+03_rb,4.6859e+03_rb,6.6623e+01_rb/)
      kao(:, 2,11,16) = (/ &
     &6.4180e+04_rb,5.6158e+04_rb,4.8135e+04_rb,4.0112e+04_rb,3.2090e+04_rb,2.4067e+04_rb, &
     &1.6045e+04_rb,8.0224e+03_rb,6.7308e+01_rb/)
      kao(:, 3,11,16) = (/ &
     &1.0143e+05_rb,8.8753e+04_rb,7.6074e+04_rb,6.3395e+04_rb,5.0716e+04_rb,3.8037e+04_rb, &
     &2.5358e+04_rb,1.2679e+04_rb,6.7849e+01_rb/)
      kao(:, 4,11,16) = (/ &
     &1.5034e+05_rb,1.3155e+05_rb,1.1275e+05_rb,9.3961e+04_rb,7.5169e+04_rb,5.6376e+04_rb, &
     &3.7584e+04_rb,1.8792e+04_rb,6.8179e+01_rb/)
      kao(:, 5,11,16) = (/ &
     &2.1140e+05_rb,1.8498e+05_rb,1.5855e+05_rb,1.3213e+05_rb,1.0570e+05_rb,7.9275e+04_rb, &
     &5.2850e+04_rb,2.6425e+04_rb,6.8333e+01_rb/)
      kao(:, 1,12,16) = (/ &
     &4.5641e+04_rb,3.9936e+04_rb,3.4231e+04_rb,2.8526e+04_rb,2.2821e+04_rb,1.7116e+04_rb, &
     &1.1410e+04_rb,5.7052e+03_rb,8.0729e+01_rb/)
      kao(:, 2,12,16) = (/ &
     &7.7843e+04_rb,6.8113e+04_rb,5.8382e+04_rb,4.8652e+04_rb,3.8922e+04_rb,2.9191e+04_rb, &
     &1.9461e+04_rb,9.7304e+03_rb,8.1511e+01_rb/)
      kao(:, 3,12,16) = (/ &
     &1.2264e+05_rb,1.0731e+05_rb,9.1979e+04_rb,7.6649e+04_rb,6.1320e+04_rb,4.5990e+04_rb, &
     &3.0660e+04_rb,1.5330e+04_rb,8.2042e+01_rb/)
      kao(:, 4,12,16) = (/ &
     &1.8173e+05_rb,1.5901e+05_rb,1.3629e+05_rb,1.1358e+05_rb,9.0863e+04_rb,6.8147e+04_rb, &
     &4.5431e+04_rb,2.2716e+04_rb,8.2374e+01_rb/)
      kao(:, 5,12,16) = (/ &
     &2.5615e+05_rb,2.2413e+05_rb,1.9211e+05_rb,1.6009e+05_rb,1.2807e+05_rb,9.6056e+04_rb, &
     &6.4037e+04_rb,3.2019e+04_rb,8.2428e+01_rb/)
      kao(:, 1,13,16) = (/ &
     &5.7163e+04_rb,5.0018e+04_rb,4.2872e+04_rb,3.5727e+04_rb,2.8582e+04_rb,2.1436e+04_rb, &
     &1.4291e+04_rb,7.1455e+03_rb,9.7539e+01_rb/)
      kao(:, 2,13,16) = (/ &
     &9.7458e+04_rb,8.5276e+04_rb,7.3094e+04_rb,6.0911e+04_rb,4.8729e+04_rb,3.6547e+04_rb, &
     &2.4365e+04_rb,1.2182e+04_rb,9.8425e+01_rb/)
      kao(:, 3,13,16) = (/ &
     &1.5404e+05_rb,1.3479e+05_rb,1.1553e+05_rb,9.6277e+04_rb,7.7021e+04_rb,5.7766e+04_rb, &
     &3.8511e+04_rb,1.9255e+04_rb,9.8903e+01_rb/)
      kao(:, 4,13,16) = (/ &
     &2.2887e+05_rb,2.0026e+05_rb,1.7165e+05_rb,1.4304e+05_rb,1.1444e+05_rb,8.5827e+04_rb, &
     &5.7218e+04_rb,2.8609e+04_rb,9.9135e+01_rb/)
      kao(:, 5,13,16) = (/ &
     &3.2310e+05_rb,2.8271e+05_rb,2.4232e+05_rb,2.0194e+05_rb,1.6155e+05_rb,1.2116e+05_rb, &
     &8.0775e+04_rb,4.0387e+04_rb,9.9042e+01_rb/)

!     The array KA_Mxx contains the absorption coefficient for 
!     a minor species at the 16 chosen g-values for a reference pressure
!     level below 100~ mb.   The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.  The second index refers to temperature 
!     in 7.2 degree increments.  For instance, JT = 1 refers to a 
!     temperature of 188.0, JT = 2 refers to 195.2, etc. The third index 
!     runs over the g-channel (1 to 16).

       kao_mn2( 1, :, 1) = (/ &
     & 3.24352e-08_rb, 3.39625e-08_rb, 3.55618e-08_rb, 3.72364e-08_rb, 3.89899e-08_rb, &
     & 4.08259e-08_rb, 4.27484e-08_rb, 4.47614e-08_rb, 4.68692e-08_rb, 4.90763e-08_rb, &
     & 5.13873e-08_rb, 5.38071e-08_rb, 5.63409e-08_rb, 5.89940e-08_rb, 6.17720e-08_rb, &
     & 6.46808e-08_rb, 6.77266e-08_rb, 7.09158e-08_rb, 7.42553e-08_rb/)
       kao_mn2( 2, :, 1) = (/ &
     & 3.44203e-08_rb, 3.60254e-08_rb, 3.77053e-08_rb, 3.94636e-08_rb, 4.13038e-08_rb, &
     & 4.32299e-08_rb, 4.52458e-08_rb, 4.73557e-08_rb, 4.95640e-08_rb, 5.18753e-08_rb, &
     & 5.42943e-08_rb, 5.68262e-08_rb, 5.94761e-08_rb, 6.22496e-08_rb, 6.51524e-08_rb, &
     & 6.81906e-08_rb, 7.13704e-08_rb, 7.46986e-08_rb, 7.81819e-08_rb/)
       kao_mn2( 3, :, 1) = (/ &
     & 3.44344e-08_rb, 3.61485e-08_rb, 3.79480e-08_rb, 3.98370e-08_rb, 4.18201e-08_rb, &
     & 4.39019e-08_rb, 4.60873e-08_rb, 4.83815e-08_rb, 5.07899e-08_rb, 5.33182e-08_rb, &
     & 5.59723e-08_rb, 5.87586e-08_rb, 6.16836e-08_rb, 6.47541e-08_rb, 6.79776e-08_rb, &
     & 7.13614e-08_rb, 7.49138e-08_rb, 7.86429e-08_rb, 8.25577e-08_rb/)
       kao_mn2( 4, :, 1) = (/ &
     & 4.21102e-08_rb, 4.38921e-08_rb, 4.57493e-08_rb, 4.76852e-08_rb, 4.97029e-08_rb, &
     & 5.18061e-08_rb, 5.39982e-08_rb, 5.62831e-08_rb, 5.86647e-08_rb, 6.11470e-08_rb, &
     & 6.37344e-08_rb, 6.64313e-08_rb, 6.92422e-08_rb, 7.21722e-08_rb, 7.52261e-08_rb, &
     & 7.84092e-08_rb, 8.17270e-08_rb, 8.51852e-08_rb, 8.87897e-08_rb/)
       kao_mn2( 5, :, 1) = (/ &
     & 4.78813e-08_rb, 5.01015e-08_rb, 5.24246e-08_rb, 5.48554e-08_rb, 5.73989e-08_rb, &
     & 6.00603e-08_rb, 6.28452e-08_rb, 6.57592e-08_rb, 6.88083e-08_rb, 7.19987e-08_rb, &
     & 7.53371e-08_rb, 7.88304e-08_rb, 8.24855e-08_rb, 8.63102e-08_rb, 9.03122e-08_rb, &
     & 9.44997e-08_rb, 9.88815e-08_rb, 1.03466e-07_rb, 1.08264e-07_rb/)
       kao_mn2( 6, :, 1) = (/ &
     & 7.03115e-08_rb, 7.27877e-08_rb, 7.53511e-08_rb, 7.80048e-08_rb, 8.07519e-08_rb, &
     & 8.35958e-08_rb, 8.65398e-08_rb, 8.95875e-08_rb, 9.27426e-08_rb, 9.60087e-08_rb, &
     & 9.93899e-08_rb, 1.02890e-07_rb, 1.06514e-07_rb, 1.10265e-07_rb, 1.14148e-07_rb, &
     & 1.18168e-07_rb, 1.22330e-07_rb, 1.26638e-07_rb, 1.31098e-07_rb/)
       kao_mn2( 7, :, 1) = (/ &
     & 8.86454e-08_rb, 9.20065e-08_rb, 9.54951e-08_rb, 9.91159e-08_rb, 1.02874e-07_rb, &
     & 1.06775e-07_rb, 1.10823e-07_rb, 1.15025e-07_rb, 1.19387e-07_rb, 1.23913e-07_rb, &
     & 1.28612e-07_rb, 1.33488e-07_rb, 1.38550e-07_rb, 1.43803e-07_rb, 1.49255e-07_rb, &
     & 1.54915e-07_rb, 1.60788e-07_rb, 1.66885e-07_rb, 1.73213e-07_rb/)
       kao_mn2( 8, :, 1) = (/ &
     & 1.34118e-07_rb, 1.38267e-07_rb, 1.42545e-07_rb, 1.46955e-07_rb, 1.51502e-07_rb, &
     & 1.56189e-07_rb, 1.61022e-07_rb, 1.66004e-07_rb, 1.71140e-07_rb, 1.76435e-07_rb, &
     & 1.81893e-07_rb, 1.87521e-07_rb, 1.93323e-07_rb, 1.99304e-07_rb, 2.05470e-07_rb, &
     & 2.11827e-07_rb, 2.18381e-07_rb, 2.25138e-07_rb, 2.32103e-07_rb/)
       kao_mn2( 9, :, 1) = (/ &
     & 5.08256e-08_rb, 5.30384e-08_rb, 5.53476e-08_rb, 5.77573e-08_rb, 6.02718e-08_rb, &
     & 6.28959e-08_rb, 6.56342e-08_rb, 6.84917e-08_rb, 7.14737e-08_rb, 7.45854e-08_rb, &
     & 7.78327e-08_rb, 8.12213e-08_rb, 8.47574e-08_rb, 8.84475e-08_rb, 9.22983e-08_rb, &
     & 9.63167e-08_rb, 1.00510e-07_rb, 1.04886e-07_rb, 1.09452e-07_rb/)
       kao_mn2( 1, :, 2) = (/ &
     & 8.23958e-08_rb, 8.39092e-08_rb, 8.54504e-08_rb, 8.70200e-08_rb, 8.86183e-08_rb, &
     & 9.02460e-08_rb, 9.19036e-08_rb, 9.35917e-08_rb, 9.53107e-08_rb, 9.70614e-08_rb, &
     & 9.88442e-08_rb, 1.00660e-07_rb, 1.02509e-07_rb, 1.04391e-07_rb, 1.06309e-07_rb, &
     & 1.08261e-07_rb, 1.10250e-07_rb, 1.12275e-07_rb, 1.14337e-07_rb/)
       kao_mn2( 2, :, 2) = (/ &
     & 8.52335e-08_rb, 8.69254e-08_rb, 8.86509e-08_rb, 9.04107e-08_rb, 9.22054e-08_rb, &
     & 9.40357e-08_rb, 9.59024e-08_rb, 9.78061e-08_rb, 9.97476e-08_rb, 1.01728e-07_rb, &
     & 1.03747e-07_rb, 1.05806e-07_rb, 1.07907e-07_rb, 1.10049e-07_rb, 1.12233e-07_rb, &
     & 1.14461e-07_rb, 1.16733e-07_rb, 1.19050e-07_rb, 1.21414e-07_rb/)
       kao_mn2( 3, :, 2) = (/ &
     & 1.04608e-07_rb, 1.06067e-07_rb, 1.07546e-07_rb, 1.09046e-07_rb, 1.10567e-07_rb, &
     & 1.12110e-07_rb, 1.13673e-07_rb, 1.15259e-07_rb, 1.16866e-07_rb, 1.18496e-07_rb, &
     & 1.20149e-07_rb, 1.21825e-07_rb, 1.23524e-07_rb, 1.25247e-07_rb, 1.26994e-07_rb, &
     & 1.28765e-07_rb, 1.30561e-07_rb, 1.32382e-07_rb, 1.34229e-07_rb/)
       kao_mn2( 4, :, 2) = (/ &
     & 1.17504e-07_rb, 1.18763e-07_rb, 1.20036e-07_rb, 1.21322e-07_rb, 1.22622e-07_rb, &
     & 1.23936e-07_rb, 1.25265e-07_rb, 1.26607e-07_rb, 1.27964e-07_rb, 1.29335e-07_rb, &
     & 1.30721e-07_rb, 1.32122e-07_rb, 1.33538e-07_rb, 1.34969e-07_rb, 1.36415e-07_rb, &
     & 1.37877e-07_rb, 1.39354e-07_rb, 1.40848e-07_rb, 1.42357e-07_rb/)
       kao_mn2( 5, :, 2) = (/ &
     & 1.23552e-07_rb, 1.25200e-07_rb, 1.26870e-07_rb, 1.28562e-07_rb, 1.30277e-07_rb, &
     & 1.32015e-07_rb, 1.33776e-07_rb, 1.35560e-07_rb, 1.37368e-07_rb, 1.39200e-07_rb, &
     & 1.41057e-07_rb, 1.42938e-07_rb, 1.44845e-07_rb, 1.46777e-07_rb, 1.48735e-07_rb, &
     & 1.50718e-07_rb, 1.52729e-07_rb, 1.54766e-07_rb, 1.56830e-07_rb/)
       kao_mn2( 6, :, 2) = (/ &
     & 1.29682e-07_rb, 1.32226e-07_rb, 1.34820e-07_rb, 1.37464e-07_rb, 1.40161e-07_rb, &
     & 1.42910e-07_rb, 1.45713e-07_rb, 1.48571e-07_rb, 1.51486e-07_rb, 1.54457e-07_rb, &
     & 1.57487e-07_rb, 1.60576e-07_rb, 1.63726e-07_rb, 1.66937e-07_rb, 1.70212e-07_rb, &
     & 1.73551e-07_rb, 1.76955e-07_rb, 1.80426e-07_rb, 1.83965e-07_rb/)
       kao_mn2( 7, :, 2) = (/ &
     & 1.77416e-07_rb, 1.78627e-07_rb, 1.79846e-07_rb, 1.81073e-07_rb, 1.82309e-07_rb, &
     & 1.83554e-07_rb, 1.84806e-07_rb, 1.86068e-07_rb, 1.87338e-07_rb, 1.88616e-07_rb, &
     & 1.89904e-07_rb, 1.91200e-07_rb, 1.92505e-07_rb, 1.93819e-07_rb, 1.95142e-07_rb, &
     & 1.96474e-07_rb, 1.97815e-07_rb, 1.99165e-07_rb, 2.00524e-07_rb/)
       kao_mn2( 8, :, 2) = (/ &
     & 2.20695e-07_rb, 2.20451e-07_rb, 2.20208e-07_rb, 2.19965e-07_rb, 2.19722e-07_rb, &
     & 2.19480e-07_rb, 2.19238e-07_rb, 2.18996e-07_rb, 2.18754e-07_rb, 2.18513e-07_rb, &
     & 2.18272e-07_rb, 2.18031e-07_rb, 2.17790e-07_rb, 2.17550e-07_rb, 2.17310e-07_rb, &
     & 2.17070e-07_rb, 2.16831e-07_rb, 2.16591e-07_rb, 2.16352e-07_rb/)
       kao_mn2( 9, :, 2) = (/ &
     & 1.23015e-07_rb, 1.24808e-07_rb, 1.26626e-07_rb, 1.28471e-07_rb, 1.30343e-07_rb, &
     & 1.32242e-07_rb, 1.34168e-07_rb, 1.36123e-07_rb, 1.38106e-07_rb, 1.40118e-07_rb, &
     & 1.42160e-07_rb, 1.44231e-07_rb, 1.46332e-07_rb, 1.48464e-07_rb, 1.50627e-07_rb, &
     & 1.52822e-07_rb, 1.55048e-07_rb, 1.57307e-07_rb, 1.59599e-07_rb/)
       kao_mn2( 1, :, 3) = (/ &
     & 1.87585e-07_rb, 1.89503e-07_rb, 1.91440e-07_rb, 1.93398e-07_rb, 1.95375e-07_rb, &
     & 1.97372e-07_rb, 1.99390e-07_rb, 2.01429e-07_rb, 2.03488e-07_rb, 2.05568e-07_rb, &
     & 2.07670e-07_rb, 2.09793e-07_rb, 2.11938e-07_rb, 2.14105e-07_rb, 2.16294e-07_rb, &
     & 2.18505e-07_rb, 2.20739e-07_rb, 2.22996e-07_rb, 2.25275e-07_rb/)
       kao_mn2( 2, :, 3) = (/ &
     & 1.82585e-07_rb, 1.84249e-07_rb, 1.85929e-07_rb, 1.87624e-07_rb, 1.89335e-07_rb, &
     & 1.91061e-07_rb, 1.92803e-07_rb, 1.94561e-07_rb, 1.96335e-07_rb, 1.98125e-07_rb, &
     & 1.99932e-07_rb, 2.01755e-07_rb, 2.03594e-07_rb, 2.05451e-07_rb, 2.07324e-07_rb, &
     & 2.09214e-07_rb, 2.11122e-07_rb, 2.13047e-07_rb, 2.14989e-07_rb/)
       kao_mn2( 3, :, 3) = (/ &
     & 1.64711e-07_rb, 1.67539e-07_rb, 1.70417e-07_rb, 1.73343e-07_rb, 1.76321e-07_rb, &
     & 1.79349e-07_rb, 1.82429e-07_rb, 1.85562e-07_rb, 1.88749e-07_rb, 1.91990e-07_rb, &
     & 1.95288e-07_rb, 1.98642e-07_rb, 2.02053e-07_rb, 2.05523e-07_rb, 2.09053e-07_rb, &
     & 2.12643e-07_rb, 2.16295e-07_rb, 2.20010e-07_rb, 2.23788e-07_rb/)
       kao_mn2( 4, :, 3) = (/ &
     & 1.67494e-07_rb, 1.71011e-07_rb, 1.74601e-07_rb, 1.78267e-07_rb, 1.82009e-07_rb, &
     & 1.85831e-07_rb, 1.89732e-07_rb, 1.93715e-07_rb, 1.97782e-07_rb, 2.01935e-07_rb, &
     & 2.06174e-07_rb, 2.10503e-07_rb, 2.14922e-07_rb, 2.19434e-07_rb, 2.24041e-07_rb, &
     & 2.28745e-07_rb, 2.33548e-07_rb, 2.38451e-07_rb, 2.43457e-07_rb/)
       kao_mn2( 5, :, 3) = (/ &
     & 1.97399e-07_rb, 2.00092e-07_rb, 2.02821e-07_rb, 2.05588e-07_rb, 2.08393e-07_rb, &
     & 2.11236e-07_rb, 2.14118e-07_rb, 2.17039e-07_rb, 2.20000e-07_rb, 2.23001e-07_rb, &
     & 2.26043e-07_rb, 2.29127e-07_rb, 2.32252e-07_rb, 2.35421e-07_rb, 2.38633e-07_rb, &
     & 2.41888e-07_rb, 2.45188e-07_rb, 2.48533e-07_rb, 2.51923e-07_rb/)
       kao_mn2( 6, :, 3) = (/ &
     & 2.24021e-07_rb, 2.24970e-07_rb, 2.25923e-07_rb, 2.26880e-07_rb, 2.27840e-07_rb, &
     & 2.28805e-07_rb, 2.29774e-07_rb, 2.30747e-07_rb, 2.31725e-07_rb, 2.32706e-07_rb, &
     & 2.33692e-07_rb, 2.34681e-07_rb, 2.35675e-07_rb, 2.36673e-07_rb, 2.37675e-07_rb, &
     & 2.38682e-07_rb, 2.39693e-07_rb, 2.40708e-07_rb, 2.41727e-07_rb/)
       kao_mn2( 7, :, 3) = (/ &
     & 1.98178e-07_rb, 2.00676e-07_rb, 2.03205e-07_rb, 2.05766e-07_rb, 2.08359e-07_rb, &
     & 2.10986e-07_rb, 2.13645e-07_rb, 2.16337e-07_rb, 2.19064e-07_rb, 2.21825e-07_rb, &
     & 2.24621e-07_rb, 2.27452e-07_rb, 2.30319e-07_rb, 2.33222e-07_rb, 2.36161e-07_rb, &
     & 2.39138e-07_rb, 2.42152e-07_rb, 2.45204e-07_rb, 2.48294e-07_rb/)
       kao_mn2( 8, :, 3) = (/ &
     & 2.83042e-07_rb, 2.89941e-07_rb, 2.97009e-07_rb, 3.04250e-07_rb, 3.11666e-07_rb, &
     & 3.19264e-07_rb, 3.27047e-07_rb, 3.35019e-07_rb, 3.43186e-07_rb, 3.51552e-07_rb, &
     & 3.60122e-07_rb, 3.68901e-07_rb, 3.77893e-07_rb, 3.87105e-07_rb, 3.96542e-07_rb, &
     & 4.06208e-07_rb, 4.16111e-07_rb, 4.26254e-07_rb, 4.36645e-07_rb/)
       kao_mn2( 9, :, 3) = (/ &
     & 1.98963e-07_rb, 2.01576e-07_rb, 2.04224e-07_rb, 2.06907e-07_rb, 2.09626e-07_rb, &
     & 2.12379e-07_rb, 2.15169e-07_rb, 2.17996e-07_rb, 2.20860e-07_rb, 2.23761e-07_rb, &
     & 2.26701e-07_rb, 2.29679e-07_rb, 2.32696e-07_rb, 2.35753e-07_rb, 2.38851e-07_rb, &
     & 2.41988e-07_rb, 2.45167e-07_rb, 2.48388e-07_rb, 2.51651e-07_rb/)
       kao_mn2( 1, :, 4) = (/ &
     & 3.75434e-07_rb, 3.79581e-07_rb, 3.83775e-07_rb, 3.88014e-07_rb, 3.92301e-07_rb, &
     & 3.96634e-07_rb, 4.01016e-07_rb, 4.05446e-07_rb, 4.09925e-07_rb, 4.14453e-07_rb, &
     & 4.19032e-07_rb, 4.23661e-07_rb, 4.28341e-07_rb, 4.33073e-07_rb, 4.37857e-07_rb, &
     & 4.42694e-07_rb, 4.47585e-07_rb, 4.52529e-07_rb, 4.57528e-07_rb/)
       kao_mn2( 2, :, 4) = (/ &
     & 3.76756e-07_rb, 3.80760e-07_rb, 3.84805e-07_rb, 3.88894e-07_rb, 3.93027e-07_rb, &
     & 3.97203e-07_rb, 4.01423e-07_rb, 4.05689e-07_rb, 4.10000e-07_rb, 4.14356e-07_rb, &
     & 4.18759e-07_rb, 4.23209e-07_rb, 4.27706e-07_rb, 4.32250e-07_rb, 4.36843e-07_rb, &
     & 4.41485e-07_rb, 4.46176e-07_rb, 4.50917e-07_rb, 4.55708e-07_rb/)
       kao_mn2( 3, :, 4) = (/ &
     & 3.76258e-07_rb, 3.78929e-07_rb, 3.81619e-07_rb, 3.84329e-07_rb, 3.87057e-07_rb, &
     & 3.89805e-07_rb, 3.92572e-07_rb, 3.95359e-07_rb, 3.98166e-07_rb, 4.00993e-07_rb, &
     & 4.03839e-07_rb, 4.06706e-07_rb, 4.09594e-07_rb, 4.12502e-07_rb, 4.15430e-07_rb, &
     & 4.18379e-07_rb, 4.21349e-07_rb, 4.24341e-07_rb, 4.27353e-07_rb/)
       kao_mn2( 4, :, 4) = (/ &
     & 3.17796e-07_rb, 3.22447e-07_rb, 3.27166e-07_rb, 3.31954e-07_rb, 3.36812e-07_rb, &
     & 3.41742e-07_rb, 3.46743e-07_rb, 3.51818e-07_rb, 3.56967e-07_rb, 3.62191e-07_rb, &
     & 3.67492e-07_rb, 3.72870e-07_rb, 3.78328e-07_rb, 3.83865e-07_rb, 3.89483e-07_rb, &
     & 3.95183e-07_rb, 4.00967e-07_rb, 4.06835e-07_rb, 4.12789e-07_rb/)
       kao_mn2( 5, :, 4) = (/ &
     & 3.33793e-07_rb, 3.38941e-07_rb, 3.44169e-07_rb, 3.49478e-07_rb, 3.54868e-07_rb, &
     & 3.60342e-07_rb, 3.65900e-07_rb, 3.71544e-07_rb, 3.77275e-07_rb, 3.83094e-07_rb, &
     & 3.89003e-07_rb, 3.95003e-07_rb, 4.01096e-07_rb, 4.07283e-07_rb, 4.13565e-07_rb, &
     & 4.19944e-07_rb, 4.26421e-07_rb, 4.32999e-07_rb, 4.39677e-07_rb/)
       kao_mn2( 6, :, 4) = (/ &
     & 3.60052e-07_rb, 3.66686e-07_rb, 3.73442e-07_rb, 3.80323e-07_rb, 3.87330e-07_rb, &
     & 3.94466e-07_rb, 4.01734e-07_rb, 4.09136e-07_rb, 4.16674e-07_rb, 4.24351e-07_rb, &
     & 4.32169e-07_rb, 4.40132e-07_rb, 4.48241e-07_rb, 4.56500e-07_rb, 4.64910e-07_rb, &
     & 4.73476e-07_rb, 4.82200e-07_rb, 4.91084e-07_rb, 5.00132e-07_rb/)
       kao_mn2( 7, :, 4) = (/ &
     & 4.14713e-07_rb, 4.21885e-07_rb, 4.29181e-07_rb, 4.36603e-07_rb, 4.44153e-07_rb, &
     & 4.51834e-07_rb, 4.59648e-07_rb, 4.67598e-07_rb, 4.75684e-07_rb, 4.83910e-07_rb, &
     & 4.92279e-07_rb, 5.00793e-07_rb, 5.09453e-07_rb, 5.18264e-07_rb, 5.27226e-07_rb, &
     & 5.36344e-07_rb, 5.45620e-07_rb, 5.55055e-07_rb, 5.64654e-07_rb/)
       kao_mn2( 8, :, 4) = (/ &
     & 4.15352e-07_rb, 4.24386e-07_rb, 4.33617e-07_rb, 4.43049e-07_rb, 4.52685e-07_rb, &
     & 4.62532e-07_rb, 4.72592e-07_rb, 4.82872e-07_rb, 4.93374e-07_rb, 5.04106e-07_rb, &
     & 5.15071e-07_rb, 5.26274e-07_rb, 5.37721e-07_rb, 5.49417e-07_rb, 5.61367e-07_rb, &
     & 5.73577e-07_rb, 5.86053e-07_rb, 5.98800e-07_rb, 6.11825e-07_rb/)
       kao_mn2( 9, :, 4) = (/ &
     & 3.33820e-07_rb, 3.39144e-07_rb, 3.44553e-07_rb, 3.50048e-07_rb, 3.55631e-07_rb, &
     & 3.61302e-07_rb, 3.67065e-07_rb, 3.72919e-07_rb, 3.78866e-07_rb, 3.84908e-07_rb, &
     & 3.91047e-07_rb, 3.97284e-07_rb, 4.03620e-07_rb, 4.10057e-07_rb, 4.16597e-07_rb, &
     & 4.23241e-07_rb, 4.29991e-07_rb, 4.36849e-07_rb, 4.43816e-07_rb/)
       kao_mn2( 1, :, 5) = (/ &
     & 6.99819e-07_rb, 7.04629e-07_rb, 7.09472e-07_rb, 7.14349e-07_rb, 7.19258e-07_rb, &
     & 7.24202e-07_rb, 7.29180e-07_rb, 7.34192e-07_rb, 7.39238e-07_rb, 7.44319e-07_rb, &
     & 7.49435e-07_rb, 7.54586e-07_rb, 7.59773e-07_rb, 7.64995e-07_rb, 7.70253e-07_rb, &
     & 7.75547e-07_rb, 7.80877e-07_rb, 7.86245e-07_rb, 7.91649e-07_rb/)
       kao_mn2( 2, :, 5) = (/ &
     & 6.98257e-07_rb, 7.03182e-07_rb, 7.08143e-07_rb, 7.13138e-07_rb, 7.18169e-07_rb, &
     & 7.23235e-07_rb, 7.28336e-07_rb, 7.33474e-07_rb, 7.38648e-07_rb, 7.43858e-07_rb, &
     & 7.49106e-07_rb, 7.54390e-07_rb, 7.59711e-07_rb, 7.65071e-07_rb, 7.70467e-07_rb, &
     & 7.75902e-07_rb, 7.81376e-07_rb, 7.86887e-07_rb, 7.92438e-07_rb/)
       kao_mn2( 3, :, 5) = (/ &
     & 6.98531e-07_rb, 7.03429e-07_rb, 7.08361e-07_rb, 7.13328e-07_rb, 7.18329e-07_rb, &
     & 7.23365e-07_rb, 7.28437e-07_rb, 7.33545e-07_rb, 7.38688e-07_rb, 7.43867e-07_rb, &
     & 7.49082e-07_rb, 7.54335e-07_rb, 7.59623e-07_rb, 7.64950e-07_rb, 7.70313e-07_rb, &
     & 7.75714e-07_rb, 7.81153e-07_rb, 7.86630e-07_rb, 7.92145e-07_rb/)
       kao_mn2( 4, :, 5) = (/ &
     & 7.37210e-07_rb, 7.38869e-07_rb, 7.40532e-07_rb, 7.42198e-07_rb, 7.43868e-07_rb, &
     & 7.45542e-07_rb, 7.47219e-07_rb, 7.48901e-07_rb, 7.50586e-07_rb, 7.52275e-07_rb, &
     & 7.53967e-07_rb, 7.55664e-07_rb, 7.57364e-07_rb, 7.59068e-07_rb, 7.60777e-07_rb, &
     & 7.62488e-07_rb, 7.64204e-07_rb, 7.65924e-07_rb, 7.67647e-07_rb/)
       kao_mn2( 5, :, 5) = (/ &
     & 6.07063e-07_rb, 6.12893e-07_rb, 6.18779e-07_rb, 6.24722e-07_rb, 6.30721e-07_rb, &
     & 6.36778e-07_rb, 6.42893e-07_rb, 6.49067e-07_rb, 6.55301e-07_rb, 6.61594e-07_rb, &
     & 6.67947e-07_rb, 6.74362e-07_rb, 6.80838e-07_rb, 6.87376e-07_rb, 6.93978e-07_rb, &
     & 7.00642e-07_rb, 7.07371e-07_rb, 7.14164e-07_rb, 7.21022e-07_rb/)
       kao_mn2( 6, :, 5) = (/ &
     & 6.13354e-07_rb, 6.20147e-07_rb, 6.27016e-07_rb, 6.33961e-07_rb, 6.40983e-07_rb, &
     & 6.48082e-07_rb, 6.55260e-07_rb, 6.62518e-07_rb, 6.69856e-07_rb, 6.77276e-07_rb, &
     & 6.84777e-07_rb, 6.92362e-07_rb, 7.00030e-07_rb, 7.07784e-07_rb, 7.15624e-07_rb, &
     & 7.23550e-07_rb, 7.31564e-07_rb, 7.39667e-07_rb, 7.47859e-07_rb/)
       kao_mn2( 7, :, 5) = (/ &
     & 6.86666e-07_rb, 6.92902e-07_rb, 6.99195e-07_rb, 7.05545e-07_rb, 7.11952e-07_rb, &
     & 7.18418e-07_rb, 7.24943e-07_rb, 7.31526e-07_rb, 7.38170e-07_rb, 7.44874e-07_rb, &
     & 7.51639e-07_rb, 7.58465e-07_rb, 7.65353e-07_rb, 7.72304e-07_rb, 7.79318e-07_rb, &
     & 7.86395e-07_rb, 7.93537e-07_rb, 8.00744e-07_rb, 8.08016e-07_rb/)
       kao_mn2( 8, :, 5) = (/ &
     & 9.39664e-07_rb, 9.39765e-07_rb, 9.39866e-07_rb, 9.39966e-07_rb, 9.40067e-07_rb, &
     & 9.40168e-07_rb, 9.40268e-07_rb, 9.40369e-07_rb, 9.40470e-07_rb, 9.40571e-07_rb, &
     & 9.40671e-07_rb, 9.40772e-07_rb, 9.40873e-07_rb, 9.40974e-07_rb, 9.41074e-07_rb, &
     & 9.41175e-07_rb, 9.41276e-07_rb, 9.41377e-07_rb, 9.41478e-07_rb/)
       kao_mn2( 9, :, 5) = (/ &
     & 6.02847e-07_rb, 6.09726e-07_rb, 6.16684e-07_rb, 6.23722e-07_rb, 6.30839e-07_rb, &
     & 6.38038e-07_rb, 6.45320e-07_rb, 6.52684e-07_rb, 6.60132e-07_rb, 6.67665e-07_rb, &
     & 6.75284e-07_rb, 6.82991e-07_rb, 6.90785e-07_rb, 6.98668e-07_rb, 7.06641e-07_rb, &
     & 7.14705e-07_rb, 7.22861e-07_rb, 7.31110e-07_rb, 7.39453e-07_rb/)
       kao_mn2( 1, :, 6) = (/ &
     & 1.13692e-06_rb, 1.13231e-06_rb, 1.12772e-06_rb, 1.12315e-06_rb, 1.11859e-06_rb, &
     & 1.11406e-06_rb, 1.10954e-06_rb, 1.10505e-06_rb, 1.10057e-06_rb, 1.09610e-06_rb, &
     & 1.09166e-06_rb, 1.08724e-06_rb, 1.08283e-06_rb, 1.07844e-06_rb, 1.07407e-06_rb, &
     & 1.06971e-06_rb, 1.06538e-06_rb, 1.06106e-06_rb, 1.05676e-06_rb/)
       kao_mn2( 2, :, 6) = (/ &
     & 1.13682e-06_rb, 1.13221e-06_rb, 1.12762e-06_rb, 1.12305e-06_rb, 1.11849e-06_rb, &
     & 1.11396e-06_rb, 1.10944e-06_rb, 1.10495e-06_rb, 1.10047e-06_rb, 1.09600e-06_rb, &
     & 1.09156e-06_rb, 1.08714e-06_rb, 1.08273e-06_rb, 1.07834e-06_rb, 1.07397e-06_rb, &
     & 1.06961e-06_rb, 1.06528e-06_rb, 1.06096e-06_rb, 1.05666e-06_rb/)
       kao_mn2( 3, :, 6) = (/ &
     & 1.13642e-06_rb, 1.13181e-06_rb, 1.12722e-06_rb, 1.12265e-06_rb, 1.11809e-06_rb, &
     & 1.11356e-06_rb, 1.10904e-06_rb, 1.10455e-06_rb, 1.10007e-06_rb, 1.09560e-06_rb, &
     & 1.09116e-06_rb, 1.08674e-06_rb, 1.08233e-06_rb, 1.07794e-06_rb, 1.07357e-06_rb, &
     & 1.06921e-06_rb, 1.06488e-06_rb, 1.06056e-06_rb, 1.05626e-06_rb/)
       kao_mn2( 4, :, 6) = (/ &
     & 1.13626e-06_rb, 1.13160e-06_rb, 1.12696e-06_rb, 1.12233e-06_rb, 1.11773e-06_rb, &
     & 1.11314e-06_rb, 1.10858e-06_rb, 1.10403e-06_rb, 1.09950e-06_rb, 1.09498e-06_rb, &
     & 1.09049e-06_rb, 1.08602e-06_rb, 1.08156e-06_rb, 1.07712e-06_rb, 1.07270e-06_rb, &
     & 1.06830e-06_rb, 1.06392e-06_rb, 1.05955e-06_rb, 1.05520e-06_rb/)
       kao_mn2( 5, :, 6) = (/ &
     & 1.22429e-06_rb, 1.21163e-06_rb, 1.19909e-06_rb, 1.18669e-06_rb, 1.17441e-06_rb, &
     & 1.16226e-06_rb, 1.15024e-06_rb, 1.13834e-06_rb, 1.12656e-06_rb, 1.11491e-06_rb, &
     & 1.10338e-06_rb, 1.09196e-06_rb, 1.08067e-06_rb, 1.06949e-06_rb, 1.05842e-06_rb, &
     & 1.04747e-06_rb, 1.03664e-06_rb, 1.02591e-06_rb, 1.01530e-06_rb/)
       kao_mn2( 6, :, 6) = (/ &
     & 1.02400e-06_rb, 1.02238e-06_rb, 1.02077e-06_rb, 1.01916e-06_rb, 1.01755e-06_rb, &
     & 1.01594e-06_rb, 1.01433e-06_rb, 1.01273e-06_rb, 1.01113e-06_rb, 1.00953e-06_rb, &
     & 1.00794e-06_rb, 1.00635e-06_rb, 1.00476e-06_rb, 1.00317e-06_rb, 1.00159e-06_rb, &
     & 1.00000e-06_rb, 9.98425e-07_rb, 9.96848e-07_rb, 9.95273e-07_rb/)
       kao_mn2( 7, :, 6) = (/ &
     & 1.08594e-06_rb, 1.08185e-06_rb, 1.07778e-06_rb, 1.07373e-06_rb, 1.06969e-06_rb, &
     & 1.06566e-06_rb, 1.06165e-06_rb, 1.05766e-06_rb, 1.05368e-06_rb, 1.04971e-06_rb, &
     & 1.04576e-06_rb, 1.04183e-06_rb, 1.03791e-06_rb, 1.03400e-06_rb, 1.03011e-06_rb, &
     & 1.02623e-06_rb, 1.02237e-06_rb, 1.01852e-06_rb, 1.01469e-06_rb/)
       kao_mn2( 8, :, 6) = (/ &
     & 1.25029e-06_rb, 1.22508e-06_rb, 1.20038e-06_rb, 1.17618e-06_rb, 1.15247e-06_rb, &
     & 1.12924e-06_rb, 1.10647e-06_rb, 1.08416e-06_rb, 1.06231e-06_rb, 1.04089e-06_rb, &
     & 1.01990e-06_rb, 9.99343e-07_rb, 9.79196e-07_rb, 9.59454e-07_rb, 9.40111e-07_rb, &
     & 9.21158e-07_rb, 9.02587e-07_rb, 8.84390e-07_rb, 8.66560e-07_rb/)
       kao_mn2( 9, :, 6) = (/ &
     & 1.21299e-06_rb, 1.19953e-06_rb, 1.18622e-06_rb, 1.17305e-06_rb, 1.16003e-06_rb, &
     & 1.14716e-06_rb, 1.13443e-06_rb, 1.12184e-06_rb, 1.10939e-06_rb, 1.09708e-06_rb, &
     & 1.08491e-06_rb, 1.07287e-06_rb, 1.06096e-06_rb, 1.04919e-06_rb, 1.03755e-06_rb, &
     & 1.02603e-06_rb, 1.01465e-06_rb, 1.00339e-06_rb, 9.92253e-07_rb/)
       kao_mn2( 1, :, 7) = (/ &
     & 1.53893e-06_rb, 1.51743e-06_rb, 1.49623e-06_rb, 1.47532e-06_rb, 1.45471e-06_rb, &
     & 1.43438e-06_rb, 1.41434e-06_rb, 1.39458e-06_rb, 1.37509e-06_rb, 1.35588e-06_rb, &
     & 1.33694e-06_rb, 1.31826e-06_rb, 1.29984e-06_rb, 1.28167e-06_rb, 1.26377e-06_rb, &
     & 1.24611e-06_rb, 1.22870e-06_rb, 1.21153e-06_rb, 1.19460e-06_rb/)
       kao_mn2( 2, :, 7) = (/ &
     & 1.53809e-06_rb, 1.51665e-06_rb, 1.49552e-06_rb, 1.47467e-06_rb, 1.45412e-06_rb, &
     & 1.43386e-06_rb, 1.41388e-06_rb, 1.39418e-06_rb, 1.37475e-06_rb, 1.35559e-06_rb, &
     & 1.33670e-06_rb, 1.31807e-06_rb, 1.29970e-06_rb, 1.28159e-06_rb, 1.26373e-06_rb, &
     & 1.24612e-06_rb, 1.22875e-06_rb, 1.21163e-06_rb, 1.19475e-06_rb/)
       kao_mn2( 3, :, 7) = (/ &
     & 1.53883e-06_rb, 1.51733e-06_rb, 1.49613e-06_rb, 1.47522e-06_rb, 1.45461e-06_rb, &
     & 1.43428e-06_rb, 1.41424e-06_rb, 1.39448e-06_rb, 1.37499e-06_rb, 1.35578e-06_rb, &
     & 1.33684e-06_rb, 1.31816e-06_rb, 1.29974e-06_rb, 1.28157e-06_rb, 1.26367e-06_rb, &
     & 1.24601e-06_rb, 1.22860e-06_rb, 1.21143e-06_rb, 1.19450e-06_rb/)
       kao_mn2( 4, :, 7) = (/ &
     & 1.53789e-06_rb, 1.51645e-06_rb, 1.49532e-06_rb, 1.47448e-06_rb, 1.45393e-06_rb, &
     & 1.43366e-06_rb, 1.41368e-06_rb, 1.39398e-06_rb, 1.37455e-06_rb, 1.35539e-06_rb, &
     & 1.33650e-06_rb, 1.31787e-06_rb, 1.29950e-06_rb, 1.28139e-06_rb, 1.26353e-06_rb, &
     & 1.24592e-06_rb, 1.22856e-06_rb, 1.21143e-06_rb, 1.19455e-06_rb/)
       kao_mn2( 5, :, 7) = (/ &
     & 1.54059e-06_rb, 1.51888e-06_rb, 1.49747e-06_rb, 1.47637e-06_rb, 1.45557e-06_rb, &
     & 1.43505e-06_rb, 1.41483e-06_rb, 1.39489e-06_rb, 1.37523e-06_rb, 1.35585e-06_rb, &
     & 1.33675e-06_rb, 1.31791e-06_rb, 1.29934e-06_rb, 1.28103e-06_rb, 1.26297e-06_rb, &
     & 1.24517e-06_rb, 1.22763e-06_rb, 1.21033e-06_rb, 1.19327e-06_rb/)
       kao_mn2( 6, :, 7) = (/ &
     & 1.70605e-06_rb, 1.65759e-06_rb, 1.61052e-06_rb, 1.56478e-06_rb, 1.52034e-06_rb, &
     & 1.47716e-06_rb, 1.43521e-06_rb, 1.39445e-06_rb, 1.35485e-06_rb, 1.31637e-06_rb, &
     & 1.27898e-06_rb, 1.24266e-06_rb, 1.20737e-06_rb, 1.17308e-06_rb, 1.13976e-06_rb, &
     & 1.10739e-06_rb, 1.07594e-06_rb, 1.04539e-06_rb, 1.01570e-06_rb/)
       kao_mn2( 7, :, 7) = (/ &
     & 1.39128e-06_rb, 1.36388e-06_rb, 1.33702e-06_rb, 1.31068e-06_rb, 1.28487e-06_rb, &
     & 1.25956e-06_rb, 1.23475e-06_rb, 1.21044e-06_rb, 1.18659e-06_rb, 1.16322e-06_rb, &
     & 1.14031e-06_rb, 1.11785e-06_rb, 1.09584e-06_rb, 1.07425e-06_rb, 1.05309e-06_rb, &
     & 1.03235e-06_rb, 1.01202e-06_rb, 9.92088e-07_rb, 9.72548e-07_rb/)
       kao_mn2( 8, :, 7) = (/ &
     & 1.15676e-06_rb, 1.13709e-06_rb, 1.11775e-06_rb, 1.09874e-06_rb, 1.08005e-06_rb, &
     & 1.06168e-06_rb, 1.04362e-06_rb, 1.02587e-06_rb, 1.00842e-06_rb, 9.91271e-07_rb, &
     & 9.74411e-07_rb, 9.57838e-07_rb, 9.41547e-07_rb, 9.25532e-07_rb, 9.09791e-07_rb, &
     & 8.94316e-07_rb, 8.79105e-07_rb, 8.64153e-07_rb, 8.49455e-07_rb/)
       kao_mn2( 9, :, 7) = (/ &
     & 1.53483e-06_rb, 1.51352e-06_rb, 1.49252e-06_rb, 1.47180e-06_rb, 1.45138e-06_rb, &
     & 1.43123e-06_rb, 1.41137e-06_rb, 1.39178e-06_rb, 1.37246e-06_rb, 1.35341e-06_rb, &
     & 1.33463e-06_rb, 1.31610e-06_rb, 1.29784e-06_rb, 1.27982e-06_rb, 1.26206e-06_rb, &
     & 1.24454e-06_rb, 1.22727e-06_rb, 1.21024e-06_rb, 1.19344e-06_rb/)
       kao_mn2( 1, :, 8) = (/ &
     & 1.70380e-06_rb, 1.67470e-06_rb, 1.64609e-06_rb, 1.61796e-06_rb, 1.59032e-06_rb, &
     & 1.56315e-06_rb, 1.53645e-06_rb, 1.51020e-06_rb, 1.48440e-06_rb, 1.45904e-06_rb, &
     & 1.43411e-06_rb, 1.40961e-06_rb, 1.38553e-06_rb, 1.36186e-06_rb, 1.33859e-06_rb, &
     & 1.31572e-06_rb, 1.29324e-06_rb, 1.27115e-06_rb, 1.24943e-06_rb/)
       kao_mn2( 2, :, 8) = (/ &
     & 1.70380e-06_rb, 1.67470e-06_rb, 1.64609e-06_rb, 1.61796e-06_rb, 1.59032e-06_rb, &
     & 1.56315e-06_rb, 1.53645e-06_rb, 1.51020e-06_rb, 1.48440e-06_rb, 1.45904e-06_rb, &
     & 1.43411e-06_rb, 1.40961e-06_rb, 1.38553e-06_rb, 1.36186e-06_rb, 1.33859e-06_rb, &
     & 1.31572e-06_rb, 1.29324e-06_rb, 1.27115e-06_rb, 1.24943e-06_rb/)
       kao_mn2( 3, :, 8) = (/ &
     & 1.70380e-06_rb, 1.67470e-06_rb, 1.64609e-06_rb, 1.61796e-06_rb, 1.59032e-06_rb, &
     & 1.56315e-06_rb, 1.53645e-06_rb, 1.51020e-06_rb, 1.48440e-06_rb, 1.45904e-06_rb, &
     & 1.43411e-06_rb, 1.40961e-06_rb, 1.38553e-06_rb, 1.36186e-06_rb, 1.33859e-06_rb, &
     & 1.31572e-06_rb, 1.29324e-06_rb, 1.27115e-06_rb, 1.24943e-06_rb/)
       kao_mn2( 4, :, 8) = (/ &
     & 1.70380e-06_rb, 1.67470e-06_rb, 1.64609e-06_rb, 1.61796e-06_rb, 1.59032e-06_rb, &
     & 1.56315e-06_rb, 1.53645e-06_rb, 1.51020e-06_rb, 1.48440e-06_rb, 1.45904e-06_rb, &
     & 1.43411e-06_rb, 1.40961e-06_rb, 1.38553e-06_rb, 1.36186e-06_rb, 1.33859e-06_rb, &
     & 1.31572e-06_rb, 1.29324e-06_rb, 1.27115e-06_rb, 1.24943e-06_rb/)
       kao_mn2( 5, :, 8) = (/ &
     & 1.70380e-06_rb, 1.67470e-06_rb, 1.64609e-06_rb, 1.61796e-06_rb, 1.59032e-06_rb, &
     & 1.56315e-06_rb, 1.53645e-06_rb, 1.51020e-06_rb, 1.48440e-06_rb, 1.45904e-06_rb, &
     & 1.43411e-06_rb, 1.40961e-06_rb, 1.38553e-06_rb, 1.36186e-06_rb, 1.33859e-06_rb, &
     & 1.31572e-06_rb, 1.29324e-06_rb, 1.27115e-06_rb, 1.24943e-06_rb/)
       kao_mn2( 6, :, 8) = (/ &
     & 1.70380e-06_rb, 1.67470e-06_rb, 1.64609e-06_rb, 1.61796e-06_rb, 1.59032e-06_rb, &
     & 1.56315e-06_rb, 1.53645e-06_rb, 1.51020e-06_rb, 1.48440e-06_rb, 1.45904e-06_rb, &
     & 1.43411e-06_rb, 1.40961e-06_rb, 1.38553e-06_rb, 1.36186e-06_rb, 1.33859e-06_rb, &
     & 1.31572e-06_rb, 1.29324e-06_rb, 1.27115e-06_rb, 1.24943e-06_rb/)
       kao_mn2( 7, :, 8) = (/ &
     & 1.71827e-06_rb, 1.65481e-06_rb, 1.59370e-06_rb, 1.53484e-06_rb, 1.47816e-06_rb, &
     & 1.42357e-06_rb, 1.37099e-06_rb, 1.32036e-06_rb, 1.27160e-06_rb, 1.22464e-06_rb, &
     & 1.17941e-06_rb, 1.13585e-06_rb, 1.09390e-06_rb, 1.05350e-06_rb, 1.01459e-06_rb, &
     & 9.77124e-07_rb, 9.41037e-07_rb, 9.06284e-07_rb, 8.72813e-07_rb/)
       kao_mn2( 8, :, 8) = (/ &
     & 1.77169e-06_rb, 1.62858e-06_rb, 1.49703e-06_rb, 1.37610e-06_rb, 1.26494e-06_rb, &
     & 1.16276e-06_rb, 1.06883e-06_rb, 9.82495e-07_rb, 9.03131e-07_rb, 8.30177e-07_rb, &
     & 7.63117e-07_rb, 7.01473e-07_rb, 6.44810e-07_rb, 5.92723e-07_rb, 5.44844e-07_rb, &
     & 5.00832e-07_rb, 4.60376e-07_rb, 4.23187e-07_rb, 3.89003e-07_rb/)
       kao_mn2( 9, :, 8) = (/ &
     & 1.70025e-06_rb, 1.67042e-06_rb, 1.64110e-06_rb, 1.61231e-06_rb, 1.58401e-06_rb, &
     & 1.55622e-06_rb, 1.52891e-06_rb, 1.50208e-06_rb, 1.47572e-06_rb, 1.44982e-06_rb, &
     & 1.42438e-06_rb, 1.39939e-06_rb, 1.37483e-06_rb, 1.35071e-06_rb, 1.32700e-06_rb, &
     & 1.30372e-06_rb, 1.28084e-06_rb, 1.25836e-06_rb, 1.23628e-06_rb/)
       kao_mn2( 1, :, 9) = (/ &
     & 1.74004e-06_rb, 1.70661e-06_rb, 1.67383e-06_rb, 1.64167e-06_rb, 1.61014e-06_rb, &
     & 1.57921e-06_rb, 1.54887e-06_rb, 1.51912e-06_rb, 1.48994e-06_rb, 1.46132e-06_rb, &
     & 1.43325e-06_rb, 1.40572e-06_rb, 1.37871e-06_rb, 1.35223e-06_rb, 1.32625e-06_rb, &
     & 1.30078e-06_rb, 1.27579e-06_rb, 1.25128e-06_rb, 1.22725e-06_rb/)
       kao_mn2( 2, :, 9) = (/ &
     & 1.74004e-06_rb, 1.70661e-06_rb, 1.67383e-06_rb, 1.64167e-06_rb, 1.61014e-06_rb, &
     & 1.57921e-06_rb, 1.54887e-06_rb, 1.51912e-06_rb, 1.48994e-06_rb, 1.46132e-06_rb, &
     & 1.43325e-06_rb, 1.40572e-06_rb, 1.37871e-06_rb, 1.35223e-06_rb, 1.32625e-06_rb, &
     & 1.30078e-06_rb, 1.27579e-06_rb, 1.25128e-06_rb, 1.22725e-06_rb/)
       kao_mn2( 3, :, 9) = (/ &
     & 1.74004e-06_rb, 1.70661e-06_rb, 1.67383e-06_rb, 1.64167e-06_rb, 1.61014e-06_rb, &
     & 1.57921e-06_rb, 1.54887e-06_rb, 1.51912e-06_rb, 1.48994e-06_rb, 1.46132e-06_rb, &
     & 1.43325e-06_rb, 1.40572e-06_rb, 1.37871e-06_rb, 1.35223e-06_rb, 1.32625e-06_rb, &
     & 1.30078e-06_rb, 1.27579e-06_rb, 1.25128e-06_rb, 1.22725e-06_rb/)
       kao_mn2( 4, :, 9) = (/ &
     & 1.74004e-06_rb, 1.70661e-06_rb, 1.67383e-06_rb, 1.64167e-06_rb, 1.61014e-06_rb, &
     & 1.57921e-06_rb, 1.54887e-06_rb, 1.51912e-06_rb, 1.48994e-06_rb, 1.46132e-06_rb, &
     & 1.43325e-06_rb, 1.40572e-06_rb, 1.37871e-06_rb, 1.35223e-06_rb, 1.32625e-06_rb, &
     & 1.30078e-06_rb, 1.27579e-06_rb, 1.25128e-06_rb, 1.22725e-06_rb/)
       kao_mn2( 5, :, 9) = (/ &
     & 1.74004e-06_rb, 1.70661e-06_rb, 1.67383e-06_rb, 1.64167e-06_rb, 1.61014e-06_rb, &
     & 1.57921e-06_rb, 1.54887e-06_rb, 1.51912e-06_rb, 1.48994e-06_rb, 1.46132e-06_rb, &
     & 1.43325e-06_rb, 1.40572e-06_rb, 1.37871e-06_rb, 1.35223e-06_rb, 1.32625e-06_rb, &
     & 1.30078e-06_rb, 1.27579e-06_rb, 1.25128e-06_rb, 1.22725e-06_rb/)
       kao_mn2( 6, :, 9) = (/ &
     & 1.74004e-06_rb, 1.70661e-06_rb, 1.67383e-06_rb, 1.64167e-06_rb, 1.61014e-06_rb, &
     & 1.57921e-06_rb, 1.54887e-06_rb, 1.51912e-06_rb, 1.48994e-06_rb, 1.46132e-06_rb, &
     & 1.43325e-06_rb, 1.40572e-06_rb, 1.37871e-06_rb, 1.35223e-06_rb, 1.32625e-06_rb, &
     & 1.30078e-06_rb, 1.27579e-06_rb, 1.25128e-06_rb, 1.22725e-06_rb/)
       kao_mn2( 7, :, 9) = (/ &
     & 1.74004e-06_rb, 1.70661e-06_rb, 1.67383e-06_rb, 1.64167e-06_rb, 1.61014e-06_rb, &
     & 1.57921e-06_rb, 1.54887e-06_rb, 1.51912e-06_rb, 1.48994e-06_rb, 1.46132e-06_rb, &
     & 1.43325e-06_rb, 1.40572e-06_rb, 1.37871e-06_rb, 1.35223e-06_rb, 1.32625e-06_rb, &
     & 1.30078e-06_rb, 1.27579e-06_rb, 1.25128e-06_rb, 1.22725e-06_rb/)
       kao_mn2( 8, :, 9) = (/ &
     & 1.08654e-06_rb, 1.09039e-06_rb, 1.09425e-06_rb, 1.09812e-06_rb, 1.10201e-06_rb, &
     & 1.10592e-06_rb, 1.10983e-06_rb, 1.11376e-06_rb, 1.11771e-06_rb, 1.12167e-06_rb, &
     & 1.12564e-06_rb, 1.12962e-06_rb, 1.13363e-06_rb, 1.13764e-06_rb, 1.14167e-06_rb, &
     & 1.14571e-06_rb, 1.14977e-06_rb, 1.15384e-06_rb, 1.15793e-06_rb/)
       kao_mn2( 9, :, 9) = (/ &
     & 1.74382e-06_rb, 1.71092e-06_rb, 1.67864e-06_rb, 1.64697e-06_rb, 1.61589e-06_rb, &
     & 1.58541e-06_rb, 1.55549e-06_rb, 1.52615e-06_rb, 1.49735e-06_rb, 1.46910e-06_rb, &
     & 1.44138e-06_rb, 1.41419e-06_rb, 1.38751e-06_rb, 1.36133e-06_rb, 1.33564e-06_rb, &
     & 1.31045e-06_rb, 1.28572e-06_rb, 1.26146e-06_rb, 1.23766e-06_rb/)
       kao_mn2( 1, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 2, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 3, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 4, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 5, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 6, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 7, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 8, :,10) = (/ &
     & 1.73703e-06_rb, 1.70249e-06_rb, 1.66863e-06_rb, 1.63544e-06_rb, 1.60292e-06_rb, &
     & 1.57104e-06_rb, 1.53980e-06_rb, 1.50917e-06_rb, 1.47916e-06_rb, 1.44974e-06_rb, &
     & 1.42091e-06_rb, 1.39265e-06_rb, 1.36496e-06_rb, 1.33781e-06_rb, 1.31121e-06_rb, &
     & 1.28513e-06_rb, 1.25957e-06_rb, 1.23452e-06_rb, 1.20997e-06_rb/)
       kao_mn2( 9, :,10) = (/ &
     & 1.82903e-06_rb, 1.78673e-06_rb, 1.74541e-06_rb, 1.70505e-06_rb, 1.66562e-06_rb, &
     & 1.62710e-06_rb, 1.58947e-06_rb, 1.55271e-06_rb, 1.51680e-06_rb, 1.48172e-06_rb, &
     & 1.44745e-06_rb, 1.41398e-06_rb, 1.38128e-06_rb, 1.34933e-06_rb, 1.31813e-06_rb, &
     & 1.28765e-06_rb, 1.25787e-06_rb, 1.22878e-06_rb, 1.20036e-06_rb/)
       kao_mn2( 1, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 2, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 3, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 4, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 5, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 6, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 7, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 8, :,11) = (/ &
     & 1.73118e-06_rb, 1.69710e-06_rb, 1.66370e-06_rb, 1.63095e-06_rb, 1.59885e-06_rb, &
     & 1.56737e-06_rb, 1.53652e-06_rb, 1.50628e-06_rb, 1.47663e-06_rb, 1.44756e-06_rb, &
     & 1.41907e-06_rb, 1.39114e-06_rb, 1.36376e-06_rb, 1.33691e-06_rb, 1.31060e-06_rb, &
     & 1.28480e-06_rb, 1.25951e-06_rb, 1.23472e-06_rb, 1.21041e-06_rb/)
       kao_mn2( 9, :,11) = (/ &
     & 1.81037e-06_rb, 1.76948e-06_rb, 1.72952e-06_rb, 1.69045e-06_rb, 1.65228e-06_rb, &
     & 1.61496e-06_rb, 1.57848e-06_rb, 1.54283e-06_rb, 1.50799e-06_rb, 1.47393e-06_rb, &
     & 1.44064e-06_rb, 1.40810e-06_rb, 1.37630e-06_rb, 1.34522e-06_rb, 1.31484e-06_rb, &
     & 1.28514e-06_rb, 1.25611e-06_rb, 1.22774e-06_rb, 1.20002e-06_rb/)
       kao_mn2( 1, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 2, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 3, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 4, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 5, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 6, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 7, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 8, :,12) = (/ &
     & 1.73338e-06_rb, 1.69915e-06_rb, 1.66560e-06_rb, 1.63271e-06_rb, 1.60046e-06_rb, &
     & 1.56886e-06_rb, 1.53788e-06_rb, 1.50751e-06_rb, 1.47774e-06_rb, 1.44856e-06_rb, &
     & 1.41995e-06_rb, 1.39191e-06_rb, 1.36442e-06_rb, 1.33748e-06_rb, 1.31107e-06_rb, &
     & 1.28518e-06_rb, 1.25980e-06_rb, 1.23492e-06_rb, 1.21053e-06_rb/)
       kao_mn2( 9, :,12) = (/ &
     & 2.04857e-06_rb, 1.98353e-06_rb, 1.92055e-06_rb, 1.85957e-06_rb, 1.80053e-06_rb, &
     & 1.74336e-06_rb, 1.68800e-06_rb, 1.63441e-06_rb, 1.58251e-06_rb, 1.53227e-06_rb, &
     & 1.48362e-06_rb, 1.43651e-06_rb, 1.39090e-06_rb, 1.34674e-06_rb, 1.30398e-06_rb, &
     & 1.26257e-06_rb, 1.22249e-06_rb, 1.18367e-06_rb, 1.14609e-06_rb/)
       kao_mn2( 1, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 2, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 3, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 4, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 5, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 6, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 7, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 8, :,13) = (/ &
     & 1.73511e-06_rb, 1.70072e-06_rb, 1.66702e-06_rb, 1.63398e-06_rb, 1.60159e-06_rb, &
     & 1.56985e-06_rb, 1.53874e-06_rb, 1.50824e-06_rb, 1.47835e-06_rb, 1.44905e-06_rb, &
     & 1.42033e-06_rb, 1.39218e-06_rb, 1.36459e-06_rb, 1.33755e-06_rb, 1.31104e-06_rb, &
     & 1.28505e-06_rb, 1.25958e-06_rb, 1.23462e-06_rb, 1.21015e-06_rb/)
       kao_mn2( 9, :,13) = (/ &
     & 2.13403e-06_rb, 2.05906e-06_rb, 1.98673e-06_rb, 1.91694e-06_rb, 1.84961e-06_rb, &
     & 1.78463e-06_rb, 1.72194e-06_rb, 1.66145e-06_rb, 1.60309e-06_rb, 1.54678e-06_rb, &
     & 1.49244e-06_rb, 1.44002e-06_rb, 1.38943e-06_rb, 1.34062e-06_rb, 1.29353e-06_rb, &
     & 1.24809e-06_rb, 1.20425e-06_rb, 1.16195e-06_rb, 1.12113e-06_rb/)
       kao_mn2( 1, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 2, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 3, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 4, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 5, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 6, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 7, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 8, :,14) = (/ &
     & 1.73398e-06_rb, 1.69941e-06_rb, 1.66553e-06_rb, 1.63233e-06_rb, 1.59979e-06_rb, &
     & 1.56790e-06_rb, 1.53664e-06_rb, 1.50601e-06_rb, 1.47598e-06_rb, 1.44656e-06_rb, &
     & 1.41772e-06_rb, 1.38946e-06_rb, 1.36176e-06_rb, 1.33461e-06_rb, 1.30801e-06_rb, &
     & 1.28193e-06_rb, 1.25637e-06_rb, 1.23133e-06_rb, 1.20678e-06_rb/)
       kao_mn2( 9, :,14) = (/ &
     & 1.83423e-06_rb, 1.79123e-06_rb, 1.74923e-06_rb, 1.70821e-06_rb, 1.66816e-06_rb, &
     & 1.62904e-06_rb, 1.59085e-06_rb, 1.55354e-06_rb, 1.51712e-06_rb, 1.48154e-06_rb, &
     & 1.44681e-06_rb, 1.41288e-06_rb, 1.37975e-06_rb, 1.34740e-06_rb, 1.31581e-06_rb, &
     & 1.28496e-06_rb, 1.25483e-06_rb, 1.22540e-06_rb, 1.19667e-06_rb/)
       kao_mn2( 1, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 2, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 3, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 4, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 5, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 6, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 7, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 8, :,15) = (/ &
     & 1.73231e-06_rb, 1.69765e-06_rb, 1.66368e-06_rb, 1.63039e-06_rb, 1.59776e-06_rb, &
     & 1.56579e-06_rb, 1.53445e-06_rb, 1.50375e-06_rb, 1.47366e-06_rb, 1.44417e-06_rb, &
     & 1.41527e-06_rb, 1.38695e-06_rb, 1.35919e-06_rb, 1.33199e-06_rb, 1.30534e-06_rb, &
     & 1.27922e-06_rb, 1.25362e-06_rb, 1.22853e-06_rb, 1.20395e-06_rb/)
       kao_mn2( 9, :,15) = (/ &
     & 1.71602e-06_rb, 1.68499e-06_rb, 1.65452e-06_rb, 1.62461e-06_rb, 1.59523e-06_rb, &
     & 1.56639e-06_rb, 1.53807e-06_rb, 1.51026e-06_rb, 1.48295e-06_rb, 1.45614e-06_rb, &
     & 1.42981e-06_rb, 1.40395e-06_rb, 1.37857e-06_rb, 1.35364e-06_rb, 1.32917e-06_rb, &
     & 1.30513e-06_rb, 1.28153e-06_rb, 1.25836e-06_rb, 1.23561e-06_rb/)
       kao_mn2( 1, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 2, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 3, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 4, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 5, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 6, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 7, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 8, :,16) = (/ &
     & 1.73310e-06_rb, 1.69826e-06_rb, 1.66413e-06_rb, 1.63069e-06_rb, 1.59791e-06_rb, &
     & 1.56580e-06_rb, 1.53433e-06_rb, 1.50349e-06_rb, 1.47328e-06_rb, 1.44367e-06_rb, &
     & 1.41465e-06_rb, 1.38622e-06_rb, 1.35836e-06_rb, 1.33106e-06_rb, 1.30431e-06_rb, &
     & 1.27810e-06_rb, 1.25241e-06_rb, 1.22724e-06_rb, 1.20257e-06_rb/)
       kao_mn2( 9, :,16) = (/ &
     & 1.79375e-06_rb, 1.75599e-06_rb, 1.71903e-06_rb, 1.68284e-06_rb, 1.64741e-06_rb, &
     & 1.61273e-06_rb, 1.57878e-06_rb, 1.54554e-06_rb, 1.51301e-06_rb, 1.48116e-06_rb, &
     & 1.44998e-06_rb, 1.41945e-06_rb, 1.38957e-06_rb, 1.36032e-06_rb, 1.33168e-06_rb, &
     & 1.30365e-06_rb, 1.27620e-06_rb, 1.24934e-06_rb, 1.22304e-06_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &1.1755e-06_rb,6.5398e-07_rb,4.3915e-07_rb,3.0753e-07_rb,1.9677e-07_rb,1.4362e-07_rb, &
     &9.4598e-08_rb,1.1848e-07_rb,1.4280e-07_rb,1.5821e-07_rb,1.5816e-07_rb,1.5769e-07_rb, &
     &1.5844e-07_rb,1.6016e-07_rb,1.6232e-07_rb,1.6320e-07_rb/)
      forrefo(2,:) = (/ &
     &1.0703e-06_rb,6.2783e-07_rb,4.7122e-07_rb,2.6300e-07_rb,1.8538e-07_rb,1.5076e-07_rb, &
     &1.9474e-07_rb,2.9543e-07_rb,2.0093e-07_rb,1.5819e-07_rb,1.5826e-07_rb,1.5737e-07_rb, &
     &1.5751e-07_rb,1.5910e-07_rb,1.6181e-07_rb,1.6320e-07_rb/)
      forrefo(3,:) = (/ &
     &1.0470e-06_rb,5.8184e-07_rb,4.8218e-07_rb,2.7771e-07_rb,1.9036e-07_rb,1.5737e-07_rb, &
     &1.8633e-07_rb,2.5754e-07_rb,4.0647e-07_rb,1.5839e-07_rb,1.5914e-07_rb,1.5788e-07_rb, &
     &1.5731e-07_rb,1.5836e-07_rb,1.6103e-07_rb,1.6320e-07_rb/)
      forrefo(4,:) = (/ &
     &1.3891e-06_rb,5.4901e-07_rb,2.8850e-07_rb,1.9176e-07_rb,1.4549e-07_rb,1.3603e-07_rb, &
     &1.7472e-07_rb,2.9796e-07_rb,3.2452e-07_rb,2.5231e-07_rb,2.8195e-07_rb,1.5527e-07_rb, &
     &1.5507e-07_rb,1.5442e-07_rb,1.5275e-07_rb,1.6057e-07_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 1.73980e-03_rb, 1.41928e-03_rb, 1.15780e-03_rb, 9.44496e-04_rb, 7.70490e-04_rb, &
     & 6.28541e-04_rb, 5.12744e-04_rb, 4.18280e-04_rb, 3.41219e-04_rb, 2.78356e-04_rb/)
      selfrefo(:, 2) = (/ &
     & 1.84082e-03_rb, 1.50228e-03_rb, 1.22600e-03_rb, 1.00053e-03_rb, 8.16525e-04_rb, &
     & 6.66359e-04_rb, 5.43811e-04_rb, 4.43800e-04_rb, 3.62182e-04_rb, 2.95574e-04_rb/)
      selfrefo(:, 3) = (/ &
     & 1.92957e-03_rb, 1.57727e-03_rb, 1.28930e-03_rb, 1.05390e-03_rb, 8.61484e-04_rb, &
     & 7.04197e-04_rb, 5.75627e-04_rb, 4.70530e-04_rb, 3.84622e-04_rb, 3.14399e-04_rb/)
      selfrefo(:, 4) = (/ &
     & 2.12958e-03_rb, 1.73572e-03_rb, 1.41470e-03_rb, 1.15305e-03_rb, 9.39798e-04_rb, &
     & 7.65984e-04_rb, 6.24317e-04_rb, 5.08850e-04_rb, 4.14739e-04_rb, 3.38034e-04_rb/)
      selfrefo(:, 5) = (/ &
     & 2.30636e-03_rb, 1.88401e-03_rb, 1.53900e-03_rb, 1.25717e-03_rb, 1.02695e-03_rb, &
     & 8.38891e-04_rb, 6.85270e-04_rb, 5.59780e-04_rb, 4.57270e-04_rb, 3.73533e-04_rb/)
      selfrefo(:, 6) = (/ &
     & 2.47824e-03_rb, 2.03278e-03_rb, 1.66740e-03_rb, 1.36769e-03_rb, 1.12185e-03_rb, &
     & 9.20206e-04_rb, 7.54803e-04_rb, 6.19130e-04_rb, 5.07844e-04_rb, 4.16561e-04_rb/)
      selfrefo(:, 7) = (/ &
     & 2.54196e-03_rb, 2.10768e-03_rb, 1.74760e-03_rb, 1.44904e-03_rb, 1.20148e-03_rb, &
     & 9.96215e-04_rb, 8.26019e-04_rb, 6.84900e-04_rb, 5.67890e-04_rb, 4.70870e-04_rb/)
      selfrefo(:, 8) = (/ &
     & 2.52650e-03_rb, 2.11773e-03_rb, 1.77510e-03_rb, 1.48790e-03_rb, 1.24717e-03_rb, &
     & 1.04539e-03_rb, 8.76251e-04_rb, 7.34480e-04_rb, 6.15646e-04_rb, 5.16039e-04_rb/)
      selfrefo(:, 9) = (/ &
     & 2.82351e-03_rb, 2.34652e-03_rb, 1.95010e-03_rb, 1.62065e-03_rb, 1.34686e-03_rb, &
     & 1.11933e-03_rb, 9.30232e-04_rb, 7.73080e-04_rb, 6.42477e-04_rb, 5.33939e-04_rb/)
      selfrefo(:,10) = (/ &
     & 2.98189e-03_rb, 2.46741e-03_rb, 2.04170e-03_rb, 1.68944e-03_rb, 1.39795e-03_rb, &
     & 1.15676e-03_rb, 9.57176e-04_rb, 7.92030e-04_rb, 6.55377e-04_rb, 5.42302e-04_rb/)
      selfrefo(:,11) = (/ &
     & 2.98239e-03_rb, 2.46774e-03_rb, 2.04190e-03_rb, 1.68954e-03_rb, 1.39799e-03_rb, &
     & 1.15675e-03_rb, 9.57137e-04_rb, 7.91970e-04_rb, 6.55305e-04_rb, 5.42224e-04_rb/)
      selfrefo(:,12) = (/ &
     & 2.97833e-03_rb, 2.46461e-03_rb, 2.03950e-03_rb, 1.68772e-03_rb, 1.39661e-03_rb, &
     & 1.15571e-03_rb, 9.56370e-04_rb, 7.91410e-04_rb, 6.54903e-04_rb, 5.41942e-04_rb/)
      selfrefo(:,13) = (/ &
     & 2.97779e-03_rb, 2.46463e-03_rb, 2.03990e-03_rb, 1.68836e-03_rb, 1.39741e-03_rb, &
     & 1.15659e-03_rb, 9.57278e-04_rb, 7.92310e-04_rb, 6.55771e-04_rb, 5.42762e-04_rb/)
      selfrefo(:,14) = (/ &
     & 2.98326e-03_rb, 2.46943e-03_rb, 2.04410e-03_rb, 1.69203e-03_rb, 1.40060e-03_rb, &
     & 1.15936e-03_rb, 9.59673e-04_rb, 7.94380e-04_rb, 6.57557e-04_rb, 5.44301e-04_rb/)
      selfrefo(:,15) = (/ &
     & 2.99407e-03_rb, 2.47825e-03_rb, 2.05130e-03_rb, 1.69790e-03_rb, 1.40539e-03_rb, &
     & 1.16327e-03_rb, 9.62862e-04_rb, 7.96980e-04_rb, 6.59676e-04_rb, 5.46028e-04_rb/)
      selfrefo(:,16) = (/ &
     & 3.00005e-03_rb, 2.48296e-03_rb, 2.05500e-03_rb, 1.70080e-03_rb, 1.40765e-03_rb, &
     & 1.16503e-03_rb, 9.64224e-04_rb, 7.98030e-04_rb, 6.60481e-04_rb, 5.46641e-04_rb/)

      end subroutine lw_kgb15

! **************************************************************************
      subroutine lw_kgb16
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrlw_kg16, only : fracrefao, fracrefbo, kao, kbo, selfrefo, forrefo

      implicit none
      save

! Planck fraction mapping level: P = 387.6100 mbar, T = 250.17 K
      fracrefao(:, 1) = (/ &
     &  1.1593e-01_rb,2.3390e-01_rb,1.9120e-01_rb,1.3121e-01_rb,1.0590e-01_rb,8.4852e-02_rb, &
     &  6.4168e-02_rb,4.2537e-02_rb,2.3220e-02_rb,2.1767e-03_rb,1.8203e-03_rb,1.3724e-03_rb, &
     &  9.5452e-04_rb,5.5015e-04_rb,1.9348e-04_rb,2.7344e-05_rb/)
      fracrefao(:, 2) = (/ &
     &  2.8101e-01_rb,1.9773e-01_rb,1.4749e-01_rb,1.1399e-01_rb,8.8190e-02_rb,7.0531e-02_rb, &
     &  4.6356e-02_rb,3.0774e-02_rb,1.7332e-02_rb,2.0054e-03_rb,1.5950e-03_rb,1.2760e-03_rb, &
     &  9.5034e-04_rb,5.4992e-04_rb,1.9349e-04_rb,2.7309e-05_rb/)
      fracrefao(:, 3) = (/ &
     &  2.9054e-01_rb,2.1263e-01_rb,1.4133e-01_rb,1.1083e-01_rb,8.5107e-02_rb,6.5247e-02_rb, &
     &  4.4542e-02_rb,2.7205e-02_rb,1.6495e-02_rb,1.8453e-03_rb,1.5222e-03_rb,1.1884e-03_rb, &
     &  8.1094e-04_rb,4.9173e-04_rb,1.9344e-04_rb,2.7286e-05_rb/)
      fracrefao(:, 4) = (/ &
     &  2.9641e-01_rb,2.1738e-01_rb,1.4228e-01_rb,1.0830e-01_rb,8.2837e-02_rb,6.1359e-02_rb, &
     &  4.4683e-02_rb,2.5027e-02_rb,1.6057e-02_rb,1.7558e-03_rb,1.4193e-03_rb,1.0970e-03_rb, &
     &  7.8281e-04_rb,4.3260e-04_rb,1.4837e-04_rb,2.2958e-05_rb/)
      fracrefao(:, 5) = (/ &
     &  2.9553e-01_rb,2.2139e-01_rb,1.4816e-01_rb,1.0601e-01_rb,8.0048e-02_rb,6.0082e-02_rb, &
     &  4.3952e-02_rb,2.3788e-02_rb,1.5734e-02_rb,1.6586e-03_rb,1.3434e-03_rb,1.0281e-03_rb, &
     &  7.0256e-04_rb,4.2577e-04_rb,1.2803e-04_rb,1.3315e-05_rb/)
      fracrefao(:, 6) = (/ &
     &  2.9313e-01_rb,2.2476e-01_rb,1.5470e-01_rb,1.0322e-01_rb,7.8904e-02_rb,5.8175e-02_rb, &
     &  4.3097e-02_rb,2.3618e-02_rb,1.5385e-02_rb,1.5942e-03_rb,1.2702e-03_rb,9.5566e-04_rb, &
     &  6.5421e-04_rb,4.0165e-04_rb,1.2805e-04_rb,1.3355e-05_rb/)
      fracrefao(:, 7) = (/ &
     &  2.9069e-01_rb,2.2823e-01_rb,1.5995e-01_rb,1.0170e-01_rb,7.7287e-02_rb,5.6780e-02_rb, &
     &  4.1752e-02_rb,2.3899e-02_rb,1.4937e-02_rb,1.4916e-03_rb,1.1909e-03_rb,9.1307e-04_rb, &
     &  6.3518e-04_rb,3.9866e-04_rb,1.2805e-04_rb,1.3298e-05_rb/)
      fracrefao(:, 8) = (/ &
     &  2.8446e-01_rb,2.2651e-01_rb,1.7133e-01_rb,1.0299e-01_rb,7.4231e-02_rb,5.6031e-02_rb, &
     &  4.1368e-02_rb,2.4318e-02_rb,1.4135e-02_rb,1.4216e-03_rb,1.1465e-03_rb,8.9800e-04_rb, &
     &  6.3553e-04_rb,3.9536e-04_rb,1.2749e-04_rb,1.3298e-05_rb/)
      fracrefao(:, 9) = (/ &
     &  2.0568e-01_rb,2.5049e-01_rb,2.0568e-01_rb,1.1781e-01_rb,7.5579e-02_rb,5.8136e-02_rb, &
     &  4.2397e-02_rb,2.6544e-02_rb,1.3067e-02_rb,1.4061e-03_rb,1.1455e-03_rb,8.9408e-04_rb, &
     &  6.3652e-04_rb,3.9450e-04_rb,1.2841e-04_rb,1.3315e-05_rb/)

! Planck fraction mapping level : P=95.58350 mb, T = 215.70 K
      fracrefbo(:) = (/ &
     &  1.8111e-01_rb,2.2612e-01_rb,1.6226e-01_rb,1.1872e-01_rb,9.9048e-02_rb,8.0390e-02_rb, &
     &  6.1648e-02_rb,4.1704e-02_rb,2.2976e-02_rb,1.9263e-03_rb,1.4694e-03_rb,1.1498e-03_rb, &
     &  7.9906e-04_rb,4.8310e-04_rb,1.6188e-04_rb,2.2651e-05_rb/)

!     The array KAO contains absorption coefs for each of the 16 g-intervals
!     for a range of pressure levels > ~100mb, temperatures, and ratios
!     of water vapor to CO2.  The first index in the array, JS, runs
!     from 1 to 10, and corresponds to different gas column amount ratios,
!     as expressed through the binary species parameter eta, defined as
!     eta = gas1/(gas1 + (rat) * gas2), where rat is the 
!     ratio of the reference MLS column amount value of gas 1 
!     to that of gas2.
!     The 2nd index in the array, JT, which runs from 1 to 5, corresponds 
!     to different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this  pressure 
!     level, JT = 2 refers to the temperature
!     TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the reference pressure level (e.g. JP = 1 is for a
!     pressure of 1053.63 mb).  The fourth index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kao(:, 1, 1, 1) = (/ &
     &2.6621e-07_rb,6.0791e-06_rb,9.3282e-06_rb,1.2208e-05_rb,1.4897e-05_rb,1.7596e-05_rb, &
     &2.0729e-05_rb,2.6390e-05_rb,2.1078e-05_rb/)
      kao(:, 2, 1, 1) = (/ &
     &2.8927e-07_rb,6.1043e-06_rb,9.4020e-06_rb,1.2345e-05_rb,1.5126e-05_rb,1.7938e-05_rb, &
     &2.1210e-05_rb,2.6769e-05_rb,2.1943e-05_rb/)
      kao(:, 3, 1, 1) = (/ &
     &3.1534e-07_rb,6.1325e-06_rb,9.4674e-06_rb,1.2459e-05_rb,1.5335e-05_rb,1.8250e-05_rb, &
     &2.1635e-05_rb,2.7290e-05_rb,2.2718e-05_rb/)
      kao(:, 4, 1, 1) = (/ &
     &3.3330e-07_rb,6.1474e-06_rb,9.4677e-06_rb,1.2509e-05_rb,1.5444e-05_rb,1.8455e-05_rb, &
     &2.1960e-05_rb,2.7695e-05_rb,2.3390e-05_rb/)
      kao(:, 5, 1, 1) = (/ &
     &3.5345e-07_rb,6.1735e-06_rb,9.4652e-06_rb,1.2538e-05_rb,1.5529e-05_rb,1.8608e-05_rb, &
     &2.2197e-05_rb,2.8038e-05_rb,2.3906e-05_rb/)
      kao(:, 1, 2, 1) = (/ &
     &3.4774e-07_rb,6.3116e-06_rb,9.2039e-06_rb,1.1662e-05_rb,1.3842e-05_rb,1.5878e-05_rb, &
     &1.7925e-05_rb,2.1213e-05_rb,1.7372e-05_rb/)
      kao(:, 2, 2, 1) = (/ &
     &3.7649e-07_rb,6.3618e-06_rb,9.2771e-06_rb,1.1777e-05_rb,1.4038e-05_rb,1.6170e-05_rb, &
     &1.8345e-05_rb,2.1716e-05_rb,1.8135e-05_rb/)
      kao(:, 3, 2, 1) = (/ &
     &4.1206e-07_rb,6.4231e-06_rb,9.3322e-06_rb,1.1891e-05_rb,1.4202e-05_rb,1.6428e-05_rb, &
     &1.8750e-05_rb,2.2212e-05_rb,1.8847e-05_rb/)
      kao(:, 4, 2, 1) = (/ &
     &4.4004e-07_rb,6.4581e-06_rb,9.3579e-06_rb,1.1962e-05_rb,1.4316e-05_rb,1.6601e-05_rb, &
     &1.9025e-05_rb,2.2633e-05_rb,1.9458e-05_rb/)
      kao(:, 5, 2, 1) = (/ &
     &4.6078e-07_rb,6.4755e-06_rb,9.3704e-06_rb,1.1973e-05_rb,1.4371e-05_rb,1.6702e-05_rb, &
     &1.9215e-05_rb,2.2953e-05_rb,1.9948e-05_rb/)
      kao(:, 1, 3, 1) = (/ &
     &5.9817e-07_rb,7.8428e-06_rb,1.0496e-05_rb,1.2549e-05_rb,1.4230e-05_rb,1.5565e-05_rb, &
     &1.6588e-05_rb,1.7671e-05_rb,1.4245e-05_rb/)
      kao(:, 2, 3, 1) = (/ &
     &6.5182e-07_rb,7.9343e-06_rb,1.0585e-05_rb,1.2668e-05_rb,1.4392e-05_rb,1.5789e-05_rb, &
     &1.6949e-05_rb,1.8125e-05_rb,1.4996e-05_rb/)
      kao(:, 3, 3, 1) = (/ &
     &7.0221e-07_rb,8.0226e-06_rb,1.0677e-05_rb,1.2755e-05_rb,1.4518e-05_rb,1.5973e-05_rb, &
     &1.7236e-05_rb,1.8551e-05_rb,1.5656e-05_rb/)
      kao(:, 4, 3, 1) = (/ &
     &7.6366e-07_rb,8.1339e-06_rb,1.0778e-05_rb,1.2848e-05_rb,1.4645e-05_rb,1.6156e-05_rb, &
     &1.7457e-05_rb,1.8931e-05_rb,1.6271e-05_rb/)
      kao(:, 5, 3, 1) = (/ &
     &7.9889e-07_rb,8.1643e-06_rb,1.0808e-05_rb,1.2886e-05_rb,1.4686e-05_rb,1.6241e-05_rb, &
     &1.7601e-05_rb,1.9193e-05_rb,1.6837e-05_rb/)
      kao(:, 1, 4, 1) = (/ &
     &1.0320e-06_rb,1.0378e-05_rb,1.2935e-05_rb,1.4632e-05_rb,1.5802e-05_rb,1.6452e-05_rb, &
     &1.6586e-05_rb,1.6155e-05_rb,1.2287e-05_rb/)
      kao(:, 2, 4, 1) = (/ &
     &1.1305e-06_rb,1.0573e-05_rb,1.3130e-05_rb,1.4798e-05_rb,1.5980e-05_rb,1.6653e-05_rb, &
     &1.6853e-05_rb,1.6526e-05_rb,1.3070e-05_rb/)
      kao(:, 3, 4, 1) = (/ &
     &1.2198e-06_rb,1.0772e-05_rb,1.3274e-05_rb,1.4971e-05_rb,1.6111e-05_rb,1.6834e-05_rb, &
     &1.7084e-05_rb,1.6873e-05_rb,1.3795e-05_rb/)
      kao(:, 4, 4, 1) = (/ &
     &1.3096e-06_rb,1.0937e-05_rb,1.3422e-05_rb,1.5096e-05_rb,1.6233e-05_rb,1.6971e-05_rb, &
     &1.7284e-05_rb,1.7149e-05_rb,1.4420e-05_rb/)
      kao(:, 5, 4, 1) = (/ &
     &1.4155e-06_rb,1.1094e-05_rb,1.3596e-05_rb,1.5273e-05_rb,1.6388e-05_rb,1.7112e-05_rb, &
     &1.7471e-05_rb,1.7381e-05_rb,1.4960e-05_rb/)
      kao(:, 1, 5, 1) = (/ &
     &1.6279e-06_rb,1.3142e-05_rb,1.6056e-05_rb,1.7394e-05_rb,1.8017e-05_rb,1.8011e-05_rb, &
     &1.7308e-05_rb,1.5625e-05_rb,1.1008e-05_rb/)
      kao(:, 2, 5, 1) = (/ &
     &1.7835e-06_rb,1.3457e-05_rb,1.6348e-05_rb,1.7668e-05_rb,1.8277e-05_rb,1.8261e-05_rb, &
     &1.7567e-05_rb,1.5970e-05_rb,1.1698e-05_rb/)
      kao(:, 3, 5, 1) = (/ &
     &1.9402e-06_rb,1.3791e-05_rb,1.6620e-05_rb,1.7922e-05_rb,1.8515e-05_rb,1.8474e-05_rb, &
     &1.7802e-05_rb,1.6252e-05_rb,1.2372e-05_rb/)
      kao(:, 4, 5, 1) = (/ &
     &2.0835e-06_rb,1.4086e-05_rb,1.6875e-05_rb,1.8160e-05_rb,1.8703e-05_rb,1.8665e-05_rb, &
     &1.7992e-05_rb,1.6486e-05_rb,1.3071e-05_rb/)
      kao(:, 5, 5, 1) = (/ &
     &2.2582e-06_rb,1.4511e-05_rb,1.7205e-05_rb,1.8422e-05_rb,1.8984e-05_rb,1.8898e-05_rb, &
     &1.8217e-05_rb,1.6724e-05_rb,1.3787e-05_rb/)
      kao(:, 1, 6, 1) = (/ &
     &2.4034e-06_rb,1.6823e-05_rb,1.9391e-05_rb,2.0723e-05_rb,2.0772e-05_rb,2.0043e-05_rb, &
     &1.8498e-05_rb,1.5704e-05_rb,9.8836e-06_rb/)
      kao(:, 2, 6, 1) = (/ &
     &2.6386e-06_rb,1.7226e-05_rb,1.9692e-05_rb,2.1124e-05_rb,2.1126e-05_rb,2.0392e-05_rb, &
     &1.8810e-05_rb,1.6034e-05_rb,1.0751e-05_rb/)
      kao(:, 3, 6, 1) = (/ &
     &2.8660e-06_rb,1.7674e-05_rb,2.0029e-05_rb,2.1524e-05_rb,2.1484e-05_rb,2.0719e-05_rb, &
     &1.9071e-05_rb,1.6298e-05_rb,1.1450e-05_rb/)
      kao(:, 4, 6, 1) = (/ &
     &3.0844e-06_rb,1.8072e-05_rb,2.0407e-05_rb,2.1899e-05_rb,2.1788e-05_rb,2.0987e-05_rb, &
     &1.9304e-05_rb,1.6540e-05_rb,1.2182e-05_rb/)
      kao(:, 5, 6, 1) = (/ &
     &3.2920e-06_rb,1.8533e-05_rb,2.0800e-05_rb,2.2106e-05_rb,2.2044e-05_rb,2.1223e-05_rb, &
     &1.9506e-05_rb,1.6741e-05_rb,1.2836e-05_rb/)
      kao(:, 1, 7, 1) = (/ &
     &3.8221e-06_rb,2.2204e-05_rb,2.4808e-05_rb,2.5531e-05_rb,2.5616e-05_rb,2.3917e-05_rb, &
     &2.1120e-05_rb,1.6870e-05_rb,8.7171e-06_rb/)
      kao(:, 2, 7, 1) = (/ &
     &4.1828e-06_rb,2.2870e-05_rb,2.5558e-05_rb,2.5953e-05_rb,2.6151e-05_rb,2.4347e-05_rb, &
     &2.1524e-05_rb,1.7202e-05_rb,9.2028e-06_rb/)
      kao(:, 3, 7, 1) = (/ &
     &4.5497e-06_rb,2.3564e-05_rb,2.6164e-05_rb,2.6493e-05_rb,2.6591e-05_rb,2.4767e-05_rb, &
     &2.1899e-05_rb,1.7498e-05_rb,9.7937e-06_rb/)
      kao(:, 4, 7, 1) = (/ &
     &4.8742e-06_rb,2.4270e-05_rb,2.6708e-05_rb,2.7099e-05_rb,2.6759e-05_rb,2.5177e-05_rb, &
     &2.2242e-05_rb,1.7759e-05_rb,1.0449e-05_rb/)
      kao(:, 5, 7, 1) = (/ &
     &5.1866e-06_rb,2.4962e-05_rb,2.7348e-05_rb,2.7664e-05_rb,2.7129e-05_rb,2.5537e-05_rb, &
     &2.2515e-05_rb,1.7998e-05_rb,1.1128e-05_rb/)
      kao(:, 1, 8, 1) = (/ &
     &7.4211e-06_rb,3.2998e-05_rb,3.6610e-05_rb,3.7073e-05_rb,3.4983e-05_rb,3.2511e-05_rb, &
     &2.7821e-05_rb,2.0603e-05_rb,7.1809e-06_rb/)
      kao(:, 2, 8, 1) = (/ &
     &8.1099e-06_rb,3.4342e-05_rb,3.7801e-05_rb,3.7976e-05_rb,3.5797e-05_rb,3.2823e-05_rb, &
     &2.8347e-05_rb,2.1030e-05_rb,7.7735e-06_rb/)
      kao(:, 3, 8, 1) = (/ &
     &8.7992e-06_rb,3.5880e-05_rb,3.8837e-05_rb,3.8975e-05_rb,3.6756e-05_rb,3.3300e-05_rb, &
     &2.8882e-05_rb,2.1428e-05_rb,8.4180e-06_rb/)
      kao(:, 4, 8, 1) = (/ &
     &9.4486e-06_rb,3.7431e-05_rb,3.9883e-05_rb,3.9775e-05_rb,3.7762e-05_rb,3.3901e-05_rb, &
     &2.9403e-05_rb,2.1792e-05_rb,9.1057e-06_rb/)
      kao(:, 5, 8, 1) = (/ &
     &1.0041e-05_rb,3.9014e-05_rb,4.1019e-05_rb,4.0539e-05_rb,3.8447e-05_rb,3.4543e-05_rb, &
     &2.9823e-05_rb,2.2110e-05_rb,9.8280e-06_rb/)
      kao(:, 1, 9, 1) = (/ &
     &2.6945e-05_rb,8.1804e-05_rb,8.6984e-05_rb,8.4462e-05_rb,7.8660e-05_rb,6.9398e-05_rb, &
     &5.6216e-05_rb,3.9134e-05_rb,6.0831e-06_rb/)
      kao(:, 2, 9, 1) = (/ &
     &2.9562e-05_rb,8.5456e-05_rb,9.1284e-05_rb,8.7620e-05_rb,8.1311e-05_rb,7.1570e-05_rb, &
     &5.7999e-05_rb,3.9483e-05_rb,6.6855e-06_rb/)
      kao(:, 3, 9, 1) = (/ &
     &3.2110e-05_rb,8.8183e-05_rb,9.6256e-05_rb,9.1033e-05_rb,8.3882e-05_rb,7.3603e-05_rb, &
     &5.9334e-05_rb,4.0067e-05_rb,7.3573e-06_rb/)
      kao(:, 4, 9, 1) = (/ &
     &3.4494e-05_rb,9.0981e-05_rb,1.0082e-04_rb,9.4837e-05_rb,8.6631e-05_rb,7.5471e-05_rb, &
     &6.0778e-05_rb,4.0870e-05_rb,8.0874e-06_rb/)
      kao(:, 5, 9, 1) = (/ &
     &3.6729e-05_rb,9.3406e-05_rb,1.0411e-04_rb,9.8963e-05_rb,8.9287e-05_rb,7.7185e-05_rb, &
     &6.1987e-05_rb,4.1723e-05_rb,8.8598e-06_rb/)
      kao(:, 1,10, 1) = (/ &
     &1.1063e-04_rb,2.0470e-04_rb,2.2594e-04_rb,2.2784e-04_rb,2.0878e-04_rb,1.7781e-04_rb, &
     &1.3876e-04_rb,9.0190e-05_rb,6.6238e-06_rb/)
      kao(:, 2,10, 1) = (/ &
     &1.2128e-04_rb,2.1612e-04_rb,2.3657e-04_rb,2.3782e-04_rb,2.2024e-04_rb,1.8545e-04_rb, &
     &1.4406e-04_rb,9.3147e-05_rb,7.2888e-06_rb/)
      kao(:, 3,10, 1) = (/ &
     &1.3155e-04_rb,2.2685e-04_rb,2.4613e-04_rb,2.4648e-04_rb,2.3100e-04_rb,1.9523e-04_rb, &
     &1.4909e-04_rb,9.5827e-05_rb,7.7784e-06_rb/)
      kao(:, 4,10, 1) = (/ &
     &1.4108e-04_rb,2.3656e-04_rb,2.5475e-04_rb,2.5395e-04_rb,2.3879e-04_rb,2.0427e-04_rb, &
     &1.5511e-04_rb,9.8368e-05_rb,8.3069e-06_rb/)
      kao(:, 5,10, 1) = (/ &
     &1.4988e-04_rb,2.4526e-04_rb,2.6296e-04_rb,2.6141e-04_rb,2.4593e-04_rb,2.1313e-04_rb, &
     &1.6065e-04_rb,1.0083e-04_rb,8.9709e-06_rb/)
      kao(:, 1,11, 1) = (/ &
     &1.6397e-04_rb,2.6056e-04_rb,2.8031e-04_rb,2.7866e-04_rb,2.6081e-04_rb,2.2383e-04_rb, &
     &1.7351e-04_rb,1.0962e-04_rb,6.8264e-06_rb/)
      kao(:, 2,11, 1) = (/ &
     &1.7933e-04_rb,2.7580e-04_rb,2.9343e-04_rb,2.9042e-04_rb,2.7290e-04_rb,2.3596e-04_rb, &
     &1.8069e-04_rb,1.1353e-04_rb,7.5502e-06_rb/)
      kao(:, 3,11, 1) = (/ &
     &1.9324e-04_rb,2.9031e-04_rb,3.0629e-04_rb,3.0108e-04_rb,2.8182e-04_rb,2.4730e-04_rb, &
     &1.8878e-04_rb,1.1713e-04_rb,7.9027e-06_rb/)
      kao(:, 4,11, 1) = (/ &
     &2.0654e-04_rb,3.0384e-04_rb,3.1739e-04_rb,3.1103e-04_rb,2.9049e-04_rb,2.5555e-04_rb, &
     &1.9705e-04_rb,1.2084e-04_rb,8.5383e-06_rb/)
      kao(:, 5,11, 1) = (/ &
     &2.1856e-04_rb,3.1574e-04_rb,3.2801e-04_rb,3.2141e-04_rb,2.9895e-04_rb,2.6285e-04_rb, &
     &2.0518e-04_rb,1.2441e-04_rb,9.1615e-06_rb/)
      kao(:, 1,12, 1) = (/ &
     &1.7554e-04_rb,2.6383e-04_rb,2.7930e-04_rb,2.7562e-04_rb,2.5860e-04_rb,2.2333e-04_rb, &
     &1.7363e-04_rb,1.0883e-04_rb,5.7385e-06_rb/)
      kao(:, 2,12, 1) = (/ &
     &1.9241e-04_rb,2.8007e-04_rb,2.9311e-04_rb,2.8776e-04_rb,2.6852e-04_rb,2.3492e-04_rb, &
     &1.8150e-04_rb,1.1266e-04_rb,6.8723e-06_rb/)
      kao(:, 3,12, 1) = (/ &
     &2.0743e-04_rb,2.9497e-04_rb,3.0602e-04_rb,2.9878e-04_rb,2.7794e-04_rb,2.4424e-04_rb, &
     &1.8850e-04_rb,1.1619e-04_rb,7.4728e-06_rb/)
      kao(:, 4,12, 1) = (/ &
     &2.2033e-04_rb,3.0884e-04_rb,3.1746e-04_rb,3.0874e-04_rb,2.8629e-04_rb,2.5127e-04_rb, &
     &1.9681e-04_rb,1.2004e-04_rb,7.8331e-06_rb/)
      kao(:, 5,12, 1) = (/ &
     &2.3385e-04_rb,3.2096e-04_rb,3.2817e-04_rb,3.1852e-04_rb,2.9492e-04_rb,2.5838e-04_rb, &
     &2.0541e-04_rb,1.2394e-04_rb,8.4107e-06_rb/)
      kao(:, 1,13, 1) = (/ &
     &1.4984e-04_rb,2.2462e-04_rb,2.3735e-04_rb,2.3350e-04_rb,2.1852e-04_rb,1.8858e-04_rb, &
     &1.4655e-04_rb,9.1919e-05_rb,4.7944e-06_rb/)
      kao(:, 2,13, 1) = (/ &
     &1.6431e-04_rb,2.3822e-04_rb,2.4852e-04_rb,2.4374e-04_rb,2.2685e-04_rb,1.9798e-04_rb, &
     &1.5298e-04_rb,9.5139e-05_rb,5.7512e-06_rb/)
      kao(:, 3,13, 1) = (/ &
     &1.7714e-04_rb,2.5149e-04_rb,2.5950e-04_rb,2.5249e-04_rb,2.3438e-04_rb,2.0591e-04_rb, &
     &1.5891e-04_rb,9.8151e-05_rb,6.2816e-06_rb/)
      kao(:, 4,13, 1) = (/ &
     &1.8918e-04_rb,2.6227e-04_rb,2.6867e-04_rb,2.6081e-04_rb,2.4142e-04_rb,2.1149e-04_rb, &
     &1.6602e-04_rb,1.0121e-04_rb,6.5742e-06_rb/)
      kao(:, 5,13, 1) = (/ &
     &1.9984e-04_rb,2.7202e-04_rb,2.7776e-04_rb,2.6884e-04_rb,2.4875e-04_rb,2.1766e-04_rb, &
     &1.7300e-04_rb,1.0449e-04_rb,7.0482e-06_rb/)
      kao(:, 1, 1, 2) = (/ &
     &1.6519e-06_rb,1.6500e-05_rb,2.5026e-05_rb,3.2759e-05_rb,4.0208e-05_rb,4.7995e-05_rb, &
     &5.7218e-05_rb,7.0450e-05_rb,6.2071e-05_rb/)
      kao(:, 2, 1, 2) = (/ &
     &1.7427e-06_rb,1.6766e-05_rb,2.5514e-05_rb,3.3509e-05_rb,4.1226e-05_rb,4.9256e-05_rb, &
     &5.8782e-05_rb,7.2775e-05_rb,6.4227e-05_rb/)
      kao(:, 3, 1, 2) = (/ &
     &1.8342e-06_rb,1.7039e-05_rb,2.6012e-05_rb,3.4217e-05_rb,4.2214e-05_rb,5.0504e-05_rb, &
     &6.0325e-05_rb,7.4727e-05_rb,6.6376e-05_rb/)
      kao(:, 4, 1, 2) = (/ &
     &1.9060e-06_rb,1.7243e-05_rb,2.6493e-05_rb,3.4981e-05_rb,4.3216e-05_rb,5.1749e-05_rb, &
     &6.1747e-05_rb,7.6716e-05_rb,6.8482e-05_rb/)
      kao(:, 5, 1, 2) = (/ &
     &1.9873e-06_rb,1.7462e-05_rb,2.6980e-05_rb,3.5686e-05_rb,4.4149e-05_rb,5.2919e-05_rb, &
     &6.3181e-05_rb,7.8643e-05_rb,7.0600e-05_rb/)
      kao(:, 1, 2, 2) = (/ &
     &2.1537e-06_rb,1.7686e-05_rb,2.5477e-05_rb,3.2084e-05_rb,3.8037e-05_rb,4.3791e-05_rb, &
     &5.0118e-05_rb,5.9749e-05_rb,5.2659e-05_rb/)
      kao(:, 2, 2, 2) = (/ &
     &2.2967e-06_rb,1.7950e-05_rb,2.5979e-05_rb,3.2808e-05_rb,3.8966e-05_rb,4.4991e-05_rb, &
     &5.1587e-05_rb,6.1578e-05_rb,5.4559e-05_rb/)
      kao(:, 3, 2, 2) = (/ &
     &2.4223e-06_rb,1.8241e-05_rb,2.6472e-05_rb,3.3476e-05_rb,3.9818e-05_rb,4.6056e-05_rb, &
     &5.2876e-05_rb,6.3201e-05_rb,5.6392e-05_rb/)
      kao(:, 4, 2, 2) = (/ &
     &2.5452e-06_rb,1.8478e-05_rb,2.6920e-05_rb,3.4112e-05_rb,4.0714e-05_rb,4.7174e-05_rb, &
     &5.4253e-05_rb,6.4831e-05_rb,5.8337e-05_rb/)
      kao(:, 5, 2, 2) = (/ &
     &2.6442e-06_rb,1.8696e-05_rb,2.7330e-05_rb,3.4790e-05_rb,4.1641e-05_rb,4.8326e-05_rb, &
     &5.5567e-05_rb,6.6400e-05_rb,6.0213e-05_rb/)
      kao(:, 1, 3, 2) = (/ &
     &3.6781e-06_rb,2.1946e-05_rb,2.9916e-05_rb,3.5559e-05_rb,4.0089e-05_rb,4.3735e-05_rb, &
     &4.6884e-05_rb,5.0541e-05_rb,4.3984e-05_rb/)
      kao(:, 2, 3, 2) = (/ &
     &3.9348e-06_rb,2.2135e-05_rb,3.0461e-05_rb,3.6306e-05_rb,4.1005e-05_rb,4.4863e-05_rb, &
     &4.8174e-05_rb,5.2207e-05_rb,4.5557e-05_rb/)
      kao(:, 3, 3, 2) = (/ &
     &4.1798e-06_rb,2.2403e-05_rb,3.0963e-05_rb,3.7011e-05_rb,4.1865e-05_rb,4.5861e-05_rb, &
     &4.9330e-05_rb,5.3612e-05_rb,4.7151e-05_rb/)
      kao(:, 4, 3, 2) = (/ &
     &4.4176e-06_rb,2.2879e-05_rb,3.1514e-05_rb,3.7717e-05_rb,4.2706e-05_rb,4.6836e-05_rb, &
     &5.0501e-05_rb,5.4941e-05_rb,4.8733e-05_rb/)
      kao(:, 5, 3, 2) = (/ &
     &4.5930e-06_rb,2.3205e-05_rb,3.1822e-05_rb,3.8274e-05_rb,4.3472e-05_rb,4.7803e-05_rb, &
     &5.1670e-05_rb,5.6326e-05_rb,5.0305e-05_rb/)
      kao(:, 1, 4, 2) = (/ &
     &6.4358e-06_rb,2.9282e-05_rb,3.6943e-05_rb,4.2596e-05_rb,4.5632e-05_rb,4.7319e-05_rb, &
     &4.7633e-05_rb,4.6727e-05_rb,3.6481e-05_rb/)
      kao(:, 2, 4, 2) = (/ &
     &6.8669e-06_rb,2.9817e-05_rb,3.7275e-05_rb,4.3436e-05_rb,4.6592e-05_rb,4.8428e-05_rb, &
     &4.8930e-05_rb,4.8153e-05_rb,3.7877e-05_rb/)
      kao(:, 3, 4, 2) = (/ &
     &7.3364e-06_rb,3.0199e-05_rb,3.7809e-05_rb,4.3827e-05_rb,4.7553e-05_rb,4.9496e-05_rb, &
     &5.0112e-05_rb,4.9423e-05_rb,3.9242e-05_rb/)
      kao(:, 4, 4, 2) = (/ &
     &7.7623e-06_rb,3.0806e-05_rb,3.8513e-05_rb,4.4137e-05_rb,4.8467e-05_rb,5.0544e-05_rb, &
     &5.1178e-05_rb,5.0627e-05_rb,4.0689e-05_rb/)
      kao(:, 5, 4, 2) = (/ &
     &8.2042e-06_rb,3.1502e-05_rb,3.9409e-05_rb,4.4849e-05_rb,4.9430e-05_rb,5.1580e-05_rb, &
     &5.2292e-05_rb,5.1835e-05_rb,4.2152e-05_rb/)
      kao(:, 1, 5, 2) = (/ &
     &1.0402e-05_rb,3.8540e-05_rb,4.6318e-05_rb,5.0020e-05_rb,5.3672e-05_rb,5.3150e-05_rb, &
     &5.0796e-05_rb,4.5989e-05_rb,3.1155e-05_rb/)
      kao(:, 2, 5, 2) = (/ &
     &1.1085e-05_rb,3.9241e-05_rb,4.7238e-05_rb,5.0920e-05_rb,5.3596e-05_rb,5.4294e-05_rb, &
     &5.2010e-05_rb,4.7261e-05_rb,3.2691e-05_rb/)
      kao(:, 3, 5, 2) = (/ &
     &1.1793e-05_rb,3.9912e-05_rb,4.8110e-05_rb,5.1979e-05_rb,5.4361e-05_rb,5.5470e-05_rb, &
     &5.3221e-05_rb,4.8498e-05_rb,3.4229e-05_rb/)
      kao(:, 4, 5, 2) = (/ &
     &1.2505e-05_rb,4.0706e-05_rb,4.8865e-05_rb,5.3127e-05_rb,5.5321e-05_rb,5.6646e-05_rb, &
     &5.4402e-05_rb,4.9696e-05_rb,3.5620e-05_rb/)
      kao(:, 5, 5, 2) = (/ &
     &1.3238e-05_rb,4.1575e-05_rb,5.0157e-05_rb,5.4552e-05_rb,5.6339e-05_rb,5.7036e-05_rb, &
     &5.5608e-05_rb,5.0863e-05_rb,3.6859e-05_rb/)
      kao(:, 1, 6, 2) = (/ &
     &1.5685e-05_rb,4.8160e-05_rb,5.7576e-05_rb,6.0816e-05_rb,6.0795e-05_rb,5.9888e-05_rb, &
     &5.5641e-05_rb,4.7076e-05_rb,2.7125e-05_rb/)
      kao(:, 2, 6, 2) = (/ &
     &1.6746e-05_rb,4.9438e-05_rb,5.8728e-05_rb,6.2091e-05_rb,6.2171e-05_rb,6.0759e-05_rb, &
     &5.6976e-05_rb,4.8239e-05_rb,2.8411e-05_rb/)
      kao(:, 3, 6, 2) = (/ &
     &1.7756e-05_rb,5.0622e-05_rb,5.9836e-05_rb,6.3203e-05_rb,6.3569e-05_rb,6.1458e-05_rb, &
     &5.8288e-05_rb,4.9429e-05_rb,2.9823e-05_rb/)
      kao(:, 4, 6, 2) = (/ &
     &1.8829e-05_rb,5.1859e-05_rb,6.0806e-05_rb,6.4509e-05_rb,6.5119e-05_rb,6.2540e-05_rb, &
     &5.8809e-05_rb,5.0652e-05_rb,3.1085e-05_rb/)
      kao(:, 5, 6, 2) = (/ &
     &1.9856e-05_rb,5.2975e-05_rb,6.2034e-05_rb,6.5989e-05_rb,6.6522e-05_rb,6.3894e-05_rb, &
     &5.9538e-05_rb,5.1853e-05_rb,3.2453e-05_rb/)
      kao(:, 1, 7, 2) = (/ &
     &2.4856e-05_rb,6.4218e-05_rb,7.4055e-05_rb,7.7980e-05_rb,7.5911e-05_rb,7.1057e-05_rb, &
     &6.3849e-05_rb,5.1579e-05_rb,2.5153e-05_rb/)
      kao(:, 2, 7, 2) = (/ &
     &2.6608e-05_rb,6.6305e-05_rb,7.5730e-05_rb,7.9533e-05_rb,7.7616e-05_rb,7.2887e-05_rb, &
     &6.4610e-05_rb,5.2907e-05_rb,2.6881e-05_rb/)
      kao(:, 3, 7, 2) = (/ &
     &2.8288e-05_rb,6.8291e-05_rb,7.7618e-05_rb,8.1185e-05_rb,7.9322e-05_rb,7.4738e-05_rb, &
     &6.5685e-05_rb,5.4242e-05_rb,2.8455e-05_rb/)
      kao(:, 4, 7, 2) = (/ &
     &3.0045e-05_rb,7.0265e-05_rb,7.9536e-05_rb,8.2799e-05_rb,8.1209e-05_rb,7.6305e-05_rb, &
     &6.7186e-05_rb,5.5601e-05_rb,2.9886e-05_rb/)
      kao(:, 5, 7, 2) = (/ &
     &3.1755e-05_rb,7.2003e-05_rb,8.1183e-05_rb,8.4367e-05_rb,8.2925e-05_rb,7.7935e-05_rb, &
     &6.8892e-05_rb,5.5914e-05_rb,3.1298e-05_rb/)
      kao(:, 1, 8, 2) = (/ &
     &4.6308e-05_rb,9.7055e-05_rb,1.0843e-04_rb,1.0990e-04_rb,1.0798e-04_rb,9.8279e-05_rb, &
     &8.2910e-05_rb,6.2908e-05_rb,2.6615e-05_rb/)
      kao(:, 2, 8, 2) = (/ &
     &4.9990e-05_rb,1.0048e-04_rb,1.1189e-04_rb,1.1329e-04_rb,1.1038e-04_rb,1.0108e-04_rb, &
     &8.5246e-05_rb,6.3800e-05_rb,2.8720e-05_rb/)
      kao(:, 3, 8, 2) = (/ &
     &5.3406e-05_rb,1.0353e-04_rb,1.1536e-04_rb,1.1646e-04_rb,1.1270e-04_rb,1.0316e-04_rb, &
     &8.7419e-05_rb,6.5105e-05_rb,3.1055e-05_rb/)
      kao(:, 4, 8, 2) = (/ &
     &5.6634e-05_rb,1.0658e-04_rb,1.1884e-04_rb,1.1968e-04_rb,1.1489e-04_rb,1.0536e-04_rb, &
     &8.9223e-05_rb,6.6739e-05_rb,3.2877e-05_rb/)
      kao(:, 5, 8, 2) = (/ &
     &5.9979e-05_rb,1.0942e-04_rb,1.2217e-04_rb,1.2302e-04_rb,1.1760e-04_rb,1.0763e-04_rb, &
     &9.1750e-05_rb,6.8524e-05_rb,3.4517e-05_rb/)
      kao(:, 1, 9, 2) = (/ &
     &1.6574e-04_rb,2.3675e-04_rb,2.5425e-04_rb,2.5367e-04_rb,2.3792e-04_rb,2.1003e-04_rb, &
     &1.7177e-04_rb,1.1936e-04_rb,3.2512e-05_rb/)
      kao(:, 2, 9, 2) = (/ &
     &1.7873e-04_rb,2.4657e-04_rb,2.6302e-04_rb,2.6299e-04_rb,2.4679e-04_rb,2.1704e-04_rb, &
     &1.7629e-04_rb,1.2297e-04_rb,3.4956e-05_rb/)
      kao(:, 3, 9, 2) = (/ &
     &1.9203e-04_rb,2.5827e-04_rb,2.7104e-04_rb,2.7133e-04_rb,2.5463e-04_rb,2.2408e-04_rb, &
     &1.8102e-04_rb,1.2579e-04_rb,3.6986e-05_rb/)
      kao(:, 4, 9, 2) = (/ &
     &2.0457e-04_rb,2.7047e-04_rb,2.8009e-04_rb,2.7979e-04_rb,2.6236e-04_rb,2.3082e-04_rb, &
     &1.8610e-04_rb,1.2867e-04_rb,3.9201e-05_rb/)
      kao(:, 5, 9, 2) = (/ &
     &2.1654e-04_rb,2.8311e-04_rb,2.9060e-04_rb,2.8789e-04_rb,2.7019e-04_rb,2.3757e-04_rb, &
     &1.9128e-04_rb,1.3157e-04_rb,4.1162e-05_rb/)
      kao(:, 1,10, 2) = (/ &
     &6.9835e-04_rb,7.7425e-04_rb,7.4212e-04_rb,6.8606e-04_rb,6.1917e-04_rb,5.3712e-04_rb, &
     &4.2860e-04_rb,2.7889e-04_rb,3.4936e-05_rb/)
      kao(:, 2,10, 2) = (/ &
     &7.4976e-04_rb,8.1587e-04_rb,7.7895e-04_rb,7.1764e-04_rb,6.4311e-04_rb,5.5763e-04_rb, &
     &4.4404e-04_rb,2.8872e-04_rb,3.8434e-05_rb/)
      kao(:, 3,10, 2) = (/ &
     &8.0421e-04_rb,8.6133e-04_rb,8.1873e-04_rb,7.5123e-04_rb,6.6805e-04_rb,5.7460e-04_rb, &
     &4.5895e-04_rb,2.9845e-04_rb,4.2197e-05_rb/)
      kao(:, 4,10, 2) = (/ &
     &8.5541e-04_rb,9.0651e-04_rb,8.6169e-04_rb,7.8842e-04_rb,6.9649e-04_rb,5.9375e-04_rb, &
     &4.7283e-04_rb,3.0790e-04_rb,4.6109e-05_rb/)
      kao(:, 5,10, 2) = (/ &
     &9.0554e-04_rb,9.5112e-04_rb,9.0374e-04_rb,8.2600e-04_rb,7.2587e-04_rb,6.1366e-04_rb, &
     &4.8709e-04_rb,3.1697e-04_rb,4.9839e-05_rb/)
      kao(:, 1,11, 2) = (/ &
     &1.0573e-03_rb,1.1046e-03_rb,1.0331e-03_rb,9.3641e-04_rb,8.2149e-04_rb,6.9220e-04_rb, &
     &5.4269e-04_rb,3.5137e-04_rb,3.2311e-05_rb/)
      kao(:, 2,11, 2) = (/ &
     &1.1354e-03_rb,1.1699e-03_rb,1.0957e-03_rb,9.8791e-04_rb,8.6081e-04_rb,7.2123e-04_rb, &
     &5.6428e-04_rb,3.6441e-04_rb,3.5797e-05_rb/)
      kao(:, 3,11, 2) = (/ &
     &1.2157e-03_rb,1.2370e-03_rb,1.1565e-03_rb,1.0415e-03_rb,9.0546e-04_rb,7.4993e-04_rb, &
     &5.8345e-04_rb,3.7768e-04_rb,3.9564e-05_rb/)
      kao(:, 4,11, 2) = (/ &
     &1.2894e-03_rb,1.3041e-03_rb,1.2183e-03_rb,1.0951e-03_rb,9.4833e-04_rb,7.8292e-04_rb, &
     &6.0424e-04_rb,3.8988e-04_rb,4.3105e-05_rb/)
      kao(:, 5,11, 2) = (/ &
     &1.3602e-03_rb,1.3714e-03_rb,1.2775e-03_rb,1.1464e-03_rb,9.9122e-04_rb,8.1361e-04_rb, &
     &6.2375e-04_rb,4.0138e-04_rb,4.6502e-05_rb/)
      kao(:, 1,12, 2) = (/ &
     &1.1559e-03_rb,1.1821e-03_rb,1.1002e-03_rb,9.8760e-04_rb,8.5869e-04_rb,7.1597e-04_rb, &
     &5.5368e-04_rb,3.5629e-04_rb,2.8926e-05_rb/)
      kao(:, 2,12, 2) = (/ &
     &1.2387e-03_rb,1.2530e-03_rb,1.1642e-03_rb,1.0435e-03_rb,9.0422e-04_rb,7.4802e-04_rb, &
     &5.7617e-04_rb,3.7052e-04_rb,3.1589e-05_rb/)
      kao(:, 3,12, 2) = (/ &
     &1.3216e-03_rb,1.3229e-03_rb,1.2283e-03_rb,1.1015e-03_rb,9.5193e-04_rb,7.8187e-04_rb, &
     &5.9954e-04_rb,3.8460e-04_rb,3.4577e-05_rb/)
      kao(:, 4,12, 2) = (/ &
     &1.3983e-03_rb,1.3942e-03_rb,1.2931e-03_rb,1.1587e-03_rb,9.9709e-04_rb,8.1633e-04_rb, &
     &6.2085e-04_rb,3.9719e-04_rb,3.7982e-05_rb/)
      kao(:, 5,12, 2) = (/ &
     &1.4751e-03_rb,1.4634e-03_rb,1.3557e-03_rb,1.2116e-03_rb,1.0389e-03_rb,8.4961e-04_rb, &
     &6.4030e-04_rb,4.0878e-04_rb,4.0969e-05_rb/)
      kao(:, 1,13, 2) = (/ &
     &1.0038e-03_rb,1.0239e-03_rb,9.5244e-04_rb,8.5524e-04_rb,7.4348e-04_rb,6.2001e-04_rb, &
     &4.7929e-04_rb,3.0767e-04_rb,2.4695e-05_rb/)
      kao(:, 2,13, 2) = (/ &
     &1.0734e-03_rb,1.0865e-03_rb,1.0094e-03_rb,9.0484e-04_rb,7.8346e-04_rb,6.4723e-04_rb, &
     &4.9815e-04_rb,3.1985e-04_rb,2.6858e-05_rb/)
      kao(:, 3,13, 2) = (/ &
     &1.1391e-03_rb,1.1434e-03_rb,1.0621e-03_rb,9.5239e-04_rb,8.2130e-04_rb,6.7579e-04_rb, &
     &5.1796e-04_rb,3.3140e-04_rb,2.9385e-05_rb/)
      kao(:, 4,13, 2) = (/ &
     &1.2032e-03_rb,1.2025e-03_rb,1.1163e-03_rb,9.9786e-04_rb,8.5960e-04_rb,7.0590e-04_rb, &
     &5.3619e-04_rb,3.4228e-04_rb,3.2189e-05_rb/)
      kao(:, 5,13, 2) = (/ &
     &1.2690e-03_rb,1.2617e-03_rb,1.1676e-03_rb,1.0418e-03_rb,8.9504e-04_rb,7.3293e-04_rb, &
     &5.5106e-04_rb,3.5176e-04_rb,3.4791e-05_rb/)
      kao(:, 1, 1, 3) = (/ &
     &3.4054e-06_rb,3.6169e-05_rb,5.7781e-05_rb,7.9702e-05_rb,9.9450e-05_rb,1.1908e-04_rb, &
     &1.4021e-04_rb,1.6904e-04_rb,1.5965e-04_rb/)
      kao(:, 2, 1, 3) = (/ &
     &3.5406e-06_rb,3.7213e-05_rb,5.9148e-05_rb,7.9930e-05_rb,1.0281e-04_rb,1.2326e-04_rb, &
     &1.4516e-04_rb,1.7473e-04_rb,1.6518e-04_rb/)
      kao(:, 3, 1, 3) = (/ &
     &3.6962e-06_rb,3.8392e-05_rb,6.0628e-05_rb,8.1614e-05_rb,1.0332e-04_rb,1.2805e-04_rb, &
     &1.5083e-04_rb,1.8127e-04_rb,1.7179e-04_rb/)
      kao(:, 4, 1, 3) = (/ &
     &3.8150e-06_rb,3.9649e-05_rb,6.2587e-05_rb,8.4276e-05_rb,1.0580e-04_rb,1.2950e-04_rb, &
     &1.5730e-04_rb,1.8884e-04_rb,1.7648e-04_rb/)
      kao(:, 5, 1, 3) = (/ &
     &3.9425e-06_rb,4.0895e-05_rb,6.4628e-05_rb,8.7067e-05_rb,1.0971e-04_rb,1.3287e-04_rb, &
     &1.5979e-04_rb,1.9707e-04_rb,1.8056e-04_rb/)
      kao(:, 1, 2, 3) = (/ &
     &4.7659e-06_rb,3.7672e-05_rb,5.6772e-05_rb,7.4964e-05_rb,9.3354e-05_rb,1.1163e-04_rb, &
     &1.2805e-04_rb,1.4897e-04_rb,1.4118e-04_rb/)
      kao(:, 2, 2, 3) = (/ &
     &4.9285e-06_rb,3.8459e-05_rb,5.8331e-05_rb,7.6310e-05_rb,9.3490e-05_rb,1.1292e-04_rb, &
     &1.3218e-04_rb,1.5402e-04_rb,1.4609e-04_rb/)
      kao(:, 3, 2, 3) = (/ &
     &5.1241e-06_rb,3.9569e-05_rb,6.0060e-05_rb,7.8269e-05_rb,9.6361e-05_rb,1.1403e-04_rb, &
     &1.3730e-04_rb,1.5986e-04_rb,1.4966e-04_rb/)
      kao(:, 4, 2, 3) = (/ &
     &5.2863e-06_rb,4.0536e-05_rb,6.2133e-05_rb,8.0760e-05_rb,9.8821e-05_rb,1.1724e-04_rb, &
     &1.3737e-04_rb,1.6656e-04_rb,1.5208e-04_rb/)
      kao(:, 5, 2, 3) = (/ &
     &5.4310e-06_rb,4.1540e-05_rb,6.3967e-05_rb,8.3433e-05_rb,1.0178e-04_rb,1.2080e-04_rb, &
     &1.4142e-04_rb,1.7063e-04_rb,1.5631e-04_rb/)
      kao(:, 1, 3, 3) = (/ &
     &8.9186e-06_rb,4.5407e-05_rb,6.3526e-05_rb,7.8550e-05_rb,9.1618e-05_rb,1.0496e-04_rb, &
     &1.2158e-04_rb,1.3284e-04_rb,1.2340e-04_rb/)
      kao(:, 2, 3, 3) = (/ &
     &9.2443e-06_rb,4.6508e-05_rb,6.4784e-05_rb,8.0506e-05_rb,9.3613e-05_rb,1.0642e-04_rb, &
     &1.2005e-04_rb,1.3711e-04_rb,1.2632e-04_rb/)
      kao(:, 3, 3, 3) = (/ &
     &9.5550e-06_rb,4.7677e-05_rb,6.6548e-05_rb,8.2736e-05_rb,9.6188e-05_rb,1.0877e-04_rb, &
     &1.2156e-04_rb,1.4196e-04_rb,1.2696e-04_rb/)
      kao(:, 4, 3, 3) = (/ &
     &9.9051e-06_rb,4.8778e-05_rb,6.8112e-05_rb,8.4988e-05_rb,9.9246e-05_rb,1.1223e-04_rb, &
     &1.2463e-04_rb,1.4159e-04_rb,1.2947e-04_rb/)
      kao(:, 5, 3, 3) = (/ &
     &1.0162e-05_rb,4.9942e-05_rb,6.9999e-05_rb,8.7421e-05_rb,1.0238e-04_rb,1.1576e-04_rb, &
     &1.2839e-04_rb,1.4461e-04_rb,1.3351e-04_rb/)
      kao(:, 1, 4, 3) = (/ &
     &1.6753e-05_rb,5.8237e-05_rb,7.6272e-05_rb,8.8204e-05_rb,9.8762e-05_rb,1.0698e-04_rb, &
     &1.1349e-04_rb,1.2379e-04_rb,1.0697e-04_rb/)
      kao(:, 2, 4, 3) = (/ &
     &1.7441e-05_rb,5.9610e-05_rb,7.7972e-05_rb,9.0358e-05_rb,1.0097e-04_rb,1.0919e-04_rb, &
     &1.1491e-04_rb,1.2294e-04_rb,1.0628e-04_rb/)
      kao(:, 3, 4, 3) = (/ &
     &1.8082e-05_rb,6.1175e-05_rb,7.9988e-05_rb,9.2267e-05_rb,1.0332e-04_rb,1.1211e-04_rb, &
     &1.1776e-04_rb,1.2346e-04_rb,1.0729e-04_rb/)
      kao(:, 4, 4, 3) = (/ &
     &1.8658e-05_rb,6.2685e-05_rb,8.1816e-05_rb,9.5481e-05_rb,1.0615e-04_rb,1.1496e-04_rb, &
     &1.2097e-04_rb,1.2608e-04_rb,1.1024e-04_rb/)
      kao(:, 5, 4, 3) = (/ &
     &1.9340e-05_rb,6.4673e-05_rb,8.3970e-05_rb,9.8481e-05_rb,1.0873e-04_rb,1.1832e-04_rb, &
     &1.2499e-04_rb,1.2979e-04_rb,1.1362e-04_rb/)
      kao(:, 1, 5, 3) = (/ &
     &2.8523e-05_rb,7.5471e-05_rb,9.2112e-05_rb,1.0383e-04_rb,1.0948e-04_rb,1.1290e-04_rb, &
     &1.1403e-04_rb,1.1277e-04_rb,8.7972e-05_rb/)
      kao(:, 2, 5, 3) = (/ &
     &2.9775e-05_rb,7.7638e-05_rb,9.4306e-05_rb,1.0580e-04_rb,1.1226e-04_rb,1.1540e-04_rb, &
     &1.1664e-04_rb,1.1378e-04_rb,8.7781e-05_rb/)
      kao(:, 3, 5, 3) = (/ &
     &3.1032e-05_rb,7.9995e-05_rb,9.6878e-05_rb,1.0807e-04_rb,1.1530e-04_rb,1.1833e-04_rb, &
     &1.1968e-04_rb,1.1643e-04_rb,8.9738e-05_rb/)
      kao(:, 4, 5, 3) = (/ &
     &3.2122e-05_rb,8.2389e-05_rb,9.9387e-05_rb,1.1070e-04_rb,1.1849e-04_rb,1.2073e-04_rb, &
     &1.2266e-04_rb,1.1931e-04_rb,9.1761e-05_rb/)
      kao(:, 5, 5, 3) = (/ &
     &3.3412e-05_rb,8.5294e-05_rb,1.0214e-04_rb,1.1395e-04_rb,1.2220e-04_rb,1.2513e-04_rb, &
     &1.2589e-04_rb,1.2317e-04_rb,9.4332e-05_rb/)
      kao(:, 1, 6, 3) = (/ &
     &4.4933e-05_rb,9.9737e-05_rb,1.1255e-04_rb,1.2118e-04_rb,1.2554e-04_rb,1.2364e-04_rb, &
     &1.1791e-04_rb,1.0852e-04_rb,7.4135e-05_rb/)
      kao(:, 2, 6, 3) = (/ &
     &4.7071e-05_rb,1.0236e-04_rb,1.1567e-04_rb,1.2397e-04_rb,1.2766e-04_rb,1.2658e-04_rb, &
     &1.2060e-04_rb,1.1056e-04_rb,7.5435e-05_rb/)
      kao(:, 3, 6, 3) = (/ &
     &4.9151e-05_rb,1.0514e-04_rb,1.1921e-04_rb,1.2711e-04_rb,1.3040e-04_rb,1.2992e-04_rb, &
     &1.2331e-04_rb,1.1371e-04_rb,7.6849e-05_rb/)
      kao(:, 4, 6, 3) = (/ &
     &5.1024e-05_rb,1.0807e-04_rb,1.2306e-04_rb,1.3034e-04_rb,1.3370e-04_rb,1.3344e-04_rb, &
     &1.2682e-04_rb,1.1612e-04_rb,7.8630e-05_rb/)
      kao(:, 5, 6, 3) = (/ &
     &5.2854e-05_rb,1.1144e-04_rb,1.2670e-04_rb,1.3361e-04_rb,1.3751e-04_rb,1.3677e-04_rb, &
     &1.3113e-04_rb,1.1903e-04_rb,8.1063e-05_rb/)
      kao(:, 1, 7, 3) = (/ &
     &7.4131e-05_rb,1.3771e-04_rb,1.5135e-04_rb,1.5339e-04_rb,1.5199e-04_rb,1.4542e-04_rb, &
     &1.3212e-04_rb,1.1157e-04_rb,6.3305e-05_rb/)
      kao(:, 2, 7, 3) = (/ &
     &7.8122e-05_rb,1.4080e-04_rb,1.5542e-04_rb,1.5789e-04_rb,1.5525e-04_rb,1.4808e-04_rb, &
     &1.3571e-04_rb,1.1400e-04_rb,6.4305e-05_rb/)
      kao(:, 3, 7, 3) = (/ &
     &8.1792e-05_rb,1.4429e-04_rb,1.5980e-04_rb,1.6259e-04_rb,1.5930e-04_rb,1.5173e-04_rb, &
     &1.3906e-04_rb,1.1682e-04_rb,6.5778e-05_rb/)
      kao(:, 4, 7, 3) = (/ &
     &8.5102e-05_rb,1.4854e-04_rb,1.6447e-04_rb,1.6719e-04_rb,1.6365e-04_rb,1.5623e-04_rb, &
     &1.4249e-04_rb,1.1910e-04_rb,6.8216e-05_rb/)
      kao(:, 5, 7, 3) = (/ &
     &8.8343e-05_rb,1.5302e-04_rb,1.6961e-04_rb,1.7246e-04_rb,1.6859e-04_rb,1.6045e-04_rb, &
     &1.4567e-04_rb,1.2333e-04_rb,7.0812e-05_rb/)
      kao(:, 1, 8, 3) = (/ &
     &1.4407e-04_rb,2.1929e-04_rb,2.3408e-04_rb,2.3175e-04_rb,2.1747e-04_rb,1.9684e-04_rb, &
     &1.7181e-04_rb,1.3227e-04_rb,5.5189e-05_rb/)
      kao(:, 2, 8, 3) = (/ &
     &1.5218e-04_rb,2.2535e-04_rb,2.3947e-04_rb,2.3742e-04_rb,2.2354e-04_rb,2.0175e-04_rb, &
     &1.7481e-04_rb,1.3567e-04_rb,5.7508e-05_rb/)
      kao(:, 3, 8, 3) = (/ &
     &1.5981e-04_rb,2.3168e-04_rb,2.4529e-04_rb,2.4373e-04_rb,2.2990e-04_rb,2.0756e-04_rb, &
     &1.7860e-04_rb,1.3869e-04_rb,5.9914e-05_rb/)
      kao(:, 4, 8, 3) = (/ &
     &1.6710e-04_rb,2.3880e-04_rb,2.5211e-04_rb,2.5020e-04_rb,2.3684e-04_rb,2.1373e-04_rb, &
     &1.8326e-04_rb,1.4183e-04_rb,6.3164e-05_rb/)
      kao(:, 5, 8, 3) = (/ &
     &1.7380e-04_rb,2.4599e-04_rb,2.5929e-04_rb,2.5766e-04_rb,2.4400e-04_rb,2.1994e-04_rb, &
     &1.8763e-04_rb,1.4519e-04_rb,6.6105e-05_rb/)
      kao(:, 1, 9, 3) = (/ &
     &5.2627e-04_rb,6.1534e-04_rb,6.0941e-04_rb,5.7646e-04_rb,5.2415e-04_rb,4.5407e-04_rb, &
     &3.6135e-04_rb,2.4279e-04_rb,6.7993e-05_rb/)
      kao(:, 2, 9, 3) = (/ &
     &5.5864e-04_rb,6.4193e-04_rb,6.3124e-04_rb,5.9396e-04_rb,5.3793e-04_rb,4.6535e-04_rb, &
     &3.7115e-04_rb,2.4862e-04_rb,7.5217e-05_rb/)
      kao(:, 3, 9, 3) = (/ &
     &5.8842e-04_rb,6.6676e-04_rb,6.5277e-04_rb,6.1271e-04_rb,5.5239e-04_rb,4.7636e-04_rb, &
     &3.8150e-04_rb,2.5543e-04_rb,8.1799e-05_rb/)
      kao(:, 4, 9, 3) = (/ &
     &6.1785e-04_rb,6.9166e-04_rb,6.7437e-04_rb,6.3091e-04_rb,5.6815e-04_rb,4.8941e-04_rb, &
     &3.9213e-04_rb,2.6284e-04_rb,8.7598e-05_rb/)
      kao(:, 5, 9, 3) = (/ &
     &6.4520e-04_rb,7.1602e-04_rb,6.9663e-04_rb,6.4913e-04_rb,5.8418e-04_rb,5.0317e-04_rb, &
     &4.0334e-04_rb,2.7060e-04_rb,9.2801e-05_rb/)
      kao(:, 1,10, 3) = (/ &
     &2.1645e-03_rb,2.1618e-03_rb,2.0014e-03_rb,1.7923e-03_rb,1.5495e-03_rb,1.2769e-03_rb, &
     &9.6609e-04_rb,6.0453e-04_rb,1.0410e-04_rb/)
      kao(:, 2,10, 3) = (/ &
     &2.3095e-03_rb,2.2836e-03_rb,2.1017e-03_rb,1.8749e-03_rb,1.6151e-03_rb,1.3240e-03_rb, &
     &9.9711e-04_rb,6.2160e-04_rb,1.1408e-04_rb/)
      kao(:, 3,10, 3) = (/ &
     &2.4464e-03_rb,2.3906e-03_rb,2.1944e-03_rb,1.9559e-03_rb,1.6773e-03_rb,1.3715e-03_rb, &
     &1.0290e-03_rb,6.3832e-04_rb,1.2224e-04_rb/)
      kao(:, 4,10, 3) = (/ &
     &2.5757e-03_rb,2.5009e-03_rb,2.2880e-03_rb,2.0317e-03_rb,1.7417e-03_rb,1.4189e-03_rb, &
     &1.0630e-03_rb,6.5627e-04_rb,1.3004e-04_rb/)
      kao(:, 5,10, 3) = (/ &
     &2.6938e-03_rb,2.6117e-03_rb,2.3825e-03_rb,2.1099e-03_rb,1.8067e-03_rb,1.4685e-03_rb, &
     &1.0955e-03_rb,6.7426e-04_rb,1.3892e-04_rb/)
      kao(:, 1,11, 3) = (/ &
     &3.2925e-03_rb,3.1585e-03_rb,2.8624e-03_rb,2.5152e-03_rb,2.1339e-03_rb,1.7245e-03_rb, &
     &1.2749e-03_rb,7.7353e-04_rb,1.1570e-04_rb/)
      kao(:, 2,11, 3) = (/ &
     &3.4939e-03_rb,3.3275e-03_rb,2.9995e-03_rb,2.6330e-03_rb,2.2285e-03_rb,1.7928e-03_rb, &
     &1.3221e-03_rb,7.9717e-04_rb,1.2609e-04_rb/)
      kao(:, 3,11, 3) = (/ &
     &3.6814e-03_rb,3.4852e-03_rb,3.1333e-03_rb,2.7438e-03_rb,2.3219e-03_rb,1.8667e-03_rb, &
     &1.3711e-03_rb,8.2069e-04_rb,1.3849e-04_rb/)
      kao(:, 4,11, 3) = (/ &
     &3.8717e-03_rb,3.6472e-03_rb,3.2720e-03_rb,2.8608e-03_rb,2.4167e-03_rb,1.9357e-03_rb, &
     &1.4194e-03_rb,8.4717e-04_rb,1.4796e-04_rb/)
      kao(:, 5,11, 3) = (/ &
     &4.0507e-03_rb,3.8076e-03_rb,3.4109e-03_rb,2.9754e-03_rb,2.5103e-03_rb,2.0111e-03_rb, &
     &1.4691e-03_rb,8.7324e-04_rb,1.5498e-04_rb/)
      kao(:, 1,12, 3) = (/ &
     &3.6937e-03_rb,3.5033e-03_rb,3.1460e-03_rb,2.7421e-03_rb,2.3088e-03_rb,1.8436e-03_rb, &
     &1.3493e-03_rb,8.0094e-04_rb,1.1298e-04_rb/)
      kao(:, 2,12, 3) = (/ &
     &3.9154e-03_rb,3.6887e-03_rb,3.2999e-03_rb,2.8717e-03_rb,2.4149e-03_rb,1.9245e-03_rb, &
     &1.4017e-03_rb,8.2771e-04_rb,1.2419e-04_rb/)
      kao(:, 3,12, 3) = (/ &
     &4.1285e-03_rb,3.8698e-03_rb,3.4585e-03_rb,3.0012e-03_rb,2.5199e-03_rb,2.0022e-03_rb, &
     &1.4553e-03_rb,8.5423e-04_rb,1.3225e-04_rb/)
      kao(:, 4,12, 3) = (/ &
     &4.3384e-03_rb,4.0509e-03_rb,3.6103e-03_rb,3.1297e-03_rb,2.6243e-03_rb,2.0861e-03_rb, &
     &1.5110e-03_rb,8.8306e-04_rb,1.4212e-04_rb/)
      kao(:, 5,12, 3) = (/ &
     &4.5406e-03_rb,4.2341e-03_rb,3.7730e-03_rb,3.2693e-03_rb,2.7384e-03_rb,2.1705e-03_rb, &
     &1.5683e-03_rb,9.1255e-04_rb,1.5213e-04_rb/)
      kao(:, 1,13, 3) = (/ &
     &3.2602e-03_rb,3.0914e-03_rb,2.7797e-03_rb,2.4220e-03_rb,2.0389e-03_rb,1.6288e-03_rb, &
     &1.1889e-03_rb,7.0223e-04_rb,9.9450e-05_rb/)
      kao(:, 2,13, 3) = (/ &
     &3.4542e-03_rb,3.2567e-03_rb,2.9218e-03_rb,2.5424e-03_rb,2.1356e-03_rb,1.7009e-03_rb, &
     &1.2356e-03_rb,7.2786e-04_rb,1.0781e-04_rb/)
      kao(:, 3,13, 3) = (/ &
     &3.6454e-03_rb,3.4269e-03_rb,3.0649e-03_rb,2.6592e-03_rb,2.2314e-03_rb,1.7744e-03_rb, &
     &1.2855e-03_rb,7.5225e-04_rb,1.1667e-04_rb/)
      kao(:, 4,13, 3) = (/ &
     &3.8341e-03_rb,3.5875e-03_rb,3.2022e-03_rb,2.7784e-03_rb,2.3283e-03_rb,1.8470e-03_rb, &
     &1.3345e-03_rb,7.7860e-04_rb,1.2556e-04_rb/)
      kao(:, 5,13, 3) = (/ &
     &4.0097e-03_rb,3.7448e-03_rb,3.3397e-03_rb,2.8965e-03_rb,2.4238e-03_rb,1.9205e-03_rb, &
     &1.3873e-03_rb,8.0728e-04_rb,1.3156e-04_rb/)
      kao(:, 1, 1, 4) = (/ &
     &7.6190e-06_rb,7.2707e-05_rb,1.2159e-04_rb,1.6797e-04_rb,2.1749e-04_rb,2.7232e-04_rb, &
     &3.3756e-04_rb,4.5000e-04_rb,3.7863e-04_rb/)
      kao(:, 2, 1, 4) = (/ &
     &7.7938e-06_rb,7.4481e-05_rb,1.2500e-04_rb,1.7416e-04_rb,2.2412e-04_rb,2.7984e-04_rb, &
     &3.4630e-04_rb,4.4903e-04_rb,3.8823e-04_rb/)
      kao(:, 3, 1, 4) = (/ &
     &8.0028e-06_rb,7.6115e-05_rb,1.2829e-04_rb,1.8057e-04_rb,2.3237e-04_rb,2.8607e-04_rb, &
     &3.5506e-04_rb,4.5741e-04_rb,3.9983e-04_rb/)
      kao(:, 4, 1, 4) = (/ &
     &8.1270e-06_rb,7.7997e-05_rb,1.3160e-04_rb,1.8544e-04_rb,2.4027e-04_rb,2.9656e-04_rb, &
     &3.6016e-04_rb,4.6576e-04_rb,4.1176e-04_rb/)
      kao(:, 5, 1, 4) = (/ &
     &8.2916e-06_rb,8.0436e-05_rb,1.3585e-04_rb,1.9127e-04_rb,2.4691e-04_rb,3.0708e-04_rb, &
     &3.7489e-04_rb,4.7422e-04_rb,4.3091e-04_rb/)
      kao(:, 1, 2, 4) = (/ &
     &1.0434e-05_rb,7.3238e-05_rb,1.1625e-04_rb,1.5526e-04_rb,1.9383e-04_rb,2.3419e-04_rb, &
     &2.8405e-04_rb,3.6155e-04_rb,3.1532e-04_rb/)
      kao(:, 2, 2, 4) = (/ &
     &1.0681e-05_rb,7.5036e-05_rb,1.1854e-04_rb,1.5997e-04_rb,2.0074e-04_rb,2.4144e-04_rb, &
     &2.9129e-04_rb,3.7038e-04_rb,3.2501e-04_rb/)
      kao(:, 3, 2, 4) = (/ &
     &1.0988e-05_rb,7.7182e-05_rb,1.2126e-04_rb,1.6374e-04_rb,2.0655e-04_rb,2.5052e-04_rb, &
     &2.9566e-04_rb,3.7520e-04_rb,3.3554e-04_rb/)
      kao(:, 4, 2, 4) = (/ &
     &1.1243e-05_rb,7.9396e-05_rb,1.2452e-04_rb,1.6827e-04_rb,2.1213e-04_rb,2.5807e-04_rb, &
     &3.0951e-04_rb,3.7973e-04_rb,3.4985e-04_rb/)
      kao(:, 5, 2, 4) = (/ &
     &1.1433e-05_rb,8.1733e-05_rb,1.2847e-04_rb,1.7400e-04_rb,2.1976e-04_rb,2.6675e-04_rb, &
     &3.1873e-04_rb,3.9060e-04_rb,3.6531e-04_rb/)
      kao(:, 1, 3, 4) = (/ &
     &1.9121e-05_rb,8.5930e-05_rb,1.2401e-04_rb,1.5771e-04_rb,1.8803e-04_rb,2.1587e-04_rb, &
     &2.4191e-04_rb,2.8659e-04_rb,2.6000e-04_rb/)
      kao(:, 2, 3, 4) = (/ &
     &1.9555e-05_rb,8.7968e-05_rb,1.2724e-04_rb,1.6033e-04_rb,1.9279e-04_rb,2.2305e-04_rb, &
     &2.5217e-04_rb,2.9281e-04_rb,2.6982e-04_rb/)
      kao(:, 3, 3, 4) = (/ &
     &2.0014e-05_rb,9.0319e-05_rb,1.3030e-04_rb,1.6432e-04_rb,1.9658e-04_rb,2.2814e-04_rb, &
     &2.6130e-04_rb,2.9767e-04_rb,2.8111e-04_rb/)
      kao(:, 4, 3, 4) = (/ &
     &2.0629e-05_rb,9.3145e-05_rb,1.3414e-04_rb,1.6939e-04_rb,2.0212e-04_rb,2.3402e-04_rb, &
     &2.6796e-04_rb,3.0992e-04_rb,2.9328e-04_rb/)
      kao(:, 5, 3, 4) = (/ &
     &2.0980e-05_rb,9.5719e-05_rb,1.3796e-04_rb,1.7464e-04_rb,2.0857e-04_rb,2.4205e-04_rb, &
     &2.7670e-04_rb,3.1994e-04_rb,3.0523e-04_rb/)
      kao(:, 1, 4, 4) = (/ &
     &3.5480e-05_rb,1.1130e-04_rb,1.4459e-04_rb,1.7323e-04_rb,1.9468e-04_rb,2.1444e-04_rb, &
     &2.2966e-04_rb,2.4128e-04_rb,2.1617e-04_rb/)
      kao(:, 2, 4, 4) = (/ &
     &3.6323e-05_rb,1.1358e-04_rb,1.4824e-04_rb,1.7664e-04_rb,1.9952e-04_rb,2.1840e-04_rb, &
     &2.3716e-04_rb,2.5075e-04_rb,2.2864e-04_rb/)
      kao(:, 3, 4, 4) = (/ &
     &3.7216e-05_rb,1.1624e-04_rb,1.5219e-04_rb,1.8108e-04_rb,2.0484e-04_rb,2.2378e-04_rb, &
     &2.4107e-04_rb,2.5972e-04_rb,2.3687e-04_rb/)
      kao(:, 4, 4, 4) = (/ &
     &3.8132e-05_rb,1.1926e-04_rb,1.5631e-04_rb,1.8514e-04_rb,2.0992e-04_rb,2.3030e-04_rb, &
     &2.4787e-04_rb,2.6623e-04_rb,2.4575e-04_rb/)
      kao(:, 5, 4, 4) = (/ &
     &3.9332e-05_rb,1.2292e-04_rb,1.6148e-04_rb,1.9104e-04_rb,2.1696e-04_rb,2.3764e-04_rb, &
     &2.5623e-04_rb,2.7468e-04_rb,2.5520e-04_rb/)
      kao(:, 1, 5, 4) = (/ &
     &6.0558e-05_rb,1.4537e-04_rb,1.7679e-04_rb,1.9677e-04_rb,2.1242e-04_rb,2.2290e-04_rb, &
     &2.2712e-04_rb,2.2468e-04_rb,1.8402e-04_rb/)
      kao(:, 2, 5, 4) = (/ &
     &6.2024e-05_rb,1.4763e-04_rb,1.8049e-04_rb,2.0169e-04_rb,2.1740e-04_rb,2.2816e-04_rb, &
     &2.3141e-04_rb,2.3210e-04_rb,1.9363e-04_rb/)
      kao(:, 3, 5, 4) = (/ &
     &6.3560e-05_rb,1.5048e-04_rb,1.8446e-04_rb,2.0674e-04_rb,2.2245e-04_rb,2.3297e-04_rb, &
     &2.3732e-04_rb,2.3625e-04_rb,1.9953e-04_rb/)
      kao(:, 4, 5, 4) = (/ &
     &6.5197e-05_rb,1.5389e-04_rb,1.8956e-04_rb,2.1242e-04_rb,2.2805e-04_rb,2.3893e-04_rb, &
     &2.4394e-04_rb,2.4242e-04_rb,2.0731e-04_rb/)
      kao(:, 5, 5, 4) = (/ &
     &6.7408e-05_rb,1.5860e-04_rb,1.9588e-04_rb,2.1916e-04_rb,2.3481e-04_rb,2.4604e-04_rb, &
     &2.5157e-04_rb,2.5058e-04_rb,2.1479e-04_rb/)
      kao(:, 1, 6, 4) = (/ &
     &9.6086e-05_rb,1.8511e-04_rb,2.1787e-04_rb,2.3286e-04_rb,2.3898e-04_rb,2.3907e-04_rb, &
     &2.3360e-04_rb,2.1667e-04_rb,1.5583e-04_rb/)
      kao(:, 2, 6, 4) = (/ &
     &9.8537e-05_rb,1.8825e-04_rb,2.2157e-04_rb,2.3776e-04_rb,2.4456e-04_rb,2.4422e-04_rb, &
     &2.3850e-04_rb,2.2088e-04_rb,1.6156e-04_rb/)
      kao(:, 3, 6, 4) = (/ &
     &1.0106e-04_rb,1.9192e-04_rb,2.2599e-04_rb,2.4309e-04_rb,2.5072e-04_rb,2.5018e-04_rb, &
     &2.4346e-04_rb,2.2608e-04_rb,1.6736e-04_rb/)
      kao(:, 4, 6, 4) = (/ &
     &1.0368e-04_rb,1.9610e-04_rb,2.3144e-04_rb,2.4948e-04_rb,2.5710e-04_rb,2.5653e-04_rb, &
     &2.4933e-04_rb,2.3314e-04_rb,1.7404e-04_rb/)
      kao(:, 5, 6, 4) = (/ &
     &1.0646e-04_rb,2.0089e-04_rb,2.3770e-04_rb,2.5701e-04_rb,2.6421e-04_rb,2.6433e-04_rb, &
     &2.5582e-04_rb,2.4028e-04_rb,1.7876e-04_rb/)
      kao(:, 1, 7, 4) = (/ &
     &1.6042e-04_rb,2.5585e-04_rb,2.8519e-04_rb,2.9642e-04_rb,2.9350e-04_rb,2.7913e-04_rb, &
     &2.5601e-04_rb,2.2211e-04_rb,1.3626e-04_rb/)
      kao(:, 2, 7, 4) = (/ &
     &1.6465e-04_rb,2.6056e-04_rb,2.8988e-04_rb,3.0133e-04_rb,2.9907e-04_rb,2.8588e-04_rb, &
     &2.6187e-04_rb,2.2729e-04_rb,1.4135e-04_rb/)
      kao(:, 3, 7, 4) = (/ &
     &1.6901e-04_rb,2.6578e-04_rb,2.9572e-04_rb,3.0741e-04_rb,3.0551e-04_rb,2.9184e-04_rb, &
     &2.6817e-04_rb,2.3205e-04_rb,1.4656e-04_rb/)
      kao(:, 4, 7, 4) = (/ &
     &1.7367e-04_rb,2.7110e-04_rb,3.0239e-04_rb,3.1497e-04_rb,3.1315e-04_rb,2.9895e-04_rb, &
     &2.7506e-04_rb,2.3781e-04_rb,1.5059e-04_rb/)
      kao(:, 5, 7, 4) = (/ &
     &1.7865e-04_rb,2.7756e-04_rb,3.1008e-04_rb,3.2315e-04_rb,3.2186e-04_rb,3.0753e-04_rb, &
     &2.8375e-04_rb,2.4404e-04_rb,1.5550e-04_rb/)
      kao(:, 1, 8, 4) = (/ &
     &3.1654e-04_rb,4.1629e-04_rb,4.3839e-04_rb,4.3773e-04_rb,4.1826e-04_rb,3.8213e-04_rb, &
     &3.2793e-04_rb,2.5576e-04_rb,1.2647e-04_rb/)
      kao(:, 2, 8, 4) = (/ &
     &3.2537e-04_rb,4.2537e-04_rb,4.4661e-04_rb,4.4494e-04_rb,4.2500e-04_rb,3.8879e-04_rb, &
     &3.3490e-04_rb,2.6206e-04_rb,1.2938e-04_rb/)
      kao(:, 3, 8, 4) = (/ &
     &3.3408e-04_rb,4.3504e-04_rb,4.5626e-04_rb,4.5382e-04_rb,4.3360e-04_rb,3.9751e-04_rb, &
     &3.4272e-04_rb,2.6854e-04_rb,1.3239e-04_rb/)
      kao(:, 4, 8, 4) = (/ &
     &3.4318e-04_rb,4.4480e-04_rb,4.6645e-04_rb,4.6411e-04_rb,4.4358e-04_rb,4.0695e-04_rb, &
     &3.5185e-04_rb,2.7598e-04_rb,1.3521e-04_rb/)
      kao(:, 5, 8, 4) = (/ &
     &3.5364e-04_rb,4.5591e-04_rb,4.7748e-04_rb,4.7551e-04_rb,4.5523e-04_rb,4.1803e-04_rb, &
     &3.6238e-04_rb,2.8454e-04_rb,1.3901e-04_rb/)
      kao(:, 1, 9, 4) = (/ &
     &1.1778e-03_rb,1.2542e-03_rb,1.1993e-03_rb,1.1088e-03_rb,9.9621e-04_rb,8.5814e-04_rb, &
     &6.9027e-04_rb,4.7338e-04_rb,1.4222e-04_rb/)
      kao(:, 2, 9, 4) = (/ &
     &1.2148e-03_rb,1.2865e-03_rb,1.2285e-03_rb,1.1318e-03_rb,1.0140e-03_rb,8.7230e-04_rb, &
     &7.0194e-04_rb,4.8264e-04_rb,1.4691e-04_rb/)
      kao(:, 3, 9, 4) = (/ &
     &1.2493e-03_rb,1.3197e-03_rb,1.2603e-03_rb,1.1590e-03_rb,1.0363e-03_rb,8.9016e-04_rb, &
     &7.1481e-04_rb,4.9330e-04_rb,1.5194e-04_rb/)
      kao(:, 4, 9, 4) = (/ &
     &1.2842e-03_rb,1.3516e-03_rb,1.2898e-03_rb,1.1881e-03_rb,1.0607e-03_rb,9.1030e-04_rb, &
     &7.3056e-04_rb,5.0488e-04_rb,1.5874e-04_rb/)
      kao(:, 5, 9, 4) = (/ &
     &1.3233e-03_rb,1.3878e-03_rb,1.3225e-03_rb,1.2203e-03_rb,1.0882e-03_rb,9.3354e-04_rb, &
     &7.4949e-04_rb,5.1819e-04_rb,1.6715e-04_rb/)
      kao(:, 1,10, 4) = (/ &
     &4.9223e-03_rb,4.6924e-03_rb,4.2372e-03_rb,3.7210e-03_rb,3.1598e-03_rb,2.5466e-03_rb, &
     &1.8894e-03_rb,1.1610e-03_rb,2.5902e-04_rb/)
      kao(:, 2,10, 4) = (/ &
     &5.1019e-03_rb,4.8420e-03_rb,4.3700e-03_rb,3.8242e-03_rb,3.2382e-03_rb,2.6075e-03_rb, &
     &1.9274e-03_rb,1.1773e-03_rb,2.7993e-04_rb/)
      kao(:, 3,10, 4) = (/ &
     &5.2624e-03_rb,4.9912e-03_rb,4.4976e-03_rb,3.9326e-03_rb,3.3295e-03_rb,2.6789e-03_rb, &
     &1.9739e-03_rb,1.2013e-03_rb,2.9275e-04_rb/)
      kao(:, 4,10, 4) = (/ &
     &5.4202e-03_rb,5.1325e-03_rb,4.6211e-03_rb,4.0415e-03_rb,3.4179e-03_rb,2.7474e-03_rb, &
     &2.0231e-03_rb,1.2303e-03_rb,3.0514e-04_rb/)
      kao(:, 5,10, 4) = (/ &
     &5.5922e-03_rb,5.2716e-03_rb,4.7430e-03_rb,4.1489e-03_rb,3.5047e-03_rb,2.8180e-03_rb, &
     &2.0794e-03_rb,1.2643e-03_rb,3.1858e-04_rb/)
      kao(:, 1,11, 4) = (/ &
     &7.2623e-03_rb,6.7979e-03_rb,6.0744e-03_rb,5.2843e-03_rb,4.4298e-03_rb,3.5303e-03_rb, &
     &2.5670e-03_rb,1.5127e-03_rb,3.1480e-04_rb/)
      kao(:, 2,11, 4) = (/ &
     &7.5607e-03_rb,7.0506e-03_rb,6.2894e-03_rb,5.4629e-03_rb,4.5682e-03_rb,3.6306e-03_rb, &
     &2.6328e-03_rb,1.5436e-03_rb,3.2537e-04_rb/)
      kao(:, 3,11, 4) = (/ &
     &7.8471e-03_rb,7.2956e-03_rb,6.4988e-03_rb,5.6385e-03_rb,4.7125e-03_rb,3.7379e-03_rb, &
     &2.7029e-03_rb,1.5776e-03_rb,3.3231e-04_rb/)
      kao(:, 4,11, 4) = (/ &
     &8.1128e-03_rb,7.5290e-03_rb,6.7020e-03_rb,5.8093e-03_rb,4.8535e-03_rb,3.8422e-03_rb, &
     &2.7753e-03_rb,1.6189e-03_rb,3.4203e-04_rb/)
      kao(:, 5,11, 4) = (/ &
     &8.3759e-03_rb,7.7545e-03_rb,6.8899e-03_rb,5.9710e-03_rb,4.9875e-03_rb,3.9509e-03_rb, &
     &2.8575e-03_rb,1.6684e-03_rb,3.5725e-04_rb/)
      kao(:, 1,12, 4) = (/ &
     &8.0096e-03_rb,7.4190e-03_rb,6.5933e-03_rb,5.7152e-03_rb,4.7757e-03_rb,3.7806e-03_rb, &
     &2.7237e-03_rb,1.5850e-03_rb,3.0906e-04_rb/)
      kao(:, 2,12, 4) = (/ &
     &8.3245e-03_rb,7.6924e-03_rb,6.8377e-03_rb,5.9147e-03_rb,4.9319e-03_rb,3.8997e-03_rb, &
     &2.8053e-03_rb,1.6213e-03_rb,3.1505e-04_rb/)
      kao(:, 3,12, 4) = (/ &
     &8.6504e-03_rb,7.9748e-03_rb,7.0681e-03_rb,6.1123e-03_rb,5.0902e-03_rb,4.0202e-03_rb, &
     &2.8850e-03_rb,1.6649e-03_rb,3.2236e-04_rb/)
      kao(:, 4,12, 4) = (/ &
     &8.9354e-03_rb,8.2303e-03_rb,7.2913e-03_rb,6.2946e-03_rb,5.2436e-03_rb,4.1358e-03_rb, &
     &2.9697e-03_rb,1.7144e-03_rb,3.2906e-04_rb/)
      kao(:, 5,12, 4) = (/ &
     &9.2169e-03_rb,8.4842e-03_rb,7.5093e-03_rb,6.4790e-03_rb,5.3967e-03_rb,4.2618e-03_rb, &
     &3.0601e-03_rb,1.7687e-03_rb,3.3757e-04_rb/)
      kao(:, 1,13, 4) = (/ &
     &7.1631e-03_rb,6.6319e-03_rb,5.8920e-03_rb,5.0960e-03_rb,4.2500e-03_rb,3.3594e-03_rb, &
     &2.4169e-03_rb,1.3985e-03_rb,2.7237e-04_rb/)
      kao(:, 2,13, 4) = (/ &
     &7.4650e-03_rb,6.8721e-03_rb,6.1020e-03_rb,5.2700e-03_rb,4.3954e-03_rb,3.4698e-03_rb, &
     &2.4897e-03_rb,1.4341e-03_rb,2.7736e-04_rb/)
      kao(:, 3,13, 4) = (/ &
     &7.7609e-03_rb,7.1282e-03_rb,6.3106e-03_rb,5.4462e-03_rb,4.5330e-03_rb,3.5736e-03_rb, &
     &2.5636e-03_rb,1.4770e-03_rb,2.8124e-04_rb/)
      kao(:, 4,13, 4) = (/ &
     &8.0101e-03_rb,7.3618e-03_rb,6.5123e-03_rb,5.6158e-03_rb,4.6733e-03_rb,3.6861e-03_rb, &
     &2.6445e-03_rb,1.5233e-03_rb,2.8716e-04_rb/)
      kao(:, 5,13, 4) = (/ &
     &8.2606e-03_rb,7.6008e-03_rb,6.7314e-03_rb,5.7990e-03_rb,4.8288e-03_rb,3.8083e-03_rb, &
     &2.7261e-03_rb,1.5712e-03_rb,2.9686e-04_rb/)
      kao(:, 1, 1, 5) = (/ &
     &1.3739e-05_rb,1.5137e-04_rb,2.5878e-04_rb,3.6225e-04_rb,4.6522e-04_rb,5.7109e-04_rb, &
     &6.9199e-04_rb,8.4168e-04_rb,8.0480e-04_rb/)
      kao(:, 2, 1, 5) = (/ &
     &1.3888e-05_rb,1.5627e-04_rb,2.6885e-04_rb,3.7698e-04_rb,4.8125e-04_rb,5.9114e-04_rb, &
     &7.1164e-04_rb,8.7936e-04_rb,8.3421e-04_rb/)
      kao(:, 3, 1, 5) = (/ &
     &1.4142e-05_rb,1.6264e-04_rb,2.8096e-04_rb,3.9281e-04_rb,5.0319e-04_rb,6.1580e-04_rb, &
     &7.3800e-04_rb,9.1340e-04_rb,8.6788e-04_rb/)
      kao(:, 4, 1, 5) = (/ &
     &1.4278e-05_rb,1.6909e-04_rb,2.9347e-04_rb,4.1125e-04_rb,5.2641e-04_rb,6.4515e-04_rb, &
     &7.7492e-04_rb,9.4989e-04_rb,9.0996e-04_rb/)
      kao(:, 5, 1, 5) = (/ &
     &1.4494e-05_rb,1.7605e-04_rb,3.0657e-04_rb,4.3067e-04_rb,5.5274e-04_rb,6.7670e-04_rb, &
     &8.1211e-04_rb,9.9682e-04_rb,9.5292e-04_rb/)
      kao(:, 1, 2, 5) = (/ &
     &1.8692e-05_rb,1.5213e-04_rb,2.4704e-04_rb,3.3748e-04_rb,4.2434e-04_rb,5.1189e-04_rb, &
     &6.0610e-04_rb,7.3668e-04_rb,6.9411e-04_rb/)
      kao(:, 2, 2, 5) = (/ &
     &1.8918e-05_rb,1.5679e-04_rb,2.5663e-04_rb,3.5044e-04_rb,4.4175e-04_rb,5.3243e-04_rb, &
     &6.2710e-04_rb,7.5567e-04_rb,7.2022e-04_rb/)
      kao(:, 3, 2, 5) = (/ &
     &1.9322e-05_rb,1.6205e-04_rb,2.6764e-04_rb,3.6642e-04_rb,4.6108e-04_rb,5.5570e-04_rb, &
     &6.5662e-04_rb,7.8535e-04_rb,7.5176e-04_rb/)
      kao(:, 4, 2, 5) = (/ &
     &1.9585e-05_rb,1.6765e-04_rb,2.7850e-04_rb,3.8286e-04_rb,4.8359e-04_rb,5.8272e-04_rb, &
     &6.8652e-04_rb,8.2260e-04_rb,7.8717e-04_rb/)
      kao(:, 5, 2, 5) = (/ &
     &1.9818e-05_rb,1.7339e-04_rb,2.8992e-04_rb,3.9889e-04_rb,5.0485e-04_rb,6.1010e-04_rb, &
     &7.2141e-04_rb,8.6363e-04_rb,8.2436e-04_rb/)
      kao(:, 1, 3, 5) = (/ &
     &3.4262e-05_rb,1.7533e-04_rb,2.6113e-04_rb,3.3571e-04_rb,4.0540e-04_rb,4.7229e-04_rb, &
     &5.3818e-04_rb,6.1661e-04_rb,5.8845e-04_rb/)
      kao(:, 2, 3, 5) = (/ &
     &3.4716e-05_rb,1.7923e-04_rb,2.6899e-04_rb,3.4836e-04_rb,4.2096e-04_rb,4.9023e-04_rb, &
     &5.6025e-04_rb,6.3703e-04_rb,6.0869e-04_rb/)
      kao(:, 3, 3, 5) = (/ &
     &3.5292e-05_rb,1.8347e-04_rb,2.7811e-04_rb,3.6147e-04_rb,4.3915e-04_rb,5.1242e-04_rb, &
     &5.8378e-04_rb,6.6557e-04_rb,6.3564e-04_rb/)
      kao(:, 4, 3, 5) = (/ &
     &3.6120e-05_rb,1.8884e-04_rb,2.8797e-04_rb,3.7545e-04_rb,4.5740e-04_rb,5.3560e-04_rb, &
     &6.1221e-04_rb,6.9822e-04_rb,6.6452e-04_rb/)
      kao(:, 5, 3, 5) = (/ &
     &3.6636e-05_rb,1.9428e-04_rb,2.9738e-04_rb,3.9013e-04_rb,4.7606e-04_rb,5.5790e-04_rb, &
     &6.3990e-04_rb,7.3219e-04_rb,6.9591e-04_rb/)
      kao(:, 1, 4, 5) = (/ &
     &6.3937e-05_rb,2.1416e-04_rb,2.9895e-04_rb,3.6305e-04_rb,4.1697e-04_rb,4.6136e-04_rb, &
     &4.9950e-04_rb,5.3752e-04_rb,4.9755e-04_rb/)
      kao(:, 2, 4, 5) = (/ &
     &6.4891e-05_rb,2.1788e-04_rb,3.0564e-04_rb,3.7373e-04_rb,4.2957e-04_rb,4.7819e-04_rb, &
     &5.1730e-04_rb,5.5769e-04_rb,5.1340e-04_rb/)
      kao(:, 3, 4, 5) = (/ &
     &6.5931e-05_rb,2.2256e-04_rb,3.1273e-04_rb,3.8551e-04_rb,4.4411e-04_rb,4.9569e-04_rb, &
     &5.4027e-04_rb,5.8134e-04_rb,5.3558e-04_rb/)
      kao(:, 4, 4, 5) = (/ &
     &6.7204e-05_rb,2.2716e-04_rb,3.2137e-04_rb,3.9776e-04_rb,4.6059e-04_rb,5.1431e-04_rb, &
     &5.6246e-04_rb,6.0834e-04_rb,5.5897e-04_rb/)
      kao(:, 5, 4, 5) = (/ &
     &6.9088e-05_rb,2.3323e-04_rb,3.3076e-04_rb,4.1010e-04_rb,4.7742e-04_rb,5.3504e-04_rb, &
     &5.8525e-04_rb,6.3600e-04_rb,5.8418e-04_rb/)
      kao(:, 1, 5, 5) = (/ &
     &1.0861e-04_rb,2.6518e-04_rb,3.4675e-04_rb,4.0531e-04_rb,4.4642e-04_rb,4.7448e-04_rb, &
     &4.8963e-04_rb,4.9041e-04_rb,4.2330e-04_rb/)
      kao(:, 2, 5, 5) = (/ &
     &1.1058e-04_rb,2.6952e-04_rb,3.5268e-04_rb,4.1375e-04_rb,4.5888e-04_rb,4.8791e-04_rb, &
     &5.0614e-04_rb,5.0767e-04_rb,4.3569e-04_rb/)
      kao(:, 3, 5, 5) = (/ &
     &1.1255e-04_rb,2.7465e-04_rb,3.6022e-04_rb,4.2322e-04_rb,4.7137e-04_rb,5.0379e-04_rb, &
     &5.2287e-04_rb,5.2860e-04_rb,4.5259e-04_rb/)
      kao(:, 4, 5, 5) = (/ &
     &1.1490e-04_rb,2.8064e-04_rb,3.6839e-04_rb,4.3377e-04_rb,4.8454e-04_rb,5.2116e-04_rb, &
     &5.4180e-04_rb,5.5017e-04_rb,4.7054e-04_rb/)
      kao(:, 5, 5, 5) = (/ &
     &1.1845e-04_rb,2.8778e-04_rb,3.7745e-04_rb,4.4611e-04_rb,4.9985e-04_rb,5.3937e-04_rb, &
     &5.6277e-04_rb,5.7154e-04_rb,4.9105e-04_rb/)
      kao(:, 1, 6, 5) = (/ &
     &1.7235e-04_rb,3.3181e-04_rb,4.0736e-04_rb,4.5766e-04_rb,4.8926e-04_rb,5.0113e-04_rb, &
     &4.9597e-04_rb,4.6976e-04_rb,3.5862e-04_rb/)
      kao(:, 2, 6, 5) = (/ &
     &1.7547e-04_rb,3.3667e-04_rb,4.1414e-04_rb,4.6509e-04_rb,4.9861e-04_rb,5.1384e-04_rb, &
     &5.0976e-04_rb,4.8537e-04_rb,3.6976e-04_rb/)
      kao(:, 3, 6, 5) = (/ &
     &1.7903e-04_rb,3.4304e-04_rb,4.2198e-04_rb,4.7494e-04_rb,5.0792e-04_rb,5.2713e-04_rb, &
     &5.2630e-04_rb,5.0120e-04_rb,3.8272e-04_rb/)
      kao(:, 4, 6, 5) = (/ &
     &1.8326e-04_rb,3.5076e-04_rb,4.3097e-04_rb,4.8545e-04_rb,5.2030e-04_rb,5.4158e-04_rb, &
     &5.4362e-04_rb,5.1767e-04_rb,3.9660e-04_rb/)
      kao(:, 5, 6, 5) = (/ &
     &1.8798e-04_rb,3.5938e-04_rb,4.4150e-04_rb,4.9742e-04_rb,5.3429e-04_rb,5.5639e-04_rb, &
     &5.6101e-04_rb,5.3674e-04_rb,4.1415e-04_rb/)
      kao(:, 1, 7, 5) = (/ &
     &2.9155e-04_rb,4.4941e-04_rb,5.1334e-04_rb,5.5160e-04_rb,5.6796e-04_rb,5.6534e-04_rb, &
     &5.3624e-04_rb,4.7508e-04_rb,3.0367e-04_rb/)
      kao(:, 2, 7, 5) = (/ &
     &2.9728e-04_rb,4.5662e-04_rb,5.2104e-04_rb,5.6062e-04_rb,5.7827e-04_rb,5.7448e-04_rb, &
     &5.4737e-04_rb,4.8748e-04_rb,3.1275e-04_rb/)
      kao(:, 3, 7, 5) = (/ &
     &3.0362e-04_rb,4.6646e-04_rb,5.3089e-04_rb,5.7148e-04_rb,5.9016e-04_rb,5.8669e-04_rb, &
     &5.6081e-04_rb,5.0270e-04_rb,3.2221e-04_rb/)
      kao(:, 4, 7, 5) = (/ &
     &3.1074e-04_rb,4.7852e-04_rb,5.4299e-04_rb,5.8375e-04_rb,6.0313e-04_rb,6.0067e-04_rb, &
     &5.7534e-04_rb,5.1934e-04_rb,3.3373e-04_rb/)
      kao(:, 5, 7, 5) = (/ &
     &3.1895e-04_rb,4.9180e-04_rb,5.5670e-04_rb,5.9847e-04_rb,6.1763e-04_rb,6.1617e-04_rb, &
     &5.9045e-04_rb,5.3590e-04_rb,3.4594e-04_rb/)
      kao(:, 1, 8, 5) = (/ &
     &5.8477e-04_rb,7.4431e-04_rb,7.7278e-04_rb,7.7426e-04_rb,7.5736e-04_rb,7.1816e-04_rb, &
     &6.5106e-04_rb,5.3737e-04_rb,2.7725e-04_rb/)
      kao(:, 2, 8, 5) = (/ &
     &5.9470e-04_rb,7.5491e-04_rb,7.8490e-04_rb,7.8595e-04_rb,7.6803e-04_rb,7.3022e-04_rb, &
     &6.6291e-04_rb,5.4897e-04_rb,2.8497e-04_rb/)
      kao(:, 3, 8, 5) = (/ &
     &6.0851e-04_rb,7.7097e-04_rb,8.0204e-04_rb,8.0136e-04_rb,7.8218e-04_rb,7.4418e-04_rb, &
     &6.7668e-04_rb,5.6182e-04_rb,2.9265e-04_rb/)
      kao(:, 4, 8, 5) = (/ &
     &6.2426e-04_rb,7.9122e-04_rb,8.2299e-04_rb,8.2167e-04_rb,7.9967e-04_rb,7.6040e-04_rb, &
     &6.9192e-04_rb,5.7414e-04_rb,3.0175e-04_rb/)
      kao(:, 5, 8, 5) = (/ &
     &6.4049e-04_rb,8.1349e-04_rb,8.4675e-04_rb,8.4414e-04_rb,8.2107e-04_rb,7.7949e-04_rb, &
     &7.0895e-04_rb,5.8878e-04_rb,3.1239e-04_rb/)
      kao(:, 1, 9, 5) = (/ &
     &2.2196e-03_rb,2.3078e-03_rb,2.1925e-03_rb,2.0227e-03_rb,1.7963e-03_rb,1.5336e-03_rb, &
     &1.2354e-03_rb,8.8457e-04_rb,3.2582e-04_rb/)
      kao(:, 2, 9, 5) = (/ &
     &2.2570e-03_rb,2.3473e-03_rb,2.2277e-03_rb,2.0540e-03_rb,1.8274e-03_rb,1.5588e-03_rb, &
     &1.2524e-03_rb,8.9724e-04_rb,3.2676e-04_rb/)
      kao(:, 3, 9, 5) = (/ &
     &2.3089e-03_rb,2.3978e-03_rb,2.2707e-03_rb,2.0928e-03_rb,1.8651e-03_rb,1.5919e-03_rb, &
     &1.2779e-03_rb,9.1370e-04_rb,3.3500e-04_rb/)
      kao(:, 4, 9, 5) = (/ &
     &2.3733e-03_rb,2.4602e-03_rb,2.3315e-03_rb,2.1419e-03_rb,1.9119e-03_rb,1.6319e-03_rb, &
     &1.3090e-03_rb,9.3384e-04_rb,3.3899e-04_rb/)
      kao(:, 5, 9, 5) = (/ &
     &2.4406e-03_rb,2.5271e-03_rb,2.3986e-03_rb,2.2033e-03_rb,1.9655e-03_rb,1.6770e-03_rb, &
     &1.3445e-03_rb,9.5837e-04_rb,3.4762e-04_rb/)
      kao(:, 1,10, 5) = (/ &
     &9.4839e-03_rb,8.9137e-03_rb,7.9871e-03_rb,6.9719e-03_rb,5.8780e-03_rb,4.7134e-03_rb, &
     &3.4759e-03_rb,2.1070e-03_rb,4.8830e-04_rb/)
      kao(:, 2,10, 5) = (/ &
     &9.6557e-03_rb,9.0801e-03_rb,8.1340e-03_rb,7.0972e-03_rb,5.9780e-03_rb,4.7945e-03_rb, &
     &3.5398e-03_rb,2.1454e-03_rb,4.8857e-04_rb/)
      kao(:, 3,10, 5) = (/ &
     &9.8746e-03_rb,9.2754e-03_rb,8.2985e-03_rb,7.2339e-03_rb,6.1011e-03_rb,4.8911e-03_rb, &
     &3.6103e-03_rb,2.1918e-03_rb,5.0136e-04_rb/)
      kao(:, 4,10, 5) = (/ &
     &1.0155e-02_rb,9.5179e-03_rb,8.5070e-03_rb,7.4122e-03_rb,6.2501e-03_rb,5.0174e-03_rb, &
     &3.7003e-03_rb,2.2452e-03_rb,5.1609e-04_rb/)
      kao(:, 5,10, 5) = (/ &
     &1.0472e-02_rb,9.8032e-03_rb,8.7668e-03_rb,7.6331e-03_rb,6.4389e-03_rb,5.1659e-03_rb, &
     &3.8012e-03_rb,2.3067e-03_rb,5.2606e-04_rb/)
      kao(:, 1,11, 5) = (/ &
     &1.4268e-02_rb,1.3170e-02_rb,1.1684e-02_rb,1.0078e-02_rb,8.4022e-03_rb,6.6241e-03_rb, &
     &4.7723e-03_rb,2.7954e-03_rb,5.2361e-04_rb/)
      kao(:, 2,11, 5) = (/ &
     &1.4540e-02_rb,1.3417e-02_rb,1.1900e-02_rb,1.0250e-02_rb,8.5500e-03_rb,6.7539e-03_rb, &
     &4.8654e-03_rb,2.8533e-03_rb,5.3348e-04_rb/)
      kao(:, 3,11, 5) = (/ &
     &1.4880e-02_rb,1.3732e-02_rb,1.2163e-02_rb,1.0481e-02_rb,8.7324e-03_rb,6.9009e-03_rb, &
     &4.9833e-03_rb,2.9218e-03_rb,5.5883e-04_rb/)
      kao(:, 4,11, 5) = (/ &
     &1.5319e-02_rb,1.4123e-02_rb,1.2507e-02_rb,1.0772e-02_rb,8.9669e-03_rb,7.0955e-03_rb, &
     &5.1189e-03_rb,2.9943e-03_rb,5.8850e-04_rb/)
      kao(:, 5,11, 5) = (/ &
     &1.5835e-02_rb,1.4577e-02_rb,1.2919e-02_rb,1.1125e-02_rb,9.2533e-03_rb,7.3128e-03_rb, &
     &5.2667e-03_rb,3.0792e-03_rb,6.1073e-04_rb/)
      kao(:, 1,12, 5) = (/ &
     &1.5802e-02_rb,1.4518e-02_rb,1.2819e-02_rb,1.1003e-02_rb,9.1252e-03_rb,7.1742e-03_rb, &
     &5.1262e-03_rb,2.9592e-03_rb,5.0816e-04_rb/)
      kao(:, 2,12, 5) = (/ &
     &1.6146e-02_rb,1.4810e-02_rb,1.3074e-02_rb,1.1225e-02_rb,9.3090e-03_rb,7.3135e-03_rb, &
     &5.2412e-03_rb,3.0249e-03_rb,5.3398e-04_rb/)
      kao(:, 3,12, 5) = (/ &
     &1.6565e-02_rb,1.5205e-02_rb,1.3415e-02_rb,1.1514e-02_rb,9.5432e-03_rb,7.5058e-03_rb, &
     &5.3826e-03_rb,3.1020e-03_rb,5.6164e-04_rb/)
      kao(:, 4,12, 5) = (/ &
     &1.7128e-02_rb,1.5682e-02_rb,1.3836e-02_rb,1.1883e-02_rb,9.8516e-03_rb,7.7437e-03_rb, &
     &5.5313e-03_rb,3.1868e-03_rb,5.8823e-04_rb/)
      kao(:, 5,12, 5) = (/ &
     &1.7790e-02_rb,1.6267e-02_rb,1.4348e-02_rb,1.2311e-02_rb,1.0183e-02_rb,7.9904e-03_rb, &
     &5.7100e-03_rb,3.2821e-03_rb,6.0779e-04_rb/)
      kao(:, 1,13, 5) = (/ &
     &1.4021e-02_rb,1.2885e-02_rb,1.1370e-02_rb,9.7666e-03_rb,8.0956e-03_rb,6.3531e-03_rb, &
     &4.5485e-03_rb,2.6326e-03_rb,4.6002e-04_rb/)
      kao(:, 2,13, 5) = (/ &
     &1.4373e-02_rb,1.3207e-02_rb,1.1632e-02_rb,9.9865e-03_rb,8.2754e-03_rb,6.5087e-03_rb, &
     &4.6669e-03_rb,2.6967e-03_rb,4.8163e-04_rb/)
      kao(:, 3,13, 5) = (/ &
     &1.4774e-02_rb,1.3588e-02_rb,1.1994e-02_rb,1.0306e-02_rb,8.5450e-03_rb,6.7168e-03_rb, &
     &4.8089e-03_rb,2.7710e-03_rb,4.9996e-04_rb/)
      kao(:, 4,13, 5) = (/ &
     &1.5343e-02_rb,1.4084e-02_rb,1.2424e-02_rb,1.0664e-02_rb,8.8287e-03_rb,6.9355e-03_rb, &
     &4.9590e-03_rb,2.8556e-03_rb,5.2093e-04_rb/)
      kao(:, 5,13, 5) = (/ &
     &1.5979e-02_rb,1.4619e-02_rb,1.2888e-02_rb,1.1066e-02_rb,9.1632e-03_rb,7.1872e-03_rb, &
     &5.1400e-03_rb,2.9489e-03_rb,5.4238e-04_rb/)
      kao(:, 1, 1, 6) = (/ &
     &2.4381e-05_rb,3.5965e-04_rb,6.3963e-04_rb,9.0999e-04_rb,1.1781e-03_rb,1.4492e-03_rb, &
     &1.7469e-03_rb,2.1949e-03_rb,2.0167e-03_rb/)
      kao(:, 2, 1, 6) = (/ &
     &2.4551e-05_rb,3.6820e-04_rb,6.5359e-04_rb,9.2911e-04_rb,1.2030e-03_rb,1.4796e-03_rb, &
     &1.7884e-03_rb,2.2546e-03_rb,2.0633e-03_rb/)
      kao(:, 3, 1, 6) = (/ &
     &2.4844e-05_rb,3.7777e-04_rb,6.7015e-04_rb,9.5166e-04_rb,1.2316e-03_rb,1.5153e-03_rb, &
     &1.8326e-03_rb,2.3125e-03_rb,2.1171e-03_rb/)
      kao(:, 4, 1, 6) = (/ &
     &2.5013e-05_rb,3.8840e-04_rb,6.8929e-04_rb,9.7743e-04_rb,1.2638e-03_rb,1.5559e-03_rb, &
     &1.8833e-03_rb,2.3873e-03_rb,2.1814e-03_rb/)
      kao(:, 5, 1, 6) = (/ &
     &2.5349e-05_rb,4.0094e-04_rb,7.1137e-04_rb,1.0079e-03_rb,1.3021e-03_rb,1.6023e-03_rb, &
     &1.9431e-03_rb,2.4704e-03_rb,2.2554e-03_rb/)
      kao(:, 1, 2, 6) = (/ &
     &3.3532e-05_rb,3.4195e-04_rb,5.9227e-04_rb,8.3115e-04_rb,1.0659e-03_rb,1.2984e-03_rb, &
     &1.5375e-03_rb,1.8307e-03_rb,1.7555e-03_rb/)
      kao(:, 2, 2, 6) = (/ &
     &3.3884e-05_rb,3.4950e-04_rb,6.0496e-04_rb,8.4827e-04_rb,1.0865e-03_rb,1.3240e-03_rb, &
     &1.5683e-03_rb,1.8747e-03_rb,1.7936e-03_rb/)
      kao(:, 3, 2, 6) = (/ &
     &3.4432e-05_rb,3.5837e-04_rb,6.1975e-04_rb,8.6913e-04_rb,1.1113e-03_rb,1.3536e-03_rb, &
     &1.6021e-03_rb,1.9222e-03_rb,1.8373e-03_rb/)
      kao(:, 4, 2, 6) = (/ &
     &3.4886e-05_rb,3.6829e-04_rb,6.3662e-04_rb,8.9220e-04_rb,1.1391e-03_rb,1.3857e-03_rb, &
     &1.6406e-03_rb,1.9755e-03_rb,1.8873e-03_rb/)
      kao(:, 5, 2, 6) = (/ &
     &3.5279e-05_rb,3.7968e-04_rb,6.5599e-04_rb,9.1909e-04_rb,1.1730e-03_rb,1.4257e-03_rb, &
     &1.6862e-03_rb,2.0365e-03_rb,1.9484e-03_rb/)
      kao(:, 1, 3, 6) = (/ &
     &6.2269e-05_rb,3.6578e-04_rb,5.8707e-04_rb,7.9355e-04_rb,9.9285e-04_rb,1.1848e-03_rb, &
     &1.3757e-03_rb,1.5822e-03_rb,1.5104e-03_rb/)
      kao(:, 2, 3, 6) = (/ &
     &6.2987e-05_rb,3.7248e-04_rb,5.9883e-04_rb,8.1016e-04_rb,1.0136e-03_rb,1.2090e-03_rb, &
     &1.4023e-03_rb,1.6140e-03_rb,1.5440e-03_rb/)
      kao(:, 3, 3, 6) = (/ &
     &6.3830e-05_rb,3.8077e-04_rb,6.1238e-04_rb,8.2947e-04_rb,1.0380e-03_rb,1.2373e-03_rb, &
     &1.4338e-03_rb,1.6476e-03_rb,1.5800e-03_rb/)
      kao(:, 4, 3, 6) = (/ &
     &6.5057e-05_rb,3.9038e-04_rb,6.2908e-04_rb,8.5186e-04_rb,1.0655e-03_rb,1.2690e-03_rb, &
     &1.4686e-03_rb,1.6847e-03_rb,1.6204e-03_rb/)
      kao(:, 5, 3, 6) = (/ &
     &6.5891e-05_rb,4.0010e-04_rb,6.4778e-04_rb,8.7680e-04_rb,1.0962e-03_rb,1.3060e-03_rb, &
     &1.5104e-03_rb,1.7307e-03_rb,1.6697e-03_rb/)
      kao(:, 1, 4, 6) = (/ &
     &1.1763e-04_rb,4.2852e-04_rb,6.2857e-04_rb,8.0251e-04_rb,9.6160e-04_rb,1.1103e-03_rb, &
     &1.2549e-03_rb,1.3952e-03_rb,1.2948e-03_rb/)
      kao(:, 2, 4, 6) = (/ &
     &1.1903e-04_rb,4.3548e-04_rb,6.3935e-04_rb,8.1698e-04_rb,9.8074e-04_rb,1.1338e-03_rb, &
     &1.2810e-03_rb,1.4231e-03_rb,1.3227e-03_rb/)
      kao(:, 3, 4, 6) = (/ &
     &1.2084e-04_rb,4.4304e-04_rb,6.5330e-04_rb,8.3469e-04_rb,1.0028e-03_rb,1.1604e-03_rb, &
     &1.3104e-03_rb,1.4544e-03_rb,1.3535e-03_rb/)
      kao(:, 4, 4, 6) = (/ &
     &1.2295e-04_rb,4.5274e-04_rb,6.6888e-04_rb,8.5565e-04_rb,1.0277e-03_rb,1.1905e-03_rb, &
     &1.3437e-03_rb,1.4894e-03_rb,1.3881e-03_rb/)
      kao(:, 5, 4, 6) = (/ &
     &1.2576e-04_rb,4.6369e-04_rb,6.8705e-04_rb,8.8171e-04_rb,1.0587e-03_rb,1.2251e-03_rb, &
     &1.3822e-03_rb,1.5303e-03_rb,1.4309e-03_rb/)
      kao(:, 1, 5, 6) = (/ &
     &2.0248e-04_rb,5.1976e-04_rb,7.0294e-04_rb,8.5252e-04_rb,9.7761e-04_rb,1.0816e-03_rb, &
     &1.1680e-03_rb,1.2476e-03_rb,1.1044e-03_rb/)
      kao(:, 2, 5, 6) = (/ &
     &2.0493e-04_rb,5.2640e-04_rb,7.1432e-04_rb,8.6690e-04_rb,9.9395e-04_rb,1.1019e-03_rb, &
     &1.1914e-03_rb,1.2719e-03_rb,1.1285e-03_rb/)
      kao(:, 3, 5, 6) = (/ &
     &2.0806e-04_rb,5.3477e-04_rb,7.2739e-04_rb,8.8542e-04_rb,1.0148e-03_rb,1.1250e-03_rb, &
     &1.2182e-03_rb,1.3005e-03_rb,1.1542e-03_rb/)
      kao(:, 4, 5, 6) = (/ &
     &2.1205e-04_rb,5.4496e-04_rb,7.4280e-04_rb,9.0607e-04_rb,1.0399e-03_rb,1.1525e-03_rb, &
     &1.2484e-03_rb,1.3331e-03_rb,1.1843e-03_rb/)
      kao(:, 5, 5, 6) = (/ &
     &2.1768e-04_rb,5.5787e-04_rb,7.6255e-04_rb,9.3016e-04_rb,1.0697e-03_rb,1.1863e-03_rb, &
     &1.2845e-03_rb,1.3711e-03_rb,1.2208e-03_rb/)
      kao(:, 1, 6, 6) = (/ &
     &3.2402e-04_rb,6.5065e-04_rb,8.1276e-04_rb,9.3175e-04_rb,1.0254e-03_rb,1.0960e-03_rb, &
     &1.1336e-03_rb,1.1359e-03_rb,9.3926e-04_rb/)
      kao(:, 2, 6, 6) = (/ &
     &3.2839e-04_rb,6.5803e-04_rb,8.2121e-04_rb,9.4691e-04_rb,1.0426e-03_rb,1.1132e-03_rb, &
     &1.1537e-03_rb,1.1594e-03_rb,9.5631e-04_rb/)
      kao(:, 3, 6, 6) = (/ &
     &3.3397e-04_rb,6.6749e-04_rb,8.3425e-04_rb,9.6403e-04_rb,1.0654e-03_rb,1.1356e-03_rb, &
     &1.1768e-03_rb,1.1847e-03_rb,9.7614e-04_rb/)
      kao(:, 4, 6, 6) = (/ &
     &3.4074e-04_rb,6.7873e-04_rb,8.5077e-04_rb,9.8417e-04_rb,1.0893e-03_rb,1.1622e-03_rb, &
     &1.2041e-03_rb,1.2139e-03_rb,1.0001e-03_rb/)
      kao(:, 5, 6, 6) = (/ &
     &3.4865e-04_rb,6.9287e-04_rb,8.6902e-04_rb,1.0071e-03_rb,1.1162e-03_rb,1.1933e-03_rb, &
     &1.2370e-03_rb,1.2465e-03_rb,1.0295e-03_rb/)
      kao(:, 1, 7, 6) = (/ &
     &5.4679e-04_rb,8.8419e-04_rb,1.0272e-03_rb,1.1131e-03_rb,1.1586e-03_rb,1.1796e-03_rb, &
     &1.1677e-03_rb,1.1006e-03_rb,7.9375e-04_rb/)
      kao(:, 2, 7, 6) = (/ &
     &5.5453e-04_rb,8.9535e-04_rb,1.0406e-03_rb,1.1278e-03_rb,1.1758e-03_rb,1.1986e-03_rb, &
     &1.1860e-03_rb,1.1193e-03_rb,8.0863e-04_rb/)
      kao(:, 3, 7, 6) = (/ &
     &5.6482e-04_rb,9.0803e-04_rb,1.0560e-03_rb,1.1451e-03_rb,1.1955e-03_rb,1.2220e-03_rb, &
     &1.2089e-03_rb,1.1408e-03_rb,8.2565e-04_rb/)
      kao(:, 4, 7, 6) = (/ &
     &5.7672e-04_rb,9.2334e-04_rb,1.0748e-03_rb,1.1654e-03_rb,1.2195e-03_rb,1.2479e-03_rb, &
     &1.2361e-03_rb,1.1655e-03_rb,8.4564e-04_rb/)
      kao(:, 5, 7, 6) = (/ &
     &5.9187e-04_rb,9.4280e-04_rb,1.0968e-03_rb,1.1862e-03_rb,1.2479e-03_rb,1.2773e-03_rb, &
     &1.2677e-03_rb,1.1958e-03_rb,8.6975e-04_rb/)
      kao(:, 1, 8, 6) = (/ &
     &1.0890e-03_rb,1.4212e-03_rb,1.5274e-03_rb,1.5635e-03_rb,1.5408e-03_rb,1.4729e-03_rb, &
     &1.3514e-03_rb,1.1730e-03_rb,6.7724e-04_rb/)
      kao(:, 2, 8, 6) = (/ &
     &1.1074e-03_rb,1.4404e-03_rb,1.5493e-03_rb,1.5864e-03_rb,1.5647e-03_rb,1.4941e-03_rb, &
     &1.3720e-03_rb,1.1894e-03_rb,6.8838e-04_rb/)
      kao(:, 3, 8, 6) = (/ &
     &1.1299e-03_rb,1.4626e-03_rb,1.5727e-03_rb,1.6115e-03_rb,1.5892e-03_rb,1.5170e-03_rb, &
     &1.3954e-03_rb,1.2114e-03_rb,7.0024e-04_rb/)
      kao(:, 4, 8, 6) = (/ &
     &1.1562e-03_rb,1.4908e-03_rb,1.6005e-03_rb,1.6404e-03_rb,1.6196e-03_rb,1.5467e-03_rb, &
     &1.4229e-03_rb,1.2388e-03_rb,7.1485e-04_rb/)
      kao(:, 5, 8, 6) = (/ &
     &1.1885e-03_rb,1.5267e-03_rb,1.6357e-03_rb,1.6752e-03_rb,1.6528e-03_rb,1.5764e-03_rb, &
     &1.4553e-03_rb,1.2696e-03_rb,7.3183e-04_rb/)
      kao(:, 1, 9, 6) = (/ &
     &4.1665e-03_rb,4.3468e-03_rb,4.1293e-03_rb,3.8165e-03_rb,3.4449e-03_rb,3.0107e-03_rb, &
     &2.4884e-03_rb,1.8186e-03_rb,7.0770e-04_rb/)
      kao(:, 2, 9, 6) = (/ &
     &4.2379e-03_rb,4.4019e-03_rb,4.1804e-03_rb,3.8702e-03_rb,3.4962e-03_rb,3.0618e-03_rb, &
     &2.5301e-03_rb,1.8480e-03_rb,7.1084e-04_rb/)
      kao(:, 3, 9, 6) = (/ &
     &4.3206e-03_rb,4.4730e-03_rb,4.2494e-03_rb,3.9358e-03_rb,3.5562e-03_rb,3.1108e-03_rb, &
     &2.5726e-03_rb,1.8787e-03_rb,7.1447e-04_rb/)
      kao(:, 4, 9, 6) = (/ &
     &4.4148e-03_rb,4.5672e-03_rb,4.3360e-03_rb,4.0233e-03_rb,3.6279e-03_rb,3.1667e-03_rb, &
     &2.6211e-03_rb,1.9152e-03_rb,7.2547e-04_rb/)
      kao(:, 5, 9, 6) = (/ &
     &4.5366e-03_rb,4.6880e-03_rb,4.4475e-03_rb,4.1247e-03_rb,3.7148e-03_rb,3.2357e-03_rb, &
     &2.6777e-03_rb,1.9564e-03_rb,7.3351e-04_rb/)
      kao(:, 1,10, 6) = (/ &
     &1.8255e-02_rb,1.7198e-02_rb,1.5391e-02_rb,1.3391e-02_rb,1.1273e-02_rb,9.0394e-03_rb, &
     &6.6601e-03_rb,4.0944e-03_rb,9.8389e-04_rb/)
      kao(:, 2,10, 6) = (/ &
     &1.8579e-02_rb,1.7439e-02_rb,1.5601e-02_rb,1.3577e-02_rb,1.1427e-02_rb,9.1759e-03_rb, &
     &6.7580e-03_rb,4.1605e-03_rb,9.8485e-04_rb/)
      kao(:, 3,10, 6) = (/ &
     &1.8934e-02_rb,1.7772e-02_rb,1.5898e-02_rb,1.3821e-02_rb,1.1616e-02_rb,9.3279e-03_rb, &
     &6.8809e-03_rb,4.2287e-03_rb,9.8296e-04_rb/)
      kao(:, 4,10, 6) = (/ &
     &1.9364e-02_rb,1.8152e-02_rb,1.6246e-02_rb,1.4125e-02_rb,1.1868e-02_rb,9.5176e-03_rb, &
     &7.0263e-03_rb,4.3137e-03_rb,9.8595e-04_rb/)
      kao(:, 5,10, 6) = (/ &
     &1.9876e-02_rb,1.8605e-02_rb,1.6643e-02_rb,1.4478e-02_rb,1.2165e-02_rb,9.7573e-03_rb, &
     &7.2015e-03_rb,4.4138e-03_rb,9.9549e-04_rb/)
      kao(:, 1,11, 6) = (/ &
     &2.7906e-02_rb,2.5779e-02_rb,2.2845e-02_rb,1.9689e-02_rb,1.6356e-02_rb,1.2926e-02_rb, &
     &9.3112e-03_rb,5.4266e-03_rb,1.1493e-03_rb/)
      kao(:, 2,11, 6) = (/ &
     &2.8474e-02_rb,2.6276e-02_rb,2.3282e-02_rb,2.0063e-02_rb,1.6649e-02_rb,1.3118e-02_rb, &
     &9.4591e-03_rb,5.5176e-03_rb,1.1605e-03_rb/)
      kao(:, 3,11, 6) = (/ &
     &2.9161e-02_rb,2.6826e-02_rb,2.3755e-02_rb,2.0435e-02_rb,1.6944e-02_rb,1.3356e-02_rb, &
     &9.6281e-03_rb,5.6370e-03_rb,1.1658e-03_rb/)
      kao(:, 4,11, 6) = (/ &
     &2.9864e-02_rb,2.7446e-02_rb,2.4305e-02_rb,2.0907e-02_rb,1.7344e-02_rb,1.3665e-02_rb, &
     &9.8501e-03_rb,5.7787e-03_rb,1.1679e-03_rb/)
      kao(:, 5,11, 6) = (/ &
     &3.0716e-02_rb,2.8218e-02_rb,2.4989e-02_rb,2.1484e-02_rb,1.7837e-02_rb,1.4047e-02_rb, &
     &1.0137e-02_rb,5.9391e-03_rb,1.1749e-03_rb/)
      kao(:, 1,12, 6) = (/ &
     &3.1456e-02_rb,2.8840e-02_rb,2.5490e-02_rb,2.1901e-02_rb,1.8104e-02_rb,1.4184e-02_rb, &
     &1.0144e-02_rb,5.8131e-03_rb,1.1656e-03_rb/)
      kao(:, 2,12, 6) = (/ &
     &3.2212e-02_rb,2.9507e-02_rb,2.6048e-02_rb,2.2347e-02_rb,1.8450e-02_rb,1.4457e-02_rb, &
     &1.0325e-02_rb,5.9330e-03_rb,1.1607e-03_rb/)
      kao(:, 3,12, 6) = (/ &
     &3.2965e-02_rb,3.0133e-02_rb,2.6613e-02_rb,2.2834e-02_rb,1.8888e-02_rb,1.4793e-02_rb, &
     &1.0553e-02_rb,6.0675e-03_rb,1.1612e-03_rb/)
      kao(:, 4,12, 6) = (/ &
     &3.3903e-02_rb,3.0999e-02_rb,2.7365e-02_rb,2.3453e-02_rb,1.9368e-02_rb,1.5160e-02_rb, &
     &1.0832e-02_rb,6.2310e-03_rb,1.1792e-03_rb/)
      kao(:, 5,12, 6) = (/ &
     &3.4986e-02_rb,3.1940e-02_rb,2.8176e-02_rb,2.4131e-02_rb,1.9947e-02_rb,1.5618e-02_rb, &
     &1.1156e-02_rb,6.4247e-03_rb,1.2061e-03_rb/)
      kao(:, 1,13, 6) = (/ &
     &2.8494e-02_rb,2.6140e-02_rb,2.3086e-02_rb,1.9829e-02_rb,1.6410e-02_rb,1.2869e-02_rb, &
     &9.1750e-03_rb,5.2477e-03_rb,1.0839e-03_rb/)
      kao(:, 2,13, 6) = (/ &
     &2.9204e-02_rb,2.6786e-02_rb,2.3660e-02_rb,2.0312e-02_rb,1.6791e-02_rb,1.3151e-02_rb, &
     &9.3741e-03_rb,5.3670e-03_rb,1.0879e-03_rb/)
      kao(:, 3,13, 6) = (/ &
     &3.0028e-02_rb,2.7500e-02_rb,2.4267e-02_rb,2.0802e-02_rb,1.7182e-02_rb,1.3458e-02_rb, &
     &9.5924e-03_rb,5.5018e-03_rb,1.1043e-03_rb/)
      kao(:, 4,13, 6) = (/ &
     &3.0958e-02_rb,2.8295e-02_rb,2.4961e-02_rb,2.1395e-02_rb,1.7690e-02_rb,1.3840e-02_rb, &
     &9.8578e-03_rb,5.6588e-03_rb,1.1197e-03_rb/)
      kao(:, 5,13, 6) = (/ &
     &3.2042e-02_rb,2.9250e-02_rb,2.5779e-02_rb,2.2082e-02_rb,1.8250e-02_rb,1.4290e-02_rb, &
     &1.0177e-02_rb,5.8420e-03_rb,1.1367e-03_rb/)
      kao(:, 1, 1, 7) = (/ &
     &4.8330e-05_rb,9.1748e-04_rb,1.6343e-03_rb,2.3037e-03_rb,2.9566e-03_rb,3.6315e-03_rb, &
     &4.4475e-03_rb,5.8406e-03_rb,5.3151e-03_rb/)
      kao(:, 2, 1, 7) = (/ &
     &4.8596e-05_rb,9.3271e-04_rb,1.6606e-03_rb,2.3393e-03_rb,3.0027e-03_rb,3.6916e-03_rb, &
     &4.5243e-03_rb,5.9244e-03_rb,5.4196e-03_rb/)
      kao(:, 3, 1, 7) = (/ &
     &4.9220e-05_rb,9.5049e-04_rb,1.6928e-03_rb,2.3820e-03_rb,3.0582e-03_rb,3.7632e-03_rb, &
     &4.6161e-03_rb,6.0368e-03_rb,5.5387e-03_rb/)
      kao(:, 4, 1, 7) = (/ &
     &4.9714e-05_rb,9.7063e-04_rb,1.7276e-03_rb,2.4311e-03_rb,3.1220e-03_rb,3.8430e-03_rb, &
     &4.7205e-03_rb,6.1613e-03_rb,5.6708e-03_rb/)
      kao(:, 5, 1, 7) = (/ &
     &5.0550e-05_rb,9.9274e-04_rb,1.7651e-03_rb,2.4850e-03_rb,3.1923e-03_rb,3.9328e-03_rb, &
     &4.8316e-03_rb,6.3027e-03_rb,5.8156e-03_rb/)
      kao(:, 1, 2, 7) = (/ &
     &6.7697e-05_rb,8.5995e-04_rb,1.5004e-03_rb,2.0933e-03_rb,2.6519e-03_rb,3.1963e-03_rb, &
     &3.7792e-03_rb,4.6863e-03_rb,4.4711e-03_rb/)
      kao(:, 2, 2, 7) = (/ &
     &6.8123e-05_rb,8.7528e-04_rb,1.5266e-03_rb,2.1301e-03_rb,2.6961e-03_rb,3.2484e-03_rb, &
     &3.8450e-03_rb,4.7682e-03_rb,4.5579e-03_rb/)
      kao(:, 3, 2, 7) = (/ &
     &6.9098e-05_rb,8.9351e-04_rb,1.5568e-03_rb,2.1723e-03_rb,2.7472e-03_rb,3.3096e-03_rb, &
     &3.9205e-03_rb,4.8697e-03_rb,4.6570e-03_rb/)
      kao(:, 4, 2, 7) = (/ &
     &7.0075e-05_rb,9.1426e-04_rb,1.5908e-03_rb,2.2183e-03_rb,2.8071e-03_rb,3.3815e-03_rb, &
     &4.0084e-03_rb,4.9811e-03_rb,4.7715e-03_rb/)
      kao(:, 5, 2, 7) = (/ &
     &7.1192e-05_rb,9.3706e-04_rb,1.6283e-03_rb,2.2693e-03_rb,2.8718e-03_rb,3.4625e-03_rb, &
     &4.1070e-03_rb,5.1037e-03_rb,4.8979e-03_rb/)
      kao(:, 1, 3, 7) = (/ &
     &1.2733e-04_rb,8.7624e-04_rb,1.4506e-03_rb,1.9805e-03_rb,2.4757e-03_rb,2.9317e-03_rb, &
     &3.3558e-03_rb,3.8170e-03_rb,3.7686e-03_rb/)
      kao(:, 2, 3, 7) = (/ &
     &1.2826e-04_rb,8.9276e-04_rb,1.4769e-03_rb,2.0153e-03_rb,2.5174e-03_rb,2.9784e-03_rb, &
     &3.4075e-03_rb,3.8801e-03_rb,3.8344e-03_rb/)
      kao(:, 3, 3, 7) = (/ &
     &1.2973e-04_rb,9.1170e-04_rb,1.5076e-03_rb,2.0539e-03_rb,2.5651e-03_rb,3.0352e-03_rb, &
     &3.4680e-03_rb,3.9525e-03_rb,3.9125e-03_rb/)
      kao(:, 4, 3, 7) = (/ &
     &1.3225e-04_rb,9.3418e-04_rb,1.5436e-03_rb,2.1000e-03_rb,2.6202e-03_rb,3.1002e-03_rb, &
     &3.5421e-03_rb,4.0383e-03_rb,4.0057e-03_rb/)
      kao(:, 5, 3, 7) = (/ &
     &1.3440e-04_rb,9.5794e-04_rb,1.5825e-03_rb,2.1509e-03_rb,2.6822e-03_rb,3.1720e-03_rb, &
     &3.6252e-03_rb,4.1370e-03_rb,4.1108e-03_rb/)
      kao(:, 1, 4, 7) = (/ &
     &2.4414e-04_rb,9.8791e-04_rb,1.4797e-03_rb,1.9337e-03_rb,2.3609e-03_rb,2.7541e-03_rb, &
     &3.0988e-03_rb,3.3862e-03_rb,3.2303e-03_rb/)
      kao(:, 2, 4, 7) = (/ &
     &2.4513e-04_rb,1.0039e-03_rb,1.5070e-03_rb,1.9692e-03_rb,2.4021e-03_rb,2.7995e-03_rb, &
     &3.1468e-03_rb,3.4333e-03_rb,3.2790e-03_rb/)
      kao(:, 3, 4, 7) = (/ &
     &2.4808e-04_rb,1.0229e-03_rb,1.5373e-03_rb,2.0098e-03_rb,2.4481e-03_rb,2.8491e-03_rb, &
     &3.2030e-03_rb,3.4903e-03_rb,3.3384e-03_rb/)
      kao(:, 4, 4, 7) = (/ &
     &2.5202e-04_rb,1.0447e-03_rb,1.5721e-03_rb,2.0572e-03_rb,2.5035e-03_rb,2.9093e-03_rb, &
     &3.2672e-03_rb,3.5611e-03_rb,3.4115e-03_rb/)
      kao(:, 5, 4, 7) = (/ &
     &2.5833e-04_rb,1.0719e-03_rb,1.6144e-03_rb,2.1102e-03_rb,2.5658e-03_rb,2.9793e-03_rb, &
     &3.3418e-03_rb,3.6423e-03_rb,3.4938e-03_rb/)
      kao(:, 1, 5, 7) = (/ &
     &4.2770e-04_rb,1.1830e-03_rb,1.6220e-03_rb,1.9840e-03_rb,2.3123e-03_rb,2.6214e-03_rb, &
     &2.8956e-03_rb,3.0934e-03_rb,2.7805e-03_rb/)
      kao(:, 2, 5, 7) = (/ &
     &4.3162e-04_rb,1.2016e-03_rb,1.6476e-03_rb,2.0183e-03_rb,2.3523e-03_rb,2.6652e-03_rb, &
     &2.9419e-03_rb,3.1389e-03_rb,2.8192e-03_rb/)
      kao(:, 3, 5, 7) = (/ &
     &4.3693e-04_rb,1.2232e-03_rb,1.6764e-03_rb,2.0550e-03_rb,2.3985e-03_rb,2.7158e-03_rb, &
     &2.9926e-03_rb,3.1918e-03_rb,2.8694e-03_rb/)
      kao(:, 4, 5, 7) = (/ &
     &4.4235e-04_rb,1.2462e-03_rb,1.7099e-03_rb,2.0981e-03_rb,2.4518e-03_rb,2.7746e-03_rb, &
     &3.0533e-03_rb,3.2519e-03_rb,2.9306e-03_rb/)
      kao(:, 5, 5, 7) = (/ &
     &4.5257e-04_rb,1.2739e-03_rb,1.7526e-03_rb,2.1524e-03_rb,2.5133e-03_rb,2.8432e-03_rb, &
     &3.1246e-03_rb,3.3219e-03_rb,2.9994e-03_rb/)
      kao(:, 1, 6, 7) = (/ &
     &6.9917e-04_rb,1.4510e-03_rb,1.8508e-03_rb,2.1612e-03_rb,2.3969e-03_rb,2.5820e-03_rb, &
     &2.7369e-03_rb,2.8411e-03_rb,2.3809e-03_rb/)
      kao(:, 2, 6, 7) = (/ &
     &7.0685e-04_rb,1.4737e-03_rb,1.8819e-03_rb,2.1948e-03_rb,2.4365e-03_rb,2.6262e-03_rb, &
     &2.7822e-03_rb,2.8845e-03_rb,2.4163e-03_rb/)
      kao(:, 3, 6, 7) = (/ &
     &7.1685e-04_rb,1.4992e-03_rb,1.9139e-03_rb,2.2300e-03_rb,2.4777e-03_rb,2.6750e-03_rb, &
     &2.8324e-03_rb,2.9331e-03_rb,2.4594e-03_rb/)
      kao(:, 4, 6, 7) = (/ &
     &7.2831e-04_rb,1.5276e-03_rb,1.9491e-03_rb,2.2715e-03_rb,2.5262e-03_rb,2.7293e-03_rb, &
     &2.8927e-03_rb,2.9913e-03_rb,2.5109e-03_rb/)
      kao(:, 5, 6, 7) = (/ &
     &7.4174e-04_rb,1.5586e-03_rb,1.9900e-03_rb,2.3214e-03_rb,2.5825e-03_rb,2.7929e-03_rb, &
     &2.9629e-03_rb,3.0587e-03_rb,2.5688e-03_rb/)
      kao(:, 1, 7, 7) = (/ &
     &1.2039e-03_rb,1.9587e-03_rb,2.2962e-03_rb,2.5331e-03_rb,2.6974e-03_rb,2.7733e-03_rb, &
     &2.7634e-03_rb,2.6777e-03_rb,2.0245e-03_rb/)
      kao(:, 2, 7, 7) = (/ &
     &1.2204e-03_rb,1.9874e-03_rb,2.3314e-03_rb,2.5722e-03_rb,2.7404e-03_rb,2.8207e-03_rb, &
     &2.8122e-03_rb,2.7221e-03_rb,2.0503e-03_rb/)
      kao(:, 3, 7, 7) = (/ &
     &1.2400e-03_rb,2.0184e-03_rb,2.3716e-03_rb,2.6156e-03_rb,2.7851e-03_rb,2.8685e-03_rb, &
     &2.8623e-03_rb,2.7709e-03_rb,2.0844e-03_rb/)
      kao(:, 4, 7, 7) = (/ &
     &1.2633e-03_rb,2.0537e-03_rb,2.4164e-03_rb,2.6645e-03_rb,2.8368e-03_rb,2.9213e-03_rb, &
     &2.9184e-03_rb,2.8302e-03_rb,2.1246e-03_rb/)
      kao(:, 5, 7, 7) = (/ &
     &1.2880e-03_rb,2.0946e-03_rb,2.4660e-03_rb,2.7241e-03_rb,2.8956e-03_rb,2.9852e-03_rb, &
     &2.9850e-03_rb,2.8977e-03_rb,2.1717e-03_rb/)
      kao(:, 1, 8, 7) = (/ &
     &2.4360e-03_rb,3.2317e-03_rb,3.4336e-03_rb,3.4928e-03_rb,3.4872e-03_rb,3.3872e-03_rb, &
     &3.1811e-03_rb,2.7989e-03_rb,1.6994e-03_rb/)
      kao(:, 2, 8, 7) = (/ &
     &2.4784e-03_rb,3.2772e-03_rb,3.4707e-03_rb,3.5477e-03_rb,3.5429e-03_rb,3.4442e-03_rb, &
     &3.2393e-03_rb,2.8503e-03_rb,1.7216e-03_rb/)
      kao(:, 3, 8, 7) = (/ &
     &2.5215e-03_rb,3.3205e-03_rb,3.5181e-03_rb,3.6097e-03_rb,3.6079e-03_rb,3.5043e-03_rb, &
     &3.2957e-03_rb,2.9015e-03_rb,1.7516e-03_rb/)
      kao(:, 4, 8, 7) = (/ &
     &2.5734e-03_rb,3.3690e-03_rb,3.5815e-03_rb,3.6774e-03_rb,3.6742e-03_rb,3.5696e-03_rb, &
     &3.3583e-03_rb,2.9597e-03_rb,1.7852e-03_rb/)
      kao(:, 5, 8, 7) = (/ &
     &2.6287e-03_rb,3.4209e-03_rb,3.6564e-03_rb,3.7526e-03_rb,3.7488e-03_rb,3.6496e-03_rb, &
     &3.4322e-03_rb,3.0257e-03_rb,1.8234e-03_rb/)
      kao(:, 1, 9, 7) = (/ &
     &9.2016e-03_rb,9.7823e-03_rb,9.4145e-03_rb,8.7411e-03_rb,7.8879e-03_rb,6.8736e-03_rb, &
     &5.6569e-03_rb,4.2025e-03_rb,1.5538e-03_rb/)
      kao(:, 2, 9, 7) = (/ &
     &9.3962e-03_rb,9.9769e-03_rb,9.5905e-03_rb,8.8876e-03_rb,8.0147e-03_rb,6.9737e-03_rb, &
     &5.7408e-03_rb,4.2797e-03_rb,1.5748e-03_rb/)
      kao(:, 3, 9, 7) = (/ &
     &9.5905e-03_rb,1.0169e-02_rb,9.7567e-03_rb,9.0284e-03_rb,8.1348e-03_rb,7.0770e-03_rb, &
     &5.8473e-03_rb,4.3619e-03_rb,1.5893e-03_rb/)
      kao(:, 4, 9, 7) = (/ &
     &9.8163e-03_rb,1.0370e-02_rb,9.9309e-03_rb,9.1820e-03_rb,8.2703e-03_rb,7.1825e-03_rb, &
     &5.9588e-03_rb,4.4477e-03_rb,1.6087e-03_rb/)
      kao(:, 5, 9, 7) = (/ &
     &1.0054e-02_rb,1.0584e-02_rb,1.0131e-02_rb,9.3626e-03_rb,8.4298e-03_rb,7.3350e-03_rb, &
     &6.0835e-03_rb,4.5437e-03_rb,1.6313e-03_rb/)
      kao(:, 1,10, 7) = (/ &
     &3.9276e-02_rb,3.6749e-02_rb,3.3301e-02_rb,2.9241e-02_rb,2.4794e-02_rb,2.0028e-02_rb, &
     &1.4912e-02_rb,9.3013e-03_rb,2.0443e-03_rb/)
      kao(:, 2,10, 7) = (/ &
     &4.0105e-02_rb,3.7556e-02_rb,3.4005e-02_rb,2.9863e-02_rb,2.5353e-02_rb,2.0438e-02_rb, &
     &1.5180e-02_rb,9.4819e-03_rb,2.0242e-03_rb/)
      kao(:, 3,10, 7) = (/ &
     &4.0916e-02_rb,3.8307e-02_rb,3.4681e-02_rb,3.0487e-02_rb,2.5876e-02_rb,2.0814e-02_rb, &
     &1.5451e-02_rb,9.6638e-03_rb,2.0047e-03_rb/)
      kao(:, 4,10, 7) = (/ &
     &4.1894e-02_rb,3.9199e-02_rb,3.5443e-02_rb,3.1118e-02_rb,2.6390e-02_rb,2.1233e-02_rb, &
     &1.5746e-02_rb,9.8447e-03_rb,2.0155e-03_rb/)
      kao(:, 5,10, 7) = (/ &
     &4.2954e-02_rb,4.0161e-02_rb,3.6290e-02_rb,3.1839e-02_rb,2.6971e-02_rb,2.1693e-02_rb, &
     &1.6115e-02_rb,1.0055e-02_rb,2.0394e-03_rb/)
      kao(:, 1,11, 7) = (/ &
     &6.0955e-02_rb,5.5737e-02_rb,4.9723e-02_rb,4.3019e-02_rb,3.5876e-02_rb,2.8379e-02_rb, &
     &2.0432e-02_rb,1.2085e-02_rb,2.3716e-03_rb/)
      kao(:, 2,11, 7) = (/ &
     &6.2198e-02_rb,5.6869e-02_rb,5.0662e-02_rb,4.3787e-02_rb,3.6565e-02_rb,2.8915e-02_rb, &
     &2.0814e-02_rb,1.2323e-02_rb,2.3542e-03_rb/)
      kao(:, 3,11, 7) = (/ &
     &6.3419e-02_rb,5.8024e-02_rb,5.1697e-02_rb,4.4702e-02_rb,3.7324e-02_rb,2.9465e-02_rb, &
     &2.1204e-02_rb,1.2555e-02_rb,2.3313e-03_rb/)
      kao(:, 4,11, 7) = (/ &
     &6.4926e-02_rb,5.9386e-02_rb,5.2869e-02_rb,4.5680e-02_rb,3.8119e-02_rb,3.0075e-02_rb, &
     &2.1667e-02_rb,1.2812e-02_rb,2.3155e-03_rb/)
      kao(:, 5,11, 7) = (/ &
     &6.6635e-02_rb,6.0864e-02_rb,5.4124e-02_rb,4.6774e-02_rb,3.9011e-02_rb,3.0829e-02_rb, &
     &2.2206e-02_rb,1.3119e-02_rb,2.3251e-03_rb/)
      kao(:, 1,12, 7) = (/ &
     &6.9869e-02_rb,6.3637e-02_rb,5.6420e-02_rb,4.8546e-02_rb,4.0307e-02_rb,3.1740e-02_rb, &
     &2.2682e-02_rb,1.3139e-02_rb,2.4428e-03_rb/)
      kao(:, 2,12, 7) = (/ &
     &7.1577e-02_rb,6.5133e-02_rb,5.7685e-02_rb,4.9634e-02_rb,4.1244e-02_rb,3.2466e-02_rb, &
     &2.3165e-02_rb,1.3402e-02_rb,2.4417e-03_rb/)
      kao(:, 3,12, 7) = (/ &
     &7.3435e-02_rb,6.6765e-02_rb,5.9111e-02_rb,5.0782e-02_rb,4.2090e-02_rb,3.3070e-02_rb, &
     &2.3631e-02_rb,1.3663e-02_rb,2.4312e-03_rb/)
      kao(:, 4,12, 7) = (/ &
     &7.5274e-02_rb,6.8406e-02_rb,6.0539e-02_rb,5.2024e-02_rb,4.3158e-02_rb,3.3897e-02_rb, &
     &2.4213e-02_rb,1.3961e-02_rb,2.4213e-03_rb/)
      kao(:, 5,12, 7) = (/ &
     &7.7518e-02_rb,7.0447e-02_rb,6.2339e-02_rb,5.3608e-02_rb,4.4457e-02_rb,3.4884e-02_rb, &
     &2.4882e-02_rb,1.4302e-02_rb,2.4144e-03_rb/)
      kao(:, 1,13, 7) = (/ &
     &6.3971e-02_rb,5.8270e-02_rb,5.1535e-02_rb,4.4263e-02_rb,3.6709e-02_rb,2.8872e-02_rb, &
     &2.0629e-02_rb,1.1917e-02_rb,2.3953e-03_rb/)
      kao(:, 2,13, 7) = (/ &
     &6.5643e-02_rb,5.9813e-02_rb,5.2890e-02_rb,4.5419e-02_rb,3.7684e-02_rb,2.9600e-02_rb, &
     &2.1119e-02_rb,1.2188e-02_rb,2.4046e-03_rb/)
      kao(:, 3,13, 7) = (/ &
     &6.7472e-02_rb,6.1457e-02_rb,5.4341e-02_rb,4.6654e-02_rb,3.8721e-02_rb,3.0402e-02_rb, &
     &2.1696e-02_rb,1.2486e-02_rb,2.3922e-03_rb/)
      kao(:, 4,13, 7) = (/ &
     &6.9579e-02_rb,6.3365e-02_rb,5.6023e-02_rb,4.8132e-02_rb,3.9923e-02_rb,3.1337e-02_rb, &
     &2.2348e-02_rb,1.2806e-02_rb,2.3768e-03_rb/)
      kao(:, 5,13, 7) = (/ &
     &7.1993e-02_rb,6.5596e-02_rb,5.7974e-02_rb,4.9776e-02_rb,4.1204e-02_rb,3.2317e-02_rb, &
     &2.3035e-02_rb,1.3183e-02_rb,2.3765e-03_rb/)
      kao(:, 1, 1, 8) = (/ &
     &1.3552e-04_rb,2.4322e-03_rb,4.1951e-03_rb,5.8754e-03_rb,7.6297e-03_rb,9.6149e-03_rb, &
     &1.1994e-02_rb,1.5380e-02_rb,1.4856e-02_rb/)
      kao(:, 2, 1, 8) = (/ &
     &1.3805e-04_rb,2.4459e-03_rb,4.2175e-03_rb,5.9121e-03_rb,7.6823e-03_rb,9.6825e-03_rb, &
     &1.2072e-02_rb,1.5469e-02_rb,1.4971e-02_rb/)
      kao(:, 3, 1, 8) = (/ &
     &1.4131e-04_rb,2.4661e-03_rb,4.2478e-03_rb,5.9602e-03_rb,7.7525e-03_rb,9.7747e-03_rb, &
     &1.2179e-02_rb,1.5577e-02_rb,1.5128e-02_rb/)
      kao(:, 4, 1, 8) = (/ &
     &1.4377e-04_rb,2.4912e-03_rb,4.2859e-03_rb,6.0170e-03_rb,7.8373e-03_rb,9.8887e-03_rb, &
     &1.2314e-02_rb,1.5728e-02_rb,1.5316e-02_rb/)
      kao(:, 5, 1, 8) = (/ &
     &1.4692e-04_rb,2.5205e-03_rb,4.3310e-03_rb,6.0836e-03_rb,7.9343e-03_rb,1.0019e-02_rb, &
     &1.2473e-02_rb,1.5904e-02_rb,1.5528e-02_rb/)
      kao(:, 1, 2, 8) = (/ &
     &1.8928e-04_rb,2.4121e-03_rb,4.0623e-03_rb,5.5541e-03_rb,6.9898e-03_rb,8.5008e-03_rb, &
     &1.0323e-02_rb,1.3028e-02_rb,1.2924e-02_rb/)
      kao(:, 2, 2, 8) = (/ &
     &1.9267e-04_rb,2.4360e-03_rb,4.0898e-03_rb,5.5901e-03_rb,7.0384e-03_rb,8.5652e-03_rb, &
     &1.0405e-02_rb,1.3116e-02_rb,1.3041e-02_rb/)
      kao(:, 3, 2, 8) = (/ &
     &1.9724e-04_rb,2.4656e-03_rb,4.1277e-03_rb,5.6355e-03_rb,7.0999e-03_rb,8.6497e-03_rb, &
     &1.0515e-02_rb,1.3234e-02_rb,1.3193e-02_rb/)
      kao(:, 4, 2, 8) = (/ &
     &2.0128e-04_rb,2.4981e-03_rb,4.1715e-03_rb,5.6919e-03_rb,7.1715e-03_rb,8.7477e-03_rb, &
     &1.0645e-02_rb,1.3388e-02_rb,1.3368e-02_rb/)
      kao(:, 5, 2, 8) = (/ &
     &2.0477e-04_rb,2.5328e-03_rb,4.2216e-03_rb,5.7557e-03_rb,7.2538e-03_rb,8.8556e-03_rb, &
     &1.0788e-02_rb,1.3565e-02_rb,1.3563e-02_rb/)
      kao(:, 1, 3, 8) = (/ &
     &3.5927e-04_rb,2.5910e-03_rb,4.2037e-03_rb,5.6032e-03_rb,6.8389e-03_rb,7.9673e-03_rb, &
     &9.0787e-03_rb,1.0659e-02_rb,1.1075e-02_rb/)
      kao(:, 2, 3, 8) = (/ &
     &3.6570e-04_rb,2.6231e-03_rb,4.2518e-03_rb,5.6510e-03_rb,6.8860e-03_rb,8.0235e-03_rb, &
     &9.1475e-03_rb,1.0746e-02_rb,1.1185e-02_rb/)
      kao(:, 3, 3, 8) = (/ &
     &3.7255e-04_rb,2.6596e-03_rb,4.3088e-03_rb,5.7121e-03_rb,6.9479e-03_rb,8.0889e-03_rb, &
     &9.2308e-03_rb,1.0860e-02_rb,1.1321e-02_rb/)
      kao(:, 4, 3, 8) = (/ &
     &3.8155e-04_rb,2.7037e-03_rb,4.3688e-03_rb,5.7837e-03_rb,7.0219e-03_rb,8.1712e-03_rb, &
     &9.3303e-03_rb,1.0993e-02_rb,1.1473e-02_rb/)
      kao(:, 5, 3, 8) = (/ &
     &3.8777e-04_rb,2.7514e-03_rb,4.4348e-03_rb,5.8612e-03_rb,7.1061e-03_rb,8.2670e-03_rb, &
     &9.4452e-03_rb,1.1141e-02_rb,1.1658e-02_rb/)
      kao(:, 1, 4, 8) = (/ &
     &6.9620e-04_rb,2.9029e-03_rb,4.4649e-03_rb,5.7896e-03_rb,6.9152e-03_rb,7.8788e-03_rb, &
     &8.6335e-03_rb,9.2668e-03_rb,9.4651e-03_rb/)
      kao(:, 2, 4, 8) = (/ &
     &7.1019e-04_rb,2.9517e-03_rb,4.5263e-03_rb,5.8664e-03_rb,6.9912e-03_rb,7.9468e-03_rb, &
     &8.6984e-03_rb,9.3401e-03_rb,9.5623e-03_rb/)
      kao(:, 3, 4, 8) = (/ &
     &7.2359e-04_rb,3.0044e-03_rb,4.5979e-03_rb,5.9489e-03_rb,7.0830e-03_rb,8.0315e-03_rb, &
     &8.7768e-03_rb,9.4291e-03_rb,9.6859e-03_rb/)
      kao(:, 4, 4, 8) = (/ &
     &7.3828e-04_rb,3.0619e-03_rb,4.6789e-03_rb,6.0379e-03_rb,7.1842e-03_rb,8.1322e-03_rb, &
     &8.8743e-03_rb,9.5317e-03_rb,9.8248e-03_rb/)
      kao(:, 5, 4, 8) = (/ &
     &7.5535e-04_rb,3.1266e-03_rb,4.7671e-03_rb,6.1369e-03_rb,7.2972e-03_rb,8.2469e-03_rb, &
     &8.9884e-03_rb,9.6595e-03_rb,9.9955e-03_rb/)
      kao(:, 1, 5, 8) = (/ &
     &1.2367e-03_rb,3.3787e-03_rb,4.7869e-03_rb,5.9863e-03_rb,7.0094e-03_rb,7.8065e-03_rb, &
     &8.3893e-03_rb,8.6393e-03_rb,8.1050e-03_rb/)
      kao(:, 2, 5, 8) = (/ &
     &1.2606e-03_rb,3.4408e-03_rb,4.8727e-03_rb,6.0829e-03_rb,7.1090e-03_rb,7.9083e-03_rb, &
     &8.4736e-03_rb,8.7084e-03_rb,8.1896e-03_rb/)
      kao(:, 3, 5, 8) = (/ &
     &1.2858e-03_rb,3.5058e-03_rb,4.9695e-03_rb,6.1896e-03_rb,7.2184e-03_rb,8.0235e-03_rb, &
     &8.5795e-03_rb,8.7948e-03_rb,8.2945e-03_rb/)
      kao(:, 4, 5, 8) = (/ &
     &1.3144e-03_rb,3.5781e-03_rb,5.0734e-03_rb,6.3066e-03_rb,7.3381e-03_rb,8.1502e-03_rb, &
     &8.7050e-03_rb,8.8972e-03_rb,8.4098e-03_rb/)
      kao(:, 5, 5, 8) = (/ &
     &1.3469e-03_rb,3.6641e-03_rb,5.1833e-03_rb,6.4362e-03_rb,7.4701e-03_rb,8.2878e-03_rb, &
     &8.8414e-03_rb,9.0166e-03_rb,8.5483e-03_rb/)
      kao(:, 1, 6, 8) = (/ &
     &2.0401e-03_rb,4.1601e-03_rb,5.3408e-03_rb,6.2948e-03_rb,7.1082e-03_rb,7.7648e-03_rb, &
     &8.1607e-03_rb,8.2016e-03_rb,6.9653e-03_rb/)
      kao(:, 2, 6, 8) = (/ &
     &2.0844e-03_rb,4.2423e-03_rb,5.4455e-03_rb,6.4160e-03_rb,7.2313e-03_rb,7.8855e-03_rb, &
     &8.2759e-03_rb,8.2810e-03_rb,7.0331e-03_rb/)
      kao(:, 3, 6, 8) = (/ &
     &2.1282e-03_rb,4.3237e-03_rb,5.5575e-03_rb,6.5512e-03_rb,7.3708e-03_rb,8.0202e-03_rb, &
     &8.4033e-03_rb,8.3806e-03_rb,7.1162e-03_rb/)
      kao(:, 4, 6, 8) = (/ &
     &2.1756e-03_rb,4.4109e-03_rb,5.6792e-03_rb,6.6960e-03_rb,7.5224e-03_rb,8.1665e-03_rb, &
     &8.5397e-03_rb,8.4981e-03_rb,7.2093e-03_rb/)
      kao(:, 5, 6, 8) = (/ &
     &2.2255e-03_rb,4.5091e-03_rb,5.8068e-03_rb,6.8485e-03_rb,7.6856e-03_rb,8.3228e-03_rb, &
     &8.6864e-03_rb,8.6279e-03_rb,7.3203e-03_rb/)
      kao(:, 1, 7, 8) = (/ &
     &3.5433e-03_rb,5.7288e-03_rb,6.6788e-03_rb,7.3197e-03_rb,7.7679e-03_rb,8.0638e-03_rb, &
     &8.2036e-03_rb,7.9558e-03_rb,6.0055e-03_rb/)
      kao(:, 2, 7, 8) = (/ &
     &3.6269e-03_rb,5.8458e-03_rb,6.8139e-03_rb,7.4674e-03_rb,7.9220e-03_rb,8.2152e-03_rb, &
     &8.3417e-03_rb,8.0664e-03_rb,6.0619e-03_rb/)
      kao(:, 3, 7, 8) = (/ &
     &3.7095e-03_rb,5.9702e-03_rb,6.9487e-03_rb,7.6264e-03_rb,8.0952e-03_rb,8.3842e-03_rb, &
     &8.4920e-03_rb,8.1889e-03_rb,6.1273e-03_rb/)
      kao(:, 4, 7, 8) = (/ &
     &3.7971e-03_rb,6.0972e-03_rb,7.0950e-03_rb,7.7981e-03_rb,8.2789e-03_rb,8.5668e-03_rb, &
     &8.6571e-03_rb,8.3173e-03_rb,6.2019e-03_rb/)
      kao(:, 5, 7, 8) = (/ &
     &3.8894e-03_rb,6.2311e-03_rb,7.2602e-03_rb,7.9756e-03_rb,8.4749e-03_rb,8.7608e-03_rb, &
     &8.8292e-03_rb,8.4586e-03_rb,6.2926e-03_rb/)
      kao(:, 1, 8, 8) = (/ &
     &7.2384e-03_rb,9.3960e-03_rb,1.0099e-02_rb,1.0331e-02_rb,1.0208e-02_rb,9.8199e-03_rb, &
     &9.1852e-03_rb,8.2577e-03_rb,5.1301e-03_rb/)
      kao(:, 2, 8, 8) = (/ &
     &7.4182e-03_rb,9.6243e-03_rb,1.0349e-02_rb,1.0556e-02_rb,1.0419e-02_rb,1.0021e-02_rb, &
     &9.3680e-03_rb,8.3968e-03_rb,5.1752e-03_rb/)
      kao(:, 3, 8, 8) = (/ &
     &7.6043e-03_rb,9.8546e-03_rb,1.0598e-02_rb,1.0777e-02_rb,1.0633e-02_rb,1.0249e-02_rb, &
     &9.5676e-03_rb,8.5508e-03_rb,5.2269e-03_rb/)
      kao(:, 4, 8, 8) = (/ &
     &7.7980e-03_rb,1.0094e-02_rb,1.0836e-02_rb,1.1012e-02_rb,1.0870e-02_rb,1.0482e-02_rb, &
     &9.7866e-03_rb,8.7162e-03_rb,5.2872e-03_rb/)
      kao(:, 5, 8, 8) = (/ &
     &8.0005e-03_rb,1.0352e-02_rb,1.1093e-02_rb,1.1272e-02_rb,1.1127e-02_rb,1.0726e-02_rb, &
     &1.0012e-02_rb,8.8875e-03_rb,5.3638e-03_rb/)
      kao(:, 1, 9, 8) = (/ &
     &2.7897e-02_rb,2.8473e-02_rb,2.7594e-02_rb,2.5806e-02_rb,2.3354e-02_rb,2.0346e-02_rb, &
     &1.6782e-02_rb,1.2341e-02_rb,4.1787e-03_rb/)
      kao(:, 2, 9, 8) = (/ &
     &2.8638e-02_rb,2.9187e-02_rb,2.8303e-02_rb,2.6475e-02_rb,2.3949e-02_rb,2.0857e-02_rb, &
     &1.7181e-02_rb,1.2602e-02_rb,4.2128e-03_rb/)
      kao(:, 3, 9, 8) = (/ &
     &2.9413e-02_rb,2.9901e-02_rb,2.8994e-02_rb,2.7143e-02_rb,2.4571e-02_rb,2.1381e-02_rb, &
     &1.7560e-02_rb,1.2888e-02_rb,4.2655e-03_rb/)
      kao(:, 4, 9, 8) = (/ &
     &3.0228e-02_rb,3.0665e-02_rb,2.9734e-02_rb,2.7828e-02_rb,2.5188e-02_rb,2.1923e-02_rb, &
     &1.7970e-02_rb,1.3183e-02_rb,4.3190e-03_rb/)
      kao(:, 5, 9, 8) = (/ &
     &3.1084e-02_rb,3.1463e-02_rb,3.0494e-02_rb,2.8550e-02_rb,2.5844e-02_rb,2.2478e-02_rb, &
     &1.8412e-02_rb,1.3496e-02_rb,4.3806e-03_rb/)
      kao(:, 1,10, 8) = (/ &
     &1.1962e-01_rb,1.0878e-01_rb,9.8349e-02_rb,8.6745e-02_rb,7.3912e-02_rb,5.9973e-02_rb, &
     &4.4839e-02_rb,2.7943e-02_rb,4.7543e-03_rb/)
      kao(:, 2,10, 8) = (/ &
     &1.2315e-01_rb,1.1182e-01_rb,1.0110e-01_rb,8.9175e-02_rb,7.5937e-02_rb,6.1630e-02_rb, &
     &4.6110e-02_rb,2.8679e-02_rb,4.8196e-03_rb/)
      kao(:, 3,10, 8) = (/ &
     &1.2693e-01_rb,1.1500e-01_rb,1.0391e-01_rb,9.1586e-02_rb,7.7974e-02_rb,6.3317e-02_rb, &
     &4.7374e-02_rb,2.9424e-02_rb,4.8761e-03_rb/)
      kao(:, 4,10, 8) = (/ &
     &1.3084e-01_rb,1.1837e-01_rb,1.0689e-01_rb,9.4192e-02_rb,8.0184e-02_rb,6.5076e-02_rb, &
     &4.8672e-02_rb,3.0173e-02_rb,4.8766e-03_rb/)
      kao(:, 5,10, 8) = (/ &
     &1.3499e-01_rb,1.2195e-01_rb,1.1004e-01_rb,9.6906e-02_rb,8.2479e-02_rb,6.6930e-02_rb, &
     &4.9998e-02_rb,3.0984e-02_rb,4.9096e-03_rb/)
      kao(:, 1,11, 8) = (/ &
     &1.7854e-01_rb,1.6015e-01_rb,1.4269e-01_rb,1.2418e-01_rb,1.0441e-01_rb,8.3289e-02_rb, &
     &6.0955e-02_rb,3.6472e-02_rb,5.4769e-03_rb/)
      kao(:, 2,11, 8) = (/ &
     &1.8442e-01_rb,1.6524e-01_rb,1.4721e-01_rb,1.2815e-01_rb,1.0771e-01_rb,8.5980e-02_rb, &
     &6.2870e-02_rb,3.7580e-02_rb,5.4728e-03_rb/)
      kao(:, 3,11, 8) = (/ &
     &1.9077e-01_rb,1.7071e-01_rb,1.5197e-01_rb,1.3224e-01_rb,1.1114e-01_rb,8.8708e-02_rb, &
     &6.4824e-02_rb,3.8654e-02_rb,5.4615e-03_rb/)
      kao(:, 4,11, 8) = (/ &
     &1.9746e-01_rb,1.7645e-01_rb,1.5699e-01_rb,1.3658e-01_rb,1.1475e-01_rb,9.1549e-02_rb, &
     &6.6833e-02_rb,3.9764e-02_rb,5.4686e-03_rb/)
      kao(:, 5,11, 8) = (/ &
     &2.0452e-01_rb,1.8255e-01_rb,1.6232e-01_rb,1.4114e-01_rb,1.1852e-01_rb,9.4452e-02_rb, &
     &6.8909e-02_rb,4.0947e-02_rb,5.4706e-03_rb/)
      kao(:, 1,12, 8) = (/ &
     &2.0120e-01_rb,1.7983e-01_rb,1.5919e-01_rb,1.3760e-01_rb,1.1465e-01_rb,9.0568e-02_rb, &
     &6.5383e-02_rb,3.8355e-02_rb,5.7670e-03_rb/)
      kao(:, 2,12, 8) = (/ &
     &2.0757e-01_rb,1.8546e-01_rb,1.6435e-01_rb,1.4209e-01_rb,1.1843e-01_rb,9.3538e-02_rb, &
     &6.7515e-02_rb,3.9583e-02_rb,5.7182e-03_rb/)
      kao(:, 3,12, 8) = (/ &
     &2.1449e-01_rb,1.9151e-01_rb,1.6961e-01_rb,1.4673e-01_rb,1.2244e-01_rb,9.6759e-02_rb, &
     &6.9782e-02_rb,4.0857e-02_rb,5.7245e-03_rb/)
      kao(:, 4,12, 8) = (/ &
     &2.2222e-01_rb,1.9817e-01_rb,1.7542e-01_rb,1.5173e-01_rb,1.2653e-01_rb,1.0003e-01_rb, &
     &7.2145e-02_rb,4.2184e-02_rb,5.7102e-03_rb/)
      kao(:, 5,12, 8) = (/ &
     &2.3023e-01_rb,2.0507e-01_rb,1.8144e-01_rb,1.5689e-01_rb,1.3087e-01_rb,1.0351e-01_rb, &
     &7.4642e-02_rb,4.3620e-02_rb,5.7450e-03_rb/)
      kao(:, 1,13, 8) = (/ &
     &1.8448e-01_rb,1.6553e-01_rb,1.4649e-01_rb,1.2620e-01_rb,1.0471e-01_rb,8.2340e-02_rb, &
     &5.9042e-02_rb,3.4367e-02_rb,5.6219e-03_rb/)
      kao(:, 2,13, 8) = (/ &
     &1.9034e-01_rb,1.7056e-01_rb,1.5103e-01_rb,1.3022e-01_rb,1.0811e-01_rb,8.5094e-02_rb, &
     &6.1006e-02_rb,3.5479e-02_rb,5.5942e-03_rb/)
      kao(:, 3,13, 8) = (/ &
     &1.9658e-01_rb,1.7595e-01_rb,1.5589e-01_rb,1.3452e-01_rb,1.1170e-01_rb,8.7865e-02_rb, &
     &6.2984e-02_rb,3.6599e-02_rb,5.6072e-03_rb/)
      kao(:, 4,13, 8) = (/ &
     &2.0338e-01_rb,1.8188e-01_rb,1.6115e-01_rb,1.3898e-01_rb,1.1533e-01_rb,9.0760e-02_rb, &
     &6.5090e-02_rb,3.7803e-02_rb,5.6436e-03_rb/)
      kao(:, 5,13, 8) = (/ &
     &2.1087e-01_rb,1.8825e-01_rb,1.6661e-01_rb,1.4369e-01_rb,1.1939e-01_rb,9.3967e-02_rb, &
     &6.7400e-02_rb,3.9148e-02_rb,5.6745e-03_rb/)
      kao(:, 1, 1, 9) = (/ &
     &9.4421e-04_rb,7.3440e-03_rb,1.3185e-02_rb,1.9434e-02_rb,2.5910e-02_rb,3.2531e-02_rb, &
     &3.9377e-02_rb,4.6620e-02_rb,5.1571e-02_rb/)
      kao(:, 2, 1, 9) = (/ &
     &9.6482e-04_rb,7.2909e-03_rb,1.3109e-02_rb,1.9331e-02_rb,2.5776e-02_rb,3.2360e-02_rb, &
     &3.9164e-02_rb,4.6323e-02_rb,5.1303e-02_rb/)
      kao(:, 3, 1, 9) = (/ &
     &9.8476e-04_rb,7.2409e-03_rb,1.3044e-02_rb,1.9248e-02_rb,2.5665e-02_rb,3.2216e-02_rb, &
     &3.8981e-02_rb,4.6102e-02_rb,5.1082e-02_rb/)
      kao(:, 4, 1, 9) = (/ &
     &1.0020e-03_rb,7.1999e-03_rb,1.3010e-02_rb,1.9215e-02_rb,2.5618e-02_rb,3.2137e-02_rb, &
     &3.8862e-02_rb,4.5932e-02_rb,5.0990e-02_rb/)
      kao(:, 5, 1, 9) = (/ &
     &1.0184e-03_rb,7.1770e-03_rb,1.3015e-02_rb,1.9243e-02_rb,2.5654e-02_rb,3.2165e-02_rb, &
     &3.8858e-02_rb,4.5863e-02_rb,5.1066e-02_rb/)
      kao(:, 1, 2, 9) = (/ &
     &1.3462e-03_rb,8.0869e-03_rb,1.3393e-02_rb,1.9013e-02_rb,2.4987e-02_rb,3.1278e-02_rb, &
     &3.7900e-02_rb,4.5296e-02_rb,4.9469e-02_rb/)
      kao(:, 2, 2, 9) = (/ &
     &1.3803e-03_rb,8.0241e-03_rb,1.3304e-02_rb,1.8903e-02_rb,2.4859e-02_rb,3.1123e-02_rb, &
     &3.7707e-02_rb,4.5049e-02_rb,4.9230e-02_rb/)
      kao(:, 3, 2, 9) = (/ &
     &1.4130e-03_rb,7.9636e-03_rb,1.3235e-02_rb,1.8828e-02_rb,2.4777e-02_rb,3.1019e-02_rb, &
     &3.7558e-02_rb,4.4839e-02_rb,4.9085e-02_rb/)
      kao(:, 4, 2, 9) = (/ &
     &1.4417e-03_rb,7.9144e-03_rb,1.3198e-02_rb,1.8807e-02_rb,2.4768e-02_rb,3.1006e-02_rb, &
     &3.7513e-02_rb,4.4699e-02_rb,4.9090e-02_rb/)
      kao(:, 5, 2, 9) = (/ &
     &1.4672e-03_rb,7.8840e-03_rb,1.3197e-02_rb,1.8844e-02_rb,2.4841e-02_rb,3.1094e-02_rb, &
     &3.7591e-02_rb,4.4701e-02_rb,4.9253e-02_rb/)
      kao(:, 1, 3, 9) = (/ &
     &2.5556e-03_rb,1.0082e-02_rb,1.5320e-02_rb,1.9920e-02_rb,2.4710e-02_rb,2.9907e-02_rb, &
     &3.5684e-02_rb,4.2430e-02_rb,4.6609e-02_rb/)
      kao(:, 2, 3, 9) = (/ &
     &2.6284e-03_rb,1.0047e-02_rb,1.5209e-02_rb,1.9796e-02_rb,2.4582e-02_rb,2.9765e-02_rb, &
     &3.5524e-02_rb,4.2220e-02_rb,4.6433e-02_rb/)
      kao(:, 3, 3, 9) = (/ &
     &2.6960e-03_rb,1.0006e-02_rb,1.5105e-02_rb,1.9696e-02_rb,2.4496e-02_rb,2.9686e-02_rb, &
     &3.5446e-02_rb,4.2085e-02_rb,4.6365e-02_rb/)
      kao(:, 4, 3, 9) = (/ &
     &2.7589e-03_rb,9.9586e-03_rb,1.5035e-02_rb,1.9644e-02_rb,2.4475e-02_rb,2.9692e-02_rb, &
     &3.5469e-02_rb,4.2071e-02_rb,4.6440e-02_rb/)
      kao(:, 5, 3, 9) = (/ &
     &2.8129e-03_rb,9.9196e-03_rb,1.4994e-02_rb,1.9639e-02_rb,2.4519e-02_rb,2.9788e-02_rb, &
     &3.5597e-02_rb,4.2188e-02_rb,4.6642e-02_rb/)
      kao(:, 1, 4, 9) = (/ &
     &4.8937e-03_rb,1.3203e-02_rb,1.8921e-02_rb,2.3296e-02_rb,2.6924e-02_rb,3.0282e-02_rb, &
     &3.4096e-02_rb,3.9071e-02_rb,4.3251e-02_rb/)
      kao(:, 2, 4, 9) = (/ &
     &5.0488e-03_rb,1.3226e-02_rb,1.8867e-02_rb,2.3170e-02_rb,2.6777e-02_rb,3.0147e-02_rb, &
     &3.3963e-02_rb,3.8948e-02_rb,4.3159e-02_rb/)
      kao(:, 3, 4, 9) = (/ &
     &5.1910e-03_rb,1.3248e-02_rb,1.8795e-02_rb,2.3060e-02_rb,2.6655e-02_rb,3.0063e-02_rb, &
     &3.3903e-02_rb,3.8899e-02_rb,4.3165e-02_rb/)
      kao(:, 4, 4, 9) = (/ &
     &5.3207e-03_rb,1.3268e-02_rb,1.8730e-02_rb,2.2981e-02_rb,2.6589e-02_rb,3.0040e-02_rb, &
     &3.3930e-02_rb,3.8970e-02_rb,4.3287e-02_rb/)
      kao(:, 5, 4, 9) = (/ &
     &5.4422e-03_rb,1.3287e-02_rb,1.8693e-02_rb,2.2945e-02_rb,2.6576e-02_rb,3.0081e-02_rb, &
     &3.4041e-02_rb,3.9135e-02_rb,4.3512e-02_rb/)
      kao(:, 1, 5, 9) = (/ &
     &8.5307e-03_rb,1.7183e-02_rb,2.3465e-02_rb,2.7970e-02_rb,3.1130e-02_rb,3.3358e-02_rb, &
     &3.4774e-02_rb,3.6613e-02_rb,3.9589e-02_rb/)
      kao(:, 2, 5, 9) = (/ &
     &8.8245e-03_rb,1.7347e-02_rb,2.3508e-02_rb,2.7937e-02_rb,3.1047e-02_rb,3.3237e-02_rb, &
     &3.4670e-02_rb,3.6530e-02_rb,3.9570e-02_rb/)
      kao(:, 3, 5, 9) = (/ &
     &9.0927e-03_rb,1.7496e-02_rb,2.3535e-02_rb,2.7881e-02_rb,3.0966e-02_rb,3.3147e-02_rb, &
     &3.4603e-02_rb,3.6513e-02_rb,3.9628e-02_rb/)
      kao(:, 4, 5, 9) = (/ &
     &9.3371e-03_rb,1.7632e-02_rb,2.3559e-02_rb,2.7829e-02_rb,3.0920e-02_rb,3.3100e-02_rb, &
     &3.4594e-02_rb,3.6581e-02_rb,3.9792e-02_rb/)
      kao(:, 5, 5, 9) = (/ &
     &9.5713e-03_rb,1.7760e-02_rb,2.3602e-02_rb,2.7816e-02_rb,3.0915e-02_rb,3.3109e-02_rb, &
     &3.4648e-02_rb,3.6731e-02_rb,4.0039e-02_rb/)
      kao(:, 1, 6, 9) = (/ &
     &1.3837e-02_rb,2.2346e-02_rb,2.8803e-02_rb,3.3403e-02_rb,3.6375e-02_rb,3.7770e-02_rb, &
     &3.7769e-02_rb,3.6158e-02_rb,3.5736e-02_rb/)
      kao(:, 2, 6, 9) = (/ &
     &1.4345e-02_rb,2.2684e-02_rb,2.9062e-02_rb,3.3523e-02_rb,3.6415e-02_rb,3.7748e-02_rb, &
     &3.7698e-02_rb,3.6127e-02_rb,3.5788e-02_rb/)
      kao(:, 3, 6, 9) = (/ &
     &1.4814e-02_rb,2.3025e-02_rb,2.9297e-02_rb,3.3616e-02_rb,3.6414e-02_rb,3.7728e-02_rb, &
     &3.7657e-02_rb,3.6132e-02_rb,3.5900e-02_rb/)
      kao(:, 4, 6, 9) = (/ &
     &1.5241e-02_rb,2.3333e-02_rb,2.9502e-02_rb,3.3707e-02_rb,3.6417e-02_rb,3.7743e-02_rb, &
     &3.7671e-02_rb,3.6177e-02_rb,3.6086e-02_rb/)
      kao(:, 5, 6, 9) = (/ &
     &1.5643e-02_rb,2.3601e-02_rb,2.9688e-02_rb,3.3808e-02_rb,3.6468e-02_rb,3.7791e-02_rb, &
     &3.7729e-02_rb,3.6273e-02_rb,3.6334e-02_rb/)
      kao(:, 1, 7, 9) = (/ &
     &2.3687e-02_rb,3.1230e-02_rb,3.7445e-02_rb,4.1636e-02_rb,4.4049e-02_rb,4.4684e-02_rb, &
     &4.3161e-02_rb,3.8835e-02_rb,3.1866e-02_rb/)
      kao(:, 2, 7, 9) = (/ &
     &2.4610e-02_rb,3.1967e-02_rb,3.8037e-02_rb,4.2097e-02_rb,4.4354e-02_rb,4.4855e-02_rb, &
     &4.3236e-02_rb,3.8850e-02_rb,3.1969e-02_rb/)
      kao(:, 3, 7, 9) = (/ &
     &2.5467e-02_rb,3.2639e-02_rb,3.8612e-02_rb,4.2519e-02_rb,4.4609e-02_rb,4.4967e-02_rb, &
     &4.3302e-02_rb,3.8885e-02_rb,3.2129e-02_rb/)
      kao(:, 4, 7, 9) = (/ &
     &2.6267e-02_rb,3.3238e-02_rb,3.9137e-02_rb,4.2894e-02_rb,4.4845e-02_rb,4.5082e-02_rb, &
     &4.3395e-02_rb,3.8965e-02_rb,3.2326e-02_rb/)
      kao(:, 5, 7, 9) = (/ &
     &2.6995e-02_rb,3.3786e-02_rb,3.9598e-02_rb,4.3239e-02_rb,4.5071e-02_rb,4.5249e-02_rb, &
     &4.3527e-02_rb,3.9067e-02_rb,3.2557e-02_rb/)
      kao(:, 1, 8, 9) = (/ &
     &4.7920e-02_rb,5.2366e-02_rb,5.6749e-02_rb,5.9168e-02_rb,5.9712e-02_rb,5.8092e-02_rb, &
     &5.3946e-02_rb,4.5783e-02_rb,2.8113e-02_rb/)
      kao(:, 2, 8, 9) = (/ &
     &4.9884e-02_rb,5.3965e-02_rb,5.8166e-02_rb,6.0347e-02_rb,6.0665e-02_rb,5.8786e-02_rb, &
     &5.4351e-02_rb,4.5985e-02_rb,2.8254e-02_rb/)
      kao(:, 3, 8, 9) = (/ &
     &5.1719e-02_rb,5.5478e-02_rb,5.9448e-02_rb,6.1483e-02_rb,6.1565e-02_rb,5.9384e-02_rb, &
     &5.4686e-02_rb,4.6159e-02_rb,2.8435e-02_rb/)
      kao(:, 4, 8, 9) = (/ &
     &5.3394e-02_rb,5.6807e-02_rb,6.0645e-02_rb,6.2509e-02_rb,6.2363e-02_rb,5.9916e-02_rb, &
     &5.4999e-02_rb,4.6361e-02_rb,2.8611e-02_rb/)
      kao(:, 5, 8, 9) = (/ &
     &5.4964e-02_rb,5.7985e-02_rb,6.1688e-02_rb,6.3442e-02_rb,6.3043e-02_rb,6.0445e-02_rb, &
     &5.5342e-02_rb,4.6592e-02_rb,2.8833e-02_rb/)
      kao(:, 1, 9, 9) = (/ &
     &1.8387e-01_rb,1.6808e-01_rb,1.5799e-01_rb,1.4671e-01_rb,1.3366e-01_rb,1.1828e-01_rb, &
     &9.9590e-02_rb,7.4670e-02_rb,2.3960e-02_rb/)
      kao(:, 2, 9, 9) = (/ &
     &1.9171e-01_rb,1.7460e-01_rb,1.6351e-01_rb,1.5144e-01_rb,1.3758e-01_rb,1.2130e-01_rb, &
     &1.0166e-01_rb,7.5740e-02_rb,2.4137e-02_rb/)
      kao(:, 3, 9, 9) = (/ &
     &1.9894e-01_rb,1.8065e-01_rb,1.6862e-01_rb,1.5587e-01_rb,1.4113e-01_rb,1.2411e-01_rb, &
     &1.0364e-01_rb,7.6719e-02_rb,2.4294e-02_rb/)
      kao(:, 4, 9, 9) = (/ &
     &2.0563e-01_rb,1.8623e-01_rb,1.7319e-01_rb,1.5976e-01_rb,1.4435e-01_rb,1.2672e-01_rb, &
     &1.0542e-01_rb,7.7598e-02_rb,2.4446e-02_rb/)
      kao(:, 5, 9, 9) = (/ &
     &2.1200e-01_rb,1.9152e-01_rb,1.7746e-01_rb,1.6330e-01_rb,1.4720e-01_rb,1.2898e-01_rb, &
     &1.0699e-01_rb,7.8411e-02_rb,2.4664e-02_rb/)
      kao(:, 1,10, 9) = (/ &
     &7.9805e-01_rb,7.0103e-01_rb,6.0984e-01_rb,5.2274e-01_rb,4.3812e-01_rb,3.5220e-01_rb, &
     &2.6270e-01_rb,1.6632e-01_rb,1.8620e-02_rb/)
      kao(:, 2,10, 9) = (/ &
     &8.3214e-01_rb,7.3068e-01_rb,6.3476e-01_rb,5.4295e-01_rb,4.5426e-01_rb,3.6461e-01_rb, &
     &2.7128e-01_rb,1.7099e-01_rb,1.8699e-02_rb/)
      kao(:, 3,10, 9) = (/ &
     &8.6405e-01_rb,7.5844e-01_rb,6.5814e-01_rb,5.6198e-01_rb,4.6922e-01_rb,3.7620e-01_rb, &
     &2.7918e-01_rb,1.7530e-01_rb,1.8761e-02_rb/)
      kao(:, 4,10, 9) = (/ &
     &8.9399e-01_rb,7.8448e-01_rb,6.8005e-01_rb,5.7978e-01_rb,4.8302e-01_rb,3.8668e-01_rb, &
     &2.8631e-01_rb,1.7930e-01_rb,1.8825e-02_rb/)
      kao(:, 5,10, 9) = (/ &
     &9.2229e-01_rb,8.0909e-01_rb,7.0076e-01_rb,5.9657e-01_rb,4.9598e-01_rb,3.9642e-01_rb, &
     &2.9286e-01_rb,1.8285e-01_rb,1.8959e-02_rb/)
      kao(:, 1,11, 9) = (/ &
     &1.2376e+00_rb,1.0851e+00_rb,9.3781e-01_rb,7.9479e-01_rb,6.5536e-01_rb,5.1636e-01_rb, &
     &3.7350e-01_rb,2.2328e-01_rb,1.9046e-02_rb/)
      kao(:, 2,11, 9) = (/ &
     &1.2888e+00_rb,1.1296e+00_rb,9.7557e-01_rb,8.2566e-01_rb,6.7951e-01_rb,5.3467e-01_rb, &
     &3.8601e-01_rb,2.3011e-01_rb,1.9117e-02_rb/)
      kao(:, 3,11, 9) = (/ &
     &1.3372e+00_rb,1.1719e+00_rb,1.0114e+00_rb,8.5500e-01_rb,7.0234e-01_rb,5.5187e-01_rb, &
     &3.9765e-01_rb,2.3639e-01_rb,1.9165e-02_rb/)
      kao(:, 4,11, 9) = (/ &
     &1.3819e+00_rb,1.2109e+00_rb,1.0444e+00_rb,8.8194e-01_rb,7.2332e-01_rb,5.6752e-01_rb, &
     &4.0827e-01_rb,2.4219e-01_rb,1.9058e-02_rb/)
      kao(:, 5,11, 9) = (/ &
     &1.4259e+00_rb,1.2494e+00_rb,1.0769e+00_rb,9.0854e-01_rb,7.4408e-01_rb,5.8286e-01_rb, &
     &4.1859e-01_rb,2.4748e-01_rb,1.9067e-02_rb/)
      kao(:, 1,12, 9) = (/ &
     &1.4243e+00_rb,1.2488e+00_rb,1.0789e+00_rb,9.1317e-01_rb,7.5092e-01_rb,5.8812e-01_rb, &
     &4.2112e-01_rb,2.4611e-01_rb,1.9872e-02_rb/)
      kao(:, 2,12, 9) = (/ &
     &1.4836e+00_rb,1.3004e+00_rb,1.1224e+00_rb,9.4876e-01_rb,7.7876e-01_rb,6.0917e-01_rb, &
     &4.3528e-01_rb,2.5376e-01_rb,2.0064e-02_rb/)
      kao(:, 3,12, 9) = (/ &
     &1.5386e+00_rb,1.3484e+00_rb,1.1631e+00_rb,9.8199e-01_rb,8.0461e-01_rb,6.2868e-01_rb, &
     &4.4843e-01_rb,2.6085e-01_rb,2.0179e-02_rb/)
      kao(:, 4,12, 9) = (/ &
     &1.5909e+00_rb,1.3940e+00_rb,1.2017e+00_rb,1.0136e+00_rb,8.2931e-01_rb,6.4713e-01_rb, &
     &4.6088e-01_rb,2.6745e-01_rb,2.0157e-02_rb/)
      kao(:, 5,12, 9) = (/ &
     &1.6423e+00_rb,1.4388e+00_rb,1.2396e+00_rb,1.0446e+00_rb,8.5343e-01_rb,6.6506e-01_rb, &
     &4.7297e-01_rb,2.7367e-01_rb,2.0135e-02_rb/)
      kao(:, 1,13, 9) = (/ &
     &1.3090e+00_rb,1.1487e+00_rb,9.9498e-01_rb,8.4529e-01_rb,6.9751e-01_rb,5.4690e-01_rb, &
     &3.9201e-01_rb,2.2831e-01_rb,1.9648e-02_rb/)
      kao(:, 2,13, 9) = (/ &
     &1.3642e+00_rb,1.1967e+00_rb,1.0354e+00_rb,8.7822e-01_rb,7.2355e-01_rb,5.6643e-01_rb, &
     &4.0528e-01_rb,2.3543e-01_rb,1.9783e-02_rb/)
      kao(:, 3,13, 9) = (/ &
     &1.4156e+00_rb,1.2415e+00_rb,1.0731e+00_rb,9.0892e-01_rb,7.4783e-01_rb,5.8491e-01_rb, &
     &4.1788e-01_rb,2.4217e-01_rb,1.9878e-02_rb/)
      kao(:, 4,13, 9) = (/ &
     &1.4652e+00_rb,1.2846e+00_rb,1.1094e+00_rb,9.3863e-01_rb,7.7135e-01_rb,6.0268e-01_rb, &
     &4.2988e-01_rb,2.4851e-01_rb,1.9864e-02_rb/)
      kao(:, 5,13, 9) = (/ &
     &1.5126e+00_rb,1.3260e+00_rb,1.1446e+00_rb,9.6741e-01_rb,7.9378e-01_rb,6.1968e-01_rb, &
     &4.4123e-01_rb,2.5440e-01_rb,1.9877e-02_rb/)
      kao(:, 1, 1,10) = (/ &
     &3.3844e-03_rb,1.5864e-02_rb,3.0967e-02_rb,4.5960e-02_rb,6.0759e-02_rb,7.5076e-02_rb, &
     &8.8299e-02_rb,9.7840e-02_rb,1.2109e-01_rb/)
      kao(:, 2, 1,10) = (/ &
     &3.3774e-03_rb,1.5840e-02_rb,3.0916e-02_rb,4.5912e-02_rb,6.0698e-02_rb,7.5010e-02_rb, &
     &8.8211e-02_rb,9.7731e-02_rb,1.2099e-01_rb/)
      kao(:, 3, 1,10) = (/ &
     &3.3575e-03_rb,1.5844e-02_rb,3.0912e-02_rb,4.5937e-02_rb,6.0728e-02_rb,7.5022e-02_rb, &
     &8.8124e-02_rb,9.7474e-02_rb,1.2106e-01_rb/)
      kao(:, 4, 1,10) = (/ &
     &3.3333e-03_rb,1.5824e-02_rb,3.0829e-02_rb,4.5838e-02_rb,6.0646e-02_rb,7.4990e-02_rb, &
     &8.8084e-02_rb,9.7173e-02_rb,1.2089e-01_rb/)
      kao(:, 5, 1,10) = (/ &
     &3.3084e-03_rb,1.5776e-02_rb,3.0693e-02_rb,4.5649e-02_rb,6.0428e-02_rb,7.4773e-02_rb, &
     &8.7909e-02_rb,9.6839e-02_rb,1.2042e-01_rb/)
      kao(:, 1, 2,10) = (/ &
     &5.1392e-03_rb,1.7464e-02_rb,3.2432e-02_rb,4.7915e-02_rb,6.3487e-02_rb,7.8847e-02_rb, &
     &9.3621e-02_rb,1.0600e-01_rb,1.2635e-01_rb/)
      kao(:, 2, 2,10) = (/ &
     &5.1325e-03_rb,1.7471e-02_rb,3.2464e-02_rb,4.7934e-02_rb,6.3521e-02_rb,7.8889e-02_rb, &
     &9.3606e-02_rb,1.0592e-01_rb,1.2642e-01_rb/)
      kao(:, 3, 2,10) = (/ &
     &5.1157e-03_rb,1.7466e-02_rb,3.2343e-02_rb,4.7787e-02_rb,6.3340e-02_rb,7.8722e-02_rb, &
     &9.3575e-02_rb,1.0588e-01_rb,1.2605e-01_rb/)
      kao(:, 4, 2,10) = (/ &
     &5.0987e-03_rb,1.7399e-02_rb,3.2075e-02_rb,4.7428e-02_rb,6.2893e-02_rb,7.8221e-02_rb, &
     &9.3145e-02_rb,1.0589e-01_rb,1.2513e-01_rb/)
      kao(:, 5, 2,10) = (/ &
     &5.0781e-03_rb,1.7287e-02_rb,3.1803e-02_rb,4.7068e-02_rb,6.2420e-02_rb,7.7655e-02_rb, &
     &9.2528e-02_rb,1.0558e-01_rb,1.2418e-01_rb/)
      kao(:, 1, 3,10) = (/ &
     &1.0449e-02_rb,2.2920e-02_rb,3.5229e-02_rb,4.9841e-02_rb,6.5300e-02_rb,8.0993e-02_rb, &
     &9.6715e-02_rb,1.1178e-01_rb,1.2900e-01_rb/)
      kao(:, 2, 3,10) = (/ &
     &1.0475e-02_rb,2.2840e-02_rb,3.5278e-02_rb,4.9881e-02_rb,6.5245e-02_rb,8.1037e-02_rb, &
     &9.6863e-02_rb,1.1212e-01_rb,1.2899e-01_rb/)
      kao(:, 3, 3,10) = (/ &
     &1.0487e-02_rb,2.2756e-02_rb,3.5213e-02_rb,4.9645e-02_rb,6.4866e-02_rb,8.0649e-02_rb, &
     &9.6451e-02_rb,1.1190e-01_rb,1.2830e-01_rb/)
      kao(:, 4, 3,10) = (/ &
     &1.0493e-02_rb,2.2711e-02_rb,3.4996e-02_rb,4.9204e-02_rb,6.4312e-02_rb,7.9994e-02_rb, &
     &9.5693e-02_rb,1.1109e-01_rb,1.2723e-01_rb/)
      kao(:, 5, 3,10) = (/ &
     &1.0487e-02_rb,2.2599e-02_rb,3.4737e-02_rb,4.8786e-02_rb,6.3856e-02_rb,7.9443e-02_rb, &
     &9.5024e-02_rb,1.1021e-01_rb,1.2638e-01_rb/)
      kao(:, 1, 4,10) = (/ &
     &2.1474e-02_rb,3.4011e-02_rb,4.5615e-02_rb,5.6742e-02_rb,6.8329e-02_rb,8.2791e-02_rb, &
     &9.7759e-02_rb,1.1367e-01_rb,1.2934e-01_rb/)
      kao(:, 2, 4,10) = (/ &
     &2.1593e-02_rb,3.3751e-02_rb,4.5521e-02_rb,5.6645e-02_rb,6.8211e-02_rb,8.2434e-02_rb, &
     &9.7517e-02_rb,1.1348e-01_rb,1.2896e-01_rb/)
      kao(:, 3, 4,10) = (/ &
     &2.1694e-02_rb,3.3429e-02_rb,4.5406e-02_rb,5.6399e-02_rb,6.7862e-02_rb,8.1741e-02_rb, &
     &9.6866e-02_rb,1.1284e-01_rb,1.2803e-01_rb/)
      kao(:, 4, 4,10) = (/ &
     &2.1761e-02_rb,3.3088e-02_rb,4.5200e-02_rb,5.5990e-02_rb,6.7313e-02_rb,8.0951e-02_rb, &
     &9.6103e-02_rb,1.1192e-01_rb,1.2712e-01_rb/)
      kao(:, 5, 4,10) = (/ &
     &2.1871e-02_rb,3.2779e-02_rb,4.4802e-02_rb,5.5607e-02_rb,6.6864e-02_rb,8.0377e-02_rb, &
     &9.5554e-02_rb,1.1124e-01_rb,1.2647e-01_rb/)
      kao(:, 1, 5,10) = (/ &
     &3.9770e-02_rb,5.1894e-02_rb,6.3327e-02_rb,7.2180e-02_rb,8.0917e-02_rb,8.8056e-02_rb, &
     &9.8598e-02_rb,1.1206e-01_rb,1.2671e-01_rb/)
      kao(:, 2, 5,10) = (/ &
     &4.0140e-02_rb,5.1409e-02_rb,6.3212e-02_rb,7.2060e-02_rb,8.0506e-02_rb,8.7647e-02_rb, &
     &9.8026e-02_rb,1.1163e-01_rb,1.2617e-01_rb/)
      kao(:, 3, 5,10) = (/ &
     &4.0492e-02_rb,5.0925e-02_rb,6.2882e-02_rb,7.1952e-02_rb,8.0008e-02_rb,8.6965e-02_rb, &
     &9.7225e-02_rb,1.1094e-01_rb,1.2542e-01_rb/)
      kao(:, 4, 5,10) = (/ &
     &4.0826e-02_rb,5.0532e-02_rb,6.2408e-02_rb,7.1612e-02_rb,7.9337e-02_rb,8.6238e-02_rb, &
     &9.6327e-02_rb,1.1019e-01_rb,1.2470e-01_rb/)
      kao(:, 5, 5,10) = (/ &
     &4.1162e-02_rb,5.0261e-02_rb,6.1890e-02_rb,7.0980e-02_rb,7.8777e-02_rb,8.5713e-02_rb, &
     &9.5778e-02_rb,1.0986e-01_rb,1.2449e-01_rb/)
      kao(:, 1, 6,10) = (/ &
     &6.7916e-02_rb,7.7649e-02_rb,8.9172e-02_rb,9.6900e-02_rb,1.0149e-01_rb,1.0503e-01_rb, &
     &1.0523e-01_rb,1.0994e-01_rb,1.2156e-01_rb/)
      kao(:, 2, 6,10) = (/ &
     &6.8757e-02_rb,7.7814e-02_rb,8.8891e-02_rb,9.7030e-02_rb,1.0144e-01_rb,1.0468e-01_rb, &
     &1.0479e-01_rb,1.0906e-01_rb,1.2102e-01_rb/)
      kao(:, 3, 6,10) = (/ &
     &6.9572e-02_rb,7.7629e-02_rb,8.8407e-02_rb,9.6716e-02_rb,1.0124e-01_rb,1.0403e-01_rb, &
     &1.0408e-01_rb,1.0810e-01_rb,1.2040e-01_rb/)
      kao(:, 4, 6,10) = (/ &
     &7.0420e-02_rb,7.7345e-02_rb,8.8011e-02_rb,9.6134e-02_rb,1.0079e-01_rb,1.0305e-01_rb, &
     &1.0321e-01_rb,1.0749e-01_rb,1.2017e-01_rb/)
      kao(:, 5, 6,10) = (/ &
     &7.1176e-02_rb,7.7213e-02_rb,8.7761e-02_rb,9.5353e-02_rb,9.9879e-02_rb,1.0215e-01_rb, &
     &1.0265e-01_rb,1.0733e-01_rb,1.2038e-01_rb/)
      kao(:, 1, 7,10) = (/ &
     &1.2127e-01_rb,1.2412e-01_rb,1.3217e-01_rb,1.3765e-01_rb,1.3951e-01_rb,1.3625e-01_rb, &
     &1.2814e-01_rb,1.1487e-01_rb,1.1426e-01_rb/)
      kao(:, 2, 7,10) = (/ &
     &1.2309e-01_rb,1.2453e-01_rb,1.3252e-01_rb,1.3798e-01_rb,1.3976e-01_rb,1.3644e-01_rb, &
     &1.2796e-01_rb,1.1426e-01_rb,1.1391e-01_rb/)
      kao(:, 3, 7,10) = (/ &
     &1.2489e-01_rb,1.2494e-01_rb,1.3228e-01_rb,1.3764e-01_rb,1.3951e-01_rb,1.3625e-01_rb, &
     &1.2756e-01_rb,1.1350e-01_rb,1.1349e-01_rb/)
      kao(:, 4, 7,10) = (/ &
     &1.2680e-01_rb,1.2557e-01_rb,1.3198e-01_rb,1.3739e-01_rb,1.3886e-01_rb,1.3552e-01_rb, &
     &1.2649e-01_rb,1.1282e-01_rb,1.1362e-01_rb/)
      kao(:, 5, 7,10) = (/ &
     &1.2852e-01_rb,1.2615e-01_rb,1.3199e-01_rb,1.3726e-01_rb,1.3802e-01_rb,1.3430e-01_rb, &
     &1.2550e-01_rb,1.1257e-01_rb,1.1419e-01_rb/)
      kao(:, 1, 8,10) = (/ &
     &2.5214e-01_rb,2.3134e-01_rb,2.3152e-01_rb,2.2506e-01_rb,2.1543e-01_rb,2.0242e-01_rb, &
     &1.8079e-01_rb,1.4566e-01_rb,1.0565e-01_rb/)
      kao(:, 2, 8,10) = (/ &
     &2.5695e-01_rb,2.3363e-01_rb,2.3275e-01_rb,2.2604e-01_rb,2.1637e-01_rb,2.0282e-01_rb, &
     &1.8105e-01_rb,1.4554e-01_rb,1.0546e-01_rb/)
      kao(:, 3, 8,10) = (/ &
     &2.6176e-01_rb,2.3588e-01_rb,2.3395e-01_rb,2.2621e-01_rb,2.1636e-01_rb,2.0257e-01_rb, &
     &1.8076e-01_rb,1.4513e-01_rb,1.0533e-01_rb/)
      kao(:, 4, 8,10) = (/ &
     &2.6651e-01_rb,2.3871e-01_rb,2.3539e-01_rb,2.2647e-01_rb,2.1633e-01_rb,2.0229e-01_rb, &
     &1.8002e-01_rb,1.4419e-01_rb,1.0597e-01_rb/)
      kao(:, 5, 8,10) = (/ &
     &2.7042e-01_rb,2.4131e-01_rb,2.3677e-01_rb,2.2700e-01_rb,2.1683e-01_rb,2.0167e-01_rb, &
     &1.7891e-01_rb,1.4342e-01_rb,1.0666e-01_rb/)
      kao(:, 1, 9,10) = (/ &
     &9.7736e-01_rb,8.5726e-01_rb,7.4530e-01_rb,6.6330e-01_rb,5.7895e-01_rb,4.8711e-01_rb, &
     &3.8944e-01_rb,2.7986e-01_rb,9.6120e-02_rb/)
      kao(:, 2, 9,10) = (/ &
     &1.0005e+00_rb,8.7740e-01_rb,7.5935e-01_rb,6.7173e-01_rb,5.8495e-01_rb,4.9078e-01_rb, &
     &3.9206e-01_rb,2.8024e-01_rb,9.6028e-02_rb/)
      kao(:, 3, 9,10) = (/ &
     &1.0238e+00_rb,8.9765e-01_rb,7.7412e-01_rb,6.7963e-01_rb,5.9056e-01_rb,4.9392e-01_rb, &
     &3.9381e-01_rb,2.7971e-01_rb,9.6387e-02_rb/)
      kao(:, 4, 9,10) = (/ &
     &1.0459e+00_rb,9.1704e-01_rb,7.8954e-01_rb,6.8917e-01_rb,5.9755e-01_rb,4.9657e-01_rb, &
     &3.9497e-01_rb,2.7927e-01_rb,9.7373e-02_rb/)
      kao(:, 5, 9,10) = (/ &
     &1.0630e+00_rb,9.3201e-01_rb,8.0170e-01_rb,6.9634e-01_rb,6.0306e-01_rb,4.9955e-01_rb, &
     &3.9651e-01_rb,2.7910e-01_rb,9.7962e-02_rb/)
      kao(:, 1,10,10) = (/ &
     &4.2057e+00_rb,3.6817e+00_rb,3.1579e+00_rb,2.6346e+00_rb,2.1198e+00_rb,1.6450e+00_rb, &
     &1.1902e+00_rb,7.0817e-01_rb,7.8303e-02_rb/)
      kao(:, 2,10,10) = (/ &
     &4.3277e+00_rb,3.7881e+00_rb,3.2491e+00_rb,2.7104e+00_rb,2.1754e+00_rb,1.6774e+00_rb, &
     &1.2094e+00_rb,7.1654e-01_rb,7.8798e-02_rb/)
      kao(:, 3,10,10) = (/ &
     &4.4500e+00_rb,3.8956e+00_rb,3.3411e+00_rb,2.7869e+00_rb,2.2338e+00_rb,1.7120e+00_rb, &
     &1.2289e+00_rb,7.2324e-01_rb,7.8857e-02_rb/)
      kao(:, 4,10,10) = (/ &
     &4.5562e+00_rb,3.9883e+00_rb,3.4205e+00_rb,2.8532e+00_rb,2.2862e+00_rb,1.7446e+00_rb, &
     &1.2492e+00_rb,7.3045e-01_rb,7.9497e-02_rb/)
      kao(:, 5,10,10) = (/ &
     &4.6429e+00_rb,4.0644e+00_rb,3.4858e+00_rb,2.9077e+00_rb,2.3298e+00_rb,1.7714e+00_rb, &
     &1.2652e+00_rb,7.3636e-01_rb,7.8315e-02_rb/)
      kao(:, 1,11,10) = (/ &
     &6.3643e+00_rb,5.5702e+00_rb,4.7761e+00_rb,3.9823e+00_rb,3.1901e+00_rb,2.4267e+00_rb, &
     &1.7139e+00_rb,9.8175e-01_rb,5.2396e-02_rb/)
      kao(:, 2,11,10) = (/ &
     &6.5707e+00_rb,5.7511e+00_rb,4.9312e+00_rb,4.1117e+00_rb,3.2928e+00_rb,2.4948e+00_rb, &
     &1.7532e+00_rb,9.9614e-01_rb,5.2761e-02_rb/)
      kao(:, 3,11,10) = (/ &
     &6.7598e+00_rb,5.9160e+00_rb,5.0729e+00_rb,4.2301e+00_rb,3.3876e+00_rb,2.5590e+00_rb, &
     &1.7914e+00_rb,1.0123e+00_rb,5.3574e-02_rb/)
      kao(:, 4,11,10) = (/ &
     &6.9211e+00_rb,6.0577e+00_rb,5.1942e+00_rb,4.3313e+00_rb,3.4687e+00_rb,2.6150e+00_rb, &
     &1.8227e+00_rb,1.0262e+00_rb,5.6608e-02_rb/)
      kao(:, 5,11,10) = (/ &
     &7.0466e+00_rb,6.1673e+00_rb,5.2885e+00_rb,4.4096e+00_rb,3.5315e+00_rb,2.6584e+00_rb, &
     &1.8463e+00_rb,1.0382e+00_rb,5.6583e-02_rb/)
      kao(:, 1,12,10) = (/ &
     &7.1591e+00_rb,6.2650e+00_rb,5.3715e+00_rb,4.4783e+00_rb,3.5872e+00_rb,2.7305e+00_rb, &
     &1.9173e+00_rb,1.0829e+00_rb,5.4364e-02_rb/)
      kao(:, 2,12,10) = (/ &
     &7.3899e+00_rb,6.4678e+00_rb,5.5455e+00_rb,4.6234e+00_rb,3.7023e+00_rb,2.8086e+00_rb, &
     &1.9670e+00_rb,1.1039e+00_rb,5.3004e-02_rb/)
      kao(:, 3,12,10) = (/ &
     &7.5932e+00_rb,6.6457e+00_rb,5.6978e+00_rb,4.7509e+00_rb,3.8044e+00_rb,2.8774e+00_rb, &
     &2.0093e+00_rb,1.1228e+00_rb,5.1959e-02_rb/)
      kao(:, 4,12,10) = (/ &
     &7.7612e+00_rb,6.7925e+00_rb,5.8241e+00_rb,4.8562e+00_rb,3.8890e+00_rb,2.9344e+00_rb, &
     &2.0429e+00_rb,1.1393e+00_rb,5.3174e-02_rb/)
      kao(:, 5,12,10) = (/ &
     &7.8969e+00_rb,6.9118e+00_rb,5.9263e+00_rb,4.9415e+00_rb,3.9571e+00_rb,2.9804e+00_rb, &
     &2.0685e+00_rb,1.1516e+00_rb,5.1704e-02_rb/)
      kao(:, 1,13,10) = (/ &
     &6.4705e+00_rb,5.6627e+00_rb,4.8551e+00_rb,4.0480e+00_rb,3.2529e+00_rb,2.5042e+00_rb, &
     &1.7607e+00_rb,9.9775e-01_rb,5.2819e-02_rb/)
      kao(:, 2,13,10) = (/ &
     &6.6649e+00_rb,5.8328e+00_rb,5.0012e+00_rb,4.1701e+00_rb,3.3469e+00_rb,2.5698e+00_rb, &
     &1.8026e+00_rb,1.0186e+00_rb,5.2318e-02_rb/)
      kao(:, 3,13,10) = (/ &
     &6.8320e+00_rb,5.9796e+00_rb,5.1272e+00_rb,4.2755e+00_rb,3.4276e+00_rb,2.6233e+00_rb, &
     &1.8383e+00_rb,1.0369e+00_rb,5.1248e-02_rb/)
      kao(:, 4,13,10) = (/ &
     &6.9674e+00_rb,6.0982e+00_rb,5.2289e+00_rb,4.3604e+00_rb,3.4933e+00_rb,2.6647e+00_rb, &
     &1.8662e+00_rb,1.0505e+00_rb,5.2356e-02_rb/)
      kao(:, 5,13,10) = (/ &
     &7.0874e+00_rb,6.2027e+00_rb,5.3188e+00_rb,4.4355e+00_rb,3.5528e+00_rb,2.7014e+00_rb, &
     &1.8899e+00_rb,1.0614e+00_rb,5.1111e-02_rb/)
      kao(:, 1, 1,11) = (/ &
     &4.6742e-03_rb,2.0279e-02_rb,3.9426e-02_rb,5.8543e-02_rb,7.7272e-02_rb,9.5241e-02_rb, &
     &1.1148e-01_rb,1.2235e-01_rb,1.5408e-01_rb/)
      kao(:, 2, 1,11) = (/ &
     &4.6073e-03_rb,2.0296e-02_rb,3.9412e-02_rb,5.8523e-02_rb,7.7232e-02_rb,9.5132e-02_rb, &
     &1.1121e-01_rb,1.2184e-01_rb,1.5399e-01_rb/)
      kao(:, 3, 1,11) = (/ &
     &4.5565e-03_rb,2.0227e-02_rb,3.9298e-02_rb,5.8356e-02_rb,7.7038e-02_rb,9.4961e-02_rb, &
     &1.1109e-01_rb,1.2151e-01_rb,1.5359e-01_rb/)
      kao(:, 4, 1,11) = (/ &
     &4.5047e-03_rb,2.0130e-02_rb,3.9180e-02_rb,5.8173e-02_rb,7.6772e-02_rb,9.4618e-02_rb, &
     &1.1074e-01_rb,1.2134e-01_rb,1.5306e-01_rb/)
      kao(:, 5, 1,11) = (/ &
     &4.4540e-03_rb,2.0027e-02_rb,3.9084e-02_rb,5.8025e-02_rb,7.6535e-02_rb,9.4251e-02_rb, &
     &1.1025e-01_rb,1.2116e-01_rb,1.5260e-01_rb/)
      kao(:, 1, 2,11) = (/ &
     &7.4064e-03_rb,2.2194e-02_rb,4.1767e-02_rb,6.1911e-02_rb,8.1938e-02_rb,1.0155e-01_rb, &
     &1.2014e-01_rb,1.3485e-01_rb,1.6319e-01_rb/)
      kao(:, 2, 2,11) = (/ &
     &7.3108e-03_rb,2.2143e-02_rb,4.1684e-02_rb,6.1877e-02_rb,8.1902e-02_rb,1.0152e-01_rb, &
     &1.2013e-01_rb,1.3472e-01_rb,1.6312e-01_rb/)
      kao(:, 3, 2,11) = (/ &
     &7.2232e-03_rb,2.2079e-02_rb,4.1677e-02_rb,6.1891e-02_rb,8.1918e-02_rb,1.0151e-01_rb, &
     &1.1999e-01_rb,1.3454e-01_rb,1.6316e-01_rb/)
      kao(:, 4, 2,11) = (/ &
     &7.1304e-03_rb,2.2033e-02_rb,4.1736e-02_rb,6.1998e-02_rb,8.2035e-02_rb,1.0158e-01_rb, &
     &1.1988e-01_rb,1.3411e-01_rb,1.6341e-01_rb/)
      kao(:, 5, 2,11) = (/ &
     &7.0357e-03_rb,2.1988e-02_rb,4.1742e-02_rb,6.2041e-02_rb,8.2077e-02_rb,1.0158e-01_rb, &
     &1.1983e-01_rb,1.3366e-01_rb,1.6350e-01_rb/)
      kao(:, 1, 3,11) = (/ &
     &1.5774e-02_rb,2.9169e-02_rb,4.5962e-02_rb,6.5827e-02_rb,8.6496e-02_rb,1.0742e-01_rb, &
     &1.2791e-01_rb,1.4653e-01_rb,1.7156e-01_rb/)
      kao(:, 2, 3,11) = (/ &
     &1.5614e-02_rb,2.8989e-02_rb,4.5846e-02_rb,6.5612e-02_rb,8.6438e-02_rb,1.0734e-01_rb, &
     &1.2776e-01_rb,1.4623e-01_rb,1.7149e-01_rb/)
      kao(:, 3, 3,11) = (/ &
     &1.5422e-02_rb,2.8825e-02_rb,4.5777e-02_rb,6.5531e-02_rb,8.6532e-02_rb,1.0747e-01_rb, &
     &1.2790e-01_rb,1.4632e-01_rb,1.7170e-01_rb/)
      kao(:, 4, 3,11) = (/ &
     &1.5220e-02_rb,2.8664e-02_rb,4.5718e-02_rb,6.5531e-02_rb,8.6682e-02_rb,1.0768e-01_rb, &
     &1.2815e-01_rb,1.4661e-01_rb,1.7201e-01_rb/)
      kao(:, 5, 3,11) = (/ &
     &1.5041e-02_rb,2.8491e-02_rb,4.5554e-02_rb,6.5387e-02_rb,8.6561e-02_rb,1.0758e-01_rb, &
     &1.2814e-01_rb,1.4679e-01_rb,1.7174e-01_rb/)
      kao(:, 1, 4,11) = (/ &
     &3.3773e-02_rb,4.4438e-02_rb,5.9281e-02_rb,7.3962e-02_rb,9.2405e-02_rb,1.1202e-01_rb, &
     &1.3347e-01_rb,1.5431e-01_rb,1.7769e-01_rb/)
      kao(:, 2, 4,11) = (/ &
     &3.3523e-02_rb,4.4307e-02_rb,5.8961e-02_rb,7.3730e-02_rb,9.2117e-02_rb,1.1213e-01_rb, &
     &1.3369e-01_rb,1.5450e-01_rb,1.7797e-01_rb/)
      kao(:, 3, 4,11) = (/ &
     &3.3236e-02_rb,4.4011e-02_rb,5.8687e-02_rb,7.3516e-02_rb,9.1754e-02_rb,1.1213e-01_rb, &
     &1.3376e-01_rb,1.5472e-01_rb,1.7805e-01_rb/)
      kao(:, 4, 4,11) = (/ &
     &3.2927e-02_rb,4.3662e-02_rb,5.8418e-02_rb,7.3212e-02_rb,9.1231e-02_rb,1.1190e-01_rb, &
     &1.3353e-01_rb,1.5459e-01_rb,1.7761e-01_rb/)
      kao(:, 5, 4,11) = (/ &
     &3.2628e-02_rb,4.3294e-02_rb,5.8201e-02_rb,7.2680e-02_rb,9.0434e-02_rb,1.1128e-01_rb, &
     &1.3281e-01_rb,1.5389e-01_rb,1.7655e-01_rb/)
      kao(:, 1, 5,11) = (/ &
     &6.5323e-02_rb,7.2628e-02_rb,8.3508e-02_rb,9.5963e-02_rb,1.0655e-01_rb,1.2020e-01_rb, &
     &1.3782e-01_rb,1.5923e-01_rb,1.8200e-01_rb/)
      kao(:, 2, 5,11) = (/ &
     &6.5064e-02_rb,7.2406e-02_rb,8.3203e-02_rb,9.5444e-02_rb,1.0632e-01_rb,1.1979e-01_rb, &
     &1.3760e-01_rb,1.5925e-01_rb,1.8207e-01_rb/)
      kao(:, 3, 5,11) = (/ &
     &6.4756e-02_rb,7.1969e-02_rb,8.2862e-02_rb,9.4908e-02_rb,1.0590e-01_rb,1.1930e-01_rb, &
     &1.3724e-01_rb,1.5904e-01_rb,1.8162e-01_rb/)
      kao(:, 4, 5,11) = (/ &
     &6.4304e-02_rb,7.1135e-02_rb,8.2400e-02_rb,9.4416e-02_rb,1.0531e-01_rb,1.1853e-01_rb, &
     &1.3656e-01_rb,1.5838e-01_rb,1.8061e-01_rb/)
      kao(:, 5, 5,11) = (/ &
     &6.3847e-02_rb,7.0072e-02_rb,8.1806e-02_rb,9.3969e-02_rb,1.0427e-01_rb,1.1735e-01_rb, &
     &1.3557e-01_rb,1.5724e-01_rb,1.7920e-01_rb/)
      kao(:, 1, 6,11) = (/ &
     &1.1614e-01_rb,1.1612e-01_rb,1.2461e-01_rb,1.3093e-01_rb,1.3774e-01_rb,1.4186e-01_rb, &
     &1.4765e-01_rb,1.6150e-01_rb,1.8364e-01_rb/)
      kao(:, 2, 6,11) = (/ &
     &1.1619e-01_rb,1.1550e-01_rb,1.2387e-01_rb,1.3036e-01_rb,1.3714e-01_rb,1.4135e-01_rb, &
     &1.4700e-01_rb,1.6131e-01_rb,1.8335e-01_rb/)
      kao(:, 3, 6,11) = (/ &
     &1.1615e-01_rb,1.1477e-01_rb,1.2280e-01_rb,1.2991e-01_rb,1.3667e-01_rb,1.4072e-01_rb, &
     &1.4596e-01_rb,1.6062e-01_rb,1.8242e-01_rb/)
      kao(:, 4, 6,11) = (/ &
     &1.1574e-01_rb,1.1387e-01_rb,1.2156e-01_rb,1.2938e-01_rb,1.3597e-01_rb,1.3977e-01_rb, &
     &1.4470e-01_rb,1.5943e-01_rb,1.8095e-01_rb/)
      kao(:, 5, 6,11) = (/ &
     &1.1520e-01_rb,1.1286e-01_rb,1.2034e-01_rb,1.2877e-01_rb,1.3515e-01_rb,1.3854e-01_rb, &
     &1.4325e-01_rb,1.5816e-01_rb,1.7948e-01_rb/)
      kao(:, 1, 7,11) = (/ &
     &2.1510e-01_rb,1.9619e-01_rb,2.0174e-01_rb,1.9935e-01_rb,1.9475e-01_rb,1.8847e-01_rb, &
     &1.8001e-01_rb,1.6816e-01_rb,1.8086e-01_rb/)
      kao(:, 2, 7,11) = (/ &
     &2.1625e-01_rb,1.9617e-01_rb,2.0091e-01_rb,1.9825e-01_rb,1.9429e-01_rb,1.8801e-01_rb, &
     &1.7902e-01_rb,1.6736e-01_rb,1.8038e-01_rb/)
      kao(:, 3, 7,11) = (/ &
     &2.1700e-01_rb,1.9569e-01_rb,1.9979e-01_rb,1.9686e-01_rb,1.9372e-01_rb,1.8767e-01_rb, &
     &1.7767e-01_rb,1.6608e-01_rb,1.7932e-01_rb/)
      kao(:, 4, 7,11) = (/ &
     &2.1676e-01_rb,1.9471e-01_rb,1.9847e-01_rb,1.9536e-01_rb,1.9293e-01_rb,1.8720e-01_rb, &
     &1.7635e-01_rb,1.6450e-01_rb,1.7799e-01_rb/)
      kao(:, 5, 7,11) = (/ &
     &2.1625e-01_rb,1.9358e-01_rb,1.9672e-01_rb,1.9373e-01_rb,1.9213e-01_rb,1.8631e-01_rb, &
     &1.7483e-01_rb,1.6262e-01_rb,1.7662e-01_rb/)
      kao(:, 1, 8,11) = (/ &
     &4.6382e-01_rb,4.0807e-01_rb,3.7899e-01_rb,3.5840e-01_rb,3.2915e-01_rb,2.9613e-01_rb, &
     &2.5854e-01_rb,2.1154e-01_rb,1.7388e-01_rb/)
      kao(:, 2, 8,11) = (/ &
     &4.6803e-01_rb,4.1204e-01_rb,3.7966e-01_rb,3.5758e-01_rb,3.2785e-01_rb,2.9605e-01_rb, &
     &2.5875e-01_rb,2.1026e-01_rb,1.7332e-01_rb/)
      kao(:, 3, 8,11) = (/ &
     &4.7076e-01_rb,4.1406e-01_rb,3.7922e-01_rb,3.5601e-01_rb,3.2603e-01_rb,2.9561e-01_rb, &
     &2.5861e-01_rb,2.0864e-01_rb,1.7242e-01_rb/)
      kao(:, 4, 8,11) = (/ &
     &4.7155e-01_rb,4.1450e-01_rb,3.7727e-01_rb,3.5413e-01_rb,3.2392e-01_rb,2.9447e-01_rb, &
     &2.5827e-01_rb,2.0694e-01_rb,1.7120e-01_rb/)
      kao(:, 5, 8,11) = (/ &
     &4.7233e-01_rb,4.1516e-01_rb,3.7581e-01_rb,3.5208e-01_rb,3.2176e-01_rb,2.9332e-01_rb, &
     &2.5728e-01_rb,2.0514e-01_rb,1.7042e-01_rb/)
      kao(:, 1, 9,11) = (/ &
     &1.8678e+00_rb,1.6362e+00_rb,1.4048e+00_rb,1.1793e+00_rb,9.9438e-01_rb,8.2136e-01_rb, &
     &6.2834e-01_rb,4.2034e-01_rb,1.6334e-01_rb/)
      kao(:, 2, 9,11) = (/ &
     &1.8906e+00_rb,1.6563e+00_rb,1.4224e+00_rb,1.1919e+00_rb,9.9937e-01_rb,8.2259e-01_rb, &
     &6.2713e-01_rb,4.2292e-01_rb,1.6311e-01_rb/)
      kao(:, 3, 9,11) = (/ &
     &1.9044e+00_rb,1.6682e+00_rb,1.4321e+00_rb,1.1983e+00_rb,1.0002e+00_rb,8.2124e-01_rb, &
     &6.2419e-01_rb,4.2359e-01_rb,1.6249e-01_rb/)
      kao(:, 4, 9,11) = (/ &
     &1.9121e+00_rb,1.6748e+00_rb,1.4376e+00_rb,1.2016e+00_rb,9.9779e-01_rb,8.1901e-01_rb, &
     &6.2159e-01_rb,4.2327e-01_rb,1.6166e-01_rb/)
      kao(:, 5, 9,11) = (/ &
     &1.9224e+00_rb,1.6839e+00_rb,1.4454e+00_rb,1.2078e+00_rb,9.9866e-01_rb,8.1827e-01_rb, &
     &6.1893e-01_rb,4.2245e-01_rb,1.6163e-01_rb/)
      kao(:, 1,10,11) = (/ &
     &8.3490e+00_rb,7.3068e+00_rb,6.2652e+00_rb,5.2233e+00_rb,4.1816e+00_rb,3.1407e+00_rb, &
     &2.1527e+00_rb,1.2369e+00_rb,1.2697e-01_rb/)
      kao(:, 2,10,11) = (/ &
     &8.4604e+00_rb,7.4045e+00_rb,6.3492e+00_rb,5.2936e+00_rb,4.2378e+00_rb,3.1825e+00_rb, &
     &2.1701e+00_rb,1.2392e+00_rb,1.2746e-01_rb/)
      kao(:, 3,10,11) = (/ &
     &8.5299e+00_rb,7.4655e+00_rb,6.4008e+00_rb,5.3364e+00_rb,4.2721e+00_rb,3.2084e+00_rb, &
     &2.1800e+00_rb,1.2385e+00_rb,1.2876e-01_rb/)
      kao(:, 4,10,11) = (/ &
     &8.6037e+00_rb,7.5302e+00_rb,6.4564e+00_rb,5.3827e+00_rb,4.3093e+00_rb,3.2362e+00_rb, &
     &2.1902e+00_rb,1.2373e+00_rb,1.3016e-01_rb/)
      kao(:, 5,10,11) = (/ &
     &8.6811e+00_rb,7.5976e+00_rb,6.5142e+00_rb,5.4315e+00_rb,4.3481e+00_rb,3.2657e+00_rb, &
     &2.2039e+00_rb,1.2408e+00_rb,1.3327e-01_rb/)
      kao(:, 1,11,11) = (/ &
     &1.2908e+01_rb,1.1296e+01_rb,9.6842e+00_rb,8.0726e+00_rb,6.4609e+00_rb,4.8491e+00_rb, &
     &3.2606e+00_rb,1.7969e+00_rb,1.0870e-01_rb/)
      kao(:, 2,11,11) = (/ &
     &1.3081e+01_rb,1.1447e+01_rb,9.8136e+00_rb,8.1796e+00_rb,6.5463e+00_rb,4.9136e+00_rb, &
     &3.2978e+00_rb,1.8090e+00_rb,1.0807e-01_rb/)
      kao(:, 3,11,11) = (/ &
     &1.3247e+01_rb,1.1593e+01_rb,9.9386e+00_rb,8.2843e+00_rb,6.6304e+00_rb,4.9764e+00_rb, &
     &3.3344e+00_rb,1.8186e+00_rb,1.0723e-01_rb/)
      kao(:, 4,11,11) = (/ &
     &1.3412e+01_rb,1.1737e+01_rb,1.0062e+01_rb,8.3876e+00_rb,6.7133e+00_rb,5.0389e+00_rb, &
     &3.3728e+00_rb,1.8302e+00_rb,1.0582e-01_rb/)
      kao(:, 5,11,11) = (/ &
     &1.3565e+01_rb,1.1871e+01_rb,1.0177e+01_rb,8.4837e+00_rb,6.7902e+00_rb,5.0972e+00_rb, &
     &3.4088e+00_rb,1.8416e+00_rb,1.0800e-01_rb/)
      kao(:, 1,12,11) = (/ &
     &1.4642e+01_rb,1.2812e+01_rb,1.0983e+01_rb,9.1554e+00_rb,7.3265e+00_rb,5.4974e+00_rb, &
     &3.7020e+00_rb,2.0284e+00_rb,8.2662e-02_rb/)
      kao(:, 2,12,11) = (/ &
     &1.4885e+01_rb,1.3026e+01_rb,1.1167e+01_rb,9.3078e+00_rb,7.4487e+00_rb,5.5898e+00_rb, &
     &3.7559e+00_rb,2.0497e+00_rb,8.1390e-02_rb/)
      kao(:, 3,12,11) = (/ &
     &1.5119e+01_rb,1.3231e+01_rb,1.1343e+01_rb,9.4543e+00_rb,7.5661e+00_rb,5.6785e+00_rb, &
     &3.8088e+00_rb,2.0698e+00_rb,8.0357e-02_rb/)
      kao(:, 4,12,11) = (/ &
     &1.5358e+01_rb,1.3440e+01_rb,1.1522e+01_rb,9.6047e+00_rb,7.6867e+00_rb,5.7694e+00_rb, &
     &3.8642e+00_rb,2.0905e+00_rb,7.9812e-02_rb/)
      kao(:, 5,12,11) = (/ &
     &1.5578e+01_rb,1.3633e+01_rb,1.1688e+01_rb,9.7428e+00_rb,7.7978e+00_rb,5.8532e+00_rb, &
     &3.9171e+00_rb,2.1122e+00_rb,8.2952e-02_rb/)
      kao(:, 1,13,11) = (/ &
     &1.3181e+01_rb,1.1536e+01_rb,9.8891e+00_rb,8.2428e+00_rb,6.5963e+00_rb,4.9547e+00_rb, &
     &3.3878e+00_rb,1.8779e+00_rb,8.5930e-02_rb/)
      kao(:, 2,13,11) = (/ &
     &1.3431e+01_rb,1.1753e+01_rb,1.0076e+01_rb,8.3993e+00_rb,6.7218e+00_rb,5.0463e+00_rb, &
     &3.4400e+00_rb,1.9003e+00_rb,8.2789e-02_rb/)
      kao(:, 3,13,11) = (/ &
     &1.3694e+01_rb,1.1985e+01_rb,1.0274e+01_rb,8.5642e+00_rb,6.8544e+00_rb,5.1452e+00_rb, &
     &3.4940e+00_rb,1.9250e+00_rb,8.1321e-02_rb/)
      kao(:, 4,13,11) = (/ &
     &1.3957e+01_rb,1.2214e+01_rb,1.0471e+01_rb,8.7279e+00_rb,6.9861e+00_rb,5.2450e+00_rb, &
     &3.5504e+00_rb,1.9524e+00_rb,7.8953e-02_rb/)
      kao(:, 5,13,11) = (/ &
     &1.4201e+01_rb,1.2428e+01_rb,1.0655e+01_rb,8.8819e+00_rb,7.1097e+00_rb,5.3381e+00_rb, &
     &3.6055e+00_rb,1.9785e+00_rb,8.1036e-02_rb/)
      kao(:, 1, 1,12) = (/ &
     &6.5787e-03_rb,2.6236e-02_rb,5.1362e-02_rb,7.6305e-02_rb,1.0073e-01_rb,1.2420e-01_rb, &
     &1.4556e-01_rb,1.6001e-01_rb,2.0088e-01_rb/)
      kao(:, 2, 1,12) = (/ &
     &6.5355e-03_rb,2.5963e-02_rb,5.1031e-02_rb,7.5853e-02_rb,1.0014e-01_rb,1.2346e-01_rb, &
     &1.4463e-01_rb,1.5894e-01_rb,1.9973e-01_rb/)
      kao(:, 3, 1,12) = (/ &
     &6.4519e-03_rb,2.5842e-02_rb,5.0885e-02_rb,7.5629e-02_rb,9.9804e-02_rb,1.2294e-01_rb, &
     &1.4394e-01_rb,1.5802e-01_rb,1.9913e-01_rb/)
      kao(:, 4, 1,12) = (/ &
     &6.3508e-03_rb,2.5827e-02_rb,5.0889e-02_rb,7.5643e-02_rb,9.9816e-02_rb,1.2289e-01_rb, &
     &1.4367e-01_rb,1.5727e-01_rb,1.9918e-01_rb/)
      kao(:, 5, 1,12) = (/ &
     &6.2468e-03_rb,2.5824e-02_rb,5.0882e-02_rb,7.5643e-02_rb,9.9840e-02_rb,1.2300e-01_rb, &
     &1.4386e-01_rb,1.5674e-01_rb,1.9923e-01_rb/)
      kao(:, 1, 2,12) = (/ &
     &1.0416e-02_rb,2.8932e-02_rb,5.5152e-02_rb,8.2023e-02_rb,1.0846e-01_rb,1.3404e-01_rb, &
     &1.5780e-01_rb,1.7629e-01_rb,2.1619e-01_rb/)
      kao(:, 2, 2,12) = (/ &
     &1.0342e-02_rb,2.8637e-02_rb,5.4892e-02_rb,8.1654e-02_rb,1.0796e-01_rb,1.3340e-01_rb, &
     &1.5695e-01_rb,1.7507e-01_rb,2.1532e-01_rb/)
      kao(:, 3, 2,12) = (/ &
     &1.0224e-02_rb,2.8457e-02_rb,5.4759e-02_rb,8.1469e-02_rb,1.0772e-01_rb,1.3310e-01_rb, &
     &1.5656e-01_rb,1.7435e-01_rb,2.1487e-01_rb/)
      kao(:, 4, 2,12) = (/ &
     &1.0091e-02_rb,2.8318e-02_rb,5.4670e-02_rb,8.1344e-02_rb,1.0759e-01_rb,1.3299e-01_rb, &
     &1.5649e-01_rb,1.7414e-01_rb,2.1459e-01_rb/)
      kao(:, 5, 2,12) = (/ &
     &9.9570e-03_rb,2.8146e-02_rb,5.4539e-02_rb,8.1138e-02_rb,1.0736e-01_rb,1.3278e-01_rb, &
     &1.5638e-01_rb,1.7422e-01_rb,2.1411e-01_rb/)
      kao(:, 1, 3,12) = (/ &
     &2.2302e-02_rb,3.7417e-02_rb,6.1097e-02_rb,8.8747e-02_rb,1.1756e-01_rb,1.4593e-01_rb, &
     &1.7314e-01_rb,1.9636e-01_rb,2.3403e-01_rb/)
      kao(:, 2, 3,12) = (/ &
     &2.2117e-02_rb,3.6950e-02_rb,6.0557e-02_rb,8.8385e-02_rb,1.1711e-01_rb,1.4537e-01_rb, &
     &1.7254e-01_rb,1.9566e-01_rb,2.3319e-01_rb/)
      kao(:, 3, 3,12) = (/ &
     &2.1903e-02_rb,3.6596e-02_rb,6.0142e-02_rb,8.8214e-02_rb,1.1684e-01_rb,1.4504e-01_rb, &
     &1.7207e-01_rb,1.9507e-01_rb,2.3270e-01_rb/)
      kao(:, 4, 3,12) = (/ &
     &2.1706e-02_rb,3.6379e-02_rb,5.9779e-02_rb,8.7937e-02_rb,1.1644e-01_rb,1.4454e-01_rb, &
     &1.7162e-01_rb,1.9475e-01_rb,2.3180e-01_rb/)
      kao(:, 5, 3,12) = (/ &
     &2.1485e-02_rb,3.6152e-02_rb,5.9419e-02_rb,8.7609e-02_rb,1.1597e-01_rb,1.4392e-01_rb, &
     &1.7083e-01_rb,1.9407e-01_rb,2.3079e-01_rb/)
      kao(:, 1, 4,12) = (/ &
     &4.8787e-02_rb,6.0363e-02_rb,7.6958e-02_rb,9.9608e-02_rb,1.2651e-01_rb,1.5698e-01_rb, &
     &1.8722e-01_rb,2.1554e-01_rb,2.5046e-01_rb/)
      kao(:, 2, 4,12) = (/ &
     &4.8476e-02_rb,5.9621e-02_rb,7.6177e-02_rb,9.8903e-02_rb,1.2601e-01_rb,1.5644e-01_rb, &
     &1.8652e-01_rb,2.1472e-01_rb,2.4963e-01_rb/)
      kao(:, 3, 4,12) = (/ &
     &4.8022e-02_rb,5.9054e-02_rb,7.5570e-02_rb,9.8295e-02_rb,1.2578e-01_rb,1.5620e-01_rb, &
     &1.8618e-01_rb,2.1420e-01_rb,2.4913e-01_rb/)
      kao(:, 4, 4,12) = (/ &
     &4.7565e-02_rb,5.8399e-02_rb,7.5073e-02_rb,9.7780e-02_rb,1.2559e-01_rb,1.5591e-01_rb, &
     &1.8579e-01_rb,2.1367e-01_rb,2.4858e-01_rb/)
      kao(:, 5, 4,12) = (/ &
     &4.7078e-02_rb,5.7702e-02_rb,7.4506e-02_rb,9.7303e-02_rb,1.2546e-01_rb,1.5568e-01_rb, &
     &1.8546e-01_rb,2.1318e-01_rb,2.4815e-01_rb/)
      kao(:, 1, 5,12) = (/ &
     &9.7499e-02_rb,9.8718e-02_rb,1.1395e-01_rb,1.2710e-01_rb,1.4448e-01_rb,1.6872e-01_rb, &
     &1.9875e-01_rb,2.3006e-01_rb,2.6441e-01_rb/)
      kao(:, 2, 5,12) = (/ &
     &9.6942e-02_rb,9.7852e-02_rb,1.1233e-01_rb,1.2606e-01_rb,1.4347e-01_rb,1.6785e-01_rb, &
     &1.9829e-01_rb,2.2956e-01_rb,2.6367e-01_rb/)
      kao(:, 3, 5,12) = (/ &
     &9.6111e-02_rb,9.7066e-02_rb,1.1087e-01_rb,1.2520e-01_rb,1.4264e-01_rb,1.6720e-01_rb, &
     &1.9804e-01_rb,2.2914e-01_rb,2.6333e-01_rb/)
      kao(:, 4, 5,12) = (/ &
     &9.5331e-02_rb,9.6285e-02_rb,1.0984e-01_rb,1.2440e-01_rb,1.4205e-01_rb,1.6680e-01_rb, &
     &1.9797e-01_rb,2.2895e-01_rb,2.6319e-01_rb/)
      kao(:, 5, 5,12) = (/ &
     &9.4443e-02_rb,9.5498e-02_rb,1.0898e-01_rb,1.2351e-01_rb,1.4175e-01_rb,1.6646e-01_rb, &
     &1.9775e-01_rb,2.2870e-01_rb,2.6275e-01_rb/)
      kao(:, 1, 6,12) = (/ &
     &1.8140e-01_rb,1.6408e-01_rb,1.7552e-01_rb,1.8196e-01_rb,1.8674e-01_rb,1.9489e-01_rb, &
     &2.1183e-01_rb,2.4078e-01_rb,2.7512e-01_rb/)
      kao(:, 2, 6,12) = (/ &
     &1.8064e-01_rb,1.6323e-01_rb,1.7439e-01_rb,1.7956e-01_rb,1.8547e-01_rb,1.9334e-01_rb, &
     &2.1068e-01_rb,2.4042e-01_rb,2.7473e-01_rb/)
      kao(:, 3, 6,12) = (/ &
     &1.7939e-01_rb,1.6216e-01_rb,1.7344e-01_rb,1.7751e-01_rb,1.8404e-01_rb,1.9207e-01_rb, &
     &2.1020e-01_rb,2.4067e-01_rb,2.7502e-01_rb/)
      kao(:, 4, 6,12) = (/ &
     &1.7796e-01_rb,1.6097e-01_rb,1.7233e-01_rb,1.7595e-01_rb,1.8277e-01_rb,1.9147e-01_rb, &
     &2.0971e-01_rb,2.4067e-01_rb,2.7492e-01_rb/)
      kao(:, 5, 6,12) = (/ &
     &1.7632e-01_rb,1.5943e-01_rb,1.7056e-01_rb,1.7456e-01_rb,1.8167e-01_rb,1.9081e-01_rb, &
     &2.0890e-01_rb,2.4019e-01_rb,2.7421e-01_rb/)
      kao(:, 1, 7,12) = (/ &
     &3.5379e-01_rb,3.1064e-01_rb,2.9565e-01_rb,2.8816e-01_rb,2.7694e-01_rb,2.6472e-01_rb, &
     &2.4939e-01_rb,2.5136e-01_rb,2.8304e-01_rb/)
      kao(:, 2, 7,12) = (/ &
     &3.5306e-01_rb,3.1059e-01_rb,2.9503e-01_rb,2.8682e-01_rb,2.7447e-01_rb,2.6248e-01_rb, &
     &2.4787e-01_rb,2.5077e-01_rb,2.8333e-01_rb/)
      kao(:, 3, 7,12) = (/ &
     &3.5170e-01_rb,3.1024e-01_rb,2.9413e-01_rb,2.8605e-01_rb,2.7226e-01_rb,2.6030e-01_rb, &
     &2.4686e-01_rb,2.5055e-01_rb,2.8358e-01_rb/)
      kao(:, 4, 7,12) = (/ &
     &3.4988e-01_rb,3.0935e-01_rb,2.9228e-01_rb,2.8440e-01_rb,2.7034e-01_rb,2.5822e-01_rb, &
     &2.4645e-01_rb,2.4996e-01_rb,2.8315e-01_rb/)
      kao(:, 5, 7,12) = (/ &
     &3.4759e-01_rb,3.0773e-01_rb,2.8986e-01_rb,2.8199e-01_rb,2.6823e-01_rb,2.5660e-01_rb, &
     &2.4547e-01_rb,2.4917e-01_rb,2.8239e-01_rb/)
      kao(:, 1, 8,12) = (/ &
     &8.0496e-01_rb,7.0527e-01_rb,6.0910e-01_rb,5.5199e-01_rb,5.0022e-01_rb,4.3776e-01_rb, &
     &3.7465e-01_rb,3.0063e-01_rb,2.8877e-01_rb/)
      kao(:, 2, 8,12) = (/ &
     &8.0672e-01_rb,7.0707e-01_rb,6.1070e-01_rb,5.5311e-01_rb,5.0061e-01_rb,4.3582e-01_rb, &
     &3.7176e-01_rb,2.9952e-01_rb,2.8902e-01_rb/)
      kao(:, 3, 8,12) = (/ &
     &8.0691e-01_rb,7.0823e-01_rb,6.1191e-01_rb,5.5364e-01_rb,5.0004e-01_rb,4.3422e-01_rb, &
     &3.6951e-01_rb,2.9892e-01_rb,2.8876e-01_rb/)
      kao(:, 4, 8,12) = (/ &
     &8.0527e-01_rb,7.0753e-01_rb,6.1153e-01_rb,5.5173e-01_rb,4.9853e-01_rb,4.3157e-01_rb, &
     &3.6699e-01_rb,2.9815e-01_rb,2.8834e-01_rb/)
      kao(:, 5, 8,12) = (/ &
     &8.0219e-01_rb,7.0523e-01_rb,6.0964e-01_rb,5.4776e-01_rb,4.9536e-01_rb,4.2816e-01_rb, &
     &3.6486e-01_rb,2.9693e-01_rb,2.8675e-01_rb/)
      kao(:, 1, 9,12) = (/ &
     &3.4182e+00_rb,2.9916e+00_rb,2.5652e+00_rb,2.1389e+00_rb,1.7153e+00_rb,1.3419e+00_rb, &
     &1.0101e+00_rb,6.4726e-01_rb,2.8926e-01_rb/)
      kao(:, 2, 9,12) = (/ &
     &3.4411e+00_rb,3.0119e+00_rb,2.5830e+00_rb,2.1544e+00_rb,1.7278e+00_rb,1.3490e+00_rb, &
     &1.0168e+00_rb,6.4570e-01_rb,2.8901e-01_rb/)
      kao(:, 3, 9,12) = (/ &
     &3.4584e+00_rb,3.0278e+00_rb,2.5976e+00_rb,2.1679e+00_rb,1.7392e+00_rb,1.3535e+00_rb, &
     &1.0208e+00_rb,6.4462e-01_rb,2.8863e-01_rb/)
      kao(:, 4, 9,12) = (/ &
     &3.4643e+00_rb,3.0337e+00_rb,2.6033e+00_rb,2.1735e+00_rb,1.7445e+00_rb,1.3538e+00_rb, &
     &1.0204e+00_rb,6.4174e-01_rb,2.8774e-01_rb/)
      kao(:, 5, 9,12) = (/ &
     &3.4625e+00_rb,3.0324e+00_rb,2.6029e+00_rb,2.1736e+00_rb,1.7453e+00_rb,1.3500e+00_rb, &
     &1.0152e+00_rb,6.3782e-01_rb,2.8595e-01_rb/)
      kao(:, 1,10,12) = (/ &
     &1.6048e+01_rb,1.4043e+01_rb,1.2038e+01_rb,1.0032e+01_rb,8.0269e+00_rb,6.0216e+00_rb, &
     &4.0164e+00_rb,2.1217e+00_rb,2.8227e-01_rb/)
      kao(:, 2,10,12) = (/ &
     &1.6256e+01_rb,1.4226e+01_rb,1.2195e+01_rb,1.0164e+01_rb,8.1333e+00_rb,6.1029e+00_rb, &
     &4.0729e+00_rb,2.1442e+00_rb,2.8170e-01_rb/)
      kao(:, 3,10,12) = (/ &
     &1.6399e+01_rb,1.4351e+01_rb,1.2303e+01_rb,1.0255e+01_rb,8.2073e+00_rb,6.1596e+00_rb, &
     &4.1123e+00_rb,2.1594e+00_rb,2.8134e-01_rb/)
      kao(:, 4,10,12) = (/ &
     &1.6465e+01_rb,1.4409e+01_rb,1.2353e+01_rb,1.0297e+01_rb,8.2414e+00_rb,6.1861e+00_rb, &
     &4.1319e+00_rb,2.1644e+00_rb,2.8049e-01_rb/)
      kao(:, 5,10,12) = (/ &
     &1.6472e+01_rb,1.4416e+01_rb,1.2359e+01_rb,1.0303e+01_rb,8.2464e+00_rb,6.1909e+00_rb, &
     &4.1366e+00_rb,2.1582e+00_rb,2.7905e-01_rb/)
      kao(:, 1,11,12) = (/ &
     &2.6190e+01_rb,2.2917e+01_rb,1.9644e+01_rb,1.6372e+01_rb,1.3098e+01_rb,9.8258e+00_rb, &
     &6.5533e+00_rb,3.3347e+00_rb,2.0130e-01_rb/)
      kao(:, 2,11,12) = (/ &
     &2.6572e+01_rb,2.3252e+01_rb,1.9932e+01_rb,1.6612e+01_rb,1.3292e+01_rb,9.9715e+00_rb, &
     &6.6522e+00_rb,3.3768e+00_rb,2.0375e-01_rb/)
      kao(:, 3,11,12) = (/ &
     &2.6794e+01_rb,2.3447e+01_rb,2.0099e+01_rb,1.6751e+01_rb,1.3404e+01_rb,1.0057e+01_rb, &
     &6.7102e+00_rb,3.4019e+00_rb,2.0648e-01_rb/)
      kao(:, 4,11,12) = (/ &
     &2.6905e+01_rb,2.3544e+01_rb,2.0183e+01_rb,1.6822e+01_rb,1.3461e+01_rb,1.0100e+01_rb, &
     &6.7401e+00_rb,3.4102e+00_rb,2.0903e-01_rb/)
      kao(:, 5,11,12) = (/ &
     &2.6968e+01_rb,2.3599e+01_rb,2.0230e+01_rb,1.6860e+01_rb,1.3492e+01_rb,1.0123e+01_rb, &
     &6.7556e+00_rb,3.4122e+00_rb,2.1045e-01_rb/)
      kao(:, 1,12,12) = (/ &
     &3.1310e+01_rb,2.7398e+01_rb,2.3484e+01_rb,1.9572e+01_rb,1.5659e+01_rb,1.1747e+01_rb, &
     &7.8342e+00_rb,3.9879e+00_rb,1.6283e-01_rb/)
      kao(:, 2,12,12) = (/ &
     &3.1751e+01_rb,2.7784e+01_rb,2.3816e+01_rb,1.9849e+01_rb,1.5882e+01_rb,1.1914e+01_rb, &
     &7.9470e+00_rb,4.0341e+00_rb,1.6415e-01_rb/)
      kao(:, 3,12,12) = (/ &
     &3.2043e+01_rb,2.8038e+01_rb,2.4036e+01_rb,2.0032e+01_rb,1.6028e+01_rb,1.2025e+01_rb, &
     &8.0217e+00_rb,4.0620e+00_rb,1.6794e-01_rb/)
      kao(:, 4,12,12) = (/ &
     &3.2220e+01_rb,2.8195e+01_rb,2.4168e+01_rb,2.0143e+01_rb,1.6116e+01_rb,1.2091e+01_rb, &
     &8.0659e+00_rb,4.0753e+00_rb,1.6850e-01_rb/)
      kao(:, 5,12,12) = (/ &
     &3.2373e+01_rb,2.8329e+01_rb,2.4283e+01_rb,2.0239e+01_rb,1.6193e+01_rb,1.2149e+01_rb, &
     &8.1043e+00_rb,4.0871e+00_rb,1.6988e-01_rb/)
      kao(:, 1,13,12) = (/ &
     &2.9544e+01_rb,2.5852e+01_rb,2.2161e+01_rb,1.8469e+01_rb,1.4777e+01_rb,1.1085e+01_rb, &
     &7.3931e+00_rb,3.8439e+00_rb,1.2880e-01_rb/)
      kao(:, 2,13,12) = (/ &
     &2.9974e+01_rb,2.6228e+01_rb,2.2483e+01_rb,1.8737e+01_rb,1.4992e+01_rb,1.1247e+01_rb, &
     &7.5021e+00_rb,3.8830e+00_rb,1.3338e-01_rb/)
      kao(:, 3,13,12) = (/ &
     &3.0254e+01_rb,2.6474e+01_rb,2.2694e+01_rb,1.8912e+01_rb,1.5132e+01_rb,1.1352e+01_rb, &
     &7.5719e+00_rb,3.9004e+00_rb,1.3721e-01_rb/)
      kao(:, 4,13,12) = (/ &
     &3.0504e+01_rb,2.6693e+01_rb,2.2881e+01_rb,1.9069e+01_rb,1.5258e+01_rb,1.1446e+01_rb, &
     &7.6348e+00_rb,3.9163e+00_rb,1.3871e-01_rb/)
      kao(:, 5,13,12) = (/ &
     &3.0781e+01_rb,2.6934e+01_rb,2.3088e+01_rb,1.9241e+01_rb,1.5395e+01_rb,1.1550e+01_rb, &
     &7.7042e+00_rb,3.9392e+00_rb,1.4036e-01_rb/)
      kao(:, 1, 1,13) = (/ &
     &9.0088e-03_rb,3.7171e-02_rb,7.3185e-02_rb,1.0841e-01_rb,1.4222e-01_rb,1.7328e-01_rb, &
     &1.9823e-01_rb,2.0424e-01_rb,2.8396e-01_rb/)
      kao(:, 2, 1,13) = (/ &
     &8.8070e-03_rb,3.7017e-02_rb,7.2761e-02_rb,1.0769e-01_rb,1.4122e-01_rb,1.7208e-01_rb, &
     &1.9703e-01_rb,2.0324e-01_rb,2.8185e-01_rb/)
      kao(:, 3, 1,13) = (/ &
     &8.6641e-03_rb,3.6729e-02_rb,7.2126e-02_rb,1.0675e-01_rb,1.4002e-01_rb,1.7072e-01_rb, &
     &1.9555e-01_rb,2.0207e-01_rb,2.7936e-01_rb/)
      kao(:, 4, 1,13) = (/ &
     &8.5351e-03_rb,3.6304e-02_rb,7.1373e-02_rb,1.0562e-01_rb,1.3853e-01_rb,1.6891e-01_rb, &
     &1.9361e-01_rb,2.0011e-01_rb,2.7631e-01_rb/)
      kao(:, 5, 1,13) = (/ &
     &8.4180e-03_rb,3.5892e-02_rb,7.0613e-02_rb,1.0456e-01_rb,1.3716e-01_rb,1.6725e-01_rb, &
     &1.9159e-01_rb,1.9822e-01_rb,2.7366e-01_rb/)
      kao(:, 1, 2,13) = (/ &
     &1.4629e-02_rb,4.1401e-02_rb,8.1344e-02_rb,1.2092e-01_rb,1.5970e-01_rb,1.9686e-01_rb, &
     &2.3007e-01_rb,2.4893e-01_rb,3.1872e-01_rb/)
      kao(:, 2, 2,13) = (/ &
     &1.4347e-02_rb,4.1149e-02_rb,8.0754e-02_rb,1.2000e-01_rb,1.5844e-01_rb,1.9525e-01_rb, &
     &2.2811e-01_rb,2.4701e-01_rb,3.1592e-01_rb/)
      kao(:, 3, 2,13) = (/ &
     &1.4116e-02_rb,4.0731e-02_rb,7.9979e-02_rb,1.1881e-01_rb,1.5685e-01_rb,1.9333e-01_rb, &
     &2.2600e-01_rb,2.4518e-01_rb,3.1267e-01_rb/)
      kao(:, 4, 2,13) = (/ &
     &1.3916e-02_rb,4.0235e-02_rb,7.9131e-02_rb,1.1757e-01_rb,1.5518e-01_rb,1.9122e-01_rb, &
     &2.2354e-01_rb,2.4260e-01_rb,3.0926e-01_rb/)
      kao(:, 5, 2,13) = (/ &
     &1.3712e-02_rb,3.9826e-02_rb,7.8358e-02_rb,1.1644e-01_rb,1.5370e-01_rb,1.8938e-01_rb, &
     &2.2131e-01_rb,2.4017e-01_rb,3.0647e-01_rb/)
      kao(:, 1, 3,13) = (/ &
     &3.2046e-02_rb,5.1387e-02_rb,8.9167e-02_rb,1.3251e-01_rb,1.7557e-01_rb,2.1798e-01_rb, &
     &2.5867e-01_rb,2.9276e-01_rb,3.4967e-01_rb/)
      kao(:, 2, 3,13) = (/ &
     &3.1484e-02_rb,5.0885e-02_rb,8.8511e-02_rb,1.3146e-01_rb,1.7409e-01_rb,2.1604e-01_rb, &
     &2.5621e-01_rb,2.8967e-01_rb,3.4641e-01_rb/)
      kao(:, 3, 3,13) = (/ &
     &3.1011e-02_rb,5.0194e-02_rb,8.7651e-02_rb,1.3011e-01_rb,1.7226e-01_rb,2.1375e-01_rb, &
     &2.5357e-01_rb,2.8697e-01_rb,3.4255e-01_rb/)
      kao(:, 4, 3,13) = (/ &
     &3.0595e-02_rb,4.9306e-02_rb,8.6747e-02_rb,1.2888e-01_rb,1.7065e-01_rb,2.1172e-01_rb, &
     &2.5094e-01_rb,2.8374e-01_rb,3.3932e-01_rb/)
      kao(:, 5, 3,13) = (/ &
     &3.0114e-02_rb,4.8480e-02_rb,8.6058e-02_rb,1.2788e-01_rb,1.6935e-01_rb,2.1010e-01_rb, &
     &2.4901e-01_rb,2.8126e-01_rb,3.3697e-01_rb/)
      kao(:, 1, 4,13) = (/ &
     &7.1437e-02_rb,7.9531e-02_rb,1.0765e-01_rb,1.4496e-01_rb,1.9060e-01_rb,2.3699e-01_rb, &
     &2.8255e-01_rb,3.2483e-01_rb,3.7856e-01_rb/)
      kao(:, 2, 4,13) = (/ &
     &7.0222e-02_rb,7.8795e-02_rb,1.0643e-01_rb,1.4338e-01_rb,1.8881e-01_rb,2.3460e-01_rb, &
     &2.7960e-01_rb,3.2131e-01_rb,3.7439e-01_rb/)
      kao(:, 3, 4,13) = (/ &
     &6.9262e-02_rb,7.7852e-02_rb,1.0486e-01_rb,1.4165e-01_rb,1.8684e-01_rb,2.3207e-01_rb, &
     &2.7650e-01_rb,3.1773e-01_rb,3.7008e-01_rb/)
      kao(:, 4, 4,13) = (/ &
     &6.8383e-02_rb,7.7009e-02_rb,1.0310e-01_rb,1.4004e-01_rb,1.8506e-01_rb,2.2994e-01_rb, &
     &2.7397e-01_rb,3.1473e-01_rb,3.6682e-01_rb/)
      kao(:, 5, 4,13) = (/ &
     &6.7482e-02_rb,7.6264e-02_rb,1.0166e-01_rb,1.3889e-01_rb,1.8370e-01_rb,2.2828e-01_rb, &
     &2.7201e-01_rb,3.1244e-01_rb,3.6430e-01_rb/)
      kao(:, 1, 5,13) = (/ &
     &1.4624e-01_rb,1.3361e-01_rb,1.5693e-01_rb,1.7752e-01_rb,2.1095e-01_rb,2.5537e-01_rb, &
     &3.0486e-01_rb,3.5270e-01_rb,4.0651e-01_rb/)
      kao(:, 2, 5,13) = (/ &
     &1.4398e-01_rb,1.3165e-01_rb,1.5512e-01_rb,1.7519e-01_rb,2.0814e-01_rb,2.5302e-01_rb, &
     &3.0184e-01_rb,3.4895e-01_rb,4.0217e-01_rb/)
      kao(:, 3, 5,13) = (/ &
     &1.4207e-01_rb,1.3000e-01_rb,1.5346e-01_rb,1.7248e-01_rb,2.0524e-01_rb,2.5056e-01_rb, &
     &2.9880e-01_rb,3.4528e-01_rb,3.9789e-01_rb/)
      kao(:, 4, 5,13) = (/ &
     &1.4038e-01_rb,1.2884e-01_rb,1.5157e-01_rb,1.6985e-01_rb,2.0246e-01_rb,2.4823e-01_rb, &
     &2.9613e-01_rb,3.4225e-01_rb,3.9462e-01_rb/)
      kao(:, 5, 5,13) = (/ &
     &1.3874e-01_rb,1.2784e-01_rb,1.4941e-01_rb,1.6780e-01_rb,2.0033e-01_rb,2.4657e-01_rb, &
     &2.9417e-01_rb,3.3986e-01_rb,3.9218e-01_rb/)
      kao(:, 1, 6,13) = (/ &
     &2.8046e-01_rb,2.4640e-01_rb,2.4558e-01_rb,2.5477e-01_rb,2.6318e-01_rb,2.8661e-01_rb, &
     &3.2769e-01_rb,3.7985e-01_rb,4.3568e-01_rb/)
      kao(:, 2, 6,13) = (/ &
     &2.7643e-01_rb,2.4318e-01_rb,2.4273e-01_rb,2.5156e-01_rb,2.5913e-01_rb,2.8275e-01_rb, &
     &3.2478e-01_rb,3.7621e-01_rb,4.3126e-01_rb/)
      kao(:, 3, 6,13) = (/ &
     &2.7290e-01_rb,2.4050e-01_rb,2.4016e-01_rb,2.4894e-01_rb,2.5526e-01_rb,2.7880e-01_rb, &
     &3.2177e-01_rb,3.7266e-01_rb,4.2706e-01_rb/)
      kao(:, 4, 6,13) = (/ &
     &2.7013e-01_rb,2.3869e-01_rb,2.3760e-01_rb,2.4573e-01_rb,2.5166e-01_rb,2.7510e-01_rb, &
     &3.1929e-01_rb,3.6982e-01_rb,4.2412e-01_rb/)
      kao(:, 5, 6,13) = (/ &
     &2.6724e-01_rb,2.3701e-01_rb,2.3563e-01_rb,2.4210e-01_rb,2.4866e-01_rb,2.7240e-01_rb, &
     &3.1774e-01_rb,3.6799e-01_rb,4.2206e-01_rb/)
      kao(:, 1, 7,13) = (/ &
     &5.6688e-01_rb,4.9674e-01_rb,4.3519e-01_rb,4.2029e-01_rb,3.9894e-01_rb,3.7594e-01_rb, &
     &3.7424e-01_rb,4.0700e-01_rb,4.6486e-01_rb/)
      kao(:, 2, 7,13) = (/ &
     &5.6062e-01_rb,4.9147e-01_rb,4.2917e-01_rb,4.1552e-01_rb,3.9325e-01_rb,3.7043e-01_rb, &
     &3.6935e-01_rb,4.0348e-01_rb,4.6044e-01_rb/)
      kao(:, 3, 7,13) = (/ &
     &5.5419e-01_rb,4.8620e-01_rb,4.2405e-01_rb,4.1052e-01_rb,3.8832e-01_rb,3.6566e-01_rb, &
     &3.6433e-01_rb,4.0051e-01_rb,4.5715e-01_rb/)
      kao(:, 4, 7,13) = (/ &
     &5.4908e-01_rb,4.8208e-01_rb,4.2069e-01_rb,4.0630e-01_rb,3.8346e-01_rb,3.6097e-01_rb, &
     &3.5952e-01_rb,3.9819e-01_rb,4.5457e-01_rb/)
      kao(:, 5, 7,13) = (/ &
     &5.4429e-01_rb,4.7829e-01_rb,4.1793e-01_rb,4.0260e-01_rb,3.7854e-01_rb,3.5674e-01_rb, &
     &3.5560e-01_rb,3.9638e-01_rb,4.5240e-01_rb/)
      kao(:, 1, 8,13) = (/ &
     &1.3466e+00_rb,1.1788e+00_rb,1.0111e+00_rb,8.5366e-01_rb,7.5173e-01_rb,6.5869e-01_rb, &
     &5.4376e-01_rb,4.6329e-01_rb,4.8785e-01_rb/)
      kao(:, 2, 8,13) = (/ &
     &1.3340e+00_rb,1.1680e+00_rb,1.0021e+00_rb,8.4354e-01_rb,7.4161e-01_rb,6.4846e-01_rb, &
     &5.3530e-01_rb,4.5751e-01_rb,4.8487e-01_rb/)
      kao(:, 3, 8,13) = (/ &
     &1.3215e+00_rb,1.1573e+00_rb,9.9337e-01_rb,8.3476e-01_rb,7.3374e-01_rb,6.3918e-01_rb, &
     &5.2901e-01_rb,4.5174e-01_rb,4.8277e-01_rb/)
      kao(:, 4, 8,13) = (/ &
     &1.3130e+00_rb,1.1502e+00_rb,9.8764e-01_rb,8.2958e-01_rb,7.2734e-01_rb,6.3251e-01_rb, &
     &5.2258e-01_rb,4.4677e-01_rb,4.8075e-01_rb/)
      kao(:, 5, 8,13) = (/ &
     &1.3037e+00_rb,1.1423e+00_rb,9.8136e-01_rb,8.2511e-01_rb,7.2245e-01_rb,6.2665e-01_rb, &
     &5.1636e-01_rb,4.4197e-01_rb,4.7928e-01_rb/)
      kao(:, 1, 9,13) = (/ &
     &6.0221e+00_rb,5.2697e+00_rb,4.5174e+00_rb,3.7653e+00_rb,3.0129e+00_rb,2.2622e+00_rb, &
     &1.5952e+00_rb,9.9726e-01_rb,5.0671e-01_rb/)
      kao(:, 2, 9,13) = (/ &
     &5.9755e+00_rb,5.2290e+00_rb,4.4828e+00_rb,3.7364e+00_rb,2.9901e+00_rb,2.2441e+00_rb, &
     &1.5732e+00_rb,9.8172e-01_rb,5.0573e-01_rb/)
      kao(:, 3, 9,13) = (/ &
     &5.9358e+00_rb,5.1947e+00_rb,4.4536e+00_rb,3.7125e+00_rb,2.9715e+00_rb,2.2305e+00_rb, &
     &1.5561e+00_rb,9.6836e-01_rb,5.0514e-01_rb/)
      kao(:, 4, 9,13) = (/ &
     &5.9085e+00_rb,5.1712e+00_rb,4.4336e+00_rb,3.6962e+00_rb,2.9588e+00_rb,2.2217e+00_rb, &
     &1.5446e+00_rb,9.5878e-01_rb,5.0373e-01_rb/)
      kao(:, 5, 9,13) = (/ &
     &5.8741e+00_rb,5.1412e+00_rb,4.4082e+00_rb,3.6753e+00_rb,2.9426e+00_rb,2.2110e+00_rb, &
     &1.5366e+00_rb,9.5070e-01_rb,5.0223e-01_rb/)
      kao(:, 1,10,13) = (/ &
     &2.9983e+01_rb,2.6236e+01_rb,2.2488e+01_rb,1.8741e+01_rb,1.4994e+01_rb,1.1246e+01_rb, &
     &7.4983e+00_rb,3.7584e+00_rb,5.2065e-01_rb/)
      kao(:, 2,10,13) = (/ &
     &2.9796e+01_rb,2.6072e+01_rb,2.2348e+01_rb,1.8624e+01_rb,1.4900e+01_rb,1.1176e+01_rb, &
     &7.4520e+00_rb,3.7308e+00_rb,5.2186e-01_rb/)
      kao(:, 3,10,13) = (/ &
     &2.9702e+01_rb,2.5989e+01_rb,2.2278e+01_rb,1.8566e+01_rb,1.4854e+01_rb,1.1142e+01_rb, &
     &7.4298e+00_rb,3.7183e+00_rb,5.2236e-01_rb/)
      kao(:, 4,10,13) = (/ &
     &2.9648e+01_rb,2.5942e+01_rb,2.2238e+01_rb,1.8533e+01_rb,1.4828e+01_rb,1.1123e+01_rb, &
     &7.4177e+00_rb,3.7128e+00_rb,5.2097e-01_rb/)
      kao(:, 5,10,13) = (/ &
     &2.9599e+01_rb,2.5901e+01_rb,2.2202e+01_rb,1.8502e+01_rb,1.4804e+01_rb,1.1105e+01_rb, &
     &7.4063e+00_rb,3.7092e+00_rb,5.1987e-01_rb/)
      kao(:, 1,11,13) = (/ &
     &5.1290e+01_rb,4.4882e+01_rb,3.8471e+01_rb,3.2058e+01_rb,2.5646e+01_rb,1.9236e+01_rb, &
     &1.2824e+01_rb,6.4141e+00_rb,5.2983e-01_rb/)
      kao(:, 2,11,13) = (/ &
     &5.1244e+01_rb,4.4838e+01_rb,3.8434e+01_rb,3.2028e+01_rb,2.5623e+01_rb,1.9219e+01_rb, &
     &1.2814e+01_rb,6.4089e+00_rb,5.3215e-01_rb/)
      kao(:, 3,11,13) = (/ &
     &5.1318e+01_rb,4.4905e+01_rb,3.8491e+01_rb,3.2076e+01_rb,2.5662e+01_rb,1.9248e+01_rb, &
     &1.2833e+01_rb,6.4196e+00_rb,5.3209e-01_rb/)
      kao(:, 4,11,13) = (/ &
     &5.1488e+01_rb,4.5053e+01_rb,3.8617e+01_rb,3.2182e+01_rb,2.5747e+01_rb,1.9312e+01_rb, &
     &1.2877e+01_rb,6.4422e+00_rb,5.3137e-01_rb/)
      kao(:, 5,11,13) = (/ &
     &5.1544e+01_rb,4.5101e+01_rb,3.8659e+01_rb,3.2218e+01_rb,2.5776e+01_rb,1.9335e+01_rb, &
     &1.2893e+01_rb,6.4519e+00_rb,5.3038e-01_rb/)
      kao(:, 1,12,13) = (/ &
     &6.4625e+01_rb,5.6547e+01_rb,4.8469e+01_rb,4.0391e+01_rb,3.2315e+01_rb,2.4238e+01_rb, &
     &1.6158e+01_rb,8.0814e+00_rb,4.6363e-01_rb/)
      kao(:, 2,12,13) = (/ &
     &6.4981e+01_rb,5.6858e+01_rb,4.8737e+01_rb,4.0614e+01_rb,3.2493e+01_rb,2.4371e+01_rb, &
     &1.6249e+01_rb,8.1265e+00_rb,4.7141e-01_rb/)
      kao(:, 3,12,13) = (/ &
     &6.5489e+01_rb,5.7301e+01_rb,4.9118e+01_rb,4.0933e+01_rb,3.2747e+01_rb,2.4562e+01_rb, &
     &1.6376e+01_rb,8.1908e+00_rb,4.7318e-01_rb/)
      kao(:, 4,12,13) = (/ &
     &6.5927e+01_rb,5.7688e+01_rb,4.9446e+01_rb,4.1208e+01_rb,3.2967e+01_rb,2.4727e+01_rb, &
     &1.6488e+01_rb,8.2475e+00_rb,4.7664e-01_rb/)
      kao(:, 5,12,13) = (/ &
     &6.6087e+01_rb,5.7823e+01_rb,4.9566e+01_rb,4.1306e+01_rb,3.3048e+01_rb,2.4787e+01_rb, &
     &1.6528e+01_rb,8.2690e+00_rb,4.7820e-01_rb/)
      kao(:, 1,13,13) = (/ &
     &6.4615e+01_rb,5.6540e+01_rb,4.8462e+01_rb,4.0385e+01_rb,3.2309e+01_rb,2.4232e+01_rb, &
     &1.6156e+01_rb,8.0805e+00_rb,4.0088e-01_rb/)
      kao(:, 2,13,13) = (/ &
     &6.5417e+01_rb,5.7241e+01_rb,4.9064e+01_rb,4.0887e+01_rb,3.2711e+01_rb,2.4534e+01_rb, &
     &1.6357e+01_rb,8.1804e+00_rb,3.9953e-01_rb/)
      kao(:, 3,13,13) = (/ &
     &6.6282e+01_rb,5.7998e+01_rb,4.9715e+01_rb,4.1430e+01_rb,3.3145e+01_rb,2.4860e+01_rb, &
     &1.6576e+01_rb,8.2909e+00_rb,4.0385e-01_rb/)
      kao(:, 4,13,13) = (/ &
     &6.6847e+01_rb,5.8491e+01_rb,5.0137e+01_rb,4.1781e+01_rb,3.3427e+01_rb,2.5072e+01_rb, &
     &1.6717e+01_rb,8.3627e+00_rb,4.1064e-01_rb/)
      kao(:, 5,13,13) = (/ &
     &6.7163e+01_rb,5.8769e+01_rb,5.0375e+01_rb,4.1983e+01_rb,3.3587e+01_rb,2.5193e+01_rb, &
     &1.6798e+01_rb,8.4033e+00_rb,4.1414e-01_rb/)
      kao(:, 1, 1,14) = (/ &
     &1.2012e-02_rb,4.7001e-02_rb,9.2646e-02_rb,1.3669e-01_rb,1.7808e-01_rb,2.1465e-01_rb, &
     &2.4110e-01_rb,2.4013e-01_rb,3.5593e-01_rb/)
      kao(:, 2, 1,14) = (/ &
     &1.1787e-02_rb,4.6471e-02_rb,9.1548e-02_rb,1.3506e-01_rb,1.7598e-01_rb,2.1215e-01_rb, &
     &2.3833e-01_rb,2.3702e-01_rb,3.5168e-01_rb/)
      kao(:, 3, 1,14) = (/ &
     &1.1552e-02_rb,4.6117e-02_rb,9.0846e-02_rb,1.3398e-01_rb,1.7452e-01_rb,2.1037e-01_rb, &
     &2.3630e-01_rb,2.3518e-01_rb,3.4867e-01_rb/)
      kao(:, 4, 1,14) = (/ &
     &1.1308e-02_rb,4.5919e-02_rb,9.0260e-02_rb,1.3312e-01_rb,1.7345e-01_rb,2.0914e-01_rb, &
     &2.3511e-01_rb,2.3456e-01_rb,3.4647e-01_rb/)
      kao(:, 5, 1,14) = (/ &
     &1.1074e-02_rb,4.5680e-02_rb,8.9686e-02_rb,1.3217e-01_rb,1.7213e-01_rb,2.0750e-01_rb, &
     &2.3330e-01_rb,2.3291e-01_rb,3.4372e-01_rb/)
      kao(:, 1, 2,14) = (/ &
     &1.9894e-02_rb,5.5751e-02_rb,1.1015e-01_rb,1.6336e-01_rb,2.1450e-01_rb,2.6180e-01_rb, &
     &3.0044e-01_rb,3.1221e-01_rb,4.2855e-01_rb/)
      kao(:, 2, 2,14) = (/ &
     &1.9495e-02_rb,5.5076e-02_rb,1.0879e-01_rb,1.6130e-01_rb,2.1180e-01_rb,2.5853e-01_rb, &
     &2.9678e-01_rb,3.0859e-01_rb,4.2309e-01_rb/)
      kao(:, 3, 2,14) = (/ &
     &1.9117e-02_rb,5.4639e-02_rb,1.0781e-01_rb,1.5983e-01_rb,2.0983e-01_rb,2.5605e-01_rb, &
     &2.9379e-01_rb,3.0533e-01_rb,4.1905e-01_rb/)
      kao(:, 4, 2,14) = (/ &
     &1.8729e-02_rb,5.4397e-02_rb,1.0705e-01_rb,1.5861e-01_rb,2.0822e-01_rb,2.5408e-01_rb, &
     &2.9159e-01_rb,3.0324e-01_rb,4.1584e-01_rb/)
      kao(:, 5, 2,14) = (/ &
     &1.8354e-02_rb,5.4135e-02_rb,1.0645e-01_rb,1.5762e-01_rb,2.0683e-01_rb,2.5235e-01_rb, &
     &2.8960e-01_rb,3.0120e-01_rb,4.1280e-01_rb/)
      kao(:, 1, 3,14) = (/ &
     &4.4808e-02_rb,6.8120e-02_rb,1.3030e-01_rb,1.9402e-01_rb,2.5661e-01_rb,3.1701e-01_rb, &
     &3.7210e-01_rb,4.0841e-01_rb,5.1219e-01_rb/)
      kao(:, 2, 3,14) = (/ &
     &4.3894e-02_rb,6.7026e-02_rb,1.2859e-01_rb,1.9144e-01_rb,2.5317e-01_rb,3.1271e-01_rb, &
     &3.6711e-01_rb,4.0328e-01_rb,5.0524e-01_rb/)
      kao(:, 3, 3,14) = (/ &
     &4.3009e-02_rb,6.6189e-02_rb,1.2745e-01_rb,1.8969e-01_rb,2.5081e-01_rb,3.0975e-01_rb, &
     &3.6348e-01_rb,3.9877e-01_rb,5.0039e-01_rb/)
      kao(:, 4, 3,14) = (/ &
     &4.2131e-02_rb,6.5604e-02_rb,1.2666e-01_rb,1.8820e-01_rb,2.4871e-01_rb,3.0709e-01_rb, &
     &3.6038e-01_rb,3.9547e-01_rb,4.9608e-01_rb/)
      kao(:, 5, 3,14) = (/ &
     &4.1300e-02_rb,6.4954e-02_rb,1.2597e-01_rb,1.8706e-01_rb,2.4708e-01_rb,3.0499e-01_rb, &
     &3.5781e-01_rb,3.9252e-01_rb,4.9231e-01_rb/)
      kao(:, 1, 4,14) = (/ &
     &1.0301e-01_rb,1.0024e-01_rb,1.5513e-01_rb,2.2660e-01_rb,3.0051e-01_rb,3.7330e-01_rb, &
     &4.4331e-01_rb,5.0277e-01_rb,5.9886e-01_rb/)
      kao(:, 2, 4,14) = (/ &
     &1.0093e-01_rb,9.7693e-02_rb,1.5244e-01_rb,2.2368e-01_rb,2.9661e-01_rb,3.6838e-01_rb, &
     &4.3740e-01_rb,4.9600e-01_rb,5.9091e-01_rb/)
      kao(:, 3, 4,14) = (/ &
     &9.8888e-02_rb,9.5814e-02_rb,1.5056e-01_rb,2.2160e-01_rb,2.9368e-01_rb,3.6475e-01_rb, &
     &4.3305e-01_rb,4.9087e-01_rb,5.8504e-01_rb/)
      kao(:, 4, 4,14) = (/ &
     &9.6900e-02_rb,9.4385e-02_rb,1.4923e-01_rb,2.1998e-01_rb,2.9118e-01_rb,3.6134e-01_rb, &
     &4.2884e-01_rb,4.8615e-01_rb,5.7910e-01_rb/)
      kao(:, 5, 4,14) = (/ &
     &9.5071e-02_rb,9.3054e-02_rb,1.4811e-01_rb,2.1854e-01_rb,2.8905e-01_rb,3.5852e-01_rb, &
     &4.2539e-01_rb,4.8206e-01_rb,5.7407e-01_rb/)
      kao(:, 1, 5,14) = (/ &
     &2.1660e-01_rb,1.9045e-01_rb,2.0807e-01_rb,2.6919e-01_rb,3.4511e-01_rb,4.2945e-01_rb, &
     &5.1231e-01_rb,5.8951e-01_rb,6.8618e-01_rb/)
      kao(:, 2, 5,14) = (/ &
     &2.1241e-01_rb,1.8672e-01_rb,2.0475e-01_rb,2.6424e-01_rb,3.4065e-01_rb,4.2384e-01_rb, &
     &5.0554e-01_rb,5.8155e-01_rb,6.7701e-01_rb/)
      kao(:, 3, 5,14) = (/ &
     &2.0837e-01_rb,1.8314e-01_rb,2.0195e-01_rb,2.6053e-01_rb,3.3744e-01_rb,4.1942e-01_rb, &
     &5.0021e-01_rb,5.7551e-01_rb,6.6986e-01_rb/)
      kao(:, 4, 5,14) = (/ &
     &2.0430e-01_rb,1.7971e-01_rb,1.9948e-01_rb,2.5756e-01_rb,3.3465e-01_rb,4.1558e-01_rb, &
     &4.9517e-01_rb,5.6937e-01_rb,6.6243e-01_rb/)
      kao(:, 5, 5,14) = (/ &
     &2.0042e-01_rb,1.7656e-01_rb,1.9774e-01_rb,2.5482e-01_rb,3.3186e-01_rb,4.1183e-01_rb, &
     &4.9052e-01_rb,5.6389e-01_rb,6.5569e-01_rb/)
      kao(:, 1, 6,14) = (/ &
     &4.2427e-01_rb,3.7203e-01_rb,3.3320e-01_rb,3.5686e-01_rb,4.1154e-01_rb,4.8492e-01_rb, &
     &5.7932e-01_rb,6.7077e-01_rb,7.7255e-01_rb/)
      kao(:, 2, 6,14) = (/ &
     &4.1690e-01_rb,3.6557e-01_rb,3.2503e-01_rb,3.5212e-01_rb,4.0345e-01_rb,4.7827e-01_rb, &
     &5.7140e-01_rb,6.6143e-01_rb,7.6181e-01_rb/)
      kao(:, 3, 6,14) = (/ &
     &4.0958e-01_rb,3.5909e-01_rb,3.1787e-01_rb,3.4706e-01_rb,3.9704e-01_rb,4.7318e-01_rb, &
     &5.6463e-01_rb,6.5338e-01_rb,7.5243e-01_rb/)
      kao(:, 4, 6,14) = (/ &
     &4.0201e-01_rb,3.5239e-01_rb,3.1191e-01_rb,3.4259e-01_rb,3.9161e-01_rb,4.6839e-01_rb, &
     &5.5839e-01_rb,6.4570e-01_rb,7.4289e-01_rb/)
      kao(:, 5, 6,14) = (/ &
     &3.9475e-01_rb,3.4591e-01_rb,3.0639e-01_rb,3.3917e-01_rb,3.8642e-01_rb,4.6363e-01_rb, &
     &5.5236e-01_rb,6.3841e-01_rb,7.3426e-01_rb/)
      kao(:, 1, 7,14) = (/ &
     &8.8029e-01_rb,7.7094e-01_rb,6.6160e-01_rb,5.8339e-01_rb,5.7398e-01_rb,5.9529e-01_rb, &
     &6.4618e-01_rb,7.4819e-01_rb,8.5701e-01_rb/)
      kao(:, 2, 7,14) = (/ &
     &8.6525e-01_rb,7.5771e-01_rb,6.5024e-01_rb,5.6937e-01_rb,5.6534e-01_rb,5.8297e-01_rb, &
     &6.3577e-01_rb,7.3677e-01_rb,8.4377e-01_rb/)
      kao(:, 3, 7,14) = (/ &
     &8.5121e-01_rb,7.4536e-01_rb,6.3948e-01_rb,5.5760e-01_rb,5.5733e-01_rb,5.7172e-01_rb, &
     &6.2728e-01_rb,7.2631e-01_rb,8.3134e-01_rb/)
      kao(:, 4, 7,14) = (/ &
     &8.3673e-01_rb,7.3261e-01_rb,6.2850e-01_rb,5.4720e-01_rb,5.4987e-01_rb,5.6219e-01_rb, &
     &6.1953e-01_rb,7.1698e-01_rb,8.1995e-01_rb/)
      kao(:, 5, 7,14) = (/ &
     &8.2233e-01_rb,7.1991e-01_rb,6.1754e-01_rb,5.3796e-01_rb,5.4354e-01_rb,5.5330e-01_rb, &
     &6.1303e-01_rb,7.0902e-01_rb,8.1040e-01_rb/)
      kao(:, 1, 8,14) = (/ &
     &2.1633e+00_rb,1.8933e+00_rb,1.6233e+00_rb,1.3534e+00_rb,1.1037e+00_rb,9.5036e-01_rb, &
     &8.5982e-01_rb,8.2829e-01_rb,9.4205e-01_rb/)
      kao(:, 2, 8,14) = (/ &
     &2.1282e+00_rb,1.8626e+00_rb,1.5970e+00_rb,1.3314e+00_rb,1.0818e+00_rb,9.3527e-01_rb, &
     &8.4498e-01_rb,8.1322e-01_rb,9.2537e-01_rb/)
      kao(:, 3, 8,14) = (/ &
     &2.0953e+00_rb,1.8337e+00_rb,1.5723e+00_rb,1.3107e+00_rb,1.0621e+00_rb,9.2114e-01_rb, &
     &8.2743e-01_rb,8.0051e-01_rb,9.1039e-01_rb/)
      kao(:, 4, 8,14) = (/ &
     &2.0607e+00_rb,1.8035e+00_rb,1.5462e+00_rb,1.2890e+00_rb,1.0433e+00_rb,9.0766e-01_rb, &
     &8.1242e-01_rb,7.8938e-01_rb,8.9703e-01_rb/)
      kao(:, 5, 8,14) = (/ &
     &2.0272e+00_rb,1.7741e+00_rb,1.5210e+00_rb,1.2680e+00_rb,1.0246e+00_rb,8.9475e-01_rb, &
     &7.9856e-01_rb,7.8051e-01_rb,8.8639e-01_rb/)
      kao(:, 1, 9,14) = (/ &
     &1.0073e+01_rb,8.8141e+00_rb,7.5553e+00_rb,6.2968e+00_rb,5.0379e+00_rb,3.7790e+00_rb, &
     &2.5252e+00_rb,1.5121e+00_rb,1.0239e+00_rb/)
      kao(:, 2, 9,14) = (/ &
     &9.9292e+00_rb,8.6881e+00_rb,7.4471e+00_rb,6.2062e+00_rb,4.9656e+00_rb,3.7249e+00_rb, &
     &2.4866e+00_rb,1.4902e+00_rb,1.0036e+00_rb/)
      kao(:, 3, 9,14) = (/ &
     &9.7859e+00_rb,8.5628e+00_rb,7.3398e+00_rb,6.1169e+00_rb,4.8940e+00_rb,3.6711e+00_rb, &
     &2.4492e+00_rb,1.4699e+00_rb,9.8554e-01_rb/)
      kao(:, 4, 9,14) = (/ &
     &9.6386e+00_rb,8.4340e+00_rb,7.2293e+00_rb,6.0249e+00_rb,4.8203e+00_rb,3.6156e+00_rb, &
     &2.4114e+00_rb,1.4506e+00_rb,9.7113e-01_rb/)
      kao(:, 5, 9,14) = (/ &
     &9.4939e+00_rb,8.3072e+00_rb,7.1206e+00_rb,5.9343e+00_rb,4.7478e+00_rb,3.5614e+00_rb, &
     &2.3752e+00_rb,1.4305e+00_rb,9.6050e-01_rb/)
      kao(:, 1,10,14) = (/ &
     &5.2612e+01_rb,4.6036e+01_rb,3.9459e+01_rb,3.2882e+01_rb,2.6307e+01_rb,1.9730e+01_rb, &
     &1.3154e+01_rb,6.5782e+00_rb,1.1007e+00_rb/)
      kao(:, 2,10,14) = (/ &
     &5.2045e+01_rb,4.5541e+01_rb,3.9035e+01_rb,3.2530e+01_rb,2.6024e+01_rb,1.9518e+01_rb, &
     &1.3013e+01_rb,6.5076e+00_rb,1.0772e+00_rb/)
      kao(:, 3,10,14) = (/ &
     &5.1396e+01_rb,4.4972e+01_rb,3.8548e+01_rb,3.2123e+01_rb,2.5700e+01_rb,1.9274e+01_rb, &
     &1.2851e+01_rb,6.4259e+00_rb,1.0576e+00_rb/)
      kao(:, 4,10,14) = (/ &
     &5.0681e+01_rb,4.4348e+01_rb,3.8014e+01_rb,3.1677e+01_rb,2.5342e+01_rb,1.9007e+01_rb, &
     &1.2671e+01_rb,6.3366e+00_rb,1.0433e+00_rb/)
      kao(:, 5,10,14) = (/ &
     &4.9909e+01_rb,4.3671e+01_rb,3.7433e+01_rb,3.1194e+01_rb,2.4957e+01_rb,1.8717e+01_rb, &
     &1.2479e+01_rb,6.2406e+00_rb,1.0315e+00_rb/)
      kao(:, 1,11,14) = (/ &
     &9.5447e+01_rb,8.3519e+01_rb,7.1586e+01_rb,5.9654e+01_rb,4.7724e+01_rb,3.5792e+01_rb, &
     &2.3862e+01_rb,1.1932e+01_rb,1.1580e+00_rb/)
      kao(:, 2,11,14) = (/ &
     &9.4633e+01_rb,8.2803e+01_rb,7.0974e+01_rb,5.9145e+01_rb,4.7317e+01_rb,3.5487e+01_rb, &
     &2.3658e+01_rb,1.1830e+01_rb,1.1359e+00_rb/)
      kao(:, 3,11,14) = (/ &
     &9.3512e+01_rb,8.1822e+01_rb,7.0137e+01_rb,5.8447e+01_rb,4.6756e+01_rb,3.5068e+01_rb, &
     &2.3379e+01_rb,1.1690e+01_rb,1.1191e+00_rb/)
      kao(:, 4,11,14) = (/ &
     &9.1943e+01_rb,8.0451e+01_rb,6.8957e+01_rb,5.7463e+01_rb,4.5972e+01_rb,3.4478e+01_rb, &
     &2.2985e+01_rb,1.1494e+01_rb,1.1048e+00_rb/)
      kao(:, 5,11,14) = (/ &
     &9.0366e+01_rb,7.9072e+01_rb,6.7775e+01_rb,5.6480e+01_rb,4.5183e+01_rb,3.3888e+01_rb, &
     &2.2593e+01_rb,1.1297e+01_rb,1.0941e+00_rb/)
      kao(:, 1,12,14) = (/ &
     &1.2829e+02_rb,1.1225e+02_rb,9.6218e+01_rb,8.0178e+01_rb,6.4148e+01_rb,4.8109e+01_rb, &
     &3.2071e+01_rb,1.6037e+01_rb,1.2082e+00_rb/)
      kao(:, 2,12,14) = (/ &
     &1.2686e+02_rb,1.1100e+02_rb,9.5144e+01_rb,7.9288e+01_rb,6.3431e+01_rb,4.7572e+01_rb, &
     &3.1716e+01_rb,1.5858e+01_rb,1.1882e+00_rb/)
      kao(:, 3,12,14) = (/ &
     &1.2492e+02_rb,1.0930e+02_rb,9.3686e+01_rb,7.8073e+01_rb,6.2461e+01_rb,4.6846e+01_rb, &
     &3.1229e+01_rb,1.5615e+01_rb,1.1735e+00_rb/)
      kao(:, 4,12,14) = (/ &
     &1.2277e+02_rb,1.0742e+02_rb,9.2079e+01_rb,7.6735e+01_rb,6.1388e+01_rb,4.6041e+01_rb, &
     &3.0695e+01_rb,1.5349e+01_rb,1.1628e+00_rb/)
      kao(:, 5,12,14) = (/ &
     &1.2083e+02_rb,1.0573e+02_rb,9.0624e+01_rb,7.5519e+01_rb,6.0418e+01_rb,4.5314e+01_rb, &
     &3.0211e+01_rb,1.5107e+01_rb,1.1538e+00_rb/)
      kao(:, 1,13,14) = (/ &
     &1.3638e+02_rb,1.1933e+02_rb,1.0229e+02_rb,8.5232e+01_rb,6.8189e+01_rb,5.1142e+01_rb, &
     &3.4096e+01_rb,1.7048e+01_rb,1.2383e+00_rb/)
      kao(:, 2,13,14) = (/ &
     &1.3472e+02_rb,1.1788e+02_rb,1.0104e+02_rb,8.4200e+01_rb,6.7359e+01_rb,5.0520e+01_rb, &
     &3.3681e+01_rb,1.6841e+01_rb,1.2326e+00_rb/)
      kao(:, 3,13,14) = (/ &
     &1.3259e+02_rb,1.1602e+02_rb,9.9447e+01_rb,8.2873e+01_rb,6.6300e+01_rb,4.9725e+01_rb, &
     &3.3151e+01_rb,1.6576e+01_rb,1.2214e+00_rb/)
      kao(:, 4,13,14) = (/ &
     &1.3062e+02_rb,1.1429e+02_rb,9.7966e+01_rb,8.1640e+01_rb,6.5311e+01_rb,4.8985e+01_rb, &
     &3.2658e+01_rb,1.6329e+01_rb,1.2124e+00_rb/)
      kao(:, 5,13,14) = (/ &
     &1.2869e+02_rb,1.1261e+02_rb,9.6524e+01_rb,8.0437e+01_rb,6.4351e+01_rb,4.8263e+01_rb, &
     &3.2178e+01_rb,1.6090e+01_rb,1.2063e+00_rb/)
      kao(:, 1, 1,15) = (/ &
     &1.6224e-02_rb,5.5981e-02_rb,1.1058e-01_rb,1.6311e-01_rb,2.1224e-01_rb,2.5534e-01_rb, &
     &2.8563e-01_rb,2.8102e-01_rb,4.2449e-01_rb/)
      kao(:, 2, 1,15) = (/ &
     &1.5819e-02_rb,5.4235e-02_rb,1.0708e-01_rb,1.5789e-01_rb,2.0538e-01_rb,2.4690e-01_rb, &
     &2.7600e-01_rb,2.7172e-01_rb,4.1075e-01_rb/)
      kao(:, 3, 1,15) = (/ &
     &1.5437e-02_rb,5.3030e-02_rb,1.0453e-01_rb,1.5410e-01_rb,2.0034e-01_rb,2.4064e-01_rb, &
     &2.6869e-01_rb,2.6394e-01_rb,4.0067e-01_rb/)
      kao(:, 4, 1,15) = (/ &
     &1.5047e-02_rb,5.2309e-02_rb,1.0294e-01_rb,1.5163e-01_rb,1.9708e-01_rb,2.3671e-01_rb, &
     &2.6420e-01_rb,2.5937e-01_rb,3.9409e-01_rb/)
      kao(:, 5, 1,15) = (/ &
     &1.4670e-02_rb,5.2264e-02_rb,1.0263e-01_rb,1.5109e-01_rb,1.9635e-01_rb,2.3592e-01_rb, &
     &2.6369e-01_rb,2.6037e-01_rb,3.9232e-01_rb/)
      kao(:, 1, 2,15) = (/ &
     &2.7874e-02_rb,6.8016e-02_rb,1.3490e-01_rb,2.0013e-01_rb,2.6255e-01_rb,3.1976e-01_rb, &
     &3.6546e-01_rb,3.7626e-01_rb,5.2506e-01_rb/)
      kao(:, 2, 2,15) = (/ &
     &2.7188e-02_rb,6.5949e-02_rb,1.3068e-01_rb,1.9379e-01_rb,2.5414e-01_rb,3.0936e-01_rb, &
     &3.5322e-01_rb,3.6307e-01_rb,5.0826e-01_rb/)
      kao(:, 3, 2,15) = (/ &
     &2.6530e-02_rb,6.4457e-02_rb,1.2748e-01_rb,1.8895e-01_rb,2.4773e-01_rb,3.0143e-01_rb, &
     &3.4397e-01_rb,3.5278e-01_rb,4.9544e-01_rb/)
      kao(:, 4, 2,15) = (/ &
     &2.5866e-02_rb,6.3478e-02_rb,1.2538e-01_rb,1.8573e-01_rb,2.4341e-01_rb,2.9608e-01_rb, &
     &3.3770e-01_rb,3.4604e-01_rb,4.8658e-01_rb/)
      kao(:, 5, 2,15) = (/ &
     &2.5184e-02_rb,6.3069e-02_rb,1.2422e-01_rb,1.8387e-01_rb,2.4089e-01_rb,2.9292e-01_rb, &
     &3.3402e-01_rb,3.4222e-01_rb,4.8125e-01_rb/)
      kao(:, 1, 3,15) = (/ &
     &6.4680e-02_rb,8.3094e-02_rb,1.6505e-01_rb,2.4624e-01_rb,3.2572e-01_rb,4.0189e-01_rb, &
     &4.7033e-01_rb,5.1238e-01_rb,6.5139e-01_rb/)
      kao(:, 2, 3,15) = (/ &
     &6.3121e-02_rb,8.0680e-02_rb,1.6005e-01_rb,2.3872e-01_rb,3.1571e-01_rb,3.8944e-01_rb, &
     &4.5551e-01_rb,4.9522e-01_rb,6.3135e-01_rb/)
      kao(:, 3, 3,15) = (/ &
     &6.1563e-02_rb,7.8867e-02_rb,1.5612e-01_rb,2.3258e-01_rb,3.0746e-01_rb,3.7914e-01_rb, &
     &4.4323e-01_rb,4.8117e-01_rb,6.1483e-01_rb/)
      kao(:, 4, 3,15) = (/ &
     &6.0025e-02_rb,7.7639e-02_rb,1.5328e-01_rb,2.2820e-01_rb,3.0153e-01_rb,3.7164e-01_rb, &
     &4.3426e-01_rb,4.7103e-01_rb,6.0254e-01_rb/)
      kao(:, 5, 3,15) = (/ &
     &5.8411e-02_rb,7.7256e-02_rb,1.5120e-01_rb,2.2490e-01_rb,2.9704e-01_rb,3.6599e-01_rb, &
     &4.2759e-01_rb,4.6376e-01_rb,5.9316e-01_rb/)
      kao(:, 1, 4,15) = (/ &
     &1.5257e-01_rb,1.3544e-01_rb,2.0077e-01_rb,3.0036e-01_rb,3.9895e-01_rb,4.9562e-01_rb, &
     &5.8758e-01_rb,6.6215e-01_rb,7.9779e-01_rb/)
      kao(:, 2, 4,15) = (/ &
     &1.4896e-01_rb,1.3204e-01_rb,1.9507e-01_rb,2.9140e-01_rb,3.8695e-01_rb,4.8067e-01_rb, &
     &5.6978e-01_rb,6.4163e-01_rb,7.7379e-01_rb/)
      kao(:, 3, 4,15) = (/ &
     &1.4526e-01_rb,1.2874e-01_rb,1.9046e-01_rb,2.8415e-01_rb,3.7702e-01_rb,4.6809e-01_rb, &
     &5.5459e-01_rb,6.2416e-01_rb,7.5329e-01_rb/)
      kao(:, 4, 4,15) = (/ &
     &1.4150e-01_rb,1.2551e-01_rb,1.8699e-01_rb,2.7869e-01_rb,3.6960e-01_rb,4.5869e-01_rb, &
     &5.4325e-01_rb,6.1092e-01_rb,7.3794e-01_rb/)
      kao(:, 5, 4,15) = (/ &
     &1.3788e-01_rb,1.2292e-01_rb,1.8424e-01_rb,2.7423e-01_rb,3.6354e-01_rb,4.5105e-01_rb, &
     &5.3408e-01_rb,6.0044e-01_rb,7.2537e-01_rb/)
      kao(:, 1, 5,15) = (/ &
     &3.2844e-01_rb,2.8856e-01_rb,2.7979e-01_rb,3.6340e-01_rb,4.8359e-01_rb,6.0259e-01_rb, &
     &7.1867e-01_rb,8.2355e-01_rb,9.6702e-01_rb/)
      kao(:, 2, 5,15) = (/ &
     &3.2063e-01_rb,2.8155e-01_rb,2.6962e-01_rb,3.5320e-01_rb,4.6960e-01_rb,5.8494e-01_rb, &
     &6.9755e-01_rb,7.9908e-01_rb,9.3875e-01_rb/)
      kao(:, 3, 5,15) = (/ &
     &3.1261e-01_rb,2.7434e-01_rb,2.6034e-01_rb,3.4461e-01_rb,4.5775e-01_rb,5.6986e-01_rb, &
     &6.7919e-01_rb,7.7764e-01_rb,9.1375e-01_rb/)
      kao(:, 4, 5,15) = (/ &
     &3.0440e-01_rb,2.6704e-01_rb,2.5252e-01_rb,3.3787e-01_rb,4.4841e-01_rb,5.5807e-01_rb, &
     &6.6481e-01_rb,7.6089e-01_rb,8.9431e-01_rb/)
      kao(:, 5, 5,15) = (/ &
     &2.9686e-01_rb,2.6035e-01_rb,2.4591e-01_rb,3.3225e-01_rb,4.4072e-01_rb,5.4821e-01_rb, &
     &6.5299e-01_rb,7.4709e-01_rb,8.7822e-01_rb/)
      kao(:, 1, 6,15) = (/ &
     &6.6195e-01_rb,5.8013e-01_rb,4.9831e-01_rb,4.9605e-01_rb,5.8178e-01_rb,7.2595e-01_rb, &
     &8.6836e-01_rb,1.0033e+00_rb,1.1633e+00_rb/)
      kao(:, 2, 6,15) = (/ &
     &6.4501e-01_rb,5.6519e-01_rb,4.8535e-01_rb,4.7586e-01_rb,5.6606e-01_rb,7.0584e-01_rb, &
     &8.4395e-01_rb,9.7492e-01_rb,1.1306e+00_rb/)
      kao(:, 3, 6,15) = (/ &
     &6.2808e-01_rb,5.5020e-01_rb,4.7240e-01_rb,4.5801e-01_rb,5.5243e-01_rb,6.8837e-01_rb, &
     &8.2269e-01_rb,9.4987e-01_rb,1.1015e+00_rb/)
      kao(:, 4, 6,15) = (/ &
     &6.1101e-01_rb,5.3520e-01_rb,4.5941e-01_rb,4.4278e-01_rb,5.4121e-01_rb,6.7408e-01_rb, &
     &8.0538e-01_rb,9.2955e-01_rb,1.0781e+00_rb/)
      kao(:, 5, 6,15) = (/ &
     &5.9524e-01_rb,5.2132e-01_rb,4.4745e-01_rb,4.2965e-01_rb,5.3187e-01_rb,6.6218e-01_rb, &
     &7.9084e-01_rb,9.1257e-01_rb,1.0584e+00_rb/)
      kao(:, 1, 7,15) = (/ &
     &1.4144e+00_rb,1.2384e+00_rb,1.0623e+00_rb,8.8626e-01_rb,8.1737e-01_rb,8.6950e-01_rb, &
     &1.0395e+00_rb,1.2069e+00_rb,1.3892e+00_rb/)
      kao(:, 2, 7,15) = (/ &
     &1.3748e+00_rb,1.2036e+00_rb,1.0323e+00_rb,8.6117e-01_rb,7.8383e-01_rb,8.4592e-01_rb, &
     &1.0122e+00_rb,1.1745e+00_rb,1.3519e+00_rb/)
      kao(:, 3, 7,15) = (/ &
     &1.3356e+00_rb,1.1692e+00_rb,1.0028e+00_rb,8.3643e-01_rb,7.5357e-01_rb,8.2575e-01_rb, &
     &9.8798e-01_rb,1.1461e+00_rb,1.3189e+00_rb/)
      kao(:, 4, 7,15) = (/ &
     &1.2972e+00_rb,1.1356e+00_rb,9.7391e-01_rb,8.1222e-01_rb,7.2715e-01_rb,8.0916e-01_rb, &
     &9.6777e-01_rb,1.1223e+00_rb,1.2914e+00_rb/)
      kao(:, 5, 7,15) = (/ &
     &1.2621e+00_rb,1.1049e+00_rb,9.4752e-01_rb,7.9014e-01_rb,7.0399e-01_rb,7.9481e-01_rb, &
     &9.5022e-01_rb,1.1017e+00_rb,1.2675e+00_rb/)
      kao(:, 1, 8,15) = (/ &
     &3.5678e+00_rb,3.1226e+00_rb,2.6774e+00_rb,2.2323e+00_rb,1.7871e+00_rb,1.4068e+00_rb, &
     &1.2751e+00_rb,1.4347e+00_rb,1.6442e+00_rb/)
      kao(:, 2, 8,15) = (/ &
     &3.4620e+00_rb,3.0298e+00_rb,2.5977e+00_rb,2.1657e+00_rb,1.7337e+00_rb,1.3525e+00_rb, &
     &1.2330e+00_rb,1.4002e+00_rb,1.6038e+00_rb/)
      kao(:, 3, 8,15) = (/ &
     &3.3581e+00_rb,2.9387e+00_rb,2.5197e+00_rb,2.1005e+00_rb,1.6813e+00_rb,1.3030e+00_rb, &
     &1.1987e+00_rb,1.3683e+00_rb,1.5668e+00_rb/)
      kao(:, 4, 8,15) = (/ &
     &3.2581e+00_rb,2.8514e+00_rb,2.4446e+00_rb,2.0378e+00_rb,1.6310e+00_rb,1.2580e+00_rb, &
     &1.1700e+00_rb,1.3407e+00_rb,1.5349e+00_rb/)
      kao(:, 5, 8,15) = (/ &
     &3.1664e+00_rb,2.7709e+00_rb,2.3756e+00_rb,1.9802e+00_rb,1.5848e+00_rb,1.2171e+00_rb, &
     &1.1461e+00_rb,1.3159e+00_rb,1.5061e+00_rb/)
      kao(:, 1, 9,15) = (/ &
     &1.6997e+01_rb,1.4874e+01_rb,1.2750e+01_rb,1.0625e+01_rb,8.5017e+00_rb,6.3777e+00_rb, &
     &4.2536e+00_rb,2.2902e+00_rb,1.9313e+00_rb/)
      kao(:, 2, 9,15) = (/ &
     &1.6471e+01_rb,1.4414e+01_rb,1.2355e+01_rb,1.0296e+01_rb,8.2389e+00_rb,6.1803e+00_rb, &
     &4.1219e+00_rb,2.1942e+00_rb,1.8874e+00_rb/)
      kao(:, 3, 9,15) = (/ &
     &1.5959e+01_rb,1.3964e+01_rb,1.1970e+01_rb,9.9759e+00_rb,7.9817e+00_rb,5.9872e+00_rb, &
     &3.9932e+00_rb,2.1070e+00_rb,1.8460e+00_rb/)
      kao(:, 4, 9,15) = (/ &
     &1.5470e+01_rb,1.3538e+01_rb,1.1604e+01_rb,9.6708e+00_rb,7.7374e+00_rb,5.8040e+00_rb, &
     &3.8707e+00_rb,2.0267e+00_rb,1.8089e+00_rb/)
      kao(:, 5, 9,15) = (/ &
     &1.5020e+01_rb,1.3143e+01_rb,1.1265e+01_rb,9.3885e+00_rb,7.5114e+00_rb,5.6345e+00_rb, &
     &3.7577e+00_rb,1.9548e+00_rb,1.7742e+00_rb/)
      kao(:, 1,10,15) = (/ &
     &9.0636e+01_rb,7.9307e+01_rb,6.7977e+01_rb,5.6649e+01_rb,4.5320e+01_rb,3.3991e+01_rb, &
     &2.2662e+01_rb,1.1334e+01_rb,2.2438e+00_rb/)
      kao(:, 2,10,15) = (/ &
     &8.7565e+01_rb,7.6621e+01_rb,6.5675e+01_rb,5.4728e+01_rb,4.3786e+01_rb,3.2842e+01_rb, &
     &2.1895e+01_rb,1.0950e+01_rb,2.1955e+00_rb/)
      kao(:, 3,10,15) = (/ &
     &8.4686e+01_rb,7.4100e+01_rb,6.3516e+01_rb,5.2929e+01_rb,4.2345e+01_rb,3.1759e+01_rb, &
     &2.1174e+01_rb,1.0590e+01_rb,2.1489e+00_rb/)
      kao(:, 4,10,15) = (/ &
     &8.2038e+01_rb,7.1783e+01_rb,6.1529e+01_rb,5.1276e+01_rb,4.1020e+01_rb,3.0766e+01_rb, &
     &2.0512e+01_rb,1.0258e+01_rb,2.1057e+00_rb/)
      kao(:, 5,10,15) = (/ &
     &7.9656e+01_rb,6.9701e+01_rb,5.9744e+01_rb,4.9787e+01_rb,3.9829e+01_rb,2.9874e+01_rb, &
     &1.9917e+01_rb,9.9593e+00_rb,2.0648e+00_rb/)
      kao(:, 1,11,15) = (/ &
     &1.6605e+02_rb,1.4529e+02_rb,1.2455e+02_rb,1.0378e+02_rb,8.3029e+01_rb,6.2280e+01_rb, &
     &4.1517e+01_rb,2.0760e+01_rb,2.5530e+00_rb/)
      kao(:, 2,11,15) = (/ &
     &1.6006e+02_rb,1.4006e+02_rb,1.2005e+02_rb,1.0004e+02_rb,8.0031e+01_rb,6.0027e+01_rb, &
     &4.0019e+01_rb,2.0011e+01_rb,2.4994e+00_rb/)
      kao(:, 3,11,15) = (/ &
     &1.5475e+02_rb,1.3539e+02_rb,1.1606e+02_rb,9.6709e+01_rb,7.7371e+01_rb,5.8029e+01_rb, &
     &3.8687e+01_rb,1.9345e+01_rb,2.4484e+00_rb/)
      kao(:, 4,11,15) = (/ &
     &1.5058e+02_rb,1.3176e+02_rb,1.1294e+02_rb,9.4118e+01_rb,7.5292e+01_rb,5.6471e+01_rb, &
     &3.7649e+01_rb,1.8825e+01_rb,2.4006e+00_rb/)
      kao(:, 5,11,15) = (/ &
     &1.4680e+02_rb,1.2845e+02_rb,1.1009e+02_rb,9.1748e+01_rb,7.3399e+01_rb,5.5050e+01_rb, &
     &3.6700e+01_rb,1.8351e+01_rb,2.3540e+00_rb/)
      kao(:, 1,12,15) = (/ &
     &2.2602e+02_rb,1.9778e+02_rb,1.6953e+02_rb,1.4127e+02_rb,1.1301e+02_rb,8.4757e+01_rb, &
     &5.6507e+01_rb,2.8257e+01_rb,2.8716e+00_rb/)
      kao(:, 2,12,15) = (/ &
     &2.1869e+02_rb,1.9135e+02_rb,1.6402e+02_rb,1.3669e+02_rb,1.0935e+02_rb,8.2013e+01_rb, &
     &5.4674e+01_rb,2.7339e+01_rb,2.8132e+00_rb/)
      kao(:, 3,12,15) = (/ &
     &2.1253e+02_rb,1.8597e+02_rb,1.5940e+02_rb,1.3282e+02_rb,1.0626e+02_rb,7.9692e+01_rb, &
     &5.3134e+01_rb,2.6567e+01_rb,2.7573e+00_rb/)
      kao(:, 4,12,15) = (/ &
     &2.0725e+02_rb,1.8134e+02_rb,1.5543e+02_rb,1.2952e+02_rb,1.0362e+02_rb,7.7720e+01_rb, &
     &5.1811e+01_rb,2.5907e+01_rb,2.7022e+00_rb/)
      kao(:, 5,12,15) = (/ &
     &2.0216e+02_rb,1.7690e+02_rb,1.5162e+02_rb,1.2635e+02_rb,1.0108e+02_rb,7.5810e+01_rb, &
     &5.0540e+01_rb,2.5271e+01_rb,2.6508e+00_rb/)
      kao(:, 1,13,15) = (/ &
     &2.4880e+02_rb,2.1770e+02_rb,1.8661e+02_rb,1.5550e+02_rb,1.2440e+02_rb,9.3305e+01_rb, &
     &6.2205e+01_rb,3.1105e+01_rb,3.1971e+00_rb/)
      kao(:, 2,13,15) = (/ &
     &2.4131e+02_rb,2.1114e+02_rb,1.8097e+02_rb,1.5081e+02_rb,1.2065e+02_rb,9.0483e+01_rb, &
     &6.0327e+01_rb,3.0165e+01_rb,3.1338e+00_rb/)
      kao(:, 3,13,15) = (/ &
     &2.3499e+02_rb,2.0562e+02_rb,1.7624e+02_rb,1.4687e+02_rb,1.1750e+02_rb,8.8123e+01_rb, &
     &5.8749e+01_rb,2.9377e+01_rb,3.0697e+00_rb/)
      kao(:, 4,13,15) = (/ &
     &2.2900e+02_rb,2.0038e+02_rb,1.7175e+02_rb,1.4313e+02_rb,1.1451e+02_rb,8.5878e+01_rb, &
     &5.7253e+01_rb,2.8627e+01_rb,3.0070e+00_rb/)
      kao(:, 5,13,15) = (/ &
     &2.2306e+02_rb,1.9518e+02_rb,1.6729e+02_rb,1.3941e+02_rb,1.1152e+02_rb,8.3649e+01_rb, &
     &5.5766e+01_rb,2.7883e+01_rb,2.9481e+00_rb/)
      kao(:, 1, 1,16) = (/ &
     &1.9445e-02_rb,6.3230e-02_rb,1.2479e-01_rb,1.8384e-01_rb,2.3882e-01_rb,2.8643e-01_rb, &
     &3.1893e-01_rb,3.1112e-01_rb,4.7763e-01_rb/)
      kao(:, 2, 1,16) = (/ &
     &1.8835e-02_rb,6.2634e-02_rb,1.2363e-01_rb,1.8213e-01_rb,2.3659e-01_rb,2.8379e-01_rb, &
     &3.1600e-01_rb,3.0828e-01_rb,4.7319e-01_rb/)
      kao(:, 3, 1,16) = (/ &
     &1.8228e-02_rb,6.1941e-02_rb,1.2224e-01_rb,1.8013e-01_rb,2.3399e-01_rb,2.8070e-01_rb, &
     &3.1257e-01_rb,3.0493e-01_rb,4.6798e-01_rb/)
      kao(:, 4, 1,16) = (/ &
     &1.7647e-02_rb,6.1170e-02_rb,1.2073e-01_rb,1.7790e-01_rb,2.3107e-01_rb,2.7723e-01_rb, &
     &3.0877e-01_rb,3.0124e-01_rb,4.6215e-01_rb/)
      kao(:, 5, 1,16) = (/ &
     &1.7079e-02_rb,6.0350e-02_rb,1.1912e-01_rb,1.7548e-01_rb,2.2801e-01_rb,2.7357e-01_rb, &
     &3.0466e-01_rb,2.9733e-01_rb,4.5602e-01_rb/)
      kao(:, 1, 2,16) = (/ &
     &3.4634e-02_rb,7.7044e-02_rb,1.5273e-01_rb,2.2632e-01_rb,2.9575e-01_rb,3.6021e-01_rb, &
     &4.0997e-01_rb,4.1812e-01_rb,5.9296e-01_rb/)
      kao(:, 2, 2,16) = (/ &
     &3.3420e-02_rb,7.6207e-02_rb,1.5110e-01_rb,2.2402e-01_rb,2.9356e-01_rb,3.5685e-01_rb, &
     &4.0638e-01_rb,4.1457e-01_rb,5.8711e-01_rb/)
      kao(:, 3, 2,16) = (/ &
     &3.2246e-02_rb,7.5373e-02_rb,1.4946e-01_rb,2.2157e-01_rb,2.9037e-01_rb,3.5299e-01_rb, &
     &4.0204e-01_rb,4.1028e-01_rb,5.8073e-01_rb/)
      kao(:, 4, 2,16) = (/ &
     &3.1106e-02_rb,7.4437e-02_rb,1.4762e-01_rb,2.1883e-01_rb,2.8679e-01_rb,3.4868e-01_rb, &
     &3.9718e-01_rb,4.0541e-01_rb,5.7357e-01_rb/)
      kao(:, 5, 2,16) = (/ &
     &3.0021e-02_rb,7.3426e-02_rb,1.4561e-01_rb,2.1587e-01_rb,2.8293e-01_rb,3.4400e-01_rb, &
     &3.9193e-01_rb,4.0015e-01_rb,5.6584e-01_rb/)
      kao(:, 1, 3,16) = (/ &
     &8.3051e-02_rb,9.4793e-02_rb,1.8876e-01_rb,2.8144e-01_rb,3.7196e-01_rb,4.5816e-01_rb, &
     &5.3450e-01_rb,5.7690e-01_rb,7.4391e-01_rb/)
      kao(:, 2, 3,16) = (/ &
     &7.9830e-02_rb,9.3082e-02_rb,1.8538e-01_rb,2.7644e-01_rb,3.6534e-01_rb,4.5017e-01_rb, &
     &5.2550e-01_rb,5.6808e-01_rb,7.3067e-01_rb/)
      kao(:, 3, 3,16) = (/ &
     &7.6737e-02_rb,9.1705e-02_rb,1.8269e-01_rb,2.7246e-01_rb,3.6014e-01_rb,4.4395e-01_rb, &
     &5.1857e-01_rb,5.6175e-01_rb,7.2028e-01_rb/)
      kao(:, 4, 3,16) = (/ &
     &7.3809e-02_rb,9.0497e-02_rb,1.8026e-01_rb,2.6888e-01_rb,3.5554e-01_rb,4.3839e-01_rb, &
     &5.1223e-01_rb,5.5508e-01_rb,7.1107e-01_rb/)
      kao(:, 5, 3,16) = (/ &
     &7.1033e-02_rb,8.9276e-02_rb,1.7779e-01_rb,2.6523e-01_rb,3.5066e-01_rb,4.3243e-01_rb, &
     &5.0536e-01_rb,5.4783e-01_rb,7.0132e-01_rb/)
      kao(:, 1, 4,16) = (/ &
     &2.0125e-01_rb,1.7779e-01_rb,2.3399e-01_rb,3.4997e-01_rb,4.6464e-01_rb,5.7674e-01_rb, &
     &6.8253e-01_rb,7.6508e-01_rb,9.2928e-01_rb/)
      kao(:, 2, 4,16) = (/ &
     &1.9265e-01_rb,1.7048e-01_rb,2.2831e-01_rb,3.4146e-01_rb,4.5338e-01_rb,5.6281e-01_rb, &
     &6.6618e-01_rb,7.4718e-01_rb,9.0679e-01_rb/)
      kao(:, 3, 4,16) = (/ &
     &1.8456e-01_rb,1.6365e-01_rb,2.2378e-01_rb,3.3476e-01_rb,4.4451e-01_rb,5.5182e-01_rb, &
     &6.5330e-01_rb,7.3313e-01_rb,8.8901e-01_rb/)
      kao(:, 4, 4,16) = (/ &
     &1.7702e-01_rb,1.5723e-01_rb,2.1978e-01_rb,3.2875e-01_rb,4.3660e-01_rb,5.4199e-01_rb, &
     &6.4188e-01_rb,7.2098e-01_rb,8.7320e-01_rb/)
      kao(:, 5, 4,16) = (/ &
     &1.6997e-01_rb,1.5116e-01_rb,2.1600e-01_rb,3.2314e-01_rb,4.2913e-01_rb,5.3288e-01_rb, &
     &6.3120e-01_rb,7.0945e-01_rb,8.5826e-01_rb/)
      kao(:, 1, 5,16) = (/ &
     &4.4228e-01_rb,3.8795e-01_rb,3.3395e-01_rb,4.3605e-01_rb,5.8013e-01_rb,7.2255e-01_rb, &
     &8.6084e-01_rb,9.8346e-01_rb,1.1603e+00_rb/)
      kao(:, 2, 5,16) = (/ &
     &4.2199e-01_rb,3.7056e-01_rb,3.1956e-01_rb,4.2215e-01_rb,5.6164e-01_rb,6.9945e-01_rb, &
     &8.3347e-01_rb,9.5217e-01_rb,1.1233e+00_rb/)
      kao(:, 3, 5,16) = (/ &
     &4.0375e-01_rb,3.5492e-01_rb,3.0638e-01_rb,4.1150e-01_rb,5.4754e-01_rb,6.8200e-01_rb, &
     &8.1260e-01_rb,9.2875e-01_rb,1.0951e+00_rb/)
      kao(:, 4, 5,16) = (/ &
     &3.8716e-01_rb,3.4051e-01_rb,2.9411e-01_rb,4.0247e-01_rb,5.3546e-01_rb,6.6706e-01_rb, &
     &7.9494e-01_rb,9.0883e-01_rb,1.0709e+00_rb/)
      kao(:, 5, 5,16) = (/ &
     &3.7160e-01_rb,3.2689e-01_rb,2.8321e-01_rb,3.9414e-01_rb,5.2444e-01_rb,6.5338e-01_rb, &
     &7.7865e-01_rb,8.9056e-01_rb,1.0489e+00_rb/)
      kao(:, 1, 6,16) = (/ &
     &9.0506e-01_rb,7.9284e-01_rb,6.8068e-01_rb,5.6916e-01_rb,7.2503e-01_rb,9.0468e-01_rb, &
     &1.0813e+00_rb,1.2473e+00_rb,1.4500e+00_rb/)
      kao(:, 2, 6,16) = (/ &
     &8.6465e-01_rb,7.5773e-01_rb,6.5086e-01_rb,5.4531e-01_rb,6.9722e-01_rb,8.6983e-01_rb, &
     &1.0397e+00_rb,1.1992e+00_rb,1.3944e+00_rb/)
      kao(:, 3, 6,16) = (/ &
     &8.2825e-01_rb,7.2598e-01_rb,6.2379e-01_rb,5.2549e-01_rb,6.7605e-01_rb,8.4340e-01_rb, &
     &1.0082e+00_rb,1.1629e+00_rb,1.3521e+00_rb/)
      kao(:, 4, 6,16) = (/ &
     &7.9420e-01_rb,6.9632e-01_rb,5.9834e-01_rb,5.1029e-01_rb,6.5811e-01_rb,8.2106e-01_rb, &
     &9.8160e-01_rb,1.1324e+00_rb,1.3162e+00_rb/)
      kao(:, 5, 6,16) = (/ &
     &7.6215e-01_rb,6.6829e-01_rb,5.7431e-01_rb,4.9359e-01_rb,6.4194e-01_rb,8.0089e-01_rb, &
     &9.5749e-01_rb,1.1049e+00_rb,1.2839e+00_rb/)
      kao(:, 1, 7,16) = (/ &
     &1.9676e+00_rb,1.7224e+00_rb,1.4775e+00_rb,1.2325e+00_rb,9.8756e-01_rb,1.1304e+00_rb, &
     &1.3539e+00_rb,1.5704e+00_rb,1.8105e+00_rb/)
      kao(:, 2, 7,16) = (/ &
     &1.8838e+00_rb,1.6493e+00_rb,1.4147e+00_rb,1.1804e+00_rb,9.4604e-01_rb,1.0806e+00_rb, &
     &1.2942e+00_rb,1.5011e+00_rb,1.7307e+00_rb/)
      kao(:, 3, 7,16) = (/ &
     &1.8058e+00_rb,1.5808e+00_rb,1.3563e+00_rb,1.1318e+00_rb,9.0772e-01_rb,1.0432e+00_rb, &
     &1.2494e+00_rb,1.4491e+00_rb,1.6707e+00_rb/)
      kao(:, 4, 7,16) = (/ &
     &1.7320e+00_rb,1.5166e+00_rb,1.3008e+00_rb,1.0854e+00_rb,8.7621e-01_rb,1.0114e+00_rb, &
     &1.2114e+00_rb,1.4051e+00_rb,1.6197e+00_rb/)
      kao(:, 5, 7,16) = (/ &
     &1.6617e+00_rb,1.4551e+00_rb,1.2484e+00_rb,1.0417e+00_rb,8.4899e-01_rb,9.8293e-01_rb, &
     &1.1773e+00_rb,1.3657e+00_rb,1.5743e+00_rb/)
      kao(:, 1, 8,16) = (/ &
     &5.0769e+00_rb,4.4432e+00_rb,3.8096e+00_rb,3.1755e+00_rb,2.5417e+00_rb,1.9082e+00_rb, &
     &1.6892e+00_rb,1.9659e+00_rb,2.2550e+00_rb/)
      kao(:, 2, 8,16) = (/ &
     &4.8681e+00_rb,4.2603e+00_rb,3.6527e+00_rb,3.0450e+00_rb,2.4373e+00_rb,1.8300e+00_rb, &
     &1.6072e+00_rb,1.8705e+00_rb,2.1455e+00_rb/)
      kao(:, 3, 8,16) = (/ &
     &4.6692e+00_rb,4.0861e+00_rb,3.5032e+00_rb,2.9205e+00_rb,2.3378e+00_rb,1.7550e+00_rb, &
     &1.5446e+00_rb,1.7976e+00_rb,2.0621e+00_rb/)
      kao(:, 4, 8,16) = (/ &
     &4.4792e+00_rb,3.9199e+00_rb,3.3609e+00_rb,2.8016e+00_rb,2.2429e+00_rb,1.6836e+00_rb, &
     &1.4923e+00_rb,1.7367e+00_rb,1.9921e+00_rb/)
      kao(:, 5, 8,16) = (/ &
     &4.2972e+00_rb,3.7609e+00_rb,3.2243e+00_rb,2.6879e+00_rb,2.1517e+00_rb,1.6153e+00_rb, &
     &1.4455e+00_rb,1.6824e+00_rb,1.9297e+00_rb/)
      kao(:, 1, 9,16) = (/ &
     &2.4893e+01_rb,2.1786e+01_rb,1.8675e+01_rb,1.5563e+01_rb,1.2450e+01_rb,9.3398e+00_rb, &
     &6.2286e+00_rb,3.1170e+00_rb,2.8014e+00_rb/)
      kao(:, 2, 9,16) = (/ &
     &2.3897e+01_rb,2.0912e+01_rb,1.7925e+01_rb,1.4937e+01_rb,1.1951e+01_rb,8.9651e+00_rb, &
     &5.9788e+00_rb,2.9923e+00_rb,2.6581e+00_rb/)
      kao(:, 3, 9,16) = (/ &
     &2.2932e+01_rb,2.0066e+01_rb,1.7201e+01_rb,1.4334e+01_rb,1.1468e+01_rb,8.6027e+00_rb, &
     &5.7371e+00_rb,2.8711e+00_rb,2.5442e+00_rb/)
      kao(:, 4, 9,16) = (/ &
     &2.1999e+01_rb,1.9250e+01_rb,1.6501e+01_rb,1.3753e+01_rb,1.1001e+01_rb,8.2533e+00_rb, &
     &5.5038e+00_rb,2.7545e+00_rb,2.4493e+00_rb/)
      kao(:, 5, 9,16) = (/ &
     &2.1105e+01_rb,1.8469e+01_rb,1.5831e+01_rb,1.3195e+01_rb,1.0556e+01_rb,7.9178e+00_rb, &
     &5.2806e+00_rb,2.6423e+00_rb,2.3650e+00_rb/)
      kao(:, 1,10,16) = (/ &
     &1.3702e+02_rb,1.1990e+02_rb,1.0277e+02_rb,8.5642e+01_rb,6.8515e+01_rb,5.1385e+01_rb, &
     &3.4262e+01_rb,1.7133e+01_rb,3.4526e+00_rb/)
      kao(:, 2,10,16) = (/ &
     &1.3162e+02_rb,1.1516e+02_rb,9.8725e+01_rb,8.2273e+01_rb,6.5802e+01_rb,4.9353e+01_rb, &
     &3.2909e+01_rb,1.6459e+01_rb,3.2683e+00_rb/)
      kao(:, 3,10,16) = (/ &
     &1.2631e+02_rb,1.1053e+02_rb,9.4741e+01_rb,7.8943e+01_rb,6.3147e+01_rb,4.7375e+01_rb, &
     &3.1582e+01_rb,1.5793e+01_rb,3.1197e+00_rb/)
      kao(:, 4,10,16) = (/ &
     &1.2113e+02_rb,1.0599e+02_rb,9.0842e+01_rb,7.5703e+01_rb,6.0562e+01_rb,4.5424e+01_rb, &
     &3.0282e+01_rb,1.5145e+01_rb,2.9946e+00_rb/)
      kao(:, 5,10,16) = (/ &
     &1.1642e+02_rb,1.0187e+02_rb,8.7320e+01_rb,7.2762e+01_rb,5.8215e+01_rb,4.3658e+01_rb, &
     &2.9108e+01_rb,1.4557e+01_rb,2.8829e+00_rb/)
      kao(:, 1,11,16) = (/ &
     &2.5880e+02_rb,2.2644e+02_rb,1.9413e+02_rb,1.6174e+02_rb,1.2941e+02_rb,9.7065e+01_rb, &
     &6.4700e+01_rb,3.2352e+01_rb,4.1212e+00_rb/)
      kao(:, 2,11,16) = (/ &
     &2.4838e+02_rb,2.1734e+02_rb,1.8628e+02_rb,1.5524e+02_rb,1.2419e+02_rb,9.3146e+01_rb, &
     &6.2097e+01_rb,3.1052e+01_rb,3.9061e+00_rb/)
      kao(:, 3,11,16) = (/ &
     &2.3869e+02_rb,2.0886e+02_rb,1.7902e+02_rb,1.4918e+02_rb,1.1935e+02_rb,8.9511e+01_rb, &
     &5.9677e+01_rb,2.9841e+01_rb,3.7301e+00_rb/)
      kao(:, 4,11,16) = (/ &
     &2.2943e+02_rb,2.0074e+02_rb,1.7207e+02_rb,1.4339e+02_rb,1.1472e+02_rb,8.6039e+01_rb, &
     &5.7356e+01_rb,2.8682e+01_rb,3.5754e+00_rb/)
      kao(:, 5,11,16) = (/ &
     &2.2131e+02_rb,1.9367e+02_rb,1.6600e+02_rb,1.3834e+02_rb,1.1067e+02_rb,8.3001e+01_rb, &
     &5.5340e+01_rb,2.7668e+01_rb,3.4351e+00_rb/)
      kao(:, 1,12,16) = (/ &
     &3.6519e+02_rb,3.1953e+02_rb,2.7390e+02_rb,2.2827e+02_rb,1.8260e+02_rb,1.3695e+02_rb, &
     &9.1294e+01_rb,4.5654e+01_rb,4.8878e+00_rb/)
      kao(:, 2,12,16) = (/ &
     &3.5163e+02_rb,3.0767e+02_rb,2.6371e+02_rb,2.1977e+02_rb,1.7580e+02_rb,1.3186e+02_rb, &
     &8.7903e+01_rb,4.3953e+01_rb,4.5957e+00_rb/)
      kao(:, 3,12,16) = (/ &
     &3.3818e+02_rb,2.9589e+02_rb,2.5364e+02_rb,2.1136e+02_rb,1.6910e+02_rb,1.2682e+02_rb, &
     &8.4547e+01_rb,4.2272e+01_rb,4.4276e+00_rb/)
      kao(:, 4,12,16) = (/ &
     &3.2523e+02_rb,2.8457e+02_rb,2.4393e+02_rb,2.0329e+02_rb,1.6262e+02_rb,1.2197e+02_rb, &
     &8.1319e+01_rb,4.0657e+01_rb,4.2352e+00_rb/)
      kao(:, 5,12,16) = (/ &
     &3.1377e+02_rb,2.7452e+02_rb,2.3534e+02_rb,1.9609e+02_rb,1.5688e+02_rb,1.1765e+02_rb, &
     &7.8446e+01_rb,3.9220e+01_rb,4.0574e+00_rb/)
      kao(:, 1,13,16) = (/ &
     &4.1560e+02_rb,3.6364e+02_rb,3.1170e+02_rb,2.5976e+02_rb,2.0781e+02_rb,1.5585e+02_rb, &
     &1.0390e+02_rb,5.1953e+01_rb,5.7636e+00_rb/)
      kao(:, 2,13,16) = (/ &
     &3.9970e+02_rb,3.4978e+02_rb,2.9980e+02_rb,2.4983e+02_rb,1.9986e+02_rb,1.4990e+02_rb, &
     &9.9932e+01_rb,4.9970e+01_rb,5.4706e+00_rb/)
      kao(:, 3,13,16) = (/ &
     &3.8378e+02_rb,3.3582e+02_rb,2.8782e+02_rb,2.3987e+02_rb,1.9190e+02_rb,1.4391e+02_rb, &
     &9.5948e+01_rb,4.7978e+01_rb,5.2080e+00_rb/)
      kao(:, 4,13,16) = (/ &
     &3.6900e+02_rb,3.2290e+02_rb,2.7675e+02_rb,2.3063e+02_rb,1.8451e+02_rb,1.3838e+02_rb, &
     &9.2254e+01_rb,4.6129e+01_rb,4.9718e+00_rb/)
      kao(:, 5,13,16) = (/ &
     &3.5587e+02_rb,3.1138e+02_rb,2.6690e+02_rb,2.2242e+02_rb,1.7795e+02_rb,1.3345e+02_rb, &
     &8.8970e+01_rb,4.4486e+01_rb,4.7553e+00_rb/)

!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.

      kbo(:,13, 1) = (/ &
     &6.6242e-04_rb,6.9229e-04_rb,7.2125e-04_rb,7.5111e-04_rb,7.7882e-04_rb/)
      kbo(:,14, 1) = (/ &
     &5.7148e-04_rb,6.0011e-04_rb,6.2434e-04_rb,6.4765e-04_rb,6.6822e-04_rb/)
      kbo(:,15, 1) = (/ &
     &5.0245e-04_rb,5.2495e-04_rb,5.4519e-04_rb,5.6261e-04_rb,5.8004e-04_rb/)
      kbo(:,16, 1) = (/ &
     &4.4219e-04_rb,4.6335e-04_rb,4.7782e-04_rb,4.9293e-04_rb,5.0687e-04_rb/)
      kbo(:,17, 1) = (/ &
     &3.9452e-04_rb,4.1172e-04_rb,4.2558e-04_rb,4.3834e-04_rb,4.5031e-04_rb/)
      kbo(:,18, 1) = (/ &
     &3.5502e-04_rb,3.7021e-04_rb,3.8062e-04_rb,3.9326e-04_rb,4.0413e-04_rb/)
      kbo(:,19, 1) = (/ &
     &3.1835e-04_rb,3.3204e-04_rb,3.4438e-04_rb,3.5308e-04_rb,3.6251e-04_rb/)
      kbo(:,20, 1) = (/ &
     &2.7685e-04_rb,2.8739e-04_rb,2.9799e-04_rb,3.0565e-04_rb,3.1346e-04_rb/)
      kbo(:,21, 1) = (/ &
     &2.3803e-04_rb,2.4613e-04_rb,2.5507e-04_rb,2.6118e-04_rb,2.6805e-04_rb/)
      kbo(:,22, 1) = (/ &
     &2.0518e-04_rb,2.1064e-04_rb,2.1799e-04_rb,2.2345e-04_rb,2.2945e-04_rb/)
      kbo(:,23, 1) = (/ &
     &1.7609e-04_rb,1.8136e-04_rb,1.8691e-04_rb,1.9179e-04_rb,1.9680e-04_rb/)
      kbo(:,24, 1) = (/ &
     &1.5133e-04_rb,1.5582e-04_rb,1.6020e-04_rb,1.6517e-04_rb,1.6912e-04_rb/)
      kbo(:,25, 1) = (/ &
     &1.3023e-04_rb,1.3355e-04_rb,1.3826e-04_rb,1.4195e-04_rb,1.4464e-04_rb/)
      kbo(:,26, 1) = (/ &
     &1.1153e-04_rb,1.1437e-04_rb,1.1820e-04_rb,1.2126e-04_rb,1.2331e-04_rb/)
      kbo(:,27, 1) = (/ &
     &9.5262e-05_rb,9.8055e-05_rb,1.0080e-04_rb,1.0342e-04_rb,1.0508e-04_rb/)
      kbo(:,28, 1) = (/ &
     &8.1680e-05_rb,8.3691e-05_rb,8.6133e-05_rb,8.8128e-05_rb,8.9712e-05_rb/)
      kbo(:,29, 1) = (/ &
     &7.0309e-05_rb,7.1883e-05_rb,7.3686e-05_rb,7.5287e-05_rb,7.6689e-05_rb/)
      kbo(:,30, 1) = (/ &
     &6.0539e-05_rb,6.1910e-05_rb,6.3188e-05_rb,6.4488e-05_rb,6.6886e-05_rb/)
      kbo(:,31, 1) = (/ &
     &5.2744e-05_rb,5.3514e-05_rb,5.4453e-05_rb,5.5692e-05_rb,5.7687e-05_rb/)
      kbo(:,32, 1) = (/ &
     &4.5908e-05_rb,4.6319e-05_rb,4.7373e-05_rb,4.8133e-05_rb,4.9705e-05_rb/)
      kbo(:,33, 1) = (/ &
     &3.9867e-05_rb,4.0547e-05_rb,4.1159e-05_rb,4.1892e-05_rb,4.3045e-05_rb/)
      kbo(:,34, 1) = (/ &
     &3.4676e-05_rb,3.5573e-05_rb,3.6016e-05_rb,3.7022e-05_rb,3.7402e-05_rb/)
      kbo(:,35, 1) = (/ &
     &3.0380e-05_rb,3.1177e-05_rb,3.1680e-05_rb,3.2397e-05_rb,3.2578e-05_rb/)
      kbo(:,36, 1) = (/ &
     &2.6710e-05_rb,2.7409e-05_rb,2.7919e-05_rb,2.8397e-05_rb,2.8592e-05_rb/)
      kbo(:,37, 1) = (/ &
     &2.2468e-05_rb,2.3018e-05_rb,2.3378e-05_rb,2.3912e-05_rb,2.4011e-05_rb/)
      kbo(:,38, 1) = (/ &
     &1.8961e-05_rb,1.9311e-05_rb,1.9590e-05_rb,2.0060e-05_rb,2.0183e-05_rb/)
      kbo(:,39, 1) = (/ &
     &1.6038e-05_rb,1.6240e-05_rb,1.6497e-05_rb,1.6653e-05_rb,1.7067e-05_rb/)
      kbo(:,40, 1) = (/ &
     &1.3126e-05_rb,1.3247e-05_rb,1.3457e-05_rb,1.3586e-05_rb,1.3950e-05_rb/)
      kbo(:,41, 1) = (/ &
     &1.0642e-05_rb,1.0779e-05_rb,1.0904e-05_rb,1.1060e-05_rb,1.1348e-05_rb/)
      kbo(:,42, 1) = (/ &
     &8.5931e-06_rb,8.7256e-06_rb,8.8249e-06_rb,8.9763e-06_rb,9.1997e-06_rb/)
      kbo(:,43, 1) = (/ &
     &6.8315e-06_rb,7.0003e-06_rb,7.0825e-06_rb,7.1921e-06_rb,7.3080e-06_rb/)
      kbo(:,44, 1) = (/ &
     &5.4113e-06_rb,5.5622e-06_rb,5.6277e-06_rb,5.7357e-06_rb,5.8407e-06_rb/)
      kbo(:,45, 1) = (/ &
     &4.2868e-06_rb,4.4145e-06_rb,4.4700e-06_rb,4.5509e-06_rb,4.6597e-06_rb/)
      kbo(:,46, 1) = (/ &
     &3.3813e-06_rb,3.5001e-06_rb,3.5350e-06_rb,3.6083e-06_rb,3.6919e-06_rb/)
      kbo(:,47, 1) = (/ &
     &2.6664e-06_rb,2.7723e-06_rb,2.7972e-06_rb,2.8533e-06_rb,2.9267e-06_rb/)
      kbo(:,48, 1) = (/ &
     &2.0946e-06_rb,2.1711e-06_rb,2.2048e-06_rb,2.2465e-06_rb,2.3102e-06_rb/)
      kbo(:,49, 1) = (/ &
     &1.6333e-06_rb,1.6996e-06_rb,1.7388e-06_rb,1.7611e-06_rb,1.8115e-06_rb/)
      kbo(:,50, 1) = (/ &
     &1.2781e-06_rb,1.3328e-06_rb,1.3698e-06_rb,1.3938e-06_rb,1.4265e-06_rb/)
      kbo(:,51, 1) = (/ &
     &1.0076e-06_rb,1.0460e-06_rb,1.0807e-06_rb,1.1011e-06_rb,1.1257e-06_rb/)
      kbo(:,52, 1) = (/ &
     &7.9394e-07_rb,8.1840e-07_rb,8.4881e-07_rb,8.6654e-07_rb,8.8734e-07_rb/)
      kbo(:,53, 1) = (/ &
     &6.1785e-07_rb,6.3992e-07_rb,6.6144e-07_rb,6.7843e-07_rb,6.9618e-07_rb/)
      kbo(:,54, 1) = (/ &
     &4.8490e-07_rb,5.0506e-07_rb,5.1952e-07_rb,5.3413e-07_rb,5.4905e-07_rb/)
      kbo(:,55, 1) = (/ &
     &3.8198e-07_rb,3.9926e-07_rb,4.1207e-07_rb,4.2268e-07_rb,4.3489e-07_rb/)
      kbo(:,56, 1) = (/ &
     &3.0062e-07_rb,3.1357e-07_rb,3.2461e-07_rb,3.3313e-07_rb,3.4390e-07_rb/)
      kbo(:,57, 1) = (/ &
     &2.3579e-07_rb,2.4608e-07_rb,2.5500e-07_rb,2.6288e-07_rb,2.7145e-07_rb/)
      kbo(:,58, 1) = (/ &
     &1.8530e-07_rb,1.9293e-07_rb,2.0021e-07_rb,2.0804e-07_rb,2.1510e-07_rb/)
      kbo(:,59, 1) = (/ &
     &1.4951e-07_rb,1.5512e-07_rb,1.6259e-07_rb,1.6844e-07_rb,1.7451e-07_rb/)
      kbo(:,13, 2) = (/ &
     &2.8194e-03_rb,3.0121e-03_rb,3.1846e-03_rb,3.3490e-03_rb,3.5100e-03_rb/)
      kbo(:,14, 2) = (/ &
     &2.4827e-03_rb,2.6492e-03_rb,2.7938e-03_rb,2.9346e-03_rb,3.0538e-03_rb/)
      kbo(:,15, 2) = (/ &
     &2.2064e-03_rb,2.3375e-03_rb,2.4556e-03_rb,2.5697e-03_rb,2.6626e-03_rb/)
      kbo(:,16, 2) = (/ &
     &1.9378e-03_rb,2.0444e-03_rb,2.1458e-03_rb,2.2321e-03_rb,2.3069e-03_rb/)
      kbo(:,17, 2) = (/ &
     &1.7072e-03_rb,1.7953e-03_rb,1.8810e-03_rb,1.9513e-03_rb,2.0122e-03_rb/)
      kbo(:,18, 2) = (/ &
     &1.5182e-03_rb,1.5881e-03_rb,1.6604e-03_rb,1.7175e-03_rb,1.7691e-03_rb/)
      kbo(:,19, 2) = (/ &
     &1.3604e-03_rb,1.4237e-03_rb,1.4711e-03_rb,1.5216e-03_rb,1.5631e-03_rb/)
      kbo(:,20, 2) = (/ &
     &1.1858e-03_rb,1.2290e-03_rb,1.2712e-03_rb,1.3090e-03_rb,1.3421e-03_rb/)
      kbo(:,21, 2) = (/ &
     &1.0189e-03_rb,1.0547e-03_rb,1.0885e-03_rb,1.1186e-03_rb,1.1432e-03_rb/)
      kbo(:,22, 2) = (/ &
     &8.7055e-04_rb,9.0481e-04_rb,9.2821e-04_rb,9.5252e-04_rb,9.7075e-04_rb/)
      kbo(:,23, 2) = (/ &
     &7.4675e-04_rb,7.7099e-04_rb,7.9133e-04_rb,8.0933e-04_rb,8.2551e-04_rb/)
      kbo(:,24, 2) = (/ &
     &6.4034e-04_rb,6.5905e-04_rb,6.7517e-04_rb,6.8951e-04_rb,7.0349e-04_rb/)
      kbo(:,25, 2) = (/ &
     &5.4635e-04_rb,5.6117e-04_rb,5.7460e-04_rb,5.8668e-04_rb,5.9880e-04_rb/)
      kbo(:,26, 2) = (/ &
     &4.6328e-04_rb,4.7565e-04_rb,4.8724e-04_rb,4.9802e-04_rb,5.0785e-04_rb/)
      kbo(:,27, 2) = (/ &
     &3.9145e-04_rb,4.0138e-04_rb,4.1206e-04_rb,4.2067e-04_rb,4.2970e-04_rb/)
      kbo(:,28, 2) = (/ &
     &3.2965e-04_rb,3.3905e-04_rb,3.4755e-04_rb,3.5536e-04_rb,3.6345e-04_rb/)
      kbo(:,29, 2) = (/ &
     &2.7722e-04_rb,2.8535e-04_rb,2.9237e-04_rb,2.9947e-04_rb,3.0668e-04_rb/)
      kbo(:,30, 2) = (/ &
     &2.3378e-04_rb,2.4040e-04_rb,2.4667e-04_rb,2.5278e-04_rb,2.6270e-04_rb/)
      kbo(:,31, 2) = (/ &
     &1.9661e-04_rb,2.0207e-04_rb,2.0803e-04_rb,2.1348e-04_rb,2.2186e-04_rb/)
      kbo(:,32, 2) = (/ &
     &1.6562e-04_rb,1.7146e-04_rb,1.7695e-04_rb,1.8193e-04_rb,1.8929e-04_rb/)
      kbo(:,33, 2) = (/ &
     &1.4107e-04_rb,1.4666e-04_rb,1.5183e-04_rb,1.5594e-04_rb,1.6243e-04_rb/)
      kbo(:,34, 2) = (/ &
     &1.2158e-04_rb,1.2598e-04_rb,1.3011e-04_rb,1.3588e-04_rb,1.4023e-04_rb/)
      kbo(:,35, 2) = (/ &
     &1.0453e-04_rb,1.0824e-04_rb,1.1246e-04_rb,1.1806e-04_rb,1.2280e-04_rb/)
      kbo(:,36, 2) = (/ &
     &9.0281e-05_rb,9.3775e-05_rb,9.7901e-05_rb,1.0364e-04_rb,1.0744e-04_rb/)
      kbo(:,37, 2) = (/ &
     &7.5786e-05_rb,7.8742e-05_rb,8.2503e-05_rb,8.7040e-05_rb,8.9489e-05_rb/)
      kbo(:,38, 2) = (/ &
     &6.3149e-05_rb,6.6113e-05_rb,6.8833e-05_rb,7.2371e-05_rb,7.4496e-05_rb/)
      kbo(:,39, 2) = (/ &
     &5.2982e-05_rb,5.5381e-05_rb,5.7239e-05_rb,5.9241e-05_rb,6.1954e-05_rb/)
      kbo(:,40, 2) = (/ &
     &4.3111e-05_rb,4.5156e-05_rb,4.6617e-05_rb,4.8324e-05_rb,5.0544e-05_rb/)
      kbo(:,41, 2) = (/ &
     &3.4957e-05_rb,3.6661e-05_rb,3.7913e-05_rb,3.9243e-05_rb,4.1170e-05_rb/)
      kbo(:,42, 2) = (/ &
     &2.8335e-05_rb,2.9735e-05_rb,3.0788e-05_rb,3.1864e-05_rb,3.3563e-05_rb/)
      kbo(:,43, 2) = (/ &
     &2.2801e-05_rb,2.3962e-05_rb,2.4808e-05_rb,2.5685e-05_rb,2.6685e-05_rb/)
      kbo(:,44, 2) = (/ &
     &1.8216e-05_rb,1.9207e-05_rb,1.9940e-05_rb,2.0623e-05_rb,2.1468e-05_rb/)
      kbo(:,45, 2) = (/ &
     &1.4497e-05_rb,1.5369e-05_rb,1.6010e-05_rb,1.6560e-05_rb,1.7214e-05_rb/)
      kbo(:,46, 2) = (/ &
     &1.1546e-05_rb,1.2283e-05_rb,1.2829e-05_rb,1.3281e-05_rb,1.3789e-05_rb/)
      kbo(:,47, 2) = (/ &
     &9.1407e-06_rb,9.7102e-06_rb,1.0256e-05_rb,1.0642e-05_rb,1.1044e-05_rb/)
      kbo(:,48, 2) = (/ &
     &7.2582e-06_rb,7.7178e-06_rb,8.1771e-06_rb,8.5159e-06_rb,8.8287e-06_rb/)
      kbo(:,49, 2) = (/ &
     &5.7413e-06_rb,6.0895e-06_rb,6.4724e-06_rb,6.7948e-06_rb,7.0423e-06_rb/)
      kbo(:,50, 2) = (/ &
     &4.5645e-06_rb,4.8219e-06_rb,5.1398e-06_rb,5.4019e-06_rb,5.6251e-06_rb/)
      kbo(:,51, 2) = (/ &
     &3.6347e-06_rb,3.8345e-06_rb,4.0925e-06_rb,4.2999e-06_rb,4.4976e-06_rb/)
      kbo(:,52, 2) = (/ &
     &2.8914e-06_rb,3.0394e-06_rb,3.2506e-06_rb,3.4210e-06_rb,3.5835e-06_rb/)
      kbo(:,53, 2) = (/ &
     &2.2827e-06_rb,2.4183e-06_rb,2.5753e-06_rb,2.7165e-06_rb,2.8435e-06_rb/)
      kbo(:,54, 2) = (/ &
     &1.8109e-06_rb,1.9319e-06_rb,2.0517e-06_rb,2.1771e-06_rb,2.2825e-06_rb/)
      kbo(:,55, 2) = (/ &
     &1.4459e-06_rb,1.5442e-06_rb,1.6423e-06_rb,1.7466e-06_rb,1.8412e-06_rb/)
      kbo(:,56, 2) = (/ &
     &1.1487e-06_rb,1.2318e-06_rb,1.3168e-06_rb,1.4029e-06_rb,1.4853e-06_rb/)
      kbo(:,57, 2) = (/ &
     &9.1432e-07_rb,9.8206e-07_rb,1.0551e-06_rb,1.1262e-06_rb,1.1922e-06_rb/)
      kbo(:,58, 2) = (/ &
     &7.2392e-07_rb,7.8435e-07_rb,8.4467e-07_rb,9.0516e-07_rb,9.5596e-07_rb/)
      kbo(:,59, 2) = (/ &
     &5.9300e-07_rb,6.4245e-07_rb,6.9378e-07_rb,7.4259e-07_rb,7.8676e-07_rb/)
      kbo(:,13, 3) = (/ &
     &8.5247e-03_rb,8.9273e-03_rb,9.3494e-03_rb,9.7833e-03_rb,1.0198e-02_rb/)
      kbo(:,14, 3) = (/ &
     &7.6220e-03_rb,7.9465e-03_rb,8.3037e-03_rb,8.6680e-03_rb,9.0184e-03_rb/)
      kbo(:,15, 3) = (/ &
     &6.7313e-03_rb,7.0250e-03_rb,7.3390e-03_rb,7.6496e-03_rb,7.9523e-03_rb/)
      kbo(:,16, 3) = (/ &
     &5.8626e-03_rb,6.1324e-03_rb,6.4081e-03_rb,6.6680e-03_rb,6.9239e-03_rb/)
      kbo(:,17, 3) = (/ &
     &5.1190e-03_rb,5.3847e-03_rb,5.6114e-03_rb,5.8394e-03_rb,6.0319e-03_rb/)
      kbo(:,18, 3) = (/ &
     &4.4937e-03_rb,4.7329e-03_rb,4.9304e-03_rb,5.1178e-03_rb,5.2582e-03_rb/)
      kbo(:,19, 3) = (/ &
     &3.9714e-03_rb,4.1523e-03_rb,4.3393e-03_rb,4.4847e-03_rb,4.6081e-03_rb/)
      kbo(:,20, 3) = (/ &
     &3.4227e-03_rb,3.5888e-03_rb,3.7323e-03_rb,3.8457e-03_rb,3.9571e-03_rb/)
      kbo(:,21, 3) = (/ &
     &2.9355e-03_rb,3.0688e-03_rb,3.1813e-03_rb,3.2800e-03_rb,3.3665e-03_rb/)
      kbo(:,22, 3) = (/ &
     &2.5131e-03_rb,2.6136e-03_rb,2.7075e-03_rb,2.7888e-03_rb,2.8576e-03_rb/)
      kbo(:,23, 3) = (/ &
     &2.1439e-03_rb,2.2229e-03_rb,2.3069e-03_rb,2.3691e-03_rb,2.4314e-03_rb/)
      kbo(:,24, 3) = (/ &
     &1.8279e-03_rb,1.8995e-03_rb,1.9649e-03_rb,2.0190e-03_rb,2.0668e-03_rb/)
      kbo(:,25, 3) = (/ &
     &1.5582e-03_rb,1.6177e-03_rb,1.6674e-03_rb,1.7146e-03_rb,1.7550e-03_rb/)
      kbo(:,26, 3) = (/ &
     &1.3272e-03_rb,1.3717e-03_rb,1.4140e-03_rb,1.4536e-03_rb,1.4858e-03_rb/)
      kbo(:,27, 3) = (/ &
     &1.1230e-03_rb,1.1594e-03_rb,1.1953e-03_rb,1.2264e-03_rb,1.2540e-03_rb/)
      kbo(:,28, 3) = (/ &
     &9.4753e-04_rb,9.7763e-04_rb,1.0074e-03_rb,1.0308e-03_rb,1.0536e-03_rb/)
      kbo(:,29, 3) = (/ &
     &7.9502e-04_rb,8.1974e-04_rb,8.4372e-04_rb,8.6478e-04_rb,8.8308e-04_rb/)
      kbo(:,30, 3) = (/ &
     &6.6596e-04_rb,6.8697e-04_rb,7.0803e-04_rb,7.2439e-04_rb,7.5784e-04_rb/)
      kbo(:,31, 3) = (/ &
     &5.5745e-04_rb,5.7516e-04_rb,5.9220e-04_rb,6.0512e-04_rb,6.3289e-04_rb/)
      kbo(:,32, 3) = (/ &
     &4.6662e-04_rb,4.8150e-04_rb,4.9505e-04_rb,5.0361e-04_rb,5.2756e-04_rb/)
      kbo(:,33, 3) = (/ &
     &3.9094e-04_rb,4.0255e-04_rb,4.1211e-04_rb,4.1938e-04_rb,4.4050e-04_rb/)
      kbo(:,34, 3) = (/ &
     &3.2769e-04_rb,3.3699e-04_rb,3.4471e-04_rb,3.6275e-04_rb,3.6949e-04_rb/)
      kbo(:,35, 3) = (/ &
     &2.7475e-04_rb,2.8204e-04_rb,2.8828e-04_rb,3.0332e-04_rb,3.0836e-04_rb/)
      kbo(:,36, 3) = (/ &
     &2.2983e-04_rb,2.3568e-04_rb,2.4113e-04_rb,2.5319e-04_rb,2.5736e-04_rb/)
      kbo(:,37, 3) = (/ &
     &1.8895e-04_rb,1.9397e-04_rb,1.9807e-04_rb,2.0882e-04_rb,2.1275e-04_rb/)
      kbo(:,38, 3) = (/ &
     &1.5549e-04_rb,1.5951e-04_rb,1.6323e-04_rb,1.7252e-04_rb,1.7561e-04_rb/)
      kbo(:,39, 3) = (/ &
     &1.2765e-04_rb,1.3102e-04_rb,1.3443e-04_rb,1.3776e-04_rb,1.4478e-04_rb/)
      kbo(:,40, 3) = (/ &
     &1.0414e-04_rb,1.0697e-04_rb,1.0972e-04_rb,1.1251e-04_rb,1.1839e-04_rb/)
      kbo(:,41, 3) = (/ &
     &8.4861e-05_rb,8.7037e-05_rb,8.9345e-05_rb,9.1552e-05_rb,9.6450e-05_rb/)
      kbo(:,42, 3) = (/ &
     &6.9029e-05_rb,7.0620e-05_rb,7.2608e-05_rb,7.4362e-05_rb,7.8451e-05_rb/)
      kbo(:,43, 3) = (/ &
     &5.5731e-05_rb,5.6934e-05_rb,5.8627e-05_rb,6.0036e-05_rb,6.1419e-05_rb/)
      kbo(:,44, 3) = (/ &
     &4.4750e-05_rb,4.5795e-05_rb,4.7066e-05_rb,4.8320e-05_rb,4.9340e-05_rb/)
      kbo(:,45, 3) = (/ &
     &3.5902e-05_rb,3.6793e-05_rb,3.7710e-05_rb,3.8772e-05_rb,3.9634e-05_rb/)
      kbo(:,46, 3) = (/ &
     &2.8669e-05_rb,2.9406e-05_rb,3.0158e-05_rb,3.1055e-05_rb,3.1783e-05_rb/)
      kbo(:,47, 3) = (/ &
     &2.2957e-05_rb,2.3586e-05_rb,2.4179e-05_rb,2.4875e-05_rb,2.5463e-05_rb/)
      kbo(:,48, 3) = (/ &
     &1.8322e-05_rb,1.8764e-05_rb,1.9281e-05_rb,1.9846e-05_rb,2.0330e-05_rb/)
      kbo(:,49, 3) = (/ &
     &1.4581e-05_rb,1.4948e-05_rb,1.5326e-05_rb,1.5796e-05_rb,1.6188e-05_rb/)
      kbo(:,50, 3) = (/ &
     &1.1658e-05_rb,1.1996e-05_rb,1.2253e-05_rb,1.2626e-05_rb,1.2988e-05_rb/)
      kbo(:,51, 3) = (/ &
     &9.3098e-06_rb,9.6196e-06_rb,9.8320e-06_rb,1.0134e-05_rb,1.0431e-05_rb/)
      kbo(:,52, 3) = (/ &
     &7.3883e-06_rb,7.7108e-06_rb,7.8752e-06_rb,8.1035e-06_rb,8.3459e-06_rb/)
      kbo(:,53, 3) = (/ &
     &5.8592e-06_rb,6.1249e-06_rb,6.2920e-06_rb,6.4688e-06_rb,6.6480e-06_rb/)
      kbo(:,54, 3) = (/ &
     &4.7059e-06_rb,4.9088e-06_rb,5.0922e-06_rb,5.2219e-06_rb,5.3713e-06_rb/)
      kbo(:,55, 3) = (/ &
     &3.7899e-06_rb,3.9660e-06_rb,4.1365e-06_rb,4.2472e-06_rb,4.3644e-06_rb/)
      kbo(:,56, 3) = (/ &
     &3.0554e-06_rb,3.2100e-06_rb,3.3483e-06_rb,3.4494e-06_rb,3.5457e-06_rb/)
      kbo(:,57, 3) = (/ &
     &2.4489e-06_rb,2.5923e-06_rb,2.7014e-06_rb,2.7985e-06_rb,2.8823e-06_rb/)
      kbo(:,58, 3) = (/ &
     &1.9694e-06_rb,2.0948e-06_rb,2.1872e-06_rb,2.2740e-06_rb,2.3512e-06_rb/)
      kbo(:,59, 3) = (/ &
     &1.6303e-06_rb,1.7449e-06_rb,1.8268e-06_rb,1.9070e-06_rb,1.9708e-06_rb/)
      kbo(:,13, 4) = (/ &
     &1.8047e-02_rb,1.8718e-02_rb,1.9333e-02_rb,1.9945e-02_rb,2.0619e-02_rb/)
      kbo(:,14, 4) = (/ &
     &1.6432e-02_rb,1.7100e-02_rb,1.7741e-02_rb,1.8401e-02_rb,1.9089e-02_rb/)
      kbo(:,15, 4) = (/ &
     &1.4807e-02_rb,1.5451e-02_rb,1.6100e-02_rb,1.6750e-02_rb,1.7413e-02_rb/)
      kbo(:,16, 4) = (/ &
     &1.3139e-02_rb,1.3704e-02_rb,1.4262e-02_rb,1.4904e-02_rb,1.5552e-02_rb/)
      kbo(:,17, 4) = (/ &
     &1.1663e-02_rb,1.2130e-02_rb,1.2669e-02_rb,1.3218e-02_rb,1.3830e-02_rb/)
      kbo(:,18, 4) = (/ &
     &1.0329e-02_rb,1.0751e-02_rb,1.1271e-02_rb,1.1798e-02_rb,1.2374e-02_rb/)
      kbo(:,19, 4) = (/ &
     &9.2112e-03_rb,9.6353e-03_rb,1.0130e-02_rb,1.0649e-02_rb,1.1157e-02_rb/)
      kbo(:,20, 4) = (/ &
     &8.0574e-03_rb,8.4807e-03_rb,8.9054e-03_rb,9.3587e-03_rb,9.7992e-03_rb/)
      kbo(:,21, 4) = (/ &
     &7.0019e-03_rb,7.3853e-03_rb,7.7827e-03_rb,8.1758e-03_rb,8.5421e-03_rb/)
      kbo(:,22, 4) = (/ &
     &6.0893e-03_rb,6.4266e-03_rb,6.7752e-03_rb,7.1040e-03_rb,7.4095e-03_rb/)
      kbo(:,23, 4) = (/ &
     &5.2966e-03_rb,5.5992e-03_rb,5.8988e-03_rb,6.1748e-03_rb,6.4166e-03_rb/)
      kbo(:,24, 4) = (/ &
     &4.6174e-03_rb,4.8835e-03_rb,5.1353e-03_rb,5.3661e-03_rb,5.5632e-03_rb/)
      kbo(:,25, 4) = (/ &
     &4.0173e-03_rb,4.2433e-03_rb,4.4670e-03_rb,4.6521e-03_rb,4.8258e-03_rb/)
      kbo(:,26, 4) = (/ &
     &3.4865e-03_rb,3.6826e-03_rb,3.8670e-03_rb,4.0200e-03_rb,4.1614e-03_rb/)
      kbo(:,27, 4) = (/ &
     &3.0102e-03_rb,3.1785e-03_rb,3.3303e-03_rb,3.4548e-03_rb,3.5736e-03_rb/)
      kbo(:,28, 4) = (/ &
     &2.5902e-03_rb,2.7336e-03_rb,2.8553e-03_rb,2.9588e-03_rb,3.0603e-03_rb/)
      kbo(:,29, 4) = (/ &
     &2.2148e-03_rb,2.3314e-03_rb,2.4216e-03_rb,2.5098e-03_rb,2.5968e-03_rb/)
      kbo(:,30, 4) = (/ &
     &1.8884e-03_rb,1.9733e-03_rb,2.0479e-03_rb,2.1223e-03_rb,2.2826e-03_rb/)
      kbo(:,31, 4) = (/ &
     &1.5995e-03_rb,1.6617e-03_rb,1.7230e-03_rb,1.7883e-03_rb,1.9307e-03_rb/)
      kbo(:,32, 4) = (/ &
     &1.3496e-03_rb,1.3999e-03_rb,1.4538e-03_rb,1.5096e-03_rb,1.6319e-03_rb/)
      kbo(:,33, 4) = (/ &
     &1.1363e-03_rb,1.1813e-03_rb,1.2287e-03_rb,1.2753e-03_rb,1.3874e-03_rb/)
      kbo(:,34, 4) = (/ &
     &9.6663e-04_rb,1.0050e-03_rb,1.0440e-03_rb,1.1401e-03_rb,1.1914e-03_rb/)
      kbo(:,35, 4) = (/ &
     &8.2252e-04_rb,8.5291e-04_rb,8.9255e-04_rb,9.7906e-04_rb,1.0213e-03_rb/)
      kbo(:,36, 4) = (/ &
     &6.9843e-04_rb,7.2671e-04_rb,7.6052e-04_rb,8.3992e-04_rb,8.7893e-04_rb/)
      kbo(:,37, 4) = (/ &
     &5.7973e-04_rb,6.0624e-04_rb,6.3703e-04_rb,7.0532e-04_rb,7.4046e-04_rb/)
      kbo(:,38, 4) = (/ &
     &4.8124e-04_rb,5.0463e-04_rb,5.3059e-04_rb,5.8984e-04_rb,6.2161e-04_rb/)
      kbo(:,39, 4) = (/ &
     &3.9797e-04_rb,4.2007e-04_rb,4.4141e-04_rb,4.6508e-04_rb,5.2126e-04_rb/)
      kbo(:,40, 4) = (/ &
     &3.2587e-04_rb,3.4475e-04_rb,3.6264e-04_rb,3.8270e-04_rb,4.3106e-04_rb/)
      kbo(:,41, 4) = (/ &
     &2.6592e-04_rb,2.8134e-04_rb,2.9669e-04_rb,3.1406e-04_rb,3.5481e-04_rb/)
      kbo(:,42, 4) = (/ &
     &2.1654e-04_rb,2.2933e-04_rb,2.4233e-04_rb,2.5702e-04_rb,2.9186e-04_rb/)
      kbo(:,43, 4) = (/ &
     &1.7468e-04_rb,1.8487e-04_rb,1.9562e-04_rb,2.0857e-04_rb,2.2203e-04_rb/)
      kbo(:,44, 4) = (/ &
     &1.3990e-04_rb,1.4813e-04_rb,1.5725e-04_rb,1.6786e-04_rb,1.7935e-04_rb/)
      kbo(:,45, 4) = (/ &
     &1.1147e-04_rb,1.1816e-04_rb,1.2590e-04_rb,1.3456e-04_rb,1.4433e-04_rb/)
      kbo(:,46, 4) = (/ &
     &8.8593e-05_rb,9.4029e-05_rb,1.0037e-04_rb,1.0744e-04_rb,1.1598e-04_rb/)
      kbo(:,47, 4) = (/ &
     &7.0072e-05_rb,7.4712e-05_rb,7.9919e-05_rb,8.5900e-05_rb,9.3033e-05_rb/)
      kbo(:,48, 4) = (/ &
     &5.5180e-05_rb,5.9132e-05_rb,6.3502e-05_rb,6.8198e-05_rb,7.4266e-05_rb/)
      kbo(:,49, 4) = (/ &
     &4.3109e-05_rb,4.6388e-05_rb,5.0075e-05_rb,5.3912e-05_rb,5.8896e-05_rb/)
      kbo(:,50, 4) = (/ &
     &3.4049e-05_rb,3.6756e-05_rb,3.9830e-05_rb,4.3128e-05_rb,4.7184e-05_rb/)
      kbo(:,51, 4) = (/ &
     &2.7005e-05_rb,2.9148e-05_rb,3.1674e-05_rb,3.4539e-05_rb,3.7748e-05_rb/)
      kbo(:,52, 4) = (/ &
     &2.1299e-05_rb,2.3058e-05_rb,2.5114e-05_rb,2.7488e-05_rb,3.0113e-05_rb/)
      kbo(:,53, 4) = (/ &
     &1.6705e-05_rb,1.8140e-05_rb,1.9875e-05_rb,2.1784e-05_rb,2.3982e-05_rb/)
      kbo(:,54, 4) = (/ &
     &1.3350e-05_rb,1.4560e-05_rb,1.5930e-05_rb,1.7502e-05_rb,1.9341e-05_rb/)
      kbo(:,55, 4) = (/ &
     &1.0773e-05_rb,1.1749e-05_rb,1.2886e-05_rb,1.4153e-05_rb,1.5702e-05_rb/)
      kbo(:,56, 4) = (/ &
     &8.6460e-06_rb,9.4510e-06_rb,1.0398e-05_rb,1.1452e-05_rb,1.2706e-05_rb/)
      kbo(:,57, 4) = (/ &
     &6.9096e-06_rb,7.6015e-06_rb,8.3864e-06_rb,9.2473e-06_rb,1.0265e-05_rb/)
      kbo(:,58, 4) = (/ &
     &5.5254e-06_rb,6.1387e-06_rb,6.7789e-06_rb,7.4818e-06_rb,8.3075e-06_rb/)
      kbo(:,59, 4) = (/ &
     &4.6551e-06_rb,5.1865e-06_rb,5.7401e-06_rb,6.3194e-06_rb,7.0521e-06_rb/)
      kbo(:,13, 5) = (/ &
     &3.4718e-02_rb,3.5594e-02_rb,3.6803e-02_rb,3.8083e-02_rb,3.9520e-02_rb/)
      kbo(:,14, 5) = (/ &
     &3.2238e-02_rb,3.3220e-02_rb,3.4363e-02_rb,3.5532e-02_rb,3.6904e-02_rb/)
      kbo(:,15, 5) = (/ &
     &3.0600e-02_rb,3.1610e-02_rb,3.2771e-02_rb,3.3966e-02_rb,3.5285e-02_rb/)
      kbo(:,16, 5) = (/ &
     &2.8159e-02_rb,2.9168e-02_rb,3.0264e-02_rb,3.1396e-02_rb,3.2647e-02_rb/)
      kbo(:,17, 5) = (/ &
     &2.5788e-02_rb,2.6724e-02_rb,2.7768e-02_rb,2.8866e-02_rb,3.0013e-02_rb/)
      kbo(:,18, 5) = (/ &
     &2.3487e-02_rb,2.4520e-02_rb,2.5499e-02_rb,2.6557e-02_rb,2.7740e-02_rb/)
      kbo(:,19, 5) = (/ &
     &2.1508e-02_rb,2.2551e-02_rb,2.3562e-02_rb,2.4611e-02_rb,2.5770e-02_rb/)
      kbo(:,20, 5) = (/ &
     &1.9153e-02_rb,2.0099e-02_rb,2.1123e-02_rb,2.2234e-02_rb,2.3370e-02_rb/)
      kbo(:,21, 5) = (/ &
     &1.6910e-02_rb,1.7820e-02_rb,1.8821e-02_rb,1.9939e-02_rb,2.1019e-02_rb/)
      kbo(:,22, 5) = (/ &
     &1.4892e-02_rb,1.5824e-02_rb,1.6836e-02_rb,1.7897e-02_rb,1.8973e-02_rb/)
      kbo(:,23, 5) = (/ &
     &1.3201e-02_rb,1.4110e-02_rb,1.5066e-02_rb,1.6122e-02_rb,1.7163e-02_rb/)
      kbo(:,24, 5) = (/ &
     &1.1786e-02_rb,1.2643e-02_rb,1.3592e-02_rb,1.4574e-02_rb,1.5604e-02_rb/)
      kbo(:,25, 5) = (/ &
     &1.0551e-02_rb,1.1376e-02_rb,1.2278e-02_rb,1.3228e-02_rb,1.4135e-02_rb/)
      kbo(:,26, 5) = (/ &
     &9.4341e-03_rb,1.0259e-02_rb,1.1075e-02_rb,1.1949e-02_rb,1.2760e-02_rb/)
      kbo(:,27, 5) = (/ &
     &8.4286e-03_rb,9.1773e-03_rb,9.9233e-03_rb,1.0676e-02_rb,1.1418e-02_rb/)
      kbo(:,28, 5) = (/ &
     &7.5098e-03_rb,8.1852e-03_rb,8.8662e-03_rb,9.5190e-03_rb,1.0161e-02_rb/)
      kbo(:,29, 5) = (/ &
     &6.6339e-03_rb,7.2146e-03_rb,7.8111e-03_rb,8.3875e-03_rb,8.9339e-03_rb/)
      kbo(:,30, 5) = (/ &
     &5.8318e-03_rb,6.3504e-03_rb,6.8644e-03_rb,7.3656e-03_rb,8.2854e-03_rb/)
      kbo(:,31, 5) = (/ &
     &5.0938e-03_rb,5.5450e-03_rb,5.9883e-03_rb,6.4048e-03_rb,7.2251e-03_rb/)
      kbo(:,32, 5) = (/ &
     &4.4534e-03_rb,4.8436e-03_rb,5.2343e-03_rb,5.5836e-03_rb,6.3649e-03_rb/)
      kbo(:,33, 5) = (/ &
     &3.8952e-03_rb,4.2551e-03_rb,4.5701e-03_rb,4.9101e-03_rb,5.6667e-03_rb/)
      kbo(:,34, 5) = (/ &
     &3.4434e-03_rb,3.7546e-03_rb,4.0585e-03_rb,4.7206e-03_rb,5.1136e-03_rb/)
      kbo(:,35, 5) = (/ &
     &3.0323e-03_rb,3.3194e-03_rb,3.6005e-03_rb,4.2419e-03_rb,4.6276e-03_rb/)
      kbo(:,36, 5) = (/ &
     &2.6713e-03_rb,2.9283e-03_rb,3.2072e-03_rb,3.8119e-03_rb,4.2091e-03_rb/)
      kbo(:,37, 5) = (/ &
     &2.2748e-03_rb,2.5138e-03_rb,2.7697e-03_rb,3.3361e-03_rb,3.7192e-03_rb/)
      kbo(:,38, 5) = (/ &
     &1.9294e-03_rb,2.1456e-03_rb,2.3858e-03_rb,2.9205e-03_rb,3.2883e-03_rb/)
      kbo(:,39, 5) = (/ &
     &1.6394e-03_rb,1.8308e-03_rb,2.0616e-03_rb,2.3311e-03_rb,2.9179e-03_rb/)
      kbo(:,40, 5) = (/ &
     &1.3611e-03_rb,1.5347e-03_rb,1.7447e-03_rb,1.9909e-03_rb,2.5336e-03_rb/)
      kbo(:,41, 5) = (/ &
     &1.1258e-03_rb,1.2811e-03_rb,1.4710e-03_rb,1.6948e-03_rb,2.1957e-03_rb/)
      kbo(:,42, 5) = (/ &
     &9.3054e-04_rb,1.0685e-03_rb,1.2380e-03_rb,1.4417e-03_rb,1.8992e-03_rb/)
      kbo(:,43, 5) = (/ &
     &7.5595e-04_rb,8.7758e-04_rb,1.0267e-03_rb,1.2093e-03_rb,1.4383e-03_rb/)
      kbo(:,44, 5) = (/ &
     &6.0708e-04_rb,7.1329e-04_rb,8.4285e-04_rb,1.0050e-03_rb,1.2098e-03_rb/)
      kbo(:,45, 5) = (/ &
     &4.8565e-04_rb,5.7704e-04_rb,6.8980e-04_rb,8.3254e-04_rb,1.0112e-03_rb/)
      kbo(:,46, 5) = (/ &
     &3.8568e-04_rb,4.6292e-04_rb,5.6071e-04_rb,6.8523e-04_rb,8.4055e-04_rb/)
      kbo(:,47, 5) = (/ &
     &3.0303e-04_rb,3.6775e-04_rb,4.5081e-04_rb,5.5621e-04_rb,6.9040e-04_rb/)
      kbo(:,48, 5) = (/ &
     &2.3629e-04_rb,2.9047e-04_rb,3.6009e-04_rb,4.4971e-04_rb,5.6449e-04_rb/)
      kbo(:,49, 5) = (/ &
     &1.8277e-04_rb,2.2768e-04_rb,2.8500e-04_rb,3.5988e-04_rb,4.5780e-04_rb/)
      kbo(:,50, 5) = (/ &
     &1.4236e-04_rb,1.7948e-04_rb,2.2683e-04_rb,2.8959e-04_rb,3.7278e-04_rb/)
      kbo(:,51, 5) = (/ &
     &1.1105e-04_rb,1.4188e-04_rb,1.8087e-04_rb,2.3323e-04_rb,3.0421e-04_rb/)
      kbo(:,52, 5) = (/ &
     &8.5949e-05_rb,1.1090e-04_rb,1.4310e-04_rb,1.8669e-04_rb,2.4668e-04_rb/)
      kbo(:,53, 5) = (/ &
     &6.6010e-05_rb,8.6189e-05_rb,1.1254e-04_rb,1.4823e-04_rb,1.9832e-04_rb/)
      kbo(:,54, 5) = (/ &
     &5.1508e-05_rb,6.7802e-05_rb,8.9425e-05_rb,1.1908e-04_rb,1.6117e-04_rb/)
      kbo(:,55, 5) = (/ &
     &4.0450e-05_rb,5.3662e-05_rb,7.1436e-05_rb,9.6039e-05_rb,1.3135e-04_rb/)
      kbo(:,56, 5) = (/ &
     &3.1739e-05_rb,4.2180e-05_rb,5.6735e-05_rb,7.6991e-05_rb,1.0631e-04_rb/)
      kbo(:,57, 5) = (/ &
     &2.4823e-05_rb,3.2939e-05_rb,4.4784e-05_rb,6.1322e-05_rb,8.5682e-05_rb/)
      kbo(:,58, 5) = (/ &
     &1.9514e-05_rb,2.5849e-05_rb,3.5397e-05_rb,4.8867e-05_rb,6.9076e-05_rb/)
      kbo(:,59, 5) = (/ &
     &1.6536e-05_rb,2.1873e-05_rb,3.0188e-05_rb,4.2152e-05_rb,6.0090e-05_rb/)
      kbo(:,13, 6) = (/ &
     &7.0936e-02_rb,7.2599e-02_rb,7.4412e-02_rb,7.6631e-02_rb,7.9197e-02_rb/)
      kbo(:,14, 6) = (/ &
     &6.5108e-02_rb,6.6883e-02_rb,6.8861e-02_rb,7.1194e-02_rb,7.3890e-02_rb/)
      kbo(:,15, 6) = (/ &
     &6.1844e-02_rb,6.3875e-02_rb,6.5914e-02_rb,6.8440e-02_rb,7.1135e-02_rb/)
      kbo(:,16, 6) = (/ &
     &5.8757e-02_rb,6.1061e-02_rb,6.3588e-02_rb,6.6243e-02_rb,6.9218e-02_rb/)
      kbo(:,17, 6) = (/ &
     &5.6075e-02_rb,5.8373e-02_rb,6.0818e-02_rb,6.3696e-02_rb,6.6956e-02_rb/)
      kbo(:,18, 6) = (/ &
     &5.3195e-02_rb,5.5446e-02_rb,5.8088e-02_rb,6.0906e-02_rb,6.3907e-02_rb/)
      kbo(:,19, 6) = (/ &
     &5.0738e-02_rb,5.2979e-02_rb,5.5433e-02_rb,5.8280e-02_rb,6.1280e-02_rb/)
      kbo(:,20, 6) = (/ &
     &4.6694e-02_rb,4.8984e-02_rb,5.1371e-02_rb,5.4034e-02_rb,5.7035e-02_rb/)
      kbo(:,21, 6) = (/ &
     &4.2499e-02_rb,4.4760e-02_rb,4.7087e-02_rb,4.9709e-02_rb,5.2664e-02_rb/)
      kbo(:,22, 6) = (/ &
     &3.8535e-02_rb,4.0776e-02_rb,4.3118e-02_rb,4.5805e-02_rb,4.8711e-02_rb/)
      kbo(:,23, 6) = (/ &
     &3.4990e-02_rb,3.7200e-02_rb,3.9611e-02_rb,4.2290e-02_rb,4.5266e-02_rb/)
      kbo(:,24, 6) = (/ &
     &3.1926e-02_rb,3.4161e-02_rb,3.6618e-02_rb,3.9305e-02_rb,4.2288e-02_rb/)
      kbo(:,25, 6) = (/ &
     &2.9165e-02_rb,3.1418e-02_rb,3.3868e-02_rb,3.6528e-02_rb,3.9610e-02_rb/)
      kbo(:,26, 6) = (/ &
     &2.6632e-02_rb,2.8869e-02_rb,3.1334e-02_rb,3.4138e-02_rb,3.7332e-02_rb/)
      kbo(:,27, 6) = (/ &
     &2.4229e-02_rb,2.6481e-02_rb,2.9027e-02_rb,3.1970e-02_rb,3.5096e-02_rb/)
      kbo(:,28, 6) = (/ &
     &2.2118e-02_rb,2.4355e-02_rb,2.6986e-02_rb,2.9969e-02_rb,3.3170e-02_rb/)
      kbo(:,29, 6) = (/ &
     &2.0075e-02_rb,2.2435e-02_rb,2.5083e-02_rb,2.8074e-02_rb,3.1267e-02_rb/)
      kbo(:,30, 6) = (/ &
     &1.8356e-02_rb,2.0732e-02_rb,2.3448e-02_rb,2.6423e-02_rb,3.0839e-02_rb/)
      kbo(:,31, 6) = (/ &
     &1.6821e-02_rb,1.9252e-02_rb,2.1963e-02_rb,2.5007e-02_rb,2.9517e-02_rb/)
      kbo(:,32, 6) = (/ &
     &1.5646e-02_rb,1.8065e-02_rb,2.0807e-02_rb,2.3917e-02_rb,2.8359e-02_rb/)
      kbo(:,33, 6) = (/ &
     &1.4697e-02_rb,1.7147e-02_rb,1.9949e-02_rb,2.3040e-02_rb,2.7566e-02_rb/)
      kbo(:,34, 6) = (/ &
     &1.3988e-02_rb,1.6488e-02_rb,1.9318e-02_rb,2.3681e-02_rb,2.7159e-02_rb/)
      kbo(:,35, 6) = (/ &
     &1.3271e-02_rb,1.5781e-02_rb,1.8680e-02_rb,2.3148e-02_rb,2.6620e-02_rb/)
      kbo(:,36, 6) = (/ &
     &1.2443e-02_rb,1.5030e-02_rb,1.7973e-02_rb,2.2405e-02_rb,2.5873e-02_rb/)
      kbo(:,37, 6) = (/ &
     &1.1274e-02_rb,1.3767e-02_rb,1.6642e-02_rb,2.1024e-02_rb,2.4462e-02_rb/)
      kbo(:,38, 6) = (/ &
     &1.0206e-02_rb,1.2636e-02_rb,1.5395e-02_rb,1.9741e-02_rb,2.3148e-02_rb/)
      kbo(:,39, 6) = (/ &
     &9.2481e-03_rb,1.1600e-02_rb,1.4300e-02_rb,1.7327e-02_rb,2.1951e-02_rb/)
      kbo(:,40, 6) = (/ &
     &8.1267e-03_rb,1.0335e-02_rb,1.2898e-02_rb,1.5836e-02_rb,2.0384e-02_rb/)
      kbo(:,41, 6) = (/ &
     &7.0946e-03_rb,9.1506e-03_rb,1.1587e-02_rb,1.4430e-02_rb,1.8862e-02_rb/)
      kbo(:,42, 6) = (/ &
     &6.1743e-03_rb,8.0792e-03_rb,1.0417e-02_rb,1.3119e-02_rb,1.7424e-02_rb/)
      kbo(:,43, 6) = (/ &
     &5.2531e-03_rb,6.9901e-03_rb,9.1605e-03_rb,1.1697e-02_rb,1.4583e-02_rb/)
      kbo(:,44, 6) = (/ &
     &4.3985e-03_rb,5.9665e-03_rb,7.9494e-03_rb,1.0327e-02_rb,1.3038e-02_rb/)
      kbo(:,45, 6) = (/ &
     &3.6636e-03_rb,5.0581e-03_rb,6.8612e-03_rb,9.0504e-03_rb,1.1599e-02_rb/)
      kbo(:,46, 6) = (/ &
     &3.0066e-03_rb,4.2365e-03_rb,5.8498e-03_rb,7.8432e-03_rb,1.0214e-02_rb/)
      kbo(:,47, 6) = (/ &
     &2.4125e-03_rb,3.4783e-03_rb,4.8978e-03_rb,6.6863e-03_rb,8.8516e-03_rb/)
      kbo(:,48, 6) = (/ &
     &1.9141e-03_rb,2.8190e-03_rb,4.0533e-03_rb,5.6391e-03_rb,7.5940e-03_rb/)
      kbo(:,49, 6) = (/ &
     &1.4995e-03_rb,2.2582e-03_rb,3.3183e-03_rb,4.7078e-03_rb,6.4551e-03_rb/)
      kbo(:,50, 6) = (/ &
     &1.1787e-03_rb,1.8162e-03_rb,2.7290e-03_rb,3.9486e-03_rb,5.5105e-03_rb/)
      kbo(:,51, 6) = (/ &
     &9.2536e-04_rb,1.4546e-03_rb,2.2372e-03_rb,3.3051e-03_rb,4.6909e-03_rb/)
      kbo(:,52, 6) = (/ &
     &7.1718e-04_rb,1.1533e-03_rb,1.8133e-03_rb,2.7360e-03_rb,3.9584e-03_rb/)
      kbo(:,53, 6) = (/ &
     &5.4913e-04_rb,9.0136e-04_rb,1.4515e-03_rb,2.2390e-03_rb,3.3052e-03_rb/)
      kbo(:,54, 6) = (/ &
     &4.2639e-04_rb,7.1524e-04_rb,1.1781e-03_rb,1.8604e-03_rb,2.8043e-03_rb/)
      kbo(:,55, 6) = (/ &
     &3.3211e-04_rb,5.6986e-04_rb,9.6021e-04_rb,1.5541e-03_rb,2.3898e-03_rb/)
      kbo(:,56, 6) = (/ &
     &2.5572e-04_rb,4.4925e-04_rb,7.7580e-04_rb,1.2883e-03_rb,2.0250e-03_rb/)
      kbo(:,57, 6) = (/ &
     &1.9517e-04_rb,3.4994e-04_rb,6.2105e-04_rb,1.0576e-03_rb,1.7032e-03_rb/)
      kbo(:,58, 6) = (/ &
     &1.4935e-04_rb,2.7300e-04_rb,4.9776e-04_rb,8.6979e-04_rb,1.4384e-03_rb/)
      kbo(:,59, 6) = (/ &
     &1.2881e-04_rb,2.4100e-04_rb,4.4942e-04_rb,8.0211e-04_rb,1.3455e-03_rb/)
      kbo(:,13, 7) = (/ &
     &1.5834e-01_rb,1.6249e-01_rb,1.6718e-01_rb,1.7299e-01_rb,1.7881e-01_rb/)
      kbo(:,14, 7) = (/ &
     &1.4697e-01_rb,1.5136e-01_rb,1.5631e-01_rb,1.6088e-01_rb,1.6631e-01_rb/)
      kbo(:,15, 7) = (/ &
     &1.4037e-01_rb,1.4484e-01_rb,1.4966e-01_rb,1.5465e-01_rb,1.6040e-01_rb/)
      kbo(:,16, 7) = (/ &
     &1.3335e-01_rb,1.3799e-01_rb,1.4282e-01_rb,1.4817e-01_rb,1.5433e-01_rb/)
      kbo(:,17, 7) = (/ &
     &1.2994e-01_rb,1.3481e-01_rb,1.4043e-01_rb,1.4652e-01_rb,1.5312e-01_rb/)
      kbo(:,18, 7) = (/ &
     &1.2814e-01_rb,1.3395e-01_rb,1.4042e-01_rb,1.4789e-01_rb,1.5627e-01_rb/)
      kbo(:,19, 7) = (/ &
     &1.2710e-01_rb,1.3361e-01_rb,1.4139e-01_rb,1.4985e-01_rb,1.5917e-01_rb/)
      kbo(:,20, 7) = (/ &
     &1.2151e-01_rb,1.2844e-01_rb,1.3635e-01_rb,1.4526e-01_rb,1.5485e-01_rb/)
      kbo(:,21, 7) = (/ &
     &1.1463e-01_rb,1.2176e-01_rb,1.2980e-01_rb,1.3882e-01_rb,1.4834e-01_rb/)
      kbo(:,22, 7) = (/ &
     &1.0733e-01_rb,1.1471e-01_rb,1.2303e-01_rb,1.3189e-01_rb,1.4224e-01_rb/)
      kbo(:,23, 7) = (/ &
     &1.0056e-01_rb,1.0811e-01_rb,1.1675e-01_rb,1.2597e-01_rb,1.3621e-01_rb/)
      kbo(:,24, 7) = (/ &
     &9.4505e-02_rb,1.0259e-01_rb,1.1127e-01_rb,1.2075e-01_rb,1.3101e-01_rb/)
      kbo(:,25, 7) = (/ &
     &8.9426e-02_rb,9.7409e-02_rb,1.0620e-01_rb,1.1605e-01_rb,1.2683e-01_rb/)
      kbo(:,26, 7) = (/ &
     &8.4533e-02_rb,9.2442e-02_rb,1.0164e-01_rb,1.1173e-01_rb,1.2271e-01_rb/)
      kbo(:,27, 7) = (/ &
     &7.9749e-02_rb,8.8003e-02_rb,9.7311e-02_rb,1.0746e-01_rb,1.1871e-01_rb/)
      kbo(:,28, 7) = (/ &
     &7.5483e-02_rb,8.3936e-02_rb,9.3313e-02_rb,1.0378e-01_rb,1.1507e-01_rb/)
      kbo(:,29, 7) = (/ &
     &7.1103e-02_rb,7.9904e-02_rb,8.9739e-02_rb,1.0032e-01_rb,1.1148e-01_rb/)
      kbo(:,30, 7) = (/ &
     &6.7483e-02_rb,7.6590e-02_rb,8.6589e-02_rb,9.7210e-02_rb,1.1098e-01_rb/)
      kbo(:,31, 7) = (/ &
     &6.4404e-02_rb,7.3695e-02_rb,8.3853e-02_rb,9.4939e-02_rb,1.0916e-01_rb/)
      kbo(:,32, 7) = (/ &
     &6.2132e-02_rb,7.1593e-02_rb,8.2197e-02_rb,9.3726e-02_rb,1.0838e-01_rb/)
      kbo(:,33, 7) = (/ &
     &6.0812e-02_rb,7.0660e-02_rb,8.1722e-02_rb,9.3787e-02_rb,1.0875e-01_rb/)
      kbo(:,34, 7) = (/ &
     &6.0340e-02_rb,7.0788e-02_rb,8.2210e-02_rb,9.6618e-02_rb,1.0965e-01_rb/)
      kbo(:,35, 7) = (/ &
     &5.9655e-02_rb,7.0346e-02_rb,8.1926e-02_rb,9.6670e-02_rb,1.0999e-01_rb/)
      kbo(:,36, 7) = (/ &
     &5.8475e-02_rb,6.9222e-02_rb,8.1005e-02_rb,9.6061e-02_rb,1.0964e-01_rb/)
      kbo(:,37, 7) = (/ &
     &5.5330e-02_rb,6.5969e-02_rb,7.7716e-02_rb,9.2736e-02_rb,1.0626e-01_rb/)
      kbo(:,38, 7) = (/ &
     &5.2353e-02_rb,6.2872e-02_rb,7.4519e-02_rb,8.9400e-02_rb,1.0278e-01_rb/)
      kbo(:,39, 7) = (/ &
     &4.9669e-02_rb,5.9933e-02_rb,7.1439e-02_rb,8.4019e-02_rb,9.9553e-02_rb/)
      kbo(:,40, 7) = (/ &
     &4.5810e-02_rb,5.5845e-02_rb,6.7001e-02_rb,7.9231e-02_rb,9.4558e-02_rb/)
      kbo(:,41, 7) = (/ &
     &4.2118e-02_rb,5.1782e-02_rb,6.2542e-02_rb,7.4429e-02_rb,8.9527e-02_rb/)
      kbo(:,42, 7) = (/ &
     &3.8602e-02_rb,4.7947e-02_rb,5.8290e-02_rb,6.9790e-02_rb,8.4691e-02_rb/)
      kbo(:,43, 7) = (/ &
     &3.4683e-02_rb,4.3605e-02_rb,5.3513e-02_rb,6.4549e-02_rb,7.6612e-02_rb/)
      kbo(:,44, 7) = (/ &
     &3.0820e-02_rb,3.9228e-02_rb,4.8707e-02_rb,5.9113e-02_rb,7.0667e-02_rb/)
      kbo(:,45, 7) = (/ &
     &2.7206e-02_rb,3.5088e-02_rb,4.4017e-02_rb,5.3958e-02_rb,6.4971e-02_rb/)
      kbo(:,46, 7) = (/ &
     &2.3728e-02_rb,3.1004e-02_rb,3.9410e-02_rb,4.8844e-02_rb,5.9195e-02_rb/)
      kbo(:,47, 7) = (/ &
     &2.0318e-02_rb,2.6968e-02_rb,3.4803e-02_rb,4.3672e-02_rb,5.3514e-02_rb/)
      kbo(:,48, 7) = (/ &
     &1.7221e-02_rb,2.3270e-02_rb,3.0475e-02_rb,3.8751e-02_rb,4.8067e-02_rb/)
      kbo(:,49, 7) = (/ &
     &1.4371e-02_rb,1.9831e-02_rb,2.6371e-02_rb,3.4011e-02_rb,4.2706e-02_rb/)
      kbo(:,50, 7) = (/ &
     &1.2067e-02_rb,1.6953e-02_rb,2.2948e-02_rb,3.0025e-02_rb,3.8163e-02_rb/)
      kbo(:,51, 7) = (/ &
     &1.0081e-02_rb,1.4494e-02_rb,1.9976e-02_rb,2.6511e-02_rb,3.4117e-02_rb/)
      kbo(:,52, 7) = (/ &
     &8.3102e-03_rb,1.2227e-02_rb,1.7157e-02_rb,2.3198e-02_rb,3.0263e-02_rb/)
      kbo(:,53, 7) = (/ &
     &6.7300e-03_rb,1.0157e-02_rb,1.4580e-02_rb,2.0030e-02_rb,2.6549e-02_rb/)
      kbo(:,54, 7) = (/ &
     &5.5699e-03_rb,8.6183e-03_rb,1.2632e-02_rb,1.7641e-02_rb,2.3782e-02_rb/)
      kbo(:,55, 7) = (/ &
     &4.6424e-03_rb,7.3686e-03_rb,1.1048e-02_rb,1.5705e-02_rb,2.1480e-02_rb/)
      kbo(:,56, 7) = (/ &
     &3.8360e-03_rb,6.2508e-03_rb,9.5901e-03_rb,1.3914e-02_rb,1.9327e-02_rb/)
      kbo(:,57, 7) = (/ &
     &3.1377e-03_rb,5.2589e-03_rb,8.2647e-03_rb,1.2235e-02_rb,1.7290e-02_rb/)
      kbo(:,58, 7) = (/ &
     &2.5678e-03_rb,4.4320e-03_rb,7.1403e-03_rb,1.0793e-02_rb,1.5530e-02_rb/)
      kbo(:,59, 7) = (/ &
     &2.3969e-03_rb,4.2221e-03_rb,6.8990e-03_rb,1.0531e-02_rb,1.5294e-02_rb/)
      kbo(:,13, 8) = (/ &
     &4.5374e-01_rb,4.6892e-01_rb,4.8468e-01_rb,5.0059e-01_rb,5.1827e-01_rb/)
      kbo(:,14, 8) = (/ &
     &4.3219e-01_rb,4.4586e-01_rb,4.6019e-01_rb,4.7608e-01_rb,4.9298e-01_rb/)
      kbo(:,15, 8) = (/ &
     &4.0920e-01_rb,4.2277e-01_rb,4.3751e-01_rb,4.5370e-01_rb,4.7139e-01_rb/)
      kbo(:,16, 8) = (/ &
     &3.8774e-01_rb,3.9959e-01_rb,4.1438e-01_rb,4.3100e-01_rb,4.4920e-01_rb/)
      kbo(:,17, 8) = (/ &
     &3.7675e-01_rb,3.9034e-01_rb,4.0535e-01_rb,4.2142e-01_rb,4.3931e-01_rb/)
      kbo(:,18, 8) = (/ &
     &3.7210e-01_rb,3.8690e-01_rb,4.0196e-01_rb,4.1888e-01_rb,4.3633e-01_rb/)
      kbo(:,19, 8) = (/ &
     &3.8004e-01_rb,3.9604e-01_rb,4.1156e-01_rb,4.2942e-01_rb,4.4872e-01_rb/)
      kbo(:,20, 8) = (/ &
     &3.8032e-01_rb,3.9826e-01_rb,4.1629e-01_rb,4.3599e-01_rb,4.5653e-01_rb/)
      kbo(:,21, 8) = (/ &
     &3.7841e-01_rb,3.9929e-01_rb,4.1819e-01_rb,4.3972e-01_rb,4.6162e-01_rb/)
      kbo(:,22, 8) = (/ &
     &3.7504e-01_rb,3.9852e-01_rb,4.2087e-01_rb,4.4494e-01_rb,4.6985e-01_rb/)
      kbo(:,23, 8) = (/ &
     &3.7298e-01_rb,3.9721e-01_rb,4.2335e-01_rb,4.5127e-01_rb,4.7949e-01_rb/)
      kbo(:,24, 8) = (/ &
     &3.7096e-01_rb,3.9820e-01_rb,4.2817e-01_rb,4.5927e-01_rb,4.9128e-01_rb/)
      kbo(:,25, 8) = (/ &
     &3.6772e-01_rb,3.9818e-01_rb,4.3145e-01_rb,4.6548e-01_rb,5.0188e-01_rb/)
      kbo(:,26, 8) = (/ &
     &3.6515e-01_rb,3.9789e-01_rb,4.3248e-01_rb,4.6959e-01_rb,5.0716e-01_rb/)
      kbo(:,27, 8) = (/ &
     &3.5912e-01_rb,3.9421e-01_rb,4.3089e-01_rb,4.6934e-01_rb,5.0871e-01_rb/)
      kbo(:,28, 8) = (/ &
     &3.5260e-01_rb,3.8990e-01_rb,4.2842e-01_rb,4.6770e-01_rb,5.0872e-01_rb/)
      kbo(:,29, 8) = (/ &
     &3.4566e-01_rb,3.8302e-01_rb,4.2135e-01_rb,4.6236e-01_rb,5.0539e-01_rb/)
      kbo(:,30, 8) = (/ &
     &3.3909e-01_rb,3.7648e-01_rb,4.1601e-01_rb,4.5869e-01_rb,5.0690e-01_rb/)
      kbo(:,31, 8) = (/ &
     &3.3097e-01_rb,3.6927e-01_rb,4.1020e-01_rb,4.5248e-01_rb,5.0201e-01_rb/)
      kbo(:,32, 8) = (/ &
     &3.2677e-01_rb,3.6552e-01_rb,4.0712e-01_rb,4.4947e-01_rb,4.9828e-01_rb/)
      kbo(:,33, 8) = (/ &
     &3.2437e-01_rb,3.6324e-01_rb,4.0445e-01_rb,4.4744e-01_rb,4.9526e-01_rb/)
      kbo(:,34, 8) = (/ &
     &3.2478e-01_rb,3.6385e-01_rb,4.0433e-01_rb,4.5266e-01_rb,4.9618e-01_rb/)
      kbo(:,35, 8) = (/ &
     &3.2335e-01_rb,3.6225e-01_rb,4.0283e-01_rb,4.5093e-01_rb,4.9595e-01_rb/)
      kbo(:,36, 8) = (/ &
     &3.1952e-01_rb,3.5913e-01_rb,4.0020e-01_rb,4.4926e-01_rb,4.9354e-01_rb/)
      kbo(:,37, 8) = (/ &
     &3.0545e-01_rb,3.4531e-01_rb,3.8604e-01_rb,4.3520e-01_rb,4.7874e-01_rb/)
      kbo(:,38, 8) = (/ &
     &2.9161e-01_rb,3.3033e-01_rb,3.7111e-01_rb,4.1959e-01_rb,4.6334e-01_rb/)
      kbo(:,39, 8) = (/ &
     &2.7728e-01_rb,3.1579e-01_rb,3.5591e-01_rb,3.9884e-01_rb,4.4658e-01_rb/)
      kbo(:,40, 8) = (/ &
     &2.6059e-01_rb,2.9767e-01_rb,3.3670e-01_rb,3.7917e-01_rb,4.2621e-01_rb/)
      kbo(:,41, 8) = (/ &
     &2.4427e-01_rb,2.7974e-01_rb,3.1821e-01_rb,3.5868e-01_rb,4.0499e-01_rb/)
      kbo(:,42, 8) = (/ &
     &2.2818e-01_rb,2.6203e-01_rb,2.9920e-01_rb,3.3811e-01_rb,3.8330e-01_rb/)
      kbo(:,43, 8) = (/ &
     &2.1001e-01_rb,2.4274e-01_rb,2.7765e-01_rb,3.1477e-01_rb,3.5346e-01_rb/)
      kbo(:,44, 8) = (/ &
     &1.9124e-01_rb,2.2202e-01_rb,2.5515e-01_rb,2.8981e-01_rb,3.2699e-01_rb/)
      kbo(:,45, 8) = (/ &
     &1.7260e-01_rb,2.0229e-01_rb,2.3352e-01_rb,2.6561e-01_rb,2.9973e-01_rb/)
      kbo(:,46, 8) = (/ &
     &1.5463e-01_rb,1.8250e-01_rb,2.1175e-01_rb,2.4206e-01_rb,2.7362e-01_rb/)
      kbo(:,47, 8) = (/ &
     &1.3753e-01_rb,1.6353e-01_rb,1.9105e-01_rb,2.1973e-01_rb,2.4911e-01_rb/)
      kbo(:,48, 8) = (/ &
     &1.2102e-01_rb,1.4531e-01_rb,1.7086e-01_rb,1.9744e-01_rb,2.2473e-01_rb/)
      kbo(:,49, 8) = (/ &
     &1.0521e-01_rb,1.2719e-01_rb,1.5065e-01_rb,1.7507e-01_rb,1.9976e-01_rb/)
      kbo(:,50, 8) = (/ &
     &9.2290e-02_rb,1.1288e-01_rb,1.3468e-01_rb,1.5758e-01_rb,1.8102e-01_rb/)
      kbo(:,51, 8) = (/ &
     &8.1396e-02_rb,1.0065e-01_rb,1.2103e-01_rb,1.4278e-01_rb,1.6472e-01_rb/)
      kbo(:,52, 8) = (/ &
     &7.1108e-02_rb,8.8827e-02_rb,1.0785e-01_rb,1.2780e-01_rb,1.4857e-01_rb/)
      kbo(:,53, 8) = (/ &
     &6.0985e-02_rb,7.7296e-02_rb,9.4909e-02_rb,1.1304e-01_rb,1.3292e-01_rb/)
      kbo(:,54, 8) = (/ &
     &5.4165e-02_rb,6.9879e-02_rb,8.6686e-02_rb,1.0438e-01_rb,1.2336e-01_rb/)
      kbo(:,55, 8) = (/ &
     &4.9072e-02_rb,6.4377e-02_rb,8.0975e-02_rb,9.8517e-02_rb,1.1715e-01_rb/)
      kbo(:,56, 8) = (/ &
     &4.4131e-02_rb,5.9064e-02_rb,7.5254e-02_rb,9.2618e-02_rb,1.1083e-01_rb/)
      kbo(:,57, 8) = (/ &
     &3.9404e-02_rb,5.3744e-02_rb,6.9648e-02_rb,8.6728e-02_rb,1.0474e-01_rb/)
      kbo(:,58, 8) = (/ &
     &3.5245e-02_rb,4.9278e-02_rb,6.4917e-02_rb,8.1857e-02_rb,9.9966e-02_rb/)
      kbo(:,59, 8) = (/ &
     &3.5641e-02_rb,5.0530e-02_rb,6.7256e-02_rb,8.5600e-02_rb,1.0518e-01_rb/)
      kbo(:,13, 9) = (/ &
     &3.0582e+00_rb,3.1802e+00_rb,3.2924e+00_rb,3.3999e+00_rb,3.5039e+00_rb/)
      kbo(:,14, 9) = (/ &
     &2.7741e+00_rb,2.8902e+00_rb,2.9983e+00_rb,3.1018e+00_rb,3.1995e+00_rb/)
      kbo(:,15, 9) = (/ &
     &2.5019e+00_rb,2.6118e+00_rb,2.7174e+00_rb,2.8165e+00_rb,2.9092e+00_rb/)
      kbo(:,16, 9) = (/ &
     &2.2794e+00_rb,2.3785e+00_rb,2.4700e+00_rb,2.5601e+00_rb,2.6476e+00_rb/)
      kbo(:,17, 9) = (/ &
     &2.1431e+00_rb,2.2237e+00_rb,2.2973e+00_rb,2.3689e+00_rb,2.4470e+00_rb/)
      kbo(:,18, 9) = (/ &
     &2.1112e+00_rb,2.1931e+00_rb,2.2642e+00_rb,2.3350e+00_rb,2.3995e+00_rb/)
      kbo(:,19, 9) = (/ &
     &2.1192e+00_rb,2.1960e+00_rb,2.2679e+00_rb,2.3312e+00_rb,2.3985e+00_rb/)
      kbo(:,20, 9) = (/ &
     &2.0719e+00_rb,2.1504e+00_rb,2.2204e+00_rb,2.2883e+00_rb,2.3541e+00_rb/)
      kbo(:,21, 9) = (/ &
     &2.0023e+00_rb,2.0787e+00_rb,2.1566e+00_rb,2.2279e+00_rb,2.2981e+00_rb/)
      kbo(:,22, 9) = (/ &
     &1.9677e+00_rb,2.0354e+00_rb,2.1023e+00_rb,2.1820e+00_rb,2.2522e+00_rb/)
      kbo(:,23, 9) = (/ &
     &1.9509e+00_rb,2.0289e+00_rb,2.0929e+00_rb,2.1547e+00_rb,2.2177e+00_rb/)
      kbo(:,24, 9) = (/ &
     &1.9605e+00_rb,2.0316e+00_rb,2.0945e+00_rb,2.1537e+00_rb,2.2147e+00_rb/)
      kbo(:,25, 9) = (/ &
     &1.9774e+00_rb,2.0460e+00_rb,2.1037e+00_rb,2.1656e+00_rb,2.2237e+00_rb/)
      kbo(:,26, 9) = (/ &
     &1.9746e+00_rb,2.0494e+00_rb,2.1183e+00_rb,2.1793e+00_rb,2.2462e+00_rb/)
      kbo(:,27, 9) = (/ &
     &1.9769e+00_rb,2.0480e+00_rb,2.1184e+00_rb,2.1915e+00_rb,2.2594e+00_rb/)
      kbo(:,28, 9) = (/ &
     &1.9705e+00_rb,2.0478e+00_rb,2.1273e+00_rb,2.2021e+00_rb,2.2684e+00_rb/)
      kbo(:,29, 9) = (/ &
     &1.9428e+00_rb,2.0233e+00_rb,2.1092e+00_rb,2.1834e+00_rb,2.2523e+00_rb/)
      kbo(:,30, 9) = (/ &
     &1.9054e+00_rb,1.9928e+00_rb,2.0804e+00_rb,2.1559e+00_rb,2.2456e+00_rb/)
      kbo(:,31, 9) = (/ &
     &1.8578e+00_rb,1.9522e+00_rb,2.0363e+00_rb,2.1176e+00_rb,2.2067e+00_rb/)
      kbo(:,32, 9) = (/ &
     &1.8167e+00_rb,1.9163e+00_rb,1.9986e+00_rb,2.0826e+00_rb,2.1707e+00_rb/)
      kbo(:,33, 9) = (/ &
     &1.7835e+00_rb,1.8773e+00_rb,1.9713e+00_rb,2.0531e+00_rb,2.1415e+00_rb/)
      kbo(:,34, 9) = (/ &
     &1.7699e+00_rb,1.8654e+00_rb,1.9579e+00_rb,2.0589e+00_rb,2.1344e+00_rb/)
      kbo(:,35, 9) = (/ &
     &1.7588e+00_rb,1.8543e+00_rb,1.9464e+00_rb,2.0494e+00_rb,2.1255e+00_rb/)
      kbo(:,36, 9) = (/ &
     &1.7548e+00_rb,1.8549e+00_rb,1.9407e+00_rb,2.0440e+00_rb,2.1280e+00_rb/)
      kbo(:,37, 9) = (/ &
     &1.7109e+00_rb,1.8072e+00_rb,1.9003e+00_rb,1.9969e+00_rb,2.0787e+00_rb/)
      kbo(:,38, 9) = (/ &
     &1.6587e+00_rb,1.7570e+00_rb,1.8515e+00_rb,1.9517e+00_rb,2.0362e+00_rb/)
      kbo(:,39, 9) = (/ &
     &1.5996e+00_rb,1.6993e+00_rb,1.7940e+00_rb,1.8844e+00_rb,1.9872e+00_rb/)
      kbo(:,40, 9) = (/ &
     &1.5318e+00_rb,1.6341e+00_rb,1.7309e+00_rb,1.8212e+00_rb,1.9247e+00_rb/)
      kbo(:,41, 9) = (/ &
     &1.4675e+00_rb,1.5693e+00_rb,1.6630e+00_rb,1.7538e+00_rb,1.8608e+00_rb/)
      kbo(:,42, 9) = (/ &
     &1.4053e+00_rb,1.5014e+00_rb,1.5957e+00_rb,1.6862e+00_rb,1.7957e+00_rb/)
      kbo(:,43, 9) = (/ &
     &1.3266e+00_rb,1.4241e+00_rb,1.5193e+00_rb,1.6083e+00_rb,1.7037e+00_rb/)
      kbo(:,44, 9) = (/ &
     &1.2422e+00_rb,1.3405e+00_rb,1.4375e+00_rb,1.5296e+00_rb,1.6219e+00_rb/)
      kbo(:,45, 9) = (/ &
     &1.1571e+00_rb,1.2540e+00_rb,1.3469e+00_rb,1.4395e+00_rb,1.5427e+00_rb/)
      kbo(:,46, 9) = (/ &
     &1.0762e+00_rb,1.1688e+00_rb,1.2622e+00_rb,1.3587e+00_rb,1.4621e+00_rb/)
      kbo(:,47, 9) = (/ &
     &9.9321e-01_rb,1.0861e+00_rb,1.1791e+00_rb,1.2774e+00_rb,1.3779e+00_rb/)
      kbo(:,48, 9) = (/ &
     &9.0640e-01_rb,1.0035e+00_rb,1.0964e+00_rb,1.1955e+00_rb,1.2984e+00_rb/)
      kbo(:,49, 9) = (/ &
     &8.2476e-01_rb,9.1593e-01_rb,1.0111e+00_rb,1.1148e+00_rb,1.2202e+00_rb/)
      kbo(:,50, 9) = (/ &
     &7.5762e-01_rb,8.4984e-01_rb,9.4454e-01_rb,1.0468e+00_rb,1.1530e+00_rb/)
      kbo(:,51, 9) = (/ &
     &6.9972e-01_rb,7.8939e-01_rb,8.8551e-01_rb,9.8599e-01_rb,1.0894e+00_rb/)
      kbo(:,52, 9) = (/ &
     &6.4292e-01_rb,7.3062e-01_rb,8.2745e-01_rb,9.2620e-01_rb,1.0265e+00_rb/)
      kbo(:,53, 9) = (/ &
     &5.8631e-01_rb,6.7367e-01_rb,7.6794e-01_rb,8.6557e-01_rb,9.6191e-01_rb/)
      kbo(:,54, 9) = (/ &
     &5.4766e-01_rb,6.3312e-01_rb,7.2658e-01_rb,8.2340e-01_rb,9.1820e-01_rb/)
      kbo(:,55, 9) = (/ &
     &5.1893e-01_rb,6.0477e-01_rb,6.9346e-01_rb,7.9003e-01_rb,8.8552e-01_rb/)
      kbo(:,56, 9) = (/ &
     &4.9077e-01_rb,5.7504e-01_rb,6.6139e-01_rb,7.5822e-01_rb,8.5232e-01_rb/)
      kbo(:,57, 9) = (/ &
     &4.6294e-01_rb,5.4644e-01_rb,6.3217e-01_rb,7.2497e-01_rb,8.2095e-01_rb/)
      kbo(:,58, 9) = (/ &
     &4.3872e-01_rb,5.2209e-01_rb,6.0717e-01_rb,6.9576e-01_rb,7.9255e-01_rb/)
      kbo(:,59, 9) = (/ &
     &4.5355e-01_rb,5.4099e-01_rb,6.2851e-01_rb,7.1405e-01_rb,8.0552e-01_rb/)
      kbo(:,13,10) = (/ &
     &1.4982e+01_rb,1.5495e+01_rb,1.5954e+01_rb,1.6348e+01_rb,1.6704e+01_rb/)
      kbo(:,14,10) = (/ &
     &1.3335e+01_rb,1.3739e+01_rb,1.4075e+01_rb,1.4394e+01_rb,1.4701e+01_rb/)
      kbo(:,15,10) = (/ &
     &1.2249e+01_rb,1.2466e+01_rb,1.2680e+01_rb,1.2851e+01_rb,1.2999e+01_rb/)
      kbo(:,16,10) = (/ &
     &1.1258e+01_rb,1.1509e+01_rb,1.1735e+01_rb,1.1863e+01_rb,1.1992e+01_rb/)
      kbo(:,17,10) = (/ &
     &9.6399e+00_rb,9.9316e+00_rb,1.0198e+01_rb,1.0495e+01_rb,1.0685e+01_rb/)
      kbo(:,18,10) = (/ &
     &8.3200e+00_rb,8.3604e+00_rb,8.4791e+00_rb,8.5483e+00_rb,8.6485e+00_rb/)
      kbo(:,19,10) = (/ &
     &8.1074e+00_rb,8.1199e+00_rb,8.1734e+00_rb,8.2725e+00_rb,8.2532e+00_rb/)
      kbo(:,20,10) = (/ &
     &8.0251e+00_rb,8.0270e+00_rb,8.1639e+00_rb,8.2375e+00_rb,8.2230e+00_rb/)
      kbo(:,21,10) = (/ &
     &7.9993e+00_rb,8.0934e+00_rb,8.2060e+00_rb,8.1449e+00_rb,8.1878e+00_rb/)
      kbo(:,22,10) = (/ &
     &7.7052e+00_rb,7.8023e+00_rb,8.0192e+00_rb,8.0278e+00_rb,8.0411e+00_rb/)
      kbo(:,23,10) = (/ &
     &7.2443e+00_rb,7.3880e+00_rb,7.6036e+00_rb,7.7289e+00_rb,7.8141e+00_rb/)
      kbo(:,24,10) = (/ &
     &6.8599e+00_rb,6.9432e+00_rb,7.2334e+00_rb,7.4508e+00_rb,7.5685e+00_rb/)
      kbo(:,25,10) = (/ &
     &6.7077e+00_rb,6.7570e+00_rb,7.0253e+00_rb,7.1690e+00_rb,7.3506e+00_rb/)
      kbo(:,26,10) = (/ &
     &6.7805e+00_rb,6.7972e+00_rb,6.9739e+00_rb,6.9966e+00_rb,7.1457e+00_rb/)
      kbo(:,27,10) = (/ &
     &6.6456e+00_rb,6.8298e+00_rb,6.9537e+00_rb,6.8703e+00_rb,6.9847e+00_rb/)
      kbo(:,28,10) = (/ &
     &6.6227e+00_rb,6.7740e+00_rb,6.7717e+00_rb,6.7969e+00_rb,6.8834e+00_rb/)
      kbo(:,29,10) = (/ &
     &6.5026e+00_rb,6.7130e+00_rb,6.6983e+00_rb,6.7888e+00_rb,6.9842e+00_rb/)
      kbo(:,30,10) = (/ &
     &6.4613e+00_rb,6.7438e+00_rb,6.7406e+00_rb,6.8001e+00_rb,7.0628e+00_rb/)
      kbo(:,31,10) = (/ &
     &6.4453e+00_rb,6.5818e+00_rb,6.6914e+00_rb,6.8343e+00_rb,7.1118e+00_rb/)
      kbo(:,32,10) = (/ &
     &6.4750e+00_rb,6.5165e+00_rb,6.6931e+00_rb,6.9311e+00_rb,7.2907e+00_rb/)
      kbo(:,33,10) = (/ &
     &6.3852e+00_rb,6.5174e+00_rb,6.6687e+00_rb,7.0429e+00_rb,7.4921e+00_rb/)
      kbo(:,34,10) = (/ &
     &6.2937e+00_rb,6.5395e+00_rb,6.7522e+00_rb,7.1615e+00_rb,7.5856e+00_rb/)
      kbo(:,35,10) = (/ &
     &6.2760e+00_rb,6.5561e+00_rb,6.8772e+00_rb,7.3182e+00_rb,7.7508e+00_rb/)
      kbo(:,36,10) = (/ &
     &6.2498e+00_rb,6.5146e+00_rb,6.9721e+00_rb,7.4134e+00_rb,7.7777e+00_rb/)
      kbo(:,37,10) = (/ &
     &6.0875e+00_rb,6.4490e+00_rb,6.8813e+00_rb,7.4029e+00_rb,7.7937e+00_rb/)
      kbo(:,38,10) = (/ &
     &5.9776e+00_rb,6.3534e+00_rb,6.8337e+00_rb,7.2840e+00_rb,7.6800e+00_rb/)
      kbo(:,39,10) = (/ &
     &5.9369e+00_rb,6.2769e+00_rb,6.7228e+00_rb,7.1879e+00_rb,7.5772e+00_rb/)
      kbo(:,40,10) = (/ &
     &5.8199e+00_rb,6.1346e+00_rb,6.5569e+00_rb,7.0308e+00_rb,7.5139e+00_rb/)
      kbo(:,41,10) = (/ &
     &5.6637e+00_rb,5.9953e+00_rb,6.3928e+00_rb,6.8959e+00_rb,7.3899e+00_rb/)
      kbo(:,42,10) = (/ &
     &5.4578e+00_rb,5.8740e+00_rb,6.2837e+00_rb,6.7615e+00_rb,7.2182e+00_rb/)
      kbo(:,43,10) = (/ &
     &5.2432e+00_rb,5.6884e+00_rb,6.0915e+00_rb,6.5892e+00_rb,7.0176e+00_rb/)
      kbo(:,44,10) = (/ &
     &5.0352e+00_rb,5.5134e+00_rb,5.9765e+00_rb,6.3927e+00_rb,6.8192e+00_rb/)
      kbo(:,45,10) = (/ &
     &4.8185e+00_rb,5.3066e+00_rb,5.8216e+00_rb,6.2708e+00_rb,6.5394e+00_rb/)
      kbo(:,46,10) = (/ &
     &4.5458e+00_rb,5.0700e+00_rb,5.5601e+00_rb,5.9708e+00_rb,6.3061e+00_rb/)
      kbo(:,47,10) = (/ &
     &4.3290e+00_rb,4.8090e+00_rb,5.3177e+00_rb,5.7187e+00_rb,6.0493e+00_rb/)
      kbo(:,48,10) = (/ &
     &4.1252e+00_rb,4.6210e+00_rb,5.0031e+00_rb,5.3950e+00_rb,5.6779e+00_rb/)
      kbo(:,49,10) = (/ &
     &3.8776e+00_rb,4.3638e+00_rb,4.7491e+00_rb,5.0119e+00_rb,5.3209e+00_rb/)
      kbo(:,50,10) = (/ &
     &3.6432e+00_rb,4.1171e+00_rb,4.5036e+00_rb,4.8339e+00_rb,5.0310e+00_rb/)
      kbo(:,51,10) = (/ &
     &3.4528e+00_rb,3.8987e+00_rb,4.2685e+00_rb,4.6158e+00_rb,4.8586e+00_rb/)
      kbo(:,52,10) = (/ &
     &3.2373e+00_rb,3.6786e+00_rb,4.0057e+00_rb,4.3579e+00_rb,4.6615e+00_rb/)
      kbo(:,53,10) = (/ &
     &3.0066e+00_rb,3.4261e+00_rb,3.7660e+00_rb,4.0904e+00_rb,4.4162e+00_rb/)
      kbo(:,54,10) = (/ &
     &2.8305e+00_rb,3.2571e+00_rb,3.6215e+00_rb,3.9208e+00_rb,4.2651e+00_rb/)
      kbo(:,55,10) = (/ &
     &2.6852e+00_rb,3.0952e+00_rb,3.4961e+00_rb,3.7995e+00_rb,4.1315e+00_rb/)
      kbo(:,56,10) = (/ &
     &2.5024e+00_rb,2.9749e+00_rb,3.3840e+00_rb,3.6764e+00_rb,4.0215e+00_rb/)
      kbo(:,57,10) = (/ &
     &2.3413e+00_rb,2.8251e+00_rb,3.2333e+00_rb,3.5901e+00_rb,3.8695e+00_rb/)
      kbo(:,58,10) = (/ &
     &2.2399e+00_rb,2.6684e+00_rb,3.1055e+00_rb,3.4841e+00_rb,3.7568e+00_rb/)
      kbo(:,59,10) = (/ &
     &2.3106e+00_rb,2.6842e+00_rb,3.0911e+00_rb,3.5243e+00_rb,3.8718e+00_rb/)
      kbo(:,13,11) = (/ &
     &3.1114e+01_rb,3.1701e+01_rb,3.2323e+01_rb,3.2943e+01_rb,3.3516e+01_rb/)
      kbo(:,14,11) = (/ &
     &2.7715e+01_rb,2.8303e+01_rb,2.8933e+01_rb,2.9548e+01_rb,3.0169e+01_rb/)
      kbo(:,15,11) = (/ &
     &2.4338e+01_rb,2.4986e+01_rb,2.5617e+01_rb,2.6312e+01_rb,2.7005e+01_rb/)
      kbo(:,16,11) = (/ &
     &2.1212e+01_rb,2.1803e+01_rb,2.2432e+01_rb,2.3085e+01_rb,2.3696e+01_rb/)
      kbo(:,17,11) = (/ &
     &1.8965e+01_rb,1.9451e+01_rb,1.9925e+01_rb,2.0308e+01_rb,2.0718e+01_rb/)
      kbo(:,18,11) = (/ &
     &1.6301e+01_rb,1.6846e+01_rb,1.7358e+01_rb,1.7884e+01_rb,1.8436e+01_rb/)
      kbo(:,19,11) = (/ &
     &1.3305e+01_rb,1.3592e+01_rb,1.3869e+01_rb,1.4251e+01_rb,1.4704e+01_rb/)
      kbo(:,20,11) = (/ &
     &1.2734e+01_rb,1.2816e+01_rb,1.2643e+01_rb,1.2573e+01_rb,1.2686e+01_rb/)
      kbo(:,21,11) = (/ &
     &1.2657e+01_rb,1.2589e+01_rb,1.2354e+01_rb,1.2271e+01_rb,1.2114e+01_rb/)
      kbo(:,22,11) = (/ &
     &1.2492e+01_rb,1.2455e+01_rb,1.2170e+01_rb,1.1916e+01_rb,1.1871e+01_rb/)
      kbo(:,23,11) = (/ &
     &1.2437e+01_rb,1.2266e+01_rb,1.2152e+01_rb,1.1923e+01_rb,1.2016e+01_rb/)
      kbo(:,24,11) = (/ &
     &1.2312e+01_rb,1.2409e+01_rb,1.2179e+01_rb,1.1958e+01_rb,1.2034e+01_rb/)
      kbo(:,25,11) = (/ &
     &1.1905e+01_rb,1.2160e+01_rb,1.2010e+01_rb,1.2119e+01_rb,1.2127e+01_rb/)
      kbo(:,26,11) = (/ &
     &1.1398e+01_rb,1.1792e+01_rb,1.1851e+01_rb,1.2073e+01_rb,1.2295e+01_rb/)
      kbo(:,27,11) = (/ &
     &1.1115e+01_rb,1.1534e+01_rb,1.1738e+01_rb,1.2206e+01_rb,1.2469e+01_rb/)
      kbo(:,28,11) = (/ &
     &1.0833e+01_rb,1.1382e+01_rb,1.1757e+01_rb,1.2168e+01_rb,1.2751e+01_rb/)
      kbo(:,29,11) = (/ &
     &1.0849e+01_rb,1.1242e+01_rb,1.1765e+01_rb,1.2112e+01_rb,1.2581e+01_rb/)
      kbo(:,30,11) = (/ &
     &1.0908e+01_rb,1.1219e+01_rb,1.1698e+01_rb,1.2283e+01_rb,1.2675e+01_rb/)
      kbo(:,31,11) = (/ &
     &1.1065e+01_rb,1.1480e+01_rb,1.1755e+01_rb,1.2188e+01_rb,1.2848e+01_rb/)
      kbo(:,32,11) = (/ &
     &1.1087e+01_rb,1.1591e+01_rb,1.1912e+01_rb,1.2336e+01_rb,1.2746e+01_rb/)
      kbo(:,33,11) = (/ &
     &1.1325e+01_rb,1.1662e+01_rb,1.2022e+01_rb,1.2470e+01_rb,1.2619e+01_rb/)
      kbo(:,34,11) = (/ &
     &1.1533e+01_rb,1.1754e+01_rb,1.2155e+01_rb,1.2455e+01_rb,1.2615e+01_rb/)
      kbo(:,35,11) = (/ &
     &1.1659e+01_rb,1.1966e+01_rb,1.2184e+01_rb,1.2500e+01_rb,1.2545e+01_rb/)
      kbo(:,36,11) = (/ &
     &1.1755e+01_rb,1.2074e+01_rb,1.2225e+01_rb,1.2599e+01_rb,1.2813e+01_rb/)
      kbo(:,37,11) = (/ &
     &1.1716e+01_rb,1.1980e+01_rb,1.2067e+01_rb,1.2383e+01_rb,1.2671e+01_rb/)
      kbo(:,38,11) = (/ &
     &1.1652e+01_rb,1.1821e+01_rb,1.1815e+01_rb,1.2298e+01_rb,1.2561e+01_rb/)
      kbo(:,39,11) = (/ &
     &1.1437e+01_rb,1.1591e+01_rb,1.1776e+01_rb,1.2140e+01_rb,1.2306e+01_rb/)
      kbo(:,40,11) = (/ &
     &1.1267e+01_rb,1.1380e+01_rb,1.1567e+01_rb,1.1899e+01_rb,1.1967e+01_rb/)
      kbo(:,41,11) = (/ &
     &1.0928e+01_rb,1.1128e+01_rb,1.1355e+01_rb,1.1698e+01_rb,1.1735e+01_rb/)
      kbo(:,42,11) = (/ &
     &1.0609e+01_rb,1.0842e+01_rb,1.1168e+01_rb,1.1452e+01_rb,1.1524e+01_rb/)
      kbo(:,43,11) = (/ &
     &1.0251e+01_rb,1.0481e+01_rb,1.0835e+01_rb,1.1143e+01_rb,1.1300e+01_rb/)
      kbo(:,44,11) = (/ &
     &9.7650e+00_rb,1.0106e+01_rb,1.0340e+01_rb,1.0725e+01_rb,1.0899e+01_rb/)
      kbo(:,45,11) = (/ &
     &9.2584e+00_rb,9.6204e+00_rb,9.9580e+00_rb,1.0325e+01_rb,1.0626e+01_rb/)
      kbo(:,46,11) = (/ &
     &8.9377e+00_rb,9.1742e+00_rb,9.5458e+00_rb,9.9839e+00_rb,1.0197e+01_rb/)
      kbo(:,47,11) = (/ &
     &8.5371e+00_rb,8.7965e+00_rb,9.0763e+00_rb,9.5717e+00_rb,9.9938e+00_rb/)
      kbo(:,48,11) = (/ &
     &7.9850e+00_rb,8.3229e+00_rb,8.7471e+00_rb,9.1748e+00_rb,9.6663e+00_rb/)
      kbo(:,49,11) = (/ &
     &7.4325e+00_rb,7.8561e+00_rb,8.3257e+00_rb,8.9344e+00_rb,9.3488e+00_rb/)
      kbo(:,50,11) = (/ &
     &6.9934e+00_rb,7.5330e+00_rb,7.9938e+00_rb,8.5231e+00_rb,9.1226e+00_rb/)
      kbo(:,51,11) = (/ &
     &6.6292e+00_rb,7.1581e+00_rb,7.7030e+00_rb,8.1971e+00_rb,8.9077e+00_rb/)
      kbo(:,52,11) = (/ &
     &6.2107e+00_rb,6.7791e+00_rb,7.3981e+00_rb,7.9369e+00_rb,8.8177e+00_rb/)
      kbo(:,53,11) = (/ &
     &5.7771e+00_rb,6.3836e+00_rb,7.0431e+00_rb,7.8871e+00_rb,8.9292e+00_rb/)
      kbo(:,54,11) = (/ &
     &5.5085e+00_rb,6.1058e+00_rb,6.7091e+00_rb,7.6421e+00_rb,8.6682e+00_rb/)
      kbo(:,55,11) = (/ &
     &5.3542e+00_rb,5.9311e+00_rb,6.5099e+00_rb,7.2327e+00_rb,8.2463e+00_rb/)
      kbo(:,56,11) = (/ &
     &5.2013e+00_rb,5.6998e+00_rb,6.2622e+00_rb,6.8978e+00_rb,7.7861e+00_rb/)
      kbo(:,57,11) = (/ &
     &4.9844e+00_rb,5.4898e+00_rb,6.0972e+00_rb,6.6262e+00_rb,7.3665e+00_rb/)
      kbo(:,58,11) = (/ &
     &4.7470e+00_rb,5.3695e+00_rb,5.9047e+00_rb,6.4509e+00_rb,7.0483e+00_rb/)
      kbo(:,59,11) = (/ &
     &4.7734e+00_rb,5.4744e+00_rb,6.0332e+00_rb,6.5609e+00_rb,7.0505e+00_rb/)
      kbo(:,13,12) = (/ &
     &6.9738e+01_rb,7.0752e+01_rb,7.1414e+01_rb,7.2004e+01_rb,7.2657e+01_rb/)
      kbo(:,14,12) = (/ &
     &6.4422e+01_rb,6.5346e+01_rb,6.6191e+01_rb,6.7103e+01_rb,6.7985e+01_rb/)
      kbo(:,15,12) = (/ &
     &5.8702e+01_rb,5.9763e+01_rb,6.0895e+01_rb,6.1978e+01_rb,6.3157e+01_rb/)
      kbo(:,16,12) = (/ &
     &5.3091e+01_rb,5.4316e+01_rb,5.5558e+01_rb,5.6938e+01_rb,5.8399e+01_rb/)
      kbo(:,17,12) = (/ &
     &4.7175e+01_rb,4.8634e+01_rb,5.0234e+01_rb,5.1992e+01_rb,5.3803e+01_rb/)
      kbo(:,18,12) = (/ &
     &4.1145e+01_rb,4.2755e+01_rb,4.4548e+01_rb,4.6436e+01_rb,4.8468e+01_rb/)
      kbo(:,19,12) = (/ &
     &3.4889e+01_rb,3.6512e+01_rb,3.8406e+01_rb,4.0570e+01_rb,4.2997e+01_rb/)
      kbo(:,20,12) = (/ &
     &2.8470e+01_rb,3.0112e+01_rb,3.2281e+01_rb,3.4662e+01_rb,3.7243e+01_rb/)
      kbo(:,21,12) = (/ &
     &2.3947e+01_rb,2.5322e+01_rb,2.7206e+01_rb,2.9369e+01_rb,3.2034e+01_rb/)
      kbo(:,22,12) = (/ &
     &2.2071e+01_rb,2.2771e+01_rb,2.4136e+01_rb,2.6151e+01_rb,2.8304e+01_rb/)
      kbo(:,23,12) = (/ &
     &2.1437e+01_rb,2.2028e+01_rb,2.2789e+01_rb,2.3769e+01_rb,2.5664e+01_rb/)
      kbo(:,24,12) = (/ &
     &2.1104e+01_rb,2.1623e+01_rb,2.2496e+01_rb,2.3486e+01_rb,2.4592e+01_rb/)
      kbo(:,25,12) = (/ &
     &2.1238e+01_rb,2.1868e+01_rb,2.2645e+01_rb,2.3461e+01_rb,2.4762e+01_rb/)
      kbo(:,26,12) = (/ &
     &2.1604e+01_rb,2.2258e+01_rb,2.3294e+01_rb,2.4121e+01_rb,2.5265e+01_rb/)
      kbo(:,27,12) = (/ &
     &2.2115e+01_rb,2.2777e+01_rb,2.3909e+01_rb,2.4766e+01_rb,2.5867e+01_rb/)
      kbo(:,28,12) = (/ &
     &2.2448e+01_rb,2.3238e+01_rb,2.4298e+01_rb,2.5624e+01_rb,2.6343e+01_rb/)
      kbo(:,29,12) = (/ &
     &2.2577e+01_rb,2.3289e+01_rb,2.4323e+01_rb,2.5634e+01_rb,2.6765e+01_rb/)
      kbo(:,30,12) = (/ &
     &2.2578e+01_rb,2.3315e+01_rb,2.4386e+01_rb,2.5500e+01_rb,2.6908e+01_rb/)
      kbo(:,31,12) = (/ &
     &2.2201e+01_rb,2.3259e+01_rb,2.4131e+01_rb,2.5149e+01_rb,2.6545e+01_rb/)
      kbo(:,32,12) = (/ &
     &2.2079e+01_rb,2.2922e+01_rb,2.4111e+01_rb,2.5430e+01_rb,2.6527e+01_rb/)
      kbo(:,33,12) = (/ &
     &2.2095e+01_rb,2.2961e+01_rb,2.4279e+01_rb,2.5414e+01_rb,2.7005e+01_rb/)
      kbo(:,34,12) = (/ &
     &2.2319e+01_rb,2.3341e+01_rb,2.4447e+01_rb,2.6277e+01_rb,2.7940e+01_rb/)
      kbo(:,35,12) = (/ &
     &2.2552e+01_rb,2.3701e+01_rb,2.5211e+01_rb,2.6844e+01_rb,2.9224e+01_rb/)
      kbo(:,36,12) = (/ &
     &2.3085e+01_rb,2.4181e+01_rb,2.5665e+01_rb,2.7600e+01_rb,3.0387e+01_rb/)
      kbo(:,37,12) = (/ &
     &2.2925e+01_rb,2.4128e+01_rb,2.5553e+01_rb,2.7616e+01_rb,3.1077e+01_rb/)
      kbo(:,38,12) = (/ &
     &2.2707e+01_rb,2.3887e+01_rb,2.5581e+01_rb,2.8109e+01_rb,3.1584e+01_rb/)
      kbo(:,39,12) = (/ &
     &2.2511e+01_rb,2.3842e+01_rb,2.5754e+01_rb,2.8880e+01_rb,3.2658e+01_rb/)
      kbo(:,40,12) = (/ &
     &2.1948e+01_rb,2.3291e+01_rb,2.5416e+01_rb,2.8638e+01_rb,3.2604e+01_rb/)
      kbo(:,41,12) = (/ &
     &2.1476e+01_rb,2.2682e+01_rb,2.5024e+01_rb,2.8295e+01_rb,3.2225e+01_rb/)
      kbo(:,42,12) = (/ &
     &2.0874e+01_rb,2.2077e+01_rb,2.4841e+01_rb,2.8049e+01_rb,3.2070e+01_rb/)
      kbo(:,43,12) = (/ &
     &2.0317e+01_rb,2.1501e+01_rb,2.4302e+01_rb,2.7503e+01_rb,3.1532e+01_rb/)
      kbo(:,44,12) = (/ &
     &1.9539e+01_rb,2.0870e+01_rb,2.3855e+01_rb,2.7092e+01_rb,3.1110e+01_rb/)
      kbo(:,45,12) = (/ &
     &1.8830e+01_rb,2.0599e+01_rb,2.3570e+01_rb,2.6837e+01_rb,3.0566e+01_rb/)
      kbo(:,46,12) = (/ &
     &1.8029e+01_rb,2.0375e+01_rb,2.3363e+01_rb,2.6419e+01_rb,3.0085e+01_rb/)
      kbo(:,47,12) = (/ &
     &1.7365e+01_rb,1.9999e+01_rb,2.2921e+01_rb,2.5901e+01_rb,2.9166e+01_rb/)
      kbo(:,48,12) = (/ &
     &1.7043e+01_rb,1.9606e+01_rb,2.2433e+01_rb,2.5542e+01_rb,2.8495e+01_rb/)
      kbo(:,49,12) = (/ &
     &1.6912e+01_rb,1.9487e+01_rb,2.2139e+01_rb,2.5022e+01_rb,2.8110e+01_rb/)
      kbo(:,50,12) = (/ &
     &1.6517e+01_rb,1.8987e+01_rb,2.1585e+01_rb,2.4422e+01_rb,2.7428e+01_rb/)
      kbo(:,51,12) = (/ &
     &1.5891e+01_rb,1.8550e+01_rb,2.1056e+01_rb,2.3767e+01_rb,2.6461e+01_rb/)
      kbo(:,52,12) = (/ &
     &1.5362e+01_rb,1.8085e+01_rb,2.0538e+01_rb,2.3135e+01_rb,2.5477e+01_rb/)
      kbo(:,53,12) = (/ &
     &1.4979e+01_rb,1.7596e+01_rb,2.0169e+01_rb,2.2304e+01_rb,2.4351e+01_rb/)
      kbo(:,54,12) = (/ &
     &1.4292e+01_rb,1.6762e+01_rb,1.9386e+01_rb,2.1407e+01_rb,2.3360e+01_rb/)
      kbo(:,55,12) = (/ &
     &1.3424e+01_rb,1.5675e+01_rb,1.8295e+01_rb,2.0550e+01_rb,2.2398e+01_rb/)
      kbo(:,56,12) = (/ &
     &1.2558e+01_rb,1.4702e+01_rb,1.7198e+01_rb,1.9642e+01_rb,2.1560e+01_rb/)
      kbo(:,57,12) = (/ &
     &1.1789e+01_rb,1.3891e+01_rb,1.6101e+01_rb,1.8574e+01_rb,2.0696e+01_rb/)
      kbo(:,58,12) = (/ &
     &1.0944e+01_rb,1.2980e+01_rb,1.5007e+01_rb,1.7361e+01_rb,1.9688e+01_rb/)
      kbo(:,59,12) = (/ &
     &1.0805e+01_rb,1.1957e+01_rb,1.3692e+01_rb,1.5846e+01_rb,1.8110e+01_rb/)
      kbo(:,13,13) = (/ &
     &1.5252e+02_rb,1.5442e+02_rb,1.5646e+02_rb,1.5779e+02_rb,1.5854e+02_rb/)
      kbo(:,14,13) = (/ &
     &1.5029e+02_rb,1.5315e+02_rb,1.5552e+02_rb,1.5711e+02_rb,1.5813e+02_rb/)
      kbo(:,15,13) = (/ &
     &1.4712e+02_rb,1.5063e+02_rb,1.5294e+02_rb,1.5483e+02_rb,1.5630e+02_rb/)
      kbo(:,16,13) = (/ &
     &1.4282e+02_rb,1.4612e+02_rb,1.4898e+02_rb,1.5141e+02_rb,1.5352e+02_rb/)
      kbo(:,17,13) = (/ &
     &1.3686e+02_rb,1.4066e+02_rb,1.4417e+02_rb,1.4736e+02_rb,1.5020e+02_rb/)
      kbo(:,18,13) = (/ &
     &1.3031e+02_rb,1.3486e+02_rb,1.3910e+02_rb,1.4308e+02_rb,1.4669e+02_rb/)
      kbo(:,19,13) = (/ &
     &1.2305e+02_rb,1.2878e+02_rb,1.3410e+02_rb,1.3882e+02_rb,1.4318e+02_rb/)
      kbo(:,20,13) = (/ &
     &1.1539e+02_rb,1.2206e+02_rb,1.2838e+02_rb,1.3444e+02_rb,1.4010e+02_rb/)
      kbo(:,21,13) = (/ &
     &1.0694e+02_rb,1.1477e+02_rb,1.2233e+02_rb,1.2972e+02_rb,1.3638e+02_rb/)
      kbo(:,22,13) = (/ &
     &9.8072e+01_rb,1.0767e+02_rb,1.1673e+02_rb,1.2508e+02_rb,1.3303e+02_rb/)
      kbo(:,23,13) = (/ &
     &8.9638e+01_rb,1.0003e+02_rb,1.1035e+02_rb,1.2081e+02_rb,1.2954e+02_rb/)
      kbo(:,24,13) = (/ &
     &8.2450e+01_rb,9.3359e+01_rb,1.0421e+02_rb,1.1509e+02_rb,1.2544e+02_rb/)
      kbo(:,25,13) = (/ &
     &7.6879e+01_rb,8.7792e+01_rb,9.9290e+01_rb,1.1048e+02_rb,1.2100e+02_rb/)
      kbo(:,26,13) = (/ &
     &7.3470e+01_rb,8.4233e+01_rb,9.5225e+01_rb,1.0688e+02_rb,1.1752e+02_rb/)
      kbo(:,27,13) = (/ &
     &7.1984e+01_rb,8.2203e+01_rb,9.3044e+01_rb,1.0438e+02_rb,1.1537e+02_rb/)
      kbo(:,28,13) = (/ &
     &7.1683e+01_rb,8.1066e+01_rb,9.2180e+01_rb,1.0291e+02_rb,1.1419e+02_rb/)
      kbo(:,29,13) = (/ &
     &7.3240e+01_rb,8.2713e+01_rb,9.3261e+01_rb,1.0420e+02_rb,1.1497e+02_rb/)
      kbo(:,30,13) = (/ &
     &7.5656e+01_rb,8.5740e+01_rb,9.5155e+01_rb,1.0607e+02_rb,1.1653e+02_rb/)
      kbo(:,31,13) = (/ &
     &7.9639e+01_rb,8.9485e+01_rb,9.9623e+01_rb,1.0965e+02_rb,1.1967e+02_rb/)
      kbo(:,32,13) = (/ &
     &8.5052e+01_rb,9.4635e+01_rb,1.0433e+02_rb,1.1321e+02_rb,1.2287e+02_rb/)
      kbo(:,33,13) = (/ &
     &9.0468e+01_rb,1.0058e+02_rb,1.0969e+02_rb,1.1864e+02_rb,1.2671e+02_rb/)
      kbo(:,34,13) = (/ &
     &9.5615e+01_rb,1.0581e+02_rb,1.1476e+02_rb,1.2258e+02_rb,1.3070e+02_rb/)
      kbo(:,35,13) = (/ &
     &9.9168e+01_rb,1.0972e+02_rb,1.1933e+02_rb,1.2712e+02_rb,1.3373e+02_rb/)
      kbo(:,36,13) = (/ &
     &1.0086e+02_rb,1.1164e+02_rb,1.2168e+02_rb,1.3014e+02_rb,1.3667e+02_rb/)
      kbo(:,37,13) = (/ &
     &1.0047e+02_rb,1.1211e+02_rb,1.2253e+02_rb,1.3099e+02_rb,1.3706e+02_rb/)
      kbo(:,38,13) = (/ &
     &1.0049e+02_rb,1.1250e+02_rb,1.2313e+02_rb,1.3108e+02_rb,1.3732e+02_rb/)
      kbo(:,39,13) = (/ &
     &1.0115e+02_rb,1.1295e+02_rb,1.2350e+02_rb,1.3105e+02_rb,1.3768e+02_rb/)
      kbo(:,40,13) = (/ &
     &9.9765e+01_rb,1.1141e+02_rb,1.2196e+02_rb,1.2992e+02_rb,1.3631e+02_rb/)
      kbo(:,41,13) = (/ &
     &9.8221e+01_rb,1.1003e+02_rb,1.2037e+02_rb,1.2825e+02_rb,1.3487e+02_rb/)
      kbo(:,42,13) = (/ &
     &9.7048e+01_rb,1.0845e+02_rb,1.1823e+02_rb,1.2668e+02_rb,1.3349e+02_rb/)
      kbo(:,43,13) = (/ &
     &9.4582e+01_rb,1.0608e+02_rb,1.1575e+02_rb,1.2424e+02_rb,1.3118e+02_rb/)
      kbo(:,44,13) = (/ &
     &9.2428e+01_rb,1.0325e+02_rb,1.1298e+02_rb,1.2159e+02_rb,1.2888e+02_rb/)
      kbo(:,45,13) = (/ &
     &9.0012e+01_rb,1.0014e+02_rb,1.0975e+02_rb,1.1855e+02_rb,1.2634e+02_rb/)
      kbo(:,46,13) = (/ &
     &8.6912e+01_rb,9.6825e+01_rb,1.0618e+02_rb,1.1559e+02_rb,1.2362e+02_rb/)
      kbo(:,47,13) = (/ &
     &8.3243e+01_rb,9.2791e+01_rb,1.0216e+02_rb,1.1173e+02_rb,1.2014e+02_rb/)
      kbo(:,48,13) = (/ &
     &7.9277e+01_rb,8.8596e+01_rb,9.7998e+01_rb,1.0757e+02_rb,1.1661e+02_rb/)
      kbo(:,49,13) = (/ &
     &7.5292e+01_rb,8.4192e+01_rb,9.3715e+01_rb,1.0350e+02_rb,1.1301e+02_rb/)
      kbo(:,50,13) = (/ &
     &7.1230e+01_rb,8.0247e+01_rb,8.9698e+01_rb,9.9552e+01_rb,1.0903e+02_rb/)
      kbo(:,51,13) = (/ &
     &6.7476e+01_rb,7.6410e+01_rb,8.5492e+01_rb,9.5376e+01_rb,1.0529e+02_rb/)
      kbo(:,52,13) = (/ &
     &6.3984e+01_rb,7.2323e+01_rb,8.1379e+01_rb,9.1669e+01_rb,1.0161e+02_rb/)
      kbo(:,53,13) = (/ &
     &6.0507e+01_rb,6.8710e+01_rb,7.7291e+01_rb,8.7376e+01_rb,9.7642e+01_rb/)
      kbo(:,54,13) = (/ &
     &5.7140e+01_rb,6.5369e+01_rb,7.3636e+01_rb,8.3548e+01_rb,9.3708e+01_rb/)
      kbo(:,55,13) = (/ &
     &5.3812e+01_rb,6.2020e+01_rb,7.0040e+01_rb,7.9975e+01_rb,8.9949e+01_rb/)
      kbo(:,56,13) = (/ &
     &5.0680e+01_rb,5.8469e+01_rb,6.6837e+01_rb,7.6130e+01_rb,8.6168e+01_rb/)
      kbo(:,57,13) = (/ &
     &4.7704e+01_rb,5.5283e+01_rb,6.3212e+01_rb,7.2482e+01_rb,8.2593e+01_rb/)
      kbo(:,58,13) = (/ &
     &4.4831e+01_rb,5.2037e+01_rb,6.0082e+01_rb,6.9516e+01_rb,7.9098e+01_rb/)
      kbo(:,59,13) = (/ &
     &4.1993e+01_rb,5.0276e+01_rb,5.8426e+01_rb,6.7849e+01_rb,7.7045e+01_rb/)
      kbo(:,13,14) = (/ &
     &3.2192e+02_rb,3.1800e+02_rb,3.1299e+02_rb,3.0833e+02_rb,3.0378e+02_rb/)
      kbo(:,14,14) = (/ &
     &3.3616e+02_rb,3.3204e+02_rb,3.2756e+02_rb,3.2328e+02_rb,3.1928e+02_rb/)
      kbo(:,15,14) = (/ &
     &3.4851e+02_rb,3.4454e+02_rb,3.4128e+02_rb,3.3796e+02_rb,3.3453e+02_rb/)
      kbo(:,16,14) = (/ &
     &3.5910e+02_rb,3.5703e+02_rb,3.5472e+02_rb,3.5212e+02_rb,3.4919e+02_rb/)
      kbo(:,17,14) = (/ &
     &3.6944e+02_rb,3.6874e+02_rb,3.6739e+02_rb,3.6541e+02_rb,3.6285e+02_rb/)
      kbo(:,18,14) = (/ &
     &3.7860e+02_rb,3.7918e+02_rb,3.7877e+02_rb,3.7733e+02_rb,3.7491e+02_rb/)
      kbo(:,19,14) = (/ &
     &3.8633e+02_rb,3.8813e+02_rb,3.8858e+02_rb,3.8768e+02_rb,3.8541e+02_rb/)
      kbo(:,20,14) = (/ &
     &3.9291e+02_rb,3.9591e+02_rb,3.9708e+02_rb,3.9648e+02_rb,3.9435e+02_rb/)
      kbo(:,21,14) = (/ &
     &3.9871e+02_rb,4.0263e+02_rb,4.0428e+02_rb,4.0401e+02_rb,4.0199e+02_rb/)
      kbo(:,22,14) = (/ &
     &4.0411e+02_rb,4.0861e+02_rb,4.1054e+02_rb,4.1035e+02_rb,4.0818e+02_rb/)
      kbo(:,23,14) = (/ &
     &4.0907e+02_rb,4.1390e+02_rb,4.1591e+02_rb,4.1564e+02_rb,4.1319e+02_rb/)
      kbo(:,24,14) = (/ &
     &4.1361e+02_rb,4.1846e+02_rb,4.2039e+02_rb,4.1994e+02_rb,4.1722e+02_rb/)
      kbo(:,25,14) = (/ &
     &4.1735e+02_rb,4.2250e+02_rb,4.2427e+02_rb,4.2345e+02_rb,4.2038e+02_rb/)
      kbo(:,26,14) = (/ &
     &4.2023e+02_rb,4.2561e+02_rb,4.2741e+02_rb,4.2618e+02_rb,4.2275e+02_rb/)
      kbo(:,27,14) = (/ &
     &4.2191e+02_rb,4.2794e+02_rb,4.2966e+02_rb,4.2833e+02_rb,4.2447e+02_rb/)
      kbo(:,28,14) = (/ &
     &4.2376e+02_rb,4.3047e+02_rb,4.3157e+02_rb,4.2984e+02_rb,4.2560e+02_rb/)
      kbo(:,29,14) = (/ &
     &4.2494e+02_rb,4.3173e+02_rb,4.3312e+02_rb,4.3087e+02_rb,4.2623e+02_rb/)
      kbo(:,30,14) = (/ &
     &4.2606e+02_rb,4.3133e+02_rb,4.3417e+02_rb,4.3157e+02_rb,4.2642e+02_rb/)
      kbo(:,31,14) = (/ &
     &4.2650e+02_rb,4.3107e+02_rb,4.3305e+02_rb,4.3172e+02_rb,4.2612e+02_rb/)
      kbo(:,32,14) = (/ &
     &4.2471e+02_rb,4.2957e+02_rb,4.3084e+02_rb,4.2984e+02_rb,4.2547e+02_rb/)
      kbo(:,33,14) = (/ &
     &4.2258e+02_rb,4.2628e+02_rb,4.2740e+02_rb,4.2572e+02_rb,4.2270e+02_rb/)
      kbo(:,34,14) = (/ &
     &4.1904e+02_rb,4.2166e+02_rb,4.2281e+02_rb,4.2119e+02_rb,4.1745e+02_rb/)
      kbo(:,35,14) = (/ &
     &4.1547e+02_rb,4.1665e+02_rb,4.1587e+02_rb,4.1430e+02_rb,4.1140e+02_rb/)
      kbo(:,36,14) = (/ &
     &4.1132e+02_rb,4.1218e+02_rb,4.1072e+02_rb,4.0729e+02_rb,4.0326e+02_rb/)
      kbo(:,37,14) = (/ &
     &4.1025e+02_rb,4.1006e+02_rb,4.0857e+02_rb,4.0548e+02_rb,4.0090e+02_rb/)
      kbo(:,38,14) = (/ &
     &4.0864e+02_rb,4.0868e+02_rb,4.0686e+02_rb,4.0372e+02_rb,3.9921e+02_rb/)
      kbo(:,39,14) = (/ &
     &4.0644e+02_rb,4.0717e+02_rb,4.0510e+02_rb,4.0183e+02_rb,3.9677e+02_rb/)
      kbo(:,40,14) = (/ &
     &4.0674e+02_rb,4.0768e+02_rb,4.0593e+02_rb,4.0263e+02_rb,3.9822e+02_rb/)
      kbo(:,41,14) = (/ &
     &4.0619e+02_rb,4.0758e+02_rb,4.0668e+02_rb,4.0411e+02_rb,4.0014e+02_rb/)
      kbo(:,42,14) = (/ &
     &4.0516e+02_rb,4.0769e+02_rb,4.0744e+02_rb,4.0512e+02_rb,4.0147e+02_rb/)
      kbo(:,43,14) = (/ &
     &4.0427e+02_rb,4.0784e+02_rb,4.0882e+02_rb,4.0743e+02_rb,4.0425e+02_rb/)
      kbo(:,44,14) = (/ &
     &4.0222e+02_rb,4.0791e+02_rb,4.0985e+02_rb,4.0932e+02_rb,4.0671e+02_rb/)
      kbo(:,45,14) = (/ &
     &4.0049e+02_rb,4.0766e+02_rb,4.1090e+02_rb,4.1120e+02_rb,4.0941e+02_rb/)
      kbo(:,46,14) = (/ &
     &3.9818e+02_rb,4.0642e+02_rb,4.1148e+02_rb,4.1248e+02_rb,4.1163e+02_rb/)
      kbo(:,47,14) = (/ &
     &3.9380e+02_rb,4.0435e+02_rb,4.1138e+02_rb,4.1374e+02_rb,4.1407e+02_rb/)
      kbo(:,48,14) = (/ &
     &3.8928e+02_rb,4.0188e+02_rb,4.1084e+02_rb,4.1473e+02_rb,4.1611e+02_rb/)
      kbo(:,49,14) = (/ &
     &3.8359e+02_rb,3.9878e+02_rb,4.0957e+02_rb,4.1507e+02_rb,4.1710e+02_rb/)
      kbo(:,50,14) = (/ &
     &3.7809e+02_rb,3.9485e+02_rb,4.0769e+02_rb,4.1481e+02_rb,4.1835e+02_rb/)
      kbo(:,51,14) = (/ &
     &3.7168e+02_rb,3.9024e+02_rb,4.0538e+02_rb,4.1442e+02_rb,4.1907e+02_rb/)
      kbo(:,52,14) = (/ &
     &3.6416e+02_rb,3.8544e+02_rb,4.0242e+02_rb,4.1266e+02_rb,4.1894e+02_rb/)
      kbo(:,53,14) = (/ &
     &3.5526e+02_rb,3.7935e+02_rb,3.9858e+02_rb,4.1114e+02_rb,4.1870e+02_rb/)
      kbo(:,54,14) = (/ &
     &3.4652e+02_rb,3.7311e+02_rb,3.9451e+02_rb,4.0902e+02_rb,4.1841e+02_rb/)
      kbo(:,55,14) = (/ &
     &3.3749e+02_rb,3.6678e+02_rb,3.9033e+02_rb,4.0637e+02_rb,4.1776e+02_rb/)
      kbo(:,56,14) = (/ &
     &3.2816e+02_rb,3.6010e+02_rb,3.8512e+02_rb,4.0357e+02_rb,4.1651e+02_rb/)
      kbo(:,57,14) = (/ &
     &3.1792e+02_rb,3.5199e+02_rb,3.7987e+02_rb,4.0009e+02_rb,4.1450e+02_rb/)
      kbo(:,58,14) = (/ &
     &3.0792e+02_rb,3.4419e+02_rb,3.7396e+02_rb,3.9574e+02_rb,4.1235e+02_rb/)
      kbo(:,59,14) = (/ &
     &3.0450e+02_rb,3.4137e+02_rb,3.7215e+02_rb,3.9490e+02_rb,4.1294e+02_rb/)
      kbo(:,13,15) = (/ &
     &5.8728e+02_rb,5.6960e+02_rb,5.5469e+02_rb,5.4056e+02_rb,5.2654e+02_rb/)
      kbo(:,14,15) = (/ &
     &6.4066e+02_rb,6.2243e+02_rb,6.0593e+02_rb,5.8946e+02_rb,5.7294e+02_rb/)
      kbo(:,15,15) = (/ &
     &6.9682e+02_rb,6.7763e+02_rb,6.5849e+02_rb,6.3897e+02_rb,6.1931e+02_rb/)
      kbo(:,16,15) = (/ &
     &7.5471e+02_rb,7.3292e+02_rb,7.1026e+02_rb,6.8716e+02_rb,6.6402e+02_rb/)
      kbo(:,17,15) = (/ &
     &8.1231e+02_rb,7.8651e+02_rb,7.5977e+02_rb,7.3284e+02_rb,7.0637e+02_rb/)
      kbo(:,18,15) = (/ &
     &8.6774e+02_rb,8.3725e+02_rb,8.0641e+02_rb,7.7597e+02_rb,7.4653e+02_rb/)
      kbo(:,19,15) = (/ &
     &9.1989e+02_rb,8.8492e+02_rb,8.5007e+02_rb,8.1594e+02_rb,7.8315e+02_rb/)
      kbo(:,20,15) = (/ &
     &9.6712e+02_rb,9.2752e+02_rb,8.8885e+02_rb,8.5117e+02_rb,8.1509e+02_rb/)
      kbo(:,21,15) = (/ &
     &1.0088e+03_rb,9.6497e+02_rb,9.2264e+02_rb,8.8175e+02_rb,8.4261e+02_rb/)
      kbo(:,22,15) = (/ &
     &1.0418e+03_rb,9.9399e+02_rb,9.4844e+02_rb,9.0461e+02_rb,8.6306e+02_rb/)
      kbo(:,23,15) = (/ &
     &1.0679e+03_rb,1.0170e+03_rb,9.6858e+02_rb,9.2246e+02_rb,8.7882e+02_rb/)
      kbo(:,24,15) = (/ &
     &1.0878e+03_rb,1.0343e+03_rb,9.8346e+02_rb,9.3530e+02_rb,8.9008e+02_rb/)
      kbo(:,25,15) = (/ &
     &1.1017e+03_rb,1.0460e+03_rb,9.9332e+02_rb,9.4368e+02_rb,8.9722e+02_rb/)
      kbo(:,26,15) = (/ &
     &1.1097e+03_rb,1.0523e+03_rb,9.9837e+02_rb,9.4761e+02_rb,9.0024e+02_rb/)
      kbo(:,27,15) = (/ &
     &1.1134e+03_rb,1.0547e+03_rb,9.9983e+02_rb,9.4851e+02_rb,9.0041e+02_rb/)
      kbo(:,28,15) = (/ &
     &1.1133e+03_rb,1.0539e+03_rb,9.9848e+02_rb,9.4685e+02_rb,8.9828e+02_rb/)
      kbo(:,29,15) = (/ &
     &1.1100e+03_rb,1.0502e+03_rb,9.9454e+02_rb,9.4272e+02_rb,8.9398e+02_rb/)
      kbo(:,30,15) = (/ &
     &1.1040e+03_rb,1.0443e+03_rb,9.8865e+02_rb,9.3687e+02_rb,8.8821e+02_rb/)
      kbo(:,31,15) = (/ &
     &1.0960e+03_rb,1.0364e+03_rb,9.8114e+02_rb,9.2948e+02_rb,8.8105e+02_rb/)
      kbo(:,32,15) = (/ &
     &1.0862e+03_rb,1.0270e+03_rb,9.7213e+02_rb,9.2091e+02_rb,8.7298e+02_rb/)
      kbo(:,33,15) = (/ &
     &1.0752e+03_rb,1.0167e+03_rb,9.6230e+02_rb,9.1136e+02_rb,8.6427e+02_rb/)
      kbo(:,34,15) = (/ &
     &1.0653e+03_rb,1.0075e+03_rb,9.5353e+02_rb,9.0314e+02_rb,8.5654e+02_rb/)
      kbo(:,35,15) = (/ &
     &1.0607e+03_rb,1.0031e+03_rb,9.4921e+02_rb,8.9900e+02_rb,8.5273e+02_rb/)
      kbo(:,36,15) = (/ &
     &1.0620e+03_rb,1.0042e+03_rb,9.5018e+02_rb,8.9983e+02_rb,8.5343e+02_rb/)
      kbo(:,37,15) = (/ &
     &1.0721e+03_rb,1.0134e+03_rb,9.5885e+02_rb,9.0773e+02_rb,8.6067e+02_rb/)
      kbo(:,38,15) = (/ &
     &1.0824e+03_rb,1.0229e+03_rb,9.6769e+02_rb,9.1592e+02_rb,8.6811e+02_rb/)
      kbo(:,39,15) = (/ &
     &1.0926e+03_rb,1.0322e+03_rb,9.7633e+02_rb,9.2388e+02_rb,8.7544e+02_rb/)
      kbo(:,40,15) = (/ &
     &1.1061e+03_rb,1.0474e+03_rb,9.9047e+02_rb,9.3721e+02_rb,8.8756e+02_rb/)
      kbo(:,41,15) = (/ &
     &1.1226e+03_rb,1.0637e+03_rb,1.0055e+03_rb,9.5118e+02_rb,9.0046e+02_rb/)
      kbo(:,42,15) = (/ &
     &1.1394e+03_rb,1.0801e+03_rb,1.0206e+03_rb,9.6537e+02_rb,9.1362e+02_rb/)
      kbo(:,43,15) = (/ &
     &1.1611e+03_rb,1.1003e+03_rb,1.0393e+03_rb,9.8274e+02_rb,9.2982e+02_rb/)
      kbo(:,44,15) = (/ &
     &1.1862e+03_rb,1.1226e+03_rb,1.0598e+03_rb,1.0018e+03_rb,9.4776e+02_rb/)
      kbo(:,45,15) = (/ &
     &1.2106e+03_rb,1.1455e+03_rb,1.0808e+03_rb,1.0212e+03_rb,9.6597e+02_rb/)
      kbo(:,46,15) = (/ &
     &1.2373e+03_rb,1.1703e+03_rb,1.1039e+03_rb,1.0424e+03_rb,9.8575e+02_rb/)
      kbo(:,47,15) = (/ &
     &1.2689e+03_rb,1.1988e+03_rb,1.1303e+03_rb,1.0668e+03_rb,1.0082e+03_rb/)
      kbo(:,48,15) = (/ &
     &1.2999e+03_rb,1.2281e+03_rb,1.1575e+03_rb,1.0920e+03_rb,1.0314e+03_rb/)
      kbo(:,49,15) = (/ &
     &1.3314e+03_rb,1.2580e+03_rb,1.1855e+03_rb,1.1179e+03_rb,1.0553e+03_rb/)
      kbo(:,50,15) = (/ &
     &1.3612e+03_rb,1.2867e+03_rb,1.2124e+03_rb,1.1430e+03_rb,1.0785e+03_rb/)
      kbo(:,51,15) = (/ &
     &1.3905e+03_rb,1.3151e+03_rb,1.2392e+03_rb,1.1679e+03_rb,1.1016e+03_rb/)
      kbo(:,52,15) = (/ &
     &1.4202e+03_rb,1.3438e+03_rb,1.2667e+03_rb,1.1935e+03_rb,1.1254e+03_rb/)
      kbo(:,53,15) = (/ &
     &1.4513e+03_rb,1.3727e+03_rb,1.2946e+03_rb,1.2198e+03_rb,1.1498e+03_rb/)
      kbo(:,54,15) = (/ &
     &1.4800e+03_rb,1.3995e+03_rb,1.3206e+03_rb,1.2445e+03_rb,1.1728e+03_rb/)
      kbo(:,55,15) = (/ &
     &1.5077e+03_rb,1.4256e+03_rb,1.3457e+03_rb,1.2685e+03_rb,1.1953e+03_rb/)
      kbo(:,56,15) = (/ &
     &1.5346e+03_rb,1.4518e+03_rb,1.3711e+03_rb,1.2930e+03_rb,1.2182e+03_rb/)
      kbo(:,57,15) = (/ &
     &1.5614e+03_rb,1.4781e+03_rb,1.3967e+03_rb,1.3177e+03_rb,1.2417e+03_rb/)
      kbo(:,58,15) = (/ &
     &1.5868e+03_rb,1.5034e+03_rb,1.4212e+03_rb,1.3415e+03_rb,1.2645e+03_rb/)
      kbo(:,59,15) = (/ &
     &1.5970e+03_rb,1.5135e+03_rb,1.4312e+03_rb,1.3511e+03_rb,1.2737e+03_rb/)
      kbo(:,13,16) = (/ &
     &9.8102e+02_rb,9.4348e+02_rb,9.0592e+02_rb,8.7102e+02_rb,8.4003e+02_rb/)
      kbo(:,14,16) = (/ &
     &1.1017e+03_rb,1.0558e+03_rb,1.0119e+03_rb,9.7251e+02_rb,9.3606e+02_rb/)
      kbo(:,15,16) = (/ &
     &1.2294e+03_rb,1.1733e+03_rb,1.1221e+03_rb,1.0765e+03_rb,1.0331e+03_rb/)
      kbo(:,16,16) = (/ &
     &1.3605e+03_rb,1.2927e+03_rb,1.2335e+03_rb,1.1795e+03_rb,1.1282e+03_rb/)
      kbo(:,17,16) = (/ &
     &1.4907e+03_rb,1.4118e+03_rb,1.3426e+03_rb,1.2786e+03_rb,1.2181e+03_rb/)
      kbo(:,18,16) = (/ &
     &1.6164e+03_rb,1.5261e+03_rb,1.4460e+03_rb,1.3720e+03_rb,1.3020e+03_rb/)
      kbo(:,19,16) = (/ &
     &1.7353e+03_rb,1.6329e+03_rb,1.5412e+03_rb,1.4568e+03_rb,1.3777e+03_rb/)
      kbo(:,20,16) = (/ &
     &1.8432e+03_rb,1.7287e+03_rb,1.6256e+03_rb,1.5311e+03_rb,1.4432e+03_rb/)
      kbo(:,21,16) = (/ &
     &1.9386e+03_rb,1.8123e+03_rb,1.6986e+03_rb,1.5948e+03_rb,1.4993e+03_rb/)
      kbo(:,22,16) = (/ &
     &2.0105e+03_rb,1.8749e+03_rb,1.7520e+03_rb,1.6411e+03_rb,1.5395e+03_rb/)
      kbo(:,23,16) = (/ &
     &2.0682e+03_rb,1.9239e+03_rb,1.7938e+03_rb,1.6758e+03_rb,1.5698e+03_rb/)
      kbo(:,24,16) = (/ &
     &2.1097e+03_rb,1.9590e+03_rb,1.8231e+03_rb,1.7013e+03_rb,1.5912e+03_rb/)
      kbo(:,25,16) = (/ &
     &2.1366e+03_rb,1.9806e+03_rb,1.8412e+03_rb,1.7160e+03_rb,1.6035e+03_rb/)
      kbo(:,26,16) = (/ &
     &2.1483e+03_rb,1.9894e+03_rb,1.8477e+03_rb,1.7210e+03_rb,1.6077e+03_rb/)
      kbo(:,27,16) = (/ &
     &2.1499e+03_rb,1.9893e+03_rb,1.8466e+03_rb,1.7191e+03_rb,1.6062e+03_rb/)
      kbo(:,28,16) = (/ &
     &2.1427e+03_rb,1.9818e+03_rb,1.8393e+03_rb,1.7119e+03_rb,1.6006e+03_rb/)
      kbo(:,29,16) = (/ &
     &2.1277e+03_rb,1.9675e+03_rb,1.8260e+03_rb,1.6998e+03_rb,1.5909e+03_rb/)
      kbo(:,30,16) = (/ &
     &2.1075e+03_rb,1.9489e+03_rb,1.8089e+03_rb,1.6842e+03_rb,1.5788e+03_rb/)
      kbo(:,31,16) = (/ &
     &2.0822e+03_rb,1.9262e+03_rb,1.7881e+03_rb,1.6663e+03_rb,1.5639e+03_rb/)
      kbo(:,32,16) = (/ &
     &2.0533e+03_rb,1.9005e+03_rb,1.7649e+03_rb,1.6467e+03_rb,1.5469e+03_rb/)
      kbo(:,33,16) = (/ &
     &2.0222e+03_rb,1.8726e+03_rb,1.7400e+03_rb,1.6262e+03_rb,1.5282e+03_rb/)
      kbo(:,34,16) = (/ &
     &1.9951e+03_rb,1.8486e+03_rb,1.7187e+03_rb,1.6087e+03_rb,1.5123e+03_rb/)
      kbo(:,35,16) = (/ &
     &1.9819e+03_rb,1.8366e+03_rb,1.7081e+03_rb,1.6001e+03_rb,1.5043e+03_rb/)
      kbo(:,36,16) = (/ &
     &1.9843e+03_rb,1.8387e+03_rb,1.7099e+03_rb,1.6017e+03_rb,1.5056e+03_rb/)
      kbo(:,37,16) = (/ &
     &2.0096e+03_rb,1.8607e+03_rb,1.7293e+03_rb,1.6180e+03_rb,1.5204e+03_rb/)
      kbo(:,38,16) = (/ &
     &2.0360e+03_rb,1.8842e+03_rb,1.7495e+03_rb,1.6347e+03_rb,1.5358e+03_rb/)
      kbo(:,39,16) = (/ &
     &2.0619e+03_rb,1.9070e+03_rb,1.7697e+03_rb,1.6516e+03_rb,1.5509e+03_rb/)
      kbo(:,40,16) = (/ &
     &2.1060e+03_rb,1.9450e+03_rb,1.8033e+03_rb,1.6798e+03_rb,1.5761e+03_rb/)
      kbo(:,41,16) = (/ &
     &2.1534e+03_rb,1.9860e+03_rb,1.8395e+03_rb,1.7107e+03_rb,1.6027e+03_rb/)
      kbo(:,42,16) = (/ &
     &2.2020e+03_rb,2.0285e+03_rb,1.8771e+03_rb,1.7432e+03_rb,1.6297e+03_rb/)
      kbo(:,43,16) = (/ &
     &2.2623e+03_rb,2.0820e+03_rb,1.9234e+03_rb,1.7841e+03_rb,1.6639e+03_rb/)
      kbo(:,44,16) = (/ &
     &2.3303e+03_rb,2.1416e+03_rb,1.9758e+03_rb,1.8302e+03_rb,1.7027e+03_rb/)
      kbo(:,45,16) = (/ &
     &2.4015e+03_rb,2.2036e+03_rb,2.0305e+03_rb,1.8787e+03_rb,1.7442e+03_rb/)
      kbo(:,46,16) = (/ &
     &2.4806e+03_rb,2.2729e+03_rb,2.0910e+03_rb,1.9319e+03_rb,1.7910e+03_rb/)
      kbo(:,47,16) = (/ &
     &2.5738e+03_rb,2.3537e+03_rb,2.1621e+03_rb,1.9939e+03_rb,1.8463e+03_rb/)
      kbo(:,48,16) = (/ &
     &2.6744e+03_rb,2.4391e+03_rb,2.2368e+03_rb,2.0594e+03_rb,1.9042e+03_rb/)
      kbo(:,49,16) = (/ &
     &2.7827e+03_rb,2.5294e+03_rb,2.3154e+03_rb,2.1283e+03_rb,1.9646e+03_rb/)
      kbo(:,50,16) = (/ &
     &2.8923e+03_rb,2.6199e+03_rb,2.3931e+03_rb,2.1963e+03_rb,2.0240e+03_rb/)
      kbo(:,51,16) = (/ &
     &3.0068e+03_rb,2.7139e+03_rb,2.4725e+03_rb,2.2657e+03_rb,2.0848e+03_rb/)
      kbo(:,52,16) = (/ &
     &3.1290e+03_rb,2.8143e+03_rb,2.5557e+03_rb,2.3381e+03_rb,2.1483e+03_rb/)
      kbo(:,53,16) = (/ &
     &3.2592e+03_rb,2.9226e+03_rb,2.6451e+03_rb,2.4144e+03_rb,2.2150e+03_rb/)
      kbo(:,54,16) = (/ &
     &3.3865e+03_rb,3.0292e+03_rb,2.7322e+03_rb,2.4877e+03_rb,2.2788e+03_rb/)
      kbo(:,55,16) = (/ &
     &3.5154e+03_rb,3.1374e+03_rb,2.8210e+03_rb,2.5614e+03_rb,2.3429e+03_rb/)
      kbo(:,56,16) = (/ &
     &3.6521e+03_rb,3.2511e+03_rb,2.9159e+03_rb,2.6395e+03_rb,2.4095e+03_rb/)
      kbo(:,57,16) = (/ &
     &3.7974e+03_rb,3.3719e+03_rb,3.0171e+03_rb,2.7223e+03_rb,2.4795e+03_rb/)
      kbo(:,58,16) = (/ &
     &3.9425e+03_rb,3.4927e+03_rb,3.1186e+03_rb,2.8056e+03_rb,2.5484e+03_rb/)
      kbo(:,59,16) = (/ &
     &4.0032e+03_rb,3.5431e+03_rb,3.1604e+03_rb,2.8404e+03_rb,2.5772e+03_rb/)

!     The array FORREFO contains the coefficient of the water vapor
!     foreign-continuum (including the energy term).  The first 
!     index refers to reference temperature (296,260,224,260) and 
!     pressure (970,475,219,3 mbar) levels.  The second index 
!     runs over the g-channel (1 to 16).

      forrefo(1,:) = (/ &
     &5.1629e-06_rb,7.7578e-06_rb,1.9043e-05_rb,1.4802e-04_rb,2.2980e-04_rb,2.8057e-04_rb, &
     &3.2824e-04_rb,3.4913e-04_rb,3.6515e-04_rb,3.8271e-04_rb,3.7499e-04_rb,3.6966e-04_rb, &
     &3.7424e-04_rb,3.8884e-04_rb,3.7117e-04_rb,4.3710e-04_rb/)
      forrefo(2,:) = (/ &
     &5.0804e-06_rb,1.3466e-05_rb,7.2606e-05_rb,1.6940e-04_rb,2.1022e-04_rb,2.5900e-04_rb, &
     &2.9106e-04_rb,3.2261e-04_rb,3.2066e-04_rb,3.5421e-04_rb,3.7128e-04_rb,3.8144e-04_rb, &
     &3.7854e-04_rb,3.8347e-04_rb,3.8921e-04_rb,3.7339e-04_rb/)
      forrefo(3,:) = (/ &
     &5.4797e-05_rb,1.0026e-04_rb,1.2422e-04_rb,1.6386e-04_rb,1.8378e-04_rb,1.9616e-04_rb, &
     &2.0711e-04_rb,2.2492e-04_rb,2.5240e-04_rb,2.6187e-04_rb,2.6058e-04_rb,2.4892e-04_rb, &
     &2.6526e-04_rb,3.2105e-04_rb,3.6903e-04_rb,3.7213e-04_rb/)
      forrefo(4,:) = (/ &
     &4.2782e-05_rb,1.4775e-04_rb,1.4588e-04_rb,1.6964e-04_rb,1.6667e-04_rb,1.7192e-04_rb, &
     &1.9057e-04_rb,2.0180e-04_rb,2.1177e-04_rb,2.2326e-04_rb,2.3801e-04_rb,2.9308e-04_rb, &
     &3.1130e-04_rb,3.1829e-04_rb,3.5035e-04_rb,3.7782e-04_rb/)

!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
     & 1.27793e-03_rb, 1.05944e-03_rb, 8.78300e-04_rb, 7.28133e-04_rb, 6.03641e-04_rb, &
     & 5.00434e-04_rb, 4.14873e-04_rb, 3.43940e-04_rb, 2.85135e-04_rb, 2.36384e-04_rb/)
      selfrefo(:, 2) = (/ &
     & 1.42785e-03_rb, 1.17602e-03_rb, 9.68600e-04_rb, 7.97765e-04_rb, 6.57060e-04_rb, &
     & 5.41172e-04_rb, 4.45724e-04_rb, 3.67110e-04_rb, 3.02361e-04_rb, 2.49033e-04_rb/)
      selfrefo(:, 3) = (/ &
     & 2.94095e-03_rb, 2.27102e-03_rb, 1.75370e-03_rb, 1.35422e-03_rb, 1.04574e-03_rb, &
     & 8.07525e-04_rb, 6.23577e-04_rb, 4.81530e-04_rb, 3.71841e-04_rb, 2.87138e-04_rb/)
      selfrefo(:, 4) = (/ &
     & 3.94894e-03_rb, 3.48184e-03_rb, 3.07000e-03_rb, 2.70687e-03_rb, 2.38669e-03_rb, &
     & 2.10439e-03_rb, 1.85547e-03_rb, 1.63600e-03_rb, 1.44249e-03_rb, 1.27187e-03_rb/)
      selfrefo(:, 5) = (/ &
     & 4.19971e-03_rb, 3.86333e-03_rb, 3.55390e-03_rb, 3.26925e-03_rb, 3.00740e-03_rb, &
     & 2.76652e-03_rb, 2.54494e-03_rb, 2.34110e-03_rb, 2.15359e-03_rb, 1.98110e-03_rb/)
      selfrefo(:, 6) = (/ &
     & 4.95922e-03_rb, 4.57134e-03_rb, 4.21380e-03_rb, 3.88422e-03_rb, 3.58042e-03_rb, &
     & 3.30038e-03_rb, 3.04225e-03_rb, 2.80430e-03_rb, 2.58496e-03_rb, 2.38278e-03_rb/)
      selfrefo(:, 7) = (/ &
     & 5.27379e-03_rb, 4.91005e-03_rb, 4.57140e-03_rb, 4.25611e-03_rb, 3.96256e-03_rb, &
     & 3.68925e-03_rb, 3.43480e-03_rb, 3.19790e-03_rb, 2.97734e-03_rb, 2.77199e-03_rb/)
      selfrefo(:, 8) = (/ &
     & 5.75341e-03_rb, 5.31533e-03_rb, 4.91060e-03_rb, 4.53669e-03_rb, 4.19126e-03_rb, &
     & 3.87212e-03_rb, 3.57729e-03_rb, 3.30490e-03_rb, 3.05325e-03_rb, 2.82077e-03_rb/)
      selfrefo(:, 9) = (/ &
     & 5.49849e-03_rb, 5.14295e-03_rb, 4.81040e-03_rb, 4.49935e-03_rb, 4.20842e-03_rb, &
     & 3.93629e-03_rb, 3.68177e-03_rb, 3.44370e-03_rb, 3.22102e-03_rb, 3.01275e-03_rb/)
      selfrefo(:,10) = (/ &
     & 6.04962e-03_rb, 5.60945e-03_rb, 5.20130e-03_rb, 4.82285e-03_rb, 4.47194e-03_rb, &
     & 4.14656e-03_rb, 3.84485e-03_rb, 3.56510e-03_rb, 3.30570e-03_rb, 3.06518e-03_rb/)
      selfrefo(:,11) = (/ &
     & 6.40108e-03_rb, 5.87551e-03_rb, 5.39310e-03_rb, 4.95029e-03_rb, 4.54385e-03_rb, &
     & 4.17077e-03_rb, 3.82833e-03_rb, 3.51400e-03_rb, 3.22548e-03_rb, 2.96065e-03_rb/)
      selfrefo(:,12) = (/ &
     & 6.77938e-03_rb, 6.15713e-03_rb, 5.59200e-03_rb, 5.07874e-03_rb, 4.61259e-03_rb, &
     & 4.18922e-03_rb, 3.80472e-03_rb, 3.45550e-03_rb, 3.13834e-03_rb, 2.85029e-03_rb/)
      selfrefo(:,13) = (/ &
     & 6.90020e-03_rb, 6.26766e-03_rb, 5.69310e-03_rb, 5.17121e-03_rb, 4.69717e-03_rb, &
     & 4.26658e-03_rb, 3.87546e-03_rb, 3.52020e-03_rb, 3.19750e-03_rb, 2.90439e-03_rb/)
      selfrefo(:,14) = (/ &
     & 6.92759e-03_rb, 6.32882e-03_rb, 5.78180e-03_rb, 5.28206e-03_rb, 4.82552e-03_rb, &
     & 4.40843e-03_rb, 4.02740e-03_rb, 3.67930e-03_rb, 3.36129e-03_rb, 3.07076e-03_rb/)
      selfrefo(:,15) = (/ &
     & 7.54539e-03_rb, 6.81161e-03_rb, 6.14920e-03_rb, 5.55120e-03_rb, 5.01136e-03_rb, &
     & 4.52402e-03_rb, 4.08407e-03_rb, 3.68690e-03_rb, 3.32836e-03_rb, 3.00468e-03_rb/)
      selfrefo(:,16) = (/ &
     & 7.62039e-03_rb, 7.10834e-03_rb, 6.63070e-03_rb, 6.18515e-03_rb, 5.76955e-03_rb, &
     & 5.38186e-03_rb, 5.02023e-03_rb, 4.68290e-03_rb, 4.36823e-03_rb, 4.07471e-03_rb/)

      end subroutine lw_kgb16
