!     path:      $Source$
!     author:    $Author: miacono $
!     revision:  $Revision: 30822 $
!     created:   $Date: 2016-12-29 15:53:24 -0500 (Thu, 29 Dec 2016) $

!----------------------------------------------------------------------------
! Copyright (c) 2002-2016, Atmospheric & Environmental Research, Inc. (AER)
! All rights reserved.
!
! Redistribution and use in source and binary forms, with or without
! modification, are permitted provided that the following conditions are met:
!  * Redistributions of source code must retain the above copyright
!    notice, this list of conditions and the following disclaimer.
!  * Redistributions in binary form must reproduce the above copyright
!    notice, this list of conditions and the following disclaimer in the
!    documentation and/or other materials provided with the distribution.
!  * Neither the name of Atmospheric & Environmental Research, Inc., nor
!    the names of its contributors may be used to endorse or promote products
!    derived from this software without specific prior written permission.
!
! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
! AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
! IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
! ARE DISCLAIMED. IN NO EVENT SHALL ATMOSPHERIC & ENVIRONMENTAL RESEARCH, INC., 
! BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
! CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
! SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
! INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
! CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
! ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
! THE POSSIBILITY OF SUCH DAMAGE.
!                        (http://www.rtweb.aer.com/)                        
!----------------------------------------------------------------------------

! **************************************************************************
!      subroutine sw_kgbnn
! **************************************************************************
!  RRTM Shortwave Radiative Transfer Model
!  Atmospheric and Environmental Research, Inc., Cambridge, MA
!
!  Original by J.Delamere, Atmospheric & Environmental Research.
!  Reformatted for F90: JJMorcrette, ECMWF
!  Further F90 and GCM revisions:  MJIacono, AER, July 2002
!  Solar variability revisions:  MJIacono, AER, November 2015
!
!  This file contains 14 subroutines that include the 
!  absorption coefficients and other data for each of the 14 shortwave
!  spectral bands used in RRTM_SW.  Here, the data are defined for 16
!  g-points, or sub-intervals, per band.  These data are combined and
!  weighted using a mapping procedure in routine RRTMG_SW_INIT to reduce
!  the total number of g-points from 224 to 112 for use in the GCM.
! **************************************************************************
      subroutine sw_kgb16
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg16, only : kao, kbo, selfrefo, forrefo, sfluxrefo, rayl, &
                            irradnceo, facbrghto, snsptdrko
      use rrsw_vsn, only: hvrkg, hnamkg

      implicit none
      save

      hvrkg = '$Revision: 30822 $'

! Kurucz solar source function
      sfluxrefo(:) = (/ &
        &  1.92269_rb    , 1.72844_rb    , 1.64326_rb    , 1.58451_rb     &
        &, 1.44031_rb    , 1.25108_rb    , 1.02724_rb    , 0.776759_rb    &
        &, 0.534444_rb   , 5.87755e-02_rb, 4.86706e-02_rb, 3.87989e-02_rb &
        &, 2.84532e-02_rb, 1.82431e-02_rb, 6.92320e-03_rb, 9.70770e-04_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:) = (/ &
        & 1.79749E+00_rb, 1.72236E+00_rb, 1.69132E+00_rb, 1.58635E+00_rb,&
        & 1.43175E+00_rb, 1.24398E+00_rb, 1.02211E+00_rb, 7.68046E-01_rb,&
        & 5.28718E-01_rb, 5.82489E-02_rb, 4.82282E-02_rb, 3.84899E-02_rb,&
        & 2.81790E-02_rb, 1.80758E-02_rb, 6.85351E-03_rb, 9.60481E-04_rb/)
! Solar variability components: facular brightening
      facbrghto(:) = (/ &
        & 5.93894E-04_rb, 5.79626E-04_rb, 5.55095E-04_rb, 5.14844E-04_rb,&
        & 4.62038E-04_rb, 3.98231E-04_rb, 3.25666E-04_rb, 2.45209E-04_rb,&
        & 1.67128E-04_rb, 1.82317E-05_rb, 1.51310E-05_rb, 1.20142E-05_rb,&
        & 8.83688E-06_rb, 5.62696E-06_rb, 2.10190E-06_rb, 2.95852E-07_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:) = (/ &
        &-2.01457E-04_rb,-1.93017E-04_rb,-1.89996E-04_rb,-1.78192E-04_rb,&
        &-1.60629E-04_rb,-1.39388E-04_rb,-1.14371E-04_rb,-8.58375E-05_rb,&
        &-5.90794E-05_rb,-6.52079E-06_rb,-5.41520E-06_rb,-4.32071E-06_rb,&
        &-3.16491E-06_rb,-2.02502E-06_rb,-7.66601E-07_rb,-1.08837E-07_rb/)

! Rayleigh extinction coefficient at v = 2925 cm-1.
      rayl = 2.91e-10_rb

!     -----------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.15349e-04_rb,0.89162e-04_rb,0.97706e-04_rb,0.96973e-04_rb,0.90703e-04_rb, &
        & 0.80393e-04_rb,0.67242e-04_rb,0.50804e-04_rb,0.23334e-04_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.16138e-04_rb,0.89235e-04_rb,0.97696e-04_rb,0.96116e-04_rb,0.89625e-04_rb, &
        & 0.79382e-04_rb,0.66587e-04_rb,0.50583e-04_rb,0.24342e-04_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.16801e-04_rb,0.89390e-04_rb,0.97492e-04_rb,0.95146e-04_rb,0.88483e-04_rb, &
        & 0.78325e-04_rb,0.65873e-04_rb,0.50342e-04_rb,0.25245e-04_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.17584e-04_rb,0.89321e-04_rb,0.97141e-04_rb,0.93952e-04_rb,0.87141e-04_rb, &
        & 0.77126e-04_rb,0.65068e-04_rb,0.49985e-04_rb,0.26046e-04_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.18258e-04_rb,0.89204e-04_rb,0.96602e-04_rb,0.92773e-04_rb,0.85832e-04_rb, &
        & 0.76015e-04_rb,0.64318e-04_rb,0.49612e-04_rb,0.26697e-04_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.12783e-04_rb,0.77013e-04_rb,0.85056e-04_rb,0.84160e-04_rb,0.79021e-04_rb, &
        & 0.70101e-04_rb,0.58520e-04_rb,0.44007e-04_rb,0.19895e-04_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.13452e-04_rb,0.77084e-04_rb,0.84929e-04_rb,0.83386e-04_rb,0.78122e-04_rb, &
        & 0.69293e-04_rb,0.57974e-04_rb,0.43855e-04_rb,0.20713e-04_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.14066e-04_rb,0.77185e-04_rb,0.84612e-04_rb,0.82498e-04_rb,0.77177e-04_rb, &
        & 0.68443e-04_rb,0.57444e-04_rb,0.43722e-04_rb,0.21494e-04_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.14678e-04_rb,0.76996e-04_rb,0.83962e-04_rb,0.81526e-04_rb,0.76049e-04_rb, &
        & 0.67485e-04_rb,0.56822e-04_rb,0.43530e-04_rb,0.22200e-04_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.15258e-04_rb,0.76788e-04_rb,0.82821e-04_rb,0.80320e-04_rb,0.74845e-04_rb, &
        & 0.66487e-04_rb,0.56144e-04_rb,0.43262e-04_rb,0.22757e-04_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.10474e-04_rb,0.66144e-04_rb,0.73703e-04_rb,0.72701e-04_rb,0.68638e-04_rb, &
        & 0.60826e-04_rb,0.50854e-04_rb,0.37989e-04_rb,0.17743e-04_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.11024e-04_rb,0.66096e-04_rb,0.73447e-04_rb,0.72029e-04_rb,0.67858e-04_rb, &
        & 0.60163e-04_rb,0.50424e-04_rb,0.37886e-04_rb,0.18519e-04_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.11534e-04_rb,0.65974e-04_rb,0.72795e-04_rb,0.71229e-04_rb,0.67005e-04_rb, &
        & 0.59442e-04_rb,0.49950e-04_rb,0.37779e-04_rb,0.19184e-04_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.12091e-04_rb,0.65752e-04_rb,0.72003e-04_rb,0.70357e-04_rb,0.66129e-04_rb, &
        & 0.58718e-04_rb,0.49503e-04_rb,0.37690e-04_rb,0.19788e-04_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.12560e-04_rb,0.65678e-04_rb,0.71013e-04_rb,0.69266e-04_rb,0.65076e-04_rb, &
        & 0.57873e-04_rb,0.48947e-04_rb,0.37519e-04_rb,0.20290e-04_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.85236e-05_rb,0.56692e-04_rb,0.63625e-04_rb,0.62567e-04_rb,0.59412e-04_rb, &
        & 0.52601e-04_rb,0.43957e-04_rb,0.32602e-04_rb,0.16862e-04_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.89886e-05_rb,0.56512e-04_rb,0.63288e-04_rb,0.61902e-04_rb,0.58733e-04_rb, &
        & 0.52036e-04_rb,0.43629e-04_rb,0.32592e-04_rb,0.17543e-04_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.94085e-05_rb,0.56287e-04_rb,0.62588e-04_rb,0.61197e-04_rb,0.58020e-04_rb, &
        & 0.51469e-04_rb,0.43247e-04_rb,0.32567e-04_rb,0.18199e-04_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.98830e-05_rb,0.55998e-04_rb,0.61716e-04_rb,0.60485e-04_rb,0.57250e-04_rb, &
        & 0.50840e-04_rb,0.42842e-04_rb,0.32520e-04_rb,0.18771e-04_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.10278e-04_rb,0.55929e-04_rb,0.61011e-04_rb,0.59745e-04_rb,0.56462e-04_rb, &
        & 0.50254e-04_rb,0.42468e-04_rb,0.32457e-04_rb,0.19296e-04_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.68852e-05_rb,0.48282e-04_rb,0.54761e-04_rb,0.53543e-04_rb,0.51345e-04_rb, &
        & 0.45380e-04_rb,0.37794e-04_rb,0.27880e-04_rb,0.14959e-04_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.72830e-05_rb,0.48206e-04_rb,0.54379e-04_rb,0.52923e-04_rb,0.50723e-04_rb, &
        & 0.44903e-04_rb,0.37536e-04_rb,0.27938e-04_rb,0.15841e-04_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.76436e-05_rb,0.47928e-04_rb,0.53597e-04_rb,0.52299e-04_rb,0.50061e-04_rb, &
        & 0.44426e-04_rb,0.37240e-04_rb,0.27962e-04_rb,0.16760e-04_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.80055e-05_rb,0.47625e-04_rb,0.52896e-04_rb,0.51678e-04_rb,0.49302e-04_rb, &
        & 0.43902e-04_rb,0.36930e-04_rb,0.27968e-04_rb,0.17586e-04_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.83555e-05_rb,0.47486e-04_rb,0.52258e-04_rb,0.51112e-04_rb,0.48444e-04_rb, &
        & 0.43409e-04_rb,0.36652e-04_rb,0.27966e-04_rb,0.18270e-04_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.55363e-05_rb,0.40995e-04_rb,0.46991e-04_rb,0.45840e-04_rb,0.44200e-04_rb, &
        & 0.38984e-04_rb,0.32341e-04_rb,0.23743e-04_rb,0.12825e-04_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.58741e-05_rb,0.40851e-04_rb,0.46490e-04_rb,0.45277e-04_rb,0.43651e-04_rb, &
        & 0.38565e-04_rb,0.32141e-04_rb,0.23813e-04_rb,0.13671e-04_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.61821e-05_rb,0.40643e-04_rb,0.45850e-04_rb,0.44728e-04_rb,0.42924e-04_rb, &
        & 0.38164e-04_rb,0.31920e-04_rb,0.23858e-04_rb,0.14647e-04_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.64614e-05_rb,0.40329e-04_rb,0.45202e-04_rb,0.44163e-04_rb,0.42054e-04_rb, &
        & 0.37723e-04_rb,0.31679e-04_rb,0.23886e-04_rb,0.15533e-04_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.67675e-05_rb,0.40137e-04_rb,0.44578e-04_rb,0.43574e-04_rb,0.41267e-04_rb, &
        & 0.37271e-04_rb,0.31420e-04_rb,0.23883e-04_rb,0.16436e-04_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.44217e-05_rb,0.34626e-04_rb,0.39970e-04_rb,0.39080e-04_rb,0.37744e-04_rb, &
        & 0.33237e-04_rb,0.27507e-04_rb,0.20074e-04_rb,0.11007e-04_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.47104e-05_rb,0.34494e-04_rb,0.39592e-04_rb,0.38568e-04_rb,0.37283e-04_rb, &
        & 0.32875e-04_rb,0.27342e-04_rb,0.20149e-04_rb,0.11834e-04_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.49764e-05_rb,0.34300e-04_rb,0.39007e-04_rb,0.38074e-04_rb,0.36415e-04_rb, &
        & 0.32528e-04_rb,0.27158e-04_rb,0.20213e-04_rb,0.12801e-04_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.52135e-05_rb,0.34067e-04_rb,0.38449e-04_rb,0.37591e-04_rb,0.35661e-04_rb, &
        & 0.32173e-04_rb,0.26959e-04_rb,0.20265e-04_rb,0.13763e-04_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.54275e-05_rb,0.33832e-04_rb,0.37875e-04_rb,0.37057e-04_rb,0.35065e-04_rb, &
        & 0.31800e-04_rb,0.26760e-04_rb,0.20278e-04_rb,0.14677e-04_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.35317e-05_rb,0.29115e-04_rb,0.33824e-04_rb,0.33370e-04_rb,0.32237e-04_rb, &
        & 0.28331e-04_rb,0.23362e-04_rb,0.16932e-04_rb,0.96086e-05_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.37854e-05_rb,0.29084e-04_rb,0.33632e-04_rb,0.32929e-04_rb,0.31805e-04_rb, &
        & 0.28018e-04_rb,0.23236e-04_rb,0.17006e-04_rb,0.10621e-04_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.40127e-05_rb,0.28935e-04_rb,0.33190e-04_rb,0.32483e-04_rb,0.30936e-04_rb, &
        & 0.27716e-04_rb,0.23074e-04_rb,0.17087e-04_rb,0.11657e-04_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.42184e-05_rb,0.28732e-04_rb,0.32711e-04_rb,0.32042e-04_rb,0.30351e-04_rb, &
        & 0.27404e-04_rb,0.22906e-04_rb,0.17140e-04_rb,0.12504e-04_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.44040e-05_rb,0.28488e-04_rb,0.32207e-04_rb,0.31581e-04_rb,0.29852e-04_rb, &
        & 0.27086e-04_rb,0.22740e-04_rb,0.17166e-04_rb,0.13401e-04_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.28004e-05_rb,0.24421e-04_rb,0.28507e-04_rb,0.28270e-04_rb,0.27415e-04_rb, &
        & 0.24038e-04_rb,0.19759e-04_rb,0.14209e-04_rb,0.88498e-05_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.30342e-05_rb,0.24385e-04_rb,0.28391e-04_rb,0.27999e-04_rb,0.26969e-04_rb, &
        & 0.23770e-04_rb,0.19651e-04_rb,0.14288e-04_rb,0.10048e-04_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.32304e-05_rb,0.24304e-04_rb,0.28136e-04_rb,0.27602e-04_rb,0.26246e-04_rb, &
        & 0.23508e-04_rb,0.19517e-04_rb,0.14360e-04_rb,0.11188e-04_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.34049e-05_rb,0.24160e-04_rb,0.27731e-04_rb,0.27211e-04_rb,0.25756e-04_rb, &
        & 0.23246e-04_rb,0.19380e-04_rb,0.14426e-04_rb,0.12319e-04_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.35648e-05_rb,0.23940e-04_rb,0.27283e-04_rb,0.26806e-04_rb,0.25320e-04_rb, &
        & 0.22960e-04_rb,0.19234e-04_rb,0.14451e-04_rb,0.13506e-04_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.22115e-05_rb,0.20358e-04_rb,0.23898e-04_rb,0.23781e-04_rb,0.23142e-04_rb, &
        & 0.20260e-04_rb,0.16593e-04_rb,0.11876e-04_rb,0.77665e-05_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.24264e-05_rb,0.20362e-04_rb,0.23820e-04_rb,0.23569e-04_rb,0.22708e-04_rb, &
        & 0.20037e-04_rb,0.16524e-04_rb,0.11954e-04_rb,0.90216e-05_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.26002e-05_rb,0.20303e-04_rb,0.23663e-04_rb,0.23295e-04_rb,0.22132e-04_rb, &
        & 0.19806e-04_rb,0.16414e-04_rb,0.12011e-04_rb,0.10379e-04_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.27486e-05_rb,0.20198e-04_rb,0.23357e-04_rb,0.22943e-04_rb,0.21707e-04_rb, &
        & 0.19580e-04_rb,0.16298e-04_rb,0.12060e-04_rb,0.11771e-04_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.28881e-05_rb,0.20019e-04_rb,0.22982e-04_rb,0.22584e-04_rb,0.21296e-04_rb, &
        & 0.19341e-04_rb,0.16164e-04_rb,0.12088e-04_rb,0.13279e-04_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.18320e-05_rb,0.16927e-04_rb,0.19968e-04_rb,0.19856e-04_rb,0.19352e-04_rb, &
        & 0.16934e-04_rb,0.13866e-04_rb,0.99116e-05_rb,0.67134e-05_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.20049e-05_rb,0.16927e-04_rb,0.19853e-04_rb,0.19674e-04_rb,0.18853e-04_rb, &
        & 0.16739e-04_rb,0.13797e-04_rb,0.99757e-05_rb,0.78968e-05_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.21502e-05_rb,0.16866e-04_rb,0.19704e-04_rb,0.19448e-04_rb,0.18441e-04_rb, &
        & 0.16541e-04_rb,0.13705e-04_rb,0.10023e-04_rb,0.91593e-05_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.22714e-05_rb,0.16777e-04_rb,0.19465e-04_rb,0.19143e-04_rb,0.18062e-04_rb, &
        & 0.16347e-04_rb,0.13601e-04_rb,0.10057e-04_rb,0.10536e-04_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.23867e-05_rb,0.16634e-04_rb,0.19123e-04_rb,0.18829e-04_rb,0.17732e-04_rb, &
        & 0.16132e-04_rb,0.13484e-04_rb,0.10082e-04_rb,0.12018e-04_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.15169e-05_rb,0.14042e-04_rb,0.16626e-04_rb,0.16552e-04_rb,0.16148e-04_rb, &
        & 0.14121e-04_rb,0.11557e-04_rb,0.82558e-05_rb,0.56116e-05_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.16576e-05_rb,0.14051e-04_rb,0.16515e-04_rb,0.16389e-04_rb,0.15651e-04_rb, &
        & 0.13956e-04_rb,0.11498e-04_rb,0.83065e-05_rb,0.66756e-05_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.17786e-05_rb,0.13996e-04_rb,0.16382e-04_rb,0.16200e-04_rb,0.15335e-04_rb, &
        & 0.13791e-04_rb,0.11414e-04_rb,0.83396e-05_rb,0.78609e-05_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.18784e-05_rb,0.13920e-04_rb,0.16173e-04_rb,0.15944e-04_rb,0.15016e-04_rb, &
        & 0.13612e-04_rb,0.11321e-04_rb,0.83696e-05_rb,0.90657e-05_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.19730e-05_rb,0.13808e-04_rb,0.15876e-04_rb,0.15671e-04_rb,0.14704e-04_rb, &
        & 0.13425e-04_rb,0.11218e-04_rb,0.83846e-05_rb,0.10342e-04_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.12548e-05_rb,0.11622e-04_rb,0.13817e-04_rb,0.13765e-04_rb,0.13366e-04_rb, &
        & 0.11761e-04_rb,0.96214e-05_rb,0.68586e-05_rb,0.46815e-05_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.13720e-05_rb,0.11638e-04_rb,0.13707e-04_rb,0.13640e-04_rb,0.13007e-04_rb, &
        & 0.11624e-04_rb,0.95672e-05_rb,0.68978e-05_rb,0.55831e-05_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.14716e-05_rb,0.11595e-04_rb,0.13603e-04_rb,0.13476e-04_rb,0.12728e-04_rb, &
        & 0.11474e-04_rb,0.94883e-05_rb,0.69284e-05_rb,0.65694e-05_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.15544e-05_rb,0.11533e-04_rb,0.13432e-04_rb,0.13265e-04_rb,0.12450e-04_rb, &
        & 0.11320e-04_rb,0.94054e-05_rb,0.69510e-05_rb,0.75699e-05_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.16324e-05_rb,0.11450e-04_rb,0.13177e-04_rb,0.13030e-04_rb,0.12193e-04_rb, &
        & 0.11159e-04_rb,0.93157e-05_rb,0.69543e-05_rb,0.86449e-05_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.10905e-03_rb,0.19084e-03_rb,0.20069e-03_rb,0.19832e-03_rb,0.18610e-03_rb, &
        & 0.17089e-03_rb,0.14990e-03_rb,0.11759e-03_rb,0.64675e-04_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.11105e-03_rb,0.18952e-03_rb,0.19885e-03_rb,0.19823e-03_rb,0.18675e-03_rb, &
        & 0.17126e-03_rb,0.15098e-03_rb,0.11929e-03_rb,0.67071e-04_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.11276e-03_rb,0.18853e-03_rb,0.19769e-03_rb,0.19892e-03_rb,0.18804e-03_rb, &
        & 0.17228e-03_rb,0.15234e-03_rb,0.12104e-03_rb,0.69562e-04_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.11331e-03_rb,0.18716e-03_rb,0.19673e-03_rb,0.19862e-03_rb,0.18922e-03_rb, &
        & 0.17345e-03_rb,0.15364e-03_rb,0.12287e-03_rb,0.72145e-04_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.11374e-03_rb,0.18626e-03_rb,0.19656e-03_rb,0.19857e-03_rb,0.19062e-03_rb, &
        & 0.17445e-03_rb,0.15373e-03_rb,0.12483e-03_rb,0.74697e-04_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.94458e-04_rb,0.16650e-03_rb,0.17483e-03_rb,0.17491e-03_rb,0.16501e-03_rb, &
        & 0.15069e-03_rb,0.13359e-03_rb,0.10498e-03_rb,0.55352e-04_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.96121e-04_rb,0.16557e-03_rb,0.17383e-03_rb,0.17514e-03_rb,0.16597e-03_rb, &
        & 0.15082e-03_rb,0.13480e-03_rb,0.10677e-03_rb,0.57531e-04_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.97581e-04_rb,0.16494e-03_rb,0.17349e-03_rb,0.17480e-03_rb,0.16726e-03_rb, &
        & 0.15206e-03_rb,0.13520e-03_rb,0.10844e-03_rb,0.59621e-04_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.98152e-04_rb,0.16427e-03_rb,0.17323e-03_rb,0.17471e-03_rb,0.16812e-03_rb, &
        & 0.15352e-03_rb,0.13523e-03_rb,0.11008e-03_rb,0.61894e-04_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.98081e-04_rb,0.16346e-03_rb,0.17334e-03_rb,0.17481e-03_rb,0.16845e-03_rb, &
        & 0.15496e-03_rb,0.13612e-03_rb,0.11175e-03_rb,0.64159e-04_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.80988e-04_rb,0.14444e-03_rb,0.15236e-03_rb,0.15355e-03_rb,0.14515e-03_rb, &
        & 0.13201e-03_rb,0.11827e-03_rb,0.92843e-04_rb,0.47818e-04_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.82627e-04_rb,0.14399e-03_rb,0.15198e-03_rb,0.15308e-03_rb,0.14622e-03_rb, &
        & 0.13285e-03_rb,0.11829e-03_rb,0.94579e-04_rb,0.49498e-04_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.83666e-04_rb,0.14365e-03_rb,0.15183e-03_rb,0.15300e-03_rb,0.14704e-03_rb, &
        & 0.13418e-03_rb,0.11832e-03_rb,0.96259e-04_rb,0.51225e-04_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.84164e-04_rb,0.14362e-03_rb,0.15205e-03_rb,0.15285e-03_rb,0.14764e-03_rb, &
        & 0.13590e-03_rb,0.11954e-03_rb,0.97885e-04_rb,0.52990e-04_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.84014e-04_rb,0.14269e-03_rb,0.15214e-03_rb,0.15331e-03_rb,0.14865e-03_rb, &
        & 0.13747e-03_rb,0.12089e-03_rb,0.99389e-04_rb,0.54936e-04_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.69055e-04_rb,0.12498e-03_rb,0.13260e-03_rb,0.13427e-03_rb,0.12748e-03_rb, &
        & 0.11593e-03_rb,0.10318e-03_rb,0.81535e-04_rb,0.43344e-04_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.70374e-04_rb,0.12479e-03_rb,0.13217e-03_rb,0.13387e-03_rb,0.12823e-03_rb, &
        & 0.11702e-03_rb,0.10316e-03_rb,0.83059e-04_rb,0.44577e-04_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.71321e-04_rb,0.12483e-03_rb,0.13205e-03_rb,0.13371e-03_rb,0.12866e-03_rb, &
        & 0.11820e-03_rb,0.10422e-03_rb,0.84648e-04_rb,0.45880e-04_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.71712e-04_rb,0.12475e-03_rb,0.13249e-03_rb,0.13379e-03_rb,0.12934e-03_rb, &
        & 0.11960e-03_rb,0.10551e-03_rb,0.86238e-04_rb,0.47243e-04_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.71917e-04_rb,0.12433e-03_rb,0.13275e-03_rb,0.13434e-03_rb,0.13049e-03_rb, &
        & 0.12131e-03_rb,0.10676e-03_rb,0.87830e-04_rb,0.48641e-04_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.58527e-04_rb,0.10840e-03_rb,0.11507e-03_rb,0.11730e-03_rb,0.11099e-03_rb, &
        & 0.10118e-03_rb,0.89566e-04_rb,0.71122e-04_rb,0.43658e-04_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.59798e-04_rb,0.10799e-03_rb,0.11447e-03_rb,0.11692e-03_rb,0.11189e-03_rb, &
        & 0.10224e-03_rb,0.90255e-04_rb,0.72476e-04_rb,0.44361e-04_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.60601e-04_rb,0.10799e-03_rb,0.11464e-03_rb,0.11677e-03_rb,0.11225e-03_rb, &
        & 0.10358e-03_rb,0.91122e-04_rb,0.73922e-04_rb,0.45004e-04_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.60971e-04_rb,0.10798e-03_rb,0.11486e-03_rb,0.11681e-03_rb,0.11304e-03_rb, &
        & 0.10512e-03_rb,0.91981e-04_rb,0.75441e-04_rb,0.45782e-04_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.61166e-04_rb,0.10769e-03_rb,0.11529e-03_rb,0.11740e-03_rb,0.11428e-03_rb, &
        & 0.10674e-03_rb,0.93329e-04_rb,0.76963e-04_rb,0.46781e-04_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.49298e-04_rb,0.93665e-04_rb,0.99302e-04_rb,0.10166e-03_rb,0.96125e-04_rb, &
        & 0.87991e-04_rb,0.77830e-04_rb,0.61651e-04_rb,0.45761e-04_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.50498e-04_rb,0.93479e-04_rb,0.98966e-04_rb,0.10139e-03_rb,0.96705e-04_rb, &
        & 0.89173e-04_rb,0.78227e-04_rb,0.62850e-04_rb,0.46039e-04_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.51269e-04_rb,0.93203e-04_rb,0.98973e-04_rb,0.10127e-03_rb,0.97298e-04_rb, &
        & 0.90362e-04_rb,0.78802e-04_rb,0.64157e-04_rb,0.46346e-04_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.51588e-04_rb,0.93137e-04_rb,0.99162e-04_rb,0.10129e-03_rb,0.98592e-04_rb, &
        & 0.91554e-04_rb,0.79929e-04_rb,0.65574e-04_rb,0.46727e-04_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.51597e-04_rb,0.92792e-04_rb,0.99458e-04_rb,0.10170e-03_rb,0.99448e-04_rb, &
        & 0.92834e-04_rb,0.81330e-04_rb,0.66366e-04_rb,0.47183e-04_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.41246e-04_rb,0.80496e-04_rb,0.85316e-04_rb,0.87435e-04_rb,0.82542e-04_rb, &
        & 0.75819e-04_rb,0.67022e-04_rb,0.53111e-04_rb,0.48352e-04_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.42332e-04_rb,0.80236e-04_rb,0.84984e-04_rb,0.87173e-04_rb,0.83192e-04_rb, &
        & 0.76872e-04_rb,0.67154e-04_rb,0.54235e-04_rb,0.49203e-04_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.43069e-04_rb,0.79969e-04_rb,0.85117e-04_rb,0.87191e-04_rb,0.83963e-04_rb, &
        & 0.78027e-04_rb,0.67911e-04_rb,0.55423e-04_rb,0.50036e-04_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.43347e-04_rb,0.79762e-04_rb,0.85176e-04_rb,0.87282e-04_rb,0.84970e-04_rb, &
        & 0.78971e-04_rb,0.69097e-04_rb,0.56659e-04_rb,0.50699e-04_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.43381e-04_rb,0.79473e-04_rb,0.85320e-04_rb,0.87550e-04_rb,0.85526e-04_rb, &
        & 0.80111e-04_rb,0.70399e-04_rb,0.56992e-04_rb,0.51218e-04_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.34402e-04_rb,0.69256e-04_rb,0.73264e-04_rb,0.74907e-04_rb,0.70733e-04_rb, &
        & 0.65058e-04_rb,0.57123e-04_rb,0.45547e-04_rb,0.49061e-04_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.35394e-04_rb,0.68938e-04_rb,0.72891e-04_rb,0.74693e-04_rb,0.71198e-04_rb, &
        & 0.66010e-04_rb,0.57419e-04_rb,0.46585e-04_rb,0.51368e-04_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.36086e-04_rb,0.68637e-04_rb,0.72875e-04_rb,0.74679e-04_rb,0.72090e-04_rb, &
        & 0.66902e-04_rb,0.58282e-04_rb,0.47622e-04_rb,0.51837e-04_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.36445e-04_rb,0.68326e-04_rb,0.72869e-04_rb,0.74856e-04_rb,0.72765e-04_rb, &
        & 0.67827e-04_rb,0.59362e-04_rb,0.48483e-04_rb,0.52297e-04_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.36483e-04_rb,0.68010e-04_rb,0.72961e-04_rb,0.75096e-04_rb,0.73241e-04_rb, &
        & 0.68587e-04_rb,0.60512e-04_rb,0.48784e-04_rb,0.53745e-04_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.28433e-04_rb,0.59232e-04_rb,0.62701e-04_rb,0.63901e-04_rb,0.60103e-04_rb, &
        & 0.55141e-04_rb,0.48531e-04_rb,0.38847e-04_rb,0.54662e-04_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.29405e-04_rb,0.58918e-04_rb,0.62216e-04_rb,0.63604e-04_rb,0.60633e-04_rb, &
        & 0.55830e-04_rb,0.48918e-04_rb,0.39795e-04_rb,0.56459e-04_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.30040e-04_rb,0.58597e-04_rb,0.61986e-04_rb,0.63627e-04_rb,0.61357e-04_rb, &
        & 0.56779e-04_rb,0.49706e-04_rb,0.40729e-04_rb,0.58698e-04_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.30412e-04_rb,0.58268e-04_rb,0.61940e-04_rb,0.63690e-04_rb,0.61916e-04_rb, &
        & 0.57700e-04_rb,0.50677e-04_rb,0.41137e-04_rb,0.60426e-04_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.30517e-04_rb,0.57913e-04_rb,0.61982e-04_rb,0.63808e-04_rb,0.62307e-04_rb, &
        & 0.58353e-04_rb,0.51602e-04_rb,0.41668e-04_rb,0.61933e-04_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.23291e-04_rb,0.50308e-04_rb,0.53243e-04_rb,0.54258e-04_rb,0.50787e-04_rb, &
        & 0.46509e-04_rb,0.41036e-04_rb,0.32969e-04_rb,0.63749e-04_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.24304e-04_rb,0.50027e-04_rb,0.52858e-04_rb,0.54035e-04_rb,0.51457e-04_rb, &
        & 0.47117e-04_rb,0.41427e-04_rb,0.33795e-04_rb,0.67891e-04_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.24884e-04_rb,0.49658e-04_rb,0.52484e-04_rb,0.53877e-04_rb,0.52001e-04_rb, &
        & 0.47908e-04_rb,0.42080e-04_rb,0.34652e-04_rb,0.71374e-04_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.25245e-04_rb,0.49329e-04_rb,0.52369e-04_rb,0.53861e-04_rb,0.52353e-04_rb, &
        & 0.48763e-04_rb,0.42918e-04_rb,0.34903e-04_rb,0.73983e-04_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.25399e-04_rb,0.49019e-04_rb,0.52344e-04_rb,0.53920e-04_rb,0.52702e-04_rb, &
        & 0.49354e-04_rb,0.43795e-04_rb,0.35336e-04_rb,0.76874e-04_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.19464e-04_rb,0.42428e-04_rb,0.44774e-04_rb,0.45764e-04_rb,0.43065e-04_rb, &
        & 0.39257e-04_rb,0.34681e-04_rb,0.28149e-04_rb,0.56538e-04_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.20274e-04_rb,0.42118e-04_rb,0.44484e-04_rb,0.45525e-04_rb,0.43594e-04_rb, &
        & 0.39908e-04_rb,0.35109e-04_rb,0.28880e-04_rb,0.61143e-04_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.20723e-04_rb,0.41781e-04_rb,0.44183e-04_rb,0.45345e-04_rb,0.43968e-04_rb, &
        & 0.40542e-04_rb,0.35722e-04_rb,0.29427e-04_rb,0.66428e-04_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.20986e-04_rb,0.41469e-04_rb,0.44005e-04_rb,0.45299e-04_rb,0.44240e-04_rb, &
        & 0.41204e-04_rb,0.36501e-04_rb,0.29565e-04_rb,0.70536e-04_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.21088e-04_rb,0.41122e-04_rb,0.44008e-04_rb,0.45344e-04_rb,0.44427e-04_rb, &
        & 0.41695e-04_rb,0.37345e-04_rb,0.29974e-04_rb,0.74164e-04_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.16205e-04_rb,0.35661e-04_rb,0.37585e-04_rb,0.38332e-04_rb,0.36196e-04_rb, &
        & 0.33054e-04_rb,0.29216e-04_rb,0.23917e-04_rb,0.49111e-04_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.16873e-04_rb,0.35314e-04_rb,0.37290e-04_rb,0.38165e-04_rb,0.36667e-04_rb, &
        & 0.33618e-04_rb,0.29598e-04_rb,0.24536e-04_rb,0.53408e-04_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.17216e-04_rb,0.35012e-04_rb,0.37013e-04_rb,0.37971e-04_rb,0.36937e-04_rb, &
        & 0.34200e-04_rb,0.30188e-04_rb,0.24727e-04_rb,0.56692e-04_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.17409e-04_rb,0.34711e-04_rb,0.36851e-04_rb,0.37885e-04_rb,0.37053e-04_rb, &
        & 0.34609e-04_rb,0.30881e-04_rb,0.24937e-04_rb,0.60809e-04_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.17480e-04_rb,0.34362e-04_rb,0.36830e-04_rb,0.37891e-04_rb,0.37213e-04_rb, &
        & 0.35045e-04_rb,0.31524e-04_rb,0.25269e-04_rb,0.65163e-04_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.13457e-04_rb,0.29884e-04_rb,0.31446e-04_rb,0.32052e-04_rb,0.30374e-04_rb, &
        & 0.27733e-04_rb,0.24511e-04_rb,0.20198e-04_rb,0.41009e-04_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.14007e-04_rb,0.29560e-04_rb,0.31187e-04_rb,0.31882e-04_rb,0.30716e-04_rb, &
        & 0.28181e-04_rb,0.24877e-04_rb,0.20737e-04_rb,0.44661e-04_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.14280e-04_rb,0.29273e-04_rb,0.30932e-04_rb,0.31685e-04_rb,0.30856e-04_rb, &
        & 0.28687e-04_rb,0.25310e-04_rb,0.20664e-04_rb,0.47397e-04_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.14434e-04_rb,0.28982e-04_rb,0.30751e-04_rb,0.31601e-04_rb,0.30957e-04_rb, &
        & 0.28945e-04_rb,0.25864e-04_rb,0.20868e-04_rb,0.50575e-04_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.14483e-04_rb,0.28660e-04_rb,0.30711e-04_rb,0.31533e-04_rb,0.31075e-04_rb, &
        & 0.29320e-04_rb,0.26408e-04_rb,0.21257e-04_rb,0.54097e-04_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.22536e-03_rb,0.32037e-03_rb,0.33781e-03_rb,0.34273e-03_rb,0.33729e-03_rb, &
        & 0.32075e-03_rb,0.29153e-03_rb,0.24856e-03_rb,0.16410e-03_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.22342e-03_rb,0.32281e-03_rb,0.34293e-03_rb,0.34673e-03_rb,0.34167e-03_rb, &
        & 0.32688e-03_rb,0.29787e-03_rb,0.25566e-03_rb,0.17029e-03_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.22157e-03_rb,0.32559e-03_rb,0.34865e-03_rb,0.35164e-03_rb,0.34805e-03_rb, &
        & 0.33353e-03_rb,0.30505e-03_rb,0.26380e-03_rb,0.17749e-03_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.21986e-03_rb,0.32894e-03_rb,0.35375e-03_rb,0.35845e-03_rb,0.35514e-03_rb, &
        & 0.33992e-03_rb,0.31266e-03_rb,0.27259e-03_rb,0.18209e-03_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.21815e-03_rb,0.33282e-03_rb,0.35982e-03_rb,0.36699e-03_rb,0.36319e-03_rb, &
        & 0.34849e-03_rb,0.32265e-03_rb,0.28174e-03_rb,0.18630e-03_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.19541e-03_rb,0.28383e-03_rb,0.30057e-03_rb,0.30208e-03_rb,0.29811e-03_rb, &
        & 0.28614e-03_rb,0.25813e-03_rb,0.22172e-03_rb,0.14533e-03_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.19410e-03_rb,0.28606e-03_rb,0.30502e-03_rb,0.30638e-03_rb,0.30342e-03_rb, &
        & 0.29123e-03_rb,0.26438e-03_rb,0.22759e-03_rb,0.15063e-03_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.19280e-03_rb,0.28892e-03_rb,0.31031e-03_rb,0.31345e-03_rb,0.30930e-03_rb, &
        & 0.29708e-03_rb,0.27075e-03_rb,0.23476e-03_rb,0.15387e-03_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.19169e-03_rb,0.29226e-03_rb,0.31596e-03_rb,0.32060e-03_rb,0.31687e-03_rb, &
        & 0.30282e-03_rb,0.27961e-03_rb,0.24147e-03_rb,0.15653e-03_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.19047e-03_rb,0.29594e-03_rb,0.32186e-03_rb,0.32831e-03_rb,0.32510e-03_rb, &
        & 0.31022e-03_rb,0.28786e-03_rb,0.24847e-03_rb,0.16195e-03_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.16871e-03_rb,0.24969e-03_rb,0.26468e-03_rb,0.26465e-03_rb,0.26214e-03_rb, &
        & 0.25247e-03_rb,0.22732e-03_rb,0.19530e-03_rb,0.12777e-03_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.16793e-03_rb,0.25161e-03_rb,0.26876e-03_rb,0.26973e-03_rb,0.26695e-03_rb, &
        & 0.25688e-03_rb,0.23244e-03_rb,0.20054e-03_rb,0.12972e-03_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.16733e-03_rb,0.25378e-03_rb,0.27385e-03_rb,0.27612e-03_rb,0.27305e-03_rb, &
        & 0.26119e-03_rb,0.23922e-03_rb,0.20542e-03_rb,0.13061e-03_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.16706e-03_rb,0.25676e-03_rb,0.27970e-03_rb,0.28379e-03_rb,0.28013e-03_rb, &
        & 0.26708e-03_rb,0.24668e-03_rb,0.21081e-03_rb,0.13394e-03_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.16637e-03_rb,0.26037e-03_rb,0.28504e-03_rb,0.29050e-03_rb,0.28611e-03_rb, &
        & 0.27355e-03_rb,0.25260e-03_rb,0.21670e-03_rb,0.13844e-03_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.14579e-03_rb,0.21908e-03_rb,0.23187e-03_rb,0.23162e-03_rb,0.22921e-03_rb, &
        & 0.22070e-03_rb,0.19997e-03_rb,0.17094e-03_rb,0.11198e-03_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.14542e-03_rb,0.22095e-03_rb,0.23576e-03_rb,0.23659e-03_rb,0.23393e-03_rb, &
        & 0.22421e-03_rb,0.20413e-03_rb,0.17485e-03_rb,0.11080e-03_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.14519e-03_rb,0.22277e-03_rb,0.24039e-03_rb,0.24264e-03_rb,0.23941e-03_rb, &
        & 0.22826e-03_rb,0.20957e-03_rb,0.17862e-03_rb,0.11236e-03_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.14504e-03_rb,0.22526e-03_rb,0.24523e-03_rb,0.24877e-03_rb,0.24473e-03_rb, &
        & 0.23381e-03_rb,0.21512e-03_rb,0.18339e-03_rb,0.11577e-03_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.14497e-03_rb,0.22869e-03_rb,0.25070e-03_rb,0.25513e-03_rb,0.25076e-03_rb, &
        & 0.23990e-03_rb,0.22117e-03_rb,0.18804e-03_rb,0.11947e-03_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.12609e-03_rb,0.19125e-03_rb,0.20302e-03_rb,0.20235e-03_rb,0.20038e-03_rb, &
        & 0.19210e-03_rb,0.17493e-03_rb,0.14918e-03_rb,0.95914e-04_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.12575e-03_rb,0.19304e-03_rb,0.20668e-03_rb,0.20708e-03_rb,0.20395e-03_rb, &
        & 0.19513e-03_rb,0.17862e-03_rb,0.15191e-03_rb,0.95952e-04_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.12572e-03_rb,0.19504e-03_rb,0.21070e-03_rb,0.21227e-03_rb,0.20862e-03_rb, &
        & 0.19877e-03_rb,0.18323e-03_rb,0.15540e-03_rb,0.98040e-04_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.12578e-03_rb,0.19736e-03_rb,0.21498e-03_rb,0.21770e-03_rb,0.21305e-03_rb, &
        & 0.20380e-03_rb,0.18766e-03_rb,0.15949e-03_rb,0.10020e-03_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.12588e-03_rb,0.20051e-03_rb,0.21971e-03_rb,0.22313e-03_rb,0.21848e-03_rb, &
        & 0.20915e-03_rb,0.19284e-03_rb,0.16321e-03_rb,0.10267e-03_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.10863e-03_rb,0.16576e-03_rb,0.17695e-03_rb,0.17678e-03_rb,0.17469e-03_rb, &
        & 0.16678e-03_rb,0.15163e-03_rb,0.12940e-03_rb,0.89550e-04_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.10863e-03_rb,0.16706e-03_rb,0.18030e-03_rb,0.18081e-03_rb,0.17775e-03_rb, &
        & 0.16938e-03_rb,0.15534e-03_rb,0.13148e-03_rb,0.89670e-04_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.10855e-03_rb,0.16916e-03_rb,0.18384e-03_rb,0.18538e-03_rb,0.18134e-03_rb, &
        & 0.17272e-03_rb,0.15912e-03_rb,0.13459e-03_rb,0.90520e-04_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.10855e-03_rb,0.17152e-03_rb,0.18761e-03_rb,0.18997e-03_rb,0.18510e-03_rb, &
        & 0.17711e-03_rb,0.16286e-03_rb,0.13757e-03_rb,0.91792e-04_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.10853e-03_rb,0.17415e-03_rb,0.19152e-03_rb,0.19450e-03_rb,0.18955e-03_rb, &
        & 0.18160e-03_rb,0.16658e-03_rb,0.14151e-03_rb,0.94030e-04_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.92930e-04_rb,0.14283e-03_rb,0.15332e-03_rb,0.15365e-03_rb,0.15192e-03_rb, &
        & 0.14476e-03_rb,0.13092e-03_rb,0.11095e-03_rb,0.91877e-04_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.92984e-04_rb,0.14401e-03_rb,0.15614e-03_rb,0.15721e-03_rb,0.15413e-03_rb, &
        & 0.14688e-03_rb,0.13443e-03_rb,0.11315e-03_rb,0.90405e-04_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.92945e-04_rb,0.14573e-03_rb,0.15905e-03_rb,0.16110e-03_rb,0.15737e-03_rb, &
        & 0.14983e-03_rb,0.13738e-03_rb,0.11591e-03_rb,0.89977e-04_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.93035e-04_rb,0.14787e-03_rb,0.16239e-03_rb,0.16472e-03_rb,0.16084e-03_rb, &
        & 0.15371e-03_rb,0.14051e-03_rb,0.11827e-03_rb,0.90955e-04_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.92985e-04_rb,0.15023e-03_rb,0.16593e-03_rb,0.16877e-03_rb,0.16496e-03_rb, &
        & 0.15743e-03_rb,0.14348e-03_rb,0.12221e-03_rb,0.92269e-04_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.79600e-04_rb,0.12290e-03_rb,0.13269e-03_rb,0.13322e-03_rb,0.13145e-03_rb, &
        & 0.12540e-03_rb,0.11354e-03_rb,0.94559e-04_rb,0.10796e-03_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.79683e-04_rb,0.12390e-03_rb,0.13495e-03_rb,0.13633e-03_rb,0.13351e-03_rb, &
        & 0.12708e-03_rb,0.11620e-03_rb,0.96898e-04_rb,0.10854e-03_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.79630e-04_rb,0.12537e-03_rb,0.13745e-03_rb,0.13937e-03_rb,0.13626e-03_rb, &
        & 0.12970e-03_rb,0.11867e-03_rb,0.99068e-04_rb,0.10974e-03_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.79625e-04_rb,0.12719e-03_rb,0.14027e-03_rb,0.14234e-03_rb,0.13941e-03_rb, &
        & 0.13283e-03_rb,0.12113e-03_rb,0.10155e-03_rb,0.11075e-03_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.79589e-04_rb,0.12914e-03_rb,0.14322e-03_rb,0.14575e-03_rb,0.14305e-03_rb, &
        & 0.13597e-03_rb,0.12361e-03_rb,0.10505e-03_rb,0.11038e-03_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.67917e-04_rb,0.10525e-03_rb,0.11421e-03_rb,0.11510e-03_rb,0.11341e-03_rb, &
        & 0.10850e-03_rb,0.97945e-04_rb,0.80197e-04_rb,0.13259e-03_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.67928e-04_rb,0.10625e-03_rb,0.11606e-03_rb,0.11762e-03_rb,0.11532e-03_rb, &
        & 0.11035e-03_rb,0.10017e-03_rb,0.82394e-04_rb,0.13427e-03_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.67856e-04_rb,0.10744e-03_rb,0.11824e-03_rb,0.11998e-03_rb,0.11756e-03_rb, &
        & 0.11235e-03_rb,0.10203e-03_rb,0.84408e-04_rb,0.13461e-03_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.67768e-04_rb,0.10890e-03_rb,0.12070e-03_rb,0.12262e-03_rb,0.12012e-03_rb, &
        & 0.11456e-03_rb,0.10420e-03_rb,0.86916e-04_rb,0.13504e-03_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.67707e-04_rb,0.11054e-03_rb,0.12323e-03_rb,0.12567e-03_rb,0.12318e-03_rb, &
        & 0.11694e-03_rb,0.10644e-03_rb,0.89648e-04_rb,0.13829e-03_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.57658e-04_rb,0.89812e-04_rb,0.97774e-04_rb,0.98825e-04_rb,0.97412e-04_rb, &
        & 0.93433e-04_rb,0.84119e-04_rb,0.68026e-04_rb,0.20137e-03_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.57461e-04_rb,0.90575e-04_rb,0.99228e-04_rb,0.10076e-03_rb,0.98930e-04_rb, &
        & 0.94984e-04_rb,0.86081e-04_rb,0.70085e-04_rb,0.20178e-03_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.57397e-04_rb,0.91666e-04_rb,0.10122e-03_rb,0.10281e-03_rb,0.10088e-03_rb, &
        & 0.96688e-04_rb,0.87695e-04_rb,0.72010e-04_rb,0.20052e-03_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.57291e-04_rb,0.92869e-04_rb,0.10334e-03_rb,0.10517e-03_rb,0.10317e-03_rb, &
        & 0.98293e-04_rb,0.89394e-04_rb,0.74251e-04_rb,0.20103e-03_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.57126e-04_rb,0.94072e-04_rb,0.10545e-03_rb,0.10789e-03_rb,0.10579e-03_rb, &
        & 0.10034e-03_rb,0.91420e-04_rb,0.76761e-04_rb,0.20035e-03_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.48600e-04_rb,0.76535e-04_rb,0.83721e-04_rb,0.84782e-04_rb,0.83379e-04_rb, &
        & 0.80481e-04_rb,0.72441e-04_rb,0.58503e-04_rb,0.20926e-03_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.48356e-04_rb,0.77272e-04_rb,0.85067e-04_rb,0.86588e-04_rb,0.84928e-04_rb, &
        & 0.81803e-04_rb,0.74015e-04_rb,0.60222e-04_rb,0.21110e-03_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.48180e-04_rb,0.78198e-04_rb,0.86939e-04_rb,0.88416e-04_rb,0.86712e-04_rb, &
        & 0.83176e-04_rb,0.75535e-04_rb,0.61973e-04_rb,0.20823e-03_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.48042e-04_rb,0.79160e-04_rb,0.88776e-04_rb,0.90704e-04_rb,0.88934e-04_rb, &
        & 0.84522e-04_rb,0.76983e-04_rb,0.64264e-04_rb,0.20745e-03_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.47901e-04_rb,0.80171e-04_rb,0.90395e-04_rb,0.92950e-04_rb,0.91253e-04_rb, &
        & 0.86364e-04_rb,0.78675e-04_rb,0.66159e-04_rb,0.20642e-03_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.40769e-04_rb,0.64775e-04_rb,0.71240e-04_rb,0.72409e-04_rb,0.71085e-04_rb, &
        & 0.68679e-04_rb,0.61925e-04_rb,0.50009e-04_rb,0.19056e-03_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.40486e-04_rb,0.65479e-04_rb,0.72574e-04_rb,0.73906e-04_rb,0.72438e-04_rb, &
        & 0.69693e-04_rb,0.63286e-04_rb,0.51717e-04_rb,0.19000e-03_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.40313e-04_rb,0.66212e-04_rb,0.74137e-04_rb,0.75686e-04_rb,0.74201e-04_rb, &
        & 0.70781e-04_rb,0.64554e-04_rb,0.53372e-04_rb,0.19242e-03_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.40107e-04_rb,0.66938e-04_rb,0.75524e-04_rb,0.77664e-04_rb,0.76142e-04_rb, &
        & 0.72280e-04_rb,0.65894e-04_rb,0.55174e-04_rb,0.19382e-03_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.39910e-04_rb,0.67782e-04_rb,0.76780e-04_rb,0.79463e-04_rb,0.78210e-04_rb, &
        & 0.73902e-04_rb,0.67292e-04_rb,0.56789e-04_rb,0.19106e-03_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.34103e-04_rb,0.54622e-04_rb,0.60315e-04_rb,0.61417e-04_rb,0.60332e-04_rb, &
        & 0.58110e-04_rb,0.52537e-04_rb,0.42612e-04_rb,0.15955e-03_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.33853e-04_rb,0.55148e-04_rb,0.61492e-04_rb,0.62745e-04_rb,0.61577e-04_rb, &
        & 0.59061e-04_rb,0.53702e-04_rb,0.43873e-04_rb,0.15892e-03_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.33647e-04_rb,0.55699e-04_rb,0.62687e-04_rb,0.64355e-04_rb,0.63160e-04_rb, &
        & 0.60072e-04_rb,0.54954e-04_rb,0.45643e-04_rb,0.16047e-03_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.33430e-04_rb,0.56291e-04_rb,0.63770e-04_rb,0.65844e-04_rb,0.64844e-04_rb, &
        & 0.61457e-04_rb,0.56064e-04_rb,0.47101e-04_rb,0.16125e-03_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.33186e-04_rb,0.56947e-04_rb,0.64747e-04_rb,0.67352e-04_rb,0.66508e-04_rb, &
        & 0.62903e-04_rb,0.57307e-04_rb,0.48379e-04_rb,0.16021e-03_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.36602e-03_rb,0.52858e-03_rb,0.59251e-03_rb,0.61027e-03_rb,0.60893e-03_rb, &
        & 0.58987e-03_rb,0.55560e-03_rb,0.49855e-03_rb,0.38105e-03_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.36935e-03_rb,0.53570e-03_rb,0.60138e-03_rb,0.62341e-03_rb,0.62297e-03_rb, &
        & 0.60228e-03_rb,0.56872e-03_rb,0.51035e-03_rb,0.39106e-03_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.37348e-03_rb,0.54590e-03_rb,0.61238e-03_rb,0.63855e-03_rb,0.63820e-03_rb, &
        & 0.61734e-03_rb,0.58475e-03_rb,0.52206e-03_rb,0.40317e-03_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.37665e-03_rb,0.55373e-03_rb,0.62261e-03_rb,0.65120e-03_rb,0.65250e-03_rb, &
        & 0.63373e-03_rb,0.59840e-03_rb,0.53517e-03_rb,0.41639e-03_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.38076e-03_rb,0.56302e-03_rb,0.63456e-03_rb,0.66611e-03_rb,0.66857e-03_rb, &
        & 0.65240e-03_rb,0.61521e-03_rb,0.55178e-03_rb,0.43702e-03_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.32225e-03_rb,0.47151e-03_rb,0.53262e-03_rb,0.55192e-03_rb,0.54706e-03_rb, &
        & 0.52658e-03_rb,0.49569e-03_rb,0.43958e-03_rb,0.31739e-03_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.32532e-03_rb,0.47816e-03_rb,0.54076e-03_rb,0.56222e-03_rb,0.55834e-03_rb, &
        & 0.53879e-03_rb,0.50612e-03_rb,0.44777e-03_rb,0.32763e-03_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.32941e-03_rb,0.48729e-03_rb,0.55083e-03_rb,0.57497e-03_rb,0.57312e-03_rb, &
        & 0.55394e-03_rb,0.51991e-03_rb,0.45969e-03_rb,0.33942e-03_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.33303e-03_rb,0.49510e-03_rb,0.56036e-03_rb,0.58720e-03_rb,0.58669e-03_rb, &
        & 0.56974e-03_rb,0.53255e-03_rb,0.47385e-03_rb,0.35453e-03_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.33657e-03_rb,0.50290e-03_rb,0.57014e-03_rb,0.59917e-03_rb,0.59997e-03_rb, &
        & 0.58475e-03_rb,0.54777e-03_rb,0.48909e-03_rb,0.36980e-03_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.28263e-03_rb,0.41968e-03_rb,0.47376e-03_rb,0.49297e-03_rb,0.48633e-03_rb, &
        & 0.46530e-03_rb,0.43654e-03_rb,0.38496e-03_rb,0.26217e-03_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.28517e-03_rb,0.42528e-03_rb,0.48017e-03_rb,0.50148e-03_rb,0.49564e-03_rb, &
        & 0.47634e-03_rb,0.44517e-03_rb,0.39228e-03_rb,0.27386e-03_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.28805e-03_rb,0.43254e-03_rb,0.48769e-03_rb,0.51075e-03_rb,0.50628e-03_rb, &
        & 0.48940e-03_rb,0.45557e-03_rb,0.40274e-03_rb,0.28527e-03_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.29156e-03_rb,0.44130e-03_rb,0.49824e-03_rb,0.52245e-03_rb,0.52001e-03_rb, &
        & 0.50355e-03_rb,0.46711e-03_rb,0.41540e-03_rb,0.29729e-03_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.29453e-03_rb,0.44828e-03_rb,0.50753e-03_rb,0.53257e-03_rb,0.53321e-03_rb, &
        & 0.51645e-03_rb,0.48189e-03_rb,0.42882e-03_rb,0.30960e-03_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.24709e-03_rb,0.37040e-03_rb,0.41811e-03_rb,0.43692e-03_rb,0.42924e-03_rb, &
        & 0.40974e-03_rb,0.38217e-03_rb,0.33504e-03_rb,0.21956e-03_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.24935e-03_rb,0.37531e-03_rb,0.42359e-03_rb,0.44356e-03_rb,0.43669e-03_rb, &
        & 0.41955e-03_rb,0.39063e-03_rb,0.34337e-03_rb,0.23257e-03_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.25192e-03_rb,0.38183e-03_rb,0.43049e-03_rb,0.45130e-03_rb,0.44572e-03_rb, &
        & 0.43016e-03_rb,0.39919e-03_rb,0.35274e-03_rb,0.24051e-03_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.25420e-03_rb,0.38887e-03_rb,0.43864e-03_rb,0.46020e-03_rb,0.45692e-03_rb, &
        & 0.44111e-03_rb,0.40882e-03_rb,0.36219e-03_rb,0.24966e-03_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.25772e-03_rb,0.39769e-03_rb,0.44993e-03_rb,0.47213e-03_rb,0.47068e-03_rb, &
        & 0.45426e-03_rb,0.42157e-03_rb,0.37400e-03_rb,0.25930e-03_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.21574e-03_rb,0.32603e-03_rb,0.36644e-03_rb,0.38307e-03_rb,0.37721e-03_rb, &
        & 0.36018e-03_rb,0.33363e-03_rb,0.29143e-03_rb,0.18896e-03_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.21778e-03_rb,0.33044e-03_rb,0.37107e-03_rb,0.38833e-03_rb,0.38426e-03_rb, &
        & 0.36797e-03_rb,0.34054e-03_rb,0.29920e-03_rb,0.19816e-03_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.22015e-03_rb,0.33584e-03_rb,0.37735e-03_rb,0.39513e-03_rb,0.39228e-03_rb, &
        & 0.37720e-03_rb,0.34842e-03_rb,0.30654e-03_rb,0.20383e-03_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.22252e-03_rb,0.34167e-03_rb,0.38484e-03_rb,0.40343e-03_rb,0.40251e-03_rb, &
        & 0.38567e-03_rb,0.35760e-03_rb,0.31401e-03_rb,0.21178e-03_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.22604e-03_rb,0.34964e-03_rb,0.39509e-03_rb,0.41443e-03_rb,0.41465e-03_rb, &
        & 0.39685e-03_rb,0.36847e-03_rb,0.32400e-03_rb,0.21929e-03_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.18783e-03_rb,0.28666e-03_rb,0.32037e-03_rb,0.33351e-03_rb,0.32945e-03_rb, &
        & 0.31516e-03_rb,0.29066e-03_rb,0.25263e-03_rb,0.16238e-03_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.18921e-03_rb,0.29078e-03_rb,0.32423e-03_rb,0.33799e-03_rb,0.33536e-03_rb, &
        & 0.32156e-03_rb,0.29643e-03_rb,0.25941e-03_rb,0.16831e-03_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.19117e-03_rb,0.29487e-03_rb,0.32978e-03_rb,0.34375e-03_rb,0.34274e-03_rb, &
        & 0.32907e-03_rb,0.30344e-03_rb,0.26560e-03_rb,0.17389e-03_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.19356e-03_rb,0.29979e-03_rb,0.33631e-03_rb,0.35135e-03_rb,0.35098e-03_rb, &
        & 0.33648e-03_rb,0.31108e-03_rb,0.27199e-03_rb,0.18056e-03_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.19643e-03_rb,0.30567e-03_rb,0.34396e-03_rb,0.35996e-03_rb,0.36067e-03_rb, &
        & 0.34559e-03_rb,0.32056e-03_rb,0.28009e-03_rb,0.18507e-03_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.16233e-03_rb,0.25159e-03_rb,0.28051e-03_rb,0.29071e-03_rb,0.28724e-03_rb, &
        & 0.27399e-03_rb,0.25257e-03_rb,0.21995e-03_rb,0.15053e-03_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.16394e-03_rb,0.25497e-03_rb,0.28374e-03_rb,0.29454e-03_rb,0.29203e-03_rb, &
        & 0.28002e-03_rb,0.25781e-03_rb,0.22523e-03_rb,0.15467e-03_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.16554e-03_rb,0.25862e-03_rb,0.28837e-03_rb,0.29946e-03_rb,0.29804e-03_rb, &
        & 0.28580e-03_rb,0.26386e-03_rb,0.23003e-03_rb,0.15858e-03_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.16755e-03_rb,0.26275e-03_rb,0.29380e-03_rb,0.30624e-03_rb,0.30499e-03_rb, &
        & 0.29233e-03_rb,0.27029e-03_rb,0.23539e-03_rb,0.16161e-03_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.17007e-03_rb,0.26765e-03_rb,0.30035e-03_rb,0.31353e-03_rb,0.31332e-03_rb, &
        & 0.30019e-03_rb,0.27871e-03_rb,0.24192e-03_rb,0.16587e-03_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.14021e-03_rb,0.21983e-03_rb,0.24557e-03_rb,0.25394e-03_rb,0.25027e-03_rb, &
        & 0.23769e-03_rb,0.21853e-03_rb,0.19056e-03_rb,0.17210e-03_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.14181e-03_rb,0.22244e-03_rb,0.24837e-03_rb,0.25730e-03_rb,0.25433e-03_rb, &
        & 0.24284e-03_rb,0.22361e-03_rb,0.19448e-03_rb,0.16707e-03_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.14344e-03_rb,0.22578e-03_rb,0.25216e-03_rb,0.26169e-03_rb,0.25950e-03_rb, &
        & 0.24760e-03_rb,0.22868e-03_rb,0.19861e-03_rb,0.16518e-03_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.14524e-03_rb,0.22935e-03_rb,0.25652e-03_rb,0.26715e-03_rb,0.26554e-03_rb, &
        & 0.25327e-03_rb,0.23470e-03_rb,0.20360e-03_rb,0.16514e-03_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.14752e-03_rb,0.23348e-03_rb,0.26209e-03_rb,0.27316e-03_rb,0.27225e-03_rb, &
        & 0.26056e-03_rb,0.24203e-03_rb,0.20916e-03_rb,0.16671e-03_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.12008e-03_rb,0.19084e-03_rb,0.21359e-03_rb,0.22137e-03_rb,0.21826e-03_rb, &
        & 0.20646e-03_rb,0.18855e-03_rb,0.16443e-03_rb,0.31379e-03_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.12170e-03_rb,0.19281e-03_rb,0.21619e-03_rb,0.22414e-03_rb,0.22156e-03_rb, &
        & 0.21036e-03_rb,0.19272e-03_rb,0.16751e-03_rb,0.30810e-03_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.12341e-03_rb,0.19553e-03_rb,0.21930e-03_rb,0.22810e-03_rb,0.22604e-03_rb, &
        & 0.21457e-03_rb,0.19724e-03_rb,0.17093e-03_rb,0.30579e-03_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.12515e-03_rb,0.19866e-03_rb,0.22306e-03_rb,0.23273e-03_rb,0.23128e-03_rb, &
        & 0.22000e-03_rb,0.20255e-03_rb,0.17520e-03_rb,0.30468e-03_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.12711e-03_rb,0.20244e-03_rb,0.22780e-03_rb,0.23779e-03_rb,0.23723e-03_rb, &
        & 0.22672e-03_rb,0.20873e-03_rb,0.18006e-03_rb,0.30071e-03_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.10239e-03_rb,0.16495e-03_rb,0.18568e-03_rb,0.19287e-03_rb,0.19038e-03_rb, &
        & 0.17962e-03_rb,0.16274e-03_rb,0.14112e-03_rb,0.46281e-03_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.10386e-03_rb,0.16652e-03_rb,0.18791e-03_rb,0.19539e-03_rb,0.19307e-03_rb, &
        & 0.18291e-03_rb,0.16628e-03_rb,0.14354e-03_rb,0.46032e-03_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.10550e-03_rb,0.16884e-03_rb,0.19060e-03_rb,0.19863e-03_rb,0.19685e-03_rb, &
        & 0.18669e-03_rb,0.17045e-03_rb,0.14674e-03_rb,0.45882e-03_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.10710e-03_rb,0.17185e-03_rb,0.19377e-03_rb,0.20238e-03_rb,0.20140e-03_rb, &
        & 0.19164e-03_rb,0.17544e-03_rb,0.15076e-03_rb,0.45556e-03_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.10887e-03_rb,0.17528e-03_rb,0.19755e-03_rb,0.20663e-03_rb,0.20650e-03_rb, &
        & 0.19756e-03_rb,0.18068e-03_rb,0.15495e-03_rb,0.45295e-03_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.87454e-04_rb,0.14214e-03_rb,0.16093e-03_rb,0.16784e-03_rb,0.16634e-03_rb, &
        & 0.15750e-03_rb,0.14233e-03_rb,0.12157e-03_rb,0.50227e-03_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.89026e-04_rb,0.14363e-03_rb,0.16297e-03_rb,0.17024e-03_rb,0.16905e-03_rb, &
        & 0.16052e-03_rb,0.14560e-03_rb,0.12384e-03_rb,0.48800e-03_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.90456e-04_rb,0.14595e-03_rb,0.16534e-03_rb,0.17342e-03_rb,0.17281e-03_rb, &
        & 0.16441e-03_rb,0.14965e-03_rb,0.12699e-03_rb,0.48638e-03_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.91788e-04_rb,0.14885e-03_rb,0.16823e-03_rb,0.17680e-03_rb,0.17688e-03_rb, &
        & 0.16911e-03_rb,0.15409e-03_rb,0.13060e-03_rb,0.48097e-03_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.92958e-04_rb,0.15220e-03_rb,0.17197e-03_rb,0.18085e-03_rb,0.18150e-03_rb, &
        & 0.17440e-03_rb,0.15892e-03_rb,0.13466e-03_rb,0.48115e-03_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.74228e-04_rb,0.12171e-03_rb,0.13828e-03_rb,0.14464e-03_rb,0.14381e-03_rb, &
        & 0.13674e-03_rb,0.12370e-03_rb,0.10454e-03_rb,0.48263e-03_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.75542e-04_rb,0.12330e-03_rb,0.13992e-03_rb,0.14688e-03_rb,0.14666e-03_rb, &
        & 0.13981e-03_rb,0.12694e-03_rb,0.10662e-03_rb,0.46961e-03_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.76696e-04_rb,0.12551e-03_rb,0.14218e-03_rb,0.14964e-03_rb,0.14989e-03_rb, &
        & 0.14346e-03_rb,0.13058e-03_rb,0.10986e-03_rb,0.45986e-03_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.77769e-04_rb,0.12821e-03_rb,0.14505e-03_rb,0.15276e-03_rb,0.15361e-03_rb, &
        & 0.14764e-03_rb,0.13466e-03_rb,0.11322e-03_rb,0.45064e-03_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.78633e-04_rb,0.13113e-03_rb,0.14863e-03_rb,0.15667e-03_rb,0.15783e-03_rb, &
        & 0.15249e-03_rb,0.13915e-03_rb,0.11696e-03_rb,0.45257e-03_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.62659e-04_rb,0.10388e-03_rb,0.11828e-03_rb,0.12392e-03_rb,0.12371e-03_rb, &
        & 0.11811e-03_rb,0.10712e-03_rb,0.89769e-04_rb,0.40925e-03_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.63709e-04_rb,0.10546e-03_rb,0.11986e-03_rb,0.12594e-03_rb,0.12627e-03_rb, &
        & 0.12095e-03_rb,0.11006e-03_rb,0.92053e-04_rb,0.39852e-03_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.64717e-04_rb,0.10750e-03_rb,0.12189e-03_rb,0.12839e-03_rb,0.12907e-03_rb, &
        & 0.12414e-03_rb,0.11317e-03_rb,0.94939e-04_rb,0.39013e-03_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.65465e-04_rb,0.10983e-03_rb,0.12464e-03_rb,0.13149e-03_rb,0.13253e-03_rb, &
        & 0.12801e-03_rb,0.11693e-03_rb,0.98001e-04_rb,0.38349e-03_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.66137e-04_rb,0.11222e-03_rb,0.12786e-03_rb,0.13517e-03_rb,0.13652e-03_rb, &
        & 0.13219e-03_rb,0.12093e-03_rb,0.10155e-03_rb,0.38305e-03_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.60998e-03_rb,0.92357e-03_rb,0.10463e-02_rb,0.11196e-02_rb,0.11571e-02_rb, &
        & 0.11569e-02_rb,0.11152e-02_rb,0.10342e-02_rb,0.80721e-03_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.61496e-03_rb,0.93510e-03_rb,0.10604e-02_rb,0.11361e-02_rb,0.11777e-02_rb, &
        & 0.11834e-02_rb,0.11457e-02_rb,0.10708e-02_rb,0.83732e-03_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.62454e-03_rb,0.95002e-03_rb,0.10819e-02_rb,0.11594e-02_rb,0.12038e-02_rb, &
        & 0.12158e-02_rb,0.11823e-02_rb,0.11158e-02_rb,0.87187e-03_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.63013e-03_rb,0.96444e-03_rb,0.11010e-02_rb,0.11847e-02_rb,0.12326e-02_rb, &
        & 0.12499e-02_rb,0.12239e-02_rb,0.11623e-02_rb,0.91505e-03_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.63867e-03_rb,0.98562e-03_rb,0.11276e-02_rb,0.12151e-02_rb,0.12673e-02_rb, &
        & 0.12886e-02_rb,0.12687e-02_rb,0.12127e-02_rb,0.95931e-03_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.54006e-03_rb,0.82110e-03_rb,0.93517e-03_rb,0.10086e-02_rb,0.10436e-02_rb, &
        & 0.10473e-02_rb,0.10071e-02_rb,0.92435e-03_rb,0.69632e-03_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.54512e-03_rb,0.83214e-03_rb,0.94586e-03_rb,0.10224e-02_rb,0.10616e-02_rb, &
        & 0.10697e-02_rb,0.10360e-02_rb,0.95897e-03_rb,0.72282e-03_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.55430e-03_rb,0.84713e-03_rb,0.96481e-03_rb,0.10423e-02_rb,0.10850e-02_rb, &
        & 0.10972e-02_rb,0.10691e-02_rb,0.99640e-03_rb,0.75512e-03_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.56089e-03_rb,0.86369e-03_rb,0.98575e-03_rb,0.10648e-02_rb,0.11112e-02_rb, &
        & 0.11281e-02_rb,0.11049e-02_rb,0.10361e-02_rb,0.79150e-03_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.56701e-03_rb,0.88105e-03_rb,0.10081e-02_rb,0.10898e-02_rb,0.11420e-02_rb, &
        & 0.11626e-02_rb,0.11406e-02_rb,0.10780e-02_rb,0.83023e-03_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.47586e-03_rb,0.72781e-03_rb,0.83369e-03_rb,0.89860e-03_rb,0.93234e-03_rb, &
        & 0.93626e-03_rb,0.89909e-03_rb,0.81585e-03_rb,0.59036e-03_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.48035e-03_rb,0.73757e-03_rb,0.84502e-03_rb,0.91276e-03_rb,0.94800e-03_rb, &
        & 0.95536e-03_rb,0.92668e-03_rb,0.84501e-03_rb,0.61080e-03_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.48667e-03_rb,0.74941e-03_rb,0.86058e-03_rb,0.92996e-03_rb,0.96794e-03_rb, &
        & 0.97737e-03_rb,0.95368e-03_rb,0.87646e-03_rb,0.63872e-03_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.49637e-03_rb,0.76538e-03_rb,0.87882e-03_rb,0.95073e-03_rb,0.99190e-03_rb, &
        & 0.10051e-02_rb,0.98438e-03_rb,0.91070e-03_rb,0.66866e-03_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.50269e-03_rb,0.78028e-03_rb,0.89635e-03_rb,0.97198e-03_rb,0.10179e-02_rb, &
        & 0.10338e-02_rb,0.10130e-02_rb,0.94553e-03_rb,0.70095e-03_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.41745e-03_rb,0.64505e-03_rb,0.73966e-03_rb,0.79761e-03_rb,0.82752e-03_rb, &
        & 0.82878e-03_rb,0.79470e-03_rb,0.71636e-03_rb,0.49987e-03_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.42212e-03_rb,0.65318e-03_rb,0.75017e-03_rb,0.80979e-03_rb,0.84194e-03_rb, &
        & 0.84473e-03_rb,0.81717e-03_rb,0.73865e-03_rb,0.51625e-03_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.42808e-03_rb,0.66407e-03_rb,0.76314e-03_rb,0.82416e-03_rb,0.85814e-03_rb, &
        & 0.86375e-03_rb,0.83972e-03_rb,0.76440e-03_rb,0.53894e-03_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.43544e-03_rb,0.67737e-03_rb,0.77911e-03_rb,0.84222e-03_rb,0.87777e-03_rb, &
        & 0.88583e-03_rb,0.86479e-03_rb,0.79362e-03_rb,0.56271e-03_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.44549e-03_rb,0.69330e-03_rb,0.79686e-03_rb,0.86274e-03_rb,0.90103e-03_rb, &
        & 0.91189e-03_rb,0.89184e-03_rb,0.82416e-03_rb,0.58887e-03_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.36593e-03_rb,0.56942e-03_rb,0.65432e-03_rb,0.70354e-03_rb,0.72916e-03_rb, &
        & 0.72850e-03_rb,0.69713e-03_rb,0.62558e-03_rb,0.42621e-03_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.37099e-03_rb,0.57654e-03_rb,0.66395e-03_rb,0.71472e-03_rb,0.74107e-03_rb, &
        & 0.74240e-03_rb,0.71535e-03_rb,0.64376e-03_rb,0.43884e-03_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.37647e-03_rb,0.58652e-03_rb,0.67530e-03_rb,0.72837e-03_rb,0.75654e-03_rb, &
        & 0.75759e-03_rb,0.73402e-03_rb,0.66546e-03_rb,0.45630e-03_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.38311e-03_rb,0.59837e-03_rb,0.68871e-03_rb,0.74328e-03_rb,0.77277e-03_rb, &
        & 0.77677e-03_rb,0.75490e-03_rb,0.68962e-03_rb,0.47449e-03_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.39217e-03_rb,0.61368e-03_rb,0.70480e-03_rb,0.76178e-03_rb,0.79297e-03_rb, &
        & 0.79889e-03_rb,0.77761e-03_rb,0.71542e-03_rb,0.49580e-03_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.32055e-03_rb,0.49922e-03_rb,0.57435e-03_rb,0.61868e-03_rb,0.63931e-03_rb, &
        & 0.63779e-03_rb,0.60689e-03_rb,0.54327e-03_rb,0.36282e-03_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.32502e-03_rb,0.50520e-03_rb,0.58251e-03_rb,0.62835e-03_rb,0.64977e-03_rb, &
        & 0.64920e-03_rb,0.62209e-03_rb,0.55837e-03_rb,0.37399e-03_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.33055e-03_rb,0.51462e-03_rb,0.59267e-03_rb,0.63972e-03_rb,0.66253e-03_rb, &
        & 0.66213e-03_rb,0.63802e-03_rb,0.57660e-03_rb,0.38710e-03_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.33647e-03_rb,0.52569e-03_rb,0.60454e-03_rb,0.65260e-03_rb,0.67645e-03_rb, &
        & 0.67638e-03_rb,0.65560e-03_rb,0.59667e-03_rb,0.40130e-03_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.34306e-03_rb,0.53846e-03_rb,0.61820e-03_rb,0.66796e-03_rb,0.69284e-03_rb, &
        & 0.69400e-03_rb,0.67340e-03_rb,0.61700e-03_rb,0.41923e-03_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.28149e-03_rb,0.43644e-03_rb,0.49974e-03_rb,0.53938e-03_rb,0.55701e-03_rb, &
        & 0.55579e-03_rb,0.52736e-03_rb,0.47001e-03_rb,0.31306e-03_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.28528e-03_rb,0.44207e-03_rb,0.50655e-03_rb,0.54763e-03_rb,0.56691e-03_rb, &
        & 0.56439e-03_rb,0.53930e-03_rb,0.48247e-03_rb,0.32173e-03_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.29004e-03_rb,0.45020e-03_rb,0.51518e-03_rb,0.55741e-03_rb,0.57771e-03_rb, &
        & 0.57585e-03_rb,0.55263e-03_rb,0.49748e-03_rb,0.33090e-03_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.29568e-03_rb,0.46070e-03_rb,0.52625e-03_rb,0.56864e-03_rb,0.58962e-03_rb, &
        & 0.58944e-03_rb,0.56702e-03_rb,0.51458e-03_rb,0.34232e-03_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.30178e-03_rb,0.47274e-03_rb,0.53886e-03_rb,0.58253e-03_rb,0.60354e-03_rb, &
        & 0.60454e-03_rb,0.58225e-03_rb,0.53128e-03_rb,0.35470e-03_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.24786e-03_rb,0.38205e-03_rb,0.43403e-03_rb,0.46727e-03_rb,0.48302e-03_rb, &
        & 0.48227e-03_rb,0.45812e-03_rb,0.40611e-03_rb,0.30468e-03_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.25026e-03_rb,0.38705e-03_rb,0.43948e-03_rb,0.47351e-03_rb,0.49158e-03_rb, &
        & 0.49003e-03_rb,0.46771e-03_rb,0.41673e-03_rb,0.30992e-03_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.25464e-03_rb,0.39453e-03_rb,0.44681e-03_rb,0.48211e-03_rb,0.50112e-03_rb, &
        & 0.49992e-03_rb,0.47844e-03_rb,0.42901e-03_rb,0.31626e-03_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.25978e-03_rb,0.40390e-03_rb,0.45683e-03_rb,0.49220e-03_rb,0.51134e-03_rb, &
        & 0.51141e-03_rb,0.48898e-03_rb,0.44238e-03_rb,0.32406e-03_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.26514e-03_rb,0.41428e-03_rb,0.46827e-03_rb,0.50433e-03_rb,0.52333e-03_rb, &
        & 0.52414e-03_rb,0.50147e-03_rb,0.45619e-03_rb,0.33317e-03_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.21824e-03_rb,0.33786e-03_rb,0.37946e-03_rb,0.40535e-03_rb,0.41752e-03_rb, &
        & 0.41661e-03_rb,0.39611e-03_rb,0.34986e-03_rb,0.52178e-03_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.22059e-03_rb,0.34230e-03_rb,0.38410e-03_rb,0.41056e-03_rb,0.42477e-03_rb, &
        & 0.42372e-03_rb,0.40485e-03_rb,0.35866e-03_rb,0.50681e-03_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.22400e-03_rb,0.34814e-03_rb,0.39039e-03_rb,0.41748e-03_rb,0.43284e-03_rb, &
        & 0.43220e-03_rb,0.41363e-03_rb,0.36885e-03_rb,0.49302e-03_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.22873e-03_rb,0.35603e-03_rb,0.39905e-03_rb,0.42659e-03_rb,0.44174e-03_rb, &
        & 0.44167e-03_rb,0.42288e-03_rb,0.37972e-03_rb,0.48294e-03_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.23347e-03_rb,0.36495e-03_rb,0.40893e-03_rb,0.43763e-03_rb,0.45181e-03_rb, &
        & 0.45234e-03_rb,0.43347e-03_rb,0.39120e-03_rb,0.47762e-03_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.19122e-03_rb,0.29954e-03_rb,0.33486e-03_rb,0.35474e-03_rb,0.36349e-03_rb, &
        & 0.36120e-03_rb,0.34300e-03_rb,0.30153e-03_rb,0.10818e-02_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.19375e-03_rb,0.30331e-03_rb,0.33858e-03_rb,0.35959e-03_rb,0.36963e-03_rb, &
        & 0.36736e-03_rb,0.35033e-03_rb,0.30927e-03_rb,0.10683e-02_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.19685e-03_rb,0.30858e-03_rb,0.34410e-03_rb,0.36599e-03_rb,0.37648e-03_rb, &
        & 0.37420e-03_rb,0.35748e-03_rb,0.31737e-03_rb,0.10704e-02_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.20091e-03_rb,0.31499e-03_rb,0.35151e-03_rb,0.37400e-03_rb,0.38421e-03_rb, &
        & 0.38207e-03_rb,0.36506e-03_rb,0.32617e-03_rb,0.10743e-02_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.20539e-03_rb,0.32294e-03_rb,0.36050e-03_rb,0.38325e-03_rb,0.39308e-03_rb, &
        & 0.39074e-03_rb,0.37406e-03_rb,0.33544e-03_rb,0.10696e-02_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.16667e-03_rb,0.26520e-03_rb,0.29671e-03_rb,0.31441e-03_rb,0.32093e-03_rb, &
        & 0.31659e-03_rb,0.30045e-03_rb,0.26340e-03_rb,0.11835e-02_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.16885e-03_rb,0.26948e-03_rb,0.30096e-03_rb,0.31914e-03_rb,0.32661e-03_rb, &
        & 0.32210e-03_rb,0.30657e-03_rb,0.27004e-03_rb,0.11814e-02_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.17223e-03_rb,0.27439e-03_rb,0.30661e-03_rb,0.32470e-03_rb,0.33272e-03_rb, &
        & 0.32842e-03_rb,0.31227e-03_rb,0.27654e-03_rb,0.11630e-02_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.17618e-03_rb,0.28049e-03_rb,0.31365e-03_rb,0.33181e-03_rb,0.33965e-03_rb, &
        & 0.33555e-03_rb,0.31903e-03_rb,0.28400e-03_rb,0.11485e-02_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.18103e-03_rb,0.28752e-03_rb,0.32181e-03_rb,0.34031e-03_rb,0.34776e-03_rb, &
        & 0.34353e-03_rb,0.32710e-03_rb,0.29221e-03_rb,0.11367e-02_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.14364e-03_rb,0.23191e-03_rb,0.26011e-03_rb,0.27602e-03_rb,0.28230e-03_rb, &
        & 0.27753e-03_rb,0.26259e-03_rb,0.22940e-03_rb,0.11242e-02_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.14586e-03_rb,0.23578e-03_rb,0.26464e-03_rb,0.28074e-03_rb,0.28728e-03_rb, &
        & 0.28268e-03_rb,0.26740e-03_rb,0.23511e-03_rb,0.11131e-02_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.14908e-03_rb,0.24042e-03_rb,0.27020e-03_rb,0.28659e-03_rb,0.29280e-03_rb, &
        & 0.28866e-03_rb,0.27254e-03_rb,0.24089e-03_rb,0.10996e-02_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.15297e-03_rb,0.24595e-03_rb,0.27661e-03_rb,0.29313e-03_rb,0.29926e-03_rb, &
        & 0.29513e-03_rb,0.27870e-03_rb,0.24706e-03_rb,0.10965e-02_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.15735e-03_rb,0.25242e-03_rb,0.28422e-03_rb,0.30069e-03_rb,0.30695e-03_rb, &
        & 0.30241e-03_rb,0.28613e-03_rb,0.25425e-03_rb,0.10839e-02_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.12345e-03_rb,0.20108e-03_rb,0.22674e-03_rb,0.24086e-03_rb,0.24638e-03_rb, &
        & 0.24238e-03_rb,0.22876e-03_rb,0.19931e-03_rb,0.96895e-03_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.12581e-03_rb,0.20480e-03_rb,0.23117e-03_rb,0.24535e-03_rb,0.25104e-03_rb, &
        & 0.24702e-03_rb,0.23303e-03_rb,0.20421e-03_rb,0.96294e-03_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.12856e-03_rb,0.20921e-03_rb,0.23606e-03_rb,0.25076e-03_rb,0.25631e-03_rb, &
        & 0.25243e-03_rb,0.23816e-03_rb,0.20920e-03_rb,0.95736e-03_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.13210e-03_rb,0.21428e-03_rb,0.24223e-03_rb,0.25691e-03_rb,0.26218e-03_rb, &
        & 0.25847e-03_rb,0.24400e-03_rb,0.21468e-03_rb,0.95389e-03_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.13599e-03_rb,0.22037e-03_rb,0.24914e-03_rb,0.26391e-03_rb,0.26911e-03_rb, &
        & 0.26543e-03_rb,0.25095e-03_rb,0.22101e-03_rb,0.94116e-03_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.10602e-02_rb,0.17123e-02_rb,0.20157e-02_rb,0.22277e-02_rb,0.23726e-02_rb, &
        & 0.24628e-02_rb,0.25069e-02_rb,0.24686e-02_rb,0.20168e-02_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.10672e-02_rb,0.17270e-02_rb,0.20414e-02_rb,0.22652e-02_rb,0.24211e-02_rb, &
        & 0.25212e-02_rb,0.25704e-02_rb,0.25272e-02_rb,0.20646e-02_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.10788e-02_rb,0.17509e-02_rb,0.20769e-02_rb,0.23112e-02_rb,0.24814e-02_rb, &
        & 0.25914e-02_rb,0.26418e-02_rb,0.25942e-02_rb,0.21193e-02_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.10874e-02_rb,0.17764e-02_rb,0.21176e-02_rb,0.23585e-02_rb,0.25381e-02_rb, &
        & 0.26622e-02_rb,0.27175e-02_rb,0.26689e-02_rb,0.21851e-02_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.11018e-02_rb,0.18090e-02_rb,0.21640e-02_rb,0.24172e-02_rb,0.26051e-02_rb, &
        & 0.27415e-02_rb,0.28075e-02_rb,0.27566e-02_rb,0.22604e-02_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.95075e-03_rb,0.15355e-02_rb,0.17976e-02_rb,0.19766e-02_rb,0.21006e-02_rb, &
        & 0.21710e-02_rb,0.21998e-02_rb,0.21663e-02_rb,0.17568e-02_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.96010e-03_rb,0.15501e-02_rb,0.18242e-02_rb,0.20112e-02_rb,0.21419e-02_rb, &
        & 0.22180e-02_rb,0.22475e-02_rb,0.22135e-02_rb,0.17960e-02_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.97337e-03_rb,0.15726e-02_rb,0.18580e-02_rb,0.20531e-02_rb,0.21905e-02_rb, &
        & 0.22742e-02_rb,0.23072e-02_rb,0.22697e-02_rb,0.18409e-02_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.98600e-03_rb,0.15975e-02_rb,0.18942e-02_rb,0.20984e-02_rb,0.22403e-02_rb, &
        & 0.23334e-02_rb,0.23735e-02_rb,0.23328e-02_rb,0.18922e-02_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.99685e-03_rb,0.16266e-02_rb,0.19353e-02_rb,0.21493e-02_rb,0.22931e-02_rb, &
        & 0.23955e-02_rb,0.24466e-02_rb,0.24068e-02_rb,0.19548e-02_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.85253e-03_rb,0.13731e-02_rb,0.15966e-02_rb,0.17555e-02_rb,0.18660e-02_rb, &
        & 0.19267e-02_rb,0.19415e-02_rb,0.18992e-02_rb,0.15125e-02_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.86131e-03_rb,0.13853e-02_rb,0.16172e-02_rb,0.17827e-02_rb,0.18992e-02_rb, &
        & 0.19616e-02_rb,0.19781e-02_rb,0.19388e-02_rb,0.15472e-02_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.87215e-03_rb,0.13990e-02_rb,0.16433e-02_rb,0.18151e-02_rb,0.19357e-02_rb, &
        & 0.20044e-02_rb,0.20244e-02_rb,0.19864e-02_rb,0.15842e-02_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.88704e-03_rb,0.14261e-02_rb,0.16795e-02_rb,0.18576e-02_rb,0.19802e-02_rb, &
        & 0.20536e-02_rb,0.20804e-02_rb,0.20410e-02_rb,0.16256e-02_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.89758e-03_rb,0.14527e-02_rb,0.17171e-02_rb,0.19025e-02_rb,0.20236e-02_rb, &
        & 0.21045e-02_rb,0.21413e-02_rb,0.21023e-02_rb,0.16764e-02_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.76168e-03_rb,0.12206e-02_rb,0.14175e-02_rb,0.15588e-02_rb,0.16575e-02_rb, &
        & 0.17098e-02_rb,0.17134e-02_rb,0.16603e-02_rb,0.12974e-02_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.76958e-03_rb,0.12314e-02_rb,0.14342e-02_rb,0.15807e-02_rb,0.16847e-02_rb, &
        & 0.17400e-02_rb,0.17442e-02_rb,0.16938e-02_rb,0.13260e-02_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.78002e-03_rb,0.12481e-02_rb,0.14573e-02_rb,0.16092e-02_rb,0.17173e-02_rb, &
        & 0.17771e-02_rb,0.17830e-02_rb,0.17337e-02_rb,0.13577e-02_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.79261e-03_rb,0.12694e-02_rb,0.14861e-02_rb,0.16420e-02_rb,0.17536e-02_rb, &
        & 0.18179e-02_rb,0.18286e-02_rb,0.17776e-02_rb,0.13930e-02_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.80905e-03_rb,0.12930e-02_rb,0.15216e-02_rb,0.16835e-02_rb,0.17969e-02_rb, &
        & 0.18651e-02_rb,0.18829e-02_rb,0.18324e-02_rb,0.14370e-02_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.67567e-03_rb,0.10847e-02_rb,0.12673e-02_rb,0.13865e-02_rb,0.14678e-02_rb, &
        & 0.15102e-02_rb,0.15099e-02_rb,0.14466e-02_rb,0.11079e-02_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.68234e-03_rb,0.10948e-02_rb,0.12810e-02_rb,0.14048e-02_rb,0.14909e-02_rb, &
        & 0.15364e-02_rb,0.15351e-02_rb,0.14749e-02_rb,0.11322e-02_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.69172e-03_rb,0.11094e-02_rb,0.12954e-02_rb,0.14275e-02_rb,0.15177e-02_rb, &
        & 0.15693e-02_rb,0.15680e-02_rb,0.15069e-02_rb,0.11581e-02_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.70397e-03_rb,0.11284e-02_rb,0.13174e-02_rb,0.14563e-02_rb,0.15501e-02_rb, &
        & 0.16053e-02_rb,0.16071e-02_rb,0.15445e-02_rb,0.11891e-02_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.72110e-03_rb,0.11540e-02_rb,0.13487e-02_rb,0.14917e-02_rb,0.15891e-02_rb, &
        & 0.16474e-02_rb,0.16536e-02_rb,0.15913e-02_rb,0.12262e-02_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.59508e-03_rb,0.96565e-03_rb,0.11271e-02_rb,0.12321e-02_rb,0.12935e-02_rb, &
        & 0.13263e-02_rb,0.13235e-02_rb,0.12583e-02_rb,0.94318e-03_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.60163e-03_rb,0.97555e-03_rb,0.11390e-02_rb,0.12428e-02_rb,0.13136e-02_rb, &
        & 0.13483e-02_rb,0.13443e-02_rb,0.12812e-02_rb,0.96040e-03_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.61035e-03_rb,0.98842e-03_rb,0.11551e-02_rb,0.12623e-02_rb,0.13370e-02_rb, &
        & 0.13759e-02_rb,0.13716e-02_rb,0.13073e-02_rb,0.98049e-03_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.62216e-03_rb,0.10038e-02_rb,0.11750e-02_rb,0.12870e-02_rb,0.13646e-02_rb, &
        & 0.14087e-02_rb,0.14042e-02_rb,0.13389e-02_rb,0.10049e-02_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.63568e-03_rb,0.10246e-02_rb,0.11986e-02_rb,0.13152e-02_rb,0.13969e-02_rb, &
        & 0.14439e-02_rb,0.14425e-02_rb,0.13760e-02_rb,0.10351e-02_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.52222e-03_rb,0.85609e-03_rb,0.99991e-03_rb,0.10870e-02_rb,0.11353e-02_rb, &
        & 0.11602e-02_rb,0.11537e-02_rb,0.10909e-02_rb,0.79969e-03_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.52839e-03_rb,0.86572e-03_rb,0.10120e-02_rb,0.11011e-02_rb,0.11521e-02_rb, &
        & 0.11786e-02_rb,0.11705e-02_rb,0.11093e-02_rb,0.81421e-03_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.53695e-03_rb,0.87703e-03_rb,0.10261e-02_rb,0.11177e-02_rb,0.11714e-02_rb, &
        & 0.12016e-02_rb,0.11930e-02_rb,0.11309e-02_rb,0.83157e-03_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.54734e-03_rb,0.89105e-03_rb,0.10435e-02_rb,0.11340e-02_rb,0.11954e-02_rb, &
        & 0.12276e-02_rb,0.12204e-02_rb,0.11559e-02_rb,0.85189e-03_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.56071e-03_rb,0.90902e-03_rb,0.10648e-02_rb,0.11575e-02_rb,0.12234e-02_rb, &
        & 0.12571e-02_rb,0.12523e-02_rb,0.11865e-02_rb,0.87630e-03_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.45809e-03_rb,0.75296e-03_rb,0.88022e-03_rb,0.95530e-03_rb,0.99775e-03_rb, &
        & 0.10107e-02_rb,0.99991e-03_rb,0.94255e-03_rb,0.69349e-03_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.46457e-03_rb,0.76241e-03_rb,0.89241e-03_rb,0.96833e-03_rb,0.10107e-02_rb, &
        & 0.10258e-02_rb,0.10136e-02_rb,0.95735e-03_rb,0.70413e-03_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.47249e-03_rb,0.77258e-03_rb,0.90578e-03_rb,0.98260e-03_rb,0.10233e-02_rb, &
        & 0.10448e-02_rb,0.10324e-02_rb,0.97481e-03_rb,0.71522e-03_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.48263e-03_rb,0.78537e-03_rb,0.92183e-03_rb,0.10012e-02_rb,0.10427e-02_rb, &
        & 0.10663e-02_rb,0.10558e-02_rb,0.99531e-03_rb,0.72922e-03_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.49501e-03_rb,0.80218e-03_rb,0.94059e-03_rb,0.10219e-02_rb,0.10669e-02_rb, &
        & 0.10907e-02_rb,0.10824e-02_rb,0.10204e-02_rb,0.74659e-03_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.40630e-03_rb,0.65697e-03_rb,0.76693e-03_rb,0.83253e-03_rb,0.86881e-03_rb, &
        & 0.87661e-03_rb,0.86393e-03_rb,0.81158e-03_rb,0.85763e-03_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.41166e-03_rb,0.66657e-03_rb,0.77941e-03_rb,0.84564e-03_rb,0.88174e-03_rb, &
        & 0.88952e-03_rb,0.87488e-03_rb,0.82351e-03_rb,0.84633e-03_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.41847e-03_rb,0.67680e-03_rb,0.79185e-03_rb,0.85875e-03_rb,0.89476e-03_rb, &
        & 0.90456e-03_rb,0.89101e-03_rb,0.83833e-03_rb,0.84017e-03_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.42661e-03_rb,0.68815e-03_rb,0.80637e-03_rb,0.87435e-03_rb,0.91036e-03_rb, &
        & 0.92205e-03_rb,0.90961e-03_rb,0.85502e-03_rb,0.83895e-03_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.43776e-03_rb,0.70278e-03_rb,0.82329e-03_rb,0.89247e-03_rb,0.92828e-03_rb, &
        & 0.94238e-03_rb,0.93168e-03_rb,0.87536e-03_rb,0.84088e-03_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.36854e-03_rb,0.57908e-03_rb,0.66869e-03_rb,0.72258e-03_rb,0.75231e-03_rb, &
        & 0.75942e-03_rb,0.74383e-03_rb,0.69709e-03_rb,0.20395e-02_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.37298e-03_rb,0.58711e-03_rb,0.68015e-03_rb,0.73349e-03_rb,0.76293e-03_rb, &
        & 0.77052e-03_rb,0.75338e-03_rb,0.70798e-03_rb,0.19202e-02_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.37876e-03_rb,0.59595e-03_rb,0.69100e-03_rb,0.74575e-03_rb,0.77497e-03_rb, &
        & 0.78295e-03_rb,0.76740e-03_rb,0.72017e-03_rb,0.18161e-02_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.38615e-03_rb,0.60708e-03_rb,0.70373e-03_rb,0.75914e-03_rb,0.78952e-03_rb, &
        & 0.79663e-03_rb,0.78331e-03_rb,0.73385e-03_rb,0.17247e-02_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.39564e-03_rb,0.62052e-03_rb,0.71856e-03_rb,0.77512e-03_rb,0.80699e-03_rb, &
        & 0.81444e-03_rb,0.80141e-03_rb,0.75081e-03_rb,0.16568e-02_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.33320e-03_rb,0.52088e-03_rb,0.59646e-03_rb,0.63812e-03_rb,0.65660e-03_rb, &
        & 0.65900e-03_rb,0.64138e-03_rb,0.60020e-03_rb,0.25696e-02_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.33882e-03_rb,0.52942e-03_rb,0.60584e-03_rb,0.64728e-03_rb,0.66639e-03_rb, &
        & 0.66995e-03_rb,0.65115e-03_rb,0.61030e-03_rb,0.24243e-02_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.34516e-03_rb,0.53993e-03_rb,0.61579e-03_rb,0.65783e-03_rb,0.67758e-03_rb, &
        & 0.68193e-03_rb,0.66413e-03_rb,0.62169e-03_rb,0.23408e-02_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.35301e-03_rb,0.55124e-03_rb,0.62753e-03_rb,0.66999e-03_rb,0.69148e-03_rb, &
        & 0.69508e-03_rb,0.67858e-03_rb,0.63426e-03_rb,0.22732e-02_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.36214e-03_rb,0.56453e-03_rb,0.64151e-03_rb,0.68481e-03_rb,0.70755e-03_rb, &
        & 0.71086e-03_rb,0.69483e-03_rb,0.64938e-03_rb,0.21869e-02_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.29653e-03_rb,0.46559e-03_rb,0.53073e-03_rb,0.56520e-03_rb,0.57894e-03_rb, &
        & 0.57686e-03_rb,0.55698e-03_rb,0.51584e-03_rb,0.26304e-02_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.30188e-03_rb,0.47447e-03_rb,0.54017e-03_rb,0.57406e-03_rb,0.58804e-03_rb, &
        & 0.58653e-03_rb,0.56630e-03_rb,0.52472e-03_rb,0.24930e-02_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.30861e-03_rb,0.48517e-03_rb,0.55102e-03_rb,0.58380e-03_rb,0.59890e-03_rb, &
        & 0.59725e-03_rb,0.57721e-03_rb,0.53486e-03_rb,0.23834e-02_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.31637e-03_rb,0.49701e-03_rb,0.56299e-03_rb,0.59618e-03_rb,0.61191e-03_rb, &
        & 0.60936e-03_rb,0.58930e-03_rb,0.54649e-03_rb,0.22824e-02_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.32604e-03_rb,0.51148e-03_rb,0.57604e-03_rb,0.61103e-03_rb,0.62611e-03_rb, &
        & 0.62412e-03_rb,0.60375e-03_rb,0.56026e-03_rb,0.22084e-02_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.26154e-03_rb,0.41366e-03_rb,0.47019e-03_rb,0.50056e-03_rb,0.51140e-03_rb, &
        & 0.50695e-03_rb,0.48611e-03_rb,0.44348e-03_rb,0.22773e-02_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.26669e-03_rb,0.42267e-03_rb,0.47957e-03_rb,0.51009e-03_rb,0.52067e-03_rb, &
        & 0.51611e-03_rb,0.49506e-03_rb,0.45138e-03_rb,0.21516e-02_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.27302e-03_rb,0.43290e-03_rb,0.49082e-03_rb,0.52081e-03_rb,0.53105e-03_rb, &
        & 0.52578e-03_rb,0.50476e-03_rb,0.46026e-03_rb,0.20522e-02_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.28114e-03_rb,0.44552e-03_rb,0.50271e-03_rb,0.53316e-03_rb,0.54314e-03_rb, &
        & 0.53754e-03_rb,0.51555e-03_rb,0.47096e-03_rb,0.19694e-02_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.29046e-03_rb,0.45904e-03_rb,0.51716e-03_rb,0.54751e-03_rb,0.55727e-03_rb, &
        & 0.55147e-03_rb,0.52828e-03_rb,0.48333e-03_rb,0.19102e-02_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.21025e-02_rb,0.35934e-02_rb,0.44505e-02_rb,0.51294e-02_rb,0.56565e-02_rb, &
        & 0.60638e-02_rb,0.63051e-02_rb,0.63060e-02_rb,0.53132e-02_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.21147e-02_rb,0.36363e-02_rb,0.45228e-02_rb,0.52203e-02_rb,0.57590e-02_rb, &
        & 0.61627e-02_rb,0.64080e-02_rb,0.64123e-02_rb,0.54186e-02_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.21423e-02_rb,0.36907e-02_rb,0.46129e-02_rb,0.53384e-02_rb,0.58809e-02_rb, &
        & 0.62832e-02_rb,0.65320e-02_rb,0.65359e-02_rb,0.55396e-02_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.21655e-02_rb,0.37303e-02_rb,0.46928e-02_rb,0.54454e-02_rb,0.60067e-02_rb, &
        & 0.64159e-02_rb,0.66758e-02_rb,0.66776e-02_rb,0.56738e-02_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.22041e-02_rb,0.37865e-02_rb,0.47900e-02_rb,0.55720e-02_rb,0.61575e-02_rb, &
        & 0.65795e-02_rb,0.68426e-02_rb,0.68332e-02_rb,0.58218e-02_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.19184e-02_rb,0.32868e-02_rb,0.40037e-02_rb,0.45520e-02_rb,0.49723e-02_rb, &
        & 0.52862e-02_rb,0.54779e-02_rb,0.54778e-02_rb,0.44696e-02_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.19302e-02_rb,0.33258e-02_rb,0.40693e-02_rb,0.46336e-02_rb,0.50634e-02_rb, &
        & 0.53829e-02_rb,0.55797e-02_rb,0.55784e-02_rb,0.45568e-02_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.19586e-02_rb,0.33762e-02_rb,0.41518e-02_rb,0.47383e-02_rb,0.51749e-02_rb, &
        & 0.54993e-02_rb,0.56986e-02_rb,0.56923e-02_rb,0.46572e-02_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.19876e-02_rb,0.34214e-02_rb,0.42297e-02_rb,0.48395e-02_rb,0.52957e-02_rb, &
        & 0.56305e-02_rb,0.58340e-02_rb,0.58219e-02_rb,0.47733e-02_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.20222e-02_rb,0.34660e-02_rb,0.43057e-02_rb,0.49428e-02_rb,0.54286e-02_rb, &
        & 0.57769e-02_rb,0.59846e-02_rb,0.59651e-02_rb,0.49022e-02_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.17389e-02_rb,0.29833e-02_rb,0.35894e-02_rb,0.40314e-02_rb,0.43535e-02_rb, &
        & 0.45848e-02_rb,0.47251e-02_rb,0.47273e-02_rb,0.37675e-02_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.17511e-02_rb,0.30210e-02_rb,0.36487e-02_rb,0.40996e-02_rb,0.44314e-02_rb, &
        & 0.46730e-02_rb,0.48141e-02_rb,0.48126e-02_rb,0.38333e-02_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.17713e-02_rb,0.30646e-02_rb,0.37133e-02_rb,0.41800e-02_rb,0.45216e-02_rb, &
        & 0.47732e-02_rb,0.49178e-02_rb,0.49079e-02_rb,0.39121e-02_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.18058e-02_rb,0.31149e-02_rb,0.37889e-02_rb,0.42777e-02_rb,0.46349e-02_rb, &
        & 0.48925e-02_rb,0.50383e-02_rb,0.50219e-02_rb,0.40071e-02_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.18379e-02_rb,0.31573e-02_rb,0.38545e-02_rb,0.43660e-02_rb,0.47479e-02_rb, &
        & 0.50186e-02_rb,0.51717e-02_rb,0.51483e-02_rb,0.41141e-02_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.15760e-02_rb,0.26881e-02_rb,0.32116e-02_rb,0.35706e-02_rb,0.38258e-02_rb, &
        & 0.39911e-02_rb,0.40833e-02_rb,0.40813e-02_rb,0.32298e-02_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.15813e-02_rb,0.27249e-02_rb,0.32627e-02_rb,0.36295e-02_rb,0.38889e-02_rb, &
        & 0.40640e-02_rb,0.41571e-02_rb,0.41534e-02_rb,0.32787e-02_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.16001e-02_rb,0.27603e-02_rb,0.33193e-02_rb,0.36971e-02_rb,0.39631e-02_rb, &
        & 0.41442e-02_rb,0.42435e-02_rb,0.42324e-02_rb,0.33394e-02_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.16258e-02_rb,0.28020e-02_rb,0.33780e-02_rb,0.37750e-02_rb,0.40494e-02_rb, &
        & 0.42369e-02_rb,0.43436e-02_rb,0.43281e-02_rb,0.34138e-02_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.16664e-02_rb,0.28640e-02_rb,0.34503e-02_rb,0.38637e-02_rb,0.41572e-02_rb, &
        & 0.43540e-02_rb,0.44611e-02_rb,0.44376e-02_rb,0.34982e-02_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.14233e-02_rb,0.24061e-02_rb,0.28484e-02_rb,0.31675e-02_rb,0.33803e-02_rb, &
        & 0.35027e-02_rb,0.35528e-02_rb,0.35417e-02_rb,0.27814e-02_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.14350e-02_rb,0.24395e-02_rb,0.28937e-02_rb,0.32168e-02_rb,0.34318e-02_rb, &
        & 0.35619e-02_rb,0.36148e-02_rb,0.35990e-02_rb,0.28208e-02_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.14521e-02_rb,0.24725e-02_rb,0.29512e-02_rb,0.32733e-02_rb,0.34917e-02_rb, &
        & 0.36258e-02_rb,0.36856e-02_rb,0.36654e-02_rb,0.28719e-02_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.14701e-02_rb,0.25117e-02_rb,0.30081e-02_rb,0.33364e-02_rb,0.35619e-02_rb, &
        & 0.37016e-02_rb,0.37673e-02_rb,0.37455e-02_rb,0.29344e-02_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.15037e-02_rb,0.25667e-02_rb,0.30753e-02_rb,0.34136e-02_rb,0.36518e-02_rb, &
        & 0.37975e-02_rb,0.38648e-02_rb,0.38367e-02_rb,0.30050e-02_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.12833e-02_rb,0.21397e-02_rb,0.25249e-02_rb,0.27989e-02_rb,0.29941e-02_rb, &
        & 0.30943e-02_rb,0.31134e-02_rb,0.30718e-02_rb,0.23837e-02_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.12955e-02_rb,0.21689e-02_rb,0.25640e-02_rb,0.28465e-02_rb,0.30386e-02_rb, &
        & 0.31442e-02_rb,0.31666e-02_rb,0.31216e-02_rb,0.24190e-02_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.13128e-02_rb,0.22001e-02_rb,0.26092e-02_rb,0.28949e-02_rb,0.30875e-02_rb, &
        & 0.31967e-02_rb,0.32254e-02_rb,0.31772e-02_rb,0.24633e-02_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.13321e-02_rb,0.22373e-02_rb,0.26583e-02_rb,0.29483e-02_rb,0.31454e-02_rb, &
        & 0.32584e-02_rb,0.32916e-02_rb,0.32451e-02_rb,0.25158e-02_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.13570e-02_rb,0.22801e-02_rb,0.27142e-02_rb,0.30110e-02_rb,0.32147e-02_rb, &
        & 0.33324e-02_rb,0.33699e-02_rb,0.33240e-02_rb,0.25751e-02_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.11500e-02_rb,0.18956e-02_rb,0.22352e-02_rb,0.24771e-02_rb,0.26462e-02_rb, &
        & 0.27285e-02_rb,0.27284e-02_rb,0.26574e-02_rb,0.20286e-02_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.11641e-02_rb,0.19221e-02_rb,0.22681e-02_rb,0.25137e-02_rb,0.26876e-02_rb, &
        & 0.27741e-02_rb,0.27762e-02_rb,0.27013e-02_rb,0.20553e-02_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.11813e-02_rb,0.19513e-02_rb,0.23065e-02_rb,0.25547e-02_rb,0.27305e-02_rb, &
        & 0.28210e-02_rb,0.28261e-02_rb,0.27499e-02_rb,0.20900e-02_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.12020e-02_rb,0.19854e-02_rb,0.23499e-02_rb,0.26064e-02_rb,0.27813e-02_rb, &
        & 0.28738e-02_rb,0.28824e-02_rb,0.28095e-02_rb,0.21311e-02_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.12251e-02_rb,0.20252e-02_rb,0.23984e-02_rb,0.26618e-02_rb,0.28401e-02_rb, &
        & 0.29374e-02_rb,0.29496e-02_rb,0.28780e-02_rb,0.21792e-02_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.10232e-02_rb,0.16906e-02_rb,0.19769e-02_rb,0.21832e-02_rb,0.23202e-02_rb, &
        & 0.23917e-02_rb,0.23852e-02_rb,0.22950e-02_rb,0.17101e-02_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.10391e-02_rb,0.17050e-02_rb,0.20071e-02_rb,0.22179e-02_rb,0.23606e-02_rb, &
        & 0.24337e-02_rb,0.24284e-02_rb,0.23335e-02_rb,0.17331e-02_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.10558e-02_rb,0.17304e-02_rb,0.20412e-02_rb,0.22560e-02_rb,0.24048e-02_rb, &
        & 0.24767e-02_rb,0.24718e-02_rb,0.23758e-02_rb,0.17638e-02_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.10757e-02_rb,0.17611e-02_rb,0.20788e-02_rb,0.22974e-02_rb,0.24509e-02_rb, &
        & 0.25242e-02_rb,0.25220e-02_rb,0.24275e-02_rb,0.17981e-02_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.10980e-02_rb,0.17974e-02_rb,0.21207e-02_rb,0.23464e-02_rb,0.25025e-02_rb, &
        & 0.25827e-02_rb,0.25792e-02_rb,0.24843e-02_rb,0.18371e-02_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.89817e-03_rb,0.14995e-02_rb,0.17424e-02_rb,0.19141e-02_rb,0.20277e-02_rb, &
        & 0.20847e-02_rb,0.20717e-02_rb,0.19779e-02_rb,0.16529e-02_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.91497e-03_rb,0.15202e-02_rb,0.17713e-02_rb,0.19473e-02_rb,0.20642e-02_rb, &
        & 0.21241e-02_rb,0.21112e-02_rb,0.20125e-02_rb,0.16590e-02_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.93269e-03_rb,0.15411e-02_rb,0.18029e-02_rb,0.19839e-02_rb,0.21017e-02_rb, &
        & 0.21633e-02_rb,0.21507e-02_rb,0.20494e-02_rb,0.16738e-02_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.95304e-03_rb,0.15646e-02_rb,0.18368e-02_rb,0.20225e-02_rb,0.21451e-02_rb, &
        & 0.22062e-02_rb,0.21951e-02_rb,0.20930e-02_rb,0.16939e-02_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.97498e-03_rb,0.15941e-02_rb,0.18742e-02_rb,0.20657e-02_rb,0.21958e-02_rb, &
        & 0.22577e-02_rb,0.22448e-02_rb,0.21409e-02_rb,0.17176e-02_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.79262e-03_rb,0.13163e-02_rb,0.15328e-02_rb,0.16707e-02_rb,0.17648e-02_rb, &
        & 0.18068e-02_rb,0.17935e-02_rb,0.17058e-02_rb,0.29700e-02_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.80762e-03_rb,0.13411e-02_rb,0.15596e-02_rb,0.17027e-02_rb,0.18000e-02_rb, &
        & 0.18435e-02_rb,0.18291e-02_rb,0.17355e-02_rb,0.28651e-02_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.82251e-03_rb,0.13646e-02_rb,0.15856e-02_rb,0.17370e-02_rb,0.18349e-02_rb, &
        & 0.18799e-02_rb,0.18643e-02_rb,0.17669e-02_rb,0.27637e-02_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.84077e-03_rb,0.13892e-02_rb,0.16170e-02_rb,0.17747e-02_rb,0.18736e-02_rb, &
        & 0.19201e-02_rb,0.19036e-02_rb,0.18042e-02_rb,0.26870e-02_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.86106e-03_rb,0.14180e-02_rb,0.16516e-02_rb,0.18147e-02_rb,0.19171e-02_rb, &
        & 0.19660e-02_rb,0.19468e-02_rb,0.18451e-02_rb,0.26342e-02_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.72744e-03_rb,0.11669e-02_rb,0.13481e-02_rb,0.14590e-02_rb,0.15393e-02_rb, &
        & 0.15747e-02_rb,0.15610e-02_rb,0.14772e-02_rb,0.38429e-02_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.73930e-03_rb,0.11904e-02_rb,0.13759e-02_rb,0.14894e-02_rb,0.15715e-02_rb, &
        & 0.16066e-02_rb,0.15925e-02_rb,0.15038e-02_rb,0.36308e-02_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.75350e-03_rb,0.12120e-02_rb,0.14044e-02_rb,0.15240e-02_rb,0.16053e-02_rb, &
        & 0.16390e-02_rb,0.16251e-02_rb,0.15339e-02_rb,0.34122e-02_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.76930e-03_rb,0.12355e-02_rb,0.14324e-02_rb,0.15603e-02_rb,0.16424e-02_rb, &
        & 0.16774e-02_rb,0.16611e-02_rb,0.15677e-02_rb,0.32391e-02_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.78824e-03_rb,0.12645e-02_rb,0.14660e-02_rb,0.15998e-02_rb,0.16846e-02_rb, &
        & 0.17204e-02_rb,0.17007e-02_rb,0.16046e-02_rb,0.31291e-02_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.66909e-03_rb,0.10517e-02_rb,0.11921e-02_rb,0.12822e-02_rb,0.13379e-02_rb, &
        & 0.13630e-02_rb,0.13492e-02_rb,0.12752e-02_rb,0.39255e-02_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.68372e-03_rb,0.10717e-02_rb,0.12175e-02_rb,0.13089e-02_rb,0.13666e-02_rb, &
        & 0.13913e-02_rb,0.13783e-02_rb,0.12999e-02_rb,0.37011e-02_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.69881e-03_rb,0.10912e-02_rb,0.12439e-02_rb,0.13402e-02_rb,0.13970e-02_rb, &
        & 0.14222e-02_rb,0.14099e-02_rb,0.13279e-02_rb,0.35002e-02_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.71487e-03_rb,0.11132e-02_rb,0.12732e-02_rb,0.13738e-02_rb,0.14317e-02_rb, &
        & 0.14583e-02_rb,0.14439e-02_rb,0.13590e-02_rb,0.33360e-02_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.73461e-03_rb,0.11382e-02_rb,0.13053e-02_rb,0.14102e-02_rb,0.14713e-02_rb, &
        & 0.14974e-02_rb,0.14808e-02_rb,0.13924e-02_rb,0.31844e-02_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.61282e-03_rb,0.95575e-03_rb,0.10730e-02_rb,0.11388e-02_rb,0.11718e-02_rb, &
        & 0.11795e-02_rb,0.11618e-02_rb,0.10974e-02_rb,0.34089e-02_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.62765e-03_rb,0.97500e-03_rb,0.10966e-02_rb,0.11619e-02_rb,0.11959e-02_rb, &
        & 0.12054e-02_rb,0.11876e-02_rb,0.11206e-02_rb,0.32141e-02_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.64409e-03_rb,0.99523e-03_rb,0.11197e-02_rb,0.11873e-02_rb,0.12235e-02_rb, &
        & 0.12347e-02_rb,0.12163e-02_rb,0.11468e-02_rb,0.30396e-02_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.66230e-03_rb,0.10180e-02_rb,0.11464e-02_rb,0.12161e-02_rb,0.12557e-02_rb, &
        & 0.12667e-02_rb,0.12475e-02_rb,0.11749e-02_rb,0.28911e-02_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.68383e-03_rb,0.10452e-02_rb,0.11767e-02_rb,0.12484e-02_rb,0.12902e-02_rb, &
        & 0.13021e-02_rb,0.12822e-02_rb,0.12049e-02_rb,0.27576e-02_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.59073e-02_rb,0.94320e-02_rb,0.12084e-01_rb,0.14049e-01_rb,0.15571e-01_rb, &
        & 0.16640e-01_rb,0.17123e-01_rb,0.16625e-01_rb,0.14851e-01_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.60213e-02_rb,0.95565e-02_rb,0.12218e-01_rb,0.14200e-01_rb,0.15736e-01_rb, &
        & 0.16810e-01_rb,0.17290e-01_rb,0.16722e-01_rb,0.14969e-01_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.61698e-02_rb,0.97176e-02_rb,0.12383e-01_rb,0.14381e-01_rb,0.15939e-01_rb, &
        & 0.17010e-01_rb,0.17476e-01_rb,0.16866e-01_rb,0.15129e-01_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.62862e-02_rb,0.98624e-02_rb,0.12544e-01_rb,0.14586e-01_rb,0.16163e-01_rb, &
        & 0.17225e-01_rb,0.17690e-01_rb,0.17043e-01_rb,0.15322e-01_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.64346e-02_rb,0.10033e-01_rb,0.12746e-01_rb,0.14817e-01_rb,0.16408e-01_rb, &
        & 0.17481e-01_rb,0.17940e-01_rb,0.17253e-01_rb,0.15543e-01_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.53740e-02_rb,0.87218e-02_rb,0.11053e-01_rb,0.12810e-01_rb,0.14176e-01_rb, &
        & 0.15120e-01_rb,0.15546e-01_rb,0.15059e-01_rb,0.12919e-01_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.54730e-02_rb,0.88522e-02_rb,0.11203e-01_rb,0.12984e-01_rb,0.14363e-01_rb, &
        & 0.15303e-01_rb,0.15712e-01_rb,0.15177e-01_rb,0.13038e-01_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.56074e-02_rb,0.90112e-02_rb,0.11386e-01_rb,0.13184e-01_rb,0.14577e-01_rb, &
        & 0.15506e-01_rb,0.15908e-01_rb,0.15341e-01_rb,0.13193e-01_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.57291e-02_rb,0.91663e-02_rb,0.11581e-01_rb,0.13408e-01_rb,0.14805e-01_rb, &
        & 0.15738e-01_rb,0.16140e-01_rb,0.15527e-01_rb,0.13373e-01_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.58374e-02_rb,0.93237e-02_rb,0.11790e-01_rb,0.13642e-01_rb,0.15063e-01_rb, &
        & 0.16008e-01_rb,0.16391e-01_rb,0.15740e-01_rb,0.13573e-01_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.49151e-02_rb,0.80083e-02_rb,0.10006e-01_rb,0.11528e-01_rb,0.12727e-01_rb, &
        & 0.13551e-01_rb,0.13911e-01_rb,0.13469e-01_rb,0.11070e-01_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.50045e-02_rb,0.81384e-02_rb,0.10158e-01_rb,0.11712e-01_rb,0.12918e-01_rb, &
        & 0.13727e-01_rb,0.14078e-01_rb,0.13598e-01_rb,0.11182e-01_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.51018e-02_rb,0.82774e-02_rb,0.10333e-01_rb,0.11906e-01_rb,0.13128e-01_rb, &
        & 0.13932e-01_rb,0.14276e-01_rb,0.13758e-01_rb,0.11318e-01_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.52290e-02_rb,0.84457e-02_rb,0.10545e-01_rb,0.12130e-01_rb,0.13362e-01_rb, &
        & 0.14179e-01_rb,0.14506e-01_rb,0.13944e-01_rb,0.11475e-01_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.53219e-02_rb,0.86004e-02_rb,0.10753e-01_rb,0.12365e-01_rb,0.13624e-01_rb, &
        & 0.14443e-01_rb,0.14743e-01_rb,0.14150e-01_rb,0.11664e-01_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.44996e-02_rb,0.73324e-02_rb,0.89976e-02_rb,0.10282e-01_rb,0.11291e-01_rb, &
        & 0.11989e-01_rb,0.12308e-01_rb,0.11915e-01_rb,0.94610e-02_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.45903e-02_rb,0.74608e-02_rb,0.91511e-02_rb,0.10467e-01_rb,0.11476e-01_rb, &
        & 0.12168e-01_rb,0.12474e-01_rb,0.12040e-01_rb,0.95583e-02_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.46786e-02_rb,0.75960e-02_rb,0.93200e-02_rb,0.10651e-01_rb,0.11685e-01_rb, &
        & 0.12375e-01_rb,0.12662e-01_rb,0.12197e-01_rb,0.96836e-02_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.47771e-02_rb,0.77407e-02_rb,0.95085e-02_rb,0.10860e-01_rb,0.11915e-01_rb, &
        & 0.12611e-01_rb,0.12868e-01_rb,0.12374e-01_rb,0.98243e-02_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.48920e-02_rb,0.79194e-02_rb,0.97305e-02_rb,0.11107e-01_rb,0.12174e-01_rb, &
        & 0.12866e-01_rb,0.13096e-01_rb,0.12573e-01_rb,0.99978e-02_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.41203e-02_rb,0.67057e-02_rb,0.80820e-02_rb,0.91247e-02_rb,0.99321e-02_rb, &
        & 0.10510e-01_rb,0.10783e-01_rb,0.10430e-01_rb,0.81020e-02_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.41993e-02_rb,0.68279e-02_rb,0.82322e-02_rb,0.92959e-02_rb,0.10115e-01_rb, &
        & 0.10684e-01_rb,0.10936e-01_rb,0.10557e-01_rb,0.81860e-02_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.42833e-02_rb,0.69588e-02_rb,0.83884e-02_rb,0.94726e-02_rb,0.10319e-01_rb, &
        & 0.10879e-01_rb,0.11113e-01_rb,0.10704e-01_rb,0.82915e-02_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.43800e-02_rb,0.70975e-02_rb,0.85585e-02_rb,0.96769e-02_rb,0.10538e-01_rb, &
        & 0.11098e-01_rb,0.11307e-01_rb,0.10873e-01_rb,0.84082e-02_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.44913e-02_rb,0.72638e-02_rb,0.87668e-02_rb,0.99106e-02_rb,0.10776e-01_rb, &
        & 0.11337e-01_rb,0.11521e-01_rb,0.11061e-01_rb,0.85492e-02_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.37432e-02_rb,0.61135e-02_rb,0.72507e-02_rb,0.80822e-02_rb,0.86984e-02_rb, &
        & 0.91424e-02_rb,0.93607e-02_rb,0.90800e-02_rb,0.69637e-02_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.38232e-02_rb,0.62277e-02_rb,0.73923e-02_rb,0.82383e-02_rb,0.88666e-02_rb, &
        & 0.93013e-02_rb,0.95051e-02_rb,0.91939e-02_rb,0.70311e-02_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.39028e-02_rb,0.63516e-02_rb,0.75330e-02_rb,0.84072e-02_rb,0.90542e-02_rb, &
        & 0.94860e-02_rb,0.96693e-02_rb,0.93306e-02_rb,0.71143e-02_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.39917e-02_rb,0.64850e-02_rb,0.76893e-02_rb,0.85935e-02_rb,0.92562e-02_rb, &
        & 0.96859e-02_rb,0.98481e-02_rb,0.94821e-02_rb,0.72080e-02_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.40851e-02_rb,0.66256e-02_rb,0.78694e-02_rb,0.87911e-02_rb,0.94733e-02_rb, &
        & 0.99019e-02_rb,0.10042e-01_rb,0.96464e-02_rb,0.73211e-02_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.33812e-02_rb,0.55475e-02_rb,0.65017e-02_rb,0.71556e-02_rb,0.76235e-02_rb, &
        & 0.79451e-02_rb,0.81101e-02_rb,0.78877e-02_rb,0.60070e-02_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.34581e-02_rb,0.56575e-02_rb,0.66301e-02_rb,0.72977e-02_rb,0.77709e-02_rb, &
        & 0.80903e-02_rb,0.82426e-02_rb,0.79917e-02_rb,0.60625e-02_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.35355e-02_rb,0.57759e-02_rb,0.67602e-02_rb,0.74532e-02_rb,0.79395e-02_rb, &
        & 0.82549e-02_rb,0.83890e-02_rb,0.81121e-02_rb,0.61274e-02_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.36191e-02_rb,0.58987e-02_rb,0.69034e-02_rb,0.76219e-02_rb,0.81209e-02_rb, &
        & 0.84343e-02_rb,0.85495e-02_rb,0.82395e-02_rb,0.62026e-02_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.37084e-02_rb,0.60308e-02_rb,0.70671e-02_rb,0.77977e-02_rb,0.83160e-02_rb, &
        & 0.86266e-02_rb,0.87205e-02_rb,0.83802e-02_rb,0.62947e-02_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.30376e-02_rb,0.49774e-02_rb,0.58172e-02_rb,0.63470e-02_rb,0.67038e-02_rb, &
        & 0.69396e-02_rb,0.70412e-02_rb,0.68509e-02_rb,0.51374e-02_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.31099e-02_rb,0.50996e-02_rb,0.59344e-02_rb,0.64736e-02_rb,0.68367e-02_rb, &
        & 0.70701e-02_rb,0.71564e-02_rb,0.69425e-02_rb,0.51813e-02_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.31853e-02_rb,0.52155e-02_rb,0.60541e-02_rb,0.66147e-02_rb,0.69858e-02_rb, &
        & 0.72120e-02_rb,0.72858e-02_rb,0.70465e-02_rb,0.52319e-02_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.32663e-02_rb,0.53312e-02_rb,0.61863e-02_rb,0.67654e-02_rb,0.71468e-02_rb, &
        & 0.73677e-02_rb,0.74252e-02_rb,0.71560e-02_rb,0.52932e-02_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.33510e-02_rb,0.54583e-02_rb,0.63345e-02_rb,0.69235e-02_rb,0.73170e-02_rb, &
        & 0.75339e-02_rb,0.75720e-02_rb,0.72770e-02_rb,0.53726e-02_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.27168e-02_rb,0.44406e-02_rb,0.51768e-02_rb,0.56339e-02_rb,0.59208e-02_rb, &
        & 0.60958e-02_rb,0.61438e-02_rb,0.59554e-02_rb,0.42511e-02_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.27853e-02_rb,0.45486e-02_rb,0.52870e-02_rb,0.57491e-02_rb,0.60397e-02_rb, &
        & 0.62076e-02_rb,0.62421e-02_rb,0.60358e-02_rb,0.42982e-02_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.28572e-02_rb,0.46609e-02_rb,0.53998e-02_rb,0.58755e-02_rb,0.61719e-02_rb, &
        & 0.63300e-02_rb,0.63536e-02_rb,0.61272e-02_rb,0.43479e-02_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.29351e-02_rb,0.47766e-02_rb,0.55244e-02_rb,0.60103e-02_rb,0.63121e-02_rb, &
        & 0.64656e-02_rb,0.64718e-02_rb,0.62231e-02_rb,0.44012e-02_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.30174e-02_rb,0.49013e-02_rb,0.56608e-02_rb,0.61542e-02_rb,0.64584e-02_rb, &
        & 0.66094e-02_rb,0.65960e-02_rb,0.63252e-02_rb,0.44655e-02_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.24055e-02_rb,0.39503e-02_rb,0.45833e-02_rb,0.49951e-02_rb,0.52507e-02_rb, &
        & 0.53857e-02_rb,0.53791e-02_rb,0.51629e-02_rb,0.55379e-02_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.24728e-02_rb,0.40487e-02_rb,0.46867e-02_rb,0.51042e-02_rb,0.53576e-02_rb, &
        & 0.54844e-02_rb,0.54704e-02_rb,0.52383e-02_rb,0.55398e-02_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.25454e-02_rb,0.41511e-02_rb,0.48010e-02_rb,0.52206e-02_rb,0.54751e-02_rb, &
        & 0.55927e-02_rb,0.55711e-02_rb,0.53216e-02_rb,0.55501e-02_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.26224e-02_rb,0.42542e-02_rb,0.49201e-02_rb,0.53438e-02_rb,0.55977e-02_rb, &
        & 0.57114e-02_rb,0.56753e-02_rb,0.54074e-02_rb,0.55558e-02_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.27039e-02_rb,0.43672e-02_rb,0.50470e-02_rb,0.54750e-02_rb,0.57276e-02_rb, &
        & 0.58363e-02_rb,0.57849e-02_rb,0.54996e-02_rb,0.55710e-02_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.21256e-02_rb,0.35221e-02_rb,0.40853e-02_rb,0.44612e-02_rb,0.46874e-02_rb, &
        & 0.47866e-02_rb,0.47389e-02_rb,0.44975e-02_rb,0.66135e-02_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.21937e-02_rb,0.36149e-02_rb,0.41854e-02_rb,0.45677e-02_rb,0.47907e-02_rb, &
        & 0.48833e-02_rb,0.48291e-02_rb,0.45707e-02_rb,0.65729e-02_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.22650e-02_rb,0.37098e-02_rb,0.42933e-02_rb,0.46764e-02_rb,0.49020e-02_rb, &
        & 0.49895e-02_rb,0.49234e-02_rb,0.46462e-02_rb,0.65598e-02_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.23425e-02_rb,0.38113e-02_rb,0.44107e-02_rb,0.47931e-02_rb,0.50165e-02_rb, &
        & 0.51000e-02_rb,0.50211e-02_rb,0.47251e-02_rb,0.65419e-02_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.24251e-02_rb,0.39222e-02_rb,0.45344e-02_rb,0.49164e-02_rb,0.51382e-02_rb, &
        & 0.52156e-02_rb,0.51233e-02_rb,0.48118e-02_rb,0.65404e-02_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.19014e-02_rb,0.31169e-02_rb,0.36336e-02_rb,0.39642e-02_rb,0.41602e-02_rb, &
        & 0.42352e-02_rb,0.41726e-02_rb,0.39183e-02_rb,0.68121e-02_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.19605e-02_rb,0.32068e-02_rb,0.37314e-02_rb,0.40676e-02_rb,0.42627e-02_rb, &
        & 0.43333e-02_rb,0.42591e-02_rb,0.39851e-02_rb,0.67229e-02_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.20227e-02_rb,0.33010e-02_rb,0.38333e-02_rb,0.41743e-02_rb,0.43708e-02_rb, &
        & 0.44355e-02_rb,0.43461e-02_rb,0.40520e-02_rb,0.66419e-02_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.20945e-02_rb,0.34006e-02_rb,0.39411e-02_rb,0.42851e-02_rb,0.44804e-02_rb, &
        & 0.45405e-02_rb,0.44353e-02_rb,0.41230e-02_rb,0.65948e-02_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.21695e-02_rb,0.35114e-02_rb,0.40581e-02_rb,0.44020e-02_rb,0.45960e-02_rb, &
        & 0.46501e-02_rb,0.45319e-02_rb,0.42045e-02_rb,0.65716e-02_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.17348e-02_rb,0.27575e-02_rb,0.32033e-02_rb,0.34905e-02_rb,0.36669e-02_rb, &
        & 0.37299e-02_rb,0.36615e-02_rb,0.34092e-02_rb,0.62173e-02_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.17868e-02_rb,0.28403e-02_rb,0.32978e-02_rb,0.35924e-02_rb,0.37684e-02_rb, &
        & 0.38248e-02_rb,0.37430e-02_rb,0.34686e-02_rb,0.61248e-02_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.18438e-02_rb,0.29272e-02_rb,0.33982e-02_rb,0.36985e-02_rb,0.38711e-02_rb, &
        & 0.39210e-02_rb,0.38240e-02_rb,0.35286e-02_rb,0.60637e-02_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.19066e-02_rb,0.30192e-02_rb,0.35016e-02_rb,0.38072e-02_rb,0.39764e-02_rb, &
        & 0.40199e-02_rb,0.39089e-02_rb,0.35954e-02_rb,0.60138e-02_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.19732e-02_rb,0.31237e-02_rb,0.36129e-02_rb,0.39207e-02_rb,0.40892e-02_rb, &
        & 0.41240e-02_rb,0.40022e-02_rb,0.36721e-02_rb,0.59944e-02_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.41219e-01_rb,0.41795e-01_rb,0.46848e-01_rb,0.50877e-01_rb,0.53437e-01_rb, &
        & 0.54488e-01_rb,0.53560e-01_rb,0.50235e-01_rb,0.51545e-01_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.42156e-01_rb,0.42406e-01_rb,0.47180e-01_rb,0.51006e-01_rb,0.53354e-01_rb, &
        & 0.54221e-01_rb,0.53185e-01_rb,0.49897e-01_rb,0.51289e-01_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.43082e-01_rb,0.43034e-01_rb,0.47543e-01_rb,0.51133e-01_rb,0.53289e-01_rb, &
        & 0.53978e-01_rb,0.52866e-01_rb,0.49591e-01_rb,0.51083e-01_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.43908e-01_rb,0.43606e-01_rb,0.47876e-01_rb,0.51241e-01_rb,0.53225e-01_rb, &
        & 0.53768e-01_rb,0.52555e-01_rb,0.49365e-01_rb,0.51010e-01_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.44717e-01_rb,0.44169e-01_rb,0.48160e-01_rb,0.51395e-01_rb,0.53206e-01_rb, &
        & 0.53607e-01_rb,0.52301e-01_rb,0.49268e-01_rb,0.51108e-01_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.38280e-01_rb,0.40368e-01_rb,0.46271e-01_rb,0.50387e-01_rb,0.53287e-01_rb, &
        & 0.54294e-01_rb,0.53147e-01_rb,0.49359e-01_rb,0.49444e-01_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.39278e-01_rb,0.41016e-01_rb,0.46659e-01_rb,0.50565e-01_rb,0.53249e-01_rb, &
        & 0.54064e-01_rb,0.52823e-01_rb,0.49016e-01_rb,0.49210e-01_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.40252e-01_rb,0.41678e-01_rb,0.47048e-01_rb,0.50745e-01_rb,0.53223e-01_rb, &
        & 0.53871e-01_rb,0.52519e-01_rb,0.48730e-01_rb,0.49081e-01_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.41132e-01_rb,0.42274e-01_rb,0.47387e-01_rb,0.50925e-01_rb,0.53214e-01_rb, &
        & 0.53693e-01_rb,0.52217e-01_rb,0.48559e-01_rb,0.49102e-01_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.41937e-01_rb,0.42806e-01_rb,0.47675e-01_rb,0.51115e-01_rb,0.53221e-01_rb, &
        & 0.53535e-01_rb,0.52007e-01_rb,0.48511e-01_rb,0.49285e-01_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.35016e-01_rb,0.38740e-01_rb,0.45035e-01_rb,0.49481e-01_rb,0.52517e-01_rb, &
        & 0.53496e-01_rb,0.52191e-01_rb,0.48044e-01_rb,0.46584e-01_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.36031e-01_rb,0.39393e-01_rb,0.45463e-01_rb,0.49713e-01_rb,0.52514e-01_rb, &
        & 0.53329e-01_rb,0.51912e-01_rb,0.47725e-01_rb,0.46413e-01_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.36991e-01_rb,0.40024e-01_rb,0.45863e-01_rb,0.49943e-01_rb,0.52531e-01_rb, &
        & 0.53167e-01_rb,0.51622e-01_rb,0.47472e-01_rb,0.46354e-01_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.37902e-01_rb,0.40620e-01_rb,0.46253e-01_rb,0.50181e-01_rb,0.52583e-01_rb, &
        & 0.53007e-01_rb,0.51384e-01_rb,0.47335e-01_rb,0.46441e-01_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.38707e-01_rb,0.41126e-01_rb,0.46598e-01_rb,0.50384e-01_rb,0.52633e-01_rb, &
        & 0.52879e-01_rb,0.51240e-01_rb,0.47311e-01_rb,0.46661e-01_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.31672e-01_rb,0.36966e-01_rb,0.43365e-01_rb,0.48073e-01_rb,0.51081e-01_rb, &
        & 0.52063e-01_rb,0.50596e-01_rb,0.46210e-01_rb,0.43228e-01_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.32685e-01_rb,0.37616e-01_rb,0.43827e-01_rb,0.48349e-01_rb,0.51147e-01_rb, &
        & 0.51967e-01_rb,0.50386e-01_rb,0.45960e-01_rb,0.43137e-01_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.33628e-01_rb,0.38241e-01_rb,0.44270e-01_rb,0.48639e-01_rb,0.51209e-01_rb, &
        & 0.51848e-01_rb,0.50172e-01_rb,0.45771e-01_rb,0.43147e-01_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.34503e-01_rb,0.38804e-01_rb,0.44698e-01_rb,0.48896e-01_rb,0.51278e-01_rb, &
        & 0.51724e-01_rb,0.50016e-01_rb,0.45686e-01_rb,0.43281e-01_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.35339e-01_rb,0.39319e-01_rb,0.45124e-01_rb,0.49160e-01_rb,0.51368e-01_rb, &
        & 0.51672e-01_rb,0.49945e-01_rb,0.45702e-01_rb,0.43520e-01_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.28453e-01_rb,0.34915e-01_rb,0.41400e-01_rb,0.46104e-01_rb,0.49014e-01_rb, &
        & 0.49952e-01_rb,0.48370e-01_rb,0.43932e-01_rb,0.39569e-01_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.29434e-01_rb,0.35588e-01_rb,0.41893e-01_rb,0.46437e-01_rb,0.49144e-01_rb, &
        & 0.49919e-01_rb,0.48254e-01_rb,0.43775e-01_rb,0.39548e-01_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.30340e-01_rb,0.36236e-01_rb,0.42373e-01_rb,0.46769e-01_rb,0.49255e-01_rb, &
        & 0.49873e-01_rb,0.48133e-01_rb,0.43667e-01_rb,0.39610e-01_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.31178e-01_rb,0.36810e-01_rb,0.42835e-01_rb,0.47061e-01_rb,0.49370e-01_rb, &
        & 0.49827e-01_rb,0.48071e-01_rb,0.43623e-01_rb,0.39779e-01_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.31999e-01_rb,0.37346e-01_rb,0.43284e-01_rb,0.47361e-01_rb,0.49516e-01_rb, &
        & 0.49857e-01_rb,0.48088e-01_rb,0.43666e-01_rb,0.40037e-01_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.25421e-01_rb,0.32626e-01_rb,0.39116e-01_rb,0.43609e-01_rb,0.46371e-01_rb, &
        & 0.47186e-01_rb,0.45634e-01_rb,0.41276e-01_rb,0.35722e-01_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.26347e-01_rb,0.33315e-01_rb,0.39629e-01_rb,0.44004e-01_rb,0.46570e-01_rb, &
        & 0.47251e-01_rb,0.45602e-01_rb,0.41217e-01_rb,0.35770e-01_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.27211e-01_rb,0.33965e-01_rb,0.40142e-01_rb,0.44371e-01_rb,0.46737e-01_rb, &
        & 0.47271e-01_rb,0.45579e-01_rb,0.41184e-01_rb,0.35883e-01_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.28013e-01_rb,0.34546e-01_rb,0.40641e-01_rb,0.44704e-01_rb,0.46903e-01_rb, &
        & 0.47305e-01_rb,0.45608e-01_rb,0.41201e-01_rb,0.36071e-01_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.28777e-01_rb,0.35080e-01_rb,0.41083e-01_rb,0.45018e-01_rb,0.47083e-01_rb, &
        & 0.47404e-01_rb,0.45685e-01_rb,0.41273e-01_rb,0.36328e-01_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.22628e-01_rb,0.30290e-01_rb,0.36520e-01_rb,0.40736e-01_rb,0.43195e-01_rb, &
        & 0.43894e-01_rb,0.42478e-01_rb,0.38357e-01_rb,0.31858e-01_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.23496e-01_rb,0.30981e-01_rb,0.37071e-01_rb,0.41159e-01_rb,0.43459e-01_rb, &
        & 0.44039e-01_rb,0.42534e-01_rb,0.38366e-01_rb,0.31954e-01_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.24311e-01_rb,0.31616e-01_rb,0.37614e-01_rb,0.41552e-01_rb,0.43690e-01_rb, &
        & 0.44135e-01_rb,0.42598e-01_rb,0.38398e-01_rb,0.32112e-01_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.25081e-01_rb,0.32194e-01_rb,0.38124e-01_rb,0.41906e-01_rb,0.43917e-01_rb, &
        & 0.44249e-01_rb,0.42700e-01_rb,0.38476e-01_rb,0.32312e-01_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.25796e-01_rb,0.32737e-01_rb,0.38583e-01_rb,0.42251e-01_rb,0.44147e-01_rb, &
        & 0.44425e-01_rb,0.42840e-01_rb,0.38587e-01_rb,0.32545e-01_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.20122e-01_rb,0.27995e-01_rb,0.33719e-01_rb,0.37485e-01_rb,0.39621e-01_rb, &
        & 0.40226e-01_rb,0.38971e-01_rb,0.35214e-01_rb,0.28110e-01_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.20930e-01_rb,0.28671e-01_rb,0.34304e-01_rb,0.37966e-01_rb,0.39957e-01_rb, &
        & 0.40447e-01_rb,0.39124e-01_rb,0.35304e-01_rb,0.28246e-01_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.21692e-01_rb,0.29291e-01_rb,0.34870e-01_rb,0.38392e-01_rb,0.40254e-01_rb, &
        & 0.40624e-01_rb,0.39266e-01_rb,0.35403e-01_rb,0.28424e-01_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.22396e-01_rb,0.29880e-01_rb,0.35388e-01_rb,0.38779e-01_rb,0.40528e-01_rb, &
        & 0.40815e-01_rb,0.39436e-01_rb,0.35538e-01_rb,0.28597e-01_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.23064e-01_rb,0.30402e-01_rb,0.35848e-01_rb,0.39151e-01_rb,0.40824e-01_rb, &
        & 0.41057e-01_rb,0.39639e-01_rb,0.35669e-01_rb,0.28823e-01_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.17909e-01_rb,0.25709e-01_rb,0.30840e-01_rb,0.34036e-01_rb,0.35851e-01_rb, &
        & 0.36360e-01_rb,0.35282e-01_rb,0.31947e-01_rb,0.24003e-01_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.18652e-01_rb,0.26369e-01_rb,0.31431e-01_rb,0.34542e-01_rb,0.36244e-01_rb, &
        & 0.36648e-01_rb,0.35513e-01_rb,0.32109e-01_rb,0.24164e-01_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.19344e-01_rb,0.26986e-01_rb,0.31997e-01_rb,0.35005e-01_rb,0.36599e-01_rb, &
        & 0.36904e-01_rb,0.35718e-01_rb,0.32277e-01_rb,0.24318e-01_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.19991e-01_rb,0.27565e-01_rb,0.32506e-01_rb,0.35425e-01_rb,0.36934e-01_rb, &
        & 0.37163e-01_rb,0.35949e-01_rb,0.32440e-01_rb,0.24470e-01_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.20615e-01_rb,0.28069e-01_rb,0.32961e-01_rb,0.35817e-01_rb,0.37281e-01_rb, &
        & 0.37462e-01_rb,0.36191e-01_rb,0.32591e-01_rb,0.24688e-01_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.16038e-01_rb,0.23520e-01_rb,0.27944e-01_rb,0.30612e-01_rb,0.32108e-01_rb, &
        & 0.32531e-01_rb,0.31594e-01_rb,0.28710e-01_rb,0.19040e-01_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.16702e-01_rb,0.24155e-01_rb,0.28537e-01_rb,0.31140e-01_rb,0.32544e-01_rb, &
        & 0.32858e-01_rb,0.31863e-01_rb,0.28938e-01_rb,0.19154e-01_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.17329e-01_rb,0.24747e-01_rb,0.29094e-01_rb,0.31618e-01_rb,0.32949e-01_rb, &
        & 0.33160e-01_rb,0.32119e-01_rb,0.29151e-01_rb,0.19214e-01_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.17922e-01_rb,0.25302e-01_rb,0.29595e-01_rb,0.32058e-01_rb,0.33330e-01_rb, &
        & 0.33472e-01_rb,0.32385e-01_rb,0.29320e-01_rb,0.19269e-01_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.18489e-01_rb,0.25799e-01_rb,0.30045e-01_rb,0.32466e-01_rb,0.33700e-01_rb, &
        & 0.33812e-01_rb,0.32629e-01_rb,0.29481e-01_rb,0.19405e-01_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.14687e-01_rb,0.21702e-01_rb,0.25384e-01_rb,0.27570e-01_rb,0.28760e-01_rb, &
        & 0.29017e-01_rb,0.28178e-01_rb,0.25716e-01_rb,0.20195e-01_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.15274e-01_rb,0.22296e-01_rb,0.25944e-01_rb,0.28087e-01_rb,0.29202e-01_rb, &
        & 0.29352e-01_rb,0.28471e-01_rb,0.25960e-01_rb,0.20265e-01_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.15835e-01_rb,0.22836e-01_rb,0.26465e-01_rb,0.28550e-01_rb,0.29611e-01_rb, &
        & 0.29694e-01_rb,0.28754e-01_rb,0.26152e-01_rb,0.20296e-01_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.16357e-01_rb,0.23338e-01_rb,0.26927e-01_rb,0.28982e-01_rb,0.29998e-01_rb, &
        & 0.30045e-01_rb,0.29008e-01_rb,0.26312e-01_rb,0.20131e-01_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.16879e-01_rb,0.23791e-01_rb,0.27359e-01_rb,0.29394e-01_rb,0.30387e-01_rb, &
        & 0.30372e-01_rb,0.29268e-01_rb,0.26507e-01_rb,0.20180e-01_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.13435e-01_rb,0.19917e-01_rb,0.22947e-01_rb,0.24741e-01_rb,0.25656e-01_rb, &
        & 0.25762e-01_rb,0.25010e-01_rb,0.22879e-01_rb,0.21935e-01_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.13973e-01_rb,0.20460e-01_rb,0.23475e-01_rb,0.25221e-01_rb,0.26084e-01_rb, &
        & 0.26126e-01_rb,0.25297e-01_rb,0.23091e-01_rb,0.22096e-01_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.14481e-01_rb,0.20956e-01_rb,0.23951e-01_rb,0.25665e-01_rb,0.26486e-01_rb, &
        & 0.26478e-01_rb,0.25558e-01_rb,0.23263e-01_rb,0.22257e-01_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.14966e-01_rb,0.21408e-01_rb,0.24388e-01_rb,0.26079e-01_rb,0.26872e-01_rb, &
        & 0.26802e-01_rb,0.25814e-01_rb,0.23444e-01_rb,0.22244e-01_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.15448e-01_rb,0.21834e-01_rb,0.24813e-01_rb,0.26492e-01_rb,0.27230e-01_rb, &
        & 0.27128e-01_rb,0.26103e-01_rb,0.23676e-01_rb,0.22318e-01_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.12253e-01_rb,0.18169e-01_rb,0.20712e-01_rb,0.22143e-01_rb,0.22829e-01_rb, &
        & 0.22840e-01_rb,0.22092e-01_rb,0.20210e-01_rb,0.22350e-01_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.12752e-01_rb,0.18675e-01_rb,0.21188e-01_rb,0.22590e-01_rb,0.23242e-01_rb, &
        & 0.23184e-01_rb,0.22356e-01_rb,0.20395e-01_rb,0.22569e-01_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.13220e-01_rb,0.19134e-01_rb,0.21627e-01_rb,0.23007e-01_rb,0.23621e-01_rb, &
        & 0.23502e-01_rb,0.22612e-01_rb,0.20571e-01_rb,0.22750e-01_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.13675e-01_rb,0.19566e-01_rb,0.22044e-01_rb,0.23405e-01_rb,0.23969e-01_rb, &
        & 0.23813e-01_rb,0.22875e-01_rb,0.20780e-01_rb,0.22726e-01_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.14122e-01_rb,0.19976e-01_rb,0.22454e-01_rb,0.23786e-01_rb,0.24318e-01_rb, &
        & 0.24140e-01_rb,0.23183e-01_rb,0.21041e-01_rb,0.22784e-01_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.14776e+00_rb,0.13250e+00_rb,0.11877e+00_rb,0.11907e+00_rb,0.11538e+00_rb, &
        & 0.11234e+00_rb,0.10926e+00_rb,0.11039e+00_rb,0.12102e+00_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.14758e+00_rb,0.13180e+00_rb,0.11760e+00_rb,0.11720e+00_rb,0.11379e+00_rb, &
        & 0.11206e+00_rb,0.10856e+00_rb,0.11029e+00_rb,0.12093e+00_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.14691e+00_rb,0.13078e+00_rb,0.11629e+00_rb,0.11563e+00_rb,0.11248e+00_rb, &
        & 0.11149e+00_rb,0.10785e+00_rb,0.11040e+00_rb,0.12104e+00_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.14609e+00_rb,0.12985e+00_rb,0.11503e+00_rb,0.11426e+00_rb,0.11150e+00_rb, &
        & 0.11061e+00_rb,0.10739e+00_rb,0.11031e+00_rb,0.12092e+00_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.14529e+00_rb,0.12911e+00_rb,0.11448e+00_rb,0.11310e+00_rb,0.11058e+00_rb, &
        & 0.10959e+00_rb,0.10721e+00_rb,0.11007e+00_rb,0.12050e+00_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.14616e+00_rb,0.13131e+00_rb,0.12350e+00_rb,0.12510e+00_rb,0.11934e+00_rb, &
        & 0.11762e+00_rb,0.11417e+00_rb,0.11460e+00_rb,0.12626e+00_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.14607e+00_rb,0.13063e+00_rb,0.12202e+00_rb,0.12312e+00_rb,0.11789e+00_rb, &
        & 0.11742e+00_rb,0.11363e+00_rb,0.11470e+00_rb,0.12636e+00_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.14577e+00_rb,0.12991e+00_rb,0.12074e+00_rb,0.12132e+00_rb,0.11671e+00_rb, &
        & 0.11676e+00_rb,0.11334e+00_rb,0.11448e+00_rb,0.12602e+00_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.14549e+00_rb,0.12948e+00_rb,0.11982e+00_rb,0.11971e+00_rb,0.11561e+00_rb, &
        & 0.11583e+00_rb,0.11311e+00_rb,0.11380e+00_rb,0.12515e+00_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.14518e+00_rb,0.12921e+00_rb,0.11930e+00_rb,0.11856e+00_rb,0.11453e+00_rb, &
        & 0.11499e+00_rb,0.11278e+00_rb,0.11303e+00_rb,0.12425e+00_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.14318e+00_rb,0.12893e+00_rb,0.12918e+00_rb,0.12942e+00_rb,0.12491e+00_rb, &
        & 0.12238e+00_rb,0.11842e+00_rb,0.11706e+00_rb,0.12892e+00_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.14362e+00_rb,0.12877e+00_rb,0.12794e+00_rb,0.12738e+00_rb,0.12367e+00_rb, &
        & 0.12214e+00_rb,0.11818e+00_rb,0.11730e+00_rb,0.12892e+00_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.14392e+00_rb,0.12859e+00_rb,0.12673e+00_rb,0.12555e+00_rb,0.12238e+00_rb, &
        & 0.12154e+00_rb,0.11804e+00_rb,0.11693e+00_rb,0.12825e+00_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.14418e+00_rb,0.12858e+00_rb,0.12588e+00_rb,0.12406e+00_rb,0.12111e+00_rb, &
        & 0.12100e+00_rb,0.11768e+00_rb,0.11605e+00_rb,0.12723e+00_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.14433e+00_rb,0.12868e+00_rb,0.12520e+00_rb,0.12307e+00_rb,0.11981e+00_rb, &
        & 0.12050e+00_rb,0.11702e+00_rb,0.11519e+00_rb,0.12642e+00_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.13900e+00_rb,0.12627e+00_rb,0.13262e+00_rb,0.13246e+00_rb,0.13018e+00_rb, &
        & 0.12656e+00_rb,0.12207e+00_rb,0.11856e+00_rb,0.12927e+00_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.13981e+00_rb,0.12631e+00_rb,0.13168e+00_rb,0.13063e+00_rb,0.12928e+00_rb, &
        & 0.12634e+00_rb,0.12180e+00_rb,0.11842e+00_rb,0.12888e+00_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.14055e+00_rb,0.12635e+00_rb,0.13064e+00_rb,0.12886e+00_rb,0.12826e+00_rb, &
        & 0.12591e+00_rb,0.12147e+00_rb,0.11781e+00_rb,0.12798e+00_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.14115e+00_rb,0.12659e+00_rb,0.12979e+00_rb,0.12765e+00_rb,0.12712e+00_rb, &
        & 0.12539e+00_rb,0.12094e+00_rb,0.11686e+00_rb,0.12709e+00_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.14205e+00_rb,0.12716e+00_rb,0.12897e+00_rb,0.12678e+00_rb,0.12607e+00_rb, &
        & 0.12463e+00_rb,0.12022e+00_rb,0.11611e+00_rb,0.12648e+00_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.13267e+00_rb,0.12451e+00_rb,0.13264e+00_rb,0.13467e+00_rb,0.13410e+00_rb, &
        & 0.12976e+00_rb,0.12532e+00_rb,0.11842e+00_rb,0.12663e+00_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.13391e+00_rb,0.12448e+00_rb,0.13222e+00_rb,0.13349e+00_rb,0.13365e+00_rb, &
        & 0.12967e+00_rb,0.12478e+00_rb,0.11793e+00_rb,0.12610e+00_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.13513e+00_rb,0.12441e+00_rb,0.13150e+00_rb,0.13226e+00_rb,0.13285e+00_rb, &
        & 0.12935e+00_rb,0.12414e+00_rb,0.11714e+00_rb,0.12535e+00_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.13635e+00_rb,0.12468e+00_rb,0.13071e+00_rb,0.13132e+00_rb,0.13192e+00_rb, &
        & 0.12883e+00_rb,0.12325e+00_rb,0.11628e+00_rb,0.12465e+00_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.13765e+00_rb,0.12523e+00_rb,0.12989e+00_rb,0.13067e+00_rb,0.13104e+00_rb, &
        & 0.12778e+00_rb,0.12230e+00_rb,0.11574e+00_rb,0.12447e+00_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.12480e+00_rb,0.12304e+00_rb,0.13086e+00_rb,0.13535e+00_rb,0.13580e+00_rb, &
        & 0.13244e+00_rb,0.12684e+00_rb,0.11687e+00_rb,0.12150e+00_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.12631e+00_rb,0.12320e+00_rb,0.13089e+00_rb,0.13488e+00_rb,0.13591e+00_rb, &
        & 0.13240e+00_rb,0.12644e+00_rb,0.11610e+00_rb,0.12096e+00_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.12783e+00_rb,0.12334e+00_rb,0.13031e+00_rb,0.13415e+00_rb,0.13555e+00_rb, &
        & 0.13212e+00_rb,0.12571e+00_rb,0.11517e+00_rb,0.12033e+00_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.12946e+00_rb,0.12375e+00_rb,0.12962e+00_rb,0.13360e+00_rb,0.13480e+00_rb, &
        & 0.13147e+00_rb,0.12458e+00_rb,0.11440e+00_rb,0.12012e+00_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.13098e+00_rb,0.12424e+00_rb,0.12926e+00_rb,0.13328e+00_rb,0.13390e+00_rb, &
        & 0.13035e+00_rb,0.12352e+00_rb,0.11410e+00_rb,0.12034e+00_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.11587e+00_rb,0.11977e+00_rb,0.12827e+00_rb,0.13413e+00_rb,0.13637e+00_rb, &
        & 0.13338e+00_rb,0.12607e+00_rb,0.11369e+00_rb,0.11422e+00_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.11755e+00_rb,0.12008e+00_rb,0.12851e+00_rb,0.13433e+00_rb,0.13653e+00_rb, &
        & 0.13350e+00_rb,0.12579e+00_rb,0.11303e+00_rb,0.11384e+00_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.11925e+00_rb,0.12045e+00_rb,0.12827e+00_rb,0.13401e+00_rb,0.13627e+00_rb, &
        & 0.13335e+00_rb,0.12531e+00_rb,0.11227e+00_rb,0.11342e+00_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.12111e+00_rb,0.12107e+00_rb,0.12798e+00_rb,0.13385e+00_rb,0.13564e+00_rb, &
        & 0.13268e+00_rb,0.12424e+00_rb,0.11167e+00_rb,0.11355e+00_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.12285e+00_rb,0.12168e+00_rb,0.12798e+00_rb,0.13376e+00_rb,0.13482e+00_rb, &
        & 0.13151e+00_rb,0.12334e+00_rb,0.11149e+00_rb,0.11415e+00_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.10591e+00_rb,0.11393e+00_rb,0.12456e+00_rb,0.13177e+00_rb,0.13532e+00_rb, &
        & 0.13189e+00_rb,0.12388e+00_rb,0.10965e+00_rb,0.10563e+00_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.10784e+00_rb,0.11446e+00_rb,0.12497e+00_rb,0.13191e+00_rb,0.13527e+00_rb, &
        & 0.13206e+00_rb,0.12372e+00_rb,0.10901e+00_rb,0.10541e+00_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.10982e+00_rb,0.11499e+00_rb,0.12496e+00_rb,0.13190e+00_rb,0.13491e+00_rb, &
        & 0.13198e+00_rb,0.12329e+00_rb,0.10836e+00_rb,0.10528e+00_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.11183e+00_rb,0.11569e+00_rb,0.12499e+00_rb,0.13184e+00_rb,0.13435e+00_rb, &
        & 0.13149e+00_rb,0.12252e+00_rb,0.10787e+00_rb,0.10590e+00_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.11351e+00_rb,0.11638e+00_rb,0.12535e+00_rb,0.13176e+00_rb,0.13360e+00_rb, &
        & 0.13054e+00_rb,0.12191e+00_rb,0.10801e+00_rb,0.10661e+00_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.95227e-01_rb,0.10714e+00_rb,0.11944e+00_rb,0.12826e+00_rb,0.13131e+00_rb, &
        & 0.12801e+00_rb,0.11973e+00_rb,0.10506e+00_rb,0.96127e-01_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.97380e-01_rb,0.10792e+00_rb,0.12010e+00_rb,0.12848e+00_rb,0.13131e+00_rb, &
        & 0.12823e+00_rb,0.11988e+00_rb,0.10446e+00_rb,0.96013e-01_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.99589e-01_rb,0.10861e+00_rb,0.12043e+00_rb,0.12826e+00_rb,0.13099e+00_rb, &
        & 0.12821e+00_rb,0.11964e+00_rb,0.10385e+00_rb,0.96337e-01_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.10172e+00_rb,0.10937e+00_rb,0.12073e+00_rb,0.12822e+00_rb,0.13048e+00_rb, &
        & 0.12786e+00_rb,0.11915e+00_rb,0.10369e+00_rb,0.97324e-01_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.10340e+00_rb,0.11015e+00_rb,0.12136e+00_rb,0.12832e+00_rb,0.12998e+00_rb, &
        & 0.12724e+00_rb,0.11892e+00_rb,0.10410e+00_rb,0.97908e-01_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.84549e-01_rb,0.10005e+00_rb,0.11370e+00_rb,0.12280e+00_rb,0.12503e+00_rb, &
        & 0.12157e+00_rb,0.11395e+00_rb,0.99558e-01_rb,0.78463e-01_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.86895e-01_rb,0.10114e+00_rb,0.11438e+00_rb,0.12270e+00_rb,0.12494e+00_rb, &
        & 0.12211e+00_rb,0.11447e+00_rb,0.98932e-01_rb,0.78293e-01_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.89280e-01_rb,0.10200e+00_rb,0.11501e+00_rb,0.12255e+00_rb,0.12466e+00_rb, &
        & 0.12232e+00_rb,0.11449e+00_rb,0.98490e-01_rb,0.78963e-01_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.91373e-01_rb,0.10297e+00_rb,0.11555e+00_rb,0.12264e+00_rb,0.12433e+00_rb, &
        & 0.12211e+00_rb,0.11429e+00_rb,0.98808e-01_rb,0.79894e-01_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.93116e-01_rb,0.10381e+00_rb,0.11616e+00_rb,0.12299e+00_rb,0.12420e+00_rb, &
        & 0.12176e+00_rb,0.11463e+00_rb,0.99400e-01_rb,0.79228e-01_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.75544e-01_rb,0.93054e-01_rb,0.10724e+00_rb,0.11489e+00_rb,0.11666e+00_rb, &
        & 0.11411e+00_rb,0.10739e+00_rb,0.92708e-01_rb,0.54552e-01_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.77908e-01_rb,0.94287e-01_rb,0.10808e+00_rb,0.11471e+00_rb,0.11661e+00_rb, &
        & 0.11478e+00_rb,0.10772e+00_rb,0.92504e-01_rb,0.54848e-01_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.80077e-01_rb,0.95473e-01_rb,0.10868e+00_rb,0.11491e+00_rb,0.11655e+00_rb, &
        & 0.11482e+00_rb,0.10782e+00_rb,0.93001e-01_rb,0.55769e-01_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.81960e-01_rb,0.96566e-01_rb,0.10943e+00_rb,0.11546e+00_rb,0.11672e+00_rb, &
        & 0.11473e+00_rb,0.10831e+00_rb,0.93879e-01_rb,0.57926e-01_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.83450e-01_rb,0.97453e-01_rb,0.11008e+00_rb,0.11599e+00_rb,0.11716e+00_rb, &
        & 0.11515e+00_rb,0.10911e+00_rb,0.94288e-01_rb,0.57746e-01_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.67513e-01_rb,0.86189e-01_rb,0.99369e-01_rb,0.10520e+00_rb,0.10724e+00_rb, &
        & 0.10586e+00_rb,0.99242e-01_rb,0.85556e-01_rb,0.53156e-01_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.69613e-01_rb,0.87484e-01_rb,0.10007e+00_rb,0.10560e+00_rb,0.10749e+00_rb, &
        & 0.10614e+00_rb,0.99761e-01_rb,0.86270e-01_rb,0.51781e-01_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.71469e-01_rb,0.88722e-01_rb,0.10094e+00_rb,0.10629e+00_rb,0.10784e+00_rb, &
        & 0.10630e+00_rb,0.10046e+00_rb,0.87349e-01_rb,0.51450e-01_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.73024e-01_rb,0.89773e-01_rb,0.10183e+00_rb,0.10713e+00_rb,0.10860e+00_rb, &
        & 0.10686e+00_rb,0.10138e+00_rb,0.88130e-01_rb,0.52601e-01_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.74303e-01_rb,0.90696e-01_rb,0.10259e+00_rb,0.10797e+00_rb,0.10971e+00_rb, &
        & 0.10792e+00_rb,0.10184e+00_rb,0.88445e-01_rb,0.51152e-01_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.60529e-01_rb,0.79607e-01_rb,0.90318e-01_rb,0.95428e-01_rb,0.97408e-01_rb, &
        & 0.96227e-01_rb,0.90699e-01_rb,0.78779e-01_rb,0.52832e-01_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.62276e-01_rb,0.80870e-01_rb,0.91247e-01_rb,0.96259e-01_rb,0.97934e-01_rb, &
        & 0.96714e-01_rb,0.91680e-01_rb,0.79941e-01_rb,0.52303e-01_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.63790e-01_rb,0.81933e-01_rb,0.92274e-01_rb,0.97270e-01_rb,0.98880e-01_rb, &
        & 0.97553e-01_rb,0.92636e-01_rb,0.80918e-01_rb,0.51576e-01_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.65028e-01_rb,0.82921e-01_rb,0.93287e-01_rb,0.98387e-01_rb,0.10023e+00_rb, &
        & 0.98785e-01_rb,0.93367e-01_rb,0.81491e-01_rb,0.53387e-01_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.66150e-01_rb,0.83739e-01_rb,0.94354e-01_rb,0.99696e-01_rb,0.10169e+00_rb, &
        & 0.99901e-01_rb,0.93863e-01_rb,0.81907e-01_rb,0.52837e-01_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.20407e+00_rb,0.18071e+00_rb,0.16029e+00_rb,0.15138e+00_rb,0.14997e+00_rb, &
        & 0.13984e+00_rb,0.13275e+00_rb,0.14129e+00_rb,0.15399e+00_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.20135e+00_rb,0.17935e+00_rb,0.15932e+00_rb,0.15017e+00_rb,0.14962e+00_rb, &
        & 0.13862e+00_rb,0.13266e+00_rb,0.14117e+00_rb,0.15393e+00_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.19936e+00_rb,0.17801e+00_rb,0.15787e+00_rb,0.14825e+00_rb,0.14805e+00_rb, &
        & 0.13791e+00_rb,0.13257e+00_rb,0.14065e+00_rb,0.15357e+00_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.19744e+00_rb,0.17636e+00_rb,0.15587e+00_rb,0.14600e+00_rb,0.14568e+00_rb, &
        & 0.13759e+00_rb,0.13216e+00_rb,0.14003e+00_rb,0.15311e+00_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.19560e+00_rb,0.17445e+00_rb,0.15370e+00_rb,0.14366e+00_rb,0.14353e+00_rb, &
        & 0.13719e+00_rb,0.13142e+00_rb,0.13943e+00_rb,0.15271e+00_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.21063e+00_rb,0.18653e+00_rb,0.16583e+00_rb,0.16261e+00_rb,0.15745e+00_rb, &
        & 0.14703e+00_rb,0.14143e+00_rb,0.14831e+00_rb,0.16307e+00_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.20807e+00_rb,0.18515e+00_rb,0.16450e+00_rb,0.16174e+00_rb,0.15680e+00_rb, &
        & 0.14604e+00_rb,0.14115e+00_rb,0.14798e+00_rb,0.16304e+00_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.20581e+00_rb,0.18338e+00_rb,0.16244e+00_rb,0.15978e+00_rb,0.15520e+00_rb, &
        & 0.14566e+00_rb,0.14067e+00_rb,0.14768e+00_rb,0.16312e+00_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.20347e+00_rb,0.18119e+00_rb,0.16010e+00_rb,0.15745e+00_rb,0.15333e+00_rb, &
        & 0.14529e+00_rb,0.14007e+00_rb,0.14753e+00_rb,0.16343e+00_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.20115e+00_rb,0.17885e+00_rb,0.15773e+00_rb,0.15483e+00_rb,0.15150e+00_rb, &
        & 0.14453e+00_rb,0.13948e+00_rb,0.14731e+00_rb,0.16359e+00_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.21616e+00_rb,0.19157e+00_rb,0.17353e+00_rb,0.17142e+00_rb,0.16301e+00_rb, &
        & 0.15566e+00_rb,0.14998e+00_rb,0.15514e+00_rb,0.17145e+00_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.21408e+00_rb,0.19026e+00_rb,0.17182e+00_rb,0.17122e+00_rb,0.16252e+00_rb, &
        & 0.15476e+00_rb,0.14939e+00_rb,0.15448e+00_rb,0.17139e+00_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.21163e+00_rb,0.18824e+00_rb,0.16950e+00_rb,0.16996e+00_rb,0.16151e+00_rb, &
        & 0.15419e+00_rb,0.14882e+00_rb,0.15406e+00_rb,0.17163e+00_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.20914e+00_rb,0.18593e+00_rb,0.16692e+00_rb,0.16757e+00_rb,0.16029e+00_rb, &
        & 0.15344e+00_rb,0.14837e+00_rb,0.15392e+00_rb,0.17198e+00_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.20701e+00_rb,0.18381e+00_rb,0.16453e+00_rb,0.16472e+00_rb,0.15897e+00_rb, &
        & 0.15236e+00_rb,0.14801e+00_rb,0.15351e+00_rb,0.17179e+00_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.21862e+00_rb,0.19379e+00_rb,0.18221e+00_rb,0.17807e+00_rb,0.16954e+00_rb, &
        & 0.16407e+00_rb,0.15829e+00_rb,0.16056e+00_rb,0.17755e+00_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.21705e+00_rb,0.19285e+00_rb,0.18086e+00_rb,0.17812e+00_rb,0.16909e+00_rb, &
        & 0.16313e+00_rb,0.15767e+00_rb,0.16003e+00_rb,0.17784e+00_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.21534e+00_rb,0.19129e+00_rb,0.17879e+00_rb,0.17719e+00_rb,0.16821e+00_rb, &
        & 0.16252e+00_rb,0.15706e+00_rb,0.15944e+00_rb,0.17796e+00_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.21357e+00_rb,0.18961e+00_rb,0.17644e+00_rb,0.17503e+00_rb,0.16720e+00_rb, &
        & 0.16179e+00_rb,0.15644e+00_rb,0.15869e+00_rb,0.17755e+00_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.21192e+00_rb,0.18796e+00_rb,0.17436e+00_rb,0.17237e+00_rb,0.16599e+00_rb, &
        & 0.16096e+00_rb,0.15564e+00_rb,0.15746e+00_rb,0.17656e+00_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.21793e+00_rb,0.19325e+00_rb,0.18965e+00_rb,0.18402e+00_rb,0.17692e+00_rb, &
        & 0.17250e+00_rb,0.16494e+00_rb,0.16474e+00_rb,0.18189e+00_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.21706e+00_rb,0.19287e+00_rb,0.18835e+00_rb,0.18355e+00_rb,0.17627e+00_rb, &
        & 0.17161e+00_rb,0.16474e+00_rb,0.16398e+00_rb,0.18195e+00_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.21612e+00_rb,0.19181e+00_rb,0.18687e+00_rb,0.18213e+00_rb,0.17553e+00_rb, &
        & 0.17088e+00_rb,0.16425e+00_rb,0.16313e+00_rb,0.18152e+00_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.21478e+00_rb,0.19045e+00_rb,0.18500e+00_rb,0.18018e+00_rb,0.17440e+00_rb, &
        & 0.16996e+00_rb,0.16342e+00_rb,0.16188e+00_rb,0.18054e+00_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.21350e+00_rb,0.18909e+00_rb,0.18333e+00_rb,0.17782e+00_rb,0.17309e+00_rb, &
        & 0.16916e+00_rb,0.16213e+00_rb,0.16034e+00_rb,0.17917e+00_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.21342e+00_rb,0.19093e+00_rb,0.19390e+00_rb,0.18953e+00_rb,0.18412e+00_rb, &
        & 0.17932e+00_rb,0.17133e+00_rb,0.16726e+00_rb,0.18355e+00_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.21345e+00_rb,0.19062e+00_rb,0.19295e+00_rb,0.18836e+00_rb,0.18309e+00_rb, &
        & 0.17848e+00_rb,0.17069e+00_rb,0.16650e+00_rb,0.18323e+00_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.21341e+00_rb,0.18999e+00_rb,0.19180e+00_rb,0.18672e+00_rb,0.18231e+00_rb, &
        & 0.17805e+00_rb,0.16996e+00_rb,0.16523e+00_rb,0.18230e+00_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.21278e+00_rb,0.18888e+00_rb,0.19035e+00_rb,0.18488e+00_rb,0.18150e+00_rb, &
        & 0.17740e+00_rb,0.16888e+00_rb,0.16361e+00_rb,0.18086e+00_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.21199e+00_rb,0.18788e+00_rb,0.18853e+00_rb,0.18312e+00_rb,0.18053e+00_rb, &
        & 0.17647e+00_rb,0.16753e+00_rb,0.16185e+00_rb,0.17942e+00_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.20554e+00_rb,0.18880e+00_rb,0.19483e+00_rb,0.19351e+00_rb,0.18975e+00_rb, &
        & 0.18476e+00_rb,0.17703e+00_rb,0.16712e+00_rb,0.18079e+00_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.20652e+00_rb,0.18855e+00_rb,0.19393e+00_rb,0.19243e+00_rb,0.18938e+00_rb, &
        & 0.18427e+00_rb,0.17598e+00_rb,0.16627e+00_rb,0.18028e+00_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.20722e+00_rb,0.18798e+00_rb,0.19279e+00_rb,0.19112e+00_rb,0.18883e+00_rb, &
        & 0.18376e+00_rb,0.17458e+00_rb,0.16493e+00_rb,0.17920e+00_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.20704e+00_rb,0.18700e+00_rb,0.19152e+00_rb,0.18972e+00_rb,0.18818e+00_rb, &
        & 0.18325e+00_rb,0.17326e+00_rb,0.16331e+00_rb,0.17789e+00_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.20670e+00_rb,0.18595e+00_rb,0.18990e+00_rb,0.18822e+00_rb,0.18756e+00_rb, &
        & 0.18244e+00_rb,0.17179e+00_rb,0.16146e+00_rb,0.17654e+00_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.19482e+00_rb,0.18644e+00_rb,0.19372e+00_rb,0.19534e+00_rb,0.19341e+00_rb, &
        & 0.18978e+00_rb,0.17991e+00_rb,0.16480e+00_rb,0.17385e+00_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.19645e+00_rb,0.18662e+00_rb,0.19286e+00_rb,0.19499e+00_rb,0.19364e+00_rb, &
        & 0.18939e+00_rb,0.17872e+00_rb,0.16377e+00_rb,0.17323e+00_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.19752e+00_rb,0.18632e+00_rb,0.19176e+00_rb,0.19416e+00_rb,0.19361e+00_rb, &
        & 0.18898e+00_rb,0.17730e+00_rb,0.16248e+00_rb,0.17231e+00_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.19786e+00_rb,0.18535e+00_rb,0.19058e+00_rb,0.19340e+00_rb,0.19352e+00_rb, &
        & 0.18844e+00_rb,0.17582e+00_rb,0.16092e+00_rb,0.17110e+00_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.19827e+00_rb,0.18472e+00_rb,0.18922e+00_rb,0.19254e+00_rb,0.19302e+00_rb, &
        & 0.18739e+00_rb,0.17431e+00_rb,0.15936e+00_rb,0.17034e+00_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.18199e+00_rb,0.18200e+00_rb,0.19084e+00_rb,0.19456e+00_rb,0.19545e+00_rb, &
        & 0.19216e+00_rb,0.18018e+00_rb,0.16033e+00_rb,0.16333e+00_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.18402e+00_rb,0.18227e+00_rb,0.19032e+00_rb,0.19515e+00_rb,0.19643e+00_rb, &
        & 0.19212e+00_rb,0.17883e+00_rb,0.15954e+00_rb,0.16306e+00_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.18527e+00_rb,0.18204e+00_rb,0.18956e+00_rb,0.19541e+00_rb,0.19690e+00_rb, &
        & 0.19201e+00_rb,0.17754e+00_rb,0.15865e+00_rb,0.16240e+00_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.18597e+00_rb,0.18146e+00_rb,0.18866e+00_rb,0.19510e+00_rb,0.19689e+00_rb, &
        & 0.19145e+00_rb,0.17626e+00_rb,0.15729e+00_rb,0.16157e+00_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.18701e+00_rb,0.18150e+00_rb,0.18776e+00_rb,0.19449e+00_rb,0.19637e+00_rb, &
        & 0.19035e+00_rb,0.17490e+00_rb,0.15605e+00_rb,0.16154e+00_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.16786e+00_rb,0.17459e+00_rb,0.18572e+00_rb,0.19196e+00_rb,0.19497e+00_rb, &
        & 0.19093e+00_rb,0.17775e+00_rb,0.15445e+00_rb,0.12845e+00_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.16990e+00_rb,0.17470e+00_rb,0.18604e+00_rb,0.19354e+00_rb,0.19670e+00_rb, &
        & 0.19124e+00_rb,0.17643e+00_rb,0.15415e+00_rb,0.12953e+00_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.17115e+00_rb,0.17447e+00_rb,0.18577e+00_rb,0.19420e+00_rb,0.19734e+00_rb, &
        & 0.19133e+00_rb,0.17544e+00_rb,0.15343e+00_rb,0.13019e+00_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.17256e+00_rb,0.17421e+00_rb,0.18540e+00_rb,0.19426e+00_rb,0.19741e+00_rb, &
        & 0.19105e+00_rb,0.17463e+00_rb,0.15245e+00_rb,0.13135e+00_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.17411e+00_rb,0.17469e+00_rb,0.18532e+00_rb,0.19400e+00_rb,0.19690e+00_rb, &
        & 0.19024e+00_rb,0.17373e+00_rb,0.15177e+00_rb,0.13443e+00_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.15326e+00_rb,0.16530e+00_rb,0.17933e+00_rb,0.18873e+00_rb,0.19204e+00_rb, &
        & 0.18645e+00_rb,0.17157e+00_rb,0.14842e+00_rb,0.10906e+00_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.15510e+00_rb,0.16508e+00_rb,0.18002e+00_rb,0.19054e+00_rb,0.19337e+00_rb, &
        & 0.18697e+00_rb,0.17132e+00_rb,0.14823e+00_rb,0.10821e+00_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.15695e+00_rb,0.16517e+00_rb,0.18033e+00_rb,0.19125e+00_rb,0.19395e+00_rb, &
        & 0.18719e+00_rb,0.17114e+00_rb,0.14748e+00_rb,0.10735e+00_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.15884e+00_rb,0.16584e+00_rb,0.18066e+00_rb,0.19107e+00_rb,0.19386e+00_rb, &
        & 0.18714e+00_rb,0.17083e+00_rb,0.14679e+00_rb,0.10767e+00_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.16065e+00_rb,0.16693e+00_rb,0.18136e+00_rb,0.19090e+00_rb,0.19313e+00_rb, &
        & 0.18652e+00_rb,0.17043e+00_rb,0.14693e+00_rb,0.10852e+00_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.13812e+00_rb,0.15482e+00_rb,0.17200e+00_rb,0.18322e+00_rb,0.18516e+00_rb, &
        & 0.17855e+00_rb,0.16404e+00_rb,0.14149e+00_rb,0.85521e-01_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.14024e+00_rb,0.15523e+00_rb,0.17295e+00_rb,0.18464e+00_rb,0.18616e+00_rb, &
        & 0.17947e+00_rb,0.16474e+00_rb,0.14107e+00_rb,0.84251e-01_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.14232e+00_rb,0.15625e+00_rb,0.17374e+00_rb,0.18482e+00_rb,0.18666e+00_rb, &
        & 0.18015e+00_rb,0.16507e+00_rb,0.14067e+00_rb,0.83221e-01_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.14452e+00_rb,0.15760e+00_rb,0.17483e+00_rb,0.18482e+00_rb,0.18644e+00_rb, &
        & 0.18031e+00_rb,0.16530e+00_rb,0.14085e+00_rb,0.83052e-01_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.14659e+00_rb,0.15898e+00_rb,0.17608e+00_rb,0.18496e+00_rb,0.18608e+00_rb, &
        & 0.18007e+00_rb,0.16596e+00_rb,0.14156e+00_rb,0.86150e-01_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.12335e+00_rb,0.14397e+00_rb,0.16332e+00_rb,0.17403e+00_rb,0.17485e+00_rb, &
        & 0.16858e+00_rb,0.15532e+00_rb,0.13315e+00_rb,0.86982e-01_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.12553e+00_rb,0.14531e+00_rb,0.16473e+00_rb,0.17477e+00_rb,0.17593e+00_rb, &
        & 0.16998e+00_rb,0.15639e+00_rb,0.13322e+00_rb,0.83311e-01_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.12788e+00_rb,0.14705e+00_rb,0.16612e+00_rb,0.17511e+00_rb,0.17644e+00_rb, &
        & 0.17090e+00_rb,0.15730e+00_rb,0.13374e+00_rb,0.80969e-01_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.13027e+00_rb,0.14889e+00_rb,0.16746e+00_rb,0.17569e+00_rb,0.17660e+00_rb, &
        & 0.17136e+00_rb,0.15852e+00_rb,0.13468e+00_rb,0.79671e-01_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.13255e+00_rb,0.15069e+00_rb,0.16867e+00_rb,0.17661e+00_rb,0.17696e+00_rb, &
        & 0.17204e+00_rb,0.15992e+00_rb,0.13520e+00_rb,0.81334e-01_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.28721e+00_rb,0.25323e+00_rb,0.21917e+00_rb,0.19050e+00_rb,0.18439e+00_rb, &
        & 0.17842e+00_rb,0.17156e+00_rb,0.18245e+00_rb,0.20075e+00_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.28561e+00_rb,0.25177e+00_rb,0.21845e+00_rb,0.19052e+00_rb,0.18309e+00_rb, &
        & 0.17638e+00_rb,0.16966e+00_rb,0.18096e+00_rb,0.19965e+00_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.28230e+00_rb,0.24910e+00_rb,0.21697e+00_rb,0.19014e+00_rb,0.18233e+00_rb, &
        & 0.17414e+00_rb,0.16821e+00_rb,0.18034e+00_rb,0.19910e+00_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.27834e+00_rb,0.24592e+00_rb,0.21540e+00_rb,0.18935e+00_rb,0.18163e+00_rb, &
        & 0.17186e+00_rb,0.16739e+00_rb,0.18033e+00_rb,0.19922e+00_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.27434e+00_rb,0.24303e+00_rb,0.21391e+00_rb,0.18803e+00_rb,0.18019e+00_rb, &
        & 0.17025e+00_rb,0.16693e+00_rb,0.18042e+00_rb,0.19937e+00_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.29623e+00_rb,0.26097e+00_rb,0.22609e+00_rb,0.20293e+00_rb,0.20182e+00_rb, &
        & 0.19085e+00_rb,0.18240e+00_rb,0.19397e+00_rb,0.21604e+00_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.29432e+00_rb,0.25931e+00_rb,0.22539e+00_rb,0.20142e+00_rb,0.20026e+00_rb, &
        & 0.18828e+00_rb,0.18057e+00_rb,0.19299e+00_rb,0.21521e+00_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.29132e+00_rb,0.25712e+00_rb,0.22457e+00_rb,0.20070e+00_rb,0.19917e+00_rb, &
        & 0.18573e+00_rb,0.17946e+00_rb,0.19254e+00_rb,0.21482e+00_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.28797e+00_rb,0.25474e+00_rb,0.22345e+00_rb,0.19986e+00_rb,0.19780e+00_rb, &
        & 0.18370e+00_rb,0.17875e+00_rb,0.19238e+00_rb,0.21459e+00_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.28466e+00_rb,0.25242e+00_rb,0.22205e+00_rb,0.19838e+00_rb,0.19562e+00_rb, &
        & 0.18242e+00_rb,0.17806e+00_rb,0.19203e+00_rb,0.21421e+00_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.30561e+00_rb,0.26889e+00_rb,0.23295e+00_rb,0.21956e+00_rb,0.21717e+00_rb, &
        & 0.20307e+00_rb,0.19524e+00_rb,0.20742e+00_rb,0.23388e+00_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.30324e+00_rb,0.26704e+00_rb,0.23237e+00_rb,0.21738e+00_rb,0.21493e+00_rb, &
        & 0.20036e+00_rb,0.19375e+00_rb,0.20659e+00_rb,0.23304e+00_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.30058e+00_rb,0.26531e+00_rb,0.23190e+00_rb,0.21598e+00_rb,0.21300e+00_rb, &
        & 0.19790e+00_rb,0.19244e+00_rb,0.20627e+00_rb,0.23259e+00_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.29826e+00_rb,0.26386e+00_rb,0.23125e+00_rb,0.21516e+00_rb,0.21099e+00_rb, &
        & 0.19626e+00_rb,0.19135e+00_rb,0.20570e+00_rb,0.23176e+00_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.29569e+00_rb,0.26204e+00_rb,0.22972e+00_rb,0.21357e+00_rb,0.20846e+00_rb, &
        & 0.19500e+00_rb,0.19003e+00_rb,0.20504e+00_rb,0.23084e+00_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.31580e+00_rb,0.27765e+00_rb,0.24256e+00_rb,0.23825e+00_rb,0.23010e+00_rb, &
        & 0.21605e+00_rb,0.20908e+00_rb,0.22073e+00_rb,0.25027e+00_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.31385e+00_rb,0.27619e+00_rb,0.24143e+00_rb,0.23561e+00_rb,0.22732e+00_rb, &
        & 0.21355e+00_rb,0.20732e+00_rb,0.21997e+00_rb,0.24945e+00_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.31115e+00_rb,0.27450e+00_rb,0.24049e+00_rb,0.23383e+00_rb,0.22480e+00_rb, &
        & 0.21136e+00_rb,0.20583e+00_rb,0.21971e+00_rb,0.24898e+00_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.30851e+00_rb,0.27268e+00_rb,0.23896e+00_rb,0.23220e+00_rb,0.22243e+00_rb, &
        & 0.20992e+00_rb,0.20459e+00_rb,0.21945e+00_rb,0.24850e+00_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.30578e+00_rb,0.27071e+00_rb,0.23717e+00_rb,0.23033e+00_rb,0.22018e+00_rb, &
        & 0.20870e+00_rb,0.20353e+00_rb,0.21927e+00_rb,0.24814e+00_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.32525e+00_rb,0.28587e+00_rb,0.25583e+00_rb,0.25377e+00_rb,0.24260e+00_rb, &
        & 0.22945e+00_rb,0.22249e+00_rb,0.23280e+00_rb,0.26422e+00_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.32339e+00_rb,0.28450e+00_rb,0.25406e+00_rb,0.25143e+00_rb,0.23950e+00_rb, &
        & 0.22738e+00_rb,0.22044e+00_rb,0.23211e+00_rb,0.26348e+00_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.32075e+00_rb,0.28298e+00_rb,0.25200e+00_rb,0.24974e+00_rb,0.23665e+00_rb, &
        & 0.22565e+00_rb,0.21882e+00_rb,0.23178e+00_rb,0.26317e+00_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.31838e+00_rb,0.28134e+00_rb,0.24992e+00_rb,0.24807e+00_rb,0.23442e+00_rb, &
        & 0.22435e+00_rb,0.21780e+00_rb,0.23180e+00_rb,0.26307e+00_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.31581e+00_rb,0.27952e+00_rb,0.24774e+00_rb,0.24618e+00_rb,0.23239e+00_rb, &
        & 0.22315e+00_rb,0.21723e+00_rb,0.23168e+00_rb,0.26269e+00_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.33335e+00_rb,0.29286e+00_rb,0.27107e+00_rb,0.26707e+00_rb,0.25627e+00_rb, &
        & 0.24388e+00_rb,0.23516e+00_rb,0.24254e+00_rb,0.27495e+00_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.33185e+00_rb,0.29196e+00_rb,0.26927e+00_rb,0.26528e+00_rb,0.25338e+00_rb, &
        & 0.24199e+00_rb,0.23321e+00_rb,0.24178e+00_rb,0.27452e+00_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.32960e+00_rb,0.29070e+00_rb,0.26757e+00_rb,0.26387e+00_rb,0.25062e+00_rb, &
        & 0.24015e+00_rb,0.23158e+00_rb,0.24188e+00_rb,0.27485e+00_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.32714e+00_rb,0.28920e+00_rb,0.26543e+00_rb,0.26229e+00_rb,0.24828e+00_rb, &
        & 0.23863e+00_rb,0.23085e+00_rb,0.24191e+00_rb,0.27476e+00_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.32443e+00_rb,0.28722e+00_rb,0.26300e+00_rb,0.25993e+00_rb,0.24627e+00_rb, &
        & 0.23716e+00_rb,0.23011e+00_rb,0.24150e+00_rb,0.27410e+00_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.33805e+00_rb,0.29686e+00_rb,0.28577e+00_rb,0.27897e+00_rb,0.26998e+00_rb, &
        & 0.25892e+00_rb,0.24589e+00_rb,0.25046e+00_rb,0.28292e+00_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.33717e+00_rb,0.29673e+00_rb,0.28505e+00_rb,0.27748e+00_rb,0.26727e+00_rb, &
        & 0.25668e+00_rb,0.24439e+00_rb,0.24991e+00_rb,0.28315e+00_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.33582e+00_rb,0.29639e+00_rb,0.28414e+00_rb,0.27664e+00_rb,0.26503e+00_rb, &
        & 0.25454e+00_rb,0.24355e+00_rb,0.24982e+00_rb,0.28339e+00_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.33418e+00_rb,0.29565e+00_rb,0.28243e+00_rb,0.27502e+00_rb,0.26319e+00_rb, &
        & 0.25262e+00_rb,0.24314e+00_rb,0.24937e+00_rb,0.28297e+00_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.33223e+00_rb,0.29432e+00_rb,0.28036e+00_rb,0.27276e+00_rb,0.26126e+00_rb, &
        & 0.25114e+00_rb,0.24223e+00_rb,0.24874e+00_rb,0.28224e+00_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.33811e+00_rb,0.29881e+00_rb,0.29770e+00_rb,0.29087e+00_rb,0.28271e+00_rb, &
        & 0.27088e+00_rb,0.25606e+00_rb,0.25702e+00_rb,0.28869e+00_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.33858e+00_rb,0.29933e+00_rb,0.29817e+00_rb,0.28968e+00_rb,0.28063e+00_rb, &
        & 0.26913e+00_rb,0.25508e+00_rb,0.25647e+00_rb,0.28886e+00_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.33853e+00_rb,0.29979e+00_rb,0.29803e+00_rb,0.28900e+00_rb,0.27895e+00_rb, &
        & 0.26698e+00_rb,0.25454e+00_rb,0.25561e+00_rb,0.28857e+00_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.33786e+00_rb,0.29958e+00_rb,0.29705e+00_rb,0.28721e+00_rb,0.27754e+00_rb, &
        & 0.26522e+00_rb,0.25386e+00_rb,0.25480e+00_rb,0.28812e+00_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.33672e+00_rb,0.29874e+00_rb,0.29538e+00_rb,0.28492e+00_rb,0.27593e+00_rb, &
        & 0.26422e+00_rb,0.25288e+00_rb,0.25337e+00_rb,0.28657e+00_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.33304e+00_rb,0.29965e+00_rb,0.30451e+00_rb,0.30108e+00_rb,0.29380e+00_rb, &
        & 0.28100e+00_rb,0.26620e+00_rb,0.26062e+00_rb,0.28923e+00_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.33493e+00_rb,0.30099e+00_rb,0.30590e+00_rb,0.30051e+00_rb,0.29241e+00_rb, &
        & 0.27971e+00_rb,0.26553e+00_rb,0.25927e+00_rb,0.28890e+00_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.33639e+00_rb,0.30210e+00_rb,0.30672e+00_rb,0.29976e+00_rb,0.29135e+00_rb, &
        & 0.27789e+00_rb,0.26459e+00_rb,0.25779e+00_rb,0.28847e+00_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.33691e+00_rb,0.30238e+00_rb,0.30636e+00_rb,0.29831e+00_rb,0.29055e+00_rb, &
        & 0.27684e+00_rb,0.26372e+00_rb,0.25629e+00_rb,0.28756e+00_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.33680e+00_rb,0.30184e+00_rb,0.30474e+00_rb,0.29664e+00_rb,0.28906e+00_rb, &
        & 0.27593e+00_rb,0.26250e+00_rb,0.25436e+00_rb,0.28576e+00_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.32265e+00_rb,0.29931e+00_rb,0.30809e+00_rb,0.30788e+00_rb,0.30155e+00_rb, &
        & 0.29032e+00_rb,0.27398e+00_rb,0.25958e+00_rb,0.28232e+00_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.32643e+00_rb,0.30217e+00_rb,0.30992e+00_rb,0.30804e+00_rb,0.30109e+00_rb, &
        & 0.28947e+00_rb,0.27364e+00_rb,0.25761e+00_rb,0.28165e+00_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.32903e+00_rb,0.30409e+00_rb,0.31077e+00_rb,0.30846e+00_rb,0.30095e+00_rb, &
        & 0.28833e+00_rb,0.27259e+00_rb,0.25601e+00_rb,0.28124e+00_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.33021e+00_rb,0.30472e+00_rb,0.31062e+00_rb,0.30777e+00_rb,0.30050e+00_rb, &
        & 0.28725e+00_rb,0.27132e+00_rb,0.25418e+00_rb,0.28034e+00_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.33035e+00_rb,0.30379e+00_rb,0.30948e+00_rb,0.30647e+00_rb,0.29984e+00_rb, &
        & 0.28648e+00_rb,0.26959e+00_rb,0.25216e+00_rb,0.27891e+00_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.31092e+00_rb,0.29828e+00_rb,0.30944e+00_rb,0.31067e+00_rb,0.30618e+00_rb, &
        & 0.29698e+00_rb,0.27822e+00_rb,0.25367e+00_rb,0.20521e+00_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.31508e+00_rb,0.30179e+00_rb,0.31084e+00_rb,0.31247e+00_rb,0.30712e+00_rb, &
        & 0.29665e+00_rb,0.27717e+00_rb,0.25137e+00_rb,0.20791e+00_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.31745e+00_rb,0.30351e+00_rb,0.31148e+00_rb,0.31329e+00_rb,0.30764e+00_rb, &
        & 0.29592e+00_rb,0.27601e+00_rb,0.24946e+00_rb,0.21041e+00_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.31864e+00_rb,0.30345e+00_rb,0.31137e+00_rb,0.31328e+00_rb,0.30818e+00_rb, &
        & 0.29525e+00_rb,0.27444e+00_rb,0.24795e+00_rb,0.21312e+00_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.31936e+00_rb,0.30268e+00_rb,0.31034e+00_rb,0.31260e+00_rb,0.30827e+00_rb, &
        & 0.29465e+00_rb,0.27259e+00_rb,0.24615e+00_rb,0.21621e+00_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.29534e+00_rb,0.29381e+00_rb,0.30706e+00_rb,0.31081e+00_rb,0.30893e+00_rb, &
        & 0.29961e+00_rb,0.27811e+00_rb,0.24533e+00_rb,0.16496e+00_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.29914e+00_rb,0.29664e+00_rb,0.30890e+00_rb,0.31347e+00_rb,0.31102e+00_rb, &
        & 0.29958e+00_rb,0.27683e+00_rb,0.24361e+00_rb,0.16812e+00_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.30164e+00_rb,0.29790e+00_rb,0.30986e+00_rb,0.31529e+00_rb,0.31277e+00_rb, &
        & 0.29960e+00_rb,0.27547e+00_rb,0.24230e+00_rb,0.17163e+00_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.30318e+00_rb,0.29799e+00_rb,0.30963e+00_rb,0.31596e+00_rb,0.31358e+00_rb, &
        & 0.29964e+00_rb,0.27410e+00_rb,0.24090e+00_rb,0.17207e+00_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.30462e+00_rb,0.29774e+00_rb,0.30883e+00_rb,0.31580e+00_rb,0.31382e+00_rb, &
        & 0.29942e+00_rb,0.27289e+00_rb,0.23940e+00_rb,0.17248e+00_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.27646e+00_rb,0.28482e+00_rb,0.30092e+00_rb,0.30845e+00_rb,0.30839e+00_rb, &
        & 0.29721e+00_rb,0.27391e+00_rb,0.23566e+00_rb,0.13145e+00_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.28013e+00_rb,0.28703e+00_rb,0.30330e+00_rb,0.31210e+00_rb,0.31152e+00_rb, &
        & 0.29799e+00_rb,0.27265e+00_rb,0.23501e+00_rb,0.13680e+00_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.28251e+00_rb,0.28781e+00_rb,0.30455e+00_rb,0.31430e+00_rb,0.31342e+00_rb, &
        & 0.29883e+00_rb,0.27192e+00_rb,0.23420e+00_rb,0.14031e+00_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.28473e+00_rb,0.28806e+00_rb,0.30532e+00_rb,0.31558e+00_rb,0.31478e+00_rb, &
        & 0.29964e+00_rb,0.27140e+00_rb,0.23324e+00_rb,0.14138e+00_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.28731e+00_rb,0.28885e+00_rb,0.30563e+00_rb,0.31603e+00_rb,0.31529e+00_rb, &
        & 0.30003e+00_rb,0.27073e+00_rb,0.23300e+00_rb,0.14291e+00_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.39331e+00_rb,0.34536e+00_rb,0.29823e+00_rb,0.25200e+00_rb,0.22257e+00_rb, &
        & 0.22153e+00_rb,0.23263e+00_rb,0.25940e+00_rb,0.28372e+00_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.38484e+00_rb,0.33824e+00_rb,0.29242e+00_rb,0.24772e+00_rb,0.21768e+00_rb, &
        & 0.21972e+00_rb,0.23098e+00_rb,0.25833e+00_rb,0.28173e+00_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.37910e+00_rb,0.33360e+00_rb,0.28884e+00_rb,0.24475e+00_rb,0.21421e+00_rb, &
        & 0.21744e+00_rb,0.22823e+00_rb,0.25633e+00_rb,0.27932e+00_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.37406e+00_rb,0.32956e+00_rb,0.28576e+00_rb,0.24212e+00_rb,0.21151e+00_rb, &
        & 0.21463e+00_rb,0.22480e+00_rb,0.25354e+00_rb,0.27637e+00_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.36968e+00_rb,0.32602e+00_rb,0.28272e+00_rb,0.24026e+00_rb,0.20970e+00_rb, &
        & 0.21134e+00_rb,0.22163e+00_rb,0.25081e+00_rb,0.27385e+00_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.41602e+00_rb,0.36529e+00_rb,0.31527e+00_rb,0.26672e+00_rb,0.24637e+00_rb, &
        & 0.24718e+00_rb,0.25604e+00_rb,0.28585e+00_rb,0.31850e+00_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.40830e+00_rb,0.35891e+00_rb,0.31022e+00_rb,0.26338e+00_rb,0.24193e+00_rb, &
        & 0.24484e+00_rb,0.25382e+00_rb,0.28396e+00_rb,0.31574e+00_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.40220e+00_rb,0.35391e+00_rb,0.30635e+00_rb,0.26042e+00_rb,0.23856e+00_rb, &
        & 0.24224e+00_rb,0.25030e+00_rb,0.28140e+00_rb,0.31256e+00_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.39708e+00_rb,0.34976e+00_rb,0.30310e+00_rb,0.25764e+00_rb,0.23590e+00_rb, &
        & 0.23895e+00_rb,0.24655e+00_rb,0.27840e+00_rb,0.30929e+00_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.39199e+00_rb,0.34559e+00_rb,0.29962e+00_rb,0.25559e+00_rb,0.23369e+00_rb, &
        & 0.23517e+00_rb,0.24316e+00_rb,0.27577e+00_rb,0.30660e+00_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.43911e+00_rb,0.38551e+00_rb,0.33254e+00_rb,0.28548e+00_rb,0.27501e+00_rb, &
        & 0.27158e+00_rb,0.27806e+00_rb,0.30961e+00_rb,0.34942e+00_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.43165e+00_rb,0.37930e+00_rb,0.32763e+00_rb,0.28158e+00_rb,0.27135e+00_rb, &
        & 0.26881e+00_rb,0.27489e+00_rb,0.30725e+00_rb,0.34620e+00_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.42551e+00_rb,0.37419e+00_rb,0.32362e+00_rb,0.27772e+00_rb,0.26799e+00_rb, &
        & 0.26571e+00_rb,0.27096e+00_rb,0.30422e+00_rb,0.34240e+00_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.42038e+00_rb,0.37000e+00_rb,0.32049e+00_rb,0.27494e+00_rb,0.26498e+00_rb, &
        & 0.26183e+00_rb,0.26698e+00_rb,0.30136e+00_rb,0.33931e+00_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.41446e+00_rb,0.36533e+00_rb,0.31734e+00_rb,0.27304e+00_rb,0.26234e+00_rb, &
        & 0.25777e+00_rb,0.26380e+00_rb,0.29913e+00_rb,0.33705e+00_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.46241e+00_rb,0.40571e+00_rb,0.34961e+00_rb,0.30954e+00_rb,0.30425e+00_rb, &
        & 0.29507e+00_rb,0.29890e+00_rb,0.33301e+00_rb,0.37827e+00_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.45465e+00_rb,0.39923e+00_rb,0.34450e+00_rb,0.30510e+00_rb,0.30124e+00_rb, &
        & 0.29145e+00_rb,0.29516e+00_rb,0.32984e+00_rb,0.37410e+00_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.44872e+00_rb,0.39432e+00_rb,0.34072e+00_rb,0.30104e+00_rb,0.29841e+00_rb, &
        & 0.28758e+00_rb,0.29090e+00_rb,0.32641e+00_rb,0.36986e+00_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.44350e+00_rb,0.39014e+00_rb,0.33810e+00_rb,0.29814e+00_rb,0.29548e+00_rb, &
        & 0.28294e+00_rb,0.28676e+00_rb,0.32342e+00_rb,0.36667e+00_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.43828e+00_rb,0.38609e+00_rb,0.33570e+00_rb,0.29627e+00_rb,0.29269e+00_rb, &
        & 0.27870e+00_rb,0.28357e+00_rb,0.32115e+00_rb,0.36426e+00_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.48784e+00_rb,0.42784e+00_rb,0.36835e+00_rb,0.33984e+00_rb,0.33294e+00_rb, &
        & 0.31846e+00_rb,0.32126e+00_rb,0.35651e+00_rb,0.40621e+00_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.48030e+00_rb,0.42153e+00_rb,0.36340e+00_rb,0.33519e+00_rb,0.32973e+00_rb, &
        & 0.31380e+00_rb,0.31683e+00_rb,0.35311e+00_rb,0.40187e+00_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.47411e+00_rb,0.41636e+00_rb,0.35974e+00_rb,0.33136e+00_rb,0.32664e+00_rb, &
        & 0.30892e+00_rb,0.31220e+00_rb,0.34964e+00_rb,0.39763e+00_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.46883e+00_rb,0.41227e+00_rb,0.35745e+00_rb,0.32776e+00_rb,0.32317e+00_rb, &
        & 0.30393e+00_rb,0.30777e+00_rb,0.34651e+00_rb,0.39440e+00_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.46388e+00_rb,0.40844e+00_rb,0.35543e+00_rb,0.32520e+00_rb,0.31964e+00_rb, &
        & 0.29971e+00_rb,0.30439e+00_rb,0.34429e+00_rb,0.39205e+00_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.51535e+00_rb,0.45180e+00_rb,0.39162e+00_rb,0.37322e+00_rb,0.36050e+00_rb, &
        & 0.34300e+00_rb,0.34505e+00_rb,0.38137e+00_rb,0.43538e+00_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.50781e+00_rb,0.44545e+00_rb,0.38586e+00_rb,0.36868e+00_rb,0.35573e+00_rb, &
        & 0.33760e+00_rb,0.34025e+00_rb,0.37783e+00_rb,0.43095e+00_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.50139e+00_rb,0.44011e+00_rb,0.38148e+00_rb,0.36480e+00_rb,0.35183e+00_rb, &
        & 0.33217e+00_rb,0.33546e+00_rb,0.37434e+00_rb,0.42674e+00_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.49657e+00_rb,0.43636e+00_rb,0.37891e+00_rb,0.36116e+00_rb,0.34800e+00_rb, &
        & 0.32704e+00_rb,0.33102e+00_rb,0.37152e+00_rb,0.42385e+00_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.49171e+00_rb,0.43263e+00_rb,0.37693e+00_rb,0.35827e+00_rb,0.34376e+00_rb, &
        & 0.32320e+00_rb,0.32782e+00_rb,0.36976e+00_rb,0.42188e+00_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.54162e+00_rb,0.47464e+00_rb,0.41834e+00_rb,0.40782e+00_rb,0.38721e+00_rb, &
        & 0.36824e+00_rb,0.37067e+00_rb,0.40670e+00_rb,0.46460e+00_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.53535e+00_rb,0.46937e+00_rb,0.41220e+00_rb,0.40296e+00_rb,0.38145e+00_rb, &
        & 0.36278e+00_rb,0.36581e+00_rb,0.40309e+00_rb,0.46013e+00_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.52914e+00_rb,0.46429e+00_rb,0.40710e+00_rb,0.39788e+00_rb,0.37673e+00_rb, &
        & 0.35819e+00_rb,0.36078e+00_rb,0.40008e+00_rb,0.45678e+00_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.52440e+00_rb,0.46050e+00_rb,0.40384e+00_rb,0.39374e+00_rb,0.37204e+00_rb, &
        & 0.35371e+00_rb,0.35616e+00_rb,0.39775e+00_rb,0.45424e+00_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.52017e+00_rb,0.45722e+00_rb,0.40132e+00_rb,0.39030e+00_rb,0.36733e+00_rb, &
        & 0.34991e+00_rb,0.35252e+00_rb,0.39600e+00_rb,0.45213e+00_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.56560e+00_rb,0.49547e+00_rb,0.44792e+00_rb,0.43980e+00_rb,0.41568e+00_rb, &
        & 0.39571e+00_rb,0.39516e+00_rb,0.42687e+00_rb,0.48767e+00_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.55986e+00_rb,0.49068e+00_rb,0.44130e+00_rb,0.43344e+00_rb,0.40921e+00_rb, &
        & 0.39033e+00_rb,0.39006e+00_rb,0.42429e+00_rb,0.48458e+00_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.55439e+00_rb,0.48622e+00_rb,0.43604e+00_rb,0.42727e+00_rb,0.40340e+00_rb, &
        & 0.38637e+00_rb,0.38517e+00_rb,0.42255e+00_rb,0.48243e+00_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.55085e+00_rb,0.48344e+00_rb,0.43251e+00_rb,0.42285e+00_rb,0.39791e+00_rb, &
        & 0.38219e+00_rb,0.38096e+00_rb,0.42076e+00_rb,0.48039e+00_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.54722e+00_rb,0.48060e+00_rb,0.42969e+00_rb,0.41964e+00_rb,0.39283e+00_rb, &
        & 0.37840e+00_rb,0.37699e+00_rb,0.41955e+00_rb,0.47894e+00_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.58672e+00_rb,0.51388e+00_rb,0.47909e+00_rb,0.46769e+00_rb,0.44479e+00_rb, &
        & 0.42421e+00_rb,0.41694e+00_rb,0.44339e+00_rb,0.50662e+00_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.58159e+00_rb,0.50951e+00_rb,0.47213e+00_rb,0.46028e+00_rb,0.43736e+00_rb, &
        & 0.41895e+00_rb,0.41246e+00_rb,0.44257e+00_rb,0.50550e+00_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.57736e+00_rb,0.50610e+00_rb,0.46657e+00_rb,0.45431e+00_rb,0.43100e+00_rb, &
        & 0.41543e+00_rb,0.40868e+00_rb,0.44206e+00_rb,0.50480e+00_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.57459e+00_rb,0.50394e+00_rb,0.46330e+00_rb,0.44994e+00_rb,0.42562e+00_rb, &
        & 0.41143e+00_rb,0.40432e+00_rb,0.44089e+00_rb,0.50333e+00_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.57136e+00_rb,0.50148e+00_rb,0.46136e+00_rb,0.44665e+00_rb,0.42115e+00_rb, &
        & 0.40792e+00_rb,0.40001e+00_rb,0.43966e+00_rb,0.50186e+00_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.60279e+00_rb,0.52914e+00_rb,0.50725e+00_rb,0.49385e+00_rb,0.47296e+00_rb, &
        & 0.45056e+00_rb,0.43650e+00_rb,0.45649e+00_rb,0.52069e+00_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.59826e+00_rb,0.52452e+00_rb,0.50048e+00_rb,0.48635e+00_rb,0.46511e+00_rb, &
        & 0.44618e+00_rb,0.43278e+00_rb,0.45737e+00_rb,0.52170e+00_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.59590e+00_rb,0.52228e+00_rb,0.49610e+00_rb,0.47997e+00_rb,0.45903e+00_rb, &
        & 0.44309e+00_rb,0.42962e+00_rb,0.45759e+00_rb,0.52208e+00_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.59454e+00_rb,0.52120e+00_rb,0.49388e+00_rb,0.47548e+00_rb,0.45447e+00_rb, &
        & 0.44006e+00_rb,0.42512e+00_rb,0.45631e+00_rb,0.52061e+00_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.59357e+00_rb,0.52070e+00_rb,0.49246e+00_rb,0.47250e+00_rb,0.45028e+00_rb, &
        & 0.43610e+00_rb,0.42114e+00_rb,0.45529e+00_rb,0.51951e+00_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.60887e+00_rb,0.53839e+00_rb,0.52807e+00_rb,0.51587e+00_rb,0.49623e+00_rb, &
        & 0.47362e+00_rb,0.45440e+00_rb,0.46645e+00_rb,0.53001e+00_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.60756e+00_rb,0.53578e+00_rb,0.52411e+00_rb,0.50806e+00_rb,0.48992e+00_rb, &
        & 0.47089e+00_rb,0.45204e+00_rb,0.46808e+00_rb,0.53215e+00_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.60793e+00_rb,0.53524e+00_rb,0.52214e+00_rb,0.50280e+00_rb,0.48540e+00_rb, &
        & 0.46881e+00_rb,0.44770e+00_rb,0.46764e+00_rb,0.53195e+00_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.60970e+00_rb,0.53618e+00_rb,0.52106e+00_rb,0.49960e+00_rb,0.48160e+00_rb, &
        & 0.46481e+00_rb,0.44371e+00_rb,0.46667e+00_rb,0.53096e+00_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.61038e+00_rb,0.53650e+00_rb,0.52014e+00_rb,0.49804e+00_rb,0.47818e+00_rb, &
        & 0.45984e+00_rb,0.44017e+00_rb,0.46579e+00_rb,0.53008e+00_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.60957e+00_rb,0.54609e+00_rb,0.54565e+00_rb,0.53521e+00_rb,0.51779e+00_rb, &
        & 0.49593e+00_rb,0.47189e+00_rb,0.47309e+00_rb,0.46983e+00_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.61214e+00_rb,0.54625e+00_rb,0.54367e+00_rb,0.52921e+00_rb,0.51339e+00_rb, &
        & 0.49443e+00_rb,0.46946e+00_rb,0.47371e+00_rb,0.47545e+00_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.61642e+00_rb,0.54787e+00_rb,0.54269e+00_rb,0.52579e+00_rb,0.50981e+00_rb, &
        & 0.49118e+00_rb,0.46605e+00_rb,0.47265e+00_rb,0.47706e+00_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.62028e+00_rb,0.54980e+00_rb,0.54303e+00_rb,0.52440e+00_rb,0.50697e+00_rb, &
        & 0.48619e+00_rb,0.46182e+00_rb,0.47123e+00_rb,0.48084e+00_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.62175e+00_rb,0.55026e+00_rb,0.54305e+00_rb,0.52373e+00_rb,0.50407e+00_rb, &
        & 0.48153e+00_rb,0.45738e+00_rb,0.46998e+00_rb,0.48415e+00_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.60459e+00_rb,0.55304e+00_rb,0.55845e+00_rb,0.55195e+00_rb,0.53769e+00_rb, &
        & 0.51606e+00_rb,0.48661e+00_rb,0.47383e+00_rb,0.40484e+00_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.61134e+00_rb,0.55569e+00_rb,0.55809e+00_rb,0.54885e+00_rb,0.53426e+00_rb, &
        & 0.51450e+00_rb,0.48473e+00_rb,0.47289e+00_rb,0.40823e+00_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.61890e+00_rb,0.55963e+00_rb,0.55924e+00_rb,0.54802e+00_rb,0.53230e+00_rb, &
        & 0.51039e+00_rb,0.48075e+00_rb,0.47078e+00_rb,0.41355e+00_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.62388e+00_rb,0.56213e+00_rb,0.56099e+00_rb,0.54740e+00_rb,0.52999e+00_rb, &
        & 0.50582e+00_rb,0.47603e+00_rb,0.46901e+00_rb,0.41977e+00_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.62688e+00_rb,0.56314e+00_rb,0.56152e+00_rb,0.54685e+00_rb,0.52723e+00_rb, &
        & 0.50138e+00_rb,0.47115e+00_rb,0.46700e+00_rb,0.42405e+00_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.52445e+00_rb,0.46003e+00_rb,0.39566e+00_rb,0.33139e+00_rb,0.26995e+00_rb, &
        & 0.25837e+00_rb,0.28528e+00_rb,0.32819e+00_rb,0.35568e+00_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.51513e+00_rb,0.45183e+00_rb,0.38858e+00_rb,0.32553e+00_rb,0.26784e+00_rb, &
        & 0.25515e+00_rb,0.28229e+00_rb,0.32456e+00_rb,0.35149e+00_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.50548e+00_rb,0.44334e+00_rb,0.38125e+00_rb,0.32054e+00_rb,0.26590e+00_rb, &
        & 0.25305e+00_rb,0.28136e+00_rb,0.32218e+00_rb,0.34858e+00_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.49563e+00_rb,0.43466e+00_rb,0.37374e+00_rb,0.31559e+00_rb,0.26350e+00_rb, &
        & 0.25115e+00_rb,0.28088e+00_rb,0.32077e+00_rb,0.34651e+00_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.48636e+00_rb,0.42650e+00_rb,0.36724e+00_rb,0.31119e+00_rb,0.26146e+00_rb, &
        & 0.24908e+00_rb,0.27907e+00_rb,0.31920e+00_rb,0.34392e+00_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.56575e+00_rb,0.49609e+00_rb,0.42647e+00_rb,0.35691e+00_rb,0.29788e+00_rb, &
        & 0.29937e+00_rb,0.33584e+00_rb,0.38701e+00_rb,0.42822e+00_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.55481e+00_rb,0.48648e+00_rb,0.41819e+00_rb,0.35023e+00_rb,0.29393e+00_rb, &
        & 0.29578e+00_rb,0.33190e+00_rb,0.38233e+00_rb,0.42282e+00_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.54468e+00_rb,0.47756e+00_rb,0.41048e+00_rb,0.34498e+00_rb,0.29076e+00_rb, &
        & 0.29330e+00_rb,0.33000e+00_rb,0.37899e+00_rb,0.41892e+00_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.53444e+00_rb,0.46857e+00_rb,0.40281e+00_rb,0.34010e+00_rb,0.28772e+00_rb, &
        & 0.29117e+00_rb,0.32899e+00_rb,0.37674e+00_rb,0.41581e+00_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.52472e+00_rb,0.46000e+00_rb,0.39601e+00_rb,0.33543e+00_rb,0.28544e+00_rb, &
        & 0.28841e+00_rb,0.32728e+00_rb,0.37488e+00_rb,0.41300e+00_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.61401e+00_rb,0.53826e+00_rb,0.46254e+00_rb,0.38687e+00_rb,0.33653e+00_rb, &
        & 0.34874e+00_rb,0.39278e+00_rb,0.45331e+00_rb,0.51178e+00_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.60181e+00_rb,0.52754e+00_rb,0.45329e+00_rb,0.37917e+00_rb,0.33018e+00_rb, &
        & 0.34432e+00_rb,0.38762e+00_rb,0.44735e+00_rb,0.50487e+00_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.59018e+00_rb,0.51732e+00_rb,0.44448e+00_rb,0.37278e+00_rb,0.32516e+00_rb, &
        & 0.34086e+00_rb,0.38493e+00_rb,0.44336e+00_rb,0.50014e+00_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.57893e+00_rb,0.50743e+00_rb,0.43595e+00_rb,0.36714e+00_rb,0.32113e+00_rb, &
        & 0.33824e+00_rb,0.38315e+00_rb,0.44016e+00_rb,0.49598e+00_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.56842e+00_rb,0.49818e+00_rb,0.42822e+00_rb,0.36203e+00_rb,0.31748e+00_rb, &
        & 0.33494e+00_rb,0.38145e+00_rb,0.43774e+00_rb,0.49238e+00_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.66683e+00_rb,0.58444e+00_rb,0.50208e+00_rb,0.41977e+00_rb,0.38494e+00_rb, &
        & 0.40485e+00_rb,0.45417e+00_rb,0.52508e+00_rb,0.59839e+00_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.65354e+00_rb,0.57275e+00_rb,0.49199e+00_rb,0.41128e+00_rb,0.37591e+00_rb, &
        & 0.39916e+00_rb,0.44835e+00_rb,0.51820e+00_rb,0.59043e+00_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.64071e+00_rb,0.56148e+00_rb,0.48227e+00_rb,0.40388e+00_rb,0.36879e+00_rb, &
        & 0.39404e+00_rb,0.44465e+00_rb,0.51320e+00_rb,0.58464e+00_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.62844e+00_rb,0.55070e+00_rb,0.47297e+00_rb,0.39753e+00_rb,0.36329e+00_rb, &
        & 0.39017e+00_rb,0.44183e+00_rb,0.50881e+00_rb,0.57888e+00_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.61748e+00_rb,0.54106e+00_rb,0.46467e+00_rb,0.39213e+00_rb,0.35832e+00_rb, &
        & 0.38631e+00_rb,0.43926e+00_rb,0.50522e+00_rb,0.57403e+00_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.72260e+00_rb,0.63317e+00_rb,0.54375e+00_rb,0.45913e+00_rb,0.43732e+00_rb, &
        & 0.46505e+00_rb,0.51783e+00_rb,0.59954e+00_rb,0.68564e+00_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.70858e+00_rb,0.62085e+00_rb,0.53314e+00_rb,0.44897e+00_rb,0.42823e+00_rb, &
        & 0.45720e+00_rb,0.51115e+00_rb,0.59162e+00_rb,0.67653e+00_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.69539e+00_rb,0.60926e+00_rb,0.52314e+00_rb,0.43994e+00_rb,0.42099e+00_rb, &
        & 0.45017e+00_rb,0.50656e+00_rb,0.58542e+00_rb,0.66941e+00_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.68235e+00_rb,0.59780e+00_rb,0.51326e+00_rb,0.43281e+00_rb,0.41557e+00_rb, &
        & 0.44468e+00_rb,0.50258e+00_rb,0.57996e+00_rb,0.66206e+00_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.67018e+00_rb,0.58711e+00_rb,0.50404e+00_rb,0.42628e+00_rb,0.41131e+00_rb, &
        & 0.43975e+00_rb,0.49863e+00_rb,0.57477e+00_rb,0.65556e+00_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.77964e+00_rb,0.68298e+00_rb,0.58633e+00_rb,0.50766e+00_rb,0.49641e+00_rb, &
        & 0.52605e+00_rb,0.58161e+00_rb,0.67425e+00_rb,0.77212e+00_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.76589e+00_rb,0.67092e+00_rb,0.57596e+00_rb,0.49531e+00_rb,0.48834e+00_rb, &
        & 0.51573e+00_rb,0.57361e+00_rb,0.66493e+00_rb,0.76124e+00_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.75251e+00_rb,0.65917e+00_rb,0.56583e+00_rb,0.48459e+00_rb,0.48115e+00_rb, &
        & 0.50704e+00_rb,0.56757e+00_rb,0.65686e+00_rb,0.75182e+00_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.73905e+00_rb,0.64728e+00_rb,0.55554e+00_rb,0.47565e+00_rb,0.47497e+00_rb, &
        & 0.50004e+00_rb,0.56191e+00_rb,0.64957e+00_rb,0.74243e+00_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.72636e+00_rb,0.63608e+00_rb,0.54581e+00_rb,0.46758e+00_rb,0.47009e+00_rb, &
        & 0.49349e+00_rb,0.55635e+00_rb,0.64252e+00_rb,0.73397e+00_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.84113e+00_rb,0.73668e+00_rb,0.63225e+00_rb,0.56459e+00_rb,0.56168e+00_rb, &
        & 0.58736e+00_rb,0.64476e+00_rb,0.74759e+00_rb,0.85655e+00_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.82630e+00_rb,0.72365e+00_rb,0.62103e+00_rb,0.55091e+00_rb,0.55367e+00_rb, &
        & 0.57467e+00_rb,0.63457e+00_rb,0.73605e+00_rb,0.84323e+00_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.81279e+00_rb,0.71173e+00_rb,0.61068e+00_rb,0.53944e+00_rb,0.54564e+00_rb, &
        & 0.56363e+00_rb,0.62610e+00_rb,0.72557e+00_rb,0.83073e+00_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.79922e+00_rb,0.69978e+00_rb,0.60035e+00_rb,0.52974e+00_rb,0.53833e+00_rb, &
        & 0.55444e+00_rb,0.61857e+00_rb,0.71625e+00_rb,0.81942e+00_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.78598e+00_rb,0.68814e+00_rb,0.59031e+00_rb,0.52130e+00_rb,0.53249e+00_rb, &
        & 0.54592e+00_rb,0.61225e+00_rb,0.70842e+00_rb,0.80994e+00_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.90865e+00_rb,0.79552e+00_rb,0.68301e+00_rb,0.62894e+00_rb,0.62980e+00_rb, &
        & 0.64956e+00_rb,0.70917e+00_rb,0.82179e+00_rb,0.94167e+00_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.89327e+00_rb,0.78202e+00_rb,0.67089e+00_rb,0.61713e+00_rb,0.62095e+00_rb, &
        & 0.63402e+00_rb,0.69617e+00_rb,0.80717e+00_rb,0.92484e+00_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.87903e+00_rb,0.76952e+00_rb,0.66001e+00_rb,0.60628e+00_rb,0.61235e+00_rb, &
        & 0.62019e+00_rb,0.68531e+00_rb,0.79440e+00_rb,0.90974e+00_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.86460e+00_rb,0.75687e+00_rb,0.64914e+00_rb,0.59649e+00_rb,0.60349e+00_rb, &
        & 0.60828e+00_rb,0.67580e+00_rb,0.78318e+00_rb,0.89639e+00_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.85091e+00_rb,0.74488e+00_rb,0.63886e+00_rb,0.58665e+00_rb,0.59566e+00_rb, &
        & 0.59754e+00_rb,0.66827e+00_rb,0.77422e+00_rb,0.88585e+00_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.98149e+00_rb,0.85912e+00_rb,0.74373e+00_rb,0.70295e+00_rb,0.70214e+00_rb, &
        & 0.71285e+00_rb,0.77237e+00_rb,0.89352e+00_rb,0.10239e+01_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.96640e+00_rb,0.84589e+00_rb,0.73081e+00_rb,0.69152e+00_rb,0.69170e+00_rb, &
        & 0.69439e+00_rb,0.75605e+00_rb,0.87558e+00_rb,0.10032e+01_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.95188e+00_rb,0.83316e+00_rb,0.71892e+00_rb,0.68080e+00_rb,0.68028e+00_rb, &
        & 0.67751e+00_rb,0.74271e+00_rb,0.86004e+00_rb,0.98493e+00_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.93737e+00_rb,0.82045e+00_rb,0.70700e+00_rb,0.67076e+00_rb,0.66785e+00_rb, &
        & 0.66265e+00_rb,0.73184e+00_rb,0.84775e+00_rb,0.97055e+00_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.92346e+00_rb,0.80828e+00_rb,0.69536e+00_rb,0.66047e+00_rb,0.65652e+00_rb, &
        & 0.65000e+00_rb,0.72388e+00_rb,0.83863e+00_rb,0.95986e+00_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.10577e+01_rb,0.92576e+00_rb,0.81364e+00_rb,0.78190e+00_rb,0.77617e+00_rb, &
        & 0.77603e+00_rb,0.83269e+00_rb,0.96109e+00_rb,0.11008e+01_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.10450e+01_rb,0.91463e+00_rb,0.80097e+00_rb,0.77046e+00_rb,0.76182e+00_rb, &
        & 0.75411e+00_rb,0.81384e+00_rb,0.94032e+00_rb,0.10769e+01_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.10312e+01_rb,0.90248e+00_rb,0.78812e+00_rb,0.75969e+00_rb,0.74632e+00_rb, &
        & 0.73436e+00_rb,0.79822e+00_rb,0.92300e+00_rb,0.10571e+01_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.10164e+01_rb,0.88956e+00_rb,0.77444e+00_rb,0.74952e+00_rb,0.73099e+00_rb, &
        & 0.71729e+00_rb,0.78702e+00_rb,0.91062e+00_rb,0.10426e+01_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.10010e+01_rb,0.87610e+00_rb,0.76045e+00_rb,0.73747e+00_rb,0.71669e+00_rb, &
        & 0.70419e+00_rb,0.77819e+00_rb,0.90060e+00_rb,0.10309e+01_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.11332e+01_rb,0.99169e+00_rb,0.88520e+00_rb,0.85938e+00_rb,0.84171e+00_rb, &
        & 0.82777e+00_rb,0.88045e+00_rb,0.10119e+01_rb,0.11581e+01_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.11221e+01_rb,0.98196e+00_rb,0.87210e+00_rb,0.84745e+00_rb,0.82287e+00_rb, &
        & 0.80383e+00_rb,0.86097e+00_rb,0.99198e+00_rb,0.11355e+01_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.11079e+01_rb,0.96953e+00_rb,0.85745e+00_rb,0.83492e+00_rb,0.80465e+00_rb, &
        & 0.78293e+00_rb,0.84710e+00_rb,0.97731e+00_rb,0.11185e+01_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.10888e+01_rb,0.95287e+00_rb,0.84018e+00_rb,0.82112e+00_rb,0.78666e+00_rb, &
        & 0.76752e+00_rb,0.83583e+00_rb,0.96514e+00_rb,0.11044e+01_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.10701e+01_rb,0.93668e+00_rb,0.82311e+00_rb,0.80490e+00_rb,0.77024e+00_rb, &
        & 0.75616e+00_rb,0.82679e+00_rb,0.95569e+00_rb,0.10935e+01_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.12101e+01_rb,0.10589e+01_rb,0.96254e+00_rb,0.93478e+00_rb,0.90424e+00_rb, &
        & 0.88111e+00_rb,0.92612e+00_rb,0.10566e+01_rb,0.12084e+01_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.11952e+01_rb,0.10459e+01_rb,0.94639e+00_rb,0.91979e+00_rb,0.88299e+00_rb, &
        & 0.85667e+00_rb,0.90737e+00_rb,0.10393e+01_rb,0.11885e+01_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.11758e+01_rb,0.10290e+01_rb,0.92827e+00_rb,0.90412e+00_rb,0.86234e+00_rb, &
        & 0.83719e+00_rb,0.89320e+00_rb,0.10260e+01_rb,0.11732e+01_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.11552e+01_rb,0.10111e+01_rb,0.90821e+00_rb,0.88659e+00_rb,0.84346e+00_rb, &
        & 0.82327e+00_rb,0.88324e+00_rb,0.10165e+01_rb,0.11624e+01_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.11370e+01_rb,0.99527e+00_rb,0.88941e+00_rb,0.86803e+00_rb,0.82686e+00_rb, &
        & 0.81098e+00_rb,0.87502e+00_rb,0.10085e+01_rb,0.11531e+01_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.12761e+01_rb,0.11169e+01_rb,0.10366e+01_rb,0.10056e+01_rb,0.96619e+00_rb, &
        & 0.93741e+00_rb,0.96971e+00_rb,0.10971e+01_rb,0.12455e+01_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.12590e+01_rb,0.11018e+01_rb,0.10193e+01_rb,0.98827e+00_rb,0.94401e+00_rb, &
        & 0.91331e+00_rb,0.95200e+00_rb,0.10824e+01_rb,0.12321e+01_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.12382e+01_rb,0.10836e+01_rb,0.99864e+00_rb,0.96922e+00_rb,0.92237e+00_rb, &
        & 0.89570e+00_rb,0.94013e+00_rb,0.10732e+01_rb,0.12212e+01_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.12192e+01_rb,0.10671e+01_rb,0.97718e+00_rb,0.94975e+00_rb,0.90316e+00_rb, &
        & 0.88110e+00_rb,0.93100e+00_rb,0.10658e+01_rb,0.12124e+01_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.12013e+01_rb,0.10515e+01_rb,0.95818e+00_rb,0.93022e+00_rb,0.88690e+00_rb, &
        & 0.86844e+00_rb,0.92401e+00_rb,0.10605e+01_rb,0.12064e+01_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.70834e+00_rb,0.62203e+00_rb,0.53582e+00_rb,0.44981e+00_rb,0.36430e+00_rb, &
        & 0.30456e+00_rb,0.33772e+00_rb,0.39113e+00_rb,0.42417e+00_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.69132e+00_rb,0.60682e+00_rb,0.52239e+00_rb,0.43815e+00_rb,0.35442e+00_rb, &
        & 0.29618e+00_rb,0.32847e+00_rb,0.37897e+00_rb,0.41053e+00_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.67551e+00_rb,0.59273e+00_rb,0.51002e+00_rb,0.42746e+00_rb,0.34540e+00_rb, &
        & 0.29170e+00_rb,0.32190e+00_rb,0.37054e+00_rb,0.40056e+00_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.65954e+00_rb,0.57855e+00_rb,0.49762e+00_rb,0.41681e+00_rb,0.33774e+00_rb, &
        & 0.29058e+00_rb,0.31932e+00_rb,0.36553e+00_rb,0.39416e+00_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.64426e+00_rb,0.56502e+00_rb,0.48582e+00_rb,0.40671e+00_rb,0.33237e+00_rb, &
        & 0.29156e+00_rb,0.32280e+00_rb,0.36526e+00_rb,0.39257e+00_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.79278e+00_rb,0.69566e+00_rb,0.59863e+00_rb,0.50180e+00_rb,0.40547e+00_rb, &
        & 0.36074e+00_rb,0.40858e+00_rb,0.47363e+00_rb,0.52463e+00_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.77384e+00_rb,0.67879e+00_rb,0.58381e+00_rb,0.48896e+00_rb,0.39453e+00_rb, &
        & 0.34912e+00_rb,0.39677e+00_rb,0.45884e+00_rb,0.50797e+00_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.75601e+00_rb,0.66296e+00_rb,0.56995e+00_rb,0.47701e+00_rb,0.38437e+00_rb, &
        & 0.34005e+00_rb,0.38826e+00_rb,0.44800e+00_rb,0.49524e+00_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.73809e+00_rb,0.64708e+00_rb,0.55610e+00_rb,0.46517e+00_rb,0.37616e+00_rb, &
        & 0.33560e+00_rb,0.38277e+00_rb,0.44095e+00_rb,0.48660e+00_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.72001e+00_rb,0.63111e+00_rb,0.54222e+00_rb,0.45336e+00_rb,0.36992e+00_rb, &
        & 0.33536e+00_rb,0.38227e+00_rb,0.43734e+00_rb,0.48144e+00_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.88640e+00_rb,0.77731e+00_rb,0.66826e+00_rb,0.55931e+00_rb,0.45077e+00_rb, &
        & 0.42993e+00_rb,0.49594e+00_rb,0.57510e+00_rb,0.65079e+00_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.86545e+00_rb,0.75870e+00_rb,0.65198e+00_rb,0.54530e+00_rb,0.43919e+00_rb, &
        & 0.41467e+00_rb,0.48152e+00_rb,0.55764e+00_rb,0.63093e+00_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.84484e+00_rb,0.74046e+00_rb,0.63609e+00_rb,0.53174e+00_rb,0.42841e+00_rb, &
        & 0.40263e+00_rb,0.47018e+00_rb,0.54358e+00_rb,0.61450e+00_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.82484e+00_rb,0.72279e+00_rb,0.62074e+00_rb,0.51870e+00_rb,0.41933e+00_rb, &
        & 0.39440e+00_rb,0.46214e+00_rb,0.53355e+00_rb,0.60235e+00_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.80403e+00_rb,0.70444e+00_rb,0.60484e+00_rb,0.50526e+00_rb,0.41249e+00_rb, &
        & 0.39177e+00_rb,0.45674e+00_rb,0.52614e+00_rb,0.59324e+00_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.98775e+00_rb,0.86568e+00_rb,0.74362e+00_rb,0.62157e+00_rb,0.50893e+00_rb, &
        & 0.51227e+00_rb,0.60104e+00_rb,0.69710e+00_rb,0.79722e+00_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.96448e+00_rb,0.84510e+00_rb,0.72572e+00_rb,0.60634e+00_rb,0.49496e+00_rb, &
        & 0.49365e+00_rb,0.58337e+00_rb,0.67615e+00_rb,0.77326e+00_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.94121e+00_rb,0.82456e+00_rb,0.70790e+00_rb,0.59124e+00_rb,0.48214e+00_rb, &
        & 0.47977e+00_rb,0.56920e+00_rb,0.65880e+00_rb,0.75281e+00_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.91776e+00_rb,0.80389e+00_rb,0.69002e+00_rb,0.57613e+00_rb,0.47050e+00_rb, &
        & 0.47009e+00_rb,0.55863e+00_rb,0.64595e+00_rb,0.73761e+00_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.89560e+00_rb,0.78437e+00_rb,0.67314e+00_rb,0.56190e+00_rb,0.46260e+00_rb, &
        & 0.46419e+00_rb,0.55009e+00_rb,0.63560e+00_rb,0.72526e+00_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.10958e+01_rb,0.95999e+00_rb,0.82419e+00_rb,0.68839e+00_rb,0.58904e+00_rb, &
        & 0.60909e+00_rb,0.72523e+00_rb,0.84139e+00_rb,0.96623e+00_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.10697e+01_rb,0.93701e+00_rb,0.80427e+00_rb,0.67151e+00_rb,0.56892e+00_rb, &
        & 0.58963e+00_rb,0.70431e+00_rb,0.81667e+00_rb,0.93800e+00_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.10434e+01_rb,0.91377e+00_rb,0.78415e+00_rb,0.65453e+00_rb,0.55082e+00_rb, &
        & 0.57510e+00_rb,0.68668e+00_rb,0.79542e+00_rb,0.91311e+00_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.10167e+01_rb,0.89033e+00_rb,0.76393e+00_rb,0.63753e+00_rb,0.53477e+00_rb, &
        & 0.56392e+00_rb,0.67291e+00_rb,0.77892e+00_rb,0.89383e+00_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.99271e+00_rb,0.86922e+00_rb,0.74573e+00_rb,0.62224e+00_rb,0.52129e+00_rb, &
        & 0.55503e+00_rb,0.66157e+00_rb,0.76538e+00_rb,0.87797e+00_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.12165e+01_rb,0.10653e+01_rb,0.91421e+00_rb,0.76309e+00_rb,0.68684e+00_rb, &
        & 0.72751e+00_rb,0.87072e+00_rb,0.10104e+01_rb,0.11626e+01_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.11851e+01_rb,0.10377e+01_rb,0.89038e+00_rb,0.74302e+00_rb,0.66078e+00_rb, &
        & 0.70740e+00_rb,0.84652e+00_rb,0.98180e+00_rb,0.11298e+01_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.11540e+01_rb,0.10104e+01_rb,0.86680e+00_rb,0.72322e+00_rb,0.63673e+00_rb, &
        & 0.69046e+00_rb,0.82558e+00_rb,0.95705e+00_rb,0.11007e+01_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.11233e+01_rb,0.98347e+00_rb,0.84364e+00_rb,0.70381e+00_rb,0.61623e+00_rb, &
        & 0.67656e+00_rb,0.80855e+00_rb,0.93682e+00_rb,0.10775e+01_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.10954e+01_rb,0.95895e+00_rb,0.82254e+00_rb,0.68614e+00_rb,0.59850e+00_rb, &
        & 0.66513e+00_rb,0.79440e+00_rb,0.92006e+00_rb,0.10579e+01_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.13516e+01_rb,0.11834e+01_rb,0.10152e+01_rb,0.84752e+00_rb,0.80189e+00_rb, &
        & 0.86751e+00_rb,0.10388e+01_rb,0.12059e+01_rb,0.13885e+01_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.13129e+01_rb,0.11495e+01_rb,0.98601e+00_rb,0.82254e+00_rb,0.76801e+00_rb, &
        & 0.84476e+00_rb,0.10112e+01_rb,0.11733e+01_rb,0.13510e+01_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.12754e+01_rb,0.11166e+01_rb,0.95771e+00_rb,0.79885e+00_rb,0.73832e+00_rb, &
        & 0.82499e+00_rb,0.98705e+00_rb,0.11448e+01_rb,0.13179e+01_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.12391e+01_rb,0.10847e+01_rb,0.93032e+00_rb,0.77593e+00_rb,0.71267e+00_rb, &
        & 0.80837e+00_rb,0.96685e+00_rb,0.11210e+01_rb,0.12906e+01_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.12065e+01_rb,0.10561e+01_rb,0.90576e+00_rb,0.75539e+00_rb,0.69035e+00_rb, &
        & 0.79409e+00_rb,0.94938e+00_rb,0.11005e+01_rb,0.12668e+01_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.14988e+01_rb,0.13122e+01_rb,0.11257e+01_rb,0.95045e+00_rb,0.92736e+00_rb, &
        & 0.10268e+01_rb,0.12293e+01_rb,0.14274e+01_rb,0.16436e+01_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.14532e+01_rb,0.12722e+01_rb,0.10913e+01_rb,0.91652e+00_rb,0.88763e+00_rb, &
        & 0.10018e+01_rb,0.11994e+01_rb,0.13921e+01_rb,0.16030e+01_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.14089e+01_rb,0.12334e+01_rb,0.10579e+01_rb,0.88636e+00_rb,0.85277e+00_rb, &
        & 0.97910e+00_rb,0.11719e+01_rb,0.13598e+01_rb,0.15656e+01_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.13671e+01_rb,0.11967e+01_rb,0.10263e+01_rb,0.85849e+00_rb,0.82373e+00_rb, &
        & 0.95955e+00_rb,0.11482e+01_rb,0.13320e+01_rb,0.15338e+01_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.13292e+01_rb,0.11635e+01_rb,0.99774e+00_rb,0.83411e+00_rb,0.79958e+00_rb, &
        & 0.94221e+00_rb,0.11271e+01_rb,0.13073e+01_rb,0.15052e+01_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.16562e+01_rb,0.14500e+01_rb,0.12437e+01_rb,0.10742e+01_rb,0.10651e+01_rb, &
        & 0.12070e+01_rb,0.14445e+01_rb,0.16775e+01_rb,0.19311e+01_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.16034e+01_rb,0.14036e+01_rb,0.12039e+01_rb,0.10312e+01_rb,0.10206e+01_rb, &
        & 0.11791e+01_rb,0.14116e+01_rb,0.16391e+01_rb,0.18869e+01_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.15525e+01_rb,0.13590e+01_rb,0.11655e+01_rb,0.99233e+00_rb,0.98476e+00_rb, &
        & 0.11530e+01_rb,0.13804e+01_rb,0.16026e+01_rb,0.18449e+01_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.15047e+01_rb,0.13171e+01_rb,0.11295e+01_rb,0.95737e+00_rb,0.95592e+00_rb, &
        & 0.11301e+01_rb,0.13529e+01_rb,0.15703e+01_rb,0.18077e+01_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.14611e+01_rb,0.12789e+01_rb,0.10967e+01_rb,0.92635e+00_rb,0.93256e+00_rb, &
        & 0.11088e+01_rb,0.13272e+01_rb,0.15403e+01_rb,0.17730e+01_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.18225e+01_rb,0.15953e+01_rb,0.13682e+01_rb,0.12131e+01_rb,0.12195e+01_rb, &
        & 0.14041e+01_rb,0.16789e+01_rb,0.19504e+01_rb,0.22442e+01_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.17586e+01_rb,0.15393e+01_rb,0.13201e+01_rb,0.11608e+01_rb,0.11760e+01_rb, &
        & 0.13729e+01_rb,0.16424e+01_rb,0.19078e+01_rb,0.21951e+01_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.16992e+01_rb,0.14873e+01_rb,0.12754e+01_rb,0.11134e+01_rb,0.11408e+01_rb, &
        & 0.13430e+01_rb,0.16073e+01_rb,0.18666e+01_rb,0.21480e+01_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.16453e+01_rb,0.14401e+01_rb,0.12349e+01_rb,0.10705e+01_rb,0.11101e+01_rb, &
        & 0.13157e+01_rb,0.15751e+01_rb,0.18293e+01_rb,0.21045e+01_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.15976e+01_rb,0.13983e+01_rb,0.11990e+01_rb,0.10350e+01_rb,0.10845e+01_rb, &
        & 0.12900e+01_rb,0.15444e+01_rb,0.17936e+01_rb,0.20634e+01_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.19716e+01_rb,0.17256e+01_rb,0.14797e+01_rb,0.13408e+01_rb,0.13798e+01_rb, &
        & 0.16011e+01_rb,0.19108e+01_rb,0.22209e+01_rb,0.25534e+01_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.18980e+01_rb,0.16613e+01_rb,0.14245e+01_rb,0.12841e+01_rb,0.13393e+01_rb, &
        & 0.15661e+01_rb,0.18703e+01_rb,0.21736e+01_rb,0.24991e+01_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.18334e+01_rb,0.16046e+01_rb,0.13758e+01_rb,0.12346e+01_rb,0.13019e+01_rb, &
        & 0.15332e+01_rb,0.18320e+01_rb,0.21288e+01_rb,0.24477e+01_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.17833e+01_rb,0.15608e+01_rb,0.13382e+01_rb,0.11928e+01_rb,0.12697e+01_rb, &
        & 0.15019e+01_rb,0.17959e+01_rb,0.20867e+01_rb,0.23990e+01_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.17385e+01_rb,0.15215e+01_rb,0.13045e+01_rb,0.11584e+01_rb,0.12412e+01_rb, &
        & 0.14723e+01_rb,0.17610e+01_rb,0.20462e+01_rb,0.23526e+01_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.21322e+01_rb,0.18661e+01_rb,0.16009e+01_rb,0.14913e+01_rb,0.15626e+01_rb, &
        & 0.18065e+01_rb,0.21498e+01_rb,0.24997e+01_rb,0.28719e+01_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.20605e+01_rb,0.18033e+01_rb,0.15462e+01_rb,0.14346e+01_rb,0.15183e+01_rb, &
        & 0.17679e+01_rb,0.21055e+01_rb,0.24481e+01_rb,0.28126e+01_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.20007e+01_rb,0.17509e+01_rb,0.15011e+01_rb,0.13841e+01_rb,0.14777e+01_rb, &
        & 0.17312e+01_rb,0.20634e+01_rb,0.23990e+01_rb,0.27564e+01_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.19501e+01_rb,0.17066e+01_rb,0.14631e+01_rb,0.13415e+01_rb,0.14420e+01_rb, &
        & 0.16956e+01_rb,0.20218e+01_rb,0.23506e+01_rb,0.27007e+01_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.19023e+01_rb,0.16648e+01_rb,0.14272e+01_rb,0.13048e+01_rb,0.14093e+01_rb, &
        & 0.16619e+01_rb,0.19834e+01_rb,0.23060e+01_rb,0.26492e+01_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.23283e+01_rb,0.20376e+01_rb,0.17570e+01_rb,0.16678e+01_rb,0.17574e+01_rb, &
        & 0.20182e+01_rb,0.23943e+01_rb,0.27850e+01_rb,0.31979e+01_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.22552e+01_rb,0.19735e+01_rb,0.16974e+01_rb,0.16068e+01_rb,0.17079e+01_rb, &
        & 0.19757e+01_rb,0.23460e+01_rb,0.27289e+01_rb,0.31334e+01_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.21945e+01_rb,0.19204e+01_rb,0.16488e+01_rb,0.15544e+01_rb,0.16639e+01_rb, &
        & 0.19336e+01_rb,0.22972e+01_rb,0.26723e+01_rb,0.30681e+01_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.21377e+01_rb,0.18706e+01_rb,0.16045e+01_rb,0.15082e+01_rb,0.16235e+01_rb, &
        & 0.18922e+01_rb,0.22504e+01_rb,0.26179e+01_rb,0.30055e+01_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.20822e+01_rb,0.18221e+01_rb,0.15623e+01_rb,0.14670e+01_rb,0.15845e+01_rb, &
        & 0.18531e+01_rb,0.22061e+01_rb,0.25663e+01_rb,0.29464e+01_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.84891e+00_rb,0.74682e+00_rb,0.64540e+00_rb,0.54510e+00_rb,0.44671e+00_rb, &
        & 0.34895e+00_rb,0.38105e+00_rb,0.44177e+00_rb,0.47725e+00_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.82292e+00_rb,0.72425e+00_rb,0.62615e+00_rb,0.52904e+00_rb,0.43334e+00_rb, &
        & 0.33791e+00_rb,0.37758e+00_rb,0.43769e+00_rb,0.47292e+00_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.79766e+00_rb,0.70221e+00_rb,0.60725e+00_rb,0.51315e+00_rb,0.42001e+00_rb, &
        & 0.32708e+00_rb,0.37351e+00_rb,0.43297e+00_rb,0.46789e+00_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.77341e+00_rb,0.68098e+00_rb,0.58899e+00_rb,0.49774e+00_rb,0.40711e+00_rb, &
        & 0.31834e+00_rb,0.36899e+00_rb,0.42775e+00_rb,0.46227e+00_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.74989e+00_rb,0.66035e+00_rb,0.57119e+00_rb,0.48264e+00_rb,0.39437e+00_rb, &
        & 0.32430e+00_rb,0.36416e+00_rb,0.42216e+00_rb,0.45636e+00_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.98489e+00_rb,0.86457e+00_rb,0.74461e+00_rb,0.62525e+00_rb,0.50699e+00_rb, &
        & 0.39502e+00_rb,0.46270e+00_rb,0.53626e+00_rb,0.59246e+00_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.95110e+00_rb,0.83523e+00_rb,0.71967e+00_rb,0.60464e+00_rb,0.49055e+00_rb, &
        & 0.38714e+00_rb,0.45775e+00_rb,0.53066e+00_rb,0.58669e+00_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.91868e+00_rb,0.80699e+00_rb,0.69557e+00_rb,0.58461e+00_rb,0.47447e+00_rb, &
        & 0.38030e+00_rb,0.45287e+00_rb,0.52498e+00_rb,0.58057e+00_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.88783e+00_rb,0.78006e+00_rb,0.67253e+00_rb,0.56540e+00_rb,0.45894e+00_rb, &
        & 0.37426e+00_rb,0.44739e+00_rb,0.51869e+00_rb,0.57355e+00_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.85824e+00_rb,0.75420e+00_rb,0.65037e+00_rb,0.54691e+00_rb,0.44407e+00_rb, &
        & 0.37572e+00_rb,0.44151e+00_rb,0.51192e+00_rb,0.56619e+00_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.11382e+01_rb,0.99791e+00_rb,0.85785e+00_rb,0.71816e+00_rb,0.57913e+00_rb, &
        & 0.47378e+00_rb,0.56732e+00_rb,0.65745e+00_rb,0.74321e+00_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.10944e+01_rb,0.95984e+00_rb,0.82550e+00_rb,0.69149e+00_rb,0.55807e+00_rb, &
        & 0.46534e+00_rb,0.55716e+00_rb,0.64588e+00_rb,0.73020e+00_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.10534e+01_rb,0.92416e+00_rb,0.79509e+00_rb,0.66633e+00_rb,0.53808e+00_rb, &
        & 0.45863e+00_rb,0.54915e+00_rb,0.63663e+00_rb,0.71995e+00_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.10141e+01_rb,0.88986e+00_rb,0.76582e+00_rb,0.64206e+00_rb,0.51870e+00_rb, &
        & 0.45265e+00_rb,0.54212e+00_rb,0.62853e+00_rb,0.71089e+00_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.97757e+00_rb,0.85802e+00_rb,0.73862e+00_rb,0.61943e+00_rb,0.50068e+00_rb, &
        & 0.44732e+00_rb,0.53486e+00_rb,0.62017e+00_rb,0.70154e+00_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.13027e+01_rb,0.11412e+01_rb,0.97988e+00_rb,0.81881e+00_rb,0.65819e+00_rb, &
        & 0.58495e+00_rb,0.70040e+00_rb,0.81176e+00_rb,0.92845e+00_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.12473e+01_rb,0.10931e+01_rb,0.93894e+00_rb,0.78506e+00_rb,0.63160e+00_rb, &
        & 0.57073e+00_rb,0.68340e+00_rb,0.79216e+00_rb,0.90610e+00_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.11956e+01_rb,0.10481e+01_rb,0.90063e+00_rb,0.75344e+00_rb,0.60663e+00_rb, &
        & 0.55958e+00_rb,0.67003e+00_rb,0.77669e+00_rb,0.88836e+00_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.11480e+01_rb,0.10066e+01_rb,0.86532e+00_rb,0.72426e+00_rb,0.58351e+00_rb, &
        & 0.54965e+00_rb,0.65819e+00_rb,0.76307e+00_rb,0.87291e+00_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.11042e+01_rb,0.96837e+00_rb,0.83267e+00_rb,0.69709e+00_rb,0.56180e+00_rb, &
        & 0.54039e+00_rb,0.64710e+00_rb,0.75045e+00_rb,0.85825e+00_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.14755e+01_rb,0.12919e+01_rb,0.11084e+01_rb,0.92503e+00_rb,0.74199e+00_rb, &
        & 0.72656e+00_rb,0.86992e+00_rb,0.10084e+01_rb,0.11593e+01_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.14077e+01_rb,0.12330e+01_rb,0.10584e+01_rb,0.88396e+00_rb,0.70985e+00_rb, &
        & 0.70337e+00_rb,0.84221e+00_rb,0.97628e+00_rb,0.11223e+01_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.13473e+01_rb,0.11805e+01_rb,0.10138e+01_rb,0.84723e+00_rb,0.68084e+00_rb, &
        & 0.68573e+00_rb,0.82117e+00_rb,0.95190e+00_rb,0.10943e+01_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.12931e+01_rb,0.11332e+01_rb,0.97336e+00_rb,0.81358e+00_rb,0.65399e+00_rb, &
        & 0.67083e+00_rb,0.80321e+00_rb,0.93120e+00_rb,0.10704e+01_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.12425e+01_rb,0.10889e+01_rb,0.93535e+00_rb,0.78185e+00_rb,0.62849e+00_rb, &
        & 0.65706e+00_rb,0.78689e+00_rb,0.91252e+00_rb,0.10486e+01_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.16632e+01_rb,0.14561e+01_rb,0.12491e+01_rb,0.10422e+01_rb,0.83545e+00_rb, &
        & 0.90592e+00_rb,0.10850e+01_rb,0.12579e+01_rb,0.14491e+01_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.15885e+01_rb,0.13911e+01_rb,0.11938e+01_rb,0.99649e+00_rb,0.79932e+00_rb, &
        & 0.87105e+00_rb,0.10430e+01_rb,0.12092e+01_rb,0.13933e+01_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.15219e+01_rb,0.13330e+01_rb,0.11441e+01_rb,0.95525e+00_rb,0.76651e+00_rb, &
        & 0.84460e+00_rb,0.10114e+01_rb,0.11726e+01_rb,0.13510e+01_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.14602e+01_rb,0.12790e+01_rb,0.10978e+01_rb,0.91668e+00_rb,0.73549e+00_rb, &
        & 0.82233e+00_rb,0.98468e+00_rb,0.11417e+01_rb,0.13152e+01_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.14027e+01_rb,0.12286e+01_rb,0.10546e+01_rb,0.88053e+00_rb,0.70846e+00_rb, &
        & 0.80220e+00_rb,0.96072e+00_rb,0.11140e+01_rb,0.12832e+01_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.18802e+01_rb,0.16460e+01_rb,0.14119e+01_rb,0.11779e+01_rb,0.95036e+00_rb, &
        & 0.11297e+01_rb,0.13528e+01_rb,0.15688e+01_rb,0.18094e+01_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.17989e+01_rb,0.15751e+01_rb,0.13512e+01_rb,0.11274e+01_rb,0.91055e+00_rb, &
        & 0.10797e+01_rb,0.12930e+01_rb,0.14993e+01_rb,0.17294e+01_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.17245e+01_rb,0.15100e+01_rb,0.12954e+01_rb,0.10809e+01_rb,0.87508e+00_rb, &
        & 0.10422e+01_rb,0.12481e+01_rb,0.14473e+01_rb,0.16693e+01_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.16543e+01_rb,0.14485e+01_rb,0.12427e+01_rb,0.10370e+01_rb,0.84724e+00_rb, &
        & 0.10105e+01_rb,0.12102e+01_rb,0.14035e+01_rb,0.16185e+01_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.15884e+01_rb,0.13908e+01_rb,0.11932e+01_rb,0.99569e+00_rb,0.82331e+00_rb, &
        & 0.98227e+00_rb,0.11764e+01_rb,0.13643e+01_rb,0.15734e+01_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.21327e+01_rb,0.18670e+01_rb,0.16012e+01_rb,0.13355e+01_rb,0.11430e+01_rb, &
        & 0.14062e+01_rb,0.16843e+01_rb,0.19535e+01_rb,0.22540e+01_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.20434e+01_rb,0.17888e+01_rb,0.15342e+01_rb,0.12796e+01_rb,0.10891e+01_rb, &
        & 0.13377e+01_rb,0.16020e+01_rb,0.18582e+01_rb,0.21442e+01_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.19592e+01_rb,0.17151e+01_rb,0.14711e+01_rb,0.12270e+01_rb,0.10443e+01_rb, &
        & 0.12854e+01_rb,0.15395e+01_rb,0.17855e+01_rb,0.20605e+01_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.18795e+01_rb,0.16454e+01_rb,0.14112e+01_rb,0.11771e+01_rb,0.10093e+01_rb, &
        & 0.12418e+01_rb,0.14873e+01_rb,0.17251e+01_rb,0.19905e+01_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.18040e+01_rb,0.15793e+01_rb,0.13546e+01_rb,0.11298e+01_rb,0.97893e+00_rb, &
        & 0.12029e+01_rb,0.14409e+01_rb,0.16715e+01_rb,0.19284e+01_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.24263e+01_rb,0.21237e+01_rb,0.18212e+01_rb,0.15186e+01_rb,0.14038e+01_rb, &
        & 0.17468e+01_rb,0.20923e+01_rb,0.24275e+01_rb,0.28010e+01_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.23266e+01_rb,0.20365e+01_rb,0.17464e+01_rb,0.14563e+01_rb,0.13334e+01_rb, &
        & 0.16567e+01_rb,0.19843e+01_rb,0.23023e+01_rb,0.26565e+01_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.22311e+01_rb,0.19529e+01_rb,0.16747e+01_rb,0.13965e+01_rb,0.12741e+01_rb, &
        & 0.15855e+01_rb,0.18989e+01_rb,0.22031e+01_rb,0.25421e+01_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.21400e+01_rb,0.18731e+01_rb,0.16063e+01_rb,0.13394e+01_rb,0.12261e+01_rb, &
        & 0.15261e+01_rb,0.18280e+01_rb,0.21210e+01_rb,0.24473e+01_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.20537e+01_rb,0.17976e+01_rb,0.15415e+01_rb,0.12854e+01_rb,0.11844e+01_rb, &
        & 0.14739e+01_rb,0.17653e+01_rb,0.20482e+01_rb,0.23631e+01_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.27556e+01_rb,0.24117e+01_rb,0.20679e+01_rb,0.17241e+01_rb,0.17250e+01_rb, &
        & 0.21531e+01_rb,0.25793e+01_rb,0.29930e+01_rb,0.34529e+01_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.26436e+01_rb,0.23137e+01_rb,0.19838e+01_rb,0.16540e+01_rb,0.16343e+01_rb, &
        & 0.20375e+01_rb,0.24406e+01_rb,0.28326e+01_rb,0.32672e+01_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.25350e+01_rb,0.22187e+01_rb,0.19024e+01_rb,0.15861e+01_rb,0.15582e+01_rb, &
        & 0.19443e+01_rb,0.23290e+01_rb,0.27026e+01_rb,0.31178e+01_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.24300e+01_rb,0.21267e+01_rb,0.18235e+01_rb,0.15203e+01_rb,0.14955e+01_rb, &
        & 0.18662e+01_rb,0.22353e+01_rb,0.25942e+01_rb,0.29924e+01_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.23355e+01_rb,0.20440e+01_rb,0.17526e+01_rb,0.14611e+01_rb,0.14399e+01_rb, &
        & 0.17966e+01_rb,0.21521e+01_rb,0.24975e+01_rb,0.28809e+01_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.30736e+01_rb,0.26899e+01_rb,0.23062e+01_rb,0.19287e+01_rb,0.20594e+01_rb, &
        & 0.25707e+01_rb,0.30796e+01_rb,0.35755e+01_rb,0.41218e+01_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.29458e+01_rb,0.25780e+01_rb,0.22102e+01_rb,0.18490e+01_rb,0.19506e+01_rb, &
        & 0.24353e+01_rb,0.29176e+01_rb,0.33867e+01_rb,0.39048e+01_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.28287e+01_rb,0.24755e+01_rb,0.21223e+01_rb,0.17723e+01_rb,0.18621e+01_rb, &
        & 0.23252e+01_rb,0.27855e+01_rb,0.32334e+01_rb,0.37281e+01_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.27181e+01_rb,0.23787e+01_rb,0.20393e+01_rb,0.17010e+01_rb,0.17848e+01_rb, &
        & 0.22284e+01_rb,0.26698e+01_rb,0.30991e+01_rb,0.35730e+01_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.26214e+01_rb,0.22941e+01_rb,0.19667e+01_rb,0.16394e+01_rb,0.17149e+01_rb, &
        & 0.21409e+01_rb,0.25649e+01_rb,0.29778e+01_rb,0.34326e+01_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.34457e+01_rb,0.30154e+01_rb,0.25850e+01_rb,0.22002e+01_rb,0.24421e+01_rb, &
        & 0.30491e+01_rb,0.36532e+01_rb,0.42422e+01_rb,0.48881e+01_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.33132e+01_rb,0.28993e+01_rb,0.24855e+01_rb,0.21037e+01_rb,0.23175e+01_rb, &
        & 0.28938e+01_rb,0.34673e+01_rb,0.40262e+01_rb,0.46397e+01_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.31843e+01_rb,0.27866e+01_rb,0.23888e+01_rb,0.20132e+01_rb,0.22107e+01_rb, &
        & 0.27603e+01_rb,0.33070e+01_rb,0.38403e+01_rb,0.44248e+01_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.30610e+01_rb,0.26787e+01_rb,0.22963e+01_rb,0.19289e+01_rb,0.21145e+01_rb, &
        & 0.26402e+01_rb,0.31632e+01_rb,0.36735e+01_rb,0.42326e+01_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.29533e+01_rb,0.25844e+01_rb,0.22155e+01_rb,0.18569e+01_rb,0.20264e+01_rb, &
        & 0.25300e+01_rb,0.30314e+01_rb,0.35205e+01_rb,0.40560e+01_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.38898e+01_rb,0.34039e+01_rb,0.29179e+01_rb,0.25320e+01_rb,0.28809e+01_rb, &
        & 0.35965e+01_rb,0.43091e+01_rb,0.50055e+01_rb,0.57650e+01_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.37371e+01_rb,0.32702e+01_rb,0.28033e+01_rb,0.24146e+01_rb,0.27330e+01_rb, &
        & 0.34120e+01_rb,0.40886e+01_rb,0.47494e+01_rb,0.54692e+01_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.35846e+01_rb,0.31367e+01_rb,0.26889e+01_rb,0.23047e+01_rb,0.26019e+01_rb, &
        & 0.32482e+01_rb,0.38921e+01_rb,0.45212e+01_rb,0.52066e+01_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.34450e+01_rb,0.30146e+01_rb,0.25842e+01_rb,0.22048e+01_rb,0.24829e+01_rb, &
        & 0.31006e+01_rb,0.37153e+01_rb,0.43162e+01_rb,0.49694e+01_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.33226e+01_rb,0.29075e+01_rb,0.24923e+01_rb,0.21144e+01_rb,0.23741e+01_rb, &
        & 0.29651e+01_rb,0.35532e+01_rb,0.41282e+01_rb,0.47532e+01_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:,13, 1) = (/ &
        & 0.78344e-03_rb,0.81478e-03_rb,0.83713e-03_rb,0.85265e-03_rb,0.85991e-03_rb /)
      kbo(:,14, 1) = (/ &
        & 0.65374e-03_rb,0.67963e-03_rb,0.69840e-03_rb,0.71131e-03_rb,0.71719e-03_rb /)
      kbo(:,15, 1) = (/ &
        & 0.55587e-03_rb,0.57751e-03_rb,0.59343e-03_rb,0.60410e-03_rb,0.60993e-03_rb /)
      kbo(:,16, 1) = (/ &
        & 0.48145e-03_rb,0.49780e-03_rb,0.51251e-03_rb,0.52093e-03_rb,0.52706e-03_rb /)
      kbo(:,17, 1) = (/ &
        & 0.42610e-03_rb,0.43577e-03_rb,0.44892e-03_rb,0.45812e-03_rb,0.46352e-03_rb /)
      kbo(:,18, 1) = (/ &
        & 0.38503e-03_rb,0.38849e-03_rb,0.40094e-03_rb,0.41067e-03_rb,0.41529e-03_rb /)
      kbo(:,19, 1) = (/ &
        & 0.35025e-03_rb,0.34902e-03_rb,0.35803e-03_rb,0.36759e-03_rb,0.37429e-03_rb /)
      kbo(:,20, 1) = (/ &
        & 0.30557e-03_rb,0.30381e-03_rb,0.30983e-03_rb,0.31782e-03_rb,0.32429e-03_rb /)
      kbo(:,21, 1) = (/ &
        & 0.26346e-03_rb,0.26127e-03_rb,0.26616e-03_rb,0.27270e-03_rb,0.27830e-03_rb /)
      kbo(:,22, 1) = (/ &
        & 0.22630e-03_rb,0.22463e-03_rb,0.22867e-03_rb,0.23419e-03_rb,0.23829e-03_rb /)
      kbo(:,23, 1) = (/ &
        & 0.19498e-03_rb,0.19376e-03_rb,0.19644e-03_rb,0.20096e-03_rb,0.20404e-03_rb /)
      kbo(:,24, 1) = (/ &
        & 0.16839e-03_rb,0.16687e-03_rb,0.16820e-03_rb,0.17215e-03_rb,0.17490e-03_rb /)
      kbo(:,25, 1) = (/ &
        & 0.14494e-03_rb,0.14347e-03_rb,0.14334e-03_rb,0.14728e-03_rb,0.14954e-03_rb /)
      kbo(:,26, 1) = (/ &
        & 0.12363e-03_rb,0.12241e-03_rb,0.12233e-03_rb,0.12518e-03_rb,0.12767e-03_rb /)
      kbo(:,27, 1) = (/ &
        & 0.10560e-03_rb,0.10482e-03_rb,0.10446e-03_rb,0.10681e-03_rb,0.10897e-03_rb /)
      kbo(:,28, 1) = (/ &
        & 0.90453e-04_rb,0.89921e-04_rb,0.89199e-04_rb,0.91278e-04_rb,0.93235e-04_rb /)
      kbo(:,29, 1) = (/ &
        & 0.77854e-04_rb,0.77467e-04_rb,0.76947e-04_rb,0.77977e-04_rb,0.79713e-04_rb /)
      kbo(:,30, 1) = (/ &
        & 0.67346e-04_rb,0.67074e-04_rb,0.66981e-04_rb,0.67340e-04_rb,0.68453e-04_rb /)
      kbo(:,31, 1) = (/ &
        & 0.58739e-04_rb,0.58583e-04_rb,0.58517e-04_rb,0.58421e-04_rb,0.59277e-04_rb /)
      kbo(:,32, 1) = (/ &
        & 0.51671e-04_rb,0.51476e-04_rb,0.51254e-04_rb,0.51199e-04_rb,0.51510e-04_rb /)
      kbo(:,33, 1) = (/ &
        & 0.45523e-04_rb,0.45406e-04_rb,0.45028e-04_rb,0.44849e-04_rb,0.44977e-04_rb /)
      kbo(:,34, 1) = (/ &
        & 0.40256e-04_rb,0.40062e-04_rb,0.39751e-04_rb,0.39416e-04_rb,0.39181e-04_rb /)
      kbo(:,35, 1) = (/ &
        & 0.35762e-04_rb,0.35429e-04_rb,0.35245e-04_rb,0.34916e-04_rb,0.34484e-04_rb /)
      kbo(:,36, 1) = (/ &
        & 0.31776e-04_rb,0.31666e-04_rb,0.31200e-04_rb,0.31013e-04_rb,0.30768e-04_rb /)
      kbo(:,37, 1) = (/ &
        & 0.26934e-04_rb,0.26971e-04_rb,0.26615e-04_rb,0.26355e-04_rb,0.26214e-04_rb /)
      kbo(:,38, 1) = (/ &
        & 0.22652e-04_rb,0.22874e-04_rb,0.22686e-04_rb,0.22466e-04_rb,0.22266e-04_rb /)
      kbo(:,39, 1) = (/ &
        & 0.18970e-04_rb,0.19273e-04_rb,0.19385e-04_rb,0.19198e-04_rb,0.19016e-04_rb /)
      kbo(:,40, 1) = (/ &
        & 0.15533e-04_rb,0.15831e-04_rb,0.15964e-04_rb,0.15836e-04_rb,0.15700e-04_rb /)
      kbo(:,41, 1) = (/ &
        & 0.12668e-04_rb,0.12826e-04_rb,0.12999e-04_rb,0.13018e-04_rb,0.12872e-04_rb /)
      kbo(:,42, 1) = (/ &
        & 0.10356e-04_rb,0.10475e-04_rb,0.10620e-04_rb,0.10666e-04_rb,0.10580e-04_rb /)
      kbo(:,43, 1) = (/ &
        & 0.83551e-05_rb,0.84490e-05_rb,0.85217e-05_rb,0.86140e-05_rb,0.86044e-05_rb /)
      kbo(:,44, 1) = (/ &
        & 0.66829e-05_rb,0.68022e-05_rb,0.68791e-05_rb,0.69429e-05_rb,0.69314e-05_rb /)
      kbo(:,45, 1) = (/ &
        & 0.53860e-05_rb,0.54494e-05_rb,0.55341e-05_rb,0.55544e-05_rb,0.55761e-05_rb /)
      kbo(:,46, 1) = (/ &
        & 0.43207e-05_rb,0.43601e-05_rb,0.44477e-05_rb,0.44761e-05_rb,0.44727e-05_rb /)
      kbo(:,47, 1) = (/ &
        & 0.34852e-05_rb,0.34924e-05_rb,0.35555e-05_rb,0.35928e-05_rb,0.35957e-05_rb /)
      kbo(:,48, 1) = (/ &
        & 0.27913e-05_rb,0.28024e-05_rb,0.28395e-05_rb,0.28713e-05_rb,0.28836e-05_rb /)
      kbo(:,49, 1) = (/ &
        & 0.22267e-05_rb,0.22549e-05_rb,0.22736e-05_rb,0.22769e-05_rb,0.23083e-05_rb /)
      kbo(:,50, 1) = (/ &
        & 0.17814e-05_rb,0.18056e-05_rb,0.18272e-05_rb,0.18178e-05_rb,0.18340e-05_rb /)
      kbo(:,51, 1) = (/ &
        & 0.14202e-05_rb,0.14390e-05_rb,0.14605e-05_rb,0.14515e-05_rb,0.14607e-05_rb /)
      kbo(:,52, 1) = (/ &
        & 0.11328e-05_rb,0.11511e-05_rb,0.11629e-05_rb,0.11627e-05_rb,0.11618e-05_rb /)
      kbo(:,53, 1) = (/ &
        & 0.90478e-06_rb,0.91291e-06_rb,0.92251e-06_rb,0.92910e-06_rb,0.92898e-06_rb /)
      kbo(:,54, 1) = (/ &
        & 0.71737e-06_rb,0.72444e-06_rb,0.73271e-06_rb,0.73646e-06_rb,0.74079e-06_rb /)
      kbo(:,55, 1) = (/ &
        & 0.56522e-06_rb,0.57444e-06_rb,0.57690e-06_rb,0.58444e-06_rb,0.59006e-06_rb /)
      kbo(:,56, 1) = (/ &
        & 0.44522e-06_rb,0.45152e-06_rb,0.45592e-06_rb,0.46208e-06_rb,0.46721e-06_rb /)
      kbo(:,57, 1) = (/ &
        & 0.35045e-06_rb,0.35432e-06_rb,0.36035e-06_rb,0.36444e-06_rb,0.36942e-06_rb /)
      kbo(:,58, 1) = (/ &
        & 0.27513e-06_rb,0.27793e-06_rb,0.28330e-06_rb,0.28729e-06_rb,0.29229e-06_rb /)
      kbo(:,59, 1) = (/ &
        & 0.21608e-06_rb,0.21966e-06_rb,0.22301e-06_rb,0.22740e-06_rb,0.23129e-06_rb /)
      kbo(:,13, 2) = (/ &
        & 0.41669e-02_rb,0.42141e-02_rb,0.42434e-02_rb,0.42683e-02_rb,0.42755e-02_rb /)
      kbo(:,14, 2) = (/ &
        & 0.34875e-02_rb,0.35201e-02_rb,0.35377e-02_rb,0.35547e-02_rb,0.35629e-02_rb /)
      kbo(:,15, 2) = (/ &
        & 0.29417e-02_rb,0.29682e-02_rb,0.29797e-02_rb,0.29877e-02_rb,0.29948e-02_rb /)
      kbo(:,16, 2) = (/ &
        & 0.25072e-02_rb,0.25337e-02_rb,0.25373e-02_rb,0.25423e-02_rb,0.25464e-02_rb /)
      kbo(:,17, 2) = (/ &
        & 0.21640e-02_rb,0.21948e-02_rb,0.21974e-02_rb,0.21951e-02_rb,0.21930e-02_rb /)
      kbo(:,18, 2) = (/ &
        & 0.18910e-02_rb,0.19229e-02_rb,0.19301e-02_rb,0.19201e-02_rb,0.19153e-02_rb /)
      kbo(:,19, 2) = (/ &
        & 0.16495e-02_rb,0.16848e-02_rb,0.16944e-02_rb,0.16885e-02_rb,0.16779e-02_rb /)
      kbo(:,20, 2) = (/ &
        & 0.14018e-02_rb,0.14333e-02_rb,0.14424e-02_rb,0.14394e-02_rb,0.14287e-02_rb /)
      kbo(:,21, 2) = (/ &
        & 0.11832e-02_rb,0.12125e-02_rb,0.12189e-02_rb,0.12151e-02_rb,0.12082e-02_rb /)
      kbo(:,22, 2) = (/ &
        & 0.99669e-03_rb,0.10233e-02_rb,0.10268e-02_rb,0.10250e-02_rb,0.10191e-02_rb /)
      kbo(:,23, 2) = (/ &
        & 0.83734e-03_rb,0.86220e-03_rb,0.86642e-03_rb,0.86487e-03_rb,0.86054e-03_rb /)
      kbo(:,24, 2) = (/ &
        & 0.70349e-03_rb,0.72654e-03_rb,0.73243e-03_rb,0.73077e-03_rb,0.72754e-03_rb /)
      kbo(:,25, 2) = (/ &
        & 0.59070e-03_rb,0.61044e-03_rb,0.61929e-03_rb,0.61773e-03_rb,0.61583e-03_rb /)
      kbo(:,26, 2) = (/ &
        & 0.49716e-03_rb,0.51139e-03_rb,0.52115e-03_rb,0.52139e-03_rb,0.52023e-03_rb /)
      kbo(:,27, 2) = (/ &
        & 0.41822e-03_rb,0.42737e-03_rb,0.43808e-03_rb,0.43978e-03_rb,0.43884e-03_rb /)
      kbo(:,28, 2) = (/ &
        & 0.35061e-03_rb,0.35848e-03_rb,0.36841e-03_rb,0.37095e-03_rb,0.37053e-03_rb /)
      kbo(:,29, 2) = (/ &
        & 0.29391e-03_rb,0.30227e-03_rb,0.30912e-03_rb,0.31376e-03_rb,0.31382e-03_rb /)
      kbo(:,30, 2) = (/ &
        & 0.24878e-03_rb,0.25551e-03_rb,0.25990e-03_rb,0.26493e-03_rb,0.26717e-03_rb /)
      kbo(:,31, 2) = (/ &
        & 0.21146e-03_rb,0.21606e-03_rb,0.21973e-03_rb,0.22385e-03_rb,0.22711e-03_rb /)
      kbo(:,32, 2) = (/ &
        & 0.17989e-03_rb,0.18248e-03_rb,0.18678e-03_rb,0.18914e-03_rb,0.19265e-03_rb /)
      kbo(:,33, 2) = (/ &
        & 0.15377e-03_rb,0.15569e-03_rb,0.15798e-03_rb,0.16109e-03_rb,0.16313e-03_rb /)
      kbo(:,34, 2) = (/ &
        & 0.13129e-03_rb,0.13289e-03_rb,0.13431e-03_rb,0.13679e-03_rb,0.13869e-03_rb /)
      kbo(:,35, 2) = (/ &
        & 0.11225e-03_rb,0.11370e-03_rb,0.11480e-03_rb,0.11616e-03_rb,0.11774e-03_rb /)
      kbo(:,36, 2) = (/ &
        & 0.96722e-04_rb,0.97093e-04_rb,0.98641e-04_rb,0.99964e-04_rb,0.10029e-03_rb /)
      kbo(:,37, 2) = (/ &
        & 0.81278e-04_rb,0.81350e-04_rb,0.82442e-04_rb,0.83555e-04_rb,0.83990e-04_rb /)
      kbo(:,38, 2) = (/ &
        & 0.68481e-04_rb,0.68398e-04_rb,0.68899e-04_rb,0.69737e-04_rb,0.70187e-04_rb /)
      kbo(:,39, 2) = (/ &
        & 0.57588e-04_rb,0.57895e-04_rb,0.57814e-04_rb,0.58095e-04_rb,0.58861e-04_rb /)
      kbo(:,40, 2) = (/ &
        & 0.47541e-04_rb,0.47609e-04_rb,0.47587e-04_rb,0.47727e-04_rb,0.48149e-04_rb /)
      kbo(:,41, 2) = (/ &
        & 0.39099e-04_rb,0.39119e-04_rb,0.39164e-04_rb,0.39161e-04_rb,0.39530e-04_rb /)
      kbo(:,42, 2) = (/ &
        & 0.32129e-04_rb,0.32067e-04_rb,0.32113e-04_rb,0.32128e-04_rb,0.32332e-04_rb /)
      kbo(:,43, 2) = (/ &
        & 0.26339e-04_rb,0.26195e-04_rb,0.26218e-04_rb,0.26313e-04_rb,0.26301e-04_rb /)
      kbo(:,44, 2) = (/ &
        & 0.21510e-04_rb,0.21341e-04_rb,0.21297e-04_rb,0.21378e-04_rb,0.21395e-04_rb /)
      kbo(:,45, 2) = (/ &
        & 0.17530e-04_rb,0.17454e-04_rb,0.17341e-04_rb,0.17352e-04_rb,0.17454e-04_rb /)
      kbo(:,46, 2) = (/ &
        & 0.14267e-04_rb,0.14213e-04_rb,0.14118e-04_rb,0.14076e-04_rb,0.14192e-04_rb /)
      kbo(:,47, 2) = (/ &
        & 0.11510e-04_rb,0.11593e-04_rb,0.11522e-04_rb,0.11431e-04_rb,0.11494e-04_rb /)
      kbo(:,48, 2) = (/ &
        & 0.92607e-05_rb,0.93963e-05_rb,0.93931e-05_rb,0.93302e-05_rb,0.93175e-05_rb /)
      kbo(:,49, 2) = (/ &
        & 0.74566e-05_rb,0.75640e-05_rb,0.76551e-05_rb,0.76195e-05_rb,0.75706e-05_rb /)
      kbo(:,50, 2) = (/ &
        & 0.60069e-05_rb,0.60789e-05_rb,0.61866e-05_rb,0.62136e-05_rb,0.61878e-05_rb /)
      kbo(:,51, 2) = (/ &
        & 0.48719e-05_rb,0.49189e-05_rb,0.49852e-05_rb,0.50599e-05_rb,0.50406e-05_rb /)
      kbo(:,52, 2) = (/ &
        & 0.39542e-05_rb,0.39622e-05_rb,0.40023e-05_rb,0.40853e-05_rb,0.41134e-05_rb /)
      kbo(:,53, 2) = (/ &
        & 0.31898e-05_rb,0.32108e-05_rb,0.32348e-05_rb,0.32873e-05_rb,0.33341e-05_rb /)
      kbo(:,54, 2) = (/ &
        & 0.25786e-05_rb,0.26009e-05_rb,0.26192e-05_rb,0.26504e-05_rb,0.26983e-05_rb /)
      kbo(:,55, 2) = (/ &
        & 0.20942e-05_rb,0.21017e-05_rb,0.21274e-05_rb,0.21442e-05_rb,0.21775e-05_rb /)
      kbo(:,56, 2) = (/ &
        & 0.16959e-05_rb,0.17018e-05_rb,0.17251e-05_rb,0.17374e-05_rb,0.17575e-05_rb /)
      kbo(:,57, 2) = (/ &
        & 0.13662e-05_rb,0.13805e-05_rb,0.13926e-05_rb,0.14091e-05_rb,0.14210e-05_rb /)
      kbo(:,58, 2) = (/ &
        & 0.10989e-05_rb,0.11199e-05_rb,0.11269e-05_rb,0.11436e-05_rb,0.11502e-05_rb /)
      kbo(:,59, 2) = (/ &
        & 0.88647e-06_rb,0.91054e-06_rb,0.91770e-06_rb,0.92872e-06_rb,0.93619e-06_rb /)
      kbo(:,13, 3) = (/ &
        & 0.91432e-02_rb,0.90907e-02_rb,0.90565e-02_rb,0.89984e-02_rb,0.89385e-02_rb /)
      kbo(:,14, 3) = (/ &
        & 0.76455e-02_rb,0.75988e-02_rb,0.75709e-02_rb,0.75150e-02_rb,0.74448e-02_rb /)
      kbo(:,15, 3) = (/ &
        & 0.64231e-02_rb,0.63826e-02_rb,0.63449e-02_rb,0.62999e-02_rb,0.62412e-02_rb /)
      kbo(:,16, 3) = (/ &
        & 0.54475e-02_rb,0.53998e-02_rb,0.53630e-02_rb,0.53153e-02_rb,0.52574e-02_rb /)
      kbo(:,17, 3) = (/ &
        & 0.46876e-02_rb,0.46363e-02_rb,0.45869e-02_rb,0.45398e-02_rb,0.44911e-02_rb /)
      kbo(:,18, 3) = (/ &
        & 0.40806e-02_rb,0.40265e-02_rb,0.39670e-02_rb,0.39185e-02_rb,0.38714e-02_rb /)
      kbo(:,19, 3) = (/ &
        & 0.35897e-02_rb,0.35272e-02_rb,0.34695e-02_rb,0.34174e-02_rb,0.33721e-02_rb /)
      kbo(:,20, 3) = (/ &
        & 0.30557e-02_rb,0.30003e-02_rb,0.29518e-02_rb,0.29027e-02_rb,0.28623e-02_rb /)
      kbo(:,21, 3) = (/ &
        & 0.25840e-02_rb,0.25346e-02_rb,0.24907e-02_rb,0.24530e-02_rb,0.24134e-02_rb /)
      kbo(:,22, 3) = (/ &
        & 0.21784e-02_rb,0.21342e-02_rb,0.20972e-02_rb,0.20639e-02_rb,0.20310e-02_rb /)
      kbo(:,23, 3) = (/ &
        & 0.18415e-02_rb,0.17987e-02_rb,0.17683e-02_rb,0.17405e-02_rb,0.17162e-02_rb /)
      kbo(:,24, 3) = (/ &
        & 0.15599e-02_rb,0.15210e-02_rb,0.14940e-02_rb,0.14715e-02_rb,0.14516e-02_rb /)
      kbo(:,25, 3) = (/ &
        & 0.13214e-02_rb,0.12882e-02_rb,0.12631e-02_rb,0.12449e-02_rb,0.12269e-02_rb /)
      kbo(:,26, 3) = (/ &
        & 0.11148e-02_rb,0.10885e-02_rb,0.10663e-02_rb,0.10485e-02_rb,0.10339e-02_rb /)
      kbo(:,27, 3) = (/ &
        & 0.94180e-03_rb,0.92291e-03_rb,0.90201e-03_rb,0.88582e-03_rb,0.87360e-03_rb /)
      kbo(:,28, 3) = (/ &
        & 0.79929e-03_rb,0.78315e-03_rb,0.76525e-03_rb,0.75093e-03_rb,0.73943e-03_rb /)
      kbo(:,29, 3) = (/ &
        & 0.68110e-03_rb,0.66580e-03_rb,0.65234e-03_rb,0.63866e-03_rb,0.62757e-03_rb /)
      kbo(:,30, 3) = (/ &
        & 0.57975e-03_rb,0.56820e-03_rb,0.55835e-03_rb,0.54581e-03_rb,0.53624e-03_rb /)
      kbo(:,31, 3) = (/ &
        & 0.49551e-03_rb,0.48804e-03_rb,0.47841e-03_rb,0.46871e-03_rb,0.45833e-03_rb /)
      kbo(:,32, 3) = (/ &
        & 0.42572e-03_rb,0.42052e-03_rb,0.41099e-03_rb,0.40377e-03_rb,0.39554e-03_rb /)
      kbo(:,33, 3) = (/ &
        & 0.36528e-03_rb,0.36135e-03_rb,0.35701e-03_rb,0.34948e-03_rb,0.34456e-03_rb /)
      kbo(:,34, 3) = (/ &
        & 0.31415e-03_rb,0.31283e-03_rb,0.30989e-03_rb,0.30612e-03_rb,0.30054e-03_rb /)
      kbo(:,35, 3) = (/ &
        & 0.27352e-03_rb,0.27102e-03_rb,0.26979e-03_rb,0.26826e-03_rb,0.26448e-03_rb /)
      kbo(:,36, 3) = (/ &
        & 0.23967e-03_rb,0.23742e-03_rb,0.23456e-03_rb,0.23474e-03_rb,0.23316e-03_rb /)
      kbo(:,37, 3) = (/ &
        & 0.20338e-03_rb,0.20142e-03_rb,0.19919e-03_rb,0.19875e-03_rb,0.19818e-03_rb /)
      kbo(:,38, 3) = (/ &
        & 0.17257e-03_rb,0.17079e-03_rb,0.16890e-03_rb,0.16818e-03_rb,0.16767e-03_rb /)
      kbo(:,39, 3) = (/ &
        & 0.14733e-03_rb,0.14508e-03_rb,0.14396e-03_rb,0.14263e-03_rb,0.14203e-03_rb /)
      kbo(:,40, 3) = (/ &
        & 0.12225e-03_rb,0.12069e-03_rb,0.11949e-03_rb,0.11833e-03_rb,0.11808e-03_rb /)
      kbo(:,41, 3) = (/ &
        & 0.10097e-03_rb,0.10002e-03_rb,0.98765e-04_rb,0.97895e-04_rb,0.97754e-04_rb /)
      kbo(:,42, 3) = (/ &
        & 0.83346e-04_rb,0.82796e-04_rb,0.81713e-04_rb,0.81007e-04_rb,0.80814e-04_rb /)
      kbo(:,43, 3) = (/ &
        & 0.68346e-04_rb,0.68043e-04_rb,0.67367e-04_rb,0.66543e-04_rb,0.66059e-04_rb /)
      kbo(:,44, 3) = (/ &
        & 0.55818e-04_rb,0.55703e-04_rb,0.55284e-04_rb,0.54584e-04_rb,0.54239e-04_rb /)
      kbo(:,45, 3) = (/ &
        & 0.45332e-04_rb,0.45541e-04_rb,0.45337e-04_rb,0.44929e-04_rb,0.44433e-04_rb /)
      kbo(:,46, 3) = (/ &
        & 0.36748e-04_rb,0.37223e-04_rb,0.37146e-04_rb,0.36884e-04_rb,0.36439e-04_rb /)
      kbo(:,47, 3) = (/ &
        & 0.29601e-04_rb,0.30289e-04_rb,0.30405e-04_rb,0.30257e-04_rb,0.29929e-04_rb /)
      kbo(:,48, 3) = (/ &
        & 0.23826e-04_rb,0.24579e-04_rb,0.24815e-04_rb,0.24762e-04_rb,0.24572e-04_rb /)
      kbo(:,49, 3) = (/ &
        & 0.19377e-04_rb,0.19814e-04_rb,0.20191e-04_rb,0.20227e-04_rb,0.20080e-04_rb /)
      kbo(:,50, 3) = (/ &
        & 0.15665e-04_rb,0.15965e-04_rb,0.16366e-04_rb,0.16467e-04_rb,0.16386e-04_rb /)
      kbo(:,51, 3) = (/ &
        & 0.12631e-04_rb,0.12915e-04_rb,0.13203e-04_rb,0.13389e-04_rb,0.13403e-04_rb /)
      kbo(:,52, 3) = (/ &
        & 0.10160e-04_rb,0.10402e-04_rb,0.10609e-04_rb,0.10858e-04_rb,0.10900e-04_rb /)
      kbo(:,53, 3) = (/ &
        & 0.81514e-05_rb,0.83717e-05_rb,0.85717e-05_rb,0.87255e-05_rb,0.88490e-05_rb /)
      kbo(:,54, 3) = (/ &
        & 0.65924e-05_rb,0.67623e-05_rb,0.69218e-05_rb,0.70363e-05_rb,0.71744e-05_rb /)
      kbo(:,55, 3) = (/ &
        & 0.53378e-05_rb,0.54823e-05_rb,0.56002e-05_rb,0.57079e-05_rb,0.58136e-05_rb /)
      kbo(:,56, 3) = (/ &
        & 0.43093e-05_rb,0.44306e-05_rb,0.45341e-05_rb,0.46443e-05_rb,0.47053e-05_rb /)
      kbo(:,57, 3) = (/ &
        & 0.34880e-05_rb,0.35826e-05_rb,0.36738e-05_rb,0.37492e-05_rb,0.38094e-05_rb /)
      kbo(:,58, 3) = (/ &
        & 0.28214e-05_rb,0.28911e-05_rb,0.29636e-05_rb,0.30269e-05_rb,0.30943e-05_rb /)
      kbo(:,59, 3) = (/ &
        & 0.23049e-05_rb,0.23610e-05_rb,0.24232e-05_rb,0.24708e-05_rb,0.25266e-05_rb /)
      kbo(:,13, 4) = (/ &
        & 0.16886e-01_rb,0.17177e-01_rb,0.17415e-01_rb,0.17613e-01_rb,0.17771e-01_rb /)
      kbo(:,14, 4) = (/ &
        & 0.14297e-01_rb,0.14572e-01_rb,0.14791e-01_rb,0.14925e-01_rb,0.15051e-01_rb /)
      kbo(:,15, 4) = (/ &
        & 0.12142e-01_rb,0.12342e-01_rb,0.12507e-01_rb,0.12640e-01_rb,0.12754e-01_rb /)
      kbo(:,16, 4) = (/ &
        & 0.10284e-01_rb,0.10422e-01_rb,0.10545e-01_rb,0.10662e-01_rb,0.10740e-01_rb /)
      kbo(:,17, 4) = (/ &
        & 0.87377e-02_rb,0.88415e-02_rb,0.89329e-02_rb,0.90175e-02_rb,0.90716e-02_rb /)
      kbo(:,18, 4) = (/ &
        & 0.74766e-02_rb,0.75366e-02_rb,0.76038e-02_rb,0.76818e-02_rb,0.77135e-02_rb /)
      kbo(:,19, 4) = (/ &
        & 0.64241e-02_rb,0.64546e-02_rb,0.65027e-02_rb,0.65449e-02_rb,0.65755e-02_rb /)
      kbo(:,20, 4) = (/ &
        & 0.54390e-02_rb,0.54528e-02_rb,0.54854e-02_rb,0.55046e-02_rb,0.55309e-02_rb /)
      kbo(:,21, 4) = (/ &
        & 0.45856e-02_rb,0.45888e-02_rb,0.46080e-02_rb,0.46219e-02_rb,0.46381e-02_rb /)
      kbo(:,22, 4) = (/ &
        & 0.38581e-02_rb,0.38555e-02_rb,0.38631e-02_rb,0.38756e-02_rb,0.38888e-02_rb /)
      kbo(:,23, 4) = (/ &
        & 0.32522e-02_rb,0.32442e-02_rb,0.32437e-02_rb,0.32522e-02_rb,0.32616e-02_rb /)
      kbo(:,24, 4) = (/ &
        & 0.27421e-02_rb,0.27302e-02_rb,0.27267e-02_rb,0.27311e-02_rb,0.27351e-02_rb /)
      kbo(:,25, 4) = (/ &
        & 0.23082e-02_rb,0.22984e-02_rb,0.22916e-02_rb,0.22928e-02_rb,0.22950e-02_rb /)
      kbo(:,26, 4) = (/ &
        & 0.19386e-02_rb,0.19299e-02_rb,0.19241e-02_rb,0.19234e-02_rb,0.19234e-02_rb /)
      kbo(:,27, 4) = (/ &
        & 0.16295e-02_rb,0.16227e-02_rb,0.16172e-02_rb,0.16132e-02_rb,0.16138e-02_rb /)
      kbo(:,28, 4) = (/ &
        & 0.13714e-02_rb,0.13653e-02_rb,0.13589e-02_rb,0.13555e-02_rb,0.13562e-02_rb /)
      kbo(:,29, 4) = (/ &
        & 0.11583e-02_rb,0.11513e-02_rb,0.11440e-02_rb,0.11416e-02_rb,0.11458e-02_rb /)
      kbo(:,30, 4) = (/ &
        & 0.98213e-03_rb,0.97362e-03_rb,0.96774e-03_rb,0.96738e-03_rb,0.97898e-03_rb /)
      kbo(:,31, 4) = (/ &
        & 0.83694e-03_rb,0.82733e-03_rb,0.82506e-03_rb,0.82584e-03_rb,0.83454e-03_rb /)
      kbo(:,32, 4) = (/ &
        & 0.71631e-03_rb,0.70972e-03_rb,0.70944e-03_rb,0.70782e-03_rb,0.71382e-03_rb /)
      kbo(:,33, 4) = (/ &
        & 0.62056e-03_rb,0.61607e-03_rb,0.61112e-03_rb,0.60974e-03_rb,0.61416e-03_rb /)
      kbo(:,34, 4) = (/ &
        & 0.54260e-03_rb,0.53530e-03_rb,0.53018e-03_rb,0.53196e-03_rb,0.53190e-03_rb /)
      kbo(:,35, 4) = (/ &
        & 0.47557e-03_rb,0.47060e-03_rb,0.46470e-03_rb,0.46463e-03_rb,0.46278e-03_rb /)
      kbo(:,36, 4) = (/ &
        & 0.42108e-03_rb,0.41716e-03_rb,0.41356e-03_rb,0.41052e-03_rb,0.40671e-03_rb /)
      kbo(:,37, 4) = (/ &
        & 0.36059e-03_rb,0.35753e-03_rb,0.35465e-03_rb,0.35204e-03_rb,0.34625e-03_rb /)
      kbo(:,38, 4) = (/ &
        & 0.30926e-03_rb,0.30682e-03_rb,0.30403e-03_rb,0.30126e-03_rb,0.29597e-03_rb /)
      kbo(:,39, 4) = (/ &
        & 0.26586e-03_rb,0.26397e-03_rb,0.26023e-03_rb,0.25654e-03_rb,0.25399e-03_rb /)
      kbo(:,40, 4) = (/ &
        & 0.22228e-03_rb,0.22028e-03_rb,0.21711e-03_rb,0.21418e-03_rb,0.21203e-03_rb /)
      kbo(:,41, 4) = (/ &
        & 0.18516e-03_rb,0.18277e-03_rb,0.18056e-03_rb,0.17797e-03_rb,0.17615e-03_rb /)
      kbo(:,42, 4) = (/ &
        & 0.15367e-03_rb,0.15191e-03_rb,0.15012e-03_rb,0.14803e-03_rb,0.14667e-03_rb /)
      kbo(:,43, 4) = (/ &
        & 0.12696e-03_rb,0.12546e-03_rb,0.12398e-03_rb,0.12239e-03_rb,0.12042e-03_rb /)
      kbo(:,44, 4) = (/ &
        & 0.10470e-03_rb,0.10323e-03_rb,0.10199e-03_rb,0.10082e-03_rb,0.99093e-04_rb /)
      kbo(:,45, 4) = (/ &
        & 0.86543e-04_rb,0.84786e-04_rb,0.83972e-04_rb,0.82789e-04_rb,0.81392e-04_rb /)
      kbo(:,46, 4) = (/ &
        & 0.71529e-04_rb,0.69963e-04_rb,0.68982e-04_rb,0.67996e-04_rb,0.66867e-04_rb /)
      kbo(:,47, 4) = (/ &
        & 0.59457e-04_rb,0.57707e-04_rb,0.56668e-04_rb,0.55899e-04_rb,0.54959e-04_rb /)
      kbo(:,48, 4) = (/ &
        & 0.49467e-04_rb,0.47679e-04_rb,0.46593e-04_rb,0.45887e-04_rb,0.45090e-04_rb /)
      kbo(:,49, 4) = (/ &
        & 0.40766e-04_rb,0.39435e-04_rb,0.38281e-04_rb,0.37548e-04_rb,0.37003e-04_rb /)
      kbo(:,50, 4) = (/ &
        & 0.33628e-04_rb,0.32753e-04_rb,0.31623e-04_rb,0.30893e-04_rb,0.30385e-04_rb /)
      kbo(:,51, 4) = (/ &
        & 0.27865e-04_rb,0.27055e-04_rb,0.26162e-04_rb,0.25458e-04_rb,0.24898e-04_rb /)
      kbo(:,52, 4) = (/ &
        & 0.22934e-04_rb,0.22328e-04_rb,0.21731e-04_rb,0.20990e-04_rb,0.20456e-04_rb /)
      kbo(:,53, 4) = (/ &
        & 0.18846e-04_rb,0.18445e-04_rb,0.17912e-04_rb,0.17357e-04_rb,0.16833e-04_rb /)
      kbo(:,54, 4) = (/ &
        & 0.15388e-04_rb,0.15212e-04_rb,0.14808e-04_rb,0.14414e-04_rb,0.13935e-04_rb /)
      kbo(:,55, 4) = (/ &
        & 0.12598e-04_rb,0.12543e-04_rb,0.12261e-04_rb,0.11957e-04_rb,0.11571e-04_rb /)
      kbo(:,56, 4) = (/ &
        & 0.10381e-04_rb,0.10344e-04_rb,0.10166e-04_rb,0.98775e-05_rb,0.96020e-05_rb /)
      kbo(:,57, 4) = (/ &
        & 0.85457e-05_rb,0.84639e-05_rb,0.83932e-05_rb,0.81757e-05_rb,0.79545e-05_rb /)
      kbo(:,58, 4) = (/ &
        & 0.70397e-05_rb,0.69486e-05_rb,0.69316e-05_rb,0.67696e-05_rb,0.65859e-05_rb /)
      kbo(:,59, 4) = (/ &
        & 0.58257e-05_rb,0.57596e-05_rb,0.57448e-05_rb,0.56300e-05_rb,0.54831e-05_rb /)
      kbo(:,13, 5) = (/ &
        & 0.33040e-01_rb,0.33623e-01_rb,0.34360e-01_rb,0.35360e-01_rb,0.36383e-01_rb /)
      kbo(:,14, 5) = (/ &
        & 0.28634e-01_rb,0.29145e-01_rb,0.29913e-01_rb,0.30773e-01_rb,0.31595e-01_rb /)
      kbo(:,15, 5) = (/ &
        & 0.24852e-01_rb,0.25387e-01_rb,0.26038e-01_rb,0.26736e-01_rb,0.27406e-01_rb /)
      kbo(:,16, 5) = (/ &
        & 0.21426e-01_rb,0.21995e-01_rb,0.22531e-01_rb,0.23082e-01_rb,0.23575e-01_rb /)
      kbo(:,17, 5) = (/ &
        & 0.18563e-01_rb,0.19011e-01_rb,0.19492e-01_rb,0.19940e-01_rb,0.20337e-01_rb /)
      kbo(:,18, 5) = (/ &
        & 0.16067e-01_rb,0.16476e-01_rb,0.16902e-01_rb,0.17235e-01_rb,0.17579e-01_rb /)
      kbo(:,19, 5) = (/ &
        & 0.13911e-01_rb,0.14318e-01_rb,0.14657e-01_rb,0.14999e-01_rb,0.15265e-01_rb /)
      kbo(:,20, 5) = (/ &
        & 0.11889e-01_rb,0.12220e-01_rb,0.12522e-01_rb,0.12793e-01_rb,0.13028e-01_rb /)
      kbo(:,21, 5) = (/ &
        & 0.10107e-01_rb,0.10365e-01_rb,0.10626e-01_rb,0.10840e-01_rb,0.11030e-01_rb /)
      kbo(:,22, 5) = (/ &
        & 0.85828e-02_rb,0.88037e-02_rb,0.90066e-02_rb,0.91828e-02_rb,0.93108e-02_rb /)
      kbo(:,23, 5) = (/ &
        & 0.72873e-02_rb,0.74698e-02_rb,0.76229e-02_rb,0.77606e-02_rb,0.78812e-02_rb /)
      kbo(:,24, 5) = (/ &
        & 0.61731e-02_rb,0.63486e-02_rb,0.64551e-02_rb,0.65686e-02_rb,0.66826e-02_rb /)
      kbo(:,25, 5) = (/ &
        & 0.52349e-02_rb,0.53827e-02_rb,0.54728e-02_rb,0.55581e-02_rb,0.56691e-02_rb /)
      kbo(:,26, 5) = (/ &
        & 0.44343e-02_rb,0.45615e-02_rb,0.46303e-02_rb,0.47140e-02_rb,0.48058e-02_rb /)
      kbo(:,27, 5) = (/ &
        & 0.37566e-02_rb,0.38599e-02_rb,0.39250e-02_rb,0.39996e-02_rb,0.40739e-02_rb /)
      kbo(:,28, 5) = (/ &
        & 0.31898e-02_rb,0.32700e-02_rb,0.33284e-02_rb,0.33863e-02_rb,0.34483e-02_rb /)
      kbo(:,29, 5) = (/ &
        & 0.27069e-02_rb,0.27638e-02_rb,0.28173e-02_rb,0.28668e-02_rb,0.29159e-02_rb /)
      kbo(:,30, 5) = (/ &
        & 0.22952e-02_rb,0.23414e-02_rb,0.23862e-02_rb,0.24319e-02_rb,0.25045e-02_rb /)
      kbo(:,31, 5) = (/ &
        & 0.19469e-02_rb,0.19866e-02_rb,0.20269e-02_rb,0.20617e-02_rb,0.21285e-02_rb /)
      kbo(:,32, 5) = (/ &
        & 0.16569e-02_rb,0.16912e-02_rb,0.17225e-02_rb,0.17537e-02_rb,0.18151e-02_rb /)
      kbo(:,33, 5) = (/ &
        & 0.14159e-02_rb,0.14407e-02_rb,0.14672e-02_rb,0.14987e-02_rb,0.15568e-02_rb /)
      kbo(:,34, 5) = (/ &
        & 0.12124e-02_rb,0.12314e-02_rb,0.12565e-02_rb,0.13096e-02_rb,0.13455e-02_rb /)
      kbo(:,35, 5) = (/ &
        & 0.10402e-02_rb,0.10574e-02_rb,0.10852e-02_rb,0.11323e-02_rb,0.11671e-02_rb /)
      kbo(:,36, 5) = (/ &
        & 0.89606e-03_rb,0.91393e-03_rb,0.93957e-03_rb,0.98258e-03_rb,0.10142e-02_rb /)
      kbo(:,37, 5) = (/ &
        & 0.75556e-03_rb,0.77347e-03_rb,0.79494e-03_rb,0.83337e-03_rb,0.86466e-03_rb /)
      kbo(:,38, 5) = (/ &
        & 0.63793e-03_rb,0.65386e-03_rb,0.67373e-03_rb,0.70925e-03_rb,0.73610e-03_rb /)
      kbo(:,39, 5) = (/ &
        & 0.53995e-03_rb,0.55394e-03_rb,0.57091e-03_rb,0.59316e-03_rb,0.62725e-03_rb /)
      kbo(:,40, 5) = (/ &
        & 0.44830e-03_rb,0.46041e-03_rb,0.47605e-03_rb,0.49517e-03_rb,0.52406e-03_rb /)
      kbo(:,41, 5) = (/ &
        & 0.37113e-03_rb,0.38143e-03_rb,0.39555e-03_rb,0.41230e-03_rb,0.43648e-03_rb /)
      kbo(:,42, 5) = (/ &
        & 0.30743e-03_rb,0.31617e-03_rb,0.32840e-03_rb,0.34246e-03_rb,0.36323e-03_rb /)
      kbo(:,43, 5) = (/ &
        & 0.25237e-03_rb,0.25981e-03_rb,0.27013e-03_rb,0.28179e-03_rb,0.29414e-03_rb /)
      kbo(:,44, 5) = (/ &
        & 0.20643e-03_rb,0.21247e-03_rb,0.22138e-03_rb,0.23093e-03_rb,0.24175e-03_rb /)
      kbo(:,45, 5) = (/ &
        & 0.16882e-03_rb,0.17379e-03_rb,0.18072e-03_rb,0.18917e-03_rb,0.19837e-03_rb /)
      kbo(:,46, 5) = (/ &
        & 0.13808e-03_rb,0.14140e-03_rb,0.14740e-03_rb,0.15462e-03_rb,0.16243e-03_rb /)
      kbo(:,47, 5) = (/ &
        & 0.11262e-03_rb,0.11523e-03_rb,0.12010e-03_rb,0.12571e-03_rb,0.13225e-03_rb /)
      kbo(:,48, 5) = (/ &
        & 0.91738e-04_rb,0.93816e-04_rb,0.97414e-04_rb,0.10204e-03_rb,0.10733e-03_rb /)
      kbo(:,49, 5) = (/ &
        & 0.74533e-04_rb,0.76356e-04_rb,0.78756e-04_rb,0.82697e-04_rb,0.86848e-04_rb /)
      kbo(:,50, 5) = (/ &
        & 0.60870e-04_rb,0.62211e-04_rb,0.64031e-04_rb,0.67056e-04_rb,0.70590e-04_rb /)
      kbo(:,51, 5) = (/ &
        & 0.49547e-04_rb,0.50786e-04_rb,0.52181e-04_rb,0.54428e-04_rb,0.57574e-04_rb /)
      kbo(:,52, 5) = (/ &
        & 0.40457e-04_rb,0.41364e-04_rb,0.42403e-04_rb,0.44095e-04_rb,0.46746e-04_rb /)
      kbo(:,53, 5) = (/ &
        & 0.33032e-04_rb,0.33540e-04_rb,0.34449e-04_rb,0.35762e-04_rb,0.37780e-04_rb /)
      kbo(:,54, 5) = (/ &
        & 0.27201e-04_rb,0.27411e-04_rb,0.28166e-04_rb,0.29168e-04_rb,0.30744e-04_rb /)
      kbo(:,55, 5) = (/ &
        & 0.22471e-04_rb,0.22532e-04_rb,0.23075e-04_rb,0.23887e-04_rb,0.25092e-04_rb /)
      kbo(:,56, 5) = (/ &
        & 0.18480e-04_rb,0.18516e-04_rb,0.18857e-04_rb,0.19555e-04_rb,0.20483e-04_rb /)
      kbo(:,57, 5) = (/ &
        & 0.15198e-04_rb,0.15262e-04_rb,0.15424e-04_rb,0.15989e-04_rb,0.16742e-04_rb /)
      kbo(:,58, 5) = (/ &
        & 0.12523e-04_rb,0.12562e-04_rb,0.12648e-04_rb,0.13089e-04_rb,0.13686e-04_rb /)
      kbo(:,59, 5) = (/ &
        & 0.10416e-04_rb,0.10433e-04_rb,0.10536e-04_rb,0.10919e-04_rb,0.11421e-04_rb /)
      kbo(:,13, 6) = (/ &
        & 0.69718e-01_rb,0.71167e-01_rb,0.72961e-01_rb,0.74885e-01_rb,0.77322e-01_rb /)
      kbo(:,14, 6) = (/ &
        & 0.62182e-01_rb,0.63649e-01_rb,0.65193e-01_rb,0.67193e-01_rb,0.69746e-01_rb /)
      kbo(:,15, 6) = (/ &
        & 0.55177e-01_rb,0.56711e-01_rb,0.58414e-01_rb,0.60422e-01_rb,0.62756e-01_rb /)
      kbo(:,16, 6) = (/ &
        & 0.48867e-01_rb,0.50234e-01_rb,0.52023e-01_rb,0.54055e-01_rb,0.56254e-01_rb /)
      kbo(:,17, 6) = (/ &
        & 0.43174e-01_rb,0.44610e-01_rb,0.46287e-01_rb,0.48302e-01_rb,0.50389e-01_rb /)
      kbo(:,18, 6) = (/ &
        & 0.38339e-01_rb,0.39749e-01_rb,0.41335e-01_rb,0.43237e-01_rb,0.45123e-01_rb /)
      kbo(:,19, 6) = (/ &
        & 0.34315e-01_rb,0.35543e-01_rb,0.37065e-01_rb,0.38754e-01_rb,0.40597e-01_rb /)
      kbo(:,20, 6) = (/ &
        & 0.30036e-01_rb,0.31291e-01_rb,0.32637e-01_rb,0.34202e-01_rb,0.35839e-01_rb /)
      kbo(:,21, 6) = (/ &
        & 0.26135e-01_rb,0.27343e-01_rb,0.28574e-01_rb,0.30000e-01_rb,0.31460e-01_rb /)
      kbo(:,22, 6) = (/ &
        & 0.22750e-01_rb,0.23832e-01_rb,0.25080e-01_rb,0.26299e-01_rb,0.27580e-01_rb /)
      kbo(:,23, 6) = (/ &
        & 0.19795e-01_rb,0.20915e-01_rb,0.22028e-01_rb,0.23060e-01_rb,0.24140e-01_rb /)
      kbo(:,24, 6) = (/ &
        & 0.17353e-01_rb,0.18346e-01_rb,0.19305e-01_rb,0.20238e-01_rb,0.21103e-01_rb /)
      kbo(:,25, 6) = (/ &
        & 0.15226e-01_rb,0.16064e-01_rb,0.16955e-01_rb,0.17722e-01_rb,0.18477e-01_rb /)
      kbo(:,26, 6) = (/ &
        & 0.13309e-01_rb,0.14087e-01_rb,0.14848e-01_rb,0.15498e-01_rb,0.16180e-01_rb /)
      kbo(:,27, 6) = (/ &
        & 0.11623e-01_rb,0.12295e-01_rb,0.12913e-01_rb,0.13525e-01_rb,0.14113e-01_rb /)
      kbo(:,28, 6) = (/ &
        & 0.10147e-01_rb,0.10710e-01_rb,0.11240e-01_rb,0.11795e-01_rb,0.12384e-01_rb /)
      kbo(:,29, 6) = (/ &
        & 0.87968e-02_rb,0.92904e-02_rb,0.97797e-02_rb,0.10288e-01_rb,0.10828e-01_rb /)
      kbo(:,30, 6) = (/ &
        & 0.76490e-02_rb,0.80771e-02_rb,0.85219e-02_rb,0.90152e-02_rb,0.97888e-02_rb /)
      kbo(:,31, 6) = (/ &
        & 0.66330e-02_rb,0.70376e-02_rb,0.74659e-02_rb,0.79113e-02_rb,0.86622e-02_rb /)
      kbo(:,32, 6) = (/ &
        & 0.58073e-02_rb,0.61697e-02_rb,0.65754e-02_rb,0.70309e-02_rb,0.77115e-02_rb /)
      kbo(:,33, 6) = (/ &
        & 0.51062e-02_rb,0.54682e-02_rb,0.58582e-02_rb,0.62737e-02_rb,0.69163e-02_rb /)
      kbo(:,34, 6) = (/ &
        & 0.45302e-02_rb,0.48855e-02_rb,0.52639e-02_rb,0.58227e-02_rb,0.62719e-02_rb /)
      kbo(:,35, 6) = (/ &
        & 0.40254e-02_rb,0.43599e-02_rb,0.47158e-02_rb,0.52732e-02_rb,0.57228e-02_rb /)
      kbo(:,36, 6) = (/ &
        & 0.35647e-02_rb,0.38837e-02_rb,0.42167e-02_rb,0.47705e-02_rb,0.52032e-02_rb /)
      kbo(:,37, 6) = (/ &
        & 0.30730e-02_rb,0.33561e-02_rb,0.36823e-02_rb,0.41868e-02_rb,0.45932e-02_rb /)
      kbo(:,38, 6) = (/ &
        & 0.26407e-02_rb,0.29052e-02_rb,0.32061e-02_rb,0.36781e-02_rb,0.40664e-02_rb /)
      kbo(:,39, 6) = (/ &
        & 0.22667e-02_rb,0.25156e-02_rb,0.28017e-02_rb,0.31173e-02_rb,0.36136e-02_rb /)
      kbo(:,40, 6) = (/ &
        & 0.19056e-02_rb,0.21267e-02_rb,0.23879e-02_rb,0.26789e-02_rb,0.31309e-02_rb /)
      kbo(:,41, 6) = (/ &
        & 0.15967e-02_rb,0.17952e-02_rb,0.20263e-02_rb,0.22865e-02_rb,0.27062e-02_rb /)
      kbo(:,42, 6) = (/ &
        & 0.13343e-02_rb,0.15081e-02_rb,0.17167e-02_rb,0.19538e-02_rb,0.23416e-02_rb /)
      kbo(:,43, 6) = (/ &
        & 0.10981e-02_rb,0.12492e-02_rb,0.14292e-02_rb,0.16430e-02_rb,0.19013e-02_rb /)
      kbo(:,44, 6) = (/ &
        & 0.89464e-03_rb,0.10267e-02_rb,0.11827e-02_rb,0.13698e-02_rb,0.16046e-02_rb /)
      kbo(:,45, 6) = (/ &
        & 0.72683e-03_rb,0.83985e-03_rb,0.97469e-03_rb,0.11376e-02_rb,0.13459e-02_rb /)
      kbo(:,46, 6) = (/ &
        & 0.58610e-03_rb,0.68242e-03_rb,0.79873e-03_rb,0.94079e-03_rb,0.11248e-02_rb /)
      kbo(:,47, 6) = (/ &
        & 0.46823e-03_rb,0.54958e-03_rb,0.64876e-03_rb,0.77111e-03_rb,0.93276e-03_rb /)
      kbo(:,48, 6) = (/ &
        & 0.37202e-03_rb,0.44089e-03_rb,0.52431e-03_rb,0.62811e-03_rb,0.76979e-03_rb /)
      kbo(:,49, 6) = (/ &
        & 0.29424e-03_rb,0.35087e-03_rb,0.42017e-03_rb,0.50926e-03_rb,0.63166e-03_rb /)
      kbo(:,50, 6) = (/ &
        & 0.23381e-03_rb,0.28036e-03_rb,0.33854e-03_rb,0.41547e-03_rb,0.52066e-03_rb /)
      kbo(:,51, 6) = (/ &
        & 0.18542e-03_rb,0.22410e-03_rb,0.27211e-03_rb,0.33806e-03_rb,0.42775e-03_rb /)
      kbo(:,52, 6) = (/ &
        & 0.14646e-03_rb,0.17824e-03_rb,0.21799e-03_rb,0.27400e-03_rb,0.34964e-03_rb /)
      kbo(:,53, 6) = (/ &
        & 0.11499e-03_rb,0.14049e-03_rb,0.17412e-03_rb,0.22038e-03_rb,0.28435e-03_rb /)
      kbo(:,54, 6) = (/ &
        & 0.91160e-04_rb,0.11217e-03_rb,0.13972e-03_rb,0.17793e-03_rb,0.23213e-03_rb /)
      kbo(:,55, 6) = (/ &
        & 0.72474e-04_rb,0.89606e-04_rb,0.11228e-03_rb,0.14368e-03_rb,0.18928e-03_rb /)
      kbo(:,56, 6) = (/ &
        & 0.57463e-04_rb,0.71291e-04_rb,0.89786e-04_rb,0.11554e-03_rb,0.15352e-03_rb /)
      kbo(:,57, 6) = (/ &
        & 0.45287e-04_rb,0.56571e-04_rb,0.71524e-04_rb,0.92319e-04_rb,0.12382e-03_rb /)
      kbo(:,58, 6) = (/ &
        & 0.35841e-04_rb,0.44939e-04_rb,0.57025e-04_rb,0.73710e-04_rb,0.99673e-04_rb /)
      kbo(:,59, 6) = (/ &
        & 0.29941e-04_rb,0.37721e-04_rb,0.48119e-04_rb,0.62420e-04_rb,0.85059e-04_rb /)
      kbo(:,13, 7) = (/ &
        & 0.16189e+00_rb,0.16561e+00_rb,0.16999e+00_rb,0.17514e+00_rb,0.18068e+00_rb /)
      kbo(:,14, 7) = (/ &
        & 0.14981e+00_rb,0.15416e+00_rb,0.15917e+00_rb,0.16420e+00_rb,0.16989e+00_rb /)
      kbo(:,15, 7) = (/ &
        & 0.13875e+00_rb,0.14297e+00_rb,0.14757e+00_rb,0.15249e+00_rb,0.15852e+00_rb /)
      kbo(:,16, 7) = (/ &
        & 0.12672e+00_rb,0.13073e+00_rb,0.13523e+00_rb,0.14037e+00_rb,0.14640e+00_rb /)
      kbo(:,17, 7) = (/ &
        & 0.11530e+00_rb,0.11958e+00_rb,0.12450e+00_rb,0.12975e+00_rb,0.13596e+00_rb /)
      kbo(:,18, 7) = (/ &
        & 0.10572e+00_rb,0.11005e+00_rb,0.11482e+00_rb,0.12031e+00_rb,0.12674e+00_rb /)
      kbo(:,19, 7) = (/ &
        & 0.97798e-01_rb,0.10221e+00_rb,0.10713e+00_rb,0.11251e+00_rb,0.11882e+00_rb /)
      kbo(:,20, 7) = (/ &
        & 0.88438e-01_rb,0.93059e-01_rb,0.98290e-01_rb,0.10406e+00_rb,0.11034e+00_rb /)
      kbo(:,21, 7) = (/ &
        & 0.79615e-01_rb,0.84383e-01_rb,0.89776e-01_rb,0.95707e-01_rb,0.10218e+00_rb /)
      kbo(:,22, 7) = (/ &
        & 0.71663e-01_rb,0.76568e-01_rb,0.81968e-01_rb,0.88106e-01_rb,0.94703e-01_rb /)
      kbo(:,23, 7) = (/ &
        & 0.64729e-01_rb,0.69601e-01_rb,0.75178e-01_rb,0.81445e-01_rb,0.88072e-01_rb /)
      kbo(:,24, 7) = (/ &
        & 0.58713e-01_rb,0.63692e-01_rb,0.69398e-01_rb,0.75609e-01_rb,0.82402e-01_rb /)
      kbo(:,25, 7) = (/ &
        & 0.53422e-01_rb,0.58504e-01_rb,0.64255e-01_rb,0.70582e-01_rb,0.77266e-01_rb /)
      kbo(:,26, 7) = (/ &
        & 0.48884e-01_rb,0.53907e-01_rb,0.59754e-01_rb,0.66093e-01_rb,0.72781e-01_rb /)
      kbo(:,27, 7) = (/ &
        & 0.44782e-01_rb,0.49959e-01_rb,0.55649e-01_rb,0.61937e-01_rb,0.68517e-01_rb /)
      kbo(:,28, 7) = (/ &
        & 0.41164e-01_rb,0.46305e-01_rb,0.52009e-01_rb,0.58205e-01_rb,0.64389e-01_rb /)
      kbo(:,29, 7) = (/ &
        & 0.37976e-01_rb,0.43033e-01_rb,0.48696e-01_rb,0.54470e-01_rb,0.60707e-01_rb /)
      kbo(:,30, 7) = (/ &
        & 0.35210e-01_rb,0.40258e-01_rb,0.45653e-01_rb,0.51435e-01_rb,0.59091e-01_rb /)
      kbo(:,31, 7) = (/ &
        & 0.32774e-01_rb,0.37557e-01_rb,0.43028e-01_rb,0.48762e-01_rb,0.56208e-01_rb /)
      kbo(:,32, 7) = (/ &
        & 0.30628e-01_rb,0.35555e-01_rb,0.40840e-01_rb,0.46513e-01_rb,0.53917e-01_rb /)
      kbo(:,33, 7) = (/ &
        & 0.29120e-01_rb,0.33908e-01_rb,0.39209e-01_rb,0.44801e-01_rb,0.52179e-01_rb /)
      kbo(:,34, 7) = (/ &
        & 0.27843e-01_rb,0.32610e-01_rb,0.37873e-01_rb,0.45101e-01_rb,0.51127e-01_rb /)
      kbo(:,35, 7) = (/ &
        & 0.26429e-01_rb,0.31201e-01_rb,0.36480e-01_rb,0.43796e-01_rb,0.49886e-01_rb /)
      kbo(:,36, 7) = (/ &
        & 0.24910e-01_rb,0.29603e-01_rb,0.34935e-01_rb,0.42317e-01_rb,0.48474e-01_rb /)
      kbo(:,37, 7) = (/ &
        & 0.22623e-01_rb,0.27186e-01_rb,0.32364e-01_rb,0.39709e-01_rb,0.45895e-01_rb /)
      kbo(:,38, 7) = (/ &
        & 0.20536e-01_rb,0.25035e-01_rb,0.30138e-01_rb,0.37341e-01_rb,0.43584e-01_rb /)
      kbo(:,39, 7) = (/ &
        & 0.18744e-01_rb,0.23143e-01_rb,0.28133e-01_rb,0.33754e-01_rb,0.41569e-01_rb /)
      kbo(:,40, 7) = (/ &
        & 0.16632e-01_rb,0.20751e-01_rb,0.25556e-01_rb,0.30972e-01_rb,0.38593e-01_rb /)
      kbo(:,41, 7) = (/ &
        & 0.14671e-01_rb,0.18541e-01_rb,0.23110e-01_rb,0.28292e-01_rb,0.35702e-01_rb /)
      kbo(:,42, 7) = (/ &
        & 0.12909e-01_rb,0.16573e-01_rb,0.20901e-01_rb,0.25885e-01_rb,0.32997e-01_rb /)
      kbo(:,43, 7) = (/ &
        & 0.11155e-01_rb,0.14521e-01_rb,0.18561e-01_rb,0.23268e-01_rb,0.28558e-01_rb /)
      kbo(:,44, 7) = (/ &
        & 0.95144e-02_rb,0.12576e-01_rb,0.16313e-01_rb,0.20662e-01_rb,0.25638e-01_rb /)
      kbo(:,45, 7) = (/ &
        & 0.80643e-02_rb,0.10847e-01_rb,0.14290e-01_rb,0.18347e-01_rb,0.23034e-01_rb /)
      kbo(:,46, 7) = (/ &
        & 0.67589e-02_rb,0.92659e-02_rb,0.12372e-01_rb,0.16082e-01_rb,0.20456e-01_rb /)
      kbo(:,47, 7) = (/ &
        & 0.55482e-02_rb,0.77474e-02_rb,0.10528e-01_rb,0.13920e-01_rb,0.17966e-01_rb /)
      kbo(:,48, 7) = (/ &
        & 0.45013e-02_rb,0.64175e-02_rb,0.88907e-02_rb,0.11958e-01_rb,0.15638e-01_rb /)
      kbo(:,49, 7) = (/ &
        & 0.36173e-02_rb,0.52743e-02_rb,0.74530e-02_rb,0.10200e-01_rb,0.13544e-01_rb /)
      kbo(:,50, 7) = (/ &
        & 0.29073e-02_rb,0.43372e-02_rb,0.62514e-02_rb,0.87078e-02_rb,0.11757e-01_rb /)
      kbo(:,51, 7) = (/ &
        & 0.23200e-02_rb,0.35429e-02_rb,0.52236e-02_rb,0.74160e-02_rb,0.10190e-01_rb /)
      kbo(:,52, 7) = (/ &
        & 0.18398e-02_rb,0.28704e-02_rb,0.43321e-02_rb,0.62743e-02_rb,0.87585e-02_rb /)
      kbo(:,53, 7) = (/ &
        & 0.14394e-02_rb,0.22999e-02_rb,0.35535e-02_rb,0.52615e-02_rb,0.74854e-02_rb /)
      kbo(:,54, 7) = (/ &
        & 0.11368e-02_rb,0.18545e-02_rb,0.29326e-02_rb,0.44372e-02_rb,0.64347e-02_rb /)
      kbo(:,55, 7) = (/ &
        & 0.89467e-03_rb,0.14935e-02_rb,0.24197e-02_rb,0.37410e-02_rb,0.55222e-02_rb /)
      kbo(:,56, 7) = (/ &
        & 0.69687e-03_rb,0.11894e-02_rb,0.19763e-02_rb,0.31286e-02_rb,0.47149e-02_rb /)
      kbo(:,57, 7) = (/ &
        & 0.53791e-03_rb,0.93747e-03_rb,0.15990e-02_rb,0.25976e-02_rb,0.39986e-02_rb /)
      kbo(:,58, 7) = (/ &
        & 0.41410e-03_rb,0.73971e-03_rb,0.12907e-02_rb,0.21482e-02_rb,0.33858e-02_rb /)
      kbo(:,59, 7) = (/ &
        & 0.35405e-03_rb,0.64717e-03_rb,0.11503e-02_rb,0.19466e-02_rb,0.31094e-02_rb /)
      kbo(:,13, 8) = (/ &
        & 0.45516e+00_rb,0.46955e+00_rb,0.48450e+00_rb,0.50029e+00_rb,0.51769e+00_rb /)
      kbo(:,14, 8) = (/ &
        & 0.43991e+00_rb,0.45312e+00_rb,0.46646e+00_rb,0.48116e+00_rb,0.49718e+00_rb /)
      kbo(:,15, 8) = (/ &
        & 0.42871e+00_rb,0.44246e+00_rb,0.45762e+00_rb,0.47406e+00_rb,0.49132e+00_rb /)
      kbo(:,16, 8) = (/ &
        & 0.40699e+00_rb,0.42153e+00_rb,0.43710e+00_rb,0.45468e+00_rb,0.47435e+00_rb /)
      kbo(:,17, 8) = (/ &
        & 0.38538e+00_rb,0.39940e+00_rb,0.41418e+00_rb,0.43146e+00_rb,0.45140e+00_rb /)
      kbo(:,18, 8) = (/ &
        & 0.36235e+00_rb,0.37741e+00_rb,0.39391e+00_rb,0.41121e+00_rb,0.43054e+00_rb /)
      kbo(:,19, 8) = (/ &
        & 0.34180e+00_rb,0.35677e+00_rb,0.37295e+00_rb,0.39070e+00_rb,0.41121e+00_rb /)
      kbo(:,20, 8) = (/ &
        & 0.31818e+00_rb,0.33243e+00_rb,0.34824e+00_rb,0.36683e+00_rb,0.38832e+00_rb /)
      kbo(:,21, 8) = (/ &
        & 0.29454e+00_rb,0.30892e+00_rb,0.32519e+00_rb,0.34434e+00_rb,0.36621e+00_rb /)
      kbo(:,22, 8) = (/ &
        & 0.27292e+00_rb,0.28819e+00_rb,0.30567e+00_rb,0.32553e+00_rb,0.34730e+00_rb /)
      kbo(:,23, 8) = (/ &
        & 0.25455e+00_rb,0.27053e+00_rb,0.28810e+00_rb,0.30845e+00_rb,0.33065e+00_rb /)
      kbo(:,24, 8) = (/ &
        & 0.23920e+00_rb,0.25523e+00_rb,0.27293e+00_rb,0.29370e+00_rb,0.31644e+00_rb /)
      kbo(:,25, 8) = (/ &
        & 0.22572e+00_rb,0.24152e+00_rb,0.26024e+00_rb,0.28092e+00_rb,0.30402e+00_rb /)
      kbo(:,26, 8) = (/ &
        & 0.21333e+00_rb,0.23053e+00_rb,0.24913e+00_rb,0.27069e+00_rb,0.29445e+00_rb /)
      kbo(:,27, 8) = (/ &
        & 0.20216e+00_rb,0.22006e+00_rb,0.23987e+00_rb,0.26172e+00_rb,0.28552e+00_rb /)
      kbo(:,28, 8) = (/ &
        & 0.19243e+00_rb,0.21106e+00_rb,0.23162e+00_rb,0.25373e+00_rb,0.27880e+00_rb /)
      kbo(:,29, 8) = (/ &
        & 0.18399e+00_rb,0.20352e+00_rb,0.22446e+00_rb,0.24815e+00_rb,0.27383e+00_rb /)
      kbo(:,30, 8) = (/ &
        & 0.17696e+00_rb,0.19707e+00_rb,0.21934e+00_rb,0.24428e+00_rb,0.27560e+00_rb /)
      kbo(:,31, 8) = (/ &
        & 0.17154e+00_rb,0.19276e+00_rb,0.21622e+00_rb,0.24297e+00_rb,0.27512e+00_rb /)
      kbo(:,32, 8) = (/ &
        & 0.16864e+00_rb,0.19074e+00_rb,0.21586e+00_rb,0.24341e+00_rb,0.27678e+00_rb /)
      kbo(:,33, 8) = (/ &
        & 0.16705e+00_rb,0.19103e+00_rb,0.21725e+00_rb,0.24571e+00_rb,0.27988e+00_rb /)
      kbo(:,34, 8) = (/ &
        & 0.16718e+00_rb,0.19214e+00_rb,0.21923e+00_rb,0.25236e+00_rb,0.28358e+00_rb /)
      kbo(:,35, 8) = (/ &
        & 0.16615e+00_rb,0.19194e+00_rb,0.21973e+00_rb,0.25386e+00_rb,0.28528e+00_rb /)
      kbo(:,36, 8) = (/ &
        & 0.16355e+00_rb,0.18981e+00_rb,0.21857e+00_rb,0.25318e+00_rb,0.28503e+00_rb /)
      kbo(:,37, 8) = (/ &
        & 0.15644e+00_rb,0.18272e+00_rb,0.21137e+00_rb,0.24627e+00_rb,0.27830e+00_rb /)
      kbo(:,38, 8) = (/ &
        & 0.14985e+00_rb,0.17574e+00_rb,0.20437e+00_rb,0.23950e+00_rb,0.27160e+00_rb /)
      kbo(:,39, 8) = (/ &
        & 0.14373e+00_rb,0.16918e+00_rb,0.19762e+00_rb,0.22814e+00_rb,0.26522e+00_rb /)
      kbo(:,40, 8) = (/ &
        & 0.13479e+00_rb,0.15968e+00_rb,0.18756e+00_rb,0.21775e+00_rb,0.25463e+00_rb /)
      kbo(:,41, 8) = (/ &
        & 0.12605e+00_rb,0.15045e+00_rb,0.17752e+00_rb,0.20726e+00_rb,0.24367e+00_rb /)
      kbo(:,42, 8) = (/ &
        & 0.11784e+00_rb,0.14153e+00_rb,0.16784e+00_rb,0.19712e+00_rb,0.23353e+00_rb /)
      kbo(:,43, 8) = (/ &
        & 0.10839e+00_rb,0.13122e+00_rb,0.15696e+00_rb,0.18528e+00_rb,0.21641e+00_rb /)
      kbo(:,44, 8) = (/ &
        & 0.98772e-01_rb,0.12078e+00_rb,0.14549e+00_rb,0.17313e+00_rb,0.20344e+00_rb /)
      kbo(:,45, 8) = (/ &
        & 0.89757e-01_rb,0.11096e+00_rb,0.13484e+00_rb,0.16135e+00_rb,0.19086e+00_rb /)
      kbo(:,46, 8) = (/ &
        & 0.80754e-01_rb,0.10111e+00_rb,0.12397e+00_rb,0.14951e+00_rb,0.17812e+00_rb /)
      kbo(:,47, 8) = (/ &
        & 0.71551e-01_rb,0.90848e-01_rb,0.11265e+00_rb,0.13712e+00_rb,0.16456e+00_rb /)
      kbo(:,48, 8) = (/ &
        & 0.63023e-01_rb,0.81188e-01_rb,0.10192e+00_rb,0.12530e+00_rb,0.15155e+00_rb /)
      kbo(:,49, 8) = (/ &
        & 0.55112e-01_rb,0.71966e-01_rb,0.91627e-01_rb,0.11386e+00_rb,0.13887e+00_rb /)
      kbo(:,50, 8) = (/ &
        & 0.48148e-01_rb,0.63904e-01_rb,0.82545e-01_rb,0.10380e+00_rb,0.12775e+00_rb /)
      kbo(:,51, 8) = (/ &
        & 0.42077e-01_rb,0.56742e-01_rb,0.74219e-01_rb,0.94436e-01_rb,0.11735e+00_rb /)
      kbo(:,52, 8) = (/ &
        & 0.36406e-01_rb,0.49929e-01_rb,0.66400e-01_rb,0.85557e-01_rb,0.10751e+00_rb /)
      kbo(:,53, 8) = (/ &
        & 0.31162e-01_rb,0.43645e-01_rb,0.58971e-01_rb,0.77049e-01_rb,0.97772e-01_rb /)
      kbo(:,54, 8) = (/ &
        & 0.26848e-01_rb,0.38389e-01_rb,0.52700e-01_rb,0.69937e-01_rb,0.89664e-01_rb /)
      kbo(:,55, 8) = (/ &
        & 0.23145e-01_rb,0.33793e-01_rb,0.47177e-01_rb,0.63590e-01_rb,0.82440e-01_rb /)
      kbo(:,56, 8) = (/ &
        & 0.19764e-01_rb,0.29522e-01_rb,0.42050e-01_rb,0.57472e-01_rb,0.75623e-01_rb /)
      kbo(:,57, 8) = (/ &
        & 0.16713e-01_rb,0.25602e-01_rb,0.37259e-01_rb,0.51705e-01_rb,0.68976e-01_rb /)
      kbo(:,58, 8) = (/ &
        & 0.14134e-01_rb,0.22215e-01_rb,0.32983e-01_rb,0.46579e-01_rb,0.62982e-01_rb /)
      kbo(:,59, 8) = (/ &
        & 0.13103e-01_rb,0.20874e-01_rb,0.31397e-01_rb,0.44700e-01_rb,0.60881e-01_rb /)
      kbo(:,13, 9) = (/ &
        & 0.30563e+01_rb,0.31744e+01_rb,0.32842e+01_rb,0.33901e+01_rb,0.34937e+01_rb /)
      kbo(:,14, 9) = (/ &
        & 0.27752e+01_rb,0.28869e+01_rb,0.29933e+01_rb,0.30958e+01_rb,0.31932e+01_rb /)
      kbo(:,15, 9) = (/ &
        & 0.25099e+01_rb,0.26164e+01_rb,0.27185e+01_rb,0.28161e+01_rb,0.29094e+01_rb /)
      kbo(:,16, 9) = (/ &
        & 0.23062e+01_rb,0.23985e+01_rb,0.24887e+01_rb,0.25791e+01_rb,0.26657e+01_rb /)
      kbo(:,17, 9) = (/ &
        & 0.21996e+01_rb,0.22783e+01_rb,0.23521e+01_rb,0.24248e+01_rb,0.25028e+01_rb /)
      kbo(:,18, 9) = (/ &
        & 0.22125e+01_rb,0.22932e+01_rb,0.23665e+01_rb,0.24375e+01_rb,0.25051e+01_rb /)
      kbo(:,19, 9) = (/ &
        & 0.22641e+01_rb,0.23491e+01_rb,0.24239e+01_rb,0.24973e+01_rb,0.25657e+01_rb /)
      kbo(:,20, 9) = (/ &
        & 0.22472e+01_rb,0.23357e+01_rb,0.24204e+01_rb,0.24964e+01_rb,0.25675e+01_rb /)
      kbo(:,21, 9) = (/ &
        & 0.21851e+01_rb,0.22830e+01_rb,0.23690e+01_rb,0.24562e+01_rb,0.25370e+01_rb /)
      kbo(:,22, 9) = (/ &
        & 0.21175e+01_rb,0.22142e+01_rb,0.23108e+01_rb,0.24049e+01_rb,0.24949e+01_rb /)
      kbo(:,23, 9) = (/ &
        & 0.20447e+01_rb,0.21452e+01_rb,0.22435e+01_rb,0.23385e+01_rb,0.24298e+01_rb /)
      kbo(:,24, 9) = (/ &
        & 0.19721e+01_rb,0.20737e+01_rb,0.21717e+01_rb,0.22760e+01_rb,0.23715e+01_rb /)
      kbo(:,25, 9) = (/ &
        & 0.18957e+01_rb,0.20036e+01_rb,0.21100e+01_rb,0.22176e+01_rb,0.23221e+01_rb /)
      kbo(:,26, 9) = (/ &
        & 0.18288e+01_rb,0.19378e+01_rb,0.20515e+01_rb,0.21609e+01_rb,0.22728e+01_rb /)
      kbo(:,27, 9) = (/ &
        & 0.17692e+01_rb,0.18810e+01_rb,0.19951e+01_rb,0.21151e+01_rb,0.22361e+01_rb /)
      kbo(:,28, 9) = (/ &
        & 0.17168e+01_rb,0.18338e+01_rb,0.19558e+01_rb,0.20789e+01_rb,0.22051e+01_rb /)
      kbo(:,29, 9) = (/ &
        & 0.16702e+01_rb,0.17936e+01_rb,0.19235e+01_rb,0.20524e+01_rb,0.21850e+01_rb /)
      kbo(:,30, 9) = (/ &
        & 0.16394e+01_rb,0.17677e+01_rb,0.19029e+01_rb,0.20369e+01_rb,0.21882e+01_rb /)
      kbo(:,31, 9) = (/ &
        & 0.16228e+01_rb,0.17523e+01_rb,0.18896e+01_rb,0.20262e+01_rb,0.21815e+01_rb /)
      kbo(:,32, 9) = (/ &
        & 0.16178e+01_rb,0.17489e+01_rb,0.18859e+01_rb,0.20233e+01_rb,0.21838e+01_rb /)
      kbo(:,33, 9) = (/ &
        & 0.16221e+01_rb,0.17529e+01_rb,0.18876e+01_rb,0.20327e+01_rb,0.21960e+01_rb /)
      kbo(:,34, 9) = (/ &
        & 0.16321e+01_rb,0.17688e+01_rb,0.19069e+01_rb,0.20650e+01_rb,0.22191e+01_rb /)
      kbo(:,35, 9) = (/ &
        & 0.16388e+01_rb,0.17830e+01_rb,0.19261e+01_rb,0.20887e+01_rb,0.22365e+01_rb /)
      kbo(:,36, 9) = (/ &
        & 0.16423e+01_rb,0.17895e+01_rb,0.19390e+01_rb,0.21032e+01_rb,0.22561e+01_rb /)
      kbo(:,37, 9) = (/ &
        & 0.16080e+01_rb,0.17565e+01_rb,0.19076e+01_rb,0.20741e+01_rb,0.22287e+01_rb /)
      kbo(:,38, 9) = (/ &
        & 0.15745e+01_rb,0.17245e+01_rb,0.18721e+01_rb,0.20419e+01_rb,0.21969e+01_rb /)
      kbo(:,39, 9) = (/ &
        & 0.15386e+01_rb,0.16919e+01_rb,0.18410e+01_rb,0.19923e+01_rb,0.21638e+01_rb /)
      kbo(:,40, 9) = (/ &
        & 0.14840e+01_rb,0.16355e+01_rb,0.17855e+01_rb,0.19337e+01_rb,0.21026e+01_rb /)
      kbo(:,41, 9) = (/ &
        & 0.14260e+01_rb,0.15758e+01_rb,0.17241e+01_rb,0.18746e+01_rb,0.20398e+01_rb /)
      kbo(:,42, 9) = (/ &
        & 0.13688e+01_rb,0.15162e+01_rb,0.16646e+01_rb,0.18157e+01_rb,0.19780e+01_rb /)
      kbo(:,43, 9) = (/ &
        & 0.13011e+01_rb,0.14473e+01_rb,0.15936e+01_rb,0.17402e+01_rb,0.18823e+01_rb /)
      kbo(:,44, 9) = (/ &
        & 0.12278e+01_rb,0.13710e+01_rb,0.15157e+01_rb,0.16568e+01_rb,0.17952e+01_rb /)
      kbo(:,45, 9) = (/ &
        & 0.11558e+01_rb,0.12957e+01_rb,0.14361e+01_rb,0.15731e+01_rb,0.17094e+01_rb /)
      kbo(:,46, 9) = (/ &
        & 0.10820e+01_rb,0.12147e+01_rb,0.13532e+01_rb,0.14890e+01_rb,0.16181e+01_rb /)
      kbo(:,47, 9) = (/ &
        & 0.10024e+01_rb,0.11346e+01_rb,0.12669e+01_rb,0.14002e+01_rb,0.15260e+01_rb /)
      kbo(:,48, 9) = (/ &
        & 0.92524e+00_rb,0.10524e+01_rb,0.11804e+01_rb,0.13068e+01_rb,0.14296e+01_rb /)
      kbo(:,49, 9) = (/ &
        & 0.84698e+00_rb,0.96805e+00_rb,0.10902e+01_rb,0.12100e+01_rb,0.13299e+01_rb /)
      kbo(:,50, 9) = (/ &
        & 0.78084e+00_rb,0.89928e+00_rb,0.10161e+01_rb,0.11323e+01_rb,0.12472e+01_rb /)
      kbo(:,51, 9) = (/ &
        & 0.72222e+00_rb,0.83480e+00_rb,0.94748e+00_rb,0.10626e+01_rb,0.11740e+01_rb /)
      kbo(:,52, 9) = (/ &
        & 0.66260e+00_rb,0.77045e+00_rb,0.88348e+00_rb,0.99181e+00_rb,0.10995e+01_rb /)
      kbo(:,53, 9) = (/ &
        & 0.60516e+00_rb,0.71083e+00_rb,0.81698e+00_rb,0.91863e+00_rb,0.10219e+01_rb /)
      kbo(:,54, 9) = (/ &
        & 0.56176e+00_rb,0.66472e+00_rb,0.77054e+00_rb,0.87249e+00_rb,0.97306e+00_rb /)
      kbo(:,55, 9) = (/ &
        & 0.52563e+00_rb,0.63037e+00_rb,0.73497e+00_rb,0.83878e+00_rb,0.93923e+00_rb /)
      kbo(:,56, 9) = (/ &
        & 0.49056e+00_rb,0.59429e+00_rb,0.69788e+00_rb,0.80509e+00_rb,0.90615e+00_rb /)
      kbo(:,57, 9) = (/ &
        & 0.45575e+00_rb,0.55861e+00_rb,0.66303e+00_rb,0.76942e+00_rb,0.87211e+00_rb /)
      kbo(:,58, 9) = (/ &
        & 0.42419e+00_rb,0.52609e+00_rb,0.63048e+00_rb,0.73708e+00_rb,0.84331e+00_rb /)
      kbo(:,59, 9) = (/ &
        & 0.42340e+00_rb,0.52953e+00_rb,0.63843e+00_rb,0.74879e+00_rb,0.86046e+00_rb /)
      kbo(:,13,10) = (/ &
        & 0.14973e+02_rb,0.15466e+02_rb,0.15907e+02_rb,0.16294e+02_rb,0.16648e+02_rb /)
      kbo(:,14,10) = (/ &
        & 0.13327e+02_rb,0.13717e+02_rb,0.14036e+02_rb,0.14348e+02_rb,0.14658e+02_rb /)
      kbo(:,15,10) = (/ &
        & 0.12219e+02_rb,0.12428e+02_rb,0.12649e+02_rb,0.12810e+02_rb,0.12959e+02_rb /)
      kbo(:,16,10) = (/ &
        & 0.11245e+02_rb,0.11501e+02_rb,0.11701e+02_rb,0.11817e+02_rb,0.11948e+02_rb /)
      kbo(:,17,10) = (/ &
        & 0.96706e+01_rb,0.99707e+01_rb,0.10257e+02_rb,0.10540e+02_rb,0.10718e+02_rb /)
      kbo(:,18,10) = (/ &
        & 0.83018e+01_rb,0.83446e+01_rb,0.84207e+01_rb,0.85262e+01_rb,0.86783e+01_rb /)
      kbo(:,19,10) = (/ &
        & 0.82250e+01_rb,0.81929e+01_rb,0.82337e+01_rb,0.82572e+01_rb,0.83177e+01_rb /)
      kbo(:,20,10) = (/ &
        & 0.81684e+01_rb,0.81782e+01_rb,0.82244e+01_rb,0.82765e+01_rb,0.82985e+01_rb /)
      kbo(:,21,10) = (/ &
        & 0.82586e+01_rb,0.83055e+01_rb,0.84017e+01_rb,0.83815e+01_rb,0.83594e+01_rb /)
      kbo(:,22,10) = (/ &
        & 0.83686e+01_rb,0.84604e+01_rb,0.85363e+01_rb,0.84786e+01_rb,0.84786e+01_rb /)
      kbo(:,23,10) = (/ &
        & 0.84301e+01_rb,0.85903e+01_rb,0.87569e+01_rb,0.87953e+01_rb,0.87965e+01_rb /)
      kbo(:,24,10) = (/ &
        & 0.85690e+01_rb,0.88097e+01_rb,0.90359e+01_rb,0.90994e+01_rb,0.92201e+01_rb /)
      kbo(:,25,10) = (/ &
        & 0.87852e+01_rb,0.90719e+01_rb,0.92672e+01_rb,0.94098e+01_rb,0.95600e+01_rb /)
      kbo(:,26,10) = (/ &
        & 0.88651e+01_rb,0.92334e+01_rb,0.94525e+01_rb,0.97256e+01_rb,0.98986e+01_rb /)
      kbo(:,27,10) = (/ &
        & 0.88645e+01_rb,0.92932e+01_rb,0.96152e+01_rb,0.98392e+01_rb,0.10037e+02_rb /)
      kbo(:,28,10) = (/ &
        & 0.88548e+01_rb,0.93070e+01_rb,0.96176e+01_rb,0.99813e+01_rb,0.10238e+02_rb /)
      kbo(:,29,10) = (/ &
        & 0.88198e+01_rb,0.92265e+01_rb,0.96024e+01_rb,0.99285e+01_rb,0.10232e+02_rb /)
      kbo(:,30,10) = (/ &
        & 0.87818e+01_rb,0.92083e+01_rb,0.95579e+01_rb,0.98976e+01_rb,0.10208e+02_rb /)
      kbo(:,31,10) = (/ &
        & 0.86036e+01_rb,0.90621e+01_rb,0.94967e+01_rb,0.98500e+01_rb,0.10159e+02_rb /)
      kbo(:,32,10) = (/ &
        & 0.84748e+01_rb,0.89960e+01_rb,0.94428e+01_rb,0.98336e+01_rb,0.10196e+02_rb /)
      kbo(:,33,10) = (/ &
        & 0.84172e+01_rb,0.89420e+01_rb,0.94383e+01_rb,0.98344e+01_rb,0.10060e+02_rb /)
      kbo(:,34,10) = (/ &
        & 0.84605e+01_rb,0.89751e+01_rb,0.94683e+01_rb,0.98852e+01_rb,0.10023e+02_rb /)
      kbo(:,35,10) = (/ &
        & 0.84589e+01_rb,0.89233e+01_rb,0.94002e+01_rb,0.98511e+01_rb,0.10024e+02_rb /)
      kbo(:,36,10) = (/ &
        & 0.84558e+01_rb,0.89580e+01_rb,0.93747e+01_rb,0.98030e+01_rb,0.10000e+02_rb /)
      kbo(:,37,10) = (/ &
        & 0.82385e+01_rb,0.87614e+01_rb,0.91541e+01_rb,0.95514e+01_rb,0.97363e+01_rb /)
      kbo(:,38,10) = (/ &
        & 0.80132e+01_rb,0.84437e+01_rb,0.89304e+01_rb,0.92902e+01_rb,0.94811e+01_rb /)
      kbo(:,39,10) = (/ &
        & 0.78203e+01_rb,0.81981e+01_rb,0.85539e+01_rb,0.89689e+01_rb,0.90927e+01_rb /)
      kbo(:,40,10) = (/ &
        & 0.75978e+01_rb,0.79610e+01_rb,0.82423e+01_rb,0.86095e+01_rb,0.88190e+01_rb /)
      kbo(:,41,10) = (/ &
        & 0.73029e+01_rb,0.76481e+01_rb,0.79708e+01_rb,0.82445e+01_rb,0.84426e+01_rb /)
      kbo(:,42,10) = (/ &
        & 0.70166e+01_rb,0.73754e+01_rb,0.76378e+01_rb,0.78862e+01_rb,0.80985e+01_rb /)
      kbo(:,43,10) = (/ &
        & 0.66293e+01_rb,0.69504e+01_rb,0.72276e+01_rb,0.75120e+01_rb,0.77099e+01_rb /)
      kbo(:,44,10) = (/ &
        & 0.62312e+01_rb,0.65696e+01_rb,0.68647e+01_rb,0.71226e+01_rb,0.73305e+01_rb /)
      kbo(:,45,10) = (/ &
        & 0.58229e+01_rb,0.61620e+01_rb,0.64682e+01_rb,0.67341e+01_rb,0.69070e+01_rb /)
      kbo(:,46,10) = (/ &
        & 0.53657e+01_rb,0.57492e+01_rb,0.60377e+01_rb,0.63031e+01_rb,0.64992e+01_rb /)
      kbo(:,47,10) = (/ &
        & 0.50350e+01_rb,0.53678e+01_rb,0.56656e+01_rb,0.59201e+01_rb,0.61262e+01_rb /)
      kbo(:,48,10) = (/ &
        & 0.46566e+01_rb,0.49901e+01_rb,0.52633e+01_rb,0.54964e+01_rb,0.57537e+01_rb /)
      kbo(:,49,10) = (/ &
        & 0.42801e+01_rb,0.46217e+01_rb,0.49093e+01_rb,0.51104e+01_rb,0.53644e+01_rb /)
      kbo(:,50,10) = (/ &
        & 0.39613e+01_rb,0.43120e+01_rb,0.45848e+01_rb,0.48588e+01_rb,0.50774e+01_rb /)
      kbo(:,51,10) = (/ &
        & 0.37006e+01_rb,0.40662e+01_rb,0.43615e+01_rb,0.46264e+01_rb,0.48701e+01_rb /)
      kbo(:,52,10) = (/ &
        & 0.34485e+01_rb,0.37934e+01_rb,0.40667e+01_rb,0.43544e+01_rb,0.46481e+01_rb /)
      kbo(:,53,10) = (/ &
        & 0.31594e+01_rb,0.35032e+01_rb,0.37934e+01_rb,0.41156e+01_rb,0.44077e+01_rb /)
      kbo(:,54,10) = (/ &
        & 0.29800e+01_rb,0.33284e+01_rb,0.36333e+01_rb,0.39624e+01_rb,0.42528e+01_rb /)
      kbo(:,55,10) = (/ &
        & 0.28569e+01_rb,0.32159e+01_rb,0.35391e+01_rb,0.38186e+01_rb,0.41402e+01_rb /)
      kbo(:,56,10) = (/ &
        & 0.26907e+01_rb,0.31071e+01_rb,0.34476e+01_rb,0.36941e+01_rb,0.40212e+01_rb /)
      kbo(:,57,10) = (/ &
        & 0.25638e+01_rb,0.29869e+01_rb,0.33345e+01_rb,0.36081e+01_rb,0.38859e+01_rb /)
      kbo(:,58,10) = (/ &
        & 0.24665e+01_rb,0.28708e+01_rb,0.32402e+01_rb,0.35331e+01_rb,0.37851e+01_rb /)
      kbo(:,59,10) = (/ &
        & 0.25432e+01_rb,0.29855e+01_rb,0.33662e+01_rb,0.37020e+01_rb,0.39522e+01_rb /)
      kbo(:,13,11) = (/ &
        & 0.31099e+02_rb,0.31647e+02_rb,0.32241e+02_rb,0.32842e+02_rb,0.33417e+02_rb /)
      kbo(:,14,11) = (/ &
        & 0.27698e+02_rb,0.28253e+02_rb,0.28859e+02_rb,0.29460e+02_rb,0.30083e+02_rb /)
      kbo(:,15,11) = (/ &
        & 0.24330e+02_rb,0.24945e+02_rb,0.25542e+02_rb,0.26234e+02_rb,0.26922e+02_rb /)
      kbo(:,16,11) = (/ &
        & 0.21190e+02_rb,0.21758e+02_rb,0.22384e+02_rb,0.23019e+02_rb,0.23628e+02_rb /)
      kbo(:,17,11) = (/ &
        & 0.18967e+02_rb,0.19413e+02_rb,0.19858e+02_rb,0.20261e+02_rb,0.20672e+02_rb /)
      kbo(:,18,11) = (/ &
        & 0.16374e+02_rb,0.16952e+02_rb,0.17497e+02_rb,0.17964e+02_rb,0.18478e+02_rb /)
      kbo(:,19,11) = (/ &
        & 0.13248e+02_rb,0.13545e+02_rb,0.13847e+02_rb,0.14297e+02_rb,0.14705e+02_rb /)
      kbo(:,20,11) = (/ &
        & 0.12757e+02_rb,0.12785e+02_rb,0.12667e+02_rb,0.12601e+02_rb,0.12612e+02_rb /)
      kbo(:,21,11) = (/ &
        & 0.12844e+02_rb,0.12624e+02_rb,0.12433e+02_rb,0.12193e+02_rb,0.12175e+02_rb /)
      kbo(:,22,11) = (/ &
        & 0.12738e+02_rb,0.12567e+02_rb,0.12225e+02_rb,0.12063e+02_rb,0.11980e+02_rb /)
      kbo(:,23,11) = (/ &
        & 0.12815e+02_rb,0.12660e+02_rb,0.12272e+02_rb,0.12108e+02_rb,0.12066e+02_rb /)
      kbo(:,24,11) = (/ &
        & 0.13079e+02_rb,0.12972e+02_rb,0.12651e+02_rb,0.12388e+02_rb,0.12367e+02_rb /)
      kbo(:,25,11) = (/ &
        & 0.13341e+02_rb,0.13256e+02_rb,0.13023e+02_rb,0.12848e+02_rb,0.12659e+02_rb /)
      kbo(:,26,11) = (/ &
        & 0.13512e+02_rb,0.13453e+02_rb,0.13443e+02_rb,0.13113e+02_rb,0.13141e+02_rb /)
      kbo(:,27,11) = (/ &
        & 0.13706e+02_rb,0.13750e+02_rb,0.13824e+02_rb,0.13564e+02_rb,0.13665e+02_rb /)
      kbo(:,28,11) = (/ &
        & 0.13979e+02_rb,0.14080e+02_rb,0.14203e+02_rb,0.13858e+02_rb,0.13981e+02_rb /)
      kbo(:,29,11) = (/ &
        & 0.13934e+02_rb,0.14086e+02_rb,0.14156e+02_rb,0.14057e+02_rb,0.14114e+02_rb /)
      kbo(:,30,11) = (/ &
        & 0.13981e+02_rb,0.13996e+02_rb,0.13975e+02_rb,0.14010e+02_rb,0.14196e+02_rb /)
      kbo(:,31,11) = (/ &
        & 0.13896e+02_rb,0.13857e+02_rb,0.13654e+02_rb,0.13807e+02_rb,0.13871e+02_rb /)
      kbo(:,32,11) = (/ &
        & 0.13655e+02_rb,0.13646e+02_rb,0.13416e+02_rb,0.13538e+02_rb,0.13488e+02_rb /)
      kbo(:,33,11) = (/ &
        & 0.13398e+02_rb,0.13279e+02_rb,0.13235e+02_rb,0.13164e+02_rb,0.13195e+02_rb /)
      kbo(:,34,11) = (/ &
        & 0.13280e+02_rb,0.13155e+02_rb,0.13035e+02_rb,0.13095e+02_rb,0.13023e+02_rb /)
      kbo(:,35,11) = (/ &
        & 0.13174e+02_rb,0.13038e+02_rb,0.12951e+02_rb,0.12819e+02_rb,0.12932e+02_rb /)
      kbo(:,36,11) = (/ &
        & 0.13159e+02_rb,0.12953e+02_rb,0.12943e+02_rb,0.12899e+02_rb,0.13077e+02_rb /)
      kbo(:,37,11) = (/ &
        & 0.12968e+02_rb,0.12722e+02_rb,0.12741e+02_rb,0.12684e+02_rb,0.12881e+02_rb /)
      kbo(:,38,11) = (/ &
        & 0.12648e+02_rb,0.12436e+02_rb,0.12390e+02_rb,0.12534e+02_rb,0.12593e+02_rb /)
      kbo(:,39,11) = (/ &
        & 0.12145e+02_rb,0.12036e+02_rb,0.12198e+02_rb,0.12171e+02_rb,0.12374e+02_rb /)
      kbo(:,40,11) = (/ &
        & 0.11784e+02_rb,0.11714e+02_rb,0.11852e+02_rb,0.11995e+02_rb,0.12020e+02_rb /)
      kbo(:,41,11) = (/ &
        & 0.11419e+02_rb,0.11429e+02_rb,0.11554e+02_rb,0.11799e+02_rb,0.11809e+02_rb /)
      kbo(:,42,11) = (/ &
        & 0.11076e+02_rb,0.11047e+02_rb,0.11287e+02_rb,0.11460e+02_rb,0.11610e+02_rb /)
      kbo(:,43,11) = (/ &
        & 0.10635e+02_rb,0.10772e+02_rb,0.10970e+02_rb,0.11110e+02_rb,0.11339e+02_rb /)
      kbo(:,44,11) = (/ &
        & 0.10081e+02_rb,0.10347e+02_rb,0.10495e+02_rb,0.10710e+02_rb,0.10998e+02_rb /)
      kbo(:,45,11) = (/ &
        & 0.95493e+01_rb,0.97720e+01_rb,0.10013e+02_rb,0.10318e+02_rb,0.10626e+02_rb /)
      kbo(:,46,11) = (/ &
        & 0.91901e+01_rb,0.92815e+01_rb,0.95285e+01_rb,0.99399e+01_rb,0.10216e+02_rb /)
      kbo(:,47,11) = (/ &
        & 0.86762e+01_rb,0.88384e+01_rb,0.91370e+01_rb,0.95143e+01_rb,0.99290e+01_rb /)
      kbo(:,48,11) = (/ &
        & 0.80969e+01_rb,0.84350e+01_rb,0.87222e+01_rb,0.91427e+01_rb,0.95881e+01_rb /)
      kbo(:,49,11) = (/ &
        & 0.74881e+01_rb,0.78692e+01_rb,0.82390e+01_rb,0.88564e+01_rb,0.92509e+01_rb /)
      kbo(:,50,11) = (/ &
        & 0.70193e+01_rb,0.74373e+01_rb,0.79400e+01_rb,0.84415e+01_rb,0.90721e+01_rb /)
      kbo(:,51,11) = (/ &
        & 0.66370e+01_rb,0.71214e+01_rb,0.76381e+01_rb,0.81180e+01_rb,0.89263e+01_rb /)
      kbo(:,52,11) = (/ &
        & 0.62242e+01_rb,0.68097e+01_rb,0.73447e+01_rb,0.79515e+01_rb,0.88951e+01_rb /)
      kbo(:,53,11) = (/ &
        & 0.58238e+01_rb,0.63775e+01_rb,0.69942e+01_rb,0.79247e+01_rb,0.89454e+01_rb /)
      kbo(:,54,11) = (/ &
        & 0.55499e+01_rb,0.61225e+01_rb,0.67115e+01_rb,0.76390e+01_rb,0.86771e+01_rb /)
      kbo(:,55,11) = (/ &
        & 0.53976e+01_rb,0.59288e+01_rb,0.64661e+01_rb,0.72387e+01_rb,0.82482e+01_rb /)
      kbo(:,56,11) = (/ &
        & 0.52864e+01_rb,0.57486e+01_rb,0.62663e+01_rb,0.68889e+01_rb,0.78153e+01_rb /)
      kbo(:,57,11) = (/ &
        & 0.51174e+01_rb,0.55624e+01_rb,0.60808e+01_rb,0.66051e+01_rb,0.73800e+01_rb /)
      kbo(:,58,11) = (/ &
        & 0.49368e+01_rb,0.54350e+01_rb,0.59533e+01_rb,0.64218e+01_rb,0.70057e+01_rb /)
      kbo(:,59,11) = (/ &
        & 0.50812e+01_rb,0.56369e+01_rb,0.60928e+01_rb,0.64990e+01_rb,0.70265e+01_rb /)
      kbo(:,13,12) = (/ &
        & 0.69704e+02_rb,0.70627e+02_rb,0.71228e+02_rb,0.71788e+02_rb,0.72437e+02_rb /)
      kbo(:,14,12) = (/ &
        & 0.64379e+02_rb,0.65225e+02_rb,0.66016e+02_rb,0.66901e+02_rb,0.67785e+02_rb /)
      kbo(:,15,12) = (/ &
        & 0.58655e+02_rb,0.59644e+02_rb,0.60731e+02_rb,0.61788e+02_rb,0.62973e+02_rb /)
      kbo(:,16,12) = (/ &
        & 0.53041e+02_rb,0.54211e+02_rb,0.55406e+02_rb,0.56764e+02_rb,0.58230e+02_rb /)
      kbo(:,17,12) = (/ &
        & 0.47145e+02_rb,0.48549e+02_rb,0.50104e+02_rb,0.51815e+02_rb,0.53646e+02_rb /)
      kbo(:,18,12) = (/ &
        & 0.41118e+02_rb,0.42628e+02_rb,0.44372e+02_rb,0.46307e+02_rb,0.48319e+02_rb /)
      kbo(:,19,12) = (/ &
        & 0.35042e+02_rb,0.36608e+02_rb,0.38443e+02_rb,0.40582e+02_rb,0.43018e+02_rb /)
      kbo(:,20,12) = (/ &
        & 0.28625e+02_rb,0.30224e+02_rb,0.32288e+02_rb,0.34780e+02_rb,0.37414e+02_rb /)
      kbo(:,21,12) = (/ &
        & 0.24017e+02_rb,0.25409e+02_rb,0.27330e+02_rb,0.29600e+02_rb,0.32108e+02_rb /)
      kbo(:,22,12) = (/ &
        & 0.21793e+02_rb,0.22620e+02_rb,0.24096e+02_rb,0.26102e+02_rb,0.28497e+02_rb /)
      kbo(:,23,12) = (/ &
        & 0.21413e+02_rb,0.21831e+02_rb,0.22594e+02_rb,0.23606e+02_rb,0.25745e+02_rb /)
      kbo(:,24,12) = (/ &
        & 0.21009e+02_rb,0.21418e+02_rb,0.22293e+02_rb,0.23189e+02_rb,0.24234e+02_rb /)
      kbo(:,25,12) = (/ &
        & 0.21184e+02_rb,0.21661e+02_rb,0.22476e+02_rb,0.23334e+02_rb,0.24471e+02_rb /)
      kbo(:,26,12) = (/ &
        & 0.21853e+02_rb,0.22488e+02_rb,0.23205e+02_rb,0.24024e+02_rb,0.25192e+02_rb /)
      kbo(:,27,12) = (/ &
        & 0.22275e+02_rb,0.22928e+02_rb,0.23659e+02_rb,0.24768e+02_rb,0.25734e+02_rb /)
      kbo(:,28,12) = (/ &
        & 0.22649e+02_rb,0.23413e+02_rb,0.24091e+02_rb,0.25549e+02_rb,0.26436e+02_rb /)
      kbo(:,29,12) = (/ &
        & 0.22704e+02_rb,0.23483e+02_rb,0.24191e+02_rb,0.25460e+02_rb,0.26653e+02_rb /)
      kbo(:,30,12) = (/ &
        & 0.22666e+02_rb,0.23444e+02_rb,0.24530e+02_rb,0.25500e+02_rb,0.26652e+02_rb /)
      kbo(:,31,12) = (/ &
        & 0.22235e+02_rb,0.23247e+02_rb,0.24248e+02_rb,0.25076e+02_rb,0.26466e+02_rb /)
      kbo(:,32,12) = (/ &
        & 0.22226e+02_rb,0.22827e+02_rb,0.24083e+02_rb,0.25139e+02_rb,0.26181e+02_rb /)
      kbo(:,33,12) = (/ &
        & 0.22210e+02_rb,0.22941e+02_rb,0.24126e+02_rb,0.25269e+02_rb,0.26783e+02_rb /)
      kbo(:,34,12) = (/ &
        & 0.22243e+02_rb,0.23077e+02_rb,0.24337e+02_rb,0.25761e+02_rb,0.27799e+02_rb /)
      kbo(:,35,12) = (/ &
        & 0.22396e+02_rb,0.23583e+02_rb,0.24898e+02_rb,0.26594e+02_rb,0.29392e+02_rb /)
      kbo(:,36,12) = (/ &
        & 0.22982e+02_rb,0.24125e+02_rb,0.25363e+02_rb,0.27311e+02_rb,0.30508e+02_rb /)
      kbo(:,37,12) = (/ &
        & 0.22918e+02_rb,0.23937e+02_rb,0.25281e+02_rb,0.27602e+02_rb,0.31235e+02_rb /)
      kbo(:,38,12) = (/ &
        & 0.22758e+02_rb,0.23720e+02_rb,0.25240e+02_rb,0.28261e+02_rb,0.32012e+02_rb /)
      kbo(:,39,12) = (/ &
        & 0.22553e+02_rb,0.23629e+02_rb,0.25559e+02_rb,0.29164e+02_rb,0.32971e+02_rb /)
      kbo(:,40,12) = (/ &
        & 0.21913e+02_rb,0.23010e+02_rb,0.25397e+02_rb,0.28931e+02_rb,0.33000e+02_rb /)
      kbo(:,41,12) = (/ &
        & 0.21428e+02_rb,0.22543e+02_rb,0.25193e+02_rb,0.28521e+02_rb,0.32655e+02_rb /)
      kbo(:,42,12) = (/ &
        & 0.20806e+02_rb,0.22057e+02_rb,0.24895e+02_rb,0.28307e+02_rb,0.32344e+02_rb /)
      kbo(:,43,12) = (/ &
        & 0.20230e+02_rb,0.21402e+02_rb,0.24438e+02_rb,0.27778e+02_rb,0.31742e+02_rb /)
      kbo(:,44,12) = (/ &
        & 0.19439e+02_rb,0.20816e+02_rb,0.24038e+02_rb,0.27280e+02_rb,0.31112e+02_rb /)
      kbo(:,45,12) = (/ &
        & 0.18615e+02_rb,0.20644e+02_rb,0.23787e+02_rb,0.26906e+02_rb,0.30508e+02_rb /)
      kbo(:,46,12) = (/ &
        & 0.17932e+02_rb,0.20436e+02_rb,0.23444e+02_rb,0.26514e+02_rb,0.30043e+02_rb /)
      kbo(:,47,12) = (/ &
        & 0.17388e+02_rb,0.20062e+02_rb,0.22865e+02_rb,0.25992e+02_rb,0.29187e+02_rb /)
      kbo(:,48,12) = (/ &
        & 0.17144e+02_rb,0.19562e+02_rb,0.22419e+02_rb,0.25556e+02_rb,0.28578e+02_rb /)
      kbo(:,49,12) = (/ &
        & 0.16956e+02_rb,0.19439e+02_rb,0.22133e+02_rb,0.25135e+02_rb,0.28141e+02_rb /)
      kbo(:,50,12) = (/ &
        & 0.16529e+02_rb,0.19034e+02_rb,0.21605e+02_rb,0.24563e+02_rb,0.27331e+02_rb /)
      kbo(:,51,12) = (/ &
        & 0.15953e+02_rb,0.18522e+02_rb,0.21071e+02_rb,0.23856e+02_rb,0.26389e+02_rb /)
      kbo(:,52,12) = (/ &
        & 0.15476e+02_rb,0.18037e+02_rb,0.20576e+02_rb,0.23092e+02_rb,0.25303e+02_rb /)
      kbo(:,53,12) = (/ &
        & 0.14987e+02_rb,0.17619e+02_rb,0.20150e+02_rb,0.22182e+02_rb,0.24323e+02_rb /)
      kbo(:,54,12) = (/ &
        & 0.14316e+02_rb,0.16769e+02_rb,0.19340e+02_rb,0.21295e+02_rb,0.23328e+02_rb /)
      kbo(:,55,12) = (/ &
        & 0.13478e+02_rb,0.15723e+02_rb,0.18284e+02_rb,0.20497e+02_rb,0.22282e+02_rb /)
      kbo(:,56,12) = (/ &
        & 0.12615e+02_rb,0.14717e+02_rb,0.17222e+02_rb,0.19626e+02_rb,0.21412e+02_rb /)
      kbo(:,57,12) = (/ &
        & 0.11789e+02_rb,0.13910e+02_rb,0.16111e+02_rb,0.18559e+02_rb,0.20605e+02_rb /)
      kbo(:,58,12) = (/ &
        & 0.10935e+02_rb,0.12994e+02_rb,0.15023e+02_rb,0.17386e+02_rb,0.19630e+02_rb /)
      kbo(:,59,12) = (/ &
        & 0.10847e+02_rb,0.11985e+02_rb,0.13699e+02_rb,0.15922e+02_rb,0.18130e+02_rb /)
      kbo(:,13,13) = (/ &
        & 0.15243e+03_rb,0.15414e+03_rb,0.15604e+03_rb,0.15731e+03_rb,0.15805e+03_rb /)
      kbo(:,14,13) = (/ &
        & 0.15017e+03_rb,0.15285e+03_rb,0.15509e+03_rb,0.15662e+03_rb,0.15764e+03_rb /)
      kbo(:,15,13) = (/ &
        & 0.14701e+03_rb,0.15032e+03_rb,0.15250e+03_rb,0.15435e+03_rb,0.15583e+03_rb /)
      kbo(:,16,13) = (/ &
        & 0.14269e+03_rb,0.14580e+03_rb,0.14857e+03_rb,0.15094e+03_rb,0.15306e+03_rb /)
      kbo(:,17,13) = (/ &
        & 0.13670e+03_rb,0.14035e+03_rb,0.14376e+03_rb,0.14689e+03_rb,0.14976e+03_rb /)
      kbo(:,18,13) = (/ &
        & 0.13015e+03_rb,0.13455e+03_rb,0.13871e+03_rb,0.14263e+03_rb,0.14625e+03_rb /)
      kbo(:,19,13) = (/ &
        & 0.12287e+03_rb,0.12845e+03_rb,0.13370e+03_rb,0.13836e+03_rb,0.14277e+03_rb /)
      kbo(:,20,13) = (/ &
        & 0.11516e+03_rb,0.12176e+03_rb,0.12807e+03_rb,0.13396e+03_rb,0.13973e+03_rb /)
      kbo(:,21,13) = (/ &
        & 0.10676e+03_rb,0.11461e+03_rb,0.12196e+03_rb,0.12926e+03_rb,0.13605e+03_rb /)
      kbo(:,22,13) = (/ &
        & 0.98495e+02_rb,0.10775e+03_rb,0.11663e+03_rb,0.12493e+03_rb,0.13260e+03_rb /)
      kbo(:,23,13) = (/ &
        & 0.89816e+02_rb,0.10015e+03_rb,0.11065e+03_rb,0.12084e+03_rb,0.12951e+03_rb /)
      kbo(:,24,13) = (/ &
        & 0.82797e+02_rb,0.93590e+02_rb,0.10444e+03_rb,0.11545e+03_rb,0.12590e+03_rb /)
      kbo(:,25,13) = (/ &
        & 0.77256e+02_rb,0.88067e+02_rb,0.99408e+02_rb,0.11069e+03_rb,0.12164e+03_rb /)
      kbo(:,26,13) = (/ &
        & 0.73787e+02_rb,0.84230e+02_rb,0.95403e+02_rb,0.10719e+03_rb,0.11797e+03_rb /)
      kbo(:,27,13) = (/ &
        & 0.72626e+02_rb,0.82623e+02_rb,0.93417e+02_rb,0.10482e+03_rb,0.11588e+03_rb /)
      kbo(:,28,13) = (/ &
        & 0.72289e+02_rb,0.81705e+02_rb,0.92658e+02_rb,0.10355e+03_rb,0.11467e+03_rb /)
      kbo(:,29,13) = (/ &
        & 0.73831e+02_rb,0.83248e+02_rb,0.93801e+02_rb,0.10474e+03_rb,0.11551e+03_rb /)
      kbo(:,30,13) = (/ &
        & 0.76016e+02_rb,0.86113e+02_rb,0.95367e+02_rb,0.10651e+03_rb,0.11733e+03_rb /)
      kbo(:,31,13) = (/ &
        & 0.80058e+02_rb,0.89705e+02_rb,0.99793e+02_rb,0.10982e+03_rb,0.12052e+03_rb /)
      kbo(:,32,13) = (/ &
        & 0.85064e+02_rb,0.94860e+02_rb,0.10434e+03_rb,0.11401e+03_rb,0.12404e+03_rb /)
      kbo(:,33,13) = (/ &
        & 0.90655e+02_rb,0.10081e+03_rb,0.10992e+03_rb,0.11933e+03_rb,0.12788e+03_rb /)
      kbo(:,34,13) = (/ &
        & 0.95857e+02_rb,0.10597e+03_rb,0.11520e+03_rb,0.12356e+03_rb,0.13127e+03_rb /)
      kbo(:,35,13) = (/ &
        & 0.99393e+02_rb,0.11014e+03_rb,0.12003e+03_rb,0.12794e+03_rb,0.13407e+03_rb /)
      kbo(:,36,13) = (/ &
        & 0.10100e+03_rb,0.11220e+03_rb,0.12221e+03_rb,0.13085e+03_rb,0.13724e+03_rb /)
      kbo(:,37,13) = (/ &
        & 0.10052e+03_rb,0.11217e+03_rb,0.12302e+03_rb,0.13127e+03_rb,0.13704e+03_rb /)
      kbo(:,38,13) = (/ &
        & 0.10046e+03_rb,0.11273e+03_rb,0.12310e+03_rb,0.13118e+03_rb,0.13742e+03_rb /)
      kbo(:,39,13) = (/ &
        & 0.10122e+03_rb,0.11326e+03_rb,0.12361e+03_rb,0.13081e+03_rb,0.13784e+03_rb /)
      kbo(:,40,13) = (/ &
        & 0.10000e+03_rb,0.11180e+03_rb,0.12195e+03_rb,0.12930e+03_rb,0.13636e+03_rb /)
      kbo(:,41,13) = (/ &
        & 0.98394e+02_rb,0.11024e+03_rb,0.11969e+03_rb,0.12788e+03_rb,0.13497e+03_rb /)
      kbo(:,42,13) = (/ &
        & 0.97071e+02_rb,0.10823e+03_rb,0.11769e+03_rb,0.12630e+03_rb,0.13330e+03_rb /)
      kbo(:,43,13) = (/ &
        & 0.94820e+02_rb,0.10592e+03_rb,0.11514e+03_rb,0.12393e+03_rb,0.13107e+03_rb /)
      kbo(:,44,13) = (/ &
        & 0.92475e+02_rb,0.10313e+03_rb,0.11198e+03_rb,0.12120e+03_rb,0.12869e+03_rb /)
      kbo(:,45,13) = (/ &
        & 0.90255e+02_rb,0.99678e+02_rb,0.10888e+03_rb,0.11824e+03_rb,0.12634e+03_rb /)
      kbo(:,46,13) = (/ &
        & 0.87084e+02_rb,0.96376e+02_rb,0.10582e+03_rb,0.11496e+03_rb,0.12350e+03_rb /)
      kbo(:,47,13) = (/ &
        & 0.83116e+02_rb,0.92026e+02_rb,0.10209e+03_rb,0.11120e+03_rb,0.12020e+03_rb /)
      kbo(:,48,13) = (/ &
        & 0.78845e+02_rb,0.88316e+02_rb,0.97984e+02_rb,0.10729e+03_rb,0.11639e+03_rb /)
      kbo(:,49,13) = (/ &
        & 0.74841e+02_rb,0.84389e+02_rb,0.93907e+02_rb,0.10286e+03_rb,0.11277e+03_rb /)
      kbo(:,50,13) = (/ &
        & 0.71259e+02_rb,0.80370e+02_rb,0.89851e+02_rb,0.99011e+02_rb,0.10882e+03_rb /)
      kbo(:,51,13) = (/ &
        & 0.67667e+02_rb,0.76332e+02_rb,0.85552e+02_rb,0.95111e+02_rb,0.10517e+03_rb /)
      kbo(:,52,13) = (/ &
        & 0.64036e+02_rb,0.72373e+02_rb,0.81228e+02_rb,0.91458e+02_rb,0.10109e+03_rb /)
      kbo(:,53,13) = (/ &
        & 0.60531e+02_rb,0.68539e+02_rb,0.77029e+02_rb,0.87314e+02_rb,0.97171e+02_rb /)
      kbo(:,54,13) = (/ &
        & 0.57169e+02_rb,0.65298e+02_rb,0.73473e+02_rb,0.83504e+02_rb,0.93303e+02_rb /)
      kbo(:,55,13) = (/ &
        & 0.53955e+02_rb,0.61813e+02_rb,0.70099e+02_rb,0.79656e+02_rb,0.89859e+02_rb /)
      kbo(:,56,13) = (/ &
        & 0.50772e+02_rb,0.58658e+02_rb,0.66829e+02_rb,0.76079e+02_rb,0.86127e+02_rb /)
      kbo(:,57,13) = (/ &
        & 0.47708e+02_rb,0.55213e+02_rb,0.63250e+02_rb,0.72545e+02_rb,0.82495e+02_rb /)
      kbo(:,58,13) = (/ &
        & 0.45026e+02_rb,0.52243e+02_rb,0.60123e+02_rb,0.69281e+02_rb,0.79017e+02_rb /)
      kbo(:,59,13) = (/ &
        & 0.42111e+02_rb,0.50365e+02_rb,0.58368e+02_rb,0.67832e+02_rb,0.76877e+02_rb /)
      kbo(:,13,14) = (/ &
        & 0.32174e+03_rb,0.31744e+03_rb,0.31218e+03_rb,0.30740e+03_rb,0.30288e+03_rb /)
      kbo(:,14,14) = (/ &
        & 0.33593e+03_rb,0.33142e+03_rb,0.32669e+03_rb,0.32231e+03_rb,0.31834e+03_rb /)
      kbo(:,15,14) = (/ &
        & 0.34823e+03_rb,0.34389e+03_rb,0.34037e+03_rb,0.33690e+03_rb,0.33355e+03_rb /)
      kbo(:,16,14) = (/ &
        & 0.35878e+03_rb,0.35631e+03_rb,0.35376e+03_rb,0.35105e+03_rb,0.34819e+03_rb /)
      kbo(:,17,14) = (/ &
        & 0.36906e+03_rb,0.36793e+03_rb,0.36636e+03_rb,0.36433e+03_rb,0.36182e+03_rb /)
      kbo(:,18,14) = (/ &
        & 0.37817e+03_rb,0.37835e+03_rb,0.37763e+03_rb,0.37618e+03_rb,0.37389e+03_rb /)
      kbo(:,19,14) = (/ &
        & 0.38585e+03_rb,0.38729e+03_rb,0.38746e+03_rb,0.38655e+03_rb,0.38433e+03_rb /)
      kbo(:,20,14) = (/ &
        & 0.39236e+03_rb,0.39495e+03_rb,0.39589e+03_rb,0.39530e+03_rb,0.39330e+03_rb /)
      kbo(:,21,14) = (/ &
        & 0.39808e+03_rb,0.40159e+03_rb,0.40312e+03_rb,0.40280e+03_rb,0.40090e+03_rb /)
      kbo(:,22,14) = (/ &
        & 0.40338e+03_rb,0.40757e+03_rb,0.40933e+03_rb,0.40908e+03_rb,0.40708e+03_rb /)
      kbo(:,23,14) = (/ &
        & 0.40834e+03_rb,0.41282e+03_rb,0.41463e+03_rb,0.41442e+03_rb,0.41222e+03_rb /)
      kbo(:,24,14) = (/ &
        & 0.41280e+03_rb,0.41739e+03_rb,0.41916e+03_rb,0.41879e+03_rb,0.41631e+03_rb /)
      kbo(:,25,14) = (/ &
        & 0.41635e+03_rb,0.42140e+03_rb,0.42295e+03_rb,0.42231e+03_rb,0.41948e+03_rb /)
      kbo(:,26,14) = (/ &
        & 0.41887e+03_rb,0.42439e+03_rb,0.42613e+03_rb,0.42509e+03_rb,0.42199e+03_rb /)
      kbo(:,27,14) = (/ &
        & 0.42034e+03_rb,0.42637e+03_rb,0.42837e+03_rb,0.42723e+03_rb,0.42380e+03_rb /)
      kbo(:,28,14) = (/ &
        & 0.42191e+03_rb,0.42841e+03_rb,0.43008e+03_rb,0.42870e+03_rb,0.42501e+03_rb /)
      kbo(:,29,14) = (/ &
        & 0.42351e+03_rb,0.42979e+03_rb,0.43162e+03_rb,0.42986e+03_rb,0.42579e+03_rb /)
      kbo(:,30,14) = (/ &
        & 0.42461e+03_rb,0.42985e+03_rb,0.43291e+03_rb,0.43068e+03_rb,0.42610e+03_rb /)
      kbo(:,31,14) = (/ &
        & 0.42505e+03_rb,0.43017e+03_rb,0.43191e+03_rb,0.43099e+03_rb,0.42603e+03_rb /)
      kbo(:,32,14) = (/ &
        & 0.42362e+03_rb,0.42860e+03_rb,0.43041e+03_rb,0.42915e+03_rb,0.42555e+03_rb /)
      kbo(:,33,14) = (/ &
        & 0.42123e+03_rb,0.42505e+03_rb,0.42663e+03_rb,0.42551e+03_rb,0.42290e+03_rb /)
      kbo(:,34,14) = (/ &
        & 0.41773e+03_rb,0.42082e+03_rb,0.42178e+03_rb,0.42110e+03_rb,0.41858e+03_rb /)
      kbo(:,35,14) = (/ &
        & 0.41435e+03_rb,0.41544e+03_rb,0.41491e+03_rb,0.41443e+03_rb,0.41216e+03_rb /)
      kbo(:,36,14) = (/ &
        & 0.41009e+03_rb,0.41065e+03_rb,0.40988e+03_rb,0.40756e+03_rb,0.40367e+03_rb /)
      kbo(:,37,14) = (/ &
        & 0.40889e+03_rb,0.40948e+03_rb,0.40755e+03_rb,0.40547e+03_rb,0.40195e+03_rb /)
      kbo(:,38,14) = (/ &
        & 0.40736e+03_rb,0.40781e+03_rb,0.40666e+03_rb,0.40322e+03_rb,0.39939e+03_rb /)
      kbo(:,39,14) = (/ &
        & 0.40514e+03_rb,0.40605e+03_rb,0.40431e+03_rb,0.40166e+03_rb,0.39666e+03_rb /)
      kbo(:,40,14) = (/ &
        & 0.40543e+03_rb,0.40633e+03_rb,0.40490e+03_rb,0.40260e+03_rb,0.39791e+03_rb /)
      kbo(:,41,14) = (/ &
        & 0.40503e+03_rb,0.40615e+03_rb,0.40599e+03_rb,0.40348e+03_rb,0.39947e+03_rb /)
      kbo(:,42,14) = (/ &
        & 0.40408e+03_rb,0.40664e+03_rb,0.40681e+03_rb,0.40441e+03_rb,0.40107e+03_rb /)
      kbo(:,43,14) = (/ &
        & 0.40250e+03_rb,0.40659e+03_rb,0.40791e+03_rb,0.40633e+03_rb,0.40362e+03_rb /)
      kbo(:,44,14) = (/ &
        & 0.40121e+03_rb,0.40653e+03_rb,0.40934e+03_rb,0.40832e+03_rb,0.40616e+03_rb /)
      kbo(:,45,14) = (/ &
        & 0.39928e+03_rb,0.40669e+03_rb,0.41014e+03_rb,0.41010e+03_rb,0.40849e+03_rb /)
      kbo(:,46,14) = (/ &
        & 0.39656e+03_rb,0.40551e+03_rb,0.41027e+03_rb,0.41167e+03_rb,0.41068e+03_rb /)
      kbo(:,47,14) = (/ &
        & 0.39317e+03_rb,0.40403e+03_rb,0.40985e+03_rb,0.41273e+03_rb,0.41264e+03_rb /)
      kbo(:,48,14) = (/ &
        & 0.38915e+03_rb,0.40116e+03_rb,0.40932e+03_rb,0.41346e+03_rb,0.41469e+03_rb /)
      kbo(:,49,14) = (/ &
        & 0.38398e+03_rb,0.39755e+03_rb,0.40789e+03_rb,0.41416e+03_rb,0.41577e+03_rb /)
      kbo(:,50,14) = (/ &
        & 0.37757e+03_rb,0.39396e+03_rb,0.40616e+03_rb,0.41381e+03_rb,0.41707e+03_rb /)
      kbo(:,51,14) = (/ &
        & 0.37117e+03_rb,0.38987e+03_rb,0.40415e+03_rb,0.41318e+03_rb,0.41740e+03_rb /)
      kbo(:,52,14) = (/ &
        & 0.36408e+03_rb,0.38505e+03_rb,0.40161e+03_rb,0.41151e+03_rb,0.41801e+03_rb /)
      kbo(:,53,14) = (/ &
        & 0.35611e+03_rb,0.37948e+03_rb,0.39824e+03_rb,0.40994e+03_rb,0.41764e+03_rb /)
      kbo(:,54,14) = (/ &
        & 0.34712e+03_rb,0.37341e+03_rb,0.39425e+03_rb,0.40798e+03_rb,0.41739e+03_rb /)
      kbo(:,55,14) = (/ &
        & 0.33857e+03_rb,0.36757e+03_rb,0.38995e+03_rb,0.40583e+03_rb,0.41654e+03_rb /)
      kbo(:,56,14) = (/ &
        & 0.32949e+03_rb,0.36056e+03_rb,0.38492e+03_rb,0.40281e+03_rb,0.41532e+03_rb /)
      kbo(:,57,14) = (/ &
        & 0.31954e+03_rb,0.35312e+03_rb,0.37997e+03_rb,0.39944e+03_rb,0.41349e+03_rb /)
      kbo(:,58,14) = (/ &
        & 0.30943e+03_rb,0.34518e+03_rb,0.37428e+03_rb,0.39571e+03_rb,0.41155e+03_rb /)
      kbo(:,59,14) = (/ &
        & 0.30628e+03_rb,0.34259e+03_rb,0.37284e+03_rb,0.39463e+03_rb,0.41220e+03_rb /)
      kbo(:,13,15) = (/ &
        & 0.58702e+03_rb,0.56862e+03_rb,0.55331e+03_rb,0.53896e+03_rb,0.52495e+03_rb /)
      kbo(:,14,15) = (/ &
        & 0.64037e+03_rb,0.62129e+03_rb,0.60436e+03_rb,0.58772e+03_rb,0.57127e+03_rb /)
      kbo(:,15,15) = (/ &
        & 0.69642e+03_rb,0.67640e+03_rb,0.65677e+03_rb,0.63714e+03_rb,0.61752e+03_rb /)
      kbo(:,16,15) = (/ &
        & 0.75408e+03_rb,0.73150e+03_rb,0.70834e+03_rb,0.68512e+03_rb,0.66206e+03_rb /)
      kbo(:,17,15) = (/ &
        & 0.81156e+03_rb,0.78496e+03_rb,0.75773e+03_rb,0.73064e+03_rb,0.70441e+03_rb /)
      kbo(:,18,15) = (/ &
        & 0.86684e+03_rb,0.83552e+03_rb,0.80425e+03_rb,0.77370e+03_rb,0.74441e+03_rb /)
      kbo(:,19,15) = (/ &
        & 0.91879e+03_rb,0.88284e+03_rb,0.84773e+03_rb,0.81356e+03_rb,0.78105e+03_rb /)
      kbo(:,20,15) = (/ &
        & 0.96599e+03_rb,0.92558e+03_rb,0.88651e+03_rb,0.84884e+03_rb,0.81303e+03_rb /)
      kbo(:,21,15) = (/ &
        & 0.10076e+04_rb,0.96291e+03_rb,0.92009e+03_rb,0.87920e+03_rb,0.84047e+03_rb /)
      kbo(:,22,15) = (/ &
        & 0.10401e+04_rb,0.99157e+03_rb,0.94567e+03_rb,0.90208e+03_rb,0.86095e+03_rb /)
      kbo(:,23,15) = (/ &
        & 0.10659e+04_rb,0.10143e+04_rb,0.96587e+03_rb,0.91983e+03_rb,0.87686e+03_rb /)
      kbo(:,24,15) = (/ &
        & 0.10857e+04_rb,0.10316e+04_rb,0.98077e+03_rb,0.93285e+03_rb,0.88829e+03_rb /)
      kbo(:,25,15) = (/ &
        & 0.10994e+04_rb,0.10434e+04_rb,0.99070e+03_rb,0.94138e+03_rb,0.89569e+03_rb /)
      kbo(:,26,15) = (/ &
        & 0.11072e+04_rb,0.10495e+04_rb,0.99578e+03_rb,0.94553e+03_rb,0.89886e+03_rb /)
      kbo(:,27,15) = (/ &
        & 0.11107e+04_rb,0.10519e+04_rb,0.99732e+03_rb,0.94654e+03_rb,0.89932e+03_rb /)
      kbo(:,28,15) = (/ &
        & 0.11104e+04_rb,0.10510e+04_rb,0.99596e+03_rb,0.94493e+03_rb,0.89744e+03_rb /)
      kbo(:,29,15) = (/ &
        & 0.11068e+04_rb,0.10472e+04_rb,0.99206e+03_rb,0.94099e+03_rb,0.89320e+03_rb /)
      kbo(:,30,15) = (/ &
        & 0.11008e+04_rb,0.10413e+04_rb,0.98636e+03_rb,0.93544e+03_rb,0.88772e+03_rb /)
      kbo(:,31,15) = (/ &
        & 0.10926e+04_rb,0.10334e+04_rb,0.97895e+03_rb,0.92827e+03_rb,0.88094e+03_rb /)
      kbo(:,32,15) = (/ &
        & 0.10829e+04_rb,0.10242e+04_rb,0.97015e+03_rb,0.91986e+03_rb,0.87315e+03_rb /)
      kbo(:,33,15) = (/ &
        & 0.10718e+04_rb,0.10140e+04_rb,0.96048e+03_rb,0.91059e+03_rb,0.86462e+03_rb /)
      kbo(:,34,15) = (/ &
        & 0.10622e+04_rb,0.10049e+04_rb,0.95198e+03_rb,0.90258e+03_rb,0.85725e+03_rb /)
      kbo(:,35,15) = (/ &
        & 0.10576e+04_rb,0.10006e+04_rb,0.94772e+03_rb,0.89854e+03_rb,0.85365e+03_rb /)
      kbo(:,36,15) = (/ &
        & 0.10589e+04_rb,0.10017e+04_rb,0.94880e+03_rb,0.89934e+03_rb,0.85436e+03_rb /)
      kbo(:,37,15) = (/ &
        & 0.10689e+04_rb,0.10108e+04_rb,0.95721e+03_rb,0.90723e+03_rb,0.86136e+03_rb /)
      kbo(:,38,15) = (/ &
        & 0.10792e+04_rb,0.10202e+04_rb,0.96581e+03_rb,0.91534e+03_rb,0.86861e+03_rb /)
      kbo(:,39,15) = (/ &
        & 0.10893e+04_rb,0.10294e+04_rb,0.97432e+03_rb,0.92308e+03_rb,0.87586e+03_rb /)
      kbo(:,40,15) = (/ &
        & 0.11025e+04_rb,0.10445e+04_rb,0.98826e+03_rb,0.93598e+03_rb,0.88770e+03_rb /)
      kbo(:,41,15) = (/ &
        & 0.11189e+04_rb,0.10606e+04_rb,0.10030e+04_rb,0.94969e+03_rb,0.90027e+03_rb /)
      kbo(:,42,15) = (/ &
        & 0.11355e+04_rb,0.10769e+04_rb,0.10180e+04_rb,0.96359e+03_rb,0.91305e+03_rb /)
      kbo(:,43,15) = (/ &
        & 0.11585e+04_rb,0.10971e+04_rb,0.10364e+04_rb,0.98067e+03_rb,0.92886e+03_rb /)
      kbo(:,44,15) = (/ &
        & 0.11827e+04_rb,0.11193e+04_rb,0.10568e+04_rb,0.99947e+03_rb,0.94636e+03_rb /)
      kbo(:,45,15) = (/ &
        & 0.12075e+04_rb,0.11422e+04_rb,0.10778e+04_rb,0.10187e+04_rb,0.96421e+03_rb /)
      kbo(:,46,15) = (/ &
        & 0.12357e+04_rb,0.11671e+04_rb,0.11007e+04_rb,0.10396e+04_rb,0.98362e+03_rb /)
      kbo(:,47,15) = (/ &
        & 0.12667e+04_rb,0.11958e+04_rb,0.11271e+04_rb,0.10639e+04_rb,0.10059e+04_rb /)
      kbo(:,48,15) = (/ &
        & 0.12983e+04_rb,0.12254e+04_rb,0.11543e+04_rb,0.10888e+04_rb,0.10288e+04_rb /)
      kbo(:,49,15) = (/ &
        & 0.13304e+04_rb,0.12559e+04_rb,0.11825e+04_rb,0.11147e+04_rb,0.10526e+04_rb /)
      kbo(:,50,15) = (/ &
        & 0.13621e+04_rb,0.12852e+04_rb,0.12096e+04_rb,0.11397e+04_rb,0.10755e+04_rb /)
      kbo(:,51,15) = (/ &
        & 0.13919e+04_rb,0.13140e+04_rb,0.12368e+04_rb,0.11648e+04_rb,0.10985e+04_rb /)
      kbo(:,52,15) = (/ &
        & 0.14222e+04_rb,0.13432e+04_rb,0.12647e+04_rb,0.11905e+04_rb,0.11221e+04_rb /)
      kbo(:,53,15) = (/ &
        & 0.14531e+04_rb,0.13729e+04_rb,0.12932e+04_rb,0.12171e+04_rb,0.11465e+04_rb /)
      kbo(:,54,15) = (/ &
        & 0.14842e+04_rb,0.14002e+04_rb,0.13197e+04_rb,0.12421e+04_rb,0.11696e+04_rb /)
      kbo(:,55,15) = (/ &
        & 0.15120e+04_rb,0.14267e+04_rb,0.13453e+04_rb,0.12667e+04_rb,0.11922e+04_rb /)
      kbo(:,56,15) = (/ &
        & 0.15401e+04_rb,0.14538e+04_rb,0.13712e+04_rb,0.12915e+04_rb,0.12155e+04_rb /)
      kbo(:,57,15) = (/ &
        & 0.15682e+04_rb,0.14811e+04_rb,0.13973e+04_rb,0.13167e+04_rb,0.12393e+04_rb /)
      kbo(:,58,15) = (/ &
        & 0.15949e+04_rb,0.15073e+04_rb,0.14224e+04_rb,0.13410e+04_rb,0.12625e+04_rb /)
      kbo(:,59,15) = (/ &
        & 0.16055e+04_rb,0.15180e+04_rb,0.14326e+04_rb,0.13507e+04_rb,0.12718e+04_rb /)
      kbo(:,13,16) = (/ &
        & 0.98069e+03_rb,0.94212e+03_rb,0.90378e+03_rb,0.86861e+03_rb,0.83780e+03_rb /)
      kbo(:,14,16) = (/ &
        & 0.11010e+04_rb,0.10540e+04_rb,0.10095e+04_rb,0.96975e+03_rb,0.93339e+03_rb /)
      kbo(:,15,16) = (/ &
        & 0.12287e+04_rb,0.11714e+04_rb,0.11195e+04_rb,0.10736e+04_rb,0.10305e+04_rb /)
      kbo(:,16,16) = (/ &
        & 0.13596e+04_rb,0.12909e+04_rb,0.12306e+04_rb,0.11762e+04_rb,0.11252e+04_rb /)
      kbo(:,17,16) = (/ &
        & 0.14895e+04_rb,0.14091e+04_rb,0.13394e+04_rb,0.12751e+04_rb,0.12151e+04_rb /)
      kbo(:,18,16) = (/ &
        & 0.16148e+04_rb,0.15230e+04_rb,0.14425e+04_rb,0.13683e+04_rb,0.12989e+04_rb /)
      kbo(:,19,16) = (/ &
        & 0.17336e+04_rb,0.16302e+04_rb,0.15375e+04_rb,0.14530e+04_rb,0.13744e+04_rb /)
      kbo(:,20,16) = (/ &
        & 0.18405e+04_rb,0.17248e+04_rb,0.16208e+04_rb,0.15267e+04_rb,0.14397e+04_rb /)
      kbo(:,21,16) = (/ &
        & 0.19359e+04_rb,0.18083e+04_rb,0.16938e+04_rb,0.15909e+04_rb,0.14962e+04_rb /)
      kbo(:,22,16) = (/ &
        & 0.20085e+04_rb,0.18707e+04_rb,0.17478e+04_rb,0.16373e+04_rb,0.15366e+04_rb /)
      kbo(:,23,16) = (/ &
        & 0.20648e+04_rb,0.19188e+04_rb,0.17890e+04_rb,0.16725e+04_rb,0.15672e+04_rb /)
      kbo(:,24,16) = (/ &
        & 0.21054e+04_rb,0.19528e+04_rb,0.18178e+04_rb,0.16967e+04_rb,0.15881e+04_rb /)
      kbo(:,25,16) = (/ &
        & 0.21318e+04_rb,0.19742e+04_rb,0.18353e+04_rb,0.17113e+04_rb,0.16005e+04_rb /)
      kbo(:,26,16) = (/ &
        & 0.21433e+04_rb,0.19836e+04_rb,0.18416e+04_rb,0.17161e+04_rb,0.16044e+04_rb /)
      kbo(:,27,16) = (/ &
        & 0.21445e+04_rb,0.19835e+04_rb,0.18409e+04_rb,0.17145e+04_rb,0.16033e+04_rb /)
      kbo(:,28,16) = (/ &
        & 0.21374e+04_rb,0.19764e+04_rb,0.18336e+04_rb,0.17081e+04_rb,0.15978e+04_rb /)
      kbo(:,29,16) = (/ &
        & 0.21225e+04_rb,0.19618e+04_rb,0.18208e+04_rb,0.16961e+04_rb,0.15894e+04_rb /)
      kbo(:,30,16) = (/ &
        & 0.21033e+04_rb,0.19434e+04_rb,0.18039e+04_rb,0.16808e+04_rb,0.15775e+04_rb /)
      kbo(:,31,16) = (/ &
        & 0.20779e+04_rb,0.19206e+04_rb,0.17837e+04_rb,0.16635e+04_rb,0.15631e+04_rb /)
      kbo(:,32,16) = (/ &
        & 0.20491e+04_rb,0.18949e+04_rb,0.17612e+04_rb,0.16447e+04_rb,0.15471e+04_rb /)
      kbo(:,33,16) = (/ &
        & 0.20177e+04_rb,0.18682e+04_rb,0.17367e+04_rb,0.16246e+04_rb,0.15302e+04_rb /)
      kbo(:,34,16) = (/ &
        & 0.19905e+04_rb,0.18445e+04_rb,0.17155e+04_rb,0.16076e+04_rb,0.15146e+04_rb /)
      kbo(:,35,16) = (/ &
        & 0.19774e+04_rb,0.18329e+04_rb,0.17061e+04_rb,0.16001e+04_rb,0.15070e+04_rb /)
      kbo(:,36,16) = (/ &
        & 0.19799e+04_rb,0.18350e+04_rb,0.17079e+04_rb,0.16017e+04_rb,0.15083e+04_rb /)
      kbo(:,37,16) = (/ &
        & 0.20051e+04_rb,0.18569e+04_rb,0.17271e+04_rb,0.16176e+04_rb,0.15230e+04_rb /)
      kbo(:,38,16) = (/ &
        & 0.20315e+04_rb,0.18800e+04_rb,0.17473e+04_rb,0.16343e+04_rb,0.15379e+04_rb /)
      kbo(:,39,16) = (/ &
        & 0.20576e+04_rb,0.19026e+04_rb,0.17672e+04_rb,0.16508e+04_rb,0.15527e+04_rb /)
      kbo(:,40,16) = (/ &
        & 0.21013e+04_rb,0.19405e+04_rb,0.18004e+04_rb,0.16787e+04_rb,0.15771e+04_rb /)
      kbo(:,41,16) = (/ &
        & 0.21482e+04_rb,0.19814e+04_rb,0.18363e+04_rb,0.17091e+04_rb,0.16029e+04_rb /)
      kbo(:,42,16) = (/ &
        & 0.21964e+04_rb,0.20236e+04_rb,0.18732e+04_rb,0.17410e+04_rb,0.16295e+04_rb /)
      kbo(:,43,16) = (/ &
        & 0.22568e+04_rb,0.20769e+04_rb,0.19193e+04_rb,0.17816e+04_rb,0.16630e+04_rb /)
      kbo(:,44,16) = (/ &
        & 0.23247e+04_rb,0.21362e+04_rb,0.19712e+04_rb,0.18270e+04_rb,0.17012e+04_rb /)
      kbo(:,45,16) = (/ &
        & 0.23959e+04_rb,0.21983e+04_rb,0.20252e+04_rb,0.18744e+04_rb,0.17420e+04_rb /)
      kbo(:,46,16) = (/ &
        & 0.24748e+04_rb,0.22670e+04_rb,0.20858e+04_rb,0.19269e+04_rb,0.17884e+04_rb /)
      kbo(:,47,16) = (/ &
        & 0.25688e+04_rb,0.23479e+04_rb,0.21568e+04_rb,0.19886e+04_rb,0.18424e+04_rb /)
      kbo(:,48,16) = (/ &
        & 0.26704e+04_rb,0.24334e+04_rb,0.22313e+04_rb,0.20542e+04_rb,0.18994e+04_rb /)
      kbo(:,49,16) = (/ &
        & 0.27805e+04_rb,0.25238e+04_rb,0.23096e+04_rb,0.21232e+04_rb,0.19594e+04_rb /)
      kbo(:,50,16) = (/ &
        & 0.28920e+04_rb,0.26151e+04_rb,0.23873e+04_rb,0.21911e+04_rb,0.20186e+04_rb /)
      kbo(:,51,16) = (/ &
        & 0.30093e+04_rb,0.27098e+04_rb,0.24666e+04_rb,0.22601e+04_rb,0.20794e+04_rb /)
      kbo(:,52,16) = (/ &
        & 0.31338e+04_rb,0.28121e+04_rb,0.25503e+04_rb,0.23323e+04_rb,0.21428e+04_rb /)
      kbo(:,53,16) = (/ &
        & 0.32663e+04_rb,0.29228e+04_rb,0.26405e+04_rb,0.24086e+04_rb,0.22095e+04_rb /)
      kbo(:,54,16) = (/ &
        & 0.33958e+04_rb,0.30317e+04_rb,0.27288e+04_rb,0.24819e+04_rb,0.22731e+04_rb /)
      kbo(:,55,16) = (/ &
        & 0.35272e+04_rb,0.31418e+04_rb,0.28194e+04_rb,0.25560e+04_rb,0.23371e+04_rb /)
      kbo(:,56,16) = (/ &
        & 0.36664e+04_rb,0.32577e+04_rb,0.29162e+04_rb,0.26350e+04_rb,0.24040e+04_rb /)
      kbo(:,57,16) = (/ &
        & 0.38139e+04_rb,0.33808e+04_rb,0.30193e+04_rb,0.27188e+04_rb,0.24736e+04_rb /)
      kbo(:,58,16) = (/ &
        & 0.39629e+04_rb,0.35038e+04_rb,0.31229e+04_rb,0.28034e+04_rb,0.25431e+04_rb /)
      kbo(:,59,16) = (/ &
        & 0.40252e+04_rb,0.35562e+04_rb,0.31659e+04_rb,0.28393e+04_rb,0.25724e+04_rb /)

!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.525585e-05_rb, 0.527618e-05_rb, 0.746929e-04_rb /)
      forrefo(:, 2) = (/ 0.794660e-05_rb, 0.136902e-04_rb, 0.849878e-04_rb /)
      forrefo(:, 3) = (/ 0.197099e-04_rb, 0.733094e-04_rb, 0.121687e-03_rb /)
      forrefo(:, 4) = (/ 0.148274e-03_rb, 0.169776e-03_rb, 0.164848e-03_rb /)
      forrefo(:, 5) = (/ 0.230296e-03_rb, 0.210384e-03_rb, 0.182028e-03_rb /)
      forrefo(:, 6) = (/ 0.280575e-03_rb, 0.259217e-03_rb, 0.196080e-03_rb /)
      forrefo(:, 7) = (/ 0.329034e-03_rb, 0.291575e-03_rb, 0.207044e-03_rb /)
      forrefo(:, 8) = (/ 0.349989e-03_rb, 0.323471e-03_rb, 0.225712e-03_rb /)
      forrefo(:, 9) = (/ 0.366097e-03_rb, 0.321519e-03_rb, 0.253150e-03_rb /)
      forrefo(:,10) = (/ 0.383589e-03_rb, 0.355314e-03_rb, 0.262555e-03_rb /)
      forrefo(:,11) = (/ 0.375933e-03_rb, 0.372443e-03_rb, 0.261313e-03_rb /)
      forrefo(:,12) = (/ 0.370652e-03_rb, 0.382366e-03_rb, 0.250070e-03_rb /)
      forrefo(:,13) = (/ 0.375092e-03_rb, 0.379542e-03_rb, 0.265794e-03_rb /)
      forrefo(:,14) = (/ 0.389705e-03_rb, 0.384274e-03_rb, 0.322135e-03_rb /)
      forrefo(:,15) = (/ 0.372084e-03_rb, 0.390422e-03_rb, 0.370035e-03_rb /)
      forrefo(:,16) = (/ 0.437802e-03_rb, 0.373406e-03_rb, 0.373222e-03_rb /)
      
!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).
!     -----------------------------------------------------------------
      
      selfrefo(:, 1) = (/ &
        &  0.126758e-02_rb, 0.105253e-02_rb, 0.873963e-03_rb, 0.725690e-03_rb, 0.602573e-03_rb, &
        &  0.500344e-03_rb, 0.415458e-03_rb, 0.344973e-03_rb, 0.286447e-03_rb, 0.237849e-03_rb /)
      selfrefo(:, 2) = (/ &
        &  0.144006e-02_rb, 0.118514e-02_rb, 0.975351e-03_rb, 0.802697e-03_rb, 0.660606e-03_rb, &
        &  0.543667e-03_rb, 0.447429e-03_rb, 0.368226e-03_rb, 0.303044e-03_rb, 0.249400e-03_rb /)
      selfrefo(:, 3) = (/ &
        &  0.294018e-02_rb, 0.227428e-02_rb, 0.175920e-02_rb, 0.136077e-02_rb, 0.105258e-02_rb, &
        &  0.814189e-03_rb, 0.629789e-03_rb, 0.487153e-03_rb, 0.376821e-03_rb, 0.291478e-03_rb /)
      selfrefo(:, 4) = (/ &
        &  0.395290e-02_rb, 0.348405e-02_rb, 0.307081e-02_rb, 0.270658e-02_rb, 0.238556e-02_rb, &
        &  0.210261e-02_rb, 0.185322e-02_rb, 0.163341e-02_rb, 0.143967e-02_rb, 0.126891e-02_rb /)
      selfrefo(:, 5) = (/ &
        &  0.419122e-02_rb, 0.385638e-02_rb, 0.354829e-02_rb, 0.326481e-02_rb, 0.300398e-02_rb, &
        &  0.276399e-02_rb, 0.254317e-02_rb, 0.234000e-02_rb, 0.215305e-02_rb, 0.198104e-02_rb /)
      selfrefo(:, 6) = (/ &
        &  0.495659e-02_rb, 0.456777e-02_rb, 0.420945e-02_rb, 0.387924e-02_rb, 0.357494e-02_rb, &
        &  0.329450e-02_rb, 0.303606e-02_rb, 0.279790e-02_rb, 0.257842e-02_rb, 0.237615e-02_rb /)
      selfrefo(:, 7) = (/ &
        &  0.526981e-02_rb, 0.490687e-02_rb, 0.456893e-02_rb, 0.425426e-02_rb, 0.396126e-02_rb, &
        &  0.368844e-02_rb, 0.343441e-02_rb, 0.319788e-02_rb, 0.297764e-02_rb, 0.277256e-02_rb /)
      selfrefo(:, 8) = (/ &
        &  0.575426e-02_rb, 0.531597e-02_rb, 0.491106e-02_rb, 0.453699e-02_rb, 0.419141e-02_rb, &
        &  0.387216e-02_rb, 0.357722e-02_rb, 0.330475e-02_rb, 0.305303e-02_rb, 0.282048e-02_rb /)
      selfrefo(:, 9) = (/ &
        &  0.549881e-02_rb, 0.514328e-02_rb, 0.481074e-02_rb, 0.449970e-02_rb, 0.420877e-02_rb, &
        &  0.393665e-02_rb, 0.368213e-02_rb, 0.344406e-02_rb, 0.322138e-02_rb, 0.301310e-02_rb /)
      selfrefo(:,10) = (/ &
        &  0.605357e-02_rb, 0.561246e-02_rb, 0.520349e-02_rb, 0.482432e-02_rb, 0.447278e-02_rb, &
        &  0.414686e-02_rb, 0.384469e-02_rb, 0.356453e-02_rb, 0.330479e-02_rb, 0.306398e-02_rb /)
      selfrefo(:,11) = (/ &
        &  0.640504e-02_rb, 0.587858e-02_rb, 0.539540e-02_rb, 0.495194e-02_rb, 0.454492e-02_rb, &
        &  0.417136e-02_rb, 0.382850e-02_rb, 0.351382e-02_rb, 0.322501e-02_rb, 0.295993e-02_rb /)
      selfrefo(:,12) = (/ &
        &  0.677803e-02_rb, 0.615625e-02_rb, 0.559152e-02_rb, 0.507859e-02_rb, 0.461271e-02_rb, &
        &  0.418957e-02_rb, 0.380524e-02_rb, 0.345617e-02_rb, 0.313913e-02_rb, 0.285116e-02_rb /)
      selfrefo(:,13) = (/ &
        &  0.690347e-02_rb, 0.627003e-02_rb, 0.569472e-02_rb, 0.517219e-02_rb, 0.469761e-02_rb, &
        &  0.426658e-02_rb, 0.387509e-02_rb, 0.351953e-02_rb, 0.319659e-02_rb, 0.290328e-02_rb /)
      selfrefo(:,14) = (/ &
        &  0.692680e-02_rb, 0.632795e-02_rb, 0.578087e-02_rb, 0.528109e-02_rb, 0.482452e-02_rb, &
        &  0.440742e-02_rb, 0.402638e-02_rb, 0.367828e-02_rb, 0.336028e-02_rb, 0.306977e-02_rb /)
      selfrefo(:,15) = (/ &
        &  0.754894e-02_rb, 0.681481e-02_rb, 0.615207e-02_rb, 0.555378e-02_rb, 0.501367e-02_rb, &
        &  0.452609e-02_rb, 0.408593e-02_rb, 0.368857e-02_rb, 0.332986e-02_rb, 0.300603e-02_rb /)
      selfrefo(:,16) = (/ &
        &  0.760689e-02_rb, 0.709755e-02_rb, 0.662232e-02_rb, 0.617891e-02_rb, 0.576519e-02_rb, &
        &  0.537917e-02_rb, 0.501899e-02_rb, 0.468293e-02_rb, 0.436938e-02_rb, 0.407682e-02_rb /)
     
      end subroutine sw_kgb16

! **************************************************************************
      subroutine sw_kgb17
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg17, only : kao, kbo, selfrefo, forrefo, sfluxrefo, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:,1) = (/ &
        & 3.15613_rb  ,  3.03449_rb  ,  2.92069_rb  ,  2.63874_rb   , &
        & 2.34581_rb  ,  2.06999_rb  ,  1.70906_rb  ,  1.29085_rb   , &
        & 0.874851_rb ,  0.0955392_rb,  0.0787813_rb,  0.0621951_rb , &
        & 0.0459076_rb,  0.0294129_rb,  0.0110387_rb,  0.00159668_rb /)
      sfluxrefo(:,2) = (/ &
        & 2.83147_rb  ,  2.95919_rb  ,  2.96674_rb  ,  2.77677_rb   , &
        & 2.46826_rb  ,  2.11481_rb  ,  1.73243_rb  ,  1.30279_rb   , &
        & 0.882714_rb ,  0.0962350_rb,  0.0802122_rb,  0.0636194_rb , &
        & 0.0472620_rb,  0.0299051_rb,  0.0110785_rb,  0.00159668_rb /)
      sfluxrefo(:,3) = (/ &
        & 2.82300_rb  ,  2.94845_rb  ,  2.95887_rb  ,  2.77593_rb   , &
        & 2.47096_rb  ,  2.12596_rb  ,  1.73847_rb  ,  1.30796_rb   , &
        & 0.884395_rb ,  0.0966936_rb,  0.0801996_rb,  0.0640199_rb , &
        & 0.0472803_rb,  0.0300515_rb,  0.0112366_rb,  0.00160814_rb /)
      sfluxrefo(:,4) = (/ &
        & 2.81715_rb  ,  2.93789_rb  ,  2.95091_rb  ,  2.77046_rb   , &
        & 2.47716_rb  ,  2.13591_rb  ,  1.74365_rb  ,  1.31277_rb   , &
        & 0.887443_rb ,  0.0967016_rb,  0.0803391_rb,  0.0642442_rb , &
        & 0.0472909_rb,  0.0300720_rb,  0.0114817_rb,  0.00161875_rb /)
      sfluxrefo(:,5) = (/ &
        & 2.82335_rb  ,  2.93168_rb  ,  2.91455_rb  ,  2.75213_rb   , &
        & 2.49168_rb  ,  2.14408_rb  ,  1.75726_rb  ,  1.32401_rb   , &
        & 0.893644_rb ,  0.0969523_rb,  0.0805197_rb,  0.0639936_rb , &
        & 0.0475099_rb,  0.0305667_rb,  0.0115372_rb,  0.00161875_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:,1) = (/ &
        & 2.79591E+00_rb, 3.18865E+00_rb, 2.92382E+00_rb, 2.65986E+00_rb,&
        & 2.40905E+00_rb, 2.06964E+00_rb, 1.69956E+00_rb, 1.28185E+00_rb,&
        & 8.67100E-01_rb, 9.46528E-02_rb, 7.80620E-02_rb, 6.16180E-02_rb,&
        & 4.54865E-02_rb, 2.91509E-02_rb, 1.09383E-02_rb, 1.58251E-03_rb/)
      irradnceo(:,2) = (/ &
        & 2.77708E+00_rb, 2.95925E+00_rb, 2.94559E+00_rb, 2.75511E+00_rb,&
        & 2.45803E+00_rb, 2.10375E+00_rb, 1.72176E+00_rb, 1.29302E+00_rb,&
        & 8.76001E-01_rb, 9.54401E-02_rb, 7.95767E-02_rb, 6.31183E-02_rb,&
        & 4.69123E-02_rb, 2.97079E-02_rb, 1.09954E-02_rb, 1.58251E-03_rb/)
      irradnceo(:,3) = (/ &
        & 2.77741E+00_rb, 2.93130E+00_rb, 2.94627E+00_rb, 2.75484E+00_rb,&
        & 2.46147E+00_rb, 2.11367E+00_rb, 1.72658E+00_rb, 1.29863E+00_rb,&
        & 8.78164E-01_rb, 9.59610E-02_rb, 7.95586E-02_rb, 6.35225E-02_rb,&
        & 4.69369E-02_rb, 2.98564E-02_rb, 1.11570E-02_rb, 1.60281E-03_rb/)
      irradnceo(:,4) = (/ &
        & 2.77368E+00_rb, 2.92756E+00_rb, 2.92789E+00_rb, 2.75419E+00_rb,&
        & 2.46434E+00_rb, 2.12314E+00_rb, 1.73162E+00_rb, 1.30392E+00_rb,&
        & 8.81275E-01_rb, 9.59660E-02_rb, 7.97631E-02_rb, 6.37496E-02_rb,&
        & 4.69225E-02_rb, 2.98658E-02_rb, 1.14339E-02_rb, 1.61722E-03_rb/)
      irradnceo(:,5) = (/ &
        & 2.72669E+00_rb, 2.95334E+00_rb, 2.90710E+00_rb, 2.72348E+00_rb,&
        & 2.46272E+00_rb, 2.14210E+00_rb, 1.75420E+00_rb, 1.32342E+00_rb,&
        & 8.93286E-01_rb, 9.64446E-02_rb, 7.99799E-02_rb, 6.35217E-02_rb,&
        & 4.71758E-02_rb, 3.03764E-02_rb, 1.14846E-02_rb, 1.61722E-03_rb/)
! Solar variability components: facular brightening
      facbrghto(:,1) = (/ &
        & 7.37162E-04_rb, 8.45480E-04_rb, 7.67194E-04_rb, 6.90215E-04_rb,&
        & 6.16469E-04_rb, 5.16057E-04_rb, 4.16194E-04_rb, 3.12251E-04_rb,&
        & 2.11532E-04_rb, 2.31175E-05_rb, 1.90961E-05_rb, 1.50919E-05_rb,&
        & 1.11020E-05_rb, 7.09120E-06_rb, 2.65743E-06_rb, 3.79238E-07_rb/)
      facbrghto(:,2) = (/ &
        & 7.27524E-04_rb, 7.76083E-04_rb, 7.69447E-04_rb, 7.12958E-04_rb,&
        & 6.27611E-04_rb, 5.27508E-04_rb, 4.28073E-04_rb, 3.21647E-04_rb,&
        & 2.18546E-04_rb, 2.38111E-05_rb, 1.98729E-05_rb, 1.57620E-05_rb,&
        & 1.17378E-05_rb, 7.44205E-06_rb, 2.68799E-06_rb, 3.79238E-07_rb/)
      facbrghto(:,3) = (/ &
        & 7.26574E-04_rb, 7.65317E-04_rb, 7.69005E-04_rb, 7.11671E-04_rb,&
        & 6.27091E-04_rb, 5.31836E-04_rb, 4.31668E-04_rb, 3.25073E-04_rb,&
        & 2.20382E-04_rb, 2.41326E-05_rb, 1.99280E-05_rb, 1.59124E-05_rb,&
        & 1.18105E-05_rb, 7.50852E-06_rb, 2.78405E-06_rb, 3.97225E-07_rb/)
      facbrghto(:,4) = (/ &
        & 7.24383E-04_rb, 7.62991E-04_rb, 7.60300E-04_rb, 7.11160E-04_rb,&
        & 6.26033E-04_rb, 5.36734E-04_rb, 4.35756E-04_rb, 3.28281E-04_rb,&
        & 2.22423E-04_rb, 2.42305E-05_rb, 2.00446E-05_rb, 1.60824E-05_rb,&
        & 1.18364E-05_rb, 7.51169E-06_rb, 2.90306E-06_rb, 4.20266E-07_rb/)
      facbrghto(:,5) = (/ &
        & 7.03475E-04_rb, 7.58556E-04_rb, 7.44351E-04_rb, 6.99560E-04_rb,&
        & 6.32207E-04_rb, 5.51203E-04_rb, 4.50246E-04_rb, 3.38818E-04_rb,&
        & 2.28227E-04_rb, 2.46893E-05_rb, 2.04362E-05_rb, 1.61415E-05_rb,&
        & 1.20336E-05_rb, 7.77588E-06_rb, 2.94915E-06_rb, 4.20266E-07_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:,1) = (/ &
        &-3.24054E-04_rb,-3.99474E-04_rb,-3.58912E-04_rb,-3.22600E-04_rb,&
        &-2.92158E-04_rb,-2.47030E-04_rb,-2.01829E-04_rb,-1.52036E-04_rb,&
        &-1.02864E-04_rb,-1.12613E-05_rb,-9.28045E-06_rb,-7.31794E-06_rb,&
        &-5.39894E-06_rb,-3.46541E-06_rb,-1.30065E-06_rb,-1.90031E-07_rb/)
      snsptdrko(:,2) = (/ &
        &-3.22115E-04_rb,-3.59260E-04_rb,-3.62458E-04_rb,-3.37747E-04_rb,&
        &-2.98412E-04_rb,-2.52991E-04_rb,-2.06518E-04_rb,-1.55002E-04_rb,&
        &-1.05226E-04_rb,-1.14752E-05_rb,-9.59026E-06_rb,-7.60979E-06_rb,&
        &-5.68261E-06_rb,-3.58411E-06_rb,-1.30966E-06_rb,-1.90031E-07_rb/)
      snsptdrko(:,3) = (/ &
        &-3.22053E-04_rb,-3.53947E-04_rb,-3.62640E-04_rb,-3.37389E-04_rb,&
        &-2.98660E-04_rb,-2.54979E-04_rb,-2.07734E-04_rb,-1.56252E-04_rb,&
        &-1.05776E-04_rb,-1.16059E-05_rb,-9.59351E-06_rb,-7.68862E-06_rb,&
        &-5.69910E-06_rb,-3.61670E-06_rb,-1.34541E-06_rb,-1.92892E-07_rb/)
      snsptdrko(:,4) = (/ &
        &-3.21064E-04_rb,-3.53190E-04_rb,-3.58824E-04_rb,-3.37435E-04_rb,&
        &-2.98905E-04_rb,-2.56847E-04_rb,-2.09071E-04_rb,-1.57452E-04_rb,&
        &-1.06456E-04_rb,-1.16091E-05_rb,-9.65088E-06_rb,-7.74682E-06_rb,&
        &-5.70841E-06_rb,-3.62191E-06_rb,-1.39278E-06_rb,-1.97310E-07_rb/)
      snsptdrko(:,5) = (/ &
        &-3.09963E-04_rb,-3.54901E-04_rb,-3.52734E-04_rb,-3.31642E-04_rb,&
        &-3.00605E-04_rb,-2.62569E-04_rb,-2.15107E-04_rb,-1.62129E-04_rb,&
        &-1.09184E-04_rb,-1.17376E-05_rb,-9.75778E-06_rb,-7.73775E-06_rb,&
        &-5.77322E-06_rb,-3.72866E-06_rb,-1.40404E-06_rb,-1.97310E-07_rb/)

! Rayleigh extinction coefficient at v = 3625 cm-1.
      rayl = 6.86e-10_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.11134e-05_rb,0.32092e-03_rb,0.55663e-03_rb,0.78227e-03_rb,0.99421e-03_rb, &
        & 0.12045e-02_rb,0.14116e-02_rb,0.16069e-02_rb,0.19330e-02_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.10544e-05_rb,0.35434e-03_rb,0.61480e-03_rb,0.86424e-03_rb,0.11055e-02_rb, &
        & 0.13431e-02_rb,0.15754e-02_rb,0.17951e-02_rb,0.21280e-02_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.98345e-06_rb,0.38989e-03_rb,0.67747e-03_rb,0.95289e-03_rb,0.12196e-02_rb, &
        & 0.14864e-02_rb,0.17449e-02_rb,0.19897e-02_rb,0.23564e-02_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.95017e-06_rb,0.42582e-03_rb,0.74419e-03_rb,0.10463e-02_rb,0.13424e-02_rb, &
        & 0.16353e-02_rb,0.19215e-02_rb,0.21939e-02_rb,0.25993e-02_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.91870e-06_rb,0.46371e-03_rb,0.81394e-03_rb,0.11464e-02_rb,0.14715e-02_rb, &
        & 0.17937e-02_rb,0.21072e-02_rb,0.24077e-02_rb,0.28540e-02_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.94184e-06_rb,0.26532e-03_rb,0.46157e-03_rb,0.63776e-03_rb,0.81087e-03_rb, &
        & 0.97728e-03_rb,0.11419e-02_rb,0.12991e-02_rb,0.15121e-02_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.89060e-06_rb,0.29417e-03_rb,0.50967e-03_rb,0.70759e-03_rb,0.90203e-03_rb, &
        & 0.10920e-02_rb,0.12764e-02_rb,0.14544e-02_rb,0.16795e-02_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.82720e-06_rb,0.32361e-03_rb,0.56124e-03_rb,0.78299e-03_rb,0.99718e-03_rb, &
        & 0.12094e-02_rb,0.14172e-02_rb,0.16165e-02_rb,0.18637e-02_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.78842e-06_rb,0.35495e-03_rb,0.61684e-03_rb,0.86176e-03_rb,0.10995e-02_rb, &
        & 0.13345e-02_rb,0.15652e-02_rb,0.17856e-02_rb,0.20621e-02_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.76610e-06_rb,0.38728e-03_rb,0.67538e-03_rb,0.94613e-03_rb,0.12082e-02_rb, &
        & 0.14675e-02_rb,0.17206e-02_rb,0.19640e-02_rb,0.22720e-02_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.79663e-06_rb,0.21369e-03_rb,0.37088e-03_rb,0.51222e-03_rb,0.64549e-03_rb, &
        & 0.77322e-03_rb,0.89939e-03_rb,0.10205e-02_rb,0.11568e-02_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.75955e-06_rb,0.23780e-03_rb,0.41153e-03_rb,0.56901e-03_rb,0.72009e-03_rb, &
        & 0.86735e-03_rb,0.10092e-02_rb,0.11476e-02_rb,0.12913e-02_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.71692e-06_rb,0.26276e-03_rb,0.45488e-03_rb,0.62922e-03_rb,0.79836e-03_rb, &
        & 0.96259e-03_rb,0.11236e-02_rb,0.12796e-02_rb,0.14398e-02_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.66348e-06_rb,0.28896e-03_rb,0.50221e-03_rb,0.69411e-03_rb,0.88226e-03_rb, &
        & 0.10658e-02_rb,0.12456e-02_rb,0.14191e-02_rb,0.15991e-02_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.64013e-06_rb,0.31643e-03_rb,0.55166e-03_rb,0.76434e-03_rb,0.97216e-03_rb, &
        & 0.11745e-02_rb,0.13731e-02_rb,0.15657e-02_rb,0.17671e-02_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.66217e-06_rb,0.16966e-03_rb,0.29358e-03_rb,0.40688e-03_rb,0.51114e-03_rb, &
        & 0.60739e-03_rb,0.70191e-03_rb,0.79446e-03_rb,0.88832e-03_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.64111e-06_rb,0.18902e-03_rb,0.32742e-03_rb,0.45364e-03_rb,0.57086e-03_rb, &
        & 0.68401e-03_rb,0.79116e-03_rb,0.89507e-03_rb,0.99655e-03_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.60729e-06_rb,0.21007e-03_rb,0.36354e-03_rb,0.50354e-03_rb,0.63459e-03_rb, &
        & 0.76113e-03_rb,0.88473e-03_rb,0.10035e-02_rb,0.11163e-02_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.57250e-06_rb,0.23203e-03_rb,0.40245e-03_rb,0.55758e-03_rb,0.70295e-03_rb, &
        & 0.84515e-03_rb,0.98379e-03_rb,0.11165e-02_rb,0.12444e-02_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.53524e-06_rb,0.25529e-03_rb,0.44399e-03_rb,0.61564e-03_rb,0.77626e-03_rb, &
        & 0.93405e-03_rb,0.10892e-02_rb,0.12369e-02_rb,0.13795e-02_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.55112e-06_rb,0.13422e-03_rb,0.23166e-03_rb,0.32119e-03_rb,0.40265e-03_rb, &
        & 0.47812e-03_rb,0.54797e-03_rb,0.61873e-03_rb,0.68912e-03_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.53669e-06_rb,0.14990e-03_rb,0.25949e-03_rb,0.35886e-03_rb,0.45239e-03_rb, &
        & 0.53936e-03_rb,0.61894e-03_rb,0.69718e-03_rb,0.77732e-03_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.51495e-06_rb,0.16706e-03_rb,0.28960e-03_rb,0.40020e-03_rb,0.50461e-03_rb, &
        & 0.60249e-03_rb,0.69677e-03_rb,0.78464e-03_rb,0.87334e-03_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.48633e-06_rb,0.18543e-03_rb,0.32159e-03_rb,0.44499e-03_rb,0.56095e-03_rb, &
        & 0.67055e-03_rb,0.77575e-03_rb,0.87717e-03_rb,0.97694e-03_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.45162e-06_rb,0.20472e-03_rb,0.35591e-03_rb,0.49350e-03_rb,0.62163e-03_rb, &
        & 0.74253e-03_rb,0.86247e-03_rb,0.97518e-03_rb,0.10864e-02_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.45055e-06_rb,0.10556e-03_rb,0.18176e-03_rb,0.25137e-03_rb,0.31438e-03_rb, &
        & 0.37363e-03_rb,0.42759e-03_rb,0.47883e-03_rb,0.53437e-03_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.44502e-06_rb,0.11814e-03_rb,0.20383e-03_rb,0.28162e-03_rb,0.35547e-03_rb, &
        & 0.42261e-03_rb,0.48340e-03_rb,0.54066e-03_rb,0.60352e-03_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.43066e-06_rb,0.13217e-03_rb,0.22836e-03_rb,0.31552e-03_rb,0.39762e-03_rb, &
        & 0.47493e-03_rb,0.54543e-03_rb,0.61066e-03_rb,0.68176e-03_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.41116e-06_rb,0.14733e-03_rb,0.25472e-03_rb,0.35204e-03_rb,0.44334e-03_rb, &
        & 0.52997e-03_rb,0.60966e-03_rb,0.68622e-03_rb,0.76607e-03_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.38748e-06_rb,0.16344e-03_rb,0.28281e-03_rb,0.39173e-03_rb,0.49305e-03_rb, &
        & 0.58880e-03_rb,0.67954e-03_rb,0.76485e-03_rb,0.85401e-03_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.36582e-06_rb,0.82958e-04_rb,0.14210e-03_rb,0.19574e-03_rb,0.24440e-03_rb, &
        & 0.29039e-03_rb,0.33281e-03_rb,0.37177e-03_rb,0.42178e-03_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.36707e-06_rb,0.92909e-04_rb,0.15977e-03_rb,0.22034e-03_rb,0.27742e-03_rb, &
        & 0.32955e-03_rb,0.37727e-03_rb,0.41956e-03_rb,0.47859e-03_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.35787e-06_rb,0.10432e-03_rb,0.17930e-03_rb,0.24769e-03_rb,0.31155e-03_rb, &
        & 0.37242e-03_rb,0.42699e-03_rb,0.47474e-03_rb,0.54083e-03_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.34482e-06_rb,0.11685e-03_rb,0.20055e-03_rb,0.27725e-03_rb,0.34866e-03_rb, &
        & 0.41677e-03_rb,0.47903e-03_rb,0.53518e-03_rb,0.60812e-03_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.32598e-06_rb,0.13001e-03_rb,0.22341e-03_rb,0.30900e-03_rb,0.38919e-03_rb, &
        & 0.46465e-03_rb,0.53467e-03_rb,0.59926e-03_rb,0.67733e-03_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.29834e-06_rb,0.65009e-04_rb,0.11183e-03_rb,0.15200e-03_rb,0.18969e-03_rb, &
        & 0.22517e-03_rb,0.25832e-03_rb,0.28730e-03_rb,0.33955e-03_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.29745e-06_rb,0.72872e-04_rb,0.12482e-03_rb,0.17221e-03_rb,0.21583e-03_rb, &
        & 0.25596e-03_rb,0.29328e-03_rb,0.32520e-03_rb,0.38620e-03_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.29421e-06_rb,0.82036e-04_rb,0.14010e-03_rb,0.19344e-03_rb,0.24362e-03_rb, &
        & 0.29071e-03_rb,0.33308e-03_rb,0.36867e-03_rb,0.43901e-03_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.28596e-06_rb,0.92256e-04_rb,0.15738e-03_rb,0.21752e-03_rb,0.27325e-03_rb, &
        & 0.32643e-03_rb,0.37578e-03_rb,0.41710e-03_rb,0.49358e-03_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.27296e-06_rb,0.10320e-03_rb,0.17598e-03_rb,0.24339e-03_rb,0.30591e-03_rb, &
        & 0.36505e-03_rb,0.42061e-03_rb,0.46872e-03_rb,0.54993e-03_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.24259e-06_rb,0.50626e-04_rb,0.86945e-04_rb,0.11778e-03_rb,0.14692e-03_rb, &
        & 0.17431e-03_rb,0.20057e-03_rb,0.22131e-03_rb,0.29919e-03_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.24058e-06_rb,0.56966e-04_rb,0.97467e-04_rb,0.13405e-03_rb,0.16750e-03_rb, &
        & 0.19814e-03_rb,0.22739e-03_rb,0.25281e-03_rb,0.34022e-03_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.24091e-06_rb,0.64324e-04_rb,0.10971e-03_rb,0.15061e-03_rb,0.18996e-03_rb, &
        & 0.22580e-03_rb,0.25885e-03_rb,0.28694e-03_rb,0.38045e-03_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.23550e-06_rb,0.72451e-04_rb,0.12345e-03_rb,0.16991e-03_rb,0.21362e-03_rb, &
        & 0.25484e-03_rb,0.29341e-03_rb,0.32523e-03_rb,0.42836e-03_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.22696e-06_rb,0.81351e-04_rb,0.13839e-03_rb,0.19090e-03_rb,0.23986e-03_rb, &
        & 0.28608e-03_rb,0.32910e-03_rb,0.36639e-03_rb,0.47925e-03_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.19679e-06_rb,0.39760e-04_rb,0.68052e-04_rb,0.92295e-04_rb,0.11498e-03_rb, &
        & 0.13656e-03_rb,0.15604e-03_rb,0.17171e-03_rb,0.29679e-03_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.19567e-06_rb,0.44847e-04_rb,0.76829e-04_rb,0.10518e-03_rb,0.13102e-03_rb, &
        & 0.15509e-03_rb,0.17764e-03_rb,0.19843e-03_rb,0.32863e-03_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.19574e-06_rb,0.50565e-04_rb,0.86466e-04_rb,0.11878e-03_rb,0.14920e-03_rb, &
        & 0.17698e-03_rb,0.20258e-03_rb,0.22526e-03_rb,0.36959e-03_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.19262e-06_rb,0.57124e-04_rb,0.97546e-04_rb,0.13398e-03_rb,0.16803e-03_rb, &
        & 0.20046e-03_rb,0.23022e-03_rb,0.25578e-03_rb,0.41543e-03_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.18747e-06_rb,0.64264e-04_rb,0.10971e-03_rb,0.15086e-03_rb,0.18923e-03_rb, &
        & 0.22565e-03_rb,0.25934e-03_rb,0.28921e-03_rb,0.46463e-03_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.16015e-06_rb,0.33025e-04_rb,0.56562e-04_rb,0.76625e-04_rb,0.95645e-04_rb, &
        & 0.11358e-03_rb,0.12955e-03_rb,0.14269e-03_rb,0.27545e-03_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.15918e-06_rb,0.37148e-04_rb,0.63775e-04_rb,0.87600e-04_rb,0.10897e-03_rb, &
        & 0.12893e-03_rb,0.14756e-03_rb,0.16465e-03_rb,0.30799e-03_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.15897e-06_rb,0.41993e-04_rb,0.72040e-04_rb,0.98928e-04_rb,0.12433e-03_rb, &
        & 0.14733e-03_rb,0.16824e-03_rb,0.18694e-03_rb,0.34340e-03_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.15578e-06_rb,0.47441e-04_rb,0.81426e-04_rb,0.11165e-03_rb,0.13985e-03_rb, &
        & 0.16673e-03_rb,0.19156e-03_rb,0.21268e-03_rb,0.38176e-03_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.15210e-06_rb,0.53398e-04_rb,0.91657e-04_rb,0.12592e-03_rb,0.15769e-03_rb, &
        & 0.18781e-03_rb,0.21569e-03_rb,0.24069e-03_rb,0.42228e-03_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.13021e-06_rb,0.27272e-04_rb,0.46832e-04_rb,0.63511e-04_rb,0.79205e-04_rb, &
        & 0.93949e-04_rb,0.10719e-03_rb,0.11827e-03_rb,0.23944e-03_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.12930e-06_rb,0.30686e-04_rb,0.52925e-04_rb,0.72782e-04_rb,0.90527e-04_rb, &
        & 0.10708e-03_rb,0.12236e-03_rb,0.13620e-03_rb,0.27121e-03_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.12900e-06_rb,0.34723e-04_rb,0.59867e-04_rb,0.82205e-04_rb,0.10326e-03_rb, &
        & 0.12238e-03_rb,0.13958e-03_rb,0.15498e-03_rb,0.29930e-03_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.12648e-06_rb,0.39252e-04_rb,0.67618e-04_rb,0.92949e-04_rb,0.11636e-03_rb, &
        & 0.13835e-03_rb,0.15924e-03_rb,0.17653e-03_rb,0.33050e-03_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.12299e-06_rb,0.44213e-04_rb,0.76244e-04_rb,0.10488e-03_rb,0.13135e-03_rb, &
        & 0.15601e-03_rb,0.17902e-03_rb,0.19976e-03_rb,0.36579e-03_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.10554e-06_rb,0.22388e-04_rb,0.38597e-04_rb,0.52378e-04_rb,0.65404e-04_rb, &
        & 0.77567e-04_rb,0.88543e-04_rb,0.97801e-04_rb,0.19599e-03_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.10480e-06_rb,0.25181e-04_rb,0.43670e-04_rb,0.60205e-04_rb,0.74883e-04_rb, &
        & 0.88605e-04_rb,0.10117e-03_rb,0.11242e-03_rb,0.22146e-03_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.10429e-06_rb,0.28530e-04_rb,0.49422e-04_rb,0.68039e-04_rb,0.85435e-04_rb, &
        & 0.10136e-03_rb,0.11557e-03_rb,0.12828e-03_rb,0.24447e-03_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.10232e-06_rb,0.32301e-04_rb,0.55797e-04_rb,0.76948e-04_rb,0.96447e-04_rb, &
        & 0.11462e-03_rb,0.13166e-03_rb,0.14616e-03_rb,0.26999e-03_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.99014e-07_rb,0.36394e-04_rb,0.62994e-04_rb,0.86946e-04_rb,0.10900e-03_rb, &
        & 0.12937e-03_rb,0.14833e-03_rb,0.16540e-03_rb,0.29916e-03_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.23488e-04_rb,0.16684e-02_rb,0.29575e-02_rb,0.40083e-02_rb,0.49953e-02_rb, &
        & 0.59021e-02_rb,0.67446e-02_rb,0.76008e-02_rb,0.87032e-02_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.25156e-04_rb,0.18462e-02_rb,0.32711e-02_rb,0.45008e-02_rb,0.56182e-02_rb, &
        & 0.66287e-02_rb,0.75909e-02_rb,0.85558e-02_rb,0.97543e-02_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.27084e-04_rb,0.20325e-02_rb,0.36034e-02_rb,0.49992e-02_rb,0.62796e-02_rb, &
        & 0.74134e-02_rb,0.84994e-02_rb,0.95694e-02_rb,0.10847e-01_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.28147e-04_rb,0.22347e-02_rb,0.39628e-02_rb,0.55005e-02_rb,0.69453e-02_rb, &
        & 0.82429e-02_rb,0.94699e-02_rb,0.10660e-01_rb,0.11991e-01_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.28719e-04_rb,0.24475e-02_rb,0.43369e-02_rb,0.60360e-02_rb,0.76183e-02_rb, &
        & 0.90878e-02_rb,0.10493e-01_rb,0.11805e-01_rb,0.13204e-01_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.20947e-04_rb,0.14015e-02_rb,0.24728e-02_rb,0.33676e-02_rb,0.41801e-02_rb, &
        & 0.49244e-02_rb,0.55787e-02_rb,0.62206e-02_rb,0.69741e-02_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.22352e-04_rb,0.15535e-02_rb,0.27396e-02_rb,0.37870e-02_rb,0.47121e-02_rb, &
        & 0.55571e-02_rb,0.62892e-02_rb,0.70132e-02_rb,0.78092e-02_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.23882e-04_rb,0.17213e-02_rb,0.30345e-02_rb,0.42020e-02_rb,0.52782e-02_rb, &
        & 0.62285e-02_rb,0.70556e-02_rb,0.78819e-02_rb,0.87330e-02_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.24650e-04_rb,0.18954e-02_rb,0.33384e-02_rb,0.46336e-02_rb,0.58333e-02_rb, &
        & 0.69311e-02_rb,0.78751e-02_rb,0.88149e-02_rb,0.96911e-02_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.25136e-04_rb,0.20806e-02_rb,0.36641e-02_rb,0.50966e-02_rb,0.64193e-02_rb, &
        & 0.76281e-02_rb,0.87403e-02_rb,0.97943e-02_rb,0.10704e-01_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.17720e-04_rb,0.11414e-02_rb,0.20045e-02_rb,0.27288e-02_rb,0.33891e-02_rb, &
        & 0.39755e-02_rb,0.44967e-02_rb,0.49389e-02_rb,0.53858e-02_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.19087e-04_rb,0.12745e-02_rb,0.22336e-02_rb,0.30841e-02_rb,0.38333e-02_rb, &
        & 0.44983e-02_rb,0.50895e-02_rb,0.55930e-02_rb,0.60666e-02_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.20125e-04_rb,0.14178e-02_rb,0.24795e-02_rb,0.34278e-02_rb,0.43106e-02_rb, &
        & 0.50607e-02_rb,0.57280e-02_rb,0.63092e-02_rb,0.68285e-02_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.20895e-04_rb,0.15666e-02_rb,0.27348e-02_rb,0.37934e-02_rb,0.47769e-02_rb, &
        & 0.56621e-02_rb,0.64046e-02_rb,0.70864e-02_rb,0.76026e-02_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.21508e-04_rb,0.17251e-02_rb,0.30153e-02_rb,0.41809e-02_rb,0.52689e-02_rb, &
        & 0.62462e-02_rb,0.71125e-02_rb,0.79104e-02_rb,0.84389e-02_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.14661e-04_rb,0.92261e-03_rb,0.16088e-02_rb,0.21788e-02_rb,0.26956e-02_rb, &
        & 0.31612e-02_rb,0.35626e-02_rb,0.38878e-02_rb,0.41525e-02_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.15607e-04_rb,0.10329e-02_rb,0.17973e-02_rb,0.24735e-02_rb,0.30696e-02_rb, &
        & 0.35953e-02_rb,0.40511e-02_rb,0.44162e-02_rb,0.46990e-02_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.16559e-04_rb,0.11550e-02_rb,0.20019e-02_rb,0.27577e-02_rb,0.34625e-02_rb, &
        & 0.40662e-02_rb,0.45831e-02_rb,0.49995e-02_rb,0.53073e-02_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.17263e-04_rb,0.12794e-02_rb,0.22184e-02_rb,0.30614e-02_rb,0.38510e-02_rb, &
        & 0.45664e-02_rb,0.51532e-02_rb,0.56290e-02_rb,0.59436e-02_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.17895e-04_rb,0.14130e-02_rb,0.24526e-02_rb,0.33805e-02_rb,0.42622e-02_rb, &
        & 0.50556e-02_rb,0.57489e-02_rb,0.63094e-02_rb,0.66256e-02_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.11885e-04_rb,0.74888e-03_rb,0.12826e-02_rb,0.17258e-02_rb,0.21346e-02_rb, &
        & 0.24972e-02_rb,0.28046e-02_rb,0.30606e-02_rb,0.32323e-02_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.12452e-04_rb,0.84084e-03_rb,0.14380e-02_rb,0.19698e-02_rb,0.24386e-02_rb, &
        & 0.28536e-02_rb,0.32075e-02_rb,0.34937e-02_rb,0.36562e-02_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.13439e-04_rb,0.94272e-03_rb,0.16065e-02_rb,0.22079e-02_rb,0.27660e-02_rb, &
        & 0.32389e-02_rb,0.36440e-02_rb,0.39673e-02_rb,0.41393e-02_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.14053e-04_rb,0.10473e-02_rb,0.17899e-02_rb,0.24603e-02_rb,0.30861e-02_rb, &
        & 0.36521e-02_rb,0.41156e-02_rb,0.44825e-02_rb,0.46529e-02_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.14586e-04_rb,0.11585e-02_rb,0.19886e-02_rb,0.27257e-02_rb,0.34272e-02_rb, &
        & 0.40599e-02_rb,0.46115e-02_rb,0.50286e-02_rb,0.52085e-02_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.94872e-05_rb,0.59793e-03_rb,0.10210e-02_rb,0.13572e-02_rb,0.16716e-02_rb, &
        & 0.19512e-02_rb,0.21890e-02_rb,0.23827e-02_rb,0.25347e-02_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.10060e-04_rb,0.67400e-03_rb,0.11432e-02_rb,0.15564e-02_rb,0.19188e-02_rb, &
        & 0.22389e-02_rb,0.25161e-02_rb,0.27308e-02_rb,0.28770e-02_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.10743e-04_rb,0.75948e-03_rb,0.12806e-02_rb,0.17551e-02_rb,0.21862e-02_rb, &
        & 0.25545e-02_rb,0.28729e-02_rb,0.31149e-02_rb,0.32716e-02_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.11322e-04_rb,0.85134e-03_rb,0.14320e-02_rb,0.19630e-02_rb,0.24514e-02_rb, &
        & 0.28930e-02_rb,0.32578e-02_rb,0.35375e-02_rb,0.36912e-02_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.11755e-04_rb,0.94611e-03_rb,0.15986e-02_rb,0.21804e-02_rb,0.27291e-02_rb, &
        & 0.32306e-02_rb,0.36695e-02_rb,0.39913e-02_rb,0.41439e-02_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.75931e-05_rb,0.47130e-03_rb,0.81211e-03_rb,0.10658e-02_rb,0.13004e-02_rb, &
        & 0.15175e-02_rb,0.17002e-02_rb,0.18473e-02_rb,0.19861e-02_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.80724e-05_rb,0.53358e-03_rb,0.91163e-03_rb,0.12261e-02_rb,0.15014e-02_rb, &
        & 0.17483e-02_rb,0.19612e-02_rb,0.21201e-02_rb,0.22591e-02_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.84404e-05_rb,0.60517e-03_rb,0.10236e-02_rb,0.13887e-02_rb,0.17204e-02_rb, &
        & 0.20031e-02_rb,0.22497e-02_rb,0.24321e-02_rb,0.25823e-02_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.90511e-05_rb,0.68037e-03_rb,0.11475e-02_rb,0.15596e-02_rb,0.19402e-02_rb, &
        & 0.22789e-02_rb,0.25633e-02_rb,0.27753e-02_rb,0.29247e-02_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.94405e-05_rb,0.76103e-03_rb,0.12836e-02_rb,0.17422e-02_rb,0.21698e-02_rb, &
        & 0.25585e-02_rb,0.28989e-02_rb,0.31444e-02_rb,0.33016e-02_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.58080e-05_rb,0.36782e-03_rb,0.63516e-03_rb,0.83799e-03_rb,0.10119e-02_rb, &
        & 0.11754e-02_rb,0.13161e-02_rb,0.14382e-02_rb,0.15837e-02_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.63156e-05_rb,0.41909e-03_rb,0.71974e-03_rb,0.96952e-03_rb,0.11711e-02_rb, &
        & 0.13617e-02_rb,0.15234e-02_rb,0.16441e-02_rb,0.18118e-02_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.66776e-05_rb,0.47829e-03_rb,0.81150e-03_rb,0.11056e-02_rb,0.13493e-02_rb, &
        & 0.15646e-02_rb,0.17524e-02_rb,0.18937e-02_rb,0.20698e-02_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.70582e-05_rb,0.54017e-03_rb,0.91330e-03_rb,0.12434e-02_rb,0.15411e-02_rb, &
        & 0.17884e-02_rb,0.20057e-02_rb,0.21704e-02_rb,0.23548e-02_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.74902e-05_rb,0.60717e-03_rb,0.10267e-02_rb,0.13908e-02_rb,0.17231e-02_rb, &
        & 0.20285e-02_rb,0.22810e-02_rb,0.24710e-02_rb,0.26631e-02_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.45061e-05_rb,0.28626e-03_rb,0.49448e-03_rb,0.65317e-03_rb,0.79081e-03_rb, &
        & 0.91066e-03_rb,0.10182e-02_rb,0.11107e-02_rb,0.14592e-02_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.49392e-05_rb,0.32734e-03_rb,0.56558e-03_rb,0.76077e-03_rb,0.91846e-03_rb, &
        & 0.10578e-02_rb,0.11776e-02_rb,0.12720e-02_rb,0.16778e-02_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.52580e-05_rb,0.37522e-03_rb,0.63845e-03_rb,0.87805e-03_rb,0.10619e-02_rb, &
        & 0.12194e-02_rb,0.13606e-02_rb,0.14680e-02_rb,0.19259e-02_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.54991e-05_rb,0.42659e-03_rb,0.72103e-03_rb,0.98756e-03_rb,0.12163e-02_rb, &
        & 0.14007e-02_rb,0.15647e-02_rb,0.16900e-02_rb,0.21772e-02_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.58769e-05_rb,0.48199e-03_rb,0.81466e-03_rb,0.11072e-02_rb,0.13684e-02_rb, &
        & 0.15984e-02_rb,0.17887e-02_rb,0.19333e-02_rb,0.24331e-02_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.34154e-05_rb,0.22534e-03_rb,0.38788e-03_rb,0.51113e-03_rb,0.62194e-03_rb, &
        & 0.71432e-03_rb,0.79743e-03_rb,0.86289e-03_rb,0.14758e-02_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.38085e-05_rb,0.25832e-03_rb,0.44748e-03_rb,0.59835e-03_rb,0.72514e-03_rb, &
        & 0.83284e-03_rb,0.91941e-03_rb,0.99583e-03_rb,0.17358e-02_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.41578e-05_rb,0.29655e-03_rb,0.50553e-03_rb,0.69511e-03_rb,0.84107e-03_rb, &
        & 0.96374e-03_rb,0.10666e-02_rb,0.11478e-02_rb,0.19625e-02_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.43917e-05_rb,0.33908e-03_rb,0.57351e-03_rb,0.78640e-03_rb,0.96862e-03_rb, &
        & 0.11109e-02_rb,0.12309e-02_rb,0.13266e-02_rb,0.22103e-02_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.45446e-05_rb,0.38548e-03_rb,0.64883e-03_rb,0.88401e-03_rb,0.10952e-02_rb, &
        & 0.12706e-02_rb,0.14136e-02_rb,0.15239e-02_rb,0.24754e-02_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.27604e-05_rb,0.18848e-03_rb,0.32532e-03_rb,0.42949e-03_rb,0.52350e-03_rb, &
        & 0.60084e-03_rb,0.66640e-03_rb,0.72070e-03_rb,0.13692e-02_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.30908e-05_rb,0.21679e-03_rb,0.37259e-03_rb,0.50366e-03_rb,0.61074e-03_rb, &
        & 0.70158e-03_rb,0.77271e-03_rb,0.83037e-03_rb,0.15689e-02_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.33674e-05_rb,0.24965e-03_rb,0.42244e-03_rb,0.58353e-03_rb,0.70923e-03_rb, &
        & 0.81330e-03_rb,0.89750e-03_rb,0.96131e-03_rb,0.17971e-02_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.35568e-05_rb,0.28602e-03_rb,0.48044e-03_rb,0.65868e-03_rb,0.81816e-03_rb, &
        & 0.93857e-03_rb,0.10357e-02_rb,0.11119e-02_rb,0.20509e-02_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.36921e-05_rb,0.32484e-03_rb,0.54623e-03_rb,0.74475e-03_rb,0.92307e-03_rb, &
        & 0.10758e-02_rb,0.11895e-02_rb,0.12777e-02_rb,0.23253e-02_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.22272e-05_rb,0.15807e-03_rb,0.27163e-03_rb,0.35935e-03_rb,0.43781e-03_rb, &
        & 0.50321e-03_rb,0.55776e-03_rb,0.60046e-03_rb,0.11852e-02_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.24898e-05_rb,0.18201e-03_rb,0.31004e-03_rb,0.42206e-03_rb,0.51226e-03_rb, &
        & 0.58824e-03_rb,0.64919e-03_rb,0.69291e-03_rb,0.13623e-02_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.27021e-05_rb,0.20958e-03_rb,0.35322e-03_rb,0.48676e-03_rb,0.59587e-03_rb, &
        & 0.68364e-03_rb,0.75527e-03_rb,0.80436e-03_rb,0.15654e-02_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.28526e-05_rb,0.24038e-03_rb,0.40262e-03_rb,0.55096e-03_rb,0.68671e-03_rb, &
        & 0.79103e-03_rb,0.87211e-03_rb,0.93124e-03_rb,0.17898e-02_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.29899e-05_rb,0.27289e-03_rb,0.45904e-03_rb,0.62448e-03_rb,0.77496e-03_rb, &
        & 0.90212e-03_rb,0.10018e-02_rb,0.10706e-02_rb,0.20265e-02_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.17576e-05_rb,0.13151e-03_rb,0.22518e-03_rb,0.29890e-03_rb,0.36398e-03_rb, &
        & 0.41884e-03_rb,0.46505e-03_rb,0.50086e-03_rb,0.97916e-03_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.19524e-05_rb,0.15189e-03_rb,0.25686e-03_rb,0.35123e-03_rb,0.42723e-03_rb, &
        & 0.49061e-03_rb,0.54293e-03_rb,0.57815e-03_rb,0.11260e-02_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.21134e-05_rb,0.17558e-03_rb,0.29382e-03_rb,0.40457e-03_rb,0.49777e-03_rb, &
        & 0.57153e-03_rb,0.63185e-03_rb,0.67213e-03_rb,0.12960e-02_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.22681e-05_rb,0.20106e-03_rb,0.33583e-03_rb,0.45977e-03_rb,0.57094e-03_rb, &
        & 0.66275e-03_rb,0.73149e-03_rb,0.77799e-03_rb,0.14856e-02_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.23940e-05_rb,0.22853e-03_rb,0.38323e-03_rb,0.52126e-03_rb,0.64684e-03_rb, &
        & 0.75455e-03_rb,0.84115e-03_rb,0.89575e-03_rb,0.16816e-02_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.87154e-04_rb,0.70456e-02_rb,0.11624e-01_rb,0.16007e-01_rb,0.20277e-01_rb, &
        & 0.24349e-01_rb,0.27902e-01_rb,0.30956e-01_rb,0.33159e-01_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.92678e-04_rb,0.75324e-02_rb,0.12552e-01_rb,0.17444e-01_rb,0.21967e-01_rb, &
        & 0.26250e-01_rb,0.30241e-01_rb,0.33348e-01_rb,0.36164e-01_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.97955e-04_rb,0.80153e-02_rb,0.13502e-01_rb,0.18812e-01_rb,0.23862e-01_rb, &
        & 0.28480e-01_rb,0.32511e-01_rb,0.35713e-01_rb,0.38349e-01_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.10404e-03_rb,0.84724e-02_rb,0.14429e-01_rb,0.20136e-01_rb,0.25672e-01_rb, &
        & 0.30688e-01_rb,0.35011e-01_rb,0.38379e-01_rb,0.40673e-01_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.11110e-03_rb,0.89123e-02_rb,0.15343e-01_rb,0.21479e-01_rb,0.27290e-01_rb, &
        & 0.32751e-01_rb,0.37455e-01_rb,0.40931e-01_rb,0.42463e-01_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.73214e-04_rb,0.61410e-02_rb,0.10125e-01_rb,0.13650e-01_rb,0.17158e-01_rb, &
        & 0.20627e-01_rb,0.23628e-01_rb,0.26251e-01_rb,0.27481e-01_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.77936e-04_rb,0.65751e-02_rb,0.10966e-01_rb,0.14877e-01_rb,0.18693e-01_rb, &
        & 0.22272e-01_rb,0.25630e-01_rb,0.28395e-01_rb,0.29894e-01_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.83061e-04_rb,0.69828e-02_rb,0.11774e-01_rb,0.16068e-01_rb,0.20317e-01_rb, &
        & 0.24244e-01_rb,0.27748e-01_rb,0.30432e-01_rb,0.31851e-01_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.88564e-04_rb,0.73985e-02_rb,0.12581e-01_rb,0.17226e-01_rb,0.21842e-01_rb, &
        & 0.26177e-01_rb,0.29969e-01_rb,0.32748e-01_rb,0.33805e-01_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.94933e-04_rb,0.77988e-02_rb,0.13372e-01_rb,0.18367e-01_rb,0.23323e-01_rb, &
        & 0.27928e-01_rb,0.31964e-01_rb,0.35013e-01_rb,0.35327e-01_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.60947e-04_rb,0.52436e-02_rb,0.86174e-02_rb,0.11520e-01_rb,0.14231e-01_rb, &
        & 0.17027e-01_rb,0.19469e-01_rb,0.21634e-01_rb,0.21981e-01_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.64921e-04_rb,0.55975e-02_rb,0.93512e-02_rb,0.12526e-01_rb,0.15516e-01_rb, &
        & 0.18471e-01_rb,0.21270e-01_rb,0.23512e-01_rb,0.24114e-01_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.69521e-04_rb,0.59648e-02_rb,0.10060e-01_rb,0.13582e-01_rb,0.16903e-01_rb, &
        & 0.20177e-01_rb,0.23055e-01_rb,0.25380e-01_rb,0.25822e-01_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.74348e-04_rb,0.63104e-02_rb,0.10767e-01_rb,0.14599e-01_rb,0.18273e-01_rb, &
        & 0.21836e-01_rb,0.24984e-01_rb,0.27358e-01_rb,0.27450e-01_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.79780e-04_rb,0.66671e-02_rb,0.11458e-01_rb,0.15590e-01_rb,0.19579e-01_rb, &
        & 0.23358e-01_rb,0.26806e-01_rb,0.29335e-01_rb,0.28786e-01_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.50799e-04_rb,0.44148e-02_rb,0.72225e-02_rb,0.96483e-02_rb,0.11767e-01_rb, &
        & 0.13828e-01_rb,0.15844e-01_rb,0.17588e-01_rb,0.17391e-01_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.54227e-04_rb,0.47231e-02_rb,0.78527e-02_rb,0.10493e-01_rb,0.12835e-01_rb, &
        & 0.15126e-01_rb,0.17466e-01_rb,0.19219e-01_rb,0.19351e-01_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.58151e-04_rb,0.50277e-02_rb,0.84823e-02_rb,0.11418e-01_rb,0.13969e-01_rb, &
        & 0.16576e-01_rb,0.18903e-01_rb,0.20840e-01_rb,0.20853e-01_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.62433e-04_rb,0.53372e-02_rb,0.90813e-02_rb,0.12325e-01_rb,0.15163e-01_rb, &
        & 0.17971e-01_rb,0.20590e-01_rb,0.22529e-01_rb,0.22273e-01_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.66852e-04_rb,0.56429e-02_rb,0.96836e-02_rb,0.13189e-01_rb,0.16280e-01_rb, &
        & 0.19312e-01_rb,0.22136e-01_rb,0.24263e-01_rb,0.23466e-01_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.42593e-04_rb,0.36760e-02_rb,0.59913e-02_rb,0.79812e-02_rb,0.97729e-02_rb, &
        & 0.11265e-01_rb,0.12874e-01_rb,0.14150e-01_rb,0.13651e-01_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.45779e-04_rb,0.39344e-02_rb,0.65401e-02_rb,0.87030e-02_rb,0.10650e-01_rb, &
        & 0.12374e-01_rb,0.14135e-01_rb,0.15529e-01_rb,0.15507e-01_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.48868e-04_rb,0.42013e-02_rb,0.70859e-02_rb,0.94912e-02_rb,0.11609e-01_rb, &
        & 0.13557e-01_rb,0.15381e-01_rb,0.16993e-01_rb,0.16785e-01_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.52509e-04_rb,0.44674e-02_rb,0.76022e-02_rb,0.10271e-01_rb,0.12630e-01_rb, &
        & 0.14736e-01_rb,0.16830e-01_rb,0.18387e-01_rb,0.18040e-01_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.56196e-04_rb,0.47381e-02_rb,0.81005e-02_rb,0.11026e-01_rb,0.13586e-01_rb, &
        & 0.15940e-01_rb,0.18160e-01_rb,0.19920e-01_rb,0.19092e-01_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.36112e-04_rb,0.30316e-02_rb,0.49038e-02_rb,0.65325e-02_rb,0.79606e-02_rb, &
        & 0.91706e-02_rb,0.10285e-01_rb,0.11252e-01_rb,0.10556e-01_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.38502e-04_rb,0.32575e-02_rb,0.53855e-02_rb,0.71302e-02_rb,0.87200e-02_rb, &
        & 0.10138e-01_rb,0.11319e-01_rb,0.12504e-01_rb,0.12109e-01_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.41070e-04_rb,0.34869e-02_rb,0.58643e-02_rb,0.78020e-02_rb,0.95232e-02_rb, &
        & 0.11081e-01_rb,0.12413e-01_rb,0.13700e-01_rb,0.13350e-01_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.44034e-04_rb,0.37142e-02_rb,0.62924e-02_rb,0.84867e-02_rb,0.10400e-01_rb, &
        & 0.12091e-01_rb,0.13614e-01_rb,0.14850e-01_rb,0.14396e-01_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.47021e-04_rb,0.39387e-02_rb,0.67171e-02_rb,0.91319e-02_rb,0.11231e-01_rb, &
        & 0.13129e-01_rb,0.14745e-01_rb,0.16155e-01_rb,0.15392e-01_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.30001e-04_rb,0.24884e-02_rb,0.39787e-02_rb,0.53262e-02_rb,0.64254e-02_rb, &
        & 0.74422e-02_rb,0.82418e-02_rb,0.87651e-02_rb,0.83241e-02_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.32067e-04_rb,0.26831e-02_rb,0.43873e-02_rb,0.58009e-02_rb,0.70881e-02_rb, &
        & 0.82303e-02_rb,0.91115e-02_rb,0.99530e-02_rb,0.95843e-02_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.34414e-04_rb,0.28762e-02_rb,0.47981e-02_rb,0.63650e-02_rb,0.77555e-02_rb, &
        & 0.90130e-02_rb,0.10051e-01_rb,0.10944e-01_rb,0.10687e-01_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.36543e-04_rb,0.30749e-02_rb,0.51697e-02_rb,0.69549e-02_rb,0.84939e-02_rb, &
        & 0.98621e-02_rb,0.11012e-01_rb,0.11950e-01_rb,0.11596e-01_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.38986e-04_rb,0.32672e-02_rb,0.55268e-02_rb,0.74950e-02_rb,0.92206e-02_rb, &
        & 0.10750e-01_rb,0.11970e-01_rb,0.13029e-01_rb,0.12461e-01_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.24718e-04_rb,0.20304e-02_rb,0.32253e-02_rb,0.42957e-02_rb,0.51621e-02_rb, &
        & 0.59477e-02_rb,0.65543e-02_rb,0.67901e-02_rb,0.66578e-02_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.26226e-04_rb,0.22027e-02_rb,0.35528e-02_rb,0.46844e-02_rb,0.57463e-02_rb, &
        & 0.66070e-02_rb,0.73566e-02_rb,0.79086e-02_rb,0.76807e-02_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.28095e-04_rb,0.23614e-02_rb,0.38960e-02_rb,0.51529e-02_rb,0.62773e-02_rb, &
        & 0.72919e-02_rb,0.81475e-02_rb,0.87344e-02_rb,0.86509e-02_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.29856e-04_rb,0.25346e-02_rb,0.42180e-02_rb,0.56507e-02_rb,0.68867e-02_rb, &
        & 0.79948e-02_rb,0.89169e-02_rb,0.96226e-02_rb,0.94212e-02_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.31432e-04_rb,0.26989e-02_rb,0.45288e-02_rb,0.61158e-02_rb,0.75231e-02_rb, &
        & 0.87319e-02_rb,0.97447e-02_rb,0.10470e-01_rb,0.10172e-01_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.20151e-04_rb,0.16622e-02_rb,0.26102e-02_rb,0.34345e-02_rb,0.41518e-02_rb, &
        & 0.47356e-02_rb,0.51198e-02_rb,0.52901e-02_rb,0.55608e-02_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.21290e-04_rb,0.18112e-02_rb,0.28622e-02_rb,0.37770e-02_rb,0.46098e-02_rb, &
        & 0.52825e-02_rb,0.58719e-02_rb,0.62115e-02_rb,0.64287e-02_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.22604e-04_rb,0.19483e-02_rb,0.31536e-02_rb,0.41418e-02_rb,0.50513e-02_rb, &
        & 0.58883e-02_rb,0.65199e-02_rb,0.70000e-02_rb,0.71641e-02_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.24039e-04_rb,0.20909e-02_rb,0.34321e-02_rb,0.45633e-02_rb,0.55575e-02_rb, &
        & 0.64453e-02_rb,0.71800e-02_rb,0.77062e-02_rb,0.77821e-02_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.25275e-04_rb,0.22325e-02_rb,0.36976e-02_rb,0.49727e-02_rb,0.60945e-02_rb, &
        & 0.70632e-02_rb,0.78696e-02_rb,0.84048e-02_rb,0.84571e-02_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.16520e-04_rb,0.13579e-02_rb,0.21217e-02_rb,0.27650e-02_rb,0.33209e-02_rb, &
        & 0.37657e-02_rb,0.40263e-02_rb,0.41632e-02_rb,0.59019e-02_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.17307e-04_rb,0.14868e-02_rb,0.23230e-02_rb,0.30616e-02_rb,0.37000e-02_rb, &
        & 0.42629e-02_rb,0.46958e-02_rb,0.48956e-02_rb,0.64729e-02_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.18210e-04_rb,0.16086e-02_rb,0.25635e-02_rb,0.33480e-02_rb,0.40827e-02_rb, &
        & 0.47590e-02_rb,0.52424e-02_rb,0.56088e-02_rb,0.71072e-02_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.19263e-04_rb,0.17308e-02_rb,0.28038e-02_rb,0.37016e-02_rb,0.44933e-02_rb, &
        & 0.52114e-02_rb,0.58224e-02_rb,0.62053e-02_rb,0.76445e-02_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.20450e-04_rb,0.18514e-02_rb,0.30286e-02_rb,0.40569e-02_rb,0.49417e-02_rb, &
        & 0.57292e-02_rb,0.63809e-02_rb,0.68162e-02_rb,0.81800e-02_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.13743e-04_rb,0.11544e-02_rb,0.18005e-02_rb,0.23380e-02_rb,0.28127e-02_rb, &
        & 0.32019e-02_rb,0.34184e-02_rb,0.35137e-02_rb,0.55132e-02_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.14314e-04_rb,0.12648e-02_rb,0.19811e-02_rb,0.25794e-02_rb,0.31283e-02_rb, &
        & 0.35835e-02_rb,0.39635e-02_rb,0.41524e-02_rb,0.60640e-02_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.14980e-04_rb,0.13667e-02_rb,0.21846e-02_rb,0.28316e-02_rb,0.34390e-02_rb, &
        & 0.40069e-02_rb,0.44283e-02_rb,0.47282e-02_rb,0.65662e-02_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.15843e-04_rb,0.14691e-02_rb,0.23844e-02_rb,0.31346e-02_rb,0.37915e-02_rb, &
        & 0.43926e-02_rb,0.49108e-02_rb,0.52431e-02_rb,0.70521e-02_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.16801e-04_rb,0.15709e-02_rb,0.25738e-02_rb,0.34250e-02_rb,0.41781e-02_rb, &
        & 0.48258e-02_rb,0.53776e-02_rb,0.57400e-02_rb,0.74913e-02_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.11265e-04_rb,0.97473e-03_rb,0.15190e-02_rb,0.19779e-02_rb,0.23708e-02_rb, &
        & 0.27002e-02_rb,0.28954e-02_rb,0.29689e-02_rb,0.49052e-02_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.11703e-04_rb,0.10660e-02_rb,0.16806e-02_rb,0.21742e-02_rb,0.26370e-02_rb, &
        & 0.30184e-02_rb,0.33461e-02_rb,0.35081e-02_rb,0.53515e-02_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.12275e-04_rb,0.11524e-02_rb,0.18532e-02_rb,0.24014e-02_rb,0.28932e-02_rb, &
        & 0.33615e-02_rb,0.37332e-02_rb,0.39759e-02_rb,0.57806e-02_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.12963e-04_rb,0.12405e-02_rb,0.20200e-02_rb,0.26573e-02_rb,0.31978e-02_rb, &
        & 0.36904e-02_rb,0.41225e-02_rb,0.44151e-02_rb,0.61956e-02_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.13696e-04_rb,0.13266e-02_rb,0.21814e-02_rb,0.28939e-02_rb,0.35284e-02_rb, &
        & 0.40629e-02_rb,0.45191e-02_rb,0.48283e-02_rb,0.65800e-02_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.91179e-05_rb,0.81531e-03_rb,0.12754e-02_rb,0.16598e-02_rb,0.19900e-02_rb, &
        & 0.22634e-02_rb,0.24441e-02_rb,0.24984e-02_rb,0.40576e-02_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.95039e-05_rb,0.89093e-03_rb,0.14150e-02_rb,0.18287e-02_rb,0.22143e-02_rb, &
        & 0.25329e-02_rb,0.28092e-02_rb,0.29546e-02_rb,0.44374e-02_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.99762e-05_rb,0.96494e-03_rb,0.15625e-02_rb,0.20288e-02_rb,0.24319e-02_rb, &
        & 0.28108e-02_rb,0.31373e-02_rb,0.33177e-02_rb,0.48094e-02_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.10509e-04_rb,0.10398e-02_rb,0.16996e-02_rb,0.22443e-02_rb,0.26925e-02_rb, &
        & 0.30942e-02_rb,0.34481e-02_rb,0.37003e-02_rb,0.51550e-02_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.11033e-04_rb,0.11134e-02_rb,0.18372e-02_rb,0.24408e-02_rb,0.29682e-02_rb, &
        & 0.34114e-02_rb,0.37859e-02_rb,0.40424e-02_rb,0.54893e-02_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.25564e-03_rb,0.23940e-01_rb,0.41535e-01_rb,0.56276e-01_rb,0.68561e-01_rb, &
        & 0.78630e-01_rb,0.86993e-01_rb,0.91794e-01_rb,0.84229e-01_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.29194e-03_rb,0.25170e-01_rb,0.43659e-01_rb,0.59016e-01_rb,0.72302e-01_rb, &
        & 0.83167e-01_rb,0.91739e-01_rb,0.96774e-01_rb,0.86838e-01_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.33934e-03_rb,0.26430e-01_rb,0.45835e-01_rb,0.61922e-01_rb,0.75614e-01_rb, &
        & 0.87285e-01_rb,0.96606e-01_rb,0.10156e+00_rb,0.90084e-01_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.38871e-03_rb,0.27733e-01_rb,0.48058e-01_rb,0.64973e-01_rb,0.79090e-01_rb, &
        & 0.91156e-01_rb,0.10084e+00_rb,0.10589e+00_rb,0.92583e-01_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.44473e-03_rb,0.29028e-01_rb,0.50287e-01_rb,0.68010e-01_rb,0.82805e-01_rb, &
        & 0.95193e-01_rb,0.10492e+00_rb,0.10995e+00_rb,0.95419e-01_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.23400e-03_rb,0.20753e-01_rb,0.35742e-01_rb,0.48709e-01_rb,0.59548e-01_rb, &
        & 0.68206e-01_rb,0.75045e-01_rb,0.78915e-01_rb,0.70180e-01_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.26775e-03_rb,0.21832e-01_rb,0.37611e-01_rb,0.51156e-01_rb,0.62768e-01_rb, &
        & 0.72267e-01_rb,0.79422e-01_rb,0.83262e-01_rb,0.72735e-01_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.30033e-03_rb,0.22961e-01_rb,0.39618e-01_rb,0.53774e-01_rb,0.65773e-01_rb, &
        & 0.75819e-01_rb,0.83532e-01_rb,0.87496e-01_rb,0.75459e-01_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.34366e-03_rb,0.24116e-01_rb,0.41597e-01_rb,0.56506e-01_rb,0.68944e-01_rb, &
        & 0.79231e-01_rb,0.87245e-01_rb,0.91336e-01_rb,0.77669e-01_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.39179e-03_rb,0.25289e-01_rb,0.43575e-01_rb,0.59282e-01_rb,0.72132e-01_rb, &
        & 0.82956e-01_rb,0.91038e-01_rb,0.94937e-01_rb,0.80272e-01_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.20730e-03_rb,0.18044e-01_rb,0.30227e-01_rb,0.41206e-01_rb,0.50626e-01_rb, &
        & 0.58095e-01_rb,0.63725e-01_rb,0.66721e-01_rb,0.57780e-01_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.24007e-03_rb,0.18972e-01_rb,0.31883e-01_rb,0.43486e-01_rb,0.53572e-01_rb, &
        & 0.61742e-01_rb,0.67489e-01_rb,0.70552e-01_rb,0.60009e-01_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.26931e-03_rb,0.19934e-01_rb,0.33681e-01_rb,0.45775e-01_rb,0.56255e-01_rb, &
        & 0.64869e-01_rb,0.71254e-01_rb,0.74232e-01_rb,0.62350e-01_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.30064e-03_rb,0.20965e-01_rb,0.35448e-01_rb,0.48199e-01_rb,0.59036e-01_rb, &
        & 0.67921e-01_rb,0.74594e-01_rb,0.77689e-01_rb,0.64410e-01_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.33967e-03_rb,0.21972e-01_rb,0.37192e-01_rb,0.50676e-01_rb,0.61884e-01_rb, &
        & 0.71207e-01_rb,0.77888e-01_rb,0.80947e-01_rb,0.66667e-01_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.18201e-03_rb,0.15595e-01_rb,0.25372e-01_rb,0.34431e-01_rb,0.42401e-01_rb, &
        & 0.48915e-01_rb,0.53613e-01_rb,0.55953e-01_rb,0.47727e-01_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.20883e-03_rb,0.16422e-01_rb,0.26851e-01_rb,0.36481e-01_rb,0.45074e-01_rb, &
        & 0.52078e-01_rb,0.56860e-01_rb,0.59287e-01_rb,0.49553e-01_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.23874e-03_rb,0.17285e-01_rb,0.28425e-01_rb,0.38514e-01_rb,0.47542e-01_rb, &
        & 0.54904e-01_rb,0.60284e-01_rb,0.62561e-01_rb,0.51595e-01_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.26707e-03_rb,0.18200e-01_rb,0.30007e-01_rb,0.40613e-01_rb,0.49982e-01_rb, &
        & 0.57662e-01_rb,0.63231e-01_rb,0.65666e-01_rb,0.53450e-01_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.30102e-03_rb,0.19113e-01_rb,0.31552e-01_rb,0.42805e-01_rb,0.52535e-01_rb, &
        & 0.60559e-01_rb,0.66195e-01_rb,0.68549e-01_rb,0.55415e-01_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.15801e-03_rb,0.13295e-01_rb,0.21524e-01_rb,0.28666e-01_rb,0.35142e-01_rb, &
        & 0.40757e-01_rb,0.44715e-01_rb,0.46740e-01_rb,0.39384e-01_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.17994e-03_rb,0.14057e-01_rb,0.22786e-01_rb,0.30487e-01_rb,0.37563e-01_rb, &
        & 0.43509e-01_rb,0.47676e-01_rb,0.49638e-01_rb,0.40849e-01_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.20535e-03_rb,0.14811e-01_rb,0.24116e-01_rb,0.32266e-01_rb,0.39764e-01_rb, &
        & 0.46122e-01_rb,0.50669e-01_rb,0.52413e-01_rb,0.42683e-01_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.23511e-03_rb,0.15598e-01_rb,0.25481e-01_rb,0.34103e-01_rb,0.41924e-01_rb, &
        & 0.48592e-01_rb,0.53283e-01_rb,0.55235e-01_rb,0.44335e-01_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.26640e-03_rb,0.16402e-01_rb,0.26844e-01_rb,0.36000e-01_rb,0.44180e-01_rb, &
        & 0.51086e-01_rb,0.55918e-01_rb,0.57796e-01_rb,0.45991e-01_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.13805e-03_rb,0.11222e-01_rb,0.18125e-01_rb,0.23877e-01_rb,0.28939e-01_rb, &
        & 0.33506e-01_rb,0.36944e-01_rb,0.38672e-01_rb,0.32335e-01_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.15439e-03_rb,0.11883e-01_rb,0.19262e-01_rb,0.25521e-01_rb,0.31068e-01_rb, &
        & 0.35882e-01_rb,0.39589e-01_rb,0.41132e-01_rb,0.33684e-01_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.17561e-03_rb,0.12558e-01_rb,0.20409e-01_rb,0.27033e-01_rb,0.33056e-01_rb, &
        & 0.38260e-01_rb,0.42155e-01_rb,0.43639e-01_rb,0.35175e-01_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.19967e-03_rb,0.13236e-01_rb,0.21628e-01_rb,0.28554e-01_rb,0.34939e-01_rb, &
        & 0.40459e-01_rb,0.44550e-01_rb,0.46147e-01_rb,0.36643e-01_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.23117e-03_rb,0.13936e-01_rb,0.22836e-01_rb,0.30183e-01_rb,0.36909e-01_rb, &
        & 0.42652e-01_rb,0.46864e-01_rb,0.48440e-01_rb,0.37992e-01_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.11987e-03_rb,0.93983e-02_rb,0.15130e-01_rb,0.19808e-01_rb,0.23796e-01_rb, &
        & 0.27285e-01_rb,0.30176e-01_rb,0.31906e-01_rb,0.26193e-01_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.13164e-03_rb,0.99743e-02_rb,0.16118e-01_rb,0.21319e-01_rb,0.25600e-01_rb, &
        & 0.29404e-01_rb,0.32485e-01_rb,0.33909e-01_rb,0.27439e-01_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.14862e-03_rb,0.10559e-01_rb,0.17124e-01_rb,0.22674e-01_rb,0.27373e-01_rb, &
        & 0.31501e-01_rb,0.34705e-01_rb,0.36112e-01_rb,0.28667e-01_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.16941e-03_rb,0.11142e-01_rb,0.18163e-01_rb,0.23992e-01_rb,0.29023e-01_rb, &
        & 0.33461e-01_rb,0.36882e-01_rb,0.38288e-01_rb,0.29990e-01_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.19444e-03_rb,0.11740e-01_rb,0.19231e-01_rb,0.25410e-01_rb,0.30702e-01_rb, &
        & 0.35380e-01_rb,0.38933e-01_rb,0.40325e-01_rb,0.31163e-01_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.10266e-03_rb,0.78132e-02_rb,0.12495e-01_rb,0.16294e-01_rb,0.19552e-01_rb, &
        & 0.22203e-01_rb,0.24522e-01_rb,0.26158e-01_rb,0.21477e-01_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.11308e-03_rb,0.82996e-02_rb,0.13385e-01_rb,0.17650e-01_rb,0.21113e-01_rb, &
        & 0.24026e-01_rb,0.26424e-01_rb,0.27779e-01_rb,0.22661e-01_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.12431e-03_rb,0.88118e-02_rb,0.14254e-01_rb,0.18852e-01_rb,0.22717e-01_rb, &
        & 0.25822e-01_rb,0.28352e-01_rb,0.29667e-01_rb,0.23674e-01_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.14146e-03_rb,0.93080e-02_rb,0.15152e-01_rb,0.20013e-01_rb,0.24158e-01_rb, &
        & 0.27569e-01_rb,0.30306e-01_rb,0.31495e-01_rb,0.24838e-01_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.16275e-03_rb,0.98287e-02_rb,0.16080e-01_rb,0.21227e-01_rb,0.25592e-01_rb, &
        & 0.29226e-01_rb,0.32075e-01_rb,0.33357e-01_rb,0.25887e-01_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.85797e-04_rb,0.64412e-02_rb,0.10231e-01_rb,0.13327e-01_rb,0.15923e-01_rb, &
        & 0.18103e-01_rb,0.19903e-01_rb,0.21189e-01_rb,0.17975e-01_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.94370e-04_rb,0.68514e-02_rb,0.11041e-01_rb,0.14472e-01_rb,0.17298e-01_rb, &
        & 0.19661e-01_rb,0.21421e-01_rb,0.22631e-01_rb,0.18990e-01_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.10603e-03_rb,0.72870e-02_rb,0.11786e-01_rb,0.15570e-01_rb,0.18710e-01_rb, &
        & 0.21195e-01_rb,0.23092e-01_rb,0.24157e-01_rb,0.19934e-01_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.11819e-03_rb,0.77217e-02_rb,0.12559e-01_rb,0.16589e-01_rb,0.19991e-01_rb, &
        & 0.22738e-01_rb,0.24769e-01_rb,0.25771e-01_rb,0.20946e-01_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.13500e-03_rb,0.81624e-02_rb,0.13346e-01_rb,0.17610e-01_rb,0.21227e-01_rb, &
        & 0.24140e-01_rb,0.26339e-01_rb,0.27388e-01_rb,0.21819e-01_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.73275e-04_rb,0.53147e-02_rb,0.83832e-02_rb,0.10890e-01_rb,0.12998e-01_rb, &
        & 0.14772e-01_rb,0.16237e-01_rb,0.17165e-01_rb,0.15972e-01_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.80303e-04_rb,0.56594e-02_rb,0.90964e-02_rb,0.11869e-01_rb,0.14181e-01_rb, &
        & 0.16065e-01_rb,0.17493e-01_rb,0.18447e-01_rb,0.17075e-01_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.88821e-04_rb,0.60286e-02_rb,0.97491e-02_rb,0.12852e-01_rb,0.15387e-01_rb, &
        & 0.17394e-01_rb,0.18909e-01_rb,0.19708e-01_rb,0.18078e-01_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.10142e-03_rb,0.63962e-02_rb,0.10392e-01_rb,0.13730e-01_rb,0.16522e-01_rb, &
        & 0.18771e-01_rb,0.20308e-01_rb,0.21096e-01_rb,0.19037e-01_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.11405e-03_rb,0.67754e-02_rb,0.11065e-01_rb,0.14605e-01_rb,0.17581e-01_rb, &
        & 0.19991e-01_rb,0.21687e-01_rb,0.22458e-01_rb,0.19875e-01_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.64517e-04_rb,0.45135e-02_rb,0.71165e-02_rb,0.92384e-02_rb,0.10998e-01_rb, &
        & 0.12474e-01_rb,0.13700e-01_rb,0.14388e-01_rb,0.14631e-01_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.71535e-04_rb,0.48070e-02_rb,0.77137e-02_rb,0.10092e-01_rb,0.12023e-01_rb, &
        & 0.13623e-01_rb,0.14792e-01_rb,0.15449e-01_rb,0.15481e-01_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.79504e-04_rb,0.51199e-02_rb,0.82600e-02_rb,0.10911e-01_rb,0.13076e-01_rb, &
        & 0.14755e-01_rb,0.16002e-01_rb,0.16546e-01_rb,0.16298e-01_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.90433e-04_rb,0.54351e-02_rb,0.88101e-02_rb,0.11647e-01_rb,0.14018e-01_rb, &
        & 0.15922e-01_rb,0.17206e-01_rb,0.17715e-01_rb,0.16991e-01_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.10146e-03_rb,0.57596e-02_rb,0.93865e-02_rb,0.12407e-01_rb,0.14922e-01_rb, &
        & 0.16951e-01_rb,0.18384e-01_rb,0.18893e-01_rb,0.17666e-01_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.56913e-04_rb,0.38361e-02_rb,0.60238e-02_rb,0.78101e-02_rb,0.92943e-02_rb, &
        & 0.10515e-01_rb,0.11514e-01_rb,0.12051e-01_rb,0.12708e-01_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.62984e-04_rb,0.40886e-02_rb,0.65153e-02_rb,0.85464e-02_rb,0.10168e-01_rb, &
        & 0.11498e-01_rb,0.12452e-01_rb,0.12938e-01_rb,0.13473e-01_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.70588e-04_rb,0.43561e-02_rb,0.69761e-02_rb,0.92137e-02_rb,0.11068e-01_rb, &
        & 0.12482e-01_rb,0.13497e-01_rb,0.13892e-01_rb,0.14090e-01_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.79556e-04_rb,0.46222e-02_rb,0.74472e-02_rb,0.98382e-02_rb,0.11852e-01_rb, &
        & 0.13461e-01_rb,0.14539e-01_rb,0.14868e-01_rb,0.14684e-01_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.89330e-04_rb,0.49037e-02_rb,0.79332e-02_rb,0.10492e-01_rb,0.12613e-01_rb, &
        & 0.14331e-01_rb,0.15523e-01_rb,0.15867e-01_rb,0.15266e-01_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.49040e-04_rb,0.32470e-02_rb,0.50782e-02_rb,0.65813e-02_rb,0.78254e-02_rb, &
        & 0.88398e-02_rb,0.96483e-02_rb,0.10090e-01_rb,0.10614e-01_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.54225e-04_rb,0.34616e-02_rb,0.54906e-02_rb,0.71951e-02_rb,0.85711e-02_rb, &
        & 0.96781e-02_rb,0.10460e-01_rb,0.10840e-01_rb,0.11206e-01_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.61136e-04_rb,0.36878e-02_rb,0.58777e-02_rb,0.77458e-02_rb,0.93183e-02_rb, &
        & 0.10528e-01_rb,0.11347e-01_rb,0.11684e-01_rb,0.11730e-01_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.68788e-04_rb,0.39213e-02_rb,0.62791e-02_rb,0.82774e-02_rb,0.99761e-02_rb, &
        & 0.11332e-01_rb,0.12247e-01_rb,0.12495e-01_rb,0.12250e-01_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.77427e-04_rb,0.41640e-02_rb,0.66890e-02_rb,0.88358e-02_rb,0.10624e-01_rb, &
        & 0.12069e-01_rb,0.13064e-01_rb,0.13341e-01_rb,0.12730e-01_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.69098e-02_rb,0.84650e-01_rb,0.13006e+00_rb,0.16652e+00_rb,0.19608e+00_rb, &
        & 0.21975e+00_rb,0.23496e+00_rb,0.23277e+00_rb,0.18007e+00_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.78200e-02_rb,0.89279e-01_rb,0.13630e+00_rb,0.17409e+00_rb,0.20398e+00_rb, &
        & 0.22718e+00_rb,0.24129e+00_rb,0.23750e+00_rb,0.18438e+00_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.89787e-02_rb,0.94428e-01_rb,0.14296e+00_rb,0.18170e+00_rb,0.21205e+00_rb, &
        & 0.23450e+00_rb,0.24726e+00_rb,0.24172e+00_rb,0.18878e+00_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.10426e-01_rb,0.10003e+00_rb,0.15009e+00_rb,0.18943e+00_rb,0.22015e+00_rb, &
        & 0.24206e+00_rb,0.25341e+00_rb,0.24571e+00_rb,0.19325e+00_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.12209e-01_rb,0.10603e+00_rb,0.15781e+00_rb,0.19756e+00_rb,0.22832e+00_rb, &
        & 0.24963e+00_rb,0.25957e+00_rb,0.24958e+00_rb,0.19748e+00_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.64020e-02_rb,0.74597e-01_rb,0.11458e+00_rb,0.14581e+00_rb,0.17103e+00_rb, &
        & 0.19105e+00_rb,0.20412e+00_rb,0.20133e+00_rb,0.15242e+00_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.72586e-02_rb,0.78857e-01_rb,0.12024e+00_rb,0.15257e+00_rb,0.17796e+00_rb, &
        & 0.19765e+00_rb,0.20972e+00_rb,0.20548e+00_rb,0.15620e+00_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.82903e-02_rb,0.83602e-01_rb,0.12623e+00_rb,0.15931e+00_rb,0.18512e+00_rb, &
        & 0.20423e+00_rb,0.21503e+00_rb,0.20937e+00_rb,0.15993e+00_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.94578e-02_rb,0.88668e-01_rb,0.13277e+00_rb,0.16635e+00_rb,0.19233e+00_rb, &
        & 0.21098e+00_rb,0.22050e+00_rb,0.21294e+00_rb,0.16397e+00_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.10865e-01_rb,0.94113e-01_rb,0.13987e+00_rb,0.17375e+00_rb,0.19975e+00_rb, &
        & 0.21771e+00_rb,0.22592e+00_rb,0.21639e+00_rb,0.16745e+00_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.56943e-02_rb,0.64084e-01_rb,0.99294e-01_rb,0.12614e+00_rb,0.14746e+00_rb, &
        & 0.16414e+00_rb,0.17503e+00_rb,0.17255e+00_rb,0.12738e+00_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.64702e-02_rb,0.67967e-01_rb,0.10427e+00_rb,0.13197e+00_rb,0.15350e+00_rb, &
        & 0.16985e+00_rb,0.18008e+00_rb,0.17631e+00_rb,0.13087e+00_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.74200e-02_rb,0.72252e-01_rb,0.10956e+00_rb,0.13801e+00_rb,0.15978e+00_rb, &
        & 0.17567e+00_rb,0.18476e+00_rb,0.17977e+00_rb,0.13421e+00_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.84907e-02_rb,0.76772e-01_rb,0.11539e+00_rb,0.14427e+00_rb,0.16615e+00_rb, &
        & 0.18168e+00_rb,0.18947e+00_rb,0.18300e+00_rb,0.13761e+00_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.97078e-02_rb,0.81671e-01_rb,0.12172e+00_rb,0.15082e+00_rb,0.17277e+00_rb, &
        & 0.18769e+00_rb,0.19427e+00_rb,0.18608e+00_rb,0.14067e+00_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.48829e-02_rb,0.54383e-01_rb,0.85075e-01_rb,0.10815e+00_rb,0.12638e+00_rb, &
        & 0.14020e+00_rb,0.14903e+00_rb,0.14670e+00_rb,0.10614e+00_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.55740e-02_rb,0.57779e-01_rb,0.89436e-01_rb,0.11323e+00_rb,0.13160e+00_rb, &
        & 0.14514e+00_rb,0.15349e+00_rb,0.15027e+00_rb,0.10949e+00_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.64015e-02_rb,0.61546e-01_rb,0.94139e-01_rb,0.11850e+00_rb,0.13701e+00_rb, &
        & 0.15017e+00_rb,0.15757e+00_rb,0.15344e+00_rb,0.11257e+00_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.73719e-02_rb,0.65521e-01_rb,0.99232e-01_rb,0.12403e+00_rb,0.14259e+00_rb, &
        & 0.15541e+00_rb,0.16175e+00_rb,0.15634e+00_rb,0.11557e+00_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.84789e-02_rb,0.69810e-01_rb,0.10478e+00_rb,0.12985e+00_rb,0.14840e+00_rb, &
        & 0.16071e+00_rb,0.16597e+00_rb,0.15910e+00_rb,0.11833e+00_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.41357e-02_rb,0.45878e-01_rb,0.71866e-01_rb,0.92130e-01_rb,0.10772e+00_rb, &
        & 0.11935e+00_rb,0.12627e+00_rb,0.12382e+00_rb,0.88490e-01_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.47278e-02_rb,0.48809e-01_rb,0.75724e-01_rb,0.96485e-01_rb,0.11218e+00_rb, &
        & 0.12363e+00_rb,0.13023e+00_rb,0.12723e+00_rb,0.91646e-01_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.54234e-02_rb,0.52108e-01_rb,0.79925e-01_rb,0.10105e+00_rb,0.11684e+00_rb, &
        & 0.12795e+00_rb,0.13389e+00_rb,0.13024e+00_rb,0.94384e-01_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.62672e-02_rb,0.55647e-01_rb,0.84438e-01_rb,0.10591e+00_rb,0.12167e+00_rb, &
        & 0.13255e+00_rb,0.13759e+00_rb,0.13288e+00_rb,0.97037e-01_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.72387e-02_rb,0.59393e-01_rb,0.89326e-01_rb,0.11102e+00_rb,0.12680e+00_rb, &
        & 0.13723e+00_rb,0.14128e+00_rb,0.13530e+00_rb,0.99585e-01_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.34820e-02_rb,0.38455e-01_rb,0.60106e-01_rb,0.77515e-01_rb,0.91143e-01_rb, &
        & 0.10094e+00_rb,0.10649e+00_rb,0.10403e+00_rb,0.73509e-01_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.39507e-02_rb,0.40927e-01_rb,0.63402e-01_rb,0.81240e-01_rb,0.94968e-01_rb, &
        & 0.10469e+00_rb,0.11001e+00_rb,0.10717e+00_rb,0.76426e-01_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.45266e-02_rb,0.43726e-01_rb,0.67041e-01_rb,0.85234e-01_rb,0.98911e-01_rb, &
        & 0.10845e+00_rb,0.11329e+00_rb,0.10996e+00_rb,0.78861e-01_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.52242e-02_rb,0.46807e-01_rb,0.70939e-01_rb,0.89547e-01_rb,0.10312e+00_rb, &
        & 0.11247e+00_rb,0.11653e+00_rb,0.11237e+00_rb,0.81250e-01_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.60449e-02_rb,0.50083e-01_rb,0.75175e-01_rb,0.94041e-01_rb,0.10760e+00_rb, &
        & 0.11657e+00_rb,0.11980e+00_rb,0.11449e+00_rb,0.83549e-01_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.28711e-02_rb,0.32523e-01_rb,0.49971e-01_rb,0.64597e-01_rb,0.76360e-01_rb, &
        & 0.84918e-01_rb,0.89523e-01_rb,0.87200e-01_rb,0.60618e-01_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.32550e-02_rb,0.34614e-01_rb,0.52803e-01_rb,0.67740e-01_rb,0.79713e-01_rb, &
        & 0.88207e-01_rb,0.92681e-01_rb,0.90060e-01_rb,0.63331e-01_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.37263e-02_rb,0.36968e-01_rb,0.55904e-01_rb,0.71153e-01_rb,0.83143e-01_rb, &
        & 0.91509e-01_rb,0.95607e-01_rb,0.92564e-01_rb,0.65595e-01_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.42953e-02_rb,0.39538e-01_rb,0.59281e-01_rb,0.74912e-01_rb,0.86822e-01_rb, &
        & 0.94956e-01_rb,0.98438e-01_rb,0.94728e-01_rb,0.67680e-01_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.49788e-02_rb,0.42264e-01_rb,0.62953e-01_rb,0.78842e-01_rb,0.90742e-01_rb, &
        & 0.98510e-01_rb,0.10129e+00_rb,0.96641e-01_rb,0.69746e-01_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.23480e-02_rb,0.27230e-01_rb,0.41671e-01_rb,0.53565e-01_rb,0.63436e-01_rb, &
        & 0.70999e-01_rb,0.75020e-01_rb,0.72906e-01_rb,0.49386e-01_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.26517e-02_rb,0.29065e-01_rb,0.44017e-01_rb,0.56225e-01_rb,0.66341e-01_rb, &
        & 0.73935e-01_rb,0.77879e-01_rb,0.75468e-01_rb,0.51750e-01_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.30285e-02_rb,0.31112e-01_rb,0.46598e-01_rb,0.59161e-01_rb,0.69302e-01_rb, &
        & 0.76809e-01_rb,0.80439e-01_rb,0.77729e-01_rb,0.53904e-01_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.34816e-02_rb,0.33348e-01_rb,0.49448e-01_rb,0.62380e-01_rb,0.72502e-01_rb, &
        & 0.79786e-01_rb,0.82887e-01_rb,0.79683e-01_rb,0.55731e-01_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.40444e-02_rb,0.35730e-01_rb,0.52545e-01_rb,0.65794e-01_rb,0.75878e-01_rb, &
        & 0.82889e-01_rb,0.85379e-01_rb,0.81391e-01_rb,0.57509e-01_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.19247e-02_rb,0.22630e-01_rb,0.34752e-01_rb,0.44387e-01_rb,0.52455e-01_rb, &
        & 0.58833e-01_rb,0.62468e-01_rb,0.60719e-01_rb,0.41880e-01_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.21557e-02_rb,0.24204e-01_rb,0.36750e-01_rb,0.46636e-01_rb,0.54976e-01_rb, &
        & 0.61421e-01_rb,0.65052e-01_rb,0.62994e-01_rb,0.44010e-01_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.24477e-02_rb,0.25958e-01_rb,0.38981e-01_rb,0.49108e-01_rb,0.57526e-01_rb, &
        & 0.63907e-01_rb,0.67351e-01_rb,0.65038e-01_rb,0.45929e-01_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.28196e-02_rb,0.27882e-01_rb,0.41431e-01_rb,0.51802e-01_rb,0.60249e-01_rb, &
        & 0.66480e-01_rb,0.69467e-01_rb,0.66788e-01_rb,0.47555e-01_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.32654e-02_rb,0.29939e-01_rb,0.44072e-01_rb,0.54714e-01_rb,0.63163e-01_rb, &
        & 0.69207e-01_rb,0.71600e-01_rb,0.68328e-01_rb,0.49077e-01_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.16200e-02_rb,0.18814e-01_rb,0.28911e-01_rb,0.37025e-01_rb,0.43479e-01_rb, &
        & 0.48691e-01_rb,0.51767e-01_rb,0.50517e-01_rb,0.36565e-01_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.18031e-02_rb,0.20150e-01_rb,0.30632e-01_rb,0.38967e-01_rb,0.45609e-01_rb, &
        & 0.50940e-01_rb,0.54046e-01_rb,0.52515e-01_rb,0.38425e-01_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.20232e-02_rb,0.21640e-01_rb,0.32572e-01_rb,0.41079e-01_rb,0.47788e-01_rb, &
        & 0.53101e-01_rb,0.56114e-01_rb,0.54338e-01_rb,0.40162e-01_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.23038e-02_rb,0.23298e-01_rb,0.34703e-01_rb,0.43393e-01_rb,0.50110e-01_rb, &
        & 0.55298e-01_rb,0.57982e-01_rb,0.55873e-01_rb,0.41782e-01_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.26649e-02_rb,0.25073e-01_rb,0.36983e-01_rb,0.45868e-01_rb,0.52617e-01_rb, &
        & 0.57641e-01_rb,0.59833e-01_rb,0.57222e-01_rb,0.43243e-01_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.14241e-02_rb,0.16065e-01_rb,0.24586e-01_rb,0.31465e-01_rb,0.36963e-01_rb, &
        & 0.41093e-01_rb,0.43617e-01_rb,0.42673e-01_rb,0.32343e-01_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.15878e-02_rb,0.17268e-01_rb,0.26144e-01_rb,0.33169e-01_rb,0.38816e-01_rb, &
        & 0.42988e-01_rb,0.45546e-01_rb,0.44327e-01_rb,0.34054e-01_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.18003e-02_rb,0.18609e-01_rb,0.27888e-01_rb,0.35066e-01_rb,0.40717e-01_rb, &
        & 0.44861e-01_rb,0.47277e-01_rb,0.45845e-01_rb,0.35562e-01_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.20513e-02_rb,0.20069e-01_rb,0.29794e-01_rb,0.37147e-01_rb,0.42781e-01_rb, &
        & 0.46774e-01_rb,0.48869e-01_rb,0.47106e-01_rb,0.36881e-01_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.23610e-02_rb,0.21636e-01_rb,0.31786e-01_rb,0.39341e-01_rb,0.44964e-01_rb, &
        & 0.48832e-01_rb,0.50478e-01_rb,0.48237e-01_rb,0.38136e-01_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.12367e-02_rb,0.13664e-01_rb,0.20841e-01_rb,0.26619e-01_rb,0.31258e-01_rb, &
        & 0.34766e-01_rb,0.36665e-01_rb,0.35873e-01_rb,0.27811e-01_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.13898e-02_rb,0.14747e-01_rb,0.22238e-01_rb,0.28134e-01_rb,0.32876e-01_rb, &
        & 0.36392e-01_rb,0.38286e-01_rb,0.37257e-01_rb,0.29212e-01_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.15822e-02_rb,0.15920e-01_rb,0.23808e-01_rb,0.29830e-01_rb,0.34558e-01_rb, &
        & 0.38014e-01_rb,0.39728e-01_rb,0.38493e-01_rb,0.30517e-01_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.18117e-02_rb,0.17195e-01_rb,0.25479e-01_rb,0.31664e-01_rb,0.36378e-01_rb, &
        & 0.39675e-01_rb,0.41098e-01_rb,0.39553e-01_rb,0.31705e-01_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.20996e-02_rb,0.18567e-01_rb,0.27225e-01_rb,0.33571e-01_rb,0.38302e-01_rb, &
        & 0.41437e-01_rb,0.42501e-01_rb,0.40525e-01_rb,0.32748e-01_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.10606e-02_rb,0.11594e-01_rb,0.17616e-01_rb,0.22456e-01_rb,0.26337e-01_rb, &
        & 0.29282e-01_rb,0.30831e-01_rb,0.30037e-01_rb,0.23251e-01_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.11975e-02_rb,0.12541e-01_rb,0.18850e-01_rb,0.23795e-01_rb,0.27745e-01_rb, &
        & 0.30676e-01_rb,0.32191e-01_rb,0.31186e-01_rb,0.24461e-01_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.13686e-02_rb,0.13557e-01_rb,0.20236e-01_rb,0.25287e-01_rb,0.29224e-01_rb, &
        & 0.32085e-01_rb,0.33411e-01_rb,0.32190e-01_rb,0.25539e-01_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.15787e-02_rb,0.14655e-01_rb,0.21685e-01_rb,0.26887e-01_rb,0.30821e-01_rb, &
        & 0.33533e-01_rb,0.34581e-01_rb,0.33091e-01_rb,0.26483e-01_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.18422e-02_rb,0.15847e-01_rb,0.23201e-01_rb,0.28535e-01_rb,0.32486e-01_rb, &
        & 0.35059e-01_rb,0.35791e-01_rb,0.33942e-01_rb,0.27340e-01_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.11257e+00_rb,0.31731e+00_rb,0.42139e+00_rb,0.49121e+00_rb,0.53179e+00_rb, &
        & 0.54416e+00_rb,0.52879e+00_rb,0.48565e+00_rb,0.38185e+00_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.12995e+00_rb,0.33757e+00_rb,0.44079e+00_rb,0.50693e+00_rb,0.54374e+00_rb, &
        & 0.55231e+00_rb,0.53411e+00_rb,0.48973e+00_rb,0.38664e+00_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.14899e+00_rb,0.35787e+00_rb,0.45940e+00_rb,0.52235e+00_rb,0.55490e+00_rb, &
        & 0.56009e+00_rb,0.53945e+00_rb,0.49411e+00_rb,0.39028e+00_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.16946e+00_rb,0.37848e+00_rb,0.47747e+00_rb,0.53692e+00_rb,0.56497e+00_rb, &
        & 0.56760e+00_rb,0.54484e+00_rb,0.49832e+00_rb,0.39391e+00_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.19143e+00_rb,0.39915e+00_rb,0.49476e+00_rb,0.55039e+00_rb,0.57468e+00_rb, &
        & 0.57486e+00_rb,0.55036e+00_rb,0.50307e+00_rb,0.39756e+00_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.97091e-01_rb,0.28037e+00_rb,0.37054e+00_rb,0.43162e+00_rb,0.46715e+00_rb, &
        & 0.47740e+00_rb,0.46327e+00_rb,0.42557e+00_rb,0.32833e+00_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.11208e+00_rb,0.29815e+00_rb,0.38764e+00_rb,0.44615e+00_rb,0.47841e+00_rb, &
        & 0.48513e+00_rb,0.46886e+00_rb,0.42998e+00_rb,0.33260e+00_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.12856e+00_rb,0.31599e+00_rb,0.40420e+00_rb,0.45996e+00_rb,0.48848e+00_rb, &
        & 0.49285e+00_rb,0.47441e+00_rb,0.43431e+00_rb,0.33632e+00_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.14648e+00_rb,0.33425e+00_rb,0.42020e+00_rb,0.47262e+00_rb,0.49778e+00_rb, &
        & 0.50015e+00_rb,0.47996e+00_rb,0.43890e+00_rb,0.33965e+00_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.16579e+00_rb,0.35250e+00_rb,0.43554e+00_rb,0.48445e+00_rb,0.50655e+00_rb, &
        & 0.50702e+00_rb,0.48572e+00_rb,0.44375e+00_rb,0.34330e+00_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.80654e-01_rb,0.24334e+00_rb,0.32145e+00_rb,0.37485e+00_rb,0.40615e+00_rb, &
        & 0.41488e+00_rb,0.40284e+00_rb,0.37005e+00_rb,0.27897e+00_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.93334e-01_rb,0.25889e+00_rb,0.33665e+00_rb,0.38799e+00_rb,0.41628e+00_rb, &
        & 0.42254e+00_rb,0.40845e+00_rb,0.37420e+00_rb,0.28281e+00_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.10735e+00_rb,0.27449e+00_rb,0.35143e+00_rb,0.40015e+00_rb,0.42551e+00_rb, &
        & 0.42981e+00_rb,0.41382e+00_rb,0.37871e+00_rb,0.28619e+00_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.12265e+00_rb,0.29043e+00_rb,0.36548e+00_rb,0.41153e+00_rb,0.43393e+00_rb, &
        & 0.43649e+00_rb,0.41932e+00_rb,0.38340e+00_rb,0.28955e+00_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.13925e+00_rb,0.30659e+00_rb,0.37895e+00_rb,0.42211e+00_rb,0.44187e+00_rb, &
        & 0.44276e+00_rb,0.42490e+00_rb,0.38830e+00_rb,0.29282e+00_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.65986e-01_rb,0.20968e+00_rb,0.27746e+00_rb,0.32321e+00_rb,0.35021e+00_rb, &
        & 0.35817e+00_rb,0.34816e+00_rb,0.31914e+00_rb,0.23714e+00_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.76550e-01_rb,0.22318e+00_rb,0.29094e+00_rb,0.33486e+00_rb,0.35941e+00_rb, &
        & 0.36546e+00_rb,0.35341e+00_rb,0.32318e+00_rb,0.24040e+00_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.88349e-01_rb,0.23676e+00_rb,0.30387e+00_rb,0.34562e+00_rb,0.36786e+00_rb, &
        & 0.37204e+00_rb,0.35871e+00_rb,0.32765e+00_rb,0.24343e+00_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.10129e+00_rb,0.25065e+00_rb,0.31610e+00_rb,0.35574e+00_rb,0.37546e+00_rb, &
        & 0.37811e+00_rb,0.36390e+00_rb,0.33220e+00_rb,0.24649e+00_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.11540e+00_rb,0.26489e+00_rb,0.32784e+00_rb,0.36504e+00_rb,0.38250e+00_rb, &
        & 0.38387e+00_rb,0.36919e+00_rb,0.33697e+00_rb,0.24943e+00_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.53393e-01_rb,0.18000e+00_rb,0.23883e+00_rb,0.27751e+00_rb,0.30053e+00_rb, &
        & 0.30742e+00_rb,0.29898e+00_rb,0.27383e+00_rb,0.20106e+00_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.62233e-01_rb,0.19175e+00_rb,0.25050e+00_rb,0.28777e+00_rb,0.30898e+00_rb, &
        & 0.31400e+00_rb,0.30389e+00_rb,0.27769e+00_rb,0.20390e+00_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.72177e-01_rb,0.20364e+00_rb,0.26165e+00_rb,0.29741e+00_rb,0.31640e+00_rb, &
        & 0.31996e+00_rb,0.30876e+00_rb,0.28174e+00_rb,0.20671e+00_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.83134e-01_rb,0.21576e+00_rb,0.27227e+00_rb,0.30617e+00_rb,0.32319e+00_rb, &
        & 0.32542e+00_rb,0.31353e+00_rb,0.28599e+00_rb,0.20956e+00_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.95090e-01_rb,0.22815e+00_rb,0.28259e+00_rb,0.31429e+00_rb,0.32943e+00_rb, &
        & 0.33059e+00_rb,0.31837e+00_rb,0.29043e+00_rb,0.21220e+00_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.42433e-01_rb,0.15317e+00_rb,0.20425e+00_rb,0.23740e+00_rb,0.25663e+00_rb, &
        & 0.26234e+00_rb,0.25512e+00_rb,0.23364e+00_rb,0.16949e+00_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.49749e-01_rb,0.16341e+00_rb,0.21435e+00_rb,0.24646e+00_rb,0.26413e+00_rb, &
        & 0.26825e+00_rb,0.25968e+00_rb,0.23716e+00_rb,0.17204e+00_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.58017e-01_rb,0.17381e+00_rb,0.22409e+00_rb,0.25485e+00_rb,0.27073e+00_rb, &
        & 0.27356e+00_rb,0.26405e+00_rb,0.24070e+00_rb,0.17477e+00_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.67231e-01_rb,0.18439e+00_rb,0.23345e+00_rb,0.26240e+00_rb,0.27677e+00_rb, &
        & 0.27839e+00_rb,0.26819e+00_rb,0.24459e+00_rb,0.17741e+00_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.77316e-01_rb,0.19517e+00_rb,0.24253e+00_rb,0.26953e+00_rb,0.28221e+00_rb, &
        & 0.28296e+00_rb,0.27246e+00_rb,0.24867e+00_rb,0.17984e+00_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.33835e-01_rb,0.12894e+00_rb,0.17377e+00_rb,0.20223e+00_rb,0.21858e+00_rb, &
        & 0.22305e+00_rb,0.21661e+00_rb,0.19799e+00_rb,0.14234e+00_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.39495e-01_rb,0.13775e+00_rb,0.18255e+00_rb,0.21017e+00_rb,0.22506e+00_rb, &
        & 0.22828e+00_rb,0.22063e+00_rb,0.20115e+00_rb,0.14469e+00_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.46213e-01_rb,0.14677e+00_rb,0.19104e+00_rb,0.21747e+00_rb,0.23085e+00_rb, &
        & 0.23293e+00_rb,0.22444e+00_rb,0.20438e+00_rb,0.14721e+00_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.53881e-01_rb,0.15604e+00_rb,0.19912e+00_rb,0.22407e+00_rb,0.23606e+00_rb, &
        & 0.23711e+00_rb,0.22803e+00_rb,0.20793e+00_rb,0.14967e+00_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.62313e-01_rb,0.16559e+00_rb,0.20703e+00_rb,0.23032e+00_rb,0.24086e+00_rb, &
        & 0.24109e+00_rb,0.23183e+00_rb,0.21164e+00_rb,0.15187e+00_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.27294e-01_rb,0.10784e+00_rb,0.14673e+00_rb,0.17152e+00_rb,0.18541e+00_rb, &
        & 0.18896e+00_rb,0.18302e+00_rb,0.16686e+00_rb,0.11892e+00_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.31945e-01_rb,0.11533e+00_rb,0.15437e+00_rb,0.17841e+00_rb,0.19097e+00_rb, &
        & 0.19348e+00_rb,0.18650e+00_rb,0.16969e+00_rb,0.12123e+00_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.37290e-01_rb,0.12309e+00_rb,0.16174e+00_rb,0.18467e+00_rb,0.19597e+00_rb, &
        & 0.19753e+00_rb,0.18983e+00_rb,0.17262e+00_rb,0.12344e+00_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.43331e-01_rb,0.13110e+00_rb,0.16878e+00_rb,0.19044e+00_rb,0.20053e+00_rb, &
        & 0.20117e+00_rb,0.19304e+00_rb,0.17581e+00_rb,0.12571e+00_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.50086e-01_rb,0.13937e+00_rb,0.17577e+00_rb,0.19586e+00_rb,0.20476e+00_rb, &
        & 0.20472e+00_rb,0.19647e+00_rb,0.17915e+00_rb,0.12770e+00_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.21763e-01_rb,0.89580e-01_rb,0.12270e+00_rb,0.14431e+00_rb,0.15639e+00_rb, &
        & 0.15936e+00_rb,0.15421e+00_rb,0.14030e+00_rb,0.10042e+00_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.25609e-01_rb,0.95974e-01_rb,0.12925e+00_rb,0.15030e+00_rb,0.16119e+00_rb, &
        & 0.16326e+00_rb,0.15719e+00_rb,0.14283e+00_rb,0.10252e+00_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.30067e-01_rb,0.10261e+00_rb,0.13551e+00_rb,0.15572e+00_rb,0.16552e+00_rb, &
        & 0.16675e+00_rb,0.16007e+00_rb,0.14539e+00_rb,0.10448e+00_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.35097e-01_rb,0.10949e+00_rb,0.14161e+00_rb,0.16075e+00_rb,0.16947e+00_rb, &
        & 0.16997e+00_rb,0.16297e+00_rb,0.14823e+00_rb,0.10645e+00_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.40711e-01_rb,0.11658e+00_rb,0.14773e+00_rb,0.16552e+00_rb,0.17324e+00_rb, &
        & 0.17317e+00_rb,0.16604e+00_rb,0.15112e+00_rb,0.10826e+00_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.17370e-01_rb,0.74466e-01_rb,0.10240e+00_rb,0.12053e+00_rb,0.13126e+00_rb, &
        & 0.13417e+00_rb,0.12989e+00_rb,0.11790e+00_rb,0.87103e-01_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.20536e-01_rb,0.79866e-01_rb,0.10796e+00_rb,0.12568e+00_rb,0.13547e+00_rb, &
        & 0.13755e+00_rb,0.13244e+00_rb,0.12017e+00_rb,0.88680e-01_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.24274e-01_rb,0.85524e-01_rb,0.11324e+00_rb,0.13036e+00_rb,0.13925e+00_rb, &
        & 0.14055e+00_rb,0.13491e+00_rb,0.12243e+00_rb,0.90062e-01_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.28508e-01_rb,0.91420e-01_rb,0.11852e+00_rb,0.13467e+00_rb,0.14267e+00_rb, &
        & 0.14339e+00_rb,0.13748e+00_rb,0.12487e+00_rb,0.91240e-01_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.33217e-01_rb,0.97522e-01_rb,0.12382e+00_rb,0.13886e+00_rb,0.14603e+00_rb, &
        & 0.14625e+00_rb,0.14015e+00_rb,0.12740e+00_rb,0.92631e-01_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.14918e-01_rb,0.64249e-01_rb,0.87193e-01_rb,0.10225e+00_rb,0.11107e+00_rb, &
        & 0.11380e+00_rb,0.11017e+00_rb,0.99775e-01_rb,0.75883e-01_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.17707e-01_rb,0.68887e-01_rb,0.91783e-01_rb,0.10653e+00_rb,0.11454e+00_rb, &
        & 0.11659e+00_rb,0.11231e+00_rb,0.10178e+00_rb,0.77400e-01_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.20985e-01_rb,0.73723e-01_rb,0.96290e-01_rb,0.11038e+00_rb,0.11765e+00_rb, &
        & 0.11911e+00_rb,0.11446e+00_rb,0.10377e+00_rb,0.78827e-01_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.24698e-01_rb,0.78820e-01_rb,0.10085e+00_rb,0.11403e+00_rb,0.12064e+00_rb, &
        & 0.12159e+00_rb,0.11674e+00_rb,0.10593e+00_rb,0.80678e-01_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.28798e-01_rb,0.84087e-01_rb,0.10545e+00_rb,0.11772e+00_rb,0.12367e+00_rb, &
        & 0.12416e+00_rb,0.11916e+00_rb,0.10818e+00_rb,0.82090e-01_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.12778e-01_rb,0.55028e-01_rb,0.74326e-01_rb,0.86458e-01_rb,0.93690e-01_rb, &
        & 0.95887e-01_rb,0.92996e-01_rb,0.84239e-01_rb,0.65531e-01_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.15226e-01_rb,0.59056e-01_rb,0.78180e-01_rb,0.90004e-01_rb,0.96562e-01_rb, &
        & 0.98210e-01_rb,0.94824e-01_rb,0.85996e-01_rb,0.67232e-01_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.18073e-01_rb,0.63328e-01_rb,0.81993e-01_rb,0.93245e-01_rb,0.99175e-01_rb, &
        & 0.10034e+00_rb,0.96751e-01_rb,0.87779e-01_rb,0.68728e-01_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.21287e-01_rb,0.67814e-01_rb,0.85944e-01_rb,0.96446e-01_rb,0.10181e+00_rb, &
        & 0.10253e+00_rb,0.98787e-01_rb,0.89700e-01_rb,0.70024e-01_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.24810e-01_rb,0.72405e-01_rb,0.89913e-01_rb,0.99700e-01_rb,0.10454e+00_rb, &
        & 0.10492e+00_rb,0.10099e+00_rb,0.91662e-01_rb,0.71222e-01_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.10897e-01_rb,0.46824e-01_rb,0.63070e-01_rb,0.73198e-01_rb,0.78826e-01_rb, &
        & 0.80547e-01_rb,0.78087e-01_rb,0.70945e-01_rb,0.55427e-01_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.13037e-01_rb,0.50331e-01_rb,0.66369e-01_rb,0.76169e-01_rb,0.81229e-01_rb, &
        & 0.82485e-01_rb,0.79688e-01_rb,0.72492e-01_rb,0.56776e-01_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.15498e-01_rb,0.54103e-01_rb,0.69687e-01_rb,0.78968e-01_rb,0.83486e-01_rb, &
        & 0.84334e-01_rb,0.81403e-01_rb,0.74109e-01_rb,0.57973e-01_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.18239e-01_rb,0.58007e-01_rb,0.73146e-01_rb,0.81751e-01_rb,0.85842e-01_rb, &
        & 0.86346e-01_rb,0.83244e-01_rb,0.75789e-01_rb,0.59123e-01_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.21231e-01_rb,0.61947e-01_rb,0.76584e-01_rb,0.84579e-01_rb,0.88268e-01_rb, &
        & 0.88508e-01_rb,0.85245e-01_rb,0.77486e-01_rb,0.60191e-01_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.71609e+00_rb,0.10081e+01_rb,0.11118e+01_rb,0.11603e+01_rb,0.11765e+01_rb, &
        & 0.11643e+01_rb,0.11202e+01_rb,0.10296e+01_rb,0.82502e+00_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.75920e+00_rb,0.10292e+01_rb,0.11252e+01_rb,0.11685e+01_rb,0.11822e+01_rb, &
        & 0.11684e+01_rb,0.11228e+01_rb,0.10318e+01_rb,0.83157e+00_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.79807e+00_rb,0.10500e+01_rb,0.11397e+01_rb,0.11794e+01_rb,0.11904e+01_rb, &
        & 0.11740e+01_rb,0.11264e+01_rb,0.10343e+01_rb,0.83726e+00_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.83183e+00_rb,0.10700e+01_rb,0.11548e+01_rb,0.11918e+01_rb,0.12010e+01_rb, &
        & 0.11815e+01_rb,0.11313e+01_rb,0.10364e+01_rb,0.84249e+00_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.86161e+00_rb,0.10887e+01_rb,0.11706e+01_rb,0.12061e+01_rb,0.12126e+01_rb, &
        & 0.11904e+01_rb,0.11363e+01_rb,0.10383e+01_rb,0.84713e+00_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.62079e+00_rb,0.89078e+00_rb,0.98349e+00_rb,0.10275e+01_rb,0.10413e+01_rb, &
        & 0.10318e+01_rb,0.99528e+00_rb,0.92037e+00_rb,0.72464e+00_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.65800e+00_rb,0.91114e+00_rb,0.99789e+00_rb,0.10381e+01_rb,0.10488e+01_rb, &
        & 0.10378e+01_rb,0.99952e+00_rb,0.92364e+00_rb,0.73140e+00_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.69092e+00_rb,0.93058e+00_rb,0.10122e+01_rb,0.10502e+01_rb,0.10590e+01_rb, &
        & 0.10454e+01_rb,0.10054e+01_rb,0.92699e+00_rb,0.73764e+00_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.71970e+00_rb,0.94840e+00_rb,0.10270e+01_rb,0.10636e+01_rb,0.10709e+01_rb, &
        & 0.10547e+01_rb,0.10116e+01_rb,0.93021e+00_rb,0.74329e+00_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.74579e+00_rb,0.96581e+00_rb,0.10421e+01_rb,0.10781e+01_rb,0.10844e+01_rb, &
        & 0.10644e+01_rb,0.10185e+01_rb,0.93369e+00_rb,0.74789e+00_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.52823e+00_rb,0.77783e+00_rb,0.86297e+00_rb,0.90265e+00_rb,0.91464e+00_rb, &
        & 0.90839e+00_rb,0.87923e+00_rb,0.81632e+00_rb,0.63067e+00_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.56021e+00_rb,0.79652e+00_rb,0.87677e+00_rb,0.91375e+00_rb,0.92363e+00_rb, &
        & 0.91571e+00_rb,0.88529e+00_rb,0.82096e+00_rb,0.63740e+00_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.58848e+00_rb,0.81389e+00_rb,0.89039e+00_rb,0.92584e+00_rb,0.93460e+00_rb, &
        & 0.92486e+00_rb,0.89232e+00_rb,0.82527e+00_rb,0.64377e+00_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.61369e+00_rb,0.83008e+00_rb,0.90459e+00_rb,0.93860e+00_rb,0.94727e+00_rb, &
        & 0.93485e+00_rb,0.89989e+00_rb,0.82959e+00_rb,0.64921e+00_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.63687e+00_rb,0.84580e+00_rb,0.91897e+00_rb,0.95268e+00_rb,0.96085e+00_rb, &
        & 0.94565e+00_rb,0.90780e+00_rb,0.83380e+00_rb,0.65367e+00_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.44508e+00_rb,0.67351e+00_rb,0.75154e+00_rb,0.78756e+00_rb,0.79977e+00_rb, &
        & 0.79563e+00_rb,0.77123e+00_rb,0.71868e+00_rb,0.54794e+00_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.47271e+00_rb,0.69026e+00_rb,0.76424e+00_rb,0.79840e+00_rb,0.80928e+00_rb, &
        & 0.80400e+00_rb,0.77859e+00_rb,0.72416e+00_rb,0.55496e+00_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.49708e+00_rb,0.70580e+00_rb,0.77670e+00_rb,0.80989e+00_rb,0.82043e+00_rb, &
        & 0.81421e+00_rb,0.78653e+00_rb,0.72918e+00_rb,0.56105e+00_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.51921e+00_rb,0.72040e+00_rb,0.78970e+00_rb,0.82209e+00_rb,0.83296e+00_rb, &
        & 0.82477e+00_rb,0.79492e+00_rb,0.73435e+00_rb,0.56612e+00_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.53991e+00_rb,0.73468e+00_rb,0.80284e+00_rb,0.83540e+00_rb,0.84599e+00_rb, &
        & 0.83588e+00_rb,0.80349e+00_rb,0.73920e+00_rb,0.57048e+00_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.37413e+00_rb,0.58022e+00_rb,0.65026e+00_rb,0.68370e+00_rb,0.69632e+00_rb, &
        & 0.69319e+00_rb,0.67239e+00_rb,0.62755e+00_rb,0.47517e+00_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.39785e+00_rb,0.59500e+00_rb,0.66160e+00_rb,0.69388e+00_rb,0.70581e+00_rb, &
        & 0.70224e+00_rb,0.68037e+00_rb,0.63372e+00_rb,0.48187e+00_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.41894e+00_rb,0.60858e+00_rb,0.67282e+00_rb,0.70454e+00_rb,0.71693e+00_rb, &
        & 0.71249e+00_rb,0.68889e+00_rb,0.63962e+00_rb,0.48765e+00_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.43810e+00_rb,0.62138e+00_rb,0.68416e+00_rb,0.71639e+00_rb,0.72873e+00_rb, &
        & 0.72302e+00_rb,0.69785e+00_rb,0.64510e+00_rb,0.49243e+00_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.45627e+00_rb,0.63437e+00_rb,0.69610e+00_rb,0.72855e+00_rb,0.74096e+00_rb, &
        & 0.73385e+00_rb,0.70672e+00_rb,0.65052e+00_rb,0.49657e+00_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.31382e+00_rb,0.49799e+00_rb,0.55977e+00_rb,0.58971e+00_rb,0.60230e+00_rb, &
        & 0.60065e+00_rb,0.58267e+00_rb,0.54392e+00_rb,0.40911e+00_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.33427e+00_rb,0.51077e+00_rb,0.56976e+00_rb,0.59903e+00_rb,0.61148e+00_rb, &
        & 0.60961e+00_rb,0.59093e+00_rb,0.55057e+00_rb,0.41554e+00_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.35247e+00_rb,0.52249e+00_rb,0.57948e+00_rb,0.60895e+00_rb,0.62185e+00_rb, &
        & 0.61942e+00_rb,0.59984e+00_rb,0.55707e+00_rb,0.42096e+00_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.36909e+00_rb,0.53360e+00_rb,0.58950e+00_rb,0.61977e+00_rb,0.63265e+00_rb, &
        & 0.62957e+00_rb,0.60872e+00_rb,0.56306e+00_rb,0.42549e+00_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.38489e+00_rb,0.54511e+00_rb,0.60025e+00_rb,0.63086e+00_rb,0.64395e+00_rb, &
        & 0.63979e+00_rb,0.61739e+00_rb,0.56897e+00_rb,0.42951e+00_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.26224e+00_rb,0.42663e+00_rb,0.48031e+00_rb,0.50651e+00_rb,0.51797e+00_rb, &
        & 0.51738e+00_rb,0.50230e+00_rb,0.46824e+00_rb,0.34967e+00_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.28013e+00_rb,0.43764e+00_rb,0.48906e+00_rb,0.51485e+00_rb,0.52656e+00_rb, &
        & 0.52602e+00_rb,0.51065e+00_rb,0.47517e+00_rb,0.35569e+00_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.29592e+00_rb,0.44770e+00_rb,0.49758e+00_rb,0.52367e+00_rb,0.53583e+00_rb, &
        & 0.53525e+00_rb,0.51944e+00_rb,0.48175e+00_rb,0.36089e+00_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.31035e+00_rb,0.45721e+00_rb,0.50658e+00_rb,0.53311e+00_rb,0.54562e+00_rb, &
        & 0.54468e+00_rb,0.52799e+00_rb,0.48800e+00_rb,0.36523e+00_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.32410e+00_rb,0.46702e+00_rb,0.51628e+00_rb,0.54315e+00_rb,0.55572e+00_rb, &
        & 0.55402e+00_rb,0.53619e+00_rb,0.49402e+00_rb,0.36906e+00_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.21802e+00_rb,0.36523e+00_rb,0.41109e+00_rb,0.43388e+00_rb,0.44363e+00_rb, &
        & 0.44316e+00_rb,0.43058e+00_rb,0.40065e+00_rb,0.29668e+00_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.23315e+00_rb,0.37455e+00_rb,0.41894e+00_rb,0.44116e+00_rb,0.45154e+00_rb, &
        & 0.45134e+00_rb,0.43880e+00_rb,0.40751e+00_rb,0.30231e+00_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.24686e+00_rb,0.38301e+00_rb,0.42637e+00_rb,0.44889e+00_rb,0.45978e+00_rb, &
        & 0.45969e+00_rb,0.44700e+00_rb,0.41408e+00_rb,0.30721e+00_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.25967e+00_rb,0.39102e+00_rb,0.43427e+00_rb,0.45708e+00_rb,0.46833e+00_rb, &
        & 0.46819e+00_rb,0.45488e+00_rb,0.42031e+00_rb,0.31131e+00_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.27188e+00_rb,0.39949e+00_rb,0.44272e+00_rb,0.46608e+00_rb,0.47706e+00_rb, &
        & 0.47653e+00_rb,0.46230e+00_rb,0.42618e+00_rb,0.31504e+00_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.18022e+00_rb,0.31183e+00_rb,0.35120e+00_rb,0.37079e+00_rb,0.37884e+00_rb, &
        & 0.37804e+00_rb,0.36693e+00_rb,0.34063e+00_rb,0.24612e+00_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.19307e+00_rb,0.31966e+00_rb,0.35801e+00_rb,0.37712e+00_rb,0.38585e+00_rb, &
        & 0.38558e+00_rb,0.37468e+00_rb,0.34716e+00_rb,0.25138e+00_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.20463e+00_rb,0.32683e+00_rb,0.36444e+00_rb,0.38387e+00_rb,0.39308e+00_rb, &
        & 0.39294e+00_rb,0.38210e+00_rb,0.35364e+00_rb,0.25593e+00_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.21550e+00_rb,0.33367e+00_rb,0.37119e+00_rb,0.39103e+00_rb,0.40042e+00_rb, &
        & 0.40039e+00_rb,0.38916e+00_rb,0.35964e+00_rb,0.25996e+00_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.22607e+00_rb,0.34108e+00_rb,0.37860e+00_rb,0.39875e+00_rb,0.40805e+00_rb, &
        & 0.40762e+00_rb,0.39578e+00_rb,0.36518e+00_rb,0.26360e+00_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.14867e+00_rb,0.26554e+00_rb,0.29985e+00_rb,0.31657e+00_rb,0.32318e+00_rb, &
        & 0.32201e+00_rb,0.31162e+00_rb,0.28815e+00_rb,0.21732e+00_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.15961e+00_rb,0.27217e+00_rb,0.30555e+00_rb,0.32206e+00_rb,0.32938e+00_rb, &
        & 0.32864e+00_rb,0.31877e+00_rb,0.29424e+00_rb,0.22230e+00_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.16940e+00_rb,0.27829e+00_rb,0.31116e+00_rb,0.32786e+00_rb,0.33559e+00_rb, &
        & 0.33509e+00_rb,0.32534e+00_rb,0.30046e+00_rb,0.22643e+00_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.17863e+00_rb,0.28410e+00_rb,0.31692e+00_rb,0.33409e+00_rb,0.34189e+00_rb, &
        & 0.34154e+00_rb,0.33152e+00_rb,0.30607e+00_rb,0.23022e+00_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.18774e+00_rb,0.29050e+00_rb,0.32341e+00_rb,0.34075e+00_rb,0.34846e+00_rb, &
        & 0.34771e+00_rb,0.33731e+00_rb,0.31120e+00_rb,0.23340e+00_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.12627e+00_rb,0.22669e+00_rb,0.25719e+00_rb,0.27172e+00_rb,0.27732e+00_rb, &
        & 0.27601e+00_rb,0.26659e+00_rb,0.24512e+00_rb,0.18584e+00_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.13516e+00_rb,0.23214e+00_rb,0.26193e+00_rb,0.27653e+00_rb,0.28265e+00_rb, &
        & 0.28160e+00_rb,0.27265e+00_rb,0.25095e+00_rb,0.18968e+00_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.14321e+00_rb,0.23722e+00_rb,0.26679e+00_rb,0.28162e+00_rb,0.28797e+00_rb, &
        & 0.28724e+00_rb,0.27822e+00_rb,0.25645e+00_rb,0.19293e+00_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.15107e+00_rb,0.24247e+00_rb,0.27199e+00_rb,0.28706e+00_rb,0.29346e+00_rb, &
        & 0.29269e+00_rb,0.28343e+00_rb,0.26131e+00_rb,0.19541e+00_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.15896e+00_rb,0.24837e+00_rb,0.27786e+00_rb,0.29309e+00_rb,0.29929e+00_rb, &
        & 0.29804e+00_rb,0.28838e+00_rb,0.26594e+00_rb,0.19802e+00_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.10682e+00_rb,0.19256e+00_rb,0.21863e+00_rb,0.23192e+00_rb,0.23695e+00_rb, &
        & 0.23560e+00_rb,0.22744e+00_rb,0.20844e+00_rb,0.16014e+00_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.11409e+00_rb,0.19699e+00_rb,0.22274e+00_rb,0.23606e+00_rb,0.24149e+00_rb, &
        & 0.24040e+00_rb,0.23256e+00_rb,0.21367e+00_rb,0.16300e+00_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.12093e+00_rb,0.20128e+00_rb,0.22702e+00_rb,0.24050e+00_rb,0.24609e+00_rb, &
        & 0.24521e+00_rb,0.23721e+00_rb,0.21831e+00_rb,0.16579e+00_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.12767e+00_rb,0.20601e+00_rb,0.23174e+00_rb,0.24542e+00_rb,0.25091e+00_rb, &
        & 0.24990e+00_rb,0.24163e+00_rb,0.22258e+00_rb,0.16866e+00_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.13446e+00_rb,0.21151e+00_rb,0.23730e+00_rb,0.25089e+00_rb,0.25608e+00_rb, &
        & 0.25469e+00_rb,0.24607e+00_rb,0.22668e+00_rb,0.17141e+00_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.90556e-01_rb,0.16301e+00_rb,0.18506e+00_rb,0.19633e+00_rb,0.20124e+00_rb, &
        & 0.20042e+00_rb,0.19353e+00_rb,0.17713e+00_rb,0.13589e+00_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.96629e-01_rb,0.16667e+00_rb,0.18855e+00_rb,0.19991e+00_rb,0.20512e+00_rb, &
        & 0.20458e+00_rb,0.19781e+00_rb,0.18150e+00_rb,0.13866e+00_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.10246e+00_rb,0.17045e+00_rb,0.19232e+00_rb,0.20386e+00_rb,0.20916e+00_rb, &
        & 0.20868e+00_rb,0.20175e+00_rb,0.18546e+00_rb,0.14157e+00_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.10831e+00_rb,0.17481e+00_rb,0.19664e+00_rb,0.20827e+00_rb,0.21340e+00_rb, &
        & 0.21278e+00_rb,0.20565e+00_rb,0.18920e+00_rb,0.14421e+00_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.11418e+00_rb,0.17993e+00_rb,0.20190e+00_rb,0.21347e+00_rb,0.21830e+00_rb, &
        & 0.21718e+00_rb,0.20964e+00_rb,0.19280e+00_rb,0.14715e+00_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.20844e+01_rb,0.23344e+01_rb,0.25576e+01_rb,0.26701e+01_rb,0.27010e+01_rb, &
        & 0.26681e+01_rb,0.25683e+01_rb,0.23655e+01_rb,0.20810e+01_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.20807e+01_rb,0.23304e+01_rb,0.25489e+01_rb,0.26611e+01_rb,0.26920e+01_rb, &
        & 0.26609e+01_rb,0.25647e+01_rb,0.23645e+01_rb,0.20859e+01_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.20877e+01_rb,0.23342e+01_rb,0.25477e+01_rb,0.26548e+01_rb,0.26843e+01_rb, &
        & 0.26541e+01_rb,0.25605e+01_rb,0.23637e+01_rb,0.20903e+01_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.21052e+01_rb,0.23454e+01_rb,0.25520e+01_rb,0.26529e+01_rb,0.26778e+01_rb, &
        & 0.26478e+01_rb,0.25565e+01_rb,0.23630e+01_rb,0.20935e+01_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.21303e+01_rb,0.23633e+01_rb,0.25619e+01_rb,0.26546e+01_rb,0.26749e+01_rb, &
        & 0.26429e+01_rb,0.25542e+01_rb,0.23619e+01_rb,0.20958e+01_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.18120e+01_rb,0.21101e+01_rb,0.23204e+01_rb,0.24266e+01_rb,0.24657e+01_rb, &
        & 0.24469e+01_rb,0.23702e+01_rb,0.21931e+01_rb,0.18669e+01_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.18187e+01_rb,0.21137e+01_rb,0.23188e+01_rb,0.24223e+01_rb,0.24620e+01_rb, &
        & 0.24436e+01_rb,0.23687e+01_rb,0.21944e+01_rb,0.18726e+01_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.18356e+01_rb,0.21255e+01_rb,0.23256e+01_rb,0.24232e+01_rb,0.24590e+01_rb, &
        & 0.24407e+01_rb,0.23674e+01_rb,0.21960e+01_rb,0.18776e+01_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.18614e+01_rb,0.21447e+01_rb,0.23386e+01_rb,0.24287e+01_rb,0.24595e+01_rb, &
        & 0.24387e+01_rb,0.23672e+01_rb,0.21970e+01_rb,0.18816e+01_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.18927e+01_rb,0.21683e+01_rb,0.23579e+01_rb,0.24392e+01_rb,0.24636e+01_rb, &
        & 0.24407e+01_rb,0.23665e+01_rb,0.21974e+01_rb,0.18864e+01_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.15702e+01_rb,0.18942e+01_rb,0.20837e+01_rb,0.21854e+01_rb,0.22324e+01_rb, &
        & 0.22266e+01_rb,0.21649e+01_rb,0.20134e+01_rb,0.16617e+01_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.15827e+01_rb,0.19038e+01_rb,0.20886e+01_rb,0.21865e+01_rb,0.22329e+01_rb, &
        & 0.22266e+01_rb,0.21649e+01_rb,0.20167e+01_rb,0.16695e+01_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.16046e+01_rb,0.19213e+01_rb,0.21019e+01_rb,0.21942e+01_rb,0.22360e+01_rb, &
        & 0.22271e+01_rb,0.21665e+01_rb,0.20195e+01_rb,0.16762e+01_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.16329e+01_rb,0.19443e+01_rb,0.21213e+01_rb,0.22070e+01_rb,0.22424e+01_rb, &
        & 0.22311e+01_rb,0.21683e+01_rb,0.20214e+01_rb,0.16830e+01_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.16658e+01_rb,0.19717e+01_rb,0.21452e+01_rb,0.22251e+01_rb,0.22530e+01_rb, &
        & 0.22371e+01_rb,0.21708e+01_rb,0.20238e+01_rb,0.16904e+01_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.13564e+01_rb,0.16869e+01_rb,0.18550e+01_rb,0.19537e+01_rb,0.20049e+01_rb, &
        & 0.20085e+01_rb,0.19601e+01_rb,0.18287e+01_rb,0.14756e+01_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.13720e+01_rb,0.17000e+01_rb,0.18655e+01_rb,0.19607e+01_rb,0.20100e+01_rb, &
        & 0.20105e+01_rb,0.19622e+01_rb,0.18339e+01_rb,0.14854e+01_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.13952e+01_rb,0.17204e+01_rb,0.18837e+01_rb,0.19740e+01_rb,0.20182e+01_rb, &
        & 0.20154e+01_rb,0.19654e+01_rb,0.18384e+01_rb,0.14948e+01_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.14229e+01_rb,0.17461e+01_rb,0.19072e+01_rb,0.19926e+01_rb,0.20303e+01_rb, &
        & 0.20234e+01_rb,0.19691e+01_rb,0.18420e+01_rb,0.15040e+01_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.14534e+01_rb,0.17751e+01_rb,0.19345e+01_rb,0.20154e+01_rb,0.20467e+01_rb, &
        & 0.20337e+01_rb,0.19742e+01_rb,0.18465e+01_rb,0.15131e+01_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.11675e+01_rb,0.14906e+01_rb,0.16422e+01_rb,0.17364e+01_rb,0.17875e+01_rb, &
        & 0.17970e+01_rb,0.17582e+01_rb,0.16459e+01_rb,0.13057e+01_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.11839e+01_rb,0.15062e+01_rb,0.16573e+01_rb,0.17483e+01_rb,0.17956e+01_rb, &
        & 0.18020e+01_rb,0.17623e+01_rb,0.16529e+01_rb,0.13176e+01_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.12061e+01_rb,0.15281e+01_rb,0.16790e+01_rb,0.17660e+01_rb,0.18082e+01_rb, &
        & 0.18106e+01_rb,0.17678e+01_rb,0.16587e+01_rb,0.13289e+01_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.12311e+01_rb,0.15549e+01_rb,0.17048e+01_rb,0.17877e+01_rb,0.18250e+01_rb, &
        & 0.18222e+01_rb,0.17742e+01_rb,0.16644e+01_rb,0.13398e+01_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.12582e+01_rb,0.15836e+01_rb,0.17338e+01_rb,0.18131e+01_rb,0.18457e+01_rb, &
        & 0.18359e+01_rb,0.17826e+01_rb,0.16710e+01_rb,0.13507e+01_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.99981e+00_rb,0.13080e+01_rb,0.14477e+01_rb,0.15356e+01_rb,0.15829e+01_rb, &
        & 0.15936e+01_rb,0.15636e+01_rb,0.14683e+01_rb,0.11482e+01_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.10151e+01_rb,0.13251e+01_rb,0.14655e+01_rb,0.15504e+01_rb,0.15941e+01_rb, &
        & 0.16015e+01_rb,0.15698e+01_rb,0.14767e+01_rb,0.11619e+01_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.10344e+01_rb,0.13476e+01_rb,0.14887e+01_rb,0.15702e+01_rb,0.16097e+01_rb, &
        & 0.16130e+01_rb,0.15772e+01_rb,0.14847e+01_rb,0.11743e+01_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.10560e+01_rb,0.13735e+01_rb,0.15155e+01_rb,0.15939e+01_rb,0.16293e+01_rb, &
        & 0.16276e+01_rb,0.15870e+01_rb,0.14924e+01_rb,0.11864e+01_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.10794e+01_rb,0.13996e+01_rb,0.15438e+01_rb,0.16205e+01_rb,0.16521e+01_rb, &
        & 0.16451e+01_rb,0.15991e+01_rb,0.15003e+01_rb,0.11982e+01_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.85239e+00_rb,0.11419e+01_rb,0.12714e+01_rb,0.13507e+01_rb,0.13930e+01_rb, &
        & 0.14039e+01_rb,0.13790e+01_rb,0.12971e+01_rb,0.10034e+01_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.86542e+00_rb,0.11591e+01_rb,0.12898e+01_rb,0.13670e+01_rb,0.14062e+01_rb, &
        & 0.14139e+01_rb,0.13871e+01_rb,0.13072e+01_rb,0.10179e+01_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.88108e+00_rb,0.11809e+01_rb,0.13130e+01_rb,0.13880e+01_rb,0.14241e+01_rb, &
        & 0.14278e+01_rb,0.13975e+01_rb,0.13172e+01_rb,0.10310e+01_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.89900e+00_rb,0.12041e+01_rb,0.13390e+01_rb,0.14123e+01_rb,0.14455e+01_rb, &
        & 0.14453e+01_rb,0.14105e+01_rb,0.13266e+01_rb,0.10434e+01_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.91862e+00_rb,0.12279e+01_rb,0.13649e+01_rb,0.14386e+01_rb,0.14692e+01_rb, &
        & 0.14664e+01_rb,0.14254e+01_rb,0.13367e+01_rb,0.10558e+01_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.72497e+00_rb,0.99201e+00_rb,0.11107e+01_rb,0.11813e+01_rb,0.12195e+01_rb, &
        & 0.12285e+01_rb,0.12058e+01_rb,0.11359e+01_rb,0.87110e+00_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.73517e+00_rb,0.10086e+01_rb,0.11285e+01_rb,0.11988e+01_rb,0.12342e+01_rb, &
        & 0.12405e+01_rb,0.12160e+01_rb,0.11479e+01_rb,0.88568e+00_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.74752e+00_rb,0.10285e+01_rb,0.11507e+01_rb,0.12198e+01_rb,0.12532e+01_rb, &
        & 0.12570e+01_rb,0.12289e+01_rb,0.11589e+01_rb,0.89927e+00_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.76213e+00_rb,0.10489e+01_rb,0.11747e+01_rb,0.12436e+01_rb,0.12757e+01_rb, &
        & 0.12769e+01_rb,0.12445e+01_rb,0.11700e+01_rb,0.91203e+00_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.77854e+00_rb,0.10697e+01_rb,0.11980e+01_rb,0.12679e+01_rb,0.13002e+01_rb, &
        & 0.12994e+01_rb,0.12621e+01_rb,0.11816e+01_rb,0.92392e+00_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.61682e+00_rb,0.85841e+00_rb,0.96403e+00_rb,0.10272e+01_rb,0.10606e+01_rb, &
        & 0.10679e+01_rb,0.10472e+01_rb,0.98715e+00_rb,0.74825e+00_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.62415e+00_rb,0.87401e+00_rb,0.98140e+00_rb,0.10446e+01_rb,0.10763e+01_rb, &
        & 0.10811e+01_rb,0.10593e+01_rb,0.10003e+01_rb,0.76294e+00_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.63387e+00_rb,0.89104e+00_rb,0.10022e+01_rb,0.10651e+01_rb,0.10962e+01_rb, &
        & 0.10997e+01_rb,0.10740e+01_rb,0.10120e+01_rb,0.77659e+00_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.64542e+00_rb,0.90872e+00_rb,0.10230e+01_rb,0.10878e+01_rb,0.11188e+01_rb, &
        & 0.11207e+01_rb,0.10911e+01_rb,0.10243e+01_rb,0.78905e+00_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.65900e+00_rb,0.92674e+00_rb,0.10436e+01_rb,0.11103e+01_rb,0.11424e+01_rb, &
        & 0.11435e+01_rb,0.11102e+01_rb,0.10375e+01_rb,0.80034e+00_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.52547e+00_rb,0.74220e+00_rb,0.83433e+00_rb,0.88890e+00_rb,0.91741e+00_rb, &
        & 0.92336e+00_rb,0.90538e+00_rb,0.85311e+00_rb,0.64696e+00_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.53064e+00_rb,0.75604e+00_rb,0.85107e+00_rb,0.90584e+00_rb,0.93361e+00_rb, &
        & 0.93836e+00_rb,0.91861e+00_rb,0.86626e+00_rb,0.66062e+00_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.53809e+00_rb,0.77037e+00_rb,0.86942e+00_rb,0.92562e+00_rb,0.95409e+00_rb, &
        & 0.95774e+00_rb,0.93423e+00_rb,0.87871e+00_rb,0.67391e+00_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.54734e+00_rb,0.78576e+00_rb,0.88734e+00_rb,0.94634e+00_rb,0.97611e+00_rb, &
        & 0.97890e+00_rb,0.95253e+00_rb,0.89191e+00_rb,0.68526e+00_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.55842e+00_rb,0.80129e+00_rb,0.90542e+00_rb,0.96675e+00_rb,0.99781e+00_rb, &
        & 0.10013e+01_rb,0.97253e+00_rb,0.90597e+00_rb,0.69583e+00_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.44902e+00_rb,0.64641e+00_rb,0.72682e+00_rb,0.77295e+00_rb,0.79679e+00_rb, &
        & 0.80095e+00_rb,0.78478e+00_rb,0.73880e+00_rb,0.57500e+00_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.45343e+00_rb,0.65819e+00_rb,0.74259e+00_rb,0.79030e+00_rb,0.81509e+00_rb, &
        & 0.81866e+00_rb,0.79960e+00_rb,0.75137e+00_rb,0.58784e+00_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.45968e+00_rb,0.67094e+00_rb,0.75812e+00_rb,0.80898e+00_rb,0.83522e+00_rb, &
        & 0.83854e+00_rb,0.81711e+00_rb,0.76461e+00_rb,0.59939e+00_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.46764e+00_rb,0.68418e+00_rb,0.77402e+00_rb,0.82711e+00_rb,0.85518e+00_rb, &
        & 0.85968e+00_rb,0.83634e+00_rb,0.77859e+00_rb,0.60954e+00_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.47750e+00_rb,0.69798e+00_rb,0.79021e+00_rb,0.84534e+00_rb,0.87480e+00_rb, &
        & 0.88011e+00_rb,0.85595e+00_rb,0.79269e+00_rb,0.61896e+00_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.38252e+00_rb,0.56123e+00_rb,0.63221e+00_rb,0.67207e+00_rb,0.69191e+00_rb, &
        & 0.69451e+00_rb,0.67853e+00_rb,0.63671e+00_rb,0.49624e+00_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.38610e+00_rb,0.57152e+00_rb,0.64559e+00_rb,0.68857e+00_rb,0.70992e+00_rb, &
        & 0.71250e+00_rb,0.69440e+00_rb,0.64948e+00_rb,0.50772e+00_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.39135e+00_rb,0.58251e+00_rb,0.65953e+00_rb,0.70459e+00_rb,0.72793e+00_rb, &
        & 0.73188e+00_rb,0.71251e+00_rb,0.66314e+00_rb,0.51771e+00_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.39838e+00_rb,0.59406e+00_rb,0.67357e+00_rb,0.72066e+00_rb,0.74570e+00_rb, &
        & 0.75075e+00_rb,0.73108e+00_rb,0.67696e+00_rb,0.52680e+00_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.40785e+00_rb,0.60628e+00_rb,0.68802e+00_rb,0.73685e+00_rb,0.76342e+00_rb, &
        & 0.76842e+00_rb,0.74857e+00_rb,0.69135e+00_rb,0.53408e+00_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.32297e+00_rb,0.48556e+00_rb,0.54760e+00_rb,0.58350e+00_rb,0.60079e+00_rb, &
        & 0.60116e+00_rb,0.58518e+00_rb,0.54699e+00_rb,0.42276e+00_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.32591e+00_rb,0.49445e+00_rb,0.55963e+00_rb,0.59764e+00_rb,0.61654e+00_rb, &
        & 0.61844e+00_rb,0.60149e+00_rb,0.55973e+00_rb,0.43280e+00_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.33057e+00_rb,0.50405e+00_rb,0.57184e+00_rb,0.61186e+00_rb,0.63239e+00_rb, &
        & 0.63557e+00_rb,0.61864e+00_rb,0.57305e+00_rb,0.44151e+00_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.33705e+00_rb,0.51427e+00_rb,0.58428e+00_rb,0.62614e+00_rb,0.64825e+00_rb, &
        & 0.65175e+00_rb,0.63492e+00_rb,0.58689e+00_rb,0.44893e+00_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.34636e+00_rb,0.52544e+00_rb,0.59740e+00_rb,0.64055e+00_rb,0.66342e+00_rb, &
        & 0.66736e+00_rb,0.65002e+00_rb,0.60055e+00_rb,0.45487e+00_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.72991e+01_rb,0.69722e+01_rb,0.72863e+01_rb,0.75002e+01_rb,0.75688e+01_rb, &
        & 0.74622e+01_rb,0.71020e+01_rb,0.65825e+01_rb,0.69144e+01_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.72210e+01_rb,0.69084e+01_rb,0.72319e+01_rb,0.74567e+01_rb,0.75409e+01_rb, &
        & 0.74459e+01_rb,0.71001e+01_rb,0.65930e+01_rb,0.69213e+01_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.71481e+01_rb,0.68475e+01_rb,0.71760e+01_rb,0.74130e+01_rb,0.75097e+01_rb, &
        & 0.74266e+01_rb,0.70905e+01_rb,0.65992e+01_rb,0.69258e+01_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.70810e+01_rb,0.67891e+01_rb,0.71222e+01_rb,0.73713e+01_rb,0.74785e+01_rb, &
        & 0.74023e+01_rb,0.70766e+01_rb,0.66037e+01_rb,0.69303e+01_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.70205e+01_rb,0.67342e+01_rb,0.70686e+01_rb,0.73258e+01_rb,0.74436e+01_rb, &
        & 0.73740e+01_rb,0.70585e+01_rb,0.66060e+01_rb,0.69339e+01_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.68023e+01_rb,0.66845e+01_rb,0.71435e+01_rb,0.74293e+01_rb,0.75528e+01_rb, &
        & 0.74805e+01_rb,0.71383e+01_rb,0.65087e+01_rb,0.66425e+01_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.67309e+01_rb,0.66228e+01_rb,0.70899e+01_rb,0.73862e+01_rb,0.75243e+01_rb, &
        & 0.74671e+01_rb,0.71358e+01_rb,0.65240e+01_rb,0.66588e+01_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.66680e+01_rb,0.65659e+01_rb,0.70365e+01_rb,0.73443e+01_rb,0.74954e+01_rb, &
        & 0.74486e+01_rb,0.71260e+01_rb,0.65367e+01_rb,0.66745e+01_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.66136e+01_rb,0.65139e+01_rb,0.69841e+01_rb,0.73016e+01_rb,0.74627e+01_rb, &
        & 0.74252e+01_rb,0.71129e+01_rb,0.65455e+01_rb,0.66889e+01_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.65687e+01_rb,0.64706e+01_rb,0.69322e+01_rb,0.72576e+01_rb,0.74266e+01_rb, &
        & 0.73967e+01_rb,0.70986e+01_rb,0.65510e+01_rb,0.67011e+01_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.62554e+01_rb,0.63717e+01_rb,0.69259e+01_rb,0.72750e+01_rb,0.74339e+01_rb, &
        & 0.73812e+01_rb,0.70638e+01_rb,0.63830e+01_rb,0.62738e+01_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.61935e+01_rb,0.63130e+01_rb,0.68741e+01_rb,0.72340e+01_rb,0.74081e+01_rb, &
        & 0.73723e+01_rb,0.70661e+01_rb,0.64055e+01_rb,0.63017e+01_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.61417e+01_rb,0.62609e+01_rb,0.68234e+01_rb,0.71924e+01_rb,0.73793e+01_rb, &
        & 0.73559e+01_rb,0.70613e+01_rb,0.64217e+01_rb,0.63259e+01_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.61027e+01_rb,0.62184e+01_rb,0.67754e+01_rb,0.71518e+01_rb,0.73493e+01_rb, &
        & 0.73341e+01_rb,0.70537e+01_rb,0.64352e+01_rb,0.63471e+01_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.60751e+01_rb,0.61846e+01_rb,0.67335e+01_rb,0.71090e+01_rb,0.73143e+01_rb, &
        & 0.73100e+01_rb,0.70431e+01_rb,0.64459e+01_rb,0.63674e+01_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.56712e+01_rb,0.60174e+01_rb,0.66407e+01_rb,0.70308e+01_rb,0.72054e+01_rb, &
        & 0.71700e+01_rb,0.68768e+01_rb,0.62076e+01_rb,0.58601e+01_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.56177e+01_rb,0.59637e+01_rb,0.65912e+01_rb,0.69914e+01_rb,0.71823e+01_rb, &
        & 0.71644e+01_rb,0.68840e+01_rb,0.62360e+01_rb,0.58965e+01_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.55798e+01_rb,0.59204e+01_rb,0.65459e+01_rb,0.69532e+01_rb,0.71589e+01_rb, &
        & 0.71503e+01_rb,0.68872e+01_rb,0.62571e+01_rb,0.59275e+01_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.55576e+01_rb,0.58874e+01_rb,0.65057e+01_rb,0.69160e+01_rb,0.71305e+01_rb, &
        & 0.71327e+01_rb,0.68860e+01_rb,0.62738e+01_rb,0.59560e+01_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.55509e+01_rb,0.58664e+01_rb,0.64735e+01_rb,0.68799e+01_rb,0.70989e+01_rb, &
        & 0.71129e+01_rb,0.68803e+01_rb,0.62864e+01_rb,0.59817e+01_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.50748e+01_rb,0.56236e+01_rb,0.62897e+01_rb,0.66937e+01_rb,0.68773e+01_rb, &
        & 0.68583e+01_rb,0.65966e+01_rb,0.59739e+01_rb,0.54247e+01_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.50332e+01_rb,0.55772e+01_rb,0.62435e+01_rb,0.66588e+01_rb,0.68602e+01_rb, &
        & 0.68553e+01_rb,0.66099e+01_rb,0.60056e+01_rb,0.54668e+01_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.50094e+01_rb,0.55429e+01_rb,0.62032e+01_rb,0.66264e+01_rb,0.68392e+01_rb, &
        & 0.68457e+01_rb,0.66198e+01_rb,0.60291e+01_rb,0.55024e+01_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.50054e+01_rb,0.55221e+01_rb,0.61728e+01_rb,0.65959e+01_rb,0.68138e+01_rb, &
        & 0.68347e+01_rb,0.66239e+01_rb,0.60504e+01_rb,0.55378e+01_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.50196e+01_rb,0.55140e+01_rb,0.61502e+01_rb,0.65696e+01_rb,0.67880e+01_rb, &
        & 0.68224e+01_rb,0.66232e+01_rb,0.60661e+01_rb,0.55660e+01_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.44929e+01_rb,0.51985e+01_rb,0.58791e+01_rb,0.62757e+01_rb,0.64716e+01_rb, &
        & 0.64669e+01_rb,0.62368e+01_rb,0.56853e+01_rb,0.49792e+01_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.44612e+01_rb,0.51596e+01_rb,0.58385e+01_rb,0.62487e+01_rb,0.64570e+01_rb, &
        & 0.64678e+01_rb,0.62575e+01_rb,0.57196e+01_rb,0.50244e+01_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.44521e+01_rb,0.51348e+01_rb,0.58071e+01_rb,0.62248e+01_rb,0.64383e+01_rb, &
        & 0.64639e+01_rb,0.62747e+01_rb,0.57459e+01_rb,0.50676e+01_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.44651e+01_rb,0.51248e+01_rb,0.57862e+01_rb,0.62029e+01_rb,0.64206e+01_rb, &
        & 0.64594e+01_rb,0.62829e+01_rb,0.57703e+01_rb,0.51061e+01_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.44974e+01_rb,0.51321e+01_rb,0.57769e+01_rb,0.61847e+01_rb,0.64045e+01_rb, &
        & 0.64518e+01_rb,0.62850e+01_rb,0.57880e+01_rb,0.51383e+01_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.39405e+01_rb,0.47599e+01_rb,0.54183e+01_rb,0.58070e+01_rb,0.60002e+01_rb, &
        & 0.60137e+01_rb,0.58231e+01_rb,0.53430e+01_rb,0.45395e+01_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.39200e+01_rb,0.47279e+01_rb,0.53865e+01_rb,0.57866e+01_rb,0.59900e+01_rb, &
        & 0.60178e+01_rb,0.58493e+01_rb,0.53803e+01_rb,0.45897e+01_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.39243e+01_rb,0.47128e+01_rb,0.53651e+01_rb,0.57689e+01_rb,0.59779e+01_rb, &
        & 0.60210e+01_rb,0.58689e+01_rb,0.54093e+01_rb,0.46370e+01_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.39509e+01_rb,0.47158e+01_rb,0.53566e+01_rb,0.57541e+01_rb,0.59686e+01_rb, &
        & 0.60221e+01_rb,0.58794e+01_rb,0.54360e+01_rb,0.46801e+01_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.39997e+01_rb,0.47365e+01_rb,0.53625e+01_rb,0.57480e+01_rb,0.59641e+01_rb, &
        & 0.60191e+01_rb,0.58864e+01_rb,0.54567e+01_rb,0.47154e+01_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.34277e+01_rb,0.43224e+01_rb,0.49312e+01_rb,0.53036e+01_rb,0.54930e+01_rb, &
        & 0.55228e+01_rb,0.53766e+01_rb,0.49651e+01_rb,0.41155e+01_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.34174e+01_rb,0.42976e+01_rb,0.49104e+01_rb,0.52883e+01_rb,0.54862e+01_rb, &
        & 0.55322e+01_rb,0.54048e+01_rb,0.50049e+01_rb,0.41706e+01_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.34328e+01_rb,0.42901e+01_rb,0.49000e+01_rb,0.52782e+01_rb,0.54821e+01_rb, &
        & 0.55409e+01_rb,0.54257e+01_rb,0.50383e+01_rb,0.42218e+01_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.34724e+01_rb,0.43054e+01_rb,0.49030e+01_rb,0.52739e+01_rb,0.54825e+01_rb, &
        & 0.55468e+01_rb,0.54402e+01_rb,0.50678e+01_rb,0.42656e+01_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.35331e+01_rb,0.43394e+01_rb,0.49199e+01_rb,0.52816e+01_rb,0.54871e+01_rb, &
        & 0.55512e+01_rb,0.54505e+01_rb,0.50918e+01_rb,0.43035e+01_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.29643e+01_rb,0.38841e+01_rb,0.44459e+01_rb,0.47885e+01_rb,0.49734e+01_rb, &
        & 0.50221e+01_rb,0.49131e+01_rb,0.45659e+01_rb,0.37057e+01_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.29616e+01_rb,0.38662e+01_rb,0.44298e+01_rb,0.47788e+01_rb,0.49743e+01_rb, &
        & 0.50365e+01_rb,0.49428e+01_rb,0.46088e+01_rb,0.37656e+01_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.29848e+01_rb,0.38693e+01_rb,0.44274e+01_rb,0.47752e+01_rb,0.49768e+01_rb, &
        & 0.50477e+01_rb,0.49663e+01_rb,0.46464e+01_rb,0.38194e+01_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.30329e+01_rb,0.38944e+01_rb,0.44404e+01_rb,0.47824e+01_rb,0.49850e+01_rb, &
        & 0.50582e+01_rb,0.49851e+01_rb,0.46790e+01_rb,0.38645e+01_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.31033e+01_rb,0.39395e+01_rb,0.44689e+01_rb,0.48029e+01_rb,0.49990e+01_rb, &
        & 0.50689e+01_rb,0.50004e+01_rb,0.47030e+01_rb,0.39063e+01_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.25528e+01_rb,0.34541e+01_rb,0.39652e+01_rb,0.42831e+01_rb,0.44659e+01_rb, &
        & 0.45285e+01_rb,0.44509e+01_rb,0.41587e+01_rb,0.32612e+01_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.25575e+01_rb,0.34458e+01_rb,0.39558e+01_rb,0.42802e+01_rb,0.44723e+01_rb, &
        & 0.45451e+01_rb,0.44820e+01_rb,0.42051e+01_rb,0.33240e+01_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.25883e+01_rb,0.34607e+01_rb,0.39625e+01_rb,0.42854e+01_rb,0.44811e+01_rb, &
        & 0.45602e+01_rb,0.45090e+01_rb,0.42460e+01_rb,0.33785e+01_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.26435e+01_rb,0.34963e+01_rb,0.39867e+01_rb,0.43038e+01_rb,0.44956e+01_rb, &
        & 0.45761e+01_rb,0.45312e+01_rb,0.42799e+01_rb,0.34270e+01_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.27203e+01_rb,0.35517e+01_rb,0.40270e+01_rb,0.43344e+01_rb,0.45189e+01_rb, &
        & 0.45941e+01_rb,0.45504e+01_rb,0.43065e+01_rb,0.34705e+01_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.21958e+01_rb,0.30498e+01_rb,0.35059e+01_rb,0.38049e+01_rb,0.39876e+01_rb, &
        & 0.40622e+01_rb,0.40123e+01_rb,0.37729e+01_rb,0.28786e+01_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.22177e+01_rb,0.30598e+01_rb,0.35116e+01_rb,0.38116e+01_rb,0.39989e+01_rb, &
        & 0.40819e+01_rb,0.40442e+01_rb,0.38191e+01_rb,0.29389e+01_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.22647e+01_rb,0.30907e+01_rb,0.35359e+01_rb,0.38306e+01_rb,0.40167e+01_rb, &
        & 0.41020e+01_rb,0.40724e+01_rb,0.38578e+01_rb,0.29915e+01_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.23337e+01_rb,0.31425e+01_rb,0.35763e+01_rb,0.38632e+01_rb,0.40437e+01_rb, &
        & 0.41245e+01_rb,0.40973e+01_rb,0.38902e+01_rb,0.30386e+01_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.24135e+01_rb,0.32120e+01_rb,0.36330e+01_rb,0.39077e+01_rb,0.40797e+01_rb, &
        & 0.41526e+01_rb,0.41204e+01_rb,0.39195e+01_rb,0.30818e+01_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.18956e+01_rb,0.26861e+01_rb,0.30931e+01_rb,0.33674e+01_rb,0.35425e+01_rb, &
        & 0.36210e+01_rb,0.35899e+01_rb,0.33921e+01_rb,0.25824e+01_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.19314e+01_rb,0.27108e+01_rb,0.31141e+01_rb,0.33855e+01_rb,0.35620e+01_rb, &
        & 0.36450e+01_rb,0.36224e+01_rb,0.34362e+01_rb,0.26351e+01_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.19899e+01_rb,0.27565e+01_rb,0.31515e+01_rb,0.34177e+01_rb,0.35909e+01_rb, &
        & 0.36718e+01_rb,0.36517e+01_rb,0.34743e+01_rb,0.26837e+01_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.20599e+01_rb,0.28211e+01_rb,0.32055e+01_rb,0.34628e+01_rb,0.36291e+01_rb, &
        & 0.37038e+01_rb,0.36800e+01_rb,0.35081e+01_rb,0.27286e+01_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.21342e+01_rb,0.28994e+01_rb,0.32752e+01_rb,0.35201e+01_rb,0.36745e+01_rb, &
        & 0.37420e+01_rb,0.37112e+01_rb,0.35392e+01_rb,0.27703e+01_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.16403e+01_rb,0.23651e+01_rb,0.27290e+01_rb,0.29746e+01_rb,0.31326e+01_rb, &
        & 0.32080e+01_rb,0.31891e+01_rb,0.30274e+01_rb,0.23307e+01_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.16863e+01_rb,0.24025e+01_rb,0.27611e+01_rb,0.30044e+01_rb,0.31618e+01_rb, &
        & 0.32369e+01_rb,0.32226e+01_rb,0.30700e+01_rb,0.23815e+01_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.17455e+01_rb,0.24598e+01_rb,0.28102e+01_rb,0.30475e+01_rb,0.32007e+01_rb, &
        & 0.32722e+01_rb,0.32554e+01_rb,0.31073e+01_rb,0.24291e+01_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.18100e+01_rb,0.25311e+01_rb,0.28759e+01_rb,0.31035e+01_rb,0.32472e+01_rb, &
        & 0.33131e+01_rb,0.32901e+01_rb,0.31417e+01_rb,0.24730e+01_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.18736e+01_rb,0.26085e+01_rb,0.29527e+01_rb,0.31711e+01_rb,0.33027e+01_rb, &
        & 0.33589e+01_rb,0.33290e+01_rb,0.31763e+01_rb,0.25126e+01_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.19065e+02_rb,0.16899e+02_rb,0.15643e+02_rb,0.15578e+02_rb,0.15043e+02_rb, &
        & 0.14288e+02_rb,0.13684e+02_rb,0.14703e+02_rb,0.16312e+02_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.18912e+02_rb,0.16761e+02_rb,0.15514e+02_rb,0.15480e+02_rb,0.14973e+02_rb, &
        & 0.14273e+02_rb,0.13717e+02_rb,0.14749e+02_rb,0.16352e+02_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.18728e+02_rb,0.16596e+02_rb,0.15380e+02_rb,0.15367e+02_rb,0.14915e+02_rb, &
        & 0.14240e+02_rb,0.13761e+02_rb,0.14772e+02_rb,0.16373e+02_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.18544e+02_rb,0.16434e+02_rb,0.15250e+02_rb,0.15223e+02_rb,0.14839e+02_rb, &
        & 0.14214e+02_rb,0.13761e+02_rb,0.14749e+02_rb,0.16351e+02_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.18333e+02_rb,0.16256e+02_rb,0.15122e+02_rb,0.15098e+02_rb,0.14749e+02_rb, &
        & 0.14176e+02_rb,0.13735e+02_rb,0.14720e+02_rb,0.16309e+02_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.19359e+02_rb,0.17148e+02_rb,0.16419e+02_rb,0.16424e+02_rb,0.16006e+02_rb, &
        & 0.15375e+02_rb,0.14545e+02_rb,0.15202e+02_rb,0.16997e+02_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.19172e+02_rb,0.16980e+02_rb,0.16267e+02_rb,0.16317e+02_rb,0.15911e+02_rb, &
        & 0.15334e+02_rb,0.14618e+02_rb,0.15283e+02_rb,0.17084e+02_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.18973e+02_rb,0.16804e+02_rb,0.16126e+02_rb,0.16185e+02_rb,0.15827e+02_rb, &
        & 0.15302e+02_rb,0.14664e+02_rb,0.15280e+02_rb,0.17092e+02_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.18751e+02_rb,0.16617e+02_rb,0.15988e+02_rb,0.16055e+02_rb,0.15738e+02_rb, &
        & 0.15272e+02_rb,0.14654e+02_rb,0.15237e+02_rb,0.17035e+02_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.18520e+02_rb,0.16417e+02_rb,0.15855e+02_rb,0.15950e+02_rb,0.15639e+02_rb, &
        & 0.15219e+02_rb,0.14617e+02_rb,0.15192e+02_rb,0.16961e+02_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.19263e+02_rb,0.17075e+02_rb,0.17060e+02_rb,0.17120e+02_rb,0.16883e+02_rb, &
        & 0.16381e+02_rb,0.15414e+02_rb,0.15455e+02_rb,0.17398e+02_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.19059e+02_rb,0.16894e+02_rb,0.16898e+02_rb,0.17000e+02_rb,0.16773e+02_rb, &
        & 0.16323e+02_rb,0.15500e+02_rb,0.15491e+02_rb,0.17437e+02_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.18835e+02_rb,0.16708e+02_rb,0.16745e+02_rb,0.16865e+02_rb,0.16669e+02_rb, &
        & 0.16285e+02_rb,0.15530e+02_rb,0.15496e+02_rb,0.17423e+02_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.18600e+02_rb,0.16513e+02_rb,0.16588e+02_rb,0.16744e+02_rb,0.16560e+02_rb, &
        & 0.16244e+02_rb,0.15515e+02_rb,0.15476e+02_rb,0.17372e+02_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.18358e+02_rb,0.16312e+02_rb,0.16416e+02_rb,0.16627e+02_rb,0.16462e+02_rb, &
        & 0.16172e+02_rb,0.15490e+02_rb,0.15433e+02_rb,0.17293e+02_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.18801e+02_rb,0.16888e+02_rb,0.17351e+02_rb,0.17638e+02_rb,0.17654e+02_rb, &
        & 0.17241e+02_rb,0.16305e+02_rb,0.15458e+02_rb,0.17321e+02_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.18579e+02_rb,0.16692e+02_rb,0.17197e+02_rb,0.17496e+02_rb,0.17520e+02_rb, &
        & 0.17188e+02_rb,0.16384e+02_rb,0.15486e+02_rb,0.17346e+02_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.18342e+02_rb,0.16488e+02_rb,0.17035e+02_rb,0.17349e+02_rb,0.17380e+02_rb, &
        & 0.17158e+02_rb,0.16385e+02_rb,0.15526e+02_rb,0.17367e+02_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.18091e+02_rb,0.16279e+02_rb,0.16863e+02_rb,0.17208e+02_rb,0.17266e+02_rb, &
        & 0.17103e+02_rb,0.16355e+02_rb,0.15560e+02_rb,0.17358e+02_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.17836e+02_rb,0.16059e+02_rb,0.16677e+02_rb,0.17068e+02_rb,0.17166e+02_rb, &
        & 0.17011e+02_rb,0.16312e+02_rb,0.15578e+02_rb,0.17344e+02_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.17961e+02_rb,0.16659e+02_rb,0.17334e+02_rb,0.17976e+02_rb,0.18171e+02_rb, &
        & 0.17832e+02_rb,0.16966e+02_rb,0.15432e+02_rb,0.16855e+02_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.17714e+02_rb,0.16442e+02_rb,0.17178e+02_rb,0.17800e+02_rb,0.18030e+02_rb, &
        & 0.17804e+02_rb,0.17004e+02_rb,0.15487e+02_rb,0.16904e+02_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.17470e+02_rb,0.16231e+02_rb,0.17012e+02_rb,0.17636e+02_rb,0.17902e+02_rb, &
        & 0.17774e+02_rb,0.16959e+02_rb,0.15590e+02_rb,0.16987e+02_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.17221e+02_rb,0.16002e+02_rb,0.16835e+02_rb,0.17477e+02_rb,0.17804e+02_rb, &
        & 0.17680e+02_rb,0.16923e+02_rb,0.15646e+02_rb,0.17004e+02_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.16955e+02_rb,0.15778e+02_rb,0.16648e+02_rb,0.17325e+02_rb,0.17709e+02_rb, &
        & 0.17576e+02_rb,0.16895e+02_rb,0.15664e+02_rb,0.17027e+02_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.16796e+02_rb,0.16241e+02_rb,0.17123e+02_rb,0.18098e+02_rb,0.18345e+02_rb, &
        & 0.18145e+02_rb,0.17297e+02_rb,0.15317e+02_rb,0.16076e+02_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.16561e+02_rb,0.16019e+02_rb,0.16973e+02_rb,0.17901e+02_rb,0.18253e+02_rb, &
        & 0.18123e+02_rb,0.17284e+02_rb,0.15411e+02_rb,0.16194e+02_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.16307e+02_rb,0.15790e+02_rb,0.16801e+02_rb,0.17722e+02_rb,0.18189e+02_rb, &
        & 0.18068e+02_rb,0.17217e+02_rb,0.15556e+02_rb,0.16309e+02_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.16045e+02_rb,0.15571e+02_rb,0.16615e+02_rb,0.17565e+02_rb,0.18113e+02_rb, &
        & 0.17968e+02_rb,0.17201e+02_rb,0.15625e+02_rb,0.16374e+02_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.15792e+02_rb,0.15350e+02_rb,0.16424e+02_rb,0.17437e+02_rb,0.17997e+02_rb, &
        & 0.17869e+02_rb,0.17203e+02_rb,0.15664e+02_rb,0.16420e+02_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.15441e+02_rb,0.15555e+02_rb,0.16836e+02_rb,0.17857e+02_rb,0.18265e+02_rb, &
        & 0.18114e+02_rb,0.17249e+02_rb,0.15143e+02_rb,0.15078e+02_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.15190e+02_rb,0.15341e+02_rb,0.16649e+02_rb,0.17679e+02_rb,0.18211e+02_rb, &
        & 0.18098e+02_rb,0.17215e+02_rb,0.15278e+02_rb,0.15251e+02_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.14946e+02_rb,0.15122e+02_rb,0.16458e+02_rb,0.17534e+02_rb,0.18165e+02_rb, &
        & 0.18029e+02_rb,0.17165e+02_rb,0.15443e+02_rb,0.15389e+02_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.14700e+02_rb,0.14908e+02_rb,0.16252e+02_rb,0.17424e+02_rb,0.18072e+02_rb, &
        & 0.17929e+02_rb,0.17187e+02_rb,0.15518e+02_rb,0.15458e+02_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.14445e+02_rb,0.14710e+02_rb,0.16048e+02_rb,0.17304e+02_rb,0.17932e+02_rb, &
        & 0.17873e+02_rb,0.17206e+02_rb,0.15573e+02_rb,0.15541e+02_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.13963e+02_rb,0.14641e+02_rb,0.16339e+02_rb,0.17287e+02_rb,0.17853e+02_rb, &
        & 0.17712e+02_rb,0.16836e+02_rb,0.14867e+02_rb,0.13958e+02_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.13720e+02_rb,0.14441e+02_rb,0.16121e+02_rb,0.17175e+02_rb,0.17806e+02_rb, &
        & 0.17701e+02_rb,0.16817e+02_rb,0.15013e+02_rb,0.14139e+02_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.13475e+02_rb,0.14253e+02_rb,0.15918e+02_rb,0.17071e+02_rb,0.17741e+02_rb, &
        & 0.17634e+02_rb,0.16820e+02_rb,0.15155e+02_rb,0.14269e+02_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.13235e+02_rb,0.14056e+02_rb,0.15727e+02_rb,0.16983e+02_rb,0.17630e+02_rb, &
        & 0.17564e+02_rb,0.16869e+02_rb,0.15238e+02_rb,0.14383e+02_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.13009e+02_rb,0.13855e+02_rb,0.15556e+02_rb,0.16857e+02_rb,0.17508e+02_rb, &
        & 0.17529e+02_rb,0.16912e+02_rb,0.15302e+02_rb,0.14496e+02_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.12422e+02_rb,0.13743e+02_rb,0.15496e+02_rb,0.16530e+02_rb,0.17068e+02_rb, &
        & 0.16944e+02_rb,0.16119e+02_rb,0.14360e+02_rb,0.12852e+02_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.12195e+02_rb,0.13543e+02_rb,0.15328e+02_rb,0.16447e+02_rb,0.17000e+02_rb, &
        & 0.16934e+02_rb,0.16148e+02_rb,0.14484e+02_rb,0.13012e+02_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.11972e+02_rb,0.13354e+02_rb,0.15163e+02_rb,0.16359e+02_rb,0.16940e+02_rb, &
        & 0.16900e+02_rb,0.16191e+02_rb,0.14599e+02_rb,0.13152e+02_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.11752e+02_rb,0.13166e+02_rb,0.15011e+02_rb,0.16249e+02_rb,0.16842e+02_rb, &
        & 0.16880e+02_rb,0.16254e+02_rb,0.14688e+02_rb,0.13295e+02_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.11542e+02_rb,0.12972e+02_rb,0.14878e+02_rb,0.16107e+02_rb,0.16750e+02_rb, &
        & 0.16884e+02_rb,0.16301e+02_rb,0.14790e+02_rb,0.13417e+02_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.10883e+02_rb,0.12799e+02_rb,0.14471e+02_rb,0.15523e+02_rb,0.15963e+02_rb, &
        & 0.15867e+02_rb,0.15188e+02_rb,0.13687e+02_rb,0.11830e+02_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.10669e+02_rb,0.12615e+02_rb,0.14351e+02_rb,0.15423e+02_rb,0.15920e+02_rb, &
        & 0.15883e+02_rb,0.15251e+02_rb,0.13796e+02_rb,0.11997e+02_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.10461e+02_rb,0.12417e+02_rb,0.14235e+02_rb,0.15340e+02_rb,0.15857e+02_rb, &
        & 0.15885e+02_rb,0.15315e+02_rb,0.13895e+02_rb,0.12154e+02_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.10271e+02_rb,0.12218e+02_rb,0.14109e+02_rb,0.15234e+02_rb,0.15808e+02_rb, &
        & 0.15905e+02_rb,0.15383e+02_rb,0.14000e+02_rb,0.12311e+02_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.10108e+02_rb,0.12035e+02_rb,0.13977e+02_rb,0.15117e+02_rb,0.15764e+02_rb, &
        & 0.15927e+02_rb,0.15450e+02_rb,0.14121e+02_rb,0.12430e+02_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.93334e+01_rb,0.11625e+02_rb,0.13302e+02_rb,0.14226e+02_rb,0.14663e+02_rb, &
        & 0.14671e+02_rb,0.14179e+02_rb,0.12958e+02_rb,0.10939e+02_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.91435e+01_rb,0.11452e+02_rb,0.13192e+02_rb,0.14163e+02_rb,0.14640e+02_rb, &
        & 0.14697e+02_rb,0.14256e+02_rb,0.13072e+02_rb,0.11104e+02_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.89744e+01_rb,0.11286e+02_rb,0.13068e+02_rb,0.14085e+02_rb,0.14619e+02_rb, &
        & 0.14728e+02_rb,0.14326e+02_rb,0.13192e+02_rb,0.11276e+02_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.88386e+01_rb,0.11134e+02_rb,0.12957e+02_rb,0.14008e+02_rb,0.14602e+02_rb, &
        & 0.14761e+02_rb,0.14404e+02_rb,0.13322e+02_rb,0.11423e+02_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.88280e+01_rb,0.11011e+02_rb,0.12844e+02_rb,0.13942e+02_rb,0.14584e+02_rb, &
        & 0.14790e+02_rb,0.14494e+02_rb,0.13437e+02_rb,0.11537e+02_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.79516e+01_rb,0.10418e+02_rb,0.11985e+02_rb,0.12856e+02_rb,0.13326e+02_rb, &
        & 0.13448e+02_rb,0.13142e+02_rb,0.12163e+02_rb,0.97408e+01_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.78004e+01_rb,0.10286e+02_rb,0.11886e+02_rb,0.12812e+02_rb,0.13327e+02_rb, &
        & 0.13488e+02_rb,0.13229e+02_rb,0.12301e+02_rb,0.99474e+01_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.76847e+01_rb,0.10169e+02_rb,0.11799e+02_rb,0.12768e+02_rb,0.13325e+02_rb, &
        & 0.13529e+02_rb,0.13323e+02_rb,0.12442e+02_rb,0.10130e+02_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.76979e+01_rb,0.10072e+02_rb,0.11716e+02_rb,0.12735e+02_rb,0.13318e+02_rb, &
        & 0.13569e+02_rb,0.13422e+02_rb,0.12581e+02_rb,0.10272e+02_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.78582e+01_rb,0.10030e+02_rb,0.11643e+02_rb,0.12707e+02_rb,0.13325e+02_rb, &
        & 0.13613e+02_rb,0.13505e+02_rb,0.12682e+02_rb,0.10410e+02_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.68077e+01_rb,0.92695e+01_rb,0.10647e+02_rb,0.11513e+02_rb,0.12044e+02_rb, &
        & 0.12269e+02_rb,0.12092e+02_rb,0.11295e+02_rb,0.85103e+01_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.67020e+01_rb,0.91675e+01_rb,0.10585e+02_rb,0.11489e+02_rb,0.12056e+02_rb, &
        & 0.12329e+02_rb,0.12198e+02_rb,0.11449e+02_rb,0.87111e+01_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.67136e+01_rb,0.90765e+01_rb,0.10537e+02_rb,0.11473e+02_rb,0.12065e+02_rb, &
        & 0.12379e+02_rb,0.12312e+02_rb,0.11602e+02_rb,0.88718e+01_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.68715e+01_rb,0.90508e+01_rb,0.10490e+02_rb,0.11454e+02_rb,0.12087e+02_rb, &
        & 0.12433e+02_rb,0.12413e+02_rb,0.11728e+02_rb,0.90172e+01_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.71183e+01_rb,0.91466e+01_rb,0.10479e+02_rb,0.11436e+02_rb,0.12120e+02_rb, &
        & 0.12489e+02_rb,0.12478e+02_rb,0.11826e+02_rb,0.91846e+01_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.25290e+02_rb,0.22378e+02_rb,0.19682e+02_rb,0.19092e+02_rb,0.18401e+02_rb, &
        & 0.17330e+02_rb,0.17194e+02_rb,0.19513e+02_rb,0.21546e+02_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.25153e+02_rb,0.22251e+02_rb,0.19574e+02_rb,0.19002e+02_rb,0.18309e+02_rb, &
        & 0.17346e+02_rb,0.17168e+02_rb,0.19469e+02_rb,0.21516e+02_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.24990e+02_rb,0.22108e+02_rb,0.19457e+02_rb,0.18891e+02_rb,0.18197e+02_rb, &
        & 0.17344e+02_rb,0.17153e+02_rb,0.19435e+02_rb,0.21516e+02_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.24785e+02_rb,0.21926e+02_rb,0.19312e+02_rb,0.18774e+02_rb,0.18111e+02_rb, &
        & 0.17300e+02_rb,0.17168e+02_rb,0.19426e+02_rb,0.21524e+02_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.24562e+02_rb,0.21729e+02_rb,0.19145e+02_rb,0.18657e+02_rb,0.18043e+02_rb, &
        & 0.17239e+02_rb,0.17190e+02_rb,0.19400e+02_rb,0.21517e+02_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.26738e+02_rb,0.23635e+02_rb,0.21164e+02_rb,0.20882e+02_rb,0.20133e+02_rb, &
        & 0.18952e+02_rb,0.18391e+02_rb,0.20397e+02_rb,0.22752e+02_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.26579e+02_rb,0.23493e+02_rb,0.21026e+02_rb,0.20770e+02_rb,0.20046e+02_rb, &
        & 0.18924e+02_rb,0.18351e+02_rb,0.20302e+02_rb,0.22673e+02_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.26369e+02_rb,0.23304e+02_rb,0.20847e+02_rb,0.20634e+02_rb,0.19955e+02_rb, &
        & 0.18867e+02_rb,0.18355e+02_rb,0.20274e+02_rb,0.22643e+02_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.26147e+02_rb,0.23108e+02_rb,0.20663e+02_rb,0.20491e+02_rb,0.19870e+02_rb, &
        & 0.18790e+02_rb,0.18365e+02_rb,0.20271e+02_rb,0.22638e+02_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.25876e+02_rb,0.22867e+02_rb,0.20457e+02_rb,0.20308e+02_rb,0.19788e+02_rb, &
        & 0.18723e+02_rb,0.18359e+02_rb,0.20242e+02_rb,0.22612e+02_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.27801e+02_rb,0.24558e+02_rb,0.22625e+02_rb,0.22456e+02_rb,0.21766e+02_rb, &
        & 0.20723e+02_rb,0.19785e+02_rb,0.21193e+02_rb,0.23811e+02_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.27586e+02_rb,0.24366e+02_rb,0.22444e+02_rb,0.22341e+02_rb,0.21670e+02_rb, &
        & 0.20651e+02_rb,0.19732e+02_rb,0.21163e+02_rb,0.23778e+02_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.27362e+02_rb,0.24166e+02_rb,0.22236e+02_rb,0.22200e+02_rb,0.21575e+02_rb, &
        & 0.20557e+02_rb,0.19718e+02_rb,0.21155e+02_rb,0.23783e+02_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.27084e+02_rb,0.23919e+02_rb,0.22013e+02_rb,0.22018e+02_rb,0.21471e+02_rb, &
        & 0.20457e+02_rb,0.19708e+02_rb,0.21136e+02_rb,0.23769e+02_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.26795e+02_rb,0.23666e+02_rb,0.21794e+02_rb,0.21827e+02_rb,0.21355e+02_rb, &
        & 0.20374e+02_rb,0.19655e+02_rb,0.21092e+02_rb,0.23724e+02_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.28327e+02_rb,0.25026e+02_rb,0.23910e+02_rb,0.23720e+02_rb,0.23265e+02_rb, &
        & 0.22396e+02_rb,0.21086e+02_rb,0.21859e+02_rb,0.24649e+02_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.28095e+02_rb,0.24816e+02_rb,0.23683e+02_rb,0.23594e+02_rb,0.23156e+02_rb, &
        & 0.22304e+02_rb,0.21023e+02_rb,0.21873e+02_rb,0.24674e+02_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.27817e+02_rb,0.24569e+02_rb,0.23454e+02_rb,0.23446e+02_rb,0.23031e+02_rb, &
        & 0.22182e+02_rb,0.21022e+02_rb,0.21878e+02_rb,0.24680e+02_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.27525e+02_rb,0.24310e+02_rb,0.23220e+02_rb,0.23268e+02_rb,0.22908e+02_rb, &
        & 0.22090e+02_rb,0.21022e+02_rb,0.21824e+02_rb,0.24620e+02_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.27191e+02_rb,0.24023e+02_rb,0.22996e+02_rb,0.23072e+02_rb,0.22769e+02_rb, &
        & 0.22022e+02_rb,0.20982e+02_rb,0.21737e+02_rb,0.24514e+02_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.28259e+02_rb,0.25014e+02_rb,0.24907e+02_rb,0.24706e+02_rb,0.24642e+02_rb, &
        & 0.23918e+02_rb,0.22330e+02_rb,0.22219e+02_rb,0.25091e+02_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.27993e+02_rb,0.24771e+02_rb,0.24666e+02_rb,0.24596e+02_rb,0.24487e+02_rb, &
        & 0.23818e+02_rb,0.22348e+02_rb,0.22269e+02_rb,0.25149e+02_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.27691e+02_rb,0.24499e+02_rb,0.24421e+02_rb,0.24432e+02_rb,0.24340e+02_rb, &
        & 0.23705e+02_rb,0.22420e+02_rb,0.22242e+02_rb,0.25109e+02_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.27357e+02_rb,0.24214e+02_rb,0.24178e+02_rb,0.24253e+02_rb,0.24191e+02_rb, &
        & 0.23642e+02_rb,0.22422e+02_rb,0.22174e+02_rb,0.25030e+02_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.27008e+02_rb,0.23916e+02_rb,0.23935e+02_rb,0.24063e+02_rb,0.24020e+02_rb, &
        & 0.23572e+02_rb,0.22355e+02_rb,0.22132e+02_rb,0.24957e+02_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.27627e+02_rb,0.24841e+02_rb,0.25414e+02_rb,0.25513e+02_rb,0.25788e+02_rb, &
        & 0.25161e+02_rb,0.23567e+02_rb,0.22384e+02_rb,0.25091e+02_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.27317e+02_rb,0.24550e+02_rb,0.25165e+02_rb,0.25384e+02_rb,0.25578e+02_rb, &
        & 0.25083e+02_rb,0.23662e+02_rb,0.22423e+02_rb,0.25102e+02_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.26994e+02_rb,0.24259e+02_rb,0.24926e+02_rb,0.25206e+02_rb,0.25391e+02_rb, &
        & 0.25015e+02_rb,0.23758e+02_rb,0.22363e+02_rb,0.25025e+02_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.26640e+02_rb,0.23945e+02_rb,0.24692e+02_rb,0.25002e+02_rb,0.25193e+02_rb, &
        & 0.24973e+02_rb,0.23733e+02_rb,0.22328e+02_rb,0.24965e+02_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.26265e+02_rb,0.23625e+02_rb,0.24446e+02_rb,0.24785e+02_rb,0.25037e+02_rb, &
        & 0.24906e+02_rb,0.23659e+02_rb,0.22352e+02_rb,0.24965e+02_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.26430e+02_rb,0.24581e+02_rb,0.25439e+02_rb,0.26140e+02_rb,0.26524e+02_rb, &
        & 0.25974e+02_rb,0.24552e+02_rb,0.22441e+02_rb,0.24498e+02_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.26105e+02_rb,0.24263e+02_rb,0.25225e+02_rb,0.25974e+02_rb,0.26315e+02_rb, &
        & 0.25946e+02_rb,0.24677e+02_rb,0.22471e+02_rb,0.24490e+02_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.25752e+02_rb,0.23938e+02_rb,0.24996e+02_rb,0.25749e+02_rb,0.26113e+02_rb, &
        & 0.25925e+02_rb,0.24758e+02_rb,0.22442e+02_rb,0.24454e+02_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.25389e+02_rb,0.23609e+02_rb,0.24761e+02_rb,0.25499e+02_rb,0.25940e+02_rb, &
        & 0.25913e+02_rb,0.24720e+02_rb,0.22473e+02_rb,0.24475e+02_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.25010e+02_rb,0.23260e+02_rb,0.24510e+02_rb,0.25262e+02_rb,0.25804e+02_rb, &
        & 0.25811e+02_rb,0.24640e+02_rb,0.22547e+02_rb,0.24534e+02_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.24762e+02_rb,0.23997e+02_rb,0.25209e+02_rb,0.26467e+02_rb,0.26748e+02_rb, &
        & 0.26365e+02_rb,0.25094e+02_rb,0.22357e+02_rb,0.23406e+02_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.24421e+02_rb,0.23672e+02_rb,0.24981e+02_rb,0.26217e+02_rb,0.26595e+02_rb, &
        & 0.26375e+02_rb,0.25195e+02_rb,0.22435e+02_rb,0.23453e+02_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.24067e+02_rb,0.23336e+02_rb,0.24754e+02_rb,0.25956e+02_rb,0.26455e+02_rb, &
        & 0.26361e+02_rb,0.25241e+02_rb,0.22458e+02_rb,0.23475e+02_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.23691e+02_rb,0.22996e+02_rb,0.24502e+02_rb,0.25698e+02_rb,0.26330e+02_rb, &
        & 0.26343e+02_rb,0.25197e+02_rb,0.22533e+02_rb,0.23575e+02_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.23311e+02_rb,0.22660e+02_rb,0.24234e+02_rb,0.25450e+02_rb,0.26215e+02_rb, &
        & 0.26209e+02_rb,0.25123e+02_rb,0.22642e+02_rb,0.23709e+02_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.22800e+02_rb,0.23011e+02_rb,0.24799e+02_rb,0.26181e+02_rb,0.26533e+02_rb, &
        & 0.26301e+02_rb,0.25044e+02_rb,0.22077e+02_rb,0.21907e+02_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.22436e+02_rb,0.22697e+02_rb,0.24546e+02_rb,0.25945e+02_rb,0.26460e+02_rb, &
        & 0.26314e+02_rb,0.25122e+02_rb,0.22204e+02_rb,0.22029e+02_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.22079e+02_rb,0.22367e+02_rb,0.24295e+02_rb,0.25706e+02_rb,0.26370e+02_rb, &
        & 0.26288e+02_rb,0.25162e+02_rb,0.22314e+02_rb,0.22151e+02_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.21719e+02_rb,0.22029e+02_rb,0.24017e+02_rb,0.25487e+02_rb,0.26273e+02_rb, &
        & 0.26217e+02_rb,0.25110e+02_rb,0.22435e+02_rb,0.22334e+02_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.21341e+02_rb,0.21707e+02_rb,0.23702e+02_rb,0.25282e+02_rb,0.26159e+02_rb, &
        & 0.26086e+02_rb,0.25065e+02_rb,0.22578e+02_rb,0.22506e+02_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.20601e+02_rb,0.21678e+02_rb,0.24058e+02_rb,0.25352e+02_rb,0.25931e+02_rb, &
        & 0.25718e+02_rb,0.24432e+02_rb,0.21607e+02_rb,0.20262e+02_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.20254e+02_rb,0.21345e+02_rb,0.23796e+02_rb,0.25183e+02_rb,0.25875e+02_rb, &
        & 0.25691e+02_rb,0.24508e+02_rb,0.21774e+02_rb,0.20445e+02_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.19899e+02_rb,0.21048e+02_rb,0.23494e+02_rb,0.25000e+02_rb,0.25800e+02_rb, &
        & 0.25662e+02_rb,0.24533e+02_rb,0.21922e+02_rb,0.20638e+02_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.19544e+02_rb,0.20766e+02_rb,0.23207e+02_rb,0.24826e+02_rb,0.25686e+02_rb, &
        & 0.25593e+02_rb,0.24527e+02_rb,0.22085e+02_rb,0.20832e+02_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.19193e+02_rb,0.20470e+02_rb,0.22936e+02_rb,0.24673e+02_rb,0.25544e+02_rb, &
        & 0.25480e+02_rb,0.24537e+02_rb,0.22218e+02_rb,0.21044e+02_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.18170e+02_rb,0.20155e+02_rb,0.22721e+02_rb,0.24145e+02_rb,0.24798e+02_rb, &
        & 0.24559e+02_rb,0.23403e+02_rb,0.20895e+02_rb,0.18740e+02_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.17830e+02_rb,0.19860e+02_rb,0.22479e+02_rb,0.24018e+02_rb,0.24721e+02_rb, &
        & 0.24556e+02_rb,0.23466e+02_rb,0.21069e+02_rb,0.18949e+02_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.17496e+02_rb,0.19576e+02_rb,0.22242e+02_rb,0.23886e+02_rb,0.24621e+02_rb, &
        & 0.24529e+02_rb,0.23525e+02_rb,0.21231e+02_rb,0.19147e+02_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.17168e+02_rb,0.19292e+02_rb,0.22005e+02_rb,0.23734e+02_rb,0.24514e+02_rb, &
        & 0.24476e+02_rb,0.23580e+02_rb,0.21376e+02_rb,0.19368e+02_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.16844e+02_rb,0.19013e+02_rb,0.21771e+02_rb,0.23570e+02_rb,0.24390e+02_rb, &
        & 0.24440e+02_rb,0.23651e+02_rb,0.21517e+02_rb,0.19612e+02_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.15762e+02_rb,0.18615e+02_rb,0.21133e+02_rb,0.22639e+02_rb,0.23214e+02_rb, &
        & 0.23022e+02_rb,0.22060e+02_rb,0.19958e+02_rb,0.17314e+02_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.15455e+02_rb,0.18341e+02_rb,0.20944e+02_rb,0.22510e+02_rb,0.23148e+02_rb, &
        & 0.23042e+02_rb,0.22164e+02_rb,0.20127e+02_rb,0.17543e+02_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.15156e+02_rb,0.18068e+02_rb,0.20768e+02_rb,0.22374e+02_rb,0.23084e+02_rb, &
        & 0.23060e+02_rb,0.22257e+02_rb,0.20275e+02_rb,0.17770e+02_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.14865e+02_rb,0.17794e+02_rb,0.20582e+02_rb,0.22237e+02_rb,0.23018e+02_rb, &
        & 0.23079e+02_rb,0.22350e+02_rb,0.20427e+02_rb,0.18026e+02_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.14612e+02_rb,0.17534e+02_rb,0.20399e+02_rb,0.22086e+02_rb,0.22956e+02_rb, &
        & 0.23112e+02_rb,0.22443e+02_rb,0.20607e+02_rb,0.18280e+02_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.13517e+02_rb,0.16914e+02_rb,0.19425e+02_rb,0.20790e+02_rb,0.21367e+02_rb, &
        & 0.21315e+02_rb,0.20607e+02_rb,0.18871e+02_rb,0.15967e+02_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.13249e+02_rb,0.16688e+02_rb,0.19279e+02_rb,0.20693e+02_rb,0.21344e+02_rb, &
        & 0.21372e+02_rb,0.20725e+02_rb,0.19043e+02_rb,0.16217e+02_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.12994e+02_rb,0.16467e+02_rb,0.19108e+02_rb,0.20602e+02_rb,0.21343e+02_rb, &
        & 0.21436e+02_rb,0.20839e+02_rb,0.19212e+02_rb,0.16494e+02_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.12784e+02_rb,0.16250e+02_rb,0.18951e+02_rb,0.20506e+02_rb,0.21335e+02_rb, &
        & 0.21503e+02_rb,0.20961e+02_rb,0.19415e+02_rb,0.16770e+02_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.12734e+02_rb,0.16049e+02_rb,0.18819e+02_rb,0.20438e+02_rb,0.21329e+02_rb, &
        & 0.21573e+02_rb,0.21104e+02_rb,0.19623e+02_rb,0.16983e+02_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.32549e+02_rb,0.28774e+02_rb,0.25031e+02_rb,0.22926e+02_rb,0.22117e+02_rb, &
        & 0.21695e+02_rb,0.23417e+02_rb,0.26761e+02_rb,0.29694e+02_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.32499e+02_rb,0.28739e+02_rb,0.25006e+02_rb,0.22835e+02_rb,0.22088e+02_rb, &
        & 0.21560e+02_rb,0.23253e+02_rb,0.26542e+02_rb,0.29377e+02_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.32407e+02_rb,0.28660e+02_rb,0.24923e+02_rb,0.22714e+02_rb,0.22014e+02_rb, &
        & 0.21461e+02_rb,0.23148e+02_rb,0.26398e+02_rb,0.29117e+02_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.32260e+02_rb,0.28512e+02_rb,0.24788e+02_rb,0.22590e+02_rb,0.21881e+02_rb, &
        & 0.21430e+02_rb,0.23071e+02_rb,0.26295e+02_rb,0.28961e+02_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.32039e+02_rb,0.28307e+02_rb,0.24614e+02_rb,0.22444e+02_rb,0.21715e+02_rb, &
        & 0.21428e+02_rb,0.22979e+02_rb,0.26201e+02_rb,0.28821e+02_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.35951e+02_rb,0.31737e+02_rb,0.27558e+02_rb,0.25749e+02_rb,0.24746e+02_rb, &
        & 0.23905e+02_rb,0.24887e+02_rb,0.28459e+02_rb,0.31791e+02_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.35862e+02_rb,0.31672e+02_rb,0.27501e+02_rb,0.25642e+02_rb,0.24725e+02_rb, &
        & 0.23810e+02_rb,0.24734e+02_rb,0.28277e+02_rb,0.31526e+02_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.35737e+02_rb,0.31562e+02_rb,0.27401e+02_rb,0.25522e+02_rb,0.24641e+02_rb, &
        & 0.23751e+02_rb,0.24591e+02_rb,0.28127e+02_rb,0.31321e+02_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.35534e+02_rb,0.31380e+02_rb,0.27248e+02_rb,0.25379e+02_rb,0.24498e+02_rb, &
        & 0.23724e+02_rb,0.24485e+02_rb,0.28006e+02_rb,0.31168e+02_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.35285e+02_rb,0.31161e+02_rb,0.27065e+02_rb,0.25222e+02_rb,0.24328e+02_rb, &
        & 0.23689e+02_rb,0.24406e+02_rb,0.27908e+02_rb,0.31060e+02_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.39089e+02_rb,0.34457e+02_rb,0.30041e+02_rb,0.28724e+02_rb,0.27574e+02_rb, &
        & 0.26213e+02_rb,0.26498e+02_rb,0.30215e+02_rb,0.33960e+02_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.38998e+02_rb,0.34382e+02_rb,0.29952e+02_rb,0.28581e+02_rb,0.27562e+02_rb, &
        & 0.26150e+02_rb,0.26309e+02_rb,0.29980e+02_rb,0.33662e+02_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.38820e+02_rb,0.34217e+02_rb,0.29804e+02_rb,0.28424e+02_rb,0.27473e+02_rb, &
        & 0.26126e+02_rb,0.26176e+02_rb,0.29817e+02_rb,0.33458e+02_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.38594e+02_rb,0.34019e+02_rb,0.29631e+02_rb,0.28264e+02_rb,0.27343e+02_rb, &
        & 0.26103e+02_rb,0.26063e+02_rb,0.29657e+02_rb,0.33303e+02_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.38290e+02_rb,0.33755e+02_rb,0.29411e+02_rb,0.28094e+02_rb,0.27189e+02_rb, &
        & 0.26048e+02_rb,0.25993e+02_rb,0.29534e+02_rb,0.33212e+02_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.41846e+02_rb,0.36848e+02_rb,0.32642e+02_rb,0.31752e+02_rb,0.30348e+02_rb, &
        & 0.28783e+02_rb,0.28429e+02_rb,0.31940e+02_rb,0.36003e+02_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.41706e+02_rb,0.36722e+02_rb,0.32481e+02_rb,0.31589e+02_rb,0.30346e+02_rb, &
        & 0.28731e+02_rb,0.28275e+02_rb,0.31704e+02_rb,0.35716e+02_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.41502e+02_rb,0.36539e+02_rb,0.32269e+02_rb,0.31417e+02_rb,0.30279e+02_rb, &
        & 0.28696e+02_rb,0.28151e+02_rb,0.31470e+02_rb,0.35494e+02_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.41217e+02_rb,0.36286e+02_rb,0.32033e+02_rb,0.31226e+02_rb,0.30155e+02_rb, &
        & 0.28647e+02_rb,0.28039e+02_rb,0.31307e+02_rb,0.35368e+02_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.40876e+02_rb,0.35985e+02_rb,0.31743e+02_rb,0.31019e+02_rb,0.30013e+02_rb, &
        & 0.28553e+02_rb,0.27986e+02_rb,0.31205e+02_rb,0.35294e+02_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.43995e+02_rb,0.38706e+02_rb,0.35200e+02_rb,0.34595e+02_rb,0.33005e+02_rb, &
        & 0.31594e+02_rb,0.30727e+02_rb,0.33547e+02_rb,0.37840e+02_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.43792e+02_rb,0.38521e+02_rb,0.34965e+02_rb,0.34413e+02_rb,0.33017e+02_rb, &
        & 0.31523e+02_rb,0.30579e+02_rb,0.33292e+02_rb,0.37593e+02_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.43530e+02_rb,0.38288e+02_rb,0.34710e+02_rb,0.34215e+02_rb,0.32953e+02_rb, &
        & 0.31452e+02_rb,0.30422e+02_rb,0.33082e+02_rb,0.37437e+02_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.43192e+02_rb,0.37991e+02_rb,0.34400e+02_rb,0.33987e+02_rb,0.32832e+02_rb, &
        & 0.31362e+02_rb,0.30341e+02_rb,0.32970e+02_rb,0.37357e+02_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.42817e+02_rb,0.37661e+02_rb,0.34079e+02_rb,0.33725e+02_rb,0.32688e+02_rb, &
        & 0.31220e+02_rb,0.30292e+02_rb,0.32861e+02_rb,0.37270e+02_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.45371e+02_rb,0.39886e+02_rb,0.37563e+02_rb,0.36985e+02_rb,0.35582e+02_rb, &
        & 0.34395e+02_rb,0.32997e+02_rb,0.34724e+02_rb,0.39202e+02_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.45116e+02_rb,0.39656e+02_rb,0.37269e+02_rb,0.36794e+02_rb,0.35596e+02_rb, &
        & 0.34289e+02_rb,0.32827e+02_rb,0.34552e+02_rb,0.39098e+02_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.44794e+02_rb,0.39373e+02_rb,0.36959e+02_rb,0.36575e+02_rb,0.35485e+02_rb, &
        & 0.34189e+02_rb,0.32665e+02_rb,0.34424e+02_rb,0.39024e+02_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.44425e+02_rb,0.39049e+02_rb,0.36596e+02_rb,0.36352e+02_rb,0.35337e+02_rb, &
        & 0.34032e+02_rb,0.32591e+02_rb,0.34348e+02_rb,0.38980e+02_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.43987e+02_rb,0.38666e+02_rb,0.36218e+02_rb,0.36087e+02_rb,0.35122e+02_rb, &
        & 0.33853e+02_rb,0.32536e+02_rb,0.34232e+02_rb,0.38883e+02_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.45829e+02_rb,0.40296e+02_rb,0.39507e+02_rb,0.38911e+02_rb,0.38052e+02_rb, &
        & 0.37051e+02_rb,0.34926e+02_rb,0.35286e+02_rb,0.39874e+02_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.45523e+02_rb,0.40015e+02_rb,0.39164e+02_rb,0.38712e+02_rb,0.37995e+02_rb, &
        & 0.36879e+02_rb,0.34780e+02_rb,0.35237e+02_rb,0.39916e+02_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.45143e+02_rb,0.39672e+02_rb,0.38788e+02_rb,0.38484e+02_rb,0.37843e+02_rb, &
        & 0.36693e+02_rb,0.34674e+02_rb,0.35202e+02_rb,0.39935e+02_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.44711e+02_rb,0.39294e+02_rb,0.38380e+02_rb,0.38261e+02_rb,0.37608e+02_rb, &
        & 0.36475e+02_rb,0.34639e+02_rb,0.35181e+02_rb,0.39952e+02_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.44229e+02_rb,0.38875e+02_rb,0.37956e+02_rb,0.37973e+02_rb,0.37347e+02_rb, &
        & 0.36293e+02_rb,0.34620e+02_rb,0.35062e+02_rb,0.39846e+02_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.45335e+02_rb,0.40267e+02_rb,0.40744e+02_rb,0.40443e+02_rb,0.40247e+02_rb, &
        & 0.39223e+02_rb,0.36584e+02_rb,0.35367e+02_rb,0.39844e+02_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.44955e+02_rb,0.39893e+02_rb,0.40398e+02_rb,0.40216e+02_rb,0.40115e+02_rb, &
        & 0.39023e+02_rb,0.36582e+02_rb,0.35440e+02_rb,0.40008e+02_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.44529e+02_rb,0.39492e+02_rb,0.39985e+02_rb,0.39979e+02_rb,0.39873e+02_rb, &
        & 0.38831e+02_rb,0.36587e+02_rb,0.35498e+02_rb,0.40121e+02_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.44061e+02_rb,0.39061e+02_rb,0.39564e+02_rb,0.39715e+02_rb,0.39565e+02_rb, &
        & 0.38623e+02_rb,0.36619e+02_rb,0.35513e+02_rb,0.40138e+02_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.43543e+02_rb,0.38605e+02_rb,0.39136e+02_rb,0.39437e+02_rb,0.39265e+02_rb, &
        & 0.38474e+02_rb,0.36637e+02_rb,0.35396e+02_rb,0.40011e+02_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.43909e+02_rb,0.39954e+02_rb,0.41167e+02_rb,0.41626e+02_rb,0.41945e+02_rb, &
        & 0.40736e+02_rb,0.38077e+02_rb,0.35393e+02_rb,0.39221e+02_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.43480e+02_rb,0.39498e+02_rb,0.40786e+02_rb,0.41338e+02_rb,0.41733e+02_rb, &
        & 0.40609e+02_rb,0.38181e+02_rb,0.35523e+02_rb,0.39397e+02_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.43002e+02_rb,0.39038e+02_rb,0.40373e+02_rb,0.41094e+02_rb,0.41410e+02_rb, &
        & 0.40471e+02_rb,0.38255e+02_rb,0.35563e+02_rb,0.39462e+02_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.42479e+02_rb,0.38560e+02_rb,0.39995e+02_rb,0.40806e+02_rb,0.41101e+02_rb, &
        & 0.40341e+02_rb,0.38344e+02_rb,0.35573e+02_rb,0.39457e+02_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.41942e+02_rb,0.38060e+02_rb,0.39621e+02_rb,0.40473e+02_rb,0.40785e+02_rb, &
        & 0.40226e+02_rb,0.38369e+02_rb,0.35494e+02_rb,0.39400e+02_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.41598e+02_rb,0.39277e+02_rb,0.40964e+02_rb,0.42358e+02_rb,0.42727e+02_rb, &
        & 0.41605e+02_rb,0.39147e+02_rb,0.35243e+02_rb,0.37755e+02_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.41110e+02_rb,0.38804e+02_rb,0.40573e+02_rb,0.42061e+02_rb,0.42547e+02_rb, &
        & 0.41580e+02_rb,0.39298e+02_rb,0.35367e+02_rb,0.37893e+02_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.40607e+02_rb,0.38301e+02_rb,0.40218e+02_rb,0.41774e+02_rb,0.42273e+02_rb, &
        & 0.41520e+02_rb,0.39406e+02_rb,0.35478e+02_rb,0.38009e+02_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.40087e+02_rb,0.37793e+02_rb,0.39835e+02_rb,0.41418e+02_rb,0.42009e+02_rb, &
        & 0.41467e+02_rb,0.39471e+02_rb,0.35532e+02_rb,0.38110e+02_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.39528e+02_rb,0.37276e+02_rb,0.39445e+02_rb,0.41009e+02_rb,0.41755e+02_rb, &
        & 0.41409e+02_rb,0.39460e+02_rb,0.35536e+02_rb,0.38153e+02_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.38423e+02_rb,0.37785e+02_rb,0.40246e+02_rb,0.42229e+02_rb,0.42700e+02_rb, &
        & 0.41861e+02_rb,0.39594e+02_rb,0.34991e+02_rb,0.35642e+02_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.37903e+02_rb,0.37296e+02_rb,0.39841e+02_rb,0.41927e+02_rb,0.42539e+02_rb, &
        & 0.41882e+02_rb,0.39751e+02_rb,0.35170e+02_rb,0.35883e+02_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.37383e+02_rb,0.36809e+02_rb,0.39442e+02_rb,0.41575e+02_rb,0.42342e+02_rb, &
        & 0.41911e+02_rb,0.39809e+02_rb,0.35335e+02_rb,0.36101e+02_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.36841e+02_rb,0.36301e+02_rb,0.39055e+02_rb,0.41199e+02_rb,0.42145e+02_rb, &
        & 0.41876e+02_rb,0.39854e+02_rb,0.35442e+02_rb,0.36276e+02_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.36274e+02_rb,0.35783e+02_rb,0.38676e+02_rb,0.40833e+02_rb,0.41945e+02_rb, &
        & 0.41808e+02_rb,0.39839e+02_rb,0.35539e+02_rb,0.36455e+02_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.34828e+02_rb,0.35770e+02_rb,0.39099e+02_rb,0.41221e+02_rb,0.41958e+02_rb, &
        & 0.41456e+02_rb,0.39253e+02_rb,0.34461e+02_rb,0.33246e+02_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.34304e+02_rb,0.35295e+02_rb,0.38700e+02_rb,0.40987e+02_rb,0.41846e+02_rb, &
        & 0.41525e+02_rb,0.39359e+02_rb,0.34698e+02_rb,0.33560e+02_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.33784e+02_rb,0.34798e+02_rb,0.38311e+02_rb,0.40707e+02_rb,0.41743e+02_rb, &
        & 0.41505e+02_rb,0.39465e+02_rb,0.34925e+02_rb,0.33882e+02_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.33241e+02_rb,0.34324e+02_rb,0.37917e+02_rb,0.40393e+02_rb,0.41632e+02_rb, &
        & 0.41461e+02_rb,0.39554e+02_rb,0.35117e+02_rb,0.34172e+02_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.32710e+02_rb,0.33867e+02_rb,0.37489e+02_rb,0.40112e+02_rb,0.41490e+02_rb, &
        & 0.41392e+02_rb,0.39590e+02_rb,0.35312e+02_rb,0.34501e+02_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.30997e+02_rb,0.33431e+02_rb,0.37385e+02_rb,0.39653e+02_rb,0.40610e+02_rb, &
        & 0.40222e+02_rb,0.38101e+02_rb,0.33638e+02_rb,0.30845e+02_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.30492e+02_rb,0.32952e+02_rb,0.37042e+02_rb,0.39451e+02_rb,0.40583e+02_rb, &
        & 0.40257e+02_rb,0.38272e+02_rb,0.33940e+02_rb,0.31245e+02_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.29985e+02_rb,0.32520e+02_rb,0.36699e+02_rb,0.39235e+02_rb,0.40521e+02_rb, &
        & 0.40274e+02_rb,0.38447e+02_rb,0.34242e+02_rb,0.31622e+02_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.29485e+02_rb,0.32102e+02_rb,0.36327e+02_rb,0.39047e+02_rb,0.40416e+02_rb, &
        & 0.40299e+02_rb,0.38596e+02_rb,0.34522e+02_rb,0.32000e+02_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.29016e+02_rb,0.31685e+02_rb,0.35962e+02_rb,0.38852e+02_rb,0.40311e+02_rb, &
        & 0.40331e+02_rb,0.38749e+02_rb,0.34802e+02_rb,0.32433e+02_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.40414e+02_rb,0.35530e+02_rb,0.30727e+02_rb,0.26813e+02_rb,0.26498e+02_rb, &
        & 0.27724e+02_rb,0.32451e+02_rb,0.37296e+02_rb,0.40920e+02_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.40378e+02_rb,0.35506e+02_rb,0.30718e+02_rb,0.26787e+02_rb,0.26360e+02_rb, &
        & 0.27557e+02_rb,0.32109e+02_rb,0.36904e+02_rb,0.40512e+02_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.40288e+02_rb,0.35435e+02_rb,0.30689e+02_rb,0.26758e+02_rb,0.26294e+02_rb, &
        & 0.27365e+02_rb,0.31721e+02_rb,0.36448e+02_rb,0.40077e+02_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.40152e+02_rb,0.35337e+02_rb,0.30607e+02_rb,0.26666e+02_rb,0.26235e+02_rb, &
        & 0.27137e+02_rb,0.31350e+02_rb,0.36026e+02_rb,0.39605e+02_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.39996e+02_rb,0.35209e+02_rb,0.30500e+02_rb,0.26544e+02_rb,0.26176e+02_rb, &
        & 0.26906e+02_rb,0.31054e+02_rb,0.35676e+02_rb,0.39227e+02_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.46201e+02_rb,0.40631e+02_rb,0.35149e+02_rb,0.31004e+02_rb,0.30338e+02_rb, &
        & 0.30975e+02_rb,0.35648e+02_rb,0.41032e+02_rb,0.45728e+02_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.46189e+02_rb,0.40616e+02_rb,0.35151e+02_rb,0.30920e+02_rb,0.30181e+02_rb, &
        & 0.30854e+02_rb,0.35321e+02_rb,0.40601e+02_rb,0.45258e+02_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.46107e+02_rb,0.40548e+02_rb,0.35109e+02_rb,0.30814e+02_rb,0.30060e+02_rb, &
        & 0.30665e+02_rb,0.34964e+02_rb,0.40159e+02_rb,0.44756e+02_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.45964e+02_rb,0.40421e+02_rb,0.35002e+02_rb,0.30664e+02_rb,0.29967e+02_rb, &
        & 0.30436e+02_rb,0.34636e+02_rb,0.39757e+02_rb,0.44299e+02_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.45779e+02_rb,0.40257e+02_rb,0.34861e+02_rb,0.30500e+02_rb,0.29866e+02_rb, &
        & 0.30215e+02_rb,0.34343e+02_rb,0.39421e+02_rb,0.43898e+02_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.52493e+02_rb,0.46184e+02_rb,0.39933e+02_rb,0.35707e+02_rb,0.34547e+02_rb, &
        & 0.34524e+02_rb,0.38680e+02_rb,0.44581e+02_rb,0.50362e+02_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.52499e+02_rb,0.46190e+02_rb,0.39958e+02_rb,0.35613e+02_rb,0.34342e+02_rb, &
        & 0.34438e+02_rb,0.38422e+02_rb,0.44213e+02_rb,0.49916e+02_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.52430e+02_rb,0.46141e+02_rb,0.39924e+02_rb,0.35477e+02_rb,0.34202e+02_rb, &
        & 0.34294e+02_rb,0.38128e+02_rb,0.43804e+02_rb,0.49438e+02_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.52296e+02_rb,0.46021e+02_rb,0.39821e+02_rb,0.35288e+02_rb,0.34063e+02_rb, &
        & 0.34077e+02_rb,0.37857e+02_rb,0.43466e+02_rb,0.49020e+02_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.52087e+02_rb,0.45828e+02_rb,0.39654e+02_rb,0.35060e+02_rb,0.33889e+02_rb, &
        & 0.33876e+02_rb,0.37613e+02_rb,0.43182e+02_rb,0.48640e+02_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.58998e+02_rb,0.51894e+02_rb,0.44809e+02_rb,0.40767e+02_rb,0.39196e+02_rb, &
        & 0.38349e+02_rb,0.41490e+02_rb,0.47820e+02_rb,0.54353e+02_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.58998e+02_rb,0.51911e+02_rb,0.44850e+02_rb,0.40692e+02_rb,0.38973e+02_rb, &
        & 0.38281e+02_rb,0.41275e+02_rb,0.47479e+02_rb,0.53941e+02_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.58946e+02_rb,0.51872e+02_rb,0.44826e+02_rb,0.40545e+02_rb,0.38818e+02_rb, &
        & 0.38152e+02_rb,0.41007e+02_rb,0.47151e+02_rb,0.53507e+02_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.58797e+02_rb,0.51741e+02_rb,0.44719e+02_rb,0.40328e+02_rb,0.38647e+02_rb, &
        & 0.37950e+02_rb,0.40768e+02_rb,0.46871e+02_rb,0.53108e+02_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.58578e+02_rb,0.51549e+02_rb,0.44555e+02_rb,0.40074e+02_rb,0.38419e+02_rb, &
        & 0.37769e+02_rb,0.40542e+02_rb,0.46625e+02_rb,0.52790e+02_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.65454e+02_rb,0.57537e+02_rb,0.49631e+02_rb,0.46171e+02_rb,0.44093e+02_rb, &
        & 0.42375e+02_rb,0.44216e+02_rb,0.50929e+02_rb,0.58021e+02_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.65480e+02_rb,0.57591e+02_rb,0.49679e+02_rb,0.46047e+02_rb,0.43916e+02_rb, &
        & 0.42317e+02_rb,0.44023e+02_rb,0.50643e+02_rb,0.57621e+02_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.65404e+02_rb,0.57534e+02_rb,0.49633e+02_rb,0.45875e+02_rb,0.43765e+02_rb, &
        & 0.42200e+02_rb,0.43791e+02_rb,0.50364e+02_rb,0.57215e+02_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.65236e+02_rb,0.57393e+02_rb,0.49518e+02_rb,0.45648e+02_rb,0.43596e+02_rb, &
        & 0.42013e+02_rb,0.43533e+02_rb,0.50076e+02_rb,0.56838e+02_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.64960e+02_rb,0.57155e+02_rb,0.49326e+02_rb,0.45390e+02_rb,0.43392e+02_rb, &
        & 0.41855e+02_rb,0.43285e+02_rb,0.49832e+02_rb,0.56537e+02_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.71553e+02_rb,0.62830e+02_rb,0.54580e+02_rb,0.51897e+02_rb,0.49249e+02_rb, &
        & 0.46670e+02_rb,0.47221e+02_rb,0.54164e+02_rb,0.61741e+02_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.71559e+02_rb,0.62858e+02_rb,0.54547e+02_rb,0.51727e+02_rb,0.49114e+02_rb, &
        & 0.46626e+02_rb,0.47093e+02_rb,0.53907e+02_rb,0.61336e+02_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.71474e+02_rb,0.62796e+02_rb,0.54422e+02_rb,0.51491e+02_rb,0.49001e+02_rb, &
        & 0.46518e+02_rb,0.46873e+02_rb,0.53599e+02_rb,0.60908e+02_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.71253e+02_rb,0.62620e+02_rb,0.54248e+02_rb,0.51208e+02_rb,0.48873e+02_rb, &
        & 0.46377e+02_rb,0.46595e+02_rb,0.53243e+02_rb,0.60469e+02_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.70931e+02_rb,0.62354e+02_rb,0.53993e+02_rb,0.50888e+02_rb,0.48719e+02_rb, &
        & 0.46222e+02_rb,0.46318e+02_rb,0.52946e+02_rb,0.60113e+02_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.76995e+02_rb,0.67566e+02_rb,0.59542e+02_rb,0.57607e+02_rb,0.54461e+02_rb, &
        & 0.51316e+02_rb,0.50984e+02_rb,0.57643e+02_rb,0.65674e+02_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.76958e+02_rb,0.67548e+02_rb,0.59397e+02_rb,0.57376e+02_rb,0.54369e+02_rb, &
        & 0.51326e+02_rb,0.50889e+02_rb,0.57330e+02_rb,0.65196e+02_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.76815e+02_rb,0.67433e+02_rb,0.59207e+02_rb,0.57127e+02_rb,0.54289e+02_rb, &
        & 0.51287e+02_rb,0.50622e+02_rb,0.56904e+02_rb,0.64662e+02_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.76535e+02_rb,0.67198e+02_rb,0.58948e+02_rb,0.56779e+02_rb,0.54224e+02_rb, &
        & 0.51158e+02_rb,0.50286e+02_rb,0.56459e+02_rb,0.64124e+02_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.76156e+02_rb,0.66876e+02_rb,0.58634e+02_rb,0.56401e+02_rb,0.54096e+02_rb, &
        & 0.50976e+02_rb,0.49973e+02_rb,0.56129e+02_rb,0.63785e+02_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.81421e+02_rb,0.71418e+02_rb,0.64455e+02_rb,0.62977e+02_rb,0.59643e+02_rb, &
        & 0.56434e+02_rb,0.55164e+02_rb,0.60758e+02_rb,0.69127e+02_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.81348e+02_rb,0.71366e+02_rb,0.64215e+02_rb,0.62779e+02_rb,0.59582e+02_rb, &
        & 0.56462e+02_rb,0.55001e+02_rb,0.60346e+02_rb,0.68584e+02_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.81121e+02_rb,0.71177e+02_rb,0.63909e+02_rb,0.62467e+02_rb,0.59508e+02_rb, &
        & 0.56437e+02_rb,0.54668e+02_rb,0.59898e+02_rb,0.68043e+02_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.80795e+02_rb,0.70901e+02_rb,0.63558e+02_rb,0.62070e+02_rb,0.59450e+02_rb, &
        & 0.56283e+02_rb,0.54317e+02_rb,0.59470e+02_rb,0.67583e+02_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.80315e+02_rb,0.70489e+02_rb,0.63126e+02_rb,0.61649e+02_rb,0.59298e+02_rb, &
        & 0.56050e+02_rb,0.54007e+02_rb,0.59179e+02_rb,0.67330e+02_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.84526e+02_rb,0.74114e+02_rb,0.69139e+02_rb,0.67766e+02_rb,0.64700e+02_rb, &
        & 0.61791e+02_rb,0.59295e+02_rb,0.62885e+02_rb,0.71468e+02_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.84370e+02_rb,0.73989e+02_rb,0.68856e+02_rb,0.67565e+02_rb,0.64593e+02_rb, &
        & 0.61811e+02_rb,0.59181e+02_rb,0.62666e+02_rb,0.71181e+02_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.84053e+02_rb,0.73721e+02_rb,0.68457e+02_rb,0.67233e+02_rb,0.64524e+02_rb, &
        & 0.61778e+02_rb,0.58870e+02_rb,0.62384e+02_rb,0.70849e+02_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.83638e+02_rb,0.73367e+02_rb,0.67980e+02_rb,0.66820e+02_rb,0.64408e+02_rb, &
        & 0.61582e+02_rb,0.58561e+02_rb,0.62049e+02_rb,0.70550e+02_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.83125e+02_rb,0.72897e+02_rb,0.67415e+02_rb,0.66418e+02_rb,0.64238e+02_rb, &
        & 0.61304e+02_rb,0.58306e+02_rb,0.61805e+02_rb,0.70366e+02_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.86099e+02_rb,0.75664e+02_rb,0.73052e+02_rb,0.71784e+02_rb,0.69581e+02_rb, &
        & 0.67013e+02_rb,0.63079e+02_rb,0.64019e+02_rb,0.72668e+02_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.85793e+02_rb,0.75364e+02_rb,0.72621e+02_rb,0.71506e+02_rb,0.69364e+02_rb, &
        & 0.66992e+02_rb,0.63098e+02_rb,0.64105e+02_rb,0.72738e+02_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.85391e+02_rb,0.74983e+02_rb,0.72119e+02_rb,0.71136e+02_rb,0.69201e+02_rb, &
        & 0.66897e+02_rb,0.62939e+02_rb,0.63964e+02_rb,0.72620e+02_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.84829e+02_rb,0.74472e+02_rb,0.71563e+02_rb,0.70768e+02_rb,0.69027e+02_rb, &
        & 0.66600e+02_rb,0.62760e+02_rb,0.63750e+02_rb,0.72497e+02_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.84210e+02_rb,0.73923e+02_rb,0.70963e+02_rb,0.70409e+02_rb,0.68788e+02_rb, &
        & 0.66285e+02_rb,0.62604e+02_rb,0.63643e+02_rb,0.72460e+02_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.85661e+02_rb,0.75976e+02_rb,0.75616e+02_rb,0.74895e+02_rb,0.73796e+02_rb, &
        & 0.71450e+02_rb,0.66595e+02_rb,0.64723e+02_rb,0.72988e+02_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.85212e+02_rb,0.75487e+02_rb,0.75133e+02_rb,0.74528e+02_rb,0.73552e+02_rb, &
        & 0.71413e+02_rb,0.66688e+02_rb,0.64835e+02_rb,0.73147e+02_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.84633e+02_rb,0.74907e+02_rb,0.74581e+02_rb,0.74183e+02_rb,0.73356e+02_rb, &
        & 0.71194e+02_rb,0.66687e+02_rb,0.64831e+02_rb,0.73292e+02_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.83975e+02_rb,0.74286e+02_rb,0.73943e+02_rb,0.73830e+02_rb,0.73088e+02_rb, &
        & 0.70916e+02_rb,0.66610e+02_rb,0.64920e+02_rb,0.73470e+02_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.83246e+02_rb,0.73605e+02_rb,0.73209e+02_rb,0.73386e+02_rb,0.72735e+02_rb, &
        & 0.70587e+02_rb,0.66523e+02_rb,0.64950e+02_rb,0.73537e+02_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.83352e+02_rb,0.75282e+02_rb,0.76925e+02_rb,0.77365e+02_rb,0.77206e+02_rb, &
        & 0.74850e+02_rb,0.69716e+02_rb,0.65048e+02_rb,0.72181e+02_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.82736e+02_rb,0.74597e+02_rb,0.76403e+02_rb,0.76949e+02_rb,0.76971e+02_rb, &
        & 0.74863e+02_rb,0.69929e+02_rb,0.65330e+02_rb,0.72664e+02_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.82054e+02_rb,0.73916e+02_rb,0.75743e+02_rb,0.76528e+02_rb,0.76666e+02_rb, &
        & 0.74738e+02_rb,0.70022e+02_rb,0.65629e+02_rb,0.73075e+02_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.81316e+02_rb,0.73173e+02_rb,0.75030e+02_rb,0.76061e+02_rb,0.76286e+02_rb, &
        & 0.74512e+02_rb,0.70083e+02_rb,0.65825e+02_rb,0.73345e+02_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.80516e+02_rb,0.72378e+02_rb,0.74331e+02_rb,0.75554e+02_rb,0.75785e+02_rb, &
        & 0.74233e+02_rb,0.70124e+02_rb,0.65929e+02_rb,0.73501e+02_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.79408e+02_rb,0.73858e+02_rb,0.76980e+02_rb,0.78869e+02_rb,0.79397e+02_rb, &
        & 0.77263e+02_rb,0.72263e+02_rb,0.65338e+02_rb,0.70604e+02_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.78716e+02_rb,0.73093e+02_rb,0.76326e+02_rb,0.78426e+02_rb,0.79146e+02_rb, &
        & 0.77408e+02_rb,0.72561e+02_rb,0.65807e+02_rb,0.71230e+02_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.77962e+02_rb,0.72251e+02_rb,0.75686e+02_rb,0.77926e+02_rb,0.78867e+02_rb, &
        & 0.77356e+02_rb,0.72810e+02_rb,0.66137e+02_rb,0.71698e+02_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.77177e+02_rb,0.71385e+02_rb,0.75042e+02_rb,0.77408e+02_rb,0.78464e+02_rb, &
        & 0.77172e+02_rb,0.73006e+02_rb,0.66333e+02_rb,0.72067e+02_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.76340e+02_rb,0.70519e+02_rb,0.74355e+02_rb,0.76889e+02_rb,0.78015e+02_rb, &
        & 0.76988e+02_rb,0.73079e+02_rb,0.66485e+02_rb,0.72407e+02_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.51404e+02_rb,0.45212e+02_rb,0.39078e+02_rb,0.32978e+02_rb,0.31213e+02_rb, &
        & 0.35689e+02_rb,0.42239e+02_rb,0.49109e+02_rb,0.53555e+02_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.51456e+02_rb,0.45263e+02_rb,0.39085e+02_rb,0.32975e+02_rb,0.31034e+02_rb, &
        & 0.35237e+02_rb,0.41756e+02_rb,0.48531e+02_rb,0.52877e+02_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.51419e+02_rb,0.45226e+02_rb,0.39013e+02_rb,0.32932e+02_rb,0.30826e+02_rb, &
        & 0.34871e+02_rb,0.41372e+02_rb,0.48070e+02_rb,0.52319e+02_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.51315e+02_rb,0.45122e+02_rb,0.38925e+02_rb,0.32886e+02_rb,0.30643e+02_rb, &
        & 0.34565e+02_rb,0.41054e+02_rb,0.47690e+02_rb,0.51853e+02_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.51134e+02_rb,0.44958e+02_rb,0.38787e+02_rb,0.32819e+02_rb,0.30476e+02_rb, &
        & 0.34330e+02_rb,0.40799e+02_rb,0.47384e+02_rb,0.51513e+02_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.59575e+02_rb,0.52343e+02_rb,0.45168e+02_rb,0.38156e+02_rb,0.36549e+02_rb, &
        & 0.40798e+02_rb,0.48433e+02_rb,0.56320e+02_rb,0.62499e+02_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.59656e+02_rb,0.52415e+02_rb,0.45196e+02_rb,0.38159e+02_rb,0.36356e+02_rb, &
        & 0.40232e+02_rb,0.47806e+02_rb,0.55603e+02_rb,0.61692e+02_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.59635e+02_rb,0.52395e+02_rb,0.45146e+02_rb,0.38116e+02_rb,0.36139e+02_rb, &
        & 0.39805e+02_rb,0.47339e+02_rb,0.55046e+02_rb,0.61068e+02_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.59522e+02_rb,0.52285e+02_rb,0.45051e+02_rb,0.38066e+02_rb,0.35889e+02_rb, &
        & 0.39470e+02_rb,0.46981e+02_rb,0.54621e+02_rb,0.60592e+02_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.59307e+02_rb,0.52094e+02_rb,0.44889e+02_rb,0.37960e+02_rb,0.35677e+02_rb, &
        & 0.39216e+02_rb,0.46687e+02_rb,0.54268e+02_rb,0.60193e+02_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.68809e+02_rb,0.60397e+02_rb,0.52025e+02_rb,0.44161e+02_rb,0.42701e+02_rb, &
        & 0.46667e+02_rb,0.55520e+02_rb,0.64493e+02_rb,0.72669e+02_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.68924e+02_rb,0.60499e+02_rb,0.52098e+02_rb,0.44133e+02_rb,0.42521e+02_rb, &
        & 0.45966e+02_rb,0.54732e+02_rb,0.63594e+02_rb,0.71705e+02_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.68950e+02_rb,0.60531e+02_rb,0.52101e+02_rb,0.44100e+02_rb,0.42285e+02_rb, &
        & 0.45418e+02_rb,0.54095e+02_rb,0.62883e+02_rb,0.70919e+02_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.68857e+02_rb,0.60434e+02_rb,0.52014e+02_rb,0.44041e+02_rb,0.42031e+02_rb, &
        & 0.45005e+02_rb,0.53564e+02_rb,0.62291e+02_rb,0.70247e+02_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.68666e+02_rb,0.60265e+02_rb,0.51871e+02_rb,0.43942e+02_rb,0.41842e+02_rb, &
        & 0.44690e+02_rb,0.53134e+02_rb,0.61806e+02_rb,0.69686e+02_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.79311e+02_rb,0.69561e+02_rb,0.59881e+02_rb,0.51276e+02_rb,0.49761e+02_rb, &
        & 0.53193e+02_rb,0.63396e+02_rb,0.73656e+02_rb,0.83652e+02_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.79511e+02_rb,0.69736e+02_rb,0.60019e+02_rb,0.51173e+02_rb,0.49594e+02_rb, &
        & 0.52466e+02_rb,0.62461e+02_rb,0.72578e+02_rb,0.82426e+02_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.79562e+02_rb,0.69789e+02_rb,0.60043e+02_rb,0.51075e+02_rb,0.49346e+02_rb, &
        & 0.51847e+02_rb,0.61639e+02_rb,0.71632e+02_rb,0.81369e+02_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.79512e+02_rb,0.69733e+02_rb,0.59989e+02_rb,0.50979e+02_rb,0.49088e+02_rb, &
        & 0.51360e+02_rb,0.60947e+02_rb,0.70836e+02_rb,0.80468e+02_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.79325e+02_rb,0.69569e+02_rb,0.59854e+02_rb,0.50871e+02_rb,0.48882e+02_rb, &
        & 0.50991e+02_rb,0.60362e+02_rb,0.70161e+02_rb,0.79701e+02_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.91112e+02_rb,0.79892e+02_rb,0.68781e+02_rb,0.59496e+02_rb,0.57988e+02_rb, &
        & 0.60415e+02_rb,0.71595e+02_rb,0.83227e+02_rb,0.94854e+02_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.91447e+02_rb,0.80173e+02_rb,0.69023e+02_rb,0.59416e+02_rb,0.57743e+02_rb, &
        & 0.59673e+02_rb,0.70483e+02_rb,0.81942e+02_rb,0.93394e+02_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.91583e+02_rb,0.80288e+02_rb,0.69106e+02_rb,0.59276e+02_rb,0.57416e+02_rb, &
        & 0.59035e+02_rb,0.69528e+02_rb,0.80829e+02_rb,0.92131e+02_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.91566e+02_rb,0.80257e+02_rb,0.69073e+02_rb,0.59136e+02_rb,0.57092e+02_rb, &
        & 0.58527e+02_rb,0.68724e+02_rb,0.79885e+02_rb,0.91053e+02_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.91390e+02_rb,0.80104e+02_rb,0.68937e+02_rb,0.58973e+02_rb,0.56776e+02_rb, &
        & 0.58107e+02_rb,0.68079e+02_rb,0.79072e+02_rb,0.90122e+02_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.10419e+03_rb,0.91394e+02_rb,0.78656e+02_rb,0.68916e+02_rb,0.67128e+02_rb, &
        & 0.68461e+02_rb,0.79894e+02_rb,0.92926e+02_rb,0.10607e+03_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.10463e+03_rb,0.91782e+02_rb,0.79002e+02_rb,0.68819e+02_rb,0.66845e+02_rb, &
        & 0.67738e+02_rb,0.78628e+02_rb,0.91458e+02_rb,0.10440e+03_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.10486e+03_rb,0.91970e+02_rb,0.79156e+02_rb,0.68698e+02_rb,0.66470e+02_rb, &
        & 0.67066e+02_rb,0.77628e+02_rb,0.90242e+02_rb,0.10302e+03_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.10494e+03_rb,0.92025e+02_rb,0.79207e+02_rb,0.68555e+02_rb,0.66081e+02_rb, &
        & 0.66543e+02_rb,0.76843e+02_rb,0.89253e+02_rb,0.10189e+03_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.10481e+03_rb,0.91892e+02_rb,0.79105e+02_rb,0.68344e+02_rb,0.65668e+02_rb, &
        & 0.66109e+02_rb,0.76190e+02_rb,0.88386e+02_rb,0.10090e+03_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.11850e+03_rb,0.10394e+03_rb,0.89423e+02_rb,0.79509e+02_rb,0.77207e+02_rb, &
        & 0.77196e+02_rb,0.87943e+02_rb,0.10236e+03_rb,0.11694e+03_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.11905e+03_rb,0.10443e+03_rb,0.89864e+02_rb,0.79528e+02_rb,0.76905e+02_rb, &
        & 0.76494e+02_rb,0.86650e+02_rb,0.10078e+03_rb,0.11515e+03_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.11939e+03_rb,0.10473e+03_rb,0.90115e+02_rb,0.79401e+02_rb,0.76523e+02_rb, &
        & 0.75853e+02_rb,0.85715e+02_rb,0.99595e+02_rb,0.11380e+03_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.11951e+03_rb,0.10484e+03_rb,0.90222e+02_rb,0.79220e+02_rb,0.76090e+02_rb, &
        & 0.75405e+02_rb,0.85022e+02_rb,0.98653e+02_rb,0.11271e+03_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.11942e+03_rb,0.10476e+03_rb,0.90169e+02_rb,0.78976e+02_rb,0.75607e+02_rb, &
        & 0.75018e+02_rb,0.84352e+02_rb,0.97783e+02_rb,0.11167e+03_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.13381e+03_rb,0.11735e+03_rb,0.10095e+03_rb,0.91226e+02_rb,0.88188e+02_rb, &
        & 0.86379e+02_rb,0.95828e+02_rb,0.11155e+03_rb,0.12755e+03_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.13446e+03_rb,0.11794e+03_rb,0.10145e+03_rb,0.91271e+02_rb,0.87947e+02_rb, &
        & 0.85766e+02_rb,0.94652e+02_rb,0.11006e+03_rb,0.12583e+03_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.13492e+03_rb,0.11834e+03_rb,0.10180e+03_rb,0.91233e+02_rb,0.87619e+02_rb, &
        & 0.85228e+02_rb,0.93811e+02_rb,0.10892e+03_rb,0.12452e+03_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.13505e+03_rb,0.11846e+03_rb,0.10192e+03_rb,0.91090e+02_rb,0.87150e+02_rb, &
        & 0.84887e+02_rb,0.93111e+02_rb,0.10797e+03_rb,0.12337e+03_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.13500e+03_rb,0.11844e+03_rb,0.10192e+03_rb,0.90797e+02_rb,0.86626e+02_rb, &
        & 0.84570e+02_rb,0.92421e+02_rb,0.10710e+03_rb,0.12226e+03_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.14952e+03_rb,0.13110e+03_rb,0.11316e+03_rb,0.10410e+03_rb,0.99927e+02_rb, &
        & 0.96074e+02_rb,0.10374e+03_rb,0.12067e+03_rb,0.13801e+03_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.15028e+03_rb,0.13179e+03_rb,0.11360e+03_rb,0.10415e+03_rb,0.99870e+02_rb, &
        & 0.95589e+02_rb,0.10257e+03_rb,0.11912e+03_rb,0.13623e+03_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.15082e+03_rb,0.13227e+03_rb,0.11392e+03_rb,0.10405e+03_rb,0.99571e+02_rb, &
        & 0.95189e+02_rb,0.10182e+03_rb,0.11803e+03_rb,0.13496e+03_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.15103e+03_rb,0.13247e+03_rb,0.11403e+03_rb,0.10385e+03_rb,0.99126e+02_rb, &
        & 0.94980e+02_rb,0.10122e+03_rb,0.11720e+03_rb,0.13388e+03_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.15094e+03_rb,0.13245e+03_rb,0.11401e+03_rb,0.10348e+03_rb,0.98550e+02_rb, &
        & 0.94746e+02_rb,0.10055e+03_rb,0.11636e+03_rb,0.13276e+03_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.16524e+03_rb,0.14486e+03_rb,0.12594e+03_rb,0.11785e+03_rb,0.11225e+03_rb, &
        & 0.10654e+03_rb,0.11202e+03_rb,0.12965e+03_rb,0.14829e+03_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.16610e+03_rb,0.14564e+03_rb,0.12636e+03_rb,0.11787e+03_rb,0.11233e+03_rb, &
        & 0.10630e+03_rb,0.11093e+03_rb,0.12811e+03_rb,0.14653e+03_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.16659e+03_rb,0.14608e+03_rb,0.12656e+03_rb,0.11769e+03_rb,0.11214e+03_rb, &
        & 0.10609e+03_rb,0.11032e+03_rb,0.12713e+03_rb,0.14532e+03_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.16681e+03_rb,0.14630e+03_rb,0.12659e+03_rb,0.11736e+03_rb,0.11169e+03_rb, &
        & 0.10597e+03_rb,0.10973e+03_rb,0.12629e+03_rb,0.14416e+03_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.16671e+03_rb,0.14623e+03_rb,0.12639e+03_rb,0.11688e+03_rb,0.11111e+03_rb, &
        & 0.10576e+03_rb,0.10909e+03_rb,0.12542e+03_rb,0.14303e+03_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.18065e+03_rb,0.15832e+03_rb,0.13903e+03_rb,0.13201e+03_rb,0.12519e+03_rb, &
        & 0.11820e+03_rb,0.12047e+03_rb,0.13760e+03_rb,0.15739e+03_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.18131e+03_rb,0.15893e+03_rb,0.13916e+03_rb,0.13183e+03_rb,0.12520e+03_rb, &
        & 0.11800e+03_rb,0.11979e+03_rb,0.13660e+03_rb,0.15612e+03_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.18162e+03_rb,0.15925e+03_rb,0.13912e+03_rb,0.13145e+03_rb,0.12487e+03_rb, &
        & 0.11789e+03_rb,0.11922e+03_rb,0.13566e+03_rb,0.15481e+03_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.18161e+03_rb,0.15927e+03_rb,0.13888e+03_rb,0.13091e+03_rb,0.12432e+03_rb, &
        & 0.11770e+03_rb,0.11858e+03_rb,0.13462e+03_rb,0.15350e+03_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.18129e+03_rb,0.15901e+03_rb,0.13849e+03_rb,0.13027e+03_rb,0.12371e+03_rb, &
        & 0.11738e+03_rb,0.11800e+03_rb,0.13378e+03_rb,0.15248e+03_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.19439e+03_rb,0.17031e+03_rb,0.15181e+03_rb,0.14577e+03_rb,0.13828e+03_rb, &
        & 0.13070e+03_rb,0.12972e+03_rb,0.14546e+03_rb,0.16618e+03_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.19483e+03_rb,0.17073e+03_rb,0.15156e+03_rb,0.14543e+03_rb,0.13812e+03_rb, &
        & 0.13043e+03_rb,0.12913e+03_rb,0.14443e+03_rb,0.16476e+03_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.19494e+03_rb,0.17087e+03_rb,0.15124e+03_rb,0.14493e+03_rb,0.13771e+03_rb, &
        & 0.13020e+03_rb,0.12847e+03_rb,0.14330e+03_rb,0.16335e+03_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.19474e+03_rb,0.17073e+03_rb,0.15078e+03_rb,0.14425e+03_rb,0.13718e+03_rb, &
        & 0.12986e+03_rb,0.12779e+03_rb,0.14240e+03_rb,0.16229e+03_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.19420e+03_rb,0.17029e+03_rb,0.15011e+03_rb,0.14337e+03_rb,0.13666e+03_rb, &
        & 0.12942e+03_rb,0.12725e+03_rb,0.14171e+03_rb,0.16147e+03_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.20562e+03_rb,0.18014e+03_rb,0.16395e+03_rb,0.15885e+03_rb,0.15131e+03_rb, &
        & 0.14344e+03_rb,0.13928e+03_rb,0.15201e+03_rb,0.17336e+03_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.20583e+03_rb,0.18034e+03_rb,0.16346e+03_rb,0.15844e+03_rb,0.15102e+03_rb, &
        & 0.14310e+03_rb,0.13867e+03_rb,0.15101e+03_rb,0.17210e+03_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.20579e+03_rb,0.18034e+03_rb,0.16282e+03_rb,0.15782e+03_rb,0.15051e+03_rb, &
        & 0.14271e+03_rb,0.13803e+03_rb,0.15026e+03_rb,0.17119e+03_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.20539e+03_rb,0.18002e+03_rb,0.16199e+03_rb,0.15696e+03_rb,0.15005e+03_rb, &
        & 0.14226e+03_rb,0.13746e+03_rb,0.14977e+03_rb,0.17061e+03_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.20471e+03_rb,0.17946e+03_rb,0.16101e+03_rb,0.15580e+03_rb,0.14942e+03_rb, &
        & 0.14163e+03_rb,0.13685e+03_rb,0.14922e+03_rb,0.16998e+03_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.61110e+02_rb,0.53539e+02_rb,0.46005e+02_rb,0.38902e+02_rb,0.35970e+02_rb, &
        & 0.43986e+02_rb,0.52608e+02_rb,0.61157e+02_rb,0.65911e+02_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.61315e+02_rb,0.53717e+02_rb,0.46235e+02_rb,0.39083e+02_rb,0.35687e+02_rb, &
        & 0.43381e+02_rb,0.51871e+02_rb,0.60290e+02_rb,0.64963e+02_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.61422e+02_rb,0.53821e+02_rb,0.46411e+02_rb,0.39188e+02_rb,0.35438e+02_rb, &
        & 0.42839e+02_rb,0.51205e+02_rb,0.59514e+02_rb,0.64157e+02_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.61394e+02_rb,0.53877e+02_rb,0.46504e+02_rb,0.39222e+02_rb,0.35189e+02_rb, &
        & 0.42365e+02_rb,0.50625e+02_rb,0.58813e+02_rb,0.63464e+02_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.61322e+02_rb,0.53865e+02_rb,0.46511e+02_rb,0.39177e+02_rb,0.34939e+02_rb, &
        & 0.41926e+02_rb,0.50070e+02_rb,0.58152e+02_rb,0.62761e+02_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.73139e+02_rb,0.64054e+02_rb,0.54971e+02_rb,0.46172e+02_rb,0.42554e+02_rb, &
        & 0.52369e+02_rb,0.62683e+02_rb,0.72904e+02_rb,0.80226e+02_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.73363e+02_rb,0.64250e+02_rb,0.55203e+02_rb,0.46423e+02_rb,0.42168e+02_rb, &
        & 0.51668e+02_rb,0.61839e+02_rb,0.71917e+02_rb,0.79110e+02_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.73487e+02_rb,0.64383e+02_rb,0.55429e+02_rb,0.46632e+02_rb,0.41828e+02_rb, &
        & 0.50986e+02_rb,0.61016e+02_rb,0.70950e+02_rb,0.78048e+02_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.73536e+02_rb,0.64501e+02_rb,0.55558e+02_rb,0.46729e+02_rb,0.41557e+02_rb, &
        & 0.50314e+02_rb,0.60204e+02_rb,0.70005e+02_rb,0.77009e+02_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.73500e+02_rb,0.64509e+02_rb,0.55582e+02_rb,0.46709e+02_rb,0.41283e+02_rb, &
        & 0.49695e+02_rb,0.59450e+02_rb,0.69119e+02_rb,0.76024e+02_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.87361e+02_rb,0.76491e+02_rb,0.65621e+02_rb,0.54890e+02_rb,0.50598e+02_rb, &
        & 0.62297e+02_rb,0.74612e+02_rb,0.86825e+02_rb,0.97550e+02_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.87685e+02_rb,0.76774e+02_rb,0.65885e+02_rb,0.55188e+02_rb,0.50206e+02_rb, &
        & 0.61585e+02_rb,0.73754e+02_rb,0.85807e+02_rb,0.96409e+02_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.87844e+02_rb,0.76919e+02_rb,0.66106e+02_rb,0.55413e+02_rb,0.49812e+02_rb, &
        & 0.60785e+02_rb,0.72789e+02_rb,0.84689e+02_rb,0.95141e+02_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.87889e+02_rb,0.77037e+02_rb,0.66247e+02_rb,0.55541e+02_rb,0.49446e+02_rb, &
        & 0.60010e+02_rb,0.71857e+02_rb,0.83593e+02_rb,0.93898e+02_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.87858e+02_rb,0.77051e+02_rb,0.66285e+02_rb,0.55583e+02_rb,0.49097e+02_rb, &
        & 0.59241e+02_rb,0.70929e+02_rb,0.82513e+02_rb,0.92686e+02_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.10387e+03_rb,0.90929e+02_rb,0.77989e+02_rb,0.65095e+02_rb,0.60436e+02_rb, &
        & 0.74039e+02_rb,0.88717e+02_rb,0.10327e+03_rb,0.11725e+03_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.10441e+03_rb,0.91405e+02_rb,0.78408e+02_rb,0.65545e+02_rb,0.59926e+02_rb, &
        & 0.73070e+02_rb,0.87549e+02_rb,0.10190e+03_rb,0.11570e+03_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.10471e+03_rb,0.91680e+02_rb,0.78722e+02_rb,0.65845e+02_rb,0.59509e+02_rb, &
        & 0.72264e+02_rb,0.86581e+02_rb,0.10076e+03_rb,0.11440e+03_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.10486e+03_rb,0.91873e+02_rb,0.78926e+02_rb,0.66032e+02_rb,0.59086e+02_rb, &
        & 0.71381e+02_rb,0.85514e+02_rb,0.99526e+02_rb,0.11299e+03_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.10483e+03_rb,0.91879e+02_rb,0.78952e+02_rb,0.66063e+02_rb,0.58651e+02_rb, &
        & 0.70432e+02_rb,0.84377e+02_rb,0.98185e+02_rb,0.11148e+03_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.12314e+03_rb,0.10778e+03_rb,0.92429e+02_rb,0.77074e+02_rb,0.72086e+02_rb, &
        & 0.87688e+02_rb,0.10511e+03_rb,0.12238e+03_rb,0.13959e+03_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.12380e+03_rb,0.10836e+03_rb,0.92922e+02_rb,0.77571e+02_rb,0.71572e+02_rb, &
        & 0.86586e+02_rb,0.10378e+03_rb,0.12082e+03_rb,0.13782e+03_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.12423e+03_rb,0.10877e+03_rb,0.93351e+02_rb,0.77983e+02_rb,0.71097e+02_rb, &
        & 0.85507e+02_rb,0.10249e+03_rb,0.11931e+03_rb,0.13609e+03_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.12450e+03_rb,0.10905e+03_rb,0.93631e+02_rb,0.78238e+02_rb,0.70653e+02_rb, &
        & 0.84488e+02_rb,0.10126e+03_rb,0.11788e+03_rb,0.13445e+03_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.12453e+03_rb,0.10910e+03_rb,0.93684e+02_rb,0.78289e+02_rb,0.70179e+02_rb, &
        & 0.83403e+02_rb,0.99954e+02_rb,0.11636e+03_rb,0.13271e+03_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.14538e+03_rb,0.12724e+03_rb,0.10910e+03_rb,0.90962e+02_rb,0.85931e+02_rb, &
        & 0.10354e+03_rb,0.12413e+03_rb,0.14456e+03_rb,0.16524e+03_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.14625e+03_rb,0.12800e+03_rb,0.10975e+03_rb,0.91547e+02_rb,0.85303e+02_rb, &
        & 0.10223e+03_rb,0.12256e+03_rb,0.14271e+03_rb,0.16313e+03_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.14683e+03_rb,0.12855e+03_rb,0.11028e+03_rb,0.92053e+02_rb,0.84796e+02_rb, &
        & 0.10095e+03_rb,0.12102e+03_rb,0.14092e+03_rb,0.16108e+03_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.14716e+03_rb,0.12887e+03_rb,0.11059e+03_rb,0.92330e+02_rb,0.84296e+02_rb, &
        & 0.99657e+02_rb,0.11947e+03_rb,0.13910e+03_rb,0.15900e+03_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.14726e+03_rb,0.12897e+03_rb,0.11069e+03_rb,0.92414e+02_rb,0.83857e+02_rb, &
        & 0.98384e+02_rb,0.11794e+03_rb,0.13732e+03_rb,0.15696e+03_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.17053e+03_rb,0.14925e+03_rb,0.12796e+03_rb,0.10716e+03_rb,0.10213e+03_rb, &
        & 0.12188e+03_rb,0.14615e+03_rb,0.17020e+03_rb,0.19480e+03_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.17172e+03_rb,0.15029e+03_rb,0.12885e+03_rb,0.10758e+03_rb,0.10144e+03_rb, &
        & 0.12029e+03_rb,0.14423e+03_rb,0.16798e+03_rb,0.19224e+03_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.17251e+03_rb,0.15103e+03_rb,0.12955e+03_rb,0.10811e+03_rb,0.10086e+03_rb, &
        & 0.11869e+03_rb,0.14231e+03_rb,0.16572e+03_rb,0.18967e+03_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.17303e+03_rb,0.15150e+03_rb,0.12997e+03_rb,0.10845e+03_rb,0.10032e+03_rb, &
        & 0.11701e+03_rb,0.14029e+03_rb,0.16338e+03_rb,0.18697e+03_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.17326e+03_rb,0.15170e+03_rb,0.13015e+03_rb,0.10860e+03_rb,0.99828e+02_rb, &
        & 0.11540e+03_rb,0.13837e+03_rb,0.16114e+03_rb,0.18440e+03_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.19897e+03_rb,0.17412e+03_rb,0.14927e+03_rb,0.12609e+03_rb,0.12102e+03_rb, &
        & 0.14307e+03_rb,0.17149e+03_rb,0.19971e+03_rb,0.22868e+03_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.20045e+03_rb,0.17541e+03_rb,0.15039e+03_rb,0.12643e+03_rb,0.12014e+03_rb, &
        & 0.14095e+03_rb,0.16896e+03_rb,0.19679e+03_rb,0.22532e+03_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.20144e+03_rb,0.17633e+03_rb,0.15122e+03_rb,0.12673e+03_rb,0.11946e+03_rb, &
        & 0.13885e+03_rb,0.16646e+03_rb,0.19386e+03_rb,0.22200e+03_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.20212e+03_rb,0.17694e+03_rb,0.15175e+03_rb,0.12692e+03_rb,0.11890e+03_rb, &
        & 0.13670e+03_rb,0.16392e+03_rb,0.19092e+03_rb,0.21864e+03_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.20244e+03_rb,0.17722e+03_rb,0.15200e+03_rb,0.12703e+03_rb,0.11832e+03_rb, &
        & 0.13474e+03_rb,0.16157e+03_rb,0.18819e+03_rb,0.21551e+03_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.23120e+03_rb,0.20232e+03_rb,0.17344e+03_rb,0.14797e+03_rb,0.14299e+03_rb, &
        & 0.16694e+03_rb,0.20011e+03_rb,0.23304e+03_rb,0.26690e+03_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.23311e+03_rb,0.20399e+03_rb,0.17488e+03_rb,0.14825e+03_rb,0.14181e+03_rb, &
        & 0.16424e+03_rb,0.19689e+03_rb,0.22930e+03_rb,0.26259e+03_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.23434e+03_rb,0.20510e+03_rb,0.17587e+03_rb,0.14844e+03_rb,0.14100e+03_rb, &
        & 0.16150e+03_rb,0.19359e+03_rb,0.22546e+03_rb,0.25821e+03_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.23518e+03_rb,0.20584e+03_rb,0.17651e+03_rb,0.14849e+03_rb,0.14028e+03_rb, &
        & 0.15873e+03_rb,0.19031e+03_rb,0.22162e+03_rb,0.25383e+03_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.23553e+03_rb,0.20616e+03_rb,0.17678e+03_rb,0.14839e+03_rb,0.13959e+03_rb, &
        & 0.15626e+03_rb,0.18736e+03_rb,0.21823e+03_rb,0.24993e+03_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.26766e+03_rb,0.23422e+03_rb,0.20086e+03_rb,0.17305e+03_rb,0.16820e+03_rb, &
        & 0.19275e+03_rb,0.23109e+03_rb,0.26916e+03_rb,0.30826e+03_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.26991e+03_rb,0.23619e+03_rb,0.20254e+03_rb,0.17334e+03_rb,0.16673e+03_rb, &
        & 0.18936e+03_rb,0.22702e+03_rb,0.26442e+03_rb,0.30283e+03_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.27155e+03_rb,0.23766e+03_rb,0.20381e+03_rb,0.17344e+03_rb,0.16570e+03_rb, &
        & 0.18588e+03_rb,0.22280e+03_rb,0.25950e+03_rb,0.29719e+03_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.27254e+03_rb,0.23853e+03_rb,0.20456e+03_rb,0.17332e+03_rb,0.16480e+03_rb, &
        & 0.18267e+03_rb,0.21887e+03_rb,0.25495e+03_rb,0.29197e+03_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.27304e+03_rb,0.23896e+03_rb,0.20491e+03_rb,0.17301e+03_rb,0.16390e+03_rb, &
        & 0.17979e+03_rb,0.21534e+03_rb,0.25082e+03_rb,0.28725e+03_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.31010e+03_rb,0.27147e+03_rb,0.23291e+03_rb,0.20181e+03_rb,0.19592e+03_rb, &
        & 0.21861e+03_rb,0.26152e+03_rb,0.30466e+03_rb,0.34889e+03_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.31237e+03_rb,0.27343e+03_rb,0.23463e+03_rb,0.20183e+03_rb,0.19444e+03_rb, &
        & 0.21467e+03_rb,0.25634e+03_rb,0.29864e+03_rb,0.34194e+03_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.31396e+03_rb,0.27478e+03_rb,0.23578e+03_rb,0.20169e+03_rb,0.19320e+03_rb, &
        & 0.21082e+03_rb,0.25169e+03_rb,0.29321e+03_rb,0.33575e+03_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.31491e+03_rb,0.27559e+03_rb,0.23646e+03_rb,0.20134e+03_rb,0.19199e+03_rb, &
        & 0.20744e+03_rb,0.24738e+03_rb,0.28820e+03_rb,0.33002e+03_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.31516e+03_rb,0.27581e+03_rb,0.23662e+03_rb,0.20063e+03_rb,0.19065e+03_rb, &
        & 0.20428e+03_rb,0.24308e+03_rb,0.28319e+03_rb,0.32426e+03_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.35746e+03_rb,0.31300e+03_rb,0.26858e+03_rb,0.23433e+03_rb,0.22701e+03_rb, &
        & 0.24649e+03_rb,0.29230e+03_rb,0.34058e+03_rb,0.38993e+03_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.35965e+03_rb,0.31492e+03_rb,0.27026e+03_rb,0.23392e+03_rb,0.22536e+03_rb, &
        & 0.24218e+03_rb,0.28684e+03_rb,0.33421e+03_rb,0.38264e+03_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.36096e+03_rb,0.31606e+03_rb,0.27127e+03_rb,0.23331e+03_rb,0.22367e+03_rb, &
        & 0.23821e+03_rb,0.28173e+03_rb,0.32828e+03_rb,0.37584e+03_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.36159e+03_rb,0.31657e+03_rb,0.27171e+03_rb,0.23251e+03_rb,0.22182e+03_rb, &
        & 0.23456e+03_rb,0.27669e+03_rb,0.32238e+03_rb,0.36910e+03_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.36141e+03_rb,0.31636e+03_rb,0.27153e+03_rb,0.23134e+03_rb,0.21976e+03_rb, &
        & 0.23095e+03_rb,0.27160e+03_rb,0.31644e+03_rb,0.36227e+03_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.40931e+03_rb,0.35842e+03_rb,0.30758e+03_rb,0.27041e+03_rb,0.26133e+03_rb, &
        & 0.27679e+03_rb,0.32393e+03_rb,0.37751e+03_rb,0.43213e+03_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.41124e+03_rb,0.36013e+03_rb,0.30908e+03_rb,0.26935e+03_rb,0.25923e+03_rb, &
        & 0.27218e+03_rb,0.31806e+03_rb,0.37065e+03_rb,0.42432e+03_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.41214e+03_rb,0.36093e+03_rb,0.30979e+03_rb,0.26811e+03_rb,0.25684e+03_rb, &
        & 0.26794e+03_rb,0.31214e+03_rb,0.36375e+03_rb,0.41639e+03_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.41225e+03_rb,0.36102e+03_rb,0.30989e+03_rb,0.26655e+03_rb,0.25418e+03_rb, &
        & 0.26378e+03_rb,0.30615e+03_rb,0.35676e+03_rb,0.40836e+03_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.41130e+03_rb,0.36017e+03_rb,0.30917e+03_rb,0.26471e+03_rb,0.25127e+03_rb, &
        & 0.25962e+03_rb,0.30054e+03_rb,0.35018e+03_rb,0.40087e+03_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.66783e+02_rb,0.58494e+02_rb,0.50207e+02_rb,0.41922e+02_rb,0.39207e+02_rb, &
        & 0.48837e+02_rb,0.58441e+02_rb,0.67973e+02_rb,0.73684e+02_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.66987e+02_rb,0.58663e+02_rb,0.50340e+02_rb,0.42083e+02_rb,0.39992e+02_rb, &
        & 0.49756e+02_rb,0.59507e+02_rb,0.69167e+02_rb,0.75232e+02_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.67128e+02_rb,0.58776e+02_rb,0.50426e+02_rb,0.42412e+02_rb,0.40738e+02_rb, &
        & 0.50635e+02_rb,0.60514e+02_rb,0.70320e+02_rb,0.76487e+02_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.67159e+02_rb,0.58797e+02_rb,0.50435e+02_rb,0.42709e+02_rb,0.41336e+02_rb, &
        & 0.51345e+02_rb,0.61329e+02_rb,0.71239e+02_rb,0.77474e+02_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.67048e+02_rb,0.58698e+02_rb,0.50349e+02_rb,0.42916e+02_rb,0.41806e+02_rb, &
        & 0.51893e+02_rb,0.61955e+02_rb,0.71939e+02_rb,0.78212e+02_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.80817e+02_rb,0.70771e+02_rb,0.60724e+02_rb,0.50680e+02_rb,0.46121e+02_rb, &
        & 0.57561e+02_rb,0.68978e+02_rb,0.80276e+02_rb,0.88125e+02_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.81120e+02_rb,0.71031e+02_rb,0.60942e+02_rb,0.50856e+02_rb,0.46531e+02_rb, &
        & 0.58009e+02_rb,0.69466e+02_rb,0.80811e+02_rb,0.88902e+02_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.81295e+02_rb,0.71179e+02_rb,0.61065e+02_rb,0.50952e+02_rb,0.47046e+02_rb, &
        & 0.58603e+02_rb,0.70144e+02_rb,0.81568e+02_rb,0.89842e+02_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.81302e+02_rb,0.71181e+02_rb,0.61061e+02_rb,0.50998e+02_rb,0.47535e+02_rb, &
        & 0.59172e+02_rb,0.70788e+02_rb,0.82285e+02_rb,0.90723e+02_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.81254e+02_rb,0.71123e+02_rb,0.60992e+02_rb,0.51150e+02_rb,0.47959e+02_rb, &
        & 0.59657e+02_rb,0.71327e+02_rb,0.82908e+02_rb,0.91460e+02_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.97635e+02_rb,0.85483e+02_rb,0.73330e+02_rb,0.61179e+02_rb,0.56207e+02_rb, &
        & 0.70215e+02_rb,0.84188e+02_rb,0.98014e+02_rb,0.10988e+03_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.98163e+02_rb,0.85943e+02_rb,0.73724e+02_rb,0.61504e+02_rb,0.55633e+02_rb, &
        & 0.69472e+02_rb,0.83276e+02_rb,0.96935e+02_rb,0.10869e+03_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.98472e+02_rb,0.86211e+02_rb,0.73951e+02_rb,0.61693e+02_rb,0.55697e+02_rb, &
        & 0.69495e+02_rb,0.83260e+02_rb,0.96885e+02_rb,0.10871e+03_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.98535e+02_rb,0.86265e+02_rb,0.73995e+02_rb,0.61728e+02_rb,0.55937e+02_rb, &
        & 0.69749e+02_rb,0.83530e+02_rb,0.97171e+02_rb,0.10906e+03_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.98433e+02_rb,0.86165e+02_rb,0.73899e+02_rb,0.61635e+02_rb,0.56204e+02_rb, &
        & 0.70040e+02_rb,0.83840e+02_rb,0.97500e+02_rb,0.10946e+03_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.11801e+03_rb,0.10331e+03_rb,0.88600e+02_rb,0.73894e+02_rb,0.68841e+02_rb, &
        & 0.86008e+02_rb,0.10313e+03_rb,0.12007e+03_rb,0.13623e+03_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.11855e+03_rb,0.10378e+03_rb,0.89007e+02_rb,0.74235e+02_rb,0.68009e+02_rb, &
        & 0.84957e+02_rb,0.10186e+03_rb,0.11859e+03_rb,0.13456e+03_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.11911e+03_rb,0.10427e+03_rb,0.89430e+02_rb,0.74590e+02_rb,0.67220e+02_rb, &
        & 0.83955e+02_rb,0.10065e+03_rb,0.11716e+03_rb,0.13292e+03_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.11932e+03_rb,0.10446e+03_rb,0.89591e+02_rb,0.74725e+02_rb,0.66996e+02_rb, &
        & 0.83620e+02_rb,0.10020e+03_rb,0.11661e+03_rb,0.13231e+03_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.11933e+03_rb,0.10445e+03_rb,0.89579e+02_rb,0.74708e+02_rb,0.67016e+02_rb, &
        & 0.83607e+02_rb,0.10016e+03_rb,0.11655e+03_rb,0.13223e+03_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.14253e+03_rb,0.12475e+03_rb,0.10698e+03_rb,0.89200e+02_rb,0.84755e+02_rb, &
        & 0.10590e+03_rb,0.12699e+03_rb,0.14785e+03_rb,0.16865e+03_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.14325e+03_rb,0.12539e+03_rb,0.10752e+03_rb,0.89655e+02_rb,0.83145e+02_rb, &
        & 0.10388e+03_rb,0.12456e+03_rb,0.14501e+03_rb,0.16542e+03_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.14383e+03_rb,0.12589e+03_rb,0.10795e+03_rb,0.90019e+02_rb,0.82092e+02_rb, &
        & 0.10255e+03_rb,0.12296e+03_rb,0.14315e+03_rb,0.16327e+03_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.14418e+03_rb,0.12620e+03_rb,0.10822e+03_rb,0.90248e+02_rb,0.81037e+02_rb, &
        & 0.10121e+03_rb,0.12133e+03_rb,0.14124e+03_rb,0.16108e+03_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.14437e+03_rb,0.12637e+03_rb,0.10836e+03_rb,0.90358e+02_rb,0.80556e+02_rb, &
        & 0.10057e+03_rb,0.12052e+03_rb,0.14027e+03_rb,0.15994e+03_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.17174e+03_rb,0.15031e+03_rb,0.12887e+03_rb,0.10744e+03_rb,0.10459e+03_rb, &
        & 0.13069e+03_rb,0.15672e+03_rb,0.18250e+03_rb,0.20866e+03_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.17281e+03_rb,0.15124e+03_rb,0.12968e+03_rb,0.10811e+03_rb,0.10205e+03_rb, &
        & 0.12751e+03_rb,0.15290e+03_rb,0.17802e+03_rb,0.20356e+03_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.17366e+03_rb,0.15199e+03_rb,0.13032e+03_rb,0.10865e+03_rb,0.10020e+03_rb, &
        & 0.12519e+03_rb,0.15010e+03_rb,0.17475e+03_rb,0.19982e+03_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.17410e+03_rb,0.15237e+03_rb,0.13065e+03_rb,0.10893e+03_rb,0.98757e+02_rb, &
        & 0.12337e+03_rb,0.14792e+03_rb,0.17221e+03_rb,0.19689e+03_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.17426e+03_rb,0.15251e+03_rb,0.13077e+03_rb,0.10903e+03_rb,0.97456e+02_rb, &
        & 0.12171e+03_rb,0.14591e+03_rb,0.16984e+03_rb,0.19419e+03_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.20639e+03_rb,0.18062e+03_rb,0.15485e+03_rb,0.12907e+03_rb,0.12889e+03_rb, &
        & 0.16105e+03_rb,0.19314e+03_rb,0.22488e+03_rb,0.25751e+03_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.20793e+03_rb,0.18197e+03_rb,0.15600e+03_rb,0.13004e+03_rb,0.12536e+03_rb, &
        & 0.15664e+03_rb,0.18784e+03_rb,0.21870e+03_rb,0.25043e+03_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.20918e+03_rb,0.18307e+03_rb,0.15695e+03_rb,0.13084e+03_rb,0.12258e+03_rb, &
        & 0.15317e+03_rb,0.18368e+03_rb,0.21386e+03_rb,0.24487e+03_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.20994e+03_rb,0.18373e+03_rb,0.15753e+03_rb,0.13132e+03_rb,0.12024e+03_rb, &
        & 0.15023e+03_rb,0.18014e+03_rb,0.20974e+03_rb,0.24016e+03_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.21017e+03_rb,0.18393e+03_rb,0.15770e+03_rb,0.13147e+03_rb,0.11817e+03_rb, &
        & 0.14763e+03_rb,0.17700e+03_rb,0.20610e+03_rb,0.23592e+03_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.24808e+03_rb,0.21709e+03_rb,0.18610e+03_rb,0.15511e+03_rb,0.15824e+03_rb, &
        & 0.19773e+03_rb,0.23715e+03_rb,0.27617e+03_rb,0.31647e+03_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.24976e+03_rb,0.21856e+03_rb,0.18737e+03_rb,0.15617e+03_rb,0.15364e+03_rb, &
        & 0.19198e+03_rb,0.23024e+03_rb,0.26808e+03_rb,0.30723e+03_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.25153e+03_rb,0.22011e+03_rb,0.18870e+03_rb,0.15729e+03_rb,0.14986e+03_rb, &
        & 0.18726e+03_rb,0.22456e+03_rb,0.26146e+03_rb,0.29964e+03_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.25266e+03_rb,0.22111e+03_rb,0.18956e+03_rb,0.15801e+03_rb,0.14651e+03_rb, &
        & 0.18308e+03_rb,0.21955e+03_rb,0.25564e+03_rb,0.29297e+03_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.25306e+03_rb,0.22146e+03_rb,0.18986e+03_rb,0.15826e+03_rb,0.14346e+03_rb, &
        & 0.17924e+03_rb,0.21492e+03_rb,0.25024e+03_rb,0.28677e+03_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.29758e+03_rb,0.26040e+03_rb,0.22322e+03_rb,0.18603e+03_rb,0.19348e+03_rb, &
        & 0.24177e+03_rb,0.28996e+03_rb,0.33765e+03_rb,0.38707e+03_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.29980e+03_rb,0.26234e+03_rb,0.22489e+03_rb,0.18744e+03_rb,0.18763e+03_rb, &
        & 0.23446e+03_rb,0.28116e+03_rb,0.32742e+03_rb,0.37536e+03_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.30182e+03_rb,0.26412e+03_rb,0.22641e+03_rb,0.18871e+03_rb,0.18264e+03_rb, &
        & 0.22823e+03_rb,0.27370e+03_rb,0.31874e+03_rb,0.36539e+03_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.30335e+03_rb,0.26546e+03_rb,0.22757e+03_rb,0.18968e+03_rb,0.17814e+03_rb, &
        & 0.22260e+03_rb,0.26695e+03_rb,0.31086e+03_rb,0.35637e+03_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.30399e+03_rb,0.26602e+03_rb,0.22805e+03_rb,0.19008e+03_rb,0.17393e+03_rb, &
        & 0.21734e+03_rb,0.26062e+03_rb,0.30347e+03_rb,0.34789e+03_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.35657e+03_rb,0.31201e+03_rb,0.26745e+03_rb,0.22289e+03_rb,0.23471e+03_rb, &
        & 0.29329e+03_rb,0.35175e+03_rb,0.40960e+03_rb,0.46960e+03_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.35941e+03_rb,0.31450e+03_rb,0.26959e+03_rb,0.22468e+03_rb,0.22747e+03_rb, &
        & 0.28425e+03_rb,0.34089e+03_rb,0.39700e+03_rb,0.45514e+03_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.36173e+03_rb,0.31654e+03_rb,0.27134e+03_rb,0.22614e+03_rb,0.22110e+03_rb, &
        & 0.27628e+03_rb,0.33132e+03_rb,0.38584e+03_rb,0.44234e+03_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.36337e+03_rb,0.31797e+03_rb,0.27257e+03_rb,0.22717e+03_rb,0.21518e+03_rb, &
        & 0.26889e+03_rb,0.32246e+03_rb,0.37553e+03_rb,0.43052e+03_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.36404e+03_rb,0.31855e+03_rb,0.27307e+03_rb,0.22759e+03_rb,0.20959e+03_rb, &
        & 0.26189e+03_rb,0.31406e+03_rb,0.36572e+03_rb,0.41925e+03_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.42778e+03_rb,0.37432e+03_rb,0.32086e+03_rb,0.26740e+03_rb,0.27872e+03_rb, &
        & 0.34830e+03_rb,0.41771e+03_rb,0.48646e+03_rb,0.55765e+03_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.43110e+03_rb,0.37723e+03_rb,0.32336e+03_rb,0.26948e+03_rb,0.27023e+03_rb, &
        & 0.33769e+03_rb,0.40499e+03_rb,0.47168e+03_rb,0.54065e+03_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.43351e+03_rb,0.37933e+03_rb,0.32516e+03_rb,0.27099e+03_rb,0.26235e+03_rb, &
        & 0.32784e+03_rb,0.39315e+03_rb,0.45790e+03_rb,0.52488e+03_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.43444e+03_rb,0.38015e+03_rb,0.32587e+03_rb,0.27158e+03_rb,0.25502e+03_rb, &
        & 0.31866e+03_rb,0.38217e+03_rb,0.44507e+03_rb,0.51019e+03_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.43455e+03_rb,0.38025e+03_rb,0.32595e+03_rb,0.27165e+03_rb,0.24835e+03_rb, &
        & 0.31008e+03_rb,0.37185e+03_rb,0.43305e+03_rb,0.49635e+03_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.51115e+03_rb,0.44727e+03_rb,0.38338e+03_rb,0.31950e+03_rb,0.32868e+03_rb, &
        & 0.41075e+03_rb,0.49264e+03_rb,0.57375e+03_rb,0.65765e+03_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.51481e+03_rb,0.45047e+03_rb,0.38614e+03_rb,0.32180e+03_rb,0.31846e+03_rb, &
        & 0.39795e+03_rb,0.47726e+03_rb,0.55589e+03_rb,0.63715e+03_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.51667e+03_rb,0.45210e+03_rb,0.38753e+03_rb,0.32296e+03_rb,0.30906e+03_rb, &
        & 0.38614e+03_rb,0.46310e+03_rb,0.53942e+03_rb,0.61825e+03_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.51678e+03_rb,0.45219e+03_rb,0.38761e+03_rb,0.32303e+03_rb,0.30046e+03_rb, &
        & 0.37494e+03_rb,0.44968e+03_rb,0.52372e+03_rb,0.60018e+03_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.51600e+03_rb,0.45151e+03_rb,0.38702e+03_rb,0.32254e+03_rb,0.29255e+03_rb, &
        & 0.36403e+03_rb,0.43655e+03_rb,0.50852e+03_rb,0.58277e+03_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.60779e+03_rb,0.53183e+03_rb,0.45587e+03_rb,0.37990e+03_rb,0.38498e+03_rb, &
        & 0.48109e+03_rb,0.57702e+03_rb,0.67207e+03_rb,0.77032e+03_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.61089e+03_rb,0.53454e+03_rb,0.45819e+03_rb,0.38184e+03_rb,0.37288e+03_rb, &
        & 0.46566e+03_rb,0.55850e+03_rb,0.65053e+03_rb,0.74557e+03_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.61168e+03_rb,0.53523e+03_rb,0.45878e+03_rb,0.38233e+03_rb,0.36190e+03_rb, &
        & 0.45112e+03_rb,0.54103e+03_rb,0.63023e+03_rb,0.72223e+03_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.61073e+03_rb,0.53440e+03_rb,0.45807e+03_rb,0.38174e+03_rb,0.35168e+03_rb, &
        & 0.43693e+03_rb,0.52405e+03_rb,0.61044e+03_rb,0.69951e+03_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.60852e+03_rb,0.53247e+03_rb,0.45641e+03_rb,0.38036e+03_rb,0.34198e+03_rb, &
        & 0.42302e+03_rb,0.50734e+03_rb,0.59097e+03_rb,0.67714e+03_rb /)

!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:, 1,13, 1) = (/ &
        & 0.10554e-06_rb,0.38596e-04_rb,0.65402e-04_rb,0.88540e-04_rb,0.19601e-03_rb /)
      kbo(:, 2,13, 1) = (/ &
        & 0.10479e-06_rb,0.43672e-04_rb,0.74889e-04_rb,0.10118e-03_rb,0.22146e-03_rb /)
      kbo(:, 3,13, 1) = (/ &
        & 0.10429e-06_rb,0.49418e-04_rb,0.85429e-04_rb,0.11557e-03_rb,0.24448e-03_rb /)
      kbo(:, 4,13, 1) = (/ &
        & 0.10232e-06_rb,0.55802e-04_rb,0.96459e-04_rb,0.13167e-03_rb,0.27001e-03_rb /)
      kbo(:, 5,13, 1) = (/ &
        & 0.99013e-07_rb,0.62996e-04_rb,0.10900e-03_rb,0.14834e-03_rb,0.29917e-03_rb /)
      kbo(:, 1,14, 1) = (/ &
        & 0.85136e-07_rb,0.31943e-04_rb,0.54402e-04_rb,0.73887e-04_rb,0.15895e-03_rb /)
      kbo(:, 2,14, 1) = (/ &
        & 0.84597e-07_rb,0.36196e-04_rb,0.62429e-04_rb,0.84479e-04_rb,0.17749e-03_rb /)
      kbo(:, 3,14, 1) = (/ &
        & 0.83629e-07_rb,0.40982e-04_rb,0.71126e-04_rb,0.96573e-04_rb,0.19740e-03_rb /)
      kbo(:, 4,14, 1) = (/ &
        & 0.82310e-07_rb,0.46344e-04_rb,0.80458e-04_rb,0.10984e-03_rb,0.21847e-03_rb /)
      kbo(:, 5,14, 1) = (/ &
        & 0.78709e-07_rb,0.52252e-04_rb,0.90989e-04_rb,0.12394e-03_rb,0.24281e-03_rb /)
      kbo(:, 1,15, 1) = (/ &
        & 0.68497e-07_rb,0.26393e-04_rb,0.45164e-04_rb,0.61570e-04_rb,0.12493e-03_rb /)
      kbo(:, 2,15, 1) = (/ &
        & 0.68062e-07_rb,0.29936e-04_rb,0.51919e-04_rb,0.70387e-04_rb,0.13953e-03_rb /)
      kbo(:, 3,15, 1) = (/ &
        & 0.66766e-07_rb,0.33863e-04_rb,0.59056e-04_rb,0.80693e-04_rb,0.15653e-03_rb /)
      kbo(:, 4,15, 1) = (/ &
        & 0.65636e-07_rb,0.38294e-04_rb,0.66885e-04_rb,0.91555e-04_rb,0.17510e-03_rb /)
      kbo(:, 5,15, 1) = (/ &
        & 0.62711e-07_rb,0.43189e-04_rb,0.75756e-04_rb,0.10355e-03_rb,0.19439e-03_rb /)
      kbo(:, 1,16, 1) = (/ &
        & 0.55083e-07_rb,0.21761e-04_rb,0.37484e-04_rb,0.51301e-04_rb,0.97875e-04_rb /)
      kbo(:, 2,16, 1) = (/ &
        & 0.54095e-07_rb,0.24670e-04_rb,0.43084e-04_rb,0.58639e-04_rb,0.10996e-03_rb /)
      kbo(:, 3,16, 1) = (/ &
        & 0.53340e-07_rb,0.27925e-04_rb,0.48976e-04_rb,0.67234e-04_rb,0.12415e-03_rb /)
      kbo(:, 4,16, 1) = (/ &
        & 0.51412e-07_rb,0.31588e-04_rb,0.55576e-04_rb,0.76352e-04_rb,0.13939e-03_rb /)
      kbo(:, 5,16, 1) = (/ &
        & 0.49341e-07_rb,0.35677e-04_rb,0.62937e-04_rb,0.86363e-04_rb,0.15543e-03_rb /)
      kbo(:, 1,17, 1) = (/ &
        & 0.44022e-07_rb,0.17963e-04_rb,0.31102e-04_rb,0.42604e-04_rb,0.78290e-04_rb /)
      kbo(:, 2,17, 1) = (/ &
        & 0.43167e-07_rb,0.20382e-04_rb,0.35754e-04_rb,0.48875e-04_rb,0.87941e-04_rb /)
      kbo(:, 3,17, 1) = (/ &
        & 0.42210e-07_rb,0.23091e-04_rb,0.40674e-04_rb,0.56059e-04_rb,0.99257e-04_rb /)
      kbo(:, 4,17, 1) = (/ &
        & 0.40402e-07_rb,0.26125e-04_rb,0.46162e-04_rb,0.63586e-04_rb,0.11166e-03_rb /)
      kbo(:, 5,17, 1) = (/ &
        & 0.38532e-07_rb,0.29494e-04_rb,0.52327e-04_rb,0.71960e-04_rb,0.12480e-03_rb /)
      kbo(:, 1,18, 1) = (/ &
        & 0.34937e-07_rb,0.14903e-04_rb,0.25868e-04_rb,0.35503e-04_rb,0.63368e-04_rb /)
      kbo(:, 2,18, 1) = (/ &
        & 0.34308e-07_rb,0.16923e-04_rb,0.29781e-04_rb,0.40789e-04_rb,0.71115e-04_rb /)
      kbo(:, 3,18, 1) = (/ &
        & 0.32995e-07_rb,0.19171e-04_rb,0.33872e-04_rb,0.46810e-04_rb,0.80302e-04_rb /)
      kbo(:, 4,18, 1) = (/ &
        & 0.31654e-07_rb,0.21679e-04_rb,0.38468e-04_rb,0.53048e-04_rb,0.90431e-04_rb /)
      kbo(:, 5,18, 1) = (/ &
        & 0.30282e-07_rb,0.24491e-04_rb,0.43596e-04_rb,0.60078e-04_rb,0.10133e-03_rb /)
      kbo(:, 1,19, 1) = (/ &
        & 0.27971e-07_rb,0.12385e-04_rb,0.21549e-04_rb,0.29618e-04_rb,0.51732e-04_rb /)
      kbo(:, 2,19, 1) = (/ &
        & 0.27128e-07_rb,0.14042e-04_rb,0.24826e-04_rb,0.34047e-04_rb,0.58064e-04_rb /)
      kbo(:, 3,19, 1) = (/ &
        & 0.26018e-07_rb,0.15920e-04_rb,0.28221e-04_rb,0.39072e-04_rb,0.65495e-04_rb /)
      kbo(:, 4,19, 1) = (/ &
        & 0.24717e-07_rb,0.18017e-04_rb,0.32056e-04_rb,0.44285e-04_rb,0.73859e-04_rb /)
      kbo(:, 5,19, 1) = (/ &
        & 0.23859e-07_rb,0.20360e-04_rb,0.36341e-04_rb,0.50170e-04_rb,0.82944e-04_rb /)
      kbo(:, 1,20, 1) = (/ &
        & 0.22529e-07_rb,0.10336e-04_rb,0.17999e-04_rb,0.24776e-04_rb,0.42602e-04_rb /)
      kbo(:, 2,20, 1) = (/ &
        & 0.21733e-07_rb,0.11685e-04_rb,0.20769e-04_rb,0.28503e-04_rb,0.47833e-04_rb /)
      kbo(:, 3,20, 1) = (/ &
        & 0.20791e-07_rb,0.13280e-04_rb,0.23571e-04_rb,0.32654e-04_rb,0.53995e-04_rb /)
      kbo(:, 4,20, 1) = (/ &
        & 0.19663e-07_rb,0.15029e-04_rb,0.26788e-04_rb,0.37066e-04_rb,0.60944e-04_rb /)
      kbo(:, 5,20, 1) = (/ &
        & 0.19169e-07_rb,0.16975e-04_rb,0.30353e-04_rb,0.41996e-04_rb,0.68344e-04_rb /)
      kbo(:, 1,21, 1) = (/ &
        & 0.18148e-07_rb,0.86039e-05_rb,0.15041e-04_rb,0.20717e-04_rb,0.35143e-04_rb /)
      kbo(:, 2,21, 1) = (/ &
        & 0.17465e-07_rb,0.97287e-05_rb,0.17348e-04_rb,0.23862e-04_rb,0.39508e-04_rb /)
      kbo(:, 3,21, 1) = (/ &
        & 0.16537e-07_rb,0.11073e-04_rb,0.19687e-04_rb,0.27271e-04_rb,0.44652e-04_rb /)
      kbo(:, 4,21, 1) = (/ &
        & 0.15860e-07_rb,0.12526e-04_rb,0.22388e-04_rb,0.31015e-04_rb,0.50269e-04_rb /)
      kbo(:, 5,21, 1) = (/ &
        & 0.15452e-07_rb,0.14158e-04_rb,0.25339e-04_rb,0.35141e-04_rb,0.56327e-04_rb /)
      kbo(:, 1,22, 1) = (/ &
        & 0.14542e-07_rb,0.71807e-05_rb,0.12682e-04_rb,0.17473e-04_rb,0.29179e-04_rb /)
      kbo(:, 2,22, 1) = (/ &
        & 0.13967e-07_rb,0.81574e-05_rb,0.14550e-04_rb,0.20147e-04_rb,0.32795e-04_rb /)
      kbo(:, 3,22, 1) = (/ &
        & 0.13241e-07_rb,0.92839e-05_rb,0.16547e-04_rb,0.22971e-04_rb,0.37036e-04_rb /)
      kbo(:, 4,22, 1) = (/ &
        & 0.12746e-07_rb,0.10522e-04_rb,0.18840e-04_rb,0.26114e-04_rb,0.41617e-04_rb /)
      kbo(:, 5,22, 1) = (/ &
        & 0.12426e-07_rb,0.11886e-04_rb,0.21313e-04_rb,0.29604e-04_rb,0.46709e-04_rb /)
      kbo(:, 1,23, 1) = (/ &
        & 0.11724e-07_rb,0.60196e-05_rb,0.10703e-04_rb,0.14753e-04_rb,0.24143e-04_rb /)
      kbo(:, 2,23, 1) = (/ &
        & 0.11124e-07_rb,0.68499e-05_rb,0.12233e-04_rb,0.17009e-04_rb,0.27147e-04_rb /)
      kbo(:, 3,23, 1) = (/ &
        & 0.10552e-07_rb,0.77897e-05_rb,0.13927e-04_rb,0.19362e-04_rb,0.30661e-04_rb /)
      kbo(:, 4,23, 1) = (/ &
        & 0.10213e-07_rb,0.88398e-05_rb,0.15850e-04_rb,0.22024e-04_rb,0.34519e-04_rb /)
      kbo(:, 5,23, 1) = (/ &
        & 0.99627e-08_rb,0.99755e-05_rb,0.17919e-04_rb,0.24944e-04_rb,0.38730e-04_rb /)
      kbo(:, 1,24, 1) = (/ &
        & 0.93835e-08_rb,0.50505e-05_rb,0.90298e-05_rb,0.12450e-04_rb,0.19940e-04_rb /)
      kbo(:, 2,24, 1) = (/ &
        & 0.88722e-08_rb,0.57494e-05_rb,0.10287e-04_rb,0.14351e-04_rb,0.22450e-04_rb /)
      kbo(:, 3,24, 1) = (/ &
        & 0.84719e-08_rb,0.65374e-05_rb,0.11727e-04_rb,0.16329e-04_rb,0.25378e-04_rb /)
      kbo(:, 4,24, 1) = (/ &
        & 0.81924e-08_rb,0.74186e-05_rb,0.13334e-04_rb,0.18567e-04_rb,0.28560e-04_rb /)
      kbo(:, 5,24, 1) = (/ &
        & 0.79194e-08_rb,0.83719e-05_rb,0.15056e-04_rb,0.21003e-04_rb,0.31990e-04_rb /)
      kbo(:, 1,25, 1) = (/ &
        & 0.74535e-08_rb,0.42437e-05_rb,0.76220e-05_rb,0.10523e-04_rb,0.16531e-04_rb /)
      kbo(:, 2,25, 1) = (/ &
        & 0.70516e-08_rb,0.48300e-05_rb,0.86667e-05_rb,0.12089e-04_rb,0.18616e-04_rb /)
      kbo(:, 3,25, 1) = (/ &
        & 0.67746e-08_rb,0.54941e-05_rb,0.98927e-05_rb,0.13790e-04_rb,0.21024e-04_rb /)
      kbo(:, 4,25, 1) = (/ &
        & 0.65375e-08_rb,0.62357e-05_rb,0.11233e-04_rb,0.15662e-04_rb,0.23642e-04_rb /)
      kbo(:, 5,25, 1) = (/ &
        & 0.63001e-08_rb,0.70282e-05_rb,0.12669e-04_rb,0.17695e-04_rb,0.26478e-04_rb /)
      kbo(:, 1,26, 1) = (/ &
        & 0.59520e-08_rb,0.35739e-05_rb,0.64350e-05_rb,0.89330e-05_rb,0.13754e-04_rb /)
      kbo(:, 2,26, 1) = (/ &
        & 0.56644e-08_rb,0.40797e-05_rb,0.73291e-05_rb,0.10228e-04_rb,0.15489e-04_rb /)
      kbo(:, 3,26, 1) = (/ &
        & 0.54229e-08_rb,0.46366e-05_rb,0.83664e-05_rb,0.11659e-04_rb,0.17486e-04_rb /)
      kbo(:, 4,26, 1) = (/ &
        & 0.52155e-08_rb,0.52546e-05_rb,0.94829e-05_rb,0.13246e-04_rb,0.19643e-04_rb /)
      kbo(:, 5,26, 1) = (/ &
        & 0.50026e-08_rb,0.59134e-05_rb,0.10676e-04_rb,0.14933e-04_rb,0.21979e-04_rb /)
      kbo(:, 1,27, 1) = (/ &
        & 0.47347e-08_rb,0.30117e-05_rb,0.54223e-05_rb,0.75810e-05_rb,0.11424e-04_rb /)
      kbo(:, 2,27, 1) = (/ &
        & 0.45215e-08_rb,0.34421e-05_rb,0.61959e-05_rb,0.86619e-05_rb,0.12884e-04_rb /)
      kbo(:, 3,27, 1) = (/ &
        & 0.43334e-08_rb,0.39143e-05_rb,0.70702e-05_rb,0.98761e-05_rb,0.14543e-04_rb /)
      kbo(:, 4,27, 1) = (/ &
        & 0.41350e-08_rb,0.44275e-05_rb,0.80005e-05_rb,0.11195e-04_rb,0.16319e-04_rb /)
      kbo(:, 5,27, 1) = (/ &
        & 0.39653e-08_rb,0.49769e-05_rb,0.89931e-05_rb,0.12598e-04_rb,0.18160e-04_rb /)
      kbo(:, 1,28, 1) = (/ &
        & 0.37948e-08_rb,0.25415e-05_rb,0.45819e-05_rb,0.64336e-05_rb,0.94956e-05_rb /)
      kbo(:, 2,28, 1) = (/ &
        & 0.36083e-08_rb,0.29038e-05_rb,0.52426e-05_rb,0.73400e-05_rb,0.10719e-04_rb /)
      kbo(:, 3,28, 1) = (/ &
        & 0.34418e-08_rb,0.33017e-05_rb,0.59733e-05_rb,0.83575e-05_rb,0.12066e-04_rb /)
      kbo(:, 4,28, 1) = (/ &
        & 0.32636e-08_rb,0.37297e-05_rb,0.67493e-05_rb,0.94558e-05_rb,0.13468e-04_rb /)
      kbo(:, 5,28, 1) = (/ &
        & 0.31340e-08_rb,0.41873e-05_rb,0.75759e-05_rb,0.10620e-04_rb,0.14953e-04_rb /)
      kbo(:, 1,29, 1) = (/ &
        & 0.30121e-08_rb,0.21536e-05_rb,0.38841e-05_rb,0.54491e-05_rb,0.79090e-05_rb /)
      kbo(:, 2,29, 1) = (/ &
        & 0.28775e-08_rb,0.24557e-05_rb,0.44467e-05_rb,0.62247e-05_rb,0.89148e-05_rb /)
      kbo(:, 3,29, 1) = (/ &
        & 0.27196e-08_rb,0.27900e-05_rb,0.50543e-05_rb,0.70782e-05_rb,0.99943e-05_rb /)
      kbo(:, 4,29, 1) = (/ &
        & 0.25798e-08_rb,0.31473e-05_rb,0.57011e-05_rb,0.79936e-05_rb,0.11135e-04_rb /)
      kbo(:, 5,29, 1) = (/ &
        & 0.24785e-08_rb,0.35287e-05_rb,0.63894e-05_rb,0.89624e-05_rb,0.12357e-04_rb /)
      kbo(:, 1,30, 1) = (/ &
        & 0.24034e-08_rb,0.18231e-05_rb,0.32945e-05_rb,0.46183e-05_rb,0.65837e-05_rb /)
      kbo(:, 2,30, 1) = (/ &
        & 0.22702e-08_rb,0.20766e-05_rb,0.37671e-05_rb,0.52735e-05_rb,0.73875e-05_rb /)
      kbo(:, 3,30, 1) = (/ &
        & 0.21551e-08_rb,0.23559e-05_rb,0.42744e-05_rb,0.59932e-05_rb,0.82665e-05_rb /)
      kbo(:, 4,30, 1) = (/ &
        & 0.20397e-08_rb,0.26541e-05_rb,0.48137e-05_rb,0.67549e-05_rb,0.92071e-05_rb /)
      kbo(:, 5,30, 1) = (/ &
        & 0.19208e-08_rb,0.29711e-05_rb,0.53880e-05_rb,0.75598e-05_rb,0.10211e-04_rb /)
      kbo(:, 1,31, 1) = (/ &
        & 0.19116e-08_rb,0.15454e-05_rb,0.27957e-05_rb,0.39228e-05_rb,0.54664e-05_rb /)
      kbo(:, 2,31, 1) = (/ &
        & 0.18045e-08_rb,0.17579e-05_rb,0.31938e-05_rb,0.44780e-05_rb,0.61205e-05_rb /)
      kbo(:, 3,31, 1) = (/ &
        & 0.17015e-08_rb,0.19911e-05_rb,0.36167e-05_rb,0.50753e-05_rb,0.68523e-05_rb /)
      kbo(:, 4,31, 1) = (/ &
        & 0.16042e-08_rb,0.22393e-05_rb,0.40666e-05_rb,0.57094e-05_rb,0.76260e-05_rb /)
      kbo(:, 5,31, 1) = (/ &
        & 0.15189e-08_rb,0.25035e-05_rb,0.45439e-05_rb,0.63785e-05_rb,0.84562e-05_rb /)
      kbo(:, 1,32, 1) = (/ &
        & 0.15134e-08_rb,0.13080e-05_rb,0.23763e-05_rb,0.33360e-05_rb,0.45382e-05_rb /)
      kbo(:, 2,32, 1) = (/ &
        & 0.14254e-08_rb,0.14886e-05_rb,0.27086e-05_rb,0.37995e-05_rb,0.50938e-05_rb /)
      kbo(:, 3,32, 1) = (/ &
        & 0.13478e-08_rb,0.16831e-05_rb,0.30605e-05_rb,0.42971e-05_rb,0.56933e-05_rb /)
      kbo(:, 4,32, 1) = (/ &
        & 0.12726e-08_rb,0.18904e-05_rb,0.34353e-05_rb,0.48247e-05_rb,0.63353e-05_rb /)
      kbo(:, 5,32, 1) = (/ &
        & 0.12027e-08_rb,0.21098e-05_rb,0.38318e-05_rb,0.53809e-05_rb,0.70302e-05_rb /)
      kbo(:, 1,33, 1) = (/ &
        & 0.12024e-08_rb,0.11084e-05_rb,0.20193e-05_rb,0.28310e-05_rb,0.37849e-05_rb /)
      kbo(:, 2,33, 1) = (/ &
        & 0.11338e-08_rb,0.12605e-05_rb,0.22949e-05_rb,0.32221e-05_rb,0.42458e-05_rb /)
      kbo(:, 3,33, 1) = (/ &
        & 0.10678e-08_rb,0.14231e-05_rb,0.25881e-05_rb,0.36365e-05_rb,0.47460e-05_rb /)
      kbo(:, 4,33, 1) = (/ &
        & 0.10052e-08_rb,0.15963e-05_rb,0.29010e-05_rb,0.40728e-05_rb,0.52804e-05_rb /)
      kbo(:, 5,33, 1) = (/ &
        & 0.95946e-09_rb,0.17779e-05_rb,0.32301e-05_rb,0.45381e-05_rb,0.58547e-05_rb /)
      kbo(:, 1,34, 1) = (/ &
        & 0.95750e-09_rb,0.93665e-06_rb,0.17068e-05_rb,0.23954e-05_rb,0.31611e-05_rb /)
      kbo(:, 2,34, 1) = (/ &
        & 0.89991e-09_rb,0.10636e-05_rb,0.19366e-05_rb,0.27209e-05_rb,0.35440e-05_rb /)
      kbo(:, 3,34, 1) = (/ &
        & 0.84944e-09_rb,0.11994e-05_rb,0.21804e-05_rb,0.30651e-05_rb,0.39600e-05_rb /)
      kbo(:, 4,34, 1) = (/ &
        & 0.80863e-09_rb,0.13431e-05_rb,0.24406e-05_rb,0.34285e-05_rb,0.44045e-05_rb /)
      kbo(:, 5,34, 1) = (/ &
        & 0.76697e-09_rb,0.14937e-05_rb,0.27138e-05_rb,0.38153e-05_rb,0.48739e-05_rb /)
      kbo(:, 1,35, 1) = (/ &
        & 0.76931e-09_rb,0.78145e-06_rb,0.14243e-05_rb,0.20004e-05_rb,0.26088e-05_rb /)
      kbo(:, 2,35, 1) = (/ &
        & 0.72554e-09_rb,0.88652e-06_rb,0.16140e-05_rb,0.22684e-05_rb,0.29255e-05_rb /)
      kbo(:, 3,35, 1) = (/ &
        & 0.68677e-09_rb,0.99872e-06_rb,0.18159e-05_rb,0.25537e-05_rb,0.32672e-05_rb /)
      kbo(:, 4,35, 1) = (/ &
        & 0.64986e-09_rb,0.11178e-05_rb,0.20312e-05_rb,0.28542e-05_rb,0.36304e-05_rb /)
      kbo(:, 5,35, 1) = (/ &
        & 0.62177e-09_rb,0.12420e-05_rb,0.22566e-05_rb,0.31737e-05_rb,0.40187e-05_rb /)
      kbo(:, 1,36, 1) = (/ &
        & 0.62245e-09_rb,0.64235e-06_rb,0.11709e-05_rb,0.16445e-05_rb,0.21213e-05_rb /)
      kbo(:, 2,36, 1) = (/ &
        & 0.58696e-09_rb,0.72876e-06_rb,0.13266e-05_rb,0.18646e-05_rb,0.23798e-05_rb /)
      kbo(:, 3,36, 1) = (/ &
        & 0.55693e-09_rb,0.82089e-06_rb,0.14927e-05_rb,0.20991e-05_rb,0.26600e-05_rb /)
      kbo(:, 4,36, 1) = (/ &
        & 0.52934e-09_rb,0.91874e-06_rb,0.16695e-05_rb,0.23457e-05_rb,0.29575e-05_rb /)
      kbo(:, 5,36, 1) = (/ &
        & 0.50585e-09_rb,0.10210e-05_rb,0.18549e-05_rb,0.26085e-05_rb,0.32758e-05_rb /)
      kbo(:, 1,37, 1) = (/ &
        & 0.51156e-09_rb,0.51872e-06_rb,0.94550e-06_rb,0.13287e-05_rb,0.17089e-05_rb /)
      kbo(:, 2,37, 1) = (/ &
        & 0.48258e-09_rb,0.58908e-06_rb,0.10722e-05_rb,0.15079e-05_rb,0.19187e-05_rb /)
      kbo(:, 3,37, 1) = (/ &
        & 0.45694e-09_rb,0.66421e-06_rb,0.12078e-05_rb,0.16992e-05_rb,0.21456e-05_rb /)
      kbo(:, 4,37, 1) = (/ &
        & 0.43502e-09_rb,0.74408e-06_rb,0.13520e-05_rb,0.19000e-05_rb,0.23876e-05_rb /)
      kbo(:, 5,37, 1) = (/ &
        & 0.41517e-09_rb,0.82758e-06_rb,0.15034e-05_rb,0.21146e-05_rb,0.26473e-05_rb /)
      kbo(:, 1,38, 1) = (/ &
        & 0.42094e-09_rb,0.41837e-06_rb,0.76260e-06_rb,0.10719e-05_rb,0.13756e-05_rb /)
      kbo(:, 2,38, 1) = (/ &
        & 0.39700e-09_rb,0.47557e-06_rb,0.86572e-06_rb,0.12180e-05_rb,0.15462e-05_rb /)
      kbo(:, 3,38, 1) = (/ &
        & 0.37563e-09_rb,0.53684e-06_rb,0.97613e-06_rb,0.13738e-05_rb,0.17289e-05_rb /)
      kbo(:, 4,38, 1) = (/ &
        & 0.35783e-09_rb,0.60193e-06_rb,0.10936e-05_rb,0.15374e-05_rb,0.19252e-05_rb /)
      kbo(:, 5,38, 1) = (/ &
        & 0.34107e-09_rb,0.67014e-06_rb,0.12173e-05_rb,0.17127e-05_rb,0.21369e-05_rb /)
      kbo(:, 1,39, 1) = (/ &
        & 0.34635e-09_rb,0.33732e-06_rb,0.61492e-06_rb,0.86447e-06_rb,0.11055e-05_rb /)
      kbo(:, 2,39, 1) = (/ &
        & 0.32654e-09_rb,0.38383e-06_rb,0.69869e-06_rb,0.98355e-06_rb,0.12440e-05_rb /)
      kbo(:, 3,39, 1) = (/ &
        & 0.30909e-09_rb,0.43371e-06_rb,0.78869e-06_rb,0.11105e-05_rb,0.13924e-05_rb /)
      kbo(:, 4,39, 1) = (/ &
        & 0.29425e-09_rb,0.48681e-06_rb,0.88446e-06_rb,0.12433e-05_rb,0.15526e-05_rb /)
      kbo(:, 5,39, 1) = (/ &
        & 0.28000e-09_rb,0.54256e-06_rb,0.98550e-06_rb,0.13868e-05_rb,0.17238e-05_rb /)
      kbo(:, 1,40, 1) = (/ &
        & 0.28731e-09_rb,0.26843e-06_rb,0.48915e-06_rb,0.68810e-06_rb,0.87867e-06_rb /)
      kbo(:, 2,40, 1) = (/ &
        & 0.27089e-09_rb,0.30600e-06_rb,0.55698e-06_rb,0.78430e-06_rb,0.99075e-06_rb /)
      kbo(:, 3,40, 1) = (/ &
        & 0.25607e-09_rb,0.34624e-06_rb,0.62966e-06_rb,0.88677e-06_rb,0.11107e-05_rb /)
      kbo(:, 4,40, 1) = (/ &
        & 0.24372e-09_rb,0.38926e-06_rb,0.70732e-06_rb,0.99430e-06_rb,0.12402e-05_rb /)
      kbo(:, 5,40, 1) = (/ &
        & 0.23176e-09_rb,0.43465e-06_rb,0.78943e-06_rb,0.11107e-05_rb,0.13787e-05_rb /)
      kbo(:, 1,41, 1) = (/ &
        & 0.23903e-09_rb,0.21323e-06_rb,0.38855e-06_rb,0.54701e-06_rb,0.69781e-06_rb /)
      kbo(:, 2,41, 1) = (/ &
        & 0.22487e-09_rb,0.24332e-06_rb,0.44328e-06_rb,0.62397e-06_rb,0.78774e-06_rb /)
      kbo(:, 3,41, 1) = (/ &
        & 0.21236e-09_rb,0.27581e-06_rb,0.50170e-06_rb,0.70664e-06_rb,0.88427e-06_rb /)
      kbo(:, 4,41, 1) = (/ &
        & 0.20204e-09_rb,0.31066e-06_rb,0.56435e-06_rb,0.79384e-06_rb,0.98899e-06_rb /)
      kbo(:, 5,41, 1) = (/ &
        & 0.19209e-09_rb,0.34753e-06_rb,0.63119e-06_rb,0.88787e-06_rb,0.11007e-05_rb /)
      kbo(:, 1,42, 1) = (/ &
        & 0.19868e-09_rb,0.16950e-06_rb,0.30885e-06_rb,0.43426e-06_rb,0.55336e-06_rb /)
      kbo(:, 2,42, 1) = (/ &
        & 0.18681e-09_rb,0.19338e-06_rb,0.35250e-06_rb,0.49622e-06_rb,0.62628e-06_rb /)
      kbo(:, 3,42, 1) = (/ &
        & 0.17649e-09_rb,0.21963e-06_rb,0.39950e-06_rb,0.56269e-06_rb,0.70396e-06_rb /)
      kbo(:, 4,42, 1) = (/ &
        & 0.16753e-09_rb,0.24778e-06_rb,0.45013e-06_rb,0.63342e-06_rb,0.78770e-06_rb /)
      kbo(:, 5,42, 1) = (/ &
        & 0.15930e-09_rb,0.27772e-06_rb,0.50439e-06_rb,0.70945e-06_rb,0.87783e-06_rb /)
      kbo(:, 1,43, 1) = (/ &
        & 0.16577e-09_rb,0.13332e-06_rb,0.24323e-06_rb,0.34184e-06_rb,0.43594e-06_rb /)
      kbo(:, 2,43, 1) = (/ &
        & 0.15597e-09_rb,0.15245e-06_rb,0.27801e-06_rb,0.39130e-06_rb,0.49460e-06_rb /)
      kbo(:, 3,43, 1) = (/ &
        & 0.14721e-09_rb,0.17354e-06_rb,0.31584e-06_rb,0.44499e-06_rb,0.55705e-06_rb /)
      kbo(:, 4,43, 1) = (/ &
        & 0.13936e-09_rb,0.19625e-06_rb,0.35661e-06_rb,0.50189e-06_rb,0.62398e-06_rb /)
      kbo(:, 5,43, 1) = (/ &
        & 0.13255e-09_rb,0.22044e-06_rb,0.40034e-06_rb,0.56308e-06_rb,0.69634e-06_rb /)
      kbo(:, 1,44, 1) = (/ &
        & 0.13861e-09_rb,0.10445e-06_rb,0.19058e-06_rb,0.26841e-06_rb,0.34233e-06_rb /)
      kbo(:, 2,44, 1) = (/ &
        & 0.13027e-09_rb,0.11974e-06_rb,0.21836e-06_rb,0.30744e-06_rb,0.38897e-06_rb /)
      kbo(:, 3,44, 1) = (/ &
        & 0.12290e-09_rb,0.13662e-06_rb,0.24864e-06_rb,0.35065e-06_rb,0.43916e-06_rb /)
      kbo(:, 4,44, 1) = (/ &
        & 0.11611e-09_rb,0.15484e-06_rb,0.28155e-06_rb,0.39627e-06_rb,0.49273e-06_rb /)
      kbo(:, 5,44, 1) = (/ &
        & 0.11042e-09_rb,0.17431e-06_rb,0.31666e-06_rb,0.44540e-06_rb,0.55087e-06_rb /)
      kbo(:, 1,45, 1) = (/ &
        & 0.11581e-09_rb,0.81915e-07_rb,0.14915e-06_rb,0.21140e-06_rb,0.26987e-06_rb /)
      kbo(:, 2,45, 1) = (/ &
        & 0.10906e-09_rb,0.94077e-07_rb,0.17143e-06_rb,0.24173e-06_rb,0.30586e-06_rb /)
      kbo(:, 3,45, 1) = (/ &
        & 0.10254e-09_rb,0.10745e-06_rb,0.19582e-06_rb,0.27606e-06_rb,0.34593e-06_rb /)
      kbo(:, 4,45, 1) = (/ &
        & 0.96757e-10_rb,0.12205e-06_rb,0.22201e-06_rb,0.31271e-06_rb,0.38876e-06_rb /)
      kbo(:, 5,45, 1) = (/ &
        & 0.91979e-10_rb,0.13772e-06_rb,0.25015e-06_rb,0.35208e-06_rb,0.43561e-06_rb /)
      kbo(:, 1,46, 1) = (/ &
        & 0.96889e-10_rb,0.64116e-07_rb,0.11708e-06_rb,0.16455e-06_rb,0.21036e-06_rb /)
      kbo(:, 2,46, 1) = (/ &
        & 0.91281e-10_rb,0.73641e-07_rb,0.13436e-06_rb,0.18895e-06_rb,0.23951e-06_rb /)
      kbo(:, 3,46, 1) = (/ &
        & 0.85830e-10_rb,0.84161e-07_rb,0.15360e-06_rb,0.21643e-06_rb,0.27176e-06_rb /)
      kbo(:, 4,46, 1) = (/ &
        & 0.81048e-10_rb,0.95867e-07_rb,0.17447e-06_rb,0.24582e-06_rb,0.30613e-06_rb /)
      kbo(:, 5,46, 1) = (/ &
        & 0.76712e-10_rb,0.10844e-06_rb,0.19706e-06_rb,0.27743e-06_rb,0.34329e-06_rb /)
      kbo(:, 1,47, 1) = (/ &
        & 0.81516e-10_rb,0.49698e-07_rb,0.90596e-07_rb,0.12727e-06_rb,0.16313e-06_rb /)
      kbo(:, 2,47, 1) = (/ &
        & 0.76609e-10_rb,0.57132e-07_rb,0.10433e-06_rb,0.14694e-06_rb,0.18672e-06_rb /)
      kbo(:, 3,47, 1) = (/ &
        & 0.71975e-10_rb,0.65534e-07_rb,0.11955e-06_rb,0.16850e-06_rb,0.21215e-06_rb /)
      kbo(:, 4,47, 1) = (/ &
        & 0.67908e-10_rb,0.74829e-07_rb,0.13619e-06_rb,0.19209e-06_rb,0.23959e-06_rb /)
      kbo(:, 5,47, 1) = (/ &
        & 0.64117e-10_rb,0.84872e-07_rb,0.15433e-06_rb,0.21733e-06_rb,0.26915e-06_rb /)
      kbo(:, 1,48, 1) = (/ &
        & 0.68429e-10_rb,0.38715e-07_rb,0.69854e-07_rb,0.98377e-07_rb,0.12619e-06_rb /)
      kbo(:, 2,48, 1) = (/ &
        & 0.64236e-10_rb,0.44375e-07_rb,0.80895e-07_rb,0.11471e-06_rb,0.14604e-06_rb /)
      kbo(:, 3,48, 1) = (/ &
        & 0.60510e-10_rb,0.51025e-07_rb,0.92977e-07_rb,0.13123e-06_rb,0.16542e-06_rb /)
      kbo(:, 4,48, 1) = (/ &
        & 0.56851e-10_rb,0.58345e-07_rb,0.10633e-06_rb,0.14991e-06_rb,0.18745e-06_rb /)
      kbo(:, 5,48, 1) = (/ &
        & 0.53674e-10_rb,0.66346e-07_rb,0.12070e-06_rb,0.17001e-06_rb,0.21095e-06_rb /)
      kbo(:, 1,49, 1) = (/ &
        & 0.57387e-10_rb,0.29800e-07_rb,0.53817e-07_rb,0.75906e-07_rb,0.97710e-07_rb /)
      kbo(:, 2,49, 1) = (/ &
        & 0.53960e-10_rb,0.34500e-07_rb,0.63205e-07_rb,0.88655e-07_rb,0.11310e-06_rb /)
      kbo(:, 3,49, 1) = (/ &
        & 0.50740e-10_rb,0.39684e-07_rb,0.72502e-07_rb,0.10193e-06_rb,0.12892e-06_rb /)
      kbo(:, 4,49, 1) = (/ &
        & 0.47710e-10_rb,0.45418e-07_rb,0.82941e-07_rb,0.11692e-06_rb,0.14649e-06_rb /)
      kbo(:, 5,49, 1) = (/ &
        & 0.45050e-10_rb,0.51797e-07_rb,0.94320e-07_rb,0.13296e-06_rb,0.16526e-06_rb /)
      kbo(:, 1,50, 1) = (/ &
        & 0.48010e-10_rb,0.22999e-07_rb,0.41682e-07_rb,0.58975e-07_rb,0.75898e-07_rb /)
      kbo(:, 2,50, 1) = (/ &
        & 0.45282e-10_rb,0.26917e-07_rb,0.48929e-07_rb,0.68765e-07_rb,0.87983e-07_rb /)
      kbo(:, 3,50, 1) = (/ &
        & 0.42531e-10_rb,0.30879e-07_rb,0.56406e-07_rb,0.79593e-07_rb,0.10109e-06_rb /)
      kbo(:, 4,50, 1) = (/ &
        & 0.40002e-10_rb,0.35470e-07_rb,0.64706e-07_rb,0.91312e-07_rb,0.11473e-06_rb /)
      kbo(:, 5,50, 1) = (/ &
        & 0.37674e-10_rb,0.40542e-07_rb,0.73834e-07_rb,0.10412e-06_rb,0.12967e-06_rb /)
      kbo(:, 1,51, 1) = (/ &
        & 0.40094e-10_rb,0.17810e-07_rb,0.32412e-07_rb,0.45985e-07_rb,0.59041e-07_rb /)
      kbo(:, 2,51, 1) = (/ &
        & 0.37901e-10_rb,0.21013e-07_rb,0.37921e-07_rb,0.53479e-07_rb,0.68479e-07_rb /)
      kbo(:, 3,51, 1) = (/ &
        & 0.35584e-10_rb,0.24100e-07_rb,0.43988e-07_rb,0.62409e-07_rb,0.79311e-07_rb /)
      kbo(:, 4,51, 1) = (/ &
        & 0.33516e-10_rb,0.27737e-07_rb,0.50544e-07_rb,0.71436e-07_rb,0.89912e-07_rb /)
      kbo(:, 5,51, 1) = (/ &
        & 0.31503e-10_rb,0.31738e-07_rb,0.57889e-07_rb,0.81591e-07_rb,0.10195e-06_rb /)
      kbo(:, 1,52, 1) = (/ &
        & 0.33462e-10_rb,0.13829e-07_rb,0.25227e-07_rb,0.35498e-07_rb,0.45793e-07_rb /)
      kbo(:, 2,52, 1) = (/ &
        & 0.31703e-10_rb,0.16272e-07_rb,0.29393e-07_rb,0.41476e-07_rb,0.53340e-07_rb /)
      kbo(:, 3,52, 1) = (/ &
        & 0.29856e-10_rb,0.18836e-07_rb,0.34537e-07_rb,0.48490e-07_rb,0.61791e-07_rb /)
      kbo(:, 4,52, 1) = (/ &
        & 0.28038e-10_rb,0.21684e-07_rb,0.39578e-07_rb,0.55751e-07_rb,0.70422e-07_rb /)
      kbo(:, 5,52, 1) = (/ &
        & 0.26375e-10_rb,0.24824e-07_rb,0.45389e-07_rb,0.63898e-07_rb,0.80071e-07_rb /)
      kbo(:, 1,53, 1) = (/ &
        & 0.27859e-10_rb,0.10757e-07_rb,0.19436e-07_rb,0.27368e-07_rb,0.35401e-07_rb /)
      kbo(:, 2,53, 1) = (/ &
        & 0.26482e-10_rb,0.12575e-07_rb,0.22786e-07_rb,0.32275e-07_rb,0.41504e-07_rb /)
      kbo(:, 3,53, 1) = (/ &
        & 0.25013e-10_rb,0.14718e-07_rb,0.26786e-07_rb,0.37657e-07_rb,0.48150e-07_rb /)
      kbo(:, 4,53, 1) = (/ &
        & 0.23481e-10_rb,0.16890e-07_rb,0.30875e-07_rb,0.43561e-07_rb,0.55284e-07_rb /)
      kbo(:, 5,53, 1) = (/ &
        & 0.22103e-10_rb,0.19405e-07_rb,0.35435e-07_rb,0.50046e-07_rb,0.62824e-07_rb /)
      kbo(:, 1,54, 1) = (/ &
        & 0.23262e-10_rb,0.83534e-08_rb,0.15033e-07_rb,0.21142e-07_rb,0.27491e-07_rb /)
      kbo(:, 2,54, 1) = (/ &
        & 0.22075e-10_rb,0.97821e-08_rb,0.17791e-07_rb,0.25298e-07_rb,0.32447e-07_rb /)
      kbo(:, 3,54, 1) = (/ &
        & 0.20887e-10_rb,0.11541e-07_rb,0.20852e-07_rb,0.29392e-07_rb,0.37643e-07_rb /)
      kbo(:, 4,54, 1) = (/ &
        & 0.19629e-10_rb,0.13228e-07_rb,0.24164e-07_rb,0.34330e-07_rb,0.43622e-07_rb /)
      kbo(:, 5,54, 1) = (/ &
        & 0.18475e-10_rb,0.15233e-07_rb,0.27779e-07_rb,0.39304e-07_rb,0.49432e-07_rb /)
      kbo(:, 1,55, 1) = (/ &
        & 0.19360e-10_rb,0.64888e-08_rb,0.11644e-07_rb,0.16387e-07_rb,0.21340e-07_rb /)
      kbo(:, 2,55, 1) = (/ &
        & 0.18372e-10_rb,0.76367e-08_rb,0.13947e-07_rb,0.19672e-07_rb,0.25401e-07_rb /)
      kbo(:, 3,55, 1) = (/ &
        & 0.17421e-10_rb,0.90083e-08_rb,0.16275e-07_rb,0.22971e-07_rb,0.29522e-07_rb /)
      kbo(:, 4,55, 1) = (/ &
        & 0.16413e-10_rb,0.10402e-07_rb,0.19081e-07_rb,0.26854e-07_rb,0.34233e-07_rb /)
      kbo(:, 5,55, 1) = (/ &
        & 0.15417e-10_rb,0.11984e-07_rb,0.21835e-07_rb,0.30856e-07_rb,0.38954e-07_rb /)
      kbo(:, 1,56, 1) = (/ &
        & 0.16094e-10_rb,0.50174e-08_rb,0.90283e-08_rb,0.12718e-07_rb,0.16576e-07_rb /)
      kbo(:, 2,56, 1) = (/ &
        & 0.15265e-10_rb,0.59747e-08_rb,0.10850e-07_rb,0.15297e-07_rb,0.19783e-07_rb /)
      kbo(:, 3,56, 1) = (/ &
        & 0.14515e-10_rb,0.70142e-08_rb,0.12699e-07_rb,0.17960e-07_rb,0.23171e-07_rb /)
      kbo(:, 4,56, 1) = (/ &
        & 0.13715e-10_rb,0.81626e-08_rb,0.14940e-07_rb,0.20994e-07_rb,0.26865e-07_rb /)
      kbo(:, 5,56, 1) = (/ &
        & 0.12880e-10_rb,0.94054e-08_rb,0.17206e-07_rb,0.24221e-07_rb,0.30739e-07_rb /)
      kbo(:, 1,57, 1) = (/ &
        & 0.13368e-10_rb,0.38838e-08_rb,0.69991e-08_rb,0.98372e-08_rb,0.12821e-07_rb /)
      kbo(:, 2,57, 1) = (/ &
        & 0.12743e-10_rb,0.46751e-08_rb,0.84243e-08_rb,0.11856e-07_rb,0.15406e-07_rb /)
      kbo(:, 3,57, 1) = (/ &
        & 0.12087e-10_rb,0.54642e-08_rb,0.99295e-08_rb,0.14069e-07_rb,0.18144e-07_rb /)
      kbo(:, 4,57, 1) = (/ &
        & 0.11442e-10_rb,0.64461e-08_rb,0.11657e-07_rb,0.16407e-07_rb,0.21072e-07_rb /)
      kbo(:, 5,57, 1) = (/ &
        & 0.10759e-10_rb,0.73711e-08_rb,0.13478e-07_rb,0.19106e-07_rb,0.24314e-07_rb /)
      kbo(:, 1,58, 1) = (/ &
        & 0.11081e-10_rb,0.30148e-08_rb,0.54165e-08_rb,0.76421e-08_rb,0.99496e-08_rb /)
      kbo(:, 2,58, 1) = (/ &
        & 0.10598e-10_rb,0.36337e-08_rb,0.65375e-08_rb,0.92041e-08_rb,0.12025e-07_rb /)
      kbo(:, 3,58, 1) = (/ &
        & 0.10052e-10_rb,0.42712e-08_rb,0.77841e-08_rb,0.11053e-07_rb,0.14306e-07_rb /)
      kbo(:, 4,58, 1) = (/ &
        & 0.95308e-11_rb,0.50429e-08_rb,0.91133e-08_rb,0.12865e-07_rb,0.16554e-07_rb /)
      kbo(:, 5,58, 1) = (/ &
        & 0.89795e-11_rb,0.57979e-08_rb,0.10615e-07_rb,0.15051e-07_rb,0.19216e-07_rb /)
      kbo(:, 1,59, 1) = (/ &
        & 0.91194e-11_rb,0.24214e-08_rb,0.43496e-08_rb,0.61531e-08_rb,0.80136e-08_rb /)
      kbo(:, 2,59, 1) = (/ &
        & 0.87287e-11_rb,0.29232e-08_rb,0.52535e-08_rb,0.73966e-08_rb,0.96830e-08_rb /)
      kbo(:, 3,59, 1) = (/ &
        & 0.82765e-11_rb,0.34412e-08_rb,0.62788e-08_rb,0.88992e-08_rb,0.11534e-07_rb /)
      kbo(:, 4,59, 1) = (/ &
        & 0.78605e-11_rb,0.40632e-08_rb,0.73454e-08_rb,0.10382e-07_rb,0.13372e-07_rb /)
      kbo(:, 5,59, 1) = (/ &
        & 0.74127e-11_rb,0.46801e-08_rb,0.85910e-08_rb,0.12137e-07_rb,0.15530e-07_rb /)
      kbo(:, 1,13, 2) = (/ &
        & 0.17576e-05_rb,0.22518e-03_rb,0.36398e-03_rb,0.46505e-03_rb,0.97920e-03_rb /)
      kbo(:, 2,13, 2) = (/ &
        & 0.19522e-05_rb,0.25688e-03_rb,0.42724e-03_rb,0.54293e-03_rb,0.11261e-02_rb /)
      kbo(:, 3,13, 2) = (/ &
        & 0.21134e-05_rb,0.29382e-03_rb,0.49778e-03_rb,0.63186e-03_rb,0.12961e-02_rb /)
      kbo(:, 4,13, 2) = (/ &
        & 0.22680e-05_rb,0.33585e-03_rb,0.57095e-03_rb,0.73149e-03_rb,0.14857e-02_rb /)
      kbo(:, 5,13, 2) = (/ &
        & 0.23939e-05_rb,0.38326e-03_rb,0.64688e-03_rb,0.84120e-03_rb,0.16815e-02_rb /)
      kbo(:, 1,14, 2) = (/ &
        & 0.13866e-05_rb,0.18816e-03_rb,0.30541e-03_rb,0.39082e-03_rb,0.80614e-03_rb /)
      kbo(:, 2,14, 2) = (/ &
        & 0.14825e-05_rb,0.21525e-03_rb,0.35906e-03_rb,0.45772e-03_rb,0.92976e-03_rb /)
      kbo(:, 3,14, 2) = (/ &
        & 0.16428e-05_rb,0.24664e-03_rb,0.41891e-03_rb,0.53341e-03_rb,0.10689e-02_rb /)
      kbo(:, 4,14, 2) = (/ &
        & 0.17807e-05_rb,0.28218e-03_rb,0.48021e-03_rb,0.61861e-03_rb,0.12236e-02_rb /)
      kbo(:, 5,14, 2) = (/ &
        & 0.18685e-05_rb,0.32204e-03_rb,0.54385e-03_rb,0.71223e-03_rb,0.13844e-02_rb /)
      kbo(:, 1,15, 2) = (/ &
        & 0.10483e-05_rb,0.15699e-03_rb,0.25571e-03_rb,0.32803e-03_rb,0.65873e-03_rb /)
      kbo(:, 2,15, 2) = (/ &
        & 0.11310e-05_rb,0.18016e-03_rb,0.30071e-03_rb,0.38534e-03_rb,0.75916e-03_rb /)
      kbo(:, 3,15, 2) = (/ &
        & 0.12330e-05_rb,0.20672e-03_rb,0.35181e-03_rb,0.44940e-03_rb,0.87041e-03_rb /)
      kbo(:, 4,15, 2) = (/ &
        & 0.13426e-05_rb,0.23650e-03_rb,0.40292e-03_rb,0.52246e-03_rb,0.99445e-03_rb /)
      kbo(:, 5,15, 2) = (/ &
        & 0.14514e-05_rb,0.26979e-03_rb,0.45684e-03_rb,0.60195e-03_rb,0.11173e-02_rb /)
      kbo(:, 1,16, 2) = (/ &
        & 0.80663e-06_rb,0.13098e-03_rb,0.21382e-03_rb,0.27539e-03_rb,0.53727e-03_rb /)
      kbo(:, 2,16, 2) = (/ &
        & 0.85747e-06_rb,0.15058e-03_rb,0.25201e-03_rb,0.32406e-03_rb,0.61573e-03_rb /)
      kbo(:, 3,16, 2) = (/ &
        & 0.92435e-06_rb,0.17283e-03_rb,0.29542e-03_rb,0.37879e-03_rb,0.70140e-03_rb /)
      kbo(:, 4,16, 2) = (/ &
        & 0.99747e-06_rb,0.19793e-03_rb,0.33807e-03_rb,0.44076e-03_rb,0.79468e-03_rb /)
      kbo(:, 5,16, 2) = (/ &
        & 0.10819e-05_rb,0.22582e-03_rb,0.38354e-03_rb,0.50663e-03_rb,0.88980e-03_rb /)
      kbo(:, 1,17, 2) = (/ &
        & 0.61772e-06_rb,0.10941e-03_rb,0.17926e-03_rb,0.23144e-03_rb,0.43554e-03_rb /)
      kbo(:, 2,17, 2) = (/ &
        & 0.65718e-06_rb,0.12576e-03_rb,0.21174e-03_rb,0.27275e-03_rb,0.49750e-03_rb /)
      kbo(:, 3,17, 2) = (/ &
        & 0.69949e-06_rb,0.14461e-03_rb,0.24852e-03_rb,0.31937e-03_rb,0.56462e-03_rb /)
      kbo(:, 4,17, 2) = (/ &
        & 0.75387e-06_rb,0.16580e-03_rb,0.28374e-03_rb,0.37213e-03_rb,0.63507e-03_rb /)
      kbo(:, 5,17, 2) = (/ &
        & 0.79715e-06_rb,0.18926e-03_rb,0.32236e-03_rb,0.42588e-03_rb,0.70728e-03_rb /)
      kbo(:, 1,18, 2) = (/ &
        & 0.47188e-06_rb,0.91952e-04_rb,0.15100e-03_rb,0.19495e-03_rb,0.35543e-03_rb /)
      kbo(:, 2,18, 2) = (/ &
        & 0.50937e-06_rb,0.10513e-03_rb,0.17852e-03_rb,0.22999e-03_rb,0.40250e-03_rb /)
      kbo(:, 3,18, 2) = (/ &
        & 0.54206e-06_rb,0.12130e-03_rb,0.20959e-03_rb,0.26979e-03_rb,0.45662e-03_rb /)
      kbo(:, 4,18, 2) = (/ &
        & 0.57328e-06_rb,0.13936e-03_rb,0.23815e-03_rb,0.31451e-03_rb,0.51113e-03_rb /)
      kbo(:, 5,18, 2) = (/ &
        & 0.61077e-06_rb,0.15916e-03_rb,0.27075e-03_rb,0.35967e-03_rb,0.57076e-03_rb /)
      kbo(:, 1,19, 2) = (/ &
        & 0.36005e-06_rb,0.76984e-04_rb,0.12743e-03_rb,0.16422e-03_rb,0.28885e-03_rb /)
      kbo(:, 2,19, 2) = (/ &
        & 0.38988e-06_rb,0.88029e-04_rb,0.15067e-03_rb,0.19396e-03_rb,0.32854e-03_rb /)
      kbo(:, 3,19, 2) = (/ &
        & 0.41989e-06_rb,0.10172e-03_rb,0.17539e-03_rb,0.22784e-03_rb,0.37045e-03_rb /)
      kbo(:, 4,19, 2) = (/ &
        & 0.44092e-06_rb,0.11727e-03_rb,0.20004e-03_rb,0.26592e-03_rb,0.41598e-03_rb /)
      kbo(:, 5,19, 2) = (/ &
        & 0.46883e-06_rb,0.13386e-03_rb,0.22769e-03_rb,0.30285e-03_rb,0.46550e-03_rb /)
      kbo(:, 1,20, 2) = (/ &
        & 0.27908e-06_rb,0.64450e-04_rb,0.10795e-03_rb,0.13889e-03_rb,0.23878e-03_rb /)
      kbo(:, 2,20, 2) = (/ &
        & 0.30934e-06_rb,0.74187e-04_rb,0.12773e-03_rb,0.16417e-03_rb,0.27017e-03_rb /)
      kbo(:, 3,20, 2) = (/ &
        & 0.33086e-06_rb,0.85801e-04_rb,0.14782e-03_rb,0.19307e-03_rb,0.30508e-03_rb /)
      kbo(:, 4,20, 2) = (/ &
        & 0.35058e-06_rb,0.98861e-04_rb,0.16868e-03_rb,0.22508e-03_rb,0.34249e-03_rb /)
      kbo(:, 5,20, 2) = (/ &
        & 0.37069e-06_rb,0.11306e-03_rb,0.19188e-03_rb,0.25601e-03_rb,0.38519e-03_rb /)
      kbo(:, 1,21, 2) = (/ &
        & 0.22418e-06_rb,0.54095e-04_rb,0.91414e-04_rb,0.11757e-03_rb,0.19697e-03_rb /)
      kbo(:, 2,21, 2) = (/ &
        & 0.24559e-06_rb,0.62567e-04_rb,0.10819e-03_rb,0.13908e-03_rb,0.22284e-03_rb /)
      kbo(:, 3,21, 2) = (/ &
        & 0.26169e-06_rb,0.72383e-04_rb,0.12481e-03_rb,0.16368e-03_rb,0.25205e-03_rb /)
      kbo(:, 4,21, 2) = (/ &
        & 0.27885e-06_rb,0.83446e-04_rb,0.14258e-03_rb,0.18910e-03_rb,0.28386e-03_rb /)
      kbo(:, 5,21, 2) = (/ &
        & 0.29344e-06_rb,0.95540e-04_rb,0.16198e-03_rb,0.21609e-03_rb,0.31947e-03_rb /)
      kbo(:, 1,22, 2) = (/ &
        & 0.17941e-06_rb,0.45830e-04_rb,0.78176e-04_rb,0.10061e-03_rb,0.16313e-03_rb /)
      kbo(:, 2,22, 2) = (/ &
        & 0.19397e-06_rb,0.53139e-04_rb,0.92556e-04_rb,0.11901e-03_rb,0.18482e-03_rb /)
      kbo(:, 3,22, 2) = (/ &
        & 0.20707e-06_rb,0.61593e-04_rb,0.10579e-03_rb,0.14008e-03_rb,0.20965e-03_rb /)
      kbo(:, 4,22, 2) = (/ &
        & 0.22071e-06_rb,0.71016e-04_rb,0.12100e-03_rb,0.16128e-03_rb,0.23661e-03_rb /)
      kbo(:, 5,22, 2) = (/ &
        & 0.23148e-06_rb,0.81286e-04_rb,0.13795e-03_rb,0.18401e-03_rb,0.26580e-03_rb /)
      kbo(:, 1,23, 2) = (/ &
        & 0.14291e-06_rb,0.38825e-04_rb,0.66866e-04_rb,0.86140e-04_rb,0.13511e-03_rb /)
      kbo(:, 2,23, 2) = (/ &
        & 0.15179e-06_rb,0.45135e-04_rb,0.78450e-04_rb,0.10194e-03_rb,0.15369e-03_rb /)
      kbo(:, 3,23, 2) = (/ &
        & 0.16326e-06_rb,0.52399e-04_rb,0.89919e-04_rb,0.12001e-03_rb,0.17441e-03_rb /)
      kbo(:, 4,23, 2) = (/ &
        & 0.17307e-06_rb,0.60440e-04_rb,0.10293e-03_rb,0.13735e-03_rb,0.19668e-03_rb /)
      kbo(:, 5,23, 2) = (/ &
        & 0.18137e-06_rb,0.69189e-04_rb,0.11759e-03_rb,0.15651e-03_rb,0.21998e-03_rb /)
      kbo(:, 1,24, 2) = (/ &
        & 0.11245e-06_rb,0.32862e-04_rb,0.57166e-04_rb,0.73828e-04_rb,0.11185e-03_rb /)
      kbo(:, 2,24, 2) = (/ &
        & 0.11958e-06_rb,0.38281e-04_rb,0.66635e-04_rb,0.87390e-04_rb,0.12740e-03_rb /)
      kbo(:, 3,24, 2) = (/ &
        & 0.12702e-06_rb,0.44519e-04_rb,0.76606e-04_rb,0.10190e-03_rb,0.14418e-03_rb /)
      kbo(:, 4,24, 2) = (/ &
        & 0.13467e-06_rb,0.51373e-04_rb,0.87622e-04_rb,0.11700e-03_rb,0.16205e-03_rb /)
      kbo(:, 5,24, 2) = (/ &
        & 0.14120e-06_rb,0.58874e-04_rb,0.10019e-03_rb,0.13348e-03_rb,0.18188e-03_rb /)
      kbo(:, 1,25, 2) = (/ &
        & 0.88242e-07_rb,0.27950e-04_rb,0.49010e-04_rb,0.63434e-04_rb,0.92877e-04_rb /)
      kbo(:, 2,25, 2) = (/ &
        & 0.94383e-07_rb,0.32610e-04_rb,0.56690e-04_rb,0.75114e-04_rb,0.10531e-03_rb /)
      kbo(:, 3,25, 2) = (/ &
        & 0.99712e-07_rb,0.37929e-04_rb,0.65204e-04_rb,0.87113e-04_rb,0.11931e-03_rb /)
      kbo(:, 4,25, 2) = (/ &
        & 0.10473e-06_rb,0.43778e-04_rb,0.74805e-04_rb,0.99950e-04_rb,0.13427e-03_rb /)
      kbo(:, 5,25, 2) = (/ &
        & 0.10988e-06_rb,0.50229e-04_rb,0.85598e-04_rb,0.11406e-03_rb,0.15050e-03_rb /)
      kbo(:, 1,26, 2) = (/ &
        & 0.69696e-07_rb,0.23895e-04_rb,0.42166e-04_rb,0.54733e-04_rb,0.77146e-04_rb /)
      kbo(:, 2,26, 2) = (/ &
        & 0.74390e-07_rb,0.27921e-04_rb,0.48387e-04_rb,0.64817e-04_rb,0.87607e-04_rb /)
      kbo(:, 3,26, 2) = (/ &
        & 0.78856e-07_rb,0.32497e-04_rb,0.55692e-04_rb,0.74691e-04_rb,0.99387e-04_rb /)
      kbo(:, 4,26, 2) = (/ &
        & 0.83253e-07_rb,0.37491e-04_rb,0.64130e-04_rb,0.85609e-04_rb,0.11193e-03_rb /)
      kbo(:, 5,26, 2) = (/ &
        & 0.87675e-07_rb,0.43011e-04_rb,0.73398e-04_rb,0.97828e-04_rb,0.12574e-03_rb /)
      kbo(:, 1,27, 2) = (/ &
        & 0.54619e-07_rb,0.20469e-04_rb,0.35869e-04_rb,0.47259e-04_rb,0.64151e-04_rb /)
      kbo(:, 2,27, 2) = (/ &
        & 0.58657e-07_rb,0.23925e-04_rb,0.41468e-04_rb,0.55499e-04_rb,0.72911e-04_rb /)
      kbo(:, 3,27, 2) = (/ &
        & 0.62532e-07_rb,0.27839e-04_rb,0.47735e-04_rb,0.63985e-04_rb,0.82955e-04_rb /)
      kbo(:, 4,27, 2) = (/ &
        & 0.66184e-07_rb,0.32141e-04_rb,0.55001e-04_rb,0.73456e-04_rb,0.93567e-04_rb /)
      kbo(:, 5,27, 2) = (/ &
        & 0.69305e-07_rb,0.36605e-04_rb,0.62968e-04_rb,0.83938e-04_rb,0.10517e-03_rb /)
      kbo(:, 1,28, 2) = (/ &
        & 0.43348e-07_rb,0.17476e-04_rb,0.30692e-04_rb,0.40803e-04_rb,0.53271e-04_rb /)
      kbo(:, 2,28, 2) = (/ &
        & 0.46348e-07_rb,0.20496e-04_rb,0.35498e-04_rb,0.47629e-04_rb,0.60809e-04_rb /)
      kbo(:, 3,28, 2) = (/ &
        & 0.49507e-07_rb,0.23857e-04_rb,0.40974e-04_rb,0.54947e-04_rb,0.69222e-04_rb /)
      kbo(:, 4,28, 2) = (/ &
        & 0.52419e-07_rb,0.27560e-04_rb,0.47205e-04_rb,0.63044e-04_rb,0.78123e-04_rb /)
      kbo(:, 5,28, 2) = (/ &
        & 0.54842e-07_rb,0.31094e-04_rb,0.54028e-04_rb,0.72116e-04_rb,0.87834e-04_rb /)
      kbo(:, 1,29, 2) = (/ &
        & 0.34480e-07_rb,0.15027e-04_rb,0.26268e-04_rb,0.35358e-04_rb,0.44711e-04_rb /)
      kbo(:, 2,29, 2) = (/ &
        & 0.36985e-07_rb,0.17638e-04_rb,0.30405e-04_rb,0.40938e-04_rb,0.51054e-04_rb /)
      kbo(:, 3,29, 2) = (/ &
        & 0.39476e-07_rb,0.20512e-04_rb,0.35237e-04_rb,0.47207e-04_rb,0.58034e-04_rb /)
      kbo(:, 4,29, 2) = (/ &
        & 0.41650e-07_rb,0.23694e-04_rb,0.40586e-04_rb,0.54286e-04_rb,0.65708e-04_rb /)
      kbo(:, 5,29, 2) = (/ &
        & 0.43693e-07_rb,0.26438e-04_rb,0.46467e-04_rb,0.62035e-04_rb,0.73872e-04_rb /)
      kbo(:, 1,30, 2) = (/ &
        & 0.27402e-07_rb,0.12932e-04_rb,0.22585e-04_rb,0.30365e-04_rb,0.37471e-04_rb /)
      kbo(:, 2,30, 2) = (/ &
        & 0.29377e-07_rb,0.15183e-04_rb,0.26158e-04_rb,0.35195e-04_rb,0.42867e-04_rb /)
      kbo(:, 3,30, 2) = (/ &
        & 0.31353e-07_rb,0.17658e-04_rb,0.30343e-04_rb,0.40606e-04_rb,0.48858e-04_rb /)
      kbo(:, 4,30, 2) = (/ &
        & 0.33056e-07_rb,0.20286e-04_rb,0.34924e-04_rb,0.46735e-04_rb,0.55329e-04_rb /)
      kbo(:, 5,30, 2) = (/ &
        & 0.34673e-07_rb,0.22451e-04_rb,0.39717e-04_rb,0.53329e-04_rb,0.62151e-04_rb /)
      kbo(:, 1,31, 2) = (/ &
        & 0.21716e-07_rb,0.11156e-04_rb,0.19427e-04_rb,0.26169e-04_rb,0.31512e-04_rb /)
      kbo(:, 2,31, 2) = (/ &
        & 0.23313e-07_rb,0.13091e-04_rb,0.22568e-04_rb,0.30377e-04_rb,0.36216e-04_rb /)
      kbo(:, 3,31, 2) = (/ &
        & 0.24803e-07_rb,0.15221e-04_rb,0.26157e-04_rb,0.35050e-04_rb,0.41256e-04_rb /)
      kbo(:, 4,31, 2) = (/ &
        & 0.26260e-07_rb,0.17277e-04_rb,0.30091e-04_rb,0.40288e-04_rb,0.46693e-04_rb /)
      kbo(:, 5,31, 2) = (/ &
        & 0.27334e-07_rb,0.19159e-04_rb,0.33773e-04_rb,0.45892e-04_rb,0.52314e-04_rb /)
      kbo(:, 1,32, 2) = (/ &
        & 0.17123e-07_rb,0.96453e-05_rb,0.16688e-04_rb,0.22528e-04_rb,0.26648e-04_rb /)
      kbo(:, 2,32, 2) = (/ &
        & 0.18489e-07_rb,0.11305e-04_rb,0.19464e-04_rb,0.26137e-04_rb,0.30617e-04_rb /)
      kbo(:, 3,32, 2) = (/ &
        & 0.19621e-07_rb,0.13143e-04_rb,0.22554e-04_rb,0.30238e-04_rb,0.34876e-04_rb /)
      kbo(:, 4,32, 2) = (/ &
        & 0.20653e-07_rb,0.14698e-04_rb,0.25928e-04_rb,0.34712e-04_rb,0.39398e-04_rb /)
      kbo(:, 5,32, 2) = (/ &
        & 0.21500e-07_rb,0.16315e-04_rb,0.28651e-04_rb,0.39302e-04_rb,0.44097e-04_rb /)
      kbo(:, 1,33, 2) = (/ &
        & 0.13414e-07_rb,0.83355e-05_rb,0.14409e-04_rb,0.19461e-04_rb,0.22582e-04_rb /)
      kbo(:, 2,33, 2) = (/ &
        & 0.14456e-07_rb,0.97647e-05_rb,0.16818e-04_rb,0.22578e-04_rb,0.25923e-04_rb /)
      kbo(:, 3,33, 2) = (/ &
        & 0.15416e-07_rb,0.11250e-04_rb,0.19451e-04_rb,0.26119e-04_rb,0.29482e-04_rb /)
      kbo(:, 4,33, 2) = (/ &
        & 0.16184e-07_rb,0.12520e-04_rb,0.22064e-04_rb,0.29911e-04_rb,0.33230e-04_rb /)
      kbo(:, 5,33, 2) = (/ &
        & 0.16861e-07_rb,0.13810e-04_rb,0.24332e-04_rb,0.33363e-04_rb,0.37135e-04_rb /)
      kbo(:, 1,34, 2) = (/ &
        & 0.10575e-07_rb,0.71786e-05_rb,0.12407e-04_rb,0.16719e-04_rb,0.19158e-04_rb /)
      kbo(:, 2,34, 2) = (/ &
        & 0.11328e-07_rb,0.84093e-05_rb,0.14457e-04_rb,0.19433e-04_rb,0.21940e-04_rb /)
      kbo(:, 3,34, 2) = (/ &
        & 0.12049e-07_rb,0.95860e-05_rb,0.16718e-04_rb,0.22425e-04_rb,0.24932e-04_rb /)
      kbo(:, 4,34, 2) = (/ &
        & 0.12689e-07_rb,0.10655e-04_rb,0.18771e-04_rb,0.25647e-04_rb,0.28065e-04_rb /)
      kbo(:, 5,34, 2) = (/ &
        & 0.13241e-07_rb,0.11703e-04_rb,0.20628e-04_rb,0.28184e-04_rb,0.31323e-04_rb /)
      kbo(:, 1,35, 2) = (/ &
        & 0.82489e-08_rb,0.60850e-05_rb,0.10505e-04_rb,0.14140e-04_rb,0.16004e-04_rb /)
      kbo(:, 2,35, 2) = (/ &
        & 0.88532e-08_rb,0.71306e-05_rb,0.12250e-04_rb,0.16463e-04_rb,0.18333e-04_rb /)
      kbo(:, 3,35, 2) = (/ &
        & 0.93468e-08_rb,0.80269e-05_rb,0.14158e-04_rb,0.18989e-04_rb,0.20835e-04_rb /)
      kbo(:, 4,35, 2) = (/ &
        & 0.98478e-08_rb,0.89450e-05_rb,0.15691e-04_rb,0.21576e-04_rb,0.23428e-04_rb /)
      kbo(:, 5,35, 2) = (/ &
        & 0.10332e-07_rb,0.98354e-05_rb,0.17293e-04_rb,0.23618e-04_rb,0.26128e-04_rb /)
      kbo(:, 1,36, 2) = (/ &
        & 0.63951e-08_rb,0.50545e-05_rb,0.87378e-05_rb,0.11778e-04_rb,0.13176e-04_rb /)
      kbo(:, 2,36, 2) = (/ &
        & 0.68252e-08_rb,0.59348e-05_rb,0.10193e-04_rb,0.13730e-04_rb,0.15104e-04_rb /)
      kbo(:, 3,36, 2) = (/ &
        & 0.72445e-08_rb,0.66480e-05_rb,0.11783e-04_rb,0.15832e-04_rb,0.17160e-04_rb /)
      kbo(:, 4,36, 2) = (/ &
        & 0.75871e-08_rb,0.73960e-05_rb,0.13005e-04_rb,0.17866e-04_rb,0.19295e-04_rb /)
      kbo(:, 5,36, 2) = (/ &
        & 0.79075e-08_rb,0.81558e-05_rb,0.14306e-04_rb,0.19617e-04_rb,0.21532e-04_rb /)
      kbo(:, 1,37, 2) = (/ &
        & 0.50574e-08_rb,0.41215e-05_rb,0.71249e-05_rb,0.95923e-05_rb,0.10686e-04_rb /)
      kbo(:, 2,37, 2) = (/ &
        & 0.53967e-08_rb,0.48518e-05_rb,0.83286e-05_rb,0.11206e-04_rb,0.12269e-04_rb /)
      kbo(:, 3,37, 2) = (/ &
        & 0.57517e-08_rb,0.54411e-05_rb,0.96383e-05_rb,0.12942e-04_rb,0.13958e-04_rb /)
      kbo(:, 4,37, 2) = (/ &
        & 0.60273e-08_rb,0.60590e-05_rb,0.10644e-04_rb,0.14619e-04_rb,0.15714e-04_rb /)
      kbo(:, 5,37, 2) = (/ &
        & 0.62775e-08_rb,0.66900e-05_rb,0.11723e-04_rb,0.16077e-04_rb,0.17560e-04_rb /)
      kbo(:, 1,38, 2) = (/ &
        & 0.39948e-08_rb,0.33535e-05_rb,0.57976e-05_rb,0.77989e-05_rb,0.86548e-05_rb /)
      kbo(:, 2,38, 2) = (/ &
        & 0.42642e-08_rb,0.39584e-05_rb,0.67932e-05_rb,0.91290e-05_rb,0.99507e-05_rb /)
      kbo(:, 3,38, 2) = (/ &
        & 0.45529e-08_rb,0.44478e-05_rb,0.78746e-05_rb,0.10563e-04_rb,0.11338e-04_rb /)
      kbo(:, 4,38, 2) = (/ &
        & 0.47911e-08_rb,0.49592e-05_rb,0.87019e-05_rb,0.11950e-04_rb,0.12783e-04_rb /)
      kbo(:, 5,38, 2) = (/ &
        & 0.49959e-08_rb,0.54820e-05_rb,0.95964e-05_rb,0.13155e-04_rb,0.14308e-04_rb /)
      kbo(:, 1,39, 2) = (/ &
        & 0.31607e-08_rb,0.27260e-05_rb,0.47143e-05_rb,0.63381e-05_rb,0.70094e-05_rb /)
      kbo(:, 2,39, 2) = (/ &
        & 0.33771e-08_rb,0.32263e-05_rb,0.55352e-05_rb,0.74322e-05_rb,0.80695e-05_rb /)
      kbo(:, 3,39, 2) = (/ &
        & 0.35907e-08_rb,0.36316e-05_rb,0.64256e-05_rb,0.86133e-05_rb,0.92019e-05_rb /)
      kbo(:, 4,39, 2) = (/ &
        & 0.37920e-08_rb,0.40555e-05_rb,0.71128e-05_rb,0.97677e-05_rb,0.10392e-04_rb /)
      kbo(:, 5,39, 2) = (/ &
        & 0.39687e-08_rb,0.44882e-05_rb,0.78499e-05_rb,0.10753e-04_rb,0.11647e-04_rb /)
      kbo(:, 1,40, 2) = (/ &
        & 0.25137e-08_rb,0.21782e-05_rb,0.37700e-05_rb,0.50636e-05_rb,0.55872e-05_rb /)
      kbo(:, 2,40, 2) = (/ &
        & 0.26953e-08_rb,0.25872e-05_rb,0.44411e-05_rb,0.59538e-05_rb,0.64693e-05_rb /)
      kbo(:, 3,40, 2) = (/ &
        & 0.28738e-08_rb,0.29415e-05_rb,0.51697e-05_rb,0.69217e-05_rb,0.73806e-05_rb /)
      kbo(:, 4,40, 2) = (/ &
        & 0.30365e-08_rb,0.32887e-05_rb,0.57574e-05_rb,0.79115e-05_rb,0.83547e-05_rb /)
      kbo(:, 5,40, 2) = (/ &
        & 0.31827e-08_rb,0.36391e-05_rb,0.63730e-05_rb,0.87114e-05_rb,0.93854e-05_rb /)
      kbo(:, 1,41, 2) = (/ &
        & 0.20049e-08_rb,0.17350e-05_rb,0.30089e-05_rb,0.40342e-05_rb,0.44437e-05_rb /)
      kbo(:, 2,41, 2) = (/ &
        & 0.21522e-08_rb,0.20685e-05_rb,0.35532e-05_rb,0.47542e-05_rb,0.51577e-05_rb /)
      kbo(:, 3,41, 2) = (/ &
        & 0.22978e-08_rb,0.23716e-05_rb,0.41490e-05_rb,0.55459e-05_rb,0.59029e-05_rb /)
      kbo(:, 4,41, 2) = (/ &
        & 0.24277e-08_rb,0.26608e-05_rb,0.46585e-05_rb,0.63928e-05_rb,0.67015e-05_rb /)
      kbo(:, 5,41, 2) = (/ &
        & 0.25592e-08_rb,0.29434e-05_rb,0.51614e-05_rb,0.70376e-05_rb,0.75472e-05_rb /)
      kbo(:, 1,42, 2) = (/ &
        & 0.16007e-08_rb,0.13793e-05_rb,0.23939e-05_rb,0.32203e-05_rb,0.35347e-05_rb /)
      kbo(:, 2,42, 2) = (/ &
        & 0.17182e-08_rb,0.16509e-05_rb,0.28388e-05_rb,0.37950e-05_rb,0.41069e-05_rb /)
      kbo(:, 3,42, 2) = (/ &
        & 0.18364e-08_rb,0.19058e-05_rb,0.33250e-05_rb,0.44386e-05_rb,0.47169e-05_rb /)
      kbo(:, 4,42, 2) = (/ &
        & 0.19459e-08_rb,0.21436e-05_rb,0.37676e-05_rb,0.51389e-05_rb,0.53710e-05_rb /)
      kbo(:, 5,42, 2) = (/ &
        & 0.20555e-08_rb,0.23781e-05_rb,0.41722e-05_rb,0.56915e-05_rb,0.60613e-05_rb /)
      kbo(:, 1,43, 2) = (/ &
        & 0.12763e-08_rb,0.10855e-05_rb,0.18842e-05_rb,0.25355e-05_rb,0.27894e-05_rb /)
      kbo(:, 2,43, 2) = (/ &
        & 0.13693e-08_rb,0.13038e-05_rb,0.22434e-05_rb,0.29991e-05_rb,0.32452e-05_rb /)
      kbo(:, 3,43, 2) = (/ &
        & 0.14669e-08_rb,0.15227e-05_rb,0.26394e-05_rb,0.35195e-05_rb,0.37403e-05_rb /)
      kbo(:, 4,43, 2) = (/ &
        & 0.15621e-08_rb,0.17145e-05_rb,0.30188e-05_rb,0.40881e-05_rb,0.42757e-05_rb /)
      kbo(:, 5,43, 2) = (/ &
        & 0.16467e-08_rb,0.19093e-05_rb,0.33481e-05_rb,0.45768e-05_rb,0.48357e-05_rb /)
      kbo(:, 1,44, 2) = (/ &
        & 0.10164e-08_rb,0.84908e-06_rb,0.14741e-05_rb,0.19864e-05_rb,0.21744e-05_rb /)
      kbo(:, 2,44, 2) = (/ &
        & 0.10926e-08_rb,0.10232e-05_rb,0.17642e-05_rb,0.23536e-05_rb,0.25562e-05_rb /)
      kbo(:, 3,44, 2) = (/ &
        & 0.11717e-08_rb,0.12156e-05_rb,0.20836e-05_rb,0.27750e-05_rb,0.29547e-05_rb /)
      kbo(:, 4,44, 2) = (/ &
        & 0.12502e-08_rb,0.13671e-05_rb,0.24149e-05_rb,0.32374e-05_rb,0.33856e-05_rb /)
      kbo(:, 5,44, 2) = (/ &
        & 0.13204e-08_rb,0.15290e-05_rb,0.26801e-05_rb,0.36631e-05_rb,0.38430e-05_rb /)
      kbo(:, 1,45, 2) = (/ &
        & 0.80967e-09_rb,0.66247e-06_rb,0.11525e-05_rb,0.15557e-05_rb,0.16911e-05_rb /)
      kbo(:, 2,45, 2) = (/ &
        & 0.87091e-09_rb,0.80114e-06_rb,0.13838e-05_rb,0.18457e-05_rb,0.20071e-05_rb /)
      kbo(:, 3,45, 2) = (/ &
        & 0.93471e-09_rb,0.96063e-06_rb,0.16414e-05_rb,0.21862e-05_rb,0.23357e-05_rb /)
      kbo(:, 4,45, 2) = (/ &
        & 0.99996e-09_rb,0.10898e-05_rb,0.19287e-05_rb,0.25587e-05_rb,0.26765e-05_rb /)
      kbo(:, 5,45, 2) = (/ &
        & 0.10594e-08_rb,0.12255e-05_rb,0.21437e-05_rb,0.29239e-05_rb,0.30491e-05_rb /)
      kbo(:, 1,46, 2) = (/ &
        & 0.64438e-09_rb,0.51408e-06_rb,0.89692e-06_rb,0.12176e-05_rb,0.13094e-05_rb /)
      kbo(:, 2,46, 2) = (/ &
        & 0.69338e-09_rb,0.62335e-06_rb,0.10783e-05_rb,0.14408e-05_rb,0.15697e-05_rb /)
      kbo(:, 3,46, 2) = (/ &
        & 0.74249e-09_rb,0.75197e-06_rb,0.12859e-05_rb,0.17116e-05_rb,0.18286e-05_rb /)
      kbo(:, 4,46, 2) = (/ &
        & 0.79710e-09_rb,0.86785e-06_rb,0.15182e-05_rb,0.20117e-05_rb,0.21072e-05_rb /)
      kbo(:, 5,46, 2) = (/ &
        & 0.84966e-09_rb,0.97923e-06_rb,0.17156e-05_rb,0.23354e-05_rb,0.24096e-05_rb /)
      kbo(:, 1,47, 2) = (/ &
        & 0.51236e-09_rb,0.39483e-06_rb,0.69100e-06_rb,0.94871e-06_rb,0.10050e-05_rb /)
      kbo(:, 2,47, 2) = (/ &
        & 0.55075e-09_rb,0.48075e-06_rb,0.83326e-06_rb,0.11171e-05_rb,0.12169e-05_rb /)
      kbo(:, 3,47, 2) = (/ &
        & 0.59153e-09_rb,0.58279e-06_rb,0.99922e-06_rb,0.13307e-05_rb,0.14227e-05_rb /)
      kbo(:, 4,47, 2) = (/ &
        & 0.63485e-09_rb,0.68347e-06_rb,0.11845e-05_rb,0.15696e-05_rb,0.16474e-05_rb /)
      kbo(:, 5,47, 2) = (/ &
        & 0.67833e-09_rb,0.77359e-06_rb,0.13593e-05_rb,0.18329e-05_rb,0.18933e-05_rb /)
      kbo(:, 1,48, 2) = (/ &
        & 0.40749e-09_rb,0.30162e-06_rb,0.53408e-06_rb,0.72777e-06_rb,0.76768e-06_rb /)
      kbo(:, 2,48, 2) = (/ &
        & 0.43682e-09_rb,0.36975e-06_rb,0.64201e-06_rb,0.86174e-06_rb,0.93334e-06_rb /)
      kbo(:, 3,48, 2) = (/ &
        & 0.46981e-09_rb,0.45015e-06_rb,0.77460e-06_rb,0.10306e-05_rb,0.11092e-05_rb /)
      kbo(:, 4,48, 2) = (/ &
        & 0.50302e-09_rb,0.53912e-06_rb,0.92171e-06_rb,0.12225e-05_rb,0.12874e-05_rb /)
      kbo(:, 5,48, 2) = (/ &
        & 0.53999e-09_rb,0.61013e-06_rb,0.10790e-05_rb,0.14334e-05_rb,0.14828e-05_rb /)
      kbo(:, 1,49, 2) = (/ &
        & 0.32288e-09_rb,0.23115e-06_rb,0.41625e-06_rb,0.55318e-06_rb,0.58476e-06_rb /)
      kbo(:, 2,49, 2) = (/ &
        & 0.34731e-09_rb,0.28355e-06_rb,0.49493e-06_rb,0.66850e-06_rb,0.71600e-06_rb /)
      kbo(:, 3,49, 2) = (/ &
        & 0.37310e-09_rb,0.34682e-06_rb,0.59820e-06_rb,0.79617e-06_rb,0.85985e-06_rb /)
      kbo(:, 4,49, 2) = (/ &
        & 0.39975e-09_rb,0.42034e-06_rb,0.71527e-06_rb,0.95046e-06_rb,0.10041e-05_rb /)
      kbo(:, 5,49, 2) = (/ &
        & 0.42970e-09_rb,0.48341e-06_rb,0.84829e-06_rb,0.11191e-05_rb,0.11582e-05_rb /)
      kbo(:, 1,50, 2) = (/ &
        & 0.25646e-09_rb,0.17743e-06_rb,0.32423e-06_rb,0.42175e-06_rb,0.44754e-06_rb /)
      kbo(:, 2,50, 2) = (/ &
        & 0.27669e-09_rb,0.21856e-06_rb,0.38220e-06_rb,0.52102e-06_rb,0.55141e-06_rb /)
      kbo(:, 3,50, 2) = (/ &
        & 0.29654e-09_rb,0.26808e-06_rb,0.46375e-06_rb,0.61921e-06_rb,0.67025e-06_rb /)
      kbo(:, 4,50, 2) = (/ &
        & 0.31872e-09_rb,0.32662e-06_rb,0.55687e-06_rb,0.73985e-06_rb,0.78291e-06_rb /)
      kbo(:, 5,50, 2) = (/ &
        & 0.34198e-09_rb,0.38136e-06_rb,0.66385e-06_rb,0.87591e-06_rb,0.90771e-06_rb /)
      kbo(:, 1,51, 2) = (/ &
        & 0.20472e-09_rb,0.13694e-06_rb,0.24726e-06_rb,0.32216e-06_rb,0.34229e-06_rb /)
      kbo(:, 2,51, 2) = (/ &
        & 0.22054e-09_rb,0.16842e-06_rb,0.29646e-06_rb,0.40635e-06_rb,0.42402e-06_rb /)
      kbo(:, 3,51, 2) = (/ &
        & 0.23653e-09_rb,0.20731e-06_rb,0.35913e-06_rb,0.48027e-06_rb,0.51714e-06_rb /)
      kbo(:, 4,51, 2) = (/ &
        & 0.25402e-09_rb,0.25379e-06_rb,0.43377e-06_rb,0.57689e-06_rb,0.61228e-06_rb /)
      kbo(:, 5,51, 2) = (/ &
        & 0.27233e-09_rb,0.30202e-06_rb,0.51940e-06_rb,0.68626e-06_rb,0.71146e-06_rb /)
      kbo(:, 1,52, 2) = (/ &
        & 0.16428e-09_rb,0.10687e-06_rb,0.18803e-06_rb,0.24517e-06_rb,0.26090e-06_rb /)
      kbo(:, 2,52, 2) = (/ &
        & 0.17520e-09_rb,0.12978e-06_rb,0.23025e-06_rb,0.31106e-06_rb,0.32512e-06_rb /)
      kbo(:, 3,52, 2) = (/ &
        & 0.18865e-09_rb,0.15994e-06_rb,0.27726e-06_rb,0.37261e-06_rb,0.39888e-06_rb /)
      kbo(:, 4,52, 2) = (/ &
        & 0.20265e-09_rb,0.19670e-06_rb,0.33708e-06_rb,0.44845e-06_rb,0.47779e-06_rb /)
      kbo(:, 5,52, 2) = (/ &
        & 0.21711e-09_rb,0.23739e-06_rb,0.40528e-06_rb,0.53668e-06_rb,0.55828e-06_rb /)
      kbo(:, 1,53, 2) = (/ &
        & 0.13206e-09_rb,0.82677e-07_rb,0.14284e-06_rb,0.18640e-06_rb,0.19898e-06_rb /)
      kbo(:, 2,53, 2) = (/ &
        & 0.13938e-09_rb,0.99597e-07_rb,0.18095e-06_rb,0.23725e-06_rb,0.24911e-06_rb /)
      kbo(:, 3,53, 2) = (/ &
        & 0.15056e-09_rb,0.12309e-06_rb,0.21494e-06_rb,0.29026e-06_rb,0.30720e-06_rb /)
      kbo(:, 4,53, 2) = (/ &
        & 0.16136e-09_rb,0.15205e-06_rb,0.26147e-06_rb,0.34774e-06_rb,0.37331e-06_rb /)
      kbo(:, 5,53, 2) = (/ &
        & 0.17318e-09_rb,0.18547e-06_rb,0.31566e-06_rb,0.41857e-06_rb,0.43649e-06_rb /)
      kbo(:, 1,54, 2) = (/ &
        & 0.10628e-09_rb,0.63529e-07_rb,0.10938e-06_rb,0.14285e-06_rb,0.15281e-06_rb /)
      kbo(:, 2,54, 2) = (/ &
        & 0.11181e-09_rb,0.77068e-07_rb,0.14036e-06_rb,0.18201e-06_rb,0.19163e-06_rb /)
      kbo(:, 3,54, 2) = (/ &
        & 0.11992e-09_rb,0.95387e-07_rb,0.16692e-06_rb,0.22878e-06_rb,0.23824e-06_rb /)
      kbo(:, 4,54, 2) = (/ &
        & 0.12900e-09_rb,0.11809e-06_rb,0.20406e-06_rb,0.27163e-06_rb,0.29078e-06_rb /)
      kbo(:, 5,54, 2) = (/ &
        & 0.13828e-09_rb,0.14510e-06_rb,0.24678e-06_rb,0.32753e-06_rb,0.34217e-06_rb /)
      kbo(:, 1,55, 2) = (/ &
        & 0.85664e-10_rb,0.48444e-07_rb,0.83932e-07_rb,0.10950e-06_rb,0.11785e-06_rb /)
      kbo(:, 2,55, 2) = (/ &
        & 0.89999e-10_rb,0.59885e-07_rb,0.10776e-06_rb,0.14023e-06_rb,0.14763e-06_rb /)
      kbo(:, 3,55, 2) = (/ &
        & 0.95836e-10_rb,0.73941e-07_rb,0.13023e-06_rb,0.17876e-06_rb,0.18426e-06_rb /)
      kbo(:, 4,55, 2) = (/ &
        & 0.10323e-09_rb,0.91791e-07_rb,0.15882e-06_rb,0.21202e-06_rb,0.22611e-06_rb /)
      kbo(:, 5,55, 2) = (/ &
        & 0.11081e-09_rb,0.11336e-06_rb,0.19316e-06_rb,0.25647e-06_rb,0.26918e-06_rb /)
      kbo(:, 1,56, 2) = (/ &
        & 0.69093e-10_rb,0.36832e-07_rb,0.64123e-07_rb,0.84030e-07_rb,0.90974e-07_rb /)
      kbo(:, 2,56, 2) = (/ &
        & 0.72433e-10_rb,0.47084e-07_rb,0.82613e-07_rb,0.10758e-06_rb,0.11346e-06_rb /)
      kbo(:, 3,56, 2) = (/ &
        & 0.76616e-10_rb,0.57388e-07_rb,0.10182e-06_rb,0.13768e-06_rb,0.14224e-06_rb /)
      kbo(:, 4,56, 2) = (/ &
        & 0.82709e-10_rb,0.71294e-07_rb,0.12355e-06_rb,0.16530e-06_rb,0.17549e-06_rb /)
      kbo(:, 5,56, 2) = (/ &
        & 0.88636e-10_rb,0.88445e-07_rb,0.15094e-06_rb,0.20067e-06_rb,0.21118e-06_rb /)
      kbo(:, 1,57, 2) = (/ &
        & 0.55980e-10_rb,0.28018e-07_rb,0.48867e-07_rb,0.64857e-07_rb,0.70055e-07_rb /)
      kbo(:, 2,57, 2) = (/ &
        & 0.58397e-10_rb,0.36620e-07_rb,0.63232e-07_rb,0.82428e-07_rb,0.87457e-07_rb /)
      kbo(:, 3,57, 2) = (/ &
        & 0.61509e-10_rb,0.44446e-07_rb,0.80512e-07_rb,0.10580e-06_rb,0.10988e-06_rb /)
      kbo(:, 4,57, 2) = (/ &
        & 0.66015e-10_rb,0.55222e-07_rb,0.96401e-07_rb,0.12935e-06_rb,0.13614e-06_rb /)
      kbo(:, 5,57, 2) = (/ &
        & 0.70976e-10_rb,0.68821e-07_rb,0.11780e-06_rb,0.15632e-06_rb,0.16586e-06_rb /)
      kbo(:, 1,58, 2) = (/ &
        & 0.45722e-10_rb,0.21506e-07_rb,0.37326e-07_rb,0.50333e-07_rb,0.54855e-07_rb /)
      kbo(:, 2,58, 2) = (/ &
        & 0.47090e-10_rb,0.28249e-07_rb,0.48612e-07_rb,0.63442e-07_rb,0.67550e-07_rb /)
      kbo(:, 3,58, 2) = (/ &
        & 0.49550e-10_rb,0.34430e-07_rb,0.62769e-07_rb,0.81509e-07_rb,0.84917e-07_rb /)
      kbo(:, 4,58, 2) = (/ &
        & 0.52817e-10_rb,0.42958e-07_rb,0.75173e-07_rb,0.10204e-06_rb,0.10603e-06_rb /)
      kbo(:, 5,58, 2) = (/ &
        & 0.56864e-10_rb,0.53637e-07_rb,0.92044e-07_rb,0.12229e-06_rb,0.13011e-06_rb /)
      kbo(:, 1,59, 2) = (/ &
        & 0.37496e-10_rb,0.17255e-07_rb,0.29967e-07_rb,0.40394e-07_rb,0.44059e-07_rb /)
      kbo(:, 2,59, 2) = (/ &
        & 0.38367e-10_rb,0.22782e-07_rb,0.39144e-07_rb,0.51019e-07_rb,0.54404e-07_rb /)
      kbo(:, 3,59, 2) = (/ &
        & 0.40346e-10_rb,0.27842e-07_rb,0.50692e-07_rb,0.65716e-07_rb,0.68440e-07_rb /)
      kbo(:, 4,59, 2) = (/ &
        & 0.42908e-10_rb,0.34761e-07_rb,0.60846e-07_rb,0.82834e-07_rb,0.85713e-07_rb /)
      kbo(:, 5,59, 2) = (/ &
        & 0.46213e-10_rb,0.43467e-07_rb,0.74626e-07_rb,0.99144e-07_rb,0.10518e-06_rb /)
      kbo(:, 1,13, 3) = (/ &
        & 0.91180e-05_rb,0.12754e-02_rb,0.19900e-02_rb,0.24440e-02_rb,0.40576e-02_rb /)
      kbo(:, 2,13, 3) = (/ &
        & 0.95037e-05_rb,0.14150e-02_rb,0.22145e-02_rb,0.28094e-02_rb,0.44375e-02_rb /)
      kbo(:, 3,13, 3) = (/ &
        & 0.99757e-05_rb,0.15626e-02_rb,0.24320e-02_rb,0.31375e-02_rb,0.48097e-02_rb /)
      kbo(:, 4,13, 3) = (/ &
        & 0.10508e-04_rb,0.16997e-02_rb,0.26926e-02_rb,0.34483e-02_rb,0.51554e-02_rb /)
      kbo(:, 5,13, 3) = (/ &
        & 0.11033e-04_rb,0.18373e-02_rb,0.29684e-02_rb,0.37861e-02_rb,0.54896e-02_rb /)
      kbo(:, 1,14, 3) = (/ &
        & 0.73209e-05_rb,0.10727e-02_rb,0.16787e-02_rb,0.20881e-02_rb,0.33041e-02_rb /)
      kbo(:, 2,14, 3) = (/ &
        & 0.76795e-05_rb,0.11937e-02_rb,0.18683e-02_rb,0.23616e-02_rb,0.36246e-02_rb /)
      kbo(:, 3,14, 3) = (/ &
        & 0.80417e-05_rb,0.13177e-02_rb,0.20567e-02_rb,0.26465e-02_rb,0.39438e-02_rb /)
      kbo(:, 4,14, 3) = (/ &
        & 0.84296e-05_rb,0.14317e-02_rb,0.22826e-02_rb,0.29022e-02_rb,0.42368e-02_rb /)
      kbo(:, 5,14, 3) = (/ &
        & 0.88950e-05_rb,0.15495e-02_rb,0.25084e-02_rb,0.31901e-02_rb,0.44992e-02_rb /)
      kbo(:, 1,15, 3) = (/ &
        & 0.58275e-05_rb,0.89895e-03_rb,0.14161e-02_rb,0.17760e-02_rb,0.26508e-02_rb /)
      kbo(:, 2,15, 3) = (/ &
        & 0.61166e-05_rb,0.10025e-02_rb,0.15706e-02_rb,0.19893e-02_rb,0.29219e-02_rb /)
      kbo(:, 3,15, 3) = (/ &
        & 0.64340e-05_rb,0.11055e-02_rb,0.17375e-02_rb,0.22237e-02_rb,0.31718e-02_rb /)
      kbo(:, 4,15, 3) = (/ &
        & 0.67501e-05_rb,0.12028e-02_rb,0.19291e-02_rb,0.24433e-02_rb,0.34015e-02_rb /)
      kbo(:, 5,15, 3) = (/ &
        & 0.70843e-05_rb,0.13034e-02_rb,0.21152e-02_rb,0.26892e-02_rb,0.36350e-02_rb /)
      kbo(:, 1,16, 3) = (/ &
        & 0.45776e-05_rb,0.75233e-03_rb,0.11906e-02_rb,0.14957e-02_rb,0.21076e-02_rb /)
      kbo(:, 2,16, 3) = (/ &
        & 0.48262e-05_rb,0.84051e-03_rb,0.13192e-02_rb,0.16780e-02_rb,0.23267e-02_rb /)
      kbo(:, 3,16, 3) = (/ &
        & 0.51024e-05_rb,0.92603e-03_rb,0.14653e-02_rb,0.18671e-02_rb,0.25242e-02_rb /)
      kbo(:, 4,16, 3) = (/ &
        & 0.53856e-05_rb,0.10093e-02_rb,0.16288e-02_rb,0.20599e-02_rb,0.27134e-02_rb /)
      kbo(:, 5,16, 3) = (/ &
        & 0.56625e-05_rb,0.10939e-02_rb,0.17834e-02_rb,0.22693e-02_rb,0.29092e-02_rb /)
      kbo(:, 1,17, 3) = (/ &
        & 0.35978e-05_rb,0.63055e-03_rb,0.10001e-02_rb,0.12618e-02_rb,0.16759e-02_rb /)
      kbo(:, 2,17, 3) = (/ &
        & 0.38090e-05_rb,0.70519e-03_rb,0.11080e-02_rb,0.14142e-02_rb,0.18563e-02_rb /)
      kbo(:, 3,17, 3) = (/ &
        & 0.40365e-05_rb,0.77593e-03_rb,0.12352e-02_rb,0.15692e-02_rb,0.20215e-02_rb /)
      kbo(:, 4,17, 3) = (/ &
        & 0.42692e-05_rb,0.84648e-03_rb,0.13753e-02_rb,0.17361e-02_rb,0.21882e-02_rb /)
      kbo(:, 5,17, 3) = (/ &
        & 0.45325e-05_rb,0.91815e-03_rb,0.15031e-02_rb,0.19156e-02_rb,0.23511e-02_rb /)
      kbo(:, 1,18, 3) = (/ &
        & 0.28577e-05_rb,0.52923e-03_rb,0.84060e-03_rb,0.10641e-02_rb,0.13524e-02_rb /)
      kbo(:, 2,18, 3) = (/ &
        & 0.30189e-05_rb,0.59275e-03_rb,0.93264e-03_rb,0.11925e-02_rb,0.15051e-02_rb /)
      kbo(:, 3,18, 3) = (/ &
        & 0.31971e-05_rb,0.65140e-03_rb,0.10422e-02_rb,0.13174e-02_rb,0.16413e-02_rb /)
      kbo(:, 4,18, 3) = (/ &
        & 0.33981e-05_rb,0.71144e-03_rb,0.11594e-02_rb,0.14613e-02_rb,0.17851e-02_rb /)
      kbo(:, 5,18, 3) = (/ &
        & 0.36127e-05_rb,0.77200e-03_rb,0.12685e-02_rb,0.16164e-02_rb,0.19166e-02_rb /)
      kbo(:, 1,19, 3) = (/ &
        & 0.22832e-05_rb,0.44489e-03_rb,0.70523e-03_rb,0.89150e-03_rb,0.11054e-02_rb /)
      kbo(:, 2,19, 3) = (/ &
        & 0.24005e-05_rb,0.49860e-03_rb,0.78596e-03_rb,0.10033e-02_rb,0.12310e-02_rb /)
      kbo(:, 3,19, 3) = (/ &
        & 0.25465e-05_rb,0.54746e-03_rb,0.88096e-03_rb,0.11084e-02_rb,0.13479e-02_rb /)
      kbo(:, 4,19, 3) = (/ &
        & 0.27167e-05_rb,0.59760e-03_rb,0.97685e-03_rb,0.12305e-02_rb,0.14668e-02_rb /)
      kbo(:, 5,19, 3) = (/ &
        & 0.28927e-05_rb,0.64844e-03_rb,0.10695e-02_rb,0.13632e-02_rb,0.15746e-02_rb /)
      kbo(:, 1,20, 3) = (/ &
        & 0.18426e-05_rb,0.37567e-03_rb,0.59276e-03_rb,0.75002e-03_rb,0.91159e-03_rb /)
      kbo(:, 2,20, 3) = (/ &
        & 0.19324e-05_rb,0.42049e-03_rb,0.66376e-03_rb,0.84304e-03_rb,0.10175e-02_rb /)
      kbo(:, 3,20, 3) = (/ &
        & 0.20563e-05_rb,0.46104e-03_rb,0.74561e-03_rb,0.93568e-03_rb,0.11153e-02_rb /)
      kbo(:, 4,20, 3) = (/ &
        & 0.21952e-05_rb,0.50302e-03_rb,0.82491e-03_rb,0.10410e-02_rb,0.12131e-02_rb /)
      kbo(:, 5,20, 3) = (/ &
        & 0.23429e-05_rb,0.54570e-03_rb,0.90324e-03_rb,0.11512e-02_rb,0.13002e-02_rb /)
      kbo(:, 1,21, 3) = (/ &
        & 0.14914e-05_rb,0.31727e-03_rb,0.49879e-03_rb,0.63141e-03_rb,0.75480e-03_rb /)
      kbo(:, 2,21, 3) = (/ &
        & 0.15711e-05_rb,0.35367e-03_rb,0.56105e-03_rb,0.70859e-03_rb,0.84278e-03_rb /)
      kbo(:, 3,21, 3) = (/ &
        & 0.16720e-05_rb,0.38776e-03_rb,0.63122e-03_rb,0.79066e-03_rb,0.92492e-03_rb /)
      kbo(:, 4,21, 3) = (/ &
        & 0.17830e-05_rb,0.42327e-03_rb,0.69655e-03_rb,0.88311e-03_rb,0.10041e-02_rb /)
      kbo(:, 5,21, 3) = (/ &
        & 0.19065e-05_rb,0.45905e-03_rb,0.76238e-03_rb,0.97320e-03_rb,0.10767e-02_rb /)
      kbo(:, 1,22, 3) = (/ &
        & 0.12007e-05_rb,0.26976e-03_rb,0.42316e-03_rb,0.53750e-03_rb,0.62747e-03_rb /)
      kbo(:, 2,22, 3) = (/ &
        & 0.12744e-05_rb,0.29880e-03_rb,0.47840e-03_rb,0.60094e-03_rb,0.70134e-03_rb /)
      kbo(:, 3,22, 3) = (/ &
        & 0.13572e-05_rb,0.32813e-03_rb,0.53672e-03_rb,0.67341e-03_rb,0.76916e-03_rb /)
      kbo(:, 4,22, 3) = (/ &
        & 0.14459e-05_rb,0.35761e-03_rb,0.59154e-03_rb,0.75421e-03_rb,0.83371e-03_rb /)
      kbo(:, 5,22, 3) = (/ &
        & 0.15538e-05_rb,0.38792e-03_rb,0.64663e-03_rb,0.82877e-03_rb,0.89529e-03_rb /)
      kbo(:, 1,23, 3) = (/ &
        & 0.96861e-06_rb,0.22918e-03_rb,0.36015e-03_rb,0.45719e-03_rb,0.52404e-03_rb /)
      kbo(:, 2,23, 3) = (/ &
        & 0.10336e-05_rb,0.25321e-03_rb,0.40834e-03_rb,0.51161e-03_rb,0.58351e-03_rb /)
      kbo(:, 3,23, 3) = (/ &
        & 0.10985e-05_rb,0.27767e-03_rb,0.45571e-03_rb,0.57459e-03_rb,0.63985e-03_rb /)
      kbo(:, 4,23, 3) = (/ &
        & 0.11768e-05_rb,0.30252e-03_rb,0.50195e-03_rb,0.64190e-03_rb,0.69303e-03_rb /)
      kbo(:, 5,23, 3) = (/ &
        & 0.12615e-05_rb,0.32794e-03_rb,0.54831e-03_rb,0.70579e-03_rb,0.74540e-03_rb /)
      kbo(:, 1,24, 3) = (/ &
        & 0.77560e-06_rb,0.19437e-03_rb,0.30748e-03_rb,0.38828e-03_rb,0.43529e-03_rb /)
      kbo(:, 2,24, 3) = (/ &
        & 0.82580e-06_rb,0.21438e-03_rb,0.34835e-03_rb,0.43617e-03_rb,0.48373e-03_rb /)
      kbo(:, 3,24, 3) = (/ &
        & 0.88200e-06_rb,0.23492e-03_rb,0.38680e-03_rb,0.49169e-03_rb,0.53189e-03_rb /)
      kbo(:, 4,24, 3) = (/ &
        & 0.94947e-06_rb,0.25573e-03_rb,0.42635e-03_rb,0.54641e-03_rb,0.57637e-03_rb /)
      kbo(:, 5,24, 3) = (/ &
        & 0.10182e-05_rb,0.27745e-03_rb,0.46607e-03_rb,0.60046e-03_rb,0.61998e-03_rb /)
      kbo(:, 1,25, 3) = (/ &
        & 0.61963e-06_rb,0.16454e-03_rb,0.26284e-03_rb,0.33016e-03_rb,0.36284e-03_rb /)
      kbo(:, 2,25, 3) = (/ &
        & 0.66146e-06_rb,0.18162e-03_rb,0.29714e-03_rb,0.37315e-03_rb,0.40370e-03_rb /)
      kbo(:, 3,25, 3) = (/ &
        & 0.71111e-06_rb,0.19882e-03_rb,0.32942e-03_rb,0.42130e-03_rb,0.44382e-03_rb /)
      kbo(:, 4,25, 3) = (/ &
        & 0.76573e-06_rb,0.21663e-03_rb,0.36228e-03_rb,0.46640e-03_rb,0.48052e-03_rb /)
      kbo(:, 5,25, 3) = (/ &
        & 0.82100e-06_rb,0.23503e-03_rb,0.39627e-03_rb,0.51132e-03_rb,0.51776e-03_rb /)
      kbo(:, 1,26, 3) = (/ &
        & 0.49718e-06_rb,0.13982e-03_rb,0.22523e-03_rb,0.28275e-03_rb,0.30558e-03_rb /)
      kbo(:, 2,26, 3) = (/ &
        & 0.53365e-06_rb,0.15431e-03_rb,0.25368e-03_rb,0.32044e-03_rb,0.33895e-03_rb /)
      kbo(:, 3,26, 3) = (/ &
        & 0.57705e-06_rb,0.16884e-03_rb,0.28082e-03_rb,0.36019e-03_rb,0.37173e-03_rb /)
      kbo(:, 4,26, 3) = (/ &
        & 0.62061e-06_rb,0.18393e-03_rb,0.30865e-03_rb,0.39866e-03_rb,0.40243e-03_rb /)
      kbo(:, 5,26, 3) = (/ &
        & 0.66427e-06_rb,0.19968e-03_rb,0.33757e-03_rb,0.43663e-03_rb,0.43309e-03_rb /)
      kbo(:, 1,27, 3) = (/ &
        & 0.39961e-06_rb,0.11889e-03_rb,0.19361e-03_rb,0.24270e-03_rb,0.25693e-03_rb /)
      kbo(:, 2,27, 3) = (/ &
        & 0.43136e-06_rb,0.13100e-03_rb,0.21632e-03_rb,0.27591e-03_rb,0.28463e-03_rb /)
      kbo(:, 3,27, 3) = (/ &
        & 0.46490e-06_rb,0.14333e-03_rb,0.23966e-03_rb,0.30849e-03_rb,0.31106e-03_rb /)
      kbo(:, 4,27, 3) = (/ &
        & 0.50009e-06_rb,0.15630e-03_rb,0.26346e-03_rb,0.34083e-03_rb,0.33713e-03_rb /)
      kbo(:, 5,27, 3) = (/ &
        & 0.53808e-06_rb,0.16999e-03_rb,0.28768e-03_rb,0.37323e-03_rb,0.36279e-03_rb /)
      kbo(:, 1,28, 3) = (/ &
        & 0.31986e-06_rb,0.10106e-03_rb,0.16585e-03_rb,0.20891e-03_rb,0.21558e-03_rb /)
      kbo(:, 2,28, 3) = (/ &
        & 0.34540e-06_rb,0.11119e-03_rb,0.18500e-03_rb,0.23760e-03_rb,0.23854e-03_rb /)
      kbo(:, 3,28, 3) = (/ &
        & 0.37226e-06_rb,0.12183e-03_rb,0.20456e-03_rb,0.26419e-03_rb,0.26053e-03_rb /)
      kbo(:, 4,28, 3) = (/ &
        & 0.40317e-06_rb,0.13290e-03_rb,0.22487e-03_rb,0.29156e-03_rb,0.28262e-03_rb /)
      kbo(:, 5,28, 3) = (/ &
        & 0.43271e-06_rb,0.14482e-03_rb,0.24541e-03_rb,0.31934e-03_rb,0.30419e-03_rb /)
      kbo(:, 1,29, 3) = (/ &
        & 0.25677e-06_rb,0.86060e-04_rb,0.14206e-03_rb,0.18023e-03_rb,0.18141e-03_rb /)
      kbo(:, 2,29, 3) = (/ &
        & 0.27736e-06_rb,0.94677e-04_rb,0.15841e-03_rb,0.20378e-03_rb,0.20088e-03_rb /)
      kbo(:, 3,29, 3) = (/ &
        & 0.30104e-06_rb,0.10372e-03_rb,0.17515e-03_rb,0.22663e-03_rb,0.21921e-03_rb /)
      kbo(:, 4,29, 3) = (/ &
        & 0.32584e-06_rb,0.11340e-03_rb,0.19232e-03_rb,0.24953e-03_rb,0.23789e-03_rb /)
      kbo(:, 5,29, 3) = (/ &
        & 0.34891e-06_rb,0.12368e-03_rb,0.20976e-03_rb,0.27364e-03_rb,0.25570e-03_rb /)
      kbo(:, 1,30, 3) = (/ &
        & 0.20556e-06_rb,0.73274e-04_rb,0.12170e-03_rb,0.15588e-03_rb,0.15312e-03_rb /)
      kbo(:, 2,30, 3) = (/ &
        & 0.22376e-06_rb,0.80651e-04_rb,0.13568e-03_rb,0.17524e-03_rb,0.16948e-03_rb /)
      kbo(:, 3,30, 3) = (/ &
        & 0.24313e-06_rb,0.88481e-04_rb,0.14992e-03_rb,0.19447e-03_rb,0.18477e-03_rb /)
      kbo(:, 4,30, 3) = (/ &
        & 0.26233e-06_rb,0.96824e-04_rb,0.16458e-03_rb,0.21409e-03_rb,0.20027e-03_rb /)
      kbo(:, 5,30, 3) = (/ &
        & 0.28249e-06_rb,0.10580e-03_rb,0.17987e-03_rb,0.23435e-03_rb,0.21532e-03_rb /)
      kbo(:, 1,31, 3) = (/ &
        & 0.16553e-06_rb,0.62536e-04_rb,0.10457e-03_rb,0.13443e-03_rb,0.12968e-03_rb /)
      kbo(:, 2,31, 3) = (/ &
        & 0.18078e-06_rb,0.68863e-04_rb,0.11643e-03_rb,0.15060e-03_rb,0.14304e-03_rb /)
      kbo(:, 3,31, 3) = (/ &
        & 0.19619e-06_rb,0.75670e-04_rb,0.12864e-03_rb,0.16681e-03_rb,0.15615e-03_rb /)
      kbo(:, 4,31, 3) = (/ &
        & 0.21158e-06_rb,0.82899e-04_rb,0.14114e-03_rb,0.18385e-03_rb,0.16911e-03_rb /)
      kbo(:, 5,31, 3) = (/ &
        & 0.22915e-06_rb,0.90629e-04_rb,0.15456e-03_rb,0.20099e-03_rb,0.18146e-03_rb /)
      kbo(:, 1,32, 3) = (/ &
        & 0.13380e-06_rb,0.53407e-04_rb,0.89930e-04_rb,0.11591e-03_rb,0.11005e-03_rb /)
      kbo(:, 2,32, 3) = (/ &
        & 0.14604e-06_rb,0.58887e-04_rb,0.10001e-03_rb,0.12963e-03_rb,0.12107e-03_rb /)
      kbo(:, 3,32, 3) = (/ &
        & 0.15848e-06_rb,0.64790e-04_rb,0.11040e-03_rb,0.14349e-03_rb,0.13230e-03_rb /)
      kbo(:, 4,32, 3) = (/ &
        & 0.17190e-06_rb,0.71169e-04_rb,0.12125e-03_rb,0.15792e-03_rb,0.14297e-03_rb /)
      kbo(:, 5,32, 3) = (/ &
        & 0.18587e-06_rb,0.77792e-04_rb,0.13292e-03_rb,0.17265e-03_rb,0.15308e-03_rb /)
      kbo(:, 1,33, 3) = (/ &
        & 0.10830e-06_rb,0.45686e-04_rb,0.77339e-04_rb,0.99867e-04_rb,0.93562e-04_rb /)
      kbo(:, 2,33, 3) = (/ &
        & 0.11765e-06_rb,0.50526e-04_rb,0.85959e-04_rb,0.11153e-03_rb,0.10269e-03_rb /)
      kbo(:, 3,33, 3) = (/ &
        & 0.12824e-06_rb,0.55627e-04_rb,0.94900e-04_rb,0.12354e-03_rb,0.11205e-03_rb /)
      kbo(:, 4,33, 3) = (/ &
        & 0.13958e-06_rb,0.61158e-04_rb,0.10440e-03_rb,0.13573e-03_rb,0.12099e-03_rb /)
      kbo(:, 5,33, 3) = (/ &
        & 0.15078e-06_rb,0.66909e-04_rb,0.11431e-03_rb,0.14861e-03_rb,0.12918e-03_rb /)
      kbo(:, 1,34, 3) = (/ &
        & 0.87537e-07_rb,0.39073e-04_rb,0.66330e-04_rb,0.85860e-04_rb,0.79296e-04_rb /)
      kbo(:, 2,34, 3) = (/ &
        & 0.95270e-07_rb,0.43282e-04_rb,0.73743e-04_rb,0.95733e-04_rb,0.87082e-04_rb /)
      kbo(:, 3,34, 3) = (/ &
        & 0.10423e-06_rb,0.47701e-04_rb,0.81447e-04_rb,0.10609e-03_rb,0.94857e-04_rb /)
      kbo(:, 4,34, 3) = (/ &
        & 0.11348e-06_rb,0.52484e-04_rb,0.89624e-04_rb,0.11638e-03_rb,0.10211e-03_rb /)
      kbo(:, 5,34, 3) = (/ &
        & 0.12289e-06_rb,0.57431e-04_rb,0.98122e-04_rb,0.12760e-03_rb,0.10866e-03_rb /)
      kbo(:, 1,35, 3) = (/ &
        & 0.69891e-07_rb,0.33129e-04_rb,0.56323e-04_rb,0.73036e-04_rb,0.66568e-04_rb /)
      kbo(:, 2,35, 3) = (/ &
        & 0.76465e-07_rb,0.36724e-04_rb,0.62627e-04_rb,0.81520e-04_rb,0.73176e-04_rb /)
      kbo(:, 3,35, 3) = (/ &
        & 0.83833e-07_rb,0.40619e-04_rb,0.69293e-04_rb,0.90161e-04_rb,0.79548e-04_rb /)
      kbo(:, 4,35, 3) = (/ &
        & 0.91436e-07_rb,0.44671e-04_rb,0.76362e-04_rb,0.99075e-04_rb,0.85497e-04_rb /)
      kbo(:, 5,35, 3) = (/ &
        & 0.99263e-07_rb,0.48886e-04_rb,0.83550e-04_rb,0.10849e-03_rb,0.90720e-04_rb /)
      kbo(:, 1,36, 3) = (/ &
        & 0.55080e-07_rb,0.27796e-04_rb,0.47261e-04_rb,0.61299e-04_rb,0.55237e-04_rb /)
      kbo(:, 2,36, 3) = (/ &
        & 0.60562e-07_rb,0.30850e-04_rb,0.52635e-04_rb,0.68515e-04_rb,0.60794e-04_rb /)
      kbo(:, 3,36, 3) = (/ &
        & 0.66440e-07_rb,0.34200e-04_rb,0.58298e-04_rb,0.75847e-04_rb,0.66097e-04_rb /)
      kbo(:, 4,36, 3) = (/ &
        & 0.72913e-07_rb,0.37650e-04_rb,0.64310e-04_rb,0.83530e-04_rb,0.71032e-04_rb /)
      kbo(:, 5,36, 3) = (/ &
        & 0.79545e-07_rb,0.41216e-04_rb,0.70397e-04_rb,0.91304e-04_rb,0.75193e-04_rb /)
      kbo(:, 1,37, 3) = (/ &
        & 0.43644e-07_rb,0.22997e-04_rb,0.39088e-04_rb,0.50714e-04_rb,0.45374e-04_rb /)
      kbo(:, 2,37, 3) = (/ &
        & 0.48182e-07_rb,0.25585e-04_rb,0.43661e-04_rb,0.56813e-04_rb,0.50057e-04_rb /)
      kbo(:, 3,37, 3) = (/ &
        & 0.53067e-07_rb,0.28429e-04_rb,0.48465e-04_rb,0.63020e-04_rb,0.54490e-04_rb /)
      kbo(:, 4,37, 3) = (/ &
        & 0.58349e-07_rb,0.31365e-04_rb,0.53603e-04_rb,0.69516e-04_rb,0.58585e-04_rb /)
      kbo(:, 5,37, 3) = (/ &
        & 0.63727e-07_rb,0.34398e-04_rb,0.58710e-04_rb,0.76032e-04_rb,0.62046e-04_rb /)
      kbo(:, 1,38, 3) = (/ &
        & 0.34661e-07_rb,0.19003e-04_rb,0.32295e-04_rb,0.41898e-04_rb,0.37207e-04_rb /)
      kbo(:, 2,38, 3) = (/ &
        & 0.38414e-07_rb,0.21200e-04_rb,0.36170e-04_rb,0.47045e-04_rb,0.41150e-04_rb /)
      kbo(:, 3,38, 3) = (/ &
        & 0.42378e-07_rb,0.23605e-04_rb,0.40247e-04_rb,0.52306e-04_rb,0.44872e-04_rb /)
      kbo(:, 4,38, 3) = (/ &
        & 0.46598e-07_rb,0.26110e-04_rb,0.44620e-04_rb,0.57800e-04_rb,0.48293e-04_rb /)
      kbo(:, 5,38, 3) = (/ &
        & 0.50995e-07_rb,0.28683e-04_rb,0.48935e-04_rb,0.63250e-04_rb,0.51155e-04_rb /)
      kbo(:, 1,39, 3) = (/ &
        & 0.27545e-07_rb,0.15700e-04_rb,0.26661e-04_rb,0.34603e-04_rb,0.30487e-04_rb /)
      kbo(:, 2,39, 3) = (/ &
        & 0.30553e-07_rb,0.17568e-04_rb,0.29948e-04_rb,0.38935e-04_rb,0.33813e-04_rb /)
      kbo(:, 3,39, 3) = (/ &
        & 0.33772e-07_rb,0.19601e-04_rb,0.33411e-04_rb,0.43403e-04_rb,0.36922e-04_rb /)
      kbo(:, 4,39, 3) = (/ &
        & 0.37164e-07_rb,0.21726e-04_rb,0.37113e-04_rb,0.48030e-04_rb,0.39771e-04_rb /)
      kbo(:, 5,39, 3) = (/ &
        & 0.40795e-07_rb,0.23901e-04_rb,0.40765e-04_rb,0.52607e-04_rb,0.42152e-04_rb /)
      kbo(:, 1,40, 3) = (/ &
        & 0.21852e-07_rb,0.12819e-04_rb,0.21757e-04_rb,0.28221e-04_rb,0.24745e-04_rb /)
      kbo(:, 2,40, 3) = (/ &
        & 0.24207e-07_rb,0.14395e-04_rb,0.24527e-04_rb,0.31867e-04_rb,0.27502e-04_rb /)
      kbo(:, 3,40, 3) = (/ &
        & 0.26806e-07_rb,0.16092e-04_rb,0.27462e-04_rb,0.35646e-04_rb,0.30145e-04_rb /)
      kbo(:, 4,40, 3) = (/ &
        & 0.29612e-07_rb,0.17894e-04_rb,0.30572e-04_rb,0.39527e-04_rb,0.32543e-04_rb /)
      kbo(:, 5,40, 3) = (/ &
        & 0.32577e-07_rb,0.19746e-04_rb,0.33672e-04_rb,0.43399e-04_rb,0.34556e-04_rb /)
      kbo(:, 1,41, 3) = (/ &
        & 0.17336e-07_rb,0.10441e-04_rb,0.17695e-04_rb,0.22954e-04_rb,0.20042e-04_rb /)
      kbo(:, 2,41, 3) = (/ &
        & 0.19187e-07_rb,0.11771e-04_rb,0.20025e-04_rb,0.26029e-04_rb,0.22346e-04_rb /)
      kbo(:, 3,41, 3) = (/ &
        & 0.21311e-07_rb,0.13190e-04_rb,0.22516e-04_rb,0.29201e-04_rb,0.24577e-04_rb /)
      kbo(:, 4,41, 3) = (/ &
        & 0.23597e-07_rb,0.14712e-04_rb,0.25122e-04_rb,0.32466e-04_rb,0.26584e-04_rb /)
      kbo(:, 5,41, 3) = (/ &
        & 0.25958e-07_rb,0.16280e-04_rb,0.27744e-04_rb,0.35762e-04_rb,0.28295e-04_rb /)
      kbo(:, 1,42, 3) = (/ &
        & 0.13752e-07_rb,0.84970e-05_rb,0.14379e-04_rb,0.18640e-04_rb,0.16205e-04_rb /)
      kbo(:, 2,42, 3) = (/ &
        & 0.15240e-07_rb,0.96082e-05_rb,0.16325e-04_rb,0.21225e-04_rb,0.18124e-04_rb /)
      kbo(:, 3,42, 3) = (/ &
        & 0.16924e-07_rb,0.10802e-04_rb,0.18430e-04_rb,0.23896e-04_rb,0.20024e-04_rb /)
      kbo(:, 4,42, 3) = (/ &
        & 0.18767e-07_rb,0.12087e-04_rb,0.20609e-04_rb,0.26651e-04_rb,0.21698e-04_rb /)
      kbo(:, 5,42, 3) = (/ &
        & 0.20666e-07_rb,0.13409e-04_rb,0.22837e-04_rb,0.29427e-04_rb,0.23153e-04_rb /)
      kbo(:, 1,43, 3) = (/ &
        & 0.10827e-07_rb,0.68624e-05_rb,0.11586e-04_rb,0.14999e-04_rb,0.12975e-04_rb /)
      kbo(:, 2,43, 3) = (/ &
        & 0.12030e-07_rb,0.77847e-05_rb,0.13205e-04_rb,0.17164e-04_rb,0.14605e-04_rb /)
      kbo(:, 3,43, 3) = (/ &
        & 0.13352e-07_rb,0.87779e-05_rb,0.14974e-04_rb,0.19410e-04_rb,0.16231e-04_rb /)
      kbo(:, 4,43, 3) = (/ &
        & 0.14829e-07_rb,0.98620e-05_rb,0.16795e-04_rb,0.21751e-04_rb,0.17634e-04_rb /)
      kbo(:, 5,43, 3) = (/ &
        & 0.16396e-07_rb,0.10975e-04_rb,0.18682e-04_rb,0.24066e-04_rb,0.18879e-04_rb /)
      kbo(:, 1,44, 3) = (/ &
        & 0.85731e-08_rb,0.55132e-05_rb,0.92862e-05_rb,0.12000e-04_rb,0.10347e-04_rb /)
      kbo(:, 2,44, 3) = (/ &
        & 0.95033e-08_rb,0.62816e-05_rb,0.10629e-04_rb,0.13817e-04_rb,0.11712e-04_rb /)
      kbo(:, 3,44, 3) = (/ &
        & 0.10537e-07_rb,0.70964e-05_rb,0.12108e-04_rb,0.15700e-04_rb,0.13095e-04_rb /)
      kbo(:, 4,44, 3) = (/ &
        & 0.11722e-07_rb,0.80120e-05_rb,0.13621e-04_rb,0.17679e-04_rb,0.14296e-04_rb /)
      kbo(:, 5,44, 3) = (/ &
        & 0.12991e-07_rb,0.89457e-05_rb,0.15220e-04_rb,0.19614e-04_rb,0.15350e-04_rb /)
      kbo(:, 1,45, 3) = (/ &
        & 0.67515e-08_rb,0.44212e-05_rb,0.74244e-05_rb,0.95776e-05_rb,0.82429e-05_rb /)
      kbo(:, 2,45, 3) = (/ &
        & 0.74999e-08_rb,0.50603e-05_rb,0.85360e-05_rb,0.11095e-04_rb,0.93791e-05_rb /)
      kbo(:, 3,45, 3) = (/ &
        & 0.83182e-08_rb,0.57411e-05_rb,0.97686e-05_rb,0.12667e-04_rb,0.10528e-04_rb /)
      kbo(:, 4,45, 3) = (/ &
        & 0.92526e-08_rb,0.64942e-05_rb,0.11028e-04_rb,0.14328e-04_rb,0.11566e-04_rb /)
      kbo(:, 5,45, 3) = (/ &
        & 0.10269e-07_rb,0.72769e-05_rb,0.12375e-04_rb,0.15964e-04_rb,0.12472e-04_rb /)
      kbo(:, 1,46, 3) = (/ &
        & 0.53031e-08_rb,0.35301e-05_rb,0.59050e-05_rb,0.75965e-05_rb,0.65452e-05_rb /)
      kbo(:, 2,46, 3) = (/ &
        & 0.58907e-08_rb,0.40561e-05_rb,0.68233e-05_rb,0.88621e-05_rb,0.74771e-05_rb /)
      kbo(:, 3,46, 3) = (/ &
        & 0.65422e-08_rb,0.46270e-05_rb,0.78393e-05_rb,0.10169e-04_rb,0.84370e-05_rb /)
      kbo(:, 4,46, 3) = (/ &
        & 0.72669e-08_rb,0.52433e-05_rb,0.89061e-05_rb,0.11563e-04_rb,0.93268e-05_rb /)
      kbo(:, 5,46, 3) = (/ &
        & 0.80880e-08_rb,0.58967e-05_rb,0.10016e-04_rb,0.12930e-04_rb,0.10106e-04_rb /)
      kbo(:, 1,47, 3) = (/ &
        & 0.41459e-08_rb,0.27965e-05_rb,0.46529e-05_rb,0.59648e-05_rb,0.51552e-05_rb /)
      kbo(:, 2,47, 3) = (/ &
        & 0.46324e-08_rb,0.32253e-05_rb,0.54100e-05_rb,0.70094e-05_rb,0.59110e-05_rb /)
      kbo(:, 3,47, 3) = (/ &
        & 0.51433e-08_rb,0.37014e-05_rb,0.62395e-05_rb,0.80976e-05_rb,0.67100e-05_rb /)
      kbo(:, 4,47, 3) = (/ &
        & 0.57045e-08_rb,0.42091e-05_rb,0.71381e-05_rb,0.92537e-05_rb,0.74896e-05_rb /)
      kbo(:, 5,47, 3) = (/ &
        & 0.63526e-08_rb,0.47541e-05_rb,0.80573e-05_rb,0.10428e-04_rb,0.81540e-05_rb /)
      kbo(:, 1,48, 3) = (/ &
        & 0.32549e-08_rb,0.22089e-05_rb,0.36571e-05_rb,0.46663e-05_rb,0.40387e-05_rb /)
      kbo(:, 2,48, 3) = (/ &
        & 0.36192e-08_rb,0.25581e-05_rb,0.42752e-05_rb,0.55265e-05_rb,0.46582e-05_rb /)
      kbo(:, 3,48, 3) = (/ &
        & 0.40215e-08_rb,0.29515e-05_rb,0.49553e-05_rb,0.64327e-05_rb,0.53236e-05_rb /)
      kbo(:, 4,48, 3) = (/ &
        & 0.44736e-08_rb,0.33690e-05_rb,0.57015e-05_rb,0.73859e-05_rb,0.59960e-05_rb /)
      kbo(:, 5,48, 3) = (/ &
        & 0.49760e-08_rb,0.38276e-05_rb,0.64596e-05_rb,0.83844e-05_rb,0.65647e-05_rb /)
      kbo(:, 1,49, 3) = (/ &
        & 0.25432e-08_rb,0.17390e-05_rb,0.28607e-05_rb,0.36438e-05_rb,0.31425e-05_rb /)
      kbo(:, 2,49, 3) = (/ &
        & 0.28364e-08_rb,0.20219e-05_rb,0.33698e-05_rb,0.43412e-05_rb,0.36654e-05_rb /)
      kbo(:, 3,49, 3) = (/ &
        & 0.31601e-08_rb,0.23472e-05_rb,0.39232e-05_rb,0.50964e-05_rb,0.42142e-05_rb /)
      kbo(:, 4,49, 3) = (/ &
        & 0.35101e-08_rb,0.26948e-05_rb,0.45429e-05_rb,0.58797e-05_rb,0.47715e-05_rb /)
      kbo(:, 5,49, 3) = (/ &
        & 0.38989e-08_rb,0.30718e-05_rb,0.51731e-05_rb,0.67190e-05_rb,0.52720e-05_rb /)
      kbo(:, 1,50, 3) = (/ &
        & 0.20093e-08_rb,0.13704e-05_rb,0.22373e-05_rb,0.28375e-05_rb,0.24550e-05_rb /)
      kbo(:, 2,50, 3) = (/ &
        & 0.22270e-08_rb,0.16020e-05_rb,0.26626e-05_rb,0.34144e-05_rb,0.28963e-05_rb /)
      kbo(:, 3,50, 3) = (/ &
        & 0.24797e-08_rb,0.18686e-05_rb,0.31131e-05_rb,0.40348e-05_rb,0.33398e-05_rb /)
      kbo(:, 4,50, 3) = (/ &
        & 0.27614e-08_rb,0.21593e-05_rb,0.36199e-05_rb,0.46856e-05_rb,0.38019e-05_rb /)
      kbo(:, 5,50, 3) = (/ &
        & 0.30674e-08_rb,0.24699e-05_rb,0.41517e-05_rb,0.53844e-05_rb,0.42322e-05_rb /)
      kbo(:, 1,51, 3) = (/ &
        & 0.15907e-08_rb,0.10803e-05_rb,0.17518e-05_rb,0.22064e-05_rb,0.19254e-05_rb /)
      kbo(:, 2,51, 3) = (/ &
        & 0.17509e-08_rb,0.12691e-05_rb,0.21000e-05_rb,0.26794e-05_rb,0.22832e-05_rb /)
      kbo(:, 3,51, 3) = (/ &
        & 0.19509e-08_rb,0.14861e-05_rb,0.24726e-05_rb,0.31957e-05_rb,0.26418e-05_rb /)
      kbo(:, 4,51, 3) = (/ &
        & 0.21703e-08_rb,0.17284e-05_rb,0.28838e-05_rb,0.37325e-05_rb,0.30257e-05_rb /)
      kbo(:, 5,51, 3) = (/ &
        & 0.24124e-08_rb,0.19844e-05_rb,0.33305e-05_rb,0.43128e-05_rb,0.34015e-05_rb /)
      kbo(:, 1,52, 3) = (/ &
        & 0.12564e-08_rb,0.84669e-06_rb,0.13717e-05_rb,0.17147e-05_rb,0.14947e-05_rb /)
      kbo(:, 2,52, 3) = (/ &
        & 0.13794e-08_rb,0.10024e-05_rb,0.16523e-05_rb,0.20993e-05_rb,0.17870e-05_rb /)
      kbo(:, 3,52, 3) = (/ &
        & 0.15320e-08_rb,0.11790e-05_rb,0.19577e-05_rb,0.25220e-05_rb,0.20845e-05_rb /)
      kbo(:, 4,52, 3) = (/ &
        & 0.17130e-08_rb,0.13791e-05_rb,0.22952e-05_rb,0.29677e-05_rb,0.24040e-05_rb /)
      kbo(:, 5,52, 3) = (/ &
        & 0.19041e-08_rb,0.15914e-05_rb,0.26635e-05_rb,0.34439e-05_rb,0.27247e-05_rb /)
      kbo(:, 1,53, 3) = (/ &
        & 0.99523e-09_rb,0.66219e-06_rb,0.10632e-05_rb,0.13341e-05_rb,0.11550e-05_rb /)
      kbo(:, 2,53, 3) = (/ &
        & 0.10903e-08_rb,0.79022e-06_rb,0.12914e-05_rb,0.16401e-05_rb,0.13929e-05_rb /)
      kbo(:, 3,53, 3) = (/ &
        & 0.12082e-08_rb,0.93266e-06_rb,0.15445e-05_rb,0.19841e-05_rb,0.16452e-05_rb /)
      kbo(:, 4,53, 3) = (/ &
        & 0.13437e-08_rb,0.10969e-05_rb,0.18212e-05_rb,0.23517e-05_rb,0.19029e-05_rb /)
      kbo(:, 5,53, 3) = (/ &
        & 0.14972e-08_rb,0.12734e-05_rb,0.21276e-05_rb,0.27431e-05_rb,0.21722e-05_rb /)
      kbo(:, 1,54, 3) = (/ &
        & 0.78782e-09_rb,0.51993e-06_rb,0.82752e-06_rb,0.10421e-05_rb,0.88632e-06_rb /)
      kbo(:, 2,54, 3) = (/ &
        & 0.86642e-09_rb,0.62440e-06_rb,0.10138e-05_rb,0.12820e-05_rb,0.10958e-05_rb /)
      kbo(:, 3,54, 3) = (/ &
        & 0.95141e-09_rb,0.74051e-06_rb,0.12221e-05_rb,0.15626e-05_rb,0.13014e-05_rb /)
      kbo(:, 4,54, 3) = (/ &
        & 0.10622e-08_rb,0.87438e-06_rb,0.14481e-05_rb,0.18675e-05_rb,0.15138e-05_rb /)
      kbo(:, 5,54, 3) = (/ &
        & 0.11829e-08_rb,0.10217e-05_rb,0.17025e-05_rb,0.21900e-05_rb,0.17349e-05_rb /)
      kbo(:, 1,55, 3) = (/ &
        & 0.62734e-09_rb,0.40844e-06_rb,0.64675e-06_rb,0.81168e-06_rb,0.68015e-06_rb /)
      kbo(:, 2,55, 3) = (/ &
        & 0.68670e-09_rb,0.49437e-06_rb,0.79822e-06_rb,0.10031e-05_rb,0.85955e-06_rb /)
      kbo(:, 3,55, 3) = (/ &
        & 0.75551e-09_rb,0.58813e-06_rb,0.96640e-06_rb,0.12305e-05_rb,0.10290e-05_rb /)
      kbo(:, 4,55, 3) = (/ &
        & 0.83896e-09_rb,0.69762e-06_rb,0.11529e-05_rb,0.14845e-05_rb,0.12025e-05_rb /)
      kbo(:, 5,55, 3) = (/ &
        & 0.93727e-09_rb,0.81992e-06_rb,0.13618e-05_rb,0.17489e-05_rb,0.13856e-05_rb /)
      kbo(:, 1,56, 3) = (/ &
        & 0.50214e-09_rb,0.32028e-06_rb,0.50524e-06_rb,0.63147e-06_rb,0.51903e-06_rb /)
      kbo(:, 2,56, 3) = (/ &
        & 0.54775e-09_rb,0.38841e-06_rb,0.62589e-06_rb,0.78445e-06_rb,0.67099e-06_rb /)
      kbo(:, 3,56, 3) = (/ &
        & 0.59971e-09_rb,0.46602e-06_rb,0.76356e-06_rb,0.96978e-06_rb,0.80912e-06_rb /)
      kbo(:, 4,56, 3) = (/ &
        & 0.66466e-09_rb,0.55509e-06_rb,0.91533e-06_rb,0.11766e-05_rb,0.95284e-06_rb /)
      kbo(:, 5,56, 3) = (/ &
        & 0.73931e-09_rb,0.65625e-06_rb,0.10864e-05_rb,0.13946e-05_rb,0.11055e-05_rb /)
      kbo(:, 1,57, 3) = (/ &
        & 0.40171e-09_rb,0.25009e-06_rb,0.39211e-06_rb,0.49250e-06_rb,0.39332e-06_rb /)
      kbo(:, 2,57, 3) = (/ &
        & 0.43379e-09_rb,0.30473e-06_rb,0.48932e-06_rb,0.61342e-06_rb,0.52076e-06_rb /)
      kbo(:, 3,57, 3) = (/ &
        & 0.47729e-09_rb,0.36874e-06_rb,0.59978e-06_rb,0.76246e-06_rb,0.63556e-06_rb /)
      kbo(:, 4,57, 3) = (/ &
        & 0.52454e-09_rb,0.44075e-06_rb,0.72488e-06_rb,0.93079e-06_rb,0.75448e-06_rb /)
      kbo(:, 5,57, 3) = (/ &
        & 0.58546e-09_rb,0.52389e-06_rb,0.86565e-06_rb,0.11107e-05_rb,0.87858e-06_rb /)
      kbo(:, 1,58, 3) = (/ &
        & 0.32222e-09_rb,0.19536e-06_rb,0.30484e-06_rb,0.38123e-06_rb,0.29798e-06_rb /)
      kbo(:, 2,58, 3) = (/ &
        & 0.34550e-09_rb,0.24003e-06_rb,0.38206e-06_rb,0.48033e-06_rb,0.40193e-06_rb /)
      kbo(:, 3,58, 3) = (/ &
        & 0.37930e-09_rb,0.29179e-06_rb,0.47226e-06_rb,0.59938e-06_rb,0.50056e-06_rb /)
      kbo(:, 4,58, 3) = (/ &
        & 0.41675e-09_rb,0.35035e-06_rb,0.57500e-06_rb,0.73642e-06_rb,0.59836e-06_rb /)
      kbo(:, 5,58, 3) = (/ &
        & 0.46320e-09_rb,0.41854e-06_rb,0.69041e-06_rb,0.88548e-06_rb,0.70003e-06_rb /)
      kbo(:, 1,59, 3) = (/ &
        & 0.26166e-09_rb,0.15868e-06_rb,0.24697e-06_rb,0.30892e-06_rb,0.23964e-06_rb /)
      kbo(:, 2,59, 3) = (/ &
        & 0.28095e-09_rb,0.19552e-06_rb,0.31068e-06_rb,0.39037e-06_rb,0.32469e-06_rb /)
      kbo(:, 3,59, 3) = (/ &
        & 0.30730e-09_rb,0.23856e-06_rb,0.38554e-06_rb,0.48902e-06_rb,0.40738e-06_rb /)
      kbo(:, 4,59, 3) = (/ &
        & 0.33781e-09_rb,0.28749e-06_rb,0.47157e-06_rb,0.60286e-06_rb,0.48803e-06_rb /)
      kbo(:, 5,59, 3) = (/ &
        & 0.37526e-09_rb,0.34439e-06_rb,0.56789e-06_rb,0.72742e-06_rb,0.57238e-06_rb /)
      kbo(:, 1,13, 4) = (/ &
        & 0.49038e-04_rb,0.50781e-02_rb,0.78254e-02_rb,0.96484e-02_rb,0.10614e-01_rb /)
      kbo(:, 2,13, 4) = (/ &
        & 0.54225e-04_rb,0.54905e-02_rb,0.85710e-02_rb,0.10460e-01_rb,0.11206e-01_rb /)
      kbo(:, 3,13, 4) = (/ &
        & 0.61134e-04_rb,0.58773e-02_rb,0.93183e-02_rb,0.11347e-01_rb,0.11729e-01_rb /)
      kbo(:, 4,13, 4) = (/ &
        & 0.68793e-04_rb,0.62794e-02_rb,0.99768e-02_rb,0.12248e-01_rb,0.12250e-01_rb /)
      kbo(:, 5,13, 4) = (/ &
        & 0.77429e-04_rb,0.66893e-02_rb,0.10624e-01_rb,0.13064e-01_rb,0.12730e-01_rb /)
      kbo(:, 1,14, 4) = (/ &
        & 0.40706e-04_rb,0.43045e-02_rb,0.66240e-02_rb,0.81227e-02_rb,0.87105e-02_rb /)
      kbo(:, 2,14, 4) = (/ &
        & 0.45789e-04_rb,0.46404e-02_rb,0.72547e-02_rb,0.88508e-02_rb,0.92183e-02_rb /)
      kbo(:, 3,14, 4) = (/ &
        & 0.51359e-04_rb,0.49670e-02_rb,0.78649e-02_rb,0.95928e-02_rb,0.96590e-02_rb /)
      kbo(:, 4,14, 4) = (/ &
        & 0.58448e-04_rb,0.53112e-02_rb,0.84165e-02_rb,0.10356e-01_rb,0.10085e-01_rb /)
      kbo(:, 5,14, 4) = (/ &
        & 0.66585e-04_rb,0.56590e-02_rb,0.89770e-02_rb,0.11022e-01_rb,0.10540e-01_rb /)
      kbo(:, 1,15, 4) = (/ &
        & 0.33161e-04_rb,0.36374e-02_rb,0.56007e-02_rb,0.68408e-02_rb,0.70399e-02_rb /)
      kbo(:, 2,15, 4) = (/ &
        & 0.37316e-04_rb,0.39145e-02_rb,0.61318e-02_rb,0.74812e-02_rb,0.74533e-02_rb /)
      kbo(:, 3,15, 4) = (/ &
        & 0.42184e-04_rb,0.41949e-02_rb,0.66272e-02_rb,0.81097e-02_rb,0.78393e-02_rb /)
      kbo(:, 4,15, 4) = (/ &
        & 0.48249e-04_rb,0.44861e-02_rb,0.70934e-02_rb,0.87330e-02_rb,0.82262e-02_rb /)
      kbo(:, 5,15, 4) = (/ &
        & 0.54912e-04_rb,0.47829e-02_rb,0.75688e-02_rb,0.92830e-02_rb,0.86058e-02_rb /)
      kbo(:, 1,16, 4) = (/ &
        & 0.26598e-04_rb,0.30649e-02_rb,0.47323e-02_rb,0.57742e-02_rb,0.56644e-02_rb /)
      kbo(:, 2,16, 4) = (/ &
        & 0.30096e-04_rb,0.32950e-02_rb,0.51778e-02_rb,0.63145e-02_rb,0.60193e-02_rb /)
      kbo(:, 3,16, 4) = (/ &
        & 0.34085e-04_rb,0.35350e-02_rb,0.55806e-02_rb,0.68447e-02_rb,0.63708e-02_rb /)
      kbo(:, 4,16, 4) = (/ &
        & 0.38919e-04_rb,0.37831e-02_rb,0.59719e-02_rb,0.73465e-02_rb,0.67088e-02_rb /)
      kbo(:, 5,16, 4) = (/ &
        & 0.44492e-04_rb,0.40393e-02_rb,0.63750e-02_rb,0.78120e-02_rb,0.70249e-02_rb /)
      kbo(:, 1,17, 4) = (/ &
        & 0.21449e-04_rb,0.25756e-02_rb,0.39936e-02_rb,0.48662e-02_rb,0.46205e-02_rb /)
      kbo(:, 2,17, 4) = (/ &
        & 0.24103e-04_rb,0.27706e-02_rb,0.43619e-02_rb,0.53164e-02_rb,0.49202e-02_rb /)
      kbo(:, 3,17, 4) = (/ &
        & 0.27451e-04_rb,0.29765e-02_rb,0.46959e-02_rb,0.57632e-02_rb,0.52122e-02_rb /)
      kbo(:, 4,17, 4) = (/ &
        & 0.31381e-04_rb,0.31884e-02_rb,0.50222e-02_rb,0.61735e-02_rb,0.54790e-02_rb /)
      kbo(:, 5,17, 4) = (/ &
        & 0.35953e-04_rb,0.34099e-02_rb,0.53645e-02_rb,0.65659e-02_rb,0.57419e-02_rb /)
      kbo(:, 1,18, 4) = (/ &
        & 0.17468e-04_rb,0.21642e-02_rb,0.33642e-02_rb,0.40972e-02_rb,0.37921e-02_rb /)
      kbo(:, 2,18, 4) = (/ &
        & 0.19506e-04_rb,0.23308e-02_rb,0.36683e-02_rb,0.44696e-02_rb,0.40385e-02_rb /)
      kbo(:, 3,18, 4) = (/ &
        & 0.22326e-04_rb,0.25072e-02_rb,0.39454e-02_rb,0.48507e-02_rb,0.42773e-02_rb /)
      kbo(:, 4,18, 4) = (/ &
        & 0.25558e-04_rb,0.26901e-02_rb,0.42240e-02_rb,0.51855e-02_rb,0.45000e-02_rb /)
      kbo(:, 5,18, 4) = (/ &
        & 0.29519e-04_rb,0.28816e-02_rb,0.45135e-02_rb,0.55168e-02_rb,0.47259e-02_rb /)
      kbo(:, 1,19, 4) = (/ &
        & 0.14162e-04_rb,0.18165e-02_rb,0.28311e-02_rb,0.34491e-02_rb,0.31263e-02_rb /)
      kbo(:, 2,19, 4) = (/ &
        & 0.15880e-04_rb,0.19594e-02_rb,0.30792e-02_rb,0.37577e-02_rb,0.33277e-02_rb /)
      kbo(:, 3,19, 4) = (/ &
        & 0.18250e-04_rb,0.21124e-02_rb,0.33110e-02_rb,0.40717e-02_rb,0.35255e-02_rb /)
      kbo(:, 4,19, 4) = (/ &
        & 0.20948e-04_rb,0.22711e-02_rb,0.35512e-02_rb,0.43541e-02_rb,0.37114e-02_rb /)
      kbo(:, 5,19, 4) = (/ &
        & 0.24279e-04_rb,0.24360e-02_rb,0.37972e-02_rb,0.46336e-02_rb,0.39063e-02_rb /)
      kbo(:, 1,20, 4) = (/ &
        & 0.11677e-04_rb,0.15268e-02_rb,0.23854e-02_rb,0.29054e-02_rb,0.25927e-02_rb /)
      kbo(:, 2,20, 4) = (/ &
        & 0.13127e-04_rb,0.16502e-02_rb,0.25884e-02_rb,0.31642e-02_rb,0.27584e-02_rb /)
      kbo(:, 3,20, 4) = (/ &
        & 0.15145e-04_rb,0.17827e-02_rb,0.27854e-02_rb,0.34198e-02_rb,0.29228e-02_rb /)
      kbo(:, 4,20, 4) = (/ &
        & 0.17426e-04_rb,0.19208e-02_rb,0.29919e-02_rb,0.36563e-02_rb,0.30836e-02_rb /)
      kbo(:, 5,20, 4) = (/ &
        & 0.20187e-04_rb,0.20638e-02_rb,0.32027e-02_rb,0.38955e-02_rb,0.32517e-02_rb /)
      kbo(:, 1,21, 4) = (/ &
        & 0.96248e-05_rb,0.12839e-02_rb,0.20069e-02_rb,0.24439e-02_rb,0.21521e-02_rb /)
      kbo(:, 2,21, 4) = (/ &
        & 0.10884e-04_rb,0.13905e-02_rb,0.21753e-02_rb,0.26634e-02_rb,0.22901e-02_rb /)
      kbo(:, 3,21, 4) = (/ &
        & 0.12576e-04_rb,0.15057e-02_rb,0.23430e-02_rb,0.28714e-02_rb,0.24271e-02_rb /)
      kbo(:, 4,21, 4) = (/ &
        & 0.14492e-04_rb,0.16245e-02_rb,0.25229e-02_rb,0.30709e-02_rb,0.25653e-02_rb /)
      kbo(:, 5,21, 4) = (/ &
        & 0.16785e-04_rb,0.17487e-02_rb,0.27053e-02_rb,0.32769e-02_rb,0.27041e-02_rb /)
      kbo(:, 1,22, 4) = (/ &
        & 0.78987e-05_rb,0.10841e-02_rb,0.16963e-02_rb,0.20646e-02_rb,0.17920e-02_rb /)
      kbo(:, 2,22, 4) = (/ &
        & 0.90688e-05_rb,0.11783e-02_rb,0.18364e-02_rb,0.22523e-02_rb,0.19054e-02_rb /)
      kbo(:, 3,22, 4) = (/ &
        & 0.10436e-04_rb,0.12776e-02_rb,0.19832e-02_rb,0.24220e-02_rb,0.20200e-02_rb /)
      kbo(:, 4,22, 4) = (/ &
        & 0.12051e-04_rb,0.13800e-02_rb,0.21391e-02_rb,0.25904e-02_rb,0.21364e-02_rb /)
      kbo(:, 5,22, 4) = (/ &
        & 0.13992e-04_rb,0.14877e-02_rb,0.22983e-02_rb,0.27690e-02_rb,0.22519e-02_rb /)
      kbo(:, 1,23, 4) = (/ &
        & 0.65143e-05_rb,0.91629e-03_rb,0.14337e-02_rb,0.17468e-02_rb,0.14890e-02_rb /)
      kbo(:, 2,23, 4) = (/ &
        & 0.75295e-05_rb,0.99880e-03_rb,0.15532e-02_rb,0.19017e-02_rb,0.15844e-02_rb /)
      kbo(:, 3,23, 4) = (/ &
        & 0.86172e-05_rb,0.10843e-02_rb,0.16828e-02_rb,0.20454e-02_rb,0.16822e-02_rb /)
      kbo(:, 4,23, 4) = (/ &
        & 0.10017e-04_rb,0.11729e-02_rb,0.18176e-02_rb,0.21912e-02_rb,0.17800e-02_rb /)
      kbo(:, 5,23, 4) = (/ &
        & 0.11640e-04_rb,0.12668e-02_rb,0.19549e-02_rb,0.23447e-02_rb,0.18765e-02_rb /)
      kbo(:, 1,24, 4) = (/ &
        & 0.53432e-05_rb,0.77571e-03_rb,0.12114e-02_rb,0.14805e-02_rb,0.12372e-02_rb /)
      kbo(:, 2,24, 4) = (/ &
        & 0.61515e-05_rb,0.84718e-03_rb,0.13153e-02_rb,0.16078e-02_rb,0.13181e-02_rb /)
      kbo(:, 3,24, 4) = (/ &
        & 0.70923e-05_rb,0.92024e-03_rb,0.14293e-02_rb,0.17300e-02_rb,0.13989e-02_rb /)
      kbo(:, 4,24, 4) = (/ &
        & 0.82684e-05_rb,0.99740e-03_rb,0.15447e-02_rb,0.18573e-02_rb,0.14811e-02_rb /)
      kbo(:, 5,24, 4) = (/ &
        & 0.96268e-05_rb,0.10789e-02_rb,0.16627e-02_rb,0.19892e-02_rb,0.15621e-02_rb /)
      kbo(:, 1,25, 4) = (/ &
        & 0.43932e-05_rb,0.65898e-03_rb,0.10259e-02_rb,0.12568e-02_rb,0.10287e-02_rb /)
      kbo(:, 2,25, 4) = (/ &
        & 0.50483e-05_rb,0.71973e-03_rb,0.11174e-02_rb,0.13610e-02_rb,0.10973e-02_rb /)
      kbo(:, 3,25, 4) = (/ &
        & 0.58527e-05_rb,0.78269e-03_rb,0.12157e-02_rb,0.14658e-02_rb,0.11648e-02_rb /)
      kbo(:, 4,25, 4) = (/ &
        & 0.68258e-05_rb,0.84968e-03_rb,0.13158e-02_rb,0.15774e-02_rb,0.12354e-02_rb /)
      kbo(:, 5,25, 4) = (/ &
        & 0.79780e-05_rb,0.92093e-03_rb,0.14171e-02_rb,0.16917e-02_rb,0.13029e-02_rb /)
      kbo(:, 1,26, 4) = (/ &
        & 0.36349e-05_rb,0.56139e-03_rb,0.87232e-03_rb,0.10671e-02_rb,0.85747e-03_rb /)
      kbo(:, 2,26, 4) = (/ &
        & 0.41763e-05_rb,0.61307e-03_rb,0.95284e-03_rb,0.11559e-02_rb,0.91640e-03_rb /)
      kbo(:, 3,26, 4) = (/ &
        & 0.48643e-05_rb,0.66769e-03_rb,0.10377e-02_rb,0.12480e-02_rb,0.97417e-03_rb /)
      kbo(:, 4,26, 4) = (/ &
        & 0.56806e-05_rb,0.72618e-03_rb,0.11238e-02_rb,0.13443e-02_rb,0.10341e-02_rb /)
      kbo(:, 5,26, 4) = (/ &
        & 0.66574e-05_rb,0.78791e-03_rb,0.12118e-02_rb,0.14426e-02_rb,0.10908e-02_rb /)
      kbo(:, 1,27, 4) = (/ &
        & 0.29849e-05_rb,0.47845e-03_rb,0.74324e-03_rb,0.90695e-03_rb,0.71654e-03_rb /)
      kbo(:, 2,27, 4) = (/ &
        & 0.34528e-05_rb,0.52305e-03_rb,0.81407e-03_rb,0.98348e-03_rb,0.76622e-03_rb /)
      kbo(:, 3,27, 4) = (/ &
        & 0.40386e-05_rb,0.57067e-03_rb,0.88635e-03_rb,0.10643e-02_rb,0.81617e-03_rb /)
      kbo(:, 4,27, 4) = (/ &
        & 0.47349e-05_rb,0.62159e-03_rb,0.96045e-03_rb,0.11476e-02_rb,0.86667e-03_rb /)
      kbo(:, 5,27, 4) = (/ &
        & 0.55504e-05_rb,0.67486e-03_rb,0.10373e-02_rb,0.12327e-02_rb,0.91437e-03_rb /)
      kbo(:, 1,28, 4) = (/ &
        & 0.24474e-05_rb,0.40819e-03_rb,0.63469e-03_rb,0.77184e-03_rb,0.59959e-03_rb /)
      kbo(:, 2,28, 4) = (/ &
        & 0.28527e-05_rb,0.44694e-03_rb,0.69550e-03_rb,0.83809e-03_rb,0.64151e-03_rb /)
      kbo(:, 3,28, 4) = (/ &
        & 0.33444e-05_rb,0.48804e-03_rb,0.75797e-03_rb,0.90921e-03_rb,0.68464e-03_rb /)
      kbo(:, 4,28, 4) = (/ &
        & 0.39332e-05_rb,0.53229e-03_rb,0.82174e-03_rb,0.98103e-03_rb,0.72680e-03_rb /)
      kbo(:, 5,28, 4) = (/ &
        & 0.46245e-05_rb,0.57864e-03_rb,0.88872e-03_rb,0.10544e-02_rb,0.76723e-03_rb /)
      kbo(:, 1,29, 4) = (/ &
        & 0.20253e-05_rb,0.34921e-03_rb,0.54365e-03_rb,0.65865e-03_rb,0.50331e-03_rb /)
      kbo(:, 2,29, 4) = (/ &
        & 0.23720e-05_rb,0.38278e-03_rb,0.59577e-03_rb,0.71729e-03_rb,0.53864e-03_rb /)
      kbo(:, 3,29, 4) = (/ &
        & 0.27945e-05_rb,0.41887e-03_rb,0.64907e-03_rb,0.77823e-03_rb,0.57581e-03_rb /)
      kbo(:, 4,29, 4) = (/ &
        & 0.32936e-05_rb,0.45699e-03_rb,0.70481e-03_rb,0.84074e-03_rb,0.61089e-03_rb /)
      kbo(:, 5,29, 4) = (/ &
        & 0.38881e-05_rb,0.49757e-03_rb,0.76325e-03_rb,0.90356e-03_rb,0.64564e-03_rb /)
      kbo(:, 1,30, 4) = (/ &
        & 0.16797e-05_rb,0.29918e-03_rb,0.46598e-03_rb,0.56316e-03_rb,0.42270e-03_rb /)
      kbo(:, 2,30, 4) = (/ &
        & 0.19764e-05_rb,0.32839e-03_rb,0.51076e-03_rb,0.61439e-03_rb,0.45306e-03_rb /)
      kbo(:, 3,30, 4) = (/ &
        & 0.23384e-05_rb,0.35985e-03_rb,0.55686e-03_rb,0.66712e-03_rb,0.48445e-03_rb /)
      kbo(:, 4,30, 4) = (/ &
        & 0.27662e-05_rb,0.39302e-03_rb,0.60525e-03_rb,0.72092e-03_rb,0.51425e-03_rb /)
      kbo(:, 5,30, 4) = (/ &
        & 0.32687e-05_rb,0.42837e-03_rb,0.65610e-03_rb,0.77570e-03_rb,0.54396e-03_rb /)
      kbo(:, 1,31, 4) = (/ &
        & 0.13991e-05_rb,0.25687e-03_rb,0.39996e-03_rb,0.48307e-03_rb,0.35570e-03_rb /)
      kbo(:, 2,31, 4) = (/ &
        & 0.16564e-05_rb,0.28246e-03_rb,0.43858e-03_rb,0.52743e-03_rb,0.38211e-03_rb /)
      kbo(:, 3,31, 4) = (/ &
        & 0.19660e-05_rb,0.30971e-03_rb,0.47856e-03_rb,0.57309e-03_rb,0.40826e-03_rb /)
      kbo(:, 4,31, 4) = (/ &
        & 0.23334e-05_rb,0.33881e-03_rb,0.52066e-03_rb,0.61919e-03_rb,0.43367e-03_rb /)
      kbo(:, 5,31, 4) = (/ &
        & 0.27587e-05_rb,0.36964e-03_rb,0.56499e-03_rb,0.66660e-03_rb,0.45969e-03_rb /)
      kbo(:, 1,32, 4) = (/ &
        & 0.11711e-05_rb,0.22102e-03_rb,0.34394e-03_rb,0.41512e-03_rb,0.29980e-03_rb /)
      kbo(:, 2,32, 4) = (/ &
        & 0.13928e-05_rb,0.24337e-03_rb,0.37713e-03_rb,0.45351e-03_rb,0.32253e-03_rb /)
      kbo(:, 3,32, 4) = (/ &
        & 0.16589e-05_rb,0.26704e-03_rb,0.41202e-03_rb,0.49272e-03_rb,0.34442e-03_rb /)
      kbo(:, 4,32, 4) = (/ &
        & 0.19740e-05_rb,0.29261e-03_rb,0.44855e-03_rb,0.53274e-03_rb,0.36633e-03_rb /)
      kbo(:, 5,32, 4) = (/ &
        & 0.23323e-05_rb,0.31973e-03_rb,0.48726e-03_rb,0.57383e-03_rb,0.38915e-03_rb /)
      kbo(:, 1,33, 4) = (/ &
        & 0.98300e-06_rb,0.19047e-03_rb,0.29592e-03_rb,0.35739e-03_rb,0.25303e-03_rb /)
      kbo(:, 2,33, 4) = (/ &
        & 0.11751e-05_rb,0.20983e-03_rb,0.32474e-03_rb,0.39043e-03_rb,0.27259e-03_rb /)
      kbo(:, 3,33, 4) = (/ &
        & 0.14027e-05_rb,0.23065e-03_rb,0.35505e-03_rb,0.42405e-03_rb,0.29120e-03_rb /)
      kbo(:, 4,33, 4) = (/ &
        & 0.16692e-05_rb,0.25302e-03_rb,0.38699e-03_rb,0.45888e-03_rb,0.31018e-03_rb /)
      kbo(:, 5,33, 4) = (/ &
        & 0.19716e-05_rb,0.27715e-03_rb,0.42119e-03_rb,0.49468e-03_rb,0.32989e-03_rb /)
      kbo(:, 1,34, 4) = (/ &
        & 0.82915e-06_rb,0.16396e-03_rb,0.25433e-03_rb,0.30700e-03_rb,0.21406e-03_rb /)
      kbo(:, 2,34, 4) = (/ &
        & 0.99352e-06_rb,0.18083e-03_rb,0.27935e-03_rb,0.33552e-03_rb,0.23048e-03_rb /)
      kbo(:, 3,34, 4) = (/ &
        & 0.11877e-05_rb,0.19915e-03_rb,0.30578e-03_rb,0.36455e-03_rb,0.24659e-03_rb /)
      kbo(:, 4,34, 4) = (/ &
        & 0.14139e-05_rb,0.21886e-03_rb,0.33384e-03_rb,0.39476e-03_rb,0.26317e-03_rb /)
      kbo(:, 5,34, 4) = (/ &
        & 0.16670e-05_rb,0.24021e-03_rb,0.36390e-03_rb,0.42603e-03_rb,0.28005e-03_rb /)
      kbo(:, 1,35, 4) = (/ &
        & 0.68670e-06_rb,0.13979e-03_rb,0.21673e-03_rb,0.26161e-03_rb,0.17974e-03_rb /)
      kbo(:, 2,35, 4) = (/ &
        & 0.82437e-06_rb,0.15448e-03_rb,0.23846e-03_rb,0.28611e-03_rb,0.19377e-03_rb /)
      kbo(:, 3,35, 4) = (/ &
        & 0.98649e-06_rb,0.17044e-03_rb,0.26139e-03_rb,0.31132e-03_rb,0.20765e-03_rb /)
      kbo(:, 4,35, 4) = (/ &
        & 0.11740e-05_rb,0.18773e-03_rb,0.28609e-03_rb,0.33740e-03_rb,0.22197e-03_rb /)
      kbo(:, 5,35, 4) = (/ &
        & 0.13833e-05_rb,0.20652e-03_rb,0.31233e-03_rb,0.36449e-03_rb,0.23658e-03_rb /)
      kbo(:, 1,36, 4) = (/ &
        & 0.55564e-06_rb,0.11786e-03_rb,0.18283e-03_rb,0.22089e-03_rb,0.14951e-03_rb /)
      kbo(:, 2,36, 4) = (/ &
        & 0.66731e-06_rb,0.13053e-03_rb,0.20151e-03_rb,0.24189e-03_rb,0.16146e-03_rb /)
      kbo(:, 3,36, 4) = (/ &
        & 0.80048e-06_rb,0.14432e-03_rb,0.22154e-03_rb,0.26364e-03_rb,0.17348e-03_rb /)
      kbo(:, 4,36, 4) = (/ &
        & 0.95195e-06_rb,0.15937e-03_rb,0.24310e-03_rb,0.28609e-03_rb,0.18577e-03_rb /)
      kbo(:, 5,36, 4) = (/ &
        & 0.11217e-05_rb,0.17580e-03_rb,0.26601e-03_rb,0.30949e-03_rb,0.19850e-03_rb /)
      kbo(:, 1,37, 4) = (/ &
        & 0.44526e-06_rb,0.98334e-04_rb,0.15236e-03_rb,0.18395e-03_rb,0.12369e-03_rb /)
      kbo(:, 2,37, 4) = (/ &
        & 0.53551e-06_rb,0.10927e-03_rb,0.16837e-03_rb,0.20194e-03_rb,0.13387e-03_rb /)
      kbo(:, 3,37, 4) = (/ &
        & 0.64307e-06_rb,0.12110e-03_rb,0.18556e-03_rb,0.22059e-03_rb,0.14418e-03_rb /)
      kbo(:, 4,37, 4) = (/ &
        & 0.76688e-06_rb,0.13411e-03_rb,0.20414e-03_rb,0.23985e-03_rb,0.15477e-03_rb /)
      kbo(:, 5,37, 4) = (/ &
        & 0.90591e-06_rb,0.14842e-03_rb,0.22405e-03_rb,0.25996e-03_rb,0.16566e-03_rb /)
      kbo(:, 1,38, 4) = (/ &
        & 0.35630e-06_rb,0.81991e-04_rb,0.12690e-03_rb,0.15312e-03_rb,0.10226e-03_rb /)
      kbo(:, 2,38, 4) = (/ &
        & 0.42911e-06_rb,0.91391e-04_rb,0.14064e-03_rb,0.16849e-03_rb,0.11095e-03_rb /)
      kbo(:, 3,38, 4) = (/ &
        & 0.51583e-06_rb,0.10163e-03_rb,0.15538e-03_rb,0.18448e-03_rb,0.11977e-03_rb /)
      kbo(:, 4,38, 4) = (/ &
        & 0.61654e-06_rb,0.11287e-03_rb,0.17138e-03_rb,0.20101e-03_rb,0.12884e-03_rb /)
      kbo(:, 5,38, 4) = (/ &
        & 0.73028e-06_rb,0.12530e-03_rb,0.18862e-03_rb,0.21826e-03_rb,0.13816e-03_rb /)
      kbo(:, 1,39, 4) = (/ &
        & 0.28455e-06_rb,0.68362e-04_rb,0.10572e-03_rb,0.12747e-03_rb,0.84494e-04_rb /)
      kbo(:, 2,39, 4) = (/ &
        & 0.34313e-06_rb,0.76413e-04_rb,0.11751e-03_rb,0.14060e-03_rb,0.91905e-04_rb /)
      kbo(:, 3,39, 4) = (/ &
        & 0.41305e-06_rb,0.85299e-04_rb,0.13016e-03_rb,0.15431e-03_rb,0.99463e-04_rb /)
      kbo(:, 4,39, 4) = (/ &
        & 0.49466e-06_rb,0.95047e-04_rb,0.14396e-03_rb,0.16851e-03_rb,0.10725e-03_rb /)
      kbo(:, 5,39, 4) = (/ &
        & 0.58721e-06_rb,0.10583e-03_rb,0.15886e-03_rb,0.18332e-03_rb,0.11516e-03_rb /)
      kbo(:, 1,40, 4) = (/ &
        & 0.22416e-06_rb,0.56451e-04_rb,0.87192e-04_rb,0.10503e-03_rb,0.69299e-04_rb /)
      kbo(:, 2,40, 4) = (/ &
        & 0.27058e-06_rb,0.63325e-04_rb,0.97241e-04_rb,0.11625e-03_rb,0.75635e-04_rb /)
      kbo(:, 3,40, 4) = (/ &
        & 0.32640e-06_rb,0.70942e-04_rb,0.10801e-03_rb,0.12792e-03_rb,0.82061e-04_rb /)
      kbo(:, 4,40, 4) = (/ &
        & 0.39178e-06_rb,0.79381e-04_rb,0.11985e-03_rb,0.14005e-03_rb,0.88722e-04_rb /)
      kbo(:, 5,40, 4) = (/ &
        & 0.46694e-06_rb,0.88673e-04_rb,0.13261e-03_rb,0.15274e-03_rb,0.95437e-04_rb /)
      kbo(:, 1,41, 4) = (/ &
        & 0.17620e-06_rb,0.46495e-04_rb,0.71829e-04_rb,0.86351e-04_rb,0.56701e-04_rb /)
      kbo(:, 2,41, 4) = (/ &
        & 0.21295e-06_rb,0.52383e-04_rb,0.80380e-04_rb,0.95937e-04_rb,0.62100e-04_rb /)
      kbo(:, 3,41, 4) = (/ &
        & 0.25720e-06_rb,0.58900e-04_rb,0.89548e-04_rb,0.10594e-03_rb,0.67568e-04_rb /)
      kbo(:, 4,41, 4) = (/ &
        & 0.30957e-06_rb,0.66136e-04_rb,0.99643e-04_rb,0.11623e-03_rb,0.73280e-04_rb /)
      kbo(:, 5,41, 4) = (/ &
        & 0.37003e-06_rb,0.74203e-04_rb,0.11061e-03_rb,0.12707e-03_rb,0.78979e-04_rb /)
      kbo(:, 1,42, 4) = (/ &
        & 0.13843e-06_rb,0.38238e-04_rb,0.59124e-04_rb,0.70982e-04_rb,0.46331e-04_rb /)
      kbo(:, 2,42, 4) = (/ &
        & 0.16726e-06_rb,0.43269e-04_rb,0.66424e-04_rb,0.79125e-04_rb,0.50956e-04_rb /)
      kbo(:, 3,42, 4) = (/ &
        & 0.20232e-06_rb,0.48870e-04_rb,0.74247e-04_rb,0.87661e-04_rb,0.55561e-04_rb /)
      kbo(:, 4,42, 4) = (/ &
        & 0.24408e-06_rb,0.55119e-04_rb,0.82827e-04_rb,0.96453e-04_rb,0.60447e-04_rb /)
      kbo(:, 5,42, 4) = (/ &
        & 0.29276e-06_rb,0.62028e-04_rb,0.92221e-04_rb,0.10570e-03_rb,0.65281e-04_rb /)
      kbo(:, 1,43, 4) = (/ &
        & 0.10793e-06_rb,0.31191e-04_rb,0.48305e-04_rb,0.57948e-04_rb,0.37666e-04_rb /)
      kbo(:, 2,43, 4) = (/ &
        & 0.13017e-06_rb,0.35465e-04_rb,0.54489e-04_rb,0.64841e-04_rb,0.41556e-04_rb /)
      kbo(:, 3,43, 4) = (/ &
        & 0.15771e-06_rb,0.40235e-04_rb,0.61176e-04_rb,0.72082e-04_rb,0.45452e-04_rb /)
      kbo(:, 4,43, 4) = (/ &
        & 0.19064e-06_rb,0.45599e-04_rb,0.68488e-04_rb,0.79576e-04_rb,0.49593e-04_rb /)
      kbo(:, 5,43, 4) = (/ &
        & 0.22944e-06_rb,0.51552e-04_rb,0.76431e-04_rb,0.87462e-04_rb,0.53715e-04_rb /)
      kbo(:, 1,44, 4) = (/ &
        & 0.83879e-07_rb,0.25325e-04_rb,0.39297e-04_rb,0.47118e-04_rb,0.30510e-04_rb /)
      kbo(:, 2,44, 4) = (/ &
        & 0.10087e-06_rb,0.28920e-04_rb,0.44532e-04_rb,0.52929e-04_rb,0.33772e-04_rb /)
      kbo(:, 3,44, 4) = (/ &
        & 0.12238e-06_rb,0.32977e-04_rb,0.50189e-04_rb,0.59075e-04_rb,0.37069e-04_rb /)
      kbo(:, 4,44, 4) = (/ &
        & 0.14813e-06_rb,0.37556e-04_rb,0.56451e-04_rb,0.65448e-04_rb,0.40567e-04_rb /)
      kbo(:, 5,44, 4) = (/ &
        & 0.17874e-06_rb,0.42672e-04_rb,0.63196e-04_rb,0.72159e-04_rb,0.44085e-04_rb /)
      kbo(:, 1,45, 4) = (/ &
        & 0.65261e-07_rb,0.20516e-04_rb,0.31895e-04_rb,0.38279e-04_rb,0.24650e-04_rb /)
      kbo(:, 2,45, 4) = (/ &
        & 0.78145e-07_rb,0.23540e-04_rb,0.36319e-04_rb,0.43161e-04_rb,0.27390e-04_rb /)
      kbo(:, 3,45, 4) = (/ &
        & 0.94805e-07_rb,0.26962e-04_rb,0.41134e-04_rb,0.48357e-04_rb,0.30190e-04_rb /)
      kbo(:, 4,45, 4) = (/ &
        & 0.11490e-06_rb,0.30858e-04_rb,0.46429e-04_rb,0.53785e-04_rb,0.33136e-04_rb /)
      kbo(:, 5,45, 4) = (/ &
        & 0.13905e-06_rb,0.35210e-04_rb,0.52216e-04_rb,0.59482e-04_rb,0.36122e-04_rb /)
      kbo(:, 1,46, 4) = (/ &
        & 0.50668e-07_rb,0.16526e-04_rb,0.25752e-04_rb,0.30954e-04_rb,0.19817e-04_rb /)
      kbo(:, 2,46, 4) = (/ &
        & 0.60361e-07_rb,0.19058e-04_rb,0.29471e-04_rb,0.35060e-04_rb,0.22123e-04_rb /)
      kbo(:, 3,46, 4) = (/ &
        & 0.73101e-07_rb,0.21937e-04_rb,0.33569e-04_rb,0.39461e-04_rb,0.24498e-04_rb /)
      kbo(:, 4,46, 4) = (/ &
        & 0.88747e-07_rb,0.25222e-04_rb,0.38048e-04_rb,0.44039e-04_rb,0.26973e-04_rb /)
      kbo(:, 5,46, 4) = (/ &
        & 0.10762e-06_rb,0.28932e-04_rb,0.42964e-04_rb,0.48871e-04_rb,0.29499e-04_rb /)
      kbo(:, 1,47, 4) = (/ &
        & 0.39185e-07_rb,0.13207e-04_rb,0.20626e-04_rb,0.24835e-04_rb,0.15826e-04_rb /)
      kbo(:, 2,47, 4) = (/ &
        & 0.46335e-07_rb,0.15303e-04_rb,0.23722e-04_rb,0.28277e-04_rb,0.17762e-04_rb /)
      kbo(:, 3,47, 4) = (/ &
        & 0.55896e-07_rb,0.17703e-04_rb,0.27162e-04_rb,0.31976e-04_rb,0.19780e-04_rb /)
      kbo(:, 4,47, 4) = (/ &
        & 0.67962e-07_rb,0.20451e-04_rb,0.30940e-04_rb,0.35867e-04_rb,0.21834e-04_rb /)
      kbo(:, 5,47, 4) = (/ &
        & 0.82535e-07_rb,0.23583e-04_rb,0.35128e-04_rb,0.39933e-04_rb,0.23972e-04_rb /)
      kbo(:, 1,48, 4) = (/ &
        & 0.30435e-07_rb,0.10528e-04_rb,0.16458e-04_rb,0.19871e-04_rb,0.12630e-04_rb /)
      kbo(:, 2,48, 4) = (/ &
        & 0.35657e-07_rb,0.12255e-04_rb,0.19034e-04_rb,0.22757e-04_rb,0.14230e-04_rb /)
      kbo(:, 3,48, 4) = (/ &
        & 0.42777e-07_rb,0.14241e-04_rb,0.21916e-04_rb,0.25856e-04_rb,0.15916e-04_rb /)
      kbo(:, 4,48, 4) = (/ &
        & 0.51977e-07_rb,0.16531e-04_rb,0.25094e-04_rb,0.29142e-04_rb,0.17632e-04_rb /)
      kbo(:, 5,48, 4) = (/ &
        & 0.63246e-07_rb,0.19162e-04_rb,0.28631e-04_rb,0.32586e-04_rb,0.19447e-04_rb /)
      kbo(:, 1,49, 4) = (/ &
        & 0.23722e-07_rb,0.83656e-05_rb,0.13088e-04_rb,0.15834e-04_rb,0.10060e-04_rb /)
      kbo(:, 2,49, 4) = (/ &
        & 0.27509e-07_rb,0.97869e-05_rb,0.15227e-04_rb,0.18252e-04_rb,0.11363e-04_rb /)
      kbo(:, 3,49, 4) = (/ &
        & 0.32769e-07_rb,0.11426e-04_rb,0.17628e-04_rb,0.20853e-04_rb,0.12767e-04_rb /)
      kbo(:, 4,49, 4) = (/ &
        & 0.39719e-07_rb,0.13326e-04_rb,0.20285e-04_rb,0.23637e-04_rb,0.14223e-04_rb /)
      kbo(:, 5,49, 4) = (/ &
        & 0.48415e-07_rb,0.15524e-04_rb,0.23268e-04_rb,0.26530e-04_rb,0.15739e-04_rb /)
      kbo(:, 1,50, 4) = (/ &
        & 0.18623e-07_rb,0.66620e-05_rb,0.10425e-04_rb,0.12634e-04_rb,0.80132e-05_rb /)
      kbo(:, 2,50, 4) = (/ &
        & 0.21393e-07_rb,0.78264e-05_rb,0.12188e-04_rb,0.14643e-04_rb,0.90757e-05_rb /)
      kbo(:, 3,50, 4) = (/ &
        & 0.25281e-07_rb,0.91787e-05_rb,0.14186e-04_rb,0.16825e-04_rb,0.10247e-04_rb /)
      kbo(:, 4,50, 4) = (/ &
        & 0.30530e-07_rb,0.10758e-04_rb,0.16419e-04_rb,0.19178e-04_rb,0.11482e-04_rb /)
      kbo(:, 5,50, 4) = (/ &
        & 0.37213e-07_rb,0.12593e-04_rb,0.18920e-04_rb,0.21643e-04_rb,0.12749e-04_rb /)
      kbo(:, 1,51, 4) = (/ &
        & 0.14681e-07_rb,0.53043e-05_rb,0.82956e-05_rb,0.10064e-04_rb,0.63669e-05_rb /)
      kbo(:, 2,51, 4) = (/ &
        & 0.16717e-07_rb,0.62508e-05_rb,0.97501e-05_rb,0.11737e-04_rb,0.72473e-05_rb /)
      kbo(:, 3,51, 4) = (/ &
        & 0.19587e-07_rb,0.73643e-05_rb,0.11400e-04_rb,0.13558e-04_rb,0.82290e-05_rb /)
      kbo(:, 4,51, 4) = (/ &
        & 0.23555e-07_rb,0.86748e-05_rb,0.13268e-04_rb,0.15537e-04_rb,0.92634e-05_rb /)
      kbo(:, 5,51, 4) = (/ &
        & 0.28671e-07_rb,0.10209e-04_rb,0.15379e-04_rb,0.17635e-04_rb,0.10316e-04_rb /)
      kbo(:, 1,52, 4) = (/ &
        & 0.11626e-07_rb,0.42149e-05_rb,0.65740e-05_rb,0.79835e-05_rb,0.50595e-05_rb /)
      kbo(:, 2,52, 4) = (/ &
        & 0.13102e-07_rb,0.49884e-05_rb,0.77758e-05_rb,0.93846e-05_rb,0.57856e-05_rb /)
      kbo(:, 3,52, 4) = (/ &
        & 0.15208e-07_rb,0.58955e-05_rb,0.91392e-05_rb,0.10898e-04_rb,0.65937e-05_rb /)
      kbo(:, 4,52, 4) = (/ &
        & 0.18174e-07_rb,0.69758e-05_rb,0.10692e-04_rb,0.12554e-04_rb,0.74564e-05_rb /)
      kbo(:, 5,52, 4) = (/ &
        & 0.22057e-07_rb,0.82505e-05_rb,0.12462e-04_rb,0.14331e-04_rb,0.83301e-05_rb /)
      kbo(:, 1,53, 4) = (/ &
        & 0.92130e-08_rb,0.33378e-05_rb,0.51998e-05_rb,0.63006e-05_rb,0.40097e-05_rb /)
      kbo(:, 2,53, 4) = (/ &
        & 0.10298e-07_rb,0.39704e-05_rb,0.61839e-05_rb,0.74775e-05_rb,0.46077e-05_rb /)
      kbo(:, 3,53, 4) = (/ &
        & 0.11831e-07_rb,0.47088e-05_rb,0.73062e-05_rb,0.87345e-05_rb,0.52620e-05_rb /)
      kbo(:, 4,53, 4) = (/ &
        & 0.14030e-07_rb,0.55942e-05_rb,0.85921e-05_rb,0.10119e-04_rb,0.59838e-05_rb /)
      kbo(:, 5,53, 4) = (/ &
        & 0.16970e-07_rb,0.66447e-05_rb,0.10063e-04_rb,0.11620e-04_rb,0.67208e-05_rb /)
      kbo(:, 1,54, 4) = (/ &
        & 0.73443e-08_rb,0.26498e-05_rb,0.41229e-05_rb,0.49788e-05_rb,0.31929e-05_rb /)
      kbo(:, 2,54, 4) = (/ &
        & 0.81453e-08_rb,0.31689e-05_rb,0.49320e-05_rb,0.59701e-05_rb,0.36706e-05_rb /)
      kbo(:, 3,54, 4) = (/ &
        & 0.92853e-08_rb,0.37746e-05_rb,0.58561e-05_rb,0.70139e-05_rb,0.42105e-05_rb /)
      kbo(:, 4,54, 4) = (/ &
        & 0.10913e-07_rb,0.44986e-05_rb,0.69192e-05_rb,0.81733e-05_rb,0.48081e-05_rb /)
      kbo(:, 5,54, 4) = (/ &
        & 0.13151e-07_rb,0.53689e-05_rb,0.81469e-05_rb,0.94377e-05_rb,0.54320e-05_rb /)
      kbo(:, 1,55, 4) = (/ &
        & 0.58690e-08_rb,0.21056e-05_rb,0.32710e-05_rb,0.39362e-05_rb,0.25357e-05_rb /)
      kbo(:, 2,55, 4) = (/ &
        & 0.64824e-08_rb,0.25292e-05_rb,0.39335e-05_rb,0.47652e-05_rb,0.29266e-05_rb /)
      kbo(:, 3,55, 4) = (/ &
        & 0.73195e-08_rb,0.30260e-05_rb,0.46942e-05_rb,0.56346e-05_rb,0.33700e-05_rb /)
      kbo(:, 4,55, 4) = (/ &
        & 0.85298e-08_rb,0.36216e-05_rb,0.55727e-05_rb,0.65990e-05_rb,0.38662e-05_rb /)
      kbo(:, 5,55, 4) = (/ &
        & 0.10227e-07_rb,0.43366e-05_rb,0.65935e-05_rb,0.76627e-05_rb,0.43889e-05_rb /)
      kbo(:, 1,56, 4) = (/ &
        & 0.46879e-08_rb,0.16674e-05_rb,0.25848e-05_rb,0.30992e-05_rb,0.20097e-05_rb /)
      kbo(:, 2,56, 4) = (/ &
        & 0.51579e-08_rb,0.20145e-05_rb,0.31307e-05_rb,0.37875e-05_rb,0.23304e-05_rb /)
      kbo(:, 3,56, 4) = (/ &
        & 0.57829e-08_rb,0.24199e-05_rb,0.37529e-05_rb,0.45119e-05_rb,0.26962e-05_rb /)
      kbo(:, 4,56, 4) = (/ &
        & 0.66817e-08_rb,0.29092e-05_rb,0.44788e-05_rb,0.53142e-05_rb,0.31037e-05_rb /)
      kbo(:, 5,56, 4) = (/ &
        & 0.79548e-08_rb,0.34985e-05_rb,0.53248e-05_rb,0.62066e-05_rb,0.35387e-05_rb /)
      kbo(:, 1,57, 4) = (/ &
        & 0.37487e-08_rb,0.13177e-05_rb,0.20362e-05_rb,0.24278e-05_rb,0.15888e-05_rb /)
      kbo(:, 2,57, 4) = (/ &
        & 0.41132e-08_rb,0.16006e-05_rb,0.24863e-05_rb,0.30001e-05_rb,0.18530e-05_rb /)
      kbo(:, 3,57, 4) = (/ &
        & 0.45796e-08_rb,0.19301e-05_rb,0.29957e-05_rb,0.36053e-05_rb,0.21515e-05_rb /)
      kbo(:, 4,57, 4) = (/ &
        & 0.52447e-08_rb,0.23301e-05_rb,0.35902e-05_rb,0.42690e-05_rb,0.24844e-05_rb /)
      kbo(:, 5,57, 4) = (/ &
        & 0.61952e-08_rb,0.28175e-05_rb,0.42894e-05_rb,0.50133e-05_rb,0.28503e-05_rb /)
      kbo(:, 1,58, 4) = (/ &
        & 0.30053e-08_rb,0.10419e-05_rb,0.16034e-05_rb,0.19071e-05_rb,0.12550e-05_rb /)
      kbo(:, 2,58, 4) = (/ &
        & 0.32906e-08_rb,0.12717e-05_rb,0.19750e-05_rb,0.23751e-05_rb,0.14761e-05_rb /)
      kbo(:, 3,58, 4) = (/ &
        & 0.36412e-08_rb,0.15413e-05_rb,0.23950e-05_rb,0.28817e-05_rb,0.17173e-05_rb /)
      kbo(:, 4,58, 4) = (/ &
        & 0.41327e-08_rb,0.18673e-05_rb,0.28800e-05_rb,0.34301e-05_rb,0.19893e-05_rb /)
      kbo(:, 5,58, 4) = (/ &
        & 0.48448e-08_rb,0.22699e-05_rb,0.34559e-05_rb,0.40496e-05_rb,0.22938e-05_rb /)
      kbo(:, 1,59, 4) = (/ &
        & 0.24517e-08_rb,0.85367e-06_rb,0.13121e-05_rb,0.15587e-05_rb,0.10209e-05_rb /)
      kbo(:, 2,59, 4) = (/ &
        & 0.26814e-08_rb,0.10459e-05_rb,0.16241e-05_rb,0.19505e-05_rb,0.12067e-05_rb /)
      kbo(:, 3,59, 4) = (/ &
        & 0.29604e-08_rb,0.12726e-05_rb,0.19786e-05_rb,0.23765e-05_rb,0.14071e-05_rb /)
      kbo(:, 4,59, 4) = (/ &
        & 0.33490e-08_rb,0.15486e-05_rb,0.23884e-05_rb,0.28393e-05_rb,0.16353e-05_rb /)
      kbo(:, 5,59, 4) = (/ &
        & 0.39177e-08_rb,0.18923e-05_rb,0.28751e-05_rb,0.33648e-05_rb,0.18896e-05_rb /)
      kbo(:, 1,13, 5) = (/ &
        & 0.10606e-02_rb,0.17616e-01_rb,0.26337e-01_rb,0.30832e-01_rb,0.23250e-01_rb /)
      kbo(:, 2,13, 5) = (/ &
        & 0.11975e-02_rb,0.18851e-01_rb,0.27746e-01_rb,0.32191e-01_rb,0.24461e-01_rb /)
      kbo(:, 3,13, 5) = (/ &
        & 0.13686e-02_rb,0.20236e-01_rb,0.29224e-01_rb,0.33409e-01_rb,0.25538e-01_rb /)
      kbo(:, 4,13, 5) = (/ &
        & 0.15788e-02_rb,0.21686e-01_rb,0.30822e-01_rb,0.34581e-01_rb,0.26483e-01_rb /)
      kbo(:, 5,13, 5) = (/ &
        & 0.18422e-02_rb,0.23201e-01_rb,0.32485e-01_rb,0.35791e-01_rb,0.27339e-01_rb /)
      kbo(:, 1,14, 5) = (/ &
        & 0.90299e-03_rb,0.14939e-01_rb,0.22235e-01_rb,0.25969e-01_rb,0.19459e-01_rb /)
      kbo(:, 2,14, 5) = (/ &
        & 0.10276e-02_rb,0.16043e-01_rb,0.23464e-01_rb,0.27102e-01_rb,0.20418e-01_rb /)
      kbo(:, 3,14, 5) = (/ &
        & 0.11848e-02_rb,0.17244e-01_rb,0.24766e-01_rb,0.28137e-01_rb,0.21279e-01_rb /)
      kbo(:, 4,14, 5) = (/ &
        & 0.13763e-02_rb,0.18494e-01_rb,0.26159e-01_rb,0.29141e-01_rb,0.22035e-01_rb /)
      kbo(:, 5,14, 5) = (/ &
        & 0.16087e-02_rb,0.19801e-01_rb,0.27586e-01_rb,0.30208e-01_rb,0.22712e-01_rb /)
      kbo(:, 1,15, 5) = (/ &
        & 0.76169e-03_rb,0.12657e-01_rb,0.18744e-01_rb,0.21807e-01_rb,0.16155e-01_rb /)
      kbo(:, 2,15, 5) = (/ &
        & 0.87429e-03_rb,0.13622e-01_rb,0.19820e-01_rb,0.22753e-01_rb,0.16908e-01_rb /)
      kbo(:, 3,15, 5) = (/ &
        & 0.10153e-02_rb,0.14650e-01_rb,0.20967e-01_rb,0.23637e-01_rb,0.17597e-01_rb /)
      kbo(:, 4,15, 5) = (/ &
        & 0.11860e-02_rb,0.15728e-01_rb,0.22159e-01_rb,0.24521e-01_rb,0.18236e-01_rb /)
      kbo(:, 5,15, 5) = (/ &
        & 0.13940e-02_rb,0.16855e-01_rb,0.23377e-01_rb,0.25442e-01_rb,0.18800e-01_rb /)
      kbo(:, 1,16, 5) = (/ &
        & 0.63979e-03_rb,0.10716e-01_rb,0.15782e-01_rb,0.18280e-01_rb,0.13330e-01_rb /)
      kbo(:, 2,16, 5) = (/ &
        & 0.73992e-03_rb,0.11550e-01_rb,0.16721e-01_rb,0.19074e-01_rb,0.13955e-01_rb /)
      kbo(:, 3,16, 5) = (/ &
        & 0.86535e-03_rb,0.12432e-01_rb,0.17715e-01_rb,0.19835e-01_rb,0.14519e-01_rb /)
      kbo(:, 4,16, 5) = (/ &
        & 0.10180e-02_rb,0.13358e-01_rb,0.18733e-01_rb,0.20612e-01_rb,0.15053e-01_rb /)
      kbo(:, 5,16, 5) = (/ &
        & 0.12020e-02_rb,0.14330e-01_rb,0.19780e-01_rb,0.21390e-01_rb,0.15523e-01_rb /)
      kbo(:, 1,17, 5) = (/ &
        & 0.53827e-03_rb,0.90675e-02_rb,0.13282e-01_rb,0.15310e-01_rb,0.11002e-01_rb /)
      kbo(:, 2,17, 5) = (/ &
        & 0.62733e-03_rb,0.97818e-02_rb,0.14093e-01_rb,0.15985e-01_rb,0.11517e-01_rb /)
      kbo(:, 3,17, 5) = (/ &
        & 0.73894e-03_rb,0.10536e-01_rb,0.14937e-01_rb,0.16642e-01_rb,0.11982e-01_rb /)
      kbo(:, 4,17, 5) = (/ &
        & 0.87471e-03_rb,0.11329e-01_rb,0.15813e-01_rb,0.17308e-01_rb,0.12431e-01_rb /)
      kbo(:, 5,17, 5) = (/ &
        & 0.10369e-02_rb,0.12167e-01_rb,0.16717e-01_rb,0.17966e-01_rb,0.12833e-01_rb /)
      kbo(:, 1,18, 5) = (/ &
        & 0.45632e-03_rb,0.76631e-02_rb,0.11175e-01_rb,0.12821e-01_rb,0.91007e-02_rb /)
      kbo(:, 2,18, 5) = (/ &
        & 0.53618e-03_rb,0.82741e-02_rb,0.11867e-01_rb,0.13402e-01_rb,0.95284e-02_rb /)
      kbo(:, 3,18, 5) = (/ &
        & 0.63520e-03_rb,0.89179e-02_rb,0.12591e-01_rb,0.13960e-01_rb,0.99227e-02_rb /)
      kbo(:, 4,18, 5) = (/ &
        & 0.75513e-03_rb,0.95966e-02_rb,0.13345e-01_rb,0.14529e-01_rb,0.10294e-01_rb /)
      kbo(:, 5,18, 5) = (/ &
        & 0.89828e-03_rb,0.10325e-01_rb,0.14127e-01_rb,0.15082e-01_rb,0.10637e-01_rb /)
      kbo(:, 1,19, 5) = (/ &
        & 0.38782e-03_rb,0.64686e-02_rb,0.93947e-02_rb,0.10738e-01_rb,0.75366e-02_rb /)
      kbo(:, 2,19, 5) = (/ &
        & 0.45826e-03_rb,0.69872e-02_rb,0.99878e-02_rb,0.11230e-01_rb,0.78920e-02_rb /)
      kbo(:, 3,19, 5) = (/ &
        & 0.54544e-03_rb,0.75386e-02_rb,0.10613e-01_rb,0.11706e-01_rb,0.82219e-02_rb /)
      kbo(:, 4,19, 5) = (/ &
        & 0.65085e-03_rb,0.81243e-02_rb,0.11261e-01_rb,0.12183e-01_rb,0.85447e-02_rb /)
      kbo(:, 5,19, 5) = (/ &
        & 0.77601e-03_rb,0.87598e-02_rb,0.11933e-01_rb,0.12660e-01_rb,0.88388e-02_rb /)
      kbo(:, 1,20, 5) = (/ &
        & 0.33304e-03_rb,0.54623e-02_rb,0.79028e-02_rb,0.89960e-02_rb,0.62635e-02_rb /)
      kbo(:, 2,20, 5) = (/ &
        & 0.39553e-03_rb,0.59055e-02_rb,0.84155e-02_rb,0.94111e-02_rb,0.65613e-02_rb /)
      kbo(:, 3,20, 5) = (/ &
        & 0.47180e-03_rb,0.63779e-02_rb,0.89500e-02_rb,0.98172e-02_rb,0.68459e-02_rb /)
      kbo(:, 4,20, 5) = (/ &
        & 0.56453e-03_rb,0.68862e-02_rb,0.95048e-02_rb,0.10225e-01_rb,0.71246e-02_rb /)
      kbo(:, 5,20, 5) = (/ &
        & 0.67486e-03_rb,0.74411e-02_rb,0.10082e-01_rb,0.10637e-01_rb,0.73786e-02_rb /)
      kbo(:, 1,21, 5) = (/ &
        & 0.28693e-03_rb,0.46180e-02_rb,0.66504e-02_rb,0.75379e-02_rb,0.52071e-02_rb /)
      kbo(:, 2,21, 5) = (/ &
        & 0.34180e-03_rb,0.49948e-02_rb,0.70898e-02_rb,0.78866e-02_rb,0.54645e-02_rb /)
      kbo(:, 3,21, 5) = (/ &
        & 0.40811e-03_rb,0.54006e-02_rb,0.75474e-02_rb,0.82331e-02_rb,0.57127e-02_rb /)
      kbo(:, 4,21, 5) = (/ &
        & 0.48873e-03_rb,0.58434e-02_rb,0.80212e-02_rb,0.85844e-02_rb,0.59470e-02_rb /)
      kbo(:, 5,21, 5) = (/ &
        & 0.58593e-03_rb,0.63260e-02_rb,0.85182e-02_rb,0.89408e-02_rb,0.61662e-02_rb /)
      kbo(:, 1,22, 5) = (/ &
        & 0.24974e-03_rb,0.39240e-02_rb,0.56208e-02_rb,0.63345e-02_rb,0.43431e-02_rb /)
      kbo(:, 2,22, 5) = (/ &
        & 0.29749e-03_rb,0.42526e-02_rb,0.59996e-02_rb,0.66266e-02_rb,0.45624e-02_rb /)
      kbo(:, 3,22, 5) = (/ &
        & 0.35608e-03_rb,0.46047e-02_rb,0.63919e-02_rb,0.69259e-02_rb,0.47738e-02_rb /)
      kbo(:, 4,22, 5) = (/ &
        & 0.42699e-03_rb,0.49920e-02_rb,0.67993e-02_rb,0.72276e-02_rb,0.49714e-02_rb /)
      kbo(:, 5,22, 5) = (/ &
        & 0.51198e-03_rb,0.54166e-02_rb,0.72302e-02_rb,0.75387e-02_rb,0.51660e-02_rb /)
      kbo(:, 1,23, 5) = (/ &
        & 0.21716e-03_rb,0.33375e-02_rb,0.47528e-02_rb,0.53238e-02_rb,0.36258e-02_rb /)
      kbo(:, 2,23, 5) = (/ &
        & 0.25886e-03_rb,0.36243e-02_rb,0.50797e-02_rb,0.55740e-02_rb,0.38107e-02_rb /)
      kbo(:, 3,23, 5) = (/ &
        & 0.31026e-03_rb,0.39353e-02_rb,0.54169e-02_rb,0.58309e-02_rb,0.39876e-02_rb /)
      kbo(:, 4,23, 5) = (/ &
        & 0.37235e-03_rb,0.42757e-02_rb,0.57711e-02_rb,0.60911e-02_rb,0.41586e-02_rb /)
      kbo(:, 5,23, 5) = (/ &
        & 0.44568e-03_rb,0.46463e-02_rb,0.61476e-02_rb,0.63633e-02_rb,0.43277e-02_rb /)
      kbo(:, 1,24, 5) = (/ &
        & 0.18738e-03_rb,0.28412e-02_rb,0.40251e-02_rb,0.44754e-02_rb,0.30232e-02_rb /)
      kbo(:, 2,24, 5) = (/ &
        & 0.22422e-03_rb,0.30930e-02_rb,0.43062e-02_rb,0.46921e-02_rb,0.31788e-02_rb /)
      kbo(:, 3,24, 5) = (/ &
        & 0.26942e-03_rb,0.33694e-02_rb,0.45983e-02_rb,0.49126e-02_rb,0.33287e-02_rb /)
      kbo(:, 4,24, 5) = (/ &
        & 0.32317e-03_rb,0.36702e-02_rb,0.49092e-02_rb,0.51393e-02_rb,0.34771e-02_rb /)
      kbo(:, 5,24, 5) = (/ &
        & 0.38638e-03_rb,0.39951e-02_rb,0.52385e-02_rb,0.53815e-02_rb,0.36232e-02_rb /)
      kbo(:, 1,25, 5) = (/ &
        & 0.16224e-03_rb,0.24232e-02_rb,0.34141e-02_rb,0.37661e-02_rb,0.25262e-02_rb /)
      kbo(:, 2,25, 5) = (/ &
        & 0.19465e-03_rb,0.26463e-02_rb,0.36575e-02_rb,0.39541e-02_rb,0.26577e-02_rb /)
      kbo(:, 3,25, 5) = (/ &
        & 0.23391e-03_rb,0.28936e-02_rb,0.39130e-02_rb,0.41446e-02_rb,0.27841e-02_rb /)
      kbo(:, 4,25, 5) = (/ &
        & 0.28044e-03_rb,0.31598e-02_rb,0.41853e-02_rb,0.43442e-02_rb,0.29098e-02_rb /)
      kbo(:, 5,25, 5) = (/ &
        & 0.33483e-03_rb,0.34437e-02_rb,0.44735e-02_rb,0.45625e-02_rb,0.30351e-02_rb /)
      kbo(:, 1,26, 5) = (/ &
        & 0.14132e-03_rb,0.20737e-02_rb,0.29036e-02_rb,0.31762e-02_rb,0.21157e-02_rb /)
      kbo(:, 2,26, 5) = (/ &
        & 0.16977e-03_rb,0.22743e-02_rb,0.31152e-02_rb,0.33382e-02_rb,0.22258e-02_rb /)
      kbo(:, 3,26, 5) = (/ &
        & 0.20409e-03_rb,0.24947e-02_rb,0.33401e-02_rb,0.35048e-02_rb,0.23338e-02_rb /)
      kbo(:, 4,26, 5) = (/ &
        & 0.24472e-03_rb,0.27297e-02_rb,0.35798e-02_rb,0.36835e-02_rb,0.24421e-02_rb /)
      kbo(:, 5,26, 5) = (/ &
        & 0.29131e-03_rb,0.29804e-02_rb,0.38308e-02_rb,0.38797e-02_rb,0.25500e-02_rb /)
      kbo(:, 1,27, 5) = (/ &
        & 0.12293e-03_rb,0.17797e-02_rb,0.24725e-02_rb,0.26822e-02_rb,0.17718e-02_rb /)
      kbo(:, 2,27, 5) = (/ &
        & 0.14789e-03_rb,0.19595e-02_rb,0.26584e-02_rb,0.28222e-02_rb,0.18655e-02_rb /)
      kbo(:, 3,27, 5) = (/ &
        & 0.17790e-03_rb,0.21552e-02_rb,0.28584e-02_rb,0.29695e-02_rb,0.19589e-02_rb /)
      kbo(:, 4,27, 5) = (/ &
        & 0.21315e-03_rb,0.23624e-02_rb,0.30688e-02_rb,0.31309e-02_rb,0.20520e-02_rb /)
      kbo(:, 5,27, 5) = (/ &
        & 0.25285e-03_rb,0.25825e-02_rb,0.32876e-02_rb,0.33068e-02_rb,0.21445e-02_rb /)
      kbo(:, 1,28, 5) = (/ &
        & 0.10657e-03_rb,0.15319e-02_rb,0.21096e-02_rb,0.22678e-02_rb,0.14844e-02_rb /)
      kbo(:, 2,28, 5) = (/ &
        & 0.12856e-03_rb,0.16920e-02_rb,0.22743e-02_rb,0.23899e-02_rb,0.15639e-02_rb /)
      kbo(:, 3,28, 5) = (/ &
        & 0.15471e-03_rb,0.18636e-02_rb,0.24511e-02_rb,0.25221e-02_rb,0.16436e-02_rb /)
      kbo(:, 4,28, 5) = (/ &
        & 0.18512e-03_rb,0.20472e-02_rb,0.26364e-02_rb,0.26681e-02_rb,0.17238e-02_rb /)
      kbo(:, 5,28, 5) = (/ &
        & 0.21872e-03_rb,0.22411e-02_rb,0.28270e-02_rb,0.28260e-02_rb,0.18048e-02_rb /)
      kbo(:, 1,29, 5) = (/ &
        & 0.92876e-04_rb,0.13237e-02_rb,0.18049e-02_rb,0.19214e-02_rb,0.12471e-02_rb /)
      kbo(:, 2,29, 5) = (/ &
        & 0.11226e-03_rb,0.14659e-02_rb,0.19518e-02_rb,0.20295e-02_rb,0.13143e-02_rb /)
      kbo(:, 3,29, 5) = (/ &
        & 0.13519e-03_rb,0.16171e-02_rb,0.21089e-02_rb,0.21498e-02_rb,0.13833e-02_rb /)
      kbo(:, 4,29, 5) = (/ &
        & 0.16102e-03_rb,0.17778e-02_rb,0.22703e-02_rb,0.22810e-02_rb,0.14528e-02_rb /)
      kbo(:, 5,29, 5) = (/ &
        & 0.18981e-03_rb,0.19484e-02_rb,0.24374e-02_rb,0.24249e-02_rb,0.15236e-02_rb /)
      kbo(:, 1,30, 5) = (/ &
        & 0.80828e-04_rb,0.11471e-02_rb,0.15486e-02_rb,0.16307e-02_rb,0.10482e-02_rb /)
      kbo(:, 2,30, 5) = (/ &
        & 0.98063e-04_rb,0.12718e-02_rb,0.16797e-02_rb,0.17287e-02_rb,0.11053e-02_rb /)
      kbo(:, 3,30, 5) = (/ &
        & 0.11772e-03_rb,0.14050e-02_rb,0.18177e-02_rb,0.18379e-02_rb,0.11648e-02_rb /)
      kbo(:, 4,30, 5) = (/ &
        & 0.13981e-03_rb,0.15469e-02_rb,0.19589e-02_rb,0.19565e-02_rb,0.12258e-02_rb /)
      kbo(:, 5,30, 5) = (/ &
        & 0.16446e-03_rb,0.16947e-02_rb,0.21055e-02_rb,0.20874e-02_rb,0.12885e-02_rb /)
      kbo(:, 1,31, 5) = (/ &
        & 0.70539e-04_rb,0.99621e-03_rb,0.13325e-02_rb,0.13879e-02_rb,0.88220e-03_rb /)
      kbo(:, 2,31, 5) = (/ &
        & 0.85589e-04_rb,0.11063e-02_rb,0.14492e-02_rb,0.14776e-02_rb,0.93113e-03_rb /)
      kbo(:, 3,31, 5) = (/ &
        & 0.10244e-03_rb,0.12240e-02_rb,0.15701e-02_rb,0.15764e-02_rb,0.98237e-03_rb /)
      kbo(:, 4,31, 5) = (/ &
        & 0.12151e-03_rb,0.13488e-02_rb,0.16942e-02_rb,0.16846e-02_rb,0.10364e-02_rb /)
      kbo(:, 5,31, 5) = (/ &
        & 0.14245e-03_rb,0.14753e-02_rb,0.18229e-02_rb,0.18042e-02_rb,0.10910e-02_rb /)
      kbo(:, 1,32, 5) = (/ &
        & 0.61598e-04_rb,0.86708e-03_rb,0.11506e-02_rb,0.11857e-02_rb,0.74396e-03_rb /)
      kbo(:, 2,32, 5) = (/ &
        & 0.74698e-04_rb,0.96443e-03_rb,0.12533e-02_rb,0.12673e-02_rb,0.78590e-03_rb /)
      kbo(:, 3,32, 5) = (/ &
        & 0.89092e-04_rb,0.10684e-02_rb,0.13590e-02_rb,0.13571e-02_rb,0.83090e-03_rb /)
      kbo(:, 4,32, 5) = (/ &
        & 0.10537e-03_rb,0.11766e-02_rb,0.14682e-02_rb,0.14560e-02_rb,0.87827e-03_rb /)
      kbo(:, 5,32, 5) = (/ &
        & 0.12327e-03_rb,0.12850e-02_rb,0.15813e-02_rb,0.15643e-02_rb,0.92599e-03_rb /)
      kbo(:, 1,33, 5) = (/ &
        & 0.53748e-04_rb,0.75629e-03_rb,0.99595e-03_rb,0.10163e-02_rb,0.62809e-03_rb /)
      kbo(:, 2,33, 5) = (/ &
        & 0.64972e-04_rb,0.84265e-03_rb,0.10856e-02_rb,0.10908e-02_rb,0.66416e-03_rb /)
      kbo(:, 3,33, 5) = (/ &
        & 0.77386e-04_rb,0.93397e-03_rb,0.11787e-02_rb,0.11727e-02_rb,0.70387e-03_rb /)
      kbo(:, 4,33, 5) = (/ &
        & 0.91138e-04_rb,0.10266e-02_rb,0.12749e-02_rb,0.12630e-02_rb,0.74507e-03_rb /)
      kbo(:, 5,33, 5) = (/ &
        & 0.10640e-03_rb,0.11189e-02_rb,0.13738e-02_rb,0.13608e-02_rb,0.78830e-03_rb /)
      kbo(:, 1,34, 5) = (/ &
        & 0.46667e-04_rb,0.65884e-03_rb,0.86049e-03_rb,0.87214e-03_rb,0.53099e-03_rb /)
      kbo(:, 2,34, 5) = (/ &
        & 0.56263e-04_rb,0.73490e-03_rb,0.93900e-03_rb,0.93946e-03_rb,0.56278e-03_rb /)
      kbo(:, 3,34, 5) = (/ &
        & 0.66853e-04_rb,0.81383e-03_rb,0.10209e-02_rb,0.10142e-02_rb,0.59776e-03_rb /)
      kbo(:, 4,34, 5) = (/ &
        & 0.78598e-04_rb,0.89314e-03_rb,0.11054e-02_rb,0.10963e-02_rb,0.63419e-03_rb /)
      kbo(:, 5,34, 5) = (/ &
        & 0.91503e-04_rb,0.97193e-03_rb,0.11927e-02_rb,0.11840e-02_rb,0.67320e-03_rb /)
      kbo(:, 1,35, 5) = (/ &
        & 0.39599e-04_rb,0.56866e-03_rb,0.73803e-03_rb,0.74491e-03_rb,0.44688e-03_rb /)
      kbo(:, 2,35, 5) = (/ &
        & 0.47677e-04_rb,0.63490e-03_rb,0.80685e-03_rb,0.80540e-03_rb,0.47466e-03_rb /)
      kbo(:, 3,35, 5) = (/ &
        & 0.56630e-04_rb,0.70279e-03_rb,0.87860e-03_rb,0.87332e-03_rb,0.50476e-03_rb /)
      kbo(:, 4,35, 5) = (/ &
        & 0.66506e-04_rb,0.77061e-03_rb,0.95262e-03_rb,0.94676e-03_rb,0.53730e-03_rb /)
      kbo(:, 5,35, 5) = (/ &
        & 0.77343e-04_rb,0.83870e-03_rb,0.10295e-02_rb,0.10250e-02_rb,0.57200e-03_rb /)
      kbo(:, 1,36, 5) = (/ &
        & 0.32767e-04_rb,0.48523e-03_rb,0.62751e-03_rb,0.63183e-03_rb,0.37353e-03_rb /)
      kbo(:, 2,36, 5) = (/ &
        & 0.39450e-04_rb,0.54261e-03_rb,0.68744e-03_rb,0.68587e-03_rb,0.39744e-03_rb /)
      kbo(:, 3,36, 5) = (/ &
        & 0.46861e-04_rb,0.60093e-03_rb,0.74977e-03_rb,0.74623e-03_rb,0.42315e-03_rb /)
      kbo(:, 4,36, 5) = (/ &
        & 0.54956e-04_rb,0.65916e-03_rb,0.81469e-03_rb,0.81158e-03_rb,0.45144e-03_rb /)
      kbo(:, 5,36, 5) = (/ &
        & 0.63908e-04_rb,0.71820e-03_rb,0.88212e-03_rb,0.88103e-03_rb,0.48165e-03_rb /)
      kbo(:, 1,37, 5) = (/ &
        & 0.26734e-04_rb,0.40754e-03_rb,0.52645e-03_rb,0.53014e-03_rb,0.31118e-03_rb /)
      kbo(:, 2,37, 5) = (/ &
        & 0.32273e-04_rb,0.45661e-03_rb,0.57825e-03_rb,0.57766e-03_rb,0.33206e-03_rb /)
      kbo(:, 3,37, 5) = (/ &
        & 0.38488e-04_rb,0.50697e-03_rb,0.63245e-03_rb,0.63058e-03_rb,0.35476e-03_rb /)
      kbo(:, 4,37, 5) = (/ &
        & 0.45228e-04_rb,0.55706e-03_rb,0.68918e-03_rb,0.68809e-03_rb,0.37944e-03_rb /)
      kbo(:, 5,37, 5) = (/ &
        & 0.52675e-04_rb,0.60808e-03_rb,0.74794e-03_rb,0.74934e-03_rb,0.40598e-03_rb /)
      kbo(:, 1,38, 5) = (/ &
        & 0.21751e-04_rb,0.34209e-03_rb,0.44146e-03_rb,0.44458e-03_rb,0.25920e-03_rb /)
      kbo(:, 2,38, 5) = (/ &
        & 0.26326e-04_rb,0.38415e-03_rb,0.48603e-03_rb,0.48632e-03_rb,0.27730e-03_rb /)
      kbo(:, 3,38, 5) = (/ &
        & 0.31515e-04_rb,0.42736e-03_rb,0.53325e-03_rb,0.53262e-03_rb,0.29714e-03_rb /)
      kbo(:, 4,38, 5) = (/ &
        & 0.37219e-04_rb,0.47064e-03_rb,0.58263e-03_rb,0.58321e-03_rb,0.31890e-03_rb /)
      kbo(:, 5,38, 5) = (/ &
        & 0.43344e-04_rb,0.51493e-03_rb,0.63402e-03_rb,0.63729e-03_rb,0.34212e-03_rb /)
      kbo(:, 1,39, 5) = (/ &
        & 0.17643e-04_rb,0.28717e-03_rb,0.37020e-03_rb,0.37288e-03_rb,0.21580e-03_rb /)
      kbo(:, 2,39, 5) = (/ &
        & 0.21418e-04_rb,0.32343e-03_rb,0.40871e-03_rb,0.40953e-03_rb,0.23151e-03_rb /)
      kbo(:, 3,39, 5) = (/ &
        & 0.25725e-04_rb,0.36044e-03_rb,0.44966e-03_rb,0.45005e-03_rb,0.24880e-03_rb /)
      kbo(:, 4,39, 5) = (/ &
        & 0.30479e-04_rb,0.39785e-03_rb,0.49263e-03_rb,0.49445e-03_rb,0.26783e-03_rb /)
      kbo(:, 5,39, 5) = (/ &
        & 0.35653e-04_rb,0.43646e-03_rb,0.53771e-03_rb,0.54230e-03_rb,0.28822e-03_rb /)
      kbo(:, 1,40, 5) = (/ &
        & 0.14102e-04_rb,0.23814e-03_rb,0.30730e-03_rb,0.30981e-03_rb,0.17864e-03_rb /)
      kbo(:, 2,40, 5) = (/ &
        & 0.17187e-04_rb,0.26934e-03_rb,0.34039e-03_rb,0.34146e-03_rb,0.19234e-03_rb /)
      kbo(:, 3,40, 5) = (/ &
        & 0.20739e-04_rb,0.30130e-03_rb,0.37571e-03_rb,0.37672e-03_rb,0.20737e-03_rb /)
      kbo(:, 4,40, 5) = (/ &
        & 0.24670e-04_rb,0.33333e-03_rb,0.41283e-03_rb,0.41538e-03_rb,0.22408e-03_rb /)
      kbo(:, 5,40, 5) = (/ &
        & 0.29046e-04_rb,0.36694e-03_rb,0.45216e-03_rb,0.45729e-03_rb,0.24187e-03_rb /)
      kbo(:, 1,41, 5) = (/ &
        & 0.11221e-04_rb,0.19710e-03_rb,0.25465e-03_rb,0.25696e-03_rb,0.14773e-03_rb /)
      kbo(:, 2,41, 5) = (/ &
        & 0.13747e-04_rb,0.22390e-03_rb,0.28305e-03_rb,0.28413e-03_rb,0.15955e-03_rb /)
      kbo(:, 3,41, 5) = (/ &
        & 0.16649e-04_rb,0.25145e-03_rb,0.31346e-03_rb,0.31467e-03_rb,0.17269e-03_rb /)
      kbo(:, 4,41, 5) = (/ &
        & 0.19899e-04_rb,0.27935e-03_rb,0.34563e-03_rb,0.34845e-03_rb,0.18718e-03_rb /)
      kbo(:, 5,41, 5) = (/ &
        & 0.23568e-04_rb,0.30830e-03_rb,0.37974e-03_rb,0.38512e-03_rb,0.20281e-03_rb /)
      kbo(:, 1,42, 5) = (/ &
        & 0.89147e-05_rb,0.16314e-03_rb,0.21088e-03_rb,0.21294e-03_rb,0.12205e-03_rb /)
      kbo(:, 2,42, 5) = (/ &
        & 0.10957e-04_rb,0.18596e-03_rb,0.23523e-03_rb,0.23628e-03_rb,0.13225e-03_rb /)
      kbo(:, 3,42, 5) = (/ &
        & 0.13327e-04_rb,0.20974e-03_rb,0.26134e-03_rb,0.26263e-03_rb,0.14356e-03_rb /)
      kbo(:, 4,42, 5) = (/ &
        & 0.16005e-04_rb,0.23378e-03_rb,0.28925e-03_rb,0.29202e-03_rb,0.15621e-03_rb /)
      kbo(:, 5,42, 5) = (/ &
        & 0.19038e-04_rb,0.25914e-03_rb,0.31897e-03_rb,0.32415e-03_rb,0.16982e-03_rb /)
      kbo(:, 1,43, 5) = (/ &
        & 0.69995e-05_rb,0.13405e-03_rb,0.17338e-03_rb,0.17535e-03_rb,0.10036e-03_rb /)
      kbo(:, 2,43, 5) = (/ &
        & 0.86406e-05_rb,0.15345e-03_rb,0.19416e-03_rb,0.19514e-03_rb,0.10910e-03_rb /)
      kbo(:, 3,43, 5) = (/ &
        & 0.10556e-04_rb,0.17373e-03_rb,0.21659e-03_rb,0.21767e-03_rb,0.11887e-03_rb /)
      kbo(:, 4,43, 5) = (/ &
        & 0.12739e-04_rb,0.19445e-03_rb,0.24053e-03_rb,0.24311e-03_rb,0.12980e-03_rb /)
      kbo(:, 5,43, 5) = (/ &
        & 0.15232e-04_rb,0.21640e-03_rb,0.26645e-03_rb,0.27107e-03_rb,0.14169e-03_rb /)
      kbo(:, 1,44, 5) = (/ &
        & 0.54712e-05_rb,0.10970e-03_rb,0.14195e-03_rb,0.14386e-03_rb,0.82260e-04_rb /)
      kbo(:, 2,44, 5) = (/ &
        & 0.67644e-05_rb,0.12612e-03_rb,0.15963e-03_rb,0.16061e-03_rb,0.89727e-04_rb /)
      kbo(:, 3,44, 5) = (/ &
        & 0.82973e-05_rb,0.14348e-03_rb,0.17878e-03_rb,0.17975e-03_rb,0.98122e-04_rb /)
      kbo(:, 4,44, 5) = (/ &
        & 0.10082e-04_rb,0.16132e-03_rb,0.19952e-03_rb,0.20155e-03_rb,0.10755e-03_rb /)
      kbo(:, 5,44, 5) = (/ &
        & 0.12107e-04_rb,0.18008e-03_rb,0.22179e-03_rb,0.22576e-03_rb,0.11787e-03_rb /)
      kbo(:, 1,45, 5) = (/ &
        & 0.42767e-05_rb,0.89680e-04_rb,0.11611e-03_rb,0.11781e-03_rb,0.67338e-04_rb /)
      kbo(:, 2,45, 5) = (/ &
        & 0.52860e-05_rb,0.10360e-03_rb,0.13107e-03_rb,0.13203e-03_rb,0.73722e-04_rb /)
      kbo(:, 3,45, 5) = (/ &
        & 0.65107e-05_rb,0.11838e-03_rb,0.14735e-03_rb,0.14829e-03_rb,0.80852e-04_rb /)
      kbo(:, 4,45, 5) = (/ &
        & 0.79501e-05_rb,0.13374e-03_rb,0.16520e-03_rb,0.16693e-03_rb,0.88969e-04_rb /)
      kbo(:, 5,45, 5) = (/ &
        & 0.95937e-05_rb,0.14993e-03_rb,0.18456e-03_rb,0.18781e-03_rb,0.97897e-04_rb /)
      kbo(:, 1,46, 5) = (/ &
        & 0.33226e-05_rb,0.72918e-04_rb,0.94635e-04_rb,0.96069e-04_rb,0.54912e-04_rb /)
      kbo(:, 2,46, 5) = (/ &
        & 0.41185e-05_rb,0.84705e-04_rb,0.10726e-03_rb,0.10809e-03_rb,0.60325e-04_rb /)
      kbo(:, 3,46, 5) = (/ &
        & 0.50796e-05_rb,0.97414e-04_rb,0.12106e-03_rb,0.12184e-03_rb,0.66415e-04_rb /)
      kbo(:, 4,46, 5) = (/ &
        & 0.62311e-05_rb,0.11055e-03_rb,0.13625e-03_rb,0.13772e-03_rb,0.73354e-04_rb /)
      kbo(:, 5,46, 5) = (/ &
        & 0.75594e-05_rb,0.12439e-03_rb,0.15297e-03_rb,0.15569e-03_rb,0.80972e-04_rb /)
      kbo(:, 1,47, 5) = (/ &
        & 0.25507e-05_rb,0.58753e-04_rb,0.76599e-04_rb,0.77798e-04_rb,0.44508e-04_rb /)
      kbo(:, 2,47, 5) = (/ &
        & 0.31800e-05_rb,0.68679e-04_rb,0.87240e-04_rb,0.87843e-04_rb,0.49115e-04_rb /)
      kbo(:, 3,47, 5) = (/ &
        & 0.39273e-05_rb,0.79497e-04_rb,0.98871e-04_rb,0.99421e-04_rb,0.54249e-04_rb /)
      kbo(:, 4,47, 5) = (/ &
        & 0.48346e-05_rb,0.90782e-04_rb,0.11173e-03_rb,0.11282e-03_rb,0.60113e-04_rb /)
      kbo(:, 5,47, 5) = (/ &
        & 0.58995e-05_rb,0.10267e-03_rb,0.12592e-03_rb,0.12815e-03_rb,0.66656e-04_rb /)
      kbo(:, 1,48, 5) = (/ &
        & 0.19539e-05_rb,0.47174e-04_rb,0.61874e-04_rb,0.62916e-04_rb,0.35973e-04_rb /)
      kbo(:, 2,48, 5) = (/ &
        & 0.24452e-05_rb,0.55575e-04_rb,0.70853e-04_rb,0.71238e-04_rb,0.39898e-04_rb /)
      kbo(:, 3,48, 5) = (/ &
        & 0.30402e-05_rb,0.64783e-04_rb,0.80633e-04_rb,0.80921e-04_rb,0.44214e-04_rb /)
      kbo(:, 4,48, 5) = (/ &
        & 0.37390e-05_rb,0.74439e-04_rb,0.91498e-04_rb,0.92208e-04_rb,0.49162e-04_rb /)
      kbo(:, 5,48, 5) = (/ &
        & 0.45836e-05_rb,0.84608e-04_rb,0.10358e-03_rb,0.10531e-03_rb,0.54702e-04_rb /)
      kbo(:, 1,49, 5) = (/ &
        & 0.14940e-05_rb,0.37716e-04_rb,0.49829e-04_rb,0.50804e-04_rb,0.28987e-04_rb /)
      kbo(:, 2,49, 5) = (/ &
        & 0.18758e-05_rb,0.44777e-04_rb,0.57392e-04_rb,0.57697e-04_rb,0.32317e-04_rb /)
      kbo(:, 3,49, 5) = (/ &
        & 0.23413e-05_rb,0.52604e-04_rb,0.65608e-04_rb,0.65748e-04_rb,0.35946e-04_rb /)
      kbo(:, 4,49, 5) = (/ &
        & 0.28961e-05_rb,0.60891e-04_rb,0.74824e-04_rb,0.75182e-04_rb,0.40099e-04_rb /)
      kbo(:, 5,49, 5) = (/ &
        & 0.35524e-05_rb,0.69587e-04_rb,0.85068e-04_rb,0.86295e-04_rb,0.44804e-04_rb /)
      kbo(:, 1,50, 5) = (/ &
        & 0.11500e-05_rb,0.30169e-04_rb,0.40157e-04_rb,0.41069e-04_rb,0.23362e-04_rb /)
      kbo(:, 2,50, 5) = (/ &
        & 0.14453e-05_rb,0.36089e-04_rb,0.46549e-04_rb,0.46799e-04_rb,0.26193e-04_rb /)
      kbo(:, 3,50, 5) = (/ &
        & 0.18129e-05_rb,0.42735e-04_rb,0.53474e-04_rb,0.53514e-04_rb,0.29249e-04_rb /)
      kbo(:, 4,50, 5) = (/ &
        & 0.22529e-05_rb,0.49843e-04_rb,0.61234e-04_rb,0.61404e-04_rb,0.32748e-04_rb /)
      kbo(:, 5,50, 5) = (/ &
        & 0.27762e-05_rb,0.57328e-04_rb,0.70027e-04_rb,0.70783e-04_rb,0.36712e-04_rb /)
      kbo(:, 1,51, 5) = (/ &
        & 0.88851e-06_rb,0.24095e-04_rb,0.32303e-04_rb,0.33210e-04_rb,0.18805e-04_rb /)
      kbo(:, 2,51, 5) = (/ &
        & 0.11177e-05_rb,0.29082e-04_rb,0.37705e-04_rb,0.37944e-04_rb,0.21209e-04_rb /)
      kbo(:, 3,51, 5) = (/ &
        & 0.14060e-05_rb,0.34667e-04_rb,0.43578e-04_rb,0.43570e-04_rb,0.23792e-04_rb /)
      kbo(:, 4,51, 5) = (/ &
        & 0.17539e-05_rb,0.40739e-04_rb,0.50114e-04_rb,0.50172e-04_rb,0.26720e-04_rb /)
      kbo(:, 5,51, 5) = (/ &
        & 0.21729e-05_rb,0.47224e-04_rb,0.57598e-04_rb,0.58070e-04_rb,0.30079e-04_rb /)
      kbo(:, 1,52, 5) = (/ &
        & 0.68525e-06_rb,0.19178e-04_rb,0.25896e-04_rb,0.26809e-04_rb,0.15097e-04_rb /)
      kbo(:, 2,52, 5) = (/ &
        & 0.86301e-06_rb,0.23336e-04_rb,0.30434e-04_rb,0.30713e-04_rb,0.17120e-04_rb /)
      kbo(:, 3,52, 5) = (/ &
        & 0.10889e-05_rb,0.28028e-04_rb,0.35422e-04_rb,0.35397e-04_rb,0.19301e-04_rb /)
      kbo(:, 4,52, 5) = (/ &
        & 0.13649e-05_rb,0.33187e-04_rb,0.40954e-04_rb,0.40921e-04_rb,0.21752e-04_rb /)
      kbo(:, 5,52, 5) = (/ &
        & 0.16982e-05_rb,0.38747e-04_rb,0.47260e-04_rb,0.47557e-04_rb,0.24588e-04_rb /)
      kbo(:, 1,53, 5) = (/ &
        & 0.52809e-06_rb,0.15206e-04_rb,0.20688e-04_rb,0.21592e-04_rb,0.12084e-04_rb /)
      kbo(:, 2,53, 5) = (/ &
        & 0.66493e-06_rb,0.18647e-04_rb,0.24479e-04_rb,0.24819e-04_rb,0.13781e-04_rb /)
      kbo(:, 3,53, 5) = (/ &
        & 0.84102e-06_rb,0.22574e-04_rb,0.28683e-04_rb,0.28690e-04_rb,0.15621e-04_rb /)
      kbo(:, 4,53, 5) = (/ &
        & 0.10585e-05_rb,0.26955e-04_rb,0.33364e-04_rb,0.33298e-04_rb,0.17677e-04_rb /)
      kbo(:, 5,53, 5) = (/ &
        & 0.13239e-05_rb,0.31710e-04_rb,0.38733e-04_rb,0.38864e-04_rb,0.20049e-04_rb /)
      kbo(:, 1,54, 5) = (/ &
        & 0.40969e-06_rb,0.12093e-04_rb,0.16564e-04_rb,0.17421e-04_rb,0.96882e-05_rb /)
      kbo(:, 2,54, 5) = (/ &
        & 0.51569e-06_rb,0.14937e-04_rb,0.19734e-04_rb,0.20094e-04_rb,0.11112e-04_rb /)
      kbo(:, 3,54, 5) = (/ &
        & 0.65296e-06_rb,0.18229e-04_rb,0.23282e-04_rb,0.23321e-04_rb,0.12664e-04_rb /)
      kbo(:, 4,54, 5) = (/ &
        & 0.82573e-06_rb,0.21950e-04_rb,0.27231e-04_rb,0.27150e-04_rb,0.14393e-04_rb /)
      kbo(:, 5,54, 5) = (/ &
        & 0.10373e-05_rb,0.26013e-04_rb,0.31791e-04_rb,0.31828e-04_rb,0.16379e-04_rb /)
      kbo(:, 1,55, 5) = (/ &
        & 0.31928e-06_rb,0.96162e-05_rb,0.13257e-04_rb,0.14053e-04_rb,0.77741e-05_rb /)
      kbo(:, 2,55, 5) = (/ &
        & 0.40112e-06_rb,0.11963e-04_rb,0.15906e-04_rb,0.16257e-04_rb,0.89578e-05_rb /)
      kbo(:, 3,55, 5) = (/ &
        & 0.50816e-06_rb,0.14717e-04_rb,0.18886e-04_rb,0.18947e-04_rb,0.10262e-04_rb /)
      kbo(:, 4,55, 5) = (/ &
        & 0.64465e-06_rb,0.17864e-04_rb,0.22207e-04_rb,0.22155e-04_rb,0.11715e-04_rb /)
      kbo(:, 5,55, 5) = (/ &
        & 0.81328e-06_rb,0.21326e-04_rb,0.26076e-04_rb,0.26061e-04_rb,0.13376e-04_rb /)
      kbo(:, 1,56, 5) = (/ &
        & 0.24829e-06_rb,0.76204e-05_rb,0.10578e-04_rb,0.11309e-04_rb,0.62208e-05_rb /)
      kbo(:, 2,56, 5) = (/ &
        & 0.31176e-06_rb,0.95488e-05_rb,0.12780e-04_rb,0.13125e-04_rb,0.72013e-05_rb /)
      kbo(:, 3,56, 5) = (/ &
        & 0.39506e-06_rb,0.11850e-04_rb,0.15279e-04_rb,0.15348e-04_rb,0.82947e-05_rb /)
      kbo(:, 4,56, 5) = (/ &
        & 0.50266e-06_rb,0.14492e-04_rb,0.18074e-04_rb,0.18040e-04_rb,0.95153e-05_rb /)
      kbo(:, 5,56, 5) = (/ &
        & 0.63638e-06_rb,0.17442e-04_rb,0.21336e-04_rb,0.21310e-04_rb,0.10904e-04_rb /)
      kbo(:, 1,57, 5) = (/ &
        & 0.19302e-06_rb,0.60187e-05_rb,0.84162e-05_rb,0.90799e-05_rb,0.49649e-05_rb /)
      kbo(:, 2,57, 5) = (/ &
        & 0.24222e-06_rb,0.75959e-05_rb,0.10233e-04_rb,0.10570e-04_rb,0.57701e-05_rb /)
      kbo(:, 3,57, 5) = (/ &
        & 0.30688e-06_rb,0.95080e-05_rb,0.12318e-04_rb,0.12403e-04_rb,0.66913e-05_rb /)
      kbo(:, 4,57, 5) = (/ &
        & 0.39089e-06_rb,0.11719e-04_rb,0.14670e-04_rb,0.14644e-04_rb,0.77166e-05_rb /)
      kbo(:, 5,57, 5) = (/ &
        & 0.49674e-06_rb,0.14214e-04_rb,0.17402e-04_rb,0.17382e-04_rb,0.88727e-05_rb /)
      kbo(:, 1,58, 5) = (/ &
        & 0.15046e-06_rb,0.47663e-05_rb,0.66998e-05_rb,0.72908e-05_rb,0.39631e-05_rb /)
      kbo(:, 2,58, 5) = (/ &
        & 0.18856e-06_rb,0.60459e-05_rb,0.81908e-05_rb,0.85206e-05_rb,0.46270e-05_rb /)
      kbo(:, 3,58, 5) = (/ &
        & 0.23869e-06_rb,0.76302e-05_rb,0.99245e-05_rb,0.10023e-04_rb,0.53977e-05_rb /)
      kbo(:, 4,58, 5) = (/ &
        & 0.30459e-06_rb,0.94767e-05_rb,0.11911e-04_rb,0.11894e-04_rb,0.62579e-05_rb /)
      kbo(:, 5,58, 5) = (/ &
        & 0.38820e-06_rb,0.11588e-04_rb,0.14205e-04_rb,0.14182e-04_rb,0.72226e-05_rb /)
      kbo(:, 1,59, 5) = (/ &
        & 0.12212e-06_rb,0.39332e-05_rb,0.55230e-05_rb,0.60094e-05_rb,0.32499e-05_rb /)
      kbo(:, 2,59, 5) = (/ &
        & 0.15324e-06_rb,0.50178e-05_rb,0.67879e-05_rb,0.70566e-05_rb,0.38108e-05_rb /)
      kbo(:, 3,59, 5) = (/ &
        & 0.19418e-06_rb,0.63678e-05_rb,0.82647e-05_rb,0.83443e-05_rb,0.44672e-05_rb /)
      kbo(:, 4,59, 5) = (/ &
        & 0.24820e-06_rb,0.79543e-05_rb,0.99670e-05_rb,0.99518e-05_rb,0.52027e-05_rb /)
      kbo(:, 5,59, 5) = (/ &
        & 0.31740e-06_rb,0.97704e-05_rb,0.11964e-04_rb,0.11938e-04_rb,0.60278e-05_rb /)
      kbo(:, 1,13, 6) = (/ &
        & 0.10896e-01_rb,0.63076e-01_rb,0.78831e-01_rb,0.78091e-01_rb,0.55428e-01_rb /)
      kbo(:, 2,13, 6) = (/ &
        & 0.13036e-01_rb,0.66370e-01_rb,0.81226e-01_rb,0.79692e-01_rb,0.56778e-01_rb /)
      kbo(:, 3,13, 6) = (/ &
        & 0.15498e-01_rb,0.69685e-01_rb,0.83483e-01_rb,0.81400e-01_rb,0.57975e-01_rb /)
      kbo(:, 4,13, 6) = (/ &
        & 0.18239e-01_rb,0.73147e-01_rb,0.85845e-01_rb,0.83249e-01_rb,0.59127e-01_rb /)
      kbo(:, 5,13, 6) = (/ &
        & 0.21232e-01_rb,0.76581e-01_rb,0.88266e-01_rb,0.85245e-01_rb,0.60192e-01_rb /)
      kbo(:, 1,14, 6) = (/ &
        & 0.94274e-02_rb,0.53526e-01_rb,0.66496e-01_rb,0.65527e-01_rb,0.46374e-01_rb /)
      kbo(:, 2,14, 6) = (/ &
        & 0.11284e-01_rb,0.56390e-01_rb,0.68527e-01_rb,0.66955e-01_rb,0.47524e-01_rb /)
      kbo(:, 3,14, 6) = (/ &
        & 0.13394e-01_rb,0.59305e-01_rb,0.70546e-01_rb,0.68479e-01_rb,0.48560e-01_rb /)
      kbo(:, 4,14, 6) = (/ &
        & 0.15751e-01_rb,0.62293e-01_rb,0.72634e-01_rb,0.70165e-01_rb,0.49587e-01_rb /)
      kbo(:, 5,14, 6) = (/ &
        & 0.18326e-01_rb,0.65240e-01_rb,0.74773e-01_rb,0.71967e-01_rb,0.50492e-01_rb /)
      kbo(:, 1,15, 6) = (/ &
        & 0.81352e-02_rb,0.45294e-01_rb,0.55939e-01_rb,0.54933e-01_rb,0.38471e-01_rb /)
      kbo(:, 2,15, 6) = (/ &
        & 0.97406e-02_rb,0.47797e-01_rb,0.57705e-01_rb,0.56201e-01_rb,0.39475e-01_rb /)
      kbo(:, 3,15, 6) = (/ &
        & 0.11542e-01_rb,0.50349e-01_rb,0.59500e-01_rb,0.57588e-01_rb,0.40391e-01_rb /)
      kbo(:, 4,15, 6) = (/ &
        & 0.13568e-01_rb,0.52924e-01_rb,0.61342e-01_rb,0.59116e-01_rb,0.41250e-01_rb /)
      kbo(:, 5,15, 6) = (/ &
        & 0.15789e-01_rb,0.55427e-01_rb,0.63250e-01_rb,0.60727e-01_rb,0.42016e-01_rb /)
      kbo(:, 1,16, 6) = (/ &
        & 0.70159e-02_rb,0.38272e-01_rb,0.46984e-01_rb,0.46002e-01_rb,0.31818e-01_rb /)
      kbo(:, 2,16, 6) = (/ &
        & 0.83846e-02_rb,0.40476e-01_rb,0.48519e-01_rb,0.47148e-01_rb,0.32640e-01_rb /)
      kbo(:, 3,16, 6) = (/ &
        & 0.99392e-02_rb,0.42699e-01_rb,0.50118e-01_rb,0.48421e-01_rb,0.33435e-01_rb /)
      kbo(:, 4,16, 6) = (/ &
        & 0.11675e-01_rb,0.44875e-01_rb,0.51760e-01_rb,0.49780e-01_rb,0.34161e-01_rb /)
      kbo(:, 5,16, 6) = (/ &
        & 0.13584e-01_rb,0.46992e-01_rb,0.53405e-01_rb,0.51224e-01_rb,0.34855e-01_rb /)
      kbo(:, 1,17, 6) = (/ &
        & 0.60402e-02_rb,0.32343e-01_rb,0.39419e-01_rb,0.38482e-01_rb,0.26323e-01_rb /)
      kbo(:, 2,17, 6) = (/ &
        & 0.72134e-02_rb,0.34265e-01_rb,0.40783e-01_rb,0.39523e-01_rb,0.27045e-01_rb /)
      kbo(:, 3,17, 6) = (/ &
        & 0.85415e-02_rb,0.36157e-01_rb,0.42200e-01_rb,0.40657e-01_rb,0.27733e-01_rb /)
      kbo(:, 4,17, 6) = (/ &
        & 0.10023e-01_rb,0.37979e-01_rb,0.43620e-01_rb,0.41877e-01_rb,0.28354e-01_rb /)
      kbo(:, 5,17, 6) = (/ &
        & 0.11662e-01_rb,0.39775e-01_rb,0.45034e-01_rb,0.43161e-01_rb,0.28989e-01_rb /)
      kbo(:, 1,18, 6) = (/ &
        & 0.51905e-02_rb,0.27334e-01_rb,0.33081e-01_rb,0.32183e-01_rb,0.21845e-01_rb /)
      kbo(:, 2,18, 6) = (/ &
        & 0.61907e-02_rb,0.28996e-01_rb,0.34295e-01_rb,0.33127e-01_rb,0.22456e-01_rb /)
      kbo(:, 3,18, 6) = (/ &
        & 0.73283e-02_rb,0.30572e-01_rb,0.35522e-01_rb,0.34131e-01_rb,0.23047e-01_rb /)
      kbo(:, 4,18, 6) = (/ &
        & 0.85960e-02_rb,0.32113e-01_rb,0.36725e-01_rb,0.35202e-01_rb,0.23589e-01_rb /)
      kbo(:, 5,18, 6) = (/ &
        & 0.10011e-01_rb,0.33654e-01_rb,0.37950e-01_rb,0.36339e-01_rb,0.24154e-01_rb /)
      kbo(:, 1,19, 6) = (/ &
        & 0.44474e-02_rb,0.23093e-01_rb,0.27758e-01_rb,0.26913e-01_rb,0.18159e-01_rb /)
      kbo(:, 2,19, 6) = (/ &
        & 0.53019e-02_rb,0.24497e-01_rb,0.28826e-01_rb,0.27764e-01_rb,0.18694e-01_rb /)
      kbo(:, 3,19, 6) = (/ &
        & 0.62735e-02_rb,0.25825e-01_rb,0.29864e-01_rb,0.28669e-01_rb,0.19187e-01_rb /)
      kbo(:, 4,19, 6) = (/ &
        & 0.73683e-02_rb,0.27139e-01_rb,0.30890e-01_rb,0.29627e-01_rb,0.19650e-01_rb /)
      kbo(:, 5,19, 6) = (/ &
        & 0.85933e-02_rb,0.28467e-01_rb,0.31975e-01_rb,0.30634e-01_rb,0.20138e-01_rb /)
      kbo(:, 1,20, 6) = (/ &
        & 0.38215e-02_rb,0.19522e-01_rb,0.23307e-01_rb,0.22532e-01_rb,0.15180e-01_rb /)
      kbo(:, 2,20, 6) = (/ &
        & 0.45489e-02_rb,0.20697e-01_rb,0.24217e-01_rb,0.23292e-01_rb,0.15626e-01_rb /)
      kbo(:, 3,20, 6) = (/ &
        & 0.53838e-02_rb,0.21827e-01_rb,0.25099e-01_rb,0.24107e-01_rb,0.16039e-01_rb /)
      kbo(:, 4,20, 6) = (/ &
        & 0.63294e-02_rb,0.22947e-01_rb,0.25996e-01_rb,0.24951e-01_rb,0.16431e-01_rb /)
      kbo(:, 5,20, 6) = (/ &
        & 0.73910e-02_rb,0.24098e-01_rb,0.26969e-01_rb,0.25840e-01_rb,0.16861e-01_rb /)
      kbo(:, 1,21, 6) = (/ &
        & 0.32849e-02_rb,0.16485e-01_rb,0.19564e-01_rb,0.18873e-01_rb,0.12724e-01_rb /)
      kbo(:, 2,21, 6) = (/ &
        & 0.39093e-02_rb,0.17476e-01_rb,0.20333e-01_rb,0.19553e-01_rb,0.13080e-01_rb /)
      kbo(:, 3,21, 6) = (/ &
        & 0.46257e-02_rb,0.18440e-01_rb,0.21099e-01_rb,0.20272e-01_rb,0.13424e-01_rb /)
      kbo(:, 4,21, 6) = (/ &
        & 0.54436e-02_rb,0.19400e-01_rb,0.21897e-01_rb,0.21018e-01_rb,0.13774e-01_rb /)
      kbo(:, 5,21, 6) = (/ &
        & 0.63568e-02_rb,0.20406e-01_rb,0.22777e-01_rb,0.21806e-01_rb,0.14153e-01_rb /)
      kbo(:, 1,22, 6) = (/ &
        & 0.28524e-02_rb,0.13970e-01_rb,0.16464e-01_rb,0.15855e-01_rb,0.10647e-01_rb /)
      kbo(:, 2,22, 6) = (/ &
        & 0.33998e-02_rb,0.14799e-01_rb,0.17121e-01_rb,0.16464e-01_rb,0.10957e-01_rb /)
      kbo(:, 3,22, 6) = (/ &
        & 0.40218e-02_rb,0.15618e-01_rb,0.17789e-01_rb,0.17098e-01_rb,0.11264e-01_rb /)
      kbo(:, 4,22, 6) = (/ &
        & 0.47350e-02_rb,0.16459e-01_rb,0.18512e-01_rb,0.17761e-01_rb,0.11585e-01_rb /)
      kbo(:, 5,22, 6) = (/ &
        & 0.55269e-02_rb,0.17342e-01_rb,0.19306e-01_rb,0.18466e-01_rb,0.11905e-01_rb /)
      kbo(:, 1,23, 6) = (/ &
        & 0.24776e-02_rb,0.11832e-01_rb,0.13862e-01_rb,0.13338e-01_rb,0.89101e-02_rb /)
      kbo(:, 2,23, 6) = (/ &
        & 0.29594e-02_rb,0.12532e-01_rb,0.14430e-01_rb,0.13874e-01_rb,0.91836e-02_rb /)
      kbo(:, 3,23, 6) = (/ &
        & 0.35060e-02_rb,0.13237e-01_rb,0.15025e-01_rb,0.14435e-01_rb,0.94582e-02_rb /)
      kbo(:, 4,23, 6) = (/ &
        & 0.41276e-02_rb,0.13976e-01_rb,0.15673e-01_rb,0.15028e-01_rb,0.97462e-02_rb /)
      kbo(:, 5,23, 6) = (/ &
        & 0.48108e-02_rb,0.14761e-01_rb,0.16387e-01_rb,0.15666e-01_rb,0.10041e-01_rb /)
      kbo(:, 1,24, 6) = (/ &
        & 0.21553e-02_rb,0.10022e-01_rb,0.11681e-01_rb,0.11233e-01_rb,0.74484e-02_rb /)
      kbo(:, 2,24, 6) = (/ &
        & 0.25764e-02_rb,0.10621e-01_rb,0.12176e-01_rb,0.11709e-01_rb,0.76914e-02_rb /)
      kbo(:, 3,24, 6) = (/ &
        & 0.30587e-02_rb,0.11236e-01_rb,0.12711e-01_rb,0.12208e-01_rb,0.79451e-02_rb /)
      kbo(:, 4,24, 6) = (/ &
        & 0.36022e-02_rb,0.11891e-01_rb,0.13293e-01_rb,0.12744e-01_rb,0.82033e-02_rb /)
      kbo(:, 5,24, 6) = (/ &
        & 0.41914e-02_rb,0.12593e-01_rb,0.13939e-01_rb,0.13324e-01_rb,0.84674e-02_rb /)
      kbo(:, 1,25, 6) = (/ &
        & 0.18796e-02_rb,0.85000e-02_rb,0.98543e-02_rb,0.94788e-02_rb,0.62281e-02_rb /)
      kbo(:, 2,25, 6) = (/ &
        & 0.22512e-02_rb,0.90173e-02_rb,0.10293e-01_rb,0.99015e-02_rb,0.64465e-02_rb /)
      kbo(:, 3,25, 6) = (/ &
        & 0.26761e-02_rb,0.95606e-02_rb,0.10772e-01_rb,0.10351e-01_rb,0.66771e-02_rb /)
      kbo(:, 4,25, 6) = (/ &
        & 0.31512e-02_rb,0.10144e-01_rb,0.11303e-01_rb,0.10838e-01_rb,0.69100e-02_rb /)
      kbo(:, 5,25, 6) = (/ &
        & 0.36575e-02_rb,0.10772e-01_rb,0.11893e-01_rb,0.11363e-01_rb,0.71526e-02_rb /)
      kbo(:, 1,26, 6) = (/ &
        & 0.16486e-02_rb,0.72263e-02_rb,0.83293e-02_rb,0.80207e-02_rb,0.52267e-02_rb /)
      kbo(:, 2,26, 6) = (/ &
        & 0.19790e-02_rb,0.76783e-02_rb,0.87239e-02_rb,0.83966e-02_rb,0.54266e-02_rb /)
      kbo(:, 3,26, 6) = (/ &
        & 0.23552e-02_rb,0.81632e-02_rb,0.91606e-02_rb,0.88030e-02_rb,0.56340e-02_rb /)
      kbo(:, 4,26, 6) = (/ &
        & 0.27655e-02_rb,0.86841e-02_rb,0.96465e-02_rb,0.92463e-02_rb,0.58421e-02_rb /)
      kbo(:, 5,26, 6) = (/ &
        & 0.32014e-02_rb,0.92461e-02_rb,0.10190e-01_rb,0.97263e-02_rb,0.60618e-02_rb /)
      kbo(:, 1,27, 6) = (/ &
        & 0.14479e-02_rb,0.61551e-02_rb,0.70548e-02_rb,0.67980e-02_rb,0.43916e-02_rb /)
      kbo(:, 2,27, 6) = (/ &
        & 0.17422e-02_rb,0.65560e-02_rb,0.74109e-02_rb,0.71369e-02_rb,0.45703e-02_rb /)
      kbo(:, 3,27, 6) = (/ &
        & 0.20711e-02_rb,0.69878e-02_rb,0.78116e-02_rb,0.75081e-02_rb,0.47565e-02_rb /)
      kbo(:, 4,27, 6) = (/ &
        & 0.24273e-02_rb,0.74543e-02_rb,0.82596e-02_rb,0.79125e-02_rb,0.49414e-02_rb /)
      kbo(:, 5,27, 6) = (/ &
        & 0.28006e-02_rb,0.79622e-02_rb,0.87633e-02_rb,0.83523e-02_rb,0.51438e-02_rb /)
      kbo(:, 1,28, 6) = (/ &
        & 0.12757e-02_rb,0.52552e-02_rb,0.59880e-02_rb,0.57731e-02_rb,0.36940e-02_rb /)
      kbo(:, 2,28, 6) = (/ &
        & 0.15355e-02_rb,0.56117e-02_rb,0.63148e-02_rb,0.60828e-02_rb,0.38537e-02_rb /)
      kbo(:, 3,28, 6) = (/ &
        & 0.18209e-02_rb,0.60000e-02_rb,0.66831e-02_rb,0.64236e-02_rb,0.40153e-02_rb /)
      kbo(:, 4,28, 6) = (/ &
        & 0.21272e-02_rb,0.64202e-02_rb,0.70979e-02_rb,0.67923e-02_rb,0.41811e-02_rb /)
      kbo(:, 5,28, 6) = (/ &
        & 0.24474e-02_rb,0.68778e-02_rb,0.75643e-02_rb,0.71983e-02_rb,0.43640e-02_rb /)
      kbo(:, 1,29, 6) = (/ &
        & 0.11295e-02_rb,0.45004e-02_rb,0.50997e-02_rb,0.49184e-02_rb,0.31153e-02_rb /)
      kbo(:, 2,29, 6) = (/ &
        & 0.13559e-02_rb,0.48180e-02_rb,0.54010e-02_rb,0.52034e-02_rb,0.32572e-02_rb /)
      kbo(:, 3,29, 6) = (/ &
        & 0.16033e-02_rb,0.51685e-02_rb,0.57423e-02_rb,0.55151e-02_rb,0.33997e-02_rb /)
      kbo(:, 4,29, 6) = (/ &
        & 0.18647e-02_rb,0.55488e-02_rb,0.61280e-02_rb,0.58545e-02_rb,0.35531e-02_rb /)
      kbo(:, 5,29, 6) = (/ &
        & 0.21368e-02_rb,0.59683e-02_rb,0.65578e-02_rb,0.62309e-02_rb,0.37192e-02_rb /)
      kbo(:, 1,30, 6) = (/ &
        & 0.10010e-02_rb,0.38621e-02_rb,0.43580e-02_rb,0.42036e-02_rb,0.26325e-02_rb /)
      kbo(:, 2,30, 6) = (/ &
        & 0.11967e-02_rb,0.41511e-02_rb,0.46366e-02_rb,0.44656e-02_rb,0.27570e-02_rb /)
      kbo(:, 3,30, 6) = (/ &
        & 0.14096e-02_rb,0.44678e-02_rb,0.49534e-02_rb,0.47515e-02_rb,0.28856e-02_rb /)
      kbo(:, 4,30, 6) = (/ &
        & 0.16325e-02_rb,0.48140e-02_rb,0.53117e-02_rb,0.50685e-02_rb,0.30242e-02_rb /)
      kbo(:, 5,30, 6) = (/ &
        & 0.18599e-02_rb,0.52016e-02_rb,0.57103e-02_rb,0.54169e-02_rb,0.31738e-02_rb /)
      kbo(:, 1,31, 6) = (/ &
        & 0.88685e-03_rb,0.33266e-02_rb,0.37394e-02_rb,0.36064e-02_rb,0.22280e-02_rb /)
      kbo(:, 2,31, 6) = (/ &
        & 0.10569e-02_rb,0.35897e-02_rb,0.39964e-02_rb,0.38465e-02_rb,0.23389e-02_rb /)
      kbo(:, 3,31, 6) = (/ &
        & 0.12382e-02_rb,0.38779e-02_rb,0.42920e-02_rb,0.41135e-02_rb,0.24573e-02_rb /)
      kbo(:, 4,31, 6) = (/ &
        & 0.14278e-02_rb,0.41961e-02_rb,0.46258e-02_rb,0.44080e-02_rb,0.25818e-02_rb /)
      kbo(:, 5,31, 6) = (/ &
        & 0.16136e-02_rb,0.45547e-02_rb,0.50001e-02_rb,0.47313e-02_rb,0.27214e-02_rb /)
      kbo(:, 1,32, 6) = (/ &
        & 0.78559e-03_rb,0.28763e-02_rb,0.32195e-02_rb,0.31059e-02_rb,0.18911e-02_rb /)
      kbo(:, 2,32, 6) = (/ &
        & 0.93178e-03_rb,0.31157e-02_rb,0.34609e-02_rb,0.33287e-02_rb,0.19930e-02_rb /)
      kbo(:, 3,32, 6) = (/ &
        & 0.10867e-02_rb,0.33798e-02_rb,0.37377e-02_rb,0.35772e-02_rb,0.21006e-02_rb /)
      kbo(:, 4,32, 6) = (/ &
        & 0.12446e-02_rb,0.36754e-02_rb,0.40497e-02_rb,0.38517e-02_rb,0.22158e-02_rb /)
      kbo(:, 5,32, 6) = (/ &
        & 0.13936e-02_rb,0.40072e-02_rb,0.44007e-02_rb,0.41538e-02_rb,0.23413e-02_rb /)
      kbo(:, 1,33, 6) = (/ &
        & 0.69486e-03_rb,0.24959e-02_rb,0.27846e-02_rb,0.26858e-02_rb,0.16103e-02_rb /)
      kbo(:, 2,33, 6) = (/ &
        & 0.82037e-03_rb,0.27142e-02_rb,0.30117e-02_rb,0.28933e-02_rb,0.17041e-02_rb /)
      kbo(:, 3,33, 6) = (/ &
        & 0.95137e-03_rb,0.29589e-02_rb,0.32703e-02_rb,0.31258e-02_rb,0.18033e-02_rb /)
      kbo(:, 4,33, 6) = (/ &
        & 0.10800e-02_rb,0.32345e-02_rb,0.35641e-02_rb,0.33810e-02_rb,0.19089e-02_rb /)
      kbo(:, 5,33, 6) = (/ &
        & 0.11986e-02_rb,0.35406e-02_rb,0.38943e-02_rb,0.36682e-02_rb,0.20234e-02_rb /)
      kbo(:, 1,34, 6) = (/ &
        & 0.61032e-03_rb,0.21671e-02_rb,0.24134e-02_rb,0.23268e-02_rb,0.13754e-02_rb /)
      kbo(:, 2,34, 6) = (/ &
        & 0.71748e-03_rb,0.23678e-02_rb,0.26264e-02_rb,0.25207e-02_rb,0.14624e-02_rb /)
      kbo(:, 3,34, 6) = (/ &
        & 0.82600e-03_rb,0.25947e-02_rb,0.28697e-02_rb,0.27368e-02_rb,0.15523e-02_rb /)
      kbo(:, 4,34, 6) = (/ &
        & 0.92939e-03_rb,0.28495e-02_rb,0.31449e-02_rb,0.29773e-02_rb,0.16488e-02_rb /)
      kbo(:, 5,34, 6) = (/ &
        & 0.10239e-02_rb,0.31299e-02_rb,0.34552e-02_rb,0.32520e-02_rb,0.17543e-02_rb /)
      kbo(:, 1,35, 6) = (/ &
        & 0.52678e-03_rb,0.18713e-02_rb,0.20838e-02_rb,0.20082e-02_rb,0.11717e-02_rb /)
      kbo(:, 2,35, 6) = (/ &
        & 0.61674e-03_rb,0.20541e-02_rb,0.22811e-02_rb,0.21873e-02_rb,0.12500e-02_rb /)
      kbo(:, 3,35, 6) = (/ &
        & 0.70570e-03_rb,0.22625e-02_rb,0.25066e-02_rb,0.23870e-02_rb,0.13326e-02_rb /)
      kbo(:, 4,35, 6) = (/ &
        & 0.78855e-03_rb,0.24971e-02_rb,0.27629e-02_rb,0.26134e-02_rb,0.14204e-02_rb /)
      kbo(:, 5,35, 6) = (/ &
        & 0.86492e-03_rb,0.27499e-02_rb,0.30514e-02_rb,0.28766e-02_rb,0.15154e-02_rb /)
      kbo(:, 1,36, 6) = (/ &
        & 0.44569e-03_rb,0.16031e-02_rb,0.17868e-02_rb,0.17221e-02_rb,0.99240e-03_rb /)
      kbo(:, 2,36, 6) = (/ &
        & 0.52055e-03_rb,0.17679e-02_rb,0.19674e-02_rb,0.18854e-02_rb,0.10623e-02_rb /)
      kbo(:, 3,36, 6) = (/ &
        & 0.59380e-03_rb,0.19567e-02_rb,0.21742e-02_rb,0.20700e-02_rb,0.11368e-02_rb /)
      kbo(:, 4,36, 6) = (/ &
        & 0.66114e-03_rb,0.21690e-02_rb,0.24093e-02_rb,0.22827e-02_rb,0.12171e-02_rb /)
      kbo(:, 5,36, 6) = (/ &
        & 0.72347e-03_rb,0.23953e-02_rb,0.26735e-02_rb,0.25294e-02_rb,0.13028e-02_rb /)
      kbo(:, 1,37, 6) = (/ &
        & 0.36881e-03_rb,0.13558e-02_rb,0.15140e-02_rb,0.14605e-02_rb,0.83527e-03_rb /)
      kbo(:, 2,37, 6) = (/ &
        & 0.43070e-03_rb,0.15018e-02_rb,0.16759e-02_rb,0.16073e-02_rb,0.89749e-03_rb /)
      kbo(:, 3,37, 6) = (/ &
        & 0.49122e-03_rb,0.16697e-02_rb,0.18620e-02_rb,0.17747e-02_rb,0.96428e-03_rb /)
      kbo(:, 4,37, 6) = (/ &
        & 0.54687e-03_rb,0.18590e-02_rb,0.20742e-02_rb,0.19690e-02_rb,0.10365e-02_rb /)
      kbo(:, 5,37, 6) = (/ &
        & 0.59826e-03_rb,0.20615e-02_rb,0.23143e-02_rb,0.21948e-02_rb,0.11140e-02_rb /)
      kbo(:, 1,38, 6) = (/ &
        & 0.30472e-03_rb,0.11463e-02_rb,0.12827e-02_rb,0.12387e-02_rb,0.70293e-03_rb /)
      kbo(:, 2,38, 6) = (/ &
        & 0.35606e-03_rb,0.12755e-02_rb,0.14277e-02_rb,0.13705e-02_rb,0.75844e-03_rb /)
      kbo(:, 3,38, 6) = (/ &
        & 0.40587e-03_rb,0.14248e-02_rb,0.15948e-02_rb,0.15225e-02_rb,0.81832e-03_rb /)
      kbo(:, 4,38, 6) = (/ &
        & 0.45170e-03_rb,0.15929e-02_rb,0.17869e-02_rb,0.16996e-02_rb,0.88272e-03_rb /)
      kbo(:, 5,38, 6) = (/ &
        & 0.49447e-03_rb,0.17743e-02_rb,0.20049e-02_rb,0.19060e-02_rb,0.95296e-03_rb /)
      kbo(:, 1,39, 6) = (/ &
        & 0.25159e-03_rb,0.96943e-03_rb,0.10877e-02_rb,0.10513e-02_rb,0.59159e-03_rb /)
      kbo(:, 2,39, 6) = (/ &
        & 0.29433e-03_rb,0.10839e-02_rb,0.12173e-02_rb,0.11700e-02_rb,0.64088e-03_rb /)
      kbo(:, 3,39, 6) = (/ &
        & 0.33554e-03_rb,0.12164e-02_rb,0.13675e-02_rb,0.13079e-02_rb,0.69454e-03_rb /)
      kbo(:, 4,39, 6) = (/ &
        & 0.37315e-03_rb,0.13660e-02_rb,0.15416e-02_rb,0.14697e-02_rb,0.75241e-03_rb /)
      kbo(:, 5,39, 6) = (/ &
        & 0.40840e-03_rb,0.15291e-02_rb,0.17396e-02_rb,0.16583e-02_rb,0.81566e-03_rb /)
      kbo(:, 1,40, 6) = (/ &
        & 0.20445e-03_rb,0.81104e-03_rb,0.91201e-03_rb,0.88303e-03_rb,0.49462e-03_rb /)
      kbo(:, 2,40, 6) = (/ &
        & 0.24013e-03_rb,0.91062e-03_rb,0.10259e-02_rb,0.98834e-03_rb,0.53806e-03_rb /)
      kbo(:, 3,40, 6) = (/ &
        & 0.27454e-03_rb,0.10267e-02_rb,0.11591e-02_rb,0.11112e-02_rb,0.58582e-03_rb /)
      kbo(:, 4,40, 6) = (/ &
        & 0.30609e-03_rb,0.11586e-02_rb,0.13148e-02_rb,0.12564e-02_rb,0.63733e-03_rb /)
      kbo(:, 5,40, 6) = (/ &
        & 0.33521e-03_rb,0.13038e-02_rb,0.14932e-02_rb,0.14267e-02_rb,0.69406e-03_rb /)
      kbo(:, 1,41, 6) = (/ &
        & 0.16568e-03_rb,0.67706e-03_rb,0.76339e-03_rb,0.74014e-03_rb,0.41311e-03_rb /)
      kbo(:, 2,41, 6) = (/ &
        & 0.19545e-03_rb,0.76370e-03_rb,0.86317e-03_rb,0.83349e-03_rb,0.45126e-03_rb /)
      kbo(:, 3,41, 6) = (/ &
        & 0.22429e-03_rb,0.86513e-03_rb,0.98057e-03_rb,0.94284e-03_rb,0.49336e-03_rb /)
      kbo(:, 4,41, 6) = (/ &
        & 0.25075e-03_rb,0.98100e-03_rb,0.11192e-02_rb,0.10723e-02_rb,0.53916e-03_rb /)
      kbo(:, 5,41, 6) = (/ &
        & 0.27523e-03_rb,0.11101e-02_rb,0.12797e-02_rb,0.12259e-02_rb,0.58980e-03_rb /)
      kbo(:, 1,42, 6) = (/ &
        & 0.13464e-03_rb,0.56478e-03_rb,0.63846e-03_rb,0.61972e-03_rb,0.34485e-03_rb /)
      kbo(:, 2,42, 6) = (/ &
        & 0.15918e-03_rb,0.64002e-03_rb,0.72573e-03_rb,0.70212e-03_rb,0.37807e-03_rb /)
      kbo(:, 3,42, 6) = (/ &
        & 0.18309e-03_rb,0.72866e-03_rb,0.82914e-03_rb,0.79959e-03_rb,0.41541e-03_rb /)
      kbo(:, 4,42, 6) = (/ &
        & 0.20534e-03_rb,0.83063e-03_rb,0.95224e-03_rb,0.91510e-03_rb,0.45609e-03_rb /)
      kbo(:, 5,42, 6) = (/ &
        & 0.22591e-03_rb,0.94536e-03_rb,0.10960e-02_rb,0.10535e-02_rb,0.50132e-03_rb /)
      kbo(:, 1,43, 6) = (/ &
        & 0.10834e-03_rb,0.46763e-03_rb,0.52964e-03_rb,0.51479e-03_rb,0.28641e-03_rb /)
      kbo(:, 2,43, 6) = (/ &
        & 0.12878e-03_rb,0.53191e-03_rb,0.60529e-03_rb,0.58655e-03_rb,0.31538e-03_rb /)
      kbo(:, 3,43, 6) = (/ &
        & 0.14877e-03_rb,0.60848e-03_rb,0.69517e-03_rb,0.67233e-03_rb,0.34781e-03_rb /)
      kbo(:, 4,43, 6) = (/ &
        & 0.16739e-03_rb,0.69818e-03_rb,0.80315e-03_rb,0.77433e-03_rb,0.38376e-03_rb /)
      kbo(:, 5,43, 6) = (/ &
        & 0.18458e-03_rb,0.79951e-03_rb,0.93071e-03_rb,0.89755e-03_rb,0.42371e-03_rb /)
      kbo(:, 1,44, 6) = (/ &
        & 0.86770e-04_rb,0.38551e-03_rb,0.43712e-03_rb,0.42544e-03_rb,0.23710e-03_rb /)
      kbo(:, 2,44, 6) = (/ &
        & 0.10382e-03_rb,0.44040e-03_rb,0.50222e-03_rb,0.48753e-03_rb,0.26235e-03_rb /)
      kbo(:, 3,44, 6) = (/ &
        & 0.12059e-03_rb,0.50571e-03_rb,0.58013e-03_rb,0.56233e-03_rb,0.29036e-03_rb /)
      kbo(:, 4,44, 6) = (/ &
        & 0.13632e-03_rb,0.58391e-03_rb,0.67429e-03_rb,0.65221e-03_rb,0.32192e-03_rb /)
      kbo(:, 5,44, 6) = (/ &
        & 0.15073e-03_rb,0.67339e-03_rb,0.78624e-03_rb,0.76086e-03_rb,0.35732e-03_rb /)
      kbo(:, 1,45, 6) = (/ &
        & 0.69317e-04_rb,0.31724e-03_rb,0.36001e-03_rb,0.35090e-03_rb,0.19579e-03_rb /)
      kbo(:, 2,45, 6) = (/ &
        & 0.83569e-04_rb,0.36417e-03_rb,0.41579e-03_rb,0.40435e-03_rb,0.21781e-03_rb /)
      kbo(:, 3,45, 6) = (/ &
        & 0.97725e-04_rb,0.42014e-03_rb,0.48329e-03_rb,0.46938e-03_rb,0.24226e-03_rb /)
      kbo(:, 4,45, 6) = (/ &
        & 0.11101e-03_rb,0.48745e-03_rb,0.56520e-03_rb,0.54804e-03_rb,0.26965e-03_rb /)
      kbo(:, 5,45, 6) = (/ &
        & 0.12310e-03_rb,0.56606e-03_rb,0.66327e-03_rb,0.64374e-03_rb,0.30080e-03_rb /)
      kbo(:, 1,46, 6) = (/ &
        & 0.55030e-04_rb,0.26007e-03_rb,0.29522e-03_rb,0.28798e-03_rb,0.16098e-03_rb /)
      kbo(:, 2,46, 6) = (/ &
        & 0.66908e-04_rb,0.29981e-03_rb,0.34235e-03_rb,0.33361e-03_rb,0.18009e-03_rb /)
      kbo(:, 3,46, 6) = (/ &
        & 0.78894e-04_rb,0.34737e-03_rb,0.40028e-03_rb,0.38969e-03_rb,0.20138e-03_rb /)
      kbo(:, 4,46, 6) = (/ &
        & 0.90228e-04_rb,0.40507e-03_rb,0.47110e-03_rb,0.45796e-03_rb,0.22525e-03_rb /)
      kbo(:, 5,46, 6) = (/ &
        & 0.10044e-03_rb,0.47380e-03_rb,0.55673e-03_rb,0.54149e-03_rb,0.25231e-03_rb /)
      kbo(:, 1,47, 6) = (/ &
        & 0.43131e-04_rb,0.21132e-03_rb,0.23992e-03_rb,0.23430e-03_rb,0.13147e-03_rb /)
      kbo(:, 2,47, 6) = (/ &
        & 0.53026e-04_rb,0.24489e-03_rb,0.27966e-03_rb,0.27282e-03_rb,0.14794e-03_rb /)
      kbo(:, 3,47, 6) = (/ &
        & 0.63179e-04_rb,0.28502e-03_rb,0.32837e-03_rb,0.32046e-03_rb,0.16642e-03_rb /)
      kbo(:, 4,47, 6) = (/ &
        & 0.72849e-04_rb,0.33391e-03_rb,0.38868e-03_rb,0.37909e-03_rb,0.18727e-03_rb /)
      kbo(:, 5,47, 6) = (/ &
        & 0.81641e-04_rb,0.39308e-03_rb,0.46293e-03_rb,0.45114e-03_rb,0.21071e-03_rb /)
      kbo(:, 1,48, 6) = (/ &
        & 0.33560e-04_rb,0.17139e-03_rb,0.19468e-03_rb,0.19001e-03_rb,0.10710e-03_rb /)
      kbo(:, 2,48, 6) = (/ &
        & 0.41736e-04_rb,0.19942e-03_rb,0.22765e-03_rb,0.22235e-03_rb,0.12121e-03_rb /)
      kbo(:, 3,48, 6) = (/ &
        & 0.50371e-04_rb,0.23319e-03_rb,0.26893e-03_rb,0.26280e-03_rb,0.13718e-03_rb /)
      kbo(:, 4,48, 6) = (/ &
        & 0.58779e-04_rb,0.27466e-03_rb,0.31973e-03_rb,0.31295e-03_rb,0.15543e-03_rb /)
      kbo(:, 5,48, 6) = (/ &
        & 0.66247e-04_rb,0.32531e-03_rb,0.38347e-03_rb,0.37465e-03_rb,0.17582e-03_rb /)
      kbo(:, 1,49, 6) = (/ &
        & 0.25958e-04_rb,0.13895e-03_rb,0.15751e-03_rb,0.15378e-03_rb,0.86997e-04_rb /)
      kbo(:, 2,49, 6) = (/ &
        & 0.32663e-04_rb,0.16210e-03_rb,0.18496e-03_rb,0.18058e-03_rb,0.99003e-04_rb /)
      kbo(:, 3,49, 6) = (/ &
        & 0.39912e-04_rb,0.19033e-03_rb,0.21954e-03_rb,0.21463e-03_rb,0.11278e-03_rb /)
      kbo(:, 4,49, 6) = (/ &
        & 0.47146e-04_rb,0.22536e-03_rb,0.26252e-03_rb,0.25743e-03_rb,0.12854e-03_rb /)
      kbo(:, 5,49, 6) = (/ &
        & 0.53714e-04_rb,0.26879e-03_rb,0.31652e-03_rb,0.31040e-03_rb,0.14638e-03_rb /)
      kbo(:, 1,50, 6) = (/ &
        & 0.20123e-04_rb,0.11283e-03_rb,0.12763e-03_rb,0.12462e-03_rb,0.70722e-04_rb /)
      kbo(:, 2,50, 6) = (/ &
        & 0.25601e-04_rb,0.13215e-03_rb,0.15042e-03_rb,0.14697e-03_rb,0.80931e-04_rb /)
      kbo(:, 3,50, 6) = (/ &
        & 0.31614e-04_rb,0.15574e-03_rb,0.17957e-03_rb,0.17552e-03_rb,0.92734e-04_rb /)
      kbo(:, 4,50, 6) = (/ &
        & 0.37764e-04_rb,0.18520e-03_rb,0.21596e-03_rb,0.21200e-03_rb,0.10641e-03_rb /)
      kbo(:, 5,50, 6) = (/ &
        & 0.43463e-04_rb,0.22248e-03_rb,0.26195e-03_rb,0.25758e-03_rb,0.12203e-03_rb /)
      kbo(:, 1,51, 6) = (/ &
        & 0.15566e-04_rb,0.91692e-04_rb,0.10348e-03_rb,0.10093e-03_rb,0.57420e-04_rb /)
      kbo(:, 2,51, 6) = (/ &
        & 0.20041e-04_rb,0.10771e-03_rb,0.12230e-03_rb,0.11952e-03_rb,0.66019e-04_rb /)
      kbo(:, 3,51, 6) = (/ &
        & 0.25022e-04_rb,0.12757e-03_rb,0.14670e-03_rb,0.14349e-03_rb,0.76182e-04_rb /)
      kbo(:, 4,51, 6) = (/ &
        & 0.30196e-04_rb,0.15226e-03_rb,0.17772e-03_rb,0.17429e-03_rb,0.87954e-04_rb /)
      kbo(:, 5,51, 6) = (/ &
        & 0.35140e-04_rb,0.18405e-03_rb,0.21697e-03_rb,0.21356e-03_rb,0.10162e-03_rb /)
      kbo(:, 1,52, 6) = (/ &
        & 0.11977e-04_rb,0.74321e-04_rb,0.83777e-04_rb,0.81525e-04_rb,0.46492e-04_rb /)
      kbo(:, 2,52, 6) = (/ &
        & 0.15618e-04_rb,0.87662e-04_rb,0.99315e-04_rb,0.96995e-04_rb,0.53744e-04_rb /)
      kbo(:, 3,52, 6) = (/ &
        & 0.19731e-04_rb,0.10434e-03_rb,0.11957e-03_rb,0.11696e-03_rb,0.62355e-04_rb /)
      kbo(:, 4,52, 6) = (/ &
        & 0.24071e-04_rb,0.12514e-03_rb,0.14574e-03_rb,0.14300e-03_rb,0.72491e-04_rb /)
      kbo(:, 5,52, 6) = (/ &
        & 0.28292e-04_rb,0.15185e-03_rb,0.17919e-03_rb,0.17642e-03_rb,0.84356e-04_rb /)
      kbo(:, 1,53, 6) = (/ &
        & 0.91836e-05_rb,0.60131e-04_rb,0.67720e-04_rb,0.65649e-04_rb,0.37522e-04_rb /)
      kbo(:, 2,53, 6) = (/ &
        & 0.12113e-04_rb,0.71192e-04_rb,0.80457e-04_rb,0.78425e-04_rb,0.43599e-04_rb /)
      kbo(:, 3,53, 6) = (/ &
        & 0.15471e-04_rb,0.85141e-04_rb,0.97276e-04_rb,0.95110e-04_rb,0.50903e-04_rb /)
      kbo(:, 4,53, 6) = (/ &
        & 0.19102e-04_rb,0.10263e-03_rb,0.11918e-03_rb,0.11694e-03_rb,0.59553e-04_rb /)
      kbo(:, 5,53, 6) = (/ &
        & 0.22693e-04_rb,0.12514e-03_rb,0.14755e-03_rb,0.14527e-03_rb,0.69812e-04_rb /)
      kbo(:, 1,54, 6) = (/ &
        & 0.70721e-05_rb,0.48764e-04_rb,0.54926e-04_rb,0.53041e-04_rb,0.30337e-04_rb /)
      kbo(:, 2,54, 6) = (/ &
        & 0.94270e-05_rb,0.57982e-04_rb,0.65461e-04_rb,0.63579e-04_rb,0.35439e-04_rb /)
      kbo(:, 3,54, 6) = (/ &
        & 0.12173e-04_rb,0.69671e-04_rb,0.79433e-04_rb,0.77530e-04_rb,0.41618e-04_rb /)
      kbo(:, 4,54, 6) = (/ &
        & 0.15195e-04_rb,0.84502e-04_rb,0.97826e-04_rb,0.96029e-04_rb,0.49017e-04_rb /)
      kbo(:, 5,54, 6) = (/ &
        & 0.18243e-04_rb,0.10358e-03_rb,0.12196e-03_rb,0.12023e-03_rb,0.57880e-04_rb /)
      kbo(:, 1,55, 6) = (/ &
        & 0.54549e-05_rb,0.39521e-04_rb,0.44580e-04_rb,0.42901e-04_rb,0.24523e-04_rb /)
      kbo(:, 2,55, 6) = (/ &
        & 0.73474e-05_rb,0.47258e-04_rb,0.53311e-04_rb,0.51627e-04_rb,0.28799e-04_rb /)
      kbo(:, 3,55, 6) = (/ &
        & 0.95818e-05_rb,0.57041e-04_rb,0.64957e-04_rb,0.63210e-04_rb,0.34015e-04_rb /)
      kbo(:, 4,55, 6) = (/ &
        & 0.12078e-04_rb,0.69528e-04_rb,0.80422e-04_rb,0.78845e-04_rb,0.40325e-04_rb /)
      kbo(:, 5,55, 6) = (/ &
        & 0.14658e-04_rb,0.85891e-04_rb,0.10087e-03_rb,0.99485e-04_rb,0.47973e-04_rb /)
      kbo(:, 1,56, 6) = (/ &
        & 0.41972e-05_rb,0.31935e-04_rb,0.36125e-04_rb,0.34628e-04_rb,0.19758e-04_rb /)
      kbo(:, 2,56, 6) = (/ &
        & 0.56951e-05_rb,0.38357e-04_rb,0.43303e-04_rb,0.41823e-04_rb,0.23341e-04_rb /)
      kbo(:, 3,56, 6) = (/ &
        & 0.75109e-05_rb,0.46586e-04_rb,0.52984e-04_rb,0.51438e-04_rb,0.27705e-04_rb /)
      kbo(:, 4,56, 6) = (/ &
        & 0.95701e-05_rb,0.57135e-04_rb,0.66020e-04_rb,0.64513e-04_rb,0.33090e-04_rb /)
      kbo(:, 5,56, 6) = (/ &
        & 0.11741e-04_rb,0.70996e-04_rb,0.83348e-04_rb,0.82172e-04_rb,0.39639e-04_rb /)
      kbo(:, 1,57, 6) = (/ &
        & 0.32187e-05_rb,0.25710e-04_rb,0.29175e-04_rb,0.27903e-04_rb,0.15874e-04_rb /)
      kbo(:, 2,57, 6) = (/ &
        & 0.44067e-05_rb,0.31037e-04_rb,0.35094e-04_rb,0.33812e-04_rb,0.18857e-04_rb /)
      kbo(:, 3,57, 6) = (/ &
        & 0.58663e-05_rb,0.37889e-04_rb,0.43100e-04_rb,0.41766e-04_rb,0.22502e-04_rb /)
      kbo(:, 4,57, 6) = (/ &
        & 0.75519e-05_rb,0.46817e-04_rb,0.54053e-04_rb,0.52634e-04_rb,0.27030e-04_rb /)
      kbo(:, 5,57, 6) = (/ &
        & 0.93649e-05_rb,0.58529e-04_rb,0.68705e-04_rb,0.67601e-04_rb,0.32661e-04_rb /)
      kbo(:, 1,58, 6) = (/ &
        & 0.24696e-05_rb,0.20698e-04_rb,0.23574e-04_rb,0.22507e-04_rb,0.12764e-04_rb /)
      kbo(:, 2,58, 6) = (/ &
        & 0.34175e-05_rb,0.25141e-04_rb,0.28438e-04_rb,0.27351e-04_rb,0.15243e-04_rb /)
      kbo(:, 3,58, 6) = (/ &
        & 0.45825e-05_rb,0.30855e-04_rb,0.35092e-04_rb,0.33951e-04_rb,0.18275e-04_rb /)
      kbo(:, 4,58, 6) = (/ &
        & 0.59665e-05_rb,0.38392e-04_rb,0.44239e-04_rb,0.43055e-04_rb,0.22112e-04_rb /)
      kbo(:, 5,58, 6) = (/ &
        & 0.74784e-05_rb,0.48383e-04_rb,0.56772e-04_rb,0.55657e-04_rb,0.26900e-04_rb /)
      kbo(:, 1,59, 6) = (/ &
        & 0.20113e-05_rb,0.17223e-04_rb,0.19628e-04_rb,0.18772e-04_rb,0.10576e-04_rb /)
      kbo(:, 2,59, 6) = (/ &
        & 0.27973e-05_rb,0.21075e-04_rb,0.23849e-04_rb,0.22960e-04_rb,0.12702e-04_rb /)
      kbo(:, 3,59, 6) = (/ &
        & 0.37714e-05_rb,0.26064e-04_rb,0.29696e-04_rb,0.28726e-04_rb,0.15326e-04_rb /)
      kbo(:, 4,59, 6) = (/ &
        & 0.49189e-05_rb,0.32724e-04_rb,0.37808e-04_rb,0.36791e-04_rb,0.18710e-04_rb /)
      kbo(:, 5,59, 6) = (/ &
        & 0.61850e-05_rb,0.41676e-04_rb,0.49043e-04_rb,0.48051e-04_rb,0.22936e-04_rb /)
      kbo(:, 1,13, 7) = (/ &
        & 0.90574e-01_rb,0.18508e+00_rb,0.20125e+00_rb,0.19353e+00_rb,0.13590e+00_rb /)
      kbo(:, 2,13, 7) = (/ &
        & 0.96633e-01_rb,0.18856e+00_rb,0.20514e+00_rb,0.19781e+00_rb,0.13867e+00_rb /)
      kbo(:, 3,13, 7) = (/ &
        & 0.10246e+00_rb,0.19231e+00_rb,0.20916e+00_rb,0.20175e+00_rb,0.14157e+00_rb /)
      kbo(:, 4,13, 7) = (/ &
        & 0.10831e+00_rb,0.19664e+00_rb,0.21340e+00_rb,0.20566e+00_rb,0.14422e+00_rb /)
      kbo(:, 5,13, 7) = (/ &
        & 0.11418e+00_rb,0.20190e+00_rb,0.21829e+00_rb,0.20964e+00_rb,0.14715e+00_rb /)
      kbo(:, 1,14, 7) = (/ &
        & 0.76731e-01_rb,0.15637e+00_rb,0.17031e+00_rb,0.16454e+00_rb,0.11471e+00_rb /)
      kbo(:, 2,14, 7) = (/ &
        & 0.81853e-01_rb,0.15935e+00_rb,0.17367e+00_rb,0.16809e+00_rb,0.11738e+00_rb /)
      kbo(:, 3,14, 7) = (/ &
        & 0.86939e-01_rb,0.16284e+00_rb,0.17719e+00_rb,0.17151e+00_rb,0.11993e+00_rb /)
      kbo(:, 4,14, 7) = (/ &
        & 0.91985e-01_rb,0.16698e+00_rb,0.18115e+00_rb,0.17500e+00_rb,0.12254e+00_rb /)
      kbo(:, 5,14, 7) = (/ &
        & 0.97005e-01_rb,0.17199e+00_rb,0.18590e+00_rb,0.17860e+00_rb,0.12524e+00_rb /)
      kbo(:, 1,15, 7) = (/ &
        & 0.64796e-01_rb,0.13180e+00_rb,0.14379e+00_rb,0.13927e+00_rb,0.96282e-01_rb /)
      kbo(:, 2,15, 7) = (/ &
        & 0.69175e-01_rb,0.13451e+00_rb,0.14669e+00_rb,0.14230e+00_rb,0.98702e-01_rb /)
      kbo(:, 3,15, 7) = (/ &
        & 0.73574e-01_rb,0.13779e+00_rb,0.14991e+00_rb,0.14533e+00_rb,0.10111e+00_rb /)
      kbo(:, 4,15, 7) = (/ &
        & 0.77907e-01_rb,0.14175e+00_rb,0.15375e+00_rb,0.14848e+00_rb,0.10336e+00_rb /)
      kbo(:, 5,15, 7) = (/ &
        & 0.82082e-01_rb,0.14644e+00_rb,0.15826e+00_rb,0.15192e+00_rb,0.10558e+00_rb /)
      kbo(:, 1,16, 7) = (/ &
        & 0.54603e-01_rb,0.11092e+00_rb,0.12113e+00_rb,0.11747e+00_rb,0.80564e-01_rb /)
      kbo(:, 2,16, 7) = (/ &
        & 0.58392e-01_rb,0.11343e+00_rb,0.12376e+00_rb,0.12011e+00_rb,0.82662e-01_rb /)
      kbo(:, 3,16, 7) = (/ &
        & 0.62149e-01_rb,0.11652e+00_rb,0.12680e+00_rb,0.12283e+00_rb,0.84744e-01_rb /)
      kbo(:, 4,16, 7) = (/ &
        & 0.65779e-01_rb,0.12031e+00_rb,0.13049e+00_rb,0.12576e+00_rb,0.86806e-01_rb /)
      kbo(:, 5,16, 7) = (/ &
        & 0.69275e-01_rb,0.12462e+00_rb,0.13474e+00_rb,0.12909e+00_rb,0.88592e-01_rb /)
      kbo(:, 1,17, 7) = (/ &
        & 0.46019e-01_rb,0.93276e-01_rb,0.10182e+00_rb,0.98886e-01_rb,0.67386e-01_rb /)
      kbo(:, 2,17, 7) = (/ &
        & 0.49257e-01_rb,0.95617e-01_rb,0.10427e+00_rb,0.10122e+00_rb,0.69214e-01_rb /)
      kbo(:, 3,17, 7) = (/ &
        & 0.52438e-01_rb,0.98544e-01_rb,0.10723e+00_rb,0.10369e+00_rb,0.71047e-01_rb /)
      kbo(:, 4,17, 7) = (/ &
        & 0.55444e-01_rb,0.10204e+00_rb,0.11074e+00_rb,0.10647e+00_rb,0.72809e-01_rb /)
      kbo(:, 5,17, 7) = (/ &
        & 0.58360e-01_rb,0.10585e+00_rb,0.11469e+00_rb,0.10967e+00_rb,0.74297e-01_rb /)
      kbo(:, 1,18, 7) = (/ &
        & 0.38789e-01_rb,0.78385e-01_rb,0.85426e-01_rb,0.83127e-01_rb,0.56424e-01_rb /)
      kbo(:, 2,18, 7) = (/ &
        & 0.41550e-01_rb,0.80626e-01_rb,0.87775e-01_rb,0.85216e-01_rb,0.58103e-01_rb /)
      kbo(:, 3,18, 7) = (/ &
        & 0.44172e-01_rb,0.83391e-01_rb,0.90633e-01_rb,0.87547e-01_rb,0.59665e-01_rb /)
      kbo(:, 4,18, 7) = (/ &
        & 0.46672e-01_rb,0.86541e-01_rb,0.93910e-01_rb,0.90224e-01_rb,0.61155e-01_rb /)
      kbo(:, 5,18, 7) = (/ &
        & 0.49119e-01_rb,0.89890e-01_rb,0.97478e-01_rb,0.93171e-01_rb,0.62440e-01_rb /)
      kbo(:, 1,19, 7) = (/ &
        & 0.32688e-01_rb,0.65866e-01_rb,0.71699e-01_rb,0.69831e-01_rb,0.47341e-01_rb /)
      kbo(:, 2,19, 7) = (/ &
        & 0.34993e-01_rb,0.67987e-01_rb,0.73938e-01_rb,0.71758e-01_rb,0.48787e-01_rb /)
      kbo(:, 3,19, 7) = (/ &
        & 0.37158e-01_rb,0.70566e-01_rb,0.76608e-01_rb,0.73972e-01_rb,0.50204e-01_rb /)
      kbo(:, 4,19, 7) = (/ &
        & 0.39257e-01_rb,0.73365e-01_rb,0.79594e-01_rb,0.76458e-01_rb,0.51459e-01_rb /)
      kbo(:, 5,19, 7) = (/ &
        & 0.41326e-01_rb,0.76343e-01_rb,0.82757e-01_rb,0.79138e-01_rb,0.52598e-01_rb /)
      kbo(:, 1,20, 7) = (/ &
        & 0.27557e-01_rb,0.55396e-01_rb,0.60366e-01_rb,0.58674e-01_rb,0.39775e-01_rb /)
      kbo(:, 2,20, 7) = (/ &
        & 0.29458e-01_rb,0.57423e-01_rb,0.62478e-01_rb,0.60497e-01_rb,0.41091e-01_rb /)
      kbo(:, 3,20, 7) = (/ &
        & 0.31261e-01_rb,0.59745e-01_rb,0.64927e-01_rb,0.62579e-01_rb,0.42317e-01_rb /)
      kbo(:, 4,20, 7) = (/ &
        & 0.33046e-01_rb,0.62240e-01_rb,0.67549e-01_rb,0.64853e-01_rb,0.43439e-01_rb /)
      kbo(:, 5,20, 7) = (/ &
        & 0.34818e-01_rb,0.64890e-01_rb,0.70320e-01_rb,0.67290e-01_rb,0.44513e-01_rb /)
      kbo(:, 1,21, 7) = (/ &
        & 0.23194e-01_rb,0.46662e-01_rb,0.50879e-01_rb,0.49331e-01_rb,0.33429e-01_rb /)
      kbo(:, 2,21, 7) = (/ &
        & 0.24776e-01_rb,0.48529e-01_rb,0.52871e-01_rb,0.51054e-01_rb,0.34590e-01_rb /)
      kbo(:, 3,21, 7) = (/ &
        & 0.26303e-01_rb,0.50578e-01_rb,0.55098e-01_rb,0.52980e-01_rb,0.35684e-01_rb /)
      kbo(:, 4,21, 7) = (/ &
        & 0.27810e-01_rb,0.52793e-01_rb,0.57429e-01_rb,0.55052e-01_rb,0.36688e-01_rb /)
      kbo(:, 5,21, 7) = (/ &
        & 0.29323e-01_rb,0.55187e-01_rb,0.59853e-01_rb,0.57245e-01_rb,0.37657e-01_rb /)
      kbo(:, 1,22, 7) = (/ &
        & 0.19600e-01_rb,0.39448e-01_rb,0.43057e-01_rb,0.41623e-01_rb,0.28169e-01_rb /)
      kbo(:, 2,22, 7) = (/ &
        & 0.20910e-01_rb,0.41135e-01_rb,0.44924e-01_rb,0.43250e-01_rb,0.29188e-01_rb /)
      kbo(:, 3,22, 7) = (/ &
        & 0.22195e-01_rb,0.42970e-01_rb,0.46916e-01_rb,0.45021e-01_rb,0.30138e-01_rb /)
      kbo(:, 4,22, 7) = (/ &
        & 0.23466e-01_rb,0.44963e-01_rb,0.48978e-01_rb,0.46906e-01_rb,0.31021e-01_rb /)
      kbo(:, 5,22, 7) = (/ &
        & 0.24791e-01_rb,0.47151e-01_rb,0.51143e-01_rb,0.48901e-01_rb,0.31919e-01_rb /)
      kbo(:, 1,23, 7) = (/ &
        & 0.16563e-01_rb,0.33405e-01_rb,0.36517e-01_rb,0.35235e-01_rb,0.23758e-01_rb /)
      kbo(:, 2,23, 7) = (/ &
        & 0.17634e-01_rb,0.34920e-01_rb,0.38215e-01_rb,0.36726e-01_rb,0.24656e-01_rb /)
      kbo(:, 3,23, 7) = (/ &
        & 0.18715e-01_rb,0.36571e-01_rb,0.39976e-01_rb,0.38337e-01_rb,0.25473e-01_rb /)
      kbo(:, 4,23, 7) = (/ &
        & 0.19810e-01_rb,0.38397e-01_rb,0.41828e-01_rb,0.40046e-01_rb,0.26273e-01_rb /)
      kbo(:, 5,23, 7) = (/ &
        & 0.20990e-01_rb,0.40406e-01_rb,0.43840e-01_rb,0.41881e-01_rb,0.27068e-01_rb /)
      kbo(:, 1,24, 7) = (/ &
        & 0.13980e-01_rb,0.28346e-01_rb,0.31027e-01_rb,0.29909e-01_rb,0.20058e-01_rb /)
      kbo(:, 2,24, 7) = (/ &
        & 0.14874e-01_rb,0.29707e-01_rb,0.32546e-01_rb,0.31266e-01_rb,0.20839e-01_rb /)
      kbo(:, 3,24, 7) = (/ &
        & 0.15797e-01_rb,0.31204e-01_rb,0.34123e-01_rb,0.32728e-01_rb,0.21557e-01_rb /)
      kbo(:, 4,24, 7) = (/ &
        & 0.16758e-01_rb,0.32875e-01_rb,0.35824e-01_rb,0.34296e-01_rb,0.22274e-01_rb /)
      kbo(:, 5,24, 7) = (/ &
        & 0.17809e-01_rb,0.34737e-01_rb,0.37711e-01_rb,0.35981e-01_rb,0.22992e-01_rb /)
      kbo(:, 1,25, 7) = (/ &
        & 0.11798e-01_rb,0.24110e-01_rb,0.26426e-01_rb,0.25457e-01_rb,0.16985e-01_rb /)
      kbo(:, 2,25, 7) = (/ &
        & 0.12561e-01_rb,0.25347e-01_rb,0.27781e-01_rb,0.26691e-01_rb,0.17655e-01_rb /)
      kbo(:, 3,25, 7) = (/ &
        & 0.13359e-01_rb,0.26711e-01_rb,0.29222e-01_rb,0.28026e-01_rb,0.18302e-01_rb /)
      kbo(:, 4,25, 7) = (/ &
        & 0.14208e-01_rb,0.28241e-01_rb,0.30806e-01_rb,0.29472e-01_rb,0.18949e-01_rb /)
      kbo(:, 5,25, 7) = (/ &
        & 0.15157e-01_rb,0.29980e-01_rb,0.32597e-01_rb,0.31029e-01_rb,0.19596e-01_rb /)
      kbo(:, 1,26, 7) = (/ &
        & 0.99745e-02_rb,0.20581e-01_rb,0.22580e-01_rb,0.21746e-01_rb,0.14432e-01_rb /)
      kbo(:, 2,26, 7) = (/ &
        & 0.10633e-01_rb,0.21703e-01_rb,0.23807e-01_rb,0.22873e-01_rb,0.15014e-01_rb /)
      kbo(:, 3,26, 7) = (/ &
        & 0.11328e-01_rb,0.22948e-01_rb,0.25141e-01_rb,0.24108e-01_rb,0.15588e-01_rb /)
      kbo(:, 4,26, 7) = (/ &
        & 0.12091e-01_rb,0.24380e-01_rb,0.26636e-01_rb,0.25439e-01_rb,0.16180e-01_rb /)
      kbo(:, 5,26, 7) = (/ &
        & 0.12943e-01_rb,0.26016e-01_rb,0.28335e-01_rb,0.26899e-01_rb,0.16787e-01_rb /)
      kbo(:, 1,27, 7) = (/ &
        & 0.84424e-02_rb,0.17603e-01_rb,0.19348e-01_rb,0.18632e-01_rb,0.12275e-01_rb /)
      kbo(:, 2,27, 7) = (/ &
        & 0.90172e-02_rb,0.18627e-01_rb,0.20475e-01_rb,0.19670e-01_rb,0.12796e-01_rb /)
      kbo(:, 3,27, 7) = (/ &
        & 0.96344e-02_rb,0.19791e-01_rb,0.21724e-01_rb,0.20817e-01_rb,0.13316e-01_rb /)
      kbo(:, 4,27, 7) = (/ &
        & 0.10319e-01_rb,0.21143e-01_rb,0.23141e-01_rb,0.22059e-01_rb,0.13859e-01_rb /)
      kbo(:, 5,27, 7) = (/ &
        & 0.11083e-01_rb,0.22688e-01_rb,0.24742e-01_rb,0.23449e-01_rb,0.14418e-01_rb /)
      kbo(:, 1,28, 7) = (/ &
        & 0.71594e-02_rb,0.15094e-01_rb,0.16638e-01_rb,0.16016e-01_rb,0.10452e-01_rb /)
      kbo(:, 2,28, 7) = (/ &
        & 0.76603e-02_rb,0.16043e-01_rb,0.17678e-01_rb,0.16989e-01_rb,0.10928e-01_rb /)
      kbo(:, 3,28, 7) = (/ &
        & 0.82161e-02_rb,0.17148e-01_rb,0.18856e-01_rb,0.18055e-01_rb,0.11409e-01_rb /)
      kbo(:, 4,28, 7) = (/ &
        & 0.88297e-02_rb,0.18429e-01_rb,0.20193e-01_rb,0.19234e-01_rb,0.11908e-01_rb /)
      kbo(:, 5,28, 7) = (/ &
        & 0.95210e-02_rb,0.19896e-01_rb,0.21697e-01_rb,0.20565e-01_rb,0.12423e-01_rb /)
      kbo(:, 1,29, 7) = (/ &
        & 0.60865e-02_rb,0.12992e-01_rb,0.14367e-01_rb,0.13830e-01_rb,0.89337e-02_rb /)
      kbo(:, 2,29, 7) = (/ &
        & 0.65273e-02_rb,0.13894e-01_rb,0.15342e-01_rb,0.14741e-01_rb,0.93649e-02_rb /)
      kbo(:, 3,29, 7) = (/ &
        & 0.70278e-02_rb,0.14949e-01_rb,0.16457e-01_rb,0.15746e-01_rb,0.98158e-02_rb /)
      kbo(:, 4,29, 7) = (/ &
        & 0.75858e-02_rb,0.16169e-01_rb,0.17720e-01_rb,0.16877e-01_rb,0.10273e-01_rb /)
      kbo(:, 5,29, 7) = (/ &
        & 0.82154e-02_rb,0.17564e-01_rb,0.19150e-01_rb,0.18159e-01_rb,0.10745e-01_rb /)
      kbo(:, 1,30, 7) = (/ &
        & 0.51820e-02_rb,0.11236e-01_rb,0.12459e-01_rb,0.12000e-01_rb,0.76507e-02_rb /)
      kbo(:, 2,30, 7) = (/ &
        & 0.55811e-02_rb,0.12098e-01_rb,0.13380e-01_rb,0.12854e-01_rb,0.80497e-02_rb /)
      kbo(:, 3,30, 7) = (/ &
        & 0.60310e-02_rb,0.13108e-01_rb,0.14440e-01_rb,0.13813e-01_rb,0.84648e-02_rb /)
      kbo(:, 4,30, 7) = (/ &
        & 0.65404e-02_rb,0.14273e-01_rb,0.15644e-01_rb,0.14905e-01_rb,0.88882e-02_rb /)
      kbo(:, 5,30, 7) = (/ &
        & 0.71170e-02_rb,0.15597e-01_rb,0.17012e-01_rb,0.16134e-01_rb,0.93248e-02_rb /)
      kbo(:, 1,31, 7) = (/ &
        & 0.44272e-02_rb,0.97757e-02_rb,0.10859e-01_rb,0.10467e-01_rb,0.65734e-02_rb /)
      kbo(:, 2,31, 7) = (/ &
        & 0.47878e-02_rb,0.10603e-01_rb,0.11740e-01_rb,0.11277e-01_rb,0.69415e-02_rb /)
      kbo(:, 3,31, 7) = (/ &
        & 0.51963e-02_rb,0.11572e-01_rb,0.12753e-01_rb,0.12201e-01_rb,0.73188e-02_rb /)
      kbo(:, 4,31, 7) = (/ &
        & 0.56609e-02_rb,0.12684e-01_rb,0.13904e-01_rb,0.13252e-01_rb,0.77091e-02_rb /)
      kbo(:, 5,31, 7) = (/ &
        & 0.61977e-02_rb,0.13944e-01_rb,0.15244e-01_rb,0.14424e-01_rb,0.81161e-02_rb /)
      kbo(:, 1,32, 7) = (/ &
        & 0.37962e-02_rb,0.85614e-02_rb,0.95233e-02_rb,0.91812e-02_rb,0.56658e-02_rb /)
      kbo(:, 2,32, 7) = (/ &
        & 0.41228e-02_rb,0.93595e-02_rb,0.10368e-01_rb,0.99574e-02_rb,0.60000e-02_rb /)
      kbo(:, 3,32, 7) = (/ &
        & 0.44947e-02_rb,0.10287e-01_rb,0.11338e-01_rb,0.10850e-01_rb,0.63462e-02_rb /)
      kbo(:, 4,32, 7) = (/ &
        & 0.49244e-02_rb,0.11349e-01_rb,0.12467e-01_rb,0.11857e-01_rb,0.67069e-02_rb /)
      kbo(:, 5,32, 7) = (/ &
        & 0.54252e-02_rb,0.12554e-01_rb,0.13767e-01_rb,0.12983e-01_rb,0.70910e-02_rb /)
      kbo(:, 1,33, 7) = (/ &
        & 0.32656e-02_rb,0.75519e-02_rb,0.84026e-02_rb,0.81028e-02_rb,0.48937e-02_rb /)
      kbo(:, 2,33, 7) = (/ &
        & 0.35630e-02_rb,0.83178e-02_rb,0.92157e-02_rb,0.88531e-02_rb,0.52004e-02_rb /)
      kbo(:, 3,33, 7) = (/ &
        & 0.39050e-02_rb,0.92063e-02_rb,0.10168e-01_rb,0.97125e-02_rb,0.55156e-02_rb /)
      kbo(:, 4,33, 7) = (/ &
        & 0.43055e-02_rb,0.10227e-01_rb,0.11270e-01_rb,0.10682e-01_rb,0.58516e-02_rb /)
      kbo(:, 5,33, 7) = (/ &
        & 0.47651e-02_rb,0.11396e-01_rb,0.12524e-01_rb,0.11769e-01_rb,0.62091e-02_rb /)
      kbo(:, 1,34, 7) = (/ &
        & 0.28105e-02_rb,0.66873e-02_rb,0.74405e-02_rb,0.71742e-02_rb,0.42413e-02_rb /)
      kbo(:, 2,34, 7) = (/ &
        & 0.30819e-02_rb,0.74197e-02_rb,0.82328e-02_rb,0.78962e-02_rb,0.45189e-02_rb /)
      kbo(:, 3,34, 7) = (/ &
        & 0.33974e-02_rb,0.82731e-02_rb,0.91582e-02_rb,0.87214e-02_rb,0.48133e-02_rb /)
      kbo(:, 4,34, 7) = (/ &
        & 0.37682e-02_rb,0.92558e-02_rb,0.10225e-01_rb,0.96550e-02_rb,0.51300e-02_rb /)
      kbo(:, 5,34, 7) = (/ &
        & 0.41836e-02_rb,0.10408e-01_rb,0.11429e-01_rb,0.10709e-01_rb,0.54671e-02_rb /)
      kbo(:, 1,35, 7) = (/ &
        & 0.24038e-02_rb,0.58948e-02_rb,0.65709e-02_rb,0.63304e-02_rb,0.36544e-02_rb /)
      kbo(:, 2,35, 7) = (/ &
        & 0.26483e-02_rb,0.65885e-02_rb,0.73295e-02_rb,0.70175e-02_rb,0.39080e-02_rb /)
      kbo(:, 3,35, 7) = (/ &
        & 0.29372e-02_rb,0.74016e-02_rb,0.82197e-02_rb,0.78032e-02_rb,0.41855e-02_rb /)
      kbo(:, 4,35, 7) = (/ &
        & 0.32728e-02_rb,0.83531e-02_rb,0.92365e-02_rb,0.86973e-02_rb,0.44777e-02_rb /)
      kbo(:, 5,35, 7) = (/ &
        & 0.36421e-02_rb,0.94681e-02_rb,0.10387e-01_rb,0.97145e-02_rb,0.47967e-02_rb /)
      kbo(:, 1,36, 7) = (/ &
        & 0.20373e-02_rb,0.51550e-02_rb,0.57662e-02_rb,0.55498e-02_rb,0.31231e-02_rb /)
      kbo(:, 2,36, 7) = (/ &
        & 0.22554e-02_rb,0.58057e-02_rb,0.64818e-02_rb,0.61953e-02_rb,0.33549e-02_rb /)
      kbo(:, 3,36, 7) = (/ &
        & 0.25127e-02_rb,0.65796e-02_rb,0.73228e-02_rb,0.69361e-02_rb,0.36112e-02_rb /)
      kbo(:, 4,36, 7) = (/ &
        & 0.28104e-02_rb,0.74827e-02_rb,0.82833e-02_rb,0.77843e-02_rb,0.38823e-02_rb /)
      kbo(:, 5,36, 7) = (/ &
        & 0.31359e-02_rb,0.85467e-02_rb,0.93763e-02_rb,0.87575e-02_rb,0.41812e-02_rb /)
      kbo(:, 1,37, 7) = (/ &
        & 0.17035e-02_rb,0.44466e-02_rb,0.49856e-02_rb,0.47970e-02_rb,0.26734e-02_rb /)
      kbo(:, 2,37, 7) = (/ &
        & 0.18934e-02_rb,0.50433e-02_rb,0.56462e-02_rb,0.53907e-02_rb,0.28873e-02_rb /)
      kbo(:, 3,37, 7) = (/ &
        & 0.21171e-02_rb,0.57558e-02_rb,0.64223e-02_rb,0.60783e-02_rb,0.31233e-02_rb /)
      kbo(:, 4,37, 7) = (/ &
        & 0.23767e-02_rb,0.65938e-02_rb,0.73141e-02_rb,0.68717e-02_rb,0.33760e-02_rb /)
      kbo(:, 5,37, 7) = (/ &
        & 0.26616e-02_rb,0.75860e-02_rb,0.83359e-02_rb,0.77859e-02_rb,0.36530e-02_rb /)
      kbo(:, 1,38, 7) = (/ &
        & 0.14234e-02_rb,0.38379e-02_rb,0.43139e-02_rb,0.41486e-02_rb,0.22904e-02_rb /)
      kbo(:, 2,38, 7) = (/ &
        & 0.15875e-02_rb,0.43850e-02_rb,0.49236e-02_rb,0.46956e-02_rb,0.24873e-02_rb /)
      kbo(:, 3,38, 7) = (/ &
        & 0.17824e-02_rb,0.50419e-02_rb,0.56400e-02_rb,0.53338e-02_rb,0.27024e-02_rb /)
      kbo(:, 4,38, 7) = (/ &
        & 0.20080e-02_rb,0.58195e-02_rb,0.64682e-02_rb,0.60767e-02_rb,0.29386e-02_rb /)
      kbo(:, 5,38, 7) = (/ &
        & 0.22567e-02_rb,0.67445e-02_rb,0.74253e-02_rb,0.69357e-02_rb,0.31979e-02_rb /)
      kbo(:, 1,39, 7) = (/ &
        & 0.11890e-02_rb,0.33180e-02_rb,0.37396e-02_rb,0.35935e-02_rb,0.19623e-02_rb /)
      kbo(:, 2,39, 7) = (/ &
        & 0.13309e-02_rb,0.38204e-02_rb,0.43017e-02_rb,0.40980e-02_rb,0.21427e-02_rb /)
      kbo(:, 3,39, 7) = (/ &
        & 0.14997e-02_rb,0.44279e-02_rb,0.49653e-02_rb,0.46926e-02_rb,0.23415e-02_rb /)
      kbo(:, 4,39, 7) = (/ &
        & 0.16956e-02_rb,0.51508e-02_rb,0.57361e-02_rb,0.53891e-02_rb,0.25595e-02_rb /)
      kbo(:, 5,39, 7) = (/ &
        & 0.19128e-02_rb,0.60136e-02_rb,0.66353e-02_rb,0.61975e-02_rb,0.28034e-02_rb /)
      kbo(:, 1,40, 7) = (/ &
        & 0.98185e-03_rb,0.28304e-02_rb,0.31989e-02_rb,0.30741e-02_rb,0.16759e-02_rb /)
      kbo(:, 2,40, 7) = (/ &
        & 0.11018e-02_rb,0.32825e-02_rb,0.37073e-02_rb,0.35318e-02_rb,0.18412e-02_rb /)
      kbo(:, 3,40, 7) = (/ &
        & 0.12459e-02_rb,0.38338e-02_rb,0.43125e-02_rb,0.40771e-02_rb,0.20230e-02_rb /)
      kbo(:, 4,40, 7) = (/ &
        & 0.14138e-02_rb,0.44950e-02_rb,0.50204e-02_rb,0.47198e-02_rb,0.22264e-02_rb /)
      kbo(:, 5,40, 7) = (/ &
        & 0.16015e-02_rb,0.52867e-02_rb,0.58524e-02_rb,0.54701e-02_rb,0.24538e-02_rb /)
      kbo(:, 1,41, 7) = (/ &
        & 0.80888e-03_rb,0.24101e-02_rb,0.27302e-02_rb,0.26250e-02_rb,0.14285e-02_rb /)
      kbo(:, 2,41, 7) = (/ &
        & 0.91042e-03_rb,0.28166e-02_rb,0.31895e-02_rb,0.30396e-02_rb,0.15812e-02_rb /)
      kbo(:, 3,41, 7) = (/ &
        & 0.10323e-02_rb,0.33144e-02_rb,0.37392e-02_rb,0.35375e-02_rb,0.17482e-02_rb /)
      kbo(:, 4,41, 7) = (/ &
        & 0.11757e-02_rb,0.39172e-02_rb,0.43885e-02_rb,0.41290e-02_rb,0.19366e-02_rb /)
      kbo(:, 5,41, 7) = (/ &
        & 0.13368e-02_rb,0.46417e-02_rb,0.51571e-02_rb,0.48237e-02_rb,0.21455e-02_rb /)
      kbo(:, 1,42, 7) = (/ &
        & 0.66584e-03_rb,0.20506e-02_rb,0.23294e-02_rb,0.22418e-02_rb,0.12147e-02_rb /)
      kbo(:, 2,42, 7) = (/ &
        & 0.75117e-03_rb,0.24164e-02_rb,0.27427e-02_rb,0.26163e-02_rb,0.13552e-02_rb /)
      kbo(:, 3,42, 7) = (/ &
        & 0.85464e-03_rb,0.28657e-02_rb,0.32419e-02_rb,0.30702e-02_rb,0.15103e-02_rb /)
      kbo(:, 4,42, 7) = (/ &
        & 0.97690e-03_rb,0.34148e-02_rb,0.38374e-02_rb,0.36136e-02_rb,0.16836e-02_rb /)
      kbo(:, 5,42, 7) = (/ &
        & 0.11154e-02_rb,0.40777e-02_rb,0.45486e-02_rb,0.42564e-02_rb,0.18765e-02_rb /)
      kbo(:, 1,43, 7) = (/ &
        & 0.54468e-03_rb,0.17260e-02_rb,0.19662e-02_rb,0.18963e-02_rb,0.10278e-02_rb /)
      kbo(:, 2,43, 7) = (/ &
        & 0.61495e-03_rb,0.20517e-02_rb,0.23339e-02_rb,0.22307e-02_rb,0.11544e-02_rb /)
      kbo(:, 3,43, 7) = (/ &
        & 0.70090e-03_rb,0.24526e-02_rb,0.27821e-02_rb,0.26393e-02_rb,0.12972e-02_rb /)
      kbo(:, 4,43, 7) = (/ &
        & 0.80460e-03_rb,0.29460e-02_rb,0.33228e-02_rb,0.31329e-02_rb,0.14583e-02_rb /)
      kbo(:, 5,43, 7) = (/ &
        & 0.92296e-03_rb,0.35462e-02_rb,0.39738e-02_rb,0.37220e-02_rb,0.16369e-02_rb /)
      kbo(:, 1,44, 7) = (/ &
        & 0.44364e-03_rb,0.14437e-02_rb,0.16496e-02_rb,0.15952e-02_rb,0.86540e-03_rb /)
      kbo(:, 2,44, 7) = (/ &
        & 0.50199e-03_rb,0.17304e-02_rb,0.19739e-02_rb,0.18912e-02_rb,0.97937e-03_rb /)
      kbo(:, 3,44, 7) = (/ &
        & 0.57242e-03_rb,0.20860e-02_rb,0.23731e-02_rb,0.22565e-02_rb,0.11106e-02_rb /)
      kbo(:, 4,44, 7) = (/ &
        & 0.65930e-03_rb,0.25283e-02_rb,0.28613e-02_rb,0.27024e-02_rb,0.12573e-02_rb /)
      kbo(:, 5,44, 7) = (/ &
        & 0.75974e-03_rb,0.30673e-02_rb,0.34531e-02_rb,0.32379e-02_rb,0.14238e-02_rb /)
      kbo(:, 1,45, 7) = (/ &
        & 0.36129e-03_rb,0.12055e-02_rb,0.13811e-02_rb,0.13394e-02_rb,0.72668e-03_rb /)
      kbo(:, 2,45, 7) = (/ &
        & 0.40925e-03_rb,0.14558e-02_rb,0.16660e-02_rb,0.16003e-02_rb,0.82937e-03_rb /)
      kbo(:, 3,45, 7) = (/ &
        & 0.46755e-03_rb,0.17705e-02_rb,0.20201e-02_rb,0.19260e-02_rb,0.94755e-03_rb /)
      kbo(:, 4,45, 7) = (/ &
        & 0.53937e-03_rb,0.21654e-02_rb,0.24593e-02_rb,0.23275e-02_rb,0.10822e-02_rb /)
      kbo(:, 5,45, 7) = (/ &
        & 0.62400e-03_rb,0.26505e-02_rb,0.29965e-02_rb,0.28141e-02_rb,0.12345e-02_rb /)
      kbo(:, 1,46, 7) = (/ &
        & 0.29308e-03_rb,0.99917e-03_rb,0.11483e-02_rb,0.11175e-02_rb,0.60699e-03_rb /)
      kbo(:, 2,46, 7) = (/ &
        & 0.33256e-03_rb,0.12167e-02_rb,0.13963e-02_rb,0.13454e-02_rb,0.69810e-03_rb /)
      kbo(:, 3,46, 7) = (/ &
        & 0.38031e-03_rb,0.14933e-02_rb,0.17091e-02_rb,0.16341e-02_rb,0.80431e-03_rb /)
      kbo(:, 4,46, 7) = (/ &
        & 0.43974e-03_rb,0.18423e-02_rb,0.21003e-02_rb,0.19926e-02_rb,0.92607e-03_rb /)
      kbo(:, 5,46, 7) = (/ &
        & 0.51042e-03_rb,0.22760e-02_rb,0.25842e-02_rb,0.24318e-02_rb,0.10656e-02_rb /)
      kbo(:, 1,47, 7) = (/ &
        & 0.23621e-03_rb,0.81920e-03_rb,0.94415e-03_rb,0.92230e-03_rb,0.50239e-03_rb /)
      kbo(:, 2,47, 7) = (/ &
        & 0.26847e-03_rb,0.10045e-02_rb,0.11560e-02_rb,0.11182e-02_rb,0.58268e-03_rb /)
      kbo(:, 3,47, 7) = (/ &
        & 0.30727e-03_rb,0.12442e-02_rb,0.14284e-02_rb,0.13701e-02_rb,0.67654e-03_rb /)
      kbo(:, 4,47, 7) = (/ &
        & 0.35565e-03_rb,0.15479e-02_rb,0.17727e-02_rb,0.16866e-02_rb,0.78602e-03_rb /)
      kbo(:, 5,47, 7) = (/ &
        & 0.41455e-03_rb,0.19320e-02_rb,0.22031e-02_rb,0.20787e-02_rb,0.91297e-03_rb /)
      kbo(:, 1,48, 7) = (/ &
        & 0.19056e-03_rb,0.66912e-03_rb,0.77329e-03_rb,0.75810e-03_rb,0.41434e-03_rb /)
      kbo(:, 2,48, 7) = (/ &
        & 0.21662e-03_rb,0.82638e-03_rb,0.95404e-03_rb,0.92602e-03_rb,0.48448e-03_rb /)
      kbo(:, 3,48, 7) = (/ &
        & 0.24805e-03_rb,0.10324e-02_rb,0.11883e-02_rb,0.11443e-02_rb,0.56714e-03_rb /)
      kbo(:, 4,48, 7) = (/ &
        & 0.28732e-03_rb,0.12967e-02_rb,0.14909e-02_rb,0.14226e-02_rb,0.66465e-03_rb /)
      kbo(:, 5,48, 7) = (/ &
        & 0.33598e-03_rb,0.16339e-02_rb,0.18720e-02_rb,0.17712e-02_rb,0.77925e-03_rb /)
      kbo(:, 1,49, 7) = (/ &
        & 0.15370e-03_rb,0.54435e-03_rb,0.63072e-03_rb,0.62031e-03_rb,0.34039e-03_rb /)
      kbo(:, 2,49, 7) = (/ &
        & 0.17493e-03_rb,0.67703e-03_rb,0.78385e-03_rb,0.76375e-03_rb,0.40110e-03_rb /)
      kbo(:, 3,49, 7) = (/ &
        & 0.20006e-03_rb,0.85331e-03_rb,0.98454e-03_rb,0.95185e-03_rb,0.47334e-03_rb /)
      kbo(:, 4,49, 7) = (/ &
        & 0.23184e-03_rb,0.10822e-02_rb,0.12482e-02_rb,0.11953e-02_rb,0.56021e-03_rb /)
      kbo(:, 5,49, 7) = (/ &
        & 0.27201e-03_rb,0.13761e-02_rb,0.15848e-02_rb,0.15038e-02_rb,0.66336e-03_rb /)
      kbo(:, 1,50, 7) = (/ &
        & 0.12445e-03_rb,0.44363e-03_rb,0.51558e-03_rb,0.50858e-03_rb,0.27983e-03_rb /)
      kbo(:, 2,50, 7) = (/ &
        & 0.14173e-03_rb,0.55572e-03_rb,0.64516e-03_rb,0.63048e-03_rb,0.33240e-03_rb /)
      kbo(:, 3,50, 7) = (/ &
        & 0.16225e-03_rb,0.70658e-03_rb,0.81789e-03_rb,0.79296e-03_rb,0.39601e-03_rb /)
      kbo(:, 4,50, 7) = (/ &
        & 0.18768e-03_rb,0.90539e-03_rb,0.10468e-02_rb,0.10060e-02_rb,0.47285e-03_rb /)
      kbo(:, 5,50, 7) = (/ &
        & 0.22071e-03_rb,0.11624e-02_rb,0.13446e-02_rb,0.12795e-02_rb,0.56523e-03_rb /)
      kbo(:, 1,51, 7) = (/ &
        & 0.10079e-03_rb,0.36142e-03_rb,0.42089e-03_rb,0.41617e-03_rb,0.23042e-03_rb /)
      kbo(:, 2,51, 7) = (/ &
        & 0.11508e-03_rb,0.45540e-03_rb,0.53023e-03_rb,0.51983e-03_rb,0.27489e-03_rb /)
      kbo(:, 3,51, 7) = (/ &
        & 0.13183e-03_rb,0.58426e-03_rb,0.67873e-03_rb,0.65942e-03_rb,0.33073e-03_rb /)
      kbo(:, 4,51, 7) = (/ &
        & 0.15243e-03_rb,0.75694e-03_rb,0.87652e-03_rb,0.84573e-03_rb,0.39869e-03_rb /)
      kbo(:, 5,51, 7) = (/ &
        & 0.17921e-03_rb,0.98295e-03_rb,0.11402e-02_rb,0.10885e-02_rb,0.48140e-03_rb /)
      kbo(:, 1,52, 7) = (/ &
        & 0.81556e-04_rb,0.29363e-03_rb,0.34244e-03_rb,0.33921e-03_rb,0.18905e-03_rb /)
      kbo(:, 2,52, 7) = (/ &
        & 0.93358e-04_rb,0.37204e-03_rb,0.43415e-03_rb,0.42707e-03_rb,0.22730e-03_rb /)
      kbo(:, 3,52, 7) = (/ &
        & 0.10711e-03_rb,0.48125e-03_rb,0.56095e-03_rb,0.54625e-03_rb,0.27517e-03_rb /)
      kbo(:, 4,52, 7) = (/ &
        & 0.12382e-03_rb,0.62999e-03_rb,0.73195e-03_rb,0.70815e-03_rb,0.33506e-03_rb /)
      kbo(:, 5,52, 7) = (/ &
        & 0.14564e-03_rb,0.82786e-03_rb,0.96258e-03_rb,0.92197e-03_rb,0.40847e-03_rb /)
      kbo(:, 1,53, 7) = (/ &
        & 0.65849e-04_rb,0.23779e-03_rb,0.27734e-03_rb,0.27519e-03_rb,0.15433e-03_rb /)
      kbo(:, 2,53, 7) = (/ &
        & 0.75546e-04_rb,0.30303e-03_rb,0.35410e-03_rb,0.34952e-03_rb,0.18741e-03_rb /)
      kbo(:, 3,53, 7) = (/ &
        & 0.86880e-04_rb,0.39464e-03_rb,0.46133e-03_rb,0.45049e-03_rb,0.22844e-03_rb /)
      kbo(:, 4,53, 7) = (/ &
        & 0.10061e-03_rb,0.52205e-03_rb,0.60866e-03_rb,0.59002e-03_rb,0.28045e-03_rb /)
      kbo(:, 5,53, 7) = (/ &
        & 0.11837e-03_rb,0.69424e-03_rb,0.80911e-03_rb,0.77763e-03_rb,0.34549e-03_rb /)
      kbo(:, 1,54, 7) = (/ &
        & 0.53194e-04_rb,0.19358e-03_rb,0.22554e-03_rb,0.22386e-03_rb,0.12634e-03_rb /)
      kbo(:, 2,54, 7) = (/ &
        & 0.61356e-04_rb,0.24774e-03_rb,0.28991e-03_rb,0.28698e-03_rb,0.15485e-03_rb /)
      kbo(:, 3,54, 7) = (/ &
        & 0.70732e-04_rb,0.32546e-03_rb,0.38104e-03_rb,0.37325e-03_rb,0.19067e-03_rb /)
      kbo(:, 4,54, 7) = (/ &
        & 0.81989e-04_rb,0.43475e-03_rb,0.50842e-03_rb,0.49348e-03_rb,0.23573e-03_rb /)
      kbo(:, 5,54, 7) = (/ &
        & 0.96606e-04_rb,0.58491e-03_rb,0.68358e-03_rb,0.65848e-03_rb,0.29331e-03_rb /)
      kbo(:, 1,55, 7) = (/ &
        & 0.42968e-04_rb,0.15774e-03_rb,0.18351e-03_rb,0.18209e-03_rb,0.10325e-03_rb /)
      kbo(:, 2,55, 7) = (/ &
        & 0.49757e-04_rb,0.20300e-03_rb,0.23752e-03_rb,0.23563e-03_rb,0.12786e-03_rb /)
      kbo(:, 3,55, 7) = (/ &
        & 0.57602e-04_rb,0.26853e-03_rb,0.31476e-03_rb,0.30938e-03_rb,0.15908e-03_rb /)
      kbo(:, 4,55, 7) = (/ &
        & 0.66944e-04_rb,0.36215e-03_rb,0.42458e-03_rb,0.41285e-03_rb,0.19867e-03_rb /)
      kbo(:, 5,55, 7) = (/ &
        & 0.78909e-04_rb,0.49332e-03_rb,0.57794e-03_rb,0.55775e-03_rb,0.24929e-03_rb /)
      kbo(:, 1,56, 7) = (/ &
        & 0.34572e-04_rb,0.12819e-03_rb,0.14868e-03_rb,0.14747e-03_rb,0.84103e-04_rb /)
      kbo(:, 2,56, 7) = (/ &
        & 0.40234e-04_rb,0.16586e-03_rb,0.19409e-03_rb,0.19266e-03_rb,0.10511e-03_rb /)
      kbo(:, 3,56, 7) = (/ &
        & 0.46756e-04_rb,0.22079e-03_rb,0.25905e-03_rb,0.25536e-03_rb,0.13228e-03_rb /)
      kbo(:, 4,56, 7) = (/ &
        & 0.54583e-04_rb,0.30103e-03_rb,0.35315e-03_rb,0.34421e-03_rb,0.16708e-03_rb /)
      kbo(:, 5,56, 7) = (/ &
        & 0.64414e-04_rb,0.41471e-03_rb,0.48701e-03_rb,0.47045e-03_rb,0.21139e-03_rb /)
      kbo(:, 1,57, 7) = (/ &
        & 0.27768e-04_rb,0.10391e-03_rb,0.12029e-03_rb,0.11888e-03_rb,0.68178e-04_rb /)
      kbo(:, 2,57, 7) = (/ &
        & 0.32463e-04_rb,0.13502e-03_rb,0.15785e-03_rb,0.15684e-03_rb,0.86073e-04_rb /)
      kbo(:, 3,57, 7) = (/ &
        & 0.37880e-04_rb,0.18114e-03_rb,0.21255e-03_rb,0.20984e-03_rb,0.10949e-03_rb /)
      kbo(:, 4,57, 7) = (/ &
        & 0.44374e-04_rb,0.24914e-03_rb,0.29262e-03_rb,0.28609e-03_rb,0.14004e-03_rb /)
      kbo(:, 5,57, 7) = (/ &
        & 0.52560e-04_rb,0.34723e-03_rb,0.40853e-03_rb,0.39531e-03_rb,0.17907e-03_rb /)
      kbo(:, 1,58, 7) = (/ &
        & 0.22293e-04_rb,0.84410e-04_rb,0.97318e-04_rb,0.95954e-04_rb,0.55246e-04_rb /)
      kbo(:, 2,58, 7) = (/ &
        & 0.26200e-04_rb,0.11013e-03_rb,0.12878e-03_rb,0.12778e-03_rb,0.70434e-04_rb /)
      kbo(:, 3,58, 7) = (/ &
        & 0.30694e-04_rb,0.14878e-03_rb,0.17475e-03_rb,0.17269e-03_rb,0.90681e-04_rb /)
      kbo(:, 4,58, 7) = (/ &
        & 0.36071e-04_rb,0.20666e-03_rb,0.24284e-03_rb,0.23795e-03_rb,0.11731e-03_rb /)
      kbo(:, 5,58, 7) = (/ &
        & 0.42845e-04_rb,0.29161e-03_rb,0.34322e-03_rb,0.33267e-03_rb,0.15208e-03_rb /)
      kbo(:, 1,59, 7) = (/ &
        & 0.18430e-04_rb,0.71706e-04_rb,0.82601e-04_rb,0.81289e-04_rb,0.46584e-04_rb /)
      kbo(:, 2,59, 7) = (/ &
        & 0.21734e-04_rb,0.94599e-04_rb,0.11060e-03_rb,0.10958e-03_rb,0.60168e-04_rb /)
      kbo(:, 3,59, 7) = (/ &
        & 0.25561e-04_rb,0.12924e-03_rb,0.15196e-03_rb,0.15008e-03_rb,0.78430e-04_rb /)
      kbo(:, 4,59, 7) = (/ &
        & 0.30185e-04_rb,0.18220e-03_rb,0.21420e-03_rb,0.20980e-03_rb,0.10280e-03_rb /)
      kbo(:, 5,59, 7) = (/ &
        & 0.36067e-04_rb,0.26061e-03_rb,0.30710e-03_rb,0.29739e-03_rb,0.13531e-03_rb /)
      kbo(:, 1,13, 8) = (/ &
        & 0.32299e+00_rb,0.54760e+00_rb,0.60079e+00_rb,0.58517e+00_rb,0.42276e+00_rb /)
      kbo(:, 2,13, 8) = (/ &
        & 0.32594e+00_rb,0.55963e+00_rb,0.61653e+00_rb,0.60147e+00_rb,0.43281e+00_rb /)
      kbo(:, 3,13, 8) = (/ &
        & 0.33056e+00_rb,0.57182e+00_rb,0.63236e+00_rb,0.61863e+00_rb,0.44149e+00_rb /)
      kbo(:, 4,13, 8) = (/ &
        & 0.33706e+00_rb,0.58426e+00_rb,0.64822e+00_rb,0.63489e+00_rb,0.44893e+00_rb /)
      kbo(:, 5,13, 8) = (/ &
        & 0.34635e+00_rb,0.59739e+00_rb,0.66343e+00_rb,0.65003e+00_rb,0.45486e+00_rb /)
      kbo(:, 1,14, 8) = (/ &
        & 0.27185e+00_rb,0.47396e+00_rb,0.52087e+00_rb,0.50518e+00_rb,0.35898e+00_rb /)
      kbo(:, 2,14, 8) = (/ &
        & 0.27455e+00_rb,0.48454e+00_rb,0.53473e+00_rb,0.52058e+00_rb,0.36725e+00_rb /)
      kbo(:, 3,14, 8) = (/ &
        & 0.27878e+00_rb,0.49523e+00_rb,0.54876e+00_rb,0.53556e+00_rb,0.37477e+00_rb /)
      kbo(:, 4,14, 8) = (/ &
        & 0.28540e+00_rb,0.50660e+00_rb,0.56244e+00_rb,0.54948e+00_rb,0.38095e+00_rb /)
      kbo(:, 5,14, 8) = (/ &
        & 0.29474e+00_rb,0.51876e+00_rb,0.57592e+00_rb,0.56255e+00_rb,0.38660e+00_rb /)
      kbo(:, 1,15, 8) = (/ &
        & 0.22818e+00_rb,0.40825e+00_rb,0.45003e+00_rb,0.43582e+00_rb,0.30350e+00_rb /)
      kbo(:, 2,15, 8) = (/ &
        & 0.23074e+00_rb,0.41749e+00_rb,0.46220e+00_rb,0.44930e+00_rb,0.31064e+00_rb /)
      kbo(:, 3,15, 8) = (/ &
        & 0.23510e+00_rb,0.42737e+00_rb,0.47434e+00_rb,0.46202e+00_rb,0.31696e+00_rb /)
      kbo(:, 4,15, 8) = (/ &
        & 0.24186e+00_rb,0.43783e+00_rb,0.48637e+00_rb,0.47387e+00_rb,0.32265e+00_rb /)
      kbo(:, 5,15, 8) = (/ &
        & 0.25121e+00_rb,0.44943e+00_rb,0.49855e+00_rb,0.48519e+00_rb,0.32816e+00_rb /)
      kbo(:, 1,16, 8) = (/ &
        & 0.19120e+00_rb,0.34975e+00_rb,0.38732e+00_rb,0.37509e+00_rb,0.25617e+00_rb /)
      kbo(:, 2,16, 8) = (/ &
        & 0.19385e+00_rb,0.35826e+00_rb,0.39799e+00_rb,0.38666e+00_rb,0.26259e+00_rb /)
      kbo(:, 3,16, 8) = (/ &
        & 0.19846e+00_rb,0.36727e+00_rb,0.40872e+00_rb,0.39750e+00_rb,0.26815e+00_rb /)
      kbo(:, 4,16, 8) = (/ &
        & 0.20537e+00_rb,0.37712e+00_rb,0.41954e+00_rb,0.40777e+00_rb,0.27330e+00_rb /)
      kbo(:, 5,16, 8) = (/ &
        & 0.21427e+00_rb,0.38868e+00_rb,0.43071e+00_rb,0.41784e+00_rb,0.27853e+00_rb /)
      kbo(:, 1,17, 8) = (/ &
        & 0.15988e+00_rb,0.29854e+00_rb,0.33180e+00_rb,0.32201e+00_rb,0.21611e+00_rb /)
      kbo(:, 2,17, 8) = (/ &
        & 0.16279e+00_rb,0.30628e+00_rb,0.34129e+00_rb,0.33188e+00_rb,0.22162e+00_rb /)
      kbo(:, 3,17, 8) = (/ &
        & 0.16763e+00_rb,0.31463e+00_rb,0.35085e+00_rb,0.34122e+00_rb,0.22664e+00_rb /)
      kbo(:, 4,17, 8) = (/ &
        & 0.17444e+00_rb,0.32432e+00_rb,0.36068e+00_rb,0.35032e+00_rb,0.23161e+00_rb /)
      kbo(:, 5,17, 8) = (/ &
        & 0.18274e+00_rb,0.33609e+00_rb,0.37140e+00_rb,0.35944e+00_rb,0.23665e+00_rb /)
      kbo(:, 1,18, 8) = (/ &
        & 0.13352e+00_rb,0.25404e+00_rb,0.28330e+00_rb,0.27577e+00_rb,0.18225e+00_rb /)
      kbo(:, 2,18, 8) = (/ &
        & 0.13667e+00_rb,0.26103e+00_rb,0.29171e+00_rb,0.28431e+00_rb,0.18712e+00_rb /)
      kbo(:, 3,18, 8) = (/ &
        & 0.14164e+00_rb,0.26910e+00_rb,0.30030e+00_rb,0.29254e+00_rb,0.19176e+00_rb /)
      kbo(:, 4,18, 8) = (/ &
        & 0.14809e+00_rb,0.27886e+00_rb,0.30959e+00_rb,0.30073e+00_rb,0.19637e+00_rb /)
      kbo(:, 5,18, 8) = (/ &
        & 0.15551e+00_rb,0.29050e+00_rb,0.32020e+00_rb,0.30930e+00_rb,0.20115e+00_rb /)
      kbo(:, 1,19, 8) = (/ &
        & 0.11177e+00_rb,0.21580e+00_rb,0.24136e+00_rb,0.23552e+00_rb,0.15392e+00_rb /)
      kbo(:, 2,19, 8) = (/ &
        & 0.11504e+00_rb,0.22243e+00_rb,0.24890e+00_rb,0.24304e+00_rb,0.15833e+00_rb /)
      kbo(:, 3,19, 8) = (/ &
        & 0.11975e+00_rb,0.23031e+00_rb,0.25687e+00_rb,0.25048e+00_rb,0.16244e+00_rb /)
      kbo(:, 4,19, 8) = (/ &
        & 0.12553e+00_rb,0.23994e+00_rb,0.26588e+00_rb,0.25812e+00_rb,0.16671e+00_rb /)
      kbo(:, 5,19, 8) = (/ &
        & 0.13204e+00_rb,0.25107e+00_rb,0.27639e+00_rb,0.26629e+00_rb,0.17110e+00_rb /)
      kbo(:, 1,20, 8) = (/ &
        & 0.93818e-01_rb,0.18336e+00_rb,0.20533e+00_rb,0.20083e+00_rb,0.13063e+00_rb /)
      kbo(:, 2,20, 8) = (/ &
        & 0.97153e-01_rb,0.18975e+00_rb,0.21222e+00_rb,0.20754e+00_rb,0.13445e+00_rb /)
      kbo(:, 3,20, 8) = (/ &
        & 0.10157e+00_rb,0.19759e+00_rb,0.21989e+00_rb,0.21439e+00_rb,0.13819e+00_rb /)
      kbo(:, 4,20, 8) = (/ &
        & 0.10656e+00_rb,0.20692e+00_rb,0.22885e+00_rb,0.22170e+00_rb,0.14200e+00_rb /)
      kbo(:, 5,20, 8) = (/ &
        & 0.11217e+00_rb,0.21741e+00_rb,0.23914e+00_rb,0.22972e+00_rb,0.14595e+00_rb /)
      kbo(:, 1,21, 8) = (/ &
        & 0.78864e-01_rb,0.15581e+00_rb,0.17467e+00_rb,0.17111e+00_rb,0.11093e+00_rb /)
      kbo(:, 2,21, 8) = (/ &
        & 0.82115e-01_rb,0.16213e+00_rb,0.18114e+00_rb,0.17722e+00_rb,0.11448e+00_rb /)
      kbo(:, 3,21, 8) = (/ &
        & 0.86091e-01_rb,0.16989e+00_rb,0.18858e+00_rb,0.18368e+00_rb,0.11795e+00_rb /)
      kbo(:, 4,21, 8) = (/ &
        & 0.90544e-01_rb,0.17883e+00_rb,0.19740e+00_rb,0.19075e+00_rb,0.12140e+00_rb /)
      kbo(:, 5,21, 8) = (/ &
        & 0.95531e-01_rb,0.18857e+00_rb,0.20719e+00_rb,0.19872e+00_rb,0.12494e+00_rb /)
      kbo(:, 1,22, 8) = (/ &
        & 0.66536e-01_rb,0.13293e+00_rb,0.14913e+00_rb,0.14618e+00_rb,0.94429e-01_rb /)
      kbo(:, 2,22, 8) = (/ &
        & 0.69639e-01_rb,0.13931e+00_rb,0.15537e+00_rb,0.15187e+00_rb,0.97680e-01_rb /)
      kbo(:, 3,22, 8) = (/ &
        & 0.73194e-01_rb,0.14688e+00_rb,0.16282e+00_rb,0.15805e+00_rb,0.10087e+00_rb /)
      kbo(:, 4,22, 8) = (/ &
        & 0.77205e-01_rb,0.15536e+00_rb,0.17135e+00_rb,0.16503e+00_rb,0.10414e+00_rb /)
      kbo(:, 5,22, 8) = (/ &
        & 0.81667e-01_rb,0.16427e+00_rb,0.18066e+00_rb,0.17291e+00_rb,0.10744e+00_rb /)
      kbo(:, 1,23, 8) = (/ &
        & 0.56261e-01_rb,0.11383e+00_rb,0.12760e+00_rb,0.12512e+00_rb,0.80559e-01_rb /)
      kbo(:, 2,23, 8) = (/ &
        & 0.59086e-01_rb,0.12014e+00_rb,0.13384e+00_rb,0.13048e+00_rb,0.83485e-01_rb /)
      kbo(:, 3,23, 8) = (/ &
        & 0.62281e-01_rb,0.12738e+00_rb,0.14121e+00_rb,0.13648e+00_rb,0.86460e-01_rb /)
      kbo(:, 4,23, 8) = (/ &
        & 0.65906e-01_rb,0.13517e+00_rb,0.14937e+00_rb,0.14336e+00_rb,0.89462e-01_rb /)
      kbo(:, 5,23, 8) = (/ &
        & 0.69998e-01_rb,0.14335e+00_rb,0.15807e+00_rb,0.15098e+00_rb,0.92588e-01_rb /)
      kbo(:, 1,24, 8) = (/ &
        & 0.47642e-01_rb,0.97933e-01_rb,0.10963e+00_rb,0.10742e+00_rb,0.68750e-01_rb /)
      kbo(:, 2,24, 8) = (/ &
        & 0.50193e-01_rb,0.10399e+00_rb,0.11587e+00_rb,0.11258e+00_rb,0.71432e-01_rb /)
      kbo(:, 3,24, 8) = (/ &
        & 0.53085e-01_rb,0.11077e+00_rb,0.12298e+00_rb,0.11845e+00_rb,0.74194e-01_rb /)
      kbo(:, 4,24, 8) = (/ &
        & 0.56384e-01_rb,0.11795e+00_rb,0.13070e+00_rb,0.12512e+00_rb,0.76973e-01_rb /)
      kbo(:, 5,24, 8) = (/ &
        & 0.60174e-01_rb,0.12555e+00_rb,0.13879e+00_rb,0.13237e+00_rb,0.79920e-01_rb /)
      kbo(:, 1,25, 8) = (/ &
        & 0.40432e-01_rb,0.84645e-01_rb,0.94724e-01_rb,0.92637e-01_rb,0.58805e-01_rb /)
      kbo(:, 2,25, 8) = (/ &
        & 0.42748e-01_rb,0.90409e-01_rb,0.10079e+00_rb,0.97654e-01_rb,0.61289e-01_rb /)
      kbo(:, 3,25, 8) = (/ &
        & 0.45392e-01_rb,0.96682e-01_rb,0.10756e+00_rb,0.10341e+00_rb,0.63818e-01_rb /)
      kbo(:, 4,25, 8) = (/ &
        & 0.48438e-01_rb,0.10335e+00_rb,0.11478e+00_rb,0.10978e+00_rb,0.66481e-01_rb /)
      kbo(:, 5,25, 8) = (/ &
        & 0.52019e-01_rb,0.11050e+00_rb,0.12234e+00_rb,0.11664e+00_rb,0.69209e-01_rb /)
      kbo(:, 1,26, 8) = (/ &
        & 0.34430e-01_rb,0.73567e-01_rb,0.82349e-01_rb,0.80341e-01_rb,0.50485e-01_rb /)
      kbo(:, 2,26, 8) = (/ &
        & 0.36558e-01_rb,0.78997e-01_rb,0.88163e-01_rb,0.85293e-01_rb,0.52835e-01_rb /)
      kbo(:, 3,26, 8) = (/ &
        & 0.39009e-01_rb,0.84818e-01_rb,0.94521e-01_rb,0.90851e-01_rb,0.55181e-01_rb /)
      kbo(:, 4,26, 8) = (/ &
        & 0.41876e-01_rb,0.91074e-01_rb,0.10127e+00_rb,0.96922e-01_rb,0.57644e-01_rb /)
      kbo(:, 5,26, 8) = (/ &
        & 0.45294e-01_rb,0.97874e-01_rb,0.10841e+00_rb,0.10340e+00_rb,0.60258e-01_rb /)
      kbo(:, 1,27, 8) = (/ &
        & 0.29418e-01_rb,0.64289e-01_rb,0.71984e-01_rb,0.70112e-01_rb,0.43514e-01_rb /)
      kbo(:, 2,27, 8) = (/ &
        & 0.31367e-01_rb,0.69333e-01_rb,0.77500e-01_rb,0.74945e-01_rb,0.45654e-01_rb /)
      kbo(:, 3,27, 8) = (/ &
        & 0.33650e-01_rb,0.74779e-01_rb,0.83451e-01_rb,0.80284e-01_rb,0.47838e-01_rb /)
      kbo(:, 4,27, 8) = (/ &
        & 0.36373e-01_rb,0.80698e-01_rb,0.89803e-01_rb,0.86035e-01_rb,0.50188e-01_rb /)
      kbo(:, 5,27, 8) = (/ &
        & 0.39625e-01_rb,0.87227e-01_rb,0.96605e-01_rb,0.92163e-01_rb,0.52661e-01_rb /)
      kbo(:, 1,28, 8) = (/ &
        & 0.25206e-01_rb,0.56436e-01_rb,0.63282e-01_rb,0.61571e-01_rb,0.37602e-01_rb /)
      kbo(:, 2,28, 8) = (/ &
        & 0.27009e-01_rb,0.61147e-01_rb,0.68481e-01_rb,0.66237e-01_rb,0.39562e-01_rb /)
      kbo(:, 3,28, 8) = (/ &
        & 0.29150e-01_rb,0.66302e-01_rb,0.74074e-01_rb,0.71341e-01_rb,0.41601e-01_rb /)
      kbo(:, 4,28, 8) = (/ &
        & 0.31750e-01_rb,0.71955e-01_rb,0.80104e-01_rb,0.76785e-01_rb,0.43794e-01_rb /)
      kbo(:, 5,28, 8) = (/ &
        & 0.34834e-01_rb,0.78268e-01_rb,0.86662e-01_rb,0.82632e-01_rb,0.46193e-01_rb /)
      kbo(:, 1,29, 8) = (/ &
        & 0.21672e-01_rb,0.49821e-01_rb,0.55985e-01_rb,0.54431e-01_rb,0.32677e-01_rb /)
      kbo(:, 2,29, 8) = (/ &
        & 0.23360e-01_rb,0.54257e-01_rb,0.60871e-01_rb,0.58918e-01_rb,0.34453e-01_rb /)
      kbo(:, 3,29, 8) = (/ &
        & 0.25420e-01_rb,0.59159e-01_rb,0.66168e-01_rb,0.63765e-01_rb,0.36398e-01_rb /)
      kbo(:, 4,29, 8) = (/ &
        & 0.27908e-01_rb,0.64629e-01_rb,0.71935e-01_rb,0.68977e-01_rb,0.38490e-01_rb /)
      kbo(:, 5,29, 8) = (/ &
        & 0.30843e-01_rb,0.70791e-01_rb,0.78323e-01_rb,0.74610e-01_rb,0.40747e-01_rb /)
      kbo(:, 1,30, 8) = (/ &
        & 0.18703e-01_rb,0.44236e-01_rb,0.49819e-01_rb,0.48423e-01_rb,0.28503e-01_rb /)
      kbo(:, 2,30, 8) = (/ &
        & 0.20315e-01_rb,0.48445e-01_rb,0.54425e-01_rb,0.52720e-01_rb,0.30174e-01_rb /)
      kbo(:, 3,30, 8) = (/ &
        & 0.22307e-01_rb,0.53163e-01_rb,0.59481e-01_rb,0.57359e-01_rb,0.32018e-01_rb /)
      kbo(:, 4,30, 8) = (/ &
        & 0.24687e-01_rb,0.58495e-01_rb,0.65064e-01_rb,0.62368e-01_rb,0.34022e-01_rb /)
      kbo(:, 5,30, 8) = (/ &
        & 0.27510e-01_rb,0.64597e-01_rb,0.71352e-01_rb,0.67860e-01_rb,0.36243e-01_rb /)
      kbo(:, 1,31, 8) = (/ &
        & 0.16227e-01_rb,0.39546e-01_rb,0.44613e-01_rb,0.43384e-01_rb,0.24947e-01_rb /)
      kbo(:, 2,31, 8) = (/ &
        & 0.17791e-01_rb,0.43576e-01_rb,0.49010e-01_rb,0.47501e-01_rb,0.26577e-01_rb /)
      kbo(:, 3,31, 8) = (/ &
        & 0.19720e-01_rb,0.48163e-01_rb,0.53877e-01_rb,0.51962e-01_rb,0.28347e-01_rb /)
      kbo(:, 4,31, 8) = (/ &
        & 0.22010e-01_rb,0.53419e-01_rb,0.59368e-01_rb,0.56842e-01_rb,0.30326e-01_rb /)
      kbo(:, 5,31, 8) = (/ &
        & 0.24778e-01_rb,0.59533e-01_rb,0.65582e-01_rb,0.62279e-01_rb,0.32464e-01_rb /)
      kbo(:, 1,32, 8) = (/ &
        & 0.14168e-01_rb,0.35610e-01_rb,0.40228e-01_rb,0.39145e-01_rb,0.21998e-01_rb /)
      kbo(:, 2,32, 8) = (/ &
        & 0.15701e-01_rb,0.39518e-01_rb,0.44455e-01_rb,0.43108e-01_rb,0.23551e-01_rb /)
      kbo(:, 3,32, 8) = (/ &
        & 0.17568e-01_rb,0.44028e-01_rb,0.49228e-01_rb,0.47442e-01_rb,0.25303e-01_rb /)
      kbo(:, 4,32, 8) = (/ &
        & 0.19822e-01_rb,0.49274e-01_rb,0.54644e-01_rb,0.52269e-01_rb,0.27219e-01_rb /)
      kbo(:, 5,32, 8) = (/ &
        & 0.22492e-01_rb,0.55429e-01_rb,0.60878e-01_rb,0.57696e-01_rb,0.29289e-01_rb /)
      kbo(:, 1,33, 8) = (/ &
        & 0.12469e-01_rb,0.32321e-01_rb,0.36548e-01_rb,0.35586e-01_rb,0.19535e-01_rb /)
      kbo(:, 2,33, 8) = (/ &
        & 0.13967e-01_rb,0.36160e-01_rb,0.40670e-01_rb,0.39414e-01_rb,0.21057e-01_rb /)
      kbo(:, 3,33, 8) = (/ &
        & 0.15804e-01_rb,0.40646e-01_rb,0.45371e-01_rb,0.43697e-01_rb,0.22748e-01_rb /)
      kbo(:, 4,33, 8) = (/ &
        & 0.18007e-01_rb,0.45935e-01_rb,0.50790e-01_rb,0.48504e-01_rb,0.24589e-01_rb /)
      kbo(:, 5,33, 8) = (/ &
        & 0.20583e-01_rb,0.52117e-01_rb,0.57083e-01_rb,0.53949e-01_rb,0.26612e-01_rb /)
      kbo(:, 1,34, 8) = (/ &
        & 0.11030e-01_rb,0.29462e-01_rb,0.33333e-01_rb,0.32457e-01_rb,0.17441e-01_rb /)
      kbo(:, 2,34, 8) = (/ &
        & 0.12485e-01_rb,0.33252e-01_rb,0.37374e-01_rb,0.36203e-01_rb,0.18902e-01_rb /)
      kbo(:, 3,34, 8) = (/ &
        & 0.14284e-01_rb,0.37735e-01_rb,0.42039e-01_rb,0.40449e-01_rb,0.20533e-01_rb /)
      kbo(:, 4,34, 8) = (/ &
        & 0.16406e-01_rb,0.43050e-01_rb,0.47473e-01_rb,0.45250e-01_rb,0.22315e-01_rb /)
      kbo(:, 5,34, 8) = (/ &
        & 0.18889e-01_rb,0.49221e-01_rb,0.53838e-01_rb,0.50709e-01_rb,0.24295e-01_rb /)
      kbo(:, 1,35, 8) = (/ &
        & 0.97045e-02_rb,0.26765e-01_rb,0.30294e-01_rb,0.29494e-01_rb,0.15527e-01_rb /)
      kbo(:, 2,35, 8) = (/ &
        & 0.11116e-01_rb,0.30471e-01_rb,0.34237e-01_rb,0.33156e-01_rb,0.16936e-01_rb /)
      kbo(:, 3,35, 8) = (/ &
        & 0.12833e-01_rb,0.34908e-01_rb,0.38827e-01_rb,0.37335e-01_rb,0.18494e-01_rb /)
      kbo(:, 4,35, 8) = (/ &
        & 0.14851e-01_rb,0.40160e-01_rb,0.44228e-01_rb,0.42095e-01_rb,0.20230e-01_rb /)
      kbo(:, 5,35, 8) = (/ &
        & 0.17220e-01_rb,0.46275e-01_rb,0.50600e-01_rb,0.47524e-01_rb,0.22137e-01_rb /)
      kbo(:, 1,36, 8) = (/ &
        & 0.84787e-02_rb,0.24131e-01_rb,0.27339e-01_rb,0.26619e-01_rb,0.13756e-01_rb /)
      kbo(:, 2,36, 8) = (/ &
        & 0.98033e-02_rb,0.27706e-01_rb,0.31147e-01_rb,0.30166e-01_rb,0.15106e-01_rb /)
      kbo(:, 3,36, 8) = (/ &
        & 0.11407e-01_rb,0.32015e-01_rb,0.35610e-01_rb,0.34239e-01_rb,0.16593e-01_rb /)
      kbo(:, 4,36, 8) = (/ &
        & 0.13302e-01_rb,0.37140e-01_rb,0.40907e-01_rb,0.38904e-01_rb,0.18257e-01_rb /)
      kbo(:, 5,36, 8) = (/ &
        & 0.15537e-01_rb,0.43129e-01_rb,0.47182e-01_rb,0.44236e-01_rb,0.20074e-01_rb /)
      kbo(:, 1,37, 8) = (/ &
        & 0.72685e-02_rb,0.21392e-01_rb,0.24301e-01_rb,0.23679e-01_rb,0.12087e-01_rb /)
      kbo(:, 2,37, 8) = (/ &
        & 0.84733e-02_rb,0.24759e-01_rb,0.27898e-01_rb,0.27046e-01_rb,0.13342e-01_rb /)
      kbo(:, 3,37, 8) = (/ &
        & 0.99353e-02_rb,0.28848e-01_rb,0.32152e-01_rb,0.30940e-01_rb,0.14748e-01_rb /)
      kbo(:, 4,37, 8) = (/ &
        & 0.11673e-01_rb,0.33734e-01_rb,0.37224e-01_rb,0.35422e-01_rb,0.16319e-01_rb /)
      kbo(:, 5,37, 8) = (/ &
        & 0.13740e-01_rb,0.39481e-01_rb,0.43256e-01_rb,0.40566e-01_rb,0.18049e-01_rb /)
      kbo(:, 1,38, 8) = (/ &
        & 0.62326e-02_rb,0.18999e-01_rb,0.21644e-01_rb,0.21103e-01_rb,0.10629e-01_rb /)
      kbo(:, 2,38, 8) = (/ &
        & 0.73269e-02_rb,0.22170e-01_rb,0.25042e-01_rb,0.24299e-01_rb,0.11807e-01_rb /)
      kbo(:, 3,38, 8) = (/ &
        & 0.86586e-02_rb,0.26045e-01_rb,0.29098e-01_rb,0.28019e-01_rb,0.13138e-01_rb /)
      kbo(:, 4,38, 8) = (/ &
        & 0.10254e-01_rb,0.30699e-01_rb,0.33952e-01_rb,0.32316e-01_rb,0.14621e-01_rb /)
      kbo(:, 5,38, 8) = (/ &
        & 0.12168e-01_rb,0.36211e-01_rb,0.39748e-01_rb,0.37288e-01_rb,0.16263e-01_rb /)
      kbo(:, 1,39, 8) = (/ &
        & 0.53522e-02_rb,0.16925e-01_rb,0.19337e-01_rb,0.18863e-01_rb,0.93697e-02_rb /)
      kbo(:, 2,39, 8) = (/ &
        & 0.63437e-02_rb,0.19916e-01_rb,0.22555e-01_rb,0.21900e-01_rb,0.10477e-01_rb /)
      kbo(:, 3,39, 8) = (/ &
        & 0.75600e-02_rb,0.23590e-01_rb,0.26428e-01_rb,0.25453e-01_rb,0.11732e-01_rb /)
      kbo(:, 4,39, 8) = (/ &
        & 0.90277e-02_rb,0.28027e-01_rb,0.31081e-01_rb,0.29581e-01_rb,0.13140e-01_rb /)
      kbo(:, 5,39, 8) = (/ &
        & 0.10804e-01_rb,0.33325e-01_rb,0.36655e-01_rb,0.34396e-01_rb,0.14714e-01_rb /)
      kbo(:, 1,40, 8) = (/ &
        & 0.45183e-02_rb,0.14862e-01_rb,0.17043e-01_rb,0.16640e-01_rb,0.81906e-02_rb /)
      kbo(:, 2,40, 8) = (/ &
        & 0.53964e-02_rb,0.17630e-01_rb,0.20043e-01_rb,0.19482e-01_rb,0.92335e-02_rb /)
      kbo(:, 3,40, 8) = (/ &
        & 0.64835e-02_rb,0.21041e-01_rb,0.23674e-01_rb,0.22821e-01_rb,0.10400e-01_rb /)
      kbo(:, 4,40, 8) = (/ &
        & 0.78100e-02_rb,0.25197e-01_rb,0.28056e-01_rb,0.26733e-01_rb,0.11718e-01_rb /)
      kbo(:, 5,40, 8) = (/ &
        & 0.94310e-02_rb,0.30203e-01_rb,0.33322e-01_rb,0.31321e-01_rb,0.13209e-01_rb /)
      kbo(:, 1,41, 8) = (/ &
        & 0.38020e-02_rb,0.13035e-01_rb,0.15007e-01_rb,0.14665e-01_rb,0.71645e-02_rb /)
      kbo(:, 2,41, 8) = (/ &
        & 0.45796e-02_rb,0.15592e-01_rb,0.17802e-01_rb,0.17320e-01_rb,0.81332e-02_rb /)
      kbo(:, 3,41, 8) = (/ &
        & 0.55464e-02_rb,0.18749e-01_rb,0.21191e-01_rb,0.20448e-01_rb,0.92312e-02_rb /)
      kbo(:, 4,41, 8) = (/ &
        & 0.67408e-02_rb,0.22629e-01_rb,0.25309e-01_rb,0.24143e-01_rb,0.10458e-01_rb /)
      kbo(:, 5,41, 8) = (/ &
        & 0.82155e-02_rb,0.27347e-01_rb,0.30277e-01_rb,0.28506e-01_rb,0.11871e-01_rb /)
      kbo(:, 1,42, 8) = (/ &
        & 0.31962e-02_rb,0.11444e-01_rb,0.13229e-01_rb,0.12937e-01_rb,0.62788e-02_rb /)
      kbo(:, 2,42, 8) = (/ &
        & 0.38815e-02_rb,0.13798e-01_rb,0.15822e-01_rb,0.15407e-01_rb,0.71774e-02_rb /)
      kbo(:, 3,42, 8) = (/ &
        & 0.47422e-02_rb,0.16722e-01_rb,0.18990e-01_rb,0.18339e-01_rb,0.81986e-02_rb /)
      kbo(:, 4,42, 8) = (/ &
        & 0.58154e-02_rb,0.20342e-01_rb,0.22857e-01_rb,0.21830e-01_rb,0.93594e-02_rb /)
      kbo(:, 5,42, 8) = (/ &
        & 0.71546e-02_rb,0.24787e-01_rb,0.27545e-01_rb,0.25973e-01_rb,0.10684e-01_rb /)
      kbo(:, 1,43, 8) = (/ &
        & 0.26542e-02_rb,0.99421e-02_rb,0.11545e-01_rb,0.11297e-01_rb,0.54681e-02_rb /)
      kbo(:, 2,43, 8) = (/ &
        & 0.32467e-02_rb,0.12085e-01_rb,0.13926e-01_rb,0.13575e-01_rb,0.63017e-02_rb /)
      kbo(:, 3,43, 8) = (/ &
        & 0.40037e-02_rb,0.14758e-01_rb,0.16848e-01_rb,0.16296e-01_rb,0.72434e-02_rb /)
      kbo(:, 4,43, 8) = (/ &
        & 0.49534e-02_rb,0.18094e-01_rb,0.20438e-01_rb,0.19556e-01_rb,0.83354e-02_rb /)
      kbo(:, 5,43, 8) = (/ &
        & 0.61525e-02_rb,0.22230e-01_rb,0.24817e-01_rb,0.23450e-01_rb,0.95751e-02_rb /)
      kbo(:, 1,44, 8) = (/ &
        & 0.21881e-02_rb,0.85857e-02_rb,0.10019e-01_rb,0.98043e-02_rb,0.47456e-02_rb /)
      kbo(:, 2,44, 8) = (/ &
        & 0.26937e-02_rb,0.10525e-01_rb,0.12192e-01_rb,0.11898e-01_rb,0.55168e-02_rb /)
      kbo(:, 3,44, 8) = (/ &
        & 0.33531e-02_rb,0.12952e-01_rb,0.14870e-01_rb,0.14408e-01_rb,0.63938e-02_rb /)
      kbo(:, 4,44, 8) = (/ &
        & 0.41916e-02_rb,0.16016e-01_rb,0.18187e-01_rb,0.17441e-01_rb,0.73995e-02_rb /)
      kbo(:, 5,44, 8) = (/ &
        & 0.52529e-02_rb,0.19825e-01_rb,0.22245e-01_rb,0.21074e-01_rb,0.85693e-02_rb /)
      kbo(:, 1,45, 8) = (/ &
        & 0.17974e-02_rb,0.74036e-02_rb,0.86819e-02_rb,0.84949e-02_rb,0.41100e-02_rb /)
      kbo(:, 2,45, 8) = (/ &
        & 0.22283e-02_rb,0.91556e-02_rb,0.10664e-01_rb,0.10414e-01_rb,0.48257e-02_rb /)
      kbo(:, 3,45, 8) = (/ &
        & 0.27993e-02_rb,0.11360e-01_rb,0.13115e-01_rb,0.12731e-01_rb,0.56423e-02_rb /)
      kbo(:, 4,45, 8) = (/ &
        & 0.35358e-02_rb,0.14164e-01_rb,0.16170e-01_rb,0.15543e-01_rb,0.65867e-02_rb /)
      kbo(:, 5,45, 8) = (/ &
        & 0.44762e-02_rb,0.17671e-01_rb,0.19928e-01_rb,0.18931e-01_rb,0.76725e-02_rb /)
      kbo(:, 1,46, 8) = (/ &
        & 0.14666e-02_rb,0.63443e-02_rb,0.74710e-02_rb,0.73099e-02_rb,0.35540e-02_rb /)
      kbo(:, 2,46, 8) = (/ &
        & 0.18286e-02_rb,0.79140e-02_rb,0.92719e-02_rb,0.90588e-02_rb,0.41981e-02_rb /)
      kbo(:, 3,46, 8) = (/ &
        & 0.23196e-02_rb,0.99089e-02_rb,0.11508e-01_rb,0.11192e-01_rb,0.49656e-02_rb /)
      kbo(:, 4,46, 8) = (/ &
        & 0.29575e-02_rb,0.12451e-01_rb,0.14296e-01_rb,0.13779e-01_rb,0.58441e-02_rb /)
      kbo(:, 5,46, 8) = (/ &
        & 0.37842e-02_rb,0.15662e-01_rb,0.17758e-01_rb,0.16921e-01_rb,0.68660e-02_rb /)
      kbo(:, 1,47, 8) = (/ &
        & 0.11822e-02_rb,0.53700e-02_rb,0.63483e-02_rb,0.62090e-02_rb,0.30500e-02_rb /)
      kbo(:, 2,47, 8) = (/ &
        & 0.14817e-02_rb,0.67616e-02_rb,0.79678e-02_rb,0.77874e-02_rb,0.36317e-02_rb /)
      kbo(:, 3,47, 8) = (/ &
        & 0.18918e-02_rb,0.85392e-02_rb,0.99818e-02_rb,0.97254e-02_rb,0.43248e-02_rb /)
      kbo(:, 4,47, 8) = (/ &
        & 0.24382e-02_rb,0.10820e-01_rb,0.12503e-01_rb,0.12088e-01_rb,0.51477e-02_rb /)
      kbo(:, 5,47, 8) = (/ &
        & 0.31533e-02_rb,0.13725e-01_rb,0.15656e-01_rb,0.14970e-01_rb,0.61000e-02_rb /)
      kbo(:, 1,48, 8) = (/ &
        & 0.94906e-03_rb,0.45298e-02_rb,0.53706e-02_rb,0.52506e-02_rb,0.26162e-02_rb /)
      kbo(:, 2,48, 8) = (/ &
        & 0.11965e-02_rb,0.57597e-02_rb,0.68213e-02_rb,0.66698e-02_rb,0.31403e-02_rb /)
      kbo(:, 3,48, 8) = (/ &
        & 0.15369e-02_rb,0.73380e-02_rb,0.86368e-02_rb,0.84246e-02_rb,0.37687e-02_rb /)
      kbo(:, 4,48, 8) = (/ &
        & 0.20008e-02_rb,0.93814e-02_rb,0.10911e-01_rb,0.10580e-01_rb,0.45213e-02_rb /)
      kbo(:, 5,48, 8) = (/ &
        & 0.26159e-02_rb,0.12002e-01_rb,0.13774e-01_rb,0.13221e-01_rb,0.54142e-02_rb /)
      kbo(:, 1,49, 8) = (/ &
        & 0.75913e-03_rb,0.38043e-02_rb,0.45203e-02_rb,0.44181e-02_rb,0.22379e-02_rb /)
      kbo(:, 2,49, 8) = (/ &
        & 0.96097e-03_rb,0.48899e-02_rb,0.58138e-02_rb,0.56878e-02_rb,0.27098e-02_rb /)
      kbo(:, 3,49, 8) = (/ &
        & 0.12447e-02_rb,0.62895e-02_rb,0.74501e-02_rb,0.72727e-02_rb,0.32859e-02_rb /)
      kbo(:, 4,49, 8) = (/ &
        & 0.16322e-02_rb,0.81122e-02_rb,0.94989e-02_rb,0.92349e-02_rb,0.39720e-02_rb /)
      kbo(:, 5,49, 8) = (/ &
        & 0.21597e-02_rb,0.10470e-01_rb,0.12092e-01_rb,0.11650e-01_rb,0.47938e-02_rb /)
      kbo(:, 1,50, 8) = (/ &
        & 0.60949e-03_rb,0.32009e-02_rb,0.38080e-02_rb,0.37204e-02_rb,0.19148e-02_rb /)
      kbo(:, 2,50, 8) = (/ &
        & 0.77332e-03_rb,0.41637e-02_rb,0.49658e-02_rb,0.48588e-02_rb,0.23498e-02_rb /)
      kbo(:, 3,50, 8) = (/ &
        & 0.10099e-02_rb,0.54095e-02_rb,0.64416e-02_rb,0.62938e-02_rb,0.28713e-02_rb /)
      kbo(:, 4,50, 8) = (/ &
        & 0.13353e-02_rb,0.70417e-02_rb,0.83004e-02_rb,0.80844e-02_rb,0.35114e-02_rb /)
      kbo(:, 5,50, 8) = (/ &
        & 0.17873e-02_rb,0.91691e-02_rb,0.10658e-01_rb,0.10303e-01_rb,0.42641e-02_rb /)
      kbo(:, 1,51, 8) = (/ &
        & 0.48960e-03_rb,0.26873e-02_rb,0.32008e-02_rb,0.31259e-02_rb,0.16378e-02_rb /)
      kbo(:, 2,51, 8) = (/ &
        & 0.62187e-03_rb,0.35423e-02_rb,0.42349e-02_rb,0.41443e-02_rb,0.20325e-02_rb /)
      kbo(:, 3,51, 8) = (/ &
        & 0.81788e-03_rb,0.46529e-02_rb,0.55656e-02_rb,0.54424e-02_rb,0.25164e-02_rb /)
      kbo(:, 4,51, 8) = (/ &
        & 0.10935e-02_rb,0.61155e-02_rb,0.72565e-02_rb,0.70765e-02_rb,0.31007e-02_rb /)
      kbo(:, 5,51, 8) = (/ &
        & 0.14776e-02_rb,0.80416e-02_rb,0.94087e-02_rb,0.91197e-02_rb,0.38085e-02_rb /)
      kbo(:, 1,52, 8) = (/ &
        & 0.39302e-03_rb,0.22435e-02_rb,0.26754e-02_rb,0.26122e-02_rb,0.13898e-02_rb /)
      kbo(:, 2,52, 8) = (/ &
        & 0.49889e-03_rb,0.30006e-02_rb,0.35943e-02_rb,0.35172e-02_rb,0.17532e-02_rb /)
      kbo(:, 3,52, 8) = (/ &
        & 0.65922e-03_rb,0.39906e-02_rb,0.47901e-02_rb,0.46880e-02_rb,0.21980e-02_rb /)
      kbo(:, 4,52, 8) = (/ &
        & 0.89075e-03_rb,0.52980e-02_rb,0.63237e-02_rb,0.61748e-02_rb,0.27412e-02_rb /)
      kbo(:, 5,52, 8) = (/ &
        & 0.12150e-02_rb,0.70312e-02_rb,0.82837e-02_rb,0.80497e-02_rb,0.33956e-02_rb /)
      kbo(:, 1,53, 8) = (/ &
        & 0.31533e-03_rb,0.18598e-02_rb,0.22216e-02_rb,0.21690e-02_rb,0.11705e-02_rb /)
      kbo(:, 2,53, 8) = (/ &
        & 0.39919e-03_rb,0.25290e-02_rb,0.30338e-02_rb,0.29679e-02_rb,0.15051e-02_rb /)
      kbo(:, 3,53, 8) = (/ &
        & 0.52958e-03_rb,0.34096e-02_rb,0.41030e-02_rb,0.40189e-02_rb,0.19137e-02_rb /)
      kbo(:, 4,53, 8) = (/ &
        & 0.72147e-03_rb,0.45756e-02_rb,0.54894e-02_rb,0.53678e-02_rb,0.24147e-02_rb /)
      kbo(:, 5,53, 8) = (/ &
        & 0.99563e-03_rb,0.61321e-02_rb,0.72739e-02_rb,0.70826e-02_rb,0.30286e-02_rb /)
      kbo(:, 1,54, 8) = (/ &
        & 0.25434e-03_rb,0.15477e-02_rb,0.18502e-02_rb,0.18075e-02_rb,0.98891e-03_rb /)
      kbo(:, 2,54, 8) = (/ &
        & 0.32221e-03_rb,0.21398e-02_rb,0.25708e-02_rb,0.25148e-02_rb,0.12935e-02_rb /)
      kbo(:, 3,54, 8) = (/ &
        & 0.42775e-03_rb,0.29272e-02_rb,0.35315e-02_rb,0.34599e-02_rb,0.16741e-02_rb /)
      kbo(:, 4,54, 8) = (/ &
        & 0.58762e-03_rb,0.39761e-02_rb,0.47907e-02_rb,0.46896e-02_rb,0.21376e-02_rb /)
      kbo(:, 5,54, 8) = (/ &
        & 0.82053e-03_rb,0.53824e-02_rb,0.64244e-02_rb,0.62664e-02_rb,0.27173e-02_rb /)
      kbo(:, 1,55, 8) = (/ &
        & 0.20618e-03_rb,0.12866e-02_rb,0.15400e-02_rb,0.15055e-02_rb,0.83572e-03_rb /)
      kbo(:, 2,55, 8) = (/ &
        & 0.26059e-03_rb,0.18094e-02_rb,0.21787e-02_rb,0.21302e-02_rb,0.11090e-02_rb /)
      kbo(:, 3,55, 8) = (/ &
        & 0.34651e-03_rb,0.25140e-02_rb,0.30392e-02_rb,0.29780e-02_rb,0.14628e-02_rb /)
      kbo(:, 4,55, 8) = (/ &
        & 0.47882e-03_rb,0.34608e-02_rb,0.41831e-02_rb,0.41003e-02_rb,0.18992e-02_rb /)
      kbo(:, 5,55, 8) = (/ &
        & 0.67671e-03_rb,0.47331e-02_rb,0.56817e-02_rb,0.55503e-02_rb,0.24360e-02_rb /)
      kbo(:, 1,56, 8) = (/ &
        & 0.16681e-03_rb,0.10621e-02_rb,0.12728e-02_rb,0.12461e-02_rb,0.70162e-03_rb /)
      kbo(:, 2,56, 8) = (/ &
        & 0.21064e-03_rb,0.15213e-02_rb,0.18350e-02_rb,0.17945e-02_rb,0.94828e-03_rb /)
      kbo(:, 3,56, 8) = (/ &
        & 0.28028e-03_rb,0.21500e-02_rb,0.26045e-02_rb,0.25521e-02_rb,0.12721e-02_rb /)
      kbo(:, 4,56, 8) = (/ &
        & 0.38906e-03_rb,0.30029e-02_rb,0.36387e-02_rb,0.35710e-02_rb,0.16833e-02_rb /)
      kbo(:, 5,56, 8) = (/ &
        & 0.55568e-03_rb,0.41534e-02_rb,0.50111e-02_rb,0.49027e-02_rb,0.21849e-02_rb /)
      kbo(:, 1,57, 8) = (/ &
        & 0.13484e-03_rb,0.87168e-03_rb,0.10444e-02_rb,0.10248e-02_rb,0.58516e-03_rb /)
      kbo(:, 2,57, 8) = (/ &
        & 0.17007e-03_rb,0.12717e-02_rb,0.15360e-02_rb,0.15024e-02_rb,0.80805e-03_rb /)
      kbo(:, 3,57, 8) = (/ &
        & 0.22603e-03_rb,0.18296e-02_rb,0.22211e-02_rb,0.21757e-02_rb,0.11006e-02_rb /)
      kbo(:, 4,57, 8) = (/ &
        & 0.31565e-03_rb,0.25963e-02_rb,0.31531e-02_rb,0.30959e-02_rb,0.14851e-02_rb /)
      kbo(:, 5,57, 8) = (/ &
        & 0.45417e-03_rb,0.36365e-02_rb,0.44058e-02_rb,0.43157e-02_rb,0.19625e-02_rb /)
      kbo(:, 1,58, 8) = (/ &
        & 0.10938e-03_rb,0.71449e-03_rb,0.85655e-03_rb,0.84240e-03_rb,0.48772e-03_rb /)
      kbo(:, 2,58, 8) = (/ &
        & 0.13752e-03_rb,0.10624e-02_rb,0.12849e-02_rb,0.12583e-02_rb,0.68689e-03_rb /)
      kbo(:, 3,58, 8) = (/ &
        & 0.18307e-03_rb,0.15572e-02_rb,0.18954e-02_rb,0.18566e-02_rb,0.95344e-03_rb /)
      kbo(:, 4,58, 8) = (/ &
        & 0.25673e-03_rb,0.22481e-02_rb,0.27363e-02_rb,0.26877e-02_rb,0.13079e-02_rb /)
      kbo(:, 5,58, 8) = (/ &
        & 0.37230e-03_rb,0.31921e-02_rb,0.38812e-02_rb,0.38070e-02_rb,0.17570e-02_rb /)
      kbo(:, 1,59, 8) = (/ &
        & 0.92153e-04_rb,0.62798e-03_rb,0.75374e-03_rb,0.74243e-03_rb,0.43161e-03_rb /)
      kbo(:, 2,59, 8) = (/ &
        & 0.11676e-03_rb,0.95019e-03_rb,0.11525e-02_rb,0.11296e-02_rb,0.61959e-03_rb /)
      kbo(:, 3,59, 8) = (/ &
        & 0.15687e-03_rb,0.14168e-02_rb,0.17285e-02_rb,0.16938e-02_rb,0.87468e-03_rb /)
      kbo(:, 4,59, 8) = (/ &
        & 0.22278e-03_rb,0.20743e-02_rb,0.25302e-02_rb,0.24863e-02_rb,0.12161e-02_rb /)
      kbo(:, 5,59, 8) = (/ &
        & 0.32724e-03_rb,0.29797e-02_rb,0.36313e-02_rb,0.35664e-02_rb,0.16577e-02_rb /)
      kbo(:, 1,13, 9) = (/ &
        & 0.16403e+01_rb,0.27290e+01_rb,0.31326e+01_rb,0.31890e+01_rb,0.23307e+01_rb /)
      kbo(:, 2,13, 9) = (/ &
        & 0.16862e+01_rb,0.27611e+01_rb,0.31617e+01_rb,0.32226e+01_rb,0.23816e+01_rb /)
      kbo(:, 3,13, 9) = (/ &
        & 0.17455e+01_rb,0.28101e+01_rb,0.32005e+01_rb,0.32552e+01_rb,0.24291e+01_rb /)
      kbo(:, 4,13, 9) = (/ &
        & 0.18100e+01_rb,0.28758e+01_rb,0.32471e+01_rb,0.32901e+01_rb,0.24729e+01_rb /)
      kbo(:, 5,13, 9) = (/ &
        & 0.18736e+01_rb,0.29528e+01_rb,0.33027e+01_rb,0.33291e+01_rb,0.25126e+01_rb /)
      kbo(:, 1,14, 9) = (/ &
        & 0.14276e+01_rb,0.24116e+01_rb,0.27645e+01_rb,0.28207e+01_rb,0.20786e+01_rb /)
      kbo(:, 2,14, 9) = (/ &
        & 0.14761e+01_rb,0.24553e+01_rb,0.28027e+01_rb,0.28564e+01_rb,0.21275e+01_rb /)
      kbo(:, 3,14, 9) = (/ &
        & 0.15303e+01_rb,0.25161e+01_rb,0.28501e+01_rb,0.28940e+01_rb,0.21750e+01_rb /)
      kbo(:, 4,14, 9) = (/ &
        & 0.15845e+01_rb,0.25879e+01_rb,0.29061e+01_rb,0.29354e+01_rb,0.22167e+01_rb /)
      kbo(:, 5,14, 9) = (/ &
        & 0.16378e+01_rb,0.26632e+01_rb,0.29707e+01_rb,0.29827e+01_rb,0.22552e+01_rb /)
      kbo(:, 1,15, 9) = (/ &
        & 0.12454e+01_rb,0.21329e+01_rb,0.24382e+01_rb,0.24839e+01_rb,0.18332e+01_rb /)
      kbo(:, 2,15, 9) = (/ &
        & 0.12895e+01_rb,0.21869e+01_rb,0.24841e+01_rb,0.25234e+01_rb,0.18816e+01_rb /)
      kbo(:, 3,15, 9) = (/ &
        & 0.13344e+01_rb,0.22528e+01_rb,0.25385e+01_rb,0.25663e+01_rb,0.19251e+01_rb /)
      kbo(:, 4,15, 9) = (/ &
        & 0.13794e+01_rb,0.23234e+01_rb,0.26021e+01_rb,0.26153e+01_rb,0.19652e+01_rb /)
      kbo(:, 5,15, 9) = (/ &
        & 0.14250e+01_rb,0.23931e+01_rb,0.26727e+01_rb,0.26688e+01_rb,0.20043e+01_rb /)
      kbo(:, 1,16, 9) = (/ &
        & 0.10848e+01_rb,0.18880e+01_rb,0.21511e+01_rb,0.21813e+01_rb,0.16071e+01_rb /)
      kbo(:, 2,16, 9) = (/ &
        & 0.11214e+01_rb,0.19471e+01_rb,0.22027e+01_rb,0.22246e+01_rb,0.16520e+01_rb /)
      kbo(:, 3,16, 9) = (/ &
        & 0.11587e+01_rb,0.20122e+01_rb,0.22636e+01_rb,0.22737e+01_rb,0.16924e+01_rb /)
      kbo(:, 4,16, 9) = (/ &
        & 0.11968e+01_rb,0.20768e+01_rb,0.23313e+01_rb,0.23282e+01_rb,0.17327e+01_rb /)
      kbo(:, 5,16, 9) = (/ &
        & 0.12369e+01_rb,0.21393e+01_rb,0.24028e+01_rb,0.23861e+01_rb,0.17713e+01_rb /)
      kbo(:, 1,17, 9) = (/ &
        & 0.94054e+00_rb,0.16727e+01_rb,0.18972e+01_rb,0.19128e+01_rb,0.14046e+01_rb /)
      kbo(:, 2,17, 9) = (/ &
        & 0.97101e+00_rb,0.17314e+01_rb,0.19542e+01_rb,0.19610e+01_rb,0.14464e+01_rb /)
      kbo(:, 3,17, 9) = (/ &
        & 0.10024e+01_rb,0.17905e+01_rb,0.20183e+01_rb,0.20147e+01_rb,0.14867e+01_rb /)
      kbo(:, 4,17, 9) = (/ &
        & 0.10357e+01_rb,0.18485e+01_rb,0.20868e+01_rb,0.20722e+01_rb,0.15247e+01_rb /)
      kbo(:, 5,17, 9) = (/ &
        & 0.10722e+01_rb,0.19059e+01_rb,0.21551e+01_rb,0.21344e+01_rb,0.15620e+01_rb /)
      kbo(:, 1,18, 9) = (/ &
        & 0.81293e+00_rb,0.14815e+01_rb,0.16736e+01_rb,0.16776e+01_rb,0.12272e+01_rb /)
      kbo(:, 2,18, 9) = (/ &
        & 0.83847e+00_rb,0.15352e+01_rb,0.17337e+01_rb,0.17293e+01_rb,0.12665e+01_rb /)
      kbo(:, 3,18, 9) = (/ &
        & 0.86557e+00_rb,0.15883e+01_rb,0.17983e+01_rb,0.17854e+01_rb,0.13052e+01_rb /)
      kbo(:, 4,18, 9) = (/ &
        & 0.89549e+00_rb,0.16413e+01_rb,0.18633e+01_rb,0.18465e+01_rb,0.13420e+01_rb /)
      kbo(:, 5,18, 9) = (/ &
        & 0.92973e+00_rb,0.16952e+01_rb,0.19268e+01_rb,0.19102e+01_rb,0.13778e+01_rb /)
      kbo(:, 1,19, 9) = (/ &
        & 0.70038e+00_rb,0.13093e+01_rb,0.14779e+01_rb,0.14733e+01_rb,0.10709e+01_rb /)
      kbo(:, 2,19, 9) = (/ &
        & 0.72263e+00_rb,0.13577e+01_rb,0.15381e+01_rb,0.15268e+01_rb,0.11090e+01_rb /)
      kbo(:, 3,19, 9) = (/ &
        & 0.74709e+00_rb,0.14065e+01_rb,0.15995e+01_rb,0.15853e+01_rb,0.11457e+01_rb /)
      kbo(:, 4,19, 9) = (/ &
        & 0.77505e+00_rb,0.14562e+01_rb,0.16599e+01_rb,0.16467e+01_rb,0.11819e+01_rb /)
      kbo(:, 5,19, 9) = (/ &
        & 0.80778e+00_rb,0.15082e+01_rb,0.17205e+01_rb,0.17095e+01_rb,0.12170e+01_rb /)
      kbo(:, 1,20, 9) = (/ &
        & 0.60096e+00_rb,0.11549e+01_rb,0.13070e+01_rb,0.12966e+01_rb,0.93504e+00_rb /)
      kbo(:, 2,20, 9) = (/ &
        & 0.62117e+00_rb,0.11996e+01_rb,0.13644e+01_rb,0.13518e+01_rb,0.97145e+00_rb /)
      kbo(:, 3,20, 9) = (/ &
        & 0.64437e+00_rb,0.12456e+01_rb,0.14215e+01_rb,0.14105e+01_rb,0.10075e+01_rb /)
      kbo(:, 4,20, 9) = (/ &
        & 0.67172e+00_rb,0.12935e+01_rb,0.14792e+01_rb,0.14711e+01_rb,0.10425e+01_rb /)
      kbo(:, 5,20, 9) = (/ &
        & 0.70425e+00_rb,0.13447e+01_rb,0.15379e+01_rb,0.15320e+01_rb,0.10771e+01_rb /)
      kbo(:, 1,21, 9) = (/ &
        & 0.51409e+00_rb,0.10184e+01_rb,0.11562e+01_rb,0.11440e+01_rb,0.81589e+00_rb /)
      kbo(:, 2,21, 9) = (/ &
        & 0.53296e+00_rb,0.10604e+01_rb,0.12096e+01_rb,0.11991e+01_rb,0.85095e+00_rb /)
      kbo(:, 3,21, 9) = (/ &
        & 0.55527e+00_rb,0.11043e+01_rb,0.12638e+01_rb,0.12567e+01_rb,0.88570e+00_rb /)
      kbo(:, 4,21, 9) = (/ &
        & 0.58243e+00_rb,0.11512e+01_rb,0.13194e+01_rb,0.13151e+01_rb,0.92023e+00_rb /)
      kbo(:, 5,21, 9) = (/ &
        & 0.61501e+00_rb,0.12017e+01_rb,0.13771e+01_rb,0.13737e+01_rb,0.95470e+00_rb /)
      kbo(:, 1,22, 9) = (/ &
        & 0.44038e+00_rb,0.90186e+00_rb,0.10262e+01_rb,0.10153e+01_rb,0.71382e+00_rb /)
      kbo(:, 2,22, 9) = (/ &
        & 0.45853e+00_rb,0.94198e+00_rb,0.10770e+01_rb,0.10695e+01_rb,0.74736e+00_rb /)
      kbo(:, 3,22, 9) = (/ &
        & 0.48068e+00_rb,0.98449e+00_rb,0.11290e+01_rb,0.11248e+01_rb,0.78150e+00_rb /)
      kbo(:, 4,22, 9) = (/ &
        & 0.50812e+00_rb,0.10306e+01_rb,0.11833e+01_rb,0.11809e+01_rb,0.81518e+00_rb /)
      kbo(:, 5,22, 9) = (/ &
        & 0.54141e+00_rb,0.10817e+01_rb,0.12405e+01_rb,0.12383e+01_rb,0.84849e+00_rb /)
      kbo(:, 1,23, 9) = (/ &
        & 0.37767e+00_rb,0.80078e+00_rb,0.91306e+00_rb,0.90433e+00_rb,0.62612e+00_rb /)
      kbo(:, 2,23, 9) = (/ &
        & 0.39558e+00_rb,0.83957e+00_rb,0.96154e+00_rb,0.95658e+00_rb,0.65837e+00_rb /)
      kbo(:, 3,23, 9) = (/ &
        & 0.41794e+00_rb,0.88149e+00_rb,0.10123e+01_rb,0.10097e+01_rb,0.69117e+00_rb /)
      kbo(:, 4,23, 9) = (/ &
        & 0.44579e+00_rb,0.92784e+00_rb,0.10657e+01_rb,0.10643e+01_rb,0.72352e+00_rb /)
      kbo(:, 5,23, 9) = (/ &
        & 0.47930e+00_rb,0.97981e+00_rb,0.11229e+01_rb,0.11209e+01_rb,0.75697e+00_rb /)
      kbo(:, 1,24, 9) = (/ &
        & 0.32501e+00_rb,0.71314e+00_rb,0.81551e+00_rb,0.80817e+00_rb,0.55055e+00_rb /)
      kbo(:, 2,24, 9) = (/ &
        & 0.34301e+00_rb,0.75171e+00_rb,0.86229e+00_rb,0.85830e+00_rb,0.58144e+00_rb /)
      kbo(:, 3,24, 9) = (/ &
        & 0.36572e+00_rb,0.79385e+00_rb,0.91196e+00_rb,0.91014e+00_rb,0.61242e+00_rb /)
      kbo(:, 4,24, 9) = (/ &
        & 0.39391e+00_rb,0.84084e+00_rb,0.96525e+00_rb,0.96386e+00_rb,0.64468e+00_rb /)
      kbo(:, 5,24, 9) = (/ &
        & 0.42682e+00_rb,0.89378e+00_rb,0.10230e+01_rb,0.10202e+01_rb,0.67755e+00_rb /)
      kbo(:, 1,25, 9) = (/ &
        & 0.28120e+00_rb,0.63782e+00_rb,0.73197e+00_rb,0.72532e+00_rb,0.48624e+00_rb /)
      kbo(:, 2,25, 9) = (/ &
        & 0.29939e+00_rb,0.67672e+00_rb,0.77772e+00_rb,0.77395e+00_rb,0.51566e+00_rb /)
      kbo(:, 3,25, 9) = (/ &
        & 0.32259e+00_rb,0.71969e+00_rb,0.82682e+00_rb,0.82484e+00_rb,0.54562e+00_rb /)
      kbo(:, 4,25, 9) = (/ &
        & 0.35054e+00_rb,0.76785e+00_rb,0.88040e+00_rb,0.87836e+00_rb,0.57709e+00_rb /)
      kbo(:, 5,25, 9) = (/ &
        & 0.38231e+00_rb,0.82200e+00_rb,0.93896e+00_rb,0.93503e+00_rb,0.60997e+00_rb /)
      kbo(:, 1,26, 9) = (/ &
        & 0.24516e+00_rb,0.57415e+00_rb,0.66146e+00_rb,0.65501e+00_rb,0.43198e+00_rb /)
      kbo(:, 2,26, 9) = (/ &
        & 0.26380e+00_rb,0.61364e+00_rb,0.70651e+00_rb,0.70261e+00_rb,0.46025e+00_rb /)
      kbo(:, 3,26, 9) = (/ &
        & 0.28716e+00_rb,0.65790e+00_rb,0.75577e+00_rb,0.75300e+00_rb,0.48987e+00_rb /)
      kbo(:, 4,26, 9) = (/ &
        & 0.31438e+00_rb,0.70747e+00_rb,0.80990e+00_rb,0.80672e+00_rb,0.52086e+00_rb /)
      kbo(:, 5,26, 9) = (/ &
        & 0.34509e+00_rb,0.76331e+00_rb,0.86957e+00_rb,0.86437e+00_rb,0.55315e+00_rb /)
      kbo(:, 1,27, 9) = (/ &
        & 0.21544e+00_rb,0.52037e+00_rb,0.60144e+00_rb,0.59535e+00_rb,0.38600e+00_rb /)
      kbo(:, 2,27, 9) = (/ &
        & 0.23456e+00_rb,0.56067e+00_rb,0.64661e+00_rb,0.64229e+00_rb,0.41342e+00_rb /)
      kbo(:, 3,27, 9) = (/ &
        & 0.25766e+00_rb,0.60633e+00_rb,0.69633e+00_rb,0.69257e+00_rb,0.44235e+00_rb /)
      kbo(:, 4,27, 9) = (/ &
        & 0.28405e+00_rb,0.65751e+00_rb,0.75133e+00_rb,0.74692e+00_rb,0.47298e+00_rb /)
      kbo(:, 5,27, 9) = (/ &
        & 0.31370e+00_rb,0.71529e+00_rb,0.81234e+00_rb,0.80601e+00_rb,0.50534e+00_rb /)
      kbo(:, 1,28, 9) = (/ &
        & 0.19107e+00_rb,0.47529e+00_rb,0.55077e+00_rb,0.54503e+00_rb,0.34666e+00_rb /)
      kbo(:, 2,28, 9) = (/ &
        & 0.21041e+00_rb,0.51675e+00_rb,0.59653e+00_rb,0.59172e+00_rb,0.37350e+00_rb /)
      kbo(:, 3,28, 9) = (/ &
        & 0.23301e+00_rb,0.56364e+00_rb,0.64698e+00_rb,0.64221e+00_rb,0.40186e+00_rb /)
      kbo(:, 4,28, 9) = (/ &
        & 0.25866e+00_rb,0.61667e+00_rb,0.70320e+00_rb,0.69763e+00_rb,0.43205e+00_rb /)
      kbo(:, 5,28, 9) = (/ &
        & 0.28710e+00_rb,0.67650e+00_rb,0.76586e+00_rb,0.75834e+00_rb,0.46367e+00_rb /)
      kbo(:, 1,29, 9) = (/ &
        & 0.17112e+00_rb,0.43830e+00_rb,0.50881e+00_rb,0.50311e+00_rb,0.31371e+00_rb /)
      kbo(:, 2,29, 9) = (/ &
        & 0.19046e+00_rb,0.48081e+00_rb,0.55535e+00_rb,0.54996e+00_rb,0.34018e+00_rb /)
      kbo(:, 3,29, 9) = (/ &
        & 0.21264e+00_rb,0.52926e+00_rb,0.60692e+00_rb,0.60125e+00_rb,0.36829e+00_rb /)
      kbo(:, 4,29, 9) = (/ &
        & 0.23734e+00_rb,0.58428e+00_rb,0.66473e+00_rb,0.65816e+00_rb,0.39807e+00_rb /)
      kbo(:, 5,29, 9) = (/ &
        & 0.26483e+00_rb,0.64604e+00_rb,0.72961e+00_rb,0.72085e+00_rb,0.42916e+00_rb /)
      kbo(:, 1,30, 9) = (/ &
        & 0.15474e+00_rb,0.40804e+00_rb,0.47416e+00_rb,0.46844e+00_rb,0.28591e+00_rb /)
      kbo(:, 2,30, 9) = (/ &
        & 0.17385e+00_rb,0.45175e+00_rb,0.52179e+00_rb,0.51582e+00_rb,0.31208e+00_rb /)
      kbo(:, 3,30, 9) = (/ &
        & 0.19550e+00_rb,0.50197e+00_rb,0.57484e+00_rb,0.56852e+00_rb,0.33963e+00_rb /)
      kbo(:, 4,30, 9) = (/ &
        & 0.21942e+00_rb,0.55897e+00_rb,0.63459e+00_rb,0.62718e+00_rb,0.36926e+00_rb /)
      kbo(:, 5,30, 9) = (/ &
        & 0.24626e+00_rb,0.62254e+00_rb,0.70206e+00_rb,0.69222e+00_rb,0.40317e+00_rb /)
      kbo(:, 1,31, 9) = (/ &
        & 0.14123e+00_rb,0.38383e+00_rb,0.44627e+00_rb,0.44042e+00_rb,0.26282e+00_rb /)
      kbo(:, 2,31, 9) = (/ &
        & 0.16008e+00_rb,0.42905e+00_rb,0.49523e+00_rb,0.48891e+00_rb,0.28832e+00_rb /)
      kbo(:, 3,31, 9) = (/ &
        & 0.18115e+00_rb,0.48120e+00_rb,0.55018e+00_rb,0.54338e+00_rb,0.31669e+00_rb /)
      kbo(:, 4,31, 9) = (/ &
        & 0.20456e+00_rb,0.54014e+00_rb,0.61227e+00_rb,0.60422e+00_rb,0.34861e+00_rb /)
      kbo(:, 5,31, 9) = (/ &
        & 0.23090e+00_rb,0.60549e+00_rb,0.68253e+00_rb,0.67193e+00_rb,0.38334e+00_rb /)
      kbo(:, 1,32, 9) = (/ &
        & 0.13011e+00_rb,0.36494e+00_rb,0.42434e+00_rb,0.41840e+00_rb,0.24343e+00_rb /)
      kbo(:, 2,32, 9) = (/ &
        & 0.14859e+00_rb,0.41192e+00_rb,0.47502e+00_rb,0.46846e+00_rb,0.27034e+00_rb /)
      kbo(:, 3,32, 9) = (/ &
        & 0.16925e+00_rb,0.46604e+00_rb,0.53210e+00_rb,0.52502e+00_rb,0.30012e+00_rb /)
      kbo(:, 4,32, 9) = (/ &
        & 0.19229e+00_rb,0.52692e+00_rb,0.59693e+00_rb,0.58841e+00_rb,0.33258e+00_rb /)
      kbo(:, 5,32, 9) = (/ &
        & 0.21839e+00_rb,0.59419e+00_rb,0.67028e+00_rb,0.65920e+00_rb,0.36819e+00_rb /)
      kbo(:, 1,33, 9) = (/ &
        & 0.12092e+00_rb,0.35074e+00_rb,0.40776e+00_rb,0.40175e+00_rb,0.22934e+00_rb /)
      kbo(:, 2,33, 9) = (/ &
        & 0.13904e+00_rb,0.39967e+00_rb,0.46044e+00_rb,0.45383e+00_rb,0.25698e+00_rb /)
      kbo(:, 3,33, 9) = (/ &
        & 0.15944e+00_rb,0.45574e+00_rb,0.51998e+00_rb,0.51285e+00_rb,0.28727e+00_rb /)
      kbo(:, 4,33, 9) = (/ &
        & 0.18230e+00_rb,0.51859e+00_rb,0.58782e+00_rb,0.57914e+00_rb,0.32052e+00_rb /)
      kbo(:, 5,33, 9) = (/ &
        & 0.20843e+00_rb,0.58793e+00_rb,0.66452e+00_rb,0.65307e+00_rb,0.35723e+00_rb /)
      kbo(:, 1,34, 9) = (/ &
        & 0.11270e+00_rb,0.33906e+00_rb,0.39419e+00_rb,0.38828e+00_rb,0.21761e+00_rb /)
      kbo(:, 2,34, 9) = (/ &
        & 0.13054e+00_rb,0.38970e+00_rb,0.44887e+00_rb,0.44251e+00_rb,0.24563e+00_rb /)
      kbo(:, 3,34, 9) = (/ &
        & 0.15070e+00_rb,0.44751e+00_rb,0.51092e+00_rb,0.50399e+00_rb,0.27652e+00_rb /)
      kbo(:, 4,34, 9) = (/ &
        & 0.17349e+00_rb,0.51224e+00_rb,0.58169e+00_rb,0.57301e+00_rb,0.31064e+00_rb /)
      kbo(:, 5,34, 9) = (/ &
        & 0.19973e+00_rb,0.58363e+00_rb,0.66163e+00_rb,0.64989e+00_rb,0.34860e+00_rb /)
      kbo(:, 1,35, 9) = (/ &
        & 0.10431e+00_rb,0.32621e+00_rb,0.37969e+00_rb,0.37421e+00_rb,0.20589e+00_rb /)
      kbo(:, 2,35, 9) = (/ &
        & 0.12172e+00_rb,0.37794e+00_rb,0.43594e+00_rb,0.43005e+00_rb,0.23415e+00_rb /)
      kbo(:, 3,35, 9) = (/ &
        & 0.14155e+00_rb,0.43696e+00_rb,0.49993e+00_rb,0.49338e+00_rb,0.26546e+00_rb /)
      kbo(:, 4,35, 9) = (/ &
        & 0.16416e+00_rb,0.50303e+00_rb,0.57285e+00_rb,0.56442e+00_rb,0.30031e+00_rb /)
      kbo(:, 5,35, 9) = (/ &
        & 0.19033e+00_rb,0.57592e+00_rb,0.65509e+00_rb,0.64342e+00_rb,0.33926e+00_rb /)
      kbo(:, 1,36, 9) = (/ &
        & 0.95441e-01_rb,0.31093e+00_rb,0.36283e+00_rb,0.35795e+00_rb,0.19348e+00_rb /)
      kbo(:, 2,36, 9) = (/ &
        & 0.11226e+00_rb,0.36299e+00_rb,0.41996e+00_rb,0.41468e+00_rb,0.22173e+00_rb /)
      kbo(:, 3,36, 9) = (/ &
        & 0.13160e+00_rb,0.42251e+00_rb,0.48508e+00_rb,0.47906e+00_rb,0.25327e+00_rb /)
      kbo(:, 4,36, 9) = (/ &
        & 0.15379e+00_rb,0.48924e+00_rb,0.55909e+00_rb,0.55119e+00_rb,0.28853e+00_rb /)
      kbo(:, 5,36, 9) = (/ &
        & 0.17959e+00_rb,0.56298e+00_rb,0.64257e+00_rb,0.63136e+00_rb,0.32808e+00_rb /)
      kbo(:, 1,37, 9) = (/ &
        & 0.85477e-01_rb,0.29072e+00_rb,0.34067e+00_rb,0.33658e+00_rb,0.17886e+00_rb /)
      kbo(:, 2,37, 9) = (/ &
        & 0.10144e+00_rb,0.34200e+00_rb,0.39763e+00_rb,0.39300e+00_rb,0.20669e+00_rb /)
      kbo(:, 3,37, 9) = (/ &
        & 0.11996e+00_rb,0.40095e+00_rb,0.46252e+00_rb,0.45719e+00_rb,0.23796e+00_rb /)
      kbo(:, 4,37, 9) = (/ &
        & 0.14137e+00_rb,0.46732e+00_rb,0.53621e+00_rb,0.52913e+00_rb,0.27304e+00_rb /)
      kbo(:, 5,37, 9) = (/ &
        & 0.16629e+00_rb,0.54086e+00_rb,0.61939e+00_rb,0.60918e+00_rb,0.31246e+00_rb /)
      kbo(:, 1,38, 9) = (/ &
        & 0.76626e-01_rb,0.27223e+00_rb,0.32047e+00_rb,0.31724e+00_rb,0.16576e+00_rb /)
      kbo(:, 2,38, 9) = (/ &
        & 0.91774e-01_rb,0.32273e+00_rb,0.37721e+00_rb,0.37319e+00_rb,0.19315e+00_rb /)
      kbo(:, 3,38, 9) = (/ &
        & 0.10950e+00_rb,0.38104e+00_rb,0.44177e+00_rb,0.43708e+00_rb,0.22413e+00_rb /)
      kbo(:, 4,38, 9) = (/ &
        & 0.13015e+00_rb,0.44697e+00_rb,0.51498e+00_rb,0.50870e+00_rb,0.25903e+00_rb /)
      kbo(:, 5,38, 9) = (/ &
        & 0.15424e+00_rb,0.52018e+00_rb,0.59766e+00_rb,0.58842e+00_rb,0.29827e+00_rb /)
      kbo(:, 1,39, 9) = (/ &
        & 0.68857e-01_rb,0.25564e+00_rb,0.30247e+00_rb,0.30005e+00_rb,0.15419e+00_rb /)
      kbo(:, 2,39, 9) = (/ &
        & 0.83231e-01_rb,0.30541e+00_rb,0.35891e+00_rb,0.35551e+00_rb,0.18117e+00_rb /)
      kbo(:, 3,39, 9) = (/ &
        & 0.10022e+00_rb,0.36305e+00_rb,0.42306e+00_rb,0.41901e+00_rb,0.21190e+00_rb /)
      kbo(:, 4,39, 9) = (/ &
        & 0.12015e+00_rb,0.42854e+00_rb,0.49574e+00_rb,0.49023e+00_rb,0.24662e+00_rb /)
      kbo(:, 5,39, 9) = (/ &
        & 0.14353e+00_rb,0.50140e+00_rb,0.57784e+00_rb,0.56950e+00_rb,0.28564e+00_rb /)
      kbo(:, 1,40, 9) = (/ &
        & 0.60721e-01_rb,0.23607e+00_rb,0.28098e+00_rb,0.27949e+00_rb,0.14134e+00_rb /)
      kbo(:, 2,40, 9) = (/ &
        & 0.74142e-01_rb,0.28436e+00_rb,0.33629e+00_rb,0.33356e+00_rb,0.16758e+00_rb /)
      kbo(:, 3,40, 9) = (/ &
        & 0.90161e-01_rb,0.34051e+00_rb,0.39918e+00_rb,0.39576e+00_rb,0.19765e+00_rb /)
      kbo(:, 4,40, 9) = (/ &
        & 0.10909e+00_rb,0.40469e+00_rb,0.47035e+00_rb,0.46571e+00_rb,0.23173e+00_rb /)
      kbo(:, 5,40, 9) = (/ &
        & 0.13146e+00_rb,0.47643e+00_rb,0.55083e+00_rb,0.54364e+00_rb,0.27006e+00_rb /)
      kbo(:, 1,41, 9) = (/ &
        & 0.53403e-01_rb,0.21760e+00_rb,0.26060e+00_rb,0.26004e+00_rb,0.12944e+00_rb /)
      kbo(:, 2,41, 9) = (/ &
        & 0.65918e-01_rb,0.26438e+00_rb,0.31466e+00_rb,0.31265e+00_rb,0.15490e+00_rb /)
      kbo(:, 3,41, 9) = (/ &
        & 0.80936e-01_rb,0.31888e+00_rb,0.37609e+00_rb,0.37327e+00_rb,0.18421e+00_rb /)
      kbo(:, 4,41, 9) = (/ &
        & 0.98889e-01_rb,0.38155e+00_rb,0.44565e+00_rb,0.44176e+00_rb,0.21756e+00_rb /)
      kbo(:, 5,41, 9) = (/ &
        & 0.12024e+00_rb,0.45195e+00_rb,0.52424e+00_rb,0.51812e+00_rb,0.25511e+00_rb /)
      kbo(:, 1,42, 9) = (/ &
        & 0.46980e-01_rb,0.20072e+00_rb,0.24188e+00_rb,0.24222e+00_rb,0.11863e+00_rb /)
      kbo(:, 2,42, 9) = (/ &
        & 0.58572e-01_rb,0.24586e+00_rb,0.29443e+00_rb,0.29314e+00_rb,0.14330e+00_rb /)
      kbo(:, 3,42, 9) = (/ &
        & 0.72653e-01_rb,0.29875e+00_rb,0.35445e+00_rb,0.35212e+00_rb,0.17184e+00_rb /)
      kbo(:, 4,42, 9) = (/ &
        & 0.89663e-01_rb,0.35979e+00_rb,0.42236e+00_rb,0.41911e+00_rb,0.20443e+00_rb /)
      kbo(:, 5,42, 9) = (/ &
        & 0.11004e+00_rb,0.42873e+00_rb,0.49904e+00_rb,0.49389e+00_rb,0.24118e+00_rb /)
      kbo(:, 1,43, 9) = (/ &
        & 0.40748e-01_rb,0.18301e+00_rb,0.22189e+00_rb,0.22309e+00_rb,0.10752e+00_rb /)
      kbo(:, 2,43, 9) = (/ &
        & 0.51357e-01_rb,0.22603e+00_rb,0.27246e+00_rb,0.27198e+00_rb,0.13119e+00_rb /)
      kbo(:, 3,43, 9) = (/ &
        & 0.64381e-01_rb,0.27684e+00_rb,0.33058e+00_rb,0.32881e+00_rb,0.15875e+00_rb /)
      kbo(:, 4,43, 9) = (/ &
        & 0.80299e-01_rb,0.33572e+00_rb,0.39639e+00_rb,0.39376e+00_rb,0.19029e+00_rb /)
      kbo(:, 5,43, 9) = (/ &
        & 0.99551e-01_rb,0.40267e+00_rb,0.47065e+00_rb,0.46647e+00_rb,0.22598e+00_rb /)
      kbo(:, 1,44, 9) = (/ &
        & 0.35065e-01_rb,0.16586e+00_rb,0.20221e+00_rb,0.20417e+00_rb,0.96831e-01_rb /)
      kbo(:, 2,44, 9) = (/ &
        & 0.44686e-01_rb,0.20650e+00_rb,0.25055e+00_rb,0.25097e+00_rb,0.11942e+00_rb /)
      kbo(:, 3,44, 9) = (/ &
        & 0.56631e-01_rb,0.25500e+00_rb,0.30652e+00_rb,0.30540e+00_rb,0.14585e+00_rb /)
      kbo(:, 4,44, 9) = (/ &
        & 0.71442e-01_rb,0.31160e+00_rb,0.37021e+00_rb,0.36812e+00_rb,0.17630e+00_rb /)
      kbo(:, 5,44, 9) = (/ &
        & 0.89487e-01_rb,0.37611e+00_rb,0.44177e+00_rb,0.43843e+00_rb,0.21076e+00_rb /)
      kbo(:, 1,45, 9) = (/ &
        & 0.30099e-01_rb,0.15014e+00_rb,0.18399e+00_rb,0.18651e+00_rb,0.87081e-01_rb /)
      kbo(:, 2,45, 9) = (/ &
        & 0.38792e-01_rb,0.18842e+00_rb,0.23006e+00_rb,0.23134e+00_rb,0.10857e+00_rb /)
      kbo(:, 3,45, 9) = (/ &
        & 0.49706e-01_rb,0.23455e+00_rb,0.28377e+00_rb,0.28337e+00_rb,0.13385e+00_rb /)
      kbo(:, 4,45, 9) = (/ &
        & 0.63416e-01_rb,0.28880e+00_rb,0.34526e+00_rb,0.34365e+00_rb,0.16313e+00_rb /)
      kbo(:, 5,45, 9) = (/ &
        & 0.80320e-01_rb,0.35087e+00_rb,0.41430e+00_rb,0.41165e+00_rb,0.19641e+00_rb /)
      kbo(:, 1,46, 9) = (/ &
        & 0.25615e-01_rb,0.13501e+00_rb,0.16621e+00_rb,0.16909e+00_rb,0.77761e-01_rb /)
      kbo(:, 2,46, 9) = (/ &
        & 0.33396e-01_rb,0.17087e+00_rb,0.20986e+00_rb,0.21193e+00_rb,0.98069e-01_rb /)
      kbo(:, 3,46, 9) = (/ &
        & 0.43320e-01_rb,0.21451e+00_rb,0.26120e+00_rb,0.26156e+00_rb,0.12212e+00_rb /)
      kbo(:, 4,46, 9) = (/ &
        & 0.55857e-01_rb,0.26606e+00_rb,0.32018e+00_rb,0.31908e+00_rb,0.15012e+00_rb /)
      kbo(:, 5,46, 9) = (/ &
        & 0.71564e-01_rb,0.32546e+00_rb,0.38657e+00_rb,0.38454e+00_rb,0.18211e+00_rb /)
      kbo(:, 1,47, 9) = (/ &
        & 0.21470e-01_rb,0.11984e+00_rb,0.14813e+00_rb,0.15126e+00_rb,0.68564e-01_rb /)
      kbo(:, 2,47, 9) = (/ &
        & 0.28327e-01_rb,0.15313e+00_rb,0.18908e+00_rb,0.19182e+00_rb,0.87445e-01_rb /)
      kbo(:, 3,47, 9) = (/ &
        & 0.37186e-01_rb,0.19381e+00_rb,0.23756e+00_rb,0.23882e+00_rb,0.11016e+00_rb /)
      kbo(:, 4,47, 9) = (/ &
        & 0.48514e-01_rb,0.24239e+00_rb,0.29373e+00_rb,0.29331e+00_rb,0.13670e+00_rb /)
      kbo(:, 5,47, 9) = (/ &
        & 0.62915e-01_rb,0.29879e+00_rb,0.35729e+00_rb,0.35580e+00_rb,0.16718e+00_rb /)
      kbo(:, 1,48, 9) = (/ &
        & 0.17915e-01_rb,0.10594e+00_rb,0.13153e+00_rb,0.13471e+00_rb,0.60247e-01_rb /)
      kbo(:, 2,48, 9) = (/ &
        & 0.23922e-01_rb,0.13687e+00_rb,0.16982e+00_rb,0.17297e+00_rb,0.77702e-01_rb /)
      kbo(:, 3,48, 9) = (/ &
        & 0.31789e-01_rb,0.17464e+00_rb,0.21543e+00_rb,0.21754e+00_rb,0.99032e-01_rb /)
      kbo(:, 4,48, 9) = (/ &
        & 0.41975e-01_rb,0.22024e+00_rb,0.26872e+00_rb,0.26906e+00_rb,0.12414e+00_rb /)
      kbo(:, 5,48, 9) = (/ &
        & 0.55099e-01_rb,0.27366e+00_rb,0.32950e+00_rb,0.32848e+00_rb,0.15313e+00_rb /)
      kbo(:, 1,49, 9) = (/ &
        & 0.14883e-01_rb,0.93224e-01_rb,0.11635e+00_rb,0.11939e+00_rb,0.52749e-01_rb /)
      kbo(:, 2,49, 9) = (/ &
        & 0.20112e-01_rb,0.12194e+00_rb,0.15191e+00_rb,0.15532e+00_rb,0.68811e-01_rb /)
      kbo(:, 3,49, 9) = (/ &
        & 0.27052e-01_rb,0.15700e+00_rb,0.19473e+00_rb,0.19754e+00_rb,0.88692e-01_rb /)
      kbo(:, 4,49, 9) = (/ &
        & 0.36170e-01_rb,0.19958e+00_rb,0.24512e+00_rb,0.24626e+00_rb,0.11239e+00_rb /)
      kbo(:, 5,49, 9) = (/ &
        & 0.48053e-01_rb,0.24998e+00_rb,0.30305e+00_rb,0.30260e+00_rb,0.13992e+00_rb /)
      kbo(:, 1,50, 9) = (/ &
        & 0.12401e-01_rb,0.82224e-01_rb,0.10323e+00_rb,0.10603e+00_rb,0.46388e-01_rb /)
      kbo(:, 2,50, 9) = (/ &
        & 0.16961e-01_rb,0.10896e+00_rb,0.13629e+00_rb,0.13978e+00_rb,0.61130e-01_rb /)
      kbo(:, 3,50, 9) = (/ &
        & 0.23097e-01_rb,0.14168e+00_rb,0.17654e+00_rb,0.17982e+00_rb,0.79649e-01_rb /)
      kbo(:, 4,50, 9) = (/ &
        & 0.31266e-01_rb,0.18143e+00_rb,0.22418e+00_rb,0.22611e+00_rb,0.10201e+00_rb /)
      kbo(:, 5,50, 9) = (/ &
        & 0.42049e-01_rb,0.22900e+00_rb,0.27939e+00_rb,0.27959e+00_rb,0.12818e+00_rb /)
      kbo(:, 1,51, 9) = (/ &
        & 0.10311e-01_rb,0.72407e-01_rb,0.91543e-01_rb,0.94015e-01_rb,0.40739e-01_rb /)
      kbo(:, 2,51, 9) = (/ &
        & 0.14299e-01_rb,0.97301e-01_rb,0.12223e+00_rb,0.12566e+00_rb,0.54336e-01_rb /)
      kbo(:, 3,51, 9) = (/ &
        & 0.19709e-01_rb,0.12788e+00_rb,0.16001e+00_rb,0.16354e+00_rb,0.71514e-01_rb /)
      kbo(:, 4,51, 9) = (/ &
        & 0.27010e-01_rb,0.16504e+00_rb,0.20502e+00_rb,0.20766e+00_rb,0.92576e-01_rb /)
      kbo(:, 5,51, 9) = (/ &
        & 0.36820e-01_rb,0.20996e+00_rb,0.25769e+00_rb,0.25854e+00_rb,0.11743e+00_rb /)
      kbo(:, 1,52, 9) = (/ &
        & 0.85238e-02_rb,0.63439e-01_rb,0.80796e-01_rb,0.82950e-01_rb,0.35726e-01_rb /)
      kbo(:, 2,52, 9) = (/ &
        & 0.12004e-01_rb,0.86535e-01_rb,0.10929e+00_rb,0.11252e+00_rb,0.48201e-01_rb /)
      kbo(:, 3,52, 9) = (/ &
        & 0.16756e-01_rb,0.11511e+00_rb,0.14456e+00_rb,0.14824e+00_rb,0.64036e-01_rb /)
      kbo(:, 4,52, 9) = (/ &
        & 0.23247e-01_rb,0.14984e+00_rb,0.18701e+00_rb,0.19020e+00_rb,0.83745e-01_rb /)
      kbo(:, 5,52, 9) = (/ &
        & 0.32110e-01_rb,0.19199e+00_rb,0.23699e+00_rb,0.23857e+00_rb,0.10729e+00_rb /)
      kbo(:, 1,53, 9) = (/ &
        & 0.70021e-02_rb,0.55279e-01_rb,0.70908e-01_rb,0.72826e-01_rb,0.31267e-01_rb /)
      kbo(:, 2,53, 9) = (/ &
        & 0.10022e-01_rb,0.76605e-01_rb,0.97337e-01_rb,0.10026e+00_rb,0.42555e-01_rb /)
      kbo(:, 3,53, 9) = (/ &
        & 0.14186e-01_rb,0.10324e+00_rb,0.13015e+00_rb,0.13384e+00_rb,0.57176e-01_rb /)
      kbo(:, 4,53, 9) = (/ &
        & 0.19924e-01_rb,0.13568e+00_rb,0.17006e+00_rb,0.17360e+00_rb,0.75508e-01_rb /)
      kbo(:, 5,53, 9) = (/ &
        & 0.27873e-01_rb,0.17508e+00_rb,0.21734e+00_rb,0.21965e+00_rb,0.97730e-01_rb /)
      kbo(:, 1,54, 9) = (/ &
        & 0.57827e-02_rb,0.48416e-01_rb,0.62508e-01_rb,0.64310e-01_rb,0.27618e-01_rb /)
      kbo(:, 2,54, 9) = (/ &
        & 0.84043e-02_rb,0.68151e-01_rb,0.87167e-01_rb,0.89761e-01_rb,0.37874e-01_rb /)
      kbo(:, 3,54, 9) = (/ &
        & 0.12088e-01_rb,0.93069e-01_rb,0.11782e+00_rb,0.12138e+00_rb,0.51418e-01_rb /)
      kbo(:, 4,54, 9) = (/ &
        & 0.17188e-01_rb,0.12356e+00_rb,0.15542e+00_rb,0.15914e+00_rb,0.68446e-01_rb /)
      kbo(:, 5,54, 9) = (/ &
        & 0.24356e-01_rb,0.16057e+00_rb,0.20027e+00_rb,0.20317e+00_rb,0.89427e-01_rb /)
      kbo(:, 1,55, 9) = (/ &
        & 0.47829e-02_rb,0.42445e-01_rb,0.55108e-01_rb,0.56879e-01_rb,0.24415e-01_rb /)
      kbo(:, 2,55, 9) = (/ &
        & 0.70515e-02_rb,0.60688e-01_rb,0.78130e-01_rb,0.80430e-01_rb,0.33849e-01_rb /)
      kbo(:, 3,55, 9) = (/ &
        & 0.10307e-01_rb,0.83954e-01_rb,0.10678e+00_rb,0.11014e+00_rb,0.46263e-01_rb /)
      kbo(:, 4,55, 9) = (/ &
        & 0.14854e-01_rb,0.11267e+00_rb,0.14216e+00_rb,0.14598e+00_rb,0.62158e-01_rb /)
      kbo(:, 5,55, 9) = (/ &
        & 0.21317e-01_rb,0.14754e+00_rb,0.18474e+00_rb,0.18809e+00_rb,0.81914e-01_rb /)
      kbo(:, 1,56, 9) = (/ &
        & 0.39349e-02_rb,0.37013e-01_rb,0.48295e-01_rb,0.50091e-01_rb,0.21529e-01_rb /)
      kbo(:, 2,56, 9) = (/ &
        & 0.58808e-02_rb,0.53785e-01_rb,0.69695e-01_rb,0.71776e-01_rb,0.30144e-01_rb /)
      kbo(:, 3,56, 9) = (/ &
        & 0.87470e-02_rb,0.75476e-01_rb,0.96519e-01_rb,0.99590e-01_rb,0.41605e-01_rb /)
      kbo(:, 4,56, 9) = (/ &
        & 0.12799e-01_rb,0.10247e+00_rb,0.12972e+00_rb,0.13354e+00_rb,0.56396e-01_rb /)
      kbo(:, 5,56, 9) = (/ &
        & 0.18597e-01_rb,0.13533e+00_rb,0.17007e+00_rb,0.17370e+00_rb,0.74854e-01_rb /)
      kbo(:, 1,57, 9) = (/ &
        & 0.32237e-02_rb,0.32103e-01_rb,0.42068e-01_rb,0.43893e-01_rb,0.18889e-01_rb /)
      kbo(:, 2,57, 9) = (/ &
        & 0.48776e-02_rb,0.47453e-01_rb,0.61873e-01_rb,0.63812e-01_rb,0.26899e-01_rb /)
      kbo(:, 3,57, 9) = (/ &
        & 0.73816e-02_rb,0.67586e-01_rb,0.86956e-01_rb,0.89701e-01_rb,0.37405e-01_rb /)
      kbo(:, 4,57, 9) = (/ &
        & 0.10984e-01_rb,0.92907e-01_rb,0.11806e+00_rb,0.12175e+00_rb,0.51002e-01_rb /)
      kbo(:, 5,57, 9) = (/ &
        & 0.16168e-01_rb,0.12385e+00_rb,0.15617e+00_rb,0.15996e+00_rb,0.68253e-01_rb /)
      kbo(:, 1,58, 9) = (/ &
        & 0.26462e-02_rb,0.27879e-01_rb,0.36685e-01_rb,0.38484e-01_rb,0.16601e-01_rb /)
      kbo(:, 2,58, 9) = (/ &
        & 0.40522e-02_rb,0.41929e-01_rb,0.54967e-01_rb,0.56858e-01_rb,0.24050e-01_rb /)
      kbo(:, 3,58, 9) = (/ &
        & 0.62364e-02_rb,0.60609e-01_rb,0.78458e-01_rb,0.80919e-01_rb,0.33657e-01_rb /)
      kbo(:, 4,58, 9) = (/ &
        & 0.94512e-02_rb,0.84383e-01_rb,0.10769e+00_rb,0.11117e+00_rb,0.46338e-01_rb /)
      kbo(:, 5,58, 9) = (/ &
        & 0.14101e-01_rb,0.11363e+00_rb,0.14367e+00_rb,0.14755e+00_rb,0.62487e-01_rb /)
      kbo(:, 1,59, 9) = (/ &
        & 0.23375e-02_rb,0.26104e-01_rb,0.34483e-01_rb,0.36273e-01_rb,0.15643e-01_rb /)
      kbo(:, 2,59, 9) = (/ &
        & 0.36353e-02_rb,0.39676e-01_rb,0.52191e-01_rb,0.54098e-01_rb,0.22879e-01_rb /)
      kbo(:, 3,59, 9) = (/ &
        & 0.56781e-02_rb,0.57810e-01_rb,0.75085e-01_rb,0.77452e-01_rb,0.32253e-01_rb /)
      kbo(:, 4,59, 9) = (/ &
        & 0.87345e-02_rb,0.80998e-01_rb,0.10362e+00_rb,0.10701e+00_rb,0.44587e-01_rb /)
      kbo(:, 5,59, 9) = (/ &
        & 0.13202e-01_rb,0.10960e+00_rb,0.13878e+00_rb,0.14268e+00_rb,0.60173e-01_rb /)
      kbo(:, 1,13,10) = (/ &
        & 0.68082e+01_rb,0.10647e+02_rb,0.12044e+02_rb,0.12092e+02_rb,0.85097e+01_rb /)
      kbo(:, 2,13,10) = (/ &
        & 0.67019e+01_rb,0.10585e+02_rb,0.12055e+02_rb,0.12198e+02_rb,0.87119e+01_rb /)
      kbo(:, 3,13,10) = (/ &
        & 0.67137e+01_rb,0.10536e+02_rb,0.12063e+02_rb,0.12312e+02_rb,0.88714e+01_rb /)
      kbo(:, 4,13,10) = (/ &
        & 0.68713e+01_rb,0.10489e+02_rb,0.12087e+02_rb,0.12413e+02_rb,0.90170e+01_rb /)
      kbo(:, 5,13,10) = (/ &
        & 0.71185e+01_rb,0.10479e+02_rb,0.12121e+02_rb,0.12479e+02_rb,0.91853e+01_rb /)
      kbo(:, 1,14,10) = (/ &
        & 0.58410e+01_rb,0.94018e+01_rb,0.10846e+02_rb,0.11035e+02_rb,0.76674e+01_rb /)
      kbo(:, 2,14,10) = (/ &
        & 0.58467e+01_rb,0.93735e+01_rb,0.10875e+02_rb,0.11156e+02_rb,0.78256e+01_rb /)
      kbo(:, 3,14,10) = (/ &
        & 0.59984e+01_rb,0.93438e+01_rb,0.10908e+02_rb,0.11271e+02_rb,0.79426e+01_rb /)
      kbo(:, 4,14,10) = (/ &
        & 0.62373e+01_rb,0.93563e+01_rb,0.10953e+02_rb,0.11364e+02_rb,0.81071e+01_rb /)
      kbo(:, 5,14,10) = (/ &
        & 0.65135e+01_rb,0.94809e+01_rb,0.10997e+02_rb,0.11429e+02_rb,0.82861e+01_rb /)
      kbo(:, 1,15,10) = (/ &
        & 0.50343e+01_rb,0.83265e+01_rb,0.97137e+01_rb,0.99946e+01_rb,0.69930e+01_rb /)
      kbo(:, 2,15,10) = (/ &
        & 0.51660e+01_rb,0.83107e+01_rb,0.97594e+01_rb,0.10123e+02_rb,0.71240e+01_rb /)
      kbo(:, 3,15,10) = (/ &
        & 0.53903e+01_rb,0.83316e+01_rb,0.98216e+01_rb,0.10239e+02_rb,0.72628e+01_rb /)
      kbo(:, 4,15,10) = (/ &
        & 0.56415e+01_rb,0.84645e+01_rb,0.98856e+01_rb,0.10324e+02_rb,0.74229e+01_rb /)
      kbo(:, 5,15,10) = (/ &
        & 0.58842e+01_rb,0.87055e+01_rb,0.99631e+01_rb,0.10400e+02_rb,0.75908e+01_rb /)
      kbo(:, 1,16,10) = (/ &
        & 0.44015e+01_rb,0.74103e+01_rb,0.86530e+01_rb,0.90071e+01_rb,0.63301e+01_rb /)
      kbo(:, 2,16,10) = (/ &
        & 0.45955e+01_rb,0.74391e+01_rb,0.87237e+01_rb,0.91359e+01_rb,0.64715e+01_rb /)
      kbo(:, 3,16,10) = (/ &
        & 0.48140e+01_rb,0.75662e+01_rb,0.88065e+01_rb,0.92379e+01_rb,0.66426e+01_rb /)
      kbo(:, 4,16,10) = (/ &
        & 0.50318e+01_rb,0.77981e+01_rb,0.89101e+01_rb,0.93283e+01_rb,0.68095e+01_rb /)
      kbo(:, 5,16,10) = (/ &
        & 0.52436e+01_rb,0.80914e+01_rb,0.90522e+01_rb,0.94292e+01_rb,0.69717e+01_rb /)
      kbo(:, 1,17,10) = (/ &
        & 0.38938e+01_rb,0.65976e+01_rb,0.77151e+01_rb,0.80672e+01_rb,0.57119e+01_rb /)
      kbo(:, 2,17,10) = (/ &
        & 0.40781e+01_rb,0.67171e+01_rb,0.78068e+01_rb,0.81879e+01_rb,0.58706e+01_rb /)
      kbo(:, 3,17,10) = (/ &
        & 0.42641e+01_rb,0.69435e+01_rb,0.79191e+01_rb,0.82934e+01_rb,0.60296e+01_rb /)
      kbo(:, 4,17,10) = (/ &
        & 0.44480e+01_rb,0.72305e+01_rb,0.80762e+01_rb,0.84106e+01_rb,0.62012e+01_rb /)
      kbo(:, 5,17,10) = (/ &
        & 0.46308e+01_rb,0.75237e+01_rb,0.83037e+01_rb,0.85418e+01_rb,0.63775e+01_rb /)
      kbo(:, 1,18,10) = (/ &
        & 0.34407e+01_rb,0.58963e+01_rb,0.69001e+01_rb,0.71891e+01_rb,0.51366e+01_rb /)
      kbo(:, 2,18,10) = (/ &
        & 0.35988e+01_rb,0.61068e+01_rb,0.70120e+01_rb,0.73051e+01_rb,0.52912e+01_rb /)
      kbo(:, 3,18,10) = (/ &
        & 0.37577e+01_rb,0.63853e+01_rb,0.71752e+01_rb,0.74336e+01_rb,0.54376e+01_rb /)
      kbo(:, 4,18,10) = (/ &
        & 0.39161e+01_rb,0.66726e+01_rb,0.74020e+01_rb,0.75773e+01_rb,0.56097e+01_rb /)
      kbo(:, 5,18,10) = (/ &
        & 0.40711e+01_rb,0.69480e+01_rb,0.76793e+01_rb,0.77395e+01_rb,0.57994e+01_rb /)
      kbo(:, 1,19,10) = (/ &
        & 0.30222e+01_rb,0.53136e+01_rb,0.61695e+01_rb,0.63815e+01_rb,0.46248e+01_rb /)
      kbo(:, 2,19,10) = (/ &
        & 0.31568e+01_rb,0.55745e+01_rb,0.63303e+01_rb,0.65186e+01_rb,0.47653e+01_rb /)
      kbo(:, 3,19,10) = (/ &
        & 0.32936e+01_rb,0.58515e+01_rb,0.65536e+01_rb,0.66691e+01_rb,0.49110e+01_rb /)
      kbo(:, 4,19,10) = (/ &
        & 0.34320e+01_rb,0.61200e+01_rb,0.68298e+01_rb,0.68431e+01_rb,0.50671e+01_rb /)
      kbo(:, 5,19,10) = (/ &
        & 0.35717e+01_rb,0.63810e+01_rb,0.71341e+01_rb,0.70485e+01_rb,0.52482e+01_rb /)
      kbo(:, 1,20,10) = (/ &
        & 0.26646e+01_rb,0.48341e+01_rb,0.55294e+01_rb,0.56818e+01_rb,0.41697e+01_rb /)
      kbo(:, 2,20,10) = (/ &
        & 0.27751e+01_rb,0.50933e+01_rb,0.57470e+01_rb,0.58376e+01_rb,0.43064e+01_rb /)
      kbo(:, 3,20,10) = (/ &
        & 0.28895e+01_rb,0.53454e+01_rb,0.60203e+01_rb,0.60166e+01_rb,0.44569e+01_rb /)
      kbo(:, 4,20,10) = (/ &
        & 0.30071e+01_rb,0.55962e+01_rb,0.63194e+01_rb,0.62235e+01_rb,0.46038e+01_rb /)
      kbo(:, 5,20,10) = (/ &
        & 0.31278e+01_rb,0.58505e+01_rb,0.66184e+01_rb,0.64780e+01_rb,0.47569e+01_rb /)
      kbo(:, 1,21,10) = (/ &
        & 0.23601e+01_rb,0.44020e+01_rb,0.49905e+01_rb,0.50776e+01_rb,0.37561e+01_rb /)
      kbo(:, 2,21,10) = (/ &
        & 0.24527e+01_rb,0.46408e+01_rb,0.52514e+01_rb,0.52563e+01_rb,0.39021e+01_rb /)
      kbo(:, 3,21,10) = (/ &
        & 0.25482e+01_rb,0.48775e+01_rb,0.55424e+01_rb,0.54642e+01_rb,0.40531e+01_rb /)
      kbo(:, 4,21,10) = (/ &
        & 0.26469e+01_rb,0.51183e+01_rb,0.58389e+01_rb,0.57185e+01_rb,0.42025e+01_rb /)
      kbo(:, 5,21,10) = (/ &
        & 0.27509e+01_rb,0.53673e+01_rb,0.61341e+01_rb,0.60025e+01_rb,0.43477e+01_rb /)
      kbo(:, 1,22,10) = (/ &
        & 0.20933e+01_rb,0.40130e+01_rb,0.45530e+01_rb,0.45722e+01_rb,0.33997e+01_rb /)
      kbo(:, 2,22,10) = (/ &
        & 0.21774e+01_rb,0.42389e+01_rb,0.48339e+01_rb,0.47801e+01_rb,0.35503e+01_rb /)
      kbo(:, 3,22,10) = (/ &
        & 0.22636e+01_rb,0.44688e+01_rb,0.51214e+01_rb,0.50286e+01_rb,0.36916e+01_rb /)
      kbo(:, 4,22,10) = (/ &
        & 0.23497e+01_rb,0.47085e+01_rb,0.54112e+01_rb,0.53109e+01_rb,0.38464e+01_rb /)
      kbo(:, 5,22,10) = (/ &
        & 0.24400e+01_rb,0.49544e+01_rb,0.57104e+01_rb,0.56159e+01_rb,0.40026e+01_rb /)
      kbo(:, 1,23,10) = (/ &
        & 0.18483e+01_rb,0.36572e+01_rb,0.41816e+01_rb,0.41476e+01_rb,0.30773e+01_rb /)
      kbo(:, 2,23,10) = (/ &
        & 0.19296e+01_rb,0.38779e+01_rb,0.44582e+01_rb,0.43906e+01_rb,0.32324e+01_rb /)
      kbo(:, 3,23,10) = (/ &
        & 0.20105e+01_rb,0.41090e+01_rb,0.47398e+01_rb,0.46676e+01_rb,0.33790e+01_rb /)
      kbo(:, 4,23,10) = (/ &
        & 0.20922e+01_rb,0.43460e+01_rb,0.50319e+01_rb,0.49689e+01_rb,0.35381e+01_rb /)
      kbo(:, 5,23,10) = (/ &
        & 0.21812e+01_rb,0.45942e+01_rb,0.53379e+01_rb,0.52907e+01_rb,0.36872e+01_rb /)
      kbo(:, 1,24,10) = (/ &
        & 0.16291e+01_rb,0.33480e+01_rb,0.38514e+01_rb,0.38051e+01_rb,0.27880e+01_rb /)
      kbo(:, 2,24,10) = (/ &
        & 0.17053e+01_rb,0.35635e+01_rb,0.41240e+01_rb,0.40716e+01_rb,0.29407e+01_rb /)
      kbo(:, 3,24,10) = (/ &
        & 0.17841e+01_rb,0.37930e+01_rb,0.44076e+01_rb,0.43668e+01_rb,0.31035e+01_rb /)
      kbo(:, 4,24,10) = (/ &
        & 0.18670e+01_rb,0.40349e+01_rb,0.47044e+01_rb,0.46831e+01_rb,0.32549e+01_rb /)
      kbo(:, 5,24,10) = (/ &
        & 0.19687e+01_rb,0.42837e+01_rb,0.50178e+01_rb,0.50123e+01_rb,0.34093e+01_rb /)
      kbo(:, 1,25,10) = (/ &
        & 0.14342e+01_rb,0.30870e+01_rb,0.35616e+01_rb,0.35262e+01_rb,0.25309e+01_rb /)
      kbo(:, 2,25,10) = (/ &
        & 0.15073e+01_rb,0.32988e+01_rb,0.38348e+01_rb,0.38083e+01_rb,0.26935e+01_rb /)
      kbo(:, 3,25,10) = (/ &
        & 0.15855e+01_rb,0.35267e+01_rb,0.41227e+01_rb,0.41156e+01_rb,0.28544e+01_rb /)
      kbo(:, 4,25,10) = (/ &
        & 0.16765e+01_rb,0.37674e+01_rb,0.44270e+01_rb,0.44387e+01_rb,0.30112e+01_rb /)
      kbo(:, 5,25,10) = (/ &
        & 0.17969e+01_rb,0.40179e+01_rb,0.47482e+01_rb,0.47807e+01_rb,0.31986e+01_rb /)
      kbo(:, 1,26,10) = (/ &
        & 0.12644e+01_rb,0.28734e+01_rb,0.33145e+01_rb,0.33023e+01_rb,0.23207e+01_rb /)
      kbo(:, 2,26,10) = (/ &
        & 0.13357e+01_rb,0.30796e+01_rb,0.35932e+01_rb,0.35956e+01_rb,0.24777e+01_rb /)
      kbo(:, 3,26,10) = (/ &
        & 0.14174e+01_rb,0.33035e+01_rb,0.38863e+01_rb,0.39081e+01_rb,0.26420e+01_rb /)
      kbo(:, 4,26,10) = (/ &
        & 0.15234e+01_rb,0.35430e+01_rb,0.41997e+01_rb,0.42431e+01_rb,0.28241e+01_rb /)
      kbo(:, 5,26,10) = (/ &
        & 0.16603e+01_rb,0.37959e+01_rb,0.45341e+01_rb,0.45996e+01_rb,0.30401e+01_rb /)
      kbo(:, 1,27,10) = (/ &
        & 0.11180e+01_rb,0.26923e+01_rb,0.31092e+01_rb,0.31181e+01_rb,0.21320e+01_rb /)
      kbo(:, 2,27,10) = (/ &
        & 0.11893e+01_rb,0.28981e+01_rb,0.33903e+01_rb,0.34176e+01_rb,0.22961e+01_rb /)
      kbo(:, 3,27,10) = (/ &
        & 0.12805e+01_rb,0.31184e+01_rb,0.36930e+01_rb,0.37398e+01_rb,0.24775e+01_rb /)
      kbo(:, 4,27,10) = (/ &
        & 0.14017e+01_rb,0.33577e+01_rb,0.40190e+01_rb,0.40909e+01_rb,0.26920e+01_rb /)
      kbo(:, 5,27,10) = (/ &
        & 0.15494e+01_rb,0.36148e+01_rb,0.43685e+01_rb,0.44626e+01_rb,0.29161e+01_rb /)
      kbo(:, 1,28,10) = (/ &
        & 0.99411e+00_rb,0.25422e+01_rb,0.29403e+01_rb,0.29678e+01_rb,0.19736e+01_rb /)
      kbo(:, 2,28,10) = (/ &
        & 0.10693e+01_rb,0.27468e+01_rb,0.32278e+01_rb,0.32747e+01_rb,0.21580e+01_rb /)
      kbo(:, 3,28,10) = (/ &
        & 0.11727e+01_rb,0.29677e+01_rb,0.35410e+01_rb,0.36136e+01_rb,0.23718e+01_rb /)
      kbo(:, 4,28,10) = (/ &
        & 0.13040e+01_rb,0.32092e+01_rb,0.38816e+01_rb,0.39808e+01_rb,0.25959e+01_rb /)
      kbo(:, 5,28,10) = (/ &
        & 0.14604e+01_rb,0.34721e+01_rb,0.42472e+01_rb,0.43708e+01_rb,0.28373e+01_rb /)
      kbo(:, 1,29,10) = (/ &
        & 0.89294e+00_rb,0.24188e+01_rb,0.28058e+01_rb,0.28515e+01_rb,0.18636e+01_rb /)
      kbo(:, 2,29,10) = (/ &
        & 0.97544e+00_rb,0.26259e+01_rb,0.31039e+01_rb,0.31712e+01_rb,0.20699e+01_rb /)
      kbo(:, 3,29,10) = (/ &
        & 0.10880e+01_rb,0.28493e+01_rb,0.34305e+01_rb,0.35278e+01_rb,0.22886e+01_rb /)
      kbo(:, 4,29,10) = (/ &
        & 0.12277e+01_rb,0.30947e+01_rb,0.37862e+01_rb,0.39124e+01_rb,0.25256e+01_rb /)
      kbo(:, 5,29,10) = (/ &
        & 0.13896e+01_rb,0.33676e+01_rb,0.41685e+01_rb,0.43225e+01_rb,0.27864e+01_rb /)
      kbo(:, 1,30,10) = (/ &
        & 0.81230e+00_rb,0.23190e+01_rb,0.27059e+01_rb,0.27666e+01_rb,0.17929e+01_rb /)
      kbo(:, 2,30,10) = (/ &
        & 0.90285e+00_rb,0.25306e+01_rb,0.30146e+01_rb,0.31051e+01_rb,0.20049e+01_rb /)
      kbo(:, 3,30,10) = (/ &
        & 0.10223e+01_rb,0.27572e+01_rb,0.33560e+01_rb,0.34779e+01_rb,0.22359e+01_rb /)
      kbo(:, 4,30,10) = (/ &
        & 0.11675e+01_rb,0.30101e+01_rb,0.37284e+01_rb,0.38827e+01_rb,0.24866e+01_rb /)
      kbo(:, 5,30,10) = (/ &
        & 0.13337e+01_rb,0.32968e+01_rb,0.41265e+01_rb,0.43099e+01_rb,0.27369e+01_rb /)
      kbo(:, 1,31,10) = (/ &
        & 0.75110e+00_rb,0.22419e+01_rb,0.26379e+01_rb,0.27157e+01_rb,0.17433e+01_rb /)
      kbo(:, 2,31,10) = (/ &
        & 0.84809e+00_rb,0.24579e+01_rb,0.29592e+01_rb,0.30730e+01_rb,0.19665e+01_rb /)
      kbo(:, 3,31,10) = (/ &
        & 0.97252e+00_rb,0.26914e+01_rb,0.33168e+01_rb,0.34645e+01_rb,0.22008e+01_rb /)
      kbo(:, 4,31,10) = (/ &
        & 0.11219e+01_rb,0.29550e+01_rb,0.37069e+01_rb,0.38878e+01_rb,0.24456e+01_rb /)
      kbo(:, 5,31,10) = (/ &
        & 0.12926e+01_rb,0.32597e+01_rb,0.41224e+01_rb,0.43304e+01_rb,0.27059e+01_rb /)
      kbo(:, 1,32,10) = (/ &
        & 0.70499e+00_rb,0.21846e+01_rb,0.25985e+01_rb,0.26966e+01_rb,0.17143e+01_rb /)
      kbo(:, 2,32,10) = (/ &
        & 0.80768e+00_rb,0.24067e+01_rb,0.29356e+01_rb,0.30726e+01_rb,0.19337e+01_rb /)
      kbo(:, 3,32,10) = (/ &
        & 0.93561e+00_rb,0.26499e+01_rb,0.33107e+01_rb,0.34825e+01_rb,0.21698e+01_rb /)
      kbo(:, 4,32,10) = (/ &
        & 0.10894e+01_rb,0.29281e+01_rb,0.37179e+01_rb,0.39220e+01_rb,0.24264e+01_rb /)
      kbo(:, 5,32,10) = (/ &
        & 0.12637e+01_rb,0.32525e+01_rb,0.41500e+01_rb,0.43764e+01_rb,0.26992e+01_rb /)
      kbo(:, 1,33,10) = (/ &
        & 0.67118e+00_rb,0.21457e+01_rb,0.25854e+01_rb,0.27057e+01_rb,0.16861e+01_rb /)
      kbo(:, 2,33,10) = (/ &
        & 0.77795e+00_rb,0.23758e+01_rb,0.29404e+01_rb,0.30997e+01_rb,0.19111e+01_rb /)
      kbo(:, 3,33,10) = (/ &
        & 0.91001e+00_rb,0.26325e+01_rb,0.33337e+01_rb,0.35269e+01_rb,0.21600e+01_rb /)
      kbo(:, 4,33,10) = (/ &
        & 0.10674e+01_rb,0.29280e+01_rb,0.37566e+01_rb,0.39787e+01_rb,0.24291e+01_rb /)
      kbo(:, 5,33,10) = (/ &
        & 0.12452e+01_rb,0.32737e+01_rb,0.42022e+01_rb,0.44428e+01_rb,0.27130e+01_rb /)
      kbo(:, 1,34,10) = (/ &
        & 0.64347e+00_rb,0.21163e+01_rb,0.25854e+01_rb,0.27260e+01_rb,0.16632e+01_rb /)
      kbo(:, 2,34,10) = (/ &
        & 0.75313e+00_rb,0.23563e+01_rb,0.29579e+01_rb,0.31342e+01_rb,0.18999e+01_rb /)
      kbo(:, 3,34,10) = (/ &
        & 0.88833e+00_rb,0.26269e+01_rb,0.33662e+01_rb,0.35749e+01_rb,0.21605e+01_rb /)
      kbo(:, 4,34,10) = (/ &
        & 0.10487e+01_rb,0.29401e+01_rb,0.38003e+01_rb,0.40358e+01_rb,0.24405e+01_rb /)
      kbo(:, 5,34,10) = (/ &
        & 0.12301e+01_rb,0.33056e+01_rb,0.42553e+01_rb,0.45073e+01_rb,0.27333e+01_rb /)
      kbo(:, 1,35,10) = (/ &
        & 0.61333e+00_rb,0.20804e+01_rb,0.25725e+01_rb,0.27255e+01_rb,0.16337e+01_rb /)
      kbo(:, 2,35,10) = (/ &
        & 0.72416e+00_rb,0.23301e+01_rb,0.29566e+01_rb,0.31440e+01_rb,0.18805e+01_rb /)
      kbo(:, 3,35,10) = (/ &
        & 0.85987e+00_rb,0.26111e+01_rb,0.33740e+01_rb,0.35934e+01_rb,0.21502e+01_rb /)
      kbo(:, 4,35,10) = (/ &
        & 0.10212e+01_rb,0.29378e+01_rb,0.38159e+01_rb,0.40603e+01_rb,0.24378e+01_rb /)
      kbo(:, 5,35,10) = (/ &
        & 0.12063e+01_rb,0.33167e+01_rb,0.42768e+01_rb,0.45362e+01_rb,0.27369e+01_rb /)
      kbo(:, 1,36,10) = (/ &
        & 0.57858e+00_rb,0.20318e+01_rb,0.25343e+01_rb,0.26932e+01_rb,0.15911e+01_rb /)
      kbo(:, 2,36,10) = (/ &
        & 0.68823e+00_rb,0.22888e+01_rb,0.29242e+01_rb,0.31171e+01_rb,0.18445e+01_rb /)
      kbo(:, 3,36,10) = (/ &
        & 0.82172e+00_rb,0.25765e+01_rb,0.33460e+01_rb,0.35706e+01_rb,0.21205e+01_rb /)
      kbo(:, 4,36,10) = (/ &
        & 0.98227e+00_rb,0.29099e+01_rb,0.37920e+01_rb,0.40410e+01_rb,0.24128e+01_rb /)
      kbo(:, 5,36,10) = (/ &
        & 0.11693e+01_rb,0.32947e+01_rb,0.42558e+01_rb,0.45198e+01_rb,0.27160e+01_rb /)
      kbo(:, 1,37,10) = (/ &
        & 0.53405e+00_rb,0.19558e+01_rb,0.24506e+01_rb,0.26071e+01_rb,0.15218e+01_rb /)
      kbo(:, 2,37,10) = (/ &
        & 0.63968e+00_rb,0.22170e+01_rb,0.28388e+01_rb,0.30310e+01_rb,0.17770e+01_rb /)
      kbo(:, 3,37,10) = (/ &
        & 0.76779e+00_rb,0.25049e+01_rb,0.32599e+01_rb,0.34842e+01_rb,0.20552e+01_rb /)
      kbo(:, 4,37,10) = (/ &
        & 0.92384e+00_rb,0.28358e+01_rb,0.37058e+01_rb,0.39557e+01_rb,0.23490e+01_rb /)
      kbo(:, 5,37,10) = (/ &
        & 0.11094e+01_rb,0.32175e+01_rb,0.41691e+01_rb,0.44348e+01_rb,0.26544e+01_rb /)
      kbo(:, 1,38,10) = (/ &
        & 0.49328e+00_rb,0.18861e+01_rb,0.23724e+01_rb,0.25236e+01_rb,0.14577e+01_rb /)
      kbo(:, 2,38,10) = (/ &
        & 0.59575e+00_rb,0.21506e+01_rb,0.27570e+01_rb,0.29471e+01_rb,0.17138e+01_rb /)
      kbo(:, 3,38,10) = (/ &
        & 0.71877e+00_rb,0.24381e+01_rb,0.31760e+01_rb,0.33985e+01_rb,0.19934e+01_rb /)
      kbo(:, 4,38,10) = (/ &
        & 0.87044e+00_rb,0.27651e+01_rb,0.36203e+01_rb,0.38701e+01_rb,0.22879e+01_rb /)
      kbo(:, 5,38,10) = (/ &
        & 0.10541e+01_rb,0.31424e+01_rb,0.40835e+01_rb,0.43497e+01_rb,0.25942e+01_rb /)
      kbo(:, 1,39,10) = (/ &
        & 0.45633e+00_rb,0.18232e+01_rb,0.23004e+01_rb,0.24453e+01_rb,0.13998e+01_rb /)
      kbo(:, 2,39,10) = (/ &
        & 0.55664e+00_rb,0.20902e+01_rb,0.26810e+01_rb,0.28682e+01_rb,0.16563e+01_rb /)
      kbo(:, 3,39,10) = (/ &
        & 0.67511e+00_rb,0.23775e+01_rb,0.30973e+01_rb,0.33174e+01_rb,0.19363e+01_rb /)
      kbo(:, 4,39,10) = (/ &
        & 0.82312e+00_rb,0.27002e+01_rb,0.35399e+01_rb,0.37881e+01_rb,0.22310e+01_rb /)
      kbo(:, 5,39,10) = (/ &
        & 0.10047e+01_rb,0.30719e+01_rb,0.40016e+01_rb,0.42681e+01_rb,0.25376e+01_rb /)
      kbo(:, 1,40,10) = (/ &
        & 0.41369e+00_rb,0.17392e+01_rb,0.21979e+01_rb,0.23287e+01_rb,0.13226e+01_rb /)
      kbo(:, 2,40,10) = (/ &
        & 0.51051e+00_rb,0.20064e+01_rb,0.25693e+01_rb,0.27492e+01_rb,0.15756e+01_rb /)
      kbo(:, 3,40,10) = (/ &
        & 0.62337e+00_rb,0.22909e+01_rb,0.29791e+01_rb,0.31928e+01_rb,0.18532e+01_rb /)
      kbo(:, 4,40,10) = (/ &
        & 0.76463e+00_rb,0.26052e+01_rb,0.34168e+01_rb,0.36603e+01_rb,0.21464e+01_rb /)
      kbo(:, 5,40,10) = (/ &
        & 0.94032e+00_rb,0.29656e+01_rb,0.38751e+01_rb,0.41396e+01_rb,0.24517e+01_rb /)
      kbo(:, 1,41,10) = (/ &
        & 0.37354e+00_rb,0.16561e+01_rb,0.20963e+01_rb,0.22111e+01_rb,0.12469e+01_rb /)
      kbo(:, 2,41,10) = (/ &
        & 0.46665e+00_rb,0.19239e+01_rb,0.24587e+01_rb,0.26289e+01_rb,0.14959e+01_rb /)
      kbo(:, 3,41,10) = (/ &
        & 0.57545e+00_rb,0.22047e+01_rb,0.28597e+01_rb,0.30659e+01_rb,0.17694e+01_rb /)
      kbo(:, 4,41,10) = (/ &
        & 0.70985e+00_rb,0.25106e+01_rb,0.32912e+01_rb,0.35291e+01_rb,0.20606e+01_rb /)
      kbo(:, 5,41,10) = (/ &
        & 0.87840e+00_rb,0.28591e+01_rb,0.37457e+01_rb,0.40073e+01_rb,0.23640e+01_rb /)
      kbo(:, 1,42,10) = (/ &
        & 0.33676e+00_rb,0.15772e+01_rb,0.19999e+01_rb,0.20973e+01_rb,0.11755e+01_rb /)
      kbo(:, 2,42,10) = (/ &
        & 0.42603e+00_rb,0.18429e+01_rb,0.23518e+01_rb,0.25105e+01_rb,0.14196e+01_rb /)
      kbo(:, 3,42,10) = (/ &
        & 0.53169e+00_rb,0.21211e+01_rb,0.27440e+01_rb,0.29424e+01_rb,0.16886e+01_rb /)
      kbo(:, 4,42,10) = (/ &
        & 0.65992e+00_rb,0.24200e+01_rb,0.31691e+01_rb,0.34004e+01_rb,0.19770e+01_rb /)
      kbo(:, 5,42,10) = (/ &
        & 0.82123e+00_rb,0.27572e+01_rb,0.36189e+01_rb,0.38764e+01_rb,0.22780e+01_rb /)
      kbo(:, 1,43,10) = (/ &
        & 0.29880e+00_rb,0.14855e+01_rb,0.18883e+01_rb,0.19649e+01_rb,0.10947e+01_rb /)
      kbo(:, 2,43,10) = (/ &
        & 0.38323e+00_rb,0.17495e+01_rb,0.22289e+01_rb,0.23704e+01_rb,0.13324e+01_rb /)
      kbo(:, 3,43,10) = (/ &
        & 0.48519e+00_rb,0.20243e+01_rb,0.26093e+01_rb,0.27970e+01_rb,0.15948e+01_rb /)
      kbo(:, 4,43,10) = (/ &
        & 0.60690e+00_rb,0.23164e+01_rb,0.30260e+01_rb,0.32477e+01_rb,0.18788e+01_rb /)
      kbo(:, 5,43,10) = (/ &
        & 0.75876e+00_rb,0.26404e+01_rb,0.34692e+01_rb,0.37199e+01_rb,0.21767e+01_rb /)
      kbo(:, 1,44,10) = (/ &
        & 0.26234e+00_rb,0.13896e+01_rb,0.17735e+01_rb,0.18283e+01_rb,0.10125e+01_rb /)
      kbo(:, 2,44,10) = (/ &
        & 0.34133e+00_rb,0.16516e+01_rb,0.21029e+01_rb,0.22231e+01_rb,0.12426e+01_rb /)
      kbo(:, 3,44,10) = (/ &
        & 0.43896e+00_rb,0.19234e+01_rb,0.24698e+01_rb,0.26446e+01_rb,0.14975e+01_rb /)
      kbo(:, 4,44,10) = (/ &
        & 0.55556e+00_rb,0.22104e+01_rb,0.28764e+01_rb,0.30872e+01_rb,0.17758e+01_rb /)
      kbo(:, 5,44,10) = (/ &
        & 0.69720e+00_rb,0.25211e+01_rb,0.33108e+01_rb,0.35530e+01_rb,0.20699e+01_rb /)
      kbo(:, 1,45,10) = (/ &
        & 0.22932e+00_rb,0.12967e+01_rb,0.16617e+01_rb,0.16976e+01_rb,0.93402e+00_rb /)
      kbo(:, 2,45,10) = (/ &
        & 0.30290e+00_rb,0.15560e+01_rb,0.19824e+01_rb,0.20804e+01_rb,0.11566e+01_rb /)
      kbo(:, 3,45,10) = (/ &
        & 0.39581e+00_rb,0.18251e+01_rb,0.23356e+01_rb,0.24949e+01_rb,0.14039e+01_rb /)
      kbo(:, 4,45,10) = (/ &
        & 0.50777e+00_rb,0.21068e+01_rb,0.27305e+01_rb,0.29300e+01_rb,0.16755e+01_rb /)
      kbo(:, 5,45,10) = (/ &
        & 0.64060e+00_rb,0.24073e+01_rb,0.31565e+01_rb,0.33892e+01_rb,0.19653e+01_rb /)
      kbo(:, 1,46,10) = (/ &
        & 0.19825e+00_rb,0.12020e+01_rb,0.15457e+01_rb,0.15662e+01_rb,0.85464e+00_rb /)
      kbo(:, 2,46,10) = (/ &
        & 0.26591e+00_rb,0.14563e+01_rb,0.18595e+01_rb,0.19337e+01_rb,0.10694e+01_rb /)
      kbo(:, 3,46,10) = (/ &
        & 0.35343e+00_rb,0.17232e+01_rb,0.22000e+01_rb,0.23390e+01_rb,0.13083e+01_rb /)
      kbo(:, 4,46,10) = (/ &
        & 0.46053e+00_rb,0.19999e+01_rb,0.25801e+01_rb,0.27669e+01_rb,0.15716e+01_rb /)
      kbo(:, 5,46,10) = (/ &
        & 0.58556e+00_rb,0.22922e+01_rb,0.29962e+01_rb,0.32174e+01_rb,0.18560e+01_rb /)
      kbo(:, 1,47,10) = (/ &
        & 0.16836e+00_rb,0.11008e+01_rb,0.14194e+01_rb,0.14270e+01_rb,0.77061e+00_rb /)
      kbo(:, 2,47,10) = (/ &
        & 0.22922e+00_rb,0.13473e+01_rb,0.17277e+01_rb,0.17767e+01_rb,0.97668e+00_rb /)
      kbo(:, 3,47,10) = (/ &
        & 0.31004e+00_rb,0.16106e+01_rb,0.20551e+01_rb,0.21688e+01_rb,0.12060e+01_rb /)
      kbo(:, 4,47,10) = (/ &
        & 0.41110e+00_rb,0.18840e+01_rb,0.24189e+01_rb,0.25893e+01_rb,0.14598e+01_rb /)
      kbo(:, 5,47,10) = (/ &
        & 0.52932e+00_rb,0.21690e+01_rb,0.28216e+01_rb,0.30298e+01_rb,0.17368e+01_rb /)
      kbo(:, 1,48,10) = (/ &
        & 0.14220e+00_rb,0.10041e+01_rb,0.12957e+01_rb,0.12963e+01_rb,0.69114e+00_rb /)
      kbo(:, 2,48,10) = (/ &
        & 0.19626e+00_rb,0.12419e+01_rb,0.15985e+01_rb,0.16273e+01_rb,0.88794e+00_rb /)
      kbo(:, 3,48,10) = (/ &
        & 0.27002e+00_rb,0.15006e+01_rb,0.19169e+01_rb,0.20039e+01_rb,0.11083e+01_rb /)
      kbo(:, 4,48,10) = (/ &
        & 0.36488e+00_rb,0.17700e+01_rb,0.22644e+01_rb,0.24151e+01_rb,0.13523e+01_rb /)
      kbo(:, 5,48,10) = (/ &
        & 0.47701e+00_rb,0.20494e+01_rb,0.26525e+01_rb,0.28466e+01_rb,0.16205e+01_rb /)
      kbo(:, 1,49,10) = (/ &
        & 0.11950e+00_rb,0.91234e+00_rb,0.11755e+01_rb,0.11741e+01_rb,0.61611e+00_rb /)
      kbo(:, 2,49,10) = (/ &
        & 0.16698e+00_rb,0.11406e+01_rb,0.14723e+01_rb,0.14860e+01_rb,0.80337e+00_rb /)
      kbo(:, 3,49,10) = (/ &
        & 0.23352e+00_rb,0.13921e+01_rb,0.17842e+01_rb,0.18450e+01_rb,0.10147e+01_rb /)
      kbo(:, 4,49,10) = (/ &
        & 0.32154e+00_rb,0.16580e+01_rb,0.21177e+01_rb,0.22439e+01_rb,0.12490e+01_rb /)
      kbo(:, 5,49,10) = (/ &
        & 0.42774e+00_rb,0.19330e+01_rb,0.24891e+01_rb,0.26683e+01_rb,0.15078e+01_rb /)
      kbo(:, 1,50,10) = (/ &
        & 0.10087e+00_rb,0.83065e+00_rb,0.10669e+01_rb,0.10674e+01_rb,0.54936e+00_rb /)
      kbo(:, 2,50,10) = (/ &
        & 0.14237e+00_rb,0.10490e+01_rb,0.13561e+01_rb,0.13605e+01_rb,0.72778e+00_rb /)
      kbo(:, 3,50,10) = (/ &
        & 0.20234e+00_rb,0.12926e+01_rb,0.16630e+01_rb,0.17021e+01_rb,0.93055e+00_rb /)
      kbo(:, 4,50,10) = (/ &
        & 0.28362e+00_rb,0.15546e+01_rb,0.19861e+01_rb,0.20879e+01_rb,0.11563e+01_rb /)
      kbo(:, 5,50,10) = (/ &
        & 0.38392e+00_rb,0.18265e+01_rb,0.23423e+01_rb,0.25044e+01_rb,0.14058e+01_rb /)
      kbo(:, 1,51,10) = (/ &
        & 0.85305e-01_rb,0.75516e+00_rb,0.96600e+00_rb,0.96989e+00_rb,0.48900e+00_rb /)
      kbo(:, 2,51,10) = (/ &
        & 0.12119e+00_rb,0.96396e+00_rb,0.12458e+01_rb,0.12458e+01_rb,0.65803e+00_rb /)
      kbo(:, 3,51,10) = (/ &
        & 0.17492e+00_rb,0.11993e+01_rb,0.15474e+01_rb,0.15706e+01_rb,0.85222e+00_rb /)
      kbo(:, 4,51,10) = (/ &
        & 0.24953e+00_rb,0.14559e+01_rb,0.18635e+01_rb,0.19410e+01_rb,0.10697e+01_rb /)
      kbo(:, 5,51,10) = (/ &
        & 0.34420e+00_rb,0.17250e+01_rb,0.22062e+01_rb,0.23483e+01_rb,0.13105e+01_rb /)
      kbo(:, 1,52,10) = (/ &
        & 0.72122e-01_rb,0.68398e+00_rb,0.87088e+00_rb,0.87737e+00_rb,0.43154e+00_rb /)
      kbo(:, 2,52,10) = (/ &
        & 0.10277e+00_rb,0.88315e+00_rb,0.11388e+01_rb,0.11383e+01_rb,0.59136e+00_rb /)
      kbo(:, 3,52,10) = (/ &
        & 0.15043e+00_rb,0.11094e+01_rb,0.14345e+01_rb,0.14453e+01_rb,0.77736e+00_rb /)
      kbo(:, 4,52,10) = (/ &
        & 0.21816e+00_rb,0.13587e+01_rb,0.17451e+01_rb,0.17993e+01_rb,0.98663e+00_rb /)
      kbo(:, 5,52,10) = (/ &
        & 0.30639e+00_rb,0.16242e+01_rb,0.20759e+01_rb,0.21951e+01_rb,0.12188e+01_rb /)
      kbo(:, 1,53,10) = (/ &
        & 0.60992e-01_rb,0.61681e+00_rb,0.78096e+00_rb,0.78801e+00_rb,0.37771e+00_rb /)
      kbo(:, 2,53,10) = (/ &
        & 0.86893e-01_rb,0.80591e+00_rb,0.10354e+01_rb,0.10374e+01_rb,0.52892e+00_rb /)
      kbo(:, 3,53,10) = (/ &
        & 0.12844e+00_rb,0.10223e+01_rb,0.13228e+01_rb,0.13263e+01_rb,0.70557e+00_rb /)
      kbo(:, 4,53,10) = (/ &
        & 0.18962e+00_rb,0.12639e+01_rb,0.16288e+01_rb,0.16633e+01_rb,0.90648e+00_rb /)
      kbo(:, 5,53,10) = (/ &
        & 0.27104e+00_rb,0.15253e+01_rb,0.19505e+01_rb,0.20457e+01_rb,0.11304e+01_rb /)
      kbo(:, 1,54,10) = (/ &
        & 0.52146e-01_rb,0.55862e+00_rb,0.70341e+00_rb,0.70953e+00_rb,0.33141e+00_rb /)
      kbo(:, 2,54,10) = (/ &
        & 0.74252e-01_rb,0.73817e+00_rb,0.94468e+00_rb,0.94968e+00_rb,0.47420e+00_rb /)
      kbo(:, 3,54,10) = (/ &
        & 0.11046e+00_rb,0.94583e+00_rb,0.12229e+01_rb,0.12232e+01_rb,0.64224e+00_rb /)
      kbo(:, 4,54,10) = (/ &
        & 0.16584e+00_rb,0.11796e+01_rb,0.15239e+01_rb,0.15447e+01_rb,0.83594e+00_rb /)
      kbo(:, 5,54,10) = (/ &
        & 0.24084e+00_rb,0.14356e+01_rb,0.18395e+01_rb,0.19125e+01_rb,0.10521e+01_rb /)
      kbo(:, 1,55,10) = (/ &
        & 0.44769e-01_rb,0.50580e+00_rb,0.63367e+00_rb,0.63830e+00_rb,0.29171e+00_rb /)
      kbo(:, 2,55,10) = (/ &
        & 0.63801e-01_rb,0.67695e+00_rb,0.86251e+00_rb,0.86950e+00_rb,0.42385e+00_rb /)
      kbo(:, 3,55,10) = (/ &
        & 0.95281e-01_rb,0.87561e+00_rb,0.11296e+01_rb,0.11298e+01_rb,0.58465e+00_rb /)
      kbo(:, 4,55,10) = (/ &
        & 0.14509e+00_rb,0.11012e+01_rb,0.14250e+01_rb,0.14355e+01_rb,0.77056e+00_rb /)
      kbo(:, 5,55,10) = (/ &
        & 0.21402e+00_rb,0.13506e+01_rb,0.17360e+01_rb,0.17888e+01_rb,0.97966e+00_rb /)
      kbo(:, 1,56,10) = (/ &
        & 0.38303e-01_rb,0.45495e+00_rb,0.56826e+00_rb,0.57046e+00_rb,0.25591e+00_rb /)
      kbo(:, 2,56,10) = (/ &
        & 0.54762e-01_rb,0.61846e+00_rb,0.78411e+00_rb,0.79150e+00_rb,0.37700e+00_rb /)
      kbo(:, 3,56,10) = (/ &
        & 0.82009e-01_rb,0.80830e+00_rb,0.10394e+01_rb,0.10417e+01_rb,0.52950e+00_rb /)
      kbo(:, 4,56,10) = (/ &
        & 0.12621e+00_rb,0.10253e+01_rb,0.13275e+01_rb,0.13316e+01_rb,0.70721e+00_rb /)
      kbo(:, 5,56,10) = (/ &
        & 0.18937e+00_rb,0.12680e+01_rb,0.16344e+01_rb,0.16700e+01_rb,0.90979e+00_rb /)
      kbo(:, 1,57,10) = (/ &
        & 0.32632e-01_rb,0.40611e+00_rb,0.50731e+00_rb,0.50659e+00_rb,0.22496e+00_rb /)
      kbo(:, 2,57,10) = (/ &
        & 0.47019e-01_rb,0.56304e+00_rb,0.71002e+00_rb,0.71651e+00_rb,0.33305e+00_rb /)
      kbo(:, 3,57,10) = (/ &
        & 0.70456e-01_rb,0.74361e+00_rb,0.95266e+00_rb,0.95775e+00_rb,0.47635e+00_rb /)
      kbo(:, 4,57,10) = (/ &
        & 0.10937e+00_rb,0.95229e+00_rb,0.12322e+01_rb,0.12329e+01_rb,0.64711e+00_rb /)
      kbo(:, 5,57,10) = (/ &
        & 0.16675e+00_rb,0.11875e+01_rb,0.15342e+01_rb,0.15565e+01_rb,0.84216e+00_rb /)
      kbo(:, 1,58,10) = (/ &
        & 0.27856e-01_rb,0.36188e+00_rb,0.45369e+00_rb,0.45016e+00_rb,0.19835e+00_rb /)
      kbo(:, 2,58,10) = (/ &
        & 0.40581e-01_rb,0.51273e+00_rb,0.64353e+00_rb,0.64865e+00_rb,0.29546e+00_rb /)
      kbo(:, 3,58,10) = (/ &
        & 0.60914e-01_rb,0.68524e+00_rb,0.87436e+00_rb,0.88146e+00_rb,0.42903e+00_rb /)
      kbo(:, 4,58,10) = (/ &
        & 0.95154e-01_rb,0.88582e+00_rb,0.11439e+01_rb,0.11441e+01_rb,0.59161e+00_rb /)
      kbo(:, 5,58,10) = (/ &
        & 0.14715e+00_rb,0.11133e+01_rb,0.14407e+01_rb,0.14528e+01_rb,0.77938e+00_rb /)
      kbo(:, 1,59,10) = (/ &
        & 0.25371e-01_rb,0.34461e+00_rb,0.43295e+00_rb,0.42860e+00_rb,0.18805e+00_rb /)
      kbo(:, 2,59,10) = (/ &
        & 0.37551e-01_rb,0.49296e+00_rb,0.61778e+00_rb,0.62208e+00_rb,0.28092e+00_rb /)
      kbo(:, 3,59,10) = (/ &
        & 0.57031e-01_rb,0.66239e+00_rb,0.84380e+00_rb,0.85137e+00_rb,0.40984e+00_rb /)
      kbo(:, 4,59,10) = (/ &
        & 0.89730e-01_rb,0.85967e+00_rb,0.11090e+01_rb,0.11096e+01_rb,0.56882e+00_rb /)
      kbo(:, 5,59,10) = (/ &
        & 0.13959e+00_rb,0.10839e+01_rb,0.14035e+01_rb,0.14125e+01_rb,0.75487e+00_rb /)
      kbo(:, 1,13,11) = (/ &
        & 0.13518e+02_rb,0.19426e+02_rb,0.21368e+02_rb,0.20607e+02_rb,0.15967e+02_rb /)
      kbo(:, 2,13,11) = (/ &
        & 0.13249e+02_rb,0.19278e+02_rb,0.21345e+02_rb,0.20725e+02_rb,0.16218e+02_rb /)
      kbo(:, 3,13,11) = (/ &
        & 0.12991e+02_rb,0.19106e+02_rb,0.21344e+02_rb,0.20838e+02_rb,0.16493e+02_rb /)
      kbo(:, 4,13,11) = (/ &
        & 0.12783e+02_rb,0.18953e+02_rb,0.21337e+02_rb,0.20959e+02_rb,0.16769e+02_rb /)
      kbo(:, 5,13,11) = (/ &
        & 0.12735e+02_rb,0.18820e+02_rb,0.21328e+02_rb,0.21104e+02_rb,0.16982e+02_rb /)
      kbo(:, 1,14,11) = (/ &
        & 0.11504e+02_rb,0.17534e+02_rb,0.19457e+02_rb,0.19131e+02_rb,0.14625e+02_rb /)
      kbo(:, 2,14,11) = (/ &
        & 0.11281e+02_rb,0.17396e+02_rb,0.19491e+02_rb,0.19271e+02_rb,0.14939e+02_rb /)
      kbo(:, 3,14,11) = (/ &
        & 0.11112e+02_rb,0.17290e+02_rb,0.19527e+02_rb,0.19415e+02_rb,0.15255e+02_rb /)
      kbo(:, 4,14,11) = (/ &
        & 0.11115e+02_rb,0.17213e+02_rb,0.19560e+02_rb,0.19580e+02_rb,0.15504e+02_rb /)
      kbo(:, 5,14,11) = (/ &
        & 0.11342e+02_rb,0.17139e+02_rb,0.19615e+02_rb,0.19759e+02_rb,0.15706e+02_rb /)
      kbo(:, 1,15,11) = (/ &
        & 0.98170e+01_rb,0.15599e+02_rb,0.17627e+02_rb,0.17666e+02_rb,0.13354e+02_rb /)
      kbo(:, 2,15,11) = (/ &
        & 0.96747e+01_rb,0.15533e+02_rb,0.17695e+02_rb,0.17842e+02_rb,0.13687e+02_rb /)
      kbo(:, 3,15,11) = (/ &
        & 0.96984e+01_rb,0.15502e+02_rb,0.17755e+02_rb,0.18036e+02_rb,0.14004e+02_rb /)
      kbo(:, 4,15,11) = (/ &
        & 0.99522e+01_rb,0.15480e+02_rb,0.17829e+02_rb,0.18235e+02_rb,0.14263e+02_rb /)
      kbo(:, 5,15,11) = (/ &
        & 0.10344e+02_rb,0.15525e+02_rb,0.17929e+02_rb,0.18418e+02_rb,0.14475e+02_rb /)
      kbo(:, 1,16,11) = (/ &
        & 0.84332e+01_rb,0.13806e+02_rb,0.15926e+02_rb,0.16215e+02_rb,0.12224e+02_rb /)
      kbo(:, 2,16,11) = (/ &
        & 0.84583e+01_rb,0.13805e+02_rb,0.16021e+02_rb,0.16437e+02_rb,0.12556e+02_rb /)
      kbo(:, 3,16,11) = (/ &
        & 0.87066e+01_rb,0.13823e+02_rb,0.16108e+02_rb,0.16660e+02_rb,0.12834e+02_rb /)
      kbo(:, 4,16,11) = (/ &
        & 0.90827e+01_rb,0.13924e+02_rb,0.16226e+02_rb,0.16883e+02_rb,0.13078e+02_rb /)
      kbo(:, 5,16,11) = (/ &
        & 0.95263e+01_rb,0.14156e+02_rb,0.16382e+02_rb,0.17079e+02_rb,0.13334e+02_rb /)
      kbo(:, 1,17,11) = (/ &
        & 0.73068e+01_rb,0.12252e+02_rb,0.14341e+02_rb,0.14810e+02_rb,0.11129e+02_rb /)
      kbo(:, 2,17,11) = (/ &
        & 0.75295e+01_rb,0.12291e+02_rb,0.14463e+02_rb,0.15052e+02_rb,0.11439e+02_rb /)
      kbo(:, 3,17,11) = (/ &
        & 0.78853e+01_rb,0.12405e+02_rb,0.14606e+02_rb,0.15309e+02_rb,0.11729e+02_rb /)
      kbo(:, 4,17,11) = (/ &
        & 0.83040e+01_rb,0.12643e+02_rb,0.14776e+02_rb,0.15548e+02_rb,0.12006e+02_rb /)
      kbo(:, 5,17,11) = (/ &
        & 0.87123e+01_rb,0.13039e+02_rb,0.14982e+02_rb,0.15762e+02_rb,0.12286e+02_rb /)
      kbo(:, 1,18,11) = (/ &
        & 0.64287e+01_rb,0.10933e+02_rb,0.12879e+02_rb,0.13451e+02_rb,0.10053e+02_rb /)
      kbo(:, 2,18,11) = (/ &
        & 0.67446e+01_rb,0.11039e+02_rb,0.13041e+02_rb,0.13740e+02_rb,0.10377e+02_rb /)
      kbo(:, 3,18,11) = (/ &
        & 0.71271e+01_rb,0.11266e+02_rb,0.13222e+02_rb,0.14018e+02_rb,0.10699e+02_rb /)
      kbo(:, 4,18,11) = (/ &
        & 0.75067e+01_rb,0.11650e+02_rb,0.13455e+02_rb,0.14263e+02_rb,0.11011e+02_rb /)
      kbo(:, 5,18,11) = (/ &
        & 0.78797e+01_rb,0.12144e+02_rb,0.13766e+02_rb,0.14517e+02_rb,0.11323e+02_rb /)
      kbo(:, 1,19,11) = (/ &
        & 0.57165e+01_rb,0.98198e+01_rb,0.11563e+02_rb,0.12210e+02_rb,0.90287e+01_rb /)
      kbo(:, 2,19,11) = (/ &
        & 0.60504e+01_rb,0.10030e+02_rb,0.11759e+02_rb,0.12511e+02_rb,0.93771e+01_rb /)
      kbo(:, 3,19,11) = (/ &
        & 0.63872e+01_rb,0.10390e+02_rb,0.12004e+02_rb,0.12787e+02_rb,0.97382e+01_rb /)
      kbo(:, 4,19,11) = (/ &
        & 0.67262e+01_rb,0.10858e+02_rb,0.12325e+02_rb,0.13071e+02_rb,0.10100e+02_rb /)
      kbo(:, 5,19,11) = (/ &
        & 0.70630e+01_rb,0.11396e+02_rb,0.12720e+02_rb,0.13371e+02_rb,0.10433e+02_rb /)
      kbo(:, 1,20,11) = (/ &
        & 0.51077e+01_rb,0.88680e+01_rb,0.10415e+02_rb,0.11053e+02_rb,0.80963e+01_rb /)
      kbo(:, 2,20,11) = (/ &
        & 0.54017e+01_rb,0.92107e+01_rb,0.10662e+02_rb,0.11361e+02_rb,0.84758e+01_rb /)
      kbo(:, 3,20,11) = (/ &
        & 0.57015e+01_rb,0.96698e+01_rb,0.10982e+02_rb,0.11668e+02_rb,0.88481e+01_rb /)
      kbo(:, 4,20,11) = (/ &
        & 0.60041e+01_rb,0.10196e+02_rb,0.11384e+02_rb,0.12000e+02_rb,0.92376e+01_rb /)
      kbo(:, 5,20,11) = (/ &
        & 0.63106e+01_rb,0.10732e+02_rb,0.11884e+02_rb,0.12349e+02_rb,0.96255e+01_rb /)
      kbo(:, 1,21,11) = (/ &
        & 0.45414e+01_rb,0.80841e+01_rb,0.94333e+01_rb,0.10001e+02_rb,0.72987e+01_rb /)
      kbo(:, 2,21,11) = (/ &
        & 0.48060e+01_rb,0.85169e+01_rb,0.97399e+01_rb,0.10324e+02_rb,0.76799e+01_rb /)
      kbo(:, 3,21,11) = (/ &
        & 0.50754e+01_rb,0.90311e+01_rb,0.10134e+02_rb,0.10673e+02_rb,0.80692e+01_rb /)
      kbo(:, 4,21,11) = (/ &
        & 0.53489e+01_rb,0.95660e+01_rb,0.10626e+02_rb,0.11046e+02_rb,0.84671e+01_rb /)
      kbo(:, 5,21,11) = (/ &
        & 0.56226e+01_rb,0.10100e+02_rb,0.11188e+02_rb,0.11467e+02_rb,0.88878e+01_rb /)
      kbo(:, 1,22,11) = (/ &
        & 0.40512e+01_rb,0.74537e+01_rb,0.86113e+01_rb,0.90760e+01_rb,0.66406e+01_rb /)
      kbo(:, 2,22,11) = (/ &
        & 0.42861e+01_rb,0.79455e+01_rb,0.89988e+01_rb,0.94326e+01_rb,0.70298e+01_rb /)
      kbo(:, 3,22,11) = (/ &
        & 0.45284e+01_rb,0.84723e+01_rb,0.94806e+01_rb,0.98282e+01_rb,0.74365e+01_rb /)
      kbo(:, 4,22,11) = (/ &
        & 0.47765e+01_rb,0.90022e+01_rb,0.10036e+02_rb,0.10267e+02_rb,0.78530e+01_rb /)
      kbo(:, 5,22,11) = (/ &
        & 0.50273e+01_rb,0.95375e+01_rb,0.10644e+02_rb,0.10762e+02_rb,0.82925e+01_rb /)
      kbo(:, 1,23,11) = (/ &
        & 0.36272e+01_rb,0.69345e+01_rb,0.79292e+01_rb,0.82821e+01_rb,0.60862e+01_rb /)
      kbo(:, 2,23,11) = (/ &
        & 0.38324e+01_rb,0.74412e+01_rb,0.84058e+01_rb,0.86779e+01_rb,0.64814e+01_rb /)
      kbo(:, 3,23,11) = (/ &
        & 0.40496e+01_rb,0.79556e+01_rb,0.89497e+01_rb,0.91277e+01_rb,0.69031e+01_rb /)
      kbo(:, 4,23,11) = (/ &
        & 0.42736e+01_rb,0.84881e+01_rb,0.95512e+01_rb,0.96353e+01_rb,0.73423e+01_rb /)
      kbo(:, 5,23,11) = (/ &
        & 0.45008e+01_rb,0.90231e+01_rb,0.10199e+02_rb,0.10196e+02_rb,0.78107e+01_rb /)
      kbo(:, 1,24,11) = (/ &
        & 0.32617e+01_rb,0.64927e+01_rb,0.73814e+01_rb,0.76204e+01_rb,0.56351e+01_rb /)
      kbo(:, 2,24,11) = (/ &
        & 0.34455e+01_rb,0.69864e+01_rb,0.79146e+01_rb,0.80668e+01_rb,0.60482e+01_rb /)
      kbo(:, 3,24,11) = (/ &
        & 0.36375e+01_rb,0.74982e+01_rb,0.85124e+01_rb,0.85688e+01_rb,0.64793e+01_rb /)
      kbo(:, 4,24,11) = (/ &
        & 0.38361e+01_rb,0.80253e+01_rb,0.91530e+01_rb,0.91384e+01_rb,0.69488e+01_rb /)
      kbo(:, 5,24,11) = (/ &
        & 0.40434e+01_rb,0.85770e+01_rb,0.98249e+01_rb,0.97701e+01_rb,0.74392e+01_rb /)
      kbo(:, 1,25,11) = (/ &
        & 0.29504e+01_rb,0.60996e+01_rb,0.69378e+01_rb,0.70933e+01_rb,0.52696e+01_rb /)
      kbo(:, 2,25,11) = (/ &
        & 0.31134e+01_rb,0.65859e+01_rb,0.75193e+01_rb,0.75874e+01_rb,0.56908e+01_rb /)
      kbo(:, 3,25,11) = (/ &
        & 0.32850e+01_rb,0.70991e+01_rb,0.81583e+01_rb,0.81455e+01_rb,0.61504e+01_rb /)
      kbo(:, 4,25,11) = (/ &
        & 0.34639e+01_rb,0.76357e+01_rb,0.88289e+01_rb,0.87766e+01_rb,0.66391e+01_rb /)
      kbo(:, 5,25,11) = (/ &
        & 0.36575e+01_rb,0.82031e+01_rb,0.95335e+01_rb,0.94728e+01_rb,0.71181e+01_rb /)
      kbo(:, 1,26,11) = (/ &
        & 0.26785e+01_rb,0.57592e+01_rb,0.65944e+01_rb,0.66793e+01_rb,0.49659e+01_rb /)
      kbo(:, 2,26,11) = (/ &
        & 0.28303e+01_rb,0.62512e+01_rb,0.72180e+01_rb,0.72292e+01_rb,0.54158e+01_rb /)
      kbo(:, 3,26,11) = (/ &
        & 0.29845e+01_rb,0.67700e+01_rb,0.78853e+01_rb,0.78517e+01_rb,0.58929e+01_rb /)
      kbo(:, 4,26,11) = (/ &
        & 0.31498e+01_rb,0.73218e+01_rb,0.85892e+01_rb,0.85405e+01_rb,0.63779e+01_rb /)
      kbo(:, 5,26,11) = (/ &
        & 0.33365e+01_rb,0.79095e+01_rb,0.93260e+01_rb,0.92876e+01_rb,0.68593e+01_rb /)
      kbo(:, 1,27,11) = (/ &
        & 0.24381e+01_rb,0.54719e+01_rb,0.63296e+01_rb,0.63699e+01_rb,0.47378e+01_rb /)
      kbo(:, 2,27,11) = (/ &
        & 0.25833e+01_rb,0.59706e+01_rb,0.69839e+01_rb,0.69809e+01_rb,0.52024e+01_rb /)
      kbo(:, 3,27,11) = (/ &
        & 0.27276e+01_rb,0.65032e+01_rb,0.76823e+01_rb,0.76623e+01_rb,0.56849e+01_rb /)
      kbo(:, 4,27,11) = (/ &
        & 0.28843e+01_rb,0.70768e+01_rb,0.84223e+01_rb,0.83973e+01_rb,0.61613e+01_rb /)
      kbo(:, 5,27,11) = (/ &
        & 0.30776e+01_rb,0.76837e+01_rb,0.91929e+01_rb,0.91871e+01_rb,0.66697e+01_rb /)
      kbo(:, 1,28,11) = (/ &
        & 0.22238e+01_rb,0.52366e+01_rb,0.61335e+01_rb,0.61574e+01_rb,0.45705e+01_rb /)
      kbo(:, 2,28,11) = (/ &
        & 0.23648e+01_rb,0.57451e+01_rb,0.68151e+01_rb,0.68266e+01_rb,0.50347e+01_rb /)
      kbo(:, 3,28,11) = (/ &
        & 0.25067e+01_rb,0.62980e+01_rb,0.75460e+01_rb,0.75548e+01_rb,0.55064e+01_rb /)
      kbo(:, 4,28,11) = (/ &
        & 0.26680e+01_rb,0.68930e+01_rb,0.83223e+01_rb,0.83308e+01_rb,0.60103e+01_rb /)
      kbo(:, 5,28,11) = (/ &
        & 0.28763e+01_rb,0.75234e+01_rb,0.91269e+01_rb,0.91547e+01_rb,0.65443e+01_rb /)
      kbo(:, 1,29,11) = (/ &
        & 0.20343e+01_rb,0.50540e+01_rb,0.60021e+01_rb,0.60360e+01_rb,0.44306e+01_rb /)
      kbo(:, 2,29,11) = (/ &
        & 0.21732e+01_rb,0.55785e+01_rb,0.67134e+01_rb,0.67544e+01_rb,0.48924e+01_rb /)
      kbo(:, 3,29,11) = (/ &
        & 0.23217e+01_rb,0.61554e+01_rb,0.74828e+01_rb,0.75217e+01_rb,0.53899e+01_rb /)
      kbo(:, 4,29,11) = (/ &
        & 0.24991e+01_rb,0.67727e+01_rb,0.82943e+01_rb,0.83341e+01_rb,0.59211e+01_rb /)
      kbo(:, 5,29,11) = (/ &
        & 0.27298e+01_rb,0.74309e+01_rb,0.91294e+01_rb,0.91877e+01_rb,0.64776e+01_rb /)
      kbo(:, 1,30,11) = (/ &
        & 0.18678e+01_rb,0.49203e+01_rb,0.59291e+01_rb,0.59875e+01_rb,0.43204e+01_rb /)
      kbo(:, 2,30,11) = (/ &
        & 0.20102e+01_rb,0.54667e+01_rb,0.66747e+01_rb,0.67430e+01_rb,0.48026e+01_rb /)
      kbo(:, 3,30,11) = (/ &
        & 0.21719e+01_rb,0.60682e+01_rb,0.74827e+01_rb,0.75477e+01_rb,0.53281e+01_rb /)
      kbo(:, 4,30,11) = (/ &
        & 0.23720e+01_rb,0.67126e+01_rb,0.83252e+01_rb,0.83910e+01_rb,0.58826e+01_rb /)
      kbo(:, 5,30,11) = (/ &
        & 0.26264e+01_rb,0.73977e+01_rb,0.91870e+01_rb,0.92723e+01_rb,0.64598e+01_rb /)
      kbo(:, 1,31,11) = (/ &
        & 0.17244e+01_rb,0.48370e+01_rb,0.59156e+01_rb,0.60001e+01_rb,0.42555e+01_rb /)
      kbo(:, 2,31,11) = (/ &
        & 0.18755e+01_rb,0.54107e+01_rb,0.66999e+01_rb,0.67904e+01_rb,0.47644e+01_rb /)
      kbo(:, 3,31,11) = (/ &
        & 0.20573e+01_rb,0.60363e+01_rb,0.75402e+01_rb,0.76274e+01_rb,0.53155e+01_rb /)
      kbo(:, 4,31,11) = (/ &
        & 0.22829e+01_rb,0.67094e+01_rb,0.84079e+01_rb,0.84962e+01_rb,0.58901e+01_rb /)
      kbo(:, 5,31,11) = (/ &
        & 0.25590e+01_rb,0.74195e+01_rb,0.92909e+01_rb,0.93994e+01_rb,0.64871e+01_rb /)
      kbo(:, 1,32,11) = (/ &
        & 0.16069e+01_rb,0.48030e+01_rb,0.59579e+01_rb,0.60639e+01_rb,0.42356e+01_rb /)
      kbo(:, 2,32,11) = (/ &
        & 0.17705e+01_rb,0.54037e+01_rb,0.67788e+01_rb,0.68874e+01_rb,0.47712e+01_rb /)
      kbo(:, 3,32,11) = (/ &
        & 0.19742e+01_rb,0.60574e+01_rb,0.76448e+01_rb,0.77491e+01_rb,0.53434e+01_rb /)
      kbo(:, 4,32,11) = (/ &
        & 0.22247e+01_rb,0.67560e+01_rb,0.85310e+01_rb,0.86385e+01_rb,0.59381e+01_rb /)
      kbo(:, 5,32,11) = (/ &
        & 0.25238e+01_rb,0.74880e+01_rb,0.94292e+01_rb,0.95570e+01_rb,0.65497e+01_rb /)
      kbo(:, 1,33,11) = (/ &
        & 0.15128e+01_rb,0.48140e+01_rb,0.60501e+01_rb,0.61714e+01_rb,0.42550e+01_rb /)
      kbo(:, 2,33,11) = (/ &
        & 0.16946e+01_rb,0.54417e+01_rb,0.69000e+01_rb,0.70212e+01_rb,0.48155e+01_rb /)
      kbo(:, 3,33,11) = (/ &
        & 0.19206e+01_rb,0.61213e+01_rb,0.77839e+01_rb,0.79020e+01_rb,0.54069e+01_rb /)
      kbo(:, 4,33,11) = (/ &
        & 0.21946e+01_rb,0.68422e+01_rb,0.86827e+01_rb,0.88075e+01_rb,0.60169e+01_rb /)
      kbo(:, 5,33,11) = (/ &
        & 0.25173e+01_rb,0.75913e+01_rb,0.95924e+01_rb,0.97373e+01_rb,0.66398e+01_rb /)
      kbo(:, 1,34,11) = (/ &
        & 0.14356e+01_rb,0.48418e+01_rb,0.61495e+01_rb,0.62828e+01_rb,0.42896e+01_rb /)
      kbo(:, 2,34,11) = (/ &
        & 0.16357e+01_rb,0.54935e+01_rb,0.70184e+01_rb,0.71516e+01_rb,0.48691e+01_rb /)
      kbo(:, 3,34,11) = (/ &
        & 0.18838e+01_rb,0.61957e+01_rb,0.79151e+01_rb,0.80457e+01_rb,0.54742e+01_rb /)
      kbo(:, 4,34,11) = (/ &
        & 0.21788e+01_rb,0.69325e+01_rb,0.88256e+01_rb,0.89643e+01_rb,0.60956e+01_rb /)
      kbo(:, 5,34,11) = (/ &
        & 0.25238e+01_rb,0.76930e+01_rb,0.97439e+01_rb,0.99021e+01_rb,0.67264e+01_rb /)
      kbo(:, 1,35,11) = (/ &
        & 0.13620e+01_rb,0.48400e+01_rb,0.61911e+01_rb,0.63341e+01_rb,0.42950e+01_rb /)
      kbo(:, 2,35,11) = (/ &
        & 0.15765e+01_rb,0.55077e+01_rb,0.70717e+01_rb,0.72149e+01_rb,0.48872e+01_rb /)
      kbo(:, 3,35,11) = (/ &
        & 0.18416e+01_rb,0.62238e+01_rb,0.79778e+01_rb,0.81173e+01_rb,0.55014e+01_rb /)
      kbo(:, 4,35,11) = (/ &
        & 0.21528e+01_rb,0.69694e+01_rb,0.88951e+01_rb,0.90430e+01_rb,0.61304e+01_rb /)
      kbo(:, 5,35,11) = (/ &
        & 0.25135e+01_rb,0.77370e+01_rb,0.98191e+01_rb,0.99865e+01_rb,0.67662e+01_rb /)
      kbo(:, 1,36,11) = (/ &
        & 0.12836e+01_rb,0.47892e+01_rb,0.61549e+01_rb,0.63050e+01_rb,0.42540e+01_rb /)
      kbo(:, 2,36,11) = (/ &
        & 0.15070e+01_rb,0.54649e+01_rb,0.70416e+01_rb,0.71923e+01_rb,0.48536e+01_rb /)
      kbo(:, 3,36,11) = (/ &
        & 0.17830e+01_rb,0.61872e+01_rb,0.79531e+01_rb,0.80994e+01_rb,0.54734e+01_rb /)
      kbo(:, 4,36,11) = (/ &
        & 0.21055e+01_rb,0.69369e+01_rb,0.88745e+01_rb,0.90288e+01_rb,0.61066e+01_rb /)
      kbo(:, 5,36,11) = (/ &
        & 0.24754e+01_rb,0.77090e+01_rb,0.98023e+01_rb,0.99759e+01_rb,0.67459e+01_rb /)
      kbo(:, 1,37,11) = (/ &
        & 0.11878e+01_rb,0.46541e+01_rb,0.59975e+01_rb,0.61531e+01_rb,0.41357e+01_rb /)
      kbo(:, 2,37,11) = (/ &
        & 0.14114e+01_rb,0.53279e+01_rb,0.68836e+01_rb,0.70407e+01_rb,0.47366e+01_rb /)
      kbo(:, 3,37,11) = (/ &
        & 0.16907e+01_rb,0.60488e+01_rb,0.77961e+01_rb,0.79492e+01_rb,0.53587e+01_rb /)
      kbo(:, 4,37,11) = (/ &
        & 0.20171e+01_rb,0.67987e+01_rb,0.87214e+01_rb,0.88782e+01_rb,0.59931e+01_rb /)
      kbo(:, 5,37,11) = (/ &
        & 0.23879e+01_rb,0.75721e+01_rb,0.96505e+01_rb,0.98263e+01_rb,0.66346e+01_rb /)
      kbo(:, 1,38,11) = (/ &
        & 0.11019e+01_rb,0.45241e+01_rb,0.58411e+01_rb,0.60015e+01_rb,0.40206e+01_rb /)
      kbo(:, 2,38,11) = (/ &
        & 0.13240e+01_rb,0.51938e+01_rb,0.67248e+01_rb,0.68881e+01_rb,0.46215e+01_rb /)
      kbo(:, 3,38,11) = (/ &
        & 0.16050e+01_rb,0.59127e+01_rb,0.76377e+01_rb,0.77969e+01_rb,0.52444e+01_rb /)
      kbo(:, 4,38,11) = (/ &
        & 0.19344e+01_rb,0.66614e+01_rb,0.85656e+01_rb,0.87241e+01_rb,0.58797e+01_rb /)
      kbo(:, 5,38,11) = (/ &
        & 0.23053e+01_rb,0.74352e+01_rb,0.94961e+01_rb,0.96730e+01_rb,0.65232e+01_rb /)
      kbo(:, 1,39,11) = (/ &
        & 0.10261e+01_rb,0.44035e+01_rb,0.56925e+01_rb,0.58571e+01_rb,0.39130e+01_rb /)
      kbo(:, 2,39,11) = (/ &
        & 0.12458e+01_rb,0.50694e+01_rb,0.65735e+01_rb,0.67411e+01_rb,0.45130e+01_rb /)
      kbo(:, 3,39,11) = (/ &
        & 0.15281e+01_rb,0.57844e+01_rb,0.74853e+01_rb,0.76497e+01_rb,0.51353e+01_rb /)
      kbo(:, 4,39,11) = (/ &
        & 0.18590e+01_rb,0.65310e+01_rb,0.84139e+01_rb,0.85752e+01_rb,0.57711e+01_rb /)
      kbo(:, 5,39,11) = (/ &
        & 0.22296e+01_rb,0.73031e+01_rb,0.93461e+01_rb,0.95233e+01_rb,0.64155e+01_rb /)
      kbo(:, 1,40,11) = (/ &
        & 0.94020e+00_rb,0.42252e+01_rb,0.54618e+01_rb,0.56305e+01_rb,0.37514e+01_rb /)
      kbo(:, 2,40,11) = (/ &
        & 0.11510e+01_rb,0.48808e+01_rb,0.63359e+01_rb,0.65051e+01_rb,0.43464e+01_rb /)
      kbo(:, 3,40,11) = (/ &
        & 0.14267e+01_rb,0.55865e+01_rb,0.72429e+01_rb,0.74116e+01_rb,0.49659e+01_rb /)
      kbo(:, 4,40,11) = (/ &
        & 0.17534e+01_rb,0.63284e+01_rb,0.81706e+01_rb,0.83346e+01_rb,0.56004e+01_rb /)
      kbo(:, 5,40,11) = (/ &
        & 0.21196e+01_rb,0.70954e+01_rb,0.91038e+01_rb,0.92790e+01_rb,0.62443e+01_rb /)
      kbo(:, 1,41,11) = (/ &
        & 0.86129e+00_rb,0.40457e+01_rb,0.52264e+01_rb,0.53976e+01_rb,0.35879e+01_rb /)
      kbo(:, 2,41,11) = (/ &
        & 0.10633e+01_rb,0.46915e+01_rb,0.60924e+01_rb,0.62635e+01_rb,0.41768e+01_rb /)
      kbo(:, 3,41,11) = (/ &
        & 0.13283e+01_rb,0.53845e+01_rb,0.69926e+01_rb,0.71649e+01_rb,0.47921e+01_rb /)
      kbo(:, 4,41,11) = (/ &
        & 0.16485e+01_rb,0.61201e+01_rb,0.79175e+01_rb,0.80844e+01_rb,0.54240e+01_rb /)
      kbo(:, 5,41,11) = (/ &
        & 0.20093e+01_rb,0.68812e+01_rb,0.88522e+01_rb,0.90249e+01_rb,0.60668e+01_rb /)
      kbo(:, 1,42,11) = (/ &
        & 0.79123e+00_rb,0.38735e+01_rb,0.49960e+01_rb,0.51699e+01_rb,0.34287e+01_rb /)
      kbo(:, 2,42,11) = (/ &
        & 0.98295e+00_rb,0.45072e+01_rb,0.58533e+01_rb,0.60254e+01_rb,0.40109e+01_rb /)
      kbo(:, 3,42,11) = (/ &
        & 0.12364e+01_rb,0.51876e+01_rb,0.67459e+01_rb,0.69214e+01_rb,0.46212e+01_rb /)
      kbo(:, 4,42,11) = (/ &
        & 0.15484e+01_rb,0.59150e+01_rb,0.76665e+01_rb,0.78373e+01_rb,0.52503e+01_rb /)
      kbo(:, 5,42,11) = (/ &
        & 0.19026e+01_rb,0.66716e+01_rb,0.86018e+01_rb,0.87728e+01_rb,0.58915e+01_rb /)
      kbo(:, 1,43,11) = (/ &
        & 0.71984e+00_rb,0.36729e+01_rb,0.47235e+01_rb,0.48975e+01_rb,0.32397e+01_rb /)
      kbo(:, 2,43,11) = (/ &
        & 0.89808e+00_rb,0.42916e+01_rb,0.55697e+01_rb,0.57443e+01_rb,0.38153e+01_rb /)
      kbo(:, 3,43,11) = (/ &
        & 0.11360e+01_rb,0.49574e+01_rb,0.64519e+01_rb,0.66289e+01_rb,0.44184e+01_rb /)
      kbo(:, 4,43,11) = (/ &
        & 0.14345e+01_rb,0.56728e+01_rb,0.73660e+01_rb,0.75410e+01_rb,0.50437e+01_rb /)
      kbo(:, 5,43,11) = (/ &
        & 0.17793e+01_rb,0.64228e+01_rb,0.82999e+01_rb,0.84705e+01_rb,0.56823e+01_rb /)
      kbo(:, 1,44,11) = (/ &
        & 0.65285e+00_rb,0.34667e+01_rb,0.44372e+01_rb,0.46082e+01_rb,0.30399e+01_rb /)
      kbo(:, 2,44,11) = (/ &
        & 0.81657e+00_rb,0.40674e+01_rb,0.52701e+01_rb,0.54466e+01_rb,0.36094e+01_rb /)
      kbo(:, 3,44,11) = (/ &
        & 0.10370e+01_rb,0.47171e+01_rb,0.61405e+01_rb,0.63175e+01_rb,0.42038e+01_rb /)
      kbo(:, 4,44,11) = (/ &
        & 0.13194e+01_rb,0.54184e+01_rb,0.70473e+01_rb,0.72248e+01_rb,0.48237e+01_rb /)
      kbo(:, 5,44,11) = (/ &
        & 0.16514e+01_rb,0.61584e+01_rb,0.79760e+01_rb,0.81477e+01_rb,0.54581e+01_rb /)
      kbo(:, 1,45,11) = (/ &
        & 0.59324e+00_rb,0.32687e+01_rb,0.41610e+01_rb,0.43270e+01_rb,0.28437e+01_rb /)
      kbo(:, 2,45,11) = (/ &
        & 0.74292e+00_rb,0.38502e+01_rb,0.49754e+01_rb,0.51525e+01_rb,0.34081e+01_rb /)
      kbo(:, 3,45,11) = (/ &
        & 0.94606e+00_rb,0.44838e+01_rb,0.58347e+01_rb,0.60112e+01_rb,0.39932e+01_rb /)
      kbo(:, 4,45,11) = (/ &
        & 0.12103e+01_rb,0.51691e+01_rb,0.67312e+01_rb,0.69113e+01_rb,0.46063e+01_rb /)
      kbo(:, 5,45,11) = (/ &
        & 0.15283e+01_rb,0.58970e+01_rb,0.76529e+01_rb,0.78283e+01_rb,0.52365e+01_rb /)
      kbo(:, 1,46,11) = (/ &
        & 0.53665e+00_rb,0.30665e+01_rb,0.38788e+01_rb,0.40344e+01_rb,0.26396e+01_rb /)
      kbo(:, 2,46,11) = (/ &
        & 0.67326e+00_rb,0.36275e+01_rb,0.46685e+01_rb,0.48445e+01_rb,0.31970e+01_rb /)
      kbo(:, 3,46,11) = (/ &
        & 0.85964e+00_rb,0.42450e+01_rb,0.55161e+01_rb,0.56940e+01_rb,0.37742e+01_rb /)
      kbo(:, 4,46,11) = (/ &
        & 0.11026e+01_rb,0.49112e+01_rb,0.63992e+01_rb,0.65795e+01_rb,0.43785e+01_rb /)
      kbo(:, 5,46,11) = (/ &
        & 0.14030e+01_rb,0.56247e+01_rb,0.73128e+01_rb,0.74918e+01_rb,0.50037e+01_rb /)
      kbo(:, 1,47,11) = (/ &
        & 0.47965e+00_rb,0.28512e+01_rb,0.35773e+01_rb,0.37190e+01_rb,0.24179e+01_rb /)
      kbo(:, 2,47,11) = (/ &
        & 0.60499e+00_rb,0.33895e+01_rb,0.43353e+01_rb,0.45070e+01_rb,0.29653e+01_rb /)
      kbo(:, 3,47,11) = (/ &
        & 0.77211e+00_rb,0.39857e+01_rb,0.51656e+01_rb,0.53439e+01_rb,0.35346e+01_rb /)
      kbo(:, 4,47,11) = (/ &
        & 0.99280e+00_rb,0.46320e+01_rb,0.60346e+01_rb,0.62138e+01_rb,0.41278e+01_rb /)
      kbo(:, 5,47,11) = (/ &
        & 0.12711e+01_rb,0.53272e+01_rb,0.69377e+01_rb,0.71189e+01_rb,0.47464e+01_rb /)
      kbo(:, 1,48,11) = (/ &
        & 0.42651e+00_rb,0.26445e+01_rb,0.32894e+01_rb,0.34148e+01_rb,0.22033e+01_rb /)
      kbo(:, 2,48,11) = (/ &
        & 0.54388e+00_rb,0.31611e+01_rb,0.40161e+01_rb,0.41791e+01_rb,0.27373e+01_rb /)
      kbo(:, 3,48,11) = (/ &
        & 0.69368e+00_rb,0.37347e+01_rb,0.48206e+01_rb,0.49989e+01_rb,0.32997e+01_rb /)
      kbo(:, 4,48,11) = (/ &
        & 0.89279e+00_rb,0.43620e+01_rb,0.56767e+01_rb,0.58554e+01_rb,0.38822e+01_rb /)
      kbo(:, 5,48,11) = (/ &
        & 0.11484e+01_rb,0.50369e+01_rb,0.65668e+01_rb,0.67495e+01_rb,0.44918e+01_rb /)
      kbo(:, 1,49,11) = (/ &
        & 0.37713e+00_rb,0.24439e+01_rb,0.30160e+01_rb,0.31206e+01_rb,0.19976e+01_rb /)
      kbo(:, 2,49,11) = (/ &
        & 0.48766e+00_rb,0.29434e+01_rb,0.37104e+01_rb,0.38599e+01_rb,0.25143e+01_rb /)
      kbo(:, 3,49,11) = (/ &
        & 0.62407e+00_rb,0.34939e+01_rb,0.44847e+01_rb,0.46596e+01_rb,0.30680e+01_rb /)
      kbo(:, 4,49,11) = (/ &
        & 0.80298e+00_rb,0.41006e+01_rb,0.53255e+01_rb,0.55049e+01_rb,0.36413e+01_rb /)
      kbo(:, 5,49,11) = (/ &
        & 0.10353e+01_rb,0.47554e+01_rb,0.62010e+01_rb,0.63820e+01_rb,0.42407e+01_rb /)
      kbo(:, 1,50,11) = (/ &
        & 0.33334e+00_rb,0.22573e+01_rb,0.27730e+01_rb,0.28537e+01_rb,0.18141e+01_rb /)
      kbo(:, 2,50,11) = (/ &
        & 0.43824e+00_rb,0.27477e+01_rb,0.34359e+01_rb,0.35712e+01_rb,0.23112e+01_rb /)
      kbo(:, 3,50,11) = (/ &
        & 0.56555e+00_rb,0.32767e+01_rb,0.41807e+01_rb,0.43497e+01_rb,0.28540e+01_rb /)
      kbo(:, 4,50,11) = (/ &
        & 0.72662e+00_rb,0.38631e+01_rb,0.50003e+01_rb,0.51798e+01_rb,0.34206e+01_rb /)
      kbo(:, 5,50,11) = (/ &
        & 0.93811e+00_rb,0.44998e+01_rb,0.58635e+01_rb,0.60433e+01_rb,0.40092e+01_rb /)
      kbo(:, 1,51,11) = (/ &
        & 0.29344e+00_rb,0.20792e+01_rb,0.25493e+01_rb,0.26045e+01_rb,0.16447e+01_rb /)
      kbo(:, 2,51,11) = (/ &
        & 0.39320e+00_rb,0.25633e+01_rb,0.31811e+01_rb,0.32996e+01_rb,0.21202e+01_rb /)
      kbo(:, 3,51,11) = (/ &
        & 0.51335e+00_rb,0.30739e+01_rb,0.38968e+01_rb,0.40554e+01_rb,0.26493e+01_rb /)
      kbo(:, 4,51,11) = (/ &
        & 0.65991e+00_rb,0.36396e+01_rb,0.46910e+01_rb,0.48692e+01_rb,0.32095e+01_rb /)
      kbo(:, 5,51,11) = (/ &
        & 0.85305e+00_rb,0.42602e+01_rb,0.55428e+01_rb,0.57227e+01_rb,0.37890e+01_rb /)
      kbo(:, 1,52,11) = (/ &
        & 0.25634e+00_rb,0.19065e+01_rb,0.23373e+01_rb,0.23675e+01_rb,0.14849e+01_rb /)
      kbo(:, 2,52,11) = (/ &
        & 0.35085e+00_rb,0.23832e+01_rb,0.29380e+01_rb,0.30363e+01_rb,0.19379e+01_rb /)
      kbo(:, 3,52,11) = (/ &
        & 0.46463e+00_rb,0.28806e+01_rb,0.36245e+01_rb,0.37705e+01_rb,0.24499e+01_rb /)
      kbo(:, 4,52,11) = (/ &
        & 0.60014e+00_rb,0.34247e+01_rb,0.43901e+01_rb,0.45644e+01_rb,0.30013e+01_rb /)
      kbo(:, 5,52,11) = (/ &
        & 0.77502e+00_rb,0.40268e+01_rb,0.52264e+01_rb,0.54064e+01_rb,0.35729e+01_rb /)
      kbo(:, 1,53,11) = (/ &
        & 0.22198e+00_rb,0.17382e+01_rb,0.21355e+01_rb,0.21443e+01_rb,0.13321e+01_rb /)
      kbo(:, 2,53,11) = (/ &
        & 0.31088e+00_rb,0.22037e+01_rb,0.27070e+01_rb,0.27809e+01_rb,0.17631e+01_rb /)
      kbo(:, 3,53,11) = (/ &
        & 0.41868e+00_rb,0.26937e+01_rb,0.33628e+01_rb,0.34937e+01_rb,0.22551e+01_rb /)
      kbo(:, 4,53,11) = (/ &
        & 0.54590e+00_rb,0.32172e+01_rb,0.40994e+01_rb,0.42669e+01_rb,0.27950e+01_rb /)
      kbo(:, 5,53,11) = (/ &
        & 0.70382e+00_rb,0.37993e+01_rb,0.49134e+01_rb,0.50934e+01_rb,0.33609e+01_rb /)
      kbo(:, 1,54,11) = (/ &
        & 0.19279e+00_rb,0.15884e+01_rb,0.19576e+01_rb,0.19516e+01_rb,0.11991e+01_rb /)
      kbo(:, 2,54,11) = (/ &
        & 0.27614e+00_rb,0.20423e+01_rb,0.25053e+01_rb,0.25559e+01_rb,0.16106e+01_rb /)
      kbo(:, 3,54,11) = (/ &
        & 0.37900e+00_rb,0.25264e+01_rb,0.31323e+01_rb,0.32473e+01_rb,0.20825e+01_rb /)
      kbo(:, 4,54,11) = (/ &
        & 0.49954e+00_rb,0.30344e+01_rb,0.38425e+01_rb,0.39994e+01_rb,0.26090e+01_rb /)
      kbo(:, 5,54,11) = (/ &
        & 0.64483e+00_rb,0.35967e+01_rb,0.46320e+01_rb,0.48104e+01_rb,0.31683e+01_rb /)
      kbo(:, 1,55,11) = (/ &
        & 0.16752e+00_rb,0.14506e+01_rb,0.17924e+01_rb,0.17780e+01_rb,0.10766e+01_rb /)
      kbo(:, 2,55,11) = (/ &
        & 0.24546e+00_rb,0.18924e+01_rb,0.23215e+01_rb,0.23502e+01_rb,0.14718e+01_rb /)
      kbo(:, 3,55,11) = (/ &
        & 0.34282e+00_rb,0.23686e+01_rb,0.29203e+01_rb,0.30171e+01_rb,0.19238e+01_rb /)
      kbo(:, 4,55,11) = (/ &
        & 0.45751e+00_rb,0.28657e+01_rb,0.36048e+01_rb,0.37504e+01_rb,0.24347e+01_rb /)
      kbo(:, 5,55,11) = (/ &
        & 0.59345e+00_rb,0.34091e+01_rb,0.43692e+01_rb,0.45436e+01_rb,0.29860e+01_rb /)
      kbo(:, 1,56,11) = (/ &
        & 0.14442e+00_rb,0.13191e+01_rb,0.16307e+01_rb,0.16158e+01_rb,0.95966e+00_rb /)
      kbo(:, 2,56,11) = (/ &
        & 0.21629e+00_rb,0.17456e+01_rb,0.21454e+01_rb,0.21553e+01_rb,0.13384e+01_rb /)
      kbo(:, 3,56,11) = (/ &
        & 0.30838e+00_rb,0.22117e+01_rb,0.27184e+01_rb,0.27939e+01_rb,0.17710e+01_rb /)
      kbo(:, 4,56,11) = (/ &
        & 0.41780e+00_rb,0.27024e+01_rb,0.33761e+01_rb,0.35083e+01_rb,0.22647e+01_rb /)
      kbo(:, 5,56,11) = (/ &
        & 0.54633e+00_rb,0.32277e+01_rb,0.41153e+01_rb,0.42835e+01_rb,0.28057e+01_rb /)
      kbo(:, 1,57,11) = (/ &
        & 0.12369e+00_rb,0.11942e+01_rb,0.14751e+01_rb,0.14660e+01_rb,0.84794e+00_rb /)
      kbo(:, 2,57,11) = (/ &
        & 0.18943e+00_rb,0.16025e+01_rb,0.19752e+01_rb,0.19706e+01_rb,0.12104e+01_rb /)
      kbo(:, 3,57,11) = (/ &
        & 0.27580e+00_rb,0.20577e+01_rb,0.25253e+01_rb,0.25785e+01_rb,0.16249e+01_rb /)
      kbo(:, 4,57,11) = (/ &
        & 0.38015e+00_rb,0.25425e+01_rb,0.31554e+01_rb,0.32724e+01_rb,0.20995e+01_rb /)
      kbo(:, 5,57,11) = (/ &
        & 0.50220e+00_rb,0.30528e+01_rb,0.38694e+01_rb,0.40275e+01_rb,0.26279e+01_rb /)
      kbo(:, 1,58,11) = (/ &
        & 0.10619e+00_rb,0.10815e+01_rb,0.13338e+01_rb,0.13340e+01_rb,0.74924e+00_rb /)
      kbo(:, 2,58,11) = (/ &
        & 0.16586e+00_rb,0.14713e+01_rb,0.18183e+01_rb,0.18047e+01_rb,0.10933e+01_rb /)
      kbo(:, 3,58,11) = (/ &
        & 0.24668e+00_rb,0.19150e+01_rb,0.23498e+01_rb,0.23824e+01_rb,0.14931e+01_rb /)
      kbo(:, 4,58,11) = (/ &
        & 0.34613e+00_rb,0.23936e+01_rb,0.29542e+01_rb,0.30545e+01_rb,0.19489e+01_rb /)
      kbo(:, 5,58,11) = (/ &
        & 0.46251e+00_rb,0.28928e+01_rb,0.36439e+01_rb,0.37914e+01_rb,0.24628e+01_rb /)
      kbo(:, 1,59,11) = (/ &
        & 0.98781e-01_rb,0.10374e+01_rb,0.12790e+01_rb,0.12834e+01_rb,0.71091e+00_rb /)
      kbo(:, 2,59,11) = (/ &
        & 0.15643e+00_rb,0.14199e+01_rb,0.17560e+01_rb,0.17406e+01_rb,0.10475e+01_rb /)
      kbo(:, 3,59,11) = (/ &
        & 0.23512e+00_rb,0.18585e+01_rb,0.22815e+01_rb,0.23061e+01_rb,0.14408e+01_rb /)
      kbo(:, 4,59,11) = (/ &
        & 0.33250e+00_rb,0.23338e+01_rb,0.28755e+01_rb,0.29686e+01_rb,0.18898e+01_rb /)
      kbo(:, 5,59,11) = (/ &
        & 0.44697e+00_rb,0.28298e+01_rb,0.35555e+01_rb,0.36984e+01_rb,0.23975e+01_rb /)
      kbo(:, 1,13,12) = (/ &
        & 0.30998e+02_rb,0.37385e+02_rb,0.40609e+02_rb,0.38100e+02_rb,0.30845e+02_rb /)
      kbo(:, 2,13,12) = (/ &
        & 0.30490e+02_rb,0.37042e+02_rb,0.40582e+02_rb,0.38272e+02_rb,0.31245e+02_rb /)
      kbo(:, 3,13,12) = (/ &
        & 0.29985e+02_rb,0.36701e+02_rb,0.40520e+02_rb,0.38445e+02_rb,0.31621e+02_rb /)
      kbo(:, 4,13,12) = (/ &
        & 0.29490e+02_rb,0.36329e+02_rb,0.40415e+02_rb,0.38594e+02_rb,0.32000e+02_rb /)
      kbo(:, 5,13,12) = (/ &
        & 0.29015e+02_rb,0.35959e+02_rb,0.40312e+02_rb,0.38750e+02_rb,0.32433e+02_rb /)
      kbo(:, 1,14,12) = (/ &
        & 0.27110e+02_rb,0.35184e+02_rb,0.38617e+02_rb,0.36399e+02_rb,0.28635e+02_rb /)
      kbo(:, 2,14,12) = (/ &
        & 0.26641e+02_rb,0.34930e+02_rb,0.38606e+02_rb,0.36649e+02_rb,0.29079e+02_rb /)
      kbo(:, 3,14,12) = (/ &
        & 0.26184e+02_rb,0.34642e+02_rb,0.38568e+02_rb,0.36926e+02_rb,0.29498e+02_rb /)
      kbo(:, 4,14,12) = (/ &
        & 0.25751e+02_rb,0.34348e+02_rb,0.38555e+02_rb,0.37188e+02_rb,0.29949e+02_rb /)
      kbo(:, 5,14,12) = (/ &
        & 0.25340e+02_rb,0.34077e+02_rb,0.38568e+02_rb,0.37453e+02_rb,0.30433e+02_rb /)
      kbo(:, 1,15,12) = (/ &
        & 0.23379e+02_rb,0.32658e+02_rb,0.36083e+02_rb,0.34396e+02_rb,0.26632e+02_rb /)
      kbo(:, 2,15,12) = (/ &
        & 0.22961e+02_rb,0.32462e+02_rb,0.36143e+02_rb,0.34739e+02_rb,0.27099e+02_rb /)
      kbo(:, 3,15,12) = (/ &
        & 0.22574e+02_rb,0.32277e+02_rb,0.36234e+02_rb,0.35089e+02_rb,0.27554e+02_rb /)
      kbo(:, 4,15,12) = (/ &
        & 0.22226e+02_rb,0.32107e+02_rb,0.36368e+02_rb,0.35457e+02_rb,0.28061e+02_rb /)
      kbo(:, 5,15,12) = (/ &
        & 0.22052e+02_rb,0.31945e+02_rb,0.36473e+02_rb,0.35860e+02_rb,0.28623e+02_rb /)
      kbo(:, 1,16,12) = (/ &
        & 0.19977e+02_rb,0.29953e+02_rb,0.33281e+02_rb,0.32282e+02_rb,0.24730e+02_rb /)
      kbo(:, 2,16,12) = (/ &
        & 0.19633e+02_rb,0.29859e+02_rb,0.33454e+02_rb,0.32701e+02_rb,0.25245e+02_rb /)
      kbo(:, 3,16,12) = (/ &
        & 0.19340e+02_rb,0.29766e+02_rb,0.33693e+02_rb,0.33152e+02_rb,0.25800e+02_rb /)
      kbo(:, 4,16,12) = (/ &
        & 0.19247e+02_rb,0.29689e+02_rb,0.33933e+02_rb,0.33616e+02_rb,0.26378e+02_rb /)
      kbo(:, 5,16,12) = (/ &
        & 0.19403e+02_rb,0.29623e+02_rb,0.34160e+02_rb,0.34111e+02_rb,0.26924e+02_rb /)
      kbo(:, 1,17,12) = (/ &
        & 0.17051e+02_rb,0.27143e+02_rb,0.30473e+02_rb,0.30161e+02_rb,0.22931e+02_rb /)
      kbo(:, 2,17,12) = (/ &
        & 0.16797e+02_rb,0.27123e+02_rb,0.30776e+02_rb,0.30670e+02_rb,0.23518e+02_rb /)
      kbo(:, 3,17,12) = (/ &
        & 0.16741e+02_rb,0.27141e+02_rb,0.31121e+02_rb,0.31186e+02_rb,0.24127e+02_rb /)
      kbo(:, 4,17,12) = (/ &
        & 0.16948e+02_rb,0.27189e+02_rb,0.31472e+02_rb,0.31736e+02_rb,0.24729e+02_rb /)
      kbo(:, 5,17,12) = (/ &
        & 0.17485e+02_rb,0.27288e+02_rb,0.31858e+02_rb,0.32324e+02_rb,0.25297e+02_rb /)
      kbo(:, 1,18,12) = (/ &
        & 0.14597e+02_rb,0.24342e+02_rb,0.27826e+02_rb,0.28130e+02_rb,0.21245e+02_rb /)
      kbo(:, 2,18,12) = (/ &
        & 0.14547e+02_rb,0.24446e+02_rb,0.28237e+02_rb,0.28704e+02_rb,0.21885e+02_rb /)
      kbo(:, 3,18,12) = (/ &
        & 0.14765e+02_rb,0.24583e+02_rb,0.28672e+02_rb,0.29288e+02_rb,0.22535e+02_rb /)
      kbo(:, 4,18,12) = (/ &
        & 0.15315e+02_rb,0.24794e+02_rb,0.29147e+02_rb,0.29934e+02_rb,0.23159e+02_rb /)
      kbo(:, 5,18,12) = (/ &
        & 0.16067e+02_rb,0.25143e+02_rb,0.29634e+02_rb,0.30628e+02_rb,0.23749e+02_rb /)
      kbo(:, 1,19,12) = (/ &
        & 0.12640e+02_rb,0.21753e+02_rb,0.25384e+02_rb,0.26151e+02_rb,0.19661e+02_rb /)
      kbo(:, 2,19,12) = (/ &
        & 0.12835e+02_rb,0.21949e+02_rb,0.25878e+02_rb,0.26810e+02_rb,0.20358e+02_rb /)
      kbo(:, 3,19,12) = (/ &
        & 0.13356e+02_rb,0.22229e+02_rb,0.26410e+02_rb,0.27515e+02_rb,0.21037e+02_rb /)
      kbo(:, 4,19,12) = (/ &
        & 0.14056e+02_rb,0.22664e+02_rb,0.26964e+02_rb,0.28265e+02_rb,0.21685e+02_rb /)
      kbo(:, 5,19,12) = (/ &
        & 0.14855e+02_rb,0.23244e+02_rb,0.27571e+02_rb,0.29046e+02_rb,0.22336e+02_rb /)
      kbo(:, 1,20,12) = (/ &
        & 0.11082e+02_rb,0.19491e+02_rb,0.23194e+02_rb,0.24313e+02_rb,0.18232e+02_rb /)
      kbo(:, 2,20,12) = (/ &
        & 0.11562e+02_rb,0.19787e+02_rb,0.23755e+02_rb,0.25073e+02_rb,0.18946e+02_rb /)
      kbo(:, 3,20,12) = (/ &
        & 0.12224e+02_rb,0.20238e+02_rb,0.24350e+02_rb,0.25880e+02_rb,0.19650e+02_rb /)
      kbo(:, 4,20,12) = (/ &
        & 0.12987e+02_rb,0.20868e+02_rb,0.25012e+02_rb,0.26726e+02_rb,0.20347e+02_rb /)
      kbo(:, 5,20,12) = (/ &
        & 0.13788e+02_rb,0.21712e+02_rb,0.25741e+02_rb,0.27588e+02_rb,0.21074e+02_rb /)
      kbo(:, 1,21,12) = (/ &
        & 0.98833e+01_rb,0.17574e+02_rb,0.21209e+02_rb,0.22623e+02_rb,0.16925e+02_rb /)
      kbo(:, 2,21,12) = (/ &
        & 0.10489e+02_rb,0.18010e+02_rb,0.21858e+02_rb,0.23471e+02_rb,0.17666e+02_rb /)
      kbo(:, 3,21,12) = (/ &
        & 0.11209e+02_rb,0.18622e+02_rb,0.22555e+02_rb,0.24382e+02_rb,0.18394e+02_rb /)
      kbo(:, 4,21,12) = (/ &
        & 0.11974e+02_rb,0.19474e+02_rb,0.23320e+02_rb,0.25315e+02_rb,0.19156e+02_rb /)
      kbo(:, 5,21,12) = (/ &
        & 0.12743e+02_rb,0.20530e+02_rb,0.24213e+02_rb,0.26280e+02_rb,0.19957e+02_rb /)
      kbo(:, 1,22,12) = (/ &
        & 0.89432e+01_rb,0.16023e+02_rb,0.19521e+02_rb,0.21142e+02_rb,0.15785e+02_rb /)
      kbo(:, 2,22,12) = (/ &
        & 0.96012e+01_rb,0.16615e+02_rb,0.20251e+02_rb,0.22105e+02_rb,0.16565e+02_rb /)
      kbo(:, 3,22,12) = (/ &
        & 0.10308e+02_rb,0.17429e+02_rb,0.21053e+02_rb,0.23102e+02_rb,0.17352e+02_rb /)
      kbo(:, 4,22,12) = (/ &
        & 0.11034e+02_rb,0.18474e+02_rb,0.21986e+02_rb,0.24140e+02_rb,0.18184e+02_rb /)
      kbo(:, 5,22,12) = (/ &
        & 0.11779e+02_rb,0.19675e+02_rb,0.23025e+02_rb,0.25198e+02_rb,0.19053e+02_rb /)
      kbo(:, 1,23,12) = (/ &
        & 0.81558e+01_rb,0.14793e+02_rb,0.18088e+02_rb,0.19855e+02_rb,0.14800e+02_rb /)
      kbo(:, 2,23,12) = (/ &
        & 0.87957e+01_rb,0.15543e+02_rb,0.18895e+02_rb,0.20923e+02_rb,0.15621e+02_rb /)
      kbo(:, 3,23,12) = (/ &
        & 0.94593e+01_rb,0.16540e+02_rb,0.19849e+02_rb,0.22025e+02_rb,0.16480e+02_rb /)
      kbo(:, 4,23,12) = (/ &
        & 0.10153e+02_rb,0.17706e+02_rb,0.20923e+02_rb,0.23160e+02_rb,0.17388e+02_rb /)
      kbo(:, 5,23,12) = (/ &
        & 0.10876e+02_rb,0.18993e+02_rb,0.22113e+02_rb,0.24310e+02_rb,0.18322e+02_rb /)
      kbo(:, 1,24,12) = (/ &
        & 0.74526e+01_rb,0.13830e+02_rb,0.16894e+02_rb,0.18769e+02_rb,0.13959e+02_rb /)
      kbo(:, 2,24,12) = (/ &
        & 0.80517e+01_rb,0.14766e+02_rb,0.17833e+02_rb,0.19936e+02_rb,0.14836e+02_rb /)
      kbo(:, 3,24,12) = (/ &
        & 0.86841e+01_rb,0.15873e+02_rb,0.18916e+02_rb,0.21150e+02_rb,0.15773e+02_rb /)
      kbo(:, 4,24,12) = (/ &
        & 0.93535e+01_rb,0.17122e+02_rb,0.20131e+02_rb,0.22383e+02_rb,0.16742e+02_rb /)
      kbo(:, 5,24,12) = (/ &
        & 0.10068e+02_rb,0.18482e+02_rb,0.21482e+02_rb,0.23634e+02_rb,0.17753e+02_rb /)
      kbo(:, 1,25,12) = (/ &
        & 0.68105e+01_rb,0.13122e+02_rb,0.15953e+02_rb,0.17890e+02_rb,0.13264e+02_rb /)
      kbo(:, 2,25,12) = (/ &
        & 0.73796e+01_rb,0.14193e+02_rb,0.17028e+02_rb,0.19160e+02_rb,0.14211e+02_rb /)
      kbo(:, 3,25,12) = (/ &
        & 0.79883e+01_rb,0.15386e+02_rb,0.18237e+02_rb,0.20473e+02_rb,0.15219e+02_rb /)
      kbo(:, 4,25,12) = (/ &
        & 0.86473e+01_rb,0.16702e+02_rb,0.19605e+02_rb,0.21810e+02_rb,0.16257e+02_rb /)
      kbo(:, 5,25,12) = (/ &
        & 0.93513e+01_rb,0.18139e+02_rb,0.21087e+02_rb,0.23158e+02_rb,0.17339e+02_rb /)
      kbo(:, 1,26,12) = (/ &
        & 0.62530e+01_rb,0.12619e+02_rb,0.15260e+02_rb,0.17241e+02_rb,0.12729e+02_rb /)
      kbo(:, 2,26,12) = (/ &
        & 0.67955e+01_rb,0.13786e+02_rb,0.16460e+02_rb,0.18602e+02_rb,0.13751e+02_rb /)
      kbo(:, 3,26,12) = (/ &
        & 0.73876e+01_rb,0.15061e+02_rb,0.17816e+02_rb,0.20014e+02_rb,0.14821e+02_rb /)
      kbo(:, 4,26,12) = (/ &
        & 0.80374e+01_rb,0.16453e+02_rb,0.19314e+02_rb,0.21444e+02_rb,0.15933e+02_rb /)
      kbo(:, 5,26,12) = (/ &
        & 0.87427e+01_rb,0.17956e+02_rb,0.20909e+02_rb,0.22878e+02_rb,0.17080e+02_rb /)
      kbo(:, 1,27,12) = (/ &
        & 0.57700e+01_rb,0.12257e+02_rb,0.14782e+02_rb,0.16780e+02_rb,0.12335e+02_rb /)
      kbo(:, 2,27,12) = (/ &
        & 0.62864e+01_rb,0.13513e+02_rb,0.16114e+02_rb,0.18231e+02_rb,0.13431e+02_rb /)
      kbo(:, 3,27,12) = (/ &
        & 0.68675e+01_rb,0.14868e+02_rb,0.17601e+02_rb,0.19730e+02_rb,0.14565e+02_rb /)
      kbo(:, 4,27,12) = (/ &
        & 0.75138e+01_rb,0.16329e+02_rb,0.19199e+02_rb,0.21244e+02_rb,0.15742e+02_rb /)
      kbo(:, 5,27,12) = (/ &
        & 0.82289e+01_rb,0.17883e+02_rb,0.20886e+02_rb,0.22748e+02_rb,0.16940e+02_rb /)
      kbo(:, 1,28,12) = (/ &
        & 0.53548e+01_rb,0.12024e+02_rb,0.14494e+02_rb,0.16490e+02_rb,0.12074e+02_rb /)
      kbo(:, 2,28,12) = (/ &
        & 0.58567e+01_rb,0.13364e+02_rb,0.15954e+02_rb,0.18023e+02_rb,0.13231e+02_rb /)
      kbo(:, 3,28,12) = (/ &
        & 0.64249e+01_rb,0.14793e+02_rb,0.17548e+02_rb,0.19599e+02_rb,0.14431e+02_rb /)
      kbo(:, 4,28,12) = (/ &
        & 0.70763e+01_rb,0.16306e+02_rb,0.19232e+02_rb,0.21185e+02_rb,0.15664e+02_rb /)
      kbo(:, 5,28,12) = (/ &
        & 0.78108e+01_rb,0.17913e+02_rb,0.20999e+02_rb,0.22748e+02_rb,0.16904e+02_rb /)
      kbo(:, 1,29,12) = (/ &
        & 0.50045e+01_rb,0.11918e+02_rb,0.14389e+02_rb,0.16363e+02_rb,0.11938e+02_rb /)
      kbo(:, 2,29,12) = (/ &
        & 0.55030e+01_rb,0.13334e+02_rb,0.15959e+02_rb,0.17970e+02_rb,0.13155e+02_rb /)
      kbo(:, 3,29,12) = (/ &
        & 0.60662e+01_rb,0.14820e+02_rb,0.17638e+02_rb,0.19611e+02_rb,0.14408e+02_rb /)
      kbo(:, 4,29,12) = (/ &
        & 0.67325e+01_rb,0.16392e+02_rb,0.19392e+02_rb,0.21253e+02_rb,0.15686e+02_rb /)
      kbo(:, 5,29,12) = (/ &
        & 0.74925e+01_rb,0.18044e+02_rb,0.21206e+02_rb,0.22856e+02_rb,0.16965e+02_rb /)
      kbo(:, 1,30,12) = (/ &
        & 0.47172e+01_rb,0.11917e+02_rb,0.14428e+02_rb,0.16368e+02_rb,0.11903e+02_rb /)
      kbo(:, 2,30,12) = (/ &
        & 0.52125e+01_rb,0.13400e+02_rb,0.16089e+02_rb,0.18040e+02_rb,0.13179e+02_rb /)
      kbo(:, 3,30,12) = (/ &
        & 0.57884e+01_rb,0.14940e+02_rb,0.17829e+02_rb,0.19728e+02_rb,0.14475e+02_rb /)
      kbo(:, 4,30,12) = (/ &
        & 0.64722e+01_rb,0.16556e+02_rb,0.19636e+02_rb,0.21408e+02_rb,0.15787e+02_rb /)
      kbo(:, 5,30,12) = (/ &
        & 0.72704e+01_rb,0.18251e+02_rb,0.21479e+02_rb,0.23035e+02_rb,0.17093e+02_rb /)
      kbo(:, 1,31,12) = (/ &
        & 0.44858e+01_rb,0.12017e+02_rb,0.14590e+02_rb,0.16495e+02_rb,0.11969e+02_rb /)
      kbo(:, 2,31,12) = (/ &
        & 0.49908e+01_rb,0.13550e+02_rb,0.16315e+02_rb,0.18215e+02_rb,0.13287e+02_rb /)
      kbo(:, 3,31,12) = (/ &
        & 0.55864e+01_rb,0.15140e+02_rb,0.18101e+02_rb,0.19933e+02_rb,0.14620e+02_rb /)
      kbo(:, 4,31,12) = (/ &
        & 0.63024e+01_rb,0.16801e+02_rb,0.19948e+02_rb,0.21636e+02_rb,0.15956e+02_rb /)
      kbo(:, 5,31,12) = (/ &
        & 0.71504e+01_rb,0.18530e+02_rb,0.21815e+02_rb,0.23280e+02_rb,0.17277e+02_rb /)
      kbo(:, 1,32,12) = (/ &
        & 0.43064e+01_rb,0.12199e+02_rb,0.14841e+02_rb,0.16712e+02_rb,0.12112e+02_rb /)
      kbo(:, 2,32,12) = (/ &
        & 0.48346e+01_rb,0.13779e+02_rb,0.16613e+02_rb,0.18463e+02_rb,0.13462e+02_rb /)
      kbo(:, 3,32,12) = (/ &
        & 0.54614e+01_rb,0.15404e+02_rb,0.18438e+02_rb,0.20209e+02_rb,0.14821e+02_rb /)
      kbo(:, 4,32,12) = (/ &
        & 0.62177e+01_rb,0.17102e+02_rb,0.20311e+02_rb,0.21920e+02_rb,0.16172e+02_rb /)
      kbo(:, 5,32,12) = (/ &
        & 0.71259e+01_rb,0.18855e+02_rb,0.22195e+02_rb,0.23574e+02_rb,0.17506e+02_rb /)
      kbo(:, 1,33,12) = (/ &
        & 0.41801e+01_rb,0.12450e+02_rb,0.15154e+02_rb,0.16994e+02_rb,0.12315e+02_rb /)
      kbo(:, 2,33,12) = (/ &
        & 0.47391e+01_rb,0.14064e+02_rb,0.16963e+02_rb,0.18765e+02_rb,0.13689e+02_rb /)
      kbo(:, 3,33,12) = (/ &
        & 0.54061e+01_rb,0.15717e+02_rb,0.18820e+02_rb,0.20527e+02_rb,0.15064e+02_rb /)
      kbo(:, 4,33,12) = (/ &
        & 0.62146e+01_rb,0.17442e+02_rb,0.20711e+02_rb,0.22242e+02_rb,0.16420e+02_rb /)
      kbo(:, 5,33,12) = (/ &
        & 0.71827e+01_rb,0.19219e+02_rb,0.22608e+02_rb,0.23903e+02_rb,0.17762e+02_rb /)
      kbo(:, 1,34,12) = (/ &
        & 0.40887e+01_rb,0.12691e+02_rb,0.15451e+02_rb,0.17259e+02_rb,0.12512e+02_rb /)
      kbo(:, 2,34,12) = (/ &
        & 0.46819e+01_rb,0.14330e+02_rb,0.17288e+02_rb,0.19046e+02_rb,0.13902e+02_rb /)
      kbo(:, 3,34,12) = (/ &
        & 0.53890e+01_rb,0.16007e+02_rb,0.19166e+02_rb,0.20816e+02_rb,0.15287e+02_rb /)
      kbo(:, 4,34,12) = (/ &
        & 0.62486e+01_rb,0.17752e+02_rb,0.21067e+02_rb,0.22532e+02_rb,0.16650e+02_rb /)
      kbo(:, 5,34,12) = (/ &
        & 0.72682e+01_rb,0.19545e+02_rb,0.22972e+02_rb,0.24194e+02_rb,0.17995e+02_rb /)
      kbo(:, 1,35,12) = (/ &
        & 0.39916e+01_rb,0.12809e+02_rb,0.15601e+02_rb,0.17383e+02_rb,0.12605e+02_rb /)
      kbo(:, 2,35,12) = (/ &
        & 0.46112e+01_rb,0.14460e+02_rb,0.17457e+02_rb,0.19181e+02_rb,0.14005e+02_rb /)
      kbo(:, 3,35,12) = (/ &
        & 0.53474e+01_rb,0.16154e+02_rb,0.19347e+02_rb,0.20958e+02_rb,0.15398e+02_rb /)
      kbo(:, 4,35,12) = (/ &
        & 0.62430e+01_rb,0.17913e+02_rb,0.21258e+02_rb,0.22679e+02_rb,0.16764e+02_rb /)
      kbo(:, 5,35,12) = (/ &
        & 0.72953e+01_rb,0.19716e+02_rb,0.23169e+02_rb,0.24345e+02_rb,0.18113e+02_rb /)
      kbo(:, 1,36,12) = (/ &
        & 0.38696e+01_rb,0.12768e+02_rb,0.15571e+02_rb,0.17332e+02_rb,0.12565e+02_rb /)
      kbo(:, 2,36,12) = (/ &
        & 0.45034e+01_rb,0.14429e+02_rb,0.17436e+02_rb,0.19141e+02_rb,0.13973e+02_rb /)
      kbo(:, 3,36,12) = (/ &
        & 0.52555e+01_rb,0.16128e+02_rb,0.19333e+02_rb,0.20927e+02_rb,0.15372e+02_rb /)
      kbo(:, 4,36,12) = (/ &
        & 0.61640e+01_rb,0.17893e+02_rb,0.21249e+02_rb,0.22656e+02_rb,0.16743e+02_rb /)
      kbo(:, 5,36,12) = (/ &
        & 0.72295e+01_rb,0.19702e+02_rb,0.23168e+02_rb,0.24329e+02_rb,0.18096e+02_rb /)
      kbo(:, 1,37,12) = (/ &
        & 0.36883e+01_rb,0.12493e+02_rb,0.15274e+02_rb,0.17021e+02_rb,0.12322e+02_rb /)
      kbo(:, 2,37,12) = (/ &
        & 0.43193e+01_rb,0.14156e+02_rb,0.17140e+02_rb,0.18841e+02_rb,0.13739e+02_rb /)
      kbo(:, 3,37,12) = (/ &
        & 0.50636e+01_rb,0.15852e+02_rb,0.19039e+02_rb,0.20638e+02_rb,0.15145e+02_rb /)
      kbo(:, 4,37,12) = (/ &
        & 0.59573e+01_rb,0.17616e+02_rb,0.20958e+02_rb,0.22386e+02_rb,0.16525e+02_rb /)
      kbo(:, 5,37,12) = (/ &
        & 0.70089e+01_rb,0.19421e+02_rb,0.22881e+02_rb,0.24071e+02_rb,0.17883e+02_rb /)
      kbo(:, 1,38,12) = (/ &
        & 0.35253e+01_rb,0.12215e+02_rb,0.14971e+02_rb,0.16706e+02_rb,0.12079e+02_rb /)
      kbo(:, 2,38,12) = (/ &
        & 0.41507e+01_rb,0.13877e+02_rb,0.16837e+02_rb,0.18535e+02_rb,0.13501e+02_rb /)
      kbo(:, 3,38,12) = (/ &
        & 0.48852e+01_rb,0.15572e+02_rb,0.18736e+02_rb,0.20342e+02_rb,0.14913e+02_rb /)
      kbo(:, 4,38,12) = (/ &
        & 0.57613e+01_rb,0.17329e+02_rb,0.20656e+02_rb,0.22106e+02_rb,0.16300e+02_rb /)
      kbo(:, 5,38,12) = (/ &
        & 0.67957e+01_rb,0.19130e+02_rb,0.22581e+02_rb,0.23804e+02_rb,0.17665e+02_rb /)
      kbo(:, 1,39,12) = (/ &
        & 0.33825e+01_rb,0.11947e+02_rb,0.14676e+02_rb,0.16402e+02_rb,0.11843e+02_rb /)
      kbo(:, 2,39,12) = (/ &
        & 0.39997e+01_rb,0.13607e+02_rb,0.16542e+02_rb,0.18237e+02_rb,0.13271e+02_rb /)
      kbo(:, 3,39,12) = (/ &
        & 0.47242e+01_rb,0.15300e+02_rb,0.18440e+02_rb,0.20053e+02_rb,0.14687e+02_rb /)
      kbo(:, 4,39,12) = (/ &
        & 0.55821e+01_rb,0.17050e+02_rb,0.20362e+02_rb,0.21831e+02_rb,0.16081e+02_rb /)
      kbo(:, 5,39,12) = (/ &
        & 0.65965e+01_rb,0.18849e+02_rb,0.22289e+02_rb,0.23541e+02_rb,0.17452e+02_rb /)
      kbo(:, 1,40,12) = (/ &
        & 0.31990e+01_rb,0.11518e+02_rb,0.14197e+02_rb,0.15916e+02_rb,0.11466e+02_rb /)
      kbo(:, 2,40,12) = (/ &
        & 0.37954e+01_rb,0.13171e+02_rb,0.16060e+02_rb,0.17759e+02_rb,0.12898e+02_rb /)
      kbo(:, 3,40,12) = (/ &
        & 0.44984e+01_rb,0.14859e+02_rb,0.17955e+02_rb,0.19586e+02_rb,0.14321e+02_rb /)
      kbo(:, 4,40,12) = (/ &
        & 0.53220e+01_rb,0.16595e+02_rb,0.19875e+02_rb,0.21381e+02_rb,0.15725e+02_rb /)
      kbo(:, 5,40,12) = (/ &
        & 0.62999e+01_rb,0.18387e+02_rb,0.21803e+02_rb,0.23109e+02_rb,0.17102e+02_rb /)
      kbo(:, 1,41,12) = (/ &
        & 0.30240e+01_rb,0.11079e+02_rb,0.13703e+02_rb,0.15411e+02_rb,0.11075e+02_rb /)
      kbo(:, 2,41,12) = (/ &
        & 0.35998e+01_rb,0.12722e+02_rb,0.15561e+02_rb,0.17259e+02_rb,0.12509e+02_rb /)
      kbo(:, 3,41,12) = (/ &
        & 0.42766e+01_rb,0.14402e+02_rb,0.17450e+02_rb,0.19096e+02_rb,0.13938e+02_rb /)
      kbo(:, 4,41,12) = (/ &
        & 0.50663e+01_rb,0.16123e+02_rb,0.19365e+02_rb,0.20905e+02_rb,0.15352e+02_rb /)
      kbo(:, 5,41,12) = (/ &
        & 0.60051e+01_rb,0.17908e+02_rb,0.21295e+02_rb,0.22653e+02_rb,0.16735e+02_rb /)
      kbo(:, 1,42,12) = (/ &
        & 0.28652e+01_rb,0.10648e+02_rb,0.13219e+02_rb,0.14913e+02_rb,0.10689e+02_rb /)
      kbo(:, 2,42,12) = (/ &
        & 0.34153e+01_rb,0.12278e+02_rb,0.15063e+02_rb,0.16761e+02_rb,0.12121e+02_rb /)
      kbo(:, 3,42,12) = (/ &
        & 0.40672e+01_rb,0.13952e+02_rb,0.16947e+02_rb,0.18606e+02_rb,0.13557e+02_rb /)
      kbo(:, 4,42,12) = (/ &
        & 0.48264e+01_rb,0.15661e+02_rb,0.18858e+02_rb,0.20426e+02_rb,0.14977e+02_rb /)
      kbo(:, 5,42,12) = (/ &
        & 0.57266e+01_rb,0.17431e+02_rb,0.20787e+02_rb,0.22197e+02_rb,0.16370e+02_rb /)
      kbo(:, 1,43,12) = (/ &
        & 0.26870e+01_rb,0.10134e+02_rb,0.12642e+02_rb,0.14316e+02_rb,0.10228e+02_rb /)
      kbo(:, 2,43,12) = (/ &
        & 0.32108e+01_rb,0.11745e+02_rb,0.14465e+02_rb,0.16161e+02_rb,0.11657e+02_rb /)
      kbo(:, 3,43,12) = (/ &
        & 0.38308e+01_rb,0.13408e+02_rb,0.16340e+02_rb,0.18012e+02_rb,0.13095e+02_rb /)
      kbo(:, 4,43,12) = (/ &
        & 0.45569e+01_rb,0.15107e+02_rb,0.18246e+02_rb,0.19843e+02_rb,0.14521e+02_rb /)
      kbo(:, 5,43,12) = (/ &
        & 0.54100e+01_rb,0.16859e+02_rb,0.20173e+02_rb,0.21637e+02_rb,0.15927e+02_rb /)
      kbo(:, 1,44,12) = (/ &
        & 0.25051e+01_rb,0.95885e+01_rb,0.12032e+02_rb,0.13682e+02_rb,0.97372e+01_rb /)
      kbo(:, 2,44,12) = (/ &
        & 0.30054e+01_rb,0.11180e+02_rb,0.13825e+02_rb,0.15517e+02_rb,0.11159e+02_rb /)
      kbo(:, 3,44,12) = (/ &
        & 0.35923e+01_rb,0.12828e+02_rb,0.15690e+02_rb,0.17372e+02_rb,0.12597e+02_rb /)
      kbo(:, 4,44,12) = (/ &
        & 0.42861e+01_rb,0.14518e+02_rb,0.17591e+02_rb,0.19215e+02_rb,0.14031e+02_rb /)
      kbo(:, 5,44,12) = (/ &
        & 0.50891e+01_rb,0.16248e+02_rb,0.19511e+02_rb,0.21026e+02_rb,0.15447e+02_rb /)
      kbo(:, 1,45,12) = (/ &
        & 0.23284e+01_rb,0.90519e+01_rb,0.11435e+02_rb,0.13049e+02_rb,0.92487e+01_rb /)
      kbo(:, 2,45,12) = (/ &
        & 0.28153e+01_rb,0.10623e+02_rb,0.13197e+02_rb,0.14880e+02_rb,0.10666e+02_rb /)
      kbo(:, 3,45,12) = (/ &
        & 0.33686e+01_rb,0.12254e+02_rb,0.15046e+02_rb,0.16733e+02_rb,0.12101e+02_rb /)
      kbo(:, 4,45,12) = (/ &
        & 0.40279e+01_rb,0.13936e+02_rb,0.16938e+02_rb,0.18583e+02_rb,0.13540e+02_rb /)
      kbo(:, 5,45,12) = (/ &
        & 0.47894e+01_rb,0.15648e+02_rb,0.18852e+02_rb,0.20408e+02_rb,0.14963e+02_rb /)
      kbo(:, 1,46,12) = (/ &
        & 0.21477e+01_rb,0.84947e+01_rb,0.10808e+02_rb,0.12382e+02_rb,0.87316e+01_rb /)
      kbo(:, 2,46,12) = (/ &
        & 0.26237e+01_rb,0.10040e+02_rb,0.12543e+02_rb,0.14206e+02_rb,0.10145e+02_rb /)
      kbo(:, 3,46,12) = (/ &
        & 0.31493e+01_rb,0.11653e+02_rb,0.14367e+02_rb,0.16054e+02_rb,0.11575e+02_rb /)
      kbo(:, 4,46,12) = (/ &
        & 0.37687e+01_rb,0.13319e+02_rb,0.16247e+02_rb,0.17910e+02_rb,0.13015e+02_rb /)
      kbo(:, 5,46,12) = (/ &
        & 0.44919e+01_rb,0.15018e+02_rb,0.18154e+02_rb,0.19746e+02_rb,0.14446e+02_rb /)
      kbo(:, 1,47,12) = (/ &
        & 0.19539e+01_rb,0.78866e+01_rb,0.10121e+02_rb,0.11642e+02_rb,0.81629e+01_rb /)
      kbo(:, 2,47,12) = (/ &
        & 0.24188e+01_rb,0.94011e+01_rb,0.11827e+02_rb,0.13461e+02_rb,0.95671e+01_rb /)
      kbo(:, 3,47,12) = (/ &
        & 0.29183e+01_rb,0.10990e+02_rb,0.13614e+02_rb,0.15297e+02_rb,0.10990e+02_rb /)
      kbo(:, 4,47,12) = (/ &
        & 0.34981e+01_rb,0.12636e+02_rb,0.15480e+02_rb,0.17155e+02_rb,0.12429e+02_rb /)
      kbo(:, 5,47,12) = (/ &
        & 0.41787e+01_rb,0.14323e+02_rb,0.17378e+02_rb,0.19005e+02_rb,0.13866e+02_rb /)
      kbo(:, 1,48,12) = (/ &
        & 0.17686e+01_rb,0.73016e+01_rb,0.94471e+01_rb,0.10908e+02_rb,0.76062e+01_rb /)
      kbo(:, 2,48,12) = (/ &
        & 0.22191e+01_rb,0.87750e+01_rb,0.11125e+02_rb,0.12718e+02_rb,0.89921e+01_rb /)
      kbo(:, 3,48,12) = (/ &
        & 0.27045e+01_rb,0.10336e+02_rb,0.12878e+02_rb,0.14548e+02_rb,0.10409e+02_rb /)
      kbo(:, 4,48,12) = (/ &
        & 0.32460e+01_rb,0.11962e+02_rb,0.14721e+02_rb,0.16399e+02_rb,0.11843e+02_rb /)
      kbo(:, 5,48,12) = (/ &
        & 0.38847e+01_rb,0.13635e+02_rb,0.16605e+02_rb,0.18256e+02_rb,0.13285e+02_rb /)
      kbo(:, 1,49,12) = (/ &
        & 0.15926e+01_rb,0.67448e+01_rb,0.87885e+01_rb,0.10183e+02_rb,0.70558e+01_rb /)
      kbo(:, 2,49,12) = (/ &
        & 0.20265e+01_rb,0.81616e+01_rb,0.10434e+02_rb,0.11978e+02_rb,0.84210e+01_rb /)
      kbo(:, 3,49,12) = (/ &
        & 0.24995e+01_rb,0.96943e+01_rb,0.12158e+02_rb,0.13804e+02_rb,0.98335e+01_rb /)
      kbo(:, 4,49,12) = (/ &
        & 0.30106e+01_rb,0.11296e+02_rb,0.13964e+02_rb,0.15645e+02_rb,0.11261e+02_rb /)
      kbo(:, 5,49,12) = (/ &
        & 0.36091e+01_rb,0.12953e+02_rb,0.15838e+02_rb,0.17505e+02_rb,0.12701e+02_rb /)
      kbo(:, 1,50,12) = (/ &
        & 0.14367e+01_rb,0.62503e+01_rb,0.81839e+01_rb,0.95114e+01_rb,0.65470e+01_rb /)
      kbo(:, 2,50,12) = (/ &
        & 0.18513e+01_rb,0.76037e+01_rb,0.97986e+01_rb,0.11290e+02_rb,0.78962e+01_rb /)
      kbo(:, 3,50,12) = (/ &
        & 0.23123e+01_rb,0.91032e+01_rb,0.11496e+02_rb,0.13108e+02_rb,0.92947e+01_rb /)
      kbo(:, 4,50,12) = (/ &
        & 0.28056e+01_rb,0.10680e+02_rb,0.13267e+02_rb,0.14942e+02_rb,0.10716e+02_rb /)
      kbo(:, 5,50,12) = (/ &
        & 0.33664e+01_rb,0.12318e+02_rb,0.15124e+02_rb,0.16800e+02_rb,0.12153e+02_rb /)
      kbo(:, 1,51,12) = (/ &
        & 0.12944e+01_rb,0.57936e+01_rb,0.76117e+01_rb,0.88742e+01_rb,0.60642e+01_rb /)
      kbo(:, 2,51,12) = (/ &
        & 0.16888e+01_rb,0.70852e+01_rb,0.91933e+01_rb,0.10628e+02_rb,0.73944e+01_rb /)
      kbo(:, 3,51,12) = (/ &
        & 0.21353e+01_rb,0.85416e+01_rb,0.10863e+02_rb,0.12436e+02_rb,0.87735e+01_rb /)
      kbo(:, 4,51,12) = (/ &
        & 0.26169e+01_rb,0.10093e+02_rb,0.12607e+02_rb,0.14266e+02_rb,0.10192e+02_rb /)
      kbo(:, 5,51,12) = (/ &
        & 0.31479e+01_rb,0.11711e+02_rb,0.14438e+02_rb,0.16116e+02_rb,0.11624e+02_rb /)
      kbo(:, 1,52,12) = (/ &
        & 0.11616e+01_rb,0.53570e+01_rb,0.70533e+01_rb,0.82560e+01_rb,0.55953e+01_rb /)
      kbo(:, 2,52,12) = (/ &
        & 0.15340e+01_rb,0.65899e+01_rb,0.86019e+01_rb,0.99750e+01_rb,0.68982e+01_rb /)
      kbo(:, 3,52,12) = (/ &
        & 0.19643e+01_rb,0.79903e+01_rb,0.10241e+02_rb,0.11768e+02_rb,0.82604e+01_rb /)
      kbo(:, 4,52,12) = (/ &
        & 0.24347e+01_rb,0.95157e+01_rb,0.11958e+02_rb,0.13593e+02_rb,0.96707e+01_rb /)
      kbo(:, 5,52,12) = (/ &
        & 0.29410e+01_rb,0.11112e+02_rb,0.13756e+02_rb,0.15434e+02_rb,0.11097e+02_rb /)
      kbo(:, 1,53,12) = (/ &
        & 0.10377e+01_rb,0.49398e+01_rb,0.65141e+01_rb,0.76518e+01_rb,0.51386e+01_rb /)
      kbo(:, 2,53,12) = (/ &
        & 0.13876e+01_rb,0.61188e+01_rb,0.80207e+01_rb,0.93291e+01_rb,0.64095e+01_rb /)
      kbo(:, 3,53,12) = (/ &
        & 0.17987e+01_rb,0.74567e+01_rb,0.96292e+01_rb,0.11104e+02_rb,0.77562e+01_rb /)
      kbo(:, 4,53,12) = (/ &
        & 0.22559e+01_rb,0.89467e+01_rb,0.11320e+02_rb,0.12921e+02_rb,0.91504e+01_rb /)
      kbo(:, 5,53,12) = (/ &
        & 0.27477e+01_rb,0.10518e+02_rb,0.13086e+02_rb,0.14756e+02_rb,0.10571e+02_rb /)
      kbo(:, 1,54,12) = (/ &
        & 0.93266e+00_rb,0.45754e+01_rb,0.60420e+01_rb,0.71108e+01_rb,0.47321e+01_rb /)
      kbo(:, 2,54,12) = (/ &
        & 0.12608e+01_rb,0.57055e+01_rb,0.75003e+01_rb,0.87512e+01_rb,0.59717e+01_rb /)
      kbo(:, 3,54,12) = (/ &
        & 0.16525e+01_rb,0.69875e+01_rb,0.90782e+01_rb,0.10501e+02_rb,0.72982e+01_rb /)
      kbo(:, 4,54,12) = (/ &
        & 0.20962e+01_rb,0.84351e+01_rb,0.10744e+02_rb,0.12308e+02_rb,0.86747e+01_rb /)
      kbo(:, 5,54,12) = (/ &
        & 0.25775e+01_rb,0.99827e+01_rb,0.12485e+02_rb,0.14138e+02_rb,0.10092e+02_rb /)
      kbo(:, 1,55,12) = (/ &
        & 0.84048e+00_rb,0.42428e+01_rb,0.56145e+01_rb,0.66054e+01_rb,0.43544e+01_rb /)
      kbo(:, 2,55,12) = (/ &
        & 0.11472e+01_rb,0.53270e+01_rb,0.70144e+01_rb,0.82122e+01_rb,0.55615e+01_rb /)
      kbo(:, 3,55,12) = (/ &
        & 0.15181e+01_rb,0.65556e+01_rb,0.85601e+01_rb,0.99280e+01_rb,0.68631e+01_rb /)
      kbo(:, 4,55,12) = (/ &
        & 0.19474e+01_rb,0.79524e+01_rb,0.10198e+02_rb,0.11721e+02_rb,0.82252e+01_rb /)
      kbo(:, 5,55,12) = (/ &
        & 0.24188e+01_rb,0.94776e+01_rb,0.11916e+02_rb,0.13548e+02_rb,0.96351e+01_rb /)
      kbo(:, 1,56,12) = (/ &
        & 0.75447e+00_rb,0.39270e+01_rb,0.52064e+01_rb,0.61093e+01_rb,0.39870e+01_rb /)
      kbo(:, 2,56,12) = (/ &
        & 0.10395e+01_rb,0.49623e+01_rb,0.65429e+01_rb,0.76840e+01_rb,0.51623e+01_rb /)
      kbo(:, 3,56,12) = (/ &
        & 0.13908e+01_rb,0.61440e+01_rb,0.80522e+01_rb,0.93640e+01_rb,0.64360e+01_rb /)
      kbo(:, 4,56,12) = (/ &
        & 0.18031e+01_rb,0.74859e+01_rb,0.96627e+01_rb,0.11140e+02_rb,0.77842e+01_rb /)
      kbo(:, 5,56,12) = (/ &
        & 0.22626e+01_rb,0.89798e+01_rb,0.11358e+02_rb,0.12960e+02_rb,0.91800e+01_rb /)
      kbo(:, 1,57,12) = (/ &
        & 0.67513e+00_rb,0.36336e+01_rb,0.48180e+01_rb,0.56247e+01_rb,0.36284e+01_rb /)
      kbo(:, 2,57,12) = (/ &
        & 0.93951e+00_rb,0.46135e+01_rb,0.60910e+01_rb,0.71671e+01_rb,0.47736e+01_rb /)
      kbo(:, 3,57,12) = (/ &
        & 0.12699e+01_rb,0.57481e+01_rb,0.75541e+01_rb,0.88107e+01_rb,0.60168e+01_rb /)
      kbo(:, 4,57,12) = (/ &
        & 0.16637e+01_rb,0.70362e+01_rb,0.91351e+01_rb,0.10564e+02_rb,0.73465e+01_rb /)
      kbo(:, 5,57,12) = (/ &
        & 0.21102e+01_rb,0.84904e+01_rb,0.10807e+02_rb,0.12374e+02_rb,0.87256e+01_rb /)
      kbo(:, 1,58,12) = (/ &
        & 0.60371e+00_rb,0.33725e+01_rb,0.44640e+01_rb,0.51801e+01_rb,0.32968e+01_rb /)
      kbo(:, 2,58,12) = (/ &
        & 0.85131e+00_rb,0.42951e+01_rb,0.56822e+01_rb,0.66873e+01_rb,0.44155e+01_rb /)
      kbo(:, 3,58,12) = (/ &
        & 0.11611e+01_rb,0.53867e+01_rb,0.70919e+01_rb,0.82985e+01_rb,0.56282e+01_rb /)
      kbo(:, 4,58,12) = (/ &
        & 0.15366e+01_rb,0.66258e+01_rb,0.86445e+01_rb,0.10022e+02_rb,0.69358e+01_rb /)
      kbo(:, 5,58,12) = (/ &
        & 0.19698e+01_rb,0.80329e+01_rb,0.10290e+02_rb,0.11819e+02_rb,0.83004e+01_rb /)
      kbo(:, 1,59,12) = (/ &
        & 0.57552e+00_rb,0.32717e+01_rb,0.43250e+01_rb,0.50045e+01_rb,0.31668e+01_rb /)
      kbo(:, 2,59,12) = (/ &
        & 0.81716e+00_rb,0.41713e+01_rb,0.55225e+01_rb,0.64964e+01_rb,0.42740e+01_rb /)
      kbo(:, 3,59,12) = (/ &
        & 0.11190e+01_rb,0.52446e+01_rb,0.69091e+01_rb,0.80966e+01_rb,0.54744e+01_rb /)
      kbo(:, 4,59,12) = (/ &
        & 0.14869e+01_rb,0.64659e+01_rb,0.84502e+01_rb,0.98065e+01_rb,0.67716e+01_rb /)
      kbo(:, 5,59,12) = (/ &
        & 0.19139e+01_rb,0.78535e+01_rb,0.10085e+02_rb,0.11598e+02_rb,0.81311e+01_rb /)
      kbo(:, 1,13,13) = (/ &
        & 0.79406e+02_rb,0.76977e+02_rb,0.79398e+02_rb,0.72265e+02_rb,0.70608e+02_rb /)
      kbo(:, 2,13,13) = (/ &
        & 0.78716e+02_rb,0.76325e+02_rb,0.79145e+02_rb,0.72561e+02_rb,0.71233e+02_rb /)
      kbo(:, 3,13,13) = (/ &
        & 0.77960e+02_rb,0.75684e+02_rb,0.78864e+02_rb,0.72811e+02_rb,0.71703e+02_rb /)
      kbo(:, 4,13,13) = (/ &
        & 0.77172e+02_rb,0.75038e+02_rb,0.78462e+02_rb,0.73003e+02_rb,0.72067e+02_rb /)
      kbo(:, 5,13,13) = (/ &
        & 0.76342e+02_rb,0.74357e+02_rb,0.78015e+02_rb,0.73077e+02_rb,0.72406e+02_rb /)
      kbo(:, 1,14,13) = (/ &
        & 0.74002e+02_rb,0.75964e+02_rb,0.80244e+02_rb,0.73931e+02_rb,0.67973e+02_rb /)
      kbo(:, 2,14,13) = (/ &
        & 0.73259e+02_rb,0.75350e+02_rb,0.80187e+02_rb,0.74373e+02_rb,0.68671e+02_rb /)
      kbo(:, 3,14,13) = (/ &
        & 0.72511e+02_rb,0.74741e+02_rb,0.79998e+02_rb,0.74718e+02_rb,0.69297e+02_rb /)
      kbo(:, 4,14,13) = (/ &
        & 0.71760e+02_rb,0.74129e+02_rb,0.79694e+02_rb,0.74933e+02_rb,0.69925e+02_rb /)
      kbo(:, 5,14,13) = (/ &
        & 0.70963e+02_rb,0.73482e+02_rb,0.79282e+02_rb,0.75062e+02_rb,0.70592e+02_rb /)
      kbo(:, 1,15,13) = (/ &
        & 0.67727e+02_rb,0.74204e+02_rb,0.79973e+02_rb,0.74524e+02_rb,0.64503e+02_rb /)
      kbo(:, 2,15,13) = (/ &
        & 0.66995e+02_rb,0.73663e+02_rb,0.80067e+02_rb,0.75070e+02_rb,0.65404e+02_rb /)
      kbo(:, 3,15,13) = (/ &
        & 0.66299e+02_rb,0.73080e+02_rb,0.80001e+02_rb,0.75489e+02_rb,0.66341e+02_rb /)
      kbo(:, 4,15,13) = (/ &
        & 0.65584e+02_rb,0.72498e+02_rb,0.79808e+02_rb,0.75814e+02_rb,0.67293e+02_rb /)
      kbo(:, 5,15,13) = (/ &
        & 0.64881e+02_rb,0.71894e+02_rb,0.79644e+02_rb,0.76073e+02_rb,0.68159e+02_rb /)
      kbo(:, 1,16,13) = (/ &
        & 0.60910e+02_rb,0.71706e+02_rb,0.78658e+02_rb,0.73971e+02_rb,0.60740e+02_rb /)
      kbo(:, 2,16,13) = (/ &
        & 0.60264e+02_rb,0.71204e+02_rb,0.78897e+02_rb,0.74647e+02_rb,0.61882e+02_rb /)
      kbo(:, 3,16,13) = (/ &
        & 0.59646e+02_rb,0.70743e+02_rb,0.78998e+02_rb,0.75253e+02_rb,0.63062e+02_rb /)
      kbo(:, 4,16,13) = (/ &
        & 0.59033e+02_rb,0.70274e+02_rb,0.79093e+02_rb,0.75819e+02_rb,0.64263e+02_rb /)
      kbo(:, 5,16,13) = (/ &
        & 0.58482e+02_rb,0.69838e+02_rb,0.79175e+02_rb,0.76359e+02_rb,0.65453e+02_rb /)
      kbo(:, 1,17,13) = (/ &
        & 0.53962e+02_rb,0.68666e+02_rb,0.76384e+02_rb,0.72467e+02_rb,0.57058e+02_rb /)
      kbo(:, 2,17,13) = (/ &
        & 0.53430e+02_rb,0.68393e+02_rb,0.76843e+02_rb,0.73413e+02_rb,0.58438e+02_rb /)
      kbo(:, 3,17,13) = (/ &
        & 0.52920e+02_rb,0.68105e+02_rb,0.77273e+02_rb,0.74323e+02_rb,0.59869e+02_rb /)
      kbo(:, 4,17,13) = (/ &
        & 0.52483e+02_rb,0.67847e+02_rb,0.77698e+02_rb,0.75214e+02_rb,0.61307e+02_rb /)
      kbo(:, 5,17,13) = (/ &
        & 0.52082e+02_rb,0.67535e+02_rb,0.78034e+02_rb,0.76077e+02_rb,0.62785e+02_rb /)
      kbo(:, 1,18,13) = (/ &
        & 0.47235e+02_rb,0.65232e+02_rb,0.73440e+02_rb,0.70305e+02_rb,0.53676e+02_rb /)
      kbo(:, 2,18,13) = (/ &
        & 0.46798e+02_rb,0.65205e+02_rb,0.74239e+02_rb,0.71589e+02_rb,0.55262e+02_rb /)
      kbo(:, 3,18,13) = (/ &
        & 0.46439e+02_rb,0.65179e+02_rb,0.75037e+02_rb,0.72870e+02_rb,0.56922e+02_rb /)
      kbo(:, 4,18,13) = (/ &
        & 0.46165e+02_rb,0.65077e+02_rb,0.75787e+02_rb,0.74113e+02_rb,0.58606e+02_rb /)
      kbo(:, 5,18,13) = (/ &
        & 0.46076e+02_rb,0.64950e+02_rb,0.76497e+02_rb,0.75319e+02_rb,0.60364e+02_rb /)
      kbo(:, 1,19,13) = (/ &
        & 0.40875e+02_rb,0.61443e+02_rb,0.70167e+02_rb,0.67815e+02_rb,0.50666e+02_rb /)
      kbo(:, 2,19,13) = (/ &
        & 0.40599e+02_rb,0.61754e+02_rb,0.71348e+02_rb,0.69451e+02_rb,0.52443e+02_rb /)
      kbo(:, 3,19,13) = (/ &
        & 0.40423e+02_rb,0.61971e+02_rb,0.72494e+02_rb,0.71103e+02_rb,0.54265e+02_rb /)
      kbo(:, 4,19,13) = (/ &
        & 0.40489e+02_rb,0.62135e+02_rb,0.73624e+02_rb,0.72723e+02_rb,0.56176e+02_rb /)
      kbo(:, 5,19,13) = (/ &
        & 0.40909e+02_rb,0.62251e+02_rb,0.74691e+02_rb,0.74287e+02_rb,0.58151e+02_rb /)
      kbo(:, 1,20,13) = (/ &
        & 0.35203e+02_rb,0.57554e+02_rb,0.66740e+02_rb,0.65266e+02_rb,0.47986e+02_rb /)
      kbo(:, 2,20,13) = (/ &
        & 0.35059e+02_rb,0.58164e+02_rb,0.68281e+02_rb,0.67270e+02_rb,0.49928e+02_rb /)
      kbo(:, 3,20,13) = (/ &
        & 0.35179e+02_rb,0.58679e+02_rb,0.69865e+02_rb,0.69298e+02_rb,0.51957e+02_rb /)
      kbo(:, 4,20,13) = (/ &
        & 0.35687e+02_rb,0.59136e+02_rb,0.71375e+02_rb,0.71295e+02_rb,0.54068e+02_rb /)
      kbo(:, 5,20,13) = (/ &
        & 0.36572e+02_rb,0.59563e+02_rb,0.72814e+02_rb,0.73197e+02_rb,0.56184e+02_rb /)
      kbo(:, 1,21,13) = (/ &
        & 0.30294e+02_rb,0.53730e+02_rb,0.63335e+02_rb,0.62843e+02_rb,0.45639e+02_rb /)
      kbo(:, 2,21,13) = (/ &
        & 0.30407e+02_rb,0.54622e+02_rb,0.65309e+02_rb,0.65230e+02_rb,0.47736e+02_rb /)
      kbo(:, 3,21,13) = (/ &
        & 0.30898e+02_rb,0.55470e+02_rb,0.67283e+02_rb,0.67630e+02_rb,0.49955e+02_rb /)
      kbo(:, 4,21,13) = (/ &
        & 0.31814e+02_rb,0.56234e+02_rb,0.69205e+02_rb,0.69979e+02_rb,0.52223e+02_rb /)
      kbo(:, 5,21,13) = (/ &
        & 0.33171e+02_rb,0.57002e+02_rb,0.70988e+02_rb,0.72170e+02_rb,0.54507e+02_rb /)
      kbo(:, 1,22,13) = (/ &
        & 0.26203e+02_rb,0.50178e+02_rb,0.60369e+02_rb,0.60883e+02_rb,0.43752e+02_rb /)
      kbo(:, 2,22,13) = (/ &
        & 0.26668e+02_rb,0.51399e+02_rb,0.62731e+02_rb,0.63628e+02_rb,0.46055e+02_rb /)
      kbo(:, 3,22,13) = (/ &
        & 0.27573e+02_rb,0.52567e+02_rb,0.65094e+02_rb,0.66337e+02_rb,0.48427e+02_rb /)
      kbo(:, 4,22,13) = (/ &
        & 0.28933e+02_rb,0.53710e+02_rb,0.67379e+02_rb,0.68992e+02_rb,0.50840e+02_rb /)
      kbo(:, 5,22,13) = (/ &
        & 0.30713e+02_rb,0.54842e+02_rb,0.69500e+02_rb,0.71453e+02_rb,0.53313e+02_rb /)
      kbo(:, 1,23,13) = (/ &
        & 0.22949e+02_rb,0.47062e+02_rb,0.57786e+02_rb,0.59287e+02_rb,0.42207e+02_rb /)
      kbo(:, 2,23,13) = (/ &
        & 0.23779e+02_rb,0.48563e+02_rb,0.60567e+02_rb,0.62337e+02_rb,0.44709e+02_rb /)
      kbo(:, 3,23,13) = (/ &
        & 0.25087e+02_rb,0.50088e+02_rb,0.63282e+02_rb,0.65361e+02_rb,0.47224e+02_rb /)
      kbo(:, 4,23,13) = (/ &
        & 0.26833e+02_rb,0.51601e+02_rb,0.65861e+02_rb,0.68253e+02_rb,0.49790e+02_rb /)
      kbo(:, 5,23,13) = (/ &
        & 0.28975e+02_rb,0.53148e+02_rb,0.68312e+02_rb,0.70979e+02_rb,0.52442e+02_rb /)
      kbo(:, 1,24,13) = (/ &
        & 0.20450e+02_rb,0.44418e+02_rb,0.55719e+02_rb,0.58070e+02_rb,0.41028e+02_rb /)
      kbo(:, 2,24,13) = (/ &
        & 0.21647e+02_rb,0.46218e+02_rb,0.58860e+02_rb,0.61413e+02_rb,0.43697e+02_rb /)
      kbo(:, 3,24,13) = (/ &
        & 0.23313e+02_rb,0.48082e+02_rb,0.61856e+02_rb,0.64696e+02_rb,0.46357e+02_rb /)
      kbo(:, 4,24,13) = (/ &
        & 0.25383e+02_rb,0.49987e+02_rb,0.64735e+02_rb,0.67792e+02_rb,0.49079e+02_rb /)
      kbo(:, 5,24,13) = (/ &
        & 0.27788e+02_rb,0.51882e+02_rb,0.67449e+02_rb,0.70725e+02_rb,0.51885e+02_rb /)
      kbo(:, 1,25,13) = (/ &
        & 0.18632e+02_rb,0.42267e+02_rb,0.54194e+02_rb,0.57249e+02_rb,0.40189e+02_rb /)
      kbo(:, 2,25,13) = (/ &
        & 0.20151e+02_rb,0.44393e+02_rb,0.57583e+02_rb,0.60858e+02_rb,0.43004e+02_rb /)
      kbo(:, 3,25,13) = (/ &
        & 0.22101e+02_rb,0.46593e+02_rb,0.60865e+02_rb,0.64337e+02_rb,0.45809e+02_rb /)
      kbo(:, 4,25,13) = (/ &
        & 0.24391e+02_rb,0.48836e+02_rb,0.63984e+02_rb,0.67613e+02_rb,0.48696e+02_rb /)
      kbo(:, 5,25,13) = (/ &
        & 0.26959e+02_rb,0.50995e+02_rb,0.66882e+02_rb,0.70682e+02_rb,0.51602e+02_rb /)
      kbo(:, 1,26,13) = (/ &
        & 0.17370e+02_rb,0.40657e+02_rb,0.53173e+02_rb,0.56858e+02_rb,0.39692e+02_rb /)
      kbo(:, 2,26,13) = (/ &
        & 0.19155e+02_rb,0.43087e+02_rb,0.56801e+02_rb,0.60671e+02_rb,0.42632e+02_rb /)
      kbo(:, 3,26,13) = (/ &
        & 0.21292e+02_rb,0.45598e+02_rb,0.60302e+02_rb,0.64290e+02_rb,0.45592e+02_rb /)
      kbo(:, 4,26,13) = (/ &
        & 0.23727e+02_rb,0.48098e+02_rb,0.63591e+02_rb,0.67690e+02_rb,0.48603e+02_rb /)
      kbo(:, 5,26,13) = (/ &
        & 0.26455e+02_rb,0.50493e+02_rb,0.66616e+02_rb,0.70861e+02_rb,0.51588e+02_rb /)
      kbo(:, 1,27,13) = (/ &
        & 0.16499e+02_rb,0.39533e+02_rb,0.52563e+02_rb,0.56787e+02_rb,0.39461e+02_rb /)
      kbo(:, 2,27,13) = (/ &
        & 0.18462e+02_rb,0.42240e+02_rb,0.56391e+02_rb,0.60738e+02_rb,0.42525e+02_rb /)
      kbo(:, 3,27,13) = (/ &
        & 0.20737e+02_rb,0.45009e+02_rb,0.60060e+02_rb,0.64459e+02_rb,0.45612e+02_rb /)
      kbo(:, 4,27,13) = (/ &
        & 0.23336e+02_rb,0.47712e+02_rb,0.63452e+02_rb,0.67944e+02_rb,0.48705e+02_rb /)
      kbo(:, 5,27,13) = (/ &
        & 0.26215e+02_rb,0.50322e+02_rb,0.66583e+02_rb,0.71174e+02_rb,0.51756e+02_rb /)
      kbo(:, 1,28,13) = (/ &
        & 0.15882e+02_rb,0.38845e+02_rb,0.52318e+02_rb,0.56965e+02_rb,0.39468e+02_rb /)
      kbo(:, 2,28,13) = (/ &
        & 0.17994e+02_rb,0.41785e+02_rb,0.56289e+02_rb,0.61007e+02_rb,0.42649e+02_rb /)
      kbo(:, 3,28,13) = (/ &
        & 0.20424e+02_rb,0.44749e+02_rb,0.60052e+02_rb,0.64808e+02_rb,0.45826e+02_rb /)
      kbo(:, 4,28,13) = (/ &
        & 0.23178e+02_rb,0.47630e+02_rb,0.63526e+02_rb,0.68347e+02_rb,0.48967e+02_rb /)
      kbo(:, 5,28,13) = (/ &
        & 0.26217e+02_rb,0.50385e+02_rb,0.66714e+02_rb,0.71584e+02_rb,0.52071e+02_rb /)
      kbo(:, 1,29,13) = (/ &
        & 0.15496e+02_rb,0.38533e+02_rb,0.52388e+02_rb,0.57377e+02_rb,0.39687e+02_rb /)
      kbo(:, 2,29,13) = (/ &
        & 0.17758e+02_rb,0.41667e+02_rb,0.56458e+02_rb,0.61470e+02_rb,0.42963e+02_rb /)
      kbo(:, 3,29,13) = (/ &
        & 0.20345e+02_rb,0.44782e+02_rb,0.60267e+02_rb,0.65320e+02_rb,0.46203e+02_rb /)
      kbo(:, 4,29,13) = (/ &
        & 0.23257e+02_rb,0.47788e+02_rb,0.63789e+02_rb,0.68865e+02_rb,0.49378e+02_rb /)
      kbo(:, 5,29,13) = (/ &
        & 0.26442e+02_rb,0.50623e+02_rb,0.67004e+02_rb,0.72085e+02_rb,0.52513e+02_rb /)
      kbo(:, 1,30,13) = (/ &
        & 0.15305e+02_rb,0.38520e+02_rb,0.52687e+02_rb,0.57942e+02_rb,0.40075e+02_rb /)
      kbo(:, 2,30,13) = (/ &
        & 0.17725e+02_rb,0.41787e+02_rb,0.56809e+02_rb,0.62069e+02_rb,0.43408e+02_rb /)
      kbo(:, 3,30,13) = (/ &
        & 0.20471e+02_rb,0.45011e+02_rb,0.60646e+02_rb,0.65935e+02_rb,0.46678e+02_rb /)
      kbo(:, 4,30,13) = (/ &
        & 0.23532e+02_rb,0.48109e+02_rb,0.64191e+02_rb,0.69457e+02_rb,0.49879e+02_rb /)
      kbo(:, 5,30,13) = (/ &
        & 0.26835e+02_rb,0.50980e+02_rb,0.67409e+02_rb,0.72648e+02_rb,0.53041e+02_rb /)
      kbo(:, 1,31,13) = (/ &
        & 0.15319e+02_rb,0.38759e+02_rb,0.53187e+02_rb,0.58637e+02_rb,0.40597e+02_rb /)
      kbo(:, 2,31,13) = (/ &
        & 0.17890e+02_rb,0.42121e+02_rb,0.57326e+02_rb,0.62778e+02_rb,0.43969e+02_rb /)
      kbo(:, 3,31,13) = (/ &
        & 0.20784e+02_rb,0.45419e+02_rb,0.61174e+02_rb,0.66627e+02_rb,0.47246e+02_rb /)
      kbo(:, 4,31,13) = (/ &
        & 0.23963e+02_rb,0.48555e+02_rb,0.64709e+02_rb,0.70119e+02_rb,0.50470e+02_rb /)
      kbo(:, 5,31,13) = (/ &
        & 0.27377e+02_rb,0.51448e+02_rb,0.67912e+02_rb,0.73265e+02_rb,0.53642e+02_rb /)
      kbo(:, 1,32,13) = (/ &
        & 0.15518e+02_rb,0.39176e+02_rb,0.53833e+02_rb,0.59441e+02_rb,0.41231e+02_rb /)
      kbo(:, 2,32,13) = (/ &
        & 0.18219e+02_rb,0.42608e+02_rb,0.57967e+02_rb,0.63575e+02_rb,0.44614e+02_rb /)
      kbo(:, 3,32,13) = (/ &
        & 0.21240e+02_rb,0.45958e+02_rb,0.61807e+02_rb,0.67376e+02_rb,0.47899e+02_rb /)
      kbo(:, 4,32,13) = (/ &
        & 0.24535e+02_rb,0.49103e+02_rb,0.65323e+02_rb,0.70819e+02_rb,0.51129e+02_rb /)
      kbo(:, 5,32,13) = (/ &
        & 0.28030e+02_rb,0.51991e+02_rb,0.68477e+02_rb,0.73912e+02_rb,0.54297e+02_rb /)
      kbo(:, 1,33,13) = (/ &
        & 0.15857e+02_rb,0.39728e+02_rb,0.54587e+02_rb,0.60326e+02_rb,0.41934e+02_rb /)
      kbo(:, 2,33,13) = (/ &
        & 0.18681e+02_rb,0.43207e+02_rb,0.58700e+02_rb,0.64428e+02_rb,0.45312e+02_rb /)
      kbo(:, 3,33,13) = (/ &
        & 0.21817e+02_rb,0.46584e+02_rb,0.62510e+02_rb,0.68164e+02_rb,0.48601e+02_rb /)
      kbo(:, 4,33,13) = (/ &
        & 0.25208e+02_rb,0.49716e+02_rb,0.65986e+02_rb,0.71546e+02_rb,0.51832e+02_rb /)
      kbo(:, 5,33,13) = (/ &
        & 0.28769e+02_rb,0.52588e+02_rb,0.69085e+02_rb,0.74576e+02_rb,0.54996e+02_rb /)
      kbo(:, 1,34,13) = (/ &
        & 0.16218e+02_rb,0.40255e+02_rb,0.55264e+02_rb,0.61104e+02_rb,0.42563e+02_rb /)
      kbo(:, 2,34,13) = (/ &
        & 0.19136e+02_rb,0.43775e+02_rb,0.59363e+02_rb,0.65173e+02_rb,0.45933e+02_rb /)
      kbo(:, 3,34,13) = (/ &
        & 0.22367e+02_rb,0.47148e+02_rb,0.63138e+02_rb,0.68848e+02_rb,0.49223e+02_rb /)
      kbo(:, 4,34,13) = (/ &
        & 0.25824e+02_rb,0.50264e+02_rb,0.66569e+02_rb,0.72180e+02_rb,0.52454e+02_rb /)
      kbo(:, 5,34,13) = (/ &
        & 0.29432e+02_rb,0.53119e+02_rb,0.69617e+02_rb,0.75149e+02_rb,0.55607e+02_rb /)
      kbo(:, 1,35,13) = (/ &
        & 0.16382e+02_rb,0.40509e+02_rb,0.55591e+02_rb,0.61512e+02_rb,0.42891e+02_rb /)
      kbo(:, 2,35,13) = (/ &
        & 0.19362e+02_rb,0.44056e+02_rb,0.59682e+02_rb,0.65556e+02_rb,0.46259e+02_rb /)
      kbo(:, 3,35,13) = (/ &
        & 0.22646e+02_rb,0.47437e+02_rb,0.63450e+02_rb,0.69212e+02_rb,0.49551e+02_rb /)
      kbo(:, 4,35,13) = (/ &
        & 0.26140e+02_rb,0.50553e+02_rb,0.66864e+02_rb,0.72519e+02_rb,0.52785e+02_rb /)
      kbo(:, 5,35,13) = (/ &
        & 0.29778e+02_rb,0.53403e+02_rb,0.69889e+02_rb,0.75460e+02_rb,0.55937e+02_rb /)
      kbo(:, 1,36,13) = (/ &
        & 0.16282e+02_rb,0.40412e+02_rb,0.55499e+02_rb,0.61489e+02_rb,0.42866e+02_rb /)
      kbo(:, 2,36,13) = (/ &
        & 0.19286e+02_rb,0.43996e+02_rb,0.59618e+02_rb,0.65550e+02_rb,0.46244e+02_rb /)
      kbo(:, 3,36,13) = (/ &
        & 0.22586e+02_rb,0.47399e+02_rb,0.63406e+02_rb,0.69212e+02_rb,0.49542e+02_rb /)
      kbo(:, 4,36,13) = (/ &
        & 0.26096e+02_rb,0.50533e+02_rb,0.66829e+02_rb,0.72526e+02_rb,0.52781e+02_rb /)
      kbo(:, 5,36,13) = (/ &
        & 0.29750e+02_rb,0.53407e+02_rb,0.69875e+02_rb,0.75478e+02_rb,0.55943e+02_rb /)
      kbo(:, 1,37,13) = (/ &
        & 0.15776e+02_rb,0.39797e+02_rb,0.54788e+02_rb,0.60844e+02_rb,0.42332e+02_rb /)
      kbo(:, 2,37,13) = (/ &
        & 0.18757e+02_rb,0.43418e+02_rb,0.58970e+02_rb,0.64965e+02_rb,0.45736e+02_rb /)
      kbo(:, 3,37,13) = (/ &
        & 0.22032e+02_rb,0.46879e+02_rb,0.62830e+02_rb,0.68697e+02_rb,0.49053e+02_rb /)
      kbo(:, 4,37,13) = (/ &
        & 0.25533e+02_rb,0.50076e+02_rb,0.66324e+02_rb,0.72072e+02_rb,0.52311e+02_rb /)
      kbo(:, 5,37,13) = (/ &
        & 0.29180e+02_rb,0.52997e+02_rb,0.69437e+02_rb,0.75080e+02_rb,0.55481e+02_rb /)
      kbo(:, 1,38,13) = (/ &
        & 0.15271e+02_rb,0.39166e+02_rb,0.54054e+02_rb,0.60165e+02_rb,0.41777e+02_rb /)
      kbo(:, 2,38,13) = (/ &
        & 0.18227e+02_rb,0.42823e+02_rb,0.58302e+02_rb,0.64350e+02_rb,0.45207e+02_rb /)
      kbo(:, 3,38,13) = (/ &
        & 0.21473e+02_rb,0.46333e+02_rb,0.62224e+02_rb,0.68144e+02_rb,0.48540e+02_rb /)
      kbo(:, 4,38,13) = (/ &
        & 0.24954e+02_rb,0.49588e+02_rb,0.65790e+02_rb,0.71578e+02_rb,0.51811e+02_rb /)
      kbo(:, 5,38,13) = (/ &
        & 0.28592e+02_rb,0.52564e+02_rb,0.68973e+02_rb,0.74653e+02_rb,0.55000e+02_rb /)
      kbo(:, 1,39,13) = (/ &
        & 0.14788e+02_rb,0.38557e+02_rb,0.53339e+02_rb,0.59493e+02_rb,0.41232e+02_rb /)
      kbo(:, 2,39,13) = (/ &
        & 0.17725e+02_rb,0.42239e+02_rb,0.57643e+02_rb,0.63735e+02_rb,0.44689e+02_rb /)
      kbo(:, 3,39,13) = (/ &
        & 0.20936e+02_rb,0.45800e+02_rb,0.61629e+02_rb,0.67599e+02_rb,0.48036e+02_rb /)
      kbo(:, 4,39,13) = (/ &
        & 0.24393e+02_rb,0.49102e+02_rb,0.65258e+02_rb,0.71089e+02_rb,0.51322e+02_rb /)
      kbo(:, 5,39,13) = (/ &
        & 0.28020e+02_rb,0.52130e+02_rb,0.68507e+02_rb,0.74225e+02_rb,0.54528e+02_rb /)
      kbo(:, 1,40,13) = (/ &
        & 0.14035e+02_rb,0.37576e+02_rb,0.52178e+02_rb,0.58374e+02_rb,0.40328e+02_rb /)
      kbo(:, 2,40,13) = (/ &
        & 0.16933e+02_rb,0.41290e+02_rb,0.56557e+02_rb,0.62694e+02_rb,0.43829e+02_rb /)
      kbo(:, 3,40,13) = (/ &
        & 0.20073e+02_rb,0.44909e+02_rb,0.60638e+02_rb,0.66668e+02_rb,0.47203e+02_rb /)
      kbo(:, 4,40,13) = (/ &
        & 0.23486e+02_rb,0.48287e+02_rb,0.64365e+02_rb,0.70250e+02_rb,0.50504e+02_rb /)
      kbo(:, 5,40,13) = (/ &
        & 0.27084e+02_rb,0.51393e+02_rb,0.67719e+02_rb,0.73485e+02_rb,0.53736e+02_rb /)
      kbo(:, 1,41,13) = (/ &
        & 0.13270e+02_rb,0.36551e+02_rb,0.50952e+02_rb,0.57187e+02_rb,0.39381e+02_rb /)
      kbo(:, 2,41,13) = (/ &
        & 0.16132e+02_rb,0.40293e+02_rb,0.55396e+02_rb,0.61573e+02_rb,0.42921e+02_rb /)
      kbo(:, 3,41,13) = (/ &
        & 0.19195e+02_rb,0.43957e+02_rb,0.59575e+02_rb,0.65658e+02_rb,0.46320e+02_rb /)
      kbo(:, 4,41,13) = (/ &
        & 0.22552e+02_rb,0.47415e+02_rb,0.63413e+02_rb,0.69340e+02_rb,0.49639e+02_rb /)
      kbo(:, 5,41,13) = (/ &
        & 0.26115e+02_rb,0.50603e+02_rb,0.66868e+02_rb,0.72675e+02_rb,0.52897e+02_rb /)
      kbo(:, 1,42,13) = (/ &
        & 0.12533e+02_rb,0.35537e+02_rb,0.49715e+02_rb,0.55979e+02_rb,0.38430e+02_rb /)
      kbo(:, 2,42,13) = (/ &
        & 0.15340e+02_rb,0.39292e+02_rb,0.54229e+02_rb,0.60445e+02_rb,0.42004e+02_rb /)
      kbo(:, 3,42,13) = (/ &
        & 0.18342e+02_rb,0.42991e+02_rb,0.58494e+02_rb,0.64631e+02_rb,0.45437e+02_rb /)
      kbo(:, 4,42,13) = (/ &
        & 0.21634e+02_rb,0.46527e+02_rb,0.62435e+02_rb,0.68417e+02_rb,0.48777e+02_rb /)
      kbo(:, 5,42,13) = (/ &
        & 0.25154e+02_rb,0.49791e+02_rb,0.65989e+02_rb,0.71836e+02_rb,0.52054e+02_rb /)
      kbo(:, 1,43,13) = (/ &
        & 0.11668e+02_rb,0.34309e+02_rb,0.48203e+02_rb,0.54498e+02_rb,0.37270e+02_rb /)
      kbo(:, 2,43,13) = (/ &
        & 0.14402e+02_rb,0.38082e+02_rb,0.52797e+02_rb,0.59048e+02_rb,0.40873e+02_rb /)
      kbo(:, 3,43,13) = (/ &
        & 0.17344e+02_rb,0.41812e+02_rb,0.57160e+02_rb,0.63341e+02_rb,0.44355e+02_rb /)
      kbo(:, 4,43,13) = (/ &
        & 0.20545e+02_rb,0.45427e+02_rb,0.61209e+02_rb,0.67260e+02_rb,0.47724e+02_rb /)
      kbo(:, 5,43,13) = (/ &
        & 0.24007e+02_rb,0.48779e+02_rb,0.64893e+02_rb,0.70794e+02_rb,0.51019e+02_rb /)
      kbo(:, 1,44,13) = (/ &
        & 0.10777e+02_rb,0.32982e+02_rb,0.46555e+02_rb,0.52865e+02_rb,0.36017e+02_rb /)
      kbo(:, 2,44,13) = (/ &
        & 0.13422e+02_rb,0.36774e+02_rb,0.51234e+02_rb,0.57518e+02_rb,0.39647e+02_rb /)
      kbo(:, 3,44,13) = (/ &
        & 0.16301e+02_rb,0.40529e+02_rb,0.55688e+02_rb,0.61909e+02_rb,0.43184e+02_rb /)
      kbo(:, 4,44,13) = (/ &
        & 0.19410e+02_rb,0.44205e+02_rb,0.59841e+02_rb,0.65956e+02_rb,0.46575e+02_rb /)
      kbo(:, 5,44,13) = (/ &
        & 0.22792e+02_rb,0.47657e+02_rb,0.63668e+02_rb,0.69623e+02_rb,0.49898e+02_rb /)
      kbo(:, 1,45,13) = (/ &
        & 0.99376e+01_rb,0.31658e+02_rb,0.44891e+02_rb,0.51201e+02_rb,0.34758e+02_rb /)
      kbo(:, 2,45,13) = (/ &
        & 0.12468e+02_rb,0.35468e+02_rb,0.49648e+02_rb,0.55955e+02_rb,0.38412e+02_rb /)
      kbo(:, 3,45,13) = (/ &
        & 0.15282e+02_rb,0.39240e+02_rb,0.54182e+02_rb,0.60441e+02_rb,0.41991e+02_rb /)
      kbo(:, 4,45,13) = (/ &
        & 0.18307e+02_rb,0.42958e+02_rb,0.58447e+02_rb,0.64617e+02_rb,0.45427e+02_rb /)
      kbo(:, 5,45,13) = (/ &
        & 0.21602e+02_rb,0.46508e+02_rb,0.62404e+02_rb,0.68417e+02_rb,0.48775e+02_rb /)
      kbo(:, 1,46,13) = (/ &
        & 0.90997e+01_rb,0.30250e+02_rb,0.43114e+02_rb,0.49420e+02_rb,0.33417e+02_rb /)
      kbo(:, 2,46,13) = (/ &
        & 0.11497e+02_rb,0.34076e+02_rb,0.47932e+02_rb,0.54263e+02_rb,0.37087e+02_rb /)
      kbo(:, 3,46,13) = (/ &
        & 0.14229e+02_rb,0.37865e+02_rb,0.52547e+02_rb,0.58836e+02_rb,0.40705e+02_rb /)
      kbo(:, 4,46,13) = (/ &
        & 0.17178e+02_rb,0.41613e+02_rb,0.56930e+02_rb,0.63142e+02_rb,0.44193e+02_rb /)
      kbo(:, 5,46,13) = (/ &
        & 0.20366e+02_rb,0.45251e+02_rb,0.61008e+02_rb,0.67091e+02_rb,0.47569e+02_rb /)
      kbo(:, 1,47,13) = (/ &
        & 0.82326e+01_rb,0.28698e+02_rb,0.41131e+02_rb,0.47406e+02_rb,0.31895e+02_rb /)
      kbo(:, 2,47,13) = (/ &
        & 0.10467e+02_rb,0.32516e+02_rb,0.45987e+02_rb,0.52322e+02_rb,0.35605e+02_rb /)
      kbo(:, 3,47,13) = (/ &
        & 0.13084e+02_rb,0.36324e+02_rb,0.50700e+02_rb,0.57015e+02_rb,0.39250e+02_rb /)
      kbo(:, 4,47,13) = (/ &
        & 0.15956e+02_rb,0.40097e+02_rb,0.55181e+02_rb,0.61437e+02_rb,0.42802e+02_rb /)
      kbo(:, 5,47,13) = (/ &
        & 0.19031e+02_rb,0.43799e+02_rb,0.59396e+02_rb,0.65543e+02_rb,0.46212e+02_rb /)
      kbo(:, 1,48,13) = (/ &
        & 0.74393e+01_rb,0.27161e+02_rb,0.39157e+02_rb,0.45343e+02_rb,0.30350e+02_rb /)
      kbo(:, 2,48,13) = (/ &
        & 0.95049e+01_rb,0.30953e+02_rb,0.44020e+02_rb,0.50347e+02_rb,0.34113e+02_rb /)
      kbo(:, 3,48,13) = (/ &
        & 0.11978e+02_rb,0.34782e+02_rb,0.48813e+02_rb,0.55151e+02_rb,0.37782e+02_rb /)
      kbo(:, 4,48,13) = (/ &
        & 0.14764e+02_rb,0.38570e+02_rb,0.53387e+02_rb,0.59681e+02_rb,0.41384e+02_rb /)
      kbo(:, 5,48,13) = (/ &
        & 0.17748e+02_rb,0.42312e+02_rb,0.57728e+02_rb,0.63933e+02_rb,0.44844e+02_rb /)
      kbo(:, 1,49,13) = (/ &
        & 0.67167e+01_rb,0.25650e+02_rb,0.37197e+02_rb,0.43247e+02_rb,0.28796e+02_rb /)
      kbo(:, 2,49,13) = (/ &
        & 0.86101e+01_rb,0.29402e+02_rb,0.42045e+02_rb,0.48354e+02_rb,0.32610e+02_rb /)
      kbo(:, 3,49,13) = (/ &
        & 0.10926e+02_rb,0.33234e+02_rb,0.46891e+02_rb,0.53243e+02_rb,0.36305e+02_rb /)
      kbo(:, 4,49,13) = (/ &
        & 0.13605e+02_rb,0.37035e+02_rb,0.51562e+02_rb,0.57884e+02_rb,0.39936e+02_rb /)
      kbo(:, 5,49,13) = (/ &
        & 0.16513e+02_rb,0.40804e+02_rb,0.56010e+02_rb,0.62260e+02_rb,0.43466e+02_rb /)
      kbo(:, 1,50,13) = (/ &
        & 0.60927e+01_rb,0.24264e+02_rb,0.35366e+02_rb,0.41271e+02_rb,0.27331e+02_rb /)
      kbo(:, 2,50,13) = (/ &
        & 0.78376e+01_rb,0.27958e+02_rb,0.40194e+02_rb,0.46451e+02_rb,0.31176e+02_rb /)
      kbo(:, 3,50,13) = (/ &
        & 0.99975e+01_rb,0.31777e+02_rb,0.45061e+02_rb,0.51408e+02_rb,0.34914e+02_rb /)
      kbo(:, 4,50,13) = (/ &
        & 0.12552e+02_rb,0.35595e+02_rb,0.49816e+02_rb,0.56159e+02_rb,0.38569e+02_rb /)
      kbo(:, 5,50,13) = (/ &
        & 0.15384e+02_rb,0.39379e+02_rb,0.54352e+02_rb,0.60637e+02_rb,0.42153e+02_rb /)
      kbo(:, 1,51,13) = (/ &
        & 0.55303e+01_rb,0.22951e+02_rb,0.33599e+02_rb,0.39341e+02_rb,0.25901e+02_rb /)
      kbo(:, 2,51,13) = (/ &
        & 0.71468e+01_rb,0.26574e+02_rb,0.38412e+02_rb,0.44565e+02_rb,0.29771e+02_rb /)
      kbo(:, 3,51,13) = (/ &
        & 0.91530e+01_rb,0.30360e+02_rb,0.43274e+02_rb,0.49608e+02_rb,0.33559e+02_rb /)
      kbo(:, 4,51,13) = (/ &
        & 0.11570e+02_rb,0.34196e+02_rb,0.48091e+02_rb,0.54457e+02_rb,0.37233e+02_rb /)
      kbo(:, 5,51,13) = (/ &
        & 0.14322e+02_rb,0.37996e+02_rb,0.52708e+02_rb,0.59018e+02_rb,0.40851e+02_rb /)
      kbo(:, 1,52,13) = (/ &
        & 0.50134e+01_rb,0.21673e+02_rb,0.31833e+02_rb,0.37382e+02_rb,0.24463e+02_rb /)
      kbo(:, 2,52,13) = (/ &
        & 0.65135e+01_rb,0.25216e+02_rb,0.36639e+02_rb,0.42658e+02_rb,0.28359e+02_rb /)
      kbo(:, 3,52,13) = (/ &
        & 0.83642e+01_rb,0.28959e+02_rb,0.41485e+02_rb,0.47797e+02_rb,0.32187e+02_rb /)
      kbo(:, 4,52,13) = (/ &
        & 0.10636e+02_rb,0.32789e+02_rb,0.46344e+02_rb,0.52709e+02_rb,0.35895e+02_rb /)
      kbo(:, 5,52,13) = (/ &
        & 0.13285e+02_rb,0.36604e+02_rb,0.51047e+02_rb,0.57380e+02_rb,0.39540e+02_rb /)
      kbo(:, 1,53,13) = (/ &
        & 0.45376e+01_rb,0.20438e+02_rb,0.30067e+02_rb,0.35396e+02_rb,0.23001e+02_rb /)
      kbo(:, 2,53,13) = (/ &
        & 0.59241e+01_rb,0.23884e+02_rb,0.34872e+02_rb,0.40744e+02_rb,0.26941e+02_rb /)
      kbo(:, 3,53,13) = (/ &
        & 0.76354e+01_rb,0.27568e+02_rb,0.39698e+02_rb,0.45939e+02_rb,0.30794e+02_rb /)
      kbo(:, 4,53,13) = (/ &
        & 0.97536e+01_rb,0.31378e+02_rb,0.44569e+02_rb,0.50924e+02_rb,0.34545e+02_rb /)
      kbo(:, 5,53,13) = (/ &
        & 0.12275e+02_rb,0.35210e+02_rb,0.49348e+02_rb,0.55701e+02_rb,0.38212e+02_rb /)
      kbo(:, 1,54,13) = (/ &
        & 0.41389e+01_rb,0.19353e+02_rb,0.28454e+02_rb,0.33563e+02_rb,0.21647e+02_rb /)
      kbo(:, 2,54,13) = (/ &
        & 0.54194e+01_rb,0.22692e+02_rb,0.33259e+02_rb,0.38976e+02_rb,0.25632e+02_rb /)
      kbo(:, 3,54,13) = (/ &
        & 0.70164e+01_rb,0.26307e+02_rb,0.38072e+02_rb,0.44208e+02_rb,0.29508e+02_rb /)
      kbo(:, 4,54,13) = (/ &
        & 0.89936e+01_rb,0.30086e+02_rb,0.42934e+02_rb,0.49276e+02_rb,0.33306e+02_rb /)
      kbo(:, 5,54,13) = (/ &
        & 0.11387e+02_rb,0.33931e+02_rb,0.47765e+02_rb,0.54132e+02_rb,0.36987e+02_rb /)
      kbo(:, 1,55,13) = (/ &
        & 0.37927e+01_rb,0.18358e+02_rb,0.26927e+02_rb,0.31801e+02_rb,0.20350e+02_rb /)
      kbo(:, 2,55,13) = (/ &
        & 0.49755e+01_rb,0.21582e+02_rb,0.31711e+02_rb,0.37250e+02_rb,0.24368e+02_rb /)
      kbo(:, 3,55,13) = (/ &
        & 0.64669e+01_rb,0.25119e+02_rb,0.36517e+02_rb,0.42532e+02_rb,0.28268e+02_rb /)
      kbo(:, 4,55,13) = (/ &
        & 0.83082e+01_rb,0.28857e+02_rb,0.41362e+02_rb,0.47679e+02_rb,0.32095e+02_rb /)
      kbo(:, 5,55,13) = (/ &
        & 0.10573e+02_rb,0.32695e+02_rb,0.46225e+02_rb,0.52593e+02_rb,0.35810e+02_rb /)
      kbo(:, 1,56,13) = (/ &
        & 0.34680e+01_rb,0.17374e+02_rb,0.25413e+02_rb,0.30039e+02_rb,0.19069e+02_rb /)
      kbo(:, 2,56,13) = (/ &
        & 0.45601e+01_rb,0.20503e+02_rb,0.30163e+02_rb,0.35509e+02_rb,0.23088e+02_rb /)
      kbo(:, 3,56,13) = (/ &
        & 0.59518e+01_rb,0.23956e+02_rb,0.34971e+02_rb,0.40858e+02_rb,0.27027e+02_rb /)
      kbo(:, 4,56,13) = (/ &
        & 0.76711e+01_rb,0.27642e+02_rb,0.39801e+02_rb,0.46057e+02_rb,0.30878e+02_rb /)
      kbo(:, 5,56,13) = (/ &
        & 0.98017e+01_rb,0.31462e+02_rb,0.44674e+02_rb,0.51031e+02_rb,0.34630e+02_rb /)
      kbo(:, 1,57,13) = (/ &
        & 0.31571e+01_rb,0.16389e+02_rb,0.23903e+02_rb,0.28270e+02_rb,0.17802e+02_rb /)
      kbo(:, 2,57,13) = (/ &
        & 0.41775e+01_rb,0.19466e+02_rb,0.28628e+02_rb,0.33765e+02_rb,0.21796e+02_rb /)
      kbo(:, 3,57,13) = (/ &
        & 0.54676e+01_rb,0.22817e+02_rb,0.33433e+02_rb,0.39170e+02_rb,0.25777e+02_rb /)
      kbo(:, 4,57,13) = (/ &
        & 0.70767e+01_rb,0.26436e+02_rb,0.38248e+02_rb,0.44404e+02_rb,0.29650e+02_rb /)
      kbo(:, 5,57,13) = (/ &
        & 0.90728e+01_rb,0.30224e+02_rb,0.43110e+02_rb,0.49455e+02_rb,0.33445e+02_rb /)
      kbo(:, 1,58,13) = (/ &
        & 0.28749e+01_rb,0.15464e+02_rb,0.22485e+02_rb,0.26596e+02_rb,0.16612e+02_rb /)
      kbo(:, 2,58,13) = (/ &
        & 0.38438e+01_rb,0.18517e+02_rb,0.27179e+02_rb,0.32099e+02_rb,0.20568e+02_rb /)
      kbo(:, 3,58,13) = (/ &
        & 0.50410e+01_rb,0.21758e+02_rb,0.31967e+02_rb,0.37546e+02_rb,0.24584e+02_rb /)
      kbo(:, 4,58,13) = (/ &
        & 0.65535e+01_rb,0.25311e+02_rb,0.36779e+02_rb,0.42823e+02_rb,0.28478e+02_rb /)
      kbo(:, 5,58,13) = (/ &
        & 0.84187e+01_rb,0.29065e+02_rb,0.41628e+02_rb,0.47951e+02_rb,0.32304e+02_rb /)
      kbo(:, 1,59,13) = (/ &
        & 0.27657e+01_rb,0.15098e+02_rb,0.21927e+02_rb,0.25930e+02_rb,0.16137e+02_rb /)
      kbo(:, 2,59,13) = (/ &
        & 0.37163e+01_rb,0.18141e+02_rb,0.26603e+02_rb,0.31433e+02_rb,0.20078e+02_rb /)
      kbo(:, 3,59,13) = (/ &
        & 0.48770e+01_rb,0.21342e+02_rb,0.31379e+02_rb,0.36884e+02_rb,0.24100e+02_rb /)
      kbo(:, 4,59,13) = (/ &
        & 0.63518e+01_rb,0.24865e+02_rb,0.36188e+02_rb,0.42182e+02_rb,0.28007e+02_rb /)
      kbo(:, 5,59,13) = (/ &
        & 0.81684e+01_rb,0.28600e+02_rb,0.41035e+02_rb,0.47343e+02_rb,0.31847e+02_rb /)
      kbo(:, 1,13,14) = (/ &
        & 0.20563e+03_rb,0.16395e+03_rb,0.15131e+03_rb,0.13928e+03_rb,0.17336e+03_rb /)
      kbo(:, 2,13,14) = (/ &
        & 0.20583e+03_rb,0.16346e+03_rb,0.15102e+03_rb,0.13867e+03_rb,0.17209e+03_rb /)
      kbo(:, 3,13,14) = (/ &
        & 0.20579e+03_rb,0.16282e+03_rb,0.15051e+03_rb,0.13803e+03_rb,0.17119e+03_rb /)
      kbo(:, 4,13,14) = (/ &
        & 0.20538e+03_rb,0.16198e+03_rb,0.15005e+03_rb,0.13746e+03_rb,0.17062e+03_rb /)
      kbo(:, 5,13,14) = (/ &
        & 0.20471e+03_rb,0.16100e+03_rb,0.14942e+03_rb,0.13685e+03_rb,0.16998e+03_rb /)
      kbo(:, 1,14,14) = (/ &
        & 0.21387e+03_rb,0.17486e+03_rb,0.16410e+03_rb,0.14896e+03_rb,0.17926e+03_rb /)
      kbo(:, 2,14,14) = (/ &
        & 0.21398e+03_rb,0.17407e+03_rb,0.16354e+03_rb,0.14849e+03_rb,0.17872e+03_rb /)
      kbo(:, 3,14,14) = (/ &
        & 0.21371e+03_rb,0.17306e+03_rb,0.16300e+03_rb,0.14796e+03_rb,0.17838e+03_rb /)
      kbo(:, 4,14,14) = (/ &
        & 0.21320e+03_rb,0.17193e+03_rb,0.16238e+03_rb,0.14748e+03_rb,0.17807e+03_rb /)
      kbo(:, 5,14,14) = (/ &
        & 0.21239e+03_rb,0.17062e+03_rb,0.16158e+03_rb,0.14661e+03_rb,0.17726e+03_rb /)
      kbo(:, 1,15,14) = (/ &
        & 0.21868e+03_rb,0.18392e+03_rb,0.17597e+03_rb,0.15877e+03_rb,0.18409e+03_rb /)
      kbo(:, 2,15,14) = (/ &
        & 0.21861e+03_rb,0.18285e+03_rb,0.17533e+03_rb,0.15845e+03_rb,0.18413e+03_rb /)
      kbo(:, 3,15,14) = (/ &
        & 0.21838e+03_rb,0.18168e+03_rb,0.17469e+03_rb,0.15814e+03_rb,0.18423e+03_rb /)
      kbo(:, 4,15,14) = (/ &
        & 0.21785e+03_rb,0.18028e+03_rb,0.17392e+03_rb,0.15747e+03_rb,0.18391e+03_rb /)
      kbo(:, 5,15,14) = (/ &
        & 0.21714e+03_rb,0.17881e+03_rb,0.17281e+03_rb,0.15644e+03_rb,0.18337e+03_rb /)
      kbo(:, 1,16,14) = (/ &
        & 0.21989e+03_rb,0.19090e+03_rb,0.18666e+03_rb,0.16823e+03_rb,0.18717e+03_rb /)
      kbo(:, 2,16,14) = (/ &
        & 0.22000e+03_rb,0.18989e+03_rb,0.18604e+03_rb,0.16822e+03_rb,0.18797e+03_rb /)
      kbo(:, 3,16,14) = (/ &
        & 0.21982e+03_rb,0.18856e+03_rb,0.18533e+03_rb,0.16794e+03_rb,0.18851e+03_rb /)
      kbo(:, 4,16,14) = (/ &
        & 0.21954e+03_rb,0.18709e+03_rb,0.18432e+03_rb,0.16715e+03_rb,0.18854e+03_rb /)
      kbo(:, 5,16,14) = (/ &
        & 0.21898e+03_rb,0.18542e+03_rb,0.18303e+03_rb,0.16600e+03_rb,0.18839e+03_rb /)
      kbo(:, 1,17,14) = (/ &
        & 0.21793e+03_rb,0.19596e+03_rb,0.19603e+03_rb,0.17703e+03_rb,0.18857e+03_rb /)
      kbo(:, 2,17,14) = (/ &
        & 0.21821e+03_rb,0.19492e+03_rb,0.19548e+03_rb,0.17725e+03_rb,0.19023e+03_rb /)
      kbo(:, 3,17,14) = (/ &
        & 0.21850e+03_rb,0.19369e+03_rb,0.19462e+03_rb,0.17701e+03_rb,0.19133e+03_rb /)
      kbo(:, 4,17,14) = (/ &
        & 0.21855e+03_rb,0.19221e+03_rb,0.19343e+03_rb,0.17616e+03_rb,0.19194e+03_rb /)
      kbo(:, 5,17,14) = (/ &
        & 0.21839e+03_rb,0.19057e+03_rb,0.19206e+03_rb,0.17490e+03_rb,0.19225e+03_rb /)
      kbo(:, 1,18,14) = (/ &
        & 0.21311e+03_rb,0.19935e+03_rb,0.20385e+03_rb,0.18492e+03_rb,0.18864e+03_rb /)
      kbo(:, 2,18,14) = (/ &
        & 0.21399e+03_rb,0.19838e+03_rb,0.20339e+03_rb,0.18536e+03_rb,0.19117e+03_rb /)
      kbo(:, 3,18,14) = (/ &
        & 0.21477e+03_rb,0.19727e+03_rb,0.20251e+03_rb,0.18517e+03_rb,0.19296e+03_rb /)
      kbo(:, 4,18,14) = (/ &
        & 0.21537e+03_rb,0.19602e+03_rb,0.20131e+03_rb,0.18433e+03_rb,0.19426e+03_rb /)
      kbo(:, 5,18,14) = (/ &
        & 0.21594e+03_rb,0.19460e+03_rb,0.19989e+03_rb,0.18299e+03_rb,0.19511e+03_rb /)
      kbo(:, 1,19,14) = (/ &
        & 0.20628e+03_rb,0.20131e+03_rb,0.21018e+03_rb,0.19181e+03_rb,0.18780e+03_rb /)
      kbo(:, 2,19,14) = (/ &
        & 0.20786e+03_rb,0.20061e+03_rb,0.20985e+03_rb,0.19249e+03_rb,0.19117e+03_rb /)
      kbo(:, 3,19,14) = (/ &
        & 0.20939e+03_rb,0.19978e+03_rb,0.20915e+03_rb,0.19245e+03_rb,0.19379e+03_rb /)
      kbo(:, 4,19,14) = (/ &
        & 0.21078e+03_rb,0.19876e+03_rb,0.20808e+03_rb,0.19159e+03_rb,0.19578e+03_rb /)
      kbo(:, 5,19,14) = (/ &
        & 0.21196e+03_rb,0.19762e+03_rb,0.20666e+03_rb,0.19020e+03_rb,0.19726e+03_rb /)
      kbo(:, 1,20,14) = (/ &
        & 0.19806e+03_rb,0.20215e+03_rb,0.21525e+03_rb,0.19781e+03_rb,0.18646e+03_rb /)
      kbo(:, 2,20,14) = (/ &
        & 0.20054e+03_rb,0.20180e+03_rb,0.21527e+03_rb,0.19874e+03_rb,0.19072e+03_rb /)
      kbo(:, 3,20,14) = (/ &
        & 0.20298e+03_rb,0.20132e+03_rb,0.21478e+03_rb,0.19877e+03_rb,0.19413e+03_rb /)
      kbo(:, 4,20,14) = (/ &
        & 0.20522e+03_rb,0.20070e+03_rb,0.21384e+03_rb,0.19793e+03_rb,0.19681e+03_rb /)
      kbo(:, 5,20,14) = (/ &
        & 0.20725e+03_rb,0.19975e+03_rb,0.21249e+03_rb,0.19659e+03_rb,0.19899e+03_rb /)
      kbo(:, 1,21,14) = (/ &
        & 0.18919e+03_rb,0.20217e+03_rb,0.21942e+03_rb,0.20291e+03_rb,0.18499e+03_rb /)
      kbo(:, 2,21,14) = (/ &
        & 0.19268e+03_rb,0.20239e+03_rb,0.21974e+03_rb,0.20415e+03_rb,0.19012e+03_rb /)
      kbo(:, 3,21,14) = (/ &
        & 0.19609e+03_rb,0.20231e+03_rb,0.21947e+03_rb,0.20425e+03_rb,0.19432e+03_rb /)
      kbo(:, 4,21,14) = (/ &
        & 0.19926e+03_rb,0.20201e+03_rb,0.21866e+03_rb,0.20346e+03_rb,0.19770e+03_rb /)
      kbo(:, 5,21,14) = (/ &
        & 0.20224e+03_rb,0.20136e+03_rb,0.21742e+03_rb,0.20212e+03_rb,0.20044e+03_rb /)
      kbo(:, 1,22,14) = (/ &
        & 0.18064e+03_rb,0.20182e+03_rb,0.22276e+03_rb,0.20740e+03_rb,0.18409e+03_rb /)
      kbo(:, 2,22,14) = (/ &
        & 0.18513e+03_rb,0.20253e+03_rb,0.22341e+03_rb,0.20876e+03_rb,0.18988e+03_rb /)
      kbo(:, 3,22,14) = (/ &
        & 0.18951e+03_rb,0.20287e+03_rb,0.22329e+03_rb,0.20890e+03_rb,0.19474e+03_rb /)
      kbo(:, 4,22,14) = (/ &
        & 0.19368e+03_rb,0.20283e+03_rb,0.22259e+03_rb,0.20812e+03_rb,0.19869e+03_rb /)
      kbo(:, 5,22,14) = (/ &
        & 0.19752e+03_rb,0.20248e+03_rb,0.22143e+03_rb,0.20676e+03_rb,0.20185e+03_rb /)
      kbo(:, 1,23,14) = (/ &
        & 0.17261e+03_rb,0.20129e+03_rb,0.22560e+03_rb,0.21126e+03_rb,0.18360e+03_rb /)
      kbo(:, 2,23,14) = (/ &
        & 0.17813e+03_rb,0.20255e+03_rb,0.22645e+03_rb,0.21277e+03_rb,0.18993e+03_rb /)
      kbo(:, 3,23,14) = (/ &
        & 0.18347e+03_rb,0.20322e+03_rb,0.22649e+03_rb,0.21288e+03_rb,0.19534e+03_rb /)
      kbo(:, 4,23,14) = (/ &
        & 0.18858e+03_rb,0.20345e+03_rb,0.22593e+03_rb,0.21211e+03_rb,0.19972e+03_rb /)
      kbo(:, 5,23,14) = (/ &
        & 0.19325e+03_rb,0.20330e+03_rb,0.22470e+03_rb,0.21066e+03_rb,0.20314e+03_rb /)
      kbo(:, 1,24,14) = (/ &
        & 0.16550e+03_rb,0.20079e+03_rb,0.22797e+03_rb,0.21459e+03_rb,0.18352e+03_rb /)
      kbo(:, 2,24,14) = (/ &
        & 0.17199e+03_rb,0.20247e+03_rb,0.22899e+03_rb,0.21614e+03_rb,0.19031e+03_rb /)
      kbo(:, 3,24,14) = (/ &
        & 0.17829e+03_rb,0.20349e+03_rb,0.22920e+03_rb,0.21621e+03_rb,0.19610e+03_rb /)
      kbo(:, 4,24,14) = (/ &
        & 0.18424e+03_rb,0.20395e+03_rb,0.22865e+03_rb,0.21544e+03_rb,0.20080e+03_rb /)
      kbo(:, 5,24,14) = (/ &
        & 0.18966e+03_rb,0.20393e+03_rb,0.22735e+03_rb,0.21388e+03_rb,0.20437e+03_rb /)
      kbo(:, 1,25,14) = (/ &
        & 0.15949e+03_rb,0.20048e+03_rb,0.22999e+03_rb,0.21758e+03_rb,0.18395e+03_rb /)
      kbo(:, 2,25,14) = (/ &
        & 0.16698e+03_rb,0.20247e+03_rb,0.23119e+03_rb,0.21903e+03_rb,0.19108e+03_rb /)
      kbo(:, 3,25,14) = (/ &
        & 0.17411e+03_rb,0.20370e+03_rb,0.23144e+03_rb,0.21904e+03_rb,0.19711e+03_rb /)
      kbo(:, 4,25,14) = (/ &
        & 0.18088e+03_rb,0.20435e+03_rb,0.23082e+03_rb,0.21817e+03_rb,0.20192e+03_rb /)
      kbo(:, 5,25,14) = (/ &
        & 0.18710e+03_rb,0.20450e+03_rb,0.22950e+03_rb,0.21648e+03_rb,0.20559e+03_rb /)
      kbo(:, 1,26,14) = (/ &
        & 0.15479e+03_rb,0.20038e+03_rb,0.23181e+03_rb,0.22016e+03_rb,0.18490e+03_rb /)
      kbo(:, 2,26,14) = (/ &
        & 0.16322e+03_rb,0.20258e+03_rb,0.23307e+03_rb,0.22145e+03_rb,0.19222e+03_rb /)
      kbo(:, 3,26,14) = (/ &
        & 0.17119e+03_rb,0.20401e+03_rb,0.23326e+03_rb,0.22140e+03_rb,0.19831e+03_rb /)
      kbo(:, 4,26,14) = (/ &
        & 0.17866e+03_rb,0.20478e+03_rb,0.23259e+03_rb,0.22038e+03_rb,0.20315e+03_rb /)
      kbo(:, 5,26,14) = (/ &
        & 0.18553e+03_rb,0.20498e+03_rb,0.23122e+03_rb,0.21854e+03_rb,0.20685e+03_rb /)
      kbo(:, 1,27,14) = (/ &
        & 0.15143e+03_rb,0.20049e+03_rb,0.23344e+03_rb,0.22241e+03_rb,0.18615e+03_rb /)
      kbo(:, 2,27,14) = (/ &
        & 0.16070e+03_rb,0.20279e+03_rb,0.23465e+03_rb,0.22352e+03_rb,0.19353e+03_rb /)
      kbo(:, 3,27,14) = (/ &
        & 0.16932e+03_rb,0.20430e+03_rb,0.23473e+03_rb,0.22331e+03_rb,0.19960e+03_rb /)
      kbo(:, 4,27,14) = (/ &
        & 0.17737e+03_rb,0.20518e+03_rb,0.23402e+03_rb,0.22212e+03_rb,0.20440e+03_rb /)
      kbo(:, 5,27,14) = (/ &
        & 0.18472e+03_rb,0.20537e+03_rb,0.23257e+03_rb,0.22015e+03_rb,0.20807e+03_rb /)
      kbo(:, 1,28,14) = (/ &
        & 0.14935e+03_rb,0.20078e+03_rb,0.23487e+03_rb,0.22436e+03_rb,0.18762e+03_rb /)
      kbo(:, 2,28,14) = (/ &
        & 0.15926e+03_rb,0.20312e+03_rb,0.23597e+03_rb,0.22523e+03_rb,0.19491e+03_rb /)
      kbo(:, 3,28,14) = (/ &
        & 0.16839e+03_rb,0.20467e+03_rb,0.23597e+03_rb,0.22484e+03_rb,0.20092e+03_rb /)
      kbo(:, 4,28,14) = (/ &
        & 0.17687e+03_rb,0.20554e+03_rb,0.23516e+03_rb,0.22345e+03_rb,0.20562e+03_rb /)
      kbo(:, 5,28,14) = (/ &
        & 0.18453e+03_rb,0.20570e+03_rb,0.23357e+03_rb,0.22135e+03_rb,0.20921e+03_rb /)
      kbo(:, 1,29,14) = (/ &
        & 0.14844e+03_rb,0.20117e+03_rb,0.23614e+03_rb,0.22601e+03_rb,0.18927e+03_rb /)
      kbo(:, 2,29,14) = (/ &
        & 0.15878e+03_rb,0.20350e+03_rb,0.23707e+03_rb,0.22664e+03_rb,0.19642e+03_rb /)
      kbo(:, 3,29,14) = (/ &
        & 0.16829e+03_rb,0.20505e+03_rb,0.23696e+03_rb,0.22601e+03_rb,0.20229e+03_rb /)
      kbo(:, 4,29,14) = (/ &
        & 0.17702e+03_rb,0.20585e+03_rb,0.23602e+03_rb,0.22445e+03_rb,0.20685e+03_rb /)
      kbo(:, 5,29,14) = (/ &
        & 0.18487e+03_rb,0.20599e+03_rb,0.23430e+03_rb,0.22220e+03_rb,0.21031e+03_rb /)
      kbo(:, 1,30,14) = (/ &
        & 0.14841e+03_rb,0.20165e+03_rb,0.23724e+03_rb,0.22742e+03_rb,0.19101e+03_rb /)
      kbo(:, 2,30,14) = (/ &
        & 0.15903e+03_rb,0.20395e+03_rb,0.23798e+03_rb,0.22774e+03_rb,0.19796e+03_rb /)
      kbo(:, 3,30,14) = (/ &
        & 0.16878e+03_rb,0.20543e+03_rb,0.23774e+03_rb,0.22689e+03_rb,0.20367e+03_rb /)
      kbo(:, 4,30,14) = (/ &
        & 0.17765e+03_rb,0.20614e+03_rb,0.23662e+03_rb,0.22515e+03_rb,0.20804e+03_rb /)
      kbo(:, 5,30,14) = (/ &
        & 0.18556e+03_rb,0.20626e+03_rb,0.23478e+03_rb,0.22274e+03_rb,0.21133e+03_rb /)
      kbo(:, 1,31,14) = (/ &
        & 0.14910e+03_rb,0.20219e+03_rb,0.23816e+03_rb,0.22857e+03_rb,0.19280e+03_rb /)
      kbo(:, 2,31,14) = (/ &
        & 0.15987e+03_rb,0.20442e+03_rb,0.23873e+03_rb,0.22860e+03_rb,0.19955e+03_rb /)
      kbo(:, 3,31,14) = (/ &
        & 0.16974e+03_rb,0.20578e+03_rb,0.23830e+03_rb,0.22751e+03_rb,0.20503e+03_rb /)
      kbo(:, 4,31,14) = (/ &
        & 0.17870e+03_rb,0.20640e+03_rb,0.23703e+03_rb,0.22559e+03_rb,0.20917e+03_rb /)
      kbo(:, 5,31,14) = (/ &
        & 0.18656e+03_rb,0.20647e+03_rb,0.23504e+03_rb,0.22303e+03_rb,0.21232e+03_rb /)
      kbo(:, 1,32,14) = (/ &
        & 0.15031e+03_rb,0.20279e+03_rb,0.23897e+03_rb,0.22949e+03_rb,0.19459e+03_rb /)
      kbo(:, 2,32,14) = (/ &
        & 0.16119e+03_rb,0.20489e+03_rb,0.23931e+03_rb,0.22922e+03_rb,0.20113e+03_rb /)
      kbo(:, 3,32,14) = (/ &
        & 0.17110e+03_rb,0.20610e+03_rb,0.23869e+03_rb,0.22791e+03_rb,0.20634e+03_rb /)
      kbo(:, 4,32,14) = (/ &
        & 0.18002e+03_rb,0.20662e+03_rb,0.23723e+03_rb,0.22582e+03_rb,0.21029e+03_rb /)
      kbo(:, 5,32,14) = (/ &
        & 0.18777e+03_rb,0.20659e+03_rb,0.23510e+03_rb,0.22308e+03_rb,0.21322e+03_rb /)
      kbo(:, 1,33,14) = (/ &
        & 0.15197e+03_rb,0.20341e+03_rb,0.23965e+03_rb,0.23019e+03_rb,0.19639e+03_rb /)
      kbo(:, 2,33,14) = (/ &
        & 0.16285e+03_rb,0.20534e+03_rb,0.23976e+03_rb,0.22964e+03_rb,0.20267e+03_rb /)
      kbo(:, 3,33,14) = (/ &
        & 0.17270e+03_rb,0.20638e+03_rb,0.23893e+03_rb,0.22811e+03_rb,0.20758e+03_rb /)
      kbo(:, 4,33,14) = (/ &
        & 0.18151e+03_rb,0.20680e+03_rb,0.23727e+03_rb,0.22584e+03_rb,0.21132e+03_rb /)
      kbo(:, 5,33,14) = (/ &
        & 0.18912e+03_rb,0.20665e+03_rb,0.23500e+03_rb,0.22296e+03_rb,0.21406e+03_rb /)
      kbo(:, 1,34,14) = (/ &
        & 0.15352e+03_rb,0.20391e+03_rb,0.24014e+03_rb,0.23069e+03_rb,0.19790e+03_rb /)
      kbo(:, 2,34,14) = (/ &
        & 0.16437e+03_rb,0.20566e+03_rb,0.24006e+03_rb,0.22992e+03_rb,0.20396e+03_rb /)
      kbo(:, 3,34,14) = (/ &
        & 0.17416e+03_rb,0.20662e+03_rb,0.23907e+03_rb,0.22823e+03_rb,0.20863e+03_rb /)
      kbo(:, 4,34,14) = (/ &
        & 0.18285e+03_rb,0.20695e+03_rb,0.23728e+03_rb,0.22580e+03_rb,0.21218e+03_rb /)
      kbo(:, 5,34,14) = (/ &
        & 0.19033e+03_rb,0.20667e+03_rb,0.23487e+03_rb,0.22280e+03_rb,0.21474e+03_rb /)
      kbo(:, 1,35,14) = (/ &
        & 0.15421e+03_rb,0.20418e+03_rb,0.24052e+03_rb,0.23110e+03_rb,0.19876e+03_rb /)
      kbo(:, 2,35,14) = (/ &
        & 0.16507e+03_rb,0.20586e+03_rb,0.24033e+03_rb,0.23020e+03_rb,0.20468e+03_rb /)
      kbo(:, 3,35,14) = (/ &
        & 0.17486e+03_rb,0.20677e+03_rb,0.23924e+03_rb,0.22841e+03_rb,0.20923e+03_rb /)
      kbo(:, 4,35,14) = (/ &
        & 0.18350e+03_rb,0.20705e+03_rb,0.23737e+03_rb,0.22589e+03_rb,0.21269e+03_rb /)
      kbo(:, 5,35,14) = (/ &
        & 0.19094e+03_rb,0.20669e+03_rb,0.23489e+03_rb,0.22281e+03_rb,0.21514e+03_rb /)
      kbo(:, 1,36,14) = (/ &
        & 0.15381e+03_rb,0.20424e+03_rb,0.24080e+03_rb,0.23145e+03_rb,0.19888e+03_rb /)
      kbo(:, 2,36,14) = (/ &
        & 0.16478e+03_rb,0.20592e+03_rb,0.24060e+03_rb,0.23055e+03_rb,0.20484e+03_rb /)
      kbo(:, 3,36,14) = (/ &
        & 0.17464e+03_rb,0.20684e+03_rb,0.23950e+03_rb,0.22874e+03_rb,0.20938e+03_rb /)
      kbo(:, 4,36,14) = (/ &
        & 0.18336e+03_rb,0.20715e+03_rb,0.23762e+03_rb,0.22618e+03_rb,0.21284e+03_rb /)
      kbo(:, 5,36,14) = (/ &
        & 0.19086e+03_rb,0.20678e+03_rb,0.23514e+03_rb,0.22311e+03_rb,0.21532e+03_rb /)
      kbo(:, 1,37,14) = (/ &
        & 0.15177e+03_rb,0.20394e+03_rb,0.24096e+03_rb,0.23178e+03_rb,0.19802e+03_rb /)
      kbo(:, 2,37,14) = (/ &
        & 0.16298e+03_rb,0.20578e+03_rb,0.24091e+03_rb,0.23102e+03_rb,0.20416e+03_rb /)
      kbo(:, 3,37,14) = (/ &
        & 0.17307e+03_rb,0.20681e+03_rb,0.23994e+03_rb,0.22934e+03_rb,0.20893e+03_rb /)
      kbo(:, 4,37,14) = (/ &
        & 0.18204e+03_rb,0.20721e+03_rb,0.23818e+03_rb,0.22688e+03_rb,0.21255e+03_rb /)
      kbo(:, 5,37,14) = (/ &
        & 0.18973e+03_rb,0.20695e+03_rb,0.23573e+03_rb,0.22383e+03_rb,0.21516e+03_rb /)
      kbo(:, 1,38,14) = (/ &
        & 0.14968e+03_rb,0.20359e+03_rb,0.24104e+03_rb,0.23202e+03_rb,0.19703e+03_rb /)
      kbo(:, 2,38,14) = (/ &
        & 0.16113e+03_rb,0.20560e+03_rb,0.24116e+03_rb,0.23143e+03_rb,0.20341e+03_rb /)
      kbo(:, 3,38,14) = (/ &
        & 0.17144e+03_rb,0.20674e+03_rb,0.24033e+03_rb,0.22988e+03_rb,0.20838e+03_rb /)
      kbo(:, 4,38,14) = (/ &
        & 0.18063e+03_rb,0.20724e+03_rb,0.23866e+03_rb,0.22752e+03_rb,0.21216e+03_rb /)
      kbo(:, 5,38,14) = (/ &
        & 0.18855e+03_rb,0.20710e+03_rb,0.23632e+03_rb,0.22453e+03_rb,0.21493e+03_rb /)
      kbo(:, 1,39,14) = (/ &
        & 0.14764e+03_rb,0.20322e+03_rb,0.24106e+03_rb,0.23220e+03_rb,0.19604e+03_rb /)
      kbo(:, 2,39,14) = (/ &
        & 0.15932e+03_rb,0.20540e+03_rb,0.24137e+03_rb,0.23180e+03_rb,0.20262e+03_rb /)
      kbo(:, 3,39,14) = (/ &
        & 0.16984e+03_rb,0.20665e+03_rb,0.24065e+03_rb,0.23036e+03_rb,0.20781e+03_rb /)
      kbo(:, 4,39,14) = (/ &
        & 0.17923e+03_rb,0.20725e+03_rb,0.23910e+03_rb,0.22810e+03_rb,0.21175e+03_rb /)
      kbo(:, 5,39,14) = (/ &
        & 0.18737e+03_rb,0.20722e+03_rb,0.23684e+03_rb,0.22517e+03_rb,0.21467e+03_rb /)
      kbo(:, 1,40,14) = (/ &
        & 0.14438e+03_rb,0.20254e+03_rb,0.24092e+03_rb,0.23225e+03_rb,0.19425e+03_rb /)
      kbo(:, 2,40,14) = (/ &
        & 0.15633e+03_rb,0.20498e+03_rb,0.24152e+03_rb,0.23217e+03_rb,0.20115e+03_rb /)
      kbo(:, 3,40,14) = (/ &
        & 0.16718e+03_rb,0.20642e+03_rb,0.24105e+03_rb,0.23096e+03_rb,0.20672e+03_rb /)
      kbo(:, 4,40,14) = (/ &
        & 0.17690e+03_rb,0.20720e+03_rb,0.23969e+03_rb,0.22887e+03_rb,0.21093e+03_rb /)
      kbo(:, 5,40,14) = (/ &
        & 0.18539e+03_rb,0.20733e+03_rb,0.23758e+03_rb,0.22609e+03_rb,0.21411e+03_rb /)
      kbo(:, 1,41,14) = (/ &
        & 0.14093e+03_rb,0.20170e+03_rb,0.24062e+03_rb,0.23214e+03_rb,0.19226e+03_rb /)
      kbo(:, 2,41,14) = (/ &
        & 0.15309e+03_rb,0.20441e+03_rb,0.24155e+03_rb,0.23243e+03_rb,0.19951e+03_rb /)
      kbo(:, 3,41,14) = (/ &
        & 0.16434e+03_rb,0.20613e+03_rb,0.24136e+03_rb,0.23149e+03_rb,0.20545e+03_rb /)
      kbo(:, 4,41,14) = (/ &
        & 0.17439e+03_rb,0.20709e+03_rb,0.24022e+03_rb,0.22963e+03_rb,0.21000e+03_rb /)
      kbo(:, 5,41,14) = (/ &
        & 0.18325e+03_rb,0.20739e+03_rb,0.23829e+03_rb,0.22700e+03_rb,0.21344e+03_rb /)
      kbo(:, 1,42,14) = (/ &
        & 0.13742e+03_rb,0.20072e+03_rb,0.24018e+03_rb,0.23188e+03_rb,0.19017e+03_rb /)
      kbo(:, 2,42,14) = (/ &
        & 0.14987e+03_rb,0.20381e+03_rb,0.24151e+03_rb,0.23260e+03_rb,0.19778e+03_rb /)
      kbo(:, 3,42,14) = (/ &
        & 0.16144e+03_rb,0.20580e+03_rb,0.24159e+03_rb,0.23194e+03_rb,0.20408e+03_rb /)
      kbo(:, 4,42,14) = (/ &
        & 0.17182e+03_rb,0.20693e+03_rb,0.24068e+03_rb,0.23030e+03_rb,0.20897e+03_rb /)
      kbo(:, 5,42,14) = (/ &
        & 0.18101e+03_rb,0.20739e+03_rb,0.23894e+03_rb,0.22785e+03_rb,0.21269e+03_rb /)
      kbo(:, 1,43,14) = (/ &
        & 0.13316e+03_rb,0.19946e+03_rb,0.23952e+03_rb,0.23134e+03_rb,0.18746e+03_rb /)
      kbo(:, 2,43,14) = (/ &
        & 0.14590e+03_rb,0.20299e+03_rb,0.24131e+03_rb,0.23265e+03_rb,0.19555e+03_rb /)
      kbo(:, 3,43,14) = (/ &
        & 0.15782e+03_rb,0.20531e+03_rb,0.24176e+03_rb,0.23237e+03_rb,0.20228e+03_rb /)
      kbo(:, 4,43,14) = (/ &
        & 0.16860e+03_rb,0.20666e+03_rb,0.24113e+03_rb,0.23099e+03_rb,0.20761e+03_rb /)
      kbo(:, 5,43,14) = (/ &
        & 0.17818e+03_rb,0.20733e+03_rb,0.23964e+03_rb,0.22878e+03_rb,0.21166e+03_rb /)
      kbo(:, 1,44,14) = (/ &
        & 0.12851e+03_rb,0.19793e+03_rb,0.23860e+03_rb,0.23051e+03_rb,0.18440e+03_rb /)
      kbo(:, 2,44,14) = (/ &
        & 0.14156e+03_rb,0.20195e+03_rb,0.24092e+03_rb,0.23249e+03_rb,0.19298e+03_rb /)
      kbo(:, 3,44,14) = (/ &
        & 0.15381e+03_rb,0.20468e+03_rb,0.24182e+03_rb,0.23268e+03_rb,0.20015e+03_rb /)
      kbo(:, 4,44,14) = (/ &
        & 0.16494e+03_rb,0.20625e+03_rb,0.24148e+03_rb,0.23163e+03_rb,0.20597e+03_rb /)
      kbo(:, 5,44,14) = (/ &
        & 0.17499e+03_rb,0.20719e+03_rb,0.24029e+03_rb,0.22969e+03_rb,0.21043e+03_rb /)
      kbo(:, 1,45,14) = (/ &
        & 0.12377e+03_rb,0.19622e+03_rb,0.23745e+03_rb,0.22938e+03_rb,0.18114e+03_rb /)
      kbo(:, 2,45,14) = (/ &
        & 0.13713e+03_rb,0.20074e+03_rb,0.24035e+03_rb,0.23210e+03_rb,0.19024e+03_rb /)
      kbo(:, 3,45,14) = (/ &
        & 0.14969e+03_rb,0.20389e+03_rb,0.24171e+03_rb,0.23283e+03_rb,0.19787e+03_rb /)
      kbo(:, 4,45,14) = (/ &
        & 0.16122e+03_rb,0.20580e+03_rb,0.24172e+03_rb,0.23215e+03_rb,0.20417e+03_rb /)
      kbo(:, 5,45,14) = (/ &
        & 0.17169e+03_rb,0.20697e+03_rb,0.24085e+03_rb,0.23051e+03_rb,0.20907e+03_rb /)
      kbo(:, 1,46,14) = (/ &
        & 0.11871e+03_rb,0.19418e+03_rb,0.23600e+03_rb,0.22790e+03_rb,0.17747e+03_rb /)
      kbo(:, 2,46,14) = (/ &
        & 0.13231e+03_rb,0.19929e+03_rb,0.23954e+03_rb,0.23143e+03_rb,0.18712e+03_rb /)
      kbo(:, 3,46,14) = (/ &
        & 0.14514e+03_rb,0.20288e+03_rb,0.24138e+03_rb,0.23280e+03_rb,0.19526e+03_rb /)
      kbo(:, 4,46,14) = (/ &
        & 0.15711e+03_rb,0.20523e+03_rb,0.24187e+03_rb,0.23257e+03_rb,0.20207e+03_rb /)
      kbo(:, 5,46,14) = (/ &
        & 0.16802e+03_rb,0.20664e+03_rb,0.24132e+03_rb,0.23126e+03_rb,0.20749e+03_rb /)
      kbo(:, 1,47,14) = (/ &
        & 0.11295e+03_rb,0.19159e+03_rb,0.23398e+03_rb,0.22588e+03_rb,0.17313e+03_rb /)
      kbo(:, 2,47,14) = (/ &
        & 0.12683e+03_rb,0.19741e+03_rb,0.23835e+03_rb,0.23031e+03_rb,0.18342e+03_rb /)
      kbo(:, 3,47,14) = (/ &
        & 0.13999e+03_rb,0.20159e+03_rb,0.24085e+03_rb,0.23253e+03_rb,0.19217e+03_rb /)
      kbo(:, 4,47,14) = (/ &
        & 0.15234e+03_rb,0.20442e+03_rb,0.24186e+03_rb,0.23286e+03_rb,0.19949e+03_rb /)
      kbo(:, 5,47,14) = (/ &
        & 0.16373e+03_rb,0.20618e+03_rb,0.24169e+03_rb,0.23194e+03_rb,0.20549e+03_rb /)
      kbo(:, 1,48,14) = (/ &
        & 0.10712e+03_rb,0.18871e+03_rb,0.23159e+03_rb,0.22347e+03_rb,0.16850e+03_rb /)
      kbo(:, 2,48,14) = (/ &
        & 0.12124e+03_rb,0.19529e+03_rb,0.23687e+03_rb,0.22882e+03_rb,0.17946e+03_rb /)
      kbo(:, 3,48,14) = (/ &
        & 0.13473e+03_rb,0.20007e+03_rb,0.24005e+03_rb,0.23191e+03_rb,0.18883e+03_rb /)
      kbo(:, 4,48,14) = (/ &
        & 0.14742e+03_rb,0.20342e+03_rb,0.24164e+03_rb,0.23294e+03_rb,0.19670e+03_rb /)
      kbo(:, 5,48,14) = (/ &
        & 0.15928e+03_rb,0.20562e+03_rb,0.24192e+03_rb,0.23247e+03_rb,0.20325e+03_rb /)
      kbo(:, 1,49,14) = (/ &
        & 0.10120e+03_rb,0.18550e+03_rb,0.22880e+03_rb,0.22071e+03_rb,0.16363e+03_rb /)
      kbo(:, 2,49,14) = (/ &
        & 0.11556e+03_rb,0.19285e+03_rb,0.23504e+03_rb,0.22697e+03_rb,0.17524e+03_rb /)
      kbo(:, 3,49,14) = (/ &
        & 0.12932e+03_rb,0.19832e+03_rb,0.23901e+03_rb,0.23097e+03_rb,0.18524e+03_rb /)
      kbo(:, 4,49,14) = (/ &
        & 0.14238e+03_rb,0.20226e+03_rb,0.24120e+03_rb,0.23278e+03_rb,0.19370e+03_rb /)
      kbo(:, 5,49,14) = (/ &
        & 0.15460e+03_rb,0.20488e+03_rb,0.24199e+03_rb,0.23283e+03_rb,0.20078e+03_rb /)
      kbo(:, 1,50,14) = (/ &
        & 0.95603e+02_rb,0.18216e+03_rb,0.22576e+03_rb,0.21768e+03_rb,0.15878e+03_rb /)
      kbo(:, 2,50,14) = (/ &
        & 0.11015e+03_rb,0.19031e+03_rb,0.23298e+03_rb,0.22490e+03_rb,0.17105e+03_rb /)
      kbo(:, 3,50,14) = (/ &
        & 0.12415e+03_rb,0.19646e+03_rb,0.23775e+03_rb,0.22975e+03_rb,0.18166e+03_rb /)
      kbo(:, 4,50,14) = (/ &
        & 0.13752e+03_rb,0.20094e+03_rb,0.24058e+03_rb,0.23236e+03_rb,0.19069e+03_rb /)
      kbo(:, 5,50,14) = (/ &
        & 0.15007e+03_rb,0.20403e+03_rb,0.24187e+03_rb,0.23299e+03_rb,0.19826e+03_rb /)
      kbo(:, 1,51,14) = (/ &
        & 0.90137e+02_rb,0.17868e+03_rb,0.22240e+03_rb,0.21442e+03_rb,0.15384e+03_rb /)
      kbo(:, 2,51,14) = (/ &
        & 0.10484e+03_rb,0.18757e+03_rb,0.23066e+03_rb,0.22258e+03_rb,0.16677e+03_rb /)
      kbo(:, 3,51,14) = (/ &
        & 0.11907e+03_rb,0.19442e+03_rb,0.23628e+03_rb,0.22824e+03_rb,0.17796e+03_rb /)
      kbo(:, 4,51,14) = (/ &
        & 0.13269e+03_rb,0.19948e+03_rb,0.23976e+03_rb,0.23167e+03_rb,0.18756e+03_rb /)
      kbo(:, 5,51,14) = (/ &
        & 0.14552e+03_rb,0.20302e+03_rb,0.24154e+03_rb,0.23296e+03_rb,0.19565e+03_rb /)
      kbo(:, 1,52,14) = (/ &
        & 0.84690e+02_rb,0.17496e+03_rb,0.21864e+03_rb,0.21080e+03_rb,0.14862e+03_rb /)
      kbo(:, 2,52,14) = (/ &
        & 0.99471e+02_rb,0.18453e+03_rb,0.22798e+03_rb,0.21992e+03_rb,0.16225e+03_rb /)
      kbo(:, 3,52,14) = (/ &
        & 0.11391e+03_rb,0.19213e+03_rb,0.23451e+03_rb,0.22644e+03_rb,0.17406e+03_rb /)
      kbo(:, 4,52,14) = (/ &
        & 0.12777e+03_rb,0.19781e+03_rb,0.23872e+03_rb,0.23070e+03_rb,0.18424e+03_rb /)
      kbo(:, 5,52,14) = (/ &
        & 0.14091e+03_rb,0.20189e+03_rb,0.24107e+03_rb,0.23273e+03_rb,0.19286e+03_rb /)
      kbo(:, 1,53,14) = (/ &
        & 0.79262e+02_rb,0.17095e+03_rb,0.21436e+03_rb,0.20681e+03_rb,0.14317e+03_rb /)
      kbo(:, 2,53,14) = (/ &
        & 0.94047e+02_rb,0.18126e+03_rb,0.22494e+03_rb,0.21690e+03_rb,0.15748e+03_rb /)
      kbo(:, 3,53,14) = (/ &
        & 0.10865e+03_rb,0.18958e+03_rb,0.23241e+03_rb,0.22434e+03_rb,0.16994e+03_rb /)
      kbo(:, 4,53,14) = (/ &
        & 0.12274e+03_rb,0.19595e+03_rb,0.23742e+03_rb,0.22941e+03_rb,0.18070e+03_rb /)
      kbo(:, 5,53,14) = (/ &
        & 0.13618e+03_rb,0.20056e+03_rb,0.24040e+03_rb,0.23224e+03_rb,0.18988e+03_rb /)
      kbo(:, 1,54,14) = (/ &
        & 0.74312e+02_rb,0.16701e+03_rb,0.21000e+03_rb,0.20283e+03_rb,0.13801e+03_rb /)
      kbo(:, 2,54,14) = (/ &
        & 0.89059e+02_rb,0.17802e+03_rb,0.22177e+03_rb,0.21381e+03_rb,0.15289e+03_rb /)
      kbo(:, 3,54,14) = (/ &
        & 0.10379e+03_rb,0.18701e+03_rb,0.23021e+03_rb,0.22214e+03_rb,0.16595e+03_rb /)
      kbo(:, 4,54,14) = (/ &
        & 0.11808e+03_rb,0.19403e+03_rb,0.23602e+03_rb,0.22797e+03_rb,0.17728e+03_rb /)
      kbo(:, 5,54,14) = (/ &
        & 0.13174e+03_rb,0.19918e+03_rb,0.23961e+03_rb,0.23155e+03_rb,0.18697e+03_rb /)
      kbo(:, 1,55,14) = (/ &
        & 0.69615e+02_rb,0.16296e+03_rb,0.20540e+03_rb,0.19866e+03_rb,0.13288e+03_rb /)
      kbo(:, 2,55,14) = (/ &
        & 0.84276e+02_rb,0.17466e+03_rb,0.21835e+03_rb,0.21056e+03_rb,0.14828e+03_rb /)
      kbo(:, 3,55,14) = (/ &
        & 0.99066e+02_rb,0.18433e+03_rb,0.22782e+03_rb,0.21977e+03_rb,0.16197e+03_rb /)
      kbo(:, 4,55,14) = (/ &
        & 0.11354e+03_rb,0.19199e+03_rb,0.23442e+03_rb,0.22636e+03_rb,0.17383e+03_rb /)
      kbo(:, 5,55,14) = (/ &
        & 0.12741e+03_rb,0.19770e+03_rb,0.23866e+03_rb,0.23065e+03_rb,0.18403e+03_rb /)
      kbo(:, 1,56,14) = (/ &
        & 0.64944e+02_rb,0.15877e+03_rb,0.20044e+03_rb,0.19421e+03_rb,0.12762e+03_rb /)
      kbo(:, 2,56,14) = (/ &
        & 0.79524e+02_rb,0.17114e+03_rb,0.21461e+03_rb,0.20707e+03_rb,0.14351e+03_rb /)
      kbo(:, 3,56,14) = (/ &
        & 0.94323e+02_rb,0.18144e+03_rb,0.22514e+03_rb,0.21711e+03_rb,0.15778e+03_rb /)
      kbo(:, 4,56,14) = (/ &
        & 0.10894e+03_rb,0.18976e+03_rb,0.23257e+03_rb,0.22452e+03_rb,0.17022e+03_rb /)
      kbo(:, 5,56,14) = (/ &
        & 0.12301e+03_rb,0.19605e+03_rb,0.23752e+03_rb,0.22954e+03_rb,0.18094e+03_rb /)
      kbo(:, 1,57,14) = (/ &
        & 0.60353e+02_rb,0.15431e+03_rb,0.19514e+03_rb,0.18945e+03_rb,0.12225e+03_rb /)
      kbo(:, 2,57,14) = (/ &
        & 0.74815e+02_rb,0.16742e+03_rb,0.21049e+03_rb,0.20329e+03_rb,0.13860e+03_rb /)
      kbo(:, 3,57,14) = (/ &
        & 0.89567e+02_rb,0.17837e+03_rb,0.22214e+03_rb,0.21419e+03_rb,0.15343e+03_rb /)
      kbo(:, 4,57,14) = (/ &
        & 0.10431e+03_rb,0.18733e+03_rb,0.23050e+03_rb,0.22244e+03_rb,0.16644e+03_rb /)
      kbo(:, 5,57,14) = (/ &
        & 0.11856e+03_rb,0.19423e+03_rb,0.23618e+03_rb,0.22816e+03_rb,0.17767e+03_rb /)
      kbo(:, 1,58,14) = (/ &
        & 0.56098e+02_rb,0.14991e+03_rb,0.18980e+03_rb,0.18465e+03_rb,0.11706e+03_rb /)
      kbo(:, 2,58,14) = (/ &
        & 0.70377e+02_rb,0.16367e+03_rb,0.20622e+03_rb,0.19942e+03_rb,0.13378e+03_rb /)
      kbo(:, 3,58,14) = (/ &
        & 0.85049e+02_rb,0.17528e+03_rb,0.21902e+03_rb,0.21119e+03_rb,0.14910e+03_rb /)
      kbo(:, 4,58,14) = (/ &
        & 0.99857e+02_rb,0.18482e+03_rb,0.22827e+03_rb,0.22022e+03_rb,0.16268e+03_rb /)
      kbo(:, 5,58,14) = (/ &
        & 0.11428e+03_rb,0.19233e+03_rb,0.23470e+03_rb,0.22666e+03_rb,0.17444e+03_rb /)
      kbo(:, 1,59,14) = (/ &
        & 0.54418e+02_rb,0.14809e+03_rb,0.18754e+03_rb,0.18261e+03_rb,0.11495e+03_rb /)
      kbo(:, 2,59,14) = (/ &
        & 0.68598e+02_rb,0.16210e+03_rb,0.20441e+03_rb,0.19778e+03_rb,0.13180e+03_rb /)
      kbo(:, 3,59,14) = (/ &
        & 0.83246e+02_rb,0.17398e+03_rb,0.21768e+03_rb,0.20993e+03_rb,0.14733e+03_rb /)
      kbo(:, 4,59,14) = (/ &
        & 0.98055e+02_rb,0.18375e+03_rb,0.22732e+03_rb,0.21927e+03_rb,0.16113e+03_rb /)
      kbo(:, 5,59,14) = (/ &
        & 0.11255e+03_rb,0.19153e+03_rb,0.23406e+03_rb,0.22601e+03_rb,0.17310e+03_rb /)
      kbo(:, 1,13,15) = (/ &
        & 0.40932e+03_rb,0.30758e+03_rb,0.26133e+03_rb,0.32395e+03_rb,0.43211e+03_rb /)
      kbo(:, 2,13,15) = (/ &
        & 0.41123e+03_rb,0.30908e+03_rb,0.25921e+03_rb,0.31805e+03_rb,0.42431e+03_rb /)
      kbo(:, 3,13,15) = (/ &
        & 0.41219e+03_rb,0.30983e+03_rb,0.25685e+03_rb,0.31214e+03_rb,0.41639e+03_rb /)
      kbo(:, 4,13,15) = (/ &
        & 0.41228e+03_rb,0.30991e+03_rb,0.25419e+03_rb,0.30614e+03_rb,0.40836e+03_rb /)
      kbo(:, 5,13,15) = (/ &
        & 0.41130e+03_rb,0.30917e+03_rb,0.25126e+03_rb,0.30055e+03_rb,0.40088e+03_rb /)
      kbo(:, 1,14,15) = (/ &
        & 0.46527e+03_rb,0.34966e+03_rb,0.29823e+03_rb,0.35533e+03_rb,0.47402e+03_rb /)
      kbo(:, 2,14,15) = (/ &
        & 0.46656e+03_rb,0.35065e+03_rb,0.29540e+03_rb,0.34864e+03_rb,0.46510e+03_rb /)
      kbo(:, 3,14,15) = (/ &
        & 0.46685e+03_rb,0.35090e+03_rb,0.29209e+03_rb,0.34189e+03_rb,0.45610e+03_rb /)
      kbo(:, 4,14,15) = (/ &
        & 0.46575e+03_rb,0.35012e+03_rb,0.28845e+03_rb,0.33526e+03_rb,0.44716e+03_rb /)
      kbo(:, 5,14,15) = (/ &
        & 0.46388e+03_rb,0.34873e+03_rb,0.28457e+03_rb,0.32965e+03_rb,0.43950e+03_rb /)
      kbo(:, 1,15,15) = (/ &
        & 0.52376e+03_rb,0.39388e+03_rb,0.33756e+03_rb,0.38636e+03_rb,0.51543e+03_rb /)
      kbo(:, 2,15,15) = (/ &
        & 0.52445e+03_rb,0.39416e+03_rb,0.33368e+03_rb,0.37892e+03_rb,0.50548e+03_rb /)
      kbo(:, 3,15,15) = (/ &
        & 0.52345e+03_rb,0.39343e+03_rb,0.32922e+03_rb,0.37141e+03_rb,0.49535e+03_rb /)
      kbo(:, 4,15,15) = (/ &
        & 0.52134e+03_rb,0.39188e+03_rb,0.32434e+03_rb,0.36468e+03_rb,0.48621e+03_rb /)
      kbo(:, 5,15,15) = (/ &
        & 0.51791e+03_rb,0.38933e+03_rb,0.31931e+03_rb,0.35855e+03_rb,0.47766e+03_rb /)
      kbo(:, 1,16,15) = (/ &
        & 0.58401e+03_rb,0.43962e+03_rb,0.37845e+03_rb,0.41772e+03_rb,0.55681e+03_rb /)
      kbo(:, 2,16,15) = (/ &
        & 0.58308e+03_rb,0.43844e+03_rb,0.37324e+03_rb,0.40927e+03_rb,0.54538e+03_rb /)
      kbo(:, 3,16,15) = (/ &
        & 0.58094e+03_rb,0.43662e+03_rb,0.36730e+03_rb,0.40107e+03_rb,0.53429e+03_rb /)
      kbo(:, 4,16,15) = (/ &
        & 0.57721e+03_rb,0.43385e+03_rb,0.36102e+03_rb,0.39367e+03_rb,0.52421e+03_rb /)
      kbo(:, 5,16,15) = (/ &
        & 0.57218e+03_rb,0.43010e+03_rb,0.35444e+03_rb,0.38654e+03_rb,0.51433e+03_rb /)
      kbo(:, 1,17,15) = (/ &
        & 0.64407e+03_rb,0.48569e+03_rb,0.41994e+03_rb,0.44912e+03_rb,0.59692e+03_rb /)
      kbo(:, 2,17,15) = (/ &
        & 0.64180e+03_rb,0.48304e+03_rb,0.41300e+03_rb,0.43957e+03_rb,0.58409e+03_rb /)
      kbo(:, 3,17,15) = (/ &
        & 0.63774e+03_rb,0.47951e+03_rb,0.40543e+03_rb,0.43048e+03_rb,0.57188e+03_rb /)
      kbo(:, 4,17,15) = (/ &
        & 0.63221e+03_rb,0.47511e+03_rb,0.39752e+03_rb,0.42195e+03_rb,0.56041e+03_rb /)
      kbo(:, 5,17,15) = (/ &
        & 0.62531e+03_rb,0.46997e+03_rb,0.38917e+03_rb,0.41353e+03_rb,0.54901e+03_rb /)
      kbo(:, 1,18,15) = (/ &
        & 0.70302e+03_rb,0.53125e+03_rb,0.46110e+03_rb,0.48015e+03_rb,0.63499e+03_rb /)
      kbo(:, 2,18,15) = (/ &
        & 0.69876e+03_rb,0.52681e+03_rb,0.45218e+03_rb,0.46935e+03_rb,0.62082e+03_rb /)
      kbo(:, 3,18,15) = (/ &
        & 0.69273e+03_rb,0.52130e+03_rb,0.44272e+03_rb,0.45897e+03_rb,0.60731e+03_rb /)
      kbo(:, 4,18,15) = (/ &
        & 0.68529e+03_rb,0.51514e+03_rb,0.43287e+03_rb,0.44907e+03_rb,0.59416e+03_rb /)
      kbo(:, 5,18,15) = (/ &
        & 0.67604e+03_rb,0.50804e+03_rb,0.42255e+03_rb,0.43929e+03_rb,0.58117e+03_rb /)
      kbo(:, 1,19,15) = (/ &
        & 0.75927e+03_rb,0.57536e+03_rb,0.50097e+03_rb,0.51007e+03_rb,0.67027e+03_rb /)
      kbo(:, 2,19,15) = (/ &
        & 0.75290e+03_rb,0.56864e+03_rb,0.48996e+03_rb,0.49791e+03_rb,0.65476e+03_rb /)
      kbo(:, 3,19,15) = (/ &
        & 0.74466e+03_rb,0.56117e+03_rb,0.47832e+03_rb,0.48599e+03_rb,0.63995e+03_rb /)
      kbo(:, 4,19,15) = (/ &
        & 0.73483e+03_rb,0.55286e+03_rb,0.46627e+03_rb,0.47459e+03_rb,0.62519e+03_rb /)
      kbo(:, 5,19,15) = (/ &
        & 0.72353e+03_rb,0.54375e+03_rb,0.45395e+03_rb,0.46331e+03_rb,0.61054e+03_rb /)
      kbo(:, 1,20,15) = (/ &
        & 0.81171e+03_rb,0.61670e+03_rb,0.53858e+03_rb,0.53802e+03_rb,0.70226e+03_rb /)
      kbo(:, 2,20,15) = (/ &
        & 0.80307e+03_rb,0.60788e+03_rb,0.52517e+03_rb,0.52417e+03_rb,0.68550e+03_rb /)
      kbo(:, 3,20,15) = (/ &
        & 0.79257e+03_rb,0.59823e+03_rb,0.51114e+03_rb,0.51080e+03_rb,0.66917e+03_rb /)
      kbo(:, 4,20,15) = (/ &
        & 0.78036e+03_rb,0.58759e+03_rb,0.49687e+03_rb,0.49775e+03_rb,0.65285e+03_rb /)
      kbo(:, 5,20,15) = (/ &
        & 0.76671e+03_rb,0.57649e+03_rb,0.48237e+03_rb,0.48486e+03_rb,0.63647e+03_rb /)
      kbo(:, 1,21,15) = (/ &
        & 0.85929e+03_rb,0.65464e+03_rb,0.57309e+03_rb,0.56354e+03_rb,0.73074e+03_rb /)
      kbo(:, 2,21,15) = (/ &
        & 0.84853e+03_rb,0.64350e+03_rb,0.55721e+03_rb,0.54790e+03_rb,0.71267e+03_rb /)
      kbo(:, 3,21,15) = (/ &
        & 0.83578e+03_rb,0.63172e+03_rb,0.54090e+03_rb,0.53299e+03_rb,0.69483e+03_rb /)
      kbo(:, 4,21,15) = (/ &
        & 0.82132e+03_rb,0.61900e+03_rb,0.52438e+03_rb,0.51831e+03_rb,0.67689e+03_rb /)
      kbo(:, 5,21,15) = (/ &
        & 0.80511e+03_rb,0.60568e+03_rb,0.50774e+03_rb,0.50398e+03_rb,0.65894e+03_rb /)
      kbo(:, 1,22,15) = (/ &
        & 0.90088e+03_rb,0.68794e+03_rb,0.60297e+03_rb,0.58517e+03_rb,0.75429e+03_rb /)
      kbo(:, 2,22,15) = (/ &
        & 0.88795e+03_rb,0.67453e+03_rb,0.58458e+03_rb,0.56794e+03_rb,0.73512e+03_rb /)
      kbo(:, 3,22,15) = (/ &
        & 0.87296e+03_rb,0.66047e+03_rb,0.56607e+03_rb,0.55151e+03_rb,0.71573e+03_rb /)
      kbo(:, 4,22,15) = (/ &
        & 0.85602e+03_rb,0.64569e+03_rb,0.54736e+03_rb,0.53534e+03_rb,0.69633e+03_rb /)
      kbo(:, 5,22,15) = (/ &
        & 0.83763e+03_rb,0.63042e+03_rb,0.52879e+03_rb,0.51959e+03_rb,0.67694e+03_rb /)
      kbo(:, 1,23,15) = (/ &
        & 0.93708e+03_rb,0.71670e+03_rb,0.62868e+03_rb,0.60370e+03_rb,0.77407e+03_rb /)
      kbo(:, 2,23,15) = (/ &
        & 0.92204e+03_rb,0.70111e+03_rb,0.60797e+03_rb,0.58479e+03_rb,0.75367e+03_rb /)
      kbo(:, 3,23,15) = (/ &
        & 0.90485e+03_rb,0.68501e+03_rb,0.58723e+03_rb,0.56696e+03_rb,0.73284e+03_rb /)
      kbo(:, 4,23,15) = (/ &
        & 0.88551e+03_rb,0.66835e+03_rb,0.56656e+03_rb,0.54939e+03_rb,0.71207e+03_rb /)
      kbo(:, 5,23,15) = (/ &
        & 0.86499e+03_rb,0.65129e+03_rb,0.54624e+03_rb,0.53240e+03_rb,0.69156e+03_rb /)
      kbo(:, 1,24,15) = (/ &
        & 0.96776e+03_rb,0.74092e+03_rb,0.64997e+03_rb,0.61898e+03_rb,0.79010e+03_rb /)
      kbo(:, 2,24,15) = (/ &
        & 0.95072e+03_rb,0.72338e+03_rb,0.62717e+03_rb,0.59867e+03_rb,0.76855e+03_rb /)
      kbo(:, 3,24,15) = (/ &
        & 0.93122e+03_rb,0.70539e+03_rb,0.60446e+03_rb,0.57944e+03_rb,0.74652e+03_rb /)
      kbo(:, 4,24,15) = (/ &
        & 0.90993e+03_rb,0.68704e+03_rb,0.58213e+03_rb,0.56067e+03_rb,0.72451e+03_rb /)
      kbo(:, 5,24,15) = (/ &
        & 0.88757e+03_rb,0.66845e+03_rb,0.56030e+03_rb,0.54254e+03_rb,0.70312e+03_rb /)
      kbo(:, 1,25,15) = (/ &
        & 0.99329e+03_rb,0.76073e+03_rb,0.66706e+03_rb,0.63101e+03_rb,0.80276e+03_rb /)
      kbo(:, 2,25,15) = (/ &
        & 0.97397e+03_rb,0.74138e+03_rb,0.64231e+03_rb,0.60949e+03_rb,0.78003e+03_rb /)
      kbo(:, 3,25,15) = (/ &
        & 0.95257e+03_rb,0.72186e+03_rb,0.61798e+03_rb,0.58904e+03_rb,0.75681e+03_rb /)
      kbo(:, 4,25,15) = (/ &
        & 0.92942e+03_rb,0.70193e+03_rb,0.59417e+03_rb,0.56923e+03_rb,0.73392e+03_rb /)
      kbo(:, 5,25,15) = (/ &
        & 0.90532e+03_rb,0.68191e+03_rb,0.57100e+03_rb,0.55015e+03_rb,0.71169e+03_rb /)
      kbo(:, 1,26,15) = (/ &
        & 0.10136e+04_rb,0.77623e+03_rb,0.67983e+03_rb,0.63982e+03_rb,0.81189e+03_rb /)
      kbo(:, 2,26,15) = (/ &
        & 0.99212e+03_rb,0.75536e+03_rb,0.65345e+03_rb,0.61726e+03_rb,0.78804e+03_rb /)
      kbo(:, 3,26,15) = (/ &
        & 0.96906e+03_rb,0.73441e+03_rb,0.62784e+03_rb,0.59585e+03_rb,0.76400e+03_rb /)
      kbo(:, 4,26,15) = (/ &
        & 0.94442e+03_rb,0.71314e+03_rb,0.60278e+03_rb,0.57514e+03_rb,0.74031e+03_rb /)
      kbo(:, 5,26,15) = (/ &
        & 0.91870e+03_rb,0.69199e+03_rb,0.57850e+03_rb,0.55532e+03_rb,0.71748e+03_rb /)
      kbo(:, 1,27,15) = (/ &
        & 0.10295e+04_rb,0.78816e+03_rb,0.68910e+03_rb,0.64601e+03_rb,0.81822e+03_rb /)
      kbo(:, 2,27,15) = (/ &
        & 0.10061e+04_rb,0.76604e+03_rb,0.66144e+03_rb,0.62260e+03_rb,0.79343e+03_rb /)
      kbo(:, 3,27,15) = (/ &
        & 0.98153e+03_rb,0.74373e+03_rb,0.63470e+03_rb,0.60041e+03_rb,0.76875e+03_rb /)
      kbo(:, 4,27,15) = (/ &
        & 0.95548e+03_rb,0.72130e+03_rb,0.60866e+03_rb,0.57907e+03_rb,0.74456e+03_rb /)
      kbo(:, 5,27,15) = (/ &
        & 0.92838e+03_rb,0.69916e+03_rb,0.58347e+03_rb,0.55864e+03_rb,0.72113e+03_rb /)
      kbo(:, 1,28,15) = (/ &
        & 0.10413e+04_rb,0.79681e+03_rb,0.69534e+03_rb,0.64995e+03_rb,0.82220e+03_rb /)
      kbo(:, 2,28,15) = (/ &
        & 0.10164e+04_rb,0.77357e+03_rb,0.66667e+03_rb,0.62595e+03_rb,0.79680e+03_rb /)
      kbo(:, 3,28,15) = (/ &
        & 0.99062e+03_rb,0.75026e+03_rb,0.63903e+03_rb,0.60314e+03_rb,0.77147e+03_rb /)
      kbo(:, 4,28,15) = (/ &
        & 0.96323e+03_rb,0.72694e+03_rb,0.61220e+03_rb,0.58130e+03_rb,0.74687e+03_rb /)
      kbo(:, 5,28,15) = (/ &
        & 0.93489e+03_rb,0.70393e+03_rb,0.58636e+03_rb,0.56041e+03_rb,0.72308e+03_rb /)
      kbo(:, 1,29,15) = (/ &
        & 0.10495e+04_rb,0.80260e+03_rb,0.69878e+03_rb,0.65184e+03_rb,0.82409e+03_rb /)
      kbo(:, 2,29,15) = (/ &
        & 0.10235e+04_rb,0.77841e+03_rb,0.66934e+03_rb,0.62739e+03_rb,0.79811e+03_rb /)
      kbo(:, 3,29,15) = (/ &
        & 0.99642e+03_rb,0.75425e+03_rb,0.64103e+03_rb,0.60412e+03_rb,0.77230e+03_rb /)
      kbo(:, 4,29,15) = (/ &
        & 0.96796e+03_rb,0.73025e+03_rb,0.61357e+03_rb,0.58189e+03_rb,0.74741e+03_rb /)
      kbo(:, 5,29,15) = (/ &
        & 0.93857e+03_rb,0.70654e+03_rb,0.58732e+03_rb,0.56077e+03_rb,0.72335e+03_rb /)
      kbo(:, 1,30,15) = (/ &
        & 0.10549e+04_rb,0.80614e+03_rb,0.70001e+03_rb,0.65202e+03_rb,0.82418e+03_rb /)
      kbo(:, 2,30,15) = (/ &
        & 0.10278e+04_rb,0.78108e+03_rb,0.67004e+03_rb,0.62730e+03_rb,0.79771e+03_rb /)
      kbo(:, 3,30,15) = (/ &
        & 0.99966e+03_rb,0.75628e+03_rb,0.64121e+03_rb,0.60374e+03_rb,0.77172e+03_rb /)
      kbo(:, 4,30,15) = (/ &
        & 0.97021e+03_rb,0.73170e+03_rb,0.61335e+03_rb,0.58130e+03_rb,0.74664e+03_rb /)
      kbo(:, 5,30,15) = (/ &
        & 0.94007e+03_rb,0.70743e+03_rb,0.58676e+03_rb,0.55999e+03_rb,0.72234e+03_rb /)
      kbo(:, 1,31,15) = (/ &
        & 0.10577e+04_rb,0.80741e+03_rb,0.69925e+03_rb,0.65081e+03_rb,0.82281e+03_rb /)
      kbo(:, 2,31,15) = (/ &
        & 0.10297e+04_rb,0.78182e+03_rb,0.66894e+03_rb,0.62588e+03_rb,0.79605e+03_rb /)
      kbo(:, 3,31,15) = (/ &
        & 0.10005e+04_rb,0.75649e+03_rb,0.63976e+03_rb,0.60219e+03_rb,0.76987e+03_rb /)
      kbo(:, 4,31,15) = (/ &
        & 0.97028e+03_rb,0.73149e+03_rb,0.61167e+03_rb,0.57962e+03_rb,0.74468e+03_rb /)
      kbo(:, 5,31,15) = (/ &
        & 0.93951e+03_rb,0.70673e+03_rb,0.58494e+03_rb,0.55830e+03_rb,0.72033e+03_rb /)
      kbo(:, 1,32,15) = (/ &
        & 0.10585e+04_rb,0.80706e+03_rb,0.69697e+03_rb,0.64853e+03_rb,0.82034e+03_rb /)
      kbo(:, 2,32,15) = (/ &
        & 0.10295e+04_rb,0.78094e+03_rb,0.66639e+03_rb,0.62348e+03_rb,0.79329e+03_rb /)
      kbo(:, 3,32,15) = (/ &
        & 0.99955e+03_rb,0.75528e+03_rb,0.63700e+03_rb,0.59969e+03_rb,0.76700e+03_rb /)
      kbo(:, 4,32,15) = (/ &
        & 0.96864e+03_rb,0.72996e+03_rb,0.60885e+03_rb,0.57712e+03_rb,0.74178e+03_rb /)
      kbo(:, 5,32,15) = (/ &
        & 0.93737e+03_rb,0.70488e+03_rb,0.58204e+03_rb,0.55579e+03_rb,0.71738e+03_rb /)
      kbo(:, 1,33,15) = (/ &
        & 0.10575e+04_rb,0.80530e+03_rb,0.69345e+03_rb,0.64529e+03_rb,0.81678e+03_rb /)
      kbo(:, 2,33,15) = (/ &
        & 0.10278e+04_rb,0.77890e+03_rb,0.66278e+03_rb,0.62027e+03_rb,0.78965e+03_rb /)
      kbo(:, 3,33,15) = (/ &
        & 0.99704e+03_rb,0.75290e+03_rb,0.63324e+03_rb,0.59645e+03_rb,0.76337e+03_rb /)
      kbo(:, 4,33,15) = (/ &
        & 0.96555e+03_rb,0.72726e+03_rb,0.60509e+03_rb,0.57394e+03_rb,0.73813e+03_rb /)
      kbo(:, 5,33,15) = (/ &
        & 0.93393e+03_rb,0.70207e+03_rb,0.57839e+03_rb,0.55267e+03_rb,0.71377e+03_rb /)
      kbo(:, 1,34,15) = (/ &
        & 0.10561e+04_rb,0.80340e+03_rb,0.69012e+03_rb,0.64228e+03_rb,0.81344e+03_rb /)
      kbo(:, 2,34,15) = (/ &
        & 0.10257e+04_rb,0.77677e+03_rb,0.65936e+03_rb,0.61726e+03_rb,0.78628e+03_rb /)
      kbo(:, 3,34,15) = (/ &
        & 0.99442e+03_rb,0.75053e+03_rb,0.62978e+03_rb,0.59351e+03_rb,0.76009e+03_rb /)
      kbo(:, 4,34,15) = (/ &
        & 0.96258e+03_rb,0.72472e+03_rb,0.60168e+03_rb,0.57104e+03_rb,0.73478e+03_rb /)
      kbo(:, 5,34,15) = (/ &
        & 0.93053e+03_rb,0.69938e+03_rb,0.57499e+03_rb,0.54984e+03_rb,0.71051e+03_rb /)
      kbo(:, 1,35,15) = (/ &
        & 0.10564e+04_rb,0.80326e+03_rb,0.68907e+03_rb,0.64121e+03_rb,0.81221e+03_rb /)
      kbo(:, 2,35,15) = (/ &
        & 0.10257e+04_rb,0.77646e+03_rb,0.65819e+03_rb,0.61612e+03_rb,0.78498e+03_rb /)
      kbo(:, 3,35,15) = (/ &
        & 0.99396e+03_rb,0.74998e+03_rb,0.62850e+03_rb,0.59231e+03_rb,0.75874e+03_rb /)
      kbo(:, 4,35,15) = (/ &
        & 0.96182e+03_rb,0.72398e+03_rb,0.60037e+03_rb,0.56987e+03_rb,0.73339e+03_rb /)
      kbo(:, 5,35,15) = (/ &
        & 0.92945e+03_rb,0.69852e+03_rb,0.57362e+03_rb,0.54864e+03_rb,0.70910e+03_rb /)
      kbo(:, 1,36,15) = (/ &
        & 0.10593e+04_rb,0.80545e+03_rb,0.69086e+03_rb,0.64252e+03_rb,0.81357e+03_rb /)
      kbo(:, 2,36,15) = (/ &
        & 0.10283e+04_rb,0.77842e+03_rb,0.65978e+03_rb,0.61726e+03_rb,0.78619e+03_rb /)
      kbo(:, 3,36,15) = (/ &
        & 0.99627e+03_rb,0.75174e+03_rb,0.62993e+03_rb,0.59335e+03_rb,0.75988e+03_rb /)
      kbo(:, 4,36,15) = (/ &
        & 0.96386e+03_rb,0.72553e+03_rb,0.60161e+03_rb,0.57080e+03_rb,0.73443e+03_rb /)
      kbo(:, 5,36,15) = (/ &
        & 0.93134e+03_rb,0.69996e+03_rb,0.57477e+03_rb,0.54947e+03_rb,0.71003e+03_rb /)
      kbo(:, 1,37,15) = (/ &
        & 0.10662e+04_rb,0.81134e+03_rb,0.69713e+03_rb,0.64751e+03_rb,0.81885e+03_rb /)
      kbo(:, 2,37,15) = (/ &
        & 0.10351e+04_rb,0.78402e+03_rb,0.66565e+03_rb,0.62189e+03_rb,0.79122e+03_rb /)
      kbo(:, 3,37,15) = (/ &
        & 0.10030e+04_rb,0.75714e+03_rb,0.63542e+03_rb,0.59764e+03_rb,0.76460e+03_rb /)
      kbo(:, 4,37,15) = (/ &
        & 0.97042e+03_rb,0.73071e+03_rb,0.60674e+03_rb,0.57477e+03_rb,0.73895e+03_rb /)
      kbo(:, 5,37,15) = (/ &
        & 0.93771e+03_rb,0.70485e+03_rb,0.57959e+03_rb,0.55322e+03_rb,0.71431e+03_rb /)
      kbo(:, 1,38,15) = (/ &
        & 0.10729e+04_rb,0.81713e+03_rb,0.70342e+03_rb,0.65252e+03_rb,0.82415e+03_rb /)
      kbo(:, 2,38,15) = (/ &
        & 0.10416e+04_rb,0.78952e+03_rb,0.67152e+03_rb,0.62655e+03_rb,0.79626e+03_rb /)
      kbo(:, 3,38,15) = (/ &
        & 0.10096e+04_rb,0.76249e+03_rb,0.64098e+03_rb,0.60198e+03_rb,0.76937e+03_rb /)
      kbo(:, 4,38,15) = (/ &
        & 0.97695e+03_rb,0.73588e+03_rb,0.61197e+03_rb,0.57883e+03_rb,0.74352e+03_rb /)
      kbo(:, 5,38,15) = (/ &
        & 0.94411e+03_rb,0.70978e+03_rb,0.58447e+03_rb,0.55704e+03_rb,0.71866e+03_rb /)
      kbo(:, 1,39,15) = (/ &
        & 0.10791e+04_rb,0.82257e+03_rb,0.70938e+03_rb,0.65729e+03_rb,0.82913e+03_rb /)
      kbo(:, 2,39,15) = (/ &
        & 0.10479e+04_rb,0.79481e+03_rb,0.67717e+03_rb,0.63099e+03_rb,0.80105e+03_rb /)
      kbo(:, 3,39,15) = (/ &
        & 0.10158e+04_rb,0.76754e+03_rb,0.64630e+03_rb,0.60614e+03_rb,0.77394e+03_rb /)
      kbo(:, 4,39,15) = (/ &
        & 0.98307e+03_rb,0.74076e+03_rb,0.61694e+03_rb,0.58271e+03_rb,0.74789e+03_rb /)
      kbo(:, 5,39,15) = (/ &
        & 0.95013e+03_rb,0.71446e+03_rb,0.58917e+03_rb,0.56072e+03_rb,0.72285e+03_rb /)
      kbo(:, 1,40,15) = (/ &
        & 0.10882e+04_rb,0.83071e+03_rb,0.71862e+03_rb,0.66478e+03_rb,0.83689e+03_rb /)
      kbo(:, 2,40,15) = (/ &
        & 0.10571e+04_rb,0.80265e+03_rb,0.68589e+03_rb,0.63804e+03_rb,0.80862e+03_rb /)
      kbo(:, 3,40,15) = (/ &
        & 0.10251e+04_rb,0.77527e+03_rb,0.65462e+03_rb,0.61278e+03_rb,0.78119e+03_rb /)
      kbo(:, 4,40,15) = (/ &
        & 0.99232e+03_rb,0.74818e+03_rb,0.62475e+03_rb,0.58893e+03_rb,0.75487e+03_rb /)
      kbo(:, 5,40,15) = (/ &
        & 0.95932e+03_rb,0.72168e+03_rb,0.59655e+03_rb,0.56653e+03_rb,0.72946e+03_rb /)
      kbo(:, 1,41,15) = (/ &
        & 0.10971e+04_rb,0.83901e+03_rb,0.72823e+03_rb,0.67265e+03_rb,0.84493e+03_rb /)
      kbo(:, 2,41,15) = (/ &
        & 0.10663e+04_rb,0.81071e+03_rb,0.69500e+03_rb,0.64543e+03_rb,0.81648e+03_rb /)
      kbo(:, 3,41,15) = (/ &
        & 0.10345e+04_rb,0.78315e+03_rb,0.66331e+03_rb,0.61971e+03_rb,0.78878e+03_rb /)
      kbo(:, 4,41,15) = (/ &
        & 0.10019e+04_rb,0.75590e+03_rb,0.63295e+03_rb,0.59541e+03_rb,0.76210e+03_rb /)
      kbo(:, 5,41,15) = (/ &
        & 0.96878e+03_rb,0.72921e+03_rb,0.60428e+03_rb,0.57260e+03_rb,0.73637e+03_rb /)
      kbo(:, 1,42,15) = (/ &
        & 0.11058e+04_rb,0.84736e+03_rb,0.73795e+03_rb,0.68059e+03_rb,0.85278e+03_rb /)
      kbo(:, 2,42,15) = (/ &
        & 0.10754e+04_rb,0.81879e+03_rb,0.70412e+03_rb,0.65281e+03_rb,0.82432e+03_rb /)
      kbo(:, 3,42,15) = (/ &
        & 0.10438e+04_rb,0.79093e+03_rb,0.67198e+03_rb,0.62664e+03_rb,0.79629e+03_rb /)
      kbo(:, 4,42,15) = (/ &
        & 0.10112e+04_rb,0.76353e+03_rb,0.64116e+03_rb,0.60189e+03_rb,0.76925e+03_rb /)
      kbo(:, 5,42,15) = (/ &
        & 0.97804e+03_rb,0.73660e+03_rb,0.61202e+03_rb,0.57870e+03_rb,0.74332e+03_rb /)
      kbo(:, 1,43,15) = (/ &
        & 0.11158e+04_rb,0.85717e+03_rb,0.74955e+03_rb,0.69024e+03_rb,0.86217e+03_rb /)
      kbo(:, 2,43,15) = (/ &
        & 0.10861e+04_rb,0.82836e+03_rb,0.71518e+03_rb,0.66177e+03_rb,0.83370e+03_rb /)
      kbo(:, 3,43,15) = (/ &
        & 0.10546e+04_rb,0.80021e+03_rb,0.68243e+03_rb,0.63507e+03_rb,0.80541e+03_rb /)
      kbo(:, 4,43,15) = (/ &
        & 0.10222e+04_rb,0.77261e+03_rb,0.65115e+03_rb,0.60984e+03_rb,0.77795e+03_rb /)
      kbo(:, 5,43,15) = (/ &
        & 0.98916e+03_rb,0.74551e+03_rb,0.62143e+03_rb,0.58614e+03_rb,0.75171e+03_rb /)
      kbo(:, 1,44,15) = (/ &
        & 0.11264e+04_rb,0.86786e+03_rb,0.76217e+03_rb,0.70088e+03_rb,0.87207e+03_rb /)
      kbo(:, 2,44,15) = (/ &
        & 0.10970e+04_rb,0.83855e+03_rb,0.72711e+03_rb,0.67158e+03_rb,0.84381e+03_rb /)
      kbo(:, 3,44,15) = (/ &
        & 0.10660e+04_rb,0.81015e+03_rb,0.69375e+03_rb,0.64429e+03_rb,0.81530e+03_rb /)
      kbo(:, 4,44,15) = (/ &
        & 0.10339e+04_rb,0.78245e+03_rb,0.66207e+03_rb,0.61859e+03_rb,0.78750e+03_rb /)
      kbo(:, 5,44,15) = (/ &
        & 0.10010e+04_rb,0.75506e+03_rb,0.63169e+03_rb,0.59432e+03_rb,0.76086e+03_rb /)
      kbo(:, 1,45,15) = (/ &
        & 0.11364e+04_rb,0.87855e+03_rb,0.77494e+03_rb,0.71181e+03_rb,0.88191e+03_rb /)
      kbo(:, 2,45,15) = (/ &
        & 0.11078e+04_rb,0.84885e+03_rb,0.73921e+03_rb,0.68154e+03_rb,0.85373e+03_rb /)
      kbo(:, 3,45,15) = (/ &
        & 0.10773e+04_rb,0.82021e+03_rb,0.70525e+03_rb,0.65361e+03_rb,0.82516e+03_rb /)
      kbo(:, 4,45,15) = (/ &
        & 0.10454e+04_rb,0.79218e+03_rb,0.67305e+03_rb,0.62740e+03_rb,0.79708e+03_rb /)
      kbo(:, 5,45,15) = (/ &
        & 0.10127e+04_rb,0.76466e+03_rb,0.64208e+03_rb,0.60256e+03_rb,0.76997e+03_rb /)
      kbo(:, 1,46,15) = (/ &
        & 0.11465e+04_rb,0.88995e+03_rb,0.78864e+03_rb,0.72357e+03_rb,0.89218e+03_rb /)
      kbo(:, 2,46,15) = (/ &
        & 0.11189e+04_rb,0.85981e+03_rb,0.75227e+03_rb,0.69242e+03_rb,0.86430e+03_rb /)
      kbo(:, 3,46,15) = (/ &
        & 0.10889e+04_rb,0.83077e+03_rb,0.71764e+03_rb,0.66370e+03_rb,0.83569e+03_rb /)
      kbo(:, 4,46,15) = (/ &
        & 0.10574e+04_rb,0.80253e+03_rb,0.68479e+03_rb,0.63689e+03_rb,0.80730e+03_rb /)
      kbo(:, 5,46,15) = (/ &
        & 0.10249e+04_rb,0.77483e+03_rb,0.65332e+03_rb,0.61153e+03_rb,0.77982e+03_rb /)
      kbo(:, 1,47,15) = (/ &
        & 0.11573e+04_rb,0.90286e+03_rb,0.80421e+03_rb,0.73669e+03_rb,0.90313e+03_rb /)
      kbo(:, 2,47,15) = (/ &
        & 0.11308e+04_rb,0.87219e+03_rb,0.76703e+03_rb,0.70495e+03_rb,0.87583e+03_rb /)
      kbo(:, 3,47,15) = (/ &
        & 0.11017e+04_rb,0.84272e+03_rb,0.73168e+03_rb,0.67524e+03_rb,0.84747e+03_rb /)
      kbo(:, 4,47,15) = (/ &
        & 0.10706e+04_rb,0.81413e+03_rb,0.69808e+03_rb,0.64772e+03_rb,0.81886e+03_rb /)
      kbo(:, 5,47,15) = (/ &
        & 0.10385e+04_rb,0.78623e+03_rb,0.66606e+03_rb,0.62176e+03_rb,0.79095e+03_rb /)
      kbo(:, 1,48,15) = (/ &
        & 0.11675e+04_rb,0.91580e+03_rb,0.81991e+03_rb,0.74998e+03_rb,0.91375e+03_rb /)
      kbo(:, 2,48,15) = (/ &
        & 0.11421e+04_rb,0.88462e+03_rb,0.78203e+03_rb,0.71783e+03_rb,0.88726e+03_rb /)
      kbo(:, 3,48,15) = (/ &
        & 0.11139e+04_rb,0.85471e+03_rb,0.74597e+03_rb,0.68710e+03_rb,0.85917e+03_rb /)
      kbo(:, 4,48,15) = (/ &
        & 0.10836e+04_rb,0.82582e+03_rb,0.71165e+03_rb,0.65875e+03_rb,0.83052e+03_rb /)
      kbo(:, 5,48,15) = (/ &
        & 0.10518e+04_rb,0.79762e+03_rb,0.67897e+03_rb,0.63216e+03_rb,0.80224e+03_rb /)
      kbo(:, 1,49,15) = (/ &
        & 0.11770e+04_rb,0.92887e+03_rb,0.83589e+03_rb,0.76347e+03_rb,0.92401e+03_rb /)
      kbo(:, 2,49,15) = (/ &
        & 0.11531e+04_rb,0.89735e+03_rb,0.79736e+03_rb,0.73085e+03_rb,0.89839e+03_rb /)
      kbo(:, 3,49,15) = (/ &
        & 0.11259e+04_rb,0.86690e+03_rb,0.76049e+03_rb,0.69928e+03_rb,0.87069e+03_rb /)
      kbo(:, 4,49,15) = (/ &
        & 0.10963e+04_rb,0.83754e+03_rb,0.72543e+03_rb,0.67003e+03_rb,0.84223e+03_rb /)
      kbo(:, 5,49,15) = (/ &
        & 0.10650e+04_rb,0.80903e+03_rb,0.69202e+03_rb,0.64278e+03_rb,0.81363e+03_rb /)
      kbo(:, 1,50,15) = (/ &
        & 0.11850e+04_rb,0.94106e+03_rb,0.85114e+03_rb,0.77623e+03_rb,0.93320e+03_rb /)
      kbo(:, 2,50,15) = (/ &
        & 0.11628e+04_rb,0.90935e+03_rb,0.81191e+03_rb,0.74315e+03_rb,0.90846e+03_rb /)
      kbo(:, 3,50,15) = (/ &
        & 0.11368e+04_rb,0.87848e+03_rb,0.77441e+03_rb,0.71118e+03_rb,0.88145e+03_rb /)
      kbo(:, 4,50,15) = (/ &
        & 0.11080e+04_rb,0.84872e+03_rb,0.73862e+03_rb,0.68089e+03_rb,0.85311e+03_rb /)
      kbo(:, 5,50,15) = (/ &
        & 0.10771e+04_rb,0.81987e+03_rb,0.70453e+03_rb,0.65297e+03_rb,0.82446e+03_rb /)
      kbo(:, 1,51,15) = (/ &
        & 0.11923e+04_rb,0.95283e+03_rb,0.86626e+03_rb,0.78865e+03_rb,0.94170e+03_rb /)
      kbo(:, 2,51,15) = (/ &
        & 0.11717e+04_rb,0.92110e+03_rb,0.82616e+03_rb,0.75525e+03_rb,0.91794e+03_rb /)
      kbo(:, 3,51,15) = (/ &
        & 0.11468e+04_rb,0.88978e+03_rb,0.78804e+03_rb,0.72292e+03_rb,0.89168e+03_rb /)
      kbo(:, 4,51,15) = (/ &
        & 0.11189e+04_rb,0.85951e+03_rb,0.75159e+03_rb,0.69174e+03_rb,0.86368e+03_rb /)
      kbo(:, 5,51,15) = (/ &
        & 0.10888e+04_rb,0.83043e+03_rb,0.71698e+03_rb,0.66309e+03_rb,0.83504e+03_rb /)
      kbo(:, 1,52,15) = (/ &
        & 0.11987e+04_rb,0.96448e+03_rb,0.88155e+03_rb,0.80101e+03_rb,0.94976e+03_rb /)
      kbo(:, 2,52,15) = (/ &
        & 0.11799e+04_rb,0.93288e+03_rb,0.84067e+03_rb,0.76742e+03_rb,0.92694e+03_rb /)
      kbo(:, 3,52,15) = (/ &
        & 0.11564e+04_rb,0.90125e+03_rb,0.80189e+03_rb,0.73467e+03_rb,0.90155e+03_rb /)
      kbo(:, 4,52,15) = (/ &
        & 0.11296e+04_rb,0.87065e+03_rb,0.76482e+03_rb,0.70289e+03_rb,0.87401e+03_rb /)
      kbo(:, 5,52,15) = (/ &
        & 0.11001e+04_rb,0.84107e+03_rb,0.72948e+03_rb,0.67337e+03_rb,0.84559e+03_rb /)
      kbo(:, 1,53,15) = (/ &
        & 0.12042e+04_rb,0.97597e+03_rb,0.89729e+03_rb,0.81334e+03_rb,0.95716e+03_rb /)
      kbo(:, 2,53,15) = (/ &
        & 0.11876e+04_rb,0.94467e+03_rb,0.85557e+03_rb,0.77990e+03_rb,0.93591e+03_rb /)
      kbo(:, 3,53,15) = (/ &
        & 0.11657e+04_rb,0.91285e+03_rb,0.81605e+03_rb,0.74663e+03_rb,0.91125e+03_rb /)
      kbo(:, 4,53,15) = (/ &
        & 0.11399e+04_rb,0.88179e+03_rb,0.77830e+03_rb,0.71447e+03_rb,0.88437e+03_rb /)
      kbo(:, 5,53,15) = (/ &
        & 0.11113e+04_rb,0.85189e+03_rb,0.74233e+03_rb,0.68396e+03_rb,0.85611e+03_rb /)
      kbo(:, 1,54,15) = (/ &
        & 0.12085e+04_rb,0.98643e+03_rb,0.91208e+03_rb,0.82462e+03_rb,0.96332e+03_rb /)
      kbo(:, 2,54,15) = (/ &
        & 0.11940e+04_rb,0.95533e+03_rb,0.86934e+03_rb,0.79114e+03_rb,0.94342e+03_rb /)
      kbo(:, 3,54,15) = (/ &
        & 0.11737e+04_rb,0.92362e+03_rb,0.82909e+03_rb,0.75767e+03_rb,0.91977e+03_rb /)
      kbo(:, 4,54,15) = (/ &
        & 0.11490e+04_rb,0.89214e+03_rb,0.79079e+03_rb,0.72522e+03_rb,0.89369e+03_rb /)
      kbo(:, 5,54,15) = (/ &
        & 0.11212e+04_rb,0.86180e+03_rb,0.75420e+03_rb,0.69390e+03_rb,0.86571e+03_rb /)
      kbo(:, 1,55,15) = (/ &
        & 0.12111e+04_rb,0.99597e+03_rb,0.92635e+03_rb,0.83534e+03_rb,0.96856e+03_rb /)
      kbo(:, 2,55,15) = (/ &
        & 0.11993e+04_rb,0.96550e+03_rb,0.88281e+03_rb,0.80196e+03_rb,0.95029e+03_rb /)
      kbo(:, 3,55,15) = (/ &
        & 0.11808e+04_rb,0.93390e+03_rb,0.84185e+03_rb,0.76841e+03_rb,0.92774e+03_rb /)
      kbo(:, 4,55,15) = (/ &
        & 0.11574e+04_rb,0.90226e+03_rb,0.80305e+03_rb,0.73557e+03_rb,0.90236e+03_rb /)
      kbo(:, 5,55,15) = (/ &
        & 0.11305e+04_rb,0.87151e+03_rb,0.76581e+03_rb,0.70377e+03_rb,0.87482e+03_rb /)
      kbo(:, 1,56,15) = (/ &
        & 0.12133e+04_rb,0.10055e+04_rb,0.94091e+03_rb,0.84599e+03_rb,0.97308e+03_rb /)
      kbo(:, 2,56,15) = (/ &
        & 0.12041e+04_rb,0.97555e+03_rb,0.89660e+03_rb,0.81280e+03_rb,0.95684e+03_rb /)
      kbo(:, 3,56,15) = (/ &
        & 0.11875e+04_rb,0.94422e+03_rb,0.85482e+03_rb,0.77921e+03_rb,0.93538e+03_rb /)
      kbo(:, 4,56,15) = (/ &
        & 0.11654e+04_rb,0.91235e+03_rb,0.81536e+03_rb,0.74599e+03_rb,0.91077e+03_rb /)
      kbo(:, 5,56,15) = (/ &
        & 0.11395e+04_rb,0.88125e+03_rb,0.77756e+03_rb,0.71383e+03_rb,0.88381e+03_rb /)
      kbo(:, 1,57,15) = (/ &
        & 0.12144e+04_rb,0.10149e+04_rb,0.95530e+03_rb,0.85635e+03_rb,0.97654e+03_rb /)
      kbo(:, 2,57,15) = (/ &
        & 0.12082e+04_rb,0.98553e+03_rb,0.91069e+03_rb,0.82360e+03_rb,0.96286e+03_rb /)
      kbo(:, 3,57,15) = (/ &
        & 0.11936e+04_rb,0.95436e+03_rb,0.86797e+03_rb,0.78997e+03_rb,0.94263e+03_rb /)
      kbo(:, 4,57,15) = (/ &
        & 0.11730e+04_rb,0.92261e+03_rb,0.82779e+03_rb,0.75653e+03_rb,0.91896e+03_rb /)
      kbo(:, 5,57,15) = (/ &
        & 0.11481e+04_rb,0.89107e+03_rb,0.78949e+03_rb,0.72412e+03_rb,0.89274e+03_rb /)
      kbo(:, 1,58,15) = (/ &
        & 0.12146e+04_rb,0.10236e+04_rb,0.96889e+03_rb,0.86606e+03_rb,0.97897e+03_rb /)
      kbo(:, 2,58,15) = (/ &
        & 0.12111e+04_rb,0.99468e+03_rb,0.92419e+03_rb,0.83372e+03_rb,0.96790e+03_rb /)
      kbo(:, 3,58,15) = (/ &
        & 0.11988e+04_rb,0.96400e+03_rb,0.88060e+03_rb,0.80022e+03_rb,0.94932e+03_rb /)
      kbo(:, 4,58,15) = (/ &
        & 0.11798e+04_rb,0.93230e+03_rb,0.83979e+03_rb,0.76663e+03_rb,0.92647e+03_rb /)
      kbo(:, 5,58,15) = (/ &
        & 0.11561e+04_rb,0.90059e+03_rb,0.80102e+03_rb,0.73388e+03_rb,0.90094e+03_rb /)
      kbo(:, 1,59,15) = (/ &
        & 0.12143e+04_rb,0.10270e+04_rb,0.97431e+03_rb,0.86988e+03_rb,0.97963e+03_rb /)
      kbo(:, 2,59,15) = (/ &
        & 0.12120e+04_rb,0.99826e+03_rb,0.92964e+03_rb,0.83774e+03_rb,0.96974e+03_rb /)
      kbo(:, 3,59,15) = (/ &
        & 0.12008e+04_rb,0.96785e+03_rb,0.88575e+03_rb,0.80426e+03_rb,0.95182e+03_rb /)
      kbo(:, 4,59,15) = (/ &
        & 0.11824e+04_rb,0.93625e+03_rb,0.84468e+03_rb,0.77075e+03_rb,0.92946e+03_rb /)
      kbo(:, 5,59,15) = (/ &
        & 0.11592e+04_rb,0.90444e+03_rb,0.80567e+03_rb,0.73781e+03_rb,0.90417e+03_rb /)
      kbo(:, 1,13,16) = (/ &
        & 0.60786e+03_rb,0.45592e+03_rb,0.38499e+03_rb,0.57704e+03_rb,0.77025e+03_rb /)
      kbo(:, 2,13,16) = (/ &
        & 0.61089e+03_rb,0.45819e+03_rb,0.37289e+03_rb,0.55854e+03_rb,0.74550e+03_rb /)
      kbo(:, 3,13,16) = (/ &
        & 0.61169e+03_rb,0.45879e+03_rb,0.36189e+03_rb,0.54102e+03_rb,0.72223e+03_rb /)
      kbo(:, 4,13,16) = (/ &
        & 0.61079e+03_rb,0.45811e+03_rb,0.35169e+03_rb,0.52404e+03_rb,0.69950e+03_rb /)
      kbo(:, 5,13,16) = (/ &
        & 0.60850e+03_rb,0.45639e+03_rb,0.34199e+03_rb,0.50729e+03_rb,0.67716e+03_rb /)
      kbo(:, 1,14,16) = (/ &
        & 0.71905e+03_rb,0.53931e+03_rb,0.44704e+03_rb,0.66948e+03_rb,0.89363e+03_rb /)
      kbo(:, 2,14,16) = (/ &
        & 0.71950e+03_rb,0.53965e+03_rb,0.43274e+03_rb,0.64598e+03_rb,0.86230e+03_rb /)
      kbo(:, 3,14,16) = (/ &
        & 0.71818e+03_rb,0.53865e+03_rb,0.41969e+03_rb,0.62353e+03_rb,0.83220e+03_rb /)
      kbo(:, 4,14,16) = (/ &
        & 0.71532e+03_rb,0.53651e+03_rb,0.40717e+03_rb,0.60193e+03_rb,0.80343e+03_rb /)
      kbo(:, 5,14,16) = (/ &
        & 0.71008e+03_rb,0.53258e+03_rb,0.39527e+03_rb,0.58080e+03_rb,0.77521e+03_rb /)
      kbo(:, 1,15,16) = (/ &
        & 0.84339e+03_rb,0.63256e+03_rb,0.51534e+03_rb,0.76897e+03_rb,0.10264e+04_rb /)
      kbo(:, 2,15,16) = (/ &
        & 0.84056e+03_rb,0.63042e+03_rb,0.49783e+03_rb,0.73927e+03_rb,0.98670e+03_rb /)
      kbo(:, 3,15,16) = (/ &
        & 0.83549e+03_rb,0.62663e+03_rb,0.48183e+03_rb,0.71056e+03_rb,0.94836e+03_rb /)
      kbo(:, 4,15,16) = (/ &
        & 0.82865e+03_rb,0.62150e+03_rb,0.46649e+03_rb,0.68266e+03_rb,0.91115e+03_rb /)
      kbo(:, 5,15,16) = (/ &
        & 0.81975e+03_rb,0.61483e+03_rb,0.45195e+03_rb,0.65670e+03_rb,0.87636e+03_rb /)
      kbo(:, 1,16,16) = (/ &
        & 0.97910e+03_rb,0.73434e+03_rb,0.58857e+03_rb,0.87272e+03_rb,0.11648e+04_rb /)
      kbo(:, 2,16,16) = (/ &
        & 0.97153e+03_rb,0.72866e+03_rb,0.56721e+03_rb,0.83540e+03_rb,0.11150e+04_rb /)
      kbo(:, 3,16,16) = (/ &
        & 0.96129e+03_rb,0.72098e+03_rb,0.54742e+03_rb,0.79937e+03_rb,0.10667e+04_rb /)
      kbo(:, 4,16,16) = (/ &
        & 0.94880e+03_rb,0.71161e+03_rb,0.52876e+03_rb,0.76497e+03_rb,0.10209e+04_rb /)
      kbo(:, 5,16,16) = (/ &
        & 0.93421e+03_rb,0.70067e+03_rb,0.51082e+03_rb,0.73352e+03_rb,0.97881e+03_rb /)
      kbo(:, 1,17,16) = (/ &
        & 0.11229e+04_rb,0.84221e+03_rb,0.66547e+03_rb,0.97911e+03_rb,0.13066e+04_rb /)
      kbo(:, 2,17,16) = (/ &
        & 0.11091e+04_rb,0.83187e+03_rb,0.63930e+03_rb,0.93226e+03_rb,0.12440e+04_rb /)
      kbo(:, 3,17,16) = (/ &
        & 0.10923e+04_rb,0.81922e+03_rb,0.61539e+03_rb,0.88802e+03_rb,0.11849e+04_rb /)
      kbo(:, 4,17,16) = (/ &
        & 0.10728e+04_rb,0.80459e+03_rb,0.59256e+03_rb,0.84691e+03_rb,0.11301e+04_rb /)
      kbo(:, 5,17,16) = (/ &
        & 0.10515e+04_rb,0.78865e+03_rb,0.57044e+03_rb,0.80936e+03_rb,0.10799e+04_rb /)
      kbo(:, 1,18,16) = (/ &
        & 0.12716e+04_rb,0.95370e+03_rb,0.74447e+03_rb,0.10843e+04_rb,0.14469e+04_rb /)
      kbo(:, 2,18,16) = (/ &
        & 0.12500e+04_rb,0.93749e+03_rb,0.71274e+03_rb,0.10272e+04_rb,0.13706e+04_rb /)
      kbo(:, 3,18,16) = (/ &
        & 0.12251e+04_rb,0.91883e+03_rb,0.68394e+03_rb,0.97432e+03_rb,0.13000e+04_rb /)
      kbo(:, 4,18,16) = (/ &
        & 0.11973e+04_rb,0.89795e+03_rb,0.65609e+03_rb,0.92626e+03_rb,0.12359e+04_rb /)
      kbo(:, 5,18,16) = (/ &
        & 0.11679e+04_rb,0.87596e+03_rb,0.62959e+03_rb,0.88232e+03_rb,0.11772e+04_rb /)
      kbo(:, 1,19,16) = (/ &
        & 0.14213e+04_rb,0.10659e+04_rb,0.82299e+03_rb,0.11854e+04_rb,0.15817e+04_rb /)
      kbo(:, 2,19,16) = (/ &
        & 0.13899e+04_rb,0.10424e+04_rb,0.78536e+03_rb,0.11181e+04_rb,0.14919e+04_rb /)
      kbo(:, 3,19,16) = (/ &
        & 0.13555e+04_rb,0.10166e+04_rb,0.75067e+03_rb,0.10563e+04_rb,0.14093e+04_rb /)
      kbo(:, 4,19,16) = (/ &
        & 0.13187e+04_rb,0.98902e+03_rb,0.71751e+03_rb,0.10008e+04_rb,0.13351e+04_rb /)
      kbo(:, 5,19,16) = (/ &
        & 0.12809e+04_rb,0.96067e+03_rb,0.68653e+03_rb,0.95065e+03_rb,0.12682e+04_rb /)
      kbo(:, 1,20,16) = (/ &
        & 0.15671e+04_rb,0.11753e+04_rb,0.89787e+03_rb,0.12782e+04_rb,0.17052e+04_rb /)
      kbo(:, 2,20,16) = (/ &
        & 0.15245e+04_rb,0.11434e+04_rb,0.85373e+03_rb,0.12013e+04_rb,0.16026e+04_rb /)
      kbo(:, 3,20,16) = (/ &
        & 0.14795e+04_rb,0.11097e+04_rb,0.81323e+03_rb,0.11307e+04_rb,0.15085e+04_rb /)
      kbo(:, 4,20,16) = (/ &
        & 0.14333e+04_rb,0.10750e+04_rb,0.77488e+03_rb,0.10684e+04_rb,0.14252e+04_rb /)
      kbo(:, 5,20,16) = (/ &
        & 0.13861e+04_rb,0.10396e+04_rb,0.73891e+03_rb,0.10119e+04_rb,0.13498e+04_rb /)
      kbo(:, 1,21,16) = (/ &
        & 0.17057e+04_rb,0.12793e+04_rb,0.96840e+03_rb,0.13620e+04_rb,0.18170e+04_rb /)
      kbo(:, 2,21,16) = (/ &
        & 0.16509e+04_rb,0.12382e+04_rb,0.91726e+03_rb,0.12758e+04_rb,0.17018e+04_rb /)
      kbo(:, 3,21,16) = (/ &
        & 0.15947e+04_rb,0.11960e+04_rb,0.87064e+03_rb,0.11975e+04_rb,0.15973e+04_rb /)
      kbo(:, 4,21,16) = (/ &
        & 0.15386e+04_rb,0.11540e+04_rb,0.82723e+03_rb,0.11286e+04_rb,0.15054e+04_rb /)
      kbo(:, 5,21,16) = (/ &
        & 0.14823e+04_rb,0.11117e+04_rb,0.78614e+03_rb,0.10661e+04_rb,0.14220e+04_rb /)
      kbo(:, 1,22,16) = (/ &
        & 0.18293e+04_rb,0.13720e+04_rb,0.10288e+04_rb,0.14300e+04_rb,0.19074e+04_rb /)
      kbo(:, 2,22,16) = (/ &
        & 0.17617e+04_rb,0.13213e+04_rb,0.97120e+03_rb,0.13351e+04_rb,0.17808e+04_rb /)
      kbo(:, 3,22,16) = (/ &
        & 0.16944e+04_rb,0.12708e+04_rb,0.91894e+03_rb,0.12506e+04_rb,0.16681e+04_rb /)
      kbo(:, 4,22,16) = (/ &
        & 0.16287e+04_rb,0.12215e+04_rb,0.87041e+03_rb,0.11757e+04_rb,0.15681e+04_rb /)
      kbo(:, 5,22,16) = (/ &
        & 0.15635e+04_rb,0.11726e+04_rb,0.82499e+03_rb,0.11083e+04_rb,0.14783e+04_rb /)
      kbo(:, 1,23,16) = (/ &
        & 0.19373e+04_rb,0.14530e+04_rb,0.10803e+04_rb,0.14855e+04_rb,0.19814e+04_rb /)
      kbo(:, 2,23,16) = (/ &
        & 0.18577e+04_rb,0.13933e+04_rb,0.10171e+04_rb,0.13836e+04_rb,0.18455e+04_rb /)
      kbo(:, 3,23,16) = (/ &
        & 0.17799e+04_rb,0.13350e+04_rb,0.95955e+03_rb,0.12936e+04_rb,0.17253e+04_rb /)
      kbo(:, 4,23,16) = (/ &
        & 0.17053e+04_rb,0.12790e+04_rb,0.90643e+03_rb,0.12138e+04_rb,0.16189e+04_rb /)
      kbo(:, 5,23,16) = (/ &
        & 0.16318e+04_rb,0.12239e+04_rb,0.85742e+03_rb,0.11422e+04_rb,0.15233e+04_rb /)
      kbo(:, 1,24,16) = (/ &
        & 0.20285e+04_rb,0.15214e+04_rb,0.11229e+04_rb,0.15287e+04_rb,0.20389e+04_rb /)
      kbo(:, 2,24,16) = (/ &
        & 0.19379e+04_rb,0.14534e+04_rb,0.10546e+04_rb,0.14215e+04_rb,0.18958e+04_rb /)
      kbo(:, 3,24,16) = (/ &
        & 0.18509e+04_rb,0.13882e+04_rb,0.99252e+03_rb,0.13268e+04_rb,0.17696e+04_rb /)
      kbo(:, 4,24,16) = (/ &
        & 0.17680e+04_rb,0.13260e+04_rb,0.93542e+03_rb,0.12431e+04_rb,0.16579e+04_rb /)
      kbo(:, 5,24,16) = (/ &
        & 0.16872e+04_rb,0.12654e+04_rb,0.88318e+03_rb,0.11679e+04_rb,0.15576e+04_rb /)
      kbo(:, 1,25,16) = (/ &
        & 0.21022e+04_rb,0.15767e+04_rb,0.11562e+04_rb,0.15596e+04_rb,0.20800e+04_rb /)
      kbo(:, 2,25,16) = (/ &
        & 0.20021e+04_rb,0.15016e+04_rb,0.10837e+04_rb,0.14483e+04_rb,0.19315e+04_rb /)
      kbo(:, 3,25,16) = (/ &
        & 0.19072e+04_rb,0.14304e+04_rb,0.10177e+04_rb,0.13507e+04_rb,0.18013e+04_rb /)
      kbo(:, 4,25,16) = (/ &
        & 0.18172e+04_rb,0.13629e+04_rb,0.95761e+03_rb,0.12639e+04_rb,0.16854e+04_rb /)
      kbo(:, 5,25,16) = (/ &
        & 0.17302e+04_rb,0.12976e+04_rb,0.90262e+03_rb,0.11861e+04_rb,0.15818e+04_rb /)
      kbo(:, 1,26,16) = (/ &
        & 0.21584e+04_rb,0.16188e+04_rb,0.11798e+04_rb,0.15785e+04_rb,0.21051e+04_rb /)
      kbo(:, 2,26,16) = (/ &
        & 0.20504e+04_rb,0.15378e+04_rb,0.11040e+04_rb,0.14649e+04_rb,0.19536e+04_rb /)
      kbo(:, 3,26,16) = (/ &
        & 0.19488e+04_rb,0.14616e+04_rb,0.10351e+04_rb,0.13650e+04_rb,0.18204e+04_rb /)
      kbo(:, 4,26,16) = (/ &
        & 0.18527e+04_rb,0.13896e+04_rb,0.97267e+03_rb,0.12762e+04_rb,0.17019e+04_rb /)
      kbo(:, 5,26,16) = (/ &
        & 0.17609e+04_rb,0.13206e+04_rb,0.91583e+03_rb,0.11969e+04_rb,0.15961e+04_rb /)
      kbo(:, 1,27,16) = (/ &
        & 0.21995e+04_rb,0.16496e+04_rb,0.11960e+04_rb,0.15887e+04_rb,0.21185e+04_rb /)
      kbo(:, 2,27,16) = (/ &
        & 0.20852e+04_rb,0.15639e+04_rb,0.11175e+04_rb,0.14736e+04_rb,0.19651e+04_rb /)
      kbo(:, 3,27,16) = (/ &
        & 0.19784e+04_rb,0.14838e+04_rb,0.10466e+04_rb,0.13723e+04_rb,0.18300e+04_rb /)
      kbo(:, 4,27,16) = (/ &
        & 0.18776e+04_rb,0.14082e+04_rb,0.98231e+03_rb,0.12823e+04_rb,0.17101e+04_rb /)
      kbo(:, 5,27,16) = (/ &
        & 0.17820e+04_rb,0.13365e+04_rb,0.92416e+03_rb,0.12022e+04_rb,0.16031e+04_rb /)
      kbo(:, 1,28,16) = (/ &
        & 0.22274e+04_rb,0.16706e+04_rb,0.12052e+04_rb,0.15913e+04_rb,0.21220e+04_rb /)
      kbo(:, 2,28,16) = (/ &
        & 0.21084e+04_rb,0.15813e+04_rb,0.11251e+04_rb,0.14757e+04_rb,0.19679e+04_rb /)
      kbo(:, 3,28,16) = (/ &
        & 0.19974e+04_rb,0.14981e+04_rb,0.10528e+04_rb,0.13738e+04_rb,0.18320e+04_rb /)
      kbo(:, 4,28,16) = (/ &
        & 0.18930e+04_rb,0.14198e+04_rb,0.98736e+03_rb,0.12834e+04_rb,0.17114e+04_rb /)
      kbo(:, 5,28,16) = (/ &
        & 0.17945e+04_rb,0.13459e+04_rb,0.92826e+03_rb,0.12029e+04_rb,0.16040e+04_rb /)
      kbo(:, 1,29,16) = (/ &
        & 0.22434e+04_rb,0.16825e+04_rb,0.12082e+04_rb,0.15868e+04_rb,0.21160e+04_rb /)
      kbo(:, 2,29,16) = (/ &
        & 0.21209e+04_rb,0.15907e+04_rb,0.11272e+04_rb,0.14716e+04_rb,0.19623e+04_rb /)
      kbo(:, 3,29,16) = (/ &
        & 0.20069e+04_rb,0.15052e+04_rb,0.10541e+04_rb,0.13700e+04_rb,0.18268e+04_rb /)
      kbo(:, 4,29,16) = (/ &
        & 0.18999e+04_rb,0.14249e+04_rb,0.98819e+03_rb,0.12798e+04_rb,0.17065e+04_rb /)
      kbo(:, 5,29,16) = (/ &
        & 0.17996e+04_rb,0.13497e+04_rb,0.92869e+03_rb,0.11992e+04_rb,0.15991e+04_rb /)
      kbo(:, 1,30,16) = (/ &
        & 0.22495e+04_rb,0.16871e+04_rb,0.12065e+04_rb,0.15776e+04_rb,0.21036e+04_rb /)
      kbo(:, 2,30,16) = (/ &
        & 0.21251e+04_rb,0.15938e+04_rb,0.11251e+04_rb,0.14633e+04_rb,0.19512e+04_rb /)
      kbo(:, 3,30,16) = (/ &
        & 0.20091e+04_rb,0.15068e+04_rb,0.10517e+04_rb,0.13621e+04_rb,0.18163e+04_rb /)
      kbo(:, 4,30,16) = (/ &
        & 0.19004e+04_rb,0.14253e+04_rb,0.98584e+03_rb,0.12727e+04_rb,0.16970e+04_rb /)
      kbo(:, 5,30,16) = (/ &
        & 0.17984e+04_rb,0.13488e+04_rb,0.92610e+03_rb,0.11928e+04_rb,0.15905e+04_rb /)
      kbo(:, 1,31,16) = (/ &
        & 0.22474e+04_rb,0.16855e+04_rb,0.12005e+04_rb,0.15635e+04_rb,0.20848e+04_rb /)
      kbo(:, 2,31,16) = (/ &
        & 0.21214e+04_rb,0.15910e+04_rb,0.11192e+04_rb,0.14507e+04_rb,0.19344e+04_rb /)
      kbo(:, 3,31,16) = (/ &
        & 0.20043e+04_rb,0.15033e+04_rb,0.10461e+04_rb,0.13510e+04_rb,0.18015e+04_rb /)
      kbo(:, 4,31,16) = (/ &
        & 0.18948e+04_rb,0.14211e+04_rb,0.98049e+03_rb,0.12625e+04_rb,0.16835e+04_rb /)
      kbo(:, 5,31,16) = (/ &
        & 0.17922e+04_rb,0.13441e+04_rb,0.92102e+03_rb,0.11837e+04_rb,0.15784e+04_rb /)
      kbo(:, 1,32,16) = (/ &
        & 0.22387e+04_rb,0.16790e+04_rb,0.11913e+04_rb,0.15463e+04_rb,0.20617e+04_rb /)
      kbo(:, 2,32,16) = (/ &
        & 0.21117e+04_rb,0.15838e+04_rb,0.11104e+04_rb,0.14353e+04_rb,0.19138e+04_rb /)
      kbo(:, 3,32,16) = (/ &
        & 0.19944e+04_rb,0.14958e+04_rb,0.10381e+04_rb,0.13373e+04_rb,0.17831e+04_rb /)
      kbo(:, 4,32,16) = (/ &
        & 0.18846e+04_rb,0.14135e+04_rb,0.97304e+03_rb,0.12502e+04_rb,0.16670e+04_rb /)
      kbo(:, 5,32,16) = (/ &
        & 0.17818e+04_rb,0.13363e+04_rb,0.91392e+03_rb,0.11724e+04_rb,0.15633e+04_rb /)
      kbo(:, 1,33,16) = (/ &
        & 0.22243e+04_rb,0.16682e+04_rb,0.11793e+04_rb,0.15266e+04_rb,0.20355e+04_rb /)
      kbo(:, 2,33,16) = (/ &
        & 0.20975e+04_rb,0.15731e+04_rb,0.10994e+04_rb,0.14177e+04_rb,0.18903e+04_rb /)
      kbo(:, 3,33,16) = (/ &
        & 0.19805e+04_rb,0.14854e+04_rb,0.10282e+04_rb,0.13217e+04_rb,0.17623e+04_rb /)
      kbo(:, 4,33,16) = (/ &
        & 0.18707e+04_rb,0.14030e+04_rb,0.96384e+03_rb,0.12361e+04_rb,0.16482e+04_rb /)
      kbo(:, 5,33,16) = (/ &
        & 0.17684e+04_rb,0.13263e+04_rb,0.90541e+03_rb,0.11600e+04_rb,0.15467e+04_rb /)
      kbo(:, 1,34,16) = (/ &
        & 0.22110e+04_rb,0.16582e+04_rb,0.11687e+04_rb,0.15094e+04_rb,0.20126e+04_rb /)
      kbo(:, 2,34,16) = (/ &
        & 0.20843e+04_rb,0.15632e+04_rb,0.10894e+04_rb,0.14021e+04_rb,0.18696e+04_rb /)
      kbo(:, 3,34,16) = (/ &
        & 0.19675e+04_rb,0.14756e+04_rb,0.10193e+04_rb,0.13079e+04_rb,0.17439e+04_rb /)
      kbo(:, 4,34,16) = (/ &
        & 0.18579e+04_rb,0.13934e+04_rb,0.95559e+03_rb,0.12238e+04_rb,0.16318e+04_rb /)
      kbo(:, 5,34,16) = (/ &
        & 0.17560e+04_rb,0.13170e+04_rb,0.89776e+03_rb,0.11489e+04_rb,0.15318e+04_rb /)
      kbo(:, 1,35,16) = (/ &
        & 0.22070e+04_rb,0.16552e+04_rb,0.11645e+04_rb,0.15017e+04_rb,0.20024e+04_rb /)
      kbo(:, 2,35,16) = (/ &
        & 0.20800e+04_rb,0.15600e+04_rb,0.10855e+04_rb,0.13953e+04_rb,0.18605e+04_rb /)
      kbo(:, 3,35,16) = (/ &
        & 0.19630e+04_rb,0.14722e+04_rb,0.10157e+04_rb,0.13017e+04_rb,0.17356e+04_rb /)
      kbo(:, 4,35,16) = (/ &
        & 0.18532e+04_rb,0.13899e+04_rb,0.95221e+03_rb,0.12182e+04_rb,0.16243e+04_rb /)
      kbo(:, 5,35,16) = (/ &
        & 0.17512e+04_rb,0.13134e+04_rb,0.89456e+03_rb,0.11438e+04_rb,0.15251e+04_rb /)
      kbo(:, 1,36,16) = (/ &
        & 0.22146e+04_rb,0.16609e+04_rb,0.11681e+04_rb,0.15050e+04_rb,0.20067e+04_rb /)
      kbo(:, 2,36,16) = (/ &
        & 0.20867e+04_rb,0.15651e+04_rb,0.10887e+04_rb,0.13982e+04_rb,0.18642e+04_rb /)
      kbo(:, 3,36,16) = (/ &
        & 0.19689e+04_rb,0.14767e+04_rb,0.10184e+04_rb,0.13041e+04_rb,0.17389e+04_rb /)
      kbo(:, 4,36,16) = (/ &
        & 0.18582e+04_rb,0.13937e+04_rb,0.95458e+03_rb,0.12204e+04_rb,0.16272e+04_rb /)
      kbo(:, 5,36,16) = (/ &
        & 0.17557e+04_rb,0.13168e+04_rb,0.89664e+03_rb,0.11458e+04_rb,0.15277e+04_rb /)
      kbo(:, 1,37,16) = (/ &
        & 0.22411e+04_rb,0.16809e+04_rb,0.11837e+04_rb,0.15248e+04_rb,0.20331e+04_rb /)
      kbo(:, 2,37,16) = (/ &
        & 0.21106e+04_rb,0.15830e+04_rb,0.11025e+04_rb,0.14155e+04_rb,0.18873e+04_rb /)
      kbo(:, 3,37,16) = (/ &
        & 0.19909e+04_rb,0.14932e+04_rb,0.10307e+04_rb,0.13195e+04_rb,0.17594e+04_rb /)
      kbo(:, 4,37,16) = (/ &
        & 0.18783e+04_rb,0.14088e+04_rb,0.96559e+03_rb,0.12340e+04_rb,0.16453e+04_rb /)
      kbo(:, 5,37,16) = (/ &
        & 0.17743e+04_rb,0.13307e+04_rb,0.90666e+03_rb,0.11578e+04_rb,0.15438e+04_rb /)
      kbo(:, 1,38,16) = (/ &
        & 0.22677e+04_rb,0.17008e+04_rb,0.11995e+04_rb,0.15451e+04_rb,0.20601e+04_rb /)
      kbo(:, 2,38,16) = (/ &
        & 0.21347e+04_rb,0.16010e+04_rb,0.11166e+04_rb,0.14334e+04_rb,0.19112e+04_rb /)
      kbo(:, 3,38,16) = (/ &
        & 0.20130e+04_rb,0.15097e+04_rb,0.10432e+04_rb,0.13353e+04_rb,0.17805e+04_rb /)
      kbo(:, 4,38,16) = (/ &
        & 0.18989e+04_rb,0.14242e+04_rb,0.97692e+03_rb,0.12480e+04_rb,0.16640e+04_rb /)
      kbo(:, 5,38,16) = (/ &
        & 0.17929e+04_rb,0.13447e+04_rb,0.91684e+03_rb,0.11704e+04_rb,0.15605e+04_rb /)
      kbo(:, 1,39,16) = (/ &
        & 0.22932e+04_rb,0.17199e+04_rb,0.12149e+04_rb,0.15649e+04_rb,0.20865e+04_rb /)
      kbo(:, 2,39,16) = (/ &
        & 0.21580e+04_rb,0.16185e+04_rb,0.11304e+04_rb,0.14510e+04_rb,0.19346e+04_rb /)
      kbo(:, 3,39,16) = (/ &
        & 0.20342e+04_rb,0.15257e+04_rb,0.10554e+04_rb,0.13508e+04_rb,0.18011e+04_rb /)
      kbo(:, 4,39,16) = (/ &
        & 0.19186e+04_rb,0.14389e+04_rb,0.98789e+03_rb,0.12618e+04_rb,0.16825e+04_rb /)
      kbo(:, 5,39,16) = (/ &
        & 0.18110e+04_rb,0.13582e+04_rb,0.92676e+03_rb,0.11826e+04_rb,0.15768e+04_rb /)
      kbo(:, 1,40,16) = (/ &
        & 0.23330e+04_rb,0.17497e+04_rb,0.12394e+04_rb,0.15975e+04_rb,0.21300e+04_rb /)
      kbo(:, 2,40,16) = (/ &
        & 0.21946e+04_rb,0.16460e+04_rb,0.11524e+04_rb,0.14797e+04_rb,0.19730e+04_rb /)
      kbo(:, 3,40,16) = (/ &
        & 0.20673e+04_rb,0.15505e+04_rb,0.10747e+04_rb,0.13759e+04_rb,0.18346e+04_rb /)
      kbo(:, 4,40,16) = (/ &
        & 0.19494e+04_rb,0.14621e+04_rb,0.10054e+04_rb,0.12842e+04_rb,0.17123e+04_rb /)
      kbo(:, 5,40,16) = (/ &
        & 0.18395e+04_rb,0.13796e+04_rb,0.94264e+03_rb,0.12026e+04_rb,0.16034e+04_rb /)
      kbo(:, 1,41,16) = (/ &
        & 0.23748e+04_rb,0.17811e+04_rb,0.12653e+04_rb,0.16323e+04_rb,0.21765e+04_rb /)
      kbo(:, 2,41,16) = (/ &
        & 0.22335e+04_rb,0.16751e+04_rb,0.11760e+04_rb,0.15106e+04_rb,0.20142e+04_rb /)
      kbo(:, 3,41,16) = (/ &
        & 0.21020e+04_rb,0.15765e+04_rb,0.10953e+04_rb,0.14029e+04_rb,0.18706e+04_rb /)
      kbo(:, 4,41,16) = (/ &
        & 0.19818e+04_rb,0.14863e+04_rb,0.10239e+04_rb,0.13082e+04_rb,0.17443e+04_rb /)
      kbo(:, 5,41,16) = (/ &
        & 0.18696e+04_rb,0.14022e+04_rb,0.95944e+03_rb,0.12238e+04_rb,0.16317e+04_rb /)
      kbo(:, 1,42,16) = (/ &
        & 0.24183e+04_rb,0.18137e+04_rb,0.12922e+04_rb,0.16686e+04_rb,0.22248e+04_rb /)
      kbo(:, 2,42,16) = (/ &
        & 0.22718e+04_rb,0.17039e+04_rb,0.11997e+04_rb,0.15420e+04_rb,0.20561e+04_rb /)
      kbo(:, 3,42,16) = (/ &
        & 0.21373e+04_rb,0.16030e+04_rb,0.11164e+04_rb,0.14305e+04_rb,0.19073e+04_rb /)
      kbo(:, 4,42,16) = (/ &
        & 0.20143e+04_rb,0.15107e+04_rb,0.10428e+04_rb,0.13327e+04_rb,0.17770e+04_rb /)
      kbo(:, 5,42,16) = (/ &
        & 0.18998e+04_rb,0.14249e+04_rb,0.97647e+03_rb,0.12457e+04_rb,0.16610e+04_rb /)
      kbo(:, 1,43,16) = (/ &
        & 0.24703e+04_rb,0.18528e+04_rb,0.13249e+04_rb,0.17136e+04_rb,0.22848e+04_rb /)
      kbo(:, 2,43,16) = (/ &
        & 0.23188e+04_rb,0.17391e+04_rb,0.12289e+04_rb,0.15811e+04_rb,0.21082e+04_rb /)
      kbo(:, 3,43,16) = (/ &
        & 0.21803e+04_rb,0.16352e+04_rb,0.11426e+04_rb,0.14651e+04_rb,0.19535e+04_rb /)
      kbo(:, 4,43,16) = (/ &
        & 0.20539e+04_rb,0.15404e+04_rb,0.10659e+04_rb,0.13633e+04_rb,0.18177e+04_rb /)
      kbo(:, 5,43,16) = (/ &
        & 0.19367e+04_rb,0.14525e+04_rb,0.99744e+03_rb,0.12728e+04_rb,0.16970e+04_rb /)
      kbo(:, 1,44,16) = (/ &
        & 0.25274e+04_rb,0.18956e+04_rb,0.13612e+04_rb,0.17642e+04_rb,0.23522e+04_rb /)
      kbo(:, 2,44,16) = (/ &
        & 0.23706e+04_rb,0.17780e+04_rb,0.12614e+04_rb,0.16248e+04_rb,0.21664e+04_rb /)
      kbo(:, 3,44,16) = (/ &
        & 0.22278e+04_rb,0.16708e+04_rb,0.11717e+04_rb,0.15040e+04_rb,0.20053e+04_rb /)
      kbo(:, 4,44,16) = (/ &
        & 0.20979e+04_rb,0.15734e+04_rb,0.10920e+04_rb,0.13972e+04_rb,0.18630e+04_rb /)
      kbo(:, 5,44,16) = (/ &
        & 0.19771e+04_rb,0.14828e+04_rb,0.10207e+04_rb,0.13030e+04_rb,0.17374e+04_rb /)
      kbo(:, 1,45,16) = (/ &
        & 0.25859e+04_rb,0.19394e+04_rb,0.13990e+04_rb,0.18173e+04_rb,0.24231e+04_rb /)
      kbo(:, 2,45,16) = (/ &
        & 0.24239e+04_rb,0.18179e+04_rb,0.12948e+04_rb,0.16708e+04_rb,0.22278e+04_rb /)
      kbo(:, 3,45,16) = (/ &
        & 0.22761e+04_rb,0.17071e+04_rb,0.12017e+04_rb,0.15441e+04_rb,0.20588e+04_rb /)
      kbo(:, 4,45,16) = (/ &
        & 0.21421e+04_rb,0.16066e+04_rb,0.11186e+04_rb,0.14324e+04_rb,0.19099e+04_rb /)
      kbo(:, 5,45,16) = (/ &
        & 0.20183e+04_rb,0.15137e+04_rb,0.10446e+04_rb,0.13343e+04_rb,0.17791e+04_rb /)
      kbo(:, 1,46,16) = (/ &
        & 0.26504e+04_rb,0.19878e+04_rb,0.14409e+04_rb,0.18767e+04_rb,0.25023e+04_rb /)
      kbo(:, 2,46,16) = (/ &
        & 0.24823e+04_rb,0.18617e+04_rb,0.13316e+04_rb,0.17220e+04_rb,0.22960e+04_rb /)
      kbo(:, 3,46,16) = (/ &
        & 0.23297e+04_rb,0.17473e+04_rb,0.12351e+04_rb,0.15885e+04_rb,0.21180e+04_rb /)
      kbo(:, 4,46,16) = (/ &
        & 0.21905e+04_rb,0.16429e+04_rb,0.11482e+04_rb,0.14718e+04_rb,0.19624e+04_rb /)
      kbo(:, 5,46,16) = (/ &
        & 0.20627e+04_rb,0.15470e+04_rb,0.10708e+04_rb,0.13689e+04_rb,0.18252e+04_rb /)
      kbo(:, 1,47,16) = (/ &
        & 0.27247e+04_rb,0.20435e+04_rb,0.14902e+04_rb,0.19477e+04_rb,0.25969e+04_rb /)
      kbo(:, 2,47,16) = (/ &
        & 0.25493e+04_rb,0.19120e+04_rb,0.13747e+04_rb,0.17822e+04_rb,0.23763e+04_rb /)
      kbo(:, 3,47,16) = (/ &
        & 0.23907e+04_rb,0.17930e+04_rb,0.12733e+04_rb,0.16405e+04_rb,0.21874e+04_rb /)
      kbo(:, 4,47,16) = (/ &
        & 0.22466e+04_rb,0.16850e+04_rb,0.11827e+04_rb,0.15177e+04_rb,0.20235e+04_rb /)
      kbo(:, 5,47,16) = (/ &
        & 0.21136e+04_rb,0.15852e+04_rb,0.11012e+04_rb,0.14091e+04_rb,0.18788e+04_rb /)
      kbo(:, 1,48,16) = (/ &
        & 0.28012e+04_rb,0.21009e+04_rb,0.15426e+04_rb,0.20233e+04_rb,0.26978e+04_rb /)
      kbo(:, 2,48,16) = (/ &
        & 0.26190e+04_rb,0.19642e+04_rb,0.14199e+04_rb,0.18460e+04_rb,0.24614e+04_rb /)
      kbo(:, 3,48,16) = (/ &
        & 0.24541e+04_rb,0.18406e+04_rb,0.13132e+04_rb,0.16956e+04_rb,0.22609e+04_rb /)
      kbo(:, 4,48,16) = (/ &
        & 0.23037e+04_rb,0.17278e+04_rb,0.12184e+04_rb,0.15658e+04_rb,0.20877e+04_rb /)
      kbo(:, 5,48,16) = (/ &
        & 0.21661e+04_rb,0.16246e+04_rb,0.11331e+04_rb,0.14514e+04_rb,0.19352e+04_rb /)
      kbo(:, 1,49,16) = (/ &
        & 0.28818e+04_rb,0.21613e+04_rb,0.15982e+04_rb,0.21038e+04_rb,0.28050e+04_rb /)
      kbo(:, 2,49,16) = (/ &
        & 0.26914e+04_rb,0.20185e+04_rb,0.14674e+04_rb,0.19143e+04_rb,0.25524e+04_rb /)
      kbo(:, 3,49,16) = (/ &
        & 0.25196e+04_rb,0.18897e+04_rb,0.13550e+04_rb,0.17541e+04_rb,0.23389e+04_rb /)
      kbo(:, 4,49,16) = (/ &
        & 0.23632e+04_rb,0.17724e+04_rb,0.12557e+04_rb,0.16163e+04_rb,0.21550e+04_rb /)
      kbo(:, 5,49,16) = (/ &
        & 0.22208e+04_rb,0.16656e+04_rb,0.11666e+04_rb,0.14961e+04_rb,0.19948e+04_rb /)
      kbo(:, 1,50,16) = (/ &
        & 0.29612e+04_rb,0.22209e+04_rb,0.16532e+04_rb,0.21839e+04_rb,0.29118e+04_rb /)
      kbo(:, 2,50,16) = (/ &
        & 0.27615e+04_rb,0.20712e+04_rb,0.15149e+04_rb,0.19827e+04_rb,0.26437e+04_rb /)
      kbo(:, 3,50,16) = (/ &
        & 0.25830e+04_rb,0.19373e+04_rb,0.13961e+04_rb,0.18120e+04_rb,0.24160e+04_rb /)
      kbo(:, 4,50,16) = (/ &
        & 0.24210e+04_rb,0.18157e+04_rb,0.12921e+04_rb,0.16664e+04_rb,0.22218e+04_rb /)
      kbo(:, 5,50,16) = (/ &
        & 0.22734e+04_rb,0.17051e+04_rb,0.11994e+04_rb,0.15401e+04_rb,0.20534e+04_rb /)
      kbo(:, 1,51,16) = (/ &
        & 0.30414e+04_rb,0.22810e+04_rb,0.17098e+04_rb,0.22662e+04_rb,0.30216e+04_rb /)
      kbo(:, 2,51,16) = (/ &
        & 0.28321e+04_rb,0.21241e+04_rb,0.15635e+04_rb,0.20530e+04_rb,0.27374e+04_rb /)
      kbo(:, 3,51,16) = (/ &
        & 0.26473e+04_rb,0.19855e+04_rb,0.14379e+04_rb,0.18712e+04_rb,0.24950e+04_rb /)
      kbo(:, 4,51,16) = (/ &
        & 0.24793e+04_rb,0.18595e+04_rb,0.13289e+04_rb,0.17174e+04_rb,0.22898e+04_rb /)
      kbo(:, 5,51,16) = (/ &
        & 0.23268e+04_rb,0.17451e+04_rb,0.12327e+04_rb,0.15846e+04_rb,0.21128e+04_rb /)
      kbo(:, 1,52,16) = (/ &
        & 0.31248e+04_rb,0.23436e+04_rb,0.17703e+04_rb,0.23534e+04_rb,0.31379e+04_rb /)
      kbo(:, 2,52,16) = (/ &
        & 0.29063e+04_rb,0.21797e+04_rb,0.16147e+04_rb,0.21274e+04_rb,0.28366e+04_rb /)
      kbo(:, 3,52,16) = (/ &
        & 0.27134e+04_rb,0.20351e+04_rb,0.14818e+04_rb,0.19344e+04_rb,0.25793e+04_rb /)
      kbo(:, 4,52,16) = (/ &
        & 0.25389e+04_rb,0.19042e+04_rb,0.13672e+04_rb,0.17712e+04_rb,0.23616e+04_rb /)
      kbo(:, 5,52,16) = (/ &
        & 0.23814e+04_rb,0.17860e+04_rb,0.12669e+04_rb,0.16310e+04_rb,0.21746e+04_rb /)
      kbo(:, 1,53,16) = (/ &
        & 0.32126e+04_rb,0.24095e+04_rb,0.18346e+04_rb,0.24459e+04_rb,0.32612e+04_rb /)
      kbo(:, 2,53,16) = (/ &
        & 0.29838e+04_rb,0.22378e+04_rb,0.16687e+04_rb,0.22061e+04_rb,0.29415e+04_rb /)
      kbo(:, 3,53,16) = (/ &
        & 0.27817e+04_rb,0.20863e+04_rb,0.15283e+04_rb,0.20017e+04_rb,0.26689e+04_rb /)
      kbo(:, 4,53,16) = (/ &
        & 0.26009e+04_rb,0.19507e+04_rb,0.14074e+04_rb,0.18280e+04_rb,0.24374e+04_rb /)
      kbo(:, 5,53,16) = (/ &
        & 0.24377e+04_rb,0.18283e+04_rb,0.13025e+04_rb,0.16802e+04_rb,0.22403e+04_rb /)
      kbo(:, 1,54,16) = (/ &
        & 0.32965e+04_rb,0.24725e+04_rb,0.18972e+04_rb,0.25356e+04_rb,0.33807e+04_rb /)
      kbo(:, 2,54,16) = (/ &
        & 0.30576e+04_rb,0.22932e+04_rb,0.17211e+04_rb,0.22822e+04_rb,0.30430e+04_rb /)
      kbo(:, 3,54,16) = (/ &
        & 0.28468e+04_rb,0.21351e+04_rb,0.15733e+04_rb,0.20668e+04_rb,0.27558e+04_rb /)
      kbo(:, 4,54,16) = (/ &
        & 0.26600e+04_rb,0.19950e+04_rb,0.14459e+04_rb,0.18829e+04_rb,0.25105e+04_rb /)
      kbo(:, 5,54,16) = (/ &
        & 0.24912e+04_rb,0.18684e+04_rb,0.13364e+04_rb,0.17275e+04_rb,0.23034e+04_rb /)
      kbo(:, 1,55,16) = (/ &
        & 0.33833e+04_rb,0.25376e+04_rb,0.19611e+04_rb,0.26264e+04_rb,0.35019e+04_rb /)
      kbo(:, 2,55,16) = (/ &
        & 0.31321e+04_rb,0.23491e+04_rb,0.17750e+04_rb,0.23596e+04_rb,0.31462e+04_rb /)
      kbo(:, 3,55,16) = (/ &
        & 0.29119e+04_rb,0.21839e+04_rb,0.16184e+04_rb,0.21327e+04_rb,0.28435e+04_rb /)
      kbo(:, 4,55,16) = (/ &
        & 0.27182e+04_rb,0.20386e+04_rb,0.14848e+04_rb,0.19390e+04_rb,0.25853e+04_rb /)
      kbo(:, 5,55,16) = (/ &
        & 0.25438e+04_rb,0.19078e+04_rb,0.13702e+04_rb,0.17749e+04_rb,0.23665e+04_rb /)
      kbo(:, 1,56,16) = (/ &
        & 0.34732e+04_rb,0.26050e+04_rb,0.20286e+04_rb,0.27221e+04_rb,0.36294e+04_rb /)
      kbo(:, 2,56,16) = (/ &
        & 0.32087e+04_rb,0.24066e+04_rb,0.18312e+04_rb,0.24406e+04_rb,0.32540e+04_rb /)
      kbo(:, 3,56,16) = (/ &
        & 0.29798e+04_rb,0.22348e+04_rb,0.16657e+04_rb,0.22014e+04_rb,0.29353e+04_rb /)
      kbo(:, 4,56,16) = (/ &
        & 0.27779e+04_rb,0.20834e+04_rb,0.15255e+04_rb,0.19978e+04_rb,0.26637e+04_rb /)
      kbo(:, 5,56,16) = (/ &
        & 0.25980e+04_rb,0.19485e+04_rb,0.14055e+04_rb,0.18247e+04_rb,0.24330e+04_rb /)
      kbo(:, 1,57,16) = (/ &
        & 0.35673e+04_rb,0.26755e+04_rb,0.21006e+04_rb,0.28229e+04_rb,0.37639e+04_rb /)
      kbo(:, 2,57,16) = (/ &
        & 0.32883e+04_rb,0.24663e+04_rb,0.18907e+04_rb,0.25257e+04_rb,0.33676e+04_rb /)
      kbo(:, 3,57,16) = (/ &
        & 0.30500e+04_rb,0.22875e+04_rb,0.17155e+04_rb,0.22739e+04_rb,0.30319e+04_rb /)
      kbo(:, 4,57,16) = (/ &
        & 0.28394e+04_rb,0.21295e+04_rb,0.15682e+04_rb,0.20597e+04_rb,0.27463e+04_rb /)
      kbo(:, 5,57,16) = (/ &
        & 0.26543e+04_rb,0.19907e+04_rb,0.14421e+04_rb,0.18768e+04_rb,0.25025e+04_rb /)
      kbo(:, 1,58,16) = (/ &
        & 0.36607e+04_rb,0.27457e+04_rb,0.21738e+04_rb,0.29233e+04_rb,0.38978e+04_rb /)
      kbo(:, 2,58,16) = (/ &
        & 0.33686e+04_rb,0.25266e+04_rb,0.19501e+04_rb,0.26108e+04_rb,0.34810e+04_rb /)
      kbo(:, 3,58,16) = (/ &
        & 0.31191e+04_rb,0.23393e+04_rb,0.17656e+04_rb,0.23462e+04_rb,0.31282e+04_rb /)
      kbo(:, 4,58,16) = (/ &
        & 0.29009e+04_rb,0.21757e+04_rb,0.16107e+04_rb,0.21213e+04_rb,0.28285e+04_rb /)
      kbo(:, 5,58,16) = (/ &
        & 0.27090e+04_rb,0.20318e+04_rb,0.14784e+04_rb,0.19292e+04_rb,0.25722e+04_rb /)
      kbo(:, 1,59,16) = (/ &
        & 0.36992e+04_rb,0.27745e+04_rb,0.22044e+04_rb,0.29651e+04_rb,0.39534e+04_rb /)
      kbo(:, 2,59,16) = (/ &
        & 0.34022e+04_rb,0.25517e+04_rb,0.19749e+04_rb,0.26461e+04_rb,0.35281e+04_rb /)
      kbo(:, 3,59,16) = (/ &
        & 0.31476e+04_rb,0.23607e+04_rb,0.17864e+04_rb,0.23762e+04_rb,0.31682e+04_rb /)
      kbo(:, 4,59,16) = (/ &
        & 0.29259e+04_rb,0.21945e+04_rb,0.16281e+04_rb,0.21469e+04_rb,0.28626e+04_rb /)
      kbo(:, 5,59,16) = (/ &
        & 0.27315e+04_rb,0.20486e+04_rb,0.14936e+04_rb,0.19510e+04_rb,0.26013e+04_rb /)

!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.553258e-03_rb, 0.555486e-03_rb, 0.601339e-03_rb, 0.708280e-03_rb /)
      forrefo(:, 2) = (/ 0.158558e-02_rb, 0.162957e-02_rb, 0.204991e-02_rb, 0.475881e-02_rb /)
      forrefo(:, 3) = (/ 0.772542e-02_rb, 0.784562e-02_rb, 0.111979e-01_rb, 0.229016e-01_rb /)
      forrefo(:, 4) = (/ 0.255097e-01_rb, 0.256272e-01_rb, 0.270691e-01_rb, 0.259505e-01_rb /)
      forrefo(:, 5) = (/ 0.323263e-01_rb, 0.324495e-01_rb, 0.305535e-01_rb, 0.263993e-01_rb /)
      forrefo(:, 6) = (/ 0.346920e-01_rb, 0.348255e-01_rb, 0.323586e-01_rb, 0.276357e-01_rb /)
      forrefo(:, 7) = (/ 0.366509e-01_rb, 0.366412e-01_rb, 0.344434e-01_rb, 0.319223e-01_rb /)
      forrefo(:, 8) = (/ 0.378451e-01_rb, 0.375341e-01_rb, 0.374369e-01_rb, 0.320334e-01_rb /)
      forrefo(:, 9) = (/ 0.407348e-01_rb, 0.396203e-01_rb, 0.393988e-01_rb, 0.318343e-01_rb /)
      forrefo(:,10) = (/ 0.433035e-01_rb, 0.426488e-01_rb, 0.408085e-01_rb, 0.332749e-01_rb /)
      forrefo(:,11) = (/ 0.428254e-01_rb, 0.441151e-01_rb, 0.408887e-01_rb, 0.327077e-01_rb /)
      forrefo(:,12) = (/ 0.443226e-01_rb, 0.446690e-01_rb, 0.404676e-01_rb, 0.350492e-01_rb /)
      forrefo(:,13) = (/ 0.466103e-01_rb, 0.460809e-01_rb, 0.401286e-01_rb, 0.370427e-01_rb /)
      forrefo(:,14) = (/ 0.483928e-01_rb, 0.477284e-01_rb, 0.380684e-01_rb, 0.387940e-01_rb /)
      forrefo(:,15) = (/ 0.506987e-01_rb, 0.490016e-01_rb, 0.467069e-01_rb, 0.368998e-01_rb /)
      forrefo(:,16) = (/ 0.510836e-01_rb, 0.522771e-01_rb, 0.500130e-01_rb, 0.483406e-01_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
        &  0.160537e-01_rb, 0.149038e-01_rb, 0.138363e-01_rb, 0.128452e-01_rb, 0.119251e-01_rb, &
        &  0.110709e-01_rb, 0.102779e-01_rb, 0.954175e-02_rb, 0.885829e-02_rb, 0.822379e-02_rb /)
      selfrefo(:, 2) = (/ &
        &  0.365753e-01_rb, 0.342267e-01_rb, 0.320288e-01_rb, 0.299720e-01_rb, 0.280474e-01_rb, &
        &  0.262463e-01_rb, 0.245609e-01_rb, 0.229837e-01_rb, 0.215078e-01_rb, 0.201267e-01_rb /)
      selfrefo(:, 3) = (/ &
        &  0.127419e+00_rb, 0.118553e+00_rb, 0.110304e+00_rb, 0.102629e+00_rb, 0.954883e-01_rb, &
        &  0.888442e-01_rb, 0.826624e-01_rb, 0.769107e-01_rb, 0.715593e-01_rb, 0.665802e-01_rb /)
      selfrefo(:, 4) = (/ &
        &  0.378687e+00_rb, 0.348961e+00_rb, 0.321568e+00_rb, 0.296325e+00_rb, 0.273064e+00_rb, &
        &  0.251629e+00_rb, 0.231876e+00_rb, 0.213674e+00_rb, 0.196901e+00_rb, 0.181444e+00_rb /)
      selfrefo(:, 5) = (/ &
        &  0.472822e+00_rb, 0.435018e+00_rb, 0.400236e+00_rb, 0.368236e+00_rb, 0.338794e+00_rb, &
        &  0.311706e+00_rb, 0.286783e+00_rb, 0.263854e+00_rb, 0.242757e+00_rb, 0.223348e+00_rb /)
      selfrefo(:, 6) = (/ &
        &  0.505620e+00_rb, 0.465050e+00_rb, 0.427736e+00_rb, 0.393416e+00_rb, 0.361849e+00_rb, &
        &  0.332815e+00_rb, 0.306111e+00_rb, 0.281550e+00_rb, 0.258959e+00_rb, 0.238181e+00_rb /)
      selfrefo(:, 7) = (/ &
        &  0.530488e+00_rb, 0.487993e+00_rb, 0.448902e+00_rb, 0.412943e+00_rb, 0.379864e+00_rb, &
        &  0.349434e+00_rb, 0.321443e+00_rb, 0.295694e+00_rb, 0.272007e+00_rb, 0.250218e+00_rb /)
      selfrefo(:, 8) = (/ &
        &  0.540222e+00_rb, 0.497746e+00_rb, 0.458610e+00_rb, 0.422551e+00_rb, 0.389327e+00_rb, &
        &  0.358716e+00_rb, 0.330511e+00_rb, 0.304524e+00_rb, 0.280580e+00_rb, 0.258519e+00_rb /)
      selfrefo(:, 9) = (/ &
        &  0.565727e+00_rb, 0.522899e+00_rb, 0.483313e+00_rb, 0.446724e+00_rb, 0.412905e+00_rb, &
        &  0.381646e+00_rb, 0.352753e+00_rb, 0.326048e+00_rb, 0.301365e+00_rb, 0.278550e+00_rb /)
      selfrefo(:,10) = (/ &
        &  0.610122e+00_rb, 0.562337e+00_rb, 0.518295e+00_rb, 0.477702e+00_rb, 0.440289e+00_rb, &
        &  0.405806e+00_rb, 0.374023e+00_rb, 0.344730e+00_rb, 0.317730e+00_rb, 0.292846e+00_rb /)
      selfrefo(:,11) = (/ &
        &  0.645176e+00_rb, 0.588957e+00_rb, 0.537636e+00_rb, 0.490788e+00_rb, 0.448022e+00_rb, &
        &  0.408982e+00_rb, 0.373344e+00_rb, 0.340812e+00_rb, 0.311114e+00_rb, 0.284004e+00_rb /)
      selfrefo(:,12) = (/ &
        &  0.651737e+00_rb, 0.596547e+00_rb, 0.546031e+00_rb, 0.499792e+00_rb, 0.457469e+00_rb, &
        &  0.418730e+00_rb, 0.383272e+00_rb, 0.350816e+00_rb, 0.321108e+00_rb, 0.293916e+00_rb /)
      selfrefo(:,13) = (/ &
        &  0.661086e+00_rb, 0.607954e+00_rb, 0.559093e+00_rb, 0.514159e+00_rb, 0.472836e+00_rb, &
        &  0.434834e+00_rb, 0.399886e+00_rb, 0.367747e+00_rb, 0.338191e+00_rb, 0.311011e+00_rb /)
      selfrefo(:,14) = (/ &
        &  0.692554e+00_rb, 0.635574e+00_rb, 0.583282e+00_rb, 0.535293e+00_rb, 0.491251e+00_rb, &
        &  0.450834e+00_rb, 0.413741e+00_rb, 0.379701e+00_rb, 0.348461e+00_rb, 0.319791e+00_rb /)
      selfrefo(:,15) = (/ &
        &  0.714646e+00_rb, 0.657179e+00_rb, 0.604334e+00_rb, 0.555737e+00_rb, 0.511049e+00_rb, &
        &  0.469954e+00_rb, 0.432164e+00_rb, 0.397412e+00_rb, 0.365455e+00_rb, 0.336068e+00_rb /)
      selfrefo(:,16) = (/ &
        &  0.782126e+00_rb, 0.710682e+00_rb, 0.645764e+00_rb, 0.586776e+00_rb, 0.533177e+00_rb, &
        &  0.484473e+00_rb, 0.440219e+00_rb, 0.400007e+00_rb, 0.363468e+00_rb, 0.330266e+00_rb /)
     
      end subroutine sw_kgb17

! **************************************************************************
      subroutine sw_kgb18
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg18, only : kao, kbo, selfrefo, forrefo, sfluxrefo, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:,1) = (/ &
        & 3.65840_rb    , 3.54375_rb    , 3.34481_rb    , 3.10534_rb    , &
        & 2.79879_rb    , 2.42841_rb    , 1.98748_rb    , 1.49377_rb    , &
        & 1.00196_rb    , 0.108342_rb   , 8.95099e-02_rb, 7.05199e-02_rb, &
        & 5.16432e-02_rb, 3.27635e-02_rb, 1.25133e-02_rb, 1.73001e-03_rb /)  
      sfluxrefo(:,2) = (/ &
        & 3.86372_rb    , 3.48521_rb    , 3.30790_rb    , 3.08103_rb    , &
        & 2.77552_rb    , 2.40722_rb    , 1.97307_rb    , 1.48023_rb    , &
        & 0.993055_rb   , 0.107691_rb   , 8.84430e-02_rb, 6.99354e-02_rb, &
        & 5.07881e-02_rb, 3.24121e-02_rb, 1.19442e-02_rb, 1.57612e-03_rb /)
      sfluxrefo(:,3) = (/ &
        & 3.90370_rb    , 3.50657_rb    , 3.30629_rb    , 3.06046_rb    , &
        & 2.76982_rb    , 2.39907_rb    , 1.96358_rb    , 1.47458_rb    , &
        & 0.988475_rb   , 0.106698_rb   , 8.75242e-02_rb, 6.85898e-02_rb, &
        & 5.04798e-02_rb, 3.13718e-02_rb, 1.09533e-02_rb, 1.57612e-03_rb /)
      sfluxrefo(:,4) = (/ &
        & 3.93165_rb    , 3.52058_rb    , 3.31346_rb    , 3.04944_rb    , &
        & 2.76074_rb    , 2.39433_rb    , 1.95556_rb    , 1.46712_rb    , &
        & 0.984056_rb   , 0.105885_rb   , 8.73062e-02_rb, 6.84054e-02_rb, &
        & 4.87443e-02_rb, 2.99295e-02_rb, 1.09533e-02_rb, 1.57612e-03_rb /)
      sfluxrefo(:,5) = (/ &
        & 3.94082_rb    , 3.55221_rb    , 3.31863_rb    , 3.04730_rb    , &
        & 2.74918_rb    , 2.38328_rb    , 1.95212_rb    , 1.45889_rb    , &
        & 0.978888_rb   , 0.105102_rb   , 8.65732e-02_rb, 6.74563e-02_rb, &
        & 4.76592e-02_rb, 2.91017e-02_rb, 1.09533e-02_rb, 1.57612e-03_rb /)
      sfluxrefo(:,6) = (/ &
        & 3.94198_rb    , 3.58743_rb    , 3.32106_rb    , 3.05866_rb    , &
        & 2.74115_rb    , 2.36939_rb    , 1.94305_rb    , 1.45180_rb    , &
        & 0.971784_rb   , 1.04045e-01_rb, 8.53731e-02_rb, 6.60654e-02_rb, &
        & 4.63228e-02_rb, 2.91016e-02_rb, 1.09552e-02_rb, 1.57612e-03_rb /)
      sfluxrefo(:,7) = (/ &
        & 3.93596_rb    , 3.63366_rb    , 3.33144_rb    , 3.06252_rb    , &
        & 2.74054_rb    , 2.35492_rb    , 1.92769_rb    , 1.44300_rb    , &
        & 0.961809_rb   , 1.02867e-01_rb, 8.34164e-02_rb, 6.41005e-02_rb, &
        & 4.61826e-02_rb, 2.91006e-02_rb, 1.09553e-02_rb, 1.57612e-03_rb /)
      sfluxrefo(:,8) = (/ &
        & 3.92520_rb    , 3.69078_rb    , 3.35656_rb    , 3.07055_rb    , &
        & 2.73862_rb    , 2.34430_rb    , 1.90187_rb    , 1.42242_rb    , &
        & 0.946676_rb   , 9.96302e-02_rb, 8.14421e-02_rb, 6.38622e-02_rb, &
        & 4.61794e-02_rb, 2.91017e-02_rb, 1.09553e-02_rb, 1.57612e-03_rb /)
      sfluxrefo(:,9) = (/ &
        & 3.80721_rb    , 3.74437_rb    , 3.50205_rb    , 3.18009_rb    , &
        & 2.75757_rb    , 2.29188_rb    , 1.84382_rb    , 1.35694_rb    , &
        & 0.914040_rb   , 9.86811e-02_rb, 8.14321e-02_rb, 6.38541e-02_rb, &
        & 4.61795e-02_rb, 2.90960e-02_rb, 1.09613e-02_rb, 1.57612e-03_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:,1) = (/ &
        & 3.49983E+00_rb, 3.58280E+00_rb, 3.40493E+00_rb, 3.14521E+00_rb,&
        & 2.81628E+00_rb, 2.43497E+00_rb, 1.99130E+00_rb, 1.49566E+00_rb,&
        & 1.00403E+00_rb, 1.08486E-01_rb, 8.95635E-02_rb, 7.05600E-02_rb,&
        & 5.16782E-02_rb, 3.28252E-02_rb, 1.24807E-02_rb, 1.73746E-03_rb/)
      irradnceo(:,2) = (/ &
        & 3.67139E+00_rb, 3.57695E+00_rb, 3.35985E+00_rb, 3.11147E+00_rb,&
        & 2.78989E+00_rb, 2.41440E+00_rb, 1.97736E+00_rb, 1.48227E+00_rb,&
        & 9.95337E-01_rb, 1.07743E-01_rb, 8.85929E-02_rb, 7.00644E-02_rb,&
        & 5.10709E-02_rb, 3.23610E-02_rb, 1.20010E-02_rb, 1.59976E-03_rb/)
      irradnceo(:,3) = (/ &
        & 3.74773E+00_rb, 3.57605E+00_rb, 3.34515E+00_rb, 3.09262E+00_rb,&
        & 2.78346E+00_rb, 2.40556E+00_rb, 1.96727E+00_rb, 1.47626E+00_rb,&
        & 9.90295E-01_rb, 1.06566E-01_rb, 8.75099E-02_rb, 6.89888E-02_rb,&
        & 5.06424E-02_rb, 3.15191E-02_rb, 1.11419E-02_rb, 1.58850E-03_rb/)
      irradnceo(:,4) = (/ &
        & 3.80817E+00_rb, 3.58074E+00_rb, 3.33392E+00_rb, 3.08151E+00_rb,&
        & 2.77159E+00_rb, 2.39964E+00_rb, 1.95902E+00_rb, 1.46917E+00_rb,&
        & 9.84942E-01_rb, 1.05891E-01_rb, 8.74113E-02_rb, 6.84411E-02_rb,&
        & 4.89060E-02_rb, 3.02555E-02_rb, 1.11419E-02_rb, 1.58850E-03_rb/)
      irradnceo(:,5) = (/ &
        & 3.86072E+00_rb, 3.58154E+00_rb, 3.34236E+00_rb, 3.06646E+00_rb,&
        & 2.75901E+00_rb, 2.38895E+00_rb, 1.95431E+00_rb, 1.46022E+00_rb,&
        & 9.79776E-01_rb, 1.05107E-01_rb, 8.65925E-02_rb, 6.74510E-02_rb,&
        & 4.78397E-02_rb, 2.92793E-02_rb, 1.11431E-02_rb, 1.58850E-03_rb/)
      irradnceo(:,6) = (/ &
        & 3.89837E+00_rb, 3.59943E+00_rb, 3.34159E+00_rb, 3.06629E+00_rb,&
        & 2.74919E+00_rb, 2.37454E+00_rb, 1.94349E+00_rb, 1.45339E+00_rb,&
        & 9.72059E-01_rb, 1.04116E-01_rb, 8.53926E-02_rb, 6.60499E-02_rb,&
        & 4.64256E-02_rb, 2.92741E-02_rb, 1.11431E-02_rb, 1.58850E-03_rb/)
      irradnceo(:,7) = (/ &
        & 3.92115E+00_rb, 3.64308E+00_rb, 3.34237E+00_rb, 3.06336E+00_rb,&
        & 2.74409E+00_rb, 2.35823E+00_rb, 1.92577E+00_rb, 1.44376E+00_rb,&
        & 9.62108E-01_rb, 1.02788E-01_rb, 8.34040E-02_rb, 6.39846E-02_rb,&
        & 4.62355E-02_rb, 2.92804E-02_rb, 1.11419E-02_rb, 1.58850E-03_rb/)
      irradnceo(:,8) = (/ &
        & 3.91652E+00_rb, 3.71781E+00_rb, 3.37231E+00_rb, 3.06423E+00_rb,&
        & 2.72925E+00_rb, 2.34232E+00_rb, 1.90068E+00_rb, 1.42111E+00_rb,&
        & 9.45487E-01_rb, 9.93815E-02_rb, 8.13343E-02_rb, 6.36723E-02_rb,&
        & 4.62312E-02_rb, 2.92833E-02_rb, 1.11431E-02_rb, 1.58850E-03_rb/)
      irradnceo(:,9) = (/ &
        & 3.82319E+00_rb, 3.69907E+00_rb, 3.53272E+00_rb, 3.21449E+00_rb,&
        & 2.75635E+00_rb, 2.28586E+00_rb, 1.83424E+00_rb, 1.35268E+00_rb,&
        & 9.12101E-01_rb, 9.84089E-02_rb, 8.13147E-02_rb, 6.36693E-02_rb,&
        & 4.62324E-02_rb, 2.92823E-02_rb, 1.11431E-02_rb, 1.58850E-03_rb/)
! Solar variability components: facular brightening
      facbrghto(:,1) = (/ &
        & 7.59826E-04_rb, 6.57277E-04_rb, 6.36294E-04_rb, 6.14163E-04_rb,&
        & 5.56168E-04_rb, 4.82172E-04_rb, 3.91839E-04_rb, 3.04013E-04_rb,&
        & 2.13822E-04_rb, 2.41248E-05_rb, 2.04145E-05_rb, 1.64273E-05_rb,&
        & 1.22533E-05_rb, 7.91198E-06_rb, 2.81886E-06_rb, 5.20181E-07_rb/)
      facbrghto(:,2) = (/ &
        & 6.52548E-04_rb, 6.53792E-04_rb, 6.68857E-04_rb, 6.37964E-04_rb,&
        & 5.72176E-04_rb, 4.96735E-04_rb, 4.00488E-04_rb, 3.12891E-04_rb,&
        & 2.18543E-04_rb, 2.44558E-05_rb, 2.10219E-05_rb, 1.66637E-05_rb,&
        & 1.23062E-05_rb, 8.01557E-06_rb, 3.14530E-06_rb, 4.44266E-07_rb/)
      facbrghto(:,3) = (/ &
        & 6.01326E-04_rb, 6.53121E-04_rb, 6.84500E-04_rb, 6.45628E-04_rb,&
        & 5.77414E-04_rb, 5.02903E-04_rb, 4.07951E-04_rb, 3.16421E-04_rb,&
        & 2.22374E-04_rb, 2.51705E-05_rb, 2.14132E-05_rb, 1.72006E-05_rb,&
        & 1.27115E-05_rb, 8.12903E-06_rb, 3.34419E-06_rb, 4.38923E-07_rb/)
      facbrghto(:,4) = (/ &
        & 5.58104E-04_rb, 6.53525E-04_rb, 6.93244E-04_rb, 6.51661E-04_rb,&
        & 5.84879E-04_rb, 5.06614E-04_rb, 4.15326E-04_rb, 3.20098E-04_rb,&
        & 2.26009E-04_rb, 2.58812E-05_rb, 2.17134E-05_rb, 1.73748E-05_rb,&
        & 1.31171E-05_rb, 8.71694E-06_rb, 3.34419E-06_rb, 4.38923E-07_rb/)
      facbrghto(:,5) = (/ &
        & 5.19527E-04_rb, 6.53480E-04_rb, 6.91582E-04_rb, 6.62710E-04_rb,&
        & 5.93482E-04_rb, 5.10767E-04_rb, 4.20898E-04_rb, 3.25218E-04_rb,&
        & 2.29663E-04_rb, 2.63117E-05_rb, 2.21641E-05_rb, 1.79253E-05_rb,&
        & 1.36299E-05_rb, 8.90304E-06_rb, 3.34444E-06_rb, 4.38923E-07_rb/)
      facbrghto(:,6) = (/ &
        & 4.88072E-04_rb, 6.44463E-04_rb, 6.95105E-04_rb, 6.63083E-04_rb,&
        & 6.02643E-04_rb, 5.18445E-04_rb, 4.28359E-04_rb, 3.30578E-04_rb,&
        & 2.34417E-04_rb, 2.70236E-05_rb, 2.26631E-05_rb, 1.82372E-05_rb,&
        & 1.42698E-05_rb, 8.90336E-06_rb, 3.34444E-06_rb, 4.38923E-07_rb/)
      facbrghto(:,7) = (/ &
        & 4.65005E-04_rb, 6.17371E-04_rb, 6.97406E-04_rb, 6.69678E-04_rb,&
        & 6.07109E-04_rb, 5.28335E-04_rb, 4.39953E-04_rb, 3.37188E-04_rb,&
        & 2.40423E-04_rb, 2.80072E-05_rb, 2.32822E-05_rb, 1.93694E-05_rb,&
        & 1.42338E-05_rb, 8.90330E-06_rb, 3.34419E-06_rb, 4.38923E-07_rb/)
      facbrghto(:,8) = (/ &
        & 4.60359E-04_rb, 5.56200E-04_rb, 6.85346E-04_rb, 6.77886E-04_rb,&
        & 6.21456E-04_rb, 5.42517E-04_rb, 4.55226E-04_rb, 3.50183E-04_rb,&
        & 2.50729E-04_rb, 2.93579E-05_rb, 2.45077E-05_rb, 1.93581E-05_rb,&
        & 1.42330E-05_rb, 8.90474E-06_rb, 3.34444E-06_rb, 4.38923E-07_rb/)
      facbrghto(:,9) = (/ &
        & 4.96362E-04_rb, 5.68639E-04_rb, 5.62769E-04_rb, 5.96117E-04_rb,&
        & 6.33898E-04_rb, 5.96321E-04_rb, 4.93513E-04_rb, 3.85413E-04_rb,&
        & 2.66521E-04_rb, 2.97374E-05_rb, 2.44798E-05_rb, 1.93544E-05_rb,&
        & 1.42327E-05_rb, 8.90420E-06_rb, 3.34444E-06_rb, 4.38923E-07_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:,1) = (/ &
        &-4.54546E-04_rb,-4.49063E-04_rb,-4.28890E-04_rb,-4.02214E-04_rb,&
        &-3.62872E-04_rb,-3.14669E-04_rb,-2.57812E-04_rb,-1.95729E-04_rb,&
        &-1.33111E-04_rb,-1.45169E-05_rb,-1.20378E-05_rb,-9.53849E-06_rb,&
        &-7.00564E-06_rb,-4.48138E-06_rb,-1.66421E-06_rb,-2.53017E-07_rb/)
      snsptdrko(:,2) = (/ &
        &-4.59112E-04_rb,-4.46718E-04_rb,-4.29328E-04_rb,-4.01939E-04_rb,&
        &-3.62153E-04_rb,-3.14453E-04_rb,-2.57335E-04_rb,-1.95483E-04_rb,&
        &-1.32706E-04_rb,-1.44536E-05_rb,-1.20062E-05_rb,-9.49904E-06_rb,&
        &-6.94522E-06_rb,-4.41874E-06_rb,-1.64096E-06_rb,-2.14845E-07_rb/)
      snsptdrko(:,3) = (/ &
        &-4.60285E-04_rb,-4.46529E-04_rb,-4.30289E-04_rb,-4.01121E-04_rb,&
        &-3.62109E-04_rb,-3.14180E-04_rb,-2.57208E-04_rb,-1.95295E-04_rb,&
        &-1.32562E-04_rb,-1.43999E-05_rb,-1.19366E-05_rb,-9.45547E-06_rb,&
        &-6.94467E-06_rb,-4.32451E-06_rb,-1.55283E-06_rb,-2.11867E-07_rb/)
      snsptdrko(:,4) = (/ &
        &-4.60123E-04_rb,-4.47815E-04_rb,-4.30578E-04_rb,-4.01094E-04_rb,&
        &-3.61575E-04_rb,-3.14195E-04_rb,-2.57098E-04_rb,-1.94987E-04_rb,&
        &-1.32364E-04_rb,-1.44168E-05_rb,-1.19555E-05_rb,-9.38802E-06_rb,&
        &-6.81338E-06_rb,-4.23742E-06_rb,-1.55283E-06_rb,-2.11867E-07_rb/)
      snsptdrko(:,5) = (/ &
        &-4.59796E-04_rb,-4.48830E-04_rb,-4.31294E-04_rb,-4.01264E-04_rb,&
        &-3.61364E-04_rb,-3.13456E-04_rb,-2.57365E-04_rb,-1.94586E-04_rb,&
        &-1.32179E-04_rb,-1.43710E-05_rb,-1.19077E-05_rb,-9.35537E-06_rb,&
        &-6.71784E-06_rb,-4.15467E-06_rb,-1.55282E-06_rb,-2.11867E-07_rb/)
      snsptdrko(:,6) = (/ &
        &-4.58986E-04_rb,-4.49775E-04_rb,-4.32551E-04_rb,-4.01343E-04_rb,&
        &-3.61542E-04_rb,-3.12820E-04_rb,-2.57261E-04_rb,-1.94384E-04_rb,&
        &-1.31844E-04_rb,-1.43440E-05_rb,-1.18020E-05_rb,-9.19837E-06_rb,&
        &-6.63490E-06_rb,-4.15470E-06_rb,-1.55282E-06_rb,-2.11867E-07_rb/)
      snsptdrko(:,7) = (/ &
        &-4.57816E-04_rb,-4.50776E-04_rb,-4.34077E-04_rb,-4.02097E-04_rb,&
        &-3.61523E-04_rb,-3.12496E-04_rb,-2.56759E-04_rb,-1.94144E-04_rb,&
        &-1.31199E-04_rb,-1.42993E-05_rb,-1.16120E-05_rb,-9.08434E-06_rb,&
        &-6.60366E-06_rb,-4.15464E-06_rb,-1.55283E-06_rb,-2.11867E-07_rb/)
      snsptdrko(:,8) = (/ &
        &-4.56768E-04_rb,-4.49595E-04_rb,-4.36602E-04_rb,-4.04371E-04_rb,&
        &-3.62088E-04_rb,-3.12511E-04_rb,-2.56182E-04_rb,-1.92876E-04_rb,&
        &-1.30323E-04_rb,-1.40143E-05_rb,-1.15145E-05_rb,-9.03752E-06_rb,&
        &-6.60362E-06_rb,-4.15481E-06_rb,-1.55282E-06_rb,-2.11867E-07_rb/)
      snsptdrko(:,9) = (/ &
        &-4.57649E-04_rb,-4.56453E-04_rb,-4.32800E-04_rb,-4.05691E-04_rb,&
        &-3.67051E-04_rb,-3.13792E-04_rb,-2.52037E-04_rb,-1.88168E-04_rb,&
        &-1.27760E-04_rb,-1.39410E-05_rb,-1.15032E-05_rb,-9.03611E-06_rb,&
        &-6.60377E-06_rb,-4.15453E-06_rb,-1.55282E-06_rb,-2.11867E-07_rb/)

! Rayleigh extinction coefficient at v = 4325 cm-1.
      rayl = 1.39e-09_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.14813e-04_rb,0.39842e-04_rb,0.43362e-04_rb,0.43095e-04_rb,0.39811e-04_rb, &
        & 0.34994e-04_rb,0.28984e-04_rb,0.20609e-04_rb,0.18845e-05_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.14432e-04_rb,0.38749e-04_rb,0.42119e-04_rb,0.41834e-04_rb,0.38900e-04_rb, &
        & 0.34365e-04_rb,0.28571e-04_rb,0.20503e-04_rb,0.14715e-05_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.13969e-04_rb,0.37666e-04_rb,0.40942e-04_rb,0.40703e-04_rb,0.37982e-04_rb, &
        & 0.33731e-04_rb,0.28158e-04_rb,0.20351e-04_rb,0.10842e-05_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.13367e-04_rb,0.36605e-04_rb,0.39729e-04_rb,0.39529e-04_rb,0.37033e-04_rb, &
        & 0.33096e-04_rb,0.27690e-04_rb,0.20154e-04_rb,0.78702e-06_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.12765e-04_rb,0.35584e-04_rb,0.38536e-04_rb,0.38361e-04_rb,0.36073e-04_rb, &
        & 0.32384e-04_rb,0.27242e-04_rb,0.19932e-04_rb,0.58296e-06_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.12299e-04_rb,0.34179e-04_rb,0.37405e-04_rb,0.37535e-04_rb,0.34714e-04_rb, &
        & 0.30335e-04_rb,0.24892e-04_rb,0.17524e-04_rb,0.13301e-05_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.12064e-04_rb,0.33225e-04_rb,0.36357e-04_rb,0.36512e-04_rb,0.33870e-04_rb, &
        & 0.29789e-04_rb,0.24578e-04_rb,0.17435e-04_rb,0.10217e-05_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.11697e-04_rb,0.32301e-04_rb,0.35316e-04_rb,0.35481e-04_rb,0.33049e-04_rb, &
        & 0.29226e-04_rb,0.24233e-04_rb,0.17304e-04_rb,0.72152e-06_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.11209e-04_rb,0.31395e-04_rb,0.34263e-04_rb,0.34436e-04_rb,0.32173e-04_rb, &
        & 0.28610e-04_rb,0.23847e-04_rb,0.17161e-04_rb,0.51787e-06_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.10706e-04_rb,0.30496e-04_rb,0.33311e-04_rb,0.33396e-04_rb,0.31295e-04_rb, &
        & 0.28002e-04_rb,0.23445e-04_rb,0.16971e-04_rb,0.40230e-06_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.10271e-04_rb,0.29273e-04_rb,0.32049e-04_rb,0.32365e-04_rb,0.29857e-04_rb, &
        & 0.26171e-04_rb,0.21352e-04_rb,0.14814e-04_rb,0.80470e-06_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.10058e-04_rb,0.28458e-04_rb,0.31196e-04_rb,0.31440e-04_rb,0.29133e-04_rb, &
        & 0.25640e-04_rb,0.21056e-04_rb,0.14756e-04_rb,0.53010e-06_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.98172e-05_rb,0.27651e-04_rb,0.30352e-04_rb,0.30609e-04_rb,0.28396e-04_rb, &
        & 0.25110e-04_rb,0.20730e-04_rb,0.14655e-04_rb,0.38652e-06_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.94577e-05_rb,0.26874e-04_rb,0.29475e-04_rb,0.29749e-04_rb,0.27669e-04_rb, &
        & 0.24553e-04_rb,0.20400e-04_rb,0.14536e-04_rb,0.33236e-06_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.90508e-05_rb,0.26128e-04_rb,0.28631e-04_rb,0.28834e-04_rb,0.26952e-04_rb, &
        & 0.24015e-04_rb,0.20045e-04_rb,0.14382e-04_rb,0.30659e-06_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.86081e-05_rb,0.24897e-04_rb,0.27317e-04_rb,0.27632e-04_rb,0.25465e-04_rb, &
        & 0.22328e-04_rb,0.18112e-04_rb,0.12468e-04_rb,0.39286e-06_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.83793e-05_rb,0.24273e-04_rb,0.26590e-04_rb,0.26865e-04_rb,0.24876e-04_rb, &
        & 0.21867e-04_rb,0.17849e-04_rb,0.12420e-04_rb,0.32266e-06_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.82296e-05_rb,0.23584e-04_rb,0.25869e-04_rb,0.26155e-04_rb,0.24255e-04_rb, &
        & 0.21403e-04_rb,0.17579e-04_rb,0.12338e-04_rb,0.30775e-06_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.79731e-05_rb,0.22915e-04_rb,0.25166e-04_rb,0.25435e-04_rb,0.23627e-04_rb, &
        & 0.20917e-04_rb,0.17282e-04_rb,0.12242e-04_rb,0.30418e-06_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.76553e-05_rb,0.22259e-04_rb,0.24440e-04_rb,0.24673e-04_rb,0.23003e-04_rb, &
        & 0.20437e-04_rb,0.16984e-04_rb,0.12112e-04_rb,0.30638e-06_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.72259e-05_rb,0.21089e-04_rb,0.23177e-04_rb,0.23419e-04_rb,0.21584e-04_rb, &
        & 0.18926e-04_rb,0.15251e-04_rb,0.10417e-04_rb,0.25170e-06_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.70215e-05_rb,0.20567e-04_rb,0.22541e-04_rb,0.22788e-04_rb,0.21102e-04_rb, &
        & 0.18526e-04_rb,0.15049e-04_rb,0.10379e-04_rb,0.25958e-06_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.68490e-05_rb,0.20018e-04_rb,0.21938e-04_rb,0.22206e-04_rb,0.20579e-04_rb, &
        & 0.18139e-04_rb,0.14806e-04_rb,0.10313e-04_rb,0.26666e-06_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.66973e-05_rb,0.19444e-04_rb,0.21340e-04_rb,0.21615e-04_rb,0.20070e-04_rb, &
        & 0.17728e-04_rb,0.14554e-04_rb,0.10239e-04_rb,0.28589e-06_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.64808e-05_rb,0.18883e-04_rb,0.20738e-04_rb,0.20994e-04_rb,0.19546e-04_rb, &
        & 0.17304e-04_rb,0.14306e-04_rb,0.10148e-04_rb,0.30886e-06_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.60727e-05_rb,0.17794e-04_rb,0.19609e-04_rb,0.19769e-04_rb,0.18228e-04_rb, &
        & 0.15946e-04_rb,0.12793e-04_rb,0.86559e-05_rb,0.18559e-06_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.58949e-05_rb,0.17390e-04_rb,0.19059e-04_rb,0.19244e-04_rb,0.17820e-04_rb, &
        & 0.15647e-04_rb,0.12626e-04_rb,0.86354e-05_rb,0.19544e-06_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.57268e-05_rb,0.16935e-04_rb,0.18564e-04_rb,0.18724e-04_rb,0.17402e-04_rb, &
        & 0.15304e-04_rb,0.12433e-04_rb,0.85882e-05_rb,0.22033e-06_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.56243e-05_rb,0.16459e-04_rb,0.18054e-04_rb,0.18263e-04_rb,0.16984e-04_rb, &
        & 0.14971e-04_rb,0.12215e-04_rb,0.85268e-05_rb,0.25072e-06_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.54584e-05_rb,0.15984e-04_rb,0.17548e-04_rb,0.17773e-04_rb,0.16551e-04_rb, &
        & 0.14601e-04_rb,0.12004e-04_rb,0.84568e-05_rb,0.28110e-06_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.50624e-05_rb,0.14963e-04_rb,0.16482e-04_rb,0.16611e-04_rb,0.15339e-04_rb, &
        & 0.13333e-04_rb,0.10656e-04_rb,0.71570e-05_rb,0.13453e-06_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.49428e-05_rb,0.14636e-04_rb,0.16040e-04_rb,0.16181e-04_rb,0.14992e-04_rb, &
        & 0.13144e-04_rb,0.10533e-04_rb,0.71532e-05_rb,0.15620e-06_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.47955e-05_rb,0.14256e-04_rb,0.15611e-04_rb,0.15735e-04_rb,0.14643e-04_rb, &
        & 0.12865e-04_rb,0.10390e-04_rb,0.71248e-05_rb,0.18571e-06_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.46743e-05_rb,0.13876e-04_rb,0.15213e-04_rb,0.15342e-04_rb,0.14293e-04_rb, &
        & 0.12576e-04_rb,0.10215e-04_rb,0.70735e-05_rb,0.21382e-06_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.45708e-05_rb,0.13473e-04_rb,0.14782e-04_rb,0.14947e-04_rb,0.13933e-04_rb, &
        & 0.12274e-04_rb,0.10036e-04_rb,0.70177e-05_rb,0.24419e-06_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.42289e-05_rb,0.12528e-04_rb,0.13817e-04_rb,0.13922e-04_rb,0.12867e-04_rb, &
        & 0.11126e-04_rb,0.88468e-05_rb,0.59010e-05_rb,0.10600e-06_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.41319e-05_rb,0.12286e-04_rb,0.13480e-04_rb,0.13582e-04_rb,0.12584e-04_rb, &
        & 0.10973e-04_rb,0.87644e-05_rb,0.59049e-05_rb,0.12951e-06_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.40317e-05_rb,0.11996e-04_rb,0.13116e-04_rb,0.13206e-04_rb,0.12295e-04_rb, &
        & 0.10786e-04_rb,0.86537e-05_rb,0.58898e-05_rb,0.15731e-06_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.39107e-05_rb,0.11666e-04_rb,0.12771e-04_rb,0.12870e-04_rb,0.12010e-04_rb, &
        & 0.10546e-04_rb,0.85244e-05_rb,0.58536e-05_rb,0.18716e-06_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.38384e-05_rb,0.11346e-04_rb,0.12435e-04_rb,0.12549e-04_rb,0.11706e-04_rb, &
        & 0.10289e-04_rb,0.83756e-05_rb,0.58104e-05_rb,0.21679e-06_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.35240e-05_rb,0.10418e-04_rb,0.11558e-04_rb,0.11687e-04_rb,0.10748e-04_rb, &
        & 0.92305e-05_rb,0.73198e-05_rb,0.48514e-05_rb,0.10176e-06_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.34478e-05_rb,0.10296e-04_rb,0.11296e-04_rb,0.11369e-04_rb,0.10532e-04_rb, &
        & 0.91375e-05_rb,0.72737e-05_rb,0.48639e-05_rb,0.12695e-06_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.33718e-05_rb,0.10068e-04_rb,0.10996e-04_rb,0.11063e-04_rb,0.10290e-04_rb, &
        & 0.89957e-05_rb,0.71844e-05_rb,0.48583e-05_rb,0.15482e-06_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.32801e-05_rb,0.97990e-05_rb,0.10712e-04_rb,0.10762e-04_rb,0.10056e-04_rb, &
        & 0.88255e-05_rb,0.70907e-05_rb,0.48364e-05_rb,0.18123e-06_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.31960e-05_rb,0.95350e-05_rb,0.10427e-04_rb,0.10501e-04_rb,0.98056e-05_rb, &
        & 0.86157e-05_rb,0.69723e-05_rb,0.48010e-05_rb,0.20738e-06_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.29197e-05_rb,0.86183e-05_rb,0.96100e-05_rb,0.97448e-05_rb,0.89187e-05_rb, &
        & 0.76354e-05_rb,0.60318e-05_rb,0.39733e-05_rb,0.10310e-06_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.28677e-05_rb,0.85565e-05_rb,0.94191e-05_rb,0.94682e-05_rb,0.87653e-05_rb, &
        & 0.75725e-05_rb,0.60089e-05_rb,0.39956e-05_rb,0.12951e-06_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.28036e-05_rb,0.84121e-05_rb,0.91902e-05_rb,0.92251e-05_rb,0.85811e-05_rb, &
        & 0.74653e-05_rb,0.59452e-05_rb,0.39967e-05_rb,0.15684e-06_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.27420e-05_rb,0.82035e-05_rb,0.89440e-05_rb,0.89748e-05_rb,0.83817e-05_rb, &
        & 0.73422e-05_rb,0.58725e-05_rb,0.39844e-05_rb,0.18564e-06_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.26596e-05_rb,0.79739e-05_rb,0.87074e-05_rb,0.87508e-05_rb,0.81784e-05_rb, &
        & 0.71781e-05_rb,0.57815e-05_rb,0.39569e-05_rb,0.21476e-06_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.23988e-05_rb,0.71135e-05_rb,0.79121e-05_rb,0.80118e-05_rb,0.73370e-05_rb, &
        & 0.62844e-05_rb,0.49610e-05_rb,0.32641e-05_rb,0.91697e-07_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.23552e-05_rb,0.70587e-05_rb,0.77519e-05_rb,0.77821e-05_rb,0.72113e-05_rb, &
        & 0.62297e-05_rb,0.49407e-05_rb,0.32815e-05_rb,0.11307e-06_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.23046e-05_rb,0.69367e-05_rb,0.75662e-05_rb,0.75790e-05_rb,0.70631e-05_rb, &
        & 0.61409e-05_rb,0.48867e-05_rb,0.32812e-05_rb,0.13715e-06_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.22557e-05_rb,0.67632e-05_rb,0.73640e-05_rb,0.73772e-05_rb,0.68972e-05_rb, &
        & 0.60381e-05_rb,0.48254e-05_rb,0.32705e-05_rb,0.16436e-06_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.21899e-05_rb,0.65748e-05_rb,0.71695e-05_rb,0.71946e-05_rb,0.67295e-05_rb, &
        & 0.59022e-05_rb,0.47488e-05_rb,0.32476e-05_rb,0.19025e-06_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.19699e-05_rb,0.58670e-05_rb,0.65075e-05_rb,0.65808e-05_rb,0.60300e-05_rb, &
        & 0.51665e-05_rb,0.40749e-05_rb,0.26788e-05_rb,0.77100e-07_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.19348e-05_rb,0.58146e-05_rb,0.63730e-05_rb,0.63891e-05_rb,0.59260e-05_rb, &
        & 0.51196e-05_rb,0.40577e-05_rb,0.26926e-05_rb,0.95693e-07_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.18947e-05_rb,0.57134e-05_rb,0.62207e-05_rb,0.62238e-05_rb,0.58090e-05_rb, &
        & 0.50459e-05_rb,0.40113e-05_rb,0.26916e-05_rb,0.11572e-06_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.18552e-05_rb,0.55709e-05_rb,0.60522e-05_rb,0.60611e-05_rb,0.56693e-05_rb, &
        & 0.49594e-05_rb,0.39606e-05_rb,0.26824e-05_rb,0.13797e-06_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.18040e-05_rb,0.54137e-05_rb,0.58935e-05_rb,0.59096e-05_rb,0.55294e-05_rb, &
        & 0.48443e-05_rb,0.38977e-05_rb,0.26633e-05_rb,0.15884e-06_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.16167e-05_rb,0.48272e-05_rb,0.53475e-05_rb,0.53983e-05_rb,0.49503e-05_rb, &
        & 0.42413e-05_rb,0.33431e-05_rb,0.21956e-05_rb,0.61381e-07_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.15895e-05_rb,0.47809e-05_rb,0.52342e-05_rb,0.52409e-05_rb,0.48652e-05_rb, &
        & 0.42021e-05_rb,0.33263e-05_rb,0.22072e-05_rb,0.76382e-07_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.15582e-05_rb,0.46982e-05_rb,0.51055e-05_rb,0.51058e-05_rb,0.47716e-05_rb, &
        & 0.41406e-05_rb,0.32886e-05_rb,0.22058e-05_rb,0.93083e-07_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.15242e-05_rb,0.45831e-05_rb,0.49696e-05_rb,0.49735e-05_rb,0.46534e-05_rb, &
        & 0.40678e-05_rb,0.32469e-05_rb,0.21969e-05_rb,0.11211e-06_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.14859e-05_rb,0.44492e-05_rb,0.48392e-05_rb,0.48479e-05_rb,0.45378e-05_rb, &
        & 0.39689e-05_rb,0.31933e-05_rb,0.21813e-05_rb,0.12970e-06_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.10138e-03_rb,0.20456e-03_rb,0.22117e-03_rb,0.21948e-03_rb,0.20805e-03_rb, &
        & 0.18373e-03_rb,0.14999e-03_rb,0.95382e-04_rb,0.33213e-05_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.10141e-03_rb,0.20889e-03_rb,0.22456e-03_rb,0.22223e-03_rb,0.20912e-03_rb, &
        & 0.18467e-03_rb,0.15060e-03_rb,0.95713e-04_rb,0.28479e-05_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.10125e-03_rb,0.21223e-03_rb,0.22712e-03_rb,0.22388e-03_rb,0.20983e-03_rb, &
        & 0.18513e-03_rb,0.15100e-03_rb,0.95881e-04_rb,0.26338e-05_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.10105e-03_rb,0.21442e-03_rb,0.22863e-03_rb,0.22421e-03_rb,0.21046e-03_rb, &
        & 0.18540e-03_rb,0.15060e-03_rb,0.95798e-04_rb,0.27638e-05_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.10040e-03_rb,0.21612e-03_rb,0.22899e-03_rb,0.22416e-03_rb,0.21046e-03_rb, &
        & 0.18570e-03_rb,0.15001e-03_rb,0.95569e-04_rb,0.29313e-05_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.89639e-04_rb,0.17797e-03_rb,0.19284e-03_rb,0.18986e-03_rb,0.17824e-03_rb, &
        & 0.15789e-03_rb,0.12906e-03_rb,0.83524e-04_rb,0.23926e-05_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.89499e-04_rb,0.18144e-03_rb,0.19563e-03_rb,0.19227e-03_rb,0.17941e-03_rb, &
        & 0.15844e-03_rb,0.12961e-03_rb,0.83632e-04_rb,0.21969e-05_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.89411e-04_rb,0.18401e-03_rb,0.19753e-03_rb,0.19350e-03_rb,0.17999e-03_rb, &
        & 0.15883e-03_rb,0.12989e-03_rb,0.83577e-04_rb,0.23158e-05_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.88841e-04_rb,0.18541e-03_rb,0.19872e-03_rb,0.19398e-03_rb,0.18053e-03_rb, &
        & 0.15907e-03_rb,0.12972e-03_rb,0.83396e-04_rb,0.23760e-05_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.87949e-04_rb,0.18659e-03_rb,0.19902e-03_rb,0.19407e-03_rb,0.18073e-03_rb, &
        & 0.15910e-03_rb,0.12937e-03_rb,0.83120e-04_rb,0.25247e-05_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.77466e-04_rb,0.15264e-03_rb,0.16704e-03_rb,0.16342e-03_rb,0.15300e-03_rb, &
        & 0.13499e-03_rb,0.11007e-03_rb,0.71881e-04_rb,0.17839e-05_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.77483e-04_rb,0.15611e-03_rb,0.16950e-03_rb,0.16551e-03_rb,0.15408e-03_rb, &
        & 0.13580e-03_rb,0.11055e-03_rb,0.72020e-04_rb,0.18564e-05_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.77142e-04_rb,0.15867e-03_rb,0.17112e-03_rb,0.16672e-03_rb,0.15481e-03_rb, &
        & 0.13616e-03_rb,0.11077e-03_rb,0.71922e-04_rb,0.20228e-05_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.76460e-04_rb,0.16058e-03_rb,0.17186e-03_rb,0.16717e-03_rb,0.15519e-03_rb, &
        & 0.13641e-03_rb,0.11060e-03_rb,0.71775e-04_rb,0.21717e-05_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.75510e-04_rb,0.16158e-03_rb,0.17210e-03_rb,0.16730e-03_rb,0.15521e-03_rb, &
        & 0.13649e-03_rb,0.11022e-03_rb,0.71516e-04_rb,0.23140e-05_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.65931e-04_rb,0.12940e-03_rb,0.14295e-03_rb,0.14005e-03_rb,0.13095e-03_rb, &
        & 0.11494e-03_rb,0.93656e-04_rb,0.61167e-04_rb,0.17094e-05_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.66090e-04_rb,0.13269e-03_rb,0.14542e-03_rb,0.14233e-03_rb,0.13201e-03_rb, &
        & 0.11558e-03_rb,0.93975e-04_rb,0.61436e-04_rb,0.18527e-05_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.65630e-04_rb,0.13532e-03_rb,0.14717e-03_rb,0.14364e-03_rb,0.13266e-03_rb, &
        & 0.11604e-03_rb,0.94126e-04_rb,0.61467e-04_rb,0.19576e-05_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.64971e-04_rb,0.13708e-03_rb,0.14818e-03_rb,0.14427e-03_rb,0.13304e-03_rb, &
        & 0.11616e-03_rb,0.94152e-04_rb,0.61334e-04_rb,0.20614e-05_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.64122e-04_rb,0.13834e-03_rb,0.14864e-03_rb,0.14451e-03_rb,0.13312e-03_rb, &
        & 0.11638e-03_rb,0.93955e-04_rb,0.61162e-04_rb,0.21640e-05_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.55594e-04_rb,0.10830e-03_rb,0.12068e-03_rb,0.11865e-03_rb,0.11127e-03_rb, &
        & 0.97925e-04_rb,0.79208e-04_rb,0.51842e-04_rb,0.15954e-05_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.55669e-04_rb,0.11117e-03_rb,0.12299e-03_rb,0.12091e-03_rb,0.11248e-03_rb, &
        & 0.98643e-04_rb,0.79433e-04_rb,0.52127e-04_rb,0.16966e-05_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.55430e-04_rb,0.11345e-03_rb,0.12478e-03_rb,0.12235e-03_rb,0.11332e-03_rb, &
        & 0.99117e-04_rb,0.79823e-04_rb,0.52286e-04_rb,0.18304e-05_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.54800e-04_rb,0.11543e-03_rb,0.12601e-03_rb,0.12304e-03_rb,0.11389e-03_rb, &
        & 0.99346e-04_rb,0.79935e-04_rb,0.52275e-04_rb,0.19479e-05_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.54027e-04_rb,0.11665e-03_rb,0.12660e-03_rb,0.12339e-03_rb,0.11410e-03_rb, &
        & 0.99535e-04_rb,0.79840e-04_rb,0.52168e-04_rb,0.20583e-05_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.46500e-04_rb,0.89887e-04_rb,0.10048e-03_rb,0.99161e-04_rb,0.93136e-04_rb, &
        & 0.82279e-04_rb,0.66872e-04_rb,0.43617e-04_rb,0.14730e-05_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.46632e-04_rb,0.92444e-04_rb,0.10271e-03_rb,0.10123e-03_rb,0.94559e-04_rb, &
        & 0.83135e-04_rb,0.67204e-04_rb,0.44046e-04_rb,0.16215e-05_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.46490e-04_rb,0.94415e-04_rb,0.10446e-03_rb,0.10269e-03_rb,0.95387e-04_rb, &
        & 0.83676e-04_rb,0.67579e-04_rb,0.44279e-04_rb,0.17358e-05_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.46004e-04_rb,0.96190e-04_rb,0.10570e-03_rb,0.10346e-03_rb,0.95883e-04_rb, &
        & 0.84038e-04_rb,0.67753e-04_rb,0.44348e-04_rb,0.18096e-05_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.45362e-04_rb,0.97511e-04_rb,0.10645e-03_rb,0.10384e-03_rb,0.96197e-04_rb, &
        & 0.84347e-04_rb,0.67758e-04_rb,0.44305e-04_rb,0.18882e-05_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.38597e-04_rb,0.74019e-04_rb,0.82882e-04_rb,0.81807e-04_rb,0.76936e-04_rb, &
        & 0.68278e-04_rb,0.55744e-04_rb,0.36507e-04_rb,0.13400e-05_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.38728e-04_rb,0.76353e-04_rb,0.85030e-04_rb,0.83822e-04_rb,0.78523e-04_rb, &
        & 0.69221e-04_rb,0.56199e-04_rb,0.36932e-04_rb,0.14766e-05_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.38688e-04_rb,0.78164e-04_rb,0.86691e-04_rb,0.85269e-04_rb,0.79424e-04_rb, &
        & 0.69849e-04_rb,0.56507e-04_rb,0.37211e-04_rb,0.16077e-05_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.38399e-04_rb,0.79654e-04_rb,0.87882e-04_rb,0.86145e-04_rb,0.79917e-04_rb, &
        & 0.70214e-04_rb,0.56698e-04_rb,0.37302e-04_rb,0.17365e-05_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.37899e-04_rb,0.81012e-04_rb,0.88706e-04_rb,0.86589e-04_rb,0.80259e-04_rb, &
        & 0.70429e-04_rb,0.56810e-04_rb,0.37287e-04_rb,0.18595e-05_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.31959e-04_rb,0.60671e-04_rb,0.67963e-04_rb,0.67053e-04_rb,0.63057e-04_rb, &
        & 0.56287e-04_rb,0.46187e-04_rb,0.30298e-04_rb,0.12071e-05_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.32062e-04_rb,0.62753e-04_rb,0.69892e-04_rb,0.68941e-04_rb,0.64624e-04_rb, &
        & 0.57187e-04_rb,0.46566e-04_rb,0.30687e-04_rb,0.13582e-05_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.32098e-04_rb,0.64435e-04_rb,0.71423e-04_rb,0.70359e-04_rb,0.65646e-04_rb, &
        & 0.57882e-04_rb,0.46897e-04_rb,0.30993e-04_rb,0.15217e-05_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.31956e-04_rb,0.65758e-04_rb,0.72650e-04_rb,0.71275e-04_rb,0.66275e-04_rb, &
        & 0.58298e-04_rb,0.47100e-04_rb,0.31079e-04_rb,0.16794e-05_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.31584e-04_rb,0.66916e-04_rb,0.73502e-04_rb,0.71819e-04_rb,0.66611e-04_rb, &
        & 0.58505e-04_rb,0.47243e-04_rb,0.31088e-04_rb,0.18250e-05_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.26355e-04_rb,0.49472e-04_rb,0.55415e-04_rb,0.54584e-04_rb,0.51361e-04_rb, &
        & 0.46135e-04_rb,0.38027e-04_rb,0.24983e-04_rb,0.13549e-05_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.26472e-04_rb,0.51302e-04_rb,0.57165e-04_rb,0.56417e-04_rb,0.52936e-04_rb, &
        & 0.47048e-04_rb,0.38417e-04_rb,0.25347e-04_rb,0.14941e-05_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.26545e-04_rb,0.52866e-04_rb,0.58563e-04_rb,0.57776e-04_rb,0.53988e-04_rb, &
        & 0.47706e-04_rb,0.38739e-04_rb,0.25638e-04_rb,0.16563e-05_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.26469e-04_rb,0.54105e-04_rb,0.59749e-04_rb,0.58746e-04_rb,0.54668e-04_rb, &
        & 0.48161e-04_rb,0.38970e-04_rb,0.25744e-04_rb,0.18262e-05_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.26248e-04_rb,0.55079e-04_rb,0.60619e-04_rb,0.59317e-04_rb,0.55099e-04_rb, &
        & 0.48435e-04_rb,0.39115e-04_rb,0.25778e-04_rb,0.20238e-05_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.21631e-04_rb,0.40297e-04_rb,0.45170e-04_rb,0.44298e-04_rb,0.41876e-04_rb, &
        & 0.37690e-04_rb,0.31212e-04_rb,0.20552e-04_rb,0.16328e-05_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.21814e-04_rb,0.41896e-04_rb,0.46681e-04_rb,0.46065e-04_rb,0.43227e-04_rb, &
        & 0.38608e-04_rb,0.31614e-04_rb,0.20857e-04_rb,0.18610e-05_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.21864e-04_rb,0.43247e-04_rb,0.47945e-04_rb,0.47327e-04_rb,0.44273e-04_rb, &
        & 0.39201e-04_rb,0.31894e-04_rb,0.21119e-04_rb,0.20775e-05_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.21835e-04_rb,0.44386e-04_rb,0.48986e-04_rb,0.48228e-04_rb,0.44916e-04_rb, &
        & 0.39637e-04_rb,0.32093e-04_rb,0.21228e-04_rb,0.22541e-05_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.21728e-04_rb,0.45263e-04_rb,0.49804e-04_rb,0.48818e-04_rb,0.45409e-04_rb, &
        & 0.39950e-04_rb,0.32269e-04_rb,0.21290e-04_rb,0.24525e-05_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.17823e-04_rb,0.33378e-04_rb,0.37337e-04_rb,0.36619e-04_rb,0.34598e-04_rb, &
        & 0.31113e-04_rb,0.25761e-04_rb,0.16956e-04_rb,0.15042e-05_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.17970e-04_rb,0.34679e-04_rb,0.38589e-04_rb,0.38065e-04_rb,0.35695e-04_rb, &
        & 0.31869e-04_rb,0.26044e-04_rb,0.17214e-04_rb,0.16847e-05_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.17997e-04_rb,0.35791e-04_rb,0.39619e-04_rb,0.39097e-04_rb,0.36539e-04_rb, &
        & 0.32374e-04_rb,0.26298e-04_rb,0.17428e-04_rb,0.18887e-05_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.17967e-04_rb,0.36717e-04_rb,0.40463e-04_rb,0.39821e-04_rb,0.37071e-04_rb, &
        & 0.32716e-04_rb,0.26461e-04_rb,0.17496e-04_rb,0.20546e-05_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.17868e-04_rb,0.37423e-04_rb,0.41124e-04_rb,0.40304e-04_rb,0.37473e-04_rb, &
        & 0.32971e-04_rb,0.26612e-04_rb,0.17532e-04_rb,0.22240e-05_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.14665e-04_rb,0.27605e-04_rb,0.30811e-04_rb,0.30250e-04_rb,0.28525e-04_rb, &
        & 0.25644e-04_rb,0.21186e-04_rb,0.13955e-04_rb,0.12588e-05_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.14775e-04_rb,0.28655e-04_rb,0.31851e-04_rb,0.31408e-04_rb,0.29418e-04_rb, &
        & 0.26261e-04_rb,0.21430e-04_rb,0.14172e-04_rb,0.14181e-05_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.14799e-04_rb,0.29571e-04_rb,0.32687e-04_rb,0.32242e-04_rb,0.30093e-04_rb, &
        & 0.26683e-04_rb,0.21636e-04_rb,0.14338e-04_rb,0.15619e-05_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.14777e-04_rb,0.30325e-04_rb,0.33371e-04_rb,0.32826e-04_rb,0.30553e-04_rb, &
        & 0.26945e-04_rb,0.21775e-04_rb,0.14380e-04_rb,0.16917e-05_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.14685e-04_rb,0.30904e-04_rb,0.33907e-04_rb,0.33214e-04_rb,0.30875e-04_rb, &
        & 0.27158e-04_rb,0.21883e-04_rb,0.14414e-04_rb,0.18423e-05_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.12044e-04_rb,0.22772e-04_rb,0.25375e-04_rb,0.24926e-04_rb,0.23458e-04_rb, &
        & 0.21059e-04_rb,0.17387e-04_rb,0.11457e-04_rb,0.99122e-06_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.12128e-04_rb,0.23637e-04_rb,0.26249e-04_rb,0.25842e-04_rb,0.24187e-04_rb, &
        & 0.21582e-04_rb,0.17590e-04_rb,0.11636e-04_rb,0.11037e-05_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.12152e-04_rb,0.24386e-04_rb,0.26910e-04_rb,0.26531e-04_rb,0.24731e-04_rb, &
        & 0.21923e-04_rb,0.17760e-04_rb,0.11755e-04_rb,0.12026e-05_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.12130e-04_rb,0.24996e-04_rb,0.27464e-04_rb,0.27006e-04_rb,0.25113e-04_rb, &
        & 0.22138e-04_rb,0.17874e-04_rb,0.11797e-04_rb,0.13028e-05_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.12050e-04_rb,0.25474e-04_rb,0.27895e-04_rb,0.27297e-04_rb,0.25373e-04_rb, &
        & 0.22302e-04_rb,0.17948e-04_rb,0.11817e-04_rb,0.14010e-05_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.36823e-03_rb,0.49613e-03_rb,0.51817e-03_rb,0.50538e-03_rb,0.45711e-03_rb, &
        & 0.39605e-03_rb,0.31486e-03_rb,0.21474e-03_rb,0.10932e-04_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.37008e-03_rb,0.50778e-03_rb,0.53361e-03_rb,0.51778e-03_rb,0.46995e-03_rb, &
        & 0.40629e-03_rb,0.32309e-03_rb,0.21878e-03_rb,0.11421e-04_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.36989e-03_rb,0.51812e-03_rb,0.54699e-03_rb,0.52922e-03_rb,0.48036e-03_rb, &
        & 0.41559e-03_rb,0.32965e-03_rb,0.22226e-03_rb,0.12395e-04_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.36721e-03_rb,0.52763e-03_rb,0.55746e-03_rb,0.53957e-03_rb,0.48891e-03_rb, &
        & 0.42313e-03_rb,0.33545e-03_rb,0.22588e-03_rb,0.13515e-04_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.36313e-03_rb,0.53652e-03_rb,0.56680e-03_rb,0.54791e-03_rb,0.49664e-03_rb, &
        & 0.42907e-03_rb,0.34043e-03_rb,0.22916e-03_rb,0.14734e-04_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.32411e-03_rb,0.42815e-03_rb,0.44725e-03_rb,0.43813e-03_rb,0.40150e-03_rb, &
        & 0.34871e-03_rb,0.27912e-03_rb,0.18929e-03_rb,0.80741e-05_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.32567e-03_rb,0.43786e-03_rb,0.46069e-03_rb,0.45027e-03_rb,0.41319e-03_rb, &
        & 0.35819e-03_rb,0.28635e-03_rb,0.19325e-03_rb,0.87340e-05_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.32429e-03_rb,0.44638e-03_rb,0.47228e-03_rb,0.46074e-03_rb,0.42272e-03_rb, &
        & 0.36602e-03_rb,0.29201e-03_rb,0.19653e-03_rb,0.96110e-05_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.32137e-03_rb,0.45531e-03_rb,0.48142e-03_rb,0.46951e-03_rb,0.42996e-03_rb, &
        & 0.37250e-03_rb,0.29709e-03_rb,0.19974e-03_rb,0.10697e-04_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.31776e-03_rb,0.46207e-03_rb,0.48880e-03_rb,0.47659e-03_rb,0.43551e-03_rb, &
        & 0.37776e-03_rb,0.30138e-03_rb,0.20274e-03_rb,0.11704e-04_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.28488e-03_rb,0.36767e-03_rb,0.38081e-03_rb,0.37438e-03_rb,0.34525e-03_rb, &
        & 0.30150e-03_rb,0.24332e-03_rb,0.16415e-03_rb,0.63970e-05_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.28539e-03_rb,0.37509e-03_rb,0.39196e-03_rb,0.38510e-03_rb,0.35566e-03_rb, &
        & 0.30998e-03_rb,0.24963e-03_rb,0.16766e-03_rb,0.72006e-05_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.28397e-03_rb,0.38215e-03_rb,0.40156e-03_rb,0.39428e-03_rb,0.36418e-03_rb, &
        & 0.31727e-03_rb,0.25446e-03_rb,0.17107e-03_rb,0.79574e-05_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.28134e-03_rb,0.38867e-03_rb,0.40959e-03_rb,0.40230e-03_rb,0.37093e-03_rb, &
        & 0.32301e-03_rb,0.25898e-03_rb,0.17447e-03_rb,0.87131e-05_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.27886e-03_rb,0.39360e-03_rb,0.41642e-03_rb,0.40852e-03_rb,0.37609e-03_rb, &
        & 0.32740e-03_rb,0.26312e-03_rb,0.17737e-03_rb,0.94704e-05_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.24923e-03_rb,0.31555e-03_rb,0.32423e-03_rb,0.31731e-03_rb,0.29410e-03_rb, &
        & 0.25671e-03_rb,0.20762e-03_rb,0.14074e-03_rb,0.55877e-05_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.24977e-03_rb,0.32120e-03_rb,0.33372e-03_rb,0.32657e-03_rb,0.30295e-03_rb, &
        & 0.26472e-03_rb,0.21303e-03_rb,0.14396e-03_rb,0.61575e-05_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.24865e-03_rb,0.32668e-03_rb,0.34180e-03_rb,0.33446e-03_rb,0.31033e-03_rb, &
        & 0.27144e-03_rb,0.21777e-03_rb,0.14714e-03_rb,0.67916e-05_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.24709e-03_rb,0.33133e-03_rb,0.34841e-03_rb,0.34134e-03_rb,0.31600e-03_rb, &
        & 0.27647e-03_rb,0.22179e-03_rb,0.15016e-03_rb,0.74653e-05_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.24455e-03_rb,0.33491e-03_rb,0.35388e-03_rb,0.34708e-03_rb,0.32069e-03_rb, &
        & 0.28005e-03_rb,0.22522e-03_rb,0.15242e-03_rb,0.81619e-05_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.21526e-03_rb,0.27115e-03_rb,0.27505e-03_rb,0.26802e-03_rb,0.24968e-03_rb, &
        & 0.21690e-03_rb,0.17481e-03_rb,0.11906e-03_rb,0.50850e-05_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.21611e-03_rb,0.27621e-03_rb,0.28298e-03_rb,0.27620e-03_rb,0.25722e-03_rb, &
        & 0.22411e-03_rb,0.18004e-03_rb,0.12197e-03_rb,0.55738e-05_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.21583e-03_rb,0.28060e-03_rb,0.28955e-03_rb,0.28282e-03_rb,0.26346e-03_rb, &
        & 0.23005e-03_rb,0.18415e-03_rb,0.12451e-03_rb,0.60864e-05_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.21446e-03_rb,0.28363e-03_rb,0.29469e-03_rb,0.28896e-03_rb,0.26860e-03_rb, &
        & 0.23435e-03_rb,0.18768e-03_rb,0.12690e-03_rb,0.66417e-05_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.21217e-03_rb,0.28626e-03_rb,0.29929e-03_rb,0.29370e-03_rb,0.27250e-03_rb, &
        & 0.23763e-03_rb,0.19073e-03_rb,0.12909e-03_rb,0.72122e-05_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.18262e-03_rb,0.23112e-03_rb,0.23388e-03_rb,0.22681e-03_rb,0.21062e-03_rb, &
        & 0.18196e-03_rb,0.14634e-03_rb,0.99707e-04_rb,0.48078e-05_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.18393e-03_rb,0.23591e-03_rb,0.24094e-03_rb,0.23364e-03_rb,0.21663e-03_rb, &
        & 0.18814e-03_rb,0.15114e-03_rb,0.10205e-03_rb,0.52252e-05_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.18379e-03_rb,0.23984e-03_rb,0.24676e-03_rb,0.23956e-03_rb,0.22221e-03_rb, &
        & 0.19356e-03_rb,0.15489e-03_rb,0.10434e-03_rb,0.56405e-05_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.18270e-03_rb,0.24258e-03_rb,0.25102e-03_rb,0.24458e-03_rb,0.22685e-03_rb, &
        & 0.19771e-03_rb,0.15805e-03_rb,0.10655e-03_rb,0.60728e-05_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.18067e-03_rb,0.24446e-03_rb,0.25467e-03_rb,0.24894e-03_rb,0.23051e-03_rb, &
        & 0.20056e-03_rb,0.16088e-03_rb,0.10844e-03_rb,0.65160e-05_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.15269e-03_rb,0.19407e-03_rb,0.19640e-03_rb,0.19073e-03_rb,0.17715e-03_rb, &
        & 0.15224e-03_rb,0.12181e-03_rb,0.83302e-04_rb,0.45185e-05_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.15450e-03_rb,0.19834e-03_rb,0.20246e-03_rb,0.19675e-03_rb,0.18242e-03_rb, &
        & 0.15776e-03_rb,0.12610e-03_rb,0.85306e-04_rb,0.49315e-05_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.15459e-03_rb,0.20178e-03_rb,0.20759e-03_rb,0.20176e-03_rb,0.18737e-03_rb, &
        & 0.16233e-03_rb,0.12951e-03_rb,0.87417e-04_rb,0.53491e-05_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.15368e-03_rb,0.20410e-03_rb,0.21160e-03_rb,0.20623e-03_rb,0.19161e-03_rb, &
        & 0.16603e-03_rb,0.13246e-03_rb,0.89237e-04_rb,0.57562e-05_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.15207e-03_rb,0.20567e-03_rb,0.21458e-03_rb,0.20991e-03_rb,0.19525e-03_rb, &
        & 0.16906e-03_rb,0.13481e-03_rb,0.90932e-04_rb,0.61659e-05_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.12672e-03_rb,0.16159e-03_rb,0.16329e-03_rb,0.15821e-03_rb,0.14746e-03_rb, &
        & 0.12624e-03_rb,0.10116e-03_rb,0.69306e-04_rb,0.46652e-05_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.12853e-03_rb,0.16539e-03_rb,0.16859e-03_rb,0.16359e-03_rb,0.15208e-03_rb, &
        & 0.13120e-03_rb,0.10513e-03_rb,0.71005e-04_rb,0.51392e-05_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.12901e-03_rb,0.16841e-03_rb,0.17301e-03_rb,0.16813e-03_rb,0.15614e-03_rb, &
        & 0.13532e-03_rb,0.10829e-03_rb,0.72863e-04_rb,0.55244e-05_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.12833e-03_rb,0.17058e-03_rb,0.17652e-03_rb,0.17184e-03_rb,0.15971e-03_rb, &
        & 0.13867e-03_rb,0.11087e-03_rb,0.74475e-04_rb,0.58193e-05_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.12715e-03_rb,0.17197e-03_rb,0.17918e-03_rb,0.17513e-03_rb,0.16292e-03_rb, &
        & 0.14144e-03_rb,0.11301e-03_rb,0.75965e-04_rb,0.61705e-05_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.10428e-03_rb,0.13348e-03_rb,0.13462e-03_rb,0.12995e-03_rb,0.12158e-03_rb, &
        & 0.10379e-03_rb,0.83005e-04_rb,0.57371e-04_rb,0.55965e-05_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.10619e-03_rb,0.13710e-03_rb,0.13932e-03_rb,0.13469e-03_rb,0.12538e-03_rb, &
        & 0.10800e-03_rb,0.86632e-04_rb,0.58954e-04_rb,0.61961e-05_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.10690e-03_rb,0.13978e-03_rb,0.14315e-03_rb,0.13877e-03_rb,0.12885e-03_rb, &
        & 0.11169e-03_rb,0.89643e-04_rb,0.60568e-04_rb,0.68526e-05_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.10668e-03_rb,0.14178e-03_rb,0.14626e-03_rb,0.14213e-03_rb,0.13207e-03_rb, &
        & 0.11468e-03_rb,0.91984e-04_rb,0.62070e-04_rb,0.75122e-05_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.10582e-03_rb,0.14316e-03_rb,0.14873e-03_rb,0.14501e-03_rb,0.13471e-03_rb, &
        & 0.11704e-03_rb,0.93876e-04_rb,0.63388e-04_rb,0.80761e-05_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.85476e-04_rb,0.10982e-03_rb,0.11034e-03_rb,0.10643e-03_rb,0.99702e-04_rb, &
        & 0.84967e-04_rb,0.68008e-04_rb,0.47229e-04_rb,0.66145e-05_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.87242e-04_rb,0.11322e-03_rb,0.11463e-03_rb,0.11045e-03_rb,0.10293e-03_rb, &
        & 0.88580e-04_rb,0.71113e-04_rb,0.48631e-04_rb,0.75452e-05_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.88111e-04_rb,0.11563e-03_rb,0.11792e-03_rb,0.11404e-03_rb,0.10587e-03_rb, &
        & 0.91877e-04_rb,0.73790e-04_rb,0.50044e-04_rb,0.84881e-05_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.88200e-04_rb,0.11743e-03_rb,0.12065e-03_rb,0.11708e-03_rb,0.10870e-03_rb, &
        & 0.94480e-04_rb,0.75897e-04_rb,0.51384e-04_rb,0.95136e-05_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.87583e-04_rb,0.11871e-03_rb,0.12292e-03_rb,0.11954e-03_rb,0.11103e-03_rb, &
        & 0.96523e-04_rb,0.77526e-04_rb,0.52518e-04_rb,0.10532e-04_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.70556e-04_rb,0.91428e-04_rb,0.91867e-04_rb,0.88463e-04_rb,0.82846e-04_rb, &
        & 0.70774e-04_rb,0.56632e-04_rb,0.39167e-04_rb,0.61872e-05_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.71933e-04_rb,0.94182e-04_rb,0.95269e-04_rb,0.91646e-04_rb,0.85383e-04_rb, &
        & 0.73738e-04_rb,0.59219e-04_rb,0.40474e-04_rb,0.69965e-05_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.72567e-04_rb,0.96098e-04_rb,0.97927e-04_rb,0.94606e-04_rb,0.87837e-04_rb, &
        & 0.76370e-04_rb,0.61326e-04_rb,0.41633e-04_rb,0.78235e-05_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.72529e-04_rb,0.97532e-04_rb,0.10011e-03_rb,0.97076e-04_rb,0.90134e-04_rb, &
        & 0.78428e-04_rb,0.63016e-04_rb,0.42754e-04_rb,0.87904e-05_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.71990e-04_rb,0.98552e-04_rb,0.10193e-03_rb,0.99080e-04_rb,0.92091e-04_rb, &
        & 0.80068e-04_rb,0.64349e-04_rb,0.43674e-04_rb,0.98011e-05_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.58072e-04_rb,0.75815e-04_rb,0.76168e-04_rb,0.73252e-04_rb,0.68521e-04_rb, &
        & 0.58748e-04_rb,0.47029e-04_rb,0.32430e-04_rb,0.51838e-05_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.59149e-04_rb,0.78005e-04_rb,0.78889e-04_rb,0.75853e-04_rb,0.70691e-04_rb, &
        & 0.61160e-04_rb,0.49105e-04_rb,0.33572e-04_rb,0.58669e-05_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.59615e-04_rb,0.79572e-04_rb,0.81043e-04_rb,0.78301e-04_rb,0.72716e-04_rb, &
        & 0.63265e-04_rb,0.50777e-04_rb,0.34506e-04_rb,0.66464e-05_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.59506e-04_rb,0.80712e-04_rb,0.82811e-04_rb,0.80321e-04_rb,0.74571e-04_rb, &
        & 0.64935e-04_rb,0.52165e-04_rb,0.35447e-04_rb,0.74412e-05_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.59067e-04_rb,0.81520e-04_rb,0.84288e-04_rb,0.81977e-04_rb,0.76172e-04_rb, &
        & 0.66259e-04_rb,0.53281e-04_rb,0.36178e-04_rb,0.82465e-05_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.47640e-04_rb,0.62583e-04_rb,0.62887e-04_rb,0.60485e-04_rb,0.56559e-04_rb, &
        & 0.48624e-04_rb,0.38904e-04_rb,0.26773e-04_rb,0.40582e-05_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.48498e-04_rb,0.64379e-04_rb,0.65087e-04_rb,0.62641e-04_rb,0.58377e-04_rb, &
        & 0.50572e-04_rb,0.40566e-04_rb,0.27717e-04_rb,0.46616e-05_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.48844e-04_rb,0.65656e-04_rb,0.66851e-04_rb,0.64628e-04_rb,0.60042e-04_rb, &
        & 0.52254e-04_rb,0.41917e-04_rb,0.28510e-04_rb,0.53204e-05_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.48722e-04_rb,0.66574e-04_rb,0.68303e-04_rb,0.66300e-04_rb,0.61547e-04_rb, &
        & 0.53614e-04_rb,0.43058e-04_rb,0.29254e-04_rb,0.59442e-05_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.48355e-04_rb,0.67217e-04_rb,0.69511e-04_rb,0.67651e-04_rb,0.62864e-04_rb, &
        & 0.54712e-04_rb,0.43998e-04_rb,0.29875e-04_rb,0.66436e-05_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.85387e-03_rb,0.97268e-03_rb,0.95201e-03_rb,0.89226e-03_rb,0.81465e-03_rb, &
        & 0.69707e-03_rb,0.54829e-03_rb,0.35882e-03_rb,0.37920e-04_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.85496e-03_rb,0.98746e-03_rb,0.97188e-03_rb,0.91775e-03_rb,0.83529e-03_rb, &
        & 0.71649e-03_rb,0.56376e-03_rb,0.37061e-03_rb,0.43155e-04_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.85154e-03_rb,0.99908e-03_rb,0.98969e-03_rb,0.94014e-03_rb,0.85604e-03_rb, &
        & 0.73379e-03_rb,0.57832e-03_rb,0.38183e-03_rb,0.48702e-04_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.84606e-03_rb,0.10090e-02_rb,0.10068e-02_rb,0.95944e-03_rb,0.87434e-03_rb, &
        & 0.74975e-03_rb,0.59243e-03_rb,0.39188e-03_rb,0.54022e-04_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.84061e-03_rb,0.10171e-02_rb,0.10218e-02_rb,0.97684e-03_rb,0.88906e-03_rb, &
        & 0.76345e-03_rb,0.60471e-03_rb,0.40180e-03_rb,0.59778e-04_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.76106e-03_rb,0.86763e-03_rb,0.84620e-03_rb,0.79337e-03_rb,0.72368e-03_rb, &
        & 0.62633e-03_rb,0.49168e-03_rb,0.32346e-03_rb,0.31322e-04_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.76153e-03_rb,0.88148e-03_rb,0.86349e-03_rb,0.81559e-03_rb,0.74480e-03_rb, &
        & 0.64448e-03_rb,0.50635e-03_rb,0.33430e-03_rb,0.35566e-04_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.75977e-03_rb,0.89195e-03_rb,0.87871e-03_rb,0.83605e-03_rb,0.76286e-03_rb, &
        & 0.66127e-03_rb,0.51959e-03_rb,0.34472e-03_rb,0.39705e-04_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.75771e-03_rb,0.89987e-03_rb,0.89406e-03_rb,0.85530e-03_rb,0.78123e-03_rb, &
        & 0.67612e-03_rb,0.53221e-03_rb,0.35447e-03_rb,0.44266e-04_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.75289e-03_rb,0.90657e-03_rb,0.90771e-03_rb,0.87200e-03_rb,0.79715e-03_rb, &
        & 0.68926e-03_rb,0.54382e-03_rb,0.36372e-03_rb,0.48708e-04_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.66893e-03_rb,0.76246e-03_rb,0.74334e-03_rb,0.69752e-03_rb,0.63501e-03_rb, &
        & 0.55362e-03_rb,0.43455e-03_rb,0.28843e-03_rb,0.24260e-04_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.67118e-03_rb,0.77588e-03_rb,0.75944e-03_rb,0.71710e-03_rb,0.65461e-03_rb, &
        & 0.57122e-03_rb,0.44991e-03_rb,0.29906e-03_rb,0.27434e-04_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.67218e-03_rb,0.78505e-03_rb,0.77328e-03_rb,0.73497e-03_rb,0.67286e-03_rb, &
        & 0.58718e-03_rb,0.46375e-03_rb,0.30933e-03_rb,0.30808e-04_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.67058e-03_rb,0.79242e-03_rb,0.78627e-03_rb,0.75181e-03_rb,0.68946e-03_rb, &
        & 0.60183e-03_rb,0.47532e-03_rb,0.31743e-03_rb,0.34324e-04_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.66467e-03_rb,0.79822e-03_rb,0.79776e-03_rb,0.76779e-03_rb,0.70481e-03_rb, &
        & 0.61510e-03_rb,0.48553e-03_rb,0.32526e-03_rb,0.37905e-04_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.58297e-03_rb,0.66020e-03_rb,0.64337e-03_rb,0.60610e-03_rb,0.55222e-03_rb, &
        & 0.48338e-03_rb,0.38154e-03_rb,0.25537e-03_rb,0.20054e-04_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.58616e-03_rb,0.67311e-03_rb,0.65764e-03_rb,0.62362e-03_rb,0.56996e-03_rb, &
        & 0.49939e-03_rb,0.39657e-03_rb,0.26418e-03_rb,0.22457e-04_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.58719e-03_rb,0.68291e-03_rb,0.67014e-03_rb,0.63926e-03_rb,0.58703e-03_rb, &
        & 0.51453e-03_rb,0.40976e-03_rb,0.27280e-03_rb,0.24974e-04_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.58451e-03_rb,0.68925e-03_rb,0.68144e-03_rb,0.65410e-03_rb,0.60312e-03_rb, &
        & 0.52903e-03_rb,0.42147e-03_rb,0.28096e-03_rb,0.27613e-04_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.57885e-03_rb,0.69369e-03_rb,0.69153e-03_rb,0.66725e-03_rb,0.61666e-03_rb, &
        & 0.54114e-03_rb,0.43123e-03_rb,0.28907e-03_rb,0.30243e-04_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.50639e-03_rb,0.56897e-03_rb,0.55412e-03_rb,0.52127e-03_rb,0.47341e-03_rb, &
        & 0.41669e-03_rb,0.33293e-03_rb,0.22215e-03_rb,0.17057e-04_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.51065e-03_rb,0.58111e-03_rb,0.56740e-03_rb,0.53642e-03_rb,0.48970e-03_rb, &
        & 0.43166e-03_rb,0.34671e-03_rb,0.23111e-03_rb,0.19191e-04_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.51131e-03_rb,0.58923e-03_rb,0.57802e-03_rb,0.54989e-03_rb,0.50519e-03_rb, &
        & 0.44520e-03_rb,0.35867e-03_rb,0.23955e-03_rb,0.21238e-04_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.50888e-03_rb,0.59432e-03_rb,0.58700e-03_rb,0.56238e-03_rb,0.51899e-03_rb, &
        & 0.45787e-03_rb,0.36960e-03_rb,0.24733e-03_rb,0.23442e-04_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.50411e-03_rb,0.59804e-03_rb,0.59497e-03_rb,0.57342e-03_rb,0.53158e-03_rb, &
        & 0.46924e-03_rb,0.37886e-03_rb,0.25450e-03_rb,0.25756e-04_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.43673e-03_rb,0.48782e-03_rb,0.47499e-03_rb,0.44629e-03_rb,0.40354e-03_rb, &
        & 0.35457e-03_rb,0.28605e-03_rb,0.19086e-03_rb,0.14702e-04_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.44102e-03_rb,0.49831e-03_rb,0.48639e-03_rb,0.45943e-03_rb,0.41879e-03_rb, &
        & 0.36808e-03_rb,0.29802e-03_rb,0.19985e-03_rb,0.16471e-04_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.44221e-03_rb,0.50518e-03_rb,0.49538e-03_rb,0.47075e-03_rb,0.43183e-03_rb, &
        & 0.37995e-03_rb,0.30882e-03_rb,0.20763e-03_rb,0.18420e-04_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.44017e-03_rb,0.50994e-03_rb,0.50323e-03_rb,0.48141e-03_rb,0.44360e-03_rb, &
        & 0.39116e-03_rb,0.31844e-03_rb,0.21460e-03_rb,0.20473e-04_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.43656e-03_rb,0.51249e-03_rb,0.51000e-03_rb,0.49068e-03_rb,0.45425e-03_rb, &
        & 0.40107e-03_rb,0.32653e-03_rb,0.22109e-03_rb,0.22600e-04_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.37627e-03_rb,0.41525e-03_rb,0.40365e-03_rb,0.37923e-03_rb,0.34257e-03_rb, &
        & 0.30147e-03_rb,0.24324e-03_rb,0.16094e-03_rb,0.13400e-04_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.38051e-03_rb,0.42489e-03_rb,0.41400e-03_rb,0.39067e-03_rb,0.35555e-03_rb, &
        & 0.31257e-03_rb,0.25353e-03_rb,0.16947e-03_rb,0.14905e-04_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.38196e-03_rb,0.43154e-03_rb,0.42169e-03_rb,0.40035e-03_rb,0.36703e-03_rb, &
        & 0.32311e-03_rb,0.26291e-03_rb,0.17672e-03_rb,0.16566e-04_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.38108e-03_rb,0.43587e-03_rb,0.42809e-03_rb,0.40890e-03_rb,0.37694e-03_rb, &
        & 0.33246e-03_rb,0.27094e-03_rb,0.18305e-03_rb,0.18352e-04_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.37854e-03_rb,0.43889e-03_rb,0.43402e-03_rb,0.41697e-03_rb,0.38532e-03_rb, &
        & 0.34074e-03_rb,0.27816e-03_rb,0.18873e-03_rb,0.20220e-04_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.31968e-03_rb,0.35301e-03_rb,0.34310e-03_rb,0.32206e-03_rb,0.28904e-03_rb, &
        & 0.25356e-03_rb,0.20523e-03_rb,0.13432e-03_rb,0.13548e-04_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.32386e-03_rb,0.36229e-03_rb,0.35288e-03_rb,0.33214e-03_rb,0.30033e-03_rb, &
        & 0.26345e-03_rb,0.21374e-03_rb,0.14198e-03_rb,0.14966e-04_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.32521e-03_rb,0.36848e-03_rb,0.35986e-03_rb,0.34056e-03_rb,0.31042e-03_rb, &
        & 0.27222e-03_rb,0.22115e-03_rb,0.14841e-03_rb,0.16278e-04_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.32527e-03_rb,0.37258e-03_rb,0.36553e-03_rb,0.34811e-03_rb,0.31916e-03_rb, &
        & 0.28034e-03_rb,0.22837e-03_rb,0.15443e-03_rb,0.17614e-04_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.32272e-03_rb,0.37533e-03_rb,0.37025e-03_rb,0.35438e-03_rb,0.32632e-03_rb, &
        & 0.28740e-03_rb,0.23484e-03_rb,0.15952e-03_rb,0.19075e-04_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.26794e-03_rb,0.29655e-03_rb,0.28851e-03_rb,0.27155e-03_rb,0.24290e-03_rb, &
        & 0.21182e-03_rb,0.17187e-03_rb,0.11158e-03_rb,0.16893e-04_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.27215e-03_rb,0.30477e-03_rb,0.29765e-03_rb,0.28075e-03_rb,0.25336e-03_rb, &
        & 0.22120e-03_rb,0.17858e-03_rb,0.11794e-03_rb,0.18576e-04_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.27400e-03_rb,0.31061e-03_rb,0.30447e-03_rb,0.28836e-03_rb,0.26224e-03_rb, &
        & 0.22886e-03_rb,0.18474e-03_rb,0.12372e-03_rb,0.20056e-04_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.27385e-03_rb,0.31424e-03_rb,0.30923e-03_rb,0.29477e-03_rb,0.26977e-03_rb, &
        & 0.23554e-03_rb,0.19093e-03_rb,0.12884e-03_rb,0.21321e-04_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.27196e-03_rb,0.31647e-03_rb,0.31308e-03_rb,0.30006e-03_rb,0.27604e-03_rb, &
        & 0.24191e-03_rb,0.19649e-03_rb,0.13352e-03_rb,0.22611e-04_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.22255e-03_rb,0.24716e-03_rb,0.24087e-03_rb,0.22672e-03_rb,0.20216e-03_rb, &
        & 0.17582e-03_rb,0.14339e-03_rb,0.92528e-04_rb,0.22471e-04_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.22663e-03_rb,0.25429e-03_rb,0.24908e-03_rb,0.23506e-03_rb,0.21167e-03_rb, &
        & 0.18377e-03_rb,0.14910e-03_rb,0.97972e-04_rb,0.24536e-04_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.22867e-03_rb,0.25963e-03_rb,0.25532e-03_rb,0.24169e-03_rb,0.21923e-03_rb, &
        & 0.19067e-03_rb,0.15458e-03_rb,0.10293e-03_rb,0.26453e-04_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.22853e-03_rb,0.26273e-03_rb,0.25960e-03_rb,0.24704e-03_rb,0.22549e-03_rb, &
        & 0.19679e-03_rb,0.15968e-03_rb,0.10728e-03_rb,0.28383e-04_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.22704e-03_rb,0.26469e-03_rb,0.26259e-03_rb,0.25141e-03_rb,0.23080e-03_rb, &
        & 0.20216e-03_rb,0.16433e-03_rb,0.11116e-03_rb,0.30111e-04_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.18570e-03_rb,0.20791e-03_rb,0.20310e-03_rb,0.19106e-03_rb,0.17066e-03_rb, &
        & 0.14797e-03_rb,0.12097e-03_rb,0.78704e-04_rb,0.20720e-04_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.18868e-03_rb,0.21356e-03_rb,0.20969e-03_rb,0.19807e-03_rb,0.17821e-03_rb, &
        & 0.15434e-03_rb,0.12584e-03_rb,0.83179e-04_rb,0.23268e-04_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.18972e-03_rb,0.21732e-03_rb,0.21447e-03_rb,0.20305e-03_rb,0.18416e-03_rb, &
        & 0.15993e-03_rb,0.13006e-03_rb,0.87134e-04_rb,0.25781e-04_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.18929e-03_rb,0.21955e-03_rb,0.21771e-03_rb,0.20711e-03_rb,0.18907e-03_rb, &
        & 0.16493e-03_rb,0.13420e-03_rb,0.90699e-04_rb,0.28181e-04_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.18766e-03_rb,0.22095e-03_rb,0.21987e-03_rb,0.21045e-03_rb,0.19312e-03_rb, &
        & 0.16934e-03_rb,0.13812e-03_rb,0.93840e-04_rb,0.30489e-04_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.15409e-03_rb,0.17399e-03_rb,0.17010e-03_rb,0.16002e-03_rb,0.14311e-03_rb, &
        & 0.12376e-03_rb,0.10121e-03_rb,0.66428e-04_rb,0.17813e-04_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.15599e-03_rb,0.17831e-03_rb,0.17521e-03_rb,0.16554e-03_rb,0.14890e-03_rb, &
        & 0.12881e-03_rb,0.10527e-03_rb,0.69964e-04_rb,0.19918e-04_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.15661e-03_rb,0.18119e-03_rb,0.17892e-03_rb,0.16934e-03_rb,0.15365e-03_rb, &
        & 0.13338e-03_rb,0.10864e-03_rb,0.73142e-04_rb,0.22121e-04_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.15608e-03_rb,0.18293e-03_rb,0.18145e-03_rb,0.17254e-03_rb,0.15743e-03_rb, &
        & 0.13747e-03_rb,0.11209e-03_rb,0.75971e-04_rb,0.24425e-04_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.15453e-03_rb,0.18383e-03_rb,0.18309e-03_rb,0.17510e-03_rb,0.16067e-03_rb, &
        & 0.14102e-03_rb,0.11529e-03_rb,0.78607e-04_rb,0.26791e-04_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.12705e-03_rb,0.14485e-03_rb,0.14155e-03_rb,0.13312e-03_rb,0.11914e-03_rb, &
        & 0.10299e-03_rb,0.84416e-04_rb,0.55598e-04_rb,0.14429e-04_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.12843e-03_rb,0.14822e-03_rb,0.14555e-03_rb,0.13748e-03_rb,0.12377e-03_rb, &
        & 0.10702e-03_rb,0.87520e-04_rb,0.58405e-04_rb,0.16147e-04_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.12875e-03_rb,0.15043e-03_rb,0.14850e-03_rb,0.14053e-03_rb,0.12752e-03_rb, &
        & 0.11081e-03_rb,0.90348e-04_rb,0.60934e-04_rb,0.17992e-04_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.12822e-03_rb,0.15175e-03_rb,0.15049e-03_rb,0.14301e-03_rb,0.13059e-03_rb, &
        & 0.11413e-03_rb,0.93171e-04_rb,0.63296e-04_rb,0.19888e-04_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.12684e-03_rb,0.15239e-03_rb,0.15179e-03_rb,0.14511e-03_rb,0.13320e-03_rb, &
        & 0.11702e-03_rb,0.95786e-04_rb,0.65468e-04_rb,0.21706e-04_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.16451e-02_rb,0.17375e-02_rb,0.16665e-02_rb,0.15396e-02_rb,0.13696e-02_rb, &
        & 0.11731e-02_rb,0.92753e-03_rb,0.61159e-03_rb,0.14680e-03_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.16450e-02_rb,0.17591e-02_rb,0.16954e-02_rb,0.15711e-02_rb,0.14087e-02_rb, &
        & 0.12092e-02_rb,0.95859e-03_rb,0.63308e-03_rb,0.16131e-03_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.16396e-02_rb,0.17762e-02_rb,0.17200e-02_rb,0.16012e-02_rb,0.14448e-02_rb, &
        & 0.12424e-02_rb,0.98862e-03_rb,0.65400e-03_rb,0.17365e-03_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.16299e-02_rb,0.17875e-02_rb,0.17415e-02_rb,0.16313e-02_rb,0.14788e-02_rb, &
        & 0.12751e-02_rb,0.10161e-02_rb,0.67524e-03_rb,0.18499e-03_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.16127e-02_rb,0.17946e-02_rb,0.17629e-02_rb,0.16594e-02_rb,0.15119e-02_rb, &
        & 0.13059e-02_rb,0.10419e-02_rb,0.69511e-03_rb,0.19999e-03_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.15084e-02_rb,0.15931e-02_rb,0.15186e-02_rb,0.13958e-02_rb,0.12369e-02_rb, &
        & 0.10500e-02_rb,0.83688e-03_rb,0.55465e-03_rb,0.11914e-03_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.15115e-02_rb,0.16146e-02_rb,0.15457e-02_rb,0.14241e-02_rb,0.12724e-02_rb, &
        & 0.10844e-02_rb,0.86669e-03_rb,0.57603e-03_rb,0.13037e-03_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.15084e-02_rb,0.16309e-02_rb,0.15669e-02_rb,0.14522e-02_rb,0.13074e-02_rb, &
        & 0.11181e-02_rb,0.89787e-03_rb,0.59657e-03_rb,0.13918e-03_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.14965e-02_rb,0.16419e-02_rb,0.15861e-02_rb,0.14781e-02_rb,0.13385e-02_rb, &
        & 0.11507e-02_rb,0.92569e-03_rb,0.61596e-03_rb,0.15053e-03_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.14804e-02_rb,0.16493e-02_rb,0.16042e-02_rb,0.15021e-02_rb,0.13704e-02_rb, &
        & 0.11804e-02_rb,0.95044e-03_rb,0.63472e-03_rb,0.16410e-03_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.13707e-02_rb,0.14436e-02_rb,0.13706e-02_rb,0.12561e-02_rb,0.11093e-02_rb, &
        & 0.93534e-03_rb,0.74505e-03_rb,0.49303e-03_rb,0.93388e-04_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.13760e-02_rb,0.14666e-02_rb,0.13986e-02_rb,0.12850e-02_rb,0.11416e-02_rb, &
        & 0.96700e-03_rb,0.77192e-03_rb,0.51430e-03_rb,0.10062e-03_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.13732e-02_rb,0.14839e-02_rb,0.14208e-02_rb,0.13112e-02_rb,0.11718e-02_rb, &
        & 0.99814e-03_rb,0.79899e-03_rb,0.53404e-03_rb,0.10964e-03_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.13644e-02_rb,0.14939e-02_rb,0.14404e-02_rb,0.13346e-02_rb,0.12002e-02_rb, &
        & 0.10295e-02_rb,0.82532e-03_rb,0.55285e-03_rb,0.12023e-03_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.13512e-02_rb,0.15024e-02_rb,0.14568e-02_rb,0.13559e-02_rb,0.12266e-02_rb, &
        & 0.10592e-02_rb,0.85085e-03_rb,0.57058e-03_rb,0.13064e-03_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.12336e-02_rb,0.12968e-02_rb,0.12255e-02_rb,0.11189e-02_rb,0.98785e-03_rb, &
        & 0.83184e-03_rb,0.66033e-03_rb,0.43467e-03_rb,0.70047e-04_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.12419e-02_rb,0.13192e-02_rb,0.12543e-02_rb,0.11482e-02_rb,0.10189e-02_rb, &
        & 0.86315e-03_rb,0.68559e-03_rb,0.45559e-03_rb,0.76629e-04_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.12416e-02_rb,0.13349e-02_rb,0.12771e-02_rb,0.11745e-02_rb,0.10469e-02_rb, &
        & 0.89182e-03_rb,0.71110e-03_rb,0.47551e-03_rb,0.84761e-04_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.12342e-02_rb,0.13470e-02_rb,0.12958e-02_rb,0.11964e-02_rb,0.10726e-02_rb, &
        & 0.91965e-03_rb,0.73487e-03_rb,0.49419e-03_rb,0.93979e-04_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.12238e-02_rb,0.13566e-02_rb,0.13110e-02_rb,0.12168e-02_rb,0.10976e-02_rb, &
        & 0.94731e-03_rb,0.75866e-03_rb,0.51080e-03_rb,0.10249e-03_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.10970e-02_rb,0.11491e-02_rb,0.10850e-02_rb,0.99037e-03_rb,0.87478e-03_rb, &
        & 0.73644e-03_rb,0.58168e-03_rb,0.38673e-03_rb,0.55328e-04_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.11051e-02_rb,0.11716e-02_rb,0.11129e-02_rb,0.10196e-02_rb,0.90431e-03_rb, &
        & 0.76445e-03_rb,0.60553e-03_rb,0.40600e-03_rb,0.61413e-04_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.11053e-02_rb,0.11877e-02_rb,0.11365e-02_rb,0.10446e-02_rb,0.92981e-03_rb, &
        & 0.79114e-03_rb,0.62919e-03_rb,0.42419e-03_rb,0.68340e-04_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.11000e-02_rb,0.12008e-02_rb,0.11557e-02_rb,0.10648e-02_rb,0.95310e-03_rb, &
        & 0.81721e-03_rb,0.65273e-03_rb,0.44128e-03_rb,0.75425e-04_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.10930e-02_rb,0.12101e-02_rb,0.11704e-02_rb,0.10855e-02_rb,0.97521e-03_rb, &
        & 0.84232e-03_rb,0.67516e-03_rb,0.45733e-03_rb,0.82112e-04_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.96254e-03_rb,0.10041e-02_rb,0.94496e-03_rb,0.86305e-03_rb,0.76515e-03_rb, &
        & 0.64666e-03_rb,0.50949e-03_rb,0.34237e-03_rb,0.45743e-04_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.97168e-03_rb,0.10257e-02_rb,0.97306e-03_rb,0.89241e-03_rb,0.79300e-03_rb, &
        & 0.67315e-03_rb,0.53212e-03_rb,0.35953e-03_rb,0.50936e-04_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.97307e-03_rb,0.10417e-02_rb,0.99540e-03_rb,0.91663e-03_rb,0.81739e-03_rb, &
        & 0.69764e-03_rb,0.55388e-03_rb,0.37625e-03_rb,0.56704e-04_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.97112e-03_rb,0.10538e-02_rb,0.10130e-02_rb,0.93600e-03_rb,0.83875e-03_rb, &
        & 0.72048e-03_rb,0.57540e-03_rb,0.39219e-03_rb,0.62214e-04_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.96524e-03_rb,0.10634e-02_rb,0.10272e-02_rb,0.95339e-03_rb,0.85814e-03_rb, &
        & 0.74256e-03_rb,0.59649e-03_rb,0.40723e-03_rb,0.68154e-04_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.83977e-03_rb,0.87150e-03_rb,0.81798e-03_rb,0.74510e-03_rb,0.66073e-03_rb, &
        & 0.55889e-03_rb,0.44038e-03_rb,0.30023e-03_rb,0.39438e-04_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.84860e-03_rb,0.89183e-03_rb,0.84311e-03_rb,0.77202e-03_rb,0.68677e-03_rb, &
        & 0.58412e-03_rb,0.46261e-03_rb,0.31513e-03_rb,0.44026e-04_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.85156e-03_rb,0.90640e-03_rb,0.86357e-03_rb,0.79472e-03_rb,0.70843e-03_rb, &
        & 0.60616e-03_rb,0.48333e-03_rb,0.32962e-03_rb,0.48169e-04_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.84996e-03_rb,0.91804e-03_rb,0.88000e-03_rb,0.81332e-03_rb,0.72741e-03_rb, &
        & 0.62668e-03_rb,0.50295e-03_rb,0.34425e-03_rb,0.52632e-04_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.84411e-03_rb,0.92502e-03_rb,0.89195e-03_rb,0.82771e-03_rb,0.74479e-03_rb, &
        & 0.64630e-03_rb,0.52177e-03_rb,0.35838e-03_rb,0.57490e-04_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.72566e-03_rb,0.75029e-03_rb,0.70410e-03_rb,0.64085e-03_rb,0.56881e-03_rb, &
        & 0.48163e-03_rb,0.37585e-03_rb,0.26029e-03_rb,0.35173e-04_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.73526e-03_rb,0.76878e-03_rb,0.72724e-03_rb,0.66569e-03_rb,0.59239e-03_rb, &
        & 0.50357e-03_rb,0.39670e-03_rb,0.27234e-03_rb,0.38442e-04_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.73942e-03_rb,0.78240e-03_rb,0.74617e-03_rb,0.68580e-03_rb,0.61175e-03_rb, &
        & 0.52308e-03_rb,0.41633e-03_rb,0.28548e-03_rb,0.42620e-04_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.73787e-03_rb,0.79331e-03_rb,0.76108e-03_rb,0.70193e-03_rb,0.62821e-03_rb, &
        & 0.54092e-03_rb,0.43381e-03_rb,0.29821e-03_rb,0.47013e-04_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.73365e-03_rb,0.79895e-03_rb,0.77125e-03_rb,0.71463e-03_rb,0.64295e-03_rb, &
        & 0.55753e-03_rb,0.45033e-03_rb,0.31091e-03_rb,0.51519e-04_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.62112e-03_rb,0.63900e-03_rb,0.60071e-03_rb,0.54641e-03_rb,0.48585e-03_rb, &
        & 0.41219e-03_rb,0.31988e-03_rb,0.22088e-03_rb,0.39225e-04_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.63089e-03_rb,0.65678e-03_rb,0.62144e-03_rb,0.56855e-03_rb,0.50663e-03_rb, &
        & 0.43158e-03_rb,0.33912e-03_rb,0.23261e-03_rb,0.43145e-04_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.63630e-03_rb,0.67023e-03_rb,0.63841e-03_rb,0.58635e-03_rb,0.52398e-03_rb, &
        & 0.44876e-03_rb,0.35619e-03_rb,0.24341e-03_rb,0.47182e-04_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.63678e-03_rb,0.68001e-03_rb,0.65143e-03_rb,0.60045e-03_rb,0.53807e-03_rb, &
        & 0.46446e-03_rb,0.37160e-03_rb,0.25517e-03_rb,0.51310e-04_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.63384e-03_rb,0.68558e-03_rb,0.66049e-03_rb,0.61133e-03_rb,0.55067e-03_rb, &
        & 0.47828e-03_rb,0.38622e-03_rb,0.26607e-03_rb,0.55652e-04_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.52818e-03_rb,0.54302e-03_rb,0.51109e-03_rb,0.46524e-03_rb,0.41303e-03_rb, &
        & 0.35089e-03_rb,0.27050e-03_rb,0.18719e-03_rb,0.51897e-04_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.53874e-03_rb,0.56000e-03_rb,0.52974e-03_rb,0.48429e-03_rb,0.43151e-03_rb, &
        & 0.36826e-03_rb,0.28749e-03_rb,0.19624e-03_rb,0.57480e-04_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.54407e-03_rb,0.57298e-03_rb,0.54499e-03_rb,0.50007e-03_rb,0.44685e-03_rb, &
        & 0.38282e-03_rb,0.30218e-03_rb,0.20623e-03_rb,0.63250e-04_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.54510e-03_rb,0.58221e-03_rb,0.55605e-03_rb,0.51259e-03_rb,0.45923e-03_rb, &
        & 0.39550e-03_rb,0.31547e-03_rb,0.21632e-03_rb,0.68339e-04_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.54335e-03_rb,0.58744e-03_rb,0.56412e-03_rb,0.52185e-03_rb,0.47008e-03_rb, &
        & 0.40728e-03_rb,0.32798e-03_rb,0.22618e-03_rb,0.72964e-04_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.44901e-03_rb,0.46427e-03_rb,0.43915e-03_rb,0.40187e-03_rb,0.35719e-03_rb, &
        & 0.30380e-03_rb,0.23427e-03_rb,0.15956e-03_rb,0.52228e-04_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.45685e-03_rb,0.47845e-03_rb,0.45475e-03_rb,0.41772e-03_rb,0.37275e-03_rb, &
        & 0.31798e-03_rb,0.24772e-03_rb,0.16817e-03_rb,0.56979e-04_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.46005e-03_rb,0.48879e-03_rb,0.46665e-03_rb,0.43038e-03_rb,0.38526e-03_rb, &
        & 0.32982e-03_rb,0.25983e-03_rb,0.17707e-03_rb,0.61788e-04_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.46029e-03_rb,0.49540e-03_rb,0.47547e-03_rb,0.44015e-03_rb,0.39539e-03_rb, &
        & 0.34025e-03_rb,0.27087e-03_rb,0.18596e-03_rb,0.66537e-04_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.45760e-03_rb,0.49879e-03_rb,0.48175e-03_rb,0.44812e-03_rb,0.40429e-03_rb, &
        & 0.34971e-03_rb,0.28105e-03_rb,0.19422e-03_rb,0.70982e-04_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.37789e-03_rb,0.39359e-03_rb,0.37439e-03_rb,0.34335e-03_rb,0.30516e-03_rb, &
        & 0.25987e-03_rb,0.20136e-03_rb,0.13616e-03_rb,0.47471e-04_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.38345e-03_rb,0.40460e-03_rb,0.38678e-03_rb,0.35616e-03_rb,0.31793e-03_rb, &
        & 0.27155e-03_rb,0.21219e-03_rb,0.14366e-03_rb,0.51576e-04_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.38556e-03_rb,0.41241e-03_rb,0.39583e-03_rb,0.36622e-03_rb,0.32808e-03_rb, &
        & 0.28132e-03_rb,0.22259e-03_rb,0.15133e-03_rb,0.55529e-04_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.38464e-03_rb,0.41687e-03_rb,0.40247e-03_rb,0.37394e-03_rb,0.33650e-03_rb, &
        & 0.28994e-03_rb,0.23157e-03_rb,0.15870e-03_rb,0.59557e-04_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.38119e-03_rb,0.41880e-03_rb,0.40725e-03_rb,0.38009e-03_rb,0.34354e-03_rb, &
        & 0.29781e-03_rb,0.23974e-03_rb,0.16572e-03_rb,0.63195e-04_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.31518e-03_rb,0.33080e-03_rb,0.31644e-03_rb,0.29051e-03_rb,0.25824e-03_rb, &
        & 0.21997e-03_rb,0.17074e-03_rb,0.11585e-03_rb,0.39419e-04_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.31945e-03_rb,0.33947e-03_rb,0.32620e-03_rb,0.30064e-03_rb,0.26839e-03_rb, &
        & 0.22968e-03_rb,0.17999e-03_rb,0.12241e-03_rb,0.42717e-04_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.32042e-03_rb,0.34518e-03_rb,0.33311e-03_rb,0.30844e-03_rb,0.27653e-03_rb, &
        & 0.23733e-03_rb,0.18840e-03_rb,0.12892e-03_rb,0.45799e-04_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.31877e-03_rb,0.34829e-03_rb,0.33813e-03_rb,0.31466e-03_rb,0.28316e-03_rb, &
        & 0.24429e-03_rb,0.19577e-03_rb,0.13500e-03_rb,0.49095e-04_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.31533e-03_rb,0.34943e-03_rb,0.34164e-03_rb,0.31953e-03_rb,0.28883e-03_rb, &
        & 0.25064e-03_rb,0.20247e-03_rb,0.14079e-03_rb,0.52225e-04_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.30073e-02_rb,0.30215e-02_rb,0.28549e-02_rb,0.26145e-02_rb,0.23223e-02_rb, &
        & 0.19698e-02_rb,0.15631e-02_rb,0.10671e-02_rb,0.34349e-03_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.30045e-02_rb,0.30464e-02_rb,0.29000e-02_rb,0.26682e-02_rb,0.23795e-02_rb, &
        & 0.20320e-02_rb,0.16241e-02_rb,0.11180e-02_rb,0.37241e-03_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.29924e-02_rb,0.30694e-02_rb,0.29376e-02_rb,0.27170e-02_rb,0.24327e-02_rb, &
        & 0.20955e-02_rb,0.16828e-02_rb,0.11692e-02_rb,0.41183e-03_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.29732e-02_rb,0.30875e-02_rb,0.29723e-02_rb,0.27617e-02_rb,0.24862e-02_rb, &
        & 0.21555e-02_rb,0.17426e-02_rb,0.12186e-02_rb,0.45672e-03_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.29511e-02_rb,0.31004e-02_rb,0.30014e-02_rb,0.28043e-02_rb,0.25375e-02_rb, &
        & 0.22174e-02_rb,0.18018e-02_rb,0.12656e-02_rb,0.49848e-03_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.28011e-02_rb,0.28061e-02_rb,0.26513e-02_rb,0.24171e-02_rb,0.21391e-02_rb, &
        & 0.18145e-02_rb,0.14290e-02_rb,0.97808e-03_rb,0.28283e-03_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.28038e-02_rb,0.28333e-02_rb,0.26971e-02_rb,0.24708e-02_rb,0.21906e-02_rb, &
        & 0.18715e-02_rb,0.14859e-02_rb,0.10236e-02_rb,0.31088e-03_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.27978e-02_rb,0.28587e-02_rb,0.27386e-02_rb,0.25159e-02_rb,0.22406e-02_rb, &
        & 0.19256e-02_rb,0.15398e-02_rb,0.10695e-02_rb,0.34605e-03_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.27864e-02_rb,0.28782e-02_rb,0.27737e-02_rb,0.25561e-02_rb,0.22888e-02_rb, &
        & 0.19778e-02_rb,0.15963e-02_rb,0.11144e-02_rb,0.38043e-03_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.27682e-02_rb,0.28941e-02_rb,0.28027e-02_rb,0.25952e-02_rb,0.23337e-02_rb, &
        & 0.20308e-02_rb,0.16518e-02_rb,0.11583e-02_rb,0.41249e-03_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.26013e-02_rb,0.25925e-02_rb,0.24368e-02_rb,0.22059e-02_rb,0.19447e-02_rb, &
        & 0.16443e-02_rb,0.12895e-02_rb,0.88356e-03_rb,0.22980e-03_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.26118e-02_rb,0.26228e-02_rb,0.24826e-02_rb,0.22596e-02_rb,0.19971e-02_rb, &
        & 0.16971e-02_rb,0.13425e-02_rb,0.92292e-03_rb,0.25666e-03_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.26121e-02_rb,0.26511e-02_rb,0.25221e-02_rb,0.23045e-02_rb,0.20442e-02_rb, &
        & 0.17465e-02_rb,0.13937e-02_rb,0.96423e-03_rb,0.28307e-03_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.26050e-02_rb,0.26757e-02_rb,0.25536e-02_rb,0.23446e-02_rb,0.20883e-02_rb, &
        & 0.17925e-02_rb,0.14453e-02_rb,0.10074e-02_rb,0.30818e-03_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.25933e-02_rb,0.26944e-02_rb,0.25823e-02_rb,0.23794e-02_rb,0.21318e-02_rb, &
        & 0.18378e-02_rb,0.14965e-02_rb,0.10492e-02_rb,0.33812e-03_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.24078e-02_rb,0.23894e-02_rb,0.22370e-02_rb,0.20125e-02_rb,0.17599e-02_rb, &
        & 0.14763e-02_rb,0.11554e-02_rb,0.78494e-03_rb,0.18813e-03_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.24225e-02_rb,0.24253e-02_rb,0.22813e-02_rb,0.20638e-02_rb,0.18093e-02_rb, &
        & 0.15251e-02_rb,0.12031e-02_rb,0.82373e-03_rb,0.21030e-03_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.24294e-02_rb,0.24564e-02_rb,0.23198e-02_rb,0.21085e-02_rb,0.18532e-02_rb, &
        & 0.15708e-02_rb,0.12482e-02_rb,0.86229e-03_rb,0.23129e-03_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.24309e-02_rb,0.24808e-02_rb,0.23546e-02_rb,0.21467e-02_rb,0.18923e-02_rb, &
        & 0.16129e-02_rb,0.12944e-02_rb,0.90119e-03_rb,0.25224e-03_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.24223e-02_rb,0.24983e-02_rb,0.23839e-02_rb,0.21783e-02_rb,0.19305e-02_rb, &
        & 0.16540e-02_rb,0.13401e-02_rb,0.94049e-03_rb,0.27779e-03_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.22071e-02_rb,0.21851e-02_rb,0.20436e-02_rb,0.18357e-02_rb,0.15952e-02_rb, &
        & 0.13304e-02_rb,0.10307e-02_rb,0.69163e-03_rb,0.14983e-03_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.22284e-02_rb,0.22243e-02_rb,0.20887e-02_rb,0.18839e-02_rb,0.16450e-02_rb, &
        & 0.13762e-02_rb,0.10736e-02_rb,0.72602e-03_rb,0.16732e-03_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.22426e-02_rb,0.22593e-02_rb,0.21270e-02_rb,0.19281e-02_rb,0.16889e-02_rb, &
        & 0.14169e-02_rb,0.11148e-02_rb,0.76201e-03_rb,0.18420e-03_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.22476e-02_rb,0.22863e-02_rb,0.21609e-02_rb,0.19676e-02_rb,0.17258e-02_rb, &
        & 0.14525e-02_rb,0.11537e-02_rb,0.79815e-03_rb,0.20325e-03_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.22399e-02_rb,0.23048e-02_rb,0.21912e-02_rb,0.19974e-02_rb,0.17586e-02_rb, &
        & 0.14880e-02_rb,0.11934e-02_rb,0.83564e-03_rb,0.22470e-03_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.20075e-02_rb,0.19795e-02_rb,0.18447e-02_rb,0.16541e-02_rb,0.14341e-02_rb, &
        & 0.11951e-02_rb,0.92238e-03_rb,0.60660e-03_rb,0.12381e-03_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.20344e-02_rb,0.20227e-02_rb,0.18912e-02_rb,0.17028e-02_rb,0.14828e-02_rb, &
        & 0.12404e-02_rb,0.96234e-03_rb,0.63833e-03_rb,0.13822e-03_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.20513e-02_rb,0.20598e-02_rb,0.19311e-02_rb,0.17466e-02_rb,0.15267e-02_rb, &
        & 0.12805e-02_rb,0.99815e-03_rb,0.66987e-03_rb,0.15138e-03_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.20576e-02_rb,0.20878e-02_rb,0.19661e-02_rb,0.17857e-02_rb,0.15647e-02_rb, &
        & 0.13147e-02_rb,0.10319e-02_rb,0.70266e-03_rb,0.16753e-03_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.20521e-02_rb,0.21067e-02_rb,0.19938e-02_rb,0.18157e-02_rb,0.15969e-02_rb, &
        & 0.13475e-02_rb,0.10644e-02_rb,0.73524e-03_rb,0.18430e-03_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.17951e-02_rb,0.17708e-02_rb,0.16476e-02_rb,0.14764e-02_rb,0.12769e-02_rb, &
        & 0.10613e-02_rb,0.82141e-03_rb,0.53387e-03_rb,0.10356e-03_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.18257e-02_rb,0.18161e-02_rb,0.16970e-02_rb,0.15264e-02_rb,0.13250e-02_rb, &
        & 0.11052e-02_rb,0.85771e-03_rb,0.56341e-03_rb,0.11419e-03_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.18457e-02_rb,0.18541e-02_rb,0.17382e-02_rb,0.15700e-02_rb,0.13685e-02_rb, &
        & 0.11444e-02_rb,0.89062e-03_rb,0.59265e-03_rb,0.12737e-03_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.18534e-02_rb,0.18833e-02_rb,0.17713e-02_rb,0.16066e-02_rb,0.14063e-02_rb, &
        & 0.11787e-02_rb,0.92220e-03_rb,0.62067e-03_rb,0.14148e-03_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.18508e-02_rb,0.19025e-02_rb,0.17986e-02_rb,0.16377e-02_rb,0.14382e-02_rb, &
        & 0.12095e-02_rb,0.95313e-03_rb,0.64883e-03_rb,0.15624e-03_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.15853e-02_rb,0.15606e-02_rb,0.14491e-02_rb,0.13005e-02_rb,0.11253e-02_rb, &
        & 0.93388e-03_rb,0.72631e-03_rb,0.46832e-03_rb,0.90402e-04_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.16191e-02_rb,0.16074e-02_rb,0.14980e-02_rb,0.13497e-02_rb,0.11730e-02_rb, &
        & 0.97691e-03_rb,0.76027e-03_rb,0.49835e-03_rb,0.10124e-03_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.16393e-02_rb,0.16464e-02_rb,0.15392e-02_rb,0.13927e-02_rb,0.12147e-02_rb, &
        & 0.10153e-02_rb,0.79129e-03_rb,0.52537e-03_rb,0.11276e-03_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.16487e-02_rb,0.16729e-02_rb,0.15721e-02_rb,0.14286e-02_rb,0.12504e-02_rb, &
        & 0.10480e-02_rb,0.82072e-03_rb,0.55187e-03_rb,0.12515e-03_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.16472e-02_rb,0.16924e-02_rb,0.15977e-02_rb,0.14588e-02_rb,0.12817e-02_rb, &
        & 0.10777e-02_rb,0.84873e-03_rb,0.57800e-03_rb,0.13784e-03_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.13936e-02_rb,0.13674e-02_rb,0.12635e-02_rb,0.11331e-02_rb,0.97871e-03_rb, &
        & 0.81331e-03_rb,0.63480e-03_rb,0.40910e-03_rb,0.92222e-04_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.14271e-02_rb,0.14126e-02_rb,0.13114e-02_rb,0.11804e-02_rb,0.10248e-02_rb, &
        & 0.85405e-03_rb,0.66828e-03_rb,0.43677e-03_rb,0.10207e-03_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.14461e-02_rb,0.14478e-02_rb,0.13500e-02_rb,0.12219e-02_rb,0.10647e-02_rb, &
        & 0.89058e-03_rb,0.69816e-03_rb,0.46305e-03_rb,0.11262e-03_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.14547e-02_rb,0.14726e-02_rb,0.13807e-02_rb,0.12557e-02_rb,0.10996e-02_rb, &
        & 0.92247e-03_rb,0.72520e-03_rb,0.48654e-03_rb,0.12376e-03_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.14563e-02_rb,0.14902e-02_rb,0.14046e-02_rb,0.12841e-02_rb,0.11288e-02_rb, &
        & 0.95105e-03_rb,0.75065e-03_rb,0.51065e-03_rb,0.13556e-03_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.12161e-02_rb,0.11928e-02_rb,0.11004e-02_rb,0.98725e-03_rb,0.85318e-03_rb, &
        & 0.70775e-03_rb,0.55081e-03_rb,0.35481e-03_rb,0.12422e-03_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.12463e-02_rb,0.12352e-02_rb,0.11444e-02_rb,0.10316e-02_rb,0.89480e-03_rb, &
        & 0.74560e-03_rb,0.58282e-03_rb,0.38195e-03_rb,0.13273e-03_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.12646e-02_rb,0.12662e-02_rb,0.11796e-02_rb,0.10687e-02_rb,0.93096e-03_rb, &
        & 0.77883e-03_rb,0.61138e-03_rb,0.40526e-03_rb,0.14193e-03_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.12733e-02_rb,0.12880e-02_rb,0.12074e-02_rb,0.10979e-02_rb,0.96169e-03_rb, &
        & 0.80809e-03_rb,0.63689e-03_rb,0.42724e-03_rb,0.15291e-03_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.12740e-02_rb,0.13044e-02_rb,0.12293e-02_rb,0.11231e-02_rb,0.98752e-03_rb, &
        & 0.83380e-03_rb,0.66030e-03_rb,0.44847e-03_rb,0.16460e-03_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.10656e-02_rb,0.10497e-02_rb,0.96881e-03_rb,0.87108e-03_rb,0.75627e-03_rb, &
        & 0.62893e-03_rb,0.48955e-03_rb,0.31741e-03_rb,0.13016e-03_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.10879e-02_rb,0.10821e-02_rb,0.10046e-02_rb,0.90804e-03_rb,0.79130e-03_rb, &
        & 0.66235e-03_rb,0.51807e-03_rb,0.34003e-03_rb,0.13779e-03_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.11001e-02_rb,0.11057e-02_rb,0.10327e-02_rb,0.93770e-03_rb,0.82120e-03_rb, &
        & 0.69087e-03_rb,0.54346e-03_rb,0.36047e-03_rb,0.14411e-03_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.11047e-02_rb,0.11226e-02_rb,0.10548e-02_rb,0.96212e-03_rb,0.84667e-03_rb, &
        & 0.71575e-03_rb,0.56548e-03_rb,0.37954e-03_rb,0.15311e-03_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.11028e-02_rb,0.11345e-02_rb,0.10715e-02_rb,0.98143e-03_rb,0.86845e-03_rb, &
        & 0.73791e-03_rb,0.58600e-03_rb,0.39874e-03_rb,0.16257e-03_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.92541e-03_rb,0.91485e-03_rb,0.84575e-03_rb,0.76287e-03_rb,0.66530e-03_rb, &
        & 0.55435e-03_rb,0.43055e-03_rb,0.28106e-03_rb,0.11779e-03_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.94153e-03_rb,0.94015e-03_rb,0.87425e-03_rb,0.79221e-03_rb,0.69422e-03_rb, &
        & 0.58249e-03_rb,0.45555e-03_rb,0.30043e-03_rb,0.12541e-03_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.95043e-03_rb,0.95859e-03_rb,0.89645e-03_rb,0.81618e-03_rb,0.71892e-03_rb, &
        & 0.60660e-03_rb,0.47691e-03_rb,0.31810e-03_rb,0.13086e-03_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.95300e-03_rb,0.97180e-03_rb,0.91414e-03_rb,0.83540e-03_rb,0.73965e-03_rb, &
        & 0.62755e-03_rb,0.49627e-03_rb,0.33494e-03_rb,0.13792e-03_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.94924e-03_rb,0.98033e-03_rb,0.92629e-03_rb,0.85073e-03_rb,0.75722e-03_rb, &
        & 0.64562e-03_rb,0.51410e-03_rb,0.35114e-03_rb,0.14650e-03_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.79344e-03_rb,0.78971e-03_rb,0.73267e-03_rb,0.66405e-03_rb,0.58083e-03_rb, &
        & 0.48392e-03_rb,0.37549e-03_rb,0.24569e-03_rb,0.97034e-04_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.80574e-03_rb,0.81004e-03_rb,0.75547e-03_rb,0.68809e-03_rb,0.60484e-03_rb, &
        & 0.50707e-03_rb,0.39618e-03_rb,0.26227e-03_rb,0.10267e-03_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.81227e-03_rb,0.82486e-03_rb,0.77408e-03_rb,0.70758e-03_rb,0.62511e-03_rb, &
        & 0.52731e-03_rb,0.41439e-03_rb,0.27759e-03_rb,0.10742e-03_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.81298e-03_rb,0.83538e-03_rb,0.78743e-03_rb,0.72262e-03_rb,0.64193e-03_rb, &
        & 0.54449e-03_rb,0.43080e-03_rb,0.29211e-03_rb,0.11325e-03_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.80893e-03_rb,0.84184e-03_rb,0.79706e-03_rb,0.73516e-03_rb,0.65638e-03_rb, &
        & 0.55984e-03_rb,0.44607e-03_rb,0.30621e-03_rb,0.12059e-03_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.54732e-02_rb,0.53034e-02_rb,0.49969e-02_rb,0.45431e-02_rb,0.40134e-02_rb, &
        & 0.34010e-02_rb,0.26972e-02_rb,0.18637e-02_rb,0.96692e-03_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.54607e-02_rb,0.53367e-02_rb,0.50546e-02_rb,0.46321e-02_rb,0.41099e-02_rb, &
        & 0.34970e-02_rb,0.27999e-02_rb,0.19729e-02_rb,0.11063e-02_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.54405e-02_rb,0.53637e-02_rb,0.51145e-02_rb,0.47117e-02_rb,0.41981e-02_rb, &
        & 0.35927e-02_rb,0.29070e-02_rb,0.20850e-02_rb,0.12360e-02_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.54052e-02_rb,0.53858e-02_rb,0.51679e-02_rb,0.47829e-02_rb,0.42844e-02_rb, &
        & 0.36942e-02_rb,0.30161e-02_rb,0.22030e-02_rb,0.13785e-02_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.53549e-02_rb,0.54029e-02_rb,0.52131e-02_rb,0.48529e-02_rb,0.43733e-02_rb, &
        & 0.37946e-02_rb,0.31262e-02_rb,0.23268e-02_rb,0.15336e-02_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.52482e-02_rb,0.50940e-02_rb,0.47786e-02_rb,0.43454e-02_rb,0.38186e-02_rb, &
        & 0.32129e-02_rb,0.25325e-02_rb,0.17227e-02_rb,0.80430e-03_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.52403e-02_rb,0.51313e-02_rb,0.48370e-02_rb,0.44315e-02_rb,0.39109e-02_rb, &
        & 0.33052e-02_rb,0.26254e-02_rb,0.18226e-02_rb,0.90614e-03_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.52188e-02_rb,0.51614e-02_rb,0.48948e-02_rb,0.45073e-02_rb,0.39963e-02_rb, &
        & 0.33961e-02_rb,0.27198e-02_rb,0.19222e-02_rb,0.10146e-02_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.51826e-02_rb,0.51837e-02_rb,0.49460e-02_rb,0.45770e-02_rb,0.40791e-02_rb, &
        & 0.34891e-02_rb,0.28146e-02_rb,0.20278e-02_rb,0.11325e-02_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.51359e-02_rb,0.51960e-02_rb,0.49902e-02_rb,0.46430e-02_rb,0.41602e-02_rb, &
        & 0.35811e-02_rb,0.29127e-02_rb,0.21431e-02_rb,0.12633e-02_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.49876e-02_rb,0.48388e-02_rb,0.45133e-02_rb,0.40992e-02_rb,0.35901e-02_rb, &
        & 0.30030e-02_rb,0.23527e-02_rb,0.15765e-02_rb,0.64714e-03_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.49850e-02_rb,0.48808e-02_rb,0.45736e-02_rb,0.41821e-02_rb,0.36790e-02_rb, &
        & 0.30941e-02_rb,0.24388e-02_rb,0.16655e-02_rb,0.72927e-03_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.49660e-02_rb,0.49117e-02_rb,0.46346e-02_rb,0.42593e-02_rb,0.37631e-02_rb, &
        & 0.31820e-02_rb,0.25243e-02_rb,0.17543e-02_rb,0.81771e-03_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.49361e-02_rb,0.49307e-02_rb,0.46909e-02_rb,0.43285e-02_rb,0.38459e-02_rb, &
        & 0.32679e-02_rb,0.26083e-02_rb,0.18450e-02_rb,0.91578e-03_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.48943e-02_rb,0.49413e-02_rb,0.47366e-02_rb,0.43959e-02_rb,0.39239e-02_rb, &
        & 0.33496e-02_rb,0.26941e-02_rb,0.19416e-02_rb,0.10197e-02_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.46848e-02_rb,0.45423e-02_rb,0.42109e-02_rb,0.38147e-02_rb,0.33318e-02_rb, &
        & 0.27806e-02_rb,0.21628e-02_rb,0.14435e-02_rb,0.52019e-03_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.46885e-02_rb,0.45846e-02_rb,0.42767e-02_rb,0.38967e-02_rb,0.34228e-02_rb, &
        & 0.28698e-02_rb,0.22469e-02_rb,0.15180e-02_rb,0.58537e-03_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.46784e-02_rb,0.46158e-02_rb,0.43389e-02_rb,0.39718e-02_rb,0.35088e-02_rb, &
        & 0.29561e-02_rb,0.23285e-02_rb,0.15932e-02_rb,0.65783e-03_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.46546e-02_rb,0.46379e-02_rb,0.43907e-02_rb,0.40421e-02_rb,0.35928e-02_rb, &
        & 0.30394e-02_rb,0.24053e-02_rb,0.16703e-02_rb,0.73788e-03_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.46218e-02_rb,0.46560e-02_rb,0.44359e-02_rb,0.41109e-02_rb,0.36691e-02_rb, &
        & 0.31156e-02_rb,0.24821e-02_rb,0.17515e-02_rb,0.82261e-03_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.43680e-02_rb,0.42271e-02_rb,0.38928e-02_rb,0.35080e-02_rb,0.30576e-02_rb, &
        & 0.25405e-02_rb,0.19725e-02_rb,0.13117e-02_rb,0.42455e-03_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.43836e-02_rb,0.42742e-02_rb,0.39603e-02_rb,0.35901e-02_rb,0.31440e-02_rb, &
        & 0.26294e-02_rb,0.20540e-02_rb,0.13812e-02_rb,0.47676e-03_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.43831e-02_rb,0.43074e-02_rb,0.40201e-02_rb,0.36636e-02_rb,0.32261e-02_rb, &
        & 0.27164e-02_rb,0.21330e-02_rb,0.14464e-02_rb,0.53568e-03_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.43690e-02_rb,0.43343e-02_rb,0.40742e-02_rb,0.37289e-02_rb,0.33089e-02_rb, &
        & 0.28017e-02_rb,0.22054e-02_rb,0.15130e-02_rb,0.59880e-03_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.43473e-02_rb,0.43575e-02_rb,0.41195e-02_rb,0.37951e-02_rb,0.33885e-02_rb, &
        & 0.28774e-02_rb,0.22767e-02_rb,0.15816e-02_rb,0.66710e-03_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.40714e-02_rb,0.39122e-02_rb,0.35777e-02_rb,0.32091e-02_rb,0.27872e-02_rb, &
        & 0.22994e-02_rb,0.17735e-02_rb,0.11790e-02_rb,0.33561e-03_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.40973e-02_rb,0.39625e-02_rb,0.36472e-02_rb,0.32877e-02_rb,0.28694e-02_rb, &
        & 0.23826e-02_rb,0.18515e-02_rb,0.12426e-02_rb,0.37756e-03_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.41108e-02_rb,0.40033e-02_rb,0.37116e-02_rb,0.33595e-02_rb,0.29464e-02_rb, &
        & 0.24646e-02_rb,0.19290e-02_rb,0.13037e-02_rb,0.42584e-03_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.41078e-02_rb,0.40382e-02_rb,0.37677e-02_rb,0.34232e-02_rb,0.30216e-02_rb, &
        & 0.25463e-02_rb,0.20016e-02_rb,0.13624e-02_rb,0.47733e-03_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.40964e-02_rb,0.40666e-02_rb,0.38151e-02_rb,0.34890e-02_rb,0.30973e-02_rb, &
        & 0.26200e-02_rb,0.20705e-02_rb,0.14239e-02_rb,0.53330e-03_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.37806e-02_rb,0.36256e-02_rb,0.32982e-02_rb,0.29389e-02_rb,0.25345e-02_rb, &
        & 0.20760e-02_rb,0.15879e-02_rb,0.10489e-02_rb,0.28018e-03_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.38204e-02_rb,0.36872e-02_rb,0.33713e-02_rb,0.30154e-02_rb,0.26103e-02_rb, &
        & 0.21557e-02_rb,0.16619e-02_rb,0.11071e-02_rb,0.31519e-03_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.38460e-02_rb,0.37374e-02_rb,0.34369e-02_rb,0.30817e-02_rb,0.26834e-02_rb, &
        & 0.22319e-02_rb,0.17344e-02_rb,0.11636e-02_rb,0.35270e-03_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.38578e-02_rb,0.37771e-02_rb,0.34946e-02_rb,0.31444e-02_rb,0.27541e-02_rb, &
        & 0.23067e-02_rb,0.18034e-02_rb,0.12192e-02_rb,0.39298e-03_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.38608e-02_rb,0.38134e-02_rb,0.35421e-02_rb,0.32035e-02_rb,0.28239e-02_rb, &
        & 0.23777e-02_rb,0.18670e-02_rb,0.12748e-02_rb,0.43632e-03_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.34769e-02_rb,0.33291e-02_rb,0.30196e-02_rb,0.26781e-02_rb,0.23024e-02_rb, &
        & 0.18824e-02_rb,0.14244e-02_rb,0.92897e-03_rb,0.24101e-03_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.35239e-02_rb,0.33972e-02_rb,0.30964e-02_rb,0.27587e-02_rb,0.23816e-02_rb, &
        & 0.19578e-02_rb,0.14914e-02_rb,0.98181e-03_rb,0.27130e-03_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.35607e-02_rb,0.34526e-02_rb,0.31656e-02_rb,0.28309e-02_rb,0.24544e-02_rb, &
        & 0.20284e-02_rb,0.15562e-02_rb,0.10333e-02_rb,0.30255e-03_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.35844e-02_rb,0.35023e-02_rb,0.32271e-02_rb,0.28968e-02_rb,0.25226e-02_rb, &
        & 0.20957e-02_rb,0.16188e-02_rb,0.10830e-02_rb,0.33737e-03_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.35961e-02_rb,0.35446e-02_rb,0.32825e-02_rb,0.29568e-02_rb,0.25866e-02_rb, &
        & 0.21594e-02_rb,0.16782e-02_rb,0.11324e-02_rb,0.37448e-03_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.31490e-02_rb,0.30162e-02_rb,0.27350e-02_rb,0.24183e-02_rb,0.20753e-02_rb, &
        & 0.16954e-02_rb,0.12750e-02_rb,0.82561e-03_rb,0.24776e-03_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.32098e-02_rb,0.30913e-02_rb,0.28147e-02_rb,0.25006e-02_rb,0.21544e-02_rb, &
        & 0.17699e-02_rb,0.13387e-02_rb,0.87243e-03_rb,0.27735e-03_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.32582e-02_rb,0.31563e-02_rb,0.28883e-02_rb,0.25742e-02_rb,0.22278e-02_rb, &
        & 0.18394e-02_rb,0.14003e-02_rb,0.91830e-03_rb,0.30712e-03_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.32928e-02_rb,0.32146e-02_rb,0.29543e-02_rb,0.26421e-02_rb,0.22954e-02_rb, &
        & 0.19038e-02_rb,0.14592e-02_rb,0.96296e-03_rb,0.33982e-03_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.33113e-02_rb,0.32637e-02_rb,0.30129e-02_rb,0.27034e-02_rb,0.23582e-02_rb, &
        & 0.19652e-02_rb,0.15140e-02_rb,0.10071e-02_rb,0.37422e-03_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.28236e-02_rb,0.26993e-02_rb,0.24463e-02_rb,0.21584e-02_rb,0.18515e-02_rb, &
        & 0.15144e-02_rb,0.11396e-02_rb,0.73456e-03_rb,0.30599e-03_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.28929e-02_rb,0.27796e-02_rb,0.25302e-02_rb,0.22416e-02_rb,0.19311e-02_rb, &
        & 0.15864e-02_rb,0.12002e-02_rb,0.77955e-03_rb,0.32947e-03_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.29497e-02_rb,0.28523e-02_rb,0.26078e-02_rb,0.23174e-02_rb,0.20043e-02_rb, &
        & 0.16542e-02_rb,0.12577e-02_rb,0.82291e-03_rb,0.35529e-03_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.29901e-02_rb,0.29165e-02_rb,0.26763e-02_rb,0.23877e-02_rb,0.20731e-02_rb, &
        & 0.17177e-02_rb,0.13124e-02_rb,0.86431e-03_rb,0.38167e-03_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.30139e-02_rb,0.29672e-02_rb,0.27348e-02_rb,0.24509e-02_rb,0.21349e-02_rb, &
        & 0.17760e-02_rb,0.13646e-02_rb,0.90496e-03_rb,0.40993e-03_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.25422e-02_rb,0.24347e-02_rb,0.22108e-02_rb,0.19505e-02_rb,0.16727e-02_rb, &
        & 0.13714e-02_rb,0.10368e-02_rb,0.66936e-03_rb,0.31350e-03_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.26126e-02_rb,0.25184e-02_rb,0.22944e-02_rb,0.20307e-02_rb,0.17483e-02_rb, &
        & 0.14377e-02_rb,0.10928e-02_rb,0.71181e-03_rb,0.33782e-03_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.26690e-02_rb,0.25901e-02_rb,0.23690e-02_rb,0.21043e-02_rb,0.18188e-02_rb, &
        & 0.15016e-02_rb,0.11462e-02_rb,0.75188e-03_rb,0.35774e-03_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.27048e-02_rb,0.26504e-02_rb,0.24329e-02_rb,0.21704e-02_rb,0.18832e-02_rb, &
        & 0.15599e-02_rb,0.11973e-02_rb,0.79054e-03_rb,0.37720e-03_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.27241e-02_rb,0.26964e-02_rb,0.24876e-02_rb,0.22289e-02_rb,0.19396e-02_rb, &
        & 0.16131e-02_rb,0.12459e-02_rb,0.82843e-03_rb,0.39980e-03_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.22863e-02_rb,0.21916e-02_rb,0.19903e-02_rb,0.17560e-02_rb,0.15053e-02_rb, &
        & 0.12361e-02_rb,0.93929e-03_rb,0.60469e-03_rb,0.28176e-03_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.23519e-02_rb,0.22705e-02_rb,0.20691e-02_rb,0.18315e-02_rb,0.15760e-02_rb, &
        & 0.12985e-02_rb,0.99139e-03_rb,0.64462e-03_rb,0.30471e-03_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.23974e-02_rb,0.23352e-02_rb,0.21377e-02_rb,0.19000e-02_rb,0.16408e-02_rb, &
        & 0.13574e-02_rb,0.10412e-02_rb,0.68214e-03_rb,0.32553e-03_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.24260e-02_rb,0.23879e-02_rb,0.21943e-02_rb,0.19602e-02_rb,0.16992e-02_rb, &
        & 0.14109e-02_rb,0.10885e-02_rb,0.71852e-03_rb,0.34169e-03_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.24412e-02_rb,0.24274e-02_rb,0.22422e-02_rb,0.20116e-02_rb,0.17502e-02_rb, &
        & 0.14591e-02_rb,0.11336e-02_rb,0.75474e-03_rb,0.36020e-03_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.20393e-02_rb,0.19603e-02_rb,0.17821e-02_rb,0.15739e-02_rb,0.13509e-02_rb, &
        & 0.11133e-02_rb,0.84826e-03_rb,0.54289e-03_rb,0.23248e-03_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.20953e-02_rb,0.20307e-02_rb,0.18542e-02_rb,0.16433e-02_rb,0.14162e-02_rb, &
        & 0.11712e-02_rb,0.89683e-03_rb,0.57958e-03_rb,0.25095e-03_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.21323e-02_rb,0.20881e-02_rb,0.19144e-02_rb,0.17046e-02_rb,0.14742e-02_rb, &
        & 0.12241e-02_rb,0.94271e-03_rb,0.61480e-03_rb,0.26566e-03_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.21562e-02_rb,0.21324e-02_rb,0.19654e-02_rb,0.17578e-02_rb,0.15256e-02_rb, &
        & 0.12715e-02_rb,0.98599e-03_rb,0.64941e-03_rb,0.28144e-03_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.21687e-02_rb,0.21674e-02_rb,0.20081e-02_rb,0.18029e-02_rb,0.15712e-02_rb, &
        & 0.13148e-02_rb,0.10270e-02_rb,0.68370e-03_rb,0.29755e-03_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.10084e-01_rb,0.95278e-02_rb,0.88730e-02_rb,0.81189e-02_rb,0.71873e-02_rb, &
        & 0.61505e-02_rb,0.50159e-02_rb,0.37383e-02_rb,0.30410e-02_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.10011e-01_rb,0.95195e-02_rb,0.89419e-02_rb,0.82272e-02_rb,0.73508e-02_rb, &
        & 0.63699e-02_rb,0.52392e-02_rb,0.39784e-02_rb,0.34237e-02_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.99285e-02_rb,0.95060e-02_rb,0.90107e-02_rb,0.83435e-02_rb,0.75299e-02_rb, &
        & 0.65814e-02_rb,0.54661e-02_rb,0.42490e-02_rb,0.38569e-02_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.98396e-02_rb,0.95039e-02_rb,0.90786e-02_rb,0.84657e-02_rb,0.77142e-02_rb, &
        & 0.67908e-02_rb,0.57132e-02_rb,0.45510e-02_rb,0.43277e-02_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.97470e-02_rb,0.95073e-02_rb,0.91451e-02_rb,0.85961e-02_rb,0.78983e-02_rb, &
        & 0.70134e-02_rb,0.59878e-02_rb,0.48898e-02_rb,0.48367e-02_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.10032e-01_rb,0.95001e-02_rb,0.88015e-02_rb,0.80012e-02_rb,0.70675e-02_rb, &
        & 0.59994e-02_rb,0.48088e-02_rb,0.34803e-02_rb,0.24973e-02_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.99727e-02_rb,0.94952e-02_rb,0.88708e-02_rb,0.81052e-02_rb,0.72227e-02_rb, &
        & 0.61975e-02_rb,0.50149e-02_rb,0.36876e-02_rb,0.28262e-02_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.99020e-02_rb,0.94876e-02_rb,0.89395e-02_rb,0.82200e-02_rb,0.73884e-02_rb, &
        & 0.63917e-02_rb,0.52241e-02_rb,0.39189e-02_rb,0.31881e-02_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.98208e-02_rb,0.94907e-02_rb,0.90072e-02_rb,0.83409e-02_rb,0.75534e-02_rb, &
        & 0.65848e-02_rb,0.54476e-02_rb,0.41701e-02_rb,0.35802e-02_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.97350e-02_rb,0.95036e-02_rb,0.90758e-02_rb,0.84664e-02_rb,0.77179e-02_rb, &
        & 0.67885e-02_rb,0.56906e-02_rb,0.44399e-02_rb,0.40032e-02_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.99249e-02_rb,0.94002e-02_rb,0.86731e-02_rb,0.78226e-02_rb,0.68627e-02_rb, &
        & 0.57870e-02_rb,0.45641e-02_rb,0.32036e-02_rb,0.20068e-02_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.98781e-02_rb,0.94039e-02_rb,0.87415e-02_rb,0.79225e-02_rb,0.70078e-02_rb, &
        & 0.59682e-02_rb,0.47504e-02_rb,0.33906e-02_rb,0.22741e-02_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.98245e-02_rb,0.94056e-02_rb,0.88077e-02_rb,0.80290e-02_rb,0.71634e-02_rb, &
        & 0.61455e-02_rb,0.49439e-02_rb,0.35923e-02_rb,0.25675e-02_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.97585e-02_rb,0.94181e-02_rb,0.88723e-02_rb,0.81452e-02_rb,0.73134e-02_rb, &
        & 0.63196e-02_rb,0.51498e-02_rb,0.38082e-02_rb,0.28828e-02_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.96800e-02_rb,0.94346e-02_rb,0.89400e-02_rb,0.82650e-02_rb,0.74596e-02_rb, &
        & 0.65093e-02_rb,0.53613e-02_rb,0.40378e-02_rb,0.32210e-02_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.97390e-02_rb,0.92223e-02_rb,0.84805e-02_rb,0.75962e-02_rb,0.66183e-02_rb, &
        & 0.55421e-02_rb,0.43153e-02_rb,0.29439e-02_rb,0.16096e-02_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.97149e-02_rb,0.92440e-02_rb,0.85515e-02_rb,0.76965e-02_rb,0.67525e-02_rb, &
        & 0.57037e-02_rb,0.44852e-02_rb,0.31152e-02_rb,0.18298e-02_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.96772e-02_rb,0.92607e-02_rb,0.86210e-02_rb,0.78004e-02_rb,0.68947e-02_rb, &
        & 0.58626e-02_rb,0.46601e-02_rb,0.32973e-02_rb,0.20675e-02_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.96222e-02_rb,0.92823e-02_rb,0.86884e-02_rb,0.79077e-02_rb,0.70317e-02_rb, &
        & 0.60213e-02_rb,0.48471e-02_rb,0.34874e-02_rb,0.23199e-02_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.95548e-02_rb,0.93032e-02_rb,0.87544e-02_rb,0.80177e-02_rb,0.71673e-02_rb, &
        & 0.61924e-02_rb,0.50398e-02_rb,0.36845e-02_rb,0.25914e-02_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.94682e-02_rb,0.89654e-02_rb,0.82210e-02_rb,0.73222e-02_rb,0.63403e-02_rb, &
        & 0.52735e-02_rb,0.40692e-02_rb,0.27114e-02_rb,0.12929e-02_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.94609e-02_rb,0.90053e-02_rb,0.83016e-02_rb,0.74278e-02_rb,0.64685e-02_rb, &
        & 0.54213e-02_rb,0.42231e-02_rb,0.28638e-02_rb,0.14728e-02_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.94353e-02_rb,0.90391e-02_rb,0.83802e-02_rb,0.75305e-02_rb,0.66027e-02_rb, &
        & 0.55646e-02_rb,0.43790e-02_rb,0.30279e-02_rb,0.16663e-02_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.93989e-02_rb,0.90684e-02_rb,0.84481e-02_rb,0.76404e-02_rb,0.67294e-02_rb, &
        & 0.57082e-02_rb,0.45495e-02_rb,0.31980e-02_rb,0.18711e-02_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.93472e-02_rb,0.90926e-02_rb,0.85136e-02_rb,0.77442e-02_rb,0.68542e-02_rb, &
        & 0.58646e-02_rb,0.47259e-02_rb,0.33701e-02_rb,0.20921e-02_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.90687e-02_rb,0.86083e-02_rb,0.78842e-02_rb,0.69920e-02_rb,0.60171e-02_rb, &
        & 0.49784e-02_rb,0.38163e-02_rb,0.24892e-02_rb,0.10375e-02_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.90845e-02_rb,0.86742e-02_rb,0.79759e-02_rb,0.71037e-02_rb,0.61494e-02_rb, &
        & 0.51176e-02_rb,0.39577e-02_rb,0.26285e-02_rb,0.11848e-02_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.90836e-02_rb,0.87225e-02_rb,0.80580e-02_rb,0.72119e-02_rb,0.62799e-02_rb, &
        & 0.52546e-02_rb,0.41006e-02_rb,0.27750e-02_rb,0.13422e-02_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.90655e-02_rb,0.87611e-02_rb,0.81305e-02_rb,0.73221e-02_rb,0.64054e-02_rb, &
        & 0.53892e-02_rb,0.42503e-02_rb,0.29301e-02_rb,0.15078e-02_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.90286e-02_rb,0.87944e-02_rb,0.82022e-02_rb,0.74247e-02_rb,0.65248e-02_rb, &
        & 0.55305e-02_rb,0.44113e-02_rb,0.30838e-02_rb,0.16859e-02_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.85850e-02_rb,0.81350e-02_rb,0.74326e-02_rb,0.65794e-02_rb,0.56482e-02_rb, &
        & 0.46582e-02_rb,0.35544e-02_rb,0.22844e-02_rb,0.80765e-03_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.86250e-02_rb,0.82179e-02_rb,0.75359e-02_rb,0.67022e-02_rb,0.57899e-02_rb, &
        & 0.47962e-02_rb,0.36869e-02_rb,0.24122e-02_rb,0.92682e-03_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.86440e-02_rb,0.82839e-02_rb,0.76315e-02_rb,0.68246e-02_rb,0.59218e-02_rb, &
        & 0.49298e-02_rb,0.38219e-02_rb,0.25431e-02_rb,0.10538e-02_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.86410e-02_rb,0.83371e-02_rb,0.77195e-02_rb,0.69421e-02_rb,0.60483e-02_rb, &
        & 0.50620e-02_rb,0.39587e-02_rb,0.26818e-02_rb,0.11890e-02_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.86213e-02_rb,0.83816e-02_rb,0.78035e-02_rb,0.70537e-02_rb,0.61695e-02_rb, &
        & 0.51947e-02_rb,0.41038e-02_rb,0.28206e-02_rb,0.13358e-02_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.80690e-02_rb,0.76260e-02_rb,0.69398e-02_rb,0.61270e-02_rb,0.52372e-02_rb, &
        & 0.42965e-02_rb,0.32695e-02_rb,0.20922e-02_rb,0.69791e-03_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.81346e-02_rb,0.77325e-02_rb,0.70609e-02_rb,0.62554e-02_rb,0.53744e-02_rb, &
        & 0.44362e-02_rb,0.34019e-02_rb,0.22104e-02_rb,0.78609e-03_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.81749e-02_rb,0.78188e-02_rb,0.71688e-02_rb,0.63791e-02_rb,0.55086e-02_rb, &
        & 0.45702e-02_rb,0.35360e-02_rb,0.23302e-02_rb,0.88068e-03_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.81959e-02_rb,0.78839e-02_rb,0.72650e-02_rb,0.64997e-02_rb,0.56387e-02_rb, &
        & 0.47059e-02_rb,0.36681e-02_rb,0.24532e-02_rb,0.97840e-03_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.81937e-02_rb,0.79411e-02_rb,0.73546e-02_rb,0.66134e-02_rb,0.57649e-02_rb, &
        & 0.48383e-02_rb,0.38022e-02_rb,0.25773e-02_rb,0.10847e-02_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.75182e-02_rb,0.70907e-02_rb,0.64287e-02_rb,0.56605e-02_rb,0.48216e-02_rb, &
        & 0.39323e-02_rb,0.29776e-02_rb,0.18987e-02_rb,0.67472e-03_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.76076e-02_rb,0.72195e-02_rb,0.65653e-02_rb,0.58009e-02_rb,0.49640e-02_rb, &
        & 0.40692e-02_rb,0.31049e-02_rb,0.20098e-02_rb,0.74914e-03_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.76735e-02_rb,0.73227e-02_rb,0.66862e-02_rb,0.59346e-02_rb,0.51012e-02_rb, &
        & 0.42015e-02_rb,0.32328e-02_rb,0.21222e-02_rb,0.83367e-03_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.77132e-02_rb,0.74045e-02_rb,0.67969e-02_rb,0.60620e-02_rb,0.52309e-02_rb, &
        & 0.43347e-02_rb,0.33599e-02_rb,0.22355e-02_rb,0.91964e-03_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.77286e-02_rb,0.74752e-02_rb,0.68965e-02_rb,0.61793e-02_rb,0.53584e-02_rb, &
        & 0.44621e-02_rb,0.34884e-02_rb,0.23497e-02_rb,0.10114e-02_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.70149e-02_rb,0.65825e-02_rb,0.59329e-02_rb,0.52017e-02_rb,0.44219e-02_rb, &
        & 0.35899e-02_rb,0.27069e-02_rb,0.17187e-02_rb,0.74519e-03_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.71248e-02_rb,0.67269e-02_rb,0.60810e-02_rb,0.53515e-02_rb,0.45670e-02_rb, &
        & 0.37282e-02_rb,0.28313e-02_rb,0.18204e-02_rb,0.83494e-03_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.72044e-02_rb,0.68394e-02_rb,0.62076e-02_rb,0.54929e-02_rb,0.47097e-02_rb, &
        & 0.38613e-02_rb,0.29540e-02_rb,0.19235e-02_rb,0.93279e-03_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.72597e-02_rb,0.69309e-02_rb,0.63267e-02_rb,0.56269e-02_rb,0.48423e-02_rb, &
        & 0.39913e-02_rb,0.30764e-02_rb,0.20273e-02_rb,0.10348e-02_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.72955e-02_rb,0.70104e-02_rb,0.64378e-02_rb,0.57479e-02_rb,0.49695e-02_rb, &
        & 0.41180e-02_rb,0.31960e-02_rb,0.21324e-02_rb,0.11387e-02_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.65628e-02_rb,0.61735e-02_rb,0.55551e-02_rb,0.48677e-02_rb,0.41265e-02_rb, &
        & 0.33334e-02_rb,0.25028e-02_rb,0.15938e-02_rb,0.74406e-03_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.66786e-02_rb,0.63151e-02_rb,0.57025e-02_rb,0.50172e-02_rb,0.42702e-02_rb, &
        & 0.34674e-02_rb,0.26229e-02_rb,0.16875e-02_rb,0.82711e-03_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.67664e-02_rb,0.64350e-02_rb,0.58347e-02_rb,0.51562e-02_rb,0.44059e-02_rb, &
        & 0.35945e-02_rb,0.27422e-02_rb,0.17822e-02_rb,0.92379e-03_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.68381e-02_rb,0.65357e-02_rb,0.59597e-02_rb,0.52850e-02_rb,0.45292e-02_rb, &
        & 0.37195e-02_rb,0.28585e-02_rb,0.18773e-02_rb,0.10214e-02_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.68910e-02_rb,0.66274e-02_rb,0.60702e-02_rb,0.54014e-02_rb,0.46503e-02_rb, &
        & 0.38396e-02_rb,0.29707e-02_rb,0.19733e-02_rb,0.11179e-02_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.60794e-02_rb,0.57316e-02_rb,0.51563e-02_rb,0.45236e-02_rb,0.38371e-02_rb, &
        & 0.31013e-02_rb,0.23239e-02_rb,0.14745e-02_rb,0.70212e-03_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.62026e-02_rb,0.58782e-02_rb,0.53110e-02_rb,0.46778e-02_rb,0.39838e-02_rb, &
        & 0.32334e-02_rb,0.24368e-02_rb,0.15615e-02_rb,0.77146e-03_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.63087e-02_rb,0.60102e-02_rb,0.54544e-02_rb,0.48220e-02_rb,0.41191e-02_rb, &
        & 0.33562e-02_rb,0.25462e-02_rb,0.16493e-02_rb,0.85116e-03_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.63948e-02_rb,0.61265e-02_rb,0.55863e-02_rb,0.49541e-02_rb,0.42453e-02_rb, &
        & 0.34753e-02_rb,0.26508e-02_rb,0.17372e-02_rb,0.93862e-03_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.64607e-02_rb,0.62309e-02_rb,0.57052e-02_rb,0.50757e-02_rb,0.43660e-02_rb, &
        & 0.35907e-02_rb,0.27548e-02_rb,0.18249e-02_rb,0.10221e-02_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.55937e-02_rb,0.52793e-02_rb,0.47517e-02_rb,0.41692e-02_rb,0.35394e-02_rb, &
        & 0.28625e-02_rb,0.21454e-02_rb,0.13668e-02_rb,0.61956e-03_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.57319e-02_rb,0.54374e-02_rb,0.49129e-02_rb,0.43258e-02_rb,0.36843e-02_rb, &
        & 0.29907e-02_rb,0.22551e-02_rb,0.14474e-02_rb,0.68007e-03_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.58515e-02_rb,0.55788e-02_rb,0.50622e-02_rb,0.44734e-02_rb,0.38194e-02_rb, &
        & 0.31146e-02_rb,0.23600e-02_rb,0.15267e-02_rb,0.75016e-03_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.59483e-02_rb,0.57068e-02_rb,0.51995e-02_rb,0.46068e-02_rb,0.39501e-02_rb, &
        & 0.32357e-02_rb,0.24610e-02_rb,0.16061e-02_rb,0.82038e-03_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.60233e-02_rb,0.58195e-02_rb,0.53236e-02_rb,0.47349e-02_rb,0.40732e-02_rb, &
        & 0.33509e-02_rb,0.25596e-02_rb,0.16868e-02_rb,0.89122e-03_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.19771e-01_rb,0.18370e-01_rb,0.17356e-01_rb,0.16079e-01_rb,0.14680e-01_rb, &
        & 0.13282e-01_rb,0.11980e-01_rb,0.11582e-01_rb,0.12985e-01_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.19636e-01_rb,0.18337e-01_rb,0.17425e-01_rb,0.16276e-01_rb,0.15036e-01_rb, &
        & 0.13800e-01_rb,0.12788e-01_rb,0.12799e-01_rb,0.14491e-01_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.19487e-01_rb,0.18291e-01_rb,0.17516e-01_rb,0.16543e-01_rb,0.15458e-01_rb, &
        & 0.14398e-01_rb,0.13717e-01_rb,0.14121e-01_rb,0.16094e-01_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.19329e-01_rb,0.18244e-01_rb,0.17665e-01_rb,0.16858e-01_rb,0.15921e-01_rb, &
        & 0.15082e-01_rb,0.14738e-01_rb,0.15534e-01_rb,0.17796e-01_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.19162e-01_rb,0.18208e-01_rb,0.17862e-01_rb,0.17203e-01_rb,0.16407e-01_rb, &
        & 0.15804e-01_rb,0.15831e-01_rb,0.17016e-01_rb,0.19554e-01_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.20491e-01_rb,0.19041e-01_rb,0.17931e-01_rb,0.16490e-01_rb,0.14871e-01_rb, &
        & 0.13204e-01_rb,0.11551e-01_rb,0.10493e-01_rb,0.11370e-01_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.20351e-01_rb,0.19021e-01_rb,0.18003e-01_rb,0.16688e-01_rb,0.15207e-01_rb, &
        & 0.13674e-01_rb,0.12228e-01_rb,0.11542e-01_rb,0.12722e-01_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.20205e-01_rb,0.18987e-01_rb,0.18097e-01_rb,0.16946e-01_rb,0.15590e-01_rb, &
        & 0.14204e-01_rb,0.13003e-01_rb,0.12703e-01_rb,0.14170e-01_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.20054e-01_rb,0.18946e-01_rb,0.18241e-01_rb,0.17244e-01_rb,0.16012e-01_rb, &
        & 0.14800e-01_rb,0.13862e-01_rb,0.13953e-01_rb,0.15702e-01_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.19890e-01_rb,0.18914e-01_rb,0.18431e-01_rb,0.17568e-01_rb,0.16464e-01_rb, &
        & 0.15445e-01_rb,0.14813e-01_rb,0.15302e-01_rb,0.17318e-01_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.21143e-01_rb,0.19662e-01_rb,0.18419e-01_rb,0.16811e-01_rb,0.15005e-01_rb, &
        & 0.13075e-01_rb,0.11094e-01_rb,0.93893e-02_rb,0.95682e-02_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.21021e-01_rb,0.19658e-01_rb,0.18498e-01_rb,0.17007e-01_rb,0.15312e-01_rb, &
        & 0.13491e-01_rb,0.11661e-01_rb,0.10256e-01_rb,0.10742e-01_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.20887e-01_rb,0.19635e-01_rb,0.18594e-01_rb,0.17251e-01_rb,0.15669e-01_rb, &
        & 0.13959e-01_rb,0.12304e-01_rb,0.11227e-01_rb,0.12012e-01_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.20742e-01_rb,0.19603e-01_rb,0.18727e-01_rb,0.17530e-01_rb,0.16058e-01_rb, &
        & 0.14485e-01_rb,0.13015e-01_rb,0.12294e-01_rb,0.13375e-01_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.20582e-01_rb,0.19578e-01_rb,0.18903e-01_rb,0.17831e-01_rb,0.16483e-01_rb, &
        & 0.15047e-01_rb,0.13818e-01_rb,0.13471e-01_rb,0.14836e-01_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.21726e-01_rb,0.20199e-01_rb,0.18812e-01_rb,0.17047e-01_rb,0.15080e-01_rb, &
        & 0.12938e-01_rb,0.10693e-01_rb,0.84650e-02_rb,0.79270e-02_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.21624e-01_rb,0.20213e-01_rb,0.18897e-01_rb,0.17238e-01_rb,0.15368e-01_rb, &
        & 0.13326e-01_rb,0.11175e-01_rb,0.91656e-02_rb,0.89409e-02_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.21502e-01_rb,0.20207e-01_rb,0.18990e-01_rb,0.17468e-01_rb,0.15708e-01_rb, &
        & 0.13755e-01_rb,0.11721e-01_rb,0.99561e-02_rb,0.10044e-01_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.21364e-01_rb,0.20186e-01_rb,0.19114e-01_rb,0.17747e-01_rb,0.16075e-01_rb, &
        & 0.14230e-01_rb,0.12327e-01_rb,0.10844e-01_rb,0.11238e-01_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.21208e-01_rb,0.20165e-01_rb,0.19280e-01_rb,0.18035e-01_rb,0.16477e-01_rb, &
        & 0.14734e-01_rb,0.13004e-01_rb,0.11837e-01_rb,0.12527e-01_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.22212e-01_rb,0.20643e-01_rb,0.19101e-01_rb,0.17193e-01_rb,0.15094e-01_rb, &
        & 0.12790e-01_rb,0.10344e-01_rb,0.77661e-02_rb,0.65292e-02_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.22131e-01_rb,0.20679e-01_rb,0.19192e-01_rb,0.17388e-01_rb,0.15373e-01_rb, &
        & 0.13156e-01_rb,0.10779e-01_rb,0.83326e-02_rb,0.74004e-02_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.22024e-01_rb,0.20689e-01_rb,0.19289e-01_rb,0.17614e-01_rb,0.15701e-01_rb, &
        & 0.13560e-01_rb,0.11259e-01_rb,0.89716e-02_rb,0.83509e-02_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.21891e-01_rb,0.20674e-01_rb,0.19419e-01_rb,0.17880e-01_rb,0.16055e-01_rb, &
        & 0.14002e-01_rb,0.11784e-01_rb,0.96942e-02_rb,0.93863e-02_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.21733e-01_rb,0.20659e-01_rb,0.19581e-01_rb,0.18166e-01_rb,0.16436e-01_rb, &
        & 0.14467e-01_rb,0.12356e-01_rb,0.10508e-01_rb,0.10499e-01_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.22561e-01_rb,0.20963e-01_rb,0.19278e-01_rb,0.17248e-01_rb,0.15027e-01_rb, &
        & 0.12605e-01_rb,0.99913e-02_rb,0.71998e-02_rb,0.53191e-02_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.22505e-01_rb,0.21027e-01_rb,0.19391e-01_rb,0.17451e-01_rb,0.15296e-01_rb, &
        & 0.12947e-01_rb,0.10394e-01_rb,0.76719e-02_rb,0.60628e-02_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.22407e-01_rb,0.21057e-01_rb,0.19505e-01_rb,0.17673e-01_rb,0.15617e-01_rb, &
        & 0.13330e-01_rb,0.10834e-01_rb,0.82004e-02_rb,0.68698e-02_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.22287e-01_rb,0.21061e-01_rb,0.19636e-01_rb,0.17933e-01_rb,0.15963e-01_rb, &
        & 0.13742e-01_rb,0.11312e-01_rb,0.87854e-02_rb,0.77488e-02_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.22148e-01_rb,0.21053e-01_rb,0.19794e-01_rb,0.18212e-01_rb,0.16329e-01_rb, &
        & 0.14180e-01_rb,0.11816e-01_rb,0.94417e-02_rb,0.86993e-02_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.22745e-01_rb,0.21135e-01_rb,0.19327e-01_rb,0.17197e-01_rb,0.14893e-01_rb, &
        & 0.12374e-01_rb,0.96536e-02_rb,0.67256e-02_rb,0.43127e-02_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.22707e-01_rb,0.21230e-01_rb,0.19474e-01_rb,0.17423e-01_rb,0.15162e-01_rb, &
        & 0.12700e-01_rb,0.10030e-01_rb,0.71370e-02_rb,0.49389e-02_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.22632e-01_rb,0.21281e-01_rb,0.19608e-01_rb,0.17649e-01_rb,0.15471e-01_rb, &
        & 0.13070e-01_rb,0.10436e-01_rb,0.75822e-02_rb,0.56174e-02_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.22537e-01_rb,0.21307e-01_rb,0.19744e-01_rb,0.17903e-01_rb,0.15813e-01_rb, &
        & 0.13461e-01_rb,0.10872e-01_rb,0.80682e-02_rb,0.63562e-02_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.22420e-01_rb,0.21314e-01_rb,0.19904e-01_rb,0.18182e-01_rb,0.16160e-01_rb, &
        & 0.13874e-01_rb,0.11331e-01_rb,0.86088e-02_rb,0.71565e-02_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.22742e-01_rb,0.21141e-01_rb,0.19244e-01_rb,0.17036e-01_rb,0.14667e-01_rb, &
        & 0.12093e-01_rb,0.93194e-02_rb,0.63016e-02_rb,0.33604e-02_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.22738e-01_rb,0.21267e-01_rb,0.19420e-01_rb,0.17289e-01_rb,0.14951e-01_rb, &
        & 0.12414e-01_rb,0.96717e-02_rb,0.66654e-02_rb,0.38903e-02_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.22699e-01_rb,0.21347e-01_rb,0.19574e-01_rb,0.17525e-01_rb,0.15260e-01_rb, &
        & 0.12775e-01_rb,0.10048e-01_rb,0.70608e-02_rb,0.44626e-02_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.22634e-01_rb,0.21402e-01_rb,0.19725e-01_rb,0.17775e-01_rb,0.15596e-01_rb, &
        & 0.13150e-01_rb,0.10455e-01_rb,0.74847e-02_rb,0.50904e-02_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.22550e-01_rb,0.21432e-01_rb,0.19888e-01_rb,0.18059e-01_rb,0.15936e-01_rb, &
        & 0.13540e-01_rb,0.10879e-01_rb,0.79454e-02_rb,0.57754e-02_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.22571e-01_rb,0.20980e-01_rb,0.19017e-01_rb,0.16764e-01_rb,0.14350e-01_rb, &
        & 0.11743e-01_rb,0.89583e-02_rb,0.59107e-02_rb,0.27753e-02_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.22607e-01_rb,0.21147e-01_rb,0.19231e-01_rb,0.17040e-01_rb,0.14648e-01_rb, &
        & 0.12069e-01_rb,0.92988e-02_rb,0.62425e-02_rb,0.31527e-02_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.22609e-01_rb,0.21268e-01_rb,0.19413e-01_rb,0.17284e-01_rb,0.14954e-01_rb, &
        & 0.12424e-01_rb,0.96568e-02_rb,0.66010e-02_rb,0.35387e-02_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.22589e-01_rb,0.21355e-01_rb,0.19582e-01_rb,0.17535e-01_rb,0.15285e-01_rb, &
        & 0.12790e-01_rb,0.10038e-01_rb,0.69765e-02_rb,0.39766e-02_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.22546e-01_rb,0.21413e-01_rb,0.19758e-01_rb,0.17821e-01_rb,0.15622e-01_rb, &
        & 0.13166e-01_rb,0.10432e-01_rb,0.73821e-02_rb,0.44450e-02_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.22137e-01_rb,0.20618e-01_rb,0.18650e-01_rb,0.16403e-01_rb,0.13972e-01_rb, &
        & 0.11364e-01_rb,0.85848e-02_rb,0.55632e-02_rb,0.31593e-02_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.22231e-01_rb,0.20835e-01_rb,0.18905e-01_rb,0.16706e-01_rb,0.14289e-01_rb, &
        & 0.11695e-01_rb,0.89167e-02_rb,0.58739e-02_rb,0.35325e-02_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.22295e-01_rb,0.21010e-01_rb,0.19131e-01_rb,0.16972e-01_rb,0.14595e-01_rb, &
        & 0.12040e-01_rb,0.92665e-02_rb,0.62059e-02_rb,0.39315e-02_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.22331e-01_rb,0.21144e-01_rb,0.19334e-01_rb,0.17233e-01_rb,0.14916e-01_rb, &
        & 0.12396e-01_rb,0.96355e-02_rb,0.65507e-02_rb,0.43409e-02_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.22332e-01_rb,0.21248e-01_rb,0.19527e-01_rb,0.17517e-01_rb,0.15252e-01_rb, &
        & 0.12765e-01_rb,0.10012e-01_rb,0.69176e-02_rb,0.47851e-02_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.21611e-01_rb,0.20177e-01_rb,0.18220e-01_rb,0.16014e-01_rb,0.13625e-01_rb, &
        & 0.11072e-01_rb,0.83376e-02_rb,0.53526e-02_rb,0.32967e-02_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.21754e-01_rb,0.20431e-01_rb,0.18507e-01_rb,0.16333e-01_rb,0.13954e-01_rb, &
        & 0.11415e-01_rb,0.86702e-02_rb,0.56558e-02_rb,0.36779e-02_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.21862e-01_rb,0.20634e-01_rb,0.18769e-01_rb,0.16624e-01_rb,0.14280e-01_rb, &
        & 0.11762e-01_rb,0.90145e-02_rb,0.59759e-02_rb,0.40891e-02_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.21929e-01_rb,0.20796e-01_rb,0.18995e-01_rb,0.16914e-01_rb,0.14621e-01_rb, &
        & 0.12122e-01_rb,0.93752e-02_rb,0.63026e-02_rb,0.45492e-02_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.21956e-01_rb,0.20925e-01_rb,0.19222e-01_rb,0.17218e-01_rb,0.14974e-01_rb, &
        & 0.12492e-01_rb,0.97406e-02_rb,0.66508e-02_rb,0.50324e-02_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.20989e-01_rb,0.19649e-01_rb,0.17719e-01_rb,0.15554e-01_rb,0.13207e-01_rb, &
        & 0.10709e-01_rb,0.80423e-02_rb,0.51308e-02_rb,0.32152e-02_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.21187e-01_rb,0.19941e-01_rb,0.18042e-01_rb,0.15892e-01_rb,0.13549e-01_rb, &
        & 0.11055e-01_rb,0.83766e-02_rb,0.54250e-02_rb,0.35851e-02_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.21336e-01_rb,0.20173e-01_rb,0.18327e-01_rb,0.16203e-01_rb,0.13894e-01_rb, &
        & 0.11411e-01_rb,0.87228e-02_rb,0.57329e-02_rb,0.40006e-02_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.21440e-01_rb,0.20364e-01_rb,0.18584e-01_rb,0.16516e-01_rb,0.14243e-01_rb, &
        & 0.11781e-01_rb,0.90825e-02_rb,0.60483e-02_rb,0.44327e-02_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.21509e-01_rb,0.20531e-01_rb,0.18841e-01_rb,0.16838e-01_rb,0.14607e-01_rb, &
        & 0.12155e-01_rb,0.94413e-02_rb,0.63798e-02_rb,0.48628e-02_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.20300e-01_rb,0.19036e-01_rb,0.17142e-01_rb,0.15033e-01_rb,0.12745e-01_rb, &
        & 0.10318e-01_rb,0.77330e-02_rb,0.48977e-02_rb,0.29186e-02_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.20539e-01_rb,0.19361e-01_rb,0.17504e-01_rb,0.15398e-01_rb,0.13114e-01_rb, &
        & 0.10679e-01_rb,0.80651e-02_rb,0.51844e-02_rb,0.32743e-02_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.20734e-01_rb,0.19633e-01_rb,0.17821e-01_rb,0.15739e-01_rb,0.13481e-01_rb, &
        & 0.11044e-01_rb,0.84099e-02_rb,0.54812e-02_rb,0.36601e-02_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.20892e-01_rb,0.19871e-01_rb,0.18120e-01_rb,0.16090e-01_rb,0.13847e-01_rb, &
        & 0.11415e-01_rb,0.87615e-02_rb,0.57863e-02_rb,0.40403e-02_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.21016e-01_rb,0.20092e-01_rb,0.18420e-01_rb,0.16433e-01_rb,0.14220e-01_rb, &
        & 0.11787e-01_rb,0.91177e-02_rb,0.61020e-02_rb,0.44775e-02_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.32077e-01_rb,0.29717e-01_rb,0.27957e-01_rb,0.27088e-01_rb,0.26148e-01_rb, &
        & 0.25463e-01_rb,0.27887e-01_rb,0.32409e-01_rb,0.37623e-01_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.31656e-01_rb,0.29512e-01_rb,0.28047e-01_rb,0.27317e-01_rb,0.26667e-01_rb, &
        & 0.26788e-01_rb,0.30344e-01_rb,0.35248e-01_rb,0.40533e-01_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.31222e-01_rb,0.29398e-01_rb,0.28183e-01_rb,0.27633e-01_rb,0.27598e-01_rb, &
        & 0.28788e-01_rb,0.33344e-01_rb,0.38694e-01_rb,0.44458e-01_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.30838e-01_rb,0.29185e-01_rb,0.28219e-01_rb,0.28269e-01_rb,0.28900e-01_rb, &
        & 0.31215e-01_rb,0.36765e-01_rb,0.42659e-01_rb,0.48710e-01_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.30476e-01_rb,0.28982e-01_rb,0.28435e-01_rb,0.29011e-01_rb,0.30822e-01_rb, &
        & 0.34422e-01_rb,0.40770e-01_rb,0.47309e-01_rb,0.54027e-01_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.34215e-01_rb,0.31859e-01_rb,0.30015e-01_rb,0.28551e-01_rb,0.26684e-01_rb, &
        & 0.25243e-01_rb,0.25874e-01_rb,0.29803e-01_rb,0.34327e-01_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.33819e-01_rb,0.31701e-01_rb,0.30099e-01_rb,0.28712e-01_rb,0.27156e-01_rb, &
        & 0.26446e-01_rb,0.28380e-01_rb,0.32886e-01_rb,0.37685e-01_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.33393e-01_rb,0.31477e-01_rb,0.30146e-01_rb,0.28931e-01_rb,0.28112e-01_rb, &
        & 0.28308e-01_rb,0.31383e-01_rb,0.36360e-01_rb,0.41605e-01_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.33012e-01_rb,0.31222e-01_rb,0.30110e-01_rb,0.29460e-01_rb,0.29343e-01_rb, &
        & 0.30463e-01_rb,0.34829e-01_rb,0.40350e-01_rb,0.46115e-01_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.32618e-01_rb,0.30977e-01_rb,0.30249e-01_rb,0.30140e-01_rb,0.31029e-01_rb, &
        & 0.33086e-01_rb,0.38518e-01_rb,0.44611e-01_rb,0.51063e-01_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.36609e-01_rb,0.34016e-01_rb,0.32008e-01_rb,0.30013e-01_rb,0.27528e-01_rb, &
        & 0.25143e-01_rb,0.24092e-01_rb,0.26746e-01_rb,0.30590e-01_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.36187e-01_rb,0.33816e-01_rb,0.32068e-01_rb,0.30121e-01_rb,0.27943e-01_rb, &
        & 0.26137e-01_rb,0.26190e-01_rb,0.29769e-01_rb,0.34003e-01_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.35755e-01_rb,0.33555e-01_rb,0.32022e-01_rb,0.30354e-01_rb,0.28696e-01_rb, &
        & 0.27732e-01_rb,0.28744e-01_rb,0.33056e-01_rb,0.37754e-01_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.35304e-01_rb,0.33277e-01_rb,0.31985e-01_rb,0.30856e-01_rb,0.29759e-01_rb, &
        & 0.29582e-01_rb,0.31793e-01_rb,0.36769e-01_rb,0.41965e-01_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.34895e-01_rb,0.32987e-01_rb,0.32094e-01_rb,0.31490e-01_rb,0.31100e-01_rb, &
        & 0.31735e-01_rb,0.35104e-01_rb,0.40646e-01_rb,0.46442e-01_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.38977e-01_rb,0.36120e-01_rb,0.33897e-01_rb,0.31410e-01_rb,0.28525e-01_rb, &
        & 0.25373e-01_rb,0.22918e-01_rb,0.23596e-01_rb,0.26905e-01_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.38508e-01_rb,0.35924e-01_rb,0.33917e-01_rb,0.31530e-01_rb,0.28933e-01_rb, &
        & 0.26161e-01_rb,0.24652e-01_rb,0.26430e-01_rb,0.30108e-01_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.38014e-01_rb,0.35658e-01_rb,0.33872e-01_rb,0.31757e-01_rb,0.29547e-01_rb, &
        & 0.27436e-01_rb,0.26616e-01_rb,0.29450e-01_rb,0.33541e-01_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.37555e-01_rb,0.35363e-01_rb,0.33895e-01_rb,0.32125e-01_rb,0.30540e-01_rb, &
        & 0.28878e-01_rb,0.28956e-01_rb,0.32793e-01_rb,0.37351e-01_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.37151e-01_rb,0.35088e-01_rb,0.33984e-01_rb,0.32788e-01_rb,0.31573e-01_rb, &
        & 0.30633e-01_rb,0.31627e-01_rb,0.36308e-01_rb,0.41371e-01_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.41153e-01_rb,0.38034e-01_rb,0.35706e-01_rb,0.32805e-01_rb,0.29387e-01_rb, &
        & 0.25927e-01_rb,0.22231e-01_rb,0.20561e-01_rb,0.23186e-01_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.40611e-01_rb,0.37835e-01_rb,0.35759e-01_rb,0.32923e-01_rb,0.29781e-01_rb, &
        & 0.26577e-01_rb,0.23565e-01_rb,0.23003e-01_rb,0.26110e-01_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.40174e-01_rb,0.37589e-01_rb,0.35747e-01_rb,0.33131e-01_rb,0.30334e-01_rb, &
        & 0.27648e-01_rb,0.25145e-01_rb,0.25696e-01_rb,0.29208e-01_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.39802e-01_rb,0.37389e-01_rb,0.35760e-01_rb,0.33447e-01_rb,0.31294e-01_rb, &
        & 0.28785e-01_rb,0.27005e-01_rb,0.28729e-01_rb,0.32665e-01_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.39471e-01_rb,0.37195e-01_rb,0.35872e-01_rb,0.34084e-01_rb,0.32227e-01_rb, &
        & 0.30087e-01_rb,0.29177e-01_rb,0.32029e-01_rb,0.36449e-01_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.43370e-01_rb,0.39997e-01_rb,0.37440e-01_rb,0.34118e-01_rb,0.30345e-01_rb, &
        & 0.26246e-01_rb,0.21955e-01_rb,0.18231e-01_rb,0.19573e-01_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.42801e-01_rb,0.39763e-01_rb,0.37480e-01_rb,0.34252e-01_rb,0.30723e-01_rb, &
        & 0.26892e-01_rb,0.22943e-01_rb,0.20080e-01_rb,0.22108e-01_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.42453e-01_rb,0.39592e-01_rb,0.37449e-01_rb,0.34486e-01_rb,0.31160e-01_rb, &
        & 0.27853e-01_rb,0.24129e-01_rb,0.22254e-01_rb,0.24945e-01_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.42151e-01_rb,0.39446e-01_rb,0.37500e-01_rb,0.34777e-01_rb,0.31970e-01_rb, &
        & 0.28872e-01_rb,0.25539e-01_rb,0.24809e-01_rb,0.28086e-01_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.41833e-01_rb,0.39344e-01_rb,0.37631e-01_rb,0.35392e-01_rb,0.32810e-01_rb, &
        & 0.29983e-01_rb,0.27309e-01_rb,0.27744e-01_rb,0.31535e-01_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.45445e-01_rb,0.41961e-01_rb,0.39238e-01_rb,0.35426e-01_rb,0.31192e-01_rb, &
        & 0.26649e-01_rb,0.21695e-01_rb,0.16623e-01_rb,0.16293e-01_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.45078e-01_rb,0.41797e-01_rb,0.39225e-01_rb,0.35507e-01_rb,0.31562e-01_rb, &
        & 0.27229e-01_rb,0.22532e-01_rb,0.18019e-01_rb,0.18534e-01_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.44870e-01_rb,0.41785e-01_rb,0.39209e-01_rb,0.35749e-01_rb,0.31958e-01_rb, &
        & 0.27976e-01_rb,0.23572e-01_rb,0.19783e-01_rb,0.21105e-01_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.44638e-01_rb,0.41726e-01_rb,0.39301e-01_rb,0.35989e-01_rb,0.32614e-01_rb, &
        & 0.28887e-01_rb,0.24743e-01_rb,0.21842e-01_rb,0.23947e-01_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.44319e-01_rb,0.41703e-01_rb,0.39403e-01_rb,0.36491e-01_rb,0.33461e-01_rb, &
        & 0.29864e-01_rb,0.26133e-01_rb,0.24238e-01_rb,0.27063e-01_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.47338e-01_rb,0.43694e-01_rb,0.40795e-01_rb,0.36657e-01_rb,0.31991e-01_rb, &
        & 0.26976e-01_rb,0.21436e-01_rb,0.15589e-01_rb,0.13371e-01_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.47172e-01_rb,0.43681e-01_rb,0.40850e-01_rb,0.36728e-01_rb,0.32341e-01_rb, &
        & 0.27474e-01_rb,0.22184e-01_rb,0.16691e-01_rb,0.15337e-01_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.47077e-01_rb,0.43885e-01_rb,0.40965e-01_rb,0.37016e-01_rb,0.32654e-01_rb, &
        & 0.28038e-01_rb,0.23084e-01_rb,0.18052e-01_rb,0.17670e-01_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.46877e-01_rb,0.43984e-01_rb,0.41138e-01_rb,0.37264e-01_rb,0.33142e-01_rb, &
        & 0.28868e-01_rb,0.24126e-01_rb,0.19696e-01_rb,0.20174e-01_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.46627e-01_rb,0.44042e-01_rb,0.41218e-01_rb,0.37582e-01_rb,0.33931e-01_rb, &
        & 0.29818e-01_rb,0.25367e-01_rb,0.21590e-01_rb,0.22899e-01_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.49069e-01_rb,0.45230e-01_rb,0.41949e-01_rb,0.37482e-01_rb,0.32547e-01_rb, &
        & 0.27221e-01_rb,0.21190e-01_rb,0.14771e-01_rb,0.82104e-02_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.48982e-01_rb,0.45358e-01_rb,0.42113e-01_rb,0.37642e-01_rb,0.32970e-01_rb, &
        & 0.27650e-01_rb,0.21850e-01_rb,0.15738e-01_rb,0.94586e-02_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.48934e-01_rb,0.45677e-01_rb,0.42390e-01_rb,0.38084e-01_rb,0.33301e-01_rb, &
        & 0.28097e-01_rb,0.22664e-01_rb,0.16844e-01_rb,0.11686e-01_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.48760e-01_rb,0.45876e-01_rb,0.42614e-01_rb,0.38405e-01_rb,0.33732e-01_rb, &
        & 0.28793e-01_rb,0.23625e-01_rb,0.18186e-01_rb,0.14417e-01_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.48535e-01_rb,0.45946e-01_rb,0.42722e-01_rb,0.38669e-01_rb,0.34439e-01_rb, &
        & 0.29718e-01_rb,0.24746e-01_rb,0.19636e-01_rb,0.17490e-01_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.50581e-01_rb,0.46588e-01_rb,0.42861e-01_rb,0.37978e-01_rb,0.32756e-01_rb, &
        & 0.27218e-01_rb,0.20963e-01_rb,0.14152e-01_rb,0.94977e-02_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.50552e-01_rb,0.46832e-01_rb,0.43092e-01_rb,0.38231e-01_rb,0.33230e-01_rb, &
        & 0.27672e-01_rb,0.21597e-01_rb,0.15000e-01_rb,0.11069e-01_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.50470e-01_rb,0.47126e-01_rb,0.43390e-01_rb,0.38763e-01_rb,0.33692e-01_rb, &
        & 0.28191e-01_rb,0.22303e-01_rb,0.15932e-01_rb,0.12529e-01_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.50251e-01_rb,0.47290e-01_rb,0.43621e-01_rb,0.39150e-01_rb,0.34250e-01_rb, &
        & 0.28907e-01_rb,0.23153e-01_rb,0.17091e-01_rb,0.14647e-01_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.50025e-01_rb,0.47335e-01_rb,0.43789e-01_rb,0.39526e-01_rb,0.34961e-01_rb, &
        & 0.29783e-01_rb,0.24207e-01_rb,0.18303e-01_rb,0.16505e-01_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.51775e-01_rb,0.47733e-01_rb,0.43604e-01_rb,0.38377e-01_rb,0.33010e-01_rb, &
        & 0.27211e-01_rb,0.20853e-01_rb,0.13952e-01_rb,0.10099e-01_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.51722e-01_rb,0.48082e-01_rb,0.43912e-01_rb,0.38859e-01_rb,0.33550e-01_rb, &
        & 0.27720e-01_rb,0.21535e-01_rb,0.14746e-01_rb,0.11460e-01_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.51577e-01_rb,0.48309e-01_rb,0.44107e-01_rb,0.39371e-01_rb,0.34132e-01_rb, &
        & 0.28431e-01_rb,0.22288e-01_rb,0.15640e-01_rb,0.12761e-01_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.51367e-01_rb,0.48474e-01_rb,0.44409e-01_rb,0.39819e-01_rb,0.34807e-01_rb, &
        & 0.29240e-01_rb,0.23192e-01_rb,0.16707e-01_rb,0.14096e-01_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.51186e-01_rb,0.48565e-01_rb,0.44658e-01_rb,0.40350e-01_rb,0.35545e-01_rb, &
        & 0.30221e-01_rb,0.24214e-01_rb,0.17744e-01_rb,0.15290e-01_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.52731e-01_rb,0.48681e-01_rb,0.44186e-01_rb,0.38733e-01_rb,0.33169e-01_rb, &
        & 0.27108e-01_rb,0.20634e-01_rb,0.13723e-01_rb,0.97949e-02_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.52654e-01_rb,0.49001e-01_rb,0.44433e-01_rb,0.39315e-01_rb,0.33776e-01_rb, &
        & 0.27772e-01_rb,0.21363e-01_rb,0.14506e-01_rb,0.11186e-01_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.52581e-01_rb,0.49321e-01_rb,0.44768e-01_rb,0.39827e-01_rb,0.34382e-01_rb, &
        & 0.28565e-01_rb,0.22205e-01_rb,0.15366e-01_rb,0.12310e-01_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.52474e-01_rb,0.49527e-01_rb,0.45170e-01_rb,0.40364e-01_rb,0.35120e-01_rb, &
        & 0.29376e-01_rb,0.23174e-01_rb,0.16305e-01_rb,0.14325e-01_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.52313e-01_rb,0.49659e-01_rb,0.45526e-01_rb,0.41026e-01_rb,0.35905e-01_rb, &
        & 0.30399e-01_rb,0.24222e-01_rb,0.17291e-01_rb,0.16045e-01_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.53347e-01_rb,0.49460e-01_rb,0.44667e-01_rb,0.39102e-01_rb,0.33300e-01_rb, &
        & 0.27005e-01_rb,0.20373e-01_rb,0.13403e-01_rb,0.88882e-02_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.53473e-01_rb,0.49893e-01_rb,0.44992e-01_rb,0.39672e-01_rb,0.33886e-01_rb, &
        & 0.27737e-01_rb,0.21157e-01_rb,0.14194e-01_rb,0.99472e-02_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.53543e-01_rb,0.50285e-01_rb,0.45484e-01_rb,0.40250e-01_rb,0.34541e-01_rb, &
        & 0.28510e-01_rb,0.22035e-01_rb,0.15061e-01_rb,0.11235e-01_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.53461e-01_rb,0.50524e-01_rb,0.45912e-01_rb,0.40827e-01_rb,0.35322e-01_rb, &
        & 0.29374e-01_rb,0.23048e-01_rb,0.15925e-01_rb,0.12822e-01_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.53278e-01_rb,0.50624e-01_rb,0.46303e-01_rb,0.41556e-01_rb,0.36186e-01_rb, &
        & 0.30414e-01_rb,0.24058e-01_rb,0.16868e-01_rb,0.14203e-01_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.37171e-01_rb,0.34434e-01_rb,0.32644e-01_rb,0.32703e-01_rb,0.33235e-01_rb, &
        & 0.36611e-01_rb,0.43526e-01_rb,0.50696e-01_rb,0.58297e-01_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.36741e-01_rb,0.34071e-01_rb,0.32455e-01_rb,0.33094e-01_rb,0.34229e-01_rb, &
        & 0.38792e-01_rb,0.46364e-01_rb,0.53998e-01_rb,0.61977e-01_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.36310e-01_rb,0.33795e-01_rb,0.32514e-01_rb,0.33548e-01_rb,0.35414e-01_rb, &
        & 0.41445e-01_rb,0.49627e-01_rb,0.57817e-01_rb,0.66049e-01_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.35845e-01_rb,0.33632e-01_rb,0.32631e-01_rb,0.34397e-01_rb,0.37428e-01_rb, &
        & 0.44985e-01_rb,0.53868e-01_rb,0.62763e-01_rb,0.71579e-01_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.35331e-01_rb,0.33379e-01_rb,0.32910e-01_rb,0.35528e-01_rb,0.39783e-01_rb, &
        & 0.48705e-01_rb,0.58316e-01_rb,0.67942e-01_rb,0.77160e-01_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.40448e-01_rb,0.37395e-01_rb,0.35029e-01_rb,0.34305e-01_rb,0.33991e-01_rb, &
        & 0.35316e-01_rb,0.40968e-01_rb,0.47712e-01_rb,0.54932e-01_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.39920e-01_rb,0.36968e-01_rb,0.34927e-01_rb,0.34599e-01_rb,0.34841e-01_rb, &
        & 0.37250e-01_rb,0.43923e-01_rb,0.51161e-01_rb,0.58827e-01_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.39405e-01_rb,0.36691e-01_rb,0.34999e-01_rb,0.35076e-01_rb,0.35924e-01_rb, &
        & 0.39772e-01_rb,0.47443e-01_rb,0.55253e-01_rb,0.63303e-01_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.38792e-01_rb,0.36414e-01_rb,0.35121e-01_rb,0.35787e-01_rb,0.37790e-01_rb, &
        & 0.43290e-01_rb,0.51773e-01_rb,0.60284e-01_rb,0.68983e-01_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.38190e-01_rb,0.36098e-01_rb,0.35328e-01_rb,0.36844e-01_rb,0.39952e-01_rb, &
        & 0.47291e-01_rb,0.56573e-01_rb,0.65870e-01_rb,0.75177e-01_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.43810e-01_rb,0.40500e-01_rb,0.37754e-01_rb,0.36293e-01_rb,0.34676e-01_rb, &
        & 0.34367e-01_rb,0.37493e-01_rb,0.43650e-01_rb,0.50087e-01_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.43199e-01_rb,0.40018e-01_rb,0.37716e-01_rb,0.36450e-01_rb,0.35395e-01_rb, &
        & 0.35866e-01_rb,0.40426e-01_rb,0.47076e-01_rb,0.53958e-01_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.42538e-01_rb,0.39757e-01_rb,0.37868e-01_rb,0.36772e-01_rb,0.36302e-01_rb, &
        & 0.37914e-01_rb,0.44070e-01_rb,0.51299e-01_rb,0.58670e-01_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.41856e-01_rb,0.39457e-01_rb,0.37979e-01_rb,0.37163e-01_rb,0.37947e-01_rb, &
        & 0.40970e-01_rb,0.48419e-01_rb,0.56334e-01_rb,0.64404e-01_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.41250e-01_rb,0.39154e-01_rb,0.38065e-01_rb,0.38049e-01_rb,0.39929e-01_rb, &
        & 0.44658e-01_rb,0.53223e-01_rb,0.61909e-01_rb,0.70665e-01_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.47088e-01_rb,0.43589e-01_rb,0.40605e-01_rb,0.38686e-01_rb,0.35713e-01_rb, &
        & 0.33942e-01_rb,0.34171e-01_rb,0.39143e-01_rb,0.44787e-01_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.46380e-01_rb,0.43081e-01_rb,0.40678e-01_rb,0.38772e-01_rb,0.36271e-01_rb, &
        & 0.35115e-01_rb,0.36728e-01_rb,0.42523e-01_rb,0.48625e-01_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.45787e-01_rb,0.42889e-01_rb,0.40853e-01_rb,0.39000e-01_rb,0.36901e-01_rb, &
        & 0.36794e-01_rb,0.40276e-01_rb,0.46812e-01_rb,0.53448e-01_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.45216e-01_rb,0.42628e-01_rb,0.40916e-01_rb,0.39286e-01_rb,0.38129e-01_rb, &
        & 0.39329e-01_rb,0.44499e-01_rb,0.51734e-01_rb,0.59041e-01_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.44698e-01_rb,0.42312e-01_rb,0.40992e-01_rb,0.39909e-01_rb,0.39896e-01_rb, &
        & 0.42322e-01_rb,0.49157e-01_rb,0.57134e-01_rb,0.65206e-01_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.50564e-01_rb,0.46864e-01_rb,0.43550e-01_rb,0.41145e-01_rb,0.37282e-01_rb, &
        & 0.33847e-01_rb,0.31921e-01_rb,0.34655e-01_rb,0.39581e-01_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.49853e-01_rb,0.46324e-01_rb,0.43735e-01_rb,0.41212e-01_rb,0.37826e-01_rb, &
        & 0.34760e-01_rb,0.33939e-01_rb,0.37988e-01_rb,0.43340e-01_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.49277e-01_rb,0.46142e-01_rb,0.43937e-01_rb,0.41409e-01_rb,0.38245e-01_rb, &
        & 0.36079e-01_rb,0.36891e-01_rb,0.42172e-01_rb,0.48104e-01_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.48731e-01_rb,0.45921e-01_rb,0.43964e-01_rb,0.41658e-01_rb,0.39073e-01_rb, &
        & 0.38270e-01_rb,0.40535e-01_rb,0.46833e-01_rb,0.53415e-01_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.48265e-01_rb,0.45648e-01_rb,0.43993e-01_rb,0.42096e-01_rb,0.40473e-01_rb, &
        & 0.40846e-01_rb,0.44703e-01_rb,0.51860e-01_rb,0.59120e-01_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.54155e-01_rb,0.50312e-01_rb,0.46699e-01_rb,0.43479e-01_rb,0.38935e-01_rb, &
        & 0.34460e-01_rb,0.30470e-01_rb,0.30214e-01_rb,0.34389e-01_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.53396e-01_rb,0.49741e-01_rb,0.46925e-01_rb,0.43573e-01_rb,0.39485e-01_rb, &
        & 0.35160e-01_rb,0.32122e-01_rb,0.33415e-01_rb,0.38087e-01_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.52823e-01_rb,0.49548e-01_rb,0.47130e-01_rb,0.43740e-01_rb,0.39828e-01_rb, &
        & 0.36044e-01_rb,0.34474e-01_rb,0.37304e-01_rb,0.42510e-01_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.52327e-01_rb,0.49384e-01_rb,0.47097e-01_rb,0.43937e-01_rb,0.40452e-01_rb, &
        & 0.37736e-01_rb,0.37393e-01_rb,0.41655e-01_rb,0.47452e-01_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.51903e-01_rb,0.49131e-01_rb,0.47026e-01_rb,0.44222e-01_rb,0.41687e-01_rb, &
        & 0.39818e-01_rb,0.40660e-01_rb,0.46275e-01_rb,0.52724e-01_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.58007e-01_rb,0.53733e-01_rb,0.49779e-01_rb,0.45887e-01_rb,0.40462e-01_rb, &
        & 0.35176e-01_rb,0.29765e-01_rb,0.26441e-01_rb,0.29320e-01_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.57151e-01_rb,0.53110e-01_rb,0.50041e-01_rb,0.45960e-01_rb,0.40973e-01_rb, &
        & 0.35887e-01_rb,0.31026e-01_rb,0.29123e-01_rb,0.32803e-01_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.56502e-01_rb,0.52846e-01_rb,0.50182e-01_rb,0.46056e-01_rb,0.41337e-01_rb, &
        & 0.36666e-01_rb,0.32781e-01_rb,0.32480e-01_rb,0.36858e-01_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.55999e-01_rb,0.52671e-01_rb,0.50072e-01_rb,0.46217e-01_rb,0.41835e-01_rb, &
        & 0.38000e-01_rb,0.35109e-01_rb,0.36308e-01_rb,0.41318e-01_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.55573e-01_rb,0.52382e-01_rb,0.49967e-01_rb,0.46401e-01_rb,0.42955e-01_rb, &
        & 0.39677e-01_rb,0.37716e-01_rb,0.40481e-01_rb,0.46095e-01_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.61929e-01_rb,0.57176e-01_rb,0.52730e-01_rb,0.48103e-01_rb,0.42015e-01_rb, &
        & 0.35869e-01_rb,0.29499e-01_rb,0.23665e-01_rb,0.24549e-01_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.61013e-01_rb,0.56553e-01_rb,0.52930e-01_rb,0.48080e-01_rb,0.42522e-01_rb, &
        & 0.36573e-01_rb,0.30581e-01_rb,0.25834e-01_rb,0.27791e-01_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.60303e-01_rb,0.56131e-01_rb,0.52872e-01_rb,0.48123e-01_rb,0.42911e-01_rb, &
        & 0.37343e-01_rb,0.31938e-01_rb,0.28446e-01_rb,0.31375e-01_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.59783e-01_rb,0.55843e-01_rb,0.52678e-01_rb,0.48331e-01_rb,0.43398e-01_rb, &
        & 0.38468e-01_rb,0.33698e-01_rb,0.31535e-01_rb,0.35398e-01_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.59221e-01_rb,0.55525e-01_rb,0.52626e-01_rb,0.48547e-01_rb,0.44309e-01_rb, &
        & 0.39969e-01_rb,0.35748e-01_rb,0.35119e-01_rb,0.39791e-01_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.65420e-01_rb,0.60290e-01_rb,0.55521e-01_rb,0.50117e-01_rb,0.43421e-01_rb, &
        & 0.36608e-01_rb,0.29349e-01_rb,0.21841e-01_rb,0.18774e-01_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.64534e-01_rb,0.59710e-01_rb,0.55627e-01_rb,0.50054e-01_rb,0.43842e-01_rb, &
        & 0.37357e-01_rb,0.30337e-01_rb,0.23525e-01_rb,0.22756e-01_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.63805e-01_rb,0.59237e-01_rb,0.55452e-01_rb,0.50019e-01_rb,0.44319e-01_rb, &
        & 0.38089e-01_rb,0.31491e-01_rb,0.25524e-01_rb,0.26401e-01_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.63315e-01_rb,0.59001e-01_rb,0.55252e-01_rb,0.50261e-01_rb,0.44802e-01_rb, &
        & 0.39006e-01_rb,0.32915e-01_rb,0.27972e-01_rb,0.30021e-01_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.62781e-01_rb,0.58825e-01_rb,0.55257e-01_rb,0.50524e-01_rb,0.45567e-01_rb, &
        & 0.40297e-01_rb,0.34643e-01_rb,0.30888e-01_rb,0.34021e-01_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.68132e-01_rb,0.62682e-01_rb,0.57743e-01_rb,0.51836e-01_rb,0.44670e-01_rb, &
        & 0.37300e-01_rb,0.29365e-01_rb,0.20724e-01_rb,0.15957e-01_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.67210e-01_rb,0.62151e-01_rb,0.57960e-01_rb,0.51867e-01_rb,0.45128e-01_rb, &
        & 0.38064e-01_rb,0.30245e-01_rb,0.22146e-01_rb,0.17704e-01_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.66632e-01_rb,0.61949e-01_rb,0.57882e-01_rb,0.51878e-01_rb,0.45637e-01_rb, &
        & 0.38758e-01_rb,0.31283e-01_rb,0.23744e-01_rb,0.20098e-01_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.66350e-01_rb,0.62044e-01_rb,0.57835e-01_rb,0.52180e-01_rb,0.46090e-01_rb, &
        & 0.39573e-01_rb,0.32509e-01_rb,0.25642e-01_rb,0.22076e-01_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.65965e-01_rb,0.62105e-01_rb,0.57959e-01_rb,0.52489e-01_rb,0.46765e-01_rb, &
        & 0.40717e-01_rb,0.34057e-01_rb,0.28015e-01_rb,0.24829e-01_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.69905e-01_rb,0.64410e-01_rb,0.59589e-01_rb,0.53150e-01_rb,0.45760e-01_rb, &
        & 0.38179e-01_rb,0.29785e-01_rb,0.20444e-01_rb,0.16092e-01_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.69271e-01_rb,0.64175e-01_rb,0.59782e-01_rb,0.53281e-01_rb,0.46486e-01_rb, &
        & 0.38949e-01_rb,0.30645e-01_rb,0.21725e-01_rb,0.17857e-01_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.69073e-01_rb,0.64491e-01_rb,0.59955e-01_rb,0.53684e-01_rb,0.46981e-01_rb, &
        & 0.39609e-01_rb,0.31674e-01_rb,0.23165e-01_rb,0.20394e-01_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.68904e-01_rb,0.64793e-01_rb,0.60179e-01_rb,0.54162e-01_rb,0.47530e-01_rb, &
        & 0.40495e-01_rb,0.32809e-01_rb,0.24930e-01_rb,0.23016e-01_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.68639e-01_rb,0.65010e-01_rb,0.60484e-01_rb,0.54605e-01_rb,0.48370e-01_rb, &
        & 0.41561e-01_rb,0.34350e-01_rb,0.26933e-01_rb,0.26488e-01_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.71529e-01_rb,0.66044e-01_rb,0.61072e-01_rb,0.54140e-01_rb,0.46654e-01_rb, &
        & 0.38794e-01_rb,0.29996e-01_rb,0.20236e-01_rb,0.16124e-01_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.71291e-01_rb,0.66315e-01_rb,0.61418e-01_rb,0.54500e-01_rb,0.47446e-01_rb, &
        & 0.39590e-01_rb,0.30905e-01_rb,0.21401e-01_rb,0.17542e-01_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.71222e-01_rb,0.66826e-01_rb,0.61770e-01_rb,0.55219e-01_rb,0.48148e-01_rb, &
        & 0.40334e-01_rb,0.31908e-01_rb,0.22727e-01_rb,0.19804e-01_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.71079e-01_rb,0.67236e-01_rb,0.62161e-01_rb,0.55849e-01_rb,0.48917e-01_rb, &
        & 0.41311e-01_rb,0.33042e-01_rb,0.24356e-01_rb,0.21082e-01_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.70901e-01_rb,0.67518e-01_rb,0.62549e-01_rb,0.56439e-01_rb,0.49900e-01_rb, &
        & 0.42406e-01_rb,0.34548e-01_rb,0.26106e-01_rb,0.23666e-01_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.73149e-01_rb,0.67708e-01_rb,0.62294e-01_rb,0.54947e-01_rb,0.47337e-01_rb, &
        & 0.39165e-01_rb,0.30053e-01_rb,0.20069e-01_rb,0.15304e-01_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.73122e-01_rb,0.68287e-01_rb,0.62803e-01_rb,0.55664e-01_rb,0.48241e-01_rb, &
        & 0.39986e-01_rb,0.31044e-01_rb,0.21133e-01_rb,0.17384e-01_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.73057e-01_rb,0.68826e-01_rb,0.63303e-01_rb,0.56536e-01_rb,0.49104e-01_rb, &
        & 0.40926e-01_rb,0.32050e-01_rb,0.22349e-01_rb,0.18964e-01_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.73037e-01_rb,0.69286e-01_rb,0.63846e-01_rb,0.57252e-01_rb,0.50018e-01_rb, &
        & 0.42025e-01_rb,0.33217e-01_rb,0.23850e-01_rb,0.20572e-01_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.73013e-01_rb,0.69604e-01_rb,0.64290e-01_rb,0.57954e-01_rb,0.51076e-01_rb, &
        & 0.43212e-01_rb,0.34653e-01_rb,0.25391e-01_rb,0.23448e-01_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.43248e-01_rb,0.41017e-01_rb,0.40213e-01_rb,0.39821e-01_rb,0.45256e-01_rb, &
        & 0.55821e-01_rb,0.66932e-01_rb,0.78037e-01_rb,0.88086e-01_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.42407e-01_rb,0.40307e-01_rb,0.40078e-01_rb,0.40681e-01_rb,0.47543e-01_rb, &
        & 0.59099e-01_rb,0.70845e-01_rb,0.82585e-01_rb,0.93228e-01_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.41692e-01_rb,0.39727e-01_rb,0.39865e-01_rb,0.41830e-01_rb,0.50093e-01_rb, &
        & 0.62450e-01_rb,0.74841e-01_rb,0.87232e-01_rb,0.98476e-01_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.41002e-01_rb,0.39248e-01_rb,0.39945e-01_rb,0.42942e-01_rb,0.52768e-01_rb, &
        & 0.65829e-01_rb,0.78890e-01_rb,0.91937e-01_rb,0.10362e+00_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.40428e-01_rb,0.39045e-01_rb,0.40116e-01_rb,0.44981e-01_rb,0.56655e-01_rb, &
        & 0.70681e-01_rb,0.84705e-01_rb,0.98731e-01_rb,0.11111e+00_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.47227e-01_rb,0.44198e-01_rb,0.42698e-01_rb,0.42511e-01_rb,0.46394e-01_rb, &
        & 0.55784e-01_rb,0.66888e-01_rb,0.77989e-01_rb,0.88812e-01_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.46375e-01_rb,0.43473e-01_rb,0.42428e-01_rb,0.43299e-01_rb,0.48252e-01_rb, &
        & 0.59000e-01_rb,0.70708e-01_rb,0.82423e-01_rb,0.93885e-01_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.45650e-01_rb,0.42988e-01_rb,0.42307e-01_rb,0.44252e-01_rb,0.50414e-01_rb, &
        & 0.62373e-01_rb,0.74750e-01_rb,0.87127e-01_rb,0.99197e-01_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.44980e-01_rb,0.42686e-01_rb,0.42476e-01_rb,0.45355e-01_rb,0.53027e-01_rb, &
        & 0.66067e-01_rb,0.79179e-01_rb,0.92288e-01_rb,0.10480e+00_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.44383e-01_rb,0.42475e-01_rb,0.42723e-01_rb,0.47155e-01_rb,0.57007e-01_rb, &
        & 0.71138e-01_rb,0.85276e-01_rb,0.99396e-01_rb,0.11272e+00_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.51829e-01_rb,0.48303e-01_rb,0.45805e-01_rb,0.45348e-01_rb,0.47474e-01_rb, &
        & 0.53773e-01_rb,0.64381e-01_rb,0.75065e-01_rb,0.85879e-01_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.50945e-01_rb,0.47626e-01_rb,0.45531e-01_rb,0.46104e-01_rb,0.48805e-01_rb, &
        & 0.57008e-01_rb,0.68342e-01_rb,0.79675e-01_rb,0.91103e-01_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.50329e-01_rb,0.47163e-01_rb,0.45393e-01_rb,0.46795e-01_rb,0.50514e-01_rb, &
        & 0.60488e-01_rb,0.72503e-01_rb,0.84519e-01_rb,0.96625e-01_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.49725e-01_rb,0.46894e-01_rb,0.45600e-01_rb,0.47784e-01_rb,0.52670e-01_rb, &
        & 0.64382e-01_rb,0.77172e-01_rb,0.89966e-01_rb,0.10271e+00_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.49109e-01_rb,0.46633e-01_rb,0.45896e-01_rb,0.49191e-01_rb,0.56129e-01_rb, &
        & 0.69438e-01_rb,0.83245e-01_rb,0.97061e-01_rb,0.11071e+00_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.57082e-01_rb,0.53110e-01_rb,0.49763e-01_rb,0.48460e-01_rb,0.48719e-01_rb, &
        & 0.51181e-01_rb,0.59984e-01_rb,0.69952e-01_rb,0.80045e-01_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.56133e-01_rb,0.52450e-01_rb,0.49568e-01_rb,0.49064e-01_rb,0.49725e-01_rb, &
        & 0.54041e-01_rb,0.64091e-01_rb,0.74728e-01_rb,0.85496e-01_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.55439e-01_rb,0.51866e-01_rb,0.49497e-01_rb,0.49475e-01_rb,0.51076e-01_rb, &
        & 0.57243e-01_rb,0.68327e-01_rb,0.79652e-01_rb,0.91098e-01_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.54777e-01_rb,0.51575e-01_rb,0.49681e-01_rb,0.50221e-01_rb,0.52765e-01_rb, &
        & 0.61084e-01_rb,0.73197e-01_rb,0.85337e-01_rb,0.97552e-01_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.54098e-01_rb,0.51233e-01_rb,0.49878e-01_rb,0.51295e-01_rb,0.55484e-01_rb, &
        & 0.66162e-01_rb,0.79298e-01_rb,0.92434e-01_rb,0.10558e+00_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.62544e-01_rb,0.58281e-01_rb,0.54351e-01_rb,0.52107e-01_rb,0.50211e-01_rb, &
        & 0.49416e-01_rb,0.54519e-01_rb,0.63528e-01_rb,0.72634e-01_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.61658e-01_rb,0.57644e-01_rb,0.54210e-01_rb,0.52563e-01_rb,0.50949e-01_rb, &
        & 0.51794e-01_rb,0.58802e-01_rb,0.68560e-01_rb,0.78383e-01_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.60918e-01_rb,0.57016e-01_rb,0.54117e-01_rb,0.52719e-01_rb,0.52074e-01_rb, &
        & 0.54457e-01_rb,0.63161e-01_rb,0.73629e-01_rb,0.84158e-01_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.60164e-01_rb,0.56692e-01_rb,0.54302e-01_rb,0.53086e-01_rb,0.53404e-01_rb, &
        & 0.57689e-01_rb,0.68126e-01_rb,0.79422e-01_rb,0.90757e-01_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.59366e-01_rb,0.56267e-01_rb,0.54377e-01_rb,0.53811e-01_rb,0.55627e-01_rb, &
        & 0.62331e-01_rb,0.74295e-01_rb,0.86596e-01_rb,0.98923e-01_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.68500e-01_rb,0.63504e-01_rb,0.59102e-01_rb,0.56112e-01_rb,0.52198e-01_rb, &
        & 0.48725e-01_rb,0.49458e-01_rb,0.56540e-01_rb,0.64611e-01_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.67642e-01_rb,0.62951e-01_rb,0.59010e-01_rb,0.56509e-01_rb,0.52640e-01_rb, &
        & 0.50781e-01_rb,0.53492e-01_rb,0.61845e-01_rb,0.70650e-01_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.66730e-01_rb,0.62270e-01_rb,0.58880e-01_rb,0.56541e-01_rb,0.53578e-01_rb, &
        & 0.52912e-01_rb,0.57553e-01_rb,0.66900e-01_rb,0.76420e-01_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.65853e-01_rb,0.61851e-01_rb,0.59079e-01_rb,0.56674e-01_rb,0.54628e-01_rb, &
        & 0.55405e-01_rb,0.62327e-01_rb,0.72621e-01_rb,0.82946e-01_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.64917e-01_rb,0.61395e-01_rb,0.59088e-01_rb,0.57140e-01_rb,0.56317e-01_rb, &
        & 0.59242e-01_rb,0.68519e-01_rb,0.79819e-01_rb,0.91147e-01_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.74814e-01_rb,0.69115e-01_rb,0.63783e-01_rb,0.59970e-01_rb,0.54833e-01_rb, &
        & 0.49011e-01_rb,0.46051e-01_rb,0.49756e-01_rb,0.56816e-01_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.73755e-01_rb,0.68509e-01_rb,0.63672e-01_rb,0.60441e-01_rb,0.55205e-01_rb, &
        & 0.50759e-01_rb,0.49447e-01_rb,0.55044e-01_rb,0.62843e-01_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.72664e-01_rb,0.67698e-01_rb,0.63630e-01_rb,0.60499e-01_rb,0.55993e-01_rb, &
        & 0.52467e-01_rb,0.52809e-01_rb,0.59966e-01_rb,0.68463e-01_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.71633e-01_rb,0.67174e-01_rb,0.63866e-01_rb,0.60581e-01_rb,0.56744e-01_rb, &
        & 0.54377e-01_rb,0.56941e-01_rb,0.65640e-01_rb,0.74923e-01_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.70621e-01_rb,0.66745e-01_rb,0.63972e-01_rb,0.60930e-01_rb,0.57801e-01_rb, &
        & 0.57488e-01_rb,0.62631e-01_rb,0.72711e-01_rb,0.82984e-01_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.81108e-01_rb,0.74731e-01_rb,0.68669e-01_rb,0.63699e-01_rb,0.57482e-01_rb, &
        & 0.50207e-01_rb,0.44226e-01_rb,0.43565e-01_rb,0.49479e-01_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.79714e-01_rb,0.73898e-01_rb,0.68659e-01_rb,0.64276e-01_rb,0.57881e-01_rb, &
        & 0.51786e-01_rb,0.46974e-01_rb,0.48378e-01_rb,0.55162e-01_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.78456e-01_rb,0.73135e-01_rb,0.68758e-01_rb,0.64370e-01_rb,0.58662e-01_rb, &
        & 0.53164e-01_rb,0.49684e-01_rb,0.53168e-01_rb,0.60666e-01_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.77309e-01_rb,0.72680e-01_rb,0.69000e-01_rb,0.64433e-01_rb,0.59353e-01_rb, &
        & 0.54540e-01_rb,0.53042e-01_rb,0.58717e-01_rb,0.66992e-01_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.76406e-01_rb,0.72288e-01_rb,0.69084e-01_rb,0.64775e-01_rb,0.60105e-01_rb, &
        & 0.56876e-01_rb,0.57740e-01_rb,0.65467e-01_rb,0.74695e-01_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.87281e-01_rb,0.80336e-01_rb,0.73586e-01_rb,0.67589e-01_rb,0.60164e-01_rb, &
        & 0.51553e-01_rb,0.43512e-01_rb,0.38366e-01_rb,0.42260e-01_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.85638e-01_rb,0.79229e-01_rb,0.73710e-01_rb,0.68169e-01_rb,0.60640e-01_rb, &
        & 0.53028e-01_rb,0.45744e-01_rb,0.42413e-01_rb,0.47570e-01_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.84292e-01_rb,0.78481e-01_rb,0.73900e-01_rb,0.68269e-01_rb,0.61342e-01_rb, &
        & 0.54381e-01_rb,0.47902e-01_rb,0.46760e-01_rb,0.52980e-01_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.83115e-01_rb,0.77996e-01_rb,0.74142e-01_rb,0.68304e-01_rb,0.62073e-01_rb, &
        & 0.55597e-01_rb,0.50525e-01_rb,0.51779e-01_rb,0.58954e-01_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.82186e-01_rb,0.77706e-01_rb,0.74161e-01_rb,0.68663e-01_rb,0.62685e-01_rb, &
        & 0.57315e-01_rb,0.54189e-01_rb,0.58003e-01_rb,0.66142e-01_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.93285e-01_rb,0.85813e-01_rb,0.78575e-01_rb,0.71552e-01_rb,0.62937e-01_rb, &
        & 0.53130e-01_rb,0.43544e-01_rb,0.34852e-01_rb,0.25971e-01_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.91673e-01_rb,0.84652e-01_rb,0.78625e-01_rb,0.72093e-01_rb,0.63376e-01_rb, &
        & 0.54541e-01_rb,0.45576e-01_rb,0.38102e-01_rb,0.28356e-01_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.90273e-01_rb,0.83828e-01_rb,0.78840e-01_rb,0.72190e-01_rb,0.64026e-01_rb, &
        & 0.55889e-01_rb,0.47404e-01_rb,0.41695e-01_rb,0.31456e-01_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.88982e-01_rb,0.83233e-01_rb,0.79026e-01_rb,0.72231e-01_rb,0.64765e-01_rb, &
        & 0.57025e-01_rb,0.49463e-01_rb,0.46003e-01_rb,0.35501e-01_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.88015e-01_rb,0.82954e-01_rb,0.78987e-01_rb,0.72605e-01_rb,0.65378e-01_rb, &
        & 0.58432e-01_rb,0.52293e-01_rb,0.51371e-01_rb,0.39951e-01_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.98224e-01_rb,0.90470e-01_rb,0.83283e-01_rb,0.75729e-01_rb,0.65953e-01_rb, &
        & 0.55333e-01_rb,0.44607e-01_rb,0.33712e-01_rb,0.29825e-01_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.96868e-01_rb,0.89565e-01_rb,0.83537e-01_rb,0.76001e-01_rb,0.66302e-01_rb, &
        & 0.56711e-01_rb,0.46387e-01_rb,0.36339e-01_rb,0.33228e-01_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.95464e-01_rb,0.88759e-01_rb,0.83774e-01_rb,0.75894e-01_rb,0.67115e-01_rb, &
        & 0.57847e-01_rb,0.48034e-01_rb,0.39456e-01_rb,0.36088e-01_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.94340e-01_rb,0.88366e-01_rb,0.83721e-01_rb,0.76044e-01_rb,0.67766e-01_rb, &
        & 0.58947e-01_rb,0.50010e-01_rb,0.43247e-01_rb,0.40125e-01_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.93518e-01_rb,0.88039e-01_rb,0.83475e-01_rb,0.76422e-01_rb,0.68414e-01_rb, &
        & 0.60495e-01_rb,0.52431e-01_rb,0.48069e-01_rb,0.44798e-01_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.10275e+00_rb,0.94729e-01_rb,0.87688e-01_rb,0.79394e-01_rb,0.68734e-01_rb, &
        & 0.57476e-01_rb,0.45697e-01_rb,0.32943e-01_rb,0.29614e-01_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.10159e+00_rb,0.94167e-01_rb,0.88202e-01_rb,0.79488e-01_rb,0.69232e-01_rb, &
        & 0.58694e-01_rb,0.47208e-01_rb,0.35215e-01_rb,0.33437e-01_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.10046e+00_rb,0.93671e-01_rb,0.88347e-01_rb,0.79409e-01_rb,0.69985e-01_rb, &
        & 0.59703e-01_rb,0.48739e-01_rb,0.37920e-01_rb,0.36670e-01_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.99712e-01_rb,0.93473e-01_rb,0.88135e-01_rb,0.79671e-01_rb,0.70541e-01_rb, &
        & 0.60786e-01_rb,0.50658e-01_rb,0.41158e-01_rb,0.41155e-01_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.99052e-01_rb,0.93236e-01_rb,0.87897e-01_rb,0.79933e-01_rb,0.71310e-01_rb, &
        & 0.62364e-01_rb,0.52850e-01_rb,0.45390e-01_rb,0.47055e-01_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.10693e+00_rb,0.98594e-01_rb,0.91730e-01_rb,0.82575e-01_rb,0.71131e-01_rb, &
        & 0.59390e-01_rb,0.46621e-01_rb,0.32428e-01_rb,0.26794e-01_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.10593e+00_rb,0.98329e-01_rb,0.92368e-01_rb,0.82651e-01_rb,0.71827e-01_rb, &
        & 0.60538e-01_rb,0.47868e-01_rb,0.34468e-01_rb,0.29114e-01_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.10521e+00_rb,0.98305e-01_rb,0.92444e-01_rb,0.82725e-01_rb,0.72549e-01_rb, &
        & 0.61456e-01_rb,0.49349e-01_rb,0.36800e-01_rb,0.32781e-01_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.10471e+00_rb,0.98393e-01_rb,0.92240e-01_rb,0.83082e-01_rb,0.73117e-01_rb, &
        & 0.62504e-01_rb,0.51152e-01_rb,0.39616e-01_rb,0.37780e-01_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.10409e+00_rb,0.98364e-01_rb,0.92131e-01_rb,0.83350e-01_rb,0.73995e-01_rb, &
        & 0.64047e-01_rb,0.53327e-01_rb,0.43375e-01_rb,0.42285e-01_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.52810e-01_rb,0.49450e-01_rb,0.49647e-01_rb,0.57104e-01_rb,0.73433e-01_rb, &
        & 0.91743e-01_rb,0.11005e+00_rb,0.12834e+00_rb,0.14491e+00_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.51653e-01_rb,0.48971e-01_rb,0.50186e-01_rb,0.59440e-01_rb,0.77723e-01_rb, &
        & 0.97098e-01_rb,0.11647e+00_rb,0.13583e+00_rb,0.15375e+00_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.50696e-01_rb,0.48582e-01_rb,0.50737e-01_rb,0.61667e-01_rb,0.81568e-01_rb, &
        & 0.10190e+00_rb,0.12223e+00_rb,0.14254e+00_rb,0.16133e+00_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.49759e-01_rb,0.48321e-01_rb,0.51484e-01_rb,0.64441e-01_rb,0.85673e-01_rb, &
        & 0.10703e+00_rb,0.12837e+00_rb,0.14971e+00_rb,0.16936e+00_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.48804e-01_rb,0.48067e-01_rb,0.52560e-01_rb,0.67354e-01_rb,0.89656e-01_rb, &
        & 0.11200e+00_rb,0.13434e+00_rb,0.15666e+00_rb,0.17718e+00_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.57042e-01_rb,0.54069e-01_rb,0.53905e-01_rb,0.58729e-01_rb,0.73592e-01_rb, &
        & 0.91941e-01_rb,0.11029e+00_rb,0.12863e+00_rb,0.14602e+00_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.55841e-01_rb,0.53550e-01_rb,0.54566e-01_rb,0.60781e-01_rb,0.78168e-01_rb, &
        & 0.97652e-01_rb,0.11714e+00_rb,0.13661e+00_rb,0.15506e+00_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.54920e-01_rb,0.52981e-01_rb,0.54943e-01_rb,0.62919e-01_rb,0.82379e-01_rb, &
        & 0.10292e+00_rb,0.12345e+00_rb,0.14397e+00_rb,0.16344e+00_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.54055e-01_rb,0.52591e-01_rb,0.55488e-01_rb,0.65511e-01_rb,0.86681e-01_rb, &
        & 0.10829e+00_rb,0.12990e+00_rb,0.15147e+00_rb,0.17196e+00_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.53226e-01_rb,0.52334e-01_rb,0.56238e-01_rb,0.68429e-01_rb,0.90830e-01_rb, &
        & 0.11345e+00_rb,0.13607e+00_rb,0.15868e+00_rb,0.17998e+00_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.62670e-01_rb,0.59309e-01_rb,0.58772e-01_rb,0.61122e-01_rb,0.72960e-01_rb, &
        & 0.91113e-01_rb,0.10930e+00_rb,0.12747e+00_rb,0.14514e+00_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.61411e-01_rb,0.58632e-01_rb,0.59230e-01_rb,0.62688e-01_rb,0.77748e-01_rb, &
        & 0.97139e-01_rb,0.11652e+00_rb,0.13590e+00_rb,0.15479e+00_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.60334e-01_rb,0.57923e-01_rb,0.59411e-01_rb,0.64539e-01_rb,0.82093e-01_rb, &
        & 0.10257e+00_rb,0.12305e+00_rb,0.14350e+00_rb,0.16343e+00_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.59451e-01_rb,0.57416e-01_rb,0.59633e-01_rb,0.66820e-01_rb,0.86494e-01_rb, &
        & 0.10804e+00_rb,0.12960e+00_rb,0.15112e+00_rb,0.17209e+00_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.58597e-01_rb,0.57143e-01_rb,0.60240e-01_rb,0.69594e-01_rb,0.91059e-01_rb, &
        & 0.11373e+00_rb,0.13640e+00_rb,0.15905e+00_rb,0.18103e+00_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.69822e-01_rb,0.65601e-01_rb,0.63945e-01_rb,0.64755e-01_rb,0.73038e-01_rb, &
        & 0.90165e-01_rb,0.10817e+00_rb,0.12616e+00_rb,0.14389e+00_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.68507e-01_rb,0.64727e-01_rb,0.64234e-01_rb,0.66213e-01_rb,0.77749e-01_rb, &
        & 0.96668e-01_rb,0.11597e+00_rb,0.13524e+00_rb,0.15427e+00_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.67276e-01_rb,0.63962e-01_rb,0.64238e-01_rb,0.67909e-01_rb,0.82185e-01_rb, &
        & 0.10254e+00_rb,0.12301e+00_rb,0.14345e+00_rb,0.16365e+00_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.66381e-01_rb,0.63334e-01_rb,0.64220e-01_rb,0.69770e-01_rb,0.86724e-01_rb, &
        & 0.10832e+00_rb,0.12994e+00_rb,0.15151e+00_rb,0.17282e+00_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.65437e-01_rb,0.62972e-01_rb,0.64673e-01_rb,0.72222e-01_rb,0.91630e-01_rb, &
        & 0.11445e+00_rb,0.13728e+00_rb,0.16006e+00_rb,0.18257e+00_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.78344e-01_rb,0.73091e-01_rb,0.69713e-01_rb,0.69214e-01_rb,0.74668e-01_rb, &
        & 0.88915e-01_rb,0.10667e+00_rb,0.12441e+00_rb,0.14209e+00_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.76773e-01_rb,0.71981e-01_rb,0.69752e-01_rb,0.70736e-01_rb,0.78904e-01_rb, &
        & 0.95797e-01_rb,0.11493e+00_rb,0.13406e+00_rb,0.15311e+00_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.75310e-01_rb,0.71068e-01_rb,0.69651e-01_rb,0.72503e-01_rb,0.83070e-01_rb, &
        & 0.10214e+00_rb,0.12251e+00_rb,0.14289e+00_rb,0.16320e+00_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.74250e-01_rb,0.70202e-01_rb,0.69499e-01_rb,0.74333e-01_rb,0.87301e-01_rb, &
        & 0.10818e+00_rb,0.12976e+00_rb,0.15132e+00_rb,0.17281e+00_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.73209e-01_rb,0.69739e-01_rb,0.69847e-01_rb,0.76315e-01_rb,0.91870e-01_rb, &
        & 0.11438e+00_rb,0.13720e+00_rb,0.16000e+00_rb,0.18274e+00_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.87696e-01_rb,0.81522e-01_rb,0.76509e-01_rb,0.74345e-01_rb,0.77107e-01_rb, &
        & 0.86456e-01_rb,0.10266e+00_rb,0.11975e+00_rb,0.13684e+00_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.85834e-01_rb,0.80104e-01_rb,0.76319e-01_rb,0.75879e-01_rb,0.80970e-01_rb, &
        & 0.93234e-01_rb,0.11135e+00_rb,0.12988e+00_rb,0.14843e+00_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.84103e-01_rb,0.79034e-01_rb,0.76204e-01_rb,0.77685e-01_rb,0.84459e-01_rb, &
        & 0.99600e-01_rb,0.11936e+00_rb,0.13921e+00_rb,0.15907e+00_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.82699e-01_rb,0.78102e-01_rb,0.75915e-01_rb,0.79277e-01_rb,0.87923e-01_rb, &
        & 0.10572e+00_rb,0.12681e+00_rb,0.14791e+00_rb,0.16901e+00_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.81604e-01_rb,0.77529e-01_rb,0.76175e-01_rb,0.80727e-01_rb,0.91640e-01_rb, &
        & 0.11182e+00_rb,0.13414e+00_rb,0.15644e+00_rb,0.17875e+00_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.97322e-01_rb,0.90329e-01_rb,0.84313e-01_rb,0.80452e-01_rb,0.80118e-01_rb, &
        & 0.84204e-01_rb,0.96138e-01_rb,0.11212e+00_rb,0.12812e+00_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.95378e-01_rb,0.88856e-01_rb,0.83999e-01_rb,0.81744e-01_rb,0.83529e-01_rb, &
        & 0.90072e-01_rb,0.10504e+00_rb,0.12254e+00_rb,0.14003e+00_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.93471e-01_rb,0.87725e-01_rb,0.83968e-01_rb,0.83424e-01_rb,0.86362e-01_rb, &
        & 0.95913e-01_rb,0.11338e+00_rb,0.13227e+00_rb,0.15114e+00_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.91861e-01_rb,0.86679e-01_rb,0.83638e-01_rb,0.84829e-01_rb,0.89126e-01_rb, &
        & 0.10167e+00_rb,0.12111e+00_rb,0.14127e+00_rb,0.16142e+00_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.90669e-01_rb,0.86079e-01_rb,0.83677e-01_rb,0.85955e-01_rb,0.92165e-01_rb, &
        & 0.10751e+00_rb,0.12862e+00_rb,0.15005e+00_rb,0.17145e+00_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.10748e+00_rb,0.99760e-01_rb,0.92641e-01_rb,0.87386e-01_rb,0.83918e-01_rb, &
        & 0.82807e-01_rb,0.88461e-01_rb,0.10199e+00_rb,0.11655e+00_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.10548e+00_rb,0.98347e-01_rb,0.92153e-01_rb,0.88724e-01_rb,0.86807e-01_rb, &
        & 0.87435e-01_rb,0.96918e-01_rb,0.11262e+00_rb,0.12866e+00_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.10345e+00_rb,0.96941e-01_rb,0.92269e-01_rb,0.90148e-01_rb,0.89023e-01_rb, &
        & 0.92390e-01_rb,0.10517e+00_rb,0.12260e+00_rb,0.14006e+00_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.10168e+00_rb,0.95648e-01_rb,0.92050e-01_rb,0.91217e-01_rb,0.91169e-01_rb, &
        & 0.97530e-01_rb,0.11302e+00_rb,0.13185e+00_rb,0.15061e+00_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.10031e+00_rb,0.94918e-01_rb,0.92030e-01_rb,0.91944e-01_rb,0.93599e-01_rb, &
        & 0.10280e+00_rb,0.12083e+00_rb,0.14095e+00_rb,0.16101e+00_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.11850e+00_rb,0.10963e+00_rb,0.10118e+00_rb,0.94592e-01_rb,0.88496e-01_rb, &
        & 0.82777e-01_rb,0.81368e-01_rb,0.90482e-01_rb,0.10334e+00_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.11636e+00_rb,0.10836e+00_rb,0.10080e+00_rb,0.96151e-01_rb,0.90961e-01_rb, &
        & 0.86437e-01_rb,0.88759e-01_rb,0.10103e+00_rb,0.11540e+00_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.11416e+00_rb,0.10675e+00_rb,0.10109e+00_rb,0.97591e-01_rb,0.92676e-01_rb, &
        & 0.90148e-01_rb,0.96288e-01_rb,0.11094e+00_rb,0.12669e+00_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.11214e+00_rb,0.10527e+00_rb,0.10096e+00_rb,0.98437e-01_rb,0.94256e-01_rb, &
        & 0.94326e-01_rb,0.10386e+00_rb,0.12050e+00_rb,0.13763e+00_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.11054e+00_rb,0.10425e+00_rb,0.10085e+00_rb,0.98717e-01_rb,0.96200e-01_rb, &
        & 0.98935e-01_rb,0.11173e+00_rb,0.13010e+00_rb,0.14859e+00_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.13016e+00_rb,0.11986e+00_rb,0.10995e+00_rb,0.10196e+00_rb,0.93774e-01_rb, &
        & 0.84360e-01_rb,0.76878e-01_rb,0.79680e-01_rb,0.67359e-01_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.12774e+00_rb,0.11853e+00_rb,0.10979e+00_rb,0.10367e+00_rb,0.96160e-01_rb, &
        & 0.87173e-01_rb,0.82710e-01_rb,0.89689e-01_rb,0.82456e-01_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.12531e+00_rb,0.11685e+00_rb,0.11022e+00_rb,0.10509e+00_rb,0.97450e-01_rb, &
        & 0.90061e-01_rb,0.89222e-01_rb,0.99588e-01_rb,0.96338e-01_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.12312e+00_rb,0.11523e+00_rb,0.11015e+00_rb,0.10584e+00_rb,0.98534e-01_rb, &
        & 0.93531e-01_rb,0.96262e-01_rb,0.10942e+00_rb,0.11043e+00_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.12128e+00_rb,0.11400e+00_rb,0.10998e+00_rb,0.10588e+00_rb,0.10008e+00_rb, &
        & 0.97359e-01_rb,0.10364e+00_rb,0.11932e+00_rb,0.12582e+00_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.14105e+00_rb,0.12987e+00_rb,0.11910e+00_rb,0.11026e+00_rb,0.10035e+00_rb, &
        & 0.88081e-01_rb,0.76687e-01_rb,0.74205e-01_rb,0.50899e-01_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.13825e+00_rb,0.12810e+00_rb,0.11908e+00_rb,0.11198e+00_rb,0.10215e+00_rb, &
        & 0.90197e-01_rb,0.81650e-01_rb,0.83605e-01_rb,0.56975e-01_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.13566e+00_rb,0.12624e+00_rb,0.11939e+00_rb,0.11316e+00_rb,0.10299e+00_rb, &
        & 0.92874e-01_rb,0.87279e-01_rb,0.93203e-01_rb,0.64736e-01_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.13334e+00_rb,0.12460e+00_rb,0.11935e+00_rb,0.11337e+00_rb,0.10395e+00_rb, &
        & 0.95956e-01_rb,0.93410e-01_rb,0.10293e+00_rb,0.71284e-01_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.13131e+00_rb,0.12363e+00_rb,0.11937e+00_rb,0.11314e+00_rb,0.10517e+00_rb, &
        & 0.98969e-01_rb,0.10016e+00_rb,0.11304e+00_rb,0.78587e-01_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.15212e+00_rb,0.13991e+00_rb,0.12851e+00_rb,0.11872e+00_rb,0.10663e+00_rb, &
        & 0.91867e-01_rb,0.77350e-01_rb,0.69757e-01_rb,0.57657e-01_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.14895e+00_rb,0.13760e+00_rb,0.12846e+00_rb,0.12032e+00_rb,0.10790e+00_rb, &
        & 0.93798e-01_rb,0.81816e-01_rb,0.78361e-01_rb,0.63811e-01_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.14609e+00_rb,0.13558e+00_rb,0.12854e+00_rb,0.12100e+00_rb,0.10856e+00_rb, &
        & 0.96260e-01_rb,0.86684e-01_rb,0.87192e-01_rb,0.72200e-01_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.14343e+00_rb,0.13400e+00_rb,0.12849e+00_rb,0.12073e+00_rb,0.10951e+00_rb, &
        & 0.98753e-01_rb,0.91530e-01_rb,0.96391e-01_rb,0.79517e-01_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.14120e+00_rb,0.13303e+00_rb,0.12833e+00_rb,0.12041e+00_rb,0.11038e+00_rb, &
        & 0.10129e+00_rb,0.97562e-01_rb,0.10658e+00_rb,0.84371e-01_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.16284e+00_rb,0.14976e+00_rb,0.13792e+00_rb,0.12684e+00_rb,0.11275e+00_rb, &
        & 0.95645e-01_rb,0.78767e-01_rb,0.66366e-01_rb,0.59038e-01_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.15947e+00_rb,0.14721e+00_rb,0.13779e+00_rb,0.12800e+00_rb,0.11359e+00_rb, &
        & 0.97597e-01_rb,0.82782e-01_rb,0.73753e-01_rb,0.67374e-01_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.15626e+00_rb,0.14498e+00_rb,0.13763e+00_rb,0.12811e+00_rb,0.11414e+00_rb, &
        & 0.99826e-01_rb,0.86643e-01_rb,0.81525e-01_rb,0.75874e-01_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.15335e+00_rb,0.14330e+00_rb,0.13746e+00_rb,0.12758e+00_rb,0.11501e+00_rb, &
        & 0.10172e+00_rb,0.90750e-01_rb,0.90246e-01_rb,0.81744e-01_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.15088e+00_rb,0.14217e+00_rb,0.13690e+00_rb,0.12723e+00_rb,0.11557e+00_rb, &
        & 0.10388e+00_rb,0.96218e-01_rb,0.10042e+00_rb,0.88529e-01_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.62837e-01_rb,0.60695e-01_rb,0.77720e-01_rb,0.11279e+00_rb,0.15033e+00_rb, &
        & 0.18788e+00_rb,0.22539e+00_rb,0.26284e+00_rb,0.29421e+00_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.61599e-01_rb,0.60148e-01_rb,0.79591e-01_rb,0.11750e+00_rb,0.15661e+00_rb, &
        & 0.19570e+00_rb,0.23478e+00_rb,0.27378e+00_rb,0.30578e+00_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.60827e-01_rb,0.59792e-01_rb,0.82175e-01_rb,0.12253e+00_rb,0.16330e+00_rb, &
        & 0.20407e+00_rb,0.24482e+00_rb,0.28548e+00_rb,0.31829e+00_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.60773e-01_rb,0.59852e-01_rb,0.85295e-01_rb,0.12765e+00_rb,0.17012e+00_rb, &
        & 0.21258e+00_rb,0.25502e+00_rb,0.29736e+00_rb,0.33114e+00_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.60579e-01_rb,0.60436e-01_rb,0.88993e-01_rb,0.13334e+00_rb,0.17771e+00_rb, &
        & 0.22206e+00_rb,0.26642e+00_rb,0.31066e+00_rb,0.34557e+00_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.70150e-01_rb,0.67933e-01_rb,0.82763e-01_rb,0.11770e+00_rb,0.15687e+00_rb, &
        & 0.19605e+00_rb,0.23520e+00_rb,0.27426e+00_rb,0.30923e+00_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.68737e-01_rb,0.67168e-01_rb,0.84453e-01_rb,0.12329e+00_rb,0.16434e+00_rb, &
        & 0.20537e+00_rb,0.24638e+00_rb,0.28735e+00_rb,0.32358e+00_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.67693e-01_rb,0.66713e-01_rb,0.87076e-01_rb,0.12893e+00_rb,0.17184e+00_rb, &
        & 0.21474e+00_rb,0.25764e+00_rb,0.30044e+00_rb,0.33811e+00_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.67352e-01_rb,0.66652e-01_rb,0.90088e-01_rb,0.13441e+00_rb,0.17914e+00_rb, &
        & 0.22388e+00_rb,0.26860e+00_rb,0.31321e+00_rb,0.35245e+00_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.66923e-01_rb,0.67069e-01_rb,0.93870e-01_rb,0.14034e+00_rb,0.18704e+00_rb, &
        & 0.23374e+00_rb,0.28042e+00_rb,0.32702e+00_rb,0.36794e+00_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.79385e-01_rb,0.76263e-01_rb,0.87808e-01_rb,0.12001e+00_rb,0.15997e+00_rb, &
        & 0.19992e+00_rb,0.23984e+00_rb,0.27971e+00_rb,0.31814e+00_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.77666e-01_rb,0.75357e-01_rb,0.89439e-01_rb,0.12673e+00_rb,0.16892e+00_rb, &
        & 0.21110e+00_rb,0.25327e+00_rb,0.29539e+00_rb,0.33569e+00_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.76193e-01_rb,0.74694e-01_rb,0.92082e-01_rb,0.13335e+00_rb,0.17774e+00_rb, &
        & 0.22213e+00_rb,0.26651e+00_rb,0.31084e+00_rb,0.35313e+00_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.75335e-01_rb,0.74405e-01_rb,0.95067e-01_rb,0.13953e+00_rb,0.18597e+00_rb, &
        & 0.23241e+00_rb,0.27884e+00_rb,0.32518e+00_rb,0.36942e+00_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.74635e-01_rb,0.74698e-01_rb,0.98457e-01_rb,0.14565e+00_rb,0.19414e+00_rb, &
        & 0.24262e+00_rb,0.29108e+00_rb,0.33947e+00_rb,0.38555e+00_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.89595e-01_rb,0.85781e-01_rb,0.93997e-01_rb,0.12072e+00_rb,0.16001e+00_rb, &
        & 0.19998e+00_rb,0.23994e+00_rb,0.27984e+00_rb,0.31934e+00_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.87421e-01_rb,0.84747e-01_rb,0.95233e-01_rb,0.12788e+00_rb,0.17024e+00_rb, &
        & 0.21274e+00_rb,0.25525e+00_rb,0.29772e+00_rb,0.33967e+00_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.85503e-01_rb,0.83808e-01_rb,0.97531e-01_rb,0.13512e+00_rb,0.18009e+00_rb, &
        & 0.22508e+00_rb,0.27005e+00_rb,0.31493e+00_rb,0.35929e+00_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.84061e-01_rb,0.83216e-01_rb,0.10039e+00_rb,0.14194e+00_rb,0.18919e+00_rb, &
        & 0.23644e+00_rb,0.28372e+00_rb,0.33088e+00_rb,0.37743e+00_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.83141e-01_rb,0.83249e-01_rb,0.10352e+00_rb,0.14853e+00_rb,0.19798e+00_rb, &
        & 0.24744e+00_rb,0.29688e+00_rb,0.34622e+00_rb,0.39491e+00_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.10093e+00_rb,0.96180e-01_rb,0.10202e+00_rb,0.12191e+00_rb,0.15810e+00_rb, &
        & 0.19761e+00_rb,0.23710e+00_rb,0.27651e+00_rb,0.31584e+00_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.98348e-01_rb,0.95154e-01_rb,0.10279e+00_rb,0.12897e+00_rb,0.16954e+00_rb, &
        & 0.21190e+00_rb,0.25425e+00_rb,0.29655e+00_rb,0.33869e+00_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.96078e-01_rb,0.94065e-01_rb,0.10459e+00_rb,0.13623e+00_rb,0.18035e+00_rb, &
        & 0.22539e+00_rb,0.27041e+00_rb,0.31537e+00_rb,0.36019e+00_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.94202e-01_rb,0.93296e-01_rb,0.10703e+00_rb,0.14326e+00_rb,0.19057e+00_rb, &
        & 0.23817e+00_rb,0.28575e+00_rb,0.33330e+00_rb,0.38060e+00_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.93055e-01_rb,0.92935e-01_rb,0.10982e+00_rb,0.15042e+00_rb,0.20048e+00_rb, &
        & 0.25056e+00_rb,0.30061e+00_rb,0.35061e+00_rb,0.40035e+00_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.11421e+00_rb,0.10796e+00_rb,0.11173e+00_rb,0.12476e+00_rb,0.15549e+00_rb, &
        & 0.19408e+00_rb,0.23286e+00_rb,0.27161e+00_rb,0.31032e+00_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.11123e+00_rb,0.10718e+00_rb,0.11218e+00_rb,0.13121e+00_rb,0.16775e+00_rb, &
        & 0.20966e+00_rb,0.25155e+00_rb,0.29340e+00_rb,0.33523e+00_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.10875e+00_rb,0.10588e+00_rb,0.11334e+00_rb,0.13798e+00_rb,0.17972e+00_rb, &
        & 0.22462e+00_rb,0.26950e+00_rb,0.31433e+00_rb,0.35914e+00_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.10666e+00_rb,0.10458e+00_rb,0.11529e+00_rb,0.14519e+00_rb,0.19132e+00_rb, &
        & 0.23911e+00_rb,0.28688e+00_rb,0.33463e+00_rb,0.38232e+00_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.10502e+00_rb,0.10381e+00_rb,0.11770e+00_rb,0.15287e+00_rb,0.20267e+00_rb, &
        & 0.25329e+00_rb,0.30392e+00_rb,0.35446e+00_rb,0.40496e+00_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.12988e+00_rb,0.12138e+00_rb,0.12291e+00_rb,0.12930e+00_rb,0.15315e+00_rb, &
        & 0.18907e+00_rb,0.22686e+00_rb,0.26460e+00_rb,0.30239e+00_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.12649e+00_rb,0.12054e+00_rb,0.12314e+00_rb,0.13530e+00_rb,0.16601e+00_rb, &
        & 0.20667e+00_rb,0.24798e+00_rb,0.28925e+00_rb,0.33056e+00_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.12370e+00_rb,0.11924e+00_rb,0.12400e+00_rb,0.14174e+00_rb,0.17907e+00_rb, &
        & 0.22369e+00_rb,0.26838e+00_rb,0.31304e+00_rb,0.35775e+00_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.12121e+00_rb,0.11770e+00_rb,0.12551e+00_rb,0.14889e+00_rb,0.19207e+00_rb, &
        & 0.24005e+00_rb,0.28802e+00_rb,0.33592e+00_rb,0.38391e+00_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.11892e+00_rb,0.11635e+00_rb,0.12739e+00_rb,0.15666e+00_rb,0.20470e+00_rb, &
        & 0.25583e+00_rb,0.30694e+00_rb,0.35802e+00_rb,0.40917e+00_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.14795e+00_rb,0.13669e+00_rb,0.13564e+00_rb,0.13646e+00_rb,0.15231e+00_rb, &
        & 0.18310e+00_rb,0.21966e+00_rb,0.25622e+00_rb,0.29284e+00_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.14430e+00_rb,0.13540e+00_rb,0.13550e+00_rb,0.14118e+00_rb,0.16490e+00_rb, &
        & 0.20238e+00_rb,0.24284e+00_rb,0.28324e+00_rb,0.32374e+00_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.14109e+00_rb,0.13403e+00_rb,0.13585e+00_rb,0.14713e+00_rb,0.17836e+00_rb, &
        & 0.22126e+00_rb,0.26548e+00_rb,0.30963e+00_rb,0.35392e+00_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.13807e+00_rb,0.13250e+00_rb,0.13707e+00_rb,0.15422e+00_rb,0.19232e+00_rb, &
        & 0.23967e+00_rb,0.28757e+00_rb,0.33540e+00_rb,0.38338e+00_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.13514e+00_rb,0.13080e+00_rb,0.13878e+00_rb,0.16197e+00_rb,0.20594e+00_rb, &
        & 0.25719e+00_rb,0.30860e+00_rb,0.35994e+00_rb,0.41142e+00_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.16812e+00_rb,0.15406e+00_rb,0.15051e+00_rb,0.14632e+00_rb,0.15371e+00_rb, &
        & 0.17730e+00_rb,0.21108e+00_rb,0.24622e+00_rb,0.28143e+00_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.16420e+00_rb,0.15230e+00_rb,0.14950e+00_rb,0.14998e+00_rb,0.16549e+00_rb, &
        & 0.19706e+00_rb,0.23609e+00_rb,0.27539e+00_rb,0.31477e+00_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.16053e+00_rb,0.15077e+00_rb,0.14903e+00_rb,0.15488e+00_rb,0.17883e+00_rb, &
        & 0.21740e+00_rb,0.26082e+00_rb,0.30423e+00_rb,0.34775e+00_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.15694e+00_rb,0.14891e+00_rb,0.14991e+00_rb,0.16140e+00_rb,0.19301e+00_rb, &
        & 0.23753e+00_rb,0.28500e+00_rb,0.33244e+00_rb,0.37999e+00_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.15342e+00_rb,0.14670e+00_rb,0.15145e+00_rb,0.16838e+00_rb,0.20634e+00_rb, &
        & 0.25567e+00_rb,0.30677e+00_rb,0.35784e+00_rb,0.40903e+00_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.18962e+00_rb,0.17319e+00_rb,0.16709e+00_rb,0.15817e+00_rb,0.15908e+00_rb, &
        & 0.17425e+00_rb,0.20287e+00_rb,0.23665e+00_rb,0.27049e+00_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.18536e+00_rb,0.17110e+00_rb,0.16526e+00_rb,0.16124e+00_rb,0.16946e+00_rb, &
        & 0.19394e+00_rb,0.22959e+00_rb,0.26781e+00_rb,0.30611e+00_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.18117e+00_rb,0.16910e+00_rb,0.16400e+00_rb,0.16579e+00_rb,0.18236e+00_rb, &
        & 0.21466e+00_rb,0.25610e+00_rb,0.29874e+00_rb,0.34146e+00_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.17697e+00_rb,0.16677e+00_rb,0.16446e+00_rb,0.17161e+00_rb,0.19589e+00_rb, &
        & 0.23479e+00_rb,0.28114e+00_rb,0.32796e+00_rb,0.37484e+00_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.17281e+00_rb,0.16399e+00_rb,0.16565e+00_rb,0.17716e+00_rb,0.20808e+00_rb, &
        & 0.25277e+00_rb,0.30323e+00_rb,0.35372e+00_rb,0.40429e+00_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.21055e+00_rb,0.19254e+00_rb,0.18359e+00_rb,0.17274e+00_rb,0.17137e+00_rb, &
        & 0.18167e+00_rb,0.20669e+00_rb,0.24044e+00_rb,0.24447e+00_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.20570e+00_rb,0.19011e+00_rb,0.18168e+00_rb,0.17577e+00_rb,0.18167e+00_rb, &
        & 0.20144e+00_rb,0.23412e+00_rb,0.27305e+00_rb,0.28987e+00_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.20088e+00_rb,0.18755e+00_rb,0.18057e+00_rb,0.18058e+00_rb,0.19347e+00_rb, &
        & 0.22124e+00_rb,0.26062e+00_rb,0.30406e+00_rb,0.33203e+00_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.19608e+00_rb,0.18439e+00_rb,0.18082e+00_rb,0.18569e+00_rb,0.20488e+00_rb, &
        & 0.23927e+00_rb,0.28460e+00_rb,0.33201e+00_rb,0.37191e+00_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.19134e+00_rb,0.18083e+00_rb,0.18137e+00_rb,0.19023e+00_rb,0.21563e+00_rb, &
        & 0.25670e+00_rb,0.30697e+00_rb,0.35809e+00_rb,0.40924e+00_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.23222e+00_rb,0.21282e+00_rb,0.20059e+00_rb,0.18856e+00_rb,0.18455e+00_rb, &
        & 0.18954e+00_rb,0.20921e+00_rb,0.24116e+00_rb,0.20678e+00_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.22666e+00_rb,0.20982e+00_rb,0.19853e+00_rb,0.19166e+00_rb,0.19460e+00_rb, &
        & 0.20805e+00_rb,0.23602e+00_rb,0.27405e+00_rb,0.24947e+00_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.22121e+00_rb,0.20649e+00_rb,0.19756e+00_rb,0.19621e+00_rb,0.20452e+00_rb, &
        & 0.22597e+00_rb,0.26146e+00_rb,0.30478e+00_rb,0.28725e+00_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.21578e+00_rb,0.20240e+00_rb,0.19746e+00_rb,0.20035e+00_rb,0.21392e+00_rb, &
        & 0.24302e+00_rb,0.28559e+00_rb,0.33317e+00_rb,0.32415e+00_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.21026e+00_rb,0.19816e+00_rb,0.19729e+00_rb,0.20402e+00_rb,0.22339e+00_rb, &
        & 0.25983e+00_rb,0.30840e+00_rb,0.35980e+00_rb,0.36272e+00_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.25447e+00_rb,0.23356e+00_rb,0.21785e+00_rb,0.20504e+00_rb,0.19797e+00_rb, &
        & 0.19705e+00_rb,0.20967e+00_rb,0.23794e+00_rb,0.19657e+00_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.24820e+00_rb,0.22981e+00_rb,0.21561e+00_rb,0.20828e+00_rb,0.20711e+00_rb, &
        & 0.21347e+00_rb,0.23520e+00_rb,0.27088e+00_rb,0.23547e+00_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.24206e+00_rb,0.22556e+00_rb,0.21467e+00_rb,0.21216e+00_rb,0.21525e+00_rb, &
        & 0.22938e+00_rb,0.25994e+00_rb,0.30180e+00_rb,0.27108e+00_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.23572e+00_rb,0.22056e+00_rb,0.21408e+00_rb,0.21517e+00_rb,0.22293e+00_rb, &
        & 0.24576e+00_rb,0.28399e+00_rb,0.33091e+00_rb,0.30909e+00_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.22932e+00_rb,0.21557e+00_rb,0.21287e+00_rb,0.21778e+00_rb,0.23130e+00_rb, &
        & 0.26208e+00_rb,0.30701e+00_rb,0.35815e+00_rb,0.34602e+00_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.79223e-01_rb,0.75104e-01_rb,0.13846e+00_rb,0.20766e+00_rb,0.27685e+00_rb, &
        & 0.34601e+00_rb,0.41512e+00_rb,0.48397e+00_rb,0.52564e+00_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.76682e-01_rb,0.76450e-01_rb,0.14571e+00_rb,0.21854e+00_rb,0.29137e+00_rb, &
        & 0.36417e+00_rb,0.43689e+00_rb,0.50929e+00_rb,0.55383e+00_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.74345e-01_rb,0.78213e-01_rb,0.15207e+00_rb,0.22808e+00_rb,0.30407e+00_rb, &
        & 0.38004e+00_rb,0.45593e+00_rb,0.53155e+00_rb,0.57851e+00_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.72607e-01_rb,0.80130e-01_rb,0.15758e+00_rb,0.23638e+00_rb,0.31513e+00_rb, &
        & 0.39385e+00_rb,0.47252e+00_rb,0.55092e+00_rb,0.60005e+00_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.72847e-01_rb,0.82433e-01_rb,0.16249e+00_rb,0.24371e+00_rb,0.32491e+00_rb, &
        & 0.40606e+00_rb,0.48713e+00_rb,0.56790e+00_rb,0.61880e+00_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.89010e-01_rb,0.84748e-01_rb,0.15528e+00_rb,0.23291e+00_rb,0.31052e+00_rb, &
        & 0.38812e+00_rb,0.46559e+00_rb,0.54282e+00_rb,0.60189e+00_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.86286e-01_rb,0.86390e-01_rb,0.16350e+00_rb,0.24523e+00_rb,0.32694e+00_rb, &
        & 0.40865e+00_rb,0.49029e+00_rb,0.57166e+00_rb,0.63436e+00_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.83865e-01_rb,0.88469e-01_rb,0.17076e+00_rb,0.25611e+00_rb,0.34145e+00_rb, &
        & 0.42675e+00_rb,0.51202e+00_rb,0.59693e+00_rb,0.66276e+00_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.81971e-01_rb,0.90581e-01_rb,0.17733e+00_rb,0.26595e+00_rb,0.35459e+00_rb, &
        & 0.44319e+00_rb,0.53165e+00_rb,0.61981e+00_rb,0.68802e+00_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.81826e-01_rb,0.93085e-01_rb,0.18319e+00_rb,0.27476e+00_rb,0.36629e+00_rb, &
        & 0.45781e+00_rb,0.54921e+00_rb,0.64033e+00_rb,0.71080e+00_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.10078e+00_rb,0.96469e-01_rb,0.17126e+00_rb,0.25687e+00_rb,0.34247e+00_rb, &
        & 0.42804e+00_rb,0.51357e+00_rb,0.59868e+00_rb,0.67473e+00_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.97698e-01_rb,0.98193e-01_rb,0.18058e+00_rb,0.27084e+00_rb,0.36108e+00_rb, &
        & 0.45128e+00_rb,0.54134e+00_rb,0.63128e+00_rb,0.71182e+00_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.95186e-01_rb,0.10027e+00_rb,0.18871e+00_rb,0.28303e+00_rb,0.37733e+00_rb, &
        & 0.47160e+00_rb,0.56577e+00_rb,0.65972e+00_rb,0.74408e+00_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.93223e-01_rb,0.10231e+00_rb,0.19621e+00_rb,0.29427e+00_rb,0.39234e+00_rb, &
        & 0.49035e+00_rb,0.58829e+00_rb,0.68586e+00_rb,0.77383e+00_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.93003e-01_rb,0.10431e+00_rb,0.20302e+00_rb,0.30448e+00_rb,0.40592e+00_rb, &
        & 0.50734e+00_rb,0.60870e+00_rb,0.70970e+00_rb,0.80084e+00_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.11556e+00_rb,0.11007e+00_rb,0.18689e+00_rb,0.28032e+00_rb,0.37373e+00_rb, &
        & 0.46710e+00_rb,0.56044e+00_rb,0.65346e+00_rb,0.74264e+00_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.11235e+00_rb,0.11167e+00_rb,0.19754e+00_rb,0.29630e+00_rb,0.39502e+00_rb, &
        & 0.49371e+00_rb,0.59235e+00_rb,0.69064e+00_rb,0.78516e+00_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.10972e+00_rb,0.11355e+00_rb,0.20696e+00_rb,0.31040e+00_rb,0.41383e+00_rb, &
        & 0.51721e+00_rb,0.62053e+00_rb,0.72352e+00_rb,0.82248e+00_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.10741e+00_rb,0.11554e+00_rb,0.21557e+00_rb,0.32331e+00_rb,0.43105e+00_rb, &
        & 0.53874e+00_rb,0.64632e+00_rb,0.75359e+00_rb,0.85679e+00_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.10619e+00_rb,0.11745e+00_rb,0.22327e+00_rb,0.33487e+00_rb,0.44646e+00_rb, &
        & 0.55799e+00_rb,0.66944e+00_rb,0.78056e+00_rb,0.88741e+00_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.13422e+00_rb,0.12619e+00_rb,0.20125e+00_rb,0.30186e+00_rb,0.40245e+00_rb, &
        & 0.50302e+00_rb,0.60348e+00_rb,0.70372e+00_rb,0.80253e+00_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.13052e+00_rb,0.12743e+00_rb,0.21345e+00_rb,0.32016e+00_rb,0.42685e+00_rb, &
        & 0.53351e+00_rb,0.64011e+00_rb,0.74634e+00_rb,0.85136e+00_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.12741e+00_rb,0.12876e+00_rb,0.22450e+00_rb,0.33672e+00_rb,0.44894e+00_rb, &
        & 0.56113e+00_rb,0.67322e+00_rb,0.78494e+00_rb,0.89542e+00_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.12456e+00_rb,0.13045e+00_rb,0.23454e+00_rb,0.35177e+00_rb,0.46901e+00_rb, &
        & 0.58622e+00_rb,0.70332e+00_rb,0.82004e+00_rb,0.93535e+00_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.12240e+00_rb,0.13234e+00_rb,0.24346e+00_rb,0.36515e+00_rb,0.48682e+00_rb, &
        & 0.60842e+00_rb,0.72992e+00_rb,0.85105e+00_rb,0.97074e+00_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.15641e+00_rb,0.14549e+00_rb,0.21406e+00_rb,0.31903e+00_rb,0.42534e+00_rb, &
        & 0.53165e+00_rb,0.63788e+00_rb,0.74385e+00_rb,0.84964e+00_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.15207e+00_rb,0.14574e+00_rb,0.22708e+00_rb,0.34022e+00_rb,0.45360e+00_rb, &
        & 0.56697e+00_rb,0.68027e+00_rb,0.79322e+00_rb,0.90603e+00_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.14825e+00_rb,0.14665e+00_rb,0.23979e+00_rb,0.35965e+00_rb,0.47950e+00_rb, &
        & 0.59932e+00_rb,0.71907e+00_rb,0.83857e+00_rb,0.95773e+00_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.14482e+00_rb,0.14790e+00_rb,0.25144e+00_rb,0.37711e+00_rb,0.50278e+00_rb, &
        & 0.62841e+00_rb,0.75400e+00_rb,0.87926e+00_rb,0.10043e+01_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.14196e+00_rb,0.14929e+00_rb,0.26158e+00_rb,0.39235e+00_rb,0.52312e+00_rb, &
        & 0.65390e+00_rb,0.78460e+00_rb,0.91489e+00_rb,0.10450e+01_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.18245e+00_rb,0.16813e+00_rb,0.22831e+00_rb,0.33360e+00_rb,0.44476e+00_rb, &
        & 0.55591e+00_rb,0.66702e+00_rb,0.77783e+00_rb,0.88903e+00_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.17706e+00_rb,0.16763e+00_rb,0.24144e+00_rb,0.35719e+00_rb,0.47625e+00_rb, &
        & 0.59524e+00_rb,0.71422e+00_rb,0.83295e+00_rb,0.95194e+00_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.17236e+00_rb,0.16756e+00_rb,0.25360e+00_rb,0.37892e+00_rb,0.50520e+00_rb, &
        & 0.63143e+00_rb,0.75766e+00_rb,0.88351e+00_rb,0.10099e+01_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.16823e+00_rb,0.16816e+00_rb,0.26586e+00_rb,0.39858e+00_rb,0.53141e+00_rb, &
        & 0.66420e+00_rb,0.79692e+00_rb,0.92941e+00_rb,0.10623e+01_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.16468e+00_rb,0.16894e+00_rb,0.27725e+00_rb,0.41585e+00_rb,0.55445e+00_rb, &
        & 0.69298e+00_rb,0.83146e+00_rb,0.96971e+00_rb,0.11082e+01_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.21221e+00_rb,0.19366e+00_rb,0.24501e+00_rb,0.34518e+00_rb,0.46022e+00_rb, &
        & 0.57522e+00_rb,0.69013e+00_rb,0.80500e+00_rb,0.92024e+00_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.20558e+00_rb,0.19344e+00_rb,0.25798e+00_rb,0.37195e+00_rb,0.49597e+00_rb, &
        & 0.61989e+00_rb,0.74379e+00_rb,0.86737e+00_rb,0.99166e+00_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.19977e+00_rb,0.19260e+00_rb,0.26963e+00_rb,0.39624e+00_rb,0.52829e+00_rb, &
        & 0.66032e+00_rb,0.79227e+00_rb,0.92396e+00_rb,0.10564e+01_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.19471e+00_rb,0.19196e+00_rb,0.28135e+00_rb,0.41773e+00_rb,0.55693e+00_rb, &
        & 0.69614e+00_rb,0.83525e+00_rb,0.97412e+00_rb,0.11137e+01_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.19023e+00_rb,0.19149e+00_rb,0.29229e+00_rb,0.43683e+00_rb,0.58243e+00_rb, &
        & 0.72800e+00_rb,0.87352e+00_rb,0.10186e+01_rb,0.11647e+01_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.24675e+00_rb,0.22274e+00_rb,0.26456e+00_rb,0.35496e+00_rb,0.47007e+00_rb, &
        & 0.58755e+00_rb,0.70496e+00_rb,0.82216e+00_rb,0.94015e+00_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.23844e+00_rb,0.22223e+00_rb,0.27692e+00_rb,0.38304e+00_rb,0.51028e+00_rb, &
        & 0.63780e+00_rb,0.76534e+00_rb,0.89259e+00_rb,0.10206e+01_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.23115e+00_rb,0.22076e+00_rb,0.28808e+00_rb,0.40973e+00_rb,0.54629e+00_rb, &
        & 0.68281e+00_rb,0.81930e+00_rb,0.95554e+00_rb,0.10926e+01_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.22487e+00_rb,0.21935e+00_rb,0.29872e+00_rb,0.43339e+00_rb,0.57783e+00_rb, &
        & 0.72230e+00_rb,0.86661e+00_rb,0.10108e+01_rb,0.11557e+01_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.21911e+00_rb,0.21807e+00_rb,0.30914e+00_rb,0.45602e+00_rb,0.60799e+00_rb, &
        & 0.75991e+00_rb,0.91189e+00_rb,0.10634e+01_rb,0.12161e+01_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.28580e+00_rb,0.25610e+00_rb,0.28839e+00_rb,0.36810e+00_rb,0.47645e+00_rb, &
        & 0.59554e+00_rb,0.71455e+00_rb,0.83348e+00_rb,0.95290e+00_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.27554e+00_rb,0.25438e+00_rb,0.29985e+00_rb,0.39603e+00_rb,0.52141e+00_rb, &
        & 0.65175e+00_rb,0.78205e+00_rb,0.91212e+00_rb,0.10429e+01_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.26639e+00_rb,0.25230e+00_rb,0.31000e+00_rb,0.42263e+00_rb,0.56074e+00_rb, &
        & 0.70092e+00_rb,0.84101e+00_rb,0.98094e+00_rb,0.11215e+01_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.25840e+00_rb,0.24961e+00_rb,0.31933e+00_rb,0.44762e+00_rb,0.59663e+00_rb, &
        & 0.74572e+00_rb,0.89477e+00_rb,0.10436e+01_rb,0.11932e+01_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.25110e+00_rb,0.24738e+00_rb,0.32882e+00_rb,0.47409e+00_rb,0.63208e+00_rb, &
        & 0.79011e+00_rb,0.94801e+00_rb,0.11055e+01_rb,0.12642e+01_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.32402e+00_rb,0.29231e+00_rb,0.32114e+00_rb,0.39449e+00_rb,0.49965e+00_rb, &
        & 0.62359e+00_rb,0.74830e+00_rb,0.87279e+00_rb,0.99780e+00_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.31203e+00_rb,0.28940e+00_rb,0.33009e+00_rb,0.42179e+00_rb,0.54573e+00_rb, &
        & 0.68214e+00_rb,0.81856e+00_rb,0.95481e+00_rb,0.10916e+01_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.30125e+00_rb,0.28556e+00_rb,0.33820e+00_rb,0.44749e+00_rb,0.58870e+00_rb, &
        & 0.73583e+00_rb,0.88295e+00_rb,0.10298e+01_rb,0.11773e+01_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.29153e+00_rb,0.28159e+00_rb,0.34632e+00_rb,0.47427e+00_rb,0.63035e+00_rb, &
        & 0.78788e+00_rb,0.94537e+00_rb,0.11027e+01_rb,0.12607e+01_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.28247e+00_rb,0.27799e+00_rb,0.35496e+00_rb,0.50259e+00_rb,0.66996e+00_rb, &
        & 0.83740e+00_rb,0.10047e+01_rb,0.11720e+01_rb,0.13400e+01_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.36545e+00_rb,0.33140e+00_rb,0.35636e+00_rb,0.42085e+00_rb,0.52250e+00_rb, &
        & 0.64898e+00_rb,0.77872e+00_rb,0.90834e+00_rb,0.10384e+01_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.35131e+00_rb,0.32685e+00_rb,0.36231e+00_rb,0.44658e+00_rb,0.56903e+00_rb, &
        & 0.71087e+00_rb,0.85304e+00_rb,0.99503e+00_rb,0.11374e+01_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.33841e+00_rb,0.32098e+00_rb,0.36825e+00_rb,0.47208e+00_rb,0.61613e+00_rb, &
        & 0.77016e+00_rb,0.92410e+00_rb,0.10779e+01_rb,0.12323e+01_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.32642e+00_rb,0.31543e+00_rb,0.37504e+00_rb,0.50058e+00_rb,0.66195e+00_rb, &
        & 0.82736e+00_rb,0.99279e+00_rb,0.11580e+01_rb,0.13239e+01_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.31541e+00_rb,0.31006e+00_rb,0.38358e+00_rb,0.53015e+00_rb,0.70512e+00_rb, &
        & 0.88131e+00_rb,0.10574e+01_rb,0.12335e+01_rb,0.14101e+01_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.40964e+00_rb,0.37314e+00_rb,0.39372e+00_rb,0.44813e+00_rb,0.54521e+00_rb, &
        & 0.67332e+00_rb,0.80793e+00_rb,0.94247e+00_rb,0.10773e+01_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.39277e+00_rb,0.36624e+00_rb,0.39629e+00_rb,0.47194e+00_rb,0.59295e+00_rb, &
        & 0.73949e+00_rb,0.88734e+00_rb,0.10350e+01_rb,0.11832e+01_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.37714e+00_rb,0.35816e+00_rb,0.39962e+00_rb,0.49769e+00_rb,0.64296e+00_rb, &
        & 0.80367e+00_rb,0.96435e+00_rb,0.11249e+01_rb,0.12860e+01_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.36268e+00_rb,0.35049e+00_rb,0.40459e+00_rb,0.52725e+00_rb,0.69174e+00_rb, &
        & 0.86463e+00_rb,0.10374e+01_rb,0.12101e+01_rb,0.13834e+01_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.34948e+00_rb,0.34295e+00_rb,0.41358e+00_rb,0.55736e+00_rb,0.73767e+00_rb, &
        & 0.92205e+00_rb,0.11064e+01_rb,0.12905e+01_rb,0.14753e+01_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.90320e-01_rb,0.86361e-01_rb,0.17271e+00_rb,0.25905e+00_rb,0.34536e+00_rb, &
        & 0.43163e+00_rb,0.51777e+00_rb,0.60344e+00_rb,0.64541e+00_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.88026e-01_rb,0.92415e-01_rb,0.18482e+00_rb,0.27721e+00_rb,0.36957e+00_rb, &
        & 0.46188e+00_rb,0.55406e+00_rb,0.64572e+00_rb,0.69076e+00_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.85829e-01_rb,0.97954e-01_rb,0.19590e+00_rb,0.29382e+00_rb,0.39172e+00_rb, &
        & 0.48957e+00_rb,0.58728e+00_rb,0.68449e+00_rb,0.73223e+00_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.83674e-01_rb,0.10298e+00_rb,0.20595e+00_rb,0.30890e+00_rb,0.41183e+00_rb, &
        & 0.51469e+00_rb,0.61741e+00_rb,0.71956e+00_rb,0.77014e+00_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.84304e-01_rb,0.10756e+00_rb,0.21511e+00_rb,0.32264e+00_rb,0.43014e+00_rb, &
        & 0.53758e+00_rb,0.64487e+00_rb,0.75164e+00_rb,0.80439e+00_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.99611e-01_rb,0.10121e+00_rb,0.20241e+00_rb,0.30359e+00_rb,0.40479e+00_rb, &
        & 0.50590e+00_rb,0.60686e+00_rb,0.70732e+00_rb,0.77453e+00_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.97224e-01_rb,0.10870e+00_rb,0.21739e+00_rb,0.32607e+00_rb,0.43471e+00_rb, &
        & 0.54336e+00_rb,0.65181e+00_rb,0.75967e+00_rb,0.83199e+00_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.94894e-01_rb,0.11558e+00_rb,0.23114e+00_rb,0.34669e+00_rb,0.46220e+00_rb, &
        & 0.57772e+00_rb,0.69302e+00_rb,0.80771e+00_rb,0.88476e+00_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.92656e-01_rb,0.12184e+00_rb,0.24366e+00_rb,0.36546e+00_rb,0.48723e+00_rb, &
        & 0.60893e+00_rb,0.73046e+00_rb,0.85132e+00_rb,0.93269e+00_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.93593e-01_rb,0.12749e+00_rb,0.25497e+00_rb,0.38243e+00_rb,0.50985e+00_rb, &
        & 0.63721e+00_rb,0.76438e+00_rb,0.89083e+00_rb,0.97622e+00_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.11521e+00_rb,0.11614e+00_rb,0.23226e+00_rb,0.34836e+00_rb,0.46443e+00_rb, &
        & 0.58044e+00_rb,0.69629e+00_rb,0.81149e+00_rb,0.90951e+00_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.11222e+00_rb,0.12555e+00_rb,0.25108e+00_rb,0.37659e+00_rb,0.50206e+00_rb, &
        & 0.62747e+00_rb,0.75258e+00_rb,0.87709e+00_rb,0.98316e+00_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.10948e+00_rb,0.13410e+00_rb,0.26819e+00_rb,0.40226e+00_rb,0.53629e+00_rb, &
        & 0.67035e+00_rb,0.80421e+00_rb,0.93726e+00_rb,0.10505e+01_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.10697e+00_rb,0.14195e+00_rb,0.28389e+00_rb,0.42580e+00_rb,0.56768e+00_rb, &
        & 0.70947e+00_rb,0.85107e+00_rb,0.99187e+00_rb,0.11120e+01_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.10462e+00_rb,0.14901e+00_rb,0.29800e+00_rb,0.44697e+00_rb,0.59590e+00_rb, &
        & 0.74474e+00_rb,0.89338e+00_rb,0.10415e+01_rb,0.11678e+01_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.13825e+00_rb,0.13238e+00_rb,0.26314e+00_rb,0.39468e+00_rb,0.52618e+00_rb, &
        & 0.65761e+00_rb,0.78893e+00_rb,0.91946e+00_rb,0.10423e+01_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.13437e+00_rb,0.14320e+00_rb,0.28638e+00_rb,0.42954e+00_rb,0.57266e+00_rb, &
        & 0.71574e+00_rb,0.85868e+00_rb,0.10008e+01_rb,0.11347e+01_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.13067e+00_rb,0.15392e+00_rb,0.30782e+00_rb,0.46170e+00_rb,0.61554e+00_rb, &
        & 0.76929e+00_rb,0.92283e+00_rb,0.10757e+01_rb,0.12195e+01_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.12733e+00_rb,0.16364e+00_rb,0.32725e+00_rb,0.49085e+00_rb,0.65439e+00_rb, &
        & 0.81785e+00_rb,0.98125e+00_rb,0.11437e+01_rb,0.12968e+01_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.12429e+00_rb,0.17242e+00_rb,0.34483e+00_rb,0.51721e+00_rb,0.68954e+00_rb, &
        & 0.86178e+00_rb,0.10339e+01_rb,0.12053e+01_rb,0.13667e+01_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.16714e+00_rb,0.15294e+00_rb,0.29770e+00_rb,0.44651e+00_rb,0.59528e+00_rb, &
        & 0.74397e+00_rb,0.89246e+00_rb,0.10402e+01_rb,0.11846e+01_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.16248e+00_rb,0.16289e+00_rb,0.32569e+00_rb,0.48850e+00_rb,0.65127e+00_rb, &
        & 0.81396e+00_rb,0.97625e+00_rb,0.11381e+01_rb,0.12962e+01_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.15803e+00_rb,0.17580e+00_rb,0.35158e+00_rb,0.52733e+00_rb,0.70303e+00_rb, &
        & 0.87864e+00_rb,0.10540e+01_rb,0.12284e+01_rb,0.13993e+01_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.15368e+00_rb,0.18757e+00_rb,0.37512e+00_rb,0.56264e+00_rb,0.75011e+00_rb, &
        & 0.93747e+00_rb,0.11247e+01_rb,0.13108e+01_rb,0.14933e+01_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.14943e+00_rb,0.19824e+00_rb,0.39645e+00_rb,0.59477e+00_rb,0.79303e+00_rb, &
        & 0.99112e+00_rb,0.11890e+01_rb,0.13858e+01_rb,0.15783e+01_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.20217e+00_rb,0.17926e+00_rb,0.33507e+00_rb,0.50257e+00_rb,0.67002e+00_rb, &
        & 0.83741e+00_rb,0.10046e+01_rb,0.11708e+01_rb,0.13365e+01_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.19655e+00_rb,0.18668e+00_rb,0.36910e+00_rb,0.55360e+00_rb,0.73806e+00_rb, &
        & 0.92245e+00_rb,0.11065e+01_rb,0.12896e+01_rb,0.14724e+01_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.19100e+00_rb,0.20029e+00_rb,0.40054e+00_rb,0.60077e+00_rb,0.80095e+00_rb, &
        & 0.10010e+01_rb,0.12008e+01_rb,0.13997e+01_rb,0.15979e+01_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.18556e+00_rb,0.21465e+00_rb,0.42926e+00_rb,0.64385e+00_rb,0.85838e+00_rb, &
        & 0.10728e+01_rb,0.12869e+01_rb,0.15000e+01_rb,0.17126e+01_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.18024e+00_rb,0.22760e+00_rb,0.45517e+00_rb,0.68271e+00_rb,0.91019e+00_rb, &
        & 0.11375e+01_rb,0.13647e+01_rb,0.15907e+01_rb,0.18161e+01_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.24417e+00_rb,0.21368e+00_rb,0.37507e+00_rb,0.56256e+00_rb,0.75006e+00_rb, &
        & 0.93742e+00_rb,0.11245e+01_rb,0.13107e+01_rb,0.14982e+01_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.23715e+00_rb,0.21661e+00_rb,0.41650e+00_rb,0.62470e+00_rb,0.83285e+00_rb, &
        & 0.10409e+01_rb,0.12487e+01_rb,0.14555e+01_rb,0.16638e+01_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.23022e+00_rb,0.22856e+00_rb,0.45485e+00_rb,0.68222e+00_rb,0.90954e+00_rb, &
        & 0.11367e+01_rb,0.13637e+01_rb,0.15896e+01_rb,0.18170e+01_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.22335e+00_rb,0.24491e+00_rb,0.48979e+00_rb,0.73464e+00_rb,0.97947e+00_rb, &
        & 0.12241e+01_rb,0.14686e+01_rb,0.17118e+01_rb,0.19566e+01_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.21655e+00_rb,0.26068e+00_rb,0.52133e+00_rb,0.78194e+00_rb,0.10425e+01_rb, &
        & 0.13029e+01_rb,0.15630e+01_rb,0.18220e+01_rb,0.20827e+01_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.29541e+00_rb,0.25848e+00_rb,0.41687e+00_rb,0.62527e+00_rb,0.83361e+00_rb, &
        & 0.10419e+01_rb,0.12499e+01_rb,0.14571e+01_rb,0.16668e+01_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.28549e+00_rb,0.25449e+00_rb,0.46724e+00_rb,0.70082e+00_rb,0.93434e+00_rb, &
        & 0.11678e+01_rb,0.14009e+01_rb,0.16328e+01_rb,0.18680e+01_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.27625e+00_rb,0.26285e+00_rb,0.51364e+00_rb,0.77040e+00_rb,0.10271e+01_rb, &
        & 0.12838e+01_rb,0.15402e+01_rb,0.17956e+01_rb,0.20538e+01_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.26742e+00_rb,0.27883e+00_rb,0.55601e+00_rb,0.83395e+00_rb,0.11118e+01_rb, &
        & 0.13896e+01_rb,0.16670e+01_rb,0.19436e+01_rb,0.22230e+01_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.25873e+00_rb,0.29706e+00_rb,0.59410e+00_rb,0.89109e+00_rb,0.11880e+01_rb, &
        & 0.14848e+01_rb,0.17812e+01_rb,0.20764e+01_rb,0.23753e+01_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.35727e+00_rb,0.31262e+00_rb,0.45859e+00_rb,0.68783e+00_rb,0.91710e+00_rb, &
        & 0.11462e+01_rb,0.13749e+01_rb,0.16027e+01_rb,0.18342e+01_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.34378e+00_rb,0.30157e+00_rb,0.51912e+00_rb,0.77863e+00_rb,0.10382e+01_rb, &
        & 0.12975e+01_rb,0.15566e+01_rb,0.18147e+01_rb,0.20764e+01_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.33111e+00_rb,0.30494e+00_rb,0.57506e+00_rb,0.86254e+00_rb,0.11500e+01_rb, &
        & 0.14374e+01_rb,0.17244e+01_rb,0.20104e+01_rb,0.23002e+01_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.31901e+00_rb,0.31860e+00_rb,0.62597e+00_rb,0.93889e+00_rb,0.12519e+01_rb, &
        & 0.15648e+01_rb,0.18772e+01_rb,0.21885e+01_rb,0.25042e+01_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.30738e+00_rb,0.33686e+00_rb,0.67173e+00_rb,0.10075e+01_rb,0.13433e+01_rb, &
        & 0.16788e+01_rb,0.20142e+01_rb,0.23482e+01_rb,0.26866e+01_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.42900e+00_rb,0.37537e+00_rb,0.50742e+00_rb,0.76107e+00_rb,0.10147e+01_rb, &
        & 0.12682e+01_rb,0.15216e+01_rb,0.17737e+01_rb,0.20298e+01_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.41120e+00_rb,0.35980e+00_rb,0.57626e+00_rb,0.86440e+00_rb,0.11524e+01_rb, &
        & 0.14403e+01_rb,0.17281e+01_rb,0.20148e+01_rb,0.23052e+01_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.39435e+00_rb,0.35582e+00_rb,0.64201e+00_rb,0.96295e+00_rb,0.12838e+01_rb, &
        & 0.16046e+01_rb,0.19250e+01_rb,0.22444e+01_rb,0.25680e+01_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.37835e+00_rb,0.36721e+00_rb,0.70220e+00_rb,0.10532e+01_rb,0.14042e+01_rb, &
        & 0.17550e+01_rb,0.21055e+01_rb,0.24551e+01_rb,0.28089e+01_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.36302e+00_rb,0.38402e+00_rb,0.75599e+00_rb,0.11339e+01_rb,0.15118e+01_rb, &
        & 0.18894e+01_rb,0.22669e+01_rb,0.26429e+01_rb,0.30243e+01_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.50130e+00_rb,0.43864e+00_rb,0.59754e+00_rb,0.89626e+00_rb,0.11949e+01_rb, &
        & 0.14934e+01_rb,0.17919e+01_rb,0.20890e+01_rb,0.23902e+01_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.47862e+00_rb,0.41971e+00_rb,0.67375e+00_rb,0.10106e+01_rb,0.13473e+01_rb, &
        & 0.16840e+01_rb,0.20203e+01_rb,0.23553e+01_rb,0.26950e+01_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.45729e+00_rb,0.41739e+00_rb,0.74475e+00_rb,0.11171e+01_rb,0.14893e+01_rb, &
        & 0.18615e+01_rb,0.22335e+01_rb,0.26038e+01_rb,0.29786e+01_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.43691e+00_rb,0.42957e+00_rb,0.80967e+00_rb,0.12144e+01_rb,0.16191e+01_rb, &
        & 0.20236e+01_rb,0.24280e+01_rb,0.28308e+01_rb,0.32389e+01_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.41748e+00_rb,0.44621e+00_rb,0.86800e+00_rb,0.13019e+01_rb,0.17360e+01_rb, &
        & 0.21698e+01_rb,0.26031e+01_rb,0.30355e+01_rb,0.34723e+01_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.58052e+00_rb,0.50796e+00_rb,0.69775e+00_rb,0.10466e+01_rb,0.13954e+01_rb, &
        & 0.17440e+01_rb,0.20924e+01_rb,0.24394e+01_rb,0.27908e+01_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.55207e+00_rb,0.48627e+00_rb,0.78177e+00_rb,0.11726e+01_rb,0.15635e+01_rb, &
        & 0.19543e+01_rb,0.23447e+01_rb,0.27336e+01_rb,0.31273e+01_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.52503e+00_rb,0.48543e+00_rb,0.85881e+00_rb,0.12881e+01_rb,0.17174e+01_rb, &
        & 0.21467e+01_rb,0.25754e+01_rb,0.30027e+01_rb,0.34355e+01_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.49945e+00_rb,0.49746e+00_rb,0.92780e+00_rb,0.13917e+01_rb,0.18554e+01_rb, &
        & 0.23191e+01_rb,0.27823e+01_rb,0.32446e+01_rb,0.37108e+01_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.47536e+00_rb,0.51344e+00_rb,0.98885e+00_rb,0.14832e+01_rb,0.19776e+01_rb, &
        & 0.24717e+01_rb,0.29655e+01_rb,0.34576e+01_rb,0.39563e+01_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.66560e+00_rb,0.58240e+00_rb,0.80598e+00_rb,0.12089e+01_rb,0.16118e+01_rb, &
        & 0.20145e+01_rb,0.24172e+01_rb,0.28183e+01_rb,0.32240e+01_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.62986e+00_rb,0.55885e+00_rb,0.89794e+00_rb,0.13468e+01_rb,0.17957e+01_rb, &
        & 0.22443e+01_rb,0.26925e+01_rb,0.31396e+01_rb,0.35914e+01_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.59613e+00_rb,0.55868e+00_rb,0.98065e+00_rb,0.14709e+01_rb,0.19611e+01_rb, &
        & 0.24513e+01_rb,0.29409e+01_rb,0.34293e+01_rb,0.39230e+01_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.56427e+00_rb,0.56950e+00_rb,0.10539e+01_rb,0.15808e+01_rb,0.21076e+01_rb, &
        & 0.26342e+01_rb,0.31605e+01_rb,0.36850e+01_rb,0.42159e+01_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.53530e+00_rb,0.58425e+00_rb,0.11174e+01_rb,0.16761e+01_rb,0.22346e+01_rb, &
        & 0.27933e+01_rb,0.33512e+01_rb,0.39078e+01_rb,0.44703e+01_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:,13, 1) = (/ &
        & 0.12810e-03_rb,0.12891e-03_rb,0.12890e-03_rb,0.12713e-03_rb,0.12477e-03_rb /)
      kbo(:,14, 1) = (/ &
        & 0.10653e-03_rb,0.10667e-03_rb,0.10648e-03_rb,0.10473e-03_rb,0.10293e-03_rb /)
      kbo(:,15, 1) = (/ &
        & 0.90176e-04_rb,0.90077e-04_rb,0.89286e-04_rb,0.87879e-04_rb,0.85987e-04_rb /)
      kbo(:,16, 1) = (/ &
        & 0.77704e-04_rb,0.77304e-04_rb,0.75879e-04_rb,0.74246e-04_rb,0.72070e-04_rb /)
      kbo(:,17, 1) = (/ &
        & 0.67183e-04_rb,0.65906e-04_rb,0.64619e-04_rb,0.63000e-04_rb,0.61056e-04_rb /)
      kbo(:,18, 1) = (/ &
        & 0.58741e-04_rb,0.57649e-04_rb,0.56028e-04_rb,0.54293e-04_rb,0.52506e-04_rb /)
      kbo(:,19, 1) = (/ &
        & 0.51839e-04_rb,0.50715e-04_rb,0.49053e-04_rb,0.47195e-04_rb,0.45368e-04_rb /)
      kbo(:,20, 1) = (/ &
        & 0.44321e-04_rb,0.43297e-04_rb,0.41923e-04_rb,0.40298e-04_rb,0.38744e-04_rb /)
      kbo(:,21, 1) = (/ &
        & 0.37527e-04_rb,0.36516e-04_rb,0.35380e-04_rb,0.34037e-04_rb,0.32951e-04_rb /)
      kbo(:,22, 1) = (/ &
        & 0.31752e-04_rb,0.30703e-04_rb,0.29586e-04_rb,0.28499e-04_rb,0.27519e-04_rb /)
      kbo(:,23, 1) = (/ &
        & 0.26638e-04_rb,0.25643e-04_rb,0.24693e-04_rb,0.23832e-04_rb,0.22913e-04_rb /)
      kbo(:,24, 1) = (/ &
        & 0.22325e-04_rb,0.21480e-04_rb,0.20621e-04_rb,0.19867e-04_rb,0.19066e-04_rb /)
      kbo(:,25, 1) = (/ &
        & 0.18764e-04_rb,0.17998e-04_rb,0.17269e-04_rb,0.16527e-04_rb,0.15821e-04_rb /)
      kbo(:,26, 1) = (/ &
        & 0.15650e-04_rb,0.14992e-04_rb,0.14356e-04_rb,0.13710e-04_rb,0.13119e-04_rb /)
      kbo(:,27, 1) = (/ &
        & 0.13054e-04_rb,0.12477e-04_rb,0.11929e-04_rb,0.11393e-04_rb,0.10948e-04_rb /)
      kbo(:,28, 1) = (/ &
        & 0.10852e-04_rb,0.10390e-04_rb,0.99195e-05_rb,0.95020e-05_rb,0.91993e-05_rb /)
      kbo(:,29, 1) = (/ &
        & 0.90990e-05_rb,0.87209e-05_rb,0.83384e-05_rb,0.80029e-05_rb,0.77245e-05_rb /)
      kbo(:,30, 1) = (/ &
        & 0.77169e-05_rb,0.73662e-05_rb,0.70552e-05_rb,0.68029e-05_rb,0.65749e-05_rb /)
      kbo(:,31, 1) = (/ &
        & 0.65250e-05_rb,0.62364e-05_rb,0.60172e-05_rb,0.57804e-05_rb,0.55504e-05_rb /)
      kbo(:,32, 1) = (/ &
        & 0.55167e-05_rb,0.53021e-05_rb,0.51088e-05_rb,0.49113e-05_rb,0.47150e-05_rb /)
      kbo(:,33, 1) = (/ &
        & 0.47261e-05_rb,0.45530e-05_rb,0.43641e-05_rb,0.41725e-05_rb,0.40077e-05_rb /)
      kbo(:,34, 1) = (/ &
        & 0.40646e-05_rb,0.39062e-05_rb,0.37472e-05_rb,0.36009e-05_rb,0.34549e-05_rb /)
      kbo(:,35, 1) = (/ &
        & 0.35479e-05_rb,0.34057e-05_rb,0.32704e-05_rb,0.31420e-05_rb,0.30219e-05_rb /)
      kbo(:,36, 1) = (/ &
        & 0.31353e-05_rb,0.30270e-05_rb,0.29024e-05_rb,0.27776e-05_rb,0.26600e-05_rb /)
      kbo(:,37, 1) = (/ &
        & 0.26595e-05_rb,0.25621e-05_rb,0.24646e-05_rb,0.23582e-05_rb,0.22563e-05_rb /)
      kbo(:,38, 1) = (/ &
        & 0.22468e-05_rb,0.21651e-05_rb,0.20812e-05_rb,0.20043e-05_rb,0.19221e-05_rb /)
      kbo(:,39, 1) = (/ &
        & 0.19009e-05_rb,0.18335e-05_rb,0.17685e-05_rb,0.17039e-05_rb,0.16406e-05_rb /)
      kbo(:,40, 1) = (/ &
        & 0.15746e-05_rb,0.15188e-05_rb,0.14669e-05_rb,0.14148e-05_rb,0.13668e-05_rb /)
      kbo(:,41, 1) = (/ &
        & 0.13001e-05_rb,0.12561e-05_rb,0.12122e-05_rb,0.11718e-05_rb,0.11306e-05_rb /)
      kbo(:,42, 1) = (/ &
        & 0.10716e-05_rb,0.10380e-05_rb,0.10018e-05_rb,0.96745e-06_rb,0.93475e-06_rb /)
      kbo(:,43, 1) = (/ &
        & 0.87651e-06_rb,0.85172e-06_rb,0.82453e-06_rb,0.79560e-06_rb,0.77011e-06_rb /)
      kbo(:,44, 1) = (/ &
        & 0.71565e-06_rb,0.69699e-06_rb,0.67699e-06_rb,0.65403e-06_rb,0.63202e-06_rb /)
      kbo(:,45, 1) = (/ &
        & 0.58391e-06_rb,0.56946e-06_rb,0.55318e-06_rb,0.53575e-06_rb,0.51774e-06_rb /)
      kbo(:,46, 1) = (/ &
        & 0.47490e-06_rb,0.46531e-06_rb,0.45175e-06_rb,0.43686e-06_rb,0.42260e-06_rb /)
      kbo(:,47, 1) = (/ &
        & 0.38718e-06_rb,0.37610e-06_rb,0.36650e-06_rb,0.35580e-06_rb,0.34417e-06_rb /)
      kbo(:,48, 1) = (/ &
        & 0.31464e-06_rb,0.30494e-06_rb,0.29872e-06_rb,0.29096e-06_rb,0.28230e-06_rb /)
      kbo(:,49, 1) = (/ &
        & 0.25542e-06_rb,0.24885e-06_rb,0.24314e-06_rb,0.23745e-06_rb,0.23092e-06_rb /)
      kbo(:,50, 1) = (/ &
        & 0.20741e-06_rb,0.20175e-06_rb,0.19699e-06_rb,0.19305e-06_rb,0.18791e-06_rb /)
      kbo(:,51, 1) = (/ &
        & 0.16760e-06_rb,0.16406e-06_rb,0.15993e-06_rb,0.15631e-06_rb,0.15327e-06_rb /)
      kbo(:,52, 1) = (/ &
        & 0.13579e-06_rb,0.13287e-06_rb,0.12985e-06_rb,0.12725e-06_rb,0.12470e-06_rb /)
      kbo(:,53, 1) = (/ &
        & 0.11006e-06_rb,0.10757e-06_rb,0.10568e-06_rb,0.10314e-06_rb,0.10101e-06_rb /)
      kbo(:,54, 1) = (/ &
        & 0.88711e-07_rb,0.87131e-07_rb,0.85162e-07_rb,0.83405e-07_rb,0.81585e-07_rb /)
      kbo(:,55, 1) = (/ &
        & 0.70889e-07_rb,0.69917e-07_rb,0.68692e-07_rb,0.67463e-07_rb,0.66047e-07_rb /)
      kbo(:,56, 1) = (/ &
        & 0.56659e-07_rb,0.56077e-07_rb,0.55518e-07_rb,0.54462e-07_rb,0.53563e-07_rb /)
      kbo(:,57, 1) = (/ &
        & 0.45380e-07_rb,0.45159e-07_rb,0.44718e-07_rb,0.44063e-07_rb,0.43116e-07_rb /)
      kbo(:,58, 1) = (/ &
        & 0.35962e-07_rb,0.36177e-07_rb,0.35997e-07_rb,0.35600e-07_rb,0.34340e-07_rb /)
      kbo(:,59, 1) = (/ &
        & 0.28578e-07_rb,0.28924e-07_rb,0.28901e-07_rb,0.28176e-07_rb,0.27246e-07_rb /)
      kbo(:,13, 2) = (/ &
        & 0.63215e-03_rb,0.64756e-03_rb,0.65734e-03_rb,0.66515e-03_rb,0.67077e-03_rb /)
      kbo(:,14, 2) = (/ &
        & 0.52378e-03_rb,0.53631e-03_rb,0.54485e-03_rb,0.55071e-03_rb,0.55605e-03_rb /)
      kbo(:,15, 2) = (/ &
        & 0.43777e-03_rb,0.44814e-03_rb,0.45521e-03_rb,0.46124e-03_rb,0.46700e-03_rb /)
      kbo(:,16, 2) = (/ &
        & 0.37023e-03_rb,0.37869e-03_rb,0.38568e-03_rb,0.39070e-03_rb,0.39760e-03_rb /)
      kbo(:,17, 2) = (/ &
        & 0.31753e-03_rb,0.32632e-03_rb,0.33254e-03_rb,0.33817e-03_rb,0.34312e-03_rb /)
      kbo(:,18, 2) = (/ &
        & 0.27580e-03_rb,0.28310e-03_rb,0.28915e-03_rb,0.29488e-03_rb,0.29855e-03_rb /)
      kbo(:,19, 2) = (/ &
        & 0.24129e-03_rb,0.24697e-03_rb,0.25302e-03_rb,0.25785e-03_rb,0.26137e-03_rb /)
      kbo(:,20, 2) = (/ &
        & 0.20470e-03_rb,0.21040e-03_rb,0.21552e-03_rb,0.21963e-03_rb,0.22260e-03_rb /)
      kbo(:,21, 2) = (/ &
        & 0.17290e-03_rb,0.17782e-03_rb,0.18188e-03_rb,0.18513e-03_rb,0.18794e-03_rb /)
      kbo(:,22, 2) = (/ &
        & 0.14599e-03_rb,0.15063e-03_rb,0.15389e-03_rb,0.15682e-03_rb,0.15965e-03_rb /)
      kbo(:,23, 2) = (/ &
        & 0.12390e-03_rb,0.12797e-03_rb,0.13071e-03_rb,0.13347e-03_rb,0.13571e-03_rb /)
      kbo(:,24, 2) = (/ &
        & 0.10553e-03_rb,0.10875e-03_rb,0.11134e-03_rb,0.11351e-03_rb,0.11513e-03_rb /)
      kbo(:,25, 2) = (/ &
        & 0.90263e-04_rb,0.92560e-04_rb,0.94667e-04_rb,0.96225e-04_rb,0.97041e-04_rb /)
      kbo(:,26, 2) = (/ &
        & 0.76542e-04_rb,0.78227e-04_rb,0.79755e-04_rb,0.80768e-04_rb,0.81412e-04_rb /)
      kbo(:,27, 2) = (/ &
        & 0.64562e-04_rb,0.66024e-04_rb,0.67121e-04_rb,0.67878e-04_rb,0.67992e-04_rb /)
      kbo(:,28, 2) = (/ &
        & 0.54587e-04_rb,0.55668e-04_rb,0.56439e-04_rb,0.56778e-04_rb,0.56649e-04_rb /)
      kbo(:,29, 2) = (/ &
        & 0.46248e-04_rb,0.46885e-04_rb,0.47469e-04_rb,0.47476e-04_rb,0.47464e-04_rb /)
      kbo(:,30, 2) = (/ &
        & 0.39084e-04_rb,0.39701e-04_rb,0.39836e-04_rb,0.39919e-04_rb,0.39762e-04_rb /)
      kbo(:,31, 2) = (/ &
        & 0.33297e-04_rb,0.33670e-04_rb,0.33781e-04_rb,0.33571e-04_rb,0.33418e-04_rb /)
      kbo(:,32, 2) = (/ &
        & 0.28475e-04_rb,0.28479e-04_rb,0.28352e-04_rb,0.28202e-04_rb,0.28107e-04_rb /)
      kbo(:,33, 2) = (/ &
        & 0.24023e-04_rb,0.23977e-04_rb,0.23874e-04_rb,0.23858e-04_rb,0.23729e-04_rb /)
      kbo(:,34, 2) = (/ &
        & 0.20276e-04_rb,0.20314e-04_rb,0.20262e-04_rb,0.20205e-04_rb,0.20013e-04_rb /)
      kbo(:,35, 2) = (/ &
        & 0.17177e-04_rb,0.17179e-04_rb,0.17181e-04_rb,0.17048e-04_rb,0.16915e-04_rb /)
      kbo(:,36, 2) = (/ &
        & 0.14586e-04_rb,0.14632e-04_rb,0.14569e-04_rb,0.14465e-04_rb,0.14404e-04_rb /)
      kbo(:,37, 2) = (/ &
        & 0.12165e-04_rb,0.12220e-04_rb,0.12164e-04_rb,0.12099e-04_rb,0.12036e-04_rb /)
      kbo(:,38, 2) = (/ &
        & 0.10178e-04_rb,0.10224e-04_rb,0.10201e-04_rb,0.10134e-04_rb,0.10060e-04_rb /)
      kbo(:,39, 2) = (/ &
        & 0.85622e-05_rb,0.85826e-05_rb,0.85520e-05_rb,0.85109e-05_rb,0.84378e-05_rb /)
      kbo(:,40, 2) = (/ &
        & 0.70263e-05_rb,0.70549e-05_rb,0.70334e-05_rb,0.70000e-05_rb,0.69500e-05_rb /)
      kbo(:,41, 2) = (/ &
        & 0.57510e-05_rb,0.57749e-05_rb,0.57692e-05_rb,0.57384e-05_rb,0.57121e-05_rb /)
      kbo(:,42, 2) = (/ &
        & 0.47038e-05_rb,0.47261e-05_rb,0.47278e-05_rb,0.47023e-05_rb,0.46867e-05_rb /)
      kbo(:,43, 2) = (/ &
        & 0.38249e-05_rb,0.38339e-05_rb,0.38498e-05_rb,0.38364e-05_rb,0.38125e-05_rb /)
      kbo(:,44, 2) = (/ &
        & 0.30973e-05_rb,0.31120e-05_rb,0.31166e-05_rb,0.31203e-05_rb,0.31022e-05_rb /)
      kbo(:,45, 2) = (/ &
        & 0.25085e-05_rb,0.25263e-05_rb,0.25293e-05_rb,0.25328e-05_rb,0.25249e-05_rb /)
      kbo(:,46, 2) = (/ &
        & 0.20261e-05_rb,0.20479e-05_rb,0.20565e-05_rb,0.20575e-05_rb,0.20588e-05_rb /)
      kbo(:,47, 2) = (/ &
        & 0.16319e-05_rb,0.16639e-05_rb,0.16745e-05_rb,0.16740e-05_rb,0.16759e-05_rb /)
      kbo(:,48, 2) = (/ &
        & 0.13097e-05_rb,0.13449e-05_rb,0.13611e-05_rb,0.13624e-05_rb,0.13589e-05_rb /)
      kbo(:,49, 2) = (/ &
        & 0.10521e-05_rb,0.10847e-05_rb,0.11052e-05_rb,0.11102e-05_rb,0.11085e-05_rb /)
      kbo(:,50, 2) = (/ &
        & 0.84668e-06_rb,0.87235e-06_rb,0.89544e-06_rb,0.90607e-06_rb,0.90616e-06_rb /)
      kbo(:,51, 2) = (/ &
        & 0.68060e-06_rb,0.70180e-06_rb,0.72187e-06_rb,0.73562e-06_rb,0.73800e-06_rb /)
      kbo(:,52, 2) = (/ &
        & 0.54634e-06_rb,0.56481e-06_rb,0.58256e-06_rb,0.59543e-06_rb,0.60122e-06_rb /)
      kbo(:,53, 2) = (/ &
        & 0.43866e-06_rb,0.45577e-06_rb,0.46957e-06_rb,0.48176e-06_rb,0.48940e-06_rb /)
      kbo(:,54, 2) = (/ &
        & 0.35062e-06_rb,0.36647e-06_rb,0.37935e-06_rb,0.38978e-06_rb,0.39697e-06_rb /)
      kbo(:,55, 2) = (/ &
        & 0.27883e-06_rb,0.29514e-06_rb,0.30397e-06_rb,0.31251e-06_rb,0.32004e-06_rb /)
      kbo(:,56, 2) = (/ &
        & 0.22098e-06_rb,0.23550e-06_rb,0.24289e-06_rb,0.25018e-06_rb,0.25708e-06_rb /)
      kbo(:,57, 2) = (/ &
        & 0.17295e-06_rb,0.18624e-06_rb,0.19332e-06_rb,0.19934e-06_rb,0.20587e-06_rb /)
      kbo(:,58, 2) = (/ &
        & 0.13623e-06_rb,0.14680e-06_rb,0.15403e-06_rb,0.15912e-06_rb,0.16499e-06_rb /)
      kbo(:,59, 2) = (/ &
        & 0.10864e-06_rb,0.11680e-06_rb,0.12353e-06_rb,0.12840e-06_rb,0.13352e-06_rb /)
      kbo(:,13, 3) = (/ &
        & 0.21069e-02_rb,0.21571e-02_rb,0.21943e-02_rb,0.22061e-02_rb,0.22124e-02_rb /)
      kbo(:,14, 3) = (/ &
        & 0.17411e-02_rb,0.17833e-02_rb,0.18097e-02_rb,0.18225e-02_rb,0.18251e-02_rb /)
      kbo(:,15, 3) = (/ &
        & 0.14440e-02_rb,0.14790e-02_rb,0.14963e-02_rb,0.15066e-02_rb,0.15066e-02_rb /)
      kbo(:,16, 3) = (/ &
        & 0.11998e-02_rb,0.12261e-02_rb,0.12398e-02_rb,0.12500e-02_rb,0.12487e-02_rb /)
      kbo(:,17, 3) = (/ &
        & 0.99980e-03_rb,0.10202e-02_rb,0.10332e-02_rb,0.10399e-02_rb,0.10392e-02_rb /)
      kbo(:,18, 3) = (/ &
        & 0.83805e-03_rb,0.85629e-03_rb,0.86827e-03_rb,0.87303e-03_rb,0.87540e-03_rb /)
      kbo(:,19, 3) = (/ &
        & 0.70743e-03_rb,0.72381e-03_rb,0.73251e-03_rb,0.73550e-03_rb,0.73931e-03_rb /)
      kbo(:,20, 3) = (/ &
        & 0.59348e-03_rb,0.60572e-03_rb,0.61137e-03_rb,0.61496e-03_rb,0.61761e-03_rb /)
      kbo(:,21, 3) = (/ &
        & 0.49556e-03_rb,0.50488e-03_rb,0.50987e-03_rb,0.51421e-03_rb,0.51581e-03_rb /)
      kbo(:,22, 3) = (/ &
        & 0.41461e-03_rb,0.42145e-03_rb,0.42584e-03_rb,0.42969e-03_rb,0.43055e-03_rb /)
      kbo(:,23, 3) = (/ &
        & 0.34683e-03_rb,0.35181e-03_rb,0.35660e-03_rb,0.35911e-03_rb,0.35958e-03_rb /)
      kbo(:,24, 3) = (/ &
        & 0.29004e-03_rb,0.29383e-03_rb,0.29787e-03_rb,0.29964e-03_rb,0.30042e-03_rb /)
      kbo(:,25, 3) = (/ &
        & 0.24146e-03_rb,0.24561e-03_rb,0.24879e-03_rb,0.25053e-03_rb,0.25134e-03_rb /)
      kbo(:,26, 3) = (/ &
        & 0.20160e-03_rb,0.20537e-03_rb,0.20774e-03_rb,0.20928e-03_rb,0.21023e-03_rb /)
      kbo(:,27, 3) = (/ &
        & 0.16916e-03_rb,0.17226e-03_rb,0.17386e-03_rb,0.17529e-03_rb,0.17634e-03_rb /)
      kbo(:,28, 3) = (/ &
        & 0.14183e-03_rb,0.14414e-03_rb,0.14604e-03_rb,0.14741e-03_rb,0.14844e-03_rb /)
      kbo(:,29, 3) = (/ &
        & 0.11954e-03_rb,0.12161e-03_rb,0.12286e-03_rb,0.12386e-03_rb,0.12448e-03_rb /)
      kbo(:,30, 3) = (/ &
        & 0.10067e-03_rb,0.10205e-03_rb,0.10331e-03_rb,0.10409e-03_rb,0.10476e-03_rb /)
      kbo(:,31, 3) = (/ &
        & 0.85014e-04_rb,0.86248e-04_rb,0.87024e-04_rb,0.87958e-04_rb,0.88579e-04_rb /)
      kbo(:,32, 3) = (/ &
        & 0.71930e-04_rb,0.73074e-04_rb,0.73985e-04_rb,0.74934e-04_rb,0.74848e-04_rb /)
      kbo(:,33, 3) = (/ &
        & 0.61205e-04_rb,0.62259e-04_rb,0.63209e-04_rb,0.63383e-04_rb,0.63466e-04_rb /)
      kbo(:,34, 3) = (/ &
        & 0.52046e-04_rb,0.52896e-04_rb,0.53420e-04_rb,0.53516e-04_rb,0.53755e-04_rb /)
      kbo(:,35, 3) = (/ &
        & 0.44380e-04_rb,0.45121e-04_rb,0.45277e-04_rb,0.45604e-04_rb,0.45761e-04_rb /)
      kbo(:,36, 3) = (/ &
        & 0.38058e-04_rb,0.38409e-04_rb,0.38657e-04_rb,0.38813e-04_rb,0.38628e-04_rb /)
      kbo(:,37, 3) = (/ &
        & 0.31735e-04_rb,0.31990e-04_rb,0.32199e-04_rb,0.32288e-04_rb,0.32179e-04_rb /)
      kbo(:,38, 3) = (/ &
        & 0.26378e-04_rb,0.26667e-04_rb,0.26778e-04_rb,0.26840e-04_rb,0.26760e-04_rb /)
      kbo(:,39, 3) = (/ &
        & 0.21949e-04_rb,0.22195e-04_rb,0.22340e-04_rb,0.22330e-04_rb,0.22281e-04_rb /)
      kbo(:,40, 3) = (/ &
        & 0.18001e-04_rb,0.18209e-04_rb,0.18338e-04_rb,0.18376e-04_rb,0.18309e-04_rb /)
      kbo(:,41, 3) = (/ &
        & 0.14715e-04_rb,0.14906e-04_rb,0.15025e-04_rb,0.15098e-04_rb,0.15026e-04_rb /)
      kbo(:,42, 3) = (/ &
        & 0.12019e-04_rb,0.12197e-04_rb,0.12312e-04_rb,0.12380e-04_rb,0.12346e-04_rb /)
      kbo(:,43, 3) = (/ &
        & 0.97566e-05_rb,0.99318e-05_rb,0.10036e-04_rb,0.10096e-04_rb,0.10118e-04_rb /)
      kbo(:,44, 3) = (/ &
        & 0.78810e-05_rb,0.80716e-05_rb,0.81637e-05_rb,0.82202e-05_rb,0.82608e-05_rb /)
      kbo(:,45, 3) = (/ &
        & 0.63541e-05_rb,0.65171e-05_rb,0.66330e-05_rb,0.66977e-05_rb,0.67239e-05_rb /)
      kbo(:,46, 3) = (/ &
        & 0.51077e-05_rb,0.52428e-05_rb,0.53636e-05_rb,0.54363e-05_rb,0.54693e-05_rb /)
      kbo(:,47, 3) = (/ &
        & 0.40883e-05_rb,0.42196e-05_rb,0.43199e-05_rb,0.44074e-05_rb,0.44462e-05_rb /)
      kbo(:,48, 3) = (/ &
        & 0.32909e-05_rb,0.33941e-05_rb,0.34746e-05_rb,0.35546e-05_rb,0.36089e-05_rb /)
      kbo(:,49, 3) = (/ &
        & 0.26401e-05_rb,0.27269e-05_rb,0.28003e-05_rb,0.28616e-05_rb,0.29188e-05_rb /)
      kbo(:,50, 3) = (/ &
        & 0.21222e-05_rb,0.22032e-05_rb,0.22638e-05_rb,0.23127e-05_rb,0.23603e-05_rb /)
      kbo(:,51, 3) = (/ &
        & 0.17056e-05_rb,0.17838e-05_rb,0.18386e-05_rb,0.18804e-05_rb,0.19134e-05_rb /)
      kbo(:,52, 3) = (/ &
        & 0.13662e-05_rb,0.14333e-05_rb,0.14855e-05_rb,0.15238e-05_rb,0.15521e-05_rb /)
      kbo(:,53, 3) = (/ &
        & 0.10889e-05_rb,0.11445e-05_rb,0.11955e-05_rb,0.12300e-05_rb,0.12560e-05_rb /)
      kbo(:,54, 3) = (/ &
        & 0.86251e-06_rb,0.91556e-06_rb,0.95999e-06_rb,0.99454e-06_rb,0.10171e-05_rb /)
      kbo(:,55, 3) = (/ &
        & 0.68640e-06_rb,0.72868e-06_rb,0.77389e-06_rb,0.80627e-06_rb,0.82696e-06_rb /)
      kbo(:,56, 3) = (/ &
        & 0.54334e-06_rb,0.58339e-06_rb,0.61953e-06_rb,0.65190e-06_rb,0.67285e-06_rb /)
      kbo(:,57, 3) = (/ &
        & 0.43336e-06_rb,0.46511e-06_rb,0.49561e-06_rb,0.52337e-06_rb,0.54491e-06_rb /)
      kbo(:,58, 3) = (/ &
        & 0.34428e-06_rb,0.37236e-06_rb,0.39687e-06_rb,0.42017e-06_rb,0.43790e-06_rb /)
      kbo(:,59, 3) = (/ &
        & 0.27883e-06_rb,0.30237e-06_rb,0.32096e-06_rb,0.33978e-06_rb,0.35485e-06_rb /)
      kbo(:,13, 4) = (/ &
        & 0.52806e-02_rb,0.53551e-02_rb,0.53836e-02_rb,0.53810e-02_rb,0.53413e-02_rb /)
      kbo(:,14, 4) = (/ &
        & 0.43594e-02_rb,0.44114e-02_rb,0.44268e-02_rb,0.44183e-02_rb,0.43877e-02_rb /)
      kbo(:,15, 4) = (/ &
        & 0.36080e-02_rb,0.36421e-02_rb,0.36603e-02_rb,0.36540e-02_rb,0.36307e-02_rb /)
      kbo(:,16, 4) = (/ &
        & 0.30007e-02_rb,0.30324e-02_rb,0.30479e-02_rb,0.30377e-02_rb,0.30175e-02_rb /)
      kbo(:,17, 4) = (/ &
        & 0.25113e-02_rb,0.25415e-02_rb,0.25489e-02_rb,0.25428e-02_rb,0.25281e-02_rb /)
      kbo(:,18, 4) = (/ &
        & 0.21119e-02_rb,0.21340e-02_rb,0.21396e-02_rb,0.21381e-02_rb,0.21232e-02_rb /)
      kbo(:,19, 4) = (/ &
        & 0.17705e-02_rb,0.17884e-02_rb,0.17942e-02_rb,0.17931e-02_rb,0.17819e-02_rb /)
      kbo(:,20, 4) = (/ &
        & 0.14714e-02_rb,0.14854e-02_rb,0.14921e-02_rb,0.14920e-02_rb,0.14874e-02_rb /)
      kbo(:,21, 4) = (/ &
        & 0.12224e-02_rb,0.12349e-02_rb,0.12432e-02_rb,0.12411e-02_rb,0.12395e-02_rb /)
      kbo(:,22, 4) = (/ &
        & 0.10170e-02_rb,0.10281e-02_rb,0.10319e-02_rb,0.10310e-02_rb,0.10286e-02_rb /)
      kbo(:,23, 4) = (/ &
        & 0.84681e-03_rb,0.85484e-03_rb,0.85517e-03_rb,0.85545e-03_rb,0.85571e-03_rb /)
      kbo(:,24, 4) = (/ &
        & 0.70330e-03_rb,0.70875e-03_rb,0.71078e-03_rb,0.71261e-03_rb,0.71112e-03_rb /)
      kbo(:,25, 4) = (/ &
        & 0.58481e-03_rb,0.58870e-03_rb,0.59157e-03_rb,0.59305e-03_rb,0.59234e-03_rb /)
      kbo(:,26, 4) = (/ &
        & 0.48640e-03_rb,0.49044e-03_rb,0.49395e-03_rb,0.49468e-03_rb,0.49466e-03_rb /)
      kbo(:,27, 4) = (/ &
        & 0.40472e-03_rb,0.40915e-03_rb,0.41235e-03_rb,0.41278e-03_rb,0.41345e-03_rb /)
      kbo(:,28, 4) = (/ &
        & 0.33826e-03_rb,0.34205e-03_rb,0.34364e-03_rb,0.34480e-03_rb,0.34528e-03_rb /)
      kbo(:,29, 4) = (/ &
        & 0.28308e-03_rb,0.28627e-03_rb,0.28792e-03_rb,0.28910e-03_rb,0.28958e-03_rb /)
      kbo(:,30, 4) = (/ &
        & 0.23759e-03_rb,0.23994e-03_rb,0.24196e-03_rb,0.24295e-03_rb,0.24270e-03_rb /)
      kbo(:,31, 4) = (/ &
        & 0.19993e-03_rb,0.20202e-03_rb,0.20390e-03_rb,0.20456e-03_rb,0.20411e-03_rb /)
      kbo(:,32, 4) = (/ &
        & 0.16851e-03_rb,0.17060e-03_rb,0.17151e-03_rb,0.17169e-03_rb,0.17133e-03_rb /)
      kbo(:,33, 4) = (/ &
        & 0.14275e-03_rb,0.14392e-03_rb,0.14445e-03_rb,0.14452e-03_rb,0.14399e-03_rb /)
      kbo(:,34, 4) = (/ &
        & 0.12060e-03_rb,0.12142e-03_rb,0.12181e-03_rb,0.12165e-03_rb,0.12105e-03_rb /)
      kbo(:,35, 4) = (/ &
        & 0.10168e-03_rb,0.10240e-03_rb,0.10276e-03_rb,0.10261e-03_rb,0.10202e-03_rb /)
      kbo(:,36, 4) = (/ &
        & 0.85780e-04_rb,0.86636e-04_rb,0.86815e-04_rb,0.86780e-04_rb,0.86722e-04_rb /)
      kbo(:,37, 4) = (/ &
        & 0.71540e-04_rb,0.72201e-04_rb,0.72422e-04_rb,0.72501e-04_rb,0.72398e-04_rb /)
      kbo(:,38, 4) = (/ &
        & 0.59609e-04_rb,0.60152e-04_rb,0.60443e-04_rb,0.60540e-04_rb,0.60534e-04_rb /)
      kbo(:,39, 4) = (/ &
        & 0.49681e-04_rb,0.50194e-04_rb,0.50481e-04_rb,0.50649e-04_rb,0.50643e-04_rb /)
      kbo(:,40, 4) = (/ &
        & 0.40815e-04_rb,0.41335e-04_rb,0.41565e-04_rb,0.41729e-04_rb,0.41776e-04_rb /)
      kbo(:,41, 4) = (/ &
        & 0.33476e-04_rb,0.33932e-04_rb,0.34171e-04_rb,0.34301e-04_rb,0.34397e-04_rb /)
      kbo(:,42, 4) = (/ &
        & 0.27419e-04_rb,0.27824e-04_rb,0.28083e-04_rb,0.28221e-04_rb,0.28296e-04_rb /)
      kbo(:,43, 4) = (/ &
        & 0.22349e-04_rb,0.22753e-04_rb,0.22975e-04_rb,0.23100e-04_rb,0.23164e-04_rb /)
      kbo(:,44, 4) = (/ &
        & 0.18197e-04_rb,0.18488e-04_rb,0.18747e-04_rb,0.18878e-04_rb,0.18943e-04_rb /)
      kbo(:,45, 4) = (/ &
        & 0.14815e-04_rb,0.15081e-04_rb,0.15284e-04_rb,0.15408e-04_rb,0.15493e-04_rb /)
      kbo(:,46, 4) = (/ &
        & 0.12046e-04_rb,0.12304e-04_rb,0.12476e-04_rb,0.12590e-04_rb,0.12654e-04_rb /)
      kbo(:,47, 4) = (/ &
        & 0.97834e-05_rb,0.10018e-04_rb,0.10187e-04_rb,0.10295e-04_rb,0.10365e-04_rb /)
      kbo(:,48, 4) = (/ &
        & 0.79058e-05_rb,0.81583e-05_rb,0.83158e-05_rb,0.84259e-05_rb,0.84791e-05_rb /)
      kbo(:,49, 4) = (/ &
        & 0.63847e-05_rb,0.66212e-05_rb,0.67735e-05_rb,0.68818e-05_rb,0.69382e-05_rb /)
      kbo(:,50, 4) = (/ &
        & 0.51619e-05_rb,0.53698e-05_rb,0.55239e-05_rb,0.56230e-05_rb,0.56931e-05_rb /)
      kbo(:,51, 4) = (/ &
        & 0.41764e-05_rb,0.43546e-05_rb,0.45018e-05_rb,0.45927e-05_rb,0.46609e-05_rb /)
      kbo(:,52, 4) = (/ &
        & 0.33696e-05_rb,0.35302e-05_rb,0.36585e-05_rb,0.37548e-05_rb,0.38091e-05_rb /)
      kbo(:,53, 4) = (/ &
        & 0.27142e-05_rb,0.28585e-05_rb,0.29700e-05_rb,0.30632e-05_rb,0.31163e-05_rb /)
      kbo(:,54, 4) = (/ &
        & 0.21974e-05_rb,0.23176e-05_rb,0.24169e-05_rb,0.24980e-05_rb,0.25555e-05_rb /)
      kbo(:,55, 4) = (/ &
        & 0.17820e-05_rb,0.18827e-05_rb,0.19647e-05_rb,0.20347e-05_rb,0.20895e-05_rb /)
      kbo(:,56, 4) = (/ &
        & 0.14445e-05_rb,0.15274e-05_rb,0.15999e-05_rb,0.16558e-05_rb,0.17004e-05_rb /)
      kbo(:,57, 4) = (/ &
        & 0.11612e-05_rb,0.12390e-05_rb,0.13020e-05_rb,0.13508e-05_rb,0.13881e-05_rb /)
      kbo(:,58, 4) = (/ &
        & 0.93606e-06_rb,0.10017e-05_rb,0.10598e-05_rb,0.11016e-05_rb,0.11367e-05_rb /)
      kbo(:,59, 4) = (/ &
        & 0.76682e-06_rb,0.82318e-06_rb,0.87281e-06_rb,0.91208e-06_rb,0.94037e-06_rb /)
      kbo(:,13, 5) = (/ &
        & 0.12856e-01_rb,0.13079e-01_rb,0.13167e-01_rb,0.13177e-01_rb,0.13097e-01_rb /)
      kbo(:,14, 5) = (/ &
        & 0.10708e-01_rb,0.10853e-01_rb,0.10911e-01_rb,0.10898e-01_rb,0.10814e-01_rb /)
      kbo(:,15, 5) = (/ &
        & 0.88690e-02_rb,0.89759e-02_rb,0.90116e-02_rb,0.89730e-02_rb,0.88895e-02_rb /)
      kbo(:,16, 5) = (/ &
        & 0.73377e-02_rb,0.74116e-02_rb,0.74224e-02_rb,0.73845e-02_rb,0.73097e-02_rb /)
      kbo(:,17, 5) = (/ &
        & 0.61050e-02_rb,0.61465e-02_rb,0.61522e-02_rb,0.61190e-02_rb,0.60607e-02_rb /)
      kbo(:,18, 5) = (/ &
        & 0.51170e-02_rb,0.51524e-02_rb,0.51567e-02_rb,0.51299e-02_rb,0.50853e-02_rb /)
      kbo(:,19, 5) = (/ &
        & 0.42872e-02_rb,0.43210e-02_rb,0.43249e-02_rb,0.43090e-02_rb,0.42772e-02_rb /)
      kbo(:,20, 5) = (/ &
        & 0.35726e-02_rb,0.35987e-02_rb,0.36030e-02_rb,0.35888e-02_rb,0.35606e-02_rb /)
      kbo(:,21, 5) = (/ &
        & 0.29687e-02_rb,0.29865e-02_rb,0.29887e-02_rb,0.29805e-02_rb,0.29536e-02_rb /)
      kbo(:,22, 5) = (/ &
        & 0.24631e-02_rb,0.24766e-02_rb,0.24818e-02_rb,0.24706e-02_rb,0.24511e-02_rb /)
      kbo(:,23, 5) = (/ &
        & 0.20423e-02_rb,0.20574e-02_rb,0.20622e-02_rb,0.20550e-02_rb,0.20390e-02_rb /)
      kbo(:,24, 5) = (/ &
        & 0.17009e-02_rb,0.17137e-02_rb,0.17151e-02_rb,0.17066e-02_rb,0.16944e-02_rb /)
      kbo(:,25, 5) = (/ &
        & 0.14175e-02_rb,0.14281e-02_rb,0.14264e-02_rb,0.14202e-02_rb,0.14101e-02_rb /)
      kbo(:,26, 5) = (/ &
        & 0.11824e-02_rb,0.11879e-02_rb,0.11852e-02_rb,0.11799e-02_rb,0.11727e-02_rb /)
      kbo(:,27, 5) = (/ &
        & 0.98538e-03_rb,0.98799e-03_rb,0.98648e-03_rb,0.98258e-03_rb,0.97798e-03_rb /)
      kbo(:,28, 5) = (/ &
        & 0.82156e-03_rb,0.82317e-03_rb,0.82299e-03_rb,0.82154e-03_rb,0.81913e-03_rb /)
      kbo(:,29, 5) = (/ &
        & 0.68689e-03_rb,0.68935e-03_rb,0.68992e-03_rb,0.69010e-03_rb,0.68854e-03_rb /)
      kbo(:,30, 5) = (/ &
        & 0.57658e-03_rb,0.57999e-03_rb,0.58178e-03_rb,0.58209e-03_rb,0.58220e-03_rb /)
      kbo(:,31, 5) = (/ &
        & 0.48638e-03_rb,0.49049e-03_rb,0.49261e-03_rb,0.49405e-03_rb,0.49565e-03_rb /)
      kbo(:,32, 5) = (/ &
        & 0.41128e-03_rb,0.41527e-03_rb,0.41847e-03_rb,0.42111e-03_rb,0.42278e-03_rb /)
      kbo(:,33, 5) = (/ &
        & 0.34942e-03_rb,0.35347e-03_rb,0.35669e-03_rb,0.35970e-03_rb,0.36033e-03_rb /)
      kbo(:,34, 5) = (/ &
        & 0.29778e-03_rb,0.30183e-03_rb,0.30543e-03_rb,0.30722e-03_rb,0.30756e-03_rb /)
      kbo(:,35, 5) = (/ &
        & 0.25439e-03_rb,0.25855e-03_rb,0.26145e-03_rb,0.26279e-03_rb,0.26328e-03_rb /)
      kbo(:,36, 5) = (/ &
        & 0.21688e-03_rb,0.22055e-03_rb,0.22366e-03_rb,0.22472e-03_rb,0.22545e-03_rb /)
      kbo(:,37, 5) = (/ &
        & 0.18184e-03_rb,0.18525e-03_rb,0.18789e-03_rb,0.18888e-03_rb,0.18981e-03_rb /)
      kbo(:,38, 5) = (/ &
        & 0.15217e-03_rb,0.15529e-03_rb,0.15752e-03_rb,0.15878e-03_rb,0.15947e-03_rb /)
      kbo(:,39, 5) = (/ &
        & 0.12738e-03_rb,0.13010e-03_rb,0.13202e-03_rb,0.13325e-03_rb,0.13405e-03_rb /)
      kbo(:,40, 5) = (/ &
        & 0.10533e-03_rb,0.10771e-03_rb,0.10944e-03_rb,0.11063e-03_rb,0.11132e-03_rb /)
      kbo(:,41, 5) = (/ &
        & 0.86935e-04_rb,0.89079e-04_rb,0.90576e-04_rb,0.91599e-04_rb,0.92136e-04_rb /)
      kbo(:,42, 5) = (/ &
        & 0.71757e-04_rb,0.73600e-04_rb,0.74913e-04_rb,0.75823e-04_rb,0.76332e-04_rb /)
      kbo(:,43, 5) = (/ &
        & 0.58933e-04_rb,0.60448e-04_rb,0.61689e-04_rb,0.62502e-04_rb,0.62968e-04_rb /)
      kbo(:,44, 5) = (/ &
        & 0.48240e-04_rb,0.49626e-04_rb,0.50671e-04_rb,0.51353e-04_rb,0.51808e-04_rb /)
      kbo(:,45, 5) = (/ &
        & 0.39365e-04_rb,0.40603e-04_rb,0.41556e-04_rb,0.42201e-04_rb,0.42661e-04_rb /)
      kbo(:,46, 5) = (/ &
        & 0.32140e-04_rb,0.33199e-04_rb,0.34102e-04_rb,0.34681e-04_rb,0.35122e-04_rb /)
      kbo(:,47, 5) = (/ &
        & 0.26212e-04_rb,0.27122e-04_rb,0.27914e-04_rb,0.28482e-04_rb,0.28883e-04_rb /)
      kbo(:,48, 5) = (/ &
        & 0.21367e-04_rb,0.22132e-04_rb,0.22808e-04_rb,0.23366e-04_rb,0.23750e-04_rb /)
      kbo(:,49, 5) = (/ &
        & 0.17366e-04_rb,0.18036e-04_rb,0.18610e-04_rb,0.19128e-04_rb,0.19501e-04_rb /)
      kbo(:,50, 5) = (/ &
        & 0.14166e-04_rb,0.14744e-04_rb,0.15227e-04_rb,0.15695e-04_rb,0.16033e-04_rb /)
      kbo(:,51, 5) = (/ &
        & 0.11520e-04_rb,0.12036e-04_rb,0.12466e-04_rb,0.12850e-04_rb,0.13177e-04_rb /)
      kbo(:,52, 5) = (/ &
        & 0.93545e-05_rb,0.97967e-05_rb,0.10185e-04_rb,0.10504e-04_rb,0.10808e-04_rb /)
      kbo(:,53, 5) = (/ &
        & 0.75900e-05_rb,0.79775e-05_rb,0.83066e-05_rb,0.85876e-05_rb,0.88446e-05_rb /)
      kbo(:,54, 5) = (/ &
        & 0.61841e-05_rb,0.65213e-05_rb,0.67983e-05_rb,0.70299e-05_rb,0.72479e-05_rb /)
      kbo(:,55, 5) = (/ &
        & 0.50267e-05_rb,0.53142e-05_rb,0.55644e-05_rb,0.57696e-05_rb,0.59449e-05_rb /)
      kbo(:,56, 5) = (/ &
        & 0.40664e-05_rb,0.43169e-05_rb,0.45344e-05_rb,0.47274e-05_rb,0.48783e-05_rb /)
      kbo(:,57, 5) = (/ &
        & 0.32881e-05_rb,0.34993e-05_rb,0.36900e-05_rb,0.38580e-05_rb,0.39983e-05_rb /)
      kbo(:,58, 5) = (/ &
        & 0.26562e-05_rb,0.28386e-05_rb,0.30033e-05_rb,0.31435e-05_rb,0.32655e-05_rb /)
      kbo(:,59, 5) = (/ &
        & 0.21842e-05_rb,0.23355e-05_rb,0.24742e-05_rb,0.25918e-05_rb,0.26979e-05_rb /)
      kbo(:,13, 6) = (/ &
        & 0.31802e-01_rb,0.32385e-01_rb,0.32761e-01_rb,0.32924e-01_rb,0.32887e-01_rb /)
      kbo(:,14, 6) = (/ &
        & 0.27075e-01_rb,0.27526e-01_rb,0.27777e-01_rb,0.27825e-01_rb,0.27714e-01_rb /)
      kbo(:,15, 6) = (/ &
        & 0.22867e-01_rb,0.23213e-01_rb,0.23346e-01_rb,0.23326e-01_rb,0.23192e-01_rb /)
      kbo(:,16, 6) = (/ &
        & 0.19243e-01_rb,0.19469e-01_rb,0.19534e-01_rb,0.19508e-01_rb,0.19374e-01_rb /)
      kbo(:,17, 6) = (/ &
        & 0.16143e-01_rb,0.16291e-01_rb,0.16340e-01_rb,0.16286e-01_rb,0.16140e-01_rb /)
      kbo(:,18, 6) = (/ &
        & 0.13508e-01_rb,0.13619e-01_rb,0.13632e-01_rb,0.13563e-01_rb,0.13444e-01_rb /)
      kbo(:,19, 6) = (/ &
        & 0.11360e-01_rb,0.11423e-01_rb,0.11418e-01_rb,0.11354e-01_rb,0.11254e-01_rb /)
      kbo(:,20, 6) = (/ &
        & 0.95257e-02_rb,0.95677e-02_rb,0.95569e-02_rb,0.94993e-02_rb,0.94147e-02_rb /)
      kbo(:,21, 6) = (/ &
        & 0.79688e-02_rb,0.80079e-02_rb,0.79930e-02_rb,0.79576e-02_rb,0.78976e-02_rb /)
      kbo(:,22, 6) = (/ &
        & 0.66527e-02_rb,0.66824e-02_rb,0.66717e-02_rb,0.66442e-02_rb,0.65988e-02_rb /)
      kbo(:,23, 6) = (/ &
        & 0.55583e-02_rb,0.55741e-02_rb,0.55685e-02_rb,0.55458e-02_rb,0.55136e-02_rb /)
      kbo(:,24, 6) = (/ &
        & 0.46281e-02_rb,0.46404e-02_rb,0.46354e-02_rb,0.46240e-02_rb,0.46070e-02_rb /)
      kbo(:,25, 6) = (/ &
        & 0.38479e-02_rb,0.38586e-02_rb,0.38647e-02_rb,0.38587e-02_rb,0.38482e-02_rb /)
      kbo(:,26, 6) = (/ &
        & 0.31996e-02_rb,0.32113e-02_rb,0.32185e-02_rb,0.32206e-02_rb,0.32221e-02_rb /)
      kbo(:,27, 6) = (/ &
        & 0.26655e-02_rb,0.26804e-02_rb,0.26909e-02_rb,0.27001e-02_rb,0.27000e-02_rb /)
      kbo(:,28, 6) = (/ &
        & 0.22269e-02_rb,0.22430e-02_rb,0.22546e-02_rb,0.22636e-02_rb,0.22685e-02_rb /)
      kbo(:,29, 6) = (/ &
        & 0.18711e-02_rb,0.18853e-02_rb,0.19010e-02_rb,0.19079e-02_rb,0.19146e-02_rb /)
      kbo(:,30, 6) = (/ &
        & 0.15792e-02_rb,0.15950e-02_rb,0.16049e-02_rb,0.16149e-02_rb,0.16250e-02_rb /)
      kbo(:,31, 6) = (/ &
        & 0.13390e-02_rb,0.13523e-02_rb,0.13631e-02_rb,0.13762e-02_rb,0.13899e-02_rb /)
      kbo(:,32, 6) = (/ &
        & 0.11423e-02_rb,0.11552e-02_rb,0.11689e-02_rb,0.11828e-02_rb,0.12002e-02_rb /)
      kbo(:,33, 6) = (/ &
        & 0.97837e-03_rb,0.99414e-03_rb,0.10078e-02_rb,0.10254e-02_rb,0.10445e-02_rb /)
      kbo(:,34, 6) = (/ &
        & 0.84508e-03_rb,0.85961e-03_rb,0.87553e-03_rb,0.89474e-03_rb,0.91454e-03_rb /)
      kbo(:,35, 6) = (/ &
        & 0.73067e-03_rb,0.74643e-03_rb,0.76522e-03_rb,0.78433e-03_rb,0.80547e-03_rb /)
      kbo(:,36, 6) = (/ &
        & 0.63501e-03_rb,0.65286e-03_rb,0.67124e-03_rb,0.69177e-03_rb,0.71245e-03_rb /)
      kbo(:,37, 6) = (/ &
        & 0.54302e-03_rb,0.56011e-03_rb,0.57838e-03_rb,0.59932e-03_rb,0.61930e-03_rb /)
      kbo(:,38, 6) = (/ &
        & 0.46413e-03_rb,0.48141e-03_rb,0.49888e-03_rb,0.51888e-03_rb,0.53851e-03_rb /)
      kbo(:,39, 6) = (/ &
        & 0.39787e-03_rb,0.41416e-03_rb,0.43137e-03_rb,0.45076e-03_rb,0.46884e-03_rb /)
      kbo(:,40, 6) = (/ &
        & 0.33542e-03_rb,0.35026e-03_rb,0.36640e-03_rb,0.38421e-03_rb,0.40074e-03_rb /)
      kbo(:,41, 6) = (/ &
        & 0.28198e-03_rb,0.29533e-03_rb,0.31033e-03_rb,0.32658e-03_rb,0.34246e-03_rb /)
      kbo(:,42, 6) = (/ &
        & 0.23644e-03_rb,0.24877e-03_rb,0.26248e-03_rb,0.27695e-03_rb,0.29184e-03_rb /)
      kbo(:,43, 6) = (/ &
        & 0.19698e-03_rb,0.20826e-03_rb,0.22042e-03_rb,0.23357e-03_rb,0.24697e-03_rb /)
      kbo(:,44, 6) = (/ &
        & 0.16338e-03_rb,0.17334e-03_rb,0.18409e-03_rb,0.19557e-03_rb,0.20792e-03_rb /)
      kbo(:,45, 6) = (/ &
        & 0.13558e-03_rb,0.14397e-03_rb,0.15333e-03_rb,0.16359e-03_rb,0.17463e-03_rb /)
      kbo(:,46, 6) = (/ &
        & 0.11191e-03_rb,0.11919e-03_rb,0.12729e-03_rb,0.13624e-03_rb,0.14629e-03_rb /)
      kbo(:,47, 6) = (/ &
        & 0.91906e-04_rb,0.98189e-04_rb,0.10520e-03_rb,0.11302e-03_rb,0.12177e-03_rb /)
      kbo(:,48, 6) = (/ &
        & 0.75087e-04_rb,0.80743e-04_rb,0.86708e-04_rb,0.93427e-04_rb,0.10110e-03_rb /)
      kbo(:,49, 6) = (/ &
        & 0.61186e-04_rb,0.66125e-04_rb,0.71313e-04_rb,0.77107e-04_rb,0.83777e-04_rb /)
      kbo(:,50, 6) = (/ &
        & 0.49938e-04_rb,0.54303e-04_rb,0.58807e-04_rb,0.63705e-04_rb,0.69474e-04_rb /)
      kbo(:,51, 6) = (/ &
        & 0.40809e-04_rb,0.44513e-04_rb,0.48369e-04_rb,0.52594e-04_rb,0.57525e-04_rb /)
      kbo(:,52, 6) = (/ &
        & 0.33309e-04_rb,0.36401e-04_rb,0.39694e-04_rb,0.43295e-04_rb,0.47517e-04_rb /)
      kbo(:,53, 6) = (/ &
        & 0.27001e-04_rb,0.29637e-04_rb,0.32443e-04_rb,0.35515e-04_rb,0.39073e-04_rb /)
      kbo(:,54, 6) = (/ &
        & 0.21920e-04_rb,0.24176e-04_rb,0.26587e-04_rb,0.29204e-04_rb,0.32173e-04_rb /)
      kbo(:,55, 6) = (/ &
        & 0.17840e-04_rb,0.19707e-04_rb,0.21784e-04_rb,0.24000e-04_rb,0.26498e-04_rb /)
      kbo(:,56, 6) = (/ &
        & 0.14500e-04_rb,0.16042e-04_rb,0.17779e-04_rb,0.19655e-04_rb,0.21743e-04_rb /)
      kbo(:,57, 6) = (/ &
        & 0.11756e-04_rb,0.13018e-04_rb,0.14458e-04_rb,0.16053e-04_rb,0.17779e-04_rb /)
      kbo(:,58, 6) = (/ &
        & 0.95229e-05_rb,0.10574e-04_rb,0.11760e-04_rb,0.13107e-04_rb,0.14585e-04_rb /)
      kbo(:,59, 6) = (/ &
        & 0.78970e-05_rb,0.87821e-05_rb,0.97860e-05_rb,0.10919e-04_rb,0.12187e-04_rb /)
      kbo(:,13, 7) = (/ &
        & 0.81347e-01_rb,0.83688e-01_rb,0.85307e-01_rb,0.86283e-01_rb,0.87010e-01_rb /)
      kbo(:,14, 7) = (/ &
        & 0.72041e-01_rb,0.73994e-01_rb,0.75287e-01_rb,0.76180e-01_rb,0.76714e-01_rb /)
      kbo(:,15, 7) = (/ &
        & 0.63278e-01_rb,0.64818e-01_rb,0.65954e-01_rb,0.66651e-01_rb,0.67143e-01_rb /)
      kbo(:,16, 7) = (/ &
        & 0.55139e-01_rb,0.56478e-01_rb,0.57384e-01_rb,0.58076e-01_rb,0.58539e-01_rb /)
      kbo(:,17, 7) = (/ &
        & 0.47894e-01_rb,0.48966e-01_rb,0.49829e-01_rb,0.50447e-01_rb,0.50916e-01_rb /)
      kbo(:,18, 7) = (/ &
        & 0.41499e-01_rb,0.42507e-01_rb,0.43240e-01_rb,0.43755e-01_rb,0.44135e-01_rb /)
      kbo(:,19, 7) = (/ &
        & 0.35885e-01_rb,0.36754e-01_rb,0.37403e-01_rb,0.37954e-01_rb,0.38313e-01_rb /)
      kbo(:,20, 7) = (/ &
        & 0.30699e-01_rb,0.31463e-01_rb,0.32108e-01_rb,0.32586e-01_rb,0.32972e-01_rb /)
      kbo(:,21, 7) = (/ &
        & 0.26192e-01_rb,0.26846e-01_rb,0.27409e-01_rb,0.27839e-01_rb,0.28217e-01_rb /)
      kbo(:,22, 7) = (/ &
        & 0.22340e-01_rb,0.22929e-01_rb,0.23404e-01_rb,0.23811e-01_rb,0.24187e-01_rb /)
      kbo(:,23, 7) = (/ &
        & 0.19095e-01_rb,0.19575e-01_rb,0.20007e-01_rb,0.20410e-01_rb,0.20766e-01_rb /)
      kbo(:,24, 7) = (/ &
        & 0.16366e-01_rb,0.16771e-01_rb,0.17186e-01_rb,0.17542e-01_rb,0.17949e-01_rb /)
      kbo(:,25, 7) = (/ &
        & 0.14069e-01_rb,0.14481e-01_rb,0.14832e-01_rb,0.15200e-01_rb,0.15580e-01_rb /)
      kbo(:,26, 7) = (/ &
        & 0.12162e-01_rb,0.12539e-01_rb,0.12888e-01_rb,0.13247e-01_rb,0.13615e-01_rb /)
      kbo(:,27, 7) = (/ &
        & 0.10528e-01_rb,0.10892e-01_rb,0.11265e-01_rb,0.11631e-01_rb,0.12012e-01_rb /)
      kbo(:,28, 7) = (/ &
        & 0.91441e-02_rb,0.95086e-02_rb,0.98896e-02_rb,0.10282e-01_rb,0.10687e-01_rb /)
      kbo(:,29, 7) = (/ &
        & 0.79938e-02_rb,0.83641e-02_rb,0.87511e-02_rb,0.91674e-02_rb,0.96121e-02_rb /)
      kbo(:,30, 7) = (/ &
        & 0.70400e-02_rb,0.74172e-02_rb,0.78125e-02_rb,0.82611e-02_rb,0.87532e-02_rb /)
      kbo(:,31, 7) = (/ &
        & 0.62612e-02_rb,0.66491e-02_rb,0.70836e-02_rb,0.75546e-02_rb,0.80730e-02_rb /)
      kbo(:,32, 7) = (/ &
        & 0.56090e-02_rb,0.60373e-02_rb,0.65021e-02_rb,0.70140e-02_rb,0.75729e-02_rb /)
      kbo(:,33, 7) = (/ &
        & 0.51011e-02_rb,0.55473e-02_rb,0.60429e-02_rb,0.66026e-02_rb,0.72780e-02_rb /)
      kbo(:,34, 7) = (/ &
        & 0.46990e-02_rb,0.51673e-02_rb,0.57072e-02_rb,0.63464e-02_rb,0.71159e-02_rb /)
      kbo(:,35, 7) = (/ &
        & 0.43314e-02_rb,0.48384e-02_rb,0.54243e-02_rb,0.61639e-02_rb,0.69924e-02_rb /)
      kbo(:,36, 7) = (/ &
        & 0.39886e-02_rb,0.45262e-02_rb,0.51941e-02_rb,0.59985e-02_rb,0.68708e-02_rb /)
      kbo(:,37, 7) = (/ &
        & 0.36017e-02_rb,0.41399e-02_rb,0.48219e-02_rb,0.56481e-02_rb,0.65560e-02_rb /)
      kbo(:,38, 7) = (/ &
        & 0.32617e-02_rb,0.38037e-02_rb,0.45057e-02_rb,0.53485e-02_rb,0.62708e-02_rb /)
      kbo(:,39, 7) = (/ &
        & 0.29777e-02_rb,0.35364e-02_rb,0.42387e-02_rb,0.50890e-02_rb,0.60544e-02_rb /)
      kbo(:,40, 7) = (/ &
        & 0.26397e-02_rb,0.31715e-02_rb,0.38573e-02_rb,0.46880e-02_rb,0.56335e-02_rb /)
      kbo(:,41, 7) = (/ &
        & 0.23227e-02_rb,0.28385e-02_rb,0.34927e-02_rb,0.42880e-02_rb,0.52154e-02_rb /)
      kbo(:,42, 7) = (/ &
        & 0.20486e-02_rb,0.25378e-02_rb,0.31647e-02_rb,0.39312e-02_rb,0.48261e-02_rb /)
      kbo(:,43, 7) = (/ &
        & 0.17755e-02_rb,0.22226e-02_rb,0.28140e-02_rb,0.35274e-02_rb,0.43895e-02_rb /)
      kbo(:,44, 7) = (/ &
        & 0.15269e-02_rb,0.19274e-02_rb,0.24688e-02_rb,0.31407e-02_rb,0.39517e-02_rb /)
      kbo(:,45, 7) = (/ &
        & 0.13050e-02_rb,0.16697e-02_rb,0.21647e-02_rb,0.27916e-02_rb,0.35423e-02_rb /)
      kbo(:,46, 7) = (/ &
        & 0.11057e-02_rb,0.14327e-02_rb,0.18777e-02_rb,0.24513e-02_rb,0.31448e-02_rb /)
      kbo(:,47, 7) = (/ &
        & 0.92448e-03_rb,0.12134e-02_rb,0.16064e-02_rb,0.21237e-02_rb,0.27575e-02_rb /)
      kbo(:,48, 7) = (/ &
        & 0.76618e-03_rb,0.10179e-02_rb,0.13627e-02_rb,0.18254e-02_rb,0.24067e-02_rb /)
      kbo(:,49, 7) = (/ &
        & 0.62940e-03_rb,0.85036e-03_rb,0.11489e-02_rb,0.15623e-02_rb,0.20896e-02_rb /)
      kbo(:,50, 7) = (/ &
        & 0.51855e-03_rb,0.71067e-03_rb,0.97602e-03_rb,0.13414e-02_rb,0.18179e-02_rb /)
      kbo(:,51, 7) = (/ &
        & 0.42590e-03_rb,0.59305e-03_rb,0.82548e-03_rb,0.11486e-02_rb,0.15798e-02_rb /)
      kbo(:,52, 7) = (/ &
        & 0.34697e-03_rb,0.49059e-03_rb,0.69535e-03_rb,0.97532e-03_rb,0.13630e-02_rb /)
      kbo(:,53, 7) = (/ &
        & 0.28092e-03_rb,0.40191e-03_rb,0.57931e-03_rb,0.82673e-03_rb,0.11685e-02_rb /)
      kbo(:,54, 7) = (/ &
        & 0.22883e-03_rb,0.33102e-03_rb,0.48584e-03_rb,0.70577e-03_rb,0.10069e-02_rb /)
      kbo(:,55, 7) = (/ &
        & 0.18567e-03_rb,0.27212e-03_rb,0.40629e-03_rb,0.59966e-03_rb,0.86643e-03_rb /)
      kbo(:,56, 7) = (/ &
        & 0.14981e-03_rb,0.22158e-03_rb,0.33702e-03_rb,0.50672e-03_rb,0.74022e-03_rb /)
      kbo(:,57, 7) = (/ &
        & 0.11956e-03_rb,0.17899e-03_rb,0.27717e-03_rb,0.42406e-03_rb,0.63003e-03_rb /)
      kbo(:,58, 7) = (/ &
        & 0.95581e-04_rb,0.14413e-03_rb,0.22637e-03_rb,0.35457e-03_rb,0.53700e-03_rb /)
      kbo(:,59, 7) = (/ &
        & 0.81967e-04_rb,0.12578e-03_rb,0.20146e-03_rb,0.32061e-03_rb,0.49099e-03_rb /)
      kbo(:,13, 8) = (/ &
        & 0.22107e+00_rb,0.22673e+00_rb,0.23188e+00_rb,0.23626e+00_rb,0.23937e+00_rb /)
      kbo(:,14, 8) = (/ &
        & 0.20348e+00_rb,0.20983e+00_rb,0.21521e+00_rb,0.21960e+00_rb,0.22328e+00_rb /)
      kbo(:,15, 8) = (/ &
        & 0.18724e+00_rb,0.19358e+00_rb,0.19903e+00_rb,0.20382e+00_rb,0.20780e+00_rb /)
      kbo(:,16, 8) = (/ &
        & 0.17271e+00_rb,0.17902e+00_rb,0.18466e+00_rb,0.18949e+00_rb,0.19358e+00_rb /)
      kbo(:,17, 8) = (/ &
        & 0.15881e+00_rb,0.16550e+00_rb,0.17139e+00_rb,0.17656e+00_rb,0.18127e+00_rb /)
      kbo(:,18, 8) = (/ &
        & 0.14632e+00_rb,0.15311e+00_rb,0.15937e+00_rb,0.16532e+00_rb,0.17049e+00_rb /)
      kbo(:,19, 8) = (/ &
        & 0.13481e+00_rb,0.14215e+00_rb,0.14899e+00_rb,0.15502e+00_rb,0.16068e+00_rb /)
      kbo(:,20, 8) = (/ &
        & 0.12437e+00_rb,0.13191e+00_rb,0.13890e+00_rb,0.14560e+00_rb,0.15188e+00_rb /)
      kbo(:,21, 8) = (/ &
        & 0.11481e+00_rb,0.12252e+00_rb,0.12982e+00_rb,0.13710e+00_rb,0.14374e+00_rb /)
      kbo(:,22, 8) = (/ &
        & 0.10661e+00_rb,0.11453e+00_rb,0.12244e+00_rb,0.12996e+00_rb,0.13720e+00_rb /)
      kbo(:,23, 8) = (/ &
        & 0.99573e-01_rb,0.10782e+00_rb,0.11598e+00_rb,0.12384e+00_rb,0.13154e+00_rb /)
      kbo(:,24, 8) = (/ &
        & 0.93585e-01_rb,0.10212e+00_rb,0.11044e+00_rb,0.11870e+00_rb,0.12683e+00_rb /)
      kbo(:,25, 8) = (/ &
        & 0.88642e-01_rb,0.97342e-01_rb,0.10600e+00_rb,0.11474e+00_rb,0.12338e+00_rb /)
      kbo(:,26, 8) = (/ &
        & 0.84783e-01_rb,0.93706e-01_rb,0.10275e+00_rb,0.11190e+00_rb,0.12089e+00_rb /)
      kbo(:,27, 8) = (/ &
        & 0.81924e-01_rb,0.91085e-01_rb,0.10043e+00_rb,0.11011e+00_rb,0.11942e+00_rb /)
      kbo(:,28, 8) = (/ &
        & 0.79898e-01_rb,0.89495e-01_rb,0.99246e-01_rb,0.10903e+00_rb,0.11866e+00_rb /)
      kbo(:,29, 8) = (/ &
        & 0.78843e-01_rb,0.88816e-01_rb,0.99112e-01_rb,0.10908e+00_rb,0.11873e+00_rb /)
      kbo(:,30, 8) = (/ &
        & 0.78672e-01_rb,0.89175e-01_rb,0.99763e-01_rb,0.10991e+00_rb,0.11974e+00_rb /)
      kbo(:,31, 8) = (/ &
        & 0.79471e-01_rb,0.90393e-01_rb,0.10090e+00_rb,0.11134e+00_rb,0.12148e+00_rb /)
      kbo(:,32, 8) = (/ &
        & 0.81097e-01_rb,0.92118e-01_rb,0.10299e+00_rb,0.11352e+00_rb,0.12362e+00_rb /)
      kbo(:,33, 8) = (/ &
        & 0.83219e-01_rb,0.94430e-01_rb,0.10536e+00_rb,0.11619e+00_rb,0.12656e+00_rb /)
      kbo(:,34, 8) = (/ &
        & 0.85363e-01_rb,0.96707e-01_rb,0.10798e+00_rb,0.11904e+00_rb,0.12986e+00_rb /)
      kbo(:,35, 8) = (/ &
        & 0.86741e-01_rb,0.98297e-01_rb,0.11004e+00_rb,0.12158e+00_rb,0.13234e+00_rb /)
      kbo(:,36, 8) = (/ &
        & 0.87017e-01_rb,0.99113e-01_rb,0.11121e+00_rb,0.12302e+00_rb,0.13390e+00_rb /)
      kbo(:,37, 8) = (/ &
        & 0.85258e-01_rb,0.97671e-01_rb,0.10985e+00_rb,0.12188e+00_rb,0.13289e+00_rb /)
      kbo(:,38, 8) = (/ &
        & 0.83622e-01_rb,0.96133e-01_rb,0.10847e+00_rb,0.12060e+00_rb,0.13189e+00_rb /)
      kbo(:,39, 8) = (/ &
        & 0.82148e-01_rb,0.94689e-01_rb,0.10723e+00_rb,0.11944e+00_rb,0.13102e+00_rb /)
      kbo(:,40, 8) = (/ &
        & 0.78933e-01_rb,0.91494e-01_rb,0.10410e+00_rb,0.11650e+00_rb,0.12816e+00_rb /)
      kbo(:,41, 8) = (/ &
        & 0.75553e-01_rb,0.88087e-01_rb,0.10066e+00_rb,0.11319e+00_rb,0.12513e+00_rb /)
      kbo(:,42, 8) = (/ &
        & 0.72215e-01_rb,0.84600e-01_rb,0.97314e-01_rb,0.10990e+00_rb,0.12202e+00_rb /)
      kbo(:,43, 8) = (/ &
        & 0.68050e-01_rb,0.80392e-01_rb,0.93038e-01_rb,0.10568e+00_rb,0.11811e+00_rb /)
      kbo(:,44, 8) = (/ &
        & 0.63504e-01_rb,0.75885e-01_rb,0.88427e-01_rb,0.10109e+00_rb,0.11352e+00_rb /)
      kbo(:,45, 8) = (/ &
        & 0.59117e-01_rb,0.71359e-01_rb,0.83835e-01_rb,0.96461e-01_rb,0.10897e+00_rb /)
      kbo(:,46, 8) = (/ &
        & 0.54516e-01_rb,0.66547e-01_rb,0.79117e-01_rb,0.91745e-01_rb,0.10426e+00_rb /)
      kbo(:,47, 8) = (/ &
        & 0.49520e-01_rb,0.61408e-01_rb,0.73788e-01_rb,0.86383e-01_rb,0.98886e-01_rb /)
      kbo(:,48, 8) = (/ &
        & 0.44727e-01_rb,0.56224e-01_rb,0.68415e-01_rb,0.81094e-01_rb,0.93621e-01_rb /)
      kbo(:,49, 8) = (/ &
        & 0.40177e-01_rb,0.51146e-01_rb,0.63191e-01_rb,0.75749e-01_rb,0.88249e-01_rb /)
      kbo(:,50, 8) = (/ &
        & 0.36081e-01_rb,0.46596e-01_rb,0.58341e-01_rb,0.70769e-01_rb,0.83353e-01_rb /)
      kbo(:,51, 8) = (/ &
        & 0.32278e-01_rb,0.42393e-01_rb,0.53717e-01_rb,0.65898e-01_rb,0.78578e-01_rb /)
      kbo(:,52, 8) = (/ &
        & 0.28586e-01_rb,0.38352e-01_rb,0.49175e-01_rb,0.61145e-01_rb,0.73682e-01_rb /)
      kbo(:,53, 8) = (/ &
        & 0.25090e-01_rb,0.34439e-01_rb,0.44807e-01_rb,0.56352e-01_rb,0.68823e-01_rb /)
      kbo(:,54, 8) = (/ &
        & 0.22080e-01_rb,0.30938e-01_rb,0.40952e-01_rb,0.52101e-01_rb,0.64364e-01_rb /)
      kbo(:,55, 8) = (/ &
        & 0.19382e-01_rb,0.27719e-01_rb,0.37405e-01_rb,0.48197e-01_rb,0.60044e-01_rb /)
      kbo(:,56, 8) = (/ &
        & 0.16835e-01_rb,0.24689e-01_rb,0.33908e-01_rb,0.44390e-01_rb,0.55911e-01_rb /)
      kbo(:,57, 8) = (/ &
        & 0.14459e-01_rb,0.21799e-01_rb,0.30531e-01_rb,0.40642e-01_rb,0.51808e-01_rb /)
      kbo(:,58, 8) = (/ &
        & 0.12380e-01_rb,0.19213e-01_rb,0.27475e-01_rb,0.37204e-01_rb,0.48065e-01_rb /)
      kbo(:,59, 8) = (/ &
        & 0.11536e-01_rb,0.18115e-01_rb,0.26160e-01_rb,0.35756e-01_rb,0.46501e-01_rb /)
      kbo(:,13, 9) = (/ &
        & 0.79231e+00_rb,0.80247e+00_rb,0.81169e+00_rb,0.81859e+00_rb,0.82469e+00_rb /)
      kbo(:,14, 9) = (/ &
        & 0.76567e+00_rb,0.77861e+00_rb,0.78976e+00_rb,0.79966e+00_rb,0.80754e+00_rb /)
      kbo(:,15, 9) = (/ &
        & 0.73926e+00_rb,0.75515e+00_rb,0.76982e+00_rb,0.78244e+00_rb,0.79228e+00_rb /)
      kbo(:,16, 9) = (/ &
        & 0.71308e+00_rb,0.73273e+00_rb,0.74955e+00_rb,0.76355e+00_rb,0.77553e+00_rb /)
      kbo(:,17, 9) = (/ &
        & 0.68845e+00_rb,0.70973e+00_rb,0.72813e+00_rb,0.74466e+00_rb,0.75909e+00_rb /)
      kbo(:,18, 9) = (/ &
        & 0.66397e+00_rb,0.68720e+00_rb,0.70872e+00_rb,0.72756e+00_rb,0.74467e+00_rb /)
      kbo(:,19, 9) = (/ &
        & 0.64097e+00_rb,0.66723e+00_rb,0.69155e+00_rb,0.71362e+00_rb,0.73339e+00_rb /)
      kbo(:,20, 9) = (/ &
        & 0.62162e+00_rb,0.65070e+00_rb,0.67764e+00_rb,0.70183e+00_rb,0.72348e+00_rb /)
      kbo(:,21, 9) = (/ &
        & 0.60571e+00_rb,0.63800e+00_rb,0.66649e+00_rb,0.69291e+00_rb,0.71712e+00_rb /)
      kbo(:,22, 9) = (/ &
        & 0.59651e+00_rb,0.62972e+00_rb,0.66058e+00_rb,0.68832e+00_rb,0.71517e+00_rb /)
      kbo(:,23, 9) = (/ &
        & 0.58974e+00_rb,0.62545e+00_rb,0.65753e+00_rb,0.68785e+00_rb,0.71519e+00_rb /)
      kbo(:,24, 9) = (/ &
        & 0.58721e+00_rb,0.62395e+00_rb,0.65801e+00_rb,0.68952e+00_rb,0.71705e+00_rb /)
      kbo(:,25, 9) = (/ &
        & 0.58931e+00_rb,0.62554e+00_rb,0.66080e+00_rb,0.69275e+00_rb,0.72212e+00_rb /)
      kbo(:,26, 9) = (/ &
        & 0.59392e+00_rb,0.63108e+00_rb,0.66576e+00_rb,0.69831e+00_rb,0.72831e+00_rb /)
      kbo(:,27, 9) = (/ &
        & 0.60088e+00_rb,0.63911e+00_rb,0.67310e+00_rb,0.70594e+00_rb,0.73492e+00_rb /)
      kbo(:,28, 9) = (/ &
        & 0.61058e+00_rb,0.64827e+00_rb,0.68351e+00_rb,0.71436e+00_rb,0.74248e+00_rb /)
      kbo(:,29, 9) = (/ &
        & 0.62259e+00_rb,0.66086e+00_rb,0.69462e+00_rb,0.72450e+00_rb,0.75205e+00_rb /)
      kbo(:,30, 9) = (/ &
        & 0.63547e+00_rb,0.67373e+00_rb,0.70715e+00_rb,0.73642e+00_rb,0.76277e+00_rb /)
      kbo(:,31, 9) = (/ &
        & 0.65060e+00_rb,0.68811e+00_rb,0.72216e+00_rb,0.75104e+00_rb,0.77730e+00_rb /)
      kbo(:,32, 9) = (/ &
        & 0.66635e+00_rb,0.70367e+00_rb,0.73703e+00_rb,0.76712e+00_rb,0.79450e+00_rb /)
      kbo(:,33, 9) = (/ &
        & 0.68330e+00_rb,0.72114e+00_rb,0.75529e+00_rb,0.78560e+00_rb,0.81251e+00_rb /)
      kbo(:,34, 9) = (/ &
        & 0.69895e+00_rb,0.73787e+00_rb,0.77209e+00_rb,0.80355e+00_rb,0.82948e+00_rb /)
      kbo(:,35, 9) = (/ &
        & 0.71249e+00_rb,0.75244e+00_rb,0.78786e+00_rb,0.82021e+00_rb,0.84807e+00_rb /)
      kbo(:,36, 9) = (/ &
        & 0.72352e+00_rb,0.76491e+00_rb,0.80192e+00_rb,0.83419e+00_rb,0.86330e+00_rb /)
      kbo(:,37, 9) = (/ &
        & 0.72315e+00_rb,0.76487e+00_rb,0.80310e+00_rb,0.83637e+00_rb,0.86732e+00_rb /)
      kbo(:,38, 9) = (/ &
        & 0.72144e+00_rb,0.76571e+00_rb,0.80444e+00_rb,0.83852e+00_rb,0.86991e+00_rb /)
      kbo(:,39, 9) = (/ &
        & 0.72061e+00_rb,0.76655e+00_rb,0.80661e+00_rb,0.84238e+00_rb,0.87357e+00_rb /)
      kbo(:,40, 9) = (/ &
        & 0.71071e+00_rb,0.75799e+00_rb,0.79882e+00_rb,0.83566e+00_rb,0.86813e+00_rb /)
      kbo(:,41, 9) = (/ &
        & 0.69960e+00_rb,0.74652e+00_rb,0.79034e+00_rb,0.82858e+00_rb,0.86201e+00_rb /)
      kbo(:,42, 9) = (/ &
        & 0.68767e+00_rb,0.73584e+00_rb,0.78080e+00_rb,0.82018e+00_rb,0.85505e+00_rb /)
      kbo(:,43, 9) = (/ &
        & 0.67181e+00_rb,0.72136e+00_rb,0.76709e+00_rb,0.80803e+00_rb,0.84449e+00_rb /)
      kbo(:,44, 9) = (/ &
        & 0.65485e+00_rb,0.70505e+00_rb,0.75190e+00_rb,0.79482e+00_rb,0.83260e+00_rb /)
      kbo(:,45, 9) = (/ &
        & 0.63639e+00_rb,0.68756e+00_rb,0.73541e+00_rb,0.78029e+00_rb,0.82060e+00_rb /)
      kbo(:,46, 9) = (/ &
        & 0.61610e+00_rb,0.67008e+00_rb,0.71893e+00_rb,0.76424e+00_rb,0.80647e+00_rb /)
      kbo(:,47, 9) = (/ &
        & 0.59373e+00_rb,0.64972e+00_rb,0.69969e+00_rb,0.74707e+00_rb,0.79068e+00_rb /)
      kbo(:,48, 9) = (/ &
        & 0.56865e+00_rb,0.62791e+00_rb,0.68069e+00_rb,0.72878e+00_rb,0.77390e+00_rb /)
      kbo(:,49, 9) = (/ &
        & 0.54350e+00_rb,0.60516e+00_rb,0.66027e+00_rb,0.71007e+00_rb,0.75592e+00_rb /)
      kbo(:,50, 9) = (/ &
        & 0.51991e+00_rb,0.58375e+00_rb,0.64136e+00_rb,0.69342e+00_rb,0.74093e+00_rb /)
      kbo(:,51, 9) = (/ &
        & 0.49673e+00_rb,0.56229e+00_rb,0.62200e+00_rb,0.67685e+00_rb,0.72556e+00_rb /)
      kbo(:,52, 9) = (/ &
        & 0.47256e+00_rb,0.53950e+00_rb,0.60133e+00_rb,0.65838e+00_rb,0.70926e+00_rb /)
      kbo(:,53, 9) = (/ &
        & 0.44771e+00_rb,0.51552e+00_rb,0.57980e+00_rb,0.63854e+00_rb,0.69202e+00_rb /)
      kbo(:,54, 9) = (/ &
        & 0.42506e+00_rb,0.49358e+00_rb,0.56009e+00_rb,0.62090e+00_rb,0.67547e+00_rb /)
      kbo(:,55, 9) = (/ &
        & 0.40218e+00_rb,0.47240e+00_rb,0.53999e+00_rb,0.60216e+00_rb,0.65925e+00_rb /)
      kbo(:,56, 9) = (/ &
        & 0.37888e+00_rb,0.45101e+00_rb,0.51913e+00_rb,0.58357e+00_rb,0.64216e+00_rb /)
      kbo(:,57, 9) = (/ &
        & 0.35513e+00_rb,0.42824e+00_rb,0.49765e+00_rb,0.56376e+00_rb,0.62409e+00_rb /)
      kbo(:,58, 9) = (/ &
        & 0.33174e+00_rb,0.40632e+00_rb,0.47713e+00_rb,0.54453e+00_rb,0.60605e+00_rb /)
      kbo(:,59, 9) = (/ &
        & 0.32315e+00_rb,0.39758e+00_rb,0.46935e+00_rb,0.53732e+00_rb,0.59942e+00_rb /)
      kbo(:,13,10) = (/ &
        & 0.20814e+01_rb,0.20759e+01_rb,0.20660e+01_rb,0.20703e+01_rb,0.20598e+01_rb /)
      kbo(:,14,10) = (/ &
        & 0.20861e+01_rb,0.20862e+01_rb,0.20902e+01_rb,0.20921e+01_rb,0.20850e+01_rb /)
      kbo(:,15,10) = (/ &
        & 0.20793e+01_rb,0.20969e+01_rb,0.21057e+01_rb,0.21014e+01_rb,0.21089e+01_rb /)
      kbo(:,16,10) = (/ &
        & 0.20801e+01_rb,0.20948e+01_rb,0.21151e+01_rb,0.21351e+01_rb,0.21560e+01_rb /)
      kbo(:,17,10) = (/ &
        & 0.20802e+01_rb,0.21249e+01_rb,0.21608e+01_rb,0.21842e+01_rb,0.21924e+01_rb /)
      kbo(:,18,10) = (/ &
        & 0.20972e+01_rb,0.21483e+01_rb,0.21814e+01_rb,0.21999e+01_rb,0.22088e+01_rb /)
      kbo(:,19,10) = (/ &
        & 0.21086e+01_rb,0.21513e+01_rb,0.21845e+01_rb,0.22099e+01_rb,0.22261e+01_rb /)
      kbo(:,20,10) = (/ &
        & 0.21001e+01_rb,0.21488e+01_rb,0.21917e+01_rb,0.22212e+01_rb,0.22394e+01_rb /)
      kbo(:,21,10) = (/ &
        & 0.20945e+01_rb,0.21494e+01_rb,0.22004e+01_rb,0.22330e+01_rb,0.22537e+01_rb /)
      kbo(:,22,10) = (/ &
        & 0.20951e+01_rb,0.21635e+01_rb,0.22151e+01_rb,0.22483e+01_rb,0.22658e+01_rb /)
      kbo(:,23,10) = (/ &
        & 0.21106e+01_rb,0.21802e+01_rb,0.22288e+01_rb,0.22593e+01_rb,0.22769e+01_rb /)
      kbo(:,24,10) = (/ &
        & 0.21251e+01_rb,0.21978e+01_rb,0.22479e+01_rb,0.22750e+01_rb,0.23069e+01_rb /)
      kbo(:,25,10) = (/ &
        & 0.21350e+01_rb,0.22127e+01_rb,0.22620e+01_rb,0.22919e+01_rb,0.23205e+01_rb /)
      kbo(:,26,10) = (/ &
        & 0.21397e+01_rb,0.22248e+01_rb,0.22778e+01_rb,0.23148e+01_rb,0.23338e+01_rb /)
      kbo(:,27,10) = (/ &
        & 0.21476e+01_rb,0.22244e+01_rb,0.22911e+01_rb,0.23271e+01_rb,0.23529e+01_rb /)
      kbo(:,28,10) = (/ &
        & 0.21502e+01_rb,0.22263e+01_rb,0.22854e+01_rb,0.23382e+01_rb,0.23803e+01_rb /)
      kbo(:,29,10) = (/ &
        & 0.21444e+01_rb,0.22159e+01_rb,0.22839e+01_rb,0.23447e+01_rb,0.24026e+01_rb /)
      kbo(:,30,10) = (/ &
        & 0.21459e+01_rb,0.22135e+01_rb,0.22803e+01_rb,0.23564e+01_rb,0.24114e+01_rb /)
      kbo(:,31,10) = (/ &
        & 0.21548e+01_rb,0.22131e+01_rb,0.22785e+01_rb,0.23559e+01_rb,0.23993e+01_rb /)
      kbo(:,32,10) = (/ &
        & 0.21820e+01_rb,0.22366e+01_rb,0.23010e+01_rb,0.23445e+01_rb,0.23808e+01_rb /)
      kbo(:,33,10) = (/ &
        & 0.22356e+01_rb,0.22739e+01_rb,0.22923e+01_rb,0.23386e+01_rb,0.23842e+01_rb /)
      kbo(:,34,10) = (/ &
        & 0.22861e+01_rb,0.23162e+01_rb,0.23352e+01_rb,0.23408e+01_rb,0.23839e+01_rb /)
      kbo(:,35,10) = (/ &
        & 0.23385e+01_rb,0.23680e+01_rb,0.23728e+01_rb,0.23741e+01_rb,0.23938e+01_rb /)
      kbo(:,36,10) = (/ &
        & 0.23851e+01_rb,0.24072e+01_rb,0.24258e+01_rb,0.24337e+01_rb,0.24446e+01_rb /)
      kbo(:,37,10) = (/ &
        & 0.23954e+01_rb,0.24271e+01_rb,0.24525e+01_rb,0.24638e+01_rb,0.24744e+01_rb /)
      kbo(:,38,10) = (/ &
        & 0.24138e+01_rb,0.24360e+01_rb,0.24658e+01_rb,0.24931e+01_rb,0.25112e+01_rb /)
      kbo(:,39,10) = (/ &
        & 0.24207e+01_rb,0.24415e+01_rb,0.24747e+01_rb,0.25054e+01_rb,0.25446e+01_rb /)
      kbo(:,40,10) = (/ &
        & 0.24183e+01_rb,0.24401e+01_rb,0.24836e+01_rb,0.25135e+01_rb,0.25532e+01_rb /)
      kbo(:,41,10) = (/ &
        & 0.24046e+01_rb,0.24486e+01_rb,0.24788e+01_rb,0.25037e+01_rb,0.25503e+01_rb /)
      kbo(:,42,10) = (/ &
        & 0.23965e+01_rb,0.24448e+01_rb,0.24659e+01_rb,0.25035e+01_rb,0.25440e+01_rb /)
      kbo(:,43,10) = (/ &
        & 0.23757e+01_rb,0.24456e+01_rb,0.24672e+01_rb,0.25043e+01_rb,0.25340e+01_rb /)
      kbo(:,44,10) = (/ &
        & 0.23417e+01_rb,0.24271e+01_rb,0.24708e+01_rb,0.24891e+01_rb,0.25157e+01_rb /)
      kbo(:,45,10) = (/ &
        & 0.23140e+01_rb,0.24224e+01_rb,0.24648e+01_rb,0.24895e+01_rb,0.25109e+01_rb /)
      kbo(:,46,10) = (/ &
        & 0.22839e+01_rb,0.23815e+01_rb,0.24548e+01_rb,0.24874e+01_rb,0.24983e+01_rb /)
      kbo(:,47,10) = (/ &
        & 0.22509e+01_rb,0.23585e+01_rb,0.24548e+01_rb,0.24976e+01_rb,0.25158e+01_rb /)
      kbo(:,48,10) = (/ &
        & 0.22243e+01_rb,0.23182e+01_rb,0.24167e+01_rb,0.24851e+01_rb,0.25144e+01_rb /)
      kbo(:,49,10) = (/ &
        & 0.21934e+01_rb,0.22838e+01_rb,0.23882e+01_rb,0.24694e+01_rb,0.25145e+01_rb /)
      kbo(:,50,10) = (/ &
        & 0.21637e+01_rb,0.22591e+01_rb,0.23593e+01_rb,0.24450e+01_rb,0.25097e+01_rb /)
      kbo(:,51,10) = (/ &
        & 0.21256e+01_rb,0.22383e+01_rb,0.23340e+01_rb,0.24239e+01_rb,0.25025e+01_rb /)
      kbo(:,52,10) = (/ &
        & 0.20804e+01_rb,0.22090e+01_rb,0.23144e+01_rb,0.24005e+01_rb,0.24803e+01_rb /)
      kbo(:,53,10) = (/ &
        & 0.20229e+01_rb,0.21712e+01_rb,0.22790e+01_rb,0.23709e+01_rb,0.24497e+01_rb /)
      kbo(:,54,10) = (/ &
        & 0.19647e+01_rb,0.21430e+01_rb,0.22514e+01_rb,0.23430e+01_rb,0.24343e+01_rb /)
      kbo(:,55,10) = (/ &
        & 0.19094e+01_rb,0.20990e+01_rb,0.22287e+01_rb,0.23283e+01_rb,0.24126e+01_rb /)
      kbo(:,56,10) = (/ &
        & 0.18499e+01_rb,0.20432e+01_rb,0.21961e+01_rb,0.22965e+01_rb,0.23921e+01_rb /)
      kbo(:,57,10) = (/ &
        & 0.17845e+01_rb,0.19868e+01_rb,0.21557e+01_rb,0.22661e+01_rb,0.23666e+01_rb /)
      kbo(:,58,10) = (/ &
        & 0.17304e+01_rb,0.19324e+01_rb,0.21130e+01_rb,0.22379e+01_rb,0.23344e+01_rb /)
      kbo(:,59,10) = (/ &
        & 0.17036e+01_rb,0.19129e+01_rb,0.20925e+01_rb,0.22299e+01_rb,0.23291e+01_rb /)
      kbo(:,13,11) = (/ &
        & 0.28487e+01_rb,0.28532e+01_rb,0.28392e+01_rb,0.28113e+01_rb,0.28077e+01_rb /)
      kbo(:,14,11) = (/ &
        & 0.29217e+01_rb,0.29135e+01_rb,0.28970e+01_rb,0.28895e+01_rb,0.29104e+01_rb /)
      kbo(:,15,11) = (/ &
        & 0.29720e+01_rb,0.29567e+01_rb,0.29515e+01_rb,0.29771e+01_rb,0.29749e+01_rb /)
      kbo(:,16,11) = (/ &
        & 0.29934e+01_rb,0.30029e+01_rb,0.30226e+01_rb,0.30339e+01_rb,0.30204e+01_rb /)
      kbo(:,17,11) = (/ &
        & 0.30408e+01_rb,0.30652e+01_rb,0.30797e+01_rb,0.30959e+01_rb,0.31120e+01_rb /)
      kbo(:,18,11) = (/ &
        & 0.31059e+01_rb,0.31457e+01_rb,0.31893e+01_rb,0.32154e+01_rb,0.32284e+01_rb /)
      kbo(:,19,11) = (/ &
        & 0.31895e+01_rb,0.32505e+01_rb,0.32863e+01_rb,0.32994e+01_rb,0.32949e+01_rb /)
      kbo(:,20,11) = (/ &
        & 0.32605e+01_rb,0.33203e+01_rb,0.33389e+01_rb,0.33469e+01_rb,0.33350e+01_rb /)
      kbo(:,21,11) = (/ &
        & 0.33097e+01_rb,0.33567e+01_rb,0.33762e+01_rb,0.33842e+01_rb,0.33798e+01_rb /)
      kbo(:,22,11) = (/ &
        & 0.33482e+01_rb,0.33901e+01_rb,0.34068e+01_rb,0.34227e+01_rb,0.34010e+01_rb /)
      kbo(:,23,11) = (/ &
        & 0.33722e+01_rb,0.34154e+01_rb,0.34438e+01_rb,0.34434e+01_rb,0.34235e+01_rb /)
      kbo(:,24,11) = (/ &
        & 0.33951e+01_rb,0.34377e+01_rb,0.34659e+01_rb,0.34658e+01_rb,0.34557e+01_rb /)
      kbo(:,25,11) = (/ &
        & 0.34209e+01_rb,0.34720e+01_rb,0.34904e+01_rb,0.34950e+01_rb,0.34725e+01_rb /)
      kbo(:,26,11) = (/ &
        & 0.34497e+01_rb,0.34967e+01_rb,0.35233e+01_rb,0.35072e+01_rb,0.34967e+01_rb /)
      kbo(:,27,11) = (/ &
        & 0.34791e+01_rb,0.35327e+01_rb,0.35370e+01_rb,0.35122e+01_rb,0.35139e+01_rb /)
      kbo(:,28,11) = (/ &
        & 0.35134e+01_rb,0.35471e+01_rb,0.35465e+01_rb,0.35405e+01_rb,0.35525e+01_rb /)
      kbo(:,29,11) = (/ &
        & 0.35424e+01_rb,0.35704e+01_rb,0.35632e+01_rb,0.35824e+01_rb,0.35567e+01_rb /)
      kbo(:,30,11) = (/ &
        & 0.35629e+01_rb,0.35947e+01_rb,0.35992e+01_rb,0.36011e+01_rb,0.36027e+01_rb /)
      kbo(:,31,11) = (/ &
        & 0.35753e+01_rb,0.36045e+01_rb,0.36365e+01_rb,0.36213e+01_rb,0.36310e+01_rb /)
      kbo(:,32,11) = (/ &
        & 0.35596e+01_rb,0.36100e+01_rb,0.36333e+01_rb,0.36490e+01_rb,0.36651e+01_rb /)
      kbo(:,33,11) = (/ &
        & 0.35241e+01_rb,0.35883e+01_rb,0.36383e+01_rb,0.36740e+01_rb,0.36685e+01_rb /)
      kbo(:,34,11) = (/ &
        & 0.35027e+01_rb,0.35491e+01_rb,0.36249e+01_rb,0.36739e+01_rb,0.36810e+01_rb /)
      kbo(:,35,11) = (/ &
        & 0.34699e+01_rb,0.35289e+01_rb,0.35926e+01_rb,0.36193e+01_rb,0.36411e+01_rb /)
      kbo(:,36,11) = (/ &
        & 0.35153e+01_rb,0.35225e+01_rb,0.35498e+01_rb,0.35989e+01_rb,0.36429e+01_rb /)
      kbo(:,37,11) = (/ &
        & 0.35644e+01_rb,0.35488e+01_rb,0.35358e+01_rb,0.35755e+01_rb,0.36247e+01_rb /)
      kbo(:,38,11) = (/ &
        & 0.35952e+01_rb,0.35902e+01_rb,0.35685e+01_rb,0.35598e+01_rb,0.35995e+01_rb /)
      kbo(:,39,11) = (/ &
        & 0.36371e+01_rb,0.36388e+01_rb,0.36159e+01_rb,0.35751e+01_rb,0.35930e+01_rb /)
      kbo(:,40,11) = (/ &
        & 0.36530e+01_rb,0.36611e+01_rb,0.36495e+01_rb,0.35923e+01_rb,0.35985e+01_rb /)
      kbo(:,41,11) = (/ &
        & 0.36431e+01_rb,0.36852e+01_rb,0.36606e+01_rb,0.36324e+01_rb,0.35949e+01_rb /)
      kbo(:,42,11) = (/ &
        & 0.36510e+01_rb,0.36940e+01_rb,0.36905e+01_rb,0.36729e+01_rb,0.36122e+01_rb /)
      kbo(:,43,11) = (/ &
        & 0.36633e+01_rb,0.36955e+01_rb,0.37103e+01_rb,0.36924e+01_rb,0.36526e+01_rb /)
      kbo(:,44,11) = (/ &
        & 0.36584e+01_rb,0.36852e+01_rb,0.37131e+01_rb,0.37219e+01_rb,0.36946e+01_rb /)
      kbo(:,45,11) = (/ &
        & 0.36636e+01_rb,0.36530e+01_rb,0.37160e+01_rb,0.37197e+01_rb,0.37150e+01_rb /)
      kbo(:,46,11) = (/ &
        & 0.36713e+01_rb,0.36779e+01_rb,0.37130e+01_rb,0.37408e+01_rb,0.37431e+01_rb /)
      kbo(:,47,11) = (/ &
        & 0.36885e+01_rb,0.36889e+01_rb,0.37012e+01_rb,0.37377e+01_rb,0.37538e+01_rb /)
      kbo(:,48,11) = (/ &
        & 0.36735e+01_rb,0.37253e+01_rb,0.37092e+01_rb,0.37311e+01_rb,0.37444e+01_rb /)
      kbo(:,49,11) = (/ &
        & 0.36543e+01_rb,0.37300e+01_rb,0.37306e+01_rb,0.37311e+01_rb,0.37688e+01_rb /)
      kbo(:,50,11) = (/ &
        & 0.36397e+01_rb,0.37435e+01_rb,0.37659e+01_rb,0.37491e+01_rb,0.37600e+01_rb /)
      kbo(:,51,11) = (/ &
        & 0.36124e+01_rb,0.37293e+01_rb,0.37963e+01_rb,0.37839e+01_rb,0.37789e+01_rb /)
      kbo(:,52,11) = (/ &
        & 0.35636e+01_rb,0.37078e+01_rb,0.37818e+01_rb,0.38114e+01_rb,0.37775e+01_rb /)
      kbo(:,53,11) = (/ &
        & 0.34978e+01_rb,0.36691e+01_rb,0.37922e+01_rb,0.38152e+01_rb,0.38092e+01_rb /)
      kbo(:,54,11) = (/ &
        & 0.34749e+01_rb,0.36230e+01_rb,0.37686e+01_rb,0.38328e+01_rb,0.38292e+01_rb /)
      kbo(:,55,11) = (/ &
        & 0.34238e+01_rb,0.35883e+01_rb,0.37261e+01_rb,0.38299e+01_rb,0.38488e+01_rb /)
      kbo(:,56,11) = (/ &
        & 0.33568e+01_rb,0.35597e+01_rb,0.37073e+01_rb,0.38247e+01_rb,0.38591e+01_rb /)
      kbo(:,57,11) = (/ &
        & 0.32794e+01_rb,0.35193e+01_rb,0.36712e+01_rb,0.37875e+01_rb,0.38581e+01_rb /)
      kbo(:,58,11) = (/ &
        & 0.31971e+01_rb,0.34544e+01_rb,0.36268e+01_rb,0.37609e+01_rb,0.38393e+01_rb /)
      kbo(:,59,11) = (/ &
        & 0.31680e+01_rb,0.34441e+01_rb,0.36338e+01_rb,0.37596e+01_rb,0.38510e+01_rb /)
      kbo(:,13,12) = (/ &
        & 0.40944e+01_rb,0.40706e+01_rb,0.40738e+01_rb,0.40615e+01_rb,0.40255e+01_rb /)
      kbo(:,14,12) = (/ &
        & 0.42568e+01_rb,0.42640e+01_rb,0.42578e+01_rb,0.42281e+01_rb,0.41677e+01_rb /)
      kbo(:,15,12) = (/ &
        & 0.44471e+01_rb,0.44460e+01_rb,0.44235e+01_rb,0.43647e+01_rb,0.43107e+01_rb /)
      kbo(:,16,12) = (/ &
        & 0.45971e+01_rb,0.45776e+01_rb,0.45277e+01_rb,0.44905e+01_rb,0.45101e+01_rb /)
      kbo(:,17,12) = (/ &
        & 0.46780e+01_rb,0.46358e+01_rb,0.46311e+01_rb,0.46380e+01_rb,0.46103e+01_rb /)
      kbo(:,18,12) = (/ &
        & 0.47294e+01_rb,0.47444e+01_rb,0.47418e+01_rb,0.47299e+01_rb,0.47059e+01_rb /)
      kbo(:,19,12) = (/ &
        & 0.48604e+01_rb,0.48842e+01_rb,0.48819e+01_rb,0.49000e+01_rb,0.48969e+01_rb /)
      kbo(:,20,12) = (/ &
        & 0.49979e+01_rb,0.50271e+01_rb,0.50614e+01_rb,0.50783e+01_rb,0.50637e+01_rb /)
      kbo(:,21,12) = (/ &
        & 0.51216e+01_rb,0.51841e+01_rb,0.52155e+01_rb,0.52098e+01_rb,0.51706e+01_rb /)
      kbo(:,22,12) = (/ &
        & 0.52796e+01_rb,0.53177e+01_rb,0.53275e+01_rb,0.53003e+01_rb,0.52540e+01_rb /)
      kbo(:,23,12) = (/ &
        & 0.54021e+01_rb,0.54214e+01_rb,0.54107e+01_rb,0.53770e+01_rb,0.53308e+01_rb /)
      kbo(:,24,12) = (/ &
        & 0.54951e+01_rb,0.54954e+01_rb,0.54655e+01_rb,0.54275e+01_rb,0.53548e+01_rb /)
      kbo(:,25,12) = (/ &
        & 0.55606e+01_rb,0.55401e+01_rb,0.55126e+01_rb,0.54611e+01_rb,0.53900e+01_rb /)
      kbo(:,26,12) = (/ &
        & 0.56084e+01_rb,0.55775e+01_rb,0.55344e+01_rb,0.54935e+01_rb,0.53765e+01_rb /)
      kbo(:,27,12) = (/ &
        & 0.56424e+01_rb,0.55975e+01_rb,0.55674e+01_rb,0.55057e+01_rb,0.54137e+01_rb /)
      kbo(:,28,12) = (/ &
        & 0.56582e+01_rb,0.56350e+01_rb,0.55811e+01_rb,0.55009e+01_rb,0.54160e+01_rb /)
      kbo(:,29,12) = (/ &
        & 0.56833e+01_rb,0.56401e+01_rb,0.55975e+01_rb,0.55109e+01_rb,0.54249e+01_rb /)
      kbo(:,30,12) = (/ &
        & 0.57003e+01_rb,0.56620e+01_rb,0.55904e+01_rb,0.55121e+01_rb,0.54542e+01_rb /)
      kbo(:,31,12) = (/ &
        & 0.57132e+01_rb,0.56848e+01_rb,0.56091e+01_rb,0.55447e+01_rb,0.55236e+01_rb /)
      kbo(:,32,12) = (/ &
        & 0.57800e+01_rb,0.57027e+01_rb,0.56220e+01_rb,0.56024e+01_rb,0.55523e+01_rb /)
      kbo(:,33,12) = (/ &
        & 0.57668e+01_rb,0.57069e+01_rb,0.56826e+01_rb,0.56406e+01_rb,0.56224e+01_rb /)
      kbo(:,34,12) = (/ &
        & 0.57877e+01_rb,0.57628e+01_rb,0.57465e+01_rb,0.57131e+01_rb,0.56752e+01_rb /)
      kbo(:,35,12) = (/ &
        & 0.58053e+01_rb,0.58032e+01_rb,0.57814e+01_rb,0.58008e+01_rb,0.57412e+01_rb /)
      kbo(:,36,12) = (/ &
        & 0.57386e+01_rb,0.58034e+01_rb,0.57934e+01_rb,0.57973e+01_rb,0.57760e+01_rb /)
      kbo(:,37,12) = (/ &
        & 0.56650e+01_rb,0.57400e+01_rb,0.57795e+01_rb,0.58096e+01_rb,0.57723e+01_rb /)
      kbo(:,38,12) = (/ &
        & 0.56068e+01_rb,0.56512e+01_rb,0.57137e+01_rb,0.58154e+01_rb,0.57795e+01_rb /)
      kbo(:,39,12) = (/ &
        & 0.55712e+01_rb,0.56156e+01_rb,0.56456e+01_rb,0.57952e+01_rb,0.57768e+01_rb /)
      kbo(:,40,12) = (/ &
        & 0.55331e+01_rb,0.55774e+01_rb,0.56049e+01_rb,0.57462e+01_rb,0.57382e+01_rb /)
      kbo(:,41,12) = (/ &
        & 0.55579e+01_rb,0.55330e+01_rb,0.55754e+01_rb,0.56805e+01_rb,0.57245e+01_rb /)
      kbo(:,42,12) = (/ &
        & 0.55684e+01_rb,0.55372e+01_rb,0.55455e+01_rb,0.56019e+01_rb,0.57291e+01_rb /)
      kbo(:,43,12) = (/ &
        & 0.56282e+01_rb,0.55547e+01_rb,0.55863e+01_rb,0.55852e+01_rb,0.56892e+01_rb /)
      kbo(:,44,12) = (/ &
        & 0.56429e+01_rb,0.55998e+01_rb,0.55663e+01_rb,0.55662e+01_rb,0.56522e+01_rb /)
      kbo(:,45,12) = (/ &
        & 0.56488e+01_rb,0.56642e+01_rb,0.55915e+01_rb,0.56136e+01_rb,0.56176e+01_rb /)
      kbo(:,46,12) = (/ &
        & 0.57246e+01_rb,0.57165e+01_rb,0.56557e+01_rb,0.56271e+01_rb,0.56227e+01_rb /)
      kbo(:,47,12) = (/ &
        & 0.57406e+01_rb,0.57550e+01_rb,0.57336e+01_rb,0.56616e+01_rb,0.56928e+01_rb /)
      kbo(:,48,12) = (/ &
        & 0.57616e+01_rb,0.57715e+01_rb,0.57765e+01_rb,0.57429e+01_rb,0.57151e+01_rb /)
      kbo(:,49,12) = (/ &
        & 0.58013e+01_rb,0.57807e+01_rb,0.58014e+01_rb,0.57796e+01_rb,0.57022e+01_rb /)
      kbo(:,50,12) = (/ &
        & 0.58611e+01_rb,0.58307e+01_rb,0.58954e+01_rb,0.58904e+01_rb,0.58254e+01_rb /)
      kbo(:,51,12) = (/ &
        & 0.59182e+01_rb,0.59013e+01_rb,0.59353e+01_rb,0.59746e+01_rb,0.59215e+01_rb /)
      kbo(:,52,12) = (/ &
        & 0.59636e+01_rb,0.59631e+01_rb,0.59638e+01_rb,0.60147e+01_rb,0.60288e+01_rb /)
      kbo(:,53,12) = (/ &
        & 0.60261e+01_rb,0.59972e+01_rb,0.59771e+01_rb,0.60154e+01_rb,0.60632e+01_rb /)
      kbo(:,54,12) = (/ &
        & 0.60204e+01_rb,0.60805e+01_rb,0.60002e+01_rb,0.60580e+01_rb,0.60854e+01_rb /)
      kbo(:,55,12) = (/ &
        & 0.60584e+01_rb,0.61138e+01_rb,0.60917e+01_rb,0.60710e+01_rb,0.61237e+01_rb /)
      kbo(:,56,12) = (/ &
        & 0.60402e+01_rb,0.61059e+01_rb,0.61154e+01_rb,0.61008e+01_rb,0.61463e+01_rb /)
      kbo(:,57,12) = (/ &
        & 0.60080e+01_rb,0.61076e+01_rb,0.61306e+01_rb,0.61548e+01_rb,0.61892e+01_rb /)
      kbo(:,58,12) = (/ &
        & 0.59781e+01_rb,0.61168e+01_rb,0.61621e+01_rb,0.61680e+01_rb,0.62246e+01_rb /)
      kbo(:,59,12) = (/ &
        & 0.59824e+01_rb,0.60878e+01_rb,0.61825e+01_rb,0.62179e+01_rb,0.62658e+01_rb /)
      kbo(:,13,13) = (/ &
        & 0.62252e+01_rb,0.61003e+01_rb,0.59711e+01_rb,0.58641e+01_rb,0.57820e+01_rb /)
      kbo(:,14,13) = (/ &
        & 0.66429e+01_rb,0.64784e+01_rb,0.63526e+01_rb,0.62330e+01_rb,0.61644e+01_rb /)
      kbo(:,15,13) = (/ &
        & 0.69743e+01_rb,0.68232e+01_rb,0.67077e+01_rb,0.66230e+01_rb,0.65438e+01_rb /)
      kbo(:,16,13) = (/ &
        & 0.72971e+01_rb,0.72050e+01_rb,0.71016e+01_rb,0.69747e+01_rb,0.67726e+01_rb /)
      kbo(:,17,13) = (/ &
        & 0.77109e+01_rb,0.75892e+01_rb,0.74253e+01_rb,0.72127e+01_rb,0.70341e+01_rb /)
      kbo(:,18,13) = (/ &
        & 0.80450e+01_rb,0.78275e+01_rb,0.75988e+01_rb,0.74227e+01_rb,0.73977e+01_rb /)
      kbo(:,19,13) = (/ &
        & 0.81727e+01_rb,0.79248e+01_rb,0.78355e+01_rb,0.77125e+01_rb,0.75391e+01_rb /)
      kbo(:,20,13) = (/ &
        & 0.82796e+01_rb,0.81265e+01_rb,0.80271e+01_rb,0.78599e+01_rb,0.77204e+01_rb /)
      kbo(:,21,13) = (/ &
        & 0.84685e+01_rb,0.83415e+01_rb,0.82106e+01_rb,0.80317e+01_rb,0.78896e+01_rb /)
      kbo(:,22,13) = (/ &
        & 0.86169e+01_rb,0.85255e+01_rb,0.83779e+01_rb,0.82528e+01_rb,0.81573e+01_rb /)
      kbo(:,23,13) = (/ &
        & 0.88276e+01_rb,0.86892e+01_rb,0.86001e+01_rb,0.84644e+01_rb,0.83125e+01_rb /)
      kbo(:,24,13) = (/ &
        & 0.89834e+01_rb,0.88963e+01_rb,0.87866e+01_rb,0.86675e+01_rb,0.84751e+01_rb /)
      kbo(:,25,13) = (/ &
        & 0.91971e+01_rb,0.90977e+01_rb,0.89941e+01_rb,0.88254e+01_rb,0.85750e+01_rb /)
      kbo(:,26,13) = (/ &
        & 0.93611e+01_rb,0.92762e+01_rb,0.91306e+01_rb,0.88795e+01_rb,0.86828e+01_rb /)
      kbo(:,27,13) = (/ &
        & 0.95231e+01_rb,0.94096e+01_rb,0.91709e+01_rb,0.89786e+01_rb,0.86968e+01_rb /)
      kbo(:,28,13) = (/ &
        & 0.96495e+01_rb,0.94564e+01_rb,0.92347e+01_rb,0.89785e+01_rb,0.86180e+01_rb /)
      kbo(:,29,13) = (/ &
        & 0.97336e+01_rb,0.94935e+01_rb,0.92561e+01_rb,0.89137e+01_rb,0.86661e+01_rb /)
      kbo(:,30,13) = (/ &
        & 0.97605e+01_rb,0.95111e+01_rb,0.92200e+01_rb,0.89355e+01_rb,0.85920e+01_rb /)
      kbo(:,31,13) = (/ &
        & 0.97562e+01_rb,0.95167e+01_rb,0.91771e+01_rb,0.88618e+01_rb,0.85414e+01_rb /)
      kbo(:,32,13) = (/ &
        & 0.97192e+01_rb,0.94434e+01_rb,0.91403e+01_rb,0.87870e+01_rb,0.85159e+01_rb /)
      kbo(:,33,13) = (/ &
        & 0.97162e+01_rb,0.94370e+01_rb,0.90971e+01_rb,0.88239e+01_rb,0.85528e+01_rb /)
      kbo(:,34,13) = (/ &
        & 0.96997e+01_rb,0.93448e+01_rb,0.90783e+01_rb,0.87958e+01_rb,0.87225e+01_rb /)
      kbo(:,35,13) = (/ &
        & 0.97261e+01_rb,0.93769e+01_rb,0.90457e+01_rb,0.89061e+01_rb,0.87695e+01_rb /)
      kbo(:,36,13) = (/ &
        & 0.97573e+01_rb,0.94639e+01_rb,0.92105e+01_rb,0.90277e+01_rb,0.88187e+01_rb /)
      kbo(:,37,13) = (/ &
        & 0.98297e+01_rb,0.95424e+01_rb,0.93169e+01_rb,0.90796e+01_rb,0.89104e+01_rb /)
      kbo(:,38,13) = (/ &
        & 0.99015e+01_rb,0.96432e+01_rb,0.94039e+01_rb,0.91616e+01_rb,0.89941e+01_rb /)
      kbo(:,39,13) = (/ &
        & 0.99137e+01_rb,0.96714e+01_rb,0.94790e+01_rb,0.92092e+01_rb,0.90352e+01_rb /)
      kbo(:,40,13) = (/ &
        & 0.99728e+01_rb,0.96872e+01_rb,0.94263e+01_rb,0.92759e+01_rb,0.90690e+01_rb /)
      kbo(:,41,13) = (/ &
        & 0.10009e+02_rb,0.96720e+01_rb,0.94529e+01_rb,0.92520e+01_rb,0.90775e+01_rb /)
      kbo(:,42,13) = (/ &
        & 0.99394e+01_rb,0.96947e+01_rb,0.94735e+01_rb,0.92670e+01_rb,0.90312e+01_rb /)
      kbo(:,43,13) = (/ &
        & 0.98412e+01_rb,0.96685e+01_rb,0.93802e+01_rb,0.91747e+01_rb,0.90019e+01_rb /)
      kbo(:,44,13) = (/ &
        & 0.98284e+01_rb,0.96419e+01_rb,0.93576e+01_rb,0.91166e+01_rb,0.88968e+01_rb /)
      kbo(:,45,13) = (/ &
        & 0.98765e+01_rb,0.96737e+01_rb,0.94176e+01_rb,0.90811e+01_rb,0.88300e+01_rb /)
      kbo(:,46,13) = (/ &
        & 0.99370e+01_rb,0.96739e+01_rb,0.94534e+01_rb,0.91692e+01_rb,0.88574e+01_rb /)
      kbo(:,47,13) = (/ &
        & 0.10145e+02_rb,0.99712e+01_rb,0.96675e+01_rb,0.93530e+01_rb,0.89895e+01_rb /)
      kbo(:,48,13) = (/ &
        & 0.10395e+02_rb,0.10167e+02_rb,0.99330e+01_rb,0.95943e+01_rb,0.92927e+01_rb /)
      kbo(:,49,13) = (/ &
        & 0.10580e+02_rb,0.10365e+02_rb,0.10114e+02_rb,0.98197e+01_rb,0.95303e+01_rb /)
      kbo(:,50,13) = (/ &
        & 0.10794e+02_rb,0.10715e+02_rb,0.10421e+02_rb,0.10154e+02_rb,0.98629e+01_rb /)
      kbo(:,51,13) = (/ &
        & 0.10946e+02_rb,0.10939e+02_rb,0.10689e+02_rb,0.10427e+02_rb,0.10145e+02_rb /)
      kbo(:,52,13) = (/ &
        & 0.11156e+02_rb,0.11126e+02_rb,0.10973e+02_rb,0.10622e+02_rb,0.10371e+02_rb /)
      kbo(:,53,13) = (/ &
        & 0.11165e+02_rb,0.11308e+02_rb,0.11212e+02_rb,0.10985e+02_rb,0.10624e+02_rb /)
      kbo(:,54,13) = (/ &
        & 0.11221e+02_rb,0.11354e+02_rb,0.11431e+02_rb,0.11243e+02_rb,0.10959e+02_rb /)
      kbo(:,55,13) = (/ &
        & 0.11238e+02_rb,0.11406e+02_rb,0.11591e+02_rb,0.11479e+02_rb,0.11275e+02_rb /)
      kbo(:,56,13) = (/ &
        & 0.11335e+02_rb,0.11405e+02_rb,0.11662e+02_rb,0.11637e+02_rb,0.11480e+02_rb /)
      kbo(:,57,13) = (/ &
        & 0.11355e+02_rb,0.11451e+02_rb,0.11592e+02_rb,0.11670e+02_rb,0.11515e+02_rb /)
      kbo(:,58,13) = (/ &
        & 0.11389e+02_rb,0.11476e+02_rb,0.11586e+02_rb,0.11657e+02_rb,0.11618e+02_rb /)
      kbo(:,59,13) = (/ &
        & 0.11480e+02_rb,0.11599e+02_rb,0.11673e+02_rb,0.11677e+02_rb,0.11653e+02_rb /)
      kbo(:,13,14) = (/ &
        & 0.97590e+01_rb,0.94275e+01_rb,0.91020e+01_rb,0.89149e+01_rb,0.87289e+01_rb /)
      kbo(:,14,14) = (/ &
        & 0.10505e+02_rb,0.10164e+02_rb,0.99232e+01_rb,0.97321e+01_rb,0.94201e+01_rb /)
      kbo(:,15,14) = (/ &
        & 0.11237e+02_rb,0.10955e+02_rb,0.10714e+02_rb,0.10370e+02_rb,0.10019e+02_rb /)
      kbo(:,16,14) = (/ &
        & 0.12124e+02_rb,0.11740e+02_rb,0.11352e+02_rb,0.10992e+02_rb,0.10632e+02_rb /)
      kbo(:,17,14) = (/ &
        & 0.12886e+02_rb,0.12429e+02_rb,0.12001e+02_rb,0.11642e+02_rb,0.11273e+02_rb /)
      kbo(:,18,14) = (/ &
        & 0.13556e+02_rb,0.13047e+02_rb,0.12733e+02_rb,0.12297e+02_rb,0.11616e+02_rb /)
      kbo(:,19,14) = (/ &
        & 0.14349e+02_rb,0.13901e+02_rb,0.13193e+02_rb,0.12562e+02_rb,0.12015e+02_rb /)
      kbo(:,20,14) = (/ &
        & 0.15084e+02_rb,0.14361e+02_rb,0.13567e+02_rb,0.12903e+02_rb,0.12393e+02_rb /)
      kbo(:,21,14) = (/ &
        & 0.15509e+02_rb,0.14641e+02_rb,0.13824e+02_rb,0.13136e+02_rb,0.12832e+02_rb /)
      kbo(:,22,14) = (/ &
        & 0.15737e+02_rb,0.14756e+02_rb,0.13944e+02_rb,0.13493e+02_rb,0.13014e+02_rb /)
      kbo(:,23,14) = (/ &
        & 0.15733e+02_rb,0.14806e+02_rb,0.14130e+02_rb,0.13708e+02_rb,0.13129e+02_rb /)
      kbo(:,24,14) = (/ &
        & 0.15735e+02_rb,0.14832e+02_rb,0.14382e+02_rb,0.13702e+02_rb,0.13090e+02_rb /)
      kbo(:,25,14) = (/ &
        & 0.15596e+02_rb,0.15046e+02_rb,0.14369e+02_rb,0.13676e+02_rb,0.13207e+02_rb /)
      kbo(:,26,14) = (/ &
        & 0.15679e+02_rb,0.15085e+02_rb,0.14382e+02_rb,0.13864e+02_rb,0.13243e+02_rb /)
      kbo(:,27,14) = (/ &
        & 0.15773e+02_rb,0.15051e+02_rb,0.14458e+02_rb,0.13792e+02_rb,0.13309e+02_rb /)
      kbo(:,28,14) = (/ &
        & 0.15791e+02_rb,0.15118e+02_rb,0.14520e+02_rb,0.13908e+02_rb,0.13506e+02_rb /)
      kbo(:,29,14) = (/ &
        & 0.15907e+02_rb,0.15282e+02_rb,0.14696e+02_rb,0.14168e+02_rb,0.13591e+02_rb /)
      kbo(:,30,14) = (/ &
        & 0.15997e+02_rb,0.15426e+02_rb,0.14908e+02_rb,0.14325e+02_rb,0.13655e+02_rb /)
      kbo(:,31,14) = (/ &
        & 0.16254e+02_rb,0.15640e+02_rb,0.15053e+02_rb,0.14410e+02_rb,0.13542e+02_rb /)
      kbo(:,32,14) = (/ &
        & 0.16522e+02_rb,0.15885e+02_rb,0.15194e+02_rb,0.14363e+02_rb,0.13365e+02_rb /)
      kbo(:,33,14) = (/ &
        & 0.16844e+02_rb,0.16068e+02_rb,0.15151e+02_rb,0.14109e+02_rb,0.13003e+02_rb /)
      kbo(:,34,14) = (/ &
        & 0.16915e+02_rb,0.16003e+02_rb,0.14953e+02_rb,0.13814e+02_rb,0.12608e+02_rb /)
      kbo(:,35,14) = (/ &
        & 0.16922e+02_rb,0.15890e+02_rb,0.14756e+02_rb,0.13405e+02_rb,0.12281e+02_rb /)
      kbo(:,36,14) = (/ &
        & 0.16966e+02_rb,0.15773e+02_rb,0.14522e+02_rb,0.13240e+02_rb,0.11917e+02_rb /)
      kbo(:,37,14) = (/ &
        & 0.17142e+02_rb,0.15897e+02_rb,0.14564e+02_rb,0.13378e+02_rb,0.11953e+02_rb /)
      kbo(:,38,14) = (/ &
        & 0.17346e+02_rb,0.16051e+02_rb,0.14752e+02_rb,0.13439e+02_rb,0.11971e+02_rb /)
      kbo(:,39,14) = (/ &
        & 0.17475e+02_rb,0.16210e+02_rb,0.14948e+02_rb,0.13510e+02_rb,0.12056e+02_rb /)
      kbo(:,40,14) = (/ &
        & 0.17754e+02_rb,0.16503e+02_rb,0.15328e+02_rb,0.13801e+02_rb,0.12328e+02_rb /)
      kbo(:,41,14) = (/ &
        & 0.18031e+02_rb,0.16909e+02_rb,0.15619e+02_rb,0.14276e+02_rb,0.12740e+02_rb /)
      kbo(:,42,14) = (/ &
        & 0.18386e+02_rb,0.17235e+02_rb,0.15966e+02_rb,0.14652e+02_rb,0.13194e+02_rb /)
      kbo(:,43,14) = (/ &
        & 0.18710e+02_rb,0.17591e+02_rb,0.16361e+02_rb,0.15070e+02_rb,0.13707e+02_rb /)
      kbo(:,44,14) = (/ &
        & 0.19098e+02_rb,0.17898e+02_rb,0.16845e+02_rb,0.15582e+02_rb,0.14265e+02_rb /)
      kbo(:,45,14) = (/ &
        & 0.19252e+02_rb,0.18167e+02_rb,0.17135e+02_rb,0.15969e+02_rb,0.14652e+02_rb /)
      kbo(:,46,14) = (/ &
        & 0.19041e+02_rb,0.18357e+02_rb,0.17265e+02_rb,0.16200e+02_rb,0.15002e+02_rb /)
      kbo(:,47,14) = (/ &
        & 0.18766e+02_rb,0.18032e+02_rb,0.17152e+02_rb,0.16208e+02_rb,0.15088e+02_rb /)
      kbo(:,48,14) = (/ &
        & 0.18411e+02_rb,0.17740e+02_rb,0.16984e+02_rb,0.16085e+02_rb,0.15089e+02_rb /)
      kbo(:,49,14) = (/ &
        & 0.18291e+02_rb,0.17667e+02_rb,0.16941e+02_rb,0.16014e+02_rb,0.14979e+02_rb /)
      kbo(:,50,14) = (/ &
        & 0.18624e+02_rb,0.17786e+02_rb,0.17174e+02_rb,0.16136e+02_rb,0.15128e+02_rb /)
      kbo(:,51,14) = (/ &
        & 0.19430e+02_rb,0.18470e+02_rb,0.17722e+02_rb,0.16775e+02_rb,0.15711e+02_rb /)
      kbo(:,52,14) = (/ &
        & 0.20123e+02_rb,0.18895e+02_rb,0.18060e+02_rb,0.17255e+02_rb,0.16173e+02_rb /)
      kbo(:,53,14) = (/ &
        & 0.20744e+02_rb,0.19415e+02_rb,0.18626e+02_rb,0.17397e+02_rb,0.16465e+02_rb /)
      kbo(:,54,14) = (/ &
        & 0.21714e+02_rb,0.20093e+02_rb,0.19173e+02_rb,0.18078e+02_rb,0.16884e+02_rb /)
      kbo(:,55,14) = (/ &
        & 0.22731e+02_rb,0.21068e+02_rb,0.19674e+02_rb,0.18569e+02_rb,0.17315e+02_rb /)
      kbo(:,56,14) = (/ &
        & 0.23924e+02_rb,0.22190e+02_rb,0.20317e+02_rb,0.18891e+02_rb,0.17603e+02_rb /)
      kbo(:,57,14) = (/ &
        & 0.24940e+02_rb,0.23270e+02_rb,0.21498e+02_rb,0.19833e+02_rb,0.18265e+02_rb /)
      kbo(:,58,14) = (/ &
        & 0.25828e+02_rb,0.24271e+02_rb,0.22518e+02_rb,0.20827e+02_rb,0.19003e+02_rb /)
      kbo(:,59,14) = (/ &
        & 0.26483e+02_rb,0.24983e+02_rb,0.23453e+02_rb,0.21676e+02_rb,0.19755e+02_rb /)
      kbo(:,13,15) = (/ &
        & 0.15960e+02_rb,0.15303e+02_rb,0.14694e+02_rb,0.13842e+02_rb,0.13008e+02_rb /)
      kbo(:,14,15) = (/ &
        & 0.17675e+02_rb,0.16896e+02_rb,0.15925e+02_rb,0.14915e+02_rb,0.14133e+02_rb /)
      kbo(:,15,15) = (/ &
        & 0.19385e+02_rb,0.18234e+02_rb,0.16959e+02_rb,0.15987e+02_rb,0.15180e+02_rb /)
      kbo(:,16,15) = (/ &
        & 0.20552e+02_rb,0.19165e+02_rb,0.17982e+02_rb,0.16927e+02_rb,0.16368e+02_rb /)
      kbo(:,17,15) = (/ &
        & 0.21349e+02_rb,0.19954e+02_rb,0.19023e+02_rb,0.18091e+02_rb,0.17398e+02_rb /)
      kbo(:,18,15) = (/ &
        & 0.22160e+02_rb,0.21597e+02_rb,0.20213e+02_rb,0.19055e+02_rb,0.18102e+02_rb /)
      kbo(:,19,15) = (/ &
        & 0.23816e+02_rb,0.22303e+02_rb,0.21008e+02_rb,0.19912e+02_rb,0.18903e+02_rb /)
      kbo(:,20,15) = (/ &
        & 0.24810e+02_rb,0.23381e+02_rb,0.21933e+02_rb,0.20846e+02_rb,0.19540e+02_rb /)
      kbo(:,21,15) = (/ &
        & 0.25893e+02_rb,0.24250e+02_rb,0.22750e+02_rb,0.21616e+02_rb,0.19627e+02_rb /)
      kbo(:,22,15) = (/ &
        & 0.26542e+02_rb,0.24773e+02_rb,0.23600e+02_rb,0.21309e+02_rb,0.19278e+02_rb /)
      kbo(:,23,15) = (/ &
        & 0.27270e+02_rb,0.25621e+02_rb,0.23587e+02_rb,0.21255e+02_rb,0.19478e+02_rb /)
      kbo(:,24,15) = (/ &
        & 0.27744e+02_rb,0.26088e+02_rb,0.23254e+02_rb,0.21242e+02_rb,0.19100e+02_rb /)
      kbo(:,25,15) = (/ &
        & 0.28557e+02_rb,0.25575e+02_rb,0.23098e+02_rb,0.20631e+02_rb,0.18206e+02_rb /)
      kbo(:,26,15) = (/ &
        & 0.28140e+02_rb,0.25135e+02_rb,0.22210e+02_rb,0.19496e+02_rb,0.17366e+02_rb /)
      kbo(:,27,15) = (/ &
        & 0.27284e+02_rb,0.24232e+02_rb,0.21249e+02_rb,0.18720e+02_rb,0.16279e+02_rb /)
      kbo(:,28,15) = (/ &
        & 0.26494e+02_rb,0.23076e+02_rb,0.20060e+02_rb,0.17566e+02_rb,0.14956e+02_rb /)
      kbo(:,29,15) = (/ &
        & 0.24773e+02_rb,0.21460e+02_rb,0.18370e+02_rb,0.15793e+02_rb,0.13302e+02_rb /)
      kbo(:,30,15) = (/ &
        & 0.23157e+02_rb,0.19587e+02_rb,0.16563e+02_rb,0.13805e+02_rb,0.11647e+02_rb /)
      kbo(:,31,15) = (/ &
        & 0.20881e+02_rb,0.17370e+02_rb,0.14460e+02_rb,0.11958e+02_rb,0.99808e+01_rb /)
      kbo(:,32,15) = (/ &
        & 0.18247e+02_rb,0.15035e+02_rb,0.12296e+02_rb,0.10101e+02_rb,0.83032e+01_rb /)
      kbo(:,33,15) = (/ &
        & 0.15492e+02_rb,0.12470e+02_rb,0.10235e+02_rb,0.81131e+01_rb,0.64390e+01_rb /)
      kbo(:,34,15) = (/ &
        & 0.13284e+02_rb,0.10782e+02_rb,0.82553e+01_rb,0.64597e+01_rb,0.42402e+01_rb /)
      kbo(:,35,15) = (/ &
        & 0.11520e+02_rb,0.89534e+01_rb,0.69691e+01_rb,0.48753e+01_rb,0.27443e+01_rb /)
      kbo(:,36,15) = (/ &
        & 0.10067e+02_rb,0.75650e+01_rb,0.54572e+01_rb,0.31866e+01_rb,0.14626e+01_rb /)
      kbo(:,37,15) = (/ &
        & 0.97744e+01_rb,0.73767e+01_rb,0.52433e+01_rb,0.27509e+01_rb,0.10380e+01_rb /)
      kbo(:,38,15) = (/ &
        & 0.94733e+01_rb,0.70859e+01_rb,0.47807e+01_rb,0.23954e+01_rb,0.74468e+00_rb /)
      kbo(:,39,15) = (/ &
        & 0.94041e+01_rb,0.67286e+01_rb,0.41966e+01_rb,0.20683e+01_rb,0.27939e+00_rb /)
      kbo(:,40,15) = (/ &
        & 0.10076e+02_rb,0.74414e+01_rb,0.47964e+01_rb,0.25258e+01_rb,0.87689e+00_rb /)
      kbo(:,41,15) = (/ &
        & 0.10878e+02_rb,0.81759e+01_rb,0.56173e+01_rb,0.30416e+01_rb,0.13231e+01_rb /)
      kbo(:,42,15) = (/ &
        & 0.11858e+02_rb,0.88284e+01_rb,0.63009e+01_rb,0.37211e+01_rb,0.17214e+01_rb /)
      kbo(:,43,15) = (/ &
        & 0.13284e+02_rb,0.99847e+01_rb,0.73905e+01_rb,0.50291e+01_rb,0.24940e+01_rb /)
      kbo(:,44,15) = (/ &
        & 0.14796e+02_rb,0.11545e+02_rb,0.85671e+01_rb,0.61687e+01_rb,0.37136e+01_rb /)
      kbo(:,45,15) = (/ &
        & 0.16662e+02_rb,0.12949e+02_rb,0.98033e+01_rb,0.73353e+01_rb,0.52329e+01_rb /)
      kbo(:,46,15) = (/ &
        & 0.19200e+02_rb,0.14684e+02_rb,0.11434e+02_rb,0.85867e+01_rb,0.64428e+01_rb /)
      kbo(:,47,15) = (/ &
        & 0.21780e+02_rb,0.16837e+02_rb,0.13158e+02_rb,0.10165e+02_rb,0.76608e+01_rb /)
      kbo(:,48,15) = (/ &
        & 0.24689e+02_rb,0.19487e+02_rb,0.15145e+02_rb,0.11845e+02_rb,0.89940e+01_rb /)
      kbo(:,49,15) = (/ &
        & 0.27130e+02_rb,0.21775e+02_rb,0.17116e+02_rb,0.13679e+02_rb,0.10905e+02_rb /)
      kbo(:,50,15) = (/ &
        & 0.27854e+02_rb,0.22306e+02_rb,0.17103e+02_rb,0.13783e+02_rb,0.10863e+02_rb /)
      kbo(:,51,15) = (/ &
        & 0.27653e+02_rb,0.21825e+02_rb,0.16692e+02_rb,0.12707e+02_rb,0.98148e+01_rb /)
      kbo(:,52,15) = (/ &
        & 0.27815e+02_rb,0.22411e+02_rb,0.17151e+02_rb,0.12787e+02_rb,0.95823e+01_rb /)
      kbo(:,53,15) = (/ &
        & 0.29119e+02_rb,0.23144e+02_rb,0.17252e+02_rb,0.13594e+02_rb,0.99767e+01_rb /)
      kbo(:,54,15) = (/ &
        & 0.29129e+02_rb,0.23469e+02_rb,0.17342e+02_rb,0.12745e+02_rb,0.95359e+01_rb /)
      kbo(:,55,15) = (/ &
        & 0.29058e+02_rb,0.23190e+02_rb,0.17458e+02_rb,0.12690e+02_rb,0.90099e+01_rb /)
      kbo(:,56,15) = (/ &
        & 0.28657e+02_rb,0.23001e+02_rb,0.17896e+02_rb,0.13433e+02_rb,0.94907e+01_rb /)
      kbo(:,57,15) = (/ &
        & 0.29240e+02_rb,0.22830e+02_rb,0.17734e+02_rb,0.13185e+02_rb,0.97411e+01_rb /)
      kbo(:,58,15) = (/ &
        & 0.29947e+02_rb,0.22946e+02_rb,0.17582e+02_rb,0.13008e+02_rb,0.96489e+01_rb /)
      kbo(:,59,15) = (/ &
        & 0.28976e+02_rb,0.21746e+02_rb,0.15522e+02_rb,0.11254e+02_rb,0.80222e+01_rb /)
      kbo(:,13,16) = (/ &
        & 0.25931e+02_rb,0.24537e+02_rb,0.23224e+02_rb,0.21983e+02_rb,0.20855e+02_rb /)
      kbo(:,14,16) = (/ &
        & 0.29220e+02_rb,0.27507e+02_rb,0.25899e+02_rb,0.24431e+02_rb,0.23088e+02_rb /)
      kbo(:,15,16) = (/ &
        & 0.32471e+02_rb,0.30409e+02_rb,0.28502e+02_rb,0.26794e+02_rb,0.25226e+02_rb /)
      kbo(:,16,16) = (/ &
        & 0.35624e+02_rb,0.33186e+02_rb,0.30989e+02_rb,0.28748e+02_rb,0.24479e+02_rb /)
      kbo(:,17,16) = (/ &
        & 0.38605e+02_rb,0.35797e+02_rb,0.30613e+02_rb,0.25949e+02_rb,0.21171e+02_rb /)
      kbo(:,18,16) = (/ &
        & 0.41350e+02_rb,0.31676e+02_rb,0.25965e+02_rb,0.21818e+02_rb,0.18145e+02_rb /)
      kbo(:,19,16) = (/ &
        & 0.34377e+02_rb,0.28144e+02_rb,0.23342e+02_rb,0.17735e+02_rb,0.13818e+02_rb /)
      kbo(:,20,16) = (/ &
        & 0.31245e+02_rb,0.24408e+02_rb,0.19618e+02_rb,0.13984e+02_rb,0.93171e+01_rb /)
      kbo(:,21,16) = (/ &
        & 0.28216e+02_rb,0.21711e+02_rb,0.16938e+02_rb,0.11225e+02_rb,0.60954e+01_rb /)
      kbo(:,22,16) = (/ &
        & 0.25410e+02_rb,0.20320e+02_rb,0.12451e+02_rb,0.80509e+01_rb,0.41742e+01_rb /)
      kbo(:,23,16) = (/ &
        & 0.22191e+02_rb,0.15284e+02_rb,0.84334e+01_rb,0.32443e+01_rb,0.10236e-04_rb /)
      kbo(:,24,16) = (/ &
        & 0.19170e+02_rb,0.98429e+01_rb,0.40822e+01_rb,0.90070e-05_rb,0.84964e-05_rb /)
      kbo(:,25,16) = (/ &
        & 0.11441e+02_rb,0.50508e+01_rb,0.77316e-05_rb,0.71796e-05_rb,0.66473e-05_rb /)
      kbo(:,26,16) = (/ &
        & 0.73617e+01_rb,0.56953e+00_rb,0.62155e-05_rb,0.57759e-05_rb,0.53447e-05_rb /)
      kbo(:,27,16) = (/ &
        & 0.37038e+01_rb,0.54098e-05_rb,0.49933e-05_rb,0.45180e-05_rb,0.42997e-05_rb /)
      kbo(:,28,16) = (/ &
        & 0.47094e-05_rb,0.43472e-05_rb,0.40149e-05_rb,0.36332e-05_rb,0.34550e-05_rb /)
      kbo(:,29,16) = (/ &
        & 0.37812e-05_rb,0.34885e-05_rb,0.32238e-05_rb,0.29187e-05_rb,0.27779e-05_rb /)
      kbo(:,30,16) = (/ &
        & 0.30339e-05_rb,0.28004e-05_rb,0.25314e-05_rb,0.23451e-05_rb,0.22313e-05_rb /)
      kbo(:,31,16) = (/ &
        & 0.24328e-05_rb,0.22460e-05_rb,0.20326e-05_rb,0.18813e-05_rb,0.17930e-05_rb /)
      kbo(:,32,16) = (/ &
        & 0.18939e-05_rb,0.18012e-05_rb,0.16316e-05_rb,0.15109e-05_rb,0.14395e-05_rb /)
      kbo(:,33,16) = (/ &
        & 0.15657e-05_rb,0.14309e-05_rb,0.13083e-05_rb,0.12124e-05_rb,0.15795e-05_rb /)
      kbo(:,34,16) = (/ &
        & 0.12582e-05_rb,0.11508e-05_rb,0.10527e-05_rb,0.97625e-06_rb,0.19757e-05_rb /)
      kbo(:,35,16) = (/ &
        & 0.10197e-05_rb,0.93275e-06_rb,0.85375e-06_rb,0.79153e-06_rb,0.21668e-05_rb /)
      kbo(:,36,16) = (/ &
        & 0.83336e-06_rb,0.76274e-06_rb,0.69805e-06_rb,0.64735e-06_rb,0.17739e-05_rb /)
      kbo(:,37,16) = (/ &
        & 0.68945e-06_rb,0.63022e-06_rb,0.57702e-06_rb,0.53486e-06_rb,0.10843e-05_rb /)
      kbo(:,38,16) = (/ &
        & 0.55406e-06_rb,0.52180e-06_rb,0.47760e-06_rb,0.44282e-06_rb,0.89395e-06_rb /)
      kbo(:,39,16) = (/ &
        & 0.45861e-06_rb,0.43223e-06_rb,0.39548e-06_rb,0.36639e-06_rb,0.47718e-06_rb /)
      kbo(:,40,16) = (/ &
        & 0.38297e-06_rb,0.36441e-06_rb,0.32989e-06_rb,0.30551e-06_rb,0.28352e-06_rb /)
      kbo(:,41,16) = (/ &
        & 0.31987e-06_rb,0.30430e-06_rb,0.27838e-06_rb,0.25513e-06_rb,0.23647e-06_rb /)
      kbo(:,42,16) = (/ &
        & 0.26734e-06_rb,0.24677e-06_rb,0.23264e-06_rb,0.21290e-06_rb,0.19743e-06_rb /)
      kbo(:,43,16) = (/ &
        & 0.22458e-06_rb,0.20712e-06_rb,0.19517e-06_rb,0.17853e-06_rb,0.16551e-06_rb /)
      kbo(:,44,16) = (/ &
        & 0.18898e-06_rb,0.17429e-06_rb,0.16587e-06_rb,0.15173e-06_rb,0.13895e-06_rb /)
      kbo(:,45,16) = (/ &
        & 0.15450e-06_rb,0.14658e-06_rb,0.13533e-06_rb,0.12759e-06_rb,0.11679e-06_rb /)
      kbo(:,46,16) = (/ &
        & 0.13028e-06_rb,0.12362e-06_rb,0.11400e-06_rb,0.10744e-06_rb,0.98279e-07_rb /)
      kbo(:,47,16) = (/ &
        & 0.11024e-06_rb,0.10159e-06_rb,0.96435e-07_rb,0.91711e-07_rb,0.83972e-07_rb /)
      kbo(:,48,16) = (/ &
        & 0.93198e-07_rb,0.85906e-07_rb,0.81590e-07_rb,0.75279e-07_rb,0.70931e-07_rb /)
      kbo(:,49,16) = (/ &
        & 0.76236e-07_rb,0.72626e-07_rb,0.68995e-07_rb,0.63641e-07_rb,0.60530e-07_rb /)
      kbo(:,50,16) = (/ &
        & 0.64328e-07_rb,0.61369e-07_rb,0.56594e-07_rb,0.53672e-07_rb,0.49555e-07_rb /)
      kbo(:,51,16) = (/ &
        & 0.54169e-07_rb,0.51722e-07_rb,0.47685e-07_rb,0.45298e-07_rb,0.41770e-07_rb /)
      kbo(:,52,16) = (/ &
        & 0.46180e-07_rb,0.42221e-07_rb,0.40251e-07_rb,0.37120e-07_rb,0.35204e-07_rb /)
      kbo(:,53,16) = (/ &
        & 0.33194e-01_rb,0.35538e-07_rb,0.33930e-07_rb,0.31297e-07_rb,0.29712e-07_rb /)
      kbo(:,54,16) = (/ &
        & 0.63212e-01_rb,0.54132e-01_rb,0.36437e-01_rb,0.71353e-02_rb,0.24978e-07_rb /)
      kbo(:,55,16) = (/ &
        & 0.62321e-01_rb,0.69163e-01_rb,0.76899e-01_rb,0.64692e-01_rb,0.42869e-01_rb /)
      kbo(:,56,16) = (/ &
        & 0.73050e-01_rb,0.83118e-01_rb,0.74962e-01_rb,0.82540e-01_rb,0.91058e-01_rb /)
      kbo(:,57,16) = (/ &
        & 0.71162e-01_rb,0.79898e-01_rb,0.89953e-01_rb,0.10014e+00_rb,0.10382e+00_rb /)
      kbo(:,58,16) = (/ &
        & 0.67770e-01_rb,0.78960e-01_rb,0.86882e-01_rb,0.97002e-01_rb,0.10740e+00_rb /)
      kbo(:,59,16) = (/ &
        & 0.66421e-01_rb,0.77608e-01_rb,0.10426e+00_rb,0.10063e+00_rb,0.10735e+00_rb /)
  
!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.860560e-06_rb, 0.130439e-05_rb, 0.382378e-05_rb /)
      forrefo(:, 2) = (/ 0.817926e-06_rb, 0.158599e-05_rb, 0.658771e-04_rb /)
      forrefo(:, 3) = (/ 0.129369e-05_rb, 0.824406e-05_rb, 0.952778e-04_rb /)
      forrefo(:, 4) = (/ 0.438918e-05_rb, 0.375356e-04_rb, 0.119111e-03_rb /)
      forrefo(:, 5) = (/ 0.306057e-04_rb, 0.622798e-04_rb, 0.100740e-03_rb /)
      forrefo(:, 6) = (/ 0.891934e-04_rb, 0.856393e-04_rb, 0.635583e-04_rb /)
      forrefo(:, 7) = (/ 0.171959e-03_rb, 0.173431e-03_rb, 0.611721e-04_rb /)
      forrefo(:, 8) = (/ 0.357795e-03_rb, 0.247261e-03_rb, 0.488864e-04_rb /)
      forrefo(:, 9) = (/ 0.326623e-03_rb, 0.289471e-03_rb, 0.548834e-04_rb /)
      forrefo(:,10) = (/ 0.345103e-03_rb, 0.320898e-03_rb, 0.633214e-04_rb /)
      forrefo(:,11) = (/ 0.392567e-03_rb, 0.325153e-03_rb, 0.744479e-04_rb /)
      forrefo(:,12) = (/ 0.349277e-03_rb, 0.345610e-03_rb, 0.916479e-04_rb /)
      forrefo(:,13) = (/ 0.425161e-03_rb, 0.348452e-03_rb, 0.125788e-03_rb /)
      forrefo(:,14) = (/ 0.407594e-03_rb, 0.435836e-03_rb, 0.287583e-03_rb /)
      forrefo(:,15) = (/ 0.521605e-03_rb, 0.486596e-03_rb, 0.483511e-03_rb /)
      forrefo(:,16) = (/ 0.773790e-03_rb, 0.737247e-03_rb, 0.665939e-03_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).
           
      selfrefo(:, 1) = (/ &
        &  0.750370e-03_rb, 0.644938e-03_rb, 0.554321e-03_rb, 0.476436e-03_rb, 0.409494e-03_rb, &
        &  0.351957e-03_rb, 0.302505e-03_rb, 0.260002e-03_rb, 0.223470e-03_rb, 0.192071e-03_rb /)
      selfrefo(:, 2) = (/ &
        &  0.136135e-02_rb, 0.113187e-02_rb, 0.941076e-03_rb, 0.782440e-03_rb, 0.650546e-03_rb, &
        &  0.540885e-03_rb, 0.449709e-03_rb, 0.373902e-03_rb, 0.310874e-03_rb, 0.258471e-03_rb /)
      selfrefo(:, 3) = (/ &
        &  0.333950e-02_rb, 0.256391e-02_rb, 0.196845e-02_rb, 0.151129e-02_rb, 0.116030e-02_rb, &
        &  0.890824e-03_rb, 0.683934e-03_rb, 0.525093e-03_rb, 0.403143e-03_rb, 0.309515e-03_rb /)
      selfrefo(:, 4) = (/ &
        &  0.793392e-02_rb, 0.589865e-02_rb, 0.438548e-02_rb, 0.326048e-02_rb, 0.242408e-02_rb, &
        &  0.180223e-02_rb, 0.133991e-02_rb, 0.996186e-03_rb, 0.740636e-03_rb, 0.550642e-03_rb /)
      selfrefo(:, 5) = (/ &
        &  0.828169e-02_rb, 0.703139e-02_rb, 0.596984e-02_rb, 0.506856e-02_rb, 0.430335e-02_rb, &
        &  0.365366e-02_rb, 0.310206e-02_rb, 0.263374e-02_rb, 0.223612e-02_rb, 0.189852e-02_rb /)
      selfrefo(:, 6) = (/ &
        &  0.834190e-02_rb, 0.780225e-02_rb, 0.729750e-02_rb, 0.682541e-02_rb, 0.638386e-02_rb, &
        &  0.597087e-02_rb, 0.558460e-02_rb, 0.522332e-02_rb, 0.488541e-02_rb, 0.456936e-02_rb /)
      selfrefo(:, 7) = (/ &
        &  0.119082e-01_rb, 0.112566e-01_rb, 0.106406e-01_rb, 0.100583e-01_rb, 0.950785e-02_rb, &
        &  0.898755e-02_rb, 0.849571e-02_rb, 0.803080e-02_rb, 0.759132e-02_rb, 0.717590e-02_rb /)
      selfrefo(:, 8) = (/ &
        &  0.144004e-01_rb, 0.141762e-01_rb, 0.139554e-01_rb, 0.137381e-01_rb, 0.135241e-01_rb, &
        &  0.133135e-01_rb, 0.131062e-01_rb, 0.129021e-01_rb, 0.127011e-01_rb, 0.125033e-01_rb /)
      selfrefo(:, 9) = (/ &
        &  0.186171e-01_rb, 0.175281e-01_rb, 0.165027e-01_rb, 0.155373e-01_rb, 0.146284e-01_rb, &
        &  0.137726e-01_rb, 0.129670e-01_rb, 0.122084e-01_rb, 0.114942e-01_rb, 0.108218e-01_rb /)
      selfrefo(:,10) = (/ &
        &  0.209396e-01_rb, 0.195077e-01_rb, 0.181737e-01_rb, 0.169309e-01_rb, 0.157731e-01_rb, &
        &  0.146945e-01_rb, 0.136897e-01_rb, 0.127535e-01_rb, 0.118814e-01_rb, 0.110689e-01_rb /)
      selfrefo(:,11) = (/ &
        &  0.203661e-01_rb, 0.193311e-01_rb, 0.183487e-01_rb, 0.174163e-01_rb, 0.165312e-01_rb, &
        &  0.156911e-01_rb, 0.148937e-01_rb, 0.141368e-01_rb, 0.134184e-01_rb, 0.127365e-01_rb /)
      selfrefo(:,12) = (/ &
        &  0.226784e-01_rb, 0.210210e-01_rb, 0.194848e-01_rb, 0.180608e-01_rb, 0.167409e-01_rb, &
        &  0.155174e-01_rb, 0.143834e-01_rb, 0.133322e-01_rb, 0.123579e-01_rb, 0.114547e-01_rb /)
      selfrefo(:,13) = (/ &
        &  0.221773e-01_rb, 0.210306e-01_rb, 0.199433e-01_rb, 0.189122e-01_rb, 0.179344e-01_rb, &
        &  0.170071e-01_rb, 0.161278e-01_rb, 0.152939e-01_rb, 0.145032e-01_rb, 0.137533e-01_rb /)
      selfrefo(:,14) = (/ &
        &  0.275920e-01_rb, 0.252595e-01_rb, 0.231241e-01_rb, 0.211693e-01_rb, 0.193797e-01_rb, &
        &  0.177415e-01_rb, 0.162417e-01_rb, 0.148687e-01_rb, 0.136117e-01_rb, 0.124610e-01_rb /)
      selfrefo(:,15) = (/ &
        &  0.288687e-01_rb, 0.269968e-01_rb, 0.252462e-01_rb, 0.236092e-01_rb, 0.220783e-01_rb, &
        &  0.206466e-01_rb, 0.193078e-01_rb, 0.180559e-01_rb, 0.168851e-01_rb, 0.157902e-01_rb /)
      selfrefo(:,16) = (/ &
        &  0.371842e-01_rb, 0.347595e-01_rb, 0.324929e-01_rb, 0.303741e-01_rb, 0.283934e-01_rb, &
        &  0.265419e-01_rb, 0.248112e-01_rb, 0.231933e-01_rb, 0.216809e-01_rb, 0.202671e-01_rb /)
  
      end subroutine sw_kgb18

! **************************************************************************
      subroutine sw_kgb19
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg19, only : kao, kbo, selfrefo, forrefo, sfluxrefo, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:,1) = (/ &
        & 3.25791_rb    , 3.29697_rb    , 3.16031_rb    , 2.96115_rb    , &
        & 2.69238_rb    , 2.33819_rb    , 1.92760_rb    , 1.44918_rb    , &
        & 0.979764_rb   , 0.107336_rb   , 8.94523e-02_rb, 6.98325e-02_rb, &
        & 5.12051e-02_rb, 3.23645e-02_rb, 1.23401e-02_rb, 1.71339e-03_rb /)
      sfluxrefo(:,2) = (/ &
        & 3.22769_rb    , 3.28817_rb    , 3.16687_rb    , 2.97662_rb    , &
        & 2.69495_rb    , 2.34392_rb    , 1.92900_rb    , 1.45391_rb    , &
        & 0.982522_rb   , 0.107638_rb   , 8.92458e-02_rb, 6.99885e-02_rb, &
        & 5.09679e-02_rb, 3.23789e-02_rb, 1.22673e-02_rb, 1.56040e-03_rb /)
      sfluxrefo(:,3) = (/ &
        & 3.22294_rb    , 3.27780_rb    , 3.17424_rb    , 2.97143_rb    , &
        & 2.69785_rb    , 2.34993_rb    , 1.93155_rb    , 1.45196_rb    , &
        & 0.985329_rb   , 0.108027_rb   , 8.93552e-02_rb, 6.99937e-02_rb, &
        & 5.11678e-02_rb, 3.24846e-02_rb, 1.20636e-02_rb, 1.56040e-03_rb /)
      sfluxrefo(:,4) = (/ &
        & 3.22445_rb    , 3.26113_rb    , 3.18438_rb    , 2.96921_rb    , &
        & 2.69579_rb    , 2.35586_rb    , 1.93454_rb    , 1.44949_rb    , &
        & 0.987347_rb   , 0.108611_rb   , 8.91643e-02_rb, 7.02236e-02_rb, &
        & 5.12980e-02_rb, 3.25282e-02_rb, 1.21189e-02_rb, 1.56040e-03_rb /)
      sfluxrefo(:,5) = (/ &
        & 3.22497_rb    , 3.25109_rb    , 3.18741_rb    , 2.96970_rb    , &
        & 2.69460_rb    , 2.36020_rb    , 1.93301_rb    , 1.45224_rb    , &
        & 0.988564_rb   , 0.108255_rb   , 8.93830e-02_rb, 7.03655e-02_rb, &
        & 5.13017e-02_rb, 3.29414e-02_rb, 1.21189e-02_rb, 1.56040e-03_rb /)
      sfluxrefo(:,6) = (/ &
        & 3.22632_rb    , 3.24174_rb    , 3.18524_rb    , 2.97402_rb    , &
        & 2.69807_rb    , 2.35742_rb    , 1.93377_rb    , 1.45621_rb    , &
        & 0.988132_rb   , 0.108344_rb   , 8.93188e-02_rb, 7.04907e-02_rb, &
        & 5.17938e-02_rb, 3.31465e-02_rb, 1.21155e-02_rb, 1.56040e-03_rb /)
      sfluxrefo(:,7) = (/ &
        & 3.22793_rb    , 3.23589_rb    , 3.17720_rb    , 2.97869_rb    , &
        & 2.70293_rb    , 2.35436_rb    , 1.93557_rb    , 1.45868_rb    , &
        & 0.988654_rb   , 0.108198_rb   , 8.93375e-02_rb, 7.09790e-02_rb, &
        & 5.24733e-02_rb, 3.31298e-02_rb, 1.21126e-02_rb, 1.56040e-03_rb /)
      sfluxrefo(:,8) = (/ &
        & 3.22966_rb    , 3.24087_rb    , 3.15676_rb    , 2.98171_rb    , &
        & 2.70894_rb    , 2.34975_rb    , 1.93855_rb    , 1.46354_rb    , &
        & 0.988544_rb   , 0.108574_rb   , 9.02522e-02_rb, 7.12908e-02_rb, &
        & 5.24844e-02_rb, 3.31084e-02_rb, 1.21060e-02_rb, 1.56040e-03_rb /)
      sfluxrefo(:,9) = (/ &
        & 3.27240_rb    , 3.24666_rb    , 3.13886_rb    , 2.95238_rb    , &
        & 2.70190_rb    , 2.34460_rb    , 1.93948_rb    , 1.47111_rb    , &
        & 0.990821_rb   , 0.108730_rb   , 9.01625e-02_rb, 7.13261e-02_rb, &
        & 5.24813e-02_rb, 3.31083e-02_rb, 1.21126e-02_rb, 1.56040e-03_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:,1) = (/ &
        & 3.29522E+00_rb, 3.24664E+00_rb, 3.15928E+00_rb, 2.95954E+00_rb,&
        & 2.70012E+00_rb, 2.35868E+00_rb, 1.94132E+00_rb, 1.45897E+00_rb,&
        & 9.87794E-01_rb, 1.08761E-01_rb, 9.04825E-02_rb, 7.04264E-02_rb,&
        & 5.15486E-02_rb, 3.26952E-02_rb, 1.24412E-02_rb, 1.76108E-03_rb/)
      irradnceo(:,2) = (/ &
        & 3.22464E+00_rb, 3.26021E+00_rb, 3.16714E+00_rb, 2.97023E+00_rb,&
        & 2.70851E+00_rb, 2.36703E+00_rb, 1.94928E+00_rb, 1.46798E+00_rb,&
        & 9.91940E-01_rb, 1.09364E-01_rb, 9.04938E-02_rb, 7.05002E-02_rb,&
        & 5.15336E-02_rb, 3.27581E-02_rb, 1.24115E-02_rb, 1.65981E-03_rb/)
      irradnceo(:,3) = (/ &
        & 3.21231E+00_rb, 3.25083E+00_rb, 3.17645E+00_rb, 2.96601E+00_rb,&
        & 2.70867E+00_rb, 2.37345E+00_rb, 1.95485E+00_rb, 1.46694E+00_rb,&
        & 9.96723E-01_rb, 1.09679E-01_rb, 9.03151E-02_rb, 7.07535E-02_rb,&
        & 5.16563E-02_rb, 3.28771E-02_rb, 1.25146E-02_rb, 1.65981E-03_rb/)
      irradnceo(:,4) = (/ &
        & 3.21210E+00_rb, 3.23679E+00_rb, 3.18145E+00_rb, 2.96259E+00_rb,&
        & 2.71020E+00_rb, 2.37711E+00_rb, 1.95647E+00_rb, 1.46794E+00_rb,&
        & 1.00016E+00_rb, 1.10322E-01_rb, 9.04787E-02_rb, 7.08121E-02_rb,&
        & 5.18928E-02_rb, 3.31462E-02_rb, 1.25553E-02_rb, 1.65981E-03_rb/)
      irradnceo(:,5) = (/ &
        & 3.21193E+00_rb, 3.22866E+00_rb, 3.17920E+00_rb, 2.96542E+00_rb,&
        & 2.70943E+00_rb, 2.37864E+00_rb, 1.95599E+00_rb, 1.47258E+00_rb,&
        & 1.00200E+00_rb, 1.09929E-01_rb, 9.06734E-02_rb, 7.14207E-02_rb,&
        & 5.19103E-02_rb, 3.36860E-02_rb, 1.25533E-02_rb, 1.65981E-03_rb/)
      irradnceo(:,6) = (/ &
        & 3.21216E+00_rb, 3.21680E+00_rb, 3.17628E+00_rb, 2.97392E+00_rb,&
        & 2.70991E+00_rb, 2.37645E+00_rb, 1.95572E+00_rb, 1.47766E+00_rb,&
        & 1.00376E+00_rb, 1.09547E-01_rb, 9.08204E-02_rb, 7.15920E-02_rb,&
        & 5.30844E-02_rb, 3.37632E-02_rb, 1.25511E-02_rb, 1.65981E-03_rb/)
      irradnceo(:,7) = (/ &
        & 3.21219E+00_rb, 3.21733E+00_rb, 3.15968E+00_rb, 2.97917E+00_rb,&
        & 2.71368E+00_rb, 2.37392E+00_rb, 1.95880E+00_rb, 1.48041E+00_rb,&
        & 1.00576E+00_rb, 1.09775E-01_rb, 9.05971E-02_rb, 7.28722E-02_rb,&
        & 5.35413E-02_rb, 3.37507E-02_rb, 1.25489E-02_rb, 1.65981E-03_rb/)
      irradnceo(:,8) = (/ &
        & 3.21254E+00_rb, 3.22241E+00_rb, 3.13600E+00_rb, 2.98333E+00_rb,&
        & 2.72378E+00_rb, 2.36800E+00_rb, 1.95807E+00_rb, 1.48814E+00_rb,&
        & 1.00643E+00_rb, 1.10737E-01_rb, 9.17661E-02_rb, 7.30206E-02_rb,&
        & 5.35157E-02_rb, 3.37346E-02_rb, 1.25378E-02_rb, 1.65981E-03_rb/)
      irradnceo(:,9) = (/ &
        & 3.26637E+00_rb, 3.22332E+00_rb, 3.12064E+00_rb, 2.95176E+00_rb,&
        & 2.71815E+00_rb, 2.35210E+00_rb, 1.96163E+00_rb, 1.49356E+00_rb,&
        & 1.01105E+00_rb, 1.10865E-01_rb, 9.17772E-02_rb, 7.29739E-02_rb,&
        & 5.35439E-02_rb, 3.37401E-02_rb, 1.25468E-02_rb, 1.65981E-03_rb/)
! Solar variability components: facular brightening
      facbrghto(:,1) = (/ &
        & 2.67470E-04_rb, 2.70892E-04_rb, 2.65755E-04_rb, 2.41537E-04_rb,&
        & 2.10019E-04_rb, 1.74776E-04_rb, 1.28340E-04_rb, 9.42299E-05_rb,&
        & 6.56174E-05_rb, 7.02011E-06_rb, 5.69822E-06_rb, 3.97073E-06_rb,&
        & 2.82411E-06_rb, 1.75957E-06_rb, 6.81702E-07_rb, 1.05015E-07_rb/)
      facbrghto(:,2) = (/ &
        & 2.63834E-04_rb, 2.70477E-04_rb, 2.58214E-04_rb, 2.38117E-04_rb,&
        & 2.11950E-04_rb, 1.73330E-04_rb, 1.34262E-04_rb, 9.96856E-05_rb,&
        & 6.81372E-05_rb, 7.09208E-06_rb, 5.74581E-06_rb, 4.10091E-06_rb,&
        & 3.01113E-06_rb, 1.82407E-06_rb, 7.36684E-07_rb, 1.79807E-07_rb/)
      facbrghto(:,3) = (/ &
        & 2.63438E-04_rb, 2.69699E-04_rb, 2.56944E-04_rb, 2.35445E-04_rb,&
        & 2.10029E-04_rb, 1.71060E-04_rb, 1.36699E-04_rb, 1.03692E-04_rb,&
        & 6.99202E-05_rb, 7.37901E-06_rb, 5.95988E-06_rb, 4.34775E-06_rb,&
        & 2.92988E-06_rb, 1.88928E-06_rb, 1.08652E-06_rb, 1.79807E-07_rb/)
      facbrghto(:,4) = (/ &
        & 2.63564E-04_rb, 2.68569E-04_rb, 2.56117E-04_rb, 2.32521E-04_rb,&
        & 2.09426E-04_rb, 1.69982E-04_rb, 1.37639E-04_rb, 1.06421E-04_rb,&
        & 7.16871E-05_rb, 7.55963E-06_rb, 5.97157E-06_rb, 4.47199E-06_rb,&
        & 3.37038E-06_rb, 2.07798E-06_rb, 1.13913E-06_rb, 1.79807E-07_rb/)
      facbrghto(:,5) = (/ &
        & 2.63697E-04_rb, 2.67955E-04_rb, 2.54698E-04_rb, 2.31952E-04_rb,&
        & 2.06197E-04_rb, 1.68486E-04_rb, 1.39955E-04_rb, 1.08507E-04_rb,&
        & 7.32936E-05_rb, 7.67108E-06_rb, 6.18486E-06_rb, 4.67470E-06_rb,&
        & 3.27926E-06_rb, 2.82680E-06_rb, 1.13974E-06_rb, 1.79807E-07_rb/)
      facbrghto(:,6) = (/ &
        & 2.63827E-04_rb, 2.66995E-04_rb, 2.52995E-04_rb, 2.31832E-04_rb,&
        & 2.00528E-04_rb, 1.70484E-04_rb, 1.41826E-04_rb, 1.10089E-04_rb,&
        & 7.48546E-05_rb, 7.72147E-06_rb, 6.38564E-06_rb, 4.86823E-06_rb,&
        & 4.01616E-06_rb, 2.95531E-06_rb, 1.14035E-06_rb, 1.79807E-07_rb/)
      facbrghto(:,7) = (/ &
        & 2.63880E-04_rb, 2.65962E-04_rb, 2.50589E-04_rb, 2.31442E-04_rb,&
        & 1.96255E-04_rb, 1.70607E-04_rb, 1.44612E-04_rb, 1.11859E-04_rb,&
        & 7.63459E-05_rb, 8.16199E-06_rb, 6.48714E-06_rb, 5.85086E-06_rb,&
        & 4.36179E-06_rb, 2.96177E-06_rb, 1.14096E-06_rb, 1.79807E-07_rb/)
      facbrghto(:,8) = (/ &
        & 2.63851E-04_rb, 2.64322E-04_rb, 2.45295E-04_rb, 2.29974E-04_rb,&
        & 1.96258E-04_rb, 1.70111E-04_rb, 1.46718E-04_rb, 1.14305E-04_rb,&
        & 7.85692E-05_rb, 9.09490E-06_rb, 7.56003E-06_rb, 5.98239E-06_rb,&
        & 4.36095E-06_rb, 2.97295E-06_rb, 1.14294E-06_rb, 1.79807E-07_rb/)
      facbrghto(:,9) = (/ &
        & 2.61726E-04_rb, 2.58115E-04_rb, 2.37387E-04_rb, 2.14399E-04_rb,&
        & 2.02794E-04_rb, 1.79125E-04_rb, 1.52561E-04_rb, 1.19450E-04_rb,&
        & 8.36834E-05_rb, 9.17006E-06_rb, 7.65575E-06_rb, 5.98517E-06_rb,&
        & 4.35613E-06_rb, 2.96911E-06_rb, 1.14157E-06_rb, 1.79807E-07_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:,1) = (/ &
        &-3.85233E-04_rb,-3.82906E-04_rb,-3.71298E-04_rb,-3.47178E-04_rb,&
        &-3.15585E-04_rb,-2.75540E-04_rb,-2.26666E-04_rb,-1.71012E-04_rb,&
        &-1.15971E-04_rb,-1.27737E-05_rb,-1.06466E-05_rb,-8.26440E-06_rb,&
        &-6.04814E-06_rb,-3.85132E-06_rb,-1.45418E-06_rb,-2.05244E-07_rb/)
      snsptdrko(:,2) = (/ &
        &-3.74016E-04_rb,-3.85296E-04_rb,-3.74270E-04_rb,-3.49850E-04_rb,&
        &-3.17080E-04_rb,-2.76122E-04_rb,-2.27310E-04_rb,-1.71389E-04_rb,&
        &-1.16089E-04_rb,-1.28343E-05_rb,-1.06244E-05_rb,-8.23831E-06_rb,&
        &-6.02828E-06_rb,-3.84911E-06_rb,-1.44826E-06_rb,-1.89435E-07_rb/)
      snsptdrko(:,3) = (/ &
        &-3.72102E-04_rb,-3.83792E-04_rb,-3.75237E-04_rb,-3.50348E-04_rb,&
        &-3.17997E-04_rb,-2.76614E-04_rb,-2.27714E-04_rb,-1.71314E-04_rb,&
        &-1.16413E-04_rb,-1.28520E-05_rb,-1.05106E-05_rb,-8.24265E-06_rb,&
        &-6.04169E-06_rb,-3.84750E-06_rb,-1.41798E-06_rb,-1.89435E-07_rb/)
      snsptdrko(:,4) = (/ &
        &-3.72084E-04_rb,-3.81229E-04_rb,-3.76213E-04_rb,-3.50136E-04_rb,&
        &-3.18925E-04_rb,-2.77111E-04_rb,-2.27794E-04_rb,-1.71391E-04_rb,&
        &-1.16623E-04_rb,-1.28794E-05_rb,-1.05660E-05_rb,-8.22555E-06_rb,&
        &-6.00534E-06_rb,-3.84861E-06_rb,-1.41357E-06_rb,-1.89435E-07_rb/)
      snsptdrko(:,5) = (/ &
        &-3.72215E-04_rb,-3.78702E-04_rb,-3.76793E-04_rb,-3.50410E-04_rb,&
        &-3.19418E-04_rb,-2.77985E-04_rb,-2.27608E-04_rb,-1.71612E-04_rb,&
        &-1.16831E-04_rb,-1.27846E-05_rb,-1.05171E-05_rb,-8.32813E-06_rb,&
        &-6.03247E-06_rb,-3.79469E-06_rb,-1.41324E-06_rb,-1.89435E-07_rb/)
      snsptdrko(:,6) = (/ &
        &-3.72402E-04_rb,-3.76631E-04_rb,-3.76006E-04_rb,-3.51759E-04_rb,&
        &-3.20130E-04_rb,-2.78176E-04_rb,-2.27639E-04_rb,-1.71833E-04_rb,&
        &-1.16992E-04_rb,-1.27316E-05_rb,-1.05153E-05_rb,-8.31179E-06_rb,&
        &-6.11931E-06_rb,-3.78502E-06_rb,-1.41290E-06_rb,-1.89435E-07_rb/)
      snsptdrko(:,7) = (/ &
        &-3.72627E-04_rb,-3.76159E-04_rb,-3.73439E-04_rb,-3.52468E-04_rb,&
        &-3.20800E-04_rb,-2.78788E-04_rb,-2.28189E-04_rb,-1.71862E-04_rb,&
        &-1.17188E-04_rb,-1.27279E-05_rb,-1.05089E-05_rb,-8.34820E-06_rb,&
        &-6.14653E-06_rb,-3.78082E-06_rb,-1.41256E-06_rb,-1.89435E-07_rb/)
      snsptdrko(:,8) = (/ &
        &-3.72958E-04_rb,-3.76352E-04_rb,-3.67866E-04_rb,-3.54451E-04_rb,&
        &-3.22226E-04_rb,-2.79334E-04_rb,-2.28834E-04_rb,-1.72450E-04_rb,&
        &-1.16952E-04_rb,-1.28078E-05_rb,-1.05327E-05_rb,-8.35383E-06_rb,&
        &-6.14311E-06_rb,-3.77329E-06_rb,-1.41168E-06_rb,-1.89435E-07_rb/)
      snsptdrko(:,9) = (/ &
        &-3.80181E-04_rb,-3.75940E-04_rb,-3.64525E-04_rb,-3.47028E-04_rb,&
        &-3.23028E-04_rb,-2.80128E-04_rb,-2.30726E-04_rb,-1.72824E-04_rb,&
        &-1.17047E-04_rb,-1.27960E-05_rb,-1.05348E-05_rb,-8.35164E-06_rb,&
        &-6.14612E-06_rb,-3.77591E-06_rb,-1.41223E-06_rb,-1.89435E-07_rb/)

! Rayleigh extinction coefficient at v = 4900 cm-1.
      rayl = 2.29e-09_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.14981e-05_rb,0.26659e-05_rb,0.31874e-05_rb,0.35509e-05_rb,0.37593e-05_rb, &
        & 0.38514e-05_rb,0.37369e-05_rb,0.35011e-05_rb,0.23894e-05_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.15103e-05_rb,0.27348e-05_rb,0.32692e-05_rb,0.36558e-05_rb,0.38926e-05_rb, &
        & 0.39737e-05_rb,0.38798e-05_rb,0.36565e-05_rb,0.24072e-05_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.15233e-05_rb,0.28090e-05_rb,0.33744e-05_rb,0.37729e-05_rb,0.40258e-05_rb, &
        & 0.41107e-05_rb,0.40389e-05_rb,0.38226e-05_rb,0.24704e-05_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.15219e-05_rb,0.28978e-05_rb,0.35015e-05_rb,0.38984e-05_rb,0.41569e-05_rb, &
        & 0.42771e-05_rb,0.42110e-05_rb,0.39963e-05_rb,0.25504e-05_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.15254e-05_rb,0.29633e-05_rb,0.36224e-05_rb,0.40435e-05_rb,0.42975e-05_rb, &
        & 0.44410e-05_rb,0.43849e-05_rb,0.41847e-05_rb,0.26420e-05_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.13024e-05_rb,0.23699e-05_rb,0.28370e-05_rb,0.31443e-05_rb,0.33326e-05_rb, &
        & 0.33798e-05_rb,0.32797e-05_rb,0.30078e-05_rb,0.18819e-05_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.13249e-05_rb,0.24299e-05_rb,0.29103e-05_rb,0.32336e-05_rb,0.34308e-05_rb, &
        & 0.35019e-05_rb,0.34046e-05_rb,0.31323e-05_rb,0.19511e-05_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.13241e-05_rb,0.25059e-05_rb,0.30019e-05_rb,0.33317e-05_rb,0.35516e-05_rb, &
        & 0.36333e-05_rb,0.35372e-05_rb,0.32615e-05_rb,0.20254e-05_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.13325e-05_rb,0.25840e-05_rb,0.31009e-05_rb,0.34399e-05_rb,0.36699e-05_rb, &
        & 0.37727e-05_rb,0.36811e-05_rb,0.34016e-05_rb,0.20611e-05_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.13321e-05_rb,0.26601e-05_rb,0.32189e-05_rb,0.35640e-05_rb,0.38012e-05_rb, &
        & 0.39113e-05_rb,0.38271e-05_rb,0.35499e-05_rb,0.21368e-05_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.11079e-05_rb,0.20757e-05_rb,0.24846e-05_rb,0.27511e-05_rb,0.28883e-05_rb, &
        & 0.29142e-05_rb,0.28203e-05_rb,0.25706e-05_rb,0.15124e-05_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.11298e-05_rb,0.21278e-05_rb,0.25551e-05_rb,0.28306e-05_rb,0.29814e-05_rb, &
        & 0.30126e-05_rb,0.29328e-05_rb,0.26721e-05_rb,0.15784e-05_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.11405e-05_rb,0.21812e-05_rb,0.26237e-05_rb,0.29130e-05_rb,0.30816e-05_rb, &
        & 0.31265e-05_rb,0.30480e-05_rb,0.27784e-05_rb,0.16468e-05_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.11347e-05_rb,0.22472e-05_rb,0.27094e-05_rb,0.30057e-05_rb,0.31821e-05_rb, &
        & 0.32536e-05_rb,0.31714e-05_rb,0.28837e-05_rb,0.17189e-05_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.11383e-05_rb,0.23152e-05_rb,0.28016e-05_rb,0.31126e-05_rb,0.33051e-05_rb, &
        & 0.33759e-05_rb,0.32961e-05_rb,0.29993e-05_rb,0.17919e-05_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.93104e-06_rb,0.18005e-05_rb,0.21693e-05_rb,0.23819e-05_rb,0.24829e-05_rb, &
        & 0.24860e-05_rb,0.23860e-05_rb,0.21586e-05_rb,0.12198e-05_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.94591e-06_rb,0.18309e-05_rb,0.22153e-05_rb,0.24487e-05_rb,0.25574e-05_rb, &
        & 0.25721e-05_rb,0.24802e-05_rb,0.22458e-05_rb,0.13105e-05_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.95638e-06_rb,0.18753e-05_rb,0.22788e-05_rb,0.25149e-05_rb,0.26499e-05_rb, &
        & 0.26671e-05_rb,0.25797e-05_rb,0.23335e-05_rb,0.14017e-05_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.95763e-06_rb,0.19379e-05_rb,0.23271e-05_rb,0.25858e-05_rb,0.27437e-05_rb, &
        & 0.27792e-05_rb,0.26790e-05_rb,0.24268e-05_rb,0.14872e-05_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.95668e-06_rb,0.19877e-05_rb,0.23997e-05_rb,0.26721e-05_rb,0.28382e-05_rb, &
        & 0.28806e-05_rb,0.27857e-05_rb,0.25225e-05_rb,0.15698e-05_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.77329e-06_rb,0.15354e-05_rb,0.18723e-05_rb,0.20411e-05_rb,0.21095e-05_rb, &
        & 0.21017e-05_rb,0.19998e-05_rb,0.18017e-05_rb,0.97629e-06_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.78241e-06_rb,0.15660e-05_rb,0.19091e-05_rb,0.20970e-05_rb,0.21786e-05_rb, &
        & 0.21730e-05_rb,0.20794e-05_rb,0.18731e-05_rb,0.10645e-05_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.79463e-06_rb,0.15948e-05_rb,0.19458e-05_rb,0.21529e-05_rb,0.22557e-05_rb, &
        & 0.22576e-05_rb,0.21631e-05_rb,0.19462e-05_rb,0.11507e-05_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.79779e-06_rb,0.16389e-05_rb,0.19877e-05_rb,0.22087e-05_rb,0.23274e-05_rb, &
        & 0.23462e-05_rb,0.22501e-05_rb,0.20204e-05_rb,0.12366e-05_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.79875e-06_rb,0.16865e-05_rb,0.20410e-05_rb,0.22789e-05_rb,0.24049e-05_rb, &
        & 0.24283e-05_rb,0.23388e-05_rb,0.21006e-05_rb,0.13155e-05_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.63799e-06_rb,0.13079e-05_rb,0.15873e-05_rb,0.17187e-05_rb,0.17689e-05_rb, &
        & 0.17509e-05_rb,0.16625e-05_rb,0.14950e-05_rb,0.77624e-06_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.64423e-06_rb,0.13180e-05_rb,0.16173e-05_rb,0.17648e-05_rb,0.18253e-05_rb, &
        & 0.18152e-05_rb,0.17299e-05_rb,0.15482e-05_rb,0.85457e-06_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.65214e-06_rb,0.13481e-05_rb,0.16476e-05_rb,0.18158e-05_rb,0.18867e-05_rb, &
        & 0.18859e-05_rb,0.17999e-05_rb,0.16080e-05_rb,0.93968e-06_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.65949e-06_rb,0.13728e-05_rb,0.16795e-05_rb,0.18618e-05_rb,0.19507e-05_rb, &
        & 0.19568e-05_rb,0.18693e-05_rb,0.16710e-05_rb,0.10256e-05_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.66012e-06_rb,0.14051e-05_rb,0.17174e-05_rb,0.19154e-05_rb,0.20111e-05_rb, &
        & 0.20230e-05_rb,0.19421e-05_rb,0.17361e-05_rb,0.11072e-05_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.52652e-06_rb,0.10995e-05_rb,0.13221e-05_rb,0.14282e-05_rb,0.14627e-05_rb, &
        & 0.14494e-05_rb,0.13754e-05_rb,0.12359e-05_rb,0.62399e-06_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.52631e-06_rb,0.11047e-05_rb,0.13498e-05_rb,0.14646e-05_rb,0.15094e-05_rb, &
        & 0.14961e-05_rb,0.14272e-05_rb,0.12791e-05_rb,0.69692e-06_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.53263e-06_rb,0.11231e-05_rb,0.13763e-05_rb,0.15059e-05_rb,0.15602e-05_rb, &
        & 0.15533e-05_rb,0.14827e-05_rb,0.13257e-05_rb,0.76959e-06_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.53788e-06_rb,0.11448e-05_rb,0.14026e-05_rb,0.15478e-05_rb,0.16122e-05_rb, &
        & 0.16137e-05_rb,0.15398e-05_rb,0.13748e-05_rb,0.84488e-06_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.54243e-06_rb,0.11660e-05_rb,0.14288e-05_rb,0.15891e-05_rb,0.16619e-05_rb, &
        & 0.16672e-05_rb,0.15978e-05_rb,0.14265e-05_rb,0.92439e-06_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.42860e-06_rb,0.91208e-06_rb,0.10887e-05_rb,0.11732e-05_rb,0.12000e-05_rb, &
        & 0.11901e-05_rb,0.11300e-05_rb,0.10181e-05_rb,0.51360e-06_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.42605e-06_rb,0.91355e-06_rb,0.11105e-05_rb,0.12004e-05_rb,0.12359e-05_rb, &
        & 0.12238e-05_rb,0.11674e-05_rb,0.10496e-05_rb,0.58593e-06_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.42910e-06_rb,0.92458e-06_rb,0.11341e-05_rb,0.12355e-05_rb,0.12754e-05_rb, &
        & 0.12667e-05_rb,0.12100e-05_rb,0.10877e-05_rb,0.65209e-06_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.43345e-06_rb,0.94024e-06_rb,0.11540e-05_rb,0.12669e-05_rb,0.13186e-05_rb, &
        & 0.13163e-05_rb,0.12567e-05_rb,0.11280e-05_rb,0.72051e-06_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.43744e-06_rb,0.95845e-06_rb,0.11764e-05_rb,0.13022e-05_rb,0.13595e-05_rb, &
        & 0.13618e-05_rb,0.13034e-05_rb,0.11671e-05_rb,0.79373e-06_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.34458e-06_rb,0.74844e-06_rb,0.89217e-06_rb,0.95875e-06_rb,0.98243e-06_rb, &
        & 0.97566e-06_rb,0.92936e-06_rb,0.83611e-06_rb,0.44181e-06_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.34797e-06_rb,0.75304e-06_rb,0.90816e-06_rb,0.98264e-06_rb,0.10080e-05_rb, &
        & 0.99889e-06_rb,0.95318e-06_rb,0.86041e-06_rb,0.51013e-06_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.34593e-06_rb,0.76020e-06_rb,0.92707e-06_rb,0.10062e-05_rb,0.10391e-05_rb, &
        & 0.10296e-05_rb,0.98530e-06_rb,0.89011e-06_rb,0.58070e-06_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.34971e-06_rb,0.77088e-06_rb,0.94623e-06_rb,0.10342e-05_rb,0.10726e-05_rb, &
        & 0.10689e-05_rb,0.10210e-05_rb,0.92020e-06_rb,0.65697e-06_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.35282e-06_rb,0.78517e-06_rb,0.96245e-06_rb,0.10609e-05_rb,0.11071e-05_rb, &
        & 0.11073e-05_rb,0.10583e-05_rb,0.95054e-06_rb,0.72956e-06_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.27840e-06_rb,0.61035e-06_rb,0.72930e-06_rb,0.78415e-06_rb,0.80451e-06_rb, &
        & 0.79924e-06_rb,0.76231e-06_rb,0.68727e-06_rb,0.40208e-06_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.28171e-06_rb,0.61798e-06_rb,0.74197e-06_rb,0.80115e-06_rb,0.82183e-06_rb, &
        & 0.81621e-06_rb,0.78030e-06_rb,0.70425e-06_rb,0.46700e-06_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.28023e-06_rb,0.62269e-06_rb,0.75685e-06_rb,0.82101e-06_rb,0.84538e-06_rb, &
        & 0.83821e-06_rb,0.80301e-06_rb,0.72663e-06_rb,0.53781e-06_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.28163e-06_rb,0.63156e-06_rb,0.77214e-06_rb,0.84244e-06_rb,0.87184e-06_rb, &
        & 0.86752e-06_rb,0.83024e-06_rb,0.75000e-06_rb,0.60478e-06_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.28518e-06_rb,0.64070e-06_rb,0.78628e-06_rb,0.86439e-06_rb,0.89969e-06_rb, &
        & 0.89910e-06_rb,0.85933e-06_rb,0.77376e-06_rb,0.67016e-06_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.22555e-06_rb,0.50042e-06_rb,0.59960e-06_rb,0.64537e-06_rb,0.66324e-06_rb, &
        & 0.65924e-06_rb,0.63003e-06_rb,0.56890e-06_rb,0.34466e-06_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.22858e-06_rb,0.50705e-06_rb,0.60997e-06_rb,0.65948e-06_rb,0.67688e-06_rb, &
        & 0.67311e-06_rb,0.64462e-06_rb,0.58333e-06_rb,0.40300e-06_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.22677e-06_rb,0.51178e-06_rb,0.62299e-06_rb,0.67572e-06_rb,0.69624e-06_rb, &
        & 0.69126e-06_rb,0.66285e-06_rb,0.60090e-06_rb,0.46066e-06_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.22825e-06_rb,0.51907e-06_rb,0.63500e-06_rb,0.69311e-06_rb,0.71796e-06_rb, &
        & 0.71484e-06_rb,0.68416e-06_rb,0.61859e-06_rb,0.51966e-06_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.23122e-06_rb,0.52598e-06_rb,0.64574e-06_rb,0.71116e-06_rb,0.74046e-06_rb, &
        & 0.73979e-06_rb,0.70672e-06_rb,0.63708e-06_rb,0.58196e-06_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.18149e-06_rb,0.40829e-06_rb,0.49022e-06_rb,0.52910e-06_rb,0.54408e-06_rb, &
        & 0.54163e-06_rb,0.51867e-06_rb,0.46933e-06_rb,0.28496e-06_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.18433e-06_rb,0.41378e-06_rb,0.49908e-06_rb,0.54027e-06_rb,0.55522e-06_rb, &
        & 0.55286e-06_rb,0.53038e-06_rb,0.48103e-06_rb,0.33336e-06_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.18246e-06_rb,0.41799e-06_rb,0.50965e-06_rb,0.55347e-06_rb,0.57134e-06_rb, &
        & 0.56787e-06_rb,0.54477e-06_rb,0.49462e-06_rb,0.38023e-06_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.18409e-06_rb,0.42347e-06_rb,0.51936e-06_rb,0.56749e-06_rb,0.58877e-06_rb, &
        & 0.58648e-06_rb,0.56149e-06_rb,0.50825e-06_rb,0.43081e-06_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.18582e-06_rb,0.42888e-06_rb,0.52830e-06_rb,0.58240e-06_rb,0.60647e-06_rb, &
        & 0.60598e-06_rb,0.57933e-06_rb,0.52277e-06_rb,0.48565e-06_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.14345e-06_rb,0.32927e-06_rb,0.39750e-06_rb,0.43000e-06_rb,0.44318e-06_rb, &
        & 0.44201e-06_rb,0.42469e-06_rb,0.38538e-06_rb,0.23377e-06_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.14571e-06_rb,0.33355e-06_rb,0.40430e-06_rb,0.43954e-06_rb,0.45256e-06_rb, &
        & 0.45159e-06_rb,0.43399e-06_rb,0.39467e-06_rb,0.27338e-06_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.14442e-06_rb,0.33719e-06_rb,0.41316e-06_rb,0.44996e-06_rb,0.46575e-06_rb, &
        & 0.46364e-06_rb,0.44528e-06_rb,0.40510e-06_rb,0.31154e-06_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.14600e-06_rb,0.34141e-06_rb,0.42136e-06_rb,0.46096e-06_rb,0.47937e-06_rb, &
        & 0.47817e-06_rb,0.45850e-06_rb,0.41598e-06_rb,0.35289e-06_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.14667e-06_rb,0.34608e-06_rb,0.42853e-06_rb,0.47296e-06_rb,0.49300e-06_rb, &
        & 0.49352e-06_rb,0.47280e-06_rb,0.42757e-06_rb,0.39752e-06_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.16288e-04_rb,0.25817e-04_rb,0.29597e-04_rb,0.31319e-04_rb,0.31220e-04_rb, &
        & 0.29539e-04_rb,0.25627e-04_rb,0.19758e-04_rb,0.72944e-05_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.18123e-04_rb,0.28565e-04_rb,0.32633e-04_rb,0.34427e-04_rb,0.34219e-04_rb, &
        & 0.32223e-04_rb,0.27987e-04_rb,0.21485e-04_rb,0.79554e-05_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.19957e-04_rb,0.31607e-04_rb,0.36033e-04_rb,0.37887e-04_rb,0.37694e-04_rb, &
        & 0.35399e-04_rb,0.30718e-04_rb,0.23412e-04_rb,0.85293e-05_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.22005e-04_rb,0.35064e-04_rb,0.39763e-04_rb,0.41694e-04_rb,0.41490e-04_rb, &
        & 0.38891e-04_rb,0.33715e-04_rb,0.25512e-04_rb,0.91232e-05_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.24240e-04_rb,0.38906e-04_rb,0.43899e-04_rb,0.45870e-04_rb,0.45580e-04_rb, &
        & 0.42616e-04_rb,0.36912e-04_rb,0.27728e-04_rb,0.97792e-05_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.14944e-04_rb,0.23223e-04_rb,0.25792e-04_rb,0.27019e-04_rb,0.26963e-04_rb, &
        & 0.25431e-04_rb,0.22069e-04_rb,0.16933e-04_rb,0.59071e-05_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.16615e-04_rb,0.25452e-04_rb,0.28322e-04_rb,0.29800e-04_rb,0.29593e-04_rb, &
        & 0.27776e-04_rb,0.24123e-04_rb,0.18444e-04_rb,0.63142e-05_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.18355e-04_rb,0.27996e-04_rb,0.31338e-04_rb,0.32892e-04_rb,0.32575e-04_rb, &
        & 0.30478e-04_rb,0.26504e-04_rb,0.20132e-04_rb,0.67898e-05_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.20092e-04_rb,0.30938e-04_rb,0.34680e-04_rb,0.36255e-04_rb,0.35836e-04_rb, &
        & 0.33500e-04_rb,0.29106e-04_rb,0.21950e-04_rb,0.73343e-05_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.21897e-04_rb,0.34150e-04_rb,0.38274e-04_rb,0.39931e-04_rb,0.39337e-04_rb, &
        & 0.36752e-04_rb,0.31831e-04_rb,0.23882e-04_rb,0.78705e-05_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.13130e-04_rb,0.20363e-04_rb,0.22377e-04_rb,0.23003e-04_rb,0.22683e-04_rb, &
        & 0.21429e-04_rb,0.18621e-04_rb,0.14177e-04_rb,0.47892e-05_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.14610e-04_rb,0.22299e-04_rb,0.24486e-04_rb,0.25140e-04_rb,0.24842e-04_rb, &
        & 0.23413e-04_rb,0.20361e-04_rb,0.15469e-04_rb,0.52250e-05_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.16133e-04_rb,0.24482e-04_rb,0.26876e-04_rb,0.27653e-04_rb,0.27357e-04_rb, &
        & 0.25669e-04_rb,0.22341e-04_rb,0.16897e-04_rb,0.56647e-05_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.17708e-04_rb,0.26887e-04_rb,0.29522e-04_rb,0.30577e-04_rb,0.30151e-04_rb, &
        & 0.28221e-04_rb,0.24545e-04_rb,0.18455e-04_rb,0.60735e-05_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.19305e-04_rb,0.29364e-04_rb,0.32520e-04_rb,0.33741e-04_rb,0.33162e-04_rb, &
        & 0.30976e-04_rb,0.26800e-04_rb,0.20103e-04_rb,0.64864e-05_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.11163e-04_rb,0.17498e-04_rb,0.19069e-04_rb,0.19561e-04_rb,0.19154e-04_rb, &
        & 0.17906e-04_rb,0.15522e-04_rb,0.11778e-04_rb,0.40461e-05_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.12505e-04_rb,0.19238e-04_rb,0.20917e-04_rb,0.21365e-04_rb,0.20913e-04_rb, &
        & 0.19471e-04_rb,0.16986e-04_rb,0.12856e-04_rb,0.43935e-05_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.13848e-04_rb,0.21132e-04_rb,0.22975e-04_rb,0.23433e-04_rb,0.22893e-04_rb, &
        & 0.21327e-04_rb,0.18662e-04_rb,0.14056e-04_rb,0.47015e-05_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.15219e-04_rb,0.23159e-04_rb,0.25250e-04_rb,0.25753e-04_rb,0.25088e-04_rb, &
        & 0.23437e-04_rb,0.20455e-04_rb,0.15366e-04_rb,0.50239e-05_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.16631e-04_rb,0.25311e-04_rb,0.27651e-04_rb,0.28243e-04_rb,0.27566e-04_rb, &
        & 0.25775e-04_rb,0.22344e-04_rb,0.16758e-04_rb,0.53594e-05_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.92688e-05_rb,0.14829e-04_rb,0.16036e-04_rb,0.16388e-04_rb,0.16087e-04_rb, &
        & 0.15049e-04_rb,0.12939e-04_rb,0.97243e-05_rb,0.35064e-05_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.10428e-04_rb,0.16289e-04_rb,0.17596e-04_rb,0.17928e-04_rb,0.17547e-04_rb, &
        & 0.16347e-04_rb,0.14108e-04_rb,0.10621e-04_rb,0.37893e-05_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.11600e-04_rb,0.17922e-04_rb,0.19403e-04_rb,0.19701e-04_rb,0.19227e-04_rb, &
        & 0.17835e-04_rb,0.15439e-04_rb,0.11627e-04_rb,0.40426e-05_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.12795e-04_rb,0.19672e-04_rb,0.21360e-04_rb,0.21677e-04_rb,0.21078e-04_rb, &
        & 0.19535e-04_rb,0.16919e-04_rb,0.12746e-04_rb,0.43015e-05_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.14029e-04_rb,0.21551e-04_rb,0.23410e-04_rb,0.23746e-04_rb,0.23106e-04_rb, &
        & 0.21397e-04_rb,0.18500e-04_rb,0.13918e-04_rb,0.45663e-05_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.75358e-05_rb,0.12284e-04_rb,0.13321e-04_rb,0.13580e-04_rb,0.13322e-04_rb, &
        & 0.12459e-04_rb,0.10757e-04_rb,0.80012e-05_rb,0.28595e-05_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.84801e-05_rb,0.13541e-04_rb,0.14600e-04_rb,0.14857e-04_rb,0.14519e-04_rb, &
        & 0.13528e-04_rb,0.11747e-04_rb,0.87199e-05_rb,0.31250e-05_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.94797e-05_rb,0.14898e-04_rb,0.16088e-04_rb,0.16329e-04_rb,0.15922e-04_rb, &
        & 0.14786e-04_rb,0.12839e-04_rb,0.95406e-05_rb,0.33925e-05_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.10510e-04_rb,0.16387e-04_rb,0.17709e-04_rb,0.17968e-04_rb,0.17465e-04_rb, &
        & 0.16205e-04_rb,0.13999e-04_rb,0.10465e-04_rb,0.36786e-05_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.11554e-04_rb,0.17973e-04_rb,0.19442e-04_rb,0.19736e-04_rb,0.19170e-04_rb, &
        & 0.17751e-04_rb,0.15288e-04_rb,0.11441e-04_rb,0.39618e-05_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.59994e-05_rb,0.10068e-04_rb,0.10903e-04_rb,0.11100e-04_rb,0.10910e-04_rb, &
        & 0.10207e-04_rb,0.88301e-05_rb,0.65755e-05_rb,0.23537e-05_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.67911e-05_rb,0.11098e-04_rb,0.11964e-04_rb,0.12161e-04_rb,0.11910e-04_rb, &
        & 0.11113e-04_rb,0.96475e-05_rb,0.71733e-05_rb,0.26021e-05_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.76012e-05_rb,0.12205e-04_rb,0.13167e-04_rb,0.13368e-04_rb,0.13058e-04_rb, &
        & 0.12136e-04_rb,0.10571e-04_rb,0.78452e-05_rb,0.28416e-05_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.84582e-05_rb,0.13412e-04_rb,0.14518e-04_rb,0.14721e-04_rb,0.14328e-04_rb, &
        & 0.13323e-04_rb,0.11527e-04_rb,0.85995e-05_rb,0.30925e-05_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.93346e-05_rb,0.14749e-04_rb,0.15973e-04_rb,0.16180e-04_rb,0.15730e-04_rb, &
        & 0.14597e-04_rb,0.12601e-04_rb,0.93973e-05_rb,0.33285e-05_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.47174e-05_rb,0.81367e-05_rb,0.88184e-05_rb,0.90007e-05_rb,0.88363e-05_rb, &
        & 0.83059e-05_rb,0.72101e-05_rb,0.53496e-05_rb,0.18862e-05_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.53543e-05_rb,0.89852e-05_rb,0.96904e-05_rb,0.98666e-05_rb,0.96509e-05_rb, &
        & 0.90368e-05_rb,0.78700e-05_rb,0.58341e-05_rb,0.21147e-05_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.60088e-05_rb,0.98855e-05_rb,0.10666e-04_rb,0.10849e-04_rb,0.10597e-04_rb, &
        & 0.98826e-05_rb,0.86243e-05_rb,0.63949e-05_rb,0.23672e-05_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.66958e-05_rb,0.10873e-04_rb,0.11751e-04_rb,0.11945e-04_rb,0.11636e-04_rb, &
        & 0.10852e-04_rb,0.94101e-05_rb,0.70262e-05_rb,0.26329e-05_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.74191e-05_rb,0.11960e-04_rb,0.12943e-04_rb,0.13152e-04_rb,0.12788e-04_rb, &
        & 0.11889e-04_rb,0.10297e-04_rb,0.76977e-05_rb,0.29019e-05_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.36886e-05_rb,0.65140e-05_rb,0.70684e-05_rb,0.72377e-05_rb,0.71080e-05_rb, &
        & 0.67076e-05_rb,0.58353e-05_rb,0.43297e-05_rb,0.16419e-05_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.41772e-05_rb,0.72104e-05_rb,0.77793e-05_rb,0.79335e-05_rb,0.77682e-05_rb, &
        & 0.72903e-05_rb,0.63640e-05_rb,0.47210e-05_rb,0.18721e-05_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.47123e-05_rb,0.79471e-05_rb,0.85662e-05_rb,0.87396e-05_rb,0.85319e-05_rb, &
        & 0.79769e-05_rb,0.69694e-05_rb,0.51793e-05_rb,0.21015e-05_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.52612e-05_rb,0.87319e-05_rb,0.94365e-05_rb,0.96191e-05_rb,0.93783e-05_rb, &
        & 0.87581e-05_rb,0.76277e-05_rb,0.56974e-05_rb,0.23505e-05_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.58401e-05_rb,0.96106e-05_rb,0.10404e-04_rb,0.10594e-04_rb,0.10308e-04_rb, &
        & 0.96030e-05_rb,0.83402e-05_rb,0.62627e-05_rb,0.26170e-05_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.29023e-05_rb,0.52230e-05_rb,0.56669e-05_rb,0.58158e-05_rb,0.57137e-05_rb, &
        & 0.54241e-05_rb,0.47416e-05_rb,0.35098e-05_rb,0.14223e-05_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.32752e-05_rb,0.57903e-05_rb,0.62548e-05_rb,0.63859e-05_rb,0.62528e-05_rb, &
        & 0.58881e-05_rb,0.51542e-05_rb,0.38257e-05_rb,0.16559e-05_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.37047e-05_rb,0.63892e-05_rb,0.68928e-05_rb,0.70312e-05_rb,0.68734e-05_rb, &
        & 0.64357e-05_rb,0.56361e-05_rb,0.42003e-05_rb,0.19248e-05_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.41471e-05_rb,0.70179e-05_rb,0.75920e-05_rb,0.77496e-05_rb,0.75508e-05_rb, &
        & 0.70624e-05_rb,0.61778e-05_rb,0.46271e-05_rb,0.22074e-05_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.46061e-05_rb,0.77215e-05_rb,0.83699e-05_rb,0.85300e-05_rb,0.83121e-05_rb, &
        & 0.77493e-05_rb,0.67551e-05_rb,0.50931e-05_rb,0.25043e-05_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.24060e-05_rb,0.43752e-05_rb,0.47372e-05_rb,0.48598e-05_rb,0.47689e-05_rb, &
        & 0.45290e-05_rb,0.39653e-05_rb,0.29444e-05_rb,0.12255e-05_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.27123e-05_rb,0.48479e-05_rb,0.52339e-05_rb,0.53482e-05_rb,0.52342e-05_rb, &
        & 0.49275e-05_rb,0.43239e-05_rb,0.32196e-05_rb,0.14282e-05_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.30632e-05_rb,0.53402e-05_rb,0.57660e-05_rb,0.58936e-05_rb,0.57515e-05_rb, &
        & 0.53873e-05_rb,0.47344e-05_rb,0.35410e-05_rb,0.16636e-05_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.34245e-05_rb,0.58653e-05_rb,0.63554e-05_rb,0.64878e-05_rb,0.63244e-05_rb, &
        & 0.59147e-05_rb,0.51855e-05_rb,0.39068e-05_rb,0.18858e-05_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.37984e-05_rb,0.64557e-05_rb,0.70032e-05_rb,0.71432e-05_rb,0.69591e-05_rb, &
        & 0.64936e-05_rb,0.56748e-05_rb,0.42889e-05_rb,0.21529e-05_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.19822e-05_rb,0.36438e-05_rb,0.39439e-05_rb,0.40428e-05_rb,0.39656e-05_rb, &
        & 0.37627e-05_rb,0.33048e-05_rb,0.24620e-05_rb,0.10349e-05_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.22347e-05_rb,0.40329e-05_rb,0.43559e-05_rb,0.44535e-05_rb,0.43556e-05_rb, &
        & 0.41018e-05_rb,0.36156e-05_rb,0.26976e-05_rb,0.12078e-05_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.25196e-05_rb,0.44405e-05_rb,0.47965e-05_rb,0.49025e-05_rb,0.47850e-05_rb, &
        & 0.44854e-05_rb,0.39542e-05_rb,0.29734e-05_rb,0.13908e-05_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.28142e-05_rb,0.48792e-05_rb,0.52882e-05_rb,0.53962e-05_rb,0.52646e-05_rb, &
        & 0.49202e-05_rb,0.43229e-05_rb,0.32817e-05_rb,0.15895e-05_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.31182e-05_rb,0.53684e-05_rb,0.58256e-05_rb,0.59413e-05_rb,0.57855e-05_rb, &
        & 0.54043e-05_rb,0.47298e-05_rb,0.35925e-05_rb,0.18050e-05_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.16206e-05_rb,0.30122e-05_rb,0.32612e-05_rb,0.33451e-05_rb,0.32794e-05_rb, &
        & 0.31144e-05_rb,0.27430e-05_rb,0.20488e-05_rb,0.85267e-06_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.18267e-05_rb,0.33331e-05_rb,0.36002e-05_rb,0.36816e-05_rb,0.36054e-05_rb, &
        & 0.33950e-05_rb,0.30021e-05_rb,0.22487e-05_rb,0.99490e-06_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.20587e-05_rb,0.36681e-05_rb,0.39651e-05_rb,0.40531e-05_rb,0.39564e-05_rb, &
        & 0.37127e-05_rb,0.32815e-05_rb,0.24795e-05_rb,0.11422e-05_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.22973e-05_rb,0.40325e-05_rb,0.43728e-05_rb,0.44600e-05_rb,0.43540e-05_rb, &
        & 0.40718e-05_rb,0.35848e-05_rb,0.27366e-05_rb,0.13051e-05_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.25452e-05_rb,0.44379e-05_rb,0.48164e-05_rb,0.49100e-05_rb,0.47833e-05_rb, &
        & 0.44674e-05_rb,0.39194e-05_rb,0.29904e-05_rb,0.14817e-05_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.66641e-04_rb,0.92990e-04_rb,0.96277e-04_rb,0.95182e-04_rb,0.91424e-04_rb, &
        & 0.85445e-04_rb,0.77014e-04_rb,0.62091e-04_rb,0.18284e-04_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.75350e-04_rb,0.10391e-03_rb,0.10751e-03_rb,0.10606e-03_rb,0.10174e-03_rb, &
        & 0.94756e-04_rb,0.84487e-04_rb,0.67267e-04_rb,0.20037e-04_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.85286e-04_rb,0.11563e-03_rb,0.11966e-03_rb,0.11777e-03_rb,0.11259e-03_rb, &
        & 0.10435e-03_rb,0.92076e-04_rb,0.72492e-04_rb,0.22229e-04_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.96137e-04_rb,0.12836e-03_rb,0.13265e-03_rb,0.13031e-03_rb,0.12404e-03_rb, &
        & 0.11409e-03_rb,0.99756e-04_rb,0.77897e-04_rb,0.24656e-04_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.10739e-03_rb,0.14209e-03_rb,0.14652e-03_rb,0.14363e-03_rb,0.13579e-03_rb, &
        & 0.12408e-03_rb,0.10749e-03_rb,0.83273e-04_rb,0.26998e-04_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.57005e-04_rb,0.80688e-04_rb,0.84199e-04_rb,0.83067e-04_rb,0.79480e-04_rb, &
        & 0.74096e-04_rb,0.66328e-04_rb,0.53532e-04_rb,0.14534e-04_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.64518e-04_rb,0.90274e-04_rb,0.94149e-04_rb,0.92496e-04_rb,0.88374e-04_rb, &
        & 0.82048e-04_rb,0.72698e-04_rb,0.57871e-04_rb,0.16202e-04_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.73014e-04_rb,0.10065e-03_rb,0.10466e-03_rb,0.10271e-03_rb,0.97880e-04_rb, &
        & 0.90372e-04_rb,0.79361e-04_rb,0.62357e-04_rb,0.18056e-04_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.82297e-04_rb,0.11195e-03_rb,0.11586e-03_rb,0.11365e-03_rb,0.10799e-03_rb, &
        & 0.98829e-04_rb,0.85923e-04_rb,0.66891e-04_rb,0.19890e-04_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.92175e-04_rb,0.12399e-03_rb,0.12794e-03_rb,0.12531e-03_rb,0.11825e-03_rb, &
        & 0.10764e-03_rb,0.92744e-04_rb,0.71608e-04_rb,0.21968e-04_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.47324e-04_rb,0.68167e-04_rb,0.71069e-04_rb,0.70293e-04_rb,0.67313e-04_rb, &
        & 0.62456e-04_rb,0.55646e-04_rb,0.44933e-04_rb,0.11374e-04_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.53658e-04_rb,0.76342e-04_rb,0.79560e-04_rb,0.78547e-04_rb,0.74997e-04_rb, &
        & 0.69264e-04_rb,0.61119e-04_rb,0.48587e-04_rb,0.12639e-04_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.60905e-04_rb,0.85247e-04_rb,0.88738e-04_rb,0.87387e-04_rb,0.83098e-04_rb, &
        & 0.76432e-04_rb,0.66733e-04_rb,0.52275e-04_rb,0.13871e-04_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.68831e-04_rb,0.94960e-04_rb,0.98638e-04_rb,0.96778e-04_rb,0.91816e-04_rb, &
        & 0.83747e-04_rb,0.72403e-04_rb,0.56165e-04_rb,0.15423e-04_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.77259e-04_rb,0.10549e-03_rb,0.10909e-03_rb,0.10672e-03_rb,0.10065e-03_rb, &
        & 0.91320e-04_rb,0.78406e-04_rb,0.60157e-04_rb,0.17141e-04_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.40102e-04_rb,0.56990e-04_rb,0.59444e-04_rb,0.58732e-04_rb,0.56091e-04_rb, &
        & 0.51902e-04_rb,0.46247e-04_rb,0.37442e-04_rb,0.94022e-05_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.44930e-04_rb,0.63893e-04_rb,0.66629e-04_rb,0.65694e-04_rb,0.62627e-04_rb, &
        & 0.57822e-04_rb,0.50819e-04_rb,0.40421e-04_rb,0.10475e-04_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.50532e-04_rb,0.71424e-04_rb,0.74352e-04_rb,0.73221e-04_rb,0.69651e-04_rb, &
        & 0.64034e-04_rb,0.55430e-04_rb,0.43529e-04_rb,0.11728e-04_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.56953e-04_rb,0.79740e-04_rb,0.82769e-04_rb,0.81314e-04_rb,0.77278e-04_rb, &
        & 0.70324e-04_rb,0.60396e-04_rb,0.46747e-04_rb,0.13063e-04_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.64019e-04_rb,0.88647e-04_rb,0.91768e-04_rb,0.89926e-04_rb,0.84958e-04_rb, &
        & 0.76740e-04_rb,0.65561e-04_rb,0.50066e-04_rb,0.14435e-04_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.33806e-04_rb,0.47826e-04_rb,0.49443e-04_rb,0.48775e-04_rb,0.46477e-04_rb, &
        & 0.42791e-04_rb,0.38155e-04_rb,0.31175e-04_rb,0.77886e-05_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.37973e-04_rb,0.53392e-04_rb,0.55526e-04_rb,0.54719e-04_rb,0.52062e-04_rb, &
        & 0.47763e-04_rb,0.42021e-04_rb,0.33510e-04_rb,0.87404e-05_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.42758e-04_rb,0.59666e-04_rb,0.62058e-04_rb,0.61127e-04_rb,0.58012e-04_rb, &
        & 0.53071e-04_rb,0.45971e-04_rb,0.36114e-04_rb,0.98499e-05_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.48006e-04_rb,0.66679e-04_rb,0.69230e-04_rb,0.67989e-04_rb,0.64400e-04_rb, &
        & 0.58521e-04_rb,0.50196e-04_rb,0.38710e-04_rb,0.11022e-04_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.53657e-04_rb,0.74244e-04_rb,0.76813e-04_rb,0.75303e-04_rb,0.71011e-04_rb, &
        & 0.64092e-04_rb,0.54599e-04_rb,0.41481e-04_rb,0.12225e-04_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.27953e-04_rb,0.40223e-04_rb,0.41331e-04_rb,0.40332e-04_rb,0.38114e-04_rb, &
        & 0.34967e-04_rb,0.31158e-04_rb,0.25701e-04_rb,0.67604e-05_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.31595e-04_rb,0.44975e-04_rb,0.46374e-04_rb,0.45229e-04_rb,0.42775e-04_rb, &
        & 0.39162e-04_rb,0.34373e-04_rb,0.27616e-04_rb,0.75558e-05_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.35649e-04_rb,0.50263e-04_rb,0.51713e-04_rb,0.50516e-04_rb,0.47855e-04_rb, &
        & 0.43605e-04_rb,0.37705e-04_rb,0.29800e-04_rb,0.84063e-05_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.40086e-04_rb,0.55905e-04_rb,0.57584e-04_rb,0.56348e-04_rb,0.53200e-04_rb, &
        & 0.48283e-04_rb,0.41331e-04_rb,0.31936e-04_rb,0.92686e-05_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.44931e-04_rb,0.62025e-04_rb,0.63942e-04_rb,0.62498e-04_rb,0.58788e-04_rb, &
        & 0.53032e-04_rb,0.45085e-04_rb,0.34229e-04_rb,0.10175e-04_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.22829e-04_rb,0.33278e-04_rb,0.34196e-04_rb,0.33381e-04_rb,0.31444e-04_rb, &
        & 0.28615e-04_rb,0.25331e-04_rb,0.21096e-04_rb,0.60732e-05_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.25917e-04_rb,0.37424e-04_rb,0.38527e-04_rb,0.37580e-04_rb,0.35302e-04_rb, &
        & 0.32020e-04_rb,0.27957e-04_rb,0.22660e-04_rb,0.68180e-05_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.29386e-04_rb,0.41933e-04_rb,0.43213e-04_rb,0.42039e-04_rb,0.39445e-04_rb, &
        & 0.35659e-04_rb,0.30780e-04_rb,0.24432e-04_rb,0.76296e-05_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.33186e-04_rb,0.46811e-04_rb,0.48218e-04_rb,0.46849e-04_rb,0.43848e-04_rb, &
        & 0.39567e-04_rb,0.33858e-04_rb,0.26257e-04_rb,0.82634e-05_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.37227e-04_rb,0.52060e-04_rb,0.53592e-04_rb,0.51948e-04_rb,0.48471e-04_rb, &
        & 0.43595e-04_rb,0.37022e-04_rb,0.28130e-04_rb,0.90053e-05_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.18333e-04_rb,0.27259e-04_rb,0.27929e-04_rb,0.27230e-04_rb,0.25672e-04_rb, &
        & 0.23374e-04_rb,0.20571e-04_rb,0.17333e-04_rb,0.54157e-05_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.20948e-04_rb,0.30774e-04_rb,0.31604e-04_rb,0.30823e-04_rb,0.29002e-04_rb, &
        & 0.26292e-04_rb,0.22734e-04_rb,0.18593e-04_rb,0.61669e-05_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.23883e-04_rb,0.34642e-04_rb,0.35656e-04_rb,0.34670e-04_rb,0.32507e-04_rb, &
        & 0.29303e-04_rb,0.25111e-04_rb,0.19985e-04_rb,0.69048e-05_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.27098e-04_rb,0.38802e-04_rb,0.40013e-04_rb,0.38777e-04_rb,0.36214e-04_rb, &
        & 0.32542e-04_rb,0.27674e-04_rb,0.21526e-04_rb,0.76591e-05_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.30462e-04_rb,0.43307e-04_rb,0.44596e-04_rb,0.43127e-04_rb,0.40142e-04_rb, &
        & 0.35890e-04_rb,0.30276e-04_rb,0.23065e-04_rb,0.84480e-05_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.14510e-04_rb,0.22049e-04_rb,0.22596e-04_rb,0.22021e-04_rb,0.20750e-04_rb, &
        & 0.18895e-04_rb,0.16685e-04_rb,0.14158e-04_rb,0.50284e-05_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.16710e-04_rb,0.25012e-04_rb,0.25686e-04_rb,0.25017e-04_rb,0.23537e-04_rb, &
        & 0.21350e-04_rb,0.18517e-04_rb,0.15170e-04_rb,0.59016e-05_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.19138e-04_rb,0.28271e-04_rb,0.29104e-04_rb,0.28272e-04_rb,0.26493e-04_rb, &
        & 0.23916e-04_rb,0.20530e-04_rb,0.16297e-04_rb,0.68413e-05_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.21798e-04_rb,0.31859e-04_rb,0.32802e-04_rb,0.31762e-04_rb,0.29603e-04_rb, &
        & 0.26643e-04_rb,0.22650e-04_rb,0.17559e-04_rb,0.78093e-05_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.24599e-04_rb,0.35618e-04_rb,0.36736e-04_rb,0.35456e-04_rb,0.32981e-04_rb, &
        & 0.29459e-04_rb,0.24877e-04_rb,0.18821e-04_rb,0.88149e-05_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.11455e-04_rb,0.17796e-04_rb,0.18282e-04_rb,0.17833e-04_rb,0.16814e-04_rb, &
        & 0.15305e-04_rb,0.13526e-04_rb,0.11628e-04_rb,0.47637e-05_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.13295e-04_rb,0.20290e-04_rb,0.20862e-04_rb,0.20316e-04_rb,0.19125e-04_rb, &
        & 0.17348e-04_rb,0.15042e-04_rb,0.12455e-04_rb,0.56495e-05_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.15297e-04_rb,0.22994e-04_rb,0.23740e-04_rb,0.23076e-04_rb,0.21626e-04_rb, &
        & 0.19529e-04_rb,0.16766e-04_rb,0.13399e-04_rb,0.65988e-05_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.17488e-04_rb,0.25986e-04_rb,0.26882e-04_rb,0.26021e-04_rb,0.24246e-04_rb, &
        & 0.21820e-04_rb,0.18554e-04_rb,0.14427e-04_rb,0.74644e-05_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.19800e-04_rb,0.29160e-04_rb,0.30208e-04_rb,0.29161e-04_rb,0.27086e-04_rb, &
        & 0.24189e-04_rb,0.20462e-04_rb,0.15474e-04_rb,0.84395e-05_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.96366e-05_rb,0.15168e-04_rb,0.15611e-04_rb,0.15230e-04_rb,0.14381e-04_rb, &
        & 0.13058e-04_rb,0.11457e-04_rb,0.98195e-05_rb,0.43210e-05_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.11221e-04_rb,0.17304e-04_rb,0.17859e-04_rb,0.17390e-04_rb,0.16383e-04_rb, &
        & 0.14837e-04_rb,0.12820e-04_rb,0.10541e-04_rb,0.50915e-05_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.12922e-04_rb,0.19611e-04_rb,0.20376e-04_rb,0.19769e-04_rb,0.18510e-04_rb, &
        & 0.16733e-04_rb,0.14297e-04_rb,0.11375e-04_rb,0.59349e-05_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.14759e-04_rb,0.22226e-04_rb,0.23087e-04_rb,0.22319e-04_rb,0.20789e-04_rb, &
        & 0.18677e-04_rb,0.15854e-04_rb,0.12233e-04_rb,0.68646e-05_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.16687e-04_rb,0.24901e-04_rb,0.25894e-04_rb,0.25020e-04_rb,0.23234e-04_rb, &
        & 0.20746e-04_rb,0.17450e-04_rb,0.13143e-04_rb,0.77945e-05_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.80728e-05_rb,0.12833e-04_rb,0.13241e-04_rb,0.12930e-04_rb,0.12209e-04_rb, &
        & 0.11077e-04_rb,0.96874e-05_rb,0.82518e-05_rb,0.36987e-05_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.94096e-05_rb,0.14647e-04_rb,0.15179e-04_rb,0.14789e-04_rb,0.13929e-04_rb, &
        & 0.12614e-04_rb,0.10863e-04_rb,0.88839e-05_rb,0.43642e-05_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.10827e-04_rb,0.16625e-04_rb,0.17336e-04_rb,0.16825e-04_rb,0.15757e-04_rb, &
        & 0.14215e-04_rb,0.12135e-04_rb,0.95702e-05_rb,0.51276e-05_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.12346e-04_rb,0.18823e-04_rb,0.19626e-04_rb,0.19011e-04_rb,0.17715e-04_rb, &
        & 0.15891e-04_rb,0.13472e-04_rb,0.10297e-04_rb,0.58818e-05_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.13954e-04_rb,0.21071e-04_rb,0.21985e-04_rb,0.21278e-04_rb,0.19773e-04_rb, &
        & 0.17642e-04_rb,0.14813e-04_rb,0.11109e-04_rb,0.66485e-05_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.67198e-05_rb,0.10777e-04_rb,0.11153e-04_rb,0.10900e-04_rb,0.10301e-04_rb, &
        & 0.93345e-05_rb,0.81433e-05_rb,0.69156e-05_rb,0.30498e-05_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.78283e-05_rb,0.12283e-04_rb,0.12805e-04_rb,0.12492e-04_rb,0.11757e-04_rb, &
        & 0.10649e-04_rb,0.91519e-05_rb,0.74495e-05_rb,0.36028e-05_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.89966e-05_rb,0.13983e-04_rb,0.14629e-04_rb,0.14218e-04_rb,0.13315e-04_rb, &
        & 0.11990e-04_rb,0.10230e-04_rb,0.80237e-05_rb,0.42382e-05_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.10250e-04_rb,0.15794e-04_rb,0.16536e-04_rb,0.16055e-04_rb,0.14979e-04_rb, &
        & 0.13421e-04_rb,0.11355e-04_rb,0.86371e-05_rb,0.48511e-05_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.11575e-04_rb,0.17689e-04_rb,0.18502e-04_rb,0.17950e-04_rb,0.16689e-04_rb, &
        & 0.14883e-04_rb,0.12476e-04_rb,0.93537e-05_rb,0.54799e-05_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.20313e-03_rb,0.26410e-03_rb,0.27054e-03_rb,0.25842e-03_rb,0.23794e-03_rb, &
        & 0.21033e-03_rb,0.17730e-03_rb,0.13831e-03_rb,0.52995e-04_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.22722e-03_rb,0.29058e-03_rb,0.29572e-03_rb,0.28200e-03_rb,0.25898e-03_rb, &
        & 0.22814e-03_rb,0.19209e-03_rb,0.14957e-03_rb,0.59096e-04_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.25054e-03_rb,0.31580e-03_rb,0.31991e-03_rb,0.30486e-03_rb,0.27955e-03_rb, &
        & 0.24634e-03_rb,0.20787e-03_rb,0.16179e-03_rb,0.65118e-04_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.27197e-03_rb,0.33971e-03_rb,0.34310e-03_rb,0.32677e-03_rb,0.30002e-03_rb, &
        & 0.26574e-03_rb,0.22502e-03_rb,0.17520e-03_rb,0.71711e-04_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.29178e-03_rb,0.36213e-03_rb,0.36503e-03_rb,0.34815e-03_rb,0.32114e-03_rb, &
        & 0.28599e-03_rb,0.24328e-03_rb,0.18958e-03_rb,0.78523e-04_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.17547e-03_rb,0.23012e-03_rb,0.23639e-03_rb,0.22606e-03_rb,0.20799e-03_rb, &
        & 0.18326e-03_rb,0.15385e-03_rb,0.11888e-03_rb,0.43633e-04_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.19601e-03_rb,0.25269e-03_rb,0.25751e-03_rb,0.24629e-03_rb,0.22623e-03_rb, &
        & 0.19926e-03_rb,0.16712e-03_rb,0.12877e-03_rb,0.48382e-04_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.21598e-03_rb,0.27423e-03_rb,0.27812e-03_rb,0.26576e-03_rb,0.24403e-03_rb, &
        & 0.21552e-03_rb,0.18141e-03_rb,0.13982e-03_rb,0.53560e-04_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.23449e-03_rb,0.29482e-03_rb,0.29820e-03_rb,0.28454e-03_rb,0.26171e-03_rb, &
        & 0.23254e-03_rb,0.19681e-03_rb,0.15192e-03_rb,0.59054e-04_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.25198e-03_rb,0.31408e-03_rb,0.31723e-03_rb,0.30341e-03_rb,0.28040e-03_rb, &
        & 0.25022e-03_rb,0.21333e-03_rb,0.16487e-03_rb,0.64779e-04_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.14736e-03_rb,0.19483e-03_rb,0.20065e-03_rb,0.19242e-03_rb,0.17728e-03_rb, &
        & 0.15627e-03_rb,0.13078e-03_rb,0.10037e-03_rb,0.34217e-04_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.16492e-03_rb,0.21441e-03_rb,0.21892e-03_rb,0.20980e-03_rb,0.19289e-03_rb, &
        & 0.17000e-03_rb,0.14249e-03_rb,0.10902e-03_rb,0.38352e-04_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.18200e-03_rb,0.23294e-03_rb,0.23658e-03_rb,0.22658e-03_rb,0.20822e-03_rb, &
        & 0.18391e-03_rb,0.15484e-03_rb,0.11888e-03_rb,0.42700e-04_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.19811e-03_rb,0.25094e-03_rb,0.25385e-03_rb,0.24276e-03_rb,0.22338e-03_rb, &
        & 0.19841e-03_rb,0.16822e-03_rb,0.12943e-03_rb,0.47257e-04_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.21352e-03_rb,0.26784e-03_rb,0.27027e-03_rb,0.25917e-03_rb,0.23966e-03_rb, &
        & 0.21399e-03_rb,0.18239e-03_rb,0.14058e-03_rb,0.51997e-04_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.12070e-03_rb,0.16282e-03_rb,0.16870e-03_rb,0.16215e-03_rb,0.14966e-03_rb, &
        & 0.13195e-03_rb,0.11003e-03_rb,0.83743e-04_rb,0.26841e-04_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.13616e-03_rb,0.17979e-03_rb,0.18420e-03_rb,0.17692e-03_rb,0.16288e-03_rb, &
        & 0.14351e-03_rb,0.12000e-03_rb,0.91324e-04_rb,0.29956e-04_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.15142e-03_rb,0.19606e-03_rb,0.19933e-03_rb,0.19132e-03_rb,0.17582e-03_rb, &
        & 0.15506e-03_rb,0.13061e-03_rb,0.99881e-04_rb,0.33175e-04_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.16591e-03_rb,0.21170e-03_rb,0.21421e-03_rb,0.20525e-03_rb,0.18888e-03_rb, &
        & 0.16747e-03_rb,0.14192e-03_rb,0.10907e-03_rb,0.36625e-04_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.17956e-03_rb,0.22655e-03_rb,0.22833e-03_rb,0.21949e-03_rb,0.20283e-03_rb, &
        & 0.18086e-03_rb,0.15393e-03_rb,0.11865e-03_rb,0.40513e-04_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.98625e-04_rb,0.13480e-03_rb,0.14099e-03_rb,0.13653e-03_rb,0.12600e-03_rb, &
        & 0.11107e-03_rb,0.92102e-04_rb,0.69414e-04_rb,0.22597e-04_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.11165e-03_rb,0.14975e-03_rb,0.15429e-03_rb,0.14886e-03_rb,0.13710e-03_rb, &
        & 0.12076e-03_rb,0.10051e-03_rb,0.76046e-04_rb,0.25265e-04_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.12461e-03_rb,0.16391e-03_rb,0.16725e-03_rb,0.16091e-03_rb,0.14796e-03_rb, &
        & 0.13047e-03_rb,0.10941e-03_rb,0.83193e-04_rb,0.27962e-04_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.13738e-03_rb,0.17761e-03_rb,0.17982e-03_rb,0.17281e-03_rb,0.15916e-03_rb, &
        & 0.14087e-03_rb,0.11892e-03_rb,0.90972e-04_rb,0.30855e-04_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.14967e-03_rb,0.19066e-03_rb,0.19231e-03_rb,0.18509e-03_rb,0.17109e-03_rb, &
        & 0.15227e-03_rb,0.12906e-03_rb,0.99136e-04_rb,0.34017e-04_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.79863e-04_rb,0.11034e-03_rb,0.11622e-03_rb,0.11400e-03_rb,0.10532e-03_rb, &
        & 0.92978e-04_rb,0.76661e-04_rb,0.57287e-04_rb,0.18664e-04_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.90753e-04_rb,0.12318e-03_rb,0.12766e-03_rb,0.12440e-03_rb,0.11482e-03_rb, &
        & 0.10110e-03_rb,0.83697e-04_rb,0.62663e-04_rb,0.20928e-04_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.10183e-03_rb,0.13532e-03_rb,0.13907e-03_rb,0.13464e-03_rb,0.12396e-03_rb, &
        & 0.10925e-03_rb,0.90984e-04_rb,0.68513e-04_rb,0.23264e-04_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.11293e-03_rb,0.14742e-03_rb,0.14995e-03_rb,0.14481e-03_rb,0.13354e-03_rb, &
        & 0.11797e-03_rb,0.98845e-04_rb,0.74965e-04_rb,0.25850e-04_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.12360e-03_rb,0.15914e-03_rb,0.16090e-03_rb,0.15521e-03_rb,0.14361e-03_rb, &
        & 0.12755e-03_rb,0.10739e-03_rb,0.81915e-04_rb,0.28635e-04_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.65679e-04_rb,0.90084e-04_rb,0.95681e-04_rb,0.94567e-04_rb,0.87338e-04_rb, &
        & 0.77275e-04_rb,0.63642e-04_rb,0.47097e-04_rb,0.15508e-04_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.74551e-04_rb,0.10089e-03_rb,0.10531e-03_rb,0.10327e-03_rb,0.95364e-04_rb, &
        & 0.84154e-04_rb,0.69569e-04_rb,0.51426e-04_rb,0.17234e-04_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.83584e-04_rb,0.11137e-03_rb,0.11469e-03_rb,0.11193e-03_rb,0.10325e-03_rb, &
        & 0.91075e-04_rb,0.75479e-04_rb,0.56228e-04_rb,0.19127e-04_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.92762e-04_rb,0.12171e-03_rb,0.12420e-03_rb,0.12060e-03_rb,0.11147e-03_rb, &
        & 0.98465e-04_rb,0.81969e-04_rb,0.61529e-04_rb,0.21452e-04_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.10189e-03_rb,0.13182e-03_rb,0.13364e-03_rb,0.12960e-03_rb,0.12008e-03_rb, &
        & 0.10652e-03_rb,0.89045e-04_rb,0.67334e-04_rb,0.23711e-04_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.53656e-04_rb,0.74109e-04_rb,0.78629e-04_rb,0.78245e-04_rb,0.72322e-04_rb, &
        & 0.63915e-04_rb,0.52625e-04_rb,0.38591e-04_rb,0.14416e-04_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.61163e-04_rb,0.83104e-04_rb,0.86832e-04_rb,0.85562e-04_rb,0.78992e-04_rb, &
        & 0.69666e-04_rb,0.57574e-04_rb,0.42098e-04_rb,0.15921e-04_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.68914e-04_rb,0.91909e-04_rb,0.94594e-04_rb,0.92816e-04_rb,0.85590e-04_rb, &
        & 0.75491e-04_rb,0.62445e-04_rb,0.46109e-04_rb,0.17209e-04_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.76847e-04_rb,0.10059e-03_rb,0.10256e-03_rb,0.10019e-03_rb,0.92496e-04_rb, &
        & 0.81729e-04_rb,0.67763e-04_rb,0.50445e-04_rb,0.18589e-04_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.84847e-04_rb,0.10902e-03_rb,0.11071e-03_rb,0.10784e-03_rb,0.99812e-04_rb, &
        & 0.88516e-04_rb,0.73718e-04_rb,0.55250e-04_rb,0.20194e-04_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.43346e-04_rb,0.60608e-04_rb,0.64818e-04_rb,0.64681e-04_rb,0.59655e-04_rb, &
        & 0.52617e-04_rb,0.43206e-04_rb,0.31573e-04_rb,0.14696e-04_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.49669e-04_rb,0.68380e-04_rb,0.71800e-04_rb,0.70860e-04_rb,0.65260e-04_rb, &
        & 0.57496e-04_rb,0.47313e-04_rb,0.34405e-04_rb,0.16164e-04_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.56274e-04_rb,0.76040e-04_rb,0.78454e-04_rb,0.76944e-04_rb,0.70665e-04_rb, &
        & 0.62347e-04_rb,0.51383e-04_rb,0.37665e-04_rb,0.17841e-04_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.63127e-04_rb,0.83451e-04_rb,0.85178e-04_rb,0.83082e-04_rb,0.76437e-04_rb, &
        & 0.67538e-04_rb,0.55797e-04_rb,0.41264e-04_rb,0.19504e-04_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.70124e-04_rb,0.90807e-04_rb,0.91960e-04_rb,0.89432e-04_rb,0.82527e-04_rb, &
        & 0.73176e-04_rb,0.60668e-04_rb,0.45179e-04_rb,0.20992e-04_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.35059e-04_rb,0.49531e-04_rb,0.53376e-04_rb,0.53665e-04_rb,0.49644e-04_rb, &
        & 0.43592e-04_rb,0.35609e-04_rb,0.25845e-04_rb,0.13231e-04_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.40376e-04_rb,0.56174e-04_rb,0.59502e-04_rb,0.59040e-04_rb,0.54399e-04_rb, &
        & 0.47687e-04_rb,0.39007e-04_rb,0.28189e-04_rb,0.15223e-04_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.46006e-04_rb,0.62899e-04_rb,0.65214e-04_rb,0.64178e-04_rb,0.58976e-04_rb, &
        & 0.51726e-04_rb,0.42386e-04_rb,0.30843e-04_rb,0.17241e-04_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.51905e-04_rb,0.69395e-04_rb,0.70988e-04_rb,0.69461e-04_rb,0.63801e-04_rb, &
        & 0.56040e-04_rb,0.46059e-04_rb,0.33831e-04_rb,0.19714e-04_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.57990e-04_rb,0.75737e-04_rb,0.76826e-04_rb,0.74718e-04_rb,0.68839e-04_rb, &
        & 0.60664e-04_rb,0.50105e-04_rb,0.37049e-04_rb,0.22308e-04_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.30150e-04_rb,0.42738e-04_rb,0.46011e-04_rb,0.46330e-04_rb,0.42907e-04_rb, &
        & 0.37605e-04_rb,0.30702e-04_rb,0.21962e-04_rb,0.11791e-04_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.34714e-04_rb,0.48516e-04_rb,0.51120e-04_rb,0.50726e-04_rb,0.46912e-04_rb, &
        & 0.41033e-04_rb,0.33505e-04_rb,0.23952e-04_rb,0.13242e-04_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.39649e-04_rb,0.54325e-04_rb,0.56146e-04_rb,0.55270e-04_rb,0.50902e-04_rb, &
        & 0.44559e-04_rb,0.36351e-04_rb,0.26266e-04_rb,0.15075e-04_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.44779e-04_rb,0.59893e-04_rb,0.61166e-04_rb,0.59766e-04_rb,0.55070e-04_rb, &
        & 0.48303e-04_rb,0.39491e-04_rb,0.28847e-04_rb,0.17164e-04_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.49946e-04_rb,0.65411e-04_rb,0.66256e-04_rb,0.64345e-04_rb,0.59474e-04_rb, &
        & 0.52287e-04_rb,0.43003e-04_rb,0.31612e-04_rb,0.19542e-04_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.25681e-04_rb,0.36622e-04_rb,0.39428e-04_rb,0.39582e-04_rb,0.36786e-04_rb, &
        & 0.32210e-04_rb,0.26270e-04_rb,0.18629e-04_rb,0.10035e-04_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.29636e-04_rb,0.41620e-04_rb,0.43701e-04_rb,0.43425e-04_rb,0.40150e-04_rb, &
        & 0.35105e-04_rb,0.28637e-04_rb,0.20365e-04_rb,0.11416e-04_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.33921e-04_rb,0.46628e-04_rb,0.48062e-04_rb,0.47301e-04_rb,0.43633e-04_rb, &
        & 0.38189e-04_rb,0.31133e-04_rb,0.22405e-04_rb,0.12946e-04_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.38341e-04_rb,0.51401e-04_rb,0.52457e-04_rb,0.51129e-04_rb,0.47238e-04_rb, &
        & 0.41429e-04_rb,0.33905e-04_rb,0.24546e-04_rb,0.14770e-04_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.42576e-04_rb,0.56153e-04_rb,0.56863e-04_rb,0.55182e-04_rb,0.51113e-04_rb, &
        & 0.44940e-04_rb,0.36987e-04_rb,0.26898e-04_rb,0.16865e-04_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.21707e-04_rb,0.31142e-04_rb,0.33487e-04_rb,0.33650e-04_rb,0.31301e-04_rb, &
        & 0.27396e-04_rb,0.22352e-04_rb,0.15744e-04_rb,0.83005e-05_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.25112e-04_rb,0.35474e-04_rb,0.37158e-04_rb,0.36970e-04_rb,0.34170e-04_rb, &
        & 0.29871e-04_rb,0.24361e-04_rb,0.17312e-04_rb,0.93559e-05_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.28762e-04_rb,0.39712e-04_rb,0.40924e-04_rb,0.40199e-04_rb,0.37169e-04_rb, &
        & 0.32565e-04_rb,0.26566e-04_rb,0.19074e-04_rb,0.10670e-04_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.32448e-04_rb,0.43808e-04_rb,0.44734e-04_rb,0.43534e-04_rb,0.40323e-04_rb, &
        & 0.35367e-04_rb,0.29011e-04_rb,0.20897e-04_rb,0.12240e-04_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.35905e-04_rb,0.47771e-04_rb,0.48482e-04_rb,0.47094e-04_rb,0.43703e-04_rb, &
        & 0.38474e-04_rb,0.31715e-04_rb,0.22899e-04_rb,0.13958e-04_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.50328e-03_rb,0.59028e-03_rb,0.59973e-03_rb,0.58955e-03_rb,0.56685e-03_rb, &
        & 0.50893e-03_rb,0.43098e-03_rb,0.31518e-03_rb,0.12599e-03_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.53454e-03_rb,0.62581e-03_rb,0.63729e-03_rb,0.62588e-03_rb,0.59902e-03_rb, &
        & 0.54179e-03_rb,0.45928e-03_rb,0.33824e-03_rb,0.13937e-03_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.56216e-03_rb,0.66315e-03_rb,0.67707e-03_rb,0.66553e-03_rb,0.63545e-03_rb, &
        & 0.57719e-03_rb,0.48913e-03_rb,0.36332e-03_rb,0.15548e-03_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.58821e-03_rb,0.70231e-03_rb,0.71934e-03_rb,0.70828e-03_rb,0.67637e-03_rb, &
        & 0.61438e-03_rb,0.52042e-03_rb,0.38978e-03_rb,0.17254e-03_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.61349e-03_rb,0.74309e-03_rb,0.76503e-03_rb,0.75486e-03_rb,0.71943e-03_rb, &
        & 0.65213e-03_rb,0.55331e-03_rb,0.41818e-03_rb,0.19176e-03_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.43720e-03_rb,0.51117e-03_rb,0.51919e-03_rb,0.50988e-03_rb,0.48875e-03_rb, &
        & 0.44138e-03_rb,0.37418e-03_rb,0.27478e-03_rb,0.10374e-03_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.46400e-03_rb,0.54460e-03_rb,0.55429e-03_rb,0.54395e-03_rb,0.52067e-03_rb, &
        & 0.47118e-03_rb,0.39949e-03_rb,0.29546e-03_rb,0.11585e-03_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.48798e-03_rb,0.57940e-03_rb,0.59177e-03_rb,0.58113e-03_rb,0.55553e-03_rb, &
        & 0.50290e-03_rb,0.42588e-03_rb,0.31727e-03_rb,0.12872e-03_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.51088e-03_rb,0.61529e-03_rb,0.63115e-03_rb,0.62163e-03_rb,0.59296e-03_rb, &
        & 0.53614e-03_rb,0.45317e-03_rb,0.34070e-03_rb,0.14334e-03_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.53331e-03_rb,0.65233e-03_rb,0.67270e-03_rb,0.66363e-03_rb,0.63091e-03_rb, &
        & 0.57023e-03_rb,0.48253e-03_rb,0.36597e-03_rb,0.15947e-03_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.37168e-03_rb,0.43544e-03_rb,0.44351e-03_rb,0.43496e-03_rb,0.41418e-03_rb, &
        & 0.37673e-03_rb,0.31849e-03_rb,0.23373e-03_rb,0.83035e-04_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.39476e-03_rb,0.46532e-03_rb,0.47527e-03_rb,0.46587e-03_rb,0.44394e-03_rb, &
        & 0.40288e-03_rb,0.34052e-03_rb,0.25172e-03_rb,0.92363e-04_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.41559e-03_rb,0.49597e-03_rb,0.50880e-03_rb,0.49944e-03_rb,0.47605e-03_rb, &
        & 0.43138e-03_rb,0.36384e-03_rb,0.27063e-03_rb,0.10327e-03_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.43551e-03_rb,0.52738e-03_rb,0.54357e-03_rb,0.53553e-03_rb,0.50924e-03_rb, &
        & 0.46069e-03_rb,0.38844e-03_rb,0.29089e-03_rb,0.11548e-03_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.45509e-03_rb,0.56036e-03_rb,0.58045e-03_rb,0.57279e-03_rb,0.54320e-03_rb, &
        & 0.49088e-03_rb,0.41461e-03_rb,0.31304e-03_rb,0.12901e-03_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.31353e-03_rb,0.36800e-03_rb,0.37478e-03_rb,0.36628e-03_rb,0.34818e-03_rb, &
        & 0.31938e-03_rb,0.26882e-03_rb,0.19725e-03_rb,0.65143e-04_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.33327e-03_rb,0.39347e-03_rb,0.40240e-03_rb,0.39421e-03_rb,0.37500e-03_rb, &
        & 0.34245e-03_rb,0.28782e-03_rb,0.21247e-03_rb,0.72911e-04_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.35116e-03_rb,0.41935e-03_rb,0.43125e-03_rb,0.42396e-03_rb,0.40374e-03_rb, &
        & 0.36745e-03_rb,0.30851e-03_rb,0.22861e-03_rb,0.82223e-04_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.36817e-03_rb,0.44654e-03_rb,0.46141e-03_rb,0.45505e-03_rb,0.43257e-03_rb, &
        & 0.39339e-03_rb,0.33051e-03_rb,0.24594e-03_rb,0.92524e-04_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.38502e-03_rb,0.47537e-03_rb,0.49387e-03_rb,0.48730e-03_rb,0.46233e-03_rb, &
        & 0.42007e-03_rb,0.35374e-03_rb,0.26504e-03_rb,0.10363e-03_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.26371e-03_rb,0.30958e-03_rb,0.31480e-03_rb,0.30643e-03_rb,0.29119e-03_rb, &
        & 0.26781e-03_rb,0.22601e-03_rb,0.16580e-03_rb,0.51526e-04_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.28046e-03_rb,0.33149e-03_rb,0.33831e-03_rb,0.33074e-03_rb,0.31455e-03_rb, &
        & 0.28912e-03_rb,0.24254e-03_rb,0.17884e-03_rb,0.57544e-04_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.29571e-03_rb,0.35368e-03_rb,0.36281e-03_rb,0.35630e-03_rb,0.33882e-03_rb, &
        & 0.31097e-03_rb,0.26070e-03_rb,0.19272e-03_rb,0.64519e-04_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.31016e-03_rb,0.37666e-03_rb,0.38893e-03_rb,0.38273e-03_rb,0.36412e-03_rb, &
        & 0.33364e-03_rb,0.28016e-03_rb,0.20757e-03_rb,0.72363e-04_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.32452e-03_rb,0.40122e-03_rb,0.41666e-03_rb,0.41046e-03_rb,0.39013e-03_rb, &
        & 0.35668e-03_rb,0.30056e-03_rb,0.22388e-03_rb,0.80981e-04_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.22069e-03_rb,0.25888e-03_rb,0.26323e-03_rb,0.25479e-03_rb,0.24246e-03_rb, &
        & 0.22303e-03_rb,0.18865e-03_rb,0.13826e-03_rb,0.42586e-04_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.23505e-03_rb,0.27730e-03_rb,0.28289e-03_rb,0.27510e-03_rb,0.26179e-03_rb, &
        & 0.24147e-03_rb,0.20303e-03_rb,0.14956e-03_rb,0.47728e-04_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.24799e-03_rb,0.29644e-03_rb,0.30324e-03_rb,0.29696e-03_rb,0.28190e-03_rb, &
        & 0.26059e-03_rb,0.21889e-03_rb,0.16173e-03_rb,0.53804e-04_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.26027e-03_rb,0.31607e-03_rb,0.32548e-03_rb,0.31928e-03_rb,0.30338e-03_rb, &
        & 0.27984e-03_rb,0.23536e-03_rb,0.17457e-03_rb,0.60579e-04_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.27261e-03_rb,0.33682e-03_rb,0.34889e-03_rb,0.34299e-03_rb,0.32589e-03_rb, &
        & 0.29875e-03_rb,0.25286e-03_rb,0.18852e-03_rb,0.67802e-04_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.18284e-03_rb,0.21563e-03_rb,0.21874e-03_rb,0.21147e-03_rb,0.20158e-03_rb, &
        & 0.18535e-03_rb,0.15699e-03_rb,0.11455e-03_rb,0.35095e-04_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.19537e-03_rb,0.23122e-03_rb,0.23571e-03_rb,0.22829e-03_rb,0.21760e-03_rb, &
        & 0.20013e-03_rb,0.16934e-03_rb,0.12429e-03_rb,0.39853e-04_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.20672e-03_rb,0.24731e-03_rb,0.25313e-03_rb,0.24636e-03_rb,0.23401e-03_rb, &
        & 0.21622e-03_rb,0.18276e-03_rb,0.13469e-03_rb,0.45173e-04_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.21741e-03_rb,0.26399e-03_rb,0.27153e-03_rb,0.26539e-03_rb,0.25198e-03_rb, &
        & 0.23212e-03_rb,0.19655e-03_rb,0.14593e-03_rb,0.50910e-04_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.22822e-03_rb,0.28170e-03_rb,0.29118e-03_rb,0.28544e-03_rb,0.27112e-03_rb, &
        & 0.24842e-03_rb,0.21129e-03_rb,0.15803e-03_rb,0.57288e-04_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.15136e-03_rb,0.17854e-03_rb,0.18119e-03_rb,0.17535e-03_rb,0.16721e-03_rb, &
        & 0.15392e-03_rb,0.13041e-03_rb,0.94463e-04_rb,0.29176e-04_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.16197e-03_rb,0.19183e-03_rb,0.19544e-03_rb,0.18940e-03_rb,0.18045e-03_rb, &
        & 0.16606e-03_rb,0.14094e-03_rb,0.10292e-03_rb,0.33234e-04_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.17149e-03_rb,0.20540e-03_rb,0.21055e-03_rb,0.20432e-03_rb,0.19421e-03_rb, &
        & 0.17914e-03_rb,0.15214e-03_rb,0.11170e-03_rb,0.37987e-04_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.18048e-03_rb,0.21966e-03_rb,0.22618e-03_rb,0.22017e-03_rb,0.20926e-03_rb, &
        & 0.19215e-03_rb,0.16364e-03_rb,0.12118e-03_rb,0.43249e-04_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.18968e-03_rb,0.23496e-03_rb,0.24258e-03_rb,0.23712e-03_rb,0.22524e-03_rb, &
        & 0.20589e-03_rb,0.17598e-03_rb,0.13139e-03_rb,0.48772e-04_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.12535e-03_rb,0.14733e-03_rb,0.14911e-03_rb,0.14464e-03_rb,0.13810e-03_rb, &
        & 0.12766e-03_rb,0.10804e-03_rb,0.77667e-04_rb,0.31607e-04_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.13428e-03_rb,0.15826e-03_rb,0.16122e-03_rb,0.15658e-03_rb,0.14914e-03_rb, &
        & 0.13752e-03_rb,0.11655e-03_rb,0.84956e-04_rb,0.35832e-04_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.14212e-03_rb,0.16949e-03_rb,0.17392e-03_rb,0.16905e-03_rb,0.16073e-03_rb, &
        & 0.14802e-03_rb,0.12582e-03_rb,0.92269e-04_rb,0.39990e-04_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.14955e-03_rb,0.18151e-03_rb,0.18720e-03_rb,0.18233e-03_rb,0.17343e-03_rb, &
        & 0.15845e-03_rb,0.13559e-03_rb,0.99998e-04_rb,0.44405e-04_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.15720e-03_rb,0.19441e-03_rb,0.20115e-03_rb,0.19658e-03_rb,0.18679e-03_rb, &
        & 0.17045e-03_rb,0.14597e-03_rb,0.10853e-03_rb,0.48354e-04_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.10518e-03_rb,0.12283e-03_rb,0.12342e-03_rb,0.11933e-03_rb,0.11376e-03_rb, &
        & 0.10615e-03_rb,0.89610e-04_rb,0.64322e-04_rb,0.38897e-04_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.11277e-03_rb,0.13167e-03_rb,0.13319e-03_rb,0.12937e-03_rb,0.12328e-03_rb, &
        & 0.11434e-03_rb,0.96637e-04_rb,0.70434e-04_rb,0.44697e-04_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.11946e-03_rb,0.14071e-03_rb,0.14389e-03_rb,0.14002e-03_rb,0.13292e-03_rb, &
        & 0.12246e-03_rb,0.10436e-03_rb,0.76473e-04_rb,0.50672e-04_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.12573e-03_rb,0.15045e-03_rb,0.15515e-03_rb,0.15121e-03_rb,0.14368e-03_rb, &
        & 0.13136e-03_rb,0.11260e-03_rb,0.82864e-04_rb,0.56090e-04_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.13210e-03_rb,0.16128e-03_rb,0.16701e-03_rb,0.16339e-03_rb,0.15508e-03_rb, &
        & 0.14154e-03_rb,0.12132e-03_rb,0.89973e-04_rb,0.61017e-04_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.90693e-04_rb,0.10606e-03_rb,0.10639e-03_rb,0.10214e-03_rb,0.97041e-04_rb, &
        & 0.90929e-04_rb,0.76654e-04_rb,0.55448e-04_rb,0.34971e-04_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.96878e-04_rb,0.11346e-03_rb,0.11478e-03_rb,0.11094e-03_rb,0.10496e-03_rb, &
        & 0.97427e-04_rb,0.82680e-04_rb,0.60468e-04_rb,0.41599e-04_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.10232e-03_rb,0.12125e-03_rb,0.12365e-03_rb,0.11979e-03_rb,0.11346e-03_rb, &
        & 0.10430e-03_rb,0.89390e-04_rb,0.65549e-04_rb,0.48186e-04_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.10772e-03_rb,0.12969e-03_rb,0.13316e-03_rb,0.12963e-03_rb,0.12290e-03_rb, &
        & 0.11234e-03_rb,0.96526e-04_rb,0.71097e-04_rb,0.54537e-04_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.11345e-03_rb,0.13912e-03_rb,0.14339e-03_rb,0.14025e-03_rb,0.13275e-03_rb, &
        & 0.12106e-03_rb,0.10412e-03_rb,0.77240e-04_rb,0.60238e-04_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.77577e-04_rb,0.91005e-04_rb,0.91202e-04_rb,0.87618e-04_rb,0.82974e-04_rb, &
        & 0.77488e-04_rb,0.65261e-04_rb,0.47539e-04_rb,0.30516e-04_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.82612e-04_rb,0.97349e-04_rb,0.98497e-04_rb,0.95069e-04_rb,0.89699e-04_rb, &
        & 0.82714e-04_rb,0.70478e-04_rb,0.51645e-04_rb,0.35968e-04_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.87206e-04_rb,0.10402e-03_rb,0.10625e-03_rb,0.10294e-03_rb,0.97013e-04_rb, &
        & 0.88723e-04_rb,0.76259e-04_rb,0.55960e-04_rb,0.41899e-04_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.91929e-04_rb,0.11153e-03_rb,0.11462e-03_rb,0.11153e-03_rb,0.10500e-03_rb, &
        & 0.95633e-04_rb,0.82267e-04_rb,0.60868e-04_rb,0.48100e-04_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.97101e-04_rb,0.11996e-03_rb,0.12377e-03_rb,0.12058e-03_rb,0.11342e-03_rb, &
        & 0.10316e-03_rb,0.88884e-04_rb,0.66077e-04_rb,0.53563e-04_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.65838e-04_rb,0.77620e-04_rb,0.77829e-04_rb,0.74661e-04_rb,0.70473e-04_rb, &
        & 0.65575e-04_rb,0.55474e-04_rb,0.40441e-04_rb,0.25521e-04_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.69998e-04_rb,0.83015e-04_rb,0.84103e-04_rb,0.81066e-04_rb,0.76507e-04_rb, &
        & 0.70172e-04_rb,0.59914e-04_rb,0.43882e-04_rb,0.30126e-04_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.74003e-04_rb,0.88878e-04_rb,0.90840e-04_rb,0.88139e-04_rb,0.82951e-04_rb, &
        & 0.75471e-04_rb,0.64689e-04_rb,0.47652e-04_rb,0.35171e-04_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.78234e-04_rb,0.95561e-04_rb,0.98237e-04_rb,0.95653e-04_rb,0.89770e-04_rb, &
        & 0.81434e-04_rb,0.69913e-04_rb,0.51882e-04_rb,0.40221e-04_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.82825e-04_rb,0.10313e-03_rb,0.10644e-03_rb,0.10352e-03_rb,0.97040e-04_rb, &
        & 0.87602e-04_rb,0.75532e-04_rb,0.56387e-04_rb,0.44675e-04_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.10615e-02_rb,0.12529e-02_rb,0.13083e-02_rb,0.12764e-02_rb,0.11881e-02_rb, &
        & 0.10853e-02_rb,0.93421e-03_rb,0.71644e-03_rb,0.30757e-03_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.10918e-02_rb,0.13136e-02_rb,0.13677e-02_rb,0.13350e-02_rb,0.12522e-02_rb, &
        & 0.11402e-02_rb,0.98392e-03_rb,0.76040e-03_rb,0.35125e-03_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.11297e-02_rb,0.13742e-02_rb,0.14266e-02_rb,0.13956e-02_rb,0.13143e-02_rb, &
        & 0.11951e-02_rb,0.10359e-02_rb,0.81032e-03_rb,0.39829e-03_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.11716e-02_rb,0.14330e-02_rb,0.14865e-02_rb,0.14580e-02_rb,0.13738e-02_rb, &
        & 0.12536e-02_rb,0.10931e-02_rb,0.86605e-03_rb,0.44907e-03_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.12137e-02_rb,0.14919e-02_rb,0.15477e-02_rb,0.15196e-02_rb,0.14358e-02_rb, &
        & 0.13166e-02_rb,0.11546e-02_rb,0.92748e-03_rb,0.50593e-03_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.92435e-03_rb,0.10874e-02_rb,0.11275e-02_rb,0.11015e-02_rb,0.10311e-02_rb, &
        & 0.94231e-03_rb,0.80960e-03_rb,0.63033e-03_rb,0.26663e-03_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.95885e-03_rb,0.11428e-02_rb,0.11829e-02_rb,0.11541e-02_rb,0.10849e-02_rb, &
        & 0.98930e-03_rb,0.85567e-03_rb,0.67200e-03_rb,0.30230e-03_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.99776e-03_rb,0.11978e-02_rb,0.12376e-02_rb,0.12093e-02_rb,0.11390e-02_rb, &
        & 0.10392e-02_rb,0.90558e-03_rb,0.71833e-03_rb,0.34245e-03_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.10380e-02_rb,0.12537e-02_rb,0.12935e-02_rb,0.12652e-02_rb,0.11933e-02_rb, &
        & 0.10921e-02_rb,0.95939e-03_rb,0.76948e-03_rb,0.38641e-03_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.10779e-02_rb,0.13105e-02_rb,0.13515e-02_rb,0.13214e-02_rb,0.12497e-02_rb, &
        & 0.11514e-02_rb,0.10182e-02_rb,0.82552e-03_rb,0.43428e-03_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.79958e-03_rb,0.93053e-03_rb,0.95669e-03_rb,0.93720e-03_rb,0.88165e-03_rb, &
        & 0.80178e-03_rb,0.69131e-03_rb,0.54027e-03_rb,0.22082e-03_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.83354e-03_rb,0.98064e-03_rb,0.10079e-02_rb,0.98484e-03_rb,0.92834e-03_rb, &
        & 0.84387e-03_rb,0.73304e-03_rb,0.57932e-03_rb,0.25149e-03_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.87012e-03_rb,0.10320e-02_rb,0.10600e-02_rb,0.10346e-02_rb,0.97491e-03_rb, &
        & 0.88683e-03_rb,0.77649e-03_rb,0.62208e-03_rb,0.28527e-03_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.90759e-03_rb,0.10841e-02_rb,0.11129e-02_rb,0.10858e-02_rb,0.10238e-02_rb, &
        & 0.93520e-03_rb,0.82494e-03_rb,0.66854e-03_rb,0.32199e-03_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.94443e-03_rb,0.11378e-02_rb,0.11668e-02_rb,0.11369e-02_rb,0.10746e-02_rb, &
        & 0.98835e-03_rb,0.87764e-03_rb,0.71868e-03_rb,0.36165e-03_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.68628e-03_rb,0.79104e-03_rb,0.80863e-03_rb,0.79421e-03_rb,0.74681e-03_rb, &
        & 0.67391e-03_rb,0.58210e-03_rb,0.45790e-03_rb,0.18137e-03_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.71790e-03_rb,0.83686e-03_rb,0.85591e-03_rb,0.83705e-03_rb,0.78808e-03_rb, &
        & 0.71172e-03_rb,0.61822e-03_rb,0.49245e-03_rb,0.20729e-03_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.75117e-03_rb,0.88426e-03_rb,0.90462e-03_rb,0.88167e-03_rb,0.82945e-03_rb, &
        & 0.75050e-03_rb,0.65652e-03_rb,0.53019e-03_rb,0.23455e-03_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.78430e-03_rb,0.93252e-03_rb,0.95418e-03_rb,0.92872e-03_rb,0.87301e-03_rb, &
        & 0.79337e-03_rb,0.69871e-03_rb,0.57160e-03_rb,0.26520e-03_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.81683e-03_rb,0.98168e-03_rb,0.10045e-02_rb,0.97638e-03_rb,0.92000e-03_rb, &
        & 0.84034e-03_rb,0.74480e-03_rb,0.61518e-03_rb,0.29808e-03_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.58497e-03_rb,0.66935e-03_rb,0.68301e-03_rb,0.66951e-03_rb,0.62962e-03_rb, &
        & 0.56579e-03_rb,0.48826e-03_rb,0.38379e-03_rb,0.14576e-03_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.61310e-03_rb,0.71031e-03_rb,0.72592e-03_rb,0.70978e-03_rb,0.66652e-03_rb, &
        & 0.59753e-03_rb,0.51826e-03_rb,0.41385e-03_rb,0.16772e-03_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.64179e-03_rb,0.75306e-03_rb,0.77095e-03_rb,0.75074e-03_rb,0.70458e-03_rb, &
        & 0.63276e-03_rb,0.55157e-03_rb,0.44695e-03_rb,0.19067e-03_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.67039e-03_rb,0.79634e-03_rb,0.81655e-03_rb,0.79420e-03_rb,0.74416e-03_rb, &
        & 0.67043e-03_rb,0.58781e-03_rb,0.48223e-03_rb,0.21651e-03_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.69870e-03_rb,0.83972e-03_rb,0.86240e-03_rb,0.83794e-03_rb,0.78674e-03_rb, &
        & 0.71204e-03_rb,0.62768e-03_rb,0.51780e-03_rb,0.24414e-03_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.49594e-03_rb,0.56319e-03_rb,0.57248e-03_rb,0.55950e-03_rb,0.52657e-03_rb, &
        & 0.47274e-03_rb,0.40631e-03_rb,0.31914e-03_rb,0.11737e-03_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.51946e-03_rb,0.59932e-03_rb,0.61091e-03_rb,0.59626e-03_rb,0.56011e-03_rb, &
        & 0.50076e-03_rb,0.43158e-03_rb,0.34422e-03_rb,0.13452e-03_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.54355e-03_rb,0.63601e-03_rb,0.65111e-03_rb,0.63466e-03_rb,0.59552e-03_rb, &
        & 0.53122e-03_rb,0.45996e-03_rb,0.37172e-03_rb,0.15179e-03_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.56762e-03_rb,0.67321e-03_rb,0.69130e-03_rb,0.67433e-03_rb,0.63164e-03_rb, &
        & 0.56508e-03_rb,0.49145e-03_rb,0.39979e-03_rb,0.17118e-03_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.59180e-03_rb,0.71065e-03_rb,0.73195e-03_rb,0.71355e-03_rb,0.66898e-03_rb, &
        & 0.60268e-03_rb,0.52617e-03_rb,0.43141e-03_rb,0.19263e-03_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.41932e-03_rb,0.47346e-03_rb,0.47850e-03_rb,0.46519e-03_rb,0.43628e-03_rb, &
        & 0.39132e-03_rb,0.33641e-03_rb,0.26485e-03_rb,0.10000e-03_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.43848e-03_rb,0.50411e-03_rb,0.51171e-03_rb,0.49803e-03_rb,0.46737e-03_rb, &
        & 0.41801e-03_rb,0.35878e-03_rb,0.28532e-03_rb,0.11408e-03_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.45836e-03_rb,0.53506e-03_rb,0.54578e-03_rb,0.53230e-03_rb,0.49970e-03_rb, &
        & 0.44506e-03_rb,0.38306e-03_rb,0.30828e-03_rb,0.13001e-03_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.47855e-03_rb,0.56639e-03_rb,0.58020e-03_rb,0.56722e-03_rb,0.53202e-03_rb, &
        & 0.47546e-03_rb,0.41064e-03_rb,0.33104e-03_rb,0.14676e-03_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.49885e-03_rb,0.59805e-03_rb,0.61555e-03_rb,0.60152e-03_rb,0.56466e-03_rb, &
        & 0.50884e-03_rb,0.44071e-03_rb,0.35766e-03_rb,0.16471e-03_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.35434e-03_rb,0.39784e-03_rb,0.40003e-03_rb,0.38643e-03_rb,0.36029e-03_rb, &
        & 0.32228e-03_rb,0.27705e-03_rb,0.21936e-03_rb,0.85304e-04_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.36972e-03_rb,0.42325e-03_rb,0.42802e-03_rb,0.41467e-03_rb,0.38745e-03_rb, &
        & 0.34637e-03_rb,0.29590e-03_rb,0.23622e-03_rb,0.97535e-04_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.38589e-03_rb,0.44905e-03_rb,0.45611e-03_rb,0.44402e-03_rb,0.41681e-03_rb, &
        & 0.37125e-03_rb,0.31793e-03_rb,0.25523e-03_rb,0.11156e-03_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.40282e-03_rb,0.47505e-03_rb,0.48489e-03_rb,0.47391e-03_rb,0.44440e-03_rb, &
        & 0.39797e-03_rb,0.34220e-03_rb,0.27395e-03_rb,0.12650e-03_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.41971e-03_rb,0.50175e-03_rb,0.51511e-03_rb,0.50365e-03_rb,0.47246e-03_rb, &
        & 0.42646e-03_rb,0.36852e-03_rb,0.29705e-03_rb,0.14272e-03_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.29829e-03_rb,0.33364e-03_rb,0.33392e-03_rb,0.32092e-03_rb,0.29738e-03_rb, &
        & 0.26469e-03_rb,0.22642e-03_rb,0.18017e-03_rb,0.77064e-04_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.31054e-03_rb,0.35463e-03_rb,0.35704e-03_rb,0.34432e-03_rb,0.32083e-03_rb, &
        & 0.28575e-03_rb,0.24402e-03_rb,0.19441e-03_rb,0.87743e-04_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.32389e-03_rb,0.37581e-03_rb,0.38023e-03_rb,0.36873e-03_rb,0.34562e-03_rb, &
        & 0.30799e-03_rb,0.26274e-03_rb,0.21037e-03_rb,0.99662e-04_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.33808e-03_rb,0.39748e-03_rb,0.40404e-03_rb,0.39387e-03_rb,0.36968e-03_rb, &
        & 0.33141e-03_rb,0.28405e-03_rb,0.22647e-03_rb,0.11282e-03_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.35231e-03_rb,0.41983e-03_rb,0.42961e-03_rb,0.41951e-03_rb,0.39352e-03_rb, &
        & 0.35528e-03_rb,0.30599e-03_rb,0.24648e-03_rb,0.12841e-03_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.25036e-03_rb,0.27999e-03_rb,0.27949e-03_rb,0.26793e-03_rb,0.24735e-03_rb, &
        & 0.21800e-03_rb,0.18604e-03_rb,0.14807e-03_rb,0.93146e-04_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.26009e-03_rb,0.29745e-03_rb,0.29870e-03_rb,0.28727e-03_rb,0.26656e-03_rb, &
        & 0.23674e-03_rb,0.20160e-03_rb,0.16012e-03_rb,0.10408e-03_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.27084e-03_rb,0.31508e-03_rb,0.31779e-03_rb,0.30720e-03_rb,0.28751e-03_rb, &
        & 0.25636e-03_rb,0.21818e-03_rb,0.17357e-03_rb,0.11611e-03_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.28251e-03_rb,0.33335e-03_rb,0.33759e-03_rb,0.32814e-03_rb,0.30785e-03_rb, &
        & 0.27625e-03_rb,0.23573e-03_rb,0.18761e-03_rb,0.12959e-03_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.29455e-03_rb,0.35211e-03_rb,0.35903e-03_rb,0.34997e-03_rb,0.32871e-03_rb, &
        & 0.29658e-03_rb,0.25454e-03_rb,0.20476e-03_rb,0.14404e-03_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.21327e-03_rb,0.24042e-03_rb,0.24026e-03_rb,0.23068e-03_rb,0.21276e-03_rb, &
        & 0.18665e-03_rb,0.15901e-03_rb,0.12570e-03_rb,0.94295e-04_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.22127e-03_rb,0.25493e-03_rb,0.25609e-03_rb,0.24693e-03_rb,0.22931e-03_rb, &
        & 0.20329e-03_rb,0.17265e-03_rb,0.13686e-03_rb,0.10535e-03_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.23042e-03_rb,0.26962e-03_rb,0.27233e-03_rb,0.26383e-03_rb,0.24677e-03_rb, &
        & 0.22030e-03_rb,0.18718e-03_rb,0.14788e-03_rb,0.11369e-03_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.24025e-03_rb,0.28500e-03_rb,0.28961e-03_rb,0.28171e-03_rb,0.26402e-03_rb, &
        & 0.23726e-03_rb,0.20232e-03_rb,0.16110e-03_rb,0.12526e-03_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.25035e-03_rb,0.30139e-03_rb,0.30818e-03_rb,0.30033e-03_rb,0.28246e-03_rb, &
        & 0.25541e-03_rb,0.21869e-03_rb,0.17592e-03_rb,0.13806e-03_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.18213e-03_rb,0.20563e-03_rb,0.20558e-03_rb,0.19744e-03_rb,0.18197e-03_rb, &
        & 0.15967e-03_rb,0.13562e-03_rb,0.10709e-03_rb,0.84300e-04_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.18873e-03_rb,0.21755e-03_rb,0.21877e-03_rb,0.21098e-03_rb,0.19617e-03_rb, &
        & 0.17419e-03_rb,0.14735e-03_rb,0.11726e-03_rb,0.96090e-04_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.19633e-03_rb,0.22994e-03_rb,0.23250e-03_rb,0.22538e-03_rb,0.21092e-03_rb, &
        & 0.18881e-03_rb,0.15987e-03_rb,0.12650e-03_rb,0.10477e-03_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.20445e-03_rb,0.24309e-03_rb,0.24729e-03_rb,0.24084e-03_rb,0.22613e-03_rb, &
        & 0.20371e-03_rb,0.17330e-03_rb,0.13785e-03_rb,0.11268e-03_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.21299e-03_rb,0.25719e-03_rb,0.26315e-03_rb,0.25711e-03_rb,0.24260e-03_rb, &
        & 0.21944e-03_rb,0.18764e-03_rb,0.15038e-03_rb,0.12367e-03_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.15527e-03_rb,0.17641e-03_rb,0.17566e-03_rb,0.16836e-03_rb,0.15534e-03_rb, &
        & 0.13636e-03_rb,0.11530e-03_rb,0.91307e-04_rb,0.70205e-04_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.16096e-03_rb,0.18612e-03_rb,0.18657e-03_rb,0.17973e-03_rb,0.16702e-03_rb, &
        & 0.14851e-03_rb,0.12550e-03_rb,0.99145e-04_rb,0.79984e-04_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.16748e-03_rb,0.19648e-03_rb,0.19822e-03_rb,0.19209e-03_rb,0.17957e-03_rb, &
        & 0.16107e-03_rb,0.13652e-03_rb,0.10753e-03_rb,0.86067e-04_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.17450e-03_rb,0.20751e-03_rb,0.21098e-03_rb,0.20534e-03_rb,0.19295e-03_rb, &
        & 0.17420e-03_rb,0.14816e-03_rb,0.11772e-03_rb,0.93252e-04_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.18233e-03_rb,0.21950e-03_rb,0.22442e-03_rb,0.21958e-03_rb,0.20754e-03_rb, &
        & 0.18842e-03_rb,0.16097e-03_rb,0.12851e-03_rb,0.10278e-03_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.23852e-02_rb,0.24955e-02_rb,0.25247e-02_rb,0.24784e-02_rb,0.23525e-02_rb, &
        & 0.21437e-02_rb,0.18704e-02_rb,0.15353e-02_rb,0.99116e-03_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.23950e-02_rb,0.25560e-02_rb,0.26150e-02_rb,0.25851e-02_rb,0.24612e-02_rb, &
        & 0.22632e-02_rb,0.20004e-02_rb,0.16585e-02_rb,0.11087e-02_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.24020e-02_rb,0.26226e-02_rb,0.27112e-02_rb,0.26940e-02_rb,0.25772e-02_rb, &
        & 0.23915e-02_rb,0.21332e-02_rb,0.17859e-02_rb,0.12435e-02_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.24093e-02_rb,0.26930e-02_rb,0.28096e-02_rb,0.28038e-02_rb,0.27017e-02_rb, &
        & 0.25227e-02_rb,0.22696e-02_rb,0.19198e-02_rb,0.13912e-02_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.24224e-02_rb,0.27647e-02_rb,0.29093e-02_rb,0.29173e-02_rb,0.28299e-02_rb, &
        & 0.26579e-02_rb,0.24103e-02_rb,0.20612e-02_rb,0.15535e-02_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.20974e-02_rb,0.22118e-02_rb,0.22296e-02_rb,0.21778e-02_rb,0.20625e-02_rb, &
        & 0.18815e-02_rb,0.16530e-02_rb,0.13562e-02_rb,0.82141e-03_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.21046e-02_rb,0.22702e-02_rb,0.23143e-02_rb,0.22813e-02_rb,0.21720e-02_rb, &
        & 0.19971e-02_rb,0.17712e-02_rb,0.14678e-02_rb,0.93003e-03_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.21127e-02_rb,0.23326e-02_rb,0.24062e-02_rb,0.23885e-02_rb,0.22861e-02_rb, &
        & 0.21193e-02_rb,0.18939e-02_rb,0.15834e-02_rb,0.10463e-02_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.21272e-02_rb,0.23971e-02_rb,0.25002e-02_rb,0.24977e-02_rb,0.24074e-02_rb, &
        & 0.22461e-02_rb,0.20177e-02_rb,0.17030e-02_rb,0.11732e-02_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.21483e-02_rb,0.24652e-02_rb,0.25947e-02_rb,0.26138e-02_rb,0.25341e-02_rb, &
        & 0.23755e-02_rb,0.21449e-02_rb,0.18299e-02_rb,0.13107e-02_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.18244e-02_rb,0.19320e-02_rb,0.19387e-02_rb,0.18795e-02_rb,0.17712e-02_rb, &
        & 0.16188e-02_rb,0.14258e-02_rb,0.11664e-02_rb,0.66404e-03_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.18322e-02_rb,0.19846e-02_rb,0.20150e-02_rb,0.19741e-02_rb,0.18746e-02_rb, &
        & 0.17292e-02_rb,0.15311e-02_rb,0.12668e-02_rb,0.75772e-03_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.18451e-02_rb,0.20412e-02_rb,0.20960e-02_rb,0.20746e-02_rb,0.19871e-02_rb, &
        & 0.18444e-02_rb,0.16430e-02_rb,0.13711e-02_rb,0.85470e-03_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.18657e-02_rb,0.21020e-02_rb,0.21815e-02_rb,0.21798e-02_rb,0.21034e-02_rb, &
        & 0.19629e-02_rb,0.17571e-02_rb,0.14786e-02_rb,0.95834e-03_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.18943e-02_rb,0.21674e-02_rb,0.22717e-02_rb,0.22915e-02_rb,0.22239e-02_rb, &
        & 0.20844e-02_rb,0.18739e-02_rb,0.15906e-02_rb,0.10775e-02_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.15758e-02_rb,0.16711e-02_rb,0.16675e-02_rb,0.16043e-02_rb,0.15088e-02_rb, &
        & 0.13785e-02_rb,0.12140e-02_rb,0.98780e-03_rb,0.53258e-03_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.15867e-02_rb,0.17169e-02_rb,0.17338e-02_rb,0.16891e-02_rb,0.16022e-02_rb, &
        & 0.14807e-02_rb,0.13108e-02_rb,0.10774e-02_rb,0.61127e-03_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.16043e-02_rb,0.17687e-02_rb,0.18047e-02_rb,0.17788e-02_rb,0.17054e-02_rb, &
        & 0.15869e-02_rb,0.14121e-02_rb,0.11700e-02_rb,0.69092e-03_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.16308e-02_rb,0.18263e-02_rb,0.18817e-02_rb,0.18741e-02_rb,0.18139e-02_rb, &
        & 0.16955e-02_rb,0.15162e-02_rb,0.12673e-02_rb,0.77859e-03_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.16631e-02_rb,0.18894e-02_rb,0.19653e-02_rb,0.19764e-02_rb,0.19240e-02_rb, &
        & 0.18069e-02_rb,0.16237e-02_rb,0.13697e-02_rb,0.87657e-03_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.13575e-02_rb,0.14376e-02_rb,0.14238e-02_rb,0.13660e-02_rb,0.12806e-02_rb, &
        & 0.11664e-02_rb,0.10231e-02_rb,0.82794e-03_rb,0.42595e-03_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.13711e-02_rb,0.14787e-02_rb,0.14807e-02_rb,0.14369e-02_rb,0.13637e-02_rb, &
        & 0.12570e-02_rb,0.11125e-02_rb,0.90729e-03_rb,0.48990e-03_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.13932e-02_rb,0.15262e-02_rb,0.15434e-02_rb,0.15158e-02_rb,0.14536e-02_rb, &
        & 0.13534e-02_rb,0.12033e-02_rb,0.99132e-03_rb,0.55848e-03_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.14217e-02_rb,0.15804e-02_rb,0.16130e-02_rb,0.16009e-02_rb,0.15482e-02_rb, &
        & 0.14537e-02_rb,0.12991e-02_rb,0.10796e-02_rb,0.63246e-03_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.14548e-02_rb,0.16406e-02_rb,0.16899e-02_rb,0.16924e-02_rb,0.16473e-02_rb, &
        & 0.15538e-02_rb,0.13988e-02_rb,0.11736e-02_rb,0.71422e-03_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.11657e-02_rb,0.12310e-02_rb,0.12114e-02_rb,0.11593e-02_rb,0.10807e-02_rb, &
        & 0.97870e-03_rb,0.85220e-03_rb,0.68679e-03_rb,0.33278e-03_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.11817e-02_rb,0.12689e-02_rb,0.12606e-02_rb,0.12200e-02_rb,0.11515e-02_rb, &
        & 0.10575e-02_rb,0.93300e-03_rb,0.75653e-03_rb,0.38390e-03_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.12052e-02_rb,0.13132e-02_rb,0.13166e-02_rb,0.12868e-02_rb,0.12281e-02_rb, &
        & 0.11419e-02_rb,0.10150e-02_rb,0.83054e-03_rb,0.44291e-03_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.12337e-02_rb,0.13638e-02_rb,0.13800e-02_rb,0.13608e-02_rb,0.13105e-02_rb, &
        & 0.12294e-02_rb,0.11010e-02_rb,0.91218e-03_rb,0.50714e-03_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.12657e-02_rb,0.14200e-02_rb,0.14498e-02_rb,0.14421e-02_rb,0.13994e-02_rb, &
        & 0.13201e-02_rb,0.11915e-02_rb,0.99447e-03_rb,0.57759e-03_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.99495e-03_rb,0.10489e-02_rb,0.10293e-02_rb,0.98184e-03_rb,0.91156e-03_rb, &
        & 0.82083e-03_rb,0.70758e-03_rb,0.56387e-03_rb,0.26025e-03_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.10122e-02_rb,0.10837e-02_rb,0.10724e-02_rb,0.10344e-02_rb,0.96982e-03_rb, &
        & 0.88589e-03_rb,0.77530e-03_rb,0.62605e-03_rb,0.29914e-03_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.10351e-02_rb,0.11253e-02_rb,0.11236e-02_rb,0.10922e-02_rb,0.10352e-02_rb, &
        & 0.95724e-03_rb,0.84889e-03_rb,0.69076e-03_rb,0.34305e-03_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.10622e-02_rb,0.11721e-02_rb,0.11813e-02_rb,0.11564e-02_rb,0.11065e-02_rb, &
        & 0.10331e-02_rb,0.92494e-03_rb,0.76297e-03_rb,0.39175e-03_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.10903e-02_rb,0.12244e-02_rb,0.12441e-02_rb,0.12287e-02_rb,0.11845e-02_rb, &
        & 0.11129e-02_rb,0.10035e-02_rb,0.83648e-03_rb,0.44712e-03_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.84532e-03_rb,0.88892e-03_rb,0.86965e-03_rb,0.82727e-03_rb,0.76674e-03_rb, &
        & 0.68807e-03_rb,0.58679e-03_rb,0.46179e-03_rb,0.21931e-03_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.86227e-03_rb,0.92051e-03_rb,0.90826e-03_rb,0.87315e-03_rb,0.81675e-03_rb, &
        & 0.74173e-03_rb,0.64502e-03_rb,0.51436e-03_rb,0.25538e-03_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.88387e-03_rb,0.95827e-03_rb,0.95473e-03_rb,0.92480e-03_rb,0.87135e-03_rb, &
        & 0.80090e-03_rb,0.70594e-03_rb,0.57141e-03_rb,0.29275e-03_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.90659e-03_rb,0.10014e-02_rb,0.10072e-02_rb,0.98186e-03_rb,0.93348e-03_rb, &
        & 0.86582e-03_rb,0.77148e-03_rb,0.63431e-03_rb,0.33522e-03_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.93066e-03_rb,0.10477e-02_rb,0.10640e-02_rb,0.10456e-02_rb,0.10022e-02_rb, &
        & 0.93597e-03_rb,0.84005e-03_rb,0.69831e-03_rb,0.38214e-03_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.71681e-03_rb,0.75151e-03_rb,0.73165e-03_rb,0.69303e-03_rb,0.64106e-03_rb, &
        & 0.57326e-03_rb,0.48762e-03_rb,0.37724e-03_rb,0.19963e-03_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.73278e-03_rb,0.77936e-03_rb,0.76580e-03_rb,0.73272e-03_rb,0.68369e-03_rb, &
        & 0.61851e-03_rb,0.53463e-03_rb,0.42190e-03_rb,0.23258e-03_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.75147e-03_rb,0.81327e-03_rb,0.80692e-03_rb,0.77827e-03_rb,0.73101e-03_rb, &
        & 0.66869e-03_rb,0.58582e-03_rb,0.47030e-03_rb,0.26770e-03_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.77011e-03_rb,0.85035e-03_rb,0.85331e-03_rb,0.82912e-03_rb,0.78414e-03_rb, &
        & 0.72425e-03_rb,0.64047e-03_rb,0.52434e-03_rb,0.30603e-03_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.79031e-03_rb,0.89033e-03_rb,0.90223e-03_rb,0.88462e-03_rb,0.84489e-03_rb, &
        & 0.78478e-03_rb,0.70032e-03_rb,0.57826e-03_rb,0.34848e-03_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.60860e-03_rb,0.63631e-03_rb,0.61722e-03_rb,0.58182e-03_rb,0.53631e-03_rb, &
        & 0.47875e-03_rb,0.40598e-03_rb,0.31001e-03_rb,0.21716e-03_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.62300e-03_rb,0.66096e-03_rb,0.64723e-03_rb,0.61624e-03_rb,0.57283e-03_rb, &
        & 0.51614e-03_rb,0.44515e-03_rb,0.34783e-03_rb,0.24030e-03_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.63811e-03_rb,0.69041e-03_rb,0.68320e-03_rb,0.65609e-03_rb,0.61386e-03_rb, &
        & 0.55902e-03_rb,0.48770e-03_rb,0.38933e-03_rb,0.26729e-03_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.65360e-03_rb,0.72213e-03_rb,0.72254e-03_rb,0.70011e-03_rb,0.66034e-03_rb, &
        & 0.60659e-03_rb,0.53507e-03_rb,0.43477e-03_rb,0.29849e-03_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.67038e-03_rb,0.75665e-03_rb,0.76418e-03_rb,0.74727e-03_rb,0.71207e-03_rb, &
        & 0.65924e-03_rb,0.58660e-03_rb,0.48108e-03_rb,0.33464e-03_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.52185e-03_rb,0.54797e-03_rb,0.53190e-03_rb,0.50128e-03_rb,0.46230e-03_rb, &
        & 0.41291e-03_rb,0.35086e-03_rb,0.26843e-03_rb,0.22407e-03_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.53416e-03_rb,0.57111e-03_rb,0.56029e-03_rb,0.53286e-03_rb,0.49524e-03_rb, &
        & 0.44633e-03_rb,0.38530e-03_rb,0.30138e-03_rb,0.24793e-03_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.54635e-03_rb,0.59685e-03_rb,0.59183e-03_rb,0.56852e-03_rb,0.53236e-03_rb, &
        & 0.48455e-03_rb,0.42287e-03_rb,0.33910e-03_rb,0.27073e-03_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.55941e-03_rb,0.62493e-03_rb,0.62582e-03_rb,0.60673e-03_rb,0.57348e-03_rb, &
        & 0.52713e-03_rb,0.46523e-03_rb,0.37861e-03_rb,0.29520e-03_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.57459e-03_rb,0.65537e-03_rb,0.66217e-03_rb,0.64854e-03_rb,0.61857e-03_rb, &
        & 0.57343e-03_rb,0.51159e-03_rb,0.41997e-03_rb,0.32408e-03_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.44547e-03_rb,0.47205e-03_rb,0.45856e-03_rb,0.43259e-03_rb,0.39885e-03_rb, &
        & 0.35629e-03_rb,0.30273e-03_rb,0.23099e-03_rb,0.20507e-03_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.45530e-03_rb,0.49200e-03_rb,0.48355e-03_rb,0.46058e-03_rb,0.42831e-03_rb, &
        & 0.38615e-03_rb,0.33301e-03_rb,0.25970e-03_rb,0.22582e-03_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.46565e-03_rb,0.51443e-03_rb,0.51077e-03_rb,0.49108e-03_rb,0.46067e-03_rb, &
        & 0.41962e-03_rb,0.36657e-03_rb,0.29355e-03_rb,0.24853e-03_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.47732e-03_rb,0.53905e-03_rb,0.54027e-03_rb,0.52415e-03_rb,0.49634e-03_rb, &
        & 0.45650e-03_rb,0.40422e-03_rb,0.32887e-03_rb,0.26943e-03_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.49131e-03_rb,0.56624e-03_rb,0.57259e-03_rb,0.56091e-03_rb,0.53532e-03_rb, &
        & 0.49724e-03_rb,0.44494e-03_rb,0.36671e-03_rb,0.29370e-03_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.37861e-03_rb,0.40433e-03_rb,0.39425e-03_rb,0.37277e-03_rb,0.34384e-03_rb, &
        & 0.30718e-03_rb,0.26082e-03_rb,0.19813e-03_rb,0.17021e-03_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.38646e-03_rb,0.42180e-03_rb,0.41569e-03_rb,0.39676e-03_rb,0.36941e-03_rb, &
        & 0.33370e-03_rb,0.28746e-03_rb,0.22445e-03_rb,0.18768e-03_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.39522e-03_rb,0.44150e-03_rb,0.43928e-03_rb,0.42293e-03_rb,0.39746e-03_rb, &
        & 0.36268e-03_rb,0.31709e-03_rb,0.25391e-03_rb,0.20463e-03_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.40576e-03_rb,0.46362e-03_rb,0.46497e-03_rb,0.45178e-03_rb,0.42852e-03_rb, &
        & 0.39486e-03_rb,0.34992e-03_rb,0.28472e-03_rb,0.22430e-03_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.41810e-03_rb,0.48813e-03_rb,0.49424e-03_rb,0.48434e-03_rb,0.46280e-03_rb, &
        & 0.43027e-03_rb,0.38540e-03_rb,0.31863e-03_rb,0.24541e-03_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.52571e-02_rb,0.51649e-02_rb,0.50998e-02_rb,0.49978e-02_rb,0.48299e-02_rb, &
        & 0.45478e-02_rb,0.41071e-02_rb,0.34884e-02_rb,0.27402e-02_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.51967e-02_rb,0.52277e-02_rb,0.52437e-02_rb,0.52099e-02_rb,0.50894e-02_rb, &
        & 0.48424e-02_rb,0.44165e-02_rb,0.38421e-02_rb,0.31643e-02_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.51548e-02_rb,0.53047e-02_rb,0.54023e-02_rb,0.54271e-02_rb,0.53544e-02_rb, &
        & 0.51370e-02_rb,0.47457e-02_rb,0.42178e-02_rb,0.36142e-02_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.51293e-02_rb,0.53942e-02_rb,0.55672e-02_rb,0.56512e-02_rb,0.56201e-02_rb, &
        & 0.54409e-02_rb,0.50927e-02_rb,0.46186e-02_rb,0.40998e-02_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.51178e-02_rb,0.54996e-02_rb,0.57406e-02_rb,0.58798e-02_rb,0.58932e-02_rb, &
        & 0.57606e-02_rb,0.54613e-02_rb,0.50399e-02_rb,0.46100e-02_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.47778e-02_rb,0.47398e-02_rb,0.47115e-02_rb,0.46169e-02_rb,0.44426e-02_rb, &
        & 0.41630e-02_rb,0.37511e-02_rb,0.31426e-02_rb,0.23122e-02_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.47384e-02_rb,0.48111e-02_rb,0.48532e-02_rb,0.48144e-02_rb,0.46755e-02_rb, &
        & 0.44342e-02_rb,0.40397e-02_rb,0.34599e-02_rb,0.26698e-02_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.47181e-02_rb,0.49025e-02_rb,0.50071e-02_rb,0.50151e-02_rb,0.49194e-02_rb, &
        & 0.47085e-02_rb,0.43402e-02_rb,0.37956e-02_rb,0.30630e-02_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.47137e-02_rb,0.50070e-02_rb,0.51691e-02_rb,0.52245e-02_rb,0.51706e-02_rb, &
        & 0.49959e-02_rb,0.46583e-02_rb,0.41574e-02_rb,0.34865e-02_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.47212e-02_rb,0.51234e-02_rb,0.53397e-02_rb,0.54379e-02_rb,0.54276e-02_rb, &
        & 0.52983e-02_rb,0.49978e-02_rb,0.45412e-02_rb,0.39357e-02_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.42971e-02_rb,0.42851e-02_rb,0.42729e-02_rb,0.41707e-02_rb,0.39926e-02_rb, &
        & 0.37146e-02_rb,0.33264e-02_rb,0.27717e-02_rb,0.18852e-02_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.42740e-02_rb,0.43647e-02_rb,0.44079e-02_rb,0.43562e-02_rb,0.42040e-02_rb, &
        & 0.39559e-02_rb,0.35908e-02_rb,0.30443e-02_rb,0.21816e-02_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.42703e-02_rb,0.44650e-02_rb,0.45616e-02_rb,0.45425e-02_rb,0.44233e-02_rb, &
        & 0.42066e-02_rb,0.38676e-02_rb,0.33396e-02_rb,0.25126e-02_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.42812e-02_rb,0.45806e-02_rb,0.47230e-02_rb,0.47371e-02_rb,0.46519e-02_rb, &
        & 0.44691e-02_rb,0.41620e-02_rb,0.36586e-02_rb,0.28704e-02_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.43030e-02_rb,0.47042e-02_rb,0.48922e-02_rb,0.49377e-02_rb,0.48939e-02_rb, &
        & 0.47477e-02_rb,0.44742e-02_rb,0.39985e-02_rb,0.32501e-02_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.38233e-02_rb,0.38279e-02_rb,0.38143e-02_rb,0.37097e-02_rb,0.35270e-02_rb, &
        & 0.32658e-02_rb,0.29019e-02_rb,0.24142e-02_rb,0.15291e-02_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.38114e-02_rb,0.39122e-02_rb,0.39475e-02_rb,0.38778e-02_rb,0.37194e-02_rb, &
        & 0.34777e-02_rb,0.31357e-02_rb,0.26541e-02_rb,0.17758e-02_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.38202e-02_rb,0.40186e-02_rb,0.40970e-02_rb,0.40542e-02_rb,0.39201e-02_rb, &
        & 0.37024e-02_rb,0.33826e-02_rb,0.29146e-02_rb,0.20543e-02_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.38442e-02_rb,0.41385e-02_rb,0.42569e-02_rb,0.42394e-02_rb,0.41310e-02_rb, &
        & 0.39403e-02_rb,0.36479e-02_rb,0.31917e-02_rb,0.23538e-02_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.38788e-02_rb,0.42695e-02_rb,0.44229e-02_rb,0.44328e-02_rb,0.43540e-02_rb, &
        & 0.41948e-02_rb,0.39317e-02_rb,0.34895e-02_rb,0.26724e-02_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.33660e-02_rb,0.33879e-02_rb,0.33669e-02_rb,0.32570e-02_rb,0.30787e-02_rb, &
        & 0.28362e-02_rb,0.25094e-02_rb,0.20819e-02_rb,0.12376e-02_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.33649e-02_rb,0.34740e-02_rb,0.34949e-02_rb,0.34119e-02_rb,0.32508e-02_rb, &
        & 0.30247e-02_rb,0.27121e-02_rb,0.22949e-02_rb,0.14462e-02_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.33839e-02_rb,0.35821e-02_rb,0.36398e-02_rb,0.35762e-02_rb,0.34359e-02_rb, &
        & 0.32230e-02_rb,0.29317e-02_rb,0.25218e-02_rb,0.16788e-02_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.34188e-02_rb,0.37047e-02_rb,0.37932e-02_rb,0.37524e-02_rb,0.36321e-02_rb, &
        & 0.34359e-02_rb,0.31677e-02_rb,0.27663e-02_rb,0.19299e-02_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.34572e-02_rb,0.38382e-02_rb,0.39563e-02_rb,0.39381e-02_rb,0.38416e-02_rb, &
        & 0.36710e-02_rb,0.34185e-02_rb,0.30298e-02_rb,0.21985e-02_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.29383e-02_rb,0.29689e-02_rb,0.29375e-02_rb,0.28254e-02_rb,0.26571e-02_rb, &
        & 0.24356e-02_rb,0.21491e-02_rb,0.17706e-02_rb,0.99284e-03_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.29460e-02_rb,0.30526e-02_rb,0.30576e-02_rb,0.29676e-02_rb,0.28114e-02_rb, &
        & 0.25995e-02_rb,0.23244e-02_rb,0.19573e-02_rb,0.11692e-02_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.29740e-02_rb,0.31582e-02_rb,0.31942e-02_rb,0.31202e-02_rb,0.29785e-02_rb, &
        & 0.27771e-02_rb,0.25160e-02_rb,0.21565e-02_rb,0.13616e-02_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.30101e-02_rb,0.32807e-02_rb,0.33417e-02_rb,0.32855e-02_rb,0.31583e-02_rb, &
        & 0.29708e-02_rb,0.27234e-02_rb,0.23688e-02_rb,0.15696e-02_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.30525e-02_rb,0.34071e-02_rb,0.34999e-02_rb,0.34618e-02_rb,0.33518e-02_rb, &
        & 0.31829e-02_rb,0.29437e-02_rb,0.25986e-02_rb,0.17941e-02_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.25527e-02_rb,0.25837e-02_rb,0.25415e-02_rb,0.24332e-02_rb,0.22747e-02_rb, &
        & 0.20758e-02_rb,0.18231e-02_rb,0.14951e-02_rb,0.78066e-03_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.25661e-02_rb,0.26638e-02_rb,0.26497e-02_rb,0.25583e-02_rb,0.24134e-02_rb, &
        & 0.22212e-02_rb,0.19774e-02_rb,0.16540e-02_rb,0.92809e-03_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.25966e-02_rb,0.27642e-02_rb,0.27764e-02_rb,0.26998e-02_rb,0.25630e-02_rb, &
        & 0.23812e-02_rb,0.21449e-02_rb,0.18264e-02_rb,0.10893e-02_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.26340e-02_rb,0.28771e-02_rb,0.29165e-02_rb,0.28538e-02_rb,0.27279e-02_rb, &
        & 0.25553e-02_rb,0.23260e-02_rb,0.20107e-02_rb,0.12665e-02_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.26829e-02_rb,0.29939e-02_rb,0.30639e-02_rb,0.30192e-02_rb,0.29089e-02_rb, &
        & 0.27450e-02_rb,0.25219e-02_rb,0.22101e-02_rb,0.14584e-02_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.22109e-02_rb,0.22392e-02_rb,0.21909e-02_rb,0.20870e-02_rb,0.19391e-02_rb, &
        & 0.17569e-02_rb,0.15368e-02_rb,0.12514e-02_rb,0.62879e-03_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.22276e-02_rb,0.23140e-02_rb,0.22874e-02_rb,0.21965e-02_rb,0.20609e-02_rb, &
        & 0.18868e-02_rb,0.16711e-02_rb,0.13887e-02_rb,0.73533e-03_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.22565e-02_rb,0.24065e-02_rb,0.24031e-02_rb,0.23233e-02_rb,0.21969e-02_rb, &
        & 0.20302e-02_rb,0.18188e-02_rb,0.15365e-02_rb,0.85609e-03_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.22974e-02_rb,0.25072e-02_rb,0.25290e-02_rb,0.24653e-02_rb,0.23482e-02_rb, &
        & 0.21882e-02_rb,0.19777e-02_rb,0.16966e-02_rb,0.98553e-03_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.23517e-02_rb,0.26173e-02_rb,0.26619e-02_rb,0.26145e-02_rb,0.25139e-02_rb, &
        & 0.23587e-02_rb,0.21497e-02_rb,0.18671e-02_rb,0.11277e-02_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.19109e-02_rb,0.19351e-02_rb,0.18847e-02_rb,0.17853e-02_rb,0.16484e-02_rb, &
        & 0.14828e-02_rb,0.12861e-02_rb,0.10403e-02_rb,0.59847e-03_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.19272e-02_rb,0.20031e-02_rb,0.19691e-02_rb,0.18817e-02_rb,0.17540e-02_rb, &
        & 0.15965e-02_rb,0.14036e-02_rb,0.11571e-02_rb,0.69475e-03_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.19559e-02_rb,0.20842e-02_rb,0.20709e-02_rb,0.19954e-02_rb,0.18765e-02_rb, &
        & 0.17233e-02_rb,0.15331e-02_rb,0.12839e-02_rb,0.80150e-03_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.19994e-02_rb,0.21761e-02_rb,0.21815e-02_rb,0.21203e-02_rb,0.20143e-02_rb, &
        & 0.18645e-02_rb,0.16732e-02_rb,0.14206e-02_rb,0.91777e-03_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.20550e-02_rb,0.22788e-02_rb,0.23027e-02_rb,0.22546e-02_rb,0.21605e-02_rb, &
        & 0.20175e-02_rb,0.18247e-02_rb,0.15687e-02_rb,0.10450e-02_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.16445e-02_rb,0.16716e-02_rb,0.16238e-02_rb,0.15300e-02_rb,0.14061e-02_rb, &
        & 0.12566e-02_rb,0.10807e-02_rb,0.86738e-03_rb,0.60827e-03_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.16600e-02_rb,0.17327e-02_rb,0.16984e-02_rb,0.16154e-02_rb,0.14994e-02_rb, &
        & 0.13556e-02_rb,0.11827e-02_rb,0.96695e-03_rb,0.70548e-03_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.16905e-02_rb,0.18053e-02_rb,0.17869e-02_rb,0.17163e-02_rb,0.16090e-02_rb, &
        & 0.14689e-02_rb,0.12961e-02_rb,0.10754e-02_rb,0.80786e-03_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.17354e-02_rb,0.18905e-02_rb,0.18865e-02_rb,0.18272e-02_rb,0.17299e-02_rb, &
        & 0.15951e-02_rb,0.14192e-02_rb,0.11936e-02_rb,0.91620e-03_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.17884e-02_rb,0.19860e-02_rb,0.19977e-02_rb,0.19501e-02_rb,0.18596e-02_rb, &
        & 0.17298e-02_rb,0.15525e-02_rb,0.13217e-02_rb,0.10338e-02_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.14133e-02_rb,0.14603e-02_rb,0.14202e-02_rb,0.13407e-02_rb,0.12334e-02_rb, &
        & 0.11026e-02_rb,0.94643e-03_rb,0.75979e-03_rb,0.57405e-03_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.14357e-02_rb,0.15193e-02_rb,0.14916e-02_rb,0.14232e-02_rb,0.13232e-02_rb, &
        & 0.11950e-02_rb,0.10392e-02_rb,0.84686e-03_rb,0.65471e-03_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.14730e-02_rb,0.15907e-02_rb,0.15765e-02_rb,0.15172e-02_rb,0.14237e-02_rb, &
        & 0.12997e-02_rb,0.11424e-02_rb,0.94298e-03_rb,0.75212e-03_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.15188e-02_rb,0.16737e-02_rb,0.16734e-02_rb,0.16227e-02_rb,0.15348e-02_rb, &
        & 0.14132e-02_rb,0.12544e-02_rb,0.10476e-02_rb,0.85445e-03_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.15705e-02_rb,0.17634e-02_rb,0.17799e-02_rb,0.17387e-02_rb,0.16551e-02_rb, &
        & 0.15344e-02_rb,0.13744e-02_rb,0.11635e-02_rb,0.96358e-03_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.12145e-02_rb,0.12721e-02_rb,0.12387e-02_rb,0.11722e-02_rb,0.10815e-02_rb, &
        & 0.96748e-03_rb,0.83052e-03_rb,0.66461e-03_rb,0.51952e-03_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.12432e-02_rb,0.13303e-02_rb,0.13076e-02_rb,0.12500e-02_rb,0.11644e-02_rb, &
        & 0.10531e-02_rb,0.91410e-03_rb,0.74198e-03_rb,0.58718e-03_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.12811e-02_rb,0.13998e-02_rb,0.13897e-02_rb,0.13394e-02_rb,0.12574e-02_rb, &
        & 0.11474e-02_rb,0.10067e-02_rb,0.82675e-03_rb,0.66821e-03_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.13251e-02_rb,0.14771e-02_rb,0.14813e-02_rb,0.14393e-02_rb,0.13610e-02_rb, &
        & 0.12504e-02_rb,0.11057e-02_rb,0.91975e-03_rb,0.76078e-03_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.13700e-02_rb,0.15597e-02_rb,0.15807e-02_rb,0.15467e-02_rb,0.14716e-02_rb, &
        & 0.13608e-02_rb,0.12138e-02_rb,0.10232e-02_rb,0.85832e-03_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.10471e-02_rb,0.11076e-02_rb,0.10790e-02_rb,0.10231e-02_rb,0.94527e-03_rb, &
        & 0.84789e-03_rb,0.72861e-03_rb,0.58068e-03_rb,0.44361e-03_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.10765e-02_rb,0.11637e-02_rb,0.11461e-02_rb,0.10966e-02_rb,0.10221e-02_rb, &
        & 0.92418e-03_rb,0.80364e-03_rb,0.64949e-03_rb,0.50409e-03_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.11121e-02_rb,0.12288e-02_rb,0.12232e-02_rb,0.11802e-02_rb,0.11084e-02_rb, &
        & 0.10099e-02_rb,0.88562e-03_rb,0.72467e-03_rb,0.57688e-03_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.11491e-02_rb,0.12989e-02_rb,0.13080e-02_rb,0.12720e-02_rb,0.12030e-02_rb, &
        & 0.11037e-02_rb,0.97404e-03_rb,0.80781e-03_rb,0.65303e-03_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.11880e-02_rb,0.13724e-02_rb,0.13975e-02_rb,0.13701e-02_rb,0.13034e-02_rb, &
        & 0.12049e-02_rb,0.10715e-02_rb,0.89944e-03_rb,0.73687e-03_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.14418e-01_rb,0.13406e-01_rb,0.12605e-01_rb,0.11857e-01_rb,0.11091e-01_rb, &
        & 0.10336e-01_rb,0.95691e-02_rb,0.87036e-02_rb,0.88524e-02_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.14296e-01_rb,0.13450e-01_rb,0.12870e-01_rb,0.12297e-01_rb,0.11684e-01_rb, &
        & 0.11070e-01_rb,0.10452e-01_rb,0.97477e-02_rb,0.10154e-01_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.14166e-01_rb,0.13524e-01_rb,0.13158e-01_rb,0.12768e-01_rb,0.12317e-01_rb, &
        & 0.11876e-01_rb,0.11397e-01_rb,0.10864e-01_rb,0.11523e-01_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.14022e-01_rb,0.13622e-01_rb,0.13487e-01_rb,0.13264e-01_rb,0.13003e-01_rb, &
        & 0.12732e-01_rb,0.12371e-01_rb,0.12050e-01_rb,0.12968e-01_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.13872e-01_rb,0.13730e-01_rb,0.13820e-01_rb,0.13802e-01_rb,0.13718e-01_rb, &
        & 0.13604e-01_rb,0.13373e-01_rb,0.13315e-01_rb,0.14498e-01_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.14547e-01_rb,0.13535e-01_rb,0.12726e-01_rb,0.11955e-01_rb,0.11121e-01_rb, &
        & 0.10242e-01_rb,0.92998e-02_rb,0.82716e-02_rb,0.79571e-02_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.14415e-01_rb,0.13577e-01_rb,0.12984e-01_rb,0.12377e-01_rb,0.11698e-01_rb, &
        & 0.10932e-01_rb,0.10129e-01_rb,0.92199e-02_rb,0.91565e-02_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.14274e-01_rb,0.13649e-01_rb,0.13264e-01_rb,0.12845e-01_rb,0.12290e-01_rb, &
        & 0.11686e-01_rb,0.11026e-01_rb,0.10231e-01_rb,0.10415e-01_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.14122e-01_rb,0.13744e-01_rb,0.13589e-01_rb,0.13325e-01_rb,0.12935e-01_rb, &
        & 0.12484e-01_rb,0.11950e-01_rb,0.11311e-01_rb,0.11744e-01_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.13965e-01_rb,0.13855e-01_rb,0.13924e-01_rb,0.13828e-01_rb,0.13608e-01_rb, &
        & 0.13305e-01_rb,0.12905e-01_rb,0.12457e-01_rb,0.13151e-01_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.14475e-01_rb,0.13482e-01_rb,0.12654e-01_rb,0.11861e-01_rb,0.10971e-01_rb, &
        & 0.99930e-02_rb,0.88735e-02_rb,0.76595e-02_rb,0.68509e-02_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.14335e-01_rb,0.13514e-01_rb,0.12906e-01_rb,0.12263e-01_rb,0.11533e-01_rb, &
        & 0.10637e-01_rb,0.96386e-02_rb,0.85394e-02_rb,0.79297e-02_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.14181e-01_rb,0.13576e-01_rb,0.13177e-01_rb,0.12719e-01_rb,0.12099e-01_rb, &
        & 0.11329e-01_rb,0.10466e-01_rb,0.94601e-02_rb,0.90731e-02_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.14024e-01_rb,0.13665e-01_rb,0.13491e-01_rb,0.13188e-01_rb,0.12703e-01_rb, &
        & 0.12072e-01_rb,0.11323e-01_rb,0.10433e-01_rb,0.10295e-01_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.13862e-01_rb,0.13772e-01_rb,0.13824e-01_rb,0.13679e-01_rb,0.13333e-01_rb, &
        & 0.12836e-01_rb,0.12214e-01_rb,0.11468e-01_rb,0.11588e-01_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.14153e-01_rb,0.13216e-01_rb,0.12406e-01_rb,0.11612e-01_rb,0.10690e-01_rb, &
        & 0.96495e-02_rb,0.84088e-02_rb,0.70121e-02_rb,0.57874e-02_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.14005e-01_rb,0.13241e-01_rb,0.12647e-01_rb,0.11995e-01_rb,0.11231e-01_rb, &
        & 0.10266e-01_rb,0.91127e-02_rb,0.78123e-02_rb,0.67372e-02_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.13845e-01_rb,0.13290e-01_rb,0.12907e-01_rb,0.12438e-01_rb,0.11783e-01_rb, &
        & 0.10918e-01_rb,0.98730e-02_rb,0.86646e-02_rb,0.77601e-02_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.13685e-01_rb,0.13375e-01_rb,0.13206e-01_rb,0.12899e-01_rb,0.12360e-01_rb, &
        & 0.11608e-01_rb,0.10665e-01_rb,0.95624e-02_rb,0.88628e-02_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.13529e-01_rb,0.13472e-01_rb,0.13531e-01_rb,0.13371e-01_rb,0.12964e-01_rb, &
        & 0.12311e-01_rb,0.11491e-01_rb,0.10508e-01_rb,0.10039e-01_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.13587e-01_rb,0.12730e-01_rb,0.11973e-01_rb,0.11204e-01_rb,0.10282e-01_rb, &
        & 0.92218e-02_rb,0.79316e-02_rb,0.64044e-02_rb,0.48583e-02_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.13430e-01_rb,0.12751e-01_rb,0.12205e-01_rb,0.11567e-01_rb,0.10798e-01_rb, &
        & 0.98163e-02_rb,0.85810e-02_rb,0.71221e-02_rb,0.56837e-02_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.13271e-01_rb,0.12793e-01_rb,0.12448e-01_rb,0.11993e-01_rb,0.11338e-01_rb, &
        & 0.10437e-01_rb,0.92786e-02_rb,0.78957e-02_rb,0.65802e-02_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.13115e-01_rb,0.12874e-01_rb,0.12735e-01_rb,0.12440e-01_rb,0.11897e-01_rb, &
        & 0.11094e-01_rb,0.10003e-01_rb,0.87146e-02_rb,0.75565e-02_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.12984e-01_rb,0.12973e-01_rb,0.13053e-01_rb,0.12899e-01_rb,0.12481e-01_rb, &
        & 0.11759e-01_rb,0.10765e-01_rb,0.95850e-02_rb,0.86117e-02_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.12817e-01_rb,0.12050e-01_rb,0.11369e-01_rb,0.10622e-01_rb,0.97180e-02_rb, &
        & 0.86745e-02_rb,0.73942e-02_rb,0.58154e-02_rb,0.40318e-02_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.12656e-01_rb,0.12077e-01_rb,0.11582e-01_rb,0.10960e-01_rb,0.10205e-01_rb, &
        & 0.92477e-02_rb,0.80000e-02_rb,0.64500e-02_rb,0.47452e-02_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.12499e-01_rb,0.12115e-01_rb,0.11810e-01_rb,0.11361e-01_rb,0.10734e-01_rb, &
        & 0.98374e-02_rb,0.86412e-02_rb,0.71467e-02_rb,0.55214e-02_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.12364e-01_rb,0.12194e-01_rb,0.12081e-01_rb,0.11798e-01_rb,0.11270e-01_rb, &
        & 0.10459e-01_rb,0.93144e-02_rb,0.78844e-02_rb,0.63718e-02_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.12255e-01_rb,0.12311e-01_rb,0.12390e-01_rb,0.12242e-01_rb,0.11833e-01_rb, &
        & 0.11093e-01_rb,0.10016e-01_rb,0.86730e-02_rb,0.72984e-02_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.11891e-01_rb,0.11226e-01_rb,0.10633e-01_rb,0.99134e-02_rb,0.90439e-02_rb, &
        & 0.80297e-02_rb,0.68136e-02_rb,0.52756e-02_rb,0.33216e-02_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.11730e-01_rb,0.11250e-01_rb,0.10822e-01_rb,0.10230e-01_rb,0.94903e-02_rb, &
        & 0.85683e-02_rb,0.73765e-02_rb,0.58347e-02_rb,0.39310e-02_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.11586e-01_rb,0.11299e-01_rb,0.11036e-01_rb,0.10599e-01_rb,0.99917e-02_rb, &
        & 0.91260e-02_rb,0.79645e-02_rb,0.64502e-02_rb,0.46031e-02_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.11473e-01_rb,0.11391e-01_rb,0.11290e-01_rb,0.11015e-01_rb,0.10500e-01_rb, &
        & 0.97132e-02_rb,0.85920e-02_rb,0.71112e-02_rb,0.53403e-02_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.11384e-01_rb,0.11524e-01_rb,0.11598e-01_rb,0.11442e-01_rb,0.11036e-01_rb, &
        & 0.10317e-01_rb,0.92441e-02_rb,0.78209e-02_rb,0.61457e-02_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.10852e-01_rb,0.10302e-01_rb,0.97896e-02_rb,0.91138e-02_rb,0.83000e-02_rb, &
        & 0.73373e-02_rb,0.61956e-02_rb,0.47644e-02_rb,0.26652e-02_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.10699e-01_rb,0.10325e-01_rb,0.99631e-02_rb,0.94045e-02_rb,0.87025e-02_rb, &
        & 0.78255e-02_rb,0.67137e-02_rb,0.52638e-02_rb,0.31922e-02_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.10574e-01_rb,0.10385e-01_rb,0.10163e-01_rb,0.97460e-02_rb,0.91609e-02_rb, &
        & 0.83418e-02_rb,0.72560e-02_rb,0.58143e-02_rb,0.37766e-02_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.10480e-01_rb,0.10499e-01_rb,0.10413e-01_rb,0.10139e-01_rb,0.96342e-02_rb, &
        & 0.88826e-02_rb,0.78405e-02_rb,0.64037e-02_rb,0.44295e-02_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.10415e-01_rb,0.10646e-01_rb,0.10720e-01_rb,0.10553e-01_rb,0.10134e-01_rb, &
        & 0.94529e-02_rb,0.84471e-02_rb,0.70384e-02_rb,0.51485e-02_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.97587e-02_rb,0.93185e-02_rb,0.88694e-02_rb,0.82517e-02_rb,0.75094e-02_rb, &
        & 0.66177e-02_rb,0.55658e-02_rb,0.42591e-02_rb,0.21931e-02_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.96183e-02_rb,0.93436e-02_rb,0.90329e-02_rb,0.85198e-02_rb,0.78654e-02_rb, &
        & 0.70507e-02_rb,0.60316e-02_rb,0.47067e-02_rb,0.25691e-02_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.95112e-02_rb,0.94166e-02_rb,0.92325e-02_rb,0.88355e-02_rb,0.82791e-02_rb, &
        & 0.75194e-02_rb,0.65208e-02_rb,0.52003e-02_rb,0.29789e-02_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.94373e-02_rb,0.95446e-02_rb,0.94815e-02_rb,0.92092e-02_rb,0.87140e-02_rb, &
        & 0.80091e-02_rb,0.70556e-02_rb,0.57344e-02_rb,0.34284e-02_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.93945e-02_rb,0.97041e-02_rb,0.97866e-02_rb,0.96055e-02_rb,0.91834e-02_rb, &
        & 0.85358e-02_rb,0.76199e-02_rb,0.63067e-02_rb,0.39360e-02_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.86571e-02_rb,0.83282e-02_rb,0.79327e-02_rb,0.73862e-02_rb,0.67217e-02_rb, &
        & 0.59210e-02_rb,0.49777e-02_rb,0.37950e-02_rb,0.25161e-02_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.85351e-02_rb,0.83629e-02_rb,0.80910e-02_rb,0.76400e-02_rb,0.70398e-02_rb, &
        & 0.63068e-02_rb,0.53903e-02_rb,0.41966e-02_rb,0.29067e-02_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.84483e-02_rb,0.84483e-02_rb,0.82944e-02_rb,0.79338e-02_rb,0.74159e-02_rb, &
        & 0.67272e-02_rb,0.58285e-02_rb,0.46411e-02_rb,0.33599e-02_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.83941e-02_rb,0.85821e-02_rb,0.85440e-02_rb,0.82866e-02_rb,0.78214e-02_rb, &
        & 0.71692e-02_rb,0.63103e-02_rb,0.51290e-02_rb,0.38620e-02_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.83806e-02_rb,0.87510e-02_rb,0.88459e-02_rb,0.86620e-02_rb,0.82607e-02_rb, &
        & 0.76554e-02_rb,0.68267e-02_rb,0.56504e-02_rb,0.44001e-02_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.75570e-02_rb,0.73902e-02_rb,0.70826e-02_rb,0.66405e-02_rb,0.60731e-02_rb, &
        & 0.53892e-02_rb,0.45656e-02_rb,0.35050e-02_rb,0.26164e-02_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.74683e-02_rb,0.74501e-02_rb,0.72578e-02_rb,0.68886e-02_rb,0.63823e-02_rb, &
        & 0.57485e-02_rb,0.49402e-02_rb,0.38790e-02_rb,0.30097e-02_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.74153e-02_rb,0.75630e-02_rb,0.74762e-02_rb,0.71912e-02_rb,0.67425e-02_rb, &
        & 0.61326e-02_rb,0.53447e-02_rb,0.42990e-02_rb,0.34771e-02_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.74057e-02_rb,0.77202e-02_rb,0.77477e-02_rb,0.75330e-02_rb,0.71268e-02_rb, &
        & 0.65549e-02_rb,0.57951e-02_rb,0.47555e-02_rb,0.39884e-02_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.74391e-02_rb,0.79220e-02_rb,0.80561e-02_rb,0.79014e-02_rb,0.75514e-02_rb, &
        & 0.70214e-02_rb,0.62779e-02_rb,0.52483e-02_rb,0.45376e-02_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.65546e-02_rb,0.65200e-02_rb,0.62910e-02_rb,0.59243e-02_rb,0.54454e-02_rb, &
        & 0.48623e-02_rb,0.41386e-02_rb,0.32013e-02_rb,0.25137e-02_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.64987e-02_rb,0.66047e-02_rb,0.64774e-02_rb,0.61754e-02_rb,0.57483e-02_rb, &
        & 0.51910e-02_rb,0.44799e-02_rb,0.35500e-02_rb,0.28998e-02_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.64867e-02_rb,0.67439e-02_rb,0.67077e-02_rb,0.64758e-02_rb,0.60840e-02_rb, &
        & 0.55472e-02_rb,0.48582e-02_rb,0.39433e-02_rb,0.33472e-02_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.65214e-02_rb,0.69269e-02_rb,0.69913e-02_rb,0.68034e-02_rb,0.64480e-02_rb, &
        & 0.59497e-02_rb,0.52811e-02_rb,0.43712e-02_rb,0.38283e-02_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.66068e-02_rb,0.71634e-02_rb,0.73032e-02_rb,0.71670e-02_rb,0.68605e-02_rb, &
        & 0.63896e-02_rb,0.57387e-02_rb,0.48340e-02_rb,0.43614e-02_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.56598e-02_rb,0.57241e-02_rb,0.55653e-02_rb,0.52616e-02_rb,0.48615e-02_rb, &
        & 0.43534e-02_rb,0.37143e-02_rb,0.28975e-02_rb,0.22483e-02_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.56408e-02_rb,0.58364e-02_rb,0.57539e-02_rb,0.55120e-02_rb,0.51475e-02_rb, &
        & 0.46559e-02_rb,0.40284e-02_rb,0.32222e-02_rb,0.26111e-02_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.56697e-02_rb,0.59982e-02_rb,0.59981e-02_rb,0.58053e-02_rb,0.54583e-02_rb, &
        & 0.49917e-02_rb,0.43867e-02_rb,0.35862e-02_rb,0.30268e-02_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.57519e-02_rb,0.62108e-02_rb,0.62874e-02_rb,0.61219e-02_rb,0.58111e-02_rb, &
        & 0.53715e-02_rb,0.47851e-02_rb,0.39858e-02_rb,0.34700e-02_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.58830e-02_rb,0.64813e-02_rb,0.66064e-02_rb,0.64830e-02_rb,0.62068e-02_rb, &
        & 0.57872e-02_rb,0.52185e-02_rb,0.44205e-02_rb,0.39619e-02_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.28544e-01_rb,0.26155e-01_rb,0.24149e-01_rb,0.22667e-01_rb,0.21632e-01_rb, &
        & 0.20279e-01_rb,0.18489e-01_rb,0.17580e-01_rb,0.19616e-01_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.28363e-01_rb,0.26242e-01_rb,0.24669e-01_rb,0.23473e-01_rb,0.22995e-01_rb, &
        & 0.21803e-01_rb,0.19985e-01_rb,0.19915e-01_rb,0.22325e-01_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.28168e-01_rb,0.26278e-01_rb,0.25280e-01_rb,0.24624e-01_rb,0.24316e-01_rb, &
        & 0.23197e-01_rb,0.21657e-01_rb,0.22779e-01_rb,0.25626e-01_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.27980e-01_rb,0.26431e-01_rb,0.25828e-01_rb,0.25920e-01_rb,0.25669e-01_rb, &
        & 0.24755e-01_rb,0.23686e-01_rb,0.25934e-01_rb,0.29281e-01_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.27732e-01_rb,0.26646e-01_rb,0.26660e-01_rb,0.27009e-01_rb,0.27233e-01_rb, &
        & 0.26525e-01_rb,0.25997e-01_rb,0.29201e-01_rb,0.33040e-01_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.30271e-01_rb,0.27672e-01_rb,0.25272e-01_rb,0.23430e-01_rb,0.22043e-01_rb, &
        & 0.20727e-01_rb,0.19083e-01_rb,0.16913e-01_rb,0.18728e-01_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.30091e-01_rb,0.27720e-01_rb,0.25820e-01_rb,0.24222e-01_rb,0.23409e-01_rb, &
        & 0.22468e-01_rb,0.20609e-01_rb,0.19148e-01_rb,0.21375e-01_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.29914e-01_rb,0.27750e-01_rb,0.26375e-01_rb,0.25267e-01_rb,0.24987e-01_rb, &
        & 0.23926e-01_rb,0.22306e-01_rb,0.21932e-01_rb,0.24591e-01_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.29701e-01_rb,0.27873e-01_rb,0.26877e-01_rb,0.26535e-01_rb,0.26327e-01_rb, &
        & 0.25568e-01_rb,0.24303e-01_rb,0.24920e-01_rb,0.28049e-01_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.29422e-01_rb,0.28052e-01_rb,0.27653e-01_rb,0.27818e-01_rb,0.27831e-01_rb, &
        & 0.27375e-01_rb,0.26338e-01_rb,0.28129e-01_rb,0.31777e-01_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.32089e-01_rb,0.29221e-01_rb,0.26504e-01_rb,0.24372e-01_rb,0.22322e-01_rb, &
        & 0.20541e-01_rb,0.19063e-01_rb,0.16413e-01_rb,0.17368e-01_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.31950e-01_rb,0.29299e-01_rb,0.26979e-01_rb,0.25078e-01_rb,0.23422e-01_rb, &
        & 0.22454e-01_rb,0.20735e-01_rb,0.18244e-01_rb,0.19956e-01_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.31786e-01_rb,0.29316e-01_rb,0.27496e-01_rb,0.25889e-01_rb,0.24965e-01_rb, &
        & 0.24089e-01_rb,0.22376e-01_rb,0.20693e-01_rb,0.22993e-01_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.31564e-01_rb,0.29393e-01_rb,0.27942e-01_rb,0.27000e-01_rb,0.26402e-01_rb, &
        & 0.25715e-01_rb,0.24406e-01_rb,0.23371e-01_rb,0.26093e-01_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.31295e-01_rb,0.29526e-01_rb,0.28627e-01_rb,0.28150e-01_rb,0.27951e-01_rb, &
        & 0.27569e-01_rb,0.26423e-01_rb,0.26332e-01_rb,0.29505e-01_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.33998e-01_rb,0.30694e-01_rb,0.27710e-01_rb,0.25291e-01_rb,0.22809e-01_rb, &
        & 0.20329e-01_rb,0.18660e-01_rb,0.16131e-01_rb,0.15800e-01_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.33852e-01_rb,0.30861e-01_rb,0.28097e-01_rb,0.25930e-01_rb,0.23654e-01_rb, &
        & 0.22029e-01_rb,0.20474e-01_rb,0.17896e-01_rb,0.18339e-01_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.33685e-01_rb,0.30889e-01_rb,0.28668e-01_rb,0.26585e-01_rb,0.25018e-01_rb, &
        & 0.23681e-01_rb,0.22215e-01_rb,0.19897e-01_rb,0.21224e-01_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.33431e-01_rb,0.30933e-01_rb,0.29107e-01_rb,0.27570e-01_rb,0.26341e-01_rb, &
        & 0.25360e-01_rb,0.24234e-01_rb,0.22138e-01_rb,0.24217e-01_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.33153e-01_rb,0.31115e-01_rb,0.29708e-01_rb,0.28626e-01_rb,0.27757e-01_rb, &
        & 0.27318e-01_rb,0.26191e-01_rb,0.24666e-01_rb,0.27426e-01_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.35630e-01_rb,0.31962e-01_rb,0.28969e-01_rb,0.26191e-01_rb,0.23359e-01_rb, &
        & 0.20419e-01_rb,0.18025e-01_rb,0.15529e-01_rb,0.14046e-01_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.35465e-01_rb,0.32227e-01_rb,0.29248e-01_rb,0.26845e-01_rb,0.24076e-01_rb, &
        & 0.21854e-01_rb,0.19842e-01_rb,0.17371e-01_rb,0.16514e-01_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.35239e-01_rb,0.32283e-01_rb,0.29869e-01_rb,0.27436e-01_rb,0.25256e-01_rb, &
        & 0.23352e-01_rb,0.21638e-01_rb,0.19381e-01_rb,0.19322e-01_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.34959e-01_rb,0.32275e-01_rb,0.30321e-01_rb,0.28325e-01_rb,0.26494e-01_rb, &
        & 0.24825e-01_rb,0.23722e-01_rb,0.21526e-01_rb,0.22221e-01_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.34633e-01_rb,0.32463e-01_rb,0.30867e-01_rb,0.29345e-01_rb,0.27730e-01_rb, &
        & 0.26661e-01_rb,0.25760e-01_rb,0.23698e-01_rb,0.25325e-01_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.36611e-01_rb,0.32766e-01_rb,0.29862e-01_rb,0.26964e-01_rb,0.23985e-01_rb, &
        & 0.20572e-01_rb,0.17499e-01_rb,0.14612e-01_rb,0.12200e-01_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.36404e-01_rb,0.32931e-01_rb,0.30135e-01_rb,0.27680e-01_rb,0.24618e-01_rb, &
        & 0.21792e-01_rb,0.19087e-01_rb,0.16488e-01_rb,0.14408e-01_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.36151e-01_rb,0.33151e-01_rb,0.30783e-01_rb,0.28229e-01_rb,0.25608e-01_rb, &
        & 0.23215e-01_rb,0.20762e-01_rb,0.18415e-01_rb,0.17025e-01_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.35835e-01_rb,0.33169e-01_rb,0.31261e-01_rb,0.29020e-01_rb,0.26813e-01_rb, &
        & 0.24559e-01_rb,0.22662e-01_rb,0.20622e-01_rb,0.19846e-01_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.35491e-01_rb,0.33330e-01_rb,0.31756e-01_rb,0.30047e-01_rb,0.27916e-01_rb, &
        & 0.26189e-01_rb,0.24713e-01_rb,0.22859e-01_rb,0.22839e-01_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.36793e-01_rb,0.32943e-01_rb,0.30074e-01_rb,0.27299e-01_rb,0.24319e-01_rb, &
        & 0.20802e-01_rb,0.17113e-01_rb,0.13465e-01_rb,0.10508e-01_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.36572e-01_rb,0.33074e-01_rb,0.30511e-01_rb,0.28032e-01_rb,0.24986e-01_rb, &
        & 0.21844e-01_rb,0.18570e-01_rb,0.15283e-01_rb,0.12467e-01_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.36275e-01_rb,0.33234e-01_rb,0.31161e-01_rb,0.28602e-01_rb,0.25904e-01_rb, &
        & 0.23164e-01_rb,0.20091e-01_rb,0.17151e-01_rb,0.14775e-01_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.35925e-01_rb,0.33332e-01_rb,0.31658e-01_rb,0.29367e-01_rb,0.27125e-01_rb, &
        & 0.24431e-01_rb,0.21734e-01_rb,0.19261e-01_rb,0.17350e-01_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.35558e-01_rb,0.33506e-01_rb,0.32097e-01_rb,0.30412e-01_rb,0.28219e-01_rb, &
        & 0.25923e-01_rb,0.23598e-01_rb,0.21456e-01_rb,0.20114e-01_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.36293e-01_rb,0.32550e-01_rb,0.29721e-01_rb,0.27154e-01_rb,0.24076e-01_rb, &
        & 0.20693e-01_rb,0.16829e-01_rb,0.12555e-01_rb,0.89579e-02_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.35985e-01_rb,0.32624e-01_rb,0.30270e-01_rb,0.27879e-01_rb,0.24830e-01_rb, &
        & 0.21669e-01_rb,0.18126e-01_rb,0.14126e-01_rb,0.10649e-01_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.35646e-01_rb,0.32730e-01_rb,0.30946e-01_rb,0.28474e-01_rb,0.25750e-01_rb, &
        & 0.23017e-01_rb,0.19564e-01_rb,0.15741e-01_rb,0.12721e-01_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.35279e-01_rb,0.32754e-01_rb,0.31424e-01_rb,0.29187e-01_rb,0.27070e-01_rb, &
        & 0.24316e-01_rb,0.21018e-01_rb,0.17682e-01_rb,0.15011e-01_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.34858e-01_rb,0.32947e-01_rb,0.31835e-01_rb,0.30265e-01_rb,0.28236e-01_rb, &
        & 0.25710e-01_rb,0.22721e-01_rb,0.19777e-01_rb,0.17474e-01_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.35060e-01_rb,0.31556e-01_rb,0.29012e-01_rb,0.26469e-01_rb,0.23391e-01_rb, &
        & 0.20141e-01_rb,0.16295e-01_rb,0.11885e-01_rb,0.58937e-02_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.34721e-01_rb,0.31589e-01_rb,0.29506e-01_rb,0.27211e-01_rb,0.24267e-01_rb, &
        & 0.21055e-01_rb,0.17547e-01_rb,0.13263e-01_rb,0.69908e-02_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.34339e-01_rb,0.31609e-01_rb,0.30075e-01_rb,0.27841e-01_rb,0.25163e-01_rb, &
        & 0.22398e-01_rb,0.18997e-01_rb,0.14640e-01_rb,0.92447e-02_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.33936e-01_rb,0.31619e-01_rb,0.30603e-01_rb,0.28503e-01_rb,0.26490e-01_rb, &
        & 0.23804e-01_rb,0.20342e-01_rb,0.16280e-01_rb,0.11993e-01_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.33530e-01_rb,0.31821e-01_rb,0.30991e-01_rb,0.29578e-01_rb,0.27714e-01_rb, &
        & 0.25218e-01_rb,0.21886e-01_rb,0.18137e-01_rb,0.14941e-01_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.33129e-01_rb,0.29964e-01_rb,0.27856e-01_rb,0.25331e-01_rb,0.22401e-01_rb, &
        & 0.19295e-01_rb,0.15555e-01_rb,0.11348e-01_rb,0.74286e-02_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.32749e-01_rb,0.29950e-01_rb,0.28249e-01_rb,0.25998e-01_rb,0.23402e-01_rb, &
        & 0.20209e-01_rb,0.16757e-01_rb,0.12616e-01_rb,0.86653e-02_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.32344e-01_rb,0.29933e-01_rb,0.28722e-01_rb,0.26736e-01_rb,0.24275e-01_rb, &
        & 0.21508e-01_rb,0.18229e-01_rb,0.13852e-01_rb,0.98082e-02_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.31944e-01_rb,0.30002e-01_rb,0.29235e-01_rb,0.27435e-01_rb,0.25548e-01_rb, &
        & 0.22964e-01_rb,0.19579e-01_rb,0.15278e-01_rb,0.11352e-01_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.31533e-01_rb,0.30276e-01_rb,0.29685e-01_rb,0.28539e-01_rb,0.26795e-01_rb, &
        & 0.24358e-01_rb,0.21106e-01_rb,0.16900e-01_rb,0.12858e-01_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.30476e-01_rb,0.27848e-01_rb,0.26383e-01_rb,0.23991e-01_rb,0.21505e-01_rb, &
        & 0.18546e-01_rb,0.15127e-01_rb,0.11242e-01_rb,0.84064e-02_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.30069e-01_rb,0.27873e-01_rb,0.26695e-01_rb,0.24743e-01_rb,0.22441e-01_rb, &
        & 0.19591e-01_rb,0.16445e-01_rb,0.12465e-01_rb,0.99520e-02_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.29683e-01_rb,0.27938e-01_rb,0.27199e-01_rb,0.25464e-01_rb,0.23474e-01_rb, &
        & 0.21008e-01_rb,0.17841e-01_rb,0.13636e-01_rb,0.11012e-01_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.29281e-01_rb,0.28147e-01_rb,0.27612e-01_rb,0.26457e-01_rb,0.24810e-01_rb, &
        & 0.22367e-01_rb,0.19198e-01_rb,0.15026e-01_rb,0.12764e-01_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.28878e-01_rb,0.28450e-01_rb,0.28275e-01_rb,0.27607e-01_rb,0.26039e-01_rb, &
        & 0.23769e-01_rb,0.20798e-01_rb,0.16479e-01_rb,0.14486e-01_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.27483e-01_rb,0.25515e-01_rb,0.24398e-01_rb,0.22423e-01_rb,0.20204e-01_rb, &
        & 0.17532e-01_rb,0.14597e-01_rb,0.10977e-01_rb,0.89256e-02_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.27095e-01_rb,0.25634e-01_rb,0.24782e-01_rb,0.23175e-01_rb,0.21102e-01_rb, &
        & 0.18740e-01_rb,0.15902e-01_rb,0.12114e-01_rb,0.10252e-01_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.26727e-01_rb,0.25761e-01_rb,0.25266e-01_rb,0.23974e-01_rb,0.22361e-01_rb, &
        & 0.20171e-01_rb,0.17193e-01_rb,0.13275e-01_rb,0.11451e-01_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.26341e-01_rb,0.26039e-01_rb,0.25768e-01_rb,0.25146e-01_rb,0.23693e-01_rb, &
        & 0.21452e-01_rb,0.18566e-01_rb,0.14626e-01_rb,0.13159e-01_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.25958e-01_rb,0.26232e-01_rb,0.26607e-01_rb,0.26277e-01_rb,0.24926e-01_rb, &
        & 0.22906e-01_rb,0.20118e-01_rb,0.16018e-01_rb,0.14784e-01_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.24404e-01_rb,0.23195e-01_rb,0.22191e-01_rb,0.20664e-01_rb,0.18645e-01_rb, &
        & 0.16397e-01_rb,0.13889e-01_rb,0.10532e-01_rb,0.85762e-02_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.24045e-01_rb,0.23274e-01_rb,0.22704e-01_rb,0.21382e-01_rb,0.19651e-01_rb, &
        & 0.17693e-01_rb,0.15133e-01_rb,0.11576e-01_rb,0.96068e-02_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.23688e-01_rb,0.23458e-01_rb,0.23185e-01_rb,0.22281e-01_rb,0.21010e-01_rb, &
        & 0.19021e-01_rb,0.16312e-01_rb,0.12755e-01_rb,0.10813e-01_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.23340e-01_rb,0.23719e-01_rb,0.23835e-01_rb,0.23529e-01_rb,0.22269e-01_rb, &
        & 0.20297e-01_rb,0.17677e-01_rb,0.14063e-01_rb,0.12469e-01_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.23007e-01_rb,0.23887e-01_rb,0.24745e-01_rb,0.24636e-01_rb,0.23538e-01_rb, &
        & 0.21795e-01_rb,0.19163e-01_rb,0.15452e-01_rb,0.13866e-01_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.38207e-01_rb,0.35023e-01_rb,0.31764e-01_rb,0.29513e-01_rb,0.27148e-01_rb, &
        & 0.25121e-01_rb,0.22678e-01_rb,0.23928e-01_rb,0.26899e-01_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.37930e-01_rb,0.35272e-01_rb,0.32250e-01_rb,0.30760e-01_rb,0.28557e-01_rb, &
        & 0.27105e-01_rb,0.25089e-01_rb,0.27604e-01_rb,0.31070e-01_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.37611e-01_rb,0.35491e-01_rb,0.32949e-01_rb,0.31869e-01_rb,0.30632e-01_rb, &
        & 0.29068e-01_rb,0.27897e-01_rb,0.31369e-01_rb,0.35419e-01_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.37244e-01_rb,0.35566e-01_rb,0.33874e-01_rb,0.33253e-01_rb,0.32466e-01_rb, &
        & 0.31095e-01_rb,0.31358e-01_rb,0.35716e-01_rb,0.40368e-01_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.36895e-01_rb,0.35750e-01_rb,0.34817e-01_rb,0.35038e-01_rb,0.34265e-01_rb, &
        & 0.33428e-01_rb,0.35709e-01_rb,0.40899e-01_rb,0.46337e-01_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.39640e-01_rb,0.36208e-01_rb,0.33140e-01_rb,0.30731e-01_rb,0.28621e-01_rb, &
        & 0.26451e-01_rb,0.23824e-01_rb,0.23418e-01_rb,0.26401e-01_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.39384e-01_rb,0.36543e-01_rb,0.33542e-01_rb,0.32043e-01_rb,0.30032e-01_rb, &
        & 0.28484e-01_rb,0.26191e-01_rb,0.27027e-01_rb,0.30540e-01_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.39069e-01_rb,0.36650e-01_rb,0.34340e-01_rb,0.33119e-01_rb,0.32037e-01_rb, &
        & 0.30787e-01_rb,0.28410e-01_rb,0.30659e-01_rb,0.34733e-01_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.38756e-01_rb,0.36738e-01_rb,0.35220e-01_rb,0.34547e-01_rb,0.34264e-01_rb, &
        & 0.32872e-01_rb,0.31204e-01_rb,0.34827e-01_rb,0.39529e-01_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.38432e-01_rb,0.36958e-01_rb,0.36077e-01_rb,0.36343e-01_rb,0.36389e-01_rb, &
        & 0.35222e-01_rb,0.34916e-01_rb,0.39622e-01_rb,0.45018e-01_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.41762e-01_rb,0.37801e-01_rb,0.34598e-01_rb,0.31385e-01_rb,0.29542e-01_rb, &
        & 0.27120e-01_rb,0.24296e-01_rb,0.22163e-01_rb,0.24873e-01_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.41521e-01_rb,0.38107e-01_rb,0.34885e-01_rb,0.32740e-01_rb,0.30892e-01_rb, &
        & 0.28927e-01_rb,0.26874e-01_rb,0.25608e-01_rb,0.28828e-01_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.41284e-01_rb,0.38250e-01_rb,0.35590e-01_rb,0.33929e-01_rb,0.32637e-01_rb, &
        & 0.31412e-01_rb,0.29268e-01_rb,0.29038e-01_rb,0.32851e-01_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.40996e-01_rb,0.38286e-01_rb,0.36402e-01_rb,0.35356e-01_rb,0.34868e-01_rb, &
        & 0.33799e-01_rb,0.31619e-01_rb,0.33158e-01_rb,0.37659e-01_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.40710e-01_rb,0.38497e-01_rb,0.37108e-01_rb,0.37161e-01_rb,0.37098e-01_rb, &
        & 0.36226e-01_rb,0.34568e-01_rb,0.37733e-01_rb,0.42953e-01_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.44432e-01_rb,0.39911e-01_rb,0.36533e-01_rb,0.32358e-01_rb,0.30116e-01_rb, &
        & 0.27425e-01_rb,0.24391e-01_rb,0.21003e-01_rb,0.22873e-01_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.44241e-01_rb,0.40248e-01_rb,0.36750e-01_rb,0.33734e-01_rb,0.31371e-01_rb, &
        & 0.29050e-01_rb,0.26929e-01_rb,0.23993e-01_rb,0.26696e-01_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.43986e-01_rb,0.40474e-01_rb,0.37358e-01_rb,0.34847e-01_rb,0.32894e-01_rb, &
        & 0.31475e-01_rb,0.29486e-01_rb,0.27256e-01_rb,0.30632e-01_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.43750e-01_rb,0.40487e-01_rb,0.38107e-01_rb,0.36115e-01_rb,0.35063e-01_rb, &
        & 0.33924e-01_rb,0.31967e-01_rb,0.31144e-01_rb,0.35198e-01_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.43427e-01_rb,0.40610e-01_rb,0.38702e-01_rb,0.37867e-01_rb,0.37251e-01_rb, &
        & 0.36357e-01_rb,0.34995e-01_rb,0.35502e-01_rb,0.40197e-01_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.47176e-01_rb,0.42142e-01_rb,0.38550e-01_rb,0.33884e-01_rb,0.30889e-01_rb, &
        & 0.27516e-01_rb,0.24267e-01_rb,0.20680e-01_rb,0.20851e-01_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.47021e-01_rb,0.42363e-01_rb,0.38770e-01_rb,0.35164e-01_rb,0.32104e-01_rb, &
        & 0.29005e-01_rb,0.26645e-01_rb,0.23247e-01_rb,0.24505e-01_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.46797e-01_rb,0.42763e-01_rb,0.39235e-01_rb,0.36269e-01_rb,0.33395e-01_rb, &
        & 0.31307e-01_rb,0.29249e-01_rb,0.25924e-01_rb,0.28324e-01_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.46572e-01_rb,0.42912e-01_rb,0.40018e-01_rb,0.37361e-01_rb,0.35308e-01_rb, &
        & 0.33789e-01_rb,0.31702e-01_rb,0.29304e-01_rb,0.32735e-01_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.46255e-01_rb,0.42923e-01_rb,0.40617e-01_rb,0.38829e-01_rb,0.37399e-01_rb, &
        & 0.36167e-01_rb,0.34686e-01_rb,0.33312e-01_rb,0.37380e-01_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.49975e-01_rb,0.44361e-01_rb,0.40341e-01_rb,0.35539e-01_rb,0.31829e-01_rb, &
        & 0.27816e-01_rb,0.23845e-01_rb,0.20093e-01_rb,0.18543e-01_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.49828e-01_rb,0.44558e-01_rb,0.40763e-01_rb,0.36628e-01_rb,0.33050e-01_rb, &
        & 0.29095e-01_rb,0.26117e-01_rb,0.22673e-01_rb,0.22099e-01_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.49608e-01_rb,0.44841e-01_rb,0.41085e-01_rb,0.37793e-01_rb,0.34135e-01_rb, &
        & 0.31097e-01_rb,0.28755e-01_rb,0.25283e-01_rb,0.25831e-01_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.49365e-01_rb,0.45144e-01_rb,0.41922e-01_rb,0.38723e-01_rb,0.35845e-01_rb, &
        & 0.33431e-01_rb,0.31237e-01_rb,0.28118e-01_rb,0.29954e-01_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.49026e-01_rb,0.45249e-01_rb,0.42535e-01_rb,0.39993e-01_rb,0.37821e-01_rb, &
        & 0.35671e-01_rb,0.34099e-01_rb,0.31399e-01_rb,0.34366e-01_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.52410e-01_rb,0.46362e-01_rb,0.41901e-01_rb,0.37210e-01_rb,0.32800e-01_rb, &
        & 0.28263e-01_rb,0.23499e-01_rb,0.19230e-01_rb,0.16237e-01_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.52214e-01_rb,0.46505e-01_rb,0.42451e-01_rb,0.38113e-01_rb,0.34021e-01_rb, &
        & 0.29480e-01_rb,0.25464e-01_rb,0.21812e-01_rb,0.19595e-01_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.51982e-01_rb,0.46700e-01_rb,0.42867e-01_rb,0.39292e-01_rb,0.35048e-01_rb, &
        & 0.31222e-01_rb,0.27987e-01_rb,0.24489e-01_rb,0.23138e-01_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.51660e-01_rb,0.46904e-01_rb,0.43697e-01_rb,0.40172e-01_rb,0.36555e-01_rb, &
        & 0.33390e-01_rb,0.30506e-01_rb,0.27287e-01_rb,0.27036e-01_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.51284e-01_rb,0.47053e-01_rb,0.44408e-01_rb,0.41338e-01_rb,0.38429e-01_rb, &
        & 0.35446e-01_rb,0.33297e-01_rb,0.30383e-01_rb,0.31263e-01_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.53849e-01_rb,0.47597e-01_rb,0.43012e-01_rb,0.38373e-01_rb,0.33860e-01_rb, &
        & 0.28679e-01_rb,0.23391e-01_rb,0.18151e-01_rb,0.14106e-01_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.53641e-01_rb,0.47671e-01_rb,0.43475e-01_rb,0.39445e-01_rb,0.35042e-01_rb, &
        & 0.29924e-01_rb,0.25055e-01_rb,0.20655e-01_rb,0.17171e-01_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.53375e-01_rb,0.47826e-01_rb,0.44020e-01_rb,0.40653e-01_rb,0.36059e-01_rb, &
        & 0.31447e-01_rb,0.27345e-01_rb,0.23357e-01_rb,0.20383e-01_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.53001e-01_rb,0.48011e-01_rb,0.44999e-01_rb,0.41558e-01_rb,0.37354e-01_rb, &
        & 0.33601e-01_rb,0.29695e-01_rb,0.26106e-01_rb,0.24004e-01_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.52592e-01_rb,0.48152e-01_rb,0.45845e-01_rb,0.42585e-01_rb,0.39198e-01_rb, &
        & 0.35588e-01_rb,0.32287e-01_rb,0.29155e-01_rb,0.27991e-01_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.54223e-01_rb,0.47973e-01_rb,0.43289e-01_rb,0.38882e-01_rb,0.34299e-01_rb, &
        & 0.28946e-01_rb,0.23303e-01_rb,0.17068e-01_rb,0.11192e-01_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.53940e-01_rb,0.47929e-01_rb,0.43759e-01_rb,0.39916e-01_rb,0.35622e-01_rb, &
        & 0.30345e-01_rb,0.24752e-01_rb,0.19298e-01_rb,0.14778e-01_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.53642e-01_rb,0.48095e-01_rb,0.44399e-01_rb,0.41241e-01_rb,0.36768e-01_rb, &
        & 0.31745e-01_rb,0.26845e-01_rb,0.21914e-01_rb,0.17754e-01_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.53257e-01_rb,0.48243e-01_rb,0.45322e-01_rb,0.42327e-01_rb,0.37988e-01_rb, &
        & 0.33799e-01_rb,0.29109e-01_rb,0.24533e-01_rb,0.21056e-01_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.52778e-01_rb,0.48340e-01_rb,0.46365e-01_rb,0.43326e-01_rb,0.39837e-01_rb, &
        & 0.35758e-01_rb,0.31465e-01_rb,0.27467e-01_rb,0.24721e-01_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.53564e-01_rb,0.47520e-01_rb,0.42897e-01_rb,0.38870e-01_rb,0.34096e-01_rb, &
        & 0.28762e-01_rb,0.23161e-01_rb,0.16420e-01_rb,0.10403e-01_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.53200e-01_rb,0.47370e-01_rb,0.43390e-01_rb,0.39812e-01_rb,0.35374e-01_rb, &
        & 0.30449e-01_rb,0.24605e-01_rb,0.18329e-01_rb,0.12727e-01_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.52804e-01_rb,0.47486e-01_rb,0.44077e-01_rb,0.41029e-01_rb,0.36828e-01_rb, &
        & 0.31909e-01_rb,0.26518e-01_rb,0.20700e-01_rb,0.14792e-01_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.52379e-01_rb,0.47585e-01_rb,0.44997e-01_rb,0.42270e-01_rb,0.38168e-01_rb, &
        & 0.33930e-01_rb,0.28762e-01_rb,0.23052e-01_rb,0.16495e-01_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.51859e-01_rb,0.47629e-01_rb,0.45989e-01_rb,0.43387e-01_rb,0.40080e-01_rb, &
        & 0.36007e-01_rb,0.30957e-01_rb,0.25773e-01_rb,0.18898e-01_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.51622e-01_rb,0.46074e-01_rb,0.42057e-01_rb,0.38618e-01_rb,0.33808e-01_rb, &
        & 0.28847e-01_rb,0.23288e-01_rb,0.16620e-01_rb,0.11907e-01_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.51193e-01_rb,0.45980e-01_rb,0.42719e-01_rb,0.39541e-01_rb,0.35253e-01_rb, &
        & 0.30566e-01_rb,0.24931e-01_rb,0.18565e-01_rb,0.13988e-01_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.50740e-01_rb,0.46089e-01_rb,0.43457e-01_rb,0.40804e-01_rb,0.36736e-01_rb, &
        & 0.32310e-01_rb,0.27095e-01_rb,0.20684e-01_rb,0.16681e-01_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.50228e-01_rb,0.46105e-01_rb,0.44454e-01_rb,0.41838e-01_rb,0.38478e-01_rb, &
        & 0.34580e-01_rb,0.29293e-01_rb,0.22846e-01_rb,0.18301e-01_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.49697e-01_rb,0.46259e-01_rb,0.45191e-01_rb,0.43198e-01_rb,0.40635e-01_rb, &
        & 0.36709e-01_rb,0.31434e-01_rb,0.25478e-01_rb,0.20947e-01_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.48647e-01_rb,0.43752e-01_rb,0.40707e-01_rb,0.37500e-01_rb,0.33089e-01_rb, &
        & 0.28489e-01_rb,0.23000e-01_rb,0.16785e-01_rb,0.12627e-01_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.48181e-01_rb,0.43711e-01_rb,0.41312e-01_rb,0.38500e-01_rb,0.34693e-01_rb, &
        & 0.30128e-01_rb,0.24939e-01_rb,0.18673e-01_rb,0.15167e-01_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.47678e-01_rb,0.43812e-01_rb,0.42104e-01_rb,0.39699e-01_rb,0.36175e-01_rb, &
        & 0.32199e-01_rb,0.27269e-01_rb,0.20637e-01_rb,0.17826e-01_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.47181e-01_rb,0.43914e-01_rb,0.43002e-01_rb,0.40767e-01_rb,0.38218e-01_rb, &
        & 0.34662e-01_rb,0.29455e-01_rb,0.22728e-01_rb,0.19785e-01_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.46685e-01_rb,0.44343e-01_rb,0.43686e-01_rb,0.42426e-01_rb,0.40422e-01_rb, &
        & 0.36842e-01_rb,0.31692e-01_rb,0.25217e-01_rb,0.22679e-01_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.44854e-01_rb,0.40724e-01_rb,0.38672e-01_rb,0.35614e-01_rb,0.31949e-01_rb, &
        & 0.27602e-01_rb,0.22498e-01_rb,0.16720e-01_rb,0.12798e-01_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.44381e-01_rb,0.40833e-01_rb,0.39190e-01_rb,0.36792e-01_rb,0.33534e-01_rb, &
        & 0.29330e-01_rb,0.24604e-01_rb,0.18629e-01_rb,0.15661e-01_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.43916e-01_rb,0.40995e-01_rb,0.40051e-01_rb,0.37975e-01_rb,0.35149e-01_rb, &
        & 0.31642e-01_rb,0.27046e-01_rb,0.20453e-01_rb,0.17792e-01_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.43441e-01_rb,0.41288e-01_rb,0.40871e-01_rb,0.39278e-01_rb,0.37365e-01_rb, &
        & 0.34126e-01_rb,0.29213e-01_rb,0.22513e-01_rb,0.19591e-01_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.42963e-01_rb,0.41916e-01_rb,0.41758e-01_rb,0.41144e-01_rb,0.39580e-01_rb, &
        & 0.36327e-01_rb,0.31558e-01_rb,0.24885e-01_rb,0.22715e-01_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.52315e-01_rb,0.47505e-01_rb,0.42884e-01_rb,0.38287e-01_rb,0.34721e-01_rb, &
        & 0.30888e-01_rb,0.29687e-01_rb,0.33559e-01_rb,0.37791e-01_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.52132e-01_rb,0.47869e-01_rb,0.43358e-01_rb,0.39519e-01_rb,0.36490e-01_rb, &
        & 0.33059e-01_rb,0.34560e-01_rb,0.39434e-01_rb,0.44537e-01_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.51859e-01_rb,0.48259e-01_rb,0.44108e-01_rb,0.41053e-01_rb,0.38159e-01_rb, &
        & 0.36325e-01_rb,0.40345e-01_rb,0.46345e-01_rb,0.52382e-01_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.51556e-01_rb,0.48650e-01_rb,0.44992e-01_rb,0.42544e-01_rb,0.40704e-01_rb, &
        & 0.40160e-01_rb,0.46427e-01_rb,0.53431e-01_rb,0.60544e-01_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.51137e-01_rb,0.48860e-01_rb,0.46168e-01_rb,0.44111e-01_rb,0.43209e-01_rb, &
        & 0.44827e-01_rb,0.52800e-01_rb,0.60972e-01_rb,0.69216e-01_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.56434e-01_rb,0.51118e-01_rb,0.46217e-01_rb,0.41561e-01_rb,0.37505e-01_rb, &
        & 0.33646e-01_rb,0.30189e-01_rb,0.33085e-01_rb,0.37424e-01_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.56174e-01_rb,0.51590e-01_rb,0.46758e-01_rb,0.42838e-01_rb,0.39459e-01_rb, &
        & 0.35755e-01_rb,0.34261e-01_rb,0.38732e-01_rb,0.43938e-01_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.55870e-01_rb,0.52113e-01_rb,0.47459e-01_rb,0.44557e-01_rb,0.41223e-01_rb, &
        & 0.38732e-01_rb,0.39807e-01_rb,0.45508e-01_rb,0.51693e-01_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.55457e-01_rb,0.52524e-01_rb,0.48507e-01_rb,0.46112e-01_rb,0.43732e-01_rb, &
        & 0.42045e-01_rb,0.45811e-01_rb,0.52732e-01_rb,0.60004e-01_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.54996e-01_rb,0.52680e-01_rb,0.49849e-01_rb,0.47721e-01_rb,0.46514e-01_rb, &
        & 0.45961e-01_rb,0.52270e-01_rb,0.60294e-01_rb,0.68714e-01_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.59831e-01_rb,0.53764e-01_rb,0.48883e-01_rb,0.44361e-01_rb,0.39553e-01_rb, &
        & 0.35958e-01_rb,0.31293e-01_rb,0.31467e-01_rb,0.35795e-01_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.59532e-01_rb,0.54423e-01_rb,0.49485e-01_rb,0.45554e-01_rb,0.41889e-01_rb, &
        & 0.38044e-01_rb,0.34149e-01_rb,0.36846e-01_rb,0.42078e-01_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.59134e-01_rb,0.55152e-01_rb,0.50116e-01_rb,0.47410e-01_rb,0.43907e-01_rb, &
        & 0.40732e-01_rb,0.38444e-01_rb,0.43145e-01_rb,0.49218e-01_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.58672e-01_rb,0.55655e-01_rb,0.51271e-01_rb,0.49059e-01_rb,0.46360e-01_rb, &
        & 0.44063e-01_rb,0.43633e-01_rb,0.49781e-01_rb,0.56746e-01_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.58134e-01_rb,0.55813e-01_rb,0.52716e-01_rb,0.50709e-01_rb,0.49278e-01_rb, &
        & 0.47427e-01_rb,0.49630e-01_rb,0.57031e-01_rb,0.65034e-01_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.62624e-01_rb,0.55864e-01_rb,0.50884e-01_rb,0.46641e-01_rb,0.41202e-01_rb, &
        & 0.37808e-01_rb,0.32728e-01_rb,0.29634e-01_rb,0.33571e-01_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.62339e-01_rb,0.56365e-01_rb,0.51618e-01_rb,0.47600e-01_rb,0.43888e-01_rb, &
        & 0.40000e-01_rb,0.35145e-01_rb,0.34591e-01_rb,0.39469e-01_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.61969e-01_rb,0.57107e-01_rb,0.52103e-01_rb,0.49599e-01_rb,0.46112e-01_rb, &
        & 0.42482e-01_rb,0.38680e-01_rb,0.40578e-01_rb,0.46387e-01_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.61515e-01_rb,0.57939e-01_rb,0.53160e-01_rb,0.51347e-01_rb,0.48544e-01_rb, &
        & 0.45843e-01_rb,0.42689e-01_rb,0.46767e-01_rb,0.53378e-01_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.60986e-01_rb,0.58238e-01_rb,0.54658e-01_rb,0.53027e-01_rb,0.51696e-01_rb, &
        & 0.49388e-01_rb,0.47217e-01_rb,0.53278e-01_rb,0.60785e-01_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.65795e-01_rb,0.58347e-01_rb,0.52585e-01_rb,0.48437e-01_rb,0.42714e-01_rb, &
        & 0.39265e-01_rb,0.34039e-01_rb,0.28235e-01_rb,0.31421e-01_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.65486e-01_rb,0.58633e-01_rb,0.53696e-01_rb,0.49213e-01_rb,0.45420e-01_rb, &
        & 0.41668e-01_rb,0.36378e-01_rb,0.32647e-01_rb,0.37055e-01_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.65183e-01_rb,0.59134e-01_rb,0.54134e-01_rb,0.51097e-01_rb,0.47829e-01_rb, &
        & 0.44052e-01_rb,0.39727e-01_rb,0.38129e-01_rb,0.43417e-01_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.64717e-01_rb,0.59894e-01_rb,0.54954e-01_rb,0.53041e-01_rb,0.50294e-01_rb, &
        & 0.47344e-01_rb,0.43654e-01_rb,0.43889e-01_rb,0.49916e-01_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.64255e-01_rb,0.60489e-01_rb,0.56373e-01_rb,0.54824e-01_rb,0.53472e-01_rb, &
        & 0.51112e-01_rb,0.47282e-01_rb,0.50031e-01_rb,0.57102e-01_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.69615e-01_rb,0.61594e-01_rb,0.54882e-01_rb,0.50187e-01_rb,0.44074e-01_rb, &
        & 0.39997e-01_rb,0.34871e-01_rb,0.27743e-01_rb,0.29013e-01_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.69442e-01_rb,0.61757e-01_rb,0.55910e-01_rb,0.50997e-01_rb,0.46497e-01_rb, &
        & 0.42613e-01_rb,0.37188e-01_rb,0.31216e-01_rb,0.34347e-01_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.69156e-01_rb,0.62106e-01_rb,0.56750e-01_rb,0.52515e-01_rb,0.49101e-01_rb, &
        & 0.44986e-01_rb,0.40395e-01_rb,0.35961e-01_rb,0.40369e-01_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.68816e-01_rb,0.62627e-01_rb,0.57231e-01_rb,0.54601e-01_rb,0.51362e-01_rb, &
        & 0.48280e-01_rb,0.44442e-01_rb,0.41156e-01_rb,0.46554e-01_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.68355e-01_rb,0.63113e-01_rb,0.58516e-01_rb,0.56347e-01_rb,0.54360e-01_rb, &
        & 0.52199e-01_rb,0.48222e-01_rb,0.46871e-01_rb,0.53241e-01_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.74032e-01_rb,0.65393e-01_rb,0.58013e-01_rb,0.51965e-01_rb,0.45915e-01_rb, &
        & 0.40526e-01_rb,0.35125e-01_rb,0.27883e-01_rb,0.26586e-01_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.73937e-01_rb,0.65493e-01_rb,0.58738e-01_rb,0.53263e-01_rb,0.47941e-01_rb, &
        & 0.43149e-01_rb,0.37585e-01_rb,0.30874e-01_rb,0.31614e-01_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.73662e-01_rb,0.65759e-01_rb,0.59591e-01_rb,0.54630e-01_rb,0.50354e-01_rb, &
        & 0.45514e-01_rb,0.40708e-01_rb,0.34889e-01_rb,0.37383e-01_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.73359e-01_rb,0.66187e-01_rb,0.60275e-01_rb,0.56599e-01_rb,0.52392e-01_rb, &
        & 0.48574e-01_rb,0.44791e-01_rb,0.39333e-01_rb,0.43356e-01_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.72923e-01_rb,0.66589e-01_rb,0.61432e-01_rb,0.58168e-01_rb,0.55043e-01_rb, &
        & 0.52561e-01_rb,0.48755e-01_rb,0.44300e-01_rb,0.49765e-01_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.78458e-01_rb,0.69177e-01_rb,0.61142e-01_rb,0.54247e-01_rb,0.47727e-01_rb, &
        & 0.41456e-01_rb,0.34819e-01_rb,0.27717e-01_rb,0.23941e-01_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.78410e-01_rb,0.69294e-01_rb,0.61857e-01_rb,0.55492e-01_rb,0.49764e-01_rb, &
        & 0.43844e-01_rb,0.37501e-01_rb,0.30644e-01_rb,0.28737e-01_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.78162e-01_rb,0.69463e-01_rb,0.62578e-01_rb,0.57026e-01_rb,0.52068e-01_rb, &
        & 0.46044e-01_rb,0.40472e-01_rb,0.34561e-01_rb,0.34380e-01_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.77871e-01_rb,0.69834e-01_rb,0.63268e-01_rb,0.59074e-01_rb,0.53918e-01_rb, &
        & 0.48672e-01_rb,0.44601e-01_rb,0.38671e-01_rb,0.40158e-01_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.77487e-01_rb,0.70209e-01_rb,0.64405e-01_rb,0.60632e-01_rb,0.56120e-01_rb, &
        & 0.52504e-01_rb,0.48787e-01_rb,0.42980e-01_rb,0.46410e-01_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.82666e-01_rb,0.72734e-01_rb,0.64105e-01_rb,0.56594e-01_rb,0.49561e-01_rb, &
        & 0.42281e-01_rb,0.34472e-01_rb,0.27105e-01_rb,0.21191e-01_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.82602e-01_rb,0.72866e-01_rb,0.64825e-01_rb,0.57780e-01_rb,0.51415e-01_rb, &
        & 0.44682e-01_rb,0.37233e-01_rb,0.30003e-01_rb,0.25697e-01_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.82349e-01_rb,0.72898e-01_rb,0.65478e-01_rb,0.59213e-01_rb,0.53844e-01_rb, &
        & 0.46925e-01_rb,0.39994e-01_rb,0.33830e-01_rb,0.31116e-01_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.82004e-01_rb,0.73222e-01_rb,0.66214e-01_rb,0.61303e-01_rb,0.55771e-01_rb, &
        & 0.49206e-01_rb,0.43963e-01_rb,0.38199e-01_rb,0.36759e-01_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.81608e-01_rb,0.73612e-01_rb,0.67180e-01_rb,0.63056e-01_rb,0.57653e-01_rb, &
        & 0.52653e-01_rb,0.48283e-01_rb,0.42441e-01_rb,0.42846e-01_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.85880e-01_rb,0.75430e-01_rb,0.66656e-01_rb,0.58773e-01_rb,0.51658e-01_rb, &
        & 0.43336e-01_rb,0.34393e-01_rb,0.26157e-01_rb,0.14663e-01_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.85756e-01_rb,0.75590e-01_rb,0.67337e-01_rb,0.60049e-01_rb,0.53266e-01_rb, &
        & 0.45473e-01_rb,0.37278e-01_rb,0.29125e-01_rb,0.16193e-01_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.85530e-01_rb,0.75588e-01_rb,0.67995e-01_rb,0.61462e-01_rb,0.55345e-01_rb, &
        & 0.48001e-01_rb,0.39939e-01_rb,0.32883e-01_rb,0.20331e-01_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.85095e-01_rb,0.75810e-01_rb,0.68684e-01_rb,0.63320e-01_rb,0.57485e-01_rb, &
        & 0.50257e-01_rb,0.43559e-01_rb,0.37443e-01_rb,0.23682e-01_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.84667e-01_rb,0.76211e-01_rb,0.69733e-01_rb,0.65095e-01_rb,0.59445e-01_rb, &
        & 0.53380e-01_rb,0.47785e-01_rb,0.41863e-01_rb,0.26616e-01_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.87471e-01_rb,0.76929e-01_rb,0.68322e-01_rb,0.60781e-01_rb,0.53951e-01_rb, &
        & 0.45056e-01_rb,0.35758e-01_rb,0.26371e-01_rb,0.18858e-01_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.87208e-01_rb,0.76940e-01_rb,0.68996e-01_rb,0.62212e-01_rb,0.55582e-01_rb, &
        & 0.47319e-01_rb,0.38590e-01_rb,0.29478e-01_rb,0.21575e-01_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.86835e-01_rb,0.76905e-01_rb,0.69723e-01_rb,0.63816e-01_rb,0.57699e-01_rb, &
        & 0.49745e-01_rb,0.41432e-01_rb,0.33605e-01_rb,0.25460e-01_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.86430e-01_rb,0.77295e-01_rb,0.70629e-01_rb,0.65834e-01_rb,0.59545e-01_rb, &
        & 0.52341e-01_rb,0.45178e-01_rb,0.38243e-01_rb,0.30364e-01_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.85874e-01_rb,0.77597e-01_rb,0.72138e-01_rb,0.67497e-01_rb,0.61709e-01_rb, &
        & 0.55855e-01_rb,0.49191e-01_rb,0.42789e-01_rb,0.33235e-01_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.87248e-01_rb,0.76929e-01_rb,0.68689e-01_rb,0.62001e-01_rb,0.55189e-01_rb, &
        & 0.46282e-01_rb,0.37018e-01_rb,0.26535e-01_rb,0.20617e-01_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.86938e-01_rb,0.76894e-01_rb,0.69591e-01_rb,0.63473e-01_rb,0.56879e-01_rb, &
        & 0.48866e-01_rb,0.39681e-01_rb,0.29869e-01_rb,0.23753e-01_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.86505e-01_rb,0.76936e-01_rb,0.70473e-01_rb,0.65213e-01_rb,0.59161e-01_rb, &
        & 0.51227e-01_rb,0.42751e-01_rb,0.34068e-01_rb,0.28028e-01_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.86014e-01_rb,0.77389e-01_rb,0.71734e-01_rb,0.67391e-01_rb,0.61041e-01_rb, &
        & 0.54172e-01_rb,0.46740e-01_rb,0.38524e-01_rb,0.33947e-01_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.85424e-01_rb,0.77682e-01_rb,0.73622e-01_rb,0.69056e-01_rb,0.63676e-01_rb, &
        & 0.57877e-01_rb,0.50578e-01_rb,0.43087e-01_rb,0.36933e-01_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.85419e-01_rb,0.75627e-01_rb,0.68076e-01_rb,0.62306e-01_rb,0.55288e-01_rb, &
        & 0.46939e-01_rb,0.37782e-01_rb,0.26869e-01_rb,0.20535e-01_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.84985e-01_rb,0.75524e-01_rb,0.69213e-01_rb,0.63729e-01_rb,0.57271e-01_rb, &
        & 0.49714e-01_rb,0.40473e-01_rb,0.30181e-01_rb,0.22959e-01_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.84529e-01_rb,0.75748e-01_rb,0.70315e-01_rb,0.65749e-01_rb,0.59660e-01_rb, &
        & 0.52260e-01_rb,0.43766e-01_rb,0.34359e-01_rb,0.28707e-01_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.84038e-01_rb,0.76252e-01_rb,0.71947e-01_rb,0.67890e-01_rb,0.61874e-01_rb, &
        & 0.55511e-01_rb,0.47935e-01_rb,0.38594e-01_rb,0.34330e-01_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.83450e-01_rb,0.76542e-01_rb,0.73924e-01_rb,0.69752e-01_rb,0.64960e-01_rb, &
        & 0.59356e-01_rb,0.51789e-01_rb,0.43079e-01_rb,0.38051e-01_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.68561e-01_rb,0.61393e-01_rb,0.55377e-01_rb,0.48367e-01_rb,0.42682e-01_rb, &
        & 0.39908e-01_rb,0.45275e-01_rb,0.51909e-01_rb,0.58629e-01_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.68429e-01_rb,0.62258e-01_rb,0.56444e-01_rb,0.49529e-01_rb,0.45167e-01_rb, &
        & 0.45643e-01_rb,0.53469e-01_rb,0.61626e-01_rb,0.69870e-01_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.68224e-01_rb,0.62810e-01_rb,0.57169e-01_rb,0.51186e-01_rb,0.48224e-01_rb, &
        & 0.52899e-01_rb,0.62764e-01_rb,0.72521e-01_rb,0.82353e-01_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.67917e-01_rb,0.63314e-01_rb,0.58187e-01_rb,0.53334e-01_rb,0.52038e-01_rb, &
        & 0.61434e-01_rb,0.73273e-01_rb,0.84850e-01_rb,0.96479e-01_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.67553e-01_rb,0.63903e-01_rb,0.59193e-01_rb,0.55737e-01_rb,0.58351e-01_rb, &
        & 0.71682e-01_rb,0.85665e-01_rb,0.99285e-01_rb,0.11295e+00_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.77509e-01_rb,0.69079e-01_rb,0.62486e-01_rb,0.54626e-01_rb,0.48075e-01_rb, &
        & 0.42537e-01_rb,0.44950e-01_rb,0.51477e-01_rb,0.58558e-01_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.77430e-01_rb,0.69950e-01_rb,0.63742e-01_rb,0.55764e-01_rb,0.50627e-01_rb, &
        & 0.46853e-01_rb,0.52532e-01_rb,0.60374e-01_rb,0.68733e-01_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.77189e-01_rb,0.70882e-01_rb,0.64573e-01_rb,0.57536e-01_rb,0.53311e-01_rb, &
        & 0.52406e-01_rb,0.61244e-01_rb,0.70634e-01_rb,0.80523e-01_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.76847e-01_rb,0.71588e-01_rb,0.65635e-01_rb,0.59928e-01_rb,0.56064e-01_rb, &
        & 0.60283e-01_rb,0.71500e-01_rb,0.82577e-01_rb,0.94220e-01_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.76386e-01_rb,0.72289e-01_rb,0.66796e-01_rb,0.62420e-01_rb,0.60016e-01_rb, &
        & 0.69350e-01_rb,0.82634e-01_rb,0.95798e-01_rb,0.10925e+00_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.86533e-01_rb,0.76872e-01_rb,0.69252e-01_rb,0.60999e-01_rb,0.53444e-01_rb, &
        & 0.46117e-01_rb,0.43982e-01_rb,0.49772e-01_rb,0.56991e-01_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.86412e-01_rb,0.77388e-01_rb,0.71047e-01_rb,0.62031e-01_rb,0.55847e-01_rb, &
        & 0.49560e-01_rb,0.50601e-01_rb,0.57831e-01_rb,0.66136e-01_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.86175e-01_rb,0.78253e-01_rb,0.72126e-01_rb,0.63678e-01_rb,0.58875e-01_rb, &
        & 0.53556e-01_rb,0.58645e-01_rb,0.67397e-01_rb,0.77181e-01_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.85802e-01_rb,0.79483e-01_rb,0.72991e-01_rb,0.66186e-01_rb,0.61530e-01_rb, &
        & 0.59229e-01_rb,0.68328e-01_rb,0.78749e-01_rb,0.90335e-01_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.85328e-01_rb,0.80671e-01_rb,0.74346e-01_rb,0.68871e-01_rb,0.64555e-01_rb, &
        & 0.67267e-01_rb,0.79478e-01_rb,0.91684e-01_rb,0.10533e+00_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.95394e-01_rb,0.84557e-01_rb,0.75442e-01_rb,0.67038e-01_rb,0.59073e-01_rb, &
        & 0.49886e-01_rb,0.44642e-01_rb,0.48508e-01_rb,0.55658e-01_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.95223e-01_rb,0.84873e-01_rb,0.77588e-01_rb,0.68316e-01_rb,0.60993e-01_rb, &
        & 0.53907e-01_rb,0.49824e-01_rb,0.55820e-01_rb,0.64074e-01_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.94965e-01_rb,0.85575e-01_rb,0.79290e-01_rb,0.69694e-01_rb,0.64420e-01_rb, &
        & 0.57295e-01_rb,0.56440e-01_rb,0.64306e-01_rb,0.73964e-01_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.94509e-01_rb,0.86370e-01_rb,0.80294e-01_rb,0.72257e-01_rb,0.67415e-01_rb, &
        & 0.61388e-01_rb,0.65357e-01_rb,0.74991e-01_rb,0.86332e-01_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.94009e-01_rb,0.87806e-01_rb,0.81595e-01_rb,0.75171e-01_rb,0.70245e-01_rb, &
        & 0.67536e-01_rb,0.76274e-01_rb,0.87845e-01_rb,0.10132e+00_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.10343e+00_rb,0.91525e-01_rb,0.80861e-01_rb,0.72378e-01_rb,0.64406e-01_rb, &
        & 0.53983e-01_rb,0.47036e-01_rb,0.47125e-01_rb,0.54095e-01_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.10334e+00_rb,0.91907e-01_rb,0.82762e-01_rb,0.74349e-01_rb,0.66049e-01_rb, &
        & 0.58213e-01_rb,0.51290e-01_rb,0.53990e-01_rb,0.61913e-01_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.10297e+00_rb,0.92357e-01_rb,0.85393e-01_rb,0.75564e-01_rb,0.69527e-01_rb, &
        & 0.62165e-01_rb,0.56265e-01_rb,0.62011e-01_rb,0.71236e-01_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.10256e+00_rb,0.93114e-01_rb,0.87093e-01_rb,0.77935e-01_rb,0.73042e-01_rb, &
        & 0.65986e-01_rb,0.63498e-01_rb,0.72109e-01_rb,0.83127e-01_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.10189e+00_rb,0.94299e-01_rb,0.88282e-01_rb,0.81218e-01_rb,0.76035e-01_rb, &
        & 0.70876e-01_rb,0.73640e-01_rb,0.84406e-01_rb,0.97393e-01_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.11045e+00_rb,0.97454e-01_rb,0.85408e-01_rb,0.76244e-01_rb,0.68972e-01_rb, &
        & 0.58251e-01_rb,0.49930e-01_rb,0.45374e-01_rb,0.51875e-01_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.11019e+00_rb,0.97819e-01_rb,0.87148e-01_rb,0.79069e-01_rb,0.71021e-01_rb, &
        & 0.62071e-01_rb,0.54179e-01_rb,0.52124e-01_rb,0.59618e-01_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.10981e+00_rb,0.98088e-01_rb,0.89384e-01_rb,0.81156e-01_rb,0.73973e-01_rb, &
        & 0.66762e-01_rb,0.58141e-01_rb,0.59499e-01_rb,0.68391e-01_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.10923e+00_rb,0.98741e-01_rb,0.91856e-01_rb,0.83093e-01_rb,0.78121e-01_rb, &
        & 0.70717e-01_rb,0.63590e-01_rb,0.68985e-01_rb,0.79490e-01_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.10855e+00_rb,0.99728e-01_rb,0.93768e-01_rb,0.86485e-01_rb,0.81499e-01_rb, &
        & 0.75384e-01_rb,0.71738e-01_rb,0.80634e-01_rb,0.92951e-01_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.11652e+00_rb,0.10245e+00_rb,0.89345e-01_rb,0.79831e-01_rb,0.71546e-01_rb, &
        & 0.62110e-01_rb,0.52889e-01_rb,0.43937e-01_rb,0.49094e-01_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.11624e+00_rb,0.10291e+00_rb,0.90769e-01_rb,0.82143e-01_rb,0.74705e-01_rb, &
        & 0.65752e-01_rb,0.57383e-01_rb,0.50125e-01_rb,0.56751e-01_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.11594e+00_rb,0.10317e+00_rb,0.92734e-01_rb,0.84722e-01_rb,0.77935e-01_rb, &
        & 0.70772e-01_rb,0.61416e-01_rb,0.57153e-01_rb,0.65344e-01_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.11535e+00_rb,0.10356e+00_rb,0.95024e-01_rb,0.87242e-01_rb,0.82357e-01_rb, &
        & 0.75212e-01_rb,0.65866e-01_rb,0.66110e-01_rb,0.76048e-01_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.11474e+00_rb,0.10447e+00_rb,0.97059e-01_rb,0.90869e-01_rb,0.86343e-01_rb, &
        & 0.79806e-01_rb,0.72467e-01_rb,0.77014e-01_rb,0.88751e-01_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.12265e+00_rb,0.10756e+00_rb,0.93593e-01_rb,0.82964e-01_rb,0.73576e-01_rb, &
        & 0.64374e-01_rb,0.55605e-01_rb,0.43559e-01_rb,0.46060e-01_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.12249e+00_rb,0.10812e+00_rb,0.94575e-01_rb,0.84973e-01_rb,0.76683e-01_rb, &
        & 0.68823e-01_rb,0.60052e-01_rb,0.49117e-01_rb,0.53798e-01_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.12216e+00_rb,0.10834e+00_rb,0.96338e-01_rb,0.87360e-01_rb,0.80349e-01_rb, &
        & 0.74206e-01_rb,0.64654e-01_rb,0.55074e-01_rb,0.62031e-01_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.12168e+00_rb,0.10860e+00_rb,0.98251e-01_rb,0.89888e-01_rb,0.85251e-01_rb, &
        & 0.79126e-01_rb,0.69120e-01_rb,0.63252e-01_rb,0.72334e-01_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.12099e+00_rb,0.10931e+00_rb,0.10017e+00_rb,0.93685e-01_rb,0.90048e-01_rb, &
        & 0.83711e-01_rb,0.75096e-01_rb,0.73779e-01_rb,0.84712e-01_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.12914e+00_rb,0.11313e+00_rb,0.98462e-01_rb,0.86441e-01_rb,0.75875e-01_rb, &
        & 0.66332e-01_rb,0.57013e-01_rb,0.43839e-01_rb,0.42411e-01_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.12911e+00_rb,0.11365e+00_rb,0.99113e-01_rb,0.88386e-01_rb,0.78651e-01_rb, &
        & 0.70676e-01_rb,0.61767e-01_rb,0.49076e-01_rb,0.50412e-01_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.12886e+00_rb,0.11401e+00_rb,0.10056e+00_rb,0.90400e-01_rb,0.82030e-01_rb, &
        & 0.75969e-01_rb,0.67212e-01_rb,0.54346e-01_rb,0.58598e-01_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.12844e+00_rb,0.11410e+00_rb,0.10221e+00_rb,0.92662e-01_rb,0.86791e-01_rb, &
        & 0.81685e-01_rb,0.71813e-01_rb,0.61251e-01_rb,0.68599e-01_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.12781e+00_rb,0.11455e+00_rb,0.10390e+00_rb,0.96184e-01_rb,0.92217e-01_rb, &
        & 0.86557e-01_rb,0.77640e-01_rb,0.70636e-01_rb,0.80447e-01_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.13661e+00_rb,0.11963e+00_rb,0.10401e+00_rb,0.90612e-01_rb,0.78552e-01_rb, &
        & 0.68665e-01_rb,0.58042e-01_rb,0.44182e-01_rb,0.20623e-01_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.13663e+00_rb,0.12000e+00_rb,0.10449e+00_rb,0.92427e-01_rb,0.81459e-01_rb, &
        & 0.72606e-01_rb,0.62588e-01_rb,0.49634e-01_rb,0.27517e-01_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.13648e+00_rb,0.12048e+00_rb,0.10572e+00_rb,0.94344e-01_rb,0.84635e-01_rb, &
        & 0.77264e-01_rb,0.68466e-01_rb,0.54964e-01_rb,0.37460e-01_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.13615e+00_rb,0.12055e+00_rb,0.10724e+00_rb,0.96538e-01_rb,0.88921e-01_rb, &
        & 0.82919e-01_rb,0.73837e-01_rb,0.61121e-01_rb,0.51408e-01_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.13560e+00_rb,0.12083e+00_rb,0.10867e+00_rb,0.99719e-01_rb,0.94157e-01_rb, &
        & 0.88097e-01_rb,0.79982e-01_rb,0.69630e-01_rb,0.68524e-01_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.14486e+00_rb,0.12685e+00_rb,0.11053e+00_rb,0.96215e-01_rb,0.83198e-01_rb, &
        & 0.72790e-01_rb,0.60483e-01_rb,0.46368e-01_rb,0.23123e-01_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.14497e+00_rb,0.12740e+00_rb,0.11115e+00_rb,0.98058e-01_rb,0.86138e-01_rb, &
        & 0.76317e-01_rb,0.65283e-01_rb,0.52226e-01_rb,0.29160e-01_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.14473e+00_rb,0.12775e+00_rb,0.11243e+00_rb,0.99891e-01_rb,0.89493e-01_rb, &
        & 0.80996e-01_rb,0.71303e-01_rb,0.57950e-01_rb,0.32977e-01_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.14430e+00_rb,0.12774e+00_rb,0.11386e+00_rb,0.10223e+00_rb,0.94098e-01_rb, &
        & 0.86252e-01_rb,0.77087e-01_rb,0.64785e-01_rb,0.38714e-01_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.14365e+00_rb,0.12806e+00_rb,0.11504e+00_rb,0.10578e+00_rb,0.98766e-01_rb, &
        & 0.91283e-01_rb,0.84242e-01_rb,0.73581e-01_rb,0.47497e-01_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.15269e+00_rb,0.13372e+00_rb,0.11691e+00_rb,0.10167e+00_rb,0.88459e-01_rb, &
        & 0.76634e-01_rb,0.62620e-01_rb,0.48153e-01_rb,0.27858e-01_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.15257e+00_rb,0.13415e+00_rb,0.11750e+00_rb,0.10380e+00_rb,0.91589e-01_rb, &
        & 0.79898e-01_rb,0.67862e-01_rb,0.54271e-01_rb,0.35587e-01_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.15233e+00_rb,0.13449e+00_rb,0.11899e+00_rb,0.10589e+00_rb,0.94966e-01_rb, &
        & 0.84666e-01_rb,0.73653e-01_rb,0.60551e-01_rb,0.39809e-01_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.15175e+00_rb,0.13439e+00_rb,0.12031e+00_rb,0.10854e+00_rb,0.99506e-01_rb, &
        & 0.89389e-01_rb,0.79517e-01_rb,0.68401e-01_rb,0.45578e-01_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.15105e+00_rb,0.13484e+00_rb,0.12149e+00_rb,0.11242e+00_rb,0.10353e+00_rb, &
        & 0.94548e-01_rb,0.87485e-01_rb,0.77323e-01_rb,0.55990e-01_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.15929e+00_rb,0.13953e+00_rb,0.12252e+00_rb,0.10682e+00_rb,0.93710e-01_rb, &
        & 0.80318e-01_rb,0.64679e-01_rb,0.49483e-01_rb,0.31231e-01_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.15914e+00_rb,0.14001e+00_rb,0.12322e+00_rb,0.10931e+00_rb,0.96798e-01_rb, &
        & 0.83533e-01_rb,0.70039e-01_rb,0.55928e-01_rb,0.39440e-01_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.15875e+00_rb,0.14019e+00_rb,0.12484e+00_rb,0.11161e+00_rb,0.10034e+00_rb, &
        & 0.88309e-01_rb,0.75681e-01_rb,0.62757e-01_rb,0.43280e-01_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.15815e+00_rb,0.14019e+00_rb,0.12619e+00_rb,0.11467e+00_rb,0.10492e+00_rb, &
        & 0.92849e-01_rb,0.81796e-01_rb,0.71221e-01_rb,0.50343e-01_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.15737e+00_rb,0.14077e+00_rb,0.12756e+00_rb,0.11888e+00_rb,0.10868e+00_rb, &
        & 0.98192e-01_rb,0.90106e-01_rb,0.80417e-01_rb,0.60332e-01_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.83965e-01_rb,0.74725e-01_rb,0.67262e-01_rb,0.61439e-01_rb,0.63569e-01_rb, &
        & 0.76568e-01_rb,0.91505e-01_rb,0.10604e+00_rb,0.12066e+00_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.84124e-01_rb,0.75639e-01_rb,0.69121e-01_rb,0.66761e-01_rb,0.75177e-01_rb, &
        & 0.93190e-01_rb,0.11140e+00_rb,0.12912e+00_rb,0.14697e+00_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.84208e-01_rb,0.76821e-01_rb,0.71401e-01_rb,0.74638e-01_rb,0.90938e-01_rb, &
        & 0.11330e+00_rb,0.13555e+00_rb,0.15732e+00_rb,0.17919e+00_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.84082e-01_rb,0.77722e-01_rb,0.74753e-01_rb,0.85505e-01_rb,0.11037e+00_rb, &
        & 0.13754e+00_rb,0.16460e+00_rb,0.19114e+00_rb,0.21779e+00_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.83810e-01_rb,0.78431e-01_rb,0.79767e-01_rb,0.99722e-01_rb,0.13195e+00_rb, &
        & 0.16448e+00_rb,0.19691e+00_rb,0.22867e+00_rb,0.26057e+00_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.98728e-01_rb,0.87584e-01_rb,0.78635e-01_rb,0.69559e-01_rb,0.65625e-01_rb, &
        & 0.74022e-01_rb,0.88098e-01_rb,0.10177e+00_rb,0.11620e+00_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.98895e-01_rb,0.88195e-01_rb,0.80583e-01_rb,0.73010e-01_rb,0.73981e-01_rb, &
        & 0.89007e-01_rb,0.10638e+00_rb,0.12319e+00_rb,0.14079e+00_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.98911e-01_rb,0.89265e-01_rb,0.82581e-01_rb,0.78421e-01_rb,0.87111e-01_rb, &
        & 0.10783e+00_rb,0.12893e+00_rb,0.14951e+00_rb,0.17084e+00_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.98795e-01_rb,0.90737e-01_rb,0.84451e-01_rb,0.86623e-01_rb,0.10466e+00_rb, &
        & 0.13034e+00_rb,0.15592e+00_rb,0.18094e+00_rb,0.20669e+00_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.98517e-01_rb,0.92027e-01_rb,0.87783e-01_rb,0.98306e-01_rb,0.12592e+00_rb, &
        & 0.15706e+00_rb,0.18794e+00_rb,0.21816e+00_rb,0.24934e+00_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.11536e+00_rb,0.10204e+00_rb,0.91076e-01_rb,0.79879e-01_rb,0.70411e-01_rb, &
        & 0.71590e-01_rb,0.83610e-01_rb,0.96241e-01_rb,0.11049e+00_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.11565e+00_rb,0.10277e+00_rb,0.93049e-01_rb,0.82303e-01_rb,0.75267e-01_rb, &
        & 0.83499e-01_rb,0.99323e-01_rb,0.11472e+00_rb,0.13181e+00_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.11566e+00_rb,0.10353e+00_rb,0.95697e-01_rb,0.85227e-01_rb,0.84157e-01_rb, &
        & 0.99917e-01_rb,0.11933e+00_rb,0.13815e+00_rb,0.15865e+00_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.11554e+00_rb,0.10447e+00_rb,0.97554e-01_rb,0.90504e-01_rb,0.97960e-01_rb, &
        & 0.12029e+00_rb,0.14386e+00_rb,0.16671e+00_rb,0.19135e+00_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.11515e+00_rb,0.10586e+00_rb,0.99141e-01_rb,0.99110e-01_rb,0.11626e+00_rb, &
        & 0.14432e+00_rb,0.17265e+00_rb,0.20040e+00_rb,0.22976e+00_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.13374e+00_rb,0.11777e+00_rb,0.10356e+00_rb,0.90948e-01_rb,0.79121e-01_rb, &
        & 0.73845e-01_rb,0.80790e-01_rb,0.92777e-01_rb,0.10713e+00_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.13413e+00_rb,0.11892e+00_rb,0.10607e+00_rb,0.94460e-01_rb,0.81628e-01_rb, &
        & 0.80960e-01_rb,0.93972e-01_rb,0.10817e+00_rb,0.12489e+00_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.13429e+00_rb,0.11957e+00_rb,0.10936e+00_rb,0.96737e-01_rb,0.86452e-01_rb, &
        & 0.93977e-01_rb,0.11135e+00_rb,0.12854e+00_rb,0.14815e+00_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.13415e+00_rb,0.12054e+00_rb,0.11264e+00_rb,0.99496e-01_rb,0.96100e-01_rb, &
        & 0.11145e+00_rb,0.13299e+00_rb,0.15385e+00_rb,0.17718e+00_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.13374e+00_rb,0.12142e+00_rb,0.11440e+00_rb,0.10500e+00_rb,0.11046e+00_rb, &
        & 0.13295e+00_rb,0.15886e+00_rb,0.18424e+00_rb,0.21188e+00_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.15386e+00_rb,0.13491e+00_rb,0.11798e+00_rb,0.10225e+00_rb,0.89703e-01_rb, &
        & 0.79220e-01_rb,0.79340e-01_rb,0.90595e-01_rb,0.10482e+00_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.15427e+00_rb,0.13635e+00_rb,0.12030e+00_rb,0.10668e+00_rb,0.92323e-01_rb, &
        & 0.84157e-01_rb,0.91777e-01_rb,0.10537e+00_rb,0.12216e+00_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.15452e+00_rb,0.13729e+00_rb,0.12299e+00_rb,0.11079e+00_rb,0.94730e-01_rb, &
        & 0.92840e-01_rb,0.10677e+00_rb,0.12287e+00_rb,0.14246e+00_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.15437e+00_rb,0.13801e+00_rb,0.12683e+00_rb,0.11313e+00_rb,0.10059e+00_rb, &
        & 0.10648e+00_rb,0.12562e+00_rb,0.14488e+00_rb,0.16770e+00_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.15403e+00_rb,0.13896e+00_rb,0.13053e+00_rb,0.11647e+00_rb,0.11129e+00_rb, &
        & 0.12495e+00_rb,0.14877e+00_rb,0.17193e+00_rb,0.19866e+00_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.17516e+00_rb,0.15332e+00_rb,0.13386e+00_rb,0.11461e+00_rb,0.98998e-01_rb, &
        & 0.86199e-01_rb,0.78839e-01_rb,0.88239e-01_rb,0.10212e+00_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.17578e+00_rb,0.15465e+00_rb,0.13554e+00_rb,0.11810e+00_rb,0.10372e+00_rb, &
        & 0.90966e-01_rb,0.90397e-01_rb,0.10318e+00_rb,0.11979e+00_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.17603e+00_rb,0.15606e+00_rb,0.13830e+00_rb,0.12297e+00_rb,0.10694e+00_rb, &
        & 0.96886e-01_rb,0.10455e+00_rb,0.12004e+00_rb,0.13936e+00_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.17594e+00_rb,0.15671e+00_rb,0.14175e+00_rb,0.12765e+00_rb,0.11049e+00_rb, &
        & 0.10701e+00_rb,0.12142e+00_rb,0.13971e+00_rb,0.16228e+00_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.17559e+00_rb,0.15762e+00_rb,0.14567e+00_rb,0.13124e+00_rb,0.11784e+00_rb, &
        & 0.12121e+00_rb,0.14182e+00_rb,0.16360e+00_rb,0.18966e+00_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.19714e+00_rb,0.17254e+00_rb,0.15016e+00_rb,0.12718e+00_rb,0.10949e+00_rb, &
        & 0.93814e-01_rb,0.79465e-01_rb,0.85103e-01_rb,0.98359e-01_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.19796e+00_rb,0.17362e+00_rb,0.15183e+00_rb,0.13102e+00_rb,0.11370e+00_rb, &
        & 0.99479e-01_rb,0.90483e-01_rb,0.10125e+00_rb,0.11726e+00_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.19829e+00_rb,0.17522e+00_rb,0.15424e+00_rb,0.13545e+00_rb,0.11863e+00_rb, &
        & 0.10457e+00_rb,0.10363e+00_rb,0.11810e+00_rb,0.13724e+00_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.19826e+00_rb,0.17625e+00_rb,0.15748e+00_rb,0.14049e+00_rb,0.12307e+00_rb, &
        & 0.11254e+00_rb,0.11942e+00_rb,0.13698e+00_rb,0.15920e+00_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.19780e+00_rb,0.17685e+00_rb,0.16171e+00_rb,0.14510e+00_rb,0.12933e+00_rb, &
        & 0.12368e+00_rb,0.13790e+00_rb,0.15875e+00_rb,0.18443e+00_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.21950e+00_rb,0.19210e+00_rb,0.16641e+00_rb,0.14074e+00_rb,0.12030e+00_rb, &
        & 0.10245e+00_rb,0.82539e-01_rb,0.82181e-01_rb,0.95126e-01_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.22029e+00_rb,0.19287e+00_rb,0.16845e+00_rb,0.14392e+00_rb,0.12476e+00_rb, &
        & 0.10750e+00_rb,0.92052e-01_rb,0.97622e-01_rb,0.11306e+00_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.22070e+00_rb,0.19423e+00_rb,0.17032e+00_rb,0.14847e+00_rb,0.12971e+00_rb, &
        & 0.11297e+00_rb,0.10375e+00_rb,0.11522e+00_rb,0.13338e+00_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.22058e+00_rb,0.19569e+00_rb,0.17355e+00_rb,0.15354e+00_rb,0.13473e+00_rb, &
        & 0.12044e+00_rb,0.11859e+00_rb,0.13469e+00_rb,0.15609e+00_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.22021e+00_rb,0.19640e+00_rb,0.17762e+00_rb,0.15857e+00_rb,0.14102e+00_rb, &
        & 0.13035e+00_rb,0.13566e+00_rb,0.15573e+00_rb,0.18054e+00_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.24109e+00_rb,0.21099e+00_rb,0.18179e+00_rb,0.15454e+00_rb,0.13112e+00_rb, &
        & 0.11092e+00_rb,0.89296e-01_rb,0.80506e-01_rb,0.93157e-01_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.24188e+00_rb,0.21170e+00_rb,0.18440e+00_rb,0.15693e+00_rb,0.13589e+00_rb, &
        & 0.11596e+00_rb,0.96808e-01_rb,0.94330e-01_rb,0.10941e+00_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.24228e+00_rb,0.21270e+00_rb,0.18618e+00_rb,0.16137e+00_rb,0.14099e+00_rb, &
        & 0.12182e+00_rb,0.10601e+00_rb,0.11139e+00_rb,0.12901e+00_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.24222e+00_rb,0.21416e+00_rb,0.18881e+00_rb,0.16653e+00_rb,0.14643e+00_rb, &
        & 0.12886e+00_rb,0.11898e+00_rb,0.13037e+00_rb,0.15086e+00_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.24175e+00_rb,0.21527e+00_rb,0.19303e+00_rb,0.17188e+00_rb,0.15251e+00_rb, &
        & 0.13886e+00_rb,0.13479e+00_rb,0.15182e+00_rb,0.17551e+00_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.26096e+00_rb,0.22837e+00_rb,0.19625e+00_rb,0.16743e+00_rb,0.14218e+00_rb, &
        & 0.11905e+00_rb,0.97365e-01_rb,0.81466e-01_rb,0.86743e-01_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.26197e+00_rb,0.22928e+00_rb,0.19902e+00_rb,0.16970e+00_rb,0.14641e+00_rb, &
        & 0.12483e+00_rb,0.10472e+00_rb,0.93843e-01_rb,0.10806e+00_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.26221e+00_rb,0.22989e+00_rb,0.20102e+00_rb,0.17345e+00_rb,0.15221e+00_rb, &
        & 0.13116e+00_rb,0.11273e+00_rb,0.10943e+00_rb,0.12658e+00_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.26216e+00_rb,0.23120e+00_rb,0.20343e+00_rb,0.17903e+00_rb,0.15795e+00_rb, &
        & 0.13876e+00_rb,0.12342e+00_rb,0.12756e+00_rb,0.14743e+00_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.26155e+00_rb,0.23254e+00_rb,0.20744e+00_rb,0.18479e+00_rb,0.16453e+00_rb, &
        & 0.14894e+00_rb,0.13716e+00_rb,0.14827e+00_rb,0.17117e+00_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.28000e+00_rb,0.24504e+00_rb,0.21069e+00_rb,0.18024e+00_rb,0.15374e+00_rb, &
        & 0.12937e+00_rb,0.10819e+00_rb,0.88668e-01_rb,0.55926e-01_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.28047e+00_rb,0.24546e+00_rb,0.21318e+00_rb,0.18272e+00_rb,0.15865e+00_rb, &
        & 0.13663e+00_rb,0.11594e+00_rb,0.10024e+00_rb,0.74129e-01_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.28078e+00_rb,0.24627e+00_rb,0.21537e+00_rb,0.18699e+00_rb,0.16514e+00_rb, &
        & 0.14391e+00_rb,0.12463e+00_rb,0.11566e+00_rb,0.10008e+00_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.28041e+00_rb,0.24739e+00_rb,0.21799e+00_rb,0.19316e+00_rb,0.17153e+00_rb, &
        & 0.15296e+00_rb,0.13573e+00_rb,0.13384e+00_rb,0.12903e+00_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.27974e+00_rb,0.24878e+00_rb,0.22235e+00_rb,0.19924e+00_rb,0.17976e+00_rb, &
        & 0.16447e+00_rb,0.14857e+00_rb,0.15437e+00_rb,0.16255e+00_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.29848e+00_rb,0.26121e+00_rb,0.22470e+00_rb,0.19300e+00_rb,0.16476e+00_rb, &
        & 0.13979e+00_rb,0.11858e+00_rb,0.95361e-01_rb,0.54706e-01_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.29901e+00_rb,0.26167e+00_rb,0.22733e+00_rb,0.19555e+00_rb,0.17000e+00_rb, &
        & 0.14834e+00_rb,0.12686e+00_rb,0.10640e+00_rb,0.56924e-01_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.29886e+00_rb,0.26222e+00_rb,0.22932e+00_rb,0.20008e+00_rb,0.17736e+00_rb, &
        & 0.15653e+00_rb,0.13669e+00_rb,0.12173e+00_rb,0.76097e-01_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.29840e+00_rb,0.26333e+00_rb,0.23210e+00_rb,0.20641e+00_rb,0.18472e+00_rb, &
        & 0.16716e+00_rb,0.14883e+00_rb,0.13980e+00_rb,0.10268e+00_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.29738e+00_rb,0.26451e+00_rb,0.23656e+00_rb,0.21275e+00_rb,0.19460e+00_rb, &
        & 0.18005e+00_rb,0.16102e+00_rb,0.16046e+00_rb,0.13219e+00_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.31629e+00_rb,0.27681e+00_rb,0.23838e+00_rb,0.20550e+00_rb,0.17561e+00_rb, &
        & 0.15005e+00_rb,0.12855e+00_rb,0.10141e+00_rb,0.58394e-01_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.31664e+00_rb,0.27713e+00_rb,0.24100e+00_rb,0.20827e+00_rb,0.18155e+00_rb, &
        & 0.15952e+00_rb,0.13765e+00_rb,0.11247e+00_rb,0.60995e-01_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.31646e+00_rb,0.27774e+00_rb,0.24298e+00_rb,0.21292e+00_rb,0.18963e+00_rb, &
        & 0.16861e+00_rb,0.14853e+00_rb,0.12785e+00_rb,0.75808e-01_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.31577e+00_rb,0.27872e+00_rb,0.24590e+00_rb,0.21953e+00_rb,0.19753e+00_rb, &
        & 0.18064e+00_rb,0.16179e+00_rb,0.14607e+00_rb,0.10217e+00_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.31458e+00_rb,0.27978e+00_rb,0.25030e+00_rb,0.22598e+00_rb,0.20858e+00_rb, &
        & 0.19488e+00_rb,0.17415e+00_rb,0.16696e+00_rb,0.13138e+00_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.94898e-01_rb,0.85055e-01_rb,0.10490e+00_rb,0.15201e+00_rb,0.20225e+00_rb, &
        & 0.25227e+00_rb,0.30184e+00_rb,0.34935e+00_rb,0.39726e+00_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.95143e-01_rb,0.87211e-01_rb,0.12685e+00_rb,0.18956e+00_rb,0.25226e+00_rb, &
        & 0.31465e+00_rb,0.37635e+00_rb,0.43547e+00_rb,0.49508e+00_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.95189e-01_rb,0.92182e-01_rb,0.15432e+00_rb,0.23093e+00_rb,0.30732e+00_rb, &
        & 0.38333e+00_rb,0.45850e+00_rb,0.53039e+00_rb,0.60271e+00_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.95231e-01_rb,0.10069e+00_rb,0.18420e+00_rb,0.27567e+00_rb,0.36689e+00_rb, &
        & 0.45778e+00_rb,0.54750e+00_rb,0.63309e+00_rb,0.71941e+00_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.95218e-01_rb,0.11224e+00_rb,0.21607e+00_rb,0.32340e+00_rb,0.43052e+00_rb, &
        & 0.53710e+00_rb,0.64228e+00_rb,0.74267e+00_rb,0.84389e+00_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.11396e+00_rb,0.10141e+00_rb,0.10674e+00_rb,0.14459e+00_rb,0.19233e+00_rb, &
        & 0.23999e+00_rb,0.28714e+00_rb,0.33260e+00_rb,0.38043e+00_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.11432e+00_rb,0.10261e+00_rb,0.12520e+00_rb,0.18276e+00_rb,0.24324e+00_rb, &
        & 0.30342e+00_rb,0.36297e+00_rb,0.42016e+00_rb,0.48074e+00_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.11451e+00_rb,0.10490e+00_rb,0.15054e+00_rb,0.22514e+00_rb,0.29969e+00_rb, &
        & 0.37388e+00_rb,0.44723e+00_rb,0.51774e+00_rb,0.59231e+00_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.11456e+00_rb,0.10971e+00_rb,0.18129e+00_rb,0.27136e+00_rb,0.36114e+00_rb, &
        & 0.45069e+00_rb,0.53922e+00_rb,0.62384e+00_rb,0.71399e+00_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.11453e+00_rb,0.11800e+00_rb,0.21439e+00_rb,0.32093e+00_rb,0.42721e+00_rb, &
        & 0.53302e+00_rb,0.63780e+00_rb,0.73784e+00_rb,0.84437e+00_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.13655e+00_rb,0.12054e+00_rb,0.11124e+00_rb,0.13153e+00_rb,0.17207e+00_rb, &
        & 0.21470e+00_rb,0.25693e+00_rb,0.29779e+00_rb,0.34231e+00_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.13707e+00_rb,0.12216e+00_rb,0.12413e+00_rb,0.16670e+00_rb,0.22186e+00_rb, &
        & 0.27683e+00_rb,0.33137e+00_rb,0.38384e+00_rb,0.44145e+00_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.13747e+00_rb,0.12319e+00_rb,0.14360e+00_rb,0.20888e+00_rb,0.27797e+00_rb, &
        & 0.34687e+00_rb,0.41517e+00_rb,0.48101e+00_rb,0.55322e+00_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.13768e+00_rb,0.12546e+00_rb,0.17113e+00_rb,0.25534e+00_rb,0.33989e+00_rb, &
        & 0.42417e+00_rb,0.50744e+00_rb,0.58796e+00_rb,0.67649e+00_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.13772e+00_rb,0.12983e+00_rb,0.20407e+00_rb,0.30559e+00_rb,0.40688e+00_rb, &
        & 0.50779e+00_rb,0.60773e+00_rb,0.70383e+00_rb,0.80998e+00_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.16353e+00_rb,0.14321e+00_rb,0.12667e+00_rb,0.12562e+00_rb,0.14945e+00_rb, &
        & 0.18589e+00_rb,0.22262e+00_rb,0.25813e+00_rb,0.29728e+00_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.16418e+00_rb,0.14525e+00_rb,0.13251e+00_rb,0.15131e+00_rb,0.19654e+00_rb, &
        & 0.24525e+00_rb,0.29354e+00_rb,0.34060e+00_rb,0.39212e+00_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.16461e+00_rb,0.14670e+00_rb,0.14414e+00_rb,0.18897e+00_rb,0.25116e+00_rb, &
        & 0.31348e+00_rb,0.37525e+00_rb,0.43507e+00_rb,0.50121e+00_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.16491e+00_rb,0.14777e+00_rb,0.16361e+00_rb,0.23441e+00_rb,0.31211e+00_rb, &
        & 0.38952e+00_rb,0.46648e+00_rb,0.54097e+00_rb,0.62321e+00_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.16511e+00_rb,0.15008e+00_rb,0.19240e+00_rb,0.28461e+00_rb,0.37902e+00_rb, &
        & 0.47310e+00_rb,0.56644e+00_rb,0.65651e+00_rb,0.75672e+00_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.19509e+00_rb,0.17082e+00_rb,0.14980e+00_rb,0.13204e+00_rb,0.13395e+00_rb, &
        & 0.16269e+00_rb,0.19465e+00_rb,0.22562e+00_rb,0.26041e+00_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.19619e+00_rb,0.17214e+00_rb,0.15205e+00_rb,0.14710e+00_rb,0.17263e+00_rb, &
        & 0.21401e+00_rb,0.25633e+00_rb,0.29729e+00_rb,0.34247e+00_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.19681e+00_rb,0.17444e+00_rb,0.15815e+00_rb,0.17421e+00_rb,0.22342e+00_rb, &
        & 0.27889e+00_rb,0.33405e+00_rb,0.38778e+00_rb,0.44627e+00_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.19721e+00_rb,0.17581e+00_rb,0.16979e+00_rb,0.21414e+00_rb,0.28278e+00_rb, &
        & 0.35302e+00_rb,0.42274e+00_rb,0.49080e+00_rb,0.56501e+00_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.19732e+00_rb,0.17699e+00_rb,0.18956e+00_rb,0.26168e+00_rb,0.34842e+00_rb, &
        & 0.43503e+00_rb,0.52113e+00_rb,0.60494e+00_rb,0.69634e+00_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.23167e+00_rb,0.20281e+00_rb,0.17680e+00_rb,0.14997e+00_rb,0.13324e+00_rb, &
        & 0.14585e+00_rb,0.17450e+00_rb,0.20194e+00_rb,0.23383e+00_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.23317e+00_rb,0.20416e+00_rb,0.17934e+00_rb,0.15819e+00_rb,0.15707e+00_rb, &
        & 0.18679e+00_rb,0.22357e+00_rb,0.25924e+00_rb,0.29921e+00_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.23428e+00_rb,0.20631e+00_rb,0.18228e+00_rb,0.17432e+00_rb,0.19753e+00_rb, &
        & 0.24343e+00_rb,0.29156e+00_rb,0.33836e+00_rb,0.38981e+00_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.23494e+00_rb,0.20857e+00_rb,0.18851e+00_rb,0.20211e+00_rb,0.25104e+00_rb, &
        & 0.31278e+00_rb,0.37475e+00_rb,0.43549e+00_rb,0.50069e+00_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.23514e+00_rb,0.20988e+00_rb,0.20063e+00_rb,0.24116e+00_rb,0.31413e+00_rb, &
        & 0.39227e+00_rb,0.47006e+00_rb,0.54611e+00_rb,0.62816e+00_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.27443e+00_rb,0.24019e+00_rb,0.20758e+00_rb,0.17610e+00_rb,0.14566e+00_rb, &
        & 0.14000e+00_rb,0.16506e+00_rb,0.19019e+00_rb,0.22128e+00_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.27620e+00_rb,0.24179e+00_rb,0.21121e+00_rb,0.17965e+00_rb,0.15984e+00_rb, &
        & 0.16864e+00_rb,0.20104e+00_rb,0.23213e+00_rb,0.26949e+00_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.27747e+00_rb,0.24292e+00_rb,0.21364e+00_rb,0.18907e+00_rb,0.18625e+00_rb, &
        & 0.21432e+00_rb,0.25652e+00_rb,0.29760e+00_rb,0.34338e+00_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.27840e+00_rb,0.24591e+00_rb,0.21750e+00_rb,0.20704e+00_rb,0.22722e+00_rb, &
        & 0.27512e+00_rb,0.32963e+00_rb,0.38304e+00_rb,0.44058e+00_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.27872e+00_rb,0.24790e+00_rb,0.22377e+00_rb,0.23568e+00_rb,0.28140e+00_rb, &
        & 0.34919e+00_rb,0.41848e+00_rb,0.48672e+00_rb,0.55917e+00_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.32341e+00_rb,0.28305e+00_rb,0.24270e+00_rb,0.20626e+00_rb,0.16778e+00_rb, &
        & 0.14644e+00_rb,0.16206e+00_rb,0.18656e+00_rb,0.21702e+00_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.32568e+00_rb,0.28506e+00_rb,0.24756e+00_rb,0.20963e+00_rb,0.17522e+00_rb, &
        & 0.16516e+00_rb,0.19045e+00_rb,0.21956e+00_rb,0.25530e+00_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.32734e+00_rb,0.28654e+00_rb,0.25088e+00_rb,0.21479e+00_rb,0.19195e+00_rb, &
        & 0.19819e+00_rb,0.23472e+00_rb,0.27107e+00_rb,0.31461e+00_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.32837e+00_rb,0.28793e+00_rb,0.25332e+00_rb,0.22544e+00_rb,0.22100e+00_rb, &
        & 0.24722e+00_rb,0.29529e+00_rb,0.34210e+00_rb,0.39526e+00_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.32863e+00_rb,0.29096e+00_rb,0.25794e+00_rb,0.24543e+00_rb,0.26342e+00_rb, &
        & 0.31218e+00_rb,0.37382e+00_rb,0.43401e+00_rb,0.50027e+00_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.37878e+00_rb,0.33149e+00_rb,0.28420e+00_rb,0.23973e+00_rb,0.19541e+00_rb, &
        & 0.16103e+00_rb,0.16029e+00_rb,0.18365e+00_rb,0.21308e+00_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.38160e+00_rb,0.33397e+00_rb,0.28731e+00_rb,0.24432e+00_rb,0.19953e+00_rb, &
        & 0.17386e+00_rb,0.18539e+00_rb,0.21378e+00_rb,0.24833e+00_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.38370e+00_rb,0.33584e+00_rb,0.29267e+00_rb,0.24777e+00_rb,0.20961e+00_rb, &
        & 0.19622e+00_rb,0.22137e+00_rb,0.25542e+00_rb,0.29676e+00_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.38483e+00_rb,0.33686e+00_rb,0.29593e+00_rb,0.25505e+00_rb,0.22908e+00_rb, &
        & 0.23318e+00_rb,0.27287e+00_rb,0.31549e+00_rb,0.36559e+00_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.38520e+00_rb,0.33870e+00_rb,0.29863e+00_rb,0.26759e+00_rb,0.26094e+00_rb, &
        & 0.28563e+00_rb,0.34036e+00_rb,0.39396e+00_rb,0.45596e+00_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.44139e+00_rb,0.38627e+00_rb,0.33114e+00_rb,0.27661e+00_rb,0.22671e+00_rb, &
        & 0.18314e+00_rb,0.16439e+00_rb,0.18365e+00_rb,0.21302e+00_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.44442e+00_rb,0.38894e+00_rb,0.33346e+00_rb,0.28317e+00_rb,0.23077e+00_rb, &
        & 0.19173e+00_rb,0.18540e+00_rb,0.21159e+00_rb,0.24601e+00_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.44688e+00_rb,0.39111e+00_rb,0.33884e+00_rb,0.28735e+00_rb,0.23759e+00_rb, &
        & 0.20864e+00_rb,0.21693e+00_rb,0.24969e+00_rb,0.29058e+00_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.44794e+00_rb,0.39207e+00_rb,0.34326e+00_rb,0.29193e+00_rb,0.25105e+00_rb, &
        & 0.23562e+00_rb,0.26192e+00_rb,0.30247e+00_rb,0.35138e+00_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.44820e+00_rb,0.39253e+00_rb,0.34576e+00_rb,0.30181e+00_rb,0.27401e+00_rb, &
        & 0.27758e+00_rb,0.32230e+00_rb,0.37259e+00_rb,0.43225e+00_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.51225e+00_rb,0.44827e+00_rb,0.38430e+00_rb,0.32096e+00_rb,0.26380e+00_rb, &
        & 0.21204e+00_rb,0.18122e+00_rb,0.19575e+00_rb,0.22762e+00_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.51551e+00_rb,0.45114e+00_rb,0.38677e+00_rb,0.32842e+00_rb,0.26803e+00_rb, &
        & 0.22031e+00_rb,0.20295e+00_rb,0.22783e+00_rb,0.26563e+00_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.51720e+00_rb,0.45265e+00_rb,0.39154e+00_rb,0.33278e+00_rb,0.27451e+00_rb, &
        & 0.23732e+00_rb,0.23493e+00_rb,0.26904e+00_rb,0.31377e+00_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.51766e+00_rb,0.45307e+00_rb,0.39664e+00_rb,0.33683e+00_rb,0.28805e+00_rb, &
        & 0.26198e+00_rb,0.27963e+00_rb,0.32252e+00_rb,0.37505e+00_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.51687e+00_rb,0.45253e+00_rb,0.39906e+00_rb,0.34713e+00_rb,0.30874e+00_rb, &
        & 0.30122e+00_rb,0.34231e+00_rb,0.39543e+00_rb,0.45934e+00_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.58922e+00_rb,0.51562e+00_rb,0.44203e+00_rb,0.36905e+00_rb,0.30406e+00_rb, &
        & 0.24465e+00_rb,0.20183e+00_rb,0.21102e+00_rb,0.20343e+00_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.59167e+00_rb,0.51780e+00_rb,0.44391e+00_rb,0.37709e+00_rb,0.30888e+00_rb, &
        & 0.25236e+00_rb,0.22396e+00_rb,0.24700e+00_rb,0.27515e+00_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.59289e+00_rb,0.51887e+00_rb,0.44833e+00_rb,0.38190e+00_rb,0.31478e+00_rb, &
        & 0.26966e+00_rb,0.25708e+00_rb,0.29183e+00_rb,0.34036e+00_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.59238e+00_rb,0.51846e+00_rb,0.45409e+00_rb,0.38554e+00_rb,0.32896e+00_rb, &
        & 0.29236e+00_rb,0.30023e+00_rb,0.34558e+00_rb,0.40217e+00_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.59040e+00_rb,0.51681e+00_rb,0.45607e+00_rb,0.39582e+00_rb,0.34740e+00_rb, &
        & 0.32863e+00_rb,0.36471e+00_rb,0.42134e+00_rb,0.48947e+00_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.67101e+00_rb,0.58718e+00_rb,0.50336e+00_rb,0.42031e+00_rb,0.34716e+00_rb, &
        & 0.27980e+00_rb,0.22553e+00_rb,0.22887e+00_rb,0.21243e+00_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.67268e+00_rb,0.58864e+00_rb,0.50464e+00_rb,0.42832e+00_rb,0.35207e+00_rb, &
        & 0.28740e+00_rb,0.24801e+00_rb,0.26765e+00_rb,0.28879e+00_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.67256e+00_rb,0.58859e+00_rb,0.50813e+00_rb,0.43372e+00_rb,0.35764e+00_rb, &
        & 0.30459e+00_rb,0.28195e+00_rb,0.31558e+00_rb,0.36763e+00_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.67050e+00_rb,0.58682e+00_rb,0.51379e+00_rb,0.43676e+00_rb,0.37243e+00_rb, &
        & 0.32590e+00_rb,0.32370e+00_rb,0.37117e+00_rb,0.43203e+00_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.66719e+00_rb,0.58400e+00_rb,0.51581e+00_rb,0.44632e+00_rb,0.38956e+00_rb, &
        & 0.35928e+00_rb,0.38845e+00_rb,0.44876e+00_rb,0.52114e+00_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.10138e+00_rb,0.11271e+00_rb,0.22512e+00_rb,0.33705e+00_rb,0.44854e+00_rb, &
        & 0.55854e+00_rb,0.66519e+00_rb,0.75942e+00_rb,0.85612e+00_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.10145e+00_rb,0.13913e+00_rb,0.27783e+00_rb,0.41606e+00_rb,0.55341e+00_rb, &
        & 0.68935e+00_rb,0.82123e+00_rb,0.93793e+00_rb,0.10573e+01_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.10144e+00_rb,0.16758e+00_rb,0.33464e+00_rb,0.50118e+00_rb,0.66683e+00_rb, &
        & 0.83041e+00_rb,0.98914e+00_rb,0.11301e+01_rb,0.12744e+01_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.10142e+00_rb,0.19759e+00_rb,0.39475e+00_rb,0.59125e+00_rb,0.78644e+00_rb, &
        & 0.97913e+00_rb,0.11666e+01_rb,0.13327e+01_rb,0.15036e+01_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.10117e+00_rb,0.22893e+00_rb,0.45719e+00_rb,0.68491e+00_rb,0.91083e+00_rb, &
        & 0.11348e+01_rb,0.13522e+01_rb,0.15448e+01_rb,0.17423e+01_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.12283e+00_rb,0.11939e+00_rb,0.23586e+00_rb,0.35344e+00_rb,0.46995e+00_rb, &
        & 0.58562e+00_rb,0.69771e+00_rb,0.79719e+00_rb,0.91456e+00_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.12300e+00_rb,0.14726e+00_rb,0.29424e+00_rb,0.44061e+00_rb,0.58626e+00_rb, &
        & 0.73031e+00_rb,0.87019e+00_rb,0.99468e+00_rb,0.11413e+01_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.12308e+00_rb,0.17880e+00_rb,0.35729e+00_rb,0.53471e+00_rb,0.71190e+00_rb, &
        & 0.88678e+00_rb,0.10560e+01_rb,0.12083e+01_rb,0.13858e+01_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.12303e+00_rb,0.21231e+00_rb,0.42408e+00_rb,0.63552e+00_rb,0.84482e+00_rb, &
        & 0.10530e+01_rb,0.12550e+01_rb,0.14355e+01_rb,0.16461e+01_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.12277e+00_rb,0.24738e+00_rb,0.49418e+00_rb,0.74021e+00_rb,0.98480e+00_rb, &
        & 0.12263e+01_rb,0.14622e+01_rb,0.16712e+01_rb,0.19192e+01_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.14866e+00_rb,0.13122e+00_rb,0.23281e+00_rb,0.34894e+00_rb,0.46454e+00_rb, &
        & 0.57840e+00_rb,0.68947e+00_rb,0.78868e+00_rb,0.91936e+00_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.14909e+00_rb,0.15020e+00_rb,0.29535e+00_rb,0.44246e+00_rb,0.58879e+00_rb, &
        & 0.73298e+00_rb,0.87446e+00_rb,0.10004e+01_rb,0.11652e+01_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.14932e+00_rb,0.18193e+00_rb,0.36339e+00_rb,0.54464e+00_rb,0.72472e+00_rb, &
        & 0.90302e+00_rb,0.10769e+01_rb,0.12326e+01_rb,0.14351e+01_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.14936e+00_rb,0.21850e+00_rb,0.43638e+00_rb,0.65369e+00_rb,0.87034e+00_rb, &
        & 0.10842e+01_rb,0.12925e+01_rb,0.14807e+01_rb,0.17238e+01_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.14913e+00_rb,0.25689e+00_rb,0.51319e+00_rb,0.76884e+00_rb,0.10226e+01_rb, &
        & 0.12750e+01_rb,0.15205e+01_rb,0.17418e+01_rb,0.20260e+01_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.17957e+00_rb,0.15775e+00_rb,0.22276e+00_rb,0.33390e+00_rb,0.44423e+00_rb, &
        & 0.55367e+00_rb,0.66019e+00_rb,0.75611e+00_rb,0.88680e+00_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.18040e+00_rb,0.16246e+00_rb,0.28812e+00_rb,0.43146e+00_rb,0.57438e+00_rb, &
        & 0.71573e+00_rb,0.85374e+00_rb,0.97839e+00_rb,0.11468e+01_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.18097e+00_rb,0.18562e+00_rb,0.36044e+00_rb,0.53994e+00_rb,0.71844e+00_rb, &
        & 0.89550e+00_rb,0.10683e+01_rb,0.12247e+01_rb,0.14347e+01_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.18121e+00_rb,0.21975e+00_rb,0.43861e+00_rb,0.65714e+00_rb,0.87443e+00_rb, &
        & 0.10900e+01_rb,0.13008e+01_rb,0.14906e+01_rb,0.17458e+01_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.18106e+00_rb,0.26092e+00_rb,0.52133e+00_rb,0.78101e+00_rb,0.10395e+01_rb, &
        & 0.12958e+01_rb,0.15460e+01_rb,0.17735e+01_rb,0.20757e+01_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.21697e+00_rb,0.19036e+00_rb,0.21215e+00_rb,0.31312e+00_rb,0.41669e+00_rb, &
        & 0.51939e+00_rb,0.61996e+00_rb,0.71133e+00_rb,0.83486e+00_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.21794e+00_rb,0.19137e+00_rb,0.27611e+00_rb,0.41368e+00_rb,0.55071e+00_rb, &
        & 0.68646e+00_rb,0.81902e+00_rb,0.94006e+00_rb,0.11028e+01_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.21886e+00_rb,0.20114e+00_rb,0.35159e+00_rb,0.52668e+00_rb,0.70100e+00_rb, &
        & 0.87403e+00_rb,0.10428e+01_rb,0.11973e+01_rb,0.14036e+01_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.21947e+00_rb,0.22776e+00_rb,0.43385e+00_rb,0.64994e+00_rb,0.86545e+00_rb, &
        & 0.10790e+01_rb,0.12882e+01_rb,0.14795e+01_rb,0.17328e+01_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.21950e+00_rb,0.26410e+00_rb,0.52266e+00_rb,0.78311e+00_rb,0.10422e+01_rb, &
        & 0.12993e+01_rb,0.15508e+01_rb,0.17813e+01_rb,0.20867e+01_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.26192e+00_rb,0.22964e+00_rb,0.21396e+00_rb,0.28442e+00_rb,0.37859e+00_rb, &
        & 0.47209e+00_rb,0.56361e+00_rb,0.64772e+00_rb,0.75904e+00_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.26327e+00_rb,0.23094e+00_rb,0.26427e+00_rb,0.38535e+00_rb,0.51302e+00_rb, &
        & 0.63961e+00_rb,0.76380e+00_rb,0.87816e+00_rb,0.10282e+01_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.26410e+00_rb,0.23177e+00_rb,0.33446e+00_rb,0.50065e+00_rb,0.66671e+00_rb, &
        & 0.83097e+00_rb,0.99257e+00_rb,0.11416e+01_rb,0.13361e+01_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.26501e+00_rb,0.24670e+00_rb,0.42047e+00_rb,0.62999e+00_rb,0.83846e+00_rb, &
        & 0.10456e+01_rb,0.12484e+01_rb,0.14350e+01_rb,0.16802e+01_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.26533e+00_rb,0.27595e+00_rb,0.51477e+00_rb,0.77155e+00_rb,0.10272e+01_rb, &
        & 0.12802e+01_rb,0.15288e+01_rb,0.17573e+01_rb,0.20585e+01_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.31548e+00_rb,0.27644e+00_rb,0.23749e+00_rb,0.26350e+00_rb,0.33563e+00_rb, &
        & 0.41860e+00_rb,0.50000e+00_rb,0.57557e+00_rb,0.67307e+00_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.31747e+00_rb,0.27830e+00_rb,0.26813e+00_rb,0.35238e+00_rb,0.46780e+00_rb, &
        & 0.58337e+00_rb,0.69695e+00_rb,0.80281e+00_rb,0.93825e+00_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.31845e+00_rb,0.27929e+00_rb,0.32577e+00_rb,0.46831e+00_rb,0.62377e+00_rb, &
        & 0.77768e+00_rb,0.92868e+00_rb,0.10691e+01_rb,0.12510e+01_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.31933e+00_rb,0.28093e+00_rb,0.40219e+00_rb,0.60218e+00_rb,0.80124e+00_rb, &
        & 0.99953e+00_rb,0.11938e+01_rb,0.13738e+01_rb,0.16072e+01_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.31962e+00_rb,0.30041e+00_rb,0.50060e+00_rb,0.75020e+00_rb,0.99895e+00_rb, &
        & 0.12457e+01_rb,0.14877e+01_rb,0.17119e+01_rb,0.20030e+01_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.37865e+00_rb,0.33167e+00_rb,0.28471e+00_rb,0.26304e+00_rb,0.29883e+00_rb, &
        & 0.36478e+00_rb,0.43575e+00_rb,0.50177e+00_rb,0.58698e+00_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.38151e+00_rb,0.33434e+00_rb,0.29226e+00_rb,0.33286e+00_rb,0.42010e+00_rb, &
        & 0.52397e+00_rb,0.62612e+00_rb,0.72153e+00_rb,0.84257e+00_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.38305e+00_rb,0.33575e+00_rb,0.33335e+00_rb,0.43312e+00_rb,0.57580e+00_rb, &
        & 0.71815e+00_rb,0.85781e+00_rb,0.98826e+00_rb,0.11549e+01_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.38418e+00_rb,0.33683e+00_rb,0.39821e+00_rb,0.56836e+00_rb,0.75670e+00_rb, &
        & 0.94382e+00_rb,0.11276e+01_rb,0.12990e+01_rb,0.15179e+01_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.38435e+00_rb,0.34032e+00_rb,0.48169e+00_rb,0.72105e+00_rb,0.95979e+00_rb, &
        & 0.11976e+01_rb,0.14311e+01_rb,0.16484e+01_rb,0.19256e+01_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.45391e+00_rb,0.39745e+00_rb,0.34100e+00_rb,0.28597e+00_rb,0.28236e+00_rb, &
        & 0.32683e+00_rb,0.39003e+00_rb,0.44780e+00_rb,0.52646e+00_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.45720e+00_rb,0.40042e+00_rb,0.34369e+00_rb,0.33249e+00_rb,0.37993e+00_rb, &
        & 0.46180e+00_rb,0.55195e+00_rb,0.63669e+00_rb,0.74252e+00_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.45930e+00_rb,0.40241e+00_rb,0.35995e+00_rb,0.41477e+00_rb,0.52224e+00_rb, &
        & 0.65162e+00_rb,0.77908e+00_rb,0.89846e+00_rb,0.10481e+01_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.46056e+00_rb,0.40360e+00_rb,0.40974e+00_rb,0.52851e+00_rb,0.70308e+00_rb, &
        & 0.87703e+00_rb,0.10485e+01_rb,0.12091e+01_rb,0.14102e+01_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.46097e+00_rb,0.40408e+00_rb,0.48083e+00_rb,0.68278e+00_rb,0.90959e+00_rb, &
        & 0.11347e+01_rb,0.13562e+01_rb,0.15644e+01_rb,0.18239e+01_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.54348e+00_rb,0.47578e+00_rb,0.40808e+00_rb,0.34039e+00_rb,0.29202e+00_rb, &
        & 0.31463e+00_rb,0.37530e+00_rb,0.43003e+00_rb,0.50713e+00_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.54726e+00_rb,0.47923e+00_rb,0.41116e+00_rb,0.35738e+00_rb,0.36873e+00_rb, &
        & 0.42815e+00_rb,0.51125e+00_rb,0.58827e+00_rb,0.68902e+00_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.54904e+00_rb,0.48087e+00_rb,0.41270e+00_rb,0.42187e+00_rb,0.48592e+00_rb, &
        & 0.59834e+00_rb,0.71541e+00_rb,0.82550e+00_rb,0.96166e+00_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.55066e+00_rb,0.48237e+00_rb,0.44342e+00_rb,0.51829e+00_rb,0.65846e+00_rb, &
        & 0.82163e+00_rb,0.98259e+00_rb,0.11344e+01_rb,0.13209e+01_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.55078e+00_rb,0.48256e+00_rb,0.50243e+00_rb,0.65112e+00_rb,0.86662e+00_rb, &
        & 0.10813e+01_rb,0.12932e+01_rb,0.14933e+01_rb,0.17377e+01_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.65114e+00_rb,0.56997e+00_rb,0.48880e+00_rb,0.40765e+00_rb,0.34114e+00_rb, &
        & 0.35299e+00_rb,0.42114e+00_rb,0.48311e+00_rb,0.56879e+00_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.65409e+00_rb,0.57269e+00_rb,0.49126e+00_rb,0.41893e+00_rb,0.41551e+00_rb, &
        & 0.46863e+00_rb,0.55986e+00_rb,0.64427e+00_rb,0.75415e+00_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.65488e+00_rb,0.57345e+00_rb,0.49197e+00_rb,0.47947e+00_rb,0.53169e+00_rb, &
        & 0.64034e+00_rb,0.76581e+00_rb,0.88450e+00_rb,0.10287e+01_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.65537e+00_rb,0.57398e+00_rb,0.51344e+00_rb,0.57405e+00_rb,0.70306e+00_rb, &
        & 0.87735e+00_rb,0.10495e+01_rb,0.12127e+01_rb,0.14094e+01_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.65395e+00_rb,0.57279e+00_rb,0.56858e+00_rb,0.69997e+00_rb,0.92273e+00_rb, &
        & 0.11516e+01_rb,0.13780e+01_rb,0.15930e+01_rb,0.18499e+01_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.77619e+00_rb,0.67938e+00_rb,0.58258e+00_rb,0.48578e+00_rb,0.39872e+00_rb, &
        & 0.39471e+00_rb,0.47009e+00_rb,0.53992e+00_rb,0.63430e+00_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.77753e+00_rb,0.68052e+00_rb,0.58367e+00_rb,0.49090e+00_rb,0.46899e+00_rb, &
        & 0.51194e+00_rb,0.61162e+00_rb,0.70447e+00_rb,0.82357e+00_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.77638e+00_rb,0.67972e+00_rb,0.58318e+00_rb,0.54530e+00_rb,0.58387e+00_rb, &
        & 0.68245e+00_rb,0.81638e+00_rb,0.94341e+00_rb,0.10965e+01_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.77440e+00_rb,0.67810e+00_rb,0.59462e+00_rb,0.63695e+00_rb,0.74739e+00_rb, &
        & 0.93197e+00_rb,0.11153e+01_rb,0.12903e+01_rb,0.14970e+01_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.77021e+00_rb,0.67464e+00_rb,0.64357e+00_rb,0.75637e+00_rb,0.97798e+00_rb, &
        & 0.12207e+01_rb,0.14609e+01_rb,0.16904e+01_rb,0.19598e+01_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.91895e+00_rb,0.80429e+00_rb,0.68964e+00_rb,0.57498e+00_rb,0.46528e+00_rb, &
        & 0.44199e+00_rb,0.52028e+00_rb,0.59834e+00_rb,0.70141e+00_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.91759e+00_rb,0.80320e+00_rb,0.68881e+00_rb,0.57463e+00_rb,0.52956e+00_rb, &
        & 0.55768e+00_rb,0.66492e+00_rb,0.76644e+00_rb,0.89471e+00_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.91361e+00_rb,0.79979e+00_rb,0.68598e+00_rb,0.61940e+00_rb,0.64101e+00_rb, &
        & 0.72712e+00_rb,0.86969e+00_rb,0.10053e+01_rb,0.11679e+01_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.90733e+00_rb,0.79439e+00_rb,0.68655e+00_rb,0.70603e+00_rb,0.79560e+00_rb, &
        & 0.98356e+00_rb,0.11773e+01_rb,0.13631e+01_rb,0.15788e+01_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.89862e+00_rb,0.78697e+00_rb,0.72670e+00_rb,0.82187e+00_rb,0.10292e+01_rb, &
        & 0.12848e+01_rb,0.15381e+01_rb,0.17818e+01_rb,0.20623e+01_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:,13, 1) = (/ &
        & 0.80849e-06_rb,0.81362e-06_rb,0.81391e-06_rb,0.81832e-06_rb,0.81000e-06_rb /)
      kbo(:,14, 1) = (/ &
        & 0.62593e-06_rb,0.63090e-06_rb,0.63158e-06_rb,0.62821e-06_rb,0.62502e-06_rb /)
      kbo(:,15, 1) = (/ &
        & 0.47825e-06_rb,0.47852e-06_rb,0.47713e-06_rb,0.47688e-06_rb,0.47989e-06_rb /)
      kbo(:,16, 1) = (/ &
        & 0.36039e-06_rb,0.35905e-06_rb,0.36064e-06_rb,0.36298e-06_rb,0.36860e-06_rb /)
      kbo(:,17, 1) = (/ &
        & 0.27679e-06_rb,0.27608e-06_rb,0.27742e-06_rb,0.28112e-06_rb,0.28759e-06_rb /)
      kbo(:,18, 1) = (/ &
        & 0.21771e-06_rb,0.21826e-06_rb,0.22019e-06_rb,0.22429e-06_rb,0.22996e-06_rb /)
      kbo(:,19, 1) = (/ &
        & 0.17334e-06_rb,0.17441e-06_rb,0.17681e-06_rb,0.18055e-06_rb,0.18537e-06_rb /)
      kbo(:,20, 1) = (/ &
        & 0.13974e-06_rb,0.14076e-06_rb,0.14326e-06_rb,0.14694e-06_rb,0.15070e-06_rb /)
      kbo(:,21, 1) = (/ &
        & 0.11289e-06_rb,0.11394e-06_rb,0.11631e-06_rb,0.11963e-06_rb,0.12284e-06_rb /)
      kbo(:,22, 1) = (/ &
        & 0.90574e-07_rb,0.91914e-07_rb,0.94046e-07_rb,0.96998e-07_rb,0.99841e-07_rb /)
      kbo(:,23, 1) = (/ &
        & 0.72483e-07_rb,0.74069e-07_rb,0.76186e-07_rb,0.78482e-07_rb,0.80964e-07_rb /)
      kbo(:,24, 1) = (/ &
        & 0.57463e-07_rb,0.59130e-07_rb,0.60802e-07_rb,0.62903e-07_rb,0.65218e-07_rb /)
      kbo(:,25, 1) = (/ &
        & 0.45767e-07_rb,0.47184e-07_rb,0.48696e-07_rb,0.50494e-07_rb,0.52527e-07_rb /)
      kbo(:,26, 1) = (/ &
        & 0.36636e-07_rb,0.37794e-07_rb,0.39217e-07_rb,0.40768e-07_rb,0.42465e-07_rb /)
      kbo(:,27, 1) = (/ &
        & 0.29173e-07_rb,0.30183e-07_rb,0.31469e-07_rb,0.32834e-07_rb,0.34213e-07_rb /)
      kbo(:,28, 1) = (/ &
        & 0.23225e-07_rb,0.24137e-07_rb,0.25167e-07_rb,0.26378e-07_rb,0.27534e-07_rb /)
      kbo(:,29, 1) = (/ &
        & 0.18675e-07_rb,0.19472e-07_rb,0.20340e-07_rb,0.21340e-07_rb,0.22286e-07_rb /)
      kbo(:,30, 1) = (/ &
        & 0.15020e-07_rb,0.15690e-07_rb,0.16468e-07_rb,0.17258e-07_rb,0.18060e-07_rb /)
      kbo(:,31, 1) = (/ &
        & 0.12149e-07_rb,0.12728e-07_rb,0.13375e-07_rb,0.14035e-07_rb,0.14711e-07_rb /)
      kbo(:,32, 1) = (/ &
        & 0.98596e-08_rb,0.10344e-07_rb,0.10879e-07_rb,0.11427e-07_rb,0.12003e-07_rb /)
      kbo(:,33, 1) = (/ &
        & 0.80127e-08_rb,0.84199e-08_rb,0.88610e-08_rb,0.93188e-08_rb,0.98102e-08_rb /)
      kbo(:,34, 1) = (/ &
        & 0.65473e-08_rb,0.68850e-08_rb,0.72470e-08_rb,0.76379e-08_rb,0.80329e-08_rb /)
      kbo(:,35, 1) = (/ &
        & 0.53107e-08_rb,0.55930e-08_rb,0.58984e-08_rb,0.62161e-08_rb,0.65511e-08_rb /)
      kbo(:,36, 1) = (/ &
        & 0.42749e-08_rb,0.45072e-08_rb,0.47592e-08_rb,0.50244e-08_rb,0.53034e-08_rb /)
      kbo(:,37, 1) = (/ &
        & 0.34593e-08_rb,0.36479e-08_rb,0.38557e-08_rb,0.40714e-08_rb,0.43031e-08_rb /)
      kbo(:,38, 1) = (/ &
        & 0.27981e-08_rb,0.29525e-08_rb,0.31212e-08_rb,0.32984e-08_rb,0.34890e-08_rb /)
      kbo(:,39, 1) = (/ &
        & 0.22616e-08_rb,0.23879e-08_rb,0.25266e-08_rb,0.26716e-08_rb,0.28295e-08_rb /)
      kbo(:,40, 1) = (/ &
        & 0.18232e-08_rb,0.19245e-08_rb,0.20362e-08_rb,0.21554e-08_rb,0.22831e-08_rb /)
      kbo(:,41, 1) = (/ &
        & 0.14679e-08_rb,0.15511e-08_rb,0.16402e-08_rb,0.17376e-08_rb,0.18413e-08_rb /)
      kbo(:,42, 1) = (/ &
        & 0.11828e-08_rb,0.12488e-08_rb,0.13200e-08_rb,0.14008e-08_rb,0.14841e-08_rb /)
      kbo(:,43, 1) = (/ &
        & 0.94970e-09_rb,0.10028e-08_rb,0.10610e-08_rb,0.11254e-08_rb,0.11933e-08_rb /)
      kbo(:,44, 1) = (/ &
        & 0.76210e-09_rb,0.80401e-09_rb,0.85098e-09_rb,0.90203e-09_rb,0.95794e-09_rb /)
      kbo(:,45, 1) = (/ &
        & 0.61242e-09_rb,0.64471e-09_rb,0.68205e-09_rb,0.72336e-09_rb,0.76846e-09_rb /)
      kbo(:,46, 1) = (/ &
        & 0.49214e-09_rb,0.51642e-09_rb,0.54633e-09_rb,0.57916e-09_rb,0.61506e-09_rb /)
      kbo(:,47, 1) = (/ &
        & 0.39552e-09_rb,0.41343e-09_rb,0.43658e-09_rb,0.46266e-09_rb,0.49121e-09_rb /)
      kbo(:,48, 1) = (/ &
        & 0.31773e-09_rb,0.33161e-09_rb,0.34878e-09_rb,0.36970e-09_rb,0.39231e-09_rb /)
      kbo(:,49, 1) = (/ &
        & 0.25526e-09_rb,0.26613e-09_rb,0.27906e-09_rb,0.29510e-09_rb,0.31346e-09_rb /)
      kbo(:,50, 1) = (/ &
        & 0.20571e-09_rb,0.21419e-09_rb,0.22379e-09_rb,0.23622e-09_rb,0.25076e-09_rb /)
      kbo(:,51, 1) = (/ &
        & 0.16573e-09_rb,0.17251e-09_rb,0.18002e-09_rb,0.18925e-09_rb,0.20074e-09_rb /)
      kbo(:,52, 1) = (/ &
        & 0.13312e-09_rb,0.13888e-09_rb,0.14480e-09_rb,0.15170e-09_rb,0.16067e-09_rb /)
      kbo(:,53, 1) = (/ &
        & 0.10717e-09_rb,0.11200e-09_rb,0.11653e-09_rb,0.12187e-09_rb,0.12864e-09_rb /)
      kbo(:,54, 1) = (/ &
        & 0.85395e-10_rb,0.90190e-10_rb,0.93978e-10_rb,0.98115e-10_rb,0.10324e-09_rb /)
      kbo(:,55, 1) = (/ &
        & 0.67933e-10_rb,0.72682e-10_rb,0.75857e-10_rb,0.79079e-10_rb,0.82942e-10_rb /)
      kbo(:,56, 1) = (/ &
        & 0.53572e-10_rb,0.58095e-10_rb,0.61231e-10_rb,0.63738e-10_rb,0.66738e-10_rb /)
      kbo(:,57, 1) = (/ &
        & 0.41729e-10_rb,0.46380e-10_rb,0.49357e-10_rb,0.51409e-10_rb,0.53743e-10_rb /)
      kbo(:,58, 1) = (/ &
        & 0.32319e-10_rb,0.36737e-10_rb,0.39572e-10_rb,0.41487e-10_rb,0.43327e-10_rb /)
      kbo(:,59, 1) = (/ &
        & 0.25820e-10_rb,0.29541e-10_rb,0.32107e-10_rb,0.33762e-10_rb,0.35250e-10_rb /)
      kbo(:,13, 2) = (/ &
        & 0.93809e-05_rb,0.10536e-04_rb,0.11777e-04_rb,0.13098e-04_rb,0.14486e-04_rb /)
      kbo(:,14, 2) = (/ &
        & 0.77050e-05_rb,0.86520e-05_rb,0.96745e-05_rb,0.10759e-04_rb,0.11891e-04_rb /)
      kbo(:,15, 2) = (/ &
        & 0.63112e-05_rb,0.70914e-05_rb,0.79320e-05_rb,0.88179e-05_rb,0.97344e-05_rb /)
      kbo(:,16, 2) = (/ &
        & 0.51672e-05_rb,0.58172e-05_rb,0.64961e-05_rb,0.72133e-05_rb,0.79595e-05_rb /)
      kbo(:,17, 2) = (/ &
        & 0.42470e-05_rb,0.47817e-05_rb,0.53396e-05_rb,0.59229e-05_rb,0.65385e-05_rb /)
      kbo(:,18, 2) = (/ &
        & 0.35105e-05_rb,0.39513e-05_rb,0.44042e-05_rb,0.48865e-05_rb,0.53983e-05_rb /)
      kbo(:,19, 2) = (/ &
        & 0.29057e-05_rb,0.32680e-05_rb,0.36409e-05_rb,0.40419e-05_rb,0.44646e-05_rb /)
      kbo(:,20, 2) = (/ &
        & 0.24132e-05_rb,0.27111e-05_rb,0.30192e-05_rb,0.33498e-05_rb,0.36985e-05_rb /)
      kbo(:,21, 2) = (/ &
        & 0.20041e-05_rb,0.22484e-05_rb,0.25052e-05_rb,0.27770e-05_rb,0.30661e-05_rb /)
      kbo(:,22, 2) = (/ &
        & 0.16762e-05_rb,0.18775e-05_rb,0.20900e-05_rb,0.23158e-05_rb,0.25543e-05_rb /)
      kbo(:,23, 2) = (/ &
        & 0.14013e-05_rb,0.15665e-05_rb,0.17433e-05_rb,0.19308e-05_rb,0.21290e-05_rb /)
      kbo(:,24, 2) = (/ &
        & 0.11680e-05_rb,0.13039e-05_rb,0.14515e-05_rb,0.16085e-05_rb,0.17733e-05_rb /)
      kbo(:,25, 2) = (/ &
        & 0.97491e-06_rb,0.10886e-05_rb,0.12114e-05_rb,0.13428e-05_rb,0.14806e-05_rb /)
      kbo(:,26, 2) = (/ &
        & 0.81608e-06_rb,0.91198e-06_rb,0.10148e-05_rb,0.11244e-05_rb,0.12392e-05_rb /)
      kbo(:,27, 2) = (/ &
        & 0.68407e-06_rb,0.76432e-06_rb,0.85048e-06_rb,0.94210e-06_rb,0.10393e-05_rb /)
      kbo(:,28, 2) = (/ &
        & 0.57340e-06_rb,0.64071e-06_rb,0.71311e-06_rb,0.78988e-06_rb,0.87304e-06_rb /)
      kbo(:,29, 2) = (/ &
        & 0.48234e-06_rb,0.53886e-06_rb,0.59992e-06_rb,0.66557e-06_rb,0.73565e-06_rb /)
      kbo(:,30, 2) = (/ &
        & 0.40628e-06_rb,0.45384e-06_rb,0.50638e-06_rb,0.56175e-06_rb,0.62086e-06_rb /)
      kbo(:,31, 2) = (/ &
        & 0.34276e-06_rb,0.38409e-06_rb,0.42817e-06_rb,0.47551e-06_rb,0.52508e-06_rb /)
      kbo(:,32, 2) = (/ &
        & 0.29024e-06_rb,0.32548e-06_rb,0.36291e-06_rb,0.40263e-06_rb,0.44423e-06_rb /)
      kbo(:,33, 2) = (/ &
        & 0.24623e-06_rb,0.27596e-06_rb,0.30748e-06_rb,0.34104e-06_rb,0.37571e-06_rb /)
      kbo(:,34, 2) = (/ &
        & 0.20814e-06_rb,0.23318e-06_rb,0.25975e-06_rb,0.28746e-06_rb,0.31674e-06_rb /)
      kbo(:,35, 2) = (/ &
        & 0.17399e-06_rb,0.19476e-06_rb,0.21698e-06_rb,0.24021e-06_rb,0.26463e-06_rb /)
      kbo(:,36, 2) = (/ &
        & 0.14352e-06_rb,0.16079e-06_rb,0.17920e-06_rb,0.19838e-06_rb,0.21852e-06_rb /)
      kbo(:,37, 2) = (/ &
        & 0.11644e-06_rb,0.13061e-06_rb,0.14569e-06_rb,0.16148e-06_rb,0.17794e-06_rb /)
      kbo(:,38, 2) = (/ &
        & 0.94327e-07_rb,0.10597e-06_rb,0.11827e-06_rb,0.13121e-06_rb,0.14471e-06_rb /)
      kbo(:,39, 2) = (/ &
        & 0.76386e-07_rb,0.85932e-07_rb,0.96004e-07_rb,0.10675e-06_rb,0.11785e-06_rb /)
      kbo(:,40, 2) = (/ &
        & 0.61093e-07_rb,0.68856e-07_rb,0.77088e-07_rb,0.85831e-07_rb,0.94878e-07_rb /)
      kbo(:,41, 2) = (/ &
        & 0.48781e-07_rb,0.55058e-07_rb,0.61725e-07_rb,0.68852e-07_rb,0.76234e-07_rb /)
      kbo(:,42, 2) = (/ &
        & 0.38909e-07_rb,0.43987e-07_rb,0.49391e-07_rb,0.55161e-07_rb,0.61197e-07_rb /)
      kbo(:,43, 2) = (/ &
        & 0.30779e-07_rb,0.34866e-07_rb,0.39222e-07_rb,0.43870e-07_rb,0.48785e-07_rb /)
      kbo(:,44, 2) = (/ &
        & 0.24249e-07_rb,0.27514e-07_rb,0.31014e-07_rb,0.34774e-07_rb,0.38772e-07_rb /)
      kbo(:,45, 2) = (/ &
        & 0.19076e-07_rb,0.21693e-07_rb,0.24514e-07_rb,0.27522e-07_rb,0.30756e-07_rb /)
      kbo(:,46, 2) = (/ &
        & 0.14949e-07_rb,0.17046e-07_rb,0.19303e-07_rb,0.21721e-07_rb,0.24320e-07_rb /)
      kbo(:,47, 2) = (/ &
        & 0.11627e-07_rb,0.13294e-07_rb,0.15102e-07_rb,0.17040e-07_rb,0.19115e-07_rb /)
      kbo(:,48, 2) = (/ &
        & 0.90208e-08_rb,0.10359e-07_rb,0.11803e-07_rb,0.13349e-07_rb,0.15003e-07_rb /)
      kbo(:,49, 2) = (/ &
        & 0.69979e-08_rb,0.80625e-08_rb,0.92075e-08_rb,0.10445e-07_rb,0.11768e-07_rb /)
      kbo(:,50, 2) = (/ &
        & 0.54507e-08_rb,0.62777e-08_rb,0.72004e-08_rb,0.81899e-08_rb,0.92488e-08_rb /)
      kbo(:,51, 2) = (/ &
        & 0.42541e-08_rb,0.48933e-08_rb,0.56304e-08_rb,0.64214e-08_rb,0.72725e-08_rb /)
      kbo(:,52, 2) = (/ &
        & 0.33161e-08_rb,0.38121e-08_rb,0.43976e-08_rb,0.50301e-08_rb,0.57140e-08_rb /)
      kbo(:,53, 2) = (/ &
        & 0.25817e-08_rb,0.29691e-08_rb,0.34291e-08_rb,0.39368e-08_rb,0.44836e-08_rb /)
      kbo(:,54, 2) = (/ &
        & 0.20203e-08_rb,0.23228e-08_rb,0.26823e-08_rb,0.30900e-08_rb,0.35285e-08_rb /)
      kbo(:,55, 2) = (/ &
        & 0.15833e-08_rb,0.18196e-08_rb,0.21023e-08_rb,0.24276e-08_rb,0.27787e-08_rb /)
      kbo(:,56, 2) = (/ &
        & 0.12403e-08_rb,0.14250e-08_rb,0.16461e-08_rb,0.19045e-08_rb,0.21855e-08_rb /)
      kbo(:,57, 2) = (/ &
        & 0.97249e-09_rb,0.11164e-08_rb,0.12881e-08_rb,0.14923e-08_rb,0.17175e-08_rb /)
      kbo(:,58, 2) = (/ &
        & 0.76431e-09_rb,0.87699e-09_rb,0.10108e-08_rb,0.11709e-08_rb,0.13516e-08_rb /)
      kbo(:,59, 2) = (/ &
        & 0.61553e-09_rb,0.70649e-09_rb,0.81463e-09_rb,0.94315e-09_rb,0.10907e-08_rb /)
      kbo(:,13, 3) = (/ &
        & 0.37708e-04_rb,0.43942e-04_rb,0.50586e-04_rb,0.57656e-04_rb,0.65139e-04_rb /)
      kbo(:,14, 3) = (/ &
        & 0.31694e-04_rb,0.36862e-04_rb,0.42350e-04_rb,0.48197e-04_rb,0.54341e-04_rb /)
      kbo(:,15, 3) = (/ &
        & 0.26564e-04_rb,0.30824e-04_rb,0.35372e-04_rb,0.40180e-04_rb,0.45231e-04_rb /)
      kbo(:,16, 3) = (/ &
        & 0.22232e-04_rb,0.25740e-04_rb,0.29512e-04_rb,0.33447e-04_rb,0.37588e-04_rb /)
      kbo(:,17, 3) = (/ &
        & 0.18578e-04_rb,0.21467e-04_rb,0.24582e-04_rb,0.27832e-04_rb,0.31221e-04_rb /)
      kbo(:,18, 3) = (/ &
        & 0.15530e-04_rb,0.17922e-04_rb,0.20498e-04_rb,0.23170e-04_rb,0.25973e-04_rb /)
      kbo(:,19, 3) = (/ &
        & 0.12972e-04_rb,0.14952e-04_rb,0.17059e-04_rb,0.19270e-04_rb,0.21580e-04_rb /)
      kbo(:,20, 3) = (/ &
        & 0.10858e-04_rb,0.12498e-04_rb,0.14228e-04_rb,0.16061e-04_rb,0.17958e-04_rb /)
      kbo(:,21, 3) = (/ &
        & 0.90819e-05_rb,0.10446e-04_rb,0.11880e-04_rb,0.13394e-04_rb,0.14951e-04_rb /)
      kbo(:,22, 3) = (/ &
        & 0.76648e-05_rb,0.87979e-05_rb,0.99931e-05_rb,0.11255e-04_rb,0.12542e-04_rb /)
      kbo(:,23, 3) = (/ &
        & 0.64634e-05_rb,0.74140e-05_rb,0.84086e-05_rb,0.94546e-05_rb,0.10527e-04_rb /)
      kbo(:,24, 3) = (/ &
        & 0.54523e-05_rb,0.62464e-05_rb,0.70809e-05_rb,0.79508e-05_rb,0.88446e-05_rb /)
      kbo(:,25, 3) = (/ &
        & 0.46110e-05_rb,0.52716e-05_rb,0.59693e-05_rb,0.66904e-05_rb,0.74322e-05_rb /)
      kbo(:,26, 3) = (/ &
        & 0.39134e-05_rb,0.44620e-05_rb,0.50425e-05_rb,0.56439e-05_rb,0.62577e-05_rb /)
      kbo(:,27, 3) = (/ &
        & 0.33208e-05_rb,0.37777e-05_rb,0.42629e-05_rb,0.47607e-05_rb,0.52714e-05_rb /)
      kbo(:,28, 3) = (/ &
        & 0.28177e-05_rb,0.32008e-05_rb,0.36031e-05_rb,0.40187e-05_rb,0.44358e-05_rb /)
      kbo(:,29, 3) = (/ &
        & 0.23932e-05_rb,0.27158e-05_rb,0.30516e-05_rb,0.33945e-05_rb,0.37416e-05_rb /)
      kbo(:,30, 3) = (/ &
        & 0.20340e-05_rb,0.23035e-05_rb,0.25827e-05_rb,0.28672e-05_rb,0.31559e-05_rb /)
      kbo(:,31, 3) = (/ &
        & 0.17308e-05_rb,0.19547e-05_rb,0.21869e-05_rb,0.24243e-05_rb,0.26649e-05_rb /)
      kbo(:,32, 3) = (/ &
        & 0.14734e-05_rb,0.16593e-05_rb,0.18536e-05_rb,0.20518e-05_rb,0.22514e-05_rb /)
      kbo(:,33, 3) = (/ &
        & 0.12538e-05_rb,0.14097e-05_rb,0.15718e-05_rb,0.17369e-05_rb,0.19013e-05_rb /)
      kbo(:,34, 3) = (/ &
        & 0.10626e-05_rb,0.11932e-05_rb,0.13290e-05_rb,0.14662e-05_rb,0.16025e-05_rb /)
      kbo(:,35, 3) = (/ &
        & 0.88990e-06_rb,0.99878e-06_rb,0.11122e-05_rb,0.12266e-05_rb,0.13395e-05_rb /)
      kbo(:,36, 3) = (/ &
        & 0.73535e-06_rb,0.82690e-06_rb,0.92099e-06_rb,0.10159e-05_rb,0.11099e-05_rb /)
      kbo(:,37, 3) = (/ &
        & 0.59728e-06_rb,0.67301e-06_rb,0.75127e-06_rb,0.83021e-06_rb,0.90845e-06_rb /)
      kbo(:,38, 3) = (/ &
        & 0.48484e-06_rb,0.54737e-06_rb,0.61229e-06_rb,0.67762e-06_rb,0.74258e-06_rb /)
      kbo(:,39, 3) = (/ &
        & 0.39347e-06_rb,0.44485e-06_rb,0.49886e-06_rb,0.55290e-06_rb,0.60674e-06_rb /)
      kbo(:,40, 3) = (/ &
        & 0.31488e-06_rb,0.35703e-06_rb,0.40148e-06_rb,0.44640e-06_rb,0.49091e-06_rb /)
      kbo(:,41, 3) = (/ &
        & 0.25130e-06_rb,0.28561e-06_rb,0.32224e-06_rb,0.35953e-06_rb,0.39618e-06_rb /)
      kbo(:,42, 3) = (/ &
        & 0.20030e-06_rb,0.22840e-06_rb,0.25834e-06_rb,0.28930e-06_rb,0.31965e-06_rb /)
      kbo(:,43, 3) = (/ &
        & 0.15807e-06_rb,0.18109e-06_rb,0.20542e-06_rb,0.23094e-06_rb,0.25635e-06_rb /)
      kbo(:,44, 3) = (/ &
        & 0.12418e-06_rb,0.14282e-06_rb,0.16261e-06_rb,0.18364e-06_rb,0.20465e-06_rb /)
      kbo(:,45, 3) = (/ &
        & 0.97383e-07_rb,0.11245e-06_rb,0.12855e-06_rb,0.14572e-06_rb,0.16307e-06_rb /)
      kbo(:,46, 3) = (/ &
        & 0.76015e-07_rb,0.88067e-07_rb,0.10120e-06_rb,0.11510e-06_rb,0.12935e-06_rb /)
      kbo(:,47, 3) = (/ &
        & 0.58813e-07_rb,0.68493e-07_rb,0.79102e-07_rb,0.90273e-07_rb,0.10200e-06_rb /)
      kbo(:,48, 3) = (/ &
        & 0.45440e-07_rb,0.53144e-07_rb,0.61597e-07_rb,0.70639e-07_rb,0.80255e-07_rb /)
      kbo(:,49, 3) = (/ &
        & 0.35015e-07_rb,0.41136e-07_rb,0.47869e-07_rb,0.55216e-07_rb,0.62994e-07_rb /)
      kbo(:,50, 3) = (/ &
        & 0.27056e-07_rb,0.31910e-07_rb,0.37267e-07_rb,0.43210e-07_rb,0.49522e-07_rb /)
      kbo(:,51, 3) = (/ &
        & 0.20908e-07_rb,0.24756e-07_rb,0.29044e-07_rb,0.33791e-07_rb,0.38904e-07_rb /)
      kbo(:,52, 3) = (/ &
        & 0.16131e-07_rb,0.19181e-07_rb,0.22589e-07_rb,0.26378e-07_rb,0.30504e-07_rb /)
      kbo(:,53, 3) = (/ &
        & 0.12418e-07_rb,0.14835e-07_rb,0.17537e-07_rb,0.20547e-07_rb,0.23860e-07_rb /)
      kbo(:,54, 3) = (/ &
        & 0.95994e-08_rb,0.11514e-07_rb,0.13662e-07_rb,0.16065e-07_rb,0.18717e-07_rb /)
      kbo(:,55, 3) = (/ &
        & 0.74239e-08_rb,0.89453e-08_rb,0.10656e-07_rb,0.12569e-07_rb,0.14693e-07_rb /)
      kbo(:,56, 3) = (/ &
        & 0.57300e-08_rb,0.69441e-08_rb,0.83030e-08_rb,0.98193e-08_rb,0.11523e-07_rb /)
      kbo(:,57, 3) = (/ &
        & 0.44152e-08_rb,0.53811e-08_rb,0.64581e-08_rb,0.76610e-08_rb,0.90201e-08_rb /)
      kbo(:,58, 3) = (/ &
        & 0.34072e-08_rb,0.41708e-08_rb,0.50297e-08_rb,0.59876e-08_rb,0.70684e-08_rb /)
      kbo(:,59, 3) = (/ &
        & 0.27302e-08_rb,0.33492e-08_rb,0.40481e-08_rb,0.48266e-08_rb,0.57046e-08_rb /)
      kbo(:,13, 4) = (/ &
        & 0.12015e-03_rb,0.13920e-03_rb,0.15937e-03_rb,0.18001e-03_rb,0.19929e-03_rb /)
      kbo(:,14, 4) = (/ &
        & 0.10256e-03_rb,0.11883e-03_rb,0.13587e-03_rb,0.15259e-03_rb,0.16812e-03_rb /)
      kbo(:,15, 4) = (/ &
        & 0.87203e-04_rb,0.10099e-03_rb,0.11498e-03_rb,0.12847e-03_rb,0.14114e-03_rb /)
      kbo(:,16, 4) = (/ &
        & 0.73859e-04_rb,0.85304e-04_rb,0.96816e-04_rb,0.10774e-03_rb,0.11805e-03_rb /)
      kbo(:,17, 4) = (/ &
        & 0.62334e-04_rb,0.71786e-04_rb,0.81225e-04_rb,0.90091e-04_rb,0.98454e-04_rb /)
      kbo(:,18, 4) = (/ &
        & 0.52581e-04_rb,0.60344e-04_rb,0.68018e-04_rb,0.75237e-04_rb,0.81903e-04_rb /)
      kbo(:,19, 4) = (/ &
        & 0.44197e-04_rb,0.50573e-04_rb,0.56840e-04_rb,0.62656e-04_rb,0.67968e-04_rb /)
      kbo(:,20, 4) = (/ &
        & 0.37151e-04_rb,0.42384e-04_rb,0.47494e-04_rb,0.52193e-04_rb,0.56467e-04_rb /)
      kbo(:,21, 4) = (/ &
        & 0.31193e-04_rb,0.35480e-04_rb,0.39600e-04_rb,0.43427e-04_rb,0.46870e-04_rb /)
      kbo(:,22, 4) = (/ &
        & 0.26374e-04_rb,0.29925e-04_rb,0.33224e-04_rb,0.36261e-04_rb,0.39040e-04_rb /)
      kbo(:,23, 4) = (/ &
        & 0.22315e-04_rb,0.25198e-04_rb,0.27833e-04_rb,0.30290e-04_rb,0.32509e-04_rb /)
      kbo(:,24, 4) = (/ &
        & 0.18883e-04_rb,0.21184e-04_rb,0.23291e-04_rb,0.25268e-04_rb,0.27046e-04_rb /)
      kbo(:,25, 4) = (/ &
        & 0.15977e-04_rb,0.17808e-04_rb,0.19529e-04_rb,0.21106e-04_rb,0.22533e-04_rb /)
      kbo(:,26, 4) = (/ &
        & 0.13524e-04_rb,0.14996e-04_rb,0.16391e-04_rb,0.17658e-04_rb,0.18796e-04_rb /)
      kbo(:,27, 4) = (/ &
        & 0.11433e-04_rb,0.12631e-04_rb,0.13757e-04_rb,0.14769e-04_rb,0.15677e-04_rb /)
      kbo(:,28, 4) = (/ &
        & 0.96558e-05_rb,0.10640e-04_rb,0.11537e-04_rb,0.12357e-04_rb,0.13086e-04_rb /)
      kbo(:,29, 4) = (/ &
        & 0.81632e-05_rb,0.89560e-05_rb,0.96801e-05_rb,0.10339e-04_rb,0.10919e-04_rb /)
      kbo(:,30, 4) = (/ &
        & 0.68972e-05_rb,0.75321e-05_rb,0.81189e-05_rb,0.86418e-05_rb,0.91120e-05_rb /)
      kbo(:,31, 4) = (/ &
        & 0.58255e-05_rb,0.63427e-05_rb,0.68115e-05_rb,0.72291e-05_rb,0.76088e-05_rb /)
      kbo(:,32, 4) = (/ &
        & 0.49235e-05_rb,0.53404e-05_rb,0.57128e-05_rb,0.60450e-05_rb,0.63540e-05_rb /)
      kbo(:,33, 4) = (/ &
        & 0.41602e-05_rb,0.44904e-05_rb,0.47873e-05_rb,0.50567e-05_rb,0.53085e-05_rb /)
      kbo(:,34, 4) = (/ &
        & 0.34993e-05_rb,0.37640e-05_rb,0.40033e-05_rb,0.42217e-05_rb,0.44321e-05_rb /)
      kbo(:,35, 4) = (/ &
        & 0.29168e-05_rb,0.31331e-05_rb,0.33264e-05_rb,0.35072e-05_rb,0.36808e-05_rb /)
      kbo(:,36, 4) = (/ &
        & 0.24105e-05_rb,0.25863e-05_rb,0.27459e-05_rb,0.28969e-05_rb,0.30433e-05_rb /)
      kbo(:,37, 4) = (/ &
        & 0.19699e-05_rb,0.21167e-05_rb,0.22499e-05_rb,0.23751e-05_rb,0.24984e-05_rb /)
      kbo(:,38, 4) = (/ &
        & 0.16085e-05_rb,0.17316e-05_rb,0.18431e-05_rb,0.19482e-05_rb,0.20520e-05_rb /)
      kbo(:,39, 4) = (/ &
        & 0.13140e-05_rb,0.14172e-05_rb,0.15102e-05_rb,0.15989e-05_rb,0.16862e-05_rb /)
      kbo(:,40, 4) = (/ &
        & 0.10643e-05_rb,0.11518e-05_rb,0.12300e-05_rb,0.13052e-05_rb,0.13789e-05_rb /)
      kbo(:,41, 4) = (/ &
        & 0.86103e-06_rb,0.93494e-06_rb,0.10013e-05_rb,0.10648e-05_rb,0.11275e-05_rb /)
      kbo(:,42, 4) = (/ &
        & 0.69566e-06_rb,0.75803e-06_rb,0.81468e-06_rb,0.86817e-06_rb,0.92113e-06_rb /)
      kbo(:,43, 4) = (/ &
        & 0.55856e-06_rb,0.61135e-06_rb,0.65992e-06_rb,0.70501e-06_rb,0.74915e-06_rb /)
      kbo(:,44, 4) = (/ &
        & 0.44644e-06_rb,0.49113e-06_rb,0.53284e-06_rb,0.57066e-06_rb,0.60776e-06_rb /)
      kbo(:,45, 4) = (/ &
        & 0.35638e-06_rb,0.39386e-06_rb,0.42947e-06_rb,0.46192e-06_rb,0.49295e-06_rb /)
      kbo(:,46, 4) = (/ &
        & 0.28317e-06_rb,0.31490e-06_rb,0.34503e-06_rb,0.37276e-06_rb,0.39894e-06_rb /)
      kbo(:,47, 4) = (/ &
        & 0.22341e-06_rb,0.25002e-06_rb,0.27560e-06_rb,0.29957e-06_rb,0.32144e-06_rb /)
      kbo(:,48, 4) = (/ &
        & 0.17554e-06_rb,0.19798e-06_rb,0.21962e-06_rb,0.24010e-06_rb,0.25876e-06_rb /)
      kbo(:,49, 4) = (/ &
        & 0.13737e-06_rb,0.15642e-06_rb,0.17451e-06_rb,0.19192e-06_rb,0.20795e-06_rb /)
      kbo(:,50, 4) = (/ &
        & 0.10754e-06_rb,0.12357e-06_rb,0.13879e-06_rb,0.15343e-06_rb,0.16712e-06_rb /)
      kbo(:,51, 4) = (/ &
        & 0.84121e-07_rb,0.97441e-07_rb,0.11018e-06_rb,0.12256e-06_rb,0.13419e-06_rb /)
      kbo(:,52, 4) = (/ &
        & 0.65604e-07_rb,0.76564e-07_rb,0.87356e-07_rb,0.97703e-07_rb,0.10745e-06_rb /)
      kbo(:,53, 4) = (/ &
        & 0.50943e-07_rb,0.59988e-07_rb,0.69048e-07_rb,0.77629e-07_rb,0.85933e-07_rb /)
      kbo(:,54, 4) = (/ &
        & 0.39679e-07_rb,0.47121e-07_rb,0.54614e-07_rb,0.61766e-07_rb,0.68783e-07_rb /)
      kbo(:,55, 4) = (/ &
        & 0.30934e-07_rb,0.36980e-07_rb,0.43113e-07_rb,0.49142e-07_rb,0.55005e-07_rb /)
      kbo(:,56, 4) = (/ &
        & 0.24050e-07_rb,0.28923e-07_rb,0.33993e-07_rb,0.39047e-07_rb,0.43928e-07_rb /)
      kbo(:,57, 4) = (/ &
        & 0.18660e-07_rb,0.22557e-07_rb,0.26748e-07_rb,0.30951e-07_rb,0.35000e-07_rb /)
      kbo(:,58, 4) = (/ &
        & 0.14488e-07_rb,0.17625e-07_rb,0.21043e-07_rb,0.24511e-07_rb,0.27884e-07_rb /)
      kbo(:,59, 4) = (/ &
        & 0.11674e-07_rb,0.14234e-07_rb,0.17044e-07_rb,0.19902e-07_rb,0.22702e-07_rb /)
      kbo(:,13, 5) = (/ &
        & 0.36288e-03_rb,0.38577e-03_rb,0.40824e-03_rb,0.43207e-03_rb,0.45772e-03_rb /)
      kbo(:,14, 5) = (/ &
        & 0.30781e-03_rb,0.32754e-03_rb,0.34745e-03_rb,0.36873e-03_rb,0.39050e-03_rb /)
      kbo(:,15, 5) = (/ &
        & 0.26023e-03_rb,0.27734e-03_rb,0.29481e-03_rb,0.31288e-03_rb,0.33087e-03_rb /)
      kbo(:,16, 5) = (/ &
        & 0.21963e-03_rb,0.23414e-03_rb,0.24882e-03_rb,0.26383e-03_rb,0.27866e-03_rb /)
      kbo(:,17, 5) = (/ &
        & 0.18486e-03_rb,0.19716e-03_rb,0.20932e-03_rb,0.22139e-03_rb,0.23350e-03_rb /)
      kbo(:,18, 5) = (/ &
        & 0.15516e-03_rb,0.16534e-03_rb,0.17526e-03_rb,0.18520e-03_rb,0.19528e-03_rb /)
      kbo(:,19, 5) = (/ &
        & 0.12986e-03_rb,0.13820e-03_rb,0.14634e-03_rb,0.15465e-03_rb,0.16304e-03_rb /)
      kbo(:,20, 5) = (/ &
        & 0.10854e-03_rb,0.11542e-03_rb,0.12213e-03_rb,0.12898e-03_rb,0.13583e-03_rb /)
      kbo(:,21, 5) = (/ &
        & 0.90628e-04_rb,0.96255e-04_rb,0.10177e-03_rb,0.10738e-03_rb,0.11299e-03_rb /)
      kbo(:,22, 5) = (/ &
        & 0.75891e-04_rb,0.80410e-04_rb,0.85018e-04_rb,0.89561e-04_rb,0.94064e-04_rb /)
      kbo(:,23, 5) = (/ &
        & 0.63476e-04_rb,0.67177e-04_rb,0.70959e-04_rb,0.74590e-04_rb,0.78334e-04_rb /)
      kbo(:,24, 5) = (/ &
        & 0.52999e-04_rb,0.56057e-04_rb,0.59157e-04_rb,0.62167e-04_rb,0.65272e-04_rb /)
      kbo(:,25, 5) = (/ &
        & 0.44256e-04_rb,0.46783e-04_rb,0.49281e-04_rb,0.51789e-04_rb,0.54376e-04_rb /)
      kbo(:,26, 5) = (/ &
        & 0.37011e-04_rb,0.39100e-04_rb,0.41098e-04_rb,0.43198e-04_rb,0.45322e-04_rb /)
      kbo(:,27, 5) = (/ &
        & 0.30966e-04_rb,0.32635e-04_rb,0.34289e-04_rb,0.36026e-04_rb,0.37776e-04_rb /)
      kbo(:,28, 5) = (/ &
        & 0.25866e-04_rb,0.27210e-04_rb,0.28616e-04_rb,0.30009e-04_rb,0.31487e-04_rb /)
      kbo(:,29, 5) = (/ &
        & 0.21609e-04_rb,0.22715e-04_rb,0.23863e-04_rb,0.25052e-04_rb,0.26299e-04_rb /)
      kbo(:,30, 5) = (/ &
        & 0.18055e-04_rb,0.18981e-04_rb,0.19946e-04_rb,0.20948e-04_rb,0.21978e-04_rb /)
      kbo(:,31, 5) = (/ &
        & 0.15103e-04_rb,0.15890e-04_rb,0.16699e-04_rb,0.17556e-04_rb,0.18410e-04_rb /)
      kbo(:,32, 5) = (/ &
        & 0.12647e-04_rb,0.13313e-04_rb,0.14001e-04_rb,0.14715e-04_rb,0.15427e-04_rb /)
      kbo(:,33, 5) = (/ &
        & 0.10601e-04_rb,0.11164e-04_rb,0.11755e-04_rb,0.12351e-04_rb,0.12931e-04_rb /)
      kbo(:,34, 5) = (/ &
        & 0.88699e-05_rb,0.93482e-05_rb,0.98481e-05_rb,0.10351e-04_rb,0.10829e-04_rb /)
      kbo(:,35, 5) = (/ &
        & 0.73864e-05_rb,0.77932e-05_rb,0.82148e-05_rb,0.86289e-05_rb,0.90270e-05_rb /)
      kbo(:,36, 5) = (/ &
        & 0.61189e-05_rb,0.64631e-05_rb,0.68137e-05_rb,0.71535e-05_rb,0.74761e-05_rb /)
      kbo(:,37, 5) = (/ &
        & 0.50349e-05_rb,0.53216e-05_rb,0.56110e-05_rb,0.58984e-05_rb,0.61738e-05_rb /)
      kbo(:,38, 5) = (/ &
        & 0.41408e-05_rb,0.43804e-05_rb,0.46209e-05_rb,0.48585e-05_rb,0.50962e-05_rb /)
      kbo(:,39, 5) = (/ &
        & 0.34100e-05_rb,0.36097e-05_rb,0.38105e-05_rb,0.40114e-05_rb,0.42081e-05_rb /)
      kbo(:,40, 5) = (/ &
        & 0.27924e-05_rb,0.29567e-05_rb,0.31274e-05_rb,0.32925e-05_rb,0.34580e-05_rb /)
      kbo(:,41, 5) = (/ &
        & 0.22821e-05_rb,0.24196e-05_rb,0.25613e-05_rb,0.27009e-05_rb,0.28413e-05_rb /)
      kbo(:,42, 5) = (/ &
        & 0.18665e-05_rb,0.19799e-05_rb,0.20975e-05_rb,0.22150e-05_rb,0.23326e-05_rb /)
      kbo(:,43, 5) = (/ &
        & 0.15195e-05_rb,0.16130e-05_rb,0.17109e-05_rb,0.18095e-05_rb,0.19094e-05_rb /)
      kbo(:,44, 5) = (/ &
        & 0.12351e-05_rb,0.13129e-05_rb,0.13926e-05_rb,0.14756e-05_rb,0.15593e-05_rb /)
      kbo(:,45, 5) = (/ &
        & 0.10023e-05_rb,0.10671e-05_rb,0.11334e-05_rb,0.12033e-05_rb,0.12728e-05_rb /)
      kbo(:,46, 5) = (/ &
        & 0.81269e-06_rb,0.86595e-06_rb,0.92094e-06_rb,0.97875e-06_rb,0.10378e-05_rb /)
      kbo(:,47, 5) = (/ &
        & 0.65647e-06_rb,0.70081e-06_rb,0.74604e-06_rb,0.79395e-06_rb,0.84339e-06_rb /)
      kbo(:,48, 5) = (/ &
        & 0.53019e-06_rb,0.56638e-06_rb,0.60399e-06_rb,0.64368e-06_rb,0.68472e-06_rb /)
      kbo(:,49, 5) = (/ &
        & 0.42776e-06_rb,0.45739e-06_rb,0.48880e-06_rb,0.52108e-06_rb,0.55567e-06_rb /)
      kbo(:,50, 5) = (/ &
        & 0.34556e-06_rb,0.36989e-06_rb,0.39599e-06_rb,0.42284e-06_rb,0.45123e-06_rb /)
      kbo(:,51, 5) = (/ &
        & 0.27905e-06_rb,0.29932e-06_rb,0.32065e-06_rb,0.34295e-06_rb,0.36646e-06_rb /)
      kbo(:,52, 5) = (/ &
        & 0.22522e-06_rb,0.24196e-06_rb,0.25940e-06_rb,0.27788e-06_rb,0.29753e-06_rb /)
      kbo(:,53, 5) = (/ &
        & 0.18137e-06_rb,0.19527e-06_rb,0.20965e-06_rb,0.22512e-06_rb,0.24112e-06_rb /)
      kbo(:,54, 5) = (/ &
        & 0.14606e-06_rb,0.15795e-06_rb,0.16971e-06_rb,0.18256e-06_rb,0.19571e-06_rb /)
      kbo(:,55, 5) = (/ &
        & 0.11745e-06_rb,0.12764e-06_rb,0.13751e-06_rb,0.14796e-06_rb,0.15891e-06_rb /)
      kbo(:,56, 5) = (/ &
        & 0.94319e-07_rb,0.10301e-06_rb,0.11131e-06_rb,0.11979e-06_rb,0.12892e-06_rb /)
      kbo(:,57, 5) = (/ &
        & 0.75512e-07_rb,0.83064e-07_rb,0.90010e-07_rb,0.96928e-07_rb,0.10441e-06_rb /)
      kbo(:,58, 5) = (/ &
        & 0.60409e-07_rb,0.66900e-07_rb,0.72712e-07_rb,0.78451e-07_rb,0.84657e-07_rb /)
      kbo(:,59, 5) = (/ &
        & 0.49299e-07_rb,0.54731e-07_rb,0.59628e-07_rb,0.64415e-07_rb,0.69585e-07_rb /)
      kbo(:,13, 6) = (/ &
        & 0.85498e-03_rb,0.88654e-03_rb,0.92216e-03_rb,0.96018e-03_rb,0.10038e-02_rb /)
      kbo(:,14, 6) = (/ &
        & 0.72755e-03_rb,0.75503e-03_rb,0.78615e-03_rb,0.82104e-03_rb,0.86252e-03_rb /)
      kbo(:,15, 6) = (/ &
        & 0.61644e-03_rb,0.64130e-03_rb,0.66934e-03_rb,0.70298e-03_rb,0.74077e-03_rb /)
      kbo(:,16, 6) = (/ &
        & 0.52100e-03_rb,0.54364e-03_rb,0.57046e-03_rb,0.60137e-03_rb,0.63388e-03_rb /)
      kbo(:,17, 6) = (/ &
        & 0.43945e-03_rb,0.46060e-03_rb,0.48533e-03_rb,0.51276e-03_rb,0.54036e-03_rb /)
      kbo(:,18, 6) = (/ &
        & 0.37082e-03_rb,0.39025e-03_rb,0.41242e-03_rb,0.43532e-03_rb,0.45905e-03_rb /)
      kbo(:,19, 6) = (/ &
        & 0.31280e-03_rb,0.33005e-03_rb,0.34867e-03_rb,0.36806e-03_rb,0.38776e-03_rb /)
      kbo(:,20, 6) = (/ &
        & 0.26373e-03_rb,0.27881e-03_rb,0.29445e-03_rb,0.31046e-03_rb,0.32667e-03_rb /)
      kbo(:,21, 6) = (/ &
        & 0.22211e-03_rb,0.23470e-03_rb,0.24766e-03_rb,0.26115e-03_rb,0.27459e-03_rb /)
      kbo(:,22, 6) = (/ &
        & 0.18729e-03_rb,0.19773e-03_rb,0.20869e-03_rb,0.22010e-03_rb,0.23102e-03_rb /)
      kbo(:,23, 6) = (/ &
        & 0.15755e-03_rb,0.16634e-03_rb,0.17574e-03_rb,0.18527e-03_rb,0.19396e-03_rb /)
      kbo(:,24, 6) = (/ &
        & 0.13244e-03_rb,0.14003e-03_rb,0.14791e-03_rb,0.15556e-03_rb,0.16235e-03_rb /)
      kbo(:,25, 6) = (/ &
        & 0.11137e-03_rb,0.11779e-03_rb,0.12437e-03_rb,0.13036e-03_rb,0.13579e-03_rb /)
      kbo(:,26, 6) = (/ &
        & 0.93817e-04_rb,0.99206e-04_rb,0.10452e-03_rb,0.10918e-03_rb,0.11356e-03_rb /)
      kbo(:,27, 6) = (/ &
        & 0.79006e-04_rb,0.83531e-04_rb,0.87717e-04_rb,0.91355e-04_rb,0.95066e-04_rb /)
      kbo(:,28, 6) = (/ &
        & 0.66464e-04_rb,0.70180e-04_rb,0.73412e-04_rb,0.76548e-04_rb,0.79673e-04_rb /)
      kbo(:,29, 6) = (/ &
        & 0.55955e-04_rb,0.58876e-04_rb,0.61539e-04_rb,0.64161e-04_rb,0.66744e-04_rb /)
      kbo(:,30, 6) = (/ &
        & 0.47057e-04_rb,0.49392e-04_rb,0.51539e-04_rb,0.53762e-04_rb,0.55883e-04_rb /)
      kbo(:,31, 6) = (/ &
        & 0.39571e-04_rb,0.41398e-04_rb,0.43230e-04_rb,0.45052e-04_rb,0.46859e-04_rb /)
      kbo(:,32, 6) = (/ &
        & 0.33232e-04_rb,0.34753e-04_rb,0.36306e-04_rb,0.37802e-04_rb,0.39368e-04_rb /)
      kbo(:,33, 6) = (/ &
        & 0.27886e-04_rb,0.29183e-04_rb,0.30428e-04_rb,0.31753e-04_rb,0.33114e-04_rb /)
      kbo(:,34, 6) = (/ &
        & 0.23408e-04_rb,0.24477e-04_rb,0.25555e-04_rb,0.26683e-04_rb,0.27811e-04_rb /)
      kbo(:,35, 6) = (/ &
        & 0.19576e-04_rb,0.20476e-04_rb,0.21380e-04_rb,0.22323e-04_rb,0.23289e-04_rb /)
      kbo(:,36, 6) = (/ &
        & 0.16284e-04_rb,0.17049e-04_rb,0.17816e-04_rb,0.18626e-04_rb,0.19444e-04_rb /)
      kbo(:,37, 6) = (/ &
        & 0.13488e-04_rb,0.14134e-04_rb,0.14781e-04_rb,0.15488e-04_rb,0.16173e-04_rb /)
      kbo(:,38, 6) = (/ &
        & 0.11176e-04_rb,0.11721e-04_rb,0.12278e-04_rb,0.12866e-04_rb,0.13450e-04_rb /)
      kbo(:,39, 6) = (/ &
        & 0.92494e-05_rb,0.97155e-05_rb,0.10200e-04_rb,0.10699e-04_rb,0.11199e-04_rb /)
      kbo(:,40, 6) = (/ &
        & 0.76168e-05_rb,0.80109e-05_rb,0.84197e-05_rb,0.88496e-05_rb,0.92853e-05_rb /)
      kbo(:,41, 6) = (/ &
        & 0.62645e-05_rb,0.66003e-05_rb,0.69472e-05_rb,0.73123e-05_rb,0.76881e-05_rb /)
      kbo(:,42, 6) = (/ &
        & 0.51581e-05_rb,0.54373e-05_rb,0.57325e-05_rb,0.60421e-05_rb,0.63612e-05_rb /)
      kbo(:,43, 6) = (/ &
        & 0.42336e-05_rb,0.44705e-05_rb,0.47241e-05_rb,0.49790e-05_rb,0.52554e-05_rb /)
      kbo(:,44, 6) = (/ &
        & 0.34635e-05_rb,0.36643e-05_rb,0.38777e-05_rb,0.40942e-05_rb,0.43335e-05_rb /)
      kbo(:,45, 6) = (/ &
        & 0.28355e-05_rb,0.30041e-05_rb,0.31829e-05_rb,0.33678e-05_rb,0.35664e-05_rb /)
      kbo(:,46, 6) = (/ &
        & 0.23151e-05_rb,0.24589e-05_rb,0.26085e-05_rb,0.27655e-05_rb,0.29287e-05_rb /)
      kbo(:,47, 6) = (/ &
        & 0.18832e-05_rb,0.20045e-05_rb,0.21306e-05_rb,0.22633e-05_rb,0.24025e-05_rb /)
      kbo(:,48, 6) = (/ &
        & 0.15291e-05_rb,0.16326e-05_rb,0.17368e-05_rb,0.18516e-05_rb,0.19681e-05_rb /)
      kbo(:,49, 6) = (/ &
        & 0.12403e-05_rb,0.13270e-05_rb,0.14154e-05_rb,0.15108e-05_rb,0.16100e-05_rb /)
      kbo(:,50, 6) = (/ &
        & 0.10056e-05_rb,0.10792e-05_rb,0.11542e-05_rb,0.12335e-05_rb,0.13173e-05_rb /)
      kbo(:,51, 6) = (/ &
        & 0.81549e-06_rb,0.87791e-06_rb,0.94116e-06_rb,0.10080e-05_rb,0.10781e-05_rb /)
      kbo(:,52, 6) = (/ &
        & 0.66029e-06_rb,0.71307e-06_rb,0.76617e-06_rb,0.82179e-06_rb,0.88107e-06_rb /)
      kbo(:,53, 6) = (/ &
        & 0.53353e-06_rb,0.57858e-06_rb,0.62276e-06_rb,0.66919e-06_rb,0.71931e-06_rb /)
      kbo(:,54, 6) = (/ &
        & 0.43199e-06_rb,0.46970e-06_rb,0.50695e-06_rb,0.54584e-06_rb,0.58793e-06_rb /)
      kbo(:,55, 6) = (/ &
        & 0.34966e-06_rb,0.38119e-06_rb,0.41298e-06_rb,0.44530e-06_rb,0.48058e-06_rb /)
      kbo(:,56, 6) = (/ &
        & 0.28227e-06_rb,0.30903e-06_rb,0.33563e-06_rb,0.36285e-06_rb,0.39216e-06_rb /)
      kbo(:,57, 6) = (/ &
        & 0.22764e-06_rb,0.25006e-06_rb,0.27265e-06_rb,0.29503e-06_rb,0.31940e-06_rb /)
      kbo(:,58, 6) = (/ &
        & 0.18380e-06_rb,0.20248e-06_rb,0.22144e-06_rb,0.24002e-06_rb,0.25999e-06_rb /)
      kbo(:,59, 6) = (/ &
        & 0.15102e-06_rb,0.16680e-06_rb,0.18267e-06_rb,0.19825e-06_rb,0.21512e-06_rb /)
      kbo(:,13, 7) = (/ &
        & 0.20763e-02_rb,0.21202e-02_rb,0.21697e-02_rb,0.22297e-02_rb,0.22985e-02_rb /)
      kbo(:,14, 7) = (/ &
        & 0.17631e-02_rb,0.18004e-02_rb,0.18452e-02_rb,0.18998e-02_rb,0.19604e-02_rb /)
      kbo(:,15, 7) = (/ &
        & 0.14970e-02_rb,0.15286e-02_rb,0.15695e-02_rb,0.16172e-02_rb,0.16748e-02_rb /)
      kbo(:,16, 7) = (/ &
        & 0.12672e-02_rb,0.12965e-02_rb,0.13340e-02_rb,0.13795e-02_rb,0.14367e-02_rb /)
      kbo(:,17, 7) = (/ &
        & 0.10695e-02_rb,0.10974e-02_rb,0.11329e-02_rb,0.11783e-02_rb,0.12365e-02_rb /)
      kbo(:,18, 7) = (/ &
        & 0.90477e-03_rb,0.93070e-03_rb,0.96470e-03_rb,0.10105e-02_rb,0.10654e-02_rb /)
      kbo(:,19, 7) = (/ &
        & 0.76431e-03_rb,0.79078e-03_rb,0.82660e-03_rb,0.86991e-03_rb,0.91843e-03_rb /)
      kbo(:,20, 7) = (/ &
        & 0.64676e-03_rb,0.67404e-03_rb,0.70901e-03_rb,0.74996e-03_rb,0.79243e-03_rb /)
      kbo(:,21, 7) = (/ &
        & 0.54776e-03_rb,0.57523e-03_rb,0.60880e-03_rb,0.64511e-03_rb,0.68243e-03_rb /)
      kbo(:,22, 7) = (/ &
        & 0.46614e-03_rb,0.49254e-03_rb,0.52330e-03_rb,0.55502e-03_rb,0.58894e-03_rb /)
      kbo(:,23, 7) = (/ &
        & 0.39726e-03_rb,0.42201e-03_rb,0.44841e-03_rb,0.47653e-03_rb,0.50717e-03_rb /)
      kbo(:,24, 7) = (/ &
        & 0.33898e-03_rb,0.36121e-03_rb,0.38421e-03_rb,0.40924e-03_rb,0.43694e-03_rb /)
      kbo(:,25, 7) = (/ &
        & 0.28981e-03_rb,0.30891e-03_rb,0.32925e-03_rb,0.35185e-03_rb,0.37616e-03_rb /)
      kbo(:,26, 7) = (/ &
        & 0.24765e-03_rb,0.26436e-03_rb,0.28268e-03_rb,0.30306e-03_rb,0.32381e-03_rb /)
      kbo(:,27, 7) = (/ &
        & 0.21161e-03_rb,0.22639e-03_rb,0.24293e-03_rb,0.26080e-03_rb,0.27811e-03_rb /)
      kbo(:,28, 7) = (/ &
        & 0.18103e-03_rb,0.19425e-03_rb,0.20911e-03_rb,0.22426e-03_rb,0.23882e-03_rb /)
      kbo(:,29, 7) = (/ &
        & 0.15524e-03_rb,0.16726e-03_rb,0.18025e-03_rb,0.19313e-03_rb,0.20539e-03_rb /)
      kbo(:,30, 7) = (/ &
        & 0.13346e-03_rb,0.14432e-03_rb,0.15555e-03_rb,0.16625e-03_rb,0.17682e-03_rb /)
      kbo(:,31, 7) = (/ &
        & 0.11513e-03_rb,0.12466e-03_rb,0.13417e-03_rb,0.14329e-03_rb,0.15286e-03_rb /)
      kbo(:,32, 7) = (/ &
        & 0.99437e-04_rb,0.10768e-03_rb,0.11569e-03_rb,0.12397e-03_rb,0.13224e-03_rb /)
      kbo(:,33, 7) = (/ &
        & 0.85975e-04_rb,0.92901e-04_rb,0.10007e-03_rb,0.10742e-03_rb,0.11487e-03_rb /)
      kbo(:,34, 7) = (/ &
        & 0.74186e-04_rb,0.80283e-04_rb,0.86524e-04_rb,0.92965e-04_rb,0.99720e-04_rb /)
      kbo(:,35, 7) = (/ &
        & 0.63518e-04_rb,0.68833e-04_rb,0.74348e-04_rb,0.80205e-04_rb,0.86286e-04_rb /)
      kbo(:,36, 7) = (/ &
        & 0.53971e-04_rb,0.58710e-04_rb,0.63581e-04_rb,0.68731e-04_rb,0.74066e-04_rb /)
      kbo(:,37, 7) = (/ &
        & 0.45287e-04_rb,0.49433e-04_rb,0.53695e-04_rb,0.58207e-04_rb,0.62933e-04_rb /)
      kbo(:,38, 7) = (/ &
        & 0.37993e-04_rb,0.41562e-04_rb,0.45327e-04_rb,0.49315e-04_rb,0.53437e-04_rb /)
      kbo(:,39, 7) = (/ &
        & 0.31889e-04_rb,0.34954e-04_rb,0.38268e-04_rb,0.41760e-04_rb,0.45472e-04_rb /)
      kbo(:,40, 7) = (/ &
        & 0.26490e-04_rb,0.29130e-04_rb,0.31995e-04_rb,0.35094e-04_rb,0.38302e-04_rb /)
      kbo(:,41, 7) = (/ &
        & 0.21977e-04_rb,0.24258e-04_rb,0.26702e-04_rb,0.29405e-04_rb,0.32254e-04_rb /)
      kbo(:,42, 7) = (/ &
        & 0.18218e-04_rb,0.20180e-04_rb,0.22299e-04_rb,0.24616e-04_rb,0.27154e-04_rb /)
      kbo(:,43, 7) = (/ &
        & 0.15034e-04_rb,0.16668e-04_rb,0.18493e-04_rb,0.20514e-04_rb,0.22747e-04_rb /)
      kbo(:,44, 7) = (/ &
        & 0.12376e-04_rb,0.13730e-04_rb,0.15276e-04_rb,0.16983e-04_rb,0.18913e-04_rb /)
      kbo(:,45, 7) = (/ &
        & 0.10149e-04_rb,0.11314e-04_rb,0.12590e-04_rb,0.14099e-04_rb,0.15742e-04_rb /)
      kbo(:,46, 7) = (/ &
        & 0.82889e-05_rb,0.92859e-05_rb,0.10354e-04_rb,0.11603e-04_rb,0.13042e-04_rb /)
      kbo(:,47, 7) = (/ &
        & 0.67264e-05_rb,0.75707e-05_rb,0.84748e-05_rb,0.95081e-05_rb,0.10721e-04_rb /)
      kbo(:,48, 7) = (/ &
        & 0.54454e-05_rb,0.61678e-05_rb,0.69329e-05_rb,0.77845e-05_rb,0.87984e-05_rb /)
      kbo(:,49, 7) = (/ &
        & 0.43942e-05_rb,0.50075e-05_rb,0.56558e-05_rb,0.63802e-05_rb,0.72168e-05_rb /)
      kbo(:,50, 7) = (/ &
        & 0.35518e-05_rb,0.40682e-05_rb,0.46246e-05_rb,0.52318e-05_rb,0.59289e-05_rb /)
      kbo(:,51, 7) = (/ &
        & 0.28695e-05_rb,0.33032e-05_rb,0.37766e-05_rb,0.42938e-05_rb,0.48871e-05_rb /)
      kbo(:,52, 7) = (/ &
        & 0.23151e-05_rb,0.26733e-05_rb,0.30760e-05_rb,0.35184e-05_rb,0.40216e-05_rb /)
      kbo(:,53, 7) = (/ &
        & 0.18635e-05_rb,0.21569e-05_rb,0.24979e-05_rb,0.28705e-05_rb,0.33004e-05_rb /)
      kbo(:,54, 7) = (/ &
        & 0.15043e-05_rb,0.17492e-05_rb,0.20309e-05_rb,0.23469e-05_rb,0.27129e-05_rb /)
      kbo(:,55, 7) = (/ &
        & 0.12156e-05_rb,0.14170e-05_rb,0.16506e-05_rb,0.19188e-05_rb,0.22303e-05_rb /)
      kbo(:,56, 7) = (/ &
        & 0.98090e-06_rb,0.11440e-05_rb,0.13395e-05_rb,0.15647e-05_rb,0.18280e-05_rb /)
      kbo(:,57, 7) = (/ &
        & 0.79029e-06_rb,0.92216e-06_rb,0.10880e-05_rb,0.12732e-05_rb,0.14934e-05_rb /)
      kbo(:,58, 7) = (/ &
        & 0.63691e-06_rb,0.74365e-06_rb,0.88187e-06_rb,0.10375e-05_rb,0.12193e-05_rb /)
      kbo(:,59, 7) = (/ &
        & 0.52696e-06_rb,0.61783e-06_rb,0.73672e-06_rb,0.87054e-06_rb,0.10284e-05_rb /)
      kbo(:,13, 8) = (/ &
        & 0.57453e-02_rb,0.59077e-02_rb,0.61032e-02_rb,0.63038e-02_rb,0.65156e-02_rb /)
      kbo(:,14, 8) = (/ &
        & 0.49738e-02_rb,0.51301e-02_rb,0.52927e-02_rb,0.54623e-02_rb,0.56575e-02_rb /)
      kbo(:,15, 8) = (/ &
        & 0.43017e-02_rb,0.44326e-02_rb,0.45703e-02_rb,0.47292e-02_rb,0.49039e-02_rb /)
      kbo(:,16, 8) = (/ &
        & 0.37082e-02_rb,0.38170e-02_rb,0.39427e-02_rb,0.40844e-02_rb,0.42438e-02_rb /)
      kbo(:,17, 8) = (/ &
        & 0.31847e-02_rb,0.32833e-02_rb,0.33960e-02_rb,0.35252e-02_rb,0.36740e-02_rb /)
      kbo(:,18, 8) = (/ &
        & 0.27225e-02_rb,0.28155e-02_rb,0.29216e-02_rb,0.30449e-02_rb,0.31911e-02_rb /)
      kbo(:,19, 8) = (/ &
        & 0.23309e-02_rb,0.24127e-02_rb,0.25113e-02_rb,0.26325e-02_rb,0.27748e-02_rb /)
      kbo(:,20, 8) = (/ &
        & 0.19939e-02_rb,0.20703e-02_rb,0.21659e-02_rb,0.22816e-02_rb,0.24232e-02_rb /)
      kbo(:,21, 8) = (/ &
        & 0.17051e-02_rb,0.17809e-02_rb,0.18754e-02_rb,0.19919e-02_rb,0.21299e-02_rb /)
      kbo(:,22, 8) = (/ &
        & 0.14632e-02_rb,0.15412e-02_rb,0.16376e-02_rb,0.17554e-02_rb,0.18936e-02_rb /)
      kbo(:,23, 8) = (/ &
        & 0.12639e-02_rb,0.13422e-02_rb,0.14407e-02_rb,0.15575e-02_rb,0.16941e-02_rb /)
      kbo(:,24, 8) = (/ &
        & 0.10971e-02_rb,0.11767e-02_rb,0.12764e-02_rb,0.13939e-02_rb,0.15238e-02_rb /)
      kbo(:,25, 8) = (/ &
        & 0.95870e-03_rb,0.10413e-02_rb,0.11414e-02_rb,0.12556e-02_rb,0.13798e-02_rb /)
      kbo(:,26, 8) = (/ &
        & 0.84686e-03_rb,0.93021e-03_rb,0.10287e-02_rb,0.11374e-02_rb,0.12572e-02_rb /)
      kbo(:,27, 8) = (/ &
        & 0.75447e-03_rb,0.83724e-03_rb,0.93234e-03_rb,0.10369e-02_rb,0.11513e-02_rb /)
      kbo(:,28, 8) = (/ &
        & 0.67798e-03_rb,0.75937e-03_rb,0.85009e-03_rb,0.95072e-03_rb,0.10606e-02_rb /)
      kbo(:,29, 8) = (/ &
        & 0.61364e-03_rb,0.69262e-03_rb,0.77994e-03_rb,0.87674e-03_rb,0.98252e-03_rb /)
      kbo(:,30, 8) = (/ &
        & 0.55903e-03_rb,0.63520e-03_rb,0.71974e-03_rb,0.81322e-03_rb,0.91554e-03_rb /)
      kbo(:,31, 8) = (/ &
        & 0.51301e-03_rb,0.58698e-03_rb,0.66923e-03_rb,0.75972e-03_rb,0.85851e-03_rb /)
      kbo(:,32, 8) = (/ &
        & 0.47350e-03_rb,0.54617e-03_rb,0.62663e-03_rb,0.71470e-03_rb,0.81105e-03_rb /)
      kbo(:,33, 8) = (/ &
        & 0.44016e-03_rb,0.51136e-03_rb,0.58994e-03_rb,0.67638e-03_rb,0.77061e-03_rb /)
      kbo(:,34, 8) = (/ &
        & 0.40944e-03_rb,0.47900e-03_rb,0.55611e-03_rb,0.64143e-03_rb,0.73316e-03_rb /)
      kbo(:,35, 8) = (/ &
        & 0.37741e-03_rb,0.44487e-03_rb,0.52031e-03_rb,0.60323e-03_rb,0.69243e-03_rb /)
      kbo(:,36, 8) = (/ &
        & 0.34297e-03_rb,0.40766e-03_rb,0.48086e-03_rb,0.56132e-03_rb,0.64787e-03_rb /)
      kbo(:,37, 8) = (/ &
        & 0.30431e-03_rb,0.36499e-03_rb,0.43465e-03_rb,0.51167e-03_rb,0.59496e-03_rb /)
      kbo(:,38, 8) = (/ &
        & 0.26970e-03_rb,0.32665e-03_rb,0.39267e-03_rb,0.46706e-03_rb,0.54760e-03_rb /)
      kbo(:,39, 8) = (/ &
        & 0.23923e-03_rb,0.29254e-03_rb,0.35537e-03_rb,0.42706e-03_rb,0.50543e-03_rb /)
      kbo(:,40, 8) = (/ &
        & 0.20758e-03_rb,0.25688e-03_rb,0.31519e-03_rb,0.38349e-03_rb,0.45925e-03_rb /)
      kbo(:,41, 8) = (/ &
        & 0.17937e-03_rb,0.22435e-03_rb,0.27886e-03_rb,0.34337e-03_rb,0.41641e-03_rb /)
      kbo(:,42, 8) = (/ &
        & 0.15469e-03_rb,0.19589e-03_rb,0.24606e-03_rb,0.30674e-03_rb,0.37723e-03_rb /)
      kbo(:,43, 8) = (/ &
        & 0.13101e-03_rb,0.16796e-03_rb,0.21371e-03_rb,0.26992e-03_rb,0.33751e-03_rb /)
      kbo(:,44, 8) = (/ &
        & 0.10980e-03_rb,0.14260e-03_rb,0.18396e-03_rb,0.23557e-03_rb,0.29905e-03_rb /)
      kbo(:,45, 8) = (/ &
        & 0.91682e-04_rb,0.12045e-03_rb,0.15770e-03_rb,0.20472e-03_rb,0.26370e-03_rb /)
      kbo(:,46, 8) = (/ &
        & 0.75628e-04_rb,0.10059e-03_rb,0.13347e-03_rb,0.17595e-03_rb,0.22995e-03_rb /)
      kbo(:,47, 8) = (/ &
        & 0.61269e-04_rb,0.82597e-04_rb,0.11115e-03_rb,0.14863e-03_rb,0.19734e-03_rb /)
      kbo(:,48, 8) = (/ &
        & 0.49295e-04_rb,0.67304e-04_rb,0.91698e-04_rb,0.12475e-03_rb,0.16838e-03_rb /)
      kbo(:,49, 8) = (/ &
        & 0.39400e-04_rb,0.54424e-04_rb,0.75252e-04_rb,0.10388e-03_rb,0.14254e-03_rb /)
      kbo(:,50, 8) = (/ &
        & 0.31621e-04_rb,0.44130e-04_rb,0.61905e-04_rb,0.86583e-04_rb,0.12082e-03_rb /)
      kbo(:,51, 8) = (/ &
        & 0.25214e-04_rb,0.35613e-04_rb,0.50609e-04_rb,0.71998e-04_rb,0.10219e-03_rb /)
      kbo(:,52, 8) = (/ &
        & 0.20045e-04_rb,0.28626e-04_rb,0.41171e-04_rb,0.59458e-04_rb,0.85829e-04_rb /)
      kbo(:,53, 8) = (/ &
        & 0.15795e-04_rb,0.22859e-04_rb,0.33218e-04_rb,0.48810e-04_rb,0.71516e-04_rb /)
      kbo(:,54, 8) = (/ &
        & 0.12537e-04_rb,0.18343e-04_rb,0.26979e-04_rb,0.40183e-04_rb,0.59926e-04_rb /)
      kbo(:,55, 8) = (/ &
        & 0.99418e-05_rb,0.14675e-04_rb,0.21893e-04_rb,0.33017e-04_rb,0.50138e-04_rb /)
      kbo(:,56, 8) = (/ &
        & 0.78293e-05_rb,0.11672e-04_rb,0.17663e-04_rb,0.26975e-04_rb,0.41690e-04_rb /)
      kbo(:,57, 8) = (/ &
        & 0.61519e-05_rb,0.92383e-05_rb,0.14184e-04_rb,0.21901e-04_rb,0.34420e-04_rb /)
      kbo(:,58, 8) = (/ &
        & 0.48509e-05_rb,0.73119e-05_rb,0.11341e-04_rb,0.17783e-04_rb,0.28416e-04_rb /)
      kbo(:,59, 8) = (/ &
        & 0.41003e-05_rb,0.62461e-05_rb,0.98307e-05_rb,0.15678e-04_rb,0.25521e-04_rb /)
      kbo(:,13, 9) = (/ &
        & 0.31088e-01_rb,0.30978e-01_rb,0.31134e-01_rb,0.31580e-01_rb,0.32297e-01_rb /)
      kbo(:,14, 9) = (/ &
        & 0.26804e-01_rb,0.26913e-01_rb,0.27323e-01_rb,0.28019e-01_rb,0.28853e-01_rb /)
      kbo(:,15, 9) = (/ &
        & 0.23192e-01_rb,0.23530e-01_rb,0.24156e-01_rb,0.24918e-01_rb,0.25770e-01_rb /)
      kbo(:,16, 9) = (/ &
        & 0.20205e-01_rb,0.20738e-01_rb,0.21411e-01_rb,0.22189e-01_rb,0.23073e-01_rb /)
      kbo(:,17, 9) = (/ &
        & 0.17721e-01_rb,0.18309e-01_rb,0.19005e-01_rb,0.19816e-01_rb,0.20712e-01_rb /)
      kbo(:,18, 9) = (/ &
        & 0.15569e-01_rb,0.16175e-01_rb,0.16902e-01_rb,0.17717e-01_rb,0.18588e-01_rb /)
      kbo(:,19, 9) = (/ &
        & 0.13679e-01_rb,0.14314e-01_rb,0.15046e-01_rb,0.15840e-01_rb,0.16695e-01_rb /)
      kbo(:,20, 9) = (/ &
        & 0.12060e-01_rb,0.12704e-01_rb,0.13416e-01_rb,0.14184e-01_rb,0.14994e-01_rb /)
      kbo(:,21, 9) = (/ &
        & 0.10678e-01_rb,0.11296e-01_rb,0.11975e-01_rb,0.12694e-01_rb,0.13474e-01_rb /)
      kbo(:,22, 9) = (/ &
        & 0.95206e-02_rb,0.10114e-01_rb,0.10744e-01_rb,0.11429e-01_rb,0.12190e-01_rb /)
      kbo(:,23, 9) = (/ &
        & 0.85244e-02_rb,0.90705e-02_rb,0.96690e-02_rb,0.10334e-01_rb,0.11084e-01_rb /)
      kbo(:,24, 9) = (/ &
        & 0.76472e-02_rb,0.81682e-02_rb,0.87428e-02_rb,0.93948e-02_rb,0.10155e-01_rb /)
      kbo(:,25, 9) = (/ &
        & 0.68906e-02_rb,0.73887e-02_rb,0.79555e-02_rb,0.86116e-02_rb,0.93920e-02_rb /)
      kbo(:,26, 9) = (/ &
        & 0.62406e-02_rb,0.67330e-02_rb,0.73033e-02_rb,0.79807e-02_rb,0.87803e-02_rb /)
      kbo(:,27, 9) = (/ &
        & 0.56879e-02_rb,0.61818e-02_rb,0.67732e-02_rb,0.74721e-02_rb,0.82924e-02_rb /)
      kbo(:,28, 9) = (/ &
        & 0.52214e-02_rb,0.57290e-02_rb,0.63416e-02_rb,0.70614e-02_rb,0.79109e-02_rb /)
      kbo(:,29, 9) = (/ &
        & 0.48396e-02_rb,0.53718e-02_rb,0.60023e-02_rb,0.67493e-02_rb,0.76290e-02_rb /)
      kbo(:,30, 9) = (/ &
        & 0.45394e-02_rb,0.50868e-02_rb,0.57419e-02_rb,0.65213e-02_rb,0.74390e-02_rb /)
      kbo(:,31, 9) = (/ &
        & 0.42997e-02_rb,0.48721e-02_rb,0.55579e-02_rb,0.63736e-02_rb,0.73370e-02_rb /)
      kbo(:,32, 9) = (/ &
        & 0.41212e-02_rb,0.47209e-02_rb,0.54422e-02_rb,0.62987e-02_rb,0.73128e-02_rb /)
      kbo(:,33, 9) = (/ &
        & 0.39976e-02_rb,0.46295e-02_rb,0.53893e-02_rb,0.62949e-02_rb,0.73658e-02_rb /)
      kbo(:,34, 9) = (/ &
        & 0.39007e-02_rb,0.45672e-02_rb,0.53672e-02_rb,0.63167e-02_rb,0.74374e-02_rb /)
      kbo(:,35, 9) = (/ &
        & 0.37869e-02_rb,0.44770e-02_rb,0.53019e-02_rb,0.62849e-02_rb,0.74424e-02_rb /)
      kbo(:,36, 9) = (/ &
        & 0.36393e-02_rb,0.43376e-02_rb,0.51746e-02_rb,0.61726e-02_rb,0.73435e-02_rb /)
      kbo(:,37, 9) = (/ &
        & 0.34235e-02_rb,0.41098e-02_rb,0.49353e-02_rb,0.59226e-02_rb,0.70781e-02_rb /)
      kbo(:,38, 9) = (/ &
        & 0.32247e-02_rb,0.38981e-02_rb,0.47125e-02_rb,0.56849e-02_rb,0.68251e-02_rb /)
      kbo(:,39, 9) = (/ &
        & 0.30456e-02_rb,0.37107e-02_rb,0.45116e-02_rb,0.54683e-02_rb,0.65906e-02_rb /)
      kbo(:,40, 9) = (/ &
        & 0.28232e-02_rb,0.34654e-02_rb,0.42389e-02_rb,0.51610e-02_rb,0.62484e-02_rb /)
      kbo(:,41, 9) = (/ &
        & 0.26119e-02_rb,0.32290e-02_rb,0.39730e-02_rb,0.48613e-02_rb,0.59121e-02_rb /)
      kbo(:,42, 9) = (/ &
        & 0.24180e-02_rb,0.30088e-02_rb,0.37246e-02_rb,0.45813e-02_rb,0.55929e-02_rb /)
      kbo(:,43, 9) = (/ &
        & 0.22093e-02_rb,0.27691e-02_rb,0.34494e-02_rb,0.42630e-02_rb,0.52317e-02_rb /)
      kbo(:,44, 9) = (/ &
        & 0.20033e-02_rb,0.25304e-02_rb,0.31741e-02_rb,0.39439e-02_rb,0.48637e-02_rb /)
      kbo(:,45, 9) = (/ &
        & 0.18130e-02_rb,0.23086e-02_rb,0.29181e-02_rb,0.36487e-02_rb,0.45175e-02_rb /)
      kbo(:,46, 9) = (/ &
        & 0.16283e-02_rb,0.20910e-02_rb,0.26646e-02_rb,0.33526e-02_rb,0.41695e-02_rb /)
      kbo(:,47, 9) = (/ &
        & 0.14435e-02_rb,0.18702e-02_rb,0.24020e-02_rb,0.30439e-02_rb,0.38103e-02_rb /)
      kbo(:,48, 9) = (/ &
        & 0.12773e-02_rb,0.16671e-02_rb,0.21576e-02_rb,0.27587e-02_rb,0.34731e-02_rb /)
      kbo(:,49, 9) = (/ &
        & 0.11238e-02_rb,0.14806e-02_rb,0.19320e-02_rb,0.24920e-02_rb,0.31602e-02_rb /)
      kbo(:,50, 9) = (/ &
        & 0.99065e-03_rb,0.13211e-02_rb,0.17354e-02_rb,0.22582e-02_rb,0.28857e-02_rb /)
      kbo(:,51, 9) = (/ &
        & 0.87220e-03_rb,0.11790e-02_rb,0.15607e-02_rb,0.20453e-02_rb,0.26367e-02_rb /)
      kbo(:,52, 9) = (/ &
        & 0.76443e-03_rb,0.10482e-02_rb,0.13992e-02_rb,0.18469e-02_rb,0.24023e-02_rb /)
      kbo(:,53, 9) = (/ &
        & 0.66506e-03_rb,0.92645e-03_rb,0.12548e-02_rb,0.16624e-02_rb,0.21789e-02_rb /)
      kbo(:,54, 9) = (/ &
        & 0.58147e-03_rb,0.82449e-03_rb,0.11302e-02_rb,0.15093e-02_rb,0.19886e-02_rb /)
      kbo(:,55, 9) = (/ &
        & 0.50762e-03_rb,0.73235e-03_rb,0.10182e-02_rb,0.13732e-02_rb,0.18200e-02_rb /)
      kbo(:,56, 9) = (/ &
        & 0.44063e-03_rb,0.64710e-03_rb,0.91345e-03_rb,0.12471e-02_rb,0.16632e-02_rb /)
      kbo(:,57, 9) = (/ &
        & 0.37891e-03_rb,0.56859e-03_rb,0.81680e-03_rb,0.11290e-02_rb,0.15165e-02_rb /)
      kbo(:,58, 9) = (/ &
        & 0.32563e-03_rb,0.50022e-03_rb,0.73037e-03_rb,0.10236e-02_rb,0.13873e-02_rb /)
      kbo(:,59, 9) = (/ &
        & 0.30307e-03_rb,0.47208e-03_rb,0.69548e-03_rb,0.98198e-03_rb,0.13371e-02_rb /)
      kbo(:,13,10) = (/ &
        & 0.13405e+00_rb,0.13208e+00_rb,0.13012e+00_rb,0.12819e+00_rb,0.12637e+00_rb /)
      kbo(:,14,10) = (/ &
        & 0.11726e+00_rb,0.11549e+00_rb,0.11376e+00_rb,0.11219e+00_rb,0.11171e+00_rb /)
      kbo(:,15,10) = (/ &
        & 0.10125e+00_rb,0.99749e-01_rb,0.98394e-01_rb,0.98385e-01_rb,0.99981e-01_rb /)
      kbo(:,16,10) = (/ &
        & 0.86553e-01_rb,0.85383e-01_rb,0.85602e-01_rb,0.87440e-01_rb,0.89929e-01_rb /)
      kbo(:,17,10) = (/ &
        & 0.73963e-01_rb,0.74028e-01_rb,0.75744e-01_rb,0.77969e-01_rb,0.80306e-01_rb /)
      kbo(:,18,10) = (/ &
        & 0.64249e-01_rb,0.65620e-01_rb,0.67504e-01_rb,0.69584e-01_rb,0.72235e-01_rb /)
      kbo(:,19,10) = (/ &
        & 0.56934e-01_rb,0.58613e-01_rb,0.60462e-01_rb,0.62855e-01_rb,0.65899e-01_rb /)
      kbo(:,20,10) = (/ &
        & 0.50721e-01_rb,0.52418e-01_rb,0.54615e-01_rb,0.57565e-01_rb,0.61049e-01_rb /)
      kbo(:,21,10) = (/ &
        & 0.45027e-01_rb,0.47161e-01_rb,0.49911e-01_rb,0.53249e-01_rb,0.56950e-01_rb /)
      kbo(:,22,10) = (/ &
        & 0.40301e-01_rb,0.42891e-01_rb,0.46164e-01_rb,0.49801e-01_rb,0.53839e-01_rb /)
      kbo(:,23,10) = (/ &
        & 0.36438e-01_rb,0.39487e-01_rb,0.43011e-01_rb,0.46965e-01_rb,0.50969e-01_rb /)
      kbo(:,24,10) = (/ &
        & 0.33455e-01_rb,0.36675e-01_rb,0.40451e-01_rb,0.44369e-01_rb,0.48399e-01_rb /)
      kbo(:,25,10) = (/ &
        & 0.31038e-01_rb,0.34461e-01_rb,0.38146e-01_rb,0.42075e-01_rb,0.46138e-01_rb /)
      kbo(:,26,10) = (/ &
        & 0.29184e-01_rb,0.32533e-01_rb,0.36194e-01_rb,0.40134e-01_rb,0.44371e-01_rb /)
      kbo(:,27,10) = (/ &
        & 0.27607e-01_rb,0.30907e-01_rb,0.34540e-01_rb,0.38584e-01_rb,0.43101e-01_rb /)
      kbo(:,28,10) = (/ &
        & 0.26262e-01_rb,0.29536e-01_rb,0.33259e-01_rb,0.37529e-01_rb,0.42311e-01_rb /)
      kbo(:,29,10) = (/ &
        & 0.25199e-01_rb,0.28472e-01_rb,0.32405e-01_rb,0.36927e-01_rb,0.42052e-01_rb /)
      kbo(:,30,10) = (/ &
        & 0.24317e-01_rb,0.27783e-01_rb,0.31948e-01_rb,0.36772e-01_rb,0.42176e-01_rb /)
      kbo(:,31,10) = (/ &
        & 0.23777e-01_rb,0.27457e-01_rb,0.31879e-01_rb,0.37001e-01_rb,0.42663e-01_rb /)
      kbo(:,32,10) = (/ &
        & 0.23540e-01_rb,0.27481e-01_rb,0.32177e-01_rb,0.37596e-01_rb,0.43567e-01_rb /)
      kbo(:,33,10) = (/ &
        & 0.23597e-01_rb,0.27797e-01_rb,0.32792e-01_rb,0.38465e-01_rb,0.44641e-01_rb /)
      kbo(:,34,10) = (/ &
        & 0.23769e-01_rb,0.28231e-01_rb,0.33430e-01_rb,0.39373e-01_rb,0.45740e-01_rb /)
      kbo(:,35,10) = (/ &
        & 0.23751e-01_rb,0.28376e-01_rb,0.33764e-01_rb,0.39814e-01_rb,0.46255e-01_rb /)
      kbo(:,36,10) = (/ &
        & 0.23409e-01_rb,0.28117e-01_rb,0.33564e-01_rb,0.39651e-01_rb,0.46138e-01_rb /)
      kbo(:,37,10) = (/ &
        & 0.22508e-01_rb,0.27169e-01_rb,0.32556e-01_rb,0.38592e-01_rb,0.45071e-01_rb /)
      kbo(:,38,10) = (/ &
        & 0.21655e-01_rb,0.26262e-01_rb,0.31543e-01_rb,0.37540e-01_rb,0.43984e-01_rb /)
      kbo(:,39,10) = (/ &
        & 0.20895e-01_rb,0.25391e-01_rb,0.30600e-01_rb,0.36519e-01_rb,0.42890e-01_rb /)
      kbo(:,40,10) = (/ &
        & 0.19794e-01_rb,0.24108e-01_rb,0.29157e-01_rb,0.34891e-01_rb,0.41215e-01_rb /)
      kbo(:,41,10) = (/ &
        & 0.18688e-01_rb,0.22822e-01_rb,0.27713e-01_rb,0.33302e-01_rb,0.39460e-01_rb /)
      kbo(:,42,10) = (/ &
        & 0.17627e-01_rb,0.21623e-01_rb,0.26302e-01_rb,0.31747e-01_rb,0.37731e-01_rb /)
      kbo(:,43,10) = (/ &
        & 0.16446e-01_rb,0.20270e-01_rb,0.24738e-01_rb,0.29940e-01_rb,0.35800e-01_rb /)
      kbo(:,44,10) = (/ &
        & 0.15234e-01_rb,0.18876e-01_rb,0.23100e-01_rb,0.28072e-01_rb,0.33725e-01_rb /)
      kbo(:,45,10) = (/ &
        & 0.14095e-01_rb,0.17542e-01_rb,0.21553e-01_rb,0.26276e-01_rb,0.31735e-01_rb /)
      kbo(:,46,10) = (/ &
        & 0.12980e-01_rb,0.16215e-01_rb,0.20030e-01_rb,0.24474e-01_rb,0.29695e-01_rb /)
      kbo(:,47,10) = (/ &
        & 0.11800e-01_rb,0.14839e-01_rb,0.18423e-01_rb,0.22582e-01_rb,0.27493e-01_rb /)
      kbo(:,48,10) = (/ &
        & 0.10669e-01_rb,0.13547e-01_rb,0.16901e-01_rb,0.20822e-01_rb,0.25431e-01_rb /)
      kbo(:,49,10) = (/ &
        & 0.96515e-02_rb,0.12347e-01_rb,0.15476e-01_rb,0.19181e-01_rb,0.23474e-01_rb /)
      kbo(:,50,10) = (/ &
        & 0.87302e-02_rb,0.11282e-01_rb,0.14230e-01_rb,0.17719e-01_rb,0.21773e-01_rb /)
      kbo(:,51,10) = (/ &
        & 0.78879e-02_rb,0.10297e-01_rb,0.13106e-01_rb,0.16387e-01_rb,0.20218e-01_rb /)
      kbo(:,52,10) = (/ &
        & 0.71141e-02_rb,0.93631e-02_rb,0.12059e-01_rb,0.15140e-01_rb,0.18761e-01_rb /)
      kbo(:,53,10) = (/ &
        & 0.64033e-02_rb,0.84706e-02_rb,0.11019e-01_rb,0.13962e-01_rb,0.17391e-01_rb /)
      kbo(:,54,10) = (/ &
        & 0.58003e-02_rb,0.77325e-02_rb,0.10121e-01_rb,0.12913e-01_rb,0.16185e-01_rb /)
      kbo(:,55,10) = (/ &
        & 0.52567e-02_rb,0.70672e-02_rb,0.93006e-02_rb,0.11949e-01_rb,0.15064e-01_rb /)
      kbo(:,56,10) = (/ &
        & 0.47304e-02_rb,0.64466e-02_rb,0.85460e-02_rb,0.11037e-01_rb,0.13988e-01_rb /)
      kbo(:,57,10) = (/ &
        & 0.42280e-02_rb,0.58745e-02_rb,0.78453e-02_rb,0.10192e-01_rb,0.12992e-01_rb /)
      kbo(:,58,10) = (/ &
        & 0.37744e-02_rb,0.53584e-02_rb,0.72089e-02_rb,0.94309e-02_rb,0.12101e-01_rb /)
      kbo(:,59,10) = (/ &
        & 0.36027e-02_rb,0.51588e-02_rb,0.69694e-02_rb,0.91382e-02_rb,0.11759e-01_rb /)
      kbo(:,13,11) = (/ &
        & 0.24636e+00_rb,0.24379e+00_rb,0.24122e+00_rb,0.23863e+00_rb,0.23598e+00_rb /)
      kbo(:,14,11) = (/ &
        & 0.22272e+00_rb,0.22032e+00_rb,0.21794e+00_rb,0.21559e+00_rb,0.21334e+00_rb /)
      kbo(:,15,11) = (/ &
        & 0.19844e+00_rb,0.19619e+00_rb,0.19411e+00_rb,0.19222e+00_rb,0.19047e+00_rb /)
      kbo(:,16,11) = (/ &
        & 0.17439e+00_rb,0.17254e+00_rb,0.17092e+00_rb,0.16946e+00_rb,0.16916e+00_rb /)
      kbo(:,17,11) = (/ &
        & 0.15164e+00_rb,0.15027e+00_rb,0.14912e+00_rb,0.14939e+00_rb,0.15176e+00_rb /)
      kbo(:,18,11) = (/ &
        & 0.13063e+00_rb,0.12971e+00_rb,0.13029e+00_rb,0.13302e+00_rb,0.13748e+00_rb /)
      kbo(:,19,11) = (/ &
        & 0.11203e+00_rb,0.11247e+00_rb,0.11508e+00_rb,0.11959e+00_rb,0.12521e+00_rb /)
      kbo(:,20,11) = (/ &
        & 0.96916e-01_rb,0.99041e-01_rb,0.10318e+00_rb,0.10837e+00_rb,0.11414e+00_rb /)
      kbo(:,21,11) = (/ &
        & 0.85382e-01_rb,0.88824e-01_rb,0.93367e-01_rb,0.98603e-01_rb,0.10483e+00_rb /)
      kbo(:,22,11) = (/ &
        & 0.76943e-01_rb,0.80844e-01_rb,0.85377e-01_rb,0.90964e-01_rb,0.97503e-01_rb /)
      kbo(:,23,11) = (/ &
        & 0.70041e-01_rb,0.74136e-01_rb,0.79065e-01_rb,0.84905e-01_rb,0.92247e-01_rb /)
      kbo(:,24,11) = (/ &
        & 0.64133e-01_rb,0.68708e-01_rb,0.74135e-01_rb,0.80762e-01_rb,0.88658e-01_rb /)
      kbo(:,25,11) = (/ &
        & 0.59352e-01_rb,0.64474e-01_rb,0.70718e-01_rb,0.78064e-01_rb,0.86394e-01_rb /)
      kbo(:,26,11) = (/ &
        & 0.55652e-01_rb,0.61613e-01_rb,0.68607e-01_rb,0.76451e-01_rb,0.85230e-01_rb /)
      kbo(:,27,11) = (/ &
        & 0.53058e-01_rb,0.59792e-01_rb,0.67298e-01_rb,0.75623e-01_rb,0.84830e-01_rb /)
      kbo(:,28,11) = (/ &
        & 0.51454e-01_rb,0.58747e-01_rb,0.66709e-01_rb,0.75478e-01_rb,0.85111e-01_rb /)
      kbo(:,29,11) = (/ &
        & 0.50601e-01_rb,0.58412e-01_rb,0.66813e-01_rb,0.76014e-01_rb,0.86051e-01_rb /)
      kbo(:,30,11) = (/ &
        & 0.50412e-01_rb,0.58677e-01_rb,0.67503e-01_rb,0.77079e-01_rb,0.87490e-01_rb /)
      kbo(:,31,11) = (/ &
        & 0.50833e-01_rb,0.59491e-01_rb,0.68713e-01_rb,0.78659e-01_rb,0.89405e-01_rb /)
      kbo(:,32,11) = (/ &
        & 0.51770e-01_rb,0.60792e-01_rb,0.70497e-01_rb,0.80769e-01_rb,0.91778e-01_rb /)
      kbo(:,33,11) = (/ &
        & 0.53128e-01_rb,0.62590e-01_rb,0.72506e-01_rb,0.83033e-01_rb,0.94354e-01_rb /)
      kbo(:,34,11) = (/ &
        & 0.54518e-01_rb,0.64238e-01_rb,0.74390e-01_rb,0.85011e-01_rb,0.96530e-01_rb /)
      kbo(:,35,11) = (/ &
        & 0.55323e-01_rb,0.65129e-01_rb,0.75324e-01_rb,0.86085e-01_rb,0.97742e-01_rb /)
      kbo(:,36,11) = (/ &
        & 0.55103e-01_rb,0.64989e-01_rb,0.75282e-01_rb,0.86060e-01_rb,0.97722e-01_rb /)
      kbo(:,37,11) = (/ &
        & 0.53513e-01_rb,0.63398e-01_rb,0.73617e-01_rb,0.84326e-01_rb,0.95934e-01_rb /)
      kbo(:,38,11) = (/ &
        & 0.51839e-01_rb,0.61729e-01_rb,0.71975e-01_rb,0.82585e-01_rb,0.94078e-01_rb /)
      kbo(:,39,11) = (/ &
        & 0.50239e-01_rb,0.60161e-01_rb,0.70354e-01_rb,0.80923e-01_rb,0.92345e-01_rb /)
      kbo(:,40,11) = (/ &
        & 0.47749e-01_rb,0.57519e-01_rb,0.67663e-01_rb,0.78238e-01_rb,0.89341e-01_rb /)
      kbo(:,41,11) = (/ &
        & 0.45220e-01_rb,0.54835e-01_rb,0.64881e-01_rb,0.75369e-01_rb,0.86308e-01_rb /)
      kbo(:,42,11) = (/ &
        & 0.42768e-01_rb,0.52225e-01_rb,0.62107e-01_rb,0.72467e-01_rb,0.83297e-01_rb /)
      kbo(:,43,11) = (/ &
        & 0.39884e-01_rb,0.49090e-01_rb,0.58932e-01_rb,0.69072e-01_rb,0.79690e-01_rb /)
      kbo(:,44,11) = (/ &
        & 0.36935e-01_rb,0.45822e-01_rb,0.55523e-01_rb,0.65525e-01_rb,0.76019e-01_rb /)
      kbo(:,45,11) = (/ &
        & 0.34132e-01_rb,0.42684e-01_rb,0.52151e-01_rb,0.62095e-01_rb,0.72392e-01_rb /)
      kbo(:,46,11) = (/ &
        & 0.31296e-01_rb,0.39456e-01_rb,0.48596e-01_rb,0.58446e-01_rb,0.68612e-01_rb /)
      kbo(:,47,11) = (/ &
        & 0.28352e-01_rb,0.35994e-01_rb,0.44764e-01_rb,0.54432e-01_rb,0.64443e-01_rb /)
      kbo(:,48,11) = (/ &
        & 0.25659e-01_rb,0.32738e-01_rb,0.41113e-01_rb,0.50437e-01_rb,0.60364e-01_rb /)
      kbo(:,49,11) = (/ &
        & 0.23119e-01_rb,0.29700e-01_rb,0.37617e-01_rb,0.46554e-01_rb,0.56334e-01_rb /)
      kbo(:,50,11) = (/ &
        & 0.20913e-01_rb,0.27063e-01_rb,0.34504e-01_rb,0.43055e-01_rb,0.52582e-01_rb /)
      kbo(:,51,11) = (/ &
        & 0.18909e-01_rb,0.24701e-01_rb,0.31626e-01_rb,0.39797e-01_rb,0.48997e-01_rb /)
      kbo(:,52,11) = (/ &
        & 0.17085e-01_rb,0.22478e-01_rb,0.28924e-01_rb,0.36689e-01_rb,0.45522e-01_rb /)
      kbo(:,53,11) = (/ &
        & 0.15363e-01_rb,0.20405e-01_rb,0.26432e-01_rb,0.33725e-01_rb,0.42180e-01_rb /)
      kbo(:,54,11) = (/ &
        & 0.13901e-01_rb,0.18604e-01_rb,0.24309e-01_rb,0.31149e-01_rb,0.39236e-01_rb /)
      kbo(:,55,11) = (/ &
        & 0.12627e-01_rb,0.16988e-01_rb,0.22381e-01_rb,0.28834e-01_rb,0.36535e-01_rb /)
      kbo(:,56,11) = (/ &
        & 0.11410e-01_rb,0.15484e-01_rb,0.20536e-01_rb,0.26668e-01_rb,0.33955e-01_rb /)
      kbo(:,57,11) = (/ &
        & 0.10296e-01_rb,0.14074e-01_rb,0.18805e-01_rb,0.24610e-01_rb,0.31477e-01_rb /)
      kbo(:,58,11) = (/ &
        & 0.93246e-02_rb,0.12835e-01_rb,0.17266e-01_rb,0.22752e-01_rb,0.29238e-01_rb /)
      kbo(:,59,11) = (/ &
        & 0.89562e-02_rb,0.12359e-01_rb,0.16676e-01_rb,0.22030e-01_rb,0.28380e-01_rb /)
      kbo(:,13,12) = (/ &
        & 0.46920e+00_rb,0.46679e+00_rb,0.46431e+00_rb,0.46162e+00_rb,0.45841e+00_rb /)
      kbo(:,14,12) = (/ &
        & 0.44978e+00_rb,0.44747e+00_rb,0.44532e+00_rb,0.44273e+00_rb,0.44032e+00_rb /)
      kbo(:,15,12) = (/ &
        & 0.42273e+00_rb,0.42112e+00_rb,0.41952e+00_rb,0.41793e+00_rb,0.41621e+00_rb /)
      kbo(:,16,12) = (/ &
        & 0.39050e+00_rb,0.38973e+00_rb,0.38909e+00_rb,0.38861e+00_rb,0.38845e+00_rb /)
      kbo(:,17,12) = (/ &
        & 0.35567e+00_rb,0.35584e+00_rb,0.35648e+00_rb,0.35743e+00_rb,0.35846e+00_rb /)
      kbo(:,18,12) = (/ &
        & 0.32023e+00_rb,0.32164e+00_rb,0.32356e+00_rb,0.32582e+00_rb,0.32854e+00_rb /)
      kbo(:,19,12) = (/ &
        & 0.28599e+00_rb,0.28851e+00_rb,0.29173e+00_rb,0.29552e+00_rb,0.30048e+00_rb /)
      kbo(:,20,12) = (/ &
        & 0.25381e+00_rb,0.25750e+00_rb,0.26208e+00_rb,0.26822e+00_rb,0.27683e+00_rb /)
      kbo(:,21,12) = (/ &
        & 0.22440e+00_rb,0.22938e+00_rb,0.23617e+00_rb,0.24559e+00_rb,0.25713e+00_rb /)
      kbo(:,22,12) = (/ &
        & 0.19852e+00_rb,0.20554e+00_rb,0.21550e+00_rb,0.22788e+00_rb,0.24207e+00_rb /)
      kbo(:,23,12) = (/ &
        & 0.17693e+00_rb,0.18660e+00_rb,0.19922e+00_rb,0.21410e+00_rb,0.23064e+00_rb /)
      kbo(:,24,12) = (/ &
        & 0.16009e+00_rb,0.17209e+00_rb,0.18673e+00_rb,0.20389e+00_rb,0.22257e+00_rb /)
      kbo(:,25,12) = (/ &
        & 0.14745e+00_rb,0.16127e+00_rb,0.17793e+00_rb,0.19685e+00_rb,0.21752e+00_rb /)
      kbo(:,26,12) = (/ &
        & 0.13842e+00_rb,0.15401e+00_rb,0.17237e+00_rb,0.19313e+00_rb,0.21550e+00_rb /)
      kbo(:,27,12) = (/ &
        & 0.13240e+00_rb,0.14962e+00_rb,0.16961e+00_rb,0.19199e+00_rb,0.21568e+00_rb /)
      kbo(:,28,12) = (/ &
        & 0.12889e+00_rb,0.14761e+00_rb,0.16917e+00_rb,0.19286e+00_rb,0.21743e+00_rb /)
      kbo(:,29,12) = (/ &
        & 0.12765e+00_rb,0.14781e+00_rb,0.17068e+00_rb,0.19527e+00_rb,0.22046e+00_rb /)
      kbo(:,30,12) = (/ &
        & 0.12821e+00_rb,0.14964e+00_rb,0.17352e+00_rb,0.19873e+00_rb,0.22436e+00_rb /)
      kbo(:,31,12) = (/ &
        & 0.13031e+00_rb,0.15282e+00_rb,0.17742e+00_rb,0.20308e+00_rb,0.22902e+00_rb /)
      kbo(:,32,12) = (/ &
        & 0.13354e+00_rb,0.15691e+00_rb,0.18189e+00_rb,0.20791e+00_rb,0.23394e+00_rb /)
      kbo(:,33,12) = (/ &
        & 0.13759e+00_rb,0.16158e+00_rb,0.18710e+00_rb,0.21336e+00_rb,0.23934e+00_rb /)
      kbo(:,34,12) = (/ &
        & 0.14143e+00_rb,0.16592e+00_rb,0.19181e+00_rb,0.21830e+00_rb,0.24425e+00_rb /)
      kbo(:,35,12) = (/ &
        & 0.14328e+00_rb,0.16813e+00_rb,0.19433e+00_rb,0.22092e+00_rb,0.24688e+00_rb /)
      kbo(:,36,12) = (/ &
        & 0.14288e+00_rb,0.16783e+00_rb,0.19405e+00_rb,0.22076e+00_rb,0.24684e+00_rb /)
      kbo(:,37,12) = (/ &
        & 0.13900e+00_rb,0.16377e+00_rb,0.19000e+00_rb,0.21677e+00_rb,0.24302e+00_rb /)
      kbo(:,38,12) = (/ &
        & 0.13517e+00_rb,0.15969e+00_rb,0.18575e+00_rb,0.21259e+00_rb,0.23902e+00_rb /)
      kbo(:,39,12) = (/ &
        & 0.13148e+00_rb,0.15568e+00_rb,0.18162e+00_rb,0.20849e+00_rb,0.23507e+00_rb /)
      kbo(:,40,12) = (/ &
        & 0.12553e+00_rb,0.14932e+00_rb,0.17494e+00_rb,0.20171e+00_rb,0.22853e+00_rb /)
      kbo(:,41,12) = (/ &
        & 0.11952e+00_rb,0.14280e+00_rb,0.16803e+00_rb,0.19459e+00_rb,0.22156e+00_rb /)
      kbo(:,42,12) = (/ &
        & 0.11373e+00_rb,0.13636e+00_rb,0.16129e+00_rb,0.18763e+00_rb,0.21464e+00_rb /)
      kbo(:,43,12) = (/ &
        & 0.10701e+00_rb,0.12880e+00_rb,0.15312e+00_rb,0.17916e+00_rb,0.20617e+00_rb /)
      kbo(:,44,12) = (/ &
        & 0.10010e+00_rb,0.12097e+00_rb,0.14449e+00_rb,0.17006e+00_rb,0.19689e+00_rb /)
      kbo(:,45,12) = (/ &
        & 0.93559e-01_rb,0.11347e+00_rb,0.13614e+00_rb,0.16117e+00_rb,0.18764e+00_rb /)
      kbo(:,46,12) = (/ &
        & 0.86985e-01_rb,0.10587e+00_rb,0.12759e+00_rb,0.15183e+00_rb,0.17790e+00_rb /)
      kbo(:,47,12) = (/ &
        & 0.79918e-01_rb,0.97829e-01_rb,0.11842e+00_rb,0.14172e+00_rb,0.16718e+00_rb /)
      kbo(:,48,12) = (/ &
        & 0.73127e-01_rb,0.90284e-01_rb,0.10972e+00_rb,0.13198e+00_rb,0.15666e+00_rb /)
      kbo(:,49,12) = (/ &
        & 0.66713e-01_rb,0.83137e-01_rb,0.10149e+00_rb,0.12264e+00_rb,0.14642e+00_rb /)
      kbo(:,50,12) = (/ &
        & 0.61017e-01_rb,0.76653e-01_rb,0.94210e-01_rb,0.11429e+00_rb,0.13713e+00_rb /)
      kbo(:,51,12) = (/ &
        & 0.55806e-01_rb,0.70628e-01_rb,0.87552e-01_rb,0.10658e+00_rb,0.12844e+00_rb /)
      kbo(:,52,12) = (/ &
        & 0.50954e-01_rb,0.64939e-01_rb,0.81159e-01_rb,0.99264e-01_rb,0.12011e+00_rb /)
      kbo(:,53,12) = (/ &
        & 0.46354e-01_rb,0.59512e-01_rb,0.74951e-01_rb,0.92338e-01_rb,0.11214e+00_rb /)
      kbo(:,54,12) = (/ &
        & 0.42433e-01_rb,0.54851e-01_rb,0.69504e-01_rb,0.86317e-01_rb,0.10517e+00_rb /)
      kbo(:,55,12) = (/ &
        & 0.38912e-01_rb,0.50588e-01_rb,0.64547e-01_rb,0.80719e-01_rb,0.98779e-01_rb /)
      kbo(:,56,12) = (/ &
        & 0.35756e-01_rb,0.46572e-01_rb,0.59794e-01_rb,0.75289e-01_rb,0.92719e-01_rb /)
      kbo(:,57,12) = (/ &
        & 0.32896e-01_rb,0.42811e-01_rb,0.55328e-01_rb,0.70070e-01_rb,0.86950e-01_rb /)
      kbo(:,58,12) = (/ &
        & 0.30447e-01_rb,0.39485e-01_rb,0.51269e-01_rb,0.65351e-01_rb,0.81643e-01_rb /)
      kbo(:,59,12) = (/ &
        & 0.29531e-01_rb,0.38222e-01_rb,0.49696e-01_rb,0.63514e-01_rb,0.79546e-01_rb /)
      kbo(:,13,13) = (/ &
        & 0.87498e+00_rb,0.87414e+00_rb,0.87195e+00_rb,0.86869e+00_rb,0.86437e+00_rb /)
      kbo(:,14,13) = (/ &
        & 0.90060e+00_rb,0.89974e+00_rb,0.89734e+00_rb,0.89423e+00_rb,0.88971e+00_rb /)
      kbo(:,15,13) = (/ &
        & 0.91305e+00_rb,0.91211e+00_rb,0.91072e+00_rb,0.90792e+00_rb,0.90457e+00_rb /)
      kbo(:,16,13) = (/ &
        & 0.91218e+00_rb,0.91265e+00_rb,0.91216e+00_rb,0.91150e+00_rb,0.90908e+00_rb /)
      kbo(:,17,13) = (/ &
        & 0.89877e+00_rb,0.90187e+00_rb,0.90414e+00_rb,0.90531e+00_rb,0.90588e+00_rb /)
      kbo(:,18,13) = (/ &
        & 0.87718e+00_rb,0.88256e+00_rb,0.88815e+00_rb,0.89271e+00_rb,0.89636e+00_rb /)
      kbo(:,19,13) = (/ &
        & 0.84784e+00_rb,0.85815e+00_rb,0.86718e+00_rb,0.87552e+00_rb,0.88267e+00_rb /)
      kbo(:,20,13) = (/ &
        & 0.81552e+00_rb,0.83012e+00_rb,0.84379e+00_rb,0.85649e+00_rb,0.86736e+00_rb /)
      kbo(:,21,13) = (/ &
        & 0.78216e+00_rb,0.80141e+00_rb,0.82018e+00_rb,0.83708e+00_rb,0.85197e+00_rb /)
      kbo(:,22,13) = (/ &
        & 0.75119e+00_rb,0.77590e+00_rb,0.79904e+00_rb,0.81994e+00_rb,0.83870e+00_rb /)
      kbo(:,23,13) = (/ &
        & 0.72390e+00_rb,0.75373e+00_rb,0.78079e+00_rb,0.80579e+00_rb,0.82760e+00_rb /)
      kbo(:,24,13) = (/ &
        & 0.70145e+00_rb,0.73523e+00_rb,0.76640e+00_rb,0.79456e+00_rb,0.81899e+00_rb /)
      kbo(:,25,13) = (/ &
        & 0.68406e+00_rb,0.72159e+00_rb,0.75604e+00_rb,0.78669e+00_rb,0.81321e+00_rb /)
      kbo(:,26,13) = (/ &
        & 0.67200e+00_rb,0.71253e+00_rb,0.74968e+00_rb,0.78214e+00_rb,0.81015e+00_rb /)
      kbo(:,27,13) = (/ &
        & 0.66447e+00_rb,0.70740e+00_rb,0.74639e+00_rb,0.77996e+00_rb,0.80903e+00_rb /)
      kbo(:,28,13) = (/ &
        & 0.66095e+00_rb,0.70586e+00_rb,0.74565e+00_rb,0.77995e+00_rb,0.80973e+00_rb /)
      kbo(:,29,13) = (/ &
        & 0.66138e+00_rb,0.70727e+00_rb,0.74725e+00_rb,0.78204e+00_rb,0.81205e+00_rb /)
      kbo(:,30,13) = (/ &
        & 0.66437e+00_rb,0.71055e+00_rb,0.75066e+00_rb,0.78572e+00_rb,0.81538e+00_rb /)
      kbo(:,31,13) = (/ &
        & 0.66969e+00_rb,0.71558e+00_rb,0.75561e+00_rb,0.79049e+00_rb,0.81954e+00_rb /)
      kbo(:,32,13) = (/ &
        & 0.67664e+00_rb,0.72206e+00_rb,0.76182e+00_rb,0.79609e+00_rb,0.82431e+00_rb /)
      kbo(:,33,13) = (/ &
        & 0.68486e+00_rb,0.72960e+00_rb,0.76872e+00_rb,0.80214e+00_rb,0.82949e+00_rb /)
      kbo(:,34,13) = (/ &
        & 0.69234e+00_rb,0.73624e+00_rb,0.77485e+00_rb,0.80729e+00_rb,0.83397e+00_rb /)
      kbo(:,35,13) = (/ &
        & 0.69580e+00_rb,0.73958e+00_rb,0.77791e+00_rb,0.80990e+00_rb,0.83618e+00_rb /)
      kbo(:,36,13) = (/ &
        & 0.69461e+00_rb,0.73878e+00_rb,0.77738e+00_rb,0.80959e+00_rb,0.83608e+00_rb /)
      kbo(:,37,13) = (/ &
        & 0.68652e+00_rb,0.73194e+00_rb,0.77165e+00_rb,0.80500e+00_rb,0.83229e+00_rb /)
      kbo(:,38,13) = (/ &
        & 0.67804e+00_rb,0.72484e+00_rb,0.76560e+00_rb,0.80009e+00_rb,0.82834e+00_rb /)
      kbo(:,39,13) = (/ &
        & 0.66970e+00_rb,0.71778e+00_rb,0.75963e+00_rb,0.79504e+00_rb,0.82429e+00_rb /)
      kbo(:,40,13) = (/ &
        & 0.65605e+00_rb,0.70590e+00_rb,0.74946e+00_rb,0.78658e+00_rb,0.81742e+00_rb /)
      kbo(:,41,13) = (/ &
        & 0.64137e+00_rb,0.69306e+00_rb,0.73837e+00_rb,0.77717e+00_rb,0.80985e+00_rb /)
      kbo(:,42,13) = (/ &
        & 0.62638e+00_rb,0.67987e+00_rb,0.72699e+00_rb,0.76756e+00_rb,0.80186e+00_rb /)
      kbo(:,43,13) = (/ &
        & 0.60761e+00_rb,0.66320e+00_rb,0.71239e+00_rb,0.75526e+00_rb,0.79130e+00_rb /)
      kbo(:,44,13) = (/ &
        & 0.58682e+00_rb,0.64468e+00_rb,0.69616e+00_rb,0.74131e+00_rb,0.77948e+00_rb /)
      kbo(:,45,13) = (/ &
        & 0.56554e+00_rb,0.62545e+00_rb,0.67917e+00_rb,0.72630e+00_rb,0.76697e+00_rb /)
      kbo(:,46,13) = (/ &
        & 0.54212e+00_rb,0.60419e+00_rb,0.66020e+00_rb,0.70997e+00_rb,0.75300e+00_rb /)
      kbo(:,47,13) = (/ &
        & 0.51537e+00_rb,0.57955e+00_rb,0.63814e+00_rb,0.69060e+00_rb,0.73632e+00_rb /)
      kbo(:,48,13) = (/ &
        & 0.48823e+00_rb,0.55406e+00_rb,0.61515e+00_rb,0.67007e+00_rb,0.71851e+00_rb /)
      kbo(:,49,13) = (/ &
        & 0.46082e+00_rb,0.52769e+00_rb,0.59101e+00_rb,0.64854e+00_rb,0.69978e+00_rb /)
      kbo(:,50,13) = (/ &
        & 0.43478e+00_rb,0.50249e+00_rb,0.56761e+00_rb,0.62745e+00_rb,0.68113e+00_rb /)
      kbo(:,51,13) = (/ &
        & 0.40951e+00_rb,0.47777e+00_rb,0.54418e+00_rb,0.60613e+00_rb,0.66212e+00_rb /)
      kbo(:,52,13) = (/ &
        & 0.38439e+00_rb,0.45288e+00_rb,0.52007e+00_rb,0.58405e+00_rb,0.64238e+00_rb /)
      kbo(:,53,13) = (/ &
        & 0.35936e+00_rb,0.42773e+00_rb,0.49564e+00_rb,0.56121e+00_rb,0.62177e+00_rb /)
      kbo(:,54,13) = (/ &
        & 0.33659e+00_rb,0.40461e+00_rb,0.47296e+00_rb,0.53959e+00_rb,0.60208e+00_rb /)
      kbo(:,55,13) = (/ &
        & 0.31505e+00_rb,0.38259e+00_rb,0.45108e+00_rb,0.51845e+00_rb,0.58256e+00_rb /)
      kbo(:,56,13) = (/ &
        & 0.29381e+00_rb,0.36074e+00_rb,0.42910e+00_rb,0.49702e+00_rb,0.56263e+00_rb /)
      kbo(:,57,13) = (/ &
        & 0.27284e+00_rb,0.33897e+00_rb,0.40701e+00_rb,0.47542e+00_rb,0.54206e+00_rb /)
      kbo(:,58,13) = (/ &
        & 0.25321e+00_rb,0.31859e+00_rb,0.38622e+00_rb,0.45477e+00_rb,0.52205e+00_rb /)
      kbo(:,59,13) = (/ &
        & 0.24543e+00_rb,0.31049e+00_rb,0.37793e+00_rb,0.44646e+00_rb,0.51396e+00_rb /)
      kbo(:,13,14) = (/ &
        & 0.17373e+01_rb,0.17393e+01_rb,0.17383e+01_rb,0.17344e+01_rb,0.17279e+01_rb /)
      kbo(:,14,14) = (/ &
        & 0.18339e+01_rb,0.18343e+01_rb,0.18325e+01_rb,0.18267e+01_rb,0.18189e+01_rb /)
      kbo(:,15,14) = (/ &
        & 0.19299e+01_rb,0.19296e+01_rb,0.19248e+01_rb,0.19184e+01_rb,0.19080e+01_rb /)
      kbo(:,16,14) = (/ &
        & 0.20259e+01_rb,0.20224e+01_rb,0.20169e+01_rb,0.20066e+01_rb,0.19945e+01_rb /)
      kbo(:,17,14) = (/ &
        & 0.21175e+01_rb,0.21127e+01_rb,0.21039e+01_rb,0.20921e+01_rb,0.20768e+01_rb /)
      kbo(:,18,14) = (/ &
        & 0.22042e+01_rb,0.21972e+01_rb,0.21859e+01_rb,0.21721e+01_rb,0.21537e+01_rb /)
      kbo(:,19,14) = (/ &
        & 0.22848e+01_rb,0.22747e+01_rb,0.22622e+01_rb,0.22448e+01_rb,0.22240e+01_rb /)
      kbo(:,20,14) = (/ &
        & 0.23560e+01_rb,0.23462e+01_rb,0.23302e+01_rb,0.23098e+01_rb,0.22867e+01_rb /)
      kbo(:,21,14) = (/ &
        & 0.24183e+01_rb,0.24082e+01_rb,0.23905e+01_rb,0.23680e+01_rb,0.23418e+01_rb /)
      kbo(:,22,14) = (/ &
        & 0.24717e+01_rb,0.24598e+01_rb,0.24413e+01_rb,0.24169e+01_rb,0.23867e+01_rb /)
      kbo(:,23,14) = (/ &
        & 0.25171e+01_rb,0.25034e+01_rb,0.24840e+01_rb,0.24570e+01_rb,0.24242e+01_rb /)
      kbo(:,24,14) = (/ &
        & 0.25547e+01_rb,0.25403e+01_rb,0.25194e+01_rb,0.24898e+01_rb,0.24547e+01_rb /)
      kbo(:,25,14) = (/ &
        & 0.25855e+01_rb,0.25703e+01_rb,0.25468e+01_rb,0.25156e+01_rb,0.24784e+01_rb /)
      kbo(:,26,14) = (/ &
        & 0.26110e+01_rb,0.25944e+01_rb,0.25683e+01_rb,0.25352e+01_rb,0.24957e+01_rb /)
      kbo(:,27,14) = (/ &
        & 0.26319e+01_rb,0.26130e+01_rb,0.25843e+01_rb,0.25495e+01_rb,0.25077e+01_rb /)
      kbo(:,28,14) = (/ &
        & 0.26480e+01_rb,0.26264e+01_rb,0.25959e+01_rb,0.25590e+01_rb,0.25150e+01_rb /)
      kbo(:,29,14) = (/ &
        & 0.26603e+01_rb,0.26355e+01_rb,0.26036e+01_rb,0.25644e+01_rb,0.25186e+01_rb /)
      kbo(:,30,14) = (/ &
        & 0.26690e+01_rb,0.26417e+01_rb,0.26076e+01_rb,0.25661e+01_rb,0.25188e+01_rb /)
      kbo(:,31,14) = (/ &
        & 0.26742e+01_rb,0.26446e+01_rb,0.26086e+01_rb,0.25648e+01_rb,0.25163e+01_rb /)
      kbo(:,32,14) = (/ &
        & 0.26768e+01_rb,0.26452e+01_rb,0.26068e+01_rb,0.25609e+01_rb,0.25113e+01_rb /)
      kbo(:,33,14) = (/ &
        & 0.26771e+01_rb,0.26434e+01_rb,0.26028e+01_rb,0.25551e+01_rb,0.25044e+01_rb /)
      kbo(:,34,14) = (/ &
        & 0.26762e+01_rb,0.26409e+01_rb,0.25981e+01_rb,0.25494e+01_rb,0.24981e+01_rb /)
      kbo(:,35,14) = (/ &
        & 0.26773e+01_rb,0.26407e+01_rb,0.25970e+01_rb,0.25475e+01_rb,0.24957e+01_rb /)
      kbo(:,36,14) = (/ &
        & 0.26808e+01_rb,0.26440e+01_rb,0.25999e+01_rb,0.25501e+01_rb,0.24981e+01_rb /)
      kbo(:,37,14) = (/ &
        & 0.26888e+01_rb,0.26526e+01_rb,0.26095e+01_rb,0.25599e+01_rb,0.25079e+01_rb /)
      kbo(:,38,14) = (/ &
        & 0.26963e+01_rb,0.26609e+01_rb,0.26186e+01_rb,0.25695e+01_rb,0.25177e+01_rb /)
      kbo(:,39,14) = (/ &
        & 0.27030e+01_rb,0.26686e+01_rb,0.26272e+01_rb,0.25787e+01_rb,0.25269e+01_rb /)
      kbo(:,40,14) = (/ &
        & 0.27118e+01_rb,0.26792e+01_rb,0.26399e+01_rb,0.25927e+01_rb,0.25414e+01_rb /)
      kbo(:,41,14) = (/ &
        & 0.27201e+01_rb,0.26898e+01_rb,0.26520e+01_rb,0.26068e+01_rb,0.25561e+01_rb /)
      kbo(:,42,14) = (/ &
        & 0.27274e+01_rb,0.27000e+01_rb,0.26636e+01_rb,0.26203e+01_rb,0.25704e+01_rb /)
      kbo(:,43,14) = (/ &
        & 0.27341e+01_rb,0.27108e+01_rb,0.26767e+01_rb,0.26357e+01_rb,0.25871e+01_rb /)
      kbo(:,44,14) = (/ &
        & 0.27398e+01_rb,0.27209e+01_rb,0.26898e+01_rb,0.26513e+01_rb,0.26050e+01_rb /)
      kbo(:,45,14) = (/ &
        & 0.27435e+01_rb,0.27298e+01_rb,0.27021e+01_rb,0.26655e+01_rb,0.26220e+01_rb /)
      kbo(:,46,14) = (/ &
        & 0.27454e+01_rb,0.27371e+01_rb,0.27139e+01_rb,0.26799e+01_rb,0.26391e+01_rb /)
      kbo(:,47,14) = (/ &
        & 0.27460e+01_rb,0.27424e+01_rb,0.27253e+01_rb,0.26950e+01_rb,0.26569e+01_rb /)
      kbo(:,48,14) = (/ &
        & 0.27436e+01_rb,0.27457e+01_rb,0.27342e+01_rb,0.27088e+01_rb,0.26736e+01_rb /)
      kbo(:,49,14) = (/ &
        & 0.27370e+01_rb,0.27470e+01_rb,0.27411e+01_rb,0.27207e+01_rb,0.26888e+01_rb /)
      kbo(:,50,14) = (/ &
        & 0.27269e+01_rb,0.27459e+01_rb,0.27451e+01_rb,0.27305e+01_rb,0.27022e+01_rb /)
      kbo(:,51,14) = (/ &
        & 0.27139e+01_rb,0.27424e+01_rb,0.27470e+01_rb,0.27377e+01_rb,0.27140e+01_rb /)
      kbo(:,52,14) = (/ &
        & 0.26965e+01_rb,0.27350e+01_rb,0.27474e+01_rb,0.27429e+01_rb,0.27243e+01_rb /)
      kbo(:,53,14) = (/ &
        & 0.26758e+01_rb,0.27241e+01_rb,0.27457e+01_rb,0.27461e+01_rb,0.27331e+01_rb /)
      kbo(:,54,14) = (/ &
        & 0.26529e+01_rb,0.27108e+01_rb,0.27417e+01_rb,0.27475e+01_rb,0.27393e+01_rb /)
      kbo(:,55,14) = (/ &
        & 0.26271e+01_rb,0.26955e+01_rb,0.27349e+01_rb,0.27476e+01_rb,0.27437e+01_rb /)
      kbo(:,56,14) = (/ &
        & 0.25970e+01_rb,0.26775e+01_rb,0.27251e+01_rb,0.27462e+01_rb,0.27464e+01_rb /)
      kbo(:,57,14) = (/ &
        & 0.25617e+01_rb,0.26559e+01_rb,0.27125e+01_rb,0.27426e+01_rb,0.27477e+01_rb /)
      kbo(:,58,14) = (/ &
        & 0.25247e+01_rb,0.26320e+01_rb,0.26988e+01_rb,0.27363e+01_rb,0.27482e+01_rb /)
      kbo(:,59,14) = (/ &
        & 0.25089e+01_rb,0.26212e+01_rb,0.26923e+01_rb,0.27333e+01_rb,0.27480e+01_rb /)
      kbo(:,13,15) = (/ &
        & 0.36859e+01_rb,0.36950e+01_rb,0.36942e+01_rb,0.36830e+01_rb,0.36649e+01_rb /)
      kbo(:,14,15) = (/ &
        & 0.41563e+01_rb,0.41556e+01_rb,0.41430e+01_rb,0.41233e+01_rb,0.40913e+01_rb /)
      kbo(:,15,15) = (/ &
        & 0.46342e+01_rb,0.46206e+01_rb,0.45970e+01_rb,0.45604e+01_rb,0.45153e+01_rb /)
      kbo(:,16,15) = (/ &
        & 0.51044e+01_rb,0.50788e+01_rb,0.50384e+01_rb,0.49882e+01_rb,0.49250e+01_rb /)
      kbo(:,17,15) = (/ &
        & 0.55612e+01_rb,0.55175e+01_rb,0.54618e+01_rb,0.53932e+01_rb,0.53123e+01_rb /)
      kbo(:,18,15) = (/ &
        & 0.59883e+01_rb,0.59323e+01_rb,0.58578e+01_rb,0.57682e+01_rb,0.56700e+01_rb /)
      kbo(:,19,15) = (/ &
        & 0.63852e+01_rb,0.63106e+01_rb,0.62177e+01_rb,0.61123e+01_rb,0.59947e+01_rb /)
      kbo(:,20,15) = (/ &
        & 0.67473e+01_rb,0.66496e+01_rb,0.65407e+01_rb,0.64159e+01_rb,0.62789e+01_rb /)
      kbo(:,21,15) = (/ &
        & 0.70720e+01_rb,0.69522e+01_rb,0.68220e+01_rb,0.66792e+01_rb,0.65258e+01_rb /)
      kbo(:,22,15) = (/ &
        & 0.73512e+01_rb,0.72082e+01_rb,0.70565e+01_rb,0.68942e+01_rb,0.67258e+01_rb /)
      kbo(:,23,15) = (/ &
        & 0.75866e+01_rb,0.74239e+01_rb,0.72515e+01_rb,0.70718e+01_rb,0.68886e+01_rb /)
      kbo(:,24,15) = (/ &
        & 0.77818e+01_rb,0.75993e+01_rb,0.74076e+01_rb,0.72128e+01_rb,0.70154e+01_rb /)
      kbo(:,25,15) = (/ &
        & 0.79381e+01_rb,0.77379e+01_rb,0.75298e+01_rb,0.73205e+01_rb,0.71111e+01_rb /)
      kbo(:,26,15) = (/ &
        & 0.80556e+01_rb,0.78384e+01_rb,0.76185e+01_rb,0.73961e+01_rb,0.71755e+01_rb /)
      kbo(:,27,15) = (/ &
        & 0.81409e+01_rb,0.79095e+01_rb,0.76795e+01_rb,0.74472e+01_rb,0.72174e+01_rb /)
      kbo(:,28,15) = (/ &
        & 0.81987e+01_rb,0.79569e+01_rb,0.77176e+01_rb,0.74766e+01_rb,0.72412e+01_rb /)
      kbo(:,29,15) = (/ &
        & 0.82302e+01_rb,0.79809e+01_rb,0.77337e+01_rb,0.74875e+01_rb,0.72468e+01_rb /)
      kbo(:,30,15) = (/ &
        & 0.82419e+01_rb,0.79875e+01_rb,0.77343e+01_rb,0.74825e+01_rb,0.72390e+01_rb /)
      kbo(:,31,15) = (/ &
        & 0.82369e+01_rb,0.79780e+01_rb,0.77201e+01_rb,0.74661e+01_rb,0.72183e+01_rb /)
      kbo(:,32,15) = (/ &
        & 0.82187e+01_rb,0.79550e+01_rb,0.76944e+01_rb,0.74386e+01_rb,0.71874e+01_rb /)
      kbo(:,33,15) = (/ &
        & 0.81889e+01_rb,0.79220e+01_rb,0.76592e+01_rb,0.74026e+01_rb,0.71493e+01_rb /)
      kbo(:,34,15) = (/ &
        & 0.81602e+01_rb,0.78912e+01_rb,0.76262e+01_rb,0.73693e+01_rb,0.71134e+01_rb /)
      kbo(:,35,15) = (/ &
        & 0.81506e+01_rb,0.78799e+01_rb,0.76144e+01_rb,0.73562e+01_rb,0.70988e+01_rb /)
      kbo(:,36,15) = (/ &
        & 0.81664e+01_rb,0.78930e+01_rb,0.76263e+01_rb,0.73664e+01_rb,0.71072e+01_rb /)
      kbo(:,37,15) = (/ &
        & 0.82200e+01_rb,0.79452e+01_rb,0.76751e+01_rb,0.74132e+01_rb,0.71537e+01_rb /)
      kbo(:,38,15) = (/ &
        & 0.82730e+01_rb,0.79962e+01_rb,0.77251e+01_rb,0.74606e+01_rb,0.72002e+01_rb /)
      kbo(:,39,15) = (/ &
        & 0.83243e+01_rb,0.80459e+01_rb,0.77713e+01_rb,0.75063e+01_rb,0.72452e+01_rb /)
      kbo(:,40,15) = (/ &
        & 0.84027e+01_rb,0.81225e+01_rb,0.78456e+01_rb,0.75775e+01_rb,0.73152e+01_rb /)
      kbo(:,41,15) = (/ &
        & 0.84834e+01_rb,0.82013e+01_rb,0.79233e+01_rb,0.76507e+01_rb,0.73874e+01_rb /)
      kbo(:,42,15) = (/ &
        & 0.85638e+01_rb,0.82794e+01_rb,0.79999e+01_rb,0.77248e+01_rb,0.74594e+01_rb /)
      kbo(:,43,15) = (/ &
        & 0.86608e+01_rb,0.83728e+01_rb,0.80910e+01_rb,0.78137e+01_rb,0.75464e+01_rb /)
      kbo(:,44,15) = (/ &
        & 0.87658e+01_rb,0.84741e+01_rb,0.81903e+01_rb,0.79102e+01_rb,0.76390e+01_rb /)
      kbo(:,45,15) = (/ &
        & 0.88709e+01_rb,0.85747e+01_rb,0.82886e+01_rb,0.80079e+01_rb,0.77318e+01_rb /)
      kbo(:,46,15) = (/ &
        & 0.89828e+01_rb,0.86839e+01_rb,0.83945e+01_rb,0.81115e+01_rb,0.78321e+01_rb /)
      kbo(:,47,15) = (/ &
        & 0.91067e+01_rb,0.88066e+01_rb,0.85118e+01_rb,0.82270e+01_rb,0.79459e+01_rb /)
      kbo(:,48,15) = (/ &
        & 0.92318e+01_rb,0.89292e+01_rb,0.86312e+01_rb,0.83427e+01_rb,0.80595e+01_rb /)
      kbo(:,49,15) = (/ &
        & 0.93597e+01_rb,0.90533e+01_rb,0.87516e+01_rb,0.84602e+01_rb,0.81749e+01_rb /)
      kbo(:,50,15) = (/ &
        & 0.94828e+01_rb,0.91681e+01_rb,0.88667e+01_rb,0.85700e+01_rb,0.82833e+01_rb /)
      kbo(:,51,15) = (/ &
        & 0.96031e+01_rb,0.92821e+01_rb,0.89785e+01_rb,0.86778e+01_rb,0.83878e+01_rb /)
      kbo(:,52,15) = (/ &
        & 0.97227e+01_rb,0.93978e+01_rb,0.90892e+01_rb,0.87879e+01_rb,0.84938e+01_rb /)
      kbo(:,53,15) = (/ &
        & 0.98436e+01_rb,0.95182e+01_rb,0.92014e+01_rb,0.88988e+01_rb,0.86011e+01_rb /)
      kbo(:,54,15) = (/ &
        & 0.99573e+01_rb,0.96274e+01_rb,0.93049e+01_rb,0.90008e+01_rb,0.87001e+01_rb /)
      kbo(:,55,15) = (/ &
        & 0.10066e+02_rb,0.97327e+01_rb,0.94075e+01_rb,0.90983e+01_rb,0.87962e+01_rb /)
      kbo(:,56,15) = (/ &
        & 0.10177e+02_rb,0.98378e+01_rb,0.95127e+01_rb,0.91960e+01_rb,0.88926e+01_rb /)
      kbo(:,57,15) = (/ &
        & 0.10293e+02_rb,0.99462e+01_rb,0.96173e+01_rb,0.92949e+01_rb,0.89909e+01_rb /)
      kbo(:,58,15) = (/ &
        & 0.10405e+02_rb,0.10049e+02_rb,0.97160e+01_rb,0.93909e+01_rb,0.90821e+01_rb /)
      kbo(:,59,15) = (/ &
        & 0.10450e+02_rb,0.10090e+02_rb,0.97556e+01_rb,0.94305e+01_rb,0.91193e+01_rb /)
      kbo(:,13,16) = (/ &
        & 0.50473e+01_rb,0.50398e+01_rb,0.50183e+01_rb,0.49834e+01_rb,0.49369e+01_rb /)
      kbo(:,14,16) = (/ &
        & 0.59220e+01_rb,0.58911e+01_rb,0.58423e+01_rb,0.57748e+01_rb,0.56924e+01_rb /)
      kbo(:,15,16) = (/ &
        & 0.68737e+01_rb,0.68078e+01_rb,0.67189e+01_rb,0.66072e+01_rb,0.64839e+01_rb /)
      kbo(:,16,16) = (/ &
        & 0.78824e+01_rb,0.77694e+01_rb,0.76268e+01_rb,0.74643e+01_rb,0.72894e+01_rb /)
      kbo(:,17,16) = (/ &
        & 0.89270e+01_rb,0.87413e+01_rb,0.85370e+01_rb,0.83130e+01_rb,0.80798e+01_rb /)
      kbo(:,18,16) = (/ &
        & 0.99711e+01_rb,0.97011e+01_rb,0.94250e+01_rb,0.91345e+01_rb,0.88363e+01_rb /)
      kbo(:,19,16) = (/ &
        & 0.10989e+02_rb,0.10627e+02_rb,0.10268e+02_rb,0.99076e+01_rb,0.95430e+01_rb /)
      kbo(:,20,16) = (/ &
        & 0.11944e+02_rb,0.11487e+02_rb,0.11042e+02_rb,0.10609e+02_rb,0.10177e+02_rb /)
      kbo(:,21,16) = (/ &
        & 0.12817e+02_rb,0.12265e+02_rb,0.11737e+02_rb,0.11232e+02_rb,0.10736e+02_rb /)
      kbo(:,22,16) = (/ &
        & 0.13548e+02_rb,0.12914e+02_rb,0.12305e+02_rb,0.11734e+02_rb,0.11183e+02_rb /)
      kbo(:,23,16) = (/ &
        & 0.14162e+02_rb,0.13448e+02_rb,0.12773e+02_rb,0.12143e+02_rb,0.11545e+02_rb /)
      kbo(:,24,16) = (/ &
        & 0.14655e+02_rb,0.13870e+02_rb,0.13141e+02_rb,0.12464e+02_rb,0.11822e+02_rb /)
      kbo(:,25,16) = (/ &
        & 0.15031e+02_rb,0.14189e+02_rb,0.13415e+02_rb,0.12699e+02_rb,0.12023e+02_rb /)
      kbo(:,26,16) = (/ &
        & 0.15288e+02_rb,0.14405e+02_rb,0.13595e+02_rb,0.12850e+02_rb,0.12149e+02_rb /)
      kbo(:,27,16) = (/ &
        & 0.15452e+02_rb,0.14541e+02_rb,0.13705e+02_rb,0.12937e+02_rb,0.12216e+02_rb /)
      kbo(:,28,16) = (/ &
        & 0.15538e+02_rb,0.14608e+02_rb,0.13754e+02_rb,0.12971e+02_rb,0.12237e+02_rb /)
      kbo(:,29,16) = (/ &
        & 0.15553e+02_rb,0.14613e+02_rb,0.13748e+02_rb,0.12957e+02_rb,0.12214e+02_rb /)
      kbo(:,30,16) = (/ &
        & 0.15516e+02_rb,0.14571e+02_rb,0.13703e+02_rb,0.12907e+02_rb,0.12159e+02_rb /)
      kbo(:,31,16) = (/ &
        & 0.15433e+02_rb,0.14488e+02_rb,0.13621e+02_rb,0.12822e+02_rb,0.12076e+02_rb /)
      kbo(:,32,16) = (/ &
        & 0.15312e+02_rb,0.14373e+02_rb,0.13511e+02_rb,0.12712e+02_rb,0.11970e+02_rb /)
      kbo(:,33,16) = (/ &
        & 0.15164e+02_rb,0.14234e+02_rb,0.13378e+02_rb,0.12584e+02_rb,0.11848e+02_rb /)
      kbo(:,34,16) = (/ &
        & 0.15029e+02_rb,0.14108e+02_rb,0.13258e+02_rb,0.12470e+02_rb,0.11741e+02_rb /)
      kbo(:,35,16) = (/ &
        & 0.14974e+02_rb,0.14055e+02_rb,0.13205e+02_rb,0.12418e+02_rb,0.11691e+02_rb /)
      kbo(:,36,16) = (/ &
        & 0.15011e+02_rb,0.14086e+02_rb,0.13232e+02_rb,0.12441e+02_rb,0.11710e+02_rb /)
      kbo(:,37,16) = (/ &
        & 0.15186e+02_rb,0.14243e+02_rb,0.13376e+02_rb,0.12573e+02_rb,0.11831e+02_rb /)
      kbo(:,38,16) = (/ &
        & 0.15364e+02_rb,0.14405e+02_rb,0.13523e+02_rb,0.12708e+02_rb,0.11955e+02_rb /)
      kbo(:,39,16) = (/ &
        & 0.15541e+02_rb,0.14562e+02_rb,0.13666e+02_rb,0.12840e+02_rb,0.12075e+02_rb /)
      kbo(:,40,16) = (/ &
        & 0.15818e+02_rb,0.14814e+02_rb,0.13896e+02_rb,0.13052e+02_rb,0.12271e+02_rb /)
      kbo(:,41,16) = (/ &
        & 0.16115e+02_rb,0.15082e+02_rb,0.14138e+02_rb,0.13278e+02_rb,0.12478e+02_rb /)
      kbo(:,42,16) = (/ &
        & 0.16420e+02_rb,0.15352e+02_rb,0.14384e+02_rb,0.13504e+02_rb,0.12687e+02_rb /)
      kbo(:,43,16) = (/ &
        & 0.16793e+02_rb,0.15690e+02_rb,0.14694e+02_rb,0.13784e+02_rb,0.12952e+02_rb /)
      kbo(:,44,16) = (/ &
        & 0.17211e+02_rb,0.16062e+02_rb,0.15031e+02_rb,0.14091e+02_rb,0.13236e+02_rb /)
      kbo(:,45,16) = (/ &
        & 0.17640e+02_rb,0.16447e+02_rb,0.15376e+02_rb,0.14411e+02_rb,0.13526e+02_rb /)
      kbo(:,46,16) = (/ &
        & 0.18112e+02_rb,0.16870e+02_rb,0.15758e+02_rb,0.14754e+02_rb,0.13842e+02_rb /)
      kbo(:,47,16) = (/ &
        & 0.18658e+02_rb,0.17363e+02_rb,0.16197e+02_rb,0.15151e+02_rb,0.14204e+02_rb /)
      kbo(:,48,16) = (/ &
        & 0.19225e+02_rb,0.17876e+02_rb,0.16658e+02_rb,0.15564e+02_rb,0.14577e+02_rb /)
      kbo(:,49,16) = (/ &
        & 0.19824e+02_rb,0.18409e+02_rb,0.17136e+02_rb,0.15993e+02_rb,0.14967e+02_rb /)
      kbo(:,50,16) = (/ &
        & 0.20411e+02_rb,0.18926e+02_rb,0.17606e+02_rb,0.16414e+02_rb,0.15344e+02_rb /)
      kbo(:,51,16) = (/ &
        & 0.21005e+02_rb,0.19452e+02_rb,0.18077e+02_rb,0.16838e+02_rb,0.15725e+02_rb /)
      kbo(:,52,16) = (/ &
        & 0.21646e+02_rb,0.20001e+02_rb,0.18565e+02_rb,0.17278e+02_rb,0.16117e+02_rb /)
      kbo(:,53,16) = (/ &
        & 0.22315e+02_rb,0.20576e+02_rb,0.19072e+02_rb,0.17736e+02_rb,0.16529e+02_rb /)
      kbo(:,54,16) = (/ &
        & 0.22958e+02_rb,0.21134e+02_rb,0.19557e+02_rb,0.18172e+02_rb,0.16919e+02_rb /)
      kbo(:,55,16) = (/ &
        & 0.23603e+02_rb,0.21694e+02_rb,0.20042e+02_rb,0.18600e+02_rb,0.17305e+02_rb /)
      kbo(:,56,16) = (/ &
        & 0.24285e+02_rb,0.22278e+02_rb,0.20543e+02_rb,0.19044e+02_rb,0.17707e+02_rb /)
      kbo(:,57,16) = (/ &
        & 0.24997e+02_rb,0.22887e+02_rb,0.21074e+02_rb,0.19504e+02_rb,0.18121e+02_rb /)
      kbo(:,58,16) = (/ &
        & 0.25697e+02_rb,0.23494e+02_rb,0.21600e+02_rb,0.19961e+02_rb,0.18525e+02_rb /)
      kbo(:,59,16) = (/ &
        & 0.25984e+02_rb,0.23748e+02_rb,0.21819e+02_rb,0.20147e+02_rb,0.18689e+02_rb /)
  
!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.106275e-05_rb, 0.104185e-05_rb, 0.420154e-05_rb /)
      forrefo(:, 2) = (/ 0.154343e-05_rb, 0.653193e-05_rb, 0.174596e-04_rb /)
      forrefo(:, 3) = (/ 0.348917e-05_rb, 0.108420e-04_rb, 0.540849e-04_rb /)
      forrefo(:, 4) = (/ 0.145822e-04_rb, 0.156027e-04_rb, 0.881263e-04_rb /)
      forrefo(:, 5) = (/ 0.220204e-04_rb, 0.819892e-04_rb, 0.817937e-04_rb /)
      forrefo(:, 6) = (/ 0.447840e-04_rb, 0.121116e-03_rb, 0.932635e-04_rb /)
      forrefo(:, 7) = (/ 0.166516e-03_rb, 0.147640e-03_rb, 0.754029e-04_rb /)
      forrefo(:, 8) = (/ 0.234756e-03_rb, 0.145934e-03_rb, 0.771734e-04_rb /)
      forrefo(:, 9) = (/ 0.289207e-03_rb, 0.146768e-03_rb, 0.677806e-04_rb /)
      forrefo(:,10) = (/ 0.334959e-03_rb, 0.125513e-03_rb, 0.636648e-04_rb /)
      forrefo(:,11) = (/ 0.333755e-03_rb, 0.136575e-03_rb, 0.593651e-04_rb /)
      forrefo(:,12) = (/ 0.340042e-03_rb, 0.116259e-03_rb, 0.595192e-04_rb /)
      forrefo(:,13) = (/ 0.422470e-03_rb, 0.148691e-03_rb, 0.630266e-04_rb /)
      forrefo(:,14) = (/ 0.440655e-03_rb, 0.461917e-04_rb, 0.108222e-04_rb /)
      forrefo(:,15) = (/ 0.486207e-03_rb, 0.428458e-03_rb, 0.108086e-04_rb /)
      forrefo(:,16) = (/ 0.657463e-03_rb, 0.657446e-03_rb, 0.126190e-04_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).
     
      selfrefo(:, 1) = (/ &
        &  0.331728e-03_rb, 0.287480e-03_rb, 0.249135e-03_rb, 0.215904e-03_rb, 0.187106e-03_rb, &
        &  0.162149e-03_rb, 0.140520e-03_rb, 0.121777e-03_rb, 0.105534e-03_rb, 0.914573e-04_rb /)
      selfrefo(:, 2) = (/ &
        &  0.882628e-03_rb, 0.698914e-03_rb, 0.553439e-03_rb, 0.438244e-03_rb, 0.347026e-03_rb, &
        &  0.274795e-03_rb, 0.217598e-03_rb, 0.172306e-03_rb, 0.136442e-03_rb, 0.108042e-03_rb /)
      selfrefo(:, 3) = (/ &
        &  0.115461e-02_rb, 0.937203e-03_rb, 0.760730e-03_rb, 0.617486e-03_rb, 0.501215e-03_rb, &
        &  0.406837e-03_rb, 0.330231e-03_rb, 0.268049e-03_rb, 0.217576e-03_rb, 0.176607e-03_rb /)
      selfrefo(:, 4) = (/ &
        &  0.103450e-02_rb, 0.960268e-03_rb, 0.891360e-03_rb, 0.827397e-03_rb, 0.768024e-03_rb, &
        &  0.712911e-03_rb, 0.661754e-03_rb, 0.614267e-03_rb, 0.570188e-03_rb, 0.529272e-03_rb /)
      selfrefo(:, 5) = (/ &
        &  0.289040e-02_rb, 0.240129e-02_rb, 0.199495e-02_rb, 0.165737e-02_rb, 0.137692e-02_rb, &
        &  0.114392e-02_rb, 0.950351e-03_rb, 0.789535e-03_rb, 0.655933e-03_rb, 0.544938e-03_rb /)
      selfrefo(:, 6) = (/ &
        &  0.361772e-02_rb, 0.306611e-02_rb, 0.259861e-02_rb, 0.220239e-02_rb, 0.186659e-02_rb, &
        &  0.158198e-02_rb, 0.134077e-02_rb, 0.113634e-02_rb, 0.963078e-03_rb, 0.816234e-03_rb /)
      selfrefo(:, 7) = (/ &
        &  0.329878e-02_rb, 0.318245e-02_rb, 0.307021e-02_rb, 0.296194e-02_rb, 0.285749e-02_rb, &
        &  0.275671e-02_rb, 0.265950e-02_rb, 0.256571e-02_rb, 0.247522e-02_rb, 0.238793e-02_rb /)
      selfrefo(:, 8) = (/ &
        &  0.293562e-02_rb, 0.300077e-02_rb, 0.306737e-02_rb, 0.313544e-02_rb, 0.320503e-02_rb, &
        &  0.327615e-02_rb, 0.334886e-02_rb, 0.342318e-02_rb, 0.349915e-02_rb, 0.357680e-02_rb /)
      selfrefo(:, 9) = (/ &
        &  0.281453e-02_rb, 0.295894e-02_rb, 0.311076e-02_rb, 0.327038e-02_rb, 0.343818e-02_rb, &
        &  0.361459e-02_rb, 0.380006e-02_rb, 0.399504e-02_rb, 0.420002e-02_rb, 0.441553e-02_rb /)
      selfrefo(:,10) = (/ &
        &  0.239488e-02_rb, 0.262487e-02_rb, 0.287696e-02_rb, 0.315325e-02_rb, 0.345607e-02_rb, &
        &  0.378798e-02_rb, 0.415176e-02_rb, 0.455048e-02_rb, 0.498749e-02_rb, 0.546647e-02_rb /)
      selfrefo(:,11) = (/ &
        &  0.271001e-02_rb, 0.292235e-02_rb, 0.315134e-02_rb, 0.339826e-02_rb, 0.366453e-02_rb, &
        &  0.395167e-02_rb, 0.426131e-02_rb, 0.459521e-02_rb, 0.495527e-02_rb, 0.534354e-02_rb /)
      selfrefo(:,12) = (/ &
        &  0.206702e-02_rb, 0.232254e-02_rb, 0.260966e-02_rb, 0.293226e-02_rb, 0.329475e-02_rb, &
        &  0.370204e-02_rb, 0.415969e-02_rb, 0.467391e-02_rb, 0.525169e-02_rb, 0.590090e-02_rb /)
      selfrefo(:,13) = (/ &
        &  0.227023e-02_rb, 0.257331e-02_rb, 0.291685e-02_rb, 0.330626e-02_rb, 0.374766e-02_rb, &
        &  0.424799e-02_rb, 0.481511e-02_rb, 0.545794e-02_rb, 0.618660e-02_rb, 0.701253e-02_rb /)
      selfrefo(:,14) = (/ &
        &  0.851078e-03_rb, 0.111512e-02_rb, 0.146109e-02_rb, 0.191439e-02_rb, 0.250832e-02_rb, &
        &  0.328653e-02_rb, 0.430617e-02_rb, 0.564215e-02_rb, 0.739261e-02_rb, 0.968616e-02_rb /)
      selfrefo(:,15) = (/ &
        &  0.742711e-02_rb, 0.721347e-02_rb, 0.700598e-02_rb, 0.680446e-02_rb, 0.660873e-02_rb, &
        &  0.641863e-02_rb, 0.623400e-02_rb, 0.605468e-02_rb, 0.588052e-02_rb, 0.571137e-02_rb /)
      selfrefo(:,16) = (/ &
        &  0.107170e-01_rb, 0.101913e-01_rb, 0.969138e-02_rb, 0.921599e-02_rb, 0.876392e-02_rb, &
        &  0.833402e-02_rb, 0.792521e-02_rb, 0.753646e-02_rb, 0.716677e-02_rb, 0.681522e-02_rb /)
  
      end subroutine sw_kgb19

! **************************************************************************
      subroutine sw_kgb20
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg20, only : kao, kbo, selfrefo, forrefo, sfluxrefo, &
                            absch4o, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:) = (/ &
        & 9.34081_rb , 8.93720_rb    , 8.19346_rb    , 7.39196_rb    , &
        & 6.12127_rb , 5.23956_rb    , 4.24941_rb    , 3.20013_rb    , &
        & 2.16047_rb , 0.234509_rb   , 0.194593_rb   , 0.151512_rb   , &
        & 0.110315_rb, 7.09959e-02_rb, 2.70573e-02_rb, 3.36042e-03_rb /)
  
! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:) = (/ &
        & 9.46001E+00_rb, 9.03828E+00_rb, 8.28405E+00_rb, 7.45458E+00_rb,&
        & 6.22816E+00_rb, 5.31367E+00_rb, 4.30982E+00_rb, 3.24707E+00_rb,&
        & 2.19828E+00_rb, 2.40554E-01_rb, 1.99644E-01_rb, 1.55615E-01_rb,&
        & 1.15119E-01_rb, 7.36741E-02_rb, 2.80316E-02_rb, 3.65491E-03_rb/)
! Solar variability components: facular brightening
      facbrghto(:) = (/ &
        &-8.87561E-05_rb,-1.00992E-04_rb,-2.21755E-05_rb,-1.24294E-05_rb,&
        & 2.37193E-04_rb, 2.43974E-04_rb, 2.07421E-04_rb, 1.58585E-04_rb,&
        & 1.09398E-04_rb, 1.17484E-05_rb, 9.91607E-06_rb, 7.80791E-06_rb,&
        & 5.85632E-06_rb, 3.56586E-06_rb, 1.41702E-06_rb, 2.11899E-07_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:) = (/ &
        &-1.12274E-03_rb,-1.07404E-03_rb,-9.68794E-04_rb,-8.79261E-04_rb,&
        &-7.26951E-04_rb,-6.25280E-04_rb,-5.05982E-04_rb,-3.80851E-04_rb,&
        &-2.57054E-04_rb,-2.80699E-05_rb,-2.32717E-05_rb,-1.83223E-05_rb,&
        &-1.34967E-05_rb,-8.63435E-06_rb,-3.25681E-06_rb,-4.61874E-07_rb/)

      absch4o(:) = (/ &  
        & 1.01381e-03_rb,6.33692e-03_rb,1.94185e-02_rb,4.83210e-02_rb, &
        & 2.36574e-03_rb,6.61973e-04_rb,5.64552e-04_rb,2.83183e-04_rb, &
        & 7.43623e-05_rb,8.90159e-07_rb,6.98728e-07_rb,6.51832e-08_rb, &
        & 2.96619e-08_rb,         0._rb,         0._rb,         0._rb /)

! Rayleigh extinction coefficient at v = 5670 cm-1.
      rayl = 4.12e-09_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1) = (/ &
        & 0.78383e-06_rb,0.86220e-06_rb,0.95359e-06_rb,0.10590e-05_rb,0.11782e-05_rb /)
      kao(:, 2, 1) = (/ &
        & 0.65040e-06_rb,0.72510e-06_rb,0.81318e-06_rb,0.90059e-06_rb,0.99786e-06_rb /)
      kao(:, 3, 1) = (/ &
        & 0.58072e-06_rb,0.65888e-06_rb,0.74265e-06_rb,0.81854e-06_rb,0.90645e-06_rb /)
      kao(:, 4, 1) = (/ &
        & 0.53601e-06_rb,0.60765e-06_rb,0.68088e-06_rb,0.75741e-06_rb,0.83801e-06_rb /)
      kao(:, 5, 1) = (/ &
        & 0.50142e-06_rb,0.56951e-06_rb,0.64028e-06_rb,0.71944e-06_rb,0.79813e-06_rb /)
      kao(:, 6, 1) = (/ &
        & 0.47164e-06_rb,0.54008e-06_rb,0.61040e-06_rb,0.68006e-06_rb,0.75034e-06_rb /)
      kao(:, 7, 1) = (/ &
        & 0.49337e-06_rb,0.56178e-06_rb,0.62215e-06_rb,0.69127e-06_rb,0.76060e-06_rb /)
      kao(:, 8, 1) = (/ &
        & 0.61581e-06_rb,0.70117e-06_rb,0.78942e-06_rb,0.87145e-06_rb,0.94647e-06_rb /)
      kao(:, 9, 1) = (/ &
        & 0.14154e-05_rb,0.15754e-05_rb,0.17261e-05_rb,0.18739e-05_rb,0.19774e-05_rb /)
      kao(:,10, 1) = (/ &
        & 0.34186e-05_rb,0.37012e-05_rb,0.39602e-05_rb,0.42235e-05_rb,0.44256e-05_rb /)
      kao(:,11, 1) = (/ &
        & 0.38574e-05_rb,0.42080e-05_rb,0.44701e-05_rb,0.47454e-05_rb,0.50002e-05_rb /)
      kao(:,12, 1) = (/ &
        & 0.35794e-05_rb,0.38686e-05_rb,0.41443e-05_rb,0.43939e-05_rb,0.46500e-05_rb /)
      kao(:,13, 1) = (/ &
        & 0.29586e-05_rb,0.31938e-05_rb,0.34167e-05_rb,0.36253e-05_rb,0.38250e-05_rb /)
      kao(:, 1, 2) = (/ &
        & 0.57098e-05_rb,0.64630e-05_rb,0.73117e-05_rb,0.82436e-05_rb,0.91947e-05_rb /)
      kao(:, 2, 2) = (/ &
        & 0.46384e-05_rb,0.53125e-05_rb,0.60301e-05_rb,0.68028e-05_rb,0.75824e-05_rb /)
      kao(:, 3, 2) = (/ &
        & 0.40657e-05_rb,0.46486e-05_rb,0.52459e-05_rb,0.58955e-05_rb,0.65546e-05_rb /)
      kao(:, 4, 2) = (/ &
        & 0.38895e-05_rb,0.44258e-05_rb,0.49759e-05_rb,0.55663e-05_rb,0.61702e-05_rb /)
      kao(:, 5, 2) = (/ &
        & 0.38971e-05_rb,0.44359e-05_rb,0.49933e-05_rb,0.55523e-05_rb,0.61234e-05_rb /)
      kao(:, 6, 2) = (/ &
        & 0.39532e-05_rb,0.44644e-05_rb,0.49734e-05_rb,0.54875e-05_rb,0.60102e-05_rb /)
      kao(:, 7, 2) = (/ &
        & 0.41068e-05_rb,0.45832e-05_rb,0.50698e-05_rb,0.55521e-05_rb,0.60175e-05_rb /)
      kao(:, 8, 2) = (/ &
        & 0.47922e-05_rb,0.52156e-05_rb,0.56817e-05_rb,0.61576e-05_rb,0.66313e-05_rb /)
      kao(:, 9, 2) = (/ &
        & 0.83199e-05_rb,0.88317e-05_rb,0.93688e-05_rb,0.99754e-05_rb,0.10620e-04_rb /)
      kao(:,10, 2) = (/ &
        & 0.16836e-04_rb,0.18526e-04_rb,0.19887e-04_rb,0.21168e-04_rb,0.22104e-04_rb /)
      kao(:,11, 2) = (/ &
        & 0.18882e-04_rb,0.21005e-04_rb,0.22896e-04_rb,0.24777e-04_rb,0.26115e-04_rb /)
      kao(:,12, 2) = (/ &
        & 0.17744e-04_rb,0.19780e-04_rb,0.21600e-04_rb,0.23523e-04_rb,0.25128e-04_rb /)
      kao(:,13, 2) = (/ &
        & 0.14736e-04_rb,0.16356e-04_rb,0.17955e-04_rb,0.19533e-04_rb,0.20861e-04_rb /)
      kao(:, 1, 3) = (/ &
        & 0.41776e-04_rb,0.48150e-04_rb,0.55097e-04_rb,0.62661e-04_rb,0.70920e-04_rb /)
      kao(:, 2, 3) = (/ &
        & 0.33909e-04_rb,0.39176e-04_rb,0.44622e-04_rb,0.50754e-04_rb,0.57336e-04_rb /)
      kao(:, 3, 3) = (/ &
        & 0.27438e-04_rb,0.31473e-04_rb,0.36122e-04_rb,0.41134e-04_rb,0.46456e-04_rb /)
      kao(:, 4, 3) = (/ &
        & 0.23222e-04_rb,0.26535e-04_rb,0.30141e-04_rb,0.34099e-04_rb,0.38416e-04_rb /)
      kao(:, 5, 3) = (/ &
        & 0.21114e-04_rb,0.23888e-04_rb,0.26883e-04_rb,0.30340e-04_rb,0.33785e-04_rb /)
      kao(:, 6, 3) = (/ &
        & 0.20750e-04_rb,0.22926e-04_rb,0.25536e-04_rb,0.28492e-04_rb,0.31676e-04_rb /)
      kao(:, 7, 3) = (/ &
        & 0.21584e-04_rb,0.24112e-04_rb,0.26872e-04_rb,0.29794e-04_rb,0.32940e-04_rb /)
      kao(:, 8, 3) = (/ &
        & 0.24194e-04_rb,0.26981e-04_rb,0.30137e-04_rb,0.33546e-04_rb,0.37182e-04_rb /)
      kao(:, 9, 3) = (/ &
        & 0.37461e-04_rb,0.42158e-04_rb,0.46718e-04_rb,0.51048e-04_rb,0.55154e-04_rb /)
      kao(:,10, 3) = (/ &
        & 0.72391e-04_rb,0.77164e-04_rb,0.84016e-04_rb,0.89658e-04_rb,0.95511e-04_rb /)
      kao(:,11, 3) = (/ &
        & 0.91736e-04_rb,0.99107e-04_rb,0.10463e-03_rb,0.10952e-03_rb,0.11549e-03_rb /)
      kao(:,12, 3) = (/ &
        & 0.91200e-04_rb,0.98812e-04_rb,0.10432e-03_rb,0.10893e-03_rb,0.11349e-03_rb /)
      kao(:,13, 3) = (/ &
        & 0.76217e-04_rb,0.82702e-04_rb,0.87500e-04_rb,0.91349e-04_rb,0.95349e-04_rb /)
      kao(:, 1, 4) = (/ &
        & 0.71705e-03_rb,0.82743e-03_rb,0.94700e-03_rb,0.10670e-02_rb,0.11902e-02_rb /)
      kao(:, 2, 4) = (/ &
        & 0.57909e-03_rb,0.67096e-03_rb,0.76724e-03_rb,0.86582e-03_rb,0.97167e-03_rb /)
      kao(:, 3, 4) = (/ &
        & 0.44771e-03_rb,0.51997e-03_rb,0.59861e-03_rb,0.67967e-03_rb,0.76676e-03_rb /)
      kao(:, 4, 4) = (/ &
        & 0.34085e-03_rb,0.39833e-03_rb,0.46226e-03_rb,0.52889e-03_rb,0.60008e-03_rb /)
      kao(:, 5, 4) = (/ &
        & 0.26678e-03_rb,0.31234e-03_rb,0.36341e-03_rb,0.41663e-03_rb,0.47418e-03_rb /)
      kao(:, 6, 4) = (/ &
        & 0.20903e-03_rb,0.24718e-03_rb,0.28922e-03_rb,0.33353e-03_rb,0.38176e-03_rb /)
      kao(:, 7, 4) = (/ &
        & 0.17173e-03_rb,0.20330e-03_rb,0.23711e-03_rb,0.27401e-03_rb,0.31445e-03_rb /)
      kao(:, 8, 4) = (/ &
        & 0.16722e-03_rb,0.19396e-03_rb,0.22307e-03_rb,0.25347e-03_rb,0.28561e-03_rb /)
      kao(:, 9, 4) = (/ &
        & 0.25043e-03_rb,0.28824e-03_rb,0.32547e-03_rb,0.36193e-03_rb,0.39806e-03_rb /)
      kao(:,10, 4) = (/ &
        & 0.43323e-03_rb,0.48667e-03_rb,0.53462e-03_rb,0.57879e-03_rb,0.62040e-03_rb /)
      kao(:,11, 4) = (/ &
        & 0.47719e-03_rb,0.52271e-03_rb,0.56402e-03_rb,0.60388e-03_rb,0.63520e-03_rb /)
      kao(:,12, 4) = (/ &
        & 0.45061e-03_rb,0.49014e-03_rb,0.52391e-03_rb,0.55276e-03_rb,0.57760e-03_rb /)
      kao(:,13, 4) = (/ &
        & 0.37664e-03_rb,0.40952e-03_rb,0.43690e-03_rb,0.46055e-03_rb,0.48018e-03_rb /)
      kao(:, 1, 5) = (/ &
        & 0.66466e-02_rb,0.68686e-02_rb,0.70897e-02_rb,0.73019e-02_rb,0.74914e-02_rb /)
      kao(:, 2, 5) = (/ &
        & 0.54023e-02_rb,0.56010e-02_rb,0.58008e-02_rb,0.59987e-02_rb,0.61685e-02_rb /)
      kao(:, 3, 5) = (/ &
        & 0.43218e-02_rb,0.45127e-02_rb,0.46848e-02_rb,0.48647e-02_rb,0.50267e-02_rb /)
      kao(:, 4, 5) = (/ &
        & 0.34639e-02_rb,0.36473e-02_rb,0.37951e-02_rb,0.39570e-02_rb,0.41049e-02_rb /)
      kao(:, 5, 5) = (/ &
        & 0.27745e-02_rb,0.29370e-02_rb,0.30754e-02_rb,0.32148e-02_rb,0.33492e-02_rb /)
      kao(:, 6, 5) = (/ &
        & 0.22020e-02_rb,0.23520e-02_rb,0.24836e-02_rb,0.25965e-02_rb,0.27119e-02_rb /)
      kao(:, 7, 5) = (/ &
        & 0.17624e-02_rb,0.18963e-02_rb,0.20161e-02_rb,0.21153e-02_rb,0.22116e-02_rb /)
      kao(:, 8, 5) = (/ &
        & 0.14169e-02_rb,0.15332e-02_rb,0.16350e-02_rb,0.17231e-02_rb,0.18121e-02_rb /)
      kao(:, 9, 5) = (/ &
        & 0.13030e-02_rb,0.13904e-02_rb,0.14690e-02_rb,0.15500e-02_rb,0.16275e-02_rb /)
      kao(:,10, 5) = (/ &
        & 0.17155e-02_rb,0.18169e-02_rb,0.18943e-02_rb,0.19620e-02_rb,0.20215e-02_rb /)
      kao(:,11, 5) = (/ &
        & 0.17156e-02_rb,0.17808e-02_rb,0.18419e-02_rb,0.18925e-02_rb,0.19313e-02_rb /)
      kao(:,12, 5) = (/ &
        & 0.15269e-02_rb,0.15811e-02_rb,0.16283e-02_rb,0.16686e-02_rb,0.17061e-02_rb /)
      kao(:,13, 5) = (/ &
        & 0.12768e-02_rb,0.13207e-02_rb,0.13555e-02_rb,0.13924e-02_rb,0.14234e-02_rb /)
      kao(:, 1, 6) = (/ &
        & 0.19191e-01_rb,0.19463e-01_rb,0.19692e-01_rb,0.19900e-01_rb,0.20112e-01_rb /)
      kao(:, 2, 6) = (/ &
        & 0.15967e-01_rb,0.16219e-01_rb,0.16426e-01_rb,0.16613e-01_rb,0.16805e-01_rb /)
      kao(:, 3, 6) = (/ &
        & 0.13186e-01_rb,0.13411e-01_rb,0.13614e-01_rb,0.13785e-01_rb,0.13944e-01_rb /)
      kao(:, 4, 6) = (/ &
        & 0.10935e-01_rb,0.11131e-01_rb,0.11318e-01_rb,0.11475e-01_rb,0.11620e-01_rb /)
      kao(:, 5, 6) = (/ &
        & 0.90541e-02_rb,0.92344e-02_rb,0.94035e-02_rb,0.95537e-02_rb,0.96862e-02_rb /)
      kao(:, 6, 6) = (/ &
        & 0.74724e-02_rb,0.76320e-02_rb,0.77794e-02_rb,0.79294e-02_rb,0.80528e-02_rb /)
      kao(:, 7, 6) = (/ &
        & 0.61183e-02_rb,0.62644e-02_rb,0.63961e-02_rb,0.65303e-02_rb,0.66478e-02_rb /)
      kao(:, 8, 6) = (/ &
        & 0.50638e-02_rb,0.51984e-02_rb,0.53222e-02_rb,0.54412e-02_rb,0.55465e-02_rb /)
      kao(:, 9, 6) = (/ &
        & 0.42870e-02_rb,0.44009e-02_rb,0.45109e-02_rb,0.45976e-02_rb,0.46796e-02_rb /)
      kao(:,10, 6) = (/ &
        & 0.45239e-02_rb,0.46073e-02_rb,0.46968e-02_rb,0.47979e-02_rb,0.48956e-02_rb /)
      kao(:,11, 6) = (/ &
        & 0.44089e-02_rb,0.45006e-02_rb,0.45881e-02_rb,0.46721e-02_rb,0.48015e-02_rb /)
      kao(:,12, 6) = (/ &
        & 0.39598e-02_rb,0.40290e-02_rb,0.41108e-02_rb,0.41899e-02_rb,0.42672e-02_rb /)
      kao(:,13, 6) = (/ &
        & 0.33425e-02_rb,0.34148e-02_rb,0.34808e-02_rb,0.35405e-02_rb,0.35983e-02_rb /)
      kao(:, 1, 7) = (/ &
        & 0.50165e-01_rb,0.50343e-01_rb,0.50452e-01_rb,0.50424e-01_rb,0.50329e-01_rb /)
      kao(:, 2, 7) = (/ &
        & 0.42723e-01_rb,0.42880e-01_rb,0.42939e-01_rb,0.42910e-01_rb,0.42861e-01_rb /)
      kao(:, 3, 7) = (/ &
        & 0.36117e-01_rb,0.36255e-01_rb,0.36301e-01_rb,0.36294e-01_rb,0.36288e-01_rb /)
      kao(:, 4, 7) = (/ &
        & 0.30585e-01_rb,0.30720e-01_rb,0.30787e-01_rb,0.30816e-01_rb,0.30842e-01_rb /)
      kao(:, 5, 7) = (/ &
        & 0.25879e-01_rb,0.26029e-01_rb,0.26116e-01_rb,0.26179e-01_rb,0.26225e-01_rb /)
      kao(:, 6, 7) = (/ &
        & 0.21822e-01_rb,0.21978e-01_rb,0.22094e-01_rb,0.22178e-01_rb,0.22244e-01_rb /)
      kao(:, 7, 7) = (/ &
        & 0.18304e-01_rb,0.18476e-01_rb,0.18606e-01_rb,0.18705e-01_rb,0.18780e-01_rb /)
      kao(:, 8, 7) = (/ &
        & 0.15224e-01_rb,0.15394e-01_rb,0.15519e-01_rb,0.15619e-01_rb,0.15690e-01_rb /)
      kao(:, 9, 7) = (/ &
        & 0.12835e-01_rb,0.13029e-01_rb,0.13169e-01_rb,0.13281e-01_rb,0.13369e-01_rb /)
      kao(:,10, 7) = (/ &
        & 0.11632e-01_rb,0.11714e-01_rb,0.11760e-01_rb,0.11776e-01_rb,0.11806e-01_rb /)
      kao(:,11, 7) = (/ &
        & 0.11170e-01_rb,0.11252e-01_rb,0.11288e-01_rb,0.11357e-01_rb,0.11397e-01_rb /)
      kao(:,12, 7) = (/ &
        & 0.10404e-01_rb,0.10533e-01_rb,0.10705e-01_rb,0.10833e-01_rb,0.10947e-01_rb /)
      kao(:,13, 7) = (/ &
        & 0.91041e-02_rb,0.92674e-02_rb,0.94068e-02_rb,0.95412e-02_rb,0.96769e-02_rb /)
      kao(:, 1, 8) = (/ &
        & 0.14527e+00_rb,0.14483e+00_rb,0.14432e+00_rb,0.14389e+00_rb,0.14349e+00_rb /)
      kao(:, 2, 8) = (/ &
        & 0.12739e+00_rb,0.12702e+00_rb,0.12661e+00_rb,0.12624e+00_rb,0.12584e+00_rb /)
      kao(:, 3, 8) = (/ &
        & 0.11091e+00_rb,0.11058e+00_rb,0.11024e+00_rb,0.10990e+00_rb,0.10953e+00_rb /)
      kao(:, 4, 8) = (/ &
        & 0.96590e-01_rb,0.96335e-01_rb,0.96105e-01_rb,0.95818e-01_rb,0.95530e-01_rb /)
      kao(:, 5, 8) = (/ &
        & 0.83799e-01_rb,0.83708e-01_rb,0.83572e-01_rb,0.83347e-01_rb,0.83126e-01_rb /)
      kao(:, 6, 8) = (/ &
        & 0.72369e-01_rb,0.72388e-01_rb,0.72289e-01_rb,0.72143e-01_rb,0.71981e-01_rb /)
      kao(:, 7, 8) = (/ &
        & 0.62158e-01_rb,0.62247e-01_rb,0.62185e-01_rb,0.62111e-01_rb,0.62007e-01_rb /)
      kao(:, 8, 8) = (/ &
        & 0.52998e-01_rb,0.53142e-01_rb,0.53162e-01_rb,0.53154e-01_rb,0.53101e-01_rb /)
      kao(:, 9, 8) = (/ &
        & 0.44776e-01_rb,0.44873e-01_rb,0.44920e-01_rb,0.44946e-01_rb,0.44885e-01_rb /)
      kao(:,10, 8) = (/ &
        & 0.38237e-01_rb,0.38514e-01_rb,0.38716e-01_rb,0.38877e-01_rb,0.38966e-01_rb /)
      kao(:,11, 8) = (/ &
        & 0.33511e-01_rb,0.33721e-01_rb,0.33843e-01_rb,0.33840e-01_rb,0.33839e-01_rb /)
      kao(:,12, 8) = (/ &
        & 0.29704e-01_rb,0.29825e-01_rb,0.29808e-01_rb,0.29787e-01_rb,0.29794e-01_rb /)
      kao(:,13, 8) = (/ &
        & 0.25973e-01_rb,0.26004e-01_rb,0.26032e-01_rb,0.26023e-01_rb,0.26107e-01_rb /)
      kao(:, 1, 9) = (/ &
        & 0.56699e+00_rb,0.56615e+00_rb,0.56479e+00_rb,0.56318e+00_rb,0.56135e+00_rb /)
      kao(:, 2, 9) = (/ &
        & 0.53130e+00_rb,0.53096e+00_rb,0.53038e+00_rb,0.52953e+00_rb,0.52828e+00_rb /)
      kao(:, 3, 9) = (/ &
        & 0.48947e+00_rb,0.48989e+00_rb,0.48991e+00_rb,0.48970e+00_rb,0.48916e+00_rb /)
      kao(:, 4, 9) = (/ &
        & 0.44646e+00_rb,0.44756e+00_rb,0.44841e+00_rb,0.44893e+00_rb,0.44859e+00_rb /)
      kao(:, 5, 9) = (/ &
        & 0.40428e+00_rb,0.40594e+00_rb,0.40746e+00_rb,0.40811e+00_rb,0.40825e+00_rb /)
      kao(:, 6, 9) = (/ &
        & 0.36305e+00_rb,0.36545e+00_rb,0.36730e+00_rb,0.36814e+00_rb,0.36854e+00_rb /)
      kao(:, 7, 9) = (/ &
        & 0.32383e+00_rb,0.32659e+00_rb,0.32849e+00_rb,0.32954e+00_rb,0.33028e+00_rb /)
      kao(:, 8, 9) = (/ &
        & 0.28691e+00_rb,0.28966e+00_rb,0.29156e+00_rb,0.29286e+00_rb,0.29379e+00_rb /)
      kao(:, 9, 9) = (/ &
        & 0.25109e+00_rb,0.25404e+00_rb,0.25607e+00_rb,0.25752e+00_rb,0.25870e+00_rb /)
      kao(:,10, 9) = (/ &
        & 0.21600e+00_rb,0.21860e+00_rb,0.22047e+00_rb,0.22164e+00_rb,0.22251e+00_rb /)
      kao(:,11, 9) = (/ &
        & 0.19149e+00_rb,0.19387e+00_rb,0.19567e+00_rb,0.19725e+00_rb,0.19809e+00_rb /)
      kao(:,12, 9) = (/ &
        & 0.16914e+00_rb,0.17134e+00_rb,0.17303e+00_rb,0.17407e+00_rb,0.17494e+00_rb /)
      kao(:,13, 9) = (/ &
        & 0.14841e+00_rb,0.15027e+00_rb,0.15244e+00_rb,0.15403e+00_rb,0.15523e+00_rb /)
      kao(:, 1,10) = (/ &
        & 0.14885e+01_rb,0.14756e+01_rb,0.14663e+01_rb,0.14567e+01_rb,0.14479e+01_rb /)
      kao(:, 2,10) = (/ &
        & 0.14911e+01_rb,0.14850e+01_rb,0.14762e+01_rb,0.14677e+01_rb,0.14597e+01_rb /)
      kao(:, 3,10) = (/ &
        & 0.14749e+01_rb,0.14716e+01_rb,0.14663e+01_rb,0.14579e+01_rb,0.14502e+01_rb /)
      kao(:, 4,10) = (/ &
        & 0.14314e+01_rb,0.14303e+01_rb,0.14232e+01_rb,0.14150e+01_rb,0.14115e+01_rb /)
      kao(:, 5,10) = (/ &
        & 0.13616e+01_rb,0.13604e+01_rb,0.13539e+01_rb,0.13506e+01_rb,0.13501e+01_rb /)
      kao(:, 6,10) = (/ &
        & 0.12710e+01_rb,0.12700e+01_rb,0.12670e+01_rb,0.12702e+01_rb,0.12745e+01_rb /)
      kao(:, 7,10) = (/ &
        & 0.11674e+01_rb,0.11690e+01_rb,0.11727e+01_rb,0.11812e+01_rb,0.11854e+01_rb /)
      kao(:, 8,10) = (/ &
        & 0.10584e+01_rb,0.10652e+01_rb,0.10757e+01_rb,0.10857e+01_rb,0.10906e+01_rb /)
      kao(:, 9,10) = (/ &
        & 0.95450e+00_rb,0.96414e+00_rb,0.97673e+00_rb,0.98621e+00_rb,0.99187e+00_rb /)
      kao(:,10,10) = (/ &
        & 0.83626e+00_rb,0.84690e+00_rb,0.85901e+00_rb,0.86824e+00_rb,0.87535e+00_rb /)
      kao(:,11,10) = (/ &
        & 0.73738e+00_rb,0.74729e+00_rb,0.75610e+00_rb,0.75994e+00_rb,0.76481e+00_rb /)
      kao(:,12,10) = (/ &
        & 0.66891e+00_rb,0.67622e+00_rb,0.68092e+00_rb,0.68498e+00_rb,0.68877e+00_rb /)
      kao(:,13,10) = (/ &
        & 0.58616e+00_rb,0.59614e+00_rb,0.59749e+00_rb,0.60310e+00_rb,0.60445e+00_rb /)
      kao(:, 1,11) = (/ &
        & 0.20080e+01_rb,0.19998e+01_rb,0.19844e+01_rb,0.19679e+01_rb,0.19504e+01_rb /)
      kao(:, 2,11) = (/ &
        & 0.20896e+01_rb,0.20752e+01_rb,0.20596e+01_rb,0.20445e+01_rb,0.20293e+01_rb /)
      kao(:, 3,11) = (/ &
        & 0.21262e+01_rb,0.21143e+01_rb,0.21043e+01_rb,0.20968e+01_rb,0.20862e+01_rb /)
      kao(:, 4,11) = (/ &
        & 0.21310e+01_rb,0.21263e+01_rb,0.21243e+01_rb,0.21203e+01_rb,0.21144e+01_rb /)
      kao(:, 5,11) = (/ &
        & 0.21041e+01_rb,0.21063e+01_rb,0.21071e+01_rb,0.21077e+01_rb,0.21021e+01_rb /)
      kao(:, 6,11) = (/ &
        & 0.20447e+01_rb,0.20491e+01_rb,0.20534e+01_rb,0.20532e+01_rb,0.20477e+01_rb /)
      kao(:, 7,11) = (/ &
        & 0.19487e+01_rb,0.19563e+01_rb,0.19608e+01_rb,0.19593e+01_rb,0.19591e+01_rb /)
      kao(:, 8,11) = (/ &
        & 0.18239e+01_rb,0.18317e+01_rb,0.18359e+01_rb,0.18391e+01_rb,0.18451e+01_rb /)
      kao(:, 9,11) = (/ &
        & 0.16772e+01_rb,0.16879e+01_rb,0.16955e+01_rb,0.17047e+01_rb,0.17179e+01_rb /)
      kao(:,10,11) = (/ &
        & 0.15007e+01_rb,0.15184e+01_rb,0.15343e+01_rb,0.15540e+01_rb,0.15756e+01_rb /)
      kao(:,11,11) = (/ &
        & 0.13041e+01_rb,0.13245e+01_rb,0.13445e+01_rb,0.13685e+01_rb,0.13888e+01_rb /)
      kao(:,12,11) = (/ &
        & 0.11590e+01_rb,0.11749e+01_rb,0.11906e+01_rb,0.12046e+01_rb,0.12212e+01_rb /)
      kao(:,13,11) = (/ &
        & 0.10681e+01_rb,0.10905e+01_rb,0.11017e+01_rb,0.10997e+01_rb,0.11116e+01_rb /)
      kao(:, 1,12) = (/ &
        & 0.26762e+01_rb,0.26701e+01_rb,0.26710e+01_rb,0.26742e+01_rb,0.26733e+01_rb /)
      kao(:, 2,12) = (/ &
        & 0.28804e+01_rb,0.28775e+01_rb,0.28798e+01_rb,0.28773e+01_rb,0.28700e+01_rb /)
      kao(:, 3,12) = (/ &
        & 0.30825e+01_rb,0.30790e+01_rb,0.30757e+01_rb,0.30631e+01_rb,0.30471e+01_rb /)
      kao(:, 4,12) = (/ &
        & 0.32355e+01_rb,0.32278e+01_rb,0.32144e+01_rb,0.31959e+01_rb,0.31753e+01_rb /)
      kao(:, 5,12) = (/ &
        & 0.33271e+01_rb,0.33191e+01_rb,0.33073e+01_rb,0.32879e+01_rb,0.32714e+01_rb /)
      kao(:, 6,12) = (/ &
        & 0.33604e+01_rb,0.33571e+01_rb,0.33478e+01_rb,0.33355e+01_rb,0.33272e+01_rb /)
      kao(:, 7,12) = (/ &
        & 0.33406e+01_rb,0.33436e+01_rb,0.33436e+01_rb,0.33435e+01_rb,0.33409e+01_rb /)
      kao(:, 8,12) = (/ &
        & 0.32648e+01_rb,0.32808e+01_rb,0.32919e+01_rb,0.32976e+01_rb,0.32999e+01_rb /)
      kao(:, 9,12) = (/ &
        & 0.31373e+01_rb,0.31630e+01_rb,0.31816e+01_rb,0.31936e+01_rb,0.31991e+01_rb /)
      kao(:,10,12) = (/ &
        & 0.29678e+01_rb,0.29982e+01_rb,0.30244e+01_rb,0.30412e+01_rb,0.30500e+01_rb /)
      kao(:,11,12) = (/ &
        & 0.27380e+01_rb,0.27731e+01_rb,0.28013e+01_rb,0.28227e+01_rb,0.28455e+01_rb /)
      kao(:,12,12) = (/ &
        & 0.24616e+01_rb,0.24905e+01_rb,0.25348e+01_rb,0.25805e+01_rb,0.26155e+01_rb /)
      kao(:,13,12) = (/ &
        & 0.22251e+01_rb,0.22562e+01_rb,0.22966e+01_rb,0.23312e+01_rb,0.23811e+01_rb /)
      kao(:, 1,13) = (/ &
        & 0.38031e+01_rb,0.37992e+01_rb,0.37926e+01_rb,0.37822e+01_rb,0.37755e+01_rb /)
      kao(:, 2,13) = (/ &
        & 0.41542e+01_rb,0.41473e+01_rb,0.41391e+01_rb,0.41329e+01_rb,0.41296e+01_rb /)
      kao(:, 3,13) = (/ &
        & 0.44998e+01_rb,0.44963e+01_rb,0.44896e+01_rb,0.44914e+01_rb,0.44937e+01_rb /)
      kao(:, 4,13) = (/ &
        & 0.48464e+01_rb,0.48445e+01_rb,0.48485e+01_rb,0.48565e+01_rb,0.48598e+01_rb /)
      kao(:, 5,13) = (/ &
        & 0.52074e+01_rb,0.52112e+01_rb,0.52168e+01_rb,0.52231e+01_rb,0.52210e+01_rb /)
      kao(:, 6,13) = (/ &
        & 0.55492e+01_rb,0.55575e+01_rb,0.55655e+01_rb,0.55693e+01_rb,0.55642e+01_rb /)
      kao(:, 7,13) = (/ &
        & 0.58380e+01_rb,0.58479e+01_rb,0.58509e+01_rb,0.58482e+01_rb,0.58453e+01_rb /)
      kao(:, 8,13) = (/ &
        & 0.60377e+01_rb,0.60538e+01_rb,0.60594e+01_rb,0.60647e+01_rb,0.60640e+01_rb /)
      kao(:, 9,13) = (/ &
        & 0.61268e+01_rb,0.61532e+01_rb,0.61730e+01_rb,0.61888e+01_rb,0.61993e+01_rb /)
      kao(:,10,13) = (/ &
        & 0.61216e+01_rb,0.61612e+01_rb,0.61936e+01_rb,0.62230e+01_rb,0.62454e+01_rb /)
      kao(:,11,13) = (/ &
        & 0.60409e+01_rb,0.60960e+01_rb,0.61462e+01_rb,0.61882e+01_rb,0.62219e+01_rb /)
      kao(:,12,13) = (/ &
        & 0.58645e+01_rb,0.59597e+01_rb,0.60234e+01_rb,0.60826e+01_rb,0.61390e+01_rb /)
      kao(:,13,13) = (/ &
        & 0.56001e+01_rb,0.57009e+01_rb,0.58017e+01_rb,0.59181e+01_rb,0.59860e+01_rb /)
      kao(:, 1,14) = (/ &
        & 0.53647e+01_rb,0.53127e+01_rb,0.52736e+01_rb,0.52471e+01_rb,0.52264e+01_rb /)
      kao(:, 2,14) = (/ &
        & 0.62359e+01_rb,0.61753e+01_rb,0.61253e+01_rb,0.60847e+01_rb,0.60573e+01_rb /)
      kao(:, 3,14) = (/ &
        & 0.72065e+01_rb,0.71249e+01_rb,0.70556e+01_rb,0.69965e+01_rb,0.69559e+01_rb /)
      kao(:, 4,14) = (/ &
        & 0.81631e+01_rb,0.80714e+01_rb,0.79828e+01_rb,0.79081e+01_rb,0.78545e+01_rb /)
      kao(:, 5,14) = (/ &
        & 0.90691e+01_rb,0.89602e+01_rb,0.88621e+01_rb,0.87858e+01_rb,0.87314e+01_rb /)
      kao(:, 6,14) = (/ &
        & 0.99707e+01_rb,0.98474e+01_rb,0.97395e+01_rb,0.96541e+01_rb,0.95904e+01_rb /)
      kao(:, 7,14) = (/ &
        & 0.10852e+02_rb,0.10724e+02_rb,0.10620e+02_rb,0.10536e+02_rb,0.10469e+02_rb /)
      kao(:, 8,14) = (/ &
        & 0.11716e+02_rb,0.11588e+02_rb,0.11487e+02_rb,0.11403e+02_rb,0.11337e+02_rb /)
      kao(:, 9,14) = (/ &
        & 0.12572e+02_rb,0.12448e+02_rb,0.12341e+02_rb,0.12260e+02_rb,0.12188e+02_rb /)
      kao(:,10,14) = (/ &
        & 0.13374e+02_rb,0.13256e+02_rb,0.13159e+02_rb,0.13080e+02_rb,0.13010e+02_rb /)
      kao(:,11,14) = (/ &
        & 0.14046e+02_rb,0.13957e+02_rb,0.13888e+02_rb,0.13824e+02_rb,0.13769e+02_rb /)
      kao(:,12,14) = (/ &
        & 0.14644e+02_rb,0.14592e+02_rb,0.14548e+02_rb,0.14504e+02_rb,0.14461e+02_rb /)
      kao(:,13,14) = (/ &
        & 0.15144e+02_rb,0.15129e+02_rb,0.15111e+02_rb,0.15095e+02_rb,0.15074e+02_rb /)
      kao(:, 1,15) = (/ &
        & 0.72019e+01_rb,0.71097e+01_rb,0.70236e+01_rb,0.69449e+01_rb,0.68844e+01_rb /)
      kao(:, 2,15) = (/ &
        & 0.87604e+01_rb,0.86468e+01_rb,0.85349e+01_rb,0.84375e+01_rb,0.83491e+01_rb /)
      kao(:, 3,15) = (/ &
        & 0.10615e+02_rb,0.10469e+02_rb,0.10333e+02_rb,0.10214e+02_rb,0.10099e+02_rb /)
      kao(:, 4,15) = (/ &
        & 0.12669e+02_rb,0.12473e+02_rb,0.12305e+02_rb,0.12161e+02_rb,0.12026e+02_rb /)
      kao(:, 5,15) = (/ &
        & 0.14978e+02_rb,0.14741e+02_rb,0.14530e+02_rb,0.14329e+02_rb,0.14143e+02_rb /)
      kao(:, 6,15) = (/ &
        & 0.17562e+02_rb,0.17282e+02_rb,0.17031e+02_rb,0.16791e+02_rb,0.16552e+02_rb /)
      kao(:, 7,15) = (/ &
        & 0.20480e+02_rb,0.20136e+02_rb,0.19827e+02_rb,0.19524e+02_rb,0.19223e+02_rb /)
      kao(:, 8,15) = (/ &
        & 0.23734e+02_rb,0.23307e+02_rb,0.22908e+02_rb,0.22521e+02_rb,0.22160e+02_rb /)
      kao(:, 9,15) = (/ &
        & 0.27249e+02_rb,0.26733e+02_rb,0.26255e+02_rb,0.25787e+02_rb,0.25354e+02_rb /)
      kao(:,10,15) = (/ &
        & 0.30889e+02_rb,0.30311e+02_rb,0.29740e+02_rb,0.29192e+02_rb,0.28690e+02_rb /)
      kao(:,11,15) = (/ &
        & 0.34366e+02_rb,0.33687e+02_rb,0.33035e+02_rb,0.32430e+02_rb,0.31843e+02_rb /)
      kao(:,12,15) = (/ &
        & 0.37839e+02_rb,0.37085e+02_rb,0.36360e+02_rb,0.35655e+02_rb,0.34957e+02_rb /)
      kao(:,13,15) = (/ &
        & 0.41323e+02_rb,0.40503e+02_rb,0.39673e+02_rb,0.38847e+02_rb,0.38049e+02_rb /)
      kao(:, 1,16) = (/ &
        & 0.82631e+01_rb,0.81761e+01_rb,0.80809e+01_rb,0.79778e+01_rb,0.79010e+01_rb /)
      kao(:, 2,16) = (/ &
        & 0.10258e+02_rb,0.10128e+02_rb,0.10005e+02_rb,0.98803e+01_rb,0.97700e+01_rb /)
      kao(:, 3,16) = (/ &
        & 0.12786e+02_rb,0.12614e+02_rb,0.12441e+02_rb,0.12267e+02_rb,0.12120e+02_rb /)
      kao(:, 4,16) = (/ &
        & 0.15843e+02_rb,0.15549e+02_rb,0.15326e+02_rb,0.15095e+02_rb,0.14871e+02_rb /)
      kao(:, 5,16) = (/ &
        & 0.19543e+02_rb,0.19089e+02_rb,0.18733e+02_rb,0.18430e+02_rb,0.18134e+02_rb /)
      kao(:, 6,16) = (/ &
        & 0.24038e+02_rb,0.23415e+02_rb,0.22893e+02_rb,0.22427e+02_rb,0.22005e+02_rb /)
      kao(:, 7,16) = (/ &
        & 0.29445e+02_rb,0.28630e+02_rb,0.27921e+02_rb,0.27268e+02_rb,0.26654e+02_rb /)
      kao(:, 8,16) = (/ &
        & 0.35889e+02_rb,0.34849e+02_rb,0.33913e+02_rb,0.33036e+02_rb,0.32194e+02_rb /)
      kao(:, 9,16) = (/ &
        & 0.43512e+02_rb,0.42175e+02_rb,0.40956e+02_rb,0.39809e+02_rb,0.38685e+02_rb /)
      kao(:,10,16) = (/ &
        & 0.52253e+02_rb,0.50582e+02_rb,0.49011e+02_rb,0.47508e+02_rb,0.46037e+02_rb /)
      kao(:,11,16) = (/ &
        & 0.61290e+02_rb,0.59240e+02_rb,0.57269e+02_rb,0.55343e+02_rb,0.53572e+02_rb /)
      kao(:,12,16) = (/ &
        & 0.71193e+02_rb,0.68607e+02_rb,0.66135e+02_rb,0.63828e+02_rb,0.61720e+02_rb /)
      kao(:,13,16) = (/ &
        & 0.81988e+02_rb,0.78665e+02_rb,0.75707e+02_rb,0.72932e+02_rb,0.70347e+02_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:,13, 1) = (/ &
        & 0.29597e-05_rb,0.31929e-05_rb,0.34159e-05_rb,0.36256e-05_rb,0.38264e-05_rb /)
      kbo(:,14, 1) = (/ &
        & 0.24233e-05_rb,0.26151e-05_rb,0.27866e-05_rb,0.29502e-05_rb,0.31170e-05_rb /)
      kbo(:,15, 1) = (/ &
        & 0.19206e-05_rb,0.20789e-05_rb,0.22099e-05_rb,0.23383e-05_rb,0.24505e-05_rb /)
      kbo(:,16, 1) = (/ &
        & 0.14933e-05_rb,0.16071e-05_rb,0.17039e-05_rb,0.17994e-05_rb,0.18816e-05_rb /)
      kbo(:,17, 1) = (/ &
        & 0.11429e-05_rb,0.12144e-05_rb,0.12895e-05_rb,0.13503e-05_rb,0.14174e-05_rb /)
      kbo(:,18, 1) = (/ &
        & 0.84134e-06_rb,0.89264e-06_rb,0.94260e-06_rb,0.99003e-06_rb,0.10391e-05_rb /)
      kbo(:,19, 1) = (/ &
        & 0.60683e-06_rb,0.64142e-06_rb,0.68018e-06_rb,0.72042e-06_rb,0.75479e-06_rb /)
      kbo(:,20, 1) = (/ &
        & 0.45809e-06_rb,0.48813e-06_rb,0.51934e-06_rb,0.54823e-06_rb,0.57261e-06_rb /)
      kbo(:,21, 1) = (/ &
        & 0.35219e-06_rb,0.37835e-06_rb,0.40322e-06_rb,0.42772e-06_rb,0.44736e-06_rb /)
      kbo(:,22, 1) = (/ &
        & 0.27419e-06_rb,0.29542e-06_rb,0.31617e-06_rb,0.33398e-06_rb,0.34932e-06_rb /)
      kbo(:,23, 1) = (/ &
        & 0.21385e-06_rb,0.23127e-06_rb,0.24670e-06_rb,0.25951e-06_rb,0.26952e-06_rb /)
      kbo(:,24, 1) = (/ &
        & 0.16767e-06_rb,0.18179e-06_rb,0.19282e-06_rb,0.20136e-06_rb,0.21113e-06_rb /)
      kbo(:,25, 1) = (/ &
        & 0.13361e-06_rb,0.14301e-06_rb,0.14980e-06_rb,0.15778e-06_rb,0.16615e-06_rb /)
      kbo(:,26, 1) = (/ &
        & 0.10615e-06_rb,0.11218e-06_rb,0.11889e-06_rb,0.12574e-06_rb,0.13111e-06_rb /)
      kbo(:,27, 1) = (/ &
        & 0.83660e-07_rb,0.89296e-07_rb,0.95019e-07_rb,0.10007e-06_rb,0.10397e-06_rb /)
      kbo(:,28, 1) = (/ &
        & 0.66667e-07_rb,0.71217e-07_rb,0.75615e-07_rb,0.79237e-07_rb,0.82239e-07_rb /)
      kbo(:,29, 1) = (/ &
        & 0.52776e-07_rb,0.56086e-07_rb,0.59828e-07_rb,0.62513e-07_rb,0.64218e-07_rb /)
      kbo(:,30, 1) = (/ &
        & 0.41429e-07_rb,0.44370e-07_rb,0.46731e-07_rb,0.48553e-07_rb,0.50143e-07_rb /)
      kbo(:,31, 1) = (/ &
        & 0.32437e-07_rb,0.34523e-07_rb,0.36208e-07_rb,0.37500e-07_rb,0.38571e-07_rb /)
      kbo(:,32, 1) = (/ &
        & 0.25237e-07_rb,0.26730e-07_rb,0.27868e-07_rb,0.28829e-07_rb,0.29943e-07_rb /)
      kbo(:,33, 1) = (/ &
        & 0.19545e-07_rb,0.20614e-07_rb,0.21467e-07_rb,0.22280e-07_rb,0.23141e-07_rb /)
      kbo(:,34, 1) = (/ &
        & 0.15146e-07_rb,0.15869e-07_rb,0.16509e-07_rb,0.17179e-07_rb,0.17980e-07_rb /)
      kbo(:,35, 1) = (/ &
        & 0.11482e-07_rb,0.12039e-07_rb,0.12547e-07_rb,0.13255e-07_rb,0.13874e-07_rb /)
      kbo(:,36, 1) = (/ &
        & 0.86172e-08_rb,0.90023e-08_rb,0.95607e-08_rb,0.10106e-07_rb,0.10595e-07_rb /)
      kbo(:,37, 1) = (/ &
        & 0.67411e-08_rb,0.70781e-08_rb,0.75553e-08_rb,0.79739e-08_rb,0.84011e-08_rb /)
      kbo(:,38, 1) = (/ &
        & 0.52604e-08_rb,0.55743e-08_rb,0.59410e-08_rb,0.63031e-08_rb,0.66516e-08_rb /)
      kbo(:,39, 1) = (/ &
        & 0.41129e-08_rb,0.43854e-08_rb,0.46851e-08_rb,0.49771e-08_rb,0.52463e-08_rb /)
      kbo(:,40, 1) = (/ &
        & 0.33089e-08_rb,0.35289e-08_rb,0.37702e-08_rb,0.40125e-08_rb,0.42415e-08_rb /)
      kbo(:,41, 1) = (/ &
        & 0.26794e-08_rb,0.28488e-08_rb,0.30428e-08_rb,0.32437e-08_rb,0.34420e-08_rb /)
      kbo(:,42, 1) = (/ &
        & 0.21670e-08_rb,0.23030e-08_rb,0.24589e-08_rb,0.26252e-08_rb,0.27867e-08_rb /)
      kbo(:,43, 1) = (/ &
        & 0.17699e-08_rb,0.18853e-08_rb,0.20001e-08_rb,0.21330e-08_rb,0.22657e-08_rb /)
      kbo(:,44, 1) = (/ &
        & 0.14446e-08_rb,0.15471e-08_rb,0.16397e-08_rb,0.17423e-08_rb,0.18526e-08_rb /)
      kbo(:,45, 1) = (/ &
        & 0.11734e-08_rb,0.12754e-08_rb,0.13542e-08_rb,0.14242e-08_rb,0.15193e-08_rb /)
      kbo(:,46, 1) = (/ &
        & 0.95743e-09_rb,0.10470e-08_rb,0.11183e-08_rb,0.11726e-08_rb,0.12389e-08_rb /)
      kbo(:,47, 1) = (/ &
        & 0.77345e-09_rb,0.85197e-09_rb,0.92184e-09_rb,0.97368e-09_rb,0.10149e-08_rb /)
      kbo(:,48, 1) = (/ &
        & 0.62500e-09_rb,0.69288e-09_rb,0.75548e-09_rb,0.80446e-09_rb,0.84428e-09_rb /)
      kbo(:,49, 1) = (/ &
        & 0.50095e-09_rb,0.55967e-09_rb,0.61904e-09_rb,0.66441e-09_rb,0.70339e-09_rb /)
      kbo(:,50, 1) = (/ &
        & 0.40806e-09_rb,0.45712e-09_rb,0.50530e-09_rb,0.54974e-09_rb,0.58246e-09_rb /)
      kbo(:,51, 1) = (/ &
        & 0.33102e-09_rb,0.37130e-09_rb,0.41121e-09_rb,0.45024e-09_rb,0.48436e-09_rb /)
      kbo(:,52, 1) = (/ &
        & 0.26906e-09_rb,0.30739e-09_rb,0.33845e-09_rb,0.36884e-09_rb,0.40024e-09_rb /)
      kbo(:,53, 1) = (/ &
        & 0.22202e-09_rb,0.25108e-09_rb,0.27937e-09_rb,0.30592e-09_rb,0.33012e-09_rb /)
      kbo(:,54, 1) = (/ &
        & 0.18662e-09_rb,0.20761e-09_rb,0.23072e-09_rb,0.25342e-09_rb,0.27445e-09_rb /)
      kbo(:,55, 1) = (/ &
        & 0.15741e-09_rb,0.17147e-09_rb,0.19146e-09_rb,0.21029e-09_rb,0.22857e-09_rb /)
      kbo(:,56, 1) = (/ &
        & 0.13151e-09_rb,0.14637e-09_rb,0.15977e-09_rb,0.17547e-09_rb,0.18974e-09_rb /)
      kbo(:,57, 1) = (/ &
        & 0.10874e-09_rb,0.12447e-09_rb,0.13634e-09_rb,0.14752e-09_rb,0.16022e-09_rb /)
      kbo(:,58, 1) = (/ &
        & 0.90206e-10_rb,0.10484e-09_rb,0.11617e-09_rb,0.12545e-09_rb,0.13474e-09_rb /)
      kbo(:,59, 1) = (/ &
        & 0.76759e-10_rb,0.88866e-10_rb,0.99340e-10_rb,0.10706e-09_rb,0.11444e-09_rb /)
      kbo(:,13, 2) = (/ &
        & 0.14727e-04_rb,0.16357e-04_rb,0.17961e-04_rb,0.19530e-04_rb,0.20852e-04_rb /)
      kbo(:,14, 2) = (/ &
        & 0.12148e-04_rb,0.13478e-04_rb,0.14818e-04_rb,0.16049e-04_rb,0.17100e-04_rb /)
      kbo(:,15, 2) = (/ &
        & 0.96902e-05_rb,0.10787e-04_rb,0.11835e-04_rb,0.12770e-04_rb,0.13515e-04_rb /)
      kbo(:,16, 2) = (/ &
        & 0.76235e-05_rb,0.84575e-05_rb,0.92217e-05_rb,0.98526e-05_rb,0.10449e-04_rb /)
      kbo(:,17, 2) = (/ &
        & 0.58049e-05_rb,0.64239e-05_rb,0.69733e-05_rb,0.74131e-05_rb,0.78082e-05_rb /)
      kbo(:,18, 2) = (/ &
        & 0.43204e-05_rb,0.47510e-05_rb,0.51165e-05_rb,0.54579e-05_rb,0.57434e-05_rb /)
      kbo(:,19, 2) = (/ &
        & 0.32211e-05_rb,0.35327e-05_rb,0.38175e-05_rb,0.40535e-05_rb,0.42245e-05_rb /)
      kbo(:,20, 2) = (/ &
        & 0.25265e-05_rb,0.27652e-05_rb,0.29644e-05_rb,0.31331e-05_rb,0.32730e-05_rb /)
      kbo(:,21, 2) = (/ &
        & 0.19921e-05_rb,0.21749e-05_rb,0.23343e-05_rb,0.24377e-05_rb,0.25458e-05_rb /)
      kbo(:,22, 2) = (/ &
        & 0.15859e-05_rb,0.17259e-05_rb,0.18236e-05_rb,0.19124e-05_rb,0.19942e-05_rb /)
      kbo(:,23, 2) = (/ &
        & 0.12591e-05_rb,0.13565e-05_rb,0.14309e-05_rb,0.15032e-05_rb,0.15642e-05_rb /)
      kbo(:,24, 2) = (/ &
        & 0.10012e-05_rb,0.10701e-05_rb,0.11246e-05_rb,0.11819e-05_rb,0.12279e-05_rb /)
      kbo(:,25, 2) = (/ &
        & 0.78985e-06_rb,0.84074e-06_rb,0.89029e-06_rb,0.93145e-06_rb,0.97085e-06_rb /)
      kbo(:,26, 2) = (/ &
        & 0.62996e-06_rb,0.66847e-06_rb,0.70637e-06_rb,0.73513e-06_rb,0.77135e-06_rb /)
      kbo(:,27, 2) = (/ &
        & 0.50283e-06_rb,0.53111e-06_rb,0.55653e-06_rb,0.58306e-06_rb,0.61799e-06_rb /)
      kbo(:,28, 2) = (/ &
        & 0.39962e-06_rb,0.42320e-06_rb,0.44265e-06_rb,0.46715e-06_rb,0.49516e-06_rb /)
      kbo(:,29, 2) = (/ &
        & 0.31810e-06_rb,0.33597e-06_rb,0.35191e-06_rb,0.37448e-06_rb,0.39768e-06_rb /)
      kbo(:,30, 2) = (/ &
        & 0.25242e-06_rb,0.26447e-06_rb,0.28109e-06_rb,0.30089e-06_rb,0.31543e-06_rb /)
      kbo(:,31, 2) = (/ &
        & 0.19837e-06_rb,0.20954e-06_rb,0.22442e-06_rb,0.23762e-06_rb,0.25183e-06_rb /)
      kbo(:,32, 2) = (/ &
        & 0.15599e-06_rb,0.16701e-06_rb,0.17789e-06_rb,0.18880e-06_rb,0.20030e-06_rb /)
      kbo(:,33, 2) = (/ &
        & 0.12307e-06_rb,0.13211e-06_rb,0.14046e-06_rb,0.15111e-06_rb,0.15988e-06_rb /)
      kbo(:,34, 2) = (/ &
        & 0.98429e-07_rb,0.10498e-06_rb,0.11326e-06_rb,0.12074e-06_rb,0.12779e-06_rb /)
      kbo(:,35, 2) = (/ &
        & 0.77743e-07_rb,0.83951e-07_rb,0.90473e-07_rb,0.96277e-07_rb,0.10204e-06_rb /)
      kbo(:,36, 2) = (/ &
        & 0.61220e-07_rb,0.66399e-07_rb,0.71102e-07_rb,0.75946e-07_rb,0.80803e-07_rb /)
      kbo(:,37, 2) = (/ &
        & 0.48819e-07_rb,0.52962e-07_rb,0.56669e-07_rb,0.60817e-07_rb,0.64709e-07_rb /)
      kbo(:,38, 2) = (/ &
        & 0.38871e-07_rb,0.42226e-07_rb,0.45376e-07_rb,0.48641e-07_rb,0.51855e-07_rb /)
      kbo(:,39, 2) = (/ &
        & 0.30869e-07_rb,0.33485e-07_rb,0.36233e-07_rb,0.38868e-07_rb,0.41523e-07_rb /)
      kbo(:,40, 2) = (/ &
        & 0.24798e-07_rb,0.26983e-07_rb,0.29180e-07_rb,0.31437e-07_rb,0.33600e-07_rb /)
      kbo(:,41, 2) = (/ &
        & 0.19951e-07_rb,0.21784e-07_rb,0.23518e-07_rb,0.25460e-07_rb,0.27219e-07_rb /)
      kbo(:,42, 2) = (/ &
        & 0.16098e-07_rb,0.17542e-07_rb,0.18979e-07_rb,0.20547e-07_rb,0.22016e-07_rb /)
      kbo(:,43, 2) = (/ &
        & 0.13020e-07_rb,0.14144e-07_rb,0.15414e-07_rb,0.16610e-07_rb,0.17870e-07_rb /)
      kbo(:,44, 2) = (/ &
        & 0.10548e-07_rb,0.11500e-07_rb,0.12492e-07_rb,0.13520e-07_rb,0.14524e-07_rb /)
      kbo(:,45, 2) = (/ &
        & 0.85602e-08_rb,0.93245e-08_rb,0.10096e-07_rb,0.10978e-07_rb,0.11819e-07_rb /)
      kbo(:,46, 2) = (/ &
        & 0.69149e-08_rb,0.75653e-08_rb,0.82154e-08_rb,0.88822e-08_rb,0.96002e-08_rb /)
      kbo(:,47, 2) = (/ &
        & 0.55708e-08_rb,0.61164e-08_rb,0.66678e-08_rb,0.71922e-08_rb,0.77633e-08_rb /)
      kbo(:,48, 2) = (/ &
        & 0.44804e-08_rb,0.49312e-08_rb,0.53967e-08_rb,0.58662e-08_rb,0.62780e-08_rb /)
      kbo(:,49, 2) = (/ &
        & 0.36467e-08_rb,0.39866e-08_rb,0.43628e-08_rb,0.47589e-08_rb,0.51208e-08_rb /)
      kbo(:,50, 2) = (/ &
        & 0.29524e-08_rb,0.32540e-08_rb,0.35471e-08_rb,0.38638e-08_rb,0.41887e-08_rb /)
      kbo(:,51, 2) = (/ &
        & 0.23887e-08_rb,0.26526e-08_rb,0.28959e-08_rb,0.31504e-08_rb,0.34112e-08_rb /)
      kbo(:,52, 2) = (/ &
        & 0.19321e-08_rb,0.21648e-08_rb,0.23625e-08_rb,0.25839e-08_rb,0.28173e-08_rb /)
      kbo(:,53, 2) = (/ &
        & 0.15684e-08_rb,0.17725e-08_rb,0.19476e-08_rb,0.21240e-08_rb,0.23118e-08_rb /)
      kbo(:,54, 2) = (/ &
        & 0.12799e-08_rb,0.14467e-08_rb,0.16073e-08_rb,0.17494e-08_rb,0.19160e-08_rb /)
      kbo(:,55, 2) = (/ &
        & 0.10442e-08_rb,0.11911e-08_rb,0.13216e-08_rb,0.14537e-08_rb,0.15771e-08_rb /)
      kbo(:,56, 2) = (/ &
        & 0.85741e-09_rb,0.97937e-09_rb,0.10936e-08_rb,0.12056e-08_rb,0.13167e-08_rb /)
      kbo(:,57, 2) = (/ &
        & 0.70358e-09_rb,0.80334e-09_rb,0.90824e-09_rb,0.10056e-08_rb,0.11010e-08_rb /)
      kbo(:,58, 2) = (/ &
        & 0.57428e-09_rb,0.66441e-09_rb,0.75420e-09_rb,0.83627e-09_rb,0.92485e-09_rb /)
      kbo(:,59, 2) = (/ &
        & 0.47922e-09_rb,0.55592e-09_rb,0.63499e-09_rb,0.70731e-09_rb,0.78678e-09_rb /)
      kbo(:,13, 3) = (/ &
        & 0.76201e-04_rb,0.82699e-04_rb,0.87499e-04_rb,0.91352e-04_rb,0.95348e-04_rb /)
      kbo(:,14, 3) = (/ &
        & 0.62821e-04_rb,0.67861e-04_rb,0.71868e-04_rb,0.75108e-04_rb,0.78557e-04_rb /)
      kbo(:,15, 3) = (/ &
        & 0.49775e-04_rb,0.53367e-04_rb,0.56639e-04_rb,0.59001e-04_rb,0.62303e-04_rb /)
      kbo(:,16, 3) = (/ &
        & 0.38005e-04_rb,0.40883e-04_rb,0.43269e-04_rb,0.45403e-04_rb,0.48139e-04_rb /)
      kbo(:,17, 3) = (/ &
        & 0.28550e-04_rb,0.30750e-04_rb,0.32331e-04_rb,0.34489e-04_rb,0.36687e-04_rb /)
      kbo(:,18, 3) = (/ &
        & 0.20976e-04_rb,0.22322e-04_rb,0.23743e-04_rb,0.25554e-04_rb,0.27223e-04_rb /)
      kbo(:,19, 3) = (/ &
        & 0.15389e-04_rb,0.16422e-04_rb,0.17792e-04_rb,0.19127e-04_rb,0.20295e-04_rb /)
      kbo(:,20, 3) = (/ &
        & 0.11883e-04_rb,0.12816e-04_rb,0.13962e-04_rb,0.14872e-04_rb,0.15725e-04_rb /)
      kbo(:,21, 3) = (/ &
        & 0.92948e-05_rb,0.10187e-04_rb,0.10986e-04_rb,0.11713e-04_rb,0.12426e-04_rb /)
      kbo(:,22, 3) = (/ &
        & 0.73969e-05_rb,0.80840e-05_rb,0.87263e-05_rb,0.93856e-05_rb,0.99533e-05_rb /)
      kbo(:,23, 3) = (/ &
        & 0.58961e-05_rb,0.64591e-05_rb,0.69648e-05_rb,0.74494e-05_rb,0.79385e-05_rb /)
      kbo(:,24, 3) = (/ &
        & 0.47445e-05_rb,0.51742e-05_rb,0.56036e-05_rb,0.59758e-05_rb,0.63724e-05_rb /)
      kbo(:,25, 3) = (/ &
        & 0.38442e-05_rb,0.41791e-05_rb,0.44837e-05_rb,0.48078e-05_rb,0.51294e-05_rb /)
      kbo(:,26, 3) = (/ &
        & 0.31335e-05_rb,0.33923e-05_rb,0.36400e-05_rb,0.38961e-05_rb,0.41487e-05_rb /)
      kbo(:,27, 3) = (/ &
        & 0.25499e-05_rb,0.27432e-05_rb,0.29560e-05_rb,0.31545e-05_rb,0.33433e-05_rb /)
      kbo(:,28, 3) = (/ &
        & 0.20635e-05_rb,0.22231e-05_rb,0.23897e-05_rb,0.25516e-05_rb,0.26938e-05_rb /)
      kbo(:,29, 3) = (/ &
        & 0.16635e-05_rb,0.18024e-05_rb,0.19279e-05_rb,0.20498e-05_rb,0.21627e-05_rb /)
      kbo(:,30, 3) = (/ &
        & 0.13359e-05_rb,0.14474e-05_rb,0.15470e-05_rb,0.16346e-05_rb,0.17457e-05_rb /)
      kbo(:,31, 3) = (/ &
        & 0.10697e-05_rb,0.11477e-05_rb,0.12254e-05_rb,0.13097e-05_rb,0.13914e-05_rb /)
      kbo(:,32, 3) = (/ &
        & 0.84958e-06_rb,0.91464e-06_rb,0.98189e-06_rb,0.10489e-05_rb,0.11150e-05_rb /)
      kbo(:,33, 3) = (/ &
        & 0.68178e-06_rb,0.73356e-06_rb,0.78905e-06_rb,0.84226e-06_rb,0.89476e-06_rb /)
      kbo(:,34, 3) = (/ &
        & 0.54737e-06_rb,0.58995e-06_rb,0.63374e-06_rb,0.67732e-06_rb,0.72182e-06_rb /)
      kbo(:,35, 3) = (/ &
        & 0.43680e-06_rb,0.47086e-06_rb,0.50771e-06_rb,0.54209e-06_rb,0.57687e-06_rb /)
      kbo(:,36, 3) = (/ &
        & 0.34612e-06_rb,0.37466e-06_rb,0.40370e-06_rb,0.43073e-06_rb,0.45979e-06_rb /)
      kbo(:,37, 3) = (/ &
        & 0.27725e-06_rb,0.30048e-06_rb,0.32451e-06_rb,0.34638e-06_rb,0.37105e-06_rb /)
      kbo(:,38, 3) = (/ &
        & 0.22228e-06_rb,0.24114e-06_rb,0.26049e-06_rb,0.27871e-06_rb,0.29935e-06_rb /)
      kbo(:,39, 3) = (/ &
        & 0.17797e-06_rb,0.19317e-06_rb,0.20878e-06_rb,0.22411e-06_rb,0.24102e-06_rb /)
      kbo(:,40, 3) = (/ &
        & 0.14346e-06_rb,0.15601e-06_rb,0.16913e-06_rb,0.18197e-06_rb,0.19566e-06_rb /)
      kbo(:,41, 3) = (/ &
        & 0.11559e-06_rb,0.12615e-06_rb,0.13697e-06_rb,0.14785e-06_rb,0.15874e-06_rb /)
      kbo(:,42, 3) = (/ &
        & 0.93358e-07_rb,0.10214e-06_rb,0.11089e-06_rb,0.11998e-06_rb,0.12904e-06_rb /)
      kbo(:,43, 3) = (/ &
        & 0.75599e-07_rb,0.82690e-07_rb,0.89808e-07_rb,0.97524e-07_rb,0.10511e-06_rb /)
      kbo(:,44, 3) = (/ &
        & 0.61285e-07_rb,0.66786e-07_rb,0.72937e-07_rb,0.79087e-07_rb,0.85525e-07_rb /)
      kbo(:,45, 3) = (/ &
        & 0.49725e-07_rb,0.54294e-07_rb,0.59263e-07_rb,0.64213e-07_rb,0.69583e-07_rb /)
      kbo(:,46, 3) = (/ &
        & 0.40336e-07_rb,0.44186e-07_rb,0.48034e-07_rb,0.52184e-07_rb,0.56534e-07_rb /)
      kbo(:,47, 3) = (/ &
        & 0.32904e-07_rb,0.35633e-07_rb,0.38959e-07_rb,0.42402e-07_rb,0.45883e-07_rb /)
      kbo(:,48, 3) = (/ &
        & 0.26725e-07_rb,0.29104e-07_rb,0.31693e-07_rb,0.34312e-07_rb,0.37304e-07_rb /)
      kbo(:,49, 3) = (/ &
        & 0.21751e-07_rb,0.23849e-07_rb,0.25803e-07_rb,0.28024e-07_rb,0.30356e-07_rb /)
      kbo(:,50, 3) = (/ &
        & 0.17587e-07_rb,0.19609e-07_rb,0.21216e-07_rb,0.23022e-07_rb,0.24942e-07_rb /)
      kbo(:,51, 3) = (/ &
        & 0.14376e-07_rb,0.15990e-07_rb,0.17493e-07_rb,0.18872e-07_rb,0.20463e-07_rb /)
      kbo(:,52, 3) = (/ &
        & 0.11811e-07_rb,0.13106e-07_rb,0.14489e-07_rb,0.15615e-07_rb,0.16832e-07_rb /)
      kbo(:,53, 3) = (/ &
        & 0.96001e-08_rb,0.10788e-07_rb,0.11921e-07_rb,0.12985e-07_rb,0.13904e-07_rb /)
      kbo(:,54, 3) = (/ &
        & 0.78243e-08_rb,0.89178e-08_rb,0.98430e-08_rb,0.10840e-07_rb,0.11619e-07_rb /)
      kbo(:,55, 3) = (/ &
        & 0.63638e-08_rb,0.73678e-08_rb,0.82214e-08_rb,0.89997e-08_rb,0.97565e-08_rb /)
      kbo(:,56, 3) = (/ &
        & 0.51753e-08_rb,0.60497e-08_rb,0.68419e-08_rb,0.74560e-08_rb,0.82362e-08_rb /)
      kbo(:,57, 3) = (/ &
        & 0.42164e-08_rb,0.49916e-08_rb,0.57066e-08_rb,0.63308e-08_rb,0.68950e-08_rb /)
      kbo(:,58, 3) = (/ &
        & 0.34618e-08_rb,0.41424e-08_rb,0.48054e-08_rb,0.53757e-08_rb,0.58095e-08_rb /)
      kbo(:,59, 3) = (/ &
        & 0.29083e-08_rb,0.35291e-08_rb,0.40932e-08_rb,0.46285e-08_rb,0.49865e-08_rb /)
      kbo(:,13, 4) = (/ &
        & 0.37674e-03_rb,0.40953e-03_rb,0.43690e-03_rb,0.46059e-03_rb,0.48007e-03_rb /)
      kbo(:,14, 4) = (/ &
        & 0.31202e-03_rb,0.33906e-03_rb,0.36066e-03_rb,0.38003e-03_rb,0.39627e-03_rb /)
      kbo(:,15, 4) = (/ &
        & 0.25135e-03_rb,0.27271e-03_rb,0.29069e-03_rb,0.30661e-03_rb,0.32005e-03_rb /)
      kbo(:,16, 4) = (/ &
        & 0.19797e-03_rb,0.21527e-03_rb,0.22955e-03_rb,0.24328e-03_rb,0.25456e-03_rb /)
      kbo(:,17, 4) = (/ &
        & 0.15354e-03_rb,0.16679e-03_rb,0.17933e-03_rb,0.19101e-03_rb,0.20041e-03_rb /)
      kbo(:,18, 4) = (/ &
        & 0.11742e-03_rb,0.12797e-03_rb,0.13923e-03_rb,0.14836e-03_rb,0.15654e-03_rb /)
      kbo(:,19, 4) = (/ &
        & 0.90519e-04_rb,0.99386e-04_rb,0.10819e-03_rb,0.11601e-03_rb,0.12319e-03_rb /)
      kbo(:,20, 4) = (/ &
        & 0.72579e-04_rb,0.79910e-04_rb,0.87075e-04_rb,0.93728e-04_rb,0.99656e-04_rb /)
      kbo(:,21, 4) = (/ &
        & 0.58866e-04_rb,0.64805e-04_rb,0.70705e-04_rb,0.76153e-04_rb,0.80996e-04_rb /)
      kbo(:,22, 4) = (/ &
        & 0.48157e-04_rb,0.52883e-04_rb,0.57740e-04_rb,0.62130e-04_rb,0.66100e-04_rb /)
      kbo(:,23, 4) = (/ &
        & 0.39337e-04_rb,0.43289e-04_rb,0.47275e-04_rb,0.50906e-04_rb,0.54161e-04_rb /)
      kbo(:,24, 4) = (/ &
        & 0.32289e-04_rb,0.35577e-04_rb,0.38808e-04_rb,0.41814e-04_rb,0.44433e-04_rb /)
      kbo(:,25, 4) = (/ &
        & 0.26615e-04_rb,0.29301e-04_rb,0.31984e-04_rb,0.34396e-04_rb,0.36509e-04_rb /)
      kbo(:,26, 4) = (/ &
        & 0.22060e-04_rb,0.24270e-04_rb,0.26460e-04_rb,0.28450e-04_rb,0.30203e-04_rb /)
      kbo(:,27, 4) = (/ &
        & 0.18344e-04_rb,0.20177e-04_rb,0.21948e-04_rb,0.23595e-04_rb,0.25001e-04_rb /)
      kbo(:,28, 4) = (/ &
        & 0.15256e-04_rb,0.16787e-04_rb,0.18242e-04_rb,0.19561e-04_rb,0.20696e-04_rb /)
      kbo(:,29, 4) = (/ &
        & 0.12679e-04_rb,0.13952e-04_rb,0.15170e-04_rb,0.16211e-04_rb,0.17163e-04_rb /)
      kbo(:,30, 4) = (/ &
        & 0.10550e-04_rb,0.11605e-04_rb,0.12579e-04_rb,0.13451e-04_rb,0.14210e-04_rb /)
      kbo(:,31, 4) = (/ &
        & 0.87512e-05_rb,0.96322e-05_rb,0.10433e-04_rb,0.11133e-04_rb,0.11762e-04_rb /)
      kbo(:,32, 4) = (/ &
        & 0.73001e-05_rb,0.80203e-05_rb,0.86745e-05_rb,0.92292e-05_rb,0.97295e-05_rb /)
      kbo(:,33, 4) = (/ &
        & 0.60958e-05_rb,0.66862e-05_rb,0.72006e-05_rb,0.76359e-05_rb,0.80576e-05_rb /)
      kbo(:,34, 4) = (/ &
        & 0.50855e-05_rb,0.55671e-05_rb,0.59670e-05_rb,0.63392e-05_rb,0.66869e-05_rb /)
      kbo(:,35, 4) = (/ &
        & 0.42033e-05_rb,0.45905e-05_rb,0.49250e-05_rb,0.52384e-05_rb,0.55369e-05_rb /)
      kbo(:,36, 4) = (/ &
        & 0.34384e-05_rb,0.37618e-05_rb,0.40430e-05_rb,0.43060e-05_rb,0.45577e-05_rb /)
      kbo(:,37, 4) = (/ &
        & 0.28039e-05_rb,0.30749e-05_rb,0.33135e-05_rb,0.35344e-05_rb,0.37458e-05_rb /)
      kbo(:,38, 4) = (/ &
        & 0.22842e-05_rb,0.25104e-05_rb,0.27126e-05_rb,0.28966e-05_rb,0.30768e-05_rb /)
      kbo(:,39, 4) = (/ &
        & 0.18585e-05_rb,0.20510e-05_rb,0.22194e-05_rb,0.23731e-05_rb,0.25249e-05_rb /)
      kbo(:,40, 4) = (/ &
        & 0.15041e-05_rb,0.16676e-05_rb,0.18097e-05_rb,0.19402e-05_rb,0.20681e-05_rb /)
      kbo(:,41, 4) = (/ &
        & 0.12154e-05_rb,0.13541e-05_rb,0.14749e-05_rb,0.15852e-05_rb,0.16934e-05_rb /)
      kbo(:,42, 4) = (/ &
        & 0.98033e-06_rb,0.10975e-05_rb,0.12011e-05_rb,0.12945e-05_rb,0.13859e-05_rb /)
      kbo(:,43, 4) = (/ &
        & 0.78724e-06_rb,0.88625e-06_rb,0.97486e-06_rb,0.10554e-05_rb,0.11307e-05_rb /)
      kbo(:,44, 4) = (/ &
        & 0.63059e-06_rb,0.71382e-06_rb,0.78948e-06_rb,0.85755e-06_rb,0.92044e-06_rb /)
      kbo(:,45, 4) = (/ &
        & 0.50426e-06_rb,0.57391e-06_rb,0.63808e-06_rb,0.69718e-06_rb,0.74972e-06_rb /)
      kbo(:,46, 4) = (/ &
        & 0.40133e-06_rb,0.45858e-06_rb,0.51414e-06_rb,0.56400e-06_rb,0.60964e-06_rb /)
      kbo(:,47, 4) = (/ &
        & 0.31580e-06_rb,0.36440e-06_rb,0.41120e-06_rb,0.45433e-06_rb,0.49348e-06_rb /)
      kbo(:,48, 4) = (/ &
        & 0.24847e-06_rb,0.28907e-06_rb,0.32879e-06_rb,0.36537e-06_rb,0.39903e-06_rb /)
      kbo(:,49, 4) = (/ &
        & 0.19530e-06_rb,0.22858e-06_rb,0.26247e-06_rb,0.29331e-06_rb,0.32217e-06_rb /)
      kbo(:,50, 4) = (/ &
        & 0.15364e-06_rb,0.18113e-06_rb,0.20911e-06_rb,0.23589e-06_rb,0.25994e-06_rb /)
      kbo(:,51, 4) = (/ &
        & 0.12072e-06_rb,0.14413e-06_rb,0.16656e-06_rb,0.18952e-06_rb,0.20983e-06_rb /)
      kbo(:,52, 4) = (/ &
        & 0.94925e-07_rb,0.11397e-06_rb,0.13255e-06_rb,0.15193e-06_rb,0.16960e-06_rb /)
      kbo(:,53, 4) = (/ &
        & 0.74473e-07_rb,0.90197e-07_rb,0.10574e-06_rb,0.12153e-06_rb,0.13661e-06_rb /)
      kbo(:,54, 4) = (/ &
        & 0.59009e-07_rb,0.71594e-07_rb,0.84781e-07_rb,0.97758e-07_rb,0.11018e-06_rb /)
      kbo(:,55, 4) = (/ &
        & 0.47008e-07_rb,0.56997e-07_rb,0.67838e-07_rb,0.78753e-07_rb,0.89116e-07_rb /)
      kbo(:,56, 4) = (/ &
        & 0.37415e-07_rb,0.45642e-07_rb,0.54420e-07_rb,0.63466e-07_rb,0.71910e-07_rb /)
      kbo(:,57, 4) = (/ &
        & 0.29896e-07_rb,0.36635e-07_rb,0.43697e-07_rb,0.51036e-07_rb,0.58267e-07_rb /)
      kbo(:,58, 4) = (/ &
        & 0.23946e-07_rb,0.29523e-07_rb,0.35316e-07_rb,0.41183e-07_rb,0.47391e-07_rb /)
      kbo(:,59, 4) = (/ &
        & 0.19881e-07_rb,0.24642e-07_rb,0.29390e-07_rb,0.34099e-07_rb,0.39234e-07_rb /)
      kbo(:,13, 5) = (/ &
        & 0.12771e-02_rb,0.13202e-02_rb,0.13557e-02_rb,0.13927e-02_rb,0.14233e-02_rb /)
      kbo(:,14, 5) = (/ &
        & 0.10608e-02_rb,0.10951e-02_rb,0.11249e-02_rb,0.11549e-02_rb,0.11788e-02_rb /)
      kbo(:,15, 5) = (/ &
        & 0.86750e-03_rb,0.89679e-03_rb,0.91986e-03_rb,0.94399e-03_rb,0.96405e-03_rb /)
      kbo(:,16, 5) = (/ &
        & 0.70073e-03_rb,0.72504e-03_rb,0.74630e-03_rb,0.76495e-03_rb,0.78165e-03_rb /)
      kbo(:,17, 5) = (/ &
        & 0.56280e-03_rb,0.58321e-03_rb,0.60153e-03_rb,0.61544e-03_rb,0.62885e-03_rb /)
      kbo(:,18, 5) = (/ &
        & 0.44940e-03_rb,0.46598e-03_rb,0.47887e-03_rb,0.49058e-03_rb,0.50223e-03_rb /)
      kbo(:,19, 5) = (/ &
        & 0.35753e-03_rb,0.37064e-03_rb,0.38290e-03_rb,0.39390e-03_rb,0.40387e-03_rb /)
      kbo(:,20, 5) = (/ &
        & 0.29013e-03_rb,0.30099e-03_rb,0.31179e-03_rb,0.32117e-03_rb,0.33011e-03_rb /)
      kbo(:,21, 5) = (/ &
        & 0.23657e-03_rb,0.24602e-03_rb,0.25504e-03_rb,0.26340e-03_rb,0.27145e-03_rb /)
      kbo(:,22, 5) = (/ &
        & 0.19423e-03_rb,0.20224e-03_rb,0.20987e-03_rb,0.21711e-03_rb,0.22439e-03_rb /)
      kbo(:,23, 5) = (/ &
        & 0.15963e-03_rb,0.16643e-03_rb,0.17308e-03_rb,0.17949e-03_rb,0.18580e-03_rb /)
      kbo(:,24, 5) = (/ &
        & 0.13166e-03_rb,0.13745e-03_rb,0.14332e-03_rb,0.14871e-03_rb,0.15428e-03_rb /)
      kbo(:,25, 5) = (/ &
        & 0.10885e-03_rb,0.11394e-03_rb,0.11896e-03_rb,0.12371e-03_rb,0.12846e-03_rb /)
      kbo(:,26, 5) = (/ &
        & 0.90403e-04_rb,0.94914e-04_rb,0.99157e-04_rb,0.10310e-03_rb,0.10729e-03_rb /)
      kbo(:,27, 5) = (/ &
        & 0.75265e-04_rb,0.79166e-04_rb,0.82648e-04_rb,0.86209e-04_rb,0.89882e-04_rb /)
      kbo(:,28, 5) = (/ &
        & 0.62870e-04_rb,0.66050e-04_rb,0.69111e-04_rb,0.72289e-04_rb,0.75451e-04_rb /)
      kbo(:,29, 5) = (/ &
        & 0.52465e-04_rb,0.55192e-04_rb,0.57887e-04_rb,0.60660e-04_rb,0.63367e-04_rb /)
      kbo(:,30, 5) = (/ &
        & 0.43887e-04_rb,0.46191e-04_rb,0.48588e-04_rb,0.50910e-04_rb,0.53278e-04_rb /)
      kbo(:,31, 5) = (/ &
        & 0.36760e-04_rb,0.38766e-04_rb,0.40783e-04_rb,0.42783e-04_rb,0.44780e-04_rb /)
      kbo(:,32, 5) = (/ &
        & 0.30831e-04_rb,0.32561e-04_rb,0.34267e-04_rb,0.36000e-04_rb,0.37594e-04_rb /)
      kbo(:,33, 5) = (/ &
        & 0.25908e-04_rb,0.27389e-04_rb,0.28832e-04_rb,0.30290e-04_rb,0.31581e-04_rb /)
      kbo(:,34, 5) = (/ &
        & 0.21776e-04_rb,0.23026e-04_rb,0.24266e-04_rb,0.25444e-04_rb,0.26510e-04_rb /)
      kbo(:,35, 5) = (/ &
        & 0.18233e-04_rb,0.19280e-04_rb,0.20324e-04_rb,0.21266e-04_rb,0.22141e-04_rb /)
      kbo(:,36, 5) = (/ &
        & 0.15160e-04_rb,0.16042e-04_rb,0.16918e-04_rb,0.17686e-04_rb,0.18406e-04_rb /)
      kbo(:,37, 5) = (/ &
        & 0.12538e-04_rb,0.13285e-04_rb,0.14015e-04_rb,0.14657e-04_rb,0.15269e-04_rb /)
      kbo(:,38, 5) = (/ &
        & 0.10357e-04_rb,0.10990e-04_rb,0.11595e-04_rb,0.12148e-04_rb,0.12651e-04_rb /)
      kbo(:,39, 5) = (/ &
        & 0.85566e-05_rb,0.90856e-05_rb,0.95986e-05_rb,0.10057e-04_rb,0.10478e-04_rb /)
      kbo(:,40, 5) = (/ &
        & 0.70390e-05_rb,0.74826e-05_rb,0.79200e-05_rb,0.83061e-05_rb,0.86634e-05_rb /)
      kbo(:,41, 5) = (/ &
        & 0.57827e-05_rb,0.61572e-05_rb,0.65269e-05_rb,0.68528e-05_rb,0.71595e-05_rb /)
      kbo(:,42, 5) = (/ &
        & 0.47477e-05_rb,0.50656e-05_rb,0.53705e-05_rb,0.56486e-05_rb,0.59100e-05_rb /)
      kbo(:,43, 5) = (/ &
        & 0.38802e-05_rb,0.41533e-05_rb,0.44089e-05_rb,0.46515e-05_rb,0.48719e-05_rb /)
      kbo(:,44, 5) = (/ &
        & 0.31658e-05_rb,0.33971e-05_rb,0.36177e-05_rb,0.38205e-05_rb,0.40089e-05_rb /)
      kbo(:,45, 5) = (/ &
        & 0.25796e-05_rb,0.27740e-05_rb,0.29635e-05_rb,0.31336e-05_rb,0.32986e-05_rb /)
      kbo(:,46, 5) = (/ &
        & 0.20945e-05_rb,0.22636e-05_rb,0.24214e-05_rb,0.25698e-05_rb,0.27077e-05_rb /)
      kbo(:,47, 5) = (/ &
        & 0.16931e-05_rb,0.18383e-05_rb,0.19729e-05_rb,0.20996e-05_rb,0.22172e-05_rb /)
      kbo(:,48, 5) = (/ &
        & 0.13650e-05_rb,0.14889e-05_rb,0.16043e-05_rb,0.17139e-05_rb,0.18136e-05_rb /)
      kbo(:,49, 5) = (/ &
        & 0.10981e-05_rb,0.12042e-05_rb,0.13028e-05_rb,0.13963e-05_rb,0.14824e-05_rb /)
      kbo(:,50, 5) = (/ &
        & 0.88407e-06_rb,0.97370e-06_rb,0.10589e-05_rb,0.11383e-05_rb,0.12124e-05_rb /)
      kbo(:,51, 5) = (/ &
        & 0.71150e-06_rb,0.78682e-06_rb,0.86048e-06_rb,0.92711e-06_rb,0.99128e-06_rb /)
      kbo(:,52, 5) = (/ &
        & 0.57140e-06_rb,0.63513e-06_rb,0.69790e-06_rb,0.75466e-06_rb,0.80946e-06_rb /)
      kbo(:,53, 5) = (/ &
        & 0.45871e-06_rb,0.51183e-06_rb,0.56494e-06_rb,0.61376e-06_rb,0.66022e-06_rb /)
      kbo(:,54, 5) = (/ &
        & 0.36889e-06_rb,0.41319e-06_rb,0.45759e-06_rb,0.49973e-06_rb,0.53933e-06_rb /)
      kbo(:,55, 5) = (/ &
        & 0.29683e-06_rb,0.33396e-06_rb,0.37110e-06_rb,0.40671e-06_rb,0.44026e-06_rb /)
      kbo(:,56, 5) = (/ &
        & 0.23828e-06_rb,0.26973e-06_rb,0.30048e-06_rb,0.33099e-06_rb,0.35949e-06_rb /)
      kbo(:,57, 5) = (/ &
        & 0.19083e-06_rb,0.21782e-06_rb,0.24350e-06_rb,0.26883e-06_rb,0.29309e-06_rb /)
      kbo(:,58, 5) = (/ &
        & 0.15334e-06_rb,0.17592e-06_rb,0.19753e-06_rb,0.21869e-06_rb,0.23903e-06_rb /)
      kbo(:,59, 5) = (/ &
        & 0.12635e-06_rb,0.14504e-06_rb,0.16321e-06_rb,0.18074e-06_rb,0.19759e-06_rb /)
      kbo(:,13, 6) = (/ &
        & 0.33433e-02_rb,0.34159e-02_rb,0.34798e-02_rb,0.35397e-02_rb,0.35995e-02_rb /)
      kbo(:,14, 6) = (/ &
        & 0.28039e-02_rb,0.28589e-02_rb,0.29090e-02_rb,0.29595e-02_rb,0.30108e-02_rb /)
      kbo(:,15, 6) = (/ &
        & 0.23135e-02_rb,0.23579e-02_rb,0.24012e-02_rb,0.24470e-02_rb,0.24900e-02_rb /)
      kbo(:,16, 6) = (/ &
        & 0.18925e-02_rb,0.19281e-02_rb,0.19671e-02_rb,0.20090e-02_rb,0.20445e-02_rb /)
      kbo(:,17, 6) = (/ &
        & 0.15365e-02_rb,0.15703e-02_rb,0.16039e-02_rb,0.16424e-02_rb,0.16760e-02_rb /)
      kbo(:,18, 6) = (/ &
        & 0.12424e-02_rb,0.12746e-02_rb,0.13067e-02_rb,0.13397e-02_rb,0.13687e-02_rb /)
      kbo(:,19, 6) = (/ &
        & 0.10113e-02_rb,0.10417e-02_rb,0.10664e-02_rb,0.10958e-02_rb,0.11231e-02_rb /)
      kbo(:,20, 6) = (/ &
        & 0.83375e-03_rb,0.85863e-03_rb,0.88098e-03_rb,0.90680e-03_rb,0.93093e-03_rb /)
      kbo(:,21, 6) = (/ &
        & 0.68849e-03_rb,0.71056e-03_rb,0.73161e-03_rb,0.75402e-03_rb,0.77432e-03_rb /)
      kbo(:,22, 6) = (/ &
        & 0.57076e-03_rb,0.59001e-03_rb,0.60925e-03_rb,0.62816e-03_rb,0.64531e-03_rb /)
      kbo(:,23, 6) = (/ &
        & 0.47462e-03_rb,0.49138e-03_rb,0.50818e-03_rb,0.52421e-03_rb,0.53930e-03_rb /)
      kbo(:,24, 6) = (/ &
        & 0.39552e-03_rb,0.41011e-03_rb,0.42460e-03_rb,0.43865e-03_rb,0.45206e-03_rb /)
      kbo(:,25, 6) = (/ &
        & 0.33041e-03_rb,0.34317e-03_rb,0.35573e-03_rb,0.36800e-03_rb,0.38005e-03_rb /)
      kbo(:,26, 6) = (/ &
        & 0.27699e-03_rb,0.28806e-03_rb,0.29892e-03_rb,0.30983e-03_rb,0.32038e-03_rb /)
      kbo(:,27, 6) = (/ &
        & 0.23253e-03_rb,0.24229e-03_rb,0.25208e-03_rb,0.26144e-03_rb,0.27039e-03_rb /)
      kbo(:,28, 6) = (/ &
        & 0.19573e-03_rb,0.20442e-03_rb,0.21276e-03_rb,0.22087e-03_rb,0.22869e-03_rb /)
      kbo(:,29, 6) = (/ &
        & 0.16528e-03_rb,0.17271e-03_rb,0.17974e-03_rb,0.18681e-03_rb,0.19373e-03_rb /)
      kbo(:,30, 6) = (/ &
        & 0.13949e-03_rb,0.14599e-03_rb,0.15214e-03_rb,0.15834e-03_rb,0.16448e-03_rb /)
      kbo(:,31, 6) = (/ &
        & 0.11793e-03_rb,0.12345e-03_rb,0.12891e-03_rb,0.13432e-03_rb,0.13975e-03_rb /)
      kbo(:,32, 6) = (/ &
        & 0.99845e-04_rb,0.10464e-03_rb,0.10941e-03_rb,0.11425e-03_rb,0.11896e-03_rb /)
      kbo(:,33, 6) = (/ &
        & 0.84723e-04_rb,0.88808e-04_rb,0.93054e-04_rb,0.97257e-04_rb,0.10144e-03_rb /)
      kbo(:,34, 6) = (/ &
        & 0.71857e-04_rb,0.75478e-04_rb,0.79245e-04_rb,0.82878e-04_rb,0.86527e-04_rb /)
      kbo(:,35, 6) = (/ &
        & 0.60741e-04_rb,0.63987e-04_rb,0.67190e-04_rb,0.70445e-04_rb,0.73634e-04_rb /)
      kbo(:,36, 6) = (/ &
        & 0.51094e-04_rb,0.53953e-04_rb,0.56787e-04_rb,0.59644e-04_rb,0.62421e-04_rb /)
      kbo(:,37, 6) = (/ &
        & 0.42809e-04_rb,0.45324e-04_rb,0.47805e-04_rb,0.50314e-04_rb,0.52797e-04_rb /)
      kbo(:,38, 6) = (/ &
        & 0.35862e-04_rb,0.38082e-04_rb,0.40249e-04_rb,0.42459e-04_rb,0.44691e-04_rb /)
      kbo(:,39, 6) = (/ &
        & 0.30047e-04_rb,0.31970e-04_rb,0.33877e-04_rb,0.35841e-04_rb,0.37792e-04_rb /)
      kbo(:,40, 6) = (/ &
        & 0.25051e-04_rb,0.26734e-04_rb,0.28429e-04_rb,0.30177e-04_rb,0.31859e-04_rb /)
      kbo(:,41, 6) = (/ &
        & 0.20858e-04_rb,0.22337e-04_rb,0.23837e-04_rb,0.25348e-04_rb,0.26845e-04_rb /)
      kbo(:,42, 6) = (/ &
        & 0.17358e-04_rb,0.18653e-04_rb,0.19947e-04_rb,0.21279e-04_rb,0.22614e-04_rb /)
      kbo(:,43, 6) = (/ &
        & 0.14382e-04_rb,0.15507e-04_rb,0.16637e-04_rb,0.17808e-04_rb,0.18975e-04_rb /)
      kbo(:,44, 6) = (/ &
        & 0.11875e-04_rb,0.12851e-04_rb,0.13847e-04_rb,0.14865e-04_rb,0.15886e-04_rb /)
      kbo(:,45, 6) = (/ &
        & 0.97824e-05_rb,0.10641e-04_rb,0.11497e-04_rb,0.12386e-04_rb,0.13288e-04_rb /)
      kbo(:,46, 6) = (/ &
        & 0.80305e-05_rb,0.87712e-05_rb,0.95170e-05_rb,0.10282e-04_rb,0.11073e-04_rb /)
      kbo(:,47, 6) = (/ &
        & 0.65554e-05_rb,0.71881e-05_rb,0.78331e-05_rb,0.85022e-05_rb,0.91799e-05_rb /)
      kbo(:,48, 6) = (/ &
        & 0.53399e-05_rb,0.58818e-05_rb,0.64364e-05_rb,0.70093e-05_rb,0.75992e-05_rb /)
      kbo(:,49, 6) = (/ &
        & 0.43396e-05_rb,0.47990e-05_rb,0.52727e-05_rb,0.57633e-05_rb,0.62785e-05_rb /)
      kbo(:,50, 6) = (/ &
        & 0.35272e-05_rb,0.39139e-05_rb,0.43230e-05_rb,0.47423e-05_rb,0.51819e-05_rb /)
      kbo(:,51, 6) = (/ &
        & 0.28624e-05_rb,0.31913e-05_rb,0.35379e-05_rb,0.38979e-05_rb,0.42744e-05_rb /)
      kbo(:,52, 6) = (/ &
        & 0.23189e-05_rb,0.25961e-05_rb,0.28907e-05_rb,0.31976e-05_rb,0.35200e-05_rb /)
      kbo(:,53, 6) = (/ &
        & 0.18736e-05_rb,0.21072e-05_rb,0.23576e-05_rb,0.26179e-05_rb,0.28943e-05_rb /)
      kbo(:,54, 6) = (/ &
        & 0.15155e-05_rb,0.17135e-05_rb,0.19245e-05_rb,0.21451e-05_rb,0.23808e-05_rb /)
      kbo(:,55, 6) = (/ &
        & 0.12250e-05_rb,0.13923e-05_rb,0.15714e-05_rb,0.17578e-05_rb,0.19597e-05_rb /)
      kbo(:,56, 6) = (/ &
        & 0.99012e-06_rb,0.11296e-05_rb,0.12801e-05_rb,0.14395e-05_rb,0.16107e-05_rb /)
      kbo(:,57, 6) = (/ &
        & 0.79875e-06_rb,0.91418e-06_rb,0.10408e-05_rb,0.11777e-05_rb,0.13212e-05_rb /)
      kbo(:,58, 6) = (/ &
        & 0.64474e-06_rb,0.74109e-06_rb,0.84664e-06_rb,0.96328e-06_rb,0.10849e-05_rb /)
      kbo(:,59, 6) = (/ &
        & 0.53296e-06_rb,0.61427e-06_rb,0.70385e-06_rb,0.80375e-06_rb,0.90863e-06_rb /)
      kbo(:,13, 7) = (/ &
        & 0.91063e-02_rb,0.92681e-02_rb,0.94045e-02_rb,0.95388e-02_rb,0.96735e-02_rb /)
      kbo(:,14, 7) = (/ &
        & 0.78483e-02_rb,0.79806e-02_rb,0.81238e-02_rb,0.82528e-02_rb,0.83603e-02_rb /)
      kbo(:,15, 7) = (/ &
        & 0.66269e-02_rb,0.67445e-02_rb,0.68845e-02_rb,0.69911e-02_rb,0.70854e-02_rb /)
      kbo(:,16, 7) = (/ &
        & 0.55183e-02_rb,0.56414e-02_rb,0.57574e-02_rb,0.58487e-02_rb,0.59407e-02_rb /)
      kbo(:,17, 7) = (/ &
        & 0.45693e-02_rb,0.46780e-02_rb,0.47743e-02_rb,0.48625e-02_rb,0.49467e-02_rb /)
      kbo(:,18, 7) = (/ &
        & 0.37611e-02_rb,0.38535e-02_rb,0.39408e-02_rb,0.40234e-02_rb,0.41007e-02_rb /)
      kbo(:,19, 7) = (/ &
        & 0.30991e-02_rb,0.31809e-02_rb,0.32647e-02_rb,0.33388e-02_rb,0.34042e-02_rb /)
      kbo(:,20, 7) = (/ &
        & 0.25858e-02_rb,0.26585e-02_rb,0.27347e-02_rb,0.27990e-02_rb,0.28579e-02_rb /)
      kbo(:,21, 7) = (/ &
        & 0.21672e-02_rb,0.22326e-02_rb,0.22973e-02_rb,0.23547e-02_rb,0.24058e-02_rb /)
      kbo(:,22, 7) = (/ &
        & 0.18211e-02_rb,0.18809e-02_rb,0.19364e-02_rb,0.19859e-02_rb,0.20352e-02_rb /)
      kbo(:,23, 7) = (/ &
        & 0.15314e-02_rb,0.15844e-02_rb,0.16332e-02_rb,0.16760e-02_rb,0.17243e-02_rb /)
      kbo(:,24, 7) = (/ &
        & 0.12924e-02_rb,0.13385e-02_rb,0.13793e-02_rb,0.14209e-02_rb,0.14659e-02_rb /)
      kbo(:,25, 7) = (/ &
        & 0.10926e-02_rb,0.11322e-02_rb,0.11689e-02_rb,0.12081e-02_rb,0.12493e-02_rb /)
      kbo(:,26, 7) = (/ &
        & 0.92756e-03_rb,0.96136e-03_rb,0.99601e-03_rb,0.10324e-02_rb,0.10695e-02_rb /)
      kbo(:,27, 7) = (/ &
        & 0.78832e-03_rb,0.81893e-03_rb,0.85137e-03_rb,0.88465e-03_rb,0.91885e-03_rb /)
      kbo(:,28, 7) = (/ &
        & 0.67097e-03_rb,0.69979e-03_rb,0.72985e-03_rb,0.76038e-03_rb,0.79213e-03_rb /)
      kbo(:,29, 7) = (/ &
        & 0.57231e-03_rb,0.59934e-03_rb,0.62727e-03_rb,0.65471e-03_rb,0.68389e-03_rb /)
      kbo(:,30, 7) = (/ &
        & 0.48996e-03_rb,0.51441e-03_rb,0.53945e-03_rb,0.56564e-03_rb,0.59164e-03_rb /)
      kbo(:,31, 7) = (/ &
        & 0.41998e-03_rb,0.44237e-03_rb,0.46551e-03_rb,0.48924e-03_rb,0.51322e-03_rb /)
      kbo(:,32, 7) = (/ &
        & 0.36133e-03_rb,0.38168e-03_rb,0.40297e-03_rb,0.42441e-03_rb,0.44722e-03_rb /)
      kbo(:,33, 7) = (/ &
        & 0.31153e-03_rb,0.33055e-03_rb,0.34980e-03_rb,0.36972e-03_rb,0.39086e-03_rb /)
      kbo(:,34, 7) = (/ &
        & 0.26941e-03_rb,0.28657e-03_rb,0.30436e-03_rb,0.32295e-03_rb,0.34256e-03_rb /)
      kbo(:,35, 7) = (/ &
        & 0.23210e-03_rb,0.24782e-03_rb,0.26411e-03_rb,0.28121e-03_rb,0.29930e-03_rb /)
      kbo(:,36, 7) = (/ &
        & 0.19904e-03_rb,0.21311e-03_rb,0.22797e-03_rb,0.24380e-03_rb,0.26030e-03_rb /)
      kbo(:,37, 7) = (/ &
        & 0.17000e-03_rb,0.18300e-03_rb,0.19666e-03_rb,0.21128e-03_rb,0.22641e-03_rb /)
      kbo(:,38, 7) = (/ &
        & 0.14528e-03_rb,0.15704e-03_rb,0.16975e-03_rb,0.18320e-03_rb,0.19735e-03_rb /)
      kbo(:,39, 7) = (/ &
        & 0.12414e-03_rb,0.13496e-03_rb,0.14654e-03_rb,0.15899e-03_rb,0.17211e-03_rb /)
      kbo(:,40, 7) = (/ &
        & 0.10567e-03_rb,0.11547e-03_rb,0.12618e-03_rb,0.13765e-03_rb,0.14994e-03_rb /)
      kbo(:,41, 7) = (/ &
        & 0.89774e-04_rb,0.98751e-04_rb,0.10852e-03_rb,0.11918e-03_rb,0.13057e-03_rb /)
      kbo(:,42, 7) = (/ &
        & 0.76232e-04_rb,0.84383e-04_rb,0.93336e-04_rb,0.10314e-03_rb,0.11381e-03_rb /)
      kbo(:,43, 7) = (/ &
        & 0.64370e-04_rb,0.71684e-04_rb,0.79797e-04_rb,0.88850e-04_rb,0.98733e-04_rb /)
      kbo(:,44, 7) = (/ &
        & 0.54125e-04_rb,0.60680e-04_rb,0.68042e-04_rb,0.76313e-04_rb,0.85432e-04_rb /)
      kbo(:,45, 7) = (/ &
        & 0.45374e-04_rb,0.51280e-04_rb,0.57932e-04_rb,0.65429e-04_rb,0.73852e-04_rb /)
      kbo(:,46, 7) = (/ &
        & 0.37861e-04_rb,0.43078e-04_rb,0.49016e-04_rb,0.55844e-04_rb,0.63538e-04_rb /)
      kbo(:,47, 7) = (/ &
        & 0.31342e-04_rb,0.35879e-04_rb,0.41175e-04_rb,0.47252e-04_rb,0.54286e-04_rb /)
      kbo(:,48, 7) = (/ &
        & 0.25821e-04_rb,0.29777e-04_rb,0.34458e-04_rb,0.39880e-04_rb,0.46218e-04_rb /)
      kbo(:,49, 7) = (/ &
        & 0.21180e-04_rb,0.24617e-04_rb,0.28719e-04_rb,0.33507e-04_rb,0.39184e-04_rb /)
      kbo(:,50, 7) = (/ &
        & 0.17377e-04_rb,0.20345e-04_rb,0.23916e-04_rb,0.28177e-04_rb,0.33237e-04_rb /)
      kbo(:,51, 7) = (/ &
        & 0.14223e-04_rb,0.16790e-04_rb,0.19868e-04_rb,0.23656e-04_rb,0.28163e-04_rb /)
      kbo(:,52, 7) = (/ &
        & 0.11593e-04_rb,0.13788e-04_rb,0.16479e-04_rb,0.19773e-04_rb,0.23765e-04_rb /)
      kbo(:,53, 7) = (/ &
        & 0.94141e-05_rb,0.11279e-04_rb,0.13591e-04_rb,0.16447e-04_rb,0.19969e-04_rb /)
      kbo(:,54, 7) = (/ &
        & 0.76622e-05_rb,0.92417e-05_rb,0.11233e-04_rb,0.13709e-04_rb,0.16819e-04_rb /)
      kbo(:,55, 7) = (/ &
        & 0.62315e-05_rb,0.75671e-05_rb,0.92699e-05_rb,0.11429e-04_rb,0.14186e-04_rb /)
      kbo(:,56, 7) = (/ &
        & 0.50473e-05_rb,0.61766e-05_rb,0.76316e-05_rb,0.94979e-05_rb,0.11890e-04_rb /)
      kbo(:,57, 7) = (/ &
        & 0.40773e-05_rb,0.50242e-05_rb,0.62540e-05_rb,0.78594e-05_rb,0.99378e-05_rb /)
      kbo(:,58, 7) = (/ &
        & 0.32937e-05_rb,0.40862e-05_rb,0.51228e-05_rb,0.64992e-05_rb,0.83112e-05_rb /)
      kbo(:,59, 7) = (/ &
        & 0.27593e-05_rb,0.34505e-05_rb,0.43716e-05_rb,0.56035e-05_rb,0.72609e-05_rb /)
      kbo(:,13, 8) = (/ &
        & 0.25990e-01_rb,0.25997e-01_rb,0.26024e-01_rb,0.26038e-01_rb,0.26132e-01_rb /)
      kbo(:,14, 8) = (/ &
        & 0.22648e-01_rb,0.22731e-01_rb,0.22803e-01_rb,0.22861e-01_rb,0.23020e-01_rb /)
      kbo(:,15, 8) = (/ &
        & 0.19641e-01_rb,0.19812e-01_rb,0.19880e-01_rb,0.19998e-01_rb,0.20141e-01_rb /)
      kbo(:,16, 8) = (/ &
        & 0.16909e-01_rb,0.17030e-01_rb,0.17148e-01_rb,0.17296e-01_rb,0.17502e-01_rb /)
      kbo(:,17, 8) = (/ &
        & 0.14365e-01_rb,0.14504e-01_rb,0.14643e-01_rb,0.14808e-01_rb,0.14996e-01_rb /)
      kbo(:,18, 8) = (/ &
        & 0.12072e-01_rb,0.12207e-01_rb,0.12346e-01_rb,0.12513e-01_rb,0.12742e-01_rb /)
      kbo(:,19, 8) = (/ &
        & 0.10135e-01_rb,0.10263e-01_rb,0.10402e-01_rb,0.10602e-01_rb,0.10794e-01_rb /)
      kbo(:,20, 8) = (/ &
        & 0.86508e-02_rb,0.87951e-02_rb,0.89765e-02_rb,0.91530e-02_rb,0.93447e-02_rb /)
      kbo(:,21, 8) = (/ &
        & 0.74228e-02_rb,0.75964e-02_rb,0.77805e-02_rb,0.79441e-02_rb,0.81266e-02_rb /)
      kbo(:,22, 8) = (/ &
        & 0.63949e-02_rb,0.65920e-02_rb,0.67584e-02_rb,0.69298e-02_rb,0.70968e-02_rb /)
      kbo(:,23, 8) = (/ &
        & 0.55389e-02_rb,0.57165e-02_rb,0.58834e-02_rb,0.60490e-02_rb,0.62163e-02_rb /)
      kbo(:,24, 8) = (/ &
        & 0.47980e-02_rb,0.49696e-02_rb,0.51333e-02_rb,0.53055e-02_rb,0.54695e-02_rb /)
      kbo(:,25, 8) = (/ &
        & 0.41676e-02_rb,0.43357e-02_rb,0.44999e-02_rb,0.46749e-02_rb,0.48179e-02_rb /)
      kbo(:,26, 8) = (/ &
        & 0.36391e-02_rb,0.38079e-02_rb,0.39834e-02_rb,0.41374e-02_rb,0.42754e-02_rb /)
      kbo(:,27, 8) = (/ &
        & 0.31917e-02_rb,0.33580e-02_rb,0.35237e-02_rb,0.36737e-02_rb,0.37982e-02_rb /)
      kbo(:,28, 8) = (/ &
        & 0.28096e-02_rb,0.29709e-02_rb,0.31242e-02_rb,0.32613e-02_rb,0.33829e-02_rb /)
      kbo(:,29, 8) = (/ &
        & 0.24786e-02_rb,0.26272e-02_rb,0.27706e-02_rb,0.28986e-02_rb,0.30130e-02_rb /)
      kbo(:,30, 8) = (/ &
        & 0.21907e-02_rb,0.23282e-02_rb,0.24574e-02_rb,0.25778e-02_rb,0.26806e-02_rb /)
      kbo(:,31, 8) = (/ &
        & 0.19346e-02_rb,0.20569e-02_rb,0.21757e-02_rb,0.22805e-02_rb,0.23742e-02_rb /)
      kbo(:,32, 8) = (/ &
        & 0.17101e-02_rb,0.18247e-02_rb,0.19308e-02_rb,0.20256e-02_rb,0.21176e-02_rb /)
      kbo(:,33, 8) = (/ &
        & 0.15171e-02_rb,0.16168e-02_rb,0.17121e-02_rb,0.18035e-02_rb,0.18911e-02_rb /)
      kbo(:,34, 8) = (/ &
        & 0.13466e-02_rb,0.14390e-02_rb,0.15246e-02_rb,0.16083e-02_rb,0.16921e-02_rb /)
      kbo(:,35, 8) = (/ &
        & 0.11871e-02_rb,0.12698e-02_rb,0.13534e-02_rb,0.14320e-02_rb,0.15071e-02_rb /)
      kbo(:,36, 8) = (/ &
        & 0.10388e-02_rb,0.11179e-02_rb,0.11934e-02_rb,0.12647e-02_rb,0.13391e-02_rb /)
      kbo(:,37, 8) = (/ &
        & 0.92233e-03_rb,0.99854e-03_rb,0.10722e-02_rb,0.11426e-02_rb,0.12118e-02_rb /)
      kbo(:,38, 8) = (/ &
        & 0.81843e-03_rb,0.89236e-03_rb,0.96605e-03_rb,0.10340e-02_rb,0.11003e-02_rb /)
      kbo(:,39, 8) = (/ &
        & 0.72738e-03_rb,0.79777e-03_rb,0.86997e-03_rb,0.93496e-03_rb,0.99852e-03_rb /)
      kbo(:,40, 8) = (/ &
        & 0.65149e-03_rb,0.72098e-03_rb,0.79235e-03_rb,0.86116e-03_rb,0.92474e-03_rb /)
      kbo(:,41, 8) = (/ &
        & 0.58318e-03_rb,0.65154e-03_rb,0.72160e-03_rb,0.79400e-03_rb,0.85973e-03_rb /)
      kbo(:,42, 8) = (/ &
        & 0.52127e-03_rb,0.58770e-03_rb,0.65781e-03_rb,0.73110e-03_rb,0.80165e-03_rb /)
      kbo(:,43, 8) = (/ &
        & 0.46404e-03_rb,0.52960e-03_rb,0.59886e-03_rb,0.67366e-03_rb,0.74727e-03_rb /)
      kbo(:,44, 8) = (/ &
        & 0.41229e-03_rb,0.47609e-03_rb,0.54424e-03_rb,0.61918e-03_rb,0.69366e-03_rb /)
      kbo(:,45, 8) = (/ &
        & 0.36534e-03_rb,0.42624e-03_rb,0.49374e-03_rb,0.56668e-03_rb,0.64364e-03_rb /)
      kbo(:,46, 8) = (/ &
        & 0.32134e-03_rb,0.37987e-03_rb,0.44597e-03_rb,0.51669e-03_rb,0.59469e-03_rb /)
      kbo(:,47, 8) = (/ &
        & 0.27855e-03_rb,0.33531e-03_rb,0.39855e-03_rb,0.46832e-03_rb,0.54562e-03_rb /)
      kbo(:,48, 8) = (/ &
        & 0.24094e-03_rb,0.29440e-03_rb,0.35477e-03_rb,0.42284e-03_rb,0.49767e-03_rb /)
      kbo(:,49, 8) = (/ &
        & 0.20710e-03_rb,0.25712e-03_rb,0.31425e-03_rb,0.38098e-03_rb,0.45465e-03_rb /)
      kbo(:,50, 8) = (/ &
        & 0.17757e-03_rb,0.22422e-03_rb,0.27906e-03_rb,0.34231e-03_rb,0.41419e-03_rb /)
      kbo(:,51, 8) = (/ &
        & 0.15198e-03_rb,0.19538e-03_rb,0.24758e-03_rb,0.30796e-03_rb,0.37765e-03_rb /)
      kbo(:,52, 8) = (/ &
        & 0.12943e-03_rb,0.16907e-03_rb,0.21812e-03_rb,0.27585e-03_rb,0.34284e-03_rb /)
      kbo(:,53, 8) = (/ &
        & 0.10946e-03_rb,0.14560e-03_rb,0.19103e-03_rb,0.24633e-03_rb,0.31049e-03_rb /)
      kbo(:,54, 8) = (/ &
        & 0.92950e-04_rb,0.12599e-03_rb,0.16848e-03_rb,0.22109e-03_rb,0.28266e-03_rb /)
      kbo(:,55, 8) = (/ &
        & 0.78722e-04_rb,0.10883e-03_rb,0.14831e-03_rb,0.19790e-03_rb,0.25770e-03_rb /)
      kbo(:,56, 8) = (/ &
        & 0.66323e-04_rb,0.93651e-04_rb,0.12983e-03_rb,0.17638e-03_rb,0.23393e-03_rb /)
      kbo(:,57, 8) = (/ &
        & 0.55644e-04_rb,0.80085e-04_rb,0.11349e-03_rb,0.15741e-03_rb,0.21274e-03_rb /)
      kbo(:,58, 8) = (/ &
        & 0.46537e-04_rb,0.68442e-04_rb,0.99190e-04_rb,0.14045e-03_rb,0.19327e-03_rb /)
      kbo(:,59, 8) = (/ &
        & 0.41558e-04_rb,0.62503e-04_rb,0.92377e-04_rb,0.13258e-03_rb,0.18528e-03_rb /)
      kbo(:,13, 9) = (/ &
        & 0.14836e+00_rb,0.15039e+00_rb,0.15239e+00_rb,0.15389e+00_rb,0.15517e+00_rb /)
      kbo(:,14, 9) = (/ &
        & 0.12935e+00_rb,0.13183e+00_rb,0.13366e+00_rb,0.13551e+00_rb,0.13674e+00_rb /)
      kbo(:,15, 9) = (/ &
        & 0.11269e+00_rb,0.11456e+00_rb,0.11672e+00_rb,0.11824e+00_rb,0.11951e+00_rb /)
      kbo(:,16, 9) = (/ &
        & 0.97577e-01_rb,0.99647e-01_rb,0.10140e+00_rb,0.10272e+00_rb,0.10390e+00_rb /)
      kbo(:,17, 9) = (/ &
        & 0.84601e-01_rb,0.86370e-01_rb,0.87860e-01_rb,0.89144e-01_rb,0.90341e-01_rb /)
      kbo(:,18, 9) = (/ &
        & 0.72851e-01_rb,0.74498e-01_rb,0.75924e-01_rb,0.77220e-01_rb,0.78434e-01_rb /)
      kbo(:,19, 9) = (/ &
        & 0.62688e-01_rb,0.64121e-01_rb,0.65574e-01_rb,0.66965e-01_rb,0.68243e-01_rb /)
      kbo(:,20, 9) = (/ &
        & 0.54382e-01_rb,0.55704e-01_rb,0.57013e-01_rb,0.58303e-01_rb,0.59763e-01_rb /)
      kbo(:,21, 9) = (/ &
        & 0.47190e-01_rb,0.48469e-01_rb,0.49752e-01_rb,0.51138e-01_rb,0.52452e-01_rb /)
      kbo(:,22, 9) = (/ &
        & 0.41104e-01_rb,0.42378e-01_rb,0.43684e-01_rb,0.45035e-01_rb,0.46495e-01_rb /)
      kbo(:,23, 9) = (/ &
        & 0.35910e-01_rb,0.37220e-01_rb,0.38484e-01_rb,0.39789e-01_rb,0.41443e-01_rb /)
      kbo(:,24, 9) = (/ &
        & 0.31558e-01_rb,0.32836e-01_rb,0.34085e-01_rb,0.35462e-01_rb,0.37160e-01_rb /)
      kbo(:,25, 9) = (/ &
        & 0.27861e-01_rb,0.29090e-01_rb,0.30363e-01_rb,0.31857e-01_rb,0.33566e-01_rb /)
      kbo(:,26, 9) = (/ &
        & 0.24743e-01_rb,0.26003e-01_rb,0.27311e-01_rb,0.28788e-01_rb,0.30553e-01_rb /)
      kbo(:,27, 9) = (/ &
        & 0.22141e-01_rb,0.23397e-01_rb,0.24754e-01_rb,0.26282e-01_rb,0.28040e-01_rb /)
      kbo(:,28, 9) = (/ &
        & 0.19941e-01_rb,0.21165e-01_rb,0.22565e-01_rb,0.24131e-01_rb,0.25980e-01_rb /)
      kbo(:,29, 9) = (/ &
        & 0.18046e-01_rb,0.19353e-01_rb,0.20736e-01_rb,0.22375e-01_rb,0.24248e-01_rb /)
      kbo(:,30, 9) = (/ &
        & 0.16429e-01_rb,0.17775e-01_rb,0.19238e-01_rb,0.20907e-01_rb,0.22847e-01_rb /)
      kbo(:,31, 9) = (/ &
        & 0.15094e-01_rb,0.16450e-01_rb,0.17992e-01_rb,0.19741e-01_rb,0.21608e-01_rb /)
      kbo(:,32, 9) = (/ &
        & 0.13975e-01_rb,0.15315e-01_rb,0.16950e-01_rb,0.18741e-01_rb,0.20564e-01_rb /)
      kbo(:,33, 9) = (/ &
        & 0.13026e-01_rb,0.14475e-01_rb,0.16113e-01_rb,0.17903e-01_rb,0.19794e-01_rb /)
      kbo(:,34, 9) = (/ &
        & 0.12248e-01_rb,0.13747e-01_rb,0.15391e-01_rb,0.17142e-01_rb,0.19155e-01_rb /)
      kbo(:,35, 9) = (/ &
        & 0.11491e-01_rb,0.12979e-01_rb,0.14611e-01_rb,0.16423e-01_rb,0.18475e-01_rb /)
      kbo(:,36, 9) = (/ &
        & 0.10695e-01_rb,0.12142e-01_rb,0.13822e-01_rb,0.15678e-01_rb,0.17729e-01_rb /)
      kbo(:,37, 9) = (/ &
        & 0.98761e-02_rb,0.11300e-01_rb,0.13000e-01_rb,0.14808e-01_rb,0.16848e-01_rb /)
      kbo(:,38, 9) = (/ &
        & 0.91358e-02_rb,0.10538e-01_rb,0.12200e-01_rb,0.13999e-01_rb,0.16082e-01_rb /)
      kbo(:,39, 9) = (/ &
        & 0.84558e-02_rb,0.98580e-02_rb,0.11508e-01_rb,0.13321e-01_rb,0.15372e-01_rb /)
      kbo(:,40, 9) = (/ &
        & 0.78020e-02_rb,0.91740e-02_rb,0.10778e-01_rb,0.12588e-01_rb,0.14594e-01_rb /)
      kbo(:,41, 9) = (/ &
        & 0.72028e-02_rb,0.85294e-02_rb,0.10093e-01_rb,0.11887e-01_rb,0.13872e-01_rb /)
      kbo(:,42, 9) = (/ &
        & 0.66335e-02_rb,0.79299e-02_rb,0.94524e-02_rb,0.11233e-01_rb,0.13162e-01_rb /)
      kbo(:,43, 9) = (/ &
        & 0.60936e-02_rb,0.73819e-02_rb,0.88361e-02_rb,0.10555e-01_rb,0.12456e-01_rb /)
      kbo(:,44, 9) = (/ &
        & 0.55851e-02_rb,0.68018e-02_rb,0.82487e-02_rb,0.98836e-02_rb,0.11744e-01_rb /)
      kbo(:,45, 9) = (/ &
        & 0.51361e-02_rb,0.62783e-02_rb,0.76910e-02_rb,0.92631e-02_rb,0.11096e-01_rb /)
      kbo(:,46, 9) = (/ &
        & 0.46789e-02_rb,0.57877e-02_rb,0.71257e-02_rb,0.86688e-02_rb,0.10399e-01_rb /)
      kbo(:,47, 9) = (/ &
        & 0.42434e-02_rb,0.53043e-02_rb,0.65448e-02_rb,0.80300e-02_rb,0.97286e-02_rb /)
      kbo(:,48, 9) = (/ &
        & 0.38917e-02_rb,0.48269e-02_rb,0.60283e-02_rb,0.74378e-02_rb,0.90783e-02_rb /)
      kbo(:,49, 9) = (/ &
        & 0.35823e-02_rb,0.43988e-02_rb,0.55409e-02_rb,0.68845e-02_rb,0.84495e-02_rb /)
      kbo(:,50, 9) = (/ &
        & 0.33116e-02_rb,0.40852e-02_rb,0.51044e-02_rb,0.63885e-02_rb,0.78773e-02_rb /)
      kbo(:,51, 9) = (/ &
        & 0.30752e-02_rb,0.38103e-02_rb,0.47346e-02_rb,0.59580e-02_rb,0.73779e-02_rb /)
      kbo(:,52, 9) = (/ &
        & 0.28626e-02_rb,0.35553e-02_rb,0.44430e-02_rb,0.55409e-02_rb,0.69343e-02_rb /)
      kbo(:,53, 9) = (/ &
        & 0.26485e-02_rb,0.33392e-02_rb,0.41695e-02_rb,0.51739e-02_rb,0.64753e-02_rb /)
      kbo(:,54, 9) = (/ &
        & 0.24736e-02_rb,0.31489e-02_rb,0.39367e-02_rb,0.48954e-02_rb,0.61401e-02_rb /)
      kbo(:,55, 9) = (/ &
        & 0.23362e-02_rb,0.29819e-02_rb,0.37593e-02_rb,0.46985e-02_rb,0.58363e-02_rb /)
      kbo(:,56, 9) = (/ &
        & 0.22243e-02_rb,0.28447e-02_rb,0.35827e-02_rb,0.44762e-02_rb,0.55720e-02_rb /)
      kbo(:,57, 9) = (/ &
        & 0.21389e-02_rb,0.27081e-02_rb,0.34315e-02_rb,0.43182e-02_rb,0.53898e-02_rb /)
      kbo(:,58, 9) = (/ &
        & 0.20428e-02_rb,0.25997e-02_rb,0.33047e-02_rb,0.41726e-02_rb,0.52071e-02_rb /)
      kbo(:,59, 9) = (/ &
        & 0.20538e-02_rb,0.26323e-02_rb,0.33247e-02_rb,0.41770e-02_rb,0.52496e-02_rb /)
      kbo(:,13,10) = (/ &
        & 0.58637e+00_rb,0.59481e+00_rb,0.59811e+00_rb,0.60451e+00_rb,0.60418e+00_rb /)
      kbo(:,14,10) = (/ &
        & 0.53191e+00_rb,0.53363e+00_rb,0.53861e+00_rb,0.53976e+00_rb,0.53933e+00_rb /)
      kbo(:,15,10) = (/ &
        & 0.47845e+00_rb,0.48447e+00_rb,0.48498e+00_rb,0.48691e+00_rb,0.48935e+00_rb /)
      kbo(:,16,10) = (/ &
        & 0.42695e+00_rb,0.43265e+00_rb,0.43442e+00_rb,0.43949e+00_rb,0.44244e+00_rb /)
      kbo(:,17,10) = (/ &
        & 0.37796e+00_rb,0.38437e+00_rb,0.38958e+00_rb,0.39367e+00_rb,0.39802e+00_rb /)
      kbo(:,18,10) = (/ &
        & 0.33399e+00_rb,0.33971e+00_rb,0.34517e+00_rb,0.35032e+00_rb,0.35265e+00_rb /)
      kbo(:,19,10) = (/ &
        & 0.29538e+00_rb,0.30247e+00_rb,0.30713e+00_rb,0.30943e+00_rb,0.31544e+00_rb /)
      kbo(:,20,10) = (/ &
        & 0.26154e+00_rb,0.26955e+00_rb,0.27264e+00_rb,0.27778e+00_rb,0.28167e+00_rb /)
      kbo(:,21,10) = (/ &
        & 0.23200e+00_rb,0.23896e+00_rb,0.24468e+00_rb,0.24838e+00_rb,0.25453e+00_rb /)
      kbo(:,22,10) = (/ &
        & 0.20714e+00_rb,0.21368e+00_rb,0.21959e+00_rb,0.22402e+00_rb,0.22934e+00_rb /)
      kbo(:,23,10) = (/ &
        & 0.18514e+00_rb,0.19070e+00_rb,0.19771e+00_rb,0.20426e+00_rb,0.20794e+00_rb /)
      kbo(:,24,10) = (/ &
        & 0.16629e+00_rb,0.17247e+00_rb,0.17943e+00_rb,0.18543e+00_rb,0.19044e+00_rb /)
      kbo(:,25,10) = (/ &
        & 0.15103e+00_rb,0.15758e+00_rb,0.16440e+00_rb,0.16947e+00_rb,0.17504e+00_rb /)
      kbo(:,26,10) = (/ &
        & 0.13782e+00_rb,0.14448e+00_rb,0.15050e+00_rb,0.15723e+00_rb,0.16403e+00_rb /)
      kbo(:,27,10) = (/ &
        & 0.12675e+00_rb,0.13284e+00_rb,0.13902e+00_rb,0.14547e+00_rb,0.15299e+00_rb /)
      kbo(:,28,10) = (/ &
        & 0.11700e+00_rb,0.12400e+00_rb,0.13029e+00_rb,0.13779e+00_rb,0.14535e+00_rb /)
      kbo(:,29,10) = (/ &
        & 0.10975e+00_rb,0.11580e+00_rb,0.12300e+00_rb,0.13106e+00_rb,0.13981e+00_rb /)
      kbo(:,30,10) = (/ &
        & 0.10326e+00_rb,0.10877e+00_rb,0.11694e+00_rb,0.12523e+00_rb,0.13409e+00_rb /)
      kbo(:,31,10) = (/ &
        & 0.97072e-01_rb,0.10392e+00_rb,0.11191e+00_rb,0.12092e+00_rb,0.13156e+00_rb /)
      kbo(:,32,10) = (/ &
        & 0.92483e-01_rb,0.10003e+00_rb,0.10847e+00_rb,0.11777e+00_rb,0.12978e+00_rb /)
      kbo(:,33,10) = (/ &
        & 0.89436e-01_rb,0.96549e-01_rb,0.10550e+00_rb,0.11582e+00_rb,0.12938e+00_rb /)
      kbo(:,34,10) = (/ &
        & 0.86605e-01_rb,0.93852e-01_rb,0.10437e+00_rb,0.11565e+00_rb,0.13025e+00_rb /)
      kbo(:,35,10) = (/ &
        & 0.83755e-01_rb,0.92443e-01_rb,0.10318e+00_rb,0.11569e+00_rb,0.13035e+00_rb /)
      kbo(:,36,10) = (/ &
        & 0.80159e-01_rb,0.90106e-01_rb,0.10185e+00_rb,0.11331e+00_rb,0.13105e+00_rb /)
      kbo(:,37,10) = (/ &
        & 0.76863e-01_rb,0.86981e-01_rb,0.98816e-01_rb,0.11094e+00_rb,0.12837e+00_rb /)
      kbo(:,38,10) = (/ &
        & 0.73911e-01_rb,0.84640e-01_rb,0.95988e-01_rb,0.10951e+00_rb,0.12524e+00_rb /)
      kbo(:,39,10) = (/ &
        & 0.71484e-01_rb,0.82465e-01_rb,0.93807e-01_rb,0.10732e+00_rb,0.12288e+00_rb /)
      kbo(:,40,10) = (/ &
        & 0.68367e-01_rb,0.78553e-01_rb,0.90288e-01_rb,0.10290e+00_rb,0.11884e+00_rb /)
      kbo(:,41,10) = (/ &
        & 0.65972e-01_rb,0.75367e-01_rb,0.87331e-01_rb,0.98979e-01_rb,0.11479e+00_rb /)
      kbo(:,42,10) = (/ &
        & 0.63899e-01_rb,0.72995e-01_rb,0.84392e-01_rb,0.96170e-01_rb,0.11066e+00_rb /)
      kbo(:,43,10) = (/ &
        & 0.61601e-01_rb,0.69361e-01_rb,0.80265e-01_rb,0.92296e-01_rb,0.10575e+00_rb /)
      kbo(:,44,10) = (/ &
        & 0.58053e-01_rb,0.67236e-01_rb,0.77057e-01_rb,0.88848e-01_rb,0.10118e+00_rb /)
      kbo(:,45,10) = (/ &
        & 0.55041e-01_rb,0.64788e-01_rb,0.73479e-01_rb,0.85189e-01_rb,0.97294e-01_rb /)
      kbo(:,46,10) = (/ &
        & 0.51538e-01_rb,0.61359e-01_rb,0.70493e-01_rb,0.81342e-01_rb,0.93711e-01_rb /)
      kbo(:,47,10) = (/ &
        & 0.48229e-01_rb,0.57754e-01_rb,0.67703e-01_rb,0.77712e-01_rb,0.88779e-01_rb /)
      kbo(:,48,10) = (/ &
        & 0.44328e-01_rb,0.53953e-01_rb,0.64327e-01_rb,0.74186e-01_rb,0.85354e-01_rb /)
      kbo(:,49,10) = (/ &
        & 0.40181e-01_rb,0.50939e-01_rb,0.60544e-01_rb,0.71079e-01_rb,0.81883e-01_rb /)
      kbo(:,50,10) = (/ &
        & 0.36327e-01_rb,0.47243e-01_rb,0.57625e-01_rb,0.67925e-01_rb,0.78806e-01_rb /)
      kbo(:,51,10) = (/ &
        & 0.33026e-01_rb,0.43278e-01_rb,0.54464e-01_rb,0.64411e-01_rb,0.75916e-01_rb /)
      kbo(:,52,10) = (/ &
        & 0.29600e-01_rb,0.39888e-01_rb,0.50913e-01_rb,0.61457e-01_rb,0.72719e-01_rb /)
      kbo(:,53,10) = (/ &
        & 0.26994e-01_rb,0.36264e-01_rb,0.46926e-01_rb,0.58631e-01_rb,0.68972e-01_rb /)
      kbo(:,54,10) = (/ &
        & 0.24928e-01_rb,0.33245e-01_rb,0.43870e-01_rb,0.55481e-01_rb,0.66121e-01_rb /)
      kbo(:,55,10) = (/ &
        & 0.23329e-01_rb,0.30647e-01_rb,0.40729e-01_rb,0.52051e-01_rb,0.63960e-01_rb /)
      kbo(:,56,10) = (/ &
        & 0.21581e-01_rb,0.28578e-01_rb,0.37651e-01_rb,0.48828e-01_rb,0.60673e-01_rb /)
      kbo(:,57,10) = (/ &
        & 0.20121e-01_rb,0.27060e-01_rb,0.35152e-01_rb,0.45854e-01_rb,0.57152e-01_rb /)
      kbo(:,58,10) = (/ &
        & 0.18692e-01_rb,0.25639e-01_rb,0.32995e-01_rb,0.42657e-01_rb,0.54655e-01_rb /)
      kbo(:,59,10) = (/ &
        & 0.18277e-01_rb,0.25273e-01_rb,0.32791e-01_rb,0.42508e-01_rb,0.53680e-01_rb /)
      kbo(:,13,11) = (/ &
        & 0.10693e+01_rb,0.10902e+01_rb,0.11032e+01_rb,0.10991e+01_rb,0.11118e+01_rb /)
      kbo(:,14,11) = (/ &
        & 0.95953e+00_rb,0.97384e+00_rb,0.98577e+00_rb,0.99728e+00_rb,0.10070e+01_rb /)
      kbo(:,15,11) = (/ &
        & 0.85940e+00_rb,0.86781e+00_rb,0.87766e+00_rb,0.88834e+00_rb,0.89855e+00_rb /)
      kbo(:,16,11) = (/ &
        & 0.76187e+00_rb,0.76651e+00_rb,0.78116e+00_rb,0.79605e+00_rb,0.80429e+00_rb /)
      kbo(:,17,11) = (/ &
        & 0.67627e+00_rb,0.68377e+00_rb,0.69734e+00_rb,0.70867e+00_rb,0.71758e+00_rb /)
      kbo(:,18,11) = (/ &
        & 0.60442e+00_rb,0.61462e+00_rb,0.62319e+00_rb,0.63477e+00_rb,0.64696e+00_rb /)
      kbo(:,19,11) = (/ &
        & 0.54250e+00_rb,0.55180e+00_rb,0.56132e+00_rb,0.57649e+00_rb,0.58604e+00_rb /)
      kbo(:,20,11) = (/ &
        & 0.49000e+00_rb,0.49922e+00_rb,0.51229e+00_rb,0.52536e+00_rb,0.53903e+00_rb /)
      kbo(:,21,11) = (/ &
        & 0.44362e+00_rb,0.45018e+00_rb,0.46523e+00_rb,0.48212e+00_rb,0.49590e+00_rb /)
      kbo(:,22,11) = (/ &
        & 0.40390e+00_rb,0.41136e+00_rb,0.42554e+00_rb,0.44299e+00_rb,0.45956e+00_rb /)
      kbo(:,23,11) = (/ &
        & 0.36600e+00_rb,0.37646e+00_rb,0.39144e+00_rb,0.40936e+00_rb,0.42622e+00_rb /)
      kbo(:,24,11) = (/ &
        & 0.33488e+00_rb,0.34802e+00_rb,0.36259e+00_rb,0.38089e+00_rb,0.39832e+00_rb /)
      kbo(:,25,11) = (/ &
        & 0.30569e+00_rb,0.32099e+00_rb,0.33730e+00_rb,0.35716e+00_rb,0.37447e+00_rb /)
      kbo(:,26,11) = (/ &
        & 0.28345e+00_rb,0.29933e+00_rb,0.31861e+00_rb,0.33716e+00_rb,0.35400e+00_rb /)
      kbo(:,27,11) = (/ &
        & 0.26524e+00_rb,0.28116e+00_rb,0.30310e+00_rb,0.32398e+00_rb,0.34341e+00_rb /)
      kbo(:,28,11) = (/ &
        & 0.25036e+00_rb,0.26748e+00_rb,0.29040e+00_rb,0.31038e+00_rb,0.33527e+00_rb /)
      kbo(:,29,11) = (/ &
        & 0.23678e+00_rb,0.25759e+00_rb,0.27958e+00_rb,0.30373e+00_rb,0.33097e+00_rb /)
      kbo(:,30,11) = (/ &
        & 0.22835e+00_rb,0.25043e+00_rb,0.27298e+00_rb,0.30247e+00_rb,0.33069e+00_rb /)
      kbo(:,31,11) = (/ &
        & 0.22381e+00_rb,0.24448e+00_rb,0.27386e+00_rb,0.30198e+00_rb,0.33135e+00_rb /)
      kbo(:,32,11) = (/ &
        & 0.21945e+00_rb,0.24581e+00_rb,0.27557e+00_rb,0.30488e+00_rb,0.33817e+00_rb /)
      kbo(:,33,11) = (/ &
        & 0.22020e+00_rb,0.24997e+00_rb,0.27933e+00_rb,0.31199e+00_rb,0.34537e+00_rb /)
      kbo(:,34,11) = (/ &
        & 0.22328e+00_rb,0.25315e+00_rb,0.28560e+00_rb,0.31999e+00_rb,0.35204e+00_rb /)
      kbo(:,35,11) = (/ &
        & 0.22463e+00_rb,0.25599e+00_rb,0.29016e+00_rb,0.32472e+00_rb,0.35705e+00_rb /)
      kbo(:,36,11) = (/ &
        & 0.22454e+00_rb,0.25719e+00_rb,0.29064e+00_rb,0.32676e+00_rb,0.35643e+00_rb /)
      kbo(:,37,11) = (/ &
        & 0.21917e+00_rb,0.25146e+00_rb,0.28533e+00_rb,0.32018e+00_rb,0.35207e+00_rb /)
      kbo(:,38,11) = (/ &
        & 0.21431e+00_rb,0.24593e+00_rb,0.28070e+00_rb,0.31440e+00_rb,0.34740e+00_rb /)
      kbo(:,39,11) = (/ &
        & 0.20995e+00_rb,0.24107e+00_rb,0.27540e+00_rb,0.30795e+00_rb,0.34247e+00_rb /)
      kbo(:,40,11) = (/ &
        & 0.20148e+00_rb,0.23319e+00_rb,0.26577e+00_rb,0.30086e+00_rb,0.33366e+00_rb /)
      kbo(:,41,11) = (/ &
        & 0.19132e+00_rb,0.22478e+00_rb,0.25632e+00_rb,0.29251e+00_rb,0.32436e+00_rb /)
      kbo(:,42,11) = (/ &
        & 0.18231e+00_rb,0.21584e+00_rb,0.24724e+00_rb,0.28164e+00_rb,0.31641e+00_rb /)
      kbo(:,43,11) = (/ &
        & 0.17273e+00_rb,0.20475e+00_rb,0.23702e+00_rb,0.26984e+00_rb,0.30535e+00_rb /)
      kbo(:,44,11) = (/ &
        & 0.16365e+00_rb,0.19313e+00_rb,0.22566e+00_rb,0.25812e+00_rb,0.29340e+00_rb /)
      kbo(:,45,11) = (/ &
        & 0.15471e+00_rb,0.18278e+00_rb,0.21464e+00_rb,0.24686e+00_rb,0.28060e+00_rb /)
      kbo(:,46,11) = (/ &
        & 0.14477e+00_rb,0.17256e+00_rb,0.20328e+00_rb,0.23515e+00_rb,0.26753e+00_rb /)
      kbo(:,47,11) = (/ &
        & 0.13513e+00_rb,0.16209e+00_rb,0.19092e+00_rb,0.22162e+00_rb,0.25481e+00_rb /)
      kbo(:,48,11) = (/ &
        & 0.12632e+00_rb,0.15056e+00_rb,0.17900e+00_rb,0.20905e+00_rb,0.24085e+00_rb /)
      kbo(:,49,11) = (/ &
        & 0.11846e+00_rb,0.14059e+00_rb,0.16812e+00_rb,0.19759e+00_rb,0.22701e+00_rb /)
      kbo(:,50,11) = (/ &
        & 0.11278e+00_rb,0.13341e+00_rb,0.15756e+00_rb,0.18662e+00_rb,0.21629e+00_rb /)
      kbo(:,51,11) = (/ &
        & 0.10711e+00_rb,0.12752e+00_rb,0.14951e+00_rb,0.17651e+00_rb,0.20555e+00_rb /)
      kbo(:,52,11) = (/ &
        & 0.10062e+00_rb,0.12140e+00_rb,0.14273e+00_rb,0.16677e+00_rb,0.19594e+00_rb /)
      kbo(:,53,11) = (/ &
        & 0.93686e-01_rb,0.11587e+00_rb,0.13705e+00_rb,0.15900e+00_rb,0.18673e+00_rb /)
      kbo(:,54,11) = (/ &
        & 0.88183e-01_rb,0.10986e+00_rb,0.13159e+00_rb,0.15362e+00_rb,0.17808e+00_rb /)
      kbo(:,55,11) = (/ &
        & 0.81924e-01_rb,0.10428e+00_rb,0.12662e+00_rb,0.14862e+00_rb,0.17143e+00_rb /)
      kbo(:,56,11) = (/ &
        & 0.75991e-01_rb,0.98633e-01_rb,0.12219e+00_rb,0.14408e+00_rb,0.16698e+00_rb /)
      kbo(:,57,11) = (/ &
        & 0.69399e-01_rb,0.92760e-01_rb,0.11664e+00_rb,0.14040e+00_rb,0.16308e+00_rb /)
      kbo(:,58,11) = (/ &
        & 0.64018e-01_rb,0.88343e-01_rb,0.11169e+00_rb,0.13686e+00_rb,0.15969e+00_rb /)
      kbo(:,59,11) = (/ &
        & 0.63145e-01_rb,0.86273e-01_rb,0.11143e+00_rb,0.13565e+00_rb,0.16187e+00_rb /)
      kbo(:,13,12) = (/ &
        & 0.22241e+01_rb,0.22556e+01_rb,0.22932e+01_rb,0.23316e+01_rb,0.23817e+01_rb /)
      kbo(:,14,12) = (/ &
        & 0.20015e+01_rb,0.20535e+01_rb,0.20976e+01_rb,0.21411e+01_rb,0.21895e+01_rb /)
      kbo(:,15,12) = (/ &
        & 0.18111e+01_rb,0.18671e+01_rb,0.19196e+01_rb,0.19734e+01_rb,0.20226e+01_rb /)
      kbo(:,16,12) = (/ &
        & 0.16513e+01_rb,0.17114e+01_rb,0.17632e+01_rb,0.18132e+01_rb,0.18623e+01_rb /)
      kbo(:,17,12) = (/ &
        & 0.15041e+01_rb,0.15648e+01_rb,0.16197e+01_rb,0.16720e+01_rb,0.17169e+01_rb /)
      kbo(:,18,12) = (/ &
        & 0.13731e+01_rb,0.14345e+01_rb,0.14937e+01_rb,0.15386e+01_rb,0.16052e+01_rb /)
      kbo(:,19,12) = (/ &
        & 0.12478e+01_rb,0.13115e+01_rb,0.13641e+01_rb,0.14279e+01_rb,0.14977e+01_rb /)
      kbo(:,20,12) = (/ &
        & 0.11298e+01_rb,0.11910e+01_rb,0.12495e+01_rb,0.13186e+01_rb,0.13914e+01_rb /)
      kbo(:,21,12) = (/ &
        & 0.10266e+01_rb,0.10915e+01_rb,0.11508e+01_rb,0.12227e+01_rb,0.13004e+01_rb /)
      kbo(:,22,12) = (/ &
        & 0.93959e+00_rb,0.10027e+01_rb,0.10735e+01_rb,0.11507e+01_rb,0.12312e+01_rb /)
      kbo(:,23,12) = (/ &
        & 0.87016e+00_rb,0.93486e+00_rb,0.10125e+01_rb,0.10930e+01_rb,0.11794e+01_rb /)
      kbo(:,24,12) = (/ &
        & 0.81213e+00_rb,0.88017e+00_rb,0.96483e+00_rb,0.10508e+01_rb,0.11396e+01_rb /)
      kbo(:,25,12) = (/ &
        & 0.77134e+00_rb,0.84295e+00_rb,0.93081e+00_rb,0.10201e+01_rb,0.11152e+01_rb /)
      kbo(:,26,12) = (/ &
        & 0.73832e+00_rb,0.81743e+00_rb,0.90746e+00_rb,0.10024e+01_rb,0.11020e+01_rb /)
      kbo(:,27,12) = (/ &
        & 0.71831e+00_rb,0.80328e+00_rb,0.89505e+00_rb,0.99247e+00_rb,0.10943e+01_rb /)
      kbo(:,28,12) = (/ &
        & 0.70848e+00_rb,0.79592e+00_rb,0.89143e+00_rb,0.99432e+00_rb,0.10940e+01_rb /)
      kbo(:,29,12) = (/ &
        & 0.70725e+00_rb,0.79781e+00_rb,0.89906e+00_rb,0.10007e+01_rb,0.11009e+01_rb /)
      kbo(:,30,12) = (/ &
        & 0.71125e+00_rb,0.80824e+00_rb,0.91144e+00_rb,0.10109e+01_rb,0.11144e+01_rb /)
      kbo(:,31,12) = (/ &
        & 0.72057e+00_rb,0.82548e+00_rb,0.92423e+00_rb,0.10285e+01_rb,0.11338e+01_rb /)
      kbo(:,32,12) = (/ &
        & 0.73855e+00_rb,0.84223e+00_rb,0.94322e+00_rb,0.10495e+01_rb,0.11523e+01_rb /)
      kbo(:,33,12) = (/ &
        & 0.75860e+00_rb,0.86137e+00_rb,0.96653e+00_rb,0.10709e+01_rb,0.11736e+01_rb /)
      kbo(:,34,12) = (/ &
        & 0.77584e+00_rb,0.88104e+00_rb,0.98358e+00_rb,0.10881e+01_rb,0.11928e+01_rb /)
      kbo(:,35,12) = (/ &
        & 0.78526e+00_rb,0.88915e+00_rb,0.99181e+00_rb,0.10955e+01_rb,0.12024e+01_rb /)
      kbo(:,36,12) = (/ &
        & 0.78420e+00_rb,0.88679e+00_rb,0.98962e+00_rb,0.10938e+01_rb,0.12013e+01_rb /)
      kbo(:,37,12) = (/ &
        & 0.76619e+00_rb,0.86975e+00_rb,0.97162e+00_rb,0.10774e+01_rb,0.11823e+01_rb /)
      kbo(:,38,12) = (/ &
        & 0.74778e+00_rb,0.85168e+00_rb,0.95355e+00_rb,0.10588e+01_rb,0.11642e+01_rb /)
      kbo(:,39,12) = (/ &
        & 0.73003e+00_rb,0.83397e+00_rb,0.93638e+00_rb,0.10427e+01_rb,0.11461e+01_rb /)
      kbo(:,40,12) = (/ &
        & 0.70694e+00_rb,0.80603e+00_rb,0.90951e+00_rb,0.10129e+01_rb,0.11179e+01_rb /)
      kbo(:,41,12) = (/ &
        & 0.68380e+00_rb,0.77629e+00_rb,0.88021e+00_rb,0.98263e+00_rb,0.10876e+01_rb /)
      kbo(:,42,12) = (/ &
        & 0.66231e+00_rb,0.74849e+00_rb,0.85130e+00_rb,0.95458e+00_rb,0.10576e+01_rb /)
      kbo(:,43,12) = (/ &
        & 0.63688e+00_rb,0.72048e+00_rb,0.81634e+00_rb,0.92017e+00_rb,0.10227e+01_rb /)
      kbo(:,44,12) = (/ &
        & 0.60858e+00_rb,0.69178e+00_rb,0.77940e+00_rb,0.88149e+00_rb,0.98460e+00_rb /)
      kbo(:,45,12) = (/ &
        & 0.58153e+00_rb,0.66520e+00_rb,0.74925e+00_rb,0.84319e+00_rb,0.94694e+00_rb /)
      kbo(:,46,12) = (/ &
        & 0.55197e+00_rb,0.63412e+00_rb,0.71872e+00_rb,0.80664e+00_rb,0.90684e+00_rb /)
      kbo(:,47,12) = (/ &
        & 0.51844e+00_rb,0.60273e+00_rb,0.68769e+00_rb,0.77076e+00_rb,0.86117e+00_rb /)
      kbo(:,48,12) = (/ &
        & 0.48120e+00_rb,0.57148e+00_rb,0.65491e+00_rb,0.74197e+00_rb,0.82485e+00_rb /)
      kbo(:,49,12) = (/ &
        & 0.44584e+00_rb,0.53846e+00_rb,0.62079e+00_rb,0.70603e+00_rb,0.79220e+00_rb /)
      kbo(:,50,12) = (/ &
        & 0.41289e+00_rb,0.50133e+00_rb,0.59314e+00_rb,0.67773e+00_rb,0.76297e+00_rb /)
      kbo(:,51,12) = (/ &
        & 0.38263e+00_rb,0.46808e+00_rb,0.55994e+00_rb,0.64654e+00_rb,0.73545e+00_rb /)
      kbo(:,52,12) = (/ &
        & 0.35216e+00_rb,0.43713e+00_rb,0.52781e+00_rb,0.61897e+00_rb,0.70212e+00_rb /)
      kbo(:,53,12) = (/ &
        & 0.33102e+00_rb,0.40477e+00_rb,0.49059e+00_rb,0.58472e+00_rb,0.67270e+00_rb /)
      kbo(:,54,12) = (/ &
        & 0.31146e+00_rb,0.37834e+00_rb,0.46271e+00_rb,0.55353e+00_rb,0.64596e+00_rb /)
      kbo(:,55,12) = (/ &
        & 0.29087e+00_rb,0.35982e+00_rb,0.43598e+00_rb,0.52416e+00_rb,0.61765e+00_rb /)
      kbo(:,56,12) = (/ &
        & 0.27304e+00_rb,0.34098e+00_rb,0.41224e+00_rb,0.49775e+00_rb,0.58771e+00_rb /)
      kbo(:,57,12) = (/ &
        & 0.25824e+00_rb,0.32066e+00_rb,0.39085e+00_rb,0.46641e+00_rb,0.55829e+00_rb /)
      kbo(:,58,12) = (/ &
        & 0.24548e+00_rb,0.30081e+00_rb,0.37338e+00_rb,0.44708e+00_rb,0.53062e+00_rb /)
      kbo(:,59,12) = (/ &
        & 0.24212e+00_rb,0.29858e+00_rb,0.36550e+00_rb,0.44064e+00_rb,0.51789e+00_rb /)
      kbo(:,13,13) = (/ &
        & 0.55998e+01_rb,0.57039e+01_rb,0.58049e+01_rb,0.59180e+01_rb,0.59854e+01_rb /)
      kbo(:,14,13) = (/ &
        & 0.53275e+01_rb,0.54387e+01_rb,0.55543e+01_rb,0.56621e+01_rb,0.57635e+01_rb /)
      kbo(:,15,13) = (/ &
        & 0.50203e+01_rb,0.51578e+01_rb,0.52898e+01_rb,0.54130e+01_rb,0.55373e+01_rb /)
      kbo(:,16,13) = (/ &
        & 0.47246e+01_rb,0.48786e+01_rb,0.50266e+01_rb,0.51689e+01_rb,0.53242e+01_rb /)
      kbo(:,17,13) = (/ &
        & 0.44403e+01_rb,0.46027e+01_rb,0.47680e+01_rb,0.49455e+01_rb,0.51299e+01_rb /)
      kbo(:,18,13) = (/ &
        & 0.41648e+01_rb,0.43473e+01_rb,0.45415e+01_rb,0.47508e+01_rb,0.49298e+01_rb /)
      kbo(:,19,13) = (/ &
        & 0.39134e+01_rb,0.41196e+01_rb,0.43493e+01_rb,0.45520e+01_rb,0.47522e+01_rb /)
      kbo(:,20,13) = (/ &
        & 0.36873e+01_rb,0.39195e+01_rb,0.41603e+01_rb,0.43826e+01_rb,0.46005e+01_rb /)
      kbo(:,21,13) = (/ &
        & 0.34974e+01_rb,0.37507e+01_rb,0.40010e+01_rb,0.42413e+01_rb,0.44759e+01_rb /)
      kbo(:,22,13) = (/ &
        & 0.33605e+01_rb,0.36305e+01_rb,0.38873e+01_rb,0.41404e+01_rb,0.43929e+01_rb /)
      kbo(:,23,13) = (/ &
        & 0.32571e+01_rb,0.35388e+01_rb,0.38042e+01_rb,0.40702e+01_rb,0.43353e+01_rb /)
      kbo(:,24,13) = (/ &
        & 0.31836e+01_rb,0.34721e+01_rb,0.37495e+01_rb,0.40277e+01_rb,0.43044e+01_rb /)
      kbo(:,25,13) = (/ &
        & 0.31346e+01_rb,0.34329e+01_rb,0.37222e+01_rb,0.40107e+01_rb,0.42953e+01_rb /)
      kbo(:,26,13) = (/ &
        & 0.31155e+01_rb,0.34221e+01_rb,0.37222e+01_rb,0.40178e+01_rb,0.43079e+01_rb /)
      kbo(:,27,13) = (/ &
        & 0.31157e+01_rb,0.34328e+01_rb,0.37392e+01_rb,0.40396e+01_rb,0.43346e+01_rb /)
      kbo(:,28,13) = (/ &
        & 0.31335e+01_rb,0.34580e+01_rb,0.37695e+01_rb,0.40734e+01_rb,0.43720e+01_rb /)
      kbo(:,29,13) = (/ &
        & 0.31697e+01_rb,0.34978e+01_rb,0.38120e+01_rb,0.41186e+01_rb,0.44189e+01_rb /)
      kbo(:,30,13) = (/ &
        & 0.32194e+01_rb,0.35475e+01_rb,0.38638e+01_rb,0.41720e+01_rb,0.44722e+01_rb /)
      kbo(:,31,13) = (/ &
        & 0.32811e+01_rb,0.36069e+01_rb,0.39241e+01_rb,0.42325e+01_rb,0.45314e+01_rb /)
      kbo(:,32,13) = (/ &
        & 0.33484e+01_rb,0.36727e+01_rb,0.39897e+01_rb,0.42974e+01_rb,0.45949e+01_rb /)
      kbo(:,33,13) = (/ &
        & 0.34186e+01_rb,0.37430e+01_rb,0.40586e+01_rb,0.43658e+01_rb,0.46602e+01_rb /)
      kbo(:,34,13) = (/ &
        & 0.34801e+01_rb,0.38048e+01_rb,0.41197e+01_rb,0.44255e+01_rb,0.47156e+01_rb /)
      kbo(:,35,13) = (/ &
        & 0.35126e+01_rb,0.38375e+01_rb,0.41525e+01_rb,0.44577e+01_rb,0.47454e+01_rb /)
      kbo(:,36,13) = (/ &
        & 0.35115e+01_rb,0.38370e+01_rb,0.41529e+01_rb,0.44584e+01_rb,0.47449e+01_rb /)
      kbo(:,37,13) = (/ &
        & 0.34620e+01_rb,0.37892e+01_rb,0.41066e+01_rb,0.44144e+01_rb,0.47052e+01_rb /)
      kbo(:,38,13) = (/ &
        & 0.34108e+01_rb,0.37391e+01_rb,0.40587e+01_rb,0.43685e+01_rb,0.46624e+01_rb /)
      kbo(:,39,13) = (/ &
        & 0.33603e+01_rb,0.36896e+01_rb,0.40114e+01_rb,0.43229e+01_rb,0.46208e+01_rb /)
      kbo(:,40,13) = (/ &
        & 0.32696e+01_rb,0.36068e+01_rb,0.39317e+01_rb,0.42460e+01_rb,0.45479e+01_rb /)
      kbo(:,41,13) = (/ &
        & 0.31746e+01_rb,0.35197e+01_rb,0.38475e+01_rb,0.41644e+01_rb,0.44716e+01_rb /)
      kbo(:,42,13) = (/ &
        & 0.30768e+01_rb,0.34312e+01_rb,0.37630e+01_rb,0.40826e+01_rb,0.43924e+01_rb /)
      kbo(:,43,13) = (/ &
        & 0.29558e+01_rb,0.33178e+01_rb,0.36587e+01_rb,0.39822e+01_rb,0.42959e+01_rb /)
      kbo(:,44,13) = (/ &
        & 0.28280e+01_rb,0.31906e+01_rb,0.35435e+01_rb,0.38727e+01_rb,0.41899e+01_rb /)
      kbo(:,45,13) = (/ &
        & 0.26988e+01_rb,0.30597e+01_rb,0.34201e+01_rb,0.37628e+01_rb,0.40831e+01_rb /)
      kbo(:,46,13) = (/ &
        & 0.25661e+01_rb,0.29262e+01_rb,0.32865e+01_rb,0.36383e+01_rb,0.39682e+01_rb /)
      kbo(:,47,13) = (/ &
        & 0.24176e+01_rb,0.27716e+01_rb,0.31308e+01_rb,0.34925e+01_rb,0.38381e+01_rb /)
      kbo(:,48,13) = (/ &
        & 0.22754e+01_rb,0.26218e+01_rb,0.29793e+01_rb,0.33363e+01_rb,0.36948e+01_rb /)
      kbo(:,49,13) = (/ &
        & 0.21329e+01_rb,0.24724e+01_rb,0.28302e+01_rb,0.31874e+01_rb,0.35454e+01_rb /)
      kbo(:,50,13) = (/ &
        & 0.20005e+01_rb,0.23398e+01_rb,0.26856e+01_rb,0.30422e+01_rb,0.34002e+01_rb /)
      kbo(:,51,13) = (/ &
        & 0.18713e+01_rb,0.22087e+01_rb,0.25515e+01_rb,0.29067e+01_rb,0.32588e+01_rb /)
      kbo(:,52,13) = (/ &
        & 0.17459e+01_rb,0.20772e+01_rb,0.24169e+01_rb,0.27666e+01_rb,0.31241e+01_rb /)
      kbo(:,53,13) = (/ &
        & 0.16095e+01_rb,0.19502e+01_rb,0.22895e+01_rb,0.26325e+01_rb,0.29855e+01_rb /)
      kbo(:,54,13) = (/ &
        & 0.15066e+01_rb,0.18320e+01_rb,0.21659e+01_rb,0.25081e+01_rb,0.28569e+01_rb /)
      kbo(:,55,13) = (/ &
        & 0.14318e+01_rb,0.17117e+01_rb,0.20491e+01_rb,0.23896e+01_rb,0.27343e+01_rb /)
      kbo(:,56,13) = (/ &
        & 0.13449e+01_rb,0.16235e+01_rb,0.19309e+01_rb,0.22682e+01_rb,0.26139e+01_rb /)
      kbo(:,57,13) = (/ &
        & 0.12667e+01_rb,0.15514e+01_rb,0.18274e+01_rb,0.21528e+01_rb,0.24919e+01_rb /)
      kbo(:,58,13) = (/ &
        & 0.11763e+01_rb,0.14696e+01_rb,0.17445e+01_rb,0.20345e+01_rb,0.23757e+01_rb /)
      kbo(:,59,13) = (/ &
        & 0.11472e+01_rb,0.14428e+01_rb,0.17312e+01_rb,0.20031e+01_rb,0.23234e+01_rb /)
      kbo(:,13,14) = (/ &
        & 0.15147e+02_rb,0.15130e+02_rb,0.15113e+02_rb,0.15099e+02_rb,0.15078e+02_rb /)
      kbo(:,14,14) = (/ &
        & 0.15510e+02_rb,0.15546e+02_rb,0.15567e+02_rb,0.15587e+02_rb,0.15585e+02_rb /)
      kbo(:,15,14) = (/ &
        & 0.15739e+02_rb,0.15841e+02_rb,0.15930e+02_rb,0.15986e+02_rb,0.16004e+02_rb /)
      kbo(:,16,14) = (/ &
        & 0.15858e+02_rb,0.16042e+02_rb,0.16196e+02_rb,0.16295e+02_rb,0.16347e+02_rb /)
      kbo(:,17,14) = (/ &
        & 0.15892e+02_rb,0.16168e+02_rb,0.16375e+02_rb,0.16519e+02_rb,0.16620e+02_rb /)
      kbo(:,18,14) = (/ &
        & 0.15890e+02_rb,0.16233e+02_rb,0.16495e+02_rb,0.16687e+02_rb,0.16843e+02_rb /)
      kbo(:,19,14) = (/ &
        & 0.15860e+02_rb,0.16260e+02_rb,0.16572e+02_rb,0.16822e+02_rb,0.17016e+02_rb /)
      kbo(:,20,14) = (/ &
        & 0.15821e+02_rb,0.16274e+02_rb,0.16642e+02_rb,0.16937e+02_rb,0.17165e+02_rb /)
      kbo(:,21,14) = (/ &
        & 0.15778e+02_rb,0.16284e+02_rb,0.16705e+02_rb,0.17036e+02_rb,0.17293e+02_rb /)
      kbo(:,22,14) = (/ &
        & 0.15777e+02_rb,0.16337e+02_rb,0.16794e+02_rb,0.17151e+02_rb,0.17417e+02_rb /)
      kbo(:,23,14) = (/ &
        & 0.15815e+02_rb,0.16408e+02_rb,0.16883e+02_rb,0.17258e+02_rb,0.17537e+02_rb /)
      kbo(:,24,14) = (/ &
        & 0.15884e+02_rb,0.16493e+02_rb,0.16982e+02_rb,0.17365e+02_rb,0.17648e+02_rb /)
      kbo(:,25,14) = (/ &
        & 0.15973e+02_rb,0.16594e+02_rb,0.17087e+02_rb,0.17470e+02_rb,0.17753e+02_rb /)
      kbo(:,26,14) = (/ &
        & 0.16098e+02_rb,0.16711e+02_rb,0.17198e+02_rb,0.17574e+02_rb,0.17853e+02_rb /)
      kbo(:,27,14) = (/ &
        & 0.16232e+02_rb,0.16836e+02_rb,0.17310e+02_rb,0.17676e+02_rb,0.17946e+02_rb /)
      kbo(:,28,14) = (/ &
        & 0.16376e+02_rb,0.16966e+02_rb,0.17425e+02_rb,0.17775e+02_rb,0.18032e+02_rb /)
      kbo(:,29,14) = (/ &
        & 0.16527e+02_rb,0.17096e+02_rb,0.17538e+02_rb,0.17871e+02_rb,0.18111e+02_rb /)
      kbo(:,30,14) = (/ &
        & 0.16681e+02_rb,0.17223e+02_rb,0.17646e+02_rb,0.17959e+02_rb,0.18183e+02_rb /)
      kbo(:,31,14) = (/ &
        & 0.16832e+02_rb,0.17348e+02_rb,0.17747e+02_rb,0.18041e+02_rb,0.18247e+02_rb /)
      kbo(:,32,14) = (/ &
        & 0.16982e+02_rb,0.17470e+02_rb,0.17844e+02_rb,0.18116e+02_rb,0.18304e+02_rb /)
      kbo(:,33,14) = (/ &
        & 0.17125e+02_rb,0.17586e+02_rb,0.17935e+02_rb,0.18187e+02_rb,0.18356e+02_rb /)
      kbo(:,34,14) = (/ &
        & 0.17243e+02_rb,0.17681e+02_rb,0.18009e+02_rb,0.18243e+02_rb,0.18396e+02_rb /)
      kbo(:,35,14) = (/ &
        & 0.17308e+02_rb,0.17736e+02_rb,0.18053e+02_rb,0.18277e+02_rb,0.18422e+02_rb /)
      kbo(:,36,14) = (/ &
        & 0.17321e+02_rb,0.17747e+02_rb,0.18064e+02_rb,0.18289e+02_rb,0.18431e+02_rb /)
      kbo(:,37,14) = (/ &
        & 0.17259e+02_rb,0.17701e+02_rb,0.18033e+02_rb,0.18270e+02_rb,0.18425e+02_rb /)
      kbo(:,38,14) = (/ &
        & 0.17190e+02_rb,0.17649e+02_rb,0.17997e+02_rb,0.18247e+02_rb,0.18415e+02_rb /)
      kbo(:,39,14) = (/ &
        & 0.17116e+02_rb,0.17594e+02_rb,0.17957e+02_rb,0.18221e+02_rb,0.18401e+02_rb /)
      kbo(:,40,14) = (/ &
        & 0.16981e+02_rb,0.17491e+02_rb,0.17882e+02_rb,0.18168e+02_rb,0.18367e+02_rb /)
      kbo(:,41,14) = (/ &
        & 0.16829e+02_rb,0.17375e+02_rb,0.17795e+02_rb,0.18107e+02_rb,0.18328e+02_rb /)
      kbo(:,42,14) = (/ &
        & 0.16667e+02_rb,0.17250e+02_rb,0.17700e+02_rb,0.18038e+02_rb,0.18281e+02_rb /)
      kbo(:,43,14) = (/ &
        & 0.16452e+02_rb,0.17082e+02_rb,0.17573e+02_rb,0.17944e+02_rb,0.18214e+02_rb /)
      kbo(:,44,14) = (/ &
        & 0.16203e+02_rb,0.16887e+02_rb,0.17421e+02_rb,0.17833e+02_rb,0.18135e+02_rb /)
      kbo(:,45,14) = (/ &
        & 0.15935e+02_rb,0.16673e+02_rb,0.17257e+02_rb,0.17708e+02_rb,0.18046e+02_rb /)
      kbo(:,46,14) = (/ &
        & 0.15628e+02_rb,0.16427e+02_rb,0.17064e+02_rb,0.17560e+02_rb,0.17936e+02_rb /)
      kbo(:,47,14) = (/ &
        & 0.15258e+02_rb,0.16126e+02_rb,0.16827e+02_rb,0.17377e+02_rb,0.17799e+02_rb /)
      kbo(:,48,14) = (/ &
        & 0.14864e+02_rb,0.15798e+02_rb,0.16564e+02_rb,0.17174e+02_rb,0.17643e+02_rb /)
      kbo(:,49,14) = (/ &
        & 0.14439e+02_rb,0.15441e+02_rb,0.16276e+02_rb,0.16946e+02_rb,0.17469e+02_rb /)
      kbo(:,50,14) = (/ &
        & 0.14012e+02_rb,0.15084e+02_rb,0.15980e+02_rb,0.16711e+02_rb,0.17289e+02_rb /)
      kbo(:,51,14) = (/ &
        & 0.13576e+02_rb,0.14715e+02_rb,0.15671e+02_rb,0.16463e+02_rb,0.17096e+02_rb /)
      kbo(:,52,14) = (/ &
        & 0.13120e+02_rb,0.14320e+02_rb,0.15341e+02_rb,0.16194e+02_rb,0.16882e+02_rb /)
      kbo(:,53,14) = (/ &
        & 0.12642e+02_rb,0.13899e+02_rb,0.14988e+02_rb,0.15902e+02_rb,0.16649e+02_rb /)
      kbo(:,54,14) = (/ &
        & 0.12147e+02_rb,0.13496e+02_rb,0.14645e+02_rb,0.15614e+02_rb,0.16418e+02_rb /)
      kbo(:,55,14) = (/ &
        & 0.11626e+02_rb,0.13090e+02_rb,0.14295e+02_rb,0.15320e+02_rb,0.16178e+02_rb /)
      kbo(:,56,14) = (/ &
        & 0.11110e+02_rb,0.12619e+02_rb,0.13926e+02_rb,0.15012e+02_rb,0.15922e+02_rb /)
      kbo(:,57,14) = (/ &
        & 0.10565e+02_rb,0.12110e+02_rb,0.13518e+02_rb,0.14685e+02_rb,0.15649e+02_rb /)
      kbo(:,58,14) = (/ &
        & 0.10061e+02_rb,0.11633e+02_rb,0.13084e+02_rb,0.14356e+02_rb,0.15374e+02_rb /)
      kbo(:,59,14) = (/ &
        & 0.98377e+01_rb,0.11416e+02_rb,0.12868e+02_rb,0.14184e+02_rb,0.15259e+02_rb /)
      kbo(:,13,15) = (/ &
        & 0.41304e+02_rb,0.40483e+02_rb,0.39654e+02_rb,0.38829e+02_rb,0.38029e+02_rb /)
      kbo(:,14,15) = (/ &
        & 0.44686e+02_rb,0.43762e+02_rb,0.42836e+02_rb,0.41888e+02_rb,0.41004e+02_rb /)
      kbo(:,15,15) = (/ &
        & 0.48004e+02_rb,0.46942e+02_rb,0.45864e+02_rb,0.44813e+02_rb,0.43844e+02_rb /)
      kbo(:,16,15) = (/ &
        & 0.51150e+02_rb,0.49938e+02_rb,0.48734e+02_rb,0.47591e+02_rb,0.46512e+02_rb /)
      kbo(:,17,15) = (/ &
        & 0.54100e+02_rb,0.52729e+02_rb,0.51435e+02_rb,0.50180e+02_rb,0.48962e+02_rb /)
      kbo(:,18,15) = (/ &
        & 0.56777e+02_rb,0.55306e+02_rb,0.53894e+02_rb,0.52525e+02_rb,0.51164e+02_rb /)
      kbo(:,19,15) = (/ &
        & 0.59186e+02_rb,0.57635e+02_rb,0.56100e+02_rb,0.54603e+02_rb,0.53129e+02_rb /)
      kbo(:,20,15) = (/ &
        & 0.61293e+02_rb,0.59648e+02_rb,0.58005e+02_rb,0.56388e+02_rb,0.54798e+02_rb /)
      kbo(:,21,15) = (/ &
        & 0.63124e+02_rb,0.61368e+02_rb,0.59613e+02_rb,0.57895e+02_rb,0.56204e+02_rb /)
      kbo(:,22,15) = (/ &
        & 0.64562e+02_rb,0.62695e+02_rb,0.60841e+02_rb,0.59040e+02_rb,0.57268e+02_rb /)
      kbo(:,23,15) = (/ &
        & 0.65690e+02_rb,0.63742e+02_rb,0.61820e+02_rb,0.59937e+02_rb,0.58087e+02_rb /)
      kbo(:,24,15) = (/ &
        & 0.66548e+02_rb,0.64535e+02_rb,0.62550e+02_rb,0.60597e+02_rb,0.58690e+02_rb /)
      kbo(:,25,15) = (/ &
        & 0.67194e+02_rb,0.65115e+02_rb,0.63062e+02_rb,0.61050e+02_rb,0.59091e+02_rb /)
      kbo(:,26,15) = (/ &
        & 0.67597e+02_rb,0.65466e+02_rb,0.63369e+02_rb,0.61312e+02_rb,0.59309e+02_rb /)
      kbo(:,27,15) = (/ &
        & 0.67831e+02_rb,0.65647e+02_rb,0.63518e+02_rb,0.61430e+02_rb,0.59393e+02_rb /)
      kbo(:,28,15) = (/ &
        & 0.67908e+02_rb,0.65687e+02_rb,0.63527e+02_rb,0.61427e+02_rb,0.59357e+02_rb /)
      kbo(:,29,15) = (/ &
        & 0.67845e+02_rb,0.65603e+02_rb,0.63419e+02_rb,0.61304e+02_rb,0.59222e+02_rb /)
      kbo(:,30,15) = (/ &
        & 0.67682e+02_rb,0.65422e+02_rb,0.63225e+02_rb,0.61094e+02_rb,0.59010e+02_rb /)
      kbo(:,31,15) = (/ &
        & 0.67415e+02_rb,0.65154e+02_rb,0.62953e+02_rb,0.60816e+02_rb,0.58721e+02_rb /)
      kbo(:,32,15) = (/ &
        & 0.67084e+02_rb,0.64817e+02_rb,0.62622e+02_rb,0.60477e+02_rb,0.58393e+02_rb /)
      kbo(:,33,15) = (/ &
        & 0.66692e+02_rb,0.64428e+02_rb,0.62242e+02_rb,0.60092e+02_rb,0.58024e+02_rb /)
      kbo(:,34,15) = (/ &
        & 0.66346e+02_rb,0.64077e+02_rb,0.61890e+02_rb,0.59745e+02_rb,0.57700e+02_rb /)
      kbo(:,35,15) = (/ &
        & 0.66188e+02_rb,0.63920e+02_rb,0.61727e+02_rb,0.59582e+02_rb,0.57542e+02_rb /)
      kbo(:,36,15) = (/ &
        & 0.66248e+02_rb,0.63978e+02_rb,0.61782e+02_rb,0.59625e+02_rb,0.57587e+02_rb /)
      kbo(:,37,15) = (/ &
        & 0.66642e+02_rb,0.64358e+02_rb,0.62144e+02_rb,0.59978e+02_rb,0.57911e+02_rb /)
      kbo(:,38,15) = (/ &
        & 0.67048e+02_rb,0.64747e+02_rb,0.62516e+02_rb,0.60340e+02_rb,0.58249e+02_rb /)
      kbo(:,39,15) = (/ &
        & 0.67435e+02_rb,0.65122e+02_rb,0.62879e+02_rb,0.60689e+02_rb,0.58578e+02_rb /)
      kbo(:,40,15) = (/ &
        & 0.68061e+02_rb,0.65724e+02_rb,0.63462e+02_rb,0.61257e+02_rb,0.59113e+02_rb /)
      kbo(:,41,15) = (/ &
        & 0.68702e+02_rb,0.66354e+02_rb,0.64067e+02_rb,0.61847e+02_rb,0.59682e+02_rb /)
      kbo(:,42,15) = (/ &
        & 0.69355e+02_rb,0.66983e+02_rb,0.64678e+02_rb,0.62437e+02_rb,0.60259e+02_rb /)
      kbo(:,43,15) = (/ &
        & 0.70142e+02_rb,0.67744e+02_rb,0.65416e+02_rb,0.63159e+02_rb,0.60959e+02_rb /)
      kbo(:,44,15) = (/ &
        & 0.70991e+02_rb,0.68567e+02_rb,0.66218e+02_rb,0.63930e+02_rb,0.61713e+02_rb /)
      kbo(:,45,15) = (/ &
        & 0.71837e+02_rb,0.69401e+02_rb,0.67026e+02_rb,0.64713e+02_rb,0.62473e+02_rb /)
      kbo(:,46,15) = (/ &
        & 0.72733e+02_rb,0.70287e+02_rb,0.67885e+02_rb,0.65552e+02_rb,0.63292e+02_rb /)
      kbo(:,47,15) = (/ &
        & 0.73741e+02_rb,0.71285e+02_rb,0.68859e+02_rb,0.66493e+02_rb,0.64198e+02_rb /)
      kbo(:,48,15) = (/ &
        & 0.74725e+02_rb,0.72283e+02_rb,0.69840e+02_rb,0.67442e+02_rb,0.65125e+02_rb /)
      kbo(:,49,15) = (/ &
        & 0.75665e+02_rb,0.73280e+02_rb,0.70828e+02_rb,0.68404e+02_rb,0.66060e+02_rb /)
      kbo(:,50,15) = (/ &
        & 0.76534e+02_rb,0.74218e+02_rb,0.71764e+02_rb,0.69323e+02_rb,0.66942e+02_rb /)
      kbo(:,51,15) = (/ &
        & 0.77333e+02_rb,0.75093e+02_rb,0.72664e+02_rb,0.70214e+02_rb,0.67806e+02_rb /)
      kbo(:,52,15) = (/ &
        & 0.78102e+02_rb,0.75935e+02_rb,0.73563e+02_rb,0.71111e+02_rb,0.68683e+02_rb /)
      kbo(:,53,15) = (/ &
        & 0.78815e+02_rb,0.76761e+02_rb,0.74462e+02_rb,0.72010e+02_rb,0.69568e+02_rb /)
      kbo(:,54,15) = (/ &
        & 0.79431e+02_rb,0.77488e+02_rb,0.75258e+02_rb,0.72840e+02_rb,0.70386e+02_rb /)
      kbo(:,55,15) = (/ &
        & 0.79987e+02_rb,0.78156e+02_rb,0.75999e+02_rb,0.73628e+02_rb,0.71177e+02_rb /)
      kbo(:,56,15) = (/ &
        & 0.80478e+02_rb,0.78776e+02_rb,0.76714e+02_rb,0.74415e+02_rb,0.71963e+02_rb /)
      kbo(:,57,15) = (/ &
        & 0.80898e+02_rb,0.79365e+02_rb,0.77410e+02_rb,0.75181e+02_rb,0.72750e+02_rb /)
      kbo(:,58,15) = (/ &
        & 0.81210e+02_rb,0.79894e+02_rb,0.78053e+02_rb,0.75880e+02_rb,0.73498e+02_rb /)
      kbo(:,59,15) = (/ &
        & 0.81312e+02_rb,0.80086e+02_rb,0.78299e+02_rb,0.76156e+02_rb,0.73801e+02_rb /)
      kbo(:,13,16) = (/ &
        & 0.81866e+02_rb,0.78569e+02_rb,0.75624e+02_rb,0.72854e+02_rb,0.70288e+02_rb /)
      kbo(:,14,16) = (/ &
        & 0.93095e+02_rb,0.88986e+02_rb,0.85351e+02_rb,0.82051e+02_rb,0.78936e+02_rb /)
      kbo(:,15,16) = (/ &
        & 0.10470e+03_rb,0.99738e+02_rb,0.95227e+02_rb,0.91249e+02_rb,0.87478e+02_rb /)
      kbo(:,16,16) = (/ &
        & 0.11635e+03_rb,0.11037e+03_rb,0.10495e+03_rb,0.10011e+03_rb,0.95646e+02_rb /)
      kbo(:,17,16) = (/ &
        & 0.12769e+03_rb,0.12074e+03_rb,0.11431e+03_rb,0.10856e+03_rb,0.10332e+03_rb /)
      kbo(:,18,16) = (/ &
        & 0.13851e+03_rb,0.13038e+03_rb,0.12299e+03_rb,0.11635e+03_rb,0.11033e+03_rb /)
      kbo(:,19,16) = (/ &
        & 0.14848e+03_rb,0.13912e+03_rb,0.13084e+03_rb,0.12336e+03_rb,0.11656e+03_rb /)
      kbo(:,20,16) = (/ &
        & 0.15718e+03_rb,0.14679e+03_rb,0.13761e+03_rb,0.12934e+03_rb,0.12188e+03_rb /)
      kbo(:,21,16) = (/ &
        & 0.16462e+03_rb,0.15341e+03_rb,0.14345e+03_rb,0.13443e+03_rb,0.12636e+03_rb /)
      kbo(:,22,16) = (/ &
        & 0.17012e+03_rb,0.15822e+03_rb,0.14760e+03_rb,0.13801e+03_rb,0.12950e+03_rb /)
      kbo(:,23,16) = (/ &
        & 0.17437e+03_rb,0.16184e+03_rb,0.15069e+03_rb,0.14069e+03_rb,0.13184e+03_rb /)
      kbo(:,24,16) = (/ &
        & 0.17733e+03_rb,0.16436e+03_rb,0.15282e+03_rb,0.14251e+03_rb,0.13340e+03_rb /)
      kbo(:,25,16) = (/ &
        & 0.17910e+03_rb,0.16583e+03_rb,0.15401e+03_rb,0.14354e+03_rb,0.13426e+03_rb /)
      kbo(:,26,16) = (/ &
        & 0.17968e+03_rb,0.16627e+03_rb,0.15433e+03_rb,0.14378e+03_rb,0.13445e+03_rb /)
      kbo(:,27,16) = (/ &
        & 0.17946e+03_rb,0.16602e+03_rb,0.15406e+03_rb,0.14351e+03_rb,0.13417e+03_rb /)
      kbo(:,28,16) = (/ &
        & 0.17860e+03_rb,0.16522e+03_rb,0.15334e+03_rb,0.14282e+03_rb,0.13355e+03_rb /)
      kbo(:,29,16) = (/ &
        & 0.17717e+03_rb,0.16389e+03_rb,0.15216e+03_rb,0.14174e+03_rb,0.13260e+03_rb /)
      kbo(:,30,16) = (/ &
        & 0.17533e+03_rb,0.16225e+03_rb,0.15066e+03_rb,0.14043e+03_rb,0.13143e+03_rb /)
      kbo(:,31,16) = (/ &
        & 0.17313e+03_rb,0.16029e+03_rb,0.14890e+03_rb,0.13886e+03_rb,0.13004e+03_rb /)
      kbo(:,32,16) = (/ &
        & 0.17069e+03_rb,0.15811e+03_rb,0.14694e+03_rb,0.13716e+03_rb,0.12847e+03_rb /)
      kbo(:,33,16) = (/ &
        & 0.16804e+03_rb,0.15577e+03_rb,0.14487e+03_rb,0.13533e+03_rb,0.12682e+03_rb /)
      kbo(:,34,16) = (/ &
        & 0.16580e+03_rb,0.15376e+03_rb,0.14309e+03_rb,0.13376e+03_rb,0.12539e+03_rb /)
      kbo(:,35,16) = (/ &
        & 0.16467e+03_rb,0.15275e+03_rb,0.14221e+03_rb,0.13298e+03_rb,0.12468e+03_rb /)
      kbo(:,36,16) = (/ &
        & 0.16486e+03_rb,0.15289e+03_rb,0.14233e+03_rb,0.13308e+03_rb,0.12477e+03_rb /)
      kbo(:,37,16) = (/ &
        & 0.16688e+03_rb,0.15469e+03_rb,0.14392e+03_rb,0.13446e+03_rb,0.12602e+03_rb /)
      kbo(:,38,16) = (/ &
        & 0.16902e+03_rb,0.15657e+03_rb,0.14555e+03_rb,0.13592e+03_rb,0.12732e+03_rb /)
      kbo(:,39,16) = (/ &
        & 0.17110e+03_rb,0.15844e+03_rb,0.14718e+03_rb,0.13734e+03_rb,0.12861e+03_rb /)
      kbo(:,40,16) = (/ &
        & 0.17468e+03_rb,0.16155e+03_rb,0.14992e+03_rb,0.13976e+03_rb,0.13077e+03_rb /)
      kbo(:,41,16) = (/ &
        & 0.17848e+03_rb,0.16489e+03_rb,0.15289e+03_rb,0.14236e+03_rb,0.13307e+03_rb /)
      kbo(:,42,16) = (/ &
        & 0.18237e+03_rb,0.16833e+03_rb,0.15592e+03_rb,0.14501e+03_rb,0.13541e+03_rb /)
      kbo(:,43,16) = (/ &
        & 0.18726e+03_rb,0.17263e+03_rb,0.15975e+03_rb,0.14834e+03_rb,0.13835e+03_rb /)
      kbo(:,44,16) = (/ &
        & 0.19275e+03_rb,0.17748e+03_rb,0.16403e+03_rb,0.15210e+03_rb,0.14166e+03_rb /)
      kbo(:,45,16) = (/ &
        & 0.19851e+03_rb,0.18250e+03_rb,0.16844e+03_rb,0.15602e+03_rb,0.14507e+03_rb /)
      kbo(:,46,16) = (/ &
        & 0.20490e+03_rb,0.18806e+03_rb,0.17333e+03_rb,0.16038e+03_rb,0.14888e+03_rb /)
      kbo(:,47,16) = (/ &
        & 0.21238e+03_rb,0.19462e+03_rb,0.17910e+03_rb,0.16545e+03_rb,0.15337e+03_rb /)
      kbo(:,48,16) = (/ &
        & 0.22029e+03_rb,0.20156e+03_rb,0.18514e+03_rb,0.17074e+03_rb,0.15806e+03_rb /)
      kbo(:,49,16) = (/ &
        & 0.22889e+03_rb,0.20882e+03_rb,0.19150e+03_rb,0.17637e+03_rb,0.16304e+03_rb /)
      kbo(:,50,16) = (/ &
        & 0.23745e+03_rb,0.21596e+03_rb,0.19782e+03_rb,0.18187e+03_rb,0.16787e+03_rb /)
      kbo(:,51,16) = (/ &
        & 0.24629e+03_rb,0.22340e+03_rb,0.20422e+03_rb,0.18747e+03_rb,0.17280e+03_rb /)
      kbo(:,52,16) = (/ &
        & 0.25553e+03_rb,0.23136e+03_rb,0.21090e+03_rb,0.19335e+03_rb,0.17798e+03_rb /)
      kbo(:,53,16) = (/ &
        & 0.26553e+03_rb,0.23980e+03_rb,0.21793e+03_rb,0.19951e+03_rb,0.18336e+03_rb /)
      kbo(:,54,16) = (/ &
        & 0.27524e+03_rb,0.24799e+03_rb,0.22485e+03_rb,0.20544e+03_rb,0.18854e+03_rb /)
      kbo(:,55,16) = (/ &
        & 0.28502e+03_rb,0.25615e+03_rb,0.23188e+03_rb,0.21133e+03_rb,0.19372e+03_rb /)
      kbo(:,56,16) = (/ &
        & 0.29529e+03_rb,0.26491e+03_rb,0.23927e+03_rb,0.21749e+03_rb,0.19913e+03_rb /)
      kbo(:,57,16) = (/ &
        & 0.30624e+03_rb,0.27413e+03_rb,0.24703e+03_rb,0.22404e+03_rb,0.20478e+03_rb /)
      kbo(:,58,16) = (/ &
        & 0.31727e+03_rb,0.28329e+03_rb,0.25471e+03_rb,0.23065e+03_rb,0.21031e+03_rb /)
      kbo(:,59,16) = (/ &
        & 0.32188e+03_rb,0.28710e+03_rb,0.25796e+03_rb,0.23341e+03_rb,0.21261e+03_rb /)

!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.214504e-06_rb, 0.460418e-06_rb, 0.357608e-05_rb, 0.192037e-05_rb /)
      forrefo(:, 2) = (/ 0.142576e-05_rb, 0.364463e-05_rb, 0.117033e-04_rb, 0.112085e-04_rb /)
      forrefo(:, 3) = (/ 0.101536e-04_rb, 0.124096e-04_rb, 0.509190e-04_rb, 0.565282e-04_rb /)
      forrefo(:, 4) = (/ 0.143394e-03_rb, 0.154700e-03_rb, 0.466498e-03_rb, 0.918829e-03_rb /)
      forrefo(:, 5) = (/ 0.251631e-02_rb, 0.241729e-02_rb, 0.240057e-02_rb, 0.350408e-02_rb /)
      forrefo(:, 6) = (/ 0.410309e-02_rb, 0.416851e-02_rb, 0.390925e-02_rb, 0.383694e-02_rb /)
      forrefo(:, 7) = (/ 0.445387e-02_rb, 0.448657e-02_rb, 0.432310e-02_rb, 0.370739e-02_rb /)
      forrefo(:, 8) = (/ 0.458150e-02_rb, 0.460014e-02_rb, 0.450245e-02_rb, 0.336718e-02_rb /)
      forrefo(:, 9) = (/ 0.465423e-02_rb, 0.465595e-02_rb, 0.467006e-02_rb, 0.368061e-02_rb /)
      forrefo(:,10) = (/ 0.493955e-02_rb, 0.490181e-02_rb, 0.481941e-02_rb, 0.367577e-02_rb /)
      forrefo(:,11) = (/ 0.511876e-02_rb, 0.490981e-02_rb, 0.493303e-02_rb, 0.357423e-02_rb /)
      forrefo(:,12) = (/ 0.509845e-02_rb, 0.511556e-02_rb, 0.504031e-02_rb, 0.355915e-02_rb /)
      forrefo(:,13) = (/ 0.523822e-02_rb, 0.530473e-02_rb, 0.523811e-02_rb, 0.414259e-02_rb /)
      forrefo(:,14) = (/ 0.551133e-02_rb, 0.535831e-02_rb, 0.546702e-02_rb, 0.473875e-02_rb /)
      forrefo(:,15) = (/ 0.609781e-02_rb, 0.589859e-02_rb, 0.561187e-02_rb, 0.528981e-02_rb /)
      forrefo(:,16) = (/ 0.644958e-02_rb, 0.631718e-02_rb, 0.625201e-02_rb, 0.600448e-02_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
        &  0.217058e-03_rb, 0.176391e-03_rb, 0.143342e-03_rb, 0.116486e-03_rb, 0.946614e-04_rb, &
        &  0.769257e-04_rb, 0.625131e-04_rb, 0.508007e-04_rb, 0.412828e-04_rb, 0.335481e-04_rb /)
      selfrefo(:, 2) = (/ &
        &  0.598055e-03_rb, 0.484805e-03_rb, 0.393000e-03_rb, 0.318580e-03_rb, 0.258252e-03_rb, &
        &  0.209348e-03_rb, 0.169705e-03_rb, 0.137569e-03_rb, 0.111518e-03_rb, 0.904008e-04_rb /)
      selfrefo(:, 3) = (/ &
        &  0.102691e-02_rb, 0.930281e-03_rb, 0.842740e-03_rb, 0.763437e-03_rb, 0.691596e-03_rb, &
        &  0.626516e-03_rb, 0.567560e-03_rb, 0.514152e-03_rb, 0.465769e-03_rb, 0.421940e-03_rb /)
      selfrefo(:, 4) = (/ &
        &  0.388569e-02_rb, 0.365098e-02_rb, 0.343045e-02_rb, 0.322324e-02_rb, 0.302854e-02_rb, &
        &  0.284561e-02_rb, 0.267372e-02_rb, 0.251222e-02_rb, 0.236047e-02_rb, 0.221789e-02_rb /)
      selfrefo(:, 5) = (/ &
        &  0.349845e-01_rb, 0.326678e-01_rb, 0.305045e-01_rb, 0.284845e-01_rb, 0.265982e-01_rb, &
        &  0.248369e-01_rb, 0.231921e-01_rb, 0.216563e-01_rb, 0.202222e-01_rb, 0.188831e-01_rb /)
      selfrefo(:, 6) = (/ &
        &  0.613705e-01_rb, 0.562676e-01_rb, 0.515890e-01_rb, 0.472994e-01_rb, 0.433665e-01_rb, &
        &  0.397606e-01_rb, 0.364545e-01_rb, 0.334233e-01_rb, 0.306442e-01_rb, 0.280961e-01_rb /)
      selfrefo(:, 7) = (/ &
        &  0.656981e-01_rb, 0.602660e-01_rb, 0.552830e-01_rb, 0.507120e-01_rb, 0.465190e-01_rb, &
        &  0.426726e-01_rb, 0.391443e-01_rb, 0.359077e-01_rb, 0.329387e-01_rb, 0.302153e-01_rb /)
      selfrefo(:, 8) = (/ &
        &  0.671782e-01_rb, 0.616461e-01_rb, 0.565695e-01_rb, 0.519110e-01_rb, 0.476361e-01_rb, &
        &  0.437132e-01_rb, 0.401134e-01_rb, 0.368100e-01_rb, 0.337787e-01_rb, 0.309970e-01_rb /)
      selfrefo(:, 9) = (/ &
        &  0.675902e-01_rb, 0.620888e-01_rb, 0.570351e-01_rb, 0.523928e-01_rb, 0.481284e-01_rb, &
        &  0.442110e-01_rb, 0.406125e-01_rb, 0.373069e-01_rb, 0.342703e-01_rb, 0.314809e-01_rb /)
      selfrefo(:,10) = (/ &
        &  0.708308e-01_rb, 0.651419e-01_rb, 0.599099e-01_rb, 0.550981e-01_rb, 0.506728e-01_rb, &
        &  0.466030e-01_rb, 0.428600e-01_rb, 0.394176e-01_rb, 0.362517e-01_rb, 0.333401e-01_rb /)
      selfrefo(:,11) = (/ &
        &  0.698445e-01_rb, 0.646584e-01_rb, 0.598573e-01_rb, 0.554128e-01_rb, 0.512982e-01_rb, &
        &  0.474892e-01_rb, 0.439630e-01_rb, 0.406986e-01_rb, 0.376766e-01_rb, 0.348791e-01_rb /)
      selfrefo(:,12) = (/ &
        &  0.743921e-01_rb, 0.682057e-01_rb, 0.625337e-01_rb, 0.573334e-01_rb, 0.525655e-01_rb, &
        &  0.481942e-01_rb, 0.441863e-01_rb, 0.405118e-01_rb, 0.371428e-01_rb, 0.340540e-01_rb /)
      selfrefo(:,13) = (/ &
        &  0.775758e-01_rb, 0.709818e-01_rb, 0.649484e-01_rb, 0.594277e-01_rb, 0.543764e-01_rb, &
        &  0.497544e-01_rb, 0.455253e-01_rb, 0.416556e-01_rb, 0.381149e-01_rb, 0.348751e-01_rb /)
      selfrefo(:,14) = (/ &
        &  0.776545e-01_rb, 0.714761e-01_rb, 0.657894e-01_rb, 0.605550e-01_rb, 0.557372e-01_rb, &
        &  0.513026e-01_rb, 0.472209e-01_rb, 0.434639e-01_rb, 0.400058e-01_rb, 0.368229e-01_rb /)
      selfrefo(:,15) = (/ &
        &  0.855675e-01_rb, 0.787337e-01_rb, 0.724456e-01_rb, 0.666598e-01_rb, 0.613360e-01_rb, &
        &  0.564374e-01_rb, 0.519301e-01_rb, 0.477827e-01_rb, 0.439666e-01_rb, 0.404552e-01_rb /)
      selfrefo(:,16) = (/ &
        &  0.934781e-01_rb, 0.855190e-01_rb, 0.782376e-01_rb, 0.715761e-01_rb, 0.654819e-01_rb, &
        &  0.599065e-01_rb, 0.548058e-01_rb, 0.501394e-01_rb, 0.458704e-01_rb, 0.419648e-01_rb /)
     
      end subroutine sw_kgb20

! **************************************************************************
      subroutine sw_kgb21
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg21, only : kao, kbo, selfrefo, forrefo, sfluxrefo, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:, 1) = (/ &
        & 16.1643_rb , 15.5806_rb, 14.7254_rb    , 13.5541_rb    , &
        & 11.9519_rb ,10.44410_rb, 8.37884_rb    , 6.26384_rb    , &
        & 4.28435_rb ,0.465228_rb, 0.385095_rb   ,0.304226_rb    , &
        & 0.222479_rb,0.143286_rb, 5.58046e-02_rb, 7.84856e-03_rb /)
      sfluxrefo(:, 2) = (/ &
        & 15.6451_rb , 15.3170_rb, 14.6987_rb    , 13.7350_rb    , &
        & 12.2267_rb ,10.51646_rb, 8.47150_rb    , 6.38873_rb    , &
        & 4.33536_rb ,0.470610_rb,0.389426_rb    ,0.306461_rb    , &
        & 0.223537_rb,0.143273_rb, 5.58179e-02_rb, 7.84856e-03_rb /)
      sfluxrefo(:, 3) = (/ &
        & 15.6092_rb , 15.3293_rb, 14.6881_rb    , 13.6693_rb    , &
        & 12.2342_rb ,10.52010_rb, 8.49442_rb    , 6.42138_rb    , &
        & 4.35865_rb ,0.473349_rb,0.391349_rb    ,0.308861_rb    , &
        & 0.224666_rb,0.144799_rb, 5.58176e-02_rb, 7.84881e-03_rb /)
      sfluxrefo(:, 4) = (/ &
        & 15.5786_rb , 15.3422_rb, 14.6894_rb    , 13.6040_rb    , &
        & 12.2567_rb ,10.49400_rb, 8.53521_rb    , 6.44427_rb    , &
        & 4.37208_rb ,0.475709_rb,0.392956_rb    ,0.309737_rb    , &
        & 0.226274_rb,0.146483_rb, 5.59325e-02_rb, 7.84881e-03_rb /)
      sfluxrefo(:, 5) = (/ &
        & 15.5380_rb , 15.3826_rb, 14.6575_rb    , 13.5722_rb    , &
        & 12.2646_rb ,10.47672_rb, 8.57158_rb    , 6.46343_rb    , &
        & 4.38259_rb ,0.477647_rb,0.393982_rb    ,0.310686_rb    , &
        & 0.227620_rb,0.148376_rb, 5.60398e-02_rb, 7.83925e-03_rb /)
      sfluxrefo(:, 6) = (/ &
        & 15.5124_rb , 15.3986_rb, 14.6240_rb    , 13.5535_rb    , &
        & 12.2468_rb ,10.48891_rb, 8.60434_rb    , 6.47985_rb    , &
        & 4.39448_rb ,0.478267_rb,0.395618_rb    ,0.311043_rb    , &
        & 0.230927_rb,0.148774_rb, 5.61189e-02_rb, 7.83925e-03_rb /)
      sfluxrefo(:, 7) = (/ &
        & 15.4910_rb , 15.4028_rb, 14.5772_rb    , 13.5507_rb    , &
        & 12.2122_rb ,10.52735_rb, 8.62650_rb    , 6.49644_rb    , &
        & 4.41173_rb ,0.478627_rb,0.396433_rb    ,0.314199_rb    , & 
        & 0.233125_rb,0.149052_rb, 5.62309e-02_rb, 7.83925e-03_rb /)
      sfluxrefo(:, 8) = (/ &
        & 15.4562_rb , 15.3928_rb, 14.5510_rb    , 13.5122_rb    , &
        & 12.1890_rb , 10.5826_rb, 8.65842_rb    , 6.51558_rb    , &
        & 4.42747_rb ,0.480669_rb,0.400143_rb    ,0.318144_rb    , &
        & 0.233937_rb,0.149119_rb, 5.62309e-02_rb, 7.83925e-03_rb /)
      sfluxrefo(:, 9) = (/ &
        & 15.0069_rb , 15.1479_rb, 14.7802_rb    , 13.6085_rb    , &
        & 12.2793_rb , 10.6929_rb, 8.72723_rb    , 6.57114_rb    , &
        & 4.46330_rb ,0.486724_rb,0.401446_rb    ,0.318879_rb    , &
        & 0.233959_rb,0.149119_rb, 5.62309e-02_rb, 7.83925e-03_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:,1) = (/ &
        & 1.56071E+01_rb, 1.54390E+01_rb, 1.49345E+01_rb, 1.39032E+01_rb,&
        & 1.25400E+01_rb, 1.07748E+01_rb, 8.63517E+00_rb, 6.39660E+00_rb,&
        & 4.35522E+00_rb, 4.73554E-01_rb, 3.91046E-01_rb, 3.11643E-01_rb,&
        & 2.33087E-01_rb, 1.49309E-01_rb, 5.63709E-02_rb, 7.88922E-03_rb/)
      irradnceo(:,2) = (/ &
        & 1.55995E+01_rb, 1.54301E+01_rb, 1.49290E+01_rb, 1.39584E+01_rb,&
        & 1.24932E+01_rb, 1.06948E+01_rb, 8.62922E+00_rb, 6.46028E+00_rb,&
        & 4.38917E+00_rb, 4.75159E-01_rb, 3.91265E-01_rb, 3.11758E-01_rb,&
        & 2.33093E-01_rb, 1.49311E-01_rb, 5.63706E-02_rb, 7.88922E-03_rb/)
      irradnceo(:,3) = (/ &
        & 1.56032E+01_rb, 1.54253E+01_rb, 1.49489E+01_rb, 1.39461E+01_rb,&
        & 1.24554E+01_rb, 1.06361E+01_rb, 8.64814E+00_rb, 6.50387E+00_rb,&
        & 4.40742E+00_rb, 4.78377E-01_rb, 3.94523E-01_rb, 3.13744E-01_rb,&
        & 2.33768E-01_rb, 1.49318E-01_rb, 5.63704E-02_rb, 7.88922E-03_rb/)
      irradnceo(:,4) = (/ &
        & 1.56051E+01_rb, 1.54153E+01_rb, 1.49735E+01_rb, 1.39297E+01_rb,&
        & 1.24106E+01_rb, 1.06271E+01_rb, 8.65339E+00_rb, 6.52896E+00_rb,&
        & 4.42207E+00_rb, 4.80073E-01_rb, 3.97385E-01_rb, 3.17578E-01_rb,&
        & 2.34116E-01_rb, 1.49503E-01_rb, 5.63711E-02_rb, 7.88925E-03_rb/)
      irradnceo(:,5) = (/ &
        & 1.56008E+01_rb, 1.54155E+01_rb, 1.50018E+01_rb, 1.39039E+01_rb,&
        & 1.23474E+01_rb, 1.06282E+01_rb, 8.67837E+00_rb, 6.54743E+00_rb,&
        & 4.43313E+00_rb, 4.81572E-01_rb, 4.02297E-01_rb, 3.19653E-01_rb,&
        & 2.34468E-01_rb, 1.49681E-01_rb, 5.63642E-02_rb, 7.92441E-03_rb/)
      irradnceo(:,6) = (/ &
        & 1.55982E+01_rb, 1.54209E+01_rb, 1.50169E+01_rb, 1.38609E+01_rb,&
        & 1.23130E+01_rb, 1.06292E+01_rb, 8.70194E+00_rb, 6.56377E+00_rb,&
        & 4.44289E+00_rb, 4.87042E-01_rb, 4.04729E-01_rb, 3.19915E-01_rb,&
        & 2.34990E-01_rb, 1.49703E-01_rb, 5.64482E-02_rb, 7.92441E-03_rb/)
      irradnceo(:,7) = (/ &
        & 1.55915E+01_rb, 1.54476E+01_rb, 1.50072E+01_rb, 1.37911E+01_rb,&
        & 1.22982E+01_rb, 1.06419E+01_rb, 8.73003E+00_rb, 6.57639E+00_rb,&
        & 4.46045E+00_rb, 4.89337E-01_rb, 4.05126E-01_rb, 3.20423E-01_rb,&
        & 2.35102E-01_rb, 1.49777E-01_rb, 5.64481E-02_rb, 7.92441E-03_rb/)
      irradnceo(:,8) = (/ &
        & 1.55773E+01_rb, 1.54964E+01_rb, 1.49853E+01_rb, 1.36933E+01_rb,&
        & 1.22974E+01_rb, 1.06694E+01_rb, 8.74767E+00_rb, 6.58874E+00_rb,&
        & 4.48764E+00_rb, 4.89983E-01_rb, 4.05507E-01_rb, 3.20536E-01_rb,&
        & 2.35164E-01_rb, 1.49777E-01_rb, 5.64482E-02_rb, 7.92441E-03_rb/)
      irradnceo(:,9) = (/ &
        & 1.52609E+01_rb, 1.53728E+01_rb, 1.50458E+01_rb, 1.38029E+01_rb,&
        & 1.24079E+01_rb, 1.07261E+01_rb, 8.79668E+00_rb, 6.63033E+00_rb,&
        & 4.49972E+00_rb, 4.90122E-01_rb, 4.05516E-01_rb, 3.20536E-01_rb,&
        & 2.35166E-01_rb, 1.49778E-01_rb, 5.64480E-02_rb, 7.92441E-03_rb/)
! Solar variability components: facular brightening
      facbrghto(:,1) = (/ &
        & 3.26788E-03_rb, 3.34175E-03_rb, 3.86321E-03_rb, 3.70941E-03_rb,&
        & 3.39992E-03_rb, 2.85385E-03_rb, 1.88671E-03_rb, 1.08743E-03_rb,&
        & 7.88966E-04_rb, 8.42840E-05_rb, 6.87570E-05_rb, 5.69869E-05_rb,&
        & 5.60352E-05_rb, 3.57519E-05_rb, 1.34449E-05_rb, 1.90320E-06_rb/)
      facbrghto(:,2) = (/ &
        & 3.25871E-03_rb, 3.33519E-03_rb, 3.83168E-03_rb, 3.76748E-03_rb,&
        & 3.32592E-03_rb, 2.64083E-03_rb, 1.84082E-03_rb, 1.30428E-03_rb,&
        & 8.88498E-04_rb, 8.89463E-05_rb, 6.96730E-05_rb, 5.71215E-05_rb,&
        & 5.60354E-05_rb, 3.57521E-05_rb, 1.34452E-05_rb, 1.90320E-06_rb/)
      facbrghto(:,3) = (/ &
        & 3.25195E-03_rb, 3.34465E-03_rb, 3.84595E-03_rb, 3.75231E-03_rb,&
        & 3.22755E-03_rb, 2.52232E-03_rb, 1.86695E-03_rb, 1.41431E-03_rb,&
        & 9.41971E-04_rb, 9.67370E-05_rb, 7.94170E-05_rb, 6.43058E-05_rb,&
        & 5.67713E-05_rb, 3.57521E-05_rb, 1.34454E-05_rb, 1.90320E-06_rb/)
      facbrghto(:,4) = (/ &
        & 3.24377E-03_rb, 3.35681E-03_rb, 3.86427E-03_rb, 3.70606E-03_rb,&
        & 3.12086E-03_rb, 2.47294E-03_rb, 1.91762E-03_rb, 1.48017E-03_rb,&
        & 9.77041E-04_rb, 1.04186E-04_rb, 8.33604E-05_rb, 7.85543E-05_rb,&
        & 5.88195E-05_rb, 3.64847E-05_rb, 1.34448E-05_rb, 1.90323E-06_rb/)
      facbrghto(:,5) = (/ &
        & 3.23274E-03_rb, 3.37478E-03_rb, 3.88106E-03_rb, 3.65308E-03_rb,&
        & 2.99201E-03_rb, 2.44305E-03_rb, 1.99974E-03_rb, 1.52814E-03_rb,&
        & 1.00838E-03_rb, 1.06314E-04_rb, 1.00488E-04_rb, 8.25151E-05_rb,&
        & 5.99135E-05_rb, 3.77030E-05_rb, 1.41837E-05_rb, 2.17964E-06_rb/)
      facbrghto(:,6) = (/ &
        & 3.22130E-03_rb, 3.39738E-03_rb, 3.88828E-03_rb, 3.53656E-03_rb,&
        & 2.91013E-03_rb, 2.45455E-03_rb, 2.07165E-03_rb, 1.56693E-03_rb,&
        & 1.03807E-03_rb, 1.24210E-04_rb, 1.05815E-04_rb, 8.36783E-05_rb,&
        & 6.06767E-05_rb, 3.95029E-05_rb, 1.53755E-05_rb, 2.17964E-06_rb/)
      facbrghto(:,7) = (/ &
        & 3.20968E-03_rb, 3.42400E-03_rb, 3.89803E-03_rb, 3.35860E-03_rb,&
        & 2.82647E-03_rb, 2.53160E-03_rb, 2.13238E-03_rb, 1.60375E-03_rb,&
        & 1.08954E-03_rb, 1.29347E-04_rb, 1.06542E-04_rb, 8.43823E-05_rb,&
        & 6.36298E-05_rb, 4.08073E-05_rb, 1.53760E-05_rb, 2.17964E-06_rb/)
      facbrghto(:,8) = (/ &
        & 3.18115E-03_rb, 3.50045E-03_rb, 3.84620E-03_rb, 3.09074E-03_rb,&
        & 2.82361E-03_rb, 2.63195E-03_rb, 2.18672E-03_rb, 1.64304E-03_rb,&
        & 1.16161E-03_rb, 1.30955E-04_rb, 1.10050E-04_rb, 8.72763E-05_rb,&
        & 6.41842E-05_rb, 4.08072E-05_rb, 1.53755E-05_rb, 2.17964E-06_rb/)
      facbrghto(:,9) = (/ &
        & 2.07139E-03_rb, 3.32945E-03_rb, 3.93127E-03_rb, 3.47902E-03_rb,&
        & 3.13407E-03_rb, 2.81818E-03_rb, 2.33260E-03_rb, 1.76176E-03_rb,&
        & 1.20457E-03_rb, 1.33668E-04_rb, 1.10454E-04_rb, 8.73068E-05_rb,&
        & 6.41825E-05_rb, 4.08217E-05_rb, 1.53638E-05_rb, 2.17964E-06_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:,1) = (/ &
        &-3.16124E-03_rb,-3.17919E-03_rb,-3.26650E-03_rb,-3.07257E-03_rb,&
        &-2.78014E-03_rb,-2.35578E-03_rb,-1.75235E-03_rb,-1.17834E-03_rb,&
        &-8.20839E-04_rb,-8.82927E-05_rb,-7.26292E-05_rb,-5.85720E-05_rb,&
        &-4.93046E-05_rb,-3.14126E-05_rb,-1.18349E-05_rb,-1.66534E-06_rb/)
      snsptdrko(:,2) = (/ &
        &-3.15579E-03_rb,-3.17738E-03_rb,-3.24883E-03_rb,-3.08536E-03_rb,&
        &-2.74115E-03_rb,-2.28539E-03_rb,-1.73594E-03_rb,-1.27119E-03_rb,&
        &-8.63414E-04_rb,-9.03379E-05_rb,-7.30244E-05_rb,-5.86315E-05_rb,&
        &-4.93045E-05_rb,-3.14126E-05_rb,-1.18350E-05_rb,-1.66534E-06_rb/)
      snsptdrko(:,3) = (/ &
        &-3.15156E-03_rb,-3.18098E-03_rb,-3.25121E-03_rb,-3.07218E-03_rb,&
        &-2.70706E-03_rb,-2.23931E-03_rb,-1.74617E-03_rb,-1.31906E-03_rb,&
        &-8.85765E-04_rb,-9.37403E-05_rb,-7.73020E-05_rb,-6.18032E-05_rb,&
        &-4.96059E-05_rb,-3.14127E-05_rb,-1.18350E-05_rb,-1.66534E-06_rb/)
      snsptdrko(:,4) = (/ &
        &-3.14654E-03_rb,-3.18548E-03_rb,-3.25612E-03_rb,-3.04895E-03_rb,&
        &-2.67115E-03_rb,-2.21717E-03_rb,-1.76746E-03_rb,-1.34766E-03_rb,&
        &-9.00370E-04_rb,-9.69758E-05_rb,-7.90149E-05_rb,-6.80642E-05_rb,&
        &-5.04917E-05_rb,-3.17197E-05_rb,-1.18350E-05_rb,-1.66533E-06_rb/)
      snsptdrko(:,5) = (/ &
        &-3.14033E-03_rb,-3.19114E-03_rb,-3.26120E-03_rb,-3.02480E-03_rb,&
        &-2.62346E-03_rb,-2.20421E-03_rb,-1.80226E-03_rb,-1.36827E-03_rb,&
        &-9.13656E-04_rb,-9.79161E-05_rb,-8.65111E-05_rb,-6.97817E-05_rb,&
        &-5.09421E-05_rb,-3.22664E-05_rb,-1.21416E-05_rb,-1.78054E-06_rb/)
      snsptdrko(:,6) = (/ &
        &-3.13372E-03_rb,-3.19755E-03_rb,-3.26211E-03_rb,-2.98193E-03_rb,&
        &-2.58812E-03_rb,-2.20891E-03_rb,-1.83391E-03_rb,-1.38430E-03_rb,&
        &-9.26522E-04_rb,-1.05739E-04_rb,-8.88822E-05_rb,-7.02583E-05_rb,&
        &-5.12442E-05_rb,-3.30384E-05_rb,-1.26557E-05_rb,-1.78054E-06_rb/)
      snsptdrko(:,7) = (/ &
        &-3.12617E-03_rb,-3.20318E-03_rb,-3.26092E-03_rb,-2.91912E-03_rb,&
        &-2.55222E-03_rb,-2.24208E-03_rb,-1.86052E-03_rb,-1.39916E-03_rb,&
        &-9.49097E-04_rb,-1.07964E-04_rb,-8.91535E-05_rb,-7.05523E-05_rb,&
        &-5.24949E-05_rb,-3.35932E-05_rb,-1.26558E-05_rb,-1.78054E-06_rb/)
      snsptdrko(:,8) = (/ &
        &-3.10846E-03_rb,-3.22490E-03_rb,-3.23631E-03_rb,-2.82182E-03_rb,&
        &-2.55131E-03_rb,-2.28688E-03_rb,-1.88295E-03_rb,-1.41556E-03_rb,&
        &-9.80693E-04_rb,-1.08634E-04_rb,-9.06019E-05_rb,-7.17839E-05_rb,&
        &-5.27263E-05_rb,-3.35929E-05_rb,-1.26557E-05_rb,-1.78054E-06_rb/)
      snsptdrko(:,9) = (/ &
        &-2.61118E-03_rb,-3.10499E-03_rb,-3.30441E-03_rb,-3.01024E-03_rb,&
        &-2.69703E-03_rb,-2.36728E-03_rb,-1.94610E-03_rb,-1.46704E-03_rb,&
        &-9.99284E-04_rb,-1.09762E-04_rb,-9.08007E-05_rb,-7.17903E-05_rb,&
        &-5.27247E-05_rb,-3.36002E-05_rb,-1.26510E-05_rb,-1.78054E-06_rb/)

! Rayleigh extinction coefficient at v = 6925 cm-1.
      rayl = 9.41e-09_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.31482e-07_rb,0.64010e-05_rb,0.95017e-05_rb,0.11735e-04_rb,0.13561e-04_rb, &
        & 0.15077e-04_rb,0.16121e-04_rb,0.16138e-04_rb,0.41687e-06_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.29791e-07_rb,0.68206e-05_rb,0.10260e-04_rb,0.12764e-04_rb,0.14880e-04_rb, &
        & 0.16665e-04_rb,0.17944e-04_rb,0.18170e-04_rb,0.44246e-06_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.28272e-07_rb,0.72448e-05_rb,0.11083e-04_rb,0.13866e-04_rb,0.16256e-04_rb, &
        & 0.18323e-04_rb,0.19886e-04_rb,0.20229e-04_rb,0.47472e-06_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.26900e-07_rb,0.76935e-05_rb,0.11934e-04_rb,0.15015e-04_rb,0.17657e-04_rb, &
        & 0.20038e-04_rb,0.21902e-04_rb,0.22361e-04_rb,0.50770e-06_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.25656e-07_rb,0.81661e-05_rb,0.12769e-04_rb,0.16181e-04_rb,0.19127e-04_rb, &
        & 0.21755e-04_rb,0.23963e-04_rb,0.24421e-04_rb,0.54205e-06_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.26396e-07_rb,0.52225e-05_rb,0.77221e-05_rb,0.95150e-05_rb,0.11004e-04_rb, &
        & 0.12195e-04_rb,0.13037e-04_rb,0.12985e-04_rb,0.32097e-06_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.24945e-07_rb,0.55737e-05_rb,0.83612e-05_rb,0.10375e-04_rb,0.12116e-04_rb, &
        & 0.13569e-04_rb,0.14561e-04_rb,0.14711e-04_rb,0.34579e-06_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.23646e-07_rb,0.59333e-05_rb,0.90531e-05_rb,0.11301e-04_rb,0.13247e-04_rb, &
        & 0.14956e-04_rb,0.16193e-04_rb,0.16396e-04_rb,0.37200e-06_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.22476e-07_rb,0.63144e-05_rb,0.97732e-05_rb,0.12274e-04_rb,0.14436e-04_rb, &
        & 0.16377e-04_rb,0.17879e-04_rb,0.18223e-04_rb,0.40051e-06_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.21415e-07_rb,0.67133e-05_rb,0.10471e-04_rb,0.13264e-04_rb,0.15681e-04_rb, &
        & 0.17843e-04_rb,0.19627e-04_rb,0.19954e-04_rb,0.42929e-06_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.22397e-07_rb,0.41930e-05_rb,0.61559e-05_rb,0.75544e-05_rb,0.86902e-05_rb, &
        & 0.95896e-05_rb,0.10269e-04_rb,0.10114e-04_rb,0.24148e-06_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.21124e-07_rb,0.44837e-05_rb,0.66786e-05_rb,0.82475e-05_rb,0.96353e-05_rb, &
        & 0.10742e-04_rb,0.11469e-04_rb,0.11556e-04_rb,0.26304e-06_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.19988e-07_rb,0.47839e-05_rb,0.72499e-05_rb,0.90229e-05_rb,0.10560e-04_rb, &
        & 0.11900e-04_rb,0.12859e-04_rb,0.12935e-04_rb,0.28618e-06_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.18968e-07_rb,0.51040e-05_rb,0.78586e-05_rb,0.98308e-05_rb,0.11551e-04_rb, &
        & 0.13094e-04_rb,0.14240e-04_rb,0.14455e-04_rb,0.30983e-06_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.18047e-07_rb,0.54366e-05_rb,0.84462e-05_rb,0.10670e-04_rb,0.12585e-04_rb, &
        & 0.14313e-04_rb,0.15695e-04_rb,0.15934e-04_rb,0.33255e-06_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.19080e-07_rb,0.33409e-05_rb,0.48738e-05_rb,0.59354e-05_rb,0.67877e-05_rb, &
        & 0.74900e-05_rb,0.79142e-05_rb,0.77200e-05_rb,0.18466e-06_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.17954e-07_rb,0.35910e-05_rb,0.52960e-05_rb,0.65037e-05_rb,0.75744e-05_rb, &
        & 0.84082e-05_rb,0.89422e-05_rb,0.89302e-05_rb,0.20292e-06_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.16954e-07_rb,0.38317e-05_rb,0.57536e-05_rb,0.71322e-05_rb,0.83424e-05_rb, &
        & 0.93788e-05_rb,0.10063e-04_rb,0.10098e-04_rb,0.22173e-06_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.16059e-07_rb,0.40932e-05_rb,0.62586e-05_rb,0.77961e-05_rb,0.91475e-05_rb, &
        & 0.10347e-04_rb,0.11222e-04_rb,0.11312e-04_rb,0.24005e-06_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.15254e-07_rb,0.43678e-05_rb,0.67525e-05_rb,0.84873e-05_rb,0.99995e-05_rb, &
        & 0.11353e-04_rb,0.12416e-04_rb,0.12586e-04_rb,0.25939e-06_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.16261e-07_rb,0.26540e-05_rb,0.38541e-05_rb,0.46443e-05_rb,0.52961e-05_rb, &
        & 0.58054e-05_rb,0.60718e-05_rb,0.58540e-05_rb,0.14274e-06_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.15265e-07_rb,0.28598e-05_rb,0.41831e-05_rb,0.51221e-05_rb,0.59110e-05_rb, &
        & 0.65207e-05_rb,0.69761e-05_rb,0.68685e-05_rb,0.15725e-06_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.14384e-07_rb,0.30623e-05_rb,0.45528e-05_rb,0.56124e-05_rb,0.65650e-05_rb, &
        & 0.73436e-05_rb,0.78208e-05_rb,0.78523e-05_rb,0.17271e-06_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.13599e-07_rb,0.32720e-05_rb,0.49622e-05_rb,0.61605e-05_rb,0.72244e-05_rb, &
        & 0.81582e-05_rb,0.87971e-05_rb,0.88096e-05_rb,0.18766e-06_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.12895e-07_rb,0.34985e-05_rb,0.53833e-05_rb,0.67240e-05_rb,0.79091e-05_rb, &
        & 0.89688e-05_rb,0.97698e-05_rb,0.98639e-05_rb,0.20375e-06_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.13898e-07_rb,0.20981e-05_rb,0.30384e-05_rb,0.36041e-05_rb,0.41040e-05_rb, &
        & 0.44334e-05_rb,0.45761e-05_rb,0.43950e-05_rb,0.11025e-06_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.13011e-07_rb,0.22645e-05_rb,0.32890e-05_rb,0.40042e-05_rb,0.45851e-05_rb, &
        & 0.50473e-05_rb,0.53250e-05_rb,0.51854e-05_rb,0.12227e-06_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.12231e-07_rb,0.24322e-05_rb,0.35842e-05_rb,0.43915e-05_rb,0.51259e-05_rb, &
        & 0.56903e-05_rb,0.60343e-05_rb,0.60166e-05_rb,0.13506e-06_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.11539e-07_rb,0.26032e-05_rb,0.39107e-05_rb,0.48321e-05_rb,0.56525e-05_rb, &
        & 0.63625e-05_rb,0.68078e-05_rb,0.68024e-05_rb,0.14719e-06_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.10921e-07_rb,0.27866e-05_rb,0.42578e-05_rb,0.52919e-05_rb,0.62181e-05_rb, &
        & 0.70362e-05_rb,0.76140e-05_rb,0.76519e-05_rb,0.16052e-06_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.11886e-07_rb,0.16563e-05_rb,0.23860e-05_rb,0.28276e-05_rb,0.31449e-05_rb, &
        & 0.33771e-05_rb,0.34305e-05_rb,0.32824e-05_rb,0.84943e-07_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.11096e-07_rb,0.17874e-05_rb,0.25861e-05_rb,0.31100e-05_rb,0.35425e-05_rb, &
        & 0.38737e-05_rb,0.40468e-05_rb,0.38933e-05_rb,0.94734e-07_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.10405e-07_rb,0.19286e-05_rb,0.28166e-05_rb,0.34417e-05_rb,0.39786e-05_rb, &
        & 0.43804e-05_rb,0.46759e-05_rb,0.45835e-05_rb,0.10531e-06_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.97948e-08_rb,0.20680e-05_rb,0.30741e-05_rb,0.37803e-05_rb,0.44163e-05_rb, &
        & 0.49420e-05_rb,0.52477e-05_rb,0.52509e-05_rb,0.11517e-06_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.92522e-08_rb,0.22140e-05_rb,0.33558e-05_rb,0.41544e-05_rb,0.48719e-05_rb, &
        & 0.54968e-05_rb,0.59225e-05_rb,0.59027e-05_rb,0.12618e-06_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.10164e-07_rb,0.13056e-05_rb,0.18690e-05_rb,0.21940e-05_rb,0.24210e-05_rb, &
        & 0.25504e-05_rb,0.25676e-05_rb,0.24444e-05_rb,0.66021e-07_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.94612e-08_rb,0.14106e-05_rb,0.20398e-05_rb,0.24136e-05_rb,0.27424e-05_rb, &
        & 0.29578e-05_rb,0.30446e-05_rb,0.29182e-05_rb,0.73985e-07_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.88489e-08_rb,0.15256e-05_rb,0.22123e-05_rb,0.26894e-05_rb,0.30704e-05_rb, &
        & 0.33808e-05_rb,0.35521e-05_rb,0.34526e-05_rb,0.82666e-07_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.83111e-08_rb,0.16396e-05_rb,0.24159e-05_rb,0.29563e-05_rb,0.34504e-05_rb, &
        & 0.38196e-05_rb,0.40425e-05_rb,0.40093e-05_rb,0.90940e-07_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.78349e-08_rb,0.17570e-05_rb,0.26398e-05_rb,0.32547e-05_rb,0.38060e-05_rb, &
        & 0.42813e-05_rb,0.45732e-05_rb,0.45465e-05_rb,0.10022e-06_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.86983e-08_rb,0.10284e-05_rb,0.14734e-05_rb,0.16973e-05_rb,0.18551e-05_rb, &
        & 0.19223e-05_rb,0.19222e-05_rb,0.18289e-05_rb,0.53460e-07_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.80712e-08_rb,0.11141e-05_rb,0.16016e-05_rb,0.18902e-05_rb,0.21027e-05_rb, &
        & 0.22504e-05_rb,0.22800e-05_rb,0.21749e-05_rb,0.60180e-07_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.75284e-08_rb,0.12033e-05_rb,0.17376e-05_rb,0.20849e-05_rb,0.23719e-05_rb, &
        & 0.25906e-05_rb,0.26977e-05_rb,0.25910e-05_rb,0.66886e-07_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.70541e-08_rb,0.12979e-05_rb,0.18959e-05_rb,0.23090e-05_rb,0.26695e-05_rb, &
        & 0.29367e-05_rb,0.31239e-05_rb,0.30574e-05_rb,0.73978e-07_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.66359e-08_rb,0.13937e-05_rb,0.20720e-05_rb,0.25448e-05_rb,0.29680e-05_rb, &
        & 0.33208e-05_rb,0.35191e-05_rb,0.35097e-05_rb,0.81724e-07_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.74006e-08_rb,0.81705e-06_rb,0.11610e-05_rb,0.13325e-05_rb,0.14305e-05_rb, &
        & 0.14668e-05_rb,0.14642e-05_rb,0.13765e-05_rb,0.44724e-07_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.68477e-08_rb,0.88306e-06_rb,0.12640e-05_rb,0.14768e-05_rb,0.16329e-05_rb, &
        & 0.17209e-05_rb,0.17306e-05_rb,0.16441e-05_rb,0.50688e-07_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.63717e-08_rb,0.95522e-06_rb,0.13783e-05_rb,0.16303e-05_rb,0.18546e-05_rb, &
        & 0.19963e-05_rb,0.20570e-05_rb,0.19678e-05_rb,0.56052e-07_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.59576e-08_rb,0.10331e-05_rb,0.14975e-05_rb,0.18180e-05_rb,0.20770e-05_rb, &
        & 0.22813e-05_rb,0.24021e-05_rb,0.23350e-05_rb,0.62412e-07_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.55940e-08_rb,0.11102e-05_rb,0.16370e-05_rb,0.19999e-05_rb,0.23349e-05_rb, &
        & 0.25900e-05_rb,0.27328e-05_rb,0.27057e-05_rb,0.69466e-07_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.60617e-08_rb,0.67167e-06_rb,0.95594e-06_rb,0.10939e-05_rb,0.11767e-05_rb, &
        & 0.12059e-05_rb,0.12032e-05_rb,0.11314e-05_rb,0.37093e-07_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.56087e-08_rb,0.72704e-06_rb,0.10406e-05_rb,0.12137e-05_rb,0.13434e-05_rb, &
        & 0.14171e-05_rb,0.14246e-05_rb,0.13521e-05_rb,0.41918e-07_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.52187e-08_rb,0.78681e-06_rb,0.11338e-05_rb,0.13419e-05_rb,0.15254e-05_rb, &
        & 0.16426e-05_rb,0.16950e-05_rb,0.16203e-05_rb,0.46339e-07_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.48793e-08_rb,0.85013e-06_rb,0.12327e-05_rb,0.14971e-05_rb,0.17109e-05_rb, &
        & 0.18776e-05_rb,0.19784e-05_rb,0.19240e-05_rb,0.51725e-07_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.45815e-08_rb,0.91420e-06_rb,0.13427e-05_rb,0.16458e-05_rb,0.19221e-05_rb, &
        & 0.21323e-05_rb,0.22490e-05_rb,0.22263e-05_rb,0.57555e-07_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.49629e-08_rb,0.55223e-06_rb,0.78677e-06_rb,0.89789e-06_rb,0.96778e-06_rb, &
        & 0.99148e-06_rb,0.98883e-06_rb,0.93028e-06_rb,0.30610e-07_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.45920e-08_rb,0.59832e-06_rb,0.85632e-06_rb,0.99730e-06_rb,0.11052e-05_rb, &
        & 0.11665e-05_rb,0.11729e-05_rb,0.11125e-05_rb,0.34553e-07_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.42727e-08_rb,0.64760e-06_rb,0.93315e-06_rb,0.11041e-05_rb,0.12552e-05_rb, &
        & 0.13521e-05_rb,0.13956e-05_rb,0.13333e-05_rb,0.38176e-07_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.39949e-08_rb,0.69926e-06_rb,0.10110e-05_rb,0.12304e-05_rb,0.14083e-05_rb, &
        & 0.15442e-05_rb,0.16283e-05_rb,0.15837e-05_rb,0.42648e-07_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.37510e-08_rb,0.75239e-06_rb,0.11006e-05_rb,0.13535e-05_rb,0.15785e-05_rb, &
        & 0.17547e-05_rb,0.18501e-05_rb,0.18298e-05_rb,0.47452e-07_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.40633e-08_rb,0.45378e-06_rb,0.64719e-06_rb,0.73637e-06_rb,0.79571e-06_rb, &
        & 0.81503e-06_rb,0.81264e-06_rb,0.76488e-06_rb,0.25080e-07_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.37596e-08_rb,0.49208e-06_rb,0.70430e-06_rb,0.81894e-06_rb,0.90856e-06_rb, &
        & 0.95919e-06_rb,0.96435e-06_rb,0.91411e-06_rb,0.28293e-07_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.34982e-08_rb,0.53259e-06_rb,0.76370e-06_rb,0.90761e-06_rb,0.10316e-05_rb, &
        & 0.11116e-05_rb,0.11472e-05_rb,0.10956e-05_rb,0.31267e-07_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.32707e-08_rb,0.57488e-06_rb,0.82833e-06_rb,0.10096e-05_rb,0.11577e-05_rb, &
        & 0.12688e-05_rb,0.13388e-05_rb,0.13015e-05_rb,0.34930e-07_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.30710e-08_rb,0.61832e-06_rb,0.90250e-06_rb,0.11120e-05_rb,0.12962e-05_rb, &
        & 0.14424e-05_rb,0.15206e-05_rb,0.15032e-05_rb,0.38861e-07_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.14215e-06_rb,0.65726e-04_rb,0.11327e-03_rb,0.15324e-03_rb,0.18473e-03_rb, &
        & 0.20658e-03_rb,0.21603e-03_rb,0.20617e-03_rb,0.13252e-04_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.13452e-06_rb,0.69521e-04_rb,0.12059e-03_rb,0.16342e-03_rb,0.19613e-03_rb, &
        & 0.21987e-03_rb,0.22946e-03_rb,0.21879e-03_rb,0.14816e-04_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.12766e-06_rb,0.73313e-04_rb,0.12786e-03_rb,0.17303e-03_rb,0.20754e-03_rb, &
        & 0.23326e-03_rb,0.24324e-03_rb,0.22956e-03_rb,0.16536e-04_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.12147e-06_rb,0.77087e-04_rb,0.13521e-03_rb,0.18263e-03_rb,0.21904e-03_rb, &
        & 0.24614e-03_rb,0.25637e-03_rb,0.24060e-03_rb,0.18092e-04_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.11584e-06_rb,0.80946e-04_rb,0.14233e-03_rb,0.19230e-03_rb,0.23047e-03_rb, &
        & 0.25869e-03_rb,0.26904e-03_rb,0.25139e-03_rb,0.19768e-04_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.11918e-06_rb,0.55167e-04_rb,0.94557e-04_rb,0.12760e-03_rb,0.15322e-03_rb, &
        & 0.17148e-03_rb,0.17894e-03_rb,0.17049e-03_rb,0.10710e-04_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.11264e-06_rb,0.58581e-04_rb,0.10113e-03_rb,0.13648e-03_rb,0.16335e-03_rb, &
        & 0.18288e-03_rb,0.19059e-03_rb,0.18131e-03_rb,0.12067e-04_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.10678e-06_rb,0.61935e-04_rb,0.10756e-03_rb,0.14497e-03_rb,0.17335e-03_rb, &
        & 0.19437e-03_rb,0.20245e-03_rb,0.19070e-03_rb,0.13553e-04_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.10149e-06_rb,0.65261e-04_rb,0.11404e-03_rb,0.15331e-03_rb,0.18323e-03_rb, &
        & 0.20559e-03_rb,0.21385e-03_rb,0.20024e-03_rb,0.14801e-04_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.96699e-07_rb,0.68698e-04_rb,0.12038e-03_rb,0.16167e-03_rb,0.19308e-03_rb, &
        & 0.21635e-03_rb,0.22479e-03_rb,0.20964e-03_rb,0.16245e-04_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.10112e-06_rb,0.45399e-04_rb,0.77226e-04_rb,0.10397e-03_rb,0.12462e-03_rb, &
        & 0.13956e-03_rb,0.14531e-03_rb,0.13816e-03_rb,0.83635e-05_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.95381e-07_rb,0.48357e-04_rb,0.82986e-04_rb,0.11173e-03_rb,0.13338e-03_rb, &
        & 0.14898e-03_rb,0.15534e-03_rb,0.14786e-03_rb,0.94915e-05_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.90257e-07_rb,0.51355e-04_rb,0.88727e-04_rb,0.11920e-03_rb,0.14215e-03_rb, &
        & 0.15859e-03_rb,0.16541e-03_rb,0.15622e-03_rb,0.10683e-04_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.85653e-07_rb,0.54322e-04_rb,0.94369e-04_rb,0.12646e-03_rb,0.15072e-03_rb, &
        & 0.16838e-03_rb,0.17536e-03_rb,0.16433e-03_rb,0.11764e-04_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.81492e-07_rb,0.57300e-04_rb,0.99940e-04_rb,0.13355e-03_rb,0.15929e-03_rb, &
        & 0.17800e-03_rb,0.18496e-03_rb,0.17234e-03_rb,0.12979e-04_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.86132e-07_rb,0.36910e-04_rb,0.62346e-04_rb,0.83628e-04_rb,0.10025e-03_rb, &
        & 0.11224e-03_rb,0.11706e-03_rb,0.11109e-03_rb,0.64715e-05_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.81062e-07_rb,0.39490e-04_rb,0.67274e-04_rb,0.90384e-04_rb,0.10785e-03_rb, &
        & 0.12026e-03_rb,0.12543e-03_rb,0.11944e-03_rb,0.73991e-05_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.76553e-07_rb,0.42055e-04_rb,0.72269e-04_rb,0.96956e-04_rb,0.11528e-03_rb, &
        & 0.12833e-03_rb,0.13396e-03_rb,0.12697e-03_rb,0.83899e-05_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.72516e-07_rb,0.44696e-04_rb,0.77235e-04_rb,0.10321e-03_rb,0.12276e-03_rb, &
        & 0.13674e-03_rb,0.14260e-03_rb,0.13384e-03_rb,0.93052e-05_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.68881e-07_rb,0.47273e-04_rb,0.82118e-04_rb,0.10935e-03_rb,0.13016e-03_rb, &
        & 0.14506e-03_rb,0.15080e-03_rb,0.14062e-03_rb,0.10314e-04_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.73401e-07_rb,0.29772e-04_rb,0.49957e-04_rb,0.66720e-04_rb,0.80104e-04_rb, &
        & 0.89761e-04_rb,0.93853e-04_rb,0.88953e-04_rb,0.50171e-05_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.68916e-07_rb,0.31976e-04_rb,0.54217e-04_rb,0.72570e-04_rb,0.86671e-04_rb, &
        & 0.96653e-04_rb,0.10073e-03_rb,0.95875e-04_rb,0.57746e-05_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.64945e-07_rb,0.34219e-04_rb,0.58417e-04_rb,0.78327e-04_rb,0.92950e-04_rb, &
        & 0.10358e-03_rb,0.10803e-03_rb,0.10274e-03_rb,0.66197e-05_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.61405e-07_rb,0.36457e-04_rb,0.62694e-04_rb,0.83756e-04_rb,0.99356e-04_rb, &
        & 0.11058e-03_rb,0.11542e-03_rb,0.10858e-03_rb,0.73729e-05_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.58229e-07_rb,0.38713e-04_rb,0.67059e-04_rb,0.89033e-04_rb,0.10573e-03_rb, &
        & 0.11764e-03_rb,0.12243e-03_rb,0.11440e-03_rb,0.82149e-05_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.62723e-07_rb,0.23750e-04_rb,0.39643e-04_rb,0.52796e-04_rb,0.63478e-04_rb, &
        & 0.71151e-04_rb,0.74640e-04_rb,0.70193e-04_rb,0.38899e-05_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.58735e-07_rb,0.25657e-04_rb,0.43274e-04_rb,0.57719e-04_rb,0.69124e-04_rb, &
        & 0.77375e-04_rb,0.80498e-04_rb,0.76543e-04_rb,0.44833e-05_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.55222e-07_rb,0.27563e-04_rb,0.46866e-04_rb,0.62699e-04_rb,0.74494e-04_rb, &
        & 0.82946e-04_rb,0.86653e-04_rb,0.82509e-04_rb,0.51635e-05_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.52103e-07_rb,0.29528e-04_rb,0.50470e-04_rb,0.67473e-04_rb,0.79899e-04_rb, &
        & 0.89004e-04_rb,0.92786e-04_rb,0.87628e-04_rb,0.58104e-05_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.49317e-07_rb,0.31444e-04_rb,0.54175e-04_rb,0.72000e-04_rb,0.85322e-04_rb, &
        & 0.94885e-04_rb,0.98826e-04_rb,0.92648e-04_rb,0.64997e-05_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.53634e-07_rb,0.18862e-04_rb,0.31325e-04_rb,0.41575e-04_rb,0.50141e-04_rb, &
        & 0.56190e-04_rb,0.58882e-04_rb,0.55026e-04_rb,0.29795e-05_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.50084e-07_rb,0.20474e-04_rb,0.34342e-04_rb,0.45732e-04_rb,0.54884e-04_rb, &
        & 0.61494e-04_rb,0.64187e-04_rb,0.60857e-04_rb,0.34589e-05_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.46973e-07_rb,0.22063e-04_rb,0.37411e-04_rb,0.49893e-04_rb,0.59475e-04_rb, &
        & 0.66279e-04_rb,0.69129e-04_rb,0.65758e-04_rb,0.39983e-05_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.44224e-07_rb,0.23764e-04_rb,0.40481e-04_rb,0.54107e-04_rb,0.64026e-04_rb, &
        & 0.71350e-04_rb,0.74328e-04_rb,0.70513e-04_rb,0.45526e-05_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.41777e-07_rb,0.25434e-04_rb,0.43555e-04_rb,0.57958e-04_rb,0.68599e-04_rb, &
        & 0.76304e-04_rb,0.79456e-04_rb,0.74649e-04_rb,0.51046e-05_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.45857e-07_rb,0.14945e-04_rb,0.24594e-04_rb,0.32696e-04_rb,0.39571e-04_rb, &
        & 0.44348e-04_rb,0.46090e-04_rb,0.43034e-04_rb,0.22833e-05_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.42697e-07_rb,0.16265e-04_rb,0.27198e-04_rb,0.36108e-04_rb,0.43381e-04_rb, &
        & 0.48642e-04_rb,0.50943e-04_rb,0.47848e-04_rb,0.26669e-05_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.39944e-07_rb,0.17616e-04_rb,0.29775e-04_rb,0.39577e-04_rb,0.47366e-04_rb, &
        & 0.52791e-04_rb,0.54992e-04_rb,0.52236e-04_rb,0.30932e-05_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.37522e-07_rb,0.19042e-04_rb,0.32351e-04_rb,0.43126e-04_rb,0.51153e-04_rb, &
        & 0.56987e-04_rb,0.59316e-04_rb,0.56438e-04_rb,0.35666e-05_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.35376e-07_rb,0.20476e-04_rb,0.34946e-04_rb,0.46521e-04_rb,0.55009e-04_rb, &
        & 0.61221e-04_rb,0.63714e-04_rb,0.60035e-04_rb,0.40037e-05_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.39233e-07_rb,0.11817e-04_rb,0.19254e-04_rb,0.25596e-04_rb,0.30843e-04_rb, &
        & 0.34574e-04_rb,0.36035e-04_rb,0.33425e-04_rb,0.17673e-05_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.36418e-07_rb,0.12914e-04_rb,0.21404e-04_rb,0.28372e-04_rb,0.34161e-04_rb, &
        & 0.38321e-04_rb,0.40053e-04_rb,0.37435e-04_rb,0.20751e-05_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.33979e-07_rb,0.14026e-04_rb,0.23574e-04_rb,0.31316e-04_rb,0.37514e-04_rb, &
        & 0.41977e-04_rb,0.43747e-04_rb,0.41346e-04_rb,0.24151e-05_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.31844e-07_rb,0.15205e-04_rb,0.25770e-04_rb,0.34231e-04_rb,0.40723e-04_rb, &
        & 0.45315e-04_rb,0.47202e-04_rb,0.44869e-04_rb,0.28151e-05_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.29961e-07_rb,0.16420e-04_rb,0.27955e-04_rb,0.37187e-04_rb,0.43941e-04_rb, &
        & 0.48873e-04_rb,0.50856e-04_rb,0.48170e-04_rb,0.31733e-05_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.33373e-07_rb,0.93505e-05_rb,0.15216e-04_rb,0.20198e-04_rb,0.24244e-04_rb, &
        & 0.27077e-04_rb,0.28186e-04_rb,0.25996e-04_rb,0.13941e-05_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.30893e-07_rb,0.10284e-04_rb,0.16902e-04_rb,0.22468e-04_rb,0.27072e-04_rb, &
        & 0.30382e-04_rb,0.31603e-04_rb,0.29456e-04_rb,0.16352e-05_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.28754e-07_rb,0.11207e-04_rb,0.18715e-04_rb,0.24849e-04_rb,0.29815e-04_rb, &
        & 0.33379e-04_rb,0.34901e-04_rb,0.32836e-04_rb,0.19188e-05_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.26892e-07_rb,0.12172e-04_rb,0.20578e-04_rb,0.27269e-04_rb,0.32536e-04_rb, &
        & 0.36182e-04_rb,0.37703e-04_rb,0.35750e-04_rb,0.22467e-05_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.25255e-07_rb,0.13195e-04_rb,0.22422e-04_rb,0.29781e-04_rb,0.35220e-04_rb, &
        & 0.39157e-04_rb,0.40719e-04_rb,0.38729e-04_rb,0.25535e-05_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.27335e-07_rb,0.77491e-05_rb,0.12607e-04_rb,0.16695e-04_rb,0.20093e-04_rb, &
        & 0.22431e-04_rb,0.23345e-04_rb,0.21522e-04_rb,0.11569e-05_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.25303e-07_rb,0.85322e-05_rb,0.14022e-04_rb,0.18593e-04_rb,0.22378e-04_rb, &
        & 0.25133e-04_rb,0.26161e-04_rb,0.24379e-04_rb,0.13709e-05_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.23551e-07_rb,0.92912e-05_rb,0.15536e-04_rb,0.20618e-04_rb,0.24722e-04_rb, &
        & 0.27647e-04_rb,0.28874e-04_rb,0.27196e-04_rb,0.16099e-05_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.22025e-07_rb,0.10117e-04_rb,0.17115e-04_rb,0.22645e-04_rb,0.26974e-04_rb, &
        & 0.29947e-04_rb,0.31210e-04_rb,0.29581e-04_rb,0.18845e-05_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.20684e-07_rb,0.10977e-04_rb,0.18645e-04_rb,0.24734e-04_rb,0.29203e-04_rb, &
        & 0.32453e-04_rb,0.33733e-04_rb,0.32056e-04_rb,0.21326e-05_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.22380e-07_rb,0.64144e-05_rb,0.10436e-04_rb,0.13802e-04_rb,0.16622e-04_rb, &
        & 0.18557e-04_rb,0.19314e-04_rb,0.17793e-04_rb,0.95906e-06_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.20716e-07_rb,0.70603e-05_rb,0.11625e-04_rb,0.15385e-04_rb,0.18503e-04_rb, &
        & 0.20776e-04_rb,0.21630e-04_rb,0.20152e-04_rb,0.11363e-05_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.19282e-07_rb,0.76960e-05_rb,0.12886e-04_rb,0.17087e-04_rb,0.20475e-04_rb, &
        & 0.22867e-04_rb,0.23856e-04_rb,0.22506e-04_rb,0.13400e-05_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.18032e-07_rb,0.83983e-05_rb,0.14206e-04_rb,0.18783e-04_rb,0.22334e-04_rb, &
        & 0.24764e-04_rb,0.25815e-04_rb,0.24461e-04_rb,0.15637e-05_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.16934e-07_rb,0.91186e-05_rb,0.15491e-04_rb,0.20513e-04_rb,0.24201e-04_rb, &
        & 0.26864e-04_rb,0.27917e-04_rb,0.26491e-04_rb,0.17707e-05_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.18323e-07_rb,0.53022e-05_rb,0.86273e-05_rb,0.11385e-04_rb,0.13729e-04_rb, &
        & 0.15318e-04_rb,0.15952e-04_rb,0.14693e-04_rb,0.78927e-06_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.16961e-07_rb,0.58316e-05_rb,0.96204e-05_rb,0.12717e-04_rb,0.15275e-04_rb, &
        & 0.17141e-04_rb,0.17863e-04_rb,0.16639e-04_rb,0.93659e-06_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.15787e-07_rb,0.63710e-05_rb,0.10675e-04_rb,0.14140e-04_rb,0.16925e-04_rb, &
        & 0.18881e-04_rb,0.19675e-04_rb,0.18564e-04_rb,0.11050e-05_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.14764e-07_rb,0.69607e-05_rb,0.11779e-04_rb,0.15554e-04_rb,0.18475e-04_rb, &
        & 0.20461e-04_rb,0.21320e-04_rb,0.20207e-04_rb,0.12868e-05_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.13865e-07_rb,0.75611e-05_rb,0.12843e-04_rb,0.16991e-04_rb,0.20029e-04_rb, &
        & 0.22208e-04_rb,0.23079e-04_rb,0.21838e-04_rb,0.14565e-05_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.55948e-06_rb,0.43472e-03_rb,0.65689e-03_rb,0.80237e-03_rb,0.90326e-03_rb, &
        & 0.95995e-03_rb,0.94242e-03_rb,0.81801e-03_rb,0.18380e-03_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.53175e-06_rb,0.45395e-03_rb,0.68779e-03_rb,0.84040e-03_rb,0.94598e-03_rb, &
        & 0.10011e-02_rb,0.98481e-03_rb,0.85748e-03_rb,0.19567e-03_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.50648e-06_rb,0.46835e-03_rb,0.71403e-03_rb,0.87684e-03_rb,0.98520e-03_rb, &
        & 0.10393e-02_rb,0.10233e-02_rb,0.89729e-03_rb,0.20725e-03_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.48330e-06_rb,0.48143e-03_rb,0.73979e-03_rb,0.91063e-03_rb,0.10212e-02_rb, &
        & 0.10762e-02_rb,0.10600e-02_rb,0.93507e-03_rb,0.21814e-03_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.46195e-06_rb,0.49413e-03_rb,0.76470e-03_rb,0.94208e-03_rb,0.10547e-02_rb, &
        & 0.11099e-02_rb,0.10960e-02_rb,0.97164e-03_rb,0.23056e-03_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.46813e-06_rb,0.36711e-03_rb,0.55743e-03_rb,0.68070e-03_rb,0.76614e-03_rb, &
        & 0.81442e-03_rb,0.79940e-03_rb,0.69691e-03_rb,0.14988e-03_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.44450e-06_rb,0.38413e-03_rb,0.58502e-03_rb,0.71506e-03_rb,0.80409e-03_rb, &
        & 0.85191e-03_rb,0.83722e-03_rb,0.73231e-03_rb,0.15985e-03_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.42301e-06_rb,0.39744e-03_rb,0.60852e-03_rb,0.74741e-03_rb,0.83920e-03_rb, &
        & 0.88588e-03_rb,0.87206e-03_rb,0.76677e-03_rb,0.16968e-03_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.40335e-06_rb,0.40927e-03_rb,0.63200e-03_rb,0.77835e-03_rb,0.87190e-03_rb, &
        & 0.91843e-03_rb,0.90467e-03_rb,0.79859e-03_rb,0.17948e-03_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.38529e-06_rb,0.42034e-03_rb,0.65353e-03_rb,0.80758e-03_rb,0.90248e-03_rb, &
        & 0.94929e-03_rb,0.93715e-03_rb,0.83136e-03_rb,0.19018e-03_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.39591e-06_rb,0.30459e-03_rb,0.46364e-03_rb,0.56590e-03_rb,0.63680e-03_rb, &
        & 0.67681e-03_rb,0.66517e-03_rb,0.58106e-03_rb,0.11936e-03_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.37538e-06_rb,0.32036e-03_rb,0.48874e-03_rb,0.59649e-03_rb,0.67170e-03_rb, &
        & 0.71161e-03_rb,0.69943e-03_rb,0.61159e-03_rb,0.12804e-03_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.35675e-06_rb,0.33321e-03_rb,0.50999e-03_rb,0.62591e-03_rb,0.70319e-03_rb, &
        & 0.74333e-03_rb,0.73015e-03_rb,0.64070e-03_rb,0.13642e-03_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.33978e-06_rb,0.34382e-03_rb,0.53102e-03_rb,0.65391e-03_rb,0.73278e-03_rb, &
        & 0.77303e-03_rb,0.75990e-03_rb,0.66979e-03_rb,0.14477e-03_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.32422e-06_rb,0.35416e-03_rb,0.55071e-03_rb,0.68042e-03_rb,0.76041e-03_rb, &
        & 0.80118e-03_rb,0.78914e-03_rb,0.69923e-03_rb,0.15365e-03_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.33598e-06_rb,0.24970e-03_rb,0.38053e-03_rb,0.46464e-03_rb,0.52234e-03_rb, &
        & 0.55488e-03_rb,0.54675e-03_rb,0.47794e-03_rb,0.95275e-04_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.31800e-06_rb,0.26415e-03_rb,0.40313e-03_rb,0.49210e-03_rb,0.55359e-03_rb, &
        & 0.58661e-03_rb,0.57694e-03_rb,0.50393e-03_rb,0.10292e-03_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.30177e-06_rb,0.27638e-03_rb,0.42268e-03_rb,0.51823e-03_rb,0.58348e-03_rb, &
        & 0.61591e-03_rb,0.60497e-03_rb,0.52947e-03_rb,0.11016e-03_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.28703e-06_rb,0.28612e-03_rb,0.44165e-03_rb,0.54326e-03_rb,0.61010e-03_rb, &
        & 0.64299e-03_rb,0.63151e-03_rb,0.55558e-03_rb,0.11708e-03_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.27356e-06_rb,0.29550e-03_rb,0.45903e-03_rb,0.56703e-03_rb,0.63522e-03_rb, &
        & 0.66839e-03_rb,0.65812e-03_rb,0.58142e-03_rb,0.12444e-03_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.28517e-06_rb,0.20305e-03_rb,0.30940e-03_rb,0.37854e-03_rb,0.42516e-03_rb, &
        & 0.45167e-03_rb,0.44617e-03_rb,0.39006e-03_rb,0.76394e-04_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.26943e-06_rb,0.21631e-03_rb,0.32942e-03_rb,0.40279e-03_rb,0.45281e-03_rb, &
        & 0.47959e-03_rb,0.47274e-03_rb,0.41219e-03_rb,0.83093e-04_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.25526e-06_rb,0.22760e-03_rb,0.34736e-03_rb,0.42603e-03_rb,0.47928e-03_rb, &
        & 0.50573e-03_rb,0.49753e-03_rb,0.43423e-03_rb,0.89137e-04_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.24245e-06_rb,0.23661e-03_rb,0.36423e-03_rb,0.44832e-03_rb,0.50373e-03_rb, &
        & 0.53021e-03_rb,0.52092e-03_rb,0.45715e-03_rb,0.95026e-04_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.23079e-06_rb,0.24490e-03_rb,0.37979e-03_rb,0.46958e-03_rb,0.52671e-03_rb, &
        & 0.55360e-03_rb,0.54422e-03_rb,0.47982e-03_rb,0.10100e-03_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.24258e-06_rb,0.16374e-03_rb,0.24889e-03_rb,0.30526e-03_rb,0.34256e-03_rb, &
        & 0.36437e-03_rb,0.36054e-03_rb,0.31643e-03_rb,0.60983e-04_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.22873e-06_rb,0.17557e-03_rb,0.26674e-03_rb,0.32662e-03_rb,0.36698e-03_rb, &
        & 0.38862e-03_rb,0.38395e-03_rb,0.33481e-03_rb,0.66717e-04_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.21632e-06_rb,0.18596e-03_rb,0.28315e-03_rb,0.34710e-03_rb,0.39032e-03_rb, &
        & 0.41186e-03_rb,0.40597e-03_rb,0.35347e-03_rb,0.71805e-04_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.20515e-06_rb,0.19412e-03_rb,0.29825e-03_rb,0.36672e-03_rb,0.41223e-03_rb, &
        & 0.43370e-03_rb,0.42654e-03_rb,0.37308e-03_rb,0.76950e-04_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.19500e-06_rb,0.20163e-03_rb,0.31177e-03_rb,0.38573e-03_rb,0.43298e-03_rb, &
        & 0.45474e-03_rb,0.44659e-03_rb,0.39273e-03_rb,0.81854e-04_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.20644e-06_rb,0.13088e-03_rb,0.19883e-03_rb,0.24453e-03_rb,0.27379e-03_rb, &
        & 0.29133e-03_rb,0.28972e-03_rb,0.25527e-03_rb,0.48483e-04_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.19422e-06_rb,0.14123e-03_rb,0.21464e-03_rb,0.26307e-03_rb,0.29520e-03_rb, &
        & 0.31262e-03_rb,0.30976e-03_rb,0.27057e-03_rb,0.53189e-04_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.18334e-06_rb,0.15056e-03_rb,0.22945e-03_rb,0.28102e-03_rb,0.31570e-03_rb, &
        & 0.33336e-03_rb,0.32921e-03_rb,0.28657e-03_rb,0.57664e-04_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.17359e-06_rb,0.15845e-03_rb,0.24254e-03_rb,0.29812e-03_rb,0.33533e-03_rb, &
        & 0.35273e-03_rb,0.34737e-03_rb,0.30290e-03_rb,0.62080e-04_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.16476e-06_rb,0.16515e-03_rb,0.25472e-03_rb,0.31492e-03_rb,0.35368e-03_rb, &
        & 0.37134e-03_rb,0.36466e-03_rb,0.31971e-03_rb,0.66283e-04_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.17564e-06_rb,0.10377e-03_rb,0.15799e-03_rb,0.19443e-03_rb,0.21775e-03_rb, &
        & 0.23221e-03_rb,0.23178e-03_rb,0.20518e-03_rb,0.38613e-04_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.16486e-06_rb,0.11291e-03_rb,0.17177e-03_rb,0.21060e-03_rb,0.23605e-03_rb, &
        & 0.25023e-03_rb,0.24860e-03_rb,0.21840e-03_rb,0.42574e-04_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.15532e-06_rb,0.12160e-03_rb,0.18472e-03_rb,0.22613e-03_rb,0.25383e-03_rb, &
        & 0.26813e-03_rb,0.26569e-03_rb,0.23149e-03_rb,0.46438e-04_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.14679e-06_rb,0.12871e-03_rb,0.19629e-03_rb,0.24117e-03_rb,0.27101e-03_rb, &
        & 0.28546e-03_rb,0.28148e-03_rb,0.24511e-03_rb,0.50132e-04_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.13913e-06_rb,0.13470e-03_rb,0.20711e-03_rb,0.25583e-03_rb,0.28735e-03_rb, &
        & 0.30171e-03_rb,0.29653e-03_rb,0.25931e-03_rb,0.53892e-04_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.14950e-06_rb,0.81941e-04_rb,0.12473e-03_rb,0.15363e-03_rb,0.17257e-03_rb, &
        & 0.18409e-03_rb,0.18424e-03_rb,0.16439e-03_rb,0.30772e-04_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.13998e-06_rb,0.89702e-04_rb,0.13655e-03_rb,0.16763e-03_rb,0.18779e-03_rb, &
        & 0.19919e-03_rb,0.19877e-03_rb,0.17581e-03_rb,0.34022e-04_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.13160e-06_rb,0.97263e-04_rb,0.14776e-03_rb,0.18089e-03_rb,0.20292e-03_rb, &
        & 0.21451e-03_rb,0.21317e-03_rb,0.18640e-03_rb,0.37381e-04_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.12415e-06_rb,0.10389e-03_rb,0.15817e-03_rb,0.19404e-03_rb,0.21794e-03_rb, &
        & 0.22980e-03_rb,0.22715e-03_rb,0.19784e-03_rb,0.40457e-04_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.11749e-06_rb,0.10934e-03_rb,0.16768e-03_rb,0.20680e-03_rb,0.23234e-03_rb, &
        & 0.24414e-03_rb,0.24026e-03_rb,0.20959e-03_rb,0.43586e-04_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.12658e-06_rb,0.64763e-04_rb,0.98690e-04_rb,0.12170e-03_rb,0.13709e-03_rb, &
        & 0.14644e-03_rb,0.14709e-03_rb,0.13215e-03_rb,0.24737e-04_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.11826e-06_rb,0.71405e-04_rb,0.10875e-03_rb,0.13368e-03_rb,0.14978e-03_rb, &
        & 0.15901e-03_rb,0.15937e-03_rb,0.14153e-03_rb,0.27487e-04_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.11097e-06_rb,0.77857e-04_rb,0.11842e-03_rb,0.14510e-03_rb,0.16261e-03_rb, &
        & 0.17204e-03_rb,0.17129e-03_rb,0.15041e-03_rb,0.30250e-04_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.10451e-06_rb,0.83764e-04_rb,0.12762e-03_rb,0.15625e-03_rb,0.17544e-03_rb, &
        & 0.18516e-03_rb,0.18345e-03_rb,0.16010e-03_rb,0.32891e-04_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.98767e-07_rb,0.88833e-04_rb,0.13595e-03_rb,0.16723e-03_rb,0.18798e-03_rb, &
        & 0.19766e-03_rb,0.19477e-03_rb,0.16979e-03_rb,0.35547e-04_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.10368e-06_rb,0.53714e-04_rb,0.81748e-04_rb,0.10085e-03_rb,0.11349e-03_rb, &
        & 0.12126e-03_rb,0.12180e-03_rb,0.10950e-03_rb,0.20787e-04_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.96854e-07_rb,0.59175e-04_rb,0.90131e-04_rb,0.11085e-03_rb,0.12425e-03_rb, &
        & 0.13166e-03_rb,0.13207e-03_rb,0.11733e-03_rb,0.23028e-04_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.90882e-07_rb,0.64593e-04_rb,0.98275e-04_rb,0.12041e-03_rb,0.13497e-03_rb, &
        & 0.14273e-03_rb,0.14215e-03_rb,0.12479e-03_rb,0.25362e-04_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.85602e-07_rb,0.69545e-04_rb,0.10592e-03_rb,0.12982e-03_rb,0.14583e-03_rb, &
        & 0.15388e-03_rb,0.15228e-03_rb,0.13297e-03_rb,0.27585e-04_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.80889e-07_rb,0.73731e-04_rb,0.11290e-03_rb,0.13914e-03_rb,0.15637e-03_rb, &
        & 0.16431e-03_rb,0.16189e-03_rb,0.14116e-03_rb,0.29828e-04_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.84886e-07_rb,0.44418e-04_rb,0.67641e-04_rb,0.83490e-04_rb,0.93906e-04_rb, &
        & 0.10034e-03_rb,0.10075e-03_rb,0.90614e-04_rb,0.17350e-04_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.79297e-07_rb,0.48981e-04_rb,0.74656e-04_rb,0.91821e-04_rb,0.10299e-03_rb, &
        & 0.10891e-03_rb,0.10933e-03_rb,0.97221e-04_rb,0.19237e-04_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.74408e-07_rb,0.53553e-04_rb,0.81503e-04_rb,0.99872e-04_rb,0.11196e-03_rb, &
        & 0.11832e-03_rb,0.11782e-03_rb,0.10341e-03_rb,0.21189e-04_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.70085e-07_rb,0.57698e-04_rb,0.87887e-04_rb,0.10781e-03_rb,0.12107e-03_rb, &
        & 0.12775e-03_rb,0.12624e-03_rb,0.11034e-03_rb,0.23038e-04_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.66226e-07_rb,0.61177e-04_rb,0.93694e-04_rb,0.11567e-03_rb,0.12995e-03_rb, &
        & 0.13645e-03_rb,0.13436e-03_rb,0.11726e-03_rb,0.24913e-04_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.69499e-07_rb,0.36716e-04_rb,0.55902e-04_rb,0.69067e-04_rb,0.77635e-04_rb, &
        & 0.82934e-04_rb,0.83302e-04_rb,0.74905e-04_rb,0.14419e-04_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.64923e-07_rb,0.40511e-04_rb,0.61779e-04_rb,0.75984e-04_rb,0.85239e-04_rb, &
        & 0.90090e-04_rb,0.90456e-04_rb,0.80264e-04_rb,0.15996e-04_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.60920e-07_rb,0.44356e-04_rb,0.67533e-04_rb,0.82746e-04_rb,0.92816e-04_rb, &
        & 0.98017e-04_rb,0.97590e-04_rb,0.85621e-04_rb,0.17623e-04_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.57381e-07_rb,0.47826e-04_rb,0.72874e-04_rb,0.89443e-04_rb,0.10040e-03_rb, &
        & 0.10588e-03_rb,0.10456e-03_rb,0.91429e-04_rb,0.19144e-04_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.54221e-07_rb,0.50750e-04_rb,0.77698e-04_rb,0.96005e-04_rb,0.10784e-03_rb, &
        & 0.11318e-03_rb,0.11139e-03_rb,0.97258e-04_rb,0.20671e-04_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.36654e-05_rb,0.13689e-02_rb,0.21108e-02_rb,0.26314e-02_rb,0.29458e-02_rb, &
        & 0.30831e-02_rb,0.30207e-02_rb,0.26003e-02_rb,0.93689e-03_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.35304e-05_rb,0.14025e-02_rb,0.21716e-02_rb,0.27078e-02_rb,0.30551e-02_rb, &
        & 0.31948e-02_rb,0.31649e-02_rb,0.27258e-02_rb,0.97983e-03_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.34059e-05_rb,0.14340e-02_rb,0.22214e-02_rb,0.27773e-02_rb,0.31348e-02_rb, &
        & 0.33049e-02_rb,0.32835e-02_rb,0.28461e-02_rb,0.10251e-02_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.32895e-05_rb,0.14610e-02_rb,0.22596e-02_rb,0.28314e-02_rb,0.32113e-02_rb, &
        & 0.34131e-02_rb,0.33921e-02_rb,0.29508e-02_rb,0.10730e-02_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.31814e-05_rb,0.14814e-02_rb,0.22905e-02_rb,0.28789e-02_rb,0.32785e-02_rb, &
        & 0.34996e-02_rb,0.34952e-02_rb,0.30528e-02_rb,0.11205e-02_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.30293e-05_rb,0.11600e-02_rb,0.17946e-02_rb,0.22370e-02_rb,0.25205e-02_rb, &
        & 0.26432e-02_rb,0.26172e-02_rb,0.22691e-02_rb,0.78466e-03_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.29173e-05_rb,0.11916e-02_rb,0.18468e-02_rb,0.23072e-02_rb,0.26094e-02_rb, &
        & 0.27477e-02_rb,0.27388e-02_rb,0.23809e-02_rb,0.82356e-03_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.28152e-05_rb,0.12203e-02_rb,0.18939e-02_rb,0.23691e-02_rb,0.26814e-02_rb, &
        & 0.28506e-02_rb,0.28414e-02_rb,0.24825e-02_rb,0.86161e-03_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.27207e-05_rb,0.12454e-02_rb,0.19291e-02_rb,0.24180e-02_rb,0.27489e-02_rb, &
        & 0.29380e-02_rb,0.29345e-02_rb,0.25808e-02_rb,0.90328e-03_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.26316e-05_rb,0.12644e-02_rb,0.19586e-02_rb,0.24624e-02_rb,0.28146e-02_rb, &
        & 0.30125e-02_rb,0.30237e-02_rb,0.26695e-02_rb,0.94678e-03_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.25288e-05_rb,0.97348e-03_rb,0.15070e-02_rb,0.18795e-02_rb,0.21206e-02_rb, &
        & 0.22362e-02_rb,0.22210e-02_rb,0.19369e-02_rb,0.64308e-03_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.24353e-05_rb,0.10031e-02_rb,0.15552e-02_rb,0.19441e-02_rb,0.22013e-02_rb, &
        & 0.23257e-02_rb,0.23286e-02_rb,0.20362e-02_rb,0.67529e-03_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.23495e-05_rb,0.10295e-02_rb,0.15986e-02_rb,0.20019e-02_rb,0.22663e-02_rb, &
        & 0.24160e-02_rb,0.24204e-02_rb,0.21276e-02_rb,0.70991e-03_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.22699e-05_rb,0.10533e-02_rb,0.16337e-02_rb,0.20489e-02_rb,0.23288e-02_rb, &
        & 0.24911e-02_rb,0.25016e-02_rb,0.22148e-02_rb,0.74635e-03_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.21951e-05_rb,0.10724e-02_rb,0.16625e-02_rb,0.20891e-02_rb,0.23896e-02_rb, &
        & 0.25582e-02_rb,0.25812e-02_rb,0.22925e-02_rb,0.78432e-03_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.21169e-05_rb,0.81053e-03_rb,0.12568e-02_rb,0.15647e-02_rb,0.17654e-02_rb, &
        & 0.18662e-02_rb,0.18568e-02_rb,0.16269e-02_rb,0.52608e-03_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.20407e-05_rb,0.83875e-03_rb,0.13019e-02_rb,0.16252e-02_rb,0.18383e-02_rb, &
        & 0.19441e-02_rb,0.19538e-02_rb,0.17139e-02_rb,0.55325e-03_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.19674e-05_rb,0.86329e-03_rb,0.13417e-02_rb,0.16788e-02_rb,0.18985e-02_rb, &
        & 0.20246e-02_rb,0.20315e-02_rb,0.17978e-02_rb,0.58253e-03_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.18994e-05_rb,0.88551e-03_rb,0.13753e-02_rb,0.17238e-02_rb,0.19566e-02_rb, &
        & 0.20923e-02_rb,0.21035e-02_rb,0.18724e-02_rb,0.61438e-03_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.18358e-05_rb,0.90442e-03_rb,0.14042e-02_rb,0.17619e-02_rb,0.20119e-02_rb, &
        & 0.21542e-02_rb,0.21749e-02_rb,0.19425e-02_rb,0.64762e-03_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.17725e-05_rb,0.67146e-03_rb,0.10396e-02_rb,0.12935e-02_rb,0.14576e-02_rb, &
        & 0.15437e-02_rb,0.15341e-02_rb,0.13532e-02_rb,0.42981e-03_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.17095e-05_rb,0.69774e-03_rb,0.10823e-02_rb,0.13477e-02_rb,0.15227e-02_rb, &
        & 0.16117e-02_rb,0.16222e-02_rb,0.14302e-02_rb,0.45286e-03_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.16478e-05_rb,0.72044e-03_rb,0.11190e-02_rb,0.13967e-02_rb,0.15783e-02_rb, &
        & 0.16817e-02_rb,0.16897e-02_rb,0.15040e-02_rb,0.47782e-03_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.15896e-05_rb,0.74136e-03_rb,0.11512e-02_rb,0.14394e-02_rb,0.16307e-02_rb, &
        & 0.17435e-02_rb,0.17536e-02_rb,0.15687e-02_rb,0.50468e-03_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.15353e-05_rb,0.75977e-03_rb,0.11790e-02_rb,0.14755e-02_rb,0.16811e-02_rb, &
        & 0.17990e-02_rb,0.18175e-02_rb,0.16279e-02_rb,0.53353e-03_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.14856e-05_rb,0.55163e-03_rb,0.85182e-03_rb,0.10583e-02_rb,0.11933e-02_rb, &
        & 0.12633e-02_rb,0.12519e-02_rb,0.11126e-02_rb,0.34969e-03_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.14323e-05_rb,0.57534e-03_rb,0.89211e-03_rb,0.11084e-02_rb,0.12510e-02_rb, &
        & 0.13248e-02_rb,0.13302e-02_rb,0.11815e-02_rb,0.36893e-03_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.13818e-05_rb,0.59705e-03_rb,0.92552e-03_rb,0.11530e-02_rb,0.13018e-02_rb, &
        & 0.13860e-02_rb,0.13942e-02_rb,0.12437e-02_rb,0.38991e-03_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.13317e-05_rb,0.61671e-03_rb,0.95568e-03_rb,0.11922e-02_rb,0.13481e-02_rb, &
        & 0.14426e-02_rb,0.14508e-02_rb,0.13001e-02_rb,0.41248e-03_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.12851e-05_rb,0.63414e-03_rb,0.98227e-03_rb,0.12265e-02_rb,0.13936e-02_rb, &
        & 0.14913e-02_rb,0.15064e-02_rb,0.13512e-02_rb,0.43696e-03_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.12453e-05_rb,0.44982e-03_rb,0.69286e-03_rb,0.85857e-03_rb,0.97115e-03_rb, &
        & 0.10268e-02_rb,0.10132e-02_rb,0.90661e-03_rb,0.28333e-03_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.11997e-05_rb,0.47163e-03_rb,0.72984e-03_rb,0.90525e-03_rb,0.10209e-02_rb, &
        & 0.10830e-02_rb,0.10815e-02_rb,0.96820e-03_rb,0.29991e-03_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.11571e-05_rb,0.49170e-03_rb,0.76064e-03_rb,0.94597e-03_rb,0.10683e-02_rb, &
        & 0.11337e-02_rb,0.11418e-02_rb,0.10210e-02_rb,0.31738e-03_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.11157e-05_rb,0.50964e-03_rb,0.78859e-03_rb,0.98236e-03_rb,0.11088e-02_rb, &
        & 0.11859e-02_rb,0.11920e-02_rb,0.10690e-02_rb,0.33600e-03_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.10757e-05_rb,0.52578e-03_rb,0.81330e-03_rb,0.10141e-02_rb,0.11493e-02_rb, &
        & 0.12285e-02_rb,0.12399e-02_rb,0.11151e-02_rb,0.35631e-03_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.10436e-05_rb,0.36475e-03_rb,0.55948e-03_rb,0.69399e-03_rb,0.78699e-03_rb, &
        & 0.82733e-03_rb,0.81546e-03_rb,0.73443e-03_rb,0.22874e-03_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.10043e-05_rb,0.38461e-03_rb,0.59365e-03_rb,0.73631e-03_rb,0.82902e-03_rb, &
        & 0.87996e-03_rb,0.87589e-03_rb,0.78745e-03_rb,0.24281e-03_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.96785e-06_rb,0.40227e-03_rb,0.62191e-03_rb,0.77183e-03_rb,0.87067e-03_rb, &
        & 0.92338e-03_rb,0.93116e-03_rb,0.83334e-03_rb,0.25735e-03_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.93367e-06_rb,0.41881e-03_rb,0.64697e-03_rb,0.80474e-03_rb,0.90754e-03_rb, &
        & 0.96806e-03_rb,0.97434e-03_rb,0.87439e-03_rb,0.27286e-03_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.90017e-06_rb,0.43377e-03_rb,0.67026e-03_rb,0.83398e-03_rb,0.94269e-03_rb, &
        & 0.10080e-02_rb,0.10154e-02_rb,0.91479e-03_rb,0.28948e-03_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.87469e-06_rb,0.29345e-03_rb,0.44857e-03_rb,0.55653e-03_rb,0.63371e-03_rb, &
        & 0.66253e-03_rb,0.65229e-03_rb,0.59009e-03_rb,0.18475e-03_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.84062e-06_rb,0.31152e-03_rb,0.47975e-03_rb,0.59422e-03_rb,0.67077e-03_rb, &
        & 0.71168e-03_rb,0.70550e-03_rb,0.63674e-03_rb,0.19669e-03_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.80947e-06_rb,0.32779e-03_rb,0.50582e-03_rb,0.62707e-03_rb,0.70705e-03_rb, &
        & 0.74936e-03_rb,0.75252e-03_rb,0.67682e-03_rb,0.20896e-03_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.78052e-06_rb,0.34262e-03_rb,0.52820e-03_rb,0.65631e-03_rb,0.74002e-03_rb, &
        & 0.78731e-03_rb,0.79257e-03_rb,0.71155e-03_rb,0.22197e-03_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.75300e-06_rb,0.35618e-03_rb,0.54940e-03_rb,0.68236e-03_rb,0.76978e-03_rb, &
        & 0.82271e-03_rb,0.82806e-03_rb,0.74658e-03_rb,0.23588e-03_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.73080e-06_rb,0.23619e-03_rb,0.36002e-03_rb,0.44769e-03_rb,0.51001e-03_rb, &
        & 0.53294e-03_rb,0.52319e-03_rb,0.47492e-03_rb,0.15005e-03_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.70142e-06_rb,0.25225e-03_rb,0.38742e-03_rb,0.48039e-03_rb,0.54285e-03_rb, &
        & 0.57407e-03_rb,0.56765e-03_rb,0.51444e-03_rb,0.16000e-03_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.67484e-06_rb,0.26687e-03_rb,0.41119e-03_rb,0.50924e-03_rb,0.57372e-03_rb, &
        & 0.60912e-03_rb,0.60813e-03_rb,0.54945e-03_rb,0.17036e-03_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.65032e-06_rb,0.28002e-03_rb,0.43104e-03_rb,0.53513e-03_rb,0.60294e-03_rb, &
        & 0.63998e-03_rb,0.64485e-03_rb,0.57888e-03_rb,0.18120e-03_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.62752e-06_rb,0.29182e-03_rb,0.44950e-03_rb,0.55836e-03_rb,0.62849e-03_rb, &
        & 0.67148e-03_rb,0.67570e-03_rb,0.60918e-03_rb,0.19271e-03_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.59846e-06_rb,0.19603e-03_rb,0.29902e-03_rb,0.37159e-03_rb,0.42367e-03_rb, &
        & 0.44241e-03_rb,0.43550e-03_rb,0.39618e-03_rb,0.12498e-03_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.57447e-06_rb,0.20953e-03_rb,0.32187e-03_rb,0.39882e-03_rb,0.45021e-03_rb, &
        & 0.47717e-03_rb,0.47240e-03_rb,0.42894e-03_rb,0.13362e-03_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.55265e-06_rb,0.22185e-03_rb,0.34133e-03_rb,0.42283e-03_rb,0.47616e-03_rb, &
        & 0.50481e-03_rb,0.50553e-03_rb,0.45654e-03_rb,0.14246e-03_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.53259e-06_rb,0.23301e-03_rb,0.35816e-03_rb,0.44448e-03_rb,0.50048e-03_rb, &
        & 0.53147e-03_rb,0.53574e-03_rb,0.48169e-03_rb,0.15168e-03_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.51379e-06_rb,0.24302e-03_rb,0.37383e-03_rb,0.46352e-03_rb,0.52203e-03_rb, &
        & 0.55783e-03_rb,0.56175e-03_rb,0.50737e-03_rb,0.16153e-03_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.49006e-06_rb,0.16259e-03_rb,0.24802e-03_rb,0.30804e-03_rb,0.35111e-03_rb, &
        & 0.36672e-03_rb,0.36166e-03_rb,0.32968e-03_rb,0.10401e-03_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.47033e-06_rb,0.17379e-03_rb,0.26697e-03_rb,0.33069e-03_rb,0.37292e-03_rb, &
        & 0.39587e-03_rb,0.39239e-03_rb,0.35672e-03_rb,0.11138e-03_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.45247e-06_rb,0.18417e-03_rb,0.28295e-03_rb,0.35045e-03_rb,0.39465e-03_rb, &
        & 0.41825e-03_rb,0.41984e-03_rb,0.37867e-03_rb,0.11882e-03_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.43605e-06_rb,0.19351e-03_rb,0.29716e-03_rb,0.36850e-03_rb,0.41489e-03_rb, &
        & 0.44074e-03_rb,0.44460e-03_rb,0.40031e-03_rb,0.12667e-03_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.42066e-06_rb,0.20206e-03_rb,0.31038e-03_rb,0.38464e-03_rb,0.43337e-03_rb, &
        & 0.46297e-03_rb,0.46667e-03_rb,0.42202e-03_rb,0.13498e-03_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.40122e-06_rb,0.13465e-03_rb,0.20541e-03_rb,0.25488e-03_rb,0.29006e-03_rb, &
        & 0.30358e-03_rb,0.29984e-03_rb,0.27375e-03_rb,0.86406e-04_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.38502e-06_rb,0.14398e-03_rb,0.22086e-03_rb,0.27376e-03_rb,0.30832e-03_rb, &
        & 0.32794e-03_rb,0.32517e-03_rb,0.29562e-03_rb,0.92614e-04_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.37045e-06_rb,0.15266e-03_rb,0.23416e-03_rb,0.29013e-03_rb,0.32675e-03_rb, &
        & 0.34622e-03_rb,0.34816e-03_rb,0.31401e-03_rb,0.98880e-04_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.35702e-06_rb,0.16050e-03_rb,0.24608e-03_rb,0.30521e-03_rb,0.34336e-03_rb, &
        & 0.36522e-03_rb,0.36857e-03_rb,0.33246e-03_rb,0.10559e-03_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.34440e-06_rb,0.16771e-03_rb,0.25742e-03_rb,0.31904e-03_rb,0.35943e-03_rb, &
        & 0.38405e-03_rb,0.38715e-03_rb,0.35087e-03_rb,0.11262e-03_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.15583e-03_rb,0.37546e-02_rb,0.53536e-02_rb,0.64161e-02_rb,0.71223e-02_rb, &
        & 0.74982e-02_rb,0.73596e-02_rb,0.65135e-02_rb,0.35175e-02_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.16148e-03_rb,0.38569e-02_rb,0.55256e-02_rb,0.66461e-02_rb,0.73757e-02_rb, &
        & 0.77530e-02_rb,0.75665e-02_rb,0.66496e-02_rb,0.36840e-02_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.16484e-03_rb,0.39353e-02_rb,0.56745e-02_rb,0.68364e-02_rb,0.76166e-02_rb, &
        & 0.79711e-02_rb,0.77717e-02_rb,0.67846e-02_rb,0.38485e-02_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.16621e-03_rb,0.39946e-02_rb,0.57947e-02_rb,0.70071e-02_rb,0.78193e-02_rb, &
        & 0.81581e-02_rb,0.79592e-02_rb,0.69132e-02_rb,0.39769e-02_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.16518e-03_rb,0.40386e-02_rb,0.58925e-02_rb,0.71530e-02_rb,0.79946e-02_rb, &
        & 0.83258e-02_rb,0.81107e-02_rb,0.70226e-02_rb,0.41134e-02_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.12482e-03_rb,0.32204e-02_rb,0.46102e-02_rb,0.55475e-02_rb,0.61730e-02_rb, &
        & 0.65000e-02_rb,0.63942e-02_rb,0.56394e-02_rb,0.30002e-02_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.13027e-03_rb,0.33099e-02_rb,0.47617e-02_rb,0.57497e-02_rb,0.64031e-02_rb, &
        & 0.67184e-02_rb,0.65829e-02_rb,0.57721e-02_rb,0.31442e-02_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.13370e-03_rb,0.33808e-02_rb,0.48845e-02_rb,0.59235e-02_rb,0.66074e-02_rb, &
        & 0.69048e-02_rb,0.67660e-02_rb,0.59018e-02_rb,0.32788e-02_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.13549e-03_rb,0.34340e-02_rb,0.49885e-02_rb,0.60745e-02_rb,0.67877e-02_rb, &
        & 0.70780e-02_rb,0.69202e-02_rb,0.60131e-02_rb,0.33946e-02_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.13543e-03_rb,0.34778e-02_rb,0.50745e-02_rb,0.61981e-02_rb,0.69282e-02_rb, &
        & 0.72229e-02_rb,0.70521e-02_rb,0.61218e-02_rb,0.35097e-02_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.98437e-04_rb,0.27209e-02_rb,0.39035e-02_rb,0.47185e-02_rb,0.52706e-02_rb, &
        & 0.55579e-02_rb,0.54846e-02_rb,0.48469e-02_rb,0.25077e-02_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.10396e-03_rb,0.28031e-02_rb,0.40384e-02_rb,0.48993e-02_rb,0.54753e-02_rb, &
        & 0.57575e-02_rb,0.56525e-02_rb,0.49684e-02_rb,0.26366e-02_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.10770e-03_rb,0.28702e-02_rb,0.41513e-02_rb,0.50504e-02_rb,0.56640e-02_rb, &
        & 0.59245e-02_rb,0.58125e-02_rb,0.50922e-02_rb,0.27478e-02_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.10996e-03_rb,0.29232e-02_rb,0.42451e-02_rb,0.51838e-02_rb,0.58257e-02_rb, &
        & 0.60793e-02_rb,0.59498e-02_rb,0.51949e-02_rb,0.28490e-02_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.11088e-03_rb,0.29645e-02_rb,0.43251e-02_rb,0.52992e-02_rb,0.59468e-02_rb, &
        & 0.62096e-02_rb,0.60648e-02_rb,0.52904e-02_rb,0.29495e-02_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.76587e-04_rb,0.22792e-02_rb,0.32731e-02_rb,0.39724e-02_rb,0.44561e-02_rb, &
        & 0.47109e-02_rb,0.46706e-02_rb,0.41444e-02_rb,0.20810e-02_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.82225e-04_rb,0.23548e-02_rb,0.33943e-02_rb,0.41325e-02_rb,0.46415e-02_rb, &
        & 0.48939e-02_rb,0.48200e-02_rb,0.42591e-02_rb,0.21925e-02_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.86203e-04_rb,0.24180e-02_rb,0.34991e-02_rb,0.42684e-02_rb,0.48074e-02_rb, &
        & 0.50416e-02_rb,0.49651e-02_rb,0.43643e-02_rb,0.22938e-02_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.88766e-04_rb,0.24694e-02_rb,0.35874e-02_rb,0.43875e-02_rb,0.49469e-02_rb, &
        & 0.51810e-02_rb,0.50850e-02_rb,0.44606e-02_rb,0.23791e-02_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.90218e-04_rb,0.25110e-02_rb,0.36613e-02_rb,0.44937e-02_rb,0.50548e-02_rb, &
        & 0.52982e-02_rb,0.51883e-02_rb,0.45498e-02_rb,0.24658e-02_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.59080e-04_rb,0.18972e-02_rb,0.27294e-02_rb,0.33195e-02_rb,0.37395e-02_rb, &
        & 0.39683e-02_rb,0.39523e-02_rb,0.35161e-02_rb,0.17195e-02_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.64508e-04_rb,0.19673e-02_rb,0.28376e-02_rb,0.34632e-02_rb,0.39033e-02_rb, &
        & 0.41343e-02_rb,0.40826e-02_rb,0.36195e-02_rb,0.18173e-02_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.68538e-04_rb,0.20264e-02_rb,0.29342e-02_rb,0.35856e-02_rb,0.40488e-02_rb, &
        & 0.42664e-02_rb,0.42141e-02_rb,0.37144e-02_rb,0.19081e-02_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.71328e-04_rb,0.20750e-02_rb,0.30165e-02_rb,0.36933e-02_rb,0.41719e-02_rb, &
        & 0.43880e-02_rb,0.43221e-02_rb,0.38006e-02_rb,0.19818e-02_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.73073e-04_rb,0.21155e-02_rb,0.30861e-02_rb,0.37892e-02_rb,0.42690e-02_rb, &
        & 0.44901e-02_rb,0.44161e-02_rb,0.38864e-02_rb,0.20583e-02_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.45069e-04_rb,0.15673e-02_rb,0.22596e-02_rb,0.27566e-02_rb,0.31117e-02_rb, &
        & 0.33162e-02_rb,0.33135e-02_rb,0.29574e-02_rb,0.14089e-02_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.49959e-04_rb,0.16342e-02_rb,0.23577e-02_rb,0.28828e-02_rb,0.32572e-02_rb, &
        & 0.34626e-02_rb,0.34302e-02_rb,0.30476e-02_rb,0.14955e-02_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.54013e-04_rb,0.16891e-02_rb,0.24457e-02_rb,0.29926e-02_rb,0.33865e-02_rb, &
        & 0.35809e-02_rb,0.35427e-02_rb,0.31358e-02_rb,0.15763e-02_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.56910e-04_rb,0.17356e-02_rb,0.25220e-02_rb,0.30903e-02_rb,0.34968e-02_rb, &
        & 0.36853e-02_rb,0.36443e-02_rb,0.32140e-02_rb,0.16428e-02_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.58852e-04_rb,0.17743e-02_rb,0.25868e-02_rb,0.31771e-02_rb,0.35857e-02_rb, &
        & 0.37782e-02_rb,0.37284e-02_rb,0.32905e-02_rb,0.17079e-02_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.34131e-04_rb,0.12873e-02_rb,0.18619e-02_rb,0.22745e-02_rb,0.25688e-02_rb, &
        & 0.27494e-02_rb,0.27574e-02_rb,0.24658e-02_rb,0.11444e-02_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.38379e-04_rb,0.13503e-02_rb,0.19490e-02_rb,0.23869e-02_rb,0.26985e-02_rb, &
        & 0.28761e-02_rb,0.28607e-02_rb,0.25443e-02_rb,0.12242e-02_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.42146e-04_rb,0.14018e-02_rb,0.20287e-02_rb,0.24843e-02_rb,0.28119e-02_rb, &
        & 0.29848e-02_rb,0.29569e-02_rb,0.26258e-02_rb,0.12956e-02_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.45116e-04_rb,0.14454e-02_rb,0.20989e-02_rb,0.25718e-02_rb,0.29115e-02_rb, &
        & 0.30740e-02_rb,0.30482e-02_rb,0.26967e-02_rb,0.13537e-02_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.47178e-04_rb,0.14818e-02_rb,0.21591e-02_rb,0.26509e-02_rb,0.29945e-02_rb, &
        & 0.31590e-02_rb,0.31235e-02_rb,0.27641e-02_rb,0.14100e-02_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.25656e-04_rb,0.10525e-02_rb,0.15266e-02_rb,0.18648e-02_rb,0.21060e-02_rb, &
        & 0.22660e-02_rb,0.22807e-02_rb,0.20415e-02_rb,0.92482e-03_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.29313e-04_rb,0.11096e-02_rb,0.16027e-02_rb,0.19628e-02_rb,0.22224e-02_rb, &
        & 0.23730e-02_rb,0.23683e-02_rb,0.21111e-02_rb,0.99366e-03_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.32643e-04_rb,0.11579e-02_rb,0.16732e-02_rb,0.20503e-02_rb,0.23231e-02_rb, &
        & 0.24721e-02_rb,0.24512e-02_rb,0.21842e-02_rb,0.10570e-02_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.35509e-04_rb,0.11985e-02_rb,0.17379e-02_rb,0.21284e-02_rb,0.24110e-02_rb, &
        & 0.25519e-02_rb,0.25339e-02_rb,0.22496e-02_rb,0.11116e-02_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.37616e-04_rb,0.12327e-02_rb,0.17933e-02_rb,0.21996e-02_rb,0.24880e-02_rb, &
        & 0.26258e-02_rb,0.26029e-02_rb,0.23088e-02_rb,0.11595e-02_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.19166e-04_rb,0.85559e-03_rb,0.12446e-02_rb,0.15213e-02_rb,0.17160e-02_rb, &
        & 0.18527e-02_rb,0.18723e-02_rb,0.16794e-02_rb,0.74316e-03_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.22251e-04_rb,0.90701e-03_rb,0.13107e-02_rb,0.16061e-02_rb,0.18177e-02_rb, &
        & 0.19452e-02_rb,0.19496e-02_rb,0.17420e-02_rb,0.80168e-03_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.25123e-04_rb,0.95106e-03_rb,0.13724e-02_rb,0.16826e-02_rb,0.19064e-02_rb, &
        & 0.20352e-02_rb,0.20241e-02_rb,0.18064e-02_rb,0.85659e-03_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.27713e-04_rb,0.98903e-03_rb,0.14301e-02_rb,0.17523e-02_rb,0.19862e-02_rb, &
        & 0.21079e-02_rb,0.20944e-02_rb,0.18656e-02_rb,0.90655e-03_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.29821e-04_rb,0.10207e-02_rb,0.14813e-02_rb,0.18163e-02_rb,0.20570e-02_rb, &
        & 0.21737e-02_rb,0.21574e-02_rb,0.19153e-02_rb,0.94882e-03_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.14455e-04_rb,0.69508e-03_rb,0.10143e-02_rb,0.12382e-02_rb,0.13977e-02_rb, &
        & 0.15101e-02_rb,0.15339e-02_rb,0.13794e-02_rb,0.60064e-03_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.16975e-04_rb,0.74055e-03_rb,0.10715e-02_rb,0.13117e-02_rb,0.14851e-02_rb, &
        & 0.15938e-02_rb,0.16026e-02_rb,0.14354e-02_rb,0.65034e-03_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.19421e-04_rb,0.77996e-03_rb,0.11243e-02_rb,0.13791e-02_rb,0.15636e-02_rb, &
        & 0.16707e-02_rb,0.16678e-02_rb,0.14908e-02_rb,0.69875e-03_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.21659e-04_rb,0.81369e-03_rb,0.11746e-02_rb,0.14408e-02_rb,0.16340e-02_rb, &
        & 0.17376e-02_rb,0.17267e-02_rb,0.15418e-02_rb,0.74336e-03_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.23616e-04_rb,0.84316e-03_rb,0.12215e-02_rb,0.14972e-02_rb,0.16971e-02_rb, &
        & 0.17945e-02_rb,0.17834e-02_rb,0.15844e-02_rb,0.78045e-03_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.11812e-04_rb,0.58067e-03_rb,0.84658e-03_rb,0.10349e-02_rb,0.11681e-02_rb, &
        & 0.12622e-02_rb,0.12806e-02_rb,0.11518e-02_rb,0.50197e-03_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.13870e-04_rb,0.61870e-03_rb,0.89364e-03_rb,0.10957e-02_rb,0.12411e-02_rb, &
        & 0.13307e-02_rb,0.13366e-02_rb,0.11990e-02_rb,0.54419e-03_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.15872e-04_rb,0.65131e-03_rb,0.93843e-03_rb,0.11520e-02_rb,0.13068e-02_rb, &
        & 0.13966e-02_rb,0.13927e-02_rb,0.12459e-02_rb,0.58412e-03_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.17706e-04_rb,0.67957e-03_rb,0.98128e-03_rb,0.12037e-02_rb,0.13653e-02_rb, &
        & 0.14510e-02_rb,0.14423e-02_rb,0.12873e-02_rb,0.62086e-03_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.19307e-04_rb,0.70477e-03_rb,0.10208e-02_rb,0.12522e-02_rb,0.14187e-02_rb, &
        & 0.15000e-02_rb,0.14882e-02_rb,0.13223e-02_rb,0.65273e-03_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.96633e-05_rb,0.48384e-03_rb,0.70464e-03_rb,0.86214e-03_rb,0.97325e-03_rb, &
        & 0.10525e-02_rb,0.10661e-02_rb,0.95818e-03_rb,0.41838e-03_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.11346e-04_rb,0.51544e-03_rb,0.74375e-03_rb,0.91255e-03_rb,0.10346e-02_rb, &
        & 0.11088e-02_rb,0.11124e-02_rb,0.99787e-03_rb,0.45436e-03_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.12983e-04_rb,0.54258e-03_rb,0.78191e-03_rb,0.95994e-03_rb,0.10893e-02_rb, &
        & 0.11636e-02_rb,0.11595e-02_rb,0.10379e-02_rb,0.48767e-03_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.14483e-04_rb,0.56662e-03_rb,0.81795e-03_rb,0.10036e-02_rb,0.11385e-02_rb, &
        & 0.12091e-02_rb,0.12012e-02_rb,0.10707e-02_rb,0.51810e-03_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.15790e-04_rb,0.58805e-03_rb,0.85165e-03_rb,0.10447e-02_rb,0.11830e-02_rb, &
        & 0.12500e-02_rb,0.12389e-02_rb,0.11012e-02_rb,0.54508e-03_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.79057e-05_rb,0.40215e-03_rb,0.58516e-03_rb,0.71666e-03_rb,0.80969e-03_rb, &
        & 0.87518e-03_rb,0.88581e-03_rb,0.79569e-03_rb,0.34825e-03_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.92808e-05_rb,0.42853e-03_rb,0.61818e-03_rb,0.75847e-03_rb,0.86082e-03_rb, &
        & 0.92160e-03_rb,0.92436e-03_rb,0.82894e-03_rb,0.37871e-03_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.10621e-04_rb,0.45134e-03_rb,0.65028e-03_rb,0.79815e-03_rb,0.90622e-03_rb, &
        & 0.96752e-03_rb,0.96291e-03_rb,0.86177e-03_rb,0.40642e-03_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.11849e-04_rb,0.47161e-03_rb,0.68090e-03_rb,0.83530e-03_rb,0.94770e-03_rb, &
        & 0.10054e-02_rb,0.99815e-03_rb,0.88908e-03_rb,0.43149e-03_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.12897e-04_rb,0.49000e-03_rb,0.70944e-03_rb,0.86982e-03_rb,0.98430e-03_rb, &
        & 0.10398e-02_rb,0.10297e-02_rb,0.91483e-03_rb,0.45459e-03_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.23575e-02_rb,0.96457e-02_rb,0.13128e-01_rb,0.15361e-01_rb,0.16662e-01_rb, &
        & 0.16999e-01_rb,0.16303e-01_rb,0.14368e-01_rb,0.10185e-01_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.24508e-02_rb,0.99768e-02_rb,0.13562e-01_rb,0.15786e-01_rb,0.17046e-01_rb, &
        & 0.17316e-01_rb,0.16539e-01_rb,0.14627e-01_rb,0.10447e-01_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.25176e-02_rb,0.10262e-01_rb,0.13947e-01_rb,0.16182e-01_rb,0.17390e-01_rb, &
        & 0.17588e-01_rb,0.16752e-01_rb,0.14857e-01_rb,0.10699e-01_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.25627e-02_rb,0.10510e-01_rb,0.14271e-01_rb,0.16517e-01_rb,0.17678e-01_rb, &
        & 0.17815e-01_rb,0.16941e-01_rb,0.15073e-01_rb,0.10970e-01_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.25904e-02_rb,0.10712e-01_rb,0.14528e-01_rb,0.16788e-01_rb,0.17912e-01_rb, &
        & 0.18013e-01_rb,0.17126e-01_rb,0.15282e-01_rb,0.11215e-01_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.19422e-02_rb,0.85179e-02_rb,0.11603e-01_rb,0.13561e-01_rb,0.14694e-01_rb, &
        & 0.14952e-01_rb,0.14274e-01_rb,0.12616e-01_rb,0.86910e-02_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.20231e-02_rb,0.88208e-02_rb,0.11992e-01_rb,0.13942e-01_rb,0.15043e-01_rb, &
        & 0.15241e-01_rb,0.14503e-01_rb,0.12849e-01_rb,0.89203e-02_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.20824e-02_rb,0.90892e-02_rb,0.12333e-01_rb,0.14283e-01_rb,0.15343e-01_rb, &
        & 0.15493e-01_rb,0.14704e-01_rb,0.13065e-01_rb,0.91412e-02_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.21235e-02_rb,0.93193e-02_rb,0.12613e-01_rb,0.14573e-01_rb,0.15586e-01_rb, &
        & 0.15696e-01_rb,0.14900e-01_rb,0.13269e-01_rb,0.93731e-02_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.21501e-02_rb,0.95064e-02_rb,0.12844e-01_rb,0.14802e-01_rb,0.15789e-01_rb, &
        & 0.15872e-01_rb,0.15079e-01_rb,0.13459e-01_rb,0.95922e-02_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.15710e-02_rb,0.73739e-02_rb,0.10084e-01_rb,0.11795e-01_rb,0.12791e-01_rb, &
        & 0.13002e-01_rb,0.12408e-01_rb,0.10948e-01_rb,0.73256e-02_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.16445e-02_rb,0.76603e-02_rb,0.10441e-01_rb,0.12136e-01_rb,0.13098e-01_rb, &
        & 0.13272e-01_rb,0.12629e-01_rb,0.11169e-01_rb,0.75277e-02_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.17016e-02_rb,0.79120e-02_rb,0.10738e-01_rb,0.12444e-01_rb,0.13365e-01_rb, &
        & 0.13502e-01_rb,0.12827e-01_rb,0.11365e-01_rb,0.77275e-02_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.17419e-02_rb,0.81204e-02_rb,0.10992e-01_rb,0.12704e-01_rb,0.13579e-01_rb, &
        & 0.13687e-01_rb,0.13022e-01_rb,0.11565e-01_rb,0.79327e-02_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.17693e-02_rb,0.82896e-02_rb,0.11201e-01_rb,0.12903e-01_rb,0.13761e-01_rb, &
        & 0.13854e-01_rb,0.13193e-01_rb,0.11760e-01_rb,0.81324e-02_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.12568e-02_rb,0.62925e-02_rb,0.86570e-02_rb,0.10156e-01_rb,0.11020e-01_rb, &
        & 0.11220e-01_rb,0.10710e-01_rb,0.94329e-02_rb,0.61812e-02_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.13242e-02_rb,0.65532e-02_rb,0.89756e-02_rb,0.10460e-01_rb,0.11292e-01_rb, &
        & 0.11463e-01_rb,0.10924e-01_rb,0.96449e-02_rb,0.63620e-02_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.13782e-02_rb,0.67807e-02_rb,0.92447e-02_rb,0.10739e-01_rb,0.11531e-01_rb, &
        & 0.11676e-01_rb,0.11115e-01_rb,0.98384e-02_rb,0.65417e-02_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.14185e-02_rb,0.69728e-02_rb,0.94724e-02_rb,0.10967e-01_rb,0.11732e-01_rb, &
        & 0.11853e-01_rb,0.11299e-01_rb,0.10029e-01_rb,0.67314e-02_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.14466e-02_rb,0.71294e-02_rb,0.96551e-02_rb,0.11150e-01_rb,0.11897e-01_rb, &
        & 0.12003e-01_rb,0.11457e-01_rb,0.10212e-01_rb,0.69134e-02_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.99815e-03_rb,0.53184e-02_rb,0.73668e-02_rb,0.86661e-02_rb,0.94189e-02_rb, &
        & 0.96088e-02_rb,0.91860e-02_rb,0.80854e-02_rb,0.52147e-02_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.10594e-02_rb,0.55502e-02_rb,0.76480e-02_rb,0.89382e-02_rb,0.96675e-02_rb, &
        & 0.98281e-02_rb,0.93908e-02_rb,0.82904e-02_rb,0.53805e-02_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.11093e-02_rb,0.57534e-02_rb,0.78820e-02_rb,0.91873e-02_rb,0.98860e-02_rb, &
        & 0.10027e-01_rb,0.95739e-02_rb,0.84774e-02_rb,0.55398e-02_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.11487e-02_rb,0.59264e-02_rb,0.80746e-02_rb,0.93952e-02_rb,0.10070e-01_rb, &
        & 0.10196e-01_rb,0.97460e-02_rb,0.86586e-02_rb,0.57118e-02_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.11770e-02_rb,0.60715e-02_rb,0.82322e-02_rb,0.95590e-02_rb,0.10225e-01_rb, &
        & 0.10337e-01_rb,0.98891e-02_rb,0.88282e-02_rb,0.58756e-02_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.78484e-03_rb,0.44557e-02_rb,0.62088e-02_rb,0.73238e-02_rb,0.79758e-02_rb, &
        & 0.81565e-02_rb,0.78234e-02_rb,0.68949e-02_rb,0.43721e-02_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.84104e-03_rb,0.46584e-02_rb,0.64548e-02_rb,0.75720e-02_rb,0.82049e-02_rb, &
        & 0.83603e-02_rb,0.80139e-02_rb,0.70912e-02_rb,0.45208e-02_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.88703e-03_rb,0.48386e-02_rb,0.66585e-02_rb,0.77939e-02_rb,0.84077e-02_rb, &
        & 0.85494e-02_rb,0.81878e-02_rb,0.72673e-02_rb,0.46664e-02_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.92429e-03_rb,0.49939e-02_rb,0.68255e-02_rb,0.79793e-02_rb,0.85811e-02_rb, &
        & 0.87123e-02_rb,0.83423e-02_rb,0.74324e-02_rb,0.48193e-02_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.95287e-03_rb,0.51269e-02_rb,0.69664e-02_rb,0.81216e-02_rb,0.87256e-02_rb, &
        & 0.88404e-02_rb,0.84793e-02_rb,0.75885e-02_rb,0.49684e-02_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.61176e-03_rb,0.37105e-02_rb,0.51921e-02_rb,0.61495e-02_rb,0.67086e-02_rb, &
        & 0.68737e-02_rb,0.66196e-02_rb,0.58499e-02_rb,0.36398e-02_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.66355e-03_rb,0.38845e-02_rb,0.54092e-02_rb,0.63684e-02_rb,0.69191e-02_rb, &
        & 0.70638e-02_rb,0.67967e-02_rb,0.60329e-02_rb,0.37725e-02_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.70584e-03_rb,0.40429e-02_rb,0.55891e-02_rb,0.65665e-02_rb,0.71061e-02_rb, &
        & 0.72392e-02_rb,0.69566e-02_rb,0.61958e-02_rb,0.39040e-02_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.74016e-03_rb,0.41804e-02_rb,0.57362e-02_rb,0.67306e-02_rb,0.72656e-02_rb, &
        & 0.73939e-02_rb,0.71002e-02_rb,0.63458e-02_rb,0.40422e-02_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.76793e-03_rb,0.43013e-02_rb,0.58634e-02_rb,0.68605e-02_rb,0.73917e-02_rb, &
        & 0.75160e-02_rb,0.72292e-02_rb,0.64867e-02_rb,0.41753e-02_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.47383e-03_rb,0.30719e-02_rb,0.43157e-02_rb,0.51299e-02_rb,0.56113e-02_rb, &
        & 0.57604e-02_rb,0.55584e-02_rb,0.49263e-02_rb,0.30099e-02_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.52030e-03_rb,0.32224e-02_rb,0.45088e-02_rb,0.53256e-02_rb,0.58049e-02_rb, &
        & 0.59369e-02_rb,0.57249e-02_rb,0.50950e-02_rb,0.31335e-02_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.55897e-03_rb,0.33601e-02_rb,0.46690e-02_rb,0.55008e-02_rb,0.59733e-02_rb, &
        & 0.60982e-02_rb,0.58703e-02_rb,0.52416e-02_rb,0.32505e-02_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.59051e-03_rb,0.34823e-02_rb,0.48003e-02_rb,0.56492e-02_rb,0.61147e-02_rb, &
        & 0.62407e-02_rb,0.60011e-02_rb,0.53776e-02_rb,0.33691e-02_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.61638e-03_rb,0.35908e-02_rb,0.49151e-02_rb,0.57682e-02_rb,0.62269e-02_rb, &
        & 0.63505e-02_rb,0.61226e-02_rb,0.55061e-02_rb,0.34893e-02_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.36452e-03_rb,0.25301e-02_rb,0.35655e-02_rb,0.42558e-02_rb,0.46668e-02_rb, &
        & 0.47995e-02_rb,0.46391e-02_rb,0.41176e-02_rb,0.24723e-02_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.40521e-03_rb,0.26609e-02_rb,0.37368e-02_rb,0.44313e-02_rb,0.48422e-02_rb, &
        & 0.49605e-02_rb,0.47876e-02_rb,0.42682e-02_rb,0.25854e-02_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.44042e-03_rb,0.27811e-02_rb,0.38822e-02_rb,0.45871e-02_rb,0.49951e-02_rb, &
        & 0.51048e-02_rb,0.49189e-02_rb,0.43988e-02_rb,0.26922e-02_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.46934e-03_rb,0.28890e-02_rb,0.40015e-02_rb,0.47204e-02_rb,0.51195e-02_rb, &
        & 0.52329e-02_rb,0.50398e-02_rb,0.45211e-02_rb,0.27947e-02_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.49296e-03_rb,0.29863e-02_rb,0.41052e-02_rb,0.48296e-02_rb,0.52212e-02_rb, &
        & 0.53320e-02_rb,0.51516e-02_rb,0.46410e-02_rb,0.29014e-02_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.28141e-03_rb,0.20839e-02_rb,0.29434e-02_rb,0.35250e-02_rb,0.38705e-02_rb, &
        & 0.39907e-02_rb,0.38574e-02_rb,0.34280e-02_rb,0.20276e-02_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.31642e-03_rb,0.21962e-02_rb,0.30947e-02_rb,0.36798e-02_rb,0.40279e-02_rb, &
        & 0.41317e-02_rb,0.39905e-02_rb,0.35591e-02_rb,0.21304e-02_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.34743e-03_rb,0.23002e-02_rb,0.32251e-02_rb,0.38167e-02_rb,0.41632e-02_rb, &
        & 0.42588e-02_rb,0.41077e-02_rb,0.36737e-02_rb,0.22246e-02_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.37337e-03_rb,0.23958e-02_rb,0.33326e-02_rb,0.39351e-02_rb,0.42727e-02_rb, &
        & 0.43699e-02_rb,0.42167e-02_rb,0.37850e-02_rb,0.23171e-02_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.39464e-03_rb,0.24818e-02_rb,0.34245e-02_rb,0.40341e-02_rb,0.43635e-02_rb, &
        & 0.44592e-02_rb,0.43133e-02_rb,0.38931e-02_rb,0.24123e-02_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.23046e-03_rb,0.17561e-02_rb,0.24821e-02_rb,0.29697e-02_rb,0.32603e-02_rb, &
        & 0.33590e-02_rb,0.32465e-02_rb,0.28911e-02_rb,0.17000e-02_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.25900e-03_rb,0.18499e-02_rb,0.26079e-02_rb,0.30983e-02_rb,0.33914e-02_rb, &
        & 0.34790e-02_rb,0.33598e-02_rb,0.30000e-02_rb,0.17879e-02_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.28431e-03_rb,0.19371e-02_rb,0.27154e-02_rb,0.32123e-02_rb,0.35009e-02_rb, &
        & 0.35810e-02_rb,0.34582e-02_rb,0.30989e-02_rb,0.18709e-02_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.30554e-03_rb,0.20175e-02_rb,0.28047e-02_rb,0.33094e-02_rb,0.35897e-02_rb, &
        & 0.36708e-02_rb,0.35500e-02_rb,0.31964e-02_rb,0.19524e-02_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.32290e-03_rb,0.20906e-02_rb,0.28827e-02_rb,0.33907e-02_rb,0.36638e-02_rb, &
        & 0.37448e-02_rb,0.36322e-02_rb,0.32921e-02_rb,0.20342e-02_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.18873e-03_rb,0.14753e-02_rb,0.20859e-02_rb,0.24932e-02_rb,0.27362e-02_rb, &
        & 0.28163e-02_rb,0.27231e-02_rb,0.24287e-02_rb,0.14226e-02_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.21206e-03_rb,0.15542e-02_rb,0.21905e-02_rb,0.25997e-02_rb,0.28436e-02_rb, &
        & 0.29146e-02_rb,0.28182e-02_rb,0.25196e-02_rb,0.14964e-02_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.23271e-03_rb,0.16282e-02_rb,0.22789e-02_rb,0.26937e-02_rb,0.29320e-02_rb, &
        & 0.29995e-02_rb,0.29004e-02_rb,0.26063e-02_rb,0.15684e-02_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.25005e-03_rb,0.16959e-02_rb,0.23539e-02_rb,0.27739e-02_rb,0.30052e-02_rb, &
        & 0.30738e-02_rb,0.29781e-02_rb,0.26924e-02_rb,0.16388e-02_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.26424e-03_rb,0.17579e-02_rb,0.24198e-02_rb,0.28418e-02_rb,0.30688e-02_rb, &
        & 0.31370e-02_rb,0.30478e-02_rb,0.27734e-02_rb,0.17090e-02_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.15451e-03_rb,0.12362e-02_rb,0.17471e-02_rb,0.20856e-02_rb,0.22870e-02_rb, &
        & 0.23530e-02_rb,0.22758e-02_rb,0.20318e-02_rb,0.11868e-02_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.17357e-03_rb,0.13027e-02_rb,0.18337e-02_rb,0.21736e-02_rb,0.23752e-02_rb, &
        & 0.24339e-02_rb,0.23549e-02_rb,0.21104e-02_rb,0.12492e-02_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.19045e-03_rb,0.13651e-02_rb,0.19071e-02_rb,0.22520e-02_rb,0.24479e-02_rb, &
        & 0.25052e-02_rb,0.24255e-02_rb,0.21862e-02_rb,0.13114e-02_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.20459e-03_rb,0.14225e-02_rb,0.19702e-02_rb,0.23195e-02_rb,0.25109e-02_rb, &
        & 0.25680e-02_rb,0.24906e-02_rb,0.22592e-02_rb,0.13717e-02_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.21621e-03_rb,0.14751e-02_rb,0.20267e-02_rb,0.23775e-02_rb,0.25654e-02_rb, &
        & 0.26221e-02_rb,0.25513e-02_rb,0.23283e-02_rb,0.14321e-02_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.14551e-01_rb,0.26282e-01_rb,0.32053e-01_rb,0.35189e-01_rb,0.36600e-01_rb, &
        & 0.36646e-01_rb,0.35552e-01_rb,0.33215e-01_rb,0.27309e-01_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.15360e-01_rb,0.26847e-01_rb,0.32411e-01_rb,0.35426e-01_rb,0.36803e-01_rb, &
        & 0.36994e-01_rb,0.36037e-01_rb,0.33674e-01_rb,0.27881e-01_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.16052e-01_rb,0.27358e-01_rb,0.32711e-01_rb,0.35602e-01_rb,0.37002e-01_rb, &
        & 0.37322e-01_rb,0.36495e-01_rb,0.34092e-01_rb,0.28365e-01_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.16634e-01_rb,0.27765e-01_rb,0.32975e-01_rb,0.35757e-01_rb,0.37198e-01_rb, &
        & 0.37629e-01_rb,0.36871e-01_rb,0.34489e-01_rb,0.28817e-01_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.17092e-01_rb,0.28088e-01_rb,0.33173e-01_rb,0.35899e-01_rb,0.37369e-01_rb, &
        & 0.37894e-01_rb,0.37206e-01_rb,0.34856e-01_rb,0.29236e-01_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.12675e-01_rb,0.23757e-01_rb,0.28827e-01_rb,0.31475e-01_rb,0.32647e-01_rb, &
        & 0.32779e-01_rb,0.31874e-01_rb,0.29659e-01_rb,0.23735e-01_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.13363e-01_rb,0.24261e-01_rb,0.29165e-01_rb,0.31734e-01_rb,0.32905e-01_rb, &
        & 0.33137e-01_rb,0.32357e-01_rb,0.30114e-01_rb,0.24241e-01_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.13952e-01_rb,0.24673e-01_rb,0.29458e-01_rb,0.31940e-01_rb,0.33167e-01_rb, &
        & 0.33486e-01_rb,0.32794e-01_rb,0.30540e-01_rb,0.24704e-01_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.14440e-01_rb,0.25010e-01_rb,0.29692e-01_rb,0.32131e-01_rb,0.33402e-01_rb, &
        & 0.33816e-01_rb,0.33185e-01_rb,0.30944e-01_rb,0.25123e-01_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.14810e-01_rb,0.25246e-01_rb,0.29865e-01_rb,0.32323e-01_rb,0.33624e-01_rb, &
        & 0.34125e-01_rb,0.33530e-01_rb,0.31299e-01_rb,0.25501e-01_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.10764e-01_rb,0.21180e-01_rb,0.25645e-01_rb,0.27934e-01_rb,0.28891e-01_rb, &
        & 0.28978e-01_rb,0.28215e-01_rb,0.26250e-01_rb,0.20340e-01_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.11371e-01_rb,0.21611e-01_rb,0.25976e-01_rb,0.28221e-01_rb,0.29204e-01_rb, &
        & 0.29347e-01_rb,0.28689e-01_rb,0.26687e-01_rb,0.20803e-01_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.11882e-01_rb,0.21977e-01_rb,0.26258e-01_rb,0.28467e-01_rb,0.29498e-01_rb, &
        & 0.29730e-01_rb,0.29113e-01_rb,0.27115e-01_rb,0.21241e-01_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.12302e-01_rb,0.22273e-01_rb,0.26477e-01_rb,0.28696e-01_rb,0.29766e-01_rb, &
        & 0.30099e-01_rb,0.29507e-01_rb,0.27503e-01_rb,0.21620e-01_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.12624e-01_rb,0.22481e-01_rb,0.26631e-01_rb,0.28900e-01_rb,0.30032e-01_rb, &
        & 0.30425e-01_rb,0.29859e-01_rb,0.27842e-01_rb,0.21966e-01_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.89860e-02_rb,0.18659e-01_rb,0.22605e-01_rb,0.24580e-01_rb,0.25383e-01_rb, &
        & 0.25435e-01_rb,0.24761e-01_rb,0.23037e-01_rb,0.17414e-01_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.95207e-02_rb,0.19048e-01_rb,0.22924e-01_rb,0.24888e-01_rb,0.25739e-01_rb, &
        & 0.25823e-01_rb,0.25200e-01_rb,0.23450e-01_rb,0.17847e-01_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.99723e-02_rb,0.19380e-01_rb,0.23182e-01_rb,0.25155e-01_rb,0.26061e-01_rb, &
        & 0.26209e-01_rb,0.25611e-01_rb,0.23862e-01_rb,0.18251e-01_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.10354e-01_rb,0.19644e-01_rb,0.23397e-01_rb,0.25401e-01_rb,0.26345e-01_rb, &
        & 0.26580e-01_rb,0.26000e-01_rb,0.24233e-01_rb,0.18600e-01_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.10637e-01_rb,0.19822e-01_rb,0.23556e-01_rb,0.25605e-01_rb,0.26619e-01_rb, &
        & 0.26913e-01_rb,0.26359e-01_rb,0.24562e-01_rb,0.18925e-01_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.74104e-02_rb,0.16264e-01_rb,0.19717e-01_rb,0.21452e-01_rb,0.22172e-01_rb, &
        & 0.22207e-01_rb,0.21598e-01_rb,0.20075e-01_rb,0.14880e-01_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.78778e-02_rb,0.16624e-01_rb,0.20022e-01_rb,0.21765e-01_rb,0.22531e-01_rb, &
        & 0.22596e-01_rb,0.21999e-01_rb,0.20458e-01_rb,0.15279e-01_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.82694e-02_rb,0.16928e-01_rb,0.20277e-01_rb,0.22036e-01_rb,0.22859e-01_rb, &
        & 0.22978e-01_rb,0.22384e-01_rb,0.20834e-01_rb,0.15650e-01_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.85919e-02_rb,0.17168e-01_rb,0.20498e-01_rb,0.22274e-01_rb,0.23143e-01_rb, &
        & 0.23326e-01_rb,0.22754e-01_rb,0.21189e-01_rb,0.15980e-01_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.88337e-02_rb,0.17332e-01_rb,0.20664e-01_rb,0.22477e-01_rb,0.23407e-01_rb, &
        & 0.23648e-01_rb,0.23111e-01_rb,0.21509e-01_rb,0.16283e-01_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.60305e-02_rb,0.14008e-01_rb,0.17028e-01_rb,0.18574e-01_rb,0.19236e-01_rb, &
        & 0.19269e-01_rb,0.18720e-01_rb,0.17347e-01_rb,0.12656e-01_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.64356e-02_rb,0.14339e-01_rb,0.17318e-01_rb,0.18872e-01_rb,0.19580e-01_rb, &
        & 0.19645e-01_rb,0.19095e-01_rb,0.17705e-01_rb,0.13025e-01_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.67708e-02_rb,0.14626e-01_rb,0.17575e-01_rb,0.19135e-01_rb,0.19883e-01_rb, &
        & 0.20004e-01_rb,0.19454e-01_rb,0.18055e-01_rb,0.13366e-01_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.70449e-02_rb,0.14853e-01_rb,0.17795e-01_rb,0.19363e-01_rb,0.20156e-01_rb, &
        & 0.20324e-01_rb,0.19811e-01_rb,0.18392e-01_rb,0.13679e-01_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.72586e-02_rb,0.15014e-01_rb,0.17954e-01_rb,0.19569e-01_rb,0.20401e-01_rb, &
        & 0.20622e-01_rb,0.20155e-01_rb,0.18706e-01_rb,0.13963e-01_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.48615e-02_rb,0.11936e-01_rb,0.14574e-01_rb,0.15963e-01_rb,0.16578e-01_rb, &
        & 0.16615e-01_rb,0.16125e-01_rb,0.14904e-01_rb,0.10718e-01_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.52093e-02_rb,0.12247e-01_rb,0.14845e-01_rb,0.16239e-01_rb,0.16890e-01_rb, &
        & 0.16965e-01_rb,0.16477e-01_rb,0.15238e-01_rb,0.11050e-01_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.54981e-02_rb,0.12520e-01_rb,0.15089e-01_rb,0.16485e-01_rb,0.17163e-01_rb, &
        & 0.17296e-01_rb,0.16823e-01_rb,0.15561e-01_rb,0.11367e-01_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.57341e-02_rb,0.12732e-01_rb,0.15302e-01_rb,0.16709e-01_rb,0.17412e-01_rb, &
        & 0.17587e-01_rb,0.17156e-01_rb,0.15882e-01_rb,0.11657e-01_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.59245e-02_rb,0.12879e-01_rb,0.15460e-01_rb,0.16900e-01_rb,0.17646e-01_rb, &
        & 0.17862e-01_rb,0.17475e-01_rb,0.16189e-01_rb,0.11924e-01_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.38903e-02_rb,0.10086e-01_rb,0.12393e-01_rb,0.13617e-01_rb,0.14179e-01_rb, &
        & 0.14230e-01_rb,0.13810e-01_rb,0.12741e-01_rb,0.90402e-02_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.41883e-02_rb,0.10379e-01_rb,0.12642e-01_rb,0.13866e-01_rb,0.14454e-01_rb, &
        & 0.14549e-01_rb,0.14140e-01_rb,0.13050e-01_rb,0.93407e-02_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.44374e-02_rb,0.10628e-01_rb,0.12866e-01_rb,0.14093e-01_rb,0.14702e-01_rb, &
        & 0.14847e-01_rb,0.14466e-01_rb,0.13354e-01_rb,0.96289e-02_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.46417e-02_rb,0.10823e-01_rb,0.13067e-01_rb,0.14297e-01_rb,0.14936e-01_rb, &
        & 0.15113e-01_rb,0.14777e-01_rb,0.13659e-01_rb,0.98946e-02_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.48106e-02_rb,0.10958e-01_rb,0.13212e-01_rb,0.14477e-01_rb,0.15155e-01_rb, &
        & 0.15369e-01_rb,0.15065e-01_rb,0.13954e-01_rb,0.10137e-01_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.30906e-02_rb,0.84716e-02_rb,0.10475e-01_rb,0.11533e-01_rb,0.12037e-01_rb, &
        & 0.12102e-01_rb,0.11757e-01_rb,0.10840e-01_rb,0.75790e-02_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.33456e-02_rb,0.87380e-02_rb,0.10698e-01_rb,0.11759e-01_rb,0.12281e-01_rb, &
        & 0.12384e-01_rb,0.12063e-01_rb,0.11124e-01_rb,0.78524e-02_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.35624e-02_rb,0.89636e-02_rb,0.10908e-01_rb,0.11964e-01_rb,0.12503e-01_rb, &
        & 0.12651e-01_rb,0.12366e-01_rb,0.11413e-01_rb,0.81116e-02_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.37409e-02_rb,0.91394e-02_rb,0.11085e-01_rb,0.12156e-01_rb,0.12717e-01_rb, &
        & 0.12895e-01_rb,0.12646e-01_rb,0.11703e-01_rb,0.83543e-02_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.38899e-02_rb,0.92663e-02_rb,0.11210e-01_rb,0.12323e-01_rb,0.12923e-01_rb, &
        & 0.13135e-01_rb,0.12909e-01_rb,0.11979e-01_rb,0.85748e-02_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.24588e-02_rb,0.70997e-02_rb,0.88156e-02_rb,0.97280e-02_rb,0.10175e-01_rb, &
        & 0.10246e-01_rb,0.99712e-02_rb,0.91951e-02_rb,0.63324e-02_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.26743e-02_rb,0.73393e-02_rb,0.90183e-02_rb,0.99289e-02_rb,0.10387e-01_rb, &
        & 0.10496e-01_rb,0.10249e-01_rb,0.94566e-02_rb,0.65770e-02_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.28610e-02_rb,0.75352e-02_rb,0.92028e-02_rb,0.10117e-01_rb,0.10586e-01_rb, &
        & 0.10730e-01_rb,0.10524e-01_rb,0.97307e-02_rb,0.68077e-02_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.30147e-02_rb,0.76899e-02_rb,0.93550e-02_rb,0.10291e-01_rb,0.10781e-01_rb, &
        & 0.10954e-01_rb,0.10778e-01_rb,0.10001e-01_rb,0.70201e-02_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.31444e-02_rb,0.78050e-02_rb,0.94668e-02_rb,0.10433e-01_rb,0.10974e-01_rb, &
        & 0.11173e-01_rb,0.11019e-01_rb,0.10253e-01_rb,0.72189e-02_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.20345e-02_rb,0.60249e-02_rb,0.74716e-02_rb,0.82496e-02_rb,0.86458e-02_rb, &
        & 0.87340e-02_rb,0.85222e-02_rb,0.78657e-02_rb,0.53714e-02_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.22071e-02_rb,0.62176e-02_rb,0.76440e-02_rb,0.84251e-02_rb,0.88273e-02_rb, &
        & 0.89461e-02_rb,0.87689e-02_rb,0.81125e-02_rb,0.55835e-02_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.23554e-02_rb,0.63729e-02_rb,0.77932e-02_rb,0.85868e-02_rb,0.90058e-02_rb, &
        & 0.91548e-02_rb,0.90075e-02_rb,0.83660e-02_rb,0.57827e-02_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.24780e-02_rb,0.64944e-02_rb,0.79106e-02_rb,0.87291e-02_rb,0.91857e-02_rb, &
        & 0.93565e-02_rb,0.92302e-02_rb,0.86036e-02_rb,0.59695e-02_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.25820e-02_rb,0.65802e-02_rb,0.79974e-02_rb,0.88427e-02_rb,0.93482e-02_rb, &
        & 0.95539e-02_rb,0.94443e-02_rb,0.88180e-02_rb,0.61512e-02_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.16789e-02_rb,0.50906e-02_rb,0.63078e-02_rb,0.69750e-02_rb,0.73172e-02_rb, &
        & 0.74078e-02_rb,0.72504e-02_rb,0.67015e-02_rb,0.45394e-02_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.18170e-02_rb,0.52437e-02_rb,0.64512e-02_rb,0.71255e-02_rb,0.74790e-02_rb, &
        & 0.75959e-02_rb,0.74655e-02_rb,0.69307e-02_rb,0.47237e-02_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.19356e-02_rb,0.53689e-02_rb,0.65701e-02_rb,0.72610e-02_rb,0.76415e-02_rb, &
        & 0.77811e-02_rb,0.76703e-02_rb,0.71508e-02_rb,0.48953e-02_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.20342e-02_rb,0.54644e-02_rb,0.66640e-02_rb,0.73792e-02_rb,0.77929e-02_rb, &
        & 0.79635e-02_rb,0.78660e-02_rb,0.73532e-02_rb,0.50607e-02_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.21182e-02_rb,0.55328e-02_rb,0.67352e-02_rb,0.74736e-02_rb,0.79282e-02_rb, &
        & 0.81307e-02_rb,0.80542e-02_rb,0.75407e-02_rb,0.52234e-02_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.13819e-02_rb,0.42823e-02_rb,0.53096e-02_rb,0.58771e-02_rb,0.61745e-02_rb, &
        & 0.62601e-02_rb,0.61411e-02_rb,0.56893e-02_rb,0.38201e-02_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.14927e-02_rb,0.44078e-02_rb,0.54261e-02_rb,0.60055e-02_rb,0.63167e-02_rb, &
        & 0.64278e-02_rb,0.63268e-02_rb,0.58891e-02_rb,0.39789e-02_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.15884e-02_rb,0.45090e-02_rb,0.55246e-02_rb,0.61217e-02_rb,0.64577e-02_rb, &
        & 0.65903e-02_rb,0.65044e-02_rb,0.60758e-02_rb,0.41288e-02_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.16683e-02_rb,0.45858e-02_rb,0.56029e-02_rb,0.62193e-02_rb,0.65859e-02_rb, &
        & 0.67454e-02_rb,0.66768e-02_rb,0.62506e-02_rb,0.42765e-02_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.17364e-02_rb,0.46419e-02_rb,0.56644e-02_rb,0.62996e-02_rb,0.66998e-02_rb, &
        & 0.68888e-02_rb,0.68404e-02_rb,0.64188e-02_rb,0.44198e-02_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.57760e-01_rb,0.69432e-01_rb,0.76402e-01_rb,0.81517e-01_rb,0.85125e-01_rb, &
        & 0.87289e-01_rb,0.87408e-01_rb,0.83317e-01_rb,0.74720e-01_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.58256e-01_rb,0.69667e-01_rb,0.76723e-01_rb,0.81911e-01_rb,0.85362e-01_rb, &
        & 0.87289e-01_rb,0.87306e-01_rb,0.83357e-01_rb,0.75177e-01_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.58710e-01_rb,0.69859e-01_rb,0.76973e-01_rb,0.82328e-01_rb,0.85690e-01_rb, &
        & 0.87420e-01_rb,0.87264e-01_rb,0.83435e-01_rb,0.75701e-01_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.59052e-01_rb,0.70050e-01_rb,0.77176e-01_rb,0.82642e-01_rb,0.86071e-01_rb, &
        & 0.87660e-01_rb,0.87341e-01_rb,0.83544e-01_rb,0.76230e-01_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.59298e-01_rb,0.70198e-01_rb,0.77374e-01_rb,0.82883e-01_rb,0.86469e-01_rb, &
        & 0.87977e-01_rb,0.87473e-01_rb,0.83691e-01_rb,0.76769e-01_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.50908e-01_rb,0.62848e-01_rb,0.70003e-01_rb,0.75263e-01_rb,0.78670e-01_rb, &
        & 0.80594e-01_rb,0.80694e-01_rb,0.77048e-01_rb,0.67048e-01_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.51469e-01_rb,0.63178e-01_rb,0.70395e-01_rb,0.75763e-01_rb,0.79033e-01_rb, &
        & 0.80775e-01_rb,0.80686e-01_rb,0.77199e-01_rb,0.67605e-01_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.51956e-01_rb,0.63519e-01_rb,0.70748e-01_rb,0.76221e-01_rb,0.79512e-01_rb, &
        & 0.81067e-01_rb,0.80797e-01_rb,0.77370e-01_rb,0.68182e-01_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.52329e-01_rb,0.63831e-01_rb,0.71115e-01_rb,0.76591e-01_rb,0.80032e-01_rb, &
        & 0.81445e-01_rb,0.81001e-01_rb,0.77568e-01_rb,0.68768e-01_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.52648e-01_rb,0.64108e-01_rb,0.71441e-01_rb,0.76941e-01_rb,0.80464e-01_rb, &
        & 0.81866e-01_rb,0.81282e-01_rb,0.77796e-01_rb,0.69377e-01_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.44636e-01_rb,0.56575e-01_rb,0.63667e-01_rb,0.68804e-01_rb,0.72146e-01_rb, &
        & 0.73940e-01_rb,0.73756e-01_rb,0.70359e-01_rb,0.59446e-01_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.45216e-01_rb,0.57005e-01_rb,0.64111e-01_rb,0.69328e-01_rb,0.72620e-01_rb, &
        & 0.74228e-01_rb,0.73854e-01_rb,0.70600e-01_rb,0.60086e-01_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.45716e-01_rb,0.57448e-01_rb,0.64591e-01_rb,0.69816e-01_rb,0.73178e-01_rb, &
        & 0.74616e-01_rb,0.74093e-01_rb,0.70843e-01_rb,0.60738e-01_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.46137e-01_rb,0.57862e-01_rb,0.65060e-01_rb,0.70283e-01_rb,0.73724e-01_rb, &
        & 0.75067e-01_rb,0.74408e-01_rb,0.71130e-01_rb,0.61385e-01_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.46490e-01_rb,0.58240e-01_rb,0.65537e-01_rb,0.70752e-01_rb,0.74187e-01_rb, &
        & 0.75545e-01_rb,0.74817e-01_rb,0.71479e-01_rb,0.61998e-01_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.38910e-01_rb,0.50823e-01_rb,0.57505e-01_rb,0.62364e-01_rb,0.65559e-01_rb, &
        & 0.67154e-01_rb,0.66798e-01_rb,0.63531e-01_rb,0.52401e-01_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.39488e-01_rb,0.51307e-01_rb,0.58022e-01_rb,0.62912e-01_rb,0.66074e-01_rb, &
        & 0.67518e-01_rb,0.67025e-01_rb,0.63840e-01_rb,0.53093e-01_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.39991e-01_rb,0.51812e-01_rb,0.58566e-01_rb,0.63451e-01_rb,0.66632e-01_rb, &
        & 0.68010e-01_rb,0.67348e-01_rb,0.64169e-01_rb,0.53782e-01_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.40387e-01_rb,0.52291e-01_rb,0.59117e-01_rb,0.64001e-01_rb,0.67196e-01_rb, &
        & 0.68514e-01_rb,0.67789e-01_rb,0.64555e-01_rb,0.54470e-01_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.40719e-01_rb,0.52739e-01_rb,0.59691e-01_rb,0.64572e-01_rb,0.67725e-01_rb, &
        & 0.69031e-01_rb,0.68307e-01_rb,0.64985e-01_rb,0.55091e-01_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.33692e-01_rb,0.45459e-01_rb,0.51705e-01_rb,0.56111e-01_rb,0.58984e-01_rb, &
        & 0.60384e-01_rb,0.59951e-01_rb,0.56770e-01_rb,0.45939e-01_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.34254e-01_rb,0.45979e-01_rb,0.52252e-01_rb,0.56664e-01_rb,0.59515e-01_rb, &
        & 0.60816e-01_rb,0.60273e-01_rb,0.57146e-01_rb,0.46647e-01_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.34738e-01_rb,0.46532e-01_rb,0.52846e-01_rb,0.57238e-01_rb,0.60077e-01_rb, &
        & 0.61352e-01_rb,0.60714e-01_rb,0.57566e-01_rb,0.47359e-01_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.35117e-01_rb,0.47047e-01_rb,0.53459e-01_rb,0.57849e-01_rb,0.60668e-01_rb, &
        & 0.61902e-01_rb,0.61259e-01_rb,0.58030e-01_rb,0.48036e-01_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.35428e-01_rb,0.47508e-01_rb,0.54083e-01_rb,0.58493e-01_rb,0.61253e-01_rb, &
        & 0.62481e-01_rb,0.61851e-01_rb,0.58523e-01_rb,0.48648e-01_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.28910e-01_rb,0.40429e-01_rb,0.46176e-01_rb,0.50108e-01_rb,0.52609e-01_rb, &
        & 0.53750e-01_rb,0.53289e-01_rb,0.50286e-01_rb,0.40061e-01_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.29433e-01_rb,0.40967e-01_rb,0.46731e-01_rb,0.50653e-01_rb,0.53149e-01_rb, &
        & 0.54232e-01_rb,0.53687e-01_rb,0.50714e-01_rb,0.40756e-01_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.29883e-01_rb,0.41520e-01_rb,0.47349e-01_rb,0.51251e-01_rb,0.53731e-01_rb, &
        & 0.54796e-01_rb,0.54207e-01_rb,0.51188e-01_rb,0.41443e-01_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.30252e-01_rb,0.42024e-01_rb,0.47998e-01_rb,0.51918e-01_rb,0.54339e-01_rb, &
        & 0.55387e-01_rb,0.54820e-01_rb,0.51711e-01_rb,0.42085e-01_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.30542e-01_rb,0.42447e-01_rb,0.48635e-01_rb,0.52598e-01_rb,0.54986e-01_rb, &
        & 0.56038e-01_rb,0.55430e-01_rb,0.52236e-01_rb,0.42675e-01_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.24600e-01_rb,0.35725e-01_rb,0.40952e-01_rb,0.44398e-01_rb,0.46558e-01_rb, &
        & 0.47462e-01_rb,0.46950e-01_rb,0.44227e-01_rb,0.34751e-01_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.25060e-01_rb,0.36250e-01_rb,0.41498e-01_rb,0.44950e-01_rb,0.47097e-01_rb, &
        & 0.47977e-01_rb,0.47399e-01_rb,0.44690e-01_rb,0.35410e-01_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.25466e-01_rb,0.36769e-01_rb,0.42119e-01_rb,0.45572e-01_rb,0.47696e-01_rb, &
        & 0.48548e-01_rb,0.47965e-01_rb,0.45198e-01_rb,0.36050e-01_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.25804e-01_rb,0.37232e-01_rb,0.42764e-01_rb,0.46249e-01_rb,0.48349e-01_rb, &
        & 0.49173e-01_rb,0.48603e-01_rb,0.45737e-01_rb,0.36649e-01_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.26075e-01_rb,0.37645e-01_rb,0.43371e-01_rb,0.46950e-01_rb,0.49045e-01_rb, &
        & 0.49856e-01_rb,0.49223e-01_rb,0.46279e-01_rb,0.37198e-01_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.20747e-01_rb,0.31325e-01_rb,0.36016e-01_rb,0.39047e-01_rb,0.40921e-01_rb, &
        & 0.41656e-01_rb,0.41097e-01_rb,0.38682e-01_rb,0.29955e-01_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.21145e-01_rb,0.31811e-01_rb,0.36541e-01_rb,0.39597e-01_rb,0.41455e-01_rb, &
        & 0.42169e-01_rb,0.41575e-01_rb,0.39145e-01_rb,0.30577e-01_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.21497e-01_rb,0.32290e-01_rb,0.37155e-01_rb,0.40238e-01_rb,0.42067e-01_rb, &
        & 0.42744e-01_rb,0.42154e-01_rb,0.39648e-01_rb,0.31178e-01_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.21796e-01_rb,0.32717e-01_rb,0.37766e-01_rb,0.40923e-01_rb,0.42738e-01_rb, &
        & 0.43394e-01_rb,0.42768e-01_rb,0.40185e-01_rb,0.31733e-01_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.22044e-01_rb,0.33112e-01_rb,0.38338e-01_rb,0.41597e-01_rb,0.43462e-01_rb, &
        & 0.44087e-01_rb,0.43385e-01_rb,0.40740e-01_rb,0.32254e-01_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.17360e-01_rb,0.27218e-01_rb,0.31401e-01_rb,0.34084e-01_rb,0.35741e-01_rb, &
        & 0.36373e-01_rb,0.35798e-01_rb,0.33594e-01_rb,0.25664e-01_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.17693e-01_rb,0.27662e-01_rb,0.31908e-01_rb,0.34622e-01_rb,0.36264e-01_rb, &
        & 0.36877e-01_rb,0.36285e-01_rb,0.34049e-01_rb,0.26251e-01_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.17992e-01_rb,0.28089e-01_rb,0.32477e-01_rb,0.35255e-01_rb,0.36874e-01_rb, &
        & 0.37444e-01_rb,0.36831e-01_rb,0.34544e-01_rb,0.26814e-01_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.18258e-01_rb,0.28481e-01_rb,0.33050e-01_rb,0.35909e-01_rb,0.37552e-01_rb, &
        & 0.38078e-01_rb,0.37421e-01_rb,0.35073e-01_rb,0.27335e-01_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.18484e-01_rb,0.28849e-01_rb,0.33601e-01_rb,0.36536e-01_rb,0.38254e-01_rb, &
        & 0.38756e-01_rb,0.38026e-01_rb,0.35616e-01_rb,0.27829e-01_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.14462e-01_rb,0.23463e-01_rb,0.27186e-01_rb,0.29588e-01_rb,0.31079e-01_rb, &
        & 0.31625e-01_rb,0.31053e-01_rb,0.29012e-01_rb,0.21916e-01_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.14737e-01_rb,0.23853e-01_rb,0.27662e-01_rb,0.30109e-01_rb,0.31581e-01_rb, &
        & 0.32116e-01_rb,0.31527e-01_rb,0.29461e-01_rb,0.22460e-01_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.14983e-01_rb,0.24238e-01_rb,0.28187e-01_rb,0.30710e-01_rb,0.32176e-01_rb, &
        & 0.32664e-01_rb,0.32044e-01_rb,0.29948e-01_rb,0.22987e-01_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.15228e-01_rb,0.24593e-01_rb,0.28718e-01_rb,0.31312e-01_rb,0.32826e-01_rb, &
        & 0.33282e-01_rb,0.32615e-01_rb,0.30467e-01_rb,0.23478e-01_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.15411e-01_rb,0.24932e-01_rb,0.29228e-01_rb,0.31903e-01_rb,0.33455e-01_rb, &
        & 0.33943e-01_rb,0.33207e-01_rb,0.30994e-01_rb,0.23939e-01_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.12097e-01_rb,0.20230e-01_rb,0.23554e-01_rb,0.25737e-01_rb,0.27070e-01_rb, &
        & 0.27526e-01_rb,0.26987e-01_rb,0.25125e-01_rb,0.18843e-01_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.12316e-01_rb,0.20564e-01_rb,0.24014e-01_rb,0.26265e-01_rb,0.27589e-01_rb, &
        & 0.28021e-01_rb,0.27462e-01_rb,0.25580e-01_rb,0.19341e-01_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.12525e-01_rb,0.20893e-01_rb,0.24494e-01_rb,0.26817e-01_rb,0.28172e-01_rb, &
        & 0.28578e-01_rb,0.27985e-01_rb,0.26073e-01_rb,0.19811e-01_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.12698e-01_rb,0.21212e-01_rb,0.24971e-01_rb,0.27358e-01_rb,0.28752e-01_rb, &
        & 0.29196e-01_rb,0.28554e-01_rb,0.26575e-01_rb,0.20254e-01_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.12813e-01_rb,0.21527e-01_rb,0.25428e-01_rb,0.27886e-01_rb,0.29316e-01_rb, &
        & 0.29794e-01_rb,0.29144e-01_rb,0.27095e-01_rb,0.20669e-01_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.10079e-01_rb,0.17332e-01_rb,0.20326e-01_rb,0.22283e-01_rb,0.23452e-01_rb, &
        & 0.23840e-01_rb,0.23351e-01_rb,0.21701e-01_rb,0.16150e-01_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.10253e-01_rb,0.17624e-01_rb,0.20738e-01_rb,0.22772e-01_rb,0.23963e-01_rb, &
        & 0.24343e-01_rb,0.23830e-01_rb,0.22159e-01_rb,0.16605e-01_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.10412e-01_rb,0.17914e-01_rb,0.21165e-01_rb,0.23257e-01_rb,0.24491e-01_rb, &
        & 0.24907e-01_rb,0.24364e-01_rb,0.22633e-01_rb,0.17031e-01_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.10528e-01_rb,0.18203e-01_rb,0.21585e-01_rb,0.23732e-01_rb,0.25019e-01_rb, &
        & 0.25466e-01_rb,0.24925e-01_rb,0.23123e-01_rb,0.17440e-01_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.10595e-01_rb,0.18491e-01_rb,0.21987e-01_rb,0.24205e-01_rb,0.25526e-01_rb, &
        & 0.26013e-01_rb,0.25497e-01_rb,0.23633e-01_rb,0.17825e-01_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.83675e-02_rb,0.14778e-01_rb,0.17455e-01_rb,0.19201e-01_rb,0.20214e-01_rb, &
        & 0.20545e-01_rb,0.20133e-01_rb,0.18703e-01_rb,0.13810e-01_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.85037e-02_rb,0.15033e-01_rb,0.17824e-01_rb,0.19621e-01_rb,0.20684e-01_rb, &
        & 0.21052e-01_rb,0.20616e-01_rb,0.19145e-01_rb,0.14218e-01_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.86129e-02_rb,0.15303e-01_rb,0.18197e-01_rb,0.20043e-01_rb,0.21163e-01_rb, &
        & 0.21577e-01_rb,0.21143e-01_rb,0.19600e-01_rb,0.14617e-01_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.86912e-02_rb,0.15568e-01_rb,0.18565e-01_rb,0.20472e-01_rb,0.21636e-01_rb, &
        & 0.22091e-01_rb,0.21674e-01_rb,0.20090e-01_rb,0.14991e-01_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.87302e-02_rb,0.15800e-01_rb,0.18917e-01_rb,0.20887e-01_rb,0.22103e-01_rb, &
        & 0.22594e-01_rb,0.22198e-01_rb,0.20586e-01_rb,0.15351e-01_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.23898e+00_rb,0.24353e+00_rb,0.26297e+00_rb,0.27410e+00_rb,0.27635e+00_rb, &
        & 0.27151e+00_rb,0.26001e+00_rb,0.24536e+00_rb,0.24840e+00_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.23703e+00_rb,0.24151e+00_rb,0.26057e+00_rb,0.27148e+00_rb,0.27415e+00_rb, &
        & 0.26993e+00_rb,0.25912e+00_rb,0.24520e+00_rb,0.24830e+00_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.23567e+00_rb,0.23998e+00_rb,0.25868e+00_rb,0.26917e+00_rb,0.27195e+00_rb, &
        & 0.26833e+00_rb,0.25809e+00_rb,0.24488e+00_rb,0.24801e+00_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.23478e+00_rb,0.23885e+00_rb,0.25713e+00_rb,0.26729e+00_rb,0.26990e+00_rb, &
        & 0.26662e+00_rb,0.25696e+00_rb,0.24453e+00_rb,0.24772e+00_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.23430e+00_rb,0.23810e+00_rb,0.25588e+00_rb,0.26568e+00_rb,0.26809e+00_rb, &
        & 0.26490e+00_rb,0.25575e+00_rb,0.24412e+00_rb,0.24745e+00_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.22071e+00_rb,0.23189e+00_rb,0.25280e+00_rb,0.26428e+00_rb,0.26846e+00_rb, &
        & 0.26544e+00_rb,0.25423e+00_rb,0.23818e+00_rb,0.23526e+00_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.21924e+00_rb,0.23025e+00_rb,0.25076e+00_rb,0.26194e+00_rb,0.26631e+00_rb, &
        & 0.26393e+00_rb,0.25356e+00_rb,0.23803e+00_rb,0.23534e+00_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.21843e+00_rb,0.22915e+00_rb,0.24919e+00_rb,0.26005e+00_rb,0.26425e+00_rb, &
        & 0.26237e+00_rb,0.25262e+00_rb,0.23788e+00_rb,0.23535e+00_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.21819e+00_rb,0.22852e+00_rb,0.24801e+00_rb,0.25855e+00_rb,0.26247e+00_rb, &
        & 0.26079e+00_rb,0.25163e+00_rb,0.23767e+00_rb,0.23532e+00_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.21836e+00_rb,0.22826e+00_rb,0.24718e+00_rb,0.25726e+00_rb,0.26101e+00_rb, &
        & 0.25924e+00_rb,0.25059e+00_rb,0.23733e+00_rb,0.23520e+00_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.20172e+00_rb,0.21909e+00_rb,0.24049e+00_rb,0.25259e+00_rb,0.25809e+00_rb, &
        & 0.25661e+00_rb,0.24776e+00_rb,0.23048e+00_rb,0.22176e+00_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.20072e+00_rb,0.21779e+00_rb,0.23872e+00_rb,0.25058e+00_rb,0.25610e+00_rb, &
        & 0.25527e+00_rb,0.24712e+00_rb,0.23055e+00_rb,0.22209e+00_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.20047e+00_rb,0.21708e+00_rb,0.23748e+00_rb,0.24902e+00_rb,0.25438e+00_rb, &
        & 0.25391e+00_rb,0.24629e+00_rb,0.23063e+00_rb,0.22233e+00_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.20080e+00_rb,0.21687e+00_rb,0.23670e+00_rb,0.24777e+00_rb,0.25298e+00_rb, &
        & 0.25252e+00_rb,0.24542e+00_rb,0.23053e+00_rb,0.22249e+00_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.20160e+00_rb,0.21710e+00_rb,0.23623e+00_rb,0.24683e+00_rb,0.25186e+00_rb, &
        & 0.25126e+00_rb,0.24453e+00_rb,0.23018e+00_rb,0.22257e+00_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.18271e+00_rb,0.20499e+00_rb,0.22645e+00_rb,0.23921e+00_rb,0.24553e+00_rb, &
        & 0.24573e+00_rb,0.23916e+00_rb,0.22190e+00_rb,0.20803e+00_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.18212e+00_rb,0.20407e+00_rb,0.22500e+00_rb,0.23755e+00_rb,0.24385e+00_rb, &
        & 0.24460e+00_rb,0.23866e+00_rb,0.22223e+00_rb,0.20862e+00_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.18234e+00_rb,0.20374e+00_rb,0.22412e+00_rb,0.23625e+00_rb,0.24249e+00_rb, &
        & 0.24341e+00_rb,0.23801e+00_rb,0.22245e+00_rb,0.20916e+00_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.18327e+00_rb,0.20396e+00_rb,0.22372e+00_rb,0.23535e+00_rb,0.24149e+00_rb, &
        & 0.24231e+00_rb,0.23730e+00_rb,0.22248e+00_rb,0.20951e+00_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.18465e+00_rb,0.20466e+00_rb,0.22368e+00_rb,0.23479e+00_rb,0.24072e+00_rb, &
        & 0.24142e+00_rb,0.23656e+00_rb,0.22226e+00_rb,0.20975e+00_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.16429e+00_rb,0.19018e+00_rb,0.21120e+00_rb,0.22440e+00_rb,0.23155e+00_rb, &
        & 0.23328e+00_rb,0.22834e+00_rb,0.21231e+00_rb,0.19389e+00_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.16411e+00_rb,0.18954e+00_rb,0.21010e+00_rb,0.22306e+00_rb,0.23026e+00_rb, &
        & 0.23235e+00_rb,0.22803e+00_rb,0.21280e+00_rb,0.19477e+00_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.16481e+00_rb,0.18955e+00_rb,0.20958e+00_rb,0.22215e+00_rb,0.22928e+00_rb, &
        & 0.23142e+00_rb,0.22762e+00_rb,0.21318e+00_rb,0.19549e+00_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.16626e+00_rb,0.19018e+00_rb,0.20952e+00_rb,0.22166e+00_rb,0.22860e+00_rb, &
        & 0.23069e+00_rb,0.22713e+00_rb,0.21333e+00_rb,0.19598e+00_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.16812e+00_rb,0.19134e+00_rb,0.20988e+00_rb,0.22152e+00_rb,0.22816e+00_rb, &
        & 0.23013e+00_rb,0.22663e+00_rb,0.21325e+00_rb,0.19645e+00_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.14677e+00_rb,0.17503e+00_rb,0.19553e+00_rb,0.20871e+00_rb,0.21680e+00_rb, &
        & 0.21948e+00_rb,0.21556e+00_rb,0.20158e+00_rb,0.17922e+00_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.14688e+00_rb,0.17460e+00_rb,0.19479e+00_rb,0.20777e+00_rb,0.21582e+00_rb, &
        & 0.21883e+00_rb,0.21549e+00_rb,0.20223e+00_rb,0.18038e+00_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.14791e+00_rb,0.17490e+00_rb,0.19459e+00_rb,0.20727e+00_rb,0.21519e+00_rb, &
        & 0.21822e+00_rb,0.21533e+00_rb,0.20278e+00_rb,0.18127e+00_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.14970e+00_rb,0.17592e+00_rb,0.19488e+00_rb,0.20713e+00_rb,0.21485e+00_rb, &
        & 0.21785e+00_rb,0.21514e+00_rb,0.20305e+00_rb,0.18200e+00_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.15169e+00_rb,0.17756e+00_rb,0.19568e+00_rb,0.20736e+00_rb,0.21479e+00_rb, &
        & 0.21766e+00_rb,0.21494e+00_rb,0.20315e+00_rb,0.18271e+00_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.13000e+00_rb,0.15987e+00_rb,0.17978e+00_rb,0.19301e+00_rb,0.20129e+00_rb, &
        & 0.20436e+00_rb,0.20140e+00_rb,0.18919e+00_rb,0.16414e+00_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.13035e+00_rb,0.15964e+00_rb,0.17934e+00_rb,0.19244e+00_rb,0.20068e+00_rb, &
        & 0.20402e+00_rb,0.20156e+00_rb,0.19006e+00_rb,0.16551e+00_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.13166e+00_rb,0.16021e+00_rb,0.17944e+00_rb,0.19225e+00_rb,0.20040e+00_rb, &
        & 0.20381e+00_rb,0.20166e+00_rb,0.19078e+00_rb,0.16662e+00_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.13361e+00_rb,0.16157e+00_rb,0.18010e+00_rb,0.19248e+00_rb,0.20041e+00_rb, &
        & 0.20379e+00_rb,0.20172e+00_rb,0.19125e+00_rb,0.16758e+00_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.13554e+00_rb,0.16344e+00_rb,0.18132e+00_rb,0.19308e+00_rb,0.20075e+00_rb, &
        & 0.20397e+00_rb,0.20182e+00_rb,0.19159e+00_rb,0.16855e+00_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.11433e+00_rb,0.14488e+00_rb,0.16431e+00_rb,0.17738e+00_rb,0.18527e+00_rb, &
        & 0.18841e+00_rb,0.18617e+00_rb,0.17535e+00_rb,0.14893e+00_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.11487e+00_rb,0.14487e+00_rb,0.16414e+00_rb,0.17713e+00_rb,0.18501e+00_rb, &
        & 0.18837e+00_rb,0.18658e+00_rb,0.17646e+00_rb,0.15049e+00_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.11635e+00_rb,0.14571e+00_rb,0.16452e+00_rb,0.17724e+00_rb,0.18506e+00_rb, &
        & 0.18853e+00_rb,0.18695e+00_rb,0.17741e+00_rb,0.15180e+00_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.11822e+00_rb,0.14733e+00_rb,0.16549e+00_rb,0.17782e+00_rb,0.18544e+00_rb, &
        & 0.18888e+00_rb,0.18734e+00_rb,0.17811e+00_rb,0.15295e+00_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.12009e+00_rb,0.14924e+00_rb,0.16704e+00_rb,0.17887e+00_rb,0.18614e+00_rb, &
        & 0.18940e+00_rb,0.18775e+00_rb,0.17867e+00_rb,0.15411e+00_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.10002e+00_rb,0.13043e+00_rb,0.14919e+00_rb,0.16171e+00_rb,0.16913e+00_rb, &
        & 0.17211e+00_rb,0.17023e+00_rb,0.16067e+00_rb,0.13380e+00_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.10070e+00_rb,0.13063e+00_rb,0.14927e+00_rb,0.16171e+00_rb,0.16923e+00_rb, &
        & 0.17237e+00_rb,0.17091e+00_rb,0.16202e+00_rb,0.13552e+00_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.10219e+00_rb,0.13166e+00_rb,0.14990e+00_rb,0.16210e+00_rb,0.16959e+00_rb, &
        & 0.17285e+00_rb,0.17161e+00_rb,0.16316e+00_rb,0.13700e+00_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.10390e+00_rb,0.13341e+00_rb,0.15118e+00_rb,0.16299e+00_rb,0.17031e+00_rb, &
        & 0.17358e+00_rb,0.17229e+00_rb,0.16405e+00_rb,0.13837e+00_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.10569e+00_rb,0.13530e+00_rb,0.15298e+00_rb,0.16444e+00_rb,0.17139e+00_rb, &
        & 0.17443e+00_rb,0.17301e+00_rb,0.16481e+00_rb,0.13976e+00_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.87148e-01_rb,0.11690e+00_rb,0.13450e+00_rb,0.14609e+00_rb,0.15307e+00_rb, &
        & 0.15588e+00_rb,0.15420e+00_rb,0.14572e+00_rb,0.11917e+00_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.87948e-01_rb,0.11730e+00_rb,0.13485e+00_rb,0.14637e+00_rb,0.15348e+00_rb, &
        & 0.15645e+00_rb,0.15513e+00_rb,0.14723e+00_rb,0.12103e+00_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.89341e-01_rb,0.11856e+00_rb,0.13575e+00_rb,0.14709e+00_rb,0.15418e+00_rb, &
        & 0.15726e+00_rb,0.15609e+00_rb,0.14852e+00_rb,0.12266e+00_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.90912e-01_rb,0.12032e+00_rb,0.13733e+00_rb,0.14835e+00_rb,0.15527e+00_rb, &
        & 0.15827e+00_rb,0.15702e+00_rb,0.14960e+00_rb,0.12424e+00_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.92651e-01_rb,0.12218e+00_rb,0.13930e+00_rb,0.15018e+00_rb,0.15676e+00_rb, &
        & 0.15942e+00_rb,0.15802e+00_rb,0.15063e+00_rb,0.12583e+00_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.75810e-01_rb,0.10456e+00_rb,0.12069e+00_rb,0.13117e+00_rb,0.13761e+00_rb, &
        & 0.14039e+00_rb,0.13895e+00_rb,0.13157e+00_rb,0.10610e+00_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.76895e-01_rb,0.10557e+00_rb,0.12151e+00_rb,0.13190e+00_rb,0.13842e+00_rb, &
        & 0.14134e+00_rb,0.14014e+00_rb,0.13312e+00_rb,0.10796e+00_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.78244e-01_rb,0.10718e+00_rb,0.12298e+00_rb,0.13318e+00_rb,0.13962e+00_rb, &
        & 0.14250e+00_rb,0.14130e+00_rb,0.13446e+00_rb,0.10971e+00_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.79823e-01_rb,0.10896e+00_rb,0.12491e+00_rb,0.13508e+00_rb,0.14123e+00_rb, &
        & 0.14382e+00_rb,0.14250e+00_rb,0.13575e+00_rb,0.11145e+00_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.81587e-01_rb,0.11081e+00_rb,0.12696e+00_rb,0.13729e+00_rb,0.14323e+00_rb, &
        & 0.14541e+00_rb,0.14385e+00_rb,0.13708e+00_rb,0.11322e+00_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.65751e-01_rb,0.93358e-01_rb,0.10793e+00_rb,0.11727e+00_rb,0.12307e+00_rb, &
        & 0.12565e+00_rb,0.12447e+00_rb,0.11782e+00_rb,0.93944e-01_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.66851e-01_rb,0.94791e-01_rb,0.10927e+00_rb,0.11849e+00_rb,0.12430e+00_rb, &
        & 0.12692e+00_rb,0.12584e+00_rb,0.11940e+00_rb,0.95825e-01_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.68215e-01_rb,0.96468e-01_rb,0.11113e+00_rb,0.12035e+00_rb,0.12596e+00_rb, &
        & 0.12837e+00_rb,0.12723e+00_rb,0.12088e+00_rb,0.97697e-01_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.69822e-01_rb,0.98288e-01_rb,0.11318e+00_rb,0.12260e+00_rb,0.12800e+00_rb, &
        & 0.13009e+00_rb,0.12875e+00_rb,0.12246e+00_rb,0.99558e-01_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.71562e-01_rb,0.10021e+00_rb,0.11536e+00_rb,0.12491e+00_rb,0.13035e+00_rb, &
        & 0.13218e+00_rb,0.13049e+00_rb,0.12412e+00_rb,0.10142e+00_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.56716e-01_rb,0.83175e-01_rb,0.96227e-01_rb,0.10452e+00_rb,0.10961e+00_rb, &
        & 0.11186e+00_rb,0.11078e+00_rb,0.10487e+00_rb,0.82822e-01_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.57829e-01_rb,0.84711e-01_rb,0.97973e-01_rb,0.10625e+00_rb,0.11123e+00_rb, &
        & 0.11339e+00_rb,0.11236e+00_rb,0.10648e+00_rb,0.84781e-01_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.59231e-01_rb,0.86426e-01_rb,0.99996e-01_rb,0.10847e+00_rb,0.11329e+00_rb, &
        & 0.11518e+00_rb,0.11401e+00_rb,0.10821e+00_rb,0.86727e-01_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.60793e-01_rb,0.88335e-01_rb,0.10220e+00_rb,0.11080e+00_rb,0.11568e+00_rb, &
        & 0.11734e+00_rb,0.11592e+00_rb,0.11003e+00_rb,0.88652e-01_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.62288e-01_rb,0.90379e-01_rb,0.10449e+00_rb,0.11321e+00_rb,0.11819e+00_rb, &
        & 0.11985e+00_rb,0.11808e+00_rb,0.11193e+00_rb,0.90608e-01_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.73370e+00_rb,0.66239e+00_rb,0.64754e+00_rb,0.65671e+00_rb,0.64798e+00_rb, &
        & 0.62443e+00_rb,0.62156e+00_rb,0.59642e+00_rb,0.65694e+00_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.72227e+00_rb,0.65245e+00_rb,0.64115e+00_rb,0.65244e+00_rb,0.64519e+00_rb, &
        & 0.62272e+00_rb,0.62178e+00_rb,0.59807e+00_rb,0.65734e+00_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.71096e+00_rb,0.64287e+00_rb,0.63490e+00_rb,0.64719e+00_rb,0.64246e+00_rb, &
        & 0.61928e+00_rb,0.62164e+00_rb,0.59971e+00_rb,0.65975e+00_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.69997e+00_rb,0.63340e+00_rb,0.62893e+00_rb,0.64188e+00_rb,0.63859e+00_rb, &
        & 0.61601e+00_rb,0.62127e+00_rb,0.59924e+00_rb,0.66003e+00_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.68921e+00_rb,0.62412e+00_rb,0.62274e+00_rb,0.63660e+00_rb,0.63341e+00_rb, &
        & 0.61371e+00_rb,0.61976e+00_rb,0.59796e+00_rb,0.65874e+00_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.71435e+00_rb,0.65898e+00_rb,0.65791e+00_rb,0.67621e+00_rb,0.66803e+00_rb, &
        & 0.63952e+00_rb,0.62227e+00_rb,0.60345e+00_rb,0.64341e+00_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.70295e+00_rb,0.64927e+00_rb,0.65209e+00_rb,0.67168e+00_rb,0.66601e+00_rb, &
        & 0.63761e+00_rb,0.62183e+00_rb,0.60472e+00_rb,0.64489e+00_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.69214e+00_rb,0.64012e+00_rb,0.64637e+00_rb,0.66639e+00_rb,0.66353e+00_rb, &
        & 0.63487e+00_rb,0.62217e+00_rb,0.60467e+00_rb,0.64521e+00_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.68145e+00_rb,0.63089e+00_rb,0.64022e+00_rb,0.66099e+00_rb,0.65898e+00_rb, &
        & 0.63312e+00_rb,0.62106e+00_rb,0.60298e+00_rb,0.64380e+00_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.67068e+00_rb,0.62208e+00_rb,0.63390e+00_rb,0.65546e+00_rb,0.65376e+00_rb, &
        & 0.63122e+00_rb,0.61818e+00_rb,0.60158e+00_rb,0.64256e+00_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.68340e+00_rb,0.64482e+00_rb,0.66290e+00_rb,0.68284e+00_rb,0.67896e+00_rb, &
        & 0.65800e+00_rb,0.61589e+00_rb,0.60026e+00_rb,0.62209e+00_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.67251e+00_rb,0.63550e+00_rb,0.65818e+00_rb,0.67891e+00_rb,0.67722e+00_rb, &
        & 0.65575e+00_rb,0.61745e+00_rb,0.60017e+00_rb,0.62247e+00_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.66182e+00_rb,0.62604e+00_rb,0.65231e+00_rb,0.67447e+00_rb,0.67369e+00_rb, &
        & 0.65314e+00_rb,0.61804e+00_rb,0.59851e+00_rb,0.62084e+00_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.65114e+00_rb,0.61727e+00_rb,0.64569e+00_rb,0.67007e+00_rb,0.66854e+00_rb, &
        & 0.65146e+00_rb,0.61713e+00_rb,0.59641e+00_rb,0.61922e+00_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.64110e+00_rb,0.60881e+00_rb,0.63926e+00_rb,0.66468e+00_rb,0.66347e+00_rb, &
        & 0.64931e+00_rb,0.61433e+00_rb,0.59600e+00_rb,0.61885e+00_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.64233e+00_rb,0.62207e+00_rb,0.65747e+00_rb,0.67845e+00_rb,0.68170e+00_rb, &
        & 0.66374e+00_rb,0.61548e+00_rb,0.59270e+00_rb,0.60473e+00_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.63185e+00_rb,0.61324e+00_rb,0.65237e+00_rb,0.67473e+00_rb,0.67959e+00_rb, &
        & 0.66215e+00_rb,0.61741e+00_rb,0.59134e+00_rb,0.60361e+00_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.62122e+00_rb,0.60440e+00_rb,0.64660e+00_rb,0.67096e+00_rb,0.67570e+00_rb, &
        & 0.66004e+00_rb,0.61834e+00_rb,0.58886e+00_rb,0.60079e+00_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.61111e+00_rb,0.59616e+00_rb,0.64034e+00_rb,0.66675e+00_rb,0.67073e+00_rb, &
        & 0.65786e+00_rb,0.61721e+00_rb,0.58692e+00_rb,0.59978e+00_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.60223e+00_rb,0.58851e+00_rb,0.63401e+00_rb,0.66136e+00_rb,0.66631e+00_rb, &
        & 0.65503e+00_rb,0.61469e+00_rb,0.58697e+00_rb,0.60040e+00_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.59185e+00_rb,0.59370e+00_rb,0.63992e+00_rb,0.66463e+00_rb,0.67279e+00_rb, &
        & 0.65610e+00_rb,0.61852e+00_rb,0.58331e+00_rb,0.58763e+00_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.58158e+00_rb,0.58584e+00_rb,0.63500e+00_rb,0.66123e+00_rb,0.67045e+00_rb, &
        & 0.65504e+00_rb,0.62015e+00_rb,0.58225e+00_rb,0.58623e+00_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.57197e+00_rb,0.57827e+00_rb,0.62932e+00_rb,0.65791e+00_rb,0.66630e+00_rb, &
        & 0.65353e+00_rb,0.61959e+00_rb,0.58025e+00_rb,0.58483e+00_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.56277e+00_rb,0.57099e+00_rb,0.62369e+00_rb,0.65326e+00_rb,0.66229e+00_rb, &
        & 0.65156e+00_rb,0.61745e+00_rb,0.57913e+00_rb,0.58544e+00_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.55581e+00_rb,0.56431e+00_rb,0.61807e+00_rb,0.64784e+00_rb,0.65855e+00_rb, &
        & 0.64845e+00_rb,0.61479e+00_rb,0.57987e+00_rb,0.58646e+00_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.53954e+00_rb,0.56255e+00_rb,0.61290e+00_rb,0.64303e+00_rb,0.64974e+00_rb, &
        & 0.64176e+00_rb,0.61714e+00_rb,0.56940e+00_rb,0.56484e+00_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.53060e+00_rb,0.55651e+00_rb,0.60789e+00_rb,0.63988e+00_rb,0.64820e+00_rb, &
        & 0.64042e+00_rb,0.61834e+00_rb,0.56932e+00_rb,0.56418e+00_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.52216e+00_rb,0.54988e+00_rb,0.60294e+00_rb,0.63619e+00_rb,0.64521e+00_rb, &
        & 0.63878e+00_rb,0.61726e+00_rb,0.56850e+00_rb,0.56474e+00_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.51474e+00_rb,0.54331e+00_rb,0.59815e+00_rb,0.63165e+00_rb,0.64200e+00_rb, &
        & 0.63625e+00_rb,0.61431e+00_rb,0.56860e+00_rb,0.56684e+00_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.51206e+00_rb,0.53704e+00_rb,0.59303e+00_rb,0.62695e+00_rb,0.63853e+00_rb, &
        & 0.63290e+00_rb,0.61169e+00_rb,0.57018e+00_rb,0.56804e+00_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.49102e+00_rb,0.52896e+00_rb,0.58005e+00_rb,0.60940e+00_rb,0.62179e+00_rb, &
        & 0.62452e+00_rb,0.60693e+00_rb,0.55666e+00_rb,0.53830e+00_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.48351e+00_rb,0.52394e+00_rb,0.57551e+00_rb,0.60647e+00_rb,0.62043e+00_rb, &
        & 0.62318e+00_rb,0.60808e+00_rb,0.55693e+00_rb,0.53902e+00_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.47604e+00_rb,0.51848e+00_rb,0.57124e+00_rb,0.60324e+00_rb,0.61835e+00_rb, &
        & 0.62080e+00_rb,0.60698e+00_rb,0.55691e+00_rb,0.54116e+00_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.47121e+00_rb,0.51294e+00_rb,0.56659e+00_rb,0.59937e+00_rb,0.61527e+00_rb, &
        & 0.61807e+00_rb,0.60502e+00_rb,0.55819e+00_rb,0.54361e+00_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.47306e+00_rb,0.50884e+00_rb,0.56199e+00_rb,0.59548e+00_rb,0.61135e+00_rb, &
        & 0.61474e+00_rb,0.60372e+00_rb,0.56024e+00_rb,0.54444e+00_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.44396e+00_rb,0.49532e+00_rb,0.54249e+00_rb,0.57145e+00_rb,0.59186e+00_rb, &
        & 0.60060e+00_rb,0.58785e+00_rb,0.54441e+00_rb,0.50913e+00_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.43707e+00_rb,0.49048e+00_rb,0.53914e+00_rb,0.56868e+00_rb,0.59107e+00_rb, &
        & 0.60049e+00_rb,0.58883e+00_rb,0.54507e+00_rb,0.51101e+00_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.43098e+00_rb,0.48531e+00_rb,0.53541e+00_rb,0.56592e+00_rb,0.58911e+00_rb, &
        & 0.59846e+00_rb,0.58812e+00_rb,0.54578e+00_rb,0.51441e+00_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.42957e+00_rb,0.48093e+00_rb,0.53173e+00_rb,0.56262e+00_rb,0.58636e+00_rb, &
        & 0.59589e+00_rb,0.58708e+00_rb,0.54776e+00_rb,0.51773e+00_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.43487e+00_rb,0.48035e+00_rb,0.52822e+00_rb,0.55903e+00_rb,0.58272e+00_rb, &
        & 0.59353e+00_rb,0.58697e+00_rb,0.54985e+00_rb,0.51918e+00_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.39531e+00_rb,0.46021e+00_rb,0.50510e+00_rb,0.53588e+00_rb,0.55895e+00_rb, &
        & 0.56979e+00_rb,0.56191e+00_rb,0.52659e+00_rb,0.47758e+00_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.38905e+00_rb,0.45517e+00_rb,0.50180e+00_rb,0.53416e+00_rb,0.55846e+00_rb, &
        & 0.57047e+00_rb,0.56355e+00_rb,0.52717e+00_rb,0.48088e+00_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.38493e+00_rb,0.45067e+00_rb,0.49854e+00_rb,0.53220e+00_rb,0.55699e+00_rb, &
        & 0.56934e+00_rb,0.56332e+00_rb,0.52868e+00_rb,0.48536e+00_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.38723e+00_rb,0.44786e+00_rb,0.49546e+00_rb,0.52928e+00_rb,0.55471e+00_rb, &
        & 0.56730e+00_rb,0.56372e+00_rb,0.53178e+00_rb,0.48902e+00_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.39433e+00_rb,0.45054e+00_rb,0.49350e+00_rb,0.52621e+00_rb,0.55192e+00_rb, &
        & 0.56604e+00_rb,0.56481e+00_rb,0.53491e+00_rb,0.49067e+00_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.34716e+00_rb,0.41992e+00_rb,0.46841e+00_rb,0.50264e+00_rb,0.52377e+00_rb, &
        & 0.53462e+00_rb,0.53096e+00_rb,0.50215e+00_rb,0.44481e+00_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.34195e+00_rb,0.41601e+00_rb,0.46506e+00_rb,0.50165e+00_rb,0.52385e+00_rb, &
        & 0.53574e+00_rb,0.53310e+00_rb,0.50401e+00_rb,0.44924e+00_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.34072e+00_rb,0.41255e+00_rb,0.46265e+00_rb,0.50007e+00_rb,0.52273e+00_rb, &
        & 0.53553e+00_rb,0.53434e+00_rb,0.50668e+00_rb,0.45414e+00_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.34572e+00_rb,0.41258e+00_rb,0.46029e+00_rb,0.49770e+00_rb,0.52100e+00_rb, &
        & 0.53482e+00_rb,0.53633e+00_rb,0.51056e+00_rb,0.45768e+00_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.35242e+00_rb,0.41826e+00_rb,0.46028e+00_rb,0.49520e+00_rb,0.51927e+00_rb, &
        & 0.53500e+00_rb,0.53892e+00_rb,0.51405e+00_rb,0.45947e+00_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.30160e+00_rb,0.37714e+00_rb,0.42971e+00_rb,0.46717e+00_rb,0.48926e+00_rb, &
        & 0.49871e+00_rb,0.49756e+00_rb,0.47436e+00_rb,0.41261e+00_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.29962e+00_rb,0.37395e+00_rb,0.42779e+00_rb,0.46654e+00_rb,0.48929e+00_rb, &
        & 0.49998e+00_rb,0.50016e+00_rb,0.47769e+00_rb,0.41819e+00_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.30371e+00_rb,0.37339e+00_rb,0.42611e+00_rb,0.46502e+00_rb,0.48821e+00_rb, &
        & 0.50080e+00_rb,0.50323e+00_rb,0.48215e+00_rb,0.42266e+00_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.30955e+00_rb,0.37857e+00_rb,0.42639e+00_rb,0.46320e+00_rb,0.48745e+00_rb, &
        & 0.50213e+00_rb,0.50694e+00_rb,0.48639e+00_rb,0.42570e+00_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.31511e+00_rb,0.38606e+00_rb,0.43107e+00_rb,0.46346e+00_rb,0.48731e+00_rb, &
        & 0.50398e+00_rb,0.50975e+00_rb,0.48945e+00_rb,0.42790e+00_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.26311e+00_rb,0.33847e+00_rb,0.39159e+00_rb,0.42993e+00_rb,0.45370e+00_rb, &
        & 0.46372e+00_rb,0.46279e+00_rb,0.44427e+00_rb,0.37907e+00_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.26583e+00_rb,0.33679e+00_rb,0.39073e+00_rb,0.42938e+00_rb,0.45415e+00_rb, &
        & 0.46569e+00_rb,0.46639e+00_rb,0.44945e+00_rb,0.38485e+00_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.27082e+00_rb,0.34071e+00_rb,0.39077e+00_rb,0.42842e+00_rb,0.45412e+00_rb, &
        & 0.46820e+00_rb,0.47089e+00_rb,0.45477e+00_rb,0.38900e+00_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.27574e+00_rb,0.34733e+00_rb,0.39532e+00_rb,0.42923e+00_rb,0.45504e+00_rb, &
        & 0.47097e+00_rb,0.47486e+00_rb,0.45848e+00_rb,0.39247e+00_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.28099e+00_rb,0.35386e+00_rb,0.40190e+00_rb,0.43447e+00_rb,0.45757e+00_rb, &
        & 0.47294e+00_rb,0.47760e+00_rb,0.46125e+00_rb,0.39629e+00_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.23133e+00_rb,0.30448e+00_rb,0.35592e+00_rb,0.39245e+00_rb,0.41703e+00_rb, &
        & 0.42944e+00_rb,0.42838e+00_rb,0.41191e+00_rb,0.34405e+00_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.23569e+00_rb,0.30675e+00_rb,0.35574e+00_rb,0.39238e+00_rb,0.41854e+00_rb, &
        & 0.43261e+00_rb,0.43319e+00_rb,0.41851e+00_rb,0.34979e+00_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.23995e+00_rb,0.31287e+00_rb,0.35958e+00_rb,0.39339e+00_rb,0.42025e+00_rb, &
        & 0.43649e+00_rb,0.43814e+00_rb,0.42387e+00_rb,0.35427e+00_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.24491e+00_rb,0.31868e+00_rb,0.36597e+00_rb,0.39887e+00_rb,0.42348e+00_rb, &
        & 0.43960e+00_rb,0.44180e+00_rb,0.42781e+00_rb,0.35946e+00_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.25204e+00_rb,0.32443e+00_rb,0.37244e+00_rb,0.40709e+00_rb,0.42924e+00_rb, &
        & 0.44176e+00_rb,0.44465e+00_rb,0.43196e+00_rb,0.36481e+00_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.10293e+01_rb,0.90445e+00_rb,0.86137e+00_rb,0.83022e+00_rb,0.84086e+00_rb, &
        & 0.86548e+00_rb,0.85587e+00_rb,0.85325e+00_rb,0.97261e+00_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.10191e+01_rb,0.89556e+00_rb,0.85472e+00_rb,0.82839e+00_rb,0.83827e+00_rb, &
        & 0.86712e+00_rb,0.85626e+00_rb,0.85096e+00_rb,0.97088e+00_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.10070e+01_rb,0.88513e+00_rb,0.84750e+00_rb,0.82570e+00_rb,0.83354e+00_rb, &
        & 0.86668e+00_rb,0.85692e+00_rb,0.84837e+00_rb,0.96710e+00_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.99477e+00_rb,0.87449e+00_rb,0.83958e+00_rb,0.82146e+00_rb,0.82897e+00_rb, &
        & 0.86436e+00_rb,0.85744e+00_rb,0.84703e+00_rb,0.96450e+00_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.98126e+00_rb,0.86272e+00_rb,0.83128e+00_rb,0.81555e+00_rb,0.82401e+00_rb, &
        & 0.85929e+00_rb,0.85825e+00_rb,0.84707e+00_rb,0.96358e+00_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.10475e+01_rb,0.93151e+00_rb,0.89830e+00_rb,0.88420e+00_rb,0.87933e+00_rb, &
        & 0.87846e+00_rb,0.87987e+00_rb,0.86086e+00_rb,0.97265e+00_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.10370e+01_rb,0.92196e+00_rb,0.89215e+00_rb,0.88284e+00_rb,0.87783e+00_rb, &
        & 0.88073e+00_rb,0.88059e+00_rb,0.85882e+00_rb,0.96972e+00_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.10244e+01_rb,0.91082e+00_rb,0.88511e+00_rb,0.87976e+00_rb,0.87455e+00_rb, &
        & 0.88046e+00_rb,0.88070e+00_rb,0.85623e+00_rb,0.96599e+00_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.10111e+01_rb,0.89911e+00_rb,0.87761e+00_rb,0.87481e+00_rb,0.87132e+00_rb, &
        & 0.87692e+00_rb,0.88074e+00_rb,0.85652e+00_rb,0.96500e+00_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.99734e+00_rb,0.88696e+00_rb,0.86940e+00_rb,0.86908e+00_rb,0.86706e+00_rb, &
        & 0.87191e+00_rb,0.88156e+00_rb,0.85742e+00_rb,0.96567e+00_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.10481e+01_rb,0.94949e+00_rb,0.92754e+00_rb,0.93134e+00_rb,0.92020e+00_rb, &
        & 0.88963e+00_rb,0.89353e+00_rb,0.87368e+00_rb,0.96341e+00_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.10364e+01_rb,0.93961e+00_rb,0.92141e+00_rb,0.92962e+00_rb,0.91951e+00_rb, &
        & 0.89266e+00_rb,0.89347e+00_rb,0.87218e+00_rb,0.95995e+00_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.10237e+01_rb,0.92877e+00_rb,0.91434e+00_rb,0.92574e+00_rb,0.91740e+00_rb, &
        & 0.89302e+00_rb,0.89281e+00_rb,0.87167e+00_rb,0.95907e+00_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.10107e+01_rb,0.91711e+00_rb,0.90716e+00_rb,0.92023e+00_rb,0.91517e+00_rb, &
        & 0.89037e+00_rb,0.89222e+00_rb,0.87266e+00_rb,0.95928e+00_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.99633e+00_rb,0.90470e+00_rb,0.89927e+00_rb,0.91439e+00_rb,0.91120e+00_rb, &
        & 0.88600e+00_rb,0.89266e+00_rb,0.87273e+00_rb,0.95886e+00_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.10264e+01_rb,0.95071e+00_rb,0.94674e+00_rb,0.96322e+00_rb,0.95099e+00_rb, &
        & 0.91374e+00_rb,0.89446e+00_rb,0.87855e+00_rb,0.93628e+00_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.10149e+01_rb,0.94036e+00_rb,0.94196e+00_rb,0.96089e+00_rb,0.95153e+00_rb, &
        & 0.91752e+00_rb,0.89401e+00_rb,0.87842e+00_rb,0.93496e+00_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.10022e+01_rb,0.92957e+00_rb,0.93566e+00_rb,0.95688e+00_rb,0.95066e+00_rb, &
        & 0.91784e+00_rb,0.89284e+00_rb,0.87926e+00_rb,0.93586e+00_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.98847e+00_rb,0.91773e+00_rb,0.92854e+00_rb,0.95134e+00_rb,0.94862e+00_rb, &
        & 0.91547e+00_rb,0.89366e+00_rb,0.87951e+00_rb,0.93591e+00_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.97390e+00_rb,0.90550e+00_rb,0.92045e+00_rb,0.94604e+00_rb,0.94431e+00_rb, &
        & 0.91218e+00_rb,0.89395e+00_rb,0.87858e+00_rb,0.93476e+00_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.98609e+00_rb,0.93424e+00_rb,0.95643e+00_rb,0.97684e+00_rb,0.96999e+00_rb, &
        & 0.93774e+00_rb,0.88693e+00_rb,0.86915e+00_rb,0.90145e+00_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.97412e+00_rb,0.92401e+00_rb,0.95137e+00_rb,0.97507e+00_rb,0.97005e+00_rb, &
        & 0.94299e+00_rb,0.88698e+00_rb,0.86947e+00_rb,0.90218e+00_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.96114e+00_rb,0.91242e+00_rb,0.94531e+00_rb,0.97096e+00_rb,0.96958e+00_rb, &
        & 0.94325e+00_rb,0.88812e+00_rb,0.86965e+00_rb,0.90220e+00_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.94742e+00_rb,0.90067e+00_rb,0.93796e+00_rb,0.96622e+00_rb,0.96802e+00_rb, &
        & 0.94082e+00_rb,0.89013e+00_rb,0.86974e+00_rb,0.90156e+00_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.93336e+00_rb,0.88866e+00_rb,0.92975e+00_rb,0.96102e+00_rb,0.96365e+00_rb, &
        & 0.93814e+00_rb,0.89133e+00_rb,0.86813e+00_rb,0.89955e+00_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.92984e+00_rb,0.90494e+00_rb,0.94972e+00_rb,0.97425e+00_rb,0.97662e+00_rb, &
        & 0.94556e+00_rb,0.88351e+00_rb,0.85584e+00_rb,0.87463e+00_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.91725e+00_rb,0.89370e+00_rb,0.94535e+00_rb,0.97222e+00_rb,0.97680e+00_rb, &
        & 0.95110e+00_rb,0.88518e+00_rb,0.85597e+00_rb,0.87435e+00_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.90427e+00_rb,0.88250e+00_rb,0.93903e+00_rb,0.96844e+00_rb,0.97606e+00_rb, &
        & 0.95304e+00_rb,0.88753e+00_rb,0.85497e+00_rb,0.87307e+00_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.89063e+00_rb,0.87150e+00_rb,0.93110e+00_rb,0.96495e+00_rb,0.97386e+00_rb, &
        & 0.95208e+00_rb,0.89056e+00_rb,0.85380e+00_rb,0.87067e+00_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.87675e+00_rb,0.86027e+00_rb,0.92261e+00_rb,0.96002e+00_rb,0.96930e+00_rb, &
        & 0.94959e+00_rb,0.89201e+00_rb,0.85155e+00_rb,0.86905e+00_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.85919e+00_rb,0.86562e+00_rb,0.92649e+00_rb,0.95965e+00_rb,0.96523e+00_rb, &
        & 0.93650e+00_rb,0.88529e+00_rb,0.84057e+00_rb,0.84722e+00_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.84717e+00_rb,0.85599e+00_rb,0.92166e+00_rb,0.95676e+00_rb,0.96677e+00_rb, &
        & 0.94156e+00_rb,0.88807e+00_rb,0.84087e+00_rb,0.84785e+00_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.83473e+00_rb,0.84556e+00_rb,0.91525e+00_rb,0.95342e+00_rb,0.96579e+00_rb, &
        & 0.94506e+00_rb,0.89084e+00_rb,0.83990e+00_rb,0.84680e+00_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.82211e+00_rb,0.83508e+00_rb,0.90782e+00_rb,0.94944e+00_rb,0.96296e+00_rb, &
        & 0.94530e+00_rb,0.89295e+00_rb,0.83822e+00_rb,0.84533e+00_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.80945e+00_rb,0.82542e+00_rb,0.90011e+00_rb,0.94513e+00_rb,0.95957e+00_rb, &
        & 0.94356e+00_rb,0.89326e+00_rb,0.83636e+00_rb,0.84582e+00_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.78535e+00_rb,0.81958e+00_rb,0.89069e+00_rb,0.92943e+00_rb,0.93702e+00_rb, &
        & 0.91895e+00_rb,0.88182e+00_rb,0.81942e+00_rb,0.81355e+00_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.77455e+00_rb,0.81120e+00_rb,0.88449e+00_rb,0.92737e+00_rb,0.93847e+00_rb, &
        & 0.92275e+00_rb,0.88568e+00_rb,0.82187e+00_rb,0.81529e+00_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.76320e+00_rb,0.80325e+00_rb,0.87799e+00_rb,0.92413e+00_rb,0.93785e+00_rb, &
        & 0.92584e+00_rb,0.88848e+00_rb,0.82246e+00_rb,0.81526e+00_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.75209e+00_rb,0.79461e+00_rb,0.87173e+00_rb,0.92037e+00_rb,0.93562e+00_rb, &
        & 0.92659e+00_rb,0.89042e+00_rb,0.82203e+00_rb,0.81583e+00_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.74191e+00_rb,0.78569e+00_rb,0.86500e+00_rb,0.91565e+00_rb,0.93275e+00_rb, &
        & 0.92508e+00_rb,0.89021e+00_rb,0.82215e+00_rb,0.81833e+00_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.71539e+00_rb,0.77045e+00_rb,0.84442e+00_rb,0.88339e+00_rb,0.89968e+00_rb, &
        & 0.89536e+00_rb,0.86841e+00_rb,0.79955e+00_rb,0.77318e+00_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.70564e+00_rb,0.76424e+00_rb,0.83867e+00_rb,0.88222e+00_rb,0.90019e+00_rb, &
        & 0.89929e+00_rb,0.87308e+00_rb,0.80390e+00_rb,0.77664e+00_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.69566e+00_rb,0.75811e+00_rb,0.83280e+00_rb,0.87940e+00_rb,0.90026e+00_rb, &
        & 0.90222e+00_rb,0.87663e+00_rb,0.80593e+00_rb,0.77834e+00_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.68611e+00_rb,0.75067e+00_rb,0.82727e+00_rb,0.87623e+00_rb,0.89845e+00_rb, &
        & 0.90248e+00_rb,0.87886e+00_rb,0.80698e+00_rb,0.78085e+00_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.67924e+00_rb,0.74336e+00_rb,0.82173e+00_rb,0.87192e+00_rb,0.89575e+00_rb, &
        & 0.90118e+00_rb,0.87929e+00_rb,0.80840e+00_rb,0.78556e+00_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.64750e+00_rb,0.72291e+00_rb,0.79049e+00_rb,0.83093e+00_rb,0.85723e+00_rb, &
        & 0.86420e+00_rb,0.84410e+00_rb,0.78062e+00_rb,0.72995e+00_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.63786e+00_rb,0.71701e+00_rb,0.78689e+00_rb,0.82920e+00_rb,0.85838e+00_rb, &
        & 0.86903e+00_rb,0.84994e+00_rb,0.78581e+00_rb,0.73484e+00_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.62879e+00_rb,0.71055e+00_rb,0.78212e+00_rb,0.82630e+00_rb,0.85919e+00_rb, &
        & 0.87234e+00_rb,0.85358e+00_rb,0.78925e+00_rb,0.73873e+00_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.62087e+00_rb,0.70389e+00_rb,0.77761e+00_rb,0.82356e+00_rb,0.85812e+00_rb, &
        & 0.87281e+00_rb,0.85602e+00_rb,0.79174e+00_rb,0.74394e+00_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.61947e+00_rb,0.69759e+00_rb,0.77360e+00_rb,0.82013e+00_rb,0.85542e+00_rb, &
        & 0.87157e+00_rb,0.85709e+00_rb,0.79529e+00_rb,0.75052e+00_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.57317e+00_rb,0.67082e+00_rb,0.73559e+00_rb,0.77934e+00_rb,0.81170e+00_rb, &
        & 0.82582e+00_rb,0.81431e+00_rb,0.75887e+00_rb,0.68710e+00_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.56447e+00_rb,0.66437e+00_rb,0.73216e+00_rb,0.77740e+00_rb,0.81370e+00_rb, &
        & 0.83183e+00_rb,0.81926e+00_rb,0.76438e+00_rb,0.69316e+00_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.55710e+00_rb,0.65798e+00_rb,0.72792e+00_rb,0.77609e+00_rb,0.81480e+00_rb, &
        & 0.83377e+00_rb,0.82304e+00_rb,0.76833e+00_rb,0.70018e+00_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.55630e+00_rb,0.65257e+00_rb,0.72437e+00_rb,0.77329e+00_rb,0.81397e+00_rb, &
        & 0.83437e+00_rb,0.82566e+00_rb,0.77320e+00_rb,0.70886e+00_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.56411e+00_rb,0.65194e+00_rb,0.72119e+00_rb,0.77017e+00_rb,0.81118e+00_rb, &
        & 0.83405e+00_rb,0.82785e+00_rb,0.77913e+00_rb,0.71658e+00_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.50093e+00_rb,0.61265e+00_rb,0.68222e+00_rb,0.73081e+00_rb,0.76376e+00_rb, &
        & 0.78196e+00_rb,0.77611e+00_rb,0.73046e+00_rb,0.64374e+00_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.49374e+00_rb,0.60693e+00_rb,0.67860e+00_rb,0.73087e+00_rb,0.76701e+00_rb, &
        & 0.78657e+00_rb,0.78196e+00_rb,0.73646e+00_rb,0.65183e+00_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.49262e+00_rb,0.60232e+00_rb,0.67517e+00_rb,0.73017e+00_rb,0.76805e+00_rb, &
        & 0.78863e+00_rb,0.78651e+00_rb,0.74261e+00_rb,0.66205e+00_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.49989e+00_rb,0.60234e+00_rb,0.67259e+00_rb,0.72790e+00_rb,0.76753e+00_rb, &
        & 0.79017e+00_rb,0.79054e+00_rb,0.75027e+00_rb,0.67174e+00_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.51177e+00_rb,0.60963e+00_rb,0.67313e+00_rb,0.72489e+00_rb,0.76606e+00_rb, &
        & 0.79187e+00_rb,0.79524e+00_rb,0.75809e+00_rb,0.67955e+00_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.43610e+00_rb,0.55280e+00_rb,0.62805e+00_rb,0.68236e+00_rb,0.71726e+00_rb, &
        & 0.73439e+00_rb,0.73316e+00_rb,0.69581e+00_rb,0.60098e+00_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.43332e+00_rb,0.54879e+00_rb,0.62602e+00_rb,0.68387e+00_rb,0.71981e+00_rb, &
        & 0.73863e+00_rb,0.74004e+00_rb,0.70395e+00_rb,0.61188e+00_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.43938e+00_rb,0.54819e+00_rb,0.62420e+00_rb,0.68392e+00_rb,0.72120e+00_rb, &
        & 0.74159e+00_rb,0.74634e+00_rb,0.71286e+00_rb,0.62343e+00_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.44997e+00_rb,0.55515e+00_rb,0.62479e+00_rb,0.68255e+00_rb,0.72173e+00_rb, &
        & 0.74510e+00_rb,0.75297e+00_rb,0.72278e+00_rb,0.63265e+00_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.46055e+00_rb,0.56787e+00_rb,0.63110e+00_rb,0.68248e+00_rb,0.72225e+00_rb, &
        & 0.74921e+00_rb,0.75987e+00_rb,0.73098e+00_rb,0.64026e+00_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.14171e+01_rb,0.12412e+01_rb,0.11453e+01_rb,0.11122e+01_rb,0.11253e+01_rb, &
        & 0.11573e+01_rb,0.11791e+01_rb,0.12973e+01_rb,0.14638e+01_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.14090e+01_rb,0.12342e+01_rb,0.11388e+01_rb,0.11095e+01_rb,0.11322e+01_rb, &
        & 0.11607e+01_rb,0.11781e+01_rb,0.12938e+01_rb,0.14611e+01_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.13994e+01_rb,0.12258e+01_rb,0.11299e+01_rb,0.11046e+01_rb,0.11339e+01_rb, &
        & 0.11641e+01_rb,0.11762e+01_rb,0.12907e+01_rb,0.14569e+01_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.13872e+01_rb,0.12151e+01_rb,0.11205e+01_rb,0.10983e+01_rb,0.11321e+01_rb, &
        & 0.11655e+01_rb,0.11746e+01_rb,0.12884e+01_rb,0.14537e+01_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.13737e+01_rb,0.12034e+01_rb,0.11101e+01_rb,0.10915e+01_rb,0.11280e+01_rb, &
        & 0.11664e+01_rb,0.11724e+01_rb,0.12834e+01_rb,0.14491e+01_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.15051e+01_rb,0.13181e+01_rb,0.12304e+01_rb,0.11719e+01_rb,0.11855e+01_rb, &
        & 0.12309e+01_rb,0.12496e+01_rb,0.13203e+01_rb,0.15049e+01_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.14984e+01_rb,0.13123e+01_rb,0.12250e+01_rb,0.11699e+01_rb,0.11925e+01_rb, &
        & 0.12340e+01_rb,0.12487e+01_rb,0.13193e+01_rb,0.15024e+01_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.14884e+01_rb,0.13036e+01_rb,0.12176e+01_rb,0.11665e+01_rb,0.11942e+01_rb, &
        & 0.12376e+01_rb,0.12478e+01_rb,0.13196e+01_rb,0.15020e+01_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.14762e+01_rb,0.12929e+01_rb,0.12082e+01_rb,0.11628e+01_rb,0.11911e+01_rb, &
        & 0.12406e+01_rb,0.12476e+01_rb,0.13168e+01_rb,0.15001e+01_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.14614e+01_rb,0.12800e+01_rb,0.11975e+01_rb,0.11575e+01_rb,0.11857e+01_rb, &
        & 0.12414e+01_rb,0.12460e+01_rb,0.13124e+01_rb,0.14945e+01_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.15822e+01_rb,0.13890e+01_rb,0.13182e+01_rb,0.12547e+01_rb,0.12476e+01_rb, &
        & 0.12873e+01_rb,0.13056e+01_rb,0.13300e+01_rb,0.15181e+01_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.15752e+01_rb,0.13826e+01_rb,0.13120e+01_rb,0.12557e+01_rb,0.12556e+01_rb, &
        & 0.12906e+01_rb,0.13067e+01_rb,0.13310e+01_rb,0.15193e+01_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.15647e+01_rb,0.13732e+01_rb,0.13046e+01_rb,0.12538e+01_rb,0.12575e+01_rb, &
        & 0.12941e+01_rb,0.13084e+01_rb,0.13307e+01_rb,0.15193e+01_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.15514e+01_rb,0.13616e+01_rb,0.12950e+01_rb,0.12500e+01_rb,0.12556e+01_rb, &
        & 0.12989e+01_rb,0.13088e+01_rb,0.13292e+01_rb,0.15174e+01_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.15358e+01_rb,0.13480e+01_rb,0.12843e+01_rb,0.12443e+01_rb,0.12517e+01_rb, &
        & 0.13009e+01_rb,0.13074e+01_rb,0.13265e+01_rb,0.15142e+01_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.16350e+01_rb,0.14490e+01_rb,0.13916e+01_rb,0.13443e+01_rb,0.13174e+01_rb, &
        & 0.13244e+01_rb,0.13476e+01_rb,0.13365e+01_rb,0.15180e+01_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.16281e+01_rb,0.14425e+01_rb,0.13866e+01_rb,0.13479e+01_rb,0.13260e+01_rb, &
        & 0.13268e+01_rb,0.13503e+01_rb,0.13390e+01_rb,0.15205e+01_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.16161e+01_rb,0.14310e+01_rb,0.13789e+01_rb,0.13476e+01_rb,0.13293e+01_rb, &
        & 0.13314e+01_rb,0.13532e+01_rb,0.13408e+01_rb,0.15220e+01_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.16014e+01_rb,0.14178e+01_rb,0.13691e+01_rb,0.13444e+01_rb,0.13282e+01_rb, &
        & 0.13379e+01_rb,0.13534e+01_rb,0.13410e+01_rb,0.15212e+01_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.15848e+01_rb,0.14026e+01_rb,0.13577e+01_rb,0.13378e+01_rb,0.13257e+01_rb, &
        & 0.13408e+01_rb,0.13534e+01_rb,0.13387e+01_rb,0.15183e+01_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.16591e+01_rb,0.14947e+01_rb,0.14480e+01_rb,0.14262e+01_rb,0.13903e+01_rb, &
        & 0.13587e+01_rb,0.13777e+01_rb,0.13491e+01_rb,0.15046e+01_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.16509e+01_rb,0.14862e+01_rb,0.14440e+01_rb,0.14316e+01_rb,0.13991e+01_rb, &
        & 0.13598e+01_rb,0.13805e+01_rb,0.13541e+01_rb,0.15083e+01_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.16386e+01_rb,0.14750e+01_rb,0.14364e+01_rb,0.14307e+01_rb,0.14045e+01_rb, &
        & 0.13655e+01_rb,0.13826e+01_rb,0.13579e+01_rb,0.15118e+01_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.16229e+01_rb,0.14605e+01_rb,0.14277e+01_rb,0.14269e+01_rb,0.14036e+01_rb, &
        & 0.13722e+01_rb,0.13840e+01_rb,0.13579e+01_rb,0.15115e+01_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.16043e+01_rb,0.14439e+01_rb,0.14169e+01_rb,0.14203e+01_rb,0.14024e+01_rb, &
        & 0.13765e+01_rb,0.13831e+01_rb,0.13575e+01_rb,0.15115e+01_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.16482e+01_rb,0.15147e+01_rb,0.14918e+01_rb,0.14883e+01_rb,0.14555e+01_rb, &
        & 0.14030e+01_rb,0.13913e+01_rb,0.13585e+01_rb,0.14685e+01_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.16385e+01_rb,0.15059e+01_rb,0.14880e+01_rb,0.14958e+01_rb,0.14629e+01_rb, &
        & 0.14056e+01_rb,0.13940e+01_rb,0.13659e+01_rb,0.14762e+01_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.16253e+01_rb,0.14941e+01_rb,0.14803e+01_rb,0.14965e+01_rb,0.14689e+01_rb, &
        & 0.14106e+01_rb,0.13970e+01_rb,0.13717e+01_rb,0.14817e+01_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.16089e+01_rb,0.14791e+01_rb,0.14713e+01_rb,0.14908e+01_rb,0.14702e+01_rb, &
        & 0.14165e+01_rb,0.13983e+01_rb,0.13748e+01_rb,0.14851e+01_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.15910e+01_rb,0.14633e+01_rb,0.14618e+01_rb,0.14838e+01_rb,0.14693e+01_rb, &
        & 0.14213e+01_rb,0.13976e+01_rb,0.13770e+01_rb,0.14876e+01_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.16064e+01_rb,0.15086e+01_rb,0.15144e+01_rb,0.15268e+01_rb,0.15040e+01_rb, &
        & 0.14507e+01_rb,0.13904e+01_rb,0.13521e+01_rb,0.14179e+01_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.15953e+01_rb,0.14985e+01_rb,0.15140e+01_rb,0.15345e+01_rb,0.15085e+01_rb, &
        & 0.14549e+01_rb,0.13952e+01_rb,0.13604e+01_rb,0.14259e+01_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.15801e+01_rb,0.14851e+01_rb,0.15073e+01_rb,0.15363e+01_rb,0.15134e+01_rb, &
        & 0.14598e+01_rb,0.13998e+01_rb,0.13678e+01_rb,0.14328e+01_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.15629e+01_rb,0.14703e+01_rb,0.14995e+01_rb,0.15318e+01_rb,0.15172e+01_rb, &
        & 0.14656e+01_rb,0.14020e+01_rb,0.13732e+01_rb,0.14388e+01_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.15443e+01_rb,0.14531e+01_rb,0.14892e+01_rb,0.15236e+01_rb,0.15169e+01_rb, &
        & 0.14692e+01_rb,0.14038e+01_rb,0.13766e+01_rb,0.14425e+01_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.15344e+01_rb,0.14766e+01_rb,0.15157e+01_rb,0.15423e+01_rb,0.15330e+01_rb, &
        & 0.14811e+01_rb,0.13889e+01_rb,0.13373e+01_rb,0.13698e+01_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.15222e+01_rb,0.14679e+01_rb,0.15175e+01_rb,0.15473e+01_rb,0.15363e+01_rb, &
        & 0.14872e+01_rb,0.13948e+01_rb,0.13442e+01_rb,0.13786e+01_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.15059e+01_rb,0.14533e+01_rb,0.15144e+01_rb,0.15489e+01_rb,0.15409e+01_rb, &
        & 0.14932e+01_rb,0.14005e+01_rb,0.13518e+01_rb,0.13861e+01_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.14877e+01_rb,0.14368e+01_rb,0.15058e+01_rb,0.15451e+01_rb,0.15445e+01_rb, &
        & 0.14989e+01_rb,0.14044e+01_rb,0.13574e+01_rb,0.13922e+01_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.14686e+01_rb,0.14200e+01_rb,0.14953e+01_rb,0.15388e+01_rb,0.15447e+01_rb, &
        & 0.15024e+01_rb,0.14082e+01_rb,0.13605e+01_rb,0.13964e+01_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.14373e+01_rb,0.14266e+01_rb,0.14938e+01_rb,0.15367e+01_rb,0.15341e+01_rb, &
        & 0.14871e+01_rb,0.13939e+01_rb,0.13173e+01_rb,0.13285e+01_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.14250e+01_rb,0.14168e+01_rb,0.14968e+01_rb,0.15390e+01_rb,0.15386e+01_rb, &
        & 0.14934e+01_rb,0.13987e+01_rb,0.13260e+01_rb,0.13388e+01_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.14085e+01_rb,0.14020e+01_rb,0.14946e+01_rb,0.15393e+01_rb,0.15430e+01_rb, &
        & 0.14996e+01_rb,0.14050e+01_rb,0.13340e+01_rb,0.13471e+01_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.13898e+01_rb,0.13862e+01_rb,0.14874e+01_rb,0.15365e+01_rb,0.15469e+01_rb, &
        & 0.15061e+01_rb,0.14095e+01_rb,0.13387e+01_rb,0.13529e+01_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.13711e+01_rb,0.13694e+01_rb,0.14772e+01_rb,0.15321e+01_rb,0.15481e+01_rb, &
        & 0.15103e+01_rb,0.14136e+01_rb,0.13413e+01_rb,0.13557e+01_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.13234e+01_rb,0.13585e+01_rb,0.14531e+01_rb,0.15073e+01_rb,0.15094e+01_rb, &
        & 0.14728e+01_rb,0.13972e+01_rb,0.12936e+01_rb,0.12835e+01_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.13109e+01_rb,0.13494e+01_rb,0.14540e+01_rb,0.15085e+01_rb,0.15139e+01_rb, &
        & 0.14784e+01_rb,0.14019e+01_rb,0.13047e+01_rb,0.12968e+01_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.12956e+01_rb,0.13371e+01_rb,0.14504e+01_rb,0.15092e+01_rb,0.15192e+01_rb, &
        & 0.14835e+01_rb,0.14094e+01_rb,0.13136e+01_rb,0.13076e+01_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.12784e+01_rb,0.13237e+01_rb,0.14435e+01_rb,0.15071e+01_rb,0.15234e+01_rb, &
        & 0.14905e+01_rb,0.14145e+01_rb,0.13190e+01_rb,0.13139e+01_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.12607e+01_rb,0.13090e+01_rb,0.14344e+01_rb,0.15038e+01_rb,0.15261e+01_rb, &
        & 0.14956e+01_rb,0.14181e+01_rb,0.13210e+01_rb,0.13178e+01_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.12047e+01_rb,0.12789e+01_rb,0.13923e+01_rb,0.14530e+01_rb,0.14659e+01_rb, &
        & 0.14483e+01_rb,0.13898e+01_rb,0.12749e+01_rb,0.12386e+01_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.11911e+01_rb,0.12701e+01_rb,0.13902e+01_rb,0.14549e+01_rb,0.14710e+01_rb, &
        & 0.14517e+01_rb,0.13981e+01_rb,0.12874e+01_rb,0.12546e+01_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.11768e+01_rb,0.12599e+01_rb,0.13852e+01_rb,0.14552e+01_rb,0.14760e+01_rb, &
        & 0.14581e+01_rb,0.14056e+01_rb,0.12968e+01_rb,0.12644e+01_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.11609e+01_rb,0.12491e+01_rb,0.13785e+01_rb,0.14542e+01_rb,0.14794e+01_rb, &
        & 0.14643e+01_rb,0.14106e+01_rb,0.13015e+01_rb,0.12707e+01_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.11453e+01_rb,0.12370e+01_rb,0.13722e+01_rb,0.14526e+01_rb,0.14820e+01_rb, &
        & 0.14680e+01_rb,0.14139e+01_rb,0.13041e+01_rb,0.12775e+01_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.10892e+01_rb,0.11978e+01_rb,0.13175e+01_rb,0.13800e+01_rb,0.14113e+01_rb, &
        & 0.14104e+01_rb,0.13697e+01_rb,0.12563e+01_rb,0.11895e+01_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.10764e+01_rb,0.11912e+01_rb,0.13142e+01_rb,0.13828e+01_rb,0.14151e+01_rb, &
        & 0.14161e+01_rb,0.13800e+01_rb,0.12704e+01_rb,0.12046e+01_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.10634e+01_rb,0.11835e+01_rb,0.13099e+01_rb,0.13835e+01_rb,0.14196e+01_rb, &
        & 0.14241e+01_rb,0.13887e+01_rb,0.12786e+01_rb,0.12148e+01_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.10495e+01_rb,0.11751e+01_rb,0.13057e+01_rb,0.13846e+01_rb,0.14230e+01_rb, &
        & 0.14300e+01_rb,0.13945e+01_rb,0.12836e+01_rb,0.12244e+01_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.10406e+01_rb,0.11650e+01_rb,0.13011e+01_rb,0.13864e+01_rb,0.14254e+01_rb, &
        & 0.14322e+01_rb,0.13980e+01_rb,0.12878e+01_rb,0.12342e+01_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.97671e+00_rb,0.11194e+01_rb,0.12352e+01_rb,0.13029e+01_rb,0.13468e+01_rb, &
        & 0.13623e+01_rb,0.13355e+01_rb,0.12363e+01_rb,0.11351e+01_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.96514e+00_rb,0.11135e+01_rb,0.12326e+01_rb,0.13049e+01_rb,0.13523e+01_rb, &
        & 0.13721e+01_rb,0.13488e+01_rb,0.12496e+01_rb,0.11500e+01_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.95350e+00_rb,0.11073e+01_rb,0.12308e+01_rb,0.13063e+01_rb,0.13575e+01_rb, &
        & 0.13815e+01_rb,0.13588e+01_rb,0.12584e+01_rb,0.11629e+01_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.94648e+00_rb,0.11002e+01_rb,0.12294e+01_rb,0.13087e+01_rb,0.13617e+01_rb, &
        & 0.13873e+01_rb,0.13660e+01_rb,0.12653e+01_rb,0.11761e+01_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.94953e+00_rb,0.10941e+01_rb,0.12277e+01_rb,0.13116e+01_rb,0.13645e+01_rb, &
        & 0.13901e+01_rb,0.13713e+01_rb,0.12717e+01_rb,0.11896e+01_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.21877e+01_rb,0.19168e+01_rb,0.16830e+01_rb,0.16306e+01_rb,0.15763e+01_rb, &
        & 0.15398e+01_rb,0.15859e+01_rb,0.18468e+01_rb,0.20593e+01_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.22022e+01_rb,0.19296e+01_rb,0.16916e+01_rb,0.16400e+01_rb,0.15796e+01_rb, &
        & 0.15432e+01_rb,0.15912e+01_rb,0.18534e+01_rb,0.20655e+01_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.22022e+01_rb,0.19297e+01_rb,0.16911e+01_rb,0.16436e+01_rb,0.15885e+01_rb, &
        & 0.15469e+01_rb,0.15941e+01_rb,0.18569e+01_rb,0.20695e+01_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.21895e+01_rb,0.19186e+01_rb,0.16803e+01_rb,0.16425e+01_rb,0.15962e+01_rb, &
        & 0.15525e+01_rb,0.15947e+01_rb,0.18575e+01_rb,0.20703e+01_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.21697e+01_rb,0.19013e+01_rb,0.16648e+01_rb,0.16344e+01_rb,0.16003e+01_rb, &
        & 0.15574e+01_rb,0.15941e+01_rb,0.18572e+01_rb,0.20709e+01_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.22550e+01_rb,0.19753e+01_rb,0.17734e+01_rb,0.17443e+01_rb,0.17102e+01_rb, &
        & 0.16986e+01_rb,0.17217e+01_rb,0.19863e+01_rb,0.22329e+01_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.22679e+01_rb,0.19867e+01_rb,0.17791e+01_rb,0.17520e+01_rb,0.17112e+01_rb, &
        & 0.17026e+01_rb,0.17283e+01_rb,0.19934e+01_rb,0.22409e+01_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.22698e+01_rb,0.19883e+01_rb,0.17767e+01_rb,0.17568e+01_rb,0.17174e+01_rb, &
        & 0.17053e+01_rb,0.17318e+01_rb,0.19966e+01_rb,0.22448e+01_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.22597e+01_rb,0.19796e+01_rb,0.17668e+01_rb,0.17548e+01_rb,0.17261e+01_rb, &
        & 0.17073e+01_rb,0.17314e+01_rb,0.19968e+01_rb,0.22451e+01_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.22409e+01_rb,0.19632e+01_rb,0.17511e+01_rb,0.17475e+01_rb,0.17318e+01_rb, &
        & 0.17099e+01_rb,0.17295e+01_rb,0.19947e+01_rb,0.22431e+01_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.23526e+01_rb,0.20603e+01_rb,0.18829e+01_rb,0.18493e+01_rb,0.18420e+01_rb, &
        & 0.18637e+01_rb,0.18729e+01_rb,0.21122e+01_rb,0.23977e+01_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.23692e+01_rb,0.20749e+01_rb,0.18925e+01_rb,0.18515e+01_rb,0.18400e+01_rb, &
        & 0.18683e+01_rb,0.18793e+01_rb,0.21211e+01_rb,0.24079e+01_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.23722e+01_rb,0.20775e+01_rb,0.18924e+01_rb,0.18562e+01_rb,0.18451e+01_rb, &
        & 0.18704e+01_rb,0.18820e+01_rb,0.21249e+01_rb,0.24125e+01_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.23635e+01_rb,0.20699e+01_rb,0.18842e+01_rb,0.18555e+01_rb,0.18524e+01_rb, &
        & 0.18695e+01_rb,0.18827e+01_rb,0.21252e+01_rb,0.24130e+01_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.23480e+01_rb,0.20564e+01_rb,0.18701e+01_rb,0.18499e+01_rb,0.18587e+01_rb, &
        & 0.18689e+01_rb,0.18824e+01_rb,0.21250e+01_rb,0.24123e+01_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.24939e+01_rb,0.21837e+01_rb,0.20232e+01_rb,0.19589e+01_rb,0.19737e+01_rb, &
        & 0.20251e+01_rb,0.20263e+01_rb,0.22145e+01_rb,0.25248e+01_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.25110e+01_rb,0.21986e+01_rb,0.20314e+01_rb,0.19589e+01_rb,0.19697e+01_rb, &
        & 0.20307e+01_rb,0.20352e+01_rb,0.22253e+01_rb,0.25374e+01_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.25161e+01_rb,0.22032e+01_rb,0.20346e+01_rb,0.19625e+01_rb,0.19727e+01_rb, &
        & 0.20334e+01_rb,0.20403e+01_rb,0.22301e+01_rb,0.25429e+01_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.25095e+01_rb,0.21974e+01_rb,0.20283e+01_rb,0.19657e+01_rb,0.19780e+01_rb, &
        & 0.20313e+01_rb,0.20420e+01_rb,0.22330e+01_rb,0.25458e+01_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.24957e+01_rb,0.21853e+01_rb,0.20162e+01_rb,0.19613e+01_rb,0.19825e+01_rb, &
        & 0.20289e+01_rb,0.20423e+01_rb,0.22370e+01_rb,0.25500e+01_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.26637e+01_rb,0.23330e+01_rb,0.21864e+01_rb,0.20904e+01_rb,0.21013e+01_rb, &
        & 0.21699e+01_rb,0.21691e+01_rb,0.22859e+01_rb,0.26102e+01_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.26795e+01_rb,0.23466e+01_rb,0.21930e+01_rb,0.20883e+01_rb,0.21007e+01_rb, &
        & 0.21775e+01_rb,0.21807e+01_rb,0.22957e+01_rb,0.26216e+01_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.26872e+01_rb,0.23531e+01_rb,0.22001e+01_rb,0.20926e+01_rb,0.21029e+01_rb, &
        & 0.21797e+01_rb,0.21879e+01_rb,0.23020e+01_rb,0.26289e+01_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.26803e+01_rb,0.23471e+01_rb,0.21948e+01_rb,0.20970e+01_rb,0.21069e+01_rb, &
        & 0.21770e+01_rb,0.21901e+01_rb,0.23087e+01_rb,0.26364e+01_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.26667e+01_rb,0.23351e+01_rb,0.21832e+01_rb,0.20959e+01_rb,0.21091e+01_rb, &
        & 0.21729e+01_rb,0.21936e+01_rb,0.23158e+01_rb,0.26444e+01_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.28246e+01_rb,0.24844e+01_rb,0.23554e+01_rb,0.22494e+01_rb,0.22325e+01_rb, &
        & 0.22877e+01_rb,0.22925e+01_rb,0.23319e+01_rb,0.26619e+01_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.28376e+01_rb,0.24941e+01_rb,0.23589e+01_rb,0.22458e+01_rb,0.22369e+01_rb, &
        & 0.22959e+01_rb,0.23048e+01_rb,0.23408e+01_rb,0.26720e+01_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.28468e+01_rb,0.25010e+01_rb,0.23646e+01_rb,0.22473e+01_rb,0.22379e+01_rb, &
        & 0.22992e+01_rb,0.23116e+01_rb,0.23486e+01_rb,0.26812e+01_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.28432e+01_rb,0.24967e+01_rb,0.23643e+01_rb,0.22531e+01_rb,0.22409e+01_rb, &
        & 0.22981e+01_rb,0.23157e+01_rb,0.23572e+01_rb,0.26911e+01_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.28266e+01_rb,0.24816e+01_rb,0.23521e+01_rb,0.22549e+01_rb,0.22433e+01_rb, &
        & 0.22953e+01_rb,0.23213e+01_rb,0.23655e+01_rb,0.27010e+01_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.29507e+01_rb,0.26186e+01_rb,0.25154e+01_rb,0.24214e+01_rb,0.23698e+01_rb, &
        & 0.23809e+01_rb,0.23936e+01_rb,0.23723e+01_rb,0.26904e+01_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.29579e+01_rb,0.26218e+01_rb,0.25131e+01_rb,0.24179e+01_rb,0.23791e+01_rb, &
        & 0.23895e+01_rb,0.24036e+01_rb,0.23830e+01_rb,0.27026e+01_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.29645e+01_rb,0.26250e+01_rb,0.25156e+01_rb,0.24180e+01_rb,0.23828e+01_rb, &
        & 0.23929e+01_rb,0.24104e+01_rb,0.23923e+01_rb,0.27131e+01_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.29630e+01_rb,0.26222e+01_rb,0.25142e+01_rb,0.24231e+01_rb,0.23851e+01_rb, &
        & 0.23923e+01_rb,0.24170e+01_rb,0.24017e+01_rb,0.27241e+01_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.29476e+01_rb,0.26071e+01_rb,0.25053e+01_rb,0.24267e+01_rb,0.23855e+01_rb, &
        & 0.23922e+01_rb,0.24228e+01_rb,0.24116e+01_rb,0.27361e+01_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.30283e+01_rb,0.27271e+01_rb,0.26499e+01_rb,0.25812e+01_rb,0.25068e+01_rb, &
        & 0.24674e+01_rb,0.24700e+01_rb,0.24088e+01_rb,0.26873e+01_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.30302e+01_rb,0.27229e+01_rb,0.26429e+01_rb,0.25805e+01_rb,0.25199e+01_rb, &
        & 0.24758e+01_rb,0.24797e+01_rb,0.24223e+01_rb,0.27017e+01_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.30313e+01_rb,0.27198e+01_rb,0.26386e+01_rb,0.25814e+01_rb,0.25257e+01_rb, &
        & 0.24788e+01_rb,0.24874e+01_rb,0.24335e+01_rb,0.27156e+01_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.30296e+01_rb,0.27153e+01_rb,0.26358e+01_rb,0.25865e+01_rb,0.25285e+01_rb, &
        & 0.24798e+01_rb,0.24941e+01_rb,0.24451e+01_rb,0.27296e+01_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.30139e+01_rb,0.26994e+01_rb,0.26290e+01_rb,0.25893e+01_rb,0.25296e+01_rb, &
        & 0.24809e+01_rb,0.25004e+01_rb,0.24568e+01_rb,0.27436e+01_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.30493e+01_rb,0.27941e+01_rb,0.27550e+01_rb,0.27119e+01_rb,0.26405e+01_rb, &
        & 0.25561e+01_rb,0.25144e+01_rb,0.24302e+01_rb,0.26456e+01_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.30457e+01_rb,0.27865e+01_rb,0.27445e+01_rb,0.27145e+01_rb,0.26540e+01_rb, &
        & 0.25649e+01_rb,0.25264e+01_rb,0.24468e+01_rb,0.26636e+01_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.30433e+01_rb,0.27809e+01_rb,0.27367e+01_rb,0.27179e+01_rb,0.26606e+01_rb, &
        & 0.25683e+01_rb,0.25354e+01_rb,0.24613e+01_rb,0.26815e+01_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.30389e+01_rb,0.27734e+01_rb,0.27309e+01_rb,0.27229e+01_rb,0.26627e+01_rb, &
        & 0.25715e+01_rb,0.25441e+01_rb,0.24765e+01_rb,0.27005e+01_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.30246e+01_rb,0.27583e+01_rb,0.27239e+01_rb,0.27244e+01_rb,0.26642e+01_rb, &
        & 0.25740e+01_rb,0.25518e+01_rb,0.24921e+01_rb,0.27199e+01_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.30121e+01_rb,0.28154e+01_rb,0.28201e+01_rb,0.28057e+01_rb,0.27531e+01_rb, &
        & 0.26438e+01_rb,0.25360e+01_rb,0.24395e+01_rb,0.25814e+01_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.30058e+01_rb,0.28055e+01_rb,0.28091e+01_rb,0.28117e+01_rb,0.27659e+01_rb, &
        & 0.26536e+01_rb,0.25505e+01_rb,0.24578e+01_rb,0.26020e+01_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.29984e+01_rb,0.27957e+01_rb,0.28033e+01_rb,0.28172e+01_rb,0.27717e+01_rb, &
        & 0.26597e+01_rb,0.25631e+01_rb,0.24765e+01_rb,0.26239e+01_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.29909e+01_rb,0.27859e+01_rb,0.27980e+01_rb,0.28231e+01_rb,0.27737e+01_rb, &
        & 0.26641e+01_rb,0.25752e+01_rb,0.24953e+01_rb,0.26473e+01_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.29757e+01_rb,0.27703e+01_rb,0.27906e+01_rb,0.28248e+01_rb,0.27751e+01_rb, &
        & 0.26687e+01_rb,0.25862e+01_rb,0.25141e+01_rb,0.26704e+01_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.29105e+01_rb,0.27849e+01_rb,0.28418e+01_rb,0.28641e+01_rb,0.28368e+01_rb, &
        & 0.27205e+01_rb,0.25555e+01_rb,0.24401e+01_rb,0.25196e+01_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.29013e+01_rb,0.27742e+01_rb,0.28363e+01_rb,0.28719e+01_rb,0.28460e+01_rb, &
        & 0.27321e+01_rb,0.25730e+01_rb,0.24607e+01_rb,0.25412e+01_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.28911e+01_rb,0.27633e+01_rb,0.28321e+01_rb,0.28799e+01_rb,0.28506e+01_rb, &
        & 0.27417e+01_rb,0.25892e+01_rb,0.24813e+01_rb,0.25665e+01_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.28805e+01_rb,0.27495e+01_rb,0.28275e+01_rb,0.28852e+01_rb,0.28551e+01_rb, &
        & 0.27503e+01_rb,0.26048e+01_rb,0.25023e+01_rb,0.25905e+01_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.28607e+01_rb,0.27299e+01_rb,0.28179e+01_rb,0.28868e+01_rb,0.28597e+01_rb, &
        & 0.27578e+01_rb,0.26198e+01_rb,0.25207e+01_rb,0.26120e+01_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.27583e+01_rb,0.27145e+01_rb,0.28231e+01_rb,0.28871e+01_rb,0.28764e+01_rb, &
        & 0.27674e+01_rb,0.25743e+01_rb,0.24278e+01_rb,0.24570e+01_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.27477e+01_rb,0.27023e+01_rb,0.28226e+01_rb,0.28962e+01_rb,0.28884e+01_rb, &
        & 0.27832e+01_rb,0.25939e+01_rb,0.24483e+01_rb,0.24817e+01_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.27363e+01_rb,0.26890e+01_rb,0.28229e+01_rb,0.29041e+01_rb,0.28966e+01_rb, &
        & 0.27972e+01_rb,0.26117e+01_rb,0.24706e+01_rb,0.25053e+01_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.27215e+01_rb,0.26719e+01_rb,0.28199e+01_rb,0.29098e+01_rb,0.29044e+01_rb, &
        & 0.28097e+01_rb,0.26305e+01_rb,0.24926e+01_rb,0.25297e+01_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.27019e+01_rb,0.26521e+01_rb,0.28123e+01_rb,0.29106e+01_rb,0.29127e+01_rb, &
        & 0.28234e+01_rb,0.26485e+01_rb,0.25112e+01_rb,0.25512e+01_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.25703e+01_rb,0.26073e+01_rb,0.27736e+01_rb,0.28686e+01_rb,0.28762e+01_rb, &
        & 0.27819e+01_rb,0.25967e+01_rb,0.24069e+01_rb,0.23967e+01_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.25597e+01_rb,0.25968e+01_rb,0.27777e+01_rb,0.28810e+01_rb,0.28904e+01_rb, &
        & 0.28011e+01_rb,0.26158e+01_rb,0.24298e+01_rb,0.24224e+01_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.25492e+01_rb,0.25857e+01_rb,0.27800e+01_rb,0.28929e+01_rb,0.29040e+01_rb, &
        & 0.28202e+01_rb,0.26351e+01_rb,0.24527e+01_rb,0.24494e+01_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.25336e+01_rb,0.25705e+01_rb,0.27801e+01_rb,0.29008e+01_rb,0.29182e+01_rb, &
        & 0.28403e+01_rb,0.26542e+01_rb,0.24758e+01_rb,0.24745e+01_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.25174e+01_rb,0.25538e+01_rb,0.27760e+01_rb,0.29061e+01_rb,0.29321e+01_rb, &
        & 0.28592e+01_rb,0.26728e+01_rb,0.24954e+01_rb,0.24962e+01_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.34915e+01_rb,0.30588e+01_rb,0.26262e+01_rb,0.23144e+01_rb,0.22122e+01_rb, &
        & 0.21892e+01_rb,0.25558e+01_rb,0.29818e+01_rb,0.32991e+01_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.35009e+01_rb,0.30672e+01_rb,0.26335e+01_rb,0.23059e+01_rb,0.22071e+01_rb, &
        & 0.21796e+01_rb,0.25444e+01_rb,0.29683e+01_rb,0.32820e+01_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.35192e+01_rb,0.30832e+01_rb,0.26472e+01_rb,0.23037e+01_rb,0.22014e+01_rb, &
        & 0.21706e+01_rb,0.25342e+01_rb,0.29565e+01_rb,0.32692e+01_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.35462e+01_rb,0.31069e+01_rb,0.26676e+01_rb,0.23054e+01_rb,0.21967e+01_rb, &
        & 0.21619e+01_rb,0.25265e+01_rb,0.29475e+01_rb,0.32591e+01_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.35714e+01_rb,0.31290e+01_rb,0.26866e+01_rb,0.23141e+01_rb,0.21952e+01_rb, &
        & 0.21552e+01_rb,0.25228e+01_rb,0.29432e+01_rb,0.32501e+01_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.38742e+01_rb,0.33939e+01_rb,0.29135e+01_rb,0.26368e+01_rb,0.25318e+01_rb, &
        & 0.24566e+01_rb,0.27984e+01_rb,0.32648e+01_rb,0.36583e+01_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.38780e+01_rb,0.33972e+01_rb,0.29165e+01_rb,0.26245e+01_rb,0.25266e+01_rb, &
        & 0.24447e+01_rb,0.27836e+01_rb,0.32475e+01_rb,0.36397e+01_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.38887e+01_rb,0.34066e+01_rb,0.29246e+01_rb,0.26139e+01_rb,0.25181e+01_rb, &
        & 0.24355e+01_rb,0.27726e+01_rb,0.32346e+01_rb,0.36242e+01_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.39063e+01_rb,0.34220e+01_rb,0.29378e+01_rb,0.26094e+01_rb,0.25085e+01_rb, &
        & 0.24291e+01_rb,0.27676e+01_rb,0.32288e+01_rb,0.36150e+01_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.39241e+01_rb,0.34376e+01_rb,0.29511e+01_rb,0.26073e+01_rb,0.25021e+01_rb, &
        & 0.24243e+01_rb,0.27677e+01_rb,0.32289e+01_rb,0.36134e+01_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.42487e+01_rb,0.37216e+01_rb,0.32039e+01_rb,0.29799e+01_rb,0.28758e+01_rb, &
        & 0.27551e+01_rb,0.30498e+01_rb,0.35581e+01_rb,0.40304e+01_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.42439e+01_rb,0.37174e+01_rb,0.31977e+01_rb,0.29675e+01_rb,0.28698e+01_rb, &
        & 0.27440e+01_rb,0.30347e+01_rb,0.35405e+01_rb,0.40101e+01_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.42511e+01_rb,0.37238e+01_rb,0.32007e+01_rb,0.29528e+01_rb,0.28598e+01_rb, &
        & 0.27364e+01_rb,0.30269e+01_rb,0.35313e+01_rb,0.39978e+01_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.42624e+01_rb,0.37337e+01_rb,0.32078e+01_rb,0.29428e+01_rb,0.28469e+01_rb, &
        & 0.27311e+01_rb,0.30242e+01_rb,0.35281e+01_rb,0.39928e+01_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.42720e+01_rb,0.37420e+01_rb,0.32138e+01_rb,0.29343e+01_rb,0.28324e+01_rb, &
        & 0.27276e+01_rb,0.30224e+01_rb,0.35261e+01_rb,0.39910e+01_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.45954e+01_rb,0.40249e+01_rb,0.35022e+01_rb,0.33403e+01_rb,0.32328e+01_rb, &
        & 0.30833e+01_rb,0.33112e+01_rb,0.38621e+01_rb,0.43957e+01_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.45847e+01_rb,0.40157e+01_rb,0.34904e+01_rb,0.33265e+01_rb,0.32256e+01_rb, &
        & 0.30748e+01_rb,0.32975e+01_rb,0.38466e+01_rb,0.43773e+01_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.45864e+01_rb,0.40172e+01_rb,0.34822e+01_rb,0.33081e+01_rb,0.32132e+01_rb, &
        & 0.30693e+01_rb,0.32915e+01_rb,0.38399e+01_rb,0.43696e+01_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.45919e+01_rb,0.40220e+01_rb,0.34808e+01_rb,0.32873e+01_rb,0.31989e+01_rb, &
        & 0.30656e+01_rb,0.32892e+01_rb,0.38374e+01_rb,0.43666e+01_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.45961e+01_rb,0.40254e+01_rb,0.34809e+01_rb,0.32746e+01_rb,0.31817e+01_rb, &
        & 0.30639e+01_rb,0.32889e+01_rb,0.38370e+01_rb,0.43660e+01_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.49114e+01_rb,0.43012e+01_rb,0.38001e+01_rb,0.37007e+01_rb,0.36023e+01_rb, &
        & 0.34476e+01_rb,0.35973e+01_rb,0.41814e+01_rb,0.47692e+01_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.49023e+01_rb,0.42934e+01_rb,0.37862e+01_rb,0.36850e+01_rb,0.35912e+01_rb, &
        & 0.34412e+01_rb,0.35890e+01_rb,0.41727e+01_rb,0.47592e+01_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.48929e+01_rb,0.42852e+01_rb,0.37666e+01_rb,0.36647e+01_rb,0.35739e+01_rb, &
        & 0.34368e+01_rb,0.35843e+01_rb,0.41678e+01_rb,0.47530e+01_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.48957e+01_rb,0.42875e+01_rb,0.37577e+01_rb,0.36399e+01_rb,0.35570e+01_rb, &
        & 0.34352e+01_rb,0.35822e+01_rb,0.41658e+01_rb,0.47507e+01_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.48955e+01_rb,0.42870e+01_rb,0.37502e+01_rb,0.36181e+01_rb,0.35379e+01_rb, &
        & 0.34333e+01_rb,0.35807e+01_rb,0.41639e+01_rb,0.47483e+01_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.52336e+01_rb,0.45828e+01_rb,0.41059e+01_rb,0.40455e+01_rb,0.39709e+01_rb, &
        & 0.38438e+01_rb,0.39195e+01_rb,0.45101e+01_rb,0.51488e+01_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.52264e+01_rb,0.45766e+01_rb,0.40903e+01_rb,0.40272e+01_rb,0.39556e+01_rb, &
        & 0.38368e+01_rb,0.39154e+01_rb,0.45074e+01_rb,0.51460e+01_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.52116e+01_rb,0.45637e+01_rb,0.40682e+01_rb,0.40069e+01_rb,0.39387e+01_rb, &
        & 0.38311e+01_rb,0.39117e+01_rb,0.45034e+01_rb,0.51413e+01_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.52061e+01_rb,0.45585e+01_rb,0.40484e+01_rb,0.39793e+01_rb,0.39195e+01_rb, &
        & 0.38275e+01_rb,0.39110e+01_rb,0.45018e+01_rb,0.51396e+01_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.52033e+01_rb,0.45559e+01_rb,0.40356e+01_rb,0.39520e+01_rb,0.38983e+01_rb, &
        & 0.38238e+01_rb,0.39105e+01_rb,0.45008e+01_rb,0.51385e+01_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.55789e+01_rb,0.48845e+01_rb,0.44392e+01_rb,0.43787e+01_rb,0.43341e+01_rb, &
        & 0.42496e+01_rb,0.42580e+01_rb,0.48177e+01_rb,0.55034e+01_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.55750e+01_rb,0.48811e+01_rb,0.44216e+01_rb,0.43602e+01_rb,0.43147e+01_rb, &
        & 0.42460e+01_rb,0.42594e+01_rb,0.48202e+01_rb,0.55064e+01_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.55641e+01_rb,0.48716e+01_rb,0.43996e+01_rb,0.43387e+01_rb,0.42975e+01_rb, &
        & 0.42414e+01_rb,0.42613e+01_rb,0.48234e+01_rb,0.55098e+01_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.55497e+01_rb,0.48588e+01_rb,0.43740e+01_rb,0.43098e+01_rb,0.42767e+01_rb, &
        & 0.42365e+01_rb,0.42657e+01_rb,0.48283e+01_rb,0.55153e+01_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.55426e+01_rb,0.48524e+01_rb,0.43530e+01_rb,0.42800e+01_rb,0.42577e+01_rb, &
        & 0.42309e+01_rb,0.42668e+01_rb,0.48295e+01_rb,0.55168e+01_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.59399e+01_rb,0.51999e+01_rb,0.48042e+01_rb,0.47185e+01_rb,0.46865e+01_rb, &
        & 0.46496e+01_rb,0.46000e+01_rb,0.50897e+01_rb,0.58161e+01_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.59397e+01_rb,0.51999e+01_rb,0.47874e+01_rb,0.46998e+01_rb,0.46665e+01_rb, &
        & 0.46493e+01_rb,0.46097e+01_rb,0.50997e+01_rb,0.58275e+01_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.59349e+01_rb,0.51955e+01_rb,0.47656e+01_rb,0.46767e+01_rb,0.46485e+01_rb, &
        & 0.46473e+01_rb,0.46203e+01_rb,0.51120e+01_rb,0.58414e+01_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.59186e+01_rb,0.51811e+01_rb,0.47381e+01_rb,0.46470e+01_rb,0.46283e+01_rb, &
        & 0.46432e+01_rb,0.46335e+01_rb,0.51262e+01_rb,0.58574e+01_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.59097e+01_rb,0.51733e+01_rb,0.47110e+01_rb,0.46158e+01_rb,0.46095e+01_rb, &
        & 0.46379e+01_rb,0.46369e+01_rb,0.51321e+01_rb,0.58644e+01_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.63127e+01_rb,0.55322e+01_rb,0.51834e+01_rb,0.50691e+01_rb,0.50247e+01_rb, &
        & 0.50209e+01_rb,0.49367e+01_rb,0.53236e+01_rb,0.60842e+01_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.63183e+01_rb,0.55346e+01_rb,0.51727e+01_rb,0.50532e+01_rb,0.50080e+01_rb, &
        & 0.50274e+01_rb,0.49592e+01_rb,0.53436e+01_rb,0.61069e+01_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.63124e+01_rb,0.55272e+01_rb,0.51546e+01_rb,0.50297e+01_rb,0.49909e+01_rb, &
        & 0.50310e+01_rb,0.49812e+01_rb,0.53668e+01_rb,0.61332e+01_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.63001e+01_rb,0.55154e+01_rb,0.51286e+01_rb,0.50009e+01_rb,0.49750e+01_rb, &
        & 0.50299e+01_rb,0.49995e+01_rb,0.53888e+01_rb,0.61587e+01_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.62858e+01_rb,0.55023e+01_rb,0.50977e+01_rb,0.49708e+01_rb,0.49573e+01_rb, &
        & 0.50270e+01_rb,0.50058e+01_rb,0.53988e+01_rb,0.61697e+01_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.66894e+01_rb,0.58832e+01_rb,0.55701e+01_rb,0.54237e+01_rb,0.53594e+01_rb, &
        & 0.53556e+01_rb,0.52686e+01_rb,0.55192e+01_rb,0.63037e+01_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.66981e+01_rb,0.58851e+01_rb,0.55643e+01_rb,0.54120e+01_rb,0.53500e+01_rb, &
        & 0.53705e+01_rb,0.53019e+01_rb,0.55500e+01_rb,0.63389e+01_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.66967e+01_rb,0.58795e+01_rb,0.55464e+01_rb,0.53921e+01_rb,0.53372e+01_rb, &
        & 0.53796e+01_rb,0.53288e+01_rb,0.55798e+01_rb,0.63734e+01_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.66854e+01_rb,0.58646e+01_rb,0.55228e+01_rb,0.53676e+01_rb,0.53254e+01_rb, &
        & 0.53837e+01_rb,0.53529e+01_rb,0.56104e+01_rb,0.64086e+01_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.66717e+01_rb,0.58486e+01_rb,0.54914e+01_rb,0.53392e+01_rb,0.53082e+01_rb, &
        & 0.53849e+01_rb,0.53627e+01_rb,0.56270e+01_rb,0.64281e+01_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.70715e+01_rb,0.62495e+01_rb,0.59582e+01_rb,0.57869e+01_rb,0.56930e+01_rb, &
        & 0.56655e+01_rb,0.56036e+01_rb,0.57057e+01_rb,0.64838e+01_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.70771e+01_rb,0.62459e+01_rb,0.59461e+01_rb,0.57734e+01_rb,0.56903e+01_rb, &
        & 0.56860e+01_rb,0.56380e+01_rb,0.57455e+01_rb,0.65310e+01_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.70723e+01_rb,0.62337e+01_rb,0.59291e+01_rb,0.57507e+01_rb,0.56859e+01_rb, &
        & 0.56988e+01_rb,0.56691e+01_rb,0.57882e+01_rb,0.65837e+01_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.70579e+01_rb,0.62141e+01_rb,0.59017e+01_rb,0.57243e+01_rb,0.56747e+01_rb, &
        & 0.57061e+01_rb,0.56917e+01_rb,0.58206e+01_rb,0.66248e+01_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.70436e+01_rb,0.61942e+01_rb,0.58711e+01_rb,0.56953e+01_rb,0.56563e+01_rb, &
        & 0.57095e+01_rb,0.57007e+01_rb,0.58439e+01_rb,0.66547e+01_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.73996e+01_rb,0.65875e+01_rb,0.63444e+01_rb,0.61468e+01_rb,0.60255e+01_rb, &
        & 0.59627e+01_rb,0.59009e+01_rb,0.58767e+01_rb,0.66113e+01_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.74059e+01_rb,0.65818e+01_rb,0.63297e+01_rb,0.61319e+01_rb,0.60289e+01_rb, &
        & 0.59857e+01_rb,0.59393e+01_rb,0.59291e+01_rb,0.66795e+01_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.73985e+01_rb,0.65639e+01_rb,0.63058e+01_rb,0.61104e+01_rb,0.60278e+01_rb, &
        & 0.60017e+01_rb,0.59765e+01_rb,0.59805e+01_rb,0.67466e+01_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.73888e+01_rb,0.65439e+01_rb,0.62733e+01_rb,0.60850e+01_rb,0.60189e+01_rb, &
        & 0.60123e+01_rb,0.59968e+01_rb,0.60157e+01_rb,0.67935e+01_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.73717e+01_rb,0.65185e+01_rb,0.62375e+01_rb,0.60536e+01_rb,0.59983e+01_rb, &
        & 0.60180e+01_rb,0.60067e+01_rb,0.60456e+01_rb,0.68365e+01_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.76469e+01_rb,0.68731e+01_rb,0.66871e+01_rb,0.64997e+01_rb,0.63502e+01_rb, &
        & 0.62466e+01_rb,0.61469e+01_rb,0.60253e+01_rb,0.66908e+01_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.76513e+01_rb,0.68603e+01_rb,0.66717e+01_rb,0.64853e+01_rb,0.63628e+01_rb, &
        & 0.62754e+01_rb,0.61942e+01_rb,0.60911e+01_rb,0.67781e+01_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.76480e+01_rb,0.68404e+01_rb,0.66430e+01_rb,0.64664e+01_rb,0.63629e+01_rb, &
        & 0.62938e+01_rb,0.62370e+01_rb,0.61432e+01_rb,0.68460e+01_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.76417e+01_rb,0.68180e+01_rb,0.66082e+01_rb,0.64389e+01_rb,0.63512e+01_rb, &
        & 0.63082e+01_rb,0.62633e+01_rb,0.61842e+01_rb,0.69071e+01_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.76283e+01_rb,0.67918e+01_rb,0.65683e+01_rb,0.64071e+01_rb,0.63329e+01_rb, &
        & 0.63131e+01_rb,0.62779e+01_rb,0.62184e+01_rb,0.69616e+01_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.48056e+01_rb,0.42094e+01_rb,0.36132e+01_rb,0.30171e+01_rb,0.28404e+01_rb, &
        & 0.34490e+01_rb,0.41388e+01_rb,0.48284e+01_rb,0.52314e+01_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.48171e+01_rb,0.42194e+01_rb,0.36219e+01_rb,0.30243e+01_rb,0.28141e+01_rb, &
        & 0.34073e+01_rb,0.40887e+01_rb,0.47701e+01_rb,0.51637e+01_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.48231e+01_rb,0.42245e+01_rb,0.36259e+01_rb,0.30275e+01_rb,0.27871e+01_rb, &
        & 0.33709e+01_rb,0.40450e+01_rb,0.47190e+01_rb,0.51029e+01_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.48223e+01_rb,0.42236e+01_rb,0.36250e+01_rb,0.30265e+01_rb,0.27616e+01_rb, &
        & 0.33346e+01_rb,0.40014e+01_rb,0.46682e+01_rb,0.50473e+01_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.48164e+01_rb,0.42183e+01_rb,0.36202e+01_rb,0.30224e+01_rb,0.27378e+01_rb, &
        & 0.32980e+01_rb,0.39575e+01_rb,0.46170e+01_rb,0.49922e+01_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.56696e+01_rb,0.49649e+01_rb,0.42601e+01_rb,0.35603e+01_rb,0.33660e+01_rb, &
        & 0.40230e+01_rb,0.48275e+01_rb,0.56320e+01_rb,0.62204e+01_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.56853e+01_rb,0.49788e+01_rb,0.42721e+01_rb,0.35690e+01_rb,0.33370e+01_rb, &
        & 0.39815e+01_rb,0.47778e+01_rb,0.55740e+01_rb,0.61522e+01_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.56912e+01_rb,0.49841e+01_rb,0.42769e+01_rb,0.35723e+01_rb,0.33088e+01_rb, &
        & 0.39404e+01_rb,0.47285e+01_rb,0.55164e+01_rb,0.60886e+01_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.56900e+01_rb,0.49829e+01_rb,0.42759e+01_rb,0.35705e+01_rb,0.32819e+01_rb, &
        & 0.38975e+01_rb,0.46769e+01_rb,0.54563e+01_rb,0.60227e+01_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.56828e+01_rb,0.49765e+01_rb,0.42703e+01_rb,0.35653e+01_rb,0.32552e+01_rb, &
        & 0.38520e+01_rb,0.46223e+01_rb,0.53926e+01_rb,0.59537e+01_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.66640e+01_rb,0.58345e+01_rb,0.50050e+01_rb,0.41960e+01_rb,0.39837e+01_rb, &
        & 0.46765e+01_rb,0.56117e+01_rb,0.65468e+01_rb,0.73624e+01_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.66851e+01_rb,0.58530e+01_rb,0.50209e+01_rb,0.42035e+01_rb,0.39538e+01_rb, &
        & 0.46274e+01_rb,0.55528e+01_rb,0.64781e+01_rb,0.72849e+01_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.66916e+01_rb,0.58588e+01_rb,0.50261e+01_rb,0.42046e+01_rb,0.39240e+01_rb, &
        & 0.45763e+01_rb,0.54915e+01_rb,0.64066e+01_rb,0.72060e+01_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.66884e+01_rb,0.58563e+01_rb,0.50241e+01_rb,0.42004e+01_rb,0.38957e+01_rb, &
        & 0.45225e+01_rb,0.54270e+01_rb,0.63313e+01_rb,0.71219e+01_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.66766e+01_rb,0.58462e+01_rb,0.50163e+01_rb,0.41933e+01_rb,0.38675e+01_rb, &
        & 0.44702e+01_rb,0.53642e+01_rb,0.62581e+01_rb,0.70387e+01_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.77848e+01_rb,0.68149e+01_rb,0.58451e+01_rb,0.49331e+01_rb,0.46977e+01_rb, &
        & 0.54033e+01_rb,0.64839e+01_rb,0.75644e+01_rb,0.85830e+01_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.78108e+01_rb,0.68377e+01_rb,0.58646e+01_rb,0.49375e+01_rb,0.46695e+01_rb, &
        & 0.53453e+01_rb,0.64143e+01_rb,0.74831e+01_rb,0.84915e+01_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.78236e+01_rb,0.68489e+01_rb,0.58743e+01_rb,0.49373e+01_rb,0.46417e+01_rb, &
        & 0.52821e+01_rb,0.63384e+01_rb,0.73947e+01_rb,0.83915e+01_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.78238e+01_rb,0.68493e+01_rb,0.58748e+01_rb,0.49301e+01_rb,0.46103e+01_rb, &
        & 0.52166e+01_rb,0.62599e+01_rb,0.73031e+01_rb,0.82871e+01_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.78090e+01_rb,0.68373e+01_rb,0.58658e+01_rb,0.49180e+01_rb,0.45786e+01_rb, &
        & 0.51516e+01_rb,0.61819e+01_rb,0.72121e+01_rb,0.81837e+01_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.90367e+01_rb,0.79101e+01_rb,0.67835e+01_rb,0.57840e+01_rb,0.55090e+01_rb, &
        & 0.61816e+01_rb,0.74174e+01_rb,0.86536e+01_rb,0.98560e+01_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.90658e+01_rb,0.79355e+01_rb,0.68052e+01_rb,0.57840e+01_rb,0.54825e+01_rb, &
        & 0.61122e+01_rb,0.73329e+01_rb,0.85548e+01_rb,0.97452e+01_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.90837e+01_rb,0.79512e+01_rb,0.68188e+01_rb,0.57766e+01_rb,0.54581e+01_rb, &
        & 0.60426e+01_rb,0.72485e+01_rb,0.84564e+01_rb,0.96328e+01_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.90830e+01_rb,0.79510e+01_rb,0.68192e+01_rb,0.57645e+01_rb,0.54297e+01_rb, &
        & 0.59729e+01_rb,0.71644e+01_rb,0.83583e+01_rb,0.95205e+01_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.90677e+01_rb,0.79386e+01_rb,0.68094e+01_rb,0.57479e+01_rb,0.53962e+01_rb, &
        & 0.58998e+01_rb,0.70740e+01_rb,0.82528e+01_rb,0.94008e+01_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.10412e+02_rb,0.91128e+01_rb,0.78140e+01_rb,0.67504e+01_rb,0.64389e+01_rb, &
        & 0.70217e+01_rb,0.83899e+01_rb,0.97882e+01_rb,0.11168e+02_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.10449e+02_rb,0.91451e+01_rb,0.78417e+01_rb,0.67466e+01_rb,0.64110e+01_rb, &
        & 0.69507e+01_rb,0.82964e+01_rb,0.96789e+01_rb,0.11043e+02_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.10469e+02_rb,0.91634e+01_rb,0.78574e+01_rb,0.67323e+01_rb,0.63834e+01_rb, &
        & 0.68804e+01_rb,0.82097e+01_rb,0.95778e+01_rb,0.10928e+02_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.10466e+02_rb,0.91614e+01_rb,0.78568e+01_rb,0.67135e+01_rb,0.63530e+01_rb, &
        & 0.68083e+01_rb,0.81167e+01_rb,0.94694e+01_rb,0.10805e+02_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.10450e+02_rb,0.91481e+01_rb,0.78461e+01_rb,0.66866e+01_rb,0.63182e+01_rb, &
        & 0.67317e+01_rb,0.80175e+01_rb,0.93536e+01_rb,0.10671e+02_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.11882e+02_rb,0.10399e+02_rb,0.89166e+01_rb,0.78253e+01_rb,0.74792e+01_rb, &
        & 0.79512e+01_rb,0.94202e+01_rb,0.10990e+02_rb,0.12550e+02_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.11932e+02_rb,0.10443e+02_rb,0.89541e+01_rb,0.78202e+01_rb,0.74549e+01_rb, &
        & 0.78754e+01_rb,0.93207e+01_rb,0.10874e+02_rb,0.12417e+02_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.11956e+02_rb,0.10464e+02_rb,0.89727e+01_rb,0.78020e+01_rb,0.74258e+01_rb, &
        & 0.78051e+01_rb,0.92218e+01_rb,0.10758e+02_rb,0.12285e+02_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.11956e+02_rb,0.10465e+02_rb,0.89740e+01_rb,0.77736e+01_rb,0.73912e+01_rb, &
        & 0.77319e+01_rb,0.91108e+01_rb,0.10629e+02_rb,0.12138e+02_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.11934e+02_rb,0.10446e+02_rb,0.89587e+01_rb,0.77346e+01_rb,0.73464e+01_rb, &
        & 0.76503e+01_rb,0.90007e+01_rb,0.10501e+02_rb,0.11991e+02_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.13440e+02_rb,0.11762e+02_rb,0.10084e+02_rb,0.90134e+01_rb,0.86298e+01_rb, &
        & 0.89787e+01_rb,0.10509e+02_rb,0.12260e+02_rb,0.14008e+02_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.13496e+02_rb,0.11811e+02_rb,0.10127e+02_rb,0.90042e+01_rb,0.86075e+01_rb, &
        & 0.89016e+01_rb,0.10401e+02_rb,0.12134e+02_rb,0.13863e+02_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.13522e+02_rb,0.11835e+02_rb,0.10148e+02_rb,0.89806e+01_rb,0.85777e+01_rb, &
        & 0.88306e+01_rb,0.10284e+02_rb,0.11998e+02_rb,0.13707e+02_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.13526e+02_rb,0.11839e+02_rb,0.10151e+02_rb,0.89422e+01_rb,0.85378e+01_rb, &
        & 0.87510e+01_rb,0.10150e+02_rb,0.11842e+02_rb,0.13529e+02_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.13498e+02_rb,0.11814e+02_rb,0.10131e+02_rb,0.88918e+01_rb,0.84819e+01_rb, &
        & 0.86621e+01_rb,0.10028e+02_rb,0.11700e+02_rb,0.13367e+02_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.15050e+02_rb,0.13171e+02_rb,0.11325e+02_rb,0.10314e+02_rb,0.98844e+01_rb, &
        & 0.10113e+02_rb,0.11655e+02_rb,0.13598e+02_rb,0.15538e+02_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.15110e+02_rb,0.13224e+02_rb,0.11358e+02_rb,0.10290e+02_rb,0.98636e+01_rb, &
        & 0.10034e+02_rb,0.11528e+02_rb,0.13449e+02_rb,0.15369e+02_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.15142e+02_rb,0.13253e+02_rb,0.11372e+02_rb,0.10253e+02_rb,0.98329e+01_rb, &
        & 0.99565e+01_rb,0.11387e+02_rb,0.13285e+02_rb,0.15182e+02_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.15140e+02_rb,0.13251e+02_rb,0.11368e+02_rb,0.10200e+02_rb,0.97822e+01_rb, &
        & 0.98656e+01_rb,0.11236e+02_rb,0.13109e+02_rb,0.14981e+02_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.15112e+02_rb,0.13227e+02_rb,0.11344e+02_rb,0.10134e+02_rb,0.97132e+01_rb, &
        & 0.97634e+01_rb,0.11104e+02_rb,0.12955e+02_rb,0.14806e+02_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.16679e+02_rb,0.14596e+02_rb,0.12639e+02_rb,0.11713e+02_rb,0.11234e+02_rb, &
        & 0.11344e+02_rb,0.12833e+02_rb,0.14972e+02_rb,0.17110e+02_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.16740e+02_rb,0.14649e+02_rb,0.12658e+02_rb,0.11669e+02_rb,0.11203e+02_rb, &
        & 0.11257e+02_rb,0.12688e+02_rb,0.14803e+02_rb,0.16918e+02_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.16768e+02_rb,0.14675e+02_rb,0.12658e+02_rb,0.11609e+02_rb,0.11161e+02_rb, &
        & 0.11169e+02_rb,0.12530e+02_rb,0.14619e+02_rb,0.16708e+02_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.16761e+02_rb,0.14669e+02_rb,0.12632e+02_rb,0.11528e+02_rb,0.11096e+02_rb, &
        & 0.11064e+02_rb,0.12360e+02_rb,0.14420e+02_rb,0.16480e+02_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.16723e+02_rb,0.14636e+02_rb,0.12592e+02_rb,0.11437e+02_rb,0.11017e+02_rb, &
        & 0.10938e+02_rb,0.12210e+02_rb,0.14245e+02_rb,0.16279e+02_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.18294e+02_rb,0.16009e+02_rb,0.13989e+02_rb,0.13135e+02_rb,0.12643e+02_rb, &
        & 0.12628e+02_rb,0.13972e+02_rb,0.16290e+02_rb,0.18616e+02_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.18347e+02_rb,0.16056e+02_rb,0.13990e+02_rb,0.13069e+02_rb,0.12593e+02_rb, &
        & 0.12524e+02_rb,0.13808e+02_rb,0.16097e+02_rb,0.18396e+02_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.18360e+02_rb,0.16067e+02_rb,0.13961e+02_rb,0.12977e+02_rb,0.12519e+02_rb, &
        & 0.12404e+02_rb,0.13621e+02_rb,0.15875e+02_rb,0.18143e+02_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.18335e+02_rb,0.16046e+02_rb,0.13912e+02_rb,0.12870e+02_rb,0.12428e+02_rb, &
        & 0.12262e+02_rb,0.13438e+02_rb,0.15668e+02_rb,0.17906e+02_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.18275e+02_rb,0.15994e+02_rb,0.13835e+02_rb,0.12741e+02_rb,0.12318e+02_rb, &
        & 0.12107e+02_rb,0.13269e+02_rb,0.15473e+02_rb,0.17684e+02_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.19924e+02_rb,0.17435e+02_rb,0.15328e+02_rb,0.14570e+02_rb,0.14103e+02_rb, &
        & 0.13966e+02_rb,0.15137e+02_rb,0.17574e+02_rb,0.20085e+02_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.19952e+02_rb,0.17460e+02_rb,0.15303e+02_rb,0.14472e+02_rb,0.14016e+02_rb, &
        & 0.13836e+02_rb,0.14951e+02_rb,0.17356e+02_rb,0.19836e+02_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.19950e+02_rb,0.17459e+02_rb,0.15252e+02_rb,0.14354e+02_rb,0.13911e+02_rb, &
        & 0.13674e+02_rb,0.14736e+02_rb,0.17121e+02_rb,0.19567e+02_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.19898e+02_rb,0.17414e+02_rb,0.15172e+02_rb,0.14207e+02_rb,0.13782e+02_rb, &
        & 0.13496e+02_rb,0.14540e+02_rb,0.16905e+02_rb,0.19321e+02_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.19813e+02_rb,0.17339e+02_rb,0.15067e+02_rb,0.14045e+02_rb,0.13635e+02_rb, &
        & 0.13302e+02_rb,0.14349e+02_rb,0.16691e+02_rb,0.19076e+02_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.21582e+02_rb,0.18886e+02_rb,0.16708e+02_rb,0.15984e+02_rb,0.15578e+02_rb, &
        & 0.15338e+02_rb,0.16324e+02_rb,0.18812e+02_rb,0.21500e+02_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.21595e+02_rb,0.18898e+02_rb,0.16645e+02_rb,0.15860e+02_rb,0.15448e+02_rb, &
        & 0.15169e+02_rb,0.16117e+02_rb,0.18583e+02_rb,0.21237e+02_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.21561e+02_rb,0.18868e+02_rb,0.16562e+02_rb,0.15696e+02_rb,0.15303e+02_rb, &
        & 0.14968e+02_rb,0.15881e+02_rb,0.18352e+02_rb,0.20974e+02_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.21483e+02_rb,0.18800e+02_rb,0.16445e+02_rb,0.15517e+02_rb,0.15132e+02_rb, &
        & 0.14742e+02_rb,0.15661e+02_rb,0.18119e+02_rb,0.20709e+02_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.21359e+02_rb,0.18692e+02_rb,0.16307e+02_rb,0.15310e+02_rb,0.14932e+02_rb, &
        & 0.14513e+02_rb,0.15442e+02_rb,0.17892e+02_rb,0.20448e+02_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.54900e+01_rb,0.48060e+01_rb,0.41232e+01_rb,0.34422e+01_rb,0.34870e+01_rb, &
        & 0.43588e+01_rb,0.52305e+01_rb,0.61020e+01_rb,0.65047e+01_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.54959e+01_rb,0.48102e+01_rb,0.41246e+01_rb,0.34396e+01_rb,0.34389e+01_rb, &
        & 0.42986e+01_rb,0.51583e+01_rb,0.60178e+01_rb,0.64181e+01_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.54964e+01_rb,0.48111e+01_rb,0.41257e+01_rb,0.34404e+01_rb,0.33906e+01_rb, &
        & 0.42383e+01_rb,0.50859e+01_rb,0.59334e+01_rb,0.63255e+01_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.54872e+01_rb,0.48032e+01_rb,0.41193e+01_rb,0.34354e+01_rb,0.33485e+01_rb, &
        & 0.41856e+01_rb,0.50227e+01_rb,0.58596e+01_rb,0.62437e+01_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.54677e+01_rb,0.47864e+01_rb,0.41051e+01_rb,0.34238e+01_rb,0.33099e+01_rb, &
        & 0.41373e+01_rb,0.49647e+01_rb,0.57920e+01_rb,0.61726e+01_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.65993e+01_rb,0.57796e+01_rb,0.49604e+01_rb,0.41421e+01_rb,0.42011e+01_rb, &
        & 0.52514e+01_rb,0.63014e+01_rb,0.73516e+01_rb,0.80279e+01_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.66032e+01_rb,0.57810e+01_rb,0.49597e+01_rb,0.41395e+01_rb,0.41430e+01_rb, &
        & 0.51787e+01_rb,0.62144e+01_rb,0.72499e+01_rb,0.79170e+01_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.66114e+01_rb,0.57864e+01_rb,0.49618e+01_rb,0.41380e+01_rb,0.40849e+01_rb, &
        & 0.51061e+01_rb,0.61273e+01_rb,0.71483e+01_rb,0.78058e+01_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.66055e+01_rb,0.57814e+01_rb,0.49573e+01_rb,0.41341e+01_rb,0.40341e+01_rb, &
        & 0.50426e+01_rb,0.60510e+01_rb,0.70593e+01_rb,0.77043e+01_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.65867e+01_rb,0.57651e+01_rb,0.49435e+01_rb,0.41221e+01_rb,0.39863e+01_rb, &
        & 0.49829e+01_rb,0.59794e+01_rb,0.69758e+01_rb,0.76124e+01_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.79380e+01_rb,0.69526e+01_rb,0.59676e+01_rb,0.49832e+01_rb,0.50788e+01_rb, &
        & 0.63485e+01_rb,0.76181e+01_rb,0.88875e+01_rb,0.99377e+01_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.79519e+01_rb,0.69640e+01_rb,0.59765e+01_rb,0.49895e+01_rb,0.50033e+01_rb, &
        & 0.62541e+01_rb,0.75048e+01_rb,0.87554e+01_rb,0.97869e+01_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.79564e+01_rb,0.69660e+01_rb,0.59762e+01_rb,0.49871e+01_rb,0.49341e+01_rb, &
        & 0.61676e+01_rb,0.74011e+01_rb,0.86343e+01_rb,0.96531e+01_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.79604e+01_rb,0.69678e+01_rb,0.59753e+01_rb,0.49833e+01_rb,0.48716e+01_rb, &
        & 0.60895e+01_rb,0.73073e+01_rb,0.85250e+01_rb,0.95280e+01_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.79454e+01_rb,0.69546e+01_rb,0.59641e+01_rb,0.49739e+01_rb,0.48117e+01_rb, &
        & 0.60146e+01_rb,0.72174e+01_rb,0.84201e+01_rb,0.94106e+01_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.95482e+01_rb,0.83619e+01_rb,0.71758e+01_rb,0.59904e+01_rb,0.61617e+01_rb, &
        & 0.77021e+01_rb,0.92425e+01_rb,0.10783e+02_rb,0.12199e+02_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.95751e+01_rb,0.83849e+01_rb,0.71952e+01_rb,0.60057e+01_rb,0.60507e+01_rb, &
        & 0.75634e+01_rb,0.90760e+01_rb,0.10588e+02_rb,0.11979e+02_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.95851e+01_rb,0.83928e+01_rb,0.72008e+01_rb,0.60093e+01_rb,0.59519e+01_rb, &
        & 0.74398e+01_rb,0.89277e+01_rb,0.10415e+02_rb,0.11784e+02_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.95899e+01_rb,0.83956e+01_rb,0.72016e+01_rb,0.60083e+01_rb,0.58751e+01_rb, &
        & 0.73438e+01_rb,0.88125e+01_rb,0.10281e+02_rb,0.11628e+02_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.95802e+01_rb,0.83858e+01_rb,0.71919e+01_rb,0.59984e+01_rb,0.58004e+01_rb, &
        & 0.72505e+01_rb,0.87005e+01_rb,0.10150e+02_rb,0.11482e+02_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.11467e+02_rb,0.10041e+02_rb,0.86153e+01_rb,0.71899e+01_rb,0.74662e+01_rb, &
        & 0.93327e+01_rb,0.11199e+02_rb,0.13065e+02_rb,0.14858e+02_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.11518e+02_rb,0.10085e+02_rb,0.86526e+01_rb,0.72201e+01_rb,0.73208e+01_rb, &
        & 0.91509e+01_rb,0.10981e+02_rb,0.12811e+02_rb,0.14568e+02_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.11535e+02_rb,0.10099e+02_rb,0.86636e+01_rb,0.72282e+01_rb,0.71844e+01_rb, &
        & 0.89805e+01_rb,0.10776e+02_rb,0.12572e+02_rb,0.14298e+02_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.11542e+02_rb,0.10105e+02_rb,0.86677e+01_rb,0.72309e+01_rb,0.70636e+01_rb, &
        & 0.88295e+01_rb,0.10595e+02_rb,0.12361e+02_rb,0.14056e+02_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.11528e+02_rb,0.10092e+02_rb,0.86561e+01_rb,0.72209e+01_rb,0.69634e+01_rb, &
        & 0.87042e+01_rb,0.10445e+02_rb,0.12185e+02_rb,0.13857e+02_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.13743e+02_rb,0.12032e+02_rb,0.10321e+02_rb,0.86104e+01_rb,0.90301e+01_rb, &
        & 0.11288e+02_rb,0.13545e+02_rb,0.15802e+02_rb,0.18010e+02_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.13809e+02_rb,0.12090e+02_rb,0.10370e+02_rb,0.86513e+01_rb,0.88422e+01_rb, &
        & 0.11053e+02_rb,0.13263e+02_rb,0.15473e+02_rb,0.17638e+02_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.13850e+02_rb,0.12125e+02_rb,0.10400e+02_rb,0.86753e+01_rb,0.86607e+01_rb, &
        & 0.10826e+02_rb,0.12991e+02_rb,0.15156e+02_rb,0.17275e+02_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.13866e+02_rb,0.12138e+02_rb,0.10411e+02_rb,0.86837e+01_rb,0.84930e+01_rb, &
        & 0.10616e+02_rb,0.12739e+02_rb,0.14862e+02_rb,0.16940e+02_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.13850e+02_rb,0.12124e+02_rb,0.10399e+02_rb,0.86732e+01_rb,0.83442e+01_rb, &
        & 0.10430e+02_rb,0.12516e+02_rb,0.14602e+02_rb,0.16644e+02_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.16450e+02_rb,0.14400e+02_rb,0.12350e+02_rb,0.10300e+02_rb,0.10874e+02_rb, &
        & 0.13593e+02_rb,0.16311e+02_rb,0.19030e+02_rb,0.21722e+02_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.16525e+02_rb,0.14465e+02_rb,0.12405e+02_rb,0.10346e+02_rb,0.10632e+02_rb, &
        & 0.13290e+02_rb,0.15948e+02_rb,0.18606e+02_rb,0.21239e+02_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.16563e+02_rb,0.14499e+02_rb,0.12434e+02_rb,0.10370e+02_rb,0.10395e+02_rb, &
        & 0.12994e+02_rb,0.15592e+02_rb,0.18191e+02_rb,0.20762e+02_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.16590e+02_rb,0.14522e+02_rb,0.12454e+02_rb,0.10386e+02_rb,0.10172e+02_rb, &
        & 0.12715e+02_rb,0.15258e+02_rb,0.17801e+02_rb,0.20317e+02_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.16576e+02_rb,0.14509e+02_rb,0.12443e+02_rb,0.10377e+02_rb,0.99688e+01_rb, &
        & 0.12461e+02_rb,0.14953e+02_rb,0.17445e+02_rb,0.19913e+02_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.19629e+02_rb,0.17181e+02_rb,0.14733e+02_rb,0.12285e+02_rb,0.13022e+02_rb, &
        & 0.16277e+02_rb,0.19532e+02_rb,0.22787e+02_rb,0.26029e+02_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.19728e+02_rb,0.17267e+02_rb,0.14807e+02_rb,0.12346e+02_rb,0.12711e+02_rb, &
        & 0.15888e+02_rb,0.19066e+02_rb,0.22243e+02_rb,0.25406e+02_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.19768e+02_rb,0.17302e+02_rb,0.14837e+02_rb,0.12371e+02_rb,0.12399e+02_rb, &
        & 0.15499e+02_rb,0.18599e+02_rb,0.21698e+02_rb,0.24782e+02_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.19773e+02_rb,0.17307e+02_rb,0.14840e+02_rb,0.12374e+02_rb,0.12111e+02_rb, &
        & 0.15139e+02_rb,0.18167e+02_rb,0.21194e+02_rb,0.24210e+02_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.19744e+02_rb,0.17281e+02_rb,0.14818e+02_rb,0.12356e+02_rb,0.11840e+02_rb, &
        & 0.14800e+02_rb,0.17760e+02_rb,0.20720e+02_rb,0.23667e+02_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.23312e+02_rb,0.20403e+02_rb,0.17494e+02_rb,0.14584e+02_rb,0.15488e+02_rb, &
        & 0.19360e+02_rb,0.23231e+02_rb,0.27103e+02_rb,0.30970e+02_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.23439e+02_rb,0.20514e+02_rb,0.17588e+02_rb,0.14663e+02_rb,0.15085e+02_rb, &
        & 0.18856e+02_rb,0.22627e+02_rb,0.26397e+02_rb,0.30166e+02_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.23492e+02_rb,0.20560e+02_rb,0.17628e+02_rb,0.14696e+02_rb,0.14687e+02_rb, &
        & 0.18356e+02_rb,0.22027e+02_rb,0.25697e+02_rb,0.29367e+02_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.23479e+02_rb,0.20549e+02_rb,0.17619e+02_rb,0.14689e+02_rb,0.14321e+02_rb, &
        & 0.17895e+02_rb,0.21474e+02_rb,0.25052e+02_rb,0.28629e+02_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.23411e+02_rb,0.20489e+02_rb,0.17567e+02_rb,0.14646e+02_rb,0.13982e+02_rb, &
        & 0.17451e+02_rb,0.20941e+02_rb,0.24431e+02_rb,0.27917e+02_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.27592e+02_rb,0.24147e+02_rb,0.20702e+02_rb,0.17264e+02_rb,0.18240e+02_rb, &
        & 0.22793e+02_rb,0.27352e+02_rb,0.31910e+02_rb,0.36473e+02_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.27703e+02_rb,0.24245e+02_rb,0.20786e+02_rb,0.17327e+02_rb,0.17749e+02_rb, &
        & 0.22147e+02_rb,0.26577e+02_rb,0.31006e+02_rb,0.35436e+02_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.27743e+02_rb,0.24279e+02_rb,0.20816e+02_rb,0.17352e+02_rb,0.17266e+02_rb, &
        & 0.21525e+02_rb,0.25830e+02_rb,0.30134e+02_rb,0.34441e+02_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.27709e+02_rb,0.24250e+02_rb,0.20790e+02_rb,0.17331e+02_rb,0.16811e+02_rb, &
        & 0.20946e+02_rb,0.25135e+02_rb,0.29324e+02_rb,0.33510e+02_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.27598e+02_rb,0.24153e+02_rb,0.20707e+02_rb,0.17262e+02_rb,0.16378e+02_rb, &
        & 0.20388e+02_rb,0.24466e+02_rb,0.28543e+02_rb,0.32622e+02_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.32570e+02_rb,0.28502e+02_rb,0.24434e+02_rb,0.20393e+02_rb,0.21083e+02_rb, &
        & 0.26245e+02_rb,0.31494e+02_rb,0.36742e+02_rb,0.41994e+02_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.32601e+02_rb,0.28529e+02_rb,0.24458e+02_rb,0.20386e+02_rb,0.20492e+02_rb, &
        & 0.25469e+02_rb,0.30563e+02_rb,0.35657e+02_rb,0.40747e+02_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.32535e+02_rb,0.28472e+02_rb,0.24408e+02_rb,0.20345e+02_rb,0.19916e+02_rb, &
        & 0.24733e+02_rb,0.29679e+02_rb,0.34625e+02_rb,0.39568e+02_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.32390e+02_rb,0.28345e+02_rb,0.24301e+02_rb,0.20256e+02_rb,0.19358e+02_rb, &
        & 0.24023e+02_rb,0.28827e+02_rb,0.33631e+02_rb,0.38435e+02_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.32175e+02_rb,0.28157e+02_rb,0.24140e+02_rb,0.20122e+02_rb,0.18819e+02_rb, &
        & 0.23330e+02_rb,0.27996e+02_rb,0.32661e+02_rb,0.37328e+02_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.38135e+02_rb,0.33371e+02_rb,0.28607e+02_rb,0.23891e+02_rb,0.24203e+02_rb, &
        & 0.29981e+02_rb,0.35977e+02_rb,0.41973e+02_rb,0.47971e+02_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.38033e+02_rb,0.33282e+02_rb,0.28531e+02_rb,0.23779e+02_rb,0.23491e+02_rb, &
        & 0.29038e+02_rb,0.34845e+02_rb,0.40652e+02_rb,0.46464e+02_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.37806e+02_rb,0.33084e+02_rb,0.28361e+02_rb,0.23639e+02_rb,0.22788e+02_rb, &
        & 0.28150e+02_rb,0.33780e+02_rb,0.39409e+02_rb,0.45036e+02_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.37497e+02_rb,0.32813e+02_rb,0.28129e+02_rb,0.23446e+02_rb,0.22100e+02_rb, &
        & 0.27270e+02_rb,0.32724e+02_rb,0.38179e+02_rb,0.43632e+02_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.37099e+02_rb,0.32465e+02_rb,0.27831e+02_rb,0.23198e+02_rb,0.21440e+02_rb, &
        & 0.26398e+02_rb,0.31678e+02_rb,0.36957e+02_rb,0.42237e+02_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.44217e+02_rb,0.38692e+02_rb,0.33167e+02_rb,0.27730e+02_rb,0.27605e+02_rb, &
        & 0.33969e+02_rb,0.40763e+02_rb,0.47556e+02_rb,0.54352e+02_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.43915e+02_rb,0.38428e+02_rb,0.32941e+02_rb,0.27455e+02_rb,0.26738e+02_rb, &
        & 0.32787e+02_rb,0.39345e+02_rb,0.45901e+02_rb,0.52455e+02_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.43489e+02_rb,0.38055e+02_rb,0.32622e+02_rb,0.27188e+02_rb,0.25861e+02_rb, &
        & 0.31686e+02_rb,0.38022e+02_rb,0.44359e+02_rb,0.50700e+02_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.42951e+02_rb,0.37585e+02_rb,0.32219e+02_rb,0.26853e+02_rb,0.25006e+02_rb, &
        & 0.30590e+02_rb,0.36708e+02_rb,0.42826e+02_rb,0.48946e+02_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.42298e+02_rb,0.37013e+02_rb,0.31729e+02_rb,0.26445e+02_rb,0.24180e+02_rb, &
        & 0.29511e+02_rb,0.35413e+02_rb,0.41314e+02_rb,0.47218e+02_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:, 1,13, 1) = (/ &
        & 0.40633e-08_rb,0.64718e-06_rb,0.79567e-06_rb,0.81264e-06_rb,0.25080e-07_rb /)
      kbo(:, 2,13, 1) = (/ &
        & 0.37596e-08_rb,0.70429e-06_rb,0.90855e-06_rb,0.96432e-06_rb,0.28292e-07_rb /)
      kbo(:, 3,13, 1) = (/ &
        & 0.34982e-08_rb,0.76370e-06_rb,0.10316e-05_rb,0.11472e-05_rb,0.31265e-07_rb /)
      kbo(:, 4,13, 1) = (/ &
        & 0.32707e-08_rb,0.82834e-06_rb,0.11577e-05_rb,0.13387e-05_rb,0.34928e-07_rb /)
      kbo(:, 5,13, 1) = (/ &
        & 0.30710e-08_rb,0.90252e-06_rb,0.12962e-05_rb,0.15206e-05_rb,0.38859e-07_rb /)
      kbo(:, 1,14, 1) = (/ &
        & 0.33026e-08_rb,0.53592e-06_rb,0.66171e-06_rb,0.67633e-06_rb,0.20767e-07_rb /)
      kbo(:, 2,14, 1) = (/ &
        & 0.30574e-08_rb,0.57959e-06_rb,0.75386e-06_rb,0.80395e-06_rb,0.23334e-07_rb /)
      kbo(:, 3,14, 1) = (/ &
        & 0.28461e-08_rb,0.62897e-06_rb,0.85486e-06_rb,0.95745e-06_rb,0.25841e-07_rb /)
      kbo(:, 4,14, 1) = (/ &
        & 0.26621e-08_rb,0.68352e-06_rb,0.96132e-06_rb,0.11148e-05_rb,0.28862e-07_rb /)
      kbo(:, 5,14, 1) = (/ &
        & 0.25005e-08_rb,0.74517e-06_rb,0.10739e-05_rb,0.12632e-05_rb,0.32093e-07_rb /)
      kbo(:, 1,15, 1) = (/ &
        & 0.26821e-08_rb,0.44081e-06_rb,0.55093e-06_rb,0.56465e-06_rb,0.17186e-07_rb /)
      kbo(:, 2,15, 1) = (/ &
        & 0.24845e-08_rb,0.47773e-06_rb,0.62636e-06_rb,0.67180e-06_rb,0.19234e-07_rb /)
      kbo(:, 3,15, 1) = (/ &
        & 0.23140e-08_rb,0.51811e-06_rb,0.70791e-06_rb,0.79658e-06_rb,0.21348e-07_rb /)
      kbo(:, 4,15, 1) = (/ &
        & 0.21654e-08_rb,0.56416e-06_rb,0.79893e-06_rb,0.92873e-06_rb,0.23832e-07_rb /)
      kbo(:, 5,15, 1) = (/ &
        & 0.20347e-08_rb,0.61535e-06_rb,0.88985e-06_rb,0.10499e-05_rb,0.26485e-07_rb /)
      kbo(:, 1,16, 1) = (/ &
        & 0.21775e-08_rb,0.36254e-06_rb,0.45870e-06_rb,0.47125e-06_rb,0.14211e-07_rb /)
      kbo(:, 2,16, 1) = (/ &
        & 0.20183e-08_rb,0.39376e-06_rb,0.52053e-06_rb,0.56163e-06_rb,0.15869e-07_rb /)
      kbo(:, 3,16, 1) = (/ &
        & 0.18808e-08_rb,0.42687e-06_rb,0.58699e-06_rb,0.66386e-06_rb,0.17627e-07_rb /)
      kbo(:, 4,16, 1) = (/ &
        & 0.17608e-08_rb,0.46559e-06_rb,0.66388e-06_rb,0.77063e-06_rb,0.19664e-07_rb /)
      kbo(:, 5,16, 1) = (/ &
        & 0.16552e-08_rb,0.50747e-06_rb,0.73699e-06_rb,0.87249e-06_rb,0.21844e-07_rb /)
      kbo(:, 1,17, 1) = (/ &
        & 0.17682e-08_rb,0.29836e-06_rb,0.38104e-06_rb,0.39314e-06_rb,0.11735e-07_rb /)
      kbo(:, 2,17, 1) = (/ &
        & 0.16399e-08_rb,0.32493e-06_rb,0.43215e-06_rb,0.46897e-06_rb,0.13077e-07_rb /)
      kbo(:, 3,17, 1) = (/ &
        & 0.15290e-08_rb,0.35203e-06_rb,0.48665e-06_rb,0.55328e-06_rb,0.14535e-07_rb /)
      kbo(:, 4,17, 1) = (/ &
        & 0.14321e-08_rb,0.38383e-06_rb,0.55033e-06_rb,0.63844e-06_rb,0.16208e-07_rb /)
      kbo(:, 5,17, 1) = (/ &
        & 0.13468e-08_rb,0.41822e-06_rb,0.60950e-06_rb,0.72376e-06_rb,0.17995e-07_rb /)
      kbo(:, 1,18, 1) = (/ &
        & 0.14362e-08_rb,0.24560e-06_rb,0.31667e-06_rb,0.32797e-06_rb,0.96831e-08_rb /)
      kbo(:, 2,18, 1) = (/ &
        & 0.13327e-08_rb,0.26784e-06_rb,0.35872e-06_rb,0.39134e-06_rb,0.10773e-07_rb /)
      kbo(:, 3,18, 1) = (/ &
        & 0.12432e-08_rb,0.29008e-06_rb,0.40334e-06_rb,0.46026e-06_rb,0.11980e-07_rb /)
      kbo(:, 4,18, 1) = (/ &
        & 0.11650e-08_rb,0.31637e-06_rb,0.45583e-06_rb,0.52880e-06_rb,0.13350e-07_rb /)
      kbo(:, 5,18, 1) = (/ &
        & 0.10960e-08_rb,0.34503e-06_rb,0.50363e-06_rb,0.59967e-06_rb,0.14815e-07_rb /)
      kbo(:, 1,19, 1) = (/ &
        & 0.11667e-08_rb,0.20231e-06_rb,0.26313e-06_rb,0.27359e-06_rb,0.79934e-08_rb /)
      kbo(:, 2,19, 1) = (/ &
        & 0.10833e-08_rb,0.22085e-06_rb,0.29760e-06_rb,0.32620e-06_rb,0.88807e-08_rb /)
      kbo(:, 3,19, 1) = (/ &
        & 0.10110e-08_rb,0.23912e-06_rb,0.33423e-06_rb,0.38271e-06_rb,0.98771e-08_rb /)
      kbo(:, 4,19, 1) = (/ &
        & 0.94778e-09_rb,0.26073e-06_rb,0.37685e-06_rb,0.43791e-06_rb,0.11002e-07_rb /)
      kbo(:, 5,19, 1) = (/ &
        & 0.89198e-09_rb,0.28425e-06_rb,0.41589e-06_rb,0.49682e-06_rb,0.12204e-07_rb /)
      kbo(:, 1,20, 1) = (/ &
        & 0.94668e-09_rb,0.16735e-06_rb,0.21822e-06_rb,0.22914e-06_rb,0.66237e-08_rb /)
      kbo(:, 2,20, 1) = (/ &
        & 0.87956e-09_rb,0.18255e-06_rb,0.24770e-06_rb,0.27290e-06_rb,0.73470e-08_rb /)
      kbo(:, 3,20, 1) = (/ &
        & 0.82132e-09_rb,0.19776e-06_rb,0.27786e-06_rb,0.31913e-06_rb,0.81732e-08_rb /)
      kbo(:, 4,20, 1) = (/ &
        & 0.77032e-09_rb,0.21557e-06_rb,0.31197e-06_rb,0.36380e-06_rb,0.91019e-08_rb /)
      kbo(:, 5,20, 1) = (/ &
        & 0.72528e-09_rb,0.23458e-06_rb,0.34449e-06_rb,0.41257e-06_rb,0.10084e-07_rb /)
      kbo(:, 1,21, 1) = (/ &
        & 0.76809e-09_rb,0.13858e-06_rb,0.18138e-06_rb,0.19188e-06_rb,0.54929e-08_rb /)
      kbo(:, 2,21, 1) = (/ &
        & 0.71408e-09_rb,0.15088e-06_rb,0.20614e-06_rb,0.22835e-06_rb,0.60825e-08_rb /)
      kbo(:, 3,21, 1) = (/ &
        & 0.66717e-09_rb,0.16366e-06_rb,0.23111e-06_rb,0.26622e-06_rb,0.67680e-08_rb /)
      kbo(:, 4,21, 1) = (/ &
        & 0.62605e-09_rb,0.17816e-06_rb,0.25834e-06_rb,0.30242e-06_rb,0.75344e-08_rb /)
      kbo(:, 5,21, 1) = (/ &
        & 0.58970e-09_rb,0.19374e-06_rb,0.28542e-06_rb,0.34222e-06_rb,0.83394e-08_rb /)
      kbo(:, 1,22, 1) = (/ &
        & 0.62013e-09_rb,0.11547e-06_rb,0.15212e-06_rb,0.16227e-06_rb,0.45926e-08_rb /)
      kbo(:, 2,22, 1) = (/ &
        & 0.57709e-09_rb,0.12552e-06_rb,0.17211e-06_rb,0.19265e-06_rb,0.50728e-08_rb /)
      kbo(:, 3,22, 1) = (/ &
        & 0.53964e-09_rb,0.13619e-06_rb,0.19363e-06_rb,0.22402e-06_rb,0.56454e-08_rb /)
      kbo(:, 4,22, 1) = (/ &
        & 0.50675e-09_rb,0.14805e-06_rb,0.21557e-06_rb,0.25320e-06_rb,0.62805e-08_rb /)
      kbo(:, 5,22, 1) = (/ &
        & 0.47764e-09_rb,0.16097e-06_rb,0.23799e-06_rb,0.28574e-06_rb,0.69414e-08_rb /)
      kbo(:, 1,23, 1) = (/ &
        & 0.50055e-09_rb,0.96165e-07_rb,0.12776e-06_rb,0.13752e-06_rb,0.37933e-08_rb /)
      kbo(:, 2,23, 1) = (/ &
        & 0.46627e-09_rb,0.10421e-06_rb,0.14387e-06_rb,0.16226e-06_rb,0.42154e-08_rb /)
      kbo(:, 3,23, 1) = (/ &
        & 0.43638e-09_rb,0.11320e-06_rb,0.16240e-06_rb,0.18806e-06_rb,0.46761e-08_rb /)
      kbo(:, 4,23, 1) = (/ &
        & 0.41009e-09_rb,0.12312e-06_rb,0.18002e-06_rb,0.21214e-06_rb,0.51724e-08_rb /)
      kbo(:, 5,23, 1) = (/ &
        & 0.38679e-09_rb,0.13380e-06_rb,0.19835e-06_rb,0.23836e-06_rb,0.56863e-08_rb /)
      kbo(:, 1,24, 1) = (/ &
        & 0.40384e-09_rb,0.80174e-07_rb,0.10742e-06_rb,0.11665e-06_rb,0.31435e-08_rb /)
      kbo(:, 2,24, 1) = (/ &
        & 0.37656e-09_rb,0.86611e-07_rb,0.12062e-06_rb,0.13694e-06_rb,0.34746e-08_rb /)
      kbo(:, 3,24, 1) = (/ &
        & 0.35273e-09_rb,0.94203e-07_rb,0.13610e-06_rb,0.15729e-06_rb,0.38586e-08_rb /)
      kbo(:, 4,24, 1) = (/ &
        & 0.33174e-09_rb,0.10246e-06_rb,0.15035e-06_rb,0.17775e-06_rb,0.42707e-08_rb /)
      kbo(:, 5,24, 1) = (/ &
        & 0.31310e-09_rb,0.11122e-06_rb,0.16543e-06_rb,0.19917e-06_rb,0.47020e-08_rb /)
      kbo(:, 1,25, 1) = (/ &
        & 0.32562e-09_rb,0.66785e-07_rb,0.90447e-07_rb,0.99059e-07_rb,0.26134e-08_rb /)
      kbo(:, 2,25, 1) = (/ &
        & 0.30393e-09_rb,0.72133e-07_rb,0.10139e-06_rb,0.11571e-06_rb,0.28872e-08_rb /)
      kbo(:, 3,25, 1) = (/ &
        & 0.28495e-09_rb,0.78448e-07_rb,0.11385e-06_rb,0.13178e-06_rb,0.32061e-08_rb /)
      kbo(:, 4,25, 1) = (/ &
        & 0.26821e-09_rb,0.85339e-07_rb,0.12563e-06_rb,0.14914e-06_rb,0.35489e-08_rb /)
      kbo(:, 5,25, 1) = (/ &
        & 0.25332e-09_rb,0.92445e-07_rb,0.13814e-06_rb,0.16674e-06_rb,0.39060e-08_rb /)
      kbo(:, 1,26, 1) = (/ &
        & 0.26224e-09_rb,0.55680e-07_rb,0.76369e-07_rb,0.84400e-07_rb,0.21759e-08_rb /)
      kbo(:, 2,26, 1) = (/ &
        & 0.24504e-09_rb,0.60204e-07_rb,0.85423e-07_rb,0.97885e-07_rb,0.24118e-08_rb /)
      kbo(:, 3,26, 1) = (/ &
        & 0.22996e-09_rb,0.65477e-07_rb,0.95321e-07_rb,0.11093e-06_rb,0.26774e-08_rb /)
      kbo(:, 4,26, 1) = (/ &
        & 0.21663e-09_rb,0.71188e-07_rb,0.10529e-06_rb,0.12544e-06_rb,0.29615e-08_rb /)
      kbo(:, 5,26, 1) = (/ &
        & 0.20475e-09_rb,0.76940e-07_rb,0.11556e-06_rb,0.13977e-06_rb,0.32569e-08_rb /)
      kbo(:, 1,27, 1) = (/ &
        & 0.21120e-09_rb,0.46382e-07_rb,0.63894e-07_rb,0.71379e-07_rb,0.18178e-08_rb /)
      kbo(:, 2,27, 1) = (/ &
        & 0.19756e-09_rb,0.50278e-07_rb,0.72033e-07_rb,0.82909e-07_rb,0.20181e-08_rb /)
      kbo(:, 3,27, 1) = (/ &
        & 0.18557e-09_rb,0.54670e-07_rb,0.79953e-07_rb,0.93505e-07_rb,0.22396e-08_rb /)
      kbo(:, 4,27, 1) = (/ &
        & 0.17496e-09_rb,0.59408e-07_rb,0.88179e-07_rb,0.10553e-06_rb,0.24737e-08_rb /)
      kbo(:, 5,27, 1) = (/ &
        & 0.16549e-09_rb,0.64025e-07_rb,0.96696e-07_rb,0.11704e-06_rb,0.27092e-08_rb /)
      kbo(:, 1,28, 1) = (/ &
        & 0.17010e-09_rb,0.38631e-07_rb,0.53773e-07_rb,0.60579e-07_rb,0.15220e-08_rb /)
      kbo(:, 2,28, 1) = (/ &
        & 0.15928e-09_rb,0.42001e-07_rb,0.60660e-07_rb,0.69727e-07_rb,0.16901e-08_rb /)
      kbo(:, 3,28, 1) = (/ &
        & 0.14976e-09_rb,0.45661e-07_rb,0.67065e-07_rb,0.78845e-07_rb,0.18696e-08_rb /)
      kbo(:, 4,28, 1) = (/ &
        & 0.14131e-09_rb,0.49571e-07_rb,0.73809e-07_rb,0.88481e-07_rb,0.20611e-08_rb /)
      kbo(:, 5,28, 1) = (/ &
        & 0.13376e-09_rb,0.53304e-07_rb,0.80910e-07_rb,0.98017e-07_rb,0.22565e-08_rb /)
      kbo(:, 1,29, 1) = (/ &
        & 0.13696e-09_rb,0.32270e-07_rb,0.45392e-07_rb,0.51438e-07_rb,0.12721e-08_rb /)
      kbo(:, 2,29, 1) = (/ &
        & 0.12839e-09_rb,0.35079e-07_rb,0.50946e-07_rb,0.58726e-07_rb,0.14118e-08_rb /)
      kbo(:, 3,29, 1) = (/ &
        & 0.12082e-09_rb,0.38150e-07_rb,0.56258e-07_rb,0.66493e-07_rb,0.15628e-08_rb /)
      kbo(:, 4,29, 1) = (/ &
        & 0.11410e-09_rb,0.41324e-07_rb,0.61843e-07_rb,0.74387e-07_rb,0.17205e-08_rb /)
      kbo(:, 5,29, 1) = (/ &
        & 0.10808e-09_rb,0.44387e-07_rb,0.67739e-07_rb,0.82216e-07_rb,0.18745e-08_rb /)
      kbo(:, 1,30, 1) = (/ &
        & 0.11029e-09_rb,0.26970e-07_rb,0.38330e-07_rb,0.43698e-07_rb,0.10655e-08_rb /)
      kbo(:, 2,30, 1) = (/ &
        & 0.10349e-09_rb,0.29319e-07_rb,0.42739e-07_rb,0.49552e-07_rb,0.11823e-08_rb /)
      kbo(:, 3,30, 1) = (/ &
        & 0.97483e-10_rb,0.31869e-07_rb,0.47210e-07_rb,0.56057e-07_rb,0.13038e-08_rb /)
      kbo(:, 4,30, 1) = (/ &
        & 0.92133e-10_rb,0.34436e-07_rb,0.51812e-07_rb,0.62485e-07_rb,0.14285e-08_rb /)
      kbo(:, 5,30, 1) = (/ &
        & 0.87340e-10_rb,0.36953e-07_rb,0.56709e-07_rb,0.68921e-07_rb,0.15563e-08_rb /)
      kbo(:, 1,31, 1) = (/ &
        & 0.88806e-10_rb,0.22555e-07_rb,0.32386e-07_rb,0.37129e-07_rb,0.89029e-09_rb /)
      kbo(:, 2,31, 1) = (/ &
        & 0.83417e-10_rb,0.24519e-07_rb,0.35921e-07_rb,0.41870e-07_rb,0.98548e-09_rb /)
      kbo(:, 3,31, 1) = (/ &
        & 0.78644e-10_rb,0.26638e-07_rb,0.39609e-07_rb,0.47273e-07_rb,0.10854e-08_rb /)
      kbo(:, 4,31, 1) = (/ &
        & 0.74388e-10_rb,0.28694e-07_rb,0.43423e-07_rb,0.52435e-07_rb,0.11880e-08_rb /)
      kbo(:, 5,31, 1) = (/ &
        & 0.70568e-10_rb,0.30776e-07_rb,0.47411e-07_rb,0.57750e-07_rb,0.12931e-08_rb /)
      kbo(:, 1,32, 1) = (/ &
        & 0.71507e-10_rb,0.18870e-07_rb,0.27332e-07_rb,0.31309e-07_rb,0.74436e-09_rb /)
      kbo(:, 2,32, 1) = (/ &
        & 0.67235e-10_rb,0.20512e-07_rb,0.30189e-07_rb,0.35407e-07_rb,0.82294e-09_rb /)
      kbo(:, 3,32, 1) = (/ &
        & 0.63444e-10_rb,0.22259e-07_rb,0.33218e-07_rb,0.39730e-07_rb,0.90483e-09_rb /)
      kbo(:, 4,32, 1) = (/ &
        & 0.60058e-10_rb,0.23923e-07_rb,0.36405e-07_rb,0.43976e-07_rb,0.98908e-09_rb /)
      kbo(:, 5,32, 1) = (/ &
        & 0.57015e-10_rb,0.25634e-07_rb,0.39548e-07_rb,0.48346e-07_rb,0.10750e-08_rb /)
      kbo(:, 1,33, 1) = (/ &
        & 0.57586e-10_rb,0.15779e-07_rb,0.22950e-07_rb,0.26415e-07_rb,0.62256e-09_rb /)
      kbo(:, 2,33, 1) = (/ &
        & 0.54199e-10_rb,0.17153e-07_rb,0.25354e-07_rb,0.29904e-07_rb,0.68734e-09_rb /)
      kbo(:, 3,33, 1) = (/ &
        & 0.51188e-10_rb,0.18569e-07_rb,0.27869e-07_rb,0.33404e-07_rb,0.75467e-09_rb /)
      kbo(:, 4,33, 1) = (/ &
        & 0.48493e-10_rb,0.19935e-07_rb,0.30437e-07_rb,0.36856e-07_rb,0.82369e-09_rb /)
      kbo(:, 5,33, 1) = (/ &
        & 0.46069e-10_rb,0.21345e-07_rb,0.32988e-07_rb,0.40478e-07_rb,0.89402e-09_rb /)
      kbo(:, 1,34, 1) = (/ &
        & 0.46487e-10_rb,0.13161e-07_rb,0.19216e-07_rb,0.22212e-07_rb,0.51910e-09_rb /)
      kbo(:, 2,34, 1) = (/ &
        & 0.43789e-10_rb,0.14301e-07_rb,0.21182e-07_rb,0.25090e-07_rb,0.57247e-09_rb /)
      kbo(:, 3,34, 1) = (/ &
        & 0.41386e-10_rb,0.15447e-07_rb,0.23208e-07_rb,0.27938e-07_rb,0.62767e-09_rb /)
      kbo(:, 4,34, 1) = (/ &
        & 0.39234e-10_rb,0.16571e-07_rb,0.25347e-07_rb,0.30795e-07_rb,0.68426e-09_rb /)
      kbo(:, 5,34, 1) = (/ &
        & 0.37294e-10_rb,0.17725e-07_rb,0.27392e-07_rb,0.33781e-07_rb,0.74179e-09_rb /)
      kbo(:, 1,35, 1) = (/ &
        & 0.37756e-10_rb,0.10890e-07_rb,0.15882e-07_rb,0.18441e-07_rb,0.42871e-09_rb /)
      kbo(:, 2,35, 1) = (/ &
        & 0.35581e-10_rb,0.11830e-07_rb,0.17514e-07_rb,0.20810e-07_rb,0.47251e-09_rb /)
      kbo(:, 3,35, 1) = (/ &
        & 0.33643e-10_rb,0.12757e-07_rb,0.19184e-07_rb,0.23148e-07_rb,0.51769e-09_rb /)
      kbo(:, 4,35, 1) = (/ &
        & 0.31905e-10_rb,0.13682e-07_rb,0.20941e-07_rb,0.25477e-07_rb,0.56394e-09_rb /)
      kbo(:, 5,35, 1) = (/ &
        & 0.30338e-10_rb,0.14628e-07_rb,0.22613e-07_rb,0.27948e-07_rb,0.61100e-09_rb /)
      kbo(:, 1,36, 1) = (/ &
        & 0.30872e-10_rb,0.89290e-08_rb,0.13007e-07_rb,0.15108e-07_rb,0.35034e-09_rb /)
      kbo(:, 2,36, 1) = (/ &
        & 0.29096e-10_rb,0.96995e-08_rb,0.14343e-07_rb,0.17051e-07_rb,0.38609e-09_rb /)
      kbo(:, 3,36, 1) = (/ &
        & 0.27513e-10_rb,0.10456e-07_rb,0.15713e-07_rb,0.18968e-07_rb,0.42296e-09_rb /)
      kbo(:, 4,36, 1) = (/ &
        & 0.26093e-10_rb,0.11213e-07_rb,0.17153e-07_rb,0.20875e-07_rb,0.46073e-09_rb /)
      kbo(:, 5,36, 1) = (/ &
        & 0.24812e-10_rb,0.11987e-07_rb,0.18522e-07_rb,0.22899e-07_rb,0.49916e-09_rb /)
      kbo(:, 1,37, 1) = (/ &
        & 0.25479e-10_rb,0.72338e-08_rb,0.10508e-07_rb,0.12165e-07_rb,0.28268e-09_rb /)
      kbo(:, 2,37, 1) = (/ &
        & 0.24002e-10_rb,0.78593e-08_rb,0.11594e-07_rb,0.13747e-07_rb,0.31173e-09_rb /)
      kbo(:, 3,37, 1) = (/ &
        & 0.22687e-10_rb,0.84845e-08_rb,0.12712e-07_rb,0.15315e-07_rb,0.34178e-09_rb /)
      kbo(:, 4,37, 1) = (/ &
        & 0.21508e-10_rb,0.90997e-08_rb,0.13892e-07_rb,0.16886e-07_rb,0.37260e-09_rb /)
      kbo(:, 5,37, 1) = (/ &
        & 0.20446e-10_rb,0.97322e-08_rb,0.15012e-07_rb,0.18526e-07_rb,0.40398e-09_rb /)
      kbo(:, 1,38, 1) = (/ &
        & 0.21041e-10_rb,0.58565e-08_rb,0.84831e-08_rb,0.97892e-08_rb,0.22791e-09_rb /)
      kbo(:, 2,38, 1) = (/ &
        & 0.19811e-10_rb,0.63636e-08_rb,0.93619e-08_rb,0.11072e-07_rb,0.25148e-09_rb /)
      kbo(:, 3,38, 1) = (/ &
        & 0.18717e-10_rb,0.68784e-08_rb,0.10276e-07_rb,0.12360e-07_rb,0.27595e-09_rb /)
      kbo(:, 4,38, 1) = (/ &
        & 0.17738e-10_rb,0.73792e-08_rb,0.11234e-07_rb,0.13638e-07_rb,0.30109e-09_rb /)
      kbo(:, 5,38, 1) = (/ &
        & 0.16856e-10_rb,0.78971e-08_rb,0.12166e-07_rb,0.14977e-07_rb,0.32671e-09_rb /)
      kbo(:, 1,39, 1) = (/ &
        & 0.17377e-10_rb,0.47418e-08_rb,0.68561e-08_rb,0.78795e-08_rb,0.18373e-09_rb /)
      kbo(:, 2,39, 1) = (/ &
        & 0.16353e-10_rb,0.51532e-08_rb,0.75620e-08_rb,0.89128e-08_rb,0.20284e-09_rb /)
      kbo(:, 3,39, 1) = (/ &
        & 0.15443e-10_rb,0.55752e-08_rb,0.83069e-08_rb,0.99666e-08_rb,0.22279e-09_rb /)
      kbo(:, 4,39, 1) = (/ &
        & 0.14629e-10_rb,0.59857e-08_rb,0.90872e-08_rb,0.11011e-07_rb,0.24327e-09_rb /)
      kbo(:, 5,39, 1) = (/ &
        & 0.13896e-10_rb,0.64086e-08_rb,0.98616e-08_rb,0.12104e-07_rb,0.26420e-09_rb /)
      kbo(:, 1,40, 1) = (/ &
        & 0.14445e-10_rb,0.38091e-08_rb,0.54799e-08_rb,0.62754e-08_rb,0.14679e-09_rb /)
      kbo(:, 2,40, 1) = (/ &
        & 0.13581e-10_rb,0.41391e-08_rb,0.60530e-08_rb,0.70909e-08_rb,0.16220e-09_rb /)
      kbo(:, 3,40, 1) = (/ &
        & 0.12815e-10_rb,0.44888e-08_rb,0.66555e-08_rb,0.79532e-08_rb,0.17842e-09_rb /)
      kbo(:, 4,40, 1) = (/ &
        & 0.12131e-10_rb,0.48207e-08_rb,0.72855e-08_rb,0.88071e-08_rb,0.19509e-09_rb /)
      kbo(:, 5,40, 1) = (/ &
        & 0.11516e-10_rb,0.51646e-08_rb,0.79268e-08_rb,0.96932e-08_rb,0.21214e-09_rb /)
      kbo(:, 1,41, 1) = (/ &
        & 0.12021e-10_rb,0.30566e-08_rb,0.43606e-08_rb,0.49921e-08_rb,0.11713e-09_rb /)
      kbo(:, 2,41, 1) = (/ &
        & 0.11291e-10_rb,0.33205e-08_rb,0.48340e-08_rb,0.56256e-08_rb,0.12954e-09_rb /)
      kbo(:, 3,41, 1) = (/ &
        & 0.10645e-10_rb,0.36053e-08_rb,0.53261e-08_rb,0.63473e-08_rb,0.14269e-09_rb /)
      kbo(:, 4,41, 1) = (/ &
        & 0.10069e-10_rb,0.38782e-08_rb,0.58347e-08_rb,0.70386e-08_rb,0.15624e-09_rb /)
      kbo(:, 5,41, 1) = (/ &
        & 0.95520e-11_rb,0.41580e-08_rb,0.63647e-08_rb,0.77515e-08_rb,0.17013e-09_rb /)
      kbo(:, 1,42, 1) = (/ &
        & 0.10007e-10_rb,0.24516e-08_rb,0.34653e-08_rb,0.39398e-08_rb,0.93475e-10_rb /)
      kbo(:, 2,42, 1) = (/ &
        & 0.93902e-11_rb,0.26631e-08_rb,0.38588e-08_rb,0.44635e-08_rb,0.10342e-09_rb /)
      kbo(:, 3,42, 1) = (/ &
        & 0.88449e-11_rb,0.28926e-08_rb,0.42600e-08_rb,0.50466e-08_rb,0.11406e-09_rb /)
      kbo(:, 4,42, 1) = (/ &
        & 0.83594e-11_rb,0.31198e-08_rb,0.46719e-08_rb,0.56218e-08_rb,0.12507e-09_rb /)
      kbo(:, 5,42, 1) = (/ &
        & 0.79244e-11_rb,0.33465e-08_rb,0.51065e-08_rb,0.61999e-08_rb,0.13640e-09_rb /)
      kbo(:, 1,43, 1) = (/ &
        & 0.83647e-11_rb,0.19578e-08_rb,0.27341e-08_rb,0.30827e-08_rb,0.74277e-10_rb /)
      kbo(:, 2,43, 1) = (/ &
        & 0.78389e-11_rb,0.21263e-08_rb,0.30690e-08_rb,0.35212e-08_rb,0.82118e-10_rb /)
      kbo(:, 3,43, 1) = (/ &
        & 0.73752e-11_rb,0.23103e-08_rb,0.33865e-08_rb,0.39836e-08_rb,0.90672e-10_rb /)
      kbo(:, 4,43, 1) = (/ &
        & 0.69633e-11_rb,0.24984e-08_rb,0.37198e-08_rb,0.44569e-08_rb,0.99628e-10_rb /)
      kbo(:, 5,43, 1) = (/ &
        & 0.65950e-11_rb,0.26833e-08_rb,0.40709e-08_rb,0.49265e-08_rb,0.10883e-09_rb /)
      kbo(:, 1,44, 1) = (/ &
        & 0.70070e-11_rb,0.15628e-08_rb,0.21521e-08_rb,0.24017e-08_rb,0.59116e-10_rb /)
      kbo(:, 2,44, 1) = (/ &
        & 0.65570e-11_rb,0.16954e-08_rb,0.24254e-08_rb,0.27772e-08_rb,0.65027e-10_rb /)
      kbo(:, 3,44, 1) = (/ &
        & 0.61612e-11_rb,0.18413e-08_rb,0.26845e-08_rb,0.31310e-08_rb,0.71892e-10_rb /)
      kbo(:, 4,44, 1) = (/ &
        & 0.58106e-11_rb,0.19977e-08_rb,0.29542e-08_rb,0.35211e-08_rb,0.79144e-10_rb /)
      kbo(:, 5,44, 1) = (/ &
        & 0.54976e-11_rb,0.21464e-08_rb,0.32360e-08_rb,0.39045e-08_rb,0.86629e-10_rb /)
      kbo(:, 1,45, 1) = (/ &
        & 0.58728e-11_rb,0.12500e-08_rb,0.17079e-08_rb,0.18743e-08_rb,0.46646e-10_rb /)
      kbo(:, 2,45, 1) = (/ &
        & 0.54873e-11_rb,0.13505e-08_rb,0.19105e-08_rb,0.21719e-08_rb,0.51499e-10_rb /)
      kbo(:, 3,45, 1) = (/ &
        & 0.51492e-11_rb,0.14670e-08_rb,0.21250e-08_rb,0.24590e-08_rb,0.56982e-10_rb /)
      kbo(:, 4,45, 1) = (/ &
        & 0.48504e-11_rb,0.15931e-08_rb,0.23464e-08_rb,0.27802e-08_rb,0.62839e-10_rb /)
      kbo(:, 5,45, 1) = (/ &
        & 0.45844e-11_rb,0.17167e-08_rb,0.25731e-08_rb,0.30954e-08_rb,0.68908e-10_rb /)
      kbo(:, 1,46, 1) = (/ &
        & 0.49328e-11_rb,0.99811e-09_rb,0.13397e-08_rb,0.14457e-08_rb,0.36694e-10_rb /)
      kbo(:, 2,46, 1) = (/ &
        & 0.46012e-11_rb,0.10742e-08_rb,0.14986e-08_rb,0.16875e-08_rb,0.40742e-10_rb /)
      kbo(:, 3,46, 1) = (/ &
        & 0.43113e-11_rb,0.11665e-08_rb,0.16813e-08_rb,0.19281e-08_rb,0.45049e-10_rb /)
      kbo(:, 4,46, 1) = (/ &
        & 0.40558e-11_rb,0.12673e-08_rb,0.18567e-08_rb,0.21817e-08_rb,0.49751e-10_rb /)
      kbo(:, 5,46, 1) = (/ &
        & 0.38288e-11_rb,0.13702e-08_rb,0.20399e-08_rb,0.24419e-08_rb,0.54679e-10_rb /)
      kbo(:, 1,47, 1) = (/ &
        & 0.41592e-11_rb,0.79205e-09_rb,0.10438e-08_rb,0.11055e-08_rb,0.28709e-10_rb /)
      kbo(:, 2,47, 1) = (/ &
        & 0.38717e-11_rb,0.85357e-09_rb,0.11710e-08_rb,0.13015e-08_rb,0.32198e-10_rb /)
      kbo(:, 3,47, 1) = (/ &
        & 0.36215e-11_rb,0.92515e-09_rb,0.13196e-08_rb,0.15103e-08_rb,0.35448e-10_rb /)
      kbo(:, 4,47, 1) = (/ &
        & 0.34016e-11_rb,0.10046e-08_rb,0.14619e-08_rb,0.17010e-08_rb,0.39203e-10_rb /)
      kbo(:, 5,47, 1) = (/ &
        & 0.32069e-11_rb,0.10904e-08_rb,0.16100e-08_rb,0.19168e-08_rb,0.43187e-10_rb /)
      kbo(:, 1,48, 1) = (/ &
        & 0.35100e-11_rb,0.62849e-09_rb,0.81319e-09_rb,0.84567e-09_rb,0.22456e-10_rb /)
      kbo(:, 2,48, 1) = (/ &
        & 0.32605e-11_rb,0.68021e-09_rb,0.92169e-09_rb,0.10040e-08_rb,0.25229e-10_rb /)
      kbo(:, 3,48, 1) = (/ &
        & 0.30441e-11_rb,0.73294e-09_rb,0.10314e-08_rb,0.11665e-08_rb,0.27903e-10_rb /)
      kbo(:, 4,48, 1) = (/ &
        & 0.28547e-11_rb,0.79615e-09_rb,0.11495e-08_rb,0.13251e-08_rb,0.30879e-10_rb /)
      kbo(:, 5,48, 1) = (/ &
        & 0.26874e-11_rb,0.86463e-09_rb,0.12703e-08_rb,0.14994e-08_rb,0.34082e-10_rb /)
      kbo(:, 1,49, 1) = (/ &
        & 0.29649e-11_rb,0.49867e-09_rb,0.63572e-09_rb,0.64557e-09_rb,0.17554e-10_rb /)
      kbo(:, 2,49, 1) = (/ &
        & 0.27480e-11_rb,0.54124e-09_rb,0.71871e-09_rb,0.76840e-09_rb,0.19749e-10_rb /)
      kbo(:, 3,49, 1) = (/ &
        & 0.25606e-11_rb,0.58140e-09_rb,0.80413e-09_rb,0.90054e-09_rb,0.22012e-10_rb /)
      kbo(:, 4,49, 1) = (/ &
        & 0.23972e-11_rb,0.63097e-09_rb,0.90637e-09_rb,0.10349e-08_rb,0.24308e-10_rb /)
      kbo(:, 5,49, 1) = (/ &
        & 0.22534e-11_rb,0.68545e-09_rb,0.10006e-08_rb,0.11704e-08_rb,0.26872e-10_rb /)
      kbo(:, 1,50, 1) = (/ &
        & 0.25019e-11_rb,0.39853e-09_rb,0.49374e-09_rb,0.49615e-09_rb,0.13762e-10_rb /)
      kbo(:, 2,50, 1) = (/ &
        & 0.23136e-11_rb,0.42952e-09_rb,0.56141e-09_rb,0.58980e-09_rb,0.15500e-10_rb /)
      kbo(:, 3,50, 1) = (/ &
        & 0.21517e-11_rb,0.46334e-09_rb,0.63284e-09_rb,0.69788e-09_rb,0.17390e-10_rb /)
      kbo(:, 4,50, 1) = (/ &
        & 0.20110e-11_rb,0.50130e-09_rb,0.71124e-09_rb,0.80903e-09_rb,0.19184e-10_rb /)
      kbo(:, 5,50, 1) = (/ &
        & 0.18875e-11_rb,0.54446e-09_rb,0.78908e-09_rb,0.91435e-09_rb,0.21230e-10_rb /)
      kbo(:, 1,51, 1) = (/ &
        & 0.21111e-11_rb,0.31534e-09_rb,0.38552e-09_rb,0.38324e-09_rb,0.10851e-10_rb /)
      kbo(:, 2,51, 1) = (/ &
        & 0.19478e-11_rb,0.34179e-09_rb,0.43986e-09_rb,0.45355e-09_rb,0.12176e-10_rb /)
      kbo(:, 3,51, 1) = (/ &
        & 0.18079e-11_rb,0.37016e-09_rb,0.49835e-09_rb,0.53919e-09_rb,0.13683e-10_rb /)
      kbo(:, 4,51, 1) = (/ &
        & 0.16868e-11_rb,0.39846e-09_rb,0.55746e-09_rb,0.62805e-09_rb,0.15164e-10_rb /)
      kbo(:, 5,51, 1) = (/ &
        & 0.15809e-11_rb,0.43272e-09_rb,0.62310e-09_rb,0.71587e-09_rb,0.16785e-10_rb /)
      kbo(:, 1,52, 1) = (/ &
        & 0.17830e-11_rb,0.24939e-09_rb,0.30033e-09_rb,0.29708e-09_rb,0.85480e-11_rb /)
      kbo(:, 2,52, 1) = (/ &
        & 0.16410e-11_rb,0.27244e-09_rb,0.34467e-09_rb,0.34870e-09_rb,0.95642e-11_rb /)
      kbo(:, 3,52, 1) = (/ &
        & 0.15200e-11_rb,0.29507e-09_rb,0.39020e-09_rb,0.41505e-09_rb,0.10764e-10_rb /)
      kbo(:, 4,52, 1) = (/ &
        & 0.14156e-11_rb,0.31736e-09_rb,0.43693e-09_rb,0.48758e-09_rb,0.12016e-10_rb /)
      kbo(:, 5,52, 1) = (/ &
        & 0.13246e-11_rb,0.34409e-09_rb,0.49243e-09_rb,0.56181e-09_rb,0.13267e-10_rb /)
      kbo(:, 1,53, 1) = (/ &
        & 0.15074e-11_rb,0.19746e-09_rb,0.23442e-09_rb,0.23027e-09_rb,0.66942e-11_rb /)
      kbo(:, 2,53, 1) = (/ &
        & 0.13838e-11_rb,0.21754e-09_rb,0.26812e-09_rb,0.26865e-09_rb,0.75095e-11_rb /)
      kbo(:, 3,53, 1) = (/ &
        & 0.12789e-11_rb,0.23445e-09_rb,0.30510e-09_rb,0.31916e-09_rb,0.84626e-11_rb /)
      kbo(:, 4,53, 1) = (/ &
        & 0.11888e-11_rb,0.25305e-09_rb,0.34496e-09_rb,0.37906e-09_rb,0.94963e-11_rb /)
      kbo(:, 5,53, 1) = (/ &
        & 0.11106e-11_rb,0.27355e-09_rb,0.38680e-09_rb,0.43882e-09_rb,0.10486e-10_rb /)
      kbo(:, 1,54, 1) = (/ &
        & 0.12722e-11_rb,0.15676e-09_rb,0.18436e-09_rb,0.18188e-09_rb,0.52601e-11_rb /)
      kbo(:, 2,54, 1) = (/ &
        & 0.11649e-11_rb,0.17256e-09_rb,0.21030e-09_rb,0.20876e-09_rb,0.59465e-11_rb /)
      kbo(:, 3,54, 1) = (/ &
        & 0.10743e-11_rb,0.18706e-09_rb,0.24022e-09_rb,0.24657e-09_rb,0.66750e-11_rb /)
      kbo(:, 4,54, 1) = (/ &
        & 0.99683e-12_rb,0.20263e-09_rb,0.27206e-09_rb,0.29337e-09_rb,0.75016e-11_rb /)
      kbo(:, 5,54, 1) = (/ &
        & 0.92975e-12_rb,0.21798e-09_rb,0.30420e-09_rb,0.34203e-09_rb,0.83182e-11_rb /)
      kbo(:, 1,55, 1) = (/ &
        & 0.10733e-11_rb,0.12552e-09_rb,0.14668e-09_rb,0.14477e-09_rb,0.41440e-11_rb /)
      kbo(:, 2,55, 1) = (/ &
        & 0.98027e-12_rb,0.13705e-09_rb,0.16480e-09_rb,0.16298e-09_rb,0.47137e-11_rb /)
      kbo(:, 3,55, 1) = (/ &
        & 0.90209e-12_rb,0.14966e-09_rb,0.18911e-09_rb,0.19117e-09_rb,0.52730e-11_rb /)
      kbo(:, 4,55, 1) = (/ &
        & 0.83546e-12_rb,0.16210e-09_rb,0.21410e-09_rb,0.22758e-09_rb,0.59349e-11_rb /)
      kbo(:, 5,55, 1) = (/ &
        & 0.77800e-12_rb,0.17423e-09_rb,0.23972e-09_rb,0.26725e-09_rb,0.66163e-11_rb /)
      kbo(:, 1,56, 1) = (/ &
        & 0.90624e-12_rb,0.99786e-10_rb,0.11674e-09_rb,0.11286e-09_rb,0.32776e-11_rb /)
      kbo(:, 2,56, 1) = (/ &
        & 0.82547e-12_rb,0.10897e-09_rb,0.12938e-09_rb,0.12705e-09_rb,0.37161e-11_rb /)
      kbo(:, 3,56, 1) = (/ &
        & 0.75791e-12_rb,0.12009e-09_rb,0.14804e-09_rb,0.14827e-09_rb,0.41646e-11_rb /)
      kbo(:, 4,56, 1) = (/ &
        & 0.70058e-12_rb,0.12925e-09_rb,0.16837e-09_rb,0.17630e-09_rb,0.46957e-11_rb /)
      kbo(:, 5,56, 1) = (/ &
        & 0.65131e-12_rb,0.13937e-09_rb,0.18999e-09_rb,0.20899e-09_rb,0.52742e-11_rb /)
      kbo(:, 1,57, 1) = (/ &
        & 0.76595e-12_rb,0.78822e-10_rb,0.93863e-10_rb,0.87529e-10_rb,0.26033e-11_rb /)
      kbo(:, 2,57, 1) = (/ &
        & 0.69568e-12_rb,0.86555e-10_rb,0.10190e-09_rb,0.10022e-09_rb,0.29288e-11_rb /)
      kbo(:, 3,57, 1) = (/ &
        & 0.63721e-12_rb,0.95385e-10_rb,0.11640e-09_rb,0.11533e-09_rb,0.32994e-11_rb /)
      kbo(:, 4,57, 1) = (/ &
        & 0.58782e-12_rb,0.10325e-09_rb,0.13264e-09_rb,0.13663e-09_rb,0.37186e-11_rb /)
      kbo(:, 5,57, 1) = (/ &
        & 0.54552e-12_rb,0.11173e-09_rb,0.15042e-09_rb,0.16253e-09_rb,0.41819e-11_rb /)
      kbo(:, 1,58, 1) = (/ &
        & 0.64693e-12_rb,0.62421e-10_rb,0.76174e-10_rb,0.68324e-10_rb,0.20833e-11_rb /)
      kbo(:, 2,58, 1) = (/ &
        & 0.58588e-12_rb,0.69278e-10_rb,0.81197e-10_rb,0.79691e-10_rb,0.23130e-11_rb /)
      kbo(:, 3,58, 1) = (/ &
        & 0.53535e-12_rb,0.75873e-10_rb,0.91414e-10_rb,0.90398e-10_rb,0.26350e-11_rb /)
      kbo(:, 4,58, 1) = (/ &
        & 0.49285e-12_rb,0.82653e-10_rb,0.10493e-09_rb,0.10617e-09_rb,0.29473e-11_rb /)
      kbo(:, 5,58, 1) = (/ &
        & 0.45660e-12_rb,0.89643e-10_rb,0.11861e-09_rb,0.12647e-09_rb,0.33231e-11_rb /)
      kbo(:, 1,59, 1) = (/ &
        & 0.53645e-12_rb,0.50557e-10_rb,0.62240e-10_rb,0.54854e-10_rb,0.16930e-11_rb /)
      kbo(:, 2,59, 1) = (/ &
        & 0.48523e-12_rb,0.56468e-10_rb,0.65604e-10_rb,0.64827e-10_rb,0.18702e-11_rb /)
      kbo(:, 3,59, 1) = (/ &
        & 0.44294e-12_rb,0.61442e-10_rb,0.73667e-10_rb,0.72688e-10_rb,0.21292e-11_rb /)
      kbo(:, 4,59, 1) = (/ &
        & 0.40744e-12_rb,0.67184e-10_rb,0.84490e-10_rb,0.85217e-10_rb,0.23832e-11_rb /)
      kbo(:, 5,59, 1) = (/ &
        & 0.37720e-12_rb,0.72657e-10_rb,0.95692e-10_rb,0.10144e-09_rb,0.26852e-11_rb /)
      kbo(:, 1,13, 2) = (/ &
        & 0.18323e-07_rb,0.86266e-05_rb,0.13728e-04_rb,0.15952e-04_rb,0.78927e-06_rb /)
      kbo(:, 2,13, 2) = (/ &
        & 0.16961e-07_rb,0.96203e-05_rb,0.15275e-04_rb,0.17863e-04_rb,0.93659e-06_rb /)
      kbo(:, 3,13, 2) = (/ &
        & 0.15787e-07_rb,0.10675e-04_rb,0.16924e-04_rb,0.19675e-04_rb,0.11050e-05_rb /)
      kbo(:, 4,13, 2) = (/ &
        & 0.14764e-07_rb,0.11779e-04_rb,0.18474e-04_rb,0.21320e-04_rb,0.12868e-05_rb /)
      kbo(:, 5,13, 2) = (/ &
        & 0.13865e-07_rb,0.12843e-04_rb,0.20029e-04_rb,0.23080e-04_rb,0.14565e-05_rb /)
      kbo(:, 1,14, 2) = (/ &
        & 0.14893e-07_rb,0.71882e-05_rb,0.11443e-04_rb,0.13271e-04_rb,0.65835e-06_rb /)
      kbo(:, 2,14, 2) = (/ &
        & 0.13793e-07_rb,0.80276e-05_rb,0.12732e-04_rb,0.14890e-04_rb,0.78132e-06_rb /)
      kbo(:, 3,14, 2) = (/ &
        & 0.12844e-07_rb,0.89189e-05_rb,0.14101e-04_rb,0.16339e-04_rb,0.92233e-06_rb /)
      kbo(:, 4,14, 2) = (/ &
        & 0.12017e-07_rb,0.98309e-05_rb,0.15384e-04_rb,0.17721e-04_rb,0.10676e-05_rb /)
      kbo(:, 5,14, 2) = (/ &
        & 0.11289e-07_rb,0.10704e-04_rb,0.16679e-04_rb,0.19194e-04_rb,0.12091e-05_rb /)
      kbo(:, 1,15, 2) = (/ &
        & 0.12096e-07_rb,0.59934e-05_rb,0.95449e-05_rb,0.11058e-04_rb,0.54969e-06_rb /)
      kbo(:, 2,15, 2) = (/ &
        & 0.11209e-07_rb,0.66932e-05_rb,0.10623e-04_rb,0.12407e-04_rb,0.65197e-06_rb /)
      kbo(:, 3,15, 2) = (/ &
        & 0.10443e-07_rb,0.74449e-05_rb,0.11746e-04_rb,0.13571e-04_rb,0.76940e-06_rb /)
      kbo(:, 4,15, 2) = (/ &
        & 0.97747e-08_rb,0.81948e-05_rb,0.12814e-04_rb,0.14732e-04_rb,0.88496e-06_rb /)
      kbo(:, 5,15, 2) = (/ &
        & 0.91864e-08_rb,0.89188e-05_rb,0.13883e-04_rb,0.15946e-04_rb,0.10023e-05_rb /)
      kbo(:, 1,16, 2) = (/ &
        & 0.98205e-08_rb,0.50027e-05_rb,0.79644e-05_rb,0.92248e-05_rb,0.45876e-06_rb /)
      kbo(:, 2,16, 2) = (/ &
        & 0.91061e-08_rb,0.55846e-05_rb,0.88593e-05_rb,0.10331e-04_rb,0.54334e-06_rb /)
      kbo(:, 3,16, 2) = (/ &
        & 0.84882e-08_rb,0.62183e-05_rb,0.97757e-05_rb,0.11279e-04_rb,0.64050e-06_rb /)
      kbo(:, 4,16, 2) = (/ &
        & 0.79486e-08_rb,0.68276e-05_rb,0.10664e-04_rb,0.12244e-04_rb,0.73344e-06_rb /)
      kbo(:, 5,16, 2) = (/ &
        & 0.74732e-08_rb,0.74268e-05_rb,0.11545e-04_rb,0.13224e-04_rb,0.82955e-06_rb /)
      kbo(:, 1,17, 2) = (/ &
        & 0.79753e-08_rb,0.41713e-05_rb,0.66153e-05_rb,0.76892e-05_rb,0.38247e-06_rb /)
      kbo(:, 2,17, 2) = (/ &
        & 0.73994e-08_rb,0.46562e-05_rb,0.73753e-05_rb,0.85922e-05_rb,0.45188e-06_rb /)
      kbo(:, 3,17, 2) = (/ &
        & 0.69008e-08_rb,0.51829e-05_rb,0.81280e-05_rb,0.93638e-05_rb,0.53211e-06_rb /)
      kbo(:, 4,17, 2) = (/ &
        & 0.64650e-08_rb,0.56765e-05_rb,0.88560e-05_rb,0.10168e-04_rb,0.60647e-06_rb /)
      kbo(:, 5,17, 2) = (/ &
        & 0.60807e-08_rb,0.61758e-05_rb,0.95894e-05_rb,0.10949e-04_rb,0.68549e-06_rb /)
      kbo(:, 1,18, 2) = (/ &
        & 0.64779e-08_rb,0.34727e-05_rb,0.54875e-05_rb,0.64000e-05_rb,0.31851e-06_rb /)
      kbo(:, 2,18, 2) = (/ &
        & 0.60135e-08_rb,0.38751e-05_rb,0.61308e-05_rb,0.71372e-05_rb,0.37552e-06_rb /)
      kbo(:, 3,18, 2) = (/ &
        & 0.56111e-08_rb,0.43132e-05_rb,0.67478e-05_rb,0.77698e-05_rb,0.44141e-06_rb /)
      kbo(:, 4,18, 2) = (/ &
        & 0.52590e-08_rb,0.47150e-05_rb,0.73444e-05_rb,0.84370e-05_rb,0.50075e-06_rb /)
      kbo(:, 5,18, 2) = (/ &
        & 0.49483e-08_rb,0.51345e-05_rb,0.79495e-05_rb,0.90637e-05_rb,0.56601e-06_rb /)
      kbo(:, 1,19, 2) = (/ &
        & 0.52627e-08_rb,0.28908e-05_rb,0.45586e-05_rb,0.53260e-05_rb,0.26516e-06_rb /)
      kbo(:, 2,19, 2) = (/ &
        & 0.48882e-08_rb,0.32260e-05_rb,0.50940e-05_rb,0.59200e-05_rb,0.31190e-06_rb /)
      kbo(:, 3,19, 2) = (/ &
        & 0.45632e-08_rb,0.35843e-05_rb,0.55992e-05_rb,0.64397e-05_rb,0.36608e-06_rb /)
      kbo(:, 4,19, 2) = (/ &
        & 0.42786e-08_rb,0.39167e-05_rb,0.60923e-05_rb,0.69972e-05_rb,0.41361e-06_rb /)
      kbo(:, 5,19, 2) = (/ &
        & 0.40273e-08_rb,0.42667e-05_rb,0.65904e-05_rb,0.75021e-05_rb,0.46741e-06_rb /)
      kbo(:, 1,20, 2) = (/ &
        & 0.42704e-08_rb,0.24074e-05_rb,0.37990e-05_rb,0.44458e-05_rb,0.22103e-06_rb /)
      kbo(:, 2,20, 2) = (/ &
        & 0.39690e-08_rb,0.26942e-05_rb,0.42393e-05_rb,0.49120e-05_rb,0.26014e-06_rb /)
      kbo(:, 3,20, 2) = (/ &
        & 0.37071e-08_rb,0.29827e-05_rb,0.46574e-05_rb,0.53487e-05_rb,0.30416e-06_rb /)
      kbo(:, 4,20, 2) = (/ &
        & 0.34776e-08_rb,0.32601e-05_rb,0.50635e-05_rb,0.58076e-05_rb,0.34304e-06_rb /)
      kbo(:, 5,20, 2) = (/ &
        & 0.32747e-08_rb,0.35518e-05_rb,0.54700e-05_rb,0.62197e-05_rb,0.38752e-06_rb /)
      kbo(:, 1,21, 2) = (/ &
        & 0.34649e-08_rb,0.20096e-05_rb,0.31689e-05_rb,0.37070e-05_rb,0.18424e-06_rb /)
      kbo(:, 2,21, 2) = (/ &
        & 0.32224e-08_rb,0.22503e-05_rb,0.35271e-05_rb,0.40784e-05_rb,0.21705e-06_rb /)
      kbo(:, 3,21, 2) = (/ &
        & 0.30114e-08_rb,0.24834e-05_rb,0.38737e-05_rb,0.44436e-05_rb,0.25229e-06_rb /)
      kbo(:, 4,21, 2) = (/ &
        & 0.28263e-08_rb,0.27149e-05_rb,0.42083e-05_rb,0.48061e-05_rb,0.28464e-06_rb /)
      kbo(:, 5,21, 2) = (/ &
        & 0.26626e-08_rb,0.29556e-05_rb,0.45394e-05_rb,0.51565e-05_rb,0.32148e-06_rb /)
      kbo(:, 1,22, 2) = (/ &
        & 0.27976e-08_rb,0.16913e-05_rb,0.26632e-05_rb,0.31108e-05_rb,0.15537e-06_rb /)
      kbo(:, 2,22, 2) = (/ &
        & 0.26043e-08_rb,0.18908e-05_rb,0.29553e-05_rb,0.34067e-05_rb,0.18311e-06_rb /)
      kbo(:, 3,22, 2) = (/ &
        & 0.24359e-08_rb,0.20811e-05_rb,0.32390e-05_rb,0.37110e-05_rb,0.21094e-06_rb /)
      kbo(:, 4,22, 2) = (/ &
        & 0.22878e-08_rb,0.22746e-05_rb,0.35144e-05_rb,0.40009e-05_rb,0.23816e-06_rb /)
      kbo(:, 5,22, 2) = (/ &
        & 0.21567e-08_rb,0.24714e-05_rb,0.37839e-05_rb,0.42910e-05_rb,0.26886e-06_rb /)
      kbo(:, 1,23, 2) = (/ &
        & 0.22583e-08_rb,0.14234e-05_rb,0.22388e-05_rb,0.26088e-05_rb,0.13143e-06_rb /)
      kbo(:, 2,23, 2) = (/ &
        & 0.21043e-08_rb,0.15864e-05_rb,0.24776e-05_rb,0.28493e-05_rb,0.15460e-06_rb /)
      kbo(:, 3,23, 2) = (/ &
        & 0.19699e-08_rb,0.17436e-05_rb,0.27072e-05_rb,0.31009e-05_rb,0.17669e-06_rb /)
      kbo(:, 4,23, 2) = (/ &
        & 0.18515e-08_rb,0.19057e-05_rb,0.29355e-05_rb,0.33308e-05_rb,0.19953e-06_rb /)
      kbo(:, 5,23, 2) = (/ &
        & 0.17465e-08_rb,0.20653e-05_rb,0.31542e-05_rb,0.35663e-05_rb,0.22501e-06_rb /)
      kbo(:, 1,24, 2) = (/ &
        & 0.18221e-08_rb,0.12001e-05_rb,0.18808e-05_rb,0.21868e-05_rb,0.11138e-06_rb /)
      kbo(:, 2,24, 2) = (/ &
        & 0.16995e-08_rb,0.13315e-05_rb,0.20779e-05_rb,0.23825e-05_rb,0.13068e-06_rb /)
      kbo(:, 3,24, 2) = (/ &
        & 0.15923e-08_rb,0.14625e-05_rb,0.22647e-05_rb,0.25884e-05_rb,0.14820e-06_rb /)
      kbo(:, 4,24, 2) = (/ &
        & 0.14978e-08_rb,0.15972e-05_rb,0.24512e-05_rb,0.27771e-05_rb,0.16734e-06_rb /)
      kbo(:, 5,24, 2) = (/ &
        & 0.14138e-08_rb,0.17254e-05_rb,0.26296e-05_rb,0.29649e-05_rb,0.18848e-06_rb /)
      kbo(:, 1,25, 2) = (/ &
        & 0.14693e-08_rb,0.10111e-05_rb,0.15809e-05_rb,0.18286e-05_rb,0.94366e-07_rb /)
      kbo(:, 2,25, 2) = (/ &
        & 0.13718e-08_rb,0.11199e-05_rb,0.17430e-05_rb,0.19952e-05_rb,0.11045e-06_rb /)
      kbo(:, 3,25, 2) = (/ &
        & 0.12864e-08_rb,0.12273e-05_rb,0.18957e-05_rb,0.21572e-05_rb,0.12445e-06_rb /)
      kbo(:, 4,25, 2) = (/ &
        & 0.12110e-08_rb,0.13373e-05_rb,0.20462e-05_rb,0.23162e-05_rb,0.14042e-06_rb /)
      kbo(:, 5,25, 2) = (/ &
        & 0.11438e-08_rb,0.14410e-05_rb,0.21916e-05_rb,0.24648e-05_rb,0.15767e-06_rb /)
      kbo(:, 1,26, 2) = (/ &
        & 0.11834e-08_rb,0.85331e-06_rb,0.13316e-05_rb,0.15329e-05_rb,0.80212e-07_rb /)
      kbo(:, 2,26, 2) = (/ &
        & 0.11061e-08_rb,0.94192e-06_rb,0.14626e-05_rb,0.16729e-05_rb,0.92775e-07_rb /)
      kbo(:, 3,26, 2) = (/ &
        & 0.10382e-08_rb,0.10317e-05_rb,0.15890e-05_rb,0.18006e-05_rb,0.10462e-06_rb /)
      kbo(:, 4,26, 2) = (/ &
        & 0.97810e-09_rb,0.11205e-05_rb,0.17099e-05_rb,0.19314e-05_rb,0.11800e-06_rb /)
      kbo(:, 5,26, 2) = (/ &
        & 0.92457e-09_rb,0.12034e-05_rb,0.18276e-05_rb,0.20504e-05_rb,0.13209e-06_rb /)
      kbo(:, 1,27, 2) = (/ &
        & 0.95311e-09_rb,0.71890e-06_rb,0.11218e-05_rb,0.12867e-05_rb,0.68084e-07_rb /)
      kbo(:, 2,27, 2) = (/ &
        & 0.89177e-09_rb,0.79258e-06_rb,0.12275e-05_rb,0.14019e-05_rb,0.77999e-07_rb /)
      kbo(:, 3,27, 2) = (/ &
        & 0.83781e-09_rb,0.86605e-06_rb,0.13308e-05_rb,0.15040e-05_rb,0.88027e-07_rb /)
      kbo(:, 4,27, 2) = (/ &
        & 0.78998e-09_rb,0.93815e-06_rb,0.14285e-05_rb,0.16091e-05_rb,0.99057e-07_rb /)
      kbo(:, 5,27, 2) = (/ &
        & 0.74729e-09_rb,0.10046e-05_rb,0.15236e-05_rb,0.17059e-05_rb,0.11074e-06_rb /)
      kbo(:, 1,28, 2) = (/ &
        & 0.76771e-09_rb,0.60623e-06_rb,0.94427e-06_rb,0.10796e-05_rb,0.57844e-07_rb /)
      kbo(:, 2,28, 2) = (/ &
        & 0.71904e-09_rb,0.66653e-06_rb,0.10298e-05_rb,0.11699e-05_rb,0.65726e-07_rb /)
      kbo(:, 3,28, 2) = (/ &
        & 0.67614e-09_rb,0.72689e-06_rb,0.11131e-05_rb,0.12568e-05_rb,0.74009e-07_rb /)
      kbo(:, 4,28, 2) = (/ &
        & 0.63806e-09_rb,0.78482e-06_rb,0.11927e-05_rb,0.13406e-05_rb,0.83200e-07_rb /)
      kbo(:, 5,28, 2) = (/ &
        & 0.60401e-09_rb,0.83806e-06_rb,0.12695e-05_rb,0.14186e-05_rb,0.92306e-07_rb /)
      kbo(:, 1,29, 2) = (/ &
        & 0.61817e-09_rb,0.51053e-06_rb,0.79450e-06_rb,0.90687e-06_rb,0.49078e-07_rb /)
      kbo(:, 2,29, 2) = (/ &
        & 0.57958e-09_rb,0.56071e-06_rb,0.86403e-06_rb,0.97690e-06_rb,0.55257e-07_rb /)
      kbo(:, 3,29, 2) = (/ &
        & 0.54551e-09_rb,0.61009e-06_rb,0.93077e-06_rb,0.10495e-05_rb,0.62286e-07_rb /)
      kbo(:, 4,29, 2) = (/ &
        & 0.51520e-09_rb,0.65631e-06_rb,0.99567e-06_rb,0.11159e-05_rb,0.69954e-07_rb /)
      kbo(:, 5,29, 2) = (/ &
        & 0.48806e-09_rb,0.69830e-06_rb,0.10576e-05_rb,0.11790e-05_rb,0.77228e-07_rb /)
      kbo(:, 1,30, 2) = (/ &
        & 0.49783e-09_rb,0.43014e-06_rb,0.66732e-06_rb,0.76058e-06_rb,0.41306e-07_rb /)
      kbo(:, 2,30, 2) = (/ &
        & 0.46723e-09_rb,0.47148e-06_rb,0.72434e-06_rb,0.81682e-06_rb,0.46537e-07_rb /)
      kbo(:, 3,30, 2) = (/ &
        & 0.44015e-09_rb,0.51146e-06_rb,0.77809e-06_rb,0.87537e-06_rb,0.52451e-07_rb /)
      kbo(:, 4,30, 2) = (/ &
        & 0.41603e-09_rb,0.54877e-06_rb,0.83073e-06_rb,0.92884e-06_rb,0.58793e-07_rb /)
      kbo(:, 5,30, 2) = (/ &
        & 0.39440e-09_rb,0.58159e-06_rb,0.88054e-06_rb,0.98030e-06_rb,0.64630e-07_rb /)
      kbo(:, 1,31, 2) = (/ &
        & 0.40087e-09_rb,0.36256e-06_rb,0.56062e-06_rb,0.63549e-06_rb,0.34778e-07_rb /)
      kbo(:, 2,31, 2) = (/ &
        & 0.37660e-09_rb,0.39620e-06_rb,0.60674e-06_rb,0.68313e-06_rb,0.39213e-07_rb /)
      kbo(:, 3,31, 2) = (/ &
        & 0.35510e-09_rb,0.42855e-06_rb,0.65037e-06_rb,0.72999e-06_rb,0.44176e-07_rb /)
      kbo(:, 4,31, 2) = (/ &
        & 0.33590e-09_rb,0.45852e-06_rb,0.69308e-06_rb,0.77338e-06_rb,0.49424e-07_rb /)
      kbo(:, 5,31, 2) = (/ &
        & 0.31866e-09_rb,0.48459e-06_rb,0.73297e-06_rb,0.81484e-06_rb,0.54137e-07_rb /)
      kbo(:, 1,32, 2) = (/ &
        & 0.32280e-09_rb,0.30547e-06_rb,0.47083e-06_rb,0.53147e-06_rb,0.29353e-07_rb /)
      kbo(:, 2,32, 2) = (/ &
        & 0.30356e-09_rb,0.33271e-06_rb,0.50770e-06_rb,0.57112e-06_rb,0.33078e-07_rb /)
      kbo(:, 3,32, 2) = (/ &
        & 0.28647e-09_rb,0.35876e-06_rb,0.54339e-06_rb,0.60852e-06_rb,0.37211e-07_rb /)
      kbo(:, 4,32, 2) = (/ &
        & 0.27120e-09_rb,0.38214e-06_rb,0.57781e-06_rb,0.64354e-06_rb,0.41295e-07_rb /)
      kbo(:, 5,32, 2) = (/ &
        & 0.25746e-09_rb,0.40374e-06_rb,0.60990e-06_rb,0.67711e-06_rb,0.45362e-07_rb /)
      kbo(:, 1,33, 2) = (/ &
        & 0.25997e-09_rb,0.25723e-06_rb,0.39505e-06_rb,0.44467e-06_rb,0.24754e-07_rb /)
      kbo(:, 2,33, 2) = (/ &
        & 0.24471e-09_rb,0.27926e-06_rb,0.42469e-06_rb,0.47702e-06_rb,0.27907e-07_rb /)
      kbo(:, 3,33, 2) = (/ &
        & 0.23113e-09_rb,0.30024e-06_rb,0.45384e-06_rb,0.50672e-06_rb,0.31344e-07_rb /)
      kbo(:, 4,33, 2) = (/ &
        & 0.21898e-09_rb,0.31862e-06_rb,0.48142e-06_rb,0.53515e-06_rb,0.34590e-07_rb /)
      kbo(:, 5,33, 2) = (/ &
        & 0.20803e-09_rb,0.33624e-06_rb,0.50721e-06_rb,0.56242e-06_rb,0.37957e-07_rb /)
      kbo(:, 1,34, 2) = (/ &
        & 0.20987e-09_rb,0.21561e-06_rb,0.33035e-06_rb,0.37093e-06_rb,0.20801e-07_rb /)
      kbo(:, 2,34, 2) = (/ &
        & 0.19771e-09_rb,0.23356e-06_rb,0.35433e-06_rb,0.39723e-06_rb,0.23443e-07_rb /)
      kbo(:, 3,34, 2) = (/ &
        & 0.18688e-09_rb,0.25042e-06_rb,0.37804e-06_rb,0.42127e-06_rb,0.26271e-07_rb /)
      kbo(:, 4,34, 2) = (/ &
        & 0.17716e-09_rb,0.26514e-06_rb,0.40029e-06_rb,0.44433e-06_rb,0.28867e-07_rb /)
      kbo(:, 5,34, 2) = (/ &
        & 0.16840e-09_rb,0.27949e-06_rb,0.42102e-06_rb,0.46612e-06_rb,0.31620e-07_rb /)
      kbo(:, 1,35, 2) = (/ &
        & 0.17046e-09_rb,0.17909e-06_rb,0.27397e-06_rb,0.30730e-06_rb,0.17283e-07_rb /)
      kbo(:, 2,35, 2) = (/ &
        & 0.16066e-09_rb,0.19378e-06_rb,0.29367e-06_rb,0.32869e-06_rb,0.19456e-07_rb /)
      kbo(:, 3,35, 2) = (/ &
        & 0.15191e-09_rb,0.20716e-06_rb,0.31287e-06_rb,0.34820e-06_rb,0.21759e-07_rb /)
      kbo(:, 4,35, 2) = (/ &
        & 0.14407e-09_rb,0.21938e-06_rb,0.33090e-06_rb,0.36703e-06_rb,0.23854e-07_rb /)
      kbo(:, 5,35, 2) = (/ &
        & 0.13699e-09_rb,0.23115e-06_rb,0.34779e-06_rb,0.38459e-06_rb,0.26128e-07_rb /)
      kbo(:, 1,36, 2) = (/ &
        & 0.13938e-09_rb,0.14742e-06_rb,0.22535e-06_rb,0.25261e-06_rb,0.14145e-07_rb /)
      kbo(:, 2,36, 2) = (/ &
        & 0.13137e-09_rb,0.15949e-06_rb,0.24155e-06_rb,0.27011e-06_rb,0.15917e-07_rb /)
      kbo(:, 3,36, 2) = (/ &
        & 0.12423e-09_rb,0.17037e-06_rb,0.25721e-06_rb,0.28606e-06_rb,0.17809e-07_rb /)
      kbo(:, 4,36, 2) = (/ &
        & 0.11782e-09_rb,0.18044e-06_rb,0.27200e-06_rb,0.30149e-06_rb,0.19530e-07_rb /)
      kbo(:, 5,36, 2) = (/ &
        & 0.11204e-09_rb,0.19011e-06_rb,0.28582e-06_rb,0.31581e-06_rb,0.21391e-07_rb /)
      kbo(:, 1,37, 2) = (/ &
        & 0.11503e-09_rb,0.11991e-06_rb,0.18339e-06_rb,0.20540e-06_rb,0.11398e-07_rb /)
      kbo(:, 2,37, 2) = (/ &
        & 0.10837e-09_rb,0.12985e-06_rb,0.19670e-06_rb,0.21991e-06_rb,0.12829e-07_rb /)
      kbo(:, 3,37, 2) = (/ &
        & 0.10244e-09_rb,0.13895e-06_rb,0.20964e-06_rb,0.23310e-06_rb,0.14370e-07_rb /)
      kbo(:, 4,37, 2) = (/ &
        & 0.97122e-10_rb,0.14725e-06_rb,0.22183e-06_rb,0.24582e-06_rb,0.15790e-07_rb /)
      kbo(:, 5,37, 2) = (/ &
        & 0.92325e-10_rb,0.15526e-06_rb,0.23327e-06_rb,0.25767e-06_rb,0.17308e-07_rb /)
      kbo(:, 1,38, 2) = (/ &
        & 0.94990e-10_rb,0.97434e-07_rb,0.14912e-06_rb,0.16699e-06_rb,0.91745e-08_rb /)
      kbo(:, 2,38, 2) = (/ &
        & 0.89449e-10_rb,0.10563e-06_rb,0.16007e-06_rb,0.17890e-06_rb,0.10330e-07_rb /)
      kbo(:, 3,38, 2) = (/ &
        & 0.84516e-10_rb,0.11322e-06_rb,0.17071e-06_rb,0.18982e-06_rb,0.11585e-07_rb /)
      kbo(:, 4,38, 2) = (/ &
        & 0.80096e-10_rb,0.12009e-06_rb,0.18084e-06_rb,0.20030e-06_rb,0.12759e-07_rb /)
      kbo(:, 5,38, 2) = (/ &
        & 0.76113e-10_rb,0.12672e-06_rb,0.19028e-06_rb,0.21012e-06_rb,0.13992e-07_rb /)
      kbo(:, 1,39, 2) = (/ &
        & 0.78444e-10_rb,0.79147e-07_rb,0.12122e-06_rb,0.13573e-06_rb,0.73846e-08_rb /)
      kbo(:, 2,39, 2) = (/ &
        & 0.73832e-10_rb,0.85901e-07_rb,0.13022e-06_rb,0.14552e-06_rb,0.83180e-08_rb /)
      kbo(:, 3,39, 2) = (/ &
        & 0.69729e-10_rb,0.92230e-07_rb,0.13899e-06_rb,0.15459e-06_rb,0.93374e-08_rb /)
      kbo(:, 4,39, 2) = (/ &
        & 0.66056e-10_rb,0.97919e-07_rb,0.14737e-06_rb,0.16325e-06_rb,0.10307e-07_rb /)
      kbo(:, 5,39, 2) = (/ &
        & 0.62749e-10_rb,0.10341e-06_rb,0.15518e-06_rb,0.17133e-06_rb,0.11312e-07_rb /)
      kbo(:, 1,40, 2) = (/ &
        & 0.65207e-10_rb,0.63715e-07_rb,0.97755e-07_rb,0.10945e-06_rb,0.58831e-08_rb /)
      kbo(:, 2,40, 2) = (/ &
        & 0.61319e-10_rb,0.69289e-07_rb,0.10524e-06_rb,0.11757e-06_rb,0.66237e-08_rb /)
      kbo(:, 3,40, 2) = (/ &
        & 0.57866e-10_rb,0.74694e-07_rb,0.11245e-06_rb,0.12515e-06_rb,0.74455e-08_rb /)
      kbo(:, 4,40, 2) = (/ &
        & 0.54779e-10_rb,0.79359e-07_rb,0.11941e-06_rb,0.13229e-06_rb,0.82595e-08_rb /)
      kbo(:, 5,40, 2) = (/ &
        & 0.52003e-10_rb,0.83957e-07_rb,0.12593e-06_rb,0.13909e-06_rb,0.90698e-08_rb /)
      kbo(:, 1,41, 2) = (/ &
        & 0.54263e-10_rb,0.51172e-07_rb,0.78714e-07_rb,0.88183e-07_rb,0.46740e-08_rb /)
      kbo(:, 2,41, 2) = (/ &
        & 0.50978e-10_rb,0.55838e-07_rb,0.84939e-07_rb,0.94828e-07_rb,0.52655e-08_rb /)
      kbo(:, 3,41, 2) = (/ &
        & 0.48066e-10_rb,0.60328e-07_rb,0.90859e-07_rb,0.10118e-06_rb,0.59273e-08_rb /)
      kbo(:, 4,41, 2) = (/ &
        & 0.45467e-10_rb,0.64260e-07_rb,0.96667e-07_rb,0.10710e-06_rb,0.66279e-08_rb /)
      kbo(:, 5,41, 2) = (/ &
        & 0.43134e-10_rb,0.68089e-07_rb,0.10208e-06_rb,0.11279e-06_rb,0.72560e-08_rb /)
      kbo(:, 1,42, 2) = (/ &
        & 0.45170e-10_rb,0.41073e-07_rb,0.63333e-07_rb,0.71067e-07_rb,0.37185e-08_rb /)
      kbo(:, 2,42, 2) = (/ &
        & 0.42393e-10_rb,0.44964e-07_rb,0.68507e-07_rb,0.76455e-07_rb,0.41864e-08_rb /)
      kbo(:, 3,42, 2) = (/ &
        & 0.39936e-10_rb,0.48667e-07_rb,0.73382e-07_rb,0.81772e-07_rb,0.47157e-08_rb /)
      kbo(:, 4,42, 2) = (/ &
        & 0.37747e-10_rb,0.51996e-07_rb,0.78220e-07_rb,0.86689e-07_rb,0.52834e-08_rb /)
      kbo(:, 5,42, 2) = (/ &
        & 0.35784e-10_rb,0.55178e-07_rb,0.82739e-07_rb,0.91434e-07_rb,0.58040e-08_rb /)
      kbo(:, 1,43, 2) = (/ &
        & 0.37754e-10_rb,0.32770e-07_rb,0.50662e-07_rb,0.56979e-07_rb,0.29405e-08_rb /)
      kbo(:, 2,43, 2) = (/ &
        & 0.35387e-10_rb,0.35997e-07_rb,0.54993e-07_rb,0.61357e-07_rb,0.33066e-08_rb /)
      kbo(:, 3,43, 2) = (/ &
        & 0.33299e-10_rb,0.39073e-07_rb,0.59036e-07_rb,0.65793e-07_rb,0.37265e-08_rb /)
      kbo(:, 4,43, 2) = (/ &
        & 0.31442e-10_rb,0.41903e-07_rb,0.63014e-07_rb,0.69920e-07_rb,0.41848e-08_rb /)
      kbo(:, 5,43, 2) = (/ &
        & 0.29781e-10_rb,0.44546e-07_rb,0.66822e-07_rb,0.73849e-07_rb,0.46204e-08_rb /)
      kbo(:, 1,44, 2) = (/ &
        & 0.31623e-10_rb,0.26054e-07_rb,0.40405e-07_rb,0.45692e-07_rb,0.22918e-08_rb /)
      kbo(:, 2,44, 2) = (/ &
        & 0.29599e-10_rb,0.28713e-07_rb,0.44010e-07_rb,0.49122e-07_rb,0.26053e-08_rb /)
      kbo(:, 3,44, 2) = (/ &
        & 0.27817e-10_rb,0.31296e-07_rb,0.47391e-07_rb,0.52815e-07_rb,0.29353e-08_rb /)
      kbo(:, 4,44, 2) = (/ &
        & 0.26236e-10_rb,0.33677e-07_rb,0.50655e-07_rb,0.56264e-07_rb,0.33023e-08_rb /)
      kbo(:, 5,44, 2) = (/ &
        & 0.24825e-10_rb,0.35896e-07_rb,0.53863e-07_rb,0.59538e-07_rb,0.36730e-08_rb /)
      kbo(:, 1,45, 2) = (/ &
        & 0.26502e-10_rb,0.20705e-07_rb,0.32152e-07_rb,0.36448e-07_rb,0.17841e-08_rb /)
      kbo(:, 2,45, 2) = (/ &
        & 0.24768e-10_rb,0.22868e-07_rb,0.35185e-07_rb,0.39329e-07_rb,0.20515e-08_rb /)
      kbo(:, 3,45, 2) = (/ &
        & 0.23247e-10_rb,0.25027e-07_rb,0.37999e-07_rb,0.42338e-07_rb,0.23118e-08_rb /)
      kbo(:, 4,45, 2) = (/ &
        & 0.21900e-10_rb,0.27063e-07_rb,0.40694e-07_rb,0.45258e-07_rb,0.26048e-08_rb /)
      kbo(:, 5,45, 2) = (/ &
        & 0.20701e-10_rb,0.28901e-07_rb,0.43361e-07_rb,0.47969e-07_rb,0.29174e-08_rb /)
      kbo(:, 1,46, 2) = (/ &
        & 0.22258e-10_rb,0.16422e-07_rb,0.25480e-07_rb,0.28991e-07_rb,0.13830e-08_rb /)
      kbo(:, 2,46, 2) = (/ &
        & 0.20767e-10_rb,0.18158e-07_rb,0.28033e-07_rb,0.31442e-07_rb,0.16144e-08_rb /)
      kbo(:, 3,46, 2) = (/ &
        & 0.19462e-10_rb,0.19953e-07_rb,0.30402e-07_rb,0.33850e-07_rb,0.18162e-08_rb /)
      kbo(:, 4,46, 2) = (/ &
        & 0.18312e-10_rb,0.21658e-07_rb,0.32639e-07_rb,0.36308e-07_rb,0.20475e-08_rb /)
      kbo(:, 5,46, 2) = (/ &
        & 0.17289e-10_rb,0.23211e-07_rb,0.34839e-07_rb,0.38583e-07_rb,0.22994e-08_rb /)
      kbo(:, 1,47, 2) = (/ &
        & 0.18764e-10_rb,0.12951e-07_rb,0.20072e-07_rb,0.22982e-07_rb,0.10645e-08_rb /)
      kbo(:, 2,47, 2) = (/ &
        & 0.17473e-10_rb,0.14335e-07_rb,0.22214e-07_rb,0.25085e-07_rb,0.12479e-08_rb /)
      kbo(:, 3,47, 2) = (/ &
        & 0.16347e-10_rb,0.15813e-07_rb,0.24205e-07_rb,0.26971e-07_rb,0.14207e-08_rb /)
      kbo(:, 4,47, 2) = (/ &
        & 0.15357e-10_rb,0.17255e-07_rb,0.26075e-07_rb,0.29015e-07_rb,0.16008e-08_rb /)
      kbo(:, 5,47, 2) = (/ &
        & 0.14480e-10_rb,0.18558e-07_rb,0.27885e-07_rb,0.30922e-07_rb,0.18018e-08_rb /)
      kbo(:, 1,48, 2) = (/ &
        & 0.15833e-10_rb,0.10186e-07_rb,0.15782e-07_rb,0.18174e-07_rb,0.81718e-09_rb /)
      kbo(:, 2,48, 2) = (/ &
        & 0.14713e-10_rb,0.11312e-07_rb,0.17555e-07_rb,0.19891e-07_rb,0.96098e-09_rb /)
      kbo(:, 3,48, 2) = (/ &
        & 0.13740e-10_rb,0.12509e-07_rb,0.19248e-07_rb,0.21482e-07_rb,0.11112e-08_rb /)
      kbo(:, 4,48, 2) = (/ &
        & 0.12887e-10_rb,0.13719e-07_rb,0.20811e-07_rb,0.23140e-07_rb,0.12515e-08_rb /)
      kbo(:, 5,48, 2) = (/ &
        & 0.12134e-10_rb,0.14840e-07_rb,0.22313e-07_rb,0.24769e-07_rb,0.14105e-08_rb /)
      kbo(:, 1,49, 2) = (/ &
        & 0.13372e-10_rb,0.80050e-08_rb,0.12403e-07_rb,0.14283e-07_rb,0.63117e-09_rb /)
      kbo(:, 2,49, 2) = (/ &
        & 0.12398e-10_rb,0.89393e-08_rb,0.13840e-07_rb,0.15751e-07_rb,0.73907e-09_rb /)
      kbo(:, 3,49, 2) = (/ &
        & 0.11556e-10_rb,0.98821e-08_rb,0.15261e-07_rb,0.17104e-07_rb,0.86645e-09_rb /)
      kbo(:, 4,49, 2) = (/ &
        & 0.10821e-10_rb,0.10884e-07_rb,0.16582e-07_rb,0.18441e-07_rb,0.97829e-09_rb /)
      kbo(:, 5,49, 2) = (/ &
        & 0.10174e-10_rb,0.11841e-07_rb,0.17828e-07_rb,0.19804e-07_rb,0.11034e-08_rb /)
      kbo(:, 1,50, 2) = (/ &
        & 0.11282e-10_rb,0.62726e-08_rb,0.97478e-08_rb,0.11226e-07_rb,0.48398e-09_rb /)
      kbo(:, 2,50, 2) = (/ &
        & 0.10437e-10_rb,0.70546e-08_rb,0.10932e-07_rb,0.12518e-07_rb,0.57051e-09_rb /)
      kbo(:, 3,50, 2) = (/ &
        & 0.97101e-11_rb,0.78193e-08_rb,0.12114e-07_rb,0.13671e-07_rb,0.66997e-09_rb /)
      kbo(:, 4,50, 2) = (/ &
        & 0.90772e-11_rb,0.86379e-08_rb,0.13218e-07_rb,0.14712e-07_rb,0.76625e-09_rb /)
      kbo(:, 5,50, 2) = (/ &
        & 0.85216e-11_rb,0.94405e-08_rb,0.14258e-07_rb,0.15844e-07_rb,0.86488e-09_rb /)
      kbo(:, 1,51, 2) = (/ &
        & 0.95179e-11_rb,0.49251e-08_rb,0.76356e-08_rb,0.87761e-08_rb,0.37213e-09_rb /)
      kbo(:, 2,51, 2) = (/ &
        & 0.87856e-11_rb,0.55681e-08_rb,0.86220e-08_rb,0.99159e-08_rb,0.44074e-09_rb /)
      kbo(:, 3,51, 2) = (/ &
        & 0.81576e-11_rb,0.61930e-08_rb,0.95989e-08_rb,0.10865e-07_rb,0.51843e-09_rb /)
      kbo(:, 4,51, 2) = (/ &
        & 0.76131e-11_rb,0.68518e-08_rb,0.10539e-07_rb,0.11746e-07_rb,0.60210e-09_rb /)
      kbo(:, 5,51, 2) = (/ &
        & 0.71366e-11_rb,0.75206e-08_rb,0.11403e-07_rb,0.12659e-07_rb,0.67841e-09_rb /)
      kbo(:, 1,52, 2) = (/ &
        & 0.80368e-11_rb,0.38608e-08_rb,0.60010e-08_rb,0.68441e-08_rb,0.28848e-09_rb /)
      kbo(:, 2,52, 2) = (/ &
        & 0.74007e-11_rb,0.43903e-08_rb,0.67974e-08_rb,0.78127e-08_rb,0.34245e-09_rb /)
      kbo(:, 3,52, 2) = (/ &
        & 0.68577e-11_rb,0.49086e-08_rb,0.75959e-08_rb,0.86279e-08_rb,0.40106e-09_rb /)
      kbo(:, 4,52, 2) = (/ &
        & 0.63886e-11_rb,0.54297e-08_rb,0.83801e-08_rb,0.93778e-08_rb,0.47058e-09_rb /)
      kbo(:, 5,52, 2) = (/ &
        & 0.59795e-11_rb,0.59829e-08_rb,0.91103e-08_rb,0.10111e-07_rb,0.53218e-09_rb /)
      kbo(:, 1,53, 2) = (/ &
        & 0.67932e-11_rb,0.30102e-08_rb,0.47269e-08_rb,0.53395e-08_rb,0.22432e-09_rb /)
      kbo(:, 2,53, 2) = (/ &
        & 0.62397e-11_rb,0.34450e-08_rb,0.53458e-08_rb,0.61449e-08_rb,0.26282e-09_rb /)
      kbo(:, 3,53, 2) = (/ &
        & 0.57693e-11_rb,0.38768e-08_rb,0.60006e-08_rb,0.68613e-08_rb,0.31020e-09_rb /)
      kbo(:, 4,53, 2) = (/ &
        & 0.53647e-11_rb,0.43002e-08_rb,0.66533e-08_rb,0.74990e-08_rb,0.36446e-09_rb /)
      kbo(:, 5,53, 2) = (/ &
        & 0.50129e-11_rb,0.47527e-08_rb,0.72638e-08_rb,0.80717e-08_rb,0.41759e-09_rb /)
      kbo(:, 1,54, 2) = (/ &
        & 0.57320e-11_rb,0.23582e-08_rb,0.36837e-08_rb,0.41317e-08_rb,0.17477e-09_rb /)
      kbo(:, 2,54, 2) = (/ &
        & 0.52518e-11_rb,0.27163e-08_rb,0.42048e-08_rb,0.48208e-08_rb,0.20321e-09_rb /)
      kbo(:, 3,54, 2) = (/ &
        & 0.48457e-11_rb,0.30678e-08_rb,0.47472e-08_rb,0.54472e-08_rb,0.24081e-09_rb /)
      kbo(:, 4,54, 2) = (/ &
        & 0.44978e-11_rb,0.34148e-08_rb,0.52851e-08_rb,0.59735e-08_rb,0.28345e-09_rb /)
      kbo(:, 5,54, 2) = (/ &
        & 0.41963e-11_rb,0.37804e-08_rb,0.58001e-08_rb,0.64570e-08_rb,0.32923e-09_rb /)
      kbo(:, 1,55, 2) = (/ &
        & 0.48347e-11_rb,0.18534e-08_rb,0.28539e-08_rb,0.31828e-08_rb,0.13492e-09_rb /)
      kbo(:, 2,55, 2) = (/ &
        & 0.44186e-11_rb,0.21392e-08_rb,0.33190e-08_rb,0.37817e-08_rb,0.15848e-09_rb /)
      kbo(:, 3,55, 2) = (/ &
        & 0.40683e-11_rb,0.24320e-08_rb,0.37592e-08_rb,0.43156e-08_rb,0.18835e-09_rb /)
      kbo(:, 4,55, 2) = (/ &
        & 0.37693e-11_rb,0.27150e-08_rb,0.41999e-08_rb,0.47625e-08_rb,0.22076e-09_rb /)
      kbo(:, 5,55, 2) = (/ &
        & 0.35111e-11_rb,0.30082e-08_rb,0.46345e-08_rb,0.51736e-08_rb,0.25916e-09_rb /)
      kbo(:, 1,56, 2) = (/ &
        & 0.40813e-11_rb,0.14543e-08_rb,0.22038e-08_rb,0.24488e-08_rb,0.10328e-09_rb /)
      kbo(:, 2,56, 2) = (/ &
        & 0.37201e-11_rb,0.16813e-08_rb,0.26371e-08_rb,0.29640e-08_rb,0.12446e-09_rb /)
      kbo(:, 3,56, 2) = (/ &
        & 0.34175e-11_rb,0.19191e-08_rb,0.29747e-08_rb,0.34115e-08_rb,0.14604e-09_rb /)
      kbo(:, 4,56, 2) = (/ &
        & 0.31604e-11_rb,0.21575e-08_rb,0.33348e-08_rb,0.38018e-08_rb,0.17202e-09_rb /)
      kbo(:, 5,56, 2) = (/ &
        & 0.29391e-11_rb,0.23932e-08_rb,0.36930e-08_rb,0.41472e-08_rb,0.20235e-09_rb /)
      kbo(:, 1,57, 2) = (/ &
        & 0.34486e-11_rb,0.11480e-08_rb,0.17032e-08_rb,0.18771e-08_rb,0.78787e-10_rb /)
      kbo(:, 2,57, 2) = (/ &
        & 0.31345e-11_rb,0.13189e-08_rb,0.20615e-08_rb,0.23121e-08_rb,0.96832e-10_rb /)
      kbo(:, 3,57, 2) = (/ &
        & 0.28728e-11_rb,0.15144e-08_rb,0.23440e-08_rb,0.26863e-08_rb,0.11317e-09_rb /)
      kbo(:, 4,57, 2) = (/ &
        & 0.26513e-11_rb,0.17102e-08_rb,0.26436e-08_rb,0.30246e-08_rb,0.13393e-09_rb /)
      kbo(:, 5,57, 2) = (/ &
        & 0.24615e-11_rb,0.19022e-08_rb,0.29394e-08_rb,0.33135e-08_rb,0.15772e-09_rb /)
      kbo(:, 1,58, 2) = (/ &
        & 0.29119e-11_rb,0.89500e-09_rb,0.13020e-08_rb,0.14383e-08_rb,0.60253e-10_rb /)
      kbo(:, 2,58, 2) = (/ &
        & 0.26392e-11_rb,0.10369e-08_rb,0.16036e-08_rb,0.17891e-08_rb,0.75765e-10_rb /)
      kbo(:, 3,58, 2) = (/ &
        & 0.24132e-11_rb,0.11975e-08_rb,0.18523e-08_rb,0.21114e-08_rb,0.88260e-10_rb /)
      kbo(:, 4,58, 2) = (/ &
        & 0.22227e-11_rb,0.13570e-08_rb,0.20959e-08_rb,0.24045e-08_rb,0.10478e-09_rb /)
      kbo(:, 5,58, 2) = (/ &
        & 0.20601e-11_rb,0.15139e-08_rb,0.23406e-08_rb,0.26456e-08_rb,0.12331e-09_rb /)
      kbo(:, 1,59, 2) = (/ &
        & 0.24143e-11_rb,0.71851e-09_rb,0.10362e-08_rb,0.11516e-08_rb,0.48086e-10_rb /)
      kbo(:, 2,59, 2) = (/ &
        & 0.21857e-11_rb,0.83732e-09_rb,0.12892e-08_rb,0.14335e-08_rb,0.60630e-10_rb /)
      kbo(:, 3,59, 2) = (/ &
        & 0.19965e-11_rb,0.96815e-09_rb,0.14990e-08_rb,0.17039e-08_rb,0.71208e-10_rb /)
      kbo(:, 4,59, 2) = (/ &
        & 0.18374e-11_rb,0.10982e-08_rb,0.16973e-08_rb,0.19455e-08_rb,0.84514e-10_rb /)
      kbo(:, 5,59, 2) = (/ &
        & 0.17017e-11_rb,0.12277e-08_rb,0.18981e-08_rb,0.21472e-08_rb,0.99289e-10_rb /)
      kbo(:, 1,13, 3) = (/ &
        & 0.69497e-07_rb,0.55900e-04_rb,0.77634e-04_rb,0.83300e-04_rb,0.14419e-04_rb /)
      kbo(:, 2,13, 3) = (/ &
        & 0.64923e-07_rb,0.61779e-04_rb,0.85238e-04_rb,0.90458e-04_rb,0.15996e-04_rb /)
      kbo(:, 3,13, 3) = (/ &
        & 0.60920e-07_rb,0.67532e-04_rb,0.92811e-04_rb,0.97588e-04_rb,0.17622e-04_rb /)
      kbo(:, 4,13, 3) = (/ &
        & 0.57379e-07_rb,0.72876e-04_rb,0.10040e-03_rb,0.10455e-03_rb,0.19144e-04_rb /)
      kbo(:, 5,13, 3) = (/ &
        & 0.54220e-07_rb,0.77699e-04_rb,0.10784e-03_rb,0.11139e-03_rb,0.20671e-04_rb /)
      kbo(:, 1,14, 3) = (/ &
        & 0.56538e-07_rb,0.46627e-04_rb,0.64678e-04_rb,0.69399e-04_rb,0.12086e-04_rb /)
      kbo(:, 2,14, 3) = (/ &
        & 0.52839e-07_rb,0.51536e-04_rb,0.71080e-04_rb,0.75292e-04_rb,0.13414e-04_rb /)
      kbo(:, 3,14, 3) = (/ &
        & 0.49600e-07_rb,0.56334e-04_rb,0.77384e-04_rb,0.81277e-04_rb,0.14732e-04_rb /)
      kbo(:, 4,14, 3) = (/ &
        & 0.46731e-07_rb,0.60751e-04_rb,0.83753e-04_rb,0.87067e-04_rb,0.15973e-04_rb /)
      kbo(:, 5,14, 3) = (/ &
        & 0.44171e-07_rb,0.64792e-04_rb,0.89963e-04_rb,0.92617e-04_rb,0.17214e-04_rb /)
      kbo(:, 1,15, 3) = (/ &
        & 0.45963e-07_rb,0.38920e-04_rb,0.53921e-04_rb,0.57824e-04_rb,0.10089e-04_rb /)
      kbo(:, 2,15, 3) = (/ &
        & 0.42976e-07_rb,0.42987e-04_rb,0.59286e-04_rb,0.62615e-04_rb,0.11238e-04_rb /)
      kbo(:, 3,15, 3) = (/ &
        & 0.40358e-07_rb,0.47014e-04_rb,0.64557e-04_rb,0.67658e-04_rb,0.12293e-04_rb /)
      kbo(:, 4,15, 3) = (/ &
        & 0.38037e-07_rb,0.50641e-04_rb,0.69879e-04_rb,0.72487e-04_rb,0.13317e-04_rb /)
      kbo(:, 5,15, 3) = (/ &
        & 0.35964e-07_rb,0.54003e-04_rb,0.75064e-04_rb,0.77083e-04_rb,0.14321e-04_rb /)
      kbo(:, 1,16, 3) = (/ &
        & 0.37354e-07_rb,0.32479e-04_rb,0.44953e-04_rb,0.48146e-04_rb,0.84079e-05_rb /)
      kbo(:, 2,16, 3) = (/ &
        & 0.34944e-07_rb,0.35873e-04_rb,0.49454e-04_rb,0.52120e-04_rb,0.93405e-05_rb /)
      kbo(:, 3,16, 3) = (/ &
        & 0.32829e-07_rb,0.39220e-04_rb,0.53861e-04_rb,0.56293e-04_rb,0.10209e-04_rb /)
      kbo(:, 4,16, 3) = (/ &
        & 0.30952e-07_rb,0.42155e-04_rb,0.58309e-04_rb,0.60268e-04_rb,0.11080e-04_rb /)
      kbo(:, 5,16, 3) = (/ &
        & 0.29274e-07_rb,0.45014e-04_rb,0.62610e-04_rb,0.64184e-04_rb,0.11887e-04_rb /)
      kbo(:, 1,17, 3) = (/ &
        & 0.30364e-07_rb,0.27086e-04_rb,0.37451e-04_rb,0.39995e-04_rb,0.70049e-05_rb /)
      kbo(:, 2,17, 3) = (/ &
        & 0.28420e-07_rb,0.29919e-04_rb,0.41194e-04_rb,0.43367e-04_rb,0.77736e-05_rb /)
      kbo(:, 3,17, 3) = (/ &
        & 0.26710e-07_rb,0.32650e-04_rb,0.44902e-04_rb,0.46817e-04_rb,0.84838e-05_rb /)
      kbo(:, 4,17, 3) = (/ &
        & 0.25192e-07_rb,0.35098e-04_rb,0.48605e-04_rb,0.50103e-04_rb,0.92038e-05_rb /)
      kbo(:, 5,17, 3) = (/ &
        & 0.23833e-07_rb,0.37507e-04_rb,0.52188e-04_rb,0.53413e-04_rb,0.98607e-05_rb /)
      kbo(:, 1,18, 3) = (/ &
        & 0.24687e-07_rb,0.22581e-04_rb,0.31211e-04_rb,0.33220e-04_rb,0.58361e-05_rb /)
      kbo(:, 2,18, 3) = (/ &
        & 0.23116e-07_rb,0.24940e-04_rb,0.34308e-04_rb,0.36078e-04_rb,0.64640e-05_rb /)
      kbo(:, 3,18, 3) = (/ &
        & 0.21734e-07_rb,0.27181e-04_rb,0.37414e-04_rb,0.38933e-04_rb,0.70344e-05_rb /)
      kbo(:, 4,18, 3) = (/ &
        & 0.20506e-07_rb,0.29216e-04_rb,0.40517e-04_rb,0.41653e-04_rb,0.76276e-05_rb /)
      kbo(:, 5,18, 3) = (/ &
        & 0.19406e-07_rb,0.31237e-04_rb,0.43490e-04_rb,0.44397e-04_rb,0.81753e-05_rb /)
      kbo(:, 1,19, 3) = (/ &
        & 0.20074e-07_rb,0.18818e-04_rb,0.25999e-04_rb,0.27597e-04_rb,0.48662e-05_rb /)
      kbo(:, 2,19, 3) = (/ &
        & 0.18805e-07_rb,0.20784e-04_rb,0.28562e-04_rb,0.29993e-04_rb,0.53723e-05_rb /)
      kbo(:, 3,19, 3) = (/ &
        & 0.17688e-07_rb,0.22626e-04_rb,0.31151e-04_rb,0.32363e-04_rb,0.58377e-05_rb /)
      kbo(:, 4,19, 3) = (/ &
        & 0.16693e-07_rb,0.24300e-04_rb,0.33760e-04_rb,0.34629e-04_rb,0.63177e-05_rb /)
      kbo(:, 5,19, 3) = (/ &
        & 0.15802e-07_rb,0.25985e-04_rb,0.36226e-04_rb,0.36903e-04_rb,0.67742e-05_rb /)
      kbo(:, 1,20, 3) = (/ &
        & 0.16306e-07_rb,0.15717e-04_rb,0.21686e-04_rb,0.22959e-04_rb,0.40680e-05_rb /)
      kbo(:, 2,20, 3) = (/ &
        & 0.15283e-07_rb,0.17355e-04_rb,0.23826e-04_rb,0.24969e-04_rb,0.44706e-05_rb /)
      kbo(:, 3,20, 3) = (/ &
        & 0.14381e-07_rb,0.18856e-04_rb,0.25992e-04_rb,0.26912e-04_rb,0.48550e-05_rb /)
      kbo(:, 4,20, 3) = (/ &
        & 0.13578e-07_rb,0.20265e-04_rb,0.28162e-04_rb,0.28817e-04_rb,0.52355e-05_rb /)
      kbo(:, 5,20, 3) = (/ &
        & 0.12857e-07_rb,0.21638e-04_rb,0.30215e-04_rb,0.30707e-04_rb,0.56224e-05_rb /)
      kbo(:, 1,21, 3) = (/ &
        & 0.13245e-07_rb,0.13128e-04_rb,0.18091e-04_rb,0.19107e-04_rb,0.33880e-05_rb /)
      kbo(:, 2,21, 3) = (/ &
        & 0.12420e-07_rb,0.14495e-04_rb,0.19887e-04_rb,0.20794e-04_rb,0.37144e-05_rb /)
      kbo(:, 3,21, 3) = (/ &
        & 0.11692e-07_rb,0.15704e-04_rb,0.21703e-04_rb,0.22397e-04_rb,0.40415e-05_rb /)
      kbo(:, 4,21, 3) = (/ &
        & 0.11043e-07_rb,0.16904e-04_rb,0.23489e-04_rb,0.23984e-04_rb,0.43455e-05_rb /)
      kbo(:, 5,21, 3) = (/ &
        & 0.10460e-07_rb,0.18037e-04_rb,0.25197e-04_rb,0.25558e-04_rb,0.46662e-05_rb /)
      kbo(:, 1,22, 3) = (/ &
        & 0.10711e-07_rb,0.11044e-04_rb,0.15199e-04_rb,0.16008e-04_rb,0.28409e-05_rb /)
      kbo(:, 2,22, 3) = (/ &
        & 0.10052e-07_rb,0.12165e-04_rb,0.16699e-04_rb,0.17419e-04_rb,0.31041e-05_rb /)
      kbo(:, 3,22, 3) = (/ &
        & 0.94690e-08_rb,0.13158e-04_rb,0.18224e-04_rb,0.18733e-04_rb,0.33768e-05_rb /)
      kbo(:, 4,22, 3) = (/ &
        & 0.89484e-08_rb,0.14166e-04_rb,0.19691e-04_rb,0.20050e-04_rb,0.36237e-05_rb /)
      kbo(:, 5,22, 3) = (/ &
        & 0.84804e-08_rb,0.15085e-04_rb,0.21105e-04_rb,0.21356e-04_rb,0.38876e-05_rb /)
      kbo(:, 1,23, 3) = (/ &
        & 0.86606e-08_rb,0.93002e-05_rb,0.12775e-04_rb,0.13420e-04_rb,0.23815e-05_rb /)
      kbo(:, 2,23, 3) = (/ &
        & 0.81340e-08_rb,0.10210e-04_rb,0.14034e-04_rb,0.14552e-04_rb,0.25955e-05_rb /)
      kbo(:, 3,23, 3) = (/ &
        & 0.76671e-08_rb,0.11035e-04_rb,0.15302e-04_rb,0.15657e-04_rb,0.28150e-05_rb /)
      kbo(:, 4,23, 3) = (/ &
        & 0.72497e-08_rb,0.11863e-04_rb,0.16513e-04_rb,0.16770e-04_rb,0.30198e-05_rb /)
      kbo(:, 5,23, 3) = (/ &
        & 0.68739e-08_rb,0.12616e-04_rb,0.17664e-04_rb,0.17848e-04_rb,0.32387e-05_rb /)
      kbo(:, 1,24, 3) = (/ &
        & 0.69995e-08_rb,0.78392e-05_rb,0.10754e-04_rb,0.11250e-04_rb,0.19986e-05_rb /)
      kbo(:, 2,24, 3) = (/ &
        & 0.65790e-08_rb,0.85674e-05_rb,0.11806e-04_rb,0.12186e-04_rb,0.21738e-05_rb /)
      kbo(:, 3,24, 3) = (/ &
        & 0.62055e-08_rb,0.92664e-05_rb,0.12850e-04_rb,0.13103e-04_rb,0.23469e-05_rb /)
      kbo(:, 4,24, 3) = (/ &
        & 0.58710e-08_rb,0.99376e-05_rb,0.13854e-04_rb,0.14027e-04_rb,0.25195e-05_rb /)
      kbo(:, 5,24, 3) = (/ &
        & 0.55696e-08_rb,0.10527e-04_rb,0.14795e-04_rb,0.14917e-04_rb,0.26988e-05_rb /)
      kbo(:, 1,25, 3) = (/ &
        & 0.56538e-08_rb,0.66118e-05_rb,0.90610e-05_rb,0.94422e-05_rb,0.16753e-05_rb /)
      kbo(:, 2,25, 3) = (/ &
        & 0.53184e-08_rb,0.71986e-05_rb,0.99389e-05_rb,0.10209e-04_rb,0.18223e-05_rb /)
      kbo(:, 3,25, 3) = (/ &
        & 0.50199e-08_rb,0.77863e-05_rb,0.10800e-04_rb,0.10981e-04_rb,0.19590e-05_rb /)
      kbo(:, 4,25, 3) = (/ &
        & 0.47522e-08_rb,0.83415e-05_rb,0.11626e-04_rb,0.11737e-04_rb,0.21020e-05_rb /)
      kbo(:, 5,25, 3) = (/ &
        & 0.45105e-08_rb,0.88017e-05_rb,0.12373e-04_rb,0.12468e-04_rb,0.22498e-05_rb /)
      kbo(:, 1,26, 3) = (/ &
        & 0.45620e-08_rb,0.55749e-05_rb,0.76463e-05_rb,0.79285e-05_rb,0.14032e-05_rb /)
      kbo(:, 2,26, 3) = (/ &
        & 0.42950e-08_rb,0.60590e-05_rb,0.83822e-05_rb,0.85652e-05_rb,0.15255e-05_rb /)
      kbo(:, 3,26, 3) = (/ &
        & 0.40569e-08_rb,0.65510e-05_rb,0.90888e-05_rb,0.92116e-05_rb,0.16373e-05_rb /)
      kbo(:, 4,26, 3) = (/ &
        & 0.38429e-08_rb,0.69918e-05_rb,0.97604e-05_rb,0.98329e-05_rb,0.17561e-05_rb /)
      kbo(:, 5,26, 3) = (/ &
        & 0.36495e-08_rb,0.73566e-05_rb,0.10367e-04_rb,0.10429e-04_rb,0.18769e-05_rb /)
      kbo(:, 1,27, 3) = (/ &
        & 0.36809e-08_rb,0.46967e-05_rb,0.64580e-05_rb,0.66649e-05_rb,0.11786e-05_rb /)
      kbo(:, 2,27, 3) = (/ &
        & 0.34683e-08_rb,0.51072e-05_rb,0.70661e-05_rb,0.71902e-05_rb,0.12759e-05_rb /)
      kbo(:, 3,27, 3) = (/ &
        & 0.32784e-08_rb,0.55053e-05_rb,0.76470e-05_rb,0.77250e-05_rb,0.13691e-05_rb /)
      kbo(:, 4,27, 3) = (/ &
        & 0.31074e-08_rb,0.58474e-05_rb,0.81898e-05_rb,0.82340e-05_rb,0.14666e-05_rb /)
      kbo(:, 5,27, 3) = (/ &
        & 0.29526e-08_rb,0.61493e-05_rb,0.86815e-05_rb,0.87199e-05_rb,0.15650e-05_rb /)
      kbo(:, 1,28, 3) = (/ &
        & 0.29701e-08_rb,0.39582e-05_rb,0.54573e-05_rb,0.55959e-05_rb,0.99037e-06_rb /)
      kbo(:, 2,28, 3) = (/ &
        & 0.28008e-08_rb,0.43048e-05_rb,0.59546e-05_rb,0.60406e-05_rb,0.10659e-05_rb /)
      kbo(:, 3,28, 3) = (/ &
        & 0.26493e-08_rb,0.46277e-05_rb,0.64286e-05_rb,0.64745e-05_rb,0.11442e-05_rb /)
      kbo(:, 4,28, 3) = (/ &
        & 0.25127e-08_rb,0.48940e-05_rb,0.68612e-05_rb,0.68918e-05_rb,0.12250e-05_rb /)
      kbo(:, 5,28, 3) = (/ &
        & 0.23888e-08_rb,0.51428e-05_rb,0.72654e-05_rb,0.72865e-05_rb,0.13046e-05_rb /)
      kbo(:, 1,29, 3) = (/ &
        & 0.23958e-08_rb,0.33409e-05_rb,0.46115e-05_rb,0.47054e-05_rb,0.83021e-06_rb /)
      kbo(:, 2,29, 3) = (/ &
        & 0.22611e-08_rb,0.36267e-05_rb,0.50192e-05_rb,0.50754e-05_rb,0.89157e-06_rb /)
      kbo(:, 3,29, 3) = (/ &
        & 0.21402e-08_rb,0.38805e-05_rb,0.54053e-05_rb,0.54291e-05_rb,0.95663e-06_rb /)
      kbo(:, 4,29, 3) = (/ &
        & 0.20311e-08_rb,0.40929e-05_rb,0.57529e-05_rb,0.57685e-05_rb,0.10232e-05_rb /)
      kbo(:, 5,29, 3) = (/ &
        & 0.19320e-08_rb,0.43014e-05_rb,0.60684e-05_rb,0.60868e-05_rb,0.10874e-05_rb /)
      kbo(:, 1,30, 3) = (/ &
        & 0.19328e-08_rb,0.28217e-05_rb,0.38955e-05_rb,0.39562e-05_rb,0.69464e-06_rb /)
      kbo(:, 2,30, 3) = (/ &
        & 0.18255e-08_rb,0.30515e-05_rb,0.42293e-05_rb,0.42605e-05_rb,0.74588e-06_rb /)
      kbo(:, 3,30, 3) = (/ &
        & 0.17291e-08_rb,0.32496e-05_rb,0.45390e-05_rb,0.45501e-05_rb,0.79957e-06_rb /)
      kbo(:, 4,30, 3) = (/ &
        & 0.16419e-08_rb,0.34235e-05_rb,0.48191e-05_rb,0.48249e-05_rb,0.85373e-06_rb /)
      kbo(:, 5,30, 3) = (/ &
        & 0.15626e-08_rb,0.35992e-05_rb,0.50632e-05_rb,0.50805e-05_rb,0.90582e-06_rb /)
      kbo(:, 1,31, 3) = (/ &
        & 0.15590e-08_rb,0.23841e-05_rb,0.32890e-05_rb,0.33282e-05_rb,0.58108e-06_rb /)
      kbo(:, 2,31, 3) = (/ &
        & 0.14736e-08_rb,0.25683e-05_rb,0.35605e-05_rb,0.35756e-05_rb,0.62401e-06_rb /)
      kbo(:, 3,31, 3) = (/ &
        & 0.13967e-08_rb,0.27223e-05_rb,0.38055e-05_rb,0.38110e-05_rb,0.66862e-06_rb /)
      kbo(:, 4,31, 3) = (/ &
        & 0.13271e-08_rb,0.28652e-05_rb,0.40295e-05_rb,0.40331e-05_rb,0.71216e-06_rb /)
      kbo(:, 5,31, 3) = (/ &
        & 0.12637e-08_rb,0.30069e-05_rb,0.42257e-05_rb,0.42393e-05_rb,0.75527e-06_rb /)
      kbo(:, 1,32, 3) = (/ &
        & 0.12575e-08_rb,0.20099e-05_rb,0.27764e-05_rb,0.27988e-05_rb,0.48614e-06_rb /)
      kbo(:, 2,32, 3) = (/ &
        & 0.11895e-08_rb,0.21551e-05_rb,0.29961e-05_rb,0.29996e-05_rb,0.52199e-06_rb /)
      kbo(:, 3,32, 3) = (/ &
        & 0.11282e-08_rb,0.22771e-05_rb,0.31901e-05_rb,0.31897e-05_rb,0.55867e-06_rb /)
      kbo(:, 4,32, 3) = (/ &
        & 0.10726e-08_rb,0.23969e-05_rb,0.33657e-05_rb,0.33696e-05_rb,0.59410e-06_rb /)
      kbo(:, 5,32, 3) = (/ &
        & 0.10218e-08_rb,0.25076e-05_rb,0.35232e-05_rb,0.35366e-05_rb,0.62965e-06_rb /)
      kbo(:, 1,33, 3) = (/ &
        & 0.10144e-08_rb,0.16940e-05_rb,0.23421e-05_rb,0.23519e-05_rb,0.40701e-06_rb /)
      kbo(:, 2,33, 3) = (/ &
        & 0.96024e-09_rb,0.18066e-05_rb,0.25146e-05_rb,0.25148e-05_rb,0.43657e-06_rb /)
      kbo(:, 3,33, 3) = (/ &
        & 0.91134e-09_rb,0.19053e-05_rb,0.26738e-05_rb,0.26690e-05_rb,0.46656e-06_rb /)
      kbo(:, 4,33, 3) = (/ &
        & 0.86689e-09_rb,0.20038e-05_rb,0.28104e-05_rb,0.28139e-05_rb,0.49516e-06_rb /)
      kbo(:, 5,33, 3) = (/ &
        & 0.82630e-09_rb,0.20860e-05_rb,0.29294e-05_rb,0.29485e-05_rb,0.52462e-06_rb /)
      kbo(:, 1,34, 3) = (/ &
        & 0.82000e-09_rb,0.14231e-05_rb,0.19661e-05_rb,0.19700e-05_rb,0.33996e-06_rb /)
      kbo(:, 2,34, 3) = (/ &
        & 0.77672e-09_rb,0.15105e-05_rb,0.21047e-05_rb,0.21018e-05_rb,0.36435e-06_rb /)
      kbo(:, 3,34, 3) = (/ &
        & 0.73757e-09_rb,0.15908e-05_rb,0.22307e-05_rb,0.22272e-05_rb,0.38851e-06_rb /)
      kbo(:, 4,34, 3) = (/ &
        & 0.70193e-09_rb,0.16706e-05_rb,0.23421e-05_rb,0.23448e-05_rb,0.41219e-06_rb /)
      kbo(:, 5,34, 3) = (/ &
        & 0.66936e-09_rb,0.17330e-05_rb,0.24342e-05_rb,0.24538e-05_rb,0.43607e-06_rb /)
      kbo(:, 1,35, 3) = (/ &
        & 0.66652e-09_rb,0.11842e-05_rb,0.16383e-05_rb,0.16382e-05_rb,0.28183e-06_rb /)
      kbo(:, 2,35, 3) = (/ &
        & 0.63156e-09_rb,0.12541e-05_rb,0.17499e-05_rb,0.17457e-05_rb,0.30195e-06_rb /)
      kbo(:, 3,35, 3) = (/ &
        & 0.59990e-09_rb,0.13209e-05_rb,0.18504e-05_rb,0.18484e-05_rb,0.32174e-06_rb /)
      kbo(:, 4,35, 3) = (/ &
        & 0.57107e-09_rb,0.13850e-05_rb,0.19418e-05_rb,0.19438e-05_rb,0.34124e-06_rb /)
      kbo(:, 5,35, 3) = (/ &
        & 0.54470e-09_rb,0.14348e-05_rb,0.20135e-05_rb,0.20335e-05_rb,0.36050e-06_rb /)
      kbo(:, 1,36, 3) = (/ &
        & 0.54506e-09_rb,0.97870e-06_rb,0.13541e-05_rb,0.13520e-05_rb,0.23190e-06_rb /)
      kbo(:, 2,36, 3) = (/ &
        & 0.51650e-09_rb,0.10354e-05_rb,0.14455e-05_rb,0.14404e-05_rb,0.24845e-06_rb /)
      kbo(:, 3,36, 3) = (/ &
        & 0.49063e-09_rb,0.10910e-05_rb,0.15270e-05_rb,0.15247e-05_rb,0.26464e-06_rb /)
      kbo(:, 4,36, 3) = (/ &
        & 0.46707e-09_rb,0.11427e-05_rb,0.16012e-05_rb,0.16033e-05_rb,0.28072e-06_rb /)
      kbo(:, 5,36, 3) = (/ &
        & 0.44553e-09_rb,0.11833e-05_rb,0.16604e-05_rb,0.16774e-05_rb,0.29643e-06_rb /)
      kbo(:, 1,37, 3) = (/ &
        & 0.44949e-09_rb,0.80104e-06_rb,0.11066e-05_rb,0.11045e-05_rb,0.18888e-06_rb /)
      kbo(:, 2,37, 3) = (/ &
        & 0.42579e-09_rb,0.84810e-06_rb,0.11826e-05_rb,0.11777e-05_rb,0.20242e-06_rb /)
      kbo(:, 3,37, 3) = (/ &
        & 0.40435e-09_rb,0.89404e-06_rb,0.12504e-05_rb,0.12477e-05_rb,0.21578e-06_rb /)
      kbo(:, 4,37, 3) = (/ &
        & 0.38483e-09_rb,0.93737e-06_rb,0.13124e-05_rb,0.13129e-05_rb,0.22892e-06_rb /)
      kbo(:, 5,37, 3) = (/ &
        & 0.36699e-09_rb,0.97118e-06_rb,0.13617e-05_rb,0.13744e-05_rb,0.24190e-06_rb /)
      kbo(:, 1,38, 3) = (/ &
        & 0.37088e-09_rb,0.65468e-06_rb,0.90347e-06_rb,0.90140e-06_rb,0.15371e-06_rb /)
      kbo(:, 2,38, 3) = (/ &
        & 0.35120e-09_rb,0.69421e-06_rb,0.96676e-06_rb,0.96222e-06_rb,0.16484e-06_rb /)
      kbo(:, 3,38, 3) = (/ &
        & 0.33340e-09_rb,0.73211e-06_rb,0.10233e-05_rb,0.10203e-05_rb,0.17578e-06_rb /)
      kbo(:, 4,38, 3) = (/ &
        & 0.31721e-09_rb,0.76838e-06_rb,0.10750e-05_rb,0.10745e-05_rb,0.18656e-06_rb /)
      kbo(:, 5,38, 3) = (/ &
        & 0.30242e-09_rb,0.79676e-06_rb,0.11164e-05_rb,0.11256e-05_rb,0.19730e-06_rb /)
      kbo(:, 1,39, 3) = (/ &
        & 0.30603e-09_rb,0.53518e-06_rb,0.73755e-06_rb,0.73560e-06_rb,0.12509e-06_rb /)
      kbo(:, 2,39, 3) = (/ &
        & 0.28967e-09_rb,0.56827e-06_rb,0.79017e-06_rb,0.78607e-06_rb,0.13420e-06_rb /)
      kbo(:, 3,39, 3) = (/ &
        & 0.27490e-09_rb,0.59946e-06_rb,0.83729e-06_rb,0.83418e-06_rb,0.14320e-06_rb /)
      kbo(:, 4,39, 3) = (/ &
        & 0.26148e-09_rb,0.62968e-06_rb,0.88045e-06_rb,0.87919e-06_rb,0.15206e-06_rb /)
      kbo(:, 5,39, 3) = (/ &
        & 0.24922e-09_rb,0.65358e-06_rb,0.91511e-06_rb,0.92187e-06_rb,0.16092e-06_rb /)
      kbo(:, 1,40, 3) = (/ &
        & 0.25401e-09_rb,0.43491e-06_rb,0.59706e-06_rb,0.59583e-06_rb,0.10103e-06_rb /)
      kbo(:, 2,40, 3) = (/ &
        & 0.24027e-09_rb,0.46243e-06_rb,0.64127e-06_rb,0.63780e-06_rb,0.10854e-06_rb /)
      kbo(:, 3,40, 3) = (/ &
        & 0.22788e-09_rb,0.48789e-06_rb,0.68111e-06_rb,0.67785e-06_rb,0.11596e-06_rb /)
      kbo(:, 4,40, 3) = (/ &
        & 0.21664e-09_rb,0.51370e-06_rb,0.71776e-06_rb,0.71557e-06_rb,0.12318e-06_rb /)
      kbo(:, 5,40, 3) = (/ &
        & 0.20639e-09_rb,0.53413e-06_rb,0.74738e-06_rb,0.75116e-06_rb,0.13055e-06_rb /)
      kbo(:, 1,41, 3) = (/ &
        & 0.21103e-09_rb,0.35162e-06_rb,0.48245e-06_rb,0.48181e-06_rb,0.81551e-07_rb /)
      kbo(:, 2,41, 3) = (/ &
        & 0.19947e-09_rb,0.37577e-06_rb,0.51975e-06_rb,0.51683e-06_rb,0.87624e-07_rb /)
      kbo(:, 3,41, 3) = (/ &
        & 0.18906e-09_rb,0.39681e-06_rb,0.55372e-06_rb,0.55016e-06_rb,0.93775e-07_rb /)
      kbo(:, 4,41, 3) = (/ &
        & 0.17963e-09_rb,0.41856e-06_rb,0.58405e-06_rb,0.58177e-06_rb,0.99677e-07_rb /)
      kbo(:, 5,41, 3) = (/ &
        & 0.17105e-09_rb,0.43610e-06_rb,0.60957e-06_rb,0.61150e-06_rb,0.10578e-06_rb /)
      kbo(:, 1,42, 3) = (/ &
        & 0.17536e-09_rb,0.28412e-06_rb,0.38948e-06_rb,0.38933e-06_rb,0.65791e-07_rb /)
      kbo(:, 2,42, 3) = (/ &
        & 0.16563e-09_rb,0.30504e-06_rb,0.42097e-06_rb,0.41854e-06_rb,0.70752e-07_rb /)
      kbo(:, 3,42, 3) = (/ &
        & 0.15688e-09_rb,0.32265e-06_rb,0.44957e-06_rb,0.44624e-06_rb,0.75790e-07_rb /)
      kbo(:, 4,42, 3) = (/ &
        & 0.14897e-09_rb,0.34080e-06_rb,0.47502e-06_rb,0.47268e-06_rb,0.80648e-07_rb /)
      kbo(:, 5,42, 3) = (/ &
        & 0.14178e-09_rb,0.35584e-06_rb,0.49696e-06_rb,0.49747e-06_rb,0.85685e-07_rb /)
      kbo(:, 1,43, 3) = (/ &
        & 0.14625e-09_rb,0.22828e-06_rb,0.31269e-06_rb,0.31301e-06_rb,0.52858e-07_rb /)
      kbo(:, 2,43, 3) = (/ &
        & 0.13800e-09_rb,0.24660e-06_rb,0.33945e-06_rb,0.33736e-06_rb,0.56872e-07_rb /)
      kbo(:, 3,43, 3) = (/ &
        & 0.13060e-09_rb,0.26150e-06_rb,0.36382e-06_rb,0.36052e-06_rb,0.61005e-07_rb /)
      kbo(:, 4,43, 3) = (/ &
        & 0.12392e-09_rb,0.27637e-06_rb,0.38496e-06_rb,0.38262e-06_rb,0.65064e-07_rb /)
      kbo(:, 5,43, 3) = (/ &
        & 0.11786e-09_rb,0.28960e-06_rb,0.40432e-06_rb,0.40346e-06_rb,0.69134e-07_rb /)
      kbo(:, 1,44, 3) = (/ &
        & 0.12220e-09_rb,0.18299e-06_rb,0.25003e-06_rb,0.25075e-06_rb,0.42426e-07_rb /)
      kbo(:, 2,44, 3) = (/ &
        & 0.11518e-09_rb,0.19858e-06_rb,0.27271e-06_rb,0.27124e-06_rb,0.45605e-07_rb /)
      kbo(:, 3,44, 3) = (/ &
        & 0.10890e-09_rb,0.21149e-06_rb,0.29306e-06_rb,0.29059e-06_rb,0.48989e-07_rb /)
      kbo(:, 4,44, 3) = (/ &
        & 0.10324e-09_rb,0.22361e-06_rb,0.31128e-06_rb,0.30909e-06_rb,0.52357e-07_rb /)
      kbo(:, 5,44, 3) = (/ &
        & 0.98119e-10_rb,0.23540e-06_rb,0.32782e-06_rb,0.32657e-06_rb,0.55650e-07_rb /)
      kbo(:, 1,45, 3) = (/ &
        & 0.10215e-09_rb,0.14604e-06_rb,0.19957e-06_rb,0.20079e-06_rb,0.34009e-07_rb /)
      kbo(:, 2,45, 3) = (/ &
        & 0.96168e-10_rb,0.15976e-06_rb,0.21859e-06_rb,0.21782e-06_rb,0.36565e-07_rb /)
      kbo(:, 3,45, 3) = (/ &
        & 0.90832e-10_rb,0.17092e-06_rb,0.23594e-06_rb,0.23403e-06_rb,0.39336e-07_rb /)
      kbo(:, 4,45, 3) = (/ &
        & 0.86039e-10_rb,0.18076e-06_rb,0.25148e-06_rb,0.24940e-06_rb,0.42120e-07_rb /)
      kbo(:, 5,45, 3) = (/ &
        & 0.81705e-10_rb,0.19091e-06_rb,0.26579e-06_rb,0.26417e-06_rb,0.44806e-07_rb /)
      kbo(:, 1,46, 3) = (/ &
        & 0.85548e-10_rb,0.11606e-06_rb,0.15858e-06_rb,0.16025e-06_rb,0.27138e-07_rb /)
      kbo(:, 2,46, 3) = (/ &
        & 0.80430e-10_rb,0.12777e-06_rb,0.17465e-06_rb,0.17439e-06_rb,0.29260e-07_rb /)
      kbo(:, 3,46, 3) = (/ &
        & 0.75882e-10_rb,0.13771e-06_rb,0.18956e-06_rb,0.18801e-06_rb,0.31519e-07_rb /)
      kbo(:, 4,46, 3) = (/ &
        & 0.71807e-10_rb,0.14597e-06_rb,0.20282e-06_rb,0.20085e-06_rb,0.33790e-07_rb /)
      kbo(:, 5,46, 3) = (/ &
        & 0.68131e-10_rb,0.15454e-06_rb,0.21485e-06_rb,0.21321e-06_rb,0.36016e-07_rb /)
      kbo(:, 1,47, 3) = (/ &
        & 0.71880e-10_rb,0.91677e-07_rb,0.12518e-06_rb,0.12717e-06_rb,0.21548e-07_rb /)
      kbo(:, 2,47, 3) = (/ &
        & 0.67474e-10_rb,0.10182e-06_rb,0.13885e-06_rb,0.13887e-06_rb,0.23366e-07_rb /)
      kbo(:, 3,47, 3) = (/ &
        & 0.63573e-10_rb,0.11037e-06_rb,0.15155e-06_rb,0.15037e-06_rb,0.25140e-07_rb /)
      kbo(:, 4,47, 3) = (/ &
        & 0.60089e-10_rb,0.11757e-06_rb,0.16292e-06_rb,0.16115e-06_rb,0.27023e-07_rb /)
      kbo(:, 5,47, 3) = (/ &
        & 0.56955e-10_rb,0.12460e-06_rb,0.17303e-06_rb,0.17150e-06_rb,0.28886e-07_rb /)
      kbo(:, 1,48, 3) = (/ &
        & 0.60438e-10_rb,0.72378e-07_rb,0.98586e-07_rb,0.10075e-06_rb,0.17068e-07_rb /)
      kbo(:, 2,48, 3) = (/ &
        & 0.56639e-10_rb,0.80662e-07_rb,0.11003e-06_rb,0.11052e-06_rb,0.18643e-07_rb /)
      kbo(:, 3,48, 3) = (/ &
        & 0.53288e-10_rb,0.88391e-07_rb,0.12069e-06_rb,0.12007e-06_rb,0.20050e-07_rb /)
      kbo(:, 4,48, 3) = (/ &
        & 0.50306e-10_rb,0.94624e-07_rb,0.13045e-06_rb,0.12916e-06_rb,0.21587e-07_rb /)
      kbo(:, 5,48, 3) = (/ &
        & 0.47632e-10_rb,0.10026e-06_rb,0.13914e-06_rb,0.13778e-06_rb,0.23136e-07_rb /)
      kbo(:, 1,49, 3) = (/ &
        & 0.50856e-10_rb,0.57046e-07_rb,0.77565e-07_rb,0.79741e-07_rb,0.13510e-07_rb /)
      kbo(:, 2,49, 3) = (/ &
        & 0.47574e-10_rb,0.63730e-07_rb,0.86939e-07_rb,0.87826e-07_rb,0.14811e-07_rb /)
      kbo(:, 3,49, 3) = (/ &
        & 0.44692e-10_rb,0.70367e-07_rb,0.95974e-07_rb,0.95707e-07_rb,0.15992e-07_rb /)
      kbo(:, 4,49, 3) = (/ &
        & 0.42136e-10_rb,0.75959e-07_rb,0.10441e-06_rb,0.10336e-06_rb,0.17235e-07_rb /)
      kbo(:, 5,49, 3) = (/ &
        & 0.39851e-10_rb,0.80662e-07_rb,0.11179e-06_rb,0.11061e-06_rb,0.18501e-07_rb /)
      kbo(:, 1,50, 3) = (/ &
        & 0.42752e-10_rb,0.44994e-07_rb,0.60933e-07_rb,0.63170e-07_rb,0.10733e-07_rb /)
      kbo(:, 2,50, 3) = (/ &
        & 0.39920e-10_rb,0.50417e-07_rb,0.68695e-07_rb,0.69776e-07_rb,0.11778e-07_rb /)
      kbo(:, 3,50, 3) = (/ &
        & 0.37445e-10_rb,0.56129e-07_rb,0.76381e-07_rb,0.76292e-07_rb,0.12796e-07_rb /)
      kbo(:, 4,50, 3) = (/ &
        & 0.35258e-10_rb,0.60916e-07_rb,0.83533e-07_rb,0.82746e-07_rb,0.13770e-07_rb /)
      kbo(:, 5,50, 3) = (/ &
        & 0.33309e-10_rb,0.65010e-07_rb,0.89933e-07_rb,0.88828e-07_rb,0.14809e-07_rb /)
      kbo(:, 1,51, 3) = (/ &
        & 0.35935e-10_rb,0.35278e-07_rb,0.47901e-07_rb,0.50056e-07_rb,0.85239e-08_rb /)
      kbo(:, 2,51, 3) = (/ &
        & 0.33494e-10_rb,0.39897e-07_rb,0.54245e-07_rb,0.55439e-07_rb,0.93562e-08_rb /)
      kbo(:, 3,51, 3) = (/ &
        & 0.31368e-10_rb,0.44577e-07_rb,0.60687e-07_rb,0.60854e-07_rb,0.10229e-07_rb /)
      kbo(:, 4,51, 3) = (/ &
        & 0.29497e-10_rb,0.48906e-07_rb,0.66668e-07_rb,0.66217e-07_rb,0.11009e-07_rb /)
      kbo(:, 5,51, 3) = (/ &
        & 0.27834e-10_rb,0.52348e-07_rb,0.72154e-07_rb,0.71322e-07_rb,0.11856e-07_rb /)
      kbo(:, 1,52, 3) = (/ &
        & 0.30227e-10_rb,0.27572e-07_rb,0.37524e-07_rb,0.39754e-07_rb,0.67198e-08_rb /)
      kbo(:, 2,52, 3) = (/ &
        & 0.28118e-10_rb,0.31560e-07_rb,0.42810e-07_rb,0.44031e-07_rb,0.74267e-08_rb /)
      kbo(:, 3,52, 3) = (/ &
        & 0.26290e-10_rb,0.35319e-07_rb,0.48079e-07_rb,0.48486e-07_rb,0.81511e-08_rb /)
      kbo(:, 4,52, 3) = (/ &
        & 0.24687e-10_rb,0.39041e-07_rb,0.53141e-07_rb,0.52898e-07_rb,0.88044e-08_rb /)
      kbo(:, 5,52, 3) = (/ &
        & 0.23267e-10_rb,0.42123e-07_rb,0.57835e-07_rb,0.57177e-07_rb,0.94934e-08_rb /)
      kbo(:, 1,53, 3) = (/ &
        & 0.25448e-10_rb,0.21456e-07_rb,0.29324e-07_rb,0.31450e-07_rb,0.53035e-08_rb /)
      kbo(:, 2,53, 3) = (/ &
        & 0.23623e-10_rb,0.24922e-07_rb,0.33706e-07_rb,0.34903e-07_rb,0.59025e-08_rb /)
      kbo(:, 3,53, 3) = (/ &
        & 0.22048e-10_rb,0.27931e-07_rb,0.38009e-07_rb,0.38548e-07_rb,0.64842e-08_rb /)
      kbo(:, 4,53, 3) = (/ &
        & 0.20673e-10_rb,0.31117e-07_rb,0.42293e-07_rb,0.42180e-07_rb,0.70454e-08_rb /)
      kbo(:, 5,53, 3) = (/ &
        & 0.19459e-10_rb,0.33800e-07_rb,0.46316e-07_rb,0.45776e-07_rb,0.75886e-08_rb /)
      kbo(:, 1,54, 3) = (/ &
        & 0.21390e-10_rb,0.16743e-07_rb,0.23031e-07_rb,0.24877e-07_rb,0.41777e-08_rb /)
      kbo(:, 2,54, 3) = (/ &
        & 0.19815e-10_rb,0.19619e-07_rb,0.26556e-07_rb,0.27722e-07_rb,0.47031e-08_rb /)
      kbo(:, 3,54, 3) = (/ &
        & 0.18463e-10_rb,0.22167e-07_rb,0.30101e-07_rb,0.30708e-07_rb,0.51646e-08_rb /)
      kbo(:, 4,54, 3) = (/ &
        & 0.17286e-10_rb,0.24804e-07_rb,0.33704e-07_rb,0.33716e-07_rb,0.56448e-08_rb /)
      kbo(:, 5,54, 3) = (/ &
        & 0.16251e-10_rb,0.27183e-07_rb,0.37049e-07_rb,0.36696e-07_rb,0.60788e-08_rb /)
      kbo(:, 1,55, 3) = (/ &
        & 0.17973e-10_rb,0.13037e-07_rb,0.18115e-07_rb,0.19727e-07_rb,0.32912e-08_rb /)
      kbo(:, 2,55, 3) = (/ &
        & 0.16615e-10_rb,0.15402e-07_rb,0.20942e-07_rb,0.22043e-07_rb,0.37301e-08_rb /)
      kbo(:, 3,55, 3) = (/ &
        & 0.15454e-10_rb,0.17592e-07_rb,0.23848e-07_rb,0.24485e-07_rb,0.41129e-08_rb /)
      kbo(:, 4,55, 3) = (/ &
        & 0.14448e-10_rb,0.19733e-07_rb,0.26815e-07_rb,0.26954e-07_rb,0.45146e-08_rb /)
      kbo(:, 5,55, 3) = (/ &
        & 0.13565e-10_rb,0.21785e-07_rb,0.29633e-07_rb,0.29414e-07_rb,0.48762e-08_rb /)
      kbo(:, 1,56, 3) = (/ &
        & 0.15112e-10_rb,0.10127e-07_rb,0.14262e-07_rb,0.15637e-07_rb,0.25986e-08_rb /)
      kbo(:, 2,56, 3) = (/ &
        & 0.13939e-10_rb,0.12082e-07_rb,0.16458e-07_rb,0.17567e-07_rb,0.29510e-08_rb /)
      kbo(:, 3,56, 3) = (/ &
        & 0.12942e-10_rb,0.13980e-07_rb,0.18876e-07_rb,0.19499e-07_rb,0.32803e-08_rb /)
      kbo(:, 4,56, 3) = (/ &
        & 0.12080e-10_rb,0.15677e-07_rb,0.21298e-07_rb,0.21516e-07_rb,0.36078e-08_rb /)
      kbo(:, 5,56, 3) = (/ &
        & 0.11328e-10_rb,0.17421e-07_rb,0.23674e-07_rb,0.23552e-07_rb,0.39142e-08_rb /)
      kbo(:, 1,57, 3) = (/ &
        & 0.12716e-10_rb,0.78423e-08_rb,0.11218e-07_rb,0.12368e-07_rb,0.20572e-08_rb /)
      kbo(:, 2,57, 3) = (/ &
        & 0.11703e-10_rb,0.94468e-08_rb,0.12944e-07_rb,0.13939e-07_rb,0.23449e-08_rb /)
      kbo(:, 3,57, 3) = (/ &
        & 0.10844e-10_rb,0.11054e-07_rb,0.14933e-07_rb,0.15504e-07_rb,0.26161e-08_rb /)
      kbo(:, 4,57, 3) = (/ &
        & 0.10105e-10_rb,0.12445e-07_rb,0.16895e-07_rb,0.17167e-07_rb,0.28789e-08_rb /)
      kbo(:, 5,57, 3) = (/ &
        & 0.94629e-11_rb,0.13929e-07_rb,0.18884e-07_rb,0.18837e-07_rb,0.31391e-08_rb /)
      kbo(:, 1,58, 3) = (/ &
        & 0.10694e-10_rb,0.61084e-08_rb,0.88278e-08_rb,0.97801e-08_rb,0.16192e-08_rb /)
      kbo(:, 2,58, 3) = (/ &
        & 0.98180e-11_rb,0.73859e-08_rb,0.10206e-07_rb,0.11064e-07_rb,0.18486e-08_rb /)
      kbo(:, 3,58, 3) = (/ &
        & 0.90797e-11_rb,0.87113e-08_rb,0.11801e-07_rb,0.12348e-07_rb,0.20872e-08_rb /)
      kbo(:, 4,58, 3) = (/ &
        & 0.84478e-11_rb,0.99003e-08_rb,0.13409e-07_rb,0.13707e-07_rb,0.22967e-08_rb /)
      kbo(:, 5,58, 3) = (/ &
        & 0.78998e-11_rb,0.11108e-07_rb,0.15069e-07_rb,0.15082e-07_rb,0.25182e-08_rb /)
      kbo(:, 1,59, 3) = (/ &
        & 0.88513e-11_rb,0.49239e-08_rb,0.71386e-08_rb,0.79170e-08_rb,0.13089e-08_rb /)
      kbo(:, 2,59, 3) = (/ &
        & 0.81187e-11_rb,0.59541e-08_rb,0.82539e-08_rb,0.89758e-08_rb,0.14952e-08_rb /)
      kbo(:, 3,59, 3) = (/ &
        & 0.75021e-11_rb,0.70450e-08_rb,0.95567e-08_rb,0.10024e-07_rb,0.16919e-08_rb /)
      kbo(:, 4,59, 3) = (/ &
        & 0.69753e-11_rb,0.80391e-08_rb,0.10874e-07_rb,0.11143e-07_rb,0.18646e-08_rb /)
      kbo(:, 5,59, 3) = (/ &
        & 0.65191e-11_rb,0.90265e-08_rb,0.12240e-07_rb,0.12268e-07_rb,0.20471e-08_rb /)
      kbo(:, 1,13, 4) = (/ &
        & 0.40115e-06_rb,0.20542e-03_rb,0.29006e-03_rb,0.29983e-03_rb,0.86404e-04_rb /)
      kbo(:, 2,13, 4) = (/ &
        & 0.38503e-06_rb,0.22086e-03_rb,0.30833e-03_rb,0.32518e-03_rb,0.92611e-04_rb /)
      kbo(:, 3,13, 4) = (/ &
        & 0.37042e-06_rb,0.23415e-03_rb,0.32674e-03_rb,0.34814e-03_rb,0.98878e-04_rb /)
      kbo(:, 4,13, 4) = (/ &
        & 0.35699e-06_rb,0.24607e-03_rb,0.34335e-03_rb,0.36858e-03_rb,0.10559e-03_rb /)
      kbo(:, 5,13, 4) = (/ &
        & 0.34439e-06_rb,0.25741e-03_rb,0.35943e-03_rb,0.38716e-03_rb,0.11262e-03_rb /)
      kbo(:, 1,14, 4) = (/ &
        & 0.32717e-06_rb,0.17086e-03_rb,0.24121e-03_rb,0.25001e-03_rb,0.72120e-04_rb /)
      kbo(:, 2,14, 4) = (/ &
        & 0.31411e-06_rb,0.18358e-03_rb,0.25599e-03_rb,0.27089e-03_rb,0.77351e-04_rb /)
      kbo(:, 3,14, 4) = (/ &
        & 0.30225e-06_rb,0.19456e-03_rb,0.27149e-03_rb,0.29002e-03_rb,0.82676e-04_rb /)
      kbo(:, 4,14, 4) = (/ &
        & 0.29132e-06_rb,0.20460e-03_rb,0.28544e-03_rb,0.30678e-03_rb,0.88331e-04_rb /)
      kbo(:, 5,14, 4) = (/ &
        & 0.28105e-06_rb,0.21410e-03_rb,0.29919e-03_rb,0.32271e-03_rb,0.94451e-04_rb /)
      kbo(:, 1,15, 4) = (/ &
        & 0.26673e-06_rb,0.14221e-03_rb,0.20048e-03_rb,0.20843e-03_rb,0.60213e-04_rb /)
      kbo(:, 2,15, 4) = (/ &
        & 0.25615e-06_rb,0.15268e-03_rb,0.21264e-03_rb,0.22531e-03_rb,0.64531e-04_rb /)
      kbo(:, 3,15, 4) = (/ &
        & 0.24652e-06_rb,0.16170e-03_rb,0.22567e-03_rb,0.24157e-03_rb,0.69102e-04_rb /)
      kbo(:, 4,15, 4) = (/ &
        & 0.23764e-06_rb,0.17023e-03_rb,0.23750e-03_rb,0.25543e-03_rb,0.73891e-04_rb /)
      kbo(:, 5,15, 4) = (/ &
        & 0.22926e-06_rb,0.17815e-03_rb,0.24912e-03_rb,0.26892e-03_rb,0.79167e-04_rb /)
      kbo(:, 1,16, 4) = (/ &
        & 0.21740e-06_rb,0.11824e-03_rb,0.16644e-03_rb,0.17381e-03_rb,0.50227e-04_rb /)
      kbo(:, 2,16, 4) = (/ &
        & 0.20884e-06_rb,0.12696e-03_rb,0.17683e-03_rb,0.18780e-03_rb,0.53903e-04_rb /)
      kbo(:, 3,16, 4) = (/ &
        & 0.20104e-06_rb,0.13434e-03_rb,0.18764e-03_rb,0.20094e-03_rb,0.57777e-04_rb /)
      kbo(:, 4,16, 4) = (/ &
        & 0.19381e-06_rb,0.14177e-03_rb,0.19757e-03_rb,0.21259e-03_rb,0.61821e-04_rb /)
      kbo(:, 5,16, 4) = (/ &
        & 0.18698e-06_rb,0.14828e-03_rb,0.20747e-03_rb,0.22416e-03_rb,0.66326e-04_rb /)
      kbo(:, 1,17, 4) = (/ &
        & 0.17724e-06_rb,0.98287e-04_rb,0.13813e-03_rb,0.14496e-03_rb,0.41851e-04_rb /)
      kbo(:, 2,17, 4) = (/ &
        & 0.17029e-06_rb,0.10548e-03_rb,0.14701e-03_rb,0.15636e-03_rb,0.44943e-04_rb /)
      kbo(:, 3,17, 4) = (/ &
        & 0.16396e-06_rb,0.11169e-03_rb,0.15605e-03_rb,0.16727e-03_rb,0.48176e-04_rb /)
      kbo(:, 4,17, 4) = (/ &
        & 0.15809e-06_rb,0.11795e-03_rb,0.16435e-03_rb,0.17684e-03_rb,0.51716e-04_rb /)
      kbo(:, 5,17, 4) = (/ &
        & 0.15251e-06_rb,0.12332e-03_rb,0.17265e-03_rb,0.18683e-03_rb,0.55480e-04_rb /)
      kbo(:, 1,18, 4) = (/ &
        & 0.14450e-06_rb,0.81703e-04_rb,0.11464e-03_rb,0.12079e-03_rb,0.34849e-04_rb /)
      kbo(:, 2,18, 4) = (/ &
        & 0.13887e-06_rb,0.87580e-04_rb,0.12224e-03_rb,0.13028e-03_rb,0.37465e-04_rb /)
      kbo(:, 3,18, 4) = (/ &
        & 0.13373e-06_rb,0.92869e-04_rb,0.12966e-03_rb,0.13906e-03_rb,0.40225e-04_rb /)
      kbo(:, 4,18, 4) = (/ &
        & 0.12896e-06_rb,0.98034e-04_rb,0.13668e-03_rb,0.14712e-03_rb,0.43227e-04_rb /)
      kbo(:, 5,18, 4) = (/ &
        & 0.12439e-06_rb,0.10250e-03_rb,0.14367e-03_rb,0.15557e-03_rb,0.46383e-04_rb /)
      kbo(:, 1,19, 4) = (/ &
        & 0.11782e-06_rb,0.67922e-04_rb,0.95121e-04_rb,0.10059e-03_rb,0.29014e-04_rb /)
      kbo(:, 2,19, 4) = (/ &
        & 0.11325e-06_rb,0.72759e-04_rb,0.10166e-03_rb,0.10843e-03_rb,0.31239e-04_rb /)
      kbo(:, 3,19, 4) = (/ &
        & 0.10909e-06_rb,0.77213e-04_rb,0.10769e-03_rb,0.11563e-03_rb,0.33594e-04_rb /)
      kbo(:, 4,19, 4) = (/ &
        & 0.10521e-06_rb,0.81507e-04_rb,0.11367e-03_rb,0.12244e-03_rb,0.36087e-04_rb /)
      kbo(:, 5,19, 4) = (/ &
        & 0.10146e-06_rb,0.85229e-04_rb,0.11951e-03_rb,0.12955e-03_rb,0.38760e-04_rb /)
      kbo(:, 1,20, 4) = (/ &
        & 0.96005e-07_rb,0.56539e-04_rb,0.79048e-04_rb,0.83638e-04_rb,0.24182e-04_rb /)
      kbo(:, 2,20, 4) = (/ &
        & 0.92311e-07_rb,0.60522e-04_rb,0.84506e-04_rb,0.90308e-04_rb,0.26052e-04_rb /)
      kbo(:, 3,20, 4) = (/ &
        & 0.88927e-07_rb,0.64305e-04_rb,0.89608e-04_rb,0.96229e-04_rb,0.28071e-04_rb /)
      kbo(:, 4,20, 4) = (/ &
        & 0.85775e-07_rb,0.67811e-04_rb,0.94659e-04_rb,0.10203e-03_rb,0.30185e-04_rb /)
      kbo(:, 5,20, 4) = (/ &
        & 0.82709e-07_rb,0.70967e-04_rb,0.99551e-04_rb,0.10796e-03_rb,0.32438e-04_rb /)
      kbo(:, 1,21, 4) = (/ &
        & 0.78225e-07_rb,0.47094e-04_rb,0.65736e-04_rb,0.69706e-04_rb,0.20168e-04_rb /)
      kbo(:, 2,21, 4) = (/ &
        & 0.75235e-07_rb,0.50383e-04_rb,0.70309e-04_rb,0.75210e-04_rb,0.21748e-04_rb /)
      kbo(:, 3,21, 4) = (/ &
        & 0.72491e-07_rb,0.53581e-04_rb,0.74579e-04_rb,0.80106e-04_rb,0.23442e-04_rb /)
      kbo(:, 4,21, 4) = (/ &
        & 0.69929e-07_rb,0.56448e-04_rb,0.78864e-04_rb,0.85108e-04_rb,0.25248e-04_rb /)
      kbo(:, 5,21, 4) = (/ &
        & 0.67419e-07_rb,0.59090e-04_rb,0.82932e-04_rb,0.89870e-04_rb,0.27098e-04_rb /)
      kbo(:, 1,22, 4) = (/ &
        & 0.63572e-07_rb,0.39426e-04_rb,0.54979e-04_rb,0.58443e-04_rb,0.16913e-04_rb /)
      kbo(:, 2,22, 4) = (/ &
        & 0.61166e-07_rb,0.42097e-04_rb,0.58779e-04_rb,0.62933e-04_rb,0.18259e-04_rb /)
      kbo(:, 3,22, 4) = (/ &
        & 0.58951e-07_rb,0.44794e-04_rb,0.62330e-04_rb,0.66952e-04_rb,0.19670e-04_rb /)
      kbo(:, 4,22, 4) = (/ &
        & 0.56872e-07_rb,0.47147e-04_rb,0.65961e-04_rb,0.71230e-04_rb,0.21221e-04_rb /)
      kbo(:, 5,22, 4) = (/ &
        & 0.54822e-07_rb,0.49377e-04_rb,0.69234e-04_rb,0.75038e-04_rb,0.22743e-04_rb /)
      kbo(:, 1,23, 4) = (/ &
        & 0.51656e-07_rb,0.32950e-04_rb,0.46052e-04_rb,0.49084e-04_rb,0.14181e-04_rb /)
      kbo(:, 2,23, 4) = (/ &
        & 0.49721e-07_rb,0.35250e-04_rb,0.49106e-04_rb,0.52675e-04_rb,0.15330e-04_rb /)
      kbo(:, 3,23, 4) = (/ &
        & 0.47930e-07_rb,0.37438e-04_rb,0.52152e-04_rb,0.56097e-04_rb,0.16532e-04_rb /)
      kbo(:, 4,23, 4) = (/ &
        & 0.46242e-07_rb,0.39418e-04_rb,0.55160e-04_rb,0.59674e-04_rb,0.17811e-04_rb /)
      kbo(:, 5,23, 4) = (/ &
        & 0.44570e-07_rb,0.41280e-04_rb,0.57899e-04_rb,0.62654e-04_rb,0.19100e-04_rb /)
      kbo(:, 1,24, 4) = (/ &
        & 0.41962e-07_rb,0.27608e-04_rb,0.38527e-04_rb,0.41167e-04_rb,0.11908e-04_rb /)
      kbo(:, 2,24, 4) = (/ &
        & 0.40405e-07_rb,0.29547e-04_rb,0.41099e-04_rb,0.44081e-04_rb,0.12880e-04_rb /)
      kbo(:, 3,24, 4) = (/ &
        & 0.38959e-07_rb,0.31321e-04_rb,0.43687e-04_rb,0.47051e-04_rb,0.13912e-04_rb /)
      kbo(:, 4,24, 4) = (/ &
        & 0.37583e-07_rb,0.32994e-04_rb,0.46190e-04_rb,0.49890e-04_rb,0.14961e-04_rb /)
      kbo(:, 5,24, 4) = (/ &
        & 0.36227e-07_rb,0.34560e-04_rb,0.48473e-04_rb,0.52358e-04_rb,0.16039e-04_rb /)
      kbo(:, 1,25, 4) = (/ &
        & 0.34075e-07_rb,0.23153e-04_rb,0.32305e-04_rb,0.34566e-04_rb,0.10014e-04_rb /)
      kbo(:, 2,25, 4) = (/ &
        & 0.32822e-07_rb,0.24769e-04_rb,0.34435e-04_rb,0.36968e-04_rb,0.10822e-04_rb /)
      kbo(:, 3,25, 4) = (/ &
        & 0.31655e-07_rb,0.26242e-04_rb,0.36635e-04_rb,0.39484e-04_rb,0.11699e-04_rb /)
      kbo(:, 4,25, 4) = (/ &
        & 0.30528e-07_rb,0.27624e-04_rb,0.38648e-04_rb,0.41770e-04_rb,0.12588e-04_rb /)
      kbo(:, 5,25, 4) = (/ &
        & 0.29434e-07_rb,0.28951e-04_rb,0.40632e-04_rb,0.43774e-04_rb,0.13479e-04_rb /)
      kbo(:, 1,26, 4) = (/ &
        & 0.27652e-07_rb,0.19460e-04_rb,0.27114e-04_rb,0.29052e-04_rb,0.84372e-05_rb /)
      kbo(:, 2,26, 4) = (/ &
        & 0.26645e-07_rb,0.20785e-04_rb,0.28933e-04_rb,0.31091e-04_rb,0.91305e-05_rb /)
      kbo(:, 3,26, 4) = (/ &
        & 0.25703e-07_rb,0.22018e-04_rb,0.30749e-04_rb,0.33173e-04_rb,0.98560e-05_rb /)
      kbo(:, 4,26, 4) = (/ &
        & 0.24780e-07_rb,0.23166e-04_rb,0.32405e-04_rb,0.34971e-04_rb,0.10593e-04_rb /)
      kbo(:, 5,26, 4) = (/ &
        & 0.23900e-07_rb,0.24286e-04_rb,0.34048e-04_rb,0.36663e-04_rb,0.11346e-04_rb /)
      kbo(:, 1,27, 4) = (/ &
        & 0.22439e-07_rb,0.16370e-04_rb,0.22763e-04_rb,0.24397e-04_rb,0.71084e-05_rb /)
      kbo(:, 2,27, 4) = (/ &
        & 0.21628e-07_rb,0.17461e-04_rb,0.24318e-04_rb,0.26140e-04_rb,0.76961e-05_rb /)
      kbo(:, 3,27, 4) = (/ &
        & 0.20865e-07_rb,0.18493e-04_rb,0.25826e-04_rb,0.27824e-04_rb,0.83080e-05_rb /)
      kbo(:, 4,27, 4) = (/ &
        & 0.20113e-07_rb,0.19452e-04_rb,0.27215e-04_rb,0.29287e-04_rb,0.89207e-05_rb /)
      kbo(:, 5,27, 4) = (/ &
        & 0.19405e-07_rb,0.20376e-04_rb,0.28543e-04_rb,0.30710e-04_rb,0.95477e-05_rb /)
      kbo(:, 1,28, 4) = (/ &
        & 0.18208e-07_rb,0.13777e-04_rb,0.19138e-04_rb,0.20524e-04_rb,0.59952e-05_rb /)
      kbo(:, 2,28, 4) = (/ &
        & 0.17555e-07_rb,0.14679e-04_rb,0.20448e-04_rb,0.21996e-04_rb,0.64907e-05_rb /)
      kbo(:, 3,28, 4) = (/ &
        & 0.16936e-07_rb,0.15522e-04_rb,0.21661e-04_rb,0.23357e-04_rb,0.69966e-05_rb /)
      kbo(:, 4,28, 4) = (/ &
        & 0.16325e-07_rb,0.16334e-04_rb,0.22846e-04_rb,0.24537e-04_rb,0.75124e-05_rb /)
      kbo(:, 5,28, 4) = (/ &
        & 0.15755e-07_rb,0.17088e-04_rb,0.23931e-04_rb,0.25722e-04_rb,0.80305e-05_rb /)
      kbo(:, 1,29, 4) = (/ &
        & 0.14772e-07_rb,0.11598e-04_rb,0.16117e-04_rb,0.17294e-04_rb,0.50602e-05_rb /)
      kbo(:, 2,29, 4) = (/ &
        & 0.14246e-07_rb,0.12346e-04_rb,0.17198e-04_rb,0.18521e-04_rb,0.54799e-05_rb /)
      kbo(:, 3,29, 4) = (/ &
        & 0.13739e-07_rb,0.13047e-04_rb,0.18205e-04_rb,0.19566e-04_rb,0.58992e-05_rb /)
      kbo(:, 4,29, 4) = (/ &
        & 0.13247e-07_rb,0.13729e-04_rb,0.19173e-04_rb,0.20574e-04_rb,0.63251e-05_rb /)
      kbo(:, 5,29, 4) = (/ &
        & 0.12789e-07_rb,0.14341e-04_rb,0.20068e-04_rb,0.21536e-04_rb,0.67496e-05_rb /)
      kbo(:, 1,30, 4) = (/ &
        & 0.11984e-07_rb,0.97679e-05_rb,0.13568e-04_rb,0.14565e-04_rb,0.42721e-05_rb /)
      kbo(:, 2,30, 4) = (/ &
        & 0.11560e-07_rb,0.10389e-04_rb,0.14461e-04_rb,0.15560e-04_rb,0.46182e-05_rb /)
      kbo(:, 3,30, 4) = (/ &
        & 0.11144e-07_rb,0.10973e-04_rb,0.15302e-04_rb,0.16410e-04_rb,0.49715e-05_rb /)
      kbo(:, 4,30, 4) = (/ &
        & 0.10750e-07_rb,0.11532e-04_rb,0.16091e-04_rb,0.17244e-04_rb,0.53212e-05_rb /)
      kbo(:, 5,30, 4) = (/ &
        & 0.10381e-07_rb,0.12034e-04_rb,0.16821e-04_rb,0.18023e-04_rb,0.56712e-05_rb /)
      kbo(:, 1,31, 4) = (/ &
        & 0.97205e-08_rb,0.82296e-05_rb,0.11440e-04_rb,0.12288e-04_rb,0.36079e-05_rb /)
      kbo(:, 2,31, 4) = (/ &
        & 0.93777e-08_rb,0.87431e-05_rb,0.12158e-04_rb,0.13067e-04_rb,0.38974e-05_rb /)
      kbo(:, 3,31, 4) = (/ &
        & 0.90391e-08_rb,0.92300e-05_rb,0.12862e-04_rb,0.13769e-04_rb,0.41885e-05_rb /)
      kbo(:, 4,31, 4) = (/ &
        & 0.87219e-08_rb,0.96854e-05_rb,0.13508e-04_rb,0.14454e-04_rb,0.44756e-05_rb /)
      kbo(:, 5,31, 4) = (/ &
        & 0.84251e-08_rb,0.10101e-04_rb,0.14090e-04_rb,0.15064e-04_rb,0.47643e-05_rb /)
      kbo(:, 1,32, 4) = (/ &
        & 0.78842e-08_rb,0.69379e-05_rb,0.96378e-05_rb,0.10350e-04_rb,0.30462e-05_rb /)
      kbo(:, 2,32, 4) = (/ &
        & 0.76058e-08_rb,0.73605e-05_rb,0.10230e-04_rb,0.10964e-04_rb,0.32879e-05_rb /)
      kbo(:, 3,32, 4) = (/ &
        & 0.73312e-08_rb,0.77685e-05_rb,0.10806e-04_rb,0.11552e-04_rb,0.35258e-05_rb /)
      kbo(:, 4,32, 4) = (/ &
        & 0.70761e-08_rb,0.81393e-05_rb,0.11329e-04_rb,0.12109e-04_rb,0.37642e-05_rb /)
      kbo(:, 5,32, 4) = (/ &
        & 0.68375e-08_rb,0.84723e-05_rb,0.11806e-04_rb,0.12582e-04_rb,0.40027e-05_rb /)
      kbo(:, 1,33, 4) = (/ &
        & 0.63946e-08_rb,0.58484e-05_rb,0.81132e-05_rb,0.87138e-05_rb,0.25722e-05_rb /)
      kbo(:, 2,33, 4) = (/ &
        & 0.61668e-08_rb,0.61985e-05_rb,0.86152e-05_rb,0.92043e-05_rb,0.27716e-05_rb /)
      kbo(:, 3,33, 4) = (/ &
        & 0.59460e-08_rb,0.65329e-05_rb,0.90761e-05_rb,0.96914e-05_rb,0.29672e-05_rb /)
      kbo(:, 4,33, 4) = (/ &
        & 0.57410e-08_rb,0.68349e-05_rb,0.94991e-05_rb,0.10128e-04_rb,0.31649e-05_rb /)
      kbo(:, 5,33, 4) = (/ &
        & 0.55490e-08_rb,0.71098e-05_rb,0.98972e-05_rb,0.10494e-04_rb,0.33613e-05_rb /)
      kbo(:, 1,34, 4) = (/ &
        & 0.51931e-08_rb,0.49155e-05_rb,0.68122e-05_rb,0.73003e-05_rb,0.21648e-05_rb /)
      kbo(:, 2,34, 4) = (/ &
        & 0.50066e-08_rb,0.52108e-05_rb,0.72299e-05_rb,0.77128e-05_rb,0.23280e-05_rb /)
      kbo(:, 3,34, 4) = (/ &
        & 0.48289e-08_rb,0.54838e-05_rb,0.76076e-05_rb,0.81077e-05_rb,0.24906e-05_rb /)
      kbo(:, 4,34, 4) = (/ &
        & 0.46636e-08_rb,0.57271e-05_rb,0.79460e-05_rb,0.84494e-05_rb,0.26534e-05_rb /)
      kbo(:, 5,34, 4) = (/ &
        & 0.45089e-08_rb,0.59543e-05_rb,0.82751e-05_rb,0.87422e-05_rb,0.28159e-05_rb /)
      kbo(:, 1,35, 4) = (/ &
        & 0.42177e-08_rb,0.41104e-05_rb,0.56914e-05_rb,0.60852e-05_rb,0.18078e-05_rb /)
      kbo(:, 2,35, 4) = (/ &
        & 0.40795e-08_rb,0.43580e-05_rb,0.60334e-05_rb,0.64274e-05_rb,0.19421e-05_rb /)
      kbo(:, 3,35, 4) = (/ &
        & 0.39353e-08_rb,0.45814e-05_rb,0.63407e-05_rb,0.67443e-05_rb,0.20767e-05_rb /)
      kbo(:, 4,35, 4) = (/ &
        & 0.38012e-08_rb,0.47803e-05_rb,0.66196e-05_rb,0.70144e-05_rb,0.22114e-05_rb /)
      kbo(:, 5,35, 4) = (/ &
        & 0.36756e-08_rb,0.49682e-05_rb,0.68884e-05_rb,0.72539e-05_rb,0.23458e-05_rb /)
      kbo(:, 1,36, 4) = (/ &
        & 0.34162e-08_rb,0.34141e-05_rb,0.47224e-05_rb,0.50412e-05_rb,0.14965e-05_rb /)
      kbo(:, 2,36, 4) = (/ &
        & 0.33196e-08_rb,0.36215e-05_rb,0.50037e-05_rb,0.53241e-05_rb,0.16078e-05_rb /)
      kbo(:, 3,36, 4) = (/ &
        & 0.32195e-08_rb,0.38064e-05_rb,0.52583e-05_rb,0.55817e-05_rb,0.17188e-05_rb /)
      kbo(:, 4,36, 4) = (/ &
        & 0.31099e-08_rb,0.39715e-05_rb,0.54881e-05_rb,0.58006e-05_rb,0.18307e-05_rb /)
      kbo(:, 5,36, 4) = (/ &
        & 0.30072e-08_rb,0.41270e-05_rb,0.57065e-05_rb,0.59974e-05_rb,0.19419e-05_rb /)
      kbo(:, 1,37, 4) = (/ &
        & 0.28008e-08_rb,0.28094e-05_rb,0.38801e-05_rb,0.41393e-05_rb,0.12248e-05_rb /)
      kbo(:, 2,37, 4) = (/ &
        & 0.27121e-08_rb,0.29836e-05_rb,0.41155e-05_rb,0.43748e-05_rb,0.13168e-05_rb /)
      kbo(:, 3,37, 4) = (/ &
        & 0.26482e-08_rb,0.31397e-05_rb,0.43282e-05_rb,0.45885e-05_rb,0.14087e-05_rb /)
      kbo(:, 4,37, 4) = (/ &
        & 0.25576e-08_rb,0.32779e-05_rb,0.45200e-05_rb,0.47706e-05_rb,0.15013e-05_rb /)
      kbo(:, 5,37, 4) = (/ &
        & 0.24727e-08_rb,0.34082e-05_rb,0.47023e-05_rb,0.49354e-05_rb,0.15935e-05_rb /)
      kbo(:, 1,38, 4) = (/ &
        & 0.22984e-08_rb,0.23106e-05_rb,0.31856e-05_rb,0.33961e-05_rb,0.10016e-05_rb /)
      kbo(:, 2,38, 4) = (/ &
        & 0.22220e-08_rb,0.24563e-05_rb,0.33826e-05_rb,0.35915e-05_rb,0.10776e-05_rb /)
      kbo(:, 3,38, 4) = (/ &
        & 0.21588e-08_rb,0.25882e-05_rb,0.35600e-05_rb,0.37696e-05_rb,0.11538e-05_rb /)
      kbo(:, 4,38, 4) = (/ &
        & 0.21040e-08_rb,0.27033e-05_rb,0.37199e-05_rb,0.39213e-05_rb,0.12304e-05_rb /)
      kbo(:, 5,38, 4) = (/ &
        & 0.20339e-08_rb,0.28122e-05_rb,0.38728e-05_rb,0.40597e-05_rb,0.13067e-05_rb /)
      kbo(:, 1,39, 4) = (/ &
        & 0.18867e-08_rb,0.18990e-05_rb,0.26147e-05_rb,0.27855e-05_rb,0.81895e-06_rb /)
      kbo(:, 2,39, 4) = (/ &
        & 0.18219e-08_rb,0.20211e-05_rb,0.27789e-05_rb,0.29482e-05_rb,0.88168e-06_rb /)
      kbo(:, 3,39, 4) = (/ &
        & 0.17671e-08_rb,0.21324e-05_rb,0.29272e-05_rb,0.30962e-05_rb,0.94475e-06_rb /)
      kbo(:, 4,39, 4) = (/ &
        & 0.17201e-08_rb,0.22287e-05_rb,0.30607e-05_rb,0.32231e-05_rb,0.10080e-05_rb /)
      kbo(:, 5,39, 4) = (/ &
        & 0.16729e-08_rb,0.23197e-05_rb,0.31885e-05_rb,0.33389e-05_rb,0.10713e-05_rb /)
      kbo(:, 1,40, 4) = (/ &
        & 0.15567e-08_rb,0.15486e-05_rb,0.21299e-05_rb,0.22697e-05_rb,0.66418e-06_rb /)
      kbo(:, 2,40, 4) = (/ &
        & 0.15030e-08_rb,0.16525e-05_rb,0.22686e-05_rb,0.24063e-05_rb,0.71591e-06_rb /)
      kbo(:, 3,40, 4) = (/ &
        & 0.14559e-08_rb,0.17468e-05_rb,0.23938e-05_rb,0.25304e-05_rb,0.76804e-06_rb /)
      kbo(:, 4,40, 4) = (/ &
        & 0.14147e-08_rb,0.18284e-05_rb,0.25062e-05_rb,0.26377e-05_rb,0.82053e-06_rb /)
      kbo(:, 5,40, 4) = (/ &
        & 0.13808e-08_rb,0.19051e-05_rb,0.26142e-05_rb,0.27362e-05_rb,0.87282e-06_rb /)
      kbo(:, 1,41, 4) = (/ &
        & 0.12855e-08_rb,0.12622e-05_rb,0.17326e-05_rb,0.18472e-05_rb,0.53778e-06_rb /)
      kbo(:, 2,41, 4) = (/ &
        & 0.12414e-08_rb,0.13486e-05_rb,0.18494e-05_rb,0.19612e-05_rb,0.58062e-06_rb /)
      kbo(:, 3,41, 4) = (/ &
        & 0.12006e-08_rb,0.14290e-05_rb,0.19551e-05_rb,0.20665e-05_rb,0.62365e-06_rb /)
      kbo(:, 4,41, 4) = (/ &
        & 0.11655e-08_rb,0.14984e-05_rb,0.20506e-05_rb,0.21570e-05_rb,0.66707e-06_rb /)
      kbo(:, 5,41, 4) = (/ &
        & 0.11366e-08_rb,0.15630e-05_rb,0.21416e-05_rb,0.22404e-05_rb,0.71035e-06_rb /)
      kbo(:, 1,42, 4) = (/ &
        & 0.10616e-08_rb,0.10273e-05_rb,0.14085e-05_rb,0.15034e-05_rb,0.43514e-06_rb /)
      kbo(:, 2,42, 4) = (/ &
        & 0.10250e-08_rb,0.10997e-05_rb,0.15070e-05_rb,0.15973e-05_rb,0.47061e-06_rb /)
      kbo(:, 3,42, 4) = (/ &
        & 0.99030e-09_rb,0.11679e-05_rb,0.15953e-05_rb,0.16861e-05_rb,0.50613e-06_rb /)
      kbo(:, 4,42, 4) = (/ &
        & 0.96062e-09_rb,0.12272e-05_rb,0.16766e-05_rb,0.17642e-05_rb,0.54216e-06_rb /)
      kbo(:, 5,42, 4) = (/ &
        & 0.93443e-09_rb,0.12817e-05_rb,0.17529e-05_rb,0.18333e-05_rb,0.57784e-06_rb /)
      kbo(:, 1,43, 4) = (/ &
        & 0.87883e-09_rb,0.83199e-06_rb,0.11402e-05_rb,0.12168e-05_rb,0.35025e-06_rb /)
      kbo(:, 2,43, 4) = (/ &
        & 0.84829e-09_rb,0.89254e-06_rb,0.12225e-05_rb,0.12958e-05_rb,0.37960e-06_rb /)
      kbo(:, 3,43, 4) = (/ &
        & 0.81903e-09_rb,0.95072e-06_rb,0.12966e-05_rb,0.13708e-05_rb,0.40893e-06_rb /)
      kbo(:, 4,43, 4) = (/ &
        & 0.79369e-09_rb,0.10020e-05_rb,0.13665e-05_rb,0.14379e-05_rb,0.43862e-06_rb /)
      kbo(:, 5,43, 4) = (/ &
        & 0.77108e-09_rb,0.10478e-05_rb,0.14302e-05_rb,0.14962e-05_rb,0.46817e-06_rb /)
      kbo(:, 1,44, 4) = (/ &
        & 0.72838e-09_rb,0.67080e-06_rb,0.91962e-06_rb,0.98102e-06_rb,0.28107e-06_rb /)
      kbo(:, 2,44, 4) = (/ &
        & 0.70279e-09_rb,0.72271e-06_rb,0.98853e-06_rb,0.10491e-05_rb,0.30540e-06_rb /)
      kbo(:, 3,44, 4) = (/ &
        & 0.67858e-09_rb,0.77203e-06_rb,0.10522e-05_rb,0.11122e-05_rb,0.32958e-06_rb /)
      kbo(:, 4,44, 4) = (/ &
        & 0.65651e-09_rb,0.81625e-06_rb,0.11115e-05_rb,0.11697e-05_rb,0.35407e-06_rb /)
      kbo(:, 5,44, 4) = (/ &
        & 0.63727e-09_rb,0.85495e-06_rb,0.11651e-05_rb,0.12192e-05_rb,0.37850e-06_rb /)
      kbo(:, 1,45, 4) = (/ &
        & 0.60379e-09_rb,0.54047e-06_rb,0.74128e-06_rb,0.79100e-06_rb,0.22535e-06_rb /)
      kbo(:, 2,45, 4) = (/ &
        & 0.58228e-09_rb,0.58429e-06_rb,0.79854e-06_rb,0.84841e-06_rb,0.24548e-06_rb /)
      kbo(:, 3,45, 4) = (/ &
        & 0.56220e-09_rb,0.62606e-06_rb,0.85283e-06_rb,0.90157e-06_rb,0.26541e-06_rb /)
      kbo(:, 4,45, 4) = (/ &
        & 0.54320e-09_rb,0.66423e-06_rb,0.90296e-06_rb,0.95015e-06_rb,0.28558e-06_rb /)
      kbo(:, 5,45, 4) = (/ &
        & 0.52686e-09_rb,0.69719e-06_rb,0.94830e-06_rb,0.99254e-06_rb,0.30580e-06_rb /)
      kbo(:, 1,46, 4) = (/ &
        & 0.50108e-09_rb,0.43361e-06_rb,0.59554e-06_rb,0.63432e-06_rb,0.18023e-06_rb /)
      kbo(:, 2,46, 4) = (/ &
        & 0.48291e-09_rb,0.47127e-06_rb,0.64349e-06_rb,0.68510e-06_rb,0.19677e-06_rb /)
      kbo(:, 3,46, 4) = (/ &
        & 0.46612e-09_rb,0.50621e-06_rb,0.68942e-06_rb,0.72929e-06_rb,0.21325e-06_rb /)
      kbo(:, 4,46, 4) = (/ &
        & 0.45004e-09_rb,0.53903e-06_rb,0.73169e-06_rb,0.77095e-06_rb,0.22986e-06_rb /)
      kbo(:, 5,46, 4) = (/ &
        & 0.43600e-09_rb,0.56740e-06_rb,0.77081e-06_rb,0.80714e-06_rb,0.24654e-06_rb /)
      kbo(:, 1,47, 4) = (/ &
        & 0.41665e-09_rb,0.34614e-06_rb,0.47565e-06_rb,0.50598e-06_rb,0.14345e-06_rb /)
      kbo(:, 2,47, 4) = (/ &
        & 0.40119e-09_rb,0.37779e-06_rb,0.51642e-06_rb,0.55015e-06_rb,0.15697e-06_rb /)
      kbo(:, 3,47, 4) = (/ &
        & 0.38702e-09_rb,0.40752e-06_rb,0.55525e-06_rb,0.58759e-06_rb,0.17064e-06_rb /)
      kbo(:, 4,47, 4) = (/ &
        & 0.37372e-09_rb,0.43563e-06_rb,0.59076e-06_rb,0.62318e-06_rb,0.18430e-06_rb /)
      kbo(:, 5,47, 4) = (/ &
        & 0.36139e-09_rb,0.46041e-06_rb,0.62474e-06_rb,0.65515e-06_rb,0.19800e-06_rb /)
      kbo(:, 1,48, 4) = (/ &
        & 0.34658e-09_rb,0.27494e-06_rb,0.37924e-06_rb,0.40349e-06_rb,0.11407e-06_rb /)
      kbo(:, 2,48, 4) = (/ &
        & 0.33340e-09_rb,0.30243e-06_rb,0.41340e-06_rb,0.44066e-06_rb,0.12508e-06_rb /)
      kbo(:, 3,48, 4) = (/ &
        & 0.32143e-09_rb,0.32756e-06_rb,0.44635e-06_rb,0.47308e-06_rb,0.13636e-06_rb /)
      kbo(:, 4,48, 4) = (/ &
        & 0.31032e-09_rb,0.35149e-06_rb,0.47669e-06_rb,0.50309e-06_rb,0.14763e-06_rb /)
      kbo(:, 5,48, 4) = (/ &
        & 0.29969e-09_rb,0.37323e-06_rb,0.50542e-06_rb,0.53062e-06_rb,0.15892e-06_rb /)
      kbo(:, 1,49, 4) = (/ &
        & 0.28846e-09_rb,0.21778e-06_rb,0.30168e-06_rb,0.32158e-06_rb,0.90607e-07_rb /)
      kbo(:, 2,49, 4) = (/ &
        & 0.27716e-09_rb,0.24139e-06_rb,0.33071e-06_rb,0.35214e-06_rb,0.99577e-07_rb /)
      kbo(:, 3,49, 4) = (/ &
        & 0.26700e-09_rb,0.26304e-06_rb,0.35804e-06_rb,0.38049e-06_rb,0.10881e-06_rb /)
      kbo(:, 4,49, 4) = (/ &
        & 0.25767e-09_rb,0.28313e-06_rb,0.38394e-06_rb,0.40566e-06_rb,0.11814e-06_rb /)
      kbo(:, 5,49, 4) = (/ &
        & 0.24875e-09_rb,0.30183e-06_rb,0.40832e-06_rb,0.42906e-06_rb,0.12742e-06_rb /)
      kbo(:, 1,50, 4) = (/ &
        & 0.23994e-09_rb,0.17276e-06_rb,0.24039e-06_rb,0.25644e-06_rb,0.72124e-07_rb /)
      kbo(:, 2,50, 4) = (/ &
        & 0.23028e-09_rb,0.19275e-06_rb,0.26429e-06_rb,0.28117e-06_rb,0.79388e-07_rb /)
      kbo(:, 3,50, 4) = (/ &
        & 0.22162e-09_rb,0.21106e-06_rb,0.28758e-06_rb,0.30606e-06_rb,0.86922e-07_rb /)
      kbo(:, 4,50, 4) = (/ &
        & 0.21377e-09_rb,0.22807e-06_rb,0.30947e-06_rb,0.32714e-06_rb,0.94628e-07_rb /)
      kbo(:, 5,50, 4) = (/ &
        & 0.20641e-09_rb,0.24400e-06_rb,0.32962e-06_rb,0.34715e-06_rb,0.10228e-06_rb /)
      kbo(:, 1,51, 4) = (/ &
        & 0.19960e-09_rb,0.13709e-06_rb,0.19101e-06_rb,0.20407e-06_rb,0.57566e-07_rb /)
      kbo(:, 2,51, 4) = (/ &
        & 0.19132e-09_rb,0.15352e-06_rb,0.21120e-06_rb,0.22455e-06_rb,0.63308e-07_rb /)
      kbo(:, 3,51, 4) = (/ &
        & 0.18395e-09_rb,0.16924e-06_rb,0.23068e-06_rb,0.24563e-06_rb,0.69421e-07_rb /)
      kbo(:, 4,51, 4) = (/ &
        & 0.17732e-09_rb,0.18352e-06_rb,0.24926e-06_rb,0.26383e-06_rb,0.75773e-07_rb /)
      kbo(:, 5,51, 4) = (/ &
        & 0.17117e-09_rb,0.19715e-06_rb,0.26630e-06_rb,0.28068e-06_rb,0.82097e-07_rb /)
      kbo(:, 1,52, 4) = (/ &
        & 0.16616e-09_rb,0.10852e-06_rb,0.15191e-06_rb,0.16137e-06_rb,0.45975e-07_rb /)
      kbo(:, 2,52, 4) = (/ &
        & 0.15901e-09_rb,0.12188e-06_rb,0.16855e-06_rb,0.17943e-06_rb,0.50454e-07_rb /)
      kbo(:, 3,52, 4) = (/ &
        & 0.15272e-09_rb,0.13535e-06_rb,0.18502e-06_rb,0.19705e-06_rb,0.55424e-07_rb /)
      kbo(:, 4,52, 4) = (/ &
        & 0.14711e-09_rb,0.14765e-06_rb,0.20042e-06_rb,0.21252e-06_rb,0.60616e-07_rb /)
      kbo(:, 5,52, 4) = (/ &
        & 0.14194e-09_rb,0.15907e-06_rb,0.21484e-06_rb,0.22673e-06_rb,0.65826e-07_rb /)
      kbo(:, 1,53, 4) = (/ &
        & 0.13839e-09_rb,0.85704e-07_rb,0.12023e-06_rb,0.12744e-06_rb,0.36523e-07_rb /)
      kbo(:, 2,53, 4) = (/ &
        & 0.13223e-09_rb,0.96600e-07_rb,0.13438e-06_rb,0.14312e-06_rb,0.40152e-07_rb /)
      kbo(:, 3,53, 4) = (/ &
        & 0.12684e-09_rb,0.10804e-06_rb,0.14784e-06_rb,0.15735e-06_rb,0.44207e-07_rb /)
      kbo(:, 4,53, 4) = (/ &
        & 0.12208e-09_rb,0.11851e-06_rb,0.16093e-06_rb,0.17108e-06_rb,0.48435e-07_rb /)
      kbo(:, 5,53, 4) = (/ &
        & 0.11772e-09_rb,0.12811e-06_rb,0.17313e-06_rb,0.18293e-06_rb,0.52733e-07_rb /)
      kbo(:, 1,54, 4) = (/ &
        & 0.11516e-09_rb,0.67802e-07_rb,0.95400e-07_rb,0.10065e-06_rb,0.29040e-07_rb /)
      kbo(:, 2,54, 4) = (/ &
        & 0.10986e-09_rb,0.76927e-07_rb,0.10712e-06_rb,0.11428e-06_rb,0.32044e-07_rb /)
      kbo(:, 3,54, 4) = (/ &
        & 0.10526e-09_rb,0.86295e-07_rb,0.11833e-06_rb,0.12580e-06_rb,0.35335e-07_rb /)
      kbo(:, 4,54, 4) = (/ &
        & 0.10122e-09_rb,0.95190e-07_rb,0.12936e-06_rb,0.13763e-06_rb,0.38764e-07_rb /)
      kbo(:, 5,54, 4) = (/ &
        & 0.97545e-10_rb,0.10324e-06_rb,0.13969e-06_rb,0.14768e-06_rb,0.42312e-07_rb /)
      kbo(:, 1,55, 4) = (/ &
        & 0.95832e-10_rb,0.53610e-07_rb,0.75981e-07_rb,0.79667e-07_rb,0.23117e-07_rb /)
      kbo(:, 2,55, 4) = (/ &
        & 0.91281e-10_rb,0.61130e-07_rb,0.85329e-07_rb,0.91005e-07_rb,0.25657e-07_rb /)
      kbo(:, 3,55, 4) = (/ &
        & 0.87335e-10_rb,0.68825e-07_rb,0.94736e-07_rb,0.10075e-06_rb,0.28264e-07_rb /)
      kbo(:, 4,55, 4) = (/ &
        & 0.83897e-10_rb,0.76435e-07_rb,0.10396e-06_rb,0.11070e-06_rb,0.31057e-07_rb /)
      kbo(:, 5,55, 4) = (/ &
        & 0.80810e-10_rb,0.83278e-07_rb,0.11269e-06_rb,0.11928e-06_rb,0.33972e-07_rb /)
      kbo(:, 1,56, 4) = (/ &
        & 0.79810e-10_rb,0.42322e-07_rb,0.60374e-07_rb,0.62926e-07_rb,0.18404e-07_rb /)
      kbo(:, 2,56, 4) = (/ &
        & 0.75864e-10_rb,0.48559e-07_rb,0.68039e-07_rb,0.72241e-07_rb,0.20548e-07_rb /)
      kbo(:, 3,56, 4) = (/ &
        & 0.72485e-10_rb,0.54773e-07_rb,0.75799e-07_rb,0.80757e-07_rb,0.22581e-07_rb /)
      kbo(:, 4,56, 4) = (/ &
        & 0.69560e-10_rb,0.61224e-07_rb,0.83554e-07_rb,0.88967e-07_rb,0.24859e-07_rb /)
      kbo(:, 5,56, 4) = (/ &
        & 0.66951e-10_rb,0.67084e-07_rb,0.90797e-07_rb,0.96246e-07_rb,0.27245e-07_rb /)
      kbo(:, 1,57, 4) = (/ &
        & 0.66524e-10_rb,0.33320e-07_rb,0.47516e-07_rb,0.49541e-07_rb,0.14620e-07_rb /)
      kbo(:, 2,57, 4) = (/ &
        & 0.63081e-10_rb,0.38443e-07_rb,0.54027e-07_rb,0.57142e-07_rb,0.16370e-07_rb /)
      kbo(:, 3,57, 4) = (/ &
        & 0.60187e-10_rb,0.43587e-07_rb,0.60611e-07_rb,0.64608e-07_rb,0.18036e-07_rb /)
      kbo(:, 4,57, 4) = (/ &
        & 0.57689e-10_rb,0.48970e-07_rb,0.66915e-07_rb,0.71233e-07_rb,0.19895e-07_rb /)
      kbo(:, 5,57, 4) = (/ &
        & 0.55478e-10_rb,0.53925e-07_rb,0.73045e-07_rb,0.77618e-07_rb,0.21831e-07_rb /)
      kbo(:, 1,58, 4) = (/ &
        & 0.55447e-10_rb,0.26240e-07_rb,0.37435e-07_rb,0.38989e-07_rb,0.11606e-07_rb /)
      kbo(:, 2,58, 4) = (/ &
        & 0.52442e-10_rb,0.30460e-07_rb,0.42955e-07_rb,0.45233e-07_rb,0.13047e-07_rb /)
      kbo(:, 3,58, 4) = (/ &
        & 0.49962e-10_rb,0.34761e-07_rb,0.48418e-07_rb,0.51680e-07_rb,0.14431e-07_rb /)
      kbo(:, 4,58, 4) = (/ &
        & 0.47826e-10_rb,0.39152e-07_rb,0.53626e-07_rb,0.57047e-07_rb,0.15934e-07_rb /)
      kbo(:, 5,58, 4) = (/ &
        & 0.45954e-10_rb,0.43380e-07_rb,0.58786e-07_rb,0.62538e-07_rb,0.17511e-07_rb /)
      kbo(:, 1,59, 4) = (/ &
        & 0.45740e-10_rb,0.21248e-07_rb,0.30261e-07_rb,0.31577e-07_rb,0.94221e-08_rb /)
      kbo(:, 2,59, 4) = (/ &
        & 0.43210e-10_rb,0.24730e-07_rb,0.34948e-07_rb,0.36722e-07_rb,0.10608e-07_rb /)
      kbo(:, 3,59, 4) = (/ &
        & 0.41141e-10_rb,0.28288e-07_rb,0.39410e-07_rb,0.42052e-07_rb,0.11754e-07_rb /)
      kbo(:, 4,59, 4) = (/ &
        & 0.39361e-10_rb,0.31904e-07_rb,0.43740e-07_rb,0.46527e-07_rb,0.12986e-07_rb /)
      kbo(:, 5,59, 4) = (/ &
        & 0.37805e-10_rb,0.35442e-07_rb,0.48020e-07_rb,0.51083e-07_rb,0.14278e-07_rb /)
      kbo(:, 1,13, 5) = (/ &
        & 0.79045e-05_rb,0.58512e-03_rb,0.80967e-03_rb,0.88578e-03_rb,0.34824e-03_rb /)
      kbo(:, 2,13, 5) = (/ &
        & 0.92807e-05_rb,0.61818e-03_rb,0.86083e-03_rb,0.92437e-03_rb,0.37870e-03_rb /)
      kbo(:, 3,13, 5) = (/ &
        & 0.10620e-04_rb,0.65025e-03_rb,0.90617e-03_rb,0.96290e-03_rb,0.40639e-03_rb /)
      kbo(:, 4,13, 5) = (/ &
        & 0.11847e-04_rb,0.68086e-03_rb,0.94767e-03_rb,0.99815e-03_rb,0.43149e-03_rb /)
      kbo(:, 5,13, 5) = (/ &
        & 0.12897e-04_rb,0.70941e-03_rb,0.98426e-03_rb,0.10296e-02_rb,0.45459e-03_rb /)
      kbo(:, 1,14, 5) = (/ &
        & 0.65683e-05_rb,0.48798e-03_rb,0.67570e-03_rb,0.73755e-03_rb,0.29172e-03_rb /)
      kbo(:, 2,14, 5) = (/ &
        & 0.76957e-05_rb,0.51555e-03_rb,0.71856e-03_rb,0.76978e-03_rb,0.31709e-03_rb /)
      kbo(:, 3,14, 5) = (/ &
        & 0.87852e-05_rb,0.54243e-03_rb,0.75604e-03_rb,0.80142e-03_rb,0.34064e-03_rb /)
      kbo(:, 4,14, 5) = (/ &
        & 0.97622e-05_rb,0.56817e-03_rb,0.79038e-03_rb,0.83091e-03_rb,0.36095e-03_rb /)
      kbo(:, 5,14, 5) = (/ &
        & 0.10614e-04_rb,0.59208e-03_rb,0.82018e-03_rb,0.85691e-03_rb,0.38095e-03_rb /)
      kbo(:, 1,15, 5) = (/ &
        & 0.54643e-05_rb,0.40660e-03_rb,0.56371e-03_rb,0.61286e-03_rb,0.24435e-03_rb /)
      kbo(:, 2,15, 5) = (/ &
        & 0.63883e-05_rb,0.42973e-03_rb,0.59914e-03_rb,0.64093e-03_rb,0.26475e-03_rb /)
      kbo(:, 3,15, 5) = (/ &
        & 0.72593e-05_rb,0.45218e-03_rb,0.63004e-03_rb,0.66659e-03_rb,0.28486e-03_rb /)
      kbo(:, 4,15, 5) = (/ &
        & 0.80526e-05_rb,0.47373e-03_rb,0.65847e-03_rb,0.69128e-03_rb,0.30200e-03_rb /)
      kbo(:, 5,15, 5) = (/ &
        & 0.87364e-05_rb,0.49393e-03_rb,0.68318e-03_rb,0.71321e-03_rb,0.31930e-03_rb /)
      kbo(:, 1,16, 5) = (/ &
        & 0.45482e-05_rb,0.33872e-03_rb,0.47003e-03_rb,0.50920e-03_rb,0.20473e-03_rb /)
      kbo(:, 2,16, 5) = (/ &
        & 0.52893e-05_rb,0.35785e-03_rb,0.49898e-03_rb,0.53306e-03_rb,0.22171e-03_rb /)
      kbo(:, 3,16, 5) = (/ &
        & 0.59989e-05_rb,0.37673e-03_rb,0.52467e-03_rb,0.55461e-03_rb,0.23803e-03_rb /)
      kbo(:, 4,16, 5) = (/ &
        & 0.66426e-05_rb,0.39470e-03_rb,0.54827e-03_rb,0.57515e-03_rb,0.25244e-03_rb /)
      kbo(:, 5,16, 5) = (/ &
        & 0.71759e-05_rb,0.41193e-03_rb,0.56909e-03_rb,0.59361e-03_rb,0.26773e-03_rb /)
      kbo(:, 1,17, 5) = (/ &
        & 0.37690e-05_rb,0.28177e-03_rb,0.39153e-03_rb,0.42265e-03_rb,0.17155e-03_rb /)
      kbo(:, 2,17, 5) = (/ &
        & 0.43790e-05_rb,0.29778e-03_rb,0.41535e-03_rb,0.44288e-03_rb,0.18563e-03_rb /)
      kbo(:, 3,17, 5) = (/ &
        & 0.49545e-05_rb,0.31371e-03_rb,0.43664e-03_rb,0.46090e-03_rb,0.19908e-03_rb /)
      kbo(:, 4,17, 5) = (/ &
        & 0.54563e-05_rb,0.32886e-03_rb,0.45637e-03_rb,0.47854e-03_rb,0.21097e-03_rb /)
      kbo(:, 5,17, 5) = (/ &
        & 0.58879e-05_rb,0.34359e-03_rb,0.47416e-03_rb,0.49387e-03_rb,0.22440e-03_rb /)
      kbo(:, 1,18, 5) = (/ &
        & 0.31260e-05_rb,0.23432e-03_rb,0.32596e-03_rb,0.35082e-03_rb,0.14348e-03_rb /)
      kbo(:, 2,18, 5) = (/ &
        & 0.36185e-05_rb,0.24786e-03_rb,0.34561e-03_rb,0.36765e-03_rb,0.15546e-03_rb /)
      kbo(:, 3,18, 5) = (/ &
        & 0.40732e-05_rb,0.26120e-03_rb,0.36343e-03_rb,0.38325e-03_rb,0.16623e-03_rb /)
      kbo(:, 4,18, 5) = (/ &
        & 0.44839e-05_rb,0.27403e-03_rb,0.37997e-03_rb,0.39811e-03_rb,0.17659e-03_rb /)
      kbo(:, 5,18, 5) = (/ &
        & 0.48304e-05_rb,0.28674e-03_rb,0.39513e-03_rb,0.41114e-03_rb,0.18813e-03_rb /)
      kbo(:, 1,19, 5) = (/ &
        & 0.25853e-05_rb,0.19483e-03_rb,0.27128e-03_rb,0.29126e-03_rb,0.11969e-03_rb /)
      kbo(:, 2,19, 5) = (/ &
        & 0.29834e-05_rb,0.20623e-03_rb,0.28749e-03_rb,0.30541e-03_rb,0.12978e-03_rb /)
      kbo(:, 3,19, 5) = (/ &
        & 0.33546e-05_rb,0.21750e-03_rb,0.30267e-03_rb,0.31871e-03_rb,0.13874e-03_rb /)
      kbo(:, 4,19, 5) = (/ &
        & 0.36863e-05_rb,0.22842e-03_rb,0.31656e-03_rb,0.33115e-03_rb,0.14789e-03_rb /)
      kbo(:, 5,19, 5) = (/ &
        & 0.39634e-05_rb,0.23942e-03_rb,0.32944e-03_rb,0.34220e-03_rb,0.15742e-03_rb /)
      kbo(:, 1,20, 5) = (/ &
        & 0.21486e-05_rb,0.16221e-03_rb,0.22609e-03_rb,0.24236e-03_rb,0.10014e-03_rb /)
      kbo(:, 2,20, 5) = (/ &
        & 0.24735e-05_rb,0.17178e-03_rb,0.23949e-03_rb,0.25399e-03_rb,0.10857e-03_rb /)
      kbo(:, 3,20, 5) = (/ &
        & 0.27744e-05_rb,0.18136e-03_rb,0.25226e-03_rb,0.26535e-03_rb,0.11604e-03_rb /)
      kbo(:, 4,20, 5) = (/ &
        & 0.30414e-05_rb,0.19076e-03_rb,0.26397e-03_rb,0.27566e-03_rb,0.12398e-03_rb /)
      kbo(:, 5,20, 5) = (/ &
        & 0.32610e-05_rb,0.20019e-03_rb,0.27488e-03_rb,0.28510e-03_rb,0.13199e-03_rb /)
      kbo(:, 1,21, 5) = (/ &
        & 0.17877e-05_rb,0.13507e-03_rb,0.18845e-03_rb,0.20163e-03_rb,0.83879e-04_rb /)
      kbo(:, 2,21, 5) = (/ &
        & 0.20518e-05_rb,0.14319e-03_rb,0.19968e-03_rb,0.21134e-03_rb,0.90858e-04_rb /)
      kbo(:, 3,21, 5) = (/ &
        & 0.22953e-05_rb,0.15137e-03_rb,0.21038e-03_rb,0.22094e-03_rb,0.97098e-04_rb /)
      kbo(:, 4,21, 5) = (/ &
        & 0.25098e-05_rb,0.15947e-03_rb,0.22019e-03_rb,0.22948e-03_rb,0.10402e-03_rb /)
      kbo(:, 5,21, 5) = (/ &
        & 0.26830e-05_rb,0.16749e-03_rb,0.22950e-03_rb,0.23761e-03_rb,0.11069e-03_rb /)
      kbo(:, 1,22, 5) = (/ &
        & 0.15022e-05_rb,0.11297e-03_rb,0.15773e-03_rb,0.16819e-03_rb,0.70808e-04_rb /)
      kbo(:, 2,22, 5) = (/ &
        & 0.17157e-05_rb,0.11998e-03_rb,0.16714e-03_rb,0.17644e-03_rb,0.76345e-04_rb /)
      kbo(:, 3,22, 5) = (/ &
        & 0.19111e-05_rb,0.12690e-03_rb,0.17611e-03_rb,0.18454e-03_rb,0.81804e-04_rb /)
      kbo(:, 4,22, 5) = (/ &
        & 0.20812e-05_rb,0.13385e-03_rb,0.18425e-03_rb,0.19164e-03_rb,0.87591e-04_rb /)
      kbo(:, 5,22, 5) = (/ &
        & 0.22144e-05_rb,0.14066e-03_rb,0.19235e-03_rb,0.19867e-03_rb,0.93228e-04_rb /)
      kbo(:, 1,23, 5) = (/ &
        & 0.12621e-05_rb,0.94681e-04_rb,0.13210e-03_rb,0.14032e-03_rb,0.59630e-04_rb /)
      kbo(:, 2,23, 5) = (/ &
        & 0.14346e-05_rb,0.10061e-03_rb,0.14009e-03_rb,0.14751e-03_rb,0.64142e-04_rb /)
      kbo(:, 3,23, 5) = (/ &
        & 0.15908e-05_rb,0.10655e-03_rb,0.14749e-03_rb,0.15413e-03_rb,0.68988e-04_rb /)
      kbo(:, 4,23, 5) = (/ &
        & 0.17253e-05_rb,0.11246e-03_rb,0.15437e-03_rb,0.16013e-03_rb,0.73893e-04_rb /)
      kbo(:, 5,23, 5) = (/ &
        & 0.18272e-05_rb,0.11825e-03_rb,0.16130e-03_rb,0.16627e-03_rb,0.78649e-04_rb /)
      kbo(:, 1,24, 5) = (/ &
        & 0.10611e-05_rb,0.79419e-04_rb,0.11079e-03_rb,0.11736e-03_rb,0.50244e-04_rb /)
      kbo(:, 2,24, 5) = (/ &
        & 0.11996e-05_rb,0.84486e-04_rb,0.11749e-03_rb,0.12345e-03_rb,0.54054e-04_rb /)
      kbo(:, 3,24, 5) = (/ &
        & 0.13241e-05_rb,0.89565e-04_rb,0.12364e-03_rb,0.12883e-03_rb,0.58259e-04_rb /)
      kbo(:, 4,24, 5) = (/ &
        & 0.14300e-05_rb,0.94592e-04_rb,0.12949e-03_rb,0.13406e-03_rb,0.62300e-04_rb /)
      kbo(:, 5,24, 5) = (/ &
        & 0.15070e-05_rb,0.99545e-04_rb,0.13538e-03_rb,0.13928e-03_rb,0.66485e-04_rb /)
      kbo(:, 1,25, 5) = (/ &
        & 0.89248e-06_rb,0.66757e-04_rb,0.93043e-04_rb,0.98269e-04_rb,0.42396e-04_rb /)
      kbo(:, 2,25, 5) = (/ &
        & 0.10034e-05_rb,0.71082e-04_rb,0.98652e-04_rb,0.10332e-03_rb,0.45719e-04_rb /)
      kbo(:, 3,25, 5) = (/ &
        & 0.11024e-05_rb,0.75404e-04_rb,0.10375e-03_rb,0.10782e-03_rb,0.49228e-04_rb /)
      kbo(:, 4,25, 5) = (/ &
        & 0.11835e-05_rb,0.79694e-04_rb,0.10882e-03_rb,0.11234e-03_rb,0.52649e-04_rb /)
      kbo(:, 5,25, 5) = (/ &
        & 0.12406e-05_rb,0.83915e-04_rb,0.11376e-03_rb,0.11677e-03_rb,0.56264e-04_rb /)
      kbo(:, 1,26, 5) = (/ &
        & 0.75147e-06_rb,0.56248e-04_rb,0.78291e-04_rb,0.82441e-04_rb,0.35781e-04_rb /)
      kbo(:, 2,26, 5) = (/ &
        & 0.84025e-06_rb,0.59929e-04_rb,0.82917e-04_rb,0.86562e-04_rb,0.38715e-04_rb /)
      kbo(:, 3,26, 5) = (/ &
        & 0.91728e-06_rb,0.63618e-04_rb,0.87231e-04_rb,0.90442e-04_rb,0.41682e-04_rb /)
      kbo(:, 4,26, 5) = (/ &
        & 0.97947e-06_rb,0.67285e-04_rb,0.91606e-04_rb,0.94311e-04_rb,0.44610e-04_rb /)
      kbo(:, 5,26, 5) = (/ &
        & 0.10222e-05_rb,0.70879e-04_rb,0.95759e-04_rb,0.97954e-04_rb,0.47678e-04_rb /)
      kbo(:, 1,27, 5) = (/ &
        & 0.63228e-06_rb,0.47448e-04_rb,0.65926e-04_rb,0.69211e-04_rb,0.30312e-04_rb /)
      kbo(:, 2,27, 5) = (/ &
        & 0.70187e-06_rb,0.50583e-04_rb,0.69752e-04_rb,0.72622e-04_rb,0.32844e-04_rb /)
      kbo(:, 3,27, 5) = (/ &
        & 0.76263e-06_rb,0.53723e-04_rb,0.73431e-04_rb,0.75943e-04_rb,0.35283e-04_rb /)
      kbo(:, 4,27, 5) = (/ &
        & 0.80975e-06_rb,0.56884e-04_rb,0.77139e-04_rb,0.79235e-04_rb,0.37837e-04_rb /)
      kbo(:, 5,27, 5) = (/ &
        & 0.84099e-06_rb,0.59937e-04_rb,0.80670e-04_rb,0.82198e-04_rb,0.40398e-04_rb /)
      kbo(:, 1,28, 5) = (/ &
        & 0.53062e-06_rb,0.40054e-04_rb,0.55536e-04_rb,0.58105e-04_rb,0.25683e-04_rb /)
      kbo(:, 2,28, 5) = (/ &
        & 0.58581e-06_rb,0.42748e-04_rb,0.58719e-04_rb,0.60988e-04_rb,0.27829e-04_rb /)
      kbo(:, 3,28, 5) = (/ &
        & 0.63283e-06_rb,0.45445e-04_rb,0.61927e-04_rb,0.63808e-04_rb,0.29914e-04_rb /)
      kbo(:, 4,28, 5) = (/ &
        & 0.66841e-06_rb,0.48141e-04_rb,0.65033e-04_rb,0.66567e-04_rb,0.32101e-04_rb /)
      kbo(:, 5,28, 5) = (/ &
        & 0.69132e-06_rb,0.50744e-04_rb,0.68002e-04_rb,0.69024e-04_rb,0.34196e-04_rb /)
      kbo(:, 1,29, 5) = (/ &
        & 0.44482e-06_rb,0.33876e-04_rb,0.46813e-04_rb,0.48820e-04_rb,0.21822e-04_rb /)
      kbo(:, 2,29, 5) = (/ &
        & 0.48830e-06_rb,0.36173e-04_rb,0.49534e-04_rb,0.51267e-04_rb,0.23598e-04_rb /)
      kbo(:, 3,29, 5) = (/ &
        & 0.52520e-06_rb,0.38507e-04_rb,0.52264e-04_rb,0.53700e-04_rb,0.25407e-04_rb /)
      kbo(:, 4,29, 5) = (/ &
        & 0.55142e-06_rb,0.40790e-04_rb,0.54879e-04_rb,0.55934e-04_rb,0.27233e-04_rb /)
      kbo(:, 5,29, 5) = (/ &
        & 0.56821e-06_rb,0.43005e-04_rb,0.57374e-04_rb,0.57995e-04_rb,0.28939e-04_rb /)
      kbo(:, 1,30, 5) = (/ &
        & 0.37240e-06_rb,0.28688e-04_rb,0.39498e-04_rb,0.41082e-04_rb,0.18542e-04_rb /)
      kbo(:, 2,30, 5) = (/ &
        & 0.40697e-06_rb,0.30662e-04_rb,0.41832e-04_rb,0.43139e-04_rb,0.20037e-04_rb /)
      kbo(:, 3,30, 5) = (/ &
        & 0.43518e-06_rb,0.32657e-04_rb,0.44137e-04_rb,0.45198e-04_rb,0.21592e-04_rb /)
      kbo(:, 4,30, 5) = (/ &
        & 0.45459e-06_rb,0.34600e-04_rb,0.46342e-04_rb,0.47030e-04_rb,0.23099e-04_rb /)
      kbo(:, 5,30, 5) = (/ &
        & 0.46674e-06_rb,0.36468e-04_rb,0.48427e-04_rb,0.48752e-04_rb,0.24524e-04_rb /)
      kbo(:, 1,31, 5) = (/ &
        & 0.31167e-06_rb,0.24328e-04_rb,0.33365e-04_rb,0.34577e-04_rb,0.15775e-04_rb /)
      kbo(:, 2,31, 5) = (/ &
        & 0.33880e-06_rb,0.26033e-04_rb,0.35376e-04_rb,0.36343e-04_rb,0.17037e-04_rb /)
      kbo(:, 3,31, 5) = (/ &
        & 0.36022e-06_rb,0.27728e-04_rb,0.37306e-04_rb,0.38025e-04_rb,0.18334e-04_rb /)
      kbo(:, 4,31, 5) = (/ &
        & 0.37443e-06_rb,0.29381e-04_rb,0.39153e-04_rb,0.39551e-04_rb,0.19559e-04_rb /)
      kbo(:, 5,31, 5) = (/ &
        & 0.38304e-06_rb,0.30940e-04_rb,0.40888e-04_rb,0.41012e-04_rb,0.20735e-04_rb /)
      kbo(:, 1,32, 5) = (/ &
        & 0.26065e-06_rb,0.20666e-04_rb,0.28226e-04_rb,0.29128e-04_rb,0.13403e-04_rb /)
      kbo(:, 2,32, 5) = (/ &
        & 0.28179e-06_rb,0.22130e-04_rb,0.29934e-04_rb,0.30642e-04_rb,0.14495e-04_rb /)
      kbo(:, 3,32, 5) = (/ &
        & 0.29791e-06_rb,0.23572e-04_rb,0.31554e-04_rb,0.32005e-04_rb,0.15576e-04_rb /)
      kbo(:, 4,32, 5) = (/ &
        & 0.30813e-06_rb,0.24963e-04_rb,0.33101e-04_rb,0.33278e-04_rb,0.16582e-04_rb /)
      kbo(:, 5,32, 5) = (/ &
        & 0.31412e-06_rb,0.26261e-04_rb,0.34510e-04_rb,0.34497e-04_rb,0.17533e-04_rb /)
      kbo(:, 1,33, 5) = (/ &
        & 0.21768e-06_rb,0.17581e-04_rb,0.23903e-04_rb,0.24558e-04_rb,0.11405e-04_rb /)
      kbo(:, 2,33, 5) = (/ &
        & 0.23409e-06_rb,0.18827e-04_rb,0.25333e-04_rb,0.25812e-04_rb,0.12331e-04_rb /)
      kbo(:, 3,33, 5) = (/ &
        & 0.24596e-06_rb,0.20053e-04_rb,0.26696e-04_rb,0.26942e-04_rb,0.13200e-04_rb /)
      kbo(:, 4,33, 5) = (/ &
        & 0.25338e-06_rb,0.21211e-04_rb,0.27978e-04_rb,0.28012e-04_rb,0.14037e-04_rb /)
      kbo(:, 5,33, 5) = (/ &
        & 0.25735e-06_rb,0.22292e-04_rb,0.29113e-04_rb,0.29019e-04_rb,0.14789e-04_rb /)
      kbo(:, 1,34, 5) = (/ &
        & 0.18111e-06_rb,0.14933e-04_rb,0.20221e-04_rb,0.20691e-04_rb,0.96853e-05_rb /)
      kbo(:, 2,34, 5) = (/ &
        & 0.19373e-06_rb,0.15993e-04_rb,0.21413e-04_rb,0.21708e-04_rb,0.10459e-04_rb /)
      kbo(:, 3,34, 5) = (/ &
        & 0.20262e-06_rb,0.17025e-04_rb,0.22551e-04_rb,0.22651e-04_rb,0.11170e-04_rb /)
      kbo(:, 4,34, 5) = (/ &
        & 0.20802e-06_rb,0.17992e-04_rb,0.23603e-04_rb,0.23540e-04_rb,0.11847e-04_rb /)
      kbo(:, 5,34, 5) = (/ &
        & 0.21068e-06_rb,0.18892e-04_rb,0.24522e-04_rb,0.24370e-04_rb,0.12444e-04_rb /)
      kbo(:, 1,35, 5) = (/ &
        & 0.14940e-06_rb,0.12604e-04_rb,0.17001e-04_rb,0.17339e-04_rb,0.81630e-05_rb /)
      kbo(:, 2,35, 5) = (/ &
        & 0.15934e-06_rb,0.13505e-04_rb,0.18008e-04_rb,0.18183e-04_rb,0.87890e-05_rb /)
      kbo(:, 3,35, 5) = (/ &
        & 0.16626e-06_rb,0.14374e-04_rb,0.18963e-04_rb,0.18976e-04_rb,0.93838e-05_rb /)
      kbo(:, 4,35, 5) = (/ &
        & 0.17040e-06_rb,0.15187e-04_rb,0.19824e-04_rb,0.19720e-04_rb,0.99229e-05_rb /)
      kbo(:, 5,35, 5) = (/ &
        & 0.17229e-06_rb,0.15939e-04_rb,0.20582e-04_rb,0.20394e-04_rb,0.10412e-04_rb /)
      kbo(:, 1,36, 5) = (/ &
        & 0.12224e-06_rb,0.10560e-04_rb,0.14203e-04_rb,0.14447e-04_rb,0.68196e-05_rb /)
      kbo(:, 2,36, 5) = (/ &
        & 0.13030e-06_rb,0.11326e-04_rb,0.15055e-04_rb,0.15156e-04_rb,0.73321e-05_rb /)
      kbo(:, 3,36, 5) = (/ &
        & 0.13590e-06_rb,0.12062e-04_rb,0.15856e-04_rb,0.15823e-04_rb,0.78230e-05_rb /)
      kbo(:, 4,36, 5) = (/ &
        & 0.13929e-06_rb,0.12750e-04_rb,0.16573e-04_rb,0.16449e-04_rb,0.82632e-05_rb /)
      kbo(:, 5,36, 5) = (/ &
        & 0.14081e-06_rb,0.13384e-04_rb,0.17208e-04_rb,0.17006e-04_rb,0.86656e-05_rb /)
      kbo(:, 1,37, 5) = (/ &
        & 0.98979e-07_rb,0.87500e-05_rb,0.11755e-04_rb,0.11943e-04_rb,0.56269e-05_rb /)
      kbo(:, 2,37, 5) = (/ &
        & 0.10583e-06_rb,0.94035e-05_rb,0.12480e-04_rb,0.12544e-04_rb,0.60528e-05_rb /)
      kbo(:, 3,37, 5) = (/ &
        & 0.11068e-06_rb,0.10031e-04_rb,0.13161e-04_rb,0.13110e-04_rb,0.64604e-05_rb /)
      kbo(:, 4,37, 5) = (/ &
        & 0.11365e-06_rb,0.10619e-04_rb,0.13767e-04_rb,0.13642e-04_rb,0.68253e-05_rb /)
      kbo(:, 5,37, 5) = (/ &
        & 0.11510e-06_rb,0.11162e-04_rb,0.14306e-04_rb,0.14108e-04_rb,0.71615e-05_rb /)
      kbo(:, 1,38, 5) = (/ &
        & 0.80050e-07_rb,0.72436e-05_rb,0.97225e-05_rb,0.98657e-05_rb,0.46375e-05_rb /)
      kbo(:, 2,38, 5) = (/ &
        & 0.85886e-07_rb,0.78006e-05_rb,0.10338e-04_rb,0.10378e-04_rb,0.49919e-05_rb /)
      kbo(:, 3,38, 5) = (/ &
        & 0.90108e-07_rb,0.83355e-05_rb,0.10916e-04_rb,0.10856e-04_rb,0.53303e-05_rb /)
      kbo(:, 4,38, 5) = (/ &
        & 0.92690e-07_rb,0.88380e-05_rb,0.11429e-04_rb,0.11306e-04_rb,0.56349e-05_rb /)
      kbo(:, 5,38, 5) = (/ &
        & 0.94046e-07_rb,0.93019e-05_rb,0.11885e-04_rb,0.11696e-04_rb,0.59152e-05_rb /)
      kbo(:, 1,39, 5) = (/ &
        & 0.64720e-07_rb,0.59955e-05_rb,0.80389e-05_rb,0.81488e-05_rb,0.38192e-05_rb /)
      kbo(:, 2,39, 5) = (/ &
        & 0.69663e-07_rb,0.64699e-05_rb,0.85613e-05_rb,0.85818e-05_rb,0.41159e-05_rb /)
      kbo(:, 3,39, 5) = (/ &
        & 0.73305e-07_rb,0.69246e-05_rb,0.90506e-05_rb,0.89861e-05_rb,0.43968e-05_rb /)
      kbo(:, 4,39, 5) = (/ &
        & 0.75588e-07_rb,0.73530e-05_rb,0.94857e-05_rb,0.93637e-05_rb,0.46505e-05_rb /)
      kbo(:, 5,39, 5) = (/ &
        & 0.76825e-07_rb,0.77492e-05_rb,0.98708e-05_rb,0.96898e-05_rb,0.48841e-05_rb /)
      kbo(:, 1,40, 5) = (/ &
        & 0.51871e-07_rb,0.49191e-05_rb,0.65994e-05_rb,0.66890e-05_rb,0.31186e-05_rb /)
      kbo(:, 2,40, 5) = (/ &
        & 0.56127e-07_rb,0.53227e-05_rb,0.70435e-05_rb,0.70578e-05_rb,0.33678e-05_rb /)
      kbo(:, 3,40, 5) = (/ &
        & 0.59395e-07_rb,0.57124e-05_rb,0.74609e-05_rb,0.74018e-05_rb,0.36025e-05_rb /)
      kbo(:, 4,40, 5) = (/ &
        & 0.61501e-07_rb,0.60790e-05_rb,0.78333e-05_rb,0.77210e-05_rb,0.38164e-05_rb /)
      kbo(:, 5,40, 5) = (/ &
        & 0.62710e-07_rb,0.64194e-05_rb,0.81613e-05_rb,0.79980e-05_rb,0.40122e-05_rb /)
      kbo(:, 1,41, 5) = (/ &
        & 0.41471e-07_rb,0.40276e-05_rb,0.54076e-05_rb,0.54815e-05_rb,0.25402e-05_rb /)
      kbo(:, 2,41, 5) = (/ &
        & 0.45127e-07_rb,0.43711e-05_rb,0.57847e-05_rb,0.57971e-05_rb,0.27507e-05_rb /)
      kbo(:, 3,41, 5) = (/ &
        & 0.48018e-07_rb,0.47040e-05_rb,0.61407e-05_rb,0.60883e-05_rb,0.29479e-05_rb /)
      kbo(:, 4,41, 5) = (/ &
        & 0.49983e-07_rb,0.50177e-05_rb,0.64595e-05_rb,0.63587e-05_rb,0.31286e-05_rb /)
      kbo(:, 5,41, 5) = (/ &
        & 0.51153e-07_rb,0.53098e-05_rb,0.67389e-05_rb,0.65950e-05_rb,0.32928e-05_rb /)
      kbo(:, 1,42, 5) = (/ &
        & 0.33108e-07_rb,0.32945e-05_rb,0.44261e-05_rb,0.44871e-05_rb,0.20674e-05_rb /)
      kbo(:, 2,42, 5) = (/ &
        & 0.36228e-07_rb,0.35861e-05_rb,0.47462e-05_rb,0.47571e-05_rb,0.22452e-05_rb /)
      kbo(:, 3,42, 5) = (/ &
        & 0.38776e-07_rb,0.38701e-05_rb,0.50505e-05_rb,0.50046e-05_rb,0.24106e-05_rb /)
      kbo(:, 4,42, 5) = (/ &
        & 0.40580e-07_rb,0.41378e-05_rb,0.53219e-05_rb,0.52323e-05_rb,0.25622e-05_rb /)
      kbo(:, 5,42, 5) = (/ &
        & 0.41689e-07_rb,0.43881e-05_rb,0.55607e-05_rb,0.54356e-05_rb,0.27015e-05_rb /)
      kbo(:, 1,43, 5) = (/ &
        & 0.26243e-07_rb,0.26780e-05_rb,0.36021e-05_rb,0.36578e-05_rb,0.16722e-05_rb /)
      kbo(:, 2,43, 5) = (/ &
        & 0.28903e-07_rb,0.29254e-05_rb,0.38754e-05_rb,0.38886e-05_rb,0.18235e-05_rb /)
      kbo(:, 3,43, 5) = (/ &
        & 0.31158e-07_rb,0.31674e-05_rb,0.41363e-05_rb,0.40998e-05_rb,0.19628e-05_rb /)
      kbo(:, 4,43, 5) = (/ &
        & 0.32828e-07_rb,0.33966e-05_rb,0.43687e-05_rb,0.42935e-05_rb,0.20906e-05_rb /)
      kbo(:, 5,43, 5) = (/ &
        & 0.33896e-07_rb,0.36112e-05_rb,0.45745e-05_rb,0.44680e-05_rb,0.22088e-05_rb /)
      kbo(:, 1,44, 5) = (/ &
        & 0.20692e-07_rb,0.21688e-05_rb,0.29226e-05_rb,0.29738e-05_rb,0.13486e-05_rb /)
      kbo(:, 2,44, 5) = (/ &
        & 0.22961e-07_rb,0.23776e-05_rb,0.31556e-05_rb,0.31699e-05_rb,0.14759e-05_rb /)
      kbo(:, 3,44, 5) = (/ &
        & 0.24934e-07_rb,0.25834e-05_rb,0.33779e-05_rb,0.33507e-05_rb,0.15926e-05_rb /)
      kbo(:, 4,44, 5) = (/ &
        & 0.26472e-07_rb,0.27800e-05_rb,0.35779e-05_rb,0.35162e-05_rb,0.17017e-05_rb /)
      kbo(:, 5,44, 5) = (/ &
        & 0.27504e-07_rb,0.29637e-05_rb,0.37558e-05_rb,0.36666e-05_rb,0.18017e-05_rb /)
      kbo(:, 1,45, 5) = (/ &
        & 0.16269e-07_rb,0.17537e-05_rb,0.23672e-05_rb,0.24133e-05_rb,0.10856e-05_rb /)
      kbo(:, 2,45, 5) = (/ &
        & 0.18194e-07_rb,0.19291e-05_rb,0.25663e-05_rb,0.25809e-05_rb,0.11927e-05_rb /)
      kbo(:, 3,45, 5) = (/ &
        & 0.19903e-07_rb,0.21039e-05_rb,0.27552e-05_rb,0.27360e-05_rb,0.12909e-05_rb /)
      kbo(:, 4,45, 5) = (/ &
        & 0.21293e-07_rb,0.22721e-05_rb,0.29270e-05_rb,0.28775e-05_rb,0.13839e-05_rb /)
      kbo(:, 5,45, 5) = (/ &
        & 0.22283e-07_rb,0.24292e-05_rb,0.30802e-05_rb,0.30067e-05_rb,0.14686e-05_rb /)
      kbo(:, 1,46, 5) = (/ &
        & 0.12712e-07_rb,0.14128e-05_rb,0.19113e-05_rb,0.19548e-05_rb,0.87114e-06_rb /)
      kbo(:, 2,46, 5) = (/ &
        & 0.14351e-07_rb,0.15596e-05_rb,0.20802e-05_rb,0.20949e-05_rb,0.96118e-06_rb /)
      kbo(:, 3,46, 5) = (/ &
        & 0.15815e-07_rb,0.17076e-05_rb,0.22404e-05_rb,0.22289e-05_rb,0.10436e-05_rb /)
      kbo(:, 4,46, 5) = (/ &
        & 0.17062e-07_rb,0.18511e-05_rb,0.23889e-05_rb,0.23492e-05_rb,0.11225e-05_rb /)
      kbo(:, 5,46, 5) = (/ &
        & 0.17993e-07_rb,0.19855e-05_rb,0.25206e-05_rb,0.24609e-05_rb,0.11948e-05_rb /)
      kbo(:, 1,47, 5) = (/ &
        & 0.98364e-08_rb,0.11306e-05_rb,0.15353e-05_rb,0.15746e-05_rb,0.69554e-06_rb /)
      kbo(:, 2,47, 5) = (/ &
        & 0.11226e-07_rb,0.12534e-05_rb,0.16765e-05_rb,0.16944e-05_rb,0.76925e-06_rb /)
      kbo(:, 3,47, 5) = (/ &
        & 0.12483e-07_rb,0.13784e-05_rb,0.18130e-05_rb,0.18087e-05_rb,0.84036e-06_rb /)
      kbo(:, 4,47, 5) = (/ &
        & 0.13587e-07_rb,0.15007e-05_rb,0.19418e-05_rb,0.19122e-05_rb,0.90712e-06_rb /)
      kbo(:, 5,47, 5) = (/ &
        & 0.14457e-07_rb,0.16158e-05_rb,0.20557e-05_rb,0.20073e-05_rb,0.96847e-06_rb /)
      kbo(:, 1,48, 5) = (/ &
        & 0.75824e-08_rb,0.90329e-06_rb,0.12307e-05_rb,0.12653e-05_rb,0.55346e-06_rb /)
      kbo(:, 2,48, 5) = (/ &
        & 0.87453e-08_rb,0.10050e-05_rb,0.13490e-05_rb,0.13676e-05_rb,0.61470e-06_rb /)
      kbo(:, 3,48, 5) = (/ &
        & 0.98194e-08_rb,0.11101e-05_rb,0.14650e-05_rb,0.14651e-05_rb,0.67492e-06_rb /)
      kbo(:, 4,48, 5) = (/ &
        & 0.10779e-07_rb,0.12141e-05_rb,0.15756e-05_rb,0.15544e-05_rb,0.73163e-06_rb /)
      kbo(:, 5,48, 5) = (/ &
        & 0.11579e-07_rb,0.13125e-05_rb,0.16743e-05_rb,0.16359e-05_rb,0.78405e-06_rb /)
      kbo(:, 1,49, 5) = (/ &
        & 0.58219e-08_rb,0.72015e-06_rb,0.98379e-06_rb,0.10143e-05_rb,0.43933e-06_rb /)
      kbo(:, 2,49, 5) = (/ &
        & 0.67817e-08_rb,0.80423e-06_rb,0.10832e-05_rb,0.11023e-05_rb,0.49018e-06_rb /)
      kbo(:, 3,49, 5) = (/ &
        & 0.76966e-08_rb,0.89205e-06_rb,0.11819e-05_rb,0.11846e-05_rb,0.54131e-06_rb /)
      kbo(:, 4,49, 5) = (/ &
        & 0.85224e-08_rb,0.98005e-06_rb,0.12760e-05_rb,0.12617e-05_rb,0.58858e-06_rb /)
      kbo(:, 5,49, 5) = (/ &
        & 0.92350e-08_rb,0.10644e-05_rb,0.13615e-05_rb,0.13319e-05_rb,0.63358e-06_rb /)
      kbo(:, 1,50, 5) = (/ &
        & 0.44790e-08_rb,0.57418e-06_rb,0.78635e-06_rb,0.81296e-06_rb,0.34881e-06_rb /)
      kbo(:, 2,50, 5) = (/ &
        & 0.52579e-08_rb,0.64388e-06_rb,0.87056e-06_rb,0.88896e-06_rb,0.39135e-06_rb /)
      kbo(:, 3,50, 5) = (/ &
        & 0.60279e-08_rb,0.71722e-06_rb,0.95348e-06_rb,0.95809e-06_rb,0.43444e-06_rb /)
      kbo(:, 4,50, 5) = (/ &
        & 0.67324e-08_rb,0.79134e-06_rb,0.10331e-05_rb,0.10243e-05_rb,0.47391e-06_rb /)
      kbo(:, 5,50, 5) = (/ &
        & 0.73551e-08_rb,0.86330e-06_rb,0.11073e-05_rb,0.10842e-05_rb,0.51234e-06_rb /)
      kbo(:, 1,51, 5) = (/ &
        & 0.34450e-08_rb,0.45753e-06_rb,0.62814e-06_rb,0.65158e-06_rb,0.27625e-06_rb /)
      kbo(:, 2,51, 5) = (/ &
        & 0.40690e-08_rb,0.51530e-06_rb,0.69892e-06_rb,0.71555e-06_rb,0.31260e-06_rb /)
      kbo(:, 3,51, 5) = (/ &
        & 0.47087e-08_rb,0.57624e-06_rb,0.76840e-06_rb,0.77477e-06_rb,0.34804e-06_rb /)
      kbo(:, 4,51, 5) = (/ &
        & 0.53086e-08_rb,0.63843e-06_rb,0.83580e-06_rb,0.83092e-06_rb,0.38181e-06_rb /)
      kbo(:, 5,51, 5) = (/ &
        & 0.58439e-08_rb,0.69956e-06_rb,0.89962e-06_rb,0.88225e-06_rb,0.41412e-06_rb /)
      kbo(:, 1,52, 5) = (/ &
        & 0.26454e-08_rb,0.36387e-06_rb,0.50007e-06_rb,0.52179e-06_rb,0.21894e-06_rb /)
      kbo(:, 2,52, 5) = (/ &
        & 0.31403e-08_rb,0.41173e-06_rb,0.55997e-06_rb,0.57479e-06_rb,0.24870e-06_rb /)
      kbo(:, 3,52, 5) = (/ &
        & 0.36655e-08_rb,0.46211e-06_rb,0.61819e-06_rb,0.62511e-06_rb,0.27840e-06_rb /)
      kbo(:, 4,52, 5) = (/ &
        & 0.41730e-08_rb,0.51415e-06_rb,0.67546e-06_rb,0.67307e-06_rb,0.30695e-06_rb /)
      kbo(:, 5,52, 5) = (/ &
        & 0.46311e-08_rb,0.56582e-06_rb,0.72973e-06_rb,0.71697e-06_rb,0.33407e-06_rb /)
      kbo(:, 1,53, 5) = (/ &
        & 0.20275e-08_rb,0.28873e-06_rb,0.39749e-06_rb,0.41647e-06_rb,0.17355e-06_rb /)
      kbo(:, 2,53, 5) = (/ &
        & 0.24190e-08_rb,0.32821e-06_rb,0.44730e-06_rb,0.46063e-06_rb,0.19774e-06_rb /)
      kbo(:, 3,53, 5) = (/ &
        & 0.28440e-08_rb,0.36981e-06_rb,0.49677e-06_rb,0.50411e-06_rb,0.22218e-06_rb /)
      kbo(:, 4,53, 5) = (/ &
        & 0.32690e-08_rb,0.41315e-06_rb,0.54488e-06_rb,0.54408e-06_rb,0.24647e-06_rb /)
      kbo(:, 5,53, 5) = (/ &
        & 0.36592e-08_rb,0.45680e-06_rb,0.59090e-06_rb,0.58189e-06_rb,0.26902e-06_rb /)
      kbo(:, 1,54, 5) = (/ &
        & 0.15611e-08_rb,0.22962e-06_rb,0.31613e-06_rb,0.33300e-06_rb,0.13730e-06_rb /)
      kbo(:, 2,54, 5) = (/ &
        & 0.18696e-08_rb,0.26187e-06_rb,0.35802e-06_rb,0.36954e-06_rb,0.15701e-06_rb /)
      kbo(:, 3,54, 5) = (/ &
        & 0.22100e-08_rb,0.29641e-06_rb,0.39970e-06_rb,0.40683e-06_rb,0.17764e-06_rb /)
      kbo(:, 4,54, 5) = (/ &
        & 0.25619e-08_rb,0.33255e-06_rb,0.44009e-06_rb,0.44063e-06_rb,0.19808e-06_rb /)
      kbo(:, 5,54, 5) = (/ &
        & 0.28928e-08_rb,0.36908e-06_rb,0.47892e-06_rb,0.47270e-06_rb,0.21696e-06_rb /)
      kbo(:, 1,55, 5) = (/ &
        & 0.12041e-08_rb,0.18259e-06_rb,0.25108e-06_rb,0.26608e-06_rb,0.10842e-06_rb /)
      kbo(:, 2,55, 5) = (/ &
        & 0.14466e-08_rb,0.20918e-06_rb,0.28633e-06_rb,0.29677e-06_rb,0.12468e-06_rb /)
      kbo(:, 3,55, 5) = (/ &
        & 0.17176e-08_rb,0.23765e-06_rb,0.32139e-06_rb,0.32799e-06_rb,0.14213e-06_rb /)
      kbo(:, 4,55, 5) = (/ &
        & 0.20058e-08_rb,0.26766e-06_rb,0.35523e-06_rb,0.35664e-06_rb,0.15907e-06_rb /)
      kbo(:, 5,55, 5) = (/ &
        & 0.22852e-08_rb,0.29818e-06_rb,0.38806e-06_rb,0.38381e-06_rb,0.17512e-06_rb /)
      kbo(:, 1,56, 5) = (/ &
        & 0.92934e-09_rb,0.14496e-06_rb,0.19887e-06_rb,0.21215e-06_rb,0.85185e-07_rb /)
      kbo(:, 2,56, 5) = (/ &
        & 0.11180e-08_rb,0.16668e-06_rb,0.22841e-06_rb,0.23799e-06_rb,0.99172e-07_rb /)
      kbo(:, 3,56, 5) = (/ &
        & 0.13333e-08_rb,0.19024e-06_rb,0.25790e-06_rb,0.26385e-06_rb,0.11331e-06_rb /)
      kbo(:, 4,56, 5) = (/ &
        & 0.15668e-08_rb,0.21512e-06_rb,0.28626e-06_rb,0.28820e-06_rb,0.12758e-06_rb /)
      kbo(:, 5,56, 5) = (/ &
        & 0.18000e-08_rb,0.24055e-06_rb,0.31405e-06_rb,0.31134e-06_rb,0.14106e-06_rb /)
      kbo(:, 1,57, 5) = (/ &
        & 0.71747e-09_rb,0.11491e-06_rb,0.15760e-06_rb,0.16907e-06_rb,0.66762e-07_rb /)
      kbo(:, 2,57, 5) = (/ &
        & 0.86318e-09_rb,0.13266e-06_rb,0.18198e-06_rb,0.19051e-06_rb,0.78844e-07_rb /)
      kbo(:, 3,57, 5) = (/ &
        & 0.10332e-08_rb,0.15199e-06_rb,0.20646e-06_rb,0.21185e-06_rb,0.90317e-07_rb /)
      kbo(:, 4,57, 5) = (/ &
        & 0.12205e-08_rb,0.17255e-06_rb,0.23049e-06_rb,0.23280e-06_rb,0.10211e-06_rb /)
      kbo(:, 5,57, 5) = (/ &
        & 0.14137e-08_rb,0.19374e-06_rb,0.25377e-06_rb,0.25209e-06_rb,0.11347e-06_rb /)
      kbo(:, 1,58, 5) = (/ &
        & 0.55534e-09_rb,0.91161e-07_rb,0.12495e-06_rb,0.13481e-06_rb,0.52524e-07_rb /)
      kbo(:, 2,58, 5) = (/ &
        & 0.66796e-09_rb,0.10564e-06_rb,0.14490e-06_rb,0.15246e-06_rb,0.62491e-07_rb /)
      kbo(:, 3,58, 5) = (/ &
        & 0.80208e-09_rb,0.12142e-06_rb,0.16540e-06_rb,0.17013e-06_rb,0.71923e-07_rb /)
      kbo(:, 4,58, 5) = (/ &
        & 0.95155e-09_rb,0.13846e-06_rb,0.18562e-06_rb,0.18803e-06_rb,0.81760e-07_rb /)
      kbo(:, 5,58, 5) = (/ &
        & 0.11100e-08_rb,0.15608e-06_rb,0.20513e-06_rb,0.20425e-06_rb,0.91405e-07_rb /)
      kbo(:, 1,59, 5) = (/ &
        & 0.44483e-09_rb,0.74201e-07_rb,0.10175e-06_rb,0.10986e-06_rb,0.42626e-07_rb /)
      kbo(:, 2,59, 5) = (/ &
        & 0.53478e-09_rb,0.86257e-07_rb,0.11818e-06_rb,0.12453e-06_rb,0.50816e-07_rb /)
      kbo(:, 3,59, 5) = (/ &
        & 0.64265e-09_rb,0.99390e-07_rb,0.13535e-06_rb,0.13928e-06_rb,0.58658e-07_rb /)
      kbo(:, 4,59, 5) = (/ &
        & 0.76369e-09_rb,0.11360e-06_rb,0.15224e-06_rb,0.15417e-06_rb,0.66845e-07_rb /)
      kbo(:, 5,59, 5) = (/ &
        & 0.89316e-09_rb,0.12826e-06_rb,0.16850e-06_rb,0.16775e-06_rb,0.74841e-07_rb /)
      kbo(:, 1,13, 6) = (/ &
        & 0.15452e-03_rb,0.17471e-02_rb,0.22870e-02_rb,0.22757e-02_rb,0.11868e-02_rb /)
      kbo(:, 2,13, 6) = (/ &
        & 0.17358e-03_rb,0.18337e-02_rb,0.23752e-02_rb,0.23549e-02_rb,0.12492e-02_rb /)
      kbo(:, 3,13, 6) = (/ &
        & 0.19043e-03_rb,0.19070e-02_rb,0.24479e-02_rb,0.24255e-02_rb,0.13114e-02_rb /)
      kbo(:, 4,13, 6) = (/ &
        & 0.20459e-03_rb,0.19702e-02_rb,0.25109e-02_rb,0.24905e-02_rb,0.13717e-02_rb /)
      kbo(:, 5,13, 6) = (/ &
        & 0.21621e-03_rb,0.20266e-02_rb,0.25653e-02_rb,0.25512e-02_rb,0.14320e-02_rb /)
      kbo(:, 1,14, 6) = (/ &
        & 0.12793e-03_rb,0.14662e-02_rb,0.19141e-02_rb,0.19032e-02_rb,0.99404e-03_rb /)
      kbo(:, 2,14, 6) = (/ &
        & 0.14335e-03_rb,0.15367e-02_rb,0.19852e-02_rb,0.19691e-02_rb,0.10462e-02_rb /)
      kbo(:, 3,14, 6) = (/ &
        & 0.15698e-03_rb,0.15974e-02_rb,0.20454e-02_rb,0.20285e-02_rb,0.10986e-02_rb /)
      kbo(:, 4,14, 6) = (/ &
        & 0.16837e-03_rb,0.16505e-02_rb,0.20982e-02_rb,0.20828e-02_rb,0.11509e-02_rb /)
      kbo(:, 5,14, 6) = (/ &
        & 0.17765e-03_rb,0.16990e-02_rb,0.21452e-02_rb,0.21357e-02_rb,0.12020e-02_rb /)
      kbo(:, 1,15, 6) = (/ &
        & 0.10600e-03_rb,0.12280e-02_rb,0.15987e-02_rb,0.15899e-02_rb,0.83160e-03_rb /)
      kbo(:, 2,15, 6) = (/ &
        & 0.11846e-03_rb,0.12858e-02_rb,0.16570e-02_rb,0.16438e-02_rb,0.87641e-03_rb /)
      kbo(:, 3,15, 6) = (/ &
        & 0.12947e-03_rb,0.13362e-02_rb,0.17073e-02_rb,0.16937e-02_rb,0.92037e-03_rb /)
      kbo(:, 4,15, 6) = (/ &
        & 0.13860e-03_rb,0.13818e-02_rb,0.17519e-02_rb,0.17401e-02_rb,0.96514e-03_rb /)
      kbo(:, 5,15, 6) = (/ &
        & 0.14605e-03_rb,0.14231e-02_rb,0.17922e-02_rb,0.17849e-02_rb,0.10088e-02_rb /)
      kbo(:, 1,16, 6) = (/ &
        & 0.87844e-04_rb,0.10268e-02_rb,0.13337e-02_rb,0.13258e-02_rb,0.69471e-03_rb /)
      kbo(:, 2,16, 6) = (/ &
        & 0.97921e-04_rb,0.10747e-02_rb,0.13816e-02_rb,0.13704e-02_rb,0.73309e-03_rb /)
      kbo(:, 3,16, 6) = (/ &
        & 0.10678e-03_rb,0.11173e-02_rb,0.14240e-02_rb,0.14122e-02_rb,0.77126e-03_rb /)
      kbo(:, 4,16, 6) = (/ &
        & 0.11410e-03_rb,0.11561e-02_rb,0.14619e-02_rb,0.14525e-02_rb,0.80983e-03_rb /)
      kbo(:, 5,16, 6) = (/ &
        & 0.12008e-03_rb,0.11913e-02_rb,0.14957e-02_rb,0.14904e-02_rb,0.84685e-03_rb /)
      kbo(:, 1,17, 6) = (/ &
        & 0.72736e-04_rb,0.85770e-03_rb,0.11110e-02_rb,0.11043e-02_rb,0.57988e-03_rb /)
      kbo(:, 2,17, 6) = (/ &
        & 0.80870e-04_rb,0.89737e-03_rb,0.11508e-02_rb,0.11412e-02_rb,0.61310e-03_rb /)
      kbo(:, 3,17, 6) = (/ &
        & 0.87984e-04_rb,0.93334e-03_rb,0.11865e-02_rb,0.11769e-02_rb,0.64563e-03_rb /)
      kbo(:, 4,17, 6) = (/ &
        & 0.93900e-04_rb,0.96637e-03_rb,0.12188e-02_rb,0.12115e-02_rb,0.67946e-03_rb /)
      kbo(:, 5,17, 6) = (/ &
        & 0.98669e-04_rb,0.99682e-03_rb,0.12477e-02_rb,0.12444e-02_rb,0.71129e-03_rb /)
      kbo(:, 1,18, 6) = (/ &
        & 0.60196e-04_rb,0.71585e-03_rb,0.92484e-03_rb,0.91885e-03_rb,0.48428e-03_rb /)
      kbo(:, 2,18, 6) = (/ &
        & 0.66759e-04_rb,0.74902e-03_rb,0.95806e-03_rb,0.95007e-03_rb,0.51254e-03_rb /)
      kbo(:, 3,18, 6) = (/ &
        & 0.72486e-04_rb,0.77936e-03_rb,0.98828e-03_rb,0.98036e-03_rb,0.54118e-03_rb /)
      kbo(:, 4,18, 6) = (/ &
        & 0.77210e-04_rb,0.80757e-03_rb,0.10157e-02_rb,0.10101e-02_rb,0.56995e-03_rb /)
      kbo(:, 5,18, 6) = (/ &
        & 0.81046e-04_rb,0.83344e-03_rb,0.10405e-02_rb,0.10390e-02_rb,0.59762e-03_rb /)
      kbo(:, 1,19, 6) = (/ &
        & 0.49797e-04_rb,0.59714e-03_rb,0.76951e-03_rb,0.76408e-03_rb,0.40487e-03_rb /)
      kbo(:, 2,19, 6) = (/ &
        & 0.55098e-04_rb,0.62505e-03_rb,0.79728e-03_rb,0.79071e-03_rb,0.42904e-03_rb /)
      kbo(:, 3,19, 6) = (/ &
        & 0.59690e-04_rb,0.65071e-03_rb,0.82294e-03_rb,0.81671e-03_rb,0.45375e-03_rb /)
      kbo(:, 4,19, 6) = (/ &
        & 0.63457e-04_rb,0.67490e-03_rb,0.84635e-03_rb,0.84245e-03_rb,0.47833e-03_rb /)
      kbo(:, 5,19, 6) = (/ &
        & 0.66538e-04_rb,0.69683e-03_rb,0.86805e-03_rb,0.86812e-03_rb,0.50238e-03_rb /)
      kbo(:, 1,20, 6) = (/ &
        & 0.41277e-04_rb,0.49854e-03_rb,0.64043e-03_rb,0.63582e-03_rb,0.33885e-03_rb /)
      kbo(:, 2,20, 6) = (/ &
        & 0.45557e-04_rb,0.52197e-03_rb,0.66408e-03_rb,0.65859e-03_rb,0.35960e-03_rb /)
      kbo(:, 3,20, 6) = (/ &
        & 0.49226e-04_rb,0.54380e-03_rb,0.68582e-03_rb,0.68111e-03_rb,0.38108e-03_rb /)
      kbo(:, 4,20, 6) = (/ &
        & 0.52238e-04_rb,0.56442e-03_rb,0.70598e-03_rb,0.70364e-03_rb,0.40199e-03_rb /)
      kbo(:, 5,20, 6) = (/ &
        & 0.54694e-04_rb,0.58320e-03_rb,0.72504e-03_rb,0.72645e-03_rb,0.42282e-03_rb /)
      kbo(:, 1,21, 6) = (/ &
        & 0.34218e-04_rb,0.41624e-03_rb,0.53312e-03_rb,0.52912e-03_rb,0.28376e-03_rb /)
      kbo(:, 2,21, 6) = (/ &
        & 0.37670e-04_rb,0.43597e-03_rb,0.55316e-03_rb,0.54893e-03_rb,0.30161e-03_rb /)
      kbo(:, 3,21, 6) = (/ &
        & 0.40602e-04_rb,0.45467e-03_rb,0.57178e-03_rb,0.56861e-03_rb,0.32023e-03_rb /)
      kbo(:, 4,21, 6) = (/ &
        & 0.43009e-04_rb,0.47220e-03_rb,0.58931e-03_rb,0.58850e-03_rb,0.33805e-03_rb /)
      kbo(:, 5,21, 6) = (/ &
        & 0.44968e-04_rb,0.48850e-03_rb,0.60624e-03_rb,0.60867e-03_rb,0.35633e-03_rb /)
      kbo(:, 1,22, 6) = (/ &
        & 0.28554e-04_rb,0.34872e-03_rb,0.44519e-03_rb,0.44203e-03_rb,0.23865e-03_rb /)
      kbo(:, 2,22, 6) = (/ &
        & 0.31312e-04_rb,0.36543e-03_rb,0.46215e-03_rb,0.45896e-03_rb,0.25429e-03_rb /)
      kbo(:, 3,22, 6) = (/ &
        & 0.33623e-04_rb,0.38136e-03_rb,0.47808e-03_rb,0.47625e-03_rb,0.27020e-03_rb /)
      kbo(:, 4,22, 6) = (/ &
        & 0.35518e-04_rb,0.39638e-03_rb,0.49352e-03_rb,0.49393e-03_rb,0.28567e-03_rb /)
      kbo(:, 5,22, 6) = (/ &
        & 0.37060e-04_rb,0.41046e-03_rb,0.50833e-03_rb,0.51152e-03_rb,0.30169e-03_rb /)
      kbo(:, 1,23, 6) = (/ &
        & 0.23825e-04_rb,0.29241e-03_rb,0.37202e-03_rb,0.36957e-03_rb,0.20120e-03_rb /)
      kbo(:, 2,23, 6) = (/ &
        & 0.26019e-04_rb,0.30664e-03_rb,0.38650e-03_rb,0.38433e-03_rb,0.21484e-03_rb /)
      kbo(:, 3,23, 6) = (/ &
        & 0.27840e-04_rb,0.32027e-03_rb,0.40038e-03_rb,0.39957e-03_rb,0.22829e-03_rb /)
      kbo(:, 4,23, 6) = (/ &
        & 0.29329e-04_rb,0.33317e-03_rb,0.41396e-03_rb,0.41498e-03_rb,0.24169e-03_rb /)
      kbo(:, 5,23, 6) = (/ &
        & 0.30542e-04_rb,0.34547e-03_rb,0.42681e-03_rb,0.43048e-03_rb,0.25568e-03_rb /)
      kbo(:, 1,24, 6) = (/ &
        & 0.19879e-04_rb,0.24552e-03_rb,0.31135e-03_rb,0.30936e-03_rb,0.16986e-03_rb /)
      kbo(:, 2,24, 6) = (/ &
        & 0.21617e-04_rb,0.25767e-03_rb,0.32376e-03_rb,0.32228e-03_rb,0.18173e-03_rb /)
      kbo(:, 3,24, 6) = (/ &
        & 0.23050e-04_rb,0.26937e-03_rb,0.33590e-03_rb,0.33579e-03_rb,0.19320e-03_rb /)
      kbo(:, 4,24, 6) = (/ &
        & 0.24217e-04_rb,0.28054e-03_rb,0.34772e-03_rb,0.34922e-03_rb,0.20501e-03_rb /)
      kbo(:, 5,24, 6) = (/ &
        & 0.25164e-04_rb,0.29121e-03_rb,0.35883e-03_rb,0.36286e-03_rb,0.21708e-03_rb /)
      kbo(:, 1,25, 6) = (/ &
        & 0.16589e-04_rb,0.20645e-03_rb,0.26087e-03_rb,0.25944e-03_rb,0.14369e-03_rb /)
      kbo(:, 2,25, 6) = (/ &
        & 0.17961e-04_rb,0.21688e-03_rb,0.27162e-03_rb,0.27083e-03_rb,0.15386e-03_rb /)
      kbo(:, 3,25, 6) = (/ &
        & 0.19083e-04_rb,0.22697e-03_rb,0.28229e-03_rb,0.28258e-03_rb,0.16383e-03_rb /)
      kbo(:, 4,25, 6) = (/ &
        & 0.19998e-04_rb,0.23668e-03_rb,0.29245e-03_rb,0.29438e-03_rb,0.17418e-03_rb /)
      kbo(:, 5,25, 6) = (/ &
        & 0.20737e-04_rb,0.24589e-03_rb,0.30216e-03_rb,0.30647e-03_rb,0.18465e-03_rb /)
      kbo(:, 1,26, 6) = (/ &
        & 0.13858e-04_rb,0.17402e-03_rb,0.21906e-03_rb,0.21804e-03_rb,0.12205e-03_rb /)
      kbo(:, 2,26, 6) = (/ &
        & 0.14930e-04_rb,0.18300e-03_rb,0.22843e-03_rb,0.22813e-03_rb,0.13065e-03_rb /)
      kbo(:, 3,26, 6) = (/ &
        & 0.15807e-04_rb,0.19174e-03_rb,0.23770e-03_rb,0.23830e-03_rb,0.13933e-03_rb /)
      kbo(:, 4,26, 6) = (/ &
        & 0.16522e-04_rb,0.20010e-03_rb,0.24640e-03_rb,0.24876e-03_rb,0.14837e-03_rb /)
      kbo(:, 5,26, 6) = (/ &
        & 0.17090e-04_rb,0.20798e-03_rb,0.25500e-03_rb,0.25943e-03_rb,0.15748e-03_rb /)
      kbo(:, 1,27, 6) = (/ &
        & 0.11565e-04_rb,0.14689e-03_rb,0.18423e-03_rb,0.18358e-03_rb,0.10372e-03_rb /)
      kbo(:, 2,27, 6) = (/ &
        & 0.12404e-04_rb,0.15466e-03_rb,0.19242e-03_rb,0.19242e-03_rb,0.11104e-03_rb /)
      kbo(:, 3,27, 6) = (/ &
        & 0.13088e-04_rb,0.16225e-03_rb,0.20035e-03_rb,0.20136e-03_rb,0.11877e-03_rb /)
      kbo(:, 4,27, 6) = (/ &
        & 0.13644e-04_rb,0.16938e-03_rb,0.20793e-03_rb,0.21054e-03_rb,0.12658e-03_rb /)
      kbo(:, 5,27, 6) = (/ &
        & 0.14079e-04_rb,0.17614e-03_rb,0.21553e-03_rb,0.22004e-03_rb,0.13455e-03_rb /)
      kbo(:, 1,28, 6) = (/ &
        & 0.96429e-05_rb,0.12422e-03_rb,0.15518e-03_rb,0.15486e-03_rb,0.88240e-04_rb /)
      kbo(:, 2,28, 6) = (/ &
        & 0.10297e-04_rb,0.13093e-03_rb,0.16229e-03_rb,0.16255e-03_rb,0.94664e-04_rb /)
      kbo(:, 3,28, 6) = (/ &
        & 0.10829e-04_rb,0.13744e-03_rb,0.16911e-03_rb,0.17043e-03_rb,0.10137e-03_rb /)
      kbo(:, 4,28, 6) = (/ &
        & 0.11263e-04_rb,0.14355e-03_rb,0.17577e-03_rb,0.17861e-03_rb,0.10818e-03_rb /)
      kbo(:, 5,28, 6) = (/ &
        & 0.11593e-04_rb,0.14936e-03_rb,0.18245e-03_rb,0.18698e-03_rb,0.11517e-03_rb /)
      kbo(:, 1,29, 6) = (/ &
        & 0.80357e-05_rb,0.10525e-03_rb,0.13098e-03_rb,0.13087e-03_rb,0.75199e-04_rb /)
      kbo(:, 2,29, 6) = (/ &
        & 0.85457e-05_rb,0.11106e-03_rb,0.13709e-03_rb,0.13756e-03_rb,0.80877e-04_rb /)
      kbo(:, 3,29, 6) = (/ &
        & 0.89585e-05_rb,0.11659e-03_rb,0.14298e-03_rb,0.14458e-03_rb,0.86703e-04_rb /)
      kbo(:, 4,29, 6) = (/ &
        & 0.92936e-05_rb,0.12185e-03_rb,0.14887e-03_rb,0.15184e-03_rb,0.92679e-04_rb /)
      kbo(:, 5,29, 6) = (/ &
        & 0.95413e-05_rb,0.12685e-03_rb,0.15476e-03_rb,0.15926e-03_rb,0.98770e-04_rb /)
      kbo(:, 1,30, 6) = (/ &
        & 0.66891e-05_rb,0.89326e-04_rb,0.11070e-03_rb,0.11075e-03_rb,0.64254e-04_rb /)
      kbo(:, 2,30, 6) = (/ &
        & 0.70855e-05_rb,0.94280e-04_rb,0.11595e-03_rb,0.11668e-03_rb,0.69215e-04_rb /)
      kbo(:, 3,30, 6) = (/ &
        & 0.74068e-05_rb,0.99029e-04_rb,0.12112e-03_rb,0.12290e-03_rb,0.74278e-04_rb /)
      kbo(:, 4,30, 6) = (/ &
        & 0.76634e-05_rb,0.10356e-03_rb,0.12631e-03_rb,0.12933e-03_rb,0.79489e-04_rb /)
      kbo(:, 5,30, 6) = (/ &
        & 0.78499e-05_rb,0.10785e-03_rb,0.13152e-03_rb,0.13588e-03_rb,0.84762e-04_rb /)
      kbo(:, 1,31, 6) = (/ &
        & 0.55647e-05_rb,0.75943e-04_rb,0.93695e-04_rb,0.93906e-04_rb,0.55002e-04_rb /)
      kbo(:, 2,31, 6) = (/ &
        & 0.58719e-05_rb,0.80167e-04_rb,0.98229e-04_rb,0.99204e-04_rb,0.59329e-04_rb /)
      kbo(:, 3,31, 6) = (/ &
        & 0.61212e-05_rb,0.84249e-04_rb,0.10281e-03_rb,0.10471e-03_rb,0.63778e-04_rb /)
      kbo(:, 4,31, 6) = (/ &
        & 0.63162e-05_rb,0.88122e-04_rb,0.10738e-03_rb,0.11041e-03_rb,0.68318e-04_rb /)
      kbo(:, 5,31, 6) = (/ &
        & 0.64553e-05_rb,0.91835e-04_rb,0.11204e-03_rb,0.11614e-03_rb,0.72907e-04_rb /)
      kbo(:, 1,32, 6) = (/ &
        & 0.46246e-05_rb,0.64633e-04_rb,0.79398e-04_rb,0.79831e-04_rb,0.47190e-04_rb /)
      kbo(:, 2,32, 6) = (/ &
        & 0.48628e-05_rb,0.68258e-04_rb,0.83383e-04_rb,0.84521e-04_rb,0.50953e-04_rb /)
      kbo(:, 3,32, 6) = (/ &
        & 0.50557e-05_rb,0.71758e-04_rb,0.87418e-04_rb,0.89417e-04_rb,0.54823e-04_rb /)
      kbo(:, 4,32, 6) = (/ &
        & 0.52035e-05_rb,0.75099e-04_rb,0.91481e-04_rb,0.94423e-04_rb,0.58785e-04_rb /)
      kbo(:, 5,32, 6) = (/ &
        & 0.53068e-05_rb,0.78317e-04_rb,0.95646e-04_rb,0.99452e-04_rb,0.62766e-04_rb /)
      kbo(:, 1,33, 6) = (/ &
        & 0.38403e-05_rb,0.55077e-04_rb,0.67410e-04_rb,0.68024e-04_rb,0.40538e-04_rb /)
      kbo(:, 2,33, 6) = (/ &
        & 0.40244e-05_rb,0.58199e-04_rb,0.70924e-04_rb,0.72180e-04_rb,0.43839e-04_rb /)
      kbo(:, 3,33, 6) = (/ &
        & 0.41730e-05_rb,0.61206e-04_rb,0.74475e-04_rb,0.76502e-04_rb,0.47222e-04_rb /)
      kbo(:, 4,33, 6) = (/ &
        & 0.42835e-05_rb,0.64091e-04_rb,0.78114e-04_rb,0.80883e-04_rb,0.50652e-04_rb /)
      kbo(:, 5,33, 6) = (/ &
        & 0.43604e-05_rb,0.66898e-04_rb,0.81798e-04_rb,0.85293e-04_rb,0.54102e-04_rb /)
      kbo(:, 1,34, 6) = (/ &
        & 0.31801e-05_rb,0.46888e-04_rb,0.57233e-04_rb,0.57945e-04_rb,0.34781e-04_rb /)
      kbo(:, 2,34, 6) = (/ &
        & 0.33244e-05_rb,0.49586e-04_rb,0.60319e-04_rb,0.61625e-04_rb,0.37646e-04_rb /)
      kbo(:, 3,34, 6) = (/ &
        & 0.34389e-05_rb,0.52183e-04_rb,0.63465e-04_rb,0.65422e-04_rb,0.40606e-04_rb /)
      kbo(:, 4,34, 6) = (/ &
        & 0.35228e-05_rb,0.54696e-04_rb,0.66697e-04_rb,0.69266e-04_rb,0.43571e-04_rb /)
      kbo(:, 5,34, 6) = (/ &
        & 0.35804e-05_rb,0.57162e-04_rb,0.69964e-04_rb,0.73120e-04_rb,0.46562e-04_rb /)
      kbo(:, 1,35, 6) = (/ &
        & 0.26200e-05_rb,0.39742e-04_rb,0.48410e-04_rb,0.49141e-04_rb,0.29639e-04_rb /)
      kbo(:, 2,35, 6) = (/ &
        & 0.27351e-05_rb,0.42075e-04_rb,0.51122e-04_rb,0.52382e-04_rb,0.32146e-04_rb /)
      kbo(:, 3,35, 6) = (/ &
        & 0.28253e-05_rb,0.44325e-04_rb,0.53916e-04_rb,0.55698e-04_rb,0.34701e-04_rb /)
      kbo(:, 4,35, 6) = (/ &
        & 0.28907e-05_rb,0.46530e-04_rb,0.56769e-04_rb,0.59068e-04_rb,0.37280e-04_rb /)
      kbo(:, 5,35, 6) = (/ &
        & 0.29354e-05_rb,0.48707e-04_rb,0.59649e-04_rb,0.62461e-04_rb,0.39864e-04_rb /)
      kbo(:, 1,36, 6) = (/ &
        & 0.21469e-05_rb,0.33498e-04_rb,0.40744e-04_rb,0.41427e-04_rb,0.25045e-04_rb /)
      kbo(:, 2,36, 6) = (/ &
        & 0.22407e-05_rb,0.35513e-04_rb,0.43124e-04_rb,0.44262e-04_rb,0.27227e-04_rb /)
      kbo(:, 3,36, 6) = (/ &
        & 0.23140e-05_rb,0.37481e-04_rb,0.45588e-04_rb,0.47157e-04_rb,0.29433e-04_rb /)
      kbo(:, 4,36, 6) = (/ &
        & 0.23673e-05_rb,0.39419e-04_rb,0.48109e-04_rb,0.50107e-04_rb,0.31672e-04_rb /)
      kbo(:, 5,36, 6) = (/ &
        & 0.24035e-05_rb,0.41345e-04_rb,0.50640e-04_rb,0.53089e-04_rb,0.33911e-04_rb /)
      kbo(:, 1,37, 6) = (/ &
        & 0.17464e-05_rb,0.27984e-04_rb,0.34007e-04_rb,0.34584e-04_rb,0.20899e-04_rb /)
      kbo(:, 2,37, 6) = (/ &
        & 0.18252e-05_rb,0.29737e-04_rb,0.36088e-04_rb,0.37051e-04_rb,0.22785e-04_rb /)
      kbo(:, 3,37, 6) = (/ &
        & 0.18877e-05_rb,0.31460e-04_rb,0.38247e-04_rb,0.39573e-04_rb,0.24687e-04_rb /)
      kbo(:, 4,37, 6) = (/ &
        & 0.19333e-05_rb,0.33165e-04_rb,0.40460e-04_rb,0.42146e-04_rb,0.26626e-04_rb /)
      kbo(:, 5,37, 6) = (/ &
        & 0.19647e-05_rb,0.34869e-04_rb,0.42691e-04_rb,0.44765e-04_rb,0.28561e-04_rb /)
      kbo(:, 1,38, 6) = (/ &
        & 0.14198e-05_rb,0.23364e-04_rb,0.28367e-04_rb,0.28854e-04_rb,0.17422e-04_rb /)
      kbo(:, 2,38, 6) = (/ &
        & 0.14862e-05_rb,0.24892e-04_rb,0.30186e-04_rb,0.30994e-04_rb,0.19047e-04_rb /)
      kbo(:, 3,38, 6) = (/ &
        & 0.15394e-05_rb,0.26399e-04_rb,0.32076e-04_rb,0.33192e-04_rb,0.20688e-04_rb /)
      kbo(:, 4,38, 6) = (/ &
        & 0.15786e-05_rb,0.27898e-04_rb,0.34020e-04_rb,0.35441e-04_rb,0.22360e-04_rb /)
      kbo(:, 5,38, 6) = (/ &
        & 0.16058e-05_rb,0.29401e-04_rb,0.35983e-04_rb,0.37735e-04_rb,0.24033e-04_rb /)
      kbo(:, 1,39, 6) = (/ &
        & 0.11541e-05_rb,0.19509e-04_rb,0.23663e-04_rb,0.24072e-04_rb,0.14521e-04_rb /)
      kbo(:, 2,39, 6) = (/ &
        & 0.12100e-05_rb,0.20837e-04_rb,0.25252e-04_rb,0.25929e-04_rb,0.15915e-04_rb /)
      kbo(:, 3,39, 6) = (/ &
        & 0.12551e-05_rb,0.22155e-04_rb,0.26902e-04_rb,0.27842e-04_rb,0.17331e-04_rb /)
      kbo(:, 4,39, 6) = (/ &
        & 0.12888e-05_rb,0.23470e-04_rb,0.28603e-04_rb,0.29806e-04_rb,0.18773e-04_rb /)
      kbo(:, 5,39, 6) = (/ &
        & 0.13123e-05_rb,0.24794e-04_rb,0.30331e-04_rb,0.31815e-04_rb,0.20216e-04_rb /)
      kbo(:, 1,40, 6) = (/ &
        & 0.93244e-06_rb,0.16171e-04_rb,0.19599e-04_rb,0.19918e-04_rb,0.11977e-04_rb /)
      kbo(:, 2,40, 6) = (/ &
        & 0.98059e-06_rb,0.17324e-04_rb,0.20976e-04_rb,0.21519e-04_rb,0.13169e-04_rb /)
      kbo(:, 3,40, 6) = (/ &
        & 0.10196e-05_rb,0.18475e-04_rb,0.22411e-04_rb,0.23178e-04_rb,0.14387e-04_rb /)
      kbo(:, 4,40, 6) = (/ &
        & 0.10496e-05_rb,0.19626e-04_rb,0.23897e-04_rb,0.24887e-04_rb,0.15625e-04_rb /)
      kbo(:, 5,40, 6) = (/ &
        & 0.10706e-05_rb,0.20792e-04_rb,0.25411e-04_rb,0.26641e-04_rb,0.16871e-04_rb /)
      kbo(:, 1,41, 6) = (/ &
        & 0.75206e-06_rb,0.13381e-04_rb,0.16204e-04_rb,0.16448e-04_rb,0.98582e-05_rb /)
      kbo(:, 2,41, 6) = (/ &
        & 0.79368e-06_rb,0.14380e-04_rb,0.17395e-04_rb,0.17824e-04_rb,0.10873e-04_rb /)
      kbo(:, 3,41, 6) = (/ &
        & 0.82742e-06_rb,0.15382e-04_rb,0.18640e-04_rb,0.19259e-04_rb,0.11916e-04_rb /)
      kbo(:, 4,41, 6) = (/ &
        & 0.85390e-06_rb,0.16388e-04_rb,0.19935e-04_rb,0.20745e-04_rb,0.12979e-04_rb /)
      kbo(:, 5,41, 6) = (/ &
        & 0.87281e-06_rb,0.17412e-04_rb,0.21261e-04_rb,0.22271e-04_rb,0.14053e-04_rb /)
      kbo(:, 1,42, 6) = (/ &
        & 0.60591e-06_rb,0.11061e-04_rb,0.13385e-04_rb,0.13568e-04_rb,0.81045e-05_rb /)
      kbo(:, 2,42, 6) = (/ &
        & 0.64182e-06_rb,0.11926e-04_rb,0.14412e-04_rb,0.14751e-04_rb,0.89661e-05_rb /)
      kbo(:, 3,42, 6) = (/ &
        & 0.67101e-06_rb,0.12797e-04_rb,0.15489e-04_rb,0.15987e-04_rb,0.98578e-05_rb /)
      kbo(:, 4,42, 6) = (/ &
        & 0.69427e-06_rb,0.13675e-04_rb,0.16616e-04_rb,0.17274e-04_rb,0.10770e-04_rb /)
      kbo(:, 5,42, 6) = (/ &
        & 0.71122e-06_rb,0.14573e-04_rb,0.17774e-04_rb,0.18601e-04_rb,0.11690e-04_rb /)
      kbo(:, 1,43, 6) = (/ &
        & 0.48565e-06_rb,0.90929e-05_rb,0.10998e-04_rb,0.11127e-04_rb,0.66136e-05_rb /)
      kbo(:, 2,43, 6) = (/ &
        & 0.51703e-06_rb,0.98401e-05_rb,0.11878e-04_rb,0.12136e-04_rb,0.73431e-05_rb /)
      kbo(:, 3,43, 6) = (/ &
        & 0.54246e-06_rb,0.10594e-04_rb,0.12807e-04_rb,0.13197e-04_rb,0.81019e-05_rb /)
      kbo(:, 4,43, 6) = (/ &
        & 0.56309e-06_rb,0.11359e-04_rb,0.13782e-04_rb,0.14306e-04_rb,0.88812e-05_rb /)
      kbo(:, 5,43, 6) = (/ &
        & 0.57854e-06_rb,0.12144e-04_rb,0.14791e-04_rb,0.15456e-04_rb,0.96685e-05_rb /)
      kbo(:, 1,44, 6) = (/ &
        & 0.38777e-06_rb,0.74478e-05_rb,0.90062e-05_rb,0.90921e-05_rb,0.53709e-05_rb /)
      kbo(:, 2,44, 6) = (/ &
        & 0.41522e-06_rb,0.80919e-05_rb,0.97570e-05_rb,0.99478e-05_rb,0.59889e-05_rb /)
      kbo(:, 3,44, 6) = (/ &
        & 0.43761e-06_rb,0.87433e-05_rb,0.10555e-04_rb,0.10856e-04_rb,0.66332e-05_rb /)
      kbo(:, 4,44, 6) = (/ &
        & 0.45584e-06_rb,0.94063e-05_rb,0.11395e-04_rb,0.11808e-04_rb,0.72955e-05_rb /)
      kbo(:, 5,44, 6) = (/ &
        & 0.46990e-06_rb,0.10091e-04_rb,0.12269e-04_rb,0.12799e-04_rb,0.79696e-05_rb /)
      kbo(:, 1,45, 6) = (/ &
        & 0.30888e-06_rb,0.60898e-05_rb,0.73632e-05_rb,0.74181e-05_rb,0.43535e-05_rb /)
      kbo(:, 2,45, 6) = (/ &
        & 0.33291e-06_rb,0.66440e-05_rb,0.80023e-05_rb,0.81416e-05_rb,0.48741e-05_rb /)
      kbo(:, 3,45, 6) = (/ &
        & 0.35255e-06_rb,0.72052e-05_rb,0.86855e-05_rb,0.89154e-05_rb,0.54207e-05_rb /)
      kbo(:, 4,45, 6) = (/ &
        & 0.36854e-06_rb,0.77787e-05_rb,0.94068e-05_rb,0.97308e-05_rb,0.59833e-05_rb /)
      kbo(:, 5,45, 6) = (/ &
        & 0.38125e-06_rb,0.83745e-05_rb,0.10163e-04_rb,0.10583e-04_rb,0.65578e-05_rb /)
      kbo(:, 1,46, 6) = (/ &
        & 0.24494e-06_rb,0.49598e-05_rb,0.59987e-05_rb,0.60289e-05_rb,0.35123e-05_rb /)
      kbo(:, 2,46, 6) = (/ &
        & 0.26595e-06_rb,0.54355e-05_rb,0.65411e-05_rb,0.66383e-05_rb,0.39494e-05_rb /)
      kbo(:, 3,46, 6) = (/ &
        & 0.28328e-06_rb,0.59173e-05_rb,0.71233e-05_rb,0.72943e-05_rb,0.44113e-05_rb /)
      kbo(:, 4,46, 6) = (/ &
        & 0.29735e-06_rb,0.64123e-05_rb,0.77406e-05_rb,0.79904e-05_rb,0.48883e-05_rb /)
      kbo(:, 5,46, 6) = (/ &
        & 0.30876e-06_rb,0.69283e-05_rb,0.83913e-05_rb,0.87211e-05_rb,0.53766e-05_rb /)
      kbo(:, 1,47, 6) = (/ &
        & 0.19287e-06_rb,0.40134e-05_rb,0.48586e-05_rb,0.48718e-05_rb,0.28114e-05_rb /)
      kbo(:, 2,47, 6) = (/ &
        & 0.21123e-06_rb,0.44208e-05_rb,0.53178e-05_rb,0.53794e-05_rb,0.31781e-05_rb /)
      kbo(:, 3,47, 6) = (/ &
        & 0.22658e-06_rb,0.48332e-05_rb,0.58104e-05_rb,0.59335e-05_rb,0.35646e-05_rb /)
      kbo(:, 4,47, 6) = (/ &
        & 0.23912e-06_rb,0.52582e-05_rb,0.63354e-05_rb,0.65230e-05_rb,0.39676e-05_rb /)
      kbo(:, 5,47, 6) = (/ &
        & 0.24931e-06_rb,0.57025e-05_rb,0.68920e-05_rb,0.71458e-05_rb,0.43826e-05_rb /)
      kbo(:, 1,48, 6) = (/ &
        & 0.15132e-06_rb,0.32395e-05_rb,0.39269e-05_rb,0.39289e-05_rb,0.22458e-05_rb /)
      kbo(:, 2,48, 6) = (/ &
        & 0.16716e-06_rb,0.35866e-05_rb,0.43144e-05_rb,0.43511e-05_rb,0.25512e-05_rb /)
      kbo(:, 3,48, 6) = (/ &
        & 0.18076e-06_rb,0.39391e-05_rb,0.47294e-05_rb,0.48161e-05_rb,0.28746e-05_rb /)
      kbo(:, 4,48, 6) = (/ &
        & 0.19194e-06_rb,0.43030e-05_rb,0.51748e-05_rb,0.53142e-05_rb,0.32135e-05_rb /)
      kbo(:, 5,48, 6) = (/ &
        & 0.20099e-06_rb,0.46842e-05_rb,0.56488e-05_rb,0.58430e-05_rb,0.35644e-05_rb /)
      kbo(:, 1,49, 6) = (/ &
        & 0.11828e-06_rb,0.26078e-05_rb,0.31672e-05_rb,0.31617e-05_rb,0.17894e-05_rb /)
      kbo(:, 2,49, 6) = (/ &
        & 0.13185e-06_rb,0.29029e-05_rb,0.34929e-05_rb,0.35125e-05_rb,0.20423e-05_rb /)
      kbo(:, 3,49, 6) = (/ &
        & 0.14378e-06_rb,0.32030e-05_rb,0.38409e-05_rb,0.39000e-05_rb,0.23121e-05_rb /)
      kbo(:, 4,49, 6) = (/ &
        & 0.15369e-06_rb,0.35139e-05_rb,0.42179e-05_rb,0.43198e-05_rb,0.25971e-05_rb /)
      kbo(:, 5,49, 6) = (/ &
        & 0.16176e-06_rb,0.38391e-05_rb,0.46198e-05_rb,0.47667e-05_rb,0.28929e-05_rb /)
      kbo(:, 1,50, 6) = (/ &
        & 0.92362e-07_rb,0.20997e-05_rb,0.25556e-05_rb,0.25471e-05_rb,0.14267e-05_rb /)
      kbo(:, 2,50, 6) = (/ &
        & 0.10390e-06_rb,0.23496e-05_rb,0.28286e-05_rb,0.28368e-05_rb,0.16356e-05_rb /)
      kbo(:, 3,50, 6) = (/ &
        & 0.11421e-06_rb,0.26049e-05_rb,0.31213e-05_rb,0.31606e-05_rb,0.18609e-05_rb /)
      kbo(:, 4,50, 6) = (/ &
        & 0.12297e-06_rb,0.28694e-05_rb,0.34392e-05_rb,0.35125e-05_rb,0.20996e-05_rb /)
      kbo(:, 5,50, 6) = (/ &
        & 0.13012e-06_rb,0.31471e-05_rb,0.37794e-05_rb,0.38900e-05_rb,0.23486e-05_rb /)
      kbo(:, 1,51, 6) = (/ &
        & 0.72029e-07_rb,0.16879e-05_rb,0.20606e-05_rb,0.20506e-05_rb,0.11365e-05_rb /)
      kbo(:, 2,51, 6) = (/ &
        & 0.81717e-07_rb,0.18994e-05_rb,0.22886e-05_rb,0.22902e-05_rb,0.13081e-05_rb /)
      kbo(:, 3,51, 6) = (/ &
        & 0.90547e-07_rb,0.21162e-05_rb,0.25347e-05_rb,0.25588e-05_rb,0.14964e-05_rb /)
      kbo(:, 4,51, 6) = (/ &
        & 0.98231e-07_rb,0.23407e-05_rb,0.28017e-05_rb,0.28537e-05_rb,0.16958e-05_rb /)
      kbo(:, 5,51, 6) = (/ &
        & 0.10454e-06_rb,0.25770e-05_rb,0.30891e-05_rb,0.31719e-05_rb,0.19053e-05_rb /)
      kbo(:, 1,52, 6) = (/ &
        & 0.55994e-07_rb,0.13535e-05_rb,0.16583e-05_rb,0.16483e-05_rb,0.90215e-06_rb /)
      kbo(:, 2,52, 6) = (/ &
        & 0.64065e-07_rb,0.15318e-05_rb,0.18483e-05_rb,0.18452e-05_rb,0.10447e-05_rb /)
      kbo(:, 3,52, 6) = (/ &
        & 0.71601e-07_rb,0.17156e-05_rb,0.20541e-05_rb,0.20679e-05_rb,0.12005e-05_rb /)
      kbo(:, 4,52, 6) = (/ &
        & 0.78265e-07_rb,0.19055e-05_rb,0.22786e-05_rb,0.23151e-05_rb,0.13673e-05_rb /)
      kbo(:, 5,52, 6) = (/ &
        & 0.83827e-07_rb,0.21062e-05_rb,0.25201e-05_rb,0.25812e-05_rb,0.15426e-05_rb /)
      kbo(:, 1,53, 6) = (/ &
        & 0.43376e-07_rb,0.10823e-05_rb,0.13316e-05_rb,0.13229e-05_rb,0.71387e-06_rb /)
      kbo(:, 2,53, 6) = (/ &
        & 0.50078e-07_rb,0.12320e-05_rb,0.14898e-05_rb,0.14843e-05_rb,0.83196e-06_rb /)
      kbo(:, 3,53, 6) = (/ &
        & 0.56440e-07_rb,0.13873e-05_rb,0.16612e-05_rb,0.16678e-05_rb,0.96085e-06_rb /)
      kbo(:, 4,53, 6) = (/ &
        & 0.62188e-07_rb,0.15481e-05_rb,0.18489e-05_rb,0.18736e-05_rb,0.10995e-05_rb /)
      kbo(:, 5,53, 6) = (/ &
        & 0.67072e-07_rb,0.17179e-05_rb,0.20520e-05_rb,0.20964e-05_rb,0.12465e-05_rb /)
      kbo(:, 1,54, 6) = (/ &
        & 0.33662e-07_rb,0.86685e-06_rb,0.10712e-05_rb,0.10638e-05_rb,0.56675e-06_rb /)
      kbo(:, 2,54, 6) = (/ &
        & 0.39188e-07_rb,0.99237e-06_rb,0.12024e-05_rb,0.11965e-05_rb,0.66407e-06_rb /)
      kbo(:, 3,54, 6) = (/ &
        & 0.44517e-07_rb,0.11234e-05_rb,0.13453e-05_rb,0.13477e-05_rb,0.77015e-06_rb /)
      kbo(:, 4,54, 6) = (/ &
        & 0.49410e-07_rb,0.12591e-05_rb,0.15022e-05_rb,0.15186e-05_rb,0.88568e-06_rb /)
      kbo(:, 5,54, 6) = (/ &
        & 0.53662e-07_rb,0.14028e-05_rb,0.16732e-05_rb,0.17054e-05_rb,0.10088e-05_rb /)
      kbo(:, 1,55, 6) = (/ &
        & 0.26107e-07_rb,0.69409e-06_rb,0.86148e-06_rb,0.85535e-06_rb,0.44993e-06_rb /)
      kbo(:, 2,55, 6) = (/ &
        & 0.30648e-07_rb,0.79874e-06_rb,0.97050e-06_rb,0.96432e-06_rb,0.52987e-06_rb /)
      kbo(:, 3,55, 6) = (/ &
        & 0.35070e-07_rb,0.90919e-06_rb,0.10893e-05_rb,0.10889e-05_rb,0.61727e-06_rb /)
      kbo(:, 4,55, 6) = (/ &
        & 0.39223e-07_rb,0.10237e-05_rb,0.12206e-05_rb,0.12308e-05_rb,0.71345e-06_rb /)
      kbo(:, 5,55, 6) = (/ &
        & 0.42897e-07_rb,0.11450e-05_rb,0.13641e-05_rb,0.13878e-05_rb,0.81614e-06_rb /)
      kbo(:, 1,56, 6) = (/ &
        & 0.20183e-07_rb,0.55461e-06_rb,0.69187e-06_rb,0.68686e-06_rb,0.35634e-06_rb /)
      kbo(:, 2,56, 6) = (/ &
        & 0.23906e-07_rb,0.64143e-06_rb,0.78193e-06_rb,0.77615e-06_rb,0.42133e-06_rb /)
      kbo(:, 3,56, 6) = (/ &
        & 0.27573e-07_rb,0.73422e-06_rb,0.88059e-06_rb,0.87848e-06_rb,0.49387e-06_rb /)
      kbo(:, 4,56, 6) = (/ &
        & 0.31066e-07_rb,0.83083e-06_rb,0.99001e-06_rb,0.99604e-06_rb,0.57362e-06_rb /)
      kbo(:, 5,56, 6) = (/ &
        & 0.34213e-07_rb,0.93302e-06_rb,0.11103e-05_rb,0.11271e-05_rb,0.65906e-06_rb /)
      kbo(:, 1,57, 6) = (/ &
        & 0.15553e-07_rb,0.44221e-06_rb,0.55475e-06_rb,0.55051e-06_rb,0.28158e-06_rb /)
      kbo(:, 2,57, 6) = (/ &
        & 0.18597e-07_rb,0.51387e-06_rb,0.62883e-06_rb,0.62381e-06_rb,0.33432e-06_rb /)
      kbo(:, 3,57, 6) = (/ &
        & 0.21627e-07_rb,0.59165e-06_rb,0.71079e-06_rb,0.70768e-06_rb,0.39423e-06_rb /)
      kbo(:, 4,57, 6) = (/ &
        & 0.24548e-07_rb,0.67286e-06_rb,0.80159e-06_rb,0.80451e-06_rb,0.46009e-06_rb /)
      kbo(:, 5,57, 6) = (/ &
        & 0.27223e-07_rb,0.75870e-06_rb,0.90196e-06_rb,0.91372e-06_rb,0.53126e-06_rb /)
      kbo(:, 1,58, 6) = (/ &
        & 0.11989e-07_rb,0.35271e-06_rb,0.44504e-06_rb,0.44164e-06_rb,0.22247e-06_rb /)
      kbo(:, 2,58, 6) = (/ &
        & 0.14473e-07_rb,0.41167e-06_rb,0.50593e-06_rb,0.50179e-06_rb,0.26557e-06_rb /)
      kbo(:, 3,58, 6) = (/ &
        & 0.16959e-07_rb,0.47681e-06_rb,0.57389e-06_rb,0.57055e-06_rb,0.31488e-06_rb /)
      kbo(:, 4,58, 6) = (/ &
        & 0.19386e-07_rb,0.54503e-06_rb,0.64933e-06_rb,0.65043e-06_rb,0.36927e-06_rb /)
      kbo(:, 5,58, 6) = (/ &
        & 0.21655e-07_rb,0.61705e-06_rb,0.73285e-06_rb,0.74092e-06_rb,0.42823e-06_rb /)
      kbo(:, 1,59, 6) = (/ &
        & 0.95745e-08_rb,0.28909e-06_rb,0.36504e-06_rb,0.36221e-06_rb,0.18143e-06_rb /)
      kbo(:, 2,59, 6) = (/ &
        & 0.11602e-07_rb,0.33869e-06_rb,0.41625e-06_rb,0.41267e-06_rb,0.21754e-06_rb /)
      kbo(:, 3,59, 6) = (/ &
        & 0.13639e-07_rb,0.39363e-06_rb,0.47345e-06_rb,0.47053e-06_rb,0.25873e-06_rb /)
      kbo(:, 4,59, 6) = (/ &
        & 0.15635e-07_rb,0.45132e-06_rb,0.53722e-06_rb,0.53779e-06_rb,0.30436e-06_rb /)
      kbo(:, 5,59, 6) = (/ &
        & 0.17515e-07_rb,0.51255e-06_rb,0.60804e-06_rb,0.61445e-06_rb,0.35395e-06_rb /)
      kbo(:, 1,13, 7) = (/ &
        & 0.13818e-02_rb,0.53095e-02_rb,0.61744e-02_rb,0.61409e-02_rb,0.38201e-02_rb /)
      kbo(:, 2,13, 7) = (/ &
        & 0.14927e-02_rb,0.54262e-02_rb,0.63169e-02_rb,0.63268e-02_rb,0.39787e-02_rb /)
      kbo(:, 3,13, 7) = (/ &
        & 0.15882e-02_rb,0.55245e-02_rb,0.64575e-02_rb,0.65042e-02_rb,0.41285e-02_rb /)
      kbo(:, 4,13, 7) = (/ &
        & 0.16683e-02_rb,0.56028e-02_rb,0.65859e-02_rb,0.66764e-02_rb,0.42766e-02_rb /)
      kbo(:, 5,13, 7) = (/ &
        & 0.17364e-02_rb,0.56642e-02_rb,0.66996e-02_rb,0.68400e-02_rb,0.44196e-02_rb /)
      kbo(:, 1,14, 7) = (/ &
        & 0.11439e-02_rb,0.44642e-02_rb,0.52052e-02_rb,0.51954e-02_rb,0.32185e-02_rb /)
      kbo(:, 2,14, 7) = (/ &
        & 0.12322e-02_rb,0.45612e-02_rb,0.53295e-02_rb,0.53554e-02_rb,0.33544e-02_rb /)
      kbo(:, 3,14, 7) = (/ &
        & 0.13082e-02_rb,0.46425e-02_rb,0.54474e-02_rb,0.55111e-02_rb,0.34869e-02_rb /)
      kbo(:, 4,14, 7) = (/ &
        & 0.13723e-02_rb,0.47078e-02_rb,0.55568e-02_rb,0.56605e-02_rb,0.36178e-02_rb /)
      kbo(:, 5,14, 7) = (/ &
        & 0.14273e-02_rb,0.47614e-02_rb,0.56536e-02_rb,0.58004e-02_rb,0.37445e-02_rb /)
      kbo(:, 1,15, 7) = (/ &
        & 0.94600e-03_rb,0.37452e-02_rb,0.43786e-02_rb,0.43839e-02_rb,0.27064e-02_rb /)
      kbo(:, 2,15, 7) = (/ &
        & 0.10166e-02_rb,0.38272e-02_rb,0.44845e-02_rb,0.45238e-02_rb,0.28259e-02_rb /)
      kbo(:, 3,15, 7) = (/ &
        & 0.10772e-02_rb,0.38951e-02_rb,0.45852e-02_rb,0.46569e-02_rb,0.29424e-02_rb /)
      kbo(:, 4,15, 7) = (/ &
        & 0.11287e-02_rb,0.39508e-02_rb,0.46776e-02_rb,0.47856e-02_rb,0.30573e-02_rb /)
      kbo(:, 5,15, 7) = (/ &
        & 0.11727e-02_rb,0.39984e-02_rb,0.47639e-02_rb,0.49094e-02_rb,0.31707e-02_rb /)
      kbo(:, 1,16, 7) = (/ &
        & 0.78170e-03_rb,0.31386e-02_rb,0.36753e-02_rb,0.36925e-02_rb,0.22749e-02_rb /)
      kbo(:, 2,16, 7) = (/ &
        & 0.83828e-03_rb,0.32067e-02_rb,0.37655e-02_rb,0.38108e-02_rb,0.23791e-02_rb /)
      kbo(:, 3,16, 7) = (/ &
        & 0.88677e-03_rb,0.32636e-02_rb,0.38524e-02_rb,0.39267e-02_rb,0.24809e-02_rb /)
      kbo(:, 4,16, 7) = (/ &
        & 0.92814e-03_rb,0.33127e-02_rb,0.39332e-02_rb,0.40389e-02_rb,0.25819e-02_rb /)
      kbo(:, 5,16, 7) = (/ &
        & 0.96342e-03_rb,0.33561e-02_rb,0.40123e-02_rb,0.41493e-02_rb,0.26825e-02_rb /)
      kbo(:, 1,17, 7) = (/ &
        & 0.64513e-03_rb,0.26258e-02_rb,0.30792e-02_rb,0.31027e-02_rb,0.19109e-02_rb /)
      kbo(:, 2,17, 7) = (/ &
        & 0.69065e-03_rb,0.26830e-02_rb,0.31573e-02_rb,0.32056e-02_rb,0.20013e-02_rb /)
      kbo(:, 3,17, 7) = (/ &
        & 0.72951e-03_rb,0.27321e-02_rb,0.32326e-02_rb,0.33061e-02_rb,0.20905e-02_rb /)
      kbo(:, 4,17, 7) = (/ &
        & 0.76279e-03_rb,0.27759e-02_rb,0.33057e-02_rb,0.34049e-02_rb,0.21791e-02_rb /)
      kbo(:, 5,17, 7) = (/ &
        & 0.79127e-03_rb,0.28164e-02_rb,0.33776e-02_rb,0.35037e-02_rb,0.22680e-02_rb /)
      kbo(:, 1,18, 7) = (/ &
        & 0.53216e-03_rb,0.21945e-02_rb,0.25768e-02_rb,0.26041e-02_rb,0.16047e-02_rb /)
      kbo(:, 2,18, 7) = (/ &
        & 0.56878e-03_rb,0.22431e-02_rb,0.26453e-02_rb,0.26934e-02_rb,0.16829e-02_rb /)
      kbo(:, 3,18, 7) = (/ &
        & 0.60002e-03_rb,0.22864e-02_rb,0.27121e-02_rb,0.27812e-02_rb,0.17605e-02_rb /)
      kbo(:, 4,18, 7) = (/ &
        & 0.62666e-03_rb,0.23265e-02_rb,0.27778e-02_rb,0.28689e-02_rb,0.18389e-02_rb /)
      kbo(:, 5,18, 7) = (/ &
        & 0.64956e-03_rb,0.23645e-02_rb,0.28436e-02_rb,0.29587e-02_rb,0.19175e-02_rb /)
      kbo(:, 1,19, 7) = (/ &
        & 0.43878e-03_rb,0.18328e-02_rb,0.21555e-02_rb,0.21844e-02_rb,0.13470e-02_rb /)
      kbo(:, 2,19, 7) = (/ &
        & 0.46832e-03_rb,0.18749e-02_rb,0.22160e-02_rb,0.22619e-02_rb,0.14147e-02_rb /)
      kbo(:, 3,19, 7) = (/ &
        & 0.49337e-03_rb,0.19136e-02_rb,0.22752e-02_rb,0.23395e-02_rb,0.14834e-02_rb /)
      kbo(:, 4,19, 7) = (/ &
        & 0.51483e-03_rb,0.19509e-02_rb,0.23344e-02_rb,0.24177e-02_rb,0.15526e-02_rb /)
      kbo(:, 5,19, 7) = (/ &
        & 0.53321e-03_rb,0.19863e-02_rb,0.23950e-02_rb,0.24983e-02_rb,0.16228e-02_rb /)
      kbo(:, 1,20, 7) = (/ &
        & 0.36218e-03_rb,0.15310e-02_rb,0.18044e-02_rb,0.18333e-02_rb,0.11320e-02_rb /)
      kbo(:, 2,20, 7) = (/ &
        & 0.38587e-03_rb,0.15681e-02_rb,0.18573e-02_rb,0.19010e-02_rb,0.11911e-02_rb /)
      kbo(:, 3,20, 7) = (/ &
        & 0.40599e-03_rb,0.16034e-02_rb,0.19105e-02_rb,0.19693e-02_rb,0.12516e-02_rb /)
      kbo(:, 4,20, 7) = (/ &
        & 0.42327e-03_rb,0.16372e-02_rb,0.19642e-02_rb,0.20400e-02_rb,0.13133e-02_rb /)
      kbo(:, 5,20, 7) = (/ &
        & 0.43800e-03_rb,0.16702e-02_rb,0.20196e-02_rb,0.21120e-02_rb,0.13760e-02_rb /)
      kbo(:, 1,21, 7) = (/ &
        & 0.29894e-03_rb,0.12794e-02_rb,0.15109e-02_rb,0.15392e-02_rb,0.95196e-03_rb /)
      kbo(:, 2,21, 7) = (/ &
        & 0.31795e-03_rb,0.13126e-02_rb,0.15578e-02_rb,0.15985e-02_rb,0.10041e-02_rb /)
      kbo(:, 3,21, 7) = (/ &
        & 0.33409e-03_rb,0.13444e-02_rb,0.16054e-02_rb,0.16590e-02_rb,0.10575e-02_rb /)
      kbo(:, 4,21, 7) = (/ &
        & 0.34798e-03_rb,0.13752e-02_rb,0.16542e-02_rb,0.17228e-02_rb,0.11124e-02_rb /)
      kbo(:, 5,21, 7) = (/ &
        & 0.35982e-03_rb,0.14062e-02_rb,0.17046e-02_rb,0.17872e-02_rb,0.11688e-02_rb /)
      kbo(:, 1,22, 7) = (/ &
        & 0.24779e-03_rb,0.10720e-02_rb,0.12687e-02_rb,0.12959e-02_rb,0.80428e-03_rb /)
      kbo(:, 2,22, 7) = (/ &
        & 0.26287e-03_rb,0.11012e-02_rb,0.13104e-02_rb,0.13483e-02_rb,0.85046e-03_rb /)
      kbo(:, 3,22, 7) = (/ &
        & 0.27570e-03_rb,0.11300e-02_rb,0.13532e-02_rb,0.14032e-02_rb,0.89796e-03_rb /)
      kbo(:, 4,22, 7) = (/ &
        & 0.28676e-03_rb,0.11583e-02_rb,0.13978e-02_rb,0.14602e-02_rb,0.94722e-03_rb /)
      kbo(:, 5,22, 7) = (/ &
        & 0.29612e-03_rb,0.11873e-02_rb,0.14439e-02_rb,0.15185e-02_rb,0.99812e-03_rb /)
      kbo(:, 1,23, 7) = (/ &
        & 0.20536e-03_rb,0.89901e-03_rb,0.10666e-02_rb,0.10925e-02_rb,0.68077e-03_rb /)
      kbo(:, 2,23, 7) = (/ &
        & 0.21732e-03_rb,0.92514e-03_rb,0.11037e-02_rb,0.11394e-02_rb,0.72189e-03_rb /)
      kbo(:, 3,23, 7) = (/ &
        & 0.22752e-03_rb,0.95109e-03_rb,0.11426e-02_rb,0.11888e-02_rb,0.76437e-03_rb /)
      kbo(:, 4,23, 7) = (/ &
        & 0.23632e-03_rb,0.97728e-03_rb,0.11832e-02_rb,0.12403e-02_rb,0.80877e-03_rb /)
      kbo(:, 5,23, 7) = (/ &
        & 0.24369e-03_rb,0.10040e-02_rb,0.12260e-02_rb,0.12934e-02_rb,0.85489e-03_rb /)
      kbo(:, 1,24, 7) = (/ &
        & 0.17021e-03_rb,0.75506e-03_rb,0.89800e-03_rb,0.92265e-03_rb,0.57788e-03_rb /)
      kbo(:, 2,24, 7) = (/ &
        & 0.17969e-03_rb,0.77853e-03_rb,0.93149e-03_rb,0.96497e-03_rb,0.61443e-03_rb /)
      kbo(:, 3,24, 7) = (/ &
        & 0.18778e-03_rb,0.80207e-03_rb,0.96680e-03_rb,0.10097e-02_rb,0.65278e-03_rb /)
      kbo(:, 4,24, 7) = (/ &
        & 0.19477e-03_rb,0.82622e-03_rb,0.10041e-02_rb,0.10564e-02_rb,0.69293e-03_rb /)
      kbo(:, 5,24, 7) = (/ &
        & 0.20055e-03_rb,0.85104e-03_rb,0.10439e-02_rb,0.11053e-02_rb,0.73424e-03_rb /)
      kbo(:, 1,25, 7) = (/ &
        & 0.14110e-03_rb,0.63531e-03_rb,0.75746e-03_rb,0.78109e-03_rb,0.49195e-03_rb /)
      kbo(:, 2,25, 7) = (/ &
        & 0.14860e-03_rb,0.65629e-03_rb,0.78801e-03_rb,0.81951e-03_rb,0.52472e-03_rb /)
      kbo(:, 3,25, 7) = (/ &
        & 0.15502e-03_rb,0.67802e-03_rb,0.82018e-03_rb,0.86005e-03_rb,0.55929e-03_rb /)
      kbo(:, 4,25, 7) = (/ &
        & 0.16054e-03_rb,0.70023e-03_rb,0.85487e-03_rb,0.90270e-03_rb,0.59546e-03_rb /)
      kbo(:, 5,25, 7) = (/ &
        & 0.16506e-03_rb,0.72333e-03_rb,0.89213e-03_rb,0.94794e-03_rb,0.63296e-03_rb /)
      kbo(:, 1,26, 7) = (/ &
        & 0.11704e-03_rb,0.53563e-03_rb,0.64089e-03_rb,0.66348e-03_rb,0.42036e-03_rb /)
      kbo(:, 2,26, 7) = (/ &
        & 0.12296e-03_rb,0.55482e-03_rb,0.66859e-03_rb,0.69842e-03_rb,0.44986e-03_rb /)
      kbo(:, 3,26, 7) = (/ &
        & 0.12804e-03_rb,0.57460e-03_rb,0.69838e-03_rb,0.73532e-03_rb,0.48115e-03_rb /)
      kbo(:, 4,26, 7) = (/ &
        & 0.13237e-03_rb,0.59528e-03_rb,0.73081e-03_rb,0.77472e-03_rb,0.51371e-03_rb /)
      kbo(:, 5,26, 7) = (/ &
        & 0.13587e-03_rb,0.61708e-03_rb,0.76562e-03_rb,0.81649e-03_rb,0.54800e-03_rb /)
      kbo(:, 1,27, 7) = (/ &
        & 0.97037e-04_rb,0.45263e-03_rb,0.54353e-03_rb,0.56528e-03_rb,0.36031e-03_rb /)
      kbo(:, 2,27, 7) = (/ &
        & 0.10171e-03_rb,0.47018e-03_rb,0.56899e-03_rb,0.59710e-03_rb,0.38698e-03_rb /)
      kbo(:, 3,27, 7) = (/ &
        & 0.10572e-03_rb,0.48833e-03_rb,0.59675e-03_rb,0.63096e-03_rb,0.41505e-03_rb /)
      kbo(:, 4,27, 7) = (/ &
        & 0.10911e-03_rb,0.50767e-03_rb,0.62713e-03_rb,0.66736e-03_rb,0.44466e-03_rb /)
      kbo(:, 5,27, 7) = (/ &
        & 0.11182e-03_rb,0.52830e-03_rb,0.65949e-03_rb,0.70600e-03_rb,0.47625e-03_rb /)
      kbo(:, 1,28, 7) = (/ &
        & 0.80419e-04_rb,0.38349e-03_rb,0.46224e-03_rb,0.48308e-03_rb,0.30982e-03_rb /)
      kbo(:, 2,28, 7) = (/ &
        & 0.84110e-04_rb,0.39943e-03_rb,0.48584e-03_rb,0.51214e-03_rb,0.33386e-03_rb /)
      kbo(:, 3,28, 7) = (/ &
        & 0.87283e-04_rb,0.41636e-03_rb,0.51189e-03_rb,0.54353e-03_rb,0.35924e-03_rb /)
      kbo(:, 4,28, 7) = (/ &
        & 0.89914e-04_rb,0.43456e-03_rb,0.54020e-03_rb,0.57698e-03_rb,0.38635e-03_rb /)
      kbo(:, 5,28, 7) = (/ &
        & 0.92003e-04_rb,0.45406e-03_rb,0.57030e-03_rb,0.61286e-03_rb,0.41553e-03_rb /)
      kbo(:, 1,29, 7) = (/ &
        & 0.66637e-04_rb,0.32580e-03_rb,0.39453e-03_rb,0.41439e-03_rb,0.26738e-03_rb /)
      kbo(:, 2,29, 7) = (/ &
        & 0.69552e-04_rb,0.34052e-03_rb,0.41660e-03_rb,0.44116e-03_rb,0.28904e-03_rb /)
      kbo(:, 3,29, 7) = (/ &
        & 0.72042e-04_rb,0.35633e-03_rb,0.44102e-03_rb,0.47005e-03_rb,0.31218e-03_rb /)
      kbo(:, 4,29, 7) = (/ &
        & 0.74082e-04_rb,0.37351e-03_rb,0.46731e-03_rb,0.50108e-03_rb,0.33714e-03_rb /)
      kbo(:, 5,29, 7) = (/ &
        & 0.75692e-04_rb,0.39190e-03_rb,0.49529e-03_rb,0.53447e-03_rb,0.36421e-03_rb /)
      kbo(:, 1,30, 7) = (/ &
        & 0.55194e-04_rb,0.27757e-03_rb,0.33804e-03_rb,0.35670e-03_rb,0.23148e-03_rb /)
      kbo(:, 2,30, 7) = (/ &
        & 0.57489e-04_rb,0.29130e-03_rb,0.35876e-03_rb,0.38143e-03_rb,0.25115e-03_rb /)
      kbo(:, 3,30, 7) = (/ &
        & 0.59445e-04_rb,0.30619e-03_rb,0.38148e-03_rb,0.40810e-03_rb,0.27242e-03_rb /)
      kbo(:, 4,30, 7) = (/ &
        & 0.61024e-04_rb,0.32233e-03_rb,0.40578e-03_rb,0.43698e-03_rb,0.29552e-03_rb /)
      kbo(:, 5,30, 7) = (/ &
        & 0.62251e-04_rb,0.33969e-03_rb,0.43185e-03_rb,0.46830e-03_rb,0.32052e-03_rb /)
      kbo(:, 1,31, 7) = (/ &
        & 0.45701e-04_rb,0.23736e-03_rb,0.29086e-03_rb,0.30834e-03_rb,0.20114e-03_rb /)
      kbo(:, 2,31, 7) = (/ &
        & 0.47516e-04_rb,0.25020e-03_rb,0.31032e-03_rb,0.33113e-03_rb,0.21916e-03_rb /)
      kbo(:, 3,31, 7) = (/ &
        & 0.49034e-04_rb,0.26424e-03_rb,0.33131e-03_rb,0.35593e-03_rb,0.23874e-03_rb /)
      kbo(:, 4,31, 7) = (/ &
        & 0.50247e-04_rb,0.27943e-03_rb,0.35395e-03_rb,0.38299e-03_rb,0.26024e-03_rb /)
      kbo(:, 5,31, 7) = (/ &
        & 0.51186e-04_rb,0.29578e-03_rb,0.37829e-03_rb,0.41250e-03_rb,0.28341e-03_rb /)
      kbo(:, 1,32, 7) = (/ &
        & 0.37829e-04_rb,0.20379e-03_rb,0.25145e-03_rb,0.26761e-03_rb,0.17549e-03_rb /)
      kbo(:, 2,32, 7) = (/ &
        & 0.39256e-04_rb,0.21585e-03_rb,0.26951e-03_rb,0.28871e-03_rb,0.19207e-03_rb /)
      kbo(:, 3,32, 7) = (/ &
        & 0.40434e-04_rb,0.22904e-03_rb,0.28903e-03_rb,0.31192e-03_rb,0.21034e-03_rb /)
      kbo(:, 4,32, 7) = (/ &
        & 0.41367e-04_rb,0.24332e-03_rb,0.31020e-03_rb,0.33744e-03_rb,0.23024e-03_rb /)
      kbo(:, 5,32, 7) = (/ &
        & 0.42072e-04_rb,0.25873e-03_rb,0.33309e-03_rb,0.36527e-03_rb,0.25179e-03_rb /)
      kbo(:, 1,33, 7) = (/ &
        & 0.31302e-04_rb,0.17572e-03_rb,0.21830e-03_rb,0.23321e-03_rb,0.15375e-03_rb /)
      kbo(:, 2,33, 7) = (/ &
        & 0.32419e-04_rb,0.18706e-03_rb,0.23504e-03_rb,0.25291e-03_rb,0.16915e-03_rb /)
      kbo(:, 3,33, 7) = (/ &
        & 0.33327e-04_rb,0.19940e-03_rb,0.25328e-03_rb,0.27479e-03_rb,0.18617e-03_rb /)
      kbo(:, 4,33, 7) = (/ &
        & 0.34039e-04_rb,0.21283e-03_rb,0.27315e-03_rb,0.29887e-03_rb,0.20465e-03_rb /)
      kbo(:, 5,33, 7) = (/ &
        & 0.34568e-04_rb,0.22731e-03_rb,0.29474e-03_rb,0.32519e-03_rb,0.22485e-03_rb /)
      kbo(:, 1,34, 7) = (/ &
        & 0.25851e-04_rb,0.15182e-03_rb,0.18976e-03_rb,0.20353e-03_rb,0.13483e-03_rb /)
      kbo(:, 2,34, 7) = (/ &
        & 0.26731e-04_rb,0.16241e-03_rb,0.20531e-03_rb,0.22199e-03_rb,0.14918e-03_rb /)
      kbo(:, 3,34, 7) = (/ &
        & 0.27437e-04_rb,0.17395e-03_rb,0.22237e-03_rb,0.24258e-03_rb,0.16491e-03_rb /)
      kbo(:, 4,34, 7) = (/ &
        & 0.27988e-04_rb,0.18656e-03_rb,0.24113e-03_rb,0.26529e-03_rb,0.18217e-03_rb /)
      kbo(:, 5,34, 7) = (/ &
        & 0.28384e-04_rb,0.20017e-03_rb,0.26144e-03_rb,0.29020e-03_rb,0.20116e-03_rb /)
      kbo(:, 1,35, 7) = (/ &
        & 0.21271e-04_rb,0.13078e-03_rb,0.16426e-03_rb,0.17685e-03_rb,0.11756e-03_rb /)
      kbo(:, 2,35, 7) = (/ &
        & 0.21975e-04_rb,0.14058e-03_rb,0.17865e-03_rb,0.19403e-03_rb,0.13072e-03_rb /)
      kbo(:, 3,35, 7) = (/ &
        & 0.22533e-04_rb,0.15132e-03_rb,0.19456e-03_rb,0.21323e-03_rb,0.14523e-03_rb /)
      kbo(:, 4,35, 7) = (/ &
        & 0.22966e-04_rb,0.16307e-03_rb,0.21206e-03_rb,0.23454e-03_rb,0.16133e-03_rb /)
      kbo(:, 5,35, 7) = (/ &
        & 0.23274e-04_rb,0.17579e-03_rb,0.23112e-03_rb,0.25800e-03_rb,0.17911e-03_rb /)
      kbo(:, 1,36, 7) = (/ &
        & 0.17431e-04_rb,0.11205e-03_rb,0.14128e-03_rb,0.15260e-03_rb,0.10161e-03_rb /)
      kbo(:, 2,36, 7) = (/ &
        & 0.18004e-04_rb,0.12103e-03_rb,0.15448e-03_rb,0.16840e-03_rb,0.11356e-03_rb /)
      kbo(:, 3,36, 7) = (/ &
        & 0.18458e-04_rb,0.13095e-03_rb,0.16918e-03_rb,0.18616e-03_rb,0.12687e-03_rb /)
      kbo(:, 4,36, 7) = (/ &
        & 0.18810e-04_rb,0.14182e-03_rb,0.18539e-03_rb,0.20596e-03_rb,0.14179e-03_rb /)
      kbo(:, 5,36, 7) = (/ &
        & 0.19059e-04_rb,0.15363e-03_rb,0.20317e-03_rb,0.22788e-03_rb,0.15831e-03_rb /)
      kbo(:, 1,37, 7) = (/ &
        & 0.14204e-04_rb,0.95010e-04_rb,0.12008e-03_rb,0.12996e-03_rb,0.86482e-04_rb /)
      kbo(:, 2,37, 7) = (/ &
        & 0.14685e-04_rb,0.10313e-03_rb,0.13200e-03_rb,0.14426e-03_rb,0.97179e-04_rb /)
      kbo(:, 3,37, 7) = (/ &
        & 0.15069e-04_rb,0.11214e-03_rb,0.14536e-03_rb,0.16042e-03_rb,0.10922e-03_rb /)
      kbo(:, 4,37, 7) = (/ &
        & 0.15369e-04_rb,0.12211e-03_rb,0.16020e-03_rb,0.17855e-03_rb,0.12282e-03_rb /)
      kbo(:, 5,37, 7) = (/ &
        & 0.15586e-04_rb,0.13296e-03_rb,0.17661e-03_rb,0.19876e-03_rb,0.13797e-03_rb /)
      kbo(:, 1,38, 7) = (/ &
        & 0.11571e-04_rb,0.80545e-04_rb,0.10206e-03_rb,0.11063e-03_rb,0.73563e-04_rb /)
      kbo(:, 2,38, 7) = (/ &
        & 0.11976e-04_rb,0.87875e-04_rb,0.11281e-03_rb,0.12357e-03_rb,0.83149e-04_rb /)
      kbo(:, 3,38, 7) = (/ &
        & 0.12301e-04_rb,0.96063e-04_rb,0.12495e-03_rb,0.13829e-03_rb,0.94026e-04_rb /)
      kbo(:, 4,38, 7) = (/ &
        & 0.12557e-04_rb,0.10518e-03_rb,0.13851e-03_rb,0.15487e-03_rb,0.10642e-03_rb /)
      kbo(:, 5,38, 7) = (/ &
        & 0.12744e-04_rb,0.11515e-03_rb,0.15364e-03_rb,0.17352e-03_rb,0.12029e-03_rb /)
      kbo(:, 1,39, 7) = (/ &
        & 0.94254e-05_rb,0.68333e-04_rb,0.86798e-04_rb,0.94247e-04_rb,0.62604e-04_rb /)
      kbo(:, 2,39, 7) = (/ &
        & 0.97657e-05_rb,0.74945e-04_rb,0.96495e-04_rb,0.10595e-03_rb,0.71185e-04_rb /)
      kbo(:, 3,39, 7) = (/ &
        & 0.10041e-04_rb,0.82376e-04_rb,0.10752e-03_rb,0.11934e-03_rb,0.81022e-04_rb /)
      kbo(:, 4,39, 7) = (/ &
        & 0.10258e-04_rb,0.90687e-04_rb,0.11992e-03_rb,0.13453e-03_rb,0.92315e-04_rb /)
      kbo(:, 5,39, 7) = (/ &
        & 0.10419e-04_rb,0.99852e-04_rb,0.13387e-03_rb,0.15177e-03_rb,0.10504e-03_rb /)
      kbo(:, 1,40, 7) = (/ &
        & 0.76452e-05_rb,0.57428e-04_rb,0.73021e-04_rb,0.79333e-04_rb,0.52568e-04_rb /)
      kbo(:, 2,40, 7) = (/ &
        & 0.79354e-05_rb,0.63315e-04_rb,0.81650e-04_rb,0.89752e-04_rb,0.60144e-04_rb /)
      kbo(:, 3,40, 7) = (/ &
        & 0.81739e-05_rb,0.69961e-04_rb,0.91542e-04_rb,0.10176e-03_rb,0.68905e-04_rb /)
      kbo(:, 4,40, 7) = (/ &
        & 0.83626e-05_rb,0.77456e-04_rb,0.10275e-03_rb,0.11552e-03_rb,0.79059e-04_rb /)
      kbo(:, 5,40, 7) = (/ &
        & 0.85064e-05_rb,0.85795e-04_rb,0.11542e-03_rb,0.13128e-03_rb,0.90578e-04_rb /)
      kbo(:, 1,41, 7) = (/ &
        & 0.61953e-05_rb,0.48163e-04_rb,0.61287e-04_rb,0.66616e-04_rb,0.44011e-04_rb /)
      kbo(:, 2,41, 7) = (/ &
        & 0.64418e-05_rb,0.53383e-04_rb,0.68934e-04_rb,0.75842e-04_rb,0.50670e-04_rb /)
      kbo(:, 3,41, 7) = (/ &
        & 0.66484e-05_rb,0.59312e-04_rb,0.77785e-04_rb,0.86580e-04_rb,0.58448e-04_rb /)
      kbo(:, 4,41, 7) = (/ &
        & 0.68134e-05_rb,0.66054e-04_rb,0.87876e-04_rb,0.98995e-04_rb,0.67543e-04_rb /)
      kbo(:, 5,41, 7) = (/ &
        & 0.69402e-05_rb,0.73614e-04_rb,0.99378e-04_rb,0.11337e-03_rb,0.77942e-04_rb /)
      kbo(:, 1,42, 7) = (/ &
        & 0.50170e-05_rb,0.40358e-04_rb,0.51378e-04_rb,0.55875e-04_rb,0.36794e-04_rb /)
      kbo(:, 2,42, 7) = (/ &
        & 0.52266e-05_rb,0.44971e-04_rb,0.58141e-04_rb,0.64024e-04_rb,0.42639e-04_rb /)
      kbo(:, 3,42, 7) = (/ &
        & 0.54046e-05_rb,0.50250e-04_rb,0.66034e-04_rb,0.73601e-04_rb,0.49527e-04_rb /)
      kbo(:, 4,42, 7) = (/ &
        & 0.55484e-05_rb,0.56302e-04_rb,0.75111e-04_rb,0.84788e-04_rb,0.57654e-04_rb /)
      kbo(:, 5,42, 7) = (/ &
        & 0.56600e-05_rb,0.63153e-04_rb,0.85551e-04_rb,0.97898e-04_rb,0.67040e-04_rb /)
      kbo(:, 1,43, 7) = (/ &
        & 0.40498e-05_rb,0.33563e-04_rb,0.42716e-04_rb,0.46433e-04_rb,0.30447e-04_rb /)
      kbo(:, 2,43, 7) = (/ &
        & 0.42291e-05_rb,0.37603e-04_rb,0.48619e-04_rb,0.53551e-04_rb,0.35516e-04_rb /)
      kbo(:, 3,43, 7) = (/ &
        & 0.43833e-05_rb,0.42260e-04_rb,0.55585e-04_rb,0.61999e-04_rb,0.41549e-04_rb /)
      kbo(:, 4,43, 7) = (/ &
        & 0.45099e-05_rb,0.47650e-04_rb,0.63668e-04_rb,0.71964e-04_rb,0.48728e-04_rb /)
      kbo(:, 5,43, 7) = (/ &
        & 0.46096e-05_rb,0.53790e-04_rb,0.73053e-04_rb,0.83779e-04_rb,0.57105e-04_rb /)
      kbo(:, 1,44, 7) = (/ &
        & 0.32616e-05_rb,0.27781e-04_rb,0.35335e-04_rb,0.38366e-04_rb,0.25032e-04_rb /)
      kbo(:, 2,44, 7) = (/ &
        & 0.34164e-05_rb,0.31296e-04_rb,0.40444e-04_rb,0.44531e-04_rb,0.29399e-04_rb /)
      kbo(:, 3,44, 7) = (/ &
        & 0.35491e-05_rb,0.35375e-04_rb,0.46540e-04_rb,0.51925e-04_rb,0.34633e-04_rb /)
      kbo(:, 4,44, 7) = (/ &
        & 0.36610e-05_rb,0.40146e-04_rb,0.53687e-04_rb,0.60738e-04_rb,0.40935e-04_rb /)
      kbo(:, 5,44, 7) = (/ &
        & 0.37500e-05_rb,0.45618e-04_rb,0.62061e-04_rb,0.71298e-04_rb,0.48356e-04_rb /)
      kbo(:, 1,45, 7) = (/ &
        & 0.26238e-05_rb,0.22952e-04_rb,0.29162e-04_rb,0.31630e-04_rb,0.20522e-04_rb /)
      kbo(:, 2,45, 7) = (/ &
        & 0.27571e-05_rb,0.25992e-04_rb,0.33567e-04_rb,0.36938e-04_rb,0.24268e-04_rb /)
      kbo(:, 3,45, 7) = (/ &
        & 0.28713e-05_rb,0.29549e-04_rb,0.38881e-04_rb,0.43383e-04_rb,0.28792e-04_rb /)
      kbo(:, 4,45, 7) = (/ &
        & 0.29693e-05_rb,0.33760e-04_rb,0.45177e-04_rb,0.51149e-04_rb,0.34303e-04_rb /)
      kbo(:, 5,45, 7) = (/ &
        & 0.30482e-05_rb,0.38627e-04_rb,0.52624e-04_rb,0.60551e-04_rb,0.40857e-04_rb /)
      kbo(:, 1,46, 7) = (/ &
        & 0.21051e-05_rb,0.18867e-04_rb,0.23935e-04_rb,0.25917e-04_rb,0.16710e-04_rb /)
      kbo(:, 2,46, 7) = (/ &
        & 0.22206e-05_rb,0.21480e-04_rb,0.27705e-04_rb,0.30449e-04_rb,0.19893e-04_rb /)
      kbo(:, 3,46, 7) = (/ &
        & 0.23191e-05_rb,0.24557e-04_rb,0.32298e-04_rb,0.36019e-04_rb,0.23769e-04_rb /)
      kbo(:, 4,46, 7) = (/ &
        & 0.24046e-05_rb,0.28239e-04_rb,0.37798e-04_rb,0.42807e-04_rb,0.28545e-04_rb /)
      kbo(:, 5,46, 7) = (/ &
        & 0.24746e-05_rb,0.32543e-04_rb,0.44374e-04_rb,0.51109e-04_rb,0.34299e-04_rb /)
      kbo(:, 1,47, 7) = (/ &
        & 0.16817e-05_rb,0.15386e-04_rb,0.19464e-04_rb,0.21024e-04_rb,0.13455e-04_rb /)
      kbo(:, 2,47, 7) = (/ &
        & 0.17822e-05_rb,0.17605e-04_rb,0.22652e-04_rb,0.24840e-04_rb,0.16124e-04_rb /)
      kbo(:, 3,47, 7) = (/ &
        & 0.18680e-05_rb,0.20236e-04_rb,0.26567e-04_rb,0.29585e-04_rb,0.19401e-04_rb /)
      kbo(:, 4,47, 7) = (/ &
        & 0.19423e-05_rb,0.23413e-04_rb,0.31317e-04_rb,0.35433e-04_rb,0.23476e-04_rb /)
      kbo(:, 5,47, 7) = (/ &
        & 0.20049e-05_rb,0.27181e-04_rb,0.37054e-04_rb,0.42668e-04_rb,0.28466e-04_rb /)
      kbo(:, 1,48, 7) = (/ &
        & 0.13405e-05_rb,0.12514e-04_rb,0.15779e-04_rb,0.16993e-04_rb,0.10791e-04_rb /)
      kbo(:, 2,48, 7) = (/ &
        & 0.14280e-05_rb,0.14385e-04_rb,0.18460e-04_rb,0.20187e-04_rb,0.13017e-04_rb /)
      kbo(:, 3,48, 7) = (/ &
        & 0.15026e-05_rb,0.16623e-04_rb,0.21773e-04_rb,0.24205e-04_rb,0.15768e-04_rb /)
      kbo(:, 4,48, 7) = (/ &
        & 0.15671e-05_rb,0.19354e-04_rb,0.25854e-04_rb,0.29218e-04_rb,0.19227e-04_rb /)
      kbo(:, 5,48, 7) = (/ &
        & 0.16224e-05_rb,0.22629e-04_rb,0.30833e-04_rb,0.35479e-04_rb,0.23525e-04_rb /)
      kbo(:, 1,49, 7) = (/ &
        & 0.10664e-05_rb,0.10149e-04_rb,0.12751e-04_rb,0.13684e-04_rb,0.86188e-05_rb /)
      kbo(:, 2,49, 7) = (/ &
        & 0.11418e-05_rb,0.11720e-04_rb,0.14999e-04_rb,0.16352e-04_rb,0.10467e-04_rb /)
      kbo(:, 3,49, 7) = (/ &
        & 0.12069e-05_rb,0.13612e-04_rb,0.17780e-04_rb,0.19716e-04_rb,0.12759e-04_rb /)
      kbo(:, 4,49, 7) = (/ &
        & 0.12630e-05_rb,0.15945e-04_rb,0.21262e-04_rb,0.23990e-04_rb,0.15676e-04_rb /)
      kbo(:, 5,49, 7) = (/ &
        & 0.13113e-05_rb,0.18775e-04_rb,0.25560e-04_rb,0.29382e-04_rb,0.19352e-04_rb /)
      kbo(:, 1,50, 7) = (/ &
        & 0.84808e-06_rb,0.82401e-05_rb,0.10318e-04_rb,0.11036e-04_rb,0.68939e-05_rb /)
      kbo(:, 2,50, 7) = (/ &
        & 0.91269e-06_rb,0.95513e-05_rb,0.12189e-04_rb,0.13247e-04_rb,0.84209e-05_rb /)
      kbo(:, 3,50, 7) = (/ &
        & 0.96928e-06_rb,0.11160e-04_rb,0.14539e-04_rb,0.16084e-04_rb,0.10336e-04_rb /)
      kbo(:, 4,50, 7) = (/ &
        & 0.10177e-05_rb,0.13146e-04_rb,0.17498e-04_rb,0.19711e-04_rb,0.12790e-04_rb /)
      kbo(:, 5,50, 7) = (/ &
        & 0.10595e-05_rb,0.15592e-04_rb,0.21207e-04_rb,0.24356e-04_rb,0.15929e-04_rb /)
      kbo(:, 1,51, 7) = (/ &
        & 0.67378e-06_rb,0.66826e-05_rb,0.83381e-05_rb,0.88828e-05_rb,0.55052e-05_rb /)
      kbo(:, 2,51, 7) = (/ &
        & 0.72873e-06_rb,0.77753e-05_rb,0.98905e-05_rb,0.10714e-04_rb,0.67636e-05_rb /)
      kbo(:, 3,51, 7) = (/ &
        & 0.77763e-06_rb,0.91368e-05_rb,0.11866e-04_rb,0.13091e-04_rb,0.83559e-05_rb /)
      kbo(:, 4,51, 7) = (/ &
        & 0.81953e-06_rb,0.10824e-04_rb,0.14378e-04_rb,0.16168e-04_rb,0.10417e-04_rb /)
      kbo(:, 5,51, 7) = (/ &
        & 0.85561e-06_rb,0.12933e-04_rb,0.17571e-04_rb,0.20159e-04_rb,0.13087e-04_rb /)
      kbo(:, 1,52, 7) = (/ &
        & 0.53444e-06_rb,0.54066e-05_rb,0.67197e-05_rb,0.71297e-05_rb,0.43814e-05_rb /)
      kbo(:, 2,52, 7) = (/ &
        & 0.58095e-06_rb,0.63124e-05_rb,0.79989e-05_rb,0.86346e-05_rb,0.54122e-05_rb /)
      kbo(:, 3,52, 7) = (/ &
        & 0.62282e-06_rb,0.74561e-05_rb,0.96489e-05_rb,0.10614e-04_rb,0.67289e-05_rb /)
      kbo(:, 4,52, 7) = (/ &
        & 0.65913e-06_rb,0.88917e-05_rb,0.11780e-04_rb,0.13220e-04_rb,0.84497e-05_rb /)
      kbo(:, 5,52, 7) = (/ &
        & 0.69032e-06_rb,0.10691e-04_rb,0.14504e-04_rb,0.16615e-04_rb,0.10706e-04_rb /)
      kbo(:, 1,53, 7) = (/ &
        & 0.42309e-06_rb,0.43635e-05_rb,0.54017e-05_rb,0.57047e-05_rb,0.34733e-05_rb /)
      kbo(:, 2,53, 7) = (/ &
        & 0.46223e-06_rb,0.51097e-05_rb,0.64465e-05_rb,0.69322e-05_rb,0.43120e-05_rb /)
      kbo(:, 3,53, 7) = (/ &
        & 0.49791e-06_rb,0.60643e-05_rb,0.78166e-05_rb,0.85690e-05_rb,0.53948e-05_rb /)
      kbo(:, 4,53, 7) = (/ &
        & 0.52942e-06_rb,0.72726e-05_rb,0.96042e-05_rb,0.10748e-04_rb,0.68211e-05_rb /)
      kbo(:, 5,53, 7) = (/ &
        & 0.55635e-06_rb,0.88074e-05_rb,0.11924e-04_rb,0.13632e-04_rb,0.87176e-05_rb /)
      kbo(:, 1,54, 7) = (/ &
        & 0.33525e-06_rb,0.35291e-05_rb,0.43533e-05_rb,0.45783e-05_rb,0.27618e-05_rb /)
      kbo(:, 2,54, 7) = (/ &
        & 0.36801e-06_rb,0.41464e-05_rb,0.52098e-05_rb,0.55821e-05_rb,0.34468e-05_rb /)
      kbo(:, 3,54, 7) = (/ &
        & 0.39830e-06_rb,0.49442e-05_rb,0.63495e-05_rb,0.69393e-05_rb,0.43390e-05_rb /)
      kbo(:, 4,54, 7) = (/ &
        & 0.42530e-06_rb,0.59638e-05_rb,0.78520e-05_rb,0.87652e-05_rb,0.55238e-05_rb /)
      kbo(:, 5,54, 7) = (/ &
        & 0.44850e-06_rb,0.72711e-05_rb,0.98256e-05_rb,0.11213e-04_rb,0.71180e-05_rb /)
      kbo(:, 1,55, 7) = (/ &
        & 0.26561e-06_rb,0.28535e-05_rb,0.35091e-05_rb,0.36747e-05_rb,0.21953e-05_rb /)
      kbo(:, 2,55, 7) = (/ &
        & 0.29299e-06_rb,0.33630e-05_rb,0.42093e-05_rb,0.44935e-05_rb,0.27543e-05_rb /)
      kbo(:, 3,55, 7) = (/ &
        & 0.31855e-06_rb,0.40286e-05_rb,0.51555e-05_rb,0.56160e-05_rb,0.34885e-05_rb /)
      kbo(:, 4,55, 7) = (/ &
        & 0.34149e-06_rb,0.48884e-05_rb,0.64166e-05_rb,0.71432e-05_rb,0.44712e-05_rb /)
      kbo(:, 5,55, 7) = (/ &
        & 0.36149e-06_rb,0.60082e-05_rb,0.81053e-05_rb,0.92338e-05_rb,0.58114e-05_rb /)
      kbo(:, 1,56, 7) = (/ &
        & 0.21017e-06_rb,0.23019e-05_rb,0.28217e-05_rb,0.29420e-05_rb,0.17403e-05_rb /)
      kbo(:, 2,56, 7) = (/ &
        & 0.23294e-06_rb,0.27215e-05_rb,0.33922e-05_rb,0.36070e-05_rb,0.21943e-05_rb /)
      kbo(:, 3,56, 7) = (/ &
        & 0.25436e-06_rb,0.32732e-05_rb,0.41720e-05_rb,0.45291e-05_rb,0.27950e-05_rb /)
      kbo(:, 4,56, 7) = (/ &
        & 0.27383e-06_rb,0.39951e-05_rb,0.52252e-05_rb,0.57990e-05_rb,0.36054e-05_rb /)
      kbo(:, 5,56, 7) = (/ &
        & 0.29106e-06_rb,0.49429e-05_rb,0.66518e-05_rb,0.75613e-05_rb,0.47246e-05_rb /)
      kbo(:, 1,57, 7) = (/ &
        & 0.16608e-06_rb,0.18531e-05_rb,0.22639e-05_rb,0.23504e-05_rb,0.13760e-05_rb /)
      kbo(:, 2,57, 7) = (/ &
        & 0.18488e-06_rb,0.21971e-05_rb,0.27271e-05_rb,0.28870e-05_rb,0.17421e-05_rb /)
      kbo(:, 3,57, 7) = (/ &
        & 0.20279e-06_rb,0.26520e-05_rb,0.33659e-05_rb,0.36392e-05_rb,0.22316e-05_rb /)
      kbo(:, 4,57, 7) = (/ &
        & 0.21931e-06_rb,0.32543e-05_rb,0.42391e-05_rb,0.46891e-05_rb,0.28949e-05_rb /)
      kbo(:, 5,57, 7) = (/ &
        & 0.23404e-06_rb,0.40527e-05_rb,0.54380e-05_rb,0.61649e-05_rb,0.38234e-05_rb /)
      kbo(:, 1,58, 7) = (/ &
        & 0.13131e-06_rb,0.14927e-05_rb,0.18179e-05_rb,0.18787e-05_rb,0.10893e-05_rb /)
      kbo(:, 2,58, 7) = (/ &
        & 0.14677e-06_rb,0.17748e-05_rb,0.21946e-05_rb,0.23122e-05_rb,0.13839e-05_rb /)
      kbo(:, 3,58, 7) = (/ &
        & 0.16171e-06_rb,0.21494e-05_rb,0.27171e-05_rb,0.29256e-05_rb,0.17827e-05_rb /)
      kbo(:, 4,58, 7) = (/ &
        & 0.17567e-06_rb,0.26514e-05_rb,0.34407e-05_rb,0.37923e-05_rb,0.23269e-05_rb /)
      kbo(:, 5,58, 7) = (/ &
        & 0.18816e-06_rb,0.33237e-05_rb,0.44466e-05_rb,0.50280e-05_rb,0.30958e-05_rb /)
      kbo(:, 1,59, 7) = (/ &
        & 0.10599e-06_rb,0.12374e-05_rb,0.15060e-05_rb,0.15543e-05_rb,0.89750e-06_rb /)
      kbo(:, 2,59, 7) = (/ &
        & 0.11868e-06_rb,0.14795e-05_rb,0.18285e-05_rb,0.19251e-05_rb,0.11479e-05_rb /)
      kbo(:, 3,59, 7) = (/ &
        & 0.13098e-06_rb,0.18042e-05_rb,0.22807e-05_rb,0.24552e-05_rb,0.14897e-05_rb /)
      kbo(:, 4,59, 7) = (/ &
        & 0.14252e-06_rb,0.22439e-05_rb,0.29152e-05_rb,0.32163e-05_rb,0.19622e-05_rb /)
      kbo(:, 5,59, 7) = (/ &
        & 0.15291e-06_rb,0.28396e-05_rb,0.38087e-05_rb,0.43138e-05_rb,0.26387e-05_rb /)
      kbo(:, 1,13, 8) = (/ &
        & 0.83673e-02_rb,0.17454e-01_rb,0.20213e-01_rb,0.20132e-01_rb,0.13810e-01_rb /)
      kbo(:, 2,13, 8) = (/ &
        & 0.85039e-02_rb,0.17824e-01_rb,0.20684e-01_rb,0.20616e-01_rb,0.14219e-01_rb /)
      kbo(:, 3,13, 8) = (/ &
        & 0.86129e-02_rb,0.18196e-01_rb,0.21163e-01_rb,0.21143e-01_rb,0.14616e-01_rb /)
      kbo(:, 4,13, 8) = (/ &
        & 0.86913e-02_rb,0.18563e-01_rb,0.21635e-01_rb,0.21674e-01_rb,0.14991e-01_rb /)
      kbo(:, 5,13, 8) = (/ &
        & 0.87301e-02_rb,0.18916e-01_rb,0.22103e-01_rb,0.22198e-01_rb,0.15351e-01_rb /)
      kbo(:, 1,14, 8) = (/ &
        & 0.69346e-02_rb,0.14952e-01_rb,0.17380e-01_rb,0.17342e-01_rb,0.11803e-01_rb /)
      kbo(:, 2,14, 8) = (/ &
        & 0.70328e-02_rb,0.15276e-01_rb,0.17808e-01_rb,0.17830e-01_rb,0.12181e-01_rb /)
      kbo(:, 3,14, 8) = (/ &
        & 0.71113e-02_rb,0.15604e-01_rb,0.18240e-01_rb,0.18321e-01_rb,0.12544e-01_rb /)
      kbo(:, 4,14, 8) = (/ &
        & 0.71580e-02_rb,0.15937e-01_rb,0.18667e-01_rb,0.18814e-01_rb,0.12884e-01_rb /)
      kbo(:, 5,14, 8) = (/ &
        & 0.71785e-02_rb,0.16228e-01_rb,0.19093e-01_rb,0.19299e-01_rb,0.13230e-01_rb /)
      kbo(:, 1,15, 8) = (/ &
        & 0.57307e-02_rb,0.12763e-01_rb,0.14890e-01_rb,0.14912e-01_rb,0.10073e-01_rb /)
      kbo(:, 2,15, 8) = (/ &
        & 0.58017e-02_rb,0.13052e-01_rb,0.15278e-01_rb,0.15370e-01_rb,0.10417e-01_rb /)
      kbo(:, 3,15, 8) = (/ &
        & 0.58533e-02_rb,0.13347e-01_rb,0.15666e-01_rb,0.15828e-01_rb,0.10742e-01_rb /)
      kbo(:, 4,15, 8) = (/ &
        & 0.58826e-02_rb,0.13628e-01_rb,0.16057e-01_rb,0.16289e-01_rb,0.11066e-01_rb /)
      kbo(:, 5,15, 8) = (/ &
        & 0.58933e-02_rb,0.13873e-01_rb,0.16444e-01_rb,0.16739e-01_rb,0.11388e-01_rb /)
      kbo(:, 1,16, 8) = (/ &
        & 0.47247e-02_rb,0.10863e-01_rb,0.12723e-01_rb,0.12788e-01_rb,0.85822e-02_rb /)
      kbo(:, 2,16, 8) = (/ &
        & 0.47755e-02_rb,0.11124e-01_rb,0.13076e-01_rb,0.13211e-01_rb,0.88907e-02_rb /)
      kbo(:, 3,16, 8) = (/ &
        & 0.48095e-02_rb,0.11385e-01_rb,0.13431e-01_rb,0.13641e-01_rb,0.91903e-02_rb /)
      kbo(:, 4,16, 8) = (/ &
        & 0.48283e-02_rb,0.11622e-01_rb,0.13790e-01_rb,0.14066e-01_rb,0.94912e-02_rb /)
      kbo(:, 5,16, 8) = (/ &
        & 0.48333e-02_rb,0.11841e-01_rb,0.14135e-01_rb,0.14487e-01_rb,0.97938e-02_rb /)
      kbo(:, 1,17, 8) = (/ &
        & 0.38881e-02_rb,0.92313e-02_rb,0.10848e-01_rb,0.10942e-01_rb,0.73006e-02_rb /)
      kbo(:, 2,17, 8) = (/ &
        & 0.39238e-02_rb,0.94601e-02_rb,0.11171e-01_rb,0.11333e-01_rb,0.75792e-02_rb /)
      kbo(:, 3,17, 8) = (/ &
        & 0.39468e-02_rb,0.96874e-02_rb,0.11500e-01_rb,0.11732e-01_rb,0.78560e-02_rb /)
      kbo(:, 4,17, 8) = (/ &
        & 0.39590e-02_rb,0.98962e-02_rb,0.11826e-01_rb,0.12129e-01_rb,0.81376e-02_rb /)
      kbo(:, 5,17, 8) = (/ &
        & 0.39612e-02_rb,0.10094e-01_rb,0.12144e-01_rb,0.12521e-01_rb,0.84293e-02_rb /)
      kbo(:, 1,18, 8) = (/ &
        & 0.31953e-02_rb,0.78309e-02_rb,0.92361e-02_rb,0.93523e-02_rb,0.62048e-02_rb /)
      kbo(:, 2,18, 8) = (/ &
        & 0.32208e-02_rb,0.80372e-02_rb,0.95350e-02_rb,0.97148e-02_rb,0.64590e-02_rb /)
      kbo(:, 3,18, 8) = (/ &
        & 0.32365e-02_rb,0.82342e-02_rb,0.98382e-02_rb,0.10082e-01_rb,0.67154e-02_rb /)
      kbo(:, 4,18, 8) = (/ &
        & 0.32447e-02_rb,0.84229e-02_rb,0.10138e-01_rb,0.10450e-01_rb,0.69838e-02_rb /)
      kbo(:, 5,18, 8) = (/ &
        & 0.32450e-02_rb,0.86061e-02_rb,0.10427e-01_rb,0.10814e-01_rb,0.72626e-02_rb /)
      kbo(:, 1,19, 8) = (/ &
        & 0.26235e-02_rb,0.66399e-02_rb,0.78599e-02_rb,0.79875e-02_rb,0.52728e-02_rb /)
      kbo(:, 2,19, 8) = (/ &
        & 0.26414e-02_rb,0.68231e-02_rb,0.81351e-02_rb,0.83230e-02_rb,0.55070e-02_rb /)
      kbo(:, 3,19, 8) = (/ &
        & 0.26530e-02_rb,0.70004e-02_rb,0.84138e-02_rb,0.86629e-02_rb,0.57468e-02_rb /)
      kbo(:, 4,19, 8) = (/ &
        & 0.26583e-02_rb,0.71756e-02_rb,0.86887e-02_rb,0.90029e-02_rb,0.60006e-02_rb /)
      kbo(:, 5,19, 8) = (/ &
        & 0.26573e-02_rb,0.73457e-02_rb,0.89549e-02_rb,0.93422e-02_rb,0.62658e-02_rb /)
      kbo(:, 1,20, 8) = (/ &
        & 0.21528e-02_rb,0.56300e-02_rb,0.66928e-02_rb,0.68275e-02_rb,0.44887e-02_rb /)
      kbo(:, 2,20, 8) = (/ &
        & 0.21659e-02_rb,0.57965e-02_rb,0.69483e-02_rb,0.71374e-02_rb,0.47044e-02_rb /)
      kbo(:, 3,20, 8) = (/ &
        & 0.21742e-02_rb,0.59617e-02_rb,0.72036e-02_rb,0.74503e-02_rb,0.49314e-02_rb /)
      kbo(:, 4,20, 8) = (/ &
        & 0.21777e-02_rb,0.61251e-02_rb,0.74562e-02_rb,0.77657e-02_rb,0.51700e-02_rb /)
      kbo(:, 5,20, 8) = (/ &
        & 0.21757e-02_rb,0.62850e-02_rb,0.77065e-02_rb,0.80891e-02_rb,0.54225e-02_rb /)
      kbo(:, 1,21, 8) = (/ &
        & 0.17659e-02_rb,0.47778e-02_rb,0.57054e-02_rb,0.58417e-02_rb,0.38271e-02_rb /)
      kbo(:, 2,21, 8) = (/ &
        & 0.17754e-02_rb,0.49299e-02_rb,0.59393e-02_rb,0.61257e-02_rb,0.40264e-02_rb /)
      kbo(:, 3,21, 8) = (/ &
        & 0.17817e-02_rb,0.50836e-02_rb,0.61750e-02_rb,0.64148e-02_rb,0.42397e-02_rb /)
      kbo(:, 4,21, 8) = (/ &
        & 0.17836e-02_rb,0.52370e-02_rb,0.64083e-02_rb,0.67125e-02_rb,0.44647e-02_rb /)
      kbo(:, 5,21, 8) = (/ &
        & 0.17811e-02_rb,0.53876e-02_rb,0.66480e-02_rb,0.70201e-02_rb,0.47076e-02_rb /)
      kbo(:, 1,22, 8) = (/ &
        & 0.14486e-02_rb,0.40679e-02_rb,0.48826e-02_rb,0.50205e-02_rb,0.32805e-02_rb /)
      kbo(:, 2,22, 8) = (/ &
        & 0.14554e-02_rb,0.42085e-02_rb,0.50980e-02_rb,0.52827e-02_rb,0.34669e-02_rb /)
      kbo(:, 3,22, 8) = (/ &
        & 0.14597e-02_rb,0.43515e-02_rb,0.53153e-02_rb,0.55535e-02_rb,0.36674e-02_rb /)
      kbo(:, 4,22, 8) = (/ &
        & 0.14606e-02_rb,0.44957e-02_rb,0.55353e-02_rb,0.58376e-02_rb,0.38816e-02_rb /)
      kbo(:, 5,22, 8) = (/ &
        & 0.14577e-02_rb,0.46383e-02_rb,0.57672e-02_rb,0.61302e-02_rb,0.41156e-02_rb /)
      kbo(:, 1,23, 8) = (/ &
        & 0.11880e-02_rb,0.34698e-02_rb,0.41861e-02_rb,0.43248e-02_rb,0.28199e-02_rb /)
      kbo(:, 2,23, 8) = (/ &
        & 0.11929e-02_rb,0.36010e-02_rb,0.43862e-02_rb,0.45689e-02_rb,0.29956e-02_rb /)
      kbo(:, 3,23, 8) = (/ &
        & 0.11958e-02_rb,0.37344e-02_rb,0.45877e-02_rb,0.48273e-02_rb,0.31843e-02_rb /)
      kbo(:, 4,23, 8) = (/ &
        & 0.11959e-02_rb,0.38697e-02_rb,0.48007e-02_rb,0.50965e-02_rb,0.33903e-02_rb /)
      kbo(:, 5,23, 8) = (/ &
        & 0.11929e-02_rb,0.40068e-02_rb,0.50232e-02_rb,0.53791e-02_rb,0.36136e-02_rb /)
      kbo(:, 1,24, 8) = (/ &
        & 0.97406e-03_rb,0.29675e-02_rb,0.36006e-02_rb,0.37379e-02_rb,0.24344e-02_rb /)
      kbo(:, 2,24, 8) = (/ &
        & 0.97756e-03_rb,0.30896e-02_rb,0.37850e-02_rb,0.39686e-02_rb,0.25994e-02_rb /)
      kbo(:, 3,24, 8) = (/ &
        & 0.97946e-03_rb,0.32150e-02_rb,0.39775e-02_rb,0.42135e-02_rb,0.27776e-02_rb /)
      kbo(:, 4,24, 8) = (/ &
        & 0.97899e-03_rb,0.33430e-02_rb,0.41825e-02_rb,0.44708e-02_rb,0.29753e-02_rb /)
      kbo(:, 5,24, 8) = (/ &
        & 0.97615e-03_rb,0.34761e-02_rb,0.43966e-02_rb,0.47457e-02_rb,0.31900e-02_rb /)
      kbo(:, 1,25, 8) = (/ &
        & 0.79856e-03_rb,0.25459e-02_rb,0.31077e-02_rb,0.32447e-02_rb,0.21114e-02_rb /)
      kbo(:, 2,25, 8) = (/ &
        & 0.80110e-03_rb,0.26605e-02_rb,0.32805e-02_rb,0.34640e-02_rb,0.22664e-02_rb /)
      kbo(:, 3,25, 8) = (/ &
        & 0.80222e-03_rb,0.27778e-02_rb,0.34666e-02_rb,0.36958e-02_rb,0.24376e-02_rb /)
      kbo(:, 4,25, 8) = (/ &
        & 0.80129e-03_rb,0.29014e-02_rb,0.36623e-02_rb,0.39444e-02_rb,0.26269e-02_rb /)
      kbo(:, 5,25, 8) = (/ &
        & 0.79857e-03_rb,0.30302e-02_rb,0.38698e-02_rb,0.42126e-02_rb,0.28335e-02_rb /)
      kbo(:, 1,26, 8) = (/ &
        & 0.65460e-03_rb,0.21933e-02_rb,0.26944e-02_rb,0.28334e-02_rb,0.18417e-02_rb /)
      kbo(:, 2,26, 8) = (/ &
        & 0.65639e-03_rb,0.23003e-02_rb,0.28599e-02_rb,0.30409e-02_rb,0.19886e-02_rb /)
      kbo(:, 3,26, 8) = (/ &
        & 0.65685e-03_rb,0.24132e-02_rb,0.30384e-02_rb,0.32639e-02_rb,0.21545e-02_rb /)
      kbo(:, 4,26, 8) = (/ &
        & 0.65577e-03_rb,0.25323e-02_rb,0.32275e-02_rb,0.35055e-02_rb,0.23366e-02_rb /)
      kbo(:, 5,26, 8) = (/ &
        & 0.65325e-03_rb,0.26572e-02_rb,0.34294e-02_rb,0.37687e-02_rb,0.25365e-02_rb /)
      kbo(:, 1,27, 8) = (/ &
        & 0.53655e-03_rb,0.18966e-02_rb,0.23474e-02_rb,0.24873e-02_rb,0.16142e-02_rb /)
      kbo(:, 2,27, 8) = (/ &
        & 0.53777e-03_rb,0.19979e-02_rb,0.25069e-02_rb,0.26849e-02_rb,0.17570e-02_rb /)
      kbo(:, 3,27, 8) = (/ &
        & 0.53783e-03_rb,0.21070e-02_rb,0.26778e-02_rb,0.29010e-02_rb,0.19163e-02_rb /)
      kbo(:, 4,27, 8) = (/ &
        & 0.53661e-03_rb,0.22218e-02_rb,0.28609e-02_rb,0.31373e-02_rb,0.20925e-02_rb /)
      kbo(:, 5,27, 8) = (/ &
        & 0.53426e-03_rb,0.23437e-02_rb,0.30591e-02_rb,0.33953e-02_rb,0.22871e-02_rb /)
      kbo(:, 1,28, 8) = (/ &
        & 0.43973e-03_rb,0.16467e-02_rb,0.20564e-02_rb,0.21950e-02_rb,0.14243e-02_rb /)
      kbo(:, 2,28, 8) = (/ &
        & 0.44046e-03_rb,0.17445e-02_rb,0.22094e-02_rb,0.23854e-02_rb,0.15622e-02_rb /)
      kbo(:, 3,28, 8) = (/ &
        & 0.44027e-03_rb,0.18495e-02_rb,0.23737e-02_rb,0.25954e-02_rb,0.17160e-02_rb /)
      kbo(:, 4,28, 8) = (/ &
        & 0.43902e-03_rb,0.19607e-02_rb,0.25527e-02_rb,0.28280e-02_rb,0.18875e-02_rb /)
      kbo(:, 5,28, 8) = (/ &
        & 0.43690e-03_rb,0.20797e-02_rb,0.27483e-02_rb,0.30809e-02_rb,0.20783e-02_rb /)
      kbo(:, 1,29, 8) = (/ &
        & 0.36035e-03_rb,0.14377e-02_rb,0.18129e-02_rb,0.19502e-02_rb,0.12665e-02_rb /)
      kbo(:, 2,29, 8) = (/ &
        & 0.36078e-03_rb,0.15322e-02_rb,0.19593e-02_rb,0.21346e-02_rb,0.13993e-02_rb /)
      kbo(:, 3,29, 8) = (/ &
        & 0.36035e-03_rb,0.16333e-02_rb,0.21194e-02_rb,0.23407e-02_rb,0.15489e-02_rb /)
      kbo(:, 4,29, 8) = (/ &
        & 0.35917e-03_rb,0.17416e-02_rb,0.22958e-02_rb,0.25693e-02_rb,0.17169e-02_rb /)
      kbo(:, 5,29, 8) = (/ &
        & 0.35722e-03_rb,0.18583e-02_rb,0.24893e-02_rb,0.28199e-02_rb,0.19058e-02_rb /)
      kbo(:, 1,30, 8) = (/ &
        & 0.29522e-03_rb,0.12625e-02_rb,0.16073e-02_rb,0.17444e-02_rb,0.11337e-02_rb /)
      kbo(:, 2,30, 8) = (/ &
        & 0.29544e-03_rb,0.13534e-02_rb,0.17493e-02_rb,0.19248e-02_rb,0.12629e-02_rb /)
      kbo(:, 3,30, 8) = (/ &
        & 0.29489e-03_rb,0.14513e-02_rb,0.19064e-02_rb,0.21281e-02_rb,0.14093e-02_rb /)
      kbo(:, 4,30, 8) = (/ &
        & 0.29377e-03_rb,0.15575e-02_rb,0.20813e-02_rb,0.23536e-02_rb,0.15751e-02_rb /)
      kbo(:, 5,30, 8) = (/ &
        & 0.29201e-03_rb,0.16725e-02_rb,0.22728e-02_rb,0.26031e-02_rb,0.17633e-02_rb /)
      kbo(:, 1,31, 8) = (/ &
        & 0.24186e-03_rb,0.11155e-02_rb,0.14353e-02_rb,0.15729e-02_rb,0.10232e-02_rb /)
      kbo(:, 2,31, 8) = (/ &
        & 0.24189e-03_rb,0.12032e-02_rb,0.15739e-02_rb,0.17508e-02_rb,0.11498e-02_rb /)
      kbo(:, 3,31, 8) = (/ &
        & 0.24130e-03_rb,0.12988e-02_rb,0.17296e-02_rb,0.19516e-02_rb,0.12942e-02_rb /)
      kbo(:, 4,31, 8) = (/ &
        & 0.24025e-03_rb,0.14034e-02_rb,0.19030e-02_rb,0.21764e-02_rb,0.14594e-02_rb /)
      kbo(:, 5,31, 8) = (/ &
        & 0.23866e-03_rb,0.15170e-02_rb,0.20940e-02_rb,0.24270e-02_rb,0.16471e-02_rb /)
      kbo(:, 1,32, 8) = (/ &
        & 0.19811e-03_rb,0.99188e-03_rb,0.12914e-02_rb,0.14311e-02_rb,0.93157e-03_rb /)
      kbo(:, 2,32, 8) = (/ &
        & 0.19799e-03_rb,0.10771e-02_rb,0.14282e-02_rb,0.16073e-02_rb,0.10562e-02_rb /)
      kbo(:, 3,32, 8) = (/ &
        & 0.19741e-03_rb,0.11711e-02_rb,0.15828e-02_rb,0.18068e-02_rb,0.11999e-02_rb /)
      kbo(:, 4,32, 8) = (/ &
        & 0.19645e-03_rb,0.12746e-02_rb,0.17559e-02_rb,0.20320e-02_rb,0.13654e-02_rb /)
      kbo(:, 5,32, 8) = (/ &
        & 0.19503e-03_rb,0.13871e-02_rb,0.19473e-02_rb,0.22851e-02_rb,0.15533e-02_rb /)
      kbo(:, 1,33, 8) = (/ &
        & 0.16225e-03_rb,0.88803e-03_rb,0.11717e-02_rb,0.13138e-02_rb,0.85556e-03_rb /)
      kbo(:, 2,33, 8) = (/ &
        & 0.16205e-03_rb,0.97174e-03_rb,0.13077e-02_rb,0.14890e-02_rb,0.97930e-03_rb /)
      kbo(:, 3,33, 8) = (/ &
        & 0.16148e-03_rb,0.10648e-02_rb,0.14621e-02_rb,0.16891e-02_rb,0.11237e-02_rb /)
      kbo(:, 4,33, 8) = (/ &
        & 0.16059e-03_rb,0.11673e-02_rb,0.16357e-02_rb,0.19165e-02_rb,0.12896e-02_rb /)
      kbo(:, 5,33, 8) = (/ &
        & 0.15934e-03_rb,0.12795e-02_rb,0.18288e-02_rb,0.21736e-02_rb,0.14791e-02_rb /)
      kbo(:, 1,34, 8) = (/ &
        & 0.13286e-03_rb,0.79835e-03_rb,0.10683e-02_rb,0.12117e-02_rb,0.78908e-03_rb /)
      kbo(:, 2,34, 8) = (/ &
        & 0.13261e-03_rb,0.88066e-03_rb,0.12036e-02_rb,0.13865e-02_rb,0.91252e-03_rb /)
      kbo(:, 3,34, 8) = (/ &
        & 0.13209e-03_rb,0.97270e-03_rb,0.13580e-02_rb,0.15871e-02_rb,0.10570e-02_rb /)
      kbo(:, 4,34, 8) = (/ &
        & 0.13130e-03_rb,0.10748e-02_rb,0.15318e-02_rb,0.18171e-02_rb,0.12236e-02_rb /)
      kbo(:, 5,34, 8) = (/ &
        & 0.13022e-03_rb,0.11866e-02_rb,0.17264e-02_rb,0.20786e-02_rb,0.14148e-02_rb /)
      kbo(:, 1,35, 8) = (/ &
        & 0.10879e-03_rb,0.71567e-03_rb,0.97075e-03_rb,0.11118e-02_rb,0.72353e-03_rb /)
      kbo(:, 2,35, 8) = (/ &
        & 0.10855e-03_rb,0.79625e-03_rb,0.11041e-02_rb,0.12845e-02_rb,0.84540e-03_rb /)
      kbo(:, 3,35, 8) = (/ &
        & 0.10811e-03_rb,0.88686e-03_rb,0.12569e-02_rb,0.14847e-02_rb,0.98831e-03_rb /)
      kbo(:, 4,35, 8) = (/ &
        & 0.10743e-03_rb,0.98761e-03_rb,0.14300e-02_rb,0.17157e-02_rb,0.11541e-02_rb /)
      kbo(:, 5,35, 8) = (/ &
        & 0.10650e-03_rb,0.10987e-02_rb,0.16251e-02_rb,0.19799e-02_rb,0.13460e-02_rb /)
      kbo(:, 1,36, 8) = (/ &
        & 0.89091e-04_rb,0.63773e-03_rb,0.87525e-03_rb,0.10106e-02_rb,0.65643e-03_rb /)
      kbo(:, 2,36, 8) = (/ &
        & 0.88892e-04_rb,0.71572e-03_rb,0.10050e-02_rb,0.11794e-02_rb,0.77482e-03_rb /)
      kbo(:, 3,36, 8) = (/ &
        & 0.88510e-04_rb,0.80390e-03_rb,0.11544e-02_rb,0.13768e-02_rb,0.91437e-03_rb /)
      kbo(:, 4,36, 8) = (/ &
        & 0.87951e-04_rb,0.90253e-03_rb,0.13255e-02_rb,0.16063e-02_rb,0.10779e-02_rb /)
      kbo(:, 5,36, 8) = (/ &
        & 0.87175e-04_rb,0.10120e-02_rb,0.15187e-02_rb,0.18703e-02_rb,0.12677e-02_rb /)
      kbo(:, 1,37, 8) = (/ &
        & 0.72967e-04_rb,0.56051e-03_rb,0.77586e-03_rb,0.90139e-03_rb,0.58319e-03_rb /)
      kbo(:, 2,37, 8) = (/ &
        & 0.72818e-04_rb,0.63459e-03_rb,0.89962e-03_rb,0.10627e-02_rb,0.69545e-03_rb /)
      kbo(:, 3,37, 8) = (/ &
        & 0.72523e-04_rb,0.71909e-03_rb,0.10433e-02_rb,0.12533e-02_rb,0.82896e-03_rb /)
      kbo(:, 4,37, 8) = (/ &
        & 0.72087e-04_rb,0.81427e-03_rb,0.12092e-02_rb,0.14766e-02_rb,0.98671e-03_rb /)
      kbo(:, 5,37, 8) = (/ &
        & 0.71486e-04_rb,0.92093e-03_rb,0.13977e-02_rb,0.17355e-02_rb,0.11708e-02_rb /)
      kbo(:, 1,38, 8) = (/ &
        & 0.59765e-04_rb,0.49320e-03_rb,0.68844e-03_rb,0.80478e-03_rb,0.51849e-03_rb /)
      kbo(:, 2,38, 8) = (/ &
        & 0.59660e-04_rb,0.56354e-03_rb,0.80640e-03_rb,0.95876e-03_rb,0.62466e-03_rb /)
      kbo(:, 3,38, 8) = (/ &
        & 0.59436e-04_rb,0.64440e-03_rb,0.94454e-03_rb,0.11426e-02_rb,0.75236e-03_rb /)
      kbo(:, 4,38, 8) = (/ &
        & 0.59102e-04_rb,0.73631e-03_rb,0.11052e-02_rb,0.13599e-02_rb,0.90433e-03_rb /)
      kbo(:, 5,38, 8) = (/ &
        & 0.58635e-04_rb,0.84009e-03_rb,0.12891e-02_rb,0.16134e-02_rb,0.10829e-02_rb /)
      kbo(:, 1,39, 8) = (/ &
        & 0.48956e-04_rb,0.43499e-03_rb,0.61231e-03_rb,0.72017e-03_rb,0.46189e-03_rb /)
      kbo(:, 2,39, 8) = (/ &
        & 0.48890e-04_rb,0.50181e-03_rb,0.72478e-03_rb,0.86730e-03_rb,0.56244e-03_rb /)
      kbo(:, 3,39, 8) = (/ &
        & 0.48717e-04_rb,0.57923e-03_rb,0.85778e-03_rb,0.10448e-02_rb,0.68464e-03_rb /)
      kbo(:, 4,39, 8) = (/ &
        & 0.48451e-04_rb,0.66799e-03_rb,0.10132e-02_rb,0.12563e-02_rb,0.83112e-03_rb /)
      kbo(:, 5,39, 8) = (/ &
        & 0.48085e-04_rb,0.76898e-03_rb,0.11928e-02_rb,0.15047e-02_rb,0.10044e-02_rb /)
      kbo(:, 1,40, 8) = (/ &
        & 0.40107e-04_rb,0.37860e-03_rb,0.53596e-03_rb,0.63302e-03_rb,0.40350e-03_rb /)
      kbo(:, 2,40, 8) = (/ &
        & 0.40068e-04_rb,0.44099e-03_rb,0.64126e-03_rb,0.77105e-03_rb,0.49697e-03_rb /)
      kbo(:, 3,40, 8) = (/ &
        & 0.39945e-04_rb,0.51399e-03_rb,0.76694e-03_rb,0.93934e-03_rb,0.61166e-03_rb /)
      kbo(:, 4,40, 8) = (/ &
        & 0.39752e-04_rb,0.59851e-03_rb,0.91528e-03_rb,0.11416e-02_rb,0.75037e-03_rb /)
      kbo(:, 5,40, 8) = (/ &
        & 0.39469e-04_rb,0.69544e-03_rb,0.10880e-02_rb,0.13809e-02_rb,0.91593e-03_rb /)
      kbo(:, 1,41, 8) = (/ &
        & 0.32851e-04_rb,0.32878e-03_rb,0.46786e-03_rb,0.55469e-03_rb,0.35120e-03_rb /)
      kbo(:, 2,41, 8) = (/ &
        & 0.32839e-04_rb,0.38683e-03_rb,0.56607e-03_rb,0.68394e-03_rb,0.43769e-03_rb /)
      kbo(:, 3,41, 8) = (/ &
        & 0.32749e-04_rb,0.45542e-03_rb,0.68444e-03_rb,0.84259e-03_rb,0.54490e-03_rb /)
      kbo(:, 4,41, 8) = (/ &
        & 0.32607e-04_rb,0.53564e-03_rb,0.82547e-03_rb,0.10352e-02_rb,0.67585e-03_rb /)
      kbo(:, 5,41, 8) = (/ &
        & 0.32395e-04_rb,0.62841e-03_rb,0.99133e-03_rb,0.12652e-02_rb,0.83332e-03_rb /)
      kbo(:, 1,42, 8) = (/ &
        & 0.26906e-04_rb,0.28534e-03_rb,0.40805e-03_rb,0.48535e-03_rb,0.30526e-03_rb /)
      kbo(:, 2,42, 8) = (/ &
        & 0.26908e-04_rb,0.33922e-03_rb,0.49950e-03_rb,0.60603e-03_rb,0.38505e-03_rb /)
      kbo(:, 3,42, 8) = (/ &
        & 0.26846e-04_rb,0.40361e-03_rb,0.61076e-03_rb,0.75564e-03_rb,0.48524e-03_rb /)
      kbo(:, 4,42, 8) = (/ &
        & 0.26739e-04_rb,0.47953e-03_rb,0.74454e-03_rb,0.93882e-03_rb,0.60861e-03_rb /)
      kbo(:, 5,42, 8) = (/ &
        & 0.26585e-04_rb,0.56816e-03_rb,0.90363e-03_rb,0.11597e-02_rb,0.75829e-03_rb /)
      kbo(:, 1,43, 8) = (/ &
        & 0.22034e-04_rb,0.24494e-03_rb,0.35126e-03_rb,0.41862e-03_rb,0.26133e-03_rb /)
      kbo(:, 2,43, 8) = (/ &
        & 0.22045e-04_rb,0.29428e-03_rb,0.43526e-03_rb,0.52960e-03_rb,0.33379e-03_rb /)
      kbo(:, 3,43, 8) = (/ &
        & 0.22009e-04_rb,0.35394e-03_rb,0.53838e-03_rb,0.66893e-03_rb,0.42599e-03_rb /)
      kbo(:, 4,43, 8) = (/ &
        & 0.21933e-04_rb,0.42510e-03_rb,0.66392e-03_rb,0.84090e-03_rb,0.54061e-03_rb /)
      kbo(:, 5,43, 8) = (/ &
        & 0.21820e-04_rb,0.50878e-03_rb,0.81474e-03_rb,0.10503e-02_rb,0.68139e-03_rb /)
      kbo(:, 1,44, 8) = (/ &
        & 0.18042e-04_rb,0.20878e-03_rb,0.29988e-03_rb,0.35782e-03_rb,0.22160e-03_rb /)
      kbo(:, 2,44, 8) = (/ &
        & 0.18058e-04_rb,0.25360e-03_rb,0.37621e-03_rb,0.45875e-03_rb,0.28670e-03_rb /)
      kbo(:, 3,44, 8) = (/ &
        & 0.18041e-04_rb,0.30845e-03_rb,0.47123e-03_rb,0.58727e-03_rb,0.37070e-03_rb /)
      kbo(:, 4,44, 8) = (/ &
        & 0.17989e-04_rb,0.37459e-03_rb,0.58804e-03_rb,0.74739e-03_rb,0.47650e-03_rb /)
      kbo(:, 5,44, 8) = (/ &
        & 0.17906e-04_rb,0.45317e-03_rb,0.73003e-03_rb,0.94454e-03_rb,0.60768e-03_rb /)
      kbo(:, 1,45, 8) = (/ &
        & 0.14770e-04_rb,0.17743e-03_rb,0.25518e-03_rb,0.30461e-03_rb,0.18718e-03_rb /)
      kbo(:, 2,45, 8) = (/ &
        & 0.14790e-04_rb,0.21794e-03_rb,0.32424e-03_rb,0.39603e-03_rb,0.24534e-03_rb /)
      kbo(:, 3,45, 8) = (/ &
        & 0.14785e-04_rb,0.26822e-03_rb,0.41132e-03_rb,0.51406e-03_rb,0.32158e-03_rb /)
      kbo(:, 4,45, 8) = (/ &
        & 0.14751e-04_rb,0.32948e-03_rb,0.51968e-03_rb,0.66288e-03_rb,0.41889e-03_rb /)
      kbo(:, 5,45, 8) = (/ &
        & 0.14692e-04_rb,0.40309e-03_rb,0.65298e-03_rb,0.84795e-03_rb,0.54078e-03_rb /)
      kbo(:, 1,46, 8) = (/ &
        & 0.12091e-04_rb,0.14956e-03_rb,0.21507e-03_rb,0.25664e-03_rb,0.15641e-03_rb /)
      kbo(:, 2,46, 8) = (/ &
        & 0.12112e-04_rb,0.18585e-03_rb,0.27689e-03_rb,0.33853e-03_rb,0.20778e-03_rb /)
      kbo(:, 3,46, 8) = (/ &
        & 0.12113e-04_rb,0.23156e-03_rb,0.35597e-03_rb,0.44575e-03_rb,0.27619e-03_rb /)
      kbo(:, 4,46, 8) = (/ &
        & 0.12094e-04_rb,0.28789e-03_rb,0.45576e-03_rb,0.58313e-03_rb,0.36492e-03_rb /)
      kbo(:, 5,46, 8) = (/ &
        & 0.12054e-04_rb,0.35628e-03_rb,0.57980e-03_rb,0.75523e-03_rb,0.47721e-03_rb /)
      kbo(:, 1,47, 8) = (/ &
        & 0.98958e-05_rb,0.12436e-03_rb,0.17838e-03_rb,0.21241e-03_rb,0.12841e-03_rb /)
      kbo(:, 2,47, 8) = (/ &
        & 0.99157e-05_rb,0.15633e-03_rb,0.23279e-03_rb,0.28443e-03_rb,0.17291e-03_rb /)
      kbo(:, 3,47, 8) = (/ &
        & 0.99224e-05_rb,0.19724e-03_rb,0.30350e-03_rb,0.38017e-03_rb,0.23323e-03_rb /)
      kbo(:, 4,47, 8) = (/ &
        & 0.99150e-05_rb,0.24831e-03_rb,0.39396e-03_rb,0.50488e-03_rb,0.31274e-03_rb /)
      kbo(:, 5,47, 8) = (/ &
        & 0.98881e-05_rb,0.31110e-03_rb,0.50788e-03_rb,0.66304e-03_rb,0.41485e-03_rb /)
      kbo(:, 1,48, 8) = (/ &
        & 0.80981e-05_rb,0.10284e-03_rb,0.14701e-03_rb,0.17457e-03_rb,0.10464e-03_rb /)
      kbo(:, 2,48, 8) = (/ &
        & 0.81170e-05_rb,0.13083e-03_rb,0.19452e-03_rb,0.23734e-03_rb,0.14298e-03_rb /)
      kbo(:, 3,48, 8) = (/ &
        & 0.81254e-05_rb,0.16716e-03_rb,0.25727e-03_rb,0.32230e-03_rb,0.19565e-03_rb /)
      kbo(:, 4,48, 8) = (/ &
        & 0.81253e-05_rb,0.21328e-03_rb,0.33887e-03_rb,0.43475e-03_rb,0.26652e-03_rb /)
      kbo(:, 5,48, 8) = (/ &
        & 0.81092e-05_rb,0.27062e-03_rb,0.44301e-03_rb,0.57972e-03_rb,0.35887e-03_rb /)
      kbo(:, 1,49, 8) = (/ &
        & 0.66262e-05_rb,0.84542e-04_rb,0.12034e-03_rb,0.14245e-03_rb,0.84636e-04_rb /)
      kbo(:, 2,49, 8) = (/ &
        & 0.66435e-05_rb,0.10903e-03_rb,0.16168e-03_rb,0.19686e-03_rb,0.11740e-03_rb /)
      kbo(:, 3,49, 8) = (/ &
        & 0.66527e-05_rb,0.14092e-03_rb,0.21685e-03_rb,0.27156e-03_rb,0.16309e-03_rb /)
      kbo(:, 4,49, 8) = (/ &
        & 0.66567e-05_rb,0.18229e-03_rb,0.28989e-03_rb,0.37214e-03_rb,0.22572e-03_rb /)
      kbo(:, 5,49, 8) = (/ &
        & 0.66489e-05_rb,0.23443e-03_rb,0.38461e-03_rb,0.50418e-03_rb,0.30883e-03_rb /)
      kbo(:, 1,50, 8) = (/ &
        & 0.54202e-05_rb,0.69608e-04_rb,0.98680e-04_rb,0.11646e-03_rb,0.68510e-04_rb /)
      kbo(:, 2,50, 8) = (/ &
        & 0.54374e-05_rb,0.90802e-04_rb,0.13434e-03_rb,0.16330e-03_rb,0.96499e-04_rb /)
      kbo(:, 3,50, 8) = (/ &
        & 0.54475e-05_rb,0.11916e-03_rb,0.18327e-03_rb,0.22932e-03_rb,0.13620e-03_rb /)
      kbo(:, 4,50, 8) = (/ &
        & 0.54521e-05_rb,0.15608e-03_rb,0.24845e-03_rb,0.31928e-03_rb,0.19154e-03_rb /)
      kbo(:, 5,50, 8) = (/ &
        & 0.54498e-05_rb,0.20348e-03_rb,0.33466e-03_rb,0.43937e-03_rb,0.26626e-03_rb /)
      kbo(:, 1,51, 8) = (/ &
        & 0.44351e-05_rb,0.57088e-04_rb,0.80582e-04_rb,0.94790e-04_rb,0.55250e-04_rb /)
      kbo(:, 2,51, 8) = (/ &
        & 0.44504e-05_rb,0.75435e-04_rb,0.11131e-03_rb,0.13503e-03_rb,0.79024e-04_rb /)
      kbo(:, 3,51, 8) = (/ &
        & 0.44599e-05_rb,0.10037e-03_rb,0.15428e-03_rb,0.19291e-03_rb,0.11343e-03_rb /)
      kbo(:, 4,51, 8) = (/ &
        & 0.44651e-05_rb,0.13336e-03_rb,0.21252e-03_rb,0.27335e-03_rb,0.16218e-03_rb /)
      kbo(:, 5,51, 8) = (/ &
        & 0.44653e-05_rb,0.17640e-03_rb,0.29075e-03_rb,0.38229e-03_rb,0.22923e-03_rb /)
      kbo(:, 1,52, 8) = (/ &
        & 0.36290e-05_rb,0.46565e-04_rb,0.65364e-04_rb,0.76598e-04_rb,0.44240e-04_rb /)
      kbo(:, 2,52, 8) = (/ &
        & 0.36420e-05_rb,0.62314e-04_rb,0.91644e-04_rb,0.11090e-03_rb,0.64234e-04_rb /)
      kbo(:, 3,52, 8) = (/ &
        & 0.36509e-05_rb,0.84102e-04_rb,0.12910e-03_rb,0.16125e-03_rb,0.93833e-04_rb /)
      kbo(:, 4,52, 8) = (/ &
        & 0.36555e-05_rb,0.11362e-03_rb,0.18115e-03_rb,0.23297e-03_rb,0.13657e-03_rb /)
      kbo(:, 5,52, 8) = (/ &
        & 0.36579e-05_rb,0.15226e-03_rb,0.25136e-03_rb,0.33099e-03_rb,0.19621e-03_rb /)
      kbo(:, 1,53, 8) = (/ &
        & 0.29692e-05_rb,0.37746e-04_rb,0.52632e-04_rb,0.61397e-04_rb,0.35165e-04_rb /)
      kbo(:, 2,53, 8) = (/ &
        & 0.29808e-05_rb,0.51149e-04_rb,0.74879e-04_rb,0.90335e-04_rb,0.51807e-04_rb /)
      kbo(:, 3,53, 8) = (/ &
        & 0.29880e-05_rb,0.70028e-04_rb,0.10724e-03_rb,0.13375e-03_rb,0.77010e-04_rb /)
      kbo(:, 4,53, 8) = (/ &
        & 0.29931e-05_rb,0.96105e-04_rb,0.15314e-03_rb,0.19689e-03_rb,0.11417e-03_rb /)
      kbo(:, 5,53, 8) = (/ &
        & 0.29959e-05_rb,0.13074e-03_rb,0.21612e-03_rb,0.28492e-03_rb,0.16697e-03_rb /)
      kbo(:, 1,54, 8) = (/ &
        & 0.24296e-05_rb,0.30698e-04_rb,0.42544e-04_rb,0.49406e-04_rb,0.28072e-04_rb /)
      kbo(:, 2,54, 8) = (/ &
        & 0.24393e-05_rb,0.42122e-04_rb,0.61454e-04_rb,0.73927e-04_rb,0.41964e-04_rb /)
      kbo(:, 3,54, 8) = (/ &
        & 0.24460e-05_rb,0.58528e-04_rb,0.89500e-04_rb,0.11146e-03_rb,0.63503e-04_rb /)
      kbo(:, 4,54, 8) = (/ &
        & 0.24507e-05_rb,0.81611e-04_rb,0.13009e-03_rb,0.16728e-03_rb,0.95947e-04_rb /)
      kbo(:, 5,54, 8) = (/ &
        & 0.24535e-05_rb,0.11276e-03_rb,0.18682e-03_rb,0.24654e-03_rb,0.14286e-03_rb /)
      kbo(:, 1,55, 8) = (/ &
        & 0.19883e-05_rb,0.24921e-04_rb,0.34312e-04_rb,0.39666e-04_rb,0.22375e-04_rb /)
      kbo(:, 2,55, 8) = (/ &
        & 0.19963e-05_rb,0.34641e-04_rb,0.50344e-04_rb,0.60381e-04_rb,0.33942e-04_rb /)
      kbo(:, 3,55, 8) = (/ &
        & 0.20020e-05_rb,0.48868e-04_rb,0.74618e-04_rb,0.92809e-04_rb,0.52280e-04_rb /)
      kbo(:, 4,55, 8) = (/ &
        & 0.20065e-05_rb,0.69255e-04_rb,0.11042e-03_rb,0.14203e-03_rb,0.80566e-04_rb /)
      kbo(:, 5,55, 8) = (/ &
        & 0.20089e-05_rb,0.97527e-04_rb,0.16182e-03_rb,0.21373e-03_rb,0.12228e-03_rb /)
      kbo(:, 1,56, 8) = (/ &
        & 0.16268e-05_rb,0.20122e-04_rb,0.27506e-04_rb,0.31629e-04_rb,0.17718e-04_rb /)
      kbo(:, 2,56, 8) = (/ &
        & 0.16337e-05_rb,0.28321e-04_rb,0.40966e-04_rb,0.48960e-04_rb,0.27271e-04_rb /)
      kbo(:, 3,56, 8) = (/ &
        & 0.16385e-05_rb,0.40579e-04_rb,0.61828e-04_rb,0.76747e-04_rb,0.42739e-04_rb /)
      kbo(:, 4,56, 8) = (/ &
        & 0.16424e-05_rb,0.58490e-04_rb,0.93220e-04_rb,0.11991e-03_rb,0.67232e-04_rb /)
      kbo(:, 5,56, 8) = (/ &
        & 0.16450e-05_rb,0.83776e-04_rb,0.13917e-03_rb,0.18401e-03_rb,0.10406e-03_rb /)
      kbo(:, 1,57, 8) = (/ &
        & 0.13311e-05_rb,0.16151e-04_rb,0.21901e-04_rb,0.25044e-04_rb,0.13934e-04_rb /)
      kbo(:, 2,57, 8) = (/ &
        & 0.13369e-05_rb,0.23005e-04_rb,0.33083e-04_rb,0.39377e-04_rb,0.21751e-04_rb /)
      kbo(:, 3,57, 8) = (/ &
        & 0.13411e-05_rb,0.33501e-04_rb,0.50888e-04_rb,0.63014e-04_rb,0.34696e-04_rb /)
      kbo(:, 4,57, 8) = (/ &
        & 0.13443e-05_rb,0.49111e-04_rb,0.78191e-04_rb,0.10051e-03_rb,0.55711e-04_rb /)
      kbo(:, 5,57, 8) = (/ &
        & 0.13467e-05_rb,0.71586e-04_rb,0.11902e-03_rb,0.15744e-03_rb,0.88025e-04_rb /)
      kbo(:, 1,58, 8) = (/ &
        & 0.10892e-05_rb,0.12962e-04_rb,0.17443e-04_rb,0.19825e-04_rb,0.10955e-04_rb /)
      kbo(:, 2,58, 8) = (/ &
        & 0.10942e-05_rb,0.18688e-04_rb,0.26716e-04_rb,0.31664e-04_rb,0.17348e-04_rb /)
      kbo(:, 3,58, 8) = (/ &
        & 0.10976e-05_rb,0.27650e-04_rb,0.41864e-04_rb,0.51716e-04_rb,0.28159e-04_rb /)
      kbo(:, 4,58, 8) = (/ &
        & 0.11004e-05_rb,0.41251e-04_rb,0.65630e-04_rb,0.84299e-04_rb,0.46156e-04_rb /)
      kbo(:, 5,58, 8) = (/ &
        & 0.11025e-05_rb,0.61261e-04_rb,0.10196e-03_rb,0.13496e-03_rb,0.74536e-04_rb /)
      kbo(:, 1,59, 8) = (/ &
        & 0.89197e-06_rb,0.11047e-04_rb,0.14899e-04_rb,0.16964e-04_rb,0.92999e-05_rb /)
      kbo(:, 2,59, 8) = (/ &
        & 0.89605e-06_rb,0.16215e-04_rb,0.23291e-04_rb,0.27686e-04_rb,0.15019e-04_rb /)
      kbo(:, 3,59, 8) = (/ &
        & 0.89875e-06_rb,0.24462e-04_rb,0.37267e-04_rb,0.46243e-04_rb,0.24905e-04_rb /)
      kbo(:, 4,59, 8) = (/ &
        & 0.90117e-06_rb,0.37175e-04_rb,0.59594e-04_rb,0.76969e-04_rb,0.41672e-04_rb /)
      kbo(:, 5,59, 8) = (/ &
        & 0.90310e-06_rb,0.56115e-04_rb,0.94187e-04_rb,0.12528e-03_rb,0.68497e-04_rb /)
      kbo(:, 1,13, 9) = (/ &
        & 0.56714e-01_rb,0.96228e-01_rb,0.10961e+00_rb,0.11078e+00_rb,0.82822e-01_rb /)
      kbo(:, 2,13, 9) = (/ &
        & 0.57828e-01_rb,0.97972e-01_rb,0.11123e+00_rb,0.11236e+00_rb,0.84782e-01_rb /)
      kbo(:, 3,13, 9) = (/ &
        & 0.59232e-01_rb,0.99998e-01_rb,0.11329e+00_rb,0.11400e+00_rb,0.86728e-01_rb /)
      kbo(:, 4,13, 9) = (/ &
        & 0.60790e-01_rb,0.10219e+00_rb,0.11568e+00_rb,0.11591e+00_rb,0.88651e-01_rb /)
      kbo(:, 5,13, 9) = (/ &
        & 0.62290e-01_rb,0.10448e+00_rb,0.11818e+00_rb,0.11808e+00_rb,0.90606e-01_rb /)
      kbo(:, 1,14, 9) = (/ &
        & 0.48757e-01_rb,0.85742e-01_rb,0.97473e-01_rb,0.98306e-01_rb,0.73073e-01_rb /)
      kbo(:, 2,14, 9) = (/ &
        & 0.49942e-01_rb,0.87692e-01_rb,0.99489e-01_rb,0.10004e+00_rb,0.75064e-01_rb /)
      kbo(:, 3,14, 9) = (/ &
        & 0.51318e-01_rb,0.89856e-01_rb,0.10188e+00_rb,0.10205e+00_rb,0.77018e-01_rb /)
      kbo(:, 4,14, 9) = (/ &
        & 0.52660e-01_rb,0.92153e-01_rb,0.10444e+00_rb,0.10434e+00_rb,0.78996e-01_rb /)
      kbo(:, 5,14, 9) = (/ &
        & 0.53901e-01_rb,0.94584e-01_rb,0.10712e+00_rb,0.10687e+00_rb,0.81028e-01_rb /)
      kbo(:, 1,15, 9) = (/ &
        & 0.41752e-01_rb,0.76198e-01_rb,0.86593e-01_rb,0.87037e-01_rb,0.64464e-01_rb /)
      kbo(:, 2,15, 9) = (/ &
        & 0.42945e-01_rb,0.78277e-01_rb,0.88919e-01_rb,0.89055e-01_rb,0.66440e-01_rb /)
      kbo(:, 3,15, 9) = (/ &
        & 0.44132e-01_rb,0.80544e-01_rb,0.91468e-01_rb,0.91397e-01_rb,0.68401e-01_rb /)
      kbo(:, 4,15, 9) = (/ &
        & 0.45260e-01_rb,0.82968e-01_rb,0.94189e-01_rb,0.93993e-01_rb,0.70410e-01_rb /)
      kbo(:, 5,15, 9) = (/ &
        & 0.46298e-01_rb,0.85502e-01_rb,0.97029e-01_rb,0.96802e-01_rb,0.72466e-01_rb /)
      kbo(:, 1,16, 9) = (/ &
        & 0.35640e-01_rb,0.67555e-01_rb,0.76928e-01_rb,0.77126e-01_rb,0.56847e-01_rb /)
      kbo(:, 2,16, 9) = (/ &
        & 0.36668e-01_rb,0.69730e-01_rb,0.79394e-01_rb,0.79411e-01_rb,0.58796e-01_rb /)
      kbo(:, 3,16, 9) = (/ &
        & 0.37665e-01_rb,0.72105e-01_rb,0.82070e-01_rb,0.81996e-01_rb,0.60763e-01_rb /)
      kbo(:, 4,16, 9) = (/ &
        & 0.38606e-01_rb,0.74641e-01_rb,0.84920e-01_rb,0.84810e-01_rb,0.62792e-01_rb /)
      kbo(:, 5,16, 9) = (/ &
        & 0.39473e-01_rb,0.77118e-01_rb,0.87904e-01_rb,0.87777e-01_rb,0.64884e-01_rb /)
      kbo(:, 1,17, 9) = (/ &
        & 0.30249e-01_rb,0.59768e-01_rb,0.68325e-01_rb,0.68492e-01_rb,0.50101e-01_rb /)
      kbo(:, 2,17, 9) = (/ &
        & 0.31103e-01_rb,0.62060e-01_rb,0.70903e-01_rb,0.70988e-01_rb,0.52022e-01_rb /)
      kbo(:, 3,17, 9) = (/ &
        & 0.31936e-01_rb,0.64534e-01_rb,0.73696e-01_rb,0.73735e-01_rb,0.54005e-01_rb /)
      kbo(:, 4,17, 9) = (/ &
        & 0.32730e-01_rb,0.67029e-01_rb,0.76655e-01_rb,0.76665e-01_rb,0.56059e-01_rb /)
      kbo(:, 5,17, 9) = (/ &
        & 0.33443e-01_rb,0.69449e-01_rb,0.79757e-01_rb,0.79742e-01_rb,0.58173e-01_rb /)
      kbo(:, 1,18, 9) = (/ &
        & 0.25545e-01_rb,0.52871e-01_rb,0.60740e-01_rb,0.60992e-01_rb,0.44172e-01_rb /)
      kbo(:, 2,18, 9) = (/ &
        & 0.26254e-01_rb,0.55233e-01_rb,0.63406e-01_rb,0.63637e-01_rb,0.46086e-01_rb /)
      kbo(:, 3,18, 9) = (/ &
        & 0.26956e-01_rb,0.57692e-01_rb,0.66286e-01_rb,0.66462e-01_rb,0.48089e-01_rb /)
      kbo(:, 4,18, 9) = (/ &
        & 0.27612e-01_rb,0.60136e-01_rb,0.69343e-01_rb,0.69465e-01_rb,0.50154e-01_rb /)
      kbo(:, 5,18, 9) = (/ &
        & 0.28200e-01_rb,0.62534e-01_rb,0.72542e-01_rb,0.72608e-01_rb,0.52275e-01_rb /)
      kbo(:, 1,19, 9) = (/ &
        & 0.21487e-01_rb,0.46786e-01_rb,0.54092e-01_rb,0.54451e-01_rb,0.38986e-01_rb /)
      kbo(:, 2,19, 9) = (/ &
        & 0.22086e-01_rb,0.49151e-01_rb,0.56837e-01_rb,0.57167e-01_rb,0.40916e-01_rb /)
      kbo(:, 3,19, 9) = (/ &
        & 0.22671e-01_rb,0.51565e-01_rb,0.59787e-01_rb,0.60059e-01_rb,0.42916e-01_rb /)
      kbo(:, 4,19, 9) = (/ &
        & 0.23212e-01_rb,0.53966e-01_rb,0.62913e-01_rb,0.63134e-01_rb,0.44993e-01_rb /)
      kbo(:, 5,19, 9) = (/ &
        & 0.23674e-01_rb,0.56335e-01_rb,0.66121e-01_rb,0.66362e-01_rb,0.47140e-01_rb /)
      kbo(:, 1,20, 9) = (/ &
        & 0.18039e-01_rb,0.41493e-01_rb,0.48348e-01_rb,0.48783e-01_rb,0.34522e-01_rb /)
      kbo(:, 2,20, 9) = (/ &
        & 0.18543e-01_rb,0.43801e-01_rb,0.51155e-01_rb,0.51541e-01_rb,0.36457e-01_rb /)
      kbo(:, 3,20, 9) = (/ &
        & 0.19027e-01_rb,0.46166e-01_rb,0.54164e-01_rb,0.54503e-01_rb,0.38468e-01_rb /)
      kbo(:, 4,20, 9) = (/ &
        & 0.19452e-01_rb,0.48538e-01_rb,0.57292e-01_rb,0.57654e-01_rb,0.40551e-01_rb /)
      kbo(:, 5,20, 9) = (/ &
        & 0.19805e-01_rb,0.50879e-01_rb,0.60490e-01_rb,0.60969e-01_rb,0.42726e-01_rb /)
      kbo(:, 1,21, 9) = (/ &
        & 0.15116e-01_rb,0.36867e-01_rb,0.43373e-01_rb,0.43860e-01_rb,0.30692e-01_rb /)
      kbo(:, 2,21, 9) = (/ &
        & 0.15537e-01_rb,0.39121e-01_rb,0.46234e-01_rb,0.46663e-01_rb,0.32616e-01_rb /)
      kbo(:, 3,21, 9) = (/ &
        & 0.15924e-01_rb,0.41438e-01_rb,0.49238e-01_rb,0.49692e-01_rb,0.34628e-01_rb /)
      kbo(:, 4,21, 9) = (/ &
        & 0.16258e-01_rb,0.43774e-01_rb,0.52361e-01_rb,0.52910e-01_rb,0.36736e-01_rb /)
      kbo(:, 5,21, 9) = (/ &
        & 0.16525e-01_rb,0.46101e-01_rb,0.55542e-01_rb,0.56309e-01_rb,0.38934e-01_rb /)
      kbo(:, 1,22, 9) = (/ &
        & 0.12675e-01_rb,0.32993e-01_rb,0.39256e-01_rb,0.39779e-01_rb,0.27529e-01_rb /)
      kbo(:, 2,22, 9) = (/ &
        & 0.13009e-01_rb,0.35201e-01_rb,0.42138e-01_rb,0.42658e-01_rb,0.29453e-01_rb /)
      kbo(:, 3,22, 9) = (/ &
        & 0.13317e-01_rb,0.37474e-01_rb,0.45144e-01_rb,0.45756e-01_rb,0.31474e-01_rb /)
      kbo(:, 4,22, 9) = (/ &
        & 0.13574e-01_rb,0.39784e-01_rb,0.48260e-01_rb,0.49049e-01_rb,0.33608e-01_rb /)
      kbo(:, 5,22, 9) = (/ &
        & 0.13778e-01_rb,0.42099e-01_rb,0.51448e-01_rb,0.52529e-01_rb,0.35844e-01_rb /)
      kbo(:, 1,23, 9) = (/ &
        & 0.10608e-01_rb,0.29677e-01_rb,0.35736e-01_rb,0.36323e-01_rb,0.24847e-01_rb /)
      kbo(:, 2,23, 9) = (/ &
        & 0.10880e-01_rb,0.31843e-01_rb,0.38613e-01_rb,0.39274e-01_rb,0.26772e-01_rb /)
      kbo(:, 3,23, 9) = (/ &
        & 0.11120e-01_rb,0.34081e-01_rb,0.41634e-01_rb,0.42440e-01_rb,0.28816e-01_rb /)
      kbo(:, 4,23, 9) = (/ &
        & 0.11313e-01_rb,0.36372e-01_rb,0.44754e-01_rb,0.45808e-01_rb,0.30978e-01_rb /)
      kbo(:, 5,23, 9) = (/ &
        & 0.11474e-01_rb,0.38689e-01_rb,0.47973e-01_rb,0.49367e-01_rb,0.33255e-01_rb /)
      kbo(:, 1,24, 9) = (/ &
        & 0.88685e-02_rb,0.26862e-01_rb,0.32736e-01_rb,0.33437e-01_rb,0.22600e-01_rb /)
      kbo(:, 2,24, 9) = (/ &
        & 0.90884e-02_rb,0.28994e-01_rb,0.35625e-01_rb,0.36451e-01_rb,0.24532e-01_rb /)
      kbo(:, 3,24, 9) = (/ &
        & 0.92759e-02_rb,0.31210e-01_rb,0.38663e-01_rb,0.39696e-01_rb,0.26600e-01_rb /)
      kbo(:, 4,24, 9) = (/ &
        & 0.94299e-02_rb,0.33488e-01_rb,0.41810e-01_rb,0.43137e-01_rb,0.28795e-01_rb /)
      kbo(:, 5,24, 9) = (/ &
        & 0.95655e-02_rb,0.35810e-01_rb,0.45067e-01_rb,0.46765e-01_rb,0.31119e-01_rb /)
      kbo(:, 1,25, 9) = (/ &
        & 0.74112e-02_rb,0.24493e-01_rb,0.30220e-01_rb,0.31056e-01_rb,0.20732e-01_rb /)
      kbo(:, 2,25, 9) = (/ &
        & 0.75852e-02_rb,0.26600e-01_rb,0.33130e-01_rb,0.34139e-01_rb,0.22685e-01_rb /)
      kbo(:, 3,25, 9) = (/ &
        & 0.77307e-02_rb,0.28805e-01_rb,0.36194e-01_rb,0.37461e-01_rb,0.24778e-01_rb /)
      kbo(:, 4,25, 9) = (/ &
        & 0.78603e-02_rb,0.31085e-01_rb,0.39384e-01_rb,0.40978e-01_rb,0.27016e-01_rb /)
      kbo(:, 5,25, 9) = (/ &
        & 0.79718e-02_rb,0.33416e-01_rb,0.42696e-01_rb,0.44682e-01_rb,0.29393e-01_rb /)
      kbo(:, 1,26, 9) = (/ &
        & 0.61902e-02_rb,0.22538e-01_rb,0.28170e-01_rb,0.29154e-01_rb,0.19220e-01_rb /)
      kbo(:, 2,26, 9) = (/ &
        & 0.63276e-02_rb,0.24632e-01_rb,0.31111e-01_rb,0.32321e-01_rb,0.21200e-01_rb /)
      kbo(:, 3,26, 9) = (/ &
        & 0.64499e-02_rb,0.26835e-01_rb,0.34213e-01_rb,0.35713e-01_rb,0.23332e-01_rb /)
      kbo(:, 4,26, 9) = (/ &
        & 0.65597e-02_rb,0.29126e-01_rb,0.37461e-01_rb,0.39319e-01_rb,0.25615e-01_rb /)
      kbo(:, 5,26, 9) = (/ &
        & 0.66551e-02_rb,0.31472e-01_rb,0.40844e-01_rb,0.43117e-01_rb,0.28057e-01_rb /)
      kbo(:, 1,27, 9) = (/ &
        & 0.51713e-02_rb,0.20911e-01_rb,0.26500e-01_rb,0.27639e-01_rb,0.17995e-01_rb /)
      kbo(:, 2,27, 9) = (/ &
        & 0.52848e-02_rb,0.23004e-01_rb,0.29475e-01_rb,0.30887e-01_rb,0.20007e-01_rb /)
      kbo(:, 3,27, 9) = (/ &
        & 0.53879e-02_rb,0.25219e-01_rb,0.32635e-01_rb,0.34364e-01_rb,0.22181e-01_rb /)
      kbo(:, 4,27, 9) = (/ &
        & 0.54763e-02_rb,0.27525e-01_rb,0.35956e-01_rb,0.38064e-01_rb,0.24520e-01_rb /)
      kbo(:, 5,27, 9) = (/ &
        & 0.55573e-02_rb,0.29889e-01_rb,0.39426e-01_rb,0.41977e-01_rb,0.27035e-01_rb /)
      kbo(:, 1,28, 9) = (/ &
        & 0.43195e-02_rb,0.19576e-01_rb,0.25160e-01_rb,0.26468e-01_rb,0.17016e-01_rb /)
      kbo(:, 2,28, 9) = (/ &
        & 0.44139e-02_rb,0.21680e-01_rb,0.28188e-01_rb,0.29800e-01_rb,0.19070e-01_rb /)
      kbo(:, 3,28, 9) = (/ &
        & 0.44969e-02_rb,0.23912e-01_rb,0.31420e-01_rb,0.33375e-01_rb,0.21294e-01_rb /)
      kbo(:, 4,28, 9) = (/ &
        & 0.45726e-02_rb,0.26235e-01_rb,0.34830e-01_rb,0.37186e-01_rb,0.23695e-01_rb /)
      kbo(:, 5,28, 9) = (/ &
        & 0.46426e-02_rb,0.28627e-01_rb,0.38395e-01_rb,0.41222e-01_rb,0.26282e-01_rb /)
      kbo(:, 1,29, 9) = (/ &
        & 0.36095e-02_rb,0.18508e-01_rb,0.24142e-01_rb,0.25625e-01_rb,0.16268e-01_rb /)
      kbo(:, 2,29, 9) = (/ &
        & 0.36856e-02_rb,0.20634e-01_rb,0.27243e-01_rb,0.29053e-01_rb,0.18372e-01_rb /)
      kbo(:, 3,29, 9) = (/ &
        & 0.37545e-02_rb,0.22890e-01_rb,0.30565e-01_rb,0.32742e-01_rb,0.20653e-01_rb /)
      kbo(:, 4,29, 9) = (/ &
        & 0.38219e-02_rb,0.25246e-01_rb,0.34070e-01_rb,0.36676e-01_rb,0.23125e-01_rb /)
      kbo(:, 5,29, 9) = (/ &
        & 0.38805e-02_rb,0.27676e-01_rb,0.37739e-01_rb,0.40837e-01_rb,0.25787e-01_rb /)
      kbo(:, 1,30, 9) = (/ &
        & 0.30149e-02_rb,0.17668e-01_rb,0.23401e-01_rb,0.25061e-01_rb,0.15717e-01_rb /)
      kbo(:, 2,30, 9) = (/ &
        & 0.30780e-02_rb,0.19823e-01_rb,0.26594e-01_rb,0.28602e-01_rb,0.17873e-01_rb /)
      kbo(:, 3,30, 9) = (/ &
        & 0.31391e-02_rb,0.22111e-01_rb,0.30012e-01_rb,0.32409e-01_rb,0.20220e-01_rb /)
      kbo(:, 4,30, 9) = (/ &
        & 0.31952e-02_rb,0.24511e-01_rb,0.33620e-01_rb,0.36467e-01_rb,0.22768e-01_rb /)
      kbo(:, 5,30, 9) = (/ &
        & 0.32462e-02_rb,0.26992e-01_rb,0.37393e-01_rb,0.40756e-01_rb,0.25513e-01_rb /)
      kbo(:, 1,31, 9) = (/ &
        & 0.25190e-02_rb,0.17039e-01_rb,0.22932e-01_rb,0.24769e-01_rb,0.15346e-01_rb /)
      kbo(:, 2,31, 9) = (/ &
        & 0.25729e-02_rb,0.19232e-01_rb,0.26227e-01_rb,0.28432e-01_rb,0.17564e-01_rb /)
      kbo(:, 3,31, 9) = (/ &
        & 0.26259e-02_rb,0.21567e-01_rb,0.29750e-01_rb,0.32369e-01_rb,0.19984e-01_rb /)
      kbo(:, 4,31, 9) = (/ &
        & 0.26746e-02_rb,0.24020e-01_rb,0.33463e-01_rb,0.36549e-01_rb,0.22614e-01_rb /)
      kbo(:, 5,31, 9) = (/ &
        & 0.27170e-02_rb,0.26556e-01_rb,0.37338e-01_rb,0.40957e-01_rb,0.25451e-01_rb /)
      kbo(:, 1,32, 9) = (/ &
        & 0.21057e-02_rb,0.16597e-01_rb,0.22710e-01_rb,0.24723e-01_rb,0.15137e-01_rb /)
      kbo(:, 2,32, 9) = (/ &
        & 0.21537e-02_rb,0.18840e-01_rb,0.26112e-01_rb,0.28512e-01_rb,0.17424e-01_rb /)
      kbo(:, 3,32, 9) = (/ &
        & 0.21983e-02_rb,0.21233e-01_rb,0.29743e-01_rb,0.32575e-01_rb,0.19926e-01_rb /)
      kbo(:, 4,32, 9) = (/ &
        & 0.22397e-02_rb,0.23745e-01_rb,0.33557e-01_rb,0.36873e-01_rb,0.22643e-01_rb /)
      kbo(:, 5,32, 9) = (/ &
        & 0.22779e-02_rb,0.26339e-01_rb,0.37525e-01_rb,0.41395e-01_rb,0.25571e-01_rb /)
      kbo(:, 1,33, 9) = (/ &
        & 0.17621e-02_rb,0.16326e-01_rb,0.22706e-01_rb,0.24894e-01_rb,0.15071e-01_rb /)
      kbo(:, 2,33, 9) = (/ &
        & 0.18037e-02_rb,0.18626e-01_rb,0.26218e-01_rb,0.28807e-01_rb,0.17434e-01_rb /)
      kbo(:, 3,33, 9) = (/ &
        & 0.18413e-02_rb,0.21083e-01_rb,0.29952e-01_rb,0.32989e-01_rb,0.20023e-01_rb /)
      kbo(:, 4,33, 9) = (/ &
        & 0.18774e-02_rb,0.23655e-01_rb,0.33863e-01_rb,0.37400e-01_rb,0.22833e-01_rb /)
      kbo(:, 5,33, 9) = (/ &
        & 0.19114e-02_rb,0.26308e-01_rb,0.37916e-01_rb,0.42025e-01_rb,0.25847e-01_rb /)
      kbo(:, 1,34, 9) = (/ &
        & 0.14751e-02_rb,0.16125e-01_rb,0.22769e-01_rb,0.25117e-01_rb,0.15050e-01_rb /)
      kbo(:, 2,34, 9) = (/ &
        & 0.15101e-02_rb,0.18488e-01_rb,0.26381e-01_rb,0.29139e-01_rb,0.17489e-01_rb /)
      kbo(:, 3,34, 9) = (/ &
        & 0.15432e-02_rb,0.21005e-01_rb,0.30205e-01_rb,0.33424e-01_rb,0.20158e-01_rb /)
      kbo(:, 4,34, 9) = (/ &
        & 0.15747e-02_rb,0.23633e-01_rb,0.34198e-01_rb,0.37931e-01_rb,0.23050e-01_rb /)
      kbo(:, 5,34, 9) = (/ &
        & 0.16051e-02_rb,0.26339e-01_rb,0.38321e-01_rb,0.42642e-01_rb,0.26140e-01_rb /)
      kbo(:, 1,35, 9) = (/ &
        & 0.12323e-02_rb,0.15834e-01_rb,0.22646e-01_rb,0.25106e-01_rb,0.14908e-01_rb /)
      kbo(:, 2,35, 9) = (/ &
        & 0.12626e-02_rb,0.18245e-01_rb,0.26331e-01_rb,0.29206e-01_rb,0.17403e-01_rb /)
      kbo(:, 3,35, 9) = (/ &
        & 0.12919e-02_rb,0.20808e-01_rb,0.30221e-01_rb,0.33562e-01_rb,0.20132e-01_rb /)
      kbo(:, 4,35, 9) = (/ &
        & 0.13197e-02_rb,0.23481e-01_rb,0.34270e-01_rb,0.38136e-01_rb,0.23081e-01_rb /)
      kbo(:, 5,35, 9) = (/ &
        & 0.13470e-02_rb,0.26228e-01_rb,0.38442e-01_rb,0.42907e-01_rb,0.26222e-01_rb /)
      kbo(:, 1,36, 9) = (/ &
        & 0.10273e-02_rb,0.15393e-01_rb,0.22247e-01_rb,0.24766e-01_rb,0.14589e-01_rb /)
      kbo(:, 2,36, 9) = (/ &
        & 0.10537e-02_rb,0.17833e-01_rb,0.25974e-01_rb,0.28911e-01_rb,0.17115e-01_rb /)
      kbo(:, 3,36, 9) = (/ &
        & 0.10795e-02_rb,0.20426e-01_rb,0.29904e-01_rb,0.33307e-01_rb,0.19875e-01_rb /)
      kbo(:, 4,36, 9) = (/ &
        & 0.11043e-02_rb,0.23131e-01_rb,0.33989e-01_rb,0.37917e-01_rb,0.22855e-01_rb /)
      kbo(:, 5,36, 9) = (/ &
        & 0.11288e-02_rb,0.25909e-01_rb,0.38197e-01_rb,0.42722e-01_rb,0.26026e-01_rb /)
      kbo(:, 1,37, 9) = (/ &
        & 0.85383e-03_rb,0.14679e-01_rb,0.21388e-01_rb,0.23897e-01_rb,0.13970e-01_rb /)
      kbo(:, 2,37, 9) = (/ &
        & 0.87682e-03_rb,0.17120e-01_rb,0.25115e-01_rb,0.28039e-01_rb,0.16489e-01_rb /)
      kbo(:, 3,37, 9) = (/ &
        & 0.89996e-03_rb,0.19721e-01_rb,0.29054e-01_rb,0.32436e-01_rb,0.19242e-01_rb /)
      kbo(:, 4,37, 9) = (/ &
        & 0.92246e-03_rb,0.22439e-01_rb,0.33150e-01_rb,0.37047e-01_rb,0.22217e-01_rb /)
      kbo(:, 5,37, 9) = (/ &
        & 0.94506e-03_rb,0.25234e-01_rb,0.37369e-01_rb,0.41852e-01_rb,0.25386e-01_rb /)
      kbo(:, 1,38, 9) = (/ &
        & 0.70953e-03_rb,0.14014e-01_rb,0.20566e-01_rb,0.23058e-01_rb,0.13383e-01_rb /)
      kbo(:, 2,38, 9) = (/ &
        & 0.73008e-03_rb,0.16455e-01_rb,0.24289e-01_rb,0.27191e-01_rb,0.15892e-01_rb /)
      kbo(:, 3,38, 9) = (/ &
        & 0.75050e-03_rb,0.19058e-01_rb,0.28230e-01_rb,0.31581e-01_rb,0.18633e-01_rb /)
      kbo(:, 4,38, 9) = (/ &
        & 0.77107e-03_rb,0.21784e-01_rb,0.32331e-01_rb,0.36184e-01_rb,0.21597e-01_rb /)
      kbo(:, 5,38, 9) = (/ &
        & 0.79117e-03_rb,0.24590e-01_rb,0.36557e-01_rb,0.40986e-01_rb,0.24760e-01_rb /)
      kbo(:, 1,39, 9) = (/ &
        & 0.58982e-03_rb,0.13410e-01_rb,0.19807e-01_rb,0.22275e-01_rb,0.12843e-01_rb /)
      kbo(:, 2,39, 9) = (/ &
        & 0.60777e-03_rb,0.15847e-01_rb,0.23520e-01_rb,0.26395e-01_rb,0.15340e-01_rb /)
      kbo(:, 3,39, 9) = (/ &
        & 0.62607e-03_rb,0.18449e-01_rb,0.27458e-01_rb,0.30773e-01_rb,0.18067e-01_rb /)
      kbo(:, 4,39, 9) = (/ &
        & 0.64471e-03_rb,0.21179e-01_rb,0.31559e-01_rb,0.35367e-01_rb,0.21018e-01_rb /)
      kbo(:, 5,39, 9) = (/ &
        & 0.66285e-03_rb,0.23994e-01_rb,0.35788e-01_rb,0.40160e-01_rb,0.24170e-01_rb /)
      kbo(:, 1,40, 9) = (/ &
        & 0.48914e-03_rb,0.12617e-01_rb,0.18733e-01_rb,0.21131e-01_rb,0.12100e-01_rb /)
      kbo(:, 2,40, 9) = (/ &
        & 0.50499e-03_rb,0.15028e-01_rb,0.22406e-01_rb,0.25206e-01_rb,0.14559e-01_rb /)
      kbo(:, 3,40, 9) = (/ &
        & 0.52119e-03_rb,0.17611e-01_rb,0.26314e-01_rb,0.29544e-01_rb,0.17245e-01_rb /)
      kbo(:, 4,40, 9) = (/ &
        & 0.53770e-03_rb,0.20330e-01_rb,0.30397e-01_rb,0.34103e-01_rb,0.20159e-01_rb /)
      kbo(:, 5,40, 9) = (/ &
        & 0.55398e-03_rb,0.23142e-01_rb,0.34616e-01_rb,0.38867e-01_rb,0.23278e-01_rb /)
      kbo(:, 1,41, 9) = (/ &
        & 0.40532e-03_rb,0.11840e-01_rb,0.17661e-01_rb,0.19981e-01_rb,0.11365e-01_rb /)
      kbo(:, 2,41, 9) = (/ &
        & 0.41916e-03_rb,0.14219e-01_rb,0.21285e-01_rb,0.24002e-01_rb,0.13782e-01_rb /)
      kbo(:, 3,41, 9) = (/ &
        & 0.43354e-03_rb,0.16775e-01_rb,0.25154e-01_rb,0.28292e-01_rb,0.16423e-01_rb /)
      kbo(:, 4,41, 9) = (/ &
        & 0.44807e-03_rb,0.19478e-01_rb,0.29214e-01_rb,0.32812e-01_rb,0.19291e-01_rb /)
      kbo(:, 5,41, 9) = (/ &
        & 0.46282e-03_rb,0.22283e-01_rb,0.33415e-01_rb,0.37538e-01_rb,0.22373e-01_rb /)
      kbo(:, 1,42, 9) = (/ &
        & 0.33573e-03_rb,0.11104e-01_rb,0.16634e-01_rb,0.18873e-01_rb,0.10664e-01_rb /)
      kbo(:, 2,42, 9) = (/ &
        & 0.34776e-03_rb,0.13446e-01_rb,0.20201e-01_rb,0.22835e-01_rb,0.13037e-01_rb /)
      kbo(:, 3,42, 9) = (/ &
        & 0.36046e-03_rb,0.15973e-01_rb,0.24027e-01_rb,0.27071e-01_rb,0.15631e-01_rb /)
      kbo(:, 4,42, 9) = (/ &
        & 0.37344e-03_rb,0.18654e-01_rb,0.28057e-01_rb,0.31550e-01_rb,0.18451e-01_rb /)
      kbo(:, 5,42, 9) = (/ &
        & 0.38663e-03_rb,0.21448e-01_rb,0.32235e-01_rb,0.36231e-01_rb,0.21492e-01_rb /)
      kbo(:, 1,43, 9) = (/ &
        & 0.27733e-03_rb,0.10283e-01_rb,0.15460e-01_rb,0.17597e-01_rb,0.98707e-02_rb /)
      kbo(:, 2,43, 9) = (/ &
        & 0.28779e-03_rb,0.12571e-01_rb,0.18947e-01_rb,0.21473e-01_rb,0.12183e-01_rb /)
      kbo(:, 3,43, 9) = (/ &
        & 0.29891e-03_rb,0.15054e-01_rb,0.22712e-01_rb,0.25638e-01_rb,0.14718e-01_rb /)
      kbo(:, 4,43, 9) = (/ &
        & 0.31033e-03_rb,0.17702e-01_rb,0.26696e-01_rb,0.30055e-01_rb,0.17477e-01_rb /)
      kbo(:, 5,43, 9) = (/ &
        & 0.32200e-03_rb,0.20476e-01_rb,0.30841e-01_rb,0.34682e-01_rb,0.20459e-01_rb /)
      kbo(:, 1,44, 9) = (/ &
        & 0.22878e-03_rb,0.94499e-02_rb,0.14251e-01_rb,0.16276e-01_rb,0.90607e-02_rb /)
      kbo(:, 2,44, 9) = (/ &
        & 0.23775e-03_rb,0.11673e-01_rb,0.17646e-01_rb,0.20049e-01_rb,0.11303e-01_rb /)
      kbo(:, 3,44, 9) = (/ &
        & 0.24747e-03_rb,0.14106e-01_rb,0.21336e-01_rb,0.24135e-01_rb,0.13772e-01_rb /)
      kbo(:, 4,44, 9) = (/ &
        & 0.25755e-03_rb,0.16710e-01_rb,0.25261e-01_rb,0.28479e-01_rb,0.16464e-01_rb /)
      kbo(:, 5,44, 9) = (/ &
        & 0.26798e-03_rb,0.19454e-01_rb,0.29363e-01_rb,0.33046e-01_rb,0.19378e-01_rb /)
      kbo(:, 1,45, 9) = (/ &
        & 0.18849e-03_rb,0.86584e-02_rb,0.13096e-01_rb,0.15007e-01_rb,0.82904e-02_rb /)
      kbo(:, 2,45, 9) = (/ &
        & 0.19636e-03_rb,0.10814e-01_rb,0.16390e-01_rb,0.18672e-01_rb,0.10458e-01_rb /)
      kbo(:, 3,45, 9) = (/ &
        & 0.20470e-03_rb,0.13189e-01_rb,0.19995e-01_rb,0.22671e-01_rb,0.12861e-01_rb /)
      kbo(:, 4,45, 9) = (/ &
        & 0.21344e-03_rb,0.15746e-01_rb,0.23854e-01_rb,0.26936e-01_rb,0.15483e-01_rb /)
      kbo(:, 5,45, 9) = (/ &
        & 0.22266e-03_rb,0.18454e-01_rb,0.27912e-01_rb,0.31439e-01_rb,0.18328e-01_rb /)
      kbo(:, 1,46, 9) = (/ &
        & 0.15507e-03_rb,0.78632e-02_rb,0.11928e-01_rb,0.13712e-01_rb,0.75164e-02_rb /)
      kbo(:, 2,46, 9) = (/ &
        & 0.16179e-03_rb,0.99413e-02_rb,0.15104e-01_rb,0.17260e-01_rb,0.95985e-02_rb /)
      kbo(:, 3,46, 9) = (/ &
        & 0.16890e-03_rb,0.12249e-01_rb,0.18612e-01_rb,0.21154e-01_rb,0.11927e-01_rb /)
      kbo(:, 4,46, 9) = (/ &
        & 0.17659e-03_rb,0.14751e-01_rb,0.22394e-01_rb,0.25333e-01_rb,0.14476e-01_rb /)
      kbo(:, 5,46, 9) = (/ &
        & 0.18474e-03_rb,0.17415e-01_rb,0.26393e-01_rb,0.29761e-01_rb,0.17246e-01_rb /)
      kbo(:, 1,47, 9) = (/ &
        & 0.12726e-03_rb,0.70304e-02_rb,0.10696e-01_rb,0.12336e-01_rb,0.67029e-02_rb /)
      kbo(:, 2,47, 9) = (/ &
        & 0.13291e-03_rb,0.90171e-02_rb,0.13730e-01_rb,0.15745e-01_rb,0.86863e-02_rb /)
      kbo(:, 3,47, 9) = (/ &
        & 0.13906e-03_rb,0.11243e-01_rb,0.17120e-01_rb,0.19510e-01_rb,0.10925e-01_rb /)
      kbo(:, 4,47, 9) = (/ &
        & 0.14561e-03_rb,0.13678e-01_rb,0.20805e-01_rb,0.23588e-01_rb,0.13392e-01_rb /)
      kbo(:, 5,47, 9) = (/ &
        & 0.15268e-03_rb,0.16286e-01_rb,0.24728e-01_rb,0.27924e-01_rb,0.16077e-01_rb /)
      kbo(:, 1,48, 9) = (/ &
        & 0.10430e-03_rb,0.62479e-02_rb,0.95325e-02_rb,0.11029e-01_rb,0.59380e-02_rb /)
      kbo(:, 2,48, 9) = (/ &
        & 0.10909e-03_rb,0.81380e-02_rb,0.12419e-01_rb,0.14291e-01_rb,0.78205e-02_rb /)
      kbo(:, 3,48, 9) = (/ &
        & 0.11429e-03_rb,0.10277e-01_rb,0.15680e-01_rb,0.17922e-01_rb,0.99641e-02_rb /)
      kbo(:, 4,48, 9) = (/ &
        & 0.11996e-03_rb,0.12637e-01_rb,0.19259e-01_rb,0.21888e-01_rb,0.12347e-01_rb /)
      kbo(:, 5,48, 9) = (/ &
        & 0.12611e-03_rb,0.15183e-01_rb,0.23095e-01_rb,0.26125e-01_rb,0.14947e-01_rb /)
      kbo(:, 1,49, 9) = (/ &
        & 0.85361e-04_rb,0.55178e-02_rb,0.84413e-02_rb,0.97939e-02_rb,0.52224e-02_rb /)
      kbo(:, 2,49, 9) = (/ &
        & 0.89682e-04_rb,0.73124e-02_rb,0.11182e-01_rb,0.12906e-01_rb,0.70024e-02_rb /)
      kbo(:, 3,49, 9) = (/ &
        & 0.93818e-04_rb,0.93518e-02_rb,0.14295e-01_rb,0.16393e-01_rb,0.90454e-02_rb /)
      kbo(:, 4,49, 9) = (/ &
        & 0.98731e-04_rb,0.11631e-01_rb,0.17758e-01_rb,0.20234e-01_rb,0.11339e-01_rb /)
      kbo(:, 5,49, 9) = (/ &
        & 0.10398e-03_rb,0.14111e-01_rb,0.21502e-01_rb,0.24370e-01_rb,0.13856e-01_rb /)
      kbo(:, 1,50, 9) = (/ &
        & 0.70068e-04_rb,0.48843e-02_rb,0.74862e-02_rb,0.87029e-02_rb,0.45961e-02_rb /)
      kbo(:, 2,50, 9) = (/ &
        & 0.73528e-04_rb,0.65706e-02_rb,0.10074e-01_rb,0.11661e-01_rb,0.62737e-02_rb /)
      kbo(:, 3,50, 9) = (/ &
        & 0.77377e-04_rb,0.85288e-02_rb,0.13057e-01_rb,0.15019e-01_rb,0.82236e-02_rb /)
      kbo(:, 4,50, 9) = (/ &
        & 0.81195e-04_rb,0.10720e-01_rb,0.16396e-01_rb,0.18728e-01_rb,0.10428e-01_rb /)
      kbo(:, 5,50, 9) = (/ &
        & 0.85703e-04_rb,0.13133e-01_rb,0.20042e-01_rb,0.22765e-01_rb,0.12867e-01_rb /)
      kbo(:, 1,51, 9) = (/ &
        & 0.57383e-04_rb,0.43058e-02_rb,0.66126e-02_rb,0.77023e-02_rb,0.40299e-02_rb /)
      kbo(:, 2,51, 9) = (/ &
        & 0.60290e-04_rb,0.58916e-02_rb,0.90543e-02_rb,0.10509e-01_rb,0.56057e-02_rb /)
      kbo(:, 3,51, 9) = (/ &
        & 0.63513e-04_rb,0.77586e-02_rb,0.11901e-01_rb,0.13729e-01_rb,0.74620e-02_rb /)
      kbo(:, 4,51, 9) = (/ &
        & 0.66727e-04_rb,0.98672e-02_rb,0.15116e-01_rb,0.17314e-01_rb,0.95779e-02_rb /)
      kbo(:, 5,51, 9) = (/ &
        & 0.70598e-04_rb,0.12209e-01_rb,0.18661e-01_rb,0.21244e-01_rb,0.11938e-01_rb /)
      kbo(:, 1,52, 9) = (/ &
        & 0.46933e-04_rb,0.37683e-02_rb,0.57987e-02_rb,0.67684e-02_rb,0.35066e-02_rb /)
      kbo(:, 2,52, 9) = (/ &
        & 0.49374e-04_rb,0.52554e-02_rb,0.80944e-02_rb,0.94163e-02_rb,0.49785e-02_rb /)
      kbo(:, 3,52, 9) = (/ &
        & 0.52069e-04_rb,0.70251e-02_rb,0.10799e-01_rb,0.12493e-01_rb,0.67384e-02_rb /)
      kbo(:, 4,52, 9) = (/ &
        & 0.55066e-04_rb,0.90579e-02_rb,0.13892e-01_rb,0.15957e-01_rb,0.87644e-02_rb /)
      kbo(:, 5,52, 9) = (/ &
        & 0.58094e-04_rb,0.11314e-01_rb,0.17321e-01_rb,0.19762e-01_rb,0.11039e-01_rb /)
      kbo(:, 1,53, 9) = (/ &
        & 0.38369e-04_rb,0.32705e-02_rb,0.50428e-02_rb,0.58997e-02_rb,0.30250e-02_rb /)
      kbo(:, 2,53, 9) = (/ &
        & 0.40389e-04_rb,0.46579e-02_rb,0.71882e-02_rb,0.83794e-02_rb,0.43908e-02_rb /)
      kbo(:, 3,53, 9) = (/ &
        & 0.42656e-04_rb,0.63275e-02_rb,0.97486e-02_rb,0.11308e-01_rb,0.60500e-02_rb /)
      kbo(:, 4,53, 9) = (/ &
        & 0.45173e-04_rb,0.82714e-02_rb,0.12706e-01_rb,0.14639e-01_rb,0.79824e-02_rb /)
      kbo(:, 5,53, 9) = (/ &
        & 0.47733e-04_rb,0.10447e-01_rb,0.16018e-01_rb,0.18320e-01_rb,0.10170e-01_rb /)
      kbo(:, 1,54, 9) = (/ &
        & 0.31373e-04_rb,0.28489e-02_rb,0.44023e-02_rb,0.51602e-02_rb,0.26198e-02_rb /)
      kbo(:, 2,54, 9) = (/ &
        & 0.33072e-04_rb,0.41448e-02_rb,0.64072e-02_rb,0.74821e-02_rb,0.38868e-02_rb /)
      kbo(:, 3,54, 9) = (/ &
        & 0.34962e-04_rb,0.57219e-02_rb,0.88331e-02_rb,0.10271e-01_rb,0.54518e-02_rb /)
      kbo(:, 4,54, 9) = (/ &
        & 0.37073e-04_rb,0.75801e-02_rb,0.11663e-01_rb,0.13472e-01_rb,0.72975e-02_rb /)
      kbo(:, 5,54, 9) = (/ &
        & 0.39224e-04_rb,0.96795e-02_rb,0.14861e-01_rb,0.17039e-01_rb,0.94030e-02_rb /)
      kbo(:, 1,55, 9) = (/ &
        & 0.25649e-04_rb,0.24774e-02_rb,0.38358e-02_rb,0.45027e-02_rb,0.22635e-02_rb /)
      kbo(:, 2,55, 9) = (/ &
        & 0.27065e-04_rb,0.36831e-02_rb,0.57040e-02_rb,0.66733e-02_rb,0.34361e-02_rb /)
      kbo(:, 3,55, 9) = (/ &
        & 0.28649e-04_rb,0.51728e-02_rb,0.80003e-02_rb,0.93199e-02_rb,0.49087e-02_rb /)
      kbo(:, 4,55, 9) = (/ &
        & 0.30419e-04_rb,0.69443e-02_rb,0.10704e-01_rb,0.12395e-01_rb,0.66688e-02_rb /)
      kbo(:, 5,55, 9) = (/ &
        & 0.32478e-04_rb,0.89781e-02_rb,0.13794e-01_rb,0.15856e-01_rb,0.86947e-02_rb /)
      kbo(:, 1,56, 9) = (/ &
        & 0.20960e-04_rb,0.21374e-02_rb,0.33161e-02_rb,0.38972e-02_rb,0.19394e-02_rb /)
      kbo(:, 2,56, 9) = (/ &
        & 0.22130e-04_rb,0.32530e-02_rb,0.50471e-02_rb,0.59166e-02_rb,0.30185e-02_rb /)
      kbo(:, 3,56, 9) = (/ &
        & 0.23455e-04_rb,0.46545e-02_rb,0.72105e-02_rb,0.84156e-02_rb,0.43975e-02_rb /)
      kbo(:, 4,56, 9) = (/ &
        & 0.24958e-04_rb,0.63377e-02_rb,0.97876e-02_rb,0.11360e-01_rb,0.60688e-02_rb /)
      kbo(:, 5,56, 9) = (/ &
        & 0.26672e-04_rb,0.82933e-02_rb,0.12759e-01_rb,0.14704e-01_rb,0.80123e-02_rb /)
      kbo(:, 1,57, 9) = (/ &
        & 0.17108e-04_rb,0.18275e-02_rb,0.28407e-02_rb,0.33429e-02_rb,0.16463e-02_rb /)
      kbo(:, 2,57, 9) = (/ &
        & 0.18082e-04_rb,0.28535e-02_rb,0.44360e-02_rb,0.52097e-02_rb,0.26326e-02_rb /)
      kbo(:, 3,57, 9) = (/ &
        & 0.19184e-04_rb,0.41668e-02_rb,0.64646e-02_rb,0.75572e-02_rb,0.39170e-02_rb /)
      kbo(:, 4,57, 9) = (/ &
        & 0.20446e-04_rb,0.57610e-02_rb,0.89128e-02_rb,0.10368e-01_rb,0.54979e-02_rb /)
      kbo(:, 5,57, 9) = (/ &
        & 0.21854e-04_rb,0.76342e-02_rb,0.11762e-01_rb,0.13588e-01_rb,0.73577e-02_rb /)
      kbo(:, 1,58, 9) = (/ &
        & 0.13963e-04_rb,0.15602e-02_rb,0.24292e-02_rb,0.28638e-02_rb,0.13965e-02_rb /)
      kbo(:, 2,58, 9) = (/ &
        & 0.14773e-04_rb,0.25031e-02_rb,0.38984e-02_rb,0.45849e-02_rb,0.22957e-02_rb /)
      kbo(:, 3,58, 9) = (/ &
        & 0.15698e-04_rb,0.37311e-02_rb,0.57975e-02_rb,0.67885e-02_rb,0.34899e-02_rb /)
      kbo(:, 4,58, 9) = (/ &
        & 0.16731e-04_rb,0.52413e-02_rb,0.81221e-02_rb,0.94657e-02_rb,0.49831e-02_rb /)
      kbo(:, 5,58, 9) = (/ &
        & 0.17926e-04_rb,0.70329e-02_rb,0.10852e-01_rb,0.12566e-01_rb,0.67622e-02_rb /)
      kbo(:, 1,59, 9) = (/ &
        & 0.11515e-04_rb,0.14557e-02_rb,0.22712e-02_rb,0.26808e-02_rb,0.13001e-02_rb /)
      kbo(:, 2,59, 9) = (/ &
        & 0.12194e-04_rb,0.23665e-02_rb,0.36913e-02_rb,0.43444e-02_rb,0.21654e-02_rb /)
      kbo(:, 3,59, 9) = (/ &
        & 0.12975e-04_rb,0.35608e-02_rb,0.55385e-02_rb,0.64911e-02_rb,0.33241e-02_rb /)
      kbo(:, 4,59, 9) = (/ &
        & 0.13854e-04_rb,0.50382e-02_rb,0.78142e-02_rb,0.91137e-02_rb,0.47825e-02_rb /)
      kbo(:, 5,59, 9) = (/ &
        & 0.14890e-04_rb,0.67966e-02_rb,0.10497e-01_rb,0.12166e-01_rb,0.65291e-02_rb /)
      kbo(:, 1,13,10) = (/ &
        & 0.23136e+00_rb,0.35590e+00_rb,0.41705e+00_rb,0.42838e+00_rb,0.34405e+00_rb /)
      kbo(:, 2,13,10) = (/ &
        & 0.23570e+00_rb,0.35573e+00_rb,0.41855e+00_rb,0.43318e+00_rb,0.34979e+00_rb /)
      kbo(:, 3,13,10) = (/ &
        & 0.23997e+00_rb,0.35949e+00_rb,0.42029e+00_rb,0.43813e+00_rb,0.35426e+00_rb /)
      kbo(:, 4,13,10) = (/ &
        & 0.24492e+00_rb,0.36590e+00_rb,0.42350e+00_rb,0.44177e+00_rb,0.35947e+00_rb /)
      kbo(:, 5,13,10) = (/ &
        & 0.25202e+00_rb,0.37242e+00_rb,0.42924e+00_rb,0.44467e+00_rb,0.36481e+00_rb /)
      kbo(:, 1,14,10) = (/ &
        & 0.20433e+00_rb,0.32280e+00_rb,0.38114e+00_rb,0.39530e+00_rb,0.30987e+00_rb /)
      kbo(:, 2,14,10) = (/ &
        & 0.20812e+00_rb,0.32580e+00_rb,0.38400e+00_rb,0.40099e+00_rb,0.31521e+00_rb /)
      kbo(:, 3,14,10) = (/ &
        & 0.21260e+00_rb,0.33207e+00_rb,0.38794e+00_rb,0.40573e+00_rb,0.32135e+00_rb /)
      kbo(:, 4,14,10) = (/ &
        & 0.21971e+00_rb,0.33857e+00_rb,0.39426e+00_rb,0.40959e+00_rb,0.32811e+00_rb /)
      kbo(:, 5,14,10) = (/ &
        & 0.22862e+00_rb,0.34522e+00_rb,0.40235e+00_rb,0.41376e+00_rb,0.33439e+00_rb /)
      kbo(:, 1,15,10) = (/ &
        & 0.17974e+00_rb,0.29385e+00_rb,0.34710e+00_rb,0.36214e+00_rb,0.27698e+00_rb /)
      kbo(:, 2,15,10) = (/ &
        & 0.18361e+00_rb,0.29985e+00_rb,0.35176e+00_rb,0.36835e+00_rb,0.28361e+00_rb /)
      kbo(:, 3,15,10) = (/ &
        & 0.19011e+00_rb,0.30650e+00_rb,0.35844e+00_rb,0.37336e+00_rb,0.29140e+00_rb /)
      kbo(:, 4,15,10) = (/ &
        & 0.19870e+00_rb,0.31340e+00_rb,0.36676e+00_rb,0.37856e+00_rb,0.29913e+00_rb /)
      kbo(:, 5,15,10) = (/ &
        & 0.20713e+00_rb,0.32067e+00_rb,0.37517e+00_rb,0.38452e+00_rb,0.30673e+00_rb /)
      kbo(:, 1,16,10) = (/ &
        & 0.15768e+00_rb,0.26909e+00_rb,0.31566e+00_rb,0.32905e+00_rb,0.24795e+00_rb /)
      kbo(:, 2,16,10) = (/ &
        & 0.16341e+00_rb,0.27579e+00_rb,0.32257e+00_rb,0.33542e+00_rb,0.25593e+00_rb /)
      kbo(:, 3,16,10) = (/ &
        & 0.17143e+00_rb,0.28283e+00_rb,0.33152e+00_rb,0.34164e+00_rb,0.26468e+00_rb /)
      kbo(:, 4,16,10) = (/ &
        & 0.17943e+00_rb,0.29026e+00_rb,0.34054e+00_rb,0.34897e+00_rb,0.27328e+00_rb /)
      kbo(:, 5,16,10) = (/ &
        & 0.18690e+00_rb,0.29935e+00_rb,0.34954e+00_rb,0.35767e+00_rb,0.28225e+00_rb /)
      kbo(:, 1,17,10) = (/ &
        & 0.13875e+00_rb,0.24652e+00_rb,0.28696e+00_rb,0.29693e+00_rb,0.22321e+00_rb /)
      kbo(:, 2,17,10) = (/ &
        & 0.14611e+00_rb,0.25366e+00_rb,0.29630e+00_rb,0.30411e+00_rb,0.23216e+00_rb /)
      kbo(:, 3,17,10) = (/ &
        & 0.15377e+00_rb,0.26122e+00_rb,0.30597e+00_rb,0.31229e+00_rb,0.24127e+00_rb /)
      kbo(:, 4,17,10) = (/ &
        & 0.16099e+00_rb,0.27029e+00_rb,0.31565e+00_rb,0.32218e+00_rb,0.25077e+00_rb /)
      kbo(:, 5,17,10) = (/ &
        & 0.16791e+00_rb,0.28100e+00_rb,0.32533e+00_rb,0.33316e+00_rb,0.26091e+00_rb /)
      kbo(:, 1,18,10) = (/ &
        & 0.12258e+00_rb,0.22568e+00_rb,0.26154e+00_rb,0.26746e+00_rb,0.20208e+00_rb /)
      kbo(:, 2,18,10) = (/ &
        & 0.12972e+00_rb,0.23340e+00_rb,0.27183e+00_rb,0.27607e+00_rb,0.21144e+00_rb /)
      kbo(:, 3,18,10) = (/ &
        & 0.13673e+00_rb,0.24228e+00_rb,0.28202e+00_rb,0.28661e+00_rb,0.22093e+00_rb /)
      kbo(:, 4,18,10) = (/ &
        & 0.14352e+00_rb,0.25285e+00_rb,0.29256e+00_rb,0.29859e+00_rb,0.23121e+00_rb /)
      kbo(:, 5,18,10) = (/ &
        & 0.14999e+00_rb,0.26447e+00_rb,0.30340e+00_rb,0.31163e+00_rb,0.24203e+00_rb /)
      kbo(:, 1,19,10) = (/ &
        & 0.10785e+00_rb,0.20667e+00_rb,0.23879e+00_rb,0.24194e+00_rb,0.18368e+00_rb /)
      kbo(:, 2,19,10) = (/ &
        & 0.11429e+00_rb,0.21546e+00_rb,0.24926e+00_rb,0.25235e+00_rb,0.19325e+00_rb /)
      kbo(:, 3,19,10) = (/ &
        & 0.12064e+00_rb,0.22577e+00_rb,0.26022e+00_rb,0.26472e+00_rb,0.20338e+00_rb /)
      kbo(:, 4,19,10) = (/ &
        & 0.12700e+00_rb,0.23751e+00_rb,0.27175e+00_rb,0.27823e+00_rb,0.21416e+00_rb /)
      kbo(:, 5,19,10) = (/ &
        & 0.13335e+00_rb,0.24924e+00_rb,0.28423e+00_rb,0.29239e+00_rb,0.22548e+00_rb /)
      kbo(:, 1,20,10) = (/ &
        & 0.94314e-01_rb,0.18969e+00_rb,0.21843e+00_rb,0.22089e+00_rb,0.16767e+00_rb /)
      kbo(:, 2,20,10) = (/ &
        & 0.10011e+00_rb,0.20009e+00_rb,0.22938e+00_rb,0.23290e+00_rb,0.17759e+00_rb /)
      kbo(:, 3,20,10) = (/ &
        & 0.10602e+00_rb,0.21171e+00_rb,0.24114e+00_rb,0.24650e+00_rb,0.18815e+00_rb /)
      kbo(:, 4,20,10) = (/ &
        & 0.11212e+00_rb,0.22366e+00_rb,0.25410e+00_rb,0.26076e+00_rb,0.19951e+00_rb /)
      kbo(:, 5,20,10) = (/ &
        & 0.11820e+00_rb,0.23566e+00_rb,0.26816e+00_rb,0.27572e+00_rb,0.21140e+00_rb /)
      kbo(:, 1,21,10) = (/ &
        & 0.82096e-01_rb,0.17498e+00_rb,0.20057e+00_rb,0.20349e+00_rb,0.15368e+00_rb /)
      kbo(:, 2,21,10) = (/ &
        & 0.87376e-01_rb,0.18671e+00_rb,0.21217e+00_rb,0.21678e+00_rb,0.16394e+00_rb /)
      kbo(:, 3,21,10) = (/ &
        & 0.92996e-01_rb,0.19901e+00_rb,0.22527e+00_rb,0.23088e+00_rb,0.17505e+00_rb /)
      kbo(:, 4,21,10) = (/ &
        & 0.98752e-01_rb,0.21131e+00_rb,0.23942e+00_rb,0.24575e+00_rb,0.18685e+00_rb /)
      kbo(:, 5,21,10) = (/ &
        & 0.10451e+00_rb,0.22356e+00_rb,0.25490e+00_rb,0.26152e+00_rb,0.19924e+00_rb /)
      kbo(:, 1,22,10) = (/ &
        & 0.71578e-01_rb,0.16309e+00_rb,0.18609e+00_rb,0.19018e+00_rb,0.14217e+00_rb /)
      kbo(:, 2,22,10) = (/ &
        & 0.76626e-01_rb,0.17564e+00_rb,0.19884e+00_rb,0.20398e+00_rb,0.15291e+00_rb /)
      kbo(:, 3,22,10) = (/ &
        & 0.81885e-01_rb,0.18840e+00_rb,0.21309e+00_rb,0.21858e+00_rb,0.16461e+00_rb /)
      kbo(:, 4,22,10) = (/ &
        & 0.87223e-01_rb,0.20113e+00_rb,0.22859e+00_rb,0.23430e+00_rb,0.17688e+00_rb /)
      kbo(:, 5,22,10) = (/ &
        & 0.92500e-01_rb,0.21374e+00_rb,0.24532e+00_rb,0.25104e+00_rb,0.18976e+00_rb /)
      kbo(:, 1,23,10) = (/ &
        & 0.62506e-01_rb,0.15298e+00_rb,0.17442e+00_rb,0.17913e+00_rb,0.13258e+00_rb /)
      kbo(:, 2,23,10) = (/ &
        & 0.67194e-01_rb,0.16597e+00_rb,0.18829e+00_rb,0.19340e+00_rb,0.14391e+00_rb /)
      kbo(:, 3,23,10) = (/ &
        & 0.72099e-01_rb,0.17922e+00_rb,0.20360e+00_rb,0.20869e+00_rb,0.15600e+00_rb /)
      kbo(:, 4,23,10) = (/ &
        & 0.77023e-01_rb,0.19243e+00_rb,0.22032e+00_rb,0.22532e+00_rb,0.16873e+00_rb /)
      kbo(:, 5,23,10) = (/ &
        & 0.81629e-01_rb,0.20537e+00_rb,0.23792e+00_rb,0.24291e+00_rb,0.18203e+00_rb /)
      kbo(:, 1,24,10) = (/ &
        & 0.54705e-01_rb,0.14435e+00_rb,0.16539e+00_rb,0.17004e+00_rb,0.12479e+00_rb /)
      kbo(:, 2,24,10) = (/ &
        & 0.59055e-01_rb,0.15781e+00_rb,0.18024e+00_rb,0.18505e+00_rb,0.13663e+00_rb /)
      kbo(:, 3,24,10) = (/ &
        & 0.63456e-01_rb,0.17157e+00_rb,0.19662e+00_rb,0.20113e+00_rb,0.14915e+00_rb /)
      kbo(:, 4,24,10) = (/ &
        & 0.67778e-01_rb,0.18520e+00_rb,0.21409e+00_rb,0.21860e+00_rb,0.16222e+00_rb /)
      kbo(:, 5,24,10) = (/ &
        & 0.71741e-01_rb,0.19852e+00_rb,0.23252e+00_rb,0.23726e+00_rb,0.17594e+00_rb /)
      kbo(:, 1,25,10) = (/ &
        & 0.47972e-01_rb,0.13725e+00_rb,0.15860e+00_rb,0.16297e+00_rb,0.11866e+00_rb /)
      kbo(:, 2,25,10) = (/ &
        & 0.51893e-01_rb,0.15121e+00_rb,0.17443e+00_rb,0.17877e+00_rb,0.13085e+00_rb /)
      kbo(:, 3,25,10) = (/ &
        & 0.55832e-01_rb,0.16535e+00_rb,0.19157e+00_rb,0.19578e+00_rb,0.14380e+00_rb /)
      kbo(:, 4,25,10) = (/ &
        & 0.59570e-01_rb,0.17943e+00_rb,0.20993e+00_rb,0.21423e+00_rb,0.15728e+00_rb /)
      kbo(:, 5,25,10) = (/ &
        & 0.62996e-01_rb,0.19323e+00_rb,0.22909e+00_rb,0.23401e+00_rb,0.17147e+00_rb /)
      kbo(:, 1,26,10) = (/ &
        & 0.42166e-01_rb,0.13173e+00_rb,0.15395e+00_rb,0.15801e+00_rb,0.11401e+00_rb /)
      kbo(:, 2,26,10) = (/ &
        & 0.45660e-01_rb,0.14616e+00_rb,0.17062e+00_rb,0.17468e+00_rb,0.12664e+00_rb /)
      kbo(:, 3,26,10) = (/ &
        & 0.49079e-01_rb,0.16071e+00_rb,0.18859e+00_rb,0.19278e+00_rb,0.14002e+00_rb /)
      kbo(:, 4,26,10) = (/ &
        & 0.52306e-01_rb,0.17521e+00_rb,0.20782e+00_rb,0.21223e+00_rb,0.15402e+00_rb /)
      kbo(:, 5,26,10) = (/ &
        & 0.55285e-01_rb,0.18955e+00_rb,0.22765e+00_rb,0.23299e+00_rb,0.16864e+00_rb /)
      kbo(:, 1,27,10) = (/ &
        & 0.37040e-01_rb,0.12755e+00_rb,0.15088e+00_rb,0.15474e+00_rb,0.11059e+00_rb /)
      kbo(:, 2,27,10) = (/ &
        & 0.40121e-01_rb,0.14241e+00_rb,0.16845e+00_rb,0.17243e+00_rb,0.12369e+00_rb /)
      kbo(:, 3,27,10) = (/ &
        & 0.43101e-01_rb,0.15742e+00_rb,0.18726e+00_rb,0.19156e+00_rb,0.13753e+00_rb /)
      kbo(:, 4,27,10) = (/ &
        & 0.45931e-01_rb,0.17241e+00_rb,0.20722e+00_rb,0.21200e+00_rb,0.15204e+00_rb /)
      kbo(:, 5,27,10) = (/ &
        & 0.48598e-01_rb,0.18733e+00_rb,0.22772e+00_rb,0.23361e+00_rb,0.16711e+00_rb /)
      kbo(:, 1,28,10) = (/ &
        & 0.32582e-01_rb,0.12457e+00_rb,0.14931e+00_rb,0.15310e+00_rb,0.10829e+00_rb /)
      kbo(:, 2,28,10) = (/ &
        & 0.35269e-01_rb,0.13987e+00_rb,0.16776e+00_rb,0.17185e+00_rb,0.12185e+00_rb /)
      kbo(:, 3,28,10) = (/ &
        & 0.37905e-01_rb,0.15538e+00_rb,0.18740e+00_rb,0.19192e+00_rb,0.13621e+00_rb /)
      kbo(:, 4,28,10) = (/ &
        & 0.40411e-01_rb,0.17101e+00_rb,0.20797e+00_rb,0.21319e+00_rb,0.15122e+00_rb /)
      kbo(:, 5,28,10) = (/ &
        & 0.42843e-01_rb,0.18643e+00_rb,0.22906e+00_rb,0.23551e+00_rb,0.16683e+00_rb /)
      kbo(:, 1,29,10) = (/ &
        & 0.28694e-01_rb,0.12285e+00_rb,0.14922e+00_rb,0.15309e+00_rb,0.10715e+00_rb /)
      kbo(:, 2,29,10) = (/ &
        & 0.31078e-01_rb,0.13867e+00_rb,0.16851e+00_rb,0.17281e+00_rb,0.12115e+00_rb /)
      kbo(:, 3,29,10) = (/ &
        & 0.33398e-01_rb,0.15475e+00_rb,0.18886e+00_rb,0.19373e+00_rb,0.13609e+00_rb /)
      kbo(:, 4,29,10) = (/ &
        & 0.35655e-01_rb,0.17087e+00_rb,0.21002e+00_rb,0.21569e+00_rb,0.15161e+00_rb /)
      kbo(:, 5,29,10) = (/ &
        & 0.37934e-01_rb,0.18672e+00_rb,0.23156e+00_rb,0.23855e+00_rb,0.16767e+00_rb /)
      kbo(:, 1,30,10) = (/ &
        & 0.25318e-01_rb,0.12223e+00_rb,0.15038e+00_rb,0.15442e+00_rb,0.10692e+00_rb /)
      kbo(:, 2,30,10) = (/ &
        & 0.27416e-01_rb,0.13863e+00_rb,0.17040e+00_rb,0.17493e+00_rb,0.12146e+00_rb /)
      kbo(:, 3,30,10) = (/ &
        & 0.29487e-01_rb,0.15524e+00_rb,0.19136e+00_rb,0.19656e+00_rb,0.13693e+00_rb /)
      kbo(:, 4,30,10) = (/ &
        & 0.31585e-01_rb,0.17178e+00_rb,0.21298e+00_rb,0.21911e+00_rb,0.15294e+00_rb /)
      kbo(:, 5,30,10) = (/ &
        & 0.33743e-01_rb,0.18794e+00_rb,0.23489e+00_rb,0.24234e+00_rb,0.16937e+00_rb /)
      kbo(:, 1,31,10) = (/ &
        & 0.22384e-01_rb,0.12276e+00_rb,0.15268e+00_rb,0.15694e+00_rb,0.10764e+00_rb /)
      kbo(:, 2,31,10) = (/ &
        & 0.24241e-01_rb,0.13972e+00_rb,0.17333e+00_rb,0.17809e+00_rb,0.12275e+00_rb /)
      kbo(:, 3,31,10) = (/ &
        & 0.26155e-01_rb,0.15678e+00_rb,0.19481e+00_rb,0.20028e+00_rb,0.13870e+00_rb /)
      kbo(:, 4,31,10) = (/ &
        & 0.28119e-01_rb,0.17362e+00_rb,0.21675e+00_rb,0.22330e+00_rb,0.15510e+00_rb /)
      kbo(:, 5,31,10) = (/ &
        & 0.30218e-01_rb,0.19000e+00_rb,0.23891e+00_rb,0.24680e+00_rb,0.17182e+00_rb /)
      kbo(:, 1,32,10) = (/ &
        & 0.19818e-01_rb,0.12431e+00_rb,0.15590e+00_rb,0.16037e+00_rb,0.10919e+00_rb /)
      kbo(:, 2,32,10) = (/ &
        & 0.21499e-01_rb,0.14174e+00_rb,0.17710e+00_rb,0.18207e+00_rb,0.12484e+00_rb /)
      kbo(:, 3,32,10) = (/ &
        & 0.23289e-01_rb,0.15910e+00_rb,0.19892e+00_rb,0.20468e+00_rb,0.14116e+00_rb /)
      kbo(:, 4,32,10) = (/ &
        & 0.25197e-01_rb,0.17612e+00_rb,0.22110e+00_rb,0.22808e+00_rb,0.15787e+00_rb /)
      kbo(:, 5,32,10) = (/ &
        & 0.27257e-01_rb,0.19267e+00_rb,0.24341e+00_rb,0.25172e+00_rb,0.17483e+00_rb /)
      kbo(:, 1,33,10) = (/ &
        & 0.17588e-01_rb,0.12670e+00_rb,0.15984e+00_rb,0.16448e+00_rb,0.11147e+00_rb /)
      kbo(:, 2,33,10) = (/ &
        & 0.19157e-01_rb,0.14450e+00_rb,0.18143e+00_rb,0.18665e+00_rb,0.12758e+00_rb /)
      kbo(:, 3,33,10) = (/ &
        & 0.20859e-01_rb,0.16204e+00_rb,0.20353e+00_rb,0.20957e+00_rb,0.14421e+00_rb /)
      kbo(:, 4,33,10) = (/ &
        & 0.22735e-01_rb,0.17915e+00_rb,0.22585e+00_rb,0.23324e+00_rb,0.16113e+00_rb /)
      kbo(:, 5,33,10) = (/ &
        & 0.24799e-01_rb,0.19574e+00_rb,0.24826e+00_rb,0.25697e+00_rb,0.17829e+00_rb /)
      kbo(:, 1,34,10) = (/ &
        & 0.15617e-01_rb,0.12913e+00_rb,0.16354e+00_rb,0.16830e+00_rb,0.11378e+00_rb /)
      kbo(:, 2,34,10) = (/ &
        & 0.17096e-01_rb,0.14716e+00_rb,0.18538e+00_rb,0.19084e+00_rb,0.13021e+00_rb /)
      kbo(:, 3,34,10) = (/ &
        & 0.18753e-01_rb,0.16482e+00_rb,0.20769e+00_rb,0.21402e+00_rb,0.14705e+00_rb /)
      kbo(:, 4,34,10) = (/ &
        & 0.20625e-01_rb,0.18194e+00_rb,0.23012e+00_rb,0.23784e+00_rb,0.16411e+00_rb /)
      kbo(:, 5,34,10) = (/ &
        & 0.22709e-01_rb,0.19848e+00_rb,0.25256e+00_rb,0.26160e+00_rb,0.18143e+00_rb /)
      kbo(:, 1,35,10) = (/ &
        & 0.13789e-01_rb,0.13026e+00_rb,0.16543e+00_rb,0.17032e+00_rb,0.11490e+00_rb /)
      kbo(:, 2,35,10) = (/ &
        & 0.15216e-01_rb,0.14846e+00_rb,0.18743e+00_rb,0.19307e+00_rb,0.13153e+00_rb /)
      kbo(:, 3,35,10) = (/ &
        & 0.16842e-01_rb,0.16621e+00_rb,0.20987e+00_rb,0.21641e+00_rb,0.14851e+00_rb /)
      kbo(:, 4,35,10) = (/ &
        & 0.18686e-01_rb,0.18333e+00_rb,0.23239e+00_rb,0.24032e+00_rb,0.16565e+00_rb /)
      kbo(:, 5,35,10) = (/ &
        & 0.20799e-01_rb,0.19983e+00_rb,0.25487e+00_rb,0.26409e+00_rb,0.18307e+00_rb /)
      kbo(:, 1,36,10) = (/ &
        & 0.12099e-01_rb,0.12969e+00_rb,0.16517e+00_rb,0.17011e+00_rb,0.11448e+00_rb /)
      kbo(:, 2,36,10) = (/ &
        & 0.13456e-01_rb,0.14803e+00_rb,0.18723e+00_rb,0.19293e+00_rb,0.13120e+00_rb /)
      kbo(:, 3,36,10) = (/ &
        & 0.15032e-01_rb,0.16587e+00_rb,0.20973e+00_rb,0.21635e+00_rb,0.14826e+00_rb /)
      kbo(:, 4,36,10) = (/ &
        & 0.16854e-01_rb,0.18305e+00_rb,0.23232e+00_rb,0.24032e+00_rb,0.16546e+00_rb /)
      kbo(:, 5,36,10) = (/ &
        & 0.18993e-01_rb,0.19959e+00_rb,0.25485e+00_rb,0.26414e+00_rb,0.18292e+00_rb /)
      kbo(:, 1,37,10) = (/ &
        & 0.10461e-01_rb,0.12652e+00_rb,0.16173e+00_rb,0.16663e+00_rb,0.11173e+00_rb /)
      kbo(:, 2,37,10) = (/ &
        & 0.11729e-01_rb,0.14498e+00_rb,0.18379e+00_rb,0.18940e+00_rb,0.12844e+00_rb /)
      kbo(:, 3,37,10) = (/ &
        & 0.13230e-01_rb,0.16298e+00_rb,0.20628e+00_rb,0.21278e+00_rb,0.14553e+00_rb /)
      kbo(:, 4,37,10) = (/ &
        & 0.15005e-01_rb,0.18033e+00_rb,0.22891e+00_rb,0.23679e+00_rb,0.16275e+00_rb /)
      kbo(:, 5,37,10) = (/ &
        & 0.17126e-01_rb,0.19699e+00_rb,0.25151e+00_rb,0.26065e+00_rb,0.18019e+00_rb /)
      kbo(:, 1,38,10) = (/ &
        & 0.90496e-02_rb,0.12333e+00_rb,0.15823e+00_rb,0.16307e+00_rb,0.10897e+00_rb /)
      kbo(:, 2,38,10) = (/ &
        & 0.10227e-01_rb,0.14188e+00_rb,0.18026e+00_rb,0.18578e+00_rb,0.12565e+00_rb /)
      kbo(:, 3,38,10) = (/ &
        & 0.11665e-01_rb,0.16002e+00_rb,0.20273e+00_rb,0.20911e+00_rb,0.14274e+00_rb /)
      kbo(:, 4,38,10) = (/ &
        & 0.13392e-01_rb,0.17754e+00_rb,0.22537e+00_rb,0.23309e+00_rb,0.15996e+00_rb /)
      kbo(:, 5,38,10) = (/ &
        & 0.15506e-01_rb,0.19433e+00_rb,0.24804e+00_rb,0.25702e+00_rb,0.17741e+00_rb /)
      kbo(:, 1,39,10) = (/ &
        & 0.78338e-02_rb,0.12027e+00_rb,0.15482e+00_rb,0.15961e+00_rb,0.10632e+00_rb /)
      kbo(:, 2,39,10) = (/ &
        & 0.89378e-02_rb,0.13888e+00_rb,0.17682e+00_rb,0.18224e+00_rb,0.12295e+00_rb /)
      kbo(:, 3,39,10) = (/ &
        & 0.10310e-01_rb,0.15714e+00_rb,0.19926e+00_rb,0.20553e+00_rb,0.14003e+00_rb /)
      kbo(:, 4,39,10) = (/ &
        & 0.12002e-01_rb,0.17480e+00_rb,0.22192e+00_rb,0.22947e+00_rb,0.15725e+00_rb /)
      kbo(:, 5,39,10) = (/ &
        & 0.14105e-01_rb,0.19172e+00_rb,0.24464e+00_rb,0.25344e+00_rb,0.17470e+00_rb /)
      kbo(:, 1,40,10) = (/ &
        & 0.66914e-02_rb,0.11542e+00_rb,0.14925e+00_rb,0.15395e+00_rb,0.10210e+00_rb /)
      kbo(:, 2,40,10) = (/ &
        & 0.77047e-02_rb,0.13403e+00_rb,0.17118e+00_rb,0.17643e+00_rb,0.11860e+00_rb /)
      kbo(:, 3,40,10) = (/ &
        & 0.89835e-02_rb,0.15246e+00_rb,0.19353e+00_rb,0.19961e+00_rb,0.13564e+00_rb /)
      kbo(:, 4,40,10) = (/ &
        & 0.10606e-01_rb,0.17030e+00_rb,0.21619e+00_rb,0.22340e+00_rb,0.15283e+00_rb /)
      kbo(:, 5,40,10) = (/ &
        & 0.12648e-01_rb,0.18743e+00_rb,0.23893e+00_rb,0.24746e+00_rb,0.17023e+00_rb /)
      kbo(:, 1,41,10) = (/ &
        & 0.57043e-02_rb,0.11041e+00_rb,0.14350e+00_rb,0.14812e+00_rb,0.97770e-01_rb /)
      kbo(:, 2,41,10) = (/ &
        & 0.66329e-02_rb,0.12897e+00_rb,0.16532e+00_rb,0.17041e+00_rb,0.11409e+00_rb /)
      kbo(:, 3,41,10) = (/ &
        & 0.78105e-02_rb,0.14755e+00_rb,0.18757e+00_rb,0.19344e+00_rb,0.13104e+00_rb /)
      kbo(:, 4,41,10) = (/ &
        & 0.93536e-02_rb,0.16558e+00_rb,0.21020e+00_rb,0.21707e+00_rb,0.14824e+00_rb /)
      kbo(:, 5,41,10) = (/ &
        & 0.11328e-01_rb,0.18290e+00_rb,0.23295e+00_rb,0.24119e+00_rb,0.16557e+00_rb /)
      kbo(:, 1,42,10) = (/ &
        & 0.48607e-02_rb,0.10547e+00_rb,0.13783e+00_rb,0.14238e+00_rb,0.93542e-01_rb /)
      kbo(:, 2,42,10) = (/ &
        & 0.56953e-02_rb,0.12396e+00_rb,0.15948e+00_rb,0.16442e+00_rb,0.10965e+00_rb /)
      kbo(:, 3,42,10) = (/ &
        & 0.67857e-02_rb,0.14262e+00_rb,0.18165e+00_rb,0.18732e+00_rb,0.12648e+00_rb /)
      kbo(:, 4,42,10) = (/ &
        & 0.82516e-02_rb,0.16083e+00_rb,0.20424e+00_rb,0.21080e+00_rb,0.14367e+00_rb /)
      kbo(:, 5,42,10) = (/ &
        & 0.10149e-01_rb,0.17836e+00_rb,0.22696e+00_rb,0.23490e+00_rb,0.16095e+00_rb /)
      kbo(:, 1,43,10) = (/ &
        & 0.40934e-02_rb,0.99600e-01_rb,0.13106e+00_rb,0.13554e+00_rb,0.88531e-01_rb /)
      kbo(:, 2,43,10) = (/ &
        & 0.48473e-02_rb,0.11796e+00_rb,0.15249e+00_rb,0.15728e+00_rb,0.10438e+00_rb /)
      kbo(:, 3,43,10) = (/ &
        & 0.58407e-02_rb,0.13665e+00_rb,0.17453e+00_rb,0.17995e+00_rb,0.12102e+00_rb /)
      kbo(:, 4,43,10) = (/ &
        & 0.71922e-02_rb,0.15505e+00_rb,0.19702e+00_rb,0.20328e+00_rb,0.13817e+00_rb /)
      kbo(:, 5,43,10) = (/ &
        & 0.89819e-02_rb,0.17282e+00_rb,0.21974e+00_rb,0.22722e+00_rb,0.15541e+00_rb /)
      kbo(:, 1,44,10) = (/ &
        & 0.34279e-02_rb,0.93376e-01_rb,0.12386e+00_rb,0.12828e+00_rb,0.83256e-01_rb /)
      kbo(:, 2,44,10) = (/ &
        & 0.40981e-02_rb,0.11153e+00_rb,0.14501e+00_rb,0.14969e+00_rb,0.98786e-01_rb /)
      kbo(:, 3,44,10) = (/ &
        & 0.49801e-02_rb,0.13017e+00_rb,0.16691e+00_rb,0.17209e+00_rb,0.11523e+00_rb /)
      kbo(:, 4,44,10) = (/ &
        & 0.62140e-02_rb,0.14877e+00_rb,0.18924e+00_rb,0.19525e+00_rb,0.13223e+00_rb /)
      kbo(:, 5,44,10) = (/ &
        & 0.78865e-02_rb,0.16678e+00_rb,0.21196e+00_rb,0.21896e+00_rb,0.14947e+00_rb /)
      kbo(:, 1,45,10) = (/ &
        & 0.28724e-02_rb,0.87295e-01_rb,0.11671e+00_rb,0.12114e+00_rb,0.78089e-01_rb /)
      kbo(:, 2,45,10) = (/ &
        & 0.34468e-02_rb,0.10517e+00_rb,0.13765e+00_rb,0.14225e+00_rb,0.93322e-01_rb /)
      kbo(:, 3,45,10) = (/ &
        & 0.42450e-02_rb,0.12371e+00_rb,0.15937e+00_rb,0.16431e+00_rb,0.10950e+00_rb /)
      kbo(:, 4,45,10) = (/ &
        & 0.53608e-02_rb,0.14242e+00_rb,0.18156e+00_rb,0.18727e+00_rb,0.12633e+00_rb /)
      kbo(:, 5,45,10) = (/ &
        & 0.69069e-02_rb,0.16067e+00_rb,0.20420e+00_rb,0.21079e+00_rb,0.14356e+00_rb /)
      kbo(:, 1,46,10) = (/ &
        & 0.23836e-02_rb,0.81023e-01_rb,0.10917e+00_rb,0.11375e+00_rb,0.72722e-01_rb /)
      kbo(:, 2,46,10) = (/ &
        & 0.28929e-02_rb,0.98528e-01_rb,0.12997e+00_rb,0.13446e+00_rb,0.87653e-01_rb /)
      kbo(:, 3,46,10) = (/ &
        & 0.35850e-02_rb,0.11690e+00_rb,0.15139e+00_rb,0.15617e+00_rb,0.10350e+00_rb /)
      kbo(:, 4,46,10) = (/ &
        & 0.45850e-02_rb,0.13564e+00_rb,0.17345e+00_rb,0.17887e+00_rb,0.12014e+00_rb /)
      kbo(:, 5,46,10) = (/ &
        & 0.59899e-02_rb,0.15409e+00_rb,0.19595e+00_rb,0.20220e+00_rb,0.13729e+00_rb /)
      kbo(:, 1,47,10) = (/ &
        & 0.19470e-02_rb,0.74238e-01_rb,0.10082e+00_rb,0.10569e+00_rb,0.66936e-01_rb /)
      kbo(:, 2,47,10) = (/ &
        & 0.23890e-02_rb,0.91244e-01_rb,0.12148e+00_rb,0.12591e+00_rb,0.81479e-01_rb /)
      kbo(:, 3,47,10) = (/ &
        & 0.29827e-02_rb,0.10934e+00_rb,0.14258e+00_rb,0.14724e+00_rb,0.96940e-01_rb /)
      kbo(:, 4,47,10) = (/ &
        & 0.38583e-02_rb,0.12800e+00_rb,0.16447e+00_rb,0.16958e+00_rb,0.11333e+00_rb /)
      kbo(:, 5,47,10) = (/ &
        & 0.51142e-02_rb,0.14666e+00_rb,0.18678e+00_rb,0.19270e+00_rb,0.13028e+00_rb /)
      kbo(:, 1,48,10) = (/ &
        & 0.15861e-02_rb,0.67739e-01_rb,0.92637e-01_rb,0.97854e-01_rb,0.61349e-01_rb /)
      kbo(:, 2,48,10) = (/ &
        & 0.19649e-02_rb,0.84157e-01_rb,0.11305e+00_rb,0.11755e+00_rb,0.75432e-01_rb /)
      kbo(:, 3,48,10) = (/ &
        & 0.24852e-02_rb,0.10189e+00_rb,0.13395e+00_rb,0.13849e+00_rb,0.90556e-01_rb /)
      kbo(:, 4,48,10) = (/ &
        & 0.32371e-02_rb,0.12039e+00_rb,0.15554e+00_rb,0.16041e+00_rb,0.10658e+00_rb /)
      kbo(:, 5,48,10) = (/ &
        & 0.43439e-02_rb,0.13914e+00_rb,0.17770e+00_rb,0.18327e+00_rb,0.12334e+00_rb /)
      kbo(:, 1,49,10) = (/ &
        & 0.12857e-02_rb,0.61441e-01_rb,0.84638e-01_rb,0.90184e-01_rb,0.55928e-01_rb /)
      kbo(:, 2,49,10) = (/ &
        & 0.16127e-02_rb,0.77337e-01_rb,0.10472e+00_rb,0.10945e+00_rb,0.69577e-01_rb /)
      kbo(:, 3,49,10) = (/ &
        & 0.20537e-02_rb,0.94575e-01_rb,0.12545e+00_rb,0.12989e+00_rb,0.84331e-01_rb /)
      kbo(:, 4,49,10) = (/ &
        & 0.26910e-02_rb,0.11284e+00_rb,0.14670e+00_rb,0.15143e+00_rb,0.99981e-01_rb /)
      kbo(:, 5,49,10) = (/ &
        & 0.36640e-02_rb,0.13156e+00_rb,0.16870e+00_rb,0.17395e+00_rb,0.11651e+00_rb /)
      kbo(:, 1,50,10) = (/ &
        & 0.10457e-02_rb,0.55773e-01_rb,0.77388e-01_rb,0.83215e-01_rb,0.51035e-01_rb /)
      kbo(:, 2,50,10) = (/ &
        & 0.13145e-02_rb,0.71139e-01_rb,0.96987e-01_rb,0.10202e+00_rb,0.64279e-01_rb /)
      kbo(:, 3,50,10) = (/ &
        & 0.17051e-02_rb,0.87902e-01_rb,0.11756e+00_rb,0.12199e+00_rb,0.78626e-01_rb /)
      kbo(:, 4,50,10) = (/ &
        & 0.22648e-02_rb,0.10582e+00_rb,0.13854e+00_rb,0.14317e+00_rb,0.93933e-01_rb /)
      kbo(:, 5,50,10) = (/ &
        & 0.31061e-02_rb,0.12442e+00_rb,0.16033e+00_rb,0.16532e+00_rb,0.11017e+00_rb /)
      kbo(:, 1,51,10) = (/ &
        & 0.84549e-03_rb,0.50472e-01_rb,0.70528e-01_rb,0.76615e-01_rb,0.46420e-01_rb /)
      kbo(:, 2,51,10) = (/ &
        & 0.10690e-02_rb,0.65329e-01_rb,0.89631e-01_rb,0.94967e-01_rb,0.59271e-01_rb /)
      kbo(:, 3,51,10) = (/ &
        & 0.14006e-02_rb,0.81572e-01_rb,0.10994e+00_rb,0.11450e+00_rb,0.73207e-01_rb /)
      kbo(:, 4,51,10) = (/ &
        & 0.18893e-02_rb,0.99123e-01_rb,0.13077e+00_rb,0.13529e+00_rb,0.88203e-01_rb /)
      kbo(:, 5,51,10) = (/ &
        & 0.26211e-02_rb,0.11756e+00_rb,0.15226e+00_rb,0.15707e+00_rb,0.10410e+00_rb /)
      kbo(:, 1,52,10) = (/ &
        & 0.67948e-03_rb,0.45452e-01_rb,0.63925e-01_rb,0.70191e-01_rb,0.41984e-01_rb /)
      kbo(:, 2,52,10) = (/ &
        & 0.86532e-03_rb,0.59697e-01_rb,0.82454e-01_rb,0.88085e-01_rb,0.54427e-01_rb /)
      kbo(:, 3,52,10) = (/ &
        & 0.11431e-02_rb,0.75434e-01_rb,0.10239e+00_rb,0.10721e+00_rb,0.67964e-01_rb /)
      kbo(:, 4,52,10) = (/ &
        & 0.15771e-02_rb,0.92585e-01_rb,0.12313e+00_rb,0.12757e+00_rb,0.82618e-01_rb /)
      kbo(:, 5,52,10) = (/ &
        & 0.22169e-02_rb,0.11074e+00_rb,0.14429e+00_rb,0.14899e+00_rb,0.98166e-01_rb /)
      kbo(:, 1,53,10) = (/ &
        & 0.54404e-03_rb,0.40689e-01_rb,0.57565e-01_rb,0.63870e-01_rb,0.37709e-01_rb /)
      kbo(:, 2,53,10) = (/ &
        & 0.69516e-03_rb,0.54249e-01_rb,0.75469e-01_rb,0.81366e-01_rb,0.49728e-01_rb /)
      kbo(:, 3,53,10) = (/ &
        & 0.92495e-03_rb,0.69505e-01_rb,0.94951e-01_rb,0.10007e+00_rb,0.62880e-01_rb /)
      kbo(:, 4,53,10) = (/ &
        & 0.12935e-02_rb,0.86139e-01_rb,0.11548e+00_rb,0.11993e+00_rb,0.77126e-01_rb /)
      kbo(:, 5,53,10) = (/ &
        & 0.18603e-02_rb,0.10397e+00_rb,0.13641e+00_rb,0.14102e+00_rb,0.92352e-01_rb /)
      kbo(:, 1,54,10) = (/ &
        & 0.43618e-03_rb,0.36557e-01_rb,0.51984e-01_rb,0.58236e-01_rb,0.33960e-01_rb /)
      kbo(:, 2,54,10) = (/ &
        & 0.56166e-03_rb,0.49456e-01_rb,0.69241e-01_rb,0.75371e-01_rb,0.45540e-01_rb /)
      kbo(:, 3,54,10) = (/ &
        & 0.75035e-03_rb,0.64217e-01_rb,0.88252e-01_rb,0.93648e-01_rb,0.58325e-01_rb /)
      kbo(:, 4,54,10) = (/ &
        & 0.10628e-02_rb,0.80385e-01_rb,0.10853e+00_rb,0.11313e+00_rb,0.72201e-01_rb /)
      kbo(:, 5,54,10) = (/ &
        & 0.15600e-02_rb,0.97867e-01_rb,0.12933e+00_rb,0.13384e+00_rb,0.87138e-01_rb /)
      kbo(:, 1,55,10) = (/ &
        & 0.34930e-03_rb,0.32793e-01_rb,0.46881e-01_rb,0.52981e-01_rb,0.30517e-01_rb /)
      kbo(:, 2,55,10) = (/ &
        & 0.45330e-03_rb,0.45087e-01_rb,0.63469e-01_rb,0.69744e-01_rb,0.41669e-01_rb /)
      kbo(:, 3,55,10) = (/ &
        & 0.61016e-03_rb,0.59300e-01_rb,0.81969e-01_rb,0.87622e-01_rb,0.54097e-01_rb /)
      kbo(:, 4,55,10) = (/ &
        & 0.87186e-03_rb,0.75017e-01_rb,0.10190e+00_rb,0.10674e+00_rb,0.67613e-01_rb /)
      kbo(:, 5,55,10) = (/ &
        & 0.13255e-02_rb,0.92144e-01_rb,0.12263e+00_rb,0.12707e+00_rb,0.82251e-01_rb /)
      kbo(:, 1,56,10) = (/ &
        & 0.27869e-03_rb,0.29255e-01_rb,0.42021e-01_rb,0.47895e-01_rb,0.27250e-01_rb /)
      kbo(:, 2,56,10) = (/ &
        & 0.36450e-03_rb,0.40920e-01_rb,0.57903e-01_rb,0.64209e-01_rb,0.37931e-01_rb /)
      kbo(:, 3,56,10) = (/ &
        & 0.49136e-03_rb,0.54534e-01_rb,0.75862e-01_rb,0.81747e-01_rb,0.49990e-01_rb /)
      kbo(:, 4,56,10) = (/ &
        & 0.71172e-03_rb,0.69828e-01_rb,0.95390e-01_rb,0.10049e+00_rb,0.63168e-01_rb /)
      kbo(:, 5,56,10) = (/ &
        & 0.11017e-02_rb,0.86502e-01_rb,0.11594e+00_rb,0.12039e+00_rb,0.77442e-01_rb /)
      kbo(:, 1,57,10) = (/ &
        & 0.22107e-03_rb,0.25925e-01_rb,0.37436e-01_rb,0.42992e-01_rb,0.24134e-01_rb /)
      kbo(:, 2,57,10) = (/ &
        & 0.29170e-03_rb,0.36963e-01_rb,0.52553e-01_rb,0.58819e-01_rb,0.34341e-01_rb /)
      kbo(:, 3,57,10) = (/ &
        & 0.39477e-03_rb,0.49943e-01_rb,0.69900e-01_rb,0.76011e-01_rb,0.45979e-01_rb /)
      kbo(:, 4,57,10) = (/ &
        & 0.57598e-03_rb,0.64771e-01_rb,0.88986e-01_rb,0.94351e-01_rb,0.58814e-01_rb /)
      kbo(:, 5,57,10) = (/ &
        & 0.90995e-03_rb,0.80998e-01_rb,0.10929e+00_rb,0.11387e+00_rb,0.72731e-01_rb /)
      kbo(:, 1,58,10) = (/ &
        & 0.17561e-03_rb,0.22952e-01_rb,0.33334e-01_rb,0.38531e-01_rb,0.21328e-01_rb /)
      kbo(:, 2,58,10) = (/ &
        & 0.23328e-03_rb,0.33378e-01_rb,0.47700e-01_rb,0.53835e-01_rb,0.31067e-01_rb /)
      kbo(:, 3,58,10) = (/ &
        & 0.31697e-03_rb,0.45784e-01_rb,0.64416e-01_rb,0.70683e-01_rb,0.42303e-01_rb /)
      kbo(:, 4,58,10) = (/ &
        & 0.46537e-03_rb,0.60108e-01_rb,0.83029e-01_rb,0.88636e-01_rb,0.54797e-01_rb /)
      kbo(:, 5,58,10) = (/ &
        & 0.74786e-03_rb,0.75908e-01_rb,0.10302e+00_rb,0.10782e+00_rb,0.68377e-01_rb /)
      kbo(:, 1,59,10) = (/ &
        & 0.14648e-03_rb,0.21806e-01_rb,0.31752e-01_rb,0.36796e-01_rb,0.20240e-01_rb /)
      kbo(:, 2,59,10) = (/ &
        & 0.19618e-03_rb,0.31995e-01_rb,0.45805e-01_rb,0.51875e-01_rb,0.29797e-01_rb /)
      kbo(:, 3,59,10) = (/ &
        & 0.27063e-03_rb,0.44166e-01_rb,0.62268e-01_rb,0.68564e-01_rb,0.40859e-01_rb /)
      kbo(:, 4,59,10) = (/ &
        & 0.40629e-03_rb,0.58271e-01_rb,0.80681e-01_rb,0.86380e-01_rb,0.53223e-01_rb /)
      kbo(:, 5,59,10) = (/ &
        & 0.67143e-03_rb,0.73904e-01_rb,0.10053e+00_rb,0.10542e+00_rb,0.66665e-01_rb /)
      kbo(:, 1,13,11) = (/ &
        & 0.43607e+00_rb,0.62808e+00_rb,0.71724e+00_rb,0.73315e+00_rb,0.60101e+00_rb /)
      kbo(:, 2,13,11) = (/ &
        & 0.43332e+00_rb,0.62604e+00_rb,0.71980e+00_rb,0.74002e+00_rb,0.61186e+00_rb /)
      kbo(:, 3,13,11) = (/ &
        & 0.43924e+00_rb,0.62431e+00_rb,0.72116e+00_rb,0.74635e+00_rb,0.62343e+00_rb /)
      kbo(:, 4,13,11) = (/ &
        & 0.44984e+00_rb,0.62482e+00_rb,0.72168e+00_rb,0.75298e+00_rb,0.63261e+00_rb /)
      kbo(:, 5,13,11) = (/ &
        & 0.46047e+00_rb,0.63112e+00_rb,0.72222e+00_rb,0.75987e+00_rb,0.64021e+00_rb /)
      kbo(:, 1,14,11) = (/ &
        & 0.38105e+00_rb,0.57533e+00_rb,0.67001e+00_rb,0.68808e+00_rb,0.55899e+00_rb /)
      kbo(:, 2,14,11) = (/ &
        & 0.38509e+00_rb,0.57518e+00_rb,0.67285e+00_rb,0.69660e+00_rb,0.57227e+00_rb /)
      kbo(:, 3,14,11) = (/ &
        & 0.39449e+00_rb,0.57649e+00_rb,0.67532e+00_rb,0.70510e+00_rb,0.58361e+00_rb /)
      kbo(:, 4,14,11) = (/ &
        & 0.40382e+00_rb,0.58297e+00_rb,0.67784e+00_rb,0.71417e+00_rb,0.59305e+00_rb /)
      kbo(:, 5,14,11) = (/ &
        & 0.41367e+00_rb,0.59430e+00_rb,0.68136e+00_rb,0.72200e+00_rb,0.60202e+00_rb /)
      kbo(:, 1,15,11) = (/ &
        & 0.33634e+00_rb,0.52577e+00_rb,0.62172e+00_rb,0.64383e+00_rb,0.51752e+00_rb /)
      kbo(:, 2,15,11) = (/ &
        & 0.34434e+00_rb,0.52795e+00_rb,0.62601e+00_rb,0.65389e+00_rb,0.53138e+00_rb /)
      kbo(:, 3,15,11) = (/ &
        & 0.35278e+00_rb,0.53440e+00_rb,0.63064e+00_rb,0.66483e+00_rb,0.54288e+00_rb /)
      kbo(:, 4,15,11) = (/ &
        & 0.36194e+00_rb,0.54564e+00_rb,0.63643e+00_rb,0.67473e+00_rb,0.55371e+00_rb /)
      kbo(:, 5,15,11) = (/ &
        & 0.37392e+00_rb,0.55836e+00_rb,0.64562e+00_rb,0.68317e+00_rb,0.56497e+00_rb /)
      kbo(:, 1,16,11) = (/ &
        & 0.29938e+00_rb,0.48063e+00_rb,0.57442e+00_rb,0.60090e+00_rb,0.47563e+00_rb /)
      kbo(:, 2,16,11) = (/ &
        & 0.30677e+00_rb,0.48684e+00_rb,0.58100e+00_rb,0.61362e+00_rb,0.49008e+00_rb /)
      kbo(:, 3,16,11) = (/ &
        & 0.31495e+00_rb,0.49797e+00_rb,0.58852e+00_rb,0.62578e+00_rb,0.50309e+00_rb /)
      kbo(:, 4,16,11) = (/ &
        & 0.32635e+00_rb,0.51101e+00_rb,0.59933e+00_rb,0.63613e+00_rb,0.51641e+00_rb /)
      kbo(:, 5,16,11) = (/ &
        & 0.34098e+00_rb,0.52383e+00_rb,0.61318e+00_rb,0.64534e+00_rb,0.52976e+00_rb /)
      kbo(:, 1,17,11) = (/ &
        & 0.26573e+00_rb,0.44123e+00_rb,0.53003e+00_rb,0.56054e+00_rb,0.43533e+00_rb /)
      kbo(:, 2,17,11) = (/ &
        & 0.27295e+00_rb,0.45155e+00_rb,0.53872e+00_rb,0.57479e+00_rb,0.45023e+00_rb /)
      kbo(:, 3,17,11) = (/ &
        & 0.28334e+00_rb,0.46499e+00_rb,0.55060e+00_rb,0.58775e+00_rb,0.46535e+00_rb /)
      kbo(:, 4,17,11) = (/ &
        & 0.29718e+00_rb,0.47854e+00_rb,0.56610e+00_rb,0.59925e+00_rb,0.48105e+00_rb /)
      kbo(:, 5,17,11) = (/ &
        & 0.31347e+00_rb,0.49215e+00_rb,0.58364e+00_rb,0.61031e+00_rb,0.49645e+00_rb /)
      kbo(:, 1,18,11) = (/ &
        & 0.23548e+00_rb,0.40722e+00_rb,0.48899e+00_rb,0.52175e+00_rb,0.39754e+00_rb /)
      kbo(:, 2,18,11) = (/ &
        & 0.24465e+00_rb,0.42091e+00_rb,0.50155e+00_rb,0.53754e+00_rb,0.41400e+00_rb /)
      kbo(:, 3,18,11) = (/ &
        & 0.25717e+00_rb,0.43494e+00_rb,0.51756e+00_rb,0.55168e+00_rb,0.43132e+00_rb /)
      kbo(:, 4,18,11) = (/ &
        & 0.27241e+00_rb,0.44915e+00_rb,0.53619e+00_rb,0.56524e+00_rb,0.44875e+00_rb /)
      kbo(:, 5,18,11) = (/ &
        & 0.28792e+00_rb,0.46398e+00_rb,0.55551e+00_rb,0.57930e+00_rb,0.46590e+00_rb /)
      kbo(:, 1,19,11) = (/ &
        & 0.20944e+00_rb,0.37826e+00_rb,0.45234e+00_rb,0.48406e+00_rb,0.36392e+00_rb /)
      kbo(:, 2,19,11) = (/ &
        & 0.22089e+00_rb,0.39288e+00_rb,0.46903e+00_rb,0.50178e+00_rb,0.38211e+00_rb /)
      kbo(:, 3,19,11) = (/ &
        & 0.23514e+00_rb,0.40761e+00_rb,0.48827e+00_rb,0.51821e+00_rb,0.40103e+00_rb /)
      kbo(:, 4,19,11) = (/ &
        & 0.24973e+00_rb,0.42303e+00_rb,0.50894e+00_rb,0.53472e+00_rb,0.42003e+00_rb /)
      kbo(:, 5,19,11) = (/ &
        & 0.26400e+00_rb,0.44033e+00_rb,0.52950e+00_rb,0.55236e+00_rb,0.43892e+00_rb /)
      kbo(:, 1,20,11) = (/ &
        & 0.18773e+00_rb,0.35288e+00_rb,0.42094e+00_rb,0.44918e+00_rb,0.33536e+00_rb /)
      kbo(:, 2,20,11) = (/ &
        & 0.20090e+00_rb,0.36792e+00_rb,0.44069e+00_rb,0.46887e+00_rb,0.35493e+00_rb /)
      kbo(:, 3,20,11) = (/ &
        & 0.21482e+00_rb,0.38362e+00_rb,0.46266e+00_rb,0.48831e+00_rb,0.37530e+00_rb /)
      kbo(:, 4,20,11) = (/ &
        & 0.22865e+00_rb,0.40117e+00_rb,0.48452e+00_rb,0.50881e+00_rb,0.39557e+00_rb /)
      kbo(:, 5,20,11) = (/ &
        & 0.24234e+00_rb,0.42087e+00_rb,0.50657e+00_rb,0.53012e+00_rb,0.41595e+00_rb /)
      kbo(:, 1,21,11) = (/ &
        & 0.16913e+00_rb,0.33019e+00_rb,0.39388e+00_rb,0.41795e+00_rb,0.31127e+00_rb /)
      kbo(:, 2,21,11) = (/ &
        & 0.18253e+00_rb,0.34600e+00_rb,0.41645e+00_rb,0.43947e+00_rb,0.33210e+00_rb /)
      kbo(:, 3,21,11) = (/ &
        & 0.19592e+00_rb,0.36350e+00_rb,0.43977e+00_rb,0.46279e+00_rb,0.35347e+00_rb /)
      kbo(:, 4,21,11) = (/ &
        & 0.20921e+00_rb,0.38317e+00_rb,0.46337e+00_rb,0.48719e+00_rb,0.37483e+00_rb /)
      kbo(:, 5,21,11) = (/ &
        & 0.22258e+00_rb,0.40476e+00_rb,0.48708e+00_rb,0.51206e+00_rb,0.39691e+00_rb /)
      kbo(:, 1,22,11) = (/ &
        & 0.15329e+00_rb,0.31121e+00_rb,0.37219e+00_rb,0.39211e+00_rb,0.29266e+00_rb /)
      kbo(:, 2,22,11) = (/ &
        & 0.16615e+00_rb,0.32878e+00_rb,0.39682e+00_rb,0.41672e+00_rb,0.31445e+00_rb /)
      kbo(:, 3,22,11) = (/ &
        & 0.17924e+00_rb,0.34835e+00_rb,0.42167e+00_rb,0.44342e+00_rb,0.33660e+00_rb /)
      kbo(:, 4,22,11) = (/ &
        & 0.19225e+00_rb,0.36990e+00_rb,0.44699e+00_rb,0.47133e+00_rb,0.35944e+00_rb /)
      kbo(:, 5,22,11) = (/ &
        & 0.20541e+00_rb,0.39323e+00_rb,0.47271e+00_rb,0.49943e+00_rb,0.38308e+00_rb /)
      kbo(:, 1,23,11) = (/ &
        & 0.13887e+00_rb,0.29534e+00_rb,0.35451e+00_rb,0.37144e+00_rb,0.27763e+00_rb /)
      kbo(:, 2,23,11) = (/ &
        & 0.15131e+00_rb,0.31502e+00_rb,0.38068e+00_rb,0.39927e+00_rb,0.30013e+00_rb /)
      kbo(:, 3,23,11) = (/ &
        & 0.16398e+00_rb,0.33651e+00_rb,0.40726e+00_rb,0.42913e+00_rb,0.32331e+00_rb /)
      kbo(:, 4,23,11) = (/ &
        & 0.17699e+00_rb,0.35967e+00_rb,0.43436e+00_rb,0.45990e+00_rb,0.34758e+00_rb /)
      kbo(:, 5,23,11) = (/ &
        & 0.19026e+00_rb,0.38467e+00_rb,0.46223e+00_rb,0.49074e+00_rb,0.37282e+00_rb /)
      kbo(:, 1,24,11) = (/ &
        & 0.12585e+00_rb,0.28281e+00_rb,0.34015e+00_rb,0.35619e+00_rb,0.26580e+00_rb /)
      kbo(:, 2,24,11) = (/ &
        & 0.13791e+00_rb,0.30436e+00_rb,0.36812e+00_rb,0.38675e+00_rb,0.28903e+00_rb /)
      kbo(:, 3,24,11) = (/ &
        & 0.15049e+00_rb,0.32769e+00_rb,0.39648e+00_rb,0.41925e+00_rb,0.31342e+00_rb /)
      kbo(:, 4,24,11) = (/ &
        & 0.16367e+00_rb,0.35256e+00_rb,0.42575e+00_rb,0.45270e+00_rb,0.33923e+00_rb /)
      kbo(:, 5,24,11) = (/ &
        & 0.17705e+00_rb,0.37855e+00_rb,0.45558e+00_rb,0.48562e+00_rb,0.36598e+00_rb /)
      kbo(:, 1,25,11) = (/ &
        & 0.11440e+00_rb,0.27340e+00_rb,0.32939e+00_rb,0.34570e+00_rb,0.25676e+00_rb /)
      kbo(:, 2,25,11) = (/ &
        & 0.12619e+00_rb,0.29667e+00_rb,0.35918e+00_rb,0.37882e+00_rb,0.28103e+00_rb /)
      kbo(:, 3,25,11) = (/ &
        & 0.13880e+00_rb,0.32182e+00_rb,0.38965e+00_rb,0.41370e+00_rb,0.30678e+00_rb /)
      kbo(:, 4,25,11) = (/ &
        & 0.15209e+00_rb,0.34806e+00_rb,0.42076e+00_rb,0.44907e+00_rb,0.33404e+00_rb /)
      kbo(:, 5,25,11) = (/ &
        & 0.16564e+00_rb,0.37478e+00_rb,0.45238e+00_rb,0.48364e+00_rb,0.36209e+00_rb /)
      kbo(:, 1,26,11) = (/ &
        & 0.10459e+00_rb,0.26698e+00_rb,0.32246e+00_rb,0.33976e+00_rb,0.25076e+00_rb /)
      kbo(:, 2,26,11) = (/ &
        & 0.11626e+00_rb,0.29209e+00_rb,0.35422e+00_rb,0.37512e+00_rb,0.27618e+00_rb /)
      kbo(:, 3,26,11) = (/ &
        & 0.12895e+00_rb,0.31885e+00_rb,0.38669e+00_rb,0.41203e+00_rb,0.30332e+00_rb /)
      kbo(:, 4,26,11) = (/ &
        & 0.14230e+00_rb,0.34607e+00_rb,0.41933e+00_rb,0.44889e+00_rb,0.33190e+00_rb /)
      kbo(:, 5,26,11) = (/ &
        & 0.15605e+00_rb,0.37346e+00_rb,0.45243e+00_rb,0.48462e+00_rb,0.36103e+00_rb /)
      kbo(:, 1,27,11) = (/ &
        & 0.96118e-01_rb,0.26319e+00_rb,0.31888e+00_rb,0.33748e+00_rb,0.24718e+00_rb /)
      kbo(:, 2,27,11) = (/ &
        & 0.10777e+00_rb,0.29006e+00_rb,0.35247e+00_rb,0.37471e+00_rb,0.27379e+00_rb /)
      kbo(:, 3,27,11) = (/ &
        & 0.12054e+00_rb,0.31789e+00_rb,0.38657e+00_rb,0.41315e+00_rb,0.30239e+00_rb /)
      kbo(:, 4,27,11) = (/ &
        & 0.13403e+00_rb,0.34597e+00_rb,0.42050e+00_rb,0.45108e+00_rb,0.33196e+00_rb /)
      kbo(:, 5,27,11) = (/ &
        & 0.14803e+00_rb,0.37409e+00_rb,0.45469e+00_rb,0.48753e+00_rb,0.36202e+00_rb /)
      kbo(:, 1,28,11) = (/ &
        & 0.88934e-01_rb,0.26189e+00_rb,0.31824e+00_rb,0.33815e+00_rb,0.24575e+00_rb /)
      kbo(:, 2,28,11) = (/ &
        & 0.10066e+00_rb,0.29004e+00_rb,0.35339e+00_rb,0.37699e+00_rb,0.27368e+00_rb /)
      kbo(:, 3,28,11) = (/ &
        & 0.11346e+00_rb,0.31878e+00_rb,0.38868e+00_rb,0.41646e+00_rb,0.30334e+00_rb /)
      kbo(:, 4,28,11) = (/ &
        & 0.12719e+00_rb,0.34746e+00_rb,0.42368e+00_rb,0.45510e+00_rb,0.33384e+00_rb /)
      kbo(:, 5,28,11) = (/ &
        & 0.14149e+00_rb,0.37622e+00_rb,0.45856e+00_rb,0.49189e+00_rb,0.36458e+00_rb /)
      kbo(:, 1,29,11) = (/ &
        & 0.82953e-01_rb,0.26276e+00_rb,0.32032e+00_rb,0.34153e+00_rb,0.24649e+00_rb /)
      kbo(:, 2,29,11) = (/ &
        & 0.94812e-01_rb,0.29186e+00_rb,0.35678e+00_rb,0.38153e+00_rb,0.27563e+00_rb /)
      kbo(:, 3,29,11) = (/ &
        & 0.10778e+00_rb,0.32138e+00_rb,0.39278e+00_rb,0.42163e+00_rb,0.30623e+00_rb /)
      kbo(:, 4,29,11) = (/ &
        & 0.12182e+00_rb,0.35053e+00_rb,0.42853e+00_rb,0.46064e+00_rb,0.33743e+00_rb /)
      kbo(:, 5,29,11) = (/ &
        & 0.13655e+00_rb,0.37990e+00_rb,0.46387e+00_rb,0.49754e+00_rb,0.36866e+00_rb /)
      kbo(:, 1,30,11) = (/ &
        & 0.78111e-01_rb,0.26530e+00_rb,0.32451e+00_rb,0.34680e+00_rb,0.24893e+00_rb /)
      kbo(:, 2,30,11) = (/ &
        & 0.90075e-01_rb,0.29517e+00_rb,0.36183e+00_rb,0.38763e+00_rb,0.27915e+00_rb /)
      kbo(:, 3,30,11) = (/ &
        & 0.10336e+00_rb,0.32522e+00_rb,0.39838e+00_rb,0.42809e+00_rb,0.31046e+00_rb /)
      kbo(:, 4,30,11) = (/ &
        & 0.11787e+00_rb,0.35483e+00_rb,0.43455e+00_rb,0.46718e+00_rb,0.34218e+00_rb /)
      kbo(:, 5,30,11) = (/ &
        & 0.13307e+00_rb,0.38452e+00_rb,0.47003e+00_rb,0.50395e+00_rb,0.37368e+00_rb /)
      kbo(:, 1,31,11) = (/ &
        & 0.74235e-01_rb,0.26927e+00_rb,0.33033e+00_rb,0.35368e+00_rb,0.25294e+00_rb /)
      kbo(:, 2,31,11) = (/ &
        & 0.86475e-01_rb,0.29980e+00_rb,0.36814e+00_rb,0.39505e+00_rb,0.28398e+00_rb /)
      kbo(:, 3,31,11) = (/ &
        & 0.10025e+00_rb,0.33020e+00_rb,0.40500e+00_rb,0.43563e+00_rb,0.31586e+00_rb /)
      kbo(:, 4,31,11) = (/ &
        & 0.11530e+00_rb,0.36008e+00_rb,0.44151e+00_rb,0.47461e+00_rb,0.34790e+00_rb /)
      kbo(:, 5,31,11) = (/ &
        & 0.13103e+00_rb,0.38989e+00_rb,0.47695e+00_rb,0.51108e+00_rb,0.37953e+00_rb /)
      kbo(:, 1,32,11) = (/ &
        & 0.71362e-01_rb,0.27445e+00_rb,0.33748e+00_rb,0.36171e+00_rb,0.25820e+00_rb /)
      kbo(:, 2,32,11) = (/ &
        & 0.84060e-01_rb,0.30547e+00_rb,0.37545e+00_rb,0.40335e+00_rb,0.28988e+00_rb /)
      kbo(:, 3,32,11) = (/ &
        & 0.98456e-01_rb,0.33600e+00_rb,0.41253e+00_rb,0.44393e+00_rb,0.32217e+00_rb /)
      kbo(:, 4,32,11) = (/ &
        & 0.11408e+00_rb,0.36612e+00_rb,0.44914e+00_rb,0.48266e+00_rb,0.35443e+00_rb /)
      kbo(:, 5,32,11) = (/ &
        & 0.13035e+00_rb,0.39590e+00_rb,0.48448e+00_rb,0.51873e+00_rb,0.38602e+00_rb /)
      kbo(:, 1,33,11) = (/ &
        & 0.69437e-01_rb,0.28056e+00_rb,0.34548e+00_rb,0.37055e+00_rb,0.26439e+00_rb /)
      kbo(:, 2,33,11) = (/ &
        & 0.82778e-01_rb,0.31179e+00_rb,0.38345e+00_rb,0.41222e+00_rb,0.29653e+00_rb /)
      kbo(:, 3,33,11) = (/ &
        & 0.97818e-01_rb,0.34234e+00_rb,0.42058e+00_rb,0.45271e+00_rb,0.32902e+00_rb /)
      kbo(:, 4,33,11) = (/ &
        & 0.11406e+00_rb,0.37264e+00_rb,0.45719e+00_rb,0.49102e+00_rb,0.36138e+00_rb /)
      kbo(:, 5,33,11) = (/ &
        & 0.13083e+00_rb,0.40228e+00_rb,0.49234e+00_rb,0.52669e+00_rb,0.39293e+00_rb /)
      kbo(:, 1,34,11) = (/ &
        & 0.68001e-01_rb,0.28611e+00_rb,0.35258e+00_rb,0.37843e+00_rb,0.27007e+00_rb /)
      kbo(:, 2,34,11) = (/ &
        & 0.82015e-01_rb,0.31742e+00_rb,0.39059e+00_rb,0.42007e+00_rb,0.30253e+00_rb /)
      kbo(:, 3,34,11) = (/ &
        & 0.97684e-01_rb,0.34800e+00_rb,0.42777e+00_rb,0.46038e+00_rb,0.33518e+00_rb /)
      kbo(:, 4,34,11) = (/ &
        & 0.11453e+00_rb,0.37836e+00_rb,0.46423e+00_rb,0.49832e+00_rb,0.36754e+00_rb /)
      kbo(:, 5,34,11) = (/ &
        & 0.13165e+00_rb,0.40799e+00_rb,0.49927e+00_rb,0.53358e+00_rb,0.39902e+00_rb /)
      kbo(:, 1,35,11) = (/ &
        & 0.66173e-01_rb,0.28901e+00_rb,0.35634e+00_rb,0.38261e+00_rb,0.27305e+00_rb /)
      kbo(:, 2,35,11) = (/ &
        & 0.80679e-01_rb,0.32036e+00_rb,0.39443e+00_rb,0.42428e+00_rb,0.30572e+00_rb /)
      kbo(:, 3,35,11) = (/ &
        & 0.96839e-01_rb,0.35097e+00_rb,0.43164e+00_rb,0.46448e+00_rb,0.33845e+00_rb /)
      kbo(:, 4,35,11) = (/ &
        & 0.11412e+00_rb,0.38139e+00_rb,0.46804e+00_rb,0.50225e+00_rb,0.37079e+00_rb /)
      kbo(:, 5,35,11) = (/ &
        & 0.13154e+00_rb,0.41104e+00_rb,0.50302e+00_rb,0.53730e+00_rb,0.40230e+00_rb /)
      kbo(:, 1,36,11) = (/ &
        & 0.63512e-01_rb,0.28876e+00_rb,0.35615e+00_rb,0.38246e+00_rb,0.27277e+00_rb /)
      kbo(:, 2,36,11) = (/ &
        & 0.78322e-01_rb,0.32017e+00_rb,0.39440e+00_rb,0.42423e+00_rb,0.30554e+00_rb /)
      kbo(:, 3,36,11) = (/ &
        & 0.94792e-01_rb,0.35084e+00_rb,0.43169e+00_rb,0.46451e+00_rb,0.33835e+00_rb /)
      kbo(:, 4,36,11) = (/ &
        & 0.11239e+00_rb,0.38129e+00_rb,0.46814e+00_rb,0.50234e+00_rb,0.37075e+00_rb /)
      kbo(:, 5,36,11) = (/ &
        & 0.13007e+00_rb,0.41105e+00_rb,0.50318e+00_rb,0.53745e+00_rb,0.40235e+00_rb /)
      kbo(:, 1,37,11) = (/ &
        & 0.59276e-01_rb,0.28388e+00_rb,0.35018e+00_rb,0.37605e+00_rb,0.26777e+00_rb /)
      kbo(:, 2,37,11) = (/ &
        & 0.74070e-01_rb,0.31543e+00_rb,0.38877e+00_rb,0.41809e+00_rb,0.30055e+00_rb /)
      kbo(:, 3,37,11) = (/ &
        & 0.90621e-01_rb,0.34621e+00_rb,0.42622e+00_rb,0.45870e+00_rb,0.33345e+00_rb /)
      kbo(:, 4,37,11) = (/ &
        & 0.10838e+00_rb,0.37677e+00_rb,0.46291e+00_rb,0.49695e+00_rb,0.36598e+00_rb /)
      kbo(:, 5,37,11) = (/ &
        & 0.12629e+00_rb,0.40664e+00_rb,0.49817e+00_rb,0.53255e+00_rb,0.39771e+00_rb /)
      kbo(:, 1,38,11) = (/ &
        & 0.55352e-01_rb,0.27885e+00_rb,0.34399e+00_rb,0.36943e+00_rb,0.26263e+00_rb /)
      kbo(:, 2,38,11) = (/ &
        & 0.70099e-01_rb,0.31053e+00_rb,0.38289e+00_rb,0.41165e+00_rb,0.29537e+00_rb /)
      kbo(:, 3,38,11) = (/ &
        & 0.86674e-01_rb,0.34144e+00_rb,0.42052e+00_rb,0.45261e+00_rb,0.32834e+00_rb /)
      kbo(:, 4,38,11) = (/ &
        & 0.10451e+00_rb,0.37207e+00_rb,0.45745e+00_rb,0.49130e+00_rb,0.36103e+00_rb /)
      kbo(:, 5,38,11) = (/ &
        & 0.12261e+00_rb,0.40208e+00_rb,0.49291e+00_rb,0.52735e+00_rb,0.39288e+00_rb /)
      kbo(:, 1,39,11) = (/ &
        & 0.51830e-01_rb,0.27388e+00_rb,0.33795e+00_rb,0.36294e+00_rb,0.25765e+00_rb /)
      kbo(:, 2,39,11) = (/ &
        & 0.66473e-01_rb,0.30570e+00_rb,0.37709e+00_rb,0.40534e+00_rb,0.29029e+00_rb /)
      kbo(:, 3,39,11) = (/ &
        & 0.83032e-01_rb,0.33678e+00_rb,0.41490e+00_rb,0.44656e+00_rb,0.32332e+00_rb /)
      kbo(:, 4,39,11) = (/ &
        & 0.10089e+00_rb,0.36746e+00_rb,0.45203e+00_rb,0.48569e+00_rb,0.35615e+00_rb /)
      kbo(:, 5,39,11) = (/ &
        & 0.11914e+00_rb,0.39759e+00_rb,0.48773e+00_rb,0.52218e+00_rb,0.38813e+00_rb /)
      kbo(:, 1,40,11) = (/ &
        & 0.47273e-01_rb,0.26566e+00_rb,0.32789e+00_rb,0.35219e+00_rb,0.24948e+00_rb /)
      kbo(:, 2,40,11) = (/ &
        & 0.61538e-01_rb,0.29768e+00_rb,0.36730e+00_rb,0.39473e+00_rb,0.28193e+00_rb /)
      kbo(:, 3,40,11) = (/ &
        & 0.77858e-01_rb,0.32903e+00_rb,0.40551e+00_rb,0.43635e+00_rb,0.31499e+00_rb /)
      kbo(:, 4,40,11) = (/ &
        & 0.95580e-01_rb,0.35975e+00_rb,0.44288e+00_rb,0.47617e+00_rb,0.34794e+00_rb /)
      kbo(:, 5,40,11) = (/ &
        & 0.11394e+00_rb,0.39008e+00_rb,0.47900e+00_rb,0.51333e+00_rb,0.38020e+00_rb /)
      kbo(:, 1,41,11) = (/ &
        & 0.42888e-01_rb,0.25708e+00_rb,0.31732e+00_rb,0.34096e+00_rb,0.24098e+00_rb /)
      kbo(:, 2,41,11) = (/ &
        & 0.56697e-01_rb,0.28927e+00_rb,0.35696e+00_rb,0.38355e+00_rb,0.27327e+00_rb /)
      kbo(:, 3,41,11) = (/ &
        & 0.72702e-01_rb,0.32083e+00_rb,0.39561e+00_rb,0.42555e+00_rb,0.30626e+00_rb /)
      kbo(:, 4,41,11) = (/ &
        & 0.90227e-01_rb,0.35164e+00_rb,0.43315e+00_rb,0.46599e+00_rb,0.33930e+00_rb /)
      kbo(:, 5,41,11) = (/ &
        & 0.10861e+00_rb,0.38219e+00_rb,0.46971e+00_rb,0.50385e+00_rb,0.37183e+00_rb /)
      kbo(:, 1,42,11) = (/ &
        & 0.38819e-01_rb,0.24850e+00_rb,0.30677e+00_rb,0.32968e+00_rb,0.23256e+00_rb /)
      kbo(:, 2,42,11) = (/ &
        & 0.52147e-01_rb,0.28080e+00_rb,0.34660e+00_rb,0.37239e+00_rb,0.26466e+00_rb /)
      kbo(:, 3,42,11) = (/ &
        & 0.67768e-01_rb,0.31263e+00_rb,0.38564e+00_rb,0.41466e+00_rb,0.29758e+00_rb /)
      kbo(:, 4,42,11) = (/ &
        & 0.85041e-01_rb,0.34356e+00_rb,0.42336e+00_rb,0.45563e+00_rb,0.33065e+00_rb /)
      kbo(:, 5,42,11) = (/ &
        & 0.10335e+00_rb,0.37426e+00_rb,0.46032e+00_rb,0.49419e+00_rb,0.36339e+00_rb /)
      kbo(:, 1,43,11) = (/ &
        & 0.34406e-01_rb,0.23801e+00_rb,0.29406e+00_rb,0.31601e+00_rb,0.22240e+00_rb /)
      kbo(:, 2,43,11) = (/ &
        & 0.47048e-01_rb,0.27050e+00_rb,0.33401e+00_rb,0.35883e+00_rb,0.25435e+00_rb /)
      kbo(:, 3,43,11) = (/ &
        & 0.62114e-01_rb,0.30257e+00_rb,0.37337e+00_rb,0.40135e+00_rb,0.28701e+00_rb /)
      kbo(:, 4,43,11) = (/ &
        & 0.79018e-01_rb,0.33381e+00_rb,0.41146e+00_rb,0.44285e+00_rb,0.32012e+00_rb /)
      kbo(:, 5,43,11) = (/ &
        & 0.97123e-01_rb,0.36459e+00_rb,0.44877e+00_rb,0.48229e+00_rb,0.35311e+00_rb /)
      kbo(:, 1,44,11) = (/ &
        & 0.30068e-01_rb,0.22661e+00_rb,0.28047e+00_rb,0.30141e+00_rb,0.21157e+00_rb /)
      kbo(:, 2,44,11) = (/ &
        & 0.41934e-01_rb,0.25939e+00_rb,0.32031e+00_rb,0.34420e+00_rb,0.24331e+00_rb /)
      kbo(:, 3,44,11) = (/ &
        & 0.56334e-01_rb,0.29165e+00_rb,0.35997e+00_rb,0.38686e+00_rb,0.27569e+00_rb /)
      kbo(:, 4,44,11) = (/ &
        & 0.72738e-01_rb,0.32319e+00_rb,0.39860e+00_rb,0.42881e+00_rb,0.30880e+00_rb /)
      kbo(:, 5,44,11) = (/ &
        & 0.90538e-01_rb,0.35407e+00_rb,0.43616e+00_rb,0.46913e+00_rb,0.34187e+00_rb /)
      kbo(:, 1,45,11) = (/ &
        & 0.26090e-01_rb,0.21511e+00_rb,0.26705e+00_rb,0.28701e+00_rb,0.20089e+00_rb /)
      kbo(:, 2,45,11) = (/ &
        & 0.37182e-01_rb,0.24828e+00_rb,0.30662e+00_rb,0.32954e+00_rb,0.23239e+00_rb /)
      kbo(:, 3,45,11) = (/ &
        & 0.50828e-01_rb,0.28069e+00_rb,0.34649e+00_rb,0.37235e+00_rb,0.26452e+00_rb /)
      kbo(:, 4,45,11) = (/ &
        & 0.66683e-01_rb,0.31254e+00_rb,0.38562e+00_rb,0.41465e+00_rb,0.29751e+00_rb /)
      kbo(:, 5,45,11) = (/ &
        & 0.84110e-01_rb,0.34354e+00_rb,0.42341e+00_rb,0.45565e+00_rb,0.33061e+00_rb /)
      kbo(:, 1,46,11) = (/ &
        & 0.22282e-01_rb,0.20276e+00_rb,0.25300e+00_rb,0.27194e+00_rb,0.18966e+00_rb /)
      kbo(:, 2,46,11) = (/ &
        & 0.32494e-01_rb,0.23634e+00_rb,0.29212e+00_rb,0.31397e+00_rb,0.22083e+00_rb /)
      kbo(:, 3,46,11) = (/ &
        & 0.45322e-01_rb,0.26893e+00_rb,0.33209e+00_rb,0.35685e+00_rb,0.25276e+00_rb /)
      kbo(:, 4,46,11) = (/ &
        & 0.60492e-01_rb,0.30105e+00_rb,0.37160e+00_rb,0.39945e+00_rb,0.28546e+00_rb /)
      kbo(:, 5,46,11) = (/ &
        & 0.77449e-01_rb,0.33241e+00_rb,0.40979e+00_rb,0.44100e+00_rb,0.31859e+00_rb /)
      kbo(:, 1,47,11) = (/ &
        & 0.18534e-01_rb,0.18898e+00_rb,0.23758e+00_rb,0.25552e+00_rb,0.17724e+00_rb /)
      kbo(:, 2,47,11) = (/ &
        & 0.27732e-01_rb,0.22283e+00_rb,0.27608e+00_rb,0.29673e+00_rb,0.20806e+00_rb /)
      kbo(:, 3,47,11) = (/ &
        & 0.39604e-01_rb,0.25581e+00_rb,0.31589e+00_rb,0.33953e+00_rb,0.23975e+00_rb /)
      kbo(:, 4,47,11) = (/ &
        & 0.53911e-01_rb,0.28814e+00_rb,0.35568e+00_rb,0.38226e+00_rb,0.27210e+00_rb /)
      kbo(:, 5,47,11) = (/ &
        & 0.70242e-01_rb,0.31982e+00_rb,0.39452e+00_rb,0.42435e+00_rb,0.30521e+00_rb /)
      kbo(:, 1,48,11) = (/ &
        & 0.15230e-01_rb,0.17512e+00_rb,0.22238e+00_rb,0.23953e+00_rb,0.16496e+00_rb /)
      kbo(:, 2,48,11) = (/ &
        & 0.23424e-01_rb,0.20917e+00_rb,0.26028e+00_rb,0.27977e+00_rb,0.19549e+00_rb /)
      kbo(:, 3,48,11) = (/ &
        & 0.34289e-01_rb,0.24261e+00_rb,0.29967e+00_rb,0.32213e+00_rb,0.22686e+00_rb /)
      kbo(:, 4,48,11) = (/ &
        & 0.47677e-01_rb,0.27510e+00_rb,0.33967e+00_rb,0.36502e+00_rb,0.25892e+00_rb /)
      kbo(:, 5,48,11) = (/ &
        & 0.63303e-01_rb,0.30711e+00_rb,0.37902e+00_rb,0.40749e+00_rb,0.29180e+00_rb /)
      kbo(:, 1,49,11) = (/ &
        & 0.12348e-01_rb,0.16145e+00_rb,0.20748e+00_rb,0.22396e+00_rb,0.15290e+00_rb /)
      kbo(:, 2,49,11) = (/ &
        & 0.19656e-01_rb,0.19547e+00_rb,0.24481e+00_rb,0.26319e+00_rb,0.18304e+00_rb /)
      kbo(:, 3,49,11) = (/ &
        & 0.29411e-01_rb,0.22922e+00_rb,0.28358e+00_rb,0.30484e+00_rb,0.21403e+00_rb /)
      kbo(:, 4,49,11) = (/ &
        & 0.41852e-01_rb,0.26199e+00_rb,0.32356e+00_rb,0.34774e+00_rb,0.24589e+00_rb /)
      kbo(:, 5,49,11) = (/ &
        & 0.56641e-01_rb,0.29425e+00_rb,0.36325e+00_rb,0.39043e+00_rb,0.27839e+00_rb /)
      kbo(:, 1,50,11) = (/ &
        & 0.10059e-01_rb,0.14901e+00_rb,0.19385e+00_rb,0.20977e+00_rb,0.14182e+00_rb /)
      kbo(:, 2,50,11) = (/ &
        & 0.16439e-01_rb,0.18255e+00_rb,0.23049e+00_rb,0.24806e+00_rb,0.17150e+00_rb /)
      kbo(:, 3,50,11) = (/ &
        & 0.25328e-01_rb,0.21656e+00_rb,0.26875e+00_rb,0.28888e+00_rb,0.20219e+00_rb /)
      kbo(:, 4,50,11) = (/ &
        & 0.36730e-01_rb,0.24971e+00_rb,0.30839e+00_rb,0.33151e+00_rb,0.23378e+00_rb /)
      kbo(:, 5,50,11) = (/ &
        & 0.50682e-01_rb,0.28213e+00_rb,0.34832e+00_rb,0.37432e+00_rb,0.26599e+00_rb /)
      kbo(:, 1,51,11) = (/ &
        & 0.81000e-02_rb,0.13718e+00_rb,0.18086e+00_rb,0.19633e+00_rb,0.13125e+00_rb /)
      kbo(:, 2,51,11) = (/ &
        & 0.13663e-01_rb,0.17002e+00_rb,0.21682e+00_rb,0.23372e+00_rb,0.16044e+00_rb /)
      kbo(:, 3,51,11) = (/ &
        & 0.21646e-01_rb,0.20410e+00_rb,0.25452e+00_rb,0.27362e+00_rb,0.19085e+00_rb /)
      kbo(:, 4,51,11) = (/ &
        & 0.32117e-01_rb,0.23766e+00_rb,0.29370e+00_rb,0.31573e+00_rb,0.22210e+00_rb /)
      kbo(:, 5,51,11) = (/ &
        & 0.45217e-01_rb,0.27026e+00_rb,0.33377e+00_rb,0.35865e+00_rb,0.25409e+00_rb /)
      kbo(:, 1,52,11) = (/ &
        & 0.64392e-02_rb,0.12568e+00_rb,0.16814e+00_rb,0.18321e+00_rb,0.12091e+00_rb /)
      kbo(:, 2,52,11) = (/ &
        & 0.11223e-01_rb,0.15773e+00_rb,0.20341e+00_rb,0.21973e+00_rb,0.14959e+00_rb /)
      kbo(:, 3,52,11) = (/ &
        & 0.18327e-01_rb,0.19165e+00_rb,0.24057e+00_rb,0.25870e+00_rb,0.17960e+00_rb /)
      kbo(:, 4,52,11) = (/ &
        & 0.27978e-01_rb,0.22553e+00_rb,0.27924e+00_rb,0.30015e+00_rb,0.21055e+00_rb /)
      kbo(:, 5,52,11) = (/ &
        & 0.40066e-01_rb,0.25840e+00_rb,0.31913e+00_rb,0.34300e+00_rb,0.24233e+00_rb /)
      kbo(:, 1,53,11) = (/ &
        & 0.50582e-02_rb,0.11452e+00_rb,0.15565e+00_rb,0.17040e+00_rb,0.11082e+00_rb /)
      kbo(:, 2,53,11) = (/ &
        & 0.90986e-02_rb,0.14571e+00_rb,0.19026e+00_rb,0.20608e+00_rb,0.13891e+00_rb /)
      kbo(:, 3,53,11) = (/ &
        & 0.15360e-01_rb,0.17911e+00_rb,0.22674e+00_rb,0.24415e+00_rb,0.16845e+00_rb /)
      kbo(:, 4,53,11) = (/ &
        & 0.24079e-01_rb,0.21320e+00_rb,0.26488e+00_rb,0.28473e+00_rb,0.19911e+00_rb /)
      kbo(:, 5,53,11) = (/ &
        & 0.35249e-01_rb,0.24647e+00_rb,0.30443e+00_rb,0.32725e+00_rb,0.23061e+00_rb /)
      kbo(:, 1,54,11) = (/ &
        & 0.39947e-02_rb,0.10467e+00_rb,0.14438e+00_rb,0.15895e+00_rb,0.10183e+00_rb /)
      kbo(:, 2,54,11) = (/ &
        & 0.74123e-02_rb,0.13495e+00_rb,0.17842e+00_rb,0.19384e+00_rb,0.12928e+00_rb /)
      kbo(:, 3,54,11) = (/ &
        & 0.12928e-01_rb,0.16768e+00_rb,0.21428e+00_rb,0.23108e+00_rb,0.15838e+00_rb /)
      kbo(:, 4,54,11) = (/ &
        & 0.20817e-01_rb,0.20178e+00_rb,0.25191e+00_rb,0.27082e+00_rb,0.18875e+00_rb /)
      kbo(:, 5,54,11) = (/ &
        & 0.31131e-01_rb,0.23540e+00_rb,0.29100e+00_rb,0.31283e+00_rb,0.21994e+00_rb /)
      kbo(:, 1,55,11) = (/ &
        & 0.31444e-02_rb,0.95645e-01_rb,0.13357e+00_rb,0.14821e+00_rb,0.93456e-01_rb /)
      kbo(:, 2,55,11) = (/ &
        & 0.60192e-02_rb,0.12488e+00_rb,0.16728e+00_rb,0.18235e+00_rb,0.12022e+00_rb /)
      kbo(:, 3,55,11) = (/ &
        & 0.10844e-01_rb,0.15692e+00_rb,0.20255e+00_rb,0.21884e+00_rb,0.14887e+00_rb /)
      kbo(:, 4,55,11) = (/ &
        & 0.17966e-01_rb,0.19085e+00_rb,0.23968e+00_rb,0.25776e+00_rb,0.17889e+00_rb /)
      kbo(:, 5,55,11) = (/ &
        & 0.27575e-01_rb,0.22475e+00_rb,0.27833e+00_rb,0.29920e+00_rb,0.20981e+00_rb /)
      kbo(:, 1,56,11) = (/ &
        & 0.24408e-02_rb,0.87042e-01_rb,0.12294e+00_rb,0.13774e+00_rb,0.85384e-01_rb /)
      kbo(:, 2,56,11) = (/ &
        & 0.48425e-02_rb,0.11512e+00_rb,0.15634e+00_rb,0.17115e+00_rb,0.11140e+00_rb /)
      kbo(:, 3,56,11) = (/ &
        & 0.90070e-02_rb,0.14641e+00_rb,0.19103e+00_rb,0.20689e+00_rb,0.13953e+00_rb /)
      kbo(:, 4,56,11) = (/ &
        & 0.15384e-01_rb,0.17987e+00_rb,0.22757e+00_rb,0.24502e+00_rb,0.16913e+00_rb /)
      kbo(:, 5,56,11) = (/ &
        & 0.24183e-01_rb,0.21396e+00_rb,0.26576e+00_rb,0.28569e+00_rb,0.19980e+00_rb /)
      kbo(:, 1,57,11) = (/ &
        & 0.18661e-02_rb,0.78869e-01_rb,0.11259e+00_rb,0.12744e+00_rb,0.77594e-01_rb /)
      kbo(:, 2,57,11) = (/ &
        & 0.38393e-02_rb,0.10567e+00_rb,0.14558e+00_rb,0.16018e+00_rb,0.10279e+00_rb /)
      kbo(:, 3,57,11) = (/ &
        & 0.73986e-02_rb,0.13611e+00_rb,0.17972e+00_rb,0.19518e+00_rb,0.13032e+00_rb /)
      kbo(:, 4,57,11) = (/ &
        & 0.13057e-01_rb,0.16895e+00_rb,0.21565e+00_rb,0.23253e+00_rb,0.15951e+00_rb /)
      kbo(:, 5,57,11) = (/ &
        & 0.21059e-01_rb,0.20305e+00_rb,0.25335e+00_rb,0.27238e+00_rb,0.18989e+00_rb /)
      kbo(:, 1,58,11) = (/ &
        & 0.14289e-02_rb,0.71481e-01_rb,0.10314e+00_rb,0.11789e+00_rb,0.70461e-01_rb /)
      kbo(:, 2,58,11) = (/ &
        & 0.30447e-02_rb,0.97100e-01_rb,0.13537e+00_rb,0.15001e+00_rb,0.94840e-01_rb /)
      kbo(:, 3,58,11) = (/ &
        & 0.60785e-02_rb,0.12656e+00_rb,0.16917e+00_rb,0.18428e+00_rb,0.12173e+00_rb /)
      kbo(:, 4,58,11) = (/ &
        & 0.11073e-01_rb,0.15875e+00_rb,0.20453e+00_rb,0.22090e+00_rb,0.15049e+00_rb /)
      kbo(:, 5,58,11) = (/ &
        & 0.18344e-01_rb,0.19271e+00_rb,0.24175e+00_rb,0.25997e+00_rb,0.18056e+00_rb /)
      kbo(:, 1,59,11) = (/ &
        & 0.12466e-02_rb,0.68608e-01_rb,0.99449e-01_rb,0.11409e+00_rb,0.67679e-01_rb /)
      kbo(:, 2,59,11) = (/ &
        & 0.27400e-02_rb,0.93756e-01_rb,0.13130e+00_rb,0.14598e+00_rb,0.91727e-01_rb /)
      kbo(:, 3,59,11) = (/ &
        & 0.55829e-02_rb,0.12280e+00_rb,0.16498e+00_rb,0.17997e+00_rb,0.11835e+00_rb /)
      kbo(:, 4,59,11) = (/ &
        & 0.10327e-01_rb,0.15471e+00_rb,0.20012e+00_rb,0.21632e+00_rb,0.14691e+00_rb /)
      kbo(:, 5,59,11) = (/ &
        & 0.17312e-01_rb,0.18856e+00_rb,0.23714e+00_rb,0.25511e+00_rb,0.17685e+00_rb /)
      kbo(:, 1,13,12) = (/ &
        & 0.97672e+00_rb,0.12351e+01_rb,0.13467e+01_rb,0.13355e+01_rb,0.11350e+01_rb /)
      kbo(:, 2,13,12) = (/ &
        & 0.96517e+00_rb,0.12326e+01_rb,0.13523e+01_rb,0.13488e+01_rb,0.11500e+01_rb /)
      kbo(:, 3,13,12) = (/ &
        & 0.95359e+00_rb,0.12308e+01_rb,0.13576e+01_rb,0.13588e+01_rb,0.11629e+01_rb /)
      kbo(:, 4,13,12) = (/ &
        & 0.94644e+00_rb,0.12294e+01_rb,0.13618e+01_rb,0.13660e+01_rb,0.11761e+01_rb /)
      kbo(:, 5,13,12) = (/ &
        & 0.94938e+00_rb,0.12277e+01_rb,0.13647e+01_rb,0.13713e+01_rb,0.11895e+01_rb /)
      kbo(:, 1,14,12) = (/ &
        & 0.86287e+00_rb,0.11528e+01_rb,0.12800e+01_rb,0.12931e+01_rb,0.10793e+01_rb /)
      kbo(:, 2,14,12) = (/ &
        & 0.85328e+00_rb,0.11519e+01_rb,0.12878e+01_rb,0.13082e+01_rb,0.10960e+01_rb /)
      kbo(:, 3,14,12) = (/ &
        & 0.84776e+00_rb,0.11522e+01_rb,0.12944e+01_rb,0.13193e+01_rb,0.11122e+01_rb /)
      kbo(:, 4,14,12) = (/ &
        & 0.85264e+00_rb,0.11532e+01_rb,0.12995e+01_rb,0.13287e+01_rb,0.11288e+01_rb /)
      kbo(:, 5,14,12) = (/ &
        & 0.86761e+00_rb,0.11556e+01_rb,0.13039e+01_rb,0.13378e+01_rb,0.11458e+01_rb /)
      kbo(:, 1,15,12) = (/ &
        & 0.75595e+00_rb,0.10736e+01_rb,0.12146e+01_rb,0.12437e+01_rb,0.10230e+01_rb /)
      kbo(:, 2,15,12) = (/ &
        & 0.75063e+00_rb,0.10744e+01_rb,0.12248e+01_rb,0.12603e+01_rb,0.10430e+01_rb /)
      kbo(:, 3,15,12) = (/ &
        & 0.75594e+00_rb,0.10769e+01_rb,0.12334e+01_rb,0.12743e+01_rb,0.10635e+01_rb /)
      kbo(:, 4,15,12) = (/ &
        & 0.77107e+00_rb,0.10807e+01_rb,0.12402e+01_rb,0.12881e+01_rb,0.10841e+01_rb /)
      kbo(:, 5,15,12) = (/ &
        & 0.79469e+00_rb,0.10902e+01_rb,0.12461e+01_rb,0.13012e+01_rb,0.11031e+01_rb /)
      kbo(:, 1,16,12) = (/ &
        & 0.66063e+00_rb,0.99640e+00_rb,0.11502e+01_rb,0.11902e+01_rb,0.96912e+00_rb /)
      kbo(:, 2,16,12) = (/ &
        & 0.66454e+00_rb,0.10001e+01_rb,0.11634e+01_rb,0.12095e+01_rb,0.99257e+00_rb /)
      kbo(:, 3,16,12) = (/ &
        & 0.67895e+00_rb,0.10049e+01_rb,0.11739e+01_rb,0.12282e+01_rb,0.10170e+01_rb /)
      kbo(:, 4,16,12) = (/ &
        & 0.70145e+00_rb,0.10153e+01_rb,0.11833e+01_rb,0.12469e+01_rb,0.10404e+01_rb /)
      kbo(:, 5,16,12) = (/ &
        & 0.72525e+00_rb,0.10330e+01_rb,0.11925e+01_rb,0.12646e+01_rb,0.10610e+01_rb /)
      kbo(:, 1,17,12) = (/ &
        & 0.58214e+00_rb,0.92297e+00_rb,0.10896e+01_rb,0.11346e+01_rb,0.91647e+00_rb /)
      kbo(:, 2,17,12) = (/ &
        & 0.59400e+00_rb,0.92929e+00_rb,0.11045e+01_rb,0.11590e+01_rb,0.94550e+00_rb /)
      kbo(:, 3,17,12) = (/ &
        & 0.61460e+00_rb,0.94000e+00_rb,0.11172e+01_rb,0.11832e+01_rb,0.97371e+00_rb /)
      kbo(:, 4,17,12) = (/ &
        & 0.63691e+00_rb,0.95830e+00_rb,0.11303e+01_rb,0.12071e+01_rb,0.99891e+00_rb /)
      kbo(:, 5,17,12) = (/ &
        & 0.66003e+00_rb,0.98295e+00_rb,0.11440e+01_rb,0.12298e+01_rb,0.10215e+01_rb /)
      kbo(:, 1,18,12) = (/ &
        & 0.51831e+00_rb,0.85586e+00_rb,0.10313e+01_rb,0.10823e+01_rb,0.86734e+00_rb /)
      kbo(:, 2,18,12) = (/ &
        & 0.53638e+00_rb,0.86645e+00_rb,0.10487e+01_rb,0.11114e+01_rb,0.90071e+00_rb /)
      kbo(:, 3,18,12) = (/ &
        & 0.55679e+00_rb,0.88410e+00_rb,0.10657e+01_rb,0.11407e+01_rb,0.93149e+00_rb /)
      kbo(:, 4,18,12) = (/ &
        & 0.57819e+00_rb,0.90902e+00_rb,0.10831e+01_rb,0.11693e+01_rb,0.95937e+00_rb /)
      kbo(:, 5,18,12) = (/ &
        & 0.60387e+00_rb,0.94010e+00_rb,0.11032e+01_rb,0.11966e+01_rb,0.98550e+00_rb /)
      kbo(:, 1,19,12) = (/ &
        & 0.46547e+00_rb,0.79533e+00_rb,0.97579e+00_rb,0.10353e+01_rb,0.82208e+00_rb /)
      kbo(:, 2,19,12) = (/ &
        & 0.48441e+00_rb,0.81281e+00_rb,0.99768e+00_rb,0.10690e+01_rb,0.85858e+00_rb /)
      kbo(:, 3,19,12) = (/ &
        & 0.50438e+00_rb,0.83723e+00_rb,0.10197e+01_rb,0.11027e+01_rb,0.89202e+00_rb /)
      kbo(:, 4,19,12) = (/ &
        & 0.52861e+00_rb,0.86772e+00_rb,0.10433e+01_rb,0.11355e+01_rb,0.92298e+00_rb /)
      kbo(:, 5,19,12) = (/ &
        & 0.55711e+00_rb,0.90301e+00_rb,0.10701e+01_rb,0.11672e+01_rb,0.95266e+00_rb /)
      kbo(:, 1,20,12) = (/ &
        & 0.41965e+00_rb,0.74319e+00_rb,0.92508e+00_rb,0.99412e+00_rb,0.78089e+00_rb /)
      kbo(:, 2,20,12) = (/ &
        & 0.43802e+00_rb,0.76766e+00_rb,0.95235e+00_rb,0.10327e+01_rb,0.82017e+00_rb /)
      kbo(:, 3,20,12) = (/ &
        & 0.46025e+00_rb,0.79818e+00_rb,0.98022e+00_rb,0.10706e+01_rb,0.85677e+00_rb /)
      kbo(:, 4,20,12) = (/ &
        & 0.48727e+00_rb,0.83370e+00_rb,0.10109e+01_rb,0.11072e+01_rb,0.89127e+00_rb /)
      kbo(:, 5,20,12) = (/ &
        & 0.51863e+00_rb,0.87091e+00_rb,0.10440e+01_rb,0.11423e+01_rb,0.92433e+00_rb /)
      kbo(:, 1,21,12) = (/ &
        & 0.37915e+00_rb,0.69998e+00_rb,0.88069e+00_rb,0.95835e+00_rb,0.74387e+00_rb /)
      kbo(:, 2,21,12) = (/ &
        & 0.39885e+00_rb,0.72965e+00_rb,0.91291e+00_rb,0.10022e+01_rb,0.78645e+00_rb /)
      kbo(:, 3,21,12) = (/ &
        & 0.42350e+00_rb,0.76561e+00_rb,0.94765e+00_rb,0.10443e+01_rb,0.82639e+00_rb /)
      kbo(:, 4,21,12) = (/ &
        & 0.45298e+00_rb,0.80432e+00_rb,0.98473e+00_rb,0.10842e+01_rb,0.86477e+00_rb /)
      kbo(:, 5,21,12) = (/ &
        & 0.48674e+00_rb,0.84344e+00_rb,0.10236e+01_rb,0.11221e+01_rb,0.90082e+00_rb /)
      kbo(:, 1,22,12) = (/ &
        & 0.34554e+00_rb,0.66631e+00_rb,0.84565e+00_rb,0.93193e+00_rb,0.71462e+00_rb /)
      kbo(:, 2,22,12) = (/ &
        & 0.36807e+00_rb,0.70096e+00_rb,0.88353e+00_rb,0.98021e+00_rb,0.76038e+00_rb /)
      kbo(:, 3,22,12) = (/ &
        & 0.39520e+00_rb,0.74099e+00_rb,0.92460e+00_rb,0.10260e+01_rb,0.80381e+00_rb /)
      kbo(:, 4,22,12) = (/ &
        & 0.42700e+00_rb,0.78213e+00_rb,0.96750e+00_rb,0.10688e+01_rb,0.84544e+00_rb /)
      kbo(:, 5,22,12) = (/ &
        & 0.46306e+00_rb,0.82357e+00_rb,0.10116e+01_rb,0.11089e+01_rb,0.88444e+00_rb /)
      kbo(:, 1,23,12) = (/ &
        & 0.31790e+00_rb,0.64007e+00_rb,0.81825e+00_rb,0.91240e+00_rb,0.69117e+00_rb /)
      kbo(:, 2,23,12) = (/ &
        & 0.34313e+00_rb,0.67943e+00_rb,0.86200e+00_rb,0.96400e+00_rb,0.74026e+00_rb /)
      kbo(:, 3,23,12) = (/ &
        & 0.37292e+00_rb,0.72179e+00_rb,0.90853e+00_rb,0.10128e+01_rb,0.78692e+00_rb /)
      kbo(:, 4,23,12) = (/ &
        & 0.40683e+00_rb,0.76528e+00_rb,0.95638e+00_rb,0.10584e+01_rb,0.83132e+00_rb /)
      kbo(:, 5,23,12) = (/ &
        & 0.44434e+00_rb,0.80906e+00_rb,0.10049e+01_rb,0.11007e+01_rb,0.87324e+00_rb /)
      kbo(:, 1,24,12) = (/ &
        & 0.29587e+00_rb,0.62091e+00_rb,0.79898e+00_rb,0.89859e+00_rb,0.67383e+00_rb /)
      kbo(:, 2,24,12) = (/ &
        & 0.32356e+00_rb,0.66370e+00_rb,0.84800e+00_rb,0.95354e+00_rb,0.72591e+00_rb /)
      kbo(:, 3,24,12) = (/ &
        & 0.35587e+00_rb,0.70805e+00_rb,0.89882e+00_rb,0.10049e+01_rb,0.77551e+00_rb /)
      kbo(:, 4,24,12) = (/ &
        & 0.39090e+00_rb,0.75345e+00_rb,0.95073e+00_rb,0.10524e+01_rb,0.82233e+00_rb /)
      kbo(:, 5,24,12) = (/ &
        & 0.42938e+00_rb,0.80019e+00_rb,0.10026e+01_rb,0.10965e+01_rb,0.86634e+00_rb /)
      kbo(:, 1,25,12) = (/ &
        & 0.27857e+00_rb,0.60799e+00_rb,0.78705e+00_rb,0.89081e+00_rb,0.66256e+00_rb /)
      kbo(:, 2,25,12) = (/ &
        & 0.30889e+00_rb,0.65303e+00_rb,0.84032e+00_rb,0.94841e+00_rb,0.71733e+00_rb /)
      kbo(:, 3,25,12) = (/ &
        & 0.34245e+00_rb,0.69924e+00_rb,0.89488e+00_rb,0.10015e+01_rb,0.76917e+00_rb /)
      kbo(:, 4,25,12) = (/ &
        & 0.37871e+00_rb,0.74693e+00_rb,0.95015e+00_rb,0.10506e+01_rb,0.81807e+00_rb /)
      kbo(:, 5,25,12) = (/ &
        & 0.41833e+00_rb,0.79644e+00_rb,0.10044e+01_rb,0.10958e+01_rb,0.86374e+00_rb /)
      kbo(:, 1,26,12) = (/ &
        & 0.26580e+00_rb,0.60063e+00_rb,0.78228e+00_rb,0.88879e+00_rb,0.65720e+00_rb /)
      kbo(:, 2,26,12) = (/ &
        & 0.29790e+00_rb,0.64759e+00_rb,0.83895e+00_rb,0.94821e+00_rb,0.71439e+00_rb /)
      kbo(:, 3,26,12) = (/ &
        & 0.33261e+00_rb,0.69574e+00_rb,0.89642e+00_rb,0.10025e+01_rb,0.76782e+00_rb /)
      kbo(:, 4,26,12) = (/ &
        & 0.37054e+00_rb,0.74591e+00_rb,0.95396e+00_rb,0.10524e+01_rb,0.81818e+00_rb /)
      kbo(:, 5,26,12) = (/ &
        & 0.41158e+00_rb,0.79751e+00_rb,0.10099e+01_rb,0.10981e+01_rb,0.86507e+00_rb /)
      kbo(:, 1,27,12) = (/ &
        & 0.25628e+00_rb,0.59727e+00_rb,0.78261e+00_rb,0.89115e+00_rb,0.65648e+00_rb /)
      kbo(:, 2,27,12) = (/ &
        & 0.28973e+00_rb,0.64617e+00_rb,0.84187e+00_rb,0.95137e+00_rb,0.71531e+00_rb /)
      kbo(:, 3,27,12) = (/ &
        & 0.32602e+00_rb,0.69640e+00_rb,0.90149e+00_rb,0.10062e+01_rb,0.76992e+00_rb /)
      kbo(:, 4,27,12) = (/ &
        & 0.36566e+00_rb,0.74858e+00_rb,0.96067e+00_rb,0.10565e+01_rb,0.82130e+00_rb /)
      kbo(:, 5,27,12) = (/ &
        & 0.40839e+00_rb,0.80160e+00_rb,0.10174e+01_rb,0.11025e+01_rb,0.86892e+00_rb /)
      kbo(:, 1,28,12) = (/ &
        & 0.24931e+00_rb,0.59767e+00_rb,0.78703e+00_rb,0.89664e+00_rb,0.65970e+00_rb /)
      kbo(:, 2,28,12) = (/ &
        & 0.28438e+00_rb,0.64828e+00_rb,0.84808e+00_rb,0.95711e+00_rb,0.71948e+00_rb /)
      kbo(:, 3,28,12) = (/ &
        & 0.32259e+00_rb,0.70046e+00_rb,0.90930e+00_rb,0.10123e+01_rb,0.77494e+00_rb /)
      kbo(:, 4,28,12) = (/ &
        & 0.36394e+00_rb,0.75423e+00_rb,0.96943e+00_rb,0.10625e+01_rb,0.82683e+00_rb /)
      kbo(:, 5,28,12) = (/ &
        & 0.40857e+00_rb,0.80828e+00_rb,0.10265e+01_rb,0.11085e+01_rb,0.87457e+00_rb /)
      kbo(:, 1,29,12) = (/ &
        & 0.24529e+00_rb,0.60153e+00_rb,0.79470e+00_rb,0.90469e+00_rb,0.66635e+00_rb /)
      kbo(:, 2,29,12) = (/ &
        & 0.28215e+00_rb,0.65374e+00_rb,0.85707e+00_rb,0.96518e+00_rb,0.72659e+00_rb /)
      kbo(:, 3,29,12) = (/ &
        & 0.32237e+00_rb,0.70744e+00_rb,0.91955e+00_rb,0.10202e+01_rb,0.78233e+00_rb /)
      kbo(:, 4,29,12) = (/ &
        & 0.36548e+00_rb,0.76246e+00_rb,0.98006e+00_rb,0.10704e+01_rb,0.83435e+00_rb /)
      kbo(:, 5,29,12) = (/ &
        & 0.41179e+00_rb,0.81715e+00_rb,0.10368e+01_rb,0.11160e+01_rb,0.88198e+00_rb /)
      kbo(:, 1,30,12) = (/ &
        & 0.24386e+00_rb,0.60816e+00_rb,0.80473e+00_rb,0.91460e+00_rb,0.67543e+00_rb /)
      kbo(:, 2,30,12) = (/ &
        & 0.28279e+00_rb,0.66165e+00_rb,0.86804e+00_rb,0.97470e+00_rb,0.73566e+00_rb /)
      kbo(:, 3,30,12) = (/ &
        & 0.32483e+00_rb,0.71650e+00_rb,0.93115e+00_rb,0.10292e+01_rb,0.79124e+00_rb /)
      kbo(:, 4,30,12) = (/ &
        & 0.36954e+00_rb,0.77208e+00_rb,0.99158e+00_rb,0.10790e+01_rb,0.84299e+00_rb /)
      kbo(:, 5,30,12) = (/ &
        & 0.41749e+00_rb,0.82738e+00_rb,0.10478e+01_rb,0.11245e+01_rb,0.89039e+00_rb /)
      kbo(:, 1,31,12) = (/ &
        & 0.24525e+00_rb,0.61702e+00_rb,0.81682e+00_rb,0.92621e+00_rb,0.68640e+00_rb /)
      kbo(:, 2,31,12) = (/ &
        & 0.28616e+00_rb,0.67142e+00_rb,0.88072e+00_rb,0.98550e+00_rb,0.74643e+00_rb /)
      kbo(:, 3,31,12) = (/ &
        & 0.32989e+00_rb,0.72710e+00_rb,0.94409e+00_rb,0.10393e+01_rb,0.80152e+00_rb /)
      kbo(:, 4,31,12) = (/ &
        & 0.37609e+00_rb,0.78314e+00_rb,0.10041e+01_rb,0.10886e+01_rb,0.85269e+00_rb /)
      kbo(:, 5,31,12) = (/ &
        & 0.42530e+00_rb,0.83896e+00_rb,0.10596e+01_rb,0.11338e+01_rb,0.89957e+00_rb /)
      kbo(:, 1,32,12) = (/ &
        & 0.24917e+00_rb,0.62738e+00_rb,0.83018e+00_rb,0.93886e+00_rb,0.69870e+00_rb /)
      kbo(:, 2,32,12) = (/ &
        & 0.29180e+00_rb,0.68257e+00_rb,0.89466e+00_rb,0.99723e+00_rb,0.75819e+00_rb /)
      kbo(:, 3,32,12) = (/ &
        & 0.33695e+00_rb,0.73898e+00_rb,0.95783e+00_rb,0.10503e+01_rb,0.81270e+00_rb /)
      kbo(:, 4,32,12) = (/ &
        & 0.38441e+00_rb,0.79527e+00_rb,0.10172e+01_rb,0.10987e+01_rb,0.86313e+00_rb /)
      kbo(:, 5,32,12) = (/ &
        & 0.43473e+00_rb,0.85145e+00_rb,0.10718e+01_rb,0.11436e+01_rb,0.90933e+00_rb /)
      kbo(:, 1,33,12) = (/ &
        & 0.25524e+00_rb,0.63888e+00_rb,0.84456e+00_rb,0.95208e+00_rb,0.71177e+00_rb /)
      kbo(:, 2,33,12) = (/ &
        & 0.29923e+00_rb,0.69477e+00_rb,0.90943e+00_rb,0.10094e+01_rb,0.77043e+00_rb /)
      kbo(:, 3,33,12) = (/ &
        & 0.34555e+00_rb,0.75166e+00_rb,0.97211e+00_rb,0.10616e+01_rb,0.82442e+00_rb /)
      kbo(:, 4,33,12) = (/ &
        & 0.39409e+00_rb,0.80804e+00_rb,0.10306e+01_rb,0.11094e+01_rb,0.87396e+00_rb /)
      kbo(:, 5,33,12) = (/ &
        & 0.44532e+00_rb,0.86454e+00_rb,0.10842e+01_rb,0.11536e+01_rb,0.91947e+00_rb /)
      kbo(:, 1,34,12) = (/ &
        & 0.26139e+00_rb,0.64937e+00_rb,0.85738e+00_rb,0.96358e+00_rb,0.72330e+00_rb /)
      kbo(:, 2,34,12) = (/ &
        & 0.30644e+00_rb,0.70578e+00_rb,0.92238e+00_rb,0.10200e+01_rb,0.78117e+00_rb /)
      kbo(:, 3,34,12) = (/ &
        & 0.35368e+00_rb,0.76284e+00_rb,0.98445e+00_rb,0.10715e+01_rb,0.83457e+00_rb /)
      kbo(:, 4,34,12) = (/ &
        & 0.40294e+00_rb,0.81944e+00_rb,0.10421e+01_rb,0.11185e+01_rb,0.88331e+00_rb /)
      kbo(:, 5,34,12) = (/ &
        & 0.45497e+00_rb,0.87610e+00_rb,0.10949e+01_rb,0.11623e+01_rb,0.92821e+00_rb /)
      kbo(:, 1,35,12) = (/ &
        & 0.26429e+00_rb,0.65495e+00_rb,0.86437e+00_rb,0.96967e+00_rb,0.72944e+00_rb /)
      kbo(:, 2,35,12) = (/ &
        & 0.31003e+00_rb,0.71171e+00_rb,0.92940e+00_rb,0.10257e+01_rb,0.78687e+00_rb /)
      kbo(:, 3,35,12) = (/ &
        & 0.35779e+00_rb,0.76892e+00_rb,0.99113e+00_rb,0.10767e+01_rb,0.83993e+00_rb /)
      kbo(:, 4,35,12) = (/ &
        & 0.40757e+00_rb,0.82572e+00_rb,0.10483e+01_rb,0.11235e+01_rb,0.88834e+00_rb /)
      kbo(:, 5,35,12) = (/ &
        & 0.46005e+00_rb,0.88243e+00_rb,0.11007e+01_rb,0.11671e+01_rb,0.93291e+00_rb /)
      kbo(:, 1,36,12) = (/ &
        & 0.26301e+00_rb,0.65478e+00_rb,0.86458e+00_rb,0.96965e+00_rb,0.72937e+00_rb /)
      kbo(:, 2,36,12) = (/ &
        & 0.30906e+00_rb,0.71167e+00_rb,0.92967e+00_rb,0.10257e+01_rb,0.78687e+00_rb /)
      kbo(:, 3,36,12) = (/ &
        & 0.35709e+00_rb,0.76907e+00_rb,0.99151e+00_rb,0.10769e+01_rb,0.84005e+00_rb /)
      kbo(:, 4,36,12) = (/ &
        & 0.40702e+00_rb,0.82602e+00_rb,0.10488e+01_rb,0.11238e+01_rb,0.88850e+00_rb /)
      kbo(:, 5,36,12) = (/ &
        & 0.45966e+00_rb,0.88282e+00_rb,0.11012e+01_rb,0.11674e+01_rb,0.93315e+00_rb /)
      kbo(:, 1,37,12) = (/ &
        & 0.25532e+00_rb,0.64621e+00_rb,0.85496e+00_rb,0.96073e+00_rb,0.72018e+00_rb /)
      kbo(:, 2,37,12) = (/ &
        & 0.30133e+00_rb,0.70315e+00_rb,0.92046e+00_rb,0.10178e+01_rb,0.77861e+00_rb /)
      kbo(:, 3,37,12) = (/ &
        & 0.34928e+00_rb,0.76070e+00_rb,0.98300e+00_rb,0.10697e+01_rb,0.83253e+00_rb /)
      kbo(:, 4,37,12) = (/ &
        & 0.39901e+00_rb,0.81782e+00_rb,0.10410e+01_rb,0.11173e+01_rb,0.88173e+00_rb /)
      kbo(:, 5,37,12) = (/ &
        & 0.45136e+00_rb,0.87470e+00_rb,0.10942e+01_rb,0.11614e+01_rb,0.92694e+00_rb /)
      kbo(:, 1,38,12) = (/ &
        & 0.24761e+00_rb,0.63742e+00_rb,0.84491e+00_rb,0.95139e+00_rb,0.71066e+00_rb /)
      kbo(:, 2,38,12) = (/ &
        & 0.29353e+00_rb,0.69428e+00_rb,0.91068e+00_rb,0.10095e+01_rb,0.76992e+00_rb /)
      kbo(:, 3,38,12) = (/ &
        & 0.34135e+00_rb,0.75196e+00_rb,0.97396e+00_rb,0.10621e+01_rb,0.82461e+00_rb /)
      kbo(:, 4,38,12) = (/ &
        & 0.39084e+00_rb,0.80925e+00_rb,0.10327e+01_rb,0.11104e+01_rb,0.87459e+00_rb /)
      kbo(:, 5,38,12) = (/ &
        & 0.44289e+00_rb,0.86617e+00_rb,0.10866e+01_rb,0.11550e+01_rb,0.92038e+00_rb /)
      kbo(:, 1,39,12) = (/ &
        & 0.24017e+00_rb,0.62886e+00_rb,0.83503e+00_rb,0.94210e+00_rb,0.70124e+00_rb /)
      kbo(:, 2,39,12) = (/ &
        & 0.28601e+00_rb,0.68569e+00_rb,0.90106e+00_rb,0.10011e+01_rb,0.76142e+00_rb /)
      kbo(:, 3,39,12) = (/ &
        & 0.33368e+00_rb,0.74335e+00_rb,0.96490e+00_rb,0.10546e+01_rb,0.81673e+00_rb /)
      kbo(:, 4,39,12) = (/ &
        & 0.38296e+00_rb,0.80081e+00_rb,0.10243e+01_rb,0.11035e+01_rb,0.86747e+00_rb /)
      kbo(:, 5,39,12) = (/ &
        & 0.43465e+00_rb,0.85778e+00_rb,0.10791e+01_rb,0.11488e+01_rb,0.91385e+00_rb /)
      kbo(:, 1,40,12) = (/ &
        & 0.22833e+00_rb,0.61463e+00_rb,0.81821e+00_rb,0.92620e+00_rb,0.68530e+00_rb /)
      kbo(:, 2,40,12) = (/ &
        & 0.27395e+00_rb,0.67134e+00_rb,0.88467e+00_rb,0.98686e+00_rb,0.74688e+00_rb /)
      kbo(:, 3,40,12) = (/ &
        & 0.32124e+00_rb,0.72890e+00_rb,0.94943e+00_rb,0.10418e+01_rb,0.80324e+00_rb /)
      kbo(:, 4,40,12) = (/ &
        & 0.37015e+00_rb,0.78652e+00_rb,0.10100e+01_rb,0.10918e+01_rb,0.85523e+00_rb /)
      kbo(:, 5,40,12) = (/ &
        & 0.42126e+00_rb,0.84372e+00_rb,0.10660e+01_rb,0.11379e+01_rb,0.90264e+00_rb /)
      kbo(:, 1,41,12) = (/ &
        & 0.21612e+00_rb,0.59971e+00_rb,0.80041e+00_rb,0.90904e+00_rb,0.66824e+00_rb /)
      kbo(:, 2,41,12) = (/ &
        & 0.26150e+00_rb,0.65630e+00_rb,0.86743e+00_rb,0.97155e+00_rb,0.73120e+00_rb /)
      kbo(:, 3,41,12) = (/ &
        & 0.30837e+00_rb,0.71376e+00_rb,0.93288e+00_rb,0.10279e+01_rb,0.78880e+00_rb /)
      kbo(:, 4,41,12) = (/ &
        & 0.35698e+00_rb,0.77150e+00_rb,0.99473e+00_rb,0.10791e+01_rb,0.84209e+00_rb /)
      kbo(:, 5,41,12) = (/ &
        & 0.40745e+00_rb,0.82887e+00_rb,0.10519e+01_rb,0.11261e+01_rb,0.89062e+00_rb /)
      kbo(:, 1,42,12) = (/ &
        & 0.20417e+00_rb,0.58472e+00_rb,0.78237e+00_rb,0.89154e+00_rb,0.65066e+00_rb /)
      kbo(:, 2,42,12) = (/ &
        & 0.24921e+00_rb,0.64136e+00_rb,0.85007e+00_rb,0.95580e+00_rb,0.71510e+00_rb /)
      kbo(:, 3,42,12) = (/ &
        & 0.29566e+00_rb,0.69861e+00_rb,0.91611e+00_rb,0.10137e+01_rb,0.77414e+00_rb /)
      kbo(:, 4,42,12) = (/ &
        & 0.34395e+00_rb,0.75651e+00_rb,0.97922e+00_rb,0.10662e+01_rb,0.82865e+00_rb /)
      kbo(:, 5,42,12) = (/ &
        & 0.39383e+00_rb,0.81397e+00_rb,0.10376e+01_rb,0.11142e+01_rb,0.87842e+00_rb /)
      kbo(:, 1,43,12) = (/ &
        & 0.18997e+00_rb,0.56647e+00_rb,0.76032e+00_rb,0.86976e+00_rb,0.62901e+00_rb /)
      kbo(:, 2,43,12) = (/ &
        & 0.23447e+00_rb,0.62342e+00_rb,0.82892e+00_rb,0.93617e+00_rb,0.69524e+00_rb /)
      kbo(:, 3,43,12) = (/ &
        & 0.28050e+00_rb,0.68037e+00_rb,0.89544e+00_rb,0.99597e+00_rb,0.75607e+00_rb /)
      kbo(:, 4,43,12) = (/ &
        & 0.32830e+00_rb,0.73817e+00_rb,0.95981e+00_rb,0.10502e+01_rb,0.81194e+00_rb /)
      kbo(:, 5,43,12) = (/ &
        & 0.37763e+00_rb,0.79580e+00_rb,0.10196e+01_rb,0.10995e+01_rb,0.86317e+00_rb /)
      kbo(:, 1,44,12) = (/ &
        & 0.17494e+00_rb,0.54678e+00_rb,0.73637e+00_rb,0.84544e+00_rb,0.60542e+00_rb /)
      kbo(:, 2,44,12) = (/ &
        & 0.21877e+00_rb,0.60406e+00_rb,0.80575e+00_rb,0.91415e+00_rb,0.67322e+00_rb /)
      kbo(:, 3,44,12) = (/ &
        & 0.26445e+00_rb,0.66075e+00_rb,0.87287e+00_rb,0.97625e+00_rb,0.73598e+00_rb /)
      kbo(:, 4,44,12) = (/ &
        & 0.31161e+00_rb,0.71841e+00_rb,0.93827e+00_rb,0.10322e+01_rb,0.79326e+00_rb /)
      kbo(:, 5,44,12) = (/ &
        & 0.36052e+00_rb,0.77622e+00_rb,0.99972e+00_rb,0.10831e+01_rb,0.84620e+00_rb /)
      kbo(:, 1,45,12) = (/ &
        & 0.16016e+00_rb,0.52729e+00_rb,0.71224e+00_rb,0.82038e+00_rb,0.58172e+00_rb /)
      kbo(:, 2,45,12) = (/ &
        & 0.20336e+00_rb,0.58455e+00_rb,0.78227e+00_rb,0.89144e+00_rb,0.65050e+00_rb /)
      kbo(:, 3,45,12) = (/ &
        & 0.24861e+00_rb,0.64128e+00_rb,0.85023e+00_rb,0.95584e+00_rb,0.71509e+00_rb /)
      kbo(:, 4,45,12) = (/ &
        & 0.29518e+00_rb,0.69870e+00_rb,0.91640e+00_rb,0.10138e+01_rb,0.77417e+00_rb /)
      kbo(:, 5,45,12) = (/ &
        & 0.34361e+00_rb,0.75661e+00_rb,0.97947e+00_rb,0.10663e+01_rb,0.82874e+00_rb /)
      kbo(:, 1,46,12) = (/ &
        & 0.14495e+00_rb,0.50668e+00_rb,0.68614e+00_rb,0.79268e+00_rb,0.55619e+00_rb /)
      kbo(:, 2,46,12) = (/ &
        & 0.18729e+00_rb,0.56366e+00_rb,0.75707e+00_rb,0.86650e+00_rb,0.62580e+00_rb /)
      kbo(:, 3,46,12) = (/ &
        & 0.23189e+00_rb,0.62077e+00_rb,0.82600e+00_rb,0.93336e+00_rb,0.69232e+00_rb /)
      kbo(:, 4,46,12) = (/ &
        & 0.27798e+00_rb,0.67784e+00_rb,0.89269e+00_rb,0.99345e+00_rb,0.75352e+00_rb /)
      kbo(:, 5,46,12) = (/ &
        & 0.32583e+00_rb,0.73560e+00_rb,0.95715e+00_rb,0.10480e+01_rb,0.80957e+00_rb /)
      kbo(:, 1,47,12) = (/ &
        & 0.12858e+00_rb,0.48369e+00_rb,0.65638e+00_rb,0.76017e+00_rb,0.52730e+00_rb /)
      kbo(:, 2,47,12) = (/ &
        & 0.16969e+00_rb,0.54041e+00_rb,0.72859e+00_rb,0.83745e+00_rb,0.59783e+00_rb /)
      kbo(:, 3,47,12) = (/ &
        & 0.21344e+00_rb,0.59783e+00_rb,0.79837e+00_rb,0.90704e+00_rb,0.66606e+00_rb /)
      kbo(:, 4,47,12) = (/ &
        & 0.25910e+00_rb,0.65455e+00_rb,0.86586e+00_rb,0.96989e+00_rb,0.72949e+00_rb /)
      kbo(:, 5,47,12) = (/ &
        & 0.30618e+00_rb,0.71222e+00_rb,0.93155e+00_rb,0.10265e+01_rb,0.78728e+00_rb /)
      kbo(:, 1,48,12) = (/ &
        & 0.11298e+00_rb,0.46082e+00_rb,0.62642e+00_rb,0.72640e+00_rb,0.49809e+00_rb /)
      kbo(:, 2,48,12) = (/ &
        & 0.15246e+00_rb,0.51740e+00_rb,0.69983e+00_rb,0.80735e+00_rb,0.56961e+00_rb /)
      kbo(:, 3,48,12) = (/ &
        & 0.19539e+00_rb,0.57459e+00_rb,0.77041e+00_rb,0.87978e+00_rb,0.63885e+00_rb /)
      kbo(:, 4,48,12) = (/ &
        & 0.24045e+00_rb,0.63157e+00_rb,0.83892e+00_rb,0.94534e+00_rb,0.70441e+00_rb /)
      kbo(:, 5,48,12) = (/ &
        & 0.28683e+00_rb,0.68881e+00_rb,0.90534e+00_rb,0.10043e+01_rb,0.76449e+00_rb /)
      kbo(:, 1,49,12) = (/ &
        & 0.98247e-01_rb,0.43787e+00_rb,0.59630e+00_rb,0.69142e+00_rb,0.46886e+00_rb /)
      kbo(:, 2,49,12) = (/ &
        & 0.13602e+00_rb,0.49453e+00_rb,0.67051e+00_rb,0.77567e+00_rb,0.54098e+00_rb /)
      kbo(:, 3,49,12) = (/ &
        & 0.17773e+00_rb,0.55136e+00_rb,0.74221e+00_rb,0.85142e+00_rb,0.61109e+00_rb /)
      kbo(:, 4,49,12) = (/ &
        & 0.22196e+00_rb,0.60876e+00_rb,0.81162e+00_rb,0.91969e+00_rb,0.67863e+00_rb /)
      kbo(:, 5,49,12) = (/ &
        & 0.26789e+00_rb,0.66564e+00_rb,0.87866e+00_rb,0.98128e+00_rb,0.74108e+00_rb /)
      kbo(:, 1,50,12) = (/ &
        & 0.85448e-01_rb,0.41621e+00_rb,0.56763e+00_rb,0.65757e+00_rb,0.44117e+00_rb /)
      kbo(:, 2,50,12) = (/ &
        & 0.12109e+00_rb,0.47310e+00_rb,0.64247e+00_rb,0.74462e+00_rb,0.51379e+00_rb /)
      kbo(:, 3,50,12) = (/ &
        & 0.16156e+00_rb,0.52985e+00_rb,0.71554e+00_rb,0.82373e+00_rb,0.58488e+00_rb /)
      kbo(:, 4,50,12) = (/ &
        & 0.20488e+00_rb,0.58711e+00_rb,0.78559e+00_rb,0.89468e+00_rb,0.65362e+00_rb /)
      kbo(:, 5,50,12) = (/ &
        & 0.25037e+00_rb,0.64398e+00_rb,0.85353e+00_rb,0.95880e+00_rb,0.71804e+00_rb /)
      kbo(:, 1,51,12) = (/ &
        & 0.73759e-01_rb,0.39485e+00_rb,0.53977e+00_rb,0.62407e+00_rb,0.41437e+00_rb /)
      kbo(:, 2,51,12) = (/ &
        & 0.10727e+00_rb,0.45237e+00_rb,0.61531e+00_rb,0.71353e+00_rb,0.48727e+00_rb /)
      kbo(:, 3,51,12) = (/ &
        & 0.14620e+00_rb,0.50899e+00_rb,0.68916e+00_rb,0.79586e+00_rb,0.55909e+00_rb /)
      kbo(:, 4,51,12) = (/ &
        & 0.18867e+00_rb,0.56597e+00_rb,0.76009e+00_rb,0.86950e+00_rb,0.62868e+00_rb /)
      kbo(:, 5,51,12) = (/ &
        & 0.23351e+00_rb,0.62321e+00_rb,0.82900e+00_rb,0.93613e+00_rb,0.69507e+00_rb /)
      kbo(:, 1,52,12) = (/ &
        & 0.62975e-01_rb,0.37352e+00_rb,0.51168e+00_rb,0.59007e+00_rb,0.38790e+00_rb /)
      kbo(:, 2,52,12) = (/ &
        & 0.94238e-01_rb,0.43155e+00_rb,0.58786e+00_rb,0.68153e+00_rb,0.46072e+00_rb /)
      kbo(:, 3,52,12) = (/ &
        & 0.13138e+00_rb,0.48824e+00_rb,0.66233e+00_rb,0.76673e+00_rb,0.53305e+00_rb /)
      kbo(:, 4,52,12) = (/ &
        & 0.17291e+00_rb,0.54507e+00_rb,0.73444e+00_rb,0.84335e+00_rb,0.60343e+00_rb /)
      kbo(:, 5,52,12) = (/ &
        & 0.21681e+00_rb,0.60245e+00_rb,0.80402e+00_rb,0.91249e+00_rb,0.67147e+00_rb /)
      kbo(:, 1,53,12) = (/ &
        & 0.53060e-01_rb,0.35230e+00_rb,0.48353e+00_rb,0.55561e+00_rb,0.36178e+00_rb /)
      kbo(:, 2,53,12) = (/ &
        & 0.81984e-01_rb,0.41041e+00_rb,0.56005e+00_rb,0.64855e+00_rb,0.43393e+00_rb /)
      kbo(:, 3,53,12) = (/ &
        & 0.11713e+00_rb,0.46754e+00_rb,0.63518e+00_rb,0.73640e+00_rb,0.50665e+00_rb /)
      kbo(:, 4,53,12) = (/ &
        & 0.15723e+00_rb,0.52417e+00_rb,0.70847e+00_rb,0.81636e+00_rb,0.57800e+00_rb /)
      kbo(:, 5,53,12) = (/ &
        & 0.20038e+00_rb,0.58144e+00_rb,0.77877e+00_rb,0.88807e+00_rb,0.64700e+00_rb /)
      kbo(:, 1,54,12) = (/ &
        & 0.44825e-01_rb,0.33285e+00_rb,0.45764e+00_rb,0.52392e+00_rb,0.33821e+00_rb /)
      kbo(:, 2,54,12) = (/ &
        & 0.71480e-01_rb,0.39083e+00_rb,0.53456e+00_rb,0.61778e+00_rb,0.40943e+00_rb /)
      kbo(:, 3,54,12) = (/ &
        & 0.10463e+00_rb,0.44857e+00_rb,0.61031e+00_rb,0.70773e+00_rb,0.48234e+00_rb /)
      kbo(:, 4,54,12) = (/ &
        & 0.14329e+00_rb,0.50511e+00_rb,0.68427e+00_rb,0.79060e+00_rb,0.55433e+00_rb /)
      kbo(:, 5,54,12) = (/ &
        & 0.18561e+00_rb,0.56209e+00_rb,0.75543e+00_rb,0.86487e+00_rb,0.62409e+00_rb /)
      kbo(:, 1,55,12) = (/ &
        & 0.37713e-01_rb,0.31417e+00_rb,0.43292e+00_rb,0.49354e+00_rb,0.31617e+00_rb /)
      kbo(:, 2,55,12) = (/ &
        & 0.62120e-01_rb,0.37210e+00_rb,0.50985e+00_rb,0.58786e+00_rb,0.38623e+00_rb /)
      kbo(:, 3,55,12) = (/ &
        & 0.93283e-01_rb,0.43021e+00_rb,0.58608e+00_rb,0.67950e+00_rb,0.45902e+00_rb /)
      kbo(:, 4,55,12) = (/ &
        & 0.13035e+00_rb,0.48690e+00_rb,0.66064e+00_rb,0.76487e+00_rb,0.53142e+00_rb /)
      kbo(:, 5,55,12) = (/ &
        & 0.17183e+00_rb,0.54374e+00_rb,0.73284e+00_rb,0.84168e+00_rb,0.60189e+00_rb /)
      kbo(:, 1,56,12) = (/ &
        & 0.31364e-01_rb,0.29533e+00_rb,0.40826e+00_rb,0.46346e+00_rb,0.29456e+00_rb /)
      kbo(:, 2,56,12) = (/ &
        & 0.53447e-01_rb,0.35352e+00_rb,0.48522e+00_rb,0.55771e+00_rb,0.36336e+00_rb /)
      kbo(:, 3,56,12) = (/ &
        & 0.82579e-01_rb,0.41171e+00_rb,0.56173e+00_rb,0.65061e+00_rb,0.43556e+00_rb /)
      kbo(:, 4,56,12) = (/ &
        & 0.11788e+00_rb,0.46877e+00_rb,0.63683e+00_rb,0.73828e+00_rb,0.50830e+00_rb /)
      kbo(:, 5,56,12) = (/ &
        & 0.15810e+00_rb,0.52547e+00_rb,0.71011e+00_rb,0.81806e+00_rb,0.57961e+00_rb /)
      kbo(:, 1,57,12) = (/ &
        & 0.25728e-01_rb,0.27641e+00_rb,0.38366e+00_rb,0.43364e+00_rb,0.27324e+00_rb /)
      kbo(:, 2,57,12) = (/ &
        & 0.45538e-01_rb,0.33495e+00_rb,0.46047e+00_rb,0.52741e+00_rb,0.34078e+00_rb /)
      kbo(:, 3,57,12) = (/ &
        & 0.72492e-01_rb,0.39302e+00_rb,0.53741e+00_rb,0.62128e+00_rb,0.41216e+00_rb /)
      kbo(:, 4,57,12) = (/ &
        & 0.10590e+00_rb,0.45065e+00_rb,0.61311e+00_rb,0.71103e+00_rb,0.48510e+00_rb /)
      kbo(:, 5,57,12) = (/ &
        & 0.14476e+00_rb,0.50725e+00_rb,0.68700e+00_rb,0.79354e+00_rb,0.55702e+00_rb /)
      kbo(:, 1,58,12) = (/ &
        & 0.21034e-01_rb,0.25862e+00_rb,0.36035e+00_rb,0.40559e+00_rb,0.25327e+00_rb /)
      kbo(:, 2,58,12) = (/ &
        & 0.38718e-01_rb,0.31732e+00_rb,0.43714e+00_rb,0.49876e+00_rb,0.31992e+00_rb /)
      kbo(:, 3,58,12) = (/ &
        & 0.63552e-01_rb,0.37529e+00_rb,0.51407e+00_rb,0.59305e+00_rb,0.39018e+00_rb /)
      kbo(:, 4,58,12) = (/ &
        & 0.95105e-01_rb,0.43334e+00_rb,0.59030e+00_rb,0.68443e+00_rb,0.46307e+00_rb /)
      kbo(:, 5,58,12) = (/ &
        & 0.13248e+00_rb,0.49002e+00_rb,0.66471e+00_rb,0.76934e+00_rb,0.53538e+00_rb /)
      kbo(:, 1,59,12) = (/ &
        & 0.19312e-01_rb,0.25156e+00_rb,0.35103e+00_rb,0.39450e+00_rb,0.24538e+00_rb /)
      kbo(:, 2,59,12) = (/ &
        & 0.36163e-01_rb,0.31020e+00_rb,0.42773e+00_rb,0.48723e+00_rb,0.31163e+00_rb /)
      kbo(:, 3,59,12) = (/ &
        & 0.60143e-01_rb,0.36820e+00_rb,0.50471e+00_rb,0.58164e+00_rb,0.38146e+00_rb /)
      kbo(:, 4,59,12) = (/ &
        & 0.90924e-01_rb,0.42637e+00_rb,0.58105e+00_rb,0.67358e+00_rb,0.45417e+00_rb /)
      kbo(:, 5,59,12) = (/ &
        & 0.12765e+00_rb,0.48313e+00_rb,0.65572e+00_rb,0.75941e+00_rb,0.52665e+00_rb /)
      kbo(:, 1,13,13) = (/ &
        & 0.25702e+01_rb,0.27735e+01_rb,0.28760e+01_rb,0.25966e+01_rb,0.23968e+01_rb /)
      kbo(:, 2,13,13) = (/ &
        & 0.25597e+01_rb,0.27776e+01_rb,0.28903e+01_rb,0.26158e+01_rb,0.24224e+01_rb /)
      kbo(:, 3,13,13) = (/ &
        & 0.25494e+01_rb,0.27801e+01_rb,0.29040e+01_rb,0.26351e+01_rb,0.24494e+01_rb /)
      kbo(:, 4,13,13) = (/ &
        & 0.25338e+01_rb,0.27802e+01_rb,0.29182e+01_rb,0.26543e+01_rb,0.24747e+01_rb /)
      kbo(:, 5,13,13) = (/ &
        & 0.25177e+01_rb,0.27760e+01_rb,0.29320e+01_rb,0.26728e+01_rb,0.24963e+01_rb /)
      kbo(:, 1,14,13) = (/ &
        & 0.23659e+01_rb,0.26936e+01_rb,0.28410e+01_rb,0.26130e+01_rb,0.23363e+01_rb /)
      kbo(:, 2,14,13) = (/ &
        & 0.23577e+01_rb,0.27024e+01_rb,0.28615e+01_rb,0.26344e+01_rb,0.23684e+01_rb /)
      kbo(:, 3,14,13) = (/ &
        & 0.23468e+01_rb,0.27102e+01_rb,0.28819e+01_rb,0.26565e+01_rb,0.24004e+01_rb /)
      kbo(:, 4,14,13) = (/ &
        & 0.23350e+01_rb,0.27153e+01_rb,0.29051e+01_rb,0.26785e+01_rb,0.24267e+01_rb /)
      kbo(:, 5,14,13) = (/ &
        & 0.23244e+01_rb,0.27141e+01_rb,0.29262e+01_rb,0.26967e+01_rb,0.24480e+01_rb /)
      kbo(:, 1,15,13) = (/ &
        & 0.21606e+01_rb,0.25927e+01_rb,0.27805e+01_rb,0.26146e+01_rb,0.22773e+01_rb /)
      kbo(:, 2,15,13) = (/ &
        & 0.21537e+01_rb,0.26084e+01_rb,0.28087e+01_rb,0.26417e+01_rb,0.23162e+01_rb /)
      kbo(:, 3,15,13) = (/ &
        & 0.21448e+01_rb,0.26237e+01_rb,0.28408e+01_rb,0.26683e+01_rb,0.23515e+01_rb /)
      kbo(:, 4,15,13) = (/ &
        & 0.21380e+01_rb,0.26327e+01_rb,0.28720e+01_rb,0.26931e+01_rb,0.23792e+01_rb /)
      kbo(:, 5,15,13) = (/ &
        & 0.21347e+01_rb,0.26387e+01_rb,0.29018e+01_rb,0.27159e+01_rb,0.24036e+01_rb /)
      kbo(:, 1,16,13) = (/ &
        & 0.19588e+01_rb,0.24824e+01_rb,0.27086e+01_rb,0.26036e+01_rb,0.22170e+01_rb /)
      kbo(:, 2,16,13) = (/ &
        & 0.19544e+01_rb,0.25053e+01_rb,0.27466e+01_rb,0.26370e+01_rb,0.22638e+01_rb /)
      kbo(:, 3,16,13) = (/ &
        & 0.19508e+01_rb,0.25256e+01_rb,0.27882e+01_rb,0.26694e+01_rb,0.23023e+01_rb /)
      kbo(:, 4,16,13) = (/ &
        & 0.19522e+01_rb,0.25422e+01_rb,0.28288e+01_rb,0.26997e+01_rb,0.23338e+01_rb /)
      kbo(:, 5,16,13) = (/ &
        & 0.19669e+01_rb,0.25568e+01_rb,0.28660e+01_rb,0.27268e+01_rb,0.23639e+01_rb /)
      kbo(:, 1,17,13) = (/ &
        & 0.17683e+01_rb,0.23717e+01_rb,0.26296e+01_rb,0.25828e+01_rb,0.21586e+01_rb /)
      kbo(:, 2,17,13) = (/ &
        & 0.17670e+01_rb,0.24002e+01_rb,0.26798e+01_rb,0.26243e+01_rb,0.22104e+01_rb /)
      kbo(:, 3,17,13) = (/ &
        & 0.17705e+01_rb,0.24261e+01_rb,0.27313e+01_rb,0.26643e+01_rb,0.22525e+01_rb /)
      kbo(:, 4,17,13) = (/ &
        & 0.17889e+01_rb,0.24495e+01_rb,0.27802e+01_rb,0.27001e+01_rb,0.22912e+01_rb /)
      kbo(:, 5,17,13) = (/ &
        & 0.18206e+01_rb,0.24734e+01_rb,0.28261e+01_rb,0.27304e+01_rb,0.23278e+01_rb /)
      kbo(:, 1,18,13) = (/ &
        & 0.15903e+01_rb,0.22606e+01_rb,0.25530e+01_rb,0.25561e+01_rb,0.21033e+01_rb /)
      kbo(:, 2,18,13) = (/ &
        & 0.15961e+01_rb,0.22974e+01_rb,0.26139e+01_rb,0.26069e+01_rb,0.21585e+01_rb /)
      kbo(:, 3,18,13) = (/ &
        & 0.16151e+01_rb,0.23303e+01_rb,0.26738e+01_rb,0.26546e+01_rb,0.22069e+01_rb /)
      kbo(:, 4,18,13) = (/ &
        & 0.16495e+01_rb,0.23625e+01_rb,0.27316e+01_rb,0.26970e+01_rb,0.22532e+01_rb /)
      kbo(:, 5,18,13) = (/ &
        & 0.16970e+01_rb,0.23957e+01_rb,0.27859e+01_rb,0.27320e+01_rb,0.22970e+01_rb /)
      kbo(:, 1,19,13) = (/ &
        & 0.14257e+01_rb,0.21563e+01_rb,0.24826e+01_rb,0.25271e+01_rb,0.20500e+01_rb /)
      kbo(:, 2,19,13) = (/ &
        & 0.14448e+01_rb,0.21999e+01_rb,0.25521e+01_rb,0.25882e+01_rb,0.21113e+01_rb /)
      kbo(:, 3,19,13) = (/ &
        & 0.14809e+01_rb,0.22420e+01_rb,0.26202e+01_rb,0.26436e+01_rb,0.21676e+01_rb /)
      kbo(:, 4,19,13) = (/ &
        & 0.15301e+01_rb,0.22839e+01_rb,0.26861e+01_rb,0.26918e+01_rb,0.22215e+01_rb /)
      kbo(:, 5,19,13) = (/ &
        & 0.15917e+01_rb,0.23278e+01_rb,0.27486e+01_rb,0.27324e+01_rb,0.22720e+01_rb /)
      kbo(:, 1,20,13) = (/ &
        & 0.12800e+01_rb,0.20626e+01_rb,0.24230e+01_rb,0.25008e+01_rb,0.20040e+01_rb /)
      kbo(:, 2,20,13) = (/ &
        & 0.13152e+01_rb,0.21132e+01_rb,0.24996e+01_rb,0.25711e+01_rb,0.20725e+01_rb /)
      kbo(:, 3,20,13) = (/ &
        & 0.13653e+01_rb,0.21643e+01_rb,0.25750e+01_rb,0.26341e+01_rb,0.21362e+01_rb /)
      kbo(:, 4,20,13) = (/ &
        & 0.14290e+01_rb,0.22166e+01_rb,0.26493e+01_rb,0.26871e+01_rb,0.21969e+01_rb /)
      kbo(:, 5,20,13) = (/ &
        & 0.15047e+01_rb,0.22739e+01_rb,0.27186e+01_rb,0.27332e+01_rb,0.22536e+01_rb /)
      kbo(:, 1,21,13) = (/ &
        & 0.11565e+01_rb,0.19804e+01_rb,0.23723e+01_rb,0.24790e+01_rb,0.19652e+01_rb /)
      kbo(:, 2,21,13) = (/ &
        & 0.12043e+01_rb,0.20401e+01_rb,0.24563e+01_rb,0.25572e+01_rb,0.20406e+01_rb /)
      kbo(:, 3,21,13) = (/ &
        & 0.12682e+01_rb,0.20995e+01_rb,0.25386e+01_rb,0.26265e+01_rb,0.21117e+01_rb /)
      kbo(:, 4,21,13) = (/ &
        & 0.13446e+01_rb,0.21634e+01_rb,0.26197e+01_rb,0.26846e+01_rb,0.21783e+01_rb /)
      kbo(:, 5,21,13) = (/ &
        & 0.14328e+01_rb,0.22322e+01_rb,0.26943e+01_rb,0.27351e+01_rb,0.22400e+01_rb /)
      kbo(:, 1,22,13) = (/ &
        & 0.10567e+01_rb,0.19161e+01_rb,0.23380e+01_rb,0.24675e+01_rb,0.19394e+01_rb /)
      kbo(:, 2,22,13) = (/ &
        & 0.11171e+01_rb,0.19843e+01_rb,0.24279e+01_rb,0.25525e+01_rb,0.20217e+01_rb /)
      kbo(:, 3,22,13) = (/ &
        & 0.11926e+01_rb,0.20529e+01_rb,0.25167e+01_rb,0.26264e+01_rb,0.20987e+01_rb /)
      kbo(:, 4,22,13) = (/ &
        & 0.12819e+01_rb,0.21280e+01_rb,0.26028e+01_rb,0.26881e+01_rb,0.21696e+01_rb /)
      kbo(:, 5,22,13) = (/ &
        & 0.13818e+01_rb,0.22069e+01_rb,0.26807e+01_rb,0.27417e+01_rb,0.22350e+01_rb /)
      kbo(:, 1,23,13) = (/ &
        & 0.97722e+00_rb,0.18665e+01_rb,0.23140e+01_rb,0.24623e+01_rb,0.19226e+01_rb /)
      kbo(:, 2,23,13) = (/ &
        & 0.10487e+01_rb,0.19424e+01_rb,0.24103e+01_rb,0.25530e+01_rb,0.20109e+01_rb /)
      kbo(:, 3,23,13) = (/ &
        & 0.11351e+01_rb,0.20216e+01_rb,0.25035e+01_rb,0.26294e+01_rb,0.20926e+01_rb /)
      kbo(:, 4,23,13) = (/ &
        & 0.12356e+01_rb,0.21056e+01_rb,0.25931e+01_rb,0.26937e+01_rb,0.21670e+01_rb /)
      kbo(:, 5,23,13) = (/ &
        & 0.13470e+01_rb,0.21926e+01_rb,0.26734e+01_rb,0.27495e+01_rb,0.22345e+01_rb /)
      kbo(:, 1,24,13) = (/ &
        & 0.91609e+00_rb,0.18302e+01_rb,0.23004e+01_rb,0.24641e+01_rb,0.19149e+01_rb /)
      kbo(:, 2,24,13) = (/ &
        & 0.99757e+00_rb,0.19150e+01_rb,0.24014e+01_rb,0.25579e+01_rb,0.20076e+01_rb /)
      kbo(:, 3,24,13) = (/ &
        & 0.10945e+01_rb,0.20031e+01_rb,0.24987e+01_rb,0.26360e+01_rb,0.20926e+01_rb /)
      kbo(:, 4,24,13) = (/ &
        & 0.12058e+01_rb,0.20942e+01_rb,0.25896e+01_rb,0.27020e+01_rb,0.21691e+01_rb /)
      kbo(:, 5,24,13) = (/ &
        & 0.13269e+01_rb,0.21869e+01_rb,0.26718e+01_rb,0.27594e+01_rb,0.22381e+01_rb /)
      kbo(:, 1,25,13) = (/ &
        & 0.87183e+00_rb,0.18068e+01_rb,0.22964e+01_rb,0.24719e+01_rb,0.19153e+01_rb /)
      kbo(:, 2,25,13) = (/ &
        & 0.96269e+00_rb,0.19004e+01_rb,0.24012e+01_rb,0.25669e+01_rb,0.20109e+01_rb /)
      kbo(:, 3,25,13) = (/ &
        & 0.10701e+01_rb,0.19953e+01_rb,0.24998e+01_rb,0.26458e+01_rb,0.20980e+01_rb /)
      kbo(:, 4,25,13) = (/ &
        & 0.11910e+01_rb,0.20919e+01_rb,0.25913e+01_rb,0.27126e+01_rb,0.21754e+01_rb /)
      kbo(:, 5,25,13) = (/ &
        & 0.13188e+01_rb,0.21891e+01_rb,0.26749e+01_rb,0.27711e+01_rb,0.22449e+01_rb /)
      kbo(:, 1,26,13) = (/ &
        & 0.84313e+00_rb,0.17965e+01_rb,0.23011e+01_rb,0.24858e+01_rb,0.19238e+01_rb /)
      kbo(:, 2,26,13) = (/ &
        & 0.94434e+00_rb,0.18970e+01_rb,0.24073e+01_rb,0.25798e+01_rb,0.20205e+01_rb /)
      kbo(:, 3,26,13) = (/ &
        & 0.10612e+01_rb,0.19972e+01_rb,0.25069e+01_rb,0.26589e+01_rb,0.21080e+01_rb /)
      kbo(:, 4,26,13) = (/ &
        & 0.11887e+01_rb,0.20979e+01_rb,0.25988e+01_rb,0.27257e+01_rb,0.21853e+01_rb /)
      kbo(:, 5,26,13) = (/ &
        & 0.13225e+01_rb,0.21985e+01_rb,0.26831e+01_rb,0.27841e+01_rb,0.22550e+01_rb /)
      kbo(:, 1,27,13) = (/ &
        & 0.82783e+00_rb,0.17960e+01_rb,0.23110e+01_rb,0.25019e+01_rb,0.19363e+01_rb /)
      kbo(:, 2,27,13) = (/ &
        & 0.93873e+00_rb,0.19014e+01_rb,0.24182e+01_rb,0.25950e+01_rb,0.20338e+01_rb /)
      kbo(:, 3,27,13) = (/ &
        & 0.10628e+01_rb,0.20052e+01_rb,0.25176e+01_rb,0.26735e+01_rb,0.21206e+01_rb /)
      kbo(:, 4,27,13) = (/ &
        & 0.11961e+01_rb,0.21092e+01_rb,0.26097e+01_rb,0.27398e+01_rb,0.21974e+01_rb /)
      kbo(:, 5,27,13) = (/ &
        & 0.13342e+01_rb,0.22123e+01_rb,0.26944e+01_rb,0.27973e+01_rb,0.22665e+01_rb /)
      kbo(:, 1,28,13) = (/ &
        & 0.82452e+00_rb,0.18027e+01_rb,0.23247e+01_rb,0.25200e+01_rb,0.19523e+01_rb /)
      kbo(:, 2,28,13) = (/ &
        & 0.94277e+00_rb,0.19121e+01_rb,0.24323e+01_rb,0.26114e+01_rb,0.20494e+01_rb /)
      kbo(:, 3,28,13) = (/ &
        & 0.10725e+01_rb,0.20183e+01_rb,0.25313e+01_rb,0.26888e+01_rb,0.21352e+01_rb /)
      kbo(:, 4,28,13) = (/ &
        & 0.12107e+01_rb,0.21244e+01_rb,0.26233e+01_rb,0.27544e+01_rb,0.22110e+01_rb /)
      kbo(:, 5,28,13) = (/ &
        & 0.13516e+01_rb,0.22292e+01_rb,0.27078e+01_rb,0.28106e+01_rb,0.22794e+01_rb /)
      kbo(:, 1,29,13) = (/ &
        & 0.83075e+00_rb,0.18160e+01_rb,0.23425e+01_rb,0.25401e+01_rb,0.19712e+01_rb /)
      kbo(:, 2,29,13) = (/ &
        & 0.95513e+00_rb,0.19278e+01_rb,0.24494e+01_rb,0.26294e+01_rb,0.20669e+01_rb /)
      kbo(:, 3,29,13) = (/ &
        & 0.10900e+01_rb,0.20358e+01_rb,0.25478e+01_rb,0.27053e+01_rb,0.21513e+01_rb /)
      kbo(:, 4,29,13) = (/ &
        & 0.12315e+01_rb,0.21431e+01_rb,0.26391e+01_rb,0.27692e+01_rb,0.22259e+01_rb /)
      kbo(:, 5,29,13) = (/ &
        & 0.13745e+01_rb,0.22488e+01_rb,0.27235e+01_rb,0.28240e+01_rb,0.22934e+01_rb /)
      kbo(:, 1,30,13) = (/ &
        & 0.84416e+00_rb,0.18336e+01_rb,0.23624e+01_rb,0.25609e+01_rb,0.19916e+01_rb /)
      kbo(:, 2,30,13) = (/ &
        & 0.97368e+00_rb,0.19464e+01_rb,0.24682e+01_rb,0.26481e+01_rb,0.20854e+01_rb /)
      kbo(:, 3,30,13) = (/ &
        & 0.11126e+01_rb,0.20559e+01_rb,0.25658e+01_rb,0.27219e+01_rb,0.21680e+01_rb /)
      kbo(:, 4,30,13) = (/ &
        & 0.12564e+01_rb,0.21643e+01_rb,0.26566e+01_rb,0.27840e+01_rb,0.22414e+01_rb /)
      kbo(:, 5,30,13) = (/ &
        & 0.14008e+01_rb,0.22700e+01_rb,0.27405e+01_rb,0.28371e+01_rb,0.23076e+01_rb /)
      kbo(:, 1,31,13) = (/ &
        & 0.86384e+00_rb,0.18548e+01_rb,0.23844e+01_rb,0.25824e+01_rb,0.20133e+01_rb /)
      kbo(:, 2,31,13) = (/ &
        & 0.99753e+00_rb,0.19681e+01_rb,0.24888e+01_rb,0.26671e+01_rb,0.21046e+01_rb /)
      kbo(:, 3,31,13) = (/ &
        & 0.11394e+01_rb,0.20787e+01_rb,0.25856e+01_rb,0.27387e+01_rb,0.21855e+01_rb /)
      kbo(:, 4,31,13) = (/ &
        & 0.12850e+01_rb,0.21876e+01_rb,0.26758e+01_rb,0.27988e+01_rb,0.22577e+01_rb /)
      kbo(:, 5,31,13) = (/ &
        & 0.14304e+01_rb,0.22931e+01_rb,0.27584e+01_rb,0.28498e+01_rb,0.23225e+01_rb /)
      kbo(:, 1,32,13) = (/ &
        & 0.88808e+00_rb,0.18788e+01_rb,0.24078e+01_rb,0.26044e+01_rb,0.20359e+01_rb /)
      kbo(:, 2,32,13) = (/ &
        & 0.10254e+01_rb,0.19921e+01_rb,0.25106e+01_rb,0.26862e+01_rb,0.21246e+01_rb /)
      kbo(:, 3,32,13) = (/ &
        & 0.11696e+01_rb,0.21031e+01_rb,0.26064e+01_rb,0.27554e+01_rb,0.22034e+01_rb /)
      kbo(:, 4,32,13) = (/ &
        & 0.13164e+01_rb,0.22122e+01_rb,0.26955e+01_rb,0.28135e+01_rb,0.22742e+01_rb /)
      kbo(:, 5,32,13) = (/ &
        & 0.14621e+01_rb,0.23175e+01_rb,0.27772e+01_rb,0.28624e+01_rb,0.23376e+01_rb /)
      kbo(:, 1,33,13) = (/ &
        & 0.91576e+00_rb,0.19044e+01_rb,0.24318e+01_rb,0.26262e+01_rb,0.20585e+01_rb /)
      kbo(:, 2,33,13) = (/ &
        & 0.10563e+01_rb,0.20178e+01_rb,0.25332e+01_rb,0.27053e+01_rb,0.21448e+01_rb /)
      kbo(:, 3,33,13) = (/ &
        & 0.12022e+01_rb,0.21286e+01_rb,0.26280e+01_rb,0.27719e+01_rb,0.22216e+01_rb /)
      kbo(:, 4,33,13) = (/ &
        & 0.13498e+01_rb,0.22379e+01_rb,0.27158e+01_rb,0.28276e+01_rb,0.22909e+01_rb /)
      kbo(:, 5,33,13) = (/ &
        & 0.14952e+01_rb,0.23427e+01_rb,0.27963e+01_rb,0.28748e+01_rb,0.23524e+01_rb /)
      kbo(:, 1,34,13) = (/ &
        & 0.94113e+00_rb,0.19270e+01_rb,0.24527e+01_rb,0.26448e+01_rb,0.20775e+01_rb /)
      kbo(:, 2,34,13) = (/ &
        & 0.10840e+01_rb,0.20405e+01_rb,0.25528e+01_rb,0.27213e+01_rb,0.21619e+01_rb /)
      kbo(:, 3,34,13) = (/ &
        & 0.12311e+01_rb,0.21512e+01_rb,0.26466e+01_rb,0.27855e+01_rb,0.22370e+01_rb /)
      kbo(:, 4,34,13) = (/ &
        & 0.13793e+01_rb,0.22602e+01_rb,0.27333e+01_rb,0.28395e+01_rb,0.23050e+01_rb /)
      kbo(:, 5,34,13) = (/ &
        & 0.15242e+01_rb,0.23644e+01_rb,0.28128e+01_rb,0.28850e+01_rb,0.23650e+01_rb /)
      kbo(:, 1,35,13) = (/ &
        & 0.95467e+00_rb,0.19391e+01_rb,0.24637e+01_rb,0.26549e+01_rb,0.20879e+01_rb /)
      kbo(:, 2,35,13) = (/ &
        & 0.10989e+01_rb,0.20529e+01_rb,0.25635e+01_rb,0.27301e+01_rb,0.21712e+01_rb /)
      kbo(:, 3,35,13) = (/ &
        & 0.12468e+01_rb,0.21638e+01_rb,0.26569e+01_rb,0.27931e+01_rb,0.22456e+01_rb /)
      kbo(:, 4,35,13) = (/ &
        & 0.13953e+01_rb,0.22726e+01_rb,0.27430e+01_rb,0.28463e+01_rb,0.23128e+01_rb /)
      kbo(:, 5,35,13) = (/ &
        & 0.15401e+01_rb,0.23764e+01_rb,0.28219e+01_rb,0.28908e+01_rb,0.23721e+01_rb /)
      kbo(:, 1,36,13) = (/ &
        & 0.95401e+00_rb,0.19393e+01_rb,0.24641e+01_rb,0.26563e+01_rb,0.20890e+01_rb /)
      kbo(:, 2,36,13) = (/ &
        & 0.10986e+01_rb,0.20534e+01_rb,0.25640e+01_rb,0.27314e+01_rb,0.21722e+01_rb /)
      kbo(:, 3,36,13) = (/ &
        & 0.12470e+01_rb,0.21646e+01_rb,0.26577e+01_rb,0.27946e+01_rb,0.22466e+01_rb /)
      kbo(:, 4,36,13) = (/ &
        & 0.13959e+01_rb,0.22738e+01_rb,0.27440e+01_rb,0.28477e+01_rb,0.23138e+01_rb /)
      kbo(:, 5,36,13) = (/ &
        & 0.15410e+01_rb,0.23778e+01_rb,0.28230e+01_rb,0.28922e+01_rb,0.23732e+01_rb /)
      kbo(:, 1,37,13) = (/ &
        & 0.93269e+00_rb,0.19221e+01_rb,0.24490e+01_rb,0.26452e+01_rb,0.20765e+01_rb /)
      kbo(:, 2,37,13) = (/ &
        & 0.10767e+01_rb,0.20372e+01_rb,0.25499e+01_rb,0.27223e+01_rb,0.21614e+01_rb /)
      kbo(:, 3,37,13) = (/ &
        & 0.12250e+01_rb,0.21490e+01_rb,0.26449e+01_rb,0.27873e+01_rb,0.22369e+01_rb /)
      kbo(:, 4,37,13) = (/ &
        & 0.13744e+01_rb,0.22588e+01_rb,0.27324e+01_rb,0.28416e+01_rb,0.23053e+01_rb /)
      kbo(:, 5,37,13) = (/ &
        & 0.15205e+01_rb,0.23639e+01_rb,0.28124e+01_rb,0.28871e+01_rb,0.23656e+01_rb /)
      kbo(:, 1,38,13) = (/ &
        & 0.91080e+00_rb,0.19040e+01_rb,0.24332e+01_rb,0.26332e+01_rb,0.20631e+01_rb /)
      kbo(:, 2,38,13) = (/ &
        & 0.10540e+01_rb,0.20201e+01_rb,0.25351e+01_rb,0.27123e+01_rb,0.21497e+01_rb /)
      kbo(:, 3,38,13) = (/ &
        & 0.12022e+01_rb,0.21326e+01_rb,0.26310e+01_rb,0.27790e+01_rb,0.22264e+01_rb /)
      kbo(:, 4,38,13) = (/ &
        & 0.13520e+01_rb,0.22432e+01_rb,0.27202e+01_rb,0.28347e+01_rb,0.22959e+01_rb /)
      kbo(:, 5,38,13) = (/ &
        & 0.14989e+01_rb,0.23490e+01_rb,0.28012e+01_rb,0.28815e+01_rb,0.23575e+01_rb /)
      kbo(:, 1,39,13) = (/ &
        & 0.88956e+00_rb,0.18861e+01_rb,0.24173e+01_rb,0.26207e+01_rb,0.20495e+01_rb /)
      kbo(:, 2,39,13) = (/ &
        & 0.10319e+01_rb,0.20032e+01_rb,0.25205e+01_rb,0.27021e+01_rb,0.21379e+01_rb /)
      kbo(:, 3,39,13) = (/ &
        & 0.11798e+01_rb,0.21163e+01_rb,0.26173e+01_rb,0.27704e+01_rb,0.22159e+01_rb /)
      kbo(:, 4,39,13) = (/ &
        & 0.13298e+01_rb,0.22272e+01_rb,0.27078e+01_rb,0.28277e+01_rb,0.22865e+01_rb /)
      kbo(:, 5,39,13) = (/ &
        & 0.14775e+01_rb,0.23341e+01_rb,0.27901e+01_rb,0.28757e+01_rb,0.23493e+01_rb /)
      kbo(:, 1,40,13) = (/ &
        & 0.85494e+00_rb,0.18558e+01_rb,0.23904e+01_rb,0.25985e+01_rb,0.20255e+01_rb /)
      kbo(:, 2,40,13) = (/ &
        & 0.99514e+00_rb,0.19745e+01_rb,0.24954e+01_rb,0.26836e+01_rb,0.21170e+01_rb /)
      kbo(:, 3,40,13) = (/ &
        & 0.11424e+01_rb,0.20885e+01_rb,0.25938e+01_rb,0.27547e+01_rb,0.21975e+01_rb /)
      kbo(:, 4,40,13) = (/ &
        & 0.12924e+01_rb,0.22001e+01_rb,0.26861e+01_rb,0.28148e+01_rb,0.22700e+01_rb /)
      kbo(:, 5,40,13) = (/ &
        & 0.14413e+01_rb,0.23083e+01_rb,0.27705e+01_rb,0.28664e+01_rb,0.23349e+01_rb /)
      kbo(:, 1,41,13) = (/ &
        & 0.81957e+00_rb,0.18237e+01_rb,0.23613e+01_rb,0.25741e+01_rb,0.19997e+01_rb /)
      kbo(:, 2,41,13) = (/ &
        & 0.95728e+00_rb,0.19441e+01_rb,0.24688e+01_rb,0.26632e+01_rb,0.20945e+01_rb /)
      kbo(:, 3,41,13) = (/ &
        & 0.11034e+01_rb,0.20591e+01_rb,0.25687e+01_rb,0.27378e+01_rb,0.21777e+01_rb /)
      kbo(:, 4,41,13) = (/ &
        & 0.12530e+01_rb,0.21714e+01_rb,0.26630e+01_rb,0.28005e+01_rb,0.22519e+01_rb /)
      kbo(:, 5,41,13) = (/ &
        & 0.14027e+01_rb,0.22807e+01_rb,0.27494e+01_rb,0.28533e+01_rb,0.23190e+01_rb /)
      kbo(:, 1,42,13) = (/ &
        & 0.78503e+00_rb,0.17918e+01_rb,0.23317e+01_rb,0.25487e+01_rb,0.19734e+01_rb /)
      kbo(:, 2,42,13) = (/ &
        & 0.92016e+00_rb,0.19132e+01_rb,0.24418e+01_rb,0.26418e+01_rb,0.20712e+01_rb /)
      kbo(:, 3,42,13) = (/ &
        & 0.10646e+01_rb,0.20294e+01_rb,0.25432e+01_rb,0.27200e+01_rb,0.21575e+01_rb /)
      kbo(:, 4,42,13) = (/ &
        & 0.12137e+01_rb,0.21426e+01_rb,0.26392e+01_rb,0.27859e+01_rb,0.22336e+01_rb /)
      kbo(:, 5,42,13) = (/ &
        & 0.13639e+01_rb,0.22528e+01_rb,0.27279e+01_rb,0.28411e+01_rb,0.23026e+01_rb /)
      kbo(:, 1,43,13) = (/ &
        & 0.74423e+00_rb,0.17527e+01_rb,0.22949e+01_rb,0.25159e+01_rb,0.19403e+01_rb /)
      kbo(:, 2,43,13) = (/ &
        & 0.87589e+00_rb,0.18750e+01_rb,0.24080e+01_rb,0.26143e+01_rb,0.20417e+01_rb /)
      kbo(:, 3,43,13) = (/ &
        & 0.10179e+01_rb,0.19932e+01_rb,0.25118e+01_rb,0.26972e+01_rb,0.21315e+01_rb /)
      kbo(:, 4,43,13) = (/ &
        & 0.11661e+01_rb,0.21073e+01_rb,0.26096e+01_rb,0.27665e+01_rb,0.22105e+01_rb /)
      kbo(:, 5,43,13) = (/ &
        & 0.13165e+01_rb,0.22185e+01_rb,0.27008e+01_rb,0.28247e+01_rb,0.22817e+01_rb /)
      kbo(:, 1,44,13) = (/ &
        & 0.70183e+00_rb,0.17101e+01_rb,0.22539e+01_rb,0.24783e+01_rb,0.19029e+01_rb /)
      kbo(:, 2,44,13) = (/ &
        & 0.82940e+00_rb,0.18334e+01_rb,0.23707e+01_rb,0.25829e+01_rb,0.20083e+01_rb /)
      kbo(:, 3,44,13) = (/ &
        & 0.96829e+00_rb,0.19537e+01_rb,0.24774e+01_rb,0.26707e+01_rb,0.21022e+01_rb /)
      kbo(:, 4,44,13) = (/ &
        & 0.11151e+01_rb,0.20687e+01_rb,0.25770e+01_rb,0.27443e+01_rb,0.21846e+01_rb /)
      kbo(:, 5,44,13) = (/ &
        & 0.12651e+01_rb,0.21808e+01_rb,0.26707e+01_rb,0.28061e+01_rb,0.22584e+01_rb /)
      kbo(:, 1,45,13) = (/ &
        & 0.66118e+00_rb,0.16676e+01_rb,0.22119e+01_rb,0.24384e+01_rb,0.18634e+01_rb /)
      kbo(:, 2,45,13) = (/ &
        & 0.78445e+00_rb,0.17917e+01_rb,0.23321e+01_rb,0.25495e+01_rb,0.19739e+01_rb /)
      kbo(:, 3,45,13) = (/ &
        & 0.91984e+00_rb,0.19133e+01_rb,0.24421e+01_rb,0.26427e+01_rb,0.20718e+01_rb /)
      kbo(:, 4,45,13) = (/ &
        & 0.10646e+01_rb,0.20301e+01_rb,0.25439e+01_rb,0.27211e+01_rb,0.21582e+01_rb /)
      kbo(:, 5,45,13) = (/ &
        & 0.12138e+01_rb,0.21430e+01_rb,0.26395e+01_rb,0.27867e+01_rb,0.22343e+01_rb /)
      kbo(:, 1,46,13) = (/ &
        & 0.61933e+00_rb,0.16221e+01_rb,0.21659e+01_rb,0.23935e+01_rb,0.18196e+01_rb /)
      kbo(:, 2,46,13) = (/ &
        & 0.73807e+00_rb,0.17470e+01_rb,0.22898e+01_rb,0.25118e+01_rb,0.19358e+01_rb /)
      kbo(:, 3,46,13) = (/ &
        & 0.86934e+00_rb,0.18696e+01_rb,0.24034e+01_rb,0.26109e+01_rb,0.20379e+01_rb /)
      kbo(:, 4,46,13) = (/ &
        & 0.10113e+01_rb,0.19884e+01_rb,0.25076e+01_rb,0.26946e+01_rb,0.21283e+01_rb /)
      kbo(:, 5,46,13) = (/ &
        & 0.11593e+01_rb,0.21025e+01_rb,0.26055e+01_rb,0.27643e+01_rb,0.22076e+01_rb /)
      kbo(:, 1,47,13) = (/ &
        & 0.57374e+00_rb,0.15709e+01_rb,0.21131e+01_rb,0.23400e+01_rb,0.17680e+01_rb /)
      kbo(:, 2,47,13) = (/ &
        & 0.68838e+00_rb,0.16965e+01_rb,0.22409e+01_rb,0.24665e+01_rb,0.18907e+01_rb /)
      kbo(:, 3,47,13) = (/ &
        & 0.81478e+00_rb,0.18201e+01_rb,0.23589e+01_rb,0.25731e+01_rb,0.19980e+01_rb /)
      kbo(:, 4,47,13) = (/ &
        & 0.95278e+00_rb,0.19412e+01_rb,0.24665e+01_rb,0.26626e+01_rb,0.20931e+01_rb /)
      kbo(:, 5,47,13) = (/ &
        & 0.10991e+01_rb,0.20567e+01_rb,0.25667e+01_rb,0.27376e+01_rb,0.21769e+01_rb /)
      kbo(:, 1,48,13) = (/ &
        & 0.52946e+00_rb,0.15198e+01_rb,0.20592e+01_rb,0.22834e+01_rb,0.17136e+01_rb /)
      kbo(:, 2,48,13) = (/ &
        & 0.64089e+00_rb,0.16459e+01_rb,0.21904e+01_rb,0.24180e+01_rb,0.18431e+01_rb /)
      kbo(:, 3,48,13) = (/ &
        & 0.76211e+00_rb,0.17708e+01_rb,0.23126e+01_rb,0.25324e+01_rb,0.19565e+01_rb /)
      kbo(:, 4,48,13) = (/ &
        & 0.89568e+00_rb,0.18928e+01_rb,0.24241e+01_rb,0.26284e+01_rb,0.20564e+01_rb /)
      kbo(:, 5,48,13) = (/ &
        & 0.10393e+01_rb,0.20105e+01_rb,0.25268e+01_rb,0.27091e+01_rb,0.21445e+01_rb /)
      kbo(:, 1,49,13) = (/ &
        & 0.48708e+00_rb,0.14681e+01_rb,0.20029e+01_rb,0.22232e+01_rb,0.16566e+01_rb /)
      kbo(:, 2,49,13) = (/ &
        & 0.59510e+00_rb,0.15951e+01_rb,0.21382e+01_rb,0.23660e+01_rb,0.17928e+01_rb /)
      kbo(:, 3,49,13) = (/ &
        & 0.71141e+00_rb,0.17205e+01_rb,0.22646e+01_rb,0.24889e+01_rb,0.19130e+01_rb /)
      kbo(:, 4,49,13) = (/ &
        & 0.84034e+00_rb,0.18437e+01_rb,0.23804e+01_rb,0.25917e+01_rb,0.20174e+01_rb /)
      kbo(:, 5,49,13) = (/ &
        & 0.98023e+00_rb,0.19637e+01_rb,0.24863e+01_rb,0.26785e+01_rb,0.21102e+01_rb /)
      kbo(:, 1,50,13) = (/ &
        & 0.44946e+00_rb,0.14198e+01_rb,0.19480e+01_rb,0.21639e+01_rb,0.16005e+01_rb /)
      kbo(:, 2,50,13) = (/ &
        & 0.55308e+00_rb,0.15472e+01_rb,0.20885e+01_rb,0.23146e+01_rb,0.17433e+01_rb /)
      kbo(:, 3,50,13) = (/ &
        & 0.66640e+00_rb,0.16732e+01_rb,0.22179e+01_rb,0.24447e+01_rb,0.18693e+01_rb /)
      kbo(:, 4,50,13) = (/ &
        & 0.79021e+00_rb,0.17975e+01_rb,0.23380e+01_rb,0.25550e+01_rb,0.19794e+01_rb /)
      kbo(:, 5,50,13) = (/ &
        & 0.92626e+00_rb,0.19191e+01_rb,0.24474e+01_rb,0.26476e+01_rb,0.20768e+01_rb /)
      kbo(:, 1,51,13) = (/ &
        & 0.41452e+00_rb,0.13727e+01_rb,0.18922e+01_rb,0.21038e+01_rb,0.15428e+01_rb /)
      kbo(:, 2,51,13) = (/ &
        & 0.51362e+00_rb,0.15005e+01_rb,0.20385e+01_rb,0.22616e+01_rb,0.16930e+01_rb /)
      kbo(:, 3,51,13) = (/ &
        & 0.62394e+00_rb,0.16272e+01_rb,0.21714e+01_rb,0.23993e+01_rb,0.18251e+01_rb /)
      kbo(:, 4,51,13) = (/ &
        & 0.74304e+00_rb,0.17523e+01_rb,0.22953e+01_rb,0.25171e+01_rb,0.19410e+01_rb /)
      kbo(:, 5,51,13) = (/ &
        & 0.87505e+00_rb,0.18748e+01_rb,0.24082e+01_rb,0.26155e+01_rb,0.20426e+01_rb /)
      kbo(:, 1,52,13) = (/ &
        & 0.38121e+00_rb,0.13250e+01_rb,0.18350e+01_rb,0.20413e+01_rb,0.14822e+01_rb /)
      kbo(:, 2,52,13) = (/ &
        & 0.47581e+00_rb,0.14538e+01_rb,0.19870e+01_rb,0.22062e+01_rb,0.16405e+01_rb /)
      kbo(:, 3,52,13) = (/ &
        & 0.58258e+00_rb,0.15810e+01_rb,0.21239e+01_rb,0.23514e+01_rb,0.17789e+01_rb /)
      kbo(:, 4,52,13) = (/ &
        & 0.69813e+00_rb,0.17065e+01_rb,0.22509e+01_rb,0.24762e+01_rb,0.19005e+01_rb /)
      kbo(:, 5,52,13) = (/ &
        & 0.82543e+00_rb,0.18301e+01_rb,0.23683e+01_rb,0.25815e+01_rb,0.20067e+01_rb /)
      kbo(:, 1,53,13) = (/ &
        & 0.34902e+00_rb,0.12769e+01_rb,0.17751e+01_rb,0.19762e+01_rb,0.14178e+01_rb /)
      kbo(:, 2,53,13) = (/ &
        & 0.43953e+00_rb,0.14068e+01_rb,0.19332e+01_rb,0.21485e+01_rb,0.15854e+01_rb /)
      kbo(:, 3,53,13) = (/ &
        & 0.54216e+00_rb,0.15348e+01_rb,0.20756e+01_rb,0.23010e+01_rb,0.17302e+01_rb /)
      kbo(:, 4,53,13) = (/ &
        & 0.65469e+00_rb,0.16608e+01_rb,0.22054e+01_rb,0.24328e+01_rb,0.18578e+01_rb /)
      kbo(:, 5,53,13) = (/ &
        & 0.77730e+00_rb,0.17854e+01_rb,0.23267e+01_rb,0.25453e+01_rb,0.19695e+01_rb /)
      kbo(:, 1,54,13) = (/ &
        & 0.32058e+00_rb,0.12324e+01_rb,0.17175e+01_rb,0.19140e+01_rb,0.13553e+01_rb /)
      kbo(:, 2,54,13) = (/ &
        & 0.40795e+00_rb,0.13637e+01_rb,0.18819e+01_rb,0.20929e+01_rb,0.15321e+01_rb /)
      kbo(:, 3,54,13) = (/ &
        & 0.50645e+00_rb,0.14920e+01_rb,0.20295e+01_rb,0.22518e+01_rb,0.16837e+01_rb /)
      kbo(:, 4,54,13) = (/ &
        & 0.61609e+00_rb,0.16187e+01_rb,0.21628e+01_rb,0.23909e+01_rb,0.18169e+01_rb /)
      kbo(:, 5,54,13) = (/ &
        & 0.73448e+00_rb,0.17439e+01_rb,0.22873e+01_rb,0.25101e+01_rb,0.19340e+01_rb /)
      kbo(:, 1,55,13) = (/ &
        & 0.29443e+00_rb,0.11892e+01_rb,0.16600e+01_rb,0.18513e+01_rb,0.12929e+01_rb /)
      kbo(:, 2,55,13) = (/ &
        & 0.37883e+00_rb,0.13219e+01_rb,0.18314e+01_rb,0.20376e+01_rb,0.14784e+01_rb /)
      kbo(:, 3,55,13) = (/ &
        & 0.47335e+00_rb,0.14508e+01_rb,0.19839e+01_rb,0.22027e+01_rb,0.16372e+01_rb /)
      kbo(:, 4,55,13) = (/ &
        & 0.57989e+00_rb,0.15780e+01_rb,0.21210e+01_rb,0.23485e+01_rb,0.17761e+01_rb /)
      kbo(:, 5,55,13) = (/ &
        & 0.69533e+00_rb,0.17037e+01_rb,0.22482e+01_rb,0.24738e+01_rb,0.18980e+01_rb /)
      kbo(:, 1,56,13) = (/ &
        & 0.26936e+00_rb,0.11459e+01_rb,0.16007e+01_rb,0.17863e+01_rb,0.12289e+01_rb /)
      kbo(:, 2,56,13) = (/ &
        & 0.35065e+00_rb,0.12797e+01_rb,0.17789e+01_rb,0.19807e+01_rb,0.14221e+01_rb /)
      kbo(:, 3,56,13) = (/ &
        & 0.44158e+00_rb,0.14097e+01_rb,0.19368e+01_rb,0.21523e+01_rb,0.15891e+01_rb /)
      kbo(:, 4,56,13) = (/ &
        & 0.54448e+00_rb,0.15375e+01_rb,0.20786e+01_rb,0.23043e+01_rb,0.17334e+01_rb /)
      kbo(:, 5,56,13) = (/ &
        & 0.65732e+00_rb,0.16637e+01_rb,0.22084e+01_rb,0.24358e+01_rb,0.18606e+01_rb /)
      kbo(:, 1,57,13) = (/ &
        & 0.24515e+00_rb,0.11022e+01_rb,0.15388e+01_rb,0.17186e+01_rb,0.11645e+01_rb /)
      kbo(:, 2,57,13) = (/ &
        & 0.32341e+00_rb,0.12372e+01_rb,0.17241e+01_rb,0.19215e+01_rb,0.13626e+01_rb /)
      kbo(:, 3,57,13) = (/ &
        & 0.41131e+00_rb,0.13686e+01_rb,0.18879e+01_rb,0.20994e+01_rb,0.15383e+01_rb /)
      kbo(:, 4,57,13) = (/ &
        & 0.51026e+00_rb,0.14967e+01_rb,0.20347e+01_rb,0.22575e+01_rb,0.16892e+01_rb /)
      kbo(:, 5,57,13) = (/ &
        & 0.62036e+00_rb,0.16234e+01_rb,0.21677e+01_rb,0.23959e+01_rb,0.18217e+01_rb /)
      kbo(:, 1,58,13) = (/ &
        & 0.22298e+00_rb,0.10601e+01_rb,0.14788e+01_rb,0.16521e+01_rb,0.11024e+01_rb /)
      kbo(:, 2,58,13) = (/ &
        & 0.29857e+00_rb,0.11965e+01_rb,0.16700e+01_rb,0.18625e+01_rb,0.13039e+01_rb /)
      kbo(:, 3,58,13) = (/ &
        & 0.38366e+00_rb,0.13291e+01_rb,0.18404e+01_rb,0.20474e+01_rb,0.14880e+01_rb /)
      kbo(:, 4,58,13) = (/ &
        & 0.47882e+00_rb,0.14579e+01_rb,0.19918e+01_rb,0.22114e+01_rb,0.16455e+01_rb /)
      kbo(:, 5,58,13) = (/ &
        & 0.58601e+00_rb,0.15850e+01_rb,0.21282e+01_rb,0.23561e+01_rb,0.17833e+01_rb /)
      kbo(:, 1,59,13) = (/ &
        & 0.21427e+00_rb,0.10429e+01_rb,0.14544e+01_rb,0.16247e+01_rb,0.10771e+01_rb /)
      kbo(:, 2,59,13) = (/ &
        & 0.28885e+00_rb,0.11801e+01_rb,0.16480e+01_rb,0.18384e+01_rb,0.12798e+01_rb /)
      kbo(:, 3,59,13) = (/ &
        & 0.37281e+00_rb,0.13132e+01_rb,0.18209e+01_rb,0.20260e+01_rb,0.14670e+01_rb /)
      kbo(:, 4,59,13) = (/ &
        & 0.46652e+00_rb,0.14422e+01_rb,0.19743e+01_rb,0.21926e+01_rb,0.16276e+01_rb /)
      kbo(:, 5,59,13) = (/ &
        & 0.57246e+00_rb,0.15697e+01_rb,0.21123e+01_rb,0.23397e+01_rb,0.17674e+01_rb /)
      kbo(:, 1,13,14) = (/ &
        & 0.76467e+01_rb,0.66870e+01_rb,0.63502e+01_rb,0.61468e+01_rb,0.66906e+01_rb /)
      kbo(:, 2,13,14) = (/ &
        & 0.76514e+01_rb,0.66716e+01_rb,0.63627e+01_rb,0.61942e+01_rb,0.67780e+01_rb /)
      kbo(:, 3,13,14) = (/ &
        & 0.76475e+01_rb,0.66428e+01_rb,0.63628e+01_rb,0.62371e+01_rb,0.68460e+01_rb /)
      kbo(:, 4,13,14) = (/ &
        & 0.76415e+01_rb,0.66083e+01_rb,0.63514e+01_rb,0.62630e+01_rb,0.69072e+01_rb /)
      kbo(:, 5,13,14) = (/ &
        & 0.76283e+01_rb,0.65686e+01_rb,0.63330e+01_rb,0.62779e+01_rb,0.69614e+01_rb /)
      kbo(:, 1,14,14) = (/ &
        & 0.77958e+01_rb,0.69779e+01_rb,0.66658e+01_rb,0.63591e+01_rb,0.67479e+01_rb /)
      kbo(:, 2,14,14) = (/ &
        & 0.78014e+01_rb,0.69599e+01_rb,0.66792e+01_rb,0.64167e+01_rb,0.68357e+01_rb /)
      kbo(:, 3,14,14) = (/ &
        & 0.78100e+01_rb,0.69314e+01_rb,0.66809e+01_rb,0.64627e+01_rb,0.69109e+01_rb /)
      kbo(:, 4,14,14) = (/ &
        & 0.78098e+01_rb,0.68950e+01_rb,0.66685e+01_rb,0.64923e+01_rb,0.69838e+01_rb /)
      kbo(:, 5,14,14) = (/ &
        & 0.77999e+01_rb,0.68585e+01_rb,0.66494e+01_rb,0.65137e+01_rb,0.70531e+01_rb /)
      kbo(:, 1,15,14) = (/ &
        & 0.78397e+01_rb,0.72065e+01_rb,0.69626e+01_rb,0.65521e+01_rb,0.67717e+01_rb /)
      kbo(:, 2,15,14) = (/ &
        & 0.78630e+01_rb,0.71900e+01_rb,0.69770e+01_rb,0.66166e+01_rb,0.68639e+01_rb /)
      kbo(:, 3,15,14) = (/ &
        & 0.78829e+01_rb,0.71661e+01_rb,0.69761e+01_rb,0.66643e+01_rb,0.69508e+01_rb /)
      kbo(:, 4,15,14) = (/ &
        & 0.78961e+01_rb,0.71374e+01_rb,0.69668e+01_rb,0.66987e+01_rb,0.70374e+01_rb /)
      kbo(:, 5,15,14) = (/ &
        & 0.78984e+01_rb,0.71026e+01_rb,0.69440e+01_rb,0.67207e+01_rb,0.71207e+01_rb /)
      kbo(:, 1,16,14) = (/ &
        & 0.77976e+01_rb,0.73768e+01_rb,0.72331e+01_rb,0.67309e+01_rb,0.67735e+01_rb /)
      kbo(:, 2,16,14) = (/ &
        & 0.78456e+01_rb,0.73704e+01_rb,0.72485e+01_rb,0.67988e+01_rb,0.68801e+01_rb /)
      kbo(:, 3,16,14) = (/ &
        & 0.78840e+01_rb,0.73590e+01_rb,0.72505e+01_rb,0.68482e+01_rb,0.69801e+01_rb /)
      kbo(:, 4,16,14) = (/ &
        & 0.79146e+01_rb,0.73379e+01_rb,0.72385e+01_rb,0.68847e+01_rb,0.70795e+01_rb /)
      kbo(:, 5,16,14) = (/ &
        & 0.79322e+01_rb,0.73060e+01_rb,0.72179e+01_rb,0.69099e+01_rb,0.71779e+01_rb /)
      kbo(:, 1,17,14) = (/ &
        & 0.76851e+01_rb,0.74978e+01_rb,0.74681e+01_rb,0.68923e+01_rb,0.67646e+01_rb /)
      kbo(:, 2,17,14) = (/ &
        & 0.77622e+01_rb,0.75112e+01_rb,0.74909e+01_rb,0.69618e+01_rb,0.68880e+01_rb /)
      kbo(:, 3,17,14) = (/ &
        & 0.78293e+01_rb,0.75089e+01_rb,0.74964e+01_rb,0.70134e+01_rb,0.70063e+01_rb /)
      kbo(:, 4,17,14) = (/ &
        & 0.78832e+01_rb,0.75014e+01_rb,0.74883e+01_rb,0.70540e+01_rb,0.71204e+01_rb /)
      kbo(:, 5,17,14) = (/ &
        & 0.79237e+01_rb,0.74752e+01_rb,0.74629e+01_rb,0.70833e+01_rb,0.72311e+01_rb /)
      kbo(:, 1,18,14) = (/ &
        & 0.75343e+01_rb,0.75844e+01_rb,0.76679e+01_rb,0.70369e+01_rb,0.67530e+01_rb /)
      kbo(:, 2,18,14) = (/ &
        & 0.76372e+01_rb,0.76127e+01_rb,0.76999e+01_rb,0.71100e+01_rb,0.68939e+01_rb /)
      kbo(:, 3,18,14) = (/ &
        & 0.77344e+01_rb,0.76293e+01_rb,0.77123e+01_rb,0.71649e+01_rb,0.70305e+01_rb /)
      kbo(:, 4,18,14) = (/ &
        & 0.78168e+01_rb,0.76302e+01_rb,0.77077e+01_rb,0.72057e+01_rb,0.71594e+01_rb /)
      kbo(:, 5,18,14) = (/ &
        & 0.78848e+01_rb,0.76120e+01_rb,0.76799e+01_rb,0.72369e+01_rb,0.72819e+01_rb /)
      kbo(:, 1,19,14) = (/ &
        & 0.73642e+01_rb,0.76427e+01_rb,0.78392e+01_rb,0.71673e+01_rb,0.67426e+01_rb /)
      kbo(:, 2,19,14) = (/ &
        & 0.75027e+01_rb,0.76890e+01_rb,0.78792e+01_rb,0.72411e+01_rb,0.69029e+01_rb /)
      kbo(:, 3,19,14) = (/ &
        & 0.76261e+01_rb,0.77218e+01_rb,0.78995e+01_rb,0.72995e+01_rb,0.70551e+01_rb /)
      kbo(:, 4,19,14) = (/ &
        & 0.77358e+01_rb,0.77317e+01_rb,0.78972e+01_rb,0.73431e+01_rb,0.71976e+01_rb /)
      kbo(:, 5,19,14) = (/ &
        & 0.78306e+01_rb,0.77206e+01_rb,0.78669e+01_rb,0.73734e+01_rb,0.73288e+01_rb /)
      kbo(:, 1,20,14) = (/ &
        & 0.71989e+01_rb,0.76856e+01_rb,0.79830e+01_rb,0.72835e+01_rb,0.67408e+01_rb /)
      kbo(:, 2,20,14) = (/ &
        & 0.73691e+01_rb,0.77492e+01_rb,0.80344e+01_rb,0.73623e+01_rb,0.69200e+01_rb /)
      kbo(:, 3,20,14) = (/ &
        & 0.75204e+01_rb,0.77952e+01_rb,0.80604e+01_rb,0.74206e+01_rb,0.70842e+01_rb /)
      kbo(:, 4,20,14) = (/ &
        & 0.76560e+01_rb,0.78150e+01_rb,0.80567e+01_rb,0.74651e+01_rb,0.72381e+01_rb /)
      kbo(:, 5,20,14) = (/ &
        & 0.77736e+01_rb,0.78108e+01_rb,0.80255e+01_rb,0.74944e+01_rb,0.73772e+01_rb /)
      kbo(:, 1,21,14) = (/ &
        & 0.70426e+01_rb,0.77166e+01_rb,0.81049e+01_rb,0.73883e+01_rb,0.67480e+01_rb /)
      kbo(:, 2,21,14) = (/ &
        & 0.72474e+01_rb,0.77946e+01_rb,0.81668e+01_rb,0.74708e+01_rb,0.69410e+01_rb /)
      kbo(:, 3,21,14) = (/ &
        & 0.74271e+01_rb,0.78518e+01_rb,0.81976e+01_rb,0.75283e+01_rb,0.71153e+01_rb /)
      kbo(:, 4,21,14) = (/ &
        & 0.75881e+01_rb,0.78806e+01_rb,0.81926e+01_rb,0.75722e+01_rb,0.72765e+01_rb /)
      kbo(:, 5,21,14) = (/ &
        & 0.77244e+01_rb,0.78840e+01_rb,0.81605e+01_rb,0.76005e+01_rb,0.74219e+01_rb /)
      kbo(:, 1,22,14) = (/ &
        & 0.69208e+01_rb,0.77467e+01_rb,0.82134e+01_rb,0.74892e+01_rb,0.67760e+01_rb /)
      kbo(:, 2,22,14) = (/ &
        & 0.71527e+01_rb,0.78380e+01_rb,0.82825e+01_rb,0.75703e+01_rb,0.69788e+01_rb /)
      kbo(:, 3,22,14) = (/ &
        & 0.73591e+01_rb,0.79026e+01_rb,0.83138e+01_rb,0.76265e+01_rb,0.71614e+01_rb /)
      kbo(:, 4,22,14) = (/ &
        & 0.75412e+01_rb,0.79369e+01_rb,0.83052e+01_rb,0.76676e+01_rb,0.73269e+01_rb /)
      kbo(:, 5,22,14) = (/ &
        & 0.76931e+01_rb,0.79423e+01_rb,0.82705e+01_rb,0.76932e+01_rb,0.74734e+01_rb /)
      kbo(:, 1,23,14) = (/ &
        & 0.68264e+01_rb,0.77768e+01_rb,0.83085e+01_rb,0.75813e+01_rb,0.68129e+01_rb /)
      kbo(:, 2,23,14) = (/ &
        & 0.70814e+01_rb,0.78791e+01_rb,0.83807e+01_rb,0.76573e+01_rb,0.70223e+01_rb /)
      kbo(:, 3,23,14) = (/ &
        & 0.73108e+01_rb,0.79473e+01_rb,0.84103e+01_rb,0.77115e+01_rb,0.72087e+01_rb /)
      kbo(:, 4,23,14) = (/ &
        & 0.75083e+01_rb,0.79850e+01_rb,0.83989e+01_rb,0.77500e+01_rb,0.73762e+01_rb /)
      kbo(:, 5,23,14) = (/ &
        & 0.76738e+01_rb,0.79912e+01_rb,0.83608e+01_rb,0.77715e+01_rb,0.75218e+01_rb /)
      kbo(:, 1,24,14) = (/ &
        & 0.67597e+01_rb,0.78092e+01_rb,0.83929e+01_rb,0.76639e+01_rb,0.68571e+01_rb /)
      kbo(:, 2,24,14) = (/ &
        & 0.70359e+01_rb,0.79159e+01_rb,0.84653e+01_rb,0.77347e+01_rb,0.70700e+01_rb /)
      kbo(:, 3,24,14) = (/ &
        & 0.72818e+01_rb,0.79869e+01_rb,0.84905e+01_rb,0.77860e+01_rb,0.72578e+01_rb /)
      kbo(:, 4,24,14) = (/ &
        & 0.74908e+01_rb,0.80269e+01_rb,0.84765e+01_rb,0.78202e+01_rb,0.74248e+01_rb /)
      kbo(:, 5,24,14) = (/ &
        & 0.76673e+01_rb,0.80324e+01_rb,0.84337e+01_rb,0.78375e+01_rb,0.75690e+01_rb /)
      kbo(:, 1,25,14) = (/ &
        & 0.67209e+01_rb,0.78436e+01_rb,0.84676e+01_rb,0.77356e+01_rb,0.69066e+01_rb /)
      kbo(:, 2,25,14) = (/ &
        & 0.70147e+01_rb,0.79506e+01_rb,0.85366e+01_rb,0.78027e+01_rb,0.71216e+01_rb /)
      kbo(:, 3,25,14) = (/ &
        & 0.72710e+01_rb,0.80222e+01_rb,0.85567e+01_rb,0.78495e+01_rb,0.73079e+01_rb /)
      kbo(:, 4,25,14) = (/ &
        & 0.74891e+01_rb,0.80627e+01_rb,0.85388e+01_rb,0.78803e+01_rb,0.74736e+01_rb /)
      kbo(:, 5,25,14) = (/ &
        & 0.76763e+01_rb,0.80663e+01_rb,0.84908e+01_rb,0.78920e+01_rb,0.76152e+01_rb /)
      kbo(:, 1,26,14) = (/ &
        & 0.67117e+01_rb,0.78809e+01_rb,0.85346e+01_rb,0.77996e+01_rb,0.69621e+01_rb /)
      kbo(:, 2,26,14) = (/ &
        & 0.70163e+01_rb,0.79848e+01_rb,0.85977e+01_rb,0.78624e+01_rb,0.71766e+01_rb /)
      kbo(:, 3,26,14) = (/ &
        & 0.72800e+01_rb,0.80555e+01_rb,0.86111e+01_rb,0.79043e+01_rb,0.73612e+01_rb /)
      kbo(:, 4,26,14) = (/ &
        & 0.75048e+01_rb,0.80927e+01_rb,0.85874e+01_rb,0.79312e+01_rb,0.75235e+01_rb /)
      kbo(:, 5,26,14) = (/ &
        & 0.76969e+01_rb,0.80929e+01_rb,0.85318e+01_rb,0.79367e+01_rb,0.76615e+01_rb /)
      kbo(:, 1,27,14) = (/ &
        & 0.67222e+01_rb,0.79165e+01_rb,0.85935e+01_rb,0.78553e+01_rb,0.70208e+01_rb /)
      kbo(:, 2,27,14) = (/ &
        & 0.70339e+01_rb,0.80174e+01_rb,0.86486e+01_rb,0.79134e+01_rb,0.72318e+01_rb /)
      kbo(:, 3,27,14) = (/ &
        & 0.73021e+01_rb,0.80859e+01_rb,0.86549e+01_rb,0.79502e+01_rb,0.74127e+01_rb /)
      kbo(:, 4,27,14) = (/ &
        & 0.75298e+01_rb,0.81175e+01_rb,0.86239e+01_rb,0.79721e+01_rb,0.75702e+01_rb /)
      kbo(:, 5,27,14) = (/ &
        & 0.77246e+01_rb,0.81143e+01_rb,0.85613e+01_rb,0.79734e+01_rb,0.77047e+01_rb /)
      kbo(:, 1,28,14) = (/ &
        & 0.67490e+01_rb,0.79508e+01_rb,0.86448e+01_rb,0.79039e+01_rb,0.70793e+01_rb /)
      kbo(:, 2,28,14) = (/ &
        & 0.70641e+01_rb,0.80481e+01_rb,0.86903e+01_rb,0.79567e+01_rb,0.72853e+01_rb /)
      kbo(:, 3,28,14) = (/ &
        & 0.73339e+01_rb,0.81119e+01_rb,0.86890e+01_rb,0.79886e+01_rb,0.74622e+01_rb /)
      kbo(:, 4,28,14) = (/ &
        & 0.75625e+01_rb,0.81383e+01_rb,0.86502e+01_rb,0.80049e+01_rb,0.76145e+01_rb /)
      kbo(:, 5,28,14) = (/ &
        & 0.77567e+01_rb,0.81304e+01_rb,0.85806e+01_rb,0.80023e+01_rb,0.77455e+01_rb /)
      kbo(:, 1,29,14) = (/ &
        & 0.67914e+01_rb,0.79845e+01_rb,0.86886e+01_rb,0.79462e+01_rb,0.71386e+01_rb /)
      kbo(:, 2,29,14) = (/ &
        & 0.71068e+01_rb,0.80770e+01_rb,0.87239e+01_rb,0.79933e+01_rb,0.73389e+01_rb /)
      kbo(:, 3,29,14) = (/ &
        & 0.73751e+01_rb,0.81344e+01_rb,0.87144e+01_rb,0.80208e+01_rb,0.75104e+01_rb /)
      kbo(:, 4,29,14) = (/ &
        & 0.76020e+01_rb,0.81550e+01_rb,0.86676e+01_rb,0.80316e+01_rb,0.76575e+01_rb /)
      kbo(:, 5,29,14) = (/ &
        & 0.77937e+01_rb,0.81425e+01_rb,0.85914e+01_rb,0.80248e+01_rb,0.77840e+01_rb /)
      kbo(:, 1,30,14) = (/ &
        & 0.68445e+01_rb,0.80166e+01_rb,0.87256e+01_rb,0.79827e+01_rb,0.71970e+01_rb /)
      kbo(:, 2,30,14) = (/ &
        & 0.71563e+01_rb,0.81037e+01_rb,0.87505e+01_rb,0.80234e+01_rb,0.73900e+01_rb /)
      kbo(:, 3,30,14) = (/ &
        & 0.74214e+01_rb,0.81535e+01_rb,0.87321e+01_rb,0.80469e+01_rb,0.75564e+01_rb /)
      kbo(:, 4,30,14) = (/ &
        & 0.76455e+01_rb,0.81683e+01_rb,0.86770e+01_rb,0.80524e+01_rb,0.76986e+01_rb /)
      kbo(:, 5,30,14) = (/ &
        & 0.78317e+01_rb,0.81499e+01_rb,0.85949e+01_rb,0.80416e+01_rb,0.78203e+01_rb /)
      kbo(:, 1,31,14) = (/ &
        & 0.69054e+01_rb,0.80478e+01_rb,0.87566e+01_rb,0.80136e+01_rb,0.72537e+01_rb /)
      kbo(:, 2,31,14) = (/ &
        & 0.72125e+01_rb,0.81281e+01_rb,0.87710e+01_rb,0.80494e+01_rb,0.74404e+01_rb /)
      kbo(:, 3,31,14) = (/ &
        & 0.74718e+01_rb,0.81700e+01_rb,0.87427e+01_rb,0.80677e+01_rb,0.76003e+01_rb /)
      kbo(:, 4,31,14) = (/ &
        & 0.76912e+01_rb,0.81779e+01_rb,0.86791e+01_rb,0.80682e+01_rb,0.77384e+01_rb /)
      kbo(:, 5,31,14) = (/ &
        & 0.78715e+01_rb,0.81532e+01_rb,0.85914e+01_rb,0.80538e+01_rb,0.78547e+01_rb /)
      kbo(:, 1,32,14) = (/ &
        & 0.69718e+01_rb,0.80767e+01_rb,0.87812e+01_rb,0.80405e+01_rb,0.73093e+01_rb /)
      kbo(:, 2,32,14) = (/ &
        & 0.72714e+01_rb,0.81493e+01_rb,0.87845e+01_rb,0.80705e+01_rb,0.74889e+01_rb /)
      kbo(:, 3,32,14) = (/ &
        & 0.75255e+01_rb,0.81843e+01_rb,0.87476e+01_rb,0.80843e+01_rb,0.76427e+01_rb /)
      kbo(:, 4,32,14) = (/ &
        & 0.77373e+01_rb,0.81836e+01_rb,0.86757e+01_rb,0.80798e+01_rb,0.77759e+01_rb /)
      kbo(:, 5,32,14) = (/ &
        & 0.79115e+01_rb,0.81523e+01_rb,0.85821e+01_rb,0.80611e+01_rb,0.78870e+01_rb /)
      kbo(:, 1,33,14) = (/ &
        & 0.70424e+01_rb,0.81041e+01_rb,0.88003e+01_rb,0.80631e+01_rb,0.73621e+01_rb /)
      kbo(:, 2,33,14) = (/ &
        & 0.73332e+01_rb,0.81674e+01_rb,0.87934e+01_rb,0.80887e+01_rb,0.75359e+01_rb /)
      kbo(:, 3,33,14) = (/ &
        & 0.75797e+01_rb,0.81949e+01_rb,0.87467e+01_rb,0.80962e+01_rb,0.76832e+01_rb /)
      kbo(:, 4,33,14) = (/ &
        & 0.77832e+01_rb,0.81860e+01_rb,0.86679e+01_rb,0.80880e+01_rb,0.78112e+01_rb /)
      kbo(:, 5,33,14) = (/ &
        & 0.79514e+01_rb,0.81480e+01_rb,0.85688e+01_rb,0.80650e+01_rb,0.79176e+01_rb /)
      kbo(:, 1,34,14) = (/ &
        & 0.71032e+01_rb,0.81256e+01_rb,0.88135e+01_rb,0.80801e+01_rb,0.74073e+01_rb /)
      kbo(:, 2,34,14) = (/ &
        & 0.73864e+01_rb,0.81812e+01_rb,0.87981e+01_rb,0.81025e+01_rb,0.75752e+01_rb /)
      kbo(:, 3,34,14) = (/ &
        & 0.76257e+01_rb,0.82022e+01_rb,0.87428e+01_rb,0.81055e+01_rb,0.77177e+01_rb /)
      kbo(:, 4,34,14) = (/ &
        & 0.78229e+01_rb,0.81866e+01_rb,0.86595e+01_rb,0.80934e+01_rb,0.78409e+01_rb /)
      kbo(:, 5,34,14) = (/ &
        & 0.79841e+01_rb,0.81428e+01_rb,0.85553e+01_rb,0.80663e+01_rb,0.79427e+01_rb /)
      kbo(:, 1,35,14) = (/ &
        & 0.71357e+01_rb,0.81386e+01_rb,0.88238e+01_rb,0.80923e+01_rb,0.74328e+01_rb /)
      kbo(:, 2,35,14) = (/ &
        & 0.74155e+01_rb,0.81901e+01_rb,0.88028e+01_rb,0.81125e+01_rb,0.75980e+01_rb /)
      kbo(:, 3,35,14) = (/ &
        & 0.76506e+01_rb,0.82068e+01_rb,0.87428e+01_rb,0.81128e+01_rb,0.77381e+01_rb /)
      kbo(:, 4,35,14) = (/ &
        & 0.78446e+01_rb,0.81876e+01_rb,0.86563e+01_rb,0.80980e+01_rb,0.78583e+01_rb /)
      kbo(:, 5,35,14) = (/ &
        & 0.80022e+01_rb,0.81408e+01_rb,0.85492e+01_rb,0.80683e+01_rb,0.79574e+01_rb /)
      kbo(:, 1,36,14) = (/ &
        & 0.71358e+01_rb,0.81426e+01_rb,0.88312e+01_rb,0.80996e+01_rb,0.74385e+01_rb /)
      kbo(:, 2,36,14) = (/ &
        & 0.74168e+01_rb,0.81942e+01_rb,0.88101e+01_rb,0.81195e+01_rb,0.76033e+01_rb /)
      kbo(:, 3,36,14) = (/ &
        & 0.76530e+01_rb,0.82111e+01_rb,0.87491e+01_rb,0.81189e+01_rb,0.77431e+01_rb /)
      kbo(:, 4,36,14) = (/ &
        & 0.78475e+01_rb,0.81916e+01_rb,0.86625e+01_rb,0.81041e+01_rb,0.78633e+01_rb /)
      kbo(:, 5,36,14) = (/ &
        & 0.80053e+01_rb,0.81444e+01_rb,0.85549e+01_rb,0.80739e+01_rb,0.79624e+01_rb /)
      kbo(:, 1,37,14) = (/ &
        & 0.70898e+01_rb,0.81353e+01_rb,0.88375e+01_rb,0.81022e+01_rb,0.74173e+01_rb /)
      kbo(:, 2,37,14) = (/ &
        & 0.73792e+01_rb,0.81923e+01_rb,0.88220e+01_rb,0.81236e+01_rb,0.75852e+01_rb /)
      kbo(:, 3,37,14) = (/ &
        & 0.76229e+01_rb,0.82146e+01_rb,0.87658e+01_rb,0.81255e+01_rb,0.77281e+01_rb /)
      kbo(:, 4,37,14) = (/ &
        & 0.78232e+01_rb,0.81996e+01_rb,0.86818e+01_rb,0.81120e+01_rb,0.78508e+01_rb /)
      kbo(:, 5,37,14) = (/ &
        & 0.79866e+01_rb,0.81554e+01_rb,0.85764e+01_rb,0.80843e+01_rb,0.79525e+01_rb /)
      kbo(:, 1,38,14) = (/ &
        & 0.70419e+01_rb,0.81271e+01_rb,0.88418e+01_rb,0.81031e+01_rb,0.73933e+01_rb /)
      kbo(:, 2,38,14) = (/ &
        & 0.73389e+01_rb,0.81889e+01_rb,0.88323e+01_rb,0.81261e+01_rb,0.75648e+01_rb /)
      kbo(:, 3,38,14) = (/ &
        & 0.75899e+01_rb,0.82161e+01_rb,0.87808e+01_rb,0.81301e+01_rb,0.77105e+01_rb /)
      kbo(:, 4,38,14) = (/ &
        & 0.77971e+01_rb,0.82068e+01_rb,0.87000e+01_rb,0.81195e+01_rb,0.78368e+01_rb /)
      kbo(:, 5,38,14) = (/ &
        & 0.79663e+01_rb,0.81665e+01_rb,0.85976e+01_rb,0.80935e+01_rb,0.79412e+01_rb /)
      kbo(:, 1,39,14) = (/ &
        & 0.69923e+01_rb,0.81165e+01_rb,0.88435e+01_rb,0.81024e+01_rb,0.73685e+01_rb /)
      kbo(:, 2,39,14) = (/ &
        & 0.72982e+01_rb,0.81845e+01_rb,0.88409e+01_rb,0.81277e+01_rb,0.75441e+01_rb /)
      kbo(:, 3,39,14) = (/ &
        & 0.75563e+01_rb,0.82164e+01_rb,0.87952e+01_rb,0.81347e+01_rb,0.76928e+01_rb /)
      kbo(:, 4,39,14) = (/ &
        & 0.77695e+01_rb,0.82122e+01_rb,0.87165e+01_rb,0.81250e+01_rb,0.78216e+01_rb /)
      kbo(:, 5,39,14) = (/ &
        & 0.79452e+01_rb,0.81762e+01_rb,0.86172e+01_rb,0.81015e+01_rb,0.79291e+01_rb /)
      kbo(:, 1,40,14) = (/ &
        & 0.69080e+01_rb,0.80958e+01_rb,0.88413e+01_rb,0.80965e+01_rb,0.73229e+01_rb /)
      kbo(:, 2,40,14) = (/ &
        & 0.72277e+01_rb,0.81734e+01_rb,0.88495e+01_rb,0.81259e+01_rb,0.75046e+01_rb /)
      kbo(:, 3,40,14) = (/ &
        & 0.74967e+01_rb,0.82131e+01_rb,0.88131e+01_rb,0.81381e+01_rb,0.76600e+01_rb /)
      kbo(:, 4,40,14) = (/ &
        & 0.77214e+01_rb,0.82184e+01_rb,0.87413e+01_rb,0.81320e+01_rb,0.77938e+01_rb /)
      kbo(:, 5,40,14) = (/ &
        & 0.79055e+01_rb,0.81894e+01_rb,0.86471e+01_rb,0.81094e+01_rb,0.79066e+01_rb /)
      kbo(:, 1,41,14) = (/ &
        & 0.68155e+01_rb,0.80708e+01_rb,0.88352e+01_rb,0.80885e+01_rb,0.72719e+01_rb /)
      kbo(:, 2,41,14) = (/ &
        & 0.71495e+01_rb,0.81585e+01_rb,0.88545e+01_rb,0.81224e+01_rb,0.74618e+01_rb /)
      kbo(:, 3,41,14) = (/ &
        & 0.74308e+01_rb,0.82074e+01_rb,0.88291e+01_rb,0.81392e+01_rb,0.76238e+01_rb /)
      kbo(:, 4,41,14) = (/ &
        & 0.76675e+01_rb,0.82222e+01_rb,0.87650e+01_rb,0.81374e+01_rb,0.77629e+01_rb /)
      kbo(:, 5,41,14) = (/ &
        & 0.78621e+01_rb,0.82014e+01_rb,0.86761e+01_rb,0.81197e+01_rb,0.78805e+01_rb /)
      kbo(:, 1,42,14) = (/ &
        & 0.67210e+01_rb,0.80436e+01_rb,0.88257e+01_rb,0.80787e+01_rb,0.72181e+01_rb /)
      kbo(:, 2,42,14) = (/ &
        & 0.70672e+01_rb,0.81413e+01_rb,0.88562e+01_rb,0.81177e+01_rb,0.74171e+01_rb /)
      kbo(:, 3,42,14) = (/ &
        & 0.73618e+01_rb,0.81993e+01_rb,0.88420e+01_rb,0.81387e+01_rb,0.75853e+01_rb /)
      kbo(:, 4,42,14) = (/ &
        & 0.76110e+01_rb,0.82240e+01_rb,0.87872e+01_rb,0.81416e+01_rb,0.77301e+01_rb /)
      kbo(:, 5,42,14) = (/ &
        & 0.78157e+01_rb,0.82114e+01_rb,0.87034e+01_rb,0.81275e+01_rb,0.78533e+01_rb /)
      kbo(:, 1,43,14) = (/ &
        & 0.66012e+01_rb,0.80077e+01_rb,0.88083e+01_rb,0.80639e+01_rb,0.71482e+01_rb /)
      kbo(:, 2,43,14) = (/ &
        & 0.69627e+01_rb,0.81158e+01_rb,0.88532e+01_rb,0.81101e+01_rb,0.73596e+01_rb /)
      kbo(:, 3,43,14) = (/ &
        & 0.72743e+01_rb,0.81862e+01_rb,0.88533e+01_rb,0.81352e+01_rb,0.75360e+01_rb /)
      kbo(:, 4,43,14) = (/ &
        & 0.75376e+01_rb,0.82203e+01_rb,0.88101e+01_rb,0.81444e+01_rb,0.76876e+01_rb /)
      kbo(:, 5,43,14) = (/ &
        & 0.77560e+01_rb,0.82200e+01_rb,0.87338e+01_rb,0.81353e+01_rb,0.78176e+01_rb /)
      kbo(:, 1,44,14) = (/ &
        & 0.64654e+01_rb,0.79624e+01_rb,0.87841e+01_rb,0.80450e+01_rb,0.70696e+01_rb /)
      kbo(:, 2,44,14) = (/ &
        & 0.68452e+01_rb,0.80842e+01_rb,0.88456e+01_rb,0.80985e+01_rb,0.72931e+01_rb /)
      kbo(:, 3,44,14) = (/ &
        & 0.71736e+01_rb,0.81668e+01_rb,0.88594e+01_rb,0.81294e+01_rb,0.74799e+01_rb /)
      kbo(:, 4,44,14) = (/ &
        & 0.74536e+01_rb,0.82135e+01_rb,0.88309e+01_rb,0.81455e+01_rb,0.76398e+01_rb /)
      kbo(:, 5,44,14) = (/ &
        & 0.76864e+01_rb,0.82252e+01_rb,0.87641e+01_rb,0.81417e+01_rb,0.77771e+01_rb /)
      kbo(:, 1,45,14) = (/ &
        & 0.63246e+01_rb,0.79121e+01_rb,0.87529e+01_rb,0.80227e+01_rb,0.69872e+01_rb /)
      kbo(:, 2,45,14) = (/ &
        & 0.67215e+01_rb,0.80478e+01_rb,0.88314e+01_rb,0.80840e+01_rb,0.72219e+01_rb /)
      kbo(:, 3,45,14) = (/ &
        & 0.70680e+01_rb,0.81447e+01_rb,0.88609e+01_rb,0.81218e+01_rb,0.74205e+01_rb /)
      kbo(:, 4,45,14) = (/ &
        & 0.73633e+01_rb,0.82020e+01_rb,0.88464e+01_rb,0.81437e+01_rb,0.75896e+01_rb /)
      kbo(:, 5,45,14) = (/ &
        & 0.76123e+01_rb,0.82264e+01_rb,0.87913e+01_rb,0.81458e+01_rb,0.77333e+01_rb /)
      kbo(:, 1,46,14) = (/ &
        & 0.61677e+01_rb,0.78522e+01_rb,0.87120e+01_rb,0.79958e+01_rb,0.68962e+01_rb /)
      kbo(:, 2,46,14) = (/ &
        & 0.65843e+01_rb,0.80056e+01_rb,0.88106e+01_rb,0.80659e+01_rb,0.71413e+01_rb /)
      kbo(:, 3,46,14) = (/ &
        & 0.69482e+01_rb,0.81147e+01_rb,0.88565e+01_rb,0.81122e+01_rb,0.73539e+01_rb /)
      kbo(:, 4,46,14) = (/ &
        & 0.72620e+01_rb,0.81857e+01_rb,0.88581e+01_rb,0.81382e+01_rb,0.75320e+01_rb /)
      kbo(:, 5,46,14) = (/ &
        & 0.75285e+01_rb,0.82222e+01_rb,0.88171e+01_rb,0.81482e+01_rb,0.76841e+01_rb /)
      kbo(:, 1,47,14) = (/ &
        & 0.59872e+01_rb,0.77759e+01_rb,0.86581e+01_rb,0.79599e+01_rb,0.67904e+01_rb /)
      kbo(:, 2,47,14) = (/ &
        & 0.64227e+01_rb,0.79505e+01_rb,0.87794e+01_rb,0.80417e+01_rb,0.70463e+01_rb /)
      kbo(:, 3,47,14) = (/ &
        & 0.68067e+01_rb,0.80750e+01_rb,0.88445e+01_rb,0.80966e+01_rb,0.72732e+01_rb /)
      kbo(:, 4,47,14) = (/ &
        & 0.71415e+01_rb,0.81619e+01_rb,0.88634e+01_rb,0.81302e+01_rb,0.74640e+01_rb /)
      kbo(:, 5,47,14) = (/ &
        & 0.74259e+01_rb,0.82118e+01_rb,0.88390e+01_rb,0.81478e+01_rb,0.76260e+01_rb /)
      kbo(:, 1,48,14) = (/ &
        & 0.57999e+01_rb,0.76897e+01_rb,0.85924e+01_rb,0.79165e+01_rb,0.66812e+01_rb /)
      kbo(:, 2,48,14) = (/ &
        & 0.62523e+01_rb,0.78878e+01_rb,0.87384e+01_rb,0.80135e+01_rb,0.69471e+01_rb /)
      kbo(:, 3,48,14) = (/ &
        & 0.66581e+01_rb,0.80303e+01_rb,0.88250e+01_rb,0.80783e+01_rb,0.71862e+01_rb /)
      kbo(:, 4,48,14) = (/ &
        & 0.70120e+01_rb,0.81321e+01_rb,0.88616e+01_rb,0.81203e+01_rb,0.73915e+01_rb /)
      kbo(:, 5,48,14) = (/ &
        & 0.73167e+01_rb,0.81963e+01_rb,0.88550e+01_rb,0.81433e+01_rb,0.75640e+01_rb /)
      kbo(:, 1,49,14) = (/ &
        & 0.56056e+01_rb,0.75967e+01_rb,0.85186e+01_rb,0.78664e+01_rb,0.65684e+01_rb /)
      kbo(:, 2,49,14) = (/ &
        & 0.60725e+01_rb,0.78139e+01_rb,0.86868e+01_rb,0.79799e+01_rb,0.68428e+01_rb /)
      kbo(:, 3,49,14) = (/ &
        & 0.65005e+01_rb,0.79789e+01_rb,0.87970e+01_rb,0.80551e+01_rb,0.70936e+01_rb /)
      kbo(:, 4,49,14) = (/ &
        & 0.68750e+01_rb,0.80955e+01_rb,0.88532e+01_rb,0.81069e+01_rb,0.73141e+01_rb /)
      kbo(:, 5,49,14) = (/ &
        & 0.72008e+01_rb,0.81753e+01_rb,0.88640e+01_rb,0.81362e+01_rb,0.74980e+01_rb /)
      kbo(:, 1,50,14) = (/ &
        & 0.54125e+01_rb,0.74991e+01_rb,0.84401e+01_rb,0.78123e+01_rb,0.64595e+01_rb /)
      kbo(:, 2,50,14) = (/ &
        & 0.59003e+01_rb,0.77375e+01_rb,0.86301e+01_rb,0.79423e+01_rb,0.67420e+01_rb /)
      kbo(:, 3,50,14) = (/ &
        & 0.63429e+01_rb,0.79215e+01_rb,0.87619e+01_rb,0.80304e+01_rb,0.70024e+01_rb /)
      kbo(:, 4,50,14) = (/ &
        & 0.67393e+01_rb,0.80557e+01_rb,0.88380e+01_rb,0.80904e+01_rb,0.72354e+01_rb /)
      kbo(:, 5,50,14) = (/ &
        & 0.70838e+01_rb,0.81507e+01_rb,0.88659e+01_rb,0.81277e+01_rb,0.74327e+01_rb /)
      kbo(:, 1,51,14) = (/ &
        & 0.52191e+01_rb,0.73982e+01_rb,0.83585e+01_rb,0.77548e+01_rb,0.63504e+01_rb /)
      kbo(:, 2,51,14) = (/ &
        & 0.57281e+01_rb,0.76566e+01_rb,0.85673e+01_rb,0.79002e+01_rb,0.66408e+01_rb /)
      kbo(:, 3,51,14) = (/ &
        & 0.61850e+01_rb,0.78612e+01_rb,0.87206e+01_rb,0.80029e+01_rb,0.69102e+01_rb /)
      kbo(:, 4,51,14) = (/ &
        & 0.66013e+01_rb,0.80131e+01_rb,0.88169e+01_rb,0.80719e+01_rb,0.71542e+01_rb /)
      kbo(:, 5,51,14) = (/ &
        & 0.69630e+01_rb,0.81207e+01_rb,0.88611e+01_rb,0.81171e+01_rb,0.73645e+01_rb /)
      kbo(:, 1,52,14) = (/ &
        & 0.50203e+01_rb,0.72853e+01_rb,0.82659e+01_rb,0.76900e+01_rb,0.62362e+01_rb /)
      kbo(:, 2,52,14) = (/ &
        & 0.55489e+01_rb,0.75692e+01_rb,0.84974e+01_rb,0.78525e+01_rb,0.65380e+01_rb /)
      kbo(:, 3,52,14) = (/ &
        & 0.60223e+01_rb,0.77933e+01_rb,0.86726e+01_rb,0.79710e+01_rb,0.68148e+01_rb /)
      kbo(:, 4,52,14) = (/ &
        & 0.64533e+01_rb,0.79619e+01_rb,0.87883e+01_rb,0.80495e+01_rb,0.70682e+01_rb /)
      kbo(:, 5,52,14) = (/ &
        & 0.68363e+01_rb,0.80853e+01_rb,0.88509e+01_rb,0.81034e+01_rb,0.72926e+01_rb /)
      kbo(:, 1,53,14) = (/ &
        & 0.48171e+01_rb,0.71626e+01_rb,0.81646e+01_rb,0.76188e+01_rb,0.61179e+01_rb /)
      kbo(:, 2,53,14) = (/ &
        & 0.53620e+01_rb,0.74745e+01_rb,0.84206e+01_rb,0.77982e+01_rb,0.64310e+01_rb /)
      kbo(:, 3,53,14) = (/ &
        & 0.58544e+01_rb,0.77168e+01_rb,0.86150e+01_rb,0.79329e+01_rb,0.67166e+01_rb /)
      kbo(:, 4,53,14) = (/ &
        & 0.63007e+01_rb,0.79064e+01_rb,0.87527e+01_rb,0.80244e+01_rb,0.69787e+01_rb /)
      kbo(:, 5,53,14) = (/ &
        & 0.67036e+01_rb,0.80457e+01_rb,0.88348e+01_rb,0.80870e+01_rb,0.72150e+01_rb /)
      kbo(:, 1,54,14) = (/ &
        & 0.46268e+01_rb,0.70415e+01_rb,0.80643e+01_rb,0.75453e+01_rb,0.60051e+01_rb /)
      kbo(:, 2,54,14) = (/ &
        & 0.51834e+01_rb,0.73793e+01_rb,0.83436e+01_rb,0.77439e+01_rb,0.63304e+01_rb /)
      kbo(:, 3,54,14) = (/ &
        & 0.56952e+01_rb,0.76416e+01_rb,0.85557e+01_rb,0.78930e+01_rb,0.66233e+01_rb /)
      kbo(:, 4,54,14) = (/ &
        & 0.61555e+01_rb,0.78501e+01_rb,0.87136e+01_rb,0.79986e+01_rb,0.68935e+01_rb /)
      kbo(:, 5,54,14) = (/ &
        & 0.65754e+01_rb,0.80055e+01_rb,0.88139e+01_rb,0.80689e+01_rb,0.71391e+01_rb /)
      kbo(:, 1,55,14) = (/ &
        & 0.44418e+01_rb,0.69152e+01_rb,0.79589e+01_rb,0.74683e+01_rb,0.58909e+01_rb /)
      kbo(:, 2,55,14) = (/ &
        & 0.50082e+01_rb,0.72786e+01_rb,0.82608e+01_rb,0.76866e+01_rb,0.62296e+01_rb /)
      kbo(:, 3,55,14) = (/ &
        & 0.55372e+01_rb,0.75641e+01_rb,0.84935e+01_rb,0.78502e+01_rb,0.65324e+01_rb /)
      kbo(:, 4,55,14) = (/ &
        & 0.60119e+01_rb,0.77891e+01_rb,0.86705e+01_rb,0.79700e+01_rb,0.68094e+01_rb /)
      kbo(:, 5,55,14) = (/ &
        & 0.64443e+01_rb,0.79596e+01_rb,0.87874e+01_rb,0.80489e+01_rb,0.70636e+01_rb /)
      kbo(:, 1,56,14) = (/ &
        & 0.42552e+01_rb,0.67816e+01_rb,0.78463e+01_rb,0.73861e+01_rb,0.57693e+01_rb /)
      kbo(:, 2,56,14) = (/ &
        & 0.48302e+01_rb,0.71710e+01_rb,0.81721e+01_rb,0.76241e+01_rb,0.61259e+01_rb /)
      kbo(:, 3,56,14) = (/ &
        & 0.53736e+01_rb,0.74812e+01_rb,0.84261e+01_rb,0.78026e+01_rb,0.64389e+01_rb /)
      kbo(:, 4,56,14) = (/ &
        & 0.58648e+01_rb,0.77223e+01_rb,0.86198e+01_rb,0.79361e+01_rb,0.67232e+01_rb /)
      kbo(:, 5,56,14) = (/ &
        & 0.63103e+01_rb,0.79105e+01_rb,0.87564e+01_rb,0.80271e+01_rb,0.69853e+01_rb /)
      kbo(:, 1,57,14) = (/ &
        & 0.40639e+01_rb,0.66394e+01_rb,0.77238e+01_rb,0.72954e+01_rb,0.56390e+01_rb /)
      kbo(:, 2,57,14) = (/ &
        & 0.46488e+01_rb,0.70563e+01_rb,0.80768e+01_rb,0.75544e+01_rb,0.60187e+01_rb /)
      kbo(:, 3,57,14) = (/ &
        & 0.52035e+01_rb,0.73909e+01_rb,0.83529e+01_rb,0.77510e+01_rb,0.63425e+01_rb /)
      kbo(:, 4,57,14) = (/ &
        & 0.57137e+01_rb,0.76510e+01_rb,0.85638e+01_rb,0.78985e+01_rb,0.66342e+01_rb /)
      kbo(:, 5,57,14) = (/ &
        & 0.61722e+01_rb,0.78574e+01_rb,0.87192e+01_rb,0.80023e+01_rb,0.69038e+01_rb /)
      kbo(:, 1,58,14) = (/ &
        & 0.38805e+01_rb,0.64976e+01_rb,0.75961e+01_rb,0.72010e+01_rb,0.55107e+01_rb /)
      kbo(:, 2,58,14) = (/ &
        & 0.44745e+01_rb,0.69383e+01_rb,0.79787e+01_rb,0.74825e+01_rb,0.59115e+01_rb /)
      kbo(:, 3,58,14) = (/ &
        & 0.50386e+01_rb,0.72976e+01_rb,0.82763e+01_rb,0.76973e+01_rb,0.62480e+01_rb /)
      kbo(:, 4,58,14) = (/ &
        & 0.55648e+01_rb,0.75780e+01_rb,0.85055e+01_rb,0.78586e+01_rb,0.65486e+01_rb /)
      kbo(:, 5,58,14) = (/ &
        & 0.60366e+01_rb,0.78002e+01_rb,0.86790e+01_rb,0.79756e+01_rb,0.68245e+01_rb /)
      kbo(:, 1,59,14) = (/ &
        & 0.38057e+01_rb,0.64388e+01_rb,0.75416e+01_rb,0.71615e+01_rb,0.54582e+01_rb /)
      kbo(:, 2,59,14) = (/ &
        & 0.44044e+01_rb,0.68893e+01_rb,0.79374e+01_rb,0.74528e+01_rb,0.58671e+01_rb /)
      kbo(:, 3,59,14) = (/ &
        & 0.49711e+01_rb,0.72574e+01_rb,0.82432e+01_rb,0.76750e+01_rb,0.62091e+01_rb /)
      kbo(:, 4,59,14) = (/ &
        & 0.55038e+01_rb,0.75476e+01_rb,0.84809e+01_rb,0.78415e+01_rb,0.65137e+01_rb /)
      kbo(:, 5,59,14) = (/ &
        & 0.59820e+01_rb,0.77764e+01_rb,0.86612e+01_rb,0.79636e+01_rb,0.67922e+01_rb /)
      kbo(:, 1,13,15) = (/ &
        & 0.21583e+02_rb,0.16708e+02_rb,0.15578e+02_rb,0.16324e+02_rb,0.21499e+02_rb /)
      kbo(:, 2,13,15) = (/ &
        & 0.21596e+02_rb,0.16646e+02_rb,0.15448e+02_rb,0.16116e+02_rb,0.21237e+02_rb /)
      kbo(:, 3,13,15) = (/ &
        & 0.21561e+02_rb,0.16562e+02_rb,0.15303e+02_rb,0.15881e+02_rb,0.20975e+02_rb /)
      kbo(:, 4,13,15) = (/ &
        & 0.21482e+02_rb,0.16444e+02_rb,0.15131e+02_rb,0.15663e+02_rb,0.20709e+02_rb /)
      kbo(:, 5,13,15) = (/ &
        & 0.21360e+02_rb,0.16308e+02_rb,0.14933e+02_rb,0.15443e+02_rb,0.20449e+02_rb /)
      kbo(:, 1,14,15) = (/ &
        & 0.23286e+02_rb,0.18105e+02_rb,0.17022e+02_rb,0.17494e+02_rb,0.22811e+02_rb /)
      kbo(:, 2,14,15) = (/ &
        & 0.23268e+02_rb,0.18001e+02_rb,0.16845e+02_rb,0.17259e+02_rb,0.22563e+02_rb /)
      kbo(:, 3,14,15) = (/ &
        & 0.23185e+02_rb,0.17865e+02_rb,0.16645e+02_rb,0.17010e+02_rb,0.22295e+02_rb /)
      kbo(:, 4,14,15) = (/ &
        & 0.23065e+02_rb,0.17707e+02_rb,0.16417e+02_rb,0.16757e+02_rb,0.22015e+02_rb /)
      kbo(:, 5,14,15) = (/ &
        & 0.22894e+02_rb,0.17515e+02_rb,0.16163e+02_rb,0.16504e+02_rb,0.21733e+02_rb /)
      kbo(:, 1,15,15) = (/ &
        & 0.24991e+02_rb,0.19505e+02_rb,0.18411e+02_rb,0.18638e+02_rb,0.24057e+02_rb /)
      kbo(:, 2,15,15) = (/ &
        & 0.24920e+02_rb,0.19349e+02_rb,0.18180e+02_rb,0.18374e+02_rb,0.23810e+02_rb /)
      kbo(:, 3,15,15) = (/ &
        & 0.24797e+02_rb,0.19155e+02_rb,0.17917e+02_rb,0.18103e+02_rb,0.23536e+02_rb /)
      kbo(:, 4,15,15) = (/ &
        & 0.24619e+02_rb,0.18942e+02_rb,0.17624e+02_rb,0.17814e+02_rb,0.23238e+02_rb /)
      kbo(:, 5,15,15) = (/ &
        & 0.24389e+02_rb,0.18694e+02_rb,0.17313e+02_rb,0.17524e+02_rb,0.22925e+02_rb /)
      kbo(:, 1,16,15) = (/ &
        & 0.26656e+02_rb,0.20879e+02_rb,0.19716e+02_rb,0.19730e+02_rb,0.25227e+02_rb /)
      kbo(:, 2,16,15) = (/ &
        & 0.26523e+02_rb,0.20660e+02_rb,0.19424e+02_rb,0.19442e+02_rb,0.24961e+02_rb /)
      kbo(:, 3,16,15) = (/ &
        & 0.26348e+02_rb,0.20405e+02_rb,0.19093e+02_rb,0.19135e+02_rb,0.24671e+02_rb /)
      kbo(:, 4,16,15) = (/ &
        & 0.26107e+02_rb,0.20124e+02_rb,0.18741e+02_rb,0.18812e+02_rb,0.24357e+02_rb /)
      kbo(:, 5,16,15) = (/ &
        & 0.25816e+02_rb,0.19816e+02_rb,0.18364e+02_rb,0.18476e+02_rb,0.24001e+02_rb /)
      kbo(:, 1,17,15) = (/ &
        & 0.28242e+02_rb,0.22184e+02_rb,0.20936e+02_rb,0.20769e+02_rb,0.26307e+02_rb /)
      kbo(:, 2,17,15) = (/ &
        & 0.28053e+02_rb,0.21895e+02_rb,0.20570e+02_rb,0.20444e+02_rb,0.26016e+02_rb /)
      kbo(:, 3,17,15) = (/ &
        & 0.27803e+02_rb,0.21584e+02_rb,0.20174e+02_rb,0.20096e+02_rb,0.25702e+02_rb /)
      kbo(:, 4,17,15) = (/ &
        & 0.27499e+02_rb,0.21233e+02_rb,0.19753e+02_rb,0.19726e+02_rb,0.25351e+02_rb /)
      kbo(:, 5,17,15) = (/ &
        & 0.27139e+02_rb,0.20859e+02_rb,0.19319e+02_rb,0.19342e+02_rb,0.24955e+02_rb /)
      kbo(:, 1,18,15) = (/ &
        & 0.29720e+02_rb,0.23411e+02_rb,0.22065e+02_rb,0.21731e+02_rb,0.27280e+02_rb /)
      kbo(:, 2,18,15) = (/ &
        & 0.29480e+02_rb,0.23053e+02_rb,0.21624e+02_rb,0.21360e+02_rb,0.26972e+02_rb /)
      kbo(:, 3,18,15) = (/ &
        & 0.29159e+02_rb,0.22667e+02_rb,0.21156e+02_rb,0.20967e+02_rb,0.26625e+02_rb /)
      kbo(:, 4,18,15) = (/ &
        & 0.28776e+02_rb,0.22248e+02_rb,0.20663e+02_rb,0.20553e+02_rb,0.26229e+02_rb /)
      kbo(:, 5,18,15) = (/ &
        & 0.28340e+02_rb,0.21810e+02_rb,0.20170e+02_rb,0.20119e+02_rb,0.25790e+02_rb /)
      kbo(:, 1,19,15) = (/ &
        & 0.31079e+02_rb,0.24533e+02_rb,0.23084e+02_rb,0.22601e+02_rb,0.28156e+02_rb /)
      kbo(:, 2,19,15) = (/ &
        & 0.30766e+02_rb,0.24108e+02_rb,0.22572e+02_rb,0.22185e+02_rb,0.27815e+02_rb /)
      kbo(:, 3,19,15) = (/ &
        & 0.30376e+02_rb,0.23641e+02_rb,0.22030e+02_rb,0.21741e+02_rb,0.27429e+02_rb /)
      kbo(:, 4,19,15) = (/ &
        & 0.29923e+02_rb,0.23160e+02_rb,0.21471e+02_rb,0.21280e+02_rb,0.26992e+02_rb /)
      kbo(:, 5,19,15) = (/ &
        & 0.29415e+02_rb,0.22660e+02_rb,0.20920e+02_rb,0.20797e+02_rb,0.26511e+02_rb /)
      kbo(:, 1,20,15) = (/ &
        & 0.32290e+02_rb,0.25527e+02_rb,0.23984e+02_rb,0.23373e+02_rb,0.28918e+02_rb /)
      kbo(:, 2,20,15) = (/ &
        & 0.31907e+02_rb,0.25035e+02_rb,0.23397e+02_rb,0.22899e+02_rb,0.28537e+02_rb /)
      kbo(:, 3,20,15) = (/ &
        & 0.31450e+02_rb,0.24496e+02_rb,0.22784e+02_rb,0.22408e+02_rb,0.28112e+02_rb /)
      kbo(:, 4,20,15) = (/ &
        & 0.30926e+02_rb,0.23946e+02_rb,0.22166e+02_rb,0.21898e+02_rb,0.27628e+02_rb /)
      kbo(:, 5,20,15) = (/ &
        & 0.30351e+02_rb,0.23384e+02_rb,0.21559e+02_rb,0.21369e+02_rb,0.27109e+02_rb /)
      kbo(:, 1,21,15) = (/ &
        & 0.33354e+02_rb,0.26397e+02_rb,0.24768e+02_rb,0.24040e+02_rb,0.29575e+02_rb /)
      kbo(:, 2,21,15) = (/ &
        & 0.32901e+02_rb,0.25842e+02_rb,0.24107e+02_rb,0.23511e+02_rb,0.29153e+02_rb /)
      kbo(:, 3,21,15) = (/ &
        & 0.32374e+02_rb,0.25240e+02_rb,0.23426e+02_rb,0.22972e+02_rb,0.28686e+02_rb /)
      kbo(:, 4,21,15) = (/ &
        & 0.31781e+02_rb,0.24624e+02_rb,0.22754e+02_rb,0.22416e+02_rb,0.28162e+02_rb /)
      kbo(:, 5,21,15) = (/ &
        & 0.31147e+02_rb,0.24003e+02_rb,0.22095e+02_rb,0.21845e+02_rb,0.27608e+02_rb /)
      kbo(:, 1,22,15) = (/ &
        & 0.34245e+02_rb,0.27109e+02_rb,0.25388e+02_rb,0.24566e+02_rb,0.30099e+02_rb /)
      kbo(:, 2,22,15) = (/ &
        & 0.33719e+02_rb,0.26481e+02_rb,0.24655e+02_rb,0.23987e+02_rb,0.29635e+02_rb /)
      kbo(:, 3,22,15) = (/ &
        & 0.33121e+02_rb,0.25821e+02_rb,0.23916e+02_rb,0.23403e+02_rb,0.29124e+02_rb /)
      kbo(:, 4,22,15) = (/ &
        & 0.32461e+02_rb,0.25147e+02_rb,0.23196e+02_rb,0.22805e+02_rb,0.28562e+02_rb /)
      kbo(:, 5,22,15) = (/ &
        & 0.31774e+02_rb,0.24482e+02_rb,0.22495e+02_rb,0.22197e+02_rb,0.27979e+02_rb /)
      kbo(:, 1,23,15) = (/ &
        & 0.34979e+02_rb,0.27686e+02_rb,0.25885e+02_rb,0.24985e+02_rb,0.30523e+02_rb /)
      kbo(:, 2,23,15) = (/ &
        & 0.34390e+02_rb,0.26989e+02_rb,0.25088e+02_rb,0.24366e+02_rb,0.30015e+02_rb /)
      kbo(:, 3,23,15) = (/ &
        & 0.33724e+02_rb,0.26279e+02_rb,0.24300e+02_rb,0.23742e+02_rb,0.29464e+02_rb /)
      kbo(:, 4,23,15) = (/ &
        & 0.33013e+02_rb,0.25562e+02_rb,0.23538e+02_rb,0.23108e+02_rb,0.28875e+02_rb /)
      kbo(:, 5,23,15) = (/ &
        & 0.32269e+02_rb,0.24853e+02_rb,0.22796e+02_rb,0.22467e+02_rb,0.28265e+02_rb /)
      kbo(:, 1,24,15) = (/ &
        & 0.35573e+02_rb,0.28136e+02_rb,0.26264e+02_rb,0.25305e+02_rb,0.30849e+02_rb /)
      kbo(:, 2,24,15) = (/ &
        & 0.34922e+02_rb,0.27387e+02_rb,0.25413e+02_rb,0.24653e+02_rb,0.30308e+02_rb /)
      kbo(:, 3,24,15) = (/ &
        & 0.34200e+02_rb,0.26632e+02_rb,0.24587e+02_rb,0.23994e+02_rb,0.29723e+02_rb /)
      kbo(:, 4,24,15) = (/ &
        & 0.33439e+02_rb,0.25872e+02_rb,0.23787e+02_rb,0.23329e+02_rb,0.29105e+02_rb /)
      kbo(:, 5,24,15) = (/ &
        & 0.32647e+02_rb,0.25128e+02_rb,0.23014e+02_rb,0.22661e+02_rb,0.28477e+02_rb /)
      kbo(:, 1,25,15) = (/ &
        & 0.36033e+02_rb,0.28469e+02_rb,0.26533e+02_rb,0.25539e+02_rb,0.31091e+02_rb /)
      kbo(:, 2,25,15) = (/ &
        & 0.35324e+02_rb,0.27679e+02_rb,0.25640e+02_rb,0.24852e+02_rb,0.30512e+02_rb /)
      kbo(:, 3,25,15) = (/ &
        & 0.34555e+02_rb,0.26887e+02_rb,0.24782e+02_rb,0.24167e+02_rb,0.29902e+02_rb /)
      kbo(:, 4,25,15) = (/ &
        & 0.33752e+02_rb,0.26092e+02_rb,0.23953e+02_rb,0.23474e+02_rb,0.29262e+02_rb /)
      kbo(:, 5,25,15) = (/ &
        & 0.32915e+02_rb,0.25319e+02_rb,0.23155e+02_rb,0.22787e+02_rb,0.28617e+02_rb /)
      kbo(:, 1,26,15) = (/ &
        & 0.36371e+02_rb,0.28696e+02_rb,0.26697e+02_rb,0.25688e+02_rb,0.31254e+02_rb /)
      kbo(:, 2,26,15) = (/ &
        & 0.35607e+02_rb,0.27872e+02_rb,0.25775e+02_rb,0.24975e+02_rb,0.30642e+02_rb /)
      kbo(:, 3,26,15) = (/ &
        & 0.34798e+02_rb,0.27047e+02_rb,0.24890e+02_rb,0.24264e+02_rb,0.30007e+02_rb /)
      kbo(:, 4,26,15) = (/ &
        & 0.33956e+02_rb,0.26227e+02_rb,0.24040e+02_rb,0.23549e+02_rb,0.29351e+02_rb /)
      kbo(:, 5,26,15) = (/ &
        & 0.33076e+02_rb,0.25425e+02_rb,0.23222e+02_rb,0.22845e+02_rb,0.28688e+02_rb /)
      kbo(:, 1,27,15) = (/ &
        & 0.36605e+02_rb,0.28839e+02_rb,0.26780e+02_rb,0.25770e+02_rb,0.31349e+02_rb /)
      kbo(:, 2,27,15) = (/ &
        & 0.35796e+02_rb,0.27987e+02_rb,0.25836e+02_rb,0.25037e+02_rb,0.30713e+02_rb /)
      kbo(:, 3,27,15) = (/ &
        & 0.34952e+02_rb,0.27136e+02_rb,0.24934e+02_rb,0.24306e+02_rb,0.30059e+02_rb /)
      kbo(:, 4,27,15) = (/ &
        & 0.34075e+02_rb,0.26294e+02_rb,0.24068e+02_rb,0.23575e+02_rb,0.29392e+02_rb /)
      kbo(:, 5,27,15) = (/ &
        & 0.33166e+02_rb,0.25473e+02_rb,0.23238e+02_rb,0.22856e+02_rb,0.28711e+02_rb /)
      kbo(:, 1,28,15) = (/ &
        & 0.36758e+02_rb,0.28917e+02_rb,0.26804e+02_rb,0.25803e+02_rb,0.31396e+02_rb /)
      kbo(:, 2,28,15) = (/ &
        & 0.35910e+02_rb,0.28037e+02_rb,0.25842e+02_rb,0.25051e+02_rb,0.30739e+02_rb /)
      kbo(:, 3,28,15) = (/ &
        & 0.35033e+02_rb,0.27165e+02_rb,0.24925e+02_rb,0.24304e+02_rb,0.30070e+02_rb /)
      kbo(:, 4,28,15) = (/ &
        & 0.34125e+02_rb,0.26307e+02_rb,0.24049e+02_rb,0.23563e+02_rb,0.29393e+02_rb /)
      kbo(:, 5,28,15) = (/ &
        & 0.33188e+02_rb,0.25468e+02_rb,0.23210e+02_rb,0.22829e+02_rb,0.28695e+02_rb /)
      kbo(:, 1,29,15) = (/ &
        & 0.36829e+02_rb,0.28926e+02_rb,0.26766e+02_rb,0.25785e+02_rb,0.31393e+02_rb /)
      kbo(:, 2,29,15) = (/ &
        & 0.35949e+02_rb,0.28025e+02_rb,0.25792e+02_rb,0.25019e+02_rb,0.30723e+02_rb /)
      kbo(:, 3,29,15) = (/ &
        & 0.35045e+02_rb,0.27141e+02_rb,0.24868e+02_rb,0.24260e+02_rb,0.30042e+02_rb /)
      kbo(:, 4,29,15) = (/ &
        & 0.34114e+02_rb,0.26272e+02_rb,0.23986e+02_rb,0.23509e+02_rb,0.29353e+02_rb /)
      kbo(:, 5,29,15) = (/ &
        & 0.33154e+02_rb,0.25419e+02_rb,0.23142e+02_rb,0.22765e+02_rb,0.28643e+02_rb /)
      kbo(:, 1,30,15) = (/ &
        & 0.36836e+02_rb,0.28884e+02_rb,0.26681e+02_rb,0.25729e+02_rb,0.31353e+02_rb /)
      kbo(:, 2,30,15) = (/ &
        & 0.35935e+02_rb,0.27972e+02_rb,0.25704e+02_rb,0.24955e+02_rb,0.30675e+02_rb /)
      kbo(:, 3,30,15) = (/ &
        & 0.35002e+02_rb,0.27075e+02_rb,0.24775e+02_rb,0.24184e+02_rb,0.29984e+02_rb /)
      kbo(:, 4,30,15) = (/ &
        & 0.34050e+02_rb,0.26196e+02_rb,0.23891e+02_rb,0.23427e+02_rb,0.29283e+02_rb /)
      kbo(:, 5,30,15) = (/ &
        & 0.33076e+02_rb,0.25337e+02_rb,0.23046e+02_rb,0.22676e+02_rb,0.28566e+02_rb /)
      kbo(:, 1,31,15) = (/ &
        & 0.36791e+02_rb,0.28797e+02_rb,0.26557e+02_rb,0.25640e+02_rb,0.31284e+02_rb /)
      kbo(:, 2,31,15) = (/ &
        & 0.35869e+02_rb,0.27877e+02_rb,0.25582e+02_rb,0.24857e+02_rb,0.30596e+02_rb /)
      kbo(:, 3,31,15) = (/ &
        & 0.34914e+02_rb,0.26971e+02_rb,0.24653e+02_rb,0.24083e+02_rb,0.29901e+02_rb /)
      kbo(:, 4,31,15) = (/ &
        & 0.33941e+02_rb,0.26084e+02_rb,0.23767e+02_rb,0.23319e+02_rb,0.29185e+02_rb /)
      kbo(:, 5,31,15) = (/ &
        & 0.32958e+02_rb,0.25224e+02_rb,0.22924e+02_rb,0.22563e+02_rb,0.28464e+02_rb /)
      kbo(:, 1,32,15) = (/ &
        & 0.36704e+02_rb,0.28676e+02_rb,0.26402e+02_rb,0.25521e+02_rb,0.31186e+02_rb /)
      kbo(:, 2,32,15) = (/ &
        & 0.35759e+02_rb,0.27748e+02_rb,0.25429e+02_rb,0.24734e+02_rb,0.30492e+02_rb /)
      kbo(:, 3,32,15) = (/ &
        & 0.34789e+02_rb,0.26839e+02_rb,0.24504e+02_rb,0.23956e+02_rb,0.29792e+02_rb /)
      kbo(:, 4,32,15) = (/ &
        & 0.33799e+02_rb,0.25947e+02_rb,0.23622e+02_rb,0.23189e+02_rb,0.29067e+02_rb /)
      kbo(:, 5,32,15) = (/ &
        & 0.32807e+02_rb,0.25086e+02_rb,0.22780e+02_rb,0.22431e+02_rb,0.28343e+02_rb /)
      kbo(:, 1,33,15) = (/ &
        & 0.36581e+02_rb,0.28529e+02_rb,0.26229e+02_rb,0.25386e+02_rb,0.31076e+02_rb /)
      kbo(:, 2,33,15) = (/ &
        & 0.35618e+02_rb,0.27597e+02_rb,0.25258e+02_rb,0.24592e+02_rb,0.30373e+02_rb /)
      kbo(:, 3,33,15) = (/ &
        & 0.34634e+02_rb,0.26685e+02_rb,0.24337e+02_rb,0.23815e+02_rb,0.29666e+02_rb /)
      kbo(:, 4,33,15) = (/ &
        & 0.33632e+02_rb,0.25791e+02_rb,0.23459e+02_rb,0.23042e+02_rb,0.28931e+02_rb /)
      kbo(:, 5,33,15) = (/ &
        & 0.32632e+02_rb,0.24931e+02_rb,0.22622e+02_rb,0.22288e+02_rb,0.28210e+02_rb /)
      kbo(:, 1,34,15) = (/ &
        & 0.36458e+02_rb,0.28392e+02_rb,0.26072e+02_rb,0.25264e+02_rb,0.30972e+02_rb /)
      kbo(:, 2,34,15) = (/ &
        & 0.35483e+02_rb,0.27459e+02_rb,0.25106e+02_rb,0.24467e+02_rb,0.30265e+02_rb /)
      kbo(:, 3,34,15) = (/ &
        & 0.34486e+02_rb,0.26542e+02_rb,0.24189e+02_rb,0.23688e+02_rb,0.29550e+02_rb /)
      kbo(:, 4,34,15) = (/ &
        & 0.33480e+02_rb,0.25653e+02_rb,0.23315e+02_rb,0.22913e+02_rb,0.28811e+02_rb /)
      kbo(:, 5,34,15) = (/ &
        & 0.32478e+02_rb,0.24793e+02_rb,0.22484e+02_rb,0.22163e+02_rb,0.28094e+02_rb /)
      kbo(:, 1,35,15) = (/ &
        & 0.36414e+02_rb,0.28334e+02_rb,0.25999e+02_rb,0.25206e+02_rb,0.30923e+02_rb /)
      kbo(:, 2,35,15) = (/ &
        & 0.35425e+02_rb,0.27395e+02_rb,0.25033e+02_rb,0.24406e+02_rb,0.30215e+02_rb /)
      kbo(:, 3,35,15) = (/ &
        & 0.34420e+02_rb,0.26476e+02_rb,0.24117e+02_rb,0.23624e+02_rb,0.29493e+02_rb /)
      kbo(:, 4,35,15) = (/ &
        & 0.33407e+02_rb,0.25585e+02_rb,0.23243e+02_rb,0.22849e+02_rb,0.28753e+02_rb /)
      kbo(:, 5,35,15) = (/ &
        & 0.32403e+02_rb,0.24726e+02_rb,0.22415e+02_rb,0.22101e+02_rb,0.28034e+02_rb /)
      kbo(:, 1,36,15) = (/ &
        & 0.36456e+02_rb,0.28365e+02_rb,0.26023e+02_rb,0.25227e+02_rb,0.30947e+02_rb /)
      kbo(:, 2,36,15) = (/ &
        & 0.35465e+02_rb,0.27424e+02_rb,0.25054e+02_rb,0.24425e+02_rb,0.30235e+02_rb /)
      kbo(:, 3,36,15) = (/ &
        & 0.34451e+02_rb,0.26498e+02_rb,0.24134e+02_rb,0.23640e+02_rb,0.29510e+02_rb /)
      kbo(:, 4,36,15) = (/ &
        & 0.33436e+02_rb,0.25606e+02_rb,0.23259e+02_rb,0.22862e+02_rb,0.28768e+02_rb /)
      kbo(:, 5,36,15) = (/ &
        & 0.32427e+02_rb,0.24743e+02_rb,0.22427e+02_rb,0.22112e+02_rb,0.28048e+02_rb /)
      kbo(:, 1,37,15) = (/ &
        & 0.36638e+02_rb,0.28533e+02_rb,0.26191e+02_rb,0.25363e+02_rb,0.31069e+02_rb /)
      kbo(:, 2,37,15) = (/ &
        & 0.35645e+02_rb,0.27585e+02_rb,0.25212e+02_rb,0.24557e+02_rb,0.30356e+02_rb /)
      kbo(:, 3,37,15) = (/ &
        & 0.34632e+02_rb,0.26655e+02_rb,0.24283e+02_rb,0.23768e+02_rb,0.29633e+02_rb /)
      kbo(:, 4,37,15) = (/ &
        & 0.33611e+02_rb,0.25753e+02_rb,0.23398e+02_rb,0.22987e+02_rb,0.28889e+02_rb /)
      kbo(:, 5,37,15) = (/ &
        & 0.32597e+02_rb,0.24885e+02_rb,0.22559e+02_rb,0.22230e+02_rb,0.28165e+02_rb /)
      kbo(:, 1,38,15) = (/ &
        & 0.36818e+02_rb,0.28702e+02_rb,0.26363e+02_rb,0.25503e+02_rb,0.31194e+02_rb /)
      kbo(:, 2,38,15) = (/ &
        & 0.35825e+02_rb,0.27749e+02_rb,0.25373e+02_rb,0.24692e+02_rb,0.30479e+02_rb /)
      kbo(:, 3,38,15) = (/ &
        & 0.34809e+02_rb,0.26811e+02_rb,0.24434e+02_rb,0.23900e+02_rb,0.29757e+02_rb /)
      kbo(:, 4,38,15) = (/ &
        & 0.33789e+02_rb,0.25904e+02_rb,0.23543e+02_rb,0.23114e+02_rb,0.29012e+02_rb /)
      kbo(:, 5,38,15) = (/ &
        & 0.32767e+02_rb,0.25028e+02_rb,0.22694e+02_rb,0.22351e+02_rb,0.28285e+02_rb /)
      kbo(:, 1,39,15) = (/ &
        & 0.36989e+02_rb,0.28865e+02_rb,0.26530e+02_rb,0.25636e+02_rb,0.31312e+02_rb /)
      kbo(:, 2,39,15) = (/ &
        & 0.35999e+02_rb,0.27907e+02_rb,0.25529e+02_rb,0.24823e+02_rb,0.30597e+02_rb /)
      kbo(:, 3,39,15) = (/ &
        & 0.34983e+02_rb,0.26966e+02_rb,0.24583e+02_rb,0.24027e+02_rb,0.29878e+02_rb /)
      kbo(:, 4,39,15) = (/ &
        & 0.33957e+02_rb,0.26048e+02_rb,0.23683e+02_rb,0.23240e+02_rb,0.29134e+02_rb /)
      kbo(:, 5,39,15) = (/ &
        & 0.32935e+02_rb,0.25169e+02_rb,0.22826e+02_rb,0.22468e+02_rb,0.28398e+02_rb /)
      kbo(:, 1,40,15) = (/ &
        & 0.37254e+02_rb,0.29125e+02_rb,0.26800e+02_rb,0.25853e+02_rb,0.31502e+02_rb /)
      kbo(:, 2,40,15) = (/ &
        & 0.36269e+02_rb,0.28159e+02_rb,0.25786e+02_rb,0.25036e+02_rb,0.30787e+02_rb /)
      kbo(:, 3,40,15) = (/ &
        & 0.35253e+02_rb,0.27211e+02_rb,0.24825e+02_rb,0.24233e+02_rb,0.30069e+02_rb /)
      kbo(:, 4,40,15) = (/ &
        & 0.34229e+02_rb,0.26287e+02_rb,0.23913e+02_rb,0.23442e+02_rb,0.29327e+02_rb /)
      kbo(:, 5,40,15) = (/ &
        & 0.33202e+02_rb,0.25396e+02_rb,0.23042e+02_rb,0.22665e+02_rb,0.28587e+02_rb /)
      kbo(:, 1,41,15) = (/ &
        & 0.37526e+02_rb,0.29396e+02_rb,0.27087e+02_rb,0.26081e+02_rb,0.31701e+02_rb /)
      kbo(:, 2,41,15) = (/ &
        & 0.36546e+02_rb,0.28422e+02_rb,0.26056e+02_rb,0.25257e+02_rb,0.30982e+02_rb /)
      kbo(:, 3,41,15) = (/ &
        & 0.35534e+02_rb,0.27467e+02_rb,0.25079e+02_rb,0.24448e+02_rb,0.30265e+02_rb /)
      kbo(:, 4,41,15) = (/ &
        & 0.34510e+02_rb,0.26534e+02_rb,0.24153e+02_rb,0.23654e+02_rb,0.29528e+02_rb /)
      kbo(:, 5,41,15) = (/ &
        & 0.33481e+02_rb,0.25633e+02_rb,0.23271e+02_rb,0.22872e+02_rb,0.28783e+02_rb /)
      kbo(:, 1,42,15) = (/ &
        & 0.37791e+02_rb,0.29665e+02_rb,0.27375e+02_rb,0.26306e+02_rb,0.31897e+02_rb /)
      kbo(:, 2,42,15) = (/ &
        & 0.36818e+02_rb,0.28682e+02_rb,0.26328e+02_rb,0.25479e+02_rb,0.31179e+02_rb /)
      kbo(:, 3,42,15) = (/ &
        & 0.35814e+02_rb,0.27725e+02_rb,0.25336e+02_rb,0.24663e+02_rb,0.30459e+02_rb /)
      kbo(:, 4,42,15) = (/ &
        & 0.34792e+02_rb,0.26784e+02_rb,0.24397e+02_rb,0.23867e+02_rb,0.29730e+02_rb /)
      kbo(:, 5,42,15) = (/ &
        & 0.33761e+02_rb,0.25873e+02_rb,0.23504e+02_rb,0.23079e+02_rb,0.28983e+02_rb /)
      kbo(:, 1,43,15) = (/ &
        & 0.38102e+02_rb,0.29987e+02_rb,0.27726e+02_rb,0.26579e+02_rb,0.32132e+02_rb /)
      kbo(:, 2,43,15) = (/ &
        & 0.37142e+02_rb,0.29000e+02_rb,0.26660e+02_rb,0.25743e+02_rb,0.31410e+02_rb /)
      kbo(:, 3,43,15) = (/ &
        & 0.36146e+02_rb,0.28034e+02_rb,0.25651e+02_rb,0.24927e+02_rb,0.30694e+02_rb /)
      kbo(:, 4,43,15) = (/ &
        & 0.35125e+02_rb,0.27087e+02_rb,0.24694e+02_rb,0.24121e+02_rb,0.29968e+02_rb /)
      kbo(:, 5,43,15) = (/ &
        & 0.34093e+02_rb,0.26161e+02_rb,0.23786e+02_rb,0.23330e+02_rb,0.29223e+02_rb /)
      kbo(:, 1,44,15) = (/ &
        & 0.38433e+02_rb,0.30339e+02_rb,0.28106e+02_rb,0.26874e+02_rb,0.32381e+02_rb /)
      kbo(:, 2,44,15) = (/ &
        & 0.37485e+02_rb,0.29344e+02_rb,0.27023e+02_rb,0.26033e+02_rb,0.31664e+02_rb /)
      kbo(:, 3,44,15) = (/ &
        & 0.36497e+02_rb,0.28368e+02_rb,0.25996e+02_rb,0.25211e+02_rb,0.30945e+02_rb /)
      kbo(:, 4,44,15) = (/ &
        & 0.35483e+02_rb,0.27414e+02_rb,0.25019e+02_rb,0.24397e+02_rb,0.30221e+02_rb /)
      kbo(:, 5,44,15) = (/ &
        & 0.34454e+02_rb,0.26480e+02_rb,0.24095e+02_rb,0.23603e+02_rb,0.29482e+02_rb /)
      kbo(:, 1,45,15) = (/ &
        & 0.38755e+02_rb,0.30690e+02_rb,0.28491e+02_rb,0.27169e+02_rb,0.32624e+02_rb /)
      kbo(:, 2,45,15) = (/ &
        & 0.37819e+02_rb,0.29686e+02_rb,0.27393e+02_rb,0.26324e+02_rb,0.31916e+02_rb /)
      kbo(:, 3,45,15) = (/ &
        & 0.36842e+02_rb,0.28700e+02_rb,0.26343e+02_rb,0.25494e+02_rb,0.31195e+02_rb /)
      kbo(:, 4,45,15) = (/ &
        & 0.35840e+02_rb,0.27743e+02_rb,0.25350e+02_rb,0.24674e+02_rb,0.30470e+02_rb /)
      kbo(:, 5,45,15) = (/ &
        & 0.34814e+02_rb,0.26800e+02_rb,0.24408e+02_rb,0.23874e+02_rb,0.29739e+02_rb /)
      kbo(:, 1,46,15) = (/ &
        & 0.39090e+02_rb,0.31065e+02_rb,0.28908e+02_rb,0.27486e+02_rb,0.32877e+02_rb /)
      kbo(:, 2,46,15) = (/ &
        & 0.38170e+02_rb,0.30050e+02_rb,0.27790e+02_rb,0.26631e+02_rb,0.32180e+02_rb /)
      kbo(:, 3,46,15) = (/ &
        & 0.37208e+02_rb,0.29060e+02_rb,0.26719e+02_rb,0.25793e+02_rb,0.31457e+02_rb /)
      kbo(:, 4,46,15) = (/ &
        & 0.36211e+02_rb,0.28091e+02_rb,0.25705e+02_rb,0.24972e+02_rb,0.30735e+02_rb /)
      kbo(:, 5,46,15) = (/ &
        & 0.35190e+02_rb,0.27143e+02_rb,0.24746e+02_rb,0.24165e+02_rb,0.30011e+02_rb /)
      kbo(:, 1,47,15) = (/ &
        & 0.39451e+02_rb,0.31490e+02_rb,0.29378e+02_rb,0.27842e+02_rb,0.33145e+02_rb /)
      kbo(:, 2,47,15) = (/ &
        & 0.38554e+02_rb,0.30463e+02_rb,0.28238e+02_rb,0.26977e+02_rb,0.32470e+02_rb /)
      kbo(:, 3,47,15) = (/ &
        & 0.37608e+02_rb,0.29464e+02_rb,0.27149e+02_rb,0.26134e+02_rb,0.31753e+02_rb /)
      kbo(:, 4,47,15) = (/ &
        & 0.36624e+02_rb,0.28485e+02_rb,0.26114e+02_rb,0.25307e+02_rb,0.31030e+02_rb /)
      kbo(:, 5,47,15) = (/ &
        & 0.35612e+02_rb,0.27529e+02_rb,0.25131e+02_rb,0.24491e+02_rb,0.30307e+02_rb /)
      kbo(:, 1,48,15) = (/ &
        & 0.39789e+02_rb,0.31912e+02_rb,0.29855e+02_rb,0.28207e+02_rb,0.33402e+02_rb /)
      kbo(:, 2,48,15) = (/ &
        & 0.38929e+02_rb,0.30877e+02_rb,0.28697e+02_rb,0.27328e+02_rb,0.32755e+02_rb /)
      kbo(:, 3,48,15) = (/ &
        & 0.38000e+02_rb,0.29868e+02_rb,0.27588e+02_rb,0.26477e+02_rb,0.32050e+02_rb /)
      kbo(:, 4,48,15) = (/ &
        & 0.37030e+02_rb,0.28879e+02_rb,0.26527e+02_rb,0.25639e+02_rb,0.31322e+02_rb /)
      kbo(:, 5,48,15) = (/ &
        & 0.36027e+02_rb,0.27916e+02_rb,0.25523e+02_rb,0.24819e+02_rb,0.30601e+02_rb /)
      kbo(:, 1,49,15) = (/ &
        & 0.40116e+02_rb,0.32336e+02_rb,0.30332e+02_rb,0.28574e+02_rb,0.33637e+02_rb /)
      kbo(:, 2,49,15) = (/ &
        & 0.39295e+02_rb,0.31299e+02_rb,0.29164e+02_rb,0.27682e+02_rb,0.33028e+02_rb /)
      kbo(:, 3,49,15) = (/ &
        & 0.38385e+02_rb,0.30275e+02_rb,0.28033e+02_rb,0.26821e+02_rb,0.32341e+02_rb /)
      kbo(:, 4,49,15) = (/ &
        & 0.37432e+02_rb,0.29281e+02_rb,0.26949e+02_rb,0.25975e+02_rb,0.31615e+02_rb /)
      kbo(:, 5,49,15) = (/ &
        & 0.36440e+02_rb,0.28306e+02_rb,0.25923e+02_rb,0.25151e+02_rb,0.30893e+02_rb /)
      kbo(:, 1,50,15) = (/ &
        & 0.40414e+02_rb,0.32736e+02_rb,0.30783e+02_rb,0.28924e+02_rb,0.33845e+02_rb /)
      kbo(:, 2,50,15) = (/ &
        & 0.39619e+02_rb,0.31694e+02_rb,0.29608e+02_rb,0.28020e+02_rb,0.33276e+02_rb /)
      kbo(:, 3,50,15) = (/ &
        & 0.38737e+02_rb,0.30661e+02_rb,0.28456e+02_rb,0.27146e+02_rb,0.32609e+02_rb /)
      kbo(:, 4,50,15) = (/ &
        & 0.37800e+02_rb,0.29658e+02_rb,0.27356e+02_rb,0.26296e+02_rb,0.31894e+02_rb /)
      kbo(:, 5,50,15) = (/ &
        & 0.36822e+02_rb,0.28673e+02_rb,0.26306e+02_rb,0.25463e+02_rb,0.31168e+02_rb /)
      kbo(:, 1,51,15) = (/ &
        & 0.40689e+02_rb,0.33123e+02_rb,0.31211e+02_rb,0.29252e+02_rb,0.34023e+02_rb /)
      kbo(:, 2,51,15) = (/ &
        & 0.39919e+02_rb,0.32076e+02_rb,0.30039e+02_rb,0.28350e+02_rb,0.33499e+02_rb /)
      kbo(:, 3,51,15) = (/ &
        & 0.39075e+02_rb,0.31039e+02_rb,0.28875e+02_rb,0.27462e+02_rb,0.32860e+02_rb /)
      kbo(:, 4,51,15) = (/ &
        & 0.38151e+02_rb,0.30023e+02_rb,0.27755e+02_rb,0.26605e+02_rb,0.32160e+02_rb /)
      kbo(:, 5,51,15) = (/ &
        & 0.37189e+02_rb,0.29033e+02_rb,0.26685e+02_rb,0.25765e+02_rb,0.31432e+02_rb /)
      kbo(:, 1,52,15) = (/ &
        & 0.40936e+02_rb,0.33510e+02_rb,0.31640e+02_rb,0.29576e+02_rb,0.34183e+02_rb /)
      kbo(:, 2,52,15) = (/ &
        & 0.40210e+02_rb,0.32459e+02_rb,0.30471e+02_rb,0.28683e+02_rb,0.33706e+02_rb /)
      kbo(:, 3,52,15) = (/ &
        & 0.39397e+02_rb,0.31418e+02_rb,0.29296e+02_rb,0.27784e+02_rb,0.33107e+02_rb /)
      kbo(:, 4,52,15) = (/ &
        & 0.38495e+02_rb,0.30394e+02_rb,0.28160e+02_rb,0.26919e+02_rb,0.32422e+02_rb /)
      kbo(:, 5,52,15) = (/ &
        & 0.37548e+02_rb,0.29397e+02_rb,0.27070e+02_rb,0.26070e+02_rb,0.31697e+02_rb /)
      kbo(:, 1,53,15) = (/ &
        & 0.41161e+02_rb,0.33900e+02_rb,0.32075e+02_rb,0.29895e+02_rb,0.34325e+02_rb /)
      kbo(:, 2,53,15) = (/ &
        & 0.40494e+02_rb,0.32844e+02_rb,0.30901e+02_rb,0.29015e+02_rb,0.33898e+02_rb /)
      kbo(:, 3,53,15) = (/ &
        & 0.39709e+02_rb,0.31802e+02_rb,0.29726e+02_rb,0.28110e+02_rb,0.33339e+02_rb /)
      kbo(:, 4,53,15) = (/ &
        & 0.38833e+02_rb,0.30766e+02_rb,0.28570e+02_rb,0.27234e+02_rb,0.32680e+02_rb /)
      kbo(:, 5,53,15) = (/ &
        & 0.37902e+02_rb,0.29761e+02_rb,0.27465e+02_rb,0.26380e+02_rb,0.31966e+02_rb /)
      kbo(:, 1,54,15) = (/ &
        & 0.41350e+02_rb,0.34259e+02_rb,0.32468e+02_rb,0.30182e+02_rb,0.34433e+02_rb /)
      kbo(:, 2,54,15) = (/ &
        & 0.40740e+02_rb,0.33197e+02_rb,0.31292e+02_rb,0.29316e+02_rb,0.34057e+02_rb /)
      kbo(:, 3,54,15) = (/ &
        & 0.39978e+02_rb,0.32149e+02_rb,0.30122e+02_rb,0.28414e+02_rb,0.33542e+02_rb /)
      kbo(:, 4,54,15) = (/ &
        & 0.39139e+02_rb,0.31111e+02_rb,0.28955e+02_rb,0.27524e+02_rb,0.32911e+02_rb /)
      kbo(:, 5,54,15) = (/ &
        & 0.38222e+02_rb,0.30095e+02_rb,0.27832e+02_rb,0.26664e+02_rb,0.32209e+02_rb /)
      kbo(:, 1,55,15) = (/ &
        & 0.41508e+02_rb,0.34608e+02_rb,0.32849e+02_rb,0.30454e+02_rb,0.34522e+02_rb /)
      kbo(:, 2,55,15) = (/ &
        & 0.40955e+02_rb,0.33539e+02_rb,0.31671e+02_rb,0.29598e+02_rb,0.34194e+02_rb /)
      kbo(:, 3,55,15) = (/ &
        & 0.40232e+02_rb,0.32485e+02_rb,0.30501e+02_rb,0.28707e+02_rb,0.33722e+02_rb /)
      kbo(:, 4,55,15) = (/ &
        & 0.39425e+02_rb,0.31448e+02_rb,0.29326e+02_rb,0.27805e+02_rb,0.33122e+02_rb /)
      kbo(:, 5,55,15) = (/ &
        & 0.38521e+02_rb,0.30419e+02_rb,0.28187e+02_rb,0.26941e+02_rb,0.32441e+02_rb /)
      kbo(:, 1,56,15) = (/ &
        & 0.41645e+02_rb,0.34957e+02_rb,0.33233e+02_rb,0.30720e+02_rb,0.34599e+02_rb /)
      kbo(:, 2,56,15) = (/ &
        & 0.41152e+02_rb,0.33880e+02_rb,0.32050e+02_rb,0.29874e+02_rb,0.34316e+02_rb /)
      kbo(:, 3,56,15) = (/ &
        & 0.40482e+02_rb,0.32823e+02_rb,0.30877e+02_rb,0.28999e+02_rb,0.33890e+02_rb /)
      kbo(:, 4,56,15) = (/ &
        & 0.39696e+02_rb,0.31783e+02_rb,0.29704e+02_rb,0.28093e+02_rb,0.33329e+02_rb /)
      kbo(:, 5,56,15) = (/ &
        & 0.38819e+02_rb,0.30747e+02_rb,0.28547e+02_rb,0.27216e+02_rb,0.32666e+02_rb /)
      kbo(:, 1,57,15) = (/ &
        & 0.41772e+02_rb,0.35313e+02_rb,0.33621e+02_rb,0.30980e+02_rb,0.34653e+02_rb /)
      kbo(:, 2,57,15) = (/ &
        & 0.41335e+02_rb,0.34223e+02_rb,0.32428e+02_rb,0.30152e+02_rb,0.34423e+02_rb /)
      kbo(:, 3,57,15) = (/ &
        & 0.40717e+02_rb,0.33158e+02_rb,0.31249e+02_rb,0.29283e+02_rb,0.34041e+02_rb /)
      kbo(:, 4,57,15) = (/ &
        & 0.39952e+02_rb,0.32113e+02_rb,0.30080e+02_rb,0.28382e+02_rb,0.33523e+02_rb /)
      kbo(:, 5,57,15) = (/ &
        & 0.39110e+02_rb,0.31075e+02_rb,0.28913e+02_rb,0.27492e+02_rb,0.32886e+02_rb /)
      kbo(:, 1,58,15) = (/ &
        & 0.41872e+02_rb,0.35648e+02_rb,0.33990e+02_rb,0.31215e+02_rb,0.34668e+02_rb /)
      kbo(:, 2,58,15) = (/ &
        & 0.41486e+02_rb,0.34551e+02_rb,0.32786e+02_rb,0.30408e+02_rb,0.34507e+02_rb /)
      kbo(:, 3,58,15) = (/ &
        & 0.40923e+02_rb,0.33481e+02_rb,0.31606e+02_rb,0.29553e+02_rb,0.34175e+02_rb /)
      kbo(:, 4,58,15) = (/ &
        & 0.40193e+02_rb,0.32431e+02_rb,0.30438e+02_rb,0.28657e+02_rb,0.33692e+02_rb /)
      kbo(:, 5,58,15) = (/ &
        & 0.39381e+02_rb,0.31393e+02_rb,0.29263e+02_rb,0.27758e+02_rb,0.33088e+02_rb /)
      kbo(:, 1,59,15) = (/ &
        & 0.41905e+02_rb,0.35780e+02_rb,0.34140e+02_rb,0.31306e+02_rb,0.34668e+02_rb /)
      kbo(:, 2,59,15) = (/ &
        & 0.41541e+02_rb,0.34683e+02_rb,0.32931e+02_rb,0.30509e+02_rb,0.34539e+02_rb /)
      kbo(:, 3,59,15) = (/ &
        & 0.41000e+02_rb,0.33610e+02_rb,0.31751e+02_rb,0.29658e+02_rb,0.34225e+02_rb /)
      kbo(:, 4,59,15) = (/ &
        & 0.40289e+02_rb,0.32559e+02_rb,0.30582e+02_rb,0.28769e+02_rb,0.33760e+02_rb /)
      kbo(:, 5,59,15) = (/ &
        & 0.39484e+02_rb,0.31518e+02_rb,0.29405e+02_rb,0.27867e+02_rb,0.33169e+02_rb /)
      kbo(:, 1,13,16) = (/ &
        & 0.44218e+02_rb,0.33168e+02_rb,0.27605e+02_rb,0.40761e+02_rb,0.54348e+02_rb /)
      kbo(:, 2,13,16) = (/ &
        & 0.43915e+02_rb,0.32941e+02_rb,0.26738e+02_rb,0.39342e+02_rb,0.52457e+02_rb /)
      kbo(:, 3,13,16) = (/ &
        & 0.43491e+02_rb,0.32624e+02_rb,0.25861e+02_rb,0.38022e+02_rb,0.50696e+02_rb /)
      kbo(:, 4,13,16) = (/ &
        & 0.42952e+02_rb,0.32220e+02_rb,0.25010e+02_rb,0.36713e+02_rb,0.48951e+02_rb /)
      kbo(:, 5,13,16) = (/ &
        & 0.42305e+02_rb,0.31735e+02_rb,0.24184e+02_rb,0.35417e+02_rb,0.47222e+02_rb /)
      kbo(:, 1,14,16) = (/ &
        & 0.50640e+02_rb,0.37984e+02_rb,0.31138e+02_rb,0.45571e+02_rb,0.60761e+02_rb /)
      kbo(:, 2,14,16) = (/ &
        & 0.50072e+02_rb,0.37558e+02_rb,0.30053e+02_rb,0.43809e+02_rb,0.58413e+02_rb /)
      kbo(:, 3,14,16) = (/ &
        & 0.49384e+02_rb,0.37043e+02_rb,0.28981e+02_rb,0.42162e+02_rb,0.56216e+02_rb /)
      kbo(:, 4,14,16) = (/ &
        & 0.48531e+02_rb,0.36404e+02_rb,0.27924e+02_rb,0.40555e+02_rb,0.54074e+02_rb /)
      kbo(:, 5,14,16) = (/ &
        & 0.47588e+02_rb,0.35696e+02_rb,0.26896e+02_rb,0.38961e+02_rb,0.51948e+02_rb /)
      kbo(:, 1,15,16) = (/ &
        & 0.57280e+02_rb,0.42964e+02_rb,0.34740e+02_rb,0.50323e+02_rb,0.67098e+02_rb /)
      kbo(:, 2,15,16) = (/ &
        & 0.56367e+02_rb,0.42280e+02_rb,0.33407e+02_rb,0.48191e+02_rb,0.64255e+02_rb /)
      kbo(:, 3,15,16) = (/ &
        & 0.55318e+02_rb,0.41493e+02_rb,0.32092e+02_rb,0.46141e+02_rb,0.61520e+02_rb /)
      kbo(:, 4,15,16) = (/ &
        & 0.54108e+02_rb,0.40586e+02_rb,0.30808e+02_rb,0.44198e+02_rb,0.58930e+02_rb /)
      kbo(:, 5,15,16) = (/ &
        & 0.52826e+02_rb,0.39625e+02_rb,0.29570e+02_rb,0.42303e+02_rb,0.56404e+02_rb /)
      kbo(:, 1,16,16) = (/ &
        & 0.63967e+02_rb,0.47979e+02_rb,0.38312e+02_rb,0.54909e+02_rb,0.73213e+02_rb /)
      kbo(:, 2,16,16) = (/ &
        & 0.62626e+02_rb,0.46973e+02_rb,0.36695e+02_rb,0.52345e+02_rb,0.69794e+02_rb /)
      kbo(:, 3,16,16) = (/ &
        & 0.61112e+02_rb,0.45838e+02_rb,0.35123e+02_rb,0.49904e+02_rb,0.66539e+02_rb /)
      kbo(:, 4,16,16) = (/ &
        & 0.59503e+02_rb,0.44632e+02_rb,0.33592e+02_rb,0.47568e+02_rb,0.63425e+02_rb /)
      kbo(:, 5,16,16) = (/ &
        & 0.57855e+02_rb,0.43396e+02_rb,0.32125e+02_rb,0.45372e+02_rb,0.60497e+02_rb /)
      kbo(:, 1,17,16) = (/ &
        & 0.70540e+02_rb,0.52908e+02_rb,0.41791e+02_rb,0.59184e+02_rb,0.78912e+02_rb /)
      kbo(:, 2,17,16) = (/ &
        & 0.68668e+02_rb,0.51504e+02_rb,0.39850e+02_rb,0.56189e+02_rb,0.74919e+02_rb /)
      kbo(:, 3,17,16) = (/ &
        & 0.66662e+02_rb,0.50000e+02_rb,0.37983e+02_rb,0.53339e+02_rb,0.71119e+02_rb /)
      kbo(:, 4,17,16) = (/ &
        & 0.64633e+02_rb,0.48479e+02_rb,0.36199e+02_rb,0.50655e+02_rb,0.67541e+02_rb /)
      kbo(:, 5,17,16) = (/ &
        & 0.62569e+02_rb,0.46932e+02_rb,0.34500e+02_rb,0.48144e+02_rb,0.64192e+02_rb /)
      kbo(:, 1,18,16) = (/ &
        & 0.76762e+02_rb,0.57574e+02_rb,0.45062e+02_rb,0.63079e+02_rb,0.84105e+02_rb /)
      kbo(:, 2,18,16) = (/ &
        & 0.74320e+02_rb,0.55743e+02_rb,0.42789e+02_rb,0.59653e+02_rb,0.79537e+02_rb /)
      kbo(:, 3,18,16) = (/ &
        & 0.71820e+02_rb,0.53869e+02_rb,0.40630e+02_rb,0.56414e+02_rb,0.75219e+02_rb /)
      kbo(:, 4,18,16) = (/ &
        & 0.69367e+02_rb,0.52029e+02_rb,0.38584e+02_rb,0.53391e+02_rb,0.71188e+02_rb /)
      kbo(:, 5,18,16) = (/ &
        & 0.66895e+02_rb,0.50176e+02_rb,0.36663e+02_rb,0.50596e+02_rb,0.67461e+02_rb /)
      kbo(:, 1,19,16) = (/ &
        & 0.82534e+02_rb,0.61903e+02_rb,0.48066e+02_rb,0.66533e+02_rb,0.88711e+02_rb /)
      kbo(:, 2,19,16) = (/ &
        & 0.79500e+02_rb,0.59627e+02_rb,0.45463e+02_rb,0.62698e+02_rb,0.83597e+02_rb /)
      kbo(:, 3,19,16) = (/ &
        & 0.76531e+02_rb,0.57402e+02_rb,0.43003e+02_rb,0.59107e+02_rb,0.78810e+02_rb /)
      kbo(:, 4,19,16) = (/ &
        & 0.73621e+02_rb,0.55219e+02_rb,0.40714e+02_rb,0.55776e+02_rb,0.74368e+02_rb /)
      kbo(:, 5,19,16) = (/ &
        & 0.70758e+02_rb,0.53072e+02_rb,0.38580e+02_rb,0.52720e+02_rb,0.70294e+02_rb /)
      kbo(:, 1,20,16) = (/ &
        & 0.87685e+02_rb,0.65766e+02_rb,0.50694e+02_rb,0.69445e+02_rb,0.92594e+02_rb /)
      kbo(:, 2,20,16) = (/ &
        & 0.84090e+02_rb,0.63070e+02_rb,0.47778e+02_rb,0.65259e+02_rb,0.87012e+02_rb /)
      kbo(:, 3,20,16) = (/ &
        & 0.80661e+02_rb,0.60499e+02_rb,0.45053e+02_rb,0.61350e+02_rb,0.81800e+02_rb /)
      kbo(:, 4,20,16) = (/ &
        & 0.77340e+02_rb,0.58008e+02_rb,0.42537e+02_rb,0.57756e+02_rb,0.77009e+02_rb /)
      kbo(:, 5,20,16) = (/ &
        & 0.74079e+02_rb,0.55564e+02_rb,0.40201e+02_rb,0.54475e+02_rb,0.72633e+02_rb /)
      kbo(:, 1,21,16) = (/ &
        & 0.92165e+02_rb,0.69126e+02_rb,0.52960e+02_rb,0.71872e+02_rb,0.95829e+02_rb /)
      kbo(:, 2,21,16) = (/ &
        & 0.88084e+02_rb,0.66065e+02_rb,0.49750e+02_rb,0.67363e+02_rb,0.89818e+02_rb /)
      kbo(:, 3,21,16) = (/ &
        & 0.84210e+02_rb,0.63159e+02_rb,0.46796e+02_rb,0.63194e+02_rb,0.84259e+02_rb /)
      kbo(:, 4,21,16) = (/ &
        & 0.80508e+02_rb,0.60384e+02_rb,0.44071e+02_rb,0.59388e+02_rb,0.79185e+02_rb /)
      kbo(:, 5,21,16) = (/ &
        & 0.76889e+02_rb,0.57670e+02_rb,0.41555e+02_rb,0.55904e+02_rb,0.74538e+02_rb /)
      kbo(:, 1,22,16) = (/ &
        & 0.95696e+02_rb,0.71774e+02_rb,0.54631e+02_rb,0.73526e+02_rb,0.98034e+02_rb /)
      kbo(:, 2,22,16) = (/ &
        & 0.91190e+02_rb,0.68395e+02_rb,0.51188e+02_rb,0.68768e+02_rb,0.91690e+02_rb /)
      kbo(:, 3,22,16) = (/ &
        & 0.86939e+02_rb,0.65206e+02_rb,0.48040e+02_rb,0.64409e+02_rb,0.85879e+02_rb /)
      kbo(:, 4,22,16) = (/ &
        & 0.82890e+02_rb,0.62171e+02_rb,0.45151e+02_rb,0.60441e+02_rb,0.80589e+02_rb /)
      kbo(:, 5,22,16) = (/ &
        & 0.78983e+02_rb,0.59241e+02_rb,0.42494e+02_rb,0.56811e+02_rb,0.75748e+02_rb /)
      kbo(:, 1,23,16) = (/ &
        & 0.98505e+02_rb,0.73881e+02_rb,0.55897e+02_rb,0.74690e+02_rb,0.99587e+02_rb /)
      kbo(:, 2,23,16) = (/ &
        & 0.93651e+02_rb,0.70240e+02_rb,0.52277e+02_rb,0.69755e+02_rb,0.93007e+02_rb /)
      kbo(:, 3,23,16) = (/ &
        & 0.89065e+02_rb,0.66801e+02_rb,0.48963e+02_rb,0.65256e+02_rb,0.87008e+02_rb /)
      kbo(:, 4,23,16) = (/ &
        & 0.84725e+02_rb,0.63546e+02_rb,0.45949e+02_rb,0.61155e+02_rb,0.81540e+02_rb /)
      kbo(:, 5,23,16) = (/ &
        & 0.80582e+02_rb,0.60440e+02_rb,0.43182e+02_rb,0.57417e+02_rb,0.76556e+02_rb /)
      kbo(:, 1,24,16) = (/ &
        & 0.10064e+03_rb,0.75484e+02_rb,0.56783e+02_rb,0.75422e+02_rb,0.10056e+03_rb /)
      kbo(:, 2,24,16) = (/ &
        & 0.95486e+02_rb,0.71616e+02_rb,0.53021e+02_rb,0.70355e+02_rb,0.93807e+02_rb /)
      kbo(:, 3,24,16) = (/ &
        & 0.90630e+02_rb,0.67975e+02_rb,0.49592e+02_rb,0.65751e+02_rb,0.87668e+02_rb /)
      kbo(:, 4,24,16) = (/ &
        & 0.86059e+02_rb,0.64548e+02_rb,0.46483e+02_rb,0.61565e+02_rb,0.82086e+02_rb /)
      kbo(:, 5,24,16) = (/ &
        & 0.81718e+02_rb,0.61292e+02_rb,0.43631e+02_rb,0.57753e+02_rb,0.77003e+02_rb /)
      kbo(:, 1,25,16) = (/ &
        & 0.10215e+03_rb,0.76618e+02_rb,0.57324e+02_rb,0.75759e+02_rb,0.10101e+03_rb /)
      kbo(:, 2,25,16) = (/ &
        & 0.96743e+02_rb,0.72560e+02_rb,0.53446e+02_rb,0.70607e+02_rb,0.94143e+02_rb /)
      kbo(:, 3,25,16) = (/ &
        & 0.91681e+02_rb,0.68763e+02_rb,0.49955e+02_rb,0.65940e+02_rb,0.87921e+02_rb /)
      kbo(:, 4,25,16) = (/ &
        & 0.86919e+02_rb,0.65192e+02_rb,0.46773e+02_rb,0.61703e+02_rb,0.82270e+02_rb /)
      kbo(:, 5,25,16) = (/ &
        & 0.82440e+02_rb,0.61834e+02_rb,0.43873e+02_rb,0.57842e+02_rb,0.77123e+02_rb /)
      kbo(:, 1,26,16) = (/ &
        & 0.10303e+03_rb,0.77274e+02_rb,0.57519e+02_rb,0.75701e+02_rb,0.10093e+03_rb /)
      kbo(:, 2,26,16) = (/ &
        & 0.97444e+02_rb,0.73085e+02_rb,0.53575e+02_rb,0.70525e+02_rb,0.94033e+02_rb /)
      kbo(:, 3,26,16) = (/ &
        & 0.92222e+02_rb,0.69170e+02_rb,0.50048e+02_rb,0.65828e+02_rb,0.87771e+02_rb /)
      kbo(:, 4,26,16) = (/ &
        & 0.87336e+02_rb,0.65505e+02_rb,0.46824e+02_rb,0.61566e+02_rb,0.82088e+02_rb /)
      kbo(:, 5,26,16) = (/ &
        & 0.82771e+02_rb,0.62082e+02_rb,0.43909e+02_rb,0.57710e+02_rb,0.76947e+02_rb /)
      kbo(:, 1,27,16) = (/ &
        & 0.10345e+03_rb,0.77590e+02_rb,0.57467e+02_rb,0.75392e+02_rb,0.10052e+03_rb /)
      kbo(:, 2,27,16) = (/ &
        & 0.97739e+02_rb,0.73306e+02_rb,0.53511e+02_rb,0.70214e+02_rb,0.93619e+02_rb /)
      kbo(:, 3,27,16) = (/ &
        & 0.92406e+02_rb,0.69307e+02_rb,0.49956e+02_rb,0.65525e+02_rb,0.87366e+02_rb /)
      kbo(:, 4,27,16) = (/ &
        & 0.87439e+02_rb,0.65583e+02_rb,0.46715e+02_rb,0.61255e+02_rb,0.81674e+02_rb /)
      kbo(:, 5,27,16) = (/ &
        & 0.82826e+02_rb,0.62123e+02_rb,0.43807e+02_rb,0.57432e+02_rb,0.76576e+02_rb /)
      kbo(:, 1,28,16) = (/ &
        & 0.10350e+03_rb,0.77625e+02_rb,0.57222e+02_rb,0.74881e+02_rb,0.99841e+02_rb /)
      kbo(:, 2,28,16) = (/ &
        & 0.97691e+02_rb,0.73271e+02_rb,0.53275e+02_rb,0.69730e+02_rb,0.92974e+02_rb /)
      kbo(:, 3,28,16) = (/ &
        & 0.92289e+02_rb,0.69219e+02_rb,0.49719e+02_rb,0.65057e+02_rb,0.86743e+02_rb /)
      kbo(:, 4,28,16) = (/ &
        & 0.87284e+02_rb,0.65466e+02_rb,0.46485e+02_rb,0.60817e+02_rb,0.81090e+02_rb /)
      kbo(:, 5,28,16) = (/ &
        & 0.82640e+02_rb,0.61984e+02_rb,0.43588e+02_rb,0.57042e+02_rb,0.76055e+02_rb /)
      kbo(:, 1,29,16) = (/ &
        & 0.10319e+03_rb,0.77394e+02_rb,0.56799e+02_rb,0.74179e+02_rb,0.98906e+02_rb /)
      kbo(:, 2,29,16) = (/ &
        & 0.97340e+02_rb,0.73008e+02_rb,0.52885e+02_rb,0.69064e+02_rb,0.92086e+02_rb /)
      kbo(:, 3,29,16) = (/ &
        & 0.91909e+02_rb,0.68935e+02_rb,0.49342e+02_rb,0.64439e+02_rb,0.85918e+02_rb /)
      kbo(:, 4,29,16) = (/ &
        & 0.86890e+02_rb,0.65171e+02_rb,0.46144e+02_rb,0.60253e+02_rb,0.80337e+02_rb /)
      kbo(:, 5,29,16) = (/ &
        & 0.82249e+02_rb,0.61690e+02_rb,0.43266e+02_rb,0.56539e+02_rb,0.75385e+02_rb /)
      kbo(:, 1,30,16) = (/ &
        & 0.10264e+03_rb,0.76981e+02_rb,0.56267e+02_rb,0.73352e+02_rb,0.97803e+02_rb /)
      kbo(:, 2,30,16) = (/ &
        & 0.96761e+02_rb,0.72573e+02_rb,0.52390e+02_rb,0.68302e+02_rb,0.91070e+02_rb /)
      kbo(:, 3,30,16) = (/ &
        & 0.91349e+02_rb,0.68515e+02_rb,0.48877e+02_rb,0.63734e+02_rb,0.84978e+02_rb /)
      kbo(:, 4,30,16) = (/ &
        & 0.86346e+02_rb,0.64763e+02_rb,0.45722e+02_rb,0.59615e+02_rb,0.79487e+02_rb /)
      kbo(:, 5,30,16) = (/ &
        & 0.81701e+02_rb,0.61280e+02_rb,0.42874e+02_rb,0.55973e+02_rb,0.74631e+02_rb /)
      kbo(:, 1,31,16) = (/ &
        & 0.10185e+03_rb,0.76389e+02_rb,0.55624e+02_rb,0.72410e+02_rb,0.96547e+02_rb /)
      kbo(:, 2,31,16) = (/ &
        & 0.95999e+02_rb,0.72002e+02_rb,0.51796e+02_rb,0.67443e+02_rb,0.89924e+02_rb /)
      kbo(:, 3,31,16) = (/ &
        & 0.90614e+02_rb,0.67963e+02_rb,0.48327e+02_rb,0.62938e+02_rb,0.83917e+02_rb /)
      kbo(:, 4,31,16) = (/ &
        & 0.85645e+02_rb,0.64237e+02_rb,0.45228e+02_rb,0.58906e+02_rb,0.78542e+02_rb /)
      kbo(:, 5,31,16) = (/ &
        & 0.81007e+02_rb,0.60759e+02_rb,0.42414e+02_rb,0.55328e+02_rb,0.73771e+02_rb /)
      kbo(:, 1,32,16) = (/ &
        & 0.10089e+03_rb,0.75672e+02_rb,0.54904e+02_rb,0.71387e+02_rb,0.95183e+02_rb /)
      kbo(:, 2,32,16) = (/ &
        & 0.95077e+02_rb,0.71310e+02_rb,0.51129e+02_rb,0.66501e+02_rb,0.88668e+02_rb /)
      kbo(:, 3,32,16) = (/ &
        & 0.89743e+02_rb,0.67310e+02_rb,0.47720e+02_rb,0.62089e+02_rb,0.82785e+02_rb /)
      kbo(:, 4,32,16) = (/ &
        & 0.84830e+02_rb,0.63626e+02_rb,0.44676e+02_rb,0.58147e+02_rb,0.77529e+02_rb /)
      kbo(:, 5,32,16) = (/ &
        & 0.80208e+02_rb,0.60160e+02_rb,0.41909e+02_rb,0.54638e+02_rb,0.72851e+02_rb /)
      kbo(:, 1,33,16) = (/ &
        & 0.99806e+02_rb,0.74857e+02_rb,0.54124e+02_rb,0.70307e+02_rb,0.93742e+02_rb /)
      kbo(:, 2,33,16) = (/ &
        & 0.94060e+02_rb,0.70548e+02_rb,0.50418e+02_rb,0.65525e+02_rb,0.87366e+02_rb /)
      kbo(:, 3,33,16) = (/ &
        & 0.88780e+02_rb,0.66588e+02_rb,0.47080e+02_rb,0.61195e+02_rb,0.81594e+02_rb /)
      kbo(:, 4,33,16) = (/ &
        & 0.83926e+02_rb,0.62948e+02_rb,0.44091e+02_rb,0.57369e+02_rb,0.76493e+02_rb /)
      kbo(:, 5,33,16) = (/ &
        & 0.79342e+02_rb,0.59511e+02_rb,0.41372e+02_rb,0.53910e+02_rb,0.71880e+02_rb /)
      kbo(:, 1,34,16) = (/ &
        & 0.98842e+02_rb,0.74134e+02_rb,0.53446e+02_rb,0.69373e+02_rb,0.92498e+02_rb /)
      kbo(:, 2,34,16) = (/ &
        & 0.93169e+02_rb,0.69879e+02_rb,0.49796e+02_rb,0.64672e+02_rb,0.86229e+02_rb /)
      kbo(:, 3,34,16) = (/ &
        & 0.87940e+02_rb,0.65958e+02_rb,0.46525e+02_rb,0.60437e+02_rb,0.80583e+02_rb /)
      kbo(:, 4,34,16) = (/ &
        & 0.83124e+02_rb,0.62347e+02_rb,0.43581e+02_rb,0.56689e+02_rb,0.75585e+02_rb /)
      kbo(:, 5,34,16) = (/ &
        & 0.78566e+02_rb,0.58929e+02_rb,0.40900e+02_rb,0.53287e+02_rb,0.71049e+02_rb /)
      kbo(:, 1,35,16) = (/ &
        & 0.98383e+02_rb,0.73790e+02_rb,0.53104e+02_rb,0.68893e+02_rb,0.91857e+02_rb /)
      kbo(:, 2,35,16) = (/ &
        & 0.92731e+02_rb,0.69551e+02_rb,0.49484e+02_rb,0.64233e+02_rb,0.85643e+02_rb /)
      kbo(:, 3,35,16) = (/ &
        & 0.87537e+02_rb,0.65656e+02_rb,0.46247e+02_rb,0.60050e+02_rb,0.80067e+02_rb /)
      kbo(:, 4,35,16) = (/ &
        & 0.82730e+02_rb,0.62051e+02_rb,0.43325e+02_rb,0.56336e+02_rb,0.75115e+02_rb /)
      kbo(:, 5,35,16) = (/ &
        & 0.78184e+02_rb,0.58642e+02_rb,0.40664e+02_rb,0.52959e+02_rb,0.70611e+02_rb /)
      kbo(:, 1,36,16) = (/ &
        & 0.98510e+02_rb,0.73885e+02_rb,0.53153e+02_rb,0.68921e+02_rb,0.91895e+02_rb /)
      kbo(:, 2,36,16) = (/ &
        & 0.92845e+02_rb,0.69637e+02_rb,0.49525e+02_rb,0.64248e+02_rb,0.85664e+02_rb /)
      kbo(:, 3,36,16) = (/ &
        & 0.87626e+02_rb,0.65723e+02_rb,0.46281e+02_rb,0.60067e+02_rb,0.80090e+02_rb /)
      kbo(:, 4,36,16) = (/ &
        & 0.82805e+02_rb,0.62108e+02_rb,0.43352e+02_rb,0.56351e+02_rb,0.75135e+02_rb /)
      kbo(:, 5,36,16) = (/ &
        & 0.78246e+02_rb,0.58689e+02_rb,0.40687e+02_rb,0.52970e+02_rb,0.70627e+02_rb /)
      kbo(:, 1,37,16) = (/ &
        & 0.99499e+02_rb,0.74626e+02_rb,0.53765e+02_rb,0.69681e+02_rb,0.92907e+02_rb /)
      kbo(:, 2,37,16) = (/ &
        & 0.93737e+02_rb,0.70305e+02_rb,0.50072e+02_rb,0.64938e+02_rb,0.86584e+02_rb /)
      kbo(:, 3,37,16) = (/ &
        & 0.88443e+02_rb,0.66336e+02_rb,0.46765e+02_rb,0.60669e+02_rb,0.80892e+02_rb /)
      kbo(:, 4,37,16) = (/ &
        & 0.83569e+02_rb,0.62680e+02_rb,0.43792e+02_rb,0.56887e+02_rb,0.75849e+02_rb /)
      kbo(:, 5,37,16) = (/ &
        & 0.78959e+02_rb,0.59223e+02_rb,0.41087e+02_rb,0.53464e+02_rb,0.71286e+02_rb /)
      kbo(:, 1,38,16) = (/ &
        & 0.10053e+03_rb,0.75401e+02_rb,0.54401e+02_rb,0.70469e+02_rb,0.93958e+02_rb /)
      kbo(:, 2,38,16) = (/ &
        & 0.94665e+02_rb,0.71001e+02_rb,0.50643e+02_rb,0.65651e+02_rb,0.87535e+02_rb /)
      kbo(:, 3,38,16) = (/ &
        & 0.89287e+02_rb,0.66968e+02_rb,0.47272e+02_rb,0.61298e+02_rb,0.81730e+02_rb /)
      kbo(:, 4,38,16) = (/ &
        & 0.84354e+02_rb,0.63269e+02_rb,0.44250e+02_rb,0.57455e+02_rb,0.76607e+02_rb /)
      kbo(:, 5,38,16) = (/ &
        & 0.79688e+02_rb,0.59770e+02_rb,0.41502e+02_rb,0.53978e+02_rb,0.71970e+02_rb /)
      kbo(:, 1,39,16) = (/ &
        & 0.10152e+03_rb,0.76145e+02_rb,0.55030e+02_rb,0.71266e+02_rb,0.95022e+02_rb /)
      kbo(:, 2,39,16) = (/ &
        & 0.95574e+02_rb,0.71683e+02_rb,0.51208e+02_rb,0.66359e+02_rb,0.88479e+02_rb /)
      kbo(:, 3,39,16) = (/ &
        & 0.90127e+02_rb,0.67598e+02_rb,0.47776e+02_rb,0.61934e+02_rb,0.82579e+02_rb /)
      kbo(:, 4,39,16) = (/ &
        & 0.85122e+02_rb,0.63845e+02_rb,0.44704e+02_rb,0.58015e+02_rb,0.77353e+02_rb /)
      kbo(:, 5,39,16) = (/ &
        & 0.80410e+02_rb,0.60311e+02_rb,0.41916e+02_rb,0.54486e+02_rb,0.72649e+02_rb /)
      kbo(:, 1,40,16) = (/ &
        & 0.10316e+03_rb,0.77371e+02_rb,0.56077e+02_rb,0.72600e+02_rb,0.96800e+02_rb /)
      kbo(:, 2,40,16) = (/ &
        & 0.97060e+02_rb,0.72797e+02_rb,0.52149e+02_rb,0.67558e+02_rb,0.90077e+02_rb /)
      kbo(:, 3,40,16) = (/ &
        & 0.91509e+02_rb,0.68635e+02_rb,0.48618e+02_rb,0.63010e+02_rb,0.84013e+02_rb /)
      kbo(:, 4,40,16) = (/ &
        & 0.86384e+02_rb,0.64791e+02_rb,0.45460e+02_rb,0.58959e+02_rb,0.78612e+02_rb /)
      kbo(:, 5,40,16) = (/ &
        & 0.81600e+02_rb,0.61204e+02_rb,0.42605e+02_rb,0.55349e+02_rb,0.73798e+02_rb /)
      kbo(:, 1,41,16) = (/ &
        & 0.10490e+03_rb,0.78677e+02_rb,0.57202e+02_rb,0.74030e+02_rb,0.98707e+02_rb /)
      kbo(:, 2,41,16) = (/ &
        & 0.98655e+02_rb,0.73994e+02_rb,0.53155e+02_rb,0.68836e+02_rb,0.91782e+02_rb /)
      kbo(:, 3,41,16) = (/ &
        & 0.92964e+02_rb,0.69726e+02_rb,0.49517e+02_rb,0.64156e+02_rb,0.85541e+02_rb /)
      kbo(:, 4,41,16) = (/ &
        & 0.87720e+02_rb,0.65793e+02_rb,0.46275e+02_rb,0.59984e+02_rb,0.79978e+02_rb /)
      kbo(:, 5,41,16) = (/ &
        & 0.82858e+02_rb,0.62147e+02_rb,0.43340e+02_rb,0.56275e+02_rb,0.75033e+02_rb /)
      kbo(:, 1,42,16) = (/ &
        & 0.10666e+03_rb,0.79997e+02_rb,0.58340e+02_rb,0.75485e+02_rb,0.10065e+03_rb /)
      kbo(:, 2,42,16) = (/ &
        & 0.10028e+03_rb,0.75211e+02_rb,0.54189e+02_rb,0.70153e+02_rb,0.93537e+02_rb /)
      kbo(:, 3,42,16) = (/ &
        & 0.94432e+02_rb,0.70827e+02_rb,0.50443e+02_rb,0.65340e+02_rb,0.87121e+02_rb /)
      kbo(:, 4,42,16) = (/ &
        & 0.89075e+02_rb,0.66809e+02_rb,0.47102e+02_rb,0.61032e+02_rb,0.81376e+02_rb /)
      kbo(:, 5,42,16) = (/ &
        & 0.84125e+02_rb,0.63097e+02_rb,0.44086e+02_rb,0.57211e+02_rb,0.76281e+02_rb /)
      kbo(:, 1,43,16) = (/ &
        & 0.10883e+03_rb,0.81624e+02_rb,0.59771e+02_rb,0.77309e+02_rb,0.10308e+03_rb /)
      kbo(:, 2,43,16) = (/ &
        & 0.10228e+03_rb,0.76711e+02_rb,0.55476e+02_rb,0.71797e+02_rb,0.95729e+02_rb /)
      kbo(:, 3,43,16) = (/ &
        & 0.96257e+02_rb,0.72196e+02_rb,0.51599e+02_rb,0.66824e+02_rb,0.89098e+02_rb /)
      kbo(:, 4,43,16) = (/ &
        & 0.90745e+02_rb,0.68062e+02_rb,0.48126e+02_rb,0.62350e+02_rb,0.83134e+02_rb /)
      kbo(:, 5,43,16) = (/ &
        & 0.85682e+02_rb,0.64265e+02_rb,0.45017e+02_rb,0.58380e+02_rb,0.77840e+02_rb /)
      kbo(:, 1,44,16) = (/ &
        & 0.11122e+03_rb,0.83416e+02_rb,0.61378e+02_rb,0.79350e+02_rb,0.10580e+03_rb /)
      kbo(:, 2,44,16) = (/ &
        & 0.10448e+03_rb,0.78362e+02_rb,0.56905e+02_rb,0.73629e+02_rb,0.98171e+02_rb /)
      kbo(:, 3,44,16) = (/ &
        & 0.98280e+02_rb,0.73712e+02_rb,0.52886e+02_rb,0.68474e+02_rb,0.91299e+02_rb /)
      kbo(:, 4,44,16) = (/ &
        & 0.92614e+02_rb,0.69464e+02_rb,0.49277e+02_rb,0.63819e+02_rb,0.85092e+02_rb /)
      kbo(:, 5,44,16) = (/ &
        & 0.87390e+02_rb,0.65546e+02_rb,0.46054e+02_rb,0.59688e+02_rb,0.79585e+02_rb /)
      kbo(:, 1,45,16) = (/ &
        & 0.11367e+03_rb,0.85251e+02_rb,0.63043e+02_rb,0.81458e+02_rb,0.10861e+03_rb /)
      kbo(:, 2,45,16) = (/ &
        & 0.10674e+03_rb,0.80056e+02_rb,0.58377e+02_rb,0.75516e+02_rb,0.10069e+03_rb /)
      kbo(:, 3,45,16) = (/ &
        & 0.10036e+03_rb,0.75271e+02_rb,0.54217e+02_rb,0.70172e+02_rb,0.93562e+02_rb /)
      kbo(:, 4,45,16) = (/ &
        & 0.94507e+02_rb,0.70883e+02_rb,0.50472e+02_rb,0.65352e+02_rb,0.87137e+02_rb /)
      kbo(:, 5,45,16) = (/ &
        & 0.89124e+02_rb,0.66846e+02_rb,0.47119e+02_rb,0.61044e+02_rb,0.81393e+02_rb /)
      kbo(:, 1,46,16) = (/ &
        & 0.11634e+03_rb,0.87256e+02_rb,0.64886e+02_rb,0.83772e+02_rb,0.11170e+03_rb /)
      kbo(:, 2,46,16) = (/ &
        & 0.10921e+03_rb,0.81909e+02_rb,0.60009e+02_rb,0.77600e+02_rb,0.10347e+03_rb /)
      kbo(:, 3,46,16) = (/ &
        & 0.10262e+03_rb,0.76969e+02_rb,0.55682e+02_rb,0.72046e+02_rb,0.96062e+02_rb /)
      kbo(:, 4,46,16) = (/ &
        & 0.96578e+02_rb,0.72436e+02_rb,0.51790e+02_rb,0.67050e+02_rb,0.89399e+02_rb /)
      kbo(:, 5,46,16) = (/ &
        & 0.91036e+02_rb,0.68280e+02_rb,0.48293e+02_rb,0.62556e+02_rb,0.83408e+02_rb /)
      kbo(:, 1,47,16) = (/ &
        & 0.11944e+03_rb,0.89580e+02_rb,0.67039e+02_rb,0.86470e+02_rb,0.11529e+03_rb /)
      kbo(:, 2,47,16) = (/ &
        & 0.11203e+03_rb,0.84025e+02_rb,0.61916e+02_rb,0.80032e+02_rb,0.10671e+03_rb /)
      kbo(:, 3,47,16) = (/ &
        & 0.10523e+03_rb,0.78928e+02_rb,0.57378e+02_rb,0.74229e+02_rb,0.98972e+02_rb /)
      kbo(:, 4,47,16) = (/ &
        & 0.98975e+02_rb,0.74234e+02_rb,0.53317e+02_rb,0.69003e+02_rb,0.92005e+02_rb /)
      kbo(:, 5,47,16) = (/ &
        & 0.93247e+02_rb,0.69938e+02_rb,0.49666e+02_rb,0.64309e+02_rb,0.85745e+02_rb /)
      kbo(:, 1,48,16) = (/ &
        & 0.12267e+03_rb,0.92006e+02_rb,0.69290e+02_rb,0.89280e+02_rb,0.11904e+03_rb /)
      kbo(:, 2,48,16) = (/ &
        & 0.11495e+03_rb,0.86215e+02_rb,0.63917e+02_rb,0.82556e+02_rb,0.11008e+03_rb /)
      kbo(:, 3,48,16) = (/ &
        & 0.10793e+03_rb,0.80951e+02_rb,0.59146e+02_rb,0.76493e+02_rb,0.10199e+03_rb /)
      kbo(:, 4,48,16) = (/ &
        & 0.10147e+03_rb,0.76101e+02_rb,0.54918e+02_rb,0.71050e+02_rb,0.94733e+02_rb /)
      kbo(:, 5,48,16) = (/ &
        & 0.95498e+02_rb,0.71626e+02_rb,0.51095e+02_rb,0.66155e+02_rb,0.88207e+02_rb /)
      kbo(:, 1,49,16) = (/ &
        & 0.12603e+03_rb,0.94521e+02_rb,0.71680e+02_rb,0.92236e+02_rb,0.12298e+03_rb /)
      kbo(:, 2,49,16) = (/ &
        & 0.11801e+03_rb,0.88509e+02_rb,0.66027e+02_rb,0.85192e+02_rb,0.11359e+03_rb /)
      kbo(:, 3,49,16) = (/ &
        & 0.11070e+03_rb,0.83029e+02_rb,0.61003e+02_rb,0.78868e+02_rb,0.10516e+03_rb /)
      kbo(:, 4,49,16) = (/ &
        & 0.10402e+03_rb,0.78016e+02_rb,0.56578e+02_rb,0.73185e+02_rb,0.97580e+02_rb /)
      kbo(:, 5,49,16) = (/ &
        & 0.97847e+02_rb,0.73387e+02_rb,0.52593e+02_rb,0.68075e+02_rb,0.90767e+02_rb /)
      kbo(:, 1,50,16) = (/ &
        & 0.12933e+03_rb,0.96996e+02_rb,0.74078e+02_rb,0.95132e+02_rb,0.12684e+03_rb /)
      kbo(:, 2,50,16) = (/ &
        & 0.12097e+03_rb,0.90726e+02_rb,0.68085e+02_rb,0.87768e+02_rb,0.11702e+03_rb /)
      kbo(:, 3,50,16) = (/ &
        & 0.11343e+03_rb,0.85073e+02_rb,0.62849e+02_rb,0.81185e+02_rb,0.10825e+03_rb /)
      kbo(:, 4,50,16) = (/ &
        & 0.10650e+03_rb,0.79879e+02_rb,0.58193e+02_rb,0.75257e+02_rb,0.10034e+03_rb /)
      kbo(:, 5,50,16) = (/ &
        & 0.10012e+03_rb,0.75094e+02_rb,0.54050e+02_rb,0.69940e+02_rb,0.93253e+02_rb /)
      kbo(:, 1,51,16) = (/ &
        & 0.13262e+03_rb,0.99463e+02_rb,0.76537e+02_rb,0.98089e+02_rb,0.13079e+03_rb /)
      kbo(:, 2,51,16) = (/ &
        & 0.12396e+03_rb,0.92972e+02_rb,0.70180e+02_rb,0.90371e+02_rb,0.12049e+03_rb /)
      kbo(:, 3,51,16) = (/ &
        & 0.11611e+03_rb,0.87085e+02_rb,0.64700e+02_rb,0.83525e+02_rb,0.11137e+03_rb /)
      kbo(:, 4,51,16) = (/ &
        & 0.10897e+03_rb,0.81728e+02_rb,0.59829e+02_rb,0.77356e+02_rb,0.10314e+03_rb /)
      kbo(:, 5,51,16) = (/ &
        & 0.10241e+03_rb,0.76807e+02_rb,0.55526e+02_rb,0.71828e+02_rb,0.95771e+02_rb /)
      kbo(:, 1,52,16) = (/ &
        & 0.13607e+03_rb,0.10205e+03_rb,0.79117e+02_rb,0.10117e+03_rb,0.13489e+03_rb /)
      kbo(:, 2,52,16) = (/ &
        & 0.12703e+03_rb,0.95276e+02_rb,0.72382e+02_rb,0.93082e+02_rb,0.12411e+03_rb /)
      kbo(:, 3,52,16) = (/ &
        & 0.11888e+03_rb,0.89162e+02_rb,0.66631e+02_rb,0.85943e+02_rb,0.11459e+03_rb /)
      kbo(:, 4,52,16) = (/ &
        & 0.11155e+03_rb,0.83666e+02_rb,0.61557e+02_rb,0.79542e+02_rb,0.10606e+03_rb /)
      kbo(:, 5,52,16) = (/ &
        & 0.10475e+03_rb,0.78563e+02_rb,0.57053e+02_rb,0.73794e+02_rb,0.98392e+02_rb /)
      kbo(:, 1,53,16) = (/ &
        & 0.13969e+03_rb,0.10477e+03_rb,0.81852e+02_rb,0.10445e+03_rb,0.13926e+03_rb /)
      kbo(:, 2,53,16) = (/ &
        & 0.13021e+03_rb,0.97660e+02_rb,0.74731e+02_rb,0.95923e+02_rb,0.12790e+03_rb /)
      kbo(:, 3,53,16) = (/ &
        & 0.12178e+03_rb,0.91333e+02_rb,0.68644e+02_rb,0.88461e+02_rb,0.11795e+03_rb /)
      kbo(:, 4,53,16) = (/ &
        & 0.11417e+03_rb,0.85628e+02_rb,0.63346e+02_rb,0.81812e+02_rb,0.10908e+03_rb /)
      kbo(:, 5,53,16) = (/ &
        & 0.10716e+03_rb,0.80376e+02_rb,0.58630e+02_rb,0.75816e+02_rb,0.10109e+03_rb /)
      kbo(:, 1,54,16) = (/ &
        & 0.14312e+03_rb,0.10734e+03_rb,0.84505e+02_rb,0.10757e+03_rb,0.14343e+03_rb /)
      kbo(:, 2,54,16) = (/ &
        & 0.13326e+03_rb,0.99943e+02_rb,0.77008e+02_rb,0.98653e+02_rb,0.13154e+03_rb /)
      kbo(:, 3,54,16) = (/ &
        & 0.12453e+03_rb,0.93400e+02_rb,0.70583e+02_rb,0.90867e+02_rb,0.12116e+03_rb /)
      kbo(:, 4,54,16) = (/ &
        & 0.11663e+03_rb,0.87472e+02_rb,0.65055e+02_rb,0.83971e+02_rb,0.11196e+03_rb /)
      kbo(:, 5,54,16) = (/ &
        & 0.10944e+03_rb,0.82085e+02_rb,0.60143e+02_rb,0.77761e+02_rb,0.10368e+03_rb /)
      kbo(:, 1,55,16) = (/ &
        & 0.14652e+03_rb,0.10989e+03_rb,0.87185e+02_rb,0.11072e+03_rb,0.14762e+03_rb /)
      kbo(:, 2,55,16) = (/ &
        & 0.13631e+03_rb,0.10223e+03_rb,0.79292e+02_rb,0.10139e+03_rb,0.13518e+03_rb /)
      kbo(:, 3,55,16) = (/ &
        & 0.12725e+03_rb,0.95436e+02_rb,0.72534e+02_rb,0.93260e+02_rb,0.12435e+03_rb /)
      kbo(:, 4,55,16) = (/ &
        & 0.11908e+03_rb,0.89315e+02_rb,0.66761e+02_rb,0.86100e+02_rb,0.11480e+03_rb /)
      kbo(:, 5,55,16) = (/ &
        & 0.11172e+03_rb,0.83789e+02_rb,0.61666e+02_rb,0.79683e+02_rb,0.10624e+03_rb /)
      kbo(:, 1,56,16) = (/ &
        & 0.15005e+03_rb,0.11254e+03_rb,0.89964e+02_rb,0.11399e+03_rb,0.15199e+03_rb /)
      kbo(:, 2,56,16) = (/ &
        & 0.13946e+03_rb,0.10459e+03_rb,0.81673e+02_rb,0.10424e+03_rb,0.13899e+03_rb /)
      kbo(:, 3,56,16) = (/ &
        & 0.13002e+03_rb,0.97512e+02_rb,0.74583e+02_rb,0.95730e+02_rb,0.12764e+03_rb /)
      kbo(:, 4,56,16) = (/ &
        & 0.12160e+03_rb,0.91205e+02_rb,0.68519e+02_rb,0.88312e+02_rb,0.11775e+03_rb /)
      kbo(:, 5,56,16) = (/ &
        & 0.11401e+03_rb,0.85509e+02_rb,0.63233e+02_rb,0.81665e+02_rb,0.10889e+03_rb /)
      kbo(:, 1,57,16) = (/ &
        & 0.15367e+03_rb,0.11525e+03_rb,0.92893e+02_rb,0.11745e+03_rb,0.15659e+03_rb /)
      kbo(:, 2,57,16) = (/ &
        & 0.14273e+03_rb,0.10705e+03_rb,0.84201e+02_rb,0.10722e+03_rb,0.14296e+03_rb /)
      kbo(:, 3,57,16) = (/ &
        & 0.13291e+03_rb,0.99681e+02_rb,0.76750e+02_rb,0.98336e+02_rb,0.13111e+03_rb /)
      kbo(:, 4,57,16) = (/ &
        & 0.12423e+03_rb,0.93171e+02_rb,0.70357e+02_rb,0.90589e+02_rb,0.12079e+03_rb /)
      kbo(:, 5,57,16) = (/ &
        & 0.11636e+03_rb,0.87269e+02_rb,0.64859e+02_rb,0.83719e+02_rb,0.11162e+03_rb /)
      kbo(:, 1,58,16) = (/ &
        & 0.15720e+03_rb,0.11790e+03_rb,0.95809e+02_rb,0.12090e+03_rb,0.16121e+03_rb /)
      kbo(:, 2,58,16) = (/ &
        & 0.14594e+03_rb,0.10946e+03_rb,0.86713e+02_rb,0.11017e+03_rb,0.14690e+03_rb /)
      kbo(:, 3,58,16) = (/ &
        & 0.13577e+03_rb,0.10183e+03_rb,0.78898e+02_rb,0.10090e+03_rb,0.13454e+03_rb /)
      kbo(:, 4,58,16) = (/ &
        & 0.12678e+03_rb,0.95087e+02_rb,0.72188e+02_rb,0.92844e+02_rb,0.12379e+03_rb /)
      kbo(:, 5,58,16) = (/ &
        & 0.11865e+03_rb,0.88989e+02_rb,0.66460e+02_rb,0.85728e+02_rb,0.11430e+03_rb /)
      kbo(:, 1,59,16) = (/ &
        & 0.15867e+03_rb,0.11900e+03_rb,0.97023e+02_rb,0.12235e+03_rb,0.16313e+03_rb /)
      kbo(:, 2,59,16) = (/ &
        & 0.14725e+03_rb,0.11044e+03_rb,0.87747e+02_rb,0.11139e+03_rb,0.14852e+03_rb /)
      kbo(:, 3,59,16) = (/ &
        & 0.13696e+03_rb,0.10272e+03_rb,0.79780e+02_rb,0.10196e+03_rb,0.13595e+03_rb /)
      kbo(:, 4,59,16) = (/ &
        & 0.12782e+03_rb,0.95865e+02_rb,0.72946e+02_rb,0.93765e+02_rb,0.12502e+03_rb /)
      kbo(:, 5,59,16) = (/ &
        & 0.11961e+03_rb,0.89706e+02_rb,0.67116e+02_rb,0.86544e+02_rb,0.11539e+03_rb /)

!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.110008e-06_rb, 0.630912e-06_rb, 0.363159e-05_rb, 0.616892e-05_rb /)
      forrefo(:, 2) = (/ 0.429709e-05_rb, 0.789174e-05_rb, 0.217416e-04_rb, 0.639393e-04_rb /)
      forrefo(:, 3) = (/ 0.436283e-04_rb, 0.526247e-04_rb, 0.116341e-03_rb, 0.205616e-03_rb /)
      forrefo(:, 4) = (/ 0.215627e-03_rb, 0.234522e-03_rb, 0.280497e-03_rb, 0.838668e-03_rb /)
      forrefo(:, 5) = (/ 0.529283e-03_rb, 0.620848e-03_rb, 0.935561e-03_rb, 0.171252e-02_rb /)
      forrefo(:, 6) = (/ 0.212267e-02_rb, 0.218564e-02_rb, 0.222227e-02_rb, 0.199650e-02_rb /)
      forrefo(:, 7) = (/ 0.291120e-02_rb, 0.281168e-02_rb, 0.259543e-02_rb, 0.210159e-02_rb /)
      forrefo(:, 8) = (/ 0.316249e-02_rb, 0.310695e-02_rb, 0.279501e-02_rb, 0.208076e-02_rb /)
      forrefo(:, 9) = (/ 0.354993e-02_rb, 0.336989e-02_rb, 0.298930e-02_rb, 0.180424e-02_rb /)
      forrefo(:,10) = (/ 0.397729e-02_rb, 0.367409e-02_rb, 0.328982e-02_rb, 0.177807e-02_rb /)
      forrefo(:,11) = (/ 0.408831e-02_rb, 0.398792e-02_rb, 0.352727e-02_rb, 0.192470e-02_rb /)
      forrefo(:,12) = (/ 0.433926e-02_rb, 0.420667e-02_rb, 0.383894e-02_rb, 0.220836e-02_rb /)
      forrefo(:,13) = (/ 0.436397e-02_rb, 0.433769e-02_rb, 0.425752e-02_rb, 0.237343e-02_rb /)
      forrefo(:,14) = (/ 0.440525e-02_rb, 0.449018e-02_rb, 0.451881e-02_rb, 0.269169e-02_rb /)
      forrefo(:,15) = (/ 0.491350e-02_rb, 0.481760e-02_rb, 0.475799e-02_rb, 0.362666e-02_rb /)
      forrefo(:,16) = (/ 0.561641e-02_rb, 0.524553e-02_rb, 0.512473e-02_rb, 0.493802e-02_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
        &  0.115887e-03_rb, 0.926537e-04_rb, 0.740783e-04_rb, 0.592270e-04_rb, 0.473530e-04_rb, &
        &  0.378596e-04_rb, 0.302694e-04_rb, 0.242010e-04_rb, 0.193491e-04_rb, 0.154700e-04_rb /)
      selfrefo(:, 2) = (/ &
        &  0.459557e-03_rb, 0.381962e-03_rb, 0.317469e-03_rb, 0.263866e-03_rb, 0.219313e-03_rb, &
        &  0.182283e-03_rb, 0.151505e-03_rb, 0.125924e-03_rb, 0.104662e-03_rb, 0.869904e-04_rb /)
      selfrefo(:, 3) = (/ &
        &  0.166821e-02_rb, 0.151103e-02_rb, 0.136866e-02_rb, 0.123970e-02_rb, 0.112290e-02_rb, &
        &  0.101710e-02_rb, 0.921266e-03_rb, 0.834463e-03_rb, 0.755839e-03_rb, 0.684623e-03_rb /)
      selfrefo(:, 4) = (/ &
        &  0.460175e-02_rb, 0.421372e-02_rb, 0.385842e-02_rb, 0.353307e-02_rb, 0.323516e-02_rb, &
        &  0.296236e-02_rb, 0.271257e-02_rb, 0.248385e-02_rb, 0.227440e-02_rb, 0.208262e-02_rb /)
      selfrefo(:, 5) = (/ &
        &  0.101589e-01_rb, 0.924742e-02_rb, 0.841772e-02_rb, 0.766247e-02_rb, 0.697497e-02_rb, &
        &  0.634917e-02_rb, 0.577951e-02_rb, 0.526096e-02_rb, 0.478893e-02_rb, 0.435926e-02_rb /)
      selfrefo(:, 6) = (/ &
        &  0.328043e-01_rb, 0.300853e-01_rb, 0.275917e-01_rb, 0.253048e-01_rb, 0.232075e-01_rb, &
        &  0.212839e-01_rb, 0.195198e-01_rb, 0.179020e-01_rb, 0.164182e-01_rb, 0.150574e-01_rb /)
      selfrefo(:, 7) = (/ &
        &  0.405936e-01_rb, 0.376032e-01_rb, 0.348331e-01_rb, 0.322671e-01_rb, 0.298901e-01_rb, &
        &  0.276883e-01_rb, 0.256486e-01_rb, 0.237591e-01_rb, 0.220089e-01_rb, 0.203876e-01_rb /)
      selfrefo(:, 8) = (/ &
        &  0.448362e-01_rb, 0.413811e-01_rb, 0.381923e-01_rb, 0.352492e-01_rb, 0.325329e-01_rb, &
        &  0.300259e-01_rb, 0.277121e-01_rb, 0.255766e-01_rb, 0.236056e-01_rb, 0.217866e-01_rb /)
      selfrefo(:, 9) = (/ &
        &  0.479741e-01_rb, 0.445389e-01_rb, 0.413497e-01_rb, 0.383889e-01_rb, 0.356400e-01_rb, &
        &  0.330880e-01_rb, 0.307188e-01_rb, 0.285191e-01_rb, 0.264770e-01_rb, 0.245812e-01_rb /)
      selfrefo(:,10) = (/ &
        &  0.519308e-01_rb, 0.484130e-01_rb, 0.451335e-01_rb, 0.420761e-01_rb, 0.392259e-01_rb, &
        &  0.365687e-01_rb, 0.340916e-01_rb, 0.317822e-01_rb, 0.296293e-01_rb, 0.276222e-01_rb /)
      selfrefo(:,11) = (/ &
        &  0.572039e-01_rb, 0.527780e-01_rb, 0.486945e-01_rb, 0.449270e-01_rb, 0.414510e-01_rb, &
        &  0.382439e-01_rb, 0.352849e-01_rb, 0.325549e-01_rb, 0.300361e-01_rb, 0.277122e-01_rb /)
      selfrefo(:,12) = (/ &
        &  0.601046e-01_rb, 0.554411e-01_rb, 0.511395e-01_rb, 0.471716e-01_rb, 0.435116e-01_rb, &
        &  0.401356e-01_rb, 0.370215e-01_rb, 0.341490e-01_rb, 0.314994e-01_rb, 0.290554e-01_rb /)
      selfrefo(:,13) = (/ &
        &  0.616595e-01_rb, 0.567145e-01_rb, 0.521662e-01_rb, 0.479826e-01_rb, 0.441346e-01_rb, &
        &  0.405951e-01_rb, 0.373395e-01_rb, 0.343450e-01_rb, 0.315906e-01_rb, 0.290571e-01_rb /)
      selfrefo(:,14) = (/ &
        &  0.647916e-01_rb, 0.592493e-01_rb, 0.541811e-01_rb, 0.495465e-01_rb, 0.453083e-01_rb, &
        &  0.414326e-01_rb, 0.378885e-01_rb, 0.346475e-01_rb, 0.316837e-01_rb, 0.289735e-01_rb /)
      selfrefo(:,15) = (/ &
        &  0.694231e-01_rb, 0.637703e-01_rb, 0.585777e-01_rb, 0.538079e-01_rb, 0.494265e-01_rb, &
        &  0.454019e-01_rb, 0.417050e-01_rb, 0.383091e-01_rb, 0.351897e-01_rb, 0.323244e-01_rb /)
      selfrefo(:,16) = (/ &
        &  0.761764e-01_rb, 0.701815e-01_rb, 0.646584e-01_rb, 0.595700e-01_rb, 0.548820e-01_rb, &
        &  0.505629e-01_rb, 0.465838e-01_rb, 0.429178e-01_rb, 0.395403e-01_rb, 0.364286e-01_rb /)
     
      end subroutine sw_kgb21

! **************************************************************************
      subroutine sw_kgb22
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg22, only : kao, kbo, selfrefo, forrefo, sfluxrefo, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:, 1) = (/ &
        & 3.71641_rb    ,3.63190_rb    ,3.44795_rb    ,3.17936_rb    , &
        & 2.86071_rb    ,2.48490_rb    ,2.02471_rb    ,1.52475_rb    , &
        & 1.03811_rb    ,0.113272_rb   ,9.37115e-02_rb,7.38969e-02_rb, &
        & 5.44713e-02_rb,3.45905e-02_rb,1.30293e-02_rb,1.84198e-03_rb /)
      sfluxrefo(:, 2) = (/ &
        & 3.73933_rb    ,3.60360_rb    ,3.43370_rb    ,3.19749_rb    , & 
        & 2.87747_rb    ,2.47926_rb    ,2.02175_rb    ,1.52010_rb    , &
        & 1.03612_rb    ,0.113265_rb   ,9.37145e-02_rb,7.38951e-02_rb, &
        & 5.44714e-02_rb,3.45906e-02_rb,1.30293e-02_rb,1.84198e-03_rb /)
      sfluxrefo(:, 3) = (/ &
        & 3.73889_rb    ,3.60279_rb    ,3.43404_rb    ,3.20560_rb    , &
        & 2.87367_rb    ,2.47515_rb    ,2.02412_rb    ,1.52315_rb    , &
        & 1.03146_rb    ,0.113272_rb   ,9.36707e-02_rb,7.39080e-02_rb, &
        & 5.44598e-02_rb,3.45906e-02_rb,1.30293e-02_rb,1.84198e-03_rb /)
      sfluxrefo(:, 4) = (/ &
        & 3.73801_rb    ,3.60530_rb    ,3.43659_rb    ,3.20640_rb    , &
        & 2.87039_rb    ,2.47330_rb    ,2.02428_rb    ,1.52509_rb    , &
        & 1.03037_rb    ,0.112553_rb   ,9.35352e-02_rb,7.39675e-02_rb, &
        & 5.43951e-02_rb,3.45669e-02_rb,1.30292e-02_rb,1.84198e-03_rb /)
      sfluxrefo(:, 5) = (/ &
        & 3.73809_rb    ,3.60996_rb    ,3.43602_rb    ,3.20364_rb    , &
        & 2.87005_rb    ,2.47343_rb    ,2.02353_rb    ,1.52617_rb    , &
        & 1.03138_rb    ,0.111172_rb   ,9.29885e-02_rb,7.35034e-02_rb, &
        & 5.42427e-02_rb,3.45732e-02_rb,1.30169e-02_rb,1.84550e-03_rb /)
      sfluxrefo(:, 6) = (/ &
        & 3.73872_rb    ,3.62054_rb    ,3.42934_rb    ,3.20110_rb    , &
        & 2.86886_rb    ,2.47379_rb    ,2.02237_rb    ,1.52754_rb    , & 
        & 1.03228_rb    ,0.111597_rb   ,9.12252e-02_rb,7.33115e-02_rb, &
        & 5.35600e-02_rb,3.45187e-02_rb,1.30184e-02_rb,1.84551e-03_rb /)
      sfluxrefo(:, 7) = (/ &
        & 3.73969_rb    ,3.65461_rb    ,3.40646_rb    ,3.19082_rb    , &
        & 2.86919_rb    ,2.47289_rb    ,2.02312_rb    ,1.52629_rb    , &
        & 1.03329_rb    ,0.111611_rb   ,9.16275e-02_rb,7.14731e-02_rb, &
        & 5.31771e-02_rb,3.44980e-02_rb,1.30190e-02_rb,1.84551e-03_rb /)
      sfluxrefo(:, 8) = (/ &
        & 3.73995_rb    ,3.65348_rb    ,3.43707_rb    ,3.16351_rb    , &
        & 2.87003_rb    ,2.47392_rb    ,2.02114_rb    ,1.52548_rb    , & 
        & 1.03306_rb    ,0.111088_rb   ,9.12422e-02_rb,7.11146e-02_rb, &
        & 5.31333e-02_rb,3.45302e-02_rb,1.30209e-02_rb,1.84554e-03_rb /)
      sfluxrefo(:, 9) = (/ &
        & 3.73788_rb    ,3.65004_rb    ,3.46938_rb    ,3.15236_rb    , &
        & 2.86381_rb    ,2.47393_rb    ,2.01715_rb    ,1.52134_rb    , &
        & 1.03163_rb    ,0.111259_rb   ,9.12948e-02_rb,7.09999e-02_rb, &
        & 5.31792e-02_rb,3.44955e-02_rb,1.30189e-02_rb,1.84551e-03_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:,1) = (/ &
        & 3.74823E+00_rb, 3.69363E+00_rb, 3.44687E+00_rb, 3.26290E+00_rb,&
        & 2.92572E+00_rb, 2.52211E+00_rb, 2.05143E+00_rb, 1.53735E+00_rb,&
        & 1.04799E+00_rb, 1.14542E-01_rb, 9.45431E-02_rb, 7.49502E-02_rb,&
        & 5.49881E-02_rb, 3.49936E-02_rb, 1.31835E-02_rb, 1.86747E-03_rb/)
      irradnceo(:,2) = (/ &
        & 3.76832E+00_rb, 3.67354E+00_rb, 3.45493E+00_rb, 3.26249E+00_rb,&
        & 2.92380E+00_rb, 2.51866E+00_rb, 2.04968E+00_rb, 1.53681E+00_rb,&
        & 1.04800E+00_rb, 1.14539E-01_rb, 9.45439E-02_rb, 7.49500E-02_rb,&
        & 5.49878E-02_rb, 3.49937E-02_rb, 1.31835E-02_rb, 1.86747E-03_rb/)
      irradnceo(:,3) = (/ &
        & 3.77870E+00_rb, 3.63525E+00_rb, 3.48698E+00_rb, 3.26122E+00_rb,&
        & 2.92415E+00_rb, 2.51645E+00_rb, 2.04776E+00_rb, 1.53949E+00_rb,&
        & 1.04631E+00_rb, 1.14472E-01_rb, 9.45361E-02_rb, 7.49503E-02_rb,&
        & 5.49880E-02_rb, 3.49936E-02_rb, 1.31835E-02_rb, 1.86747E-03_rb/)
      irradnceo(:,4) = (/ &
        & 3.77006E+00_rb, 3.63847E+00_rb, 3.49238E+00_rb, 3.26338E+00_rb,&
        & 2.92466E+00_rb, 2.51391E+00_rb, 2.04656E+00_rb, 1.54256E+00_rb,&
        & 1.04445E+00_rb, 1.14377E-01_rb, 9.46071E-02_rb, 7.48761E-02_rb,&
        & 5.49550E-02_rb, 3.49932E-02_rb, 1.31835E-02_rb, 1.86747E-03_rb/)
      irradnceo(:,5) = (/ &
        & 3.77525E+00_rb, 3.63235E+00_rb, 3.50041E+00_rb, 3.25827E+00_rb,&
        & 2.92525E+00_rb, 2.51237E+00_rb, 2.04501E+00_rb, 1.54408E+00_rb,&
        & 1.04357E+00_rb, 1.14310E-01_rb, 9.46663E-02_rb, 7.48674E-02_rb,&
        & 5.49219E-02_rb, 3.49193E-02_rb, 1.31693E-02_rb, 1.86747E-03_rb/)
      irradnceo(:,6) = (/ &
        & 3.77674E+00_rb, 3.63211E+00_rb, 3.50486E+00_rb, 3.25808E+00_rb,&
        & 2.91942E+00_rb, 2.51307E+00_rb, 2.04530E+00_rb, 1.54377E+00_rb,&
        & 1.04559E+00_rb, 1.12572E-01_rb, 9.41925E-02_rb, 7.48240E-02_rb,&
        & 5.49011E-02_rb, 3.48985E-02_rb, 1.31116E-02_rb, 1.85577E-03_rb/)
      irradnceo(:,7) = (/ &
        & 3.77856E+00_rb, 3.63388E+00_rb, 3.50754E+00_rb, 3.25647E+00_rb,&
        & 2.91595E+00_rb, 2.50572E+00_rb, 2.05135E+00_rb, 1.54396E+00_rb,&
        & 1.04734E+00_rb, 1.13415E-01_rb, 9.32393E-02_rb, 7.31846E-02_rb,&
        & 5.48668E-02_rb, 3.48458E-02_rb, 1.31035E-02_rb, 1.85577E-03_rb/)
      irradnceo(:,8) = (/ &
        & 3.77132E+00_rb, 3.64520E+00_rb, 3.51269E+00_rb, 3.25252E+00_rb,&
        & 2.91024E+00_rb, 2.50813E+00_rb, 2.04965E+00_rb, 1.54379E+00_rb,&
        & 1.04874E+00_rb, 1.13280E-01_rb, 9.29902E-02_rb, 7.21233E-02_rb,&
        & 5.48165E-02_rb, 3.48348E-02_rb, 1.31032E-02_rb, 1.85577E-03_rb/)
      irradnceo(:,9) = (/ &
        & 3.76660E+00_rb, 3.67878E+00_rb, 3.49903E+00_rb, 3.27505E+00_rb,&
        & 2.86732E+00_rb, 2.51860E+00_rb, 2.04901E+00_rb, 1.54137E+00_rb,&
        & 1.04675E+00_rb, 1.13257E-01_rb, 9.29404E-02_rb, 7.19629E-02_rb,&
        & 5.48106E-02_rb, 3.48468E-02_rb, 1.31035E-02_rb, 1.85577E-03_rb/)
! Solar variability components: facular brightening
      facbrghto(:,1) = (/ &
        & 1.24230E-03_rb, 1.27853E-03_rb, 1.28342E-03_rb, 1.17466E-03_rb,&
        & 1.03530E-03_rb, 8.87667E-04_rb, 7.27702E-04_rb, 5.46169E-04_rb,&
        & 3.69604E-04_rb, 4.03178E-05_rb, 3.33264E-05_rb, 2.64022E-05_rb,&
        & 1.93895E-05_rb, 1.23250E-05_rb, 4.62947E-06_rb, 6.55660E-07_rb/)
      facbrghto(:,2) = (/ &
        & 1.24514E-03_rb, 1.31532E-03_rb, 1.26991E-03_rb, 1.16032E-03_rb,&
        & 1.02942E-03_rb, 8.86496E-04_rb, 7.26274E-04_rb, 5.43905E-04_rb,&
        & 3.68546E-04_rb, 4.03157E-05_rb, 3.33255E-05_rb, 2.64027E-05_rb,&
        & 1.93893E-05_rb, 1.23251E-05_rb, 4.62947E-06_rb, 6.55660E-07_rb/)
      facbrghto(:,3) = (/ &
        & 1.25408E-03_rb, 1.32923E-03_rb, 1.26245E-03_rb, 1.15647E-03_rb,&
        & 1.02624E-03_rb, 8.84906E-04_rb, 7.24849E-04_rb, 5.41353E-04_rb,&
        & 3.66141E-04_rb, 4.00498E-05_rb, 3.32354E-05_rb, 2.64018E-05_rb,&
        & 1.93894E-05_rb, 1.23250E-05_rb, 4.62947E-06_rb, 6.55660E-07_rb/)
      facbrghto(:,4) = (/ &
        & 1.28391E-03_rb, 1.32107E-03_rb, 1.25660E-03_rb, 1.14939E-03_rb,&
        & 1.02440E-03_rb, 8.84924E-04_rb, 7.23282E-04_rb, 5.38899E-04_rb,&
        & 3.64029E-04_rb, 3.99099E-05_rb, 3.30567E-05_rb, 2.60687E-05_rb,&
        & 1.92312E-05_rb, 1.23246E-05_rb, 4.62947E-06_rb, 6.55660E-07_rb/)
      facbrghto(:,5) = (/ &
        & 1.31673E-03_rb, 1.31296E-03_rb, 1.24834E-03_rb, 1.14401E-03_rb,&
        & 1.02049E-03_rb, 8.84333E-04_rb, 7.21637E-04_rb, 5.36859E-04_rb,&
        & 3.62280E-04_rb, 3.96506E-05_rb, 3.28655E-05_rb, 2.60171E-05_rb,&
        & 1.90123E-05_rb, 1.19941E-05_rb, 4.56203E-06_rb, 6.55660E-07_rb/)
      facbrghto(:,6) = (/ &
        & 1.34438E-03_rb, 1.31469E-03_rb, 1.24120E-03_rb, 1.13630E-03_rb,&
        & 1.01815E-03_rb, 8.81682E-04_rb, 7.17127E-04_rb, 5.35246E-04_rb,&
        & 3.60614E-04_rb, 3.91996E-05_rb, 3.25414E-05_rb, 2.57895E-05_rb,&
        & 1.87422E-05_rb, 1.18334E-05_rb, 4.28864E-06_rb, 6.01334E-07_rb/)
      facbrghto(:,7) = (/ &
        & 1.39070E-03_rb, 1.29569E-03_rb, 1.23827E-03_rb, 1.13222E-03_rb,&
        & 1.01383E-03_rb, 8.80293E-04_rb, 7.09395E-04_rb, 5.32113E-04_rb,&
        & 3.58950E-04_rb, 3.90364E-05_rb, 3.20271E-05_rb, 2.51886E-05_rb,&
        & 1.84124E-05_rb, 1.14123E-05_rb, 4.25130E-06_rb, 6.01334E-07_rb/)
      facbrghto(:,8) = (/ &
        & 1.41492E-03_rb, 1.32898E-03_rb, 1.21785E-03_rb, 1.12547E-03_rb,&
        & 1.02034E-03_rb, 8.65733E-04_rb, 6.98615E-04_rb, 5.25991E-04_rb,&
        & 3.56012E-04_rb, 3.82393E-05_rb, 3.14450E-05_rb, 2.45516E-05_rb,&
        & 1.80131E-05_rb, 1.13804E-05_rb, 4.24982E-06_rb, 6.01335E-07_rb/)
      facbrghto(:,9) = (/ &
        & 1.40699E-03_rb, 1.34250E-03_rb, 1.25250E-03_rb, 1.13065E-03_rb,&
        & 1.01052E-03_rb, 8.54580E-04_rb, 6.89949E-04_rb, 5.16942E-04_rb,&
        & 3.49858E-04_rb, 3.78531E-05_rb, 3.12354E-05_rb, 2.45527E-05_rb,&
        & 1.80169E-05_rb, 1.13875E-05_rb, 4.25141E-06_rb, 6.01334E-07_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:,1) = (/ &
        &-8.73693E-04_rb,-8.59167E-04_rb,-8.33250E-04_rb,-7.66092E-04_rb,&
        &-6.80210E-04_rb,-5.83742E-04_rb,-4.78051E-04_rb,-3.59287E-04_rb,&
        &-2.43371E-04_rb,-2.65249E-05_rb,-2.19401E-05_rb,-1.73541E-05_rb,&
        &-1.27422E-05_rb,-8.10434E-06_rb,-3.04800E-06_rb,-4.31668E-07_rb/)
      snsptdrko(:,2) = (/ &
        &-8.72183E-04_rb,-8.68616E-04_rb,-8.30294E-04_rb,-7.62215E-04_rb,&
        &-6.79093E-04_rb,-5.83942E-04_rb,-4.77976E-04_rb,-3.59167E-04_rb,&
        &-2.43377E-04_rb,-2.65244E-05_rb,-2.19399E-05_rb,-1.73542E-05_rb,&
        &-1.27421E-05_rb,-8.10438E-06_rb,-3.04801E-06_rb,-4.31668E-07_rb/)
      snsptdrko(:,3) = (/ &
        &-8.72636E-04_rb,-8.71854E-04_rb,-8.29136E-04_rb,-7.61031E-04_rb,&
        &-6.78220E-04_rb,-5.83978E-04_rb,-4.77721E-04_rb,-3.58969E-04_rb,&
        &-2.43313E-04_rb,-2.65261E-05_rb,-2.19418E-05_rb,-1.73540E-05_rb,&
        &-1.27421E-05_rb,-8.10435E-06_rb,-3.04801E-06_rb,-4.31668E-07_rb/)
      snsptdrko(:,4) = (/ &
        &-8.78227E-04_rb,-8.70172E-04_rb,-8.27708E-04_rb,-7.59231E-04_rb,&
        &-6.77945E-04_rb,-5.84134E-04_rb,-4.77439E-04_rb,-3.58813E-04_rb,&
        &-2.43190E-04_rb,-2.65182E-05_rb,-2.19470E-05_rb,-1.73556E-05_rb,&
        &-1.27448E-05_rb,-8.10421E-06_rb,-3.04801E-06_rb,-4.31668E-07_rb/)
      snsptdrko(:,5) = (/ &
        &-8.84857E-04_rb,-8.68601E-04_rb,-8.24729E-04_rb,-7.58008E-04_rb,&
        &-6.77343E-04_rb,-5.84286E-04_rb,-4.77308E-04_rb,-3.58630E-04_rb,&
        &-2.43125E-04_rb,-2.65016E-05_rb,-2.19443E-05_rb,-1.73439E-05_rb,&
        &-1.27406E-05_rb,-8.10801E-06_rb,-3.05041E-06_rb,-4.31668E-07_rb/)
      snsptdrko(:,6) = (/ &
        &-8.91549E-04_rb,-8.67780E-04_rb,-8.21909E-04_rb,-7.56142E-04_rb,&
        &-6.76746E-04_rb,-5.84214E-04_rb,-4.77029E-04_rb,-3.58503E-04_rb,&
        &-2.43064E-04_rb,-2.64930E-05_rb,-2.19309E-05_rb,-1.73373E-05_rb,&
        &-1.27202E-05_rb,-8.09712E-06_rb,-3.05850E-06_rb,-4.33583E-07_rb/)
      snsptdrko(:,7) = (/ &
        &-9.03905E-04_rb,-8.61341E-04_rb,-8.18541E-04_rb,-7.54329E-04_rb,&
        &-6.76749E-04_rb,-5.84256E-04_rb,-4.76492E-04_rb,-3.58414E-04_rb,&
        &-2.43007E-04_rb,-2.64668E-05_rb,-2.19091E-05_rb,-1.73104E-05_rb,&
        &-1.26963E-05_rb,-8.09748E-06_rb,-3.05946E-06_rb,-4.33583E-07_rb/)
      snsptdrko(:,8) = (/ &
        &-9.10235E-04_rb,-8.66606E-04_rb,-8.10940E-04_rb,-7.52439E-04_rb,&
        &-6.76679E-04_rb,-5.82916E-04_rb,-4.76178E-04_rb,-3.58278E-04_rb,&
        &-2.42870E-04_rb,-2.64375E-05_rb,-2.18589E-05_rb,-1.72857E-05_rb,&
        &-1.26925E-05_rb,-8.09876E-06_rb,-3.05990E-06_rb,-4.33583E-07_rb/)
      snsptdrko(:,9) = (/ &
        &-9.06042E-04_rb,-8.69423E-04_rb,-8.20125E-04_rb,-7.51274E-04_rb,&
        &-6.72670E-04_rb,-5.81362E-04_rb,-4.75615E-04_rb,-3.58038E-04_rb,&
        &-2.42619E-04_rb,-2.64182E-05_rb,-2.18547E-05_rb,-1.72840E-05_rb,&
        &-1.26932E-05_rb,-8.09654E-06_rb,-3.05942E-06_rb,-4.33583E-07_rb/)

! Rayleigh extinction coefficient at v = 8000 cm-1.
      rayl = 1.54e-08_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.14658e-08_rb,0.32044e-07_rb,0.36289e-07_rb,0.38717e-07_rb,0.40590e-07_rb, &
        & 0.43330e-07_rb,0.46916e-07_rb,0.56519e-07_rb,0.42446e-07_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.17883e-08_rb,0.36129e-07_rb,0.40724e-07_rb,0.43498e-07_rb,0.45371e-07_rb, &
        & 0.47093e-07_rb,0.51195e-07_rb,0.58562e-07_rb,0.44474e-07_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.21286e-08_rb,0.46092e-07_rb,0.53991e-07_rb,0.59439e-07_rb,0.63497e-07_rb, &
        & 0.66730e-07_rb,0.70296e-07_rb,0.83990e-07_rb,0.65058e-07_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.24816e-08_rb,0.43399e-07_rb,0.49167e-07_rb,0.52969e-07_rb,0.55586e-07_rb, &
        & 0.57260e-07_rb,0.58809e-07_rb,0.64046e-07_rb,0.48780e-07_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.28409e-08_rb,0.46683e-07_rb,0.53128e-07_rb,0.57526e-07_rb,0.60605e-07_rb, &
        & 0.62715e-07_rb,0.64284e-07_rb,0.67579e-07_rb,0.52346e-07_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.10965e-08_rb,0.24934e-07_rb,0.28268e-07_rb,0.30156e-07_rb,0.31820e-07_rb, &
        & 0.34498e-07_rb,0.36563e-07_rb,0.46609e-07_rb,0.30202e-07_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.13538e-08_rb,0.28338e-07_rb,0.31923e-07_rb,0.34048e-07_rb,0.35529e-07_rb, &
        & 0.37136e-07_rb,0.41583e-07_rb,0.46831e-07_rb,0.32817e-07_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.16260e-08_rb,0.36648e-07_rb,0.42882e-07_rb,0.47128e-07_rb,0.50288e-07_rb, &
        & 0.52851e-07_rb,0.56093e-07_rb,0.70511e-07_rb,0.48182e-07_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.19102e-08_rb,0.39471e-07_rb,0.46225e-07_rb,0.50974e-07_rb,0.54565e-07_rb, &
        & 0.57380e-07_rb,0.60079e-07_rb,0.67707e-07_rb,0.50168e-07_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.22030e-08_rb,0.37203e-07_rb,0.42234e-07_rb,0.45622e-07_rb,0.47990e-07_rb, &
        & 0.49546e-07_rb,0.50777e-07_rb,0.53867e-07_rb,0.39769e-07_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.77844e-09_rb,0.18777e-07_rb,0.21347e-07_rb,0.22946e-07_rb,0.24368e-07_rb, &
        & 0.25870e-07_rb,0.28084e-07_rb,0.39197e-07_rb,0.20196e-07_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.97891e-09_rb,0.21591e-07_rb,0.24379e-07_rb,0.25974e-07_rb,0.27213e-07_rb, &
        & 0.29039e-07_rb,0.31487e-07_rb,0.37725e-07_rb,0.23228e-07_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.11949e-08_rb,0.24305e-07_rb,0.27342e-07_rb,0.29177e-07_rb,0.30428e-07_rb, &
        & 0.31562e-07_rb,0.34299e-07_rb,0.39267e-07_rb,0.23943e-07_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.14225e-08_rb,0.30991e-07_rb,0.36244e-07_rb,0.39880e-07_rb,0.42596e-07_rb, &
        & 0.44752e-07_rb,0.47132e-07_rb,0.56253e-07_rb,0.37250e-07_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.16586e-08_rb,0.29152e-07_rb,0.32988e-07_rb,0.35508e-07_rb,0.37264e-07_rb, &
        & 0.38387e-07_rb,0.39417e-07_rb,0.42936e-07_rb,0.29439e-07_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.53935e-09_rb,0.13922e-07_rb,0.15779e-07_rb,0.16979e-07_rb,0.18157e-07_rb, &
        & 0.19199e-07_rb,0.22168e-07_rb,0.31060e-07_rb,0.13772e-07_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.69404e-09_rb,0.16220e-07_rb,0.18381e-07_rb,0.19646e-07_rb,0.20841e-07_rb, &
        & 0.22407e-07_rb,0.23846e-07_rb,0.31992e-07_rb,0.15532e-07_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.86327e-09_rb,0.18514e-07_rb,0.20845e-07_rb,0.22209e-07_rb,0.23196e-07_rb, &
        & 0.24396e-07_rb,0.27026e-07_rb,0.31067e-07_rb,0.17116e-07_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.10436e-08_rb,0.20666e-07_rb,0.23248e-07_rb,0.24835e-07_rb,0.25904e-07_rb, &
        & 0.26757e-07_rb,0.28384e-07_rb,0.33085e-07_rb,0.19320e-07_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.12327e-08_rb,0.26056e-07_rb,0.30481e-07_rb,0.33580e-07_rb,0.35908e-07_rb, &
        & 0.37744e-07_rb,0.39586e-07_rb,0.45434e-07_rb,0.29787e-07_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.36905e-09_rb,0.10245e-07_rb,0.11596e-07_rb,0.12576e-07_rb,0.13293e-07_rb, &
        & 0.14477e-07_rb,0.18355e-07_rb,0.25395e-07_rb,0.95950e-08_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.48708e-09_rb,0.12119e-07_rb,0.13746e-07_rb,0.14791e-07_rb,0.15718e-07_rb, &
        & 0.16646e-07_rb,0.18396e-07_rb,0.25737e-07_rb,0.10793e-07_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.61847e-09_rb,0.13998e-07_rb,0.15813e-07_rb,0.16851e-07_rb,0.17722e-07_rb, &
        & 0.19232e-07_rb,0.20414e-07_rb,0.25223e-07_rb,0.12354e-07_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.76052e-09_rb,0.15846e-07_rb,0.17816e-07_rb,0.18991e-07_rb,0.19806e-07_rb, &
        & 0.20628e-07_rb,0.23016e-07_rb,0.25874e-07_rb,0.14269e-07_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.91109e-09_rb,0.20382e-07_rb,0.23823e-07_rb,0.26180e-07_rb,0.27929e-07_rb, &
        & 0.29345e-07_rb,0.31039e-07_rb,0.38471e-07_rb,0.22626e-07_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.24637e-09_rb,0.74144e-08_rb,0.83997e-08_rb,0.90789e-08_rb,0.97769e-08_rb, &
        & 0.11308e-07_rb,0.14223e-07_rb,0.20837e-07_rb,0.66398e-08_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.33527e-09_rb,0.89450e-08_rb,0.10119e-07_rb,0.10892e-07_rb,0.11613e-07_rb, &
        & 0.12382e-07_rb,0.14900e-07_rb,0.20638e-07_rb,0.76422e-08_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.43587e-09_rb,0.10479e-07_rb,0.11902e-07_rb,0.12737e-07_rb,0.13564e-07_rb, &
        & 0.14416e-07_rb,0.15545e-07_rb,0.21677e-07_rb,0.89455e-08_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.54672e-09_rb,0.12017e-07_rb,0.13540e-07_rb,0.14423e-07_rb,0.15090e-07_rb, &
        & 0.16030e-07_rb,0.17496e-07_rb,0.20678e-07_rb,0.10490e-07_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.66579e-09_rb,0.13493e-07_rb,0.15163e-07_rb,0.16178e-07_rb,0.16873e-07_rb, &
        & 0.17474e-07_rb,0.18852e-07_rb,0.21699e-07_rb,0.12216e-07_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.16149e-09_rb,0.53201e-08_rb,0.60079e-08_rb,0.65349e-08_rb,0.72837e-08_rb, &
        & 0.90792e-08_rb,0.11028e-07_rb,0.15917e-07_rb,0.45574e-08_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.22765e-09_rb,0.65457e-08_rb,0.74143e-08_rb,0.80126e-08_rb,0.85152e-08_rb, &
        & 0.94934e-08_rb,0.12210e-07_rb,0.16917e-07_rb,0.54301e-08_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.30418e-09_rb,0.77971e-08_rb,0.88287e-08_rb,0.94927e-08_rb,0.10154e-07_rb, &
        & 0.10712e-07_rb,0.12099e-07_rb,0.16996e-07_rb,0.64594e-08_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.38990e-09_rb,0.90563e-08_rb,0.10243e-07_rb,0.10928e-07_rb,0.11554e-07_rb, &
        & 0.12512e-07_rb,0.13254e-07_rb,0.17320e-07_rb,0.76835e-08_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.48334e-09_rb,0.10307e-07_rb,0.11588e-07_rb,0.12345e-07_rb,0.12884e-07_rb, &
        & 0.13512e-07_rb,0.15051e-07_rb,0.17108e-07_rb,0.90662e-08_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.10442e-09_rb,0.37895e-08_rb,0.42781e-08_rb,0.47285e-08_rb,0.55672e-08_rb, &
        & 0.67246e-08_rb,0.87615e-08_rb,0.12496e-07_rb,0.31063e-08_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.15294e-09_rb,0.47545e-08_rb,0.53771e-08_rb,0.58176e-08_rb,0.63249e-08_rb, &
        & 0.74935e-08_rb,0.93024e-08_rb,0.13484e-07_rb,0.38514e-08_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.21056e-09_rb,0.57727e-08_rb,0.65270e-08_rb,0.70416e-08_rb,0.74786e-08_rb, &
        & 0.80475e-08_rb,0.10012e-07_rb,0.13784e-07_rb,0.46496e-08_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.27636e-09_rb,0.67987e-08_rb,0.77111e-08_rb,0.82983e-08_rb,0.87947e-08_rb, &
        & 0.93313e-08_rb,0.10204e-07_rb,0.14244e-07_rb,0.56141e-08_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.34924e-09_rb,0.78297e-08_rb,0.88319e-08_rb,0.94055e-08_rb,0.98667e-08_rb, &
        & 0.10620e-07_rb,0.11396e-07_rb,0.13825e-07_rb,0.67139e-08_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.66385e-10_rb,0.26743e-08_rb,0.30313e-08_rb,0.34595e-08_rb,0.42156e-08_rb, &
        & 0.50940e-08_rb,0.66728e-08_rb,0.10021e-07_rb,0.20965e-08_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.10109e-09_rb,0.34278e-08_rb,0.38685e-08_rb,0.42217e-08_rb,0.47673e-08_rb, &
        & 0.59120e-08_rb,0.72992e-08_rb,0.10438e-07_rb,0.27321e-08_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.14405e-09_rb,0.42386e-08_rb,0.48029e-08_rb,0.51840e-08_rb,0.55416e-08_rb, &
        & 0.62759e-08_rb,0.79851e-08_rb,0.11314e-07_rb,0.33332e-08_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.19414e-09_rb,0.50774e-08_rb,0.57440e-08_rb,0.61764e-08_rb,0.66005e-08_rb, &
        & 0.69887e-08_rb,0.81552e-08_rb,0.11319e-07_rb,0.40828e-08_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.25056e-09_rb,0.59198e-08_rb,0.67025e-08_rb,0.71627e-08_rb,0.76155e-08_rb, &
        & 0.81557e-08_rb,0.87035e-08_rb,0.11788e-07_rb,0.49512e-08_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.43147e-10_rb,0.19136e-08_rb,0.21883e-08_rb,0.26153e-08_rb,0.31055e-08_rb, &
        & 0.39720e-08_rb,0.51107e-08_rb,0.84498e-08_rb,0.15100e-08_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.68069e-10_rb,0.25012e-08_rb,0.28230e-08_rb,0.31278e-08_rb,0.37084e-08_rb, &
        & 0.44577e-08_rb,0.58316e-08_rb,0.82732e-08_rb,0.20294e-08_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.10013e-09_rb,0.31445e-08_rb,0.35530e-08_rb,0.38466e-08_rb,0.41961e-08_rb, &
        & 0.50063e-08_rb,0.61960e-08_rb,0.89421e-08_rb,0.24447e-08_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.13837e-09_rb,0.38258e-08_rb,0.43236e-08_rb,0.46732e-08_rb,0.49545e-08_rb, &
        & 0.53447e-08_rb,0.67172e-08_rb,0.92217e-08_rb,0.30274e-08_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.18212e-09_rb,0.45123e-08_rb,0.51146e-08_rb,0.55026e-08_rb,0.58368e-08_rb, &
        & 0.61896e-08_rb,0.67974e-08_rb,0.94910e-08_rb,0.36975e-08_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.35238e-10_rb,0.15650e-08_rb,0.17900e-08_rb,0.21398e-08_rb,0.25395e-08_rb, &
        & 0.32442e-08_rb,0.41684e-08_rb,0.68456e-08_rb,0.12890e-08_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.55588e-10_rb,0.20451e-08_rb,0.23079e-08_rb,0.25570e-08_rb,0.30342e-08_rb, &
        & 0.36465e-08_rb,0.47717e-08_rb,0.67353e-08_rb,0.16499e-08_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.81825e-10_rb,0.25719e-08_rb,0.29058e-08_rb,0.31458e-08_rb,0.34324e-08_rb, &
        & 0.40982e-08_rb,0.50669e-08_rb,0.73083e-08_rb,0.20031e-08_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.11311e-09_rb,0.31293e-08_rb,0.35365e-08_rb,0.38229e-08_rb,0.40518e-08_rb, &
        & 0.43730e-08_rb,0.54993e-08_rb,0.75470e-08_rb,0.24754e-08_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.14891e-09_rb,0.36915e-08_rb,0.41839e-08_rb,0.45010e-08_rb,0.47744e-08_rb, &
        & 0.50622e-08_rb,0.55621e-08_rb,0.77689e-08_rb,0.30237e-08_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.28851e-10_rb,0.12816e-08_rb,0.14656e-08_rb,0.17508e-08_rb,0.20791e-08_rb, &
        & 0.26542e-08_rb,0.34061e-08_rb,0.55940e-08_rb,0.10876e-08_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.45511e-10_rb,0.16747e-08_rb,0.18898e-08_rb,0.20935e-08_rb,0.24841e-08_rb, &
        & 0.29844e-08_rb,0.39040e-08_rb,0.54988e-08_rb,0.13569e-08_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.66972e-10_rb,0.21053e-08_rb,0.23788e-08_rb,0.25750e-08_rb,0.28091e-08_rb, &
        & 0.33541e-08_rb,0.41474e-08_rb,0.59783e-08_rb,0.16488e-08_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.92606e-10_rb,0.25620e-08_rb,0.28952e-08_rb,0.31304e-08_rb,0.33162e-08_rb, &
        & 0.35801e-08_rb,0.45003e-08_rb,0.61789e-08_rb,0.20293e-08_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.12192e-09_rb,0.30226e-08_rb,0.34255e-08_rb,0.36843e-08_rb,0.39084e-08_rb, &
        & 0.41445e-08_rb,0.45524e-08_rb,0.63605e-08_rb,0.24756e-08_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.23621e-10_rb,0.10494e-08_rb,0.11996e-08_rb,0.14331e-08_rb,0.17023e-08_rb, &
        & 0.21711e-08_rb,0.27844e-08_rb,0.45532e-08_rb,0.89489e-09_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.37261e-10_rb,0.13712e-08_rb,0.15471e-08_rb,0.17138e-08_rb,0.20333e-08_rb, &
        & 0.24433e-08_rb,0.31958e-08_rb,0.44925e-08_rb,0.11124e-08_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.54831e-10_rb,0.17238e-08_rb,0.19475e-08_rb,0.21082e-08_rb,0.23003e-08_rb, &
        & 0.27446e-08_rb,0.33959e-08_rb,0.48947e-08_rb,0.13514e-08_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.75799e-10_rb,0.20971e-08_rb,0.23698e-08_rb,0.25624e-08_rb,0.27143e-08_rb, &
        & 0.29304e-08_rb,0.36822e-08_rb,0.50589e-08_rb,0.16628e-08_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.99820e-10_rb,0.24745e-08_rb,0.28041e-08_rb,0.30159e-08_rb,0.31995e-08_rb, &
        & 0.33924e-08_rb,0.37279e-08_rb,0.52074e-08_rb,0.20270e-08_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.13785e-07_rb,0.24738e-06_rb,0.32984e-06_rb,0.38970e-06_rb,0.43737e-06_rb, &
        & 0.50889e-06_rb,0.56985e-06_rb,0.52821e-06_rb,0.22857e-06_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.15768e-07_rb,0.27583e-06_rb,0.36371e-06_rb,0.42333e-06_rb,0.46175e-06_rb, &
        & 0.50608e-06_rb,0.57108e-06_rb,0.57199e-06_rb,0.21992e-06_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.17740e-07_rb,0.31295e-06_rb,0.41668e-06_rb,0.48111e-06_rb,0.52105e-06_rb, &
        & 0.56257e-06_rb,0.61638e-06_rb,0.64253e-06_rb,0.25630e-06_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.19676e-07_rb,0.33378e-06_rb,0.43874e-06_rb,0.50021e-06_rb,0.53506e-06_rb, &
        & 0.56909e-06_rb,0.60800e-06_rb,0.63155e-06_rb,0.24206e-06_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.21537e-07_rb,0.36160e-06_rb,0.47596e-06_rb,0.54056e-06_rb,0.57557e-06_rb, &
        & 0.60002e-06_rb,0.61606e-06_rb,0.61827e-06_rb,0.25207e-06_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.10621e-07_rb,0.19636e-06_rb,0.26336e-06_rb,0.31062e-06_rb,0.35012e-06_rb, &
        & 0.41219e-06_rb,0.46827e-06_rb,0.42345e-06_rb,0.16044e-06_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.12238e-07_rb,0.21970e-06_rb,0.28920e-06_rb,0.33755e-06_rb,0.37073e-06_rb, &
        & 0.41004e-06_rb,0.46315e-06_rb,0.45777e-06_rb,0.15877e-06_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.13863e-07_rb,0.24955e-06_rb,0.33252e-06_rb,0.38359e-06_rb,0.41679e-06_rb, &
        & 0.45237e-06_rb,0.50277e-06_rb,0.51752e-06_rb,0.18617e-06_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.15467e-07_rb,0.27294e-06_rb,0.36292e-06_rb,0.41560e-06_rb,0.44818e-06_rb, &
        & 0.47909e-06_rb,0.51811e-06_rb,0.54528e-06_rb,0.19311e-06_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.17013e-07_rb,0.28946e-06_rb,0.38040e-06_rb,0.43177e-06_rb,0.46055e-06_rb, &
        & 0.48627e-06_rb,0.50036e-06_rb,0.50711e-06_rb,0.19056e-06_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.79041e-08_rb,0.15193e-06_rb,0.20367e-06_rb,0.24255e-06_rb,0.27584e-06_rb, &
        & 0.33106e-06_rb,0.39018e-06_rb,0.33367e-06_rb,0.10147e-06_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.92256e-08_rb,0.17005e-06_rb,0.22457e-06_rb,0.26442e-06_rb,0.29755e-06_rb, &
        & 0.33937e-06_rb,0.38659e-06_rb,0.36019e-06_rb,0.10982e-06_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.10548e-07_rb,0.18874e-06_rb,0.24827e-06_rb,0.28648e-06_rb,0.31109e-06_rb, &
        & 0.33996e-06_rb,0.38319e-06_rb,0.38934e-06_rb,0.11953e-06_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.11869e-07_rb,0.21314e-06_rb,0.28402e-06_rb,0.32565e-06_rb,0.35249e-06_rb, &
        & 0.38039e-06_rb,0.41637e-06_rb,0.43423e-06_rb,0.13779e-06_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.13170e-07_rb,0.22682e-06_rb,0.29805e-06_rb,0.33849e-06_rb,0.36193e-06_rb, &
        & 0.38387e-06_rb,0.41078e-06_rb,0.42298e-06_rb,0.13819e-06_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.57872e-08_rb,0.11704e-06_rb,0.15599e-06_rb,0.19092e-06_rb,0.22043e-06_rb, &
        & 0.26778e-06_rb,0.30128e-06_rb,0.25769e-06_rb,0.68111e-07_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.68505e-08_rb,0.13037e-06_rb,0.17389e-06_rb,0.20556e-06_rb,0.23194e-06_rb, &
        & 0.27485e-06_rb,0.31447e-06_rb,0.28139e-06_rb,0.78259e-07_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.79353e-08_rb,0.14574e-06_rb,0.19135e-06_rb,0.22328e-06_rb,0.24623e-06_rb, &
        & 0.27497e-06_rb,0.31125e-06_rb,0.30440e-06_rb,0.85397e-07_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.90207e-08_rb,0.16075e-06_rb,0.21140e-06_rb,0.24210e-06_rb,0.26199e-06_rb, &
        & 0.28412e-06_rb,0.31832e-06_rb,0.32424e-06_rb,0.93374e-07_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.10099e-07_rb,0.18068e-06_rb,0.24072e-06_rb,0.27515e-06_rb,0.29704e-06_rb, &
        & 0.31830e-06_rb,0.34575e-06_rb,0.36301e-06_rb,0.10959e-06_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.42071e-08_rb,0.88799e-07_rb,0.11974e-06_rb,0.15503e-06_rb,0.18473e-06_rb, &
        & 0.21750e-06_rb,0.23111e-06_rb,0.19568e-06_rb,0.47645e-07_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.50589e-08_rb,0.10003e-06_rb,0.13353e-06_rb,0.15994e-06_rb,0.18265e-06_rb, &
        & 0.22030e-06_rb,0.25851e-06_rb,0.22196e-06_rb,0.56400e-07_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.59363e-08_rb,0.11183e-06_rb,0.14743e-06_rb,0.17446e-06_rb,0.19617e-06_rb, &
        & 0.22887e-06_rb,0.25813e-06_rb,0.23893e-06_rb,0.62471e-07_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.68265e-08_rb,0.12408e-06_rb,0.16316e-06_rb,0.18879e-06_rb,0.20540e-06_rb, &
        & 0.22681e-06_rb,0.25640e-06_rb,0.25817e-06_rb,0.69233e-07_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.77164e-08_rb,0.14022e-06_rb,0.18715e-06_rb,0.21470e-06_rb,0.23265e-06_rb, &
        & 0.25223e-06_rb,0.27783e-06_rb,0.28837e-06_rb,0.82855e-07_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.30108e-08_rb,0.66997e-07_rb,0.92811e-07_rb,0.12074e-06_rb,0.15295e-06_rb, &
        & 0.17839e-06_rb,0.17246e-06_rb,0.14892e-06_rb,0.33739e-07_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.36900e-08_rb,0.76605e-07_rb,0.10196e-06_rb,0.12672e-06_rb,0.14857e-06_rb, &
        & 0.17805e-06_rb,0.19825e-06_rb,0.16899e-06_rb,0.40438e-07_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.43974e-08_rb,0.85358e-07_rb,0.11411e-06_rb,0.13507e-06_rb,0.15283e-06_rb, &
        & 0.18257e-06_rb,0.21172e-06_rb,0.18597e-06_rb,0.46134e-07_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.51213e-08_rb,0.95538e-07_rb,0.12544e-06_rb,0.14691e-06_rb,0.16455e-06_rb, &
        & 0.18481e-06_rb,0.20979e-06_rb,0.20070e-06_rb,0.51618e-07_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.58500e-08_rb,0.10543e-06_rb,0.13870e-06_rb,0.15918e-06_rb,0.17259e-06_rb, &
        & 0.18830e-06_rb,0.21165e-06_rb,0.21507e-06_rb,0.58184e-07_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.21343e-08_rb,0.50731e-07_rb,0.73677e-07_rb,0.95876e-07_rb,0.12180e-06_rb, &
        & 0.14539e-06_rb,0.12792e-06_rb,0.11548e-06_rb,0.24049e-07_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.26695e-08_rb,0.57886e-07_rb,0.78431e-07_rb,0.10179e-06_rb,0.12404e-06_rb, &
        & 0.14498e-06_rb,0.15075e-06_rb,0.12789e-06_rb,0.28929e-07_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.32350e-08_rb,0.65356e-07_rb,0.87120e-07_rb,0.10537e-06_rb,0.12108e-06_rb, &
        & 0.14722e-06_rb,0.16980e-06_rb,0.14529e-06_rb,0.34426e-07_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.38202e-08_rb,0.73009e-07_rb,0.96634e-07_rb,0.11454e-06_rb,0.12893e-06_rb, &
        & 0.15239e-06_rb,0.17280e-06_rb,0.15699e-06_rb,0.38936e-07_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.44151e-08_rb,0.81203e-07_rb,0.10676e-06_rb,0.12425e-06_rb,0.13576e-06_rb, &
        & 0.15119e-06_rb,0.17122e-06_rb,0.16974e-06_rb,0.43692e-07_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.15010e-08_rb,0.38595e-07_rb,0.58295e-07_rb,0.77729e-07_rb,0.98683e-07_rb, &
        & 0.10992e-06_rb,0.95145e-07_rb,0.92252e-07_rb,0.17319e-07_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.19201e-08_rb,0.43791e-07_rb,0.61750e-07_rb,0.79921e-07_rb,0.10136e-06_rb, &
        & 0.11894e-06_rb,0.11276e-06_rb,0.97846e-07_rb,0.20720e-07_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.23693e-08_rb,0.50071e-07_rb,0.66873e-07_rb,0.85063e-07_rb,0.10052e-06_rb, &
        & 0.11892e-06_rb,0.13009e-06_rb,0.11045e-06_rb,0.25527e-07_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.28395e-08_rb,0.55947e-07_rb,0.74757e-07_rb,0.88934e-07_rb,0.10114e-06_rb, &
        & 0.12142e-06_rb,0.14341e-06_rb,0.12281e-06_rb,0.28938e-07_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.33227e-08_rb,0.62620e-07_rb,0.82343e-07_rb,0.96807e-07_rb,0.10922e-06_rb, &
        & 0.12542e-06_rb,0.14247e-06_rb,0.13252e-06_rb,0.32804e-07_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.10437e-08_rb,0.30107e-07_rb,0.47694e-07_rb,0.63218e-07_rb,0.80199e-07_rb, &
        & 0.81465e-07_rb,0.70748e-07_rb,0.72461e-07_rb,0.12620e-07_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.13691e-08_rb,0.33315e-07_rb,0.48674e-07_rb,0.63756e-07_rb,0.81165e-07_rb, &
        & 0.96892e-07_rb,0.83814e-07_rb,0.76311e-07_rb,0.14799e-07_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.17229e-08_rb,0.37985e-07_rb,0.51777e-07_rb,0.67272e-07_rb,0.83770e-07_rb, &
        & 0.97220e-07_rb,0.98951e-07_rb,0.84136e-07_rb,0.18369e-07_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.20990e-08_rb,0.43091e-07_rb,0.57320e-07_rb,0.69933e-07_rb,0.81214e-07_rb, &
        & 0.98518e-07_rb,0.11188e-06_rb,0.95453e-07_rb,0.21724e-07_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.24895e-08_rb,0.48030e-07_rb,0.63808e-07_rb,0.75595e-07_rb,0.85237e-07_rb, &
        & 0.10122e-06_rb,0.11598e-06_rb,0.10350e-06_rb,0.24792e-07_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.73565e-09_rb,0.23201e-07_rb,0.38192e-07_rb,0.51890e-07_rb,0.66677e-07_rb, &
        & 0.60941e-07_rb,0.53794e-07_rb,0.57060e-07_rb,0.95538e-08_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.98838e-09_rb,0.25706e-07_rb,0.38945e-07_rb,0.51950e-07_rb,0.65881e-07_rb, &
        & 0.73501e-07_rb,0.63459e-07_rb,0.61276e-07_rb,0.10909e-07_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.12675e-08_rb,0.29153e-07_rb,0.41180e-07_rb,0.53354e-07_rb,0.67729e-07_rb, &
        & 0.79616e-07_rb,0.75117e-07_rb,0.65122e-07_rb,0.13531e-07_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.15675e-08_rb,0.33329e-07_rb,0.44560e-07_rb,0.57066e-07_rb,0.67627e-07_rb, &
        & 0.79638e-07_rb,0.86678e-07_rb,0.73448e-07_rb,0.16804e-07_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.18817e-08_rb,0.37257e-07_rb,0.49746e-07_rb,0.59306e-07_rb,0.67554e-07_rb, &
        & 0.81218e-07_rb,0.96337e-07_rb,0.82154e-07_rb,0.18958e-07_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.60140e-09_rb,0.19007e-07_rb,0.31253e-07_rb,0.42433e-07_rb,0.54566e-07_rb, &
        & 0.50044e-07_rb,0.44177e-07_rb,0.46695e-07_rb,0.77009e-08_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.80782e-09_rb,0.21033e-07_rb,0.31865e-07_rb,0.42505e-07_rb,0.53930e-07_rb, &
        & 0.60271e-07_rb,0.51969e-07_rb,0.49836e-07_rb,0.88971e-08_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.10365e-08_rb,0.23860e-07_rb,0.33711e-07_rb,0.43668e-07_rb,0.55447e-07_rb, &
        & 0.65129e-07_rb,0.61514e-07_rb,0.53242e-07_rb,0.11044e-07_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.12819e-08_rb,0.27278e-07_rb,0.36477e-07_rb,0.46671e-07_rb,0.55334e-07_rb, &
        & 0.65191e-07_rb,0.70955e-07_rb,0.60123e-07_rb,0.13732e-07_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.15394e-08_rb,0.30488e-07_rb,0.40723e-07_rb,0.48551e-07_rb,0.55301e-07_rb, &
        & 0.66490e-07_rb,0.78877e-07_rb,0.67264e-07_rb,0.15488e-07_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.49234e-09_rb,0.15554e-07_rb,0.25575e-07_rb,0.34616e-07_rb,0.44615e-07_rb, &
        & 0.41056e-07_rb,0.36097e-07_rb,0.38091e-07_rb,0.62448e-08_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.66141e-09_rb,0.17225e-07_rb,0.26080e-07_rb,0.34787e-07_rb,0.44118e-07_rb, &
        & 0.49432e-07_rb,0.42531e-07_rb,0.40718e-07_rb,0.72624e-08_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.84839e-09_rb,0.19539e-07_rb,0.27604e-07_rb,0.35754e-07_rb,0.45370e-07_rb, &
        & 0.53306e-07_rb,0.50418e-07_rb,0.43565e-07_rb,0.90256e-08_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.10495e-08_rb,0.22342e-07_rb,0.29867e-07_rb,0.38196e-07_rb,0.45322e-07_rb, &
        & 0.53368e-07_rb,0.58112e-07_rb,0.49244e-07_rb,0.11197e-07_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.12604e-08_rb,0.24969e-07_rb,0.33348e-07_rb,0.39742e-07_rb,0.45269e-07_rb, &
        & 0.54454e-07_rb,0.64580e-07_rb,0.55079e-07_rb,0.12657e-07_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.40306e-09_rb,0.12738e-07_rb,0.20936e-07_rb,0.28312e-07_rb,0.36505e-07_rb, &
        & 0.33674e-07_rb,0.29525e-07_rb,0.31139e-07_rb,0.51040e-08_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.54151e-09_rb,0.14106e-07_rb,0.21346e-07_rb,0.28471e-07_rb,0.36139e-07_rb, &
        & 0.40516e-07_rb,0.34830e-07_rb,0.33325e-07_rb,0.59425e-08_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.69468e-09_rb,0.15999e-07_rb,0.22600e-07_rb,0.29268e-07_rb,0.37150e-07_rb, &
        & 0.43635e-07_rb,0.41297e-07_rb,0.35646e-07_rb,0.73862e-08_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.85913e-09_rb,0.18290e-07_rb,0.24452e-07_rb,0.31282e-07_rb,0.37096e-07_rb, &
        & 0.43709e-07_rb,0.47585e-07_rb,0.40319e-07_rb,0.91581e-08_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.10319e-08_rb,0.20440e-07_rb,0.27299e-07_rb,0.32532e-07_rb,0.37047e-07_rb, &
        & 0.44575e-07_rb,0.52880e-07_rb,0.45086e-07_rb,0.10367e-07_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.77542e-07_rb,0.72331e-06_rb,0.98561e-06_rb,0.11860e-05_rb,0.13497e-05_rb, &
        & 0.13448e-05_rb,0.13214e-05_rb,0.14401e-05_rb,0.51217e-06_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.83965e-07_rb,0.81202e-06_rb,0.11096e-05_rb,0.13150e-05_rb,0.14966e-05_rb, &
        & 0.14837e-05_rb,0.13715e-05_rb,0.14446e-05_rb,0.53558e-06_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.89893e-07_rb,0.91147e-06_rb,0.12494e-05_rb,0.14817e-05_rb,0.16581e-05_rb, &
        & 0.16379e-05_rb,0.14913e-05_rb,0.13540e-05_rb,0.57527e-06_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.95336e-07_rb,0.98977e-06_rb,0.13580e-05_rb,0.16044e-05_rb,0.17602e-05_rb, &
        & 0.17289e-05_rb,0.15601e-05_rb,0.13435e-05_rb,0.60153e-06_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.10024e-06_rb,0.10817e-05_rb,0.14820e-05_rb,0.17354e-05_rb,0.18806e-05_rb, &
        & 0.18468e-05_rb,0.16817e-05_rb,0.14550e-05_rb,0.63092e-06_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.61222e-07_rb,0.57382e-06_rb,0.78272e-06_rb,0.94950e-06_rb,0.10796e-05_rb, &
        & 0.10829e-05_rb,0.10794e-05_rb,0.12292e-05_rb,0.35440e-06_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.66652e-07_rb,0.64604e-06_rb,0.88511e-06_rb,0.10523e-05_rb,0.12028e-05_rb, &
        & 0.11924e-05_rb,0.11103e-05_rb,0.11987e-05_rb,0.38232e-06_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.71672e-07_rb,0.73084e-06_rb,0.10012e-05_rb,0.11880e-05_rb,0.13384e-05_rb, &
        & 0.13220e-05_rb,0.12030e-05_rb,0.11381e-05_rb,0.43262e-06_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.76301e-07_rb,0.80259e-06_rb,0.11033e-05_rb,0.13071e-05_rb,0.14343e-05_rb, &
        & 0.14171e-05_rb,0.12785e-05_rb,0.11080e-05_rb,0.46030e-06_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.80500e-07_rb,0.87105e-06_rb,0.11939e-05_rb,0.14005e-05_rb,0.15153e-05_rb, &
        & 0.14864e-05_rb,0.13529e-05_rb,0.11688e-05_rb,0.48117e-06_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.47304e-07_rb,0.44051e-06_rb,0.60584e-06_rb,0.74769e-06_rb,0.84468e-06_rb, &
        & 0.85077e-06_rb,0.85665e-06_rb,0.10494e-05_rb,0.24668e-06_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.51943e-07_rb,0.50181e-06_rb,0.68762e-06_rb,0.82034e-06_rb,0.93657e-06_rb, &
        & 0.92789e-06_rb,0.88766e-06_rb,0.99297e-06_rb,0.26836e-06_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.56252e-07_rb,0.56363e-06_rb,0.77259e-06_rb,0.91469e-06_rb,0.10403e-05_rb, &
        & 0.10243e-05_rb,0.93713e-06_rb,0.99098e-06_rb,0.29882e-06_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.60224e-07_rb,0.63130e-06_rb,0.86724e-06_rb,0.10288e-05_rb,0.11344e-05_rb, &
        & 0.11222e-05_rb,0.10162e-05_rb,0.91320e-06_rb,0.33854e-06_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.63868e-07_rb,0.68673e-06_rb,0.94084e-06_rb,0.11069e-05_rb,0.12001e-05_rb, &
        & 0.11819e-05_rb,0.10625e-05_rb,0.91639e-06_rb,0.36042e-06_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.36151e-07_rb,0.33292e-06_rb,0.46957e-06_rb,0.57176e-06_rb,0.64965e-06_rb, &
        & 0.66578e-06_rb,0.71030e-06_rb,0.90686e-06_rb,0.17407e-06_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.40107e-07_rb,0.38424e-06_rb,0.52571e-06_rb,0.63554e-06_rb,0.72553e-06_rb, &
        & 0.72856e-06_rb,0.72697e-06_rb,0.84339e-06_rb,0.19253e-06_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.43816e-07_rb,0.43401e-06_rb,0.59546e-06_rb,0.70644e-06_rb,0.80752e-06_rb, &
        & 0.79519e-06_rb,0.74520e-06_rb,0.81506e-06_rb,0.21830e-06_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.47242e-07_rb,0.48695e-06_rb,0.66577e-06_rb,0.78531e-06_rb,0.87970e-06_rb, &
        & 0.86799e-06_rb,0.78705e-06_rb,0.76812e-06_rb,0.24161e-06_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.50394e-07_rb,0.54183e-06_rb,0.74240e-06_rb,0.87572e-06_rb,0.95669e-06_rb, &
        & 0.94640e-06_rb,0.85379e-06_rb,0.74722e-06_rb,0.28102e-06_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.27462e-07_rb,0.25134e-06_rb,0.35698e-06_rb,0.43146e-06_rb,0.48552e-06_rb, &
        & 0.53177e-06_rb,0.62976e-06_rb,0.75904e-06_rb,0.12452e-06_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.30835e-07_rb,0.29261e-06_rb,0.40164e-06_rb,0.49701e-06_rb,0.56250e-06_rb, &
        & 0.56722e-06_rb,0.57761e-06_rb,0.72004e-06_rb,0.13996e-06_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.33997e-07_rb,0.33394e-06_rb,0.45684e-06_rb,0.54380e-06_rb,0.62140e-06_rb, &
        & 0.61241e-06_rb,0.60037e-06_rb,0.67265e-06_rb,0.16106e-06_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.36946e-07_rb,0.37590e-06_rb,0.51390e-06_rb,0.60628e-06_rb,0.69027e-06_rb, &
        & 0.67692e-06_rb,0.62237e-06_rb,0.67035e-06_rb,0.18128e-06_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.39670e-07_rb,0.42211e-06_rb,0.57710e-06_rb,0.68190e-06_rb,0.75209e-06_rb, &
        & 0.74450e-06_rb,0.67496e-06_rb,0.61709e-06_rb,0.21416e-06_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.20602e-07_rb,0.18703e-06_rb,0.26972e-06_rb,0.33456e-06_rb,0.36532e-06_rb, &
        & 0.42767e-06_rb,0.56721e-06_rb,0.66482e-06_rb,0.87936e-07_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.23484e-07_rb,0.21971e-06_rb,0.30953e-06_rb,0.37643e-06_rb,0.42627e-06_rb, &
        & 0.44517e-06_rb,0.48560e-06_rb,0.61604e-06_rb,0.10185e-06_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.26181e-07_rb,0.25382e-06_rb,0.34607e-06_rb,0.41971e-06_rb,0.47851e-06_rb, &
        & 0.48129e-06_rb,0.48156e-06_rb,0.56916e-06_rb,0.11864e-06_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.28705e-07_rb,0.28735e-06_rb,0.39181e-06_rb,0.46499e-06_rb,0.52971e-06_rb, &
        & 0.52262e-06_rb,0.49397e-06_rb,0.54792e-06_rb,0.13605e-06_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.31065e-07_rb,0.32265e-06_rb,0.43948e-06_rb,0.51712e-06_rb,0.58131e-06_rb, &
        & 0.57275e-06_rb,0.52154e-06_rb,0.53300e-06_rb,0.15326e-06_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.15315e-07_rb,0.13867e-06_rb,0.20497e-06_rb,0.25791e-06_rb,0.28251e-06_rb, &
        & 0.36209e-06_rb,0.48615e-06_rb,0.57617e-06_rb,0.62083e-07_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.17770e-07_rb,0.16467e-06_rb,0.23430e-06_rb,0.28445e-06_rb,0.31749e-06_rb, &
        & 0.35373e-06_rb,0.43530e-06_rb,0.51750e-06_rb,0.74598e-07_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.20071e-07_rb,0.19194e-06_rb,0.26349e-06_rb,0.32575e-06_rb,0.37090e-06_rb, &
        & 0.37391e-06_rb,0.38838e-06_rb,0.49613e-06_rb,0.88392e-07_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.22230e-07_rb,0.21953e-06_rb,0.29840e-06_rb,0.35680e-06_rb,0.40841e-06_rb, &
        & 0.40516e-06_rb,0.40513e-06_rb,0.45518e-06_rb,0.10149e-06_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.24244e-07_rb,0.24730e-06_rb,0.33583e-06_rb,0.39740e-06_rb,0.45364e-06_rb, &
        & 0.44496e-06_rb,0.41471e-06_rb,0.45022e-06_rb,0.11554e-06_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.11318e-07_rb,0.10309e-06_rb,0.15679e-06_rb,0.19765e-06_rb,0.22300e-06_rb, &
        & 0.30242e-06_rb,0.40824e-06_rb,0.49442e-06_rb,0.44061e-07_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.13358e-07_rb,0.12273e-06_rb,0.17705e-06_rb,0.22062e-06_rb,0.24115e-06_rb, &
        & 0.28999e-06_rb,0.39012e-06_rb,0.45451e-06_rb,0.54847e-07_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.15323e-07_rb,0.14448e-06_rb,0.20316e-06_rb,0.24566e-06_rb,0.27787e-06_rb, &
        & 0.29655e-06_rb,0.33237e-06_rb,0.41480e-06_rb,0.64157e-07_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.17158e-07_rb,0.16673e-06_rb,0.22673e-06_rb,0.27760e-06_rb,0.31491e-06_rb, &
        & 0.31778e-06_rb,0.31818e-06_rb,0.38422e-06_rb,0.75698e-07_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.18880e-07_rb,0.18905e-06_rb,0.25711e-06_rb,0.30558e-06_rb,0.34820e-06_rb, &
        & 0.34226e-06_rb,0.32757e-06_rb,0.36903e-06_rb,0.87559e-07_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.83060e-08_rb,0.76710e-07_rb,0.11565e-06_rb,0.14764e-06_rb,0.17329e-06_rb, &
        & 0.25783e-06_rb,0.35957e-06_rb,0.41243e-06_rb,0.31797e-07_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.99651e-08_rb,0.91242e-07_rb,0.13632e-06_rb,0.17065e-06_rb,0.18967e-06_rb, &
        & 0.24393e-06_rb,0.32914e-06_rb,0.39160e-06_rb,0.39225e-07_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.11621e-07_rb,0.10845e-06_rb,0.15426e-06_rb,0.18813e-06_rb,0.20803e-06_rb, &
        & 0.23565e-06_rb,0.29600e-06_rb,0.35083e-06_rb,0.47417e-07_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.13182e-07_rb,0.12617e-06_rb,0.17363e-06_rb,0.21434e-06_rb,0.24338e-06_rb, &
        & 0.24767e-06_rb,0.26174e-06_rb,0.33967e-06_rb,0.56851e-07_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.14647e-07_rb,0.14462e-06_rb,0.19609e-06_rb,0.23554e-06_rb,0.26966e-06_rb, &
        & 0.27000e-06_rb,0.26860e-06_rb,0.30941e-06_rb,0.65848e-07_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.61513e-08_rb,0.59201e-07_rb,0.89564e-07_rb,0.11096e-06_rb,0.13963e-06_rb, &
        & 0.22868e-06_rb,0.30519e-06_rb,0.34949e-06_rb,0.23910e-07_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.74969e-08_rb,0.68921e-07_rb,0.10513e-06_rb,0.13256e-06_rb,0.14959e-06_rb, &
        & 0.20339e-06_rb,0.27500e-06_rb,0.33281e-06_rb,0.28611e-07_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.88625e-08_rb,0.82016e-07_rb,0.11818e-06_rb,0.14725e-06_rb,0.16097e-06_rb, &
        & 0.19608e-06_rb,0.26185e-06_rb,0.30530e-06_rb,0.35935e-07_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.10184e-07_rb,0.96432e-07_rb,0.13542e-06_rb,0.16348e-06_rb,0.18477e-06_rb, &
        & 0.19886e-06_rb,0.22449e-06_rb,0.27787e-06_rb,0.42133e-07_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.11421e-07_rb,0.11117e-06_rb,0.15107e-06_rb,0.18528e-06_rb,0.21005e-06_rb, &
        & 0.21209e-06_rb,0.21257e-06_rb,0.25796e-06_rb,0.49988e-07_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.50310e-08_rb,0.48563e-07_rb,0.73473e-07_rb,0.91094e-07_rb,0.11455e-06_rb, &
        & 0.18734e-06_rb,0.24995e-06_rb,0.28620e-06_rb,0.19440e-07_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.61304e-08_rb,0.56532e-07_rb,0.86340e-07_rb,0.10878e-06_rb,0.12244e-06_rb, &
        & 0.16646e-06_rb,0.22541e-06_rb,0.27316e-06_rb,0.23338e-07_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.72491e-08_rb,0.67226e-07_rb,0.96798e-07_rb,0.12061e-06_rb,0.13181e-06_rb, &
        & 0.16043e-06_rb,0.21436e-06_rb,0.24928e-06_rb,0.29352e-07_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.83326e-08_rb,0.78998e-07_rb,0.11085e-06_rb,0.13391e-06_rb,0.15135e-06_rb, &
        & 0.16281e-06_rb,0.18360e-06_rb,0.22741e-06_rb,0.34425e-07_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.93458e-08_rb,0.91055e-07_rb,0.12364e-06_rb,0.15181e-06_rb,0.17196e-06_rb, &
        & 0.17367e-06_rb,0.17400e-06_rb,0.21108e-06_rb,0.40855e-07_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.41189e-08_rb,0.39856e-07_rb,0.60140e-07_rb,0.74963e-07_rb,0.94012e-07_rb, &
        & 0.15333e-06_rb,0.20441e-06_rb,0.23474e-06_rb,0.15853e-07_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.50193e-08_rb,0.46348e-07_rb,0.70855e-07_rb,0.89301e-07_rb,0.10040e-06_rb, &
        & 0.13626e-06_rb,0.18429e-06_rb,0.22399e-06_rb,0.19078e-07_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.59344e-08_rb,0.55100e-07_rb,0.79255e-07_rb,0.98783e-07_rb,0.10799e-06_rb, &
        & 0.13125e-06_rb,0.17544e-06_rb,0.20357e-06_rb,0.24013e-07_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.68228e-08_rb,0.64743e-07_rb,0.90843e-07_rb,0.10971e-06_rb,0.12396e-06_rb, &
        & 0.13321e-06_rb,0.15026e-06_rb,0.18611e-06_rb,0.28198e-07_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.76512e-08_rb,0.74591e-07_rb,0.10123e-06_rb,0.12433e-06_rb,0.14085e-06_rb, &
        & 0.14222e-06_rb,0.14246e-06_rb,0.17276e-06_rb,0.33441e-07_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.33724e-08_rb,0.32680e-07_rb,0.49222e-07_rb,0.61513e-07_rb,0.77133e-07_rb, &
        & 0.12555e-06_rb,0.16713e-06_rb,0.19233e-06_rb,0.12966e-07_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.41100e-08_rb,0.37987e-07_rb,0.58121e-07_rb,0.73233e-07_rb,0.82218e-07_rb, &
        & 0.11156e-06_rb,0.15077e-06_rb,0.18269e-06_rb,0.15614e-07_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.48589e-08_rb,0.45157e-07_rb,0.64912e-07_rb,0.80902e-07_rb,0.88466e-07_rb, &
        & 0.10747e-06_rb,0.14360e-06_rb,0.16616e-06_rb,0.19653e-07_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.55845e-08_rb,0.53032e-07_rb,0.74357e-07_rb,0.89835e-07_rb,0.10150e-06_rb, &
        & 0.10902e-06_rb,0.12309e-06_rb,0.15231e-06_rb,0.23083e-07_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.62642e-08_rb,0.61086e-07_rb,0.82886e-07_rb,0.10172e-06_rb,0.11529e-06_rb, &
        & 0.11645e-06_rb,0.11667e-06_rb,0.14140e-06_rb,0.27368e-07_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.38523e-06_rb,0.18405e-05_rb,0.24561e-05_rb,0.26688e-05_rb,0.26627e-05_rb, &
        & 0.26678e-05_rb,0.26221e-05_rb,0.23434e-05_rb,0.10243e-05_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.41258e-06_rb,0.20340e-05_rb,0.26799e-05_rb,0.29289e-05_rb,0.29138e-05_rb, &
        & 0.29293e-05_rb,0.29106e-05_rb,0.25138e-05_rb,0.11179e-05_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.43826e-06_rb,0.22391e-05_rb,0.29055e-05_rb,0.31733e-05_rb,0.31848e-05_rb, &
        & 0.32210e-05_rb,0.31726e-05_rb,0.29277e-05_rb,0.13122e-05_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.46214e-06_rb,0.24098e-05_rb,0.30735e-05_rb,0.33543e-05_rb,0.34157e-05_rb, &
        & 0.34436e-05_rb,0.34008e-05_rb,0.32157e-05_rb,0.14111e-05_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.48360e-06_rb,0.25684e-05_rb,0.32470e-05_rb,0.35527e-05_rb,0.36650e-05_rb, &
        & 0.37054e-05_rb,0.36463e-05_rb,0.35130e-05_rb,0.16128e-05_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.30632e-06_rb,0.14811e-05_rb,0.19813e-05_rb,0.21525e-05_rb,0.21480e-05_rb, &
        & 0.21651e-05_rb,0.21352e-05_rb,0.19097e-05_rb,0.76078e-06_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.32924e-06_rb,0.16454e-05_rb,0.21736e-05_rb,0.23701e-05_rb,0.23496e-05_rb, &
        & 0.23721e-05_rb,0.23664e-05_rb,0.20501e-05_rb,0.85884e-06_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.35084e-06_rb,0.18164e-05_rb,0.23625e-05_rb,0.25754e-05_rb,0.25799e-05_rb, &
        & 0.26089e-05_rb,0.25758e-05_rb,0.23569e-05_rb,0.98156e-06_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.37092e-06_rb,0.19724e-05_rb,0.25182e-05_rb,0.27461e-05_rb,0.27987e-05_rb, &
        & 0.28122e-05_rb,0.27766e-05_rb,0.26535e-05_rb,0.11297e-05_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.38936e-06_rb,0.20952e-05_rb,0.26519e-05_rb,0.29005e-05_rb,0.29889e-05_rb, &
        & 0.30120e-05_rb,0.29564e-05_rb,0.28641e-05_rb,0.12488e-05_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.23924e-06_rb,0.11590e-05_rb,0.15536e-05_rb,0.16834e-05_rb,0.16891e-05_rb, &
        & 0.17668e-05_rb,0.16904e-05_rb,0.15951e-05_rb,0.52632e-06_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.25864e-06_rb,0.12971e-05_rb,0.17221e-05_rb,0.18731e-05_rb,0.18563e-05_rb, &
        & 0.18804e-05_rb,0.18628e-05_rb,0.16576e-05_rb,0.62738e-06_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.27696e-06_rb,0.14325e-05_rb,0.18710e-05_rb,0.20345e-05_rb,0.20276e-05_rb, &
        & 0.20548e-05_rb,0.20259e-05_rb,0.17837e-05_rb,0.70461e-06_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.29413e-06_rb,0.15757e-05_rb,0.20191e-05_rb,0.22050e-05_rb,0.22319e-05_rb, &
        & 0.22354e-05_rb,0.22195e-05_rb,0.20885e-05_rb,0.84521e-06_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.31013e-06_rb,0.16829e-05_rb,0.21333e-05_rb,0.23274e-05_rb,0.23871e-05_rb, &
        & 0.23939e-05_rb,0.23475e-05_rb,0.22805e-05_rb,0.93762e-06_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.18537e-06_rb,0.89469e-06_rb,0.11953e-05_rb,0.13099e-05_rb,0.13271e-05_rb, &
        & 0.13910e-05_rb,0.13233e-05_rb,0.12842e-05_rb,0.37595e-06_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.20178e-06_rb,0.10096e-05_rb,0.13476e-05_rb,0.14636e-05_rb,0.14567e-05_rb, &
        & 0.14730e-05_rb,0.14642e-05_rb,0.13131e-05_rb,0.45273e-06_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.21732e-06_rb,0.11251e-05_rb,0.14779e-05_rb,0.16107e-05_rb,0.15945e-05_rb, &
        & 0.16194e-05_rb,0.16016e-05_rb,0.14026e-05_rb,0.52023e-06_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.23198e-06_rb,0.12348e-05_rb,0.15955e-05_rb,0.17391e-05_rb,0.17432e-05_rb, &
        & 0.17515e-05_rb,0.17226e-05_rb,0.15707e-05_rb,0.60921e-06_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.24566e-06_rb,0.13471e-05_rb,0.17138e-05_rb,0.18677e-05_rb,0.19071e-05_rb, &
        & 0.19021e-05_rb,0.18647e-05_rb,0.18054e-05_rb,0.72437e-06_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.14315e-06_rb,0.68654e-06_rb,0.91703e-06_rb,0.10136e-05_rb,0.10640e-05_rb, &
        & 0.10899e-05_rb,0.10066e-05_rb,0.10387e-05_rb,0.27115e-06_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.15693e-06_rb,0.78124e-06_rb,0.10460e-05_rb,0.11319e-05_rb,0.11350e-05_rb, &
        & 0.11836e-05_rb,0.11463e-05_rb,0.10954e-05_rb,0.33462e-06_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.17013e-06_rb,0.87619e-06_rb,0.11607e-05_rb,0.12633e-05_rb,0.12502e-05_rb, &
        & 0.12683e-05_rb,0.12476e-05_rb,0.11184e-05_rb,0.38421e-06_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.18255e-06_rb,0.96990e-06_rb,0.12617e-05_rb,0.13726e-05_rb,0.13641e-05_rb, &
        & 0.13787e-05_rb,0.13521e-05_rb,0.11952e-05_rb,0.45446e-06_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.19427e-06_rb,0.10668e-05_rb,0.13642e-05_rb,0.14869e-05_rb,0.15021e-05_rb, &
        & 0.14945e-05_rb,0.14801e-05_rb,0.13920e-05_rb,0.54371e-06_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.10978e-06_rb,0.52395e-06_rb,0.69476e-06_rb,0.76850e-06_rb,0.83675e-06_rb, &
        & 0.86491e-06_rb,0.77625e-06_rb,0.85871e-06_rb,0.19533e-06_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.12132e-06_rb,0.59825e-06_rb,0.79672e-06_rb,0.87685e-06_rb,0.88945e-06_rb, &
        & 0.93176e-06_rb,0.88241e-06_rb,0.87959e-06_rb,0.24185e-06_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.13245e-06_rb,0.67730e-06_rb,0.90458e-06_rb,0.97978e-06_rb,0.97457e-06_rb, &
        & 0.98659e-06_rb,0.98199e-06_rb,0.88654e-06_rb,0.28188e-06_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.14304e-06_rb,0.75520e-06_rb,0.99285e-06_rb,0.10803e-05_rb,0.10647e-05_rb, &
        & 0.10764e-05_rb,0.10623e-05_rb,0.93315e-06_rb,0.33738e-06_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.15298e-06_rb,0.83133e-06_rb,0.10714e-05_rb,0.11654e-05_rb,0.11612e-05_rb, &
        & 0.11628e-05_rb,0.11409e-05_rb,0.10211e-05_rb,0.39343e-06_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.83795e-07_rb,0.39583e-06_rb,0.52221e-06_rb,0.57623e-06_rb,0.66801e-06_rb, &
        & 0.68350e-06_rb,0.61427e-06_rb,0.74508e-06_rb,0.14027e-06_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.93456e-07_rb,0.45651e-06_rb,0.60707e-06_rb,0.67267e-06_rb,0.70886e-06_rb, &
        & 0.72875e-06_rb,0.66655e-06_rb,0.70837e-06_rb,0.17333e-06_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.10278e-06_rb,0.52065e-06_rb,0.69548e-06_rb,0.75346e-06_rb,0.75243e-06_rb, &
        & 0.79752e-06_rb,0.76130e-06_rb,0.73768e-06_rb,0.20746e-06_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.11172e-06_rb,0.58539e-06_rb,0.77614e-06_rb,0.84147e-06_rb,0.82956e-06_rb, &
        & 0.83892e-06_rb,0.81830e-06_rb,0.74649e-06_rb,0.24711e-06_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.12020e-06_rb,0.64890e-06_rb,0.84430e-06_rb,0.91375e-06_rb,0.90365e-06_rb, &
        & 0.91211e-06_rb,0.89073e-06_rb,0.78796e-06_rb,0.29290e-06_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.63771e-07_rb,0.29724e-06_rb,0.38901e-06_rb,0.42952e-06_rb,0.53569e-06_rb, &
        & 0.52739e-06_rb,0.51421e-06_rb,0.65416e-06_rb,0.10106e-06_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.71835e-07_rb,0.34855e-06_rb,0.45895e-06_rb,0.50927e-06_rb,0.55878e-06_rb, &
        & 0.57413e-06_rb,0.51386e-06_rb,0.58703e-06_rb,0.12396e-06_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.79626e-07_rb,0.39836e-06_rb,0.52930e-06_rb,0.58238e-06_rb,0.59321e-06_rb, &
        & 0.61917e-06_rb,0.58275e-06_rb,0.59574e-06_rb,0.15556e-06_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.87141e-07_rb,0.45236e-06_rb,0.60287e-06_rb,0.64978e-06_rb,0.64621e-06_rb, &
        & 0.65548e-06_rb,0.65037e-06_rb,0.59243e-06_rb,0.18170e-06_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.94308e-07_rb,0.50478e-06_rb,0.66216e-06_rb,0.71791e-06_rb,0.70542e-06_rb, &
        & 0.71265e-06_rb,0.70440e-06_rb,0.61750e-06_rb,0.21806e-06_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.48334e-07_rb,0.22150e-06_rb,0.29066e-06_rb,0.32650e-06_rb,0.41774e-06_rb, &
        & 0.40737e-06_rb,0.44502e-06_rb,0.56524e-06_rb,0.72879e-07_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.54994e-07_rb,0.26340e-06_rb,0.34441e-06_rb,0.38160e-06_rb,0.44410e-06_rb, &
        & 0.45746e-06_rb,0.41390e-06_rb,0.50482e-06_rb,0.89924e-07_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.61501e-07_rb,0.30416e-06_rb,0.40299e-06_rb,0.44693e-06_rb,0.47253e-06_rb, &
        & 0.48623e-06_rb,0.44624e-06_rb,0.47646e-06_rb,0.11220e-06_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.67792e-07_rb,0.34758e-06_rb,0.46191e-06_rb,0.50102e-06_rb,0.50055e-06_rb, &
        & 0.52975e-06_rb,0.50335e-06_rb,0.48665e-06_rb,0.13456e-06_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.73830e-07_rb,0.39095e-06_rb,0.51738e-06_rb,0.55894e-06_rb,0.55048e-06_rb, &
        & 0.55492e-06_rb,0.54317e-06_rb,0.49144e-06_rb,0.16088e-06_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.36931e-07_rb,0.16614e-06_rb,0.21895e-06_rb,0.26116e-06_rb,0.33023e-06_rb, &
        & 0.31438e-06_rb,0.35810e-06_rb,0.46819e-06_rb,0.53649e-07_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.42386e-07_rb,0.20028e-06_rb,0.26120e-06_rb,0.28858e-06_rb,0.36090e-06_rb, &
        & 0.35506e-06_rb,0.34639e-06_rb,0.43640e-06_rb,0.66551e-07_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.47801e-07_rb,0.23474e-06_rb,0.30842e-06_rb,0.34242e-06_rb,0.37552e-06_rb, &
        & 0.38348e-06_rb,0.34637e-06_rb,0.39519e-06_rb,0.81897e-07_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.53030e-07_rb,0.26856e-06_rb,0.35532e-06_rb,0.39019e-06_rb,0.39727e-06_rb, &
        & 0.41391e-06_rb,0.38815e-06_rb,0.39845e-06_rb,0.10316e-06_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.58073e-07_rb,0.30468e-06_rb,0.40415e-06_rb,0.43508e-06_rb,0.43212e-06_rb, &
        & 0.43860e-06_rb,0.43462e-06_rb,0.39513e-06_rb,0.12065e-06_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.30210e-07_rb,0.13672e-06_rb,0.18028e-06_rb,0.21388e-06_rb,0.27067e-06_rb, &
        & 0.25748e-06_rb,0.29347e-06_rb,0.38486e-06_rb,0.43992e-07_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.34676e-07_rb,0.16480e-06_rb,0.21447e-06_rb,0.23703e-06_rb,0.29658e-06_rb, &
        & 0.29189e-06_rb,0.28346e-06_rb,0.35425e-06_rb,0.54558e-07_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.39112e-07_rb,0.19301e-06_rb,0.25343e-06_rb,0.28112e-06_rb,0.30767e-06_rb, &
        & 0.31424e-06_rb,0.28354e-06_rb,0.32323e-06_rb,0.67060e-07_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.43392e-07_rb,0.22069e-06_rb,0.29158e-06_rb,0.31987e-06_rb,0.32514e-06_rb, &
        & 0.33920e-06_rb,0.31782e-06_rb,0.32572e-06_rb,0.84444e-07_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.47520e-07_rb,0.25011e-06_rb,0.33134e-06_rb,0.35641e-06_rb,0.35390e-06_rb, &
        & 0.35921e-06_rb,0.35594e-06_rb,0.32298e-06_rb,0.98845e-07_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.24733e-07_rb,0.11230e-06_rb,0.14856e-06_rb,0.17472e-06_rb,0.22197e-06_rb, &
        & 0.21118e-06_rb,0.24122e-06_rb,0.31477e-06_rb,0.36119e-07_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.28389e-07_rb,0.13551e-06_rb,0.17612e-06_rb,0.19468e-06_rb,0.24363e-06_rb, &
        & 0.23963e-06_rb,0.23290e-06_rb,0.28830e-06_rb,0.44709e-07_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.32017e-07_rb,0.15859e-06_rb,0.20791e-06_rb,0.23074e-06_rb,0.25230e-06_rb, &
        & 0.25771e-06_rb,0.23238e-06_rb,0.26334e-06_rb,0.54941e-07_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.35526e-07_rb,0.18123e-06_rb,0.23912e-06_rb,0.26228e-06_rb,0.26637e-06_rb, &
        & 0.27804e-06_rb,0.26039e-06_rb,0.26652e-06_rb,0.69162e-07_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.38912e-07_rb,0.20524e-06_rb,0.27165e-06_rb,0.29207e-06_rb,0.28991e-06_rb, &
        & 0.29419e-06_rb,0.29154e-06_rb,0.26412e-06_rb,0.80995e-07_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.20250e-07_rb,0.92231e-07_rb,0.12203e-06_rb,0.14275e-06_rb,0.18179e-06_rb, &
        & 0.17279e-06_rb,0.19796e-06_rb,0.25817e-06_rb,0.29574e-07_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.23243e-07_rb,0.11133e-06_rb,0.14442e-06_rb,0.15980e-06_rb,0.20001e-06_rb, &
        & 0.19638e-06_rb,0.19075e-06_rb,0.23541e-06_rb,0.36618e-07_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.26215e-07_rb,0.13018e-06_rb,0.17053e-06_rb,0.18925e-06_rb,0.20654e-06_rb, &
        & 0.21104e-06_rb,0.19040e-06_rb,0.21495e-06_rb,0.44987e-07_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.29082e-07_rb,0.14865e-06_rb,0.19601e-06_rb,0.21490e-06_rb,0.21820e-06_rb, &
        & 0.22779e-06_rb,0.21313e-06_rb,0.21809e-06_rb,0.56635e-07_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.31856e-07_rb,0.16826e-06_rb,0.22260e-06_rb,0.23930e-06_rb,0.23753e-06_rb, &
        & 0.24098e-06_rb,0.23875e-06_rb,0.21605e-06_rb,0.66316e-07_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.17059e-05_rb,0.38862e-05_rb,0.46330e-05_rb,0.51258e-05_rb,0.53516e-05_rb, &
        & 0.52890e-05_rb,0.49050e-05_rb,0.49561e-05_rb,0.24914e-05_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.17702e-05_rb,0.41613e-05_rb,0.49979e-05_rb,0.55352e-05_rb,0.58535e-05_rb, &
        & 0.58620e-05_rb,0.55540e-05_rb,0.53944e-05_rb,0.30111e-05_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.18268e-05_rb,0.44218e-05_rb,0.53631e-05_rb,0.59894e-05_rb,0.63892e-05_rb, &
        & 0.64888e-05_rb,0.63488e-05_rb,0.59957e-05_rb,0.35253e-05_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.18763e-05_rb,0.46467e-05_rb,0.56969e-05_rb,0.64112e-05_rb,0.68701e-05_rb, &
        & 0.70687e-05_rb,0.69575e-05_rb,0.65330e-05_rb,0.40117e-05_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.19168e-05_rb,0.48702e-05_rb,0.60413e-05_rb,0.68595e-05_rb,0.73828e-05_rb, &
        & 0.76555e-05_rb,0.76620e-05_rb,0.72236e-05_rb,0.44814e-05_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.13807e-05_rb,0.32052e-05_rb,0.37981e-05_rb,0.41868e-05_rb,0.43666e-05_rb, &
        & 0.42966e-05_rb,0.39397e-05_rb,0.40523e-05_rb,0.19972e-05_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.14355e-05_rb,0.34375e-05_rb,0.40968e-05_rb,0.45345e-05_rb,0.47869e-05_rb, &
        & 0.47900e-05_rb,0.45121e-05_rb,0.44176e-05_rb,0.23759e-05_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.14839e-05_rb,0.36628e-05_rb,0.44128e-05_rb,0.49223e-05_rb,0.52399e-05_rb, &
        & 0.53302e-05_rb,0.51982e-05_rb,0.49130e-05_rb,0.28052e-05_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.15261e-05_rb,0.38551e-05_rb,0.47116e-05_rb,0.52947e-05_rb,0.56758e-05_rb, &
        & 0.58601e-05_rb,0.57606e-05_rb,0.54224e-05_rb,0.31665e-05_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.15621e-05_rb,0.40345e-05_rb,0.49797e-05_rb,0.56344e-05_rb,0.60672e-05_rb, &
        & 0.63079e-05_rb,0.63076e-05_rb,0.59671e-05_rb,0.35368e-05_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.11048e-05_rb,0.25908e-05_rb,0.30458e-05_rb,0.33480e-05_rb,0.34767e-05_rb, &
        & 0.33444e-05_rb,0.30982e-05_rb,0.31520e-05_rb,0.14943e-05_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.11526e-05_rb,0.27857e-05_rb,0.32915e-05_rb,0.36364e-05_rb,0.38272e-05_rb, &
        & 0.38169e-05_rb,0.35492e-05_rb,0.35141e-05_rb,0.17575e-05_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.11948e-05_rb,0.29688e-05_rb,0.35427e-05_rb,0.39401e-05_rb,0.41747e-05_rb, &
        & 0.42239e-05_rb,0.40766e-05_rb,0.39111e-05_rb,0.20688e-05_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.12319e-05_rb,0.31426e-05_rb,0.38135e-05_rb,0.42625e-05_rb,0.45635e-05_rb, &
        & 0.47062e-05_rb,0.46027e-05_rb,0.43389e-05_rb,0.23873e-05_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.12642e-05_rb,0.32923e-05_rb,0.40402e-05_rb,0.45497e-05_rb,0.48934e-05_rb, &
        & 0.50814e-05_rb,0.50577e-05_rb,0.47614e-05_rb,0.26860e-05_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.87810e-06_rb,0.20726e-05_rb,0.24188e-05_rb,0.26498e-05_rb,0.27201e-05_rb, &
        & 0.25966e-05_rb,0.24215e-05_rb,0.25596e-05_rb,0.10910e-05_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.92113e-06_rb,0.22378e-05_rb,0.26243e-05_rb,0.28859e-05_rb,0.30228e-05_rb, &
        & 0.29855e-05_rb,0.27257e-05_rb,0.28497e-05_rb,0.12952e-05_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.95847e-06_rb,0.23927e-05_rb,0.28323e-05_rb,0.31308e-05_rb,0.33238e-05_rb, &
        & 0.33287e-05_rb,0.31597e-05_rb,0.31032e-05_rb,0.15368e-05_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.99115e-06_rb,0.25350e-05_rb,0.30423e-05_rb,0.33883e-05_rb,0.36085e-05_rb, &
        & 0.36829e-05_rb,0.35850e-05_rb,0.33993e-05_rb,0.17502e-05_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.10199e-05_rb,0.26768e-05_rb,0.32656e-05_rb,0.36641e-05_rb,0.39303e-05_rb, &
        & 0.40820e-05_rb,0.40444e-05_rb,0.38038e-05_rb,0.20588e-05_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.69521e-06_rb,0.16480e-05_rb,0.19136e-05_rb,0.20758e-05_rb,0.21053e-05_rb, &
        & 0.19983e-05_rb,0.19384e-05_rb,0.21119e-05_rb,0.79759e-06_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.73399e-06_rb,0.17885e-05_rb,0.20863e-05_rb,0.22802e-05_rb,0.23681e-05_rb, &
        & 0.23078e-05_rb,0.21093e-05_rb,0.21684e-05_rb,0.95048e-06_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.76704e-06_rb,0.19210e-05_rb,0.22551e-05_rb,0.24823e-05_rb,0.26201e-05_rb, &
        & 0.26181e-05_rb,0.24369e-05_rb,0.24387e-05_rb,0.11396e-05_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.79598e-06_rb,0.20422e-05_rb,0.24305e-05_rb,0.26936e-05_rb,0.28610e-05_rb, &
        & 0.28938e-05_rb,0.28032e-05_rb,0.26848e-05_rb,0.13054e-05_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.82146e-06_rb,0.21630e-05_rb,0.26149e-05_rb,0.29202e-05_rb,0.31281e-05_rb, &
        & 0.32364e-05_rb,0.31639e-05_rb,0.29903e-05_rb,0.15503e-05_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.54790e-06_rb,0.12968e-05_rb,0.14998e-05_rb,0.16199e-05_rb,0.16168e-05_rb, &
        & 0.15151e-05_rb,0.14932e-05_rb,0.16380e-05_rb,0.58437e-06_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.58180e-06_rb,0.14204e-05_rb,0.16474e-05_rb,0.17877e-05_rb,0.18382e-05_rb, &
        & 0.17645e-05_rb,0.16359e-05_rb,0.17414e-05_rb,0.70191e-06_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.61169e-06_rb,0.15332e-05_rb,0.17837e-05_rb,0.19554e-05_rb,0.20462e-05_rb, &
        & 0.20343e-05_rb,0.18483e-05_rb,0.19670e-05_rb,0.83824e-06_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.63729e-06_rb,0.16375e-05_rb,0.19280e-05_rb,0.21247e-05_rb,0.22559e-05_rb, &
        & 0.22588e-05_rb,0.21468e-05_rb,0.21190e-05_rb,0.96829e-06_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.65987e-06_rb,0.17326e-05_rb,0.20710e-05_rb,0.23017e-05_rb,0.24525e-05_rb, &
        & 0.25042e-05_rb,0.24377e-05_rb,0.23029e-05_rb,0.11330e-05_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.43067e-06_rb,0.10147e-05_rb,0.11663e-05_rb,0.12622e-05_rb,0.12316e-05_rb, &
        & 0.11186e-05_rb,0.12227e-05_rb,0.12153e-05_rb,0.41960e-06_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.45965e-06_rb,0.11216e-05_rb,0.12929e-05_rb,0.13935e-05_rb,0.14129e-05_rb, &
        & 0.13426e-05_rb,0.12944e-05_rb,0.14463e-05_rb,0.51416e-06_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.48636e-06_rb,0.12186e-05_rb,0.14109e-05_rb,0.15331e-05_rb,0.15936e-05_rb, &
        & 0.15473e-05_rb,0.14164e-05_rb,0.14707e-05_rb,0.61314e-06_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.50928e-06_rb,0.13071e-05_rb,0.15237e-05_rb,0.16740e-05_rb,0.17644e-05_rb, &
        & 0.17562e-05_rb,0.16373e-05_rb,0.16431e-05_rb,0.72862e-06_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.52919e-06_rb,0.13890e-05_rb,0.16443e-05_rb,0.18186e-05_rb,0.19296e-05_rb, &
        & 0.19451e-05_rb,0.18786e-05_rb,0.17942e-05_rb,0.84235e-06_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.33792e-06_rb,0.78856e-06_rb,0.90464e-06_rb,0.98317e-06_rb,0.92848e-06_rb, &
        & 0.86387e-06_rb,0.95438e-06_rb,0.90887e-06_rb,0.29529e-06_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.36263e-06_rb,0.88071e-06_rb,0.10121e-05_rb,0.10878e-05_rb,0.10817e-05_rb, &
        & 0.10133e-05_rb,0.10031e-05_rb,0.11161e-05_rb,0.37413e-06_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.38563e-06_rb,0.96560e-06_rb,0.11116e-05_rb,0.12011e-05_rb,0.12324e-05_rb, &
        & 0.11848e-05_rb,0.10927e-05_rb,0.11776e-05_rb,0.44987e-06_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.40622e-06_rb,0.10407e-05_rb,0.12047e-05_rb,0.13138e-05_rb,0.13745e-05_rb, &
        & 0.13598e-05_rb,0.12356e-05_rb,0.13203e-05_rb,0.54288e-06_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.42386e-06_rb,0.11111e-05_rb,0.13019e-05_rb,0.14302e-05_rb,0.15146e-05_rb, &
        & 0.15112e-05_rb,0.14255e-05_rb,0.14090e-05_rb,0.62729e-06_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.26432e-06_rb,0.60726e-06_rb,0.70036e-06_rb,0.76229e-06_rb,0.71577e-06_rb, &
        & 0.69749e-06_rb,0.70164e-06_rb,0.72668e-06_rb,0.20659e-06_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.28549e-06_rb,0.68848e-06_rb,0.78705e-06_rb,0.84862e-06_rb,0.82241e-06_rb, &
        & 0.74707e-06_rb,0.82519e-06_rb,0.82938e-06_rb,0.27449e-06_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.30511e-06_rb,0.76168e-06_rb,0.87199e-06_rb,0.93359e-06_rb,0.94531e-06_rb, &
        & 0.89794e-06_rb,0.85947e-06_rb,0.97111e-06_rb,0.33410e-06_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.32329e-06_rb,0.82660e-06_rb,0.95163e-06_rb,0.10287e-05_rb,0.10680e-05_rb, &
        & 0.10381e-05_rb,0.94558e-06_rb,0.98374e-06_rb,0.39896e-06_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.33897e-06_rb,0.88614e-06_rb,0.10276e-05_rb,0.11239e-05_rb,0.11817e-05_rb, &
        & 0.11723e-05_rb,0.10892e-05_rb,0.10943e-05_rb,0.47438e-06_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.20748e-06_rb,0.47023e-06_rb,0.54165e-06_rb,0.58159e-06_rb,0.54422e-06_rb, &
        & 0.55116e-06_rb,0.57712e-06_rb,0.60571e-06_rb,0.14752e-06_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.22561e-06_rb,0.54027e-06_rb,0.61555e-06_rb,0.66550e-06_rb,0.62550e-06_rb, &
        & 0.58306e-06_rb,0.64781e-06_rb,0.63365e-06_rb,0.19821e-06_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.24219e-06_rb,0.60251e-06_rb,0.68703e-06_rb,0.73412e-06_rb,0.72965e-06_rb, &
        & 0.68255e-06_rb,0.67291e-06_rb,0.76422e-06_rb,0.24874e-06_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.25766e-06_rb,0.65861e-06_rb,0.75375e-06_rb,0.81214e-06_rb,0.83171e-06_rb, &
        & 0.80129e-06_rb,0.73385e-06_rb,0.78864e-06_rb,0.29867e-06_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.27157e-06_rb,0.70870e-06_rb,0.81638e-06_rb,0.88593e-06_rb,0.92651e-06_rb, &
        & 0.91449e-06_rb,0.83048e-06_rb,0.88651e-06_rb,0.36087e-06_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.16984e-06_rb,0.38981e-06_rb,0.44742e-06_rb,0.48022e-06_rb,0.44950e-06_rb, &
        & 0.45173e-06_rb,0.47333e-06_rb,0.50203e-06_rb,0.12166e-06_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.18466e-06_rb,0.44753e-06_rb,0.50754e-06_rb,0.54744e-06_rb,0.51342e-06_rb, &
        & 0.47893e-06_rb,0.53296e-06_rb,0.51992e-06_rb,0.16283e-06_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.19826e-06_rb,0.49803e-06_rb,0.56578e-06_rb,0.60333e-06_rb,0.60031e-06_rb, &
        & 0.56085e-06_rb,0.55152e-06_rb,0.63442e-06_rb,0.20412e-06_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.21087e-06_rb,0.54349e-06_rb,0.62058e-06_rb,0.66722e-06_rb,0.68339e-06_rb, &
        & 0.65819e-06_rb,0.60140e-06_rb,0.64555e-06_rb,0.24501e-06_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.22228e-06_rb,0.58430e-06_rb,0.67139e-06_rb,0.72733e-06_rb,0.76067e-06_rb, &
        & 0.75025e-06_rb,0.68133e-06_rb,0.72307e-06_rb,0.29566e-06_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.13908e-06_rb,0.32322e-06_rb,0.36926e-06_rb,0.39668e-06_rb,0.37094e-06_rb, &
        & 0.37104e-06_rb,0.38736e-06_rb,0.41059e-06_rb,0.10018e-06_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.15119e-06_rb,0.37037e-06_rb,0.41804e-06_rb,0.44988e-06_rb,0.42138e-06_rb, &
        & 0.39334e-06_rb,0.43800e-06_rb,0.42649e-06_rb,0.13384e-06_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.16232e-06_rb,0.41107e-06_rb,0.46554e-06_rb,0.49568e-06_rb,0.49349e-06_rb, &
        & 0.46062e-06_rb,0.45205e-06_rb,0.52165e-06_rb,0.16749e-06_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.17268e-06_rb,0.44802e-06_rb,0.51028e-06_rb,0.54757e-06_rb,0.56096e-06_rb, &
        & 0.54041e-06_rb,0.49303e-06_rb,0.52723e-06_rb,0.20095e-06_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.18199e-06_rb,0.48092e-06_rb,0.55149e-06_rb,0.59694e-06_rb,0.62405e-06_rb, &
        & 0.61534e-06_rb,0.55851e-06_rb,0.58926e-06_rb,0.24228e-06_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.11387e-06_rb,0.26741e-06_rb,0.30441e-06_rb,0.32687e-06_rb,0.30479e-06_rb, &
        & 0.30477e-06_rb,0.31768e-06_rb,0.33316e-06_rb,0.82090e-07_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.12381e-06_rb,0.30579e-06_rb,0.34391e-06_rb,0.36935e-06_rb,0.34578e-06_rb, &
        & 0.32317e-06_rb,0.35995e-06_rb,0.35099e-06_rb,0.10981e-06_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.13290e-06_rb,0.33871e-06_rb,0.38265e-06_rb,0.40672e-06_rb,0.40511e-06_rb, &
        & 0.37796e-06_rb,0.36986e-06_rb,0.42813e-06_rb,0.13721e-06_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.14135e-06_rb,0.36878e-06_rb,0.41898e-06_rb,0.44916e-06_rb,0.46012e-06_rb, &
        & 0.44348e-06_rb,0.40383e-06_rb,0.43087e-06_rb,0.16460e-06_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.14898e-06_rb,0.39524e-06_rb,0.45245e-06_rb,0.48939e-06_rb,0.51148e-06_rb, &
        & 0.50420e-06_rb,0.45753e-06_rb,0.48088e-06_rb,0.19843e-06_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.49835e-05_rb,0.73512e-05_rb,0.88016e-05_rb,0.97115e-05_rb,0.10342e-04_rb, &
        & 0.10688e-04_rb,0.10747e-04_rb,0.97587e-05_rb,0.57353e-05_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.49823e-05_rb,0.77944e-05_rb,0.95403e-05_rb,0.10739e-04_rb,0.11522e-04_rb, &
        & 0.11974e-04_rb,0.12019e-04_rb,0.11200e-04_rb,0.69362e-05_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.49638e-05_rb,0.82513e-05_rb,0.10343e-04_rb,0.11836e-04_rb,0.12804e-04_rb, &
        & 0.13375e-04_rb,0.13427e-04_rb,0.12807e-04_rb,0.83325e-05_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.49300e-05_rb,0.87230e-05_rb,0.11131e-04_rb,0.12834e-04_rb,0.13980e-04_rb, &
        & 0.14660e-04_rb,0.14790e-04_rb,0.14186e-04_rb,0.94893e-05_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.48886e-05_rb,0.92046e-05_rb,0.11933e-04_rb,0.13885e-04_rb,0.15255e-04_rb, &
        & 0.16071e-04_rb,0.16239e-04_rb,0.15605e-04_rb,0.10917e-04_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.41174e-05_rb,0.62362e-05_rb,0.73897e-05_rb,0.81161e-05_rb,0.86061e-05_rb, &
        & 0.88364e-05_rb,0.89272e-05_rb,0.81534e-05_rb,0.45187e-05_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.41154e-05_rb,0.66112e-05_rb,0.80274e-05_rb,0.90030e-05_rb,0.96264e-05_rb, &
        & 0.99529e-05_rb,0.10004e-04_rb,0.93836e-05_rb,0.55300e-05_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.40999e-05_rb,0.69972e-05_rb,0.87232e-05_rb,0.99464e-05_rb,0.10738e-04_rb, &
        & 0.11186e-04_rb,0.11235e-04_rb,0.10760e-04_rb,0.66656e-05_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.40728e-05_rb,0.73933e-05_rb,0.94032e-05_rb,0.10842e-04_rb,0.11807e-04_rb, &
        & 0.12371e-04_rb,0.12495e-04_rb,0.12016e-04_rb,0.78073e-05_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.40371e-05_rb,0.77954e-05_rb,0.10060e-04_rb,0.11710e-04_rb,0.12855e-04_rb, &
        & 0.13529e-04_rb,0.13658e-04_rb,0.13169e-04_rb,0.88462e-05_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.33911e-05_rb,0.51807e-05_rb,0.60524e-05_rb,0.65745e-05_rb,0.69209e-05_rb, &
        & 0.70496e-05_rb,0.71376e-05_rb,0.64864e-05_rb,0.34232e-05_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.33916e-05_rb,0.54910e-05_rb,0.65875e-05_rb,0.73243e-05_rb,0.77878e-05_rb, &
        & 0.79847e-05_rb,0.80332e-05_rb,0.75309e-05_rb,0.42228e-05_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.33813e-05_rb,0.58057e-05_rb,0.71503e-05_rb,0.80869e-05_rb,0.86798e-05_rb, &
        & 0.89845e-05_rb,0.90042e-05_rb,0.85938e-05_rb,0.50519e-05_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.33618e-05_rb,0.61503e-05_rb,0.77470e-05_rb,0.88947e-05_rb,0.96421e-05_rb, &
        & 0.10056e-04_rb,0.10131e-04_rb,0.97836e-05_rb,0.60817e-05_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.33346e-05_rb,0.64834e-05_rb,0.82992e-05_rb,0.96266e-05_rb,0.10527e-04_rb, &
        & 0.11042e-04_rb,0.11154e-04_rb,0.10788e-04_rb,0.69208e-05_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.27862e-05_rb,0.42534e-05_rb,0.48987e-05_rb,0.52435e-05_rb,0.54733e-05_rb, &
        & 0.55364e-05_rb,0.55872e-05_rb,0.49226e-05_rb,0.25436e-05_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.27893e-05_rb,0.45093e-05_rb,0.53394e-05_rb,0.58691e-05_rb,0.61855e-05_rb, &
        & 0.63070e-05_rb,0.63399e-05_rb,0.58599e-05_rb,0.31726e-05_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.27843e-05_rb,0.47714e-05_rb,0.58075e-05_rb,0.65048e-05_rb,0.69158e-05_rb, &
        & 0.71341e-05_rb,0.71529e-05_rb,0.67578e-05_rb,0.38485e-05_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.27713e-05_rb,0.50491e-05_rb,0.62824e-05_rb,0.71451e-05_rb,0.76931e-05_rb, &
        & 0.79968e-05_rb,0.80403e-05_rb,0.77391e-05_rb,0.46093e-05_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.27517e-05_rb,0.53526e-05_rb,0.67897e-05_rb,0.78357e-05_rb,0.85357e-05_rb, &
        & 0.89260e-05_rb,0.90160e-05_rb,0.87287e-05_rb,0.54226e-05_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.22852e-05_rb,0.34704e-05_rb,0.39351e-05_rb,0.41626e-05_rb,0.42797e-05_rb, &
        & 0.43078e-05_rb,0.42289e-05_rb,0.36572e-05_rb,0.18725e-05_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.22904e-05_rb,0.36828e-05_rb,0.42969e-05_rb,0.46636e-05_rb,0.48818e-05_rb, &
        & 0.49227e-05_rb,0.49640e-05_rb,0.45776e-05_rb,0.23620e-05_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.22883e-05_rb,0.38995e-05_rb,0.46830e-05_rb,0.51887e-05_rb,0.54814e-05_rb, &
        & 0.56039e-05_rb,0.56350e-05_rb,0.52624e-05_rb,0.29139e-05_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.22808e-05_rb,0.41317e-05_rb,0.50763e-05_rb,0.57196e-05_rb,0.61208e-05_rb, &
        & 0.63389e-05_rb,0.63549e-05_rb,0.60859e-05_rb,0.35219e-05_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.22671e-05_rb,0.43851e-05_rb,0.54991e-05_rb,0.62948e-05_rb,0.68249e-05_rb, &
        & 0.71098e-05_rb,0.71755e-05_rb,0.69558e-05_rb,0.41900e-05_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.18699e-05_rb,0.28114e-05_rb,0.31400e-05_rb,0.32683e-05_rb,0.33153e-05_rb, &
        & 0.33008e-05_rb,0.31920e-05_rb,0.27860e-05_rb,0.13420e-05_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.18773e-05_rb,0.29859e-05_rb,0.34309e-05_rb,0.36695e-05_rb,0.38007e-05_rb, &
        & 0.38091e-05_rb,0.38180e-05_rb,0.34314e-05_rb,0.17179e-05_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.18783e-05_rb,0.31642e-05_rb,0.37448e-05_rb,0.40997e-05_rb,0.43019e-05_rb, &
        & 0.43546e-05_rb,0.43740e-05_rb,0.40084e-05_rb,0.21605e-05_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.18743e-05_rb,0.33566e-05_rb,0.40684e-05_rb,0.45371e-05_rb,0.48151e-05_rb, &
        & 0.49624e-05_rb,0.49624e-05_rb,0.47228e-05_rb,0.26431e-05_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.18657e-05_rb,0.35541e-05_rb,0.43955e-05_rb,0.49826e-05_rb,0.53649e-05_rb, &
        & 0.55694e-05_rb,0.55990e-05_rb,0.54328e-05_rb,0.31359e-05_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.15270e-05_rb,0.22681e-05_rb,0.24953e-05_rb,0.25538e-05_rb,0.25397e-05_rb, &
        & 0.25315e-05_rb,0.23299e-05_rb,0.21035e-05_rb,0.95748e-06_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.15365e-05_rb,0.24113e-05_rb,0.27304e-05_rb,0.28767e-05_rb,0.29339e-05_rb, &
        & 0.29290e-05_rb,0.28728e-05_rb,0.24852e-05_rb,0.12383e-05_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.15398e-05_rb,0.25576e-05_rb,0.29821e-05_rb,0.32226e-05_rb,0.33508e-05_rb, &
        & 0.33584e-05_rb,0.33733e-05_rb,0.31044e-05_rb,0.15900e-05_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.15385e-05_rb,0.27150e-05_rb,0.32458e-05_rb,0.35802e-05_rb,0.37697e-05_rb, &
        & 0.38500e-05_rb,0.38510e-05_rb,0.36169e-05_rb,0.19605e-05_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.15336e-05_rb,0.28792e-05_rb,0.35139e-05_rb,0.39460e-05_rb,0.42149e-05_rb, &
        & 0.43622e-05_rb,0.43664e-05_rb,0.42187e-05_rb,0.23523e-05_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.12444e-05_rb,0.18266e-05_rb,0.19784e-05_rb,0.19900e-05_rb,0.19390e-05_rb, &
        & 0.19235e-05_rb,0.17231e-05_rb,0.16995e-05_rb,0.68590e-06_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.12557e-05_rb,0.19427e-05_rb,0.21667e-05_rb,0.22465e-05_rb,0.22625e-05_rb, &
        & 0.22377e-05_rb,0.21596e-05_rb,0.18759e-05_rb,0.89490e-06_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.12608e-05_rb,0.20627e-05_rb,0.23694e-05_rb,0.25224e-05_rb,0.25961e-05_rb, &
        & 0.25892e-05_rb,0.25887e-05_rb,0.23400e-05_rb,0.11490e-05_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.12616e-05_rb,0.21916e-05_rb,0.25816e-05_rb,0.28156e-05_rb,0.29418e-05_rb, &
        & 0.29759e-05_rb,0.29788e-05_rb,0.27379e-05_rb,0.14416e-05_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.12590e-05_rb,0.23261e-05_rb,0.28013e-05_rb,0.31131e-05_rb,0.32990e-05_rb, &
        & 0.33976e-05_rb,0.33946e-05_rb,0.32600e-05_rb,0.17529e-05_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.10123e-05_rb,0.14681e-05_rb,0.15612e-05_rb,0.15411e-05_rb,0.14778e-05_rb, &
        & 0.14343e-05_rb,0.12992e-05_rb,0.13172e-05_rb,0.49290e-06_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.10245e-05_rb,0.15617e-05_rb,0.17149e-05_rb,0.17499e-05_rb,0.17312e-05_rb, &
        & 0.17144e-05_rb,0.15772e-05_rb,0.14382e-05_rb,0.63132e-06_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.10311e-05_rb,0.16600e-05_rb,0.18786e-05_rb,0.19738e-05_rb,0.20017e-05_rb, &
        & 0.19859e-05_rb,0.19416e-05_rb,0.17139e-05_rb,0.82278e-06_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.10334e-05_rb,0.17639e-05_rb,0.20493e-05_rb,0.22069e-05_rb,0.22845e-05_rb, &
        & 0.22769e-05_rb,0.22855e-05_rb,0.21115e-05_rb,0.10531e-05_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.10328e-05_rb,0.18738e-05_rb,0.22265e-05_rb,0.24495e-05_rb,0.25737e-05_rb, &
        & 0.26232e-05_rb,0.26173e-05_rb,0.24699e-05_rb,0.12950e-05_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.82261e-06_rb,0.11811e-05_rb,0.12403e-05_rb,0.12060e-05_rb,0.11493e-05_rb, &
        & 0.10907e-05_rb,0.98035e-06_rb,0.10980e-05_rb,0.37316e-06_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.83528e-06_rb,0.12595e-05_rb,0.13646e-05_rb,0.13745e-05_rb,0.13365e-05_rb, &
        & 0.13172e-05_rb,0.11846e-05_rb,0.11491e-05_rb,0.46158e-06_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.84255e-06_rb,0.13392e-05_rb,0.14968e-05_rb,0.15528e-05_rb,0.15573e-05_rb, &
        & 0.15339e-05_rb,0.14802e-05_rb,0.12811e-05_rb,0.60300e-06_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.84587e-06_rb,0.14245e-05_rb,0.16355e-05_rb,0.17381e-05_rb,0.17835e-05_rb, &
        & 0.17693e-05_rb,0.17683e-05_rb,0.16097e-05_rb,0.77730e-06_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.84627e-06_rb,0.15137e-05_rb,0.17779e-05_rb,0.19378e-05_rb,0.20192e-05_rb, &
        & 0.20387e-05_rb,0.20340e-05_rb,0.18636e-05_rb,0.96863e-06_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.67388e-06_rb,0.97942e-06_rb,0.10286e-05_rb,0.10023e-05_rb,0.95317e-06_rb, &
        & 0.90847e-06_rb,0.81776e-06_rb,0.89345e-06_rb,0.30408e-06_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.68407e-06_rb,0.10438e-05_rb,0.11334e-05_rb,0.11425e-05_rb,0.11096e-05_rb, &
        & 0.10886e-05_rb,0.98240e-06_rb,0.94798e-06_rb,0.38086e-06_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.69000e-06_rb,0.11106e-05_rb,0.12435e-05_rb,0.12890e-05_rb,0.12895e-05_rb, &
        & 0.12679e-05_rb,0.12202e-05_rb,0.10522e-05_rb,0.49623e-06_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.69276e-06_rb,0.11824e-05_rb,0.13564e-05_rb,0.14402e-05_rb,0.14763e-05_rb, &
        & 0.14628e-05_rb,0.14599e-05_rb,0.13328e-05_rb,0.63979e-06_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.69302e-06_rb,0.12555e-05_rb,0.14740e-05_rb,0.16049e-05_rb,0.16694e-05_rb, &
        & 0.16839e-05_rb,0.16772e-05_rb,0.15421e-05_rb,0.79514e-06_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.55214e-06_rb,0.81079e-06_rb,0.85249e-06_rb,0.83175e-06_rb,0.78783e-06_rb, &
        & 0.75281e-06_rb,0.67567e-06_rb,0.74075e-06_rb,0.24910e-06_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.56029e-06_rb,0.86391e-06_rb,0.94032e-06_rb,0.94758e-06_rb,0.91883e-06_rb, &
        & 0.89834e-06_rb,0.81189e-06_rb,0.77883e-06_rb,0.31399e-06_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.56504e-06_rb,0.92066e-06_rb,0.10308e-05_rb,0.10667e-05_rb,0.10655e-05_rb, &
        & 0.10460e-05_rb,0.10067e-05_rb,0.87150e-06_rb,0.40815e-06_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.56721e-06_rb,0.97985e-06_rb,0.11224e-05_rb,0.11915e-05_rb,0.12197e-05_rb, &
        & 0.12063e-05_rb,0.12014e-05_rb,0.10950e-05_rb,0.52630e-06_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.56742e-06_rb,0.10399e-05_rb,0.12195e-05_rb,0.13255e-05_rb,0.13771e-05_rb, &
        & 0.13876e-05_rb,0.13807e-05_rb,0.12724e-05_rb,0.65264e-06_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.45223e-06_rb,0.67006e-06_rb,0.70582e-06_rb,0.68873e-06_rb,0.65038e-06_rb, &
        & 0.62084e-06_rb,0.55442e-06_rb,0.60996e-06_rb,0.20428e-06_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.45880e-06_rb,0.71490e-06_rb,0.77859e-06_rb,0.78376e-06_rb,0.75834e-06_rb, &
        & 0.73987e-06_rb,0.66959e-06_rb,0.63711e-06_rb,0.25817e-06_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.46276e-06_rb,0.76231e-06_rb,0.85214e-06_rb,0.88098e-06_rb,0.87867e-06_rb, &
        & 0.86188e-06_rb,0.83033e-06_rb,0.72166e-06_rb,0.33548e-06_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.46442e-06_rb,0.81043e-06_rb,0.92714e-06_rb,0.98329e-06_rb,0.10050e-05_rb, &
        & 0.99247e-06_rb,0.98770e-06_rb,0.89931e-06_rb,0.43220e-06_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.46456e-06_rb,0.86048e-06_rb,0.10065e-05_rb,0.10920e-05_rb,0.11333e-05_rb, &
        & 0.11411e-05_rb,0.11342e-05_rb,0.10473e-05_rb,0.53527e-06_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.12335e-04_rb,0.15987e-04_rb,0.18565e-04_rb,0.20406e-04_rb,0.21636e-04_rb, &
        & 0.22077e-04_rb,0.21364e-04_rb,0.19544e-04_rb,0.14733e-04_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.12351e-04_rb,0.17014e-04_rb,0.20216e-04_rb,0.22480e-04_rb,0.24098e-04_rb, &
        & 0.24791e-04_rb,0.24379e-04_rb,0.22600e-04_rb,0.17292e-04_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.12364e-04_rb,0.18152e-04_rb,0.21907e-04_rb,0.24643e-04_rb,0.26659e-04_rb, &
        & 0.27630e-04_rb,0.27512e-04_rb,0.25730e-04_rb,0.20236e-04_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.12365e-04_rb,0.19320e-04_rb,0.23679e-04_rb,0.26885e-04_rb,0.29225e-04_rb, &
        & 0.30430e-04_rb,0.30461e-04_rb,0.28726e-04_rb,0.23208e-04_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.12355e-04_rb,0.20495e-04_rb,0.25505e-04_rb,0.29189e-04_rb,0.31816e-04_rb, &
        & 0.33308e-04_rb,0.33478e-04_rb,0.31898e-04_rb,0.26469e-04_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.10350e-04_rb,0.13753e-04_rb,0.16101e-04_rb,0.17726e-04_rb,0.18733e-04_rb, &
        & 0.19034e-04_rb,0.18319e-04_rb,0.16594e-04_rb,0.11961e-04_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.10365e-04_rb,0.14706e-04_rb,0.17542e-04_rb,0.19561e-04_rb,0.20895e-04_rb, &
        & 0.21420e-04_rb,0.21012e-04_rb,0.19409e-04_rb,0.14130e-04_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.10364e-04_rb,0.15733e-04_rb,0.19081e-04_rb,0.21505e-04_rb,0.23181e-04_rb, &
        & 0.23958e-04_rb,0.23844e-04_rb,0.22334e-04_rb,0.16750e-04_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.10354e-04_rb,0.16759e-04_rb,0.20670e-04_rb,0.23471e-04_rb,0.25440e-04_rb, &
        & 0.26474e-04_rb,0.26560e-04_rb,0.25188e-04_rb,0.19386e-04_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.10329e-04_rb,0.17823e-04_rb,0.22292e-04_rb,0.25488e-04_rb,0.27703e-04_rb, &
        & 0.28957e-04_rb,0.29205e-04_rb,0.27936e-04_rb,0.22026e-04_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.86408e-05_rb,0.11607e-04_rb,0.13624e-04_rb,0.14928e-04_rb,0.15622e-04_rb, &
        & 0.15768e-04_rb,0.15034e-04_rb,0.13540e-04_rb,0.91950e-05_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.86478e-05_rb,0.12463e-04_rb,0.14889e-04_rb,0.16508e-04_rb,0.17482e-04_rb, &
        & 0.17819e-04_rb,0.17439e-04_rb,0.15994e-04_rb,0.11026e-04_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.86444e-05_rb,0.13354e-04_rb,0.16234e-04_rb,0.18159e-04_rb,0.19394e-04_rb, &
        & 0.19962e-04_rb,0.19837e-04_rb,0.18485e-04_rb,0.13087e-04_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.86273e-05_rb,0.14262e-04_rb,0.17647e-04_rb,0.19921e-04_rb,0.21457e-04_rb, &
        & 0.22290e-04_rb,0.22338e-04_rb,0.21203e-04_rb,0.15397e-04_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.85960e-05_rb,0.15200e-04_rb,0.19031e-04_rb,0.21649e-04_rb,0.23415e-04_rb, &
        & 0.24453e-04_rb,0.24660e-04_rb,0.23666e-04_rb,0.17582e-04_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.71803e-05_rb,0.97101e-05_rb,0.11373e-04_rb,0.12321e-04_rb,0.12789e-04_rb, &
        & 0.12803e-04_rb,0.12062e-04_rb,0.10784e-04_rb,0.69624e-05_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.71855e-05_rb,0.10459e-04_rb,0.12456e-04_rb,0.13673e-04_rb,0.14371e-04_rb, &
        & 0.14569e-04_rb,0.14153e-04_rb,0.12788e-04_rb,0.84564e-05_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.71795e-05_rb,0.11229e-04_rb,0.13608e-04_rb,0.15091e-04_rb,0.16013e-04_rb, &
        & 0.16409e-04_rb,0.16232e-04_rb,0.15085e-04_rb,0.10120e-04_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.71584e-05_rb,0.12023e-04_rb,0.14800e-04_rb,0.16568e-04_rb,0.17732e-04_rb, &
        & 0.18336e-04_rb,0.18317e-04_rb,0.17347e-04_rb,0.11917e-04_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.71267e-05_rb,0.12845e-04_rb,0.16058e-04_rb,0.18162e-04_rb,0.19580e-04_rb, &
        & 0.20399e-04_rb,0.20543e-04_rb,0.19767e-04_rb,0.13958e-04_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.59442e-05_rb,0.80861e-05_rb,0.93878e-05_rb,0.10066e-04_rb,0.10362e-04_rb, &
        & 0.10281e-04_rb,0.95734e-05_rb,0.84778e-05_rb,0.52204e-05_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.59498e-05_rb,0.87281e-05_rb,0.10301e-04_rb,0.11210e-04_rb,0.11702e-04_rb, &
        & 0.11782e-04_rb,0.11315e-04_rb,0.10068e-04_rb,0.64302e-05_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.59430e-05_rb,0.93925e-05_rb,0.11277e-04_rb,0.12419e-04_rb,0.13099e-04_rb, &
        & 0.13351e-04_rb,0.13104e-04_rb,0.12095e-04_rb,0.77633e-05_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.59217e-05_rb,0.10078e-04_rb,0.12288e-04_rb,0.13686e-04_rb,0.14573e-04_rb, &
        & 0.14994e-04_rb,0.14908e-04_rb,0.14023e-04_rb,0.92337e-05_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.58935e-05_rb,0.10789e-04_rb,0.13390e-04_rb,0.15066e-04_rb,0.16172e-04_rb, &
        & 0.16788e-04_rb,0.16832e-04_rb,0.16140e-04_rb,0.10913e-04_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.49077e-05_rb,0.66672e-05_rb,0.76412e-05_rb,0.81181e-05_rb,0.82831e-05_rb, &
        & 0.81137e-05_rb,0.74381e-05_rb,0.64600e-05_rb,0.38540e-05_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.49136e-05_rb,0.72093e-05_rb,0.84100e-05_rb,0.90761e-05_rb,0.94026e-05_rb, &
        & 0.93849e-05_rb,0.88761e-05_rb,0.77852e-05_rb,0.48097e-05_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.49088e-05_rb,0.77732e-05_rb,0.92304e-05_rb,0.10089e-04_rb,0.10581e-04_rb, &
        & 0.10707e-04_rb,0.10412e-04_rb,0.94217e-05_rb,0.58741e-05_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.48919e-05_rb,0.83551e-05_rb,0.10086e-04_rb,0.11166e-04_rb,0.11831e-04_rb, &
        & 0.12095e-04_rb,0.11941e-04_rb,0.11106e-04_rb,0.70559e-05_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.48652e-05_rb,0.89412e-05_rb,0.10983e-04_rb,0.12289e-04_rb,0.13118e-04_rb, &
        & 0.13538e-04_rb,0.13477e-04_rb,0.12811e-04_rb,0.83378e-05_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.40428e-05_rb,0.54588e-05_rb,0.61687e-05_rb,0.64926e-05_rb,0.65568e-05_rb, &
        & 0.63259e-05_rb,0.56948e-05_rb,0.49607e-05_rb,0.28237e-05_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.40484e-05_rb,0.59019e-05_rb,0.68052e-05_rb,0.72891e-05_rb,0.74881e-05_rb, &
        & 0.73951e-05_rb,0.68736e-05_rb,0.59771e-05_rb,0.35700e-05_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.40443e-05_rb,0.63719e-05_rb,0.74936e-05_rb,0.81309e-05_rb,0.84699e-05_rb, &
        & 0.84975e-05_rb,0.81643e-05_rb,0.72224e-05_rb,0.43995e-05_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.40304e-05_rb,0.68636e-05_rb,0.82100e-05_rb,0.90329e-05_rb,0.95142e-05_rb, &
        & 0.96616e-05_rb,0.94576e-05_rb,0.86932e-05_rb,0.53385e-05_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.40097e-05_rb,0.73587e-05_rb,0.89665e-05_rb,0.99825e-05_rb,0.10592e-04_rb, &
        & 0.10860e-04_rb,0.10751e-04_rb,0.10114e-04_rb,0.63905e-05_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.33247e-05_rb,0.44437e-05_rb,0.49476e-05_rb,0.51623e-05_rb,0.51521e-05_rb, &
        & 0.48927e-05_rb,0.43130e-05_rb,0.36508e-05_rb,0.20433e-05_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.33302e-05_rb,0.48035e-05_rb,0.54757e-05_rb,0.58129e-05_rb,0.59219e-05_rb, &
        & 0.57796e-05_rb,0.52788e-05_rb,0.45081e-05_rb,0.26233e-05_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.33275e-05_rb,0.51944e-05_rb,0.60436e-05_rb,0.65119e-05_rb,0.67329e-05_rb, &
        & 0.66992e-05_rb,0.63336e-05_rb,0.54989e-05_rb,0.32839e-05_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.33167e-05_rb,0.56042e-05_rb,0.66425e-05_rb,0.72606e-05_rb,0.75965e-05_rb, &
        & 0.76619e-05_rb,0.74319e-05_rb,0.66922e-05_rb,0.40228e-05_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.33014e-05_rb,0.60212e-05_rb,0.72748e-05_rb,0.80496e-05_rb,0.84940e-05_rb, &
        & 0.86532e-05_rb,0.85181e-05_rb,0.78992e-05_rb,0.48672e-05_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.27310e-05_rb,0.35992e-05_rb,0.39468e-05_rb,0.40831e-05_rb,0.40161e-05_rb, &
        & 0.37266e-05_rb,0.32070e-05_rb,0.27591e-05_rb,0.14628e-05_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.27361e-05_rb,0.38904e-05_rb,0.43781e-05_rb,0.46075e-05_rb,0.46475e-05_rb, &
        & 0.44710e-05_rb,0.39993e-05_rb,0.33922e-05_rb,0.19203e-05_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.27349e-05_rb,0.42103e-05_rb,0.48448e-05_rb,0.51814e-05_rb,0.53150e-05_rb, &
        & 0.52403e-05_rb,0.48712e-05_rb,0.41669e-05_rb,0.24268e-05_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.27270e-05_rb,0.45504e-05_rb,0.53414e-05_rb,0.57970e-05_rb,0.60255e-05_rb, &
        & 0.60359e-05_rb,0.57915e-05_rb,0.51016e-05_rb,0.30070e-05_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.27146e-05_rb,0.48994e-05_rb,0.58660e-05_rb,0.64467e-05_rb,0.67684e-05_rb, &
        & 0.68529e-05_rb,0.66981e-05_rb,0.61431e-05_rb,0.36677e-05_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.22419e-05_rb,0.29237e-05_rb,0.31619e-05_rb,0.32419e-05_rb,0.31445e-05_rb, &
        & 0.28618e-05_rb,0.23945e-05_rb,0.20454e-05_rb,0.10493e-05_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.22458e-05_rb,0.31578e-05_rb,0.35134e-05_rb,0.36703e-05_rb,0.36665e-05_rb, &
        & 0.34835e-05_rb,0.30593e-05_rb,0.25559e-05_rb,0.14125e-05_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.22450e-05_rb,0.34192e-05_rb,0.38967e-05_rb,0.41378e-05_rb,0.42160e-05_rb, &
        & 0.41184e-05_rb,0.37724e-05_rb,0.31954e-05_rb,0.18099e-05_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.22393e-05_rb,0.36988e-05_rb,0.43039e-05_rb,0.46449e-05_rb,0.48027e-05_rb, &
        & 0.47756e-05_rb,0.45281e-05_rb,0.39284e-05_rb,0.22652e-05_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.22295e-05_rb,0.39888e-05_rb,0.47391e-05_rb,0.51827e-05_rb,0.54135e-05_rb, &
        & 0.54517e-05_rb,0.52919e-05_rb,0.47894e-05_rb,0.27859e-05_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.18412e-05_rb,0.24512e-05_rb,0.26550e-05_rb,0.27210e-05_rb,0.26438e-05_rb, &
        & 0.24068e-05_rb,0.20134e-05_rb,0.16980e-05_rb,0.87310e-06_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.18429e-05_rb,0.26525e-05_rb,0.29507e-05_rb,0.30835e-05_rb,0.30829e-05_rb, &
        & 0.29354e-05_rb,0.25810e-05_rb,0.21539e-05_rb,0.11715e-05_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.18405e-05_rb,0.28725e-05_rb,0.32735e-05_rb,0.34815e-05_rb,0.35507e-05_rb, &
        & 0.34733e-05_rb,0.31901e-05_rb,0.26957e-05_rb,0.15018e-05_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.18348e-05_rb,0.31060e-05_rb,0.36191e-05_rb,0.39123e-05_rb,0.40426e-05_rb, &
        & 0.40206e-05_rb,0.38184e-05_rb,0.33125e-05_rb,0.18804e-05_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.18263e-05_rb,0.33539e-05_rb,0.39862e-05_rb,0.43590e-05_rb,0.45516e-05_rb, &
        & 0.45841e-05_rb,0.44525e-05_rb,0.40371e-05_rb,0.23130e-05_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.15111e-05_rb,0.20512e-05_rb,0.22220e-05_rb,0.22783e-05_rb,0.22181e-05_rb, &
        & 0.20219e-05_rb,0.16992e-05_rb,0.14085e-05_rb,0.72438e-06_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.15111e-05_rb,0.22209e-05_rb,0.24719e-05_rb,0.25841e-05_rb,0.25877e-05_rb, &
        & 0.24700e-05_rb,0.21742e-05_rb,0.18087e-05_rb,0.96963e-06_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.15082e-05_rb,0.24051e-05_rb,0.27419e-05_rb,0.29223e-05_rb,0.29832e-05_rb, &
        & 0.29201e-05_rb,0.26833e-05_rb,0.22609e-05_rb,0.12444e-05_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.15029e-05_rb,0.26018e-05_rb,0.30364e-05_rb,0.32843e-05_rb,0.33933e-05_rb, &
        & 0.33730e-05_rb,0.32057e-05_rb,0.27875e-05_rb,0.15571e-05_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.14954e-05_rb,0.28127e-05_rb,0.33443e-05_rb,0.36551e-05_rb,0.38145e-05_rb, &
        & 0.38425e-05_rb,0.37317e-05_rb,0.33866e-05_rb,0.19133e-05_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.12390e-05_rb,0.17127e-05_rb,0.18553e-05_rb,0.19025e-05_rb,0.18568e-05_rb, &
        & 0.16968e-05_rb,0.14297e-05_rb,0.11813e-05_rb,0.59950e-06_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.12381e-05_rb,0.18537e-05_rb,0.20641e-05_rb,0.21603e-05_rb,0.21689e-05_rb, &
        & 0.20712e-05_rb,0.18216e-05_rb,0.15132e-05_rb,0.80096e-06_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.12352e-05_rb,0.20080e-05_rb,0.22929e-05_rb,0.24467e-05_rb,0.24992e-05_rb, &
        & 0.24434e-05_rb,0.22447e-05_rb,0.18839e-05_rb,0.10290e-05_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.12307e-05_rb,0.21754e-05_rb,0.25409e-05_rb,0.27489e-05_rb,0.28366e-05_rb, &
        & 0.28171e-05_rb,0.26776e-05_rb,0.23303e-05_rb,0.12853e-05_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.12246e-05_rb,0.23532e-05_rb,0.27977e-05_rb,0.30556e-05_rb,0.31857e-05_rb, &
        & 0.32077e-05_rb,0.31127e-05_rb,0.28260e-05_rb,0.15788e-05_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.43566e-04_rb,0.45402e-04_rb,0.48829e-04_rb,0.50194e-04_rb,0.49978e-04_rb, &
        & 0.48581e-04_rb,0.45902e-04_rb,0.40781e-04_rb,0.34170e-04_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.43974e-04_rb,0.47415e-04_rb,0.52012e-04_rb,0.54325e-04_rb,0.54921e-04_rb, &
        & 0.53975e-04_rb,0.51279e-04_rb,0.46650e-04_rb,0.40945e-04_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.44333e-04_rb,0.49545e-04_rb,0.55453e-04_rb,0.58909e-04_rb,0.60094e-04_rb, &
        & 0.59376e-04_rb,0.56940e-04_rb,0.52843e-04_rb,0.48266e-04_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.44647e-04_rb,0.51771e-04_rb,0.59155e-04_rb,0.63607e-04_rb,0.65306e-04_rb, &
        & 0.64935e-04_rb,0.62961e-04_rb,0.59262e-04_rb,0.55812e-04_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.44938e-04_rb,0.54202e-04_rb,0.63063e-04_rb,0.68229e-04_rb,0.70615e-04_rb, &
        & 0.70683e-04_rb,0.69315e-04_rb,0.65992e-04_rb,0.63855e-04_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.37849e-04_rb,0.40606e-04_rb,0.43836e-04_rb,0.45171e-04_rb,0.45318e-04_rb, &
        & 0.44251e-04_rb,0.41853e-04_rb,0.37247e-04_rb,0.29303e-04_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.38202e-04_rb,0.42545e-04_rb,0.46901e-04_rb,0.49156e-04_rb,0.50002e-04_rb, &
        & 0.49256e-04_rb,0.46845e-04_rb,0.42755e-04_rb,0.35187e-04_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.38525e-04_rb,0.44570e-04_rb,0.50219e-04_rb,0.53465e-04_rb,0.54833e-04_rb, &
        & 0.54348e-04_rb,0.52176e-04_rb,0.48578e-04_rb,0.41697e-04_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.38828e-04_rb,0.46787e-04_rb,0.53767e-04_rb,0.57818e-04_rb,0.59705e-04_rb, &
        & 0.59611e-04_rb,0.57787e-04_rb,0.54523e-04_rb,0.48350e-04_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.39076e-04_rb,0.49105e-04_rb,0.57380e-04_rb,0.62144e-04_rb,0.64743e-04_rb, &
        & 0.65049e-04_rb,0.63700e-04_rb,0.60747e-04_rb,0.55348e-04_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.32392e-04_rb,0.35504e-04_rb,0.38232e-04_rb,0.39483e-04_rb,0.39809e-04_rb, &
        & 0.39031e-04_rb,0.36901e-04_rb,0.32585e-04_rb,0.23735e-04_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.32715e-04_rb,0.37303e-04_rb,0.41039e-04_rb,0.43202e-04_rb,0.44102e-04_rb, &
        & 0.43582e-04_rb,0.41484e-04_rb,0.37593e-04_rb,0.28689e-04_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.33011e-04_rb,0.39216e-04_rb,0.44127e-04_rb,0.47180e-04_rb,0.48515e-04_rb, &
        & 0.48269e-04_rb,0.46352e-04_rb,0.42866e-04_rb,0.34062e-04_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.33271e-04_rb,0.41283e-04_rb,0.47451e-04_rb,0.51158e-04_rb,0.53031e-04_rb, &
        & 0.53191e-04_rb,0.51599e-04_rb,0.48489e-04_rb,0.40122e-04_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.33495e-04_rb,0.43439e-04_rb,0.50779e-04_rb,0.55143e-04_rb,0.57687e-04_rb, &
        & 0.58208e-04_rb,0.56935e-04_rb,0.54138e-04_rb,0.46109e-04_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.27415e-04_rb,0.30543e-04_rb,0.32764e-04_rb,0.33940e-04_rb,0.34340e-04_rb, &
        & 0.33775e-04_rb,0.31812e-04_rb,0.27722e-04_rb,0.18770e-04_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.27715e-04_rb,0.32169e-04_rb,0.35304e-04_rb,0.37332e-04_rb,0.38224e-04_rb, &
        & 0.37868e-04_rb,0.35986e-04_rb,0.32265e-04_rb,0.22928e-04_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.27982e-04_rb,0.33932e-04_rb,0.38141e-04_rb,0.40925e-04_rb,0.42223e-04_rb, &
        & 0.42122e-04_rb,0.40409e-04_rb,0.37026e-04_rb,0.27474e-04_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.28215e-04_rb,0.35808e-04_rb,0.41184e-04_rb,0.44532e-04_rb,0.46320e-04_rb, &
        & 0.46554e-04_rb,0.45091e-04_rb,0.41998e-04_rb,0.32462e-04_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.28437e-04_rb,0.37809e-04_rb,0.44202e-04_rb,0.48199e-04_rb,0.50577e-04_rb, &
        & 0.51211e-04_rb,0.50186e-04_rb,0.47427e-04_rb,0.38088e-04_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.23010e-04_rb,0.25978e-04_rb,0.27838e-04_rb,0.28899e-04_rb,0.29286e-04_rb, &
        & 0.28721e-04_rb,0.26945e-04_rb,0.23196e-04_rb,0.14687e-04_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.23283e-04_rb,0.27434e-04_rb,0.30115e-04_rb,0.31931e-04_rb,0.32743e-04_rb, &
        & 0.32398e-04_rb,0.30674e-04_rb,0.27269e-04_rb,0.18140e-04_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.23522e-04_rb,0.29002e-04_rb,0.32686e-04_rb,0.35125e-04_rb,0.36325e-04_rb, &
        & 0.36232e-04_rb,0.34626e-04_rb,0.31511e-04_rb,0.21987e-04_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.23738e-04_rb,0.30687e-04_rb,0.35404e-04_rb,0.38366e-04_rb,0.40013e-04_rb, &
        & 0.40198e-04_rb,0.38830e-04_rb,0.35973e-04_rb,0.26236e-04_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.23936e-04_rb,0.32534e-04_rb,0.38093e-04_rb,0.41748e-04_rb,0.43875e-04_rb, &
        & 0.44440e-04_rb,0.43519e-04_rb,0.40954e-04_rb,0.31086e-04_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.19172e-04_rb,0.21867e-04_rb,0.23409e-04_rb,0.24321e-04_rb,0.24529e-04_rb, &
        & 0.23927e-04_rb,0.22333e-04_rb,0.18955e-04_rb,0.11218e-04_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.19419e-04_rb,0.23150e-04_rb,0.25412e-04_rb,0.26974e-04_rb,0.27574e-04_rb, &
        & 0.27129e-04_rb,0.25625e-04_rb,0.22590e-04_rb,0.14082e-04_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.19636e-04_rb,0.24533e-04_rb,0.27708e-04_rb,0.29785e-04_rb,0.30733e-04_rb, &
        & 0.30517e-04_rb,0.29114e-04_rb,0.26329e-04_rb,0.17293e-04_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.19836e-04_rb,0.26054e-04_rb,0.30093e-04_rb,0.32660e-04_rb,0.34002e-04_rb, &
        & 0.34049e-04_rb,0.32819e-04_rb,0.30271e-04_rb,0.20869e-04_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.20002e-04_rb,0.27734e-04_rb,0.32461e-04_rb,0.35638e-04_rb,0.37355e-04_rb, &
        & 0.37686e-04_rb,0.36788e-04_rb,0.34519e-04_rb,0.24769e-04_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.15886e-04_rb,0.18280e-04_rb,0.19557e-04_rb,0.20235e-04_rb,0.20297e-04_rb, &
        & 0.19642e-04_rb,0.18202e-04_rb,0.15130e-04_rb,0.84335e-05_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.16105e-04_rb,0.19408e-04_rb,0.21322e-04_rb,0.22532e-04_rb,0.22888e-04_rb, &
        & 0.22409e-04_rb,0.21064e-04_rb,0.18396e-04_rb,0.10766e-04_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.16304e-04_rb,0.20629e-04_rb,0.23329e-04_rb,0.24999e-04_rb,0.25612e-04_rb, &
        & 0.25350e-04_rb,0.24115e-04_rb,0.21685e-04_rb,0.13408e-04_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.16482e-04_rb,0.21985e-04_rb,0.25409e-04_rb,0.27506e-04_rb,0.28475e-04_rb, &
        & 0.28447e-04_rb,0.27347e-04_rb,0.25116e-04_rb,0.16377e-04_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.16621e-04_rb,0.23483e-04_rb,0.27507e-04_rb,0.30111e-04_rb,0.31449e-04_rb, &
        & 0.31651e-04_rb,0.30823e-04_rb,0.28853e-04_rb,0.19647e-04_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.13104e-04_rb,0.15202e-04_rb,0.16227e-04_rb,0.16675e-04_rb,0.16618e-04_rb, &
        & 0.15949e-04_rb,0.14642e-04_rb,0.11862e-04_rb,0.62741e-05_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.13296e-04_rb,0.16183e-04_rb,0.17762e-04_rb,0.18645e-04_rb,0.18799e-04_rb, &
        & 0.18302e-04_rb,0.17108e-04_rb,0.14749e-04_rb,0.81431e-05_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.13475e-04_rb,0.17250e-04_rb,0.19501e-04_rb,0.20748e-04_rb,0.21144e-04_rb, &
        & 0.20825e-04_rb,0.19726e-04_rb,0.17623e-04_rb,0.10291e-04_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.13631e-04_rb,0.18462e-04_rb,0.21319e-04_rb,0.22916e-04_rb,0.23622e-04_rb, &
        & 0.23509e-04_rb,0.22509e-04_rb,0.20604e-04_rb,0.12727e-04_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.13752e-04_rb,0.19773e-04_rb,0.23162e-04_rb,0.25182e-04_rb,0.26210e-04_rb, &
        & 0.26298e-04_rb,0.25526e-04_rb,0.23837e-04_rb,0.15433e-04_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.10771e-04_rb,0.12580e-04_rb,0.13349e-04_rb,0.13617e-04_rb,0.13459e-04_rb, &
        & 0.12816e-04_rb,0.11635e-04_rb,0.90636e-05_rb,0.45849e-05_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.10937e-04_rb,0.13431e-04_rb,0.14667e-04_rb,0.15270e-04_rb,0.15289e-04_rb, &
        & 0.14786e-04_rb,0.13720e-04_rb,0.11614e-04_rb,0.60707e-05_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.11095e-04_rb,0.14367e-04_rb,0.16153e-04_rb,0.17055e-04_rb,0.17287e-04_rb, &
        & 0.16925e-04_rb,0.15948e-04_rb,0.14098e-04_rb,0.78037e-05_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.11230e-04_rb,0.15436e-04_rb,0.17715e-04_rb,0.18921e-04_rb,0.19404e-04_rb, &
        & 0.19208e-04_rb,0.18318e-04_rb,0.16668e-04_rb,0.97810e-05_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.11337e-04_rb,0.16572e-04_rb,0.19310e-04_rb,0.20882e-04_rb,0.21623e-04_rb, &
        & 0.21621e-04_rb,0.20902e-04_rb,0.19439e-04_rb,0.11994e-04_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.88397e-05_rb,0.10407e-04_rb,0.10992e-04_rb,0.11130e-04_rb,0.10914e-04_rb, &
        & 0.10319e-04_rb,0.92746e-05_rb,0.69230e-05_rb,0.33878e-05_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.89820e-05_rb,0.11148e-04_rb,0.12114e-04_rb,0.12509e-04_rb,0.12455e-04_rb, &
        & 0.11973e-04_rb,0.11026e-04_rb,0.91303e-05_rb,0.45777e-05_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.91201e-05_rb,0.11975e-04_rb,0.13369e-04_rb,0.14032e-04_rb,0.14150e-04_rb, &
        & 0.13774e-04_rb,0.12899e-04_rb,0.11289e-04_rb,0.59718e-05_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.92338e-05_rb,0.12910e-04_rb,0.14708e-04_rb,0.15633e-04_rb,0.15947e-04_rb, &
        & 0.15711e-04_rb,0.14916e-04_rb,0.13483e-04_rb,0.75778e-05_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.93270e-05_rb,0.13899e-04_rb,0.16089e-04_rb,0.17313e-04_rb,0.17838e-04_rb, &
        & 0.17779e-04_rb,0.17126e-04_rb,0.15852e-04_rb,0.93849e-05_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.72957e-05_rb,0.88518e-05_rb,0.94180e-05_rb,0.95477e-05_rb,0.93627e-05_rb, &
        & 0.88681e-05_rb,0.79846e-05_rb,0.60665e-05_rb,0.28938e-05_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.74164e-05_rb,0.95244e-05_rb,0.10413e-04_rb,0.10760e-04_rb,0.10725e-04_rb, &
        & 0.10300e-04_rb,0.94914e-05_rb,0.79265e-05_rb,0.39145e-05_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.75267e-05_rb,0.10290e-04_rb,0.11516e-04_rb,0.12091e-04_rb,0.12186e-04_rb, &
        & 0.11866e-04_rb,0.11116e-04_rb,0.97843e-05_rb,0.51152e-05_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.76165e-05_rb,0.11127e-04_rb,0.12682e-04_rb,0.13483e-04_rb,0.13746e-04_rb, &
        & 0.13546e-04_rb,0.12885e-04_rb,0.11704e-04_rb,0.64876e-05_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.76866e-05_rb,0.11972e-04_rb,0.13896e-04_rb,0.14938e-04_rb,0.15405e-04_rb, &
        & 0.15374e-04_rb,0.14838e-04_rb,0.13790e-04_rb,0.80319e-05_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.60137e-05_rb,0.75193e-05_rb,0.80348e-05_rb,0.81538e-05_rb,0.80064e-05_rb, &
        & 0.75822e-05_rb,0.68354e-05_rb,0.52502e-05_rb,0.24674e-05_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.61150e-05_rb,0.81325e-05_rb,0.89089e-05_rb,0.92208e-05_rb,0.91831e-05_rb, &
        & 0.88171e-05_rb,0.81283e-05_rb,0.68396e-05_rb,0.33429e-05_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.62020e-05_rb,0.88113e-05_rb,0.98818e-05_rb,0.10369e-04_rb,0.10440e-04_rb, &
        & 0.10164e-04_rb,0.95355e-05_rb,0.84372e-05_rb,0.43683e-05_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.62729e-05_rb,0.95382e-05_rb,0.10890e-04_rb,0.11569e-04_rb,0.11793e-04_rb, &
        & 0.11630e-04_rb,0.11082e-04_rb,0.10108e-04_rb,0.55358e-05_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.63199e-05_rb,0.10271e-04_rb,0.11934e-04_rb,0.12838e-04_rb,0.13254e-04_rb, &
        & 0.13231e-04_rb,0.12796e-04_rb,0.11934e-04_rb,0.68492e-05_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.49517e-05_rb,0.63774e-05_rb,0.68261e-05_rb,0.69411e-05_rb,0.68151e-05_rb, &
        & 0.64513e-05_rb,0.58220e-05_rb,0.45056e-05_rb,0.20980e-05_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.50331e-05_rb,0.69207e-05_rb,0.75990e-05_rb,0.78660e-05_rb,0.78231e-05_rb, &
        & 0.75119e-05_rb,0.69333e-05_rb,0.58688e-05_rb,0.28426e-05_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.51036e-05_rb,0.75155e-05_rb,0.84393e-05_rb,0.88490e-05_rb,0.89047e-05_rb, &
        & 0.86720e-05_rb,0.81449e-05_rb,0.72410e-05_rb,0.37114e-05_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.51548e-05_rb,0.81445e-05_rb,0.93105e-05_rb,0.98793e-05_rb,0.10078e-04_rb, &
        & 0.99535e-05_rb,0.94951e-05_rb,0.86973e-05_rb,0.47002e-05_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.51867e-05_rb,0.87857e-05_rb,0.10211e-04_rb,0.10992e-04_rb,0.11352e-04_rb, &
        & 0.11351e-04_rb,0.10990e-04_rb,0.10277e-04_rb,0.58153e-05_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.18292e-03_rb,0.16357e-03_rb,0.15341e-03_rb,0.14585e-03_rb,0.13797e-03_rb, &
        & 0.12826e-03_rb,0.11477e-03_rb,0.99322e-04_rb,0.99311e-04_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.18277e-03_rb,0.16502e-03_rb,0.15805e-03_rb,0.15290e-03_rb,0.14696e-03_rb, &
        & 0.13850e-03_rb,0.12664e-03_rb,0.11419e-03_rb,0.11887e-03_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.18261e-03_rb,0.16694e-03_rb,0.16322e-03_rb,0.16061e-03_rb,0.15658e-03_rb, &
        & 0.14997e-03_rb,0.14010e-03_rb,0.13154e-03_rb,0.14123e-03_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.18244e-03_rb,0.16952e-03_rb,0.16897e-03_rb,0.16909e-03_rb,0.16740e-03_rb, &
        & 0.16268e-03_rb,0.15556e-03_rb,0.15163e-03_rb,0.16610e-03_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.18218e-03_rb,0.17247e-03_rb,0.17521e-03_rb,0.17844e-03_rb,0.17911e-03_rb, &
        & 0.17696e-03_rb,0.17241e-03_rb,0.17370e-03_rb,0.19290e-03_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.17473e-03_rb,0.15735e-03_rb,0.14910e-03_rb,0.14227e-03_rb,0.13471e-03_rb, &
        & 0.12515e-03_rb,0.11200e-03_rb,0.94986e-04_rb,0.89520e-04_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.17467e-03_rb,0.15907e-03_rb,0.15401e-03_rb,0.14953e-03_rb,0.14402e-03_rb, &
        & 0.13581e-03_rb,0.12401e-03_rb,0.10893e-03_rb,0.10811e-03_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.17463e-03_rb,0.16142e-03_rb,0.15942e-03_rb,0.15773e-03_rb,0.15417e-03_rb, &
        & 0.14769e-03_rb,0.13747e-03_rb,0.12528e-03_rb,0.12924e-03_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.17449e-03_rb,0.16426e-03_rb,0.16549e-03_rb,0.16671e-03_rb,0.16541e-03_rb, &
        & 0.16085e-03_rb,0.15263e-03_rb,0.14389e-03_rb,0.15270e-03_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.17439e-03_rb,0.16753e-03_rb,0.17238e-03_rb,0.17658e-03_rb,0.17759e-03_rb, &
        & 0.17537e-03_rb,0.16910e-03_rb,0.16442e-03_rb,0.17796e-03_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.16429e-03_rb,0.14895e-03_rb,0.14201e-03_rb,0.13557e-03_rb,0.12797e-03_rb, &
        & 0.11829e-03_rb,0.10548e-03_rb,0.88188e-04_rb,0.77280e-04_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.16436e-03_rb,0.15092e-03_rb,0.14708e-03_rb,0.14277e-03_rb,0.13731e-03_rb, &
        & 0.12899e-03_rb,0.11719e-03_rb,0.10108e-03_rb,0.94071e-04_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.16438e-03_rb,0.15344e-03_rb,0.15259e-03_rb,0.15105e-03_rb,0.14760e-03_rb, &
        & 0.14078e-03_rb,0.13037e-03_rb,0.11602e-03_rb,0.11331e-03_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.16440e-03_rb,0.15651e-03_rb,0.15877e-03_rb,0.16021e-03_rb,0.15883e-03_rb, &
        & 0.15394e-03_rb,0.14519e-03_rb,0.13303e-03_rb,0.13463e-03_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.16444e-03_rb,0.16001e-03_rb,0.16595e-03_rb,0.17023e-03_rb,0.17105e-03_rb, &
        & 0.16855e-03_rb,0.16116e-03_rb,0.15159e-03_rb,0.15764e-03_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.15195e-03_rb,0.13883e-03_rb,0.13294e-03_rb,0.12699e-03_rb,0.11928e-03_rb, &
        & 0.10948e-03_rb,0.97355e-04_rb,0.80985e-04_rb,0.65303e-04_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.15212e-03_rb,0.14097e-03_rb,0.13802e-03_rb,0.13404e-03_rb,0.12836e-03_rb, &
        & 0.11996e-03_rb,0.10855e-03_rb,0.92884e-04_rb,0.80285e-04_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.15229e-03_rb,0.14364e-03_rb,0.14356e-03_rb,0.14222e-03_rb,0.13844e-03_rb, &
        & 0.13147e-03_rb,0.12124e-03_rb,0.10661e-03_rb,0.97473e-04_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.15247e-03_rb,0.14694e-03_rb,0.14987e-03_rb,0.15132e-03_rb,0.14947e-03_rb, &
        & 0.14428e-03_rb,0.13562e-03_rb,0.12218e-03_rb,0.11647e-03_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.15268e-03_rb,0.15064e-03_rb,0.15719e-03_rb,0.16126e-03_rb,0.16162e-03_rb, &
        & 0.15869e-03_rb,0.15101e-03_rb,0.13904e-03_rb,0.13733e-03_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.13846e-03_rb,0.12761e-03_rb,0.12265e-03_rb,0.11713e-03_rb,0.10945e-03_rb, &
        & 0.10011e-03_rb,0.88713e-04_rb,0.73647e-04_rb,0.54834e-04_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.13872e-03_rb,0.12987e-03_rb,0.12766e-03_rb,0.12392e-03_rb,0.11814e-03_rb, &
        & 0.11013e-03_rb,0.99375e-04_rb,0.84759e-04_rb,0.68007e-04_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.13899e-03_rb,0.13272e-03_rb,0.13319e-03_rb,0.13187e-03_rb,0.12795e-03_rb, &
        & 0.12108e-03_rb,0.11145e-03_rb,0.97499e-04_rb,0.83096e-04_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.13930e-03_rb,0.13617e-03_rb,0.13955e-03_rb,0.14081e-03_rb,0.13862e-03_rb, &
        & 0.13340e-03_rb,0.12522e-03_rb,0.11183e-03_rb,0.99891e-04_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.13961e-03_rb,0.13997e-03_rb,0.14692e-03_rb,0.15049e-03_rb,0.15045e-03_rb, &
        & 0.14736e-03_rb,0.14024e-03_rb,0.12733e-03_rb,0.11865e-03_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.12419e-03_rb,0.11551e-03_rb,0.11132e-03_rb,0.10604e-03_rb,0.98912e-04_rb, &
        & 0.90174e-04_rb,0.79553e-04_rb,0.65781e-04_rb,0.45228e-04_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.12454e-03_rb,0.11786e-03_rb,0.11619e-03_rb,0.11259e-03_rb,0.10704e-03_rb, &
        & 0.99552e-04_rb,0.89546e-04_rb,0.76061e-04_rb,0.56748e-04_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.12490e-03_rb,0.12081e-03_rb,0.12161e-03_rb,0.12024e-03_rb,0.11632e-03_rb, &
        & 0.10988e-03_rb,0.10081e-03_rb,0.87950e-04_rb,0.69901e-04_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.12528e-03_rb,0.12425e-03_rb,0.12796e-03_rb,0.12887e-03_rb,0.12649e-03_rb, &
        & 0.12150e-03_rb,0.11381e-03_rb,0.10130e-03_rb,0.84575e-04_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.12568e-03_rb,0.12811e-03_rb,0.13519e-03_rb,0.13816e-03_rb,0.13780e-03_rb, &
        & 0.13476e-03_rb,0.12819e-03_rb,0.11553e-03_rb,0.10091e-03_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.10978e-03_rb,0.10308e-03_rb,0.99508e-04_rb,0.94571e-04_rb,0.88120e-04_rb, &
        & 0.80226e-04_rb,0.70492e-04_rb,0.57993e-04_rb,0.36699e-04_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.11018e-03_rb,0.10544e-03_rb,0.10415e-03_rb,0.10080e-03_rb,0.95769e-04_rb, &
        & 0.88865e-04_rb,0.79698e-04_rb,0.67450e-04_rb,0.46683e-04_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.11058e-03_rb,0.10837e-03_rb,0.10936e-03_rb,0.10807e-03_rb,0.10453e-03_rb, &
        & 0.98497e-04_rb,0.90104e-04_rb,0.78443e-04_rb,0.58212e-04_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.11104e-03_rb,0.11179e-03_rb,0.11551e-03_rb,0.11629e-03_rb,0.11409e-03_rb, &
        & 0.10934e-03_rb,0.10217e-03_rb,0.90913e-04_rb,0.71131e-04_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.11149e-03_rb,0.11560e-03_rb,0.12257e-03_rb,0.12521e-03_rb,0.12472e-03_rb, &
        & 0.12180e-03_rb,0.11572e-03_rb,0.10421e-03_rb,0.85542e-04_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.95759e-04_rb,0.90773e-04_rb,0.87806e-04_rb,0.83290e-04_rb,0.77515e-04_rb, &
        & 0.70528e-04_rb,0.61795e-04_rb,0.50465e-04_rb,0.29380e-04_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.96189e-04_rb,0.93120e-04_rb,0.92132e-04_rb,0.89103e-04_rb,0.84653e-04_rb, &
        & 0.78456e-04_rb,0.70159e-04_rb,0.59177e-04_rb,0.37867e-04_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.96665e-04_rb,0.96018e-04_rb,0.97008e-04_rb,0.95975e-04_rb,0.92760e-04_rb, &
        & 0.87379e-04_rb,0.79687e-04_rb,0.69248e-04_rb,0.47855e-04_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.97168e-04_rb,0.99322e-04_rb,0.10289e-03_rb,0.10373e-03_rb,0.10169e-03_rb, &
        & 0.97397e-04_rb,0.90796e-04_rb,0.80805e-04_rb,0.59240e-04_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.97641e-04_rb,0.10302e-03_rb,0.10967e-03_rb,0.11218e-03_rb,0.11166e-03_rb, &
        & 0.10896e-03_rb,0.10343e-03_rb,0.93210e-04_rb,0.71998e-04_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.82482e-04_rb,0.78941e-04_rb,0.76398e-04_rb,0.72429e-04_rb,0.67369e-04_rb, &
        & 0.61188e-04_rb,0.53486e-04_rb,0.43219e-04_rb,0.23144e-04_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.82941e-04_rb,0.81194e-04_rb,0.80391e-04_rb,0.77783e-04_rb,0.73924e-04_rb, &
        & 0.68383e-04_rb,0.61055e-04_rb,0.51145e-04_rb,0.30235e-04_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.83462e-04_rb,0.83998e-04_rb,0.84977e-04_rb,0.84161e-04_rb,0.81323e-04_rb, &
        & 0.76538e-04_rb,0.69739e-04_rb,0.60350e-04_rb,0.38714e-04_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.83976e-04_rb,0.87153e-04_rb,0.90583e-04_rb,0.91413e-04_rb,0.89589e-04_rb, &
        & 0.85741e-04_rb,0.79900e-04_rb,0.70947e-04_rb,0.48553e-04_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.84461e-04_rb,0.90734e-04_rb,0.96938e-04_rb,0.99296e-04_rb,0.98864e-04_rb, &
        & 0.96399e-04_rb,0.91511e-04_rb,0.82490e-04_rb,0.59777e-04_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.70371e-04_rb,0.68110e-04_rb,0.65986e-04_rb,0.62636e-04_rb,0.58303e-04_rb, &
        & 0.52961e-04_rb,0.46122e-04_rb,0.37036e-04_rb,0.18341e-04_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.70853e-04_rb,0.70288e-04_rb,0.69669e-04_rb,0.67543e-04_rb,0.64271e-04_rb, &
        & 0.59496e-04_rb,0.53052e-04_rb,0.44224e-04_rb,0.24306e-04_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.71373e-04_rb,0.72956e-04_rb,0.74034e-04_rb,0.73423e-04_rb,0.71076e-04_rb, &
        & 0.66934e-04_rb,0.61048e-04_rb,0.52623e-04_rb,0.31537e-04_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.71862e-04_rb,0.75946e-04_rb,0.79283e-04_rb,0.80149e-04_rb,0.78712e-04_rb, &
        & 0.75412e-04_rb,0.70399e-04_rb,0.62323e-04_rb,0.40041e-04_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.72376e-04_rb,0.79405e-04_rb,0.85220e-04_rb,0.87544e-04_rb,0.87332e-04_rb, &
        & 0.85268e-04_rb,0.81104e-04_rb,0.73085e-04_rb,0.49845e-04_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.59776e-04_rb,0.59155e-04_rb,0.57996e-04_rb,0.55642e-04_rb,0.52323e-04_rb, &
        & 0.47930e-04_rb,0.42036e-04_rb,0.34179e-04_rb,0.16514e-04_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.60275e-04_rb,0.61436e-04_rb,0.61654e-04_rb,0.60490e-04_rb,0.58055e-04_rb, &
        & 0.54242e-04_rb,0.48750e-04_rb,0.41144e-04_rb,0.21987e-04_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.60746e-04_rb,0.64075e-04_rb,0.66078e-04_rb,0.66233e-04_rb,0.64629e-04_rb, &
        & 0.61455e-04_rb,0.56611e-04_rb,0.49290e-04_rb,0.28648e-04_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.61244e-04_rb,0.67066e-04_rb,0.71217e-04_rb,0.72696e-04_rb,0.72050e-04_rb, &
        & 0.69816e-04_rb,0.65797e-04_rb,0.58590e-04_rb,0.36515e-04_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.61769e-04_rb,0.70693e-04_rb,0.76979e-04_rb,0.79872e-04_rb,0.80561e-04_rb, &
        & 0.79478e-04_rb,0.76089e-04_rb,0.68898e-04_rb,0.45628e-04_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.50518e-04_rb,0.51187e-04_rb,0.50822e-04_rb,0.49284e-04_rb,0.46769e-04_rb, &
        & 0.43097e-04_rb,0.38110e-04_rb,0.31379e-04_rb,0.14783e-04_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.50964e-04_rb,0.53463e-04_rb,0.54457e-04_rb,0.54020e-04_rb,0.52266e-04_rb, &
        & 0.49174e-04_rb,0.44595e-04_rb,0.38042e-04_rb,0.19796e-04_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.51421e-04_rb,0.56033e-04_rb,0.58762e-04_rb,0.59481e-04_rb,0.58578e-04_rb, &
        & 0.56207e-04_rb,0.52260e-04_rb,0.45820e-04_rb,0.25919e-04_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.51920e-04_rb,0.59121e-04_rb,0.63731e-04_rb,0.65674e-04_rb,0.65806e-04_rb, &
        & 0.64464e-04_rb,0.61120e-04_rb,0.54677e-04_rb,0.33164e-04_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.52396e-04_rb,0.62780e-04_rb,0.69274e-04_rb,0.72637e-04_rb,0.74145e-04_rb, &
        & 0.73831e-04_rb,0.71039e-04_rb,0.64514e-04_rb,0.41597e-04_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.42486e-04_rb,0.44174e-04_rb,0.44463e-04_rb,0.43555e-04_rb,0.41568e-04_rb, &
        & 0.38563e-04_rb,0.34384e-04_rb,0.28625e-04_rb,0.13171e-04_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.42904e-04_rb,0.46376e-04_rb,0.47979e-04_rb,0.48065e-04_rb,0.46867e-04_rb, &
        & 0.44391e-04_rb,0.40615e-04_rb,0.34938e-04_rb,0.17746e-04_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.43349e-04_rb,0.48965e-04_rb,0.52137e-04_rb,0.53262e-04_rb,0.52962e-04_rb, &
        & 0.51246e-04_rb,0.47996e-04_rb,0.42282e-04_rb,0.23342e-04_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.43790e-04_rb,0.52076e-04_rb,0.56889e-04_rb,0.59186e-04_rb,0.60002e-04_rb, &
        & 0.59254e-04_rb,0.56460e-04_rb,0.50705e-04_rb,0.29999e-04_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.44254e-04_rb,0.55629e-04_rb,0.62209e-04_rb,0.65993e-04_rb,0.68105e-04_rb, &
        & 0.68270e-04_rb,0.65941e-04_rb,0.60145e-04_rb,0.37728e-04_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.42864e-03_rb,0.37538e-03_rb,0.32907e-03_rb,0.29701e-03_rb,0.26399e-03_rb, &
        & 0.23587e-03_rb,0.21519e-03_rb,0.20736e-03_rb,0.23690e-03_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.42770e-03_rb,0.37463e-03_rb,0.33468e-03_rb,0.30443e-03_rb,0.27635e-03_rb, &
        & 0.25424e-03_rb,0.24068e-03_rb,0.24289e-03_rb,0.27885e-03_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.42742e-03_rb,0.37444e-03_rb,0.34109e-03_rb,0.31378e-03_rb,0.29265e-03_rb, &
        & 0.27618e-03_rb,0.26729e-03_rb,0.27967e-03_rb,0.32090e-03_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.42756e-03_rb,0.37461e-03_rb,0.34714e-03_rb,0.32647e-03_rb,0.30721e-03_rb, &
        & 0.29888e-03_rb,0.29503e-03_rb,0.31853e-03_rb,0.36629e-03_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.42817e-03_rb,0.37676e-03_rb,0.35506e-03_rb,0.34027e-03_rb,0.32593e-03_rb, &
        & 0.32117e-03_rb,0.33159e-03_rb,0.36650e-03_rb,0.42307e-03_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.44647e-03_rb,0.39099e-03_rb,0.34399e-03_rb,0.31315e-03_rb,0.27681e-03_rb, &
        & 0.24757e-03_rb,0.21977e-03_rb,0.20260e-03_rb,0.22465e-03_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.44675e-03_rb,0.39126e-03_rb,0.35094e-03_rb,0.32069e-03_rb,0.29009e-03_rb, &
        & 0.26696e-03_rb,0.24338e-03_rb,0.23431e-03_rb,0.26448e-03_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.44728e-03_rb,0.39176e-03_rb,0.35848e-03_rb,0.33002e-03_rb,0.30752e-03_rb, &
        & 0.28763e-03_rb,0.26835e-03_rb,0.26923e-03_rb,0.30805e-03_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.44805e-03_rb,0.39321e-03_rb,0.36547e-03_rb,0.34473e-03_rb,0.32431e-03_rb, &
        & 0.30833e-03_rb,0.29659e-03_rb,0.31021e-03_rb,0.35789e-03_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.44833e-03_rb,0.39641e-03_rb,0.37373e-03_rb,0.35955e-03_rb,0.34286e-03_rb, &
        & 0.33059e-03_rb,0.33253e-03_rb,0.36056e-03_rb,0.41785e-03_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.45575e-03_rb,0.39903e-03_rb,0.35239e-03_rb,0.32061e-03_rb,0.28610e-03_rb, &
        & 0.25358e-03_rb,0.22160e-03_rb,0.19044e-03_rb,0.19754e-03_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.45658e-03_rb,0.39983e-03_rb,0.35980e-03_rb,0.32984e-03_rb,0.29898e-03_rb, &
        & 0.27505e-03_rb,0.24328e-03_rb,0.21905e-03_rb,0.23515e-03_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.45737e-03_rb,0.40126e-03_rb,0.36866e-03_rb,0.34025e-03_rb,0.31808e-03_rb, &
        & 0.29648e-03_rb,0.26565e-03_rb,0.25100e-03_rb,0.27824e-03_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.45822e-03_rb,0.40344e-03_rb,0.37727e-03_rb,0.35639e-03_rb,0.33753e-03_rb, &
        & 0.31544e-03_rb,0.29107e-03_rb,0.29011e-03_rb,0.33040e-03_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.45803e-03_rb,0.40724e-03_rb,0.38685e-03_rb,0.37303e-03_rb,0.35804e-03_rb, &
        & 0.33477e-03_rb,0.32542e-03_rb,0.34063e-03_rb,0.39188e-03_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.45696e-03_rb,0.40009e-03_rb,0.35533e-03_rb,0.32134e-03_rb,0.28977e-03_rb, &
        & 0.25642e-03_rb,0.22005e-03_rb,0.17825e-03_rb,0.16960e-03_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.45799e-03_rb,0.40164e-03_rb,0.36303e-03_rb,0.33235e-03_rb,0.30343e-03_rb, &
        & 0.27796e-03_rb,0.24341e-03_rb,0.20303e-03_rb,0.20347e-03_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.45889e-03_rb,0.40391e-03_rb,0.37251e-03_rb,0.34409e-03_rb,0.32396e-03_rb, &
        & 0.30068e-03_rb,0.26403e-03_rb,0.23112e-03_rb,0.24434e-03_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.45955e-03_rb,0.40640e-03_rb,0.38203e-03_rb,0.36158e-03_rb,0.34561e-03_rb, &
        & 0.32162e-03_rb,0.28613e-03_rb,0.26715e-03_rb,0.29513e-03_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.45899e-03_rb,0.41016e-03_rb,0.39290e-03_rb,0.38024e-03_rb,0.36719e-03_rb, &
        & 0.34183e-03_rb,0.31736e-03_rb,0.31348e-03_rb,0.35430e-03_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.44860e-03_rb,0.39309e-03_rb,0.35143e-03_rb,0.31765e-03_rb,0.28887e-03_rb, &
        & 0.25439e-03_rb,0.21652e-03_rb,0.16876e-03_rb,0.14333e-03_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.45003e-03_rb,0.39579e-03_rb,0.35959e-03_rb,0.33075e-03_rb,0.30371e-03_rb, &
        & 0.27559e-03_rb,0.24058e-03_rb,0.19058e-03_rb,0.17442e-03_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.45146e-03_rb,0.39890e-03_rb,0.36951e-03_rb,0.34437e-03_rb,0.32442e-03_rb, &
        & 0.30040e-03_rb,0.26223e-03_rb,0.21520e-03_rb,0.21295e-03_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.45218e-03_rb,0.40151e-03_rb,0.38022e-03_rb,0.36228e-03_rb,0.34802e-03_rb, &
        & 0.32336e-03_rb,0.28327e-03_rb,0.24766e-03_rb,0.26132e-03_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.45207e-03_rb,0.40582e-03_rb,0.39175e-03_rb,0.38293e-03_rb,0.37072e-03_rb, &
        & 0.34500e-03_rb,0.31037e-03_rb,0.29012e-03_rb,0.31661e-03_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.43374e-03_rb,0.38081e-03_rb,0.34182e-03_rb,0.31060e-03_rb,0.28045e-03_rb, &
        & 0.24697e-03_rb,0.20889e-03_rb,0.16055e-03_rb,0.12036e-03_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.43531e-03_rb,0.38407e-03_rb,0.35000e-03_rb,0.32358e-03_rb,0.29721e-03_rb, &
        & 0.26821e-03_rb,0.23298e-03_rb,0.18104e-03_rb,0.14668e-03_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.43706e-03_rb,0.38738e-03_rb,0.35982e-03_rb,0.33840e-03_rb,0.31879e-03_rb, &
        & 0.29390e-03_rb,0.25605e-03_rb,0.20200e-03_rb,0.18191e-03_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.43807e-03_rb,0.39050e-03_rb,0.37037e-03_rb,0.35740e-03_rb,0.34372e-03_rb, &
        & 0.31863e-03_rb,0.27734e-03_rb,0.23003e-03_rb,0.22665e-03_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.43877e-03_rb,0.39551e-03_rb,0.38302e-03_rb,0.38004e-03_rb,0.36796e-03_rb, &
        & 0.34140e-03_rb,0.30328e-03_rb,0.26832e-03_rb,0.27676e-03_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.41256e-03_rb,0.36341e-03_rb,0.32806e-03_rb,0.30023e-03_rb,0.26855e-03_rb, &
        & 0.23444e-03_rb,0.19788e-03_rb,0.15062e-03_rb,0.10262e-03_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.41482e-03_rb,0.36772e-03_rb,0.33624e-03_rb,0.31297e-03_rb,0.28556e-03_rb, &
        & 0.25627e-03_rb,0.22127e-03_rb,0.17072e-03_rb,0.12601e-03_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.41718e-03_rb,0.37142e-03_rb,0.34661e-03_rb,0.32744e-03_rb,0.30719e-03_rb, &
        & 0.28254e-03_rb,0.24553e-03_rb,0.19044e-03_rb,0.15628e-03_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.41872e-03_rb,0.37555e-03_rb,0.35755e-03_rb,0.34713e-03_rb,0.33306e-03_rb, &
        & 0.30832e-03_rb,0.26791e-03_rb,0.21481e-03_rb,0.19464e-03_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.42048e-03_rb,0.38195e-03_rb,0.37042e-03_rb,0.37076e-03_rb,0.35893e-03_rb, &
        & 0.33177e-03_rb,0.29351e-03_rb,0.24858e-03_rb,0.23890e-03_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.38528e-03_rb,0.34132e-03_rb,0.30950e-03_rb,0.28435e-03_rb,0.25387e-03_rb, &
        & 0.21998e-03_rb,0.18381e-03_rb,0.13995e-03_rb,0.86291e-04_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.38813e-03_rb,0.34604e-03_rb,0.31840e-03_rb,0.29716e-03_rb,0.26994e-03_rb, &
        & 0.24125e-03_rb,0.20696e-03_rb,0.15919e-03_rb,0.10863e-03_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.39073e-03_rb,0.35038e-03_rb,0.33010e-03_rb,0.31128e-03_rb,0.29227e-03_rb, &
        & 0.26641e-03_rb,0.23195e-03_rb,0.17813e-03_rb,0.13616e-03_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.39281e-03_rb,0.35556e-03_rb,0.34140e-03_rb,0.33132e-03_rb,0.31814e-03_rb, &
        & 0.29308e-03_rb,0.25493e-03_rb,0.20064e-03_rb,0.16931e-03_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.39565e-03_rb,0.36362e-03_rb,0.35480e-03_rb,0.35632e-03_rb,0.34483e-03_rb, &
        & 0.31779e-03_rb,0.27994e-03_rb,0.23183e-03_rb,0.20721e-03_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.35353e-03_rb,0.31534e-03_rb,0.28844e-03_rb,0.26382e-03_rb,0.23534e-03_rb, &
        & 0.20347e-03_rb,0.16850e-03_rb,0.12923e-03_rb,0.71856e-04_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.35704e-03_rb,0.32078e-03_rb,0.29797e-03_rb,0.27689e-03_rb,0.25047e-03_rb, &
        & 0.22397e-03_rb,0.19066e-03_rb,0.14773e-03_rb,0.92561e-04_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.36000e-03_rb,0.32561e-03_rb,0.31012e-03_rb,0.29082e-03_rb,0.27253e-03_rb, &
        & 0.24851e-03_rb,0.21457e-03_rb,0.16607e-03_rb,0.11811e-03_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.36298e-03_rb,0.33189e-03_rb,0.32100e-03_rb,0.31077e-03_rb,0.29911e-03_rb, &
        & 0.27480e-03_rb,0.23785e-03_rb,0.18730e-03_rb,0.14849e-03_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.36645e-03_rb,0.34041e-03_rb,0.33535e-03_rb,0.33583e-03_rb,0.32646e-03_rb, &
        & 0.29975e-03_rb,0.26277e-03_rb,0.21618e-03_rb,0.18220e-03_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.31882e-03_rb,0.28657e-03_rb,0.26462e-03_rb,0.24132e-03_rb,0.21534e-03_rb, &
        & 0.18636e-03_rb,0.15549e-03_rb,0.11817e-03_rb,0.60706e-04_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.32301e-03_rb,0.29267e-03_rb,0.27519e-03_rb,0.25478e-03_rb,0.23046e-03_rb, &
        & 0.20560e-03_rb,0.17582e-03_rb,0.13638e-03_rb,0.79531e-04_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.32639e-03_rb,0.29816e-03_rb,0.28709e-03_rb,0.26927e-03_rb,0.25170e-03_rb, &
        & 0.23027e-03_rb,0.19794e-03_rb,0.15520e-03_rb,0.10253e-03_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.33008e-03_rb,0.30559e-03_rb,0.29823e-03_rb,0.28948e-03_rb,0.27786e-03_rb, &
        & 0.25639e-03_rb,0.22020e-03_rb,0.17621e-03_rb,0.13035e-03_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.33429e-03_rb,0.31481e-03_rb,0.31383e-03_rb,0.31396e-03_rb,0.30544e-03_rb, &
        & 0.28143e-03_rb,0.24487e-03_rb,0.20336e-03_rb,0.16246e-03_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.28433e-03_rb,0.25922e-03_rb,0.24227e-03_rb,0.22327e-03_rb,0.20067e-03_rb, &
        & 0.17618e-03_rb,0.15050e-03_rb,0.11425e-03_rb,0.57921e-04_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.28828e-03_rb,0.26532e-03_rb,0.25425e-03_rb,0.23723e-03_rb,0.21797e-03_rb, &
        & 0.19695e-03_rb,0.17056e-03_rb,0.13252e-03_rb,0.76361e-04_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.29236e-03_rb,0.27228e-03_rb,0.26588e-03_rb,0.25415e-03_rb,0.24099e-03_rb, &
        & 0.22135e-03_rb,0.19174e-03_rb,0.15232e-03_rb,0.98360e-04_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.29717e-03_rb,0.28209e-03_rb,0.27970e-03_rb,0.27650e-03_rb,0.26714e-03_rb, &
        & 0.24681e-03_rb,0.21371e-03_rb,0.17631e-03_rb,0.12518e-03_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.30177e-03_rb,0.29180e-03_rb,0.29905e-03_rb,0.30274e-03_rb,0.29362e-03_rb, &
        & 0.27208e-03_rb,0.24087e-03_rb,0.20461e-03_rb,0.15616e-03_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.24998e-03_rb,0.23220e-03_rb,0.21962e-03_rb,0.20418e-03_rb,0.18598e-03_rb, &
        & 0.16635e-03_rb,0.14319e-03_rb,0.11002e-03_rb,0.54657e-04_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.25422e-03_rb,0.23880e-03_rb,0.23171e-03_rb,0.21907e-03_rb,0.20484e-03_rb, &
        & 0.18724e-03_rb,0.16371e-03_rb,0.12837e-03_rb,0.72183e-04_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.25909e-03_rb,0.24827e-03_rb,0.24443e-03_rb,0.23867e-03_rb,0.22868e-03_rb, &
        & 0.21084e-03_rb,0.18482e-03_rb,0.14974e-03_rb,0.93395e-04_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.26399e-03_rb,0.25851e-03_rb,0.26208e-03_rb,0.26298e-03_rb,0.25438e-03_rb, &
        & 0.23482e-03_rb,0.20834e-03_rb,0.17589e-03_rb,0.11924e-03_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.26914e-03_rb,0.26866e-03_rb,0.28401e-03_rb,0.28945e-03_rb,0.28022e-03_rb, &
        & 0.26150e-03_rb,0.23750e-03_rb,0.20542e-03_rb,0.14923e-03_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.21751e-03_rb,0.20608e-03_rb,0.19731e-03_rb,0.18540e-03_rb,0.17208e-03_rb, &
        & 0.15624e-03_rb,0.13474e-03_rb,0.10527e-03_rb,0.50953e-04_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.22228e-03_rb,0.21439e-03_rb,0.20968e-03_rb,0.20158e-03_rb,0.19151e-03_rb, &
        & 0.17720e-03_rb,0.15564e-03_rb,0.12461e-03_rb,0.67516e-04_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.22727e-03_rb,0.22462e-03_rb,0.22474e-03_rb,0.22318e-03_rb,0.21469e-03_rb, &
        & 0.19989e-03_rb,0.17790e-03_rb,0.14782e-03_rb,0.88180e-04_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.23242e-03_rb,0.23488e-03_rb,0.24462e-03_rb,0.24805e-03_rb,0.24003e-03_rb, &
        & 0.22364e-03_rb,0.20382e-03_rb,0.17470e-03_rb,0.11335e-03_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.23819e-03_rb,0.24692e-03_rb,0.26797e-03_rb,0.27413e-03_rb,0.26611e-03_rb, &
        & 0.25207e-03_rb,0.23456e-03_rb,0.20493e-03_rb,0.14302e-03_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.54188e-03_rb,0.47438e-03_rb,0.41220e-03_rb,0.36319e-03_rb,0.32177e-03_rb, &
        & 0.28178e-03_rb,0.26195e-03_rb,0.26768e-03_rb,0.30988e-03_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.53863e-03_rb,0.47157e-03_rb,0.41235e-03_rb,0.37142e-03_rb,0.33314e-03_rb, &
        & 0.30667e-03_rb,0.29622e-03_rb,0.31573e-03_rb,0.36594e-03_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.53537e-03_rb,0.46873e-03_rb,0.41881e-03_rb,0.37881e-03_rb,0.35190e-03_rb, &
        & 0.33275e-03_rb,0.33720e-03_rb,0.36928e-03_rb,0.42810e-03_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.53255e-03_rb,0.46630e-03_rb,0.42443e-03_rb,0.38945e-03_rb,0.37640e-03_rb, &
        & 0.36424e-03_rb,0.38086e-03_rb,0.42782e-03_rb,0.49486e-03_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.53016e-03_rb,0.46423e-03_rb,0.43029e-03_rb,0.40524e-03_rb,0.39835e-03_rb, &
        & 0.40134e-03_rb,0.42785e-03_rb,0.48998e-03_rb,0.56469e-03_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.58031e-03_rb,0.50799e-03_rb,0.44209e-03_rb,0.39038e-03_rb,0.34592e-03_rb, &
        & 0.30091e-03_rb,0.27060e-03_rb,0.26179e-03_rb,0.30262e-03_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.57783e-03_rb,0.50583e-03_rb,0.44303e-03_rb,0.40158e-03_rb,0.35771e-03_rb, &
        & 0.32458e-03_rb,0.30412e-03_rb,0.31051e-03_rb,0.36041e-03_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.57540e-03_rb,0.50373e-03_rb,0.45070e-03_rb,0.41078e-03_rb,0.37604e-03_rb, &
        & 0.34973e-03_rb,0.34205e-03_rb,0.36376e-03_rb,0.42111e-03_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.57390e-03_rb,0.50246e-03_rb,0.45850e-03_rb,0.42273e-03_rb,0.40005e-03_rb, &
        & 0.37925e-03_rb,0.38380e-03_rb,0.42075e-03_rb,0.48570e-03_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.57251e-03_rb,0.50128e-03_rb,0.46686e-03_rb,0.44000e-03_rb,0.42177e-03_rb, &
        & 0.41338e-03_rb,0.42764e-03_rb,0.47996e-03_rb,0.55489e-03_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.61606e-03_rb,0.53921e-03_rb,0.46963e-03_rb,0.41409e-03_rb,0.36561e-03_rb, &
        & 0.31888e-03_rb,0.27397e-03_rb,0.24837e-03_rb,0.28056e-03_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.61406e-03_rb,0.53752e-03_rb,0.47091e-03_rb,0.42713e-03_rb,0.37975e-03_rb, &
        & 0.33844e-03_rb,0.30659e-03_rb,0.29320e-03_rb,0.33782e-03_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.61257e-03_rb,0.53626e-03_rb,0.47829e-03_rb,0.43902e-03_rb,0.39761e-03_rb, &
        & 0.36364e-03_rb,0.34206e-03_rb,0.34371e-03_rb,0.39648e-03_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.61154e-03_rb,0.53538e-03_rb,0.48822e-03_rb,0.45223e-03_rb,0.42130e-03_rb, &
        & 0.39347e-03_rb,0.37907e-03_rb,0.39708e-03_rb,0.45765e-03_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.61140e-03_rb,0.53529e-03_rb,0.49797e-03_rb,0.47117e-03_rb,0.44115e-03_rb, &
        & 0.42497e-03_rb,0.41979e-03_rb,0.45564e-03_rb,0.52659e-03_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.64443e-03_rb,0.56411e-03_rb,0.49232e-03_rb,0.43467e-03_rb,0.38190e-03_rb, &
        & 0.33277e-03_rb,0.27825e-03_rb,0.23501e-03_rb,0.25182e-03_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.64292e-03_rb,0.56277e-03_rb,0.49409e-03_rb,0.44787e-03_rb,0.39817e-03_rb, &
        & 0.35161e-03_rb,0.30747e-03_rb,0.27717e-03_rb,0.30629e-03_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.64243e-03_rb,0.56236e-03_rb,0.50119e-03_rb,0.46200e-03_rb,0.41580e-03_rb, &
        & 0.37809e-03_rb,0.34162e-03_rb,0.32168e-03_rb,0.36251e-03_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.64248e-03_rb,0.56245e-03_rb,0.51257e-03_rb,0.47612e-03_rb,0.44107e-03_rb, &
        & 0.40610e-03_rb,0.37659e-03_rb,0.36992e-03_rb,0.42339e-03_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.64345e-03_rb,0.56414e-03_rb,0.52333e-03_rb,0.49621e-03_rb,0.46345e-03_rb, &
        & 0.43530e-03_rb,0.41388e-03_rb,0.42704e-03_rb,0.49356e-03_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.66205e-03_rb,0.57941e-03_rb,0.50732e-03_rb,0.44788e-03_rb,0.39504e-03_rb, &
        & 0.34261e-03_rb,0.28274e-03_rb,0.22440e-03_rb,0.22048e-03_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.66163e-03_rb,0.57912e-03_rb,0.51033e-03_rb,0.46104e-03_rb,0.41289e-03_rb, &
        & 0.36220e-03_rb,0.30969e-03_rb,0.26291e-03_rb,0.27073e-03_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.66197e-03_rb,0.57946e-03_rb,0.51799e-03_rb,0.47594e-03_rb,0.43132e-03_rb, &
        & 0.39012e-03_rb,0.34192e-03_rb,0.30335e-03_rb,0.32369e-03_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.66310e-03_rb,0.58124e-03_rb,0.52962e-03_rb,0.49191e-03_rb,0.45677e-03_rb, &
        & 0.42022e-03_rb,0.37529e-03_rb,0.34603e-03_rb,0.38247e-03_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.66411e-03_rb,0.58365e-03_rb,0.54225e-03_rb,0.51373e-03_rb,0.48391e-03_rb, &
        & 0.44802e-03_rb,0.41016e-03_rb,0.39855e-03_rb,0.45388e-03_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.66761e-03_rb,0.58431e-03_rb,0.51405e-03_rb,0.45280e-03_rb,0.40155e-03_rb, &
        & 0.34610e-03_rb,0.28249e-03_rb,0.21395e-03_rb,0.18888e-03_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.66804e-03_rb,0.58472e-03_rb,0.51807e-03_rb,0.46619e-03_rb,0.41922e-03_rb, &
        & 0.36797e-03_rb,0.30908e-03_rb,0.24851e-03_rb,0.23408e-03_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.66946e-03_rb,0.58664e-03_rb,0.52673e-03_rb,0.48103e-03_rb,0.43940e-03_rb, &
        & 0.39605e-03_rb,0.34204e-03_rb,0.28507e-03_rb,0.28224e-03_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.67150e-03_rb,0.59004e-03_rb,0.53959e-03_rb,0.49825e-03_rb,0.46612e-03_rb, &
        & 0.42814e-03_rb,0.37432e-03_rb,0.32325e-03_rb,0.33817e-03_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.67315e-03_rb,0.59352e-03_rb,0.55351e-03_rb,0.52125e-03_rb,0.49748e-03_rb, &
        & 0.45943e-03_rb,0.40603e-03_rb,0.37064e-03_rb,0.40786e-03_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.65825e-03_rb,0.57610e-03_rb,0.50973e-03_rb,0.44919e-03_rb,0.40091e-03_rb, &
        & 0.34393e-03_rb,0.27863e-03_rb,0.20584e-03_rb,0.15896e-03_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.66067e-03_rb,0.57854e-03_rb,0.51604e-03_rb,0.46345e-03_rb,0.41921e-03_rb, &
        & 0.36628e-03_rb,0.30597e-03_rb,0.23625e-03_rb,0.19866e-03_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.66419e-03_rb,0.58318e-03_rb,0.52599e-03_rb,0.48043e-03_rb,0.43994e-03_rb, &
        & 0.39562e-03_rb,0.33865e-03_rb,0.26970e-03_rb,0.24286e-03_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.66717e-03_rb,0.58792e-03_rb,0.54016e-03_rb,0.49893e-03_rb,0.46857e-03_rb, &
        & 0.43005e-03_rb,0.37266e-03_rb,0.30411e-03_rb,0.29541e-03_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.67024e-03_rb,0.59309e-03_rb,0.55482e-03_rb,0.52246e-03_rb,0.50380e-03_rb, &
        & 0.46446e-03_rb,0.40297e-03_rb,0.34641e-03_rb,0.36121e-03_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.63858e-03_rb,0.55888e-03_rb,0.49736e-03_rb,0.43942e-03_rb,0.39082e-03_rb, &
        & 0.33512e-03_rb,0.27058e-03_rb,0.19736e-03_rb,0.13589e-03_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.64324e-03_rb,0.56410e-03_rb,0.50530e-03_rb,0.45504e-03_rb,0.41149e-03_rb, &
        & 0.35701e-03_rb,0.29879e-03_rb,0.22618e-03_rb,0.16884e-03_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.64790e-03_rb,0.57033e-03_rb,0.51640e-03_rb,0.47474e-03_rb,0.43226e-03_rb, &
        & 0.38846e-03_rb,0.33154e-03_rb,0.25769e-03_rb,0.20679e-03_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.65242e-03_rb,0.57688e-03_rb,0.53191e-03_rb,0.49447e-03_rb,0.46234e-03_rb, &
        & 0.42468e-03_rb,0.36669e-03_rb,0.28907e-03_rb,0.25449e-03_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.65716e-03_rb,0.58346e-03_rb,0.54799e-03_rb,0.51859e-03_rb,0.49982e-03_rb, &
        & 0.46218e-03_rb,0.39875e-03_rb,0.32573e-03_rb,0.31655e-03_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.61155e-03_rb,0.53549e-03_rb,0.47878e-03_rb,0.42579e-03_rb,0.37499e-03_rb, &
        & 0.31961e-03_rb,0.25793e-03_rb,0.18655e-03_rb,0.11490e-03_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.61794e-03_rb,0.54314e-03_rb,0.48878e-03_rb,0.44250e-03_rb,0.39687e-03_rb, &
        & 0.34151e-03_rb,0.28569e-03_rb,0.21436e-03_rb,0.14511e-03_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.62338e-03_rb,0.55078e-03_rb,0.50083e-03_rb,0.46354e-03_rb,0.41796e-03_rb, &
        & 0.37306e-03_rb,0.31954e-03_rb,0.24508e-03_rb,0.17854e-03_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.62936e-03_rb,0.55894e-03_rb,0.51722e-03_rb,0.48335e-03_rb,0.44775e-03_rb, &
        & 0.41130e-03_rb,0.35579e-03_rb,0.27533e-03_rb,0.21988e-03_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.63545e-03_rb,0.56698e-03_rb,0.53473e-03_rb,0.50887e-03_rb,0.48603e-03_rb, &
        & 0.45259e-03_rb,0.39001e-03_rb,0.30792e-03_rb,0.27464e-03_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.57726e-03_rb,0.50676e-03_rb,0.45606e-03_rb,0.40779e-03_rb,0.35629e-03_rb, &
        & 0.30220e-03_rb,0.24273e-03_rb,0.17731e-03_rb,0.97262e-04_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.58382e-03_rb,0.51548e-03_rb,0.46729e-03_rb,0.42570e-03_rb,0.37915e-03_rb, &
        & 0.32497e-03_rb,0.26965e-03_rb,0.20381e-03_rb,0.12529e-03_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.59071e-03_rb,0.52512e-03_rb,0.48044e-03_rb,0.44753e-03_rb,0.40142e-03_rb, &
        & 0.35504e-03_rb,0.30526e-03_rb,0.23284e-03_rb,0.15783e-03_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.59829e-03_rb,0.53444e-03_rb,0.49886e-03_rb,0.46810e-03_rb,0.43172e-03_rb, &
        & 0.39380e-03_rb,0.34313e-03_rb,0.26284e-03_rb,0.19647e-03_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.60558e-03_rb,0.54444e-03_rb,0.51788e-03_rb,0.49497e-03_rb,0.47022e-03_rb, &
        & 0.43733e-03_rb,0.37954e-03_rb,0.29404e-03_rb,0.24439e-03_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.53880e-03_rb,0.47669e-03_rb,0.43422e-03_rb,0.39066e-03_rb,0.34441e-03_rb, &
        & 0.29300e-03_rb,0.23722e-03_rb,0.17846e-03_rb,0.93053e-04_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.54672e-03_rb,0.48762e-03_rb,0.44694e-03_rb,0.41155e-03_rb,0.36791e-03_rb, &
        & 0.31854e-03_rb,0.26663e-03_rb,0.20551e-03_rb,0.12100e-03_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.55543e-03_rb,0.49853e-03_rb,0.46450e-03_rb,0.43411e-03_rb,0.39364e-03_rb, &
        & 0.35233e-03_rb,0.30378e-03_rb,0.23442e-03_rb,0.15474e-03_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.56418e-03_rb,0.50955e-03_rb,0.48543e-03_rb,0.45785e-03_rb,0.42947e-03_rb, &
        & 0.39274e-03_rb,0.34329e-03_rb,0.26304e-03_rb,0.19545e-03_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.57281e-03_rb,0.52221e-03_rb,0.50507e-03_rb,0.49044e-03_rb,0.47093e-03_rb, &
        & 0.43647e-03_rb,0.38057e-03_rb,0.29685e-03_rb,0.24345e-03_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.49534e-03_rb,0.44336e-03_rb,0.40843e-03_rb,0.37029e-03_rb,0.32924e-03_rb, &
        & 0.28268e-03_rb,0.23204e-03_rb,0.17692e-03_rb,0.89557e-04_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.50529e-03_rb,0.45586e-03_rb,0.42428e-03_rb,0.39360e-03_rb,0.35384e-03_rb, &
        & 0.31099e-03_rb,0.26296e-03_rb,0.20575e-03_rb,0.11779e-03_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.51591e-03_rb,0.46841e-03_rb,0.44557e-03_rb,0.41710e-03_rb,0.38396e-03_rb, &
        & 0.34779e-03_rb,0.29981e-03_rb,0.23475e-03_rb,0.15231e-03_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.52638e-03_rb,0.48189e-03_rb,0.46689e-03_rb,0.44567e-03_rb,0.42354e-03_rb, &
        & 0.38951e-03_rb,0.33959e-03_rb,0.26406e-03_rb,0.19354e-03_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.53740e-03_rb,0.49847e-03_rb,0.48961e-03_rb,0.48352e-03_rb,0.46848e-03_rb, &
        & 0.43322e-03_rb,0.37807e-03_rb,0.30031e-03_rb,0.24179e-03_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.44967e-03_rb,0.40765e-03_rb,0.37964e-03_rb,0.34767e-03_rb,0.31190e-03_rb, &
        & 0.27110e-03_rb,0.22687e-03_rb,0.17331e-03_rb,0.86456e-04_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.46197e-03_rb,0.42196e-03_rb,0.40010e-03_rb,0.37282e-03_rb,0.33881e-03_rb, &
        & 0.30170e-03_rb,0.25812e-03_rb,0.20314e-03_rb,0.11471e-03_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.47416e-03_rb,0.43662e-03_rb,0.42347e-03_rb,0.39902e-03_rb,0.37341e-03_rb, &
        & 0.34073e-03_rb,0.29426e-03_rb,0.23321e-03_rb,0.14921e-03_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.48670e-03_rb,0.45348e-03_rb,0.44656e-03_rb,0.43321e-03_rb,0.41603e-03_rb, &
        & 0.38366e-03_rb,0.33328e-03_rb,0.26489e-03_rb,0.19032e-03_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.49941e-03_rb,0.47298e-03_rb,0.47361e-03_rb,0.47457e-03_rb,0.46263e-03_rb, &
        & 0.42791e-03_rb,0.37404e-03_rb,0.30319e-03_rb,0.23864e-03_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.67194e-03_rb,0.58821e-03_rb,0.50483e-03_rb,0.44022e-03_rb,0.39428e-03_rb, &
        & 0.35001e-03_rb,0.32802e-03_rb,0.34712e-03_rb,0.40424e-03_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.66779e-03_rb,0.58462e-03_rb,0.50560e-03_rb,0.45370e-03_rb,0.41018e-03_rb, &
        & 0.37788e-03_rb,0.37804e-03_rb,0.41641e-03_rb,0.48433e-03_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.66375e-03_rb,0.58112e-03_rb,0.50831e-03_rb,0.46547e-03_rb,0.43045e-03_rb, &
        & 0.42028e-03_rb,0.43766e-03_rb,0.49681e-03_rb,0.57585e-03_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.65976e-03_rb,0.57768e-03_rb,0.51951e-03_rb,0.47941e-03_rb,0.45883e-03_rb, &
        & 0.47280e-03_rb,0.50926e-03_rb,0.58385e-03_rb,0.67578e-03_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.65493e-03_rb,0.57349e-03_rb,0.52976e-03_rb,0.49649e-03_rb,0.49872e-03_rb, &
        & 0.52477e-03_rb,0.58910e-03_rb,0.67477e-03_rb,0.78177e-03_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.74429e-03_rb,0.65147e-03_rb,0.55958e-03_rb,0.48439e-03_rb,0.43163e-03_rb, &
        & 0.37450e-03_rb,0.34047e-03_rb,0.34702e-03_rb,0.40780e-03_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.73897e-03_rb,0.64685e-03_rb,0.56004e-03_rb,0.49710e-03_rb,0.44831e-03_rb, &
        & 0.39986e-03_rb,0.38897e-03_rb,0.41601e-03_rb,0.48800e-03_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.73423e-03_rb,0.64274e-03_rb,0.56120e-03_rb,0.51084e-03_rb,0.46668e-03_rb, &
        & 0.44211e-03_rb,0.44391e-03_rb,0.49187e-03_rb,0.57573e-03_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.72906e-03_rb,0.63825e-03_rb,0.57090e-03_rb,0.52414e-03_rb,0.49429e-03_rb, &
        & 0.48962e-03_rb,0.50615e-03_rb,0.57487e-03_rb,0.67078e-03_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.72487e-03_rb,0.63461e-03_rb,0.58043e-03_rb,0.54078e-03_rb,0.53373e-03_rb, &
        & 0.54010e-03_rb,0.58187e-03_rb,0.66737e-03_rb,0.77655e-03_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.81958e-03_rb,0.71735e-03_rb,0.61664e-03_rb,0.53266e-03_rb,0.46619e-03_rb, &
        & 0.39818e-03_rb,0.35035e-03_rb,0.33455e-03_rb,0.39417e-03_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.81517e-03_rb,0.71349e-03_rb,0.61778e-03_rb,0.54292e-03_rb,0.48245e-03_rb, &
        & 0.42187e-03_rb,0.39187e-03_rb,0.40203e-03_rb,0.47500e-03_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.81131e-03_rb,0.71014e-03_rb,0.61887e-03_rb,0.55563e-03_rb,0.49928e-03_rb, &
        & 0.45852e-03_rb,0.44599e-03_rb,0.47656e-03_rb,0.56304e-03_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.80684e-03_rb,0.70624e-03_rb,0.62502e-03_rb,0.56854e-03_rb,0.52516e-03_rb, &
        & 0.50282e-03_rb,0.50405e-03_rb,0.55674e-03_rb,0.65701e-03_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.80277e-03_rb,0.70271e-03_rb,0.63387e-03_rb,0.58439e-03_rb,0.56394e-03_rb, &
        & 0.54984e-03_rb,0.57036e-03_rb,0.64435e-03_rb,0.75883e-03_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.88962e-03_rb,0.77859e-03_rb,0.66931e-03_rb,0.57740e-03_rb,0.50138e-03_rb, &
        & 0.42396e-03_rb,0.36076e-03_rb,0.31901e-03_rb,0.36723e-03_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.88665e-03_rb,0.77601e-03_rb,0.67190e-03_rb,0.58651e-03_rb,0.51766e-03_rb, &
        & 0.44641e-03_rb,0.39755e-03_rb,0.38137e-03_rb,0.44921e-03_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.88362e-03_rb,0.77338e-03_rb,0.67394e-03_rb,0.60034e-03_rb,0.53406e-03_rb, &
        & 0.47716e-03_rb,0.44855e-03_rb,0.45695e-03_rb,0.53953e-03_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.87936e-03_rb,0.76967e-03_rb,0.67789e-03_rb,0.61530e-03_rb,0.55605e-03_rb, &
        & 0.52026e-03_rb,0.50535e-03_rb,0.53746e-03_rb,0.63432e-03_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.87542e-03_rb,0.76625e-03_rb,0.68901e-03_rb,0.63085e-03_rb,0.59283e-03_rb, &
        & 0.56499e-03_rb,0.56855e-03_rb,0.62330e-03_rb,0.73477e-03_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.95206e-03_rb,0.83319e-03_rb,0.71670e-03_rb,0.61887e-03_rb,0.53098e-03_rb, &
        & 0.45065e-03_rb,0.37161e-03_rb,0.30790e-03_rb,0.33661e-03_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.95024e-03_rb,0.83163e-03_rb,0.72052e-03_rb,0.62686e-03_rb,0.54899e-03_rb, &
        & 0.47363e-03_rb,0.40570e-03_rb,0.36303e-03_rb,0.41611e-03_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.94830e-03_rb,0.82995e-03_rb,0.72350e-03_rb,0.64294e-03_rb,0.56882e-03_rb, &
        & 0.49923e-03_rb,0.45370e-03_rb,0.43446e-03_rb,0.50740e-03_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.94590e-03_rb,0.82788e-03_rb,0.72796e-03_rb,0.66075e-03_rb,0.59106e-03_rb, &
        & 0.53960e-03_rb,0.50922e-03_rb,0.51449e-03_rb,0.60347e-03_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.94431e-03_rb,0.82652e-03_rb,0.73985e-03_rb,0.67685e-03_rb,0.62406e-03_rb, &
        & 0.58535e-03_rb,0.56960e-03_rb,0.60003e-03_rb,0.70205e-03_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.10063e-02_rb,0.88067e-03_rb,0.75816e-03_rb,0.65590e-03_rb,0.55704e-03_rb, &
        & 0.47312e-03_rb,0.38218e-03_rb,0.29812e-03_rb,0.29923e-03_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.10075e-02_rb,0.88171e-03_rb,0.76490e-03_rb,0.66447e-03_rb,0.57639e-03_rb, &
        & 0.49685e-03_rb,0.41345e-03_rb,0.34724e-03_rb,0.37557e-03_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.10072e-02_rb,0.88152e-03_rb,0.76915e-03_rb,0.68176e-03_rb,0.59750e-03_rb, &
        & 0.52339e-03_rb,0.45689e-03_rb,0.41130e-03_rb,0.46505e-03_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.10060e-02_rb,0.88049e-03_rb,0.77394e-03_rb,0.69974e-03_rb,0.62189e-03_rb, &
        & 0.55958e-03_rb,0.51188e-03_rb,0.48580e-03_rb,0.55924e-03_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.10051e-02_rb,0.87967e-03_rb,0.78569e-03_rb,0.71720e-03_rb,0.65527e-03_rb, &
        & 0.60501e-03_rb,0.57082e-03_rb,0.56756e-03_rb,0.65737e-03_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.10487e-02_rb,0.91773e-03_rb,0.79062e-03_rb,0.68516e-03_rb,0.57878e-03_rb, &
        & 0.49035e-03_rb,0.39066e-03_rb,0.28990e-03_rb,0.26192e-03_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.10509e-02_rb,0.91972e-03_rb,0.79894e-03_rb,0.69452e-03_rb,0.59953e-03_rb, &
        & 0.51487e-03_rb,0.42128e-03_rb,0.33451e-03_rb,0.33284e-03_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.10507e-02_rb,0.91957e-03_rb,0.80398e-03_rb,0.71157e-03_rb,0.62221e-03_rb, &
        & 0.54150e-03_rb,0.46197e-03_rb,0.39208e-03_rb,0.41685e-03_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.10509e-02_rb,0.91973e-03_rb,0.80950e-03_rb,0.73042e-03_rb,0.64659e-03_rb, &
        & 0.57916e-03_rb,0.51360e-03_rb,0.46065e-03_rb,0.50813e-03_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.10509e-02_rb,0.91978e-03_rb,0.82196e-03_rb,0.75027e-03_rb,0.67740e-03_rb, &
        & 0.62606e-03_rb,0.57168e-03_rb,0.53418e-03_rb,0.60461e-03_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.10710e-02_rb,0.93729e-03_rb,0.80929e-03_rb,0.70366e-03_rb,0.59621e-03_rb, &
        & 0.50074e-03_rb,0.39602e-03_rb,0.28309e-03_rb,0.22630e-03_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.10749e-02_rb,0.94069e-03_rb,0.82032e-03_rb,0.71447e-03_rb,0.61694e-03_rb, &
        & 0.52833e-03_rb,0.42406e-03_rb,0.32400e-03_rb,0.29100e-03_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.10773e-02_rb,0.94281e-03_rb,0.82736e-03_rb,0.73051e-03_rb,0.64296e-03_rb, &
        & 0.55298e-03_rb,0.46605e-03_rb,0.37591e-03_rb,0.36869e-03_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.10796e-02_rb,0.94487e-03_rb,0.83459e-03_rb,0.75111e-03_rb,0.66786e-03_rb, &
        & 0.59048e-03_rb,0.51603e-03_rb,0.43920e-03_rb,0.45502e-03_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.10814e-02_rb,0.94726e-03_rb,0.84809e-03_rb,0.77185e-03_rb,0.69811e-03_rb, &
        & 0.64090e-03_rb,0.57338e-03_rb,0.50692e-03_rb,0.54643e-03_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.10730e-02_rb,0.93903e-03_rb,0.81373e-03_rb,0.70878e-03_rb,0.60523e-03_rb, &
        & 0.50463e-03_rb,0.39690e-03_rb,0.27640e-03_rb,0.19492e-03_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.10796e-02_rb,0.94482e-03_rb,0.82740e-03_rb,0.72216e-03_rb,0.62667e-03_rb, &
        & 0.53533e-03_rb,0.42406e-03_rb,0.31368e-03_rb,0.25100e-03_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.10853e-02_rb,0.94982e-03_rb,0.83742e-03_rb,0.73904e-03_rb,0.65592e-03_rb, &
        & 0.56092e-03_rb,0.46365e-03_rb,0.36183e-03_rb,0.32022e-03_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.10900e-02_rb,0.95441e-03_rb,0.84774e-03_rb,0.76272e-03_rb,0.68162e-03_rb, &
        & 0.59735e-03_rb,0.51414e-03_rb,0.41927e-03_rb,0.39972e-03_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.10952e-02_rb,0.96081e-03_rb,0.86364e-03_rb,0.78596e-03_rb,0.71341e-03_rb, &
        & 0.64539e-03_rb,0.57286e-03_rb,0.48249e-03_rb,0.48653e-03_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.10558e-02_rb,0.92391e-03_rb,0.80426e-03_rb,0.70294e-03_rb,0.60555e-03_rb, &
        & 0.50218e-03_rb,0.39384e-03_rb,0.27052e-03_rb,0.17175e-03_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.10672e-02_rb,0.93391e-03_rb,0.82187e-03_rb,0.72055e-03_rb,0.62861e-03_rb, &
        & 0.53517e-03_rb,0.42394e-03_rb,0.30520e-03_rb,0.22066e-03_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.10767e-02_rb,0.94234e-03_rb,0.83636e-03_rb,0.73971e-03_rb,0.66074e-03_rb, &
        & 0.56459e-03_rb,0.46058e-03_rb,0.35232e-03_rb,0.28101e-03_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.10858e-02_rb,0.95208e-03_rb,0.85075e-03_rb,0.76653e-03_rb,0.68963e-03_rb, &
        & 0.60125e-03_rb,0.51089e-03_rb,0.40664e-03_rb,0.35284e-03_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.10953e-02_rb,0.96285e-03_rb,0.86971e-03_rb,0.79369e-03_rb,0.72347e-03_rb, &
        & 0.65043e-03_rb,0.57028e-03_rb,0.46529e-03_rb,0.43428e-03_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.10314e-02_rb,0.90258e-03_rb,0.79467e-03_rb,0.69943e-03_rb,0.60546e-03_rb, &
        & 0.50625e-03_rb,0.39977e-03_rb,0.27736e-03_rb,0.16855e-03_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.10451e-02_rb,0.91471e-03_rb,0.81404e-03_rb,0.71994e-03_rb,0.63569e-03_rb, &
        & 0.54231e-03_rb,0.43348e-03_rb,0.31480e-03_rb,0.21751e-03_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.10591e-02_rb,0.92888e-03_rb,0.83171e-03_rb,0.74582e-03_rb,0.67129e-03_rb, &
        & 0.57545e-03_rb,0.47649e-03_rb,0.36327e-03_rb,0.27482e-03_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.10732e-02_rb,0.94400e-03_rb,0.85133e-03_rb,0.77904e-03_rb,0.70324e-03_rb, &
        & 0.62021e-03_rb,0.53030e-03_rb,0.41771e-03_rb,0.34284e-03_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.10880e-02_rb,0.96010e-03_rb,0.87843e-03_rb,0.80902e-03_rb,0.74479e-03_rb, &
        & 0.67711e-03_rb,0.59146e-03_rb,0.47518e-03_rb,0.42405e-03_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.99800e-03_rb,0.87344e-03_rb,0.77838e-03_rb,0.69077e-03_rb,0.60021e-03_rb, &
        & 0.50540e-03_rb,0.40237e-03_rb,0.28326e-03_rb,0.16340e-03_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.10163e-02_rb,0.89149e-03_rb,0.80050e-03_rb,0.71513e-03_rb,0.63616e-03_rb, &
        & 0.54455e-03_rb,0.44056e-03_rb,0.32235e-03_rb,0.21276e-03_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.10345e-02_rb,0.91067e-03_rb,0.82237e-03_rb,0.74899e-03_rb,0.67503e-03_rb, &
        & 0.58381e-03_rb,0.48916e-03_rb,0.37171e-03_rb,0.26817e-03_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.10541e-02_rb,0.93146e-03_rb,0.84943e-03_rb,0.78581e-03_rb,0.71327e-03_rb, &
        & 0.63724e-03_rb,0.54703e-03_rb,0.42621e-03_rb,0.33361e-03_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.10724e-02_rb,0.95127e-03_rb,0.88178e-03_rb,0.82065e-03_rb,0.76248e-03_rb, &
        & 0.70028e-03_rb,0.60892e-03_rb,0.48179e-03_rb,0.41458e-03_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.95912e-03_rb,0.84178e-03_rb,0.75778e-03_rb,0.67765e-03_rb,0.59123e-03_rb, &
        & 0.50112e-03_rb,0.40170e-03_rb,0.28784e-03_rb,0.15761e-03_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.98187e-03_rb,0.86541e-03_rb,0.78344e-03_rb,0.70851e-03_rb,0.63200e-03_rb, &
        & 0.54351e-03_rb,0.44541e-03_rb,0.32864e-03_rb,0.20682e-03_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.10063e-02_rb,0.89078e-03_rb,0.81111e-03_rb,0.74766e-03_rb,0.67473e-03_rb, &
        & 0.58963e-03_rb,0.49872e-03_rb,0.37736e-03_rb,0.26208e-03_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.10299e-02_rb,0.91565e-03_rb,0.84455e-03_rb,0.78769e-03_rb,0.71894e-03_rb, &
        & 0.65012e-03_rb,0.55995e-03_rb,0.43170e-03_rb,0.32660e-03_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.10525e-02_rb,0.93912e-03_rb,0.88061e-03_rb,0.82863e-03_rb,0.77810e-03_rb, &
        & 0.71766e-03_rb,0.62330e-03_rb,0.48589e-03_rb,0.40661e-03_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.84224e-03_rb,0.73740e-03_rb,0.63257e-03_rb,0.54853e-03_rb,0.48911e-03_rb, &
        & 0.46139e-03_rb,0.45309e-03_rb,0.51018e-03_rb,0.59300e-03_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.83841e-03_rb,0.73411e-03_rb,0.62983e-03_rb,0.55802e-03_rb,0.51855e-03_rb, &
        & 0.50261e-03_rb,0.52209e-03_rb,0.59394e-03_rb,0.69158e-03_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.83593e-03_rb,0.73202e-03_rb,0.62914e-03_rb,0.58596e-03_rb,0.54957e-03_rb, &
        & 0.55151e-03_rb,0.60969e-03_rb,0.69097e-03_rb,0.80707e-03_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.83246e-03_rb,0.72902e-03_rb,0.63274e-03_rb,0.60961e-03_rb,0.59022e-03_rb, &
        & 0.61063e-03_rb,0.70961e-03_rb,0.80207e-03_rb,0.93855e-03_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.82958e-03_rb,0.72654e-03_rb,0.65011e-03_rb,0.63711e-03_rb,0.63941e-03_rb, &
        & 0.69313e-03_rb,0.82070e-03_rb,0.92941e-03_rb,0.10860e-02_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.94737e-03_rb,0.82928e-03_rb,0.71121e-03_rb,0.61292e-03_rb,0.53888e-03_rb, &
        & 0.49608e-03_rb,0.46449e-03_rb,0.50895e-03_rb,0.59638e-03_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.94309e-03_rb,0.82559e-03_rb,0.70809e-03_rb,0.61981e-03_rb,0.56880e-03_rb, &
        & 0.53738e-03_rb,0.52906e-03_rb,0.59714e-03_rb,0.70098e-03_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.93813e-03_rb,0.82131e-03_rb,0.70667e-03_rb,0.64442e-03_rb,0.60095e-03_rb, &
        & 0.58581e-03_rb,0.62140e-03_rb,0.70455e-03_rb,0.83109e-03_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.93357e-03_rb,0.81735e-03_rb,0.70879e-03_rb,0.66885e-03_rb,0.63704e-03_rb, &
        & 0.64975e-03_rb,0.73250e-03_rb,0.82836e-03_rb,0.97995e-03_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.92926e-03_rb,0.81362e-03_rb,0.72487e-03_rb,0.69759e-03_rb,0.68443e-03_rb, &
        & 0.72989e-03_rb,0.85114e-03_rb,0.96045e-03_rb,0.11390e-02_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.10589e-02_rb,0.92685e-03_rb,0.79478e-03_rb,0.68185e-03_rb,0.58961e-03_rb, &
        & 0.52592e-03_rb,0.47382e-03_rb,0.49304e-03_rb,0.58474e-03_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.10532e-02_rb,0.92189e-03_rb,0.79057e-03_rb,0.68536e-03_rb,0.61794e-03_rb, &
        & 0.56535e-03_rb,0.53040e-03_rb,0.57934e-03_rb,0.68761e-03_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.10466e-02_rb,0.91613e-03_rb,0.78805e-03_rb,0.70517e-03_rb,0.65240e-03_rb, &
        & 0.61071e-03_rb,0.61067e-03_rb,0.68776e-03_rb,0.81827e-03_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.10406e-02_rb,0.91088e-03_rb,0.78949e-03_rb,0.73011e-03_rb,0.68643e-03_rb, &
        & 0.67486e-03_rb,0.72316e-03_rb,0.82048e-03_rb,0.97591e-03_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.10350e-02_rb,0.90609e-03_rb,0.80184e-03_rb,0.75651e-03_rb,0.73102e-03_rb, &
        & 0.75672e-03_rb,0.85035e-03_rb,0.96398e-03_rb,0.11483e-02_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.11788e-02_rb,0.10317e-02_rb,0.88462e-03_rb,0.75562e-03_rb,0.64370e-03_rb, &
        & 0.55868e-03_rb,0.48899e-03_rb,0.48008e-03_rb,0.57112e-03_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.11725e-02_rb,0.10263e-02_rb,0.87998e-03_rb,0.75781e-03_rb,0.66989e-03_rb, &
        & 0.59814e-03_rb,0.54177e-03_rb,0.56164e-03_rb,0.66999e-03_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.11654e-02_rb,0.10201e-02_rb,0.87720e-03_rb,0.76990e-03_rb,0.70381e-03_rb, &
        & 0.63928e-03_rb,0.60787e-03_rb,0.66447e-03_rb,0.79300e-03_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.11592e-02_rb,0.10147e-02_rb,0.87854e-03_rb,0.79340e-03_rb,0.73787e-03_rb, &
        & 0.69602e-03_rb,0.70539e-03_rb,0.79266e-03_rb,0.94522e-03_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.11532e-02_rb,0.10094e-02_rb,0.88452e-03_rb,0.81797e-03_rb,0.78065e-03_rb, &
        & 0.77762e-03_rb,0.82734e-03_rb,0.94053e-03_rb,0.11199e-02_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.13090e-02_rb,0.11455e-02_rb,0.98213e-03_rb,0.83571e-03_rb,0.70447e-03_rb, &
        & 0.59736e-03_rb,0.51100e-03_rb,0.46541e-03_rb,0.54756e-03_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.13011e-02_rb,0.11387e-02_rb,0.97633e-03_rb,0.83818e-03_rb,0.72563e-03_rb, &
        & 0.63146e-03_rb,0.56040e-03_rb,0.54671e-03_rb,0.65045e-03_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.12919e-02_rb,0.11307e-02_rb,0.97249e-03_rb,0.84355e-03_rb,0.75615e-03_rb, &
        & 0.67456e-03_rb,0.62106e-03_rb,0.64503e-03_rb,0.77294e-03_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.12834e-02_rb,0.11233e-02_rb,0.97228e-03_rb,0.86290e-03_rb,0.79001e-03_rb, &
        & 0.72644e-03_rb,0.70406e-03_rb,0.76742e-03_rb,0.92034e-03_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.12754e-02_rb,0.11163e-02_rb,0.97447e-03_rb,0.88787e-03_rb,0.82956e-03_rb, &
        & 0.80041e-03_rb,0.81433e-03_rb,0.91302e-03_rb,0.10923e-02_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.14384e-02_rb,0.12588e-02_rb,0.10791e-02_rb,0.91745e-03_rb,0.77222e-03_rb, &
        & 0.63777e-03_rb,0.53552e-03_rb,0.44917e-03_rb,0.50804e-03_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.14296e-02_rb,0.12511e-02_rb,0.10726e-02_rb,0.92106e-03_rb,0.78721e-03_rb, &
        & 0.67056e-03_rb,0.58135e-03_rb,0.52600e-03_rb,0.61657e-03_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.14197e-02_rb,0.12425e-02_rb,0.10687e-02_rb,0.92311e-03_rb,0.81466e-03_rb, &
        & 0.70925e-03_rb,0.63651e-03_rb,0.62263e-03_rb,0.74116e-03_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.14113e-02_rb,0.12351e-02_rb,0.10686e-02_rb,0.93888e-03_rb,0.84557e-03_rb, &
        & 0.75721e-03_rb,0.71295e-03_rb,0.74435e-03_rb,0.88914e-03_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.14033e-02_rb,0.12282e-02_rb,0.10694e-02_rb,0.96147e-03_rb,0.87768e-03_rb, &
        & 0.82519e-03_rb,0.81219e-03_rb,0.88552e-03_rb,0.10599e-02_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.15646e-02_rb,0.13692e-02_rb,0.11737e-02_rb,0.99670e-03_rb,0.83910e-03_rb, &
        & 0.68337e-03_rb,0.56133e-03_rb,0.43784e-03_rb,0.46053e-03_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.15575e-02_rb,0.13630e-02_rb,0.11685e-02_rb,0.10035e-02_rb,0.85062e-03_rb, &
        & 0.71436e-03_rb,0.60284e-03_rb,0.50378e-03_rb,0.56665e-03_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.15495e-02_rb,0.13560e-02_rb,0.11666e-02_rb,0.10059e-02_rb,0.87670e-03_rb, &
        & 0.74963e-03_rb,0.65339e-03_rb,0.59583e-03_rb,0.69467e-03_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.15420e-02_rb,0.13494e-02_rb,0.11678e-02_rb,0.10189e-02_rb,0.90417e-03_rb, &
        & 0.79237e-03_rb,0.72265e-03_rb,0.71412e-03_rb,0.84581e-03_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.15345e-02_rb,0.13429e-02_rb,0.11695e-02_rb,0.10414e-02_rb,0.93588e-03_rb, &
        & 0.84978e-03_rb,0.81799e-03_rb,0.85682e-03_rb,0.10184e-02_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.16856e-02_rb,0.14750e-02_rb,0.12645e-02_rb,0.10725e-02_rb,0.90201e-03_rb, &
        & 0.73159e-03_rb,0.58478e-03_rb,0.43268e-03_rb,0.41101e-03_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.16806e-02_rb,0.14706e-02_rb,0.12609e-02_rb,0.10836e-02_rb,0.91363e-03_rb, &
        & 0.76033e-03_rb,0.62810e-03_rb,0.48786e-03_rb,0.51346e-03_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.16744e-02_rb,0.14652e-02_rb,0.12612e-02_rb,0.10887e-02_rb,0.93599e-03_rb, &
        & 0.79815e-03_rb,0.66916e-03_rb,0.56926e-03_rb,0.63978e-03_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.16684e-02_rb,0.14601e-02_rb,0.12641e-02_rb,0.11000e-02_rb,0.96490e-03_rb, &
        & 0.83543e-03_rb,0.73243e-03_rb,0.67845e-03_rb,0.78892e-03_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.16618e-02_rb,0.14543e-02_rb,0.12671e-02_rb,0.11238e-02_rb,0.99451e-03_rb, &
        & 0.88426e-03_rb,0.81935e-03_rb,0.81741e-03_rb,0.96330e-03_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.17934e-02_rb,0.15694e-02_rb,0.13453e-02_rb,0.11410e-02_rb,0.95885e-03_rb, &
        & 0.77818e-03_rb,0.60653e-03_rb,0.43079e-03_rb,0.36156e-03_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.17902e-02_rb,0.15666e-02_rb,0.13435e-02_rb,0.11557e-02_rb,0.97291e-03_rb, &
        & 0.80328e-03_rb,0.65101e-03_rb,0.47930e-03_rb,0.45791e-03_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.17870e-02_rb,0.15637e-02_rb,0.13465e-02_rb,0.11644e-02_rb,0.99290e-03_rb, &
        & 0.84365e-03_rb,0.69043e-03_rb,0.54887e-03_rb,0.58027e-03_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.17836e-02_rb,0.15608e-02_rb,0.13520e-02_rb,0.11761e-02_rb,0.10245e-02_rb, &
        & 0.87956e-03_rb,0.74487e-03_rb,0.64626e-03_rb,0.72649e-03_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.17784e-02_rb,0.15563e-02_rb,0.13567e-02_rb,0.11994e-02_rb,0.10543e-02_rb, &
        & 0.92517e-03_rb,0.82194e-03_rb,0.77306e-03_rb,0.89652e-03_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.18870e-02_rb,0.16513e-02_rb,0.14155e-02_rb,0.12026e-02_rb,0.10094e-02_rb, &
        & 0.82157e-03_rb,0.62812e-03_rb,0.43279e-03_rb,0.32100e-03_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.18874e-02_rb,0.16516e-02_rb,0.14168e-02_rb,0.12199e-02_rb,0.10267e-02_rb, &
        & 0.84527e-03_rb,0.67192e-03_rb,0.47951e-03_rb,0.41207e-03_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.18869e-02_rb,0.16512e-02_rb,0.14226e-02_rb,0.12326e-02_rb,0.10456e-02_rb, &
        & 0.88603e-03_rb,0.71450e-03_rb,0.53998e-03_rb,0.52708e-03_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.18844e-02_rb,0.16490e-02_rb,0.14297e-02_rb,0.12452e-02_rb,0.10784e-02_rb, &
        & 0.92565e-03_rb,0.76291e-03_rb,0.62910e-03_rb,0.66842e-03_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.18812e-02_rb,0.16463e-02_rb,0.14369e-02_rb,0.12677e-02_rb,0.11133e-02_rb, &
        & 0.96733e-03_rb,0.83676e-03_rb,0.74378e-03_rb,0.83514e-03_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.19652e-02_rb,0.17197e-02_rb,0.14741e-02_rb,0.12638e-02_rb,0.10628e-02_rb, &
        & 0.86832e-03_rb,0.66314e-03_rb,0.45418e-03_rb,0.32052e-03_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.19706e-02_rb,0.17244e-02_rb,0.14828e-02_rb,0.12836e-02_rb,0.10826e-02_rb, &
        & 0.89693e-03_rb,0.71120e-03_rb,0.50274e-03_rb,0.41269e-03_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.19727e-02_rb,0.17263e-02_rb,0.14935e-02_rb,0.12979e-02_rb,0.11060e-02_rb, &
        & 0.94341e-03_rb,0.75411e-03_rb,0.56772e-03_rb,0.53374e-03_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.19726e-02_rb,0.17262e-02_rb,0.15047e-02_rb,0.13132e-02_rb,0.11439e-02_rb, &
        & 0.98450e-03_rb,0.80814e-03_rb,0.66163e-03_rb,0.68012e-03_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.19717e-02_rb,0.17254e-02_rb,0.15111e-02_rb,0.13415e-02_rb,0.11812e-02_rb, &
        & 0.10302e-02_rb,0.88976e-03_rb,0.77856e-03_rb,0.85217e-03_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.20238e-02_rb,0.17709e-02_rb,0.15209e-02_rb,0.13140e-02_rb,0.11103e-02_rb, &
        & 0.90805e-03_rb,0.69487e-03_rb,0.47365e-03_rb,0.32170e-03_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.20339e-02_rb,0.17798e-02_rb,0.15376e-02_rb,0.13369e-02_rb,0.11339e-02_rb, &
        & 0.94414e-03_rb,0.74487e-03_rb,0.52470e-03_rb,0.41340e-03_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.20400e-02_rb,0.17851e-02_rb,0.15537e-02_rb,0.13538e-02_rb,0.11634e-02_rb, &
        & 0.99316e-03_rb,0.79139e-03_rb,0.59483e-03_rb,0.53800e-03_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.20440e-02_rb,0.17887e-02_rb,0.15672e-02_rb,0.13751e-02_rb,0.12054e-02_rb, &
        & 0.10358e-02_rb,0.85239e-03_rb,0.69118e-03_rb,0.69062e-03_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.20478e-02_rb,0.17920e-02_rb,0.15770e-02_rb,0.14084e-02_rb,0.12434e-02_rb, &
        & 0.10883e-02_rb,0.94102e-03_rb,0.81111e-03_rb,0.86624e-03_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.20618e-02_rb,0.18042e-02_rb,0.15564e-02_rb,0.13520e-02_rb,0.11495e-02_rb, &
        & 0.94146e-03_rb,0.72197e-03_rb,0.49143e-03_rb,0.32080e-03_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.20769e-02_rb,0.18174e-02_rb,0.15793e-02_rb,0.13781e-02_rb,0.11781e-02_rb, &
        & 0.98504e-03_rb,0.77410e-03_rb,0.54584e-03_rb,0.41409e-03_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.20884e-02_rb,0.18274e-02_rb,0.16007e-02_rb,0.14008e-02_rb,0.12152e-02_rb, &
        & 0.10359e-02_rb,0.82528e-03_rb,0.62085e-03_rb,0.53970e-03_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.21000e-02_rb,0.18376e-02_rb,0.16180e-02_rb,0.14292e-02_rb,0.12615e-02_rb, &
        & 0.10817e-02_rb,0.89457e-03_rb,0.71909e-03_rb,0.69474e-03_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.21077e-02_rb,0.18456e-02_rb,0.16335e-02_rb,0.14673e-02_rb,0.12995e-02_rb, &
        & 0.11413e-02_rb,0.98821e-03_rb,0.84127e-03_rb,0.87402e-03_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.11199e-02_rb,0.98039e-03_rb,0.84084e-03_rb,0.70943e-03_rb,0.63603e-03_rb, &
        & 0.60649e-03_rb,0.68312e-03_rb,0.76350e-03_rb,0.90110e-03_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.11157e-02_rb,0.97670e-03_rb,0.83773e-03_rb,0.72395e-03_rb,0.68641e-03_rb, &
        & 0.69858e-03_rb,0.81907e-03_rb,0.92056e-03_rb,0.10823e-02_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.11108e-02_rb,0.97243e-03_rb,0.83426e-03_rb,0.74791e-03_rb,0.75340e-03_rb, &
        & 0.82026e-03_rb,0.97233e-03_rb,0.10968e-02_rb,0.12857e-02_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.11077e-02_rb,0.96981e-03_rb,0.83196e-03_rb,0.80006e-03_rb,0.82593e-03_rb, &
        & 0.96481e-03_rb,0.11451e-02_rb,0.12942e-02_rb,0.15153e-02_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.11057e-02_rb,0.96811e-03_rb,0.83287e-03_rb,0.85771e-03_rb,0.92243e-03_rb, &
        & 0.11291e-02_rb,0.13401e-02_rb,0.15139e-02_rb,0.17730e-02_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.12618e-02_rb,0.11045e-02_rb,0.94731e-03_rb,0.80059e-03_rb,0.70460e-03_rb, &
        & 0.66164e-03_rb,0.71798e-03_rb,0.80280e-03_rb,0.96182e-03_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.12573e-02_rb,0.11007e-02_rb,0.94415e-03_rb,0.81463e-03_rb,0.74844e-03_rb, &
        & 0.74157e-03_rb,0.85097e-03_rb,0.95625e-03_rb,0.11395e-02_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.12537e-02_rb,0.10976e-02_rb,0.94166e-03_rb,0.83421e-03_rb,0.81760e-03_rb, &
        & 0.85124e-03_rb,0.10029e-02_rb,0.11319e-02_rb,0.13421e-02_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.12517e-02_rb,0.10959e-02_rb,0.94019e-03_rb,0.88029e-03_rb,0.88890e-03_rb, &
        & 0.98937e-03_rb,0.11747e-02_rb,0.13301e-02_rb,0.15715e-02_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.12484e-02_rb,0.10932e-02_rb,0.93873e-03_rb,0.93946e-03_rb,0.97784e-03_rb, &
        & 0.11555e-02_rb,0.13735e-02_rb,0.15606e-02_rb,0.18367e-02_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.14334e-02_rb,0.12547e-02_rb,0.10760e-02_rb,0.90369e-03_rb,0.78163e-03_rb, &
        & 0.71064e-03_rb,0.72074e-03_rb,0.81006e-03_rb,0.97636e-03_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.14266e-02_rb,0.12488e-02_rb,0.10711e-02_rb,0.91657e-03_rb,0.81392e-03_rb, &
        & 0.78379e-03_rb,0.85758e-03_rb,0.96540e-03_rb,0.11622e-02_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.14223e-02_rb,0.12452e-02_rb,0.10680e-02_rb,0.93146e-03_rb,0.87510e-03_rb, &
        & 0.87756e-03_rb,0.10059e-02_rb,0.11359e-02_rb,0.13619e-02_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.14175e-02_rb,0.12411e-02_rb,0.10646e-02_rb,0.96483e-03_rb,0.94572e-03_rb, &
        & 0.99384e-03_rb,0.11710e-02_rb,0.13256e-02_rb,0.15838e-02_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.14120e-02_rb,0.12363e-02_rb,0.10614e-02_rb,0.10223e-02_rb,0.10225e-02_rb, &
        & 0.11461e-02_rb,0.13616e-02_rb,0.15455e-02_rb,0.18394e-02_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.16334e-02_rb,0.14296e-02_rb,0.12259e-02_rb,0.10262e-02_rb,0.87529e-03_rb, &
        & 0.76396e-03_rb,0.72119e-03_rb,0.79306e-03_rb,0.95651e-03_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.16232e-02_rb,0.14208e-02_rb,0.12185e-02_rb,0.10348e-02_rb,0.89367e-03_rb, &
        & 0.83199e-03_rb,0.84657e-03_rb,0.95561e-03_rb,0.11474e-02_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.16152e-02_rb,0.14140e-02_rb,0.12127e-02_rb,0.10485e-02_rb,0.94236e-03_rb, &
        & 0.91572e-03_rb,0.99902e-03_rb,0.11299e-02_rb,0.13583e-02_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.16069e-02_rb,0.14067e-02_rb,0.12065e-02_rb,0.10685e-02_rb,0.10126e-02_rb, &
        & 0.10172e-02_rb,0.11639e-02_rb,0.13179e-02_rb,0.15822e-02_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.15986e-02_rb,0.13996e-02_rb,0.12009e-02_rb,0.11161e-02_rb,0.10822e-02_rb, &
        & 0.11448e-02_rb,0.13482e-02_rb,0.15289e-02_rb,0.18307e-02_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.18523e-02_rb,0.16211e-02_rb,0.13900e-02_rb,0.11617e-02_rb,0.98609e-03_rb, &
        & 0.83071e-03_rb,0.73506e-03_rb,0.77421e-03_rb,0.93359e-03_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.18399e-02_rb,0.16104e-02_rb,0.13808e-02_rb,0.11680e-02_rb,0.99824e-03_rb, &
        & 0.89112e-03_rb,0.84383e-03_rb,0.93250e-03_rb,0.11210e-02_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.18296e-02_rb,0.16015e-02_rb,0.13734e-02_rb,0.11806e-02_rb,0.10315e-02_rb, &
        & 0.96812e-03_rb,0.98130e-03_rb,0.11095e-02_rb,0.13275e-02_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.18179e-02_rb,0.15913e-02_rb,0.13647e-02_rb,0.11924e-02_rb,0.10905e-02_rb, &
        & 0.10578e-02_rb,0.11513e-02_rb,0.13088e-02_rb,0.15638e-02_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.18067e-02_rb,0.15816e-02_rb,0.13575e-02_rb,0.12252e-02_rb,0.11627e-02_rb, &
        & 0.11690e-02_rb,0.13357e-02_rb,0.15168e-02_rb,0.18162e-02_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.20925e-02_rb,0.18312e-02_rb,0.15700e-02_rb,0.13095e-02_rb,0.11041e-02_rb, &
        & 0.91669e-03_rb,0.76437e-03_rb,0.75155e-03_rb,0.90862e-03_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.20749e-02_rb,0.18159e-02_rb,0.15569e-02_rb,0.13117e-02_rb,0.11170e-02_rb, &
        & 0.95740e-03_rb,0.85349e-03_rb,0.90211e-03_rb,0.10843e-02_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.20605e-02_rb,0.18034e-02_rb,0.15463e-02_rb,0.13221e-02_rb,0.11384e-02_rb, &
        & 0.10293e-02_rb,0.97333e-03_rb,0.10765e-02_rb,0.12853e-02_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.20459e-02_rb,0.17907e-02_rb,0.15355e-02_rb,0.13321e-02_rb,0.11878e-02_rb, &
        & 0.11138e-02_rb,0.11236e-02_rb,0.12716e-02_rb,0.15170e-02_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.20310e-02_rb,0.17778e-02_rb,0.15259e-02_rb,0.13542e-02_rb,0.12517e-02_rb, &
        & 0.12095e-02_rb,0.13081e-02_rb,0.14898e-02_rb,0.17755e-02_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.23595e-02_rb,0.20648e-02_rb,0.17701e-02_rb,0.14754e-02_rb,0.12323e-02_rb, &
        & 0.10132e-02_rb,0.80835e-03_rb,0.73377e-03_rb,0.88458e-03_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.23333e-02_rb,0.20420e-02_rb,0.17506e-02_rb,0.14700e-02_rb,0.12449e-02_rb, &
        & 0.10427e-02_rb,0.88482e-03_rb,0.87865e-03_rb,0.10548e-02_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.23118e-02_rb,0.20232e-02_rb,0.17345e-02_rb,0.14754e-02_rb,0.12581e-02_rb, &
        & 0.11040e-02_rb,0.98425e-03_rb,0.10425e-02_rb,0.12451e-02_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.22911e-02_rb,0.20052e-02_rb,0.17193e-02_rb,0.14831e-02_rb,0.12996e-02_rb, &
        & 0.11778e-02_rb,0.11137e-02_rb,0.12310e-02_rb,0.14656e-02_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.22715e-02_rb,0.19881e-02_rb,0.17061e-02_rb,0.14949e-02_rb,0.13595e-02_rb, &
        & 0.12702e-02_rb,0.12792e-02_rb,0.14480e-02_rb,0.17220e-02_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.26465e-02_rb,0.23159e-02_rb,0.19853e-02_rb,0.16547e-02_rb,0.13704e-02_rb, &
        & 0.11161e-02_rb,0.87227e-03_rb,0.72579e-03_rb,0.85316e-03_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.26139e-02_rb,0.22874e-02_rb,0.19609e-02_rb,0.16420e-02_rb,0.13837e-02_rb, &
        & 0.11377e-02_rb,0.93183e-03_rb,0.85926e-03_rb,0.10245e-02_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.25857e-02_rb,0.22628e-02_rb,0.19400e-02_rb,0.16425e-02_rb,0.13917e-02_rb, &
        & 0.11859e-02_rb,0.10228e-02_rb,0.10157e-02_rb,0.12103e-02_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.25573e-02_rb,0.22380e-02_rb,0.19187e-02_rb,0.16463e-02_rb,0.14196e-02_rb, &
        & 0.12582e-02_rb,0.11286e-02_rb,0.11958e-02_rb,0.14229e-02_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.25310e-02_rb,0.22151e-02_rb,0.19006e-02_rb,0.16478e-02_rb,0.14771e-02_rb, &
        & 0.13382e-02_rb,0.12711e-02_rb,0.14043e-02_rb,0.16688e-02_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.29437e-02_rb,0.25759e-02_rb,0.22081e-02_rb,0.18403e-02_rb,0.15126e-02_rb, &
        & 0.12266e-02_rb,0.94502e-03_rb,0.72553e-03_rb,0.80872e-03_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.29059e-02_rb,0.25428e-02_rb,0.21798e-02_rb,0.18221e-02_rb,0.15299e-02_rb, &
        & 0.12446e-02_rb,0.99114e-03_rb,0.84495e-03_rb,0.98123e-03_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.28735e-02_rb,0.25146e-02_rb,0.21557e-02_rb,0.18193e-02_rb,0.15373e-02_rb, &
        & 0.12770e-02_rb,0.10730e-02_rb,0.99151e-03_rb,0.11701e-02_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.28399e-02_rb,0.24853e-02_rb,0.21306e-02_rb,0.18209e-02_rb,0.15526e-02_rb, &
        & 0.13434e-02_rb,0.11682e-02_rb,0.11617e-02_rb,0.13773e-02_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.28070e-02_rb,0.24565e-02_rb,0.21073e-02_rb,0.18175e-02_rb,0.15973e-02_rb, &
        & 0.14179e-02_rb,0.12830e-02_rb,0.13638e-02_rb,0.16173e-02_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.32399e-02_rb,0.28350e-02_rb,0.24302e-02_rb,0.20252e-02_rb,0.16589e-02_rb, &
        & 0.13407e-02_rb,0.10274e-02_rb,0.73986e-03_rb,0.76203e-03_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.31965e-02_rb,0.27971e-02_rb,0.23977e-02_rb,0.20029e-02_rb,0.16776e-02_rb, &
        & 0.13600e-02_rb,0.10652e-02_rb,0.84551e-03_rb,0.93841e-03_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.31601e-02_rb,0.27653e-02_rb,0.23707e-02_rb,0.19975e-02_rb,0.16883e-02_rb, &
        & 0.13843e-02_rb,0.11385e-02_rb,0.98230e-03_rb,0.11352e-02_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.31230e-02_rb,0.27329e-02_rb,0.23428e-02_rb,0.19986e-02_rb,0.16986e-02_rb, &
        & 0.14381e-02_rb,0.12318e-02_rb,0.11431e-02_rb,0.13466e-02_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.30838e-02_rb,0.26986e-02_rb,0.23157e-02_rb,0.19957e-02_rb,0.17285e-02_rb, &
        & 0.15111e-02_rb,0.13270e-02_rb,0.13396e-02_rb,0.15853e-02_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.35110e-02_rb,0.30722e-02_rb,0.26335e-02_rb,0.21948e-02_rb,0.18129e-02_rb, &
        & 0.14641e-02_rb,0.11256e-02_rb,0.79561e-03_rb,0.78300e-03_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.34682e-02_rb,0.30349e-02_rb,0.26015e-02_rb,0.21788e-02_rb,0.18300e-02_rb, &
        & 0.14846e-02_rb,0.11655e-02_rb,0.90541e-03_rb,0.97352e-03_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.34263e-02_rb,0.29982e-02_rb,0.25700e-02_rb,0.21750e-02_rb,0.18411e-02_rb, &
        & 0.15104e-02_rb,0.12444e-02_rb,0.10462e-02_rb,0.11863e-02_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.33817e-02_rb,0.29592e-02_rb,0.25369e-02_rb,0.21760e-02_rb,0.18525e-02_rb, &
        & 0.15666e-02_rb,0.13382e-02_rb,0.12088e-02_rb,0.14124e-02_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.33351e-02_rb,0.29184e-02_rb,0.25088e-02_rb,0.21673e-02_rb,0.18858e-02_rb, &
        & 0.16404e-02_rb,0.14339e-02_rb,0.14160e-02_rb,0.16713e-02_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.37813e-02_rb,0.33087e-02_rb,0.28362e-02_rb,0.23654e-02_rb,0.19642e-02_rb, &
        & 0.15868e-02_rb,0.12220e-02_rb,0.85134e-03_rb,0.79810e-03_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.37337e-02_rb,0.32671e-02_rb,0.28006e-02_rb,0.23535e-02_rb,0.19812e-02_rb, &
        & 0.16073e-02_rb,0.12646e-02_rb,0.96230e-03_rb,0.10021e-02_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.36838e-02_rb,0.32235e-02_rb,0.27633e-02_rb,0.23494e-02_rb,0.19898e-02_rb, &
        & 0.16363e-02_rb,0.13472e-02_rb,0.11056e-02_rb,0.12297e-02_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.36315e-02_rb,0.31778e-02_rb,0.27268e-02_rb,0.23457e-02_rb,0.20010e-02_rb, &
        & 0.16953e-02_rb,0.14396e-02_rb,0.12736e-02_rb,0.14737e-02_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.35780e-02_rb,0.31310e-02_rb,0.26962e-02_rb,0.23324e-02_rb,0.20383e-02_rb, &
        & 0.17680e-02_rb,0.15379e-02_rb,0.14913e-02_rb,0.17515e-02_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.40401e-02_rb,0.35352e-02_rb,0.30303e-02_rb,0.25331e-02_rb,0.21125e-02_rb, &
        & 0.17061e-02_rb,0.13144e-02_rb,0.90469e-03_rb,0.81129e-03_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.39897e-02_rb,0.34912e-02_rb,0.29926e-02_rb,0.25245e-02_rb,0.21272e-02_rb, &
        & 0.17273e-02_rb,0.13608e-02_rb,0.10148e-02_rb,0.10248e-02_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.39313e-02_rb,0.34401e-02_rb,0.29487e-02_rb,0.25178e-02_rb,0.21303e-02_rb, &
        & 0.17587e-02_rb,0.14454e-02_rb,0.11592e-02_rb,0.12656e-02_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.38694e-02_rb,0.33859e-02_rb,0.29093e-02_rb,0.25068e-02_rb,0.21412e-02_rb, &
        & 0.18203e-02_rb,0.15361e-02_rb,0.13357e-02_rb,0.15298e-02_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.38109e-02_rb,0.33347e-02_rb,0.28761e-02_rb,0.24907e-02_rb,0.21821e-02_rb, &
        & 0.18916e-02_rb,0.16371e-02_rb,0.15629e-02_rb,0.18275e-02_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.15071e-02_rb,0.13191e-02_rb,0.11311e-02_rb,0.94313e-03_rb,0.82178e-03_rb, &
        & 0.90230e-03_rb,0.10652e-02_rb,0.11879e-02_rb,0.14045e-02_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.15053e-02_rb,0.13176e-02_rb,0.11298e-02_rb,0.94210e-03_rb,0.87562e-03_rb, &
        & 0.10432e-02_rb,0.12302e-02_rb,0.13664e-02_rb,0.16201e-02_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.15019e-02_rb,0.13147e-02_rb,0.11275e-02_rb,0.94187e-03_rb,0.97652e-03_rb, &
        & 0.12093e-02_rb,0.14252e-02_rb,0.15816e-02_rb,0.18766e-02_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.14976e-02_rb,0.13109e-02_rb,0.11243e-02_rb,0.96063e-03_rb,0.11391e-02_rb, &
        & 0.14106e-02_rb,0.16622e-02_rb,0.18446e-02_rb,0.21889e-02_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.14918e-02_rb,0.13060e-02_rb,0.11201e-02_rb,0.10256e-02_rb,0.13135e-02_rb, &
        & 0.16256e-02_rb,0.19138e-02_rb,0.21174e-02_rb,0.25177e-02_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.17368e-02_rb,0.15199e-02_rb,0.13031e-02_rb,0.10865e-02_rb,0.93589e-03_rb, &
        & 0.95913e-03_rb,0.11327e-02_rb,0.12643e-02_rb,0.15179e-02_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.17347e-02_rb,0.15183e-02_rb,0.13018e-02_rb,0.10853e-02_rb,0.98162e-03_rb, &
        & 0.11204e-02_rb,0.13214e-02_rb,0.14695e-02_rb,0.17719e-02_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.17301e-02_rb,0.15143e-02_rb,0.12982e-02_rb,0.10867e-02_rb,0.10624e-02_rb, &
        & 0.13058e-02_rb,0.15388e-02_rb,0.17088e-02_rb,0.20640e-02_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.17235e-02_rb,0.15085e-02_rb,0.12935e-02_rb,0.11059e-02_rb,0.12337e-02_rb, &
        & 0.15285e-02_rb,0.18009e-02_rb,0.19982e-02_rb,0.24152e-02_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.17181e-02_rb,0.15038e-02_rb,0.12895e-02_rb,0.11598e-02_rb,0.14379e-02_rb, &
        & 0.17807e-02_rb,0.20976e-02_rb,0.23230e-02_rb,0.28132e-02_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.19949e-02_rb,0.17458e-02_rb,0.14967e-02_rb,0.12475e-02_rb,0.10641e-02_rb, &
        & 0.10100e-02_rb,0.11774e-02_rb,0.13116e-02_rb,0.16051e-02_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.19910e-02_rb,0.17424e-02_rb,0.14939e-02_rb,0.12453e-02_rb,0.11049e-02_rb, &
        & 0.11564e-02_rb,0.13657e-02_rb,0.15222e-02_rb,0.18601e-02_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.19848e-02_rb,0.17370e-02_rb,0.14892e-02_rb,0.12476e-02_rb,0.11571e-02_rb, &
        & 0.13573e-02_rb,0.16014e-02_rb,0.17817e-02_rb,0.21839e-02_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.19813e-02_rb,0.17340e-02_rb,0.14867e-02_rb,0.12671e-02_rb,0.12941e-02_rb, &
        & 0.15954e-02_rb,0.18821e-02_rb,0.20919e-02_rb,0.25670e-02_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.19771e-02_rb,0.17304e-02_rb,0.14836e-02_rb,0.13013e-02_rb,0.15091e-02_rb, &
        & 0.18697e-02_rb,0.22058e-02_rb,0.24525e-02_rb,0.30084e-02_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.23002e-02_rb,0.20126e-02_rb,0.17253e-02_rb,0.14382e-02_rb,0.12005e-02_rb, &
        & 0.10932e-02_rb,0.12228e-02_rb,0.13639e-02_rb,0.16798e-02_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.22929e-02_rb,0.20065e-02_rb,0.17201e-02_rb,0.14337e-02_rb,0.12491e-02_rb, &
        & 0.11967e-02_rb,0.14046e-02_rb,0.15632e-02_rb,0.19322e-02_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.22828e-02_rb,0.19977e-02_rb,0.17127e-02_rb,0.14287e-02_rb,0.12914e-02_rb, &
        & 0.13860e-02_rb,0.16360e-02_rb,0.18240e-02_rb,0.22487e-02_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.22757e-02_rb,0.19915e-02_rb,0.17074e-02_rb,0.14457e-02_rb,0.13800e-02_rb, &
        & 0.16339e-02_rb,0.19297e-02_rb,0.21519e-02_rb,0.26493e-02_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.22684e-02_rb,0.19852e-02_rb,0.17019e-02_rb,0.14754e-02_rb,0.15614e-02_rb, &
        & 0.19242e-02_rb,0.22727e-02_rb,0.25341e-02_rb,0.31196e-02_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.26562e-02_rb,0.23246e-02_rb,0.19929e-02_rb,0.16611e-02_rb,0.13598e-02_rb, &
        & 0.11996e-02_rb,0.12698e-02_rb,0.14174e-02_rb,0.17501e-02_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.26415e-02_rb,0.23116e-02_rb,0.19818e-02_rb,0.16521e-02_rb,0.14129e-02_rb, &
        & 0.12866e-02_rb,0.14524e-02_rb,0.16197e-02_rb,0.20022e-02_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.26277e-02_rb,0.22994e-02_rb,0.19710e-02_rb,0.16428e-02_rb,0.14513e-02_rb, &
        & 0.14339e-02_rb,0.16856e-02_rb,0.18777e-02_rb,0.23236e-02_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.26179e-02_rb,0.22909e-02_rb,0.19639e-02_rb,0.16526e-02_rb,0.15154e-02_rb, &
        & 0.16607e-02_rb,0.19618e-02_rb,0.21909e-02_rb,0.27021e-02_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.26074e-02_rb,0.22818e-02_rb,0.19561e-02_rb,0.16780e-02_rb,0.16497e-02_rb, &
        & 0.19592e-02_rb,0.23163e-02_rb,0.25915e-02_rb,0.31846e-02_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.30676e-02_rb,0.26845e-02_rb,0.23014e-02_rb,0.19182e-02_rb,0.15508e-02_rb, &
        & 0.13265e-02_rb,0.13165e-02_rb,0.14712e-02_rb,0.18123e-02_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.30472e-02_rb,0.26667e-02_rb,0.22862e-02_rb,0.19057e-02_rb,0.15947e-02_rb, &
        & 0.14067e-02_rb,0.15004e-02_rb,0.16750e-02_rb,0.20695e-02_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.30255e-02_rb,0.26477e-02_rb,0.22705e-02_rb,0.18926e-02_rb,0.16408e-02_rb, &
        & 0.15149e-02_rb,0.17276e-02_rb,0.19275e-02_rb,0.23847e-02_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.30083e-02_rb,0.26326e-02_rb,0.22569e-02_rb,0.18906e-02_rb,0.16765e-02_rb, &
        & 0.17066e-02_rb,0.20035e-02_rb,0.22386e-02_rb,0.27616e-02_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.29915e-02_rb,0.26179e-02_rb,0.22443e-02_rb,0.19114e-02_rb,0.17836e-02_rb, &
        & 0.19800e-02_rb,0.23411e-02_rb,0.26194e-02_rb,0.32234e-02_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.35311e-02_rb,0.30900e-02_rb,0.26490e-02_rb,0.22082e-02_rb,0.17722e-02_rb, &
        & 0.14850e-02_rb,0.13781e-02_rb,0.15158e-02_rb,0.18578e-02_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.35031e-02_rb,0.30656e-02_rb,0.26281e-02_rb,0.21907e-02_rb,0.18048e-02_rb, &
        & 0.15528e-02_rb,0.15481e-02_rb,0.17322e-02_rb,0.21307e-02_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.34762e-02_rb,0.30421e-02_rb,0.26077e-02_rb,0.21737e-02_rb,0.18550e-02_rb, &
        & 0.16362e-02_rb,0.17771e-02_rb,0.19863e-02_rb,0.24506e-02_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.34506e-02_rb,0.30197e-02_rb,0.25888e-02_rb,0.21628e-02_rb,0.18864e-02_rb, &
        & 0.17891e-02_rb,0.20527e-02_rb,0.22948e-02_rb,0.28308e-02_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.34242e-02_rb,0.29966e-02_rb,0.25690e-02_rb,0.21752e-02_rb,0.19463e-02_rb, &
        & 0.20224e-02_rb,0.23798e-02_rb,0.26659e-02_rb,0.32785e-02_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.40545e-02_rb,0.35480e-02_rb,0.30415e-02_rb,0.25348e-02_rb,0.20284e-02_rb, &
        & 0.16698e-02_rb,0.14585e-02_rb,0.15473e-02_rb,0.18938e-02_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.40094e-02_rb,0.35086e-02_rb,0.30078e-02_rb,0.25070e-02_rb,0.20421e-02_rb, &
        & 0.17325e-02_rb,0.16159e-02_rb,0.17783e-02_rb,0.21732e-02_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.39679e-02_rb,0.34723e-02_rb,0.29772e-02_rb,0.24816e-02_rb,0.20882e-02_rb, &
        & 0.17972e-02_rb,0.18217e-02_rb,0.20430e-02_rb,0.25042e-02_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.39321e-02_rb,0.34411e-02_rb,0.29500e-02_rb,0.24630e-02_rb,0.21284e-02_rb, &
        & 0.19080e-02_rb,0.21043e-02_rb,0.23586e-02_rb,0.28966e-02_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.38953e-02_rb,0.34088e-02_rb,0.29224e-02_rb,0.24683e-02_rb,0.21576e-02_rb, &
        & 0.21070e-02_rb,0.24382e-02_rb,0.27335e-02_rb,0.33549e-02_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.46508e-02_rb,0.40697e-02_rb,0.34886e-02_rb,0.29075e-02_rb,0.23264e-02_rb, &
        & 0.18591e-02_rb,0.15578e-02_rb,0.15738e-02_rb,0.19224e-02_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.45823e-02_rb,0.40098e-02_rb,0.34374e-02_rb,0.28649e-02_rb,0.23092e-02_rb, &
        & 0.19366e-02_rb,0.17051e-02_rb,0.18133e-02_rb,0.22104e-02_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.45166e-02_rb,0.39524e-02_rb,0.33880e-02_rb,0.28238e-02_rb,0.23394e-02_rb, &
        & 0.19917e-02_rb,0.18843e-02_rb,0.20815e-02_rb,0.25365e-02_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.44602e-02_rb,0.39031e-02_rb,0.33460e-02_rb,0.27890e-02_rb,0.23856e-02_rb, &
        & 0.20682e-02_rb,0.21419e-02_rb,0.24093e-02_rb,0.29396e-02_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.44050e-02_rb,0.38549e-02_rb,0.33047e-02_rb,0.27817e-02_rb,0.24123e-02_rb, &
        & 0.22258e-02_rb,0.24850e-02_rb,0.27956e-02_rb,0.34119e-02_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.53098e-02_rb,0.46463e-02_rb,0.39828e-02_rb,0.33194e-02_rb,0.26559e-02_rb, &
        & 0.20766e-02_rb,0.16840e-02_rb,0.16129e-02_rb,0.19635e-02_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.52134e-02_rb,0.45620e-02_rb,0.39106e-02_rb,0.32592e-02_rb,0.26145e-02_rb, &
        & 0.21601e-02_rb,0.18269e-02_rb,0.18690e-02_rb,0.22681e-02_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.51182e-02_rb,0.44788e-02_rb,0.38392e-02_rb,0.31998e-02_rb,0.26226e-02_rb, &
        & 0.22138e-02_rb,0.19824e-02_rb,0.21416e-02_rb,0.25956e-02_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.50344e-02_rb,0.44054e-02_rb,0.37765e-02_rb,0.31477e-02_rb,0.26616e-02_rb, &
        & 0.22709e-02_rb,0.22099e-02_rb,0.24682e-02_rb,0.29962e-02_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.49550e-02_rb,0.43361e-02_rb,0.37172e-02_rb,0.31183e-02_rb,0.26883e-02_rb, &
        & 0.23873e-02_rb,0.25425e-02_rb,0.28708e-02_rb,0.34790e-02_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.59619e-02_rb,0.52169e-02_rb,0.44719e-02_rb,0.37267e-02_rb,0.29817e-02_rb, &
        & 0.23525e-02_rb,0.18854e-02_rb,0.17723e-02_rb,0.21445e-02_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.58318e-02_rb,0.51031e-02_rb,0.43744e-02_rb,0.36457e-02_rb,0.29351e-02_rb, &
        & 0.24253e-02_rb,0.20278e-02_rb,0.20449e-02_rb,0.24715e-02_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.57112e-02_rb,0.49976e-02_rb,0.42840e-02_rb,0.35704e-02_rb,0.29381e-02_rb, &
        & 0.24724e-02_rb,0.21871e-02_rb,0.23372e-02_rb,0.28231e-02_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.56008e-02_rb,0.49010e-02_rb,0.42013e-02_rb,0.35025e-02_rb,0.29677e-02_rb, &
        & 0.25269e-02_rb,0.24222e-02_rb,0.27009e-02_rb,0.32614e-02_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.54961e-02_rb,0.48096e-02_rb,0.41230e-02_rb,0.34666e-02_rb,0.29811e-02_rb, &
        & 0.26438e-02_rb,0.27691e-02_rb,0.31324e-02_rb,0.37747e-02_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.66218e-02_rb,0.57942e-02_rb,0.49667e-02_rb,0.41391e-02_rb,0.33116e-02_rb, &
        & 0.26382e-02_rb,0.20922e-02_rb,0.19344e-02_rb,0.23267e-02_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.64584e-02_rb,0.56513e-02_rb,0.48443e-02_rb,0.40372e-02_rb,0.32618e-02_rb, &
        & 0.26974e-02_rb,0.22343e-02_rb,0.22270e-02_rb,0.26783e-02_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.63097e-02_rb,0.55213e-02_rb,0.47331e-02_rb,0.39445e-02_rb,0.32599e-02_rb, &
        & 0.27357e-02_rb,0.23983e-02_rb,0.25418e-02_rb,0.30577e-02_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.61694e-02_rb,0.53986e-02_rb,0.46278e-02_rb,0.38631e-02_rb,0.32772e-02_rb, &
        & 0.27857e-02_rb,0.26412e-02_rb,0.29352e-02_rb,0.35233e-02_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.60360e-02_rb,0.52820e-02_rb,0.45279e-02_rb,0.38177e-02_rb,0.32735e-02_rb, &
        & 0.29011e-02_rb,0.29998e-02_rb,0.33948e-02_rb,0.40685e-02_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.72856e-02_rb,0.63751e-02_rb,0.54646e-02_rb,0.45539e-02_rb,0.36434e-02_rb, &
        & 0.29246e-02_rb,0.23024e-02_rb,0.20968e-02_rb,0.25004e-02_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.70792e-02_rb,0.61945e-02_rb,0.53099e-02_rb,0.44252e-02_rb,0.35875e-02_rb, &
        & 0.29681e-02_rb,0.24397e-02_rb,0.24073e-02_rb,0.28787e-02_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.68966e-02_rb,0.60348e-02_rb,0.51733e-02_rb,0.43112e-02_rb,0.35811e-02_rb, &
        & 0.29976e-02_rb,0.26071e-02_rb,0.27479e-02_rb,0.32886e-02_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.67258e-02_rb,0.58855e-02_rb,0.50451e-02_rb,0.42204e-02_rb,0.35830e-02_rb, &
        & 0.30400e-02_rb,0.28573e-02_rb,0.31625e-02_rb,0.37791e-02_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.65607e-02_rb,0.57411e-02_rb,0.49214e-02_rb,0.41617e-02_rb,0.35593e-02_rb, &
        & 0.31530e-02_rb,0.32258e-02_rb,0.36487e-02_rb,0.43508e-02_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.16712e-02_rb,0.14631e-02_rb,0.12550e-02_rb,0.10468e-02_rb,0.92082e-03_rb, &
        & 0.11371e-02_rb,0.13321e-02_rb,0.14544e-02_rb,0.17472e-02_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.16787e-02_rb,0.14701e-02_rb,0.12615e-02_rb,0.10530e-02_rb,0.10440e-02_rb, &
        & 0.12893e-02_rb,0.15114e-02_rb,0.16524e-02_rb,0.19826e-02_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.16875e-02_rb,0.14779e-02_rb,0.12683e-02_rb,0.10588e-02_rb,0.11676e-02_rb, &
        & 0.14424e-02_rb,0.16914e-02_rb,0.18523e-02_rb,0.22202e-02_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.16926e-02_rb,0.14826e-02_rb,0.12725e-02_rb,0.10626e-02_rb,0.12909e-02_rb, &
        & 0.15950e-02_rb,0.18720e-02_rb,0.20532e-02_rb,0.24579e-02_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.16942e-02_rb,0.14841e-02_rb,0.12739e-02_rb,0.11419e-02_rb,0.15149e-02_rb, &
        & 0.18796e-02_rb,0.22246e-02_rb,0.24968e-02_rb,0.29379e-02_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.19607e-02_rb,0.17158e-02_rb,0.14714e-02_rb,0.12274e-02_rb,0.10440e-02_rb, &
        & 0.12890e-02_rb,0.15097e-02_rb,0.16471e-02_rb,0.20285e-02_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.19666e-02_rb,0.17216e-02_rb,0.14767e-02_rb,0.12314e-02_rb,0.11872e-02_rb, &
        & 0.14659e-02_rb,0.17177e-02_rb,0.18761e-02_rb,0.23077e-02_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.19740e-02_rb,0.17284e-02_rb,0.14830e-02_rb,0.12374e-02_rb,0.13301e-02_rb, &
        & 0.16429e-02_rb,0.19255e-02_rb,0.21058e-02_rb,0.25866e-02_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.19796e-02_rb,0.17334e-02_rb,0.14872e-02_rb,0.12416e-02_rb,0.14717e-02_rb, &
        & 0.18182e-02_rb,0.21321e-02_rb,0.23342e-02_rb,0.28637e-02_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.19818e-02_rb,0.17354e-02_rb,0.14891e-02_rb,0.12544e-02_rb,0.16318e-02_rb, &
        & 0.20217e-02_rb,0.23832e-02_rb,0.26470e-02_rb,0.31951e-02_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.23039e-02_rb,0.20168e-02_rb,0.17292e-02_rb,0.14410e-02_rb,0.11614e-02_rb, &
        & 0.14213e-02_rb,0.16646e-02_rb,0.18149e-02_rb,0.22923e-02_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.23064e-02_rb,0.20187e-02_rb,0.17311e-02_rb,0.14440e-02_rb,0.13190e-02_rb, &
        & 0.16288e-02_rb,0.19083e-02_rb,0.20825e-02_rb,0.26267e-02_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.23119e-02_rb,0.20237e-02_rb,0.17351e-02_rb,0.14470e-02_rb,0.14864e-02_rb, &
        & 0.18354e-02_rb,0.21515e-02_rb,0.23501e-02_rb,0.29596e-02_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.23154e-02_rb,0.20269e-02_rb,0.17385e-02_rb,0.14496e-02_rb,0.16513e-02_rb, &
        & 0.20395e-02_rb,0.23915e-02_rb,0.26154e-02_rb,0.32883e-02_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.23212e-02_rb,0.20321e-02_rb,0.17430e-02_rb,0.14536e-02_rb,0.18129e-02_rb, &
        & 0.22398e-02_rb,0.26272e-02_rb,0.28760e-02_rb,0.36101e-02_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.27129e-02_rb,0.23736e-02_rb,0.20348e-02_rb,0.16965e-02_rb,0.13577e-02_rb, &
        & 0.15487e-02_rb,0.18138e-02_rb,0.19776e-02_rb,0.25289e-02_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.27133e-02_rb,0.23745e-02_rb,0.20358e-02_rb,0.16966e-02_rb,0.14516e-02_rb, &
        & 0.17925e-02_rb,0.21002e-02_rb,0.22917e-02_rb,0.29262e-02_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.27155e-02_rb,0.23766e-02_rb,0.20379e-02_rb,0.16990e-02_rb,0.16483e-02_rb, &
        & 0.20360e-02_rb,0.23858e-02_rb,0.26061e-02_rb,0.33224e-02_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.27175e-02_rb,0.23784e-02_rb,0.20394e-02_rb,0.17007e-02_rb,0.18426e-02_rb, &
        & 0.22757e-02_rb,0.26681e-02_rb,0.29171e-02_rb,0.37132e-02_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.27186e-02_rb,0.23796e-02_rb,0.20406e-02_rb,0.17019e-02_rb,0.20320e-02_rb, &
        & 0.25101e-02_rb,0.29443e-02_rb,0.32220e-02_rb,0.40947e-02_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.32119e-02_rb,0.28114e-02_rb,0.24099e-02_rb,0.20078e-02_rb,0.16065e-02_rb, &
        & 0.16767e-02_rb,0.19643e-02_rb,0.21431e-02_rb,0.27523e-02_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.32085e-02_rb,0.28077e-02_rb,0.24069e-02_rb,0.20064e-02_rb,0.16195e-02_rb, &
        & 0.19633e-02_rb,0.23007e-02_rb,0.25120e-02_rb,0.32215e-02_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.32085e-02_rb,0.28078e-02_rb,0.24068e-02_rb,0.20061e-02_rb,0.18214e-02_rb, &
        & 0.22501e-02_rb,0.26375e-02_rb,0.28827e-02_rb,0.36897e-02_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.32059e-02_rb,0.28056e-02_rb,0.24053e-02_rb,0.20050e-02_rb,0.20501e-02_rb, &
        & 0.25321e-02_rb,0.29699e-02_rb,0.32491e-02_rb,0.41521e-02_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.31984e-02_rb,0.27990e-02_rb,0.23997e-02_rb,0.20000e-02_rb,0.22727e-02_rb, &
        & 0.28084e-02_rb,0.32949e-02_rb,0.36079e-02_rb,0.46028e-02_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.38076e-02_rb,0.33317e-02_rb,0.28559e-02_rb,0.23799e-02_rb,0.19041e-02_rb, &
        & 0.17889e-02_rb,0.20969e-02_rb,0.22903e-02_rb,0.29420e-02_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.37964e-02_rb,0.33220e-02_rb,0.28476e-02_rb,0.23732e-02_rb,0.18989e-02_rb, &
        & 0.21234e-02_rb,0.24900e-02_rb,0.27226e-02_rb,0.34908e-02_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.37877e-02_rb,0.33145e-02_rb,0.28416e-02_rb,0.23683e-02_rb,0.19985e-02_rb, &
        & 0.24600e-02_rb,0.28851e-02_rb,0.31576e-02_rb,0.40421e-02_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.37785e-02_rb,0.33065e-02_rb,0.28344e-02_rb,0.23627e-02_rb,0.22594e-02_rb, &
        & 0.27919e-02_rb,0.32761e-02_rb,0.35889e-02_rb,0.45855e-02_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.37630e-02_rb,0.32929e-02_rb,0.28229e-02_rb,0.23532e-02_rb,0.25213e-02_rb, &
        & 0.31161e-02_rb,0.36579e-02_rb,0.40115e-02_rb,0.51155e-02_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.45014e-02_rb,0.39388e-02_rb,0.33762e-02_rb,0.28135e-02_rb,0.22509e-02_rb, &
        & 0.18859e-02_rb,0.22117e-02_rb,0.24208e-02_rb,0.31038e-02_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.44788e-02_rb,0.39190e-02_rb,0.33593e-02_rb,0.27996e-02_rb,0.22399e-02_rb, &
        & 0.22735e-02_rb,0.26677e-02_rb,0.29222e-02_rb,0.37394e-02_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.44593e-02_rb,0.39020e-02_rb,0.33439e-02_rb,0.27867e-02_rb,0.22518e-02_rb, &
        & 0.26648e-02_rb,0.31283e-02_rb,0.34304e-02_rb,0.43806e-02_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.44374e-02_rb,0.38829e-02_rb,0.33284e-02_rb,0.27732e-02_rb,0.24705e-02_rb, &
        & 0.30533e-02_rb,0.35850e-02_rb,0.39361e-02_rb,0.50160e-02_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.44113e-02_rb,0.38601e-02_rb,0.33089e-02_rb,0.27567e-02_rb,0.27756e-02_rb, &
        & 0.34321e-02_rb,0.40324e-02_rb,0.44314e-02_rb,0.56359e-02_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.53031e-02_rb,0.46403e-02_rb,0.39774e-02_rb,0.33149e-02_rb,0.26520e-02_rb, &
        & 0.20316e-02_rb,0.23080e-02_rb,0.25314e-02_rb,0.32334e-02_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.52648e-02_rb,0.46067e-02_rb,0.39487e-02_rb,0.32907e-02_rb,0.26325e-02_rb, &
        & 0.24096e-02_rb,0.28298e-02_rb,0.31081e-02_rb,0.39611e-02_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.52246e-02_rb,0.45716e-02_rb,0.39189e-02_rb,0.32659e-02_rb,0.26125e-02_rb, &
        & 0.28611e-02_rb,0.33614e-02_rb,0.36958e-02_rb,0.47006e-02_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.51821e-02_rb,0.45344e-02_rb,0.38868e-02_rb,0.32390e-02_rb,0.27289e-02_rb, &
        & 0.33103e-02_rb,0.38912e-02_rb,0.42829e-02_rb,0.54355e-02_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.51356e-02_rb,0.44938e-02_rb,0.38520e-02_rb,0.32108e-02_rb,0.30299e-02_rb, &
        & 0.37493e-02_rb,0.44090e-02_rb,0.48604e-02_rb,0.61537e-02_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.62116e-02_rb,0.54352e-02_rb,0.46587e-02_rb,0.38825e-02_rb,0.31060e-02_rb, &
        & 0.23296e-02_rb,0.23830e-02_rb,0.26152e-02_rb,0.33400e-02_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.61510e-02_rb,0.53822e-02_rb,0.46134e-02_rb,0.38445e-02_rb,0.30756e-02_rb, &
        & 0.25421e-02_rb,0.29622e-02_rb,0.32625e-02_rb,0.41372e-02_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.60862e-02_rb,0.53255e-02_rb,0.45651e-02_rb,0.38044e-02_rb,0.30427e-02_rb, &
        & 0.30332e-02_rb,0.35680e-02_rb,0.39355e-02_rb,0.49777e-02_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.60169e-02_rb,0.52649e-02_rb,0.45129e-02_rb,0.37608e-02_rb,0.30629e-02_rb, &
        & 0.35468e-02_rb,0.41747e-02_rb,0.46107e-02_rb,0.58172e-02_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.59428e-02_rb,0.52001e-02_rb,0.44573e-02_rb,0.37148e-02_rb,0.33039e-02_rb, &
        & 0.40510e-02_rb,0.47683e-02_rb,0.52747e-02_rb,0.66403e-02_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.72226e-02_rb,0.63198e-02_rb,0.54170e-02_rb,0.45138e-02_rb,0.36111e-02_rb, &
        & 0.27087e-02_rb,0.25246e-02_rb,0.27733e-02_rb,0.35348e-02_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.71308e-02_rb,0.62395e-02_rb,0.53482e-02_rb,0.44569e-02_rb,0.35656e-02_rb, &
        & 0.27737e-02_rb,0.31148e-02_rb,0.34380e-02_rb,0.43453e-02_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.70337e-02_rb,0.61546e-02_rb,0.52755e-02_rb,0.43963e-02_rb,0.35178e-02_rb, &
        & 0.32195e-02_rb,0.37898e-02_rb,0.41956e-02_rb,0.52718e-02_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.69308e-02_rb,0.60645e-02_rb,0.51982e-02_rb,0.43312e-02_rb,0.34725e-02_rb, &
        & 0.37977e-02_rb,0.44773e-02_rb,0.49627e-02_rb,0.62190e-02_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.68213e-02_rb,0.59687e-02_rb,0.51162e-02_rb,0.42630e-02_rb,0.36577e-02_rb, &
        & 0.43690e-02_rb,0.51523e-02_rb,0.57195e-02_rb,0.71486e-02_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.82729e-02_rb,0.72388e-02_rb,0.62047e-02_rb,0.51704e-02_rb,0.41364e-02_rb, &
        & 0.31022e-02_rb,0.29450e-02_rb,0.32481e-02_rb,0.41092e-02_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.81377e-02_rb,0.71207e-02_rb,0.61034e-02_rb,0.50862e-02_rb,0.40694e-02_rb, &
        & 0.32023e-02_rb,0.35972e-02_rb,0.39817e-02_rb,0.50045e-02_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.79927e-02_rb,0.69937e-02_rb,0.59945e-02_rb,0.49955e-02_rb,0.39980e-02_rb, &
        & 0.36849e-02_rb,0.43305e-02_rb,0.48162e-02_rb,0.60016e-02_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.78435e-02_rb,0.68631e-02_rb,0.58827e-02_rb,0.49014e-02_rb,0.39530e-02_rb, &
        & 0.43152e-02_rb,0.50942e-02_rb,0.56750e-02_rb,0.70494e-02_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.76868e-02_rb,0.67260e-02_rb,0.57653e-02_rb,0.48045e-02_rb,0.41698e-02_rb, &
        & 0.49442e-02_rb,0.58408e-02_rb,0.65151e-02_rb,0.80731e-02_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.93677e-02_rb,0.81967e-02_rb,0.70258e-02_rb,0.58546e-02_rb,0.46837e-02_rb, &
        & 0.35124e-02_rb,0.33991e-02_rb,0.37659e-02_rb,0.47255e-02_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.91725e-02_rb,0.80259e-02_rb,0.68794e-02_rb,0.57329e-02_rb,0.45871e-02_rb, &
        & 0.36555e-02_rb,0.41158e-02_rb,0.45739e-02_rb,0.57058e-02_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.89743e-02_rb,0.78526e-02_rb,0.67312e-02_rb,0.56083e-02_rb,0.44874e-02_rb, &
        & 0.41738e-02_rb,0.49000e-02_rb,0.54681e-02_rb,0.67707e-02_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.87651e-02_rb,0.76695e-02_rb,0.65740e-02_rb,0.54780e-02_rb,0.44541e-02_rb, &
        & 0.48422e-02_rb,0.57276e-02_rb,0.64103e-02_rb,0.78928e-02_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.85557e-02_rb,0.74863e-02_rb,0.64169e-02_rb,0.53471e-02_rb,0.46965e-02_rb, &
        & 0.55261e-02_rb,0.65398e-02_rb,0.73300e-02_rb,0.90026e-02_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.10484e-01_rb,0.91737e-02_rb,0.78632e-02_rb,0.65525e-02_rb,0.52420e-02_rb, &
        & 0.39330e-02_rb,0.38753e-02_rb,0.43130e-02_rb,0.53655e-02_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.10212e-01_rb,0.89352e-02_rb,0.76588e-02_rb,0.63823e-02_rb,0.51063e-02_rb, &
        & 0.41214e-02_rb,0.46538e-02_rb,0.51962e-02_rb,0.64287e-02_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.99472e-02_rb,0.87039e-02_rb,0.74620e-02_rb,0.62164e-02_rb,0.49733e-02_rb, &
        & 0.46683e-02_rb,0.54886e-02_rb,0.61482e-02_rb,0.75590e-02_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.96776e-02_rb,0.84679e-02_rb,0.72583e-02_rb,0.60495e-02_rb,0.49611e-02_rb, &
        & 0.53643e-02_rb,0.63558e-02_rb,0.71458e-02_rb,0.87290e-02_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.94084e-02_rb,0.82324e-02_rb,0.70564e-02_rb,0.58807e-02_rb,0.52210e-02_rb, &
        & 0.60971e-02_rb,0.72271e-02_rb,0.81397e-02_rb,0.99102e-02_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:,13, 1) = (/ &
        & 0.33062e-11_rb,0.42306e-11_rb,0.53172e-11_rb,0.65673e-11_rb,0.78335e-11_rb /)
      kbo(:,14, 1) = (/ &
        & 0.27467e-11_rb,0.35252e-11_rb,0.44160e-11_rb,0.54171e-11_rb,0.64630e-11_rb /)
      kbo(:,15, 1) = (/ &
        & 0.23431e-11_rb,0.29962e-11_rb,0.37507e-11_rb,0.46179e-11_rb,0.54790e-11_rb /)
      kbo(:,16, 1) = (/ &
        & 0.20178e-11_rb,0.25698e-11_rb,0.32330e-11_rb,0.40024e-11_rb,0.46899e-11_rb /)
      kbo(:,17, 1) = (/ &
        & 0.17389e-11_rb,0.22126e-11_rb,0.27864e-11_rb,0.34615e-11_rb,0.40184e-11_rb /)
      kbo(:,18, 1) = (/ &
        & 0.15164e-11_rb,0.19304e-11_rb,0.24263e-11_rb,0.29918e-11_rb,0.34734e-11_rb /)
      kbo(:,19, 1) = (/ &
        & 0.13194e-11_rb,0.16684e-11_rb,0.20851e-11_rb,0.25471e-11_rb,0.29992e-11_rb /)
      kbo(:,20, 1) = (/ &
        & 0.11345e-11_rb,0.14317e-11_rb,0.17813e-11_rb,0.21686e-11_rb,0.25523e-11_rb /)
      kbo(:,21, 1) = (/ &
        & 0.97614e-12_rb,0.12233e-11_rb,0.15124e-11_rb,0.18436e-11_rb,0.21624e-11_rb /)
      kbo(:,22, 1) = (/ &
        & 0.84972e-12_rb,0.10604e-11_rb,0.13001e-11_rb,0.15853e-11_rb,0.18511e-11_rb /)
      kbo(:,23, 1) = (/ &
        & 0.73919e-12_rb,0.91863e-12_rb,0.11179e-11_rb,0.13612e-11_rb,0.15791e-11_rb /)
      kbo(:,24, 1) = (/ &
        & 0.63952e-12_rb,0.79203e-12_rb,0.96272e-12_rb,0.11643e-11_rb,0.13384e-11_rb /)
      kbo(:,25, 1) = (/ &
        & 0.55198e-12_rb,0.68176e-12_rb,0.82782e-12_rb,0.99347e-12_rb,0.11342e-11_rb /)
      kbo(:,26, 1) = (/ &
        & 0.47648e-12_rb,0.58395e-12_rb,0.71214e-12_rb,0.84565e-12_rb,0.96146e-12_rb /)
      kbo(:,27, 1) = (/ &
        & 0.41185e-12_rb,0.50403e-12_rb,0.61156e-12_rb,0.71539e-12_rb,0.81410e-12_rb /)
      kbo(:,28, 1) = (/ &
        & 0.35568e-12_rb,0.43404e-12_rb,0.52441e-12_rb,0.60652e-12_rb,0.68860e-12_rb /)
      kbo(:,29, 1) = (/ &
        & 0.30863e-12_rb,0.37487e-12_rb,0.45120e-12_rb,0.51591e-12_rb,0.58367e-12_rb /)
      kbo(:,30, 1) = (/ &
        & 0.26804e-12_rb,0.32390e-12_rb,0.38615e-12_rb,0.43956e-12_rb,0.49890e-12_rb /)
      kbo(:,31, 1) = (/ &
        & 0.23316e-12_rb,0.28106e-12_rb,0.33068e-12_rb,0.37478e-12_rb,0.42377e-12_rb /)
      kbo(:,32, 1) = (/ &
        & 0.20255e-12_rb,0.24397e-12_rb,0.28293e-12_rb,0.31878e-12_rb,0.36224e-12_rb /)
      kbo(:,33, 1) = (/ &
        & 0.17541e-12_rb,0.21018e-12_rb,0.24052e-12_rb,0.27083e-12_rb,0.30963e-12_rb /)
      kbo(:,34, 1) = (/ &
        & 0.15014e-12_rb,0.17890e-12_rb,0.20293e-12_rb,0.23058e-12_rb,0.26131e-12_rb /)
      kbo(:,35, 1) = (/ &
        & 0.12610e-12_rb,0.14972e-12_rb,0.16906e-12_rb,0.19250e-12_rb,0.21752e-12_rb /)
      kbo(:,36, 1) = (/ &
        & 0.10365e-12_rb,0.12301e-12_rb,0.13878e-12_rb,0.15810e-12_rb,0.17856e-12_rb /)
      kbo(:,37, 1) = (/ &
        & 0.82572e-13_rb,0.98388e-13_rb,0.11143e-12_rb,0.12645e-12_rb,0.14343e-12_rb /)
      kbo(:,38, 1) = (/ &
        & 0.65632e-13_rb,0.78531e-13_rb,0.89359e-13_rb,0.10135e-12_rb,0.11495e-12_rb /)
      kbo(:,39, 1) = (/ &
        & 0.52174e-13_rb,0.62607e-13_rb,0.71650e-13_rb,0.80738e-13_rb,0.92112e-13_rb /)
      kbo(:,40, 1) = (/ &
        & 0.40434e-13_rb,0.48683e-13_rb,0.56479e-13_rb,0.63806e-13_rb,0.72155e-13_rb /)
      kbo(:,41, 1) = (/ &
        & 0.31179e-13_rb,0.37691e-13_rb,0.44339e-13_rb,0.50247e-13_rb,0.56982e-13_rb /)
      kbo(:,42, 1) = (/ &
        & 0.24040e-13_rb,0.29161e-13_rb,0.34731e-13_rb,0.39467e-13_rb,0.44922e-13_rb /)
      kbo(:,43, 1) = (/ &
        & 0.18259e-13_rb,0.22185e-13_rb,0.26753e-13_rb,0.30617e-13_rb,0.34721e-13_rb /)
      kbo(:,44, 1) = (/ &
        & 0.13667e-13_rb,0.16662e-13_rb,0.20308e-13_rb,0.23577e-13_rb,0.26865e-13_rb /)
      kbo(:,45, 1) = (/ &
        & 0.10182e-13_rb,0.12571e-13_rb,0.15345e-13_rb,0.18028e-13_rb,0.20725e-13_rb /)
      kbo(:,46, 1) = (/ &
        & 0.75196e-14_rb,0.94092e-14_rb,0.11475e-13_rb,0.13704e-13_rb,0.15878e-13_rb /)
      kbo(:,47, 1) = (/ &
        & 0.54721e-14_rb,0.69523e-14_rb,0.85280e-14_rb,0.10311e-13_rb,0.11952e-13_rb /)
      kbo(:,48, 1) = (/ &
        & 0.39619e-14_rb,0.50439e-14_rb,0.62896e-14_rb,0.77193e-14_rb,0.89282e-14_rb /)
      kbo(:,49, 1) = (/ &
        & 0.28508e-14_rb,0.36419e-14_rb,0.46044e-14_rb,0.57276e-14_rb,0.66866e-14_rb /)
      kbo(:,50, 1) = (/ &
        & 0.20973e-14_rb,0.26588e-14_rb,0.33754e-14_rb,0.42404e-14_rb,0.50338e-14_rb /)
      kbo(:,51, 1) = (/ &
        & 0.15010e-14_rb,0.19461e-14_rb,0.24807e-14_rb,0.31095e-14_rb,0.37789e-14_rb /)
      kbo(:,52, 1) = (/ &
        & 0.10686e-14_rb,0.14205e-14_rb,0.18164e-14_rb,0.22651e-14_rb,0.27790e-14_rb /)
      kbo(:,53, 1) = (/ &
        & 0.75469e-15_rb,0.10279e-14_rb,0.13211e-14_rb,0.16657e-14_rb,0.20344e-14_rb /)
      kbo(:,54, 1) = (/ &
        & 0.53837e-15_rb,0.75440e-15_rb,0.97064e-15_rb,0.12260e-14_rb,0.15156e-14_rb /)
      kbo(:,55, 1) = (/ &
        & 0.38620e-15_rb,0.54750e-15_rb,0.71223e-15_rb,0.90921e-15_rb,0.11265e-14_rb /)
      kbo(:,56, 1) = (/ &
        & 0.27852e-15_rb,0.39135e-15_rb,0.51858e-15_rb,0.66760e-15_rb,0.83486e-15_rb /)
      kbo(:,57, 1) = (/ &
        & 0.20280e-15_rb,0.27940e-15_rb,0.37558e-15_rb,0.48656e-15_rb,0.61658e-15_rb /)
      kbo(:,58, 1) = (/ &
        & 0.14586e-15_rb,0.19788e-15_rb,0.27273e-15_rb,0.35391e-15_rb,0.45250e-15_rb /)
      kbo(:,59, 1) = (/ &
        & 0.11164e-15_rb,0.15082e-15_rb,0.20804e-15_rb,0.27081e-15_rb,0.49339e-15_rb /)
      kbo(:,13, 2) = (/ &
        & 0.15800e-10_rb,0.20781e-10_rb,0.26154e-10_rb,0.31936e-10_rb,0.38062e-10_rb /)
      kbo(:,14, 2) = (/ &
        & 0.13333e-10_rb,0.17459e-10_rb,0.21881e-10_rb,0.26666e-10_rb,0.31696e-10_rb /)
      kbo(:,15, 2) = (/ &
        & 0.11381e-10_rb,0.14784e-10_rb,0.18449e-10_rb,0.22397e-10_rb,0.26533e-10_rb /)
      kbo(:,16, 2) = (/ &
        & 0.97593e-11_rb,0.12591e-10_rb,0.15641e-10_rb,0.18845e-10_rb,0.22503e-10_rb /)
      kbo(:,17, 2) = (/ &
        & 0.83473e-11_rb,0.10709e-10_rb,0.13261e-10_rb,0.15857e-10_rb,0.19052e-10_rb /)
      kbo(:,18, 2) = (/ &
        & 0.71523e-11_rb,0.91255e-11_rb,0.11229e-10_rb,0.13436e-10_rb,0.16145e-10_rb /)
      kbo(:,19, 2) = (/ &
        & 0.60777e-11_rb,0.77489e-11_rb,0.94796e-11_rb,0.11415e-10_rb,0.13591e-10_rb /)
      kbo(:,20, 2) = (/ &
        & 0.51693e-11_rb,0.65609e-11_rb,0.80124e-11_rb,0.96944e-11_rb,0.11427e-10_rb /)
      kbo(:,21, 2) = (/ &
        & 0.43931e-11_rb,0.55492e-11_rb,0.67698e-11_rb,0.81825e-11_rb,0.95836e-11_rb /)
      kbo(:,22, 2) = (/ &
        & 0.37967e-11_rb,0.47515e-11_rb,0.57759e-11_rb,0.69676e-11_rb,0.81090e-11_rb /)
      kbo(:,23, 2) = (/ &
        & 0.32747e-11_rb,0.40830e-11_rb,0.49616e-11_rb,0.59280e-11_rb,0.68614e-11_rb /)
      kbo(:,24, 2) = (/ &
        & 0.28215e-11_rb,0.34976e-11_rb,0.42532e-11_rb,0.50228e-11_rb,0.57971e-11_rb /)
      kbo(:,25, 2) = (/ &
        & 0.24334e-11_rb,0.29960e-11_rb,0.36396e-11_rb,0.42606e-11_rb,0.49056e-11_rb /)
      kbo(:,26, 2) = (/ &
        & 0.21117e-11_rb,0.25752e-11_rb,0.31134e-11_rb,0.36244e-11_rb,0.41489e-11_rb /)
      kbo(:,27, 2) = (/ &
        & 0.18253e-11_rb,0.22243e-11_rb,0.26576e-11_rb,0.30793e-11_rb,0.35048e-11_rb /)
      kbo(:,28, 2) = (/ &
        & 0.15682e-11_rb,0.19141e-11_rb,0.22636e-11_rb,0.26109e-11_rb,0.29589e-11_rb /)
      kbo(:,29, 2) = (/ &
        & 0.13535e-11_rb,0.16466e-11_rb,0.19296e-11_rb,0.22145e-11_rb,0.25005e-11_rb /)
      kbo(:,30, 2) = (/ &
        & 0.11709e-11_rb,0.14136e-11_rb,0.16432e-11_rb,0.18792e-11_rb,0.21240e-11_rb /)
      kbo(:,31, 2) = (/ &
        & 0.10216e-11_rb,0.12142e-11_rb,0.14045e-11_rb,0.15978e-11_rb,0.18001e-11_rb /)
      kbo(:,32, 2) = (/ &
        & 0.88417e-12_rb,0.10400e-11_rb,0.11973e-11_rb,0.13566e-11_rb,0.15303e-11_rb /)
      kbo(:,33, 2) = (/ &
        & 0.76210e-12_rb,0.89004e-12_rb,0.10199e-11_rb,0.11509e-11_rb,0.12984e-11_rb /)
      kbo(:,34, 2) = (/ &
        & 0.64910e-12_rb,0.75460e-12_rb,0.86122e-12_rb,0.97227e-12_rb,0.10908e-11_rb /)
      kbo(:,35, 2) = (/ &
        & 0.54374e-12_rb,0.63015e-12_rb,0.71745e-12_rb,0.80944e-12_rb,0.90628e-12_rb /)
      kbo(:,36, 2) = (/ &
        & 0.44718e-12_rb,0.51791e-12_rb,0.58946e-12_rb,0.66527e-12_rb,0.74411e-12_rb /)
      kbo(:,37, 2) = (/ &
        & 0.35793e-12_rb,0.41591e-12_rb,0.47443e-12_rb,0.53549e-12_rb,0.60041e-12_rb /)
      kbo(:,38, 2) = (/ &
        & 0.28618e-12_rb,0.33345e-12_rb,0.38124e-12_rb,0.43139e-12_rb,0.48394e-12_rb /)
      kbo(:,39, 2) = (/ &
        & 0.22858e-12_rb,0.26729e-12_rb,0.30628e-12_rb,0.34554e-12_rb,0.38998e-12_rb /)
      kbo(:,40, 2) = (/ &
        & 0.17867e-12_rb,0.21032e-12_rb,0.24207e-12_rb,0.27407e-12_rb,0.30925e-12_rb /)
      kbo(:,41, 2) = (/ &
        & 0.13875e-12_rb,0.16498e-12_rb,0.19072e-12_rb,0.21688e-12_rb,0.24410e-12_rb /)
      kbo(:,42, 2) = (/ &
        & 0.10714e-12_rb,0.12915e-12_rb,0.14999e-12_rb,0.17156e-12_rb,0.19364e-12_rb /)
      kbo(:,43, 2) = (/ &
        & 0.81856e-13_rb,0.99592e-13_rb,0.11661e-12_rb,0.13414e-12_rb,0.15145e-12_rb /)
      kbo(:,44, 2) = (/ &
        & 0.62221e-13_rb,0.75886e-13_rb,0.90106e-13_rb,0.10425e-12_rb,0.11838e-12_rb /)
      kbo(:,45, 2) = (/ &
        & 0.47139e-13_rb,0.57421e-13_rb,0.69401e-13_rb,0.80830e-13_rb,0.92462e-13_rb /)
      kbo(:,46, 2) = (/ &
        & 0.35469e-13_rb,0.43603e-13_rb,0.52913e-13_rb,0.62345e-13_rb,0.71769e-13_rb /)
      kbo(:,47, 2) = (/ &
        & 0.26308e-13_rb,0.32606e-13_rb,0.39610e-13_rb,0.47505e-13_rb,0.55354e-13_rb /)
      kbo(:,48, 2) = (/ &
        & 0.19388e-13_rb,0.24430e-13_rb,0.29903e-13_rb,0.35902e-13_rb,0.42540e-13_rb /)
      kbo(:,49, 2) = (/ &
        & 0.14144e-13_rb,0.18180e-13_rb,0.22467e-13_rb,0.26985e-13_rb,0.32471e-13_rb /)
      kbo(:,50, 2) = (/ &
        & 0.10280e-13_rb,0.13471e-13_rb,0.16900e-13_rb,0.20465e-13_rb,0.24665e-13_rb /)
      kbo(:,51, 2) = (/ &
        & 0.74898e-14_rb,0.99392e-14_rb,0.12674e-13_rb,0.15571e-13_rb,0.18663e-13_rb /)
      kbo(:,52, 2) = (/ &
        & 0.54214e-14_rb,0.72917e-14_rb,0.94517e-14_rb,0.11816e-13_rb,0.14256e-13_rb /)
      kbo(:,53, 2) = (/ &
        & 0.38970e-14_rb,0.53165e-14_rb,0.69989e-14_rb,0.88828e-14_rb,0.10862e-13_rb /)
      kbo(:,54, 2) = (/ &
        & 0.27972e-14_rb,0.38775e-14_rb,0.51927e-14_rb,0.66775e-14_rb,0.82659e-14_rb /)
      kbo(:,55, 2) = (/ &
        & 0.19962e-14_rb,0.28257e-14_rb,0.38491e-14_rb,0.50059e-14_rb,0.62864e-14_rb /)
      kbo(:,56, 2) = (/ &
        & 0.14049e-14_rb,0.20447e-14_rb,0.28389e-14_rb,0.37444e-14_rb,0.47474e-14_rb /)
      kbo(:,57, 2) = (/ &
        & 0.97467e-15_rb,0.14635e-14_rb,0.20787e-14_rb,0.27878e-14_rb,0.35737e-14_rb /)
      kbo(:,58, 2) = (/ &
        & 0.67770e-15_rb,0.10398e-14_rb,0.15169e-14_rb,0.20739e-14_rb,0.26951e-14_rb /)
      kbo(:,59, 2) = (/ &
        & 0.50857e-15_rb,0.79496e-15_rb,0.11749e-14_rb,0.16262e-14_rb,0.21270e-14_rb /)
      kbo(:,13, 3) = (/ &
        & 0.93671e-10_rb,0.11591e-09_rb,0.13715e-09_rb,0.15779e-09_rb,0.17870e-09_rb /)
      kbo(:,14, 3) = (/ &
        & 0.78883e-10_rb,0.96971e-10_rb,0.11437e-09_rb,0.13160e-09_rb,0.14863e-09_rb /)
      kbo(:,15, 3) = (/ &
        & 0.66900e-10_rb,0.81568e-10_rb,0.96052e-10_rb,0.11028e-09_rb,0.12439e-09_rb /)
      kbo(:,16, 3) = (/ &
        & 0.56737e-10_rb,0.68853e-10_rb,0.80870e-10_rb,0.92769e-10_rb,0.10422e-09_rb /)
      kbo(:,17, 3) = (/ &
        & 0.48148e-10_rb,0.58209e-10_rb,0.68168e-10_rb,0.78006e-10_rb,0.87308e-10_rb /)
      kbo(:,18, 3) = (/ &
        & 0.40874e-10_rb,0.49268e-10_rb,0.57591e-10_rb,0.65627e-10_rb,0.73296e-10_rb /)
      kbo(:,19, 3) = (/ &
        & 0.34619e-10_rb,0.41578e-10_rb,0.48414e-10_rb,0.55049e-10_rb,0.61235e-10_rb /)
      kbo(:,20, 3) = (/ &
        & 0.29370e-10_rb,0.35034e-10_rb,0.40682e-10_rb,0.46014e-10_rb,0.51115e-10_rb /)
      kbo(:,21, 3) = (/ &
        & 0.24821e-10_rb,0.29490e-10_rb,0.34141e-10_rb,0.38423e-10_rb,0.42621e-10_rb /)
      kbo(:,22, 3) = (/ &
        & 0.21204e-10_rb,0.25041e-10_rb,0.28846e-10_rb,0.32305e-10_rb,0.35757e-10_rb /)
      kbo(:,23, 3) = (/ &
        & 0.18107e-10_rb,0.21244e-10_rb,0.24304e-10_rb,0.27144e-10_rb,0.30005e-10_rb /)
      kbo(:,24, 3) = (/ &
        & 0.15432e-10_rb,0.18015e-10_rb,0.20470e-10_rb,0.22803e-10_rb,0.25180e-10_rb /)
      kbo(:,25, 3) = (/ &
        & 0.13155e-10_rb,0.15266e-10_rb,0.17234e-10_rb,0.19170e-10_rb,0.21096e-10_rb /)
      kbo(:,26, 3) = (/ &
        & 0.11220e-10_rb,0.12944e-10_rb,0.14527e-10_rb,0.16132e-10_rb,0.17690e-10_rb /)
      kbo(:,27, 3) = (/ &
        & 0.95562e-11_rb,0.10942e-10_rb,0.12242e-10_rb,0.13560e-10_rb,0.14820e-10_rb /)
      kbo(:,28, 3) = (/ &
        & 0.81359e-11_rb,0.92394e-11_rb,0.10309e-10_rb,0.11386e-10_rb,0.12407e-10_rb /)
      kbo(:,29, 3) = (/ &
        & 0.69193e-11_rb,0.78077e-11_rb,0.87009e-11_rb,0.95682e-11_rb,0.10386e-10_rb /)
      kbo(:,30, 3) = (/ &
        & 0.58737e-11_rb,0.65956e-11_rb,0.73381e-11_rb,0.80325e-11_rb,0.87147e-11_rb /)
      kbo(:,31, 3) = (/ &
        & 0.49782e-11_rb,0.55816e-11_rb,0.61826e-11_rb,0.67463e-11_rb,0.73000e-11_rb /)
      kbo(:,32, 3) = (/ &
        & 0.42202e-11_rb,0.47232e-11_rb,0.52043e-11_rb,0.56623e-11_rb,0.61016e-11_rb /)
      kbo(:,33, 3) = (/ &
        & 0.35744e-11_rb,0.39897e-11_rb,0.43757e-11_rb,0.47465e-11_rb,0.50931e-11_rb /)
      kbo(:,34, 3) = (/ &
        & 0.30127e-11_rb,0.33496e-11_rb,0.36630e-11_rb,0.39738e-11_rb,0.42413e-11_rb /)
      kbo(:,35, 3) = (/ &
        & 0.25098e-11_rb,0.27837e-11_rb,0.30393e-11_rb,0.32916e-11_rb,0.35093e-11_rb /)
      kbo(:,36, 3) = (/ &
        & 0.20644e-11_rb,0.22886e-11_rb,0.24982e-11_rb,0.27043e-11_rb,0.28833e-11_rb /)
      kbo(:,37, 3) = (/ &
        & 0.16684e-11_rb,0.18536e-11_rb,0.20259e-11_rb,0.21979e-11_rb,0.23456e-11_rb /)
      kbo(:,38, 3) = (/ &
        & 0.13466e-11_rb,0.14996e-11_rb,0.16425e-11_rb,0.17834e-11_rb,0.19065e-11_rb /)
      kbo(:,39, 3) = (/ &
        & 0.10861e-11_rb,0.12130e-11_rb,0.13309e-11_rb,0.14440e-11_rb,0.15498e-11_rb /)
      kbo(:,40, 3) = (/ &
        & 0.86498e-12_rb,0.96921e-12_rb,0.10678e-11_rb,0.11601e-11_rb,0.12509e-11_rb /)
      kbo(:,41, 3) = (/ &
        & 0.68788e-12_rb,0.77191e-12_rb,0.85472e-12_rb,0.93133e-12_rb,0.10084e-11_rb /)
      kbo(:,42, 3) = (/ &
        & 0.54661e-12_rb,0.61437e-12_rb,0.68360e-12_rb,0.74699e-12_rb,0.81088e-12_rb /)
      kbo(:,43, 3) = (/ &
        & 0.42920e-12_rb,0.48515e-12_rb,0.54174e-12_rb,0.59564e-12_rb,0.64658e-12_rb /)
      kbo(:,44, 3) = (/ &
        & 0.33491e-12_rb,0.38198e-12_rb,0.42756e-12_rb,0.47274e-12_rb,0.51490e-12_rb /)
      kbo(:,45, 3) = (/ &
        & 0.26072e-12_rb,0.29981e-12_rb,0.33699e-12_rb,0.37442e-12_rb,0.40968e-12_rb /)
      kbo(:,46, 3) = (/ &
        & 0.20167e-12_rb,0.23370e-12_rb,0.26473e-12_rb,0.29535e-12_rb,0.32495e-12_rb /)
      kbo(:,47, 3) = (/ &
        & 0.15427e-12_rb,0.18070e-12_rb,0.20644e-12_rb,0.23129e-12_rb,0.25612e-12_rb /)
      kbo(:,48, 3) = (/ &
        & 0.11746e-12_rb,0.13905e-12_rb,0.16032e-12_rb,0.18088e-12_rb,0.20125e-12_rb /)
      kbo(:,49, 3) = (/ &
        & 0.88976e-13_rb,0.10661e-12_rb,0.12415e-12_rb,0.14128e-12_rb,0.15777e-12_rb /)
      kbo(:,50, 3) = (/ &
        & 0.67378e-13_rb,0.81791e-13_rb,0.96162e-13_rb,0.11029e-12_rb,0.12384e-12_rb /)
      kbo(:,51, 3) = (/ &
        & 0.50786e-13_rb,0.62576e-13_rb,0.74348e-13_rb,0.85995e-13_rb,0.97345e-13_rb /)
      kbo(:,52, 3) = (/ &
        & 0.37951e-13_rb,0.47645e-13_rb,0.57250e-13_rb,0.66805e-13_rb,0.76257e-13_rb /)
      kbo(:,53, 3) = (/ &
        & 0.28159e-13_rb,0.36102e-13_rb,0.43964e-13_rb,0.51774e-13_rb,0.59583e-13_rb /)
      kbo(:,54, 3) = (/ &
        & 0.21004e-13_rb,0.27366e-13_rb,0.33776e-13_rb,0.40177e-13_rb,0.46583e-13_rb /)
      kbo(:,55, 3) = (/ &
        & 0.15637e-13_rb,0.20632e-13_rb,0.25941e-13_rb,0.31184e-13_rb,0.36409e-13_rb /)
      kbo(:,56, 3) = (/ &
        & 0.11581e-13_rb,0.15522e-13_rb,0.19844e-13_rb,0.24130e-13_rb,0.28399e-13_rb /)
      kbo(:,57, 3) = (/ &
        & 0.85125e-14_rb,0.11631e-13_rb,0.15093e-13_rb,0.18610e-13_rb,0.22102e-13_rb /)
      kbo(:,58, 3) = (/ &
        & 0.62435e-14_rb,0.87193e-14_rb,0.11437e-13_rb,0.14347e-13_rb,0.17208e-13_rb /)
      kbo(:,59, 3) = (/ &
        & 0.48886e-14_rb,0.68810e-14_rb,0.90777e-14_rb,0.11450e-13_rb,0.13800e-13_rb /)
      kbo(:,13, 4) = (/ &
        & 0.49769e-09_rb,0.57573e-09_rb,0.65456e-09_rb,0.73233e-09_rb,0.81030e-09_rb /)
      kbo(:,14, 4) = (/ &
        & 0.41687e-09_rb,0.48070e-09_rb,0.54629e-09_rb,0.61156e-09_rb,0.67509e-09_rb /)
      kbo(:,15, 4) = (/ &
        & 0.35004e-09_rb,0.40346e-09_rb,0.45778e-09_rb,0.51155e-09_rb,0.56442e-09_rb /)
      kbo(:,16, 4) = (/ &
        & 0.29428e-09_rb,0.33893e-09_rb,0.38370e-09_rb,0.42831e-09_rb,0.47172e-09_rb /)
      kbo(:,17, 4) = (/ &
        & 0.24736e-09_rb,0.28447e-09_rb,0.32184e-09_rb,0.35882e-09_rb,0.39576e-09_rb /)
      kbo(:,18, 4) = (/ &
        & 0.20799e-09_rb,0.23913e-09_rb,0.26980e-09_rb,0.30118e-09_rb,0.33239e-09_rb /)
      kbo(:,19, 4) = (/ &
        & 0.17474e-09_rb,0.20037e-09_rb,0.22650e-09_rb,0.25275e-09_rb,0.27848e-09_rb /)
      kbo(:,20, 4) = (/ &
        & 0.14678e-09_rb,0.16826e-09_rb,0.19015e-09_rb,0.21182e-09_rb,0.23321e-09_rb /)
      kbo(:,21, 4) = (/ &
        & 0.12329e-09_rb,0.14137e-09_rb,0.15951e-09_rb,0.17762e-09_rb,0.19539e-09_rb /)
      kbo(:,22, 4) = (/ &
        & 0.10475e-09_rb,0.11989e-09_rb,0.13488e-09_rb,0.14993e-09_rb,0.16481e-09_rb /)
      kbo(:,23, 4) = (/ &
        & 0.89090e-10_rb,0.10156e-09_rb,0.11416e-09_rb,0.12661e-09_rb,0.13928e-09_rb /)
      kbo(:,24, 4) = (/ &
        & 0.75648e-10_rb,0.86094e-10_rb,0.96530e-10_rb,0.10692e-09_rb,0.11716e-09_rb /)
      kbo(:,25, 4) = (/ &
        & 0.64419e-10_rb,0.73036e-10_rb,0.81659e-10_rb,0.90142e-10_rb,0.98524e-10_rb /)
      kbo(:,26, 4) = (/ &
        & 0.54857e-10_rb,0.61972e-10_rb,0.69147e-10_rb,0.76125e-10_rb,0.82778e-10_rb /)
      kbo(:,27, 4) = (/ &
        & 0.46656e-10_rb,0.52591e-10_rb,0.58413e-10_rb,0.64168e-10_rb,0.69451e-10_rb /)
      kbo(:,28, 4) = (/ &
        & 0.39665e-10_rb,0.44561e-10_rb,0.49317e-10_rb,0.53971e-10_rb,0.58228e-10_rb /)
      kbo(:,29, 4) = (/ &
        & 0.33755e-10_rb,0.37736e-10_rb,0.41652e-10_rb,0.45382e-10_rb,0.48825e-10_rb /)
      kbo(:,30, 4) = (/ &
        & 0.28671e-10_rb,0.31949e-10_rb,0.35144e-10_rb,0.38141e-10_rb,0.41003e-10_rb /)
      kbo(:,31, 4) = (/ &
        & 0.24360e-10_rb,0.27027e-10_rb,0.29632e-10_rb,0.32038e-10_rb,0.34391e-10_rb /)
      kbo(:,32, 4) = (/ &
        & 0.20671e-10_rb,0.22852e-10_rb,0.24963e-10_rb,0.26903e-10_rb,0.28818e-10_rb /)
      kbo(:,33, 4) = (/ &
        & 0.17515e-10_rb,0.19306e-10_rb,0.20998e-10_rb,0.22562e-10_rb,0.24113e-10_rb /)
      kbo(:,34, 4) = (/ &
        & 0.14755e-10_rb,0.16223e-10_rb,0.17588e-10_rb,0.18905e-10_rb,0.20105e-10_rb /)
      kbo(:,35, 4) = (/ &
        & 0.12305e-10_rb,0.13505e-10_rb,0.14616e-10_rb,0.15703e-10_rb,0.16661e-10_rb /)
      kbo(:,36, 4) = (/ &
        & 0.10152e-10_rb,0.11132e-10_rb,0.12046e-10_rb,0.12938e-10_rb,0.13718e-10_rb /)
      kbo(:,37, 4) = (/ &
        & 0.82437e-11_rb,0.90546e-11_rb,0.98101e-11_rb,0.10549e-10_rb,0.11200e-10_rb /)
      kbo(:,38, 4) = (/ &
        & 0.66891e-11_rb,0.73572e-11_rb,0.79832e-11_rb,0.85923e-11_rb,0.91388e-11_rb /)
      kbo(:,39, 4) = (/ &
        & 0.54271e-11_rb,0.59789e-11_rb,0.64946e-11_rb,0.69804e-11_rb,0.74536e-11_rb /)
      kbo(:,40, 4) = (/ &
        & 0.43504e-11_rb,0.48077e-11_rb,0.52359e-11_rb,0.56420e-11_rb,0.60390e-11_rb /)
      kbo(:,41, 4) = (/ &
        & 0.34805e-11_rb,0.38570e-11_rb,0.42158e-11_rb,0.45532e-11_rb,0.48856e-11_rb /)
      kbo(:,42, 4) = (/ &
        & 0.27817e-11_rb,0.30896e-11_rb,0.33906e-11_rb,0.36712e-11_rb,0.39504e-11_rb /)
      kbo(:,43, 4) = (/ &
        & 0.22059e-11_rb,0.24603e-11_rb,0.27112e-11_rb,0.29449e-11_rb,0.31673e-11_rb /)
      kbo(:,44, 4) = (/ &
        & 0.17407e-11_rb,0.19508e-11_rb,0.21593e-11_rb,0.23549e-11_rb,0.25394e-11_rb /)
      kbo(:,45, 4) = (/ &
        & 0.13717e-11_rb,0.15444e-11_rb,0.17149e-11_rb,0.18808e-11_rb,0.20349e-11_rb /)
      kbo(:,46, 4) = (/ &
        & 0.10758e-11_rb,0.12178e-11_rb,0.13582e-11_rb,0.14973e-11_rb,0.16264e-11_rb /)
      kbo(:,47, 4) = (/ &
        & 0.83623e-12_rb,0.95326e-12_rb,0.10692e-11_rb,0.11850e-11_rb,0.12934e-11_rb /)
      kbo(:,48, 4) = (/ &
        & 0.64811e-12_rb,0.74426e-12_rb,0.84002e-12_rb,0.93446e-12_rb,0.10270e-11_rb /)
      kbo(:,49, 4) = (/ &
        & 0.50093e-12_rb,0.57976e-12_rb,0.65824e-12_rb,0.73652e-12_rb,0.81364e-12_rb /)
      kbo(:,50, 4) = (/ &
        & 0.38740e-12_rb,0.45167e-12_rb,0.51620e-12_rb,0.58084e-12_rb,0.64495e-12_rb /)
      kbo(:,51, 4) = (/ &
        & 0.29910e-12_rb,0.35149e-12_rb,0.40454e-12_rb,0.45768e-12_rb,0.51010e-12_rb /)
      kbo(:,52, 4) = (/ &
        & 0.23004e-12_rb,0.27291e-12_rb,0.31637e-12_rb,0.35993e-12_rb,0.40313e-12_rb /)
      kbo(:,53, 4) = (/ &
        & 0.17654e-12_rb,0.21129e-12_rb,0.24667e-12_rb,0.28251e-12_rb,0.31798e-12_rb /)
      kbo(:,54, 4) = (/ &
        & 0.13566e-12_rb,0.16388e-12_rb,0.19277e-12_rb,0.22203e-12_rb,0.25140e-12_rb /)
      kbo(:,55, 4) = (/ &
        & 0.10414e-12_rb,0.12705e-12_rb,0.15062e-12_rb,0.17458e-12_rb,0.19863e-12_rb /)
      kbo(:,56, 4) = (/ &
        & 0.79695e-13_rb,0.98154e-13_rb,0.11737e-12_rb,0.13696e-12_rb,0.15674e-12_rb /)
      kbo(:,57, 4) = (/ &
        & 0.60735e-13_rb,0.75667e-13_rb,0.91265e-13_rb,0.10718e-12_rb,0.12346e-12_rb /)
      kbo(:,58, 4) = (/ &
        & 0.46331e-13_rb,0.58316e-13_rb,0.71018e-13_rb,0.83950e-13_rb,0.97215e-13_rb /)
      kbo(:,59, 4) = (/ &
        & 0.36896e-13_rb,0.46643e-13_rb,0.57036e-13_rb,0.67613e-13_rb,0.78545e-13_rb /)
      kbo(:,13, 5) = (/ &
        & 0.26254e-08_rb,0.28627e-08_rb,0.30909e-08_rb,0.33000e-08_rb,0.34892e-08_rb /)
      kbo(:,14, 5) = (/ &
        & 0.21830e-08_rb,0.23794e-08_rb,0.25635e-08_rb,0.27354e-08_rb,0.28987e-08_rb /)
      kbo(:,15, 5) = (/ &
        & 0.18164e-08_rb,0.19772e-08_rb,0.21299e-08_rb,0.22819e-08_rb,0.24177e-08_rb /)
      kbo(:,16, 5) = (/ &
        & 0.15141e-08_rb,0.16500e-08_rb,0.17819e-08_rb,0.19057e-08_rb,0.20139e-08_rb /)
      kbo(:,17, 5) = (/ &
        & 0.12644e-08_rb,0.13791e-08_rb,0.14854e-08_rb,0.15868e-08_rb,0.16779e-08_rb /)
      kbo(:,18, 5) = (/ &
        & 0.10569e-08_rb,0.11490e-08_rb,0.12387e-08_rb,0.13249e-08_rb,0.14004e-08_rb /)
      kbo(:,19, 5) = (/ &
        & 0.88077e-09_rb,0.95771e-09_rb,0.10342e-08_rb,0.11045e-08_rb,0.11654e-08_rb /)
      kbo(:,20, 5) = (/ &
        & 0.73433e-09_rb,0.79954e-09_rb,0.86343e-09_rb,0.92049e-09_rb,0.97111e-09_rb /)
      kbo(:,21, 5) = (/ &
        & 0.61317e-09_rb,0.66751e-09_rb,0.72065e-09_rb,0.76613e-09_rb,0.80891e-09_rb /)
      kbo(:,22, 5) = (/ &
        & 0.51519e-09_rb,0.56066e-09_rb,0.60290e-09_rb,0.64113e-09_rb,0.67719e-09_rb /)
      kbo(:,23, 5) = (/ &
        & 0.43331e-09_rb,0.47103e-09_rb,0.50532e-09_rb,0.53747e-09_rb,0.56589e-09_rb /)
      kbo(:,24, 5) = (/ &
        & 0.36478e-09_rb,0.39531e-09_rb,0.42421e-09_rb,0.45023e-09_rb,0.47333e-09_rb /)
      kbo(:,25, 5) = (/ &
        & 0.30661e-09_rb,0.33249e-09_rb,0.35642e-09_rb,0.37711e-09_rb,0.39645e-09_rb /)
      kbo(:,26, 5) = (/ &
        & 0.25851e-09_rb,0.28030e-09_rb,0.29893e-09_rb,0.31621e-09_rb,0.33249e-09_rb /)
      kbo(:,27, 5) = (/ &
        & 0.21849e-09_rb,0.23581e-09_rb,0.25124e-09_rb,0.26548e-09_rb,0.27889e-09_rb /)
      kbo(:,28, 5) = (/ &
        & 0.18476e-09_rb,0.19866e-09_rb,0.21115e-09_rb,0.22285e-09_rb,0.23419e-09_rb /)
      kbo(:,29, 5) = (/ &
        & 0.15608e-09_rb,0.16725e-09_rb,0.17764e-09_rb,0.18744e-09_rb,0.19627e-09_rb /)
      kbo(:,30, 5) = (/ &
        & 0.13180e-09_rb,0.14084e-09_rb,0.14944e-09_rb,0.15734e-09_rb,0.16419e-09_rb /)
      kbo(:,31, 5) = (/ &
        & 0.11122e-09_rb,0.11878e-09_rb,0.12548e-09_rb,0.13183e-09_rb,0.13690e-09_rb /)
      kbo(:,32, 5) = (/ &
        & 0.93806e-10_rb,0.99909e-10_rb,0.10547e-09_rb,0.11013e-09_rb,0.11393e-09_rb /)
      kbo(:,33, 5) = (/ &
        & 0.79057e-10_rb,0.83869e-10_rb,0.88337e-10_rb,0.91833e-10_rb,0.94728e-10_rb /)
      kbo(:,34, 5) = (/ &
        & 0.66383e-10_rb,0.70216e-10_rb,0.73644e-10_rb,0.76467e-10_rb,0.78589e-10_rb /)
      kbo(:,35, 5) = (/ &
        & 0.55303e-10_rb,0.58435e-10_rb,0.61122e-10_rb,0.63317e-10_rb,0.64984e-10_rb /)
      kbo(:,36, 5) = (/ &
        & 0.45788e-10_rb,0.48318e-10_rb,0.50463e-10_rb,0.52237e-10_rb,0.53589e-10_rb /)
      kbo(:,37, 5) = (/ &
        & 0.37517e-10_rb,0.39593e-10_rb,0.41407e-10_rb,0.42883e-10_rb,0.44012e-10_rb /)
      kbo(:,38, 5) = (/ &
        & 0.30717e-10_rb,0.32436e-10_rb,0.33950e-10_rb,0.35197e-10_rb,0.36146e-10_rb /)
      kbo(:,39, 5) = (/ &
        & 0.25129e-10_rb,0.26560e-10_rb,0.27827e-10_rb,0.28835e-10_rb,0.29676e-10_rb /)
      kbo(:,40, 5) = (/ &
        & 0.20420e-10_rb,0.21620e-10_rb,0.22699e-10_rb,0.23564e-10_rb,0.24290e-10_rb /)
      kbo(:,41, 5) = (/ &
        & 0.16561e-10_rb,0.17583e-10_rb,0.18496e-10_rb,0.19245e-10_rb,0.19868e-10_rb /)
      kbo(:,42, 5) = (/ &
        & 0.13418e-10_rb,0.14291e-10_rb,0.15059e-10_rb,0.15706e-10_rb,0.16243e-10_rb /)
      kbo(:,43, 5) = (/ &
        & 0.10810e-10_rb,0.11562e-10_rb,0.12210e-10_rb,0.12774e-10_rb,0.13223e-10_rb /)
      kbo(:,44, 5) = (/ &
        & 0.86843e-11_rb,0.93265e-11_rb,0.98789e-11_rb,0.10369e-10_rb,0.10770e-10_rb /)
      kbo(:,45, 5) = (/ &
        & 0.69593e-11_rb,0.75111e-11_rb,0.79891e-11_rb,0.84084e-11_rb,0.87638e-11_rb /)
      kbo(:,46, 5) = (/ &
        & 0.55576e-11_rb,0.60308e-11_rb,0.64400e-11_rb,0.68030e-11_rb,0.71149e-11_rb /)
      kbo(:,47, 5) = (/ &
        & 0.44097e-11_rb,0.48155e-11_rb,0.51709e-11_rb,0.54794e-11_rb,0.57570e-11_rb /)
      kbo(:,48, 5) = (/ &
        & 0.34877e-11_rb,0.38384e-11_rb,0.41433e-11_rb,0.44120e-11_rb,0.46521e-11_rb /)
      kbo(:,49, 5) = (/ &
        & 0.27512e-11_rb,0.30491e-11_rb,0.33123e-11_rb,0.35456e-11_rb,0.37509e-11_rb /)
      kbo(:,50, 5) = (/ &
        & 0.21734e-11_rb,0.24185e-11_rb,0.26481e-11_rb,0.28485e-11_rb,0.30236e-11_rb /)
      kbo(:,51, 5) = (/ &
        & 0.17150e-11_rb,0.19185e-11_rb,0.21147e-11_rb,0.22857e-11_rb,0.24371e-11_rb /)
      kbo(:,52, 5) = (/ &
        & 0.13499e-11_rb,0.15196e-11_rb,0.16853e-11_rb,0.18315e-11_rb,0.19613e-11_rb /)
      kbo(:,53, 5) = (/ &
        & 0.10590e-11_rb,0.11989e-11_rb,0.13374e-11_rb,0.14641e-11_rb,0.15756e-11_rb /)
      kbo(:,54, 5) = (/ &
        & 0.83182e-12_rb,0.94832e-12_rb,0.10625e-11_rb,0.11713e-11_rb,0.12669e-11_rb /)
      kbo(:,55, 5) = (/ &
        & 0.65257e-12_rb,0.74992e-12_rb,0.84445e-12_rb,0.93664e-12_rb,0.10175e-11_rb /)
      kbo(:,56, 5) = (/ &
        & 0.51109e-12_rb,0.59162e-12_rb,0.67030e-12_rb,0.74706e-12_rb,0.81634e-12_rb /)
      kbo(:,57, 5) = (/ &
        & 0.39868e-12_rb,0.46532e-12_rb,0.53042e-12_rb,0.59449e-12_rb,0.65374e-12_rb /)
      kbo(:,58, 5) = (/ &
        & 0.31062e-12_rb,0.36629e-12_rb,0.42009e-12_rb,0.47298e-12_rb,0.52357e-12_rb /)
      kbo(:,59, 5) = (/ &
        & 0.24996e-12_rb,0.29597e-12_rb,0.34052e-12_rb,0.38403e-12_rb,0.42584e-12_rb /)
      kbo(:,13, 6) = (/ &
        & 0.10275e-07_rb,0.10425e-07_rb,0.10505e-07_rb,0.10571e-07_rb,0.10619e-07_rb /)
      kbo(:,14, 6) = (/ &
        & 0.84309e-08_rb,0.85528e-08_rb,0.86280e-08_rb,0.86935e-08_rb,0.87236e-08_rb /)
      kbo(:,15, 6) = (/ &
        & 0.69259e-08_rb,0.70218e-08_rb,0.71031e-08_rb,0.71419e-08_rb,0.71430e-08_rb /)
      kbo(:,16, 6) = (/ &
        & 0.56933e-08_rb,0.57765e-08_rb,0.58298e-08_rb,0.58527e-08_rb,0.58704e-08_rb /)
      kbo(:,17, 6) = (/ &
        & 0.46880e-08_rb,0.47555e-08_rb,0.47985e-08_rb,0.48270e-08_rb,0.48471e-08_rb /)
      kbo(:,18, 6) = (/ &
        & 0.38628e-08_rb,0.39183e-08_rb,0.39591e-08_rb,0.39923e-08_rb,0.40106e-08_rb /)
      kbo(:,19, 6) = (/ &
        & 0.31833e-08_rb,0.32370e-08_rb,0.32750e-08_rb,0.32992e-08_rb,0.33139e-08_rb /)
      kbo(:,20, 6) = (/ &
        & 0.26315e-08_rb,0.26765e-08_rb,0.27033e-08_rb,0.27208e-08_rb,0.27412e-08_rb /)
      kbo(:,21, 6) = (/ &
        & 0.21768e-08_rb,0.22074e-08_rb,0.22281e-08_rb,0.22492e-08_rb,0.22648e-08_rb /)
      kbo(:,22, 6) = (/ &
        & 0.17970e-08_rb,0.18204e-08_rb,0.18421e-08_rb,0.18603e-08_rb,0.18697e-08_rb /)
      kbo(:,23, 6) = (/ &
        & 0.14828e-08_rb,0.15043e-08_rb,0.15235e-08_rb,0.15373e-08_rb,0.15469e-08_rb /)
      kbo(:,24, 6) = (/ &
        & 0.12265e-08_rb,0.12457e-08_rb,0.12602e-08_rb,0.12704e-08_rb,0.12795e-08_rb /)
      kbo(:,25, 6) = (/ &
        & 0.10153e-08_rb,0.10303e-08_rb,0.10406e-08_rb,0.10511e-08_rb,0.10591e-08_rb /)
      kbo(:,26, 6) = (/ &
        & 0.84067e-09_rb,0.85220e-09_rb,0.86139e-09_rb,0.87145e-09_rb,0.87606e-09_rb /)
      kbo(:,27, 6) = (/ &
        & 0.69556e-09_rb,0.70441e-09_rb,0.71396e-09_rb,0.72098e-09_rb,0.72486e-09_rb /)
      kbo(:,28, 6) = (/ &
        & 0.57505e-09_rb,0.58326e-09_rb,0.59186e-09_rb,0.59662e-09_rb,0.60056e-09_rb /)
      kbo(:,29, 6) = (/ &
        & 0.47621e-09_rb,0.48459e-09_rb,0.49079e-09_rb,0.49432e-09_rb,0.49717e-09_rb /)
      kbo(:,30, 6) = (/ &
        & 0.39513e-09_rb,0.40232e-09_rb,0.40640e-09_rb,0.40995e-09_rb,0.41263e-09_rb /)
      kbo(:,31, 6) = (/ &
        & 0.32890e-09_rb,0.33390e-09_rb,0.33797e-09_rb,0.34036e-09_rb,0.34440e-09_rb /)
      kbo(:,32, 6) = (/ &
        & 0.27372e-09_rb,0.27757e-09_rb,0.28057e-09_rb,0.28360e-09_rb,0.28668e-09_rb /)
      kbo(:,33, 6) = (/ &
        & 0.22771e-09_rb,0.23074e-09_rb,0.23349e-09_rb,0.23595e-09_rb,0.23797e-09_rb /)
      kbo(:,34, 6) = (/ &
        & 0.18899e-09_rb,0.19179e-09_rb,0.19391e-09_rb,0.19591e-09_rb,0.19720e-09_rb /)
      kbo(:,35, 6) = (/ &
        & 0.15677e-09_rb,0.15897e-09_rb,0.16075e-09_rb,0.16219e-09_rb,0.16318e-09_rb /)
      kbo(:,36, 6) = (/ &
        & 0.12972e-09_rb,0.13147e-09_rb,0.13291e-09_rb,0.13412e-09_rb,0.13484e-09_rb /)
      kbo(:,37, 6) = (/ &
        & 0.10704e-09_rb,0.10855e-09_rb,0.10970e-09_rb,0.11065e-09_rb,0.11129e-09_rb /)
      kbo(:,38, 6) = (/ &
        & 0.88219e-10_rb,0.89571e-10_rb,0.90489e-10_rb,0.91263e-10_rb,0.91821e-10_rb /)
      kbo(:,39, 6) = (/ &
        & 0.72709e-10_rb,0.73884e-10_rb,0.74615e-10_rb,0.75144e-10_rb,0.75719e-10_rb /)
      kbo(:,40, 6) = (/ &
        & 0.59779e-10_rb,0.60793e-10_rb,0.61424e-10_rb,0.61873e-10_rb,0.62379e-10_rb /)
      kbo(:,41, 6) = (/ &
        & 0.49104e-10_rb,0.49999e-10_rb,0.50541e-10_rb,0.50924e-10_rb,0.51369e-10_rb /)
      kbo(:,42, 6) = (/ &
        & 0.40324e-10_rb,0.41079e-10_rb,0.41584e-10_rb,0.41914e-10_rb,0.42279e-10_rb /)
      kbo(:,43, 6) = (/ &
        & 0.33050e-10_rb,0.33695e-10_rb,0.34173e-10_rb,0.34462e-10_rb,0.34716e-10_rb /)
      kbo(:,44, 6) = (/ &
        & 0.27054e-10_rb,0.27604e-10_rb,0.28056e-10_rb,0.28313e-10_rb,0.28542e-10_rb /)
      kbo(:,45, 6) = (/ &
        & 0.22112e-10_rb,0.22613e-10_rb,0.23018e-10_rb,0.23255e-10_rb,0.23441e-10_rb /)
      kbo(:,46, 6) = (/ &
        & 0.18060e-10_rb,0.18501e-10_rb,0.18867e-10_rb,0.19082e-10_rb,0.19252e-10_rb /)
      kbo(:,47, 6) = (/ &
        & 0.14722e-10_rb,0.15111e-10_rb,0.15432e-10_rb,0.15658e-10_rb,0.15802e-10_rb /)
      kbo(:,48, 6) = (/ &
        & 0.11985e-10_rb,0.12318e-10_rb,0.12624e-10_rb,0.12826e-10_rb,0.12965e-10_rb /)
      kbo(:,49, 6) = (/ &
        & 0.97421e-11_rb,0.10040e-10_rb,0.10306e-10_rb,0.10499e-10_rb,0.10630e-10_rb /)
      kbo(:,50, 6) = (/ &
        & 0.79055e-11_rb,0.81823e-11_rb,0.84121e-11_rb,0.85902e-11_rb,0.87163e-11_rb /)
      kbo(:,51, 6) = (/ &
        & 0.64164e-11_rb,0.66612e-11_rb,0.68621e-11_rb,0.70240e-11_rb,0.71428e-11_rb /)
      kbo(:,52, 6) = (/ &
        & 0.51935e-11_rb,0.54159e-11_rb,0.55861e-11_rb,0.57406e-11_rb,0.58465e-11_rb /)
      kbo(:,53, 6) = (/ &
        & 0.42017e-11_rb,0.43985e-11_rb,0.45525e-11_rb,0.46862e-11_rb,0.47817e-11_rb /)
      kbo(:,54, 6) = (/ &
        & 0.33953e-11_rb,0.35701e-11_rb,0.37080e-11_rb,0.38204e-11_rb,0.39118e-11_rb /)
      kbo(:,55, 6) = (/ &
        & 0.27420e-11_rb,0.28948e-11_rb,0.30166e-11_rb,0.31148e-11_rb,0.31981e-11_rb /)
      kbo(:,56, 6) = (/ &
        & 0.22097e-11_rb,0.23440e-11_rb,0.24518e-11_rb,0.25378e-11_rb,0.26122e-11_rb /)
      kbo(:,57, 6) = (/ &
        & 0.17765e-11_rb,0.18968e-11_rb,0.19913e-11_rb,0.20659e-11_rb,0.21319e-11_rb /)
      kbo(:,58, 6) = (/ &
        & 0.14264e-11_rb,0.15324e-11_rb,0.16166e-11_rb,0.16816e-11_rb,0.17374e-11_rb /)
      kbo(:,59, 6) = (/ &
        & 0.11621e-11_rb,0.12515e-11_rb,0.13230e-11_rb,0.13773e-11_rb,0.14240e-11_rb /)
      kbo(:,13, 7) = (/ &
        & 0.28079e-07_rb,0.28035e-07_rb,0.27949e-07_rb,0.27835e-07_rb,0.27661e-07_rb /)
      kbo(:,14, 7) = (/ &
        & 0.23041e-07_rb,0.22982e-07_rb,0.22914e-07_rb,0.22814e-07_rb,0.22653e-07_rb /)
      kbo(:,15, 7) = (/ &
        & 0.18899e-07_rb,0.18860e-07_rb,0.18790e-07_rb,0.18675e-07_rb,0.18596e-07_rb /)
      kbo(:,16, 7) = (/ &
        & 0.15489e-07_rb,0.15457e-07_rb,0.15389e-07_rb,0.15337e-07_rb,0.15329e-07_rb /)
      kbo(:,17, 7) = (/ &
        & 0.12699e-07_rb,0.12665e-07_rb,0.12634e-07_rb,0.12640e-07_rb,0.12612e-07_rb /)
      kbo(:,18, 7) = (/ &
        & 0.10409e-07_rb,0.10397e-07_rb,0.10401e-07_rb,0.10391e-07_rb,0.10368e-07_rb /)
      kbo(:,19, 7) = (/ &
        & 0.85482e-08_rb,0.85403e-08_rb,0.85408e-08_rb,0.85379e-08_rb,0.85325e-08_rb /)
      kbo(:,20, 7) = (/ &
        & 0.70183e-08_rb,0.70212e-08_rb,0.70225e-08_rb,0.70232e-08_rb,0.70243e-08_rb /)
      kbo(:,21, 7) = (/ &
        & 0.57704e-08_rb,0.57836e-08_rb,0.57919e-08_rb,0.57990e-08_rb,0.58135e-08_rb /)
      kbo(:,22, 7) = (/ &
        & 0.47522e-08_rb,0.47673e-08_rb,0.47834e-08_rb,0.48084e-08_rb,0.48239e-08_rb /)
      kbo(:,23, 7) = (/ &
        & 0.39186e-08_rb,0.39439e-08_rb,0.39679e-08_rb,0.39863e-08_rb,0.40063e-08_rb /)
      kbo(:,24, 7) = (/ &
        & 0.32407e-08_rb,0.32645e-08_rb,0.32840e-08_rb,0.33039e-08_rb,0.33353e-08_rb /)
      kbo(:,25, 7) = (/ &
        & 0.26846e-08_rb,0.27012e-08_rb,0.27199e-08_rb,0.27477e-08_rb,0.27761e-08_rb /)
      kbo(:,26, 7) = (/ &
        & 0.22196e-08_rb,0.22367e-08_rb,0.22646e-08_rb,0.22896e-08_rb,0.23141e-08_rb /)
      kbo(:,27, 7) = (/ &
        & 0.18370e-08_rb,0.18616e-08_rb,0.18864e-08_rb,0.19087e-08_rb,0.19319e-08_rb /)
      kbo(:,28, 7) = (/ &
        & 0.15259e-08_rb,0.15495e-08_rb,0.15695e-08_rb,0.15912e-08_rb,0.16152e-08_rb /)
      kbo(:,29, 7) = (/ &
        & 0.12717e-08_rb,0.12907e-08_rb,0.13087e-08_rb,0.13304e-08_rb,0.13523e-08_rb /)
      kbo(:,30, 7) = (/ &
        & 0.10599e-08_rb,0.10765e-08_rb,0.10939e-08_rb,0.11139e-08_rb,0.11374e-08_rb /)
      kbo(:,31, 7) = (/ &
        & 0.88411e-09_rb,0.89833e-09_rb,0.91628e-09_rb,0.93662e-09_rb,0.95540e-09_rb /)
      kbo(:,32, 7) = (/ &
        & 0.73723e-09_rb,0.75198e-09_rb,0.77042e-09_rb,0.78608e-09_rb,0.80601e-09_rb /)
      kbo(:,33, 7) = (/ &
        & 0.61511e-09_rb,0.63161e-09_rb,0.64646e-09_rb,0.66232e-09_rb,0.68095e-09_rb /)
      kbo(:,34, 7) = (/ &
        & 0.51621e-09_rb,0.53001e-09_rb,0.54332e-09_rb,0.55986e-09_rb,0.57326e-09_rb /)
      kbo(:,35, 7) = (/ &
        & 0.43217e-09_rb,0.44439e-09_rb,0.45713e-09_rb,0.47103e-09_rb,0.48242e-09_rb /)
      kbo(:,36, 7) = (/ &
        & 0.36090e-09_rb,0.37166e-09_rb,0.38311e-09_rb,0.39461e-09_rb,0.40545e-09_rb /)
      kbo(:,37, 7) = (/ &
        & 0.29990e-09_rb,0.30916e-09_rb,0.31937e-09_rb,0.32943e-09_rb,0.34038e-09_rb /)
      kbo(:,38, 7) = (/ &
        & 0.24927e-09_rb,0.25737e-09_rb,0.26638e-09_rb,0.27543e-09_rb,0.28596e-09_rb /)
      kbo(:,39, 7) = (/ &
        & 0.20718e-09_rb,0.21434e-09_rb,0.22203e-09_rb,0.23026e-09_rb,0.23966e-09_rb /)
      kbo(:,40, 7) = (/ &
        & 0.17122e-09_rb,0.17745e-09_rb,0.18425e-09_rb,0.19138e-09_rb,0.19937e-09_rb /)
      kbo(:,41, 7) = (/ &
        & 0.14145e-09_rb,0.14673e-09_rb,0.15273e-09_rb,0.15885e-09_rb,0.16544e-09_rb /)
      kbo(:,42, 7) = (/ &
        & 0.11673e-09_rb,0.12130e-09_rb,0.12626e-09_rb,0.13156e-09_rb,0.13718e-09_rb /)
      kbo(:,43, 7) = (/ &
        & 0.95979e-10_rb,0.99773e-10_rb,0.10407e-09_rb,0.10848e-09_rb,0.11289e-09_rb /)
      kbo(:,44, 7) = (/ &
        & 0.78684e-10_rb,0.81894e-10_rb,0.85476e-10_rb,0.89249e-10_rb,0.92971e-10_rb /)
      kbo(:,45, 7) = (/ &
        & 0.64482e-10_rb,0.67161e-10_rb,0.70163e-10_rb,0.73315e-10_rb,0.76486e-10_rb /)
      kbo(:,46, 7) = (/ &
        & 0.52720e-10_rb,0.54998e-10_rb,0.57509e-10_rb,0.60128e-10_rb,0.62787e-10_rb /)
      kbo(:,47, 7) = (/ &
        & 0.42902e-10_rb,0.44887e-10_rb,0.46948e-10_rb,0.49098e-10_rb,0.51392e-10_rb /)
      kbo(:,48, 7) = (/ &
        & 0.34914e-10_rb,0.36612e-10_rb,0.38268e-10_rb,0.40114e-10_rb,0.41998e-10_rb /)
      kbo(:,49, 7) = (/ &
        & 0.28363e-10_rb,0.29830e-10_rb,0.31160e-10_rb,0.32712e-10_rb,0.34278e-10_rb /)
      kbo(:,50, 7) = (/ &
        & 0.23044e-10_rb,0.24298e-10_rb,0.25411e-10_rb,0.26663e-10_rb,0.27956e-10_rb /)
      kbo(:,51, 7) = (/ &
        & 0.18719e-10_rb,0.19762e-10_rb,0.20717e-10_rb,0.21748e-10_rb,0.22834e-10_rb /)
      kbo(:,52, 7) = (/ &
        & 0.15214e-10_rb,0.16060e-10_rb,0.16882e-10_rb,0.17722e-10_rb,0.18609e-10_rb /)
      kbo(:,53, 7) = (/ &
        & 0.12339e-10_rb,0.13039e-10_rb,0.13738e-10_rb,0.14413e-10_rb,0.15144e-10_rb /)
      kbo(:,54, 7) = (/ &
        & 0.10028e-10_rb,0.10589e-10_rb,0.11176e-10_rb,0.11743e-10_rb,0.12340e-10_rb /)
      kbo(:,55, 7) = (/ &
        & 0.81332e-11_rb,0.85988e-11_rb,0.90904e-11_rb,0.95545e-11_rb,0.10057e-10_rb /)
      kbo(:,56, 7) = (/ &
        & 0.66047e-11_rb,0.69777e-11_rb,0.73796e-11_rb,0.77755e-11_rb,0.81880e-11_rb /)
      kbo(:,57, 7) = (/ &
        & 0.53536e-11_rb,0.56554e-11_rb,0.59940e-11_rb,0.63232e-11_rb,0.66612e-11_rb /)
      kbo(:,58, 7) = (/ &
        & 0.43329e-11_rb,0.45903e-11_rb,0.48661e-11_rb,0.51412e-11_rb,0.54200e-11_rb /)
      kbo(:,59, 7) = (/ &
        & 0.35522e-11_rb,0.37646e-11_rb,0.39940e-11_rb,0.42211e-11_rb,0.44525e-11_rb /)
      kbo(:,13, 8) = (/ &
        & 0.11230e-06_rb,0.11409e-06_rb,0.11566e-06_rb,0.11672e-06_rb,0.11734e-06_rb /)
      kbo(:,14, 8) = (/ &
        & 0.92428e-07_rb,0.93878e-07_rb,0.95014e-07_rb,0.95711e-07_rb,0.96110e-07_rb /)
      kbo(:,15, 8) = (/ &
        & 0.76021e-07_rb,0.77148e-07_rb,0.77913e-07_rb,0.78505e-07_rb,0.78756e-07_rb /)
      kbo(:,16, 8) = (/ &
        & 0.62512e-07_rb,0.63402e-07_rb,0.63979e-07_rb,0.64292e-07_rb,0.64352e-07_rb /)
      kbo(:,17, 8) = (/ &
        & 0.51480e-07_rb,0.52099e-07_rb,0.52465e-07_rb,0.52634e-07_rb,0.52753e-07_rb /)
      kbo(:,18, 8) = (/ &
        & 0.42336e-07_rb,0.42773e-07_rb,0.43015e-07_rb,0.43145e-07_rb,0.43290e-07_rb /)
      kbo(:,19, 8) = (/ &
        & 0.34820e-07_rb,0.35138e-07_rb,0.35326e-07_rb,0.35465e-07_rb,0.35546e-07_rb /)
      kbo(:,20, 8) = (/ &
        & 0.28634e-07_rb,0.28882e-07_rb,0.29099e-07_rb,0.29190e-07_rb,0.29248e-07_rb /)
      kbo(:,21, 8) = (/ &
        & 0.23527e-07_rb,0.23767e-07_rb,0.23879e-07_rb,0.24001e-07_rb,0.24136e-07_rb /)
      kbo(:,22, 8) = (/ &
        & 0.19380e-07_rb,0.19526e-07_rb,0.19668e-07_rb,0.19823e-07_rb,0.20000e-07_rb /)
      kbo(:,23, 8) = (/ &
        & 0.15955e-07_rb,0.16103e-07_rb,0.16252e-07_rb,0.16425e-07_rb,0.16561e-07_rb /)
      kbo(:,24, 8) = (/ &
        & 0.13163e-07_rb,0.13294e-07_rb,0.13466e-07_rb,0.13609e-07_rb,0.13701e-07_rb /)
      kbo(:,25, 8) = (/ &
        & 0.10863e-07_rb,0.11029e-07_rb,0.11155e-07_rb,0.11256e-07_rb,0.11368e-07_rb /)
      kbo(:,26, 8) = (/ &
        & 0.90043e-08_rb,0.91173e-08_rb,0.92226e-08_rb,0.93261e-08_rb,0.94322e-08_rb /)
      kbo(:,27, 8) = (/ &
        & 0.74604e-08_rb,0.75548e-08_rb,0.76410e-08_rb,0.77390e-08_rb,0.78488e-08_rb /)
      kbo(:,28, 8) = (/ &
        & 0.61944e-08_rb,0.62754e-08_rb,0.63658e-08_rb,0.64728e-08_rb,0.65523e-08_rb /)
      kbo(:,29, 8) = (/ &
        & 0.51553e-08_rb,0.52390e-08_rb,0.53444e-08_rb,0.54244e-08_rb,0.55104e-08_rb /)
      kbo(:,30, 8) = (/ &
        & 0.43022e-08_rb,0.44056e-08_rb,0.44906e-08_rb,0.45753e-08_rb,0.46815e-08_rb /)
      kbo(:,31, 8) = (/ &
        & 0.36260e-08_rb,0.37147e-08_rb,0.37854e-08_rb,0.38857e-08_rb,0.39725e-08_rb /)
      kbo(:,32, 8) = (/ &
        & 0.30574e-08_rb,0.31222e-08_rb,0.32149e-08_rb,0.32959e-08_rb,0.33679e-08_rb /)
      kbo(:,33, 8) = (/ &
        & 0.25690e-08_rb,0.26475e-08_rb,0.27306e-08_rb,0.27942e-08_rb,0.28607e-08_rb /)
      kbo(:,34, 8) = (/ &
        & 0.21739e-08_rb,0.22522e-08_rb,0.23131e-08_rb,0.23724e-08_rb,0.24398e-08_rb /)
      kbo(:,35, 8) = (/ &
        & 0.18403e-08_rb,0.19033e-08_rb,0.19580e-08_rb,0.20191e-08_rb,0.20792e-08_rb /)
      kbo(:,36, 8) = (/ &
        & 0.15538e-08_rb,0.16068e-08_rb,0.16535e-08_rb,0.17159e-08_rb,0.17681e-08_rb /)
      kbo(:,37, 8) = (/ &
        & 0.13002e-08_rb,0.13483e-08_rb,0.13923e-08_rb,0.14493e-08_rb,0.14927e-08_rb /)
      kbo(:,38, 8) = (/ &
        & 0.10893e-08_rb,0.11319e-08_rb,0.11765e-08_rb,0.12253e-08_rb,0.12624e-08_rb /)
      kbo(:,39, 8) = (/ &
        & 0.91348e-09_rb,0.95067e-09_rb,0.99134e-09_rb,0.10293e-08_rb,0.10668e-08_rb /)
      kbo(:,40, 8) = (/ &
        & 0.76170e-09_rb,0.79404e-09_rb,0.82953e-09_rb,0.86548e-09_rb,0.90041e-09_rb /)
      kbo(:,41, 8) = (/ &
        & 0.63292e-09_rb,0.66170e-09_rb,0.69363e-09_rb,0.72573e-09_rb,0.75710e-09_rb /)
      kbo(:,42, 8) = (/ &
        & 0.52609e-09_rb,0.55120e-09_rb,0.57974e-09_rb,0.60803e-09_rb,0.63636e-09_rb /)
      kbo(:,43, 8) = (/ &
        & 0.43444e-09_rb,0.45674e-09_rb,0.48206e-09_rb,0.50747e-09_rb,0.53053e-09_rb /)
      kbo(:,44, 8) = (/ &
        & 0.35755e-09_rb,0.37758e-09_rb,0.39965e-09_rb,0.42169e-09_rb,0.44276e-09_rb /)
      kbo(:,45, 8) = (/ &
        & 0.29410e-09_rb,0.31126e-09_rb,0.33079e-09_rb,0.35014e-09_rb,0.36874e-09_rb /)
      kbo(:,46, 8) = (/ &
        & 0.24113e-09_rb,0.25605e-09_rb,0.27273e-09_rb,0.28962e-09_rb,0.30646e-09_rb /)
      kbo(:,47, 8) = (/ &
        & 0.19675e-09_rb,0.20937e-09_rb,0.22390e-09_rb,0.23865e-09_rb,0.25330e-09_rb /)
      kbo(:,48, 8) = (/ &
        & 0.16029e-09_rb,0.17125e-09_rb,0.18339e-09_rb,0.19620e-09_rb,0.20918e-09_rb /)
      kbo(:,49, 8) = (/ &
        & 0.13018e-09_rb,0.13961e-09_rb,0.15020e-09_rb,0.16108e-09_rb,0.17231e-09_rb /)
      kbo(:,50, 8) = (/ &
        & 0.10559e-09_rb,0.11394e-09_rb,0.12271e-09_rb,0.13208e-09_rb,0.14200e-09_rb /)
      kbo(:,51, 8) = (/ &
        & 0.85569e-10_rb,0.92775e-10_rb,0.10011e-09_rb,0.10818e-09_rb,0.11665e-09_rb /)
      kbo(:,52, 8) = (/ &
        & 0.69342e-10_rb,0.75341e-10_rb,0.81596e-10_rb,0.88527e-10_rb,0.95876e-10_rb /)
      kbo(:,53, 8) = (/ &
        & 0.56003e-10_rb,0.61203e-10_rb,0.66389e-10_rb,0.72376e-10_rb,0.78642e-10_rb /)
      kbo(:,54, 8) = (/ &
        & 0.45380e-10_rb,0.49599e-10_rb,0.54006e-10_rb,0.59069e-10_rb,0.64380e-10_rb /)
      kbo(:,55, 8) = (/ &
        & 0.36637e-10_rb,0.40078e-10_rb,0.43990e-10_rb,0.48279e-10_rb,0.52815e-10_rb /)
      kbo(:,56, 8) = (/ &
        & 0.29596e-10_rb,0.32455e-10_rb,0.35694e-10_rb,0.39315e-10_rb,0.43169e-10_rb /)
      kbo(:,57, 8) = (/ &
        & 0.23911e-10_rb,0.26268e-10_rb,0.28916e-10_rb,0.31929e-10_rb,0.35182e-10_rb /)
      kbo(:,58, 8) = (/ &
        & 0.19299e-10_rb,0.21198e-10_rb,0.23407e-10_rb,0.25936e-10_rb,0.28689e-10_rb /)
      kbo(:,59, 8) = (/ &
        & 0.15839e-10_rb,0.17448e-10_rb,0.19334e-10_rb,0.21469e-10_rb,0.23829e-10_rb /)
      kbo(:,13, 9) = (/ &
        & 0.96471e-06_rb,0.97408e-06_rb,0.98436e-06_rb,0.99439e-06_rb,0.10050e-05_rb /)
      kbo(:,14, 9) = (/ &
        & 0.80904e-06_rb,0.81838e-06_rb,0.82742e-06_rb,0.83703e-06_rb,0.84742e-06_rb /)
      kbo(:,15, 9) = (/ &
        & 0.67733e-06_rb,0.68563e-06_rb,0.69403e-06_rb,0.70345e-06_rb,0.71287e-06_rb /)
      kbo(:,16, 9) = (/ &
        & 0.56582e-06_rb,0.57313e-06_rb,0.58159e-06_rb,0.59012e-06_rb,0.59880e-06_rb /)
      kbo(:,17, 9) = (/ &
        & 0.47164e-06_rb,0.47892e-06_rb,0.48671e-06_rb,0.49436e-06_rb,0.50258e-06_rb /)
      kbo(:,18, 9) = (/ &
        & 0.39308e-06_rb,0.39992e-06_rb,0.40685e-06_rb,0.41415e-06_rb,0.42143e-06_rb /)
      kbo(:,19, 9) = (/ &
        & 0.32764e-06_rb,0.33370e-06_rb,0.34000e-06_rb,0.34660e-06_rb,0.35294e-06_rb /)
      kbo(:,20, 9) = (/ &
        & 0.27304e-06_rb,0.27855e-06_rb,0.28433e-06_rb,0.29002e-06_rb,0.29550e-06_rb /)
      kbo(:,21, 9) = (/ &
        & 0.22762e-06_rb,0.23263e-06_rb,0.23776e-06_rb,0.24264e-06_rb,0.24751e-06_rb /)
      kbo(:,22, 9) = (/ &
        & 0.19016e-06_rb,0.19467e-06_rb,0.19902e-06_rb,0.20331e-06_rb,0.20743e-06_rb /)
      kbo(:,23, 9) = (/ &
        & 0.15902e-06_rb,0.16285e-06_rb,0.16664e-06_rb,0.17032e-06_rb,0.17381e-06_rb /)
      kbo(:,24, 9) = (/ &
        & 0.13300e-06_rb,0.13639e-06_rb,0.13965e-06_rb,0.14271e-06_rb,0.14575e-06_rb /)
      kbo(:,25, 9) = (/ &
        & 0.11136e-06_rb,0.11428e-06_rb,0.11705e-06_rb,0.11966e-06_rb,0.12234e-06_rb /)
      kbo(:,26, 9) = (/ &
        & 0.93355e-07_rb,0.95870e-07_rb,0.98179e-07_rb,0.10046e-06_rb,0.10295e-06_rb /)
      kbo(:,27, 9) = (/ &
        & 0.78546e-07_rb,0.80656e-07_rb,0.82694e-07_rb,0.84793e-07_rb,0.87129e-07_rb /)
      kbo(:,28, 9) = (/ &
        & 0.65991e-07_rb,0.67786e-07_rb,0.69602e-07_rb,0.71624e-07_rb,0.74063e-07_rb /)
      kbo(:,29, 9) = (/ &
        & 0.55459e-07_rb,0.56995e-07_rb,0.58695e-07_rb,0.60741e-07_rb,0.63281e-07_rb /)
      kbo(:,30, 9) = (/ &
        & 0.46750e-07_rb,0.48054e-07_rb,0.49703e-07_rb,0.51793e-07_rb,0.54403e-07_rb /)
      kbo(:,31, 9) = (/ &
        & 0.39452e-07_rb,0.40727e-07_rb,0.42319e-07_rb,0.44486e-07_rb,0.47323e-07_rb /)
      kbo(:,32, 9) = (/ &
        & 0.33473e-07_rb,0.34884e-07_rb,0.36502e-07_rb,0.38781e-07_rb,0.41735e-07_rb /)
      kbo(:,33, 9) = (/ &
        & 0.28581e-07_rb,0.30008e-07_rb,0.31748e-07_rb,0.34217e-07_rb,0.37511e-07_rb /)
      kbo(:,34, 9) = (/ &
        & 0.24450e-07_rb,0.25951e-07_rb,0.27867e-07_rb,0.30565e-07_rb,0.34125e-07_rb /)
      kbo(:,35, 9) = (/ &
        & 0.20942e-07_rb,0.22554e-07_rb,0.24574e-07_rb,0.27409e-07_rb,0.31159e-07_rb /)
      kbo(:,36, 9) = (/ &
        & 0.18081e-07_rb,0.19648e-07_rb,0.21657e-07_rb,0.24573e-07_rb,0.28429e-07_rb /)
      kbo(:,37, 9) = (/ &
        & 0.15436e-07_rb,0.16978e-07_rb,0.19037e-07_rb,0.21710e-07_rb,0.25509e-07_rb /)
      kbo(:,38, 9) = (/ &
        & 0.13187e-07_rb,0.14690e-07_rb,0.16738e-07_rb,0.19257e-07_rb,0.22992e-07_rb /)
      kbo(:,39, 9) = (/ &
        & 0.11282e-07_rb,0.12751e-07_rb,0.14794e-07_rb,0.17292e-07_rb,0.20852e-07_rb /)
      kbo(:,40, 9) = (/ &
        & 0.96293e-08_rb,0.10973e-07_rb,0.12917e-07_rb,0.15272e-07_rb,0.18582e-07_rb /)
      kbo(:,41, 9) = (/ &
        & 0.81770e-08_rb,0.94352e-08_rb,0.11220e-07_rb,0.13516e-07_rb,0.16546e-07_rb /)
      kbo(:,42, 9) = (/ &
        & 0.69323e-08_rb,0.80750e-08_rb,0.97479e-08_rb,0.11964e-07_rb,0.14736e-07_rb /)
      kbo(:,43, 9) = (/ &
        & 0.58312e-08_rb,0.68571e-08_rb,0.83595e-08_rb,0.10414e-07_rb,0.12928e-07_rb /)
      kbo(:,44, 9) = (/ &
        & 0.48754e-08_rb,0.58128e-08_rb,0.71504e-08_rb,0.90270e-08_rb,0.11362e-07_rb /)
      kbo(:,45, 9) = (/ &
        & 0.40522e-08_rb,0.48841e-08_rb,0.60765e-08_rb,0.77697e-08_rb,0.99439e-08_rb /)
      kbo(:,46, 9) = (/ &
        & 0.33429e-08_rb,0.40820e-08_rb,0.51038e-08_rb,0.66237e-08_rb,0.86427e-08_rb /)
      kbo(:,47, 9) = (/ &
        & 0.27297e-08_rb,0.33571e-08_rb,0.42806e-08_rb,0.55863e-08_rb,0.73605e-08_rb /)
      kbo(:,48, 9) = (/ &
        & 0.22143e-08_rb,0.27445e-08_rb,0.35414e-08_rb,0.46711e-08_rb,0.62204e-08_rb /)
      kbo(:,49, 9) = (/ &
        & 0.18030e-08_rb,0.22208e-08_rb,0.28995e-08_rb,0.38639e-08_rb,0.52216e-08_rb /)
      kbo(:,50, 9) = (/ &
        & 0.14670e-08_rb,0.18085e-08_rb,0.23853e-08_rb,0.32522e-08_rb,0.44284e-08_rb /)
      kbo(:,51, 9) = (/ &
        & 0.11873e-08_rb,0.14637e-08_rb,0.19592e-08_rb,0.27073e-08_rb,0.37306e-08_rb /)
      kbo(:,52, 9) = (/ &
        & 0.96876e-09_rb,0.11883e-08_rb,0.15881e-08_rb,0.22316e-08_rb,0.31252e-08_rb /)
      kbo(:,53, 9) = (/ &
        & 0.79526e-09_rb,0.95443e-09_rb,0.12843e-08_rb,0.18180e-08_rb,0.25992e-08_rb /)
      kbo(:,54, 9) = (/ &
        & 0.65045e-09_rb,0.77197e-09_rb,0.10500e-08_rb,0.14963e-08_rb,0.21953e-08_rb /)
      kbo(:,55, 9) = (/ &
        & 0.53001e-09_rb,0.62144e-09_rb,0.84441e-09_rb,0.12265e-08_rb,0.18415e-08_rb /)
      kbo(:,56, 9) = (/ &
        & 0.43274e-09_rb,0.50433e-09_rb,0.67633e-09_rb,0.10051e-08_rb,0.15294e-08_rb /)
      kbo(:,57, 9) = (/ &
        & 0.35151e-09_rb,0.40652e-09_rb,0.53819e-09_rb,0.80840e-09_rb,0.12530e-08_rb /)
      kbo(:,58, 9) = (/ &
        & 0.28511e-09_rb,0.33047e-09_rb,0.42809e-09_rb,0.65584e-09_rb,0.10383e-08_rb /)
      kbo(:,59, 9) = (/ &
        & 0.23557e-09_rb,0.27695e-09_rb,0.36588e-09_rb,0.57468e-09_rb,0.92796e-09_rb /)
      kbo(:,13,10) = (/ &
        & 0.49379e-05_rb,0.50469e-05_rb,0.51595e-05_rb,0.52778e-05_rb,0.54099e-05_rb /)
      kbo(:,14,10) = (/ &
        & 0.42812e-05_rb,0.43888e-05_rb,0.45022e-05_rb,0.46302e-05_rb,0.47669e-05_rb /)
      kbo(:,15,10) = (/ &
        & 0.36929e-05_rb,0.37965e-05_rb,0.39164e-05_rb,0.40454e-05_rb,0.41827e-05_rb /)
      kbo(:,16,10) = (/ &
        & 0.31721e-05_rb,0.32791e-05_rb,0.33975e-05_rb,0.35246e-05_rb,0.36634e-05_rb /)
      kbo(:,17,10) = (/ &
        & 0.27212e-05_rb,0.28264e-05_rb,0.29414e-05_rb,0.30666e-05_rb,0.32085e-05_rb /)
      kbo(:,18,10) = (/ &
        & 0.23339e-05_rb,0.24330e-05_rb,0.25438e-05_rb,0.26711e-05_rb,0.28184e-05_rb /)
      kbo(:,19,10) = (/ &
        & 0.19987e-05_rb,0.20932e-05_rb,0.22036e-05_rb,0.23332e-05_rb,0.24834e-05_rb /)
      kbo(:,20,10) = (/ &
        & 0.17132e-05_rb,0.18061e-05_rb,0.19176e-05_rb,0.20491e-05_rb,0.21989e-05_rb /)
      kbo(:,21,10) = (/ &
        & 0.14710e-05_rb,0.15640e-05_rb,0.16759e-05_rb,0.18061e-05_rb,0.19600e-05_rb /)
      kbo(:,22,10) = (/ &
        & 0.12737e-05_rb,0.13668e-05_rb,0.14788e-05_rb,0.16132e-05_rb,0.17763e-05_rb /)
      kbo(:,23,10) = (/ &
        & 0.11088e-05_rb,0.12015e-05_rb,0.13166e-05_rb,0.14583e-05_rb,0.16304e-05_rb /)
      kbo(:,24,10) = (/ &
        & 0.97145e-06_rb,0.10662e-05_rb,0.11867e-05_rb,0.13370e-05_rb,0.15170e-05_rb /)
      kbo(:,25,10) = (/ &
        & 0.85857e-06_rb,0.95791e-06_rb,0.10855e-05_rb,0.12438e-05_rb,0.14327e-05_rb /)
      kbo(:,26,10) = (/ &
        & 0.76914e-06_rb,0.87488e-06_rb,0.10107e-05_rb,0.11782e-05_rb,0.13762e-05_rb /)
      kbo(:,27,10) = (/ &
        & 0.69709e-06_rb,0.81000e-06_rb,0.95427e-06_rb,0.11311e-05_rb,0.13395e-05_rb /)
      kbo(:,28,10) = (/ &
        & 0.64352e-06_rb,0.76453e-06_rb,0.91846e-06_rb,0.11047e-05_rb,0.13225e-05_rb /)
      kbo(:,29,10) = (/ &
        & 0.60601e-06_rb,0.73668e-06_rb,0.90033e-06_rb,0.10967e-05_rb,0.13241e-05_rb /)
      kbo(:,30,10) = (/ &
        & 0.58144e-06_rb,0.72277e-06_rb,0.89681e-06_rb,0.11031e-05_rb,0.13400e-05_rb /)
      kbo(:,31,10) = (/ &
        & 0.57013e-06_rb,0.72161e-06_rb,0.90691e-06_rb,0.11230e-05_rb,0.13684e-05_rb /)
      kbo(:,32,10) = (/ &
        & 0.57021e-06_rb,0.73098e-06_rb,0.92629e-06_rb,0.11521e-05_rb,0.14071e-05_rb /)
      kbo(:,33,10) = (/ &
        & 0.58025e-06_rb,0.75106e-06_rb,0.95601e-06_rb,0.11909e-05_rb,0.14530e-05_rb /)
      kbo(:,34,10) = (/ &
        & 0.59352e-06_rb,0.77298e-06_rb,0.98581e-06_rb,0.12280e-05_rb,0.14964e-05_rb /)
      kbo(:,35,10) = (/ &
        & 0.59731e-06_rb,0.78147e-06_rb,0.99919e-06_rb,0.12457e-05_rb,0.15176e-05_rb /)
      kbo(:,36,10) = (/ &
        & 0.58516e-06_rb,0.77138e-06_rb,0.99076e-06_rb,0.12378e-05_rb,0.15105e-05_rb /)
      kbo(:,37,10) = (/ &
        & 0.55140e-06_rb,0.73346e-06_rb,0.94817e-06_rb,0.11937e-05_rb,0.14631e-05_rb /)
      kbo(:,38,10) = (/ &
        & 0.51916e-06_rb,0.69705e-06_rb,0.90723e-06_rb,0.11496e-05_rb,0.14156e-05_rb /)
      kbo(:,39,10) = (/ &
        & 0.48960e-06_rb,0.66318e-06_rb,0.86863e-06_rb,0.11068e-05_rb,0.13701e-05_rb /)
      kbo(:,40,10) = (/ &
        & 0.44595e-06_rb,0.61209e-06_rb,0.81011e-06_rb,0.10412e-05_rb,0.12987e-05_rb /)
      kbo(:,41,10) = (/ &
        & 0.40399e-06_rb,0.56185e-06_rb,0.75199e-06_rb,0.97449e-06_rb,0.12265e-05_rb /)
      kbo(:,42,10) = (/ &
        & 0.36503e-06_rb,0.51494e-06_rb,0.69685e-06_rb,0.91108e-06_rb,0.11565e-05_rb /)
      kbo(:,43,10) = (/ &
        & 0.32174e-06_rb,0.46170e-06_rb,0.63377e-06_rb,0.83820e-06_rb,0.10749e-05_rb /)
      kbo(:,44,10) = (/ &
        & 0.27930e-06_rb,0.40829e-06_rb,0.56968e-06_rb,0.76331e-06_rb,0.98961e-06_rb /)
      kbo(:,45,10) = (/ &
        & 0.24096e-06_rb,0.35938e-06_rb,0.50997e-06_rb,0.69291e-06_rb,0.90837e-06_rb /)
      kbo(:,46,10) = (/ &
        & 0.20431e-06_rb,0.31166e-06_rb,0.45096e-06_rb,0.62224e-06_rb,0.82595e-06_rb /)
      kbo(:,47,10) = (/ &
        & 0.16816e-06_rb,0.26359e-06_rb,0.38972e-06_rb,0.54827e-06_rb,0.73938e-06_rb /)
      kbo(:,48,10) = (/ &
        & 0.13670e-06_rb,0.22070e-06_rb,0.33435e-06_rb,0.47996e-06_rb,0.65830e-06_rb /)
      kbo(:,49,10) = (/ &
        & 0.10953e-06_rb,0.18282e-06_rb,0.28430e-06_rb,0.41725e-06_rb,0.58243e-06_rb /)
      kbo(:,50,10) = (/ &
        & 0.88267e-07_rb,0.15143e-06_rb,0.24199e-06_rb,0.36271e-06_rb,0.51595e-06_rb /)
      kbo(:,51,10) = (/ &
        & 0.70298e-07_rb,0.12484e-06_rb,0.20511e-06_rb,0.31462e-06_rb,0.45605e-06_rb /)
      kbo(:,52,10) = (/ &
        & 0.55131e-07_rb,0.10217e-06_rb,0.17234e-06_rb,0.27088e-06_rb,0.40072e-06_rb /)
      kbo(:,53,10) = (/ &
        & 0.43153e-07_rb,0.82195e-07_rb,0.14318e-06_rb,0.23139e-06_rb,0.34955e-06_rb /)
      kbo(:,54,10) = (/ &
        & 0.33903e-07_rb,0.66466e-07_rb,0.12020e-06_rb,0.19856e-06_rb,0.30630e-06_rb /)
      kbo(:,55,10) = (/ &
        & 0.26416e-07_rb,0.53605e-07_rb,0.10034e-06_rb,0.17017e-06_rb,0.26833e-06_rb /)
      kbo(:,56,10) = (/ &
        & 0.20206e-07_rb,0.42611e-07_rb,0.82928e-07_rb,0.14532e-06_rb,0.23365e-06_rb /)
      kbo(:,57,10) = (/ &
        & 0.15310e-07_rb,0.33413e-07_rb,0.67820e-07_rb,0.12262e-06_rb,0.20213e-06_rb /)
      kbo(:,58,10) = (/ &
        & 0.11722e-07_rb,0.26420e-07_rb,0.55414e-07_rb,0.10347e-06_rb,0.17567e-06_rb /)
      kbo(:,59,10) = (/ &
        & 0.10312e-07_rb,0.23851e-07_rb,0.50918e-07_rb,0.96421e-07_rb,0.16547e-06_rb /)
      kbo(:,13,11) = (/ &
        & 0.10213e-04_rb,0.10492e-04_rb,0.10768e-04_rb,0.11052e-04_rb,0.11341e-04_rb /)
      kbo(:,14,11) = (/ &
        & 0.92065e-05_rb,0.95170e-05_rb,0.98374e-05_rb,0.10166e-04_rb,0.10515e-04_rb /)
      kbo(:,15,11) = (/ &
        & 0.82447e-05_rb,0.85845e-05_rb,0.89388e-05_rb,0.93243e-05_rb,0.97313e-05_rb /)
      kbo(:,16,11) = (/ &
        & 0.73562e-05_rb,0.77204e-05_rb,0.81177e-05_rb,0.85472e-05_rb,0.89941e-05_rb /)
      kbo(:,17,11) = (/ &
        & 0.65534e-05_rb,0.69440e-05_rb,0.73753e-05_rb,0.78408e-05_rb,0.83202e-05_rb /)
      kbo(:,18,11) = (/ &
        & 0.58421e-05_rb,0.62603e-05_rb,0.67231e-05_rb,0.72105e-05_rb,0.77276e-05_rb /)
      kbo(:,19,11) = (/ &
        & 0.52249e-05_rb,0.56681e-05_rb,0.61484e-05_rb,0.66668e-05_rb,0.72285e-05_rb /)
      kbo(:,20,11) = (/ &
        & 0.47032e-05_rb,0.51617e-05_rb,0.56660e-05_rb,0.62226e-05_rb,0.68302e-05_rb /)
      kbo(:,21,11) = (/ &
        & 0.42601e-05_rb,0.47338e-05_rb,0.52698e-05_rb,0.58670e-05_rb,0.65065e-05_rb /)
      kbo(:,22,11) = (/ &
        & 0.39167e-05_rb,0.44205e-05_rb,0.49953e-05_rb,0.56267e-05_rb,0.62956e-05_rb /)
      kbo(:,23,11) = (/ &
        & 0.36499e-05_rb,0.41883e-05_rb,0.47965e-05_rb,0.54568e-05_rb,0.61529e-05_rb /)
      kbo(:,24,11) = (/ &
        & 0.34565e-05_rb,0.40283e-05_rb,0.46653e-05_rb,0.53526e-05_rb,0.60723e-05_rb /)
      kbo(:,25,11) = (/ &
        & 0.33292e-05_rb,0.39315e-05_rb,0.45958e-05_rb,0.53082e-05_rb,0.60473e-05_rb /)
      kbo(:,26,11) = (/ &
        & 0.32653e-05_rb,0.38970e-05_rb,0.45879e-05_rb,0.53204e-05_rb,0.60776e-05_rb /)
      kbo(:,27,11) = (/ &
        & 0.32497e-05_rb,0.39083e-05_rb,0.46221e-05_rb,0.53722e-05_rb,0.61437e-05_rb /)
      kbo(:,28,11) = (/ &
        & 0.32741e-05_rb,0.39580e-05_rb,0.46902e-05_rb,0.54541e-05_rb,0.62354e-05_rb /)
      kbo(:,29,11) = (/ &
        & 0.33369e-05_rb,0.40429e-05_rb,0.47902e-05_rb,0.55664e-05_rb,0.63553e-05_rb /)
      kbo(:,30,11) = (/ &
        & 0.34283e-05_rb,0.41504e-05_rb,0.49124e-05_rb,0.56972e-05_rb,0.64922e-05_rb /)
      kbo(:,31,11) = (/ &
        & 0.35444e-05_rb,0.42823e-05_rb,0.50547e-05_rb,0.58477e-05_rb,0.66477e-05_rb /)
      kbo(:,32,11) = (/ &
        & 0.36801e-05_rb,0.44317e-05_rb,0.52138e-05_rb,0.60136e-05_rb,0.68176e-05_rb /)
      kbo(:,33,11) = (/ &
        & 0.38318e-05_rb,0.45951e-05_rb,0.53855e-05_rb,0.61901e-05_rb,0.69963e-05_rb /)
      kbo(:,34,11) = (/ &
        & 0.39692e-05_rb,0.47416e-05_rb,0.55387e-05_rb,0.63463e-05_rb,0.71529e-05_rb /)
      kbo(:,35,11) = (/ &
        & 0.40413e-05_rb,0.48197e-05_rb,0.56200e-05_rb,0.64302e-05_rb,0.72378e-05_rb /)
      kbo(:,36,11) = (/ &
        & 0.40356e-05_rb,0.48162e-05_rb,0.56178e-05_rb,0.64290e-05_rb,0.72385e-05_rb /)
      kbo(:,37,11) = (/ &
        & 0.39181e-05_rb,0.46953e-05_rb,0.54967e-05_rb,0.63083e-05_rb,0.71198e-05_rb /)
      kbo(:,38,11) = (/ &
        & 0.37977e-05_rb,0.45715e-05_rb,0.53704e-05_rb,0.61820e-05_rb,0.69964e-05_rb /)
      kbo(:,39,11) = (/ &
        & 0.36811e-05_rb,0.44507e-05_rb,0.52481e-05_rb,0.60593e-05_rb,0.68739e-05_rb /)
      kbo(:,40,11) = (/ &
        & 0.34909e-05_rb,0.42522e-05_rb,0.50444e-05_rb,0.58543e-05_rb,0.66697e-05_rb /)
      kbo(:,41,11) = (/ &
        & 0.32953e-05_rb,0.40465e-05_rb,0.48326e-05_rb,0.56401e-05_rb,0.64551e-05_rb /)
      kbo(:,42,11) = (/ &
        & 0.31040e-05_rb,0.38444e-05_rb,0.46230e-05_rb,0.54267e-05_rb,0.62419e-05_rb /)
      kbo(:,43,11) = (/ &
        & 0.28773e-05_rb,0.36028e-05_rb,0.43716e-05_rb,0.51693e-05_rb,0.59825e-05_rb /)
      kbo(:,44,11) = (/ &
        & 0.26404e-05_rb,0.33480e-05_rb,0.41036e-05_rb,0.48939e-05_rb,0.57034e-05_rb /)
      kbo(:,45,11) = (/ &
        & 0.24120e-05_rb,0.30991e-05_rb,0.38405e-05_rb,0.46208e-05_rb,0.54260e-05_rb /)
      kbo(:,46,11) = (/ &
        & 0.21786e-05_rb,0.28418e-05_rb,0.35659e-05_rb,0.43343e-05_rb,0.51318e-05_rb /)
      kbo(:,47,11) = (/ &
        & 0.19311e-05_rb,0.25641e-05_rb,0.32660e-05_rb,0.40180e-05_rb,0.48060e-05_rb /)
      kbo(:,48,11) = (/ &
        & 0.16967e-05_rb,0.22980e-05_rb,0.29746e-05_rb,0.37093e-05_rb,0.44842e-05_rb /)
      kbo(:,49,11) = (/ &
        & 0.14774e-05_rb,0.20448e-05_rb,0.26926e-05_rb,0.34064e-05_rb,0.41668e-05_rb /)
      kbo(:,50,11) = (/ &
        & 0.12873e-05_rb,0.18193e-05_rb,0.24384e-05_rb,0.31296e-05_rb,0.38742e-05_rb /)
      kbo(:,51,11) = (/ &
        & 0.11146e-05_rb,0.16126e-05_rb,0.22020e-05_rb,0.28686e-05_rb,0.35959e-05_rb /)
      kbo(:,52,11) = (/ &
        & 0.95604e-06_rb,0.14200e-05_rb,0.19761e-05_rb,0.26163e-05_rb,0.33236e-05_rb /)
      kbo(:,53,11) = (/ &
        & 0.80966e-06_rb,0.12381e-05_rb,0.17610e-05_rb,0.23732e-05_rb,0.30580e-05_rb /)
      kbo(:,54,11) = (/ &
        & 0.68766e-06_rb,0.10830e-05_rb,0.15763e-05_rb,0.21583e-05_rb,0.28207e-05_rb /)
      kbo(:,55,11) = (/ &
        & 0.58212e-06_rb,0.94518e-06_rb,0.14071e-05_rb,0.19616e-05_rb,0.26001e-05_rb /)
      kbo(:,56,11) = (/ &
        & 0.48769e-06_rb,0.81784e-06_rb,0.12480e-05_rb,0.17754e-05_rb,0.23872e-05_rb /)
      kbo(:,57,11) = (/ &
        & 0.40334e-06_rb,0.70099e-06_rb,0.10993e-05_rb,0.15960e-05_rb,0.21818e-05_rb /)
      kbo(:,58,11) = (/ &
        & 0.33254e-06_rb,0.59979e-06_rb,0.96749e-06_rb,0.14348e-05_rb,0.19961e-05_rb /)
      kbo(:,59,11) = (/ &
        & 0.30667e-06_rb,0.56182e-06_rb,0.91722e-06_rb,0.13728e-05_rb,0.19233e-05_rb /)
      kbo(:,13,12) = (/ &
        & 0.21780e-04_rb,0.22297e-04_rb,0.22851e-04_rb,0.23387e-04_rb,0.23897e-04_rb /)
      kbo(:,14,12) = (/ &
        & 0.20875e-04_rb,0.21531e-04_rb,0.22189e-04_rb,0.22823e-04_rb,0.23424e-04_rb /)
      kbo(:,15,12) = (/ &
        & 0.19941e-04_rb,0.20728e-04_rb,0.21492e-04_rb,0.22225e-04_rb,0.22948e-04_rb /)
      kbo(:,16,12) = (/ &
        & 0.19013e-04_rb,0.19918e-04_rb,0.20796e-04_rb,0.21654e-04_rb,0.22503e-04_rb /)
      kbo(:,17,12) = (/ &
        & 0.18129e-04_rb,0.19150e-04_rb,0.20149e-04_rb,0.21144e-04_rb,0.22094e-04_rb /)
      kbo(:,18,12) = (/ &
        & 0.17308e-04_rb,0.18451e-04_rb,0.19578e-04_rb,0.20689e-04_rb,0.21728e-04_rb /)
      kbo(:,19,12) = (/ &
        & 0.16581e-04_rb,0.17835e-04_rb,0.19087e-04_rb,0.20285e-04_rb,0.21402e-04_rb /)
      kbo(:,20,12) = (/ &
        & 0.15978e-04_rb,0.17349e-04_rb,0.18704e-04_rb,0.19972e-04_rb,0.21143e-04_rb /)
      kbo(:,21,12) = (/ &
        & 0.15496e-04_rb,0.16976e-04_rb,0.18403e-04_rb,0.19724e-04_rb,0.20952e-04_rb /)
      kbo(:,22,12) = (/ &
        & 0.15240e-04_rb,0.16798e-04_rb,0.18272e-04_rb,0.19635e-04_rb,0.20901e-04_rb /)
      kbo(:,23,12) = (/ &
        & 0.15105e-04_rb,0.16716e-04_rb,0.18220e-04_rb,0.19617e-04_rb,0.20916e-04_rb /)
      kbo(:,24,12) = (/ &
        & 0.15076e-04_rb,0.16719e-04_rb,0.18249e-04_rb,0.19672e-04_rb,0.20991e-04_rb /)
      kbo(:,25,12) = (/ &
        & 0.15141e-04_rb,0.16802e-04_rb,0.18352e-04_rb,0.19786e-04_rb,0.21116e-04_rb /)
      kbo(:,26,12) = (/ &
        & 0.15306e-04_rb,0.16974e-04_rb,0.18529e-04_rb,0.19965e-04_rb,0.21296e-04_rb /)
      kbo(:,27,12) = (/ &
        & 0.15522e-04_rb,0.17192e-04_rb,0.18747e-04_rb,0.20182e-04_rb,0.21501e-04_rb /)
      kbo(:,28,12) = (/ &
        & 0.15784e-04_rb,0.17449e-04_rb,0.18996e-04_rb,0.20419e-04_rb,0.21733e-04_rb /)
      kbo(:,29,12) = (/ &
        & 0.16090e-04_rb,0.17744e-04_rb,0.19276e-04_rb,0.20687e-04_rb,0.21987e-04_rb /)
      kbo(:,30,12) = (/ &
        & 0.16419e-04_rb,0.18062e-04_rb,0.19575e-04_rb,0.20970e-04_rb,0.22248e-04_rb /)
      kbo(:,31,12) = (/ &
        & 0.16777e-04_rb,0.18399e-04_rb,0.19892e-04_rb,0.21268e-04_rb,0.22521e-04_rb /)
      kbo(:,32,12) = (/ &
        & 0.17152e-04_rb,0.18750e-04_rb,0.20222e-04_rb,0.21573e-04_rb,0.22802e-04_rb /)
      kbo(:,33,12) = (/ &
        & 0.17537e-04_rb,0.19109e-04_rb,0.20556e-04_rb,0.21883e-04_rb,0.23082e-04_rb /)
      kbo(:,34,12) = (/ &
        & 0.17871e-04_rb,0.19418e-04_rb,0.20844e-04_rb,0.22147e-04_rb,0.23322e-04_rb /)
      kbo(:,35,12) = (/ &
        & 0.18052e-04_rb,0.19587e-04_rb,0.21002e-04_rb,0.22291e-04_rb,0.23455e-04_rb /)
      kbo(:,36,12) = (/ &
        & 0.18063e-04_rb,0.19601e-04_rb,0.21016e-04_rb,0.22306e-04_rb,0.23468e-04_rb /)
      kbo(:,37,12) = (/ &
        & 0.17833e-04_rb,0.19389e-04_rb,0.20824e-04_rb,0.22132e-04_rb,0.23314e-04_rb /)
      kbo(:,38,12) = (/ &
        & 0.17587e-04_rb,0.19167e-04_rb,0.20623e-04_rb,0.21950e-04_rb,0.23152e-04_rb /)
      kbo(:,39,12) = (/ &
        & 0.17343e-04_rb,0.18945e-04_rb,0.20418e-04_rb,0.21765e-04_rb,0.22984e-04_rb /)
      kbo(:,40,12) = (/ &
        & 0.16926e-04_rb,0.18563e-04_rb,0.20065e-04_rb,0.21446e-04_rb,0.22697e-04_rb /)
      kbo(:,41,12) = (/ &
        & 0.16478e-04_rb,0.18152e-04_rb,0.19687e-04_rb,0.21101e-04_rb,0.22388e-04_rb /)
      kbo(:,42,12) = (/ &
        & 0.16023e-04_rb,0.17729e-04_rb,0.19301e-04_rb,0.20748e-04_rb,0.22068e-04_rb /)
      kbo(:,43,12) = (/ &
        & 0.15458e-04_rb,0.17203e-04_rb,0.18821e-04_rb,0.20305e-04_rb,0.21667e-04_rb /)
      kbo(:,44,12) = (/ &
        & 0.14831e-04_rb,0.16623e-04_rb,0.18287e-04_rb,0.19814e-04_rb,0.21219e-04_rb /)
      kbo(:,45,12) = (/ &
        & 0.14192e-04_rb,0.16029e-04_rb,0.17735e-04_rb,0.19308e-04_rb,0.20757e-04_rb /)
      kbo(:,46,12) = (/ &
        & 0.13493e-04_rb,0.15380e-04_rb,0.17134e-04_rb,0.18756e-04_rb,0.20247e-04_rb /)
      kbo(:,47,12) = (/ &
        & 0.12693e-04_rb,0.14634e-04_rb,0.16440e-04_rb,0.18118e-04_rb,0.19661e-04_rb /)
      kbo(:,48,12) = (/ &
        & 0.11878e-04_rb,0.13868e-04_rb,0.15729e-04_rb,0.17458e-04_rb,0.19054e-04_rb /)
      kbo(:,49,12) = (/ &
        & 0.11052e-04_rb,0.13077e-04_rb,0.14993e-04_rb,0.16776e-04_rb,0.18429e-04_rb /)
      kbo(:,50,12) = (/ &
        & 0.10274e-04_rb,0.12322e-04_rb,0.14286e-04_rb,0.16119e-04_rb,0.17820e-04_rb /)
      kbo(:,51,12) = (/ &
        & 0.95165e-05_rb,0.11577e-04_rb,0.13581e-04_rb,0.15463e-04_rb,0.17211e-04_rb /)
      kbo(:,52,12) = (/ &
        & 0.87654e-05_rb,0.10825e-04_rb,0.12859e-04_rb,0.14791e-04_rb,0.16589e-04_rb /)
      kbo(:,53,12) = (/ &
        & 0.80213e-05_rb,0.10071e-04_rb,0.12124e-04_rb,0.14102e-04_rb,0.15949e-04_rb /)
      kbo(:,54,12) = (/ &
        & 0.73473e-05_rb,0.93776e-05_rb,0.11439e-04_rb,0.13452e-04_rb,0.15344e-04_rb /)
      kbo(:,55,12) = (/ &
        & 0.67127e-05_rb,0.87188e-05_rb,0.10780e-04_rb,0.12816e-04_rb,0.14751e-04_rb /)
      kbo(:,56,12) = (/ &
        & 0.60935e-05_rb,0.80668e-05_rb,0.10118e-04_rb,0.12169e-04_rb,0.14148e-04_rb /)
      kbo(:,57,12) = (/ &
        & 0.54908e-05_rb,0.74223e-05_rb,0.94571e-05_rb,0.11517e-04_rb,0.13528e-04_rb /)
      kbo(:,58,12) = (/ &
        & 0.49372e-05_rb,0.68201e-05_rb,0.88324e-05_rb,0.10894e-04_rb,0.12926e-04_rb /)
      kbo(:,59,12) = (/ &
        & 0.47213e-05_rb,0.65842e-05_rb,0.85835e-05_rb,0.10644e-04_rb,0.12683e-04_rb /)
      kbo(:,13,13) = (/ &
        & 0.46820e-04_rb,0.47158e-04_rb,0.47420e-04_rb,0.47688e-04_rb,0.47860e-04_rb /)
      kbo(:,14,13) = (/ &
        & 0.47343e-04_rb,0.47824e-04_rb,0.48275e-04_rb,0.48677e-04_rb,0.48937e-04_rb /)
      kbo(:,15,13) = (/ &
        & 0.47643e-04_rb,0.48345e-04_rb,0.48980e-04_rb,0.49469e-04_rb,0.49813e-04_rb /)
      kbo(:,16,13) = (/ &
        & 0.47844e-04_rb,0.48762e-04_rb,0.49536e-04_rb,0.50120e-04_rb,0.50548e-04_rb /)
      kbo(:,17,13) = (/ &
        & 0.47956e-04_rb,0.49079e-04_rb,0.49982e-04_rb,0.50640e-04_rb,0.51155e-04_rb /)
      kbo(:,18,13) = (/ &
        & 0.48046e-04_rb,0.49334e-04_rb,0.50343e-04_rb,0.51068e-04_rb,0.51673e-04_rb /)
      kbo(:,19,13) = (/ &
        & 0.48132e-04_rb,0.49550e-04_rb,0.50630e-04_rb,0.51453e-04_rb,0.52124e-04_rb /)
      kbo(:,20,13) = (/ &
        & 0.48262e-04_rb,0.49765e-04_rb,0.50907e-04_rb,0.51808e-04_rb,0.52533e-04_rb /)
      kbo(:,21,13) = (/ &
        & 0.48402e-04_rb,0.49959e-04_rb,0.51170e-04_rb,0.52136e-04_rb,0.52894e-04_rb /)
      kbo(:,22,13) = (/ &
        & 0.48671e-04_rb,0.50243e-04_rb,0.51485e-04_rb,0.52481e-04_rb,0.53256e-04_rb /)
      kbo(:,23,13) = (/ &
        & 0.48954e-04_rb,0.50544e-04_rb,0.51806e-04_rb,0.52811e-04_rb,0.53587e-04_rb /)
      kbo(:,24,13) = (/ &
        & 0.49272e-04_rb,0.50858e-04_rb,0.52128e-04_rb,0.53125e-04_rb,0.53892e-04_rb /)
      kbo(:,25,13) = (/ &
        & 0.49616e-04_rb,0.51186e-04_rb,0.52450e-04_rb,0.53425e-04_rb,0.54174e-04_rb /)
      kbo(:,26,13) = (/ &
        & 0.50001e-04_rb,0.51551e-04_rb,0.52773e-04_rb,0.53723e-04_rb,0.54445e-04_rb /)
      kbo(:,27,13) = (/ &
        & 0.50406e-04_rb,0.51904e-04_rb,0.53086e-04_rb,0.54001e-04_rb,0.54687e-04_rb /)
      kbo(:,28,13) = (/ &
        & 0.50802e-04_rb,0.52248e-04_rb,0.53391e-04_rb,0.54260e-04_rb,0.54906e-04_rb /)
      kbo(:,29,13) = (/ &
        & 0.51204e-04_rb,0.52593e-04_rb,0.53683e-04_rb,0.54507e-04_rb,0.55109e-04_rb /)
      kbo(:,30,13) = (/ &
        & 0.51608e-04_rb,0.52928e-04_rb,0.53956e-04_rb,0.54730e-04_rb,0.55285e-04_rb /)
      kbo(:,31,13) = (/ &
        & 0.51993e-04_rb,0.53249e-04_rb,0.54213e-04_rb,0.54939e-04_rb,0.55442e-04_rb /)
      kbo(:,32,13) = (/ &
        & 0.52364e-04_rb,0.53549e-04_rb,0.54461e-04_rb,0.55127e-04_rb,0.55583e-04_rb /)
      kbo(:,33,13) = (/ &
        & 0.52722e-04_rb,0.53841e-04_rb,0.54688e-04_rb,0.55301e-04_rb,0.55705e-04_rb /)
      kbo(:,34,13) = (/ &
        & 0.53018e-04_rb,0.54074e-04_rb,0.54867e-04_rb,0.55434e-04_rb,0.55799e-04_rb /)
      kbo(:,35,13) = (/ &
        & 0.53187e-04_rb,0.54211e-04_rb,0.54980e-04_rb,0.55515e-04_rb,0.55855e-04_rb /)
      kbo(:,36,13) = (/ &
        & 0.53222e-04_rb,0.54242e-04_rb,0.55009e-04_rb,0.55544e-04_rb,0.55878e-04_rb /)
      kbo(:,37,13) = (/ &
        & 0.53077e-04_rb,0.54137e-04_rb,0.54934e-04_rb,0.55495e-04_rb,0.55862e-04_rb /)
      kbo(:,38,13) = (/ &
        & 0.52907e-04_rb,0.54013e-04_rb,0.54850e-04_rb,0.55445e-04_rb,0.55837e-04_rb /)
      kbo(:,39,13) = (/ &
        & 0.52736e-04_rb,0.53884e-04_rb,0.54754e-04_rb,0.55383e-04_rb,0.55805e-04_rb /)
      kbo(:,40,13) = (/ &
        & 0.52415e-04_rb,0.53638e-04_rb,0.54576e-04_rb,0.55258e-04_rb,0.55727e-04_rb /)
      kbo(:,41,13) = (/ &
        & 0.52053e-04_rb,0.53364e-04_rb,0.54368e-04_rb,0.55117e-04_rb,0.55636e-04_rb /)
      kbo(:,42,13) = (/ &
        & 0.51661e-04_rb,0.53061e-04_rb,0.54142e-04_rb,0.54952e-04_rb,0.55527e-04_rb /)
      kbo(:,43,13) = (/ &
        & 0.51150e-04_rb,0.52667e-04_rb,0.53836e-04_rb,0.54732e-04_rb,0.55377e-04_rb /)
      kbo(:,44,13) = (/ &
        & 0.50559e-04_rb,0.52194e-04_rb,0.53480e-04_rb,0.54465e-04_rb,0.55186e-04_rb /)
      kbo(:,45,13) = (/ &
        & 0.49921e-04_rb,0.51684e-04_rb,0.53086e-04_rb,0.54162e-04_rb,0.54971e-04_rb /)
      kbo(:,46,13) = (/ &
        & 0.49197e-04_rb,0.51093e-04_rb,0.52628e-04_rb,0.53809e-04_rb,0.54714e-04_rb /)
      kbo(:,47,13) = (/ &
        & 0.48328e-04_rb,0.50372e-04_rb,0.52058e-04_rb,0.53374e-04_rb,0.54380e-04_rb /)
      kbo(:,48,13) = (/ &
        & 0.47381e-04_rb,0.49603e-04_rb,0.51420e-04_rb,0.52887e-04_rb,0.54013e-04_rb /)
      kbo(:,49,13) = (/ &
        & 0.46342e-04_rb,0.48755e-04_rb,0.50735e-04_rb,0.52347e-04_rb,0.53594e-04_rb /)
      kbo(:,50,13) = (/ &
        & 0.45266e-04_rb,0.47911e-04_rb,0.50033e-04_rb,0.51782e-04_rb,0.53163e-04_rb /)
      kbo(:,51,13) = (/ &
        & 0.44139e-04_rb,0.47021e-04_rb,0.49310e-04_rb,0.51184e-04_rb,0.52699e-04_rb /)
      kbo(:,52,13) = (/ &
        & 0.42930e-04_rb,0.46044e-04_rb,0.48523e-04_rb,0.50540e-04_rb,0.52191e-04_rb /)
      kbo(:,53,13) = (/ &
        & 0.41632e-04_rb,0.44983e-04_rb,0.47688e-04_rb,0.49847e-04_rb,0.51629e-04_rb /)
      kbo(:,54,13) = (/ &
        & 0.40359e-04_rb,0.43928e-04_rb,0.46843e-04_rb,0.49172e-04_rb,0.51075e-04_rb /)
      kbo(:,55,13) = (/ &
        & 0.39074e-04_rb,0.42858e-04_rb,0.45984e-04_rb,0.48480e-04_rb,0.50508e-04_rb /)
      kbo(:,56,13) = (/ &
        & 0.37728e-04_rb,0.41722e-04_rb,0.45056e-04_rb,0.47740e-04_rb,0.49900e-04_rb /)
      kbo(:,57,13) = (/ &
        & 0.36320e-04_rb,0.40515e-04_rb,0.44055e-04_rb,0.46942e-04_rb,0.49258e-04_rb /)
      kbo(:,58,13) = (/ &
        & 0.34927e-04_rb,0.39308e-04_rb,0.43052e-04_rb,0.46141e-04_rb,0.48596e-04_rb /)
      kbo(:,59,13) = (/ &
        & 0.34349e-04_rb,0.38808e-04_rb,0.42631e-04_rb,0.45802e-04_rb,0.48325e-04_rb /)
      kbo(:,13,14) = (/ &
        & 0.91739e-04_rb,0.90588e-04_rb,0.89270e-04_rb,0.87862e-04_rb,0.86541e-04_rb /)
      kbo(:,14,14) = (/ &
        & 0.97111e-04_rb,0.95879e-04_rb,0.94382e-04_rb,0.92799e-04_rb,0.91344e-04_rb /)
      kbo(:,15,14) = (/ &
        & 0.10201e-03_rb,0.10059e-03_rb,0.98963e-04_rb,0.97297e-04_rb,0.95661e-04_rb /)
      kbo(:,16,14) = (/ &
        & 0.10632e-03_rb,0.10477e-03_rb,0.10303e-03_rb,0.10127e-03_rb,0.99489e-04_rb /)
      kbo(:,17,14) = (/ &
        & 0.11011e-03_rb,0.10842e-03_rb,0.10659e-03_rb,0.10474e-03_rb,0.10285e-03_rb /)
      kbo(:,18,14) = (/ &
        & 0.11338e-03_rb,0.11158e-03_rb,0.10968e-03_rb,0.10773e-03_rb,0.10569e-03_rb /)
      kbo(:,19,14) = (/ &
        & 0.11615e-03_rb,0.11428e-03_rb,0.11231e-03_rb,0.11026e-03_rb,0.10810e-03_rb /)
      kbo(:,20,14) = (/ &
        & 0.11847e-03_rb,0.11653e-03_rb,0.11450e-03_rb,0.11235e-03_rb,0.11005e-03_rb /)
      kbo(:,21,14) = (/ &
        & 0.12039e-03_rb,0.11839e-03_rb,0.11628e-03_rb,0.11404e-03_rb,0.11166e-03_rb /)
      kbo(:,22,14) = (/ &
        & 0.12185e-03_rb,0.11978e-03_rb,0.11759e-03_rb,0.11525e-03_rb,0.11275e-03_rb /)
      kbo(:,23,14) = (/ &
        & 0.12298e-03_rb,0.12085e-03_rb,0.11858e-03_rb,0.11615e-03_rb,0.11356e-03_rb /)
      kbo(:,24,14) = (/ &
        & 0.12384e-03_rb,0.12163e-03_rb,0.11927e-03_rb,0.11676e-03_rb,0.11410e-03_rb /)
      kbo(:,25,14) = (/ &
        & 0.12445e-03_rb,0.12216e-03_rb,0.11972e-03_rb,0.11712e-03_rb,0.11440e-03_rb /)
      kbo(:,26,14) = (/ &
        & 0.12480e-03_rb,0.12244e-03_rb,0.11993e-03_rb,0.11725e-03_rb,0.11449e-03_rb /)
      kbo(:,27,14) = (/ &
        & 0.12497e-03_rb,0.12254e-03_rb,0.11997e-03_rb,0.11723e-03_rb,0.11442e-03_rb /)
      kbo(:,28,14) = (/ &
        & 0.12498e-03_rb,0.12249e-03_rb,0.11986e-03_rb,0.11707e-03_rb,0.11425e-03_rb /)
      kbo(:,29,14) = (/ &
        & 0.12486e-03_rb,0.12231e-03_rb,0.11961e-03_rb,0.11677e-03_rb,0.11393e-03_rb /)
      kbo(:,30,14) = (/ &
        & 0.12462e-03_rb,0.12202e-03_rb,0.11925e-03_rb,0.11640e-03_rb,0.11355e-03_rb /)
      kbo(:,31,14) = (/ &
        & 0.12428e-03_rb,0.12163e-03_rb,0.11882e-03_rb,0.11594e-03_rb,0.11310e-03_rb /)
      kbo(:,32,14) = (/ &
        & 0.12387e-03_rb,0.12116e-03_rb,0.11831e-03_rb,0.11542e-03_rb,0.11260e-03_rb /)
      kbo(:,33,14) = (/ &
        & 0.12338e-03_rb,0.12061e-03_rb,0.11773e-03_rb,0.11485e-03_rb,0.11205e-03_rb /)
      kbo(:,34,14) = (/ &
        & 0.12293e-03_rb,0.12013e-03_rb,0.11722e-03_rb,0.11434e-03_rb,0.11156e-03_rb /)
      kbo(:,35,14) = (/ &
        & 0.12271e-03_rb,0.11987e-03_rb,0.11696e-03_rb,0.11408e-03_rb,0.11132e-03_rb /)
      kbo(:,36,14) = (/ &
        & 0.12276e-03_rb,0.11992e-03_rb,0.11700e-03_rb,0.11412e-03_rb,0.11135e-03_rb /)
      kbo(:,37,14) = (/ &
        & 0.12321e-03_rb,0.12039e-03_rb,0.11748e-03_rb,0.11457e-03_rb,0.11178e-03_rb /)
      kbo(:,38,14) = (/ &
        & 0.12368e-03_rb,0.12086e-03_rb,0.11795e-03_rb,0.11504e-03_rb,0.11223e-03_rb /)
      kbo(:,39,14) = (/ &
        & 0.12412e-03_rb,0.12132e-03_rb,0.11841e-03_rb,0.11549e-03_rb,0.11266e-03_rb /)
      kbo(:,40,14) = (/ &
        & 0.12481e-03_rb,0.12205e-03_rb,0.11917e-03_rb,0.11624e-03_rb,0.11338e-03_rb /)
      kbo(:,41,14) = (/ &
        & 0.12552e-03_rb,0.12282e-03_rb,0.11995e-03_rb,0.11701e-03_rb,0.11413e-03_rb /)
      kbo(:,42,14) = (/ &
        & 0.12624e-03_rb,0.12356e-03_rb,0.12073e-03_rb,0.11779e-03_rb,0.11489e-03_rb /)
      kbo(:,43,14) = (/ &
        & 0.12708e-03_rb,0.12444e-03_rb,0.12166e-03_rb,0.11873e-03_rb,0.11581e-03_rb /)
      kbo(:,44,14) = (/ &
        & 0.12796e-03_rb,0.12537e-03_rb,0.12263e-03_rb,0.11974e-03_rb,0.11681e-03_rb /)
      kbo(:,45,14) = (/ &
        & 0.12882e-03_rb,0.12627e-03_rb,0.12360e-03_rb,0.12075e-03_rb,0.11782e-03_rb /)
      kbo(:,46,14) = (/ &
        & 0.12968e-03_rb,0.12723e-03_rb,0.12459e-03_rb,0.12181e-03_rb,0.11889e-03_rb /)
      kbo(:,47,14) = (/ &
        & 0.13062e-03_rb,0.12827e-03_rb,0.12567e-03_rb,0.12295e-03_rb,0.12008e-03_rb /)
      kbo(:,48,14) = (/ &
        & 0.13150e-03_rb,0.12926e-03_rb,0.12675e-03_rb,0.12408e-03_rb,0.12126e-03_rb /)
      kbo(:,49,14) = (/ &
        & 0.13233e-03_rb,0.13021e-03_rb,0.12780e-03_rb,0.12517e-03_rb,0.12242e-03_rb /)
      kbo(:,50,14) = (/ &
        & 0.13307e-03_rb,0.13105e-03_rb,0.12874e-03_rb,0.12618e-03_rb,0.12349e-03_rb /)
      kbo(:,51,14) = (/ &
        & 0.13374e-03_rb,0.13183e-03_rb,0.12963e-03_rb,0.12715e-03_rb,0.12449e-03_rb /)
      kbo(:,52,14) = (/ &
        & 0.13431e-03_rb,0.13255e-03_rb,0.13046e-03_rb,0.12807e-03_rb,0.12548e-03_rb /)
      kbo(:,53,14) = (/ &
        & 0.13479e-03_rb,0.13325e-03_rb,0.13127e-03_rb,0.12898e-03_rb,0.12645e-03_rb /)
      kbo(:,54,14) = (/ &
        & 0.13511e-03_rb,0.13385e-03_rb,0.13196e-03_rb,0.12979e-03_rb,0.12732e-03_rb /)
      kbo(:,55,14) = (/ &
        & 0.13533e-03_rb,0.13436e-03_rb,0.13261e-03_rb,0.13052e-03_rb,0.12814e-03_rb /)
      kbo(:,56,14) = (/ &
        & 0.13543e-03_rb,0.13477e-03_rb,0.13321e-03_rb,0.13121e-03_rb,0.12893e-03_rb /)
      kbo(:,57,14) = (/ &
        & 0.13540e-03_rb,0.13509e-03_rb,0.13379e-03_rb,0.13190e-03_rb,0.12969e-03_rb /)
      kbo(:,58,14) = (/ &
        & 0.13520e-03_rb,0.13532e-03_rb,0.13428e-03_rb,0.13250e-03_rb,0.13039e-03_rb /)
      kbo(:,59,14) = (/ &
        & 0.13510e-03_rb,0.13536e-03_rb,0.13445e-03_rb,0.13273e-03_rb,0.13066e-03_rb /)
      kbo(:,13,15) = (/ &
        & 0.16545e-03_rb,0.16076e-03_rb,0.15662e-03_rb,0.15273e-03_rb,0.14899e-03_rb /)
      kbo(:,14,15) = (/ &
        & 0.17968e-03_rb,0.17402e-03_rb,0.16899e-03_rb,0.16434e-03_rb,0.15984e-03_rb /)
      kbo(:,15,15) = (/ &
        & 0.19299e-03_rb,0.18634e-03_rb,0.18041e-03_rb,0.17491e-03_rb,0.16965e-03_rb /)
      kbo(:,16,15) = (/ &
        & 0.20519e-03_rb,0.19756e-03_rb,0.19072e-03_rb,0.18435e-03_rb,0.17834e-03_rb /)
      kbo(:,17,15) = (/ &
        & 0.21616e-03_rb,0.20753e-03_rb,0.19981e-03_rb,0.19268e-03_rb,0.18599e-03_rb /)
      kbo(:,18,15) = (/ &
        & 0.22578e-03_rb,0.21627e-03_rb,0.20768e-03_rb,0.19983e-03_rb,0.19254e-03_rb /)
      kbo(:,19,15) = (/ &
        & 0.23404e-03_rb,0.22371e-03_rb,0.21439e-03_rb,0.20593e-03_rb,0.19813e-03_rb /)
      kbo(:,20,15) = (/ &
        & 0.24079e-03_rb,0.22975e-03_rb,0.21986e-03_rb,0.21084e-03_rb,0.20261e-03_rb /)
      kbo(:,21,15) = (/ &
        & 0.24629e-03_rb,0.23467e-03_rb,0.22423e-03_rb,0.21475e-03_rb,0.20622e-03_rb /)
      kbo(:,22,15) = (/ &
        & 0.24979e-03_rb,0.23775e-03_rb,0.22702e-03_rb,0.21724e-03_rb,0.20849e-03_rb /)
      kbo(:,23,15) = (/ &
        & 0.25216e-03_rb,0.23989e-03_rb,0.22890e-03_rb,0.21892e-03_rb,0.21005e-03_rb /)
      kbo(:,24,15) = (/ &
        & 0.25355e-03_rb,0.24109e-03_rb,0.22997e-03_rb,0.21990e-03_rb,0.21094e-03_rb /)
      kbo(:,25,15) = (/ &
        & 0.25401e-03_rb,0.24151e-03_rb,0.23030e-03_rb,0.22024e-03_rb,0.21125e-03_rb /)
      kbo(:,26,15) = (/ &
        & 0.25359e-03_rb,0.24114e-03_rb,0.22993e-03_rb,0.21995e-03_rb,0.21097e-03_rb /)
      kbo(:,27,15) = (/ &
        & 0.25262e-03_rb,0.24030e-03_rb,0.22913e-03_rb,0.21926e-03_rb,0.21037e-03_rb /)
      kbo(:,28,15) = (/ &
        & 0.25126e-03_rb,0.23904e-03_rb,0.22801e-03_rb,0.21831e-03_rb,0.20948e-03_rb /)
      kbo(:,29,15) = (/ &
        & 0.24948e-03_rb,0.23745e-03_rb,0.22663e-03_rb,0.21704e-03_rb,0.20834e-03_rb /)
      kbo(:,30,15) = (/ &
        & 0.24748e-03_rb,0.23558e-03_rb,0.22502e-03_rb,0.21558e-03_rb,0.20699e-03_rb /)
      kbo(:,31,15) = (/ &
        & 0.24521e-03_rb,0.23353e-03_rb,0.22323e-03_rb,0.21394e-03_rb,0.20549e-03_rb /)
      kbo(:,32,15) = (/ &
        & 0.24273e-03_rb,0.23132e-03_rb,0.22129e-03_rb,0.21221e-03_rb,0.20387e-03_rb /)
      kbo(:,33,15) = (/ &
        & 0.24020e-03_rb,0.22911e-03_rb,0.21929e-03_rb,0.21039e-03_rb,0.20217e-03_rb /)
      kbo(:,34,15) = (/ &
        & 0.23799e-03_rb,0.22719e-03_rb,0.21754e-03_rb,0.20880e-03_rb,0.20071e-03_rb /)
      kbo(:,35,15) = (/ &
        & 0.23687e-03_rb,0.22619e-03_rb,0.21668e-03_rb,0.20798e-03_rb,0.19994e-03_rb /)
      kbo(:,36,15) = (/ &
        & 0.23693e-03_rb,0.22624e-03_rb,0.21673e-03_rb,0.20802e-03_rb,0.19997e-03_rb /)
      kbo(:,37,15) = (/ &
        & 0.23866e-03_rb,0.22780e-03_rb,0.21810e-03_rb,0.20930e-03_rb,0.20116e-03_rb /)
      kbo(:,38,15) = (/ &
        & 0.24049e-03_rb,0.22941e-03_rb,0.21958e-03_rb,0.21064e-03_rb,0.20240e-03_rb /)
      kbo(:,39,15) = (/ &
        & 0.24235e-03_rb,0.23100e-03_rb,0.22101e-03_rb,0.21195e-03_rb,0.20362e-03_rb /)
      kbo(:,40,15) = (/ &
        & 0.24544e-03_rb,0.23374e-03_rb,0.22344e-03_rb,0.21416e-03_rb,0.20567e-03_rb /)
      kbo(:,41,15) = (/ &
        & 0.24874e-03_rb,0.23671e-03_rb,0.22606e-03_rb,0.21653e-03_rb,0.20786e-03_rb /)
      kbo(:,42,15) = (/ &
        & 0.25214e-03_rb,0.23974e-03_rb,0.22874e-03_rb,0.21892e-03_rb,0.21007e-03_rb /)
      kbo(:,43,15) = (/ &
        & 0.25633e-03_rb,0.24355e-03_rb,0.23207e-03_rb,0.22195e-03_rb,0.21281e-03_rb /)
      kbo(:,44,15) = (/ &
        & 0.26108e-03_rb,0.24779e-03_rb,0.23584e-03_rb,0.22529e-03_rb,0.21581e-03_rb /)
      kbo(:,45,15) = (/ &
        & 0.26596e-03_rb,0.25216e-03_rb,0.23977e-03_rb,0.22875e-03_rb,0.21897e-03_rb /)
      kbo(:,46,15) = (/ &
        & 0.27137e-03_rb,0.25699e-03_rb,0.24413e-03_rb,0.23258e-03_rb,0.22240e-03_rb /)
      kbo(:,47,15) = (/ &
        & 0.27767e-03_rb,0.26264e-03_rb,0.24919e-03_rb,0.23707e-03_rb,0.22639e-03_rb /)
      kbo(:,48,15) = (/ &
        & 0.28422e-03_rb,0.26851e-03_rb,0.25443e-03_rb,0.24179e-03_rb,0.23054e-03_rb /)
      kbo(:,49,15) = (/ &
        & 0.29109e-03_rb,0.27465e-03_rb,0.25993e-03_rb,0.24676e-03_rb,0.23489e-03_rb /)
      kbo(:,50,15) = (/ &
        & 0.29784e-03_rb,0.28066e-03_rb,0.26532e-03_rb,0.25156e-03_rb,0.23921e-03_rb /)
      kbo(:,51,15) = (/ &
        & 0.30464e-03_rb,0.28674e-03_rb,0.27072e-03_rb,0.25642e-03_rb,0.24358e-03_rb /)
      kbo(:,52,15) = (/ &
        & 0.31175e-03_rb,0.29305e-03_rb,0.27640e-03_rb,0.26149e-03_rb,0.24813e-03_rb /)
      kbo(:,53,15) = (/ &
        & 0.31921e-03_rb,0.29964e-03_rb,0.28229e-03_rb,0.26674e-03_rb,0.25282e-03_rb /)
      kbo(:,54,15) = (/ &
        & 0.32632e-03_rb,0.30596e-03_rb,0.28789e-03_rb,0.27174e-03_rb,0.25731e-03_rb /)
      kbo(:,55,15) = (/ &
        & 0.33342e-03_rb,0.31218e-03_rb,0.29344e-03_rb,0.27674e-03_rb,0.26179e-03_rb /)
      kbo(:,56,15) = (/ &
        & 0.34053e-03_rb,0.31870e-03_rb,0.29920e-03_rb,0.28191e-03_rb,0.26637e-03_rb /)
      kbo(:,57,15) = (/ &
        & 0.34801e-03_rb,0.32554e-03_rb,0.30518e-03_rb,0.28726e-03_rb,0.27114e-03_rb /)
      kbo(:,58,15) = (/ &
        & 0.35537e-03_rb,0.33214e-03_rb,0.31109e-03_rb,0.29248e-03_rb,0.27588e-03_rb /)
      kbo(:,59,15) = (/ &
        & 0.35841e-03_rb,0.33487e-03_rb,0.31352e-03_rb,0.29463e-03_rb,0.27781e-03_rb /)
      kbo(:,13,16) = (/ &
        & 0.23817e-03_rb,0.23193e-03_rb,0.22589e-03_rb,0.21979e-03_rb,0.21365e-03_rb /)
      kbo(:,14,16) = (/ &
        & 0.26266e-03_rb,0.25442e-03_rb,0.24663e-03_rb,0.23897e-03_rb,0.23152e-03_rb /)
      kbo(:,15,16) = (/ &
        & 0.28586e-03_rb,0.27559e-03_rb,0.26587e-03_rb,0.25663e-03_rb,0.24800e-03_rb /)
      kbo(:,16,16) = (/ &
        & 0.30726e-03_rb,0.29496e-03_rb,0.28337e-03_rb,0.27251e-03_rb,0.26278e-03_rb /)
      kbo(:,17,16) = (/ &
        & 0.32665e-03_rb,0.31224e-03_rb,0.29895e-03_rb,0.28665e-03_rb,0.27581e-03_rb /)
      kbo(:,18,16) = (/ &
        & 0.34367e-03_rb,0.32749e-03_rb,0.31249e-03_rb,0.29896e-03_rb,0.28707e-03_rb /)
      kbo(:,19,16) = (/ &
        & 0.35833e-03_rb,0.34046e-03_rb,0.32413e-03_rb,0.30949e-03_rb,0.29662e-03_rb /)
      kbo(:,20,16) = (/ &
        & 0.37032e-03_rb,0.35107e-03_rb,0.33355e-03_rb,0.31795e-03_rb,0.30443e-03_rb /)
      kbo(:,21,16) = (/ &
        & 0.38004e-03_rb,0.35962e-03_rb,0.34106e-03_rb,0.32482e-03_rb,0.31071e-03_rb /)
      kbo(:,22,16) = (/ &
        & 0.38628e-03_rb,0.36502e-03_rb,0.34582e-03_rb,0.32929e-03_rb,0.31473e-03_rb /)
      kbo(:,23,16) = (/ &
        & 0.39046e-03_rb,0.36870e-03_rb,0.34902e-03_rb,0.33239e-03_rb,0.31753e-03_rb /)
      kbo(:,24,16) = (/ &
        & 0.39290e-03_rb,0.37079e-03_rb,0.35085e-03_rb,0.33417e-03_rb,0.31909e-03_rb /)
      kbo(:,25,16) = (/ &
        & 0.39368e-03_rb,0.37132e-03_rb,0.35155e-03_rb,0.33489e-03_rb,0.31974e-03_rb /)
      kbo(:,26,16) = (/ &
        & 0.39288e-03_rb,0.37052e-03_rb,0.35111e-03_rb,0.33452e-03_rb,0.31938e-03_rb /)
      kbo(:,27,16) = (/ &
        & 0.39120e-03_rb,0.36896e-03_rb,0.34999e-03_rb,0.33347e-03_rb,0.31836e-03_rb /)
      kbo(:,28,16) = (/ &
        & 0.38862e-03_rb,0.36669e-03_rb,0.34828e-03_rb,0.33189e-03_rb,0.31693e-03_rb /)
      kbo(:,29,16) = (/ &
        & 0.38540e-03_rb,0.36396e-03_rb,0.34596e-03_rb,0.32975e-03_rb,0.31502e-03_rb /)
      kbo(:,30,16) = (/ &
        & 0.38165e-03_rb,0.36095e-03_rb,0.34333e-03_rb,0.32738e-03_rb,0.31277e-03_rb /)
      kbo(:,31,16) = (/ &
        & 0.37756e-03_rb,0.35762e-03_rb,0.34034e-03_rb,0.32467e-03_rb,0.31030e-03_rb /)
      kbo(:,32,16) = (/ &
        & 0.37323e-03_rb,0.35410e-03_rb,0.33709e-03_rb,0.32164e-03_rb,0.30764e-03_rb /)
      kbo(:,33,16) = (/ &
        & 0.36881e-03_rb,0.35033e-03_rb,0.33368e-03_rb,0.31855e-03_rb,0.30481e-03_rb /)
      kbo(:,34,16) = (/ &
        & 0.36518e-03_rb,0.34710e-03_rb,0.33080e-03_rb,0.31589e-03_rb,0.30241e-03_rb /)
      kbo(:,35,16) = (/ &
        & 0.36331e-03_rb,0.34548e-03_rb,0.32924e-03_rb,0.31455e-03_rb,0.30119e-03_rb /)
      kbo(:,36,16) = (/ &
        & 0.36345e-03_rb,0.34559e-03_rb,0.32934e-03_rb,0.31463e-03_rb,0.30125e-03_rb /)
      kbo(:,37,16) = (/ &
        & 0.36636e-03_rb,0.34821e-03_rb,0.33175e-03_rb,0.31676e-03_rb,0.30324e-03_rb /)
      kbo(:,38,16) = (/ &
        & 0.36948e-03_rb,0.35092e-03_rb,0.33423e-03_rb,0.31902e-03_rb,0.30527e-03_rb /)
      kbo(:,39,16) = (/ &
        & 0.37259e-03_rb,0.35368e-03_rb,0.33668e-03_rb,0.32128e-03_rb,0.30727e-03_rb /)
      kbo(:,40,16) = (/ &
        & 0.37795e-03_rb,0.35829e-03_rb,0.34085e-03_rb,0.32505e-03_rb,0.31069e-03_rb /)
      kbo(:,41,16) = (/ &
        & 0.38379e-03_rb,0.36312e-03_rb,0.34529e-03_rb,0.32908e-03_rb,0.31437e-03_rb /)
      kbo(:,42,16) = (/ &
        & 0.38990e-03_rb,0.36819e-03_rb,0.34980e-03_rb,0.33322e-03_rb,0.31806e-03_rb /)
      kbo(:,43,16) = (/ &
        & 0.39755e-03_rb,0.37469e-03_rb,0.35548e-03_rb,0.33833e-03_rb,0.32274e-03_rb /)
      kbo(:,44,16) = (/ &
        & 0.40604e-03_rb,0.38206e-03_rb,0.36178e-03_rb,0.34404e-03_rb,0.32793e-03_rb /)
      kbo(:,45,16) = (/ &
        & 0.41497e-03_rb,0.38998e-03_rb,0.36828e-03_rb,0.34986e-03_rb,0.33329e-03_rb /)
      kbo(:,46,16) = (/ &
        & 0.42497e-03_rb,0.39871e-03_rb,0.37566e-03_rb,0.35633e-03_rb,0.33909e-03_rb /)
      kbo(:,47,16) = (/ &
        & 0.43684e-03_rb,0.40880e-03_rb,0.38451e-03_rb,0.36380e-03_rb,0.34586e-03_rb /)
      kbo(:,48,16) = (/ &
        & 0.44932e-03_rb,0.41960e-03_rb,0.39404e-03_rb,0.37171e-03_rb,0.35286e-03_rb /)
      kbo(:,49,16) = (/ &
        & 0.46270e-03_rb,0.43113e-03_rb,0.40397e-03_rb,0.38027e-03_rb,0.36021e-03_rb /)
      kbo(:,50,16) = (/ &
        & 0.47605e-03_rb,0.44249e-03_rb,0.41370e-03_rb,0.38887e-03_rb,0.36733e-03_rb /)
      kbo(:,51,16) = (/ &
        & 0.48981e-03_rb,0.45414e-03_rb,0.42376e-03_rb,0.39765e-03_rb,0.37477e-03_rb /)
      kbo(:,52,16) = (/ &
        & 0.50430e-03_rb,0.46661e-03_rb,0.43437e-03_rb,0.40673e-03_rb,0.38268e-03_rb /)
      kbo(:,53,16) = (/ &
        & 0.51980e-03_rb,0.47970e-03_rb,0.44558e-03_rb,0.41639e-03_rb,0.39115e-03_rb /)
      kbo(:,54,16) = (/ &
        & 0.53506e-03_rb,0.49244e-03_rb,0.45647e-03_rb,0.42564e-03_rb,0.39926e-03_rb /)
      kbo(:,55,16) = (/ &
        & 0.55131e-03_rb,0.50528e-03_rb,0.46739e-03_rb,0.43505e-03_rb,0.40730e-03_rb /)
      kbo(:,56,16) = (/ &
        & 0.56909e-03_rb,0.51883e-03_rb,0.47884e-03_rb,0.44497e-03_rb,0.41571e-03_rb /)
      kbo(:,57,16) = (/ &
        & 0.58824e-03_rb,0.53334e-03_rb,0.49097e-03_rb,0.45523e-03_rb,0.42457e-03_rb /)
      kbo(:,58,16) = (/ &
        & 0.60745e-03_rb,0.54837e-03_rb,0.50301e-03_rb,0.46546e-03_rb,0.43352e-03_rb /)
      kbo(:,59,16) = (/ &
        & 0.61557e-03_rb,0.55493e-03_rb,0.50805e-03_rb,0.46962e-03_rb,0.43716e-03_rb /)

!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.351362e-07_rb, 0.341136e-07_rb, 0.181317e-06_rb /)
      forrefo(:, 2) = (/ 0.109648e-06_rb, 0.344240e-06_rb, 0.139709e-05_rb /)
      forrefo(:, 3) = (/ 0.374823e-06_rb, 0.103424e-05_rb, 0.188717e-05_rb /)
      forrefo(:, 4) = (/ 0.580041e-06_rb, 0.116876e-05_rb, 0.121183e-05_rb /)
      forrefo(:, 5) = (/ 0.115608e-05_rb, 0.148110e-05_rb, 0.836083e-06_rb /)
      forrefo(:, 6) = (/ 0.181460e-05_rb, 0.133313e-05_rb, 0.500167e-06_rb /)
      forrefo(:, 7) = (/ 0.199096e-05_rb, 0.115276e-05_rb, 0.432994e-06_rb /)
      forrefo(:, 8) = (/ 0.183730e-05_rb, 0.122260e-05_rb, 0.433248e-06_rb /)
      forrefo(:, 9) = (/ 0.198386e-05_rb, 0.100130e-05_rb, 0.269712e-06_rb /)
      forrefo(:,10) = (/ 0.276382e-05_rb, 0.749215e-06_rb, 0.236919e-06_rb /)
      forrefo(:,11) = (/ 0.298202e-05_rb, 0.629688e-06_rb, 0.228388e-06_rb /)
      forrefo(:,12) = (/ 0.364604e-05_rb, 0.455336e-06_rb, 0.206130e-06_rb /)
      forrefo(:,13) = (/ 0.373339e-05_rb, 0.245210e-06_rb, 0.201987e-06_rb /)
      forrefo(:,14) = (/ 0.480378e-05_rb, 0.177591e-06_rb, 0.171458e-06_rb /)
      forrefo(:,15) = (/ 0.521700e-05_rb, 0.203358e-06_rb, 0.189559e-06_rb /)
      forrefo(:,16) = (/ 0.542717e-05_rb, 0.219022e-06_rb, 0.218271e-06_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
        &  0.538526e-04_rb, 0.464603e-04_rb, 0.400828e-04_rb, 0.345807e-04_rb, 0.298339e-04_rb, &
        &  0.257386e-04_rb, 0.222055e-04_rb, 0.191574e-04_rb, 0.165277e-04_rb, 0.142590e-04_rb /)
      selfrefo(:, 2) = (/ &
        &  0.162409e-03_rb, 0.128347e-03_rb, 0.101430e-03_rb, 0.801571e-04_rb, 0.633460e-04_rb, &
        &  0.500607e-04_rb, 0.395616e-04_rb, 0.312645e-04_rb, 0.247075e-04_rb, 0.195257e-04_rb /)
      selfrefo(:, 3) = (/ &
        &  0.262882e-03_rb, 0.212793e-03_rb, 0.172247e-03_rb, 0.139427e-03_rb, 0.112860e-03_rb, &
        &  0.913557e-04_rb, 0.739487e-04_rb, 0.598584e-04_rb, 0.484529e-04_rb, 0.392206e-04_rb /)
      selfrefo(:, 4) = (/ &
        &  0.242873e-03_rb, 0.204225e-03_rb, 0.171726e-03_rb, 0.144399e-03_rb, 0.121421e-03_rb, &
        &  0.102099e-03_rb, 0.858516e-04_rb, 0.721899e-04_rb, 0.607022e-04_rb, 0.510426e-04_rb /)
      selfrefo(:, 5) = (/ &
        &  0.235614e-03_rb, 0.207814e-03_rb, 0.183293e-03_rb, 0.161666e-03_rb, 0.142591e-03_rb, &
        &  0.125766e-03_rb, 0.110927e-03_rb, 0.978381e-04_rb, 0.862939e-04_rb, 0.761119e-04_rb /)
      selfrefo(:, 6) = (/ &
        &  0.205508e-03_rb, 0.190174e-03_rb, 0.175985e-03_rb, 0.162854e-03_rb, 0.150702e-03_rb, &
        &  0.139458e-03_rb, 0.129052e-03_rb, 0.119423e-03_rb, 0.110513e-03_rb, 0.102267e-03_rb /)
      selfrefo(:, 7) = (/ &
        &  0.185027e-03_rb, 0.175148e-03_rb, 0.165796e-03_rb, 0.156944e-03_rb, 0.148565e-03_rb, &
        &  0.140633e-03_rb, 0.133124e-03_rb, 0.126016e-03_rb, 0.119288e-03_rb, 0.112919e-03_rb /)
      selfrefo(:, 8) = (/ &
        &  0.192634e-03_rb, 0.180192e-03_rb, 0.168554e-03_rb, 0.157668e-03_rb, 0.147484e-03_rb, &
        &  0.137959e-03_rb, 0.129048e-03_rb, 0.120713e-03_rb, 0.112917e-03_rb, 0.105624e-03_rb /)
      selfrefo(:, 9) = (/ &
        &  0.161632e-03_rb, 0.155919e-03_rb, 0.150408e-03_rb, 0.145092e-03_rb, 0.139963e-03_rb, &
        &  0.135016e-03_rb, 0.130244e-03_rb, 0.125640e-03_rb, 0.121199e-03_rb, 0.116915e-03_rb /)
      selfrefo(:,10) = (/ &
        &  0.120880e-03_rb, 0.125265e-03_rb, 0.129810e-03_rb, 0.134520e-03_rb, 0.139400e-03_rb, &
        &  0.144458e-03_rb, 0.149699e-03_rb, 0.155130e-03_rb, 0.160758e-03_rb, 0.166591e-03_rb /)
      selfrefo(:,11) = (/ &
        &  0.104705e-03_rb, 0.111761e-03_rb, 0.119291e-03_rb, 0.127330e-03_rb, 0.135910e-03_rb, &
        &  0.145068e-03_rb, 0.154843e-03_rb, 0.165277e-03_rb, 0.176414e-03_rb, 0.188302e-03_rb /)
      selfrefo(:,12) = (/ &
        &  0.846335e-04_rb, 0.951236e-04_rb, 0.106914e-03_rb, 0.120166e-03_rb, 0.135060e-03_rb, &
        &  0.151800e-03_rb, 0.170616e-03_rb, 0.191763e-03_rb, 0.215532e-03_rb, 0.242246e-03_rb /)
      selfrefo(:,13) = (/ &
        &  0.669754e-04_rb, 0.781902e-04_rb, 0.912829e-04_rb, 0.106568e-03_rb, 0.124413e-03_rb, &
        &  0.145245e-03_rb, 0.169566e-03_rb, 0.197959e-03_rb, 0.231107e-03_rb, 0.269805e-03_rb /)
      selfrefo(:,14) = (/ &
        &  0.597091e-04_rb, 0.722265e-04_rb, 0.873679e-04_rb, 0.105684e-03_rb, 0.127839e-03_rb, &
        &  0.154639e-03_rb, 0.187057e-03_rb, 0.226272e-03_rb, 0.273707e-03_rb, 0.331087e-03_rb /)
      selfrefo(:,15) = (/ &
        &  0.640410e-04_rb, 0.771879e-04_rb, 0.930338e-04_rb, 0.112133e-03_rb, 0.135152e-03_rb, &
        &  0.162897e-03_rb, 0.196338e-03_rb, 0.236644e-03_rb, 0.285225e-03_rb, 0.343778e-03_rb /)
      selfrefo(:,16) = (/ &
        &  0.666420e-04_rb, 0.801056e-04_rb, 0.962892e-04_rb, 0.115742e-03_rb, 0.139126e-03_rb, &
        &  0.167233e-03_rb, 0.201019e-03_rb, 0.241630e-03_rb, 0.290446e-03_rb, 0.349125e-03_rb /)
     
      end subroutine sw_kgb22

! **************************************************************************
      subroutine sw_kgb23
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg23, only : kao, selfrefo, forrefo, sfluxrefo, raylo, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:) = (/ &
        & 53.2101_rb , 51.4143_rb, 49.3348_rb, 45.4612_rb    , &
        & 40.8294_rb , 35.1801_rb, 28.6947_rb, 21.5751_rb    , &
        & 14.6388_rb , 1.59111_rb, 1.31860_rb, 1.04018_rb    , &
        & 0.762140_rb,0.484214_rb,0.182275_rb, 2.54948e-02_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:) = (/ &
        & 5.29476E+01_rb, 5.09485E+01_rb, 4.87661E+01_rb, 4.53147E+01_rb,&
        & 4.06945E+01_rb, 3.51934E+01_rb, 2.88227E+01_rb, 2.17136E+01_rb,&
        & 1.48095E+01_rb, 1.62016E+00_rb, 1.33844E+00_rb, 1.05820E+00_rb,&
        & 7.76279E-01_rb, 4.94502E-01_rb, 1.85651E-01_rb, 2.60357E-02_rb/)
! Solar variability components: facular brightening
      facbrghto(:) = (/ &
        & 3.10485E-02_rb, 2.99823E-02_rb, 2.84716E-02_rb, 2.62066E-02_rb,&
        & 2.35032E-02_rb, 2.00740E-02_rb, 1.61883E-02_rb, 1.21762E-02_rb,&
        & 8.29774E-03_rb, 8.85211E-04_rb, 7.20532E-04_rb, 5.69280E-04_rb,&
        & 4.11804E-04_rb, 2.60757E-04_rb, 9.33791E-05_rb, 1.31117E-05_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:) = (/ &
        &-1.47311E-02_rb,-1.45718E-02_rb,-1.38484E-02_rb,-1.26884E-02_rb,&
        &-1.12738E-02_rb,-9.58404E-03_rb,-7.70597E-03_rb,-5.77752E-03_rb,&
        &-3.91308E-03_rb,-4.18780E-04_rb,-3.44599E-04_rb,-2.72218E-04_rb,&
        &-1.98944E-04_rb,-1.26261E-04_rb,-4.69964E-05_rb,-6.59469E-06_rb/)

! Rayleigh extinction coefficient at all v 
      raylo(:) = (/ &
        & 5.94837e-08_rb,5.70593e-08_rb,6.27845e-08_rb,5.56602e-08_rb, &
        & 5.25571e-08_rb,4.73388e-08_rb,4.17466e-08_rb,3.98097e-08_rb, &
        & 4.00786e-08_rb,3.67478e-08_rb,3.45186e-08_rb,3.46156e-08_rb, &
        & 3.32155e-08_rb,3.23642e-08_rb,2.72590e-08_rb,2.96813e-08_rb /)

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1) = (/ &
        & 0.33078e-07_rb,0.34034e-07_rb,0.35124e-07_rb,0.34187e-07_rb,0.34744e-07_rb /)
      kao(:, 2, 1) = (/ &
        & 0.25544e-07_rb,0.25873e-07_rb,0.26742e-07_rb,0.27512e-07_rb,0.27504e-07_rb /)
      kao(:, 3, 1) = (/ &
        & 0.18549e-07_rb,0.19611e-07_rb,0.20840e-07_rb,0.22548e-07_rb,0.23069e-07_rb /)
      kao(:, 4, 1) = (/ &
        & 0.28794e-07_rb,0.30837e-07_rb,0.32679e-07_rb,0.34307e-07_rb,0.36901e-07_rb /)
      kao(:, 5, 1) = (/ &
        & 0.36776e-07_rb,0.39144e-07_rb,0.41300e-07_rb,0.43264e-07_rb,0.46626e-07_rb /)
      kao(:, 6, 1) = (/ &
        & 0.59710e-07_rb,0.62941e-07_rb,0.65500e-07_rb,0.67353e-07_rb,0.68774e-07_rb /)
      kao(:, 7, 1) = (/ &
        & 0.12143e-06_rb,0.12932e-06_rb,0.13250e-06_rb,0.13526e-06_rb,0.13849e-06_rb /)
      kao(:, 8, 1) = (/ &
        & 0.12531e-06_rb,0.13241e-06_rb,0.13939e-06_rb,0.14705e-06_rb,0.15465e-06_rb /)
      kao(:, 9, 1) = (/ &
        & 0.20209e-06_rb,0.21134e-06_rb,0.22163e-06_rb,0.23098e-06_rb,0.24004e-06_rb /)
      kao(:,10, 1) = (/ &
        & 0.10750e-05_rb,0.11204e-05_rb,0.11575e-05_rb,0.11923e-05_rb,0.12227e-05_rb /)
      kao(:,11, 1) = (/ &
        & 0.27782e-05_rb,0.28204e-05_rb,0.28406e-05_rb,0.28380e-05_rb,0.28440e-05_rb /)
      kao(:,12, 1) = (/ &
        & 0.38510e-05_rb,0.39934e-05_rb,0.40697e-05_rb,0.41102e-05_rb,0.41571e-05_rb /)
      kao(:,13, 1) = (/ &
        & 0.43157e-05_rb,0.44488e-05_rb,0.45799e-05_rb,0.46585e-05_rb,0.47223e-05_rb /)
      kao(:, 1, 2) = (/ &
        & 0.84637e-06_rb,0.86989e-06_rb,0.90697e-06_rb,0.90000e-06_rb,0.91373e-06_rb /)
      kao(:, 2, 2) = (/ &
        & 0.67062e-06_rb,0.68649e-06_rb,0.72334e-06_rb,0.73645e-06_rb,0.72978e-06_rb /)
      kao(:, 3, 2) = (/ &
        & 0.52317e-06_rb,0.53924e-06_rb,0.55425e-06_rb,0.58347e-06_rb,0.57813e-06_rb /)
      kao(:, 4, 2) = (/ &
        & 0.39868e-06_rb,0.41431e-06_rb,0.42761e-06_rb,0.43892e-06_rb,0.45982e-06_rb /)
      kao(:, 5, 2) = (/ &
        & 0.32074e-06_rb,0.33452e-06_rb,0.34754e-06_rb,0.35582e-06_rb,0.37378e-06_rb /)
      kao(:, 6, 2) = (/ &
        & 0.42465e-06_rb,0.44058e-06_rb,0.45605e-06_rb,0.47192e-06_rb,0.48493e-06_rb /)
      kao(:, 7, 2) = (/ &
        & 0.47581e-06_rb,0.50000e-06_rb,0.52487e-06_rb,0.54192e-06_rb,0.55955e-06_rb /)
      kao(:, 8, 2) = (/ &
        & 0.10592e-05_rb,0.11093e-05_rb,0.11483e-05_rb,0.11923e-05_rb,0.12169e-05_rb /)
      kao(:, 9, 2) = (/ &
        & 0.50835e-05_rb,0.51710e-05_rb,0.52329e-05_rb,0.52644e-05_rb,0.52818e-05_rb /)
      kao(:,10, 2) = (/ &
        & 0.77867e-05_rb,0.82156e-05_rb,0.86002e-05_rb,0.89664e-05_rb,0.92394e-05_rb /)
      kao(:,11, 2) = (/ &
        & 0.89031e-05_rb,0.93573e-05_rb,0.97686e-05_rb,0.10236e-04_rb,0.10591e-04_rb /)
      kao(:,12, 2) = (/ &
        & 0.98068e-05_rb,0.10238e-04_rb,0.10700e-04_rb,0.11108e-04_rb,0.11471e-04_rb /)
      kao(:,13, 2) = (/ &
        & 0.11145e-04_rb,0.11697e-04_rb,0.12123e-04_rb,0.12482e-04_rb,0.12856e-04_rb /)
      kao(:, 1, 3) = (/ &
        & 0.66049e-05_rb,0.67547e-05_rb,0.70104e-05_rb,0.69745e-05_rb,0.70687e-05_rb /)
      kao(:, 2, 3) = (/ &
        & 0.54104e-05_rb,0.55245e-05_rb,0.57623e-05_rb,0.58532e-05_rb,0.58099e-05_rb /)
      kao(:, 3, 3) = (/ &
        & 0.43608e-05_rb,0.44784e-05_rb,0.45778e-05_rb,0.47665e-05_rb,0.47295e-05_rb /)
      kao(:, 4, 3) = (/ &
        & 0.35399e-05_rb,0.36454e-05_rb,0.37356e-05_rb,0.38107e-05_rb,0.39586e-05_rb /)
      kao(:, 5, 3) = (/ &
        & 0.28576e-05_rb,0.29552e-05_rb,0.30325e-05_rb,0.31028e-05_rb,0.32276e-05_rb /)
      kao(:, 6, 3) = (/ &
        & 0.21017e-05_rb,0.21836e-05_rb,0.22554e-05_rb,0.23121e-05_rb,0.23486e-05_rb /)
      kao(:, 7, 3) = (/ &
        & 0.19384e-05_rb,0.19914e-05_rb,0.20428e-05_rb,0.20879e-05_rb,0.21191e-05_rb /)
      kao(:, 8, 3) = (/ &
        & 0.32672e-05_rb,0.33753e-05_rb,0.34694e-05_rb,0.35273e-05_rb,0.35984e-05_rb /)
      kao(:, 9, 3) = (/ &
        & 0.82257e-05_rb,0.84975e-05_rb,0.86969e-05_rb,0.88995e-05_rb,0.90123e-05_rb /)
      kao(:,10, 3) = (/ &
        & 0.35363e-04_rb,0.36204e-04_rb,0.36916e-04_rb,0.37708e-04_rb,0.38294e-04_rb /)
      kao(:,11, 3) = (/ &
        & 0.48837e-04_rb,0.50952e-04_rb,0.52710e-04_rb,0.54347e-04_rb,0.55976e-04_rb /)
      kao(:,12, 3) = (/ &
        & 0.56059e-04_rb,0.58301e-04_rb,0.60100e-04_rb,0.61776e-04_rb,0.63179e-04_rb /)
      kao(:,13, 3) = (/ &
        & 0.57871e-04_rb,0.59505e-04_rb,0.60930e-04_rb,0.62289e-04_rb,0.63278e-04_rb /)
      kao(:, 1, 4) = (/ &
        & 0.27624e-04_rb,0.28017e-04_rb,0.29078e-04_rb,0.28567e-04_rb,0.28724e-04_rb /)
      kao(:, 2, 4) = (/ &
        & 0.23107e-04_rb,0.23464e-04_rb,0.24357e-04_rb,0.24519e-04_rb,0.24024e-04_rb /)
      kao(:, 3, 4) = (/ &
        & 0.19113e-04_rb,0.19400e-04_rb,0.19615e-04_rb,0.20420e-04_rb,0.19962e-04_rb /)
      kao(:, 4, 4) = (/ &
        & 0.15873e-04_rb,0.16138e-04_rb,0.16349e-04_rb,0.16528e-04_rb,0.17176e-04_rb /)
      kao(:, 5, 4) = (/ &
        & 0.13198e-04_rb,0.13437e-04_rb,0.13641e-04_rb,0.13810e-04_rb,0.14393e-04_rb /)
      kao(:, 6, 4) = (/ &
        & 0.10951e-04_rb,0.11172e-04_rb,0.11352e-04_rb,0.11506e-04_rb,0.11631e-04_rb /)
      kao(:, 7, 4) = (/ &
        & 0.86121e-05_rb,0.88300e-05_rb,0.90149e-05_rb,0.91565e-05_rb,0.92594e-05_rb /)
      kao(:, 8, 4) = (/ &
        & 0.71478e-05_rb,0.72918e-05_rb,0.74035e-05_rb,0.74959e-05_rb,0.75566e-05_rb /)
      kao(:, 9, 4) = (/ &
        & 0.16458e-04_rb,0.17092e-04_rb,0.17686e-04_rb,0.17967e-04_rb,0.18273e-04_rb /)
      kao(:,10, 4) = (/ &
        & 0.47953e-04_rb,0.49663e-04_rb,0.51524e-04_rb,0.52800e-04_rb,0.54192e-04_rb /)
      kao(:,11, 4) = (/ &
        & 0.94263e-04_rb,0.95557e-04_rb,0.96513e-04_rb,0.97430e-04_rb,0.97733e-04_rb /)
      kao(:,12, 4) = (/ &
        & 0.12087e-03_rb,0.12152e-03_rb,0.12240e-03_rb,0.12318e-03_rb,0.12403e-03_rb /)
      kao(:,13, 4) = (/ &
        & 0.12781e-03_rb,0.12897e-03_rb,0.13049e-03_rb,0.13171e-03_rb,0.13337e-03_rb /)
      kao(:, 1, 5) = (/ &
        & 0.82859e-04_rb,0.84817e-04_rb,0.89056e-04_rb,0.88057e-04_rb,0.89410e-04_rb /)
      kao(:, 2, 5) = (/ &
        & 0.70937e-04_rb,0.72685e-04_rb,0.76796e-04_rb,0.77955e-04_rb,0.76735e-04_rb /)
      kao(:, 3, 5) = (/ &
        & 0.59876e-04_rb,0.61448e-04_rb,0.62837e-04_rb,0.66293e-04_rb,0.64996e-04_rb /)
      kao(:, 4, 5) = (/ &
        & 0.50598e-04_rb,0.52054e-04_rb,0.53293e-04_rb,0.54332e-04_rb,0.57333e-04_rb /)
      kao(:, 5, 5) = (/ &
        & 0.42742e-04_rb,0.44035e-04_rb,0.45164e-04_rb,0.46134e-04_rb,0.48935e-04_rb /)
      kao(:, 6, 5) = (/ &
        & 0.35769e-04_rb,0.36975e-04_rb,0.38038e-04_rb,0.38917e-04_rb,0.39681e-04_rb /)
      kao(:, 7, 5) = (/ &
        & 0.29747e-04_rb,0.30824e-04_rb,0.31756e-04_rb,0.32589e-04_rb,0.33314e-04_rb /)
      kao(:, 8, 5) = (/ &
        & 0.21994e-04_rb,0.22945e-04_rb,0.23786e-04_rb,0.24517e-04_rb,0.25155e-04_rb /)
      kao(:, 9, 5) = (/ &
        & 0.22298e-04_rb,0.22688e-04_rb,0.23127e-04_rb,0.23803e-04_rb,0.24335e-04_rb /)
      kao(:,10, 5) = (/ &
        & 0.88898e-04_rb,0.91280e-04_rb,0.93333e-04_rb,0.95224e-04_rb,0.96298e-04_rb /)
      kao(:,11, 5) = (/ &
        & 0.12299e-03_rb,0.12407e-03_rb,0.12536e-03_rb,0.12642e-03_rb,0.12813e-03_rb /)
      kao(:,12, 5) = (/ &
        & 0.14539e-03_rb,0.14851e-03_rb,0.15022e-03_rb,0.15157e-03_rb,0.15204e-03_rb /)
      kao(:,13, 5) = (/ &
        & 0.15949e-03_rb,0.16239e-03_rb,0.16467e-03_rb,0.16667e-03_rb,0.16801e-03_rb /)
      kao(:, 1, 6) = (/ &
        & 0.25339e-03_rb,0.25995e-03_rb,0.27170e-03_rb,0.26963e-03_rb,0.27413e-03_rb /)
      kao(:, 2, 6) = (/ &
        & 0.21908e-03_rb,0.22404e-03_rb,0.23522e-03_rb,0.23902e-03_rb,0.23717e-03_rb /)
      kao(:, 3, 6) = (/ &
        & 0.18611e-03_rb,0.19076e-03_rb,0.19518e-03_rb,0.20503e-03_rb,0.20222e-03_rb /)
      kao(:, 4, 6) = (/ &
        & 0.15769e-03_rb,0.16210e-03_rb,0.16597e-03_rb,0.16948e-03_rb,0.17817e-03_rb /)
      kao(:, 5, 6) = (/ &
        & 0.13402e-03_rb,0.13792e-03_rb,0.14153e-03_rb,0.14434e-03_rb,0.15267e-03_rb /)
      kao(:, 6, 6) = (/ &
        & 0.11390e-03_rb,0.11743e-03_rb,0.12065e-03_rb,0.12316e-03_rb,0.12543e-03_rb /)
      kao(:, 7, 6) = (/ &
        & 0.96417e-04_rb,0.99612e-04_rb,0.10233e-03_rb,0.10453e-03_rb,0.10652e-03_rb /)
      kao(:, 8, 6) = (/ &
        & 0.81395e-04_rb,0.84205e-04_rb,0.86346e-04_rb,0.88406e-04_rb,0.90122e-04_rb /)
      kao(:, 9, 6) = (/ &
        & 0.47776e-04_rb,0.48971e-04_rb,0.49736e-04_rb,0.49917e-04_rb,0.50289e-04_rb /)
      kao(:,10, 6) = (/ &
        & 0.10698e-03_rb,0.10815e-03_rb,0.10817e-03_rb,0.10799e-03_rb,0.10851e-03_rb /)
      kao(:,11, 6) = (/ &
        & 0.20220e-03_rb,0.20727e-03_rb,0.21241e-03_rb,0.21675e-03_rb,0.21989e-03_rb /)
      kao(:,12, 6) = (/ &
        & 0.23474e-03_rb,0.23601e-03_rb,0.23974e-03_rb,0.24383e-03_rb,0.24876e-03_rb /)
      kao(:,13, 6) = (/ &
        & 0.23410e-03_rb,0.23809e-03_rb,0.24185e-03_rb,0.24554e-03_rb,0.24952e-03_rb /)
      kao(:, 1, 7) = (/ &
        & 0.67024e-03_rb,0.68026e-03_rb,0.70419e-03_rb,0.70159e-03_rb,0.71089e-03_rb /)
      kao(:, 2, 7) = (/ &
        & 0.58729e-03_rb,0.59778e-03_rb,0.62097e-03_rb,0.62912e-03_rb,0.62423e-03_rb /)
      kao(:, 3, 7) = (/ &
        & 0.50967e-03_rb,0.51900e-03_rb,0.52765e-03_rb,0.54794e-03_rb,0.54266e-03_rb /)
      kao(:, 4, 7) = (/ &
        & 0.44167e-03_rb,0.45006e-03_rb,0.45793e-03_rb,0.46469e-03_rb,0.48335e-03_rb /)
      kao(:, 5, 7) = (/ &
        & 0.38096e-03_rb,0.38881e-03_rb,0.39576e-03_rb,0.40259e-03_rb,0.42086e-03_rb /)
      kao(:, 6, 7) = (/ &
        & 0.32818e-03_rb,0.33539e-03_rb,0.34192e-03_rb,0.34829e-03_rb,0.35405e-03_rb /)
      kao(:, 7, 7) = (/ &
        & 0.28259e-03_rb,0.28946e-03_rb,0.29584e-03_rb,0.30203e-03_rb,0.30742e-03_rb /)
      kao(:, 8, 7) = (/ &
        & 0.24273e-03_rb,0.24912e-03_rb,0.25546e-03_rb,0.26110e-03_rb,0.26607e-03_rb /)
      kao(:, 9, 7) = (/ &
        & 0.19937e-03_rb,0.20653e-03_rb,0.21314e-03_rb,0.21968e-03_rb,0.22520e-03_rb /)
      kao(:,10, 7) = (/ &
        & 0.13306e-03_rb,0.13331e-03_rb,0.13393e-03_rb,0.13538e-03_rb,0.13616e-03_rb /)
      kao(:,11, 7) = (/ &
        & 0.16236e-03_rb,0.16154e-03_rb,0.16187e-03_rb,0.16113e-03_rb,0.16209e-03_rb /)
      kao(:,12, 7) = (/ &
        & 0.17872e-03_rb,0.18355e-03_rb,0.18612e-03_rb,0.18792e-03_rb,0.18745e-03_rb /)
      kao(:,13, 7) = (/ &
        & 0.18970e-03_rb,0.19384e-03_rb,0.19773e-03_rb,0.20261e-03_rb,0.20377e-03_rb /)
      kao(:, 1, 8) = (/ &
        & 0.18130e-02_rb,0.18305e-02_rb,0.18716e-02_rb,0.18655e-02_rb,0.18814e-02_rb /)
      kao(:, 2, 8) = (/ &
        & 0.16420e-02_rb,0.16600e-02_rb,0.17006e-02_rb,0.17156e-02_rb,0.17108e-02_rb /)
      kao(:, 3, 8) = (/ &
        & 0.14687e-02_rb,0.14870e-02_rb,0.15042e-02_rb,0.15423e-02_rb,0.15376e-02_rb /)
      kao(:, 4, 8) = (/ &
        & 0.13068e-02_rb,0.13248e-02_rb,0.13421e-02_rb,0.13592e-02_rb,0.13960e-02_rb /)
      kao(:, 5, 8) = (/ &
        & 0.11574e-02_rb,0.11753e-02_rb,0.11928e-02_rb,0.12097e-02_rb,0.12467e-02_rb /)
      kao(:, 6, 8) = (/ &
        & 0.10167e-02_rb,0.10342e-02_rb,0.10515e-02_rb,0.10681e-02_rb,0.10840e-02_rb /)
      kao(:, 7, 8) = (/ &
        & 0.88992e-03_rb,0.90662e-03_rb,0.92299e-03_rb,0.93844e-03_rb,0.95347e-03_rb /)
      kao(:, 8, 8) = (/ &
        & 0.78445e-03_rb,0.80031e-03_rb,0.81639e-03_rb,0.83131e-03_rb,0.84631e-03_rb /)
      kao(:, 9, 8) = (/ &
        & 0.69812e-03_rb,0.71312e-03_rb,0.72801e-03_rb,0.74222e-03_rb,0.75605e-03_rb /)
      kao(:,10, 8) = (/ &
        & 0.32521e-03_rb,0.33835e-03_rb,0.35194e-03_rb,0.36381e-03_rb,0.37594e-03_rb /)
      kao(:,11, 8) = (/ &
        & 0.31406e-03_rb,0.32013e-03_rb,0.32456e-03_rb,0.33261e-03_rb,0.33741e-03_rb /)
      kao(:,12, 8) = (/ &
        & 0.28132e-03_rb,0.28732e-03_rb,0.29674e-03_rb,0.30509e-03_rb,0.31393e-03_rb /)
      kao(:,13, 8) = (/ &
        & 0.25704e-03_rb,0.26316e-03_rb,0.27195e-03_rb,0.27732e-03_rb,0.28905e-03_rb /)
      kao(:, 1, 9) = (/ &
        & 0.67370e-02_rb,0.67873e-02_rb,0.68896e-02_rb,0.68819e-02_rb,0.69268e-02_rb /)
      kao(:, 2, 9) = (/ &
        & 0.63111e-02_rb,0.63622e-02_rb,0.64623e-02_rb,0.65060e-02_rb,0.65027e-02_rb /)
      kao(:, 3, 9) = (/ &
        & 0.58834e-02_rb,0.59361e-02_rb,0.59858e-02_rb,0.60811e-02_rb,0.60806e-02_rb /)
      kao(:, 4, 9) = (/ &
        & 0.54753e-02_rb,0.55309e-02_rb,0.55823e-02_rb,0.56306e-02_rb,0.57229e-02_rb /)
      kao(:, 5, 9) = (/ &
        & 0.50781e-02_rb,0.51373e-02_rb,0.51895e-02_rb,0.52391e-02_rb,0.53328e-02_rb /)
      kao(:, 6, 9) = (/ &
        & 0.46791e-02_rb,0.47408e-02_rb,0.47949e-02_rb,0.48470e-02_rb,0.48993e-02_rb /)
      kao(:, 7, 9) = (/ &
        & 0.42724e-02_rb,0.43381e-02_rb,0.43951e-02_rb,0.44503e-02_rb,0.45061e-02_rb /)
      kao(:, 8, 9) = (/ &
        & 0.38568e-02_rb,0.39254e-02_rb,0.39848e-02_rb,0.40431e-02_rb,0.41005e-02_rb /)
      kao(:, 9, 9) = (/ &
        & 0.35657e-02_rb,0.36362e-02_rb,0.36994e-02_rb,0.37608e-02_rb,0.38215e-02_rb /)
      kao(:,10, 9) = (/ &
        & 0.33774e-02_rb,0.34501e-02_rb,0.35171e-02_rb,0.35811e-02_rb,0.36450e-02_rb /)
      kao(:,11, 9) = (/ &
        & 0.23923e-02_rb,0.24623e-02_rb,0.25263e-02_rb,0.25890e-02_rb,0.26517e-02_rb /)
      kao(:,12, 9) = (/ &
        & 0.16959e-02_rb,0.17542e-02_rb,0.18077e-02_rb,0.18614e-02_rb,0.19187e-02_rb /)
      kao(:,13, 9) = (/ &
        & 0.11732e-02_rb,0.12232e-02_rb,0.12712e-02_rb,0.13221e-02_rb,0.13677e-02_rb /)
      kao(:, 1,10) = (/ &
        & 0.19604e-01_rb,0.19698e-01_rb,0.19938e-01_rb,0.19854e-01_rb,0.19950e-01_rb /)
      kao(:, 2,10) = (/ &
        & 0.18714e-01_rb,0.18803e-01_rb,0.19035e-01_rb,0.19131e-01_rb,0.19102e-01_rb /)
      kao(:, 3,10) = (/ &
        & 0.17676e-01_rb,0.17785e-01_rb,0.17904e-01_rb,0.18189e-01_rb,0.18156e-01_rb /)
      kao(:, 4,10) = (/ &
        & 0.16662e-01_rb,0.16773e-01_rb,0.16908e-01_rb,0.17056e-01_rb,0.17303e-01_rb /)
      kao(:, 5,10) = (/ &
        & 0.15655e-01_rb,0.15775e-01_rb,0.15942e-01_rb,0.16103e-01_rb,0.16359e-01_rb /)
      kao(:, 6,10) = (/ &
        & 0.14694e-01_rb,0.14838e-01_rb,0.15029e-01_rb,0.15200e-01_rb,0.15320e-01_rb /)
      kao(:, 7,10) = (/ &
        & 0.13797e-01_rb,0.13959e-01_rb,0.14174e-01_rb,0.14350e-01_rb,0.14471e-01_rb /)
      kao(:, 8,10) = (/ &
        & 0.12902e-01_rb,0.13089e-01_rb,0.13313e-01_rb,0.13489e-01_rb,0.13626e-01_rb /)
      kao(:, 9,10) = (/ &
        & 0.11897e-01_rb,0.12105e-01_rb,0.12333e-01_rb,0.12512e-01_rb,0.12656e-01_rb /)
      kao(:,10,10) = (/ &
        & 0.11834e-01_rb,0.12072e-01_rb,0.12331e-01_rb,0.12548e-01_rb,0.12721e-01_rb /)
      kao(:,11,10) = (/ &
        & 0.11416e-01_rb,0.11699e-01_rb,0.11968e-01_rb,0.12141e-01_rb,0.12311e-01_rb /)
      kao(:,12,10) = (/ &
        & 0.10776e-01_rb,0.11070e-01_rb,0.11309e-01_rb,0.11507e-01_rb,0.11702e-01_rb /)
      kao(:,13,10) = (/ &
        & 0.99577e-02_rb,0.10263e-01_rb,0.10492e-01_rb,0.10719e-01_rb,0.10951e-01_rb /)
      kao(:, 1,11) = (/ &
        & 0.29783e-01_rb,0.29883e-01_rb,0.30210e-01_rb,0.30141e-01_rb,0.30248e-01_rb /)
      kao(:, 2,11) = (/ &
        & 0.28562e-01_rb,0.28743e-01_rb,0.29186e-01_rb,0.29339e-01_rb,0.29258e-01_rb /)
      kao(:, 3,11) = (/ &
        & 0.27212e-01_rb,0.27429e-01_rb,0.27654e-01_rb,0.28083e-01_rb,0.28023e-01_rb /)
      kao(:, 4,11) = (/ &
        & 0.25949e-01_rb,0.26197e-01_rb,0.26424e-01_rb,0.26623e-01_rb,0.27069e-01_rb /)
      kao(:, 5,11) = (/ &
        & 0.24686e-01_rb,0.24942e-01_rb,0.25176e-01_rb,0.25403e-01_rb,0.25908e-01_rb /)
      kao(:, 6,11) = (/ &
        & 0.23430e-01_rb,0.23686e-01_rb,0.23923e-01_rb,0.24158e-01_rb,0.24433e-01_rb /)
      kao(:, 7,11) = (/ &
        & 0.22171e-01_rb,0.22424e-01_rb,0.22653e-01_rb,0.22909e-01_rb,0.23203e-01_rb /)
      kao(:, 8,11) = (/ &
        & 0.20928e-01_rb,0.21171e-01_rb,0.21407e-01_rb,0.21699e-01_rb,0.22018e-01_rb /)
      kao(:, 9,11) = (/ &
        & 0.19076e-01_rb,0.19320e-01_rb,0.19548e-01_rb,0.19858e-01_rb,0.20150e-01_rb /)
      kao(:,10,11) = (/ &
        & 0.19537e-01_rb,0.19788e-01_rb,0.20064e-01_rb,0.20391e-01_rb,0.20843e-01_rb /)
      kao(:,11,11) = (/ &
        & 0.19137e-01_rb,0.19444e-01_rb,0.19793e-01_rb,0.20268e-01_rb,0.20695e-01_rb /)
      kao(:,12,11) = (/ &
        & 0.18393e-01_rb,0.18715e-01_rb,0.19143e-01_rb,0.19652e-01_rb,0.20037e-01_rb /)
      kao(:,13,11) = (/ &
        & 0.17255e-01_rb,0.17680e-01_rb,0.18170e-01_rb,0.18641e-01_rb,0.19010e-01_rb /)
      kao(:, 1,12) = (/ &
        & 0.46641e-01_rb,0.46796e-01_rb,0.47107e-01_rb,0.46977e-01_rb,0.47093e-01_rb /)
      kao(:, 2,12) = (/ &
        & 0.46819e-01_rb,0.46956e-01_rb,0.47337e-01_rb,0.47426e-01_rb,0.47339e-01_rb /)
      kao(:, 3,12) = (/ &
        & 0.46276e-01_rb,0.46366e-01_rb,0.46462e-01_rb,0.46935e-01_rb,0.46862e-01_rb /)
      kao(:, 4,12) = (/ &
        & 0.44986e-01_rb,0.45103e-01_rb,0.45287e-01_rb,0.45535e-01_rb,0.46152e-01_rb /)
      kao(:, 5,12) = (/ &
        & 0.43367e-01_rb,0.43552e-01_rb,0.43761e-01_rb,0.44035e-01_rb,0.44674e-01_rb /)
      kao(:, 6,12) = (/ &
        & 0.41584e-01_rb,0.41793e-01_rb,0.42020e-01_rb,0.42344e-01_rb,0.42701e-01_rb /)
      kao(:, 7,12) = (/ &
        & 0.39785e-01_rb,0.40007e-01_rb,0.40269e-01_rb,0.40635e-01_rb,0.41003e-01_rb /)
      kao(:, 8,12) = (/ &
        & 0.37918e-01_rb,0.38155e-01_rb,0.38468e-01_rb,0.38839e-01_rb,0.39210e-01_rb /)
      kao(:, 9,12) = (/ &
        & 0.35060e-01_rb,0.35328e-01_rb,0.35698e-01_rb,0.36105e-01_rb,0.36551e-01_rb /)
      kao(:,10,12) = (/ &
        & 0.35216e-01_rb,0.35547e-01_rb,0.35872e-01_rb,0.36327e-01_rb,0.36542e-01_rb /)
      kao(:,11,12) = (/ &
        & 0.35158e-01_rb,0.35374e-01_rb,0.35869e-01_rb,0.36242e-01_rb,0.36691e-01_rb /)
      kao(:,12,12) = (/ &
        & 0.34264e-01_rb,0.34598e-01_rb,0.35106e-01_rb,0.35505e-01_rb,0.36021e-01_rb /)
      kao(:,13,12) = (/ &
        & 0.32716e-01_rb,0.33195e-01_rb,0.33830e-01_rb,0.34228e-01_rb,0.34925e-01_rb /)
      kao(:, 1,13) = (/ &
        & 0.76084e-01_rb,0.76052e-01_rb,0.76051e-01_rb,0.75851e-01_rb,0.75753e-01_rb /)
      kao(:, 2,13) = (/ &
        & 0.79580e-01_rb,0.79564e-01_rb,0.79664e-01_rb,0.79619e-01_rb,0.79414e-01_rb /)
      kao(:, 3,13) = (/ &
        & 0.82218e-01_rb,0.82302e-01_rb,0.82367e-01_rb,0.82543e-01_rb,0.82246e-01_rb /)
      kao(:, 4,13) = (/ &
        & 0.83613e-01_rb,0.83740e-01_rb,0.83824e-01_rb,0.83866e-01_rb,0.84103e-01_rb /)
      kao(:, 5,13) = (/ &
        & 0.83913e-01_rb,0.84123e-01_rb,0.84289e-01_rb,0.84403e-01_rb,0.84840e-01_rb /)
      kao(:, 6,13) = (/ &
        & 0.83159e-01_rb,0.83442e-01_rb,0.83718e-01_rb,0.83891e-01_rb,0.83993e-01_rb /)
      kao(:, 7,13) = (/ &
        & 0.81401e-01_rb,0.81826e-01_rb,0.82202e-01_rb,0.82451e-01_rb,0.82656e-01_rb /)
      kao(:, 8,13) = (/ &
        & 0.78949e-01_rb,0.79505e-01_rb,0.79978e-01_rb,0.80351e-01_rb,0.80690e-01_rb /)
      kao(:, 9,13) = (/ &
        & 0.76002e-01_rb,0.76671e-01_rb,0.77279e-01_rb,0.77752e-01_rb,0.78257e-01_rb /)
      kao(:,10,13) = (/ &
        & 0.69777e-01_rb,0.70425e-01_rb,0.71054e-01_rb,0.71706e-01_rb,0.72167e-01_rb /)
      kao(:,11,13) = (/ &
        & 0.72929e-01_rb,0.73732e-01_rb,0.74323e-01_rb,0.75246e-01_rb,0.75786e-01_rb /)
      kao(:,12,13) = (/ &
        & 0.73007e-01_rb,0.74429e-01_rb,0.74922e-01_rb,0.75916e-01_rb,0.76947e-01_rb /)
      kao(:,13,13) = (/ &
        & 0.71376e-01_rb,0.72507e-01_rb,0.73710e-01_rb,0.74716e-01_rb,0.75702e-01_rb /)
      kao(:, 1,14) = (/ &
        & 0.12585e+00_rb,0.12569e+00_rb,0.12576e+00_rb,0.12571e+00_rb,0.12570e+00_rb /)
      kao(:, 2,14) = (/ &
        & 0.13868e+00_rb,0.13853e+00_rb,0.13849e+00_rb,0.13824e+00_rb,0.13803e+00_rb /)
      kao(:, 3,14) = (/ &
        & 0.15142e+00_rb,0.15135e+00_rb,0.15112e+00_rb,0.15093e+00_rb,0.15074e+00_rb /)
      kao(:, 4,14) = (/ &
        & 0.16359e+00_rb,0.16359e+00_rb,0.16339e+00_rb,0.16311e+00_rb,0.16285e+00_rb /)
      kao(:, 5,14) = (/ &
        & 0.17462e+00_rb,0.17472e+00_rb,0.17456e+00_rb,0.17421e+00_rb,0.17393e+00_rb /)
      kao(:, 6,14) = (/ &
        & 0.18403e+00_rb,0.18447e+00_rb,0.18450e+00_rb,0.18424e+00_rb,0.18395e+00_rb /)
      kao(:, 7,14) = (/ &
        & 0.19179e+00_rb,0.19239e+00_rb,0.19256e+00_rb,0.19253e+00_rb,0.19252e+00_rb /)
      kao(:, 8,14) = (/ &
        & 0.19772e+00_rb,0.19863e+00_rb,0.19901e+00_rb,0.19923e+00_rb,0.19947e+00_rb /)
      kao(:, 9,14) = (/ &
        & 0.20154e+00_rb,0.20279e+00_rb,0.20355e+00_rb,0.20417e+00_rb,0.20473e+00_rb /)
      kao(:,10,14) = (/ &
        & 0.18853e+00_rb,0.19028e+00_rb,0.19160e+00_rb,0.19253e+00_rb,0.19393e+00_rb /)
      kao(:,11,14) = (/ &
        & 0.18013e+00_rb,0.18167e+00_rb,0.18320e+00_rb,0.18375e+00_rb,0.18507e+00_rb /)
      kao(:,12,14) = (/ &
        & 0.19011e+00_rb,0.19027e+00_rb,0.19283e+00_rb,0.19402e+00_rb,0.19478e+00_rb /)
      kao(:,13,14) = (/ &
        & 0.19594e+00_rb,0.19738e+00_rb,0.19911e+00_rb,0.20124e+00_rb,0.20282e+00_rb /)
      kao(:, 1,15) = (/ &
        & 0.22369e+00_rb,0.22259e+00_rb,0.22155e+00_rb,0.22059e+00_rb,0.21997e+00_rb /)
      kao(:, 2,15) = (/ &
        & 0.25602e+00_rb,0.25478e+00_rb,0.25377e+00_rb,0.25306e+00_rb,0.25237e+00_rb /)
      kao(:, 3,15) = (/ &
        & 0.29258e+00_rb,0.29107e+00_rb,0.28998e+00_rb,0.28920e+00_rb,0.28830e+00_rb /)
      kao(:, 4,15) = (/ &
        & 0.33067e+00_rb,0.32888e+00_rb,0.32753e+00_rb,0.32646e+00_rb,0.32566e+00_rb /)
      kao(:, 5,15) = (/ &
        & 0.37114e+00_rb,0.36880e+00_rb,0.36713e+00_rb,0.36598e+00_rb,0.36499e+00_rb /)
      kao(:, 6,15) = (/ &
        & 0.41494e+00_rb,0.41167e+00_rb,0.40935e+00_rb,0.40779e+00_rb,0.40636e+00_rb /)
      kao(:, 7,15) = (/ &
        & 0.46115e+00_rb,0.45729e+00_rb,0.45455e+00_rb,0.45230e+00_rb,0.45004e+00_rb /)
      kao(:, 8,15) = (/ &
        & 0.50906e+00_rb,0.50463e+00_rb,0.50137e+00_rb,0.49843e+00_rb,0.49550e+00_rb /)
      kao(:, 9,15) = (/ &
        & 0.55829e+00_rb,0.55330e+00_rb,0.54936e+00_rb,0.54557e+00_rb,0.54168e+00_rb /)
      kao(:,10,15) = (/ &
        & 0.60814e+00_rb,0.60274e+00_rb,0.59806e+00_rb,0.59326e+00_rb,0.58833e+00_rb /)
      kao(:,11,15) = (/ &
        & 0.62954e+00_rb,0.62588e+00_rb,0.62076e+00_rb,0.61665e+00_rb,0.61184e+00_rb /)
      kao(:,12,15) = (/ &
        & 0.62585e+00_rb,0.62437e+00_rb,0.61807e+00_rb,0.61303e+00_rb,0.60869e+00_rb /)
      kao(:,13,15) = (/ &
        & 0.64856e+00_rb,0.64505e+00_rb,0.63861e+00_rb,0.63277e+00_rb,0.62702e+00_rb /)
      kao(:, 1,16) = (/ &
        & 0.33327e+00_rb,0.33385e+00_rb,0.33538e+00_rb,0.33638e+00_rb,0.33736e+00_rb /)
      kao(:, 2,16) = (/ &
        & 0.40916e+00_rb,0.40842e+00_rb,0.40848e+00_rb,0.40865e+00_rb,0.40854e+00_rb /)
      kao(:, 3,16) = (/ &
        & 0.50099e+00_rb,0.49888e+00_rb,0.49727e+00_rb,0.49588e+00_rb,0.49422e+00_rb /)
      kao(:, 4,16) = (/ &
        & 0.60389e+00_rb,0.60029e+00_rb,0.59704e+00_rb,0.59367e+00_rb,0.59071e+00_rb /)
      kao(:, 5,16) = (/ &
        & 0.71868e+00_rb,0.71337e+00_rb,0.70835e+00_rb,0.70318e+00_rb,0.69852e+00_rb /)
      kao(:, 6,16) = (/ &
        & 0.84815e+00_rb,0.84138e+00_rb,0.83446e+00_rb,0.82728e+00_rb,0.82023e+00_rb /)
      kao(:, 7,16) = (/ &
        & 0.99512e+00_rb,0.98644e+00_rb,0.97698e+00_rb,0.96695e+00_rb,0.95712e+00_rb /)
      kao(:, 8,16) = (/ &
        & 0.11606e+01_rb,0.11485e+01_rb,0.11354e+01_rb,0.11218e+01_rb,0.11077e+01_rb /)
      kao(:, 9,16) = (/ &
        & 0.13444e+01_rb,0.13282e+01_rb,0.13102e+01_rb,0.12917e+01_rb,0.12735e+01_rb /)
      kao(:,10,16) = (/ &
        & 0.15423e+01_rb,0.15207e+01_rb,0.14970e+01_rb,0.14733e+01_rb,0.14494e+01_rb /)
      kao(:,11,16) = (/ &
        & 0.17462e+01_rb,0.17138e+01_rb,0.16827e+01_rb,0.16518e+01_rb,0.16199e+01_rb /)
      kao(:,12,16) = (/ &
        & 0.19577e+01_rb,0.19145e+01_rb,0.18728e+01_rb,0.18320e+01_rb,0.17910e+01_rb /)
      kao(:,13,16) = (/ &
        & 0.21716e+01_rb,0.21171e+01_rb,0.20639e+01_rb,0.20113e+01_rb,0.19587e+01_rb /)
  
!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.315770e-07_rb, 0.671978e-07_rb, 0.440649e-06_rb /)
      forrefo(:, 2) = (/ 0.313674e-06_rb, 0.285252e-06_rb, 0.421024e-05_rb /)
      forrefo(:, 3) = (/ 0.135818e-05_rb, 0.145071e-05_rb, 0.611285e-05_rb /)
      forrefo(:, 4) = (/ 0.534065e-05_rb, 0.586268e-05_rb, 0.933970e-05_rb /)
      forrefo(:, 5) = (/ 0.964007e-05_rb, 0.107110e-04_rb, 0.104486e-04_rb /)
      forrefo(:, 6) = (/ 0.302775e-04_rb, 0.357530e-04_rb, 0.340724e-04_rb /)
      forrefo(:, 7) = (/ 0.102437e-03_rb, 0.108475e-03_rb, 0.105245e-03_rb /)
      forrefo(:, 8) = (/ 0.146054e-03_rb, 0.141490e-03_rb, 0.133071e-03_rb /)
      forrefo(:, 9) = (/ 0.163978e-03_rb, 0.150208e-03_rb, 0.142864e-03_rb /)
      forrefo(:,10) = (/ 0.220412e-03_rb, 0.182943e-03_rb, 0.150941e-03_rb /)
      forrefo(:,11) = (/ 0.228877e-03_rb, 0.197679e-03_rb, 0.163220e-03_rb /)
      forrefo(:,12) = (/ 0.234177e-03_rb, 0.217734e-03_rb, 0.185038e-03_rb /)
      forrefo(:,13) = (/ 0.257187e-03_rb, 0.241570e-03_rb, 0.221178e-03_rb /)
      forrefo(:,14) = (/ 0.272455e-03_rb, 0.270637e-03_rb, 0.256269e-03_rb /)
      forrefo(:,15) = (/ 0.339445e-03_rb, 0.300268e-03_rb, 0.286574e-03_rb /)
      forrefo(:,16) = (/ 0.338841e-03_rb, 0.355428e-03_rb, 0.353794e-03_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
        &  0.100945e-04_rb, 0.801113e-05_rb, 0.635771e-05_rb, 0.504554e-05_rb, 0.400419e-05_rb, &
        &  0.317777e-05_rb, 0.252191e-05_rb, 0.200141e-05_rb, 0.158834e-05_rb, 0.126052e-05_rb /)
      selfrefo(:, 2) = (/ &
        &  0.107573e-04_rb, 0.999809e-05_rb, 0.929245e-05_rb, 0.863661e-05_rb, 0.802706e-05_rb, &
        &  0.746053e-05_rb, 0.693399e-05_rb, 0.644460e-05_rb, 0.598976e-05_rb, 0.556702e-05_rb /)
      selfrefo(:, 3) = (/ &
        &  0.350389e-04_rb, 0.319234e-04_rb, 0.290850e-04_rb, 0.264989e-04_rb, 0.241428e-04_rb, &
        &  0.219962e-04_rb, 0.200404e-04_rb, 0.182586e-04_rb, 0.166351e-04_rb, 0.151560e-04_rb /)
      selfrefo(:, 4) = (/ &
        &  0.122993e-03_rb, 0.110885e-03_rb, 0.999691e-04_rb, 0.901277e-04_rb, 0.812551e-04_rb, &
        &  0.732559e-04_rb, 0.660443e-04_rb, 0.595426e-04_rb, 0.536809e-04_rb, 0.483963e-04_rb /)
      selfrefo(:, 5) = (/ &
        &  0.206434e-03_rb, 0.187435e-03_rb, 0.170185e-03_rb, 0.154522e-03_rb, 0.140301e-03_rb, &
        &  0.127388e-03_rb, 0.115664e-03_rb, 0.105019e-03_rb, 0.953540e-04_rb, 0.865783e-04_rb /)
      selfrefo(:, 6) = (/ &
        &  0.590645e-03_rb, 0.533109e-03_rb, 0.481177e-03_rb, 0.434305e-03_rb, 0.391998e-03_rb, &
        &  0.353812e-03_rb, 0.319346e-03_rb, 0.288238e-03_rb, 0.260160e-03_rb, 0.234817e-03_rb /)
      selfrefo(:, 7) = (/ &
        &  0.163029e-02_rb, 0.148773e-02_rb, 0.135763e-02_rb, 0.123891e-02_rb, 0.113057e-02_rb, &
        &  0.103170e-02_rb, 0.941483e-03_rb, 0.859153e-03_rb, 0.784023e-03_rb, 0.715462e-03_rb /)
      selfrefo(:, 8) = (/ &
        &  0.204528e-02_rb, 0.189258e-02_rb, 0.175128e-02_rb, 0.162053e-02_rb, 0.149954e-02_rb, &
        &  0.138758e-02_rb, 0.128398e-02_rb, 0.118812e-02_rb, 0.109941e-02_rb, 0.101733e-02_rb /)
      selfrefo(:, 9) = (/ &
        &  0.210589e-02_rb, 0.197078e-02_rb, 0.184434e-02_rb, 0.172601e-02_rb, 0.161528e-02_rb, &
        &  0.151164e-02_rb, 0.141466e-02_rb, 0.132390e-02_rb, 0.123896e-02_rb, 0.115947e-02_rb /)
      selfrefo(:,10) = (/ &
        &  0.245098e-02_rb, 0.233745e-02_rb, 0.222918e-02_rb, 0.212592e-02_rb, 0.202745e-02_rb, &
        &  0.193353e-02_rb, 0.184397e-02_rb, 0.175856e-02_rb, 0.167710e-02_rb, 0.159941e-02_rb /)
      selfrefo(:,11) = (/ &
        &  0.267460e-02_rb, 0.253325e-02_rb, 0.239936e-02_rb, 0.227255e-02_rb, 0.215244e-02_rb, &
        &  0.203868e-02_rb, 0.193093e-02_rb, 0.182888e-02_rb, 0.173222e-02_rb, 0.164067e-02_rb /)
      selfrefo(:,12) = (/ &
        &  0.304510e-02_rb, 0.283919e-02_rb, 0.264720e-02_rb, 0.246820e-02_rb, 0.230130e-02_rb, &
        &  0.214568e-02_rb, 0.200059e-02_rb, 0.186531e-02_rb, 0.173918e-02_rb, 0.162157e-02_rb /)
      selfrefo(:,13) = (/ &
        &  0.338445e-02_rb, 0.314719e-02_rb, 0.292655e-02_rb, 0.272139e-02_rb, 0.253060e-02_rb, &
        &  0.235319e-02_rb, 0.218822e-02_rb, 0.203482e-02_rb, 0.189217e-02_rb, 0.175952e-02_rb /)
      selfrefo(:,14) = (/ &
        &  0.388649e-02_rb, 0.357018e-02_rb, 0.327961e-02_rb, 0.301269e-02_rb, 0.276750e-02_rb, &
        &  0.254226e-02_rb, 0.233535e-02_rb, 0.214528e-02_rb, 0.197068e-02_rb, 0.181029e-02_rb /)
      selfrefo(:,15) = (/ &
        &  0.412547e-02_rb, 0.387413e-02_rb, 0.363810e-02_rb, 0.341646e-02_rb, 0.320831e-02_rb, &
        &  0.301285e-02_rb, 0.282930e-02_rb, 0.265693e-02_rb, 0.249506e-02_rb, 0.234305e-02_rb /)
      selfrefo(:,16) = (/ &
        &  0.534327e-02_rb, 0.482967e-02_rb, 0.436544e-02_rb, 0.394583e-02_rb, 0.356655e-02_rb, &
        &  0.322373e-02_rb, 0.291387e-02_rb, 0.263378e-02_rb, 0.238062e-02_rb, 0.215179e-02_rb /)
     
      end subroutine sw_kgb23

! **************************************************************************
      subroutine sw_kgb24
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg24, only : kao, kbo, selfrefo, forrefo, sfluxrefo, &
                            raylao, raylbo, abso3ao, abso3bo, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:,1) = (/ &
        & 34.3610_rb , 33.1240_rb, 31.3948_rb, 28.7248_rb, &
        & 24.7884_rb , 21.4892_rb, 17.3972_rb, 13.7928_rb, &
        & 9.54462_rb , 1.05002_rb,0.867332_rb,0.685753_rb, &
        & 0.504718_rb,0.323112_rb,0.122183_rb, 1.70288e-02_rb /)
      sfluxrefo(:,2) = (/ &
        & 34.2367_rb , 32.4327_rb, 30.0863_rb, 28.2085_rb, & 
        & 25.6533_rb , 22.3412_rb, 18.3112_rb, 13.8521_rb, &
        & 9.51035_rb , 1.04138_rb,0.863493_rb,0.682790_rb, &
        & 0.504721_rb,0.323102_rb,0.122193_rb, 1.70288e-02_rb /)
      sfluxrefo(:,3) = (/ &
        & 34.1883_rb , 32.2479_rb, 30.2650_rb, 28.2914_rb, &
        & 25.6626_rb , 22.3163_rb, 18.3327_rb, 13.8508_rb, &
        & 9.49190_rb , 1.03672_rb,0.858272_rb,0.681485_rb, &
        & 0.501363_rb,0.323110_rb,0.122183_rb, 1.70288e-02_rb /)
      sfluxrefo(:,4) = (/ &
        & 34.1365_rb , 32.2316_rb, 30.3325_rb, 28.3305_rb, &
        & 25.6420_rb , 22.3223_rb, 18.3411_rb, 13.8471_rb, &
        & 9.47492_rb , 1.03376_rb,0.855380_rb,0.679085_rb, &
        & 0.497998_rb,0.323053_rb,0.122183_rb, 1.70288e-02_rb /)
      sfluxrefo(:,5) = (/ &
        & 34.0460_rb , 32.2795_rb, 30.4147_rb, 28.3123_rb, &
        & 25.6438_rb , 22.3238_rb, 18.3441_rb, 13.8528_rb, &
        & 9.45222_rb , 1.03058_rb,0.854037_rb,0.675554_rb, &
        & 0.498344_rb,0.320072_rb,0.122193_rb, 1.70288e-02_rb /)
      sfluxrefo(:,6) = (/ &
        & 33.9909_rb , 32.3127_rb, 30.4854_rb, 28.3005_rb, &
        & 25.6310_rb , 22.3294_rb, 18.3459_rb, 13.8488_rb, &
        & 9.43336_rb , 1.02901_rb,0.852728_rb,0.672322_rb, &
        & 0.498056_rb,0.317753_rb,0.122183_rb, 1.70288e-02_rb /)
      sfluxrefo(:,7) = (/ &
        & 33.9225_rb , 32.4097_rb, 30.5125_rb, 28.2810_rb, &
        & 25.6387_rb , 22.3080_rb, 18.3715_rb, 13.8248_rb, &
        & 9.41834_rb , 1.02735_rb,0.850807_rb,0.671379_rb, &
        & 0.496975_rb,0.317158_rb,0.119297_rb, 1.70207e-02_rb /)
      sfluxrefo(:,8) = (/ &
        & 33.8940_rb , 32.4951_rb, 30.5494_rb, 28.2788_rb, &
        & 25.5975_rb , 22.3225_rb, 18.3358_rb, 13.8199_rb, &
        & 9.40283_rb , 1.02751_rb,0.850729_rb,0.670152_rb, &
        & 0.494294_rb,0.315829_rb,0.116195_rb, 1.64138e-02_rb /)
      sfluxrefo(:,9) = (/ &
        & 34.6501_rb , 32.6690_rb, 30.2872_rb, 28.0955_rb, &
        & 25.4662_rb , 22.1446_rb, 18.2754_rb, 13.7573_rb, &
        & 9.36645_rb , 1.02356_rb,0.847154_rb,0.668519_rb, &
        & 0.489186_rb,0.313790_rb,0.117074_rb, 1.60943e-02_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:,1) = (/ &
        & 3.40566E+01_rb, 3.27989E+01_rb, 3.11999E+01_rb, 2.82247E+01_rb,&
        & 2.47177E+01_rb, 2.15214E+01_rb, 1.74022E+01_rb, 1.30846E+01_rb,&
        & 9.44530E+00_rb, 1.03637E+00_rb, 8.57403E-01_rb, 6.79774E-01_rb,&
        & 4.99662E-01_rb, 3.23045E-01_rb, 1.21804E-01_rb, 1.70541E-02_rb/)
      irradnceo(:,2) = (/ &
        & 3.41026E+01_rb, 3.23263E+01_rb, 3.05619E+01_rb, 2.77705E+01_rb,&
        & 2.50954E+01_rb, 2.14057E+01_rb, 1.80581E+01_rb, 1.37399E+01_rb,&
        & 9.40607E+00_rb, 1.02770E+00_rb, 8.54246E-01_rb, 6.76237E-01_rb,&
        & 4.99670E-01_rb, 3.23045E-01_rb, 1.21804E-01_rb, 1.70541E-02_rb/)
      irradnceo(:,3) = (/ &
        & 3.41297E+01_rb, 3.20804E+01_rb, 3.04875E+01_rb, 2.77139E+01_rb,&
        & 2.48050E+01_rb, 2.19489E+01_rb, 1.81673E+01_rb, 1.37549E+01_rb,&
        & 9.39199E+00_rb, 1.02429E+00_rb, 8.49725E-01_rb, 6.75337E-01_rb,&
        & 4.95615E-01_rb, 3.23045E-01_rb, 1.21804E-01_rb, 1.70541E-02_rb/)
      irradnceo(:,4) = (/ &
        & 3.41313E+01_rb, 3.19856E+01_rb, 3.02667E+01_rb, 2.77211E+01_rb,&
        & 2.49851E+01_rb, 2.20569E+01_rb, 1.82127E+01_rb, 1.37562E+01_rb,&
        & 9.37321E+00_rb, 1.02438E+00_rb, 8.45560E-01_rb, 6.73680E-01_rb,&
        & 4.92139E-01_rb, 3.23044E-01_rb, 1.21804E-01_rb, 1.70541E-02_rb/)
      irradnceo(:,5) = (/ &
        & 3.40972E+01_rb, 3.19724E+01_rb, 3.01978E+01_rb, 2.75618E+01_rb,&
        & 2.52216E+01_rb, 2.21247E+01_rb, 1.82094E+01_rb, 1.37595E+01_rb,&
        & 9.35487E+00_rb, 1.02187E+00_rb, 8.45535E-01_rb, 6.68574E-01_rb,&
        & 4.93342E-01_rb, 3.18943E-01_rb, 1.21804E-01_rb, 1.70541E-02_rb/)
      irradnceo(:,6) = (/ &
        & 3.40386E+01_rb, 3.20289E+01_rb, 3.00114E+01_rb, 2.76645E+01_rb,&
        & 2.53088E+01_rb, 2.21485E+01_rb, 1.82090E+01_rb, 1.37616E+01_rb,&
        & 9.33863E+00_rb, 1.01797E+00_rb, 8.45288E-01_rb, 6.65491E-01_rb,&
        & 4.93134E-01_rb, 3.15694E-01_rb, 1.21804E-01_rb, 1.70541E-02_rb/)
      irradnceo(:,7) = (/ &
        & 3.39295E+01_rb, 3.20944E+01_rb, 2.99238E+01_rb, 2.77535E+01_rb,&
        & 2.53872E+01_rb, 2.21242E+01_rb, 1.82349E+01_rb, 1.37448E+01_rb,&
        & 9.32773E+00_rb, 1.01540E+00_rb, 8.44145E-01_rb, 6.62854E-01_rb,&
        & 4.93307E-01_rb, 3.14959E-01_rb, 1.18572E-01_rb, 1.70541E-02_rb/)
      irradnceo(:,8) = (/ &
        & 3.37599E+01_rb, 3.21370E+01_rb, 2.99582E+01_rb, 2.79343E+01_rb,&
        & 2.53426E+01_rb, 2.21431E+01_rb, 1.82001E+01_rb, 1.37448E+01_rb,&
        & 9.30758E+00_rb, 1.01532E+00_rb, 8.44469E-01_rb, 6.64363E-01_rb,&
        & 4.90104E-01_rb, 3.13708E-01_rb, 1.14840E-01_rb, 1.59848E-02_rb/)
      irradnceo(:,9) = (/ &
        & 3.43669E+01_rb, 3.23185E+01_rb, 3.04635E+01_rb, 2.75237E+01_rb,&
        & 2.49196E+01_rb, 2.18971E+01_rb, 1.80961E+01_rb, 1.36856E+01_rb,&
        & 9.28137E+00_rb, 1.01165E+00_rb, 8.34889E-01_rb, 6.62800E-01_rb,&
        & 4.82903E-01_rb, 3.10708E-01_rb, 1.15840E-01_rb, 1.53240E-02_rb/)
! Solar variability components: facular brightening
      facbrghto(:,1) = (/ &
        & 2.26866E-02_rb, 2.28221E-02_rb, 2.16403E-02_rb, 1.93032E-02_rb,&
        & 1.70330E-02_rb, 1.43461E-02_rb, 1.19685E-02_rb, 9.11456E-03_rb,&
        & 6.17266E-03_rb, 6.69966E-04_rb, 5.54740E-04_rb, 4.38622E-04_rb,&
        & 3.20265E-04_rb, 2.03716E-04_rb, 7.67337E-05_rb, 1.07563E-05_rb/)
      facbrghto(:,2) = (/ &
        & 2.26474E-02_rb, 2.22601E-02_rb, 2.09047E-02_rb, 1.90829E-02_rb,&
        & 1.71453E-02_rb, 1.49086E-02_rb, 1.24166E-02_rb, 9.38205E-03_rb,&
        & 6.31849E-03_rb, 6.81403E-04_rb, 5.62229E-04_rb, 4.40711E-04_rb,&
        & 3.20262E-04_rb, 2.03717E-04_rb, 7.67337E-05_rb, 1.07563E-05_rb/)
      facbrghto(:,3) = (/ &
        & 2.26331E-02_rb, 2.19821E-02_rb, 2.08551E-02_rb, 1.90466E-02_rb,&
        & 1.72159E-02_rb, 1.50922E-02_rb, 1.24752E-02_rb, 9.40153E-03_rb,&
        & 6.34286E-03_rb, 6.88893E-04_rb, 5.67322E-04_rb, 4.46851E-04_rb,&
        & 3.22927E-04_rb, 2.03717E-04_rb, 7.67337E-05_rb, 1.07563E-05_rb/)
      facbrghto(:,4) = (/ &
        & 2.25909E-02_rb, 2.18778E-02_rb, 2.07242E-02_rb, 1.91078E-02_rb,&
        & 1.73203E-02_rb, 1.51497E-02_rb, 1.24809E-02_rb, 9.41590E-03_rb,&
        & 6.36011E-03_rb, 6.93376E-04_rb, 5.71551E-04_rb, 4.50119E-04_rb,&
        & 3.27981E-04_rb, 2.03719E-04_rb, 7.67321E-05_rb, 1.07563E-05_rb/)
      facbrghto(:,5) = (/ &
        & 2.25576E-02_rb, 2.18299E-02_rb, 2.06491E-02_rb, 1.91336E-02_rb,&
        & 1.73909E-02_rb, 1.51643E-02_rb, 1.24867E-02_rb, 9.41759E-03_rb,&
        & 6.38363E-03_rb, 6.95969E-04_rb, 5.75312E-04_rb, 4.53322E-04_rb,&
        & 3.30661E-04_rb, 2.05788E-04_rb, 7.67337E-05_rb, 1.07563E-05_rb/)
      facbrghto(:,6) = (/ &
        & 2.25312E-02_rb, 2.18025E-02_rb, 2.05876E-02_rb, 1.91679E-02_rb,&
        & 1.74203E-02_rb, 1.51723E-02_rb, 1.24950E-02_rb, 9.41967E-03_rb,&
        & 6.40279E-03_rb, 6.99333E-04_rb, 5.78621E-04_rb, 4.55632E-04_rb,&
        & 3.32782E-04_rb, 2.08830E-04_rb, 7.67337E-05_rb, 1.07563E-05_rb/)
      facbrghto(:,7) = (/ &
        & 2.25071E-02_rb, 2.17666E-02_rb, 2.05631E-02_rb, 1.91898E-02_rb,&
        & 1.74277E-02_rb, 1.51802E-02_rb, 1.24892E-02_rb, 9.44172E-03_rb,&
        & 6.42039E-03_rb, 7.03006E-04_rb, 5.80404E-04_rb, 4.57992E-04_rb,&
        & 3.34317E-04_rb, 2.11576E-04_rb, 7.82141E-05_rb, 1.07563E-05_rb/)
      facbrghto(:,8) = (/ &
        & 2.24490E-02_rb, 2.17542E-02_rb, 2.05847E-02_rb, 1.91677E-02_rb,&
        & 1.74239E-02_rb, 1.51812E-02_rb, 1.24951E-02_rb, 9.47142E-03_rb,&
        & 6.44414E-03_rb, 7.05092E-04_rb, 5.82692E-04_rb, 4.59839E-04_rb,&
        & 3.36408E-04_rb, 2.13649E-04_rb, 8.13734E-05_rb, 1.14603E-05_rb/)
      facbrghto(:,9) = (/ &
        & 2.23361E-02_rb, 2.20452E-02_rb, 2.07790E-02_rb, 1.90364E-02_rb,&
        & 1.72455E-02_rb, 1.50846E-02_rb, 1.24949E-02_rb, 9.47626E-03_rb,&
        & 6.45396E-03_rb, 7.07322E-04_rb, 5.87193E-04_rb, 4.64037E-04_rb,&
        & 3.40845E-04_rb, 2.17138E-04_rb, 8.19530E-05_rb, 1.14698E-05_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:,1) = (/ &
        &-1.12694E-02_rb,-1.12257E-02_rb,-1.10882E-02_rb,-1.05213E-02_rb,&
        &-9.20404E-03_rb,-8.01291E-03_rb,-6.81855E-03_rb,-5.15428E-03_rb,&
        &-3.20971E-03_rb,-3.41584E-04_rb,-2.83625E-04_rb,-2.24653E-04_rb,&
        &-1.61258E-04_rb,-1.02501E-04_rb,-3.86114E-05_rb,-5.40370E-06_rb/)
      snsptdrko(:,2) = (/ &
        &-1.12586E-02_rb,-1.17234E-02_rb,-1.12007E-02_rb,-1.02423E-02_rb,&
        &-9.37200E-03_rb,-8.26111E-03_rb,-6.46423E-03_rb,-4.78670E-03_rb,&
        &-3.19070E-03_rb,-3.44178E-04_rb,-2.84995E-04_rb,-2.25033E-04_rb,&
        &-1.61253E-04_rb,-1.02501E-04_rb,-3.86114E-05_rb,-5.40370E-06_rb/)
      snsptdrko(:,3) = (/ &
        &-1.12610E-02_rb,-1.18543E-02_rb,-1.10818E-02_rb,-1.04173E-02_rb,&
        &-9.52794E-03_rb,-7.98175E-03_rb,-6.40546E-03_rb,-4.77182E-03_rb,&
        &-3.19388E-03_rb,-3.46414E-04_rb,-2.85868E-04_rb,-2.25712E-04_rb,&
        &-1.61963E-04_rb,-1.02501E-04_rb,-3.86114E-05_rb,-5.40370E-06_rb/)
      snsptdrko(:,4) = (/ &
        &-1.12869E-02_rb,-1.18819E-02_rb,-1.11247E-02_rb,-1.05150E-02_rb,&
        &-9.41617E-03_rb,-7.91160E-03_rb,-6.38843E-03_rb,-4.76753E-03_rb,&
        &-3.19942E-03_rb,-3.47670E-04_rb,-2.87019E-04_rb,-2.25301E-04_rb,&
        &-1.63603E-04_rb,-1.02502E-04_rb,-3.86109E-05_rb,-5.40370E-06_rb/)
      snsptdrko(:,5) = (/ &
        &-1.13331E-02_rb,-1.18571E-02_rb,-1.11913E-02_rb,-1.05878E-02_rb,&
        &-9.28017E-03_rb,-7.88910E-03_rb,-6.38055E-03_rb,-4.76028E-03_rb,&
        &-3.20902E-03_rb,-3.48800E-04_rb,-2.87791E-04_rb,-2.25066E-04_rb,&
        &-1.64394E-04_rb,-1.03245E-04_rb,-3.86114E-05_rb,-5.40370E-06_rb/)
      snsptdrko(:,6) = (/ &
        &-1.13821E-02_rb,-1.18212E-02_rb,-1.13138E-02_rb,-1.05115E-02_rb,&
        &-9.22975E-03_rb,-7.87470E-03_rb,-6.37837E-03_rb,-4.75722E-03_rb,&
        &-3.21687E-03_rb,-3.49564E-04_rb,-2.87488E-04_rb,-2.25767E-04_rb,&
        &-1.65075E-04_rb,-1.04254E-04_rb,-3.86114E-05_rb,-5.40370E-06_rb/)
      snsptdrko(:,7) = (/ &
        &-1.14453E-02_rb,-1.17859E-02_rb,-1.14074E-02_rb,-1.04151E-02_rb,&
        &-9.19942E-03_rb,-7.87318E-03_rb,-6.36874E-03_rb,-4.76520E-03_rb,&
        &-3.22239E-03_rb,-3.49013E-04_rb,-2.87967E-04_rb,-2.26720E-04_rb,&
        &-1.65615E-04_rb,-1.05071E-04_rb,-3.91471E-05_rb,-5.40370E-06_rb/)
      snsptdrko(:,8) = (/ &
        &-1.15426E-02_rb,-1.17519E-02_rb,-1.14088E-02_rb,-1.03539E-02_rb,&
        &-9.18402E-03_rb,-7.86234E-03_rb,-6.37601E-03_rb,-4.77385E-03_rb,&
        &-3.22479E-03_rb,-3.49545E-04_rb,-2.88626E-04_rb,-2.27414E-04_rb,&
        &-1.66350E-04_rb,-1.05679E-04_rb,-4.02017E-05_rb,-5.67264E-06_rb/)
      snsptdrko(:,9) = (/ &
        &-1.11732E-02_rb,-1.16700E-02_rb,-1.12183E-02_rb,-1.06520E-02_rb,&
        &-9.36341E-03_rb,-7.93408E-03_rb,-6.41969E-03_rb,-4.79861E-03_rb,&
        &-3.23889E-03_rb,-3.51995E-04_rb,-2.90944E-04_rb,-2.29695E-04_rb,&
        &-1.68002E-04_rb,-1.06828E-04_rb,-4.03572E-05_rb,-5.67676E-06_rb/)

! Rayleigh extinction coefficient at all v
      raylao(:,1) = (/ &
        & 1.28405e-07_rb,1.45501e-07_rb,1.67272e-07_rb,1.94856e-07_rb, &
        & 2.15248e-07_rb,2.34920e-07_rb,2.48558e-07_rb,1.80004e-07_rb, &
        & 1.46504e-07_rb,1.31355e-07_rb,1.33562e-07_rb,1.35618e-07_rb, &
        & 1.22412e-07_rb,1.19842e-07_rb,1.19924e-07_rb,1.20264e-07_rb /)
      raylao(:,2) = (/ &
        & 1.41622e-07_rb,1.93436e-07_rb,2.25057e-07_rb,2.01025e-07_rb, &
        & 1.85138e-07_rb,1.72672e-07_rb,1.64771e-07_rb,1.59312e-07_rb, &
        & 1.44961e-07_rb,1.37448e-07_rb,1.37506e-07_rb,1.38081e-07_rb, &
        & 1.22432e-07_rb,1.19844e-07_rb,1.19921e-07_rb,1.20287e-07_rb /)
      raylao(:,3) = (/ &
        & 1.45382e-07_rb,1.97020e-07_rb,2.22781e-07_rb,1.96062e-07_rb, &
        & 1.83495e-07_rb,1.72495e-07_rb,1.64910e-07_rb,1.58797e-07_rb, &
        & 1.46208e-07_rb,1.42274e-07_rb,1.40445e-07_rb,1.39496e-07_rb, &
        & 1.26940e-07_rb,1.19844e-07_rb,1.19921e-07_rb,1.20287e-07_rb /)
      raylao(:,4) = (/ &
        & 1.48247e-07_rb,1.99958e-07_rb,2.18048e-07_rb,1.93896e-07_rb, &
        & 1.83125e-07_rb,1.73244e-07_rb,1.64320e-07_rb,1.58298e-07_rb, &
        & 1.48428e-07_rb,1.44769e-07_rb,1.43704e-07_rb,1.38498e-07_rb, &
        & 1.31732e-07_rb,1.22299e-07_rb,1.19921e-07_rb,1.20287e-07_rb /)
      raylao(:,5) = (/ &
        & 1.51343e-07_rb,1.99621e-07_rb,2.14563e-07_rb,1.93824e-07_rb, &
        & 1.82992e-07_rb,1.73143e-07_rb,1.64587e-07_rb,1.57355e-07_rb, &
        & 1.51198e-07_rb,1.46373e-07_rb,1.45438e-07_rb,1.38095e-07_rb, &
        & 1.35026e-07_rb,1.27504e-07_rb,1.19921e-07_rb,1.20287e-07_rb /)
      raylao(:,6) = (/ &
        & 1.54462e-07_rb,1.97610e-07_rb,2.11992e-07_rb,1.93831e-07_rb, &
        & 1.83900e-07_rb,1.73125e-07_rb,1.64093e-07_rb,1.57651e-07_rb, &
        & 1.53158e-07_rb,1.46843e-07_rb,1.44733e-07_rb,1.40611e-07_rb, &
        & 1.37320e-07_rb,1.33932e-07_rb,1.20423e-07_rb,1.20287e-07_rb /)
      raylao(:,7) = (/ &
        & 1.59068e-07_rb,1.92757e-07_rb,2.09865e-07_rb,1.95132e-07_rb, &
        & 1.83641e-07_rb,1.73778e-07_rb,1.63215e-07_rb,1.59462e-07_rb, &
        & 1.54331e-07_rb,1.46177e-07_rb,1.45819e-07_rb,1.43177e-07_rb, &
        & 1.39797e-07_rb,1.36780e-07_rb,1.33385e-07_rb,1.20287e-07_rb /)
      raylao(:,8) = (/ &
        & 1.62066e-07_rb,1.87529e-07_rb,2.07191e-07_rb,1.97788e-07_rb, &
        & 1.84920e-07_rb,1.72951e-07_rb,1.65450e-07_rb,1.60344e-07_rb, &
        & 1.54403e-07_rb,1.47679e-07_rb,1.47287e-07_rb,1.44951e-07_rb, &
        & 1.42517e-07_rb,1.41107e-07_rb,1.48688e-07_rb,1.51127e-07_rb /)
      raylao(:,9) = (/ &
        & 1.19177e-07_rb,1.86522e-07_rb,2.20324e-07_rb,2.13543e-07_rb, &
        & 1.92198e-07_rb,1.81641e-07_rb,1.70092e-07_rb,1.65072e-07_rb, &
        & 1.59804e-07_rb,1.56745e-07_rb,1.51235e-07_rb,1.51400e-07_rb, &
        & 1.49635e-07_rb,1.48056e-07_rb,1.49046e-07_rb,1.51010e-07_rb /)

      raylbo(:) = (/ &
        & 1.23766e-07_rb,1.40524e-07_rb,1.61610e-07_rb,1.83232e-07_rb, &
        & 2.02951e-07_rb,2.21367e-07_rb,2.38367e-07_rb,2.53019e-07_rb, &
        & 2.12202e-07_rb,1.36977e-07_rb,1.39118e-07_rb,1.37097e-07_rb, &
        & 1.33223e-07_rb,1.38695e-07_rb,1.19868e-07_rb,1.20062e-07_rb /)

      abso3ao(:) = (/ &
        & 8.03067e-02_rb,0.180926_rb   ,0.227484_rb   ,0.168015_rb   , &
        & 0.138284_rb   ,0.114537_rb   ,9.50114e-02_rb,8.06816e-02_rb, &
        & 6.76406e-02_rb,5.69802e-02_rb,5.63283e-02_rb,4.57592e-02_rb, &
        & 4.21862e-02_rb,3.47949e-02_rb,2.65731e-02_rb,2.67628e-02_rb /)

      abso3bo(:) = (/ &
        & 2.94848e-02_rb,4.33642e-02_rb,6.70197e-02_rb,0.104990_rb   , &
        & 0.156180_rb   ,0.214638_rb   ,0.266281_rb   ,0.317941_rb   , &
        & 0.355327_rb   ,0.371241_rb   ,0.374396_rb   ,0.326847_rb   , &
        & 0.126497_rb   ,6.95264e-02_rb,2.58175e-02_rb,2.52862e-02_rb /)

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.73110e-09_rb,0.95257e-08_rb,0.84454e-08_rb,0.69367e-08_rb,0.55267e-08_rb, &
        & 0.41107e-08_rb,0.32800e-08_rb,0.33548e-08_rb,0.22111e-08_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.71850e-09_rb,0.98856e-08_rb,0.88084e-08_rb,0.72442e-08_rb,0.57586e-08_rb, &
        & 0.43475e-08_rb,0.34021e-08_rb,0.32425e-08_rb,0.21911e-08_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.57048e-09_rb,0.11944e-07_rb,0.10693e-07_rb,0.91787e-08_rb,0.75423e-08_rb, &
        & 0.56949e-08_rb,0.43124e-08_rb,0.42335e-08_rb,0.57788e-08_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.45283e-09_rb,0.10517e-07_rb,0.93142e-08_rb,0.77267e-08_rb,0.61927e-08_rb, &
        & 0.47485e-08_rb,0.34557e-08_rb,0.32691e-08_rb,0.21836e-08_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.40037e-09_rb,0.10599e-07_rb,0.93529e-08_rb,0.78411e-08_rb,0.63421e-08_rb, &
        & 0.49082e-08_rb,0.36312e-08_rb,0.32874e-08_rb,0.21861e-08_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.77671e-09_rb,0.11163e-07_rb,0.95032e-08_rb,0.79805e-08_rb,0.60721e-08_rb, &
        & 0.45440e-08_rb,0.31263e-08_rb,0.28631e-08_rb,0.19430e-08_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.85527e-09_rb,0.11271e-07_rb,0.97269e-08_rb,0.82280e-08_rb,0.62706e-08_rb, &
        & 0.47926e-08_rb,0.32802e-08_rb,0.28152e-08_rb,0.19155e-08_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.85773e-09_rb,0.13292e-07_rb,0.11943e-07_rb,0.10177e-07_rb,0.81637e-08_rb, &
        & 0.62054e-08_rb,0.44045e-08_rb,0.35504e-08_rb,0.48274e-08_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.76956e-09_rb,0.13343e-07_rb,0.12140e-07_rb,0.10403e-07_rb,0.83916e-08_rb, &
        & 0.64414e-08_rb,0.46212e-08_rb,0.36611e-08_rb,0.46523e-08_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.53187e-09_rb,0.11759e-07_rb,0.10564e-07_rb,0.89416e-08_rb,0.69566e-08_rb, &
        & 0.54336e-08_rb,0.36797e-08_rb,0.28238e-08_rb,0.18838e-08_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.99065e-09_rb,0.13223e-07_rb,0.11222e-07_rb,0.94282e-08_rb,0.74664e-08_rb, &
        & 0.54084e-08_rb,0.35205e-08_rb,0.24806e-08_rb,0.20205e-08_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.98793e-09_rb,0.13673e-07_rb,0.11444e-07_rb,0.96735e-08_rb,0.76364e-08_rb, &
        & 0.55809e-08_rb,0.37259e-08_rb,0.25419e-08_rb,0.20086e-08_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.10274e-08_rb,0.14243e-07_rb,0.12051e-07_rb,0.10168e-07_rb,0.80110e-08_rb, &
        & 0.59129e-08_rb,0.38720e-08_rb,0.26806e-08_rb,0.19959e-08_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.10879e-08_rb,0.15991e-07_rb,0.14290e-07_rb,0.12193e-07_rb,0.99943e-08_rb, &
        & 0.76411e-08_rb,0.49949e-08_rb,0.32117e-08_rb,0.45196e-08_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.10733e-08_rb,0.14460e-07_rb,0.12483e-07_rb,0.10703e-07_rb,0.84924e-08_rb, &
        & 0.62896e-08_rb,0.42753e-08_rb,0.27408e-08_rb,0.19679e-08_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.11948e-08_rb,0.15804e-07_rb,0.13423e-07_rb,0.10964e-07_rb,0.86345e-08_rb, &
        & 0.61421e-08_rb,0.39130e-08_rb,0.24271e-08_rb,0.26282e-08_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.12796e-08_rb,0.16985e-07_rb,0.14121e-07_rb,0.11784e-07_rb,0.93415e-08_rb, &
        & 0.67310e-08_rb,0.43124e-08_rb,0.25170e-08_rb,0.26033e-08_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.12903e-08_rb,0.17863e-07_rb,0.14703e-07_rb,0.12387e-07_rb,0.99243e-08_rb, &
        & 0.72283e-08_rb,0.46892e-08_rb,0.26092e-08_rb,0.25739e-08_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.13030e-08_rb,0.17982e-07_rb,0.14488e-07_rb,0.12389e-07_rb,0.10284e-07_rb, &
        & 0.76192e-08_rb,0.50049e-08_rb,0.27184e-08_rb,0.25450e-08_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.13295e-08_rb,0.20084e-07_rb,0.17213e-07_rb,0.15086e-07_rb,0.12255e-07_rb, &
        & 0.93192e-08_rb,0.61747e-08_rb,0.34151e-08_rb,0.50409e-08_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.14930e-08_rb,0.19740e-07_rb,0.16353e-07_rb,0.13382e-07_rb,0.10370e-07_rb, &
        & 0.76167e-08_rb,0.46913e-08_rb,0.23740e-08_rb,0.37254e-08_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.15750e-08_rb,0.20826e-07_rb,0.17352e-07_rb,0.14080e-07_rb,0.11109e-07_rb, &
        & 0.82293e-08_rb,0.51142e-08_rb,0.25900e-08_rb,0.36977e-08_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.16427e-08_rb,0.22313e-07_rb,0.18005e-07_rb,0.14545e-07_rb,0.11545e-07_rb, &
        & 0.85987e-08_rb,0.53849e-08_rb,0.27523e-08_rb,0.36628e-08_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.17106e-08_rb,0.23821e-07_rb,0.18882e-07_rb,0.15226e-07_rb,0.12313e-07_rb, &
        & 0.92367e-08_rb,0.58646e-08_rb,0.29020e-08_rb,0.36336e-08_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.17465e-08_rb,0.26750e-07_rb,0.22103e-07_rb,0.18690e-07_rb,0.15320e-07_rb, &
        & 0.11508e-07_rb,0.77281e-08_rb,0.34929e-08_rb,0.69801e-08_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.17561e-08_rb,0.17298e-07_rb,0.18888e-07_rb,0.16164e-07_rb,0.12676e-07_rb, &
        & 0.91128e-08_rb,0.57292e-08_rb,0.25945e-08_rb,0.52718e-08_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.19997e-08_rb,0.20866e-07_rb,0.20612e-07_rb,0.17467e-07_rb,0.13431e-07_rb, &
        & 0.96178e-08_rb,0.60009e-08_rb,0.28959e-08_rb,0.52609e-08_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.21808e-08_rb,0.24689e-07_rb,0.22590e-07_rb,0.18672e-07_rb,0.14573e-07_rb, &
        & 0.10599e-07_rb,0.66802e-08_rb,0.31104e-08_rb,0.52649e-08_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.22321e-08_rb,0.28043e-07_rb,0.23802e-07_rb,0.19473e-07_rb,0.15182e-07_rb, &
        & 0.11372e-07_rb,0.73223e-08_rb,0.34820e-08_rb,0.52977e-08_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.23033e-08_rb,0.30784e-07_rb,0.25188e-07_rb,0.20376e-07_rb,0.16019e-07_rb, &
        & 0.12009e-07_rb,0.77933e-08_rb,0.37322e-08_rb,0.52777e-08_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.22679e-08_rb,0.13507e-07_rb,0.20320e-07_rb,0.19549e-07_rb,0.15905e-07_rb, &
        & 0.11277e-07_rb,0.68710e-08_rb,0.29328e-08_rb,0.79703e-08_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.25718e-08_rb,0.16957e-07_rb,0.24375e-07_rb,0.21175e-07_rb,0.17080e-07_rb, &
        & 0.12233e-07_rb,0.77114e-08_rb,0.34113e-08_rb,0.79963e-08_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.27598e-08_rb,0.19896e-07_rb,0.28092e-07_rb,0.23059e-07_rb,0.18272e-07_rb, &
        & 0.13117e-07_rb,0.84449e-08_rb,0.37817e-08_rb,0.79824e-08_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.28979e-08_rb,0.22997e-07_rb,0.29556e-07_rb,0.24211e-07_rb,0.19134e-07_rb, &
        & 0.13799e-07_rb,0.90980e-08_rb,0.41849e-08_rb,0.79622e-08_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.31158e-08_rb,0.26193e-07_rb,0.32426e-07_rb,0.25685e-07_rb,0.19970e-07_rb, &
        & 0.14504e-07_rb,0.96379e-08_rb,0.45034e-08_rb,0.78914e-08_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.27286e-08_rb,0.10085e-07_rb,0.15014e-07_rb,0.19090e-07_rb,0.19831e-07_rb, &
        & 0.15054e-07_rb,0.91059e-08_rb,0.39324e-08_rb,0.14155e-07_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.30146e-08_rb,0.12680e-07_rb,0.19259e-07_rb,0.23896e-07_rb,0.21584e-07_rb, &
        & 0.16217e-07_rb,0.99712e-08_rb,0.44108e-08_rb,0.14243e-07_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.35571e-08_rb,0.15989e-07_rb,0.23328e-07_rb,0.28003e-07_rb,0.22999e-07_rb, &
        & 0.17316e-07_rb,0.10856e-07_rb,0.49279e-08_rb,0.13770e-07_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.40065e-08_rb,0.19481e-07_rb,0.27247e-07_rb,0.30922e-07_rb,0.24592e-07_rb, &
        & 0.18005e-07_rb,0.11331e-07_rb,0.53242e-08_rb,0.13758e-07_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.44047e-08_rb,0.23368e-07_rb,0.31337e-07_rb,0.33478e-07_rb,0.26467e-07_rb, &
        & 0.19093e-07_rb,0.12194e-07_rb,0.56088e-08_rb,0.13665e-07_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.32692e-08_rb,0.79127e-08_rb,0.11108e-07_rb,0.13962e-07_rb,0.15949e-07_rb, &
        & 0.18125e-07_rb,0.12367e-07_rb,0.53666e-08_rb,0.28048e-07_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.39872e-08_rb,0.10056e-07_rb,0.14185e-07_rb,0.17838e-07_rb,0.20262e-07_rb, &
        & 0.19824e-07_rb,0.13459e-07_rb,0.59111e-08_rb,0.29491e-07_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.46117e-08_rb,0.12472e-07_rb,0.17902e-07_rb,0.22130e-07_rb,0.25362e-07_rb, &
        & 0.21745e-07_rb,0.14226e-07_rb,0.62741e-08_rb,0.30373e-07_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.52024e-08_rb,0.15275e-07_rb,0.21923e-07_rb,0.27295e-07_rb,0.30210e-07_rb, &
        & 0.23240e-07_rb,0.15087e-07_rb,0.66869e-08_rb,0.31196e-07_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.56198e-08_rb,0.17975e-07_rb,0.26684e-07_rb,0.30762e-07_rb,0.34012e-07_rb, &
        & 0.25509e-07_rb,0.16208e-07_rb,0.72951e-08_rb,0.31583e-07_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.48178e-08_rb,0.70531e-08_rb,0.93321e-08_rb,0.10834e-07_rb,0.12386e-07_rb, &
        & 0.13718e-07_rb,0.14795e-07_rb,0.73464e-08_rb,0.35934e-07_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.55752e-08_rb,0.86985e-08_rb,0.11492e-07_rb,0.13823e-07_rb,0.15291e-07_rb, &
        & 0.17278e-07_rb,0.18153e-07_rb,0.81439e-08_rb,0.37177e-07_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.63274e-08_rb,0.10735e-07_rb,0.13985e-07_rb,0.17136e-07_rb,0.19303e-07_rb, &
        & 0.20940e-07_rb,0.20044e-07_rb,0.88008e-08_rb,0.38431e-07_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.70007e-08_rb,0.12858e-07_rb,0.17195e-07_rb,0.20946e-07_rb,0.23543e-07_rb, &
        & 0.25732e-07_rb,0.21748e-07_rb,0.95846e-08_rb,0.39768e-07_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.75858e-08_rb,0.15576e-07_rb,0.20803e-07_rb,0.25331e-07_rb,0.28222e-07_rb, &
        & 0.30974e-07_rb,0.23346e-07_rb,0.10532e-07_rb,0.41126e-07_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.71066e-08_rb,0.91105e-08_rb,0.91823e-08_rb,0.10078e-07_rb,0.11080e-07_rb, &
        & 0.11808e-07_rb,0.12592e-07_rb,0.10585e-07_rb,0.36646e-07_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.75651e-08_rb,0.10196e-07_rb,0.11288e-07_rb,0.12341e-07_rb,0.13264e-07_rb, &
        & 0.14635e-07_rb,0.15304e-07_rb,0.12113e-07_rb,0.37821e-07_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.84521e-08_rb,0.11641e-07_rb,0.13494e-07_rb,0.15134e-07_rb,0.16494e-07_rb, &
        & 0.17667e-07_rb,0.18637e-07_rb,0.13998e-07_rb,0.39151e-07_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.96535e-08_rb,0.13091e-07_rb,0.16210e-07_rb,0.18132e-07_rb,0.20108e-07_rb, &
        & 0.21506e-07_rb,0.22578e-07_rb,0.15372e-07_rb,0.40534e-07_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.10866e-07_rb,0.15259e-07_rb,0.19222e-07_rb,0.21816e-07_rb,0.23808e-07_rb, &
        & 0.25779e-07_rb,0.26640e-07_rb,0.16560e-07_rb,0.41783e-07_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.80834e-08_rb,0.10420e-07_rb,0.92387e-08_rb,0.95448e-08_rb,0.10004e-07_rb, &
        & 0.10210e-07_rb,0.10715e-07_rb,0.12956e-07_rb,0.37307e-07_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.93300e-08_rb,0.11721e-07_rb,0.10935e-07_rb,0.11240e-07_rb,0.12099e-07_rb, &
        & 0.12529e-07_rb,0.12978e-07_rb,0.14968e-07_rb,0.38619e-07_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.10509e-07_rb,0.12830e-07_rb,0.12793e-07_rb,0.13649e-07_rb,0.14634e-07_rb, &
        & 0.15024e-07_rb,0.15635e-07_rb,0.17087e-07_rb,0.40001e-07_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.11973e-07_rb,0.14173e-07_rb,0.14811e-07_rb,0.16236e-07_rb,0.17652e-07_rb, &
        & 0.18139e-07_rb,0.18897e-07_rb,0.19167e-07_rb,0.41426e-07_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.13412e-07_rb,0.16145e-07_rb,0.17438e-07_rb,0.18946e-07_rb,0.20873e-07_rb, &
        & 0.21649e-07_rb,0.22446e-07_rb,0.20222e-07_rb,0.42866e-07_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.96752e-08_rb,0.12525e-07_rb,0.10944e-07_rb,0.96402e-08_rb,0.92613e-08_rb, &
        & 0.89183e-08_rb,0.91757e-08_rb,0.10962e-07_rb,0.37700e-07_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.10573e-07_rb,0.14013e-07_rb,0.12095e-07_rb,0.11243e-07_rb,0.11108e-07_rb, &
        & 0.10880e-07_rb,0.11099e-07_rb,0.12700e-07_rb,0.39054e-07_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.12096e-07_rb,0.15476e-07_rb,0.13719e-07_rb,0.13357e-07_rb,0.13060e-07_rb, &
        & 0.13122e-07_rb,0.13232e-07_rb,0.14320e-07_rb,0.40767e-07_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.13461e-07_rb,0.16674e-07_rb,0.15454e-07_rb,0.15611e-07_rb,0.15616e-07_rb, &
        & 0.15505e-07_rb,0.16233e-07_rb,0.16510e-07_rb,0.42192e-07_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.14863e-07_rb,0.17307e-07_rb,0.17245e-07_rb,0.18041e-07_rb,0.18314e-07_rb, &
        & 0.18445e-07_rb,0.19222e-07_rb,0.18635e-07_rb,0.43353e-07_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.11698e-08_rb,0.30848e-07_rb,0.36454e-07_rb,0.38710e-07_rb,0.45901e-07_rb, &
        & 0.55227e-07_rb,0.66976e-07_rb,0.88407e-07_rb,0.23422e-07_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.11855e-08_rb,0.31924e-07_rb,0.37139e-07_rb,0.40576e-07_rb,0.48515e-07_rb, &
        & 0.57453e-07_rb,0.68267e-07_rb,0.87661e-07_rb,0.23548e-07_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.14595e-08_rb,0.39594e-07_rb,0.49562e-07_rb,0.62806e-07_rb,0.77187e-07_rb, &
        & 0.93412e-07_rb,0.11204e-06_rb,0.13944e-06_rb,0.36327e-07_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.16033e-08_rb,0.32903e-07_rb,0.39616e-07_rb,0.43600e-07_rb,0.51878e-07_rb, &
        & 0.61189e-07_rb,0.71838e-07_rb,0.88579e-07_rb,0.23754e-07_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.16713e-08_rb,0.33816e-07_rb,0.40271e-07_rb,0.44761e-07_rb,0.53112e-07_rb, &
        & 0.62817e-07_rb,0.73214e-07_rb,0.89918e-07_rb,0.23712e-07_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.14413e-08_rb,0.29542e-07_rb,0.33672e-07_rb,0.38233e-07_rb,0.38429e-07_rb, &
        & 0.45175e-07_rb,0.55138e-07_rb,0.73579e-07_rb,0.25050e-07_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.13819e-08_rb,0.33839e-07_rb,0.35660e-07_rb,0.39637e-07_rb,0.40862e-07_rb, &
        & 0.47496e-07_rb,0.56716e-07_rb,0.72256e-07_rb,0.25722e-07_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.14034e-08_rb,0.38888e-07_rb,0.49224e-07_rb,0.54694e-07_rb,0.64529e-07_rb, &
        & 0.76758e-07_rb,0.91615e-07_rb,0.11521e-06_rb,0.39473e-07_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.15427e-08_rb,0.40736e-07_rb,0.49562e-07_rb,0.55626e-07_rb,0.65520e-07_rb, &
        & 0.77421e-07_rb,0.92434e-07_rb,0.11487e-06_rb,0.39089e-07_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.19146e-08_rb,0.37020e-07_rb,0.37998e-07_rb,0.40828e-07_rb,0.46417e-07_rb, &
        & 0.52511e-07_rb,0.60306e-07_rb,0.73583e-07_rb,0.26882e-07_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.16720e-08_rb,0.23360e-07_rb,0.34106e-07_rb,0.35883e-07_rb,0.37371e-07_rb, &
        & 0.37719e-07_rb,0.44564e-07_rb,0.59521e-07_rb,0.60171e-07_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.17699e-08_rb,0.26546e-07_rb,0.36000e-07_rb,0.36846e-07_rb,0.38381e-07_rb, &
        & 0.39667e-07_rb,0.46368e-07_rb,0.60306e-07_rb,0.60604e-07_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.18009e-08_rb,0.29450e-07_rb,0.37614e-07_rb,0.38630e-07_rb,0.39041e-07_rb, &
        & 0.41600e-07_rb,0.48142e-07_rb,0.59638e-07_rb,0.61274e-07_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.18222e-08_rb,0.39219e-07_rb,0.48109e-07_rb,0.53485e-07_rb,0.57150e-07_rb, &
        & 0.64444e-07_rb,0.75574e-07_rb,0.94813e-07_rb,0.76759e-07_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.18210e-08_rb,0.34020e-07_rb,0.40366e-07_rb,0.41710e-07_rb,0.42164e-07_rb, &
        & 0.45001e-07_rb,0.49878e-07_rb,0.60009e-07_rb,0.61481e-07_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.21152e-08_rb,0.23706e-07_rb,0.29784e-07_rb,0.35302e-07_rb,0.35282e-07_rb, &
        & 0.34361e-07_rb,0.35938e-07_rb,0.47065e-07_rb,0.17050e-06_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.22014e-08_rb,0.24318e-07_rb,0.33239e-07_rb,0.37390e-07_rb,0.36975e-07_rb, &
        & 0.35774e-07_rb,0.38207e-07_rb,0.49025e-07_rb,0.17312e-06_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.21940e-08_rb,0.26005e-07_rb,0.37067e-07_rb,0.38353e-07_rb,0.38894e-07_rb, &
        & 0.37527e-07_rb,0.40105e-07_rb,0.50082e-07_rb,0.17777e-06_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.23140e-08_rb,0.28378e-07_rb,0.41743e-07_rb,0.40523e-07_rb,0.40492e-07_rb, &
        & 0.38652e-07_rb,0.40876e-07_rb,0.49640e-07_rb,0.18336e-06_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.23213e-08_rb,0.34154e-07_rb,0.50461e-07_rb,0.52357e-07_rb,0.54187e-07_rb, &
        & 0.57113e-07_rb,0.62843e-07_rb,0.77605e-07_rb,0.21695e-06_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.26738e-08_rb,0.26329e-07_rb,0.23971e-07_rb,0.31112e-07_rb,0.35671e-07_rb, &
        & 0.34087e-07_rb,0.29799e-07_rb,0.37789e-07_rb,0.52514e-06_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.25455e-08_rb,0.28564e-07_rb,0.26376e-07_rb,0.34876e-07_rb,0.38195e-07_rb, &
        & 0.36126e-07_rb,0.31889e-07_rb,0.38787e-07_rb,0.53894e-06_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.27955e-08_rb,0.30168e-07_rb,0.29598e-07_rb,0.39256e-07_rb,0.39488e-07_rb, &
        & 0.38121e-07_rb,0.34110e-07_rb,0.40225e-07_rb,0.55617e-06_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.30518e-08_rb,0.31696e-07_rb,0.32642e-07_rb,0.43581e-07_rb,0.40810e-07_rb, &
        & 0.38887e-07_rb,0.35568e-07_rb,0.41837e-07_rb,0.57141e-06_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.29545e-08_rb,0.34626e-07_rb,0.42873e-07_rb,0.52742e-07_rb,0.53043e-07_rb, &
        & 0.52428e-07_rb,0.54304e-07_rb,0.64247e-07_rb,0.63906e-06_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.30127e-08_rb,0.42606e-07_rb,0.28980e-07_rb,0.24923e-07_rb,0.29265e-07_rb, &
        & 0.34534e-07_rb,0.31090e-07_rb,0.30814e-07_rb,0.12844e-05_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.33504e-08_rb,0.42102e-07_rb,0.30491e-07_rb,0.27666e-07_rb,0.33477e-07_rb, &
        & 0.35643e-07_rb,0.33324e-07_rb,0.31448e-07_rb,0.13231e-05_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.37714e-08_rb,0.40516e-07_rb,0.32080e-07_rb,0.30623e-07_rb,0.37391e-07_rb, &
        & 0.37255e-07_rb,0.35144e-07_rb,0.32296e-07_rb,0.13598e-05_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.40809e-08_rb,0.41419e-07_rb,0.33638e-07_rb,0.33756e-07_rb,0.41072e-07_rb, &
        & 0.38814e-07_rb,0.36264e-07_rb,0.33664e-07_rb,0.13955e-05_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.38112e-08_rb,0.42764e-07_rb,0.35694e-07_rb,0.36747e-07_rb,0.45193e-07_rb, &
        & 0.40460e-07_rb,0.37371e-07_rb,0.35107e-07_rb,0.14282e-05_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.36577e-08_rb,0.64857e-07_rb,0.37268e-07_rb,0.28977e-07_rb,0.24480e-07_rb, &
        & 0.26864e-07_rb,0.30486e-07_rb,0.25122e-07_rb,0.20521e-05_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.39427e-08_rb,0.65731e-07_rb,0.37248e-07_rb,0.30585e-07_rb,0.26773e-07_rb, &
        & 0.30114e-07_rb,0.31576e-07_rb,0.26187e-07_rb,0.21480e-05_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.43811e-08_rb,0.66809e-07_rb,0.38170e-07_rb,0.32043e-07_rb,0.28837e-07_rb, &
        & 0.34005e-07_rb,0.32765e-07_rb,0.27322e-07_rb,0.22122e-05_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.46591e-08_rb,0.68244e-07_rb,0.41020e-07_rb,0.34183e-07_rb,0.31347e-07_rb, &
        & 0.37946e-07_rb,0.34473e-07_rb,0.28264e-07_rb,0.22876e-05_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.51635e-08_rb,0.69768e-07_rb,0.42241e-07_rb,0.36125e-07_rb,0.34689e-07_rb, &
        & 0.41422e-07_rb,0.36208e-07_rb,0.29319e-07_rb,0.23528e-05_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.48063e-08_rb,0.99492e-07_rb,0.67977e-07_rb,0.40583e-07_rb,0.27462e-07_rb, &
        & 0.23261e-07_rb,0.24790e-07_rb,0.24175e-07_rb,0.34404e-05_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.46794e-08_rb,0.10207e-06_rb,0.68838e-07_rb,0.41170e-07_rb,0.30068e-07_rb, &
        & 0.24536e-07_rb,0.27696e-07_rb,0.25318e-07_rb,0.35579e-05_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.51560e-08_rb,0.10436e-06_rb,0.69782e-07_rb,0.42105e-07_rb,0.32065e-07_rb, &
        & 0.26776e-07_rb,0.30613e-07_rb,0.26859e-07_rb,0.36807e-05_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.59944e-08_rb,0.10484e-06_rb,0.71027e-07_rb,0.44319e-07_rb,0.34843e-07_rb, &
        & 0.29788e-07_rb,0.33876e-07_rb,0.28922e-07_rb,0.37740e-05_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.64682e-08_rb,0.10541e-06_rb,0.71736e-07_rb,0.46124e-07_rb,0.36934e-07_rb, &
        & 0.32173e-07_rb,0.36502e-07_rb,0.30991e-07_rb,0.38805e-05_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.56433e-08_rb,0.13757e-06_rb,0.10641e-06_rb,0.72870e-07_rb,0.43453e-07_rb, &
        & 0.25530e-07_rb,0.19778e-07_rb,0.25383e-07_rb,0.95912e-05_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.61882e-08_rb,0.14041e-06_rb,0.11046e-06_rb,0.75424e-07_rb,0.44204e-07_rb, &
        & 0.28513e-07_rb,0.22344e-07_rb,0.27258e-07_rb,0.98257e-05_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.68724e-08_rb,0.14355e-06_rb,0.11272e-06_rb,0.76977e-07_rb,0.44934e-07_rb, &
        & 0.30972e-07_rb,0.24609e-07_rb,0.29074e-07_rb,0.10035e-04_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.75288e-08_rb,0.14628e-06_rb,0.11397e-06_rb,0.77508e-07_rb,0.45913e-07_rb, &
        & 0.33498e-07_rb,0.27308e-07_rb,0.31237e-07_rb,0.10206e-04_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.82361e-08_rb,0.14989e-06_rb,0.11575e-06_rb,0.78052e-07_rb,0.47282e-07_rb, &
        & 0.34986e-07_rb,0.29571e-07_rb,0.31933e-07_rb,0.10332e-04_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.80705e-08_rb,0.18916e-06_rb,0.16332e-06_rb,0.12182e-06_rb,0.80167e-07_rb, &
        & 0.44659e-07_rb,0.24027e-07_rb,0.20514e-07_rb,0.18076e-04_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.86522e-08_rb,0.19825e-06_rb,0.16839e-06_rb,0.12817e-06_rb,0.85984e-07_rb, &
        & 0.48117e-07_rb,0.24937e-07_rb,0.22855e-07_rb,0.18392e-04_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.93838e-08_rb,0.20474e-06_rb,0.17472e-06_rb,0.13185e-06_rb,0.89897e-07_rb, &
        & 0.50484e-07_rb,0.27069e-07_rb,0.24729e-07_rb,0.18827e-04_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.10442e-07_rb,0.21110e-06_rb,0.17958e-06_rb,0.13546e-06_rb,0.92462e-07_rb, &
        & 0.51368e-07_rb,0.29958e-07_rb,0.26382e-07_rb,0.19325e-04_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.11203e-07_rb,0.21676e-06_rb,0.18359e-06_rb,0.14012e-06_rb,0.94090e-07_rb, &
        & 0.51431e-07_rb,0.32637e-07_rb,0.27900e-07_rb,0.19654e-04_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.10898e-07_rb,0.20490e-06_rb,0.23352e-06_rb,0.18592e-06_rb,0.13108e-06_rb, &
        & 0.82594e-07_rb,0.41478e-07_rb,0.20359e-07_rb,0.24656e-04_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.12100e-07_rb,0.21595e-06_rb,0.24259e-06_rb,0.19598e-06_rb,0.14307e-06_rb, &
        & 0.90287e-07_rb,0.44897e-07_rb,0.21400e-07_rb,0.25439e-04_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.12997e-07_rb,0.22559e-06_rb,0.25028e-06_rb,0.20624e-06_rb,0.15074e-06_rb, &
        & 0.95763e-07_rb,0.46972e-07_rb,0.22656e-07_rb,0.26128e-04_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.12774e-07_rb,0.23494e-06_rb,0.26130e-06_rb,0.21398e-06_rb,0.15730e-06_rb, &
        & 0.98985e-07_rb,0.48309e-07_rb,0.24026e-07_rb,0.26834e-04_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.13184e-07_rb,0.24219e-06_rb,0.26811e-06_rb,0.22059e-06_rb,0.16314e-06_rb, &
        & 0.10271e-06_rb,0.49802e-07_rb,0.25441e-07_rb,0.27644e-04_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.11394e-07_rb,0.19172e-06_rb,0.27076e-06_rb,0.23579e-06_rb,0.17648e-06_rb, &
        & 0.11366e-06_rb,0.56463e-07_rb,0.20869e-07_rb,0.27151e-04_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.12908e-07_rb,0.20309e-06_rb,0.28462e-06_rb,0.24753e-06_rb,0.18719e-06_rb, &
        & 0.12033e-06_rb,0.60157e-07_rb,0.21646e-07_rb,0.27962e-04_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.13752e-07_rb,0.21619e-06_rb,0.29476e-06_rb,0.25594e-06_rb,0.19641e-06_rb, &
        & 0.12734e-06_rb,0.64272e-07_rb,0.22557e-07_rb,0.28912e-04_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.14906e-07_rb,0.22665e-06_rb,0.30298e-06_rb,0.26272e-06_rb,0.20331e-06_rb, &
        & 0.13316e-06_rb,0.68243e-07_rb,0.23417e-07_rb,0.29816e-04_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.15969e-07_rb,0.23477e-06_rb,0.31068e-06_rb,0.27029e-06_rb,0.20850e-06_rb, &
        & 0.13902e-06_rb,0.71857e-07_rb,0.25264e-07_rb,0.30765e-04_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.13593e-07_rb,0.19801e-06_rb,0.26490e-06_rb,0.27826e-06_rb,0.22531e-06_rb, &
        & 0.15112e-06_rb,0.78691e-07_rb,0.27273e-07_rb,0.28055e-04_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.15426e-07_rb,0.20839e-06_rb,0.27732e-06_rb,0.28785e-06_rb,0.23371e-06_rb, &
        & 0.16027e-06_rb,0.86131e-07_rb,0.29011e-07_rb,0.29165e-04_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.16638e-07_rb,0.21808e-06_rb,0.28896e-06_rb,0.29666e-06_rb,0.24201e-06_rb, &
        & 0.16845e-06_rb,0.91791e-07_rb,0.30670e-07_rb,0.30372e-04_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.17929e-07_rb,0.22577e-06_rb,0.29798e-06_rb,0.30533e-06_rb,0.24895e-06_rb, &
        & 0.17573e-06_rb,0.96085e-07_rb,0.31360e-07_rb,0.31437e-04_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.18861e-07_rb,0.23397e-06_rb,0.30715e-06_rb,0.31382e-06_rb,0.25665e-06_rb, &
        & 0.18246e-06_rb,0.10017e-06_rb,0.32454e-07_rb,0.32266e-04_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.39751e-08_rb,0.10094e-06_rb,0.20247e-06_rb,0.30028e-06_rb,0.39102e-06_rb, &
        & 0.48711e-06_rb,0.59106e-06_rb,0.72709e-06_rb,0.20885e-06_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.43080e-08_rb,0.10546e-06_rb,0.20982e-06_rb,0.31135e-06_rb,0.40303e-06_rb, &
        & 0.49632e-06_rb,0.59855e-06_rb,0.74477e-06_rb,0.21012e-06_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.42986e-08_rb,0.15269e-06_rb,0.29890e-06_rb,0.43493e-06_rb,0.56429e-06_rb, &
        & 0.69336e-06_rb,0.83181e-06_rb,0.10423e-05_rb,0.36557e-06_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.43554e-08_rb,0.11271e-06_rb,0.22069e-06_rb,0.32651e-06_rb,0.42391e-06_rb, &
        & 0.51393e-06_rb,0.61294e-06_rb,0.75229e-06_rb,0.20945e-06_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.45047e-08_rb,0.11576e-06_rb,0.22657e-06_rb,0.33173e-06_rb,0.43142e-06_rb, &
        & 0.52151e-06_rb,0.61871e-06_rb,0.74980e-06_rb,0.20891e-06_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.46065e-08_rb,0.81519e-07_rb,0.15960e-06_rb,0.23610e-06_rb,0.31656e-06_rb, &
        & 0.39839e-06_rb,0.48591e-06_rb,0.60182e-06_rb,0.18230e-06_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.46837e-08_rb,0.82106e-07_rb,0.16429e-06_rb,0.24471e-06_rb,0.32626e-06_rb, &
        & 0.40531e-06_rb,0.49330e-06_rb,0.61810e-06_rb,0.18355e-06_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.50163e-08_rb,0.12328e-06_rb,0.23602e-06_rb,0.35332e-06_rb,0.46387e-06_rb, &
        & 0.57515e-06_rb,0.69445e-06_rb,0.88167e-06_rb,0.30546e-06_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.49809e-08_rb,0.12454e-06_rb,0.23839e-06_rb,0.35429e-06_rb,0.46515e-06_rb, &
        & 0.57491e-06_rb,0.69031e-06_rb,0.86630e-06_rb,0.29807e-06_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.51988e-08_rb,0.91291e-07_rb,0.17869e-06_rb,0.26485e-06_rb,0.34907e-06_rb, &
        & 0.42509e-06_rb,0.50894e-06_rb,0.62361e-06_rb,0.18234e-06_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.51916e-08_rb,0.88867e-07_rb,0.12220e-06_rb,0.18411e-06_rb,0.24720e-06_rb, &
        & 0.32303e-06_rb,0.39526e-06_rb,0.49595e-06_rb,0.27125e-06_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.56319e-08_rb,0.87189e-07_rb,0.12678e-06_rb,0.19176e-06_rb,0.25624e-06_rb, &
        & 0.32740e-06_rb,0.40564e-06_rb,0.50601e-06_rb,0.28010e-06_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.59026e-08_rb,0.86017e-07_rb,0.13043e-06_rb,0.19649e-06_rb,0.26497e-06_rb, &
        & 0.33305e-06_rb,0.40679e-06_rb,0.51963e-06_rb,0.28660e-06_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.59909e-08_rb,0.10759e-06_rb,0.18790e-06_rb,0.28104e-06_rb,0.37702e-06_rb, &
        & 0.47222e-06_rb,0.57322e-06_rb,0.72906e-06_rb,0.37657e-06_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.64422e-08_rb,0.89472e-07_rb,0.13687e-06_rb,0.20453e-06_rb,0.27802e-06_rb, &
        & 0.34266e-06_rb,0.41565e-06_rb,0.51746e-06_rb,0.29675e-06_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.62298e-08_rb,0.99509e-07_rb,0.10306e-06_rb,0.14232e-06_rb,0.19390e-06_rb, &
        & 0.25435e-06_rb,0.31764e-06_rb,0.40578e-06_rb,0.49031e-06_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.68419e-08_rb,0.10415e-06_rb,0.10349e-06_rb,0.14672e-06_rb,0.19860e-06_rb, &
        & 0.26012e-06_rb,0.32720e-06_rb,0.41327e-06_rb,0.51522e-06_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.71418e-08_rb,0.10626e-06_rb,0.10522e-06_rb,0.15175e-06_rb,0.20471e-06_rb, &
        & 0.26435e-06_rb,0.33148e-06_rb,0.42294e-06_rb,0.53456e-06_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.72470e-08_rb,0.10891e-06_rb,0.10648e-06_rb,0.15457e-06_rb,0.21007e-06_rb, &
        & 0.26937e-06_rb,0.33376e-06_rb,0.42939e-06_rb,0.55207e-06_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.77585e-08_rb,0.12954e-06_rb,0.14999e-06_rb,0.22224e-06_rb,0.29994e-06_rb, &
        & 0.38053e-06_rb,0.47027e-06_rb,0.60032e-06_rb,0.76592e-06_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.68852e-08_rb,0.11288e-06_rb,0.11969e-06_rb,0.11689e-06_rb,0.14994e-06_rb, &
        & 0.19637e-06_rb,0.25309e-06_rb,0.33018e-06_rb,0.84551e-06_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.75119e-08_rb,0.11838e-06_rb,0.12255e-06_rb,0.11926e-06_rb,0.15341e-06_rb, &
        & 0.20169e-06_rb,0.26231e-06_rb,0.33745e-06_rb,0.85714e-06_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.84416e-08_rb,0.12128e-06_rb,0.12324e-06_rb,0.12015e-06_rb,0.15812e-06_rb, &
        & 0.20463e-06_rb,0.26808e-06_rb,0.34461e-06_rb,0.86734e-06_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.92293e-08_rb,0.12493e-06_rb,0.12330e-06_rb,0.12093e-06_rb,0.16146e-06_rb, &
        & 0.20899e-06_rb,0.26904e-06_rb,0.35101e-06_rb,0.88204e-06_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.10279e-07_rb,0.14631e-06_rb,0.14078e-06_rb,0.17511e-06_rb,0.23546e-06_rb, &
        & 0.30442e-06_rb,0.38374e-06_rb,0.50000e-06_rb,0.10431e-05_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.51921e-08_rb,0.14437e-06_rb,0.12917e-06_rb,0.11673e-06_rb,0.12444e-06_rb, &
        & 0.14966e-06_rb,0.19338e-06_rb,0.26674e-06_rb,0.16987e-05_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.69052e-08_rb,0.14822e-06_rb,0.13564e-06_rb,0.12080e-06_rb,0.12884e-06_rb, &
        & 0.15616e-06_rb,0.20083e-06_rb,0.27373e-06_rb,0.17147e-05_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.93932e-08_rb,0.15309e-06_rb,0.13785e-06_rb,0.12361e-06_rb,0.12904e-06_rb, &
        & 0.16004e-06_rb,0.20790e-06_rb,0.27928e-06_rb,0.17384e-05_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.10364e-07_rb,0.15577e-06_rb,0.14281e-06_rb,0.12445e-06_rb,0.13032e-06_rb, &
        & 0.16178e-06_rb,0.21225e-06_rb,0.28499e-06_rb,0.17733e-05_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.10950e-07_rb,0.15768e-06_rb,0.14728e-06_rb,0.12674e-06_rb,0.12961e-06_rb, &
        & 0.16455e-06_rb,0.21080e-06_rb,0.28744e-06_rb,0.17869e-05_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.51527e-08_rb,0.13331e-06_rb,0.15220e-06_rb,0.13781e-06_rb,0.11791e-06_rb, &
        & 0.12291e-06_rb,0.14839e-06_rb,0.21430e-06_rb,0.45802e-05_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.52409e-08_rb,0.14109e-06_rb,0.15809e-06_rb,0.14393e-06_rb,0.12362e-06_rb, &
        & 0.12831e-06_rb,0.15572e-06_rb,0.22059e-06_rb,0.46088e-05_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.66901e-08_rb,0.14833e-06_rb,0.16524e-06_rb,0.14893e-06_rb,0.12670e-06_rb, &
        & 0.13152e-06_rb,0.16110e-06_rb,0.22404e-06_rb,0.46200e-05_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.90836e-08_rb,0.15267e-06_rb,0.17113e-06_rb,0.15541e-06_rb,0.12966e-06_rb, &
        & 0.13240e-06_rb,0.16410e-06_rb,0.22868e-06_rb,0.45999e-05_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.11721e-07_rb,0.15605e-06_rb,0.17580e-06_rb,0.16064e-06_rb,0.13194e-06_rb, &
        & 0.13127e-06_rb,0.16676e-06_rb,0.23245e-06_rb,0.46103e-05_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.53770e-08_rb,0.12906e-06_rb,0.19117e-06_rb,0.16928e-06_rb,0.14454e-06_rb, &
        & 0.11318e-06_rb,0.12091e-06_rb,0.16771e-06_rb,0.10838e-04_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.71217e-08_rb,0.13171e-06_rb,0.20115e-06_rb,0.17525e-06_rb,0.15189e-06_rb, &
        & 0.11980e-06_rb,0.12589e-06_rb,0.17247e-06_rb,0.10844e-04_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.76051e-08_rb,0.13463e-06_rb,0.20837e-06_rb,0.18092e-06_rb,0.15831e-06_rb, &
        & 0.12491e-06_rb,0.12782e-06_rb,0.17568e-06_rb,0.10855e-04_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.87276e-08_rb,0.14002e-06_rb,0.21488e-06_rb,0.18616e-06_rb,0.16272e-06_rb, &
        & 0.12991e-06_rb,0.13139e-06_rb,0.17775e-06_rb,0.10873e-04_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.98102e-08_rb,0.14390e-06_rb,0.21715e-06_rb,0.19383e-06_rb,0.16843e-06_rb, &
        & 0.13102e-06_rb,0.13320e-06_rb,0.18001e-06_rb,0.10883e-04_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.78846e-08_rb,0.14921e-06_rb,0.16266e-06_rb,0.21810e-06_rb,0.17257e-06_rb, &
        & 0.13914e-06_rb,0.10621e-06_rb,0.13009e-06_rb,0.22193e-04_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.87670e-08_rb,0.15667e-06_rb,0.17179e-06_rb,0.22870e-06_rb,0.18453e-06_rb, &
        & 0.14806e-06_rb,0.11198e-06_rb,0.13228e-06_rb,0.22470e-04_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.94042e-08_rb,0.16331e-06_rb,0.17929e-06_rb,0.23732e-06_rb,0.19309e-06_rb, &
        & 0.15669e-06_rb,0.11769e-06_rb,0.13462e-06_rb,0.22759e-04_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.10052e-07_rb,0.16738e-06_rb,0.18654e-06_rb,0.24428e-06_rb,0.19714e-06_rb, &
        & 0.16469e-06_rb,0.12194e-06_rb,0.13694e-06_rb,0.23037e-04_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.99609e-08_rb,0.16974e-06_rb,0.19096e-06_rb,0.25212e-06_rb,0.20604e-06_rb, &
        & 0.17005e-06_rb,0.12529e-06_rb,0.13965e-06_rb,0.23331e-04_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.10600e-07_rb,0.19194e-06_rb,0.19694e-06_rb,0.19228e-06_rb,0.23306e-06_rb, &
        & 0.18017e-06_rb,0.13563e-06_rb,0.11106e-06_rb,0.57833e-04_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.11513e-07_rb,0.19670e-06_rb,0.20600e-06_rb,0.20153e-06_rb,0.24712e-06_rb, &
        & 0.19420e-06_rb,0.14374e-06_rb,0.11223e-06_rb,0.59252e-04_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.12267e-07_rb,0.20052e-06_rb,0.21125e-06_rb,0.21502e-06_rb,0.25879e-06_rb, &
        & 0.20809e-06_rb,0.15257e-06_rb,0.11412e-06_rb,0.60474e-04_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.11377e-07_rb,0.20485e-06_rb,0.21479e-06_rb,0.22527e-06_rb,0.26884e-06_rb, &
        & 0.21723e-06_rb,0.15872e-06_rb,0.11733e-06_rb,0.61380e-04_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.12141e-07_rb,0.20611e-06_rb,0.22439e-06_rb,0.22986e-06_rb,0.27411e-06_rb, &
        & 0.22402e-06_rb,0.16342e-06_rb,0.11916e-06_rb,0.62353e-04_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.12251e-07_rb,0.29022e-06_rb,0.24496e-06_rb,0.24059e-06_rb,0.21631e-06_rb, &
        & 0.23923e-06_rb,0.16597e-06_rb,0.10842e-06_rb,0.80074e-04_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.12224e-07_rb,0.29680e-06_rb,0.25368e-06_rb,0.24691e-06_rb,0.22294e-06_rb, &
        & 0.24912e-06_rb,0.17747e-06_rb,0.10969e-06_rb,0.81125e-04_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.14264e-07_rb,0.29886e-06_rb,0.26040e-06_rb,0.25256e-06_rb,0.23184e-06_rb, &
        & 0.25957e-06_rb,0.18845e-06_rb,0.11123e-06_rb,0.82960e-04_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.16608e-07_rb,0.30056e-06_rb,0.26198e-06_rb,0.25724e-06_rb,0.23897e-06_rb, &
        & 0.26977e-06_rb,0.19716e-06_rb,0.11521e-06_rb,0.84911e-04_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.18982e-07_rb,0.30200e-06_rb,0.26559e-06_rb,0.26090e-06_rb,0.24427e-06_rb, &
        & 0.27763e-06_rb,0.20825e-06_rb,0.11858e-06_rb,0.86586e-04_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.16186e-07_rb,0.36864e-06_rb,0.27735e-06_rb,0.24475e-06_rb,0.23659e-06_rb, &
        & 0.20178e-06_rb,0.20651e-06_rb,0.11504e-06_rb,0.90323e-04_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.16900e-07_rb,0.36903e-06_rb,0.27865e-06_rb,0.25015e-06_rb,0.24279e-06_rb, &
        & 0.21379e-06_rb,0.22149e-06_rb,0.12739e-06_rb,0.92788e-04_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.18900e-07_rb,0.36546e-06_rb,0.28369e-06_rb,0.25919e-06_rb,0.24650e-06_rb, &
        & 0.22349e-06_rb,0.23741e-06_rb,0.13455e-06_rb,0.94381e-04_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.24666e-07_rb,0.36289e-06_rb,0.28992e-06_rb,0.26755e-06_rb,0.25261e-06_rb, &
        & 0.23012e-06_rb,0.24862e-06_rb,0.14012e-06_rb,0.95689e-04_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.28047e-07_rb,0.36144e-06_rb,0.29439e-06_rb,0.27280e-06_rb,0.25910e-06_rb, &
        & 0.23527e-06_rb,0.25804e-06_rb,0.14441e-06_rb,0.97273e-04_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.27099e-07_rb,0.42452e-06_rb,0.36469e-06_rb,0.28012e-06_rb,0.24206e-06_rb, &
        & 0.21452e-06_rb,0.19737e-06_rb,0.12552e-06_rb,0.93221e-04_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.30641e-07_rb,0.42348e-06_rb,0.36821e-06_rb,0.29049e-06_rb,0.25191e-06_rb, &
        & 0.22447e-06_rb,0.20459e-06_rb,0.13174e-06_rb,0.95326e-04_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.31538e-07_rb,0.42218e-06_rb,0.36978e-06_rb,0.29801e-06_rb,0.26042e-06_rb, &
        & 0.23413e-06_rb,0.21354e-06_rb,0.13999e-06_rb,0.97244e-04_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.33087e-07_rb,0.42233e-06_rb,0.36980e-06_rb,0.30223e-06_rb,0.26697e-06_rb, &
        & 0.24077e-06_rb,0.22289e-06_rb,0.14849e-06_rb,0.99282e-04_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.35771e-07_rb,0.42124e-06_rb,0.36949e-06_rb,0.30814e-06_rb,0.27423e-06_rb, &
        & 0.24510e-06_rb,0.22998e-06_rb,0.15431e-06_rb,0.10189e-03_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.60599e-08_rb,0.49839e-06_rb,0.90796e-06_rb,0.12825e-05_rb,0.16387e-05_rb, &
        & 0.20000e-05_rb,0.24097e-05_rb,0.30198e-05_rb,0.14012e-05_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.57361e-08_rb,0.51373e-06_rb,0.93344e-06_rb,0.13150e-05_rb,0.16776e-05_rb, &
        & 0.20412e-05_rb,0.24476e-05_rb,0.30155e-05_rb,0.14079e-05_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.61895e-08_rb,0.64074e-06_rb,0.11706e-05_rb,0.16523e-05_rb,0.21138e-05_rb, &
        & 0.25765e-05_rb,0.30714e-05_rb,0.37148e-05_rb,0.20046e-05_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.63590e-08_rb,0.53918e-06_rb,0.97233e-06_rb,0.13614e-05_rb,0.17274e-05_rb, &
        & 0.21018e-05_rb,0.24992e-05_rb,0.30391e-05_rb,0.14106e-05_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.66907e-08_rb,0.54769e-06_rb,0.98459e-06_rb,0.13756e-05_rb,0.17428e-05_rb, &
        & 0.21216e-05_rb,0.25190e-05_rb,0.30528e-05_rb,0.14057e-05_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.63247e-08_rb,0.40976e-06_rb,0.75743e-06_rb,0.10729e-05_rb,0.13776e-05_rb, &
        & 0.16847e-05_rb,0.20386e-05_rb,0.25923e-05_rb,0.11016e-05_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.71971e-08_rb,0.42277e-06_rb,0.77866e-06_rb,0.11006e-05_rb,0.14075e-05_rb, &
        & 0.17208e-05_rb,0.20689e-05_rb,0.25842e-05_rb,0.11093e-05_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.68862e-08_rb,0.54111e-06_rb,0.99924e-06_rb,0.14135e-05_rb,0.18137e-05_rb, &
        & 0.22190e-05_rb,0.26613e-05_rb,0.32636e-05_rb,0.16377e-05_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.74570e-08_rb,0.54374e-06_rb,0.10016e-05_rb,0.14142e-05_rb,0.18118e-05_rb, &
        & 0.22093e-05_rb,0.26451e-05_rb,0.32306e-05_rb,0.16062e-05_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.76975e-08_rb,0.44890e-06_rb,0.82016e-06_rb,0.11488e-05_rb,0.14591e-05_rb, &
        & 0.17814e-05_rb,0.21241e-05_rb,0.26089e-05_rb,0.11145e-05_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.83656e-08_rb,0.31311e-06_rb,0.61949e-06_rb,0.88176e-06_rb,0.11379e-05_rb, &
        & 0.13977e-05_rb,0.17043e-05_rb,0.22078e-05_rb,0.73836e-06_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.85171e-08_rb,0.32573e-06_rb,0.63573e-06_rb,0.90517e-06_rb,0.11641e-05_rb, &
        & 0.14286e-05_rb,0.17247e-05_rb,0.21938e-05_rb,0.74285e-06_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.78702e-08_rb,0.33673e-06_rb,0.64948e-06_rb,0.92347e-06_rb,0.11844e-05_rb, &
        & 0.14525e-05_rb,0.17473e-05_rb,0.21823e-05_rb,0.74417e-06_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.88135e-08_rb,0.44484e-06_rb,0.84006e-06_rb,0.11924e-05_rb,0.15326e-05_rb, &
        & 0.18766e-05_rb,0.22592e-05_rb,0.27980e-05_rb,0.11629e-05_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.83594e-08_rb,0.34852e-06_rb,0.67171e-06_rb,0.94786e-06_rb,0.12080e-05_rb, &
        & 0.14818e-05_rb,0.17721e-05_rb,0.22017e-05_rb,0.74162e-06_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.82233e-08_rb,0.22714e-06_rb,0.49574e-06_rb,0.71618e-06_rb,0.92897e-06_rb, &
        & 0.11522e-05_rb,0.14150e-05_rb,0.18749e-05_rb,0.10829e-05_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.90906e-08_rb,0.23281e-06_rb,0.51082e-06_rb,0.73615e-06_rb,0.95322e-06_rb, &
        & 0.11749e-05_rb,0.14287e-05_rb,0.18456e-05_rb,0.10964e-05_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.10403e-07_rb,0.23873e-06_rb,0.52083e-06_rb,0.75142e-06_rb,0.97057e-06_rb, &
        & 0.11959e-05_rb,0.14456e-05_rb,0.18299e-05_rb,0.11024e-05_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.11681e-07_rb,0.24287e-06_rb,0.52982e-06_rb,0.76518e-06_rb,0.98418e-06_rb, &
        & 0.12105e-05_rb,0.14593e-05_rb,0.18314e-05_rb,0.11002e-05_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.11781e-07_rb,0.32919e-06_rb,0.69041e-06_rb,0.99008e-06_rb,0.12781e-05_rb, &
        & 0.15684e-05_rb,0.18900e-05_rb,0.23626e-05_rb,0.11507e-05_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.10746e-07_rb,0.16960e-06_rb,0.36271e-06_rb,0.57393e-06_rb,0.75088e-06_rb, &
        & 0.94078e-06_rb,0.11700e-05_rb,0.15610e-05_rb,0.23060e-05_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.12322e-07_rb,0.17145e-06_rb,0.37466e-06_rb,0.58975e-06_rb,0.77156e-06_rb, &
        & 0.96051e-06_rb,0.11773e-05_rb,0.15431e-05_rb,0.23550e-05_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.12362e-07_rb,0.17491e-06_rb,0.38517e-06_rb,0.60273e-06_rb,0.78779e-06_rb, &
        & 0.97831e-06_rb,0.11895e-05_rb,0.15264e-05_rb,0.23946e-05_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.11932e-07_rb,0.17603e-06_rb,0.39310e-06_rb,0.61276e-06_rb,0.80198e-06_rb, &
        & 0.99168e-06_rb,0.12014e-05_rb,0.15209e-05_rb,0.24122e-05_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.12827e-07_rb,0.23797e-06_rb,0.54915e-06_rb,0.81314e-06_rb,0.10605e-05_rb, &
        & 0.13079e-05_rb,0.15848e-05_rb,0.19982e-05_rb,0.26265e-05_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.16887e-07_rb,0.16918e-06_rb,0.26182e-06_rb,0.43955e-06_rb,0.59876e-06_rb, &
        & 0.76220e-06_rb,0.96205e-06_rb,0.12977e-05_rb,0.30196e-05_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.16463e-07_rb,0.17256e-06_rb,0.26723e-06_rb,0.45266e-06_rb,0.61330e-06_rb, &
        & 0.77949e-06_rb,0.96881e-06_rb,0.12938e-05_rb,0.31027e-05_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.14297e-07_rb,0.17174e-06_rb,0.27626e-06_rb,0.46141e-06_rb,0.62873e-06_rb, &
        & 0.79335e-06_rb,0.97523e-06_rb,0.12716e-05_rb,0.31487e-05_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.15984e-07_rb,0.17093e-06_rb,0.27951e-06_rb,0.46972e-06_rb,0.64062e-06_rb, &
        & 0.80616e-06_rb,0.98183e-06_rb,0.12602e-05_rb,0.31591e-05_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.17832e-07_rb,0.17197e-06_rb,0.28124e-06_rb,0.47610e-06_rb,0.64900e-06_rb, &
        & 0.81537e-06_rb,0.99403e-06_rb,0.12613e-05_rb,0.32001e-05_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.19543e-07_rb,0.20424e-06_rb,0.20827e-06_rb,0.30613e-06_rb,0.46038e-06_rb, &
        & 0.61052e-06_rb,0.78796e-06_rb,0.10769e-05_rb,0.53495e-05_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.21943e-07_rb,0.20486e-06_rb,0.21292e-06_rb,0.31650e-06_rb,0.47369e-06_rb, &
        & 0.62546e-06_rb,0.78997e-06_rb,0.10696e-05_rb,0.53612e-05_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.23096e-07_rb,0.20519e-06_rb,0.21260e-06_rb,0.32419e-06_rb,0.48522e-06_rb, &
        & 0.63494e-06_rb,0.79556e-06_rb,0.10562e-05_rb,0.54125e-05_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.23811e-07_rb,0.20761e-06_rb,0.21334e-06_rb,0.32752e-06_rb,0.49467e-06_rb, &
        & 0.64456e-06_rb,0.80411e-06_rb,0.10435e-05_rb,0.54920e-05_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.23502e-07_rb,0.20918e-06_rb,0.21430e-06_rb,0.32878e-06_rb,0.50037e-06_rb, &
        & 0.65413e-06_rb,0.80879e-06_rb,0.10409e-05_rb,0.55076e-05_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.28818e-07_rb,0.26386e-06_rb,0.23322e-06_rb,0.23394e-06_rb,0.31766e-06_rb, &
        & 0.47372e-06_rb,0.64268e-06_rb,0.89272e-06_rb,0.10691e-04_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.30094e-07_rb,0.27235e-06_rb,0.23524e-06_rb,0.24245e-06_rb,0.32636e-06_rb, &
        & 0.48411e-06_rb,0.63773e-06_rb,0.88040e-06_rb,0.10770e-04_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.32474e-07_rb,0.27799e-06_rb,0.23717e-06_rb,0.24789e-06_rb,0.33554e-06_rb, &
        & 0.49304e-06_rb,0.64255e-06_rb,0.87788e-06_rb,0.10799e-04_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.33631e-07_rb,0.28020e-06_rb,0.23819e-06_rb,0.25000e-06_rb,0.34187e-06_rb, &
        & 0.49874e-06_rb,0.64803e-06_rb,0.86373e-06_rb,0.10824e-04_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.34597e-07_rb,0.28107e-06_rb,0.24266e-06_rb,0.24933e-06_rb,0.34607e-06_rb, &
        & 0.50692e-06_rb,0.65081e-06_rb,0.85686e-06_rb,0.10819e-04_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.37049e-07_rb,0.33070e-06_rb,0.32147e-06_rb,0.25828e-06_rb,0.24460e-06_rb, &
        & 0.33057e-06_rb,0.51182e-06_rb,0.72970e-06_rb,0.32667e-04_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.39634e-07_rb,0.34282e-06_rb,0.32875e-06_rb,0.26785e-06_rb,0.25757e-06_rb, &
        & 0.33579e-06_rb,0.50462e-06_rb,0.72240e-06_rb,0.32638e-04_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.42012e-07_rb,0.34998e-06_rb,0.33495e-06_rb,0.27514e-06_rb,0.26581e-06_rb, &
        & 0.33988e-06_rb,0.50360e-06_rb,0.72105e-06_rb,0.32828e-04_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.43216e-07_rb,0.35061e-06_rb,0.33953e-06_rb,0.28140e-06_rb,0.27172e-06_rb, &
        & 0.34266e-06_rb,0.50914e-06_rb,0.70829e-06_rb,0.32970e-04_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.43838e-07_rb,0.35447e-06_rb,0.34674e-06_rb,0.28252e-06_rb,0.27122e-06_rb, &
        & 0.34740e-06_rb,0.51058e-06_rb,0.70194e-06_rb,0.33023e-04_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.46492e-07_rb,0.34673e-06_rb,0.43959e-06_rb,0.39620e-06_rb,0.30969e-06_rb, &
        & 0.25359e-06_rb,0.35579e-06_rb,0.58999e-06_rb,0.75308e-04_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.51642e-07_rb,0.35814e-06_rb,0.45251e-06_rb,0.40694e-06_rb,0.31643e-06_rb, &
        & 0.25658e-06_rb,0.34700e-06_rb,0.58340e-06_rb,0.75445e-04_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.55139e-07_rb,0.36959e-06_rb,0.46273e-06_rb,0.41592e-06_rb,0.32168e-06_rb, &
        & 0.26320e-06_rb,0.34598e-06_rb,0.57972e-06_rb,0.75796e-04_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.58094e-07_rb,0.37731e-06_rb,0.47160e-06_rb,0.41969e-06_rb,0.32536e-06_rb, &
        & 0.27186e-06_rb,0.34692e-06_rb,0.57173e-06_rb,0.76680e-04_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.62665e-07_rb,0.38529e-06_rb,0.47122e-06_rb,0.42322e-06_rb,0.32990e-06_rb, &
        & 0.28125e-06_rb,0.34920e-06_rb,0.56204e-06_rb,0.77515e-04_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.71997e-07_rb,0.39039e-06_rb,0.48930e-06_rb,0.52037e-06_rb,0.44835e-06_rb, &
        & 0.32920e-06_rb,0.25424e-06_rb,0.45719e-06_rb,0.97599e-04_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.84174e-07_rb,0.39220e-06_rb,0.49753e-06_rb,0.53466e-06_rb,0.46035e-06_rb, &
        & 0.34028e-06_rb,0.25580e-06_rb,0.44804e-06_rb,0.98017e-04_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.88241e-07_rb,0.39809e-06_rb,0.50514e-06_rb,0.54464e-06_rb,0.47069e-06_rb, &
        & 0.34919e-06_rb,0.25830e-06_rb,0.44783e-06_rb,0.97609e-04_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.93462e-07_rb,0.40271e-06_rb,0.51216e-06_rb,0.55531e-06_rb,0.48178e-06_rb, &
        & 0.35530e-06_rb,0.26505e-06_rb,0.43524e-06_rb,0.97099e-04_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.96966e-07_rb,0.40660e-06_rb,0.51682e-06_rb,0.56367e-06_rb,0.48662e-06_rb, &
        & 0.35706e-06_rb,0.26903e-06_rb,0.42986e-06_rb,0.97653e-04_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.99102e-07_rb,0.42179e-06_rb,0.46920e-06_rb,0.54974e-06_rb,0.50699e-06_rb, &
        & 0.41543e-06_rb,0.26692e-06_rb,0.34287e-06_rb,0.10692e-03_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.10220e-06_rb,0.42343e-06_rb,0.48263e-06_rb,0.56179e-06_rb,0.52088e-06_rb, &
        & 0.42826e-06_rb,0.27416e-06_rb,0.32165e-06_rb,0.10849e-03_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.10736e-06_rb,0.42994e-06_rb,0.49237e-06_rb,0.56848e-06_rb,0.53837e-06_rb, &
        & 0.43862e-06_rb,0.28070e-06_rb,0.31402e-06_rb,0.10890e-03_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.10535e-06_rb,0.42853e-06_rb,0.49830e-06_rb,0.57259e-06_rb,0.55337e-06_rb, &
        & 0.45117e-06_rb,0.28964e-06_rb,0.30229e-06_rb,0.11005e-03_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.10635e-06_rb,0.42826e-06_rb,0.50038e-06_rb,0.57688e-06_rb,0.56182e-06_rb, &
        & 0.45862e-06_rb,0.29473e-06_rb,0.29904e-06_rb,0.11056e-03_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.11521e-06_rb,0.43118e-06_rb,0.49222e-06_rb,0.52624e-06_rb,0.55771e-06_rb, &
        & 0.47186e-06_rb,0.32691e-06_rb,0.24833e-06_rb,0.10886e-03_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.11730e-06_rb,0.43904e-06_rb,0.49350e-06_rb,0.54164e-06_rb,0.56845e-06_rb, &
        & 0.49098e-06_rb,0.34261e-06_rb,0.23938e-06_rb,0.11077e-03_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.11952e-06_rb,0.44327e-06_rb,0.50284e-06_rb,0.55816e-06_rb,0.58198e-06_rb, &
        & 0.50372e-06_rb,0.35861e-06_rb,0.22850e-06_rb,0.11296e-03_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.11965e-06_rb,0.44664e-06_rb,0.51058e-06_rb,0.56661e-06_rb,0.59093e-06_rb, &
        & 0.51146e-06_rb,0.36596e-06_rb,0.22349e-06_rb,0.11560e-03_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.12171e-06_rb,0.45566e-06_rb,0.52364e-06_rb,0.57439e-06_rb,0.59010e-06_rb, &
        & 0.52011e-06_rb,0.37200e-06_rb,0.22417e-06_rb,0.11598e-03_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.64393e-08_rb,0.14477e-05_rb,0.26383e-05_rb,0.37578e-05_rb,0.48497e-05_rb, &
        & 0.58771e-05_rb,0.69748e-05_rb,0.83337e-05_rb,0.52972e-05_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.74788e-08_rb,0.14813e-05_rb,0.26820e-05_rb,0.38123e-05_rb,0.49043e-05_rb, &
        & 0.59719e-05_rb,0.70794e-05_rb,0.84777e-05_rb,0.53686e-05_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.70698e-08_rb,0.17381e-05_rb,0.31521e-05_rb,0.44815e-05_rb,0.57370e-05_rb, &
        & 0.69696e-05_rb,0.82426e-05_rb,0.98195e-05_rb,0.67445e-05_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.75728e-08_rb,0.15363e-05_rb,0.27544e-05_rb,0.38956e-05_rb,0.49855e-05_rb, &
        & 0.60556e-05_rb,0.72284e-05_rb,0.86770e-05_rb,0.54442e-05_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.72646e-08_rb,0.15573e-05_rb,0.27840e-05_rb,0.39266e-05_rb,0.50091e-05_rb, &
        & 0.60781e-05_rb,0.72327e-05_rb,0.87344e-05_rb,0.54537e-05_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.91682e-08_rb,0.12471e-05_rb,0.22727e-05_rb,0.32462e-05_rb,0.42007e-05_rb, &
        & 0.51012e-05_rb,0.60761e-05_rb,0.73379e-05_rb,0.44448e-05_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.78841e-08_rb,0.12745e-05_rb,0.23098e-05_rb,0.32938e-05_rb,0.42384e-05_rb, &
        & 0.51817e-05_rb,0.61658e-05_rb,0.74615e-05_rb,0.44925e-05_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.88457e-08_rb,0.15305e-05_rb,0.27824e-05_rb,0.39573e-05_rb,0.50820e-05_rb, &
        & 0.61863e-05_rb,0.73547e-05_rb,0.88186e-05_rb,0.58192e-05_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.81699e-08_rb,0.15379e-05_rb,0.27884e-05_rb,0.39554e-05_rb,0.50640e-05_rb, &
        & 0.61598e-05_rb,0.73260e-05_rb,0.88047e-05_rb,0.57596e-05_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.81267e-08_rb,0.13345e-05_rb,0.23941e-05_rb,0.33760e-05_rb,0.43121e-05_rb, &
        & 0.52406e-05_rb,0.62663e-05_rb,0.76657e-05_rb,0.45393e-05_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.91717e-08_rb,0.10541e-05_rb,0.19313e-05_rb,0.27682e-05_rb,0.35915e-05_rb, &
        & 0.43666e-05_rb,0.52222e-05_rb,0.63759e-05_rb,0.36110e-05_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.10331e-07_rb,0.10789e-05_rb,0.19645e-05_rb,0.28075e-05_rb,0.36240e-05_rb, &
        & 0.44392e-05_rb,0.52990e-05_rb,0.64883e-05_rb,0.36530e-05_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.12324e-07_rb,0.10988e-05_rb,0.19944e-05_rb,0.28363e-05_rb,0.36532e-05_rb, &
        & 0.44650e-05_rb,0.53622e-05_rb,0.65761e-05_rb,0.36791e-05_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.11596e-07_rb,0.13389e-05_rb,0.24365e-05_rb,0.34597e-05_rb,0.44354e-05_rb, &
        & 0.54167e-05_rb,0.64681e-05_rb,0.78697e-05_rb,0.48568e-05_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.11476e-07_rb,0.11282e-05_rb,0.20370e-05_rb,0.28768e-05_rb,0.36785e-05_rb, &
        & 0.44764e-05_rb,0.53828e-05_rb,0.66660e-05_rb,0.36841e-05_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.12942e-07_rb,0.87057e-06_rb,0.16185e-05_rb,0.23328e-05_rb,0.30327e-05_rb, &
        & 0.36939e-05_rb,0.44351e-05_rb,0.54720e-05_rb,0.21271e-05_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.12911e-07_rb,0.89351e-06_rb,0.16491e-05_rb,0.23657e-05_rb,0.30660e-05_rb, &
        & 0.37589e-05_rb,0.45049e-05_rb,0.55808e-05_rb,0.21315e-05_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.11919e-07_rb,0.91285e-06_rb,0.16762e-05_rb,0.23907e-05_rb,0.30911e-05_rb, &
        & 0.37889e-05_rb,0.45584e-05_rb,0.56554e-05_rb,0.21302e-05_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.12644e-07_rb,0.92680e-06_rb,0.16966e-05_rb,0.24115e-05_rb,0.31023e-05_rb, &
        & 0.37973e-05_rb,0.46002e-05_rb,0.57029e-05_rb,0.21090e-05_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.14252e-07_rb,0.11501e-05_rb,0.21042e-05_rb,0.29827e-05_rb,0.38200e-05_rb, &
        & 0.46680e-05_rb,0.55977e-05_rb,0.69211e-05_rb,0.32123e-05_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.14457e-07_rb,0.68848e-06_rb,0.13403e-05_rb,0.19448e-05_rb,0.25364e-05_rb, &
        & 0.30956e-05_rb,0.37236e-05_rb,0.46561e-05_rb,0.23476e-05_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.13664e-07_rb,0.70724e-06_rb,0.13666e-05_rb,0.19744e-05_rb,0.25701e-05_rb, &
        & 0.31515e-05_rb,0.37903e-05_rb,0.47386e-05_rb,0.23292e-05_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.16363e-07_rb,0.72382e-06_rb,0.13894e-05_rb,0.19966e-05_rb,0.25908e-05_rb, &
        & 0.31942e-05_rb,0.38380e-05_rb,0.48063e-05_rb,0.23210e-05_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.18854e-07_rb,0.73645e-06_rb,0.14075e-05_rb,0.20143e-05_rb,0.25995e-05_rb, &
        & 0.31924e-05_rb,0.38751e-05_rb,0.48509e-05_rb,0.23084e-05_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.19502e-07_rb,0.95711e-06_rb,0.17948e-05_rb,0.25536e-05_rb,0.32756e-05_rb, &
        & 0.40127e-05_rb,0.48356e-05_rb,0.60502e-05_rb,0.26522e-05_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.20265e-07_rb,0.45716e-06_rb,0.10840e-05_rb,0.16034e-05_rb,0.20999e-05_rb, &
        & 0.25681e-05_rb,0.30977e-05_rb,0.39107e-05_rb,0.52450e-05_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.21598e-07_rb,0.47132e-06_rb,0.11079e-05_rb,0.16263e-05_rb,0.21396e-05_rb, &
        & 0.26164e-05_rb,0.31577e-05_rb,0.39701e-05_rb,0.52182e-05_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.24575e-07_rb,0.48601e-06_rb,0.11257e-05_rb,0.16486e-05_rb,0.21513e-05_rb, &
        & 0.26585e-05_rb,0.32022e-05_rb,0.40354e-05_rb,0.51968e-05_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.24397e-07_rb,0.49804e-06_rb,0.11416e-05_rb,0.16645e-05_rb,0.21605e-05_rb, &
        & 0.26639e-05_rb,0.32353e-05_rb,0.40822e-05_rb,0.51575e-05_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.24456e-07_rb,0.50499e-06_rb,0.11522e-05_rb,0.16738e-05_rb,0.21632e-05_rb, &
        & 0.26629e-05_rb,0.32463e-05_rb,0.41051e-05_rb,0.51085e-05_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.28039e-07_rb,0.29800e-06_rb,0.81783e-06_rb,0.13008e-05_rb,0.17226e-05_rb, &
        & 0.21163e-05_rb,0.25578e-05_rb,0.32519e-05_rb,0.82307e-05_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.29388e-07_rb,0.30492e-06_rb,0.83771e-06_rb,0.13192e-05_rb,0.17591e-05_rb, &
        & 0.21563e-05_rb,0.26128e-05_rb,0.33130e-05_rb,0.81908e-05_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.29556e-07_rb,0.30866e-06_rb,0.85444e-06_rb,0.13376e-05_rb,0.17722e-05_rb, &
        & 0.21943e-05_rb,0.26541e-05_rb,0.33584e-05_rb,0.81819e-05_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.26709e-07_rb,0.30951e-06_rb,0.86739e-06_rb,0.13505e-05_rb,0.17805e-05_rb, &
        & 0.22093e-05_rb,0.26812e-05_rb,0.34009e-05_rb,0.81017e-05_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.27877e-07_rb,0.31158e-06_rb,0.87645e-06_rb,0.13597e-05_rb,0.17817e-05_rb, &
        & 0.22083e-05_rb,0.26996e-05_rb,0.34240e-05_rb,0.80774e-05_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.34498e-07_rb,0.30504e-06_rb,0.50025e-06_rb,0.99104e-06_rb,0.13907e-05_rb, &
        & 0.17284e-05_rb,0.20950e-05_rb,0.26875e-05_rb,0.12492e-04_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.34468e-07_rb,0.31253e-06_rb,0.51376e-06_rb,0.10029e-05_rb,0.14233e-05_rb, &
        & 0.17665e-05_rb,0.21486e-05_rb,0.27439e-05_rb,0.12811e-04_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.33481e-07_rb,0.31885e-06_rb,0.52532e-06_rb,0.10176e-05_rb,0.14361e-05_rb, &
        & 0.18000e-05_rb,0.21864e-05_rb,0.27713e-05_rb,0.12979e-04_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.35023e-07_rb,0.32212e-06_rb,0.53171e-06_rb,0.10292e-05_rb,0.14414e-05_rb, &
        & 0.18222e-05_rb,0.22113e-05_rb,0.28111e-05_rb,0.13061e-04_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.38480e-07_rb,0.32386e-06_rb,0.53818e-06_rb,0.10355e-05_rb,0.14400e-05_rb, &
        & 0.18162e-05_rb,0.22298e-05_rb,0.28367e-05_rb,0.13100e-04_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.41152e-07_rb,0.31870e-06_rb,0.36402e-06_rb,0.61613e-06_rb,0.10619e-05_rb, &
        & 0.13940e-05_rb,0.17098e-05_rb,0.22173e-05_rb,0.27198e-04_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.47020e-07_rb,0.31765e-06_rb,0.37312e-06_rb,0.61961e-06_rb,0.10763e-05_rb, &
        & 0.14273e-05_rb,0.17555e-05_rb,0.22573e-05_rb,0.28218e-04_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.48171e-07_rb,0.31526e-06_rb,0.38468e-06_rb,0.62540e-06_rb,0.10803e-05_rb, &
        & 0.14543e-05_rb,0.17919e-05_rb,0.22822e-05_rb,0.28600e-04_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.52826e-07_rb,0.32481e-06_rb,0.38733e-06_rb,0.62721e-06_rb,0.10854e-05_rb, &
        & 0.14750e-05_rb,0.18124e-05_rb,0.23140e-05_rb,0.29051e-04_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.56456e-07_rb,0.32727e-06_rb,0.38424e-06_rb,0.63178e-06_rb,0.10853e-05_rb, &
        & 0.14692e-05_rb,0.18309e-05_rb,0.23369e-05_rb,0.29958e-04_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.62033e-07_rb,0.45418e-06_rb,0.43247e-06_rb,0.44775e-06_rb,0.62834e-06_rb, &
        & 0.10534e-05_rb,0.13865e-05_rb,0.18301e-05_rb,0.47779e-04_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.66259e-07_rb,0.45321e-06_rb,0.43983e-06_rb,0.45223e-06_rb,0.63440e-06_rb, &
        & 0.10746e-05_rb,0.14253e-05_rb,0.18534e-05_rb,0.48468e-04_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.78966e-07_rb,0.45246e-06_rb,0.44780e-06_rb,0.45868e-06_rb,0.63026e-06_rb, &
        & 0.10842e-05_rb,0.14519e-05_rb,0.18794e-05_rb,0.49251e-04_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.87881e-07_rb,0.44642e-06_rb,0.45426e-06_rb,0.46583e-06_rb,0.63116e-06_rb, &
        & 0.10925e-05_rb,0.14710e-05_rb,0.18965e-05_rb,0.49742e-04_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.92572e-07_rb,0.44310e-06_rb,0.45326e-06_rb,0.47005e-06_rb,0.63094e-06_rb, &
        & 0.10763e-05_rb,0.14843e-05_rb,0.19214e-05_rb,0.50457e-04_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.10522e-06_rb,0.58600e-06_rb,0.57473e-06_rb,0.52150e-06_rb,0.51202e-06_rb, &
        & 0.62383e-06_rb,0.10987e-05_rb,0.15032e-05_rb,0.50557e-04_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.98332e-07_rb,0.58669e-06_rb,0.57910e-06_rb,0.52997e-06_rb,0.52591e-06_rb, &
        & 0.62992e-06_rb,0.11144e-05_rb,0.15271e-05_rb,0.52915e-04_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.10383e-06_rb,0.58976e-06_rb,0.58650e-06_rb,0.53573e-06_rb,0.53394e-06_rb, &
        & 0.63743e-06_rb,0.11251e-05_rb,0.15415e-05_rb,0.55206e-04_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.10626e-06_rb,0.58622e-06_rb,0.58747e-06_rb,0.53868e-06_rb,0.53891e-06_rb, &
        & 0.62645e-06_rb,0.11296e-05_rb,0.15628e-05_rb,0.58020e-04_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.10795e-06_rb,0.57800e-06_rb,0.58664e-06_rb,0.53883e-06_rb,0.54784e-06_rb, &
        & 0.62145e-06_rb,0.11279e-05_rb,0.15782e-05_rb,0.60369e-04_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.11905e-06_rb,0.60759e-06_rb,0.68611e-06_rb,0.59318e-06_rb,0.53805e-06_rb, &
        & 0.44962e-06_rb,0.74828e-06_rb,0.12279e-05_rb,0.57098e-04_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.12660e-06_rb,0.61695e-06_rb,0.68094e-06_rb,0.60079e-06_rb,0.55064e-06_rb, &
        & 0.44953e-06_rb,0.74858e-06_rb,0.12489e-05_rb,0.55969e-04_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.12824e-06_rb,0.61561e-06_rb,0.67586e-06_rb,0.61662e-06_rb,0.55533e-06_rb, &
        & 0.46796e-06_rb,0.74406e-06_rb,0.12615e-05_rb,0.57249e-04_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.12834e-06_rb,0.62554e-06_rb,0.67923e-06_rb,0.62343e-06_rb,0.55964e-06_rb, &
        & 0.48091e-06_rb,0.73622e-06_rb,0.12795e-05_rb,0.58462e-04_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.13022e-06_rb,0.63206e-06_rb,0.68726e-06_rb,0.62663e-06_rb,0.56231e-06_rb, &
        & 0.48008e-06_rb,0.72942e-06_rb,0.12885e-05_rb,0.57642e-04_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.15325e-06_rb,0.60909e-06_rb,0.76952e-06_rb,0.71232e-06_rb,0.57983e-06_rb, &
        & 0.50489e-06_rb,0.48627e-06_rb,0.99183e-06_rb,0.58778e-04_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.15519e-06_rb,0.62152e-06_rb,0.78639e-06_rb,0.71888e-06_rb,0.59866e-06_rb, &
        & 0.50930e-06_rb,0.48247e-06_rb,0.10069e-05_rb,0.57959e-04_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.17242e-06_rb,0.64099e-06_rb,0.78686e-06_rb,0.71502e-06_rb,0.60678e-06_rb, &
        & 0.51633e-06_rb,0.47220e-06_rb,0.10180e-05_rb,0.56844e-04_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.18585e-06_rb,0.65002e-06_rb,0.78368e-06_rb,0.71421e-06_rb,0.61538e-06_rb, &
        & 0.52650e-06_rb,0.46857e-06_rb,0.10263e-05_rb,0.55757e-04_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.19235e-06_rb,0.64166e-06_rb,0.76203e-06_rb,0.70622e-06_rb,0.61837e-06_rb, &
        & 0.53042e-06_rb,0.45529e-06_rb,0.10275e-05_rb,0.55475e-04_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.95617e-08_rb,0.36774e-05_rb,0.65849e-05_rb,0.92221e-05_rb,0.11700e-04_rb, &
        & 0.14161e-04_rb,0.16732e-04_rb,0.20350e-04_rb,0.14554e-04_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.93327e-08_rb,0.37537e-05_rb,0.66916e-05_rb,0.93495e-05_rb,0.11842e-04_rb, &
        & 0.14289e-04_rb,0.16848e-04_rb,0.20318e-04_rb,0.14747e-04_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.98298e-08_rb,0.41944e-05_rb,0.75022e-05_rb,0.10503e-04_rb,0.13381e-04_rb, &
        & 0.16104e-04_rb,0.19004e-04_rb,0.22601e-04_rb,0.17337e-04_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.92318e-08_rb,0.38872e-05_rb,0.68816e-05_rb,0.95784e-05_rb,0.12112e-04_rb, &
        & 0.14510e-04_rb,0.17052e-04_rb,0.20522e-04_rb,0.14967e-04_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.98053e-08_rb,0.39438e-05_rb,0.69574e-05_rb,0.96665e-05_rb,0.12218e-04_rb, &
        & 0.14607e-04_rb,0.17135e-04_rb,0.20557e-04_rb,0.15005e-04_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.99669e-08_rb,0.32456e-05_rb,0.58216e-05_rb,0.81575e-05_rb,0.10349e-04_rb, &
        & 0.12573e-04_rb,0.14901e-04_rb,0.18163e-04_rb,0.12615e-04_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.12231e-07_rb,0.33138e-05_rb,0.59221e-05_rb,0.82752e-05_rb,0.10501e-04_rb, &
        & 0.12666e-04_rb,0.15004e-04_rb,0.18147e-04_rb,0.12797e-04_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.10745e-07_rb,0.37581e-05_rb,0.67327e-05_rb,0.94426e-05_rb,0.12044e-04_rb, &
        & 0.14533e-04_rb,0.17181e-04_rb,0.20503e-04_rb,0.15464e-04_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.11723e-07_rb,0.37897e-05_rb,0.67581e-05_rb,0.94584e-05_rb,0.12035e-04_rb, &
        & 0.14508e-04_rb,0.17115e-04_rb,0.20450e-04_rb,0.15378e-04_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.10775e-07_rb,0.34864e-05_rb,0.61478e-05_rb,0.85479e-05_rb,0.10823e-04_rb, &
        & 0.12985e-04_rb,0.15273e-04_rb,0.18381e-04_rb,0.12972e-04_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.12159e-07_rb,0.28311e-05_rb,0.50923e-05_rb,0.71433e-05_rb,0.90735e-05_rb, &
        & 0.11042e-04_rb,0.13160e-04_rb,0.16141e-04_rb,0.10790e-04_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.11403e-07_rb,0.28950e-05_rb,0.51829e-05_rb,0.72481e-05_rb,0.92149e-05_rb, &
        & 0.11135e-04_rb,0.13267e-04_rb,0.16084e-04_rb,0.10912e-04_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.12436e-07_rb,0.29531e-05_rb,0.52642e-05_rb,0.73461e-05_rb,0.93337e-05_rb, &
        & 0.11250e-04_rb,0.13358e-04_rb,0.16208e-04_rb,0.10995e-04_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.14856e-07_rb,0.33723e-05_rb,0.60342e-05_rb,0.84579e-05_rb,0.10776e-04_rb, &
        & 0.13025e-04_rb,0.15434e-04_rb,0.18466e-04_rb,0.13550e-04_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.16537e-07_rb,0.30506e-05_rb,0.53821e-05_rb,0.74897e-05_rb,0.94861e-05_rb, &
        & 0.11421e-04_rb,0.13503e-04_rb,0.16308e-04_rb,0.11045e-04_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.13114e-07_rb,0.24467e-05_rb,0.44159e-05_rb,0.61941e-05_rb,0.78845e-05_rb, &
        & 0.96031e-05_rb,0.11513e-04_rb,0.14234e-04_rb,0.89642e-05_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.15028e-07_rb,0.25040e-05_rb,0.44954e-05_rb,0.62931e-05_rb,0.80050e-05_rb, &
        & 0.97056e-05_rb,0.11618e-04_rb,0.14175e-04_rb,0.90802e-05_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.18213e-07_rb,0.25556e-05_rb,0.45678e-05_rb,0.63789e-05_rb,0.81071e-05_rb, &
        & 0.98077e-05_rb,0.11694e-04_rb,0.14301e-04_rb,0.91538e-05_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.19097e-07_rb,0.26037e-05_rb,0.46288e-05_rb,0.64476e-05_rb,0.81897e-05_rb, &
        & 0.98941e-05_rb,0.11740e-04_rb,0.14344e-04_rb,0.91968e-05_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.18014e-07_rb,0.29993e-05_rb,0.53622e-05_rb,0.75209e-05_rb,0.95686e-05_rb, &
        & 0.11571e-04_rb,0.13783e-04_rb,0.16522e-04_rb,0.11644e-04_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.21237e-07_rb,0.20921e-05_rb,0.37961e-05_rb,0.53264e-05_rb,0.67905e-05_rb, &
        & 0.82846e-05_rb,0.99803e-05_rb,0.12512e-04_rb,0.44075e-05_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.23885e-07_rb,0.21444e-05_rb,0.38676e-05_rb,0.54146e-05_rb,0.68931e-05_rb, &
        & 0.83832e-05_rb,0.10077e-04_rb,0.12417e-04_rb,0.44982e-05_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.20851e-07_rb,0.21906e-05_rb,0.39301e-05_rb,0.54900e-05_rb,0.69833e-05_rb, &
        & 0.84513e-05_rb,0.10145e-04_rb,0.12513e-04_rb,0.45299e-05_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.22487e-07_rb,0.22322e-05_rb,0.39843e-05_rb,0.55506e-05_rb,0.70502e-05_rb, &
        & 0.85357e-05_rb,0.10189e-04_rb,0.12546e-04_rb,0.45220e-05_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.23015e-07_rb,0.26442e-05_rb,0.47363e-05_rb,0.66520e-05_rb,0.84676e-05_rb, &
        & 0.10272e-04_rb,0.12268e-04_rb,0.14801e-04_rb,0.72945e-05_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.22693e-07_rb,0.17592e-05_rb,0.32289e-05_rb,0.45327e-05_rb,0.57876e-05_rb, &
        & 0.70822e-05_rb,0.85643e-05_rb,0.10966e-04_rb,0.53375e-05_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.22407e-07_rb,0.18081e-05_rb,0.32912e-05_rb,0.46136e-05_rb,0.58716e-05_rb, &
        & 0.71710e-05_rb,0.86530e-05_rb,0.10805e-04_rb,0.53845e-05_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.22705e-07_rb,0.18504e-05_rb,0.33475e-05_rb,0.46786e-05_rb,0.59585e-05_rb, &
        & 0.72252e-05_rb,0.87123e-05_rb,0.10876e-04_rb,0.53808e-05_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.23941e-07_rb,0.18862e-05_rb,0.33938e-05_rb,0.47325e-05_rb,0.60159e-05_rb, &
        & 0.72910e-05_rb,0.87573e-05_rb,0.10893e-04_rb,0.53357e-05_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.27004e-07_rb,0.19136e-05_rb,0.34294e-05_rb,0.47766e-05_rb,0.60573e-05_rb, &
        & 0.73309e-05_rb,0.87892e-05_rb,0.10892e-04_rb,0.52484e-05_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.27131e-07_rb,0.14236e-05_rb,0.27128e-05_rb,0.38203e-05_rb,0.48870e-05_rb, &
        & 0.59983e-05_rb,0.72827e-05_rb,0.94729e-05_rb,0.86942e-05_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.26214e-07_rb,0.14673e-05_rb,0.27677e-05_rb,0.38940e-05_rb,0.49627e-05_rb, &
        & 0.60727e-05_rb,0.73587e-05_rb,0.93332e-05_rb,0.87651e-05_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.29249e-07_rb,0.15085e-05_rb,0.28175e-05_rb,0.39503e-05_rb,0.50362e-05_rb, &
        & 0.61230e-05_rb,0.74095e-05_rb,0.93608e-05_rb,0.87497e-05_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.35846e-07_rb,0.15423e-05_rb,0.28570e-05_rb,0.39988e-05_rb,0.50875e-05_rb, &
        & 0.61709e-05_rb,0.74491e-05_rb,0.93691e-05_rb,0.88300e-05_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.36255e-07_rb,0.15665e-05_rb,0.28868e-05_rb,0.40378e-05_rb,0.51248e-05_rb, &
        & 0.62083e-05_rb,0.74689e-05_rb,0.93597e-05_rb,0.88065e-05_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.37619e-07_rb,0.92017e-06_rb,0.22295e-05_rb,0.31821e-05_rb,0.40883e-05_rb, &
        & 0.50331e-05_rb,0.61447e-05_rb,0.80461e-05_rb,0.15512e-04_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.43015e-07_rb,0.95048e-06_rb,0.22782e-05_rb,0.32493e-05_rb,0.41585e-05_rb, &
        & 0.50968e-05_rb,0.61995e-05_rb,0.80232e-05_rb,0.15472e-04_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.47635e-07_rb,0.97934e-06_rb,0.23207e-05_rb,0.33002e-05_rb,0.42181e-05_rb, &
        & 0.51430e-05_rb,0.62468e-05_rb,0.79821e-05_rb,0.15652e-04_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.48140e-07_rb,0.10026e-05_rb,0.23582e-05_rb,0.33416e-05_rb,0.42650e-05_rb, &
        & 0.51734e-05_rb,0.62779e-05_rb,0.79833e-05_rb,0.15762e-04_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.45231e-07_rb,0.10212e-05_rb,0.23819e-05_rb,0.33754e-05_rb,0.42991e-05_rb, &
        & 0.52146e-05_rb,0.62945e-05_rb,0.79701e-05_rb,0.15840e-04_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.48601e-07_rb,0.49350e-06_rb,0.16585e-05_rb,0.26009e-05_rb,0.33796e-05_rb, &
        & 0.41888e-05_rb,0.51434e-05_rb,0.67867e-05_rb,0.26875e-04_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.49825e-07_rb,0.51680e-06_rb,0.16935e-05_rb,0.26607e-05_rb,0.34457e-05_rb, &
        & 0.42434e-05_rb,0.51839e-05_rb,0.68587e-05_rb,0.27475e-04_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.52843e-07_rb,0.53573e-06_rb,0.17199e-05_rb,0.27038e-05_rb,0.35027e-05_rb, &
        & 0.42858e-05_rb,0.52233e-05_rb,0.67410e-05_rb,0.28097e-04_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.49841e-07_rb,0.54248e-06_rb,0.17493e-05_rb,0.27418e-05_rb,0.35427e-05_rb, &
        & 0.43127e-05_rb,0.52517e-05_rb,0.67418e-05_rb,0.28632e-04_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.50649e-07_rb,0.54940e-06_rb,0.17700e-05_rb,0.27695e-05_rb,0.35712e-05_rb, &
        & 0.43482e-05_rb,0.52645e-05_rb,0.67282e-05_rb,0.28662e-04_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.82685e-07_rb,0.54022e-06_rb,0.87761e-06_rb,0.18752e-05_rb,0.27293e-05_rb, &
        & 0.34506e-05_rb,0.42821e-05_rb,0.56501e-05_rb,0.44067e-04_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.87011e-07_rb,0.54017e-06_rb,0.89732e-06_rb,0.19208e-05_rb,0.27945e-05_rb, &
        & 0.35005e-05_rb,0.43124e-05_rb,0.57881e-05_rb,0.45858e-04_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.79909e-07_rb,0.55355e-06_rb,0.90995e-06_rb,0.19451e-05_rb,0.28460e-05_rb, &
        & 0.35425e-05_rb,0.43438e-05_rb,0.56593e-05_rb,0.45805e-04_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.75533e-07_rb,0.55705e-06_rb,0.92147e-06_rb,0.19661e-05_rb,0.28785e-05_rb, &
        & 0.35628e-05_rb,0.43635e-05_rb,0.56526e-05_rb,0.46295e-04_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.76631e-07_rb,0.55159e-06_rb,0.93362e-06_rb,0.19832e-05_rb,0.29003e-05_rb, &
        & 0.35965e-05_rb,0.43731e-05_rb,0.56385e-05_rb,0.46720e-04_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.11164e-06_rb,0.57710e-06_rb,0.71279e-06_rb,0.10003e-05_rb,0.19253e-05_rb, &
        & 0.28027e-05_rb,0.35523e-05_rb,0.47607e-05_rb,0.51144e-04_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.11484e-06_rb,0.58649e-06_rb,0.71918e-06_rb,0.10167e-05_rb,0.19538e-05_rb, &
        & 0.28431e-05_rb,0.35713e-05_rb,0.48333e-05_rb,0.49887e-04_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.11442e-06_rb,0.58483e-06_rb,0.70503e-06_rb,0.10320e-05_rb,0.19684e-05_rb, &
        & 0.28674e-05_rb,0.35911e-05_rb,0.47191e-05_rb,0.50222e-04_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.11737e-06_rb,0.58895e-06_rb,0.70242e-06_rb,0.10442e-05_rb,0.19738e-05_rb, &
        & 0.28962e-05_rb,0.36035e-05_rb,0.47045e-05_rb,0.48754e-04_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.12314e-06_rb,0.59299e-06_rb,0.70874e-06_rb,0.10470e-05_rb,0.19736e-05_rb, &
        & 0.29100e-05_rb,0.36075e-05_rb,0.46813e-05_rb,0.45986e-04_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.14446e-06_rb,0.57459e-06_rb,0.72800e-06_rb,0.69912e-06_rb,0.11693e-05_rb, &
        & 0.21457e-05_rb,0.29151e-05_rb,0.39683e-05_rb,0.42970e-04_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.14738e-06_rb,0.58568e-06_rb,0.75374e-06_rb,0.69814e-06_rb,0.11797e-05_rb, &
        & 0.21759e-05_rb,0.29287e-05_rb,0.39942e-05_rb,0.41434e-04_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.15006e-06_rb,0.60144e-06_rb,0.75420e-06_rb,0.70559e-06_rb,0.11849e-05_rb, &
        & 0.21718e-05_rb,0.29383e-05_rb,0.39185e-05_rb,0.42909e-04_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.15745e-06_rb,0.60776e-06_rb,0.75984e-06_rb,0.71270e-06_rb,0.11804e-05_rb, &
        & 0.21578e-05_rb,0.29477e-05_rb,0.38939e-05_rb,0.41761e-04_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.16583e-06_rb,0.62207e-06_rb,0.74591e-06_rb,0.69746e-06_rb,0.11751e-05_rb, &
        & 0.21552e-05_rb,0.29514e-05_rb,0.38695e-05_rb,0.45252e-04_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.18492e-06_rb,0.65841e-06_rb,0.74904e-06_rb,0.82698e-06_rb,0.69790e-06_rb, &
        & 0.13356e-05_rb,0.23333e-05_rb,0.33010e-05_rb,0.43502e-04_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.18447e-06_rb,0.66115e-06_rb,0.74200e-06_rb,0.80748e-06_rb,0.68676e-06_rb, &
        & 0.13419e-05_rb,0.23369e-05_rb,0.32873e-05_rb,0.44908e-04_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.20107e-06_rb,0.65387e-06_rb,0.73155e-06_rb,0.79291e-06_rb,0.67848e-06_rb, &
        & 0.13368e-05_rb,0.23434e-05_rb,0.32337e-05_rb,0.47417e-04_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.20493e-06_rb,0.66191e-06_rb,0.72707e-06_rb,0.79839e-06_rb,0.67208e-06_rb, &
        & 0.13273e-05_rb,0.23466e-05_rb,0.32075e-05_rb,0.48703e-04_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.20003e-06_rb,0.66753e-06_rb,0.73390e-06_rb,0.79794e-06_rb,0.66936e-06_rb, &
        & 0.13150e-05_rb,0.23558e-05_rb,0.31828e-05_rb,0.50012e-04_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.45955e-07_rb,0.96988e-05_rb,0.16766e-04_rb,0.23073e-04_rb,0.28829e-04_rb, &
        & 0.34350e-04_rb,0.39744e-04_rb,0.45071e-04_rb,0.36783e-04_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.57268e-07_rb,0.98413e-05_rb,0.16970e-04_rb,0.23311e-04_rb,0.29149e-04_rb, &
        & 0.34726e-04_rb,0.40229e-04_rb,0.45682e-04_rb,0.37190e-04_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.70060e-07_rb,0.10517e-04_rb,0.18168e-04_rb,0.24965e-04_rb,0.31264e-04_rb, &
        & 0.37439e-04_rb,0.43299e-04_rb,0.49335e-04_rb,0.41254e-04_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.84370e-07_rb,0.10120e-04_rb,0.17302e-04_rb,0.23633e-04_rb,0.29544e-04_rb, &
        & 0.35368e-04_rb,0.40885e-04_rb,0.46535e-04_rb,0.37819e-04_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.97346e-07_rb,0.10254e-04_rb,0.17458e-04_rb,0.23773e-04_rb,0.29672e-04_rb, &
        & 0.35542e-04_rb,0.41140e-04_rb,0.46864e-04_rb,0.38054e-04_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.27937e-07_rb,0.87332e-05_rb,0.15102e-04_rb,0.20785e-04_rb,0.26002e-04_rb, &
        & 0.30993e-04_rb,0.35918e-04_rb,0.41124e-04_rb,0.32692e-04_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.34502e-07_rb,0.88655e-05_rb,0.15279e-04_rb,0.20992e-04_rb,0.26281e-04_rb, &
        & 0.31393e-04_rb,0.36363e-04_rb,0.41768e-04_rb,0.33047e-04_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.45243e-07_rb,0.95567e-05_rb,0.16535e-04_rb,0.22740e-04_rb,0.28503e-04_rb, &
        & 0.34161e-04_rb,0.39521e-04_rb,0.45509e-04_rb,0.37184e-04_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.56029e-07_rb,0.96404e-05_rb,0.16599e-04_rb,0.22759e-04_rb,0.28526e-04_rb, &
        & 0.34207e-04_rb,0.39586e-04_rb,0.45601e-04_rb,0.37152e-04_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.67453e-07_rb,0.92372e-05_rb,0.15744e-04_rb,0.21435e-04_rb,0.26769e-04_rb, &
        & 0.32085e-04_rb,0.37228e-04_rb,0.42908e-04_rb,0.33841e-04_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.21909e-07_rb,0.77821e-05_rb,0.13457e-04_rb,0.18525e-04_rb,0.23194e-04_rb, &
        & 0.27662e-04_rb,0.32086e-04_rb,0.37089e-04_rb,0.28700e-04_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.25816e-07_rb,0.78998e-05_rb,0.13627e-04_rb,0.18723e-04_rb,0.23459e-04_rb, &
        & 0.28049e-04_rb,0.32494e-04_rb,0.37775e-04_rb,0.29060e-04_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.28009e-07_rb,0.80177e-05_rb,0.13778e-04_rb,0.18883e-04_rb,0.23652e-04_rb, &
        & 0.28348e-04_rb,0.32831e-04_rb,0.38183e-04_rb,0.29363e-04_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.32250e-07_rb,0.87000e-05_rb,0.14997e-04_rb,0.20591e-04_rb,0.25858e-04_rb, &
        & 0.30984e-04_rb,0.35855e-04_rb,0.41716e-04_rb,0.33352e-04_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.38565e-07_rb,0.82442e-05_rb,0.14048e-04_rb,0.19147e-04_rb,0.23963e-04_rb, &
        & 0.28712e-04_rb,0.33322e-04_rb,0.38841e-04_rb,0.29827e-04_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.27845e-07_rb,0.68782e-05_rb,0.11887e-04_rb,0.16370e-04_rb,0.20522e-04_rb, &
        & 0.24507e-04_rb,0.28445e-04_rb,0.33146e-04_rb,0.25091e-04_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.30162e-07_rb,0.69860e-05_rb,0.12053e-04_rb,0.16569e-04_rb,0.20786e-04_rb, &
        & 0.24863e-04_rb,0.28833e-04_rb,0.33823e-04_rb,0.25439e-04_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.31547e-07_rb,0.70934e-05_rb,0.12196e-04_rb,0.16734e-04_rb,0.20984e-04_rb, &
        & 0.25140e-04_rb,0.29167e-04_rb,0.34188e-04_rb,0.25743e-04_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.30733e-07_rb,0.72009e-05_rb,0.12319e-04_rb,0.16870e-04_rb,0.21150e-04_rb, &
        & 0.25350e-04_rb,0.29432e-04_rb,0.34551e-04_rb,0.26011e-04_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.33104e-07_rb,0.78742e-05_rb,0.13511e-04_rb,0.18536e-04_rb,0.23314e-04_rb, &
        & 0.27934e-04_rb,0.32284e-04_rb,0.37983e-04_rb,0.29911e-04_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.31453e-07_rb,0.60344e-05_rb,0.10431e-04_rb,0.14360e-04_rb,0.18025e-04_rb, &
        & 0.21556e-04_rb,0.25057e-04_rb,0.29352e-04_rb,0.21244e-04_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.32645e-07_rb,0.61373e-05_rb,0.10589e-04_rb,0.14561e-04_rb,0.18288e-04_rb, &
        & 0.21896e-04_rb,0.25427e-04_rb,0.30041e-04_rb,0.21582e-04_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.36285e-07_rb,0.62371e-05_rb,0.10724e-04_rb,0.14730e-04_rb,0.18494e-04_rb, &
        & 0.22175e-04_rb,0.25749e-04_rb,0.30401e-04_rb,0.21875e-04_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.35840e-07_rb,0.63361e-05_rb,0.10841e-04_rb,0.14872e-04_rb,0.18672e-04_rb, &
        & 0.22386e-04_rb,0.26001e-04_rb,0.30772e-04_rb,0.22136e-04_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.34391e-07_rb,0.70384e-05_rb,0.12100e-04_rb,0.16639e-04_rb,0.20967e-04_rb, &
        & 0.25106e-04_rb,0.29033e-04_rb,0.34382e-04_rb,0.26458e-04_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.38317e-07_rb,0.52554e-05_rb,0.90847e-05_rb,0.12506e-04_rb,0.15706e-04_rb, &
        & 0.18802e-04_rb,0.21926e-04_rb,0.25721e-04_rb,0.82786e-05_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.42993e-07_rb,0.53521e-05_rb,0.92425e-05_rb,0.12708e-04_rb,0.15965e-04_rb, &
        & 0.19131e-04_rb,0.22269e-04_rb,0.26449e-04_rb,0.85145e-05_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.43612e-07_rb,0.54474e-05_rb,0.93707e-05_rb,0.12875e-04_rb,0.16176e-04_rb, &
        & 0.19407e-04_rb,0.22575e-04_rb,0.26830e-04_rb,0.87594e-05_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.42757e-07_rb,0.55405e-05_rb,0.94823e-05_rb,0.13019e-04_rb,0.16363e-04_rb, &
        & 0.19625e-04_rb,0.22815e-04_rb,0.27200e-04_rb,0.90170e-05_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.43664e-07_rb,0.56274e-05_rb,0.95923e-05_rb,0.13140e-04_rb,0.16520e-04_rb, &
        & 0.19797e-04_rb,0.22995e-04_rb,0.27528e-04_rb,0.91836e-05_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.39995e-07_rb,0.45414e-05_rb,0.78502e-05_rb,0.10801e-04_rb,0.13562e-04_rb, &
        & 0.16248e-04_rb,0.19009e-04_rb,0.22439e-04_rb,0.93804e-05_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.43614e-07_rb,0.46331e-05_rb,0.80059e-05_rb,0.11000e-04_rb,0.13813e-04_rb, &
        & 0.16577e-04_rb,0.19346e-04_rb,0.23103e-04_rb,0.95334e-05_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.45687e-07_rb,0.47201e-05_rb,0.81316e-05_rb,0.11167e-04_rb,0.14031e-04_rb, &
        & 0.16844e-04_rb,0.19646e-04_rb,0.23513e-04_rb,0.96503e-05_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.48386e-07_rb,0.48081e-05_rb,0.82413e-05_rb,0.11310e-04_rb,0.14223e-04_rb, &
        & 0.17063e-04_rb,0.19885e-04_rb,0.23881e-04_rb,0.95360e-05_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.54287e-07_rb,0.48899e-05_rb,0.83473e-05_rb,0.11434e-04_rb,0.14383e-04_rb, &
        & 0.17240e-04_rb,0.20064e-04_rb,0.24201e-04_rb,0.94267e-05_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.45720e-07_rb,0.38685e-05_rb,0.67269e-05_rb,0.92537e-05_rb,0.11613e-04_rb, &
        & 0.13930e-04_rb,0.16333e-04_rb,0.19500e-04_rb,0.15537e-04_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.46817e-07_rb,0.39568e-05_rb,0.68775e-05_rb,0.94493e-05_rb,0.11857e-04_rb, &
        & 0.14250e-04_rb,0.16679e-04_rb,0.19983e-04_rb,0.15822e-04_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.48700e-07_rb,0.40362e-05_rb,0.70012e-05_rb,0.96148e-05_rb,0.12076e-04_rb, &
        & 0.14508e-04_rb,0.16969e-04_rb,0.20440e-04_rb,0.15878e-04_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.54460e-07_rb,0.41199e-05_rb,0.71098e-05_rb,0.97580e-05_rb,0.12268e-04_rb, &
        & 0.14725e-04_rb,0.17206e-04_rb,0.20805e-04_rb,0.16014e-04_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.59565e-07_rb,0.42000e-05_rb,0.72128e-05_rb,0.98789e-05_rb,0.12424e-04_rb, &
        & 0.14896e-04_rb,0.17382e-04_rb,0.21113e-04_rb,0.15894e-04_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.60502e-07_rb,0.31758e-05_rb,0.57011e-05_rb,0.78602e-05_rb,0.98646e-05_rb, &
        & 0.11842e-04_rb,0.13914e-04_rb,0.16812e-04_rb,0.32738e-04_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.61920e-07_rb,0.32583e-05_rb,0.58474e-05_rb,0.80501e-05_rb,0.10097e-04_rb, &
        & 0.12150e-04_rb,0.14260e-04_rb,0.17128e-04_rb,0.32707e-04_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.66102e-07_rb,0.33410e-05_rb,0.59686e-05_rb,0.82148e-05_rb,0.10310e-04_rb, &
        & 0.12398e-04_rb,0.14542e-04_rb,0.17650e-04_rb,0.33629e-04_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.86168e-07_rb,0.34261e-05_rb,0.60763e-05_rb,0.83539e-05_rb,0.10496e-04_rb, &
        & 0.12604e-04_rb,0.14773e-04_rb,0.17994e-04_rb,0.33878e-04_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.91586e-07_rb,0.35057e-05_rb,0.61764e-05_rb,0.84703e-05_rb,0.10646e-04_rb, &
        & 0.12765e-04_rb,0.14939e-04_rb,0.18282e-04_rb,0.33645e-04_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.96997e-07_rb,0.19211e-05_rb,0.47098e-05_rb,0.66155e-05_rb,0.83225e-05_rb, &
        & 0.10005e-04_rb,0.11773e-04_rb,0.14404e-04_rb,0.53631e-04_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.96064e-07_rb,0.20141e-05_rb,0.48491e-05_rb,0.67941e-05_rb,0.85382e-05_rb, &
        & 0.10290e-04_rb,0.12102e-04_rb,0.14626e-04_rb,0.54289e-04_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.10431e-06_rb,0.20834e-05_rb,0.49690e-05_rb,0.69520e-05_rb,0.87398e-05_rb, &
        & 0.10520e-04_rb,0.12369e-04_rb,0.15124e-04_rb,0.55603e-04_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.11432e-06_rb,0.21653e-05_rb,0.50755e-05_rb,0.70852e-05_rb,0.89148e-05_rb, &
        & 0.10714e-04_rb,0.12591e-04_rb,0.15457e-04_rb,0.56630e-04_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.11074e-06_rb,0.22480e-05_rb,0.51732e-05_rb,0.71951e-05_rb,0.90530e-05_rb, &
        & 0.10860e-04_rb,0.12745e-04_rb,0.15721e-04_rb,0.54963e-04_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.15854e-06_rb,0.91297e-06_rb,0.32555e-05_rb,0.55113e-05_rb,0.70464e-05_rb, &
        & 0.85121e-05_rb,0.10027e-04_rb,0.12303e-04_rb,0.52607e-04_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.16751e-06_rb,0.96634e-06_rb,0.33790e-05_rb,0.56722e-05_rb,0.72351e-05_rb, &
        & 0.87519e-05_rb,0.10311e-04_rb,0.12547e-04_rb,0.52883e-04_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.16741e-06_rb,0.10318e-05_rb,0.35064e-05_rb,0.58058e-05_rb,0.74099e-05_rb, &
        & 0.89413e-05_rb,0.10546e-04_rb,0.12998e-04_rb,0.52407e-04_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.16708e-06_rb,0.10925e-05_rb,0.36064e-05_rb,0.59145e-05_rb,0.75531e-05_rb, &
        & 0.90996e-05_rb,0.10715e-04_rb,0.13285e-04_rb,0.53203e-04_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.16578e-06_rb,0.11490e-05_rb,0.36852e-05_rb,0.60104e-05_rb,0.76652e-05_rb, &
        & 0.92182e-05_rb,0.10838e-04_rb,0.13496e-04_rb,0.54467e-04_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.17287e-06_rb,0.67222e-06_rb,0.20177e-05_rb,0.42045e-05_rb,0.58863e-05_rb, &
        & 0.71911e-05_rb,0.84904e-05_rb,0.10467e-04_rb,0.53824e-04_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.17534e-06_rb,0.66930e-06_rb,0.20902e-05_rb,0.43510e-05_rb,0.60518e-05_rb, &
        & 0.73834e-05_rb,0.87285e-05_rb,0.10723e-04_rb,0.57713e-04_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.19862e-06_rb,0.69271e-06_rb,0.21807e-05_rb,0.44473e-05_rb,0.61938e-05_rb, &
        & 0.75395e-05_rb,0.89230e-05_rb,0.11082e-04_rb,0.58727e-04_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.20481e-06_rb,0.70155e-06_rb,0.22437e-05_rb,0.45330e-05_rb,0.63072e-05_rb, &
        & 0.76639e-05_rb,0.90515e-05_rb,0.11316e-04_rb,0.62478e-04_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.22362e-06_rb,0.66686e-06_rb,0.23107e-05_rb,0.46277e-05_rb,0.64004e-05_rb, &
        & 0.77560e-05_rb,0.91460e-05_rb,0.11480e-04_rb,0.61459e-04_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.24394e-06_rb,0.78850e-06_rb,0.10005e-05_rb,0.25869e-05_rb,0.46671e-05_rb, &
        & 0.60110e-05_rb,0.71462e-05_rb,0.88357e-05_rb,0.59707e-04_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.25387e-06_rb,0.77656e-06_rb,0.10565e-05_rb,0.27093e-05_rb,0.48105e-05_rb, &
        & 0.61610e-05_rb,0.73436e-05_rb,0.90989e-05_rb,0.60151e-04_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.22260e-06_rb,0.76604e-06_rb,0.11188e-05_rb,0.28168e-05_rb,0.49259e-05_rb, &
        & 0.62864e-05_rb,0.74912e-05_rb,0.93777e-05_rb,0.58560e-04_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.22499e-06_rb,0.75979e-06_rb,0.11746e-05_rb,0.28869e-05_rb,0.50154e-05_rb, &
        & 0.63819e-05_rb,0.75933e-05_rb,0.95633e-05_rb,0.61656e-04_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.22527e-06_rb,0.74699e-06_rb,0.12195e-05_rb,0.29526e-05_rb,0.50915e-05_rb, &
        & 0.64528e-05_rb,0.76669e-05_rb,0.96909e-05_rb,0.60588e-04_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.29788e-05_rb,0.29380e-04_rb,0.47103e-04_rb,0.62484e-04_rb,0.75923e-04_rb, &
        & 0.86945e-04_rb,0.96459e-04_rb,0.10727e-03_rb,0.95715e-04_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.31829e-05_rb,0.29542e-04_rb,0.47406e-04_rb,0.62987e-04_rb,0.76500e-04_rb, &
        & 0.87575e-04_rb,0.97244e-04_rb,0.10847e-03_rb,0.96769e-04_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.34017e-05_rb,0.30451e-04_rb,0.49181e-04_rb,0.65673e-04_rb,0.79848e-04_rb, &
        & 0.91475e-04_rb,0.10206e-03_rb,0.11418e-03_rb,0.10339e-03_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.36204e-05_rb,0.29858e-04_rb,0.47964e-04_rb,0.63849e-04_rb,0.77416e-04_rb, &
        & 0.88495e-04_rb,0.98624e-04_rb,0.11030e-03_rb,0.98665e-04_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.38336e-05_rb,0.30032e-04_rb,0.48176e-04_rb,0.64174e-04_rb,0.77743e-04_rb, &
        & 0.88899e-04_rb,0.99219e-04_rb,0.11117e-03_rb,0.99410e-04_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.24422e-05_rb,0.27039e-04_rb,0.43472e-04_rb,0.57657e-04_rb,0.69985e-04_rb, &
        & 0.80297e-04_rb,0.89716e-04_rb,0.10091e-03_rb,0.87814e-04_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.26194e-05_rb,0.27224e-04_rb,0.43834e-04_rb,0.58227e-04_rb,0.70564e-04_rb, &
        & 0.80899e-04_rb,0.90555e-04_rb,0.10201e-03_rb,0.88900e-04_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.28026e-05_rb,0.28185e-04_rb,0.45675e-04_rb,0.60941e-04_rb,0.73932e-04_rb, &
        & 0.84933e-04_rb,0.95539e-04_rb,0.10795e-03_rb,0.95932e-04_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.29807e-05_rb,0.28297e-04_rb,0.45818e-04_rb,0.61151e-04_rb,0.74119e-04_rb, &
        & 0.85132e-04_rb,0.95905e-04_rb,0.10857e-03_rb,0.96316e-04_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.31559e-05_rb,0.27777e-04_rb,0.44649e-04_rb,0.59431e-04_rb,0.71821e-04_rb, &
        & 0.82324e-04_rb,0.92537e-04_rb,0.10487e-03_rb,0.91446e-04_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.19527e-05_rb,0.24728e-04_rb,0.39864e-04_rb,0.52777e-04_rb,0.63893e-04_rb, &
        & 0.73441e-04_rb,0.82582e-04_rb,0.93968e-04_rb,0.79571e-04_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.21028e-05_rb,0.24948e-04_rb,0.40273e-04_rb,0.53366e-04_rb,0.64472e-04_rb, &
        & 0.74070e-04_rb,0.83496e-04_rb,0.95166e-04_rb,0.80711e-04_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.22571e-05_rb,0.25146e-04_rb,0.40613e-04_rb,0.53873e-04_rb,0.64998e-04_rb, &
        & 0.74638e-04_rb,0.84298e-04_rb,0.96249e-04_rb,0.81670e-04_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.24052e-05_rb,0.26082e-04_rb,0.42367e-04_rb,0.56407e-04_rb,0.68204e-04_rb, &
        & 0.78600e-04_rb,0.89199e-04_rb,0.10217e-03_rb,0.88502e-04_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.25493e-05_rb,0.25521e-04_rb,0.41156e-04_rb,0.54597e-04_rb,0.65795e-04_rb, &
        & 0.75610e-04_rb,0.85600e-04_rb,0.98234e-04_rb,0.83132e-04_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.15249e-05_rb,0.22458e-04_rb,0.36230e-04_rb,0.47832e-04_rb,0.57788e-04_rb, &
        & 0.66572e-04_rb,0.75267e-04_rb,0.86613e-04_rb,0.71765e-04_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.16472e-05_rb,0.22706e-04_rb,0.36674e-04_rb,0.48420e-04_rb,0.58373e-04_rb, &
        & 0.67233e-04_rb,0.76205e-04_rb,0.87918e-04_rb,0.72872e-04_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.17727e-05_rb,0.22924e-04_rb,0.37041e-04_rb,0.48925e-04_rb,0.58917e-04_rb, &
        & 0.67822e-04_rb,0.77056e-04_rb,0.89127e-04_rb,0.73812e-04_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.18979e-05_rb,0.23116e-04_rb,0.37363e-04_rb,0.49353e-04_rb,0.59386e-04_rb, &
        & 0.68373e-04_rb,0.77816e-04_rb,0.90208e-04_rb,0.74597e-04_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.20187e-05_rb,0.24032e-04_rb,0.39090e-04_rb,0.51784e-04_rb,0.62486e-04_rb, &
        & 0.72274e-04_rb,0.82652e-04_rb,0.95944e-04_rb,0.81360e-04_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.11616e-05_rb,0.20230e-04_rb,0.32593e-04_rb,0.43015e-04_rb,0.51913e-04_rb, &
        & 0.59885e-04_rb,0.68041e-04_rb,0.79091e-04_rb,0.64237e-04_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.12634e-05_rb,0.20493e-04_rb,0.33052e-04_rb,0.43575e-04_rb,0.52485e-04_rb, &
        & 0.60579e-04_rb,0.68993e-04_rb,0.80447e-04_rb,0.65308e-04_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.13665e-05_rb,0.20722e-04_rb,0.33449e-04_rb,0.44078e-04_rb,0.53031e-04_rb, &
        & 0.61206e-04_rb,0.69878e-04_rb,0.81712e-04_rb,0.66219e-04_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.14654e-05_rb,0.20926e-04_rb,0.33806e-04_rb,0.44507e-04_rb,0.53511e-04_rb, &
        & 0.61775e-04_rb,0.70644e-04_rb,0.82800e-04_rb,0.67010e-04_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.15654e-05_rb,0.21919e-04_rb,0.35683e-04_rb,0.47135e-04_rb,0.56892e-04_rb, &
        & 0.66014e-04_rb,0.75901e-04_rb,0.88948e-04_rb,0.74367e-04_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.85477e-06_rb,0.18056e-04_rb,0.29058e-04_rb,0.38374e-04_rb,0.46318e-04_rb, &
        & 0.53503e-04_rb,0.60983e-04_rb,0.71567e-04_rb,0.55220e-04_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.93399e-06_rb,0.18333e-04_rb,0.29518e-04_rb,0.38924e-04_rb,0.46903e-04_rb, &
        & 0.54228e-04_rb,0.61990e-04_rb,0.72960e-04_rb,0.56327e-04_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.10173e-05_rb,0.18568e-04_rb,0.29940e-04_rb,0.39430e-04_rb,0.47457e-04_rb, &
        & 0.54891e-04_rb,0.62912e-04_rb,0.74219e-04_rb,0.57262e-04_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.10990e-05_rb,0.18773e-04_rb,0.30321e-04_rb,0.39858e-04_rb,0.47937e-04_rb, &
        & 0.55463e-04_rb,0.63685e-04_rb,0.75339e-04_rb,0.58060e-04_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.11736e-05_rb,0.18980e-04_rb,0.30632e-04_rb,0.40210e-04_rb,0.48338e-04_rb, &
        & 0.55952e-04_rb,0.64349e-04_rb,0.76243e-04_rb,0.58787e-04_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.60315e-06_rb,0.15982e-04_rb,0.25725e-04_rb,0.33991e-04_rb,0.41066e-04_rb, &
        & 0.47516e-04_rb,0.54302e-04_rb,0.64196e-04_rb,0.19175e-04_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.66792e-06_rb,0.16270e-04_rb,0.26184e-04_rb,0.34534e-04_rb,0.41673e-04_rb, &
        & 0.48253e-04_rb,0.55322e-04_rb,0.65617e-04_rb,0.20214e-04_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.72908e-06_rb,0.16517e-04_rb,0.26616e-04_rb,0.35037e-04_rb,0.42229e-04_rb, &
        & 0.48924e-04_rb,0.56239e-04_rb,0.66903e-04_rb,0.21171e-04_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.78570e-06_rb,0.16735e-04_rb,0.27002e-04_rb,0.35463e-04_rb,0.42700e-04_rb, &
        & 0.49504e-04_rb,0.57015e-04_rb,0.68011e-04_rb,0.22118e-04_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.83832e-06_rb,0.16948e-04_rb,0.27325e-04_rb,0.35815e-04_rb,0.43102e-04_rb, &
        & 0.50003e-04_rb,0.57693e-04_rb,0.68917e-04_rb,0.22989e-04_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.37835e-06_rb,0.14042e-04_rb,0.22619e-04_rb,0.29906e-04_rb,0.36184e-04_rb, &
        & 0.41943e-04_rb,0.48058e-04_rb,0.57159e-04_rb,0.15030e-04_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.42400e-06_rb,0.14336e-04_rb,0.23076e-04_rb,0.30441e-04_rb,0.36790e-04_rb, &
        & 0.42677e-04_rb,0.49055e-04_rb,0.58574e-04_rb,0.15267e-04_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.47003e-06_rb,0.14598e-04_rb,0.23503e-04_rb,0.30926e-04_rb,0.37338e-04_rb, &
        & 0.43345e-04_rb,0.49944e-04_rb,0.59871e-04_rb,0.15168e-04_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.50983e-06_rb,0.14832e-04_rb,0.23887e-04_rb,0.31342e-04_rb,0.37806e-04_rb, &
        & 0.43932e-04_rb,0.50714e-04_rb,0.60954e-04_rb,0.15479e-04_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.55238e-06_rb,0.15050e-04_rb,0.24211e-04_rb,0.31698e-04_rb,0.38215e-04_rb, &
        & 0.44447e-04_rb,0.51404e-04_rb,0.61843e-04_rb,0.15864e-04_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.20614e-06_rb,0.12247e-04_rb,0.19755e-04_rb,0.26136e-04_rb,0.31671e-04_rb, &
        & 0.36795e-04_rb,0.42267e-04_rb,0.50480e-04_rb,0.31415e-04_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.22820e-06_rb,0.12543e-04_rb,0.20203e-04_rb,0.26660e-04_rb,0.32273e-04_rb, &
        & 0.37518e-04_rb,0.43228e-04_rb,0.51895e-04_rb,0.32257e-04_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.25381e-06_rb,0.12814e-04_rb,0.20621e-04_rb,0.27129e-04_rb,0.32814e-04_rb, &
        & 0.38186e-04_rb,0.44095e-04_rb,0.53151e-04_rb,0.31766e-04_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.26266e-06_rb,0.13056e-04_rb,0.20991e-04_rb,0.27538e-04_rb,0.33285e-04_rb, &
        & 0.38781e-04_rb,0.44859e-04_rb,0.54214e-04_rb,0.32184e-04_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.28878e-06_rb,0.13277e-04_rb,0.21308e-04_rb,0.27899e-04_rb,0.33704e-04_rb, &
        & 0.39313e-04_rb,0.45556e-04_rb,0.55111e-04_rb,0.32436e-04_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.12421e-06_rb,0.10598e-04_rb,0.17170e-04_rb,0.22745e-04_rb,0.27597e-04_rb, &
        & 0.32129e-04_rb,0.37024e-04_rb,0.44410e-04_rb,0.36288e-04_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.11833e-06_rb,0.10888e-04_rb,0.17606e-04_rb,0.23251e-04_rb,0.28191e-04_rb, &
        & 0.32843e-04_rb,0.37957e-04_rb,0.45757e-04_rb,0.36706e-04_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.11564e-06_rb,0.11153e-04_rb,0.18009e-04_rb,0.23706e-04_rb,0.28726e-04_rb, &
        & 0.33507e-04_rb,0.38804e-04_rb,0.46971e-04_rb,0.39285e-04_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.11888e-06_rb,0.11400e-04_rb,0.18364e-04_rb,0.24104e-04_rb,0.29196e-04_rb, &
        & 0.34101e-04_rb,0.39555e-04_rb,0.47996e-04_rb,0.37765e-04_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.13250e-06_rb,0.11625e-04_rb,0.18671e-04_rb,0.24463e-04_rb,0.29622e-04_rb, &
        & 0.34646e-04_rb,0.40249e-04_rb,0.48876e-04_rb,0.41184e-04_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.17958e-06_rb,0.90757e-05_rb,0.15007e-04_rb,0.19889e-04_rb,0.24154e-04_rb, &
        & 0.28187e-04_rb,0.32636e-04_rb,0.39437e-04_rb,0.34162e-04_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.18139e-06_rb,0.93666e-05_rb,0.15418e-04_rb,0.20361e-04_rb,0.24716e-04_rb, &
        & 0.28864e-04_rb,0.33507e-04_rb,0.40680e-04_rb,0.35787e-04_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.18275e-06_rb,0.96291e-05_rb,0.15787e-04_rb,0.20786e-04_rb,0.25221e-04_rb, &
        & 0.29495e-04_rb,0.34295e-04_rb,0.41765e-04_rb,0.33878e-04_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.18574e-06_rb,0.98743e-05_rb,0.16117e-04_rb,0.21165e-04_rb,0.25673e-04_rb, &
        & 0.30069e-04_rb,0.35015e-04_rb,0.42686e-04_rb,0.32841e-04_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.20082e-06_rb,0.10102e-04_rb,0.16409e-04_rb,0.21512e-04_rb,0.26096e-04_rb, &
        & 0.30600e-04_rb,0.35678e-04_rb,0.43526e-04_rb,0.33904e-04_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.23860e-06_rb,0.72127e-05_rb,0.13015e-04_rb,0.17297e-04_rb,0.21033e-04_rb, &
        & 0.24600e-04_rb,0.28604e-04_rb,0.34818e-04_rb,0.40933e-04_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.23391e-06_rb,0.75352e-05_rb,0.13408e-04_rb,0.17740e-04_rb,0.21564e-04_rb, &
        & 0.25251e-04_rb,0.29426e-04_rb,0.35947e-04_rb,0.39399e-04_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.24701e-06_rb,0.78057e-05_rb,0.13760e-04_rb,0.18144e-04_rb,0.22045e-04_rb, &
        & 0.25844e-04_rb,0.30162e-04_rb,0.36924e-04_rb,0.38294e-04_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.26761e-06_rb,0.80774e-05_rb,0.14071e-04_rb,0.18509e-04_rb,0.22485e-04_rb, &
        & 0.26399e-04_rb,0.30850e-04_rb,0.37793e-04_rb,0.37310e-04_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.23665e-06_rb,0.83711e-05_rb,0.14360e-04_rb,0.18850e-04_rb,0.22903e-04_rb, &
        & 0.26918e-04_rb,0.31485e-04_rb,0.38601e-04_rb,0.34593e-04_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.27521e-06_rb,0.50755e-05_rb,0.11059e-04_rb,0.14942e-04_rb,0.18221e-04_rb, &
        & 0.21366e-04_rb,0.24941e-04_rb,0.30569e-04_rb,0.35909e-04_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.25317e-06_rb,0.53749e-05_rb,0.11450e-04_rb,0.15364e-04_rb,0.18725e-04_rb, &
        & 0.21982e-04_rb,0.25707e-04_rb,0.31598e-04_rb,0.37411e-04_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.23895e-06_rb,0.56684e-05_rb,0.11800e-04_rb,0.15745e-04_rb,0.19182e-04_rb, &
        & 0.22548e-04_rb,0.26403e-04_rb,0.32490e-04_rb,0.35715e-04_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.23323e-06_rb,0.59219e-05_rb,0.12110e-04_rb,0.16104e-04_rb,0.19621e-04_rb, &
        & 0.23090e-04_rb,0.27069e-04_rb,0.33309e-04_rb,0.26489e-04_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.23115e-06_rb,0.61949e-05_rb,0.12408e-04_rb,0.16438e-04_rb,0.20031e-04_rb, &
        & 0.23602e-04_rb,0.27683e-04_rb,0.34068e-04_rb,0.26612e-04_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.91507e-04_rb,0.14710e-03_rb,0.19128e-03_rb,0.22605e-03_rb,0.25532e-03_rb, &
        & 0.28077e-03_rb,0.30345e-03_rb,0.32646e-03_rb,0.30807e-03_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.91242e-04_rb,0.14642e-03_rb,0.19074e-03_rb,0.22564e-03_rb,0.25500e-03_rb, &
        & 0.28086e-03_rb,0.30369e-03_rb,0.32690e-03_rb,0.30957e-03_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.90877e-04_rb,0.14692e-03_rb,0.19251e-03_rb,0.22869e-03_rb,0.25946e-03_rb, &
        & 0.28682e-03_rb,0.31087e-03_rb,0.33538e-03_rb,0.32011e-03_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.90459e-04_rb,0.14494e-03_rb,0.18943e-03_rb,0.22459e-03_rb,0.25465e-03_rb, &
        & 0.28114e-03_rb,0.30414e-03_rb,0.32755e-03_rb,0.31196e-03_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.89993e-04_rb,0.14416e-03_rb,0.18875e-03_rb,0.22408e-03_rb,0.25453e-03_rb, &
        & 0.28137e-03_rb,0.30449e-03_rb,0.32780e-03_rb,0.31307e-03_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.77262e-04_rb,0.13432e-03_rb,0.17769e-03_rb,0.21240e-03_rb,0.24204e-03_rb, &
        & 0.26866e-03_rb,0.29235e-03_rb,0.31628e-03_rb,0.29582e-03_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.77061e-04_rb,0.13375e-03_rb,0.17727e-03_rb,0.21210e-03_rb,0.24206e-03_rb, &
        & 0.26905e-03_rb,0.29288e-03_rb,0.31707e-03_rb,0.29754e-03_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.76803e-04_rb,0.13443e-03_rb,0.17926e-03_rb,0.21557e-03_rb,0.24713e-03_rb, &
        & 0.27557e-03_rb,0.30072e-03_rb,0.32610e-03_rb,0.30880e-03_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.76502e-04_rb,0.13372e-03_rb,0.17855e-03_rb,0.21503e-03_rb,0.24689e-03_rb, &
        & 0.27560e-03_rb,0.30084e-03_rb,0.32601e-03_rb,0.30963e-03_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.76149e-04_rb,0.13198e-03_rb,0.17568e-03_rb,0.21133e-03_rb,0.24254e-03_rb, &
        & 0.27051e-03_rb,0.29485e-03_rb,0.31890e-03_rb,0.30223e-03_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.65103e-04_rb,0.12258e-03_rb,0.16452e-03_rb,0.19865e-03_rb,0.22866e-03_rb, &
        & 0.25591e-03_rb,0.28055e-03_rb,0.30511e-03_rb,0.28178e-03_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.64961e-04_rb,0.12216e-03_rb,0.16424e-03_rb,0.19863e-03_rb,0.22903e-03_rb, &
        & 0.25664e-03_rb,0.28152e-03_rb,0.30626e-03_rb,0.28379e-03_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.64796e-04_rb,0.12170e-03_rb,0.16393e-03_rb,0.19861e-03_rb,0.22948e-03_rb, &
        & 0.25740e-03_rb,0.28253e-03_rb,0.30727e-03_rb,0.28578e-03_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.64583e-04_rb,0.12248e-03_rb,0.16600e-03_rb,0.20227e-03_rb,0.23481e-03_rb, &
        & 0.26421e-03_rb,0.29055e-03_rb,0.31636e-03_rb,0.29736e-03_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.64341e-04_rb,0.12081e-03_rb,0.16325e-03_rb,0.19869e-03_rb,0.23043e-03_rb, &
        & 0.25903e-03_rb,0.28445e-03_rb,0.30920e-03_rb,0.28976e-03_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.54784e-04_rb,0.11174e-03_rb,0.15188e-03_rb,0.18523e-03_rb,0.21503e-03_rb, &
        & 0.24239e-03_rb,0.26770e-03_rb,0.29311e-03_rb,0.26685e-03_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.54700e-04_rb,0.11144e-03_rb,0.15177e-03_rb,0.18550e-03_rb,0.21578e-03_rb, &
        & 0.24348e-03_rb,0.26903e-03_rb,0.29466e-03_rb,0.26928e-03_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.54592e-04_rb,0.11111e-03_rb,0.15167e-03_rb,0.18578e-03_rb,0.21655e-03_rb, &
        & 0.24456e-03_rb,0.27029e-03_rb,0.29609e-03_rb,0.27162e-03_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.54457e-04_rb,0.11078e-03_rb,0.15152e-03_rb,0.18607e-03_rb,0.21728e-03_rb, &
        & 0.24565e-03_rb,0.27151e-03_rb,0.29746e-03_rb,0.27399e-03_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.54312e-04_rb,0.11162e-03_rb,0.15369e-03_rb,0.18993e-03_rb,0.22281e-03_rb, &
        & 0.25263e-03_rb,0.27972e-03_rb,0.30664e-03_rb,0.28593e-03_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.46088e-04_rb,0.10164e-03_rb,0.13985e-03_rb,0.17192e-03_rb,0.20105e-03_rb, &
        & 0.22810e-03_rb,0.25365e-03_rb,0.28019e-03_rb,0.25169e-03_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.46037e-04_rb,0.10150e-03_rb,0.13996e-03_rb,0.17251e-03_rb,0.20208e-03_rb, &
        & 0.22949e-03_rb,0.25532e-03_rb,0.28211e-03_rb,0.25438e-03_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.45971e-04_rb,0.10129e-03_rb,0.14008e-03_rb,0.17302e-03_rb,0.20310e-03_rb, &
        & 0.23085e-03_rb,0.25690e-03_rb,0.28385e-03_rb,0.25702e-03_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.45890e-04_rb,0.10111e-03_rb,0.14015e-03_rb,0.17355e-03_rb,0.20405e-03_rb, &
        & 0.23221e-03_rb,0.25843e-03_rb,0.28547e-03_rb,0.25970e-03_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.45808e-04_rb,0.10210e-03_rb,0.14263e-03_rb,0.17783e-03_rb,0.21010e-03_rb, &
        & 0.23994e-03_rb,0.26750e-03_rb,0.29545e-03_rb,0.27288e-03_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.38759e-04_rb,0.92287e-04_rb,0.12830e-03_rb,0.15873e-03_rb,0.18673e-03_rb, &
        & 0.21305e-03_rb,0.23858e-03_rb,0.26603e-03_rb,0.23576e-03_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.38742e-04_rb,0.92285e-04_rb,0.12860e-03_rb,0.15956e-03_rb,0.18804e-03_rb, &
        & 0.21479e-03_rb,0.24061e-03_rb,0.26833e-03_rb,0.23876e-03_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.38716e-04_rb,0.92248e-04_rb,0.12890e-03_rb,0.16031e-03_rb,0.18934e-03_rb, &
        & 0.21647e-03_rb,0.24252e-03_rb,0.27042e-03_rb,0.24167e-03_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.38672e-04_rb,0.92189e-04_rb,0.12914e-03_rb,0.16108e-03_rb,0.19055e-03_rb, &
        & 0.21815e-03_rb,0.24441e-03_rb,0.27232e-03_rb,0.24468e-03_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.38631e-04_rb,0.92110e-04_rb,0.12940e-03_rb,0.16181e-03_rb,0.19180e-03_rb, &
        & 0.21982e-03_rb,0.24634e-03_rb,0.27420e-03_rb,0.24778e-03_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.32524e-04_rb,0.83578e-04_rb,0.11715e-03_rb,0.14581e-03_rb,0.17242e-03_rb, &
        & 0.19778e-03_rb,0.22289e-03_rb,0.25099e-03_rb,0.21544e-03_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.32544e-04_rb,0.83724e-04_rb,0.11765e-03_rb,0.14685e-03_rb,0.17402e-03_rb, &
        & 0.19984e-03_rb,0.22531e-03_rb,0.25360e-03_rb,0.21897e-03_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.32541e-04_rb,0.83837e-04_rb,0.11812e-03_rb,0.14785e-03_rb,0.17560e-03_rb, &
        & 0.20184e-03_rb,0.22764e-03_rb,0.25604e-03_rb,0.22234e-03_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.32522e-04_rb,0.83911e-04_rb,0.11855e-03_rb,0.14886e-03_rb,0.17710e-03_rb, &
        & 0.20381e-03_rb,0.22989e-03_rb,0.25831e-03_rb,0.22586e-03_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.32511e-04_rb,0.83972e-04_rb,0.11899e-03_rb,0.14985e-03_rb,0.17863e-03_rb, &
        & 0.20577e-03_rb,0.23212e-03_rb,0.26056e-03_rb,0.22930e-03_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.27162e-04_rb,0.75477e-04_rb,0.10657e-03_rb,0.13337e-03_rb,0.15848e-03_rb, &
        & 0.18267e-03_rb,0.20707e-03_rb,0.23532e-03_rb,0.10685e-03_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.27219e-04_rb,0.75747e-04_rb,0.10724e-03_rb,0.13464e-03_rb,0.16036e-03_rb, &
        & 0.18507e-03_rb,0.20988e-03_rb,0.23834e-03_rb,0.11049e-03_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.27240e-04_rb,0.75981e-04_rb,0.10788e-03_rb,0.13591e-03_rb,0.16217e-03_rb, &
        & 0.18731e-03_rb,0.21250e-03_rb,0.24115e-03_rb,0.11463e-03_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.27241e-04_rb,0.76175e-04_rb,0.10851e-03_rb,0.13715e-03_rb,0.16392e-03_rb, &
        & 0.18952e-03_rb,0.21504e-03_rb,0.24375e-03_rb,0.11830e-03_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.27254e-04_rb,0.76338e-04_rb,0.10913e-03_rb,0.13836e-03_rb,0.16569e-03_rb, &
        & 0.19172e-03_rb,0.21755e-03_rb,0.24638e-03_rb,0.12227e-03_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.22546e-04_rb,0.67903e-04_rb,0.96628e-04_rb,0.12150e-03_rb,0.14489e-03_rb, &
        & 0.16757e-03_rb,0.19095e-03_rb,0.21913e-03_rb,0.25626e-04_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.22635e-04_rb,0.68268e-04_rb,0.97460e-04_rb,0.12301e-03_rb,0.14702e-03_rb, &
        & 0.17028e-03_rb,0.19411e-03_rb,0.22260e-03_rb,0.25839e-04_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.22685e-04_rb,0.68616e-04_rb,0.98277e-04_rb,0.12450e-03_rb,0.14909e-03_rb, &
        & 0.17284e-03_rb,0.19709e-03_rb,0.22578e-03_rb,0.27443e-04_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.22709e-04_rb,0.68929e-04_rb,0.99095e-04_rb,0.12593e-03_rb,0.15108e-03_rb, &
        & 0.17530e-03_rb,0.19995e-03_rb,0.22878e-03_rb,0.28531e-04_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.22740e-04_rb,0.69227e-04_rb,0.99896e-04_rb,0.12734e-03_rb,0.15306e-03_rb, &
        & 0.17777e-03_rb,0.20277e-03_rb,0.23170e-03_rb,0.28894e-04_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.18403e-04_rb,0.60856e-04_rb,0.87340e-04_rb,0.11017e-03_rb,0.13174e-03_rb, &
        & 0.15284e-03_rb,0.17492e-03_rb,0.20267e-03_rb,0.31931e-04_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.18527e-04_rb,0.61329e-04_rb,0.88371e-04_rb,0.11191e-03_rb,0.13412e-03_rb, &
        & 0.15579e-03_rb,0.17838e-03_rb,0.20650e-03_rb,0.29084e-04_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.18600e-04_rb,0.61796e-04_rb,0.89354e-04_rb,0.11363e-03_rb,0.13643e-03_rb, &
        & 0.15861e-03_rb,0.18170e-03_rb,0.21009e-03_rb,0.27893e-04_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.18642e-04_rb,0.62226e-04_rb,0.90339e-04_rb,0.11526e-03_rb,0.13867e-03_rb, &
        & 0.16137e-03_rb,0.18486e-03_rb,0.21346e-03_rb,0.33895e-04_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.18688e-04_rb,0.62651e-04_rb,0.91302e-04_rb,0.11690e-03_rb,0.14093e-03_rb, &
        & 0.16416e-03_rb,0.18804e-03_rb,0.21676e-03_rb,0.31724e-04_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.14636e-04_rb,0.54588e-04_rb,0.79114e-04_rb,0.10030e-03_rb,0.12035e-03_rb, &
        & 0.14003e-03_rb,0.16089e-03_rb,0.18797e-03_rb,0.24886e-04_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.14721e-04_rb,0.55182e-04_rb,0.80295e-04_rb,0.10221e-03_rb,0.12288e-03_rb, &
        & 0.14315e-03_rb,0.16455e-03_rb,0.19200e-03_rb,0.28280e-04_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.14784e-04_rb,0.55751e-04_rb,0.81465e-04_rb,0.10405e-03_rb,0.12535e-03_rb, &
        & 0.14614e-03_rb,0.16802e-03_rb,0.19578e-03_rb,0.31405e-04_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.14831e-04_rb,0.56293e-04_rb,0.82611e-04_rb,0.10589e-03_rb,0.12782e-03_rb, &
        & 0.14918e-03_rb,0.17147e-03_rb,0.19942e-03_rb,0.34684e-04_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.14856e-04_rb,0.56829e-04_rb,0.83754e-04_rb,0.10776e-03_rb,0.13034e-03_rb, &
        & 0.15225e-03_rb,0.17499e-03_rb,0.20312e-03_rb,0.30970e-04_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.11572e-04_rb,0.48876e-04_rb,0.71553e-04_rb,0.91191e-04_rb,0.10973e-03_rb, &
        & 0.12799e-03_rb,0.14756e-03_rb,0.17351e-03_rb,0.30682e-04_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.11642e-04_rb,0.49592e-04_rb,0.72882e-04_rb,0.93233e-04_rb,0.11240e-03_rb, &
        & 0.13124e-03_rb,0.15132e-03_rb,0.17770e-03_rb,0.31240e-04_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.11629e-04_rb,0.50261e-04_rb,0.74205e-04_rb,0.95210e-04_rb,0.11505e-03_rb, &
        & 0.13447e-03_rb,0.15502e-03_rb,0.18170e-03_rb,0.31943e-04_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.11633e-04_rb,0.50925e-04_rb,0.75501e-04_rb,0.97246e-04_rb,0.11773e-03_rb, &
        & 0.13775e-03_rb,0.15878e-03_rb,0.18567e-03_rb,0.23441e-04_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.11677e-04_rb,0.51560e-04_rb,0.76851e-04_rb,0.99310e-04_rb,0.12044e-03_rb, &
        & 0.14103e-03_rb,0.16255e-03_rb,0.18968e-03_rb,0.23249e-04_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.88185e-05_rb,0.43709e-04_rb,0.64627e-04_rb,0.82824e-04_rb,0.99889e-04_rb, &
        & 0.11672e-03_rb,0.13488e-03_rb,0.15952e-03_rb,0.30903e-04_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.88986e-05_rb,0.44515e-04_rb,0.66093e-04_rb,0.84980e-04_rb,0.10270e-03_rb, &
        & 0.12016e-03_rb,0.13885e-03_rb,0.16386e-03_rb,0.25840e-04_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.89568e-05_rb,0.45280e-04_rb,0.67556e-04_rb,0.87134e-04_rb,0.10555e-03_rb, &
        & 0.12361e-03_rb,0.14282e-03_rb,0.16813e-03_rb,0.26878e-04_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.89829e-05_rb,0.46045e-04_rb,0.69040e-04_rb,0.89346e-04_rb,0.10842e-03_rb, &
        & 0.12710e-03_rb,0.14682e-03_rb,0.17243e-03_rb,0.28852e-04_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.90413e-05_rb,0.46787e-04_rb,0.70568e-04_rb,0.91561e-04_rb,0.11129e-03_rb, &
        & 0.13056e-03_rb,0.15080e-03_rb,0.17669e-03_rb,0.28623e-04_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.44112e-03_rb,0.47423e-03_rb,0.54876e-03_rb,0.61567e-03_rb,0.66971e-03_rb, &
        & 0.71699e-03_rb,0.76347e-03_rb,0.77814e-03_rb,0.77298e-03_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.43794e-03_rb,0.47285e-03_rb,0.54643e-03_rb,0.61370e-03_rb,0.66959e-03_rb, &
        & 0.71530e-03_rb,0.76226e-03_rb,0.77693e-03_rb,0.76966e-03_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.43480e-03_rb,0.47183e-03_rb,0.54721e-03_rb,0.61831e-03_rb,0.67726e-03_rb, &
        & 0.72423e-03_rb,0.77288e-03_rb,0.78901e-03_rb,0.78603e-03_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.43220e-03_rb,0.46825e-03_rb,0.54213e-03_rb,0.61349e-03_rb,0.66880e-03_rb, &
        & 0.71215e-03_rb,0.75938e-03_rb,0.77341e-03_rb,0.76775e-03_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.42987e-03_rb,0.46597e-03_rb,0.54025e-03_rb,0.61237e-03_rb,0.66800e-03_rb, &
        & 0.70968e-03_rb,0.75599e-03_rb,0.77132e-03_rb,0.76754e-03_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.37934e-03_rb,0.43540e-03_rb,0.51852e-03_rb,0.59327e-03_rb,0.65919e-03_rb, &
        & 0.71028e-03_rb,0.75523e-03_rb,0.78660e-03_rb,0.77018e-03_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.37616e-03_rb,0.43418e-03_rb,0.51743e-03_rb,0.59232e-03_rb,0.65869e-03_rb, &
        & 0.70879e-03_rb,0.75385e-03_rb,0.78506e-03_rb,0.76914e-03_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.37285e-03_rb,0.43338e-03_rb,0.51992e-03_rb,0.59796e-03_rb,0.66588e-03_rb, &
        & 0.71863e-03_rb,0.76525e-03_rb,0.79746e-03_rb,0.78726e-03_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.36993e-03_rb,0.43092e-03_rb,0.51858e-03_rb,0.59729e-03_rb,0.66425e-03_rb, &
        & 0.71582e-03_rb,0.76129e-03_rb,0.79428e-03_rb,0.78557e-03_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.36795e-03_rb,0.42667e-03_rb,0.51498e-03_rb,0.59125e-03_rb,0.65451e-03_rb, &
        & 0.70295e-03_rb,0.74646e-03_rb,0.77706e-03_rb,0.76846e-03_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.32398e-03_rb,0.39840e-03_rb,0.48910e-03_rb,0.57202e-03_rb,0.64247e-03_rb, &
        & 0.69920e-03_rb,0.74272e-03_rb,0.78560e-03_rb,0.76158e-03_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.32107e-03_rb,0.39683e-03_rb,0.48877e-03_rb,0.57106e-03_rb,0.64200e-03_rb, &
        & 0.69780e-03_rb,0.74068e-03_rb,0.78389e-03_rb,0.76255e-03_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.31808e-03_rb,0.39502e-03_rb,0.48851e-03_rb,0.57104e-03_rb,0.64052e-03_rb, &
        & 0.69677e-03_rb,0.73846e-03_rb,0.78156e-03_rb,0.76341e-03_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.31534e-03_rb,0.39419e-03_rb,0.49223e-03_rb,0.57621e-03_rb,0.64746e-03_rb, &
        & 0.70569e-03_rb,0.74986e-03_rb,0.79350e-03_rb,0.78192e-03_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.31343e-03_rb,0.39031e-03_rb,0.48789e-03_rb,0.56911e-03_rb,0.63804e-03_rb, &
        & 0.69322e-03_rb,0.73529e-03_rb,0.77554e-03_rb,0.76249e-03_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.27516e-03_rb,0.36405e-03_rb,0.46107e-03_rb,0.54790e-03_rb,0.62253e-03_rb, &
        & 0.68384e-03_rb,0.73057e-03_rb,0.77856e-03_rb,0.74793e-03_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.27272e-03_rb,0.36262e-03_rb,0.46080e-03_rb,0.54700e-03_rb,0.62201e-03_rb, &
        & 0.68300e-03_rb,0.72976e-03_rb,0.77591e-03_rb,0.74919e-03_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.26991e-03_rb,0.36139e-03_rb,0.46064e-03_rb,0.54673e-03_rb,0.62063e-03_rb, &
        & 0.68269e-03_rb,0.72862e-03_rb,0.77298e-03_rb,0.75070e-03_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.26757e-03_rb,0.35993e-03_rb,0.46053e-03_rb,0.54625e-03_rb,0.61982e-03_rb, &
        & 0.68167e-03_rb,0.72799e-03_rb,0.77005e-03_rb,0.75046e-03_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.26556e-03_rb,0.36025e-03_rb,0.46433e-03_rb,0.55132e-03_rb,0.62683e-03_rb, &
        & 0.69143e-03_rb,0.74137e-03_rb,0.78366e-03_rb,0.76845e-03_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.23277e-03_rb,0.33279e-03_rb,0.43313e-03_rb,0.52236e-03_rb,0.59970e-03_rb, &
        & 0.66464e-03_rb,0.71686e-03_rb,0.76626e-03_rb,0.72457e-03_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.23078e-03_rb,0.33167e-03_rb,0.43303e-03_rb,0.52181e-03_rb,0.60054e-03_rb, &
        & 0.66529e-03_rb,0.71807e-03_rb,0.76458e-03_rb,0.72852e-03_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.22839e-03_rb,0.33111e-03_rb,0.43245e-03_rb,0.52229e-03_rb,0.60068e-03_rb, &
        & 0.66640e-03_rb,0.71880e-03_rb,0.76367e-03_rb,0.73165e-03_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.22634e-03_rb,0.33010e-03_rb,0.43234e-03_rb,0.52220e-03_rb,0.60179e-03_rb, &
        & 0.66699e-03_rb,0.71996e-03_rb,0.76305e-03_rb,0.73316e-03_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.22442e-03_rb,0.33158e-03_rb,0.43680e-03_rb,0.52864e-03_rb,0.61018e-03_rb, &
        & 0.67770e-03_rb,0.73371e-03_rb,0.78031e-03_rb,0.75138e-03_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.19685e-03_rb,0.30442e-03_rb,0.40547e-03_rb,0.49610e-03_rb,0.57453e-03_rb, &
        & 0.64299e-03_rb,0.69899e-03_rb,0.74954e-03_rb,0.69460e-03_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.19517e-03_rb,0.30338e-03_rb,0.40585e-03_rb,0.49683e-03_rb,0.57671e-03_rb, &
        & 0.64425e-03_rb,0.70130e-03_rb,0.75004e-03_rb,0.70036e-03_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.19324e-03_rb,0.30268e-03_rb,0.40573e-03_rb,0.49834e-03_rb,0.57803e-03_rb, &
        & 0.64627e-03_rb,0.70310e-03_rb,0.75130e-03_rb,0.70596e-03_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.19132e-03_rb,0.30224e-03_rb,0.40644e-03_rb,0.49947e-03_rb,0.58014e-03_rb, &
        & 0.64741e-03_rb,0.70472e-03_rb,0.75293e-03_rb,0.70917e-03_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.18952e-03_rb,0.30177e-03_rb,0.40672e-03_rb,0.50101e-03_rb,0.58137e-03_rb, &
        & 0.64847e-03_rb,0.70602e-03_rb,0.75537e-03_rb,0.71121e-03_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.16684e-03_rb,0.27825e-03_rb,0.37860e-03_rb,0.46883e-03_rb,0.54680e-03_rb, &
        & 0.61510e-03_rb,0.67386e-03_rb,0.72824e-03_rb,0.66391e-03_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.16553e-03_rb,0.27719e-03_rb,0.37963e-03_rb,0.47077e-03_rb,0.54985e-03_rb, &
        & 0.61768e-03_rb,0.67717e-03_rb,0.73093e-03_rb,0.67156e-03_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.16397e-03_rb,0.27663e-03_rb,0.38054e-03_rb,0.47308e-03_rb,0.55166e-03_rb, &
        & 0.62067e-03_rb,0.67975e-03_rb,0.73342e-03_rb,0.67787e-03_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.16246e-03_rb,0.27634e-03_rb,0.38198e-03_rb,0.47489e-03_rb,0.55414e-03_rb, &
        & 0.62280e-03_rb,0.68232e-03_rb,0.73600e-03_rb,0.68175e-03_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.16089e-03_rb,0.27624e-03_rb,0.38314e-03_rb,0.47665e-03_rb,0.55585e-03_rb, &
        & 0.62443e-03_rb,0.68436e-03_rb,0.73852e-03_rb,0.68559e-03_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.14215e-03_rb,0.25407e-03_rb,0.35215e-03_rb,0.43863e-03_rb,0.51443e-03_rb, &
        & 0.58154e-03_rb,0.64180e-03_rb,0.70004e-03_rb,0.62276e-03_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.14104e-03_rb,0.25332e-03_rb,0.35394e-03_rb,0.44179e-03_rb,0.51810e-03_rb, &
        & 0.58508e-03_rb,0.64584e-03_rb,0.70407e-03_rb,0.63285e-03_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.13977e-03_rb,0.25313e-03_rb,0.35560e-03_rb,0.44458e-03_rb,0.52101e-03_rb, &
        & 0.58953e-03_rb,0.65037e-03_rb,0.70785e-03_rb,0.64108e-03_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.13869e-03_rb,0.25332e-03_rb,0.35744e-03_rb,0.44722e-03_rb,0.52464e-03_rb, &
        & 0.59336e-03_rb,0.65468e-03_rb,0.71201e-03_rb,0.64611e-03_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.13742e-03_rb,0.25367e-03_rb,0.35927e-03_rb,0.44959e-03_rb,0.52752e-03_rb, &
        & 0.59656e-03_rb,0.65824e-03_rb,0.71554e-03_rb,0.65066e-03_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.12109e-03_rb,0.23210e-03_rb,0.32446e-03_rb,0.40715e-03_rb,0.48065e-03_rb, &
        & 0.54715e-03_rb,0.60775e-03_rb,0.66825e-03_rb,0.18336e-04_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.12018e-03_rb,0.23207e-03_rb,0.32699e-03_rb,0.41058e-03_rb,0.48505e-03_rb, &
        & 0.55180e-03_rb,0.61323e-03_rb,0.67309e-03_rb,0.24327e-04_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.11917e-03_rb,0.23229e-03_rb,0.32956e-03_rb,0.41390e-03_rb,0.48910e-03_rb, &
        & 0.55710e-03_rb,0.61911e-03_rb,0.67851e-03_rb,0.23065e-04_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.11843e-03_rb,0.23292e-03_rb,0.33197e-03_rb,0.41764e-03_rb,0.49354e-03_rb, &
        & 0.56238e-03_rb,0.62497e-03_rb,0.68395e-03_rb,0.23680e-04_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.11748e-03_rb,0.23365e-03_rb,0.33458e-03_rb,0.42109e-03_rb,0.49774e-03_rb, &
        & 0.56667e-03_rb,0.62987e-03_rb,0.68909e-03_rb,0.25905e-04_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.10273e-03_rb,0.21204e-03_rb,0.29794e-03_rb,0.37761e-03_rb,0.44937e-03_rb, &
        & 0.51495e-03_rb,0.57567e-03_rb,0.63761e-03_rb,0.19724e-04_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.10190e-03_rb,0.21272e-03_rb,0.30059e-03_rb,0.38150e-03_rb,0.45434e-03_rb, &
        & 0.52089e-03_rb,0.58270e-03_rb,0.64346e-03_rb,0.24197e-04_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.10112e-03_rb,0.21334e-03_rb,0.30377e-03_rb,0.38507e-03_rb,0.45917e-03_rb, &
        & 0.52739e-03_rb,0.58976e-03_rb,0.64941e-03_rb,0.34326e-04_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.10061e-03_rb,0.21443e-03_rb,0.30659e-03_rb,0.38965e-03_rb,0.46453e-03_rb, &
        & 0.53354e-03_rb,0.59675e-03_rb,0.65639e-03_rb,0.33702e-04_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.99890e-04_rb,0.21543e-03_rb,0.30998e-03_rb,0.39375e-03_rb,0.46925e-03_rb, &
        & 0.53804e-03_rb,0.60223e-03_rb,0.66255e-03_rb,0.31412e-04_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.86338e-04_rb,0.19398e-03_rb,0.27516e-03_rb,0.35204e-03_rb,0.42116e-03_rb, &
        & 0.48556e-03_rb,0.54615e-03_rb,0.60927e-03_rb,0.19945e-04_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.85730e-04_rb,0.19502e-03_rb,0.27834e-03_rb,0.35636e-03_rb,0.42792e-03_rb, &
        & 0.49403e-03_rb,0.55597e-03_rb,0.61737e-03_rb,0.90093e-05_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.85275e-04_rb,0.19618e-03_rb,0.28168e-03_rb,0.36165e-03_rb,0.43438e-03_rb, &
        & 0.50224e-03_rb,0.56492e-03_rb,0.62614e-03_rb,0.37496e-04_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.84865e-04_rb,0.19761e-03_rb,0.28526e-03_rb,0.36657e-03_rb,0.44042e-03_rb, &
        & 0.50863e-03_rb,0.57263e-03_rb,0.63413e-03_rb,0.15044e-04_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.84443e-04_rb,0.19914e-03_rb,0.28886e-03_rb,0.37069e-03_rb,0.44492e-03_rb, &
        & 0.51377e-03_rb,0.57803e-03_rb,0.63993e-03_rb,0.48683e-04_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.72237e-04_rb,0.17794e-03_rb,0.25491e-03_rb,0.32737e-03_rb,0.39376e-03_rb, &
        & 0.45690e-03_rb,0.51684e-03_rb,0.58057e-03_rb,0.10800e-04_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.71846e-04_rb,0.17916e-03_rb,0.25896e-03_rb,0.33355e-03_rb,0.40234e-03_rb, &
        & 0.46708e-03_rb,0.52856e-03_rb,0.59174e-03_rb,0.37304e-04_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.71641e-04_rb,0.18095e-03_rb,0.26291e-03_rb,0.34013e-03_rb,0.41022e-03_rb, &
        & 0.47645e-03_rb,0.53911e-03_rb,0.60240e-03_rb,0.20066e-04_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.71337e-04_rb,0.18255e-03_rb,0.26698e-03_rb,0.34557e-03_rb,0.41722e-03_rb, &
        & 0.48390e-03_rb,0.54753e-03_rb,0.61134e-03_rb,0.20267e-04_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.71187e-04_rb,0.18456e-03_rb,0.27032e-03_rb,0.35012e-03_rb,0.42265e-03_rb, &
        & 0.49079e-03_rb,0.55462e-03_rb,0.61760e-03_rb,0.29175e-04_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.60308e-04_rb,0.16344e-03_rb,0.23674e-03_rb,0.30451e-03_rb,0.36859e-03_rb, &
        & 0.42973e-03_rb,0.48896e-03_rb,0.55265e-03_rb,0.35870e-04_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.60075e-04_rb,0.16518e-03_rb,0.24182e-03_rb,0.31202e-03_rb,0.37817e-03_rb, &
        & 0.44097e-03_rb,0.50163e-03_rb,0.56563e-03_rb,0.24717e-04_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.59963e-04_rb,0.16736e-03_rb,0.24636e-03_rb,0.31956e-03_rb,0.38717e-03_rb, &
        & 0.45122e-03_rb,0.51282e-03_rb,0.57786e-03_rb,0.24967e-04_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.59808e-04_rb,0.16950e-03_rb,0.25081e-03_rb,0.32559e-03_rb,0.39488e-03_rb, &
        & 0.45992e-03_rb,0.52243e-03_rb,0.58716e-03_rb,0.25073e-04_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.59738e-04_rb,0.17182e-03_rb,0.25430e-03_rb,0.33121e-03_rb,0.40220e-03_rb, &
        & 0.46925e-03_rb,0.53224e-03_rb,0.59573e-03_rb,0.14031e-04_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.79109e-03_rb,0.77354e-03_rb,0.84795e-03_rb,0.91837e-03_rb,0.98287e-03_rb, &
        & 0.10467e-02_rb,0.10876e-02_rb,0.10899e-02_rb,0.10918e-02_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.79035e-03_rb,0.76999e-03_rb,0.84502e-03_rb,0.91315e-03_rb,0.97569e-03_rb, &
        & 0.10420e-02_rb,0.10843e-02_rb,0.10840e-02_rb,0.10904e-02_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.79094e-03_rb,0.76969e-03_rb,0.84501e-03_rb,0.91212e-03_rb,0.97616e-03_rb, &
        & 0.10502e-02_rb,0.10978e-02_rb,0.10968e-02_rb,0.11098e-02_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.79012e-03_rb,0.76698e-03_rb,0.84052e-03_rb,0.90224e-03_rb,0.96448e-03_rb, &
        & 0.10367e-02_rb,0.10810e-02_rb,0.10798e-02_rb,0.10838e-02_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.78840e-03_rb,0.76416e-03_rb,0.83816e-03_rb,0.89971e-03_rb,0.96199e-03_rb, &
        & 0.10344e-02_rb,0.10796e-02_rb,0.10773e-02_rb,0.10800e-02_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.69789e-03_rb,0.72093e-03_rb,0.81475e-03_rb,0.89922e-03_rb,0.96655e-03_rb, &
        & 0.10286e-02_rb,0.10852e-02_rb,0.10968e-02_rb,0.10998e-02_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.69799e-03_rb,0.71831e-03_rb,0.81026e-03_rb,0.89390e-03_rb,0.96162e-03_rb, &
        & 0.10278e-02_rb,0.10853e-02_rb,0.10953e-02_rb,0.10980e-02_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.69849e-03_rb,0.71872e-03_rb,0.81039e-03_rb,0.89368e-03_rb,0.96713e-03_rb, &
        & 0.10387e-02_rb,0.11003e-02_rb,0.11139e-02_rb,0.11199e-02_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.69786e-03_rb,0.71702e-03_rb,0.80831e-03_rb,0.88926e-03_rb,0.96426e-03_rb, &
        & 0.10353e-02_rb,0.10976e-02_rb,0.11107e-02_rb,0.11152e-02_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.69639e-03_rb,0.71420e-03_rb,0.80246e-03_rb,0.88289e-03_rb,0.95515e-03_rb, &
        & 0.10226e-02_rb,0.10812e-02_rb,0.10915e-02_rb,0.10904e-02_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.60717e-03_rb,0.66595e-03_rb,0.77916e-03_rb,0.87509e-03_rb,0.95372e-03_rb, &
        & 0.10167e-02_rb,0.10820e-02_rb,0.11070e-02_rb,0.11005e-02_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.60740e-03_rb,0.66447e-03_rb,0.77425e-03_rb,0.87078e-03_rb,0.95087e-03_rb, &
        & 0.10164e-02_rb,0.10820e-02_rb,0.11070e-02_rb,0.10996e-02_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.60743e-03_rb,0.66421e-03_rb,0.77076e-03_rb,0.86668e-03_rb,0.94936e-03_rb, &
        & 0.10146e-02_rb,0.10809e-02_rb,0.11066e-02_rb,0.10979e-02_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.60721e-03_rb,0.66487e-03_rb,0.77207e-03_rb,0.87093e-03_rb,0.95679e-03_rb, &
        & 0.10247e-02_rb,0.10950e-02_rb,0.11248e-02_rb,0.11171e-02_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.60632e-03_rb,0.66259e-03_rb,0.76645e-03_rb,0.86469e-03_rb,0.94611e-03_rb, &
        & 0.10116e-02_rb,0.10778e-02_rb,0.11040e-02_rb,0.10953e-02_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.52196e-03_rb,0.61309e-03_rb,0.74127e-03_rb,0.84825e-03_rb,0.93805e-03_rb, &
        & 0.10078e-02_rb,0.10732e-02_rb,0.11156e-02_rb,0.10964e-02_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.52205e-03_rb,0.61209e-03_rb,0.73709e-03_rb,0.84586e-03_rb,0.93585e-03_rb, &
        & 0.10074e-02_rb,0.10727e-02_rb,0.11168e-02_rb,0.10976e-02_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.52218e-03_rb,0.61175e-03_rb,0.73426e-03_rb,0.84316e-03_rb,0.93485e-03_rb, &
        & 0.10055e-02_rb,0.10718e-02_rb,0.11162e-02_rb,0.10985e-02_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.52176e-03_rb,0.61131e-03_rb,0.73315e-03_rb,0.84162e-03_rb,0.93319e-03_rb, &
        & 0.10035e-02_rb,0.10709e-02_rb,0.11149e-02_rb,0.11000e-02_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.52100e-03_rb,0.61215e-03_rb,0.73639e-03_rb,0.84868e-03_rb,0.94102e-03_rb, &
        & 0.10132e-02_rb,0.10822e-02_rb,0.11303e-02_rb,0.11192e-02_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.44537e-03_rb,0.56415e-03_rb,0.70334e-03_rb,0.81986e-03_rb,0.91737e-03_rb, &
        & 0.99505e-03_rb,0.10590e-02_rb,0.11155e-02_rb,0.10855e-02_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.44499e-03_rb,0.56304e-03_rb,0.70021e-03_rb,0.81824e-03_rb,0.91496e-03_rb, &
        & 0.99465e-03_rb,0.10577e-02_rb,0.11163e-02_rb,0.10886e-02_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.44481e-03_rb,0.56231e-03_rb,0.69884e-03_rb,0.81557e-03_rb,0.91387e-03_rb, &
        & 0.99269e-03_rb,0.10566e-02_rb,0.11154e-02_rb,0.10925e-02_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.44459e-03_rb,0.56155e-03_rb,0.69827e-03_rb,0.81462e-03_rb,0.91145e-03_rb, &
        & 0.99105e-03_rb,0.10558e-02_rb,0.11141e-02_rb,0.10950e-02_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.44401e-03_rb,0.56266e-03_rb,0.70342e-03_rb,0.82262e-03_rb,0.92017e-03_rb, &
        & 0.10017e-02_rb,0.10686e-02_rb,0.11300e-02_rb,0.11166e-02_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.37714e-03_rb,0.51787e-03_rb,0.66544e-03_rb,0.78793e-03_rb,0.89178e-03_rb, &
        & 0.97506e-03_rb,0.10421e-02_rb,0.11072e-02_rb,0.10679e-02_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.37645e-03_rb,0.51719e-03_rb,0.66329e-03_rb,0.78619e-03_rb,0.88946e-03_rb, &
        & 0.97613e-03_rb,0.10421e-02_rb,0.11076e-02_rb,0.10727e-02_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.37621e-03_rb,0.51663e-03_rb,0.66248e-03_rb,0.78421e-03_rb,0.88840e-03_rb, &
        & 0.97530e-03_rb,0.10423e-02_rb,0.11074e-02_rb,0.10773e-02_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.37625e-03_rb,0.51591e-03_rb,0.66198e-03_rb,0.78342e-03_rb,0.88665e-03_rb, &
        & 0.97510e-03_rb,0.10427e-02_rb,0.11065e-02_rb,0.10807e-02_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.37578e-03_rb,0.51583e-03_rb,0.66231e-03_rb,0.78322e-03_rb,0.88597e-03_rb, &
        & 0.97458e-03_rb,0.10416e-02_rb,0.11045e-02_rb,0.10828e-02_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.31849e-03_rb,0.47598e-03_rb,0.62784e-03_rb,0.75245e-03_rb,0.86097e-03_rb, &
        & 0.95037e-03_rb,0.10228e-02_rb,0.10907e-02_rb,0.10381e-02_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.31748e-03_rb,0.47565e-03_rb,0.62625e-03_rb,0.75127e-03_rb,0.86016e-03_rb, &
        & 0.95292e-03_rb,0.10249e-02_rb,0.10924e-02_rb,0.10446e-02_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.31709e-03_rb,0.47527e-03_rb,0.62527e-03_rb,0.75044e-03_rb,0.86086e-03_rb, &
        & 0.95398e-03_rb,0.10272e-02_rb,0.10943e-02_rb,0.10513e-02_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.31690e-03_rb,0.47506e-03_rb,0.62534e-03_rb,0.75023e-03_rb,0.86098e-03_rb, &
        & 0.95576e-03_rb,0.10298e-02_rb,0.10956e-02_rb,0.10570e-02_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.31661e-03_rb,0.47556e-03_rb,0.62564e-03_rb,0.75129e-03_rb,0.86194e-03_rb, &
        & 0.95752e-03_rb,0.10312e-02_rb,0.10967e-02_rb,0.10605e-02_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.26920e-03_rb,0.43740e-03_rb,0.58907e-03_rb,0.71604e-03_rb,0.82585e-03_rb, &
        & 0.92017e-03_rb,0.99828e-03_rb,0.10677e-02_rb,0.99191e-03_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.26802e-03_rb,0.43738e-03_rb,0.58846e-03_rb,0.71590e-03_rb,0.82753e-03_rb, &
        & 0.92484e-03_rb,0.10036e-02_rb,0.10726e-02_rb,0.10010e-02_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.26738e-03_rb,0.43742e-03_rb,0.58827e-03_rb,0.71679e-03_rb,0.83051e-03_rb, &
        & 0.92821e-03_rb,0.10079e-02_rb,0.10772e-02_rb,0.10102e-02_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.26701e-03_rb,0.43751e-03_rb,0.58933e-03_rb,0.71793e-03_rb,0.83304e-03_rb, &
        & 0.93190e-03_rb,0.10124e-02_rb,0.10810e-02_rb,0.10188e-02_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.26686e-03_rb,0.43824e-03_rb,0.58999e-03_rb,0.72050e-03_rb,0.83607e-03_rb, &
        & 0.93581e-03_rb,0.10160e-02_rb,0.10848e-02_rb,0.10263e-02_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.22904e-03_rb,0.40123e-03_rb,0.55115e-03_rb,0.67712e-03_rb,0.78732e-03_rb, &
        & 0.88313e-03_rb,0.96568e-03_rb,0.10387e-02_rb,0.14403e-04_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.22775e-03_rb,0.40129e-03_rb,0.55155e-03_rb,0.67918e-03_rb,0.79168e-03_rb, &
        & 0.89045e-03_rb,0.97356e-03_rb,0.10468e-02_rb,0.17396e-04_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.22691e-03_rb,0.40187e-03_rb,0.55239e-03_rb,0.68238e-03_rb,0.79664e-03_rb, &
        & 0.89641e-03_rb,0.98013e-03_rb,0.10538e-02_rb,0.17983e-04_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.22637e-03_rb,0.40220e-03_rb,0.55452e-03_rb,0.68525e-03_rb,0.80145e-03_rb, &
        & 0.90223e-03_rb,0.98673e-03_rb,0.10599e-02_rb,0.14884e-04_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.22595e-03_rb,0.40316e-03_rb,0.55613e-03_rb,0.68950e-03_rb,0.80623e-03_rb, &
        & 0.90849e-03_rb,0.99244e-03_rb,0.10655e-02_rb,0.19485e-04_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.19610e-03_rb,0.36707e-03_rb,0.51305e-03_rb,0.63612e-03_rb,0.74337e-03_rb, &
        & 0.83875e-03_rb,0.92458e-03_rb,0.10007e-02_rb,0.30583e-04_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.19499e-03_rb,0.36759e-03_rb,0.51501e-03_rb,0.64066e-03_rb,0.75082e-03_rb, &
        & 0.84853e-03_rb,0.93443e-03_rb,0.10117e-02_rb,0.20484e-04_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.19427e-03_rb,0.36886e-03_rb,0.51745e-03_rb,0.64625e-03_rb,0.75815e-03_rb, &
        & 0.85733e-03_rb,0.94303e-03_rb,0.10217e-02_rb,0.20784e-04_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.19367e-03_rb,0.36958e-03_rb,0.52106e-03_rb,0.65093e-03_rb,0.76545e-03_rb, &
        & 0.86571e-03_rb,0.95244e-03_rb,0.10301e-02_rb,0.30511e-04_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.19319e-03_rb,0.37098e-03_rb,0.52409e-03_rb,0.65675e-03_rb,0.77258e-03_rb, &
        & 0.87491e-03_rb,0.96110e-03_rb,0.10378e-02_rb,0.19883e-04_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.16767e-03_rb,0.33624e-03_rb,0.47635e-03_rb,0.59543e-03_rb,0.70301e-03_rb, &
        & 0.79872e-03_rb,0.88652e-03_rb,0.96550e-03_rb,0.27483e-04_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.16687e-03_rb,0.33764e-03_rb,0.48055e-03_rb,0.60283e-03_rb,0.71186e-03_rb, &
        & 0.80978e-03_rb,0.89673e-03_rb,0.97758e-03_rb,0.20639e-04_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.16664e-03_rb,0.33942e-03_rb,0.48524e-03_rb,0.60962e-03_rb,0.72075e-03_rb, &
        & 0.81953e-03_rb,0.90753e-03_rb,0.98729e-03_rb,0.27689e-04_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.16611e-03_rb,0.34104e-03_rb,0.48994e-03_rb,0.61708e-03_rb,0.72992e-03_rb, &
        & 0.83047e-03_rb,0.91898e-03_rb,0.99769e-03_rb,0.13719e-04_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.16596e-03_rb,0.34313e-03_rb,0.49467e-03_rb,0.62538e-03_rb,0.73960e-03_rb, &
        & 0.84171e-03_rb,0.92999e-03_rb,0.10075e-02_rb,0.27137e-04_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.14282e-03_rb,0.30890e-03_rb,0.44183e-03_rb,0.55831e-03_rb,0.66503e-03_rb, &
        & 0.76077e-03_rb,0.84890e-03_rb,0.93128e-03_rb,0.21228e-04_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.14255e-03_rb,0.31125e-03_rb,0.44731e-03_rb,0.56685e-03_rb,0.67491e-03_rb, &
        & 0.77277e-03_rb,0.86092e-03_rb,0.94274e-03_rb,0.32331e-04_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.14244e-03_rb,0.31334e-03_rb,0.45354e-03_rb,0.57479e-03_rb,0.68552e-03_rb, &
        & 0.78420e-03_rb,0.87396e-03_rb,0.95434e-03_rb,0.19978e-06_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.14239e-03_rb,0.31614e-03_rb,0.45984e-03_rb,0.58419e-03_rb,0.69607e-03_rb, &
        & 0.79750e-03_rb,0.88765e-03_rb,0.96637e-03_rb,0.21475e-04_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.14252e-03_rb,0.31876e-03_rb,0.46658e-03_rb,0.59385e-03_rb,0.70775e-03_rb, &
        & 0.80986e-03_rb,0.90037e-03_rb,0.97814e-03_rb,0.21476e-04_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.12125e-03_rb,0.28444e-03_rb,0.41080e-03_rb,0.52444e-03_rb,0.62932e-03_rb, &
        & 0.72481e-03_rb,0.81245e-03_rb,0.89716e-03_rb,0.27405e-04_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.12138e-03_rb,0.28788e-03_rb,0.41712e-03_rb,0.53396e-03_rb,0.64078e-03_rb, &
        & 0.73819e-03_rb,0.82737e-03_rb,0.91020e-03_rb,0.13699e-04_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.12142e-03_rb,0.29086e-03_rb,0.42441e-03_rb,0.54338e-03_rb,0.65260e-03_rb, &
        & 0.75224e-03_rb,0.84352e-03_rb,0.92398e-03_rb,0.13820e-04_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.12174e-03_rb,0.29430e-03_rb,0.43176e-03_rb,0.55440e-03_rb,0.66546e-03_rb, &
        & 0.76738e-03_rb,0.85914e-03_rb,0.93821e-03_rb,0.26505e-06_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.12197e-03_rb,0.29750e-03_rb,0.43989e-03_rb,0.56500e-03_rb,0.67844e-03_rb, &
        & 0.78071e-03_rb,0.87280e-03_rb,0.95108e-03_rb,0.33764e-06_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.15537e-02_rb,0.13818e-02_rb,0.14273e-02_rb,0.15055e-02_rb,0.15786e-02_rb, &
        & 0.16339e-02_rb,0.16467e-02_rb,0.15800e-02_rb,0.16050e-02_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.15485e-02_rb,0.13765e-02_rb,0.14181e-02_rb,0.14978e-02_rb,0.15727e-02_rb, &
        & 0.16256e-02_rb,0.16371e-02_rb,0.15728e-02_rb,0.16005e-02_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.15437e-02_rb,0.13722e-02_rb,0.14122e-02_rb,0.14971e-02_rb,0.15752e-02_rb, &
        & 0.16261e-02_rb,0.16396e-02_rb,0.15797e-02_rb,0.16209e-02_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.15424e-02_rb,0.13691e-02_rb,0.14015e-02_rb,0.14839e-02_rb,0.15593e-02_rb, &
        & 0.16040e-02_rb,0.16150e-02_rb,0.15487e-02_rb,0.15957e-02_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.15425e-02_rb,0.13681e-02_rb,0.13961e-02_rb,0.14766e-02_rb,0.15502e-02_rb, &
        & 0.15943e-02_rb,0.16046e-02_rb,0.15376e-02_rb,0.15923e-02_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.14274e-02_rb,0.13101e-02_rb,0.13901e-02_rb,0.14665e-02_rb,0.15597e-02_rb, &
        & 0.16365e-02_rb,0.16682e-02_rb,0.16286e-02_rb,0.16381e-02_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.14223e-02_rb,0.13039e-02_rb,0.13828e-02_rb,0.14629e-02_rb,0.15522e-02_rb, &
        & 0.16256e-02_rb,0.16567e-02_rb,0.16183e-02_rb,0.16362e-02_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.14195e-02_rb,0.12999e-02_rb,0.13791e-02_rb,0.14641e-02_rb,0.15538e-02_rb, &
        & 0.16259e-02_rb,0.16605e-02_rb,0.16235e-02_rb,0.16585e-02_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.14181e-02_rb,0.12958e-02_rb,0.13726e-02_rb,0.14576e-02_rb,0.15447e-02_rb, &
        & 0.16149e-02_rb,0.16487e-02_rb,0.16101e-02_rb,0.16540e-02_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.14191e-02_rb,0.12933e-02_rb,0.13650e-02_rb,0.14448e-02_rb,0.15270e-02_rb, &
        & 0.15959e-02_rb,0.16273e-02_rb,0.15850e-02_rb,0.16279e-02_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.12975e-02_rb,0.12495e-02_rb,0.13516e-02_rb,0.14440e-02_rb,0.15390e-02_rb, &
        & 0.16298e-02_rb,0.16807e-02_rb,0.16634e-02_rb,0.16697e-02_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.12938e-02_rb,0.12434e-02_rb,0.13457e-02_rb,0.14406e-02_rb,0.15301e-02_rb, &
        & 0.16196e-02_rb,0.16709e-02_rb,0.16534e-02_rb,0.16671e-02_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.12920e-02_rb,0.12378e-02_rb,0.13412e-02_rb,0.14340e-02_rb,0.15221e-02_rb, &
        & 0.16094e-02_rb,0.16610e-02_rb,0.16426e-02_rb,0.16655e-02_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.12905e-02_rb,0.12343e-02_rb,0.13399e-02_rb,0.14335e-02_rb,0.15224e-02_rb, &
        & 0.16137e-02_rb,0.16692e-02_rb,0.16519e-02_rb,0.16891e-02_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.12916e-02_rb,0.12305e-02_rb,0.13333e-02_rb,0.14197e-02_rb,0.15049e-02_rb, &
        & 0.15945e-02_rb,0.16481e-02_rb,0.16282e-02_rb,0.16593e-02_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.11586e-02_rb,0.11791e-02_rb,0.13059e-02_rb,0.14246e-02_rb,0.15191e-02_rb, &
        & 0.16155e-02_rb,0.16888e-02_rb,0.16884e-02_rb,0.16900e-02_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.11563e-02_rb,0.11735e-02_rb,0.13020e-02_rb,0.14191e-02_rb,0.15117e-02_rb, &
        & 0.16067e-02_rb,0.16804e-02_rb,0.16804e-02_rb,0.16885e-02_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.11547e-02_rb,0.11684e-02_rb,0.12989e-02_rb,0.14121e-02_rb,0.15049e-02_rb, &
        & 0.15986e-02_rb,0.16736e-02_rb,0.16742e-02_rb,0.16864e-02_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.11549e-02_rb,0.11640e-02_rb,0.12947e-02_rb,0.14053e-02_rb,0.14971e-02_rb, &
        & 0.15940e-02_rb,0.16690e-02_rb,0.16701e-02_rb,0.16826e-02_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.11568e-02_rb,0.11629e-02_rb,0.12947e-02_rb,0.14048e-02_rb,0.15017e-02_rb, &
        & 0.16054e-02_rb,0.16836e-02_rb,0.16886e-02_rb,0.17044e-02_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.10197e-02_rb,0.11002e-02_rb,0.12563e-02_rb,0.13948e-02_rb,0.15048e-02_rb, &
        & 0.16026e-02_rb,0.16898e-02_rb,0.17082e-02_rb,0.17013e-02_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.10187e-02_rb,0.10963e-02_rb,0.12532e-02_rb,0.13909e-02_rb,0.14993e-02_rb, &
        & 0.15953e-02_rb,0.16843e-02_rb,0.17040e-02_rb,0.16991e-02_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.10179e-02_rb,0.10925e-02_rb,0.12502e-02_rb,0.13858e-02_rb,0.14932e-02_rb, &
        & 0.15900e-02_rb,0.16803e-02_rb,0.17019e-02_rb,0.16952e-02_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.10188e-02_rb,0.10896e-02_rb,0.12461e-02_rb,0.13806e-02_rb,0.14878e-02_rb, &
        & 0.15873e-02_rb,0.16767e-02_rb,0.17001e-02_rb,0.16925e-02_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.10211e-02_rb,0.10901e-02_rb,0.12464e-02_rb,0.13834e-02_rb,0.14970e-02_rb, &
        & 0.16026e-02_rb,0.16952e-02_rb,0.17210e-02_rb,0.17180e-02_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.88455e-03_rb,0.10185e-02_rb,0.12021e-02_rb,0.13605e-02_rb,0.14894e-02_rb, &
        & 0.15922e-02_rb,0.16870e-02_rb,0.17277e-02_rb,0.17006e-02_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.88487e-03_rb,0.10158e-02_rb,0.12001e-02_rb,0.13587e-02_rb,0.14854e-02_rb, &
        & 0.15856e-02_rb,0.16819e-02_rb,0.17253e-02_rb,0.17001e-02_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.88512e-03_rb,0.10136e-02_rb,0.11981e-02_rb,0.13544e-02_rb,0.14820e-02_rb, &
        & 0.15811e-02_rb,0.16785e-02_rb,0.17233e-02_rb,0.16989e-02_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.88698e-03_rb,0.10127e-02_rb,0.11948e-02_rb,0.13505e-02_rb,0.14784e-02_rb, &
        & 0.15789e-02_rb,0.16749e-02_rb,0.17211e-02_rb,0.16988e-02_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.88923e-03_rb,0.10128e-02_rb,0.11919e-02_rb,0.13468e-02_rb,0.14756e-02_rb, &
        & 0.15764e-02_rb,0.16734e-02_rb,0.17190e-02_rb,0.16990e-02_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.75937e-03_rb,0.93876e-03_rb,0.11443e-02_rb,0.13228e-02_rb,0.14681e-02_rb, &
        & 0.15807e-02_rb,0.16771e-02_rb,0.17403e-02_rb,0.16920e-02_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.75998e-03_rb,0.93770e-03_rb,0.11447e-02_rb,0.13222e-02_rb,0.14656e-02_rb, &
        & 0.15751e-02_rb,0.16722e-02_rb,0.17374e-02_rb,0.16943e-02_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.76057e-03_rb,0.93698e-03_rb,0.11442e-02_rb,0.13192e-02_rb,0.14630e-02_rb, &
        & 0.15711e-02_rb,0.16684e-02_rb,0.17351e-02_rb,0.16966e-02_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.76306e-03_rb,0.93700e-03_rb,0.11417e-02_rb,0.13175e-02_rb,0.14605e-02_rb, &
        & 0.15679e-02_rb,0.16649e-02_rb,0.17327e-02_rb,0.16998e-02_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.76563e-03_rb,0.93740e-03_rb,0.11401e-02_rb,0.13155e-02_rb,0.14583e-02_rb, &
        & 0.15655e-02_rb,0.16637e-02_rb,0.17306e-02_rb,0.17045e-02_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.64586e-03_rb,0.86421e-03_rb,0.10884e-02_rb,0.12806e-02_rb,0.14383e-02_rb, &
        & 0.15636e-02_rb,0.16605e-02_rb,0.17434e-02_rb,0.16711e-02_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.64665e-03_rb,0.86473e-03_rb,0.10896e-02_rb,0.12820e-02_rb,0.14363e-02_rb, &
        & 0.15592e-02_rb,0.16561e-02_rb,0.17403e-02_rb,0.16771e-02_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.64816e-03_rb,0.86506e-03_rb,0.10903e-02_rb,0.12802e-02_rb,0.14357e-02_rb, &
        & 0.15554e-02_rb,0.16529e-02_rb,0.17367e-02_rb,0.16846e-02_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.65115e-03_rb,0.86561e-03_rb,0.10893e-02_rb,0.12797e-02_rb,0.14336e-02_rb, &
        & 0.15533e-02_rb,0.16505e-02_rb,0.17349e-02_rb,0.16920e-02_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.65335e-03_rb,0.86692e-03_rb,0.10897e-02_rb,0.12788e-02_rb,0.14323e-02_rb, &
        & 0.15519e-02_rb,0.16494e-02_rb,0.17328e-02_rb,0.17005e-02_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.54495e-03_rb,0.79374e-03_rb,0.10331e-02_rb,0.12325e-02_rb,0.13999e-02_rb, &
        & 0.15343e-02_rb,0.16359e-02_rb,0.17304e-02_rb,0.88289e-03_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.54631e-03_rb,0.79621e-03_rb,0.10351e-02_rb,0.12356e-02_rb,0.13999e-02_rb, &
        & 0.15332e-02_rb,0.16343e-02_rb,0.17287e-02_rb,0.90009e-03_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.54838e-03_rb,0.79738e-03_rb,0.10366e-02_rb,0.12358e-02_rb,0.14008e-02_rb, &
        & 0.15312e-02_rb,0.16325e-02_rb,0.17267e-02_rb,0.91715e-03_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.55103e-03_rb,0.79925e-03_rb,0.10373e-02_rb,0.12370e-02_rb,0.14003e-02_rb, &
        & 0.15313e-02_rb,0.16317e-02_rb,0.17265e-02_rb,0.93948e-03_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.55358e-03_rb,0.80158e-03_rb,0.10391e-02_rb,0.12370e-02_rb,0.14008e-02_rb, &
        & 0.15320e-02_rb,0.16328e-02_rb,0.17271e-02_rb,0.97099e-03_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.45828e-03_rb,0.73128e-03_rb,0.97725e-03_rb,0.11801e-02_rb,0.13541e-02_rb, &
        & 0.14966e-02_rb,0.16059e-02_rb,0.17081e-02_rb,0.25261e-04_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.45994e-03_rb,0.73433e-03_rb,0.98047e-03_rb,0.11852e-02_rb,0.13571e-02_rb, &
        & 0.14993e-02_rb,0.16071e-02_rb,0.17093e-02_rb,0.32778e-04_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.46172e-03_rb,0.73698e-03_rb,0.98332e-03_rb,0.11877e-02_rb,0.13607e-02_rb, &
        & 0.15001e-02_rb,0.16089e-02_rb,0.17098e-02_rb,0.30108e-04_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.46419e-03_rb,0.73967e-03_rb,0.98568e-03_rb,0.11903e-02_rb,0.13620e-02_rb, &
        & 0.15042e-02_rb,0.16117e-02_rb,0.17131e-02_rb,0.14814e-04_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.46710e-03_rb,0.74287e-03_rb,0.98852e-03_rb,0.11927e-02_rb,0.13653e-02_rb, &
        & 0.15080e-02_rb,0.16153e-02_rb,0.17174e-02_rb,0.18206e-04_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.38711e-03_rb,0.67660e-03_rb,0.92352e-03_rb,0.11285e-02_rb,0.13042e-02_rb, &
        & 0.14533e-02_rb,0.15680e-02_rb,0.16768e-02_rb,0.35375e-04_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.38827e-03_rb,0.68076e-03_rb,0.92811e-03_rb,0.11355e-02_rb,0.13110e-02_rb, &
        & 0.14578e-02_rb,0.15732e-02_rb,0.16802e-02_rb,0.17785e-04_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.39017e-03_rb,0.68339e-03_rb,0.93243e-03_rb,0.11404e-02_rb,0.13179e-02_rb, &
        & 0.14648e-02_rb,0.15809e-02_rb,0.16870e-02_rb,0.17310e-04_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.39259e-03_rb,0.68706e-03_rb,0.93609e-03_rb,0.11451e-02_rb,0.13243e-02_rb, &
        & 0.14746e-02_rb,0.15892e-02_rb,0.16958e-02_rb,0.17795e-04_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.39544e-03_rb,0.69011e-03_rb,0.94016e-03_rb,0.11496e-02_rb,0.13318e-02_rb, &
        & 0.14825e-02_rb,0.15980e-02_rb,0.17052e-02_rb,0.87581e-05_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.32882e-03_rb,0.62658e-03_rb,0.87124e-03_rb,0.10770e-02_rb,0.12526e-02_rb, &
        & 0.14042e-02_rb,0.15246e-02_rb,0.16367e-02_rb,0.13823e-04_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.33008e-03_rb,0.63068e-03_rb,0.87790e-03_rb,0.10857e-02_rb,0.12638e-02_rb, &
        & 0.14155e-02_rb,0.15376e-02_rb,0.16473e-02_rb,0.27356e-04_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.33191e-03_rb,0.63479e-03_rb,0.88401e-03_rb,0.10939e-02_rb,0.12749e-02_rb, &
        & 0.14298e-02_rb,0.15521e-02_rb,0.16615e-02_rb,0.26924e-04_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.33384e-03_rb,0.63914e-03_rb,0.88945e-03_rb,0.11023e-02_rb,0.12871e-02_rb, &
        & 0.14439e-02_rb,0.15651e-02_rb,0.16755e-02_rb,0.13486e-04_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.33630e-03_rb,0.64288e-03_rb,0.89554e-03_rb,0.11106e-02_rb,0.12985e-02_rb, &
        & 0.14567e-02_rb,0.15792e-02_rb,0.16910e-02_rb,0.54470e-04_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.28179e-03_rb,0.58039e-03_rb,0.82131e-03_rb,0.10258e-02_rb,0.12001e-02_rb, &
        & 0.13522e-02_rb,0.14762e-02_rb,0.15892e-02_rb,0.17365e-04_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.28322e-03_rb,0.58517e-03_rb,0.83048e-03_rb,0.10376e-02_rb,0.12180e-02_rb, &
        & 0.13721e-02_rb,0.14980e-02_rb,0.16092e-02_rb,0.19167e-10_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.28449e-03_rb,0.59004e-03_rb,0.83867e-03_rb,0.10495e-02_rb,0.12349e-02_rb, &
        & 0.13937e-02_rb,0.15189e-02_rb,0.16302e-02_rb,0.17686e-04_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.28623e-03_rb,0.59506e-03_rb,0.84661e-03_rb,0.10616e-02_rb,0.12511e-02_rb, &
        & 0.14124e-02_rb,0.15380e-02_rb,0.16505e-02_rb,0.27174e-10_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.28834e-03_rb,0.60000e-03_rb,0.85510e-03_rb,0.10735e-02_rb,0.12664e-02_rb, &
        & 0.14298e-02_rb,0.15583e-02_rb,0.16724e-02_rb,0.30930e-10_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.39229e-02_rb,0.34326e-02_rb,0.30162e-02_rb,0.28699e-02_rb,0.28381e-02_rb, &
        & 0.27734e-02_rb,0.26260e-02_rb,0.23062e-02_rb,0.25522e-02_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.39213e-02_rb,0.34310e-02_rb,0.30108e-02_rb,0.28565e-02_rb,0.28216e-02_rb, &
        & 0.27552e-02_rb,0.26075e-02_rb,0.22956e-02_rb,0.25296e-02_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.39180e-02_rb,0.34283e-02_rb,0.30095e-02_rb,0.28490e-02_rb,0.28140e-02_rb, &
        & 0.27516e-02_rb,0.26057e-02_rb,0.23031e-02_rb,0.25320e-02_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.39161e-02_rb,0.34265e-02_rb,0.30039e-02_rb,0.28320e-02_rb,0.27864e-02_rb, &
        & 0.27263e-02_rb,0.25779e-02_rb,0.22767e-02_rb,0.24834e-02_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.39129e-02_rb,0.34239e-02_rb,0.29985e-02_rb,0.28202e-02_rb,0.27725e-02_rb, &
        & 0.27107e-02_rb,0.25647e-02_rb,0.22654e-02_rb,0.24632e-02_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.35717e-02_rb,0.31250e-02_rb,0.28583e-02_rb,0.28716e-02_rb,0.28885e-02_rb, &
        & 0.28569e-02_rb,0.27419e-02_rb,0.24528e-02_rb,0.26589e-02_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.35704e-02_rb,0.31241e-02_rb,0.28509e-02_rb,0.28521e-02_rb,0.28717e-02_rb, &
        & 0.28411e-02_rb,0.27277e-02_rb,0.24440e-02_rb,0.26352e-02_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.35692e-02_rb,0.31228e-02_rb,0.28479e-02_rb,0.28438e-02_rb,0.28636e-02_rb, &
        & 0.28419e-02_rb,0.27327e-02_rb,0.24542e-02_rb,0.26382e-02_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.35651e-02_rb,0.31194e-02_rb,0.28382e-02_rb,0.28288e-02_rb,0.28472e-02_rb, &
        & 0.28288e-02_rb,0.27211e-02_rb,0.24428e-02_rb,0.26140e-02_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.35614e-02_rb,0.31162e-02_rb,0.28280e-02_rb,0.28080e-02_rb,0.28270e-02_rb, &
        & 0.28018e-02_rb,0.26919e-02_rb,0.24122e-02_rb,0.25721e-02_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.32370e-02_rb,0.28408e-02_rb,0.27429e-02_rb,0.28459e-02_rb,0.29107e-02_rb, &
        & 0.29163e-02_rb,0.28452e-02_rb,0.26013e-02_rb,0.27589e-02_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.32378e-02_rb,0.28409e-02_rb,0.27346e-02_rb,0.28282e-02_rb,0.28967e-02_rb, &
        & 0.29036e-02_rb,0.28357e-02_rb,0.25912e-02_rb,0.27381e-02_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.32344e-02_rb,0.28366e-02_rb,0.27224e-02_rb,0.28159e-02_rb,0.28824e-02_rb, &
        & 0.28940e-02_rb,0.28268e-02_rb,0.25801e-02_rb,0.27164e-02_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.32328e-02_rb,0.28349e-02_rb,0.27139e-02_rb,0.28071e-02_rb,0.28820e-02_rb, &
        & 0.28985e-02_rb,0.28328e-02_rb,0.25870e-02_rb,0.27280e-02_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.32337e-02_rb,0.28348e-02_rb,0.27027e-02_rb,0.27877e-02_rb,0.28613e-02_rb, &
        & 0.28723e-02_rb,0.28004e-02_rb,0.25498e-02_rb,0.26848e-02_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.29442e-02_rb,0.26158e-02_rb,0.26686e-02_rb,0.28002e-02_rb,0.29074e-02_rb, &
        & 0.29569e-02_rb,0.29354e-02_rb,0.27381e-02_rb,0.28491e-02_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.29416e-02_rb,0.26110e-02_rb,0.26570e-02_rb,0.27866e-02_rb,0.28959e-02_rb, &
        & 0.29477e-02_rb,0.29295e-02_rb,0.27262e-02_rb,0.28287e-02_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.29392e-02_rb,0.26064e-02_rb,0.26421e-02_rb,0.27765e-02_rb,0.28845e-02_rb, &
        & 0.29406e-02_rb,0.29202e-02_rb,0.27107e-02_rb,0.28099e-02_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.29406e-02_rb,0.26055e-02_rb,0.26309e-02_rb,0.27645e-02_rb,0.28769e-02_rb, &
        & 0.29312e-02_rb,0.29057e-02_rb,0.26928e-02_rb,0.27972e-02_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.29379e-02_rb,0.26016e-02_rb,0.26230e-02_rb,0.27605e-02_rb,0.28772e-02_rb, &
        & 0.29338e-02_rb,0.29114e-02_rb,0.26956e-02_rb,0.28225e-02_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.26861e-02_rb,0.24510e-02_rb,0.26014e-02_rb,0.27478e-02_rb,0.28811e-02_rb, &
        & 0.29878e-02_rb,0.30091e-02_rb,0.28610e-02_rb,0.29320e-02_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.26822e-02_rb,0.24426e-02_rb,0.25898e-02_rb,0.27369e-02_rb,0.28725e-02_rb, &
        & 0.29815e-02_rb,0.30039e-02_rb,0.28485e-02_rb,0.29157e-02_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.26829e-02_rb,0.24377e-02_rb,0.25766e-02_rb,0.27289e-02_rb,0.28648e-02_rb, &
        & 0.29741e-02_rb,0.29940e-02_rb,0.28292e-02_rb,0.29014e-02_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.26816e-02_rb,0.24329e-02_rb,0.25674e-02_rb,0.27192e-02_rb,0.28602e-02_rb, &
        & 0.29643e-02_rb,0.29808e-02_rb,0.28105e-02_rb,0.28911e-02_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.26776e-02_rb,0.24242e-02_rb,0.25584e-02_rb,0.27174e-02_rb,0.28631e-02_rb, &
        & 0.29690e-02_rb,0.29916e-02_rb,0.28176e-02_rb,0.29253e-02_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.24590e-02_rb,0.23376e-02_rb,0.25355e-02_rb,0.26945e-02_rb,0.28553e-02_rb, &
        & 0.29996e-02_rb,0.30574e-02_rb,0.29544e-02_rb,0.30033e-02_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.24565e-02_rb,0.23281e-02_rb,0.25242e-02_rb,0.26862e-02_rb,0.28495e-02_rb, &
        & 0.29966e-02_rb,0.30559e-02_rb,0.29459e-02_rb,0.29901e-02_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.24594e-02_rb,0.23224e-02_rb,0.25135e-02_rb,0.26796e-02_rb,0.28441e-02_rb, &
        & 0.29914e-02_rb,0.30494e-02_rb,0.29315e-02_rb,0.29796e-02_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.24561e-02_rb,0.23120e-02_rb,0.25070e-02_rb,0.26723e-02_rb,0.28398e-02_rb, &
        & 0.29830e-02_rb,0.30431e-02_rb,0.29165e-02_rb,0.29718e-02_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.24544e-02_rb,0.23009e-02_rb,0.24971e-02_rb,0.26678e-02_rb,0.28335e-02_rb, &
        & 0.29758e-02_rb,0.30342e-02_rb,0.29019e-02_rb,0.29654e-02_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.22277e-02_rb,0.22296e-02_rb,0.24669e-02_rb,0.26497e-02_rb,0.28285e-02_rb, &
        & 0.29885e-02_rb,0.30855e-02_rb,0.30292e-02_rb,0.30616e-02_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.22293e-02_rb,0.22219e-02_rb,0.24569e-02_rb,0.26437e-02_rb,0.28247e-02_rb, &
        & 0.29924e-02_rb,0.30879e-02_rb,0.30266e-02_rb,0.30510e-02_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.22351e-02_rb,0.22176e-02_rb,0.24495e-02_rb,0.26382e-02_rb,0.28222e-02_rb, &
        & 0.29913e-02_rb,0.30877e-02_rb,0.30175e-02_rb,0.30415e-02_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.22344e-02_rb,0.22079e-02_rb,0.24453e-02_rb,0.26324e-02_rb,0.28192e-02_rb, &
        & 0.29887e-02_rb,0.30862e-02_rb,0.30074e-02_rb,0.30342e-02_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.22361e-02_rb,0.21982e-02_rb,0.24378e-02_rb,0.26291e-02_rb,0.28143e-02_rb, &
        & 0.29850e-02_rb,0.30822e-02_rb,0.29969e-02_rb,0.30270e-02_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.19878e-02_rb,0.21080e-02_rb,0.23886e-02_rb,0.26087e-02_rb,0.27952e-02_rb, &
        & 0.29676e-02_rb,0.31003e-02_rb,0.30904e-02_rb,0.31018e-02_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.19936e-02_rb,0.21034e-02_rb,0.23825e-02_rb,0.26049e-02_rb,0.27978e-02_rb, &
        & 0.29755e-02_rb,0.31107e-02_rb,0.30954e-02_rb,0.30940e-02_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.20012e-02_rb,0.21016e-02_rb,0.23778e-02_rb,0.26010e-02_rb,0.27982e-02_rb, &
        & 0.29815e-02_rb,0.31166e-02_rb,0.30929e-02_rb,0.30865e-02_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.20034e-02_rb,0.20960e-02_rb,0.23764e-02_rb,0.25979e-02_rb,0.27992e-02_rb, &
        & 0.29823e-02_rb,0.31192e-02_rb,0.30868e-02_rb,0.30812e-02_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.20095e-02_rb,0.20891e-02_rb,0.23711e-02_rb,0.25960e-02_rb,0.27964e-02_rb, &
        & 0.29828e-02_rb,0.31181e-02_rb,0.30806e-02_rb,0.30733e-02_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.17523e-02_rb,0.19789e-02_rb,0.23024e-02_rb,0.25572e-02_rb,0.27629e-02_rb, &
        & 0.29479e-02_rb,0.31085e-02_rb,0.31444e-02_rb,0.30417e-02_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.17607e-02_rb,0.19777e-02_rb,0.23013e-02_rb,0.25587e-02_rb,0.27691e-02_rb, &
        & 0.29613e-02_rb,0.31227e-02_rb,0.31535e-02_rb,0.30346e-02_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.17693e-02_rb,0.19800e-02_rb,0.23011e-02_rb,0.25605e-02_rb,0.27739e-02_rb, &
        & 0.29714e-02_rb,0.31329e-02_rb,0.31549e-02_rb,0.30299e-02_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.17751e-02_rb,0.19773e-02_rb,0.23025e-02_rb,0.25606e-02_rb,0.27770e-02_rb, &
        & 0.29770e-02_rb,0.31405e-02_rb,0.31532e-02_rb,0.30280e-02_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.17845e-02_rb,0.19738e-02_rb,0.23006e-02_rb,0.25633e-02_rb,0.27766e-02_rb, &
        & 0.29802e-02_rb,0.31452e-02_rb,0.31517e-02_rb,0.30317e-02_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.15244e-02_rb,0.18437e-02_rb,0.22100e-02_rb,0.24972e-02_rb,0.27331e-02_rb, &
        & 0.29304e-02_rb,0.31059e-02_rb,0.31844e-02_rb,0.23817e-04_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.15360e-02_rb,0.18467e-02_rb,0.22145e-02_rb,0.25068e-02_rb,0.27437e-02_rb, &
        & 0.29473e-02_rb,0.31260e-02_rb,0.31986e-02_rb,0.12454e-04_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.15476e-02_rb,0.18512e-02_rb,0.22181e-02_rb,0.25119e-02_rb,0.27538e-02_rb, &
        & 0.29593e-02_rb,0.31389e-02_rb,0.32042e-02_rb,0.30814e-04_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.15571e-02_rb,0.18528e-02_rb,0.22235e-02_rb,0.25174e-02_rb,0.27591e-02_rb, &
        & 0.29666e-02_rb,0.31497e-02_rb,0.32063e-02_rb,0.11698e-04_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.15695e-02_rb,0.18534e-02_rb,0.22262e-02_rb,0.25236e-02_rb,0.27611e-02_rb, &
        & 0.29733e-02_rb,0.31600e-02_rb,0.32093e-02_rb,0.22330e-04_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.13188e-02_rb,0.17123e-02_rb,0.21150e-02_rb,0.24398e-02_rb,0.27060e-02_rb, &
        & 0.29190e-02_rb,0.31061e-02_rb,0.32197e-02_rb,0.11689e-04_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.13341e-02_rb,0.17189e-02_rb,0.21240e-02_rb,0.24532e-02_rb,0.27222e-02_rb, &
        & 0.29371e-02_rb,0.31262e-02_rb,0.32338e-02_rb,0.11845e-04_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.13447e-02_rb,0.17274e-02_rb,0.21339e-02_rb,0.24651e-02_rb,0.27343e-02_rb, &
        & 0.29498e-02_rb,0.31430e-02_rb,0.32440e-02_rb,0.24255e-04_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.13578e-02_rb,0.17324e-02_rb,0.21454e-02_rb,0.24765e-02_rb,0.27400e-02_rb, &
        & 0.29585e-02_rb,0.31579e-02_rb,0.32502e-02_rb,0.35648e-04_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.13715e-02_rb,0.17394e-02_rb,0.21518e-02_rb,0.24842e-02_rb,0.27436e-02_rb, &
        & 0.29689e-02_rb,0.31719e-02_rb,0.32570e-02_rb,0.48216e-04_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.11367e-02_rb,0.15942e-02_rb,0.20255e-02_rb,0.23827e-02_rb,0.26757e-02_rb, &
        & 0.29001e-02_rb,0.30944e-02_rb,0.32386e-02_rb,0.48616e-12_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.11495e-02_rb,0.16057e-02_rb,0.20405e-02_rb,0.24019e-02_rb,0.26965e-02_rb, &
        & 0.29177e-02_rb,0.31160e-02_rb,0.32553e-02_rb,0.63803e-12_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.11626e-02_rb,0.16147e-02_rb,0.20558e-02_rb,0.24193e-02_rb,0.27102e-02_rb, &
        & 0.29330e-02_rb,0.31363e-02_rb,0.32673e-02_rb,0.18451e-04_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.11784e-02_rb,0.16224e-02_rb,0.20681e-02_rb,0.24334e-02_rb,0.27187e-02_rb, &
        & 0.29473e-02_rb,0.31560e-02_rb,0.32789e-02_rb,0.93725e-12_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.11935e-02_rb,0.16327e-02_rb,0.20790e-02_rb,0.24438e-02_rb,0.27275e-02_rb, &
        & 0.29614e-02_rb,0.31756e-02_rb,0.32901e-02_rb,0.36773e-04_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.97386e-03_rb,0.14865e-02_rb,0.19422e-02_rb,0.23265e-02_rb,0.26377e-02_rb, &
        & 0.28742e-02_rb,0.30758e-02_rb,0.32440e-02_rb,0.42838e-12_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.98704e-03_rb,0.15018e-02_rb,0.19617e-02_rb,0.23518e-02_rb,0.26608e-02_rb, &
        & 0.28955e-02_rb,0.31025e-02_rb,0.32660e-02_rb,0.54956e-12_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.10034e-02_rb,0.15127e-02_rb,0.19822e-02_rb,0.23722e-02_rb,0.26791e-02_rb, &
        & 0.29131e-02_rb,0.31272e-02_rb,0.32827e-02_rb,0.68281e-12_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.10185e-02_rb,0.15248e-02_rb,0.19990e-02_rb,0.23896e-02_rb,0.26942e-02_rb, &
        & 0.29331e-02_rb,0.31525e-02_rb,0.33003e-02_rb,0.47348e-04_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.10353e-02_rb,0.15370e-02_rb,0.20122e-02_rb,0.24032e-02_rb,0.27096e-02_rb, &
        & 0.29526e-02_rb,0.31774e-02_rb,0.33154e-02_rb,0.94094e-12_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.11127e-01_rb,0.97365e-02_rb,0.83450e-02_rb,0.69546e-02_rb,0.57518e-02_rb, &
        & 0.48972e-02_rb,0.42517e-02_rb,0.34771e-02_rb,0.44616e-02_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.11168e-01_rb,0.97719e-02_rb,0.83762e-02_rb,0.69801e-02_rb,0.57613e-02_rb, &
        & 0.48914e-02_rb,0.42324e-02_rb,0.34356e-02_rb,0.44183e-02_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.11206e-01_rb,0.98061e-02_rb,0.84051e-02_rb,0.70040e-02_rb,0.57786e-02_rb, &
        & 0.48973e-02_rb,0.42300e-02_rb,0.34167e-02_rb,0.44088e-02_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.11230e-01_rb,0.98265e-02_rb,0.84228e-02_rb,0.70190e-02_rb,0.57794e-02_rb, &
        & 0.48702e-02_rb,0.41927e-02_rb,0.33658e-02_rb,0.43365e-02_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.11247e-01_rb,0.98411e-02_rb,0.84348e-02_rb,0.70292e-02_rb,0.57800e-02_rb, &
        & 0.48600e-02_rb,0.41735e-02_rb,0.33367e-02_rb,0.43003e-02_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.11360e-01_rb,0.99403e-02_rb,0.85204e-02_rb,0.71449e-02_rb,0.60754e-02_rb, &
        & 0.53678e-02_rb,0.47333e-02_rb,0.38804e-02_rb,0.48459e-02_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.11402e-01_rb,0.99768e-02_rb,0.85517e-02_rb,0.71687e-02_rb,0.60816e-02_rb, &
        & 0.53522e-02_rb,0.47062e-02_rb,0.38343e-02_rb,0.48034e-02_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.11432e-01_rb,0.10003e-01_rb,0.85744e-02_rb,0.71872e-02_rb,0.60934e-02_rb, &
        & 0.53498e-02_rb,0.46968e-02_rb,0.38177e-02_rb,0.48004e-02_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.11460e-01_rb,0.10028e-01_rb,0.85956e-02_rb,0.72000e-02_rb,0.60909e-02_rb, &
        & 0.53282e-02_rb,0.46686e-02_rb,0.37852e-02_rb,0.47598e-02_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.11484e-01_rb,0.10049e-01_rb,0.86132e-02_rb,0.72108e-02_rb,0.60794e-02_rb, &
        & 0.52951e-02_rb,0.46284e-02_rb,0.37347e-02_rb,0.46895e-02_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.11375e-01_rb,0.99535e-02_rb,0.85318e-02_rb,0.72748e-02_rb,0.63965e-02_rb, &
        & 0.58374e-02_rb,0.52215e-02_rb,0.43002e-02_rb,0.52258e-02_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.11412e-01_rb,0.99855e-02_rb,0.85592e-02_rb,0.72909e-02_rb,0.63926e-02_rb, &
        & 0.58142e-02_rb,0.51838e-02_rb,0.42530e-02_rb,0.51857e-02_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.11451e-01_rb,0.10019e-01_rb,0.85873e-02_rb,0.73027e-02_rb,0.63856e-02_rb, &
        & 0.57886e-02_rb,0.51485e-02_rb,0.42153e-02_rb,0.51479e-02_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.11483e-01_rb,0.10047e-01_rb,0.86122e-02_rb,0.73190e-02_rb,0.63840e-02_rb, &
        & 0.57742e-02_rb,0.51342e-02_rb,0.42027e-02_rb,0.51482e-02_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.11512e-01_rb,0.10073e-01_rb,0.86339e-02_rb,0.73301e-02_rb,0.63706e-02_rb, &
        & 0.57344e-02_rb,0.50899e-02_rb,0.41496e-02_rb,0.50782e-02_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.11149e-01_rb,0.97560e-02_rb,0.83906e-02_rb,0.73568e-02_rb,0.67050e-02_rb, &
        & 0.62789e-02_rb,0.56809e-02_rb,0.47244e-02_rb,0.55742e-02_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.11201e-01_rb,0.98016e-02_rb,0.84237e-02_rb,0.73684e-02_rb,0.66927e-02_rb, &
        & 0.62506e-02_rb,0.56398e-02_rb,0.46775e-02_rb,0.55381e-02_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.11250e-01_rb,0.98437e-02_rb,0.84576e-02_rb,0.73794e-02_rb,0.66793e-02_rb, &
        & 0.62192e-02_rb,0.56018e-02_rb,0.46402e-02_rb,0.55028e-02_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.11293e-01_rb,0.98814e-02_rb,0.84883e-02_rb,0.73928e-02_rb,0.66658e-02_rb, &
        & 0.61879e-02_rb,0.55699e-02_rb,0.46073e-02_rb,0.54669e-02_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.11341e-01_rb,0.99237e-02_rb,0.85206e-02_rb,0.74085e-02_rb,0.66636e-02_rb, &
        & 0.61710e-02_rb,0.55548e-02_rb,0.45934e-02_rb,0.54646e-02_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.10704e-01_rb,0.93660e-02_rb,0.81568e-02_rb,0.74021e-02_rb,0.69945e-02_rb, &
        & 0.66654e-02_rb,0.61172e-02_rb,0.51470e-02_rb,0.58967e-02_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.10771e-01_rb,0.94250e-02_rb,0.81953e-02_rb,0.74101e-02_rb,0.69769e-02_rb, &
        & 0.66347e-02_rb,0.60743e-02_rb,0.50992e-02_rb,0.58616e-02_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.10834e-01_rb,0.94795e-02_rb,0.82348e-02_rb,0.74202e-02_rb,0.69586e-02_rb, &
        & 0.66047e-02_rb,0.60376e-02_rb,0.50623e-02_rb,0.58279e-02_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.10899e-01_rb,0.95365e-02_rb,0.82726e-02_rb,0.74343e-02_rb,0.69400e-02_rb, &
        & 0.65760e-02_rb,0.60076e-02_rb,0.50289e-02_rb,0.57953e-02_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.10965e-01_rb,0.95941e-02_rb,0.83123e-02_rb,0.74464e-02_rb,0.69292e-02_rb, &
        & 0.65626e-02_rb,0.59940e-02_rb,0.50178e-02_rb,0.57938e-02_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.10078e-01_rb,0.88178e-02_rb,0.78666e-02_rb,0.74112e-02_rb,0.72308e-02_rb, &
        & 0.70100e-02_rb,0.65312e-02_rb,0.55779e-02_rb,0.62140e-02_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.10161e-01_rb,0.88906e-02_rb,0.79113e-02_rb,0.74185e-02_rb,0.72160e-02_rb, &
        & 0.69843e-02_rb,0.64888e-02_rb,0.55247e-02_rb,0.61818e-02_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.10243e-01_rb,0.89616e-02_rb,0.79583e-02_rb,0.74318e-02_rb,0.71999e-02_rb, &
        & 0.69579e-02_rb,0.64523e-02_rb,0.54833e-02_rb,0.61492e-02_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.10327e-01_rb,0.90374e-02_rb,0.79989e-02_rb,0.74443e-02_rb,0.71832e-02_rb, &
        & 0.69350e-02_rb,0.64210e-02_rb,0.54479e-02_rb,0.61172e-02_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.10409e-01_rb,0.91075e-02_rb,0.80421e-02_rb,0.74489e-02_rb,0.71645e-02_rb, &
        & 0.69118e-02_rb,0.63952e-02_rb,0.54129e-02_rb,0.60849e-02_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.93799e-02_rb,0.82246e-02_rb,0.75645e-02_rb,0.73983e-02_rb,0.74124e-02_rb, &
        & 0.73054e-02_rb,0.69103e-02_rb,0.59986e-02_rb,0.65205e-02_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.94746e-02_rb,0.83048e-02_rb,0.76105e-02_rb,0.74041e-02_rb,0.74008e-02_rb, &
        & 0.72835e-02_rb,0.68745e-02_rb,0.59435e-02_rb,0.64911e-02_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.95665e-02_rb,0.83814e-02_rb,0.76560e-02_rb,0.74150e-02_rb,0.73885e-02_rb, &
        & 0.72645e-02_rb,0.68399e-02_rb,0.59000e-02_rb,0.64612e-02_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.96610e-02_rb,0.84615e-02_rb,0.76953e-02_rb,0.74218e-02_rb,0.73786e-02_rb, &
        & 0.72467e-02_rb,0.68110e-02_rb,0.58618e-02_rb,0.64309e-02_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.97576e-02_rb,0.85445e-02_rb,0.77428e-02_rb,0.74271e-02_rb,0.73666e-02_rb, &
        & 0.72295e-02_rb,0.67856e-02_rb,0.58248e-02_rb,0.64018e-02_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.86695e-02_rb,0.76539e-02_rb,0.72832e-02_rb,0.73811e-02_rb,0.75539e-02_rb, &
        & 0.75395e-02_rb,0.72391e-02_rb,0.63979e-02_rb,0.68067e-02_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.87714e-02_rb,0.77350e-02_rb,0.73260e-02_rb,0.73889e-02_rb,0.75481e-02_rb, &
        & 0.75307e-02_rb,0.72059e-02_rb,0.63386e-02_rb,0.67797e-02_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.88648e-02_rb,0.78065e-02_rb,0.73635e-02_rb,0.73928e-02_rb,0.75428e-02_rb, &
        & 0.75176e-02_rb,0.71786e-02_rb,0.62948e-02_rb,0.67512e-02_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.89696e-02_rb,0.78903e-02_rb,0.74008e-02_rb,0.73947e-02_rb,0.75364e-02_rb, &
        & 0.75091e-02_rb,0.71562e-02_rb,0.62573e-02_rb,0.67249e-02_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.90827e-02_rb,0.79827e-02_rb,0.74524e-02_rb,0.74004e-02_rb,0.75307e-02_rb, &
        & 0.75004e-02_rb,0.71406e-02_rb,0.62196e-02_rb,0.66995e-02_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.79522e-02_rb,0.71197e-02_rb,0.70358e-02_rb,0.73522e-02_rb,0.76328e-02_rb, &
        & 0.77097e-02_rb,0.75175e-02_rb,0.67703e-02_rb,0.70524e-02_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.80598e-02_rb,0.71969e-02_rb,0.70715e-02_rb,0.73653e-02_rb,0.76374e-02_rb, &
        & 0.77113e-02_rb,0.74908e-02_rb,0.67151e-02_rb,0.70338e-02_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.81632e-02_rb,0.72678e-02_rb,0.71027e-02_rb,0.73712e-02_rb,0.76458e-02_rb, &
        & 0.77123e-02_rb,0.74742e-02_rb,0.66728e-02_rb,0.70182e-02_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.82823e-02_rb,0.73561e-02_rb,0.71392e-02_rb,0.73768e-02_rb,0.76576e-02_rb, &
        & 0.77143e-02_rb,0.74622e-02_rb,0.66357e-02_rb,0.69991e-02_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.84105e-02_rb,0.74558e-02_rb,0.71910e-02_rb,0.73874e-02_rb,0.76648e-02_rb, &
        & 0.77180e-02_rb,0.74509e-02_rb,0.65977e-02_rb,0.69736e-02_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.72630e-02_rb,0.66377e-02_rb,0.68373e-02_rb,0.72984e-02_rb,0.76509e-02_rb, &
        & 0.78294e-02_rb,0.77460e-02_rb,0.71207e-02_rb,0.20344e-04_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.73671e-02_rb,0.67055e-02_rb,0.68587e-02_rb,0.73198e-02_rb,0.76710e-02_rb, &
        & 0.78437e-02_rb,0.77323e-02_rb,0.70655e-02_rb,0.20564e-04_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.74810e-02_rb,0.67793e-02_rb,0.68858e-02_rb,0.73353e-02_rb,0.76947e-02_rb, &
        & 0.78590e-02_rb,0.77307e-02_rb,0.70272e-02_rb,0.37563e-05_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.76171e-02_rb,0.68754e-02_rb,0.69230e-02_rb,0.73502e-02_rb,0.77241e-02_rb, &
        & 0.78765e-02_rb,0.77271e-02_rb,0.69915e-02_rb,0.47236e-05_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.77608e-02_rb,0.69811e-02_rb,0.69732e-02_rb,0.73676e-02_rb,0.77455e-02_rb, &
        & 0.78941e-02_rb,0.77236e-02_rb,0.69531e-02_rb,0.26047e-04_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.66493e-02_rb,0.62366e-02_rb,0.66807e-02_rb,0.72344e-02_rb,0.76404e-02_rb, &
        & 0.79217e-02_rb,0.79405e-02_rb,0.74097e-02_rb,0.00000e+00_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.67616e-02_rb,0.63044e-02_rb,0.67053e-02_rb,0.72656e-02_rb,0.76819e-02_rb, &
        & 0.79515e-02_rb,0.79506e-02_rb,0.73718e-02_rb,0.18408e-04_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.69005e-02_rb,0.63894e-02_rb,0.67349e-02_rb,0.72931e-02_rb,0.77291e-02_rb, &
        & 0.79845e-02_rb,0.79576e-02_rb,0.73388e-02_rb,0.37607e-04_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.70530e-02_rb,0.64912e-02_rb,0.67712e-02_rb,0.73223e-02_rb,0.77726e-02_rb, &
        & 0.80170e-02_rb,0.79618e-02_rb,0.73054e-02_rb,0.56367e-04_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.72039e-02_rb,0.65911e-02_rb,0.68118e-02_rb,0.73572e-02_rb,0.78096e-02_rb, &
        & 0.80477e-02_rb,0.79648e-02_rb,0.72708e-02_rb,0.00000e+00_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.60858e-02_rb,0.58916e-02_rb,0.65335e-02_rb,0.71525e-02_rb,0.76284e-02_rb, &
        & 0.80006e-02_rb,0.81294e-02_rb,0.76675e-02_rb,0.58588e-04_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.62191e-02_rb,0.59612e-02_rb,0.65675e-02_rb,0.71998e-02_rb,0.76898e-02_rb, &
        & 0.80500e-02_rb,0.81534e-02_rb,0.76479e-02_rb,0.00000e+00_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.63725e-02_rb,0.60546e-02_rb,0.66095e-02_rb,0.72445e-02_rb,0.77558e-02_rb, &
        & 0.80973e-02_rb,0.81695e-02_rb,0.76255e-02_rb,0.29841e-04_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.65241e-02_rb,0.61494e-02_rb,0.66519e-02_rb,0.72916e-02_rb,0.78126e-02_rb, &
        & 0.81445e-02_rb,0.81839e-02_rb,0.75955e-02_rb,0.00000e+00_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.66908e-02_rb,0.62534e-02_rb,0.66904e-02_rb,0.73429e-02_rb,0.78615e-02_rb, &
        & 0.81879e-02_rb,0.81898e-02_rb,0.75652e-02_rb,0.00000e+00_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.55891e-02_rb,0.56116e-02_rb,0.63936e-02_rb,0.70621e-02_rb,0.76241e-02_rb, &
        & 0.80837e-02_rb,0.82964e-02_rb,0.79007e-02_rb,0.00000e+00_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.57354e-02_rb,0.56825e-02_rb,0.64439e-02_rb,0.71280e-02_rb,0.77039e-02_rb, &
        & 0.81460e-02_rb,0.83322e-02_rb,0.78896e-02_rb,0.38090e-04_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.58847e-02_rb,0.57651e-02_rb,0.64971e-02_rb,0.71963e-02_rb,0.77792e-02_rb, &
        & 0.82047e-02_rb,0.83632e-02_rb,0.78746e-02_rb,0.00000e+00_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.60503e-02_rb,0.58579e-02_rb,0.65448e-02_rb,0.72663e-02_rb,0.78467e-02_rb, &
        & 0.82617e-02_rb,0.83886e-02_rb,0.78555e-02_rb,0.00000e+00_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.62326e-02_rb,0.59678e-02_rb,0.65939e-02_rb,0.73313e-02_rb,0.79071e-02_rb, &
        & 0.83133e-02_rb,0.83975e-02_rb,0.78294e-02_rb,0.00000e+00_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.22784e-01_rb,0.19937e-01_rb,0.17089e-01_rb,0.14240e-01_rb,0.11393e-01_rb, &
        & 0.85439e-02_rb,0.62360e-02_rb,0.50515e-02_rb,0.76952e-02_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.22781e-01_rb,0.19933e-01_rb,0.17085e-01_rb,0.14238e-01_rb,0.11390e-01_rb, &
        & 0.85426e-02_rb,0.62118e-02_rb,0.49990e-02_rb,0.75953e-02_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.22781e-01_rb,0.19934e-01_rb,0.17086e-01_rb,0.14239e-01_rb,0.11390e-01_rb, &
        & 0.85428e-02_rb,0.62149e-02_rb,0.49913e-02_rb,0.75531e-02_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.22807e-01_rb,0.19955e-01_rb,0.17104e-01_rb,0.14253e-01_rb,0.11403e-01_rb, &
        & 0.85521e-02_rb,0.61662e-02_rb,0.48921e-02_rb,0.74213e-02_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.22846e-01_rb,0.19991e-01_rb,0.17134e-01_rb,0.14278e-01_rb,0.11422e-01_rb, &
        & 0.85672e-02_rb,0.61477e-02_rb,0.48442e-02_rb,0.73437e-02_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.25963e-01_rb,0.22716e-01_rb,0.19471e-01_rb,0.16227e-01_rb,0.12981e-01_rb, &
        & 0.97432e-02_rb,0.72799e-02_rb,0.58913e-02_rb,0.89930e-02_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.25957e-01_rb,0.22712e-01_rb,0.19468e-01_rb,0.16223e-01_rb,0.12978e-01_rb, &
        & 0.97376e-02_rb,0.72409e-02_rb,0.58243e-02_rb,0.88738e-02_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.25967e-01_rb,0.22721e-01_rb,0.19475e-01_rb,0.16229e-01_rb,0.12984e-01_rb, &
        & 0.97376e-02_rb,0.72294e-02_rb,0.57970e-02_rb,0.88082e-02_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.25991e-01_rb,0.22741e-01_rb,0.19493e-01_rb,0.16243e-01_rb,0.12995e-01_rb, &
        & 0.97464e-02_rb,0.71939e-02_rb,0.57194e-02_rb,0.86974e-02_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.26012e-01_rb,0.22759e-01_rb,0.19508e-01_rb,0.16256e-01_rb,0.13005e-01_rb, &
        & 0.97539e-02_rb,0.71378e-02_rb,0.56167e-02_rb,0.85506e-02_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.29273e-01_rb,0.25614e-01_rb,0.21954e-01_rb,0.18296e-01_rb,0.14638e-01_rb, &
        & 0.11059e-01_rb,0.84536e-02_rb,0.68488e-02_rb,0.10453e-01_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.29273e-01_rb,0.25613e-01_rb,0.21954e-01_rb,0.18296e-01_rb,0.14636e-01_rb, &
        & 0.11040e-01_rb,0.84006e-02_rb,0.67671e-02_rb,0.10308e-01_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.29281e-01_rb,0.25620e-01_rb,0.21957e-01_rb,0.18299e-01_rb,0.14639e-01_rb, &
        & 0.11029e-01_rb,0.83532e-02_rb,0.66803e-02_rb,0.10170e-01_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.29284e-01_rb,0.25622e-01_rb,0.21963e-01_rb,0.18303e-01_rb,0.14642e-01_rb, &
        & 0.11027e-01_rb,0.83265e-02_rb,0.66328e-02_rb,0.10072e-01_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.29273e-01_rb,0.25615e-01_rb,0.21955e-01_rb,0.18297e-01_rb,0.14636e-01_rb, &
        & 0.11016e-01_rb,0.82524e-02_rb,0.65141e-02_rb,0.98927e-02_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.32689e-01_rb,0.28601e-01_rb,0.24517e-01_rb,0.20431e-01_rb,0.16343e-01_rb, &
        & 0.12498e-01_rb,0.97860e-02_rb,0.79359e-02_rb,0.11964e-01_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.32673e-01_rb,0.28590e-01_rb,0.24505e-01_rb,0.20420e-01_rb,0.16336e-01_rb, &
        & 0.12462e-01_rb,0.97090e-02_rb,0.78356e-02_rb,0.11793e-01_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.32657e-01_rb,0.28575e-01_rb,0.24494e-01_rb,0.20410e-01_rb,0.16330e-01_rb, &
        & 0.12434e-01_rb,0.96384e-02_rb,0.77321e-02_rb,0.11625e-01_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.32631e-01_rb,0.28553e-01_rb,0.24473e-01_rb,0.20396e-01_rb,0.16316e-01_rb, &
        & 0.12404e-01_rb,0.95671e-02_rb,0.76286e-02_rb,0.11457e-01_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.32595e-01_rb,0.28521e-01_rb,0.24445e-01_rb,0.20372e-01_rb,0.16297e-01_rb, &
        & 0.12376e-01_rb,0.95137e-02_rb,0.75694e-02_rb,0.11318e-01_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.36129e-01_rb,0.31613e-01_rb,0.27097e-01_rb,0.22582e-01_rb,0.18065e-01_rb, &
        & 0.14059e-01_rb,0.11253e-01_rb,0.91497e-02_rb,0.13513e-01_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.36091e-01_rb,0.31580e-01_rb,0.27068e-01_rb,0.22557e-01_rb,0.18046e-01_rb, &
        & 0.13996e-01_rb,0.11159e-01_rb,0.90273e-02_rb,0.13314e-01_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.36050e-01_rb,0.31543e-01_rb,0.27038e-01_rb,0.22530e-01_rb,0.18025e-01_rb, &
        & 0.13938e-01_rb,0.11057e-01_rb,0.89022e-02_rb,0.13117e-01_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.35994e-01_rb,0.31496e-01_rb,0.26994e-01_rb,0.22495e-01_rb,0.17996e-01_rb, &
        & 0.13877e-01_rb,0.10953e-01_rb,0.87755e-02_rb,0.12914e-01_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.35944e-01_rb,0.31450e-01_rb,0.26957e-01_rb,0.22466e-01_rb,0.17973e-01_rb, &
        & 0.13819e-01_rb,0.10863e-01_rb,0.86942e-02_rb,0.12744e-01_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.39506e-01_rb,0.34566e-01_rb,0.29627e-01_rb,0.24691e-01_rb,0.19819e-01_rb, &
        & 0.15734e-01_rb,0.12852e-01_rb,0.10482e-01_rb,0.15100e-01_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.39438e-01_rb,0.34510e-01_rb,0.29581e-01_rb,0.24651e-01_rb,0.19767e-01_rb, &
        & 0.15626e-01_rb,0.12723e-01_rb,0.10339e-01_rb,0.14869e-01_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.39363e-01_rb,0.34443e-01_rb,0.29524e-01_rb,0.24601e-01_rb,0.19714e-01_rb, &
        & 0.15531e-01_rb,0.12593e-01_rb,0.10189e-01_rb,0.14636e-01_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.39295e-01_rb,0.34380e-01_rb,0.29468e-01_rb,0.24559e-01_rb,0.19663e-01_rb, &
        & 0.15432e-01_rb,0.12453e-01_rb,0.10038e-01_rb,0.14402e-01_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.39232e-01_rb,0.34326e-01_rb,0.29423e-01_rb,0.24518e-01_rb,0.19622e-01_rb, &
        & 0.15336e-01_rb,0.12308e-01_rb,0.98939e-02_rb,0.14172e-01_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.42642e-01_rb,0.37314e-01_rb,0.31983e-01_rb,0.26652e-01_rb,0.21609e-01_rb, &
        & 0.17542e-01_rb,0.14584e-01_rb,0.11924e-01_rb,0.16714e-01_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.42588e-01_rb,0.37267e-01_rb,0.31942e-01_rb,0.26620e-01_rb,0.21537e-01_rb, &
        & 0.17385e-01_rb,0.14405e-01_rb,0.11750e-01_rb,0.16453e-01_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.42519e-01_rb,0.37203e-01_rb,0.31888e-01_rb,0.26575e-01_rb,0.21450e-01_rb, &
        & 0.17237e-01_rb,0.14233e-01_rb,0.11569e-01_rb,0.16190e-01_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.42450e-01_rb,0.37145e-01_rb,0.31837e-01_rb,0.26531e-01_rb,0.21364e-01_rb, &
        & 0.17089e-01_rb,0.14051e-01_rb,0.11390e-01_rb,0.15924e-01_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.42372e-01_rb,0.37075e-01_rb,0.31780e-01_rb,0.26482e-01_rb,0.21293e-01_rb, &
        & 0.16947e-01_rb,0.13866e-01_rb,0.11216e-01_rb,0.15662e-01_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.45438e-01_rb,0.39757e-01_rb,0.34077e-01_rb,0.28427e-01_rb,0.23418e-01_rb, &
        & 0.19456e-01_rb,0.16427e-01_rb,0.13456e-01_rb,0.18359e-01_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.45420e-01_rb,0.39742e-01_rb,0.34065e-01_rb,0.28396e-01_rb,0.23308e-01_rb, &
        & 0.19242e-01_rb,0.16196e-01_rb,0.13253e-01_rb,0.18076e-01_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.45404e-01_rb,0.39728e-01_rb,0.34054e-01_rb,0.28379e-01_rb,0.23190e-01_rb, &
        & 0.19044e-01_rb,0.15968e-01_rb,0.13036e-01_rb,0.17789e-01_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.45355e-01_rb,0.39684e-01_rb,0.34015e-01_rb,0.28348e-01_rb,0.23074e-01_rb, &
        & 0.18841e-01_rb,0.15735e-01_rb,0.12818e-01_rb,0.17487e-01_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.45268e-01_rb,0.39604e-01_rb,0.33951e-01_rb,0.28294e-01_rb,0.22964e-01_rb, &
        & 0.18639e-01_rb,0.15495e-01_rb,0.12608e-01_rb,0.17187e-01_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.47828e-01_rb,0.41848e-01_rb,0.35870e-01_rb,0.30096e-01_rb,0.25238e-01_rb, &
        & 0.21453e-01_rb,0.18348e-01_rb,0.15048e-01_rb,0.20030e-01_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.47883e-01_rb,0.41899e-01_rb,0.35911e-01_rb,0.30061e-01_rb,0.25102e-01_rb, &
        & 0.21178e-01_rb,0.18071e-01_rb,0.14812e-01_rb,0.19720e-01_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.47917e-01_rb,0.41930e-01_rb,0.35941e-01_rb,0.30033e-01_rb,0.24934e-01_rb, &
        & 0.20917e-01_rb,0.17786e-01_rb,0.14566e-01_rb,0.19396e-01_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.47914e-01_rb,0.41927e-01_rb,0.35936e-01_rb,0.29997e-01_rb,0.24771e-01_rb, &
        & 0.20657e-01_rb,0.17491e-01_rb,0.14311e-01_rb,0.19056e-01_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.47853e-01_rb,0.41872e-01_rb,0.35889e-01_rb,0.29935e-01_rb,0.24616e-01_rb, &
        & 0.20393e-01_rb,0.17196e-01_rb,0.14061e-01_rb,0.18721e-01_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.49786e-01_rb,0.43562e-01_rb,0.37346e-01_rb,0.31673e-01_rb,0.27058e-01_rb, &
        & 0.23446e-01_rb,0.20298e-01_rb,0.16634e-01_rb,0.20401e-01_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.49957e-01_rb,0.43713e-01_rb,0.37470e-01_rb,0.31624e-01_rb,0.26879e-01_rb, &
        & 0.23121e-01_rb,0.19962e-01_rb,0.16379e-01_rb,0.19980e-01_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.50073e-01_rb,0.43811e-01_rb,0.37553e-01_rb,0.31591e-01_rb,0.26668e-01_rb, &
        & 0.22810e-01_rb,0.19616e-01_rb,0.16101e-01_rb,0.19150e-01_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.50104e-01_rb,0.43839e-01_rb,0.37576e-01_rb,0.31542e-01_rb,0.26451e-01_rb, &
        & 0.22487e-01_rb,0.19263e-01_rb,0.15810e-01_rb,0.18309e-01_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.50083e-01_rb,0.43821e-01_rb,0.37560e-01_rb,0.31463e-01_rb,0.26247e-01_rb, &
        & 0.22153e-01_rb,0.18906e-01_rb,0.15516e-01_rb,0.17790e-01_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.51449e-01_rb,0.45016e-01_rb,0.38642e-01_rb,0.33136e-01_rb,0.28759e-01_rb, &
        & 0.25254e-01_rb,0.22053e-01_rb,0.18120e-01_rb,0.53897e-02_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.51718e-01_rb,0.45255e-01_rb,0.38815e-01_rb,0.33090e-01_rb,0.28517e-01_rb, &
        & 0.24894e-01_rb,0.21659e-01_rb,0.17825e-01_rb,0.47196e-02_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.51885e-01_rb,0.45397e-01_rb,0.38919e-01_rb,0.33034e-01_rb,0.28248e-01_rb, &
        & 0.24520e-01_rb,0.21252e-01_rb,0.17499e-01_rb,0.36639e-02_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.51965e-01_rb,0.45468e-01_rb,0.38969e-01_rb,0.32946e-01_rb,0.27987e-01_rb, &
        & 0.24125e-01_rb,0.20834e-01_rb,0.17164e-01_rb,0.30436e-02_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.52007e-01_rb,0.45508e-01_rb,0.39008e-01_rb,0.32828e-01_rb,0.27732e-01_rb, &
        & 0.23720e-01_rb,0.20415e-01_rb,0.16822e-01_rb,0.23024e-02_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.52772e-01_rb,0.46179e-01_rb,0.39786e-01_rb,0.34518e-01_rb,0.30341e-01_rb, &
        & 0.26996e-01_rb,0.23719e-01_rb,0.19625e-01_rb,0.00000e+00_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.53131e-01_rb,0.46489e-01_rb,0.39976e-01_rb,0.34451e-01_rb,0.30050e-01_rb, &
        & 0.26583e-01_rb,0.23271e-01_rb,0.19259e-01_rb,0.79250e-04_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.53372e-01_rb,0.46707e-01_rb,0.40090e-01_rb,0.34369e-01_rb,0.29734e-01_rb, &
        & 0.26140e-01_rb,0.22802e-01_rb,0.18870e-01_rb,0.00000e+00_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.53549e-01_rb,0.46855e-01_rb,0.40175e-01_rb,0.34238e-01_rb,0.29424e-01_rb, &
        & 0.25661e-01_rb,0.22318e-01_rb,0.18486e-01_rb,0.00000e+00_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.53641e-01_rb,0.46944e-01_rb,0.40238e-01_rb,0.34087e-01_rb,0.29108e-01_rb, &
        & 0.25184e-01_rb,0.21836e-01_rb,0.18091e-01_rb,0.00000e+00_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.53794e-01_rb,0.47070e-01_rb,0.40802e-01_rb,0.35804e-01_rb,0.31808e-01_rb, &
        & 0.28611e-01_rb,0.25317e-01_rb,0.21103e-01_rb,0.00000e+00_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.54248e-01_rb,0.47468e-01_rb,0.40994e-01_rb,0.35717e-01_rb,0.31468e-01_rb, &
        & 0.28147e-01_rb,0.24800e-01_rb,0.20666e-01_rb,0.99549e-04_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.54616e-01_rb,0.47795e-01_rb,0.41127e-01_rb,0.35591e-01_rb,0.31112e-01_rb, &
        & 0.27634e-01_rb,0.24251e-01_rb,0.20223e-01_rb,0.00000e+00_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.54881e-01_rb,0.48017e-01_rb,0.41234e-01_rb,0.35410e-01_rb,0.30749e-01_rb, &
        & 0.27090e-01_rb,0.23692e-01_rb,0.19766e-01_rb,0.00000e+00_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.55030e-01_rb,0.48150e-01_rb,0.41295e-01_rb,0.35229e-01_rb,0.30369e-01_rb, &
        & 0.26544e-01_rb,0.23150e-01_rb,0.19313e-01_rb,0.00000e+00_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.31412e-01_rb,0.27484e-01_rb,0.23556e-01_rb,0.19630e-01_rb,0.15703e-01_rb, &
        & 0.11777e-01_rb,0.78503e-02_rb,0.63859e-02_rb,0.10546e-01_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.31306e-01_rb,0.27393e-01_rb,0.23480e-01_rb,0.19566e-01_rb,0.15652e-01_rb, &
        & 0.11737e-01_rb,0.78244e-02_rb,0.64039e-02_rb,0.10543e-01_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.31189e-01_rb,0.27291e-01_rb,0.23390e-01_rb,0.19492e-01_rb,0.15594e-01_rb, &
        & 0.11694e-01_rb,0.77949e-02_rb,0.65607e-02_rb,0.10621e-01_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.31060e-01_rb,0.27175e-01_rb,0.23293e-01_rb,0.19412e-01_rb,0.15526e-01_rb, &
        & 0.11645e-01_rb,0.77621e-02_rb,0.64197e-02_rb,0.10470e-01_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.30934e-01_rb,0.27069e-01_rb,0.23201e-01_rb,0.19334e-01_rb,0.15466e-01_rb, &
        & 0.11599e-01_rb,0.77309e-02_rb,0.64181e-02_rb,0.10407e-01_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.37345e-01_rb,0.32673e-01_rb,0.28007e-01_rb,0.23341e-01_rb,0.18670e-01_rb, &
        & 0.14002e-01_rb,0.93330e-02_rb,0.76035e-02_rb,0.12759e-01_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.37160e-01_rb,0.32513e-01_rb,0.27871e-01_rb,0.23225e-01_rb,0.18579e-01_rb, &
        & 0.13933e-01_rb,0.92868e-02_rb,0.75902e-02_rb,0.12702e-01_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.36977e-01_rb,0.32353e-01_rb,0.27732e-01_rb,0.23111e-01_rb,0.18487e-01_rb, &
        & 0.13864e-01_rb,0.92415e-02_rb,0.77168e-02_rb,0.12729e-01_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.36807e-01_rb,0.32204e-01_rb,0.27602e-01_rb,0.23002e-01_rb,0.18402e-01_rb, &
        & 0.13798e-01_rb,0.91981e-02_rb,0.77023e-02_rb,0.12641e-01_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.36620e-01_rb,0.32046e-01_rb,0.27465e-01_rb,0.22887e-01_rb,0.18309e-01_rb, &
        & 0.13731e-01_rb,0.91527e-02_rb,0.75684e-02_rb,0.12457e-01_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.44232e-01_rb,0.38700e-01_rb,0.33173e-01_rb,0.27644e-01_rb,0.22114e-01_rb, &
        & 0.16581e-01_rb,0.11054e-01_rb,0.91028e-02_rb,0.15510e-01_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.43957e-01_rb,0.38463e-01_rb,0.32969e-01_rb,0.27474e-01_rb,0.21977e-01_rb, &
        & 0.16483e-01_rb,0.10987e-01_rb,0.90298e-02_rb,0.15354e-01_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.43680e-01_rb,0.38217e-01_rb,0.32756e-01_rb,0.27297e-01_rb,0.21838e-01_rb, &
        & 0.16376e-01_rb,0.10918e-01_rb,0.89896e-02_rb,0.15195e-01_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.43409e-01_rb,0.37982e-01_rb,0.32556e-01_rb,0.27128e-01_rb,0.21703e-01_rb, &
        & 0.16277e-01_rb,0.10849e-01_rb,0.90630e-02_rb,0.15090e-01_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.43148e-01_rb,0.37757e-01_rb,0.32362e-01_rb,0.26967e-01_rb,0.21574e-01_rb, &
        & 0.16179e-01_rb,0.10784e-01_rb,0.89323e-02_rb,0.14861e-01_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.52173e-01_rb,0.45650e-01_rb,0.39129e-01_rb,0.32609e-01_rb,0.26084e-01_rb, &
        & 0.19564e-01_rb,0.13063e-01_rb,0.10928e-01_rb,0.18665e-01_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.51767e-01_rb,0.45298e-01_rb,0.38824e-01_rb,0.32358e-01_rb,0.25882e-01_rb, &
        & 0.19411e-01_rb,0.12948e-01_rb,0.10795e-01_rb,0.18386e-01_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.51358e-01_rb,0.44935e-01_rb,0.38514e-01_rb,0.32093e-01_rb,0.25677e-01_rb, &
        & 0.19255e-01_rb,0.12843e-01_rb,0.10694e-01_rb,0.18120e-01_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.50955e-01_rb,0.44587e-01_rb,0.38218e-01_rb,0.31847e-01_rb,0.25475e-01_rb, &
        & 0.19107e-01_rb,0.12737e-01_rb,0.10607e-01_rb,0.17851e-01_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.50544e-01_rb,0.44227e-01_rb,0.37909e-01_rb,0.31588e-01_rb,0.25270e-01_rb, &
        & 0.18952e-01_rb,0.12634e-01_rb,0.10599e-01_rb,0.17618e-01_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.61214e-01_rb,0.53565e-01_rb,0.45917e-01_rb,0.38262e-01_rb,0.30607e-01_rb, &
        & 0.22954e-01_rb,0.15462e-01_rb,0.13125e-01_rb,0.22278e-01_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.60651e-01_rb,0.53068e-01_rb,0.45484e-01_rb,0.37903e-01_rb,0.30322e-01_rb, &
        & 0.22741e-01_rb,0.15244e-01_rb,0.12927e-01_rb,0.21870e-01_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.60040e-01_rb,0.52533e-01_rb,0.45028e-01_rb,0.37524e-01_rb,0.30019e-01_rb, &
        & 0.22514e-01_rb,0.15074e-01_rb,0.12758e-01_rb,0.21465e-01_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.59437e-01_rb,0.52006e-01_rb,0.44573e-01_rb,0.37145e-01_rb,0.29715e-01_rb, &
        & 0.22286e-01_rb,0.14911e-01_rb,0.12602e-01_rb,0.21062e-01_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.58809e-01_rb,0.51461e-01_rb,0.44106e-01_rb,0.36755e-01_rb,0.29405e-01_rb, &
        & 0.22053e-01_rb,0.14746e-01_rb,0.12499e-01_rb,0.20685e-01_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.71375e-01_rb,0.62451e-01_rb,0.53529e-01_rb,0.44607e-01_rb,0.35687e-01_rb, &
        & 0.26759e-01_rb,0.18273e-01_rb,0.15758e-01_rb,0.26378e-01_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.70571e-01_rb,0.61748e-01_rb,0.52933e-01_rb,0.44107e-01_rb,0.35285e-01_rb, &
        & 0.26462e-01_rb,0.17959e-01_rb,0.15470e-01_rb,0.25822e-01_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.69734e-01_rb,0.61022e-01_rb,0.52301e-01_rb,0.43583e-01_rb,0.34867e-01_rb, &
        & 0.26147e-01_rb,0.17660e-01_rb,0.15207e-01_rb,0.25265e-01_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.68870e-01_rb,0.60256e-01_rb,0.51654e-01_rb,0.43043e-01_rb,0.34432e-01_rb, &
        & 0.25826e-01_rb,0.17391e-01_rb,0.14968e-01_rb,0.24710e-01_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.67988e-01_rb,0.59482e-01_rb,0.50988e-01_rb,0.42488e-01_rb,0.33987e-01_rb, &
        & 0.25491e-01_rb,0.17159e-01_rb,0.14740e-01_rb,0.24148e-01_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.82705e-01_rb,0.72369e-01_rb,0.62031e-01_rb,0.51694e-01_rb,0.41353e-01_rb, &
        & 0.31012e-01_rb,0.21511e-01_rb,0.18843e-01_rb,0.31008e-01_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.81487e-01_rb,0.71304e-01_rb,0.61113e-01_rb,0.50924e-01_rb,0.40740e-01_rb, &
        & 0.30554e-01_rb,0.21074e-01_rb,0.18457e-01_rb,0.30235e-01_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.80318e-01_rb,0.70281e-01_rb,0.60235e-01_rb,0.50200e-01_rb,0.40157e-01_rb, &
        & 0.30117e-01_rb,0.20636e-01_rb,0.18102e-01_rb,0.29494e-01_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.79154e-01_rb,0.69255e-01_rb,0.59365e-01_rb,0.49470e-01_rb,0.39576e-01_rb, &
        & 0.29680e-01_rb,0.20228e-01_rb,0.17748e-01_rb,0.28759e-01_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.77937e-01_rb,0.68190e-01_rb,0.58450e-01_rb,0.48707e-01_rb,0.38965e-01_rb, &
        & 0.29223e-01_rb,0.19858e-01_rb,0.17406e-01_rb,0.28017e-01_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.95173e-01_rb,0.83270e-01_rb,0.71377e-01_rb,0.59480e-01_rb,0.47582e-01_rb, &
        & 0.35684e-01_rb,0.25169e-01_rb,0.22449e-01_rb,0.36274e-01_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.93409e-01_rb,0.81740e-01_rb,0.70058e-01_rb,0.58384e-01_rb,0.46709e-01_rb, &
        & 0.35028e-01_rb,0.24581e-01_rb,0.21893e-01_rb,0.35207e-01_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.91737e-01_rb,0.80263e-01_rb,0.68796e-01_rb,0.57330e-01_rb,0.45863e-01_rb, &
        & 0.34396e-01_rb,0.23987e-01_rb,0.21409e-01_rb,0.34174e-01_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.90113e-01_rb,0.78852e-01_rb,0.67585e-01_rb,0.56324e-01_rb,0.45060e-01_rb, &
        & 0.33794e-01_rb,0.23415e-01_rb,0.20940e-01_rb,0.33185e-01_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.88500e-01_rb,0.77445e-01_rb,0.66378e-01_rb,0.55317e-01_rb,0.44252e-01_rb, &
        & 0.33190e-01_rb,0.22890e-01_rb,0.20471e-01_rb,0.32236e-01_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.10878e+00_rb,0.95186e-01_rb,0.81587e-01_rb,0.67986e-01_rb,0.54386e-01_rb, &
        & 0.40792e-01_rb,0.29314e-01_rb,0.26669e-01_rb,0.42265e-01_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.10631e+00_rb,0.93021e-01_rb,0.79732e-01_rb,0.66444e-01_rb,0.53150e-01_rb, &
        & 0.39864e-01_rb,0.28508e-01_rb,0.25873e-01_rb,0.40792e-01_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.10400e+00_rb,0.91002e-01_rb,0.78003e-01_rb,0.64997e-01_rb,0.51997e-01_rb, &
        & 0.39001e-01_rb,0.27707e-01_rb,0.25186e-01_rb,0.39383e-01_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.10173e+00_rb,0.89018e-01_rb,0.76292e-01_rb,0.63582e-01_rb,0.50863e-01_rb, &
        & 0.38144e-01_rb,0.26926e-01_rb,0.24532e-01_rb,0.38084e-01_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.99555e-01_rb,0.87113e-01_rb,0.74666e-01_rb,0.62221e-01_rb,0.49774e-01_rb, &
        & 0.37334e-01_rb,0.26211e-01_rb,0.23902e-01_rb,0.36825e-01_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.12320e+00_rb,0.10779e+00_rb,0.92400e-01_rb,0.76993e-01_rb,0.61601e-01_rb, &
        & 0.46197e-01_rb,0.33879e-01_rb,0.31475e-01_rb,0.48792e-01_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.11986e+00_rb,0.10487e+00_rb,0.89890e-01_rb,0.74910e-01_rb,0.59927e-01_rb, &
        & 0.44946e-01_rb,0.32786e-01_rb,0.30373e-01_rb,0.46838e-01_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.11671e+00_rb,0.10212e+00_rb,0.87531e-01_rb,0.72938e-01_rb,0.58350e-01_rb, &
        & 0.43767e-01_rb,0.31728e-01_rb,0.29415e-01_rb,0.45011e-01_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.11373e+00_rb,0.99514e-01_rb,0.85291e-01_rb,0.71084e-01_rb,0.56859e-01_rb, &
        & 0.42648e-01_rb,0.30712e-01_rb,0.28518e-01_rb,0.43306e-01_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.11086e+00_rb,0.97005e-01_rb,0.83130e-01_rb,0.69287e-01_rb,0.55428e-01_rb, &
        & 0.41573e-01_rb,0.29748e-01_rb,0.27669e-01_rb,0.41673e-01_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.13635e+00_rb,0.11930e+00_rb,0.10226e+00_rb,0.85213e-01_rb,0.68170e-01_rb, &
        & 0.51126e-01_rb,0.38257e-01_rb,0.36212e-01_rb,0.54706e-01_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.13203e+00_rb,0.11554e+00_rb,0.99030e-01_rb,0.82523e-01_rb,0.66019e-01_rb, &
        & 0.49513e-01_rb,0.36799e-01_rb,0.34833e-01_rb,0.52312e-01_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.12805e+00_rb,0.11203e+00_rb,0.96022e-01_rb,0.80023e-01_rb,0.64019e-01_rb, &
        & 0.48015e-01_rb,0.35435e-01_rb,0.33591e-01_rb,0.50057e-01_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.12432e+00_rb,0.10877e+00_rb,0.93237e-01_rb,0.77695e-01_rb,0.62158e-01_rb, &
        & 0.46618e-01_rb,0.34181e-01_rb,0.32427e-01_rb,0.47918e-01_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.12077e+00_rb,0.10567e+00_rb,0.90572e-01_rb,0.75471e-01_rb,0.60381e-01_rb, &
        & 0.45284e-01_rb,0.32973e-01_rb,0.31331e-01_rb,0.45895e-01_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.14909e+00_rb,0.13045e+00_rb,0.11182e+00_rb,0.93176e-01_rb,0.74537e-01_rb, &
        & 0.55927e-01_rb,0.42736e-01_rb,0.41194e-01_rb,0.43864e-01_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.14380e+00_rb,0.12582e+00_rb,0.10785e+00_rb,0.89883e-01_rb,0.71900e-01_rb, &
        & 0.53925e-01_rb,0.40874e-01_rb,0.39509e-01_rb,0.34410e-01_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.13887e+00_rb,0.12152e+00_rb,0.10414e+00_rb,0.86796e-01_rb,0.69431e-01_rb, &
        & 0.52073e-01_rb,0.39165e-01_rb,0.37950e-01_rb,0.27228e-01_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.13436e+00_rb,0.11758e+00_rb,0.10078e+00_rb,0.83978e-01_rb,0.67186e-01_rb, &
        & 0.50383e-01_rb,0.37599e-01_rb,0.36473e-01_rb,0.23165e-01_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.13007e+00_rb,0.11382e+00_rb,0.97560e-01_rb,0.81293e-01_rb,0.65042e-01_rb, &
        & 0.48775e-01_rb,0.36131e-01_rb,0.35083e-01_rb,0.14412e-01_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.16113e+00_rb,0.14100e+00_rb,0.12085e+00_rb,0.10071e+00_rb,0.80563e-01_rb, &
        & 0.60580e-01_rb,0.47246e-01_rb,0.46365e-01_rb,0.26456e-01_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.15486e+00_rb,0.13550e+00_rb,0.11616e+00_rb,0.96791e-01_rb,0.77431e-01_rb, &
        & 0.58082e-01_rb,0.44915e-01_rb,0.44314e-01_rb,0.19340e-01_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.14906e+00_rb,0.13042e+00_rb,0.11179e+00_rb,0.93156e-01_rb,0.74526e-01_rb, &
        & 0.55893e-01_rb,0.42831e-01_rb,0.42380e-01_rb,0.11993e-01_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.14369e+00_rb,0.12574e+00_rb,0.10777e+00_rb,0.89817e-01_rb,0.71849e-01_rb, &
        & 0.53884e-01_rb,0.40915e-01_rb,0.40556e-01_rb,0.50486e-02_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.13868e+00_rb,0.12134e+00_rb,0.10400e+00_rb,0.86669e-01_rb,0.69338e-01_rb, &
        & 0.52006e-01_rb,0.39136e-01_rb,0.38851e-01_rb,0.00000e+00_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:,13, 1) = (/ &
        & 0.12158e-08_rb,0.13238e-08_rb,0.15146e-08_rb,0.16862e-08_rb,0.18609e-08_rb /)
      kbo(:,14, 1) = (/ &
        & 0.16931e-08_rb,0.20047e-08_rb,0.23006e-08_rb,0.25636e-08_rb,0.28912e-08_rb /)
      kbo(:,15, 1) = (/ &
        & 0.31521e-08_rb,0.36963e-08_rb,0.41550e-08_rb,0.46365e-08_rb,0.50834e-08_rb /)
      kbo(:,16, 1) = (/ &
        & 0.46152e-08_rb,0.50173e-08_rb,0.55381e-08_rb,0.60748e-08_rb,0.66355e-08_rb /)
      kbo(:,17, 1) = (/ &
        & 0.62662e-08_rb,0.67518e-08_rb,0.71765e-08_rb,0.80302e-08_rb,0.89056e-08_rb /)
      kbo(:,18, 1) = (/ &
        & 0.81764e-08_rb,0.91818e-08_rb,0.10385e-07_rb,0.11708e-07_rb,0.12536e-07_rb /)
      kbo(:,19, 1) = (/ &
        & 0.12946e-07_rb,0.13631e-07_rb,0.14453e-07_rb,0.15133e-07_rb,0.15911e-07_rb /)
      kbo(:,20, 1) = (/ &
        & 0.15868e-07_rb,0.16654e-07_rb,0.17281e-07_rb,0.17857e-07_rb,0.18383e-07_rb /)
      kbo(:,21, 1) = (/ &
        & 0.18151e-07_rb,0.19056e-07_rb,0.20057e-07_rb,0.20708e-07_rb,0.21229e-07_rb /)
      kbo(:,22, 1) = (/ &
        & 0.20234e-07_rb,0.21482e-07_rb,0.22599e-07_rb,0.23778e-07_rb,0.25115e-07_rb /)
      kbo(:,23, 1) = (/ &
        & 0.23827e-07_rb,0.25405e-07_rb,0.26545e-07_rb,0.27716e-07_rb,0.28422e-07_rb /)
      kbo(:,24, 1) = (/ &
        & 0.27638e-07_rb,0.28518e-07_rb,0.29101e-07_rb,0.29639e-07_rb,0.30077e-07_rb /)
      kbo(:,25, 1) = (/ &
        & 0.28942e-07_rb,0.29436e-07_rb,0.29831e-07_rb,0.30066e-07_rb,0.30242e-07_rb /)
      kbo(:,26, 1) = (/ &
        & 0.28774e-07_rb,0.29004e-07_rb,0.29272e-07_rb,0.29431e-07_rb,0.29616e-07_rb /)
      kbo(:,27, 1) = (/ &
        & 0.27317e-07_rb,0.27462e-07_rb,0.27698e-07_rb,0.27835e-07_rb,0.27843e-07_rb /)
      kbo(:,28, 1) = (/ &
        & 0.25463e-07_rb,0.25523e-07_rb,0.25678e-07_rb,0.25658e-07_rb,0.25687e-07_rb /)
      kbo(:,29, 1) = (/ &
        & 0.22603e-07_rb,0.22672e-07_rb,0.22704e-07_rb,0.22668e-07_rb,0.22698e-07_rb /)
      kbo(:,30, 1) = (/ &
        & 0.19657e-07_rb,0.19747e-07_rb,0.19687e-07_rb,0.19659e-07_rb,0.19680e-07_rb /)
      kbo(:,31, 1) = (/ &
        & 0.16553e-07_rb,0.16571e-07_rb,0.16522e-07_rb,0.16545e-07_rb,0.16563e-07_rb /)
      kbo(:,32, 1) = (/ &
        & 0.13867e-07_rb,0.13795e-07_rb,0.13808e-07_rb,0.13815e-07_rb,0.13799e-07_rb /)
      kbo(:,33, 1) = (/ &
        & 0.11414e-07_rb,0.11409e-07_rb,0.11395e-07_rb,0.11384e-07_rb,0.11416e-07_rb /)
      kbo(:,34, 1) = (/ &
        & 0.95639e-08_rb,0.95514e-08_rb,0.95251e-08_rb,0.95607e-08_rb,0.95767e-08_rb /)
      kbo(:,35, 1) = (/ &
        & 0.80018e-08_rb,0.79882e-08_rb,0.79720e-08_rb,0.79934e-08_rb,0.79942e-08_rb /)
      kbo(:,36, 1) = (/ &
        & 0.67279e-08_rb,0.67135e-08_rb,0.67089e-08_rb,0.67131e-08_rb,0.67290e-08_rb /)
      kbo(:,37, 1) = (/ &
        & 0.57292e-08_rb,0.57278e-08_rb,0.57247e-08_rb,0.57298e-08_rb,0.57269e-08_rb /)
      kbo(:,38, 1) = (/ &
        & 0.48406e-08_rb,0.48461e-08_rb,0.48394e-08_rb,0.48468e-08_rb,0.48401e-08_rb /)
      kbo(:,39, 1) = (/ &
        & 0.40368e-08_rb,0.40419e-08_rb,0.40353e-08_rb,0.40432e-08_rb,0.40267e-08_rb /)
      kbo(:,40, 1) = (/ &
        & 0.34875e-08_rb,0.34931e-08_rb,0.34872e-08_rb,0.34940e-08_rb,0.34782e-08_rb /)
      kbo(:,41, 1) = (/ &
        & 0.30195e-08_rb,0.30243e-08_rb,0.30129e-08_rb,0.30148e-08_rb,0.30016e-08_rb /)
      kbo(:,42, 1) = (/ &
        & 0.25993e-08_rb,0.25924e-08_rb,0.25861e-08_rb,0.25841e-08_rb,0.25734e-08_rb /)
      kbo(:,43, 1) = (/ &
        & 0.22290e-08_rb,0.22166e-08_rb,0.22080e-08_rb,0.22067e-08_rb,0.21992e-08_rb /)
      kbo(:,44, 1) = (/ &
        & 0.18916e-08_rb,0.18847e-08_rb,0.18733e-08_rb,0.18685e-08_rb,0.18685e-08_rb /)
      kbo(:,45, 1) = (/ &
        & 0.15840e-08_rb,0.15787e-08_rb,0.15719e-08_rb,0.15641e-08_rb,0.15644e-08_rb /)
      kbo(:,46, 1) = (/ &
        & 0.13197e-08_rb,0.13138e-08_rb,0.13098e-08_rb,0.13024e-08_rb,0.13049e-08_rb /)
      kbo(:,47, 1) = (/ &
        & 0.11052e-08_rb,0.11003e-08_rb,0.10954e-08_rb,0.10927e-08_rb,0.10941e-08_rb /)
      kbo(:,48, 1) = (/ &
        & 0.91115e-09_rb,0.90837e-09_rb,0.90520e-09_rb,0.90009e-09_rb,0.90047e-09_rb /)
      kbo(:,49, 1) = (/ &
        & 0.73101e-09_rb,0.73246e-09_rb,0.73006e-09_rb,0.72687e-09_rb,0.72508e-09_rb /)
      kbo(:,50, 1) = (/ &
        & 0.60565e-09_rb,0.60526e-09_rb,0.60121e-09_rb,0.60117e-09_rb,0.60056e-09_rb /)
      kbo(:,51, 1) = (/ &
        & 0.50736e-09_rb,0.50750e-09_rb,0.50421e-09_rb,0.50378e-09_rb,0.50259e-09_rb /)
      kbo(:,52, 1) = (/ &
        & 0.41843e-09_rb,0.41856e-09_rb,0.41499e-09_rb,0.41535e-09_rb,0.41353e-09_rb /)
      kbo(:,53, 1) = (/ &
        & 0.33766e-09_rb,0.33794e-09_rb,0.33511e-09_rb,0.33405e-09_rb,0.33405e-09_rb /)
      kbo(:,54, 1) = (/ &
        & 0.29473e-09_rb,0.29448e-09_rb,0.29284e-09_rb,0.29146e-09_rb,0.29053e-09_rb /)
      kbo(:,55, 1) = (/ &
        & 0.26948e-09_rb,0.27034e-09_rb,0.27016e-09_rb,0.26799e-09_rb,0.26580e-09_rb /)
      kbo(:,56, 1) = (/ &
        & 0.24773e-09_rb,0.24784e-09_rb,0.24734e-09_rb,0.24436e-09_rb,0.24377e-09_rb /)
      kbo(:,57, 1) = (/ &
        & 0.22707e-09_rb,0.22765e-09_rb,0.22683e-09_rb,0.22503e-09_rb,0.22447e-09_rb /)
      kbo(:,58, 1) = (/ &
        & 0.21180e-09_rb,0.21264e-09_rb,0.21254e-09_rb,0.21338e-09_rb,0.21089e-09_rb /)
      kbo(:,59, 1) = (/ &
        & 0.22898e-09_rb,0.23089e-09_rb,0.23471e-09_rb,0.23606e-09_rb,0.23388e-09_rb /)
      kbo(:,13, 2) = (/ &
        & 0.18001e-08_rb,0.19945e-08_rb,0.21612e-08_rb,0.23316e-08_rb,0.25983e-08_rb /)
      kbo(:,14, 2) = (/ &
        & 0.39853e-08_rb,0.42632e-08_rb,0.44602e-08_rb,0.46784e-08_rb,0.48071e-08_rb /)
      kbo(:,15, 2) = (/ &
        & 0.60848e-08_rb,0.61494e-08_rb,0.61069e-08_rb,0.61295e-08_rb,0.65707e-08_rb /)
      kbo(:,16, 2) = (/ &
        & 0.75494e-08_rb,0.85999e-08_rb,0.91057e-08_rb,0.10175e-07_rb,0.10943e-07_rb /)
      kbo(:,17, 2) = (/ &
        & 0.12037e-07_rb,0.12753e-07_rb,0.13314e-07_rb,0.13239e-07_rb,0.13306e-07_rb /)
      kbo(:,18, 2) = (/ &
        & 0.15519e-07_rb,0.15517e-07_rb,0.15097e-07_rb,0.14435e-07_rb,0.13983e-07_rb /)
      kbo(:,19, 2) = (/ &
        & 0.16420e-07_rb,0.17025e-07_rb,0.17639e-07_rb,0.17633e-07_rb,0.17513e-07_rb /)
      kbo(:,20, 2) = (/ &
        & 0.18021e-07_rb,0.19301e-07_rb,0.20621e-07_rb,0.22238e-07_rb,0.23351e-07_rb /)
      kbo(:,21, 2) = (/ &
        & 0.22378e-07_rb,0.23582e-07_rb,0.24409e-07_rb,0.24915e-07_rb,0.25516e-07_rb /)
      kbo(:,22, 2) = (/ &
        & 0.24712e-07_rb,0.24864e-07_rb,0.24556e-07_rb,0.23953e-07_rb,0.23424e-07_rb /)
      kbo(:,23, 2) = (/ &
        & 0.22545e-07_rb,0.22002e-07_rb,0.21325e-07_rb,0.20492e-07_rb,0.20022e-07_rb /)
      kbo(:,24, 2) = (/ &
        & 0.18798e-07_rb,0.18566e-07_rb,0.18366e-07_rb,0.18099e-07_rb,0.17971e-07_rb /)
      kbo(:,25, 2) = (/ &
        & 0.16773e-07_rb,0.16803e-07_rb,0.16541e-07_rb,0.16450e-07_rb,0.16482e-07_rb /)
      kbo(:,26, 2) = (/ &
        & 0.15222e-07_rb,0.15398e-07_rb,0.15177e-07_rb,0.15147e-07_rb,0.14951e-07_rb /)
      kbo(:,27, 2) = (/ &
        & 0.13792e-07_rb,0.13944e-07_rb,0.13534e-07_rb,0.13546e-07_rb,0.13516e-07_rb /)
      kbo(:,28, 2) = (/ &
        & 0.12157e-07_rb,0.12221e-07_rb,0.11930e-07_rb,0.12115e-07_rb,0.12027e-07_rb /)
      kbo(:,29, 2) = (/ &
        & 0.10487e-07_rb,0.10393e-07_rb,0.10366e-07_rb,0.10455e-07_rb,0.10269e-07_rb /)
      kbo(:,30, 2) = (/ &
        & 0.90084e-08_rb,0.88590e-08_rb,0.90055e-08_rb,0.88655e-08_rb,0.87392e-08_rb /)
      kbo(:,31, 2) = (/ &
        & 0.74765e-08_rb,0.74889e-08_rb,0.73915e-08_rb,0.73289e-08_rb,0.72219e-08_rb /)
      kbo(:,32, 2) = (/ &
        & 0.62311e-08_rb,0.62461e-08_rb,0.61378e-08_rb,0.59795e-08_rb,0.60309e-08_rb /)
      kbo(:,33, 2) = (/ &
        & 0.51421e-08_rb,0.50519e-08_rb,0.50031e-08_rb,0.49630e-08_rb,0.49489e-08_rb /)
      kbo(:,34, 2) = (/ &
        & 0.42880e-08_rb,0.42042e-08_rb,0.41366e-08_rb,0.41198e-08_rb,0.41265e-08_rb /)
      kbo(:,35, 2) = (/ &
        & 0.35718e-08_rb,0.34728e-08_rb,0.34599e-08_rb,0.34385e-08_rb,0.34695e-08_rb /)
      kbo(:,36, 2) = (/ &
        & 0.29713e-08_rb,0.29023e-08_rb,0.28707e-08_rb,0.28790e-08_rb,0.28873e-08_rb /)
      kbo(:,37, 2) = (/ &
        & 0.25246e-08_rb,0.24463e-08_rb,0.24323e-08_rb,0.24308e-08_rb,0.24753e-08_rb /)
      kbo(:,38, 2) = (/ &
        & 0.21159e-08_rb,0.20527e-08_rb,0.20532e-08_rb,0.20325e-08_rb,0.20615e-08_rb /)
      kbo(:,39, 2) = (/ &
        & 0.17631e-08_rb,0.17097e-08_rb,0.17043e-08_rb,0.16987e-08_rb,0.17298e-08_rb /)
      kbo(:,40, 2) = (/ &
        & 0.15260e-08_rb,0.14757e-08_rb,0.14566e-08_rb,0.14513e-08_rb,0.14651e-08_rb /)
      kbo(:,41, 2) = (/ &
        & 0.13009e-08_rb,0.12457e-08_rb,0.12523e-08_rb,0.12490e-08_rb,0.12671e-08_rb /)
      kbo(:,42, 2) = (/ &
        & 0.10990e-08_rb,0.10789e-08_rb,0.10741e-08_rb,0.10762e-08_rb,0.10873e-08_rb /)
      kbo(:,43, 2) = (/ &
        & 0.94313e-09_rb,0.92236e-09_rb,0.92369e-09_rb,0.91495e-09_rb,0.92405e-09_rb /)
      kbo(:,44, 2) = (/ &
        & 0.80746e-09_rb,0.78453e-09_rb,0.77646e-09_rb,0.77477e-09_rb,0.77888e-09_rb /)
      kbo(:,45, 2) = (/ &
        & 0.67944e-09_rb,0.66381e-09_rb,0.65747e-09_rb,0.65286e-09_rb,0.65668e-09_rb /)
      kbo(:,46, 2) = (/ &
        & 0.56677e-09_rb,0.55682e-09_rb,0.54825e-09_rb,0.54623e-09_rb,0.54843e-09_rb /)
      kbo(:,47, 2) = (/ &
        & 0.47375e-09_rb,0.47120e-09_rb,0.45836e-09_rb,0.45709e-09_rb,0.45689e-09_rb /)
      kbo(:,48, 2) = (/ &
        & 0.39797e-09_rb,0.39204e-09_rb,0.37735e-09_rb,0.37505e-09_rb,0.37687e-09_rb /)
      kbo(:,49, 2) = (/ &
        & 0.32772e-09_rb,0.31857e-09_rb,0.30808e-09_rb,0.30232e-09_rb,0.30140e-09_rb /)
      kbo(:,50, 2) = (/ &
        & 0.27300e-09_rb,0.26581e-09_rb,0.26125e-09_rb,0.24922e-09_rb,0.25029e-09_rb /)
      kbo(:,51, 2) = (/ &
        & 0.23073e-09_rb,0.22159e-09_rb,0.21979e-09_rb,0.20969e-09_rb,0.21117e-09_rb /)
      kbo(:,52, 2) = (/ &
        & 0.18868e-09_rb,0.18466e-09_rb,0.18517e-09_rb,0.17470e-09_rb,0.17469e-09_rb /)
      kbo(:,53, 2) = (/ &
        & 0.15290e-09_rb,0.14970e-09_rb,0.15032e-09_rb,0.14201e-09_rb,0.14053e-09_rb /)
      kbo(:,54, 2) = (/ &
        & 0.13616e-09_rb,0.13340e-09_rb,0.13256e-09_rb,0.12713e-09_rb,0.12621e-09_rb /)
      kbo(:,55, 2) = (/ &
        & 0.12605e-09_rb,0.12524e-09_rb,0.12106e-09_rb,0.11994e-09_rb,0.11735e-09_rb /)
      kbo(:,56, 2) = (/ &
        & 0.11625e-09_rb,0.11350e-09_rb,0.11287e-09_rb,0.11534e-09_rb,0.10942e-09_rb /)
      kbo(:,57, 2) = (/ &
        & 0.10168e-09_rb,0.10628e-09_rb,0.10684e-09_rb,0.10799e-09_rb,0.10313e-09_rb /)
      kbo(:,58, 2) = (/ &
        & 0.92659e-10_rb,0.10050e-09_rb,0.10168e-09_rb,0.97877e-10_rb,0.97971e-10_rb /)
      kbo(:,59, 2) = (/ &
        & 0.10198e-09_rb,0.11632e-09_rb,0.11127e-09_rb,0.10947e-09_rb,0.11000e-09_rb /)
      kbo(:,13, 3) = (/ &
        & 0.33788e-08_rb,0.38685e-08_rb,0.40059e-08_rb,0.41301e-08_rb,0.43316e-08_rb /)
      kbo(:,14, 3) = (/ &
        & 0.54022e-08_rb,0.55644e-08_rb,0.55814e-08_rb,0.55195e-08_rb,0.57708e-08_rb /)
      kbo(:,15, 3) = (/ &
        & 0.95769e-08_rb,0.10205e-07_rb,0.12445e-07_rb,0.13419e-07_rb,0.13357e-07_rb /)
      kbo(:,16, 3) = (/ &
        & 0.17233e-07_rb,0.16177e-07_rb,0.16092e-07_rb,0.15590e-07_rb,0.15068e-07_rb /)
      kbo(:,17, 3) = (/ &
        & 0.19731e-07_rb,0.19947e-07_rb,0.20278e-07_rb,0.20235e-07_rb,0.20699e-07_rb /)
      kbo(:,18, 3) = (/ &
        & 0.23168e-07_rb,0.23970e-07_rb,0.25507e-07_rb,0.27034e-07_rb,0.29135e-07_rb /)
      kbo(:,19, 3) = (/ &
        & 0.33749e-07_rb,0.35374e-07_rb,0.37333e-07_rb,0.38730e-07_rb,0.39816e-07_rb /)
      kbo(:,20, 3) = (/ &
        & 0.39844e-07_rb,0.39698e-07_rb,0.39651e-07_rb,0.38618e-07_rb,0.37699e-07_rb /)
      kbo(:,21, 3) = (/ &
        & 0.37013e-07_rb,0.35923e-07_rb,0.35560e-07_rb,0.35217e-07_rb,0.34594e-07_rb /)
      kbo(:,22, 3) = (/ &
        & 0.32381e-07_rb,0.31844e-07_rb,0.32272e-07_rb,0.32430e-07_rb,0.31922e-07_rb /)
      kbo(:,23, 3) = (/ &
        & 0.29924e-07_rb,0.29834e-07_rb,0.30537e-07_rb,0.30494e-07_rb,0.30313e-07_rb /)
      kbo(:,24, 3) = (/ &
        & 0.27849e-07_rb,0.28147e-07_rb,0.29005e-07_rb,0.29081e-07_rb,0.28673e-07_rb /)
      kbo(:,25, 3) = (/ &
        & 0.25534e-07_rb,0.25854e-07_rb,0.26978e-07_rb,0.27173e-07_rb,0.27116e-07_rb /)
      kbo(:,26, 3) = (/ &
        & 0.23413e-07_rb,0.24124e-07_rb,0.25341e-07_rb,0.25322e-07_rb,0.25266e-07_rb /)
      kbo(:,27, 3) = (/ &
        & 0.21218e-07_rb,0.22105e-07_rb,0.23470e-07_rb,0.23280e-07_rb,0.23250e-07_rb /)
      kbo(:,28, 3) = (/ &
        & 0.19002e-07_rb,0.20309e-07_rb,0.21106e-07_rb,0.20908e-07_rb,0.21199e-07_rb /)
      kbo(:,29, 3) = (/ &
        & 0.16901e-07_rb,0.18244e-07_rb,0.18426e-07_rb,0.18398e-07_rb,0.18708e-07_rb /)
      kbo(:,30, 3) = (/ &
        & 0.15065e-07_rb,0.15814e-07_rb,0.15711e-07_rb,0.16012e-07_rb,0.16145e-07_rb /)
      kbo(:,31, 3) = (/ &
        & 0.13100e-07_rb,0.13194e-07_rb,0.13385e-07_rb,0.13455e-07_rb,0.13435e-07_rb /)
      kbo(:,32, 3) = (/ &
        & 0.10972e-07_rb,0.10986e-07_rb,0.11085e-07_rb,0.11182e-07_rb,0.11030e-07_rb /)
      kbo(:,33, 3) = (/ &
        & 0.90750e-08_rb,0.90832e-08_rb,0.91769e-08_rb,0.89877e-08_rb,0.89513e-08_rb /)
      kbo(:,34, 3) = (/ &
        & 0.75008e-08_rb,0.76569e-08_rb,0.76237e-08_rb,0.74547e-08_rb,0.74719e-08_rb /)
      kbo(:,35, 3) = (/ &
        & 0.63063e-08_rb,0.64050e-08_rb,0.62067e-08_rb,0.61836e-08_rb,0.61775e-08_rb /)
      kbo(:,36, 3) = (/ &
        & 0.53250e-08_rb,0.52712e-08_rb,0.52454e-08_rb,0.51153e-08_rb,0.51657e-08_rb /)
      kbo(:,37, 3) = (/ &
        & 0.45450e-08_rb,0.45358e-08_rb,0.44274e-08_rb,0.43893e-08_rb,0.43616e-08_rb /)
      kbo(:,38, 3) = (/ &
        & 0.38039e-08_rb,0.38254e-08_rb,0.37172e-08_rb,0.37203e-08_rb,0.37278e-08_rb /)
      kbo(:,39, 3) = (/ &
        & 0.31821e-08_rb,0.31632e-08_rb,0.30774e-08_rb,0.30626e-08_rb,0.30979e-08_rb /)
      kbo(:,40, 3) = (/ &
        & 0.27339e-08_rb,0.27443e-08_rb,0.26754e-08_rb,0.26505e-08_rb,0.27022e-08_rb /)
      kbo(:,41, 3) = (/ &
        & 0.23939e-08_rb,0.24228e-08_rb,0.23415e-08_rb,0.23287e-08_rb,0.23327e-08_rb /)
      kbo(:,42, 3) = (/ &
        & 0.20895e-08_rb,0.20874e-08_rb,0.20298e-08_rb,0.20180e-08_rb,0.19876e-08_rb /)
      kbo(:,43, 3) = (/ &
        & 0.17869e-08_rb,0.17800e-08_rb,0.17392e-08_rb,0.17376e-08_rb,0.16927e-08_rb /)
      kbo(:,44, 3) = (/ &
        & 0.15169e-08_rb,0.15335e-08_rb,0.14778e-08_rb,0.14668e-08_rb,0.14407e-08_rb /)
      kbo(:,45, 3) = (/ &
        & 0.12692e-08_rb,0.12902e-08_rb,0.12439e-08_rb,0.12324e-08_rb,0.11996e-08_rb /)
      kbo(:,46, 3) = (/ &
        & 0.10611e-08_rb,0.10656e-08_rb,0.10472e-08_rb,0.10282e-08_rb,0.10015e-08_rb /)
      kbo(:,47, 3) = (/ &
        & 0.90394e-09_rb,0.88143e-09_rb,0.87958e-09_rb,0.84984e-09_rb,0.85121e-09_rb /)
      kbo(:,48, 3) = (/ &
        & 0.73451e-09_rb,0.71978e-09_rb,0.73014e-09_rb,0.70436e-09_rb,0.69840e-09_rb /)
      kbo(:,49, 3) = (/ &
        & 0.58678e-09_rb,0.58259e-09_rb,0.58316e-09_rb,0.55960e-09_rb,0.56079e-09_rb /)
      kbo(:,50, 3) = (/ &
        & 0.48502e-09_rb,0.47984e-09_rb,0.48693e-09_rb,0.47265e-09_rb,0.46016e-09_rb /)
      kbo(:,51, 3) = (/ &
        & 0.40704e-09_rb,0.40367e-09_rb,0.41046e-09_rb,0.39764e-09_rb,0.38714e-09_rb /)
      kbo(:,52, 3) = (/ &
        & 0.33771e-09_rb,0.33070e-09_rb,0.32984e-09_rb,0.33231e-09_rb,0.32299e-09_rb /)
      kbo(:,53, 3) = (/ &
        & 0.27062e-09_rb,0.27028e-09_rb,0.26516e-09_rb,0.26816e-09_rb,0.25661e-09_rb /)
      kbo(:,54, 3) = (/ &
        & 0.23501e-09_rb,0.23579e-09_rb,0.23460e-09_rb,0.23532e-09_rb,0.22777e-09_rb /)
      kbo(:,55, 3) = (/ &
        & 0.20561e-09_rb,0.21843e-09_rb,0.22171e-09_rb,0.22358e-09_rb,0.21702e-09_rb /)
      kbo(:,56, 3) = (/ &
        & 0.18557e-09_rb,0.20864e-09_rb,0.21196e-09_rb,0.20646e-09_rb,0.20759e-09_rb /)
      kbo(:,57, 3) = (/ &
        & 0.17099e-09_rb,0.19489e-09_rb,0.19486e-09_rb,0.19102e-09_rb,0.19124e-09_rb /)
      kbo(:,58, 3) = (/ &
        & 0.16535e-09_rb,0.16955e-09_rb,0.18769e-09_rb,0.18586e-09_rb,0.18358e-09_rb /)
      kbo(:,59, 3) = (/ &
        & 0.18472e-09_rb,0.17614e-09_rb,0.20835e-09_rb,0.19900e-09_rb,0.17587e-09_rb /)
      kbo(:,13, 4) = (/ &
        & 0.14276e-07_rb,0.14530e-07_rb,0.14762e-07_rb,0.14823e-07_rb,0.15070e-07_rb /)
      kbo(:,14, 4) = (/ &
        & 0.22710e-07_rb,0.23419e-07_rb,0.26577e-07_rb,0.28008e-07_rb,0.28298e-07_rb /)
      kbo(:,15, 4) = (/ &
        & 0.40218e-07_rb,0.40306e-07_rb,0.38786e-07_rb,0.38421e-07_rb,0.37982e-07_rb /)
      kbo(:,16, 4) = (/ &
        & 0.46010e-07_rb,0.46709e-07_rb,0.48658e-07_rb,0.49166e-07_rb,0.49018e-07_rb /)
      kbo(:,17, 4) = (/ &
        & 0.57896e-07_rb,0.59606e-07_rb,0.62582e-07_rb,0.65981e-07_rb,0.66467e-07_rb /)
      kbo(:,18, 4) = (/ &
        & 0.74731e-07_rb,0.75116e-07_rb,0.75181e-07_rb,0.74922e-07_rb,0.73535e-07_rb /)
      kbo(:,19, 4) = (/ &
        & 0.76222e-07_rb,0.73788e-07_rb,0.71345e-07_rb,0.70864e-07_rb,0.69839e-07_rb /)
      kbo(:,20, 4) = (/ &
        & 0.69638e-07_rb,0.67680e-07_rb,0.66694e-07_rb,0.67021e-07_rb,0.67066e-07_rb /)
      kbo(:,21, 4) = (/ &
        & 0.65779e-07_rb,0.64200e-07_rb,0.63508e-07_rb,0.64106e-07_rb,0.64461e-07_rb /)
      kbo(:,22, 4) = (/ &
        & 0.61488e-07_rb,0.60248e-07_rb,0.59719e-07_rb,0.60704e-07_rb,0.61392e-07_rb /)
      kbo(:,23, 4) = (/ &
        & 0.56738e-07_rb,0.55143e-07_rb,0.55071e-07_rb,0.56796e-07_rb,0.58031e-07_rb /)
      kbo(:,24, 4) = (/ &
        & 0.53299e-07_rb,0.52091e-07_rb,0.52308e-07_rb,0.54157e-07_rb,0.55094e-07_rb /)
      kbo(:,25, 4) = (/ &
        & 0.49763e-07_rb,0.48954e-07_rb,0.49068e-07_rb,0.51533e-07_rb,0.52367e-07_rb /)
      kbo(:,26, 4) = (/ &
        & 0.46006e-07_rb,0.45410e-07_rb,0.45860e-07_rb,0.48840e-07_rb,0.49788e-07_rb /)
      kbo(:,27, 4) = (/ &
        & 0.42208e-07_rb,0.41876e-07_rb,0.43006e-07_rb,0.46350e-07_rb,0.47032e-07_rb /)
      kbo(:,28, 4) = (/ &
        & 0.38508e-07_rb,0.38430e-07_rb,0.40481e-07_rb,0.43465e-07_rb,0.43170e-07_rb /)
      kbo(:,29, 4) = (/ &
        & 0.33913e-07_rb,0.34466e-07_rb,0.37202e-07_rb,0.38288e-07_rb,0.38327e-07_rb /)
      kbo(:,30, 4) = (/ &
        & 0.29454e-07_rb,0.30842e-07_rb,0.33204e-07_rb,0.33354e-07_rb,0.33727e-07_rb /)
      kbo(:,31, 4) = (/ &
        & 0.25253e-07_rb,0.27183e-07_rb,0.28228e-07_rb,0.27946e-07_rb,0.28495e-07_rb /)
      kbo(:,32, 4) = (/ &
        & 0.21784e-07_rb,0.23351e-07_rb,0.23372e-07_rb,0.23449e-07_rb,0.23246e-07_rb /)
      kbo(:,33, 4) = (/ &
        & 0.18910e-07_rb,0.19134e-07_rb,0.18984e-07_rb,0.19294e-07_rb,0.18955e-07_rb /)
      kbo(:,34, 4) = (/ &
        & 0.15933e-07_rb,0.15755e-07_rb,0.16059e-07_rb,0.15913e-07_rb,0.15386e-07_rb /)
      kbo(:,35, 4) = (/ &
        & 0.13356e-07_rb,0.13247e-07_rb,0.13429e-07_rb,0.13202e-07_rb,0.12675e-07_rb /)
      kbo(:,36, 4) = (/ &
        & 0.11207e-07_rb,0.11196e-07_rb,0.11048e-07_rb,0.10870e-07_rb,0.10431e-07_rb /)
      kbo(:,37, 4) = (/ &
        & 0.95811e-08_rb,0.94647e-08_rb,0.93875e-08_rb,0.90968e-08_rb,0.88553e-08_rb /)
      kbo(:,38, 4) = (/ &
        & 0.81243e-08_rb,0.80255e-08_rb,0.78559e-08_rb,0.75538e-08_rb,0.74057e-08_rb /)
      kbo(:,39, 4) = (/ &
        & 0.66871e-08_rb,0.66553e-08_rb,0.65003e-08_rb,0.62809e-08_rb,0.60862e-08_rb /)
      kbo(:,40, 4) = (/ &
        & 0.58254e-08_rb,0.57657e-08_rb,0.56185e-08_rb,0.54242e-08_rb,0.52135e-08_rb /)
      kbo(:,41, 4) = (/ &
        & 0.50967e-08_rb,0.50027e-08_rb,0.48304e-08_rb,0.46813e-08_rb,0.44864e-08_rb /)
      kbo(:,42, 4) = (/ &
        & 0.43731e-08_rb,0.43157e-08_rb,0.41432e-08_rb,0.39968e-08_rb,0.38854e-08_rb /)
      kbo(:,43, 4) = (/ &
        & 0.37430e-08_rb,0.37088e-08_rb,0.35491e-08_rb,0.34012e-08_rb,0.33022e-08_rb /)
      kbo(:,44, 4) = (/ &
        & 0.32207e-08_rb,0.31034e-08_rb,0.30690e-08_rb,0.29015e-08_rb,0.27644e-08_rb /)
      kbo(:,45, 4) = (/ &
        & 0.27111e-08_rb,0.25582e-08_rb,0.26092e-08_rb,0.24352e-08_rb,0.23234e-08_rb /)
      kbo(:,46, 4) = (/ &
        & 0.22675e-08_rb,0.21455e-08_rb,0.21930e-08_rb,0.20490e-08_rb,0.19277e-08_rb /)
      kbo(:,47, 4) = (/ &
        & 0.18722e-08_rb,0.18163e-08_rb,0.18222e-08_rb,0.17256e-08_rb,0.16025e-08_rb /)
      kbo(:,48, 4) = (/ &
        & 0.15670e-08_rb,0.15133e-08_rb,0.14751e-08_rb,0.14433e-08_rb,0.13269e-08_rb /)
      kbo(:,49, 4) = (/ &
        & 0.12576e-08_rb,0.12106e-08_rb,0.11779e-08_rb,0.11664e-08_rb,0.10722e-08_rb /)
      kbo(:,50, 4) = (/ &
        & 0.10308e-08_rb,0.10019e-08_rb,0.98151e-09_rb,0.96758e-09_rb,0.89739e-09_rb /)
      kbo(:,51, 4) = (/ &
        & 0.84050e-09_rb,0.85047e-09_rb,0.83131e-09_rb,0.81346e-09_rb,0.77303e-09_rb /)
      kbo(:,52, 4) = (/ &
        & 0.67669e-09_rb,0.70190e-09_rb,0.68695e-09_rb,0.66858e-09_rb,0.63719e-09_rb /)
      kbo(:,53, 4) = (/ &
        & 0.52755e-09_rb,0.56097e-09_rb,0.54716e-09_rb,0.53877e-09_rb,0.51047e-09_rb /)
      kbo(:,54, 4) = (/ &
        & 0.44864e-09_rb,0.48984e-09_rb,0.48831e-09_rb,0.46728e-09_rb,0.45462e-09_rb /)
      kbo(:,55, 4) = (/ &
        & 0.40938e-09_rb,0.45501e-09_rb,0.45212e-09_rb,0.43586e-09_rb,0.42245e-09_rb /)
      kbo(:,56, 4) = (/ &
        & 0.38293e-09_rb,0.40217e-09_rb,0.41760e-09_rb,0.39645e-09_rb,0.38652e-09_rb /)
      kbo(:,57, 4) = (/ &
        & 0.35808e-09_rb,0.34306e-09_rb,0.38440e-09_rb,0.37442e-09_rb,0.33472e-09_rb /)
      kbo(:,58, 4) = (/ &
        & 0.33278e-09_rb,0.31839e-09_rb,0.35327e-09_rb,0.33061e-09_rb,0.29093e-09_rb /)
      kbo(:,59, 4) = (/ &
        & 0.32589e-09_rb,0.30766e-09_rb,0.30185e-09_rb,0.35182e-09_rb,0.34474e-09_rb /)
      kbo(:,13, 5) = (/ &
        & 0.19090e-07_rb,0.19323e-07_rb,0.21491e-07_rb,0.23162e-07_rb,0.23970e-07_rb /)
      kbo(:,14, 5) = (/ &
        & 0.37888e-07_rb,0.37108e-07_rb,0.33727e-07_rb,0.34146e-07_rb,0.33930e-07_rb /)
      kbo(:,15, 5) = (/ &
        & 0.46395e-07_rb,0.47265e-07_rb,0.48201e-07_rb,0.50512e-07_rb,0.51101e-07_rb /)
      kbo(:,16, 5) = (/ &
        & 0.60743e-07_rb,0.64371e-07_rb,0.67862e-07_rb,0.71479e-07_rb,0.74200e-07_rb /)
      kbo(:,17, 5) = (/ &
        & 0.79052e-07_rb,0.79011e-07_rb,0.77558e-07_rb,0.75900e-07_rb,0.76450e-07_rb /)
      kbo(:,18, 5) = (/ &
        & 0.75325e-07_rb,0.74009e-07_rb,0.72866e-07_rb,0.72803e-07_rb,0.73869e-07_rb /)
      kbo(:,19, 5) = (/ &
        & 0.74009e-07_rb,0.74054e-07_rb,0.73970e-07_rb,0.73800e-07_rb,0.75638e-07_rb /)
      kbo(:,20, 5) = (/ &
        & 0.73077e-07_rb,0.73306e-07_rb,0.72248e-07_rb,0.73120e-07_rb,0.74630e-07_rb /)
      kbo(:,21, 5) = (/ &
        & 0.68928e-07_rb,0.70100e-07_rb,0.69203e-07_rb,0.69478e-07_rb,0.71860e-07_rb /)
      kbo(:,22, 5) = (/ &
        & 0.65324e-07_rb,0.66570e-07_rb,0.66457e-07_rb,0.66345e-07_rb,0.68314e-07_rb /)
      kbo(:,23, 5) = (/ &
        & 0.62411e-07_rb,0.63996e-07_rb,0.63443e-07_rb,0.63576e-07_rb,0.65766e-07_rb /)
      kbo(:,24, 5) = (/ &
        & 0.60058e-07_rb,0.60956e-07_rb,0.60176e-07_rb,0.60222e-07_rb,0.63287e-07_rb /)
      kbo(:,25, 5) = (/ &
        & 0.57223e-07_rb,0.58225e-07_rb,0.58348e-07_rb,0.58584e-07_rb,0.61933e-07_rb /)
      kbo(:,26, 5) = (/ &
        & 0.55089e-07_rb,0.55723e-07_rb,0.55839e-07_rb,0.56337e-07_rb,0.60335e-07_rb /)
      kbo(:,27, 5) = (/ &
        & 0.52294e-07_rb,0.51745e-07_rb,0.52750e-07_rb,0.52701e-07_rb,0.57676e-07_rb /)
      kbo(:,28, 5) = (/ &
        & 0.48915e-07_rb,0.48234e-07_rb,0.48708e-07_rb,0.49873e-07_rb,0.54442e-07_rb /)
      kbo(:,29, 5) = (/ &
        & 0.43357e-07_rb,0.43230e-07_rb,0.43679e-07_rb,0.47285e-07_rb,0.49519e-07_rb /)
      kbo(:,30, 5) = (/ &
        & 0.37379e-07_rb,0.38427e-07_rb,0.39703e-07_rb,0.43492e-07_rb,0.43278e-07_rb /)
      kbo(:,31, 5) = (/ &
        & 0.31354e-07_rb,0.32238e-07_rb,0.35376e-07_rb,0.37358e-07_rb,0.37130e-07_rb /)
      kbo(:,32, 5) = (/ &
        & 0.26553e-07_rb,0.27801e-07_rb,0.30978e-07_rb,0.30698e-07_rb,0.31234e-07_rb /)
      kbo(:,33, 5) = (/ &
        & 0.21542e-07_rb,0.24274e-07_rb,0.24944e-07_rb,0.25194e-07_rb,0.25415e-07_rb /)
      kbo(:,34, 5) = (/ &
        & 0.19097e-07_rb,0.20582e-07_rb,0.20324e-07_rb,0.21006e-07_rb,0.20944e-07_rb /)
      kbo(:,35, 5) = (/ &
        & 0.16377e-07_rb,0.16881e-07_rb,0.16957e-07_rb,0.16743e-07_rb,0.16958e-07_rb /)
      kbo(:,36, 5) = (/ &
        & 0.13727e-07_rb,0.13834e-07_rb,0.13766e-07_rb,0.13776e-07_rb,0.13546e-07_rb /)
      kbo(:,37, 5) = (/ &
        & 0.11550e-07_rb,0.11601e-07_rb,0.11757e-07_rb,0.11723e-07_rb,0.11413e-07_rb /)
      kbo(:,38, 5) = (/ &
        & 0.98076e-08_rb,0.96320e-08_rb,0.10028e-07_rb,0.98197e-08_rb,0.91408e-08_rb /)
      kbo(:,39, 5) = (/ &
        & 0.80861e-08_rb,0.80113e-08_rb,0.81586e-08_rb,0.80121e-08_rb,0.75021e-08_rb /)
      kbo(:,40, 5) = (/ &
        & 0.70348e-08_rb,0.69980e-08_rb,0.71058e-08_rb,0.69171e-08_rb,0.64103e-08_rb /)
      kbo(:,41, 5) = (/ &
        & 0.61395e-08_rb,0.60379e-08_rb,0.61651e-08_rb,0.59006e-08_rb,0.54433e-08_rb /)
      kbo(:,42, 5) = (/ &
        & 0.51740e-08_rb,0.51391e-08_rb,0.53717e-08_rb,0.50506e-08_rb,0.46901e-08_rb /)
      kbo(:,43, 5) = (/ &
        & 0.44374e-08_rb,0.44002e-08_rb,0.45405e-08_rb,0.42888e-08_rb,0.39390e-08_rb /)
      kbo(:,44, 5) = (/ &
        & 0.36635e-08_rb,0.37555e-08_rb,0.37320e-08_rb,0.35598e-08_rb,0.33280e-08_rb /)
      kbo(:,45, 5) = (/ &
        & 0.30833e-08_rb,0.31320e-08_rb,0.30264e-08_rb,0.29827e-08_rb,0.28298e-08_rb /)
      kbo(:,46, 5) = (/ &
        & 0.25366e-08_rb,0.26077e-08_rb,0.24905e-08_rb,0.24281e-08_rb,0.22727e-08_rb /)
      kbo(:,47, 5) = (/ &
        & 0.20576e-08_rb,0.22029e-08_rb,0.20905e-08_rb,0.21040e-08_rb,0.19027e-08_rb /)
      kbo(:,48, 5) = (/ &
        & 0.16047e-08_rb,0.18190e-08_rb,0.17008e-08_rb,0.17298e-08_rb,0.16059e-08_rb /)
      kbo(:,49, 5) = (/ &
        & 0.12276e-08_rb,0.14135e-08_rb,0.13565e-08_rb,0.13704e-08_rb,0.13029e-08_rb /)
      kbo(:,50, 5) = (/ &
        & 0.98291e-09_rb,0.11837e-08_rb,0.11000e-08_rb,0.11342e-08_rb,0.10621e-08_rb /)
      kbo(:,51, 5) = (/ &
        & 0.79374e-09_rb,0.97756e-09_rb,0.90252e-09_rb,0.90527e-09_rb,0.86815e-09_rb /)
      kbo(:,52, 5) = (/ &
        & 0.64781e-09_rb,0.78454e-09_rb,0.75690e-09_rb,0.74124e-09_rb,0.73195e-09_rb /)
      kbo(:,53, 5) = (/ &
        & 0.51536e-09_rb,0.61158e-09_rb,0.60936e-09_rb,0.59007e-09_rb,0.58458e-09_rb /)
      kbo(:,54, 5) = (/ &
        & 0.46178e-09_rb,0.51944e-09_rb,0.52167e-09_rb,0.53965e-09_rb,0.50524e-09_rb /)
      kbo(:,55, 5) = (/ &
        & 0.44467e-09_rb,0.42857e-09_rb,0.50799e-09_rb,0.47528e-09_rb,0.43900e-09_rb /)
      kbo(:,56, 5) = (/ &
        & 0.39667e-09_rb,0.37862e-09_rb,0.45808e-09_rb,0.43270e-09_rb,0.38817e-09_rb /)
      kbo(:,57, 5) = (/ &
        & 0.35926e-09_rb,0.35410e-09_rb,0.38962e-09_rb,0.37502e-09_rb,0.40935e-09_rb /)
      kbo(:,58, 5) = (/ &
        & 0.31517e-09_rb,0.32964e-09_rb,0.29323e-09_rb,0.39945e-09_rb,0.40389e-09_rb /)
      kbo(:,59, 5) = (/ &
        & 0.36198e-09_rb,0.38633e-09_rb,0.41096e-09_rb,0.53914e-09_rb,0.57631e-09_rb /)
      kbo(:,13, 6) = (/ &
        & 0.23095e-07_rb,0.23025e-07_rb,0.25126e-07_rb,0.25618e-07_rb,0.24969e-07_rb /)
      kbo(:,14, 6) = (/ &
        & 0.31573e-07_rb,0.32360e-07_rb,0.33220e-07_rb,0.32611e-07_rb,0.32150e-07_rb /)
      kbo(:,15, 6) = (/ &
        & 0.45219e-07_rb,0.45787e-07_rb,0.47088e-07_rb,0.49543e-07_rb,0.54072e-07_rb /)
      kbo(:,16, 6) = (/ &
        & 0.65293e-07_rb,0.67448e-07_rb,0.64557e-07_rb,0.63976e-07_rb,0.64905e-07_rb /)
      kbo(:,17, 6) = (/ &
        & 0.64862e-07_rb,0.63259e-07_rb,0.62851e-07_rb,0.62500e-07_rb,0.63112e-07_rb /)
      kbo(:,18, 6) = (/ &
        & 0.61849e-07_rb,0.62108e-07_rb,0.60565e-07_rb,0.60920e-07_rb,0.61810e-07_rb /)
      kbo(:,19, 6) = (/ &
        & 0.63930e-07_rb,0.63195e-07_rb,0.62478e-07_rb,0.62282e-07_rb,0.62250e-07_rb /)
      kbo(:,20, 6) = (/ &
        & 0.60100e-07_rb,0.60269e-07_rb,0.59919e-07_rb,0.57578e-07_rb,0.57933e-07_rb /)
      kbo(:,21, 6) = (/ &
        & 0.56465e-07_rb,0.56674e-07_rb,0.56992e-07_rb,0.55969e-07_rb,0.54719e-07_rb /)
      kbo(:,22, 6) = (/ &
        & 0.54280e-07_rb,0.53760e-07_rb,0.52615e-07_rb,0.52261e-07_rb,0.51523e-07_rb /)
      kbo(:,23, 6) = (/ &
        & 0.49921e-07_rb,0.49472e-07_rb,0.49168e-07_rb,0.47959e-07_rb,0.47449e-07_rb /)
      kbo(:,24, 6) = (/ &
        & 0.45803e-07_rb,0.46271e-07_rb,0.46322e-07_rb,0.46536e-07_rb,0.46494e-07_rb /)
      kbo(:,25, 6) = (/ &
        & 0.43723e-07_rb,0.44225e-07_rb,0.44324e-07_rb,0.43726e-07_rb,0.43579e-07_rb /)
      kbo(:,26, 6) = (/ &
        & 0.41617e-07_rb,0.42588e-07_rb,0.42306e-07_rb,0.41086e-07_rb,0.41943e-07_rb /)
      kbo(:,27, 6) = (/ &
        & 0.38855e-07_rb,0.40231e-07_rb,0.39090e-07_rb,0.39461e-07_rb,0.39072e-07_rb /)
      kbo(:,28, 6) = (/ &
        & 0.35912e-07_rb,0.36691e-07_rb,0.35670e-07_rb,0.35644e-07_rb,0.38295e-07_rb /)
      kbo(:,29, 6) = (/ &
        & 0.33357e-07_rb,0.33134e-07_rb,0.32394e-07_rb,0.32909e-07_rb,0.36234e-07_rb /)
      kbo(:,30, 6) = (/ &
        & 0.30529e-07_rb,0.29196e-07_rb,0.29147e-07_rb,0.30292e-07_rb,0.33912e-07_rb /)
      kbo(:,31, 6) = (/ &
        & 0.27228e-07_rb,0.27081e-07_rb,0.26630e-07_rb,0.30010e-07_rb,0.28781e-07_rb /)
      kbo(:,32, 6) = (/ &
        & 0.24109e-07_rb,0.24388e-07_rb,0.25546e-07_rb,0.26823e-07_rb,0.26681e-07_rb /)
      kbo(:,33, 6) = (/ &
        & 0.21402e-07_rb,0.22303e-07_rb,0.24668e-07_rb,0.24556e-07_rb,0.24369e-07_rb /)
      kbo(:,34, 6) = (/ &
        & 0.18136e-07_rb,0.20944e-07_rb,0.21211e-07_rb,0.21286e-07_rb,0.21648e-07_rb /)
      kbo(:,35, 6) = (/ &
        & 0.16306e-07_rb,0.17531e-07_rb,0.17744e-07_rb,0.18061e-07_rb,0.18282e-07_rb /)
      kbo(:,36, 6) = (/ &
        & 0.13873e-07_rb,0.14963e-07_rb,0.15398e-07_rb,0.15527e-07_rb,0.15391e-07_rb /)
      kbo(:,37, 6) = (/ &
        & 0.11948e-07_rb,0.12614e-07_rb,0.12946e-07_rb,0.12916e-07_rb,0.13085e-07_rb /)
      kbo(:,38, 6) = (/ &
        & 0.98235e-08_rb,0.10612e-07_rb,0.10029e-07_rb,0.10618e-07_rb,0.10860e-07_rb /)
      kbo(:,39, 6) = (/ &
        & 0.81514e-08_rb,0.84716e-08_rb,0.81123e-08_rb,0.83644e-08_rb,0.81668e-08_rb /)
      kbo(:,40, 6) = (/ &
        & 0.68179e-08_rb,0.69030e-08_rb,0.69393e-08_rb,0.69969e-08_rb,0.71027e-08_rb /)
      kbo(:,41, 6) = (/ &
        & 0.59350e-08_rb,0.60668e-08_rb,0.59026e-08_rb,0.60181e-08_rb,0.60779e-08_rb /)
      kbo(:,42, 6) = (/ &
        & 0.51181e-08_rb,0.52127e-08_rb,0.48788e-08_rb,0.51397e-08_rb,0.50230e-08_rb /)
      kbo(:,43, 6) = (/ &
        & 0.42653e-08_rb,0.44600e-08_rb,0.41100e-08_rb,0.44735e-08_rb,0.42607e-08_rb /)
      kbo(:,44, 6) = (/ &
        & 0.34724e-08_rb,0.37628e-08_rb,0.35340e-08_rb,0.38309e-08_rb,0.36188e-08_rb /)
      kbo(:,45, 6) = (/ &
        & 0.27994e-08_rb,0.31690e-08_rb,0.28599e-08_rb,0.30290e-08_rb,0.27730e-08_rb /)
      kbo(:,46, 6) = (/ &
        & 0.22089e-08_rb,0.25761e-08_rb,0.23856e-08_rb,0.24874e-08_rb,0.23603e-08_rb /)
      kbo(:,47, 6) = (/ &
        & 0.18078e-08_rb,0.20304e-08_rb,0.19457e-08_rb,0.19366e-08_rb,0.19286e-08_rb /)
      kbo(:,48, 6) = (/ &
        & 0.14716e-08_rb,0.15703e-08_rb,0.17393e-08_rb,0.14836e-08_rb,0.15316e-08_rb /)
      kbo(:,49, 6) = (/ &
        & 0.11334e-08_rb,0.12901e-08_rb,0.13556e-08_rb,0.11610e-08_rb,0.11617e-08_rb /)
      kbo(:,50, 6) = (/ &
        & 0.90881e-09_rb,0.10323e-08_rb,0.11378e-08_rb,0.95509e-09_rb,0.96291e-09_rb /)
      kbo(:,51, 6) = (/ &
        & 0.76751e-09_rb,0.81710e-09_rb,0.97432e-09_rb,0.82408e-09_rb,0.81205e-09_rb /)
      kbo(:,52, 6) = (/ &
        & 0.62668e-09_rb,0.66223e-09_rb,0.76870e-09_rb,0.68347e-09_rb,0.65771e-09_rb /)
      kbo(:,53, 6) = (/ &
        & 0.50547e-09_rb,0.49991e-09_rb,0.61494e-09_rb,0.55640e-09_rb,0.52803e-09_rb /)
      kbo(:,54, 6) = (/ &
        & 0.42841e-09_rb,0.42678e-09_rb,0.51796e-09_rb,0.46340e-09_rb,0.44830e-09_rb /)
      kbo(:,55, 6) = (/ &
        & 0.40234e-09_rb,0.40065e-09_rb,0.45944e-09_rb,0.42252e-09_rb,0.44365e-09_rb /)
      kbo(:,56, 6) = (/ &
        & 0.36764e-09_rb,0.35937e-09_rb,0.36570e-09_rb,0.44631e-09_rb,0.45684e-09_rb /)
      kbo(:,57, 6) = (/ &
        & 0.33321e-09_rb,0.29808e-09_rb,0.31056e-09_rb,0.44585e-09_rb,0.41122e-09_rb /)
      kbo(:,58, 6) = (/ &
        & 0.31198e-09_rb,0.29542e-09_rb,0.33327e-09_rb,0.43871e-09_rb,0.45291e-09_rb /)
      kbo(:,59, 6) = (/ &
        & 0.38710e-09_rb,0.42412e-09_rb,0.44197e-09_rb,0.49964e-09_rb,0.70387e-09_rb /)
      kbo(:,13, 7) = (/ &
        & 0.30377e-07_rb,0.31586e-07_rb,0.27647e-07_rb,0.27937e-07_rb,0.28036e-07_rb /)
      kbo(:,14, 7) = (/ &
        & 0.37766e-07_rb,0.39819e-07_rb,0.39822e-07_rb,0.40069e-07_rb,0.42091e-07_rb /)
      kbo(:,15, 7) = (/ &
        & 0.57996e-07_rb,0.64474e-07_rb,0.69368e-07_rb,0.70909e-07_rb,0.70748e-07_rb /)
      kbo(:,16, 7) = (/ &
        & 0.73220e-07_rb,0.70124e-07_rb,0.69135e-07_rb,0.68058e-07_rb,0.66524e-07_rb /)
      kbo(:,17, 7) = (/ &
        & 0.66529e-07_rb,0.66306e-07_rb,0.63797e-07_rb,0.63551e-07_rb,0.63462e-07_rb /)
      kbo(:,18, 7) = (/ &
        & 0.66492e-07_rb,0.65156e-07_rb,0.66323e-07_rb,0.64485e-07_rb,0.63285e-07_rb /)
      kbo(:,19, 7) = (/ &
        & 0.62980e-07_rb,0.62900e-07_rb,0.62718e-07_rb,0.61114e-07_rb,0.59765e-07_rb /)
      kbo(:,20, 7) = (/ &
        & 0.59754e-07_rb,0.58066e-07_rb,0.59540e-07_rb,0.58926e-07_rb,0.57835e-07_rb /)
      kbo(:,21, 7) = (/ &
        & 0.57368e-07_rb,0.55161e-07_rb,0.55685e-07_rb,0.55360e-07_rb,0.53982e-07_rb /)
      kbo(:,22, 7) = (/ &
        & 0.51435e-07_rb,0.49998e-07_rb,0.50952e-07_rb,0.50442e-07_rb,0.49717e-07_rb /)
      kbo(:,23, 7) = (/ &
        & 0.47663e-07_rb,0.46845e-07_rb,0.47849e-07_rb,0.48876e-07_rb,0.48885e-07_rb /)
      kbo(:,24, 7) = (/ &
        & 0.47926e-07_rb,0.45619e-07_rb,0.45793e-07_rb,0.46467e-07_rb,0.45938e-07_rb /)
      kbo(:,25, 7) = (/ &
        & 0.44687e-07_rb,0.43904e-07_rb,0.44284e-07_rb,0.44176e-07_rb,0.45520e-07_rb /)
      kbo(:,26, 7) = (/ &
        & 0.42751e-07_rb,0.41938e-07_rb,0.42889e-07_rb,0.42927e-07_rb,0.43868e-07_rb /)
      kbo(:,27, 7) = (/ &
        & 0.40180e-07_rb,0.40529e-07_rb,0.40621e-07_rb,0.40949e-07_rb,0.41005e-07_rb /)
      kbo(:,28, 7) = (/ &
        & 0.35890e-07_rb,0.36716e-07_rb,0.37875e-07_rb,0.38308e-07_rb,0.37626e-07_rb /)
      kbo(:,29, 7) = (/ &
        & 0.31499e-07_rb,0.33284e-07_rb,0.32725e-07_rb,0.32493e-07_rb,0.35018e-07_rb /)
      kbo(:,30, 7) = (/ &
        & 0.29364e-07_rb,0.28675e-07_rb,0.28942e-07_rb,0.28761e-07_rb,0.33960e-07_rb /)
      kbo(:,31, 7) = (/ &
        & 0.25558e-07_rb,0.23679e-07_rb,0.24292e-07_rb,0.26053e-07_rb,0.29211e-07_rb /)
      kbo(:,32, 7) = (/ &
        & 0.20860e-07_rb,0.21491e-07_rb,0.20910e-07_rb,0.25598e-07_rb,0.24209e-07_rb /)
      kbo(:,33, 7) = (/ &
        & 0.18874e-07_rb,0.18554e-07_rb,0.21206e-07_rb,0.20708e-07_rb,0.21100e-07_rb /)
      kbo(:,34, 7) = (/ &
        & 0.18632e-07_rb,0.18625e-07_rb,0.20707e-07_rb,0.19570e-07_rb,0.18956e-07_rb /)
      kbo(:,35, 7) = (/ &
        & 0.15958e-07_rb,0.19141e-07_rb,0.19004e-07_rb,0.19249e-07_rb,0.17793e-07_rb /)
      kbo(:,36, 7) = (/ &
        & 0.14552e-07_rb,0.16561e-07_rb,0.16607e-07_rb,0.16784e-07_rb,0.16326e-07_rb /)
      kbo(:,37, 7) = (/ &
        & 0.13127e-07_rb,0.14300e-07_rb,0.14021e-07_rb,0.14385e-07_rb,0.13328e-07_rb /)
      kbo(:,38, 7) = (/ &
        & 0.11514e-07_rb,0.12356e-07_rb,0.12677e-07_rb,0.12181e-07_rb,0.11774e-07_rb /)
      kbo(:,39, 7) = (/ &
        & 0.93434e-08_rb,0.10196e-07_rb,0.10605e-07_rb,0.10201e-07_rb,0.10584e-07_rb /)
      kbo(:,40, 7) = (/ &
        & 0.81790e-08_rb,0.89000e-08_rb,0.85315e-08_rb,0.89663e-08_rb,0.91635e-08_rb /)
      kbo(:,41, 7) = (/ &
        & 0.65541e-08_rb,0.74884e-08_rb,0.72364e-08_rb,0.72692e-08_rb,0.75649e-08_rb /)
      kbo(:,42, 7) = (/ &
        & 0.54665e-08_rb,0.61796e-08_rb,0.61564e-08_rb,0.61304e-08_rb,0.64285e-08_rb /)
      kbo(:,43, 7) = (/ &
        & 0.43531e-08_rb,0.47798e-08_rb,0.51393e-08_rb,0.49135e-08_rb,0.53520e-08_rb /)
      kbo(:,44, 7) = (/ &
        & 0.35872e-08_rb,0.38233e-08_rb,0.42659e-08_rb,0.39384e-08_rb,0.44541e-08_rb /)
      kbo(:,45, 7) = (/ &
        & 0.28156e-08_rb,0.32775e-08_rb,0.36262e-08_rb,0.32518e-08_rb,0.36805e-08_rb /)
      kbo(:,46, 7) = (/ &
        & 0.23879e-08_rb,0.26299e-08_rb,0.28972e-08_rb,0.27379e-08_rb,0.30333e-08_rb /)
      kbo(:,47, 7) = (/ &
        & 0.19282e-08_rb,0.22066e-08_rb,0.23812e-08_rb,0.23512e-08_rb,0.23794e-08_rb /)
      kbo(:,48, 7) = (/ &
        & 0.15092e-08_rb,0.18255e-08_rb,0.17414e-08_rb,0.19322e-08_rb,0.19399e-08_rb /)
      kbo(:,49, 7) = (/ &
        & 0.11905e-08_rb,0.13964e-08_rb,0.14306e-08_rb,0.14791e-08_rb,0.14939e-08_rb /)
      kbo(:,50, 7) = (/ &
        & 0.98537e-09_rb,0.10687e-08_rb,0.11753e-08_rb,0.11334e-08_rb,0.13178e-08_rb /)
      kbo(:,51, 7) = (/ &
        & 0.84255e-09_rb,0.88069e-09_rb,0.95073e-09_rb,0.92632e-09_rb,0.10457e-08_rb /)
      kbo(:,52, 7) = (/ &
        & 0.68238e-09_rb,0.70376e-09_rb,0.74795e-09_rb,0.75152e-09_rb,0.78131e-09_rb /)
      kbo(:,53, 7) = (/ &
        & 0.56271e-09_rb,0.58527e-09_rb,0.61397e-09_rb,0.56343e-09_rb,0.61138e-09_rb /)
      kbo(:,54, 7) = (/ &
        & 0.52493e-09_rb,0.46040e-09_rb,0.53769e-09_rb,0.51459e-09_rb,0.52912e-09_rb /)
      kbo(:,55, 7) = (/ &
        & 0.40372e-09_rb,0.38720e-09_rb,0.40004e-09_rb,0.56294e-09_rb,0.55844e-09_rb /)
      kbo(:,56, 7) = (/ &
        & 0.37372e-09_rb,0.35358e-09_rb,0.39920e-09_rb,0.52819e-09_rb,0.50147e-09_rb /)
      kbo(:,57, 7) = (/ &
        & 0.31149e-09_rb,0.39454e-09_rb,0.38951e-09_rb,0.52086e-09_rb,0.54542e-09_rb /)
      kbo(:,58, 7) = (/ &
        & 0.30564e-09_rb,0.36251e-09_rb,0.38551e-09_rb,0.43845e-09_rb,0.61772e-09_rb /)
      kbo(:,59, 7) = (/ &
        & 0.46934e-09_rb,0.57639e-09_rb,0.58591e-09_rb,0.63021e-09_rb,0.71158e-09_rb /)
      kbo(:,13, 8) = (/ &
        & 0.34320e-07_rb,0.31607e-07_rb,0.29840e-07_rb,0.29114e-07_rb,0.28831e-07_rb /)
      kbo(:,14, 8) = (/ &
        & 0.38157e-07_rb,0.37907e-07_rb,0.40351e-07_rb,0.45015e-07_rb,0.44786e-07_rb /)
      kbo(:,15, 8) = (/ &
        & 0.71231e-07_rb,0.71647e-07_rb,0.71034e-07_rb,0.71647e-07_rb,0.71755e-07_rb /)
      kbo(:,16, 8) = (/ &
        & 0.72049e-07_rb,0.69075e-07_rb,0.67707e-07_rb,0.67019e-07_rb,0.66780e-07_rb /)
      kbo(:,17, 8) = (/ &
        & 0.69751e-07_rb,0.66211e-07_rb,0.66017e-07_rb,0.63095e-07_rb,0.63343e-07_rb /)
      kbo(:,18, 8) = (/ &
        & 0.67728e-07_rb,0.66581e-07_rb,0.64787e-07_rb,0.62981e-07_rb,0.62245e-07_rb /)
      kbo(:,19, 8) = (/ &
        & 0.65864e-07_rb,0.63789e-07_rb,0.62712e-07_rb,0.62065e-07_rb,0.61005e-07_rb /)
      kbo(:,20, 8) = (/ &
        & 0.61816e-07_rb,0.63755e-07_rb,0.63424e-07_rb,0.61945e-07_rb,0.58720e-07_rb /)
      kbo(:,21, 8) = (/ &
        & 0.59112e-07_rb,0.58061e-07_rb,0.57019e-07_rb,0.54795e-07_rb,0.56069e-07_rb /)
      kbo(:,22, 8) = (/ &
        & 0.56217e-07_rb,0.53887e-07_rb,0.54812e-07_rb,0.52500e-07_rb,0.53303e-07_rb /)
      kbo(:,23, 8) = (/ &
        & 0.54428e-07_rb,0.52930e-07_rb,0.53278e-07_rb,0.52797e-07_rb,0.51840e-07_rb /)
      kbo(:,24, 8) = (/ &
        & 0.51236e-07_rb,0.52532e-07_rb,0.53348e-07_rb,0.51715e-07_rb,0.53224e-07_rb /)
      kbo(:,25, 8) = (/ &
        & 0.49989e-07_rb,0.50872e-07_rb,0.49932e-07_rb,0.52702e-07_rb,0.51493e-07_rb /)
      kbo(:,26, 8) = (/ &
        & 0.49845e-07_rb,0.48733e-07_rb,0.48348e-07_rb,0.51346e-07_rb,0.51109e-07_rb /)
      kbo(:,27, 8) = (/ &
        & 0.43883e-07_rb,0.43853e-07_rb,0.46523e-07_rb,0.47281e-07_rb,0.47292e-07_rb /)
      kbo(:,28, 8) = (/ &
        & 0.41424e-07_rb,0.42057e-07_rb,0.43773e-07_rb,0.43422e-07_rb,0.41438e-07_rb /)
      kbo(:,29, 8) = (/ &
        & 0.37736e-07_rb,0.34915e-07_rb,0.37971e-07_rb,0.38437e-07_rb,0.38122e-07_rb /)
      kbo(:,30, 8) = (/ &
        & 0.30209e-07_rb,0.32936e-07_rb,0.31853e-07_rb,0.30285e-07_rb,0.31746e-07_rb /)
      kbo(:,31, 8) = (/ &
        & 0.25886e-07_rb,0.26990e-07_rb,0.27039e-07_rb,0.26019e-07_rb,0.34246e-07_rb /)
      kbo(:,32, 8) = (/ &
        & 0.24750e-07_rb,0.20653e-07_rb,0.21921e-07_rb,0.25087e-07_rb,0.25874e-07_rb /)
      kbo(:,33, 8) = (/ &
        & 0.19410e-07_rb,0.18820e-07_rb,0.18768e-07_rb,0.23223e-07_rb,0.20942e-07_rb /)
      kbo(:,34, 8) = (/ &
        & 0.16037e-07_rb,0.16060e-07_rb,0.19012e-07_rb,0.17686e-07_rb,0.18319e-07_rb /)
      kbo(:,35, 8) = (/ &
        & 0.16085e-07_rb,0.15072e-07_rb,0.16472e-07_rb,0.15719e-07_rb,0.16406e-07_rb /)
      kbo(:,36, 8) = (/ &
        & 0.15014e-07_rb,0.17144e-07_rb,0.16353e-07_rb,0.15223e-07_rb,0.14884e-07_rb /)
      kbo(:,37, 8) = (/ &
        & 0.12909e-07_rb,0.15628e-07_rb,0.16086e-07_rb,0.15318e-07_rb,0.15605e-07_rb /)
      kbo(:,38, 8) = (/ &
        & 0.11803e-07_rb,0.12970e-07_rb,0.13587e-07_rb,0.13257e-07_rb,0.13920e-07_rb /)
      kbo(:,39, 8) = (/ &
        & 0.10299e-07_rb,0.10951e-07_rb,0.11006e-07_rb,0.11507e-07_rb,0.10553e-07_rb /)
      kbo(:,40, 8) = (/ &
        & 0.87605e-08_rb,0.10094e-07_rb,0.10641e-07_rb,0.94199e-08_rb,0.97115e-08_rb /)
      kbo(:,41, 8) = (/ &
        & 0.69747e-08_rb,0.82105e-08_rb,0.93516e-08_rb,0.90234e-08_rb,0.91657e-08_rb /)
      kbo(:,42, 8) = (/ &
        & 0.60201e-08_rb,0.73523e-08_rb,0.76249e-08_rb,0.76962e-08_rb,0.80489e-08_rb /)
      kbo(:,43, 8) = (/ &
        & 0.50249e-08_rb,0.64035e-08_rb,0.62997e-08_rb,0.63745e-08_rb,0.65773e-08_rb /)
      kbo(:,44, 8) = (/ &
        & 0.43021e-08_rb,0.55682e-08_rb,0.52391e-08_rb,0.55432e-08_rb,0.53417e-08_rb /)
      kbo(:,45, 8) = (/ &
        & 0.34659e-08_rb,0.41011e-08_rb,0.41998e-08_rb,0.45328e-08_rb,0.45452e-08_rb /)
      kbo(:,46, 8) = (/ &
        & 0.26629e-08_rb,0.30573e-08_rb,0.34050e-08_rb,0.36018e-08_rb,0.38146e-08_rb /)
      kbo(:,47, 8) = (/ &
        & 0.20352e-08_rb,0.25120e-08_rb,0.29668e-08_rb,0.27818e-08_rb,0.34015e-08_rb /)
      kbo(:,48, 8) = (/ &
        & 0.16537e-08_rb,0.17568e-08_rb,0.23894e-08_rb,0.22154e-08_rb,0.25802e-08_rb /)
      kbo(:,49, 8) = (/ &
        & 0.13981e-08_rb,0.14349e-08_rb,0.17246e-08_rb,0.17767e-08_rb,0.19090e-08_rb /)
      kbo(:,50, 8) = (/ &
        & 0.10918e-08_rb,0.11356e-08_rb,0.12258e-08_rb,0.14996e-08_rb,0.12493e-08_rb /)
      kbo(:,51, 8) = (/ &
        & 0.96204e-09_rb,0.88786e-09_rb,0.10061e-08_rb,0.12415e-08_rb,0.10774e-08_rb /)
      kbo(:,52, 8) = (/ &
        & 0.79848e-09_rb,0.70289e-09_rb,0.84515e-09_rb,0.10143e-08_rb,0.85552e-09_rb /)
      kbo(:,53, 8) = (/ &
        & 0.60435e-09_rb,0.54746e-09_rb,0.63806e-09_rb,0.74615e-09_rb,0.64507e-09_rb /)
      kbo(:,54, 8) = (/ &
        & 0.47378e-09_rb,0.48059e-09_rb,0.54886e-09_rb,0.73215e-09_rb,0.65721e-09_rb /)
      kbo(:,55, 8) = (/ &
        & 0.45346e-09_rb,0.46291e-09_rb,0.55380e-09_rb,0.67829e-09_rb,0.56756e-09_rb /)
      kbo(:,56, 8) = (/ &
        & 0.43938e-09_rb,0.46043e-09_rb,0.50980e-09_rb,0.62685e-09_rb,0.70094e-09_rb /)
      kbo(:,57, 8) = (/ &
        & 0.42587e-09_rb,0.43184e-09_rb,0.47819e-09_rb,0.57152e-09_rb,0.76528e-09_rb /)
      kbo(:,58, 8) = (/ &
        & 0.47403e-09_rb,0.46387e-09_rb,0.50904e-09_rb,0.58886e-09_rb,0.70162e-09_rb /)
      kbo(:,59, 8) = (/ &
        & 0.54448e-09_rb,0.50172e-09_rb,0.65155e-09_rb,0.65748e-09_rb,0.98321e-09_rb /)
      kbo(:,13, 9) = (/ &
        & 0.10995e-05_rb,0.11095e-05_rb,0.11168e-05_rb,0.11200e-05_rb,0.11274e-05_rb /)
      kbo(:,14, 9) = (/ &
        & 0.73340e-06_rb,0.73567e-06_rb,0.73740e-06_rb,0.73330e-06_rb,0.73919e-06_rb /)
      kbo(:,15, 9) = (/ &
        & 0.32510e-06_rb,0.31252e-06_rb,0.30243e-06_rb,0.29146e-06_rb,0.28830e-06_rb /)
      kbo(:,16, 9) = (/ &
        & 0.61567e-07_rb,0.60590e-07_rb,0.61057e-07_rb,0.58821e-07_rb,0.58536e-07_rb /)
      kbo(:,17, 9) = (/ &
        & 0.60009e-07_rb,0.60924e-07_rb,0.61205e-07_rb,0.63591e-07_rb,0.59950e-07_rb /)
      kbo(:,18, 9) = (/ &
        & 0.61458e-07_rb,0.61692e-07_rb,0.60339e-07_rb,0.59878e-07_rb,0.58575e-07_rb /)
      kbo(:,19, 9) = (/ &
        & 0.57878e-07_rb,0.60299e-07_rb,0.60236e-07_rb,0.59512e-07_rb,0.58126e-07_rb /)
      kbo(:,20, 9) = (/ &
        & 0.58329e-07_rb,0.57451e-07_rb,0.54760e-07_rb,0.57082e-07_rb,0.56805e-07_rb /)
      kbo(:,21, 9) = (/ &
        & 0.53092e-07_rb,0.51700e-07_rb,0.51833e-07_rb,0.52853e-07_rb,0.52467e-07_rb /)
      kbo(:,22, 9) = (/ &
        & 0.47591e-07_rb,0.51357e-07_rb,0.49075e-07_rb,0.53548e-07_rb,0.55924e-07_rb /)
      kbo(:,23, 9) = (/ &
        & 0.49724e-07_rb,0.51803e-07_rb,0.51793e-07_rb,0.49922e-07_rb,0.52665e-07_rb /)
      kbo(:,24, 9) = (/ &
        & 0.47737e-07_rb,0.49279e-07_rb,0.51207e-07_rb,0.49000e-07_rb,0.51613e-07_rb /)
      kbo(:,25, 9) = (/ &
        & 0.49014e-07_rb,0.50174e-07_rb,0.54886e-07_rb,0.47520e-07_rb,0.49798e-07_rb /)
      kbo(:,26, 9) = (/ &
        & 0.42783e-07_rb,0.51200e-07_rb,0.50600e-07_rb,0.47620e-07_rb,0.43709e-07_rb /)
      kbo(:,27, 9) = (/ &
        & 0.42685e-07_rb,0.50611e-07_rb,0.42800e-07_rb,0.43026e-07_rb,0.46518e-07_rb /)
      kbo(:,28, 9) = (/ &
        & 0.40023e-07_rb,0.44433e-07_rb,0.38380e-07_rb,0.40945e-07_rb,0.42005e-07_rb /)
      kbo(:,29, 9) = (/ &
        & 0.34641e-07_rb,0.38943e-07_rb,0.36153e-07_rb,0.36773e-07_rb,0.32051e-07_rb /)
      kbo(:,30, 9) = (/ &
        & 0.31936e-07_rb,0.28774e-07_rb,0.32629e-07_rb,0.32074e-07_rb,0.31547e-07_rb /)
      kbo(:,31, 9) = (/ &
        & 0.25468e-07_rb,0.25946e-07_rb,0.23671e-07_rb,0.23770e-07_rb,0.28704e-07_rb /)
      kbo(:,32, 9) = (/ &
        & 0.18317e-07_rb,0.22069e-07_rb,0.20055e-07_rb,0.21665e-07_rb,0.25653e-07_rb /)
      kbo(:,33, 9) = (/ &
        & 0.18319e-07_rb,0.17920e-07_rb,0.16523e-07_rb,0.20049e-07_rb,0.19951e-07_rb /)
      kbo(:,34, 9) = (/ &
        & 0.15127e-07_rb,0.13546e-07_rb,0.16257e-07_rb,0.16818e-07_rb,0.15661e-07_rb /)
      kbo(:,35, 9) = (/ &
        & 0.12192e-07_rb,0.13829e-07_rb,0.15792e-07_rb,0.15440e-07_rb,0.14596e-07_rb /)
      kbo(:,36, 9) = (/ &
        & 0.11593e-07_rb,0.10509e-07_rb,0.13211e-07_rb,0.13026e-07_rb,0.13917e-07_rb /)
      kbo(:,37, 9) = (/ &
        & 0.12302e-07_rb,0.13196e-07_rb,0.12558e-07_rb,0.11760e-07_rb,0.12490e-07_rb /)
      kbo(:,38, 9) = (/ &
        & 0.10116e-07_rb,0.12498e-07_rb,0.12546e-07_rb,0.12890e-07_rb,0.12908e-07_rb /)
      kbo(:,39, 9) = (/ &
        & 0.88258e-08_rb,0.10400e-07_rb,0.11293e-07_rb,0.11361e-07_rb,0.11468e-07_rb /)
      kbo(:,40, 9) = (/ &
        & 0.70753e-08_rb,0.81481e-08_rb,0.94292e-08_rb,0.10943e-07_rb,0.90300e-08_rb /)
      kbo(:,41, 9) = (/ &
        & 0.71994e-08_rb,0.84394e-08_rb,0.89108e-08_rb,0.89407e-08_rb,0.87875e-08_rb /)
      kbo(:,42, 9) = (/ &
        & 0.55536e-08_rb,0.66846e-08_rb,0.83965e-08_rb,0.80560e-08_rb,0.79305e-08_rb /)
      kbo(:,43, 9) = (/ &
        & 0.50223e-08_rb,0.59958e-08_rb,0.69583e-08_rb,0.70648e-08_rb,0.76990e-08_rb /)
      kbo(:,44, 9) = (/ &
        & 0.40990e-08_rb,0.43884e-08_rb,0.50706e-08_rb,0.56081e-08_rb,0.62414e-08_rb /)
      kbo(:,45, 9) = (/ &
        & 0.35590e-08_rb,0.38731e-08_rb,0.41923e-08_rb,0.44278e-08_rb,0.44950e-08_rb /)
      kbo(:,46, 9) = (/ &
        & 0.26814e-08_rb,0.33691e-08_rb,0.34101e-08_rb,0.34324e-08_rb,0.35321e-08_rb /)
      kbo(:,47, 9) = (/ &
        & 0.21424e-08_rb,0.21249e-08_rb,0.27532e-08_rb,0.28475e-08_rb,0.25748e-08_rb /)
      kbo(:,48, 9) = (/ &
        & 0.16514e-08_rb,0.18931e-08_rb,0.22507e-08_rb,0.22575e-08_rb,0.21238e-08_rb /)
      kbo(:,49, 9) = (/ &
        & 0.12235e-08_rb,0.12503e-08_rb,0.16643e-08_rb,0.16742e-08_rb,0.16438e-08_rb /)
      kbo(:,50, 9) = (/ &
        & 0.11340e-08_rb,0.11248e-08_rb,0.13120e-08_rb,0.13854e-08_rb,0.13503e-08_rb /)
      kbo(:,51, 9) = (/ &
        & 0.79176e-09_rb,0.91681e-09_rb,0.99908e-09_rb,0.13532e-08_rb,0.10405e-08_rb /)
      kbo(:,52, 9) = (/ &
        & 0.64785e-09_rb,0.57185e-09_rb,0.84784e-09_rb,0.90912e-09_rb,0.87721e-09_rb /)
      kbo(:,53, 9) = (/ &
        & 0.51330e-09_rb,0.43209e-09_rb,0.61659e-09_rb,0.79798e-09_rb,0.65017e-09_rb /)
      kbo(:,54, 9) = (/ &
        & 0.40227e-09_rb,0.45562e-09_rb,0.54691e-09_rb,0.63160e-09_rb,0.62902e-09_rb /)
      kbo(:,55, 9) = (/ &
        & 0.44280e-09_rb,0.43833e-09_rb,0.48132e-09_rb,0.55748e-09_rb,0.72672e-09_rb /)
      kbo(:,56, 9) = (/ &
        & 0.33767e-09_rb,0.43448e-09_rb,0.42774e-09_rb,0.52401e-09_rb,0.66273e-09_rb /)
      kbo(:,57, 9) = (/ &
        & 0.40113e-09_rb,0.38794e-09_rb,0.46286e-09_rb,0.57261e-09_rb,0.60773e-09_rb /)
      kbo(:,58, 9) = (/ &
        & 0.36624e-09_rb,0.52965e-09_rb,0.49530e-09_rb,0.54114e-09_rb,0.65708e-09_rb /)
      kbo(:,59, 9) = (/ &
        & 0.52598e-09_rb,0.55940e-09_rb,0.70094e-09_rb,0.71749e-09_rb,0.81086e-09_rb /)
      kbo(:,13,10) = (/ &
        & 0.75197e-05_rb,0.74908e-05_rb,0.74768e-05_rb,0.74579e-05_rb,0.74485e-05_rb /)
      kbo(:,14,10) = (/ &
        & 0.62427e-05_rb,0.62400e-05_rb,0.62314e-05_rb,0.62357e-05_rb,0.62230e-05_rb /)
      kbo(:,15,10) = (/ &
        & 0.51235e-05_rb,0.51220e-05_rb,0.51270e-05_rb,0.51388e-05_rb,0.51377e-05_rb /)
      kbo(:,16,10) = (/ &
        & 0.36653e-05_rb,0.36812e-05_rb,0.36844e-05_rb,0.36893e-05_rb,0.37175e-05_rb /)
      kbo(:,17,10) = (/ &
        & 0.10067e-05_rb,0.10571e-05_rb,0.10948e-05_rb,0.11331e-05_rb,0.11791e-05_rb /)
      kbo(:,18,10) = (/ &
        & 0.37963e-07_rb,0.35028e-07_rb,0.38085e-07_rb,0.36252e-07_rb,0.39741e-07_rb /)
      kbo(:,19,10) = (/ &
        & 0.58861e-07_rb,0.39909e-07_rb,0.43872e-07_rb,0.52963e-07_rb,0.37762e-07_rb /)
      kbo(:,20,10) = (/ &
        & 0.65438e-07_rb,0.43429e-07_rb,0.52709e-07_rb,0.35811e-07_rb,0.59833e-07_rb /)
      kbo(:,21,10) = (/ &
        & 0.50452e-07_rb,0.50995e-07_rb,0.40371e-07_rb,0.51145e-07_rb,0.52725e-07_rb /)
      kbo(:,22,10) = (/ &
        & 0.42139e-07_rb,0.45011e-07_rb,0.61867e-07_rb,0.41292e-07_rb,0.37916e-07_rb /)
      kbo(:,23,10) = (/ &
        & 0.30176e-07_rb,0.29856e-07_rb,0.39637e-07_rb,0.38957e-07_rb,0.64896e-07_rb /)
      kbo(:,24,10) = (/ &
        & 0.36010e-07_rb,0.42155e-07_rb,0.32787e-07_rb,0.46750e-07_rb,0.46940e-07_rb /)
      kbo(:,25,10) = (/ &
        & 0.34802e-07_rb,0.41607e-07_rb,0.40854e-07_rb,0.61643e-07_rb,0.53762e-07_rb /)
      kbo(:,26,10) = (/ &
        & 0.41295e-07_rb,0.39697e-07_rb,0.46005e-07_rb,0.46334e-07_rb,0.53334e-07_rb /)
      kbo(:,27,10) = (/ &
        & 0.61711e-07_rb,0.29641e-07_rb,0.51823e-07_rb,0.42625e-07_rb,0.36336e-07_rb /)
      kbo(:,28,10) = (/ &
        & 0.45196e-07_rb,0.56637e-07_rb,0.31716e-07_rb,0.38569e-07_rb,0.45693e-07_rb /)
      kbo(:,29,10) = (/ &
        & 0.47349e-07_rb,0.38492e-07_rb,0.26330e-07_rb,0.30953e-07_rb,0.33330e-07_rb /)
      kbo(:,30,10) = (/ &
        & 0.20700e-07_rb,0.24999e-07_rb,0.19626e-07_rb,0.33645e-07_rb,0.21883e-07_rb /)
      kbo(:,31,10) = (/ &
        & 0.31879e-07_rb,0.26245e-07_rb,0.28697e-07_rb,0.26867e-07_rb,0.26801e-07_rb /)
      kbo(:,32,10) = (/ &
        & 0.29616e-07_rb,0.18927e-07_rb,0.18903e-07_rb,0.14917e-07_rb,0.25602e-07_rb /)
      kbo(:,33,10) = (/ &
        & 0.22437e-07_rb,0.26260e-07_rb,0.17262e-07_rb,0.13354e-07_rb,0.18413e-07_rb /)
      kbo(:,34,10) = (/ &
        & 0.15957e-07_rb,0.11226e-07_rb,0.16852e-07_rb,0.11987e-07_rb,0.14358e-07_rb /)
      kbo(:,35,10) = (/ &
        & 0.11233e-07_rb,0.90736e-08_rb,0.13960e-07_rb,0.76472e-08_rb,0.11733e-07_rb /)
      kbo(:,36,10) = (/ &
        & 0.13919e-07_rb,0.10081e-07_rb,0.10035e-07_rb,0.10345e-07_rb,0.10622e-07_rb /)
      kbo(:,37,10) = (/ &
        & 0.76780e-08_rb,0.12331e-07_rb,0.10794e-07_rb,0.16023e-07_rb,0.11524e-07_rb /)
      kbo(:,38,10) = (/ &
        & 0.65416e-08_rb,0.97484e-08_rb,0.12132e-07_rb,0.10472e-07_rb,0.11363e-07_rb /)
      kbo(:,39,10) = (/ &
        & 0.78511e-08_rb,0.70392e-08_rb,0.91444e-08_rb,0.84352e-08_rb,0.95946e-08_rb /)
      kbo(:,40,10) = (/ &
        & 0.81878e-08_rb,0.94957e-08_rb,0.84996e-08_rb,0.11246e-07_rb,0.13024e-07_rb /)
      kbo(:,41,10) = (/ &
        & 0.65901e-08_rb,0.80717e-08_rb,0.78357e-08_rb,0.67309e-08_rb,0.85966e-08_rb /)
      kbo(:,42,10) = (/ &
        & 0.64210e-08_rb,0.59979e-08_rb,0.56936e-08_rb,0.70018e-08_rb,0.70433e-08_rb /)
      kbo(:,43,10) = (/ &
        & 0.40388e-08_rb,0.71452e-08_rb,0.63548e-08_rb,0.84902e-08_rb,0.75828e-08_rb /)
      kbo(:,44,10) = (/ &
        & 0.39142e-08_rb,0.55915e-08_rb,0.49863e-08_rb,0.52254e-08_rb,0.62790e-08_rb /)
      kbo(:,45,10) = (/ &
        & 0.42931e-08_rb,0.31608e-08_rb,0.46572e-08_rb,0.47490e-08_rb,0.59912e-08_rb /)
      kbo(:,46,10) = (/ &
        & 0.38722e-08_rb,0.16146e-08_rb,0.40010e-08_rb,0.24242e-08_rb,0.45711e-08_rb /)
      kbo(:,47,10) = (/ &
        & 0.14942e-08_rb,0.28671e-08_rb,0.27168e-08_rb,0.40090e-08_rb,0.42343e-08_rb /)
      kbo(:,48,10) = (/ &
        & 0.11911e-08_rb,0.13582e-08_rb,0.26580e-08_rb,0.29275e-08_rb,0.17855e-08_rb /)
      kbo(:,49,10) = (/ &
        & 0.14130e-08_rb,0.10553e-08_rb,0.10543e-08_rb,0.11428e-08_rb,0.14141e-08_rb /)
      kbo(:,50,10) = (/ &
        & 0.12182e-08_rb,0.14973e-08_rb,0.13182e-08_rb,0.84197e-09_rb,0.12115e-08_rb /)
      kbo(:,51,10) = (/ &
        & 0.81227e-09_rb,0.90496e-09_rb,0.77903e-09_rb,0.83007e-09_rb,0.86051e-09_rb /)
      kbo(:,52,10) = (/ &
        & 0.96715e-09_rb,0.52504e-09_rb,0.42702e-09_rb,0.99435e-09_rb,0.91829e-09_rb /)
      kbo(:,53,10) = (/ &
        & 0.37232e-09_rb,0.22359e-09_rb,0.89279e-09_rb,0.85173e-09_rb,0.51790e-09_rb /)
      kbo(:,54,10) = (/ &
        & 0.34402e-09_rb,0.40116e-09_rb,0.54696e-09_rb,0.50848e-09_rb,0.53123e-09_rb /)
      kbo(:,55,10) = (/ &
        & 0.66100e-09_rb,0.65370e-09_rb,0.58427e-09_rb,0.74840e-09_rb,0.49778e-09_rb /)
      kbo(:,56,10) = (/ &
        & 0.43138e-09_rb,0.39155e-09_rb,0.52944e-09_rb,0.58374e-09_rb,0.77481e-09_rb /)
      kbo(:,57,10) = (/ &
        & 0.48190e-09_rb,0.37726e-09_rb,0.54783e-09_rb,0.43462e-09_rb,0.46295e-09_rb /)
      kbo(:,58,10) = (/ &
        & 0.42276e-09_rb,0.62831e-09_rb,0.68493e-09_rb,0.49089e-09_rb,0.43799e-09_rb /)
      kbo(:,59,10) = (/ &
        & 0.37023e-09_rb,0.90764e-09_rb,0.36527e-09_rb,0.99235e-09_rb,0.10719e-08_rb /)
      kbo(:,13,11) = (/ &
        & 0.15119e-04_rb,0.15135e-04_rb,0.15140e-04_rb,0.15181e-04_rb,0.15209e-04_rb /)
      kbo(:,14,11) = (/ &
        & 0.12807e-04_rb,0.12837e-04_rb,0.12873e-04_rb,0.12938e-04_rb,0.12999e-04_rb /)
      kbo(:,15,11) = (/ &
        & 0.10789e-04_rb,0.10839e-04_rb,0.10921e-04_rb,0.10986e-04_rb,0.11078e-04_rb /)
      kbo(:,16,11) = (/ &
        & 0.90472e-05_rb,0.91077e-05_rb,0.91930e-05_rb,0.92812e-05_rb,0.93959e-05_rb /)
      kbo(:,17,11) = (/ &
        & 0.74085e-05_rb,0.75116e-05_rb,0.76103e-05_rb,0.77238e-05_rb,0.78364e-05_rb /)
      kbo(:,18,11) = (/ &
        & 0.47863e-05_rb,0.49607e-05_rb,0.51634e-05_rb,0.53824e-05_rb,0.55724e-05_rb /)
      kbo(:,19,11) = (/ &
        & 0.17849e-05_rb,0.19797e-05_rb,0.21641e-05_rb,0.23597e-05_rb,0.26015e-05_rb /)
      kbo(:,20,11) = (/ &
        & 0.14011e-06_rb,0.26424e-06_rb,0.39902e-06_rb,0.56382e-06_rb,0.75666e-06_rb /)
      kbo(:,21,11) = (/ &
        & 0.32681e-07_rb,0.30855e-07_rb,0.23688e-07_rb,0.28954e-07_rb,0.30262e-07_rb /)
      kbo(:,22,11) = (/ &
        & 0.39000e-07_rb,0.34641e-07_rb,0.36220e-07_rb,0.29230e-07_rb,0.25365e-07_rb /)
      kbo(:,23,11) = (/ &
        & 0.39936e-07_rb,0.40586e-07_rb,0.26460e-07_rb,0.13886e-07_rb,0.18108e-07_rb /)
      kbo(:,24,11) = (/ &
        & 0.34546e-07_rb,0.30818e-07_rb,0.33243e-07_rb,0.31408e-07_rb,0.37755e-07_rb /)
      kbo(:,25,11) = (/ &
        & 0.36078e-07_rb,0.45586e-07_rb,0.18117e-07_rb,0.20176e-07_rb,0.46256e-07_rb /)
      kbo(:,26,11) = (/ &
        & 0.61382e-07_rb,0.31185e-07_rb,0.38578e-07_rb,0.18908e-07_rb,0.43919e-07_rb /)
      kbo(:,27,11) = (/ &
        & 0.37585e-07_rb,0.28323e-07_rb,0.45638e-07_rb,0.34261e-07_rb,0.26227e-07_rb /)
      kbo(:,28,11) = (/ &
        & 0.41226e-07_rb,0.24793e-07_rb,0.23296e-07_rb,0.20952e-07_rb,0.27899e-07_rb /)
      kbo(:,29,11) = (/ &
        & 0.21318e-07_rb,0.23768e-07_rb,0.39220e-07_rb,0.28896e-07_rb,0.15703e-07_rb /)
      kbo(:,30,11) = (/ &
        & 0.25522e-07_rb,0.29485e-07_rb,0.39326e-07_rb,0.29293e-07_rb,0.36529e-07_rb /)
      kbo(:,31,11) = (/ &
        & 0.14494e-07_rb,0.25417e-07_rb,0.13732e-07_rb,0.97827e-08_rb,0.20014e-07_rb /)
      kbo(:,32,11) = (/ &
        & 0.14101e-07_rb,0.26239e-07_rb,0.22312e-07_rb,0.10163e-07_rb,0.17547e-07_rb /)
      kbo(:,33,11) = (/ &
        & 0.17347e-07_rb,0.17261e-07_rb,0.13014e-07_rb,0.21130e-07_rb,0.31576e-08_rb /)
      kbo(:,34,11) = (/ &
        & 0.11923e-07_rb,0.13818e-07_rb,0.14692e-07_rb,0.14120e-07_rb,0.12000e-07_rb /)
      kbo(:,35,11) = (/ &
        & 0.86917e-08_rb,0.10083e-07_rb,0.11872e-07_rb,0.58578e-08_rb,0.13194e-07_rb /)
      kbo(:,36,11) = (/ &
        & 0.74538e-08_rb,0.94606e-08_rb,0.85633e-08_rb,0.15394e-07_rb,0.81997e-08_rb /)
      kbo(:,37,11) = (/ &
        & 0.59982e-08_rb,0.58805e-08_rb,0.92255e-08_rb,0.69472e-08_rb,0.63632e-08_rb /)
      kbo(:,38,11) = (/ &
        & 0.75481e-08_rb,0.82094e-08_rb,0.10405e-07_rb,0.62921e-08_rb,0.70021e-08_rb /)
      kbo(:,39,11) = (/ &
        & 0.65770e-08_rb,0.73715e-08_rb,0.70899e-08_rb,0.49884e-08_rb,0.73567e-08_rb /)
      kbo(:,40,11) = (/ &
        & 0.62200e-08_rb,0.79227e-08_rb,0.65710e-08_rb,0.35793e-08_rb,0.48653e-08_rb /)
      kbo(:,41,11) = (/ &
        & 0.67810e-08_rb,0.64933e-08_rb,0.26651e-08_rb,0.71067e-08_rb,0.42136e-08_rb /)
      kbo(:,42,11) = (/ &
        & 0.32351e-08_rb,0.79993e-08_rb,0.25727e-08_rb,0.70581e-08_rb,0.58932e-08_rb /)
      kbo(:,43,11) = (/ &
        & 0.54331e-08_rb,0.21653e-08_rb,0.69372e-08_rb,0.30142e-08_rb,0.23071e-07_rb /)
      kbo(:,44,11) = (/ &
        & 0.26834e-08_rb,0.43047e-08_rb,0.44246e-08_rb,0.29474e-08_rb,0.52315e-07_rb /)
      kbo(:,45,11) = (/ &
        & 0.22521e-08_rb,0.21920e-08_rb,0.35952e-08_rb,0.29799e-08_rb,0.81164e-07_rb /)
      kbo(:,46,11) = (/ &
        & 0.28580e-08_rb,0.12605e-08_rb,0.64390e-09_rb,0.24255e-07_rb,0.96879e-07_rb /)
      kbo(:,47,11) = (/ &
        & 0.13005e-08_rb,0.17967e-08_rb,0.21618e-08_rb,0.30225e-07_rb,0.10319e-06_rb /)
      kbo(:,48,11) = (/ &
        & 0.13676e-08_rb,0.96043e-09_rb,0.14275e-08_rb,0.39735e-07_rb,0.11261e-06_rb /)
      kbo(:,49,11) = (/ &
        & 0.79619e-09_rb,0.85338e-09_rb,0.14300e-08_rb,0.51821e-07_rb,0.12169e-06_rb /)
      kbo(:,50,11) = (/ &
        & 0.49284e-09_rb,0.21801e-09_rb,0.75017e-08_rb,0.50308e-07_rb,0.11953e-06_rb /)
      kbo(:,51,11) = (/ &
        & 0.51956e-09_rb,0.59927e-09_rb,0.87276e-08_rb,0.45028e-07_rb,0.11141e-06_rb /)
      kbo(:,52,11) = (/ &
        & 0.35814e-09_rb,0.78982e-09_rb,0.10205e-07_rb,0.45574e-07_rb,0.10332e-06_rb /)
      kbo(:,53,11) = (/ &
        & 0.69729e-09_rb,0.23757e-09_rb,0.11741e-07_rb,0.45086e-07_rb,0.98023e-07_rb /)
      kbo(:,54,11) = (/ &
        & 0.38203e-09_rb,0.28792e-09_rb,0.72239e-08_rb,0.35770e-07_rb,0.80771e-07_rb /)
      kbo(:,55,11) = (/ &
        & 0.33870e-09_rb,0.39698e-09_rb,0.15883e-08_rb,0.22568e-07_rb,0.61465e-07_rb /)
      kbo(:,56,11) = (/ &
        & 0.50424e-09_rb,0.25383e-09_rb,0.46226e-09_rb,0.12214e-07_rb,0.43776e-07_rb /)
      kbo(:,57,11) = (/ &
        & 0.18824e-09_rb,0.14340e-09_rb,0.96547e-10_rb,0.36771e-08_rb,0.28437e-07_rb /)
      kbo(:,58,11) = (/ &
        & 0.53960e-09_rb,0.71198e-09_rb,0.40290e-09_rb,0.38600e-09_rb,0.14449e-07_rb /)
      kbo(:,59,11) = (/ &
        & 0.64570e-09_rb,0.80856e-09_rb,0.54439e-09_rb,0.66021e-09_rb,0.28306e-09_rb /)
      kbo(:,13,12) = (/ &
        & 0.35137e-04_rb,0.35315e-04_rb,0.35473e-04_rb,0.35691e-04_rb,0.35953e-04_rb /)
      kbo(:,14,12) = (/ &
        & 0.30387e-04_rb,0.30535e-04_rb,0.30693e-04_rb,0.30906e-04_rb,0.31121e-04_rb /)
      kbo(:,15,12) = (/ &
        & 0.26404e-04_rb,0.26560e-04_rb,0.26776e-04_rb,0.26963e-04_rb,0.27157e-04_rb /)
      kbo(:,16,12) = (/ &
        & 0.22908e-04_rb,0.23161e-04_rb,0.23420e-04_rb,0.23636e-04_rb,0.23864e-04_rb /)
      kbo(:,17,12) = (/ &
        & 0.19849e-04_rb,0.20170e-04_rb,0.20453e-04_rb,0.20723e-04_rb,0.21011e-04_rb /)
      kbo(:,18,12) = (/ &
        & 0.17158e-04_rb,0.17516e-04_rb,0.17820e-04_rb,0.18149e-04_rb,0.18467e-04_rb /)
      kbo(:,19,12) = (/ &
        & 0.14706e-04_rb,0.15074e-04_rb,0.15426e-04_rb,0.15809e-04_rb,0.16128e-04_rb /)
      kbo(:,20,12) = (/ &
        & 0.12200e-04_rb,0.12654e-04_rb,0.13122e-04_rb,0.13561e-04_rb,0.13942e-04_rb /)
      kbo(:,21,12) = (/ &
        & 0.88744e-05_rb,0.95294e-05_rb,0.10194e-04_rb,0.10811e-04_rb,0.11447e-04_rb /)
      kbo(:,22,12) = (/ &
        & 0.63063e-05_rb,0.69160e-05_rb,0.75170e-05_rb,0.81696e-05_rb,0.87829e-05_rb /)
      kbo(:,23,12) = (/ &
        & 0.42549e-05_rb,0.48576e-05_rb,0.54224e-05_rb,0.60649e-05_rb,0.65852e-05_rb /)
      kbo(:,24,12) = (/ &
        & 0.26155e-05_rb,0.31537e-05_rb,0.36816e-05_rb,0.42795e-05_rb,0.47356e-05_rb /)
      kbo(:,25,12) = (/ &
        & 0.14576e-05_rb,0.18762e-05_rb,0.23875e-05_rb,0.29308e-05_rb,0.33846e-05_rb /)
      kbo(:,26,12) = (/ &
        & 0.67369e-06_rb,0.10096e-05_rb,0.14730e-05_rb,0.19592e-05_rb,0.24356e-05_rb /)
      kbo(:,27,12) = (/ &
        & 0.27240e-06_rb,0.59733e-06_rb,0.98553e-06_rb,0.13968e-05_rb,0.18466e-05_rb /)
      kbo(:,28,12) = (/ &
        & 0.81179e-07_rb,0.35063e-06_rb,0.74692e-06_rb,0.11081e-05_rb,0.15613e-05_rb /)
      kbo(:,29,12) = (/ &
        & 0.94800e-07_rb,0.40513e-06_rb,0.73547e-06_rb,0.10914e-05_rb,0.16188e-05_rb /)
      kbo(:,30,12) = (/ &
        & 0.19045e-06_rb,0.51192e-06_rb,0.80594e-06_rb,0.12694e-05_rb,0.16954e-05_rb /)
      kbo(:,31,12) = (/ &
        & 0.38493e-06_rb,0.71230e-06_rb,0.11392e-05_rb,0.15875e-05_rb,0.20565e-05_rb /)
      kbo(:,32,12) = (/ &
        & 0.61444e-06_rb,0.97779e-06_rb,0.14257e-05_rb,0.18883e-05_rb,0.26649e-05_rb /)
      kbo(:,33,12) = (/ &
        & 0.84230e-06_rb,0.12390e-05_rb,0.17304e-05_rb,0.23322e-05_rb,0.32859e-05_rb /)
      kbo(:,34,12) = (/ &
        & 0.10458e-05_rb,0.14709e-05_rb,0.19634e-05_rb,0.28004e-05_rb,0.37724e-05_rb /)
      kbo(:,35,12) = (/ &
        & 0.11901e-05_rb,0.16306e-05_rb,0.22411e-05_rb,0.31089e-05_rb,0.41007e-05_rb /)
      kbo(:,36,12) = (/ &
        & 0.12859e-05_rb,0.17604e-05_rb,0.24414e-05_rb,0.32964e-05_rb,0.43267e-05_rb /)
      kbo(:,37,12) = (/ &
        & 0.13015e-05_rb,0.17577e-05_rb,0.24562e-05_rb,0.33117e-05_rb,0.43238e-05_rb /)
      kbo(:,38,12) = (/ &
        & 0.13522e-05_rb,0.18247e-05_rb,0.25116e-05_rb,0.33483e-05_rb,0.43352e-05_rb /)
      kbo(:,39,12) = (/ &
        & 0.14322e-05_rb,0.19336e-05_rb,0.26107e-05_rb,0.34258e-05_rb,0.44158e-05_rb /)
      kbo(:,40,12) = (/ &
        & 0.14006e-05_rb,0.18728e-05_rb,0.25304e-05_rb,0.33179e-05_rb,0.42738e-05_rb /)
      kbo(:,41,12) = (/ &
        & 0.13551e-05_rb,0.18012e-05_rb,0.24357e-05_rb,0.32084e-05_rb,0.41141e-05_rb /)
      kbo(:,42,12) = (/ &
        & 0.13297e-05_rb,0.17512e-05_rb,0.23561e-05_rb,0.30946e-05_rb,0.39665e-05_rb /)
      kbo(:,43,12) = (/ &
        & 0.12687e-05_rb,0.16768e-05_rb,0.22464e-05_rb,0.29540e-05_rb,0.37527e-05_rb /)
      kbo(:,44,12) = (/ &
        & 0.12099e-05_rb,0.15968e-05_rb,0.21385e-05_rb,0.28006e-05_rb,0.35240e-05_rb /)
      kbo(:,45,12) = (/ &
        & 0.11534e-05_rb,0.15243e-05_rb,0.20279e-05_rb,0.26581e-05_rb,0.33062e-05_rb /)
      kbo(:,46,12) = (/ &
        & 0.10898e-05_rb,0.14434e-05_rb,0.19097e-05_rb,0.24739e-05_rb,0.30807e-05_rb /)
      kbo(:,47,12) = (/ &
        & 0.10111e-05_rb,0.13412e-05_rb,0.17607e-05_rb,0.22724e-05_rb,0.28391e-05_rb /)
      kbo(:,48,12) = (/ &
        & 0.92540e-06_rb,0.12447e-05_rb,0.16295e-05_rb,0.20863e-05_rb,0.26097e-05_rb /)
      kbo(:,49,12) = (/ &
        & 0.84684e-06_rb,0.11506e-05_rb,0.15141e-05_rb,0.19160e-05_rb,0.23957e-05_rb /)
      kbo(:,50,12) = (/ &
        & 0.76391e-06_rb,0.10519e-05_rb,0.13910e-05_rb,0.17635e-05_rb,0.22060e-05_rb /)
      kbo(:,51,12) = (/ &
        & 0.68139e-06_rb,0.95640e-06_rb,0.12771e-05_rb,0.16252e-05_rb,0.20363e-05_rb /)
      kbo(:,52,12) = (/ &
        & 0.60392e-06_rb,0.86706e-06_rb,0.11671e-05_rb,0.14940e-05_rb,0.18793e-05_rb /)
      kbo(:,53,12) = (/ &
        & 0.53856e-06_rb,0.78259e-06_rb,0.10620e-05_rb,0.13737e-05_rb,0.17330e-05_rb /)
      kbo(:,54,12) = (/ &
        & 0.46919e-06_rb,0.69589e-06_rb,0.96582e-06_rb,0.12666e-05_rb,0.16070e-05_rb /)
      kbo(:,55,12) = (/ &
        & 0.40013e-06_rb,0.61215e-06_rb,0.87450e-06_rb,0.11663e-05_rb,0.14929e-05_rb /)
      kbo(:,56,12) = (/ &
        & 0.33987e-06_rb,0.53286e-06_rb,0.78520e-06_rb,0.10697e-05_rb,0.13857e-05_rb /)
      kbo(:,57,12) = (/ &
        & 0.28415e-06_rb,0.45936e-06_rb,0.69784e-06_rb,0.97544e-06_rb,0.12829e-05_rb /)
      kbo(:,58,12) = (/ &
        & 0.23457e-06_rb,0.39019e-06_rb,0.61528e-06_rb,0.88439e-06_rb,0.11866e-05_rb /)
      kbo(:,59,12) = (/ &
        & 0.19576e-06_rb,0.34385e-06_rb,0.56055e-06_rb,0.82347e-06_rb,0.11274e-05_rb /)
      kbo(:,13,13) = (/ &
        & 0.12143e-03_rb,0.12308e-03_rb,0.12512e-03_rb,0.12699e-03_rb,0.12910e-03_rb /)
      kbo(:,14,13) = (/ &
        & 0.10385e-03_rb,0.10569e-03_rb,0.10770e-03_rb,0.10972e-03_rb,0.11214e-03_rb /)
      kbo(:,15,13) = (/ &
        & 0.88944e-04_rb,0.90837e-04_rb,0.92688e-04_rb,0.94910e-04_rb,0.97472e-04_rb /)
      kbo(:,16,13) = (/ &
        & 0.76542e-04_rb,0.78268e-04_rb,0.80179e-04_rb,0.82469e-04_rb,0.84734e-04_rb /)
      kbo(:,17,13) = (/ &
        & 0.66332e-04_rb,0.67885e-04_rb,0.69742e-04_rb,0.71831e-04_rb,0.74034e-04_rb /)
      kbo(:,18,13) = (/ &
        & 0.57811e-04_rb,0.59251e-04_rb,0.61096e-04_rb,0.62954e-04_rb,0.65175e-04_rb /)
      kbo(:,19,13) = (/ &
        & 0.50669e-04_rb,0.52124e-04_rb,0.53790e-04_rb,0.55608e-04_rb,0.57861e-04_rb /)
      kbo(:,20,13) = (/ &
        & 0.44810e-04_rb,0.46258e-04_rb,0.47774e-04_rb,0.49573e-04_rb,0.51715e-04_rb /)
      kbo(:,21,13) = (/ &
        & 0.40009e-04_rb,0.41361e-04_rb,0.42755e-04_rb,0.44560e-04_rb,0.46559e-04_rb /)
      kbo(:,22,13) = (/ &
        & 0.36116e-04_rb,0.37297e-04_rb,0.38718e-04_rb,0.40472e-04_rb,0.42441e-04_rb /)
      kbo(:,23,13) = (/ &
        & 0.32736e-04_rb,0.33861e-04_rb,0.35361e-04_rb,0.37075e-04_rb,0.39134e-04_rb /)
      kbo(:,24,13) = (/ &
        & 0.29883e-04_rb,0.31056e-04_rb,0.32552e-04_rb,0.34296e-04_rb,0.36499e-04_rb /)
      kbo(:,25,13) = (/ &
        & 0.27373e-04_rb,0.28693e-04_rb,0.30179e-04_rb,0.32033e-04_rb,0.34358e-04_rb /)
      kbo(:,26,13) = (/ &
        & 0.25273e-04_rb,0.26749e-04_rb,0.28306e-04_rb,0.30341e-04_rb,0.32793e-04_rb /)
      kbo(:,27,13) = (/ &
        & 0.23652e-04_rb,0.25175e-04_rb,0.26905e-04_rb,0.29145e-04_rb,0.31814e-04_rb /)
      kbo(:,28,13) = (/ &
        & 0.22453e-04_rb,0.24077e-04_rb,0.26031e-04_rb,0.28419e-04_rb,0.31259e-04_rb /)
      kbo(:,29,13) = (/ &
        & 0.21794e-04_rb,0.23553e-04_rb,0.25672e-04_rb,0.28266e-04_rb,0.31243e-04_rb /)
      kbo(:,30,13) = (/ &
        & 0.21501e-04_rb,0.23414e-04_rb,0.25687e-04_rb,0.28389e-04_rb,0.31547e-04_rb /)
      kbo(:,31,13) = (/ &
        & 0.21532e-04_rb,0.23577e-04_rb,0.25947e-04_rb,0.28793e-04_rb,0.32070e-04_rb /)
      kbo(:,32,13) = (/ &
        & 0.21702e-04_rb,0.23850e-04_rb,0.26365e-04_rb,0.29337e-04_rb,0.32730e-04_rb /)
      kbo(:,33,13) = (/ &
        & 0.21999e-04_rb,0.24243e-04_rb,0.26895e-04_rb,0.29970e-04_rb,0.33492e-04_rb /)
      kbo(:,34,13) = (/ &
        & 0.22272e-04_rb,0.24615e-04_rb,0.27365e-04_rb,0.30535e-04_rb,0.34149e-04_rb /)
      kbo(:,35,13) = (/ &
        & 0.22408e-04_rb,0.24786e-04_rb,0.27609e-04_rb,0.30838e-04_rb,0.34518e-04_rb /)
      kbo(:,36,13) = (/ &
        & 0.22324e-04_rb,0.24717e-04_rb,0.27535e-04_rb,0.30781e-04_rb,0.34466e-04_rb /)
      kbo(:,37,13) = (/ &
        & 0.21914e-04_rb,0.24278e-04_rb,0.27037e-04_rb,0.30223e-04_rb,0.33847e-04_rb /)
      kbo(:,38,13) = (/ &
        & 0.21518e-04_rb,0.23846e-04_rb,0.26529e-04_rb,0.29664e-04_rb,0.33219e-04_rb /)
      kbo(:,39,13) = (/ &
        & 0.21138e-04_rb,0.23443e-04_rb,0.26064e-04_rb,0.29132e-04_rb,0.32628e-04_rb /)
      kbo(:,40,13) = (/ &
        & 0.20535e-04_rb,0.22813e-04_rb,0.25328e-04_rb,0.28289e-04_rb,0.31682e-04_rb /)
      kbo(:,41,13) = (/ &
        & 0.19918e-04_rb,0.22174e-04_rb,0.24598e-04_rb,0.27447e-04_rb,0.30722e-04_rb /)
      kbo(:,42,13) = (/ &
        & 0.19309e-04_rb,0.21550e-04_rb,0.23909e-04_rb,0.26639e-04_rb,0.29801e-04_rb /)
      kbo(:,43,13) = (/ &
        & 0.18566e-04_rb,0.20812e-04_rb,0.23112e-04_rb,0.25700e-04_rb,0.28732e-04_rb /)
      kbo(:,44,13) = (/ &
        & 0.17767e-04_rb,0.20035e-04_rb,0.22299e-04_rb,0.24756e-04_rb,0.27639e-04_rb /)
      kbo(:,45,13) = (/ &
        & 0.16962e-04_rb,0.19260e-04_rb,0.21500e-04_rb,0.23864e-04_rb,0.26597e-04_rb /)
      kbo(:,46,13) = (/ &
        & 0.16086e-04_rb,0.18421e-04_rb,0.20671e-04_rb,0.22972e-04_rb,0.25545e-04_rb /)
      kbo(:,47,13) = (/ &
        & 0.15115e-04_rb,0.17481e-04_rb,0.19766e-04_rb,0.22019e-04_rb,0.24442e-04_rb /)
      kbo(:,48,13) = (/ &
        & 0.14158e-04_rb,0.16526e-04_rb,0.18843e-04_rb,0.21088e-04_rb,0.23419e-04_rb /)
      kbo(:,49,13) = (/ &
        & 0.13195e-04_rb,0.15552e-04_rb,0.17911e-04_rb,0.20179e-04_rb,0.22454e-04_rb /)
      kbo(:,50,13) = (/ &
        & 0.12314e-04_rb,0.14653e-04_rb,0.17032e-04_rb,0.19326e-04_rb,0.21576e-04_rb /)
      kbo(:,51,13) = (/ &
        & 0.11480e-04_rb,0.13788e-04_rb,0.16154e-04_rb,0.18484e-04_rb,0.20739e-04_rb /)
      kbo(:,52,13) = (/ &
        & 0.10629e-04_rb,0.12922e-04_rb,0.15279e-04_rb,0.17642e-04_rb,0.19921e-04_rb /)
      kbo(:,53,13) = (/ &
        & 0.97710e-05_rb,0.12082e-04_rb,0.14410e-04_rb,0.16788e-04_rb,0.19091e-04_rb /)
      kbo(:,54,13) = (/ &
        & 0.90074e-05_rb,0.11316e-04_rb,0.13617e-04_rb,0.15981e-04_rb,0.18321e-04_rb /)
      kbo(:,55,13) = (/ &
        & 0.83042e-05_rb,0.10569e-04_rb,0.12859e-04_rb,0.15214e-04_rb,0.17581e-04_rb /)
      kbo(:,56,13) = (/ &
        & 0.76202e-05_rb,0.98296e-05_rb,0.12125e-04_rb,0.14457e-04_rb,0.16835e-04_rb /)
      kbo(:,57,13) = (/ &
        & 0.69561e-05_rb,0.91024e-05_rb,0.11398e-04_rb,0.13700e-04_rb,0.16068e-04_rb /)
      kbo(:,58,13) = (/ &
        & 0.63426e-05_rb,0.84347e-05_rb,0.10695e-04_rb,0.12984e-04_rb,0.15340e-04_rb /)
      kbo(:,59,13) = (/ &
        & 0.61056e-05_rb,0.81719e-05_rb,0.10410e-04_rb,0.12698e-04_rb,0.15054e-04_rb /)
      kbo(:,13,14) = (/ &
        & 0.69692e-03_rb,0.71516e-03_rb,0.73378e-03_rb,0.75443e-03_rb,0.77716e-03_rb /)
      kbo(:,14,14) = (/ &
        & 0.64436e-03_rb,0.66211e-03_rb,0.68199e-03_rb,0.70460e-03_rb,0.72891e-03_rb /)
      kbo(:,15,14) = (/ &
        & 0.59775e-03_rb,0.61618e-03_rb,0.63785e-03_rb,0.66198e-03_rb,0.68803e-03_rb /)
      kbo(:,16,14) = (/ &
        & 0.55668e-03_rb,0.57752e-03_rb,0.60052e-03_rb,0.62615e-03_rb,0.65391e-03_rb /)
      kbo(:,17,14) = (/ &
        & 0.52104e-03_rb,0.54458e-03_rb,0.56934e-03_rb,0.59627e-03_rb,0.62539e-03_rb /)
      kbo(:,18,14) = (/ &
        & 0.49086e-03_rb,0.51685e-03_rb,0.54337e-03_rb,0.57147e-03_rb,0.60182e-03_rb /)
      kbo(:,19,14) = (/ &
        & 0.46544e-03_rb,0.49373e-03_rb,0.52192e-03_rb,0.55124e-03_rb,0.58277e-03_rb /)
      kbo(:,20,14) = (/ &
        & 0.44496e-03_rb,0.47526e-03_rb,0.50494e-03_rb,0.53564e-03_rb,0.56857e-03_rb /)
      kbo(:,21,14) = (/ &
        & 0.42855e-03_rb,0.46070e-03_rb,0.49202e-03_rb,0.52375e-03_rb,0.55795e-03_rb /)
      kbo(:,22,14) = (/ &
        & 0.41783e-03_rb,0.45160e-03_rb,0.48421e-03_rb,0.51739e-03_rb,0.55269e-03_rb /)
      kbo(:,23,14) = (/ &
        & 0.41085e-03_rb,0.44564e-03_rb,0.47947e-03_rb,0.51389e-03_rb,0.55016e-03_rb /)
      kbo(:,24,14) = (/ &
        & 0.40699e-03_rb,0.44263e-03_rb,0.47766e-03_rb,0.51290e-03_rb,0.55008e-03_rb /)
      kbo(:,25,14) = (/ &
        & 0.40592e-03_rb,0.44215e-03_rb,0.47812e-03_rb,0.51419e-03_rb,0.55216e-03_rb /)
      kbo(:,26,14) = (/ &
        & 0.40741e-03_rb,0.44425e-03_rb,0.48085e-03_rb,0.51770e-03_rb,0.55654e-03_rb /)
      kbo(:,27,14) = (/ &
        & 0.41074e-03_rb,0.44811e-03_rb,0.48506e-03_rb,0.52264e-03_rb,0.56211e-03_rb /)
      kbo(:,28,14) = (/ &
        & 0.41538e-03_rb,0.45333e-03_rb,0.49042e-03_rb,0.52868e-03_rb,0.56877e-03_rb /)
      kbo(:,29,14) = (/ &
        & 0.42139e-03_rb,0.45956e-03_rb,0.49700e-03_rb,0.53588e-03_rb,0.57647e-03_rb /)
      kbo(:,30,14) = (/ &
        & 0.42836e-03_rb,0.46659e-03_rb,0.50435e-03_rb,0.54377e-03_rb,0.58489e-03_rb /)
      kbo(:,31,14) = (/ &
        & 0.43618e-03_rb,0.47431e-03_rb,0.51248e-03_rb,0.55256e-03_rb,0.59401e-03_rb /)
      kbo(:,32,14) = (/ &
        & 0.44466e-03_rb,0.48256e-03_rb,0.52138e-03_rb,0.56187e-03_rb,0.60364e-03_rb /)
      kbo(:,33,14) = (/ &
        & 0.45349e-03_rb,0.49137e-03_rb,0.53076e-03_rb,0.57166e-03_rb,0.61359e-03_rb /)
      kbo(:,34,14) = (/ &
        & 0.46108e-03_rb,0.49922e-03_rb,0.53902e-03_rb,0.58028e-03_rb,0.62237e-03_rb /)
      kbo(:,35,14) = (/ &
        & 0.46528e-03_rb,0.50354e-03_rb,0.54368e-03_rb,0.58501e-03_rb,0.62724e-03_rb /)
      kbo(:,36,14) = (/ &
        & 0.46539e-03_rb,0.50375e-03_rb,0.54391e-03_rb,0.58538e-03_rb,0.62763e-03_rb /)
      kbo(:,37,14) = (/ &
        & 0.45984e-03_rb,0.49817e-03_rb,0.53806e-03_rb,0.57948e-03_rb,0.62153e-03_rb /)
      kbo(:,38,14) = (/ &
        & 0.45408e-03_rb,0.49229e-03_rb,0.53205e-03_rb,0.57322e-03_rb,0.61533e-03_rb /)
      kbo(:,39,14) = (/ &
        & 0.44837e-03_rb,0.48652e-03_rb,0.52604e-03_rb,0.56712e-03_rb,0.60916e-03_rb /)
      kbo(:,40,14) = (/ &
        & 0.43865e-03_rb,0.47705e-03_rb,0.51607e-03_rb,0.55678e-03_rb,0.59870e-03_rb /)
      kbo(:,41,14) = (/ &
        & 0.42850e-03_rb,0.46708e-03_rb,0.50578e-03_rb,0.54611e-03_rb,0.58790e-03_rb /)
      kbo(:,42,14) = (/ &
        & 0.41832e-03_rb,0.45718e-03_rb,0.49552e-03_rb,0.53551e-03_rb,0.57688e-03_rb /)
      kbo(:,43,14) = (/ &
        & 0.40611e-03_rb,0.44501e-03_rb,0.48328e-03_rb,0.52271e-03_rb,0.56375e-03_rb /)
      kbo(:,44,14) = (/ &
        & 0.39281e-03_rb,0.43169e-03_rb,0.47025e-03_rb,0.50904e-03_rb,0.54960e-03_rb /)
      kbo(:,45,14) = (/ &
        & 0.37939e-03_rb,0.41841e-03_rb,0.45722e-03_rb,0.49565e-03_rb,0.53564e-03_rb /)
      kbo(:,46,14) = (/ &
        & 0.36481e-03_rb,0.40443e-03_rb,0.44334e-03_rb,0.48171e-03_rb,0.52104e-03_rb /)
      kbo(:,47,14) = (/ &
        & 0.34813e-03_rb,0.38867e-03_rb,0.42754e-03_rb,0.46626e-03_rb,0.50489e-03_rb /)
      kbo(:,48,14) = (/ &
        & 0.33101e-03_rb,0.37257e-03_rb,0.41188e-03_rb,0.45078e-03_rb,0.48906e-03_rb /)
      kbo(:,49,14) = (/ &
        & 0.31369e-03_rb,0.35613e-03_rb,0.39621e-03_rb,0.43508e-03_rb,0.47354e-03_rb /)
      kbo(:,50,14) = (/ &
        & 0.29739e-03_rb,0.34030e-03_rb,0.38132e-03_rb,0.42035e-03_rb,0.45918e-03_rb /)
      kbo(:,51,14) = (/ &
        & 0.28140e-03_rb,0.32461e-03_rb,0.36662e-03_rb,0.40619e-03_rb,0.44511e-03_rb /)
      kbo(:,52,14) = (/ &
        & 0.26563e-03_rb,0.30898e-03_rb,0.35161e-03_rb,0.39193e-03_rb,0.43080e-03_rb /)
      kbo(:,53,14) = (/ &
        & 0.24997e-03_rb,0.29311e-03_rb,0.33618e-03_rb,0.37741e-03_rb,0.41655e-03_rb /)
      kbo(:,54,14) = (/ &
        & 0.23534e-03_rb,0.27846e-03_rb,0.32175e-03_rb,0.36391e-03_rb,0.40359e-03_rb /)
      kbo(:,55,14) = (/ &
        & 0.22140e-03_rb,0.26468e-03_rb,0.30799e-03_rb,0.35062e-03_rb,0.39107e-03_rb /)
      kbo(:,56,14) = (/ &
        & 0.20748e-03_rb,0.25097e-03_rb,0.29409e-03_rb,0.33714e-03_rb,0.37835e-03_rb /)
      kbo(:,57,14) = (/ &
        & 0.19363e-03_rb,0.23702e-03_rb,0.28013e-03_rb,0.32339e-03_rb,0.36545e-03_rb /)
      kbo(:,58,14) = (/ &
        & 0.18066e-03_rb,0.22379e-03_rb,0.26710e-03_rb,0.31040e-03_rb,0.35295e-03_rb /)
      kbo(:,59,14) = (/ &
        & 0.17550e-03_rb,0.21855e-03_rb,0.26188e-03_rb,0.30515e-03_rb,0.34793e-03_rb /)
      kbo(:,13,15) = (/ &
        & 0.67077e-02_rb,0.67643e-02_rb,0.68102e-02_rb,0.68432e-02_rb,0.68617e-02_rb /)
      kbo(:,14,15) = (/ &
        & 0.68112e-02_rb,0.68842e-02_rb,0.69413e-02_rb,0.69806e-02_rb,0.70063e-02_rb /)
      kbo(:,15,15) = (/ &
        & 0.68961e-02_rb,0.69833e-02_rb,0.70488e-02_rb,0.70965e-02_rb,0.71269e-02_rb /)
      kbo(:,16,15) = (/ &
        & 0.69706e-02_rb,0.70648e-02_rb,0.71387e-02_rb,0.71925e-02_rb,0.72279e-02_rb /)
      kbo(:,17,15) = (/ &
        & 0.70342e-02_rb,0.71339e-02_rb,0.72130e-02_rb,0.72737e-02_rb,0.73138e-02_rb /)
      kbo(:,18,15) = (/ &
        & 0.70890e-02_rb,0.71933e-02_rb,0.72776e-02_rb,0.73426e-02_rb,0.73856e-02_rb /)
      kbo(:,19,15) = (/ &
        & 0.71380e-02_rb,0.72441e-02_rb,0.73326e-02_rb,0.74006e-02_rb,0.74464e-02_rb /)
      kbo(:,20,15) = (/ &
        & 0.71838e-02_rb,0.72907e-02_rb,0.73816e-02_rb,0.74514e-02_rb,0.74969e-02_rb /)
      kbo(:,21,15) = (/ &
        & 0.72239e-02_rb,0.73326e-02_rb,0.74235e-02_rb,0.74941e-02_rb,0.75405e-02_rb /)
      kbo(:,22,15) = (/ &
        & 0.72699e-02_rb,0.73767e-02_rb,0.74650e-02_rb,0.75340e-02_rb,0.75785e-02_rb /)
      kbo(:,23,15) = (/ &
        & 0.73111e-02_rb,0.74172e-02_rb,0.75033e-02_rb,0.75687e-02_rb,0.76113e-02_rb /)
      kbo(:,24,15) = (/ &
        & 0.73510e-02_rb,0.74537e-02_rb,0.75371e-02_rb,0.76008e-02_rb,0.76395e-02_rb /)
      kbo(:,25,15) = (/ &
        & 0.73875e-02_rb,0.74885e-02_rb,0.75684e-02_rb,0.76282e-02_rb,0.76634e-02_rb /)
      kbo(:,26,15) = (/ &
        & 0.74256e-02_rb,0.75226e-02_rb,0.75985e-02_rb,0.76530e-02_rb,0.76835e-02_rb /)
      kbo(:,27,15) = (/ &
        & 0.74605e-02_rb,0.75517e-02_rb,0.76248e-02_rb,0.76742e-02_rb,0.77008e-02_rb /)
      kbo(:,28,15) = (/ &
        & 0.74927e-02_rb,0.75800e-02_rb,0.76483e-02_rb,0.76927e-02_rb,0.77139e-02_rb /)
      kbo(:,29,15) = (/ &
        & 0.75244e-02_rb,0.76057e-02_rb,0.76690e-02_rb,0.77086e-02_rb,0.77249e-02_rb /)
      kbo(:,30,15) = (/ &
        & 0.75518e-02_rb,0.76303e-02_rb,0.76877e-02_rb,0.77209e-02_rb,0.77334e-02_rb /)
      kbo(:,31,15) = (/ &
        & 0.75781e-02_rb,0.76519e-02_rb,0.77036e-02_rb,0.77320e-02_rb,0.77395e-02_rb /)
      kbo(:,32,15) = (/ &
        & 0.76025e-02_rb,0.76727e-02_rb,0.77178e-02_rb,0.77409e-02_rb,0.77439e-02_rb /)
      kbo(:,33,15) = (/ &
        & 0.76266e-02_rb,0.76908e-02_rb,0.77289e-02_rb,0.77461e-02_rb,0.77464e-02_rb /)
      kbo(:,34,15) = (/ &
        & 0.76450e-02_rb,0.77038e-02_rb,0.77387e-02_rb,0.77507e-02_rb,0.77465e-02_rb /)
      kbo(:,35,15) = (/ &
        & 0.76558e-02_rb,0.77117e-02_rb,0.77433e-02_rb,0.77536e-02_rb,0.77475e-02_rb /)
      kbo(:,36,15) = (/ &
        & 0.76587e-02_rb,0.77150e-02_rb,0.77452e-02_rb,0.77564e-02_rb,0.77495e-02_rb /)
      kbo(:,37,15) = (/ &
        & 0.76522e-02_rb,0.77109e-02_rb,0.77439e-02_rb,0.77579e-02_rb,0.77534e-02_rb /)
      kbo(:,38,15) = (/ &
        & 0.76429e-02_rb,0.77056e-02_rb,0.77426e-02_rb,0.77582e-02_rb,0.77560e-02_rb /)
      kbo(:,39,15) = (/ &
        & 0.76339e-02_rb,0.76993e-02_rb,0.77402e-02_rb,0.77587e-02_rb,0.77588e-02_rb /)
      kbo(:,40,15) = (/ &
        & 0.76166e-02_rb,0.76865e-02_rb,0.77335e-02_rb,0.77564e-02_rb,0.77613e-02_rb /)
      kbo(:,41,15) = (/ &
        & 0.75961e-02_rb,0.76706e-02_rb,0.77239e-02_rb,0.77534e-02_rb,0.77621e-02_rb /)
      kbo(:,42,15) = (/ &
        & 0.75752e-02_rb,0.76537e-02_rb,0.77143e-02_rb,0.77487e-02_rb,0.77622e-02_rb /)
      kbo(:,43,15) = (/ &
        & 0.75459e-02_rb,0.76317e-02_rb,0.76988e-02_rb,0.77406e-02_rb,0.77602e-02_rb /)
      kbo(:,44,15) = (/ &
        & 0.75134e-02_rb,0.76061e-02_rb,0.76788e-02_rb,0.77299e-02_rb,0.77567e-02_rb /)
      kbo(:,45,15) = (/ &
        & 0.74772e-02_rb,0.75773e-02_rb,0.76562e-02_rb,0.77161e-02_rb,0.77502e-02_rb /)
      kbo(:,46,15) = (/ &
        & 0.74372e-02_rb,0.75438e-02_rb,0.76298e-02_rb,0.76975e-02_rb,0.77405e-02_rb /)
      kbo(:,47,15) = (/ &
        & 0.73875e-02_rb,0.75032e-02_rb,0.75980e-02_rb,0.76724e-02_rb,0.77261e-02_rb /)
      kbo(:,48,15) = (/ &
        & 0.73358e-02_rb,0.74597e-02_rb,0.75632e-02_rb,0.76453e-02_rb,0.77080e-02_rb /)
      kbo(:,49,15) = (/ &
        & 0.72811e-02_rb,0.74117e-02_rb,0.75238e-02_rb,0.76147e-02_rb,0.76865e-02_rb /)
      kbo(:,50,15) = (/ &
        & 0.72248e-02_rb,0.73656e-02_rb,0.74842e-02_rb,0.75831e-02_rb,0.76614e-02_rb /)
      kbo(:,51,15) = (/ &
        & 0.71662e-02_rb,0.73174e-02_rb,0.74427e-02_rb,0.75493e-02_rb,0.76342e-02_rb /)
      kbo(:,52,15) = (/ &
        & 0.71006e-02_rb,0.72655e-02_rb,0.73997e-02_rb,0.75133e-02_rb,0.76064e-02_rb /)
      kbo(:,53,15) = (/ &
        & 0.70300e-02_rb,0.72101e-02_rb,0.73531e-02_rb,0.74734e-02_rb,0.75745e-02_rb /)
      kbo(:,54,15) = (/ &
        & 0.69596e-02_rb,0.71548e-02_rb,0.73082e-02_rb,0.74357e-02_rb,0.75431e-02_rb /)
      kbo(:,55,15) = (/ &
        & 0.68873e-02_rb,0.70968e-02_rb,0.72620e-02_rb,0.73967e-02_rb,0.75110e-02_rb /)
      kbo(:,56,15) = (/ &
        & 0.68089e-02_rb,0.70345e-02_rb,0.72141e-02_rb,0.73575e-02_rb,0.74766e-02_rb /)
      kbo(:,57,15) = (/ &
        & 0.67241e-02_rb,0.69678e-02_rb,0.71623e-02_rb,0.73141e-02_rb,0.74401e-02_rb /)
      kbo(:,58,15) = (/ &
        & 0.66371e-02_rb,0.69009e-02_rb,0.71063e-02_rb,0.72699e-02_rb,0.74044e-02_rb /)
      kbo(:,59,15) = (/ &
        & 0.66009e-02_rb,0.68723e-02_rb,0.70852e-02_rb,0.72532e-02_rb,0.73888e-02_rb /)
      kbo(:,13,16) = (/ &
        & 0.20091e-01_rb,0.19310e-01_rb,0.18586e-01_rb,0.17917e-01_rb,0.17293e-01_rb /)
      kbo(:,14,16) = (/ &
        & 0.21405e-01_rb,0.20507e-01_rb,0.19679e-01_rb,0.18912e-01_rb,0.18201e-01_rb /)
      kbo(:,15,16) = (/ &
        & 0.22567e-01_rb,0.21555e-01_rb,0.20633e-01_rb,0.19775e-01_rb,0.18992e-01_rb /)
      kbo(:,16,16) = (/ &
        & 0.23562e-01_rb,0.22456e-01_rb,0.21445e-01_rb,0.20508e-01_rb,0.19661e-01_rb /)
      kbo(:,17,16) = (/ &
        & 0.24411e-01_rb,0.23217e-01_rb,0.22126e-01_rb,0.21126e-01_rb,0.20220e-01_rb /)
      kbo(:,18,16) = (/ &
        & 0.25116e-01_rb,0.23847e-01_rb,0.22688e-01_rb,0.21635e-01_rb,0.20680e-01_rb /)
      kbo(:,19,16) = (/ &
        & 0.25693e-01_rb,0.24361e-01_rb,0.23149e-01_rb,0.22047e-01_rb,0.21053e-01_rb /)
      kbo(:,20,16) = (/ &
        & 0.26126e-01_rb,0.24747e-01_rb,0.23489e-01_rb,0.22355e-01_rb,0.21332e-01_rb /)
      kbo(:,21,16) = (/ &
        & 0.26457e-01_rb,0.25034e-01_rb,0.23751e-01_rb,0.22588e-01_rb,0.21537e-01_rb /)
      kbo(:,22,16) = (/ &
        & 0.26586e-01_rb,0.25153e-01_rb,0.23857e-01_rb,0.22677e-01_rb,0.21621e-01_rb /)
      kbo(:,23,16) = (/ &
        & 0.26640e-01_rb,0.25191e-01_rb,0.23890e-01_rb,0.22706e-01_rb,0.21649e-01_rb /)
      kbo(:,24,16) = (/ &
        & 0.26616e-01_rb,0.25160e-01_rb,0.23861e-01_rb,0.22680e-01_rb,0.21622e-01_rb /)
      kbo(:,25,16) = (/ &
        & 0.26518e-01_rb,0.25075e-01_rb,0.23777e-01_rb,0.22603e-01_rb,0.21555e-01_rb /)
      kbo(:,26,16) = (/ &
        & 0.26348e-01_rb,0.24922e-01_rb,0.23637e-01_rb,0.22478e-01_rb,0.21437e-01_rb /)
      kbo(:,27,16) = (/ &
        & 0.26138e-01_rb,0.24737e-01_rb,0.23465e-01_rb,0.22328e-01_rb,0.21299e-01_rb /)
      kbo(:,28,16) = (/ &
        & 0.25904e-01_rb,0.24526e-01_rb,0.23272e-01_rb,0.22156e-01_rb,0.21140e-01_rb /)
      kbo(:,29,16) = (/ &
        & 0.25645e-01_rb,0.24289e-01_rb,0.23059e-01_rb,0.21960e-01_rb,0.20965e-01_rb /)
      kbo(:,30,16) = (/ &
        & 0.25364e-01_rb,0.24034e-01_rb,0.22831e-01_rb,0.21758e-01_rb,0.20776e-01_rb /)
      kbo(:,31,16) = (/ &
        & 0.25075e-01_rb,0.23766e-01_rb,0.22595e-01_rb,0.21541e-01_rb,0.20581e-01_rb /)
      kbo(:,32,16) = (/ &
        & 0.24769e-01_rb,0.23490e-01_rb,0.22348e-01_rb,0.21316e-01_rb,0.20377e-01_rb /)
      kbo(:,33,16) = (/ &
        & 0.24460e-01_rb,0.23211e-01_rb,0.22098e-01_rb,0.21090e-01_rb,0.20170e-01_rb /)
      kbo(:,34,16) = (/ &
        & 0.24193e-01_rb,0.22975e-01_rb,0.21887e-01_rb,0.20894e-01_rb,0.19994e-01_rb /)
      kbo(:,35,16) = (/ &
        & 0.24050e-01_rb,0.22849e-01_rb,0.21770e-01_rb,0.20789e-01_rb,0.19901e-01_rb /)
      kbo(:,36,16) = (/ &
        & 0.24043e-01_rb,0.22842e-01_rb,0.21764e-01_rb,0.20784e-01_rb,0.19894e-01_rb /)
      kbo(:,37,16) = (/ &
        & 0.24229e-01_rb,0.23006e-01_rb,0.21915e-01_rb,0.20918e-01_rb,0.20018e-01_rb /)
      kbo(:,38,16) = (/ &
        & 0.24421e-01_rb,0.23180e-01_rb,0.22070e-01_rb,0.21062e-01_rb,0.20144e-01_rb /)
      kbo(:,39,16) = (/ &
        & 0.24617e-01_rb,0.23352e-01_rb,0.22226e-01_rb,0.21205e-01_rb,0.20274e-01_rb /)
      kbo(:,40,16) = (/ &
        & 0.24953e-01_rb,0.23654e-01_rb,0.22496e-01_rb,0.21449e-01_rb,0.20495e-01_rb /)
      kbo(:,41,16) = (/ &
        & 0.25312e-01_rb,0.23980e-01_rb,0.22786e-01_rb,0.21712e-01_rb,0.20738e-01_rb /)
      kbo(:,42,16) = (/ &
        & 0.25679e-01_rb,0.24315e-01_rb,0.23082e-01_rb,0.21982e-01_rb,0.20981e-01_rb /)
      kbo(:,43,16) = (/ &
        & 0.26140e-01_rb,0.24730e-01_rb,0.23454e-01_rb,0.22317e-01_rb,0.21287e-01_rb /)
      kbo(:,44,16) = (/ &
        & 0.26653e-01_rb,0.25200e-01_rb,0.23871e-01_rb,0.22691e-01_rb,0.21628e-01_rb /)
      kbo(:,45,16) = (/ &
        & 0.27190e-01_rb,0.25674e-01_rb,0.24311e-01_rb,0.23078e-01_rb,0.21977e-01_rb /)
      kbo(:,46,16) = (/ &
        & 0.27783e-01_rb,0.26205e-01_rb,0.24787e-01_rb,0.23504e-01_rb,0.22362e-01_rb /)
      kbo(:,47,16) = (/ &
        & 0.28471e-01_rb,0.26819e-01_rb,0.25343e-01_rb,0.24006e-01_rb,0.22809e-01_rb /)
      kbo(:,48,16) = (/ &
        & 0.29185e-01_rb,0.27464e-01_rb,0.25918e-01_rb,0.24532e-01_rb,0.23275e-01_rb /)
      kbo(:,49,16) = (/ &
        & 0.29935e-01_rb,0.28138e-01_rb,0.26520e-01_rb,0.25074e-01_rb,0.23762e-01_rb /)
      kbo(:,50,16) = (/ &
        & 0.30661e-01_rb,0.28793e-01_rb,0.27111e-01_rb,0.25603e-01_rb,0.24243e-01_rb /)
      kbo(:,51,16) = (/ &
        & 0.31401e-01_rb,0.29455e-01_rb,0.27706e-01_rb,0.26135e-01_rb,0.24726e-01_rb /)
      kbo(:,52,16) = (/ &
        & 0.32182e-01_rb,0.30140e-01_rb,0.28322e-01_rb,0.26686e-01_rb,0.25224e-01_rb /)
      kbo(:,53,16) = (/ &
        & 0.33002e-01_rb,0.30856e-01_rb,0.28966e-01_rb,0.27268e-01_rb,0.25738e-01_rb /)
      kbo(:,54,16) = (/ &
        & 0.33792e-01_rb,0.31541e-01_rb,0.29577e-01_rb,0.27816e-01_rb,0.26234e-01_rb /)
      kbo(:,55,16) = (/ &
        & 0.34581e-01_rb,0.32233e-01_rb,0.30182e-01_rb,0.28363e-01_rb,0.26721e-01_rb /)
      kbo(:,56,16) = (/ &
        & 0.35416e-01_rb,0.32947e-01_rb,0.30807e-01_rb,0.28926e-01_rb,0.27227e-01_rb /)
      kbo(:,57,16) = (/ &
        & 0.36291e-01_rb,0.33701e-01_rb,0.31460e-01_rb,0.29510e-01_rb,0.27750e-01_rb /)
      kbo(:,58,16) = (/ &
        & 0.37161e-01_rb,0.34436e-01_rb,0.32106e-01_rb,0.30077e-01_rb,0.28264e-01_rb /)
      kbo(:,59,16) = (/ &
        & 0.37521e-01_rb,0.34750e-01_rb,0.32373e-01_rb,0.30309e-01_rb,0.28477e-01_rb /)

!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.515619e-08_rb, 0.131078e-06_rb, 0.349038e-06_rb /)
      forrefo(:, 2) = (/ 0.329605e-07_rb, 0.430497e-06_rb, 0.458569e-05_rb /)
      forrefo(:, 3) = (/ 0.188244e-06_rb, 0.792931e-06_rb, 0.267176e-05_rb /)
      forrefo(:, 4) = (/ 0.611237e-06_rb, 0.798868e-06_rb, 0.411583e-06_rb /)
      forrefo(:, 5) = (/ 0.111903e-05_rb, 0.914895e-06_rb, 0.444828e-06_rb /)
      forrefo(:, 6) = (/ 0.235399e-05_rb, 0.269099e-05_rb, 0.739855e-06_rb /)
      forrefo(:, 7) = (/ 0.400131e-05_rb, 0.378135e-05_rb, 0.231265e-06_rb /)
      forrefo(:, 8) = (/ 0.464257e-05_rb, 0.371927e-05_rb, 0.460611e-06_rb /)
      forrefo(:, 9) = (/ 0.476792e-05_rb, 0.311841e-05_rb, 0.934811e-06_rb /)
      forrefo(:,10) = (/ 0.555683e-05_rb, 0.238129e-05_rb, 0.400334e-07_rb /)
      forrefo(:,11) = (/ 0.569068e-05_rb, 0.196039e-05_rb, 0.374476e-07_rb /)
      forrefo(:,12) = (/ 0.554154e-05_rb, 0.131724e-05_rb, 0.399720e-07_rb /)
      forrefo(:,13) = (/ 0.462684e-05_rb, 0.238826e-07_rb, 0.325793e-07_rb /)
      forrefo(:,14) = (/ 0.808644e-06_rb, 0.105126e-11_rb, 0.148691e-07_rb /)
      forrefo(:,15) = (/ 0.865024e-12_rb, 0.822434e-12_rb, 0.825756e-12_rb /)
      forrefo(:,16) = (/ 0.945747e-12_rb, 0.802065e-12_rb, 0.724732e-12_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
        &  0.637755e-05_rb, 0.403921e-05_rb, 0.255823e-05_rb, 0.162025e-05_rb, 0.102618e-05_rb, &
        &  0.649930e-06_rb, 0.411632e-06_rb, 0.260707e-06_rb, 0.165118e-06_rb, 0.104577e-06_rb /)
      selfrefo(:, 2) = (/ &
        &  0.180887e-04_rb, 0.108890e-04_rb, 0.655493e-05_rb, 0.394592e-05_rb, 0.237536e-05_rb, &
        &  0.142991e-05_rb, 0.860774e-06_rb, 0.518167e-06_rb, 0.311925e-06_rb, 0.187772e-06_rb /)
      selfrefo(:, 3) = (/ &
        &  0.212261e-04_rb, 0.150697e-04_rb, 0.106989e-04_rb, 0.759581e-05_rb, 0.539274e-05_rb, &
        &  0.382864e-05_rb, 0.271819e-05_rb, 0.192981e-05_rb, 0.137009e-05_rb, 0.972711e-06_rb /)
      selfrefo(:, 4) = (/ &
        &  0.132497e-04_rb, 0.118071e-04_rb, 0.105216e-04_rb, 0.937599e-05_rb, 0.835516e-05_rb, &
        &  0.744547e-05_rb, 0.663482e-05_rb, 0.591243e-05_rb, 0.526870e-05_rb, 0.469506e-05_rb /)
      selfrefo(:, 5) = (/ &
        &  0.124069e-04_rb, 0.120785e-04_rb, 0.117589e-04_rb, 0.114477e-04_rb, 0.111447e-04_rb, &
        &  0.108498e-04_rb, 0.105626e-04_rb, 0.102831e-04_rb, 0.100109e-04_rb, 0.974601e-05_rb /)
      selfrefo(:, 6) = (/ &
        &  0.411994e-04_rb, 0.372560e-04_rb, 0.336901e-04_rb, 0.304654e-04_rb, 0.275494e-04_rb, &
        &  0.249126e-04_rb, 0.225281e-04_rb, 0.203718e-04_rb, 0.184219e-04_rb, 0.166587e-04_rb /)
      selfrefo(:, 7) = (/ &
        &  0.537376e-04_rb, 0.501002e-04_rb, 0.467090e-04_rb, 0.435473e-04_rb, 0.405996e-04_rb, &
        &  0.378515e-04_rb, 0.352893e-04_rb, 0.329006e-04_rb, 0.306736e-04_rb, 0.285974e-04_rb /)
      selfrefo(:, 8) = (/ &
        &  0.494279e-04_rb, 0.475365e-04_rb, 0.457175e-04_rb, 0.439681e-04_rb, 0.422857e-04_rb, &
        &  0.406676e-04_rb, 0.391114e-04_rb, 0.376148e-04_rb, 0.361755e-04_rb, 0.347912e-04_rb /)
      selfrefo(:, 9) = (/ &
        &  0.377444e-04_rb, 0.378199e-04_rb, 0.378956e-04_rb, 0.379715e-04_rb, 0.380475e-04_rb, &
        &  0.381236e-04_rb, 0.381999e-04_rb, 0.382763e-04_rb, 0.383529e-04_rb, 0.384297e-04_rb /)
      selfrefo(:,10) = (/ &
        &  0.245916e-04_rb, 0.267183e-04_rb, 0.290289e-04_rb, 0.315394e-04_rb, 0.342669e-04_rb, &
        &  0.372304e-04_rb, 0.404501e-04_rb, 0.439483e-04_rb, 0.477490e-04_rb, 0.518784e-04_rb /)
      selfrefo(:,11) = (/ &
        &  0.186528e-04_rb, 0.211417e-04_rb, 0.239628e-04_rb, 0.271603e-04_rb, 0.307845e-04_rb, &
        &  0.348923e-04_rb, 0.395482e-04_rb, 0.448254e-04_rb, 0.508068e-04_rb, 0.575863e-04_rb /)
      selfrefo(:,12) = (/ &
        &  0.109896e-04_rb, 0.133794e-04_rb, 0.162890e-04_rb, 0.198312e-04_rb, 0.241438e-04_rb, &
        &  0.293942e-04_rb, 0.357864e-04_rb, 0.435686e-04_rb, 0.530432e-04_rb, 0.645781e-04_rb /)
      selfrefo(:,13) = (/ &
        &  0.183885e-06_rb, 0.391019e-06_rb, 0.831472e-06_rb, 0.176806e-05_rb, 0.375966e-05_rb, &
        &  0.799463e-05_rb, 0.170000e-04_rb, 0.361492e-04_rb, 0.768686e-04_rb, 0.163455e-03_rb /)
      selfrefo(:,14) = (/ &
        &  0.466057e-07_rb, 0.937419e-07_rb, 0.188551e-06_rb, 0.379248e-06_rb, 0.762813e-06_rb, &
        &  0.153431e-05_rb, 0.308608e-05_rb, 0.620729e-05_rb, 0.124852e-04_rb, 0.251126e-04_rb /)
      selfrefo(:,15) = (/ &
        &  0.248961e-06_rb, 0.216780e-06_rb, 0.188758e-06_rb, 0.164358e-06_rb, 0.143113e-06_rb, &
        &  0.124613e-06_rb, 0.108505e-06_rb, 0.944795e-07_rb, 0.822667e-07_rb, 0.716326e-07_rb /)
      selfrefo(:,16) = (/ &
        &  0.252246e-06_rb, 0.220335e-06_rb, 0.192462e-06_rb, 0.168114e-06_rb, 0.146847e-06_rb, &
        &  0.128270e-06_rb, 0.112043e-06_rb, 0.978688e-07_rb, 0.854878e-07_rb, 0.746731e-07_rb /)
     
      end subroutine sw_kgb24

! **************************************************************************
      subroutine sw_kgb25
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg25, only : kao, sfluxrefo, &
                            raylo, abso3ao, abso3bo, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:) = (/ &
        & 42.6858_rb , 45.7720_rb, 44.9872_rb, 45.9662_rb    , &
        & 46.5458_rb , 41.6926_rb, 32.2893_rb, 24.0928_rb    , &
        & 16.7686_rb , 1.86048_rb, 1.54057_rb, 1.23503_rb    , &
        & 0.915085_rb,0.590099_rb,0.218622_rb, 3.21287e-02_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:) = (/ &
        & 6.34457E+01_rb, 5.73080E+01_rb, 5.09719E+01_rb, 4.37029E+01_rb,&
        & 3.79060E+01_rb, 3.17827E+01_rb, 2.49233E+01_rb, 1.78197E+01_rb,&
        & 1.09800E+01_rb, 1.33817E+00_rb, 1.34363E+00_rb, 1.16603E+00_rb,&
        & 8.93949E-01_rb, 5.79737E-01_rb, 2.16588E-01_rb, 3.17993E-02_rb/)
! Solar variability components: facular brightening
      facbrghto(:) = (/ &
        & 4.35675E-02_rb, 3.98252E-02_rb, 3.52334E-02_rb, 3.21250E-02_rb,&
        & 2.55637E-02_rb, 2.20128E-02_rb, 1.62401E-02_rb, 1.12279E-02_rb,&
        & 8.03851E-03_rb, 9.82524E-04_rb, 9.51398E-04_rb, 8.12233E-04_rb,&
        & 6.05680E-04_rb, 3.91474E-04_rb, 1.47301E-04_rb, 2.06957E-05_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:) = (/ &
        &-2.69085E-02_rb,-2.62481E-02_rb,-2.48799E-02_rb,-2.27186E-02_rb,&
        &-1.93158E-02_rb,-1.70554E-02_rb,-1.37448E-02_rb,-1.02321E-02_rb,&
        &-6.73504E-03_rb,-7.51258E-04_rb,-6.57594E-04_rb,-5.34977E-04_rb,&
        &-3.94494E-04_rb,-2.51928E-04_rb,-9.50367E-05_rb,-1.33483E-05_rb/)

! Rayleigh extinction coefficient at v = 2925 cm-1.
      raylo(:) = (/ &
        & 9.81132e-07_rb,8.25605e-07_rb,6.71302e-07_rb,5.53556e-07_rb, & 
        & 3.97383e-07_rb,3.68206e-07_rb,4.42379e-07_rb,4.57799e-07_rb, &
        & 4.22683e-07_rb,3.87113e-07_rb,3.79810e-07_rb,3.63192e-07_rb, &
        & 3.51921e-07_rb,3.34231e-07_rb,3.34294e-07_rb,3.32673e-07_rb /)
     
      abso3ao(:) = (/ &
        & 2.32664e-02_rb,5.76154e-02_rb,0.125389_rb,0.250158_rb, &
        & 0.378756_rb   ,0.402196_rb   ,0.352026_rb,0.352036_rb, &
        & 0.386253_rb   ,0.414598_rb   ,0.420079_rb,0.435471_rb, &
        & 0.445487_rb   ,0.459549_rb   ,0.452920_rb,0.456838_rb /)

      abso3bo(:) = (/ &     
        & 1.76917e-02_rb,4.64185e-02_rb,1.03640e-01_rb,0.189469_rb, &
        & 0.303858_rb   ,0.400248_rb   ,0.447357_rb   ,0.470009_rb, &
        & 0.498673_rb   ,0.515696_rb   ,0.517053_rb   ,0.517930_rb, &
        & 0.518345_rb   ,0.524952_rb   ,0.508244_rb   ,0.468981_rb /)

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1) = (/ &
        & 0.16461e-08_rb,0.16782e-08_rb,0.19339e-08_rb,0.17100e-08_rb,0.17045e-08_rb /)
      kao(:, 2, 1) = (/ &
        & 0.28759e-08_rb,0.29469e-08_rb,0.33789e-08_rb,0.34357e-08_rb,0.28833e-08_rb /)
      kao(:, 3, 1) = (/ &
        & 0.55148e-08_rb,0.54808e-08_rb,0.54190e-08_rb,0.68260e-08_rb,0.51972e-08_rb /)
      kao(:, 4, 1) = (/ &
        & 0.95336e-08_rb,0.94552e-08_rb,0.93001e-08_rb,0.90961e-08_rb,0.14451e-07_rb /)
      kao(:, 5, 1) = (/ &
        & 0.14930e-07_rb,0.14736e-07_rb,0.14432e-07_rb,0.14074e-07_rb,0.24102e-07_rb /)
      kao(:, 6, 1) = (/ &
        & 0.22770e-07_rb,0.22301e-07_rb,0.21778e-07_rb,0.21194e-07_rb,0.20569e-07_rb /)
      kao(:, 7, 1) = (/ &
        & 0.34699e-07_rb,0.33951e-07_rb,0.33124e-07_rb,0.32144e-07_rb,0.31220e-07_rb /)
      kao(:, 8, 1) = (/ &
        & 0.62339e-07_rb,0.60405e-07_rb,0.59548e-07_rb,0.58214e-07_rb,0.56977e-07_rb /)
      kao(:, 9, 1) = (/ &
        & 0.17411e-06_rb,0.17654e-06_rb,0.18315e-06_rb,0.18100e-06_rb,0.17839e-06_rb /)
      kao(:,10, 1) = (/ &
        & 0.23526e-06_rb,0.22729e-06_rb,0.21947e-06_rb,0.21188e-06_rb,0.20454e-06_rb /)
      kao(:,11, 1) = (/ &
        & 0.23535e-06_rb,0.22737e-06_rb,0.21956e-06_rb,0.21196e-06_rb,0.20461e-06_rb /)
      kao(:,12, 1) = (/ &
        & 0.23539e-06_rb,0.22740e-06_rb,0.21959e-06_rb,0.21199e-06_rb,0.20465e-06_rb /)
      kao(:,13, 1) = (/ &
        & 0.23543e-06_rb,0.22744e-06_rb,0.21962e-06_rb,0.21202e-06_rb,0.20467e-06_rb /)
      kao(:, 1, 2) = (/ &
        & 0.62912e-08_rb,0.61559e-08_rb,0.84640e-08_rb,0.59240e-08_rb,0.58217e-08_rb /)
      kao(:, 2, 2) = (/ &
        & 0.83749e-08_rb,0.80756e-08_rb,0.11623e-07_rb,0.11272e-07_rb,0.73636e-08_rb /)
      kao(:, 3, 2) = (/ &
        & 0.13304e-07_rb,0.12795e-07_rb,0.12343e-07_rb,0.21235e-07_rb,0.11577e-07_rb /)
      kao(:, 4, 2) = (/ &
        & 0.20704e-07_rb,0.19736e-07_rb,0.18900e-07_rb,0.18228e-07_rb,0.31601e-07_rb /)
      kao(:, 5, 2) = (/ &
        & 0.31149e-07_rb,0.29669e-07_rb,0.28318e-07_rb,0.27101e-07_rb,0.49649e-07_rb /)
      kao(:, 6, 2) = (/ &
        & 0.45713e-07_rb,0.43519e-07_rb,0.41488e-07_rb,0.39918e-07_rb,0.38291e-07_rb /)
      kao(:, 7, 2) = (/ &
        & 0.77265e-07_rb,0.73848e-07_rb,0.70437e-07_rb,0.67945e-07_rb,0.66127e-07_rb /)
      kao(:, 8, 2) = (/ &
        & 0.15754e-06_rb,0.15664e-06_rb,0.15378e-06_rb,0.15027e-06_rb,0.14633e-06_rb /)
      kao(:, 9, 2) = (/ &
        & 0.16439e-06_rb,0.14678e-06_rb,0.12610e-06_rb,0.11532e-06_rb,0.10591e-06_rb /)
      kao(:,10, 2) = (/ &
        & 0.14366e-06_rb,0.13506e-06_rb,0.12583e-06_rb,0.11774e-06_rb,0.11011e-06_rb /)
      kao(:,11, 2) = (/ &
        & 0.14521e-06_rb,0.13766e-06_rb,0.13072e-06_rb,0.12218e-06_rb,0.11400e-06_rb /)
      kao(:,12, 2) = (/ &
        & 0.14524e-06_rb,0.13769e-06_rb,0.13074e-06_rb,0.12241e-06_rb,0.11552e-06_rb /)
      kao(:,13, 2) = (/ &
        & 0.14525e-06_rb,0.13770e-06_rb,0.13075e-06_rb,0.12252e-06_rb,0.11553e-06_rb /)
      kao(:, 1, 3) = (/ &
        & 0.14060e-07_rb,0.13587e-07_rb,0.24644e-07_rb,0.12716e-07_rb,0.12367e-07_rb /)
      kao(:, 2, 3) = (/ &
        & 0.17055e-07_rb,0.16577e-07_rb,0.32443e-07_rb,0.31273e-07_rb,0.15381e-07_rb /)
      kao(:, 3, 3) = (/ &
        & 0.25414e-07_rb,0.24672e-07_rb,0.23874e-07_rb,0.47281e-07_rb,0.22346e-07_rb /)
      kao(:, 4, 3) = (/ &
        & 0.39536e-07_rb,0.38124e-07_rb,0.36836e-07_rb,0.35587e-07_rb,0.72260e-07_rb /)
      kao(:, 5, 3) = (/ &
        & 0.59488e-07_rb,0.57630e-07_rb,0.55623e-07_rb,0.53878e-07_rb,0.11230e-06_rb /)
      kao(:, 6, 3) = (/ &
        & 0.99996e-07_rb,0.96206e-07_rb,0.93184e-07_rb,0.90812e-07_rb,0.89206e-07_rb /)
      kao(:, 7, 3) = (/ &
        & 0.17678e-06_rb,0.17554e-06_rb,0.17358e-06_rb,0.17091e-06_rb,0.16830e-06_rb /)
      kao(:, 8, 3) = (/ &
        & 0.18672e-06_rb,0.17850e-06_rb,0.16967e-06_rb,0.16275e-06_rb,0.15875e-06_rb /)
      kao(:, 9, 3) = (/ &
        & 0.13558e-06_rb,0.13493e-06_rb,0.13633e-06_rb,0.13799e-06_rb,0.13932e-06_rb /)
      kao(:,10, 3) = (/ &
        & 0.18883e-06_rb,0.20452e-06_rb,0.22206e-06_rb,0.24347e-06_rb,0.26091e-06_rb /)
      kao(:,11, 3) = (/ &
        & 0.21296e-06_rb,0.23580e-06_rb,0.26439e-06_rb,0.30148e-06_rb,0.34942e-06_rb /)
      kao(:,12, 3) = (/ &
        & 0.22072e-06_rb,0.25535e-06_rb,0.28661e-06_rb,0.34814e-06_rb,0.39337e-06_rb /)
      kao(:,13, 3) = (/ &
        & 0.22515e-06_rb,0.26161e-06_rb,0.30833e-06_rb,0.36527e-06_rb,0.40123e-06_rb /)
      kao(:, 1, 4) = (/ &
        & 0.32735e-07_rb,0.31345e-07_rb,0.58846e-07_rb,0.28258e-07_rb,0.27022e-07_rb /)
      kao(:, 2, 4) = (/ &
        & 0.37754e-07_rb,0.36873e-07_rb,0.82776e-07_rb,0.80947e-07_rb,0.35190e-07_rb /)
      kao(:, 3, 4) = (/ &
        & 0.76368e-07_rb,0.75292e-07_rb,0.74075e-07_rb,0.10820e-06_rb,0.73183e-07_rb /)
      kao(:, 4, 4) = (/ &
        & 0.16392e-06_rb,0.16130e-06_rb,0.15926e-06_rb,0.15700e-06_rb,0.22041e-06_rb /)
      kao(:, 5, 4) = (/ &
        & 0.29704e-06_rb,0.28924e-06_rb,0.28301e-06_rb,0.27633e-06_rb,0.42284e-06_rb /)
      kao(:, 6, 4) = (/ &
        & 0.48466e-06_rb,0.47240e-06_rb,0.46143e-06_rb,0.45012e-06_rb,0.43867e-06_rb /)
      kao(:, 7, 4) = (/ &
        & 0.71637e-06_rb,0.69847e-06_rb,0.67384e-06_rb,0.65368e-06_rb,0.63375e-06_rb /)
      kao(:, 8, 4) = (/ &
        & 0.11904e-05_rb,0.11714e-05_rb,0.11524e-05_rb,0.11354e-05_rb,0.11172e-05_rb /)
      kao(:, 9, 4) = (/ &
        & 0.21976e-05_rb,0.21606e-05_rb,0.21332e-05_rb,0.20944e-05_rb,0.20536e-05_rb /)
      kao(:,10, 4) = (/ &
        & 0.21713e-05_rb,0.21144e-05_rb,0.20553e-05_rb,0.19901e-05_rb,0.19286e-05_rb /)
      kao(:,11, 4) = (/ &
        & 0.21443e-05_rb,0.20785e-05_rb,0.20048e-05_rb,0.19232e-05_rb,0.18295e-05_rb /)
      kao(:,12, 4) = (/ &
        & 0.21363e-05_rb,0.20578e-05_rb,0.19811e-05_rb,0.18729e-05_rb,0.17807e-05_rb /)
      kao(:,13, 4) = (/ &
        & 0.21319e-05_rb,0.20513e-05_rb,0.19580e-05_rb,0.18546e-05_rb,0.17725e-05_rb /)
      kao(:, 1, 5) = (/ &
        & 0.36050e-07_rb,0.36125e-07_rb,0.46253e-07_rb,0.37280e-07_rb,0.37359e-07_rb /)
      kao(:, 2, 5) = (/ &
        & 0.65102e-07_rb,0.64266e-07_rb,0.68896e-07_rb,0.65925e-07_rb,0.61190e-07_rb /)
      kao(:, 3, 5) = (/ &
        & 0.12173e-06_rb,0.11889e-06_rb,0.11625e-06_rb,0.17574e-06_rb,0.10921e-06_rb /)
      kao(:, 4, 5) = (/ &
        & 0.20555e-06_rb,0.19853e-06_rb,0.19068e-06_rb,0.18313e-06_rb,0.30241e-06_rb /)
      kao(:, 5, 5) = (/ &
        & 0.30900e-06_rb,0.29996e-06_rb,0.28857e-06_rb,0.27772e-06_rb,0.51631e-06_rb /)
      kao(:, 6, 5) = (/ &
        & 0.43774e-06_rb,0.42465e-06_rb,0.40920e-06_rb,0.39315e-06_rb,0.37901e-06_rb /)
      kao(:, 7, 5) = (/ &
        & 0.63869e-06_rb,0.61654e-06_rb,0.60324e-06_rb,0.58966e-06_rb,0.57948e-06_rb /)
      kao(:, 8, 5) = (/ &
        & 0.98362e-06_rb,0.96271e-06_rb,0.94180e-06_rb,0.92206e-06_rb,0.91105e-06_rb /)
      kao(:, 9, 5) = (/ &
        & 0.12061e-05_rb,0.11895e-05_rb,0.11564e-05_rb,0.11296e-05_rb,0.11110e-05_rb /)
      kao(:,10, 5) = (/ &
        & 0.12958e-05_rb,0.12694e-05_rb,0.12425e-05_rb,0.12153e-05_rb,0.11880e-05_rb /)
      kao(:,11, 5) = (/ &
        & 0.12962e-05_rb,0.12698e-05_rb,0.12429e-05_rb,0.12156e-05_rb,0.11883e-05_rb /)
      kao(:,12, 5) = (/ &
        & 0.12964e-05_rb,0.12701e-05_rb,0.12431e-05_rb,0.12158e-05_rb,0.11885e-05_rb /)
      kao(:,13, 5) = (/ &
        & 0.12966e-05_rb,0.12702e-05_rb,0.12433e-05_rb,0.12160e-05_rb,0.11886e-05_rb /)
      kao(:, 1, 6) = (/ &
        & 0.73925e-07_rb,0.70231e-07_rb,0.21454e-06_rb,0.63477e-07_rb,0.60912e-07_rb /)
      kao(:, 2, 6) = (/ &
        & 0.67794e-07_rb,0.65807e-07_rb,0.13854e-06_rb,0.13061e-06_rb,0.59361e-07_rb /)
      kao(:, 3, 6) = (/ &
        & 0.98353e-07_rb,0.95275e-07_rb,0.92426e-07_rb,0.15768e-06_rb,0.87986e-07_rb /)
      kao(:, 4, 6) = (/ &
        & 0.15855e-06_rb,0.15394e-06_rb,0.14948e-06_rb,0.14655e-06_rb,0.23172e-06_rb /)
      kao(:, 5, 6) = (/ &
        & 0.27764e-06_rb,0.26941e-06_rb,0.26299e-06_rb,0.25975e-06_rb,0.40526e-06_rb /)
      kao(:, 6, 6) = (/ &
        & 0.45469e-06_rb,0.44417e-06_rb,0.43276e-06_rb,0.42440e-06_rb,0.41489e-06_rb /)
      kao(:, 7, 6) = (/ &
        & 0.71540e-06_rb,0.71291e-06_rb,0.70656e-06_rb,0.69823e-06_rb,0.68342e-06_rb /)
      kao(:, 8, 6) = (/ &
        & 0.79651e-06_rb,0.79807e-06_rb,0.80621e-06_rb,0.80941e-06_rb,0.79835e-06_rb /)
      kao(:, 9, 6) = (/ &
        & 0.18716e-06_rb,0.16713e-06_rb,0.14725e-06_rb,0.13728e-06_rb,0.11763e-06_rb /)
      kao(:,10, 6) = (/ &
        & 0.92638e-07_rb,0.86207e-07_rb,0.80877e-07_rb,0.70432e-07_rb,0.64517e-07_rb /)
      kao(:,11, 6) = (/ &
        & 0.13396e-06_rb,0.12820e-06_rb,0.12387e-06_rb,0.10427e-06_rb,0.94091e-07_rb /)
      kao(:,12, 6) = (/ &
        & 0.14877e-06_rb,0.14827e-06_rb,0.14350e-06_rb,0.12154e-06_rb,0.10552e-06_rb /)
      kao(:,13, 6) = (/ &
        & 0.15437e-06_rb,0.15323e-06_rb,0.14992e-06_rb,0.12715e-06_rb,0.10933e-06_rb /)
      kao(:, 1, 7) = (/ &
        & 0.72717e-06_rb,0.70656e-06_rb,0.13933e-05_rb,0.66449e-06_rb,0.64269e-06_rb /)
      kao(:, 2, 7) = (/ &
        & 0.52595e-06_rb,0.50791e-06_rb,0.11171e-05_rb,0.10538e-05_rb,0.45644e-06_rb /)
      kao(:, 3, 7) = (/ &
        & 0.29919e-06_rb,0.29227e-06_rb,0.28284e-06_rb,0.65215e-06_rb,0.26347e-06_rb /)
      kao(:, 4, 7) = (/ &
        & 0.27961e-06_rb,0.27579e-06_rb,0.27068e-06_rb,0.26343e-06_rb,0.41265e-06_rb /)
      kao(:, 5, 7) = (/ &
        & 0.37031e-06_rb,0.36318e-06_rb,0.35475e-06_rb,0.34488e-06_rb,0.53740e-06_rb /)
      kao(:, 6, 7) = (/ &
        & 0.53195e-06_rb,0.52692e-06_rb,0.52224e-06_rb,0.51934e-06_rb,0.51146e-06_rb /)
      kao(:, 7, 7) = (/ &
        & 0.83043e-06_rb,0.84552e-06_rb,0.84833e-06_rb,0.82800e-06_rb,0.80930e-06_rb /)
      kao(:, 8, 7) = (/ &
        & 0.14910e-05_rb,0.15179e-05_rb,0.15248e-05_rb,0.15091e-05_rb,0.14853e-05_rb /)
      kao(:, 9, 7) = (/ &
        & 0.37340e-05_rb,0.37823e-05_rb,0.38311e-05_rb,0.38453e-05_rb,0.38567e-05_rb /)
      kao(:,10, 7) = (/ &
        & 0.86791e-05_rb,0.89697e-05_rb,0.92118e-05_rb,0.93991e-05_rb,0.95564e-05_rb /)
      kao(:,11, 7) = (/ &
        & 0.11878e-04_rb,0.12201e-04_rb,0.12588e-04_rb,0.12897e-04_rb,0.13151e-04_rb /)
      kao(:,12, 7) = (/ &
        & 0.13192e-04_rb,0.13732e-04_rb,0.14137e-04_rb,0.14465e-04_rb,0.14643e-04_rb /)
      kao(:,13, 7) = (/ &
        & 0.13716e-04_rb,0.14229e-04_rb,0.14617e-04_rb,0.14944e-04_rb,0.15182e-04_rb /)
      kao(:, 1, 8) = (/ &
        & 0.39538e-05_rb,0.38949e-05_rb,0.56188e-05_rb,0.37475e-05_rb,0.36648e-05_rb /)
      kao(:, 2, 8) = (/ &
        & 0.34231e-05_rb,0.33633e-05_rb,0.51877e-05_rb,0.50048e-05_rb,0.31425e-05_rb /)
      kao(:, 3, 8) = (/ &
        & 0.28073e-05_rb,0.27497e-05_rb,0.26875e-05_rb,0.44405e-05_rb,0.25492e-05_rb /)
      kao(:, 4, 8) = (/ &
        & 0.19229e-05_rb,0.18818e-05_rb,0.18382e-05_rb,0.17896e-05_rb,0.33073e-05_rb /)
      kao(:, 5, 8) = (/ &
        & 0.11453e-05_rb,0.11293e-05_rb,0.11095e-05_rb,0.10866e-05_rb,0.19344e-05_rb /)
      kao(:, 6, 8) = (/ &
        & 0.14565e-05_rb,0.14517e-05_rb,0.14369e-05_rb,0.14141e-05_rb,0.13944e-05_rb /)
      kao(:, 7, 8) = (/ &
        & 0.23228e-05_rb,0.22753e-05_rb,0.22395e-05_rb,0.22124e-05_rb,0.21731e-05_rb /)
      kao(:, 8, 8) = (/ &
        & 0.34877e-05_rb,0.34362e-05_rb,0.33796e-05_rb,0.33389e-05_rb,0.32924e-05_rb /)
      kao(:, 9, 8) = (/ &
        & 0.63448e-05_rb,0.63701e-05_rb,0.63619e-05_rb,0.62632e-05_rb,0.61645e-05_rb /)
      kao(:,10, 8) = (/ &
        & 0.12155e-04_rb,0.11880e-04_rb,0.11762e-04_rb,0.11759e-04_rb,0.11651e-04_rb /)
      kao(:,11, 8) = (/ &
        & 0.14093e-04_rb,0.13835e-04_rb,0.13547e-04_rb,0.13205e-04_rb,0.12690e-04_rb /)
      kao(:,12, 8) = (/ &
        & 0.14428e-04_rb,0.14056e-04_rb,0.13932e-04_rb,0.13396e-04_rb,0.12885e-04_rb /)
      kao(:,13, 8) = (/ &
        & 0.15229e-04_rb,0.14534e-04_rb,0.13849e-04_rb,0.13292e-04_rb,0.12704e-04_rb /)
      kao(:, 1, 9) = (/ &
        & 0.19250e-04_rb,0.19148e-04_rb,0.21702e-04_rb,0.18906e-04_rb,0.18761e-04_rb /)
      kao(:, 2, 9) = (/ &
        & 0.18132e-04_rb,0.18040e-04_rb,0.20884e-04_rb,0.20523e-04_rb,0.17656e-04_rb /)
      kao(:, 3, 9) = (/ &
        & 0.16928e-04_rb,0.16843e-04_rb,0.16742e-04_rb,0.19715e-04_rb,0.16470e-04_rb /)
      kao(:, 4, 9) = (/ &
        & 0.15526e-04_rb,0.15463e-04_rb,0.15377e-04_rb,0.15268e-04_rb,0.18367e-04_rb /)
      kao(:, 5, 9) = (/ &
        & 0.13545e-04_rb,0.13511e-04_rb,0.13455e-04_rb,0.13362e-04_rb,0.16722e-04_rb /)
      kao(:, 6, 9) = (/ &
        & 0.97183e-05_rb,0.97218e-05_rb,0.97084e-05_rb,0.96717e-05_rb,0.96030e-05_rb /)
      kao(:, 7, 9) = (/ &
        & 0.50307e-05_rb,0.50984e-05_rb,0.51628e-05_rb,0.52093e-05_rb,0.52354e-05_rb /)
      kao(:, 8, 9) = (/ &
        & 0.45837e-05_rb,0.45939e-05_rb,0.45938e-05_rb,0.45639e-05_rb,0.45109e-05_rb /)
      kao(:, 9, 9) = (/ &
        & 0.12254e-04_rb,0.12319e-04_rb,0.12397e-04_rb,0.12584e-04_rb,0.12620e-04_rb /)
      kao(:,10, 9) = (/ &
        & 0.21545e-04_rb,0.21836e-04_rb,0.21718e-04_rb,0.21511e-04_rb,0.21211e-04_rb /)
      kao(:,11, 9) = (/ &
        & 0.20079e-04_rb,0.19539e-04_rb,0.18859e-04_rb,0.18393e-04_rb,0.18181e-04_rb /)
      kao(:,12, 9) = (/ &
        & 0.17115e-04_rb,0.16357e-04_rb,0.15410e-04_rb,0.15220e-04_rb,0.15207e-04_rb /)
      kao(:,13, 9) = (/ &
        & 0.14935e-04_rb,0.14679e-04_rb,0.14593e-04_rb,0.14448e-04_rb,0.14436e-04_rb /)
      kao(:, 1,10) = (/ &
        & 0.53569e-04_rb,0.53042e-04_rb,0.55454e-04_rb,0.52098e-04_rb,0.51678e-04_rb /)
      kao(:, 2,10) = (/ &
        & 0.52196e-04_rb,0.51739e-04_rb,0.54777e-04_rb,0.54075e-04_rb,0.50624e-04_rb /)
      kao(:, 3,10) = (/ &
        & 0.50339e-04_rb,0.50046e-04_rb,0.49769e-04_rb,0.53168e-04_rb,0.49370e-04_rb /)
      kao(:, 4,10) = (/ &
        & 0.48505e-04_rb,0.48316e-04_rb,0.48143e-04_rb,0.47993e-04_rb,0.51621e-04_rb /)
      kao(:, 5,10) = (/ &
        & 0.46313e-04_rb,0.46267e-04_rb,0.46119e-04_rb,0.46064e-04_rb,0.50279e-04_rb /)
      kao(:, 6,10) = (/ &
        & 0.42662e-04_rb,0.42818e-04_rb,0.42935e-04_rb,0.43007e-04_rb,0.43099e-04_rb /)
      kao(:, 7,10) = (/ &
        & 0.35762e-04_rb,0.36149e-04_rb,0.36450e-04_rb,0.36639e-04_rb,0.36887e-04_rb /)
      kao(:, 8,10) = (/ &
        & 0.13516e-05_rb,0.18607e-05_rb,0.23061e-05_rb,0.27339e-05_rb,0.36516e-05_rb /)
      kao(:, 9,10) = (/ &
        & 0.36432e-05_rb,0.40739e-05_rb,0.43830e-05_rb,0.41136e-05_rb,0.43128e-05_rb /)
      kao(:,10,10) = (/ &
        & 0.62049e-05_rb,0.69116e-05_rb,0.73244e-05_rb,0.65087e-05_rb,0.78951e-05_rb /)
      kao(:,11,10) = (/ &
        & 0.32156e-05_rb,0.38834e-05_rb,0.41231e-05_rb,0.43386e-05_rb,0.43405e-05_rb /)
      kao(:,12,10) = (/ &
        & 0.22152e-05_rb,0.26754e-05_rb,0.31971e-05_rb,0.34911e-05_rb,0.37935e-05_rb /)
      kao(:,13,10) = (/ &
        & 0.19792e-05_rb,0.26543e-05_rb,0.31511e-05_rb,0.34597e-05_rb,0.40624e-05_rb /)
      kao(:, 1,11) = (/ &
        & 0.75384e-04_rb,0.75103e-04_rb,0.77406e-04_rb,0.74222e-04_rb,0.73734e-04_rb /)
      kao(:, 2,11) = (/ &
        & 0.75458e-04_rb,0.75244e-04_rb,0.77778e-04_rb,0.77018e-04_rb,0.73942e-04_rb /)
      kao(:, 3,11) = (/ &
        & 0.75023e-04_rb,0.74844e-04_rb,0.74477e-04_rb,0.77271e-04_rb,0.73633e-04_rb /)
      kao(:, 4,11) = (/ &
        & 0.73633e-04_rb,0.73539e-04_rb,0.73257e-04_rb,0.72934e-04_rb,0.76232e-04_rb /)
      kao(:, 5,11) = (/ &
        & 0.71348e-04_rb,0.71322e-04_rb,0.71227e-04_rb,0.71069e-04_rb,0.75258e-04_rb /)
      kao(:, 6,11) = (/ &
        & 0.67784e-04_rb,0.67873e-04_rb,0.67974e-04_rb,0.67924e-04_rb,0.67903e-04_rb /)
      kao(:, 7,11) = (/ &
        & 0.61855e-04_rb,0.61922e-04_rb,0.61973e-04_rb,0.62206e-04_rb,0.62496e-04_rb /)
      kao(:, 8,11) = (/ &
        & 0.36622e-04_rb,0.37413e-04_rb,0.38740e-04_rb,0.40550e-04_rb,0.41833e-04_rb /)
      kao(:, 9,11) = (/ &
        & 0.28544e-05_rb,0.28831e-05_rb,0.31445e-05_rb,0.32900e-05_rb,0.27967e-05_rb /)
      kao(:,10,11) = (/ &
        & 0.53755e-05_rb,0.42123e-05_rb,0.51154e-05_rb,0.63481e-05_rb,0.54219e-05_rb /)
      kao(:,11,11) = (/ &
        & 0.12605e-05_rb,0.14078e-05_rb,0.19167e-05_rb,0.23729e-05_rb,0.30161e-05_rb /)
      kao(:,12,11) = (/ &
        & 0.11370e-05_rb,0.91524e-06_rb,0.11150e-05_rb,0.14746e-05_rb,0.20128e-05_rb /)
      kao(:,13,11) = (/ &
        & 0.10511e-05_rb,0.10014e-05_rb,0.11405e-05_rb,0.13852e-05_rb,0.15576e-05_rb /)
      kao(:, 1,12) = (/ &
        & 0.11184e-03_rb,0.11117e-03_rb,0.11327e-03_rb,0.10989e-03_rb,0.10910e-03_rb /)
      kao(:, 2,12) = (/ &
        & 0.11379e-03_rb,0.11322e-03_rb,0.11555e-03_rb,0.11462e-03_rb,0.11135e-03_rb /)
      kao(:, 3,12) = (/ &
        & 0.11508e-03_rb,0.11459e-03_rb,0.11421e-03_rb,0.11671e-03_rb,0.11339e-03_rb /)
      kao(:, 4,12) = (/ &
        & 0.11596e-03_rb,0.11563e-03_rb,0.11538e-03_rb,0.11511e-03_rb,0.11770e-03_rb /)
      kao(:, 5,12) = (/ &
        & 0.11597e-03_rb,0.11581e-03_rb,0.11569e-03_rb,0.11553e-03_rb,0.11890e-03_rb /)
      kao(:, 6,12) = (/ &
        & 0.11443e-03_rb,0.11445e-03_rb,0.11443e-03_rb,0.11443e-03_rb,0.11438e-03_rb /)
      kao(:, 7,12) = (/ &
        & 0.10852e-03_rb,0.10888e-03_rb,0.10912e-03_rb,0.10934e-03_rb,0.10942e-03_rb /)
      kao(:, 8,12) = (/ &
        & 0.93194e-04_rb,0.94766e-04_rb,0.95355e-04_rb,0.95090e-04_rb,0.94926e-04_rb /)
      kao(:, 9,12) = (/ &
        & 0.11836e-05_rb,0.16115e-05_rb,0.12883e-05_rb,0.14202e-05_rb,0.16541e-05_rb /)
      kao(:,10,12) = (/ &
        & 0.18748e-05_rb,0.34401e-05_rb,0.39984e-05_rb,0.44576e-05_rb,0.33683e-05_rb /)
      kao(:,11,12) = (/ &
        & 0.29890e-06_rb,0.48741e-06_rb,0.66276e-06_rb,0.99698e-06_rb,0.19230e-05_rb /)
      kao(:,12,12) = (/ &
        & 0.15034e-06_rb,0.39966e-06_rb,0.56523e-06_rb,0.70494e-06_rb,0.10046e-05_rb /)
      kao(:,13,12) = (/ &
        & 0.15016e-06_rb,0.25751e-06_rb,0.48928e-06_rb,0.63534e-06_rb,0.93575e-06_rb /)
      kao(:, 1,13) = (/ &
        & 0.17305e-03_rb,0.17234e-03_rb,0.17389e-03_rb,0.17055e-03_rb,0.16974e-03_rb /)
      kao(:, 2,13) = (/ &
        & 0.18170e-03_rb,0.18075e-03_rb,0.18265e-03_rb,0.18138e-03_rb,0.17772e-03_rb /)
      kao(:, 3,13) = (/ &
        & 0.18990e-03_rb,0.18892e-03_rb,0.18776e-03_rb,0.18950e-03_rb,0.18494e-03_rb /)
      kao(:, 4,13) = (/ &
        & 0.19649e-03_rb,0.19552e-03_rb,0.19424e-03_rb,0.19281e-03_rb,0.19464e-03_rb /)
      kao(:, 5,13) = (/ &
        & 0.20197e-03_rb,0.20109e-03_rb,0.19993e-03_rb,0.19856e-03_rb,0.20092e-03_rb /)
      kao(:, 6,13) = (/ &
        & 0.20595e-03_rb,0.20549e-03_rb,0.20452e-03_rb,0.20331e-03_rb,0.20199e-03_rb /)
      kao(:, 7,13) = (/ &
        & 0.20703e-03_rb,0.20710e-03_rb,0.20649e-03_rb,0.20552e-03_rb,0.20428e-03_rb /)
      kao(:, 8,13) = (/ &
        & 0.19874e-03_rb,0.19767e-03_rb,0.19696e-03_rb,0.19655e-03_rb,0.19591e-03_rb /)
      kao(:, 9,13) = (/ &
        & 0.20434e-04_rb,0.23398e-04_rb,0.27400e-04_rb,0.32409e-04_rb,0.38451e-04_rb /)
      kao(:,10,13) = (/ &
        & 0.18617e-05_rb,0.99513e-06_rb,0.10554e-05_rb,0.16516e-05_rb,0.37792e-05_rb /)
      kao(:,11,13) = (/ &
        & 0.12517e-06_rb,0.29518e-06_rb,0.77058e-06_rb,0.11660e-05_rb,0.15349e-05_rb /)
      kao(:,12,13) = (/ &
        & 0.12734e-06_rb,0.36524e-06_rb,0.66699e-06_rb,0.10362e-05_rb,0.14158e-05_rb /)
      kao(:,13,13) = (/ &
        & 0.12431e-06_rb,0.39389e-06_rb,0.67331e-06_rb,0.10292e-05_rb,0.14448e-05_rb /)
      kao(:, 1,14) = (/ &
        & 0.29365e-03_rb,0.29046e-03_rb,0.29008e-03_rb,0.28509e-03_rb,0.28286e-03_rb /)
      kao(:, 2,14) = (/ &
        & 0.31990e-03_rb,0.31668e-03_rb,0.31617e-03_rb,0.31332e-03_rb,0.30885e-03_rb /)
      kao(:, 3,14) = (/ &
        & 0.34787e-03_rb,0.34432e-03_rb,0.34112e-03_rb,0.34052e-03_rb,0.33589e-03_rb /)
      kao(:, 4,14) = (/ &
        & 0.37401e-03_rb,0.37027e-03_rb,0.36696e-03_rb,0.36394e-03_rb,0.36355e-03_rb /)
      kao(:, 5,14) = (/ &
        & 0.39840e-03_rb,0.39446e-03_rb,0.39082e-03_rb,0.38763e-03_rb,0.38760e-03_rb /)
      kao(:, 6,14) = (/ &
        & 0.42165e-03_rb,0.41729e-03_rb,0.41335e-03_rb,0.41006e-03_rb,0.40721e-03_rb /)
      kao(:, 7,14) = (/ &
        & 0.44257e-03_rb,0.43782e-03_rb,0.43364e-03_rb,0.43014e-03_rb,0.42736e-03_rb /)
      kao(:, 8,14) = (/ &
        & 0.45299e-03_rb,0.44953e-03_rb,0.44586e-03_rb,0.44260e-03_rb,0.44006e-03_rb /)
      kao(:, 9,14) = (/ &
        & 0.40190e-03_rb,0.39751e-03_rb,0.39238e-03_rb,0.38812e-03_rb,0.38612e-03_rb /)
      kao(:,10,14) = (/ &
        & 0.64278e-05_rb,0.18248e-05_rb,0.16996e-05_rb,0.31086e-06_rb,0.16836e-06_rb /)
      kao(:,11,14) = (/ &
        & 0.14350e-05_rb,0.94778e-06_rb,0.41349e-06_rb,0.20817e-06_rb,0.20238e-06_rb /)
      kao(:,12,14) = (/ &
        & 0.16805e-05_rb,0.15323e-05_rb,0.62348e-06_rb,0.99743e-07_rb,0.12977e-06_rb /)
      kao(:,13,14) = (/ &
        & 0.16858e-05_rb,0.17103e-05_rb,0.80574e-06_rb,0.15825e-06_rb,0.15032e-06_rb /)
      kao(:, 1,15) = (/ &
        & 0.52181e-03_rb,0.51578e-03_rb,0.51251e-03_rb,0.50356e-03_rb,0.49731e-03_rb /)
      kao(:, 2,15) = (/ &
        & 0.59491e-03_rb,0.58822e-03_rb,0.58413e-03_rb,0.57646e-03_rb,0.56692e-03_rb /)
      kao(:, 3,15) = (/ &
        & 0.67653e-03_rb,0.66881e-03_rb,0.66126e-03_rb,0.65540e-03_rb,0.64461e-03_rb /)
      kao(:, 4,15) = (/ &
        & 0.76388e-03_rb,0.75456e-03_rb,0.74556e-03_rb,0.73649e-03_rb,0.72840e-03_rb /)
      kao(:, 5,15) = (/ &
        & 0.85507e-03_rb,0.84417e-03_rb,0.83378e-03_rb,0.82338e-03_rb,0.81349e-03_rb /)
      kao(:, 6,15) = (/ &
        & 0.95034e-03_rb,0.93798e-03_rb,0.92553e-03_rb,0.91287e-03_rb,0.89957e-03_rb /)
      kao(:, 7,15) = (/ &
        & 0.10496e-02_rb,0.10352e-02_rb,0.10206e-02_rb,0.10054e-02_rb,0.98958e-03_rb /)
      kao(:, 8,15) = (/ &
        & 0.11507e-02_rb,0.11337e-02_rb,0.11169e-02_rb,0.10991e-02_rb,0.10806e-02_rb /)
      kao(:, 9,15) = (/ &
        & 0.12408e-02_rb,0.12207e-02_rb,0.11996e-02_rb,0.11773e-02_rb,0.11531e-02_rb /)
      kao(:,10,15) = (/ &
        & 0.12042e-03_rb,0.11501e-03_rb,0.11424e-03_rb,0.11450e-03_rb,0.13219e-03_rb /)
      kao(:,11,15) = (/ &
        & 0.68914e-06_rb,0.83960e-06_rb,0.74591e-06_rb,0.18660e-05_rb,0.32503e-05_rb /)
      kao(:,12,15) = (/ &
        & 0.35963e-07_rb,0.46256e-06_rb,0.56223e-06_rb,0.98816e-06_rb,0.92366e-06_rb /)
      kao(:,13,15) = (/ &
        & 0.36605e-07_rb,0.56591e-06_rb,0.84008e-06_rb,0.86042e-06_rb,0.68452e-06_rb /)
      kao(:, 1,16) = (/ &
        & 0.76517e-03_rb,0.75944e-03_rb,0.76010e-03_rb,0.76100e-03_rb,0.76498e-03_rb /)
      kao(:, 2,16) = (/ &
        & 0.92375e-03_rb,0.91357e-03_rb,0.90997e-03_rb,0.90997e-03_rb,0.90993e-03_rb /)
      kao(:, 3,16) = (/ &
        & 0.11142e-02_rb,0.10974e-02_rb,0.10835e-02_rb,0.10789e-02_rb,0.10748e-02_rb /)
      kao(:, 4,16) = (/ &
        & 0.13278e-02_rb,0.13025e-02_rb,0.12802e-02_rb,0.12631e-02_rb,0.12539e-02_rb /)
      kao(:, 5,16) = (/ &
        & 0.15712e-02_rb,0.15343e-02_rb,0.15017e-02_rb,0.14713e-02_rb,0.14497e-02_rb /)
      kao(:, 6,16) = (/ &
        & 0.18525e-02_rb,0.17982e-02_rb,0.17525e-02_rb,0.17101e-02_rb,0.16714e-02_rb /)
      kao(:, 7,16) = (/ &
        & 0.21731e-02_rb,0.20986e-02_rb,0.20340e-02_rb,0.19757e-02_rb,0.19210e-02_rb /)
      kao(:, 8,16) = (/ &
        & 0.25325e-02_rb,0.24346e-02_rb,0.23473e-02_rb,0.22687e-02_rb,0.21950e-02_rb /)
      kao(:, 9,16) = (/ &
        & 0.29269e-02_rb,0.28006e-02_rb,0.26863e-02_rb,0.25805e-02_rb,0.24878e-02_rb /)
      kao(:,10,16) = (/ &
        & 0.29442e-02_rb,0.27008e-02_rb,0.23913e-02_rb,0.21437e-02_rb,0.18865e-02_rb /)
      kao(:,11,16) = (/ &
        & 0.23220e-05_rb,0.22310e-04_rb,0.48349e-04_rb,0.67183e-04_rb,0.88908e-04_rb /)
      kao(:,12,16) = (/ &
        & 0.22857e-05_rb,0.11848e-04_rb,0.42066e-04_rb,0.67613e-04_rb,0.86033e-04_rb /)
      kao(:,13,16) = (/ &
        & 0.22823e-05_rb,0.69105e-05_rb,0.36212e-04_rb,0.66247e-04_rb,0.85488e-04_rb /)
  
      end subroutine sw_kgb25

! **************************************************************************
      subroutine sw_kgb26
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg26, only : sfluxrefo, raylo, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:) = (/ &
!         &     129.462_rb, 15*0._rb /)
        &   29.0079_rb,  28.4088_rb,     20.3099_rb,  13.0283_rb &
        &,  11.8619_rb,  9.95840_rb,     6.68696_rb,  5.38987_rb &
        &,  3.49829_rb, 0.407693_rb,    0.299027_rb, 0.236827_rb &
        &, 0.188502_rb, 0.163489_rb, 4.64335e-02_rb, 2.72662e-03_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:) = (/ &
        & 3.04845E+01_rb, 2.82835E+01_rb, 2.06520E+01_rb, 1.33606E+01_rb,&
        & 1.19253E+01_rb, 9.46266E+00_rb, 6.49295E+00_rb, 4.95470E+00_rb,&
        & 3.04469E+00_rb, 3.68830E-01_rb, 2.69394E-01_rb, 2.19130E-01_rb,&
        & 1.67527E-01_rb, 1.51647E-01_rb, 3.99119E-02_rb, 0.00000E+00_rb/)
! Solar variability components: facular brightening
      facbrghto(:) = (/ &
        & 3.63487E-02_rb, 3.07171E-02_rb, 2.93798E-02_rb, 3.97712E-02_rb,&
        & 2.31774E-02_rb, 1.35487E-02_rb, 1.22683E-02_rb, 8.39258E-03_rb,&
        & 5.82190E-03_rb, 6.43794E-04_rb, 5.41994E-04_rb, 4.30282E-04_rb,&
        & 3.14737E-04_rb, 1.99894E-04_rb, 7.52298E-05_rb, 1.04596E-05_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:) = (/ &
        &-1.85717E-02_rb,-1.87356E-02_rb,-1.39340E-02_rb,-8.79268E-03_rb,&
        &-8.22724E-03_rb,-6.20947E-03_rb,-4.30472E-03_rb,-3.59906E-03_rb,&
        &-2.11540E-03_rb,-2.34184E-04_rb,-1.94365E-04_rb,-1.52493E-04_rb,&
        &-1.10488E-04_rb,-6.94776E-05_rb,-2.59857E-05_rb,-3.60660E-06_rb/)

! Rayleigh extinction coefficient at all v 
      raylo(:) = (/ &
        &  1.21263e-06_rb,1.43428e-06_rb,1.67677e-06_rb,1.93255e-06_rb &
        &, 2.19177e-06_rb,2.44195e-06_rb,2.66926e-06_rb,2.85990e-06_rb &
        &, 3.00380e-06_rb,3.06996e-06_rb,3.08184e-06_rb,3.09172e-06_rb &
        &, 3.09938e-06_rb,3.10456e-06_rb,3.10727e-06_rb,3.10818e-06_rb /)

      end subroutine sw_kgb26

! **************************************************************************
      subroutine sw_kgb27
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg27, only : kao, kbo, sfluxrefo, raylo, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
! The following values were obtained using the "low resolution"
! version of the Kurucz solar source function.  For unknown reasons,
! the total irradiance in this band differs from the corresponding
! total in the "high-resolution" version of the Kurucz function.
! Therefore, these values are scaled later by the factor SCALEKUR.
      sfluxrefo(:) = (/ &
        & 14.0526_rb    , 11.4794_rb    , 8.72590_rb    , 5.56966_rb    , &
        & 3.80927_rb    , 1.57690_rb    , 1.15099_rb    , 1.10012_rb    , &
        & 0.658212_rb   , 5.86859e-02_rb, 5.56186e-02_rb, 4.68040e-02_rb, &
        & 3.64897e-02_rb, 3.58053e-02_rb, 1.38130e-02_rb, 1.90193e-03_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:) = (/ &
        & 1.46986E+01_rb, 1.06945E+01_rb, 7.67515E+00_rb, 5.14238E+00_rb,&
        & 3.29604E+00_rb, 1.44947E+00_rb, 1.02329E+00_rb, 9.82851E-01_rb,&
        & 6.02371E-01_rb, 5.25598E-02_rb, 5.70254E-02_rb, 4.26860E-02_rb,&
        & 2.73350E-02_rb, 1.15740E-02_rb, 5.74391E-03_rb, 8.86142E-04_rb/)
! Solar variability components: facular brightening
      facbrghto(:) = (/ &
        & 2.58280E-02_rb, 2.06263E-02_rb, 2.04012E-02_rb, 1.66200E-02_rb,&
        & 9.18474E-03_rb, 9.50986E-03_rb, 4.83035E-03_rb, 3.79881E-03_rb,&
        & 2.67988E-03_rb, 2.81871E-04_rb, 2.62704E-04_rb, 2.12805E-04_rb,&
        & 1.58723E-04_rb, 1.02352E-04_rb, 3.85458E-05_rb, 5.42318E-06_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:) = (/ &
        &-1.07802E-02_rb,-8.60331E-03_rb,-6.34647E-03_rb,-4.37003E-03_rb,&
        &-2.52409E-03_rb,-8.79192E-04_rb,-7.06100E-04_rb,-7.53687E-04_rb,&
        &-4.54109E-04_rb,-3.56957E-05_rb,-3.85754E-05_rb,-3.18825E-05_rb,&
        &-2.41248E-05_rb,-1.57380E-05_rb,-5.92491E-06_rb,-8.33126E-07_rb/)

! Rayleigh extinction coefficient at v = 2925 cm-1.
      raylo(:) = (/ &
        & 3.44534e-06_rb,4.14480e-06_rb,4.95069e-06_rb,5.81204e-06_rb, &
        & 6.69748e-06_rb,7.56488e-06_rb,8.36344e-06_rb,9.04135e-06_rb, &
        & 9.58324e-06_rb,9.81542e-06_rb,9.75119e-06_rb,9.74533e-06_rb, &
        & 9.74139e-06_rb,9.73525e-06_rb,9.73577e-06_rb,9.73618e-06_rb /)

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1) = (/ &
        & 0.22907e+00_rb,0.25625e+00_rb,0.28779e+00_rb,0.32376e+00_rb,0.36426e+00_rb /)
      kao(:, 2, 1) = (/ &
        & 0.21913e+00_rb,0.24445e+00_rb,0.27422e+00_rb,0.30832e+00_rb,0.34694e+00_rb /)
      kao(:, 3, 1) = (/ &
        & 0.20602e+00_rb,0.22864e+00_rb,0.25574e+00_rb,0.28721e+00_rb,0.32310e+00_rb /)
      kao(:, 4, 1) = (/ &
        & 0.19379e+00_rb,0.21352e+00_rb,0.23771e+00_rb,0.26643e+00_rb,0.29940e+00_rb /)
      kao(:, 5, 1) = (/ &
        & 0.18369e+00_rb,0.20061e+00_rb,0.22200e+00_rb,0.24786e+00_rb,0.27816e+00_rb /)
      kao(:, 6, 1) = (/ &
        & 0.17517e+00_rb,0.18920e+00_rb,0.20771e+00_rb,0.23069e+00_rb,0.25817e+00_rb /)
      kao(:, 7, 1) = (/ &
        & 0.16888e+00_rb,0.17988e+00_rb,0.19559e+00_rb,0.21577e+00_rb,0.24042e+00_rb /)
      kao(:, 8, 1) = (/ &
        & 0.16442e+00_rb,0.17273e+00_rb,0.18565e+00_rb,0.20315e+00_rb,0.22512e+00_rb /)
      kao(:, 9, 1) = (/ &
        & 0.16159e+00_rb,0.16738e+00_rb,0.17752e+00_rb,0.19242e+00_rb,0.21179e+00_rb /)
      kao(:,10, 1) = (/ &
        & 0.16037e+00_rb,0.16402e+00_rb,0.17204e+00_rb,0.18463e+00_rb,0.20183e+00_rb /)
      kao(:,11, 1) = (/ &
        & 0.16036e+00_rb,0.16399e+00_rb,0.17198e+00_rb,0.18455e+00_rb,0.20173e+00_rb /)
      kao(:,12, 1) = (/ &
        & 0.16036e+00_rb,0.16399e+00_rb,0.17198e+00_rb,0.18455e+00_rb,0.20173e+00_rb /)
      kao(:,13, 1) = (/ &
        & 0.16036e+00_rb,0.16399e+00_rb,0.17198e+00_rb,0.18455e+00_rb,0.20173e+00_rb /)
      kao(:, 1, 2) = (/ &
        & 0.20716e+01_rb,0.21925e+01_rb,0.23327e+01_rb,0.24921e+01_rb,0.26726e+01_rb /)
      kao(:, 2, 2) = (/ &
        & 0.20272e+01_rb,0.21401e+01_rb,0.22723e+01_rb,0.24238e+01_rb,0.25950e+01_rb /)
      kao(:, 3, 2) = (/ &
        & 0.19684e+01_rb,0.20697e+01_rb,0.21902e+01_rb,0.23301e+01_rb,0.24892e+01_rb /)
      kao(:, 4, 2) = (/ &
        & 0.19133e+01_rb,0.20021e+01_rb,0.21101e+01_rb,0.22375e+01_rb,0.23842e+01_rb /)
      kao(:, 5, 2) = (/ &
        & 0.18673e+01_rb,0.19441e+01_rb,0.20400e+01_rb,0.21553e+01_rb,0.22898e+01_rb /)
      kao(:, 6, 2) = (/ &
        & 0.18277e+01_rb,0.18925e+01_rb,0.19760e+01_rb,0.20788e+01_rb,0.22009e+01_rb /)
      kao(:, 7, 2) = (/ &
        & 0.17963e+01_rb,0.18497e+01_rb,0.19214e+01_rb,0.20121e+01_rb,0.21222e+01_rb /)
      kao(:, 8, 2) = (/ &
        & 0.17749e+01_rb,0.18156e+01_rb,0.18763e+01_rb,0.19555e+01_rb,0.20540e+01_rb /)
      kao(:, 9, 2) = (/ &
        & 0.17600e+01_rb,0.17885e+01_rb,0.18388e+01_rb,0.19071e+01_rb,0.19943e+01_rb /)
      kao(:,10, 2) = (/ &
        & 0.17526e+01_rb,0.17729e+01_rb,0.18121e+01_rb,0.18716e+01_rb,0.19496e+01_rb /)
      kao(:,11, 2) = (/ &
        & 0.17525e+01_rb,0.17727e+01_rb,0.18119e+01_rb,0.18712e+01_rb,0.19491e+01_rb /)
      kao(:,12, 2) = (/ &
        & 0.17525e+01_rb,0.17727e+01_rb,0.18119e+01_rb,0.18712e+01_rb,0.19491e+01_rb /)
      kao(:,13, 2) = (/ &
        & 0.17525e+01_rb,0.17727e+01_rb,0.18119e+01_rb,0.18712e+01_rb,0.19491e+01_rb /)
      kao(:, 1, 3) = (/ &
        & 0.12189e+02_rb,0.12619e+02_rb,0.13110e+02_rb,0.13663e+02_rb,0.14275e+02_rb /)
      kao(:, 2, 3) = (/ &
        & 0.12030e+02_rb,0.12434e+02_rb,0.12899e+02_rb,0.13427e+02_rb,0.14015e+02_rb /)
      kao(:, 3, 3) = (/ &
        & 0.11816e+02_rb,0.12183e+02_rb,0.12611e+02_rb,0.13101e+02_rb,0.13653e+02_rb /)
      kao(:, 4, 3) = (/ &
        & 0.11611e+02_rb,0.11938e+02_rb,0.12327e+02_rb,0.12777e+02_rb,0.13289e+02_rb /)
      kao(:, 5, 3) = (/ &
        & 0.11437e+02_rb,0.11726e+02_rb,0.12076e+02_rb,0.12488e+02_rb,0.12961e+02_rb /)
      kao(:, 6, 3) = (/ &
        & 0.11284e+02_rb,0.11533e+02_rb,0.11843e+02_rb,0.12215e+02_rb,0.12649e+02_rb /)
      kao(:, 7, 3) = (/ &
        & 0.11160e+02_rb,0.11370e+02_rb,0.11641e+02_rb,0.11975e+02_rb,0.12370e+02_rb /)
      kao(:, 8, 3) = (/ &
        & 0.11063e+02_rb,0.11237e+02_rb,0.11471e+02_rb,0.11768e+02_rb,0.12126e+02_rb /)
      kao(:, 9, 3) = (/ &
        & 0.10990e+02_rb,0.11129e+02_rb,0.11327e+02_rb,0.11588e+02_rb,0.11910e+02_rb /)
      kao(:,10, 3) = (/ &
        & 0.10945e+02_rb,0.11053e+02_rb,0.11223e+02_rb,0.11454e+02_rb,0.11746e+02_rb /)
      kao(:,11, 3) = (/ &
        & 0.10945e+02_rb,0.11052e+02_rb,0.11222e+02_rb,0.11452e+02_rb,0.11744e+02_rb /)
      kao(:,12, 3) = (/ &
        & 0.10945e+02_rb,0.11052e+02_rb,0.11222e+02_rb,0.11452e+02_rb,0.11744e+02_rb /)
      kao(:,13, 3) = (/ &
        & 0.10945e+02_rb,0.11052e+02_rb,0.11222e+02_rb,0.11452e+02_rb,0.11744e+02_rb /)
      kao(:, 1, 4) = (/ &
        & 0.55057e+02_rb,0.56143e+02_rb,0.57364e+02_rb,0.58719e+02_rb,0.60208e+02_rb /)
      kao(:, 2, 4) = (/ &
        & 0.54647e+02_rb,0.55677e+02_rb,0.56842e+02_rb,0.58142e+02_rb,0.59575e+02_rb /)
      kao(:, 3, 4) = (/ &
        & 0.54090e+02_rb,0.55039e+02_rb,0.56123e+02_rb,0.57342e+02_rb,0.58694e+02_rb /)
      kao(:, 4, 4) = (/ &
        & 0.53548e+02_rb,0.54410e+02_rb,0.55408e+02_rb,0.56539e+02_rb,0.57805e+02_rb /)
      kao(:, 5, 4) = (/ &
        & 0.53074e+02_rb,0.53853e+02_rb,0.54766e+02_rb,0.55813e+02_rb,0.56995e+02_rb /)
      kao(:, 6, 4) = (/ &
        & 0.52644e+02_rb,0.53336e+02_rb,0.54162e+02_rb,0.55123e+02_rb,0.56218e+02_rb /)
      kao(:, 7, 4) = (/ &
        & 0.52279e+02_rb,0.52887e+02_rb,0.53629e+02_rb,0.54505e+02_rb,0.55516e+02_rb /)
      kao(:, 8, 4) = (/ &
        & 0.51979e+02_rb,0.52507e+02_rb,0.53168e+02_rb,0.53965e+02_rb,0.54895e+02_rb /)
      kao(:, 9, 4) = (/ &
        & 0.51734e+02_rb,0.52183e+02_rb,0.52767e+02_rb,0.53485e+02_rb,0.54337e+02_rb /)
      kao(:,10, 4) = (/ &
        & 0.51565e+02_rb,0.51948e+02_rb,0.52467e+02_rb,0.53120e+02_rb,0.53907e+02_rb /)
      kao(:,11, 4) = (/ &
        & 0.51563e+02_rb,0.51946e+02_rb,0.52464e+02_rb,0.53116e+02_rb,0.53902e+02_rb /)
      kao(:,12, 4) = (/ &
        & 0.51563e+02_rb,0.51946e+02_rb,0.52464e+02_rb,0.53116e+02_rb,0.53902e+02_rb /)
      kao(:,13, 4) = (/ &
        & 0.51563e+02_rb,0.51946e+02_rb,0.52464e+02_rb,0.53116e+02_rb,0.53902e+02_rb /)
      kao(:, 1, 5) = (/ &
        & 0.17794e+03_rb,0.17973e+03_rb,0.18164e+03_rb,0.18366e+03_rb,0.18581e+03_rb /)
      kao(:, 2, 5) = (/ &
        & 0.17724e+03_rb,0.17898e+03_rb,0.18083e+03_rb,0.18281e+03_rb,0.18491e+03_rb /)
      kao(:, 3, 5) = (/ &
        & 0.17624e+03_rb,0.17791e+03_rb,0.17970e+03_rb,0.18160e+03_rb,0.18363e+03_rb /)
      kao(:, 4, 5) = (/ &
        & 0.17523e+03_rb,0.17682e+03_rb,0.17853e+03_rb,0.18036e+03_rb,0.18231e+03_rb /)
      kao(:, 5, 5) = (/ &
        & 0.17429e+03_rb,0.17581e+03_rb,0.17744e+03_rb,0.17920e+03_rb,0.18107e+03_rb /)
      kao(:, 6, 5) = (/ &
        & 0.17338e+03_rb,0.17482e+03_rb,0.17638e+03_rb,0.17805e+03_rb,0.17985e+03_rb /)
      kao(:, 7, 5) = (/ &
        & 0.17253e+03_rb,0.17390e+03_rb,0.17539e+03_rb,0.17699e+03_rb,0.17871e+03_rb /)
      kao(:, 8, 5) = (/ &
        & 0.17177e+03_rb,0.17307e+03_rb,0.17448e+03_rb,0.17601e+03_rb,0.17767e+03_rb /)
      kao(:, 9, 5) = (/ &
        & 0.17107e+03_rb,0.17230e+03_rb,0.17364e+03_rb,0.17511e+03_rb,0.17669e+03_rb /)
      kao(:,10, 5) = (/ &
        & 0.17052e+03_rb,0.17169e+03_rb,0.17298e+03_rb,0.17438e+03_rb,0.17591e+03_rb /)
      kao(:,11, 5) = (/ &
        & 0.17051e+03_rb,0.17168e+03_rb,0.17297e+03_rb,0.17437e+03_rb,0.17590e+03_rb /)
      kao(:,12, 5) = (/ &
        & 0.17051e+03_rb,0.17168e+03_rb,0.17297e+03_rb,0.17437e+03_rb,0.17590e+03_rb /)
      kao(:,13, 5) = (/ &
        & 0.17051e+03_rb,0.17168e+03_rb,0.17297e+03_rb,0.17437e+03_rb,0.17590e+03_rb /)
      kao(:, 1, 6) = (/ &
        & 0.37680e+05_rb,0.36801e+05_rb,0.36011e+05_rb,0.35298e+05_rb,0.34715e+05_rb /)
      kao(:, 2, 6) = (/ &
        & 0.39286e+03_rb,0.39463e+03_rb,0.39616e+03_rb,0.39748e+03_rb,0.39858e+03_rb /)
      kao(:, 3, 6) = (/ &
        & 0.39169e+03_rb,0.39359e+03_rb,0.39526e+03_rb,0.39671e+03_rb,0.39794e+03_rb /)
      kao(:, 4, 6) = (/ &
        & 0.39034e+03_rb,0.39239e+03_rb,0.39421e+03_rb,0.39580e+03_rb,0.39717e+03_rb /)
      kao(:, 5, 6) = (/ &
        & 0.38895e+03_rb,0.39113e+03_rb,0.39309e+03_rb,0.39483e+03_rb,0.39634e+03_rb /)
      kao(:, 6, 6) = (/ &
        & 0.38742e+03_rb,0.38975e+03_rb,0.39185e+03_rb,0.39373e+03_rb,0.39539e+03_rb /)
      kao(:, 7, 6) = (/ &
        & 0.38585e+03_rb,0.38832e+03_rb,0.39056e+03_rb,0.39258e+03_rb,0.39438e+03_rb /)
      kao(:, 8, 6) = (/ &
        & 0.38426e+03_rb,0.38686e+03_rb,0.38924e+03_rb,0.39140e+03_rb,0.39333e+03_rb /)
      kao(:, 9, 6) = (/ &
        & 0.38264e+03_rb,0.38538e+03_rb,0.38789e+03_rb,0.39017e+03_rb,0.39223e+03_rb /)
      kao(:,10, 6) = (/ &
        & 0.38123e+03_rb,0.38408e+03_rb,0.38670e+03_rb,0.38909e+03_rb,0.39126e+03_rb /)
      kao(:,11, 6) = (/ &
        & 0.38122e+03_rb,0.38406e+03_rb,0.38668e+03_rb,0.38908e+03_rb,0.39125e+03_rb /)
      kao(:,12, 6) = (/ &
        & 0.38122e+03_rb,0.38406e+03_rb,0.38668e+03_rb,0.38908e+03_rb,0.39125e+03_rb /)
      kao(:,13, 6) = (/ &
        & 0.38122e+03_rb,0.38406e+03_rb,0.38668e+03_rb,0.38908e+03_rb,0.39125e+03_rb /)
      kao(:, 1, 7) = (/ &
        & 0.18202e+07_rb,0.17771e+07_rb,0.17382e+07_rb,0.17034e+07_rb,0.16716e+07_rb /)
      kao(:, 2, 7) = (/ &
        & 0.63027e+03_rb,0.63194e+03_rb,0.63388e+03_rb,0.63607e+03_rb,0.63849e+03_rb /)
      kao(:, 3, 7) = (/ &
        & 0.62938e+03_rb,0.63090e+03_rb,0.63268e+03_rb,0.63472e+03_rb,0.63701e+03_rb /)
      kao(:, 4, 7) = (/ &
        & 0.62853e+03_rb,0.62989e+03_rb,0.63150e+03_rb,0.63337e+03_rb,0.63550e+03_rb /)
      kao(:, 5, 7) = (/ &
        & 0.62781e+03_rb,0.62901e+03_rb,0.63046e+03_rb,0.63217e+03_rb,0.63413e+03_rb /)
      kao(:, 6, 7) = (/ &
        & 0.62717e+03_rb,0.62821e+03_rb,0.62950e+03_rb,0.63104e+03_rb,0.63283e+03_rb /)
      kao(:, 7, 7) = (/ &
        & 0.62666e+03_rb,0.62753e+03_rb,0.62867e+03_rb,0.63005e+03_rb,0.63168e+03_rb /)
      kao(:, 8, 7) = (/ &
        & 0.62627e+03_rb,0.62698e+03_rb,0.62795e+03_rb,0.62919e+03_rb,0.63067e+03_rb /)
      kao(:, 9, 7) = (/ &
        & 0.62600e+03_rb,0.62653e+03_rb,0.62735e+03_rb,0.62844e+03_rb,0.62978e+03_rb /)
      kao(:,10, 7) = (/ &
        & 0.62583e+03_rb,0.62624e+03_rb,0.62692e+03_rb,0.62788e+03_rb,0.62910e+03_rb /)
      kao(:,11, 7) = (/ &
        & 0.62583e+03_rb,0.62623e+03_rb,0.62691e+03_rb,0.62787e+03_rb,0.62909e+03_rb /)
      kao(:,12, 7) = (/ &
        & 0.62583e+03_rb,0.62623e+03_rb,0.62691e+03_rb,0.62787e+03_rb,0.62909e+03_rb /)
      kao(:,13, 7) = (/ &
        & 0.62583e+03_rb,0.62623e+03_rb,0.62691e+03_rb,0.62787e+03_rb,0.62909e+03_rb /)
      kao(:, 1, 8) = (/ &
        & 0.55488e+07_rb,0.54180e+07_rb,0.53005e+07_rb,0.51943e+07_rb,0.50978e+07_rb /)
      kao(:, 2, 8) = (/ &
        & 0.82143e+03_rb,0.82232e+03_rb,0.82327e+03_rb,0.82429e+03_rb,0.82541e+03_rb /)
      kao(:, 3, 8) = (/ &
        & 0.82092e+03_rb,0.82177e+03_rb,0.82269e+03_rb,0.82366e+03_rb,0.82472e+03_rb /)
      kao(:, 4, 8) = (/ &
        & 0.82041e+03_rb,0.82121e+03_rb,0.82209e+03_rb,0.82303e+03_rb,0.82402e+03_rb /)
      kao(:, 5, 8) = (/ &
        & 0.81996e+03_rb,0.82070e+03_rb,0.82152e+03_rb,0.82243e+03_rb,0.82339e+03_rb /)
      kao(:, 6, 8) = (/ &
        & 0.81952e+03_rb,0.82021e+03_rb,0.82098e+03_rb,0.82184e+03_rb,0.82277e+03_rb /)
      kao(:, 7, 8) = (/ &
        & 0.81917e+03_rb,0.81977e+03_rb,0.82048e+03_rb,0.82129e+03_rb,0.82218e+03_rb /)
      kao(:, 8, 8) = (/ &
        & 0.81881e+03_rb,0.81938e+03_rb,0.82004e+03_rb,0.82080e+03_rb,0.82163e+03_rb /)
      kao(:, 9, 8) = (/ &
        & 0.81846e+03_rb,0.81906e+03_rb,0.81965e+03_rb,0.82035e+03_rb,0.82114e+03_rb /)
      kao(:,10, 8) = (/ &
        & 0.81821e+03_rb,0.81877e+03_rb,0.81934e+03_rb,0.82000e+03_rb,0.82074e+03_rb /)
      kao(:,11, 8) = (/ &
        & 0.81821e+03_rb,0.81877e+03_rb,0.81934e+03_rb,0.81999e+03_rb,0.82074e+03_rb /)
      kao(:,12, 8) = (/ &
        & 0.81821e+03_rb,0.81877e+03_rb,0.81934e+03_rb,0.81999e+03_rb,0.82074e+03_rb /)
      kao(:,13, 8) = (/ &
        & 0.81821e+03_rb,0.81877e+03_rb,0.81935e+03_rb,0.81999e+03_rb,0.82074e+03_rb /)
      kao(:, 1, 9) = (/ &
        & 0.10204e+08_rb,0.99477e+07_rb,0.97209e+07_rb,0.95103e+07_rb,0.92960e+07_rb /)
      kao(:, 2, 9) = (/ &
        & 0.95102e+03_rb,0.95144e+03_rb,0.95133e+03_rb,0.95066e+03_rb,0.94964e+03_rb /)
      kao(:, 3, 9) = (/ &
        & 0.95048e+03_rb,0.95118e+03_rb,0.95136e+03_rb,0.95103e+03_rb,0.95020e+03_rb /)
      kao(:, 4, 9) = (/ &
        & 0.94969e+03_rb,0.95077e+03_rb,0.95126e+03_rb,0.95124e+03_rb,0.95074e+03_rb /)
      kao(:, 5, 9) = (/ &
        & 0.94872e+03_rb,0.95015e+03_rb,0.95101e+03_rb,0.95129e+03_rb,0.95105e+03_rb /)
      kao(:, 6, 9) = (/ &
        & 0.94752e+03_rb,0.94929e+03_rb,0.95052e+03_rb,0.95116e+03_rb,0.95126e+03_rb /)
      kao(:, 7, 9) = (/ &
        & 0.94612e+03_rb,0.94825e+03_rb,0.94981e+03_rb,0.95083e+03_rb,0.95126e+03_rb /)
      kao(:, 8, 9) = (/ &
        & 0.94461e+03_rb,0.94704e+03_rb,0.94894e+03_rb,0.95029e+03_rb,0.95107e+03_rb /)
      kao(:, 9, 9) = (/ &
        & 0.94291e+03_rb,0.94567e+03_rb,0.94791e+03_rb,0.94957e+03_rb,0.95069e+03_rb /)
      kao(:,10, 9) = (/ &
        & 0.94135e+03_rb,0.94441e+03_rb,0.94689e+03_rb,0.94884e+03_rb,0.95022e+03_rb /)
      kao(:,11, 9) = (/ &
        & 0.94133e+03_rb,0.94440e+03_rb,0.94688e+03_rb,0.94883e+03_rb,0.95021e+03_rb /)
      kao(:,12, 9) = (/ &
        & 0.94133e+03_rb,0.94438e+03_rb,0.94688e+03_rb,0.94883e+03_rb,0.95021e+03_rb /)
      kao(:,13, 9) = (/ &
        & 0.94133e+03_rb,0.94438e+03_rb,0.94687e+03_rb,0.94883e+03_rb,0.95021e+03_rb /)
      kao(:, 1,10) = (/ &
        & 0.12764e+08_rb,0.12408e+08_rb,0.12012e+08_rb,0.11438e+08_rb,0.11217e+08_rb /)
      kao(:, 2,10) = (/ &
        & 0.99589e+03_rb,0.99493e+03_rb,0.99263e+03_rb,0.98974e+03_rb,0.98742e+03_rb /)
      kao(:, 3,10) = (/ &
        & 0.99532e+03_rb,0.99571e+03_rb,0.99447e+03_rb,0.99207e+03_rb,0.98779e+03_rb /)
      kao(:, 4,10) = (/ &
        & 0.99446e+03_rb,0.99559e+03_rb,0.99536e+03_rb,0.99386e+03_rb,0.99088e+03_rb /)
      kao(:, 5,10) = (/ &
        & 0.99288e+03_rb,0.99493e+03_rb,0.99558e+03_rb,0.99487e+03_rb,0.99336e+03_rb /)
      kao(:, 6,10) = (/ &
        & 0.99069e+03_rb,0.99366e+03_rb,0.99519e+03_rb,0.99534e+03_rb,0.99428e+03_rb /)
      kao(:, 7,10) = (/ &
        & 0.98811e+03_rb,0.99191e+03_rb,0.99430e+03_rb,0.99510e+03_rb,0.99504e+03_rb /)
      kao(:, 8,10) = (/ &
        & 0.98531e+03_rb,0.98996e+03_rb,0.99300e+03_rb,0.99479e+03_rb,0.99513e+03_rb /)
      kao(:, 9,10) = (/ &
        & 0.98264e+03_rb,0.98737e+03_rb,0.99138e+03_rb,0.99390e+03_rb,0.99503e+03_rb /)
      kao(:,10,10) = (/ &
        & 0.98010e+03_rb,0.98517e+03_rb,0.98974e+03_rb,0.99281e+03_rb,0.99470e+03_rb /)
      kao(:,11,10) = (/ &
        & 0.98010e+03_rb,0.98509e+03_rb,0.98985e+03_rb,0.99280e+03_rb,0.99468e+03_rb /)
      kao(:,12,10) = (/ &
        & 0.98035e+03_rb,0.98524e+03_rb,0.98973e+03_rb,0.99279e+03_rb,0.99468e+03_rb /)
      kao(:,13,10) = (/ &
        & 0.98012e+03_rb,0.98526e+03_rb,0.98973e+03_rb,0.99279e+03_rb,0.99468e+03_rb /)
      kao(:, 1,11) = (/ &
        & 0.12266e+08_rb,0.12000e+08_rb,0.11724e+08_rb,0.11828e+08_rb,0.11523e+08_rb /)
      kao(:, 2,11) = (/ &
        & 0.10001e+04_rb,0.99984e+03_rb,0.99745e+03_rb,0.99535e+03_rb,0.98599e+03_rb /)
      kao(:, 3,11) = (/ &
        & 0.99963e+03_rb,0.99927e+03_rb,0.99848e+03_rb,0.99596e+03_rb,0.99390e+03_rb /)
      kao(:, 4,11) = (/ &
        & 0.99904e+03_rb,0.99942e+03_rb,0.99870e+03_rb,0.99747e+03_rb,0.99542e+03_rb /)
      kao(:, 5,11) = (/ &
        & 0.99826e+03_rb,0.99927e+03_rb,0.99919e+03_rb,0.99821e+03_rb,0.99627e+03_rb /)
      kao(:, 6,11) = (/ &
        & 0.99730e+03_rb,0.99882e+03_rb,0.99938e+03_rb,0.99902e+03_rb,0.99764e+03_rb /)
      kao(:, 7,11) = (/ &
        & 0.99551e+03_rb,0.99810e+03_rb,0.99919e+03_rb,0.99961e+03_rb,0.99863e+03_rb /)
      kao(:, 8,11) = (/ &
        & 0.99324e+03_rb,0.99688e+03_rb,0.99871e+03_rb,0.99940e+03_rb,0.99942e+03_rb /)
      kao(:, 9,11) = (/ &
        & 0.99058e+03_rb,0.99489e+03_rb,0.99773e+03_rb,0.99912e+03_rb,0.99954e+03_rb /)
      kao(:,10,11) = (/ &
        & 0.98802e+03_rb,0.99297e+03_rb,0.99656e+03_rb,0.99861e+03_rb,0.99937e+03_rb /)
      kao(:,11,11) = (/ &
        & 0.98800e+03_rb,0.99310e+03_rb,0.99655e+03_rb,0.99860e+03_rb,0.99936e+03_rb /)
      kao(:,12,11) = (/ &
        & 0.98771e+03_rb,0.99296e+03_rb,0.99669e+03_rb,0.99859e+03_rb,0.99936e+03_rb /)
      kao(:,13,11) = (/ &
        & 0.98800e+03_rb,0.99296e+03_rb,0.99655e+03_rb,0.99859e+03_rb,0.99936e+03_rb /)
      kao(:, 1,12) = (/ &
        & 0.12503e+08_rb,0.11946e+08_rb,0.11557e+08_rb,0.11503e+08_rb,0.11901e+08_rb /)
      kao(:, 2,12) = (/ &
        & 0.10015e+04_rb,0.10004e+04_rb,0.99932e+03_rb,0.99373e+03_rb,0.98591e+03_rb /)
      kao(:, 3,12) = (/ &
        & 0.10024e+04_rb,0.10021e+04_rb,0.10003e+04_rb,0.99817e+03_rb,0.99146e+03_rb /)
      kao(:, 4,12) = (/ &
        & 0.10022e+04_rb,0.10027e+04_rb,0.10021e+04_rb,0.99956e+03_rb,0.99610e+03_rb /)
      kao(:, 5,12) = (/ &
        & 0.10024e+04_rb,0.10024e+04_rb,0.10028e+04_rb,0.10015e+04_rb,0.99826e+03_rb /)
      kao(:, 6,12) = (/ &
        & 0.10017e+04_rb,0.10025e+04_rb,0.10025e+04_rb,0.10023e+04_rb,0.10008e+04_rb /)
      kao(:, 7,12) = (/ &
        & 0.10005e+04_rb,0.10020e+04_rb,0.10028e+04_rb,0.10028e+04_rb,0.10019e+04_rb /)
      kao(:, 8,12) = (/ &
        & 0.10000e+04_rb,0.10010e+04_rb,0.10024e+04_rb,0.10029e+04_rb,0.10023e+04_rb /)
      kao(:, 9,12) = (/ &
        & 0.99906e+03_rb,0.10004e+04_rb,0.10018e+04_rb,0.10029e+04_rb,0.10028e+04_rb /)
      kao(:,10,12) = (/ &
        & 0.99791e+03_rb,0.99989e+03_rb,0.10010e+04_rb,0.10024e+04_rb,0.10031e+04_rb /)
      kao(:,11,12) = (/ &
        & 0.99769e+03_rb,0.99967e+03_rb,0.10008e+04_rb,0.10024e+04_rb,0.10029e+04_rb /)
      kao(:,12,12) = (/ &
        & 0.99787e+03_rb,0.99985e+03_rb,0.10008e+04_rb,0.10025e+04_rb,0.10029e+04_rb /)
      kao(:,13,12) = (/ &
        & 0.99768e+03_rb,0.99984e+03_rb,0.10012e+04_rb,0.10026e+04_rb,0.10029e+04_rb /)
      kao(:, 1,13) = (/ &
        & 0.12359e+08_rb,0.12469e+08_rb,0.12868e+08_rb,0.12776e+08_rb,0.12590e+08_rb /)
      kao(:, 2,13) = (/ &
        & 0.10045e+04_rb,0.10033e+04_rb,0.99840e+03_rb,0.99184e+03_rb,0.98861e+03_rb /)
      kao(:, 3,13) = (/ &
        & 0.10050e+04_rb,0.10041e+04_rb,0.10024e+04_rb,0.99701e+03_rb,0.99083e+03_rb /)
      kao(:, 4,13) = (/ &
        & 0.10053e+04_rb,0.10043e+04_rb,0.10035e+04_rb,0.10017e+04_rb,0.99538e+03_rb /)
      kao(:, 5,13) = (/ &
        & 0.10048e+04_rb,0.10056e+04_rb,0.10038e+04_rb,0.10026e+04_rb,0.10008e+04_rb /)
      kao(:, 6,13) = (/ &
        & 0.10042e+04_rb,0.10054e+04_rb,0.10054e+04_rb,0.10039e+04_rb,0.10017e+04_rb /)
      kao(:, 7,13) = (/ &
        & 0.10029e+04_rb,0.10046e+04_rb,0.10055e+04_rb,0.10046e+04_rb,0.10033e+04_rb /)
      kao(:, 8,13) = (/ &
        & 0.10002e+04_rb,0.10039e+04_rb,0.10051e+04_rb,0.10054e+04_rb,0.10046e+04_rb /)
      kao(:, 9,13) = (/ &
        & 0.99825e+03_rb,0.10019e+04_rb,0.10043e+04_rb,0.10051e+04_rb,0.10052e+04_rb /)
      kao(:,10,13) = (/ &
        & 0.99595e+03_rb,0.99983e+03_rb,0.10034e+04_rb,0.10050e+04_rb,0.10051e+04_rb /)
      kao(:,11,13) = (/ &
        & 0.99621e+03_rb,0.99979e+03_rb,0.10034e+04_rb,0.10050e+04_rb,0.10054e+04_rb /)
      kao(:,12,13) = (/ &
        & 0.99597e+03_rb,0.99979e+03_rb,0.10031e+04_rb,0.10047e+04_rb,0.10054e+04_rb /)
      kao(:,13,13) = (/ &
        & 0.99623e+03_rb,0.99978e+03_rb,0.10031e+04_rb,0.10047e+04_rb,0.10054e+04_rb /)
      kao(:, 1,14) = (/ &
        & 0.13636e+08_rb,0.13818e+08_rb,0.13518e+08_rb,0.13266e+08_rb,0.13048e+08_rb /)
      kao(:, 2,14) = (/ &
        & 0.10049e+04_rb,0.99946e+03_rb,0.99774e+03_rb,0.99774e+03_rb,0.99771e+03_rb /)
      kao(:, 3,14) = (/ &
        & 0.10051e+04_rb,0.10068e+04_rb,0.10007e+04_rb,0.99775e+03_rb,0.99778e+03_rb /)
      kao(:, 4,14) = (/ &
        & 0.10071e+04_rb,0.10061e+04_rb,0.10050e+04_rb,0.10005e+04_rb,0.99776e+03_rb /)
      kao(:, 5,14) = (/ &
        & 0.10067e+04_rb,0.10072e+04_rb,0.10061e+04_rb,0.10044e+04_rb,0.99972e+03_rb /)
      kao(:, 6,14) = (/ &
        & 0.10049e+04_rb,0.10070e+04_rb,0.10074e+04_rb,0.10057e+04_rb,0.10037e+04_rb /)
      kao(:, 7,14) = (/ &
        & 0.10033e+04_rb,0.10063e+04_rb,0.10072e+04_rb,0.10067e+04_rb,0.10050e+04_rb /)
      kao(:, 8,14) = (/ &
        & 0.10025e+04_rb,0.10048e+04_rb,0.10068e+04_rb,0.10071e+04_rb,0.10062e+04_rb /)
      kao(:, 9,14) = (/ &
        & 0.99880e+03_rb,0.10031e+04_rb,0.10060e+04_rb,0.10071e+04_rb,0.10069e+04_rb /)
      kao(:,10,14) = (/ &
        & 0.99610e+03_rb,0.10023e+04_rb,0.10047e+04_rb,0.10067e+04_rb,0.10072e+04_rb /)
      kao(:,11,14) = (/ &
        & 0.99601e+03_rb,0.10023e+04_rb,0.10047e+04_rb,0.10067e+04_rb,0.10072e+04_rb /)
      kao(:,12,14) = (/ &
        & 0.99598e+03_rb,0.10023e+04_rb,0.10051e+04_rb,0.10067e+04_rb,0.10076e+04_rb /)
      kao(:,13,14) = (/ &
        & 0.99596e+03_rb,0.10027e+04_rb,0.10047e+04_rb,0.10067e+04_rb,0.10072e+04_rb /)
      kao(:, 1,15) = (/ &
        & 0.14285e+08_rb,0.13947e+08_rb,0.13644e+08_rb,0.13317e+08_rb,0.13015e+08_rb /)
      kao(:, 2,15) = (/ &
        & 0.99932e+03_rb,0.10011e+04_rb,0.10018e+04_rb,0.10007e+04_rb,0.99706e+03_rb /)
      kao(:, 3,15) = (/ &
        & 0.10097e+04_rb,0.99743e+03_rb,0.10015e+04_rb,0.10018e+04_rb,0.99920e+03_rb /)
      kao(:, 4,15) = (/ &
        & 0.10099e+04_rb,0.10076e+04_rb,0.10030e+04_rb,0.10017e+04_rb,0.10013e+04_rb /)
      kao(:, 5,15) = (/ &
        & 0.10075e+04_rb,0.10079e+04_rb,0.10073e+04_rb,0.10035e+04_rb,0.10018e+04_rb /)
      kao(:, 6,15) = (/ &
        & 0.10068e+04_rb,0.10088e+04_rb,0.10068e+04_rb,0.10090e+04_rb,0.10033e+04_rb /)
      kao(:, 7,15) = (/ &
        & 0.10057e+04_rb,0.10073e+04_rb,0.10090e+04_rb,0.10087e+04_rb,0.10062e+04_rb /)
      kao(:, 8,15) = (/ &
        & 0.10010e+04_rb,0.10054e+04_rb,0.10076e+04_rb,0.10079e+04_rb,0.10072e+04_rb /)
      kao(:, 9,15) = (/ &
        & 0.99853e+03_rb,0.10054e+04_rb,0.10071e+04_rb,0.10079e+04_rb,0.10078e+04_rb /)
      kao(:,10,15) = (/ &
        & 0.99742e+03_rb,0.10007e+04_rb,0.10064e+04_rb,0.10076e+04_rb,0.10080e+04_rb /)
      kao(:,11,15) = (/ &
        & 0.99635e+03_rb,0.10006e+04_rb,0.10064e+04_rb,0.10087e+04_rb,0.10090e+04_rb /)
      kao(:,12,15) = (/ &
        & 0.99635e+03_rb,0.10006e+04_rb,0.10064e+04_rb,0.10087e+04_rb,0.10069e+04_rb /)
      kao(:,13,15) = (/ &
        & 0.99635e+03_rb,0.99960e+03_rb,0.10064e+04_rb,0.10076e+04_rb,0.10080e+04_rb /)
      kao(:, 1,16) = (/ &
        & 0.14326e+08_rb,0.13988e+08_rb,0.13684e+08_rb,0.13409e+08_rb,0.13021e+08_rb /)
      kao(:, 2,16) = (/ &
        & 0.10008e+04_rb,0.10027e+04_rb,0.10036e+04_rb,0.10037e+04_rb,0.99673e+03_rb /)
      kao(:, 3,16) = (/ &
        & 0.99286e+03_rb,0.10016e+04_rb,0.10031e+04_rb,0.10038e+04_rb,0.10035e+04_rb /)
      kao(:, 4,16) = (/ &
        & 0.99278e+03_rb,0.10078e+04_rb,0.10023e+04_rb,0.10035e+04_rb,0.10038e+04_rb /)
      kao(:, 5,16) = (/ &
        & 0.10074e+04_rb,0.10079e+04_rb,0.10075e+04_rb,0.10028e+04_rb,0.10037e+04_rb /)
      kao(:, 6,16) = (/ &
        & 0.10069e+04_rb,0.10002e+04_rb,0.10079e+04_rb,0.99212e+03_rb,0.10032e+04_rb /)
      kao(:, 7,16) = (/ &
        & 0.10060e+04_rb,0.10072e+04_rb,0.10004e+04_rb,0.10003e+04_rb,0.10063e+04_rb /)
      kao(:, 8,16) = (/ &
        & 0.10009e+04_rb,0.10066e+04_rb,0.10075e+04_rb,0.10080e+04_rb,0.10074e+04_rb /)
      kao(:, 9,16) = (/ &
        & 0.99860e+03_rb,0.10057e+04_rb,0.10071e+04_rb,0.10078e+04_rb,0.10079e+04_rb /)
      kao(:,10,16) = (/ &
        & 0.98908e+03_rb,0.10006e+04_rb,0.10065e+04_rb,0.10075e+04_rb,0.10080e+04_rb /)
      kao(:,11,16) = (/ &
        & 0.99644e+03_rb,0.10006e+04_rb,0.10065e+04_rb,0.10000e+04_rb,0.10005e+04_rb /)
      kao(:,12,16) = (/ &
        & 0.99644e+03_rb,0.10006e+04_rb,0.10065e+04_rb,0.10000e+04_rb,0.10080e+04_rb /)
      kao(:,13,16) = (/ &
        & 0.99644e+03_rb,0.10006e+04_rb,0.10065e+04_rb,0.10075e+04_rb,0.10080e+04_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------
  
      kbo(:,13, 1) = (/ &
        & 0.16036e+00_rb,0.16399e+00_rb,0.17198e+00_rb,0.18455e+00_rb,0.20173e+00_rb /)
      kbo(:,14, 1) = (/ &
        & 0.16050e+00_rb,0.16454e+00_rb,0.17292e+00_rb,0.18592e+00_rb,0.20351e+00_rb /)
      kbo(:,15, 1) = (/ &
        & 0.16071e+00_rb,0.16519e+00_rb,0.17400e+00_rb,0.18750e+00_rb,0.20554e+00_rb /)
      kbo(:,16, 1) = (/ &
        & 0.16098e+00_rb,0.16593e+00_rb,0.17517e+00_rb,0.18921e+00_rb,0.20772e+00_rb /)
      kbo(:,17, 1) = (/ &
        & 0.16129e+00_rb,0.16670e+00_rb,0.17642e+00_rb,0.19093e+00_rb,0.20991e+00_rb /)
      kbo(:,18, 1) = (/ &
        & 0.16164e+00_rb,0.16750e+00_rb,0.17771e+00_rb,0.19267e+00_rb,0.21211e+00_rb /)
      kbo(:,19, 1) = (/ &
        & 0.16203e+00_rb,0.16832e+00_rb,0.17901e+00_rb,0.19443e+00_rb,0.21431e+00_rb /)
      kbo(:,20, 1) = (/ &
        & 0.16255e+00_rb,0.16934e+00_rb,0.18059e+00_rb,0.19653e+00_rb,0.21694e+00_rb /)
      kbo(:,21, 1) = (/ &
        & 0.16313e+00_rb,0.17044e+00_rb,0.18225e+00_rb,0.19873e+00_rb,0.21967e+00_rb /)
      kbo(:,22, 1) = (/ &
        & 0.16416e+00_rb,0.17227e+00_rb,0.18498e+00_rb,0.20229e+00_rb,0.22406e+00_rb /)
      kbo(:,23, 1) = (/ &
        & 0.16539e+00_rb,0.17431e+00_rb,0.18796e+00_rb,0.20612e+00_rb,0.22875e+00_rb /)
      kbo(:,24, 1) = (/ &
        & 0.16684e+00_rb,0.17666e+00_rb,0.19125e+00_rb,0.21032e+00_rb,0.23385e+00_rb /)
      kbo(:,25, 1) = (/ &
        & 0.16856e+00_rb,0.17939e+00_rb,0.19493e+00_rb,0.21495e+00_rb,0.23943e+00_rb /)
      kbo(:,26, 1) = (/ &
        & 0.17067e+00_rb,0.18260e+00_rb,0.19919e+00_rb,0.22024e+00_rb,0.24578e+00_rb /)
      kbo(:,27, 1) = (/ &
        & 0.17306e+00_rb,0.18613e+00_rb,0.20377e+00_rb,0.22588e+00_rb,0.25248e+00_rb /)
      kbo(:,28, 1) = (/ &
        & 0.17570e+00_rb,0.18994e+00_rb,0.20865e+00_rb,0.23184e+00_rb,0.25952e+00_rb /)
      kbo(:,29, 1) = (/ &
        & 0.17882e+00_rb,0.19417e+00_rb,0.21399e+00_rb,0.23828e+00_rb,0.26708e+00_rb /)
      kbo(:,30, 1) = (/ &
        & 0.18223e+00_rb,0.19870e+00_rb,0.21964e+00_rb,0.24505e+00_rb,0.27492e+00_rb /)
      kbo(:,31, 1) = (/ &
        & 0.18602e+00_rb,0.20364e+00_rb,0.22572e+00_rb,0.25229e+00_rb,0.28325e+00_rb /)
      kbo(:,32, 1) = (/ &
        & 0.19018e+00_rb,0.20895e+00_rb,0.23219e+00_rb,0.25994e+00_rb,0.29200e+00_rb /)
      kbo(:,33, 1) = (/ &
        & 0.19466e+00_rb,0.21461e+00_rb,0.23902e+00_rb,0.26795e+00_rb,0.30114e+00_rb /)
      kbo(:,34, 1) = (/ &
        & 0.19880e+00_rb,0.21976e+00_rb,0.24520e+00_rb,0.27509e+00_rb,0.30932e+00_rb /)
      kbo(:,35, 1) = (/ &
        & 0.20129e+00_rb,0.22283e+00_rb,0.24886e+00_rb,0.27931e+00_rb,0.31412e+00_rb /)
      kbo(:,36, 1) = (/ &
        & 0.20170e+00_rb,0.22334e+00_rb,0.24946e+00_rb,0.28000e+00_rb,0.31492e+00_rb /)
      kbo(:,37, 1) = (/ &
        & 0.19920e+00_rb,0.22026e+00_rb,0.24579e+00_rb,0.27578e+00_rb,0.31010e+00_rb /)
      kbo(:,38, 1) = (/ &
        & 0.19662e+00_rb,0.21706e+00_rb,0.24197e+00_rb,0.27135e+00_rb,0.30504e+00_rb /)
      kbo(:,39, 1) = (/ &
        & 0.19414e+00_rb,0.21396e+00_rb,0.23824e+00_rb,0.26704e+00_rb,0.30010e+00_rb /)
      kbo(:,40, 1) = (/ &
        & 0.19003e+00_rb,0.20877e+00_rb,0.23197e+00_rb,0.25968e+00_rb,0.29171e+00_rb /)
      kbo(:,41, 1) = (/ &
        & 0.18597e+00_rb,0.20357e+00_rb,0.22564e+00_rb,0.25219e+00_rb,0.28314e+00_rb /)
      kbo(:,42, 1) = (/ &
        & 0.18220e+00_rb,0.19865e+00_rb,0.21958e+00_rb,0.24498e+00_rb,0.27484e+00_rb /)
      kbo(:,43, 1) = (/ &
        & 0.17798e+00_rb,0.19304e+00_rb,0.21257e+00_rb,0.23657e+00_rb,0.26510e+00_rb /)
      kbo(:,44, 1) = (/ &
        & 0.17397e+00_rb,0.18746e+00_rb,0.20548e+00_rb,0.22798e+00_rb,0.25496e+00_rb /)
      kbo(:,45, 1) = (/ &
        & 0.17053e+00_rb,0.18239e+00_rb,0.19890e+00_rb,0.21989e+00_rb,0.24535e+00_rb /)
      kbo(:,46, 1) = (/ &
        & 0.16739e+00_rb,0.17754e+00_rb,0.19245e+00_rb,0.21183e+00_rb,0.23568e+00_rb /)
      kbo(:,47, 1) = (/ &
        & 0.16454e+00_rb,0.17292e+00_rb,0.18592e+00_rb,0.20351e+00_rb,0.22556e+00_rb /)
      kbo(:,48, 1) = (/ &
        & 0.16239e+00_rb,0.16903e+00_rb,0.18011e+00_rb,0.19590e+00_rb,0.21615e+00_rb /)
      kbo(:,49, 1) = (/ &
        & 0.16095e+00_rb,0.16584e+00_rb,0.17504e+00_rb,0.18902e+00_rb,0.20747e+00_rb /)
      kbo(:,50, 1) = (/ &
        & 0.16025e+00_rb,0.16348e+00_rb,0.17109e+00_rb,0.18322e+00_rb,0.20000e+00_rb /)
      kbo(:,51, 1) = (/ &
        & 0.16021e+00_rb,0.16179e+00_rb,0.16781e+00_rb,0.17821e+00_rb,0.19335e+00_rb /)
      kbo(:,52, 1) = (/ &
        & 0.16078e+00_rb,0.16068e+00_rb,0.16511e+00_rb,0.17387e+00_rb,0.18731e+00_rb /)
      kbo(:,53, 1) = (/ &
        & 0.16197e+00_rb,0.16018e+00_rb,0.16299e+00_rb,0.17017e+00_rb,0.18186e+00_rb /)
      kbo(:,54, 1) = (/ &
        & 0.16362e+00_rb,0.16026e+00_rb,0.16155e+00_rb,0.16729e+00_rb,0.17738e+00_rb /)
      kbo(:,55, 1) = (/ &
        & 0.16570e+00_rb,0.16083e+00_rb,0.16064e+00_rb,0.16498e+00_rb,0.17365e+00_rb /)
      kbo(:,56, 1) = (/ &
        & 0.16825e+00_rb,0.16187e+00_rb,0.16020e+00_rb,0.16311e+00_rb,0.17041e+00_rb /)
      kbo(:,57, 1) = (/ &
        & 0.17129e+00_rb,0.16340e+00_rb,0.16023e+00_rb,0.16169e+00_rb,0.16761e+00_rb /)
      kbo(:,58, 1) = (/ &
        & 0.17462e+00_rb,0.16529e+00_rb,0.16069e+00_rb,0.16077e+00_rb,0.16535e+00_rb /)
      kbo(:,59, 1) = (/ &
        & 0.17607e+00_rb,0.16617e+00_rb,0.16100e+00_rb,0.16051e+00_rb,0.16456e+00_rb /)
      kbo(:,13, 2) = (/ &
        & 0.17525e+01_rb,0.17727e+01_rb,0.18119e+01_rb,0.18712e+01_rb,0.19491e+01_rb /)
      kbo(:,14, 2) = (/ &
        & 0.17536e+01_rb,0.17755e+01_rb,0.18165e+01_rb,0.18775e+01_rb,0.19571e+01_rb /)
      kbo(:,15, 2) = (/ &
        & 0.17549e+01_rb,0.17787e+01_rb,0.18219e+01_rb,0.18847e+01_rb,0.19662e+01_rb /)
      kbo(:,16, 2) = (/ &
        & 0.17565e+01_rb,0.17819e+01_rb,0.18278e+01_rb,0.18925e+01_rb,0.19761e+01_rb /)
      kbo(:,17, 2) = (/ &
        & 0.17583e+01_rb,0.17850e+01_rb,0.18336e+01_rb,0.19003e+01_rb,0.19859e+01_rb /)
      kbo(:,18, 2) = (/ &
        & 0.17603e+01_rb,0.17891e+01_rb,0.18396e+01_rb,0.19082e+01_rb,0.19957e+01_rb /)
      kbo(:,19, 2) = (/ &
        & 0.17624e+01_rb,0.17934e+01_rb,0.18457e+01_rb,0.19162e+01_rb,0.20056e+01_rb /)
      kbo(:,20, 2) = (/ &
        & 0.17652e+01_rb,0.17986e+01_rb,0.18530e+01_rb,0.19257e+01_rb,0.20174e+01_rb /)
      kbo(:,21, 2) = (/ &
        & 0.17683e+01_rb,0.18042e+01_rb,0.18607e+01_rb,0.19356e+01_rb,0.20296e+01_rb /)
      kbo(:,22, 2) = (/ &
        & 0.17736e+01_rb,0.18133e+01_rb,0.18732e+01_rb,0.19516e+01_rb,0.20492e+01_rb /)
      kbo(:,23, 2) = (/ &
        & 0.17796e+01_rb,0.18234e+01_rb,0.18868e+01_rb,0.19689e+01_rb,0.20702e+01_rb /)
      kbo(:,24, 2) = (/ &
        & 0.17857e+01_rb,0.18347e+01_rb,0.19018e+01_rb,0.19877e+01_rb,0.20929e+01_rb /)
      kbo(:,25, 2) = (/ &
        & 0.17946e+01_rb,0.18475e+01_rb,0.19185e+01_rb,0.20085e+01_rb,0.21178e+01_rb /)
      kbo(:,26, 2) = (/ &
        & 0.18053e+01_rb,0.18623e+01_rb,0.19377e+01_rb,0.20322e+01_rb,0.21460e+01_rb /)
      kbo(:,27, 2) = (/ &
        & 0.18172e+01_rb,0.18785e+01_rb,0.19583e+01_rb,0.20574e+01_rb,0.21757e+01_rb /)
      kbo(:,28, 2) = (/ &
        & 0.18303e+01_rb,0.18959e+01_rb,0.19802e+01_rb,0.20839e+01_rb,0.22069e+01_rb /)
      kbo(:,29, 2) = (/ &
        & 0.18448e+01_rb,0.19150e+01_rb,0.20042e+01_rb,0.21126e+01_rb,0.22404e+01_rb /)
      kbo(:,30, 2) = (/ &
        & 0.18606e+01_rb,0.19355e+01_rb,0.20295e+01_rb,0.21428e+01_rb,0.22754e+01_rb /)
      kbo(:,31, 2) = (/ &
        & 0.18780e+01_rb,0.19577e+01_rb,0.20566e+01_rb,0.21749e+01_rb,0.23125e+01_rb /)
      kbo(:,32, 2) = (/ &
        & 0.18969e+01_rb,0.19816e+01_rb,0.20855e+01_rb,0.22088e+01_rb,0.23514e+01_rb /)
      kbo(:,33, 2) = (/ &
        & 0.19172e+01_rb,0.20069e+01_rb,0.21160e+01_rb,0.22443e+01_rb,0.23920e+01_rb /)
      kbo(:,34, 2) = (/ &
        & 0.19359e+01_rb,0.20300e+01_rb,0.21434e+01_rb,0.22762e+01_rb,0.24282e+01_rb /)
      kbo(:,35, 2) = (/ &
        & 0.19471e+01_rb,0.20437e+01_rb,0.21597e+01_rb,0.22949e+01_rb,0.24495e+01_rb /)
      kbo(:,36, 2) = (/ &
        & 0.19490e+01_rb,0.20460e+01_rb,0.21624e+01_rb,0.22980e+01_rb,0.24530e+01_rb /)
      kbo(:,37, 2) = (/ &
        & 0.19377e+01_rb,0.20322e+01_rb,0.21461e+01_rb,0.22792e+01_rb,0.24317e+01_rb /)
      kbo(:,38, 2) = (/ &
        & 0.19261e+01_rb,0.20179e+01_rb,0.21291e+01_rb,0.22595e+01_rb,0.24093e+01_rb /)
      kbo(:,39, 2) = (/ &
        & 0.19149e+01_rb,0.20040e+01_rb,0.21125e+01_rb,0.22402e+01_rb,0.23873e+01_rb /)
      kbo(:,40, 2) = (/ &
        & 0.18963e+01_rb,0.19807e+01_rb,0.20845e+01_rb,0.22076e+01_rb,0.23501e+01_rb /)
      kbo(:,41, 2) = (/ &
        & 0.18778e+01_rb,0.19574e+01_rb,0.20563e+01_rb,0.21745e+01_rb,0.23120e+01_rb /)
      kbo(:,42, 2) = (/ &
        & 0.18604e+01_rb,0.19352e+01_rb,0.20292e+01_rb,0.21425e+01_rb,0.22750e+01_rb /)
      kbo(:,43, 2) = (/ &
        & 0.18409e+01_rb,0.19099e+01_rb,0.19978e+01_rb,0.21050e+01_rb,0.22315e+01_rb /)
      kbo(:,44, 2) = (/ &
        & 0.18217e+01_rb,0.18845e+01_rb,0.19660e+01_rb,0.20667e+01_rb,0.21867e+01_rb /)
      kbo(:,45, 2) = (/ &
        & 0.18046e+01_rb,0.18613e+01_rb,0.19364e+01_rb,0.20306e+01_rb,0.21441e+01_rb /)
      kbo(:,46, 2) = (/ &
        & 0.17886e+01_rb,0.18389e+01_rb,0.19072e+01_rb,0.19945e+01_rb,0.21011e+01_rb /)
      kbo(:,47, 2) = (/ &
        & 0.17755e+01_rb,0.18165e+01_rb,0.18775e+01_rb,0.19571e+01_rb,0.20559e+01_rb /)
      kbo(:,48, 2) = (/ &
        & 0.17644e+01_rb,0.17970e+01_rb,0.18508e+01_rb,0.19228e+01_rb,0.20139e+01_rb /)
      kbo(:,49, 2) = (/ &
        & 0.17563e+01_rb,0.17816e+01_rb,0.18271e+01_rb,0.18916e+01_rb,0.19749e+01_rb /)
      kbo(:,50, 2) = (/ &
        & 0.17517e+01_rb,0.17702e+01_rb,0.18074e+01_rb,0.18651e+01_rb,0.19413e+01_rb /)
      kbo(:,51, 2) = (/ &
        & 0.17504e+01_rb,0.17611e+01_rb,0.17908e+01_rb,0.18420e+01_rb,0.19113e+01_rb /)
      kbo(:,52, 2) = (/ &
        & 0.17523e+01_rb,0.17547e+01_rb,0.17783e+01_rb,0.18212e+01_rb,0.18839e+01_rb /)
      kbo(:,53, 2) = (/ &
        & 0.17566e+01_rb,0.17510e+01_rb,0.17675e+01_rb,0.18028e+01_rb,0.18589e+01_rb /)
      kbo(:,54, 2) = (/ &
        & 0.17623e+01_rb,0.17507e+01_rb,0.17598e+01_rb,0.17881e+01_rb,0.18381e+01_rb /)
      kbo(:,55, 2) = (/ &
        & 0.17691e+01_rb,0.17524e+01_rb,0.17544e+01_rb,0.17777e+01_rb,0.18201e+01_rb /)
      kbo(:,56, 2) = (/ &
        & 0.17776e+01_rb,0.17562e+01_rb,0.17512e+01_rb,0.17682e+01_rb,0.18040e+01_rb /)
      kbo(:,57, 2) = (/ &
        & 0.17881e+01_rb,0.17616e+01_rb,0.17505e+01_rb,0.17606e+01_rb,0.17897e+01_rb /)
      kbo(:,58, 2) = (/ &
        & 0.17996e+01_rb,0.17678e+01_rb,0.17520e+01_rb,0.17552e+01_rb,0.17794e+01_rb /)
      kbo(:,59, 2) = (/ &
        & 0.18045e+01_rb,0.17707e+01_rb,0.17530e+01_rb,0.17536e+01_rb,0.17756e+01_rb /)
      kbo(:,13, 3) = (/ &
        & 0.10945e+02_rb,0.11052e+02_rb,0.11222e+02_rb,0.11452e+02_rb,0.11744e+02_rb /)
      kbo(:,14, 3) = (/ &
        & 0.10952e+02_rb,0.11065e+02_rb,0.11240e+02_rb,0.11476e+02_rb,0.11774e+02_rb /)
      kbo(:,15, 3) = (/ &
        & 0.10961e+02_rb,0.11080e+02_rb,0.11261e+02_rb,0.11503e+02_rb,0.11807e+02_rb /)
      kbo(:,16, 3) = (/ &
        & 0.10971e+02_rb,0.11097e+02_rb,0.11284e+02_rb,0.11533e+02_rb,0.11844e+02_rb /)
      kbo(:,17, 3) = (/ &
        & 0.10981e+02_rb,0.11115e+02_rb,0.11307e+02_rb,0.11562e+02_rb,0.11879e+02_rb /)
      kbo(:,18, 3) = (/ &
        & 0.10992e+02_rb,0.11132e+02_rb,0.11331e+02_rb,0.11592e+02_rb,0.11915e+02_rb /)
      kbo(:,19, 3) = (/ &
        & 0.11003e+02_rb,0.11149e+02_rb,0.11354e+02_rb,0.11622e+02_rb,0.11951e+02_rb /)
      kbo(:,20, 3) = (/ &
        & 0.11017e+02_rb,0.11170e+02_rb,0.11382e+02_rb,0.11657e+02_rb,0.11994e+02_rb /)
      kbo(:,21, 3) = (/ &
        & 0.11031e+02_rb,0.11192e+02_rb,0.11412e+02_rb,0.11694e+02_rb,0.12038e+02_rb /)
      kbo(:,22, 3) = (/ &
        & 0.11056e+02_rb,0.11228e+02_rb,0.11460e+02_rb,0.11754e+02_rb,0.12109e+02_rb /)
      kbo(:,23, 3) = (/ &
        & 0.11085e+02_rb,0.11267e+02_rb,0.11511e+02_rb,0.11817e+02_rb,0.12184e+02_rb /)
      kbo(:,24, 3) = (/ &
        & 0.11118e+02_rb,0.11312e+02_rb,0.11568e+02_rb,0.11886e+02_rb,0.12266e+02_rb /)
      kbo(:,25, 3) = (/ &
        & 0.11154e+02_rb,0.11361e+02_rb,0.11630e+02_rb,0.11962e+02_rb,0.12354e+02_rb /)
      kbo(:,26, 3) = (/ &
        & 0.11196e+02_rb,0.11418e+02_rb,0.11702e+02_rb,0.12048e+02_rb,0.12455e+02_rb /)
      kbo(:,27, 3) = (/ &
        & 0.11243e+02_rb,0.11480e+02_rb,0.11778e+02_rb,0.12138e+02_rb,0.12560e+02_rb /)
      kbo(:,28, 3) = (/ &
        & 0.11294e+02_rb,0.11545e+02_rb,0.11859e+02_rb,0.12234e+02_rb,0.12670e+02_rb /)
      kbo(:,29, 3) = (/ &
        & 0.11351e+02_rb,0.11617e+02_rb,0.11946e+02_rb,0.12336e+02_rb,0.12788e+02_rb /)
      kbo(:,30, 3) = (/ &
        & 0.11411e+02_rb,0.11694e+02_rb,0.12038e+02_rb,0.12443e+02_rb,0.12910e+02_rb /)
      kbo(:,31, 3) = (/ &
        & 0.11478e+02_rb,0.11776e+02_rb,0.12136e+02_rb,0.12557e+02_rb,0.13040e+02_rb /)
      kbo(:,32, 3) = (/ &
        & 0.11549e+02_rb,0.11864e+02_rb,0.12239e+02_rb,0.12677e+02_rb,0.13175e+02_rb /)
      kbo(:,33, 3) = (/ &
        & 0.11626e+02_rb,0.11956e+02_rb,0.12348e+02_rb,0.12801e+02_rb,0.13316e+02_rb /)
      kbo(:,34, 3) = (/ &
        & 0.11695e+02_rb,0.12040e+02_rb,0.12446e+02_rb,0.12913e+02_rb,0.13442e+02_rb /)
      kbo(:,35, 3) = (/ &
        & 0.11737e+02_rb,0.12089e+02_rb,0.12503e+02_rb,0.12979e+02_rb,0.13515e+02_rb /)
      kbo(:,36, 3) = (/ &
        & 0.11744e+02_rb,0.12098e+02_rb,0.12513e+02_rb,0.12989e+02_rb,0.13528e+02_rb /)
      kbo(:,37, 3) = (/ &
        & 0.11702e+02_rb,0.12048e+02_rb,0.12455e+02_rb,0.12924e+02_rb,0.13454e+02_rb /)
      kbo(:,38, 3) = (/ &
        & 0.11659e+02_rb,0.11996e+02_rb,0.12395e+02_rb,0.12855e+02_rb,0.13376e+02_rb /)
      kbo(:,39, 3) = (/ &
        & 0.11617e+02_rb,0.11946e+02_rb,0.12336e+02_rb,0.12787e+02_rb,0.13300e+02_rb /)
      kbo(:,40, 3) = (/ &
        & 0.11547e+02_rb,0.11861e+02_rb,0.12236e+02_rb,0.12673e+02_rb,0.13171e+02_rb /)
      kbo(:,41, 3) = (/ &
        & 0.11477e+02_rb,0.11775e+02_rb,0.12134e+02_rb,0.12555e+02_rb,0.13038e+02_rb /)
      kbo(:,42, 3) = (/ &
        & 0.11411e+02_rb,0.11693e+02_rb,0.12037e+02_rb,0.12442e+02_rb,0.12909e+02_rb /)
      kbo(:,43, 3) = (/ &
        & 0.11335e+02_rb,0.11598e+02_rb,0.11923e+02_rb,0.12309e+02_rb,0.12757e+02_rb /)
      kbo(:,44, 3) = (/ &
        & 0.11261e+02_rb,0.11503e+02_rb,0.11807e+02_rb,0.12172e+02_rb,0.12599e+02_rb /)
      kbo(:,45, 3) = (/ &
        & 0.11193e+02_rb,0.11414e+02_rb,0.11697e+02_rb,0.12042e+02_rb,0.12448e+02_rb /)
      kbo(:,46, 3) = (/ &
        & 0.11129e+02_rb,0.11328e+02_rb,0.11588e+02_rb,0.11911e+02_rb,0.12295e+02_rb /)
      kbo(:,47, 3) = (/ &
        & 0.11065e+02_rb,0.11240e+02_rb,0.11476e+02_rb,0.11774e+02_rb,0.12133e+02_rb /)
      kbo(:,48, 3) = (/ &
        & 0.11012e+02_rb,0.11163e+02_rb,0.11374e+02_rb,0.11647e+02_rb,0.11981e+02_rb /)
      kbo(:,49, 3) = (/ &
        & 0.10969e+02_rb,0.11095e+02_rb,0.11282e+02_rb,0.11530e+02_rb,0.11839e+02_rb /)
      kbo(:,50, 3) = (/ &
        & 0.10938e+02_rb,0.11040e+02_rb,0.11204e+02_rb,0.11429e+02_rb,0.11715e+02_rb /)
      kbo(:,51, 3) = (/ &
        & 0.10916e+02_rb,0.10996e+02_rb,0.11138e+02_rb,0.11340e+02_rb,0.11604e+02_rb /)
      kbo(:,52, 3) = (/ &
        & 0.10900e+02_rb,0.10960e+02_rb,0.11079e+02_rb,0.11259e+02_rb,0.11500e+02_rb /)
      kbo(:,53, 3) = (/ &
        & 0.10894e+02_rb,0.10932e+02_rb,0.11028e+02_rb,0.11186e+02_rb,0.11405e+02_rb /)
      kbo(:,54, 3) = (/ &
        & 0.10895e+02_rb,0.10912e+02_rb,0.10989e+02_rb,0.11127e+02_rb,0.11325e+02_rb /)
      kbo(:,55, 3) = (/ &
        & 0.10903e+02_rb,0.10900e+02_rb,0.10958e+02_rb,0.11075e+02_rb,0.11255e+02_rb /)
      kbo(:,56, 3) = (/ &
        & 0.10921e+02_rb,0.10894e+02_rb,0.10933e+02_rb,0.11031e+02_rb,0.11191e+02_rb /)
      kbo(:,57, 3) = (/ &
        & 0.10944e+02_rb,0.10894e+02_rb,0.10914e+02_rb,0.10993e+02_rb,0.11134e+02_rb /)
      kbo(:,58, 3) = (/ &
        & 0.10972e+02_rb,0.10901e+02_rb,0.10902e+02_rb,0.10963e+02_rb,0.11084e+02_rb /)
      kbo(:,59, 3) = (/ &
        & 0.10984e+02_rb,0.10906e+02_rb,0.10898e+02_rb,0.10952e+02_rb,0.11066e+02_rb /)
      kbo(:,13, 4) = (/ &
        & 0.51563e+02_rb,0.51946e+02_rb,0.52464e+02_rb,0.53116e+02_rb,0.53902e+02_rb /)
      kbo(:,14, 4) = (/ &
        & 0.51593e+02_rb,0.51987e+02_rb,0.52517e+02_rb,0.53182e+02_rb,0.53980e+02_rb /)
      kbo(:,15, 4) = (/ &
        & 0.51626e+02_rb,0.52035e+02_rb,0.52579e+02_rb,0.53256e+02_rb,0.54069e+02_rb /)
      kbo(:,16, 4) = (/ &
        & 0.51664e+02_rb,0.52087e+02_rb,0.52644e+02_rb,0.53337e+02_rb,0.54163e+02_rb /)
      kbo(:,17, 4) = (/ &
        & 0.51701e+02_rb,0.52138e+02_rb,0.52710e+02_rb,0.53416e+02_rb,0.54257e+02_rb /)
      kbo(:,18, 4) = (/ &
        & 0.51740e+02_rb,0.52191e+02_rb,0.52776e+02_rb,0.53496e+02_rb,0.54351e+02_rb /)
      kbo(:,19, 4) = (/ &
        & 0.51779e+02_rb,0.52244e+02_rb,0.52843e+02_rb,0.53576e+02_rb,0.54444e+02_rb /)
      kbo(:,20, 4) = (/ &
        & 0.51827e+02_rb,0.52307e+02_rb,0.52922e+02_rb,0.53671e+02_rb,0.54555e+02_rb /)
      kbo(:,21, 4) = (/ &
        & 0.51877e+02_rb,0.52373e+02_rb,0.53004e+02_rb,0.53770e+02_rb,0.54669e+02_rb /)
      kbo(:,22, 4) = (/ &
        & 0.51959e+02_rb,0.52481e+02_rb,0.53136e+02_rb,0.53927e+02_rb,0.54851e+02_rb /)
      kbo(:,23, 4) = (/ &
        & 0.52049e+02_rb,0.52596e+02_rb,0.53278e+02_rb,0.54094e+02_rb,0.55044e+02_rb /)
      kbo(:,24, 4) = (/ &
        & 0.52148e+02_rb,0.52723e+02_rb,0.53431e+02_rb,0.54274e+02_rb,0.55252e+02_rb /)
      kbo(:,25, 4) = (/ &
        & 0.52259e+02_rb,0.52862e+02_rb,0.53599e+02_rb,0.54471e+02_rb,0.55477e+02_rb /)
      kbo(:,26, 4) = (/ &
        & 0.52387e+02_rb,0.53022e+02_rb,0.53790e+02_rb,0.54693e+02_rb,0.55730e+02_rb /)
      kbo(:,27, 4) = (/ &
        & 0.52525e+02_rb,0.53191e+02_rb,0.53992e+02_rb,0.54926e+02_rb,0.55995e+02_rb /)
      kbo(:,28, 4) = (/ &
        & 0.52672e+02_rb,0.53371e+02_rb,0.54203e+02_rb,0.55170e+02_rb,0.56271e+02_rb /)
      kbo(:,29, 4) = (/ &
        & 0.52833e+02_rb,0.53565e+02_rb,0.54430e+02_rb,0.55430e+02_rb,0.56565e+02_rb /)
      kbo(:,30, 4) = (/ &
        & 0.53003e+02_rb,0.53768e+02_rb,0.54668e+02_rb,0.55702e+02_rb,0.56870e+02_rb /)
      kbo(:,31, 4) = (/ &
        & 0.53186e+02_rb,0.53986e+02_rb,0.54920e+02_rb,0.55988e+02_rb,0.57190e+02_rb /)
      kbo(:,32, 4) = (/ &
        & 0.53381e+02_rb,0.54216e+02_rb,0.55184e+02_rb,0.56287e+02_rb,0.57525e+02_rb /)
      kbo(:,33, 4) = (/ &
        & 0.53587e+02_rb,0.54457e+02_rb,0.55460e+02_rb,0.56599e+02_rb,0.57871e+02_rb /)
      kbo(:,34, 4) = (/ &
        & 0.53773e+02_rb,0.54673e+02_rb,0.55707e+02_rb,0.56876e+02_rb,0.58179e+02_rb /)
      kbo(:,35, 4) = (/ &
        & 0.53883e+02_rb,0.54800e+02_rb,0.55852e+02_rb,0.57039e+02_rb,0.58359e+02_rb /)
      kbo(:,36, 4) = (/ &
        & 0.53901e+02_rb,0.54822e+02_rb,0.55876e+02_rb,0.57066e+02_rb,0.58389e+02_rb /)
      kbo(:,37, 4) = (/ &
        & 0.53791e+02_rb,0.54694e+02_rb,0.55731e+02_rb,0.56903e+02_rb,0.58208e+02_rb /)
      kbo(:,38, 4) = (/ &
        & 0.53676e+02_rb,0.54560e+02_rb,0.55578e+02_rb,0.56731e+02_rb,0.58018e+02_rb /)
      kbo(:,39, 4) = (/ &
        & 0.53563e+02_rb,0.54429e+02_rb,0.55429e+02_rb,0.56563e+02_rb,0.57832e+02_rb /)
      kbo(:,40, 4) = (/ &
        & 0.53375e+02_rb,0.54208e+02_rb,0.55175e+02_rb,0.56277e+02_rb,0.57513e+02_rb /)
      kbo(:,41, 4) = (/ &
        & 0.53184e+02_rb,0.53983e+02_rb,0.54916e+02_rb,0.55984e+02_rb,0.57186e+02_rb /)
      kbo(:,42, 4) = (/ &
        & 0.53001e+02_rb,0.53766e+02_rb,0.54665e+02_rb,0.55699e+02_rb,0.56866e+02_rb /)
      kbo(:,43, 4) = (/ &
        & 0.52790e+02_rb,0.53513e+02_rb,0.54370e+02_rb,0.55362e+02_rb,0.56488e+02_rb /)
      kbo(:,44, 4) = (/ &
        & 0.52577e+02_rb,0.53254e+02_rb,0.54066e+02_rb,0.55012e+02_rb,0.56093e+02_rb /)
      kbo(:,45, 4) = (/ &
        & 0.52379e+02_rb,0.53011e+02_rb,0.53777e+02_rb,0.54678e+02_rb,0.55713e+02_rb /)
      kbo(:,46, 4) = (/ &
        & 0.52184e+02_rb,0.52768e+02_rb,0.53486e+02_rb,0.54339e+02_rb,0.55326e+02_rb /)
      kbo(:,47, 4) = (/ &
        & 0.51987e+02_rb,0.52517e+02_rb,0.53182e+02_rb,0.53980e+02_rb,0.54913e+02_rb /)
      kbo(:,48, 4) = (/ &
        & 0.51812e+02_rb,0.52288e+02_rb,0.52898e+02_rb,0.53643e+02_rb,0.54522e+02_rb /)
      kbo(:,49, 4) = (/ &
        & 0.51659e+02_rb,0.52081e+02_rb,0.52637e+02_rb,0.53327e+02_rb,0.54152e+02_rb /)
      kbo(:,50, 4) = (/ &
        & 0.51535e+02_rb,0.51906e+02_rb,0.52412e+02_rb,0.53052e+02_rb,0.53826e+02_rb /)
      kbo(:,51, 4) = (/ &
        & 0.51433e+02_rb,0.51755e+02_rb,0.52211e+02_rb,0.52802e+02_rb,0.53527e+02_rb /)
      kbo(:,52, 4) = (/ &
        & 0.51349e+02_rb,0.51622e+02_rb,0.52029e+02_rb,0.52571e+02_rb,0.53247e+02_rb /)
      kbo(:,53, 4) = (/ &
        & 0.51285e+02_rb,0.51507e+02_rb,0.51865e+02_rb,0.52358e+02_rb,0.52985e+02_rb /)
      kbo(:,54, 4) = (/ &
        & 0.51240e+02_rb,0.51417e+02_rb,0.51730e+02_rb,0.52178e+02_rb,0.52760e+02_rb /)
      kbo(:,55, 4) = (/ &
        & 0.51211e+02_rb,0.51346e+02_rb,0.51615e+02_rb,0.52020e+02_rb,0.52559e+02_rb /)
      kbo(:,56, 4) = (/ &
        & 0.51194e+02_rb,0.51288e+02_rb,0.51514e+02_rb,0.51875e+02_rb,0.52371e+02_rb /)
      kbo(:,57, 4) = (/ &
        & 0.51190e+02_rb,0.51245e+02_rb,0.51426e+02_rb,0.51746e+02_rb,0.52198e+02_rb /)
      kbo(:,58, 4) = (/ &
        & 0.51200e+02_rb,0.51216e+02_rb,0.51357e+02_rb,0.51635e+02_rb,0.52046e+02_rb /)
      kbo(:,59, 4) = (/ &
        & 0.51207e+02_rb,0.51207e+02_rb,0.51333e+02_rb,0.51594e+02_rb,0.51989e+02_rb /)
      kbo(:,13, 5) = (/ &
        & 0.17051e+03_rb,0.17168e+03_rb,0.17297e+03_rb,0.17437e+03_rb,0.17590e+03_rb /)
      kbo(:,14, 5) = (/ &
        & 0.17061e+03_rb,0.17179e+03_rb,0.17309e+03_rb,0.17451e+03_rb,0.17604e+03_rb /)
      kbo(:,15, 5) = (/ &
        & 0.17073e+03_rb,0.17192e+03_rb,0.17323e+03_rb,0.17466e+03_rb,0.17621e+03_rb /)
      kbo(:,16, 5) = (/ &
        & 0.17085e+03_rb,0.17205e+03_rb,0.17338e+03_rb,0.17482e+03_rb,0.17638e+03_rb /)
      kbo(:,17, 5) = (/ &
        & 0.17097e+03_rb,0.17218e+03_rb,0.17352e+03_rb,0.17497e+03_rb,0.17655e+03_rb /)
      kbo(:,18, 5) = (/ &
        & 0.17109e+03_rb,0.17232e+03_rb,0.17366e+03_rb,0.17513e+03_rb,0.17671e+03_rb /)
      kbo(:,19, 5) = (/ &
        & 0.17121e+03_rb,0.17245e+03_rb,0.17381e+03_rb,0.17528e+03_rb,0.17688e+03_rb /)
      kbo(:,20, 5) = (/ &
        & 0.17134e+03_rb,0.17260e+03_rb,0.17397e+03_rb,0.17547e+03_rb,0.17708e+03_rb /)
      kbo(:,21, 5) = (/ &
        & 0.17149e+03_rb,0.17276e+03_rb,0.17415e+03_rb,0.17565e+03_rb,0.17728e+03_rb /)
      kbo(:,22, 5) = (/ &
        & 0.17172e+03_rb,0.17301e+03_rb,0.17442e+03_rb,0.17594e+03_rb,0.17759e+03_rb /)
      kbo(:,23, 5) = (/ &
        & 0.17195e+03_rb,0.17327e+03_rb,0.17470e+03_rb,0.17625e+03_rb,0.17792e+03_rb /)
      kbo(:,24, 5) = (/ &
        & 0.17221e+03_rb,0.17355e+03_rb,0.17500e+03_rb,0.17658e+03_rb,0.17827e+03_rb /)
      kbo(:,25, 5) = (/ &
        & 0.17248e+03_rb,0.17385e+03_rb,0.17533e+03_rb,0.17693e+03_rb,0.17865e+03_rb /)
      kbo(:,26, 5) = (/ &
        & 0.17279e+03_rb,0.17418e+03_rb,0.17569e+03_rb,0.17732e+03_rb,0.17906e+03_rb /)
      kbo(:,27, 5) = (/ &
        & 0.17311e+03_rb,0.17453e+03_rb,0.17606e+03_rb,0.17772e+03_rb,0.17949e+03_rb /)
      kbo(:,28, 5) = (/ &
        & 0.17344e+03_rb,0.17488e+03_rb,0.17645e+03_rb,0.17813e+03_rb,0.17994e+03_rb /)
      kbo(:,29, 5) = (/ &
        & 0.17379e+03_rb,0.17526e+03_rb,0.17686e+03_rb,0.17857e+03_rb,0.18040e+03_rb /)
      kbo(:,30, 5) = (/ &
        & 0.17414e+03_rb,0.17565e+03_rb,0.17727e+03_rb,0.17902e+03_rb,0.18088e+03_rb /)
      kbo(:,31, 5) = (/ &
        & 0.17452e+03_rb,0.17605e+03_rb,0.17771e+03_rb,0.17948e+03_rb,0.18137e+03_rb /)
      kbo(:,32, 5) = (/ &
        & 0.17491e+03_rb,0.17647e+03_rb,0.17816e+03_rb,0.17996e+03_rb,0.18188e+03_rb /)
      kbo(:,33, 5) = (/ &
        & 0.17531e+03_rb,0.17690e+03_rb,0.17862e+03_rb,0.18045e+03_rb,0.18241e+03_rb /)
      kbo(:,34, 5) = (/ &
        & 0.17566e+03_rb,0.17728e+03_rb,0.17903e+03_rb,0.18089e+03_rb,0.18287e+03_rb /)
      kbo(:,35, 5) = (/ &
        & 0.17586e+03_rb,0.17750e+03_rb,0.17926e+03_rb,0.18114e+03_rb,0.18313e+03_rb /)
      kbo(:,36, 5) = (/ &
        & 0.17590e+03_rb,0.17754e+03_rb,0.17930e+03_rb,0.18118e+03_rb,0.18318e+03_rb /)
      kbo(:,37, 5) = (/ &
        & 0.17569e+03_rb,0.17732e+03_rb,0.17906e+03_rb,0.18093e+03_rb,0.18291e+03_rb /)
      kbo(:,38, 5) = (/ &
        & 0.17547e+03_rb,0.17708e+03_rb,0.17881e+03_rb,0.18066e+03_rb,0.18263e+03_rb /)
      kbo(:,39, 5) = (/ &
        & 0.17526e+03_rb,0.17685e+03_rb,0.17857e+03_rb,0.18040e+03_rb,0.18235e+03_rb /)
      kbo(:,40, 5) = (/ &
        & 0.17489e+03_rb,0.17646e+03_rb,0.17814e+03_rb,0.17995e+03_rb,0.18187e+03_rb /)
      kbo(:,41, 5) = (/ &
        & 0.17451e+03_rb,0.17605e+03_rb,0.17770e+03_rb,0.17948e+03_rb,0.18137e+03_rb /)
      kbo(:,42, 5) = (/ &
        & 0.17414e+03_rb,0.17565e+03_rb,0.17727e+03_rb,0.17901e+03_rb,0.18087e+03_rb /)
      kbo(:,43, 5) = (/ &
        & 0.17369e+03_rb,0.17516e+03_rb,0.17675e+03_rb,0.17846e+03_rb,0.18028e+03_rb /)
      kbo(:,44, 5) = (/ &
        & 0.17323e+03_rb,0.17465e+03_rb,0.17620e+03_rb,0.17787e+03_rb,0.17965e+03_rb /)
      kbo(:,45, 5) = (/ &
        & 0.17277e+03_rb,0.17416e+03_rb,0.17567e+03_rb,0.17729e+03_rb,0.17904e+03_rb /)
      kbo(:,46, 5) = (/ &
        & 0.17230e+03_rb,0.17365e+03_rb,0.17511e+03_rb,0.17669e+03_rb,0.17840e+03_rb /)
      kbo(:,47, 5) = (/ &
        & 0.17179e+03_rb,0.17309e+03_rb,0.17451e+03_rb,0.17604e+03_rb,0.17770e+03_rb /)
      kbo(:,48, 5) = (/ &
        & 0.17130e+03_rb,0.17255e+03_rb,0.17392e+03_rb,0.17541e+03_rb,0.17702e+03_rb /)
      kbo(:,49, 5) = (/ &
        & 0.17083e+03_rb,0.17204e+03_rb,0.17336e+03_rb,0.17480e+03_rb,0.17636e+03_rb /)
      kbo(:,50, 5) = (/ &
        & 0.17041e+03_rb,0.17157e+03_rb,0.17285e+03_rb,0.17424e+03_rb,0.17576e+03_rb /)
      kbo(:,51, 5) = (/ &
        & 0.17002e+03_rb,0.17113e+03_rb,0.17237e+03_rb,0.17372e+03_rb,0.17519e+03_rb /)
      kbo(:,52, 5) = (/ &
        & 0.16964e+03_rb,0.17071e+03_rb,0.17190e+03_rb,0.17321e+03_rb,0.17464e+03_rb /)
      kbo(:,53, 5) = (/ &
        & 0.16927e+03_rb,0.17031e+03_rb,0.17145e+03_rb,0.17272e+03_rb,0.17411e+03_rb /)
      kbo(:,54, 5) = (/ &
        & 0.16896e+03_rb,0.16995e+03_rb,0.17106e+03_rb,0.17228e+03_rb,0.17363e+03_rb /)
      kbo(:,55, 5) = (/ &
        & 0.16866e+03_rb,0.16962e+03_rb,0.17069e+03_rb,0.17188e+03_rb,0.17318e+03_rb /)
      kbo(:,56, 5) = (/ &
        & 0.16839e+03_rb,0.16930e+03_rb,0.17033e+03_rb,0.17148e+03_rb,0.17275e+03_rb /)
      kbo(:,57, 5) = (/ &
        & 0.16812e+03_rb,0.16899e+03_rb,0.16999e+03_rb,0.17110e+03_rb,0.17233e+03_rb /)
      kbo(:,58, 5) = (/ &
        & 0.16788e+03_rb,0.16871e+03_rb,0.16968e+03_rb,0.17075e+03_rb,0.17195e+03_rb /)
      kbo(:,59, 5) = (/ &
        & 0.16779e+03_rb,0.16861e+03_rb,0.16955e+03_rb,0.17062e+03_rb,0.17180e+03_rb /)
      kbo(:,13, 6) = (/ &
        & 0.38122e+03_rb,0.38406e+03_rb,0.38668e+03_rb,0.38908e+03_rb,0.39125e+03_rb /)
      kbo(:,14, 6) = (/ &
        & 0.38148e+03_rb,0.38431e+03_rb,0.38691e+03_rb,0.38929e+03_rb,0.39144e+03_rb /)
      kbo(:,15, 6) = (/ &
        & 0.38178e+03_rb,0.38458e+03_rb,0.38716e+03_rb,0.38951e+03_rb,0.39164e+03_rb /)
      kbo(:,16, 6) = (/ &
        & 0.38209e+03_rb,0.38487e+03_rb,0.38742e+03_rb,0.38975e+03_rb,0.39186e+03_rb /)
      kbo(:,17, 6) = (/ &
        & 0.38239e+03_rb,0.38514e+03_rb,0.38767e+03_rb,0.38998e+03_rb,0.39206e+03_rb /)
      kbo(:,18, 6) = (/ &
        & 0.38268e+03_rb,0.38541e+03_rb,0.38792e+03_rb,0.39020e+03_rb,0.39226e+03_rb /)
      kbo(:,19, 6) = (/ &
        & 0.38297e+03_rb,0.38568e+03_rb,0.38816e+03_rb,0.39042e+03_rb,0.39246e+03_rb /)
      kbo(:,20, 6) = (/ &
        & 0.38330e+03_rb,0.38598e+03_rb,0.38844e+03_rb,0.39067e+03_rb,0.39268e+03_rb /)
      kbo(:,21, 6) = (/ &
        & 0.38363e+03_rb,0.38629e+03_rb,0.38872e+03_rb,0.39092e+03_rb,0.39291e+03_rb /)
      kbo(:,22, 6) = (/ &
        & 0.38414e+03_rb,0.38675e+03_rb,0.38914e+03_rb,0.39131e+03_rb,0.39325e+03_rb /)
      kbo(:,23, 6) = (/ &
        & 0.38466e+03_rb,0.38723e+03_rb,0.38958e+03_rb,0.39170e+03_rb,0.39360e+03_rb /)
      kbo(:,24, 6) = (/ &
        & 0.38520e+03_rb,0.38772e+03_rb,0.39002e+03_rb,0.39210e+03_rb,0.39395e+03_rb /)
      kbo(:,25, 6) = (/ &
        & 0.38575e+03_rb,0.38823e+03_rb,0.39048e+03_rb,0.39251e+03_rb,0.39432e+03_rb /)
      kbo(:,26, 6) = (/ &
        & 0.38635e+03_rb,0.38877e+03_rb,0.39098e+03_rb,0.39295e+03_rb,0.39470e+03_rb /)
      kbo(:,27, 6) = (/ &
        & 0.38694e+03_rb,0.38932e+03_rb,0.39146e+03_rb,0.39339e+03_rb,0.39509e+03_rb /)
      kbo(:,28, 6) = (/ &
        & 0.38753e+03_rb,0.38985e+03_rb,0.39194e+03_rb,0.39381e+03_rb,0.39546e+03_rb /)
      kbo(:,29, 6) = (/ &
        & 0.38813e+03_rb,0.39039e+03_rb,0.39243e+03_rb,0.39424e+03_rb,0.39583e+03_rb /)
      kbo(:,30, 6) = (/ &
        & 0.38871e+03_rb,0.39092e+03_rb,0.39290e+03_rb,0.39466e+03_rb,0.39620e+03_rb /)
      kbo(:,31, 6) = (/ &
        & 0.38930e+03_rb,0.39145e+03_rb,0.39338e+03_rb,0.39508e+03_rb,0.39655e+03_rb /)
      kbo(:,32, 6) = (/ &
        & 0.38988e+03_rb,0.39197e+03_rb,0.39384e+03_rb,0.39548e+03_rb,0.39690e+03_rb /)
      kbo(:,33, 6) = (/ &
        & 0.39045e+03_rb,0.39248e+03_rb,0.39429e+03_rb,0.39587e+03_rb,0.39723e+03_rb /)
      kbo(:,34, 6) = (/ &
        & 0.39093e+03_rb,0.39291e+03_rb,0.39467e+03_rb,0.39620e+03_rb,0.39751e+03_rb /)
      kbo(:,35, 6) = (/ &
        & 0.39120e+03_rb,0.39316e+03_rb,0.39488e+03_rb,0.39639e+03_rb,0.39766e+03_rb /)
      kbo(:,36, 6) = (/ &
        & 0.39125e+03_rb,0.39320e+03_rb,0.39492e+03_rb,0.39642e+03_rb,0.39769e+03_rb /)
      kbo(:,37, 6) = (/ &
        & 0.39098e+03_rb,0.39295e+03_rb,0.39471e+03_rb,0.39623e+03_rb,0.39754e+03_rb /)
      kbo(:,38, 6) = (/ &
        & 0.39068e+03_rb,0.39269e+03_rb,0.39447e+03_rb,0.39603e+03_rb,0.39737e+03_rb /)
      kbo(:,39, 6) = (/ &
        & 0.39039e+03_rb,0.39243e+03_rb,0.39424e+03_rb,0.39583e+03_rb,0.39720e+03_rb /)
      kbo(:,40, 6) = (/ &
        & 0.38986e+03_rb,0.39196e+03_rb,0.39382e+03_rb,0.39547e+03_rb,0.39689e+03_rb /)
      kbo(:,41, 6) = (/ &
        & 0.38929e+03_rb,0.39144e+03_rb,0.39337e+03_rb,0.39507e+03_rb,0.39655e+03_rb /)
      kbo(:,42, 6) = (/ &
        & 0.38871e+03_rb,0.39092e+03_rb,0.39290e+03_rb,0.39466e+03_rb,0.39619e+03_rb /)
      kbo(:,43, 6) = (/ &
        & 0.38797e+03_rb,0.39025e+03_rb,0.39230e+03_rb,0.39413e+03_rb,0.39574e+03_rb /)
      kbo(:,44, 6) = (/ &
        & 0.38715e+03_rb,0.38951e+03_rb,0.39164e+03_rb,0.39354e+03_rb,0.39522e+03_rb /)
      kbo(:,45, 6) = (/ &
        & 0.38631e+03_rb,0.38874e+03_rb,0.39094e+03_rb,0.39292e+03_rb,0.39468e+03_rb /)
      kbo(:,46, 6) = (/ &
        & 0.38538e+03_rb,0.38789e+03_rb,0.39018e+03_rb,0.39224e+03_rb,0.39407e+03_rb /)
      kbo(:,47, 6) = (/ &
        & 0.38431e+03_rb,0.38691e+03_rb,0.38929e+03_rb,0.39144e+03_rb,0.39336e+03_rb /)
      kbo(:,48, 6) = (/ &
        & 0.38320e+03_rb,0.38589e+03_rb,0.38836e+03_rb,0.39060e+03_rb,0.39262e+03_rb /)
      kbo(:,49, 6) = (/ &
        & 0.38206e+03_rb,0.38484e+03_rb,0.38739e+03_rb,0.38972e+03_rb,0.39183e+03_rb /)
      kbo(:,50, 6) = (/ &
        & 0.38095e+03_rb,0.38382e+03_rb,0.38646e+03_rb,0.38887e+03_rb,0.39106e+03_rb /)
      kbo(:,51, 6) = (/ &
        & 0.37985e+03_rb,0.38279e+03_rb,0.38552e+03_rb,0.38802e+03_rb,0.39029e+03_rb /)
      kbo(:,52, 6) = (/ &
        & 0.37873e+03_rb,0.38175e+03_rb,0.38455e+03_rb,0.38713e+03_rb,0.38949e+03_rb /)
      kbo(:,53, 6) = (/ &
        & 0.37762e+03_rb,0.38067e+03_rb,0.38355e+03_rb,0.38621e+03_rb,0.38865e+03_rb /)
      kbo(:,54, 6) = (/ &
        & 0.37661e+03_rb,0.37965e+03_rb,0.38261e+03_rb,0.38535e+03_rb,0.38786e+03_rb /)
      kbo(:,55, 6) = (/ &
        & 0.37559e+03_rb,0.37867e+03_rb,0.38169e+03_rb,0.38450e+03_rb,0.38708e+03_rb /)
      kbo(:,56, 6) = (/ &
        & 0.37459e+03_rb,0.37768e+03_rb,0.38074e+03_rb,0.38362e+03_rb,0.38628e+03_rb /)
      kbo(:,57, 6) = (/ &
        & 0.37353e+03_rb,0.37673e+03_rb,0.37977e+03_rb,0.38272e+03_rb,0.38545e+03_rb /)
      kbo(:,58, 6) = (/ &
        & 0.37251e+03_rb,0.37577e+03_rb,0.37885e+03_rb,0.38185e+03_rb,0.38465e+03_rb /)
      kbo(:,59, 6) = (/ &
        & 0.37212e+03_rb,0.37539e+03_rb,0.37847e+03_rb,0.38150e+03_rb,0.38432e+03_rb /)
      kbo(:,13, 7) = (/ &
        & 0.62583e+03_rb,0.62623e+03_rb,0.62691e+03_rb,0.62787e+03_rb,0.62909e+03_rb /)
      kbo(:,14, 7) = (/ &
        & 0.62586e+03_rb,0.62628e+03_rb,0.62699e+03_rb,0.62797e+03_rb,0.62921e+03_rb /)
      kbo(:,15, 7) = (/ &
        & 0.62589e+03_rb,0.62634e+03_rb,0.62708e+03_rb,0.62809e+03_rb,0.62935e+03_rb /)
      kbo(:,16, 7) = (/ &
        & 0.62593e+03_rb,0.62640e+03_rb,0.62717e+03_rb,0.62821e+03_rb,0.62950e+03_rb /)
      kbo(:,17, 7) = (/ &
        & 0.62597e+03_rb,0.62647e+03_rb,0.62727e+03_rb,0.62833e+03_rb,0.62965e+03_rb /)
      kbo(:,18, 7) = (/ &
        & 0.62601e+03_rb,0.62654e+03_rb,0.62737e+03_rb,0.62846e+03_rb,0.62980e+03_rb /)
      kbo(:,19, 7) = (/ &
        & 0.62605e+03_rb,0.62661e+03_rb,0.62746e+03_rb,0.62858e+03_rb,0.62995e+03_rb /)
      kbo(:,20, 7) = (/ &
        & 0.62610e+03_rb,0.62670e+03_rb,0.62758e+03_rb,0.62873e+03_rb,0.63012e+03_rb /)
      kbo(:,21, 7) = (/ &
        & 0.62615e+03_rb,0.62679e+03_rb,0.62770e+03_rb,0.62888e+03_rb,0.63031e+03_rb /)
      kbo(:,22, 7) = (/ &
        & 0.62625e+03_rb,0.62694e+03_rb,0.62790e+03_rb,0.62913e+03_rb,0.63060e+03_rb /)
      kbo(:,23, 7) = (/ &
        & 0.62636e+03_rb,0.62710e+03_rb,0.62812e+03_rb,0.62939e+03_rb,0.63091e+03_rb /)
      kbo(:,24, 7) = (/ &
        & 0.62648e+03_rb,0.62729e+03_rb,0.62835e+03_rb,0.62968e+03_rb,0.63125e+03_rb /)
      kbo(:,25, 7) = (/ &
        & 0.62663e+03_rb,0.62749e+03_rb,0.62861e+03_rb,0.62999e+03_rb,0.63162e+03_rb /)
      kbo(:,26, 7) = (/ &
        & 0.62681e+03_rb,0.62773e+03_rb,0.62891e+03_rb,0.63035e+03_rb,0.63203e+03_rb /)
      kbo(:,27, 7) = (/ &
        & 0.62700e+03_rb,0.62799e+03_rb,0.62923e+03_rb,0.63072e+03_rb,0.63247e+03_rb /)
      kbo(:,28, 7) = (/ &
        & 0.62721e+03_rb,0.62826e+03_rb,0.62956e+03_rb,0.63112e+03_rb,0.63292e+03_rb /)
      kbo(:,29, 7) = (/ &
        & 0.62745e+03_rb,0.62856e+03_rb,0.62993e+03_rb,0.63154e+03_rb,0.63341e+03_rb /)
      kbo(:,30, 7) = (/ &
        & 0.62770e+03_rb,0.62888e+03_rb,0.63031e+03_rb,0.63198e+03_rb,0.63392e+03_rb /)
      kbo(:,31, 7) = (/ &
        & 0.62798e+03_rb,0.62922e+03_rb,0.63071e+03_rb,0.63245e+03_rb,0.63446e+03_rb /)
      kbo(:,32, 7) = (/ &
        & 0.62828e+03_rb,0.62958e+03_rb,0.63114e+03_rb,0.63295e+03_rb,0.63503e+03_rb /)
      kbo(:,33, 7) = (/ &
        & 0.62860e+03_rb,0.62997e+03_rb,0.63159e+03_rb,0.63347e+03_rb,0.63562e+03_rb /)
      kbo(:,34, 7) = (/ &
        & 0.62889e+03_rb,0.63031e+03_rb,0.63199e+03_rb,0.63393e+03_rb,0.63614e+03_rb /)
      kbo(:,35, 7) = (/ &
        & 0.62906e+03_rb,0.63052e+03_rb,0.63223e+03_rb,0.63421e+03_rb,0.63644e+03_rb /)
      kbo(:,36, 7) = (/ &
        & 0.62909e+03_rb,0.63055e+03_rb,0.63227e+03_rb,0.63425e+03_rb,0.63649e+03_rb /)
      kbo(:,37, 7) = (/ &
        & 0.62891e+03_rb,0.63035e+03_rb,0.63203e+03_rb,0.63397e+03_rb,0.63619e+03_rb /)
      kbo(:,38, 7) = (/ &
        & 0.62873e+03_rb,0.63013e+03_rb,0.63178e+03_rb,0.63369e+03_rb,0.63586e+03_rb /)
      kbo(:,39, 7) = (/ &
        & 0.62856e+03_rb,0.62992e+03_rb,0.63154e+03_rb,0.63341e+03_rb,0.63555e+03_rb /)
      kbo(:,40, 7) = (/ &
        & 0.62827e+03_rb,0.62957e+03_rb,0.63113e+03_rb,0.63293e+03_rb,0.63501e+03_rb /)
      kbo(:,41, 7) = (/ &
        & 0.62798e+03_rb,0.62922e+03_rb,0.63071e+03_rb,0.63245e+03_rb,0.63445e+03_rb /)
      kbo(:,42, 7) = (/ &
        & 0.62770e+03_rb,0.62887e+03_rb,0.63030e+03_rb,0.63198e+03_rb,0.63391e+03_rb /)
      kbo(:,43, 7) = (/ &
        & 0.62739e+03_rb,0.62848e+03_rb,0.62983e+03_rb,0.63143e+03_rb,0.63328e+03_rb /)
      kbo(:,44, 7) = (/ &
        & 0.62708e+03_rb,0.62808e+03_rb,0.62935e+03_rb,0.63086e+03_rb,0.63263e+03_rb /)
      kbo(:,45, 7) = (/ &
        & 0.62679e+03_rb,0.62771e+03_rb,0.62889e+03_rb,0.63032e+03_rb,0.63200e+03_rb /)
      kbo(:,46, 7) = (/ &
        & 0.62653e+03_rb,0.62735e+03_rb,0.62844e+03_rb,0.62978e+03_rb,0.63137e+03_rb /)
      kbo(:,47, 7) = (/ &
        & 0.62629e+03_rb,0.62699e+03_rb,0.62797e+03_rb,0.62921e+03_rb,0.63070e+03_rb /)
      kbo(:,48, 7) = (/ &
        & 0.62608e+03_rb,0.62667e+03_rb,0.62755e+03_rb,0.62868e+03_rb,0.63007e+03_rb /)
      kbo(:,49, 7) = (/ &
        & 0.62592e+03_rb,0.62640e+03_rb,0.62716e+03_rb,0.62820e+03_rb,0.62948e+03_rb /)
      kbo(:,50, 7) = (/ &
        & 0.62580e+03_rb,0.62619e+03_rb,0.62684e+03_rb,0.62778e+03_rb,0.62897e+03_rb /)
      kbo(:,51, 7) = (/ &
        & 0.62580e+03_rb,0.62602e+03_rb,0.62657e+03_rb,0.62741e+03_rb,0.62850e+03_rb /)
      kbo(:,52, 7) = (/ &
        & 0.62578e+03_rb,0.62589e+03_rb,0.62633e+03_rb,0.62707e+03_rb,0.62807e+03_rb /)
      kbo(:,53, 7) = (/ &
        & 0.62574e+03_rb,0.62578e+03_rb,0.62614e+03_rb,0.62677e+03_rb,0.62768e+03_rb /)
      kbo(:,54, 7) = (/ &
        & 0.62576e+03_rb,0.62580e+03_rb,0.62600e+03_rb,0.62652e+03_rb,0.62734e+03_rb /)
      kbo(:,55, 7) = (/ &
        & 0.62585e+03_rb,0.62578e+03_rb,0.62588e+03_rb,0.62632e+03_rb,0.62705e+03_rb /)
      kbo(:,56, 7) = (/ &
        & 0.62587e+03_rb,0.62576e+03_rb,0.62579e+03_rb,0.62615e+03_rb,0.62678e+03_rb /)
      kbo(:,57, 7) = (/ &
        & 0.62591e+03_rb,0.62574e+03_rb,0.62580e+03_rb,0.62601e+03_rb,0.62655e+03_rb /)
      kbo(:,58, 7) = (/ &
        & 0.62591e+03_rb,0.62583e+03_rb,0.62578e+03_rb,0.62590e+03_rb,0.62636e+03_rb /)
      kbo(:,59, 7) = (/ &
        & 0.62590e+03_rb,0.62586e+03_rb,0.62578e+03_rb,0.62586e+03_rb,0.62628e+03_rb /)
      kbo(:,13, 8) = (/ &
        & 0.81821e+03_rb,0.81877e+03_rb,0.81935e+03_rb,0.81999e+03_rb,0.82074e+03_rb /)
      kbo(:,14, 8) = (/ &
        & 0.81826e+03_rb,0.81882e+03_rb,0.81939e+03_rb,0.82006e+03_rb,0.82081e+03_rb /)
      kbo(:,15, 8) = (/ &
        & 0.81831e+03_rb,0.81888e+03_rb,0.81946e+03_rb,0.82012e+03_rb,0.82089e+03_rb /)
      kbo(:,16, 8) = (/ &
        & 0.81836e+03_rb,0.81895e+03_rb,0.81952e+03_rb,0.82021e+03_rb,0.82098e+03_rb /)
      kbo(:,17, 8) = (/ &
        & 0.81842e+03_rb,0.81901e+03_rb,0.81959e+03_rb,0.82028e+03_rb,0.82106e+03_rb /)
      kbo(:,18, 8) = (/ &
        & 0.81847e+03_rb,0.81907e+03_rb,0.81965e+03_rb,0.82036e+03_rb,0.82116e+03_rb /)
      kbo(:,19, 8) = (/ &
        & 0.81853e+03_rb,0.81913e+03_rb,0.81972e+03_rb,0.82043e+03_rb,0.82123e+03_rb /)
      kbo(:,20, 8) = (/ &
        & 0.81860e+03_rb,0.81919e+03_rb,0.81980e+03_rb,0.82052e+03_rb,0.82133e+03_rb /)
      kbo(:,21, 8) = (/ &
        & 0.81868e+03_rb,0.81925e+03_rb,0.81988e+03_rb,0.82061e+03_rb,0.82143e+03_rb /)
      kbo(:,22, 8) = (/ &
        & 0.81879e+03_rb,0.81936e+03_rb,0.82001e+03_rb,0.82076e+03_rb,0.82160e+03_rb /)
      kbo(:,23, 8) = (/ &
        & 0.81889e+03_rb,0.81947e+03_rb,0.82015e+03_rb,0.82091e+03_rb,0.82177e+03_rb /)
      kbo(:,24, 8) = (/ &
        & 0.81902e+03_rb,0.81960e+03_rb,0.82030e+03_rb,0.82108e+03_rb,0.82195e+03_rb /)
      kbo(:,25, 8) = (/ &
        & 0.81914e+03_rb,0.81974e+03_rb,0.82046e+03_rb,0.82126e+03_rb,0.82214e+03_rb /)
      kbo(:,26, 8) = (/ &
        & 0.81927e+03_rb,0.81990e+03_rb,0.82063e+03_rb,0.82145e+03_rb,0.82236e+03_rb /)
      kbo(:,27, 8) = (/ &
        & 0.81941e+03_rb,0.82006e+03_rb,0.82082e+03_rb,0.82166e+03_rb,0.82258e+03_rb /)
      kbo(:,28, 8) = (/ &
        & 0.81957e+03_rb,0.82024e+03_rb,0.82101e+03_rb,0.82188e+03_rb,0.82281e+03_rb /)
      kbo(:,29, 8) = (/ &
        & 0.81971e+03_rb,0.82042e+03_rb,0.82122e+03_rb,0.82210e+03_rb,0.82305e+03_rb /)
      kbo(:,30, 8) = (/ &
        & 0.81988e+03_rb,0.82061e+03_rb,0.82143e+03_rb,0.82233e+03_rb,0.82330e+03_rb /)
      kbo(:,31, 8) = (/ &
        & 0.82006e+03_rb,0.82081e+03_rb,0.82166e+03_rb,0.82258e+03_rb,0.82355e+03_rb /)
      kbo(:,32, 8) = (/ &
        & 0.82025e+03_rb,0.82103e+03_rb,0.82189e+03_rb,0.82283e+03_rb,0.82380e+03_rb /)
      kbo(:,33, 8) = (/ &
        & 0.82044e+03_rb,0.82124e+03_rb,0.82213e+03_rb,0.82308e+03_rb,0.82408e+03_rb /)
      kbo(:,34, 8) = (/ &
        & 0.82062e+03_rb,0.82144e+03_rb,0.82234e+03_rb,0.82330e+03_rb,0.82432e+03_rb /)
      kbo(:,35, 8) = (/ &
        & 0.82072e+03_rb,0.82155e+03_rb,0.82246e+03_rb,0.82342e+03_rb,0.82446e+03_rb /)
      kbo(:,36, 8) = (/ &
        & 0.82074e+03_rb,0.82157e+03_rb,0.82248e+03_rb,0.82345e+03_rb,0.82448e+03_rb /)
      kbo(:,37, 8) = (/ &
        & 0.82063e+03_rb,0.82146e+03_rb,0.82236e+03_rb,0.82332e+03_rb,0.82434e+03_rb /)
      kbo(:,38, 8) = (/ &
        & 0.82053e+03_rb,0.82134e+03_rb,0.82223e+03_rb,0.82319e+03_rb,0.82419e+03_rb /)
      kbo(:,39, 8) = (/ &
        & 0.82042e+03_rb,0.82122e+03_rb,0.82210e+03_rb,0.82305e+03_rb,0.82405e+03_rb /)
      kbo(:,40, 8) = (/ &
        & 0.82024e+03_rb,0.82102e+03_rb,0.82188e+03_rb,0.82282e+03_rb,0.82380e+03_rb /)
      kbo(:,41, 8) = (/ &
        & 0.82006e+03_rb,0.82081e+03_rb,0.82165e+03_rb,0.82257e+03_rb,0.82354e+03_rb /)
      kbo(:,42, 8) = (/ &
        & 0.81988e+03_rb,0.82061e+03_rb,0.82143e+03_rb,0.82233e+03_rb,0.82329e+03_rb /)
      kbo(:,43, 8) = (/ &
        & 0.81967e+03_rb,0.82037e+03_rb,0.82117e+03_rb,0.82204e+03_rb,0.82299e+03_rb /)
      kbo(:,44, 8) = (/ &
        & 0.81946e+03_rb,0.82013e+03_rb,0.82089e+03_rb,0.82174e+03_rb,0.82267e+03_rb /)
      kbo(:,45, 8) = (/ &
        & 0.81926e+03_rb,0.81989e+03_rb,0.82062e+03_rb,0.82144e+03_rb,0.82234e+03_rb /)
      kbo(:,46, 8) = (/ &
        & 0.81907e+03_rb,0.81965e+03_rb,0.82034e+03_rb,0.82114e+03_rb,0.82201e+03_rb /)
      kbo(:,47, 8) = (/ &
        & 0.81882e+03_rb,0.81939e+03_rb,0.82006e+03_rb,0.82081e+03_rb,0.82165e+03_rb /)
      kbo(:,48, 8) = (/ &
        & 0.81858e+03_rb,0.81917e+03_rb,0.81978e+03_rb,0.82050e+03_rb,0.82130e+03_rb /)
      kbo(:,49, 8) = (/ &
        & 0.81836e+03_rb,0.81894e+03_rb,0.81953e+03_rb,0.82019e+03_rb,0.82097e+03_rb /)
      kbo(:,50, 8) = (/ &
        & 0.81817e+03_rb,0.81871e+03_rb,0.81929e+03_rb,0.81993e+03_rb,0.82067e+03_rb /)
      kbo(:,51, 8) = (/ &
        & 0.81790e+03_rb,0.81850e+03_rb,0.81910e+03_rb,0.81967e+03_rb,0.82039e+03_rb /)
      kbo(:,52, 8) = (/ &
        & 0.81779e+03_rb,0.81830e+03_rb,0.81888e+03_rb,0.81945e+03_rb,0.82012e+03_rb /)
      kbo(:,53, 8) = (/ &
        & 0.81766e+03_rb,0.81812e+03_rb,0.81866e+03_rb,0.81924e+03_rb,0.81986e+03_rb /)
      kbo(:,54, 8) = (/ &
        & 0.81744e+03_rb,0.81785e+03_rb,0.81846e+03_rb,0.81906e+03_rb,0.81964e+03_rb /)
      kbo(:,55, 8) = (/ &
        & 0.81733e+03_rb,0.81777e+03_rb,0.81829e+03_rb,0.81886e+03_rb,0.81944e+03_rb /)
      kbo(:,56, 8) = (/ &
        & 0.81741e+03_rb,0.81767e+03_rb,0.81813e+03_rb,0.81867e+03_rb,0.81925e+03_rb /)
      kbo(:,57, 8) = (/ &
        & 0.81785e+03_rb,0.81749e+03_rb,0.81788e+03_rb,0.81848e+03_rb,0.81908e+03_rb /)
      kbo(:,58, 8) = (/ &
        & 0.81817e+03_rb,0.81731e+03_rb,0.81779e+03_rb,0.81832e+03_rb,0.81889e+03_rb /)
      kbo(:,59, 8) = (/ &
        & 0.81824e+03_rb,0.81736e+03_rb,0.81776e+03_rb,0.81826e+03_rb,0.81883e+03_rb /)
      kbo(:,13, 9) = (/ &
        & 0.94133e+03_rb,0.94438e+03_rb,0.94687e+03_rb,0.94883e+03_rb,0.95021e+03_rb /)
      kbo(:,14, 9) = (/ &
        & 0.94163e+03_rb,0.94463e+03_rb,0.94708e+03_rb,0.94898e+03_rb,0.95031e+03_rb /)
      kbo(:,15, 9) = (/ &
        & 0.94195e+03_rb,0.94490e+03_rb,0.94731e+03_rb,0.94914e+03_rb,0.95042e+03_rb /)
      kbo(:,16, 9) = (/ &
        & 0.94229e+03_rb,0.94518e+03_rb,0.94753e+03_rb,0.94930e+03_rb,0.95052e+03_rb /)
      kbo(:,17, 9) = (/ &
        & 0.94262e+03_rb,0.94544e+03_rb,0.94774e+03_rb,0.94946e+03_rb,0.95061e+03_rb /)
      kbo(:,18, 9) = (/ &
        & 0.94293e+03_rb,0.94570e+03_rb,0.94795e+03_rb,0.94960e+03_rb,0.95069e+03_rb /)
      kbo(:,19, 9) = (/ &
        & 0.94323e+03_rb,0.94595e+03_rb,0.94814e+03_rb,0.94974e+03_rb,0.95078e+03_rb /)
      kbo(:,20, 9) = (/ &
        & 0.94357e+03_rb,0.94624e+03_rb,0.94835e+03_rb,0.94989e+03_rb,0.95087e+03_rb /)
      kbo(:,21, 9) = (/ &
        & 0.94393e+03_rb,0.94654e+03_rb,0.94857e+03_rb,0.95003e+03_rb,0.95096e+03_rb /)
      kbo(:,22, 9) = (/ &
        & 0.94445e+03_rb,0.94695e+03_rb,0.94888e+03_rb,0.95025e+03_rb,0.95105e+03_rb /)
      kbo(:,23, 9) = (/ &
        & 0.94499e+03_rb,0.94737e+03_rb,0.94919e+03_rb,0.95045e+03_rb,0.95113e+03_rb /)
      kbo(:,24, 9) = (/ &
        & 0.94549e+03_rb,0.94778e+03_rb,0.94948e+03_rb,0.95063e+03_rb,0.95120e+03_rb /)
      kbo(:,25, 9) = (/ &
        & 0.94602e+03_rb,0.94819e+03_rb,0.94978e+03_rb,0.95082e+03_rb,0.95125e+03_rb /)
      kbo(:,26, 9) = (/ &
        & 0.94658e+03_rb,0.94861e+03_rb,0.95006e+03_rb,0.95096e+03_rb,0.95127e+03_rb /)
      kbo(:,27, 9) = (/ &
        & 0.94710e+03_rb,0.94900e+03_rb,0.95033e+03_rb,0.95109e+03_rb,0.95126e+03_rb /)
      kbo(:,28, 9) = (/ &
        & 0.94759e+03_rb,0.94937e+03_rb,0.95056e+03_rb,0.95118e+03_rb,0.95123e+03_rb /)
      kbo(:,29, 9) = (/ &
        & 0.94811e+03_rb,0.94973e+03_rb,0.95077e+03_rb,0.95124e+03_rb,0.95116e+03_rb /)
      kbo(:,30, 9) = (/ &
        & 0.94856e+03_rb,0.95003e+03_rb,0.95095e+03_rb,0.95127e+03_rb,0.95105e+03_rb /)
      kbo(:,31, 9) = (/ &
        & 0.94901e+03_rb,0.95032e+03_rb,0.95108e+03_rb,0.95127e+03_rb,0.95090e+03_rb /)
      kbo(:,32, 9) = (/ &
        & 0.94939e+03_rb,0.95057e+03_rb,0.95118e+03_rb,0.95121e+03_rb,0.95071e+03_rb /)
      kbo(:,33, 9) = (/ &
        & 0.94975e+03_rb,0.95079e+03_rb,0.95125e+03_rb,0.95115e+03_rb,0.95050e+03_rb /)
      kbo(:,34, 9) = (/ &
        & 0.95004e+03_rb,0.95095e+03_rb,0.95128e+03_rb,0.95105e+03_rb,0.95030e+03_rb /)
      kbo(:,35, 9) = (/ &
        & 0.95020e+03_rb,0.95102e+03_rb,0.95127e+03_rb,0.95098e+03_rb,0.95016e+03_rb /)
      kbo(:,36, 9) = (/ &
        & 0.95021e+03_rb,0.95103e+03_rb,0.95127e+03_rb,0.95096e+03_rb,0.95023e+03_rb /)
      kbo(:,37, 9) = (/ &
        & 0.95006e+03_rb,0.95096e+03_rb,0.95127e+03_rb,0.95104e+03_rb,0.95027e+03_rb /)
      kbo(:,38, 9) = (/ &
        & 0.94990e+03_rb,0.95087e+03_rb,0.95126e+03_rb,0.95110e+03_rb,0.95040e+03_rb /)
      kbo(:,39, 9) = (/ &
        & 0.94972e+03_rb,0.95077e+03_rb,0.95124e+03_rb,0.95117e+03_rb,0.95053e+03_rb /)
      kbo(:,40, 9) = (/ &
        & 0.94939e+03_rb,0.95057e+03_rb,0.95118e+03_rb,0.95123e+03_rb,0.95072e+03_rb /)
      kbo(:,41, 9) = (/ &
        & 0.94899e+03_rb,0.95032e+03_rb,0.95108e+03_rb,0.95127e+03_rb,0.95090e+03_rb /)
      kbo(:,42, 9) = (/ &
        & 0.94856e+03_rb,0.95004e+03_rb,0.95094e+03_rb,0.95127e+03_rb,0.95105e+03_rb /)
      kbo(:,43, 9) = (/ &
        & 0.94799e+03_rb,0.94963e+03_rb,0.95073e+03_rb,0.95123e+03_rb,0.95118e+03_rb /)
      kbo(:,44, 9) = (/ &
        & 0.94729e+03_rb,0.94914e+03_rb,0.95041e+03_rb,0.95112e+03_rb,0.95126e+03_rb /)
      kbo(:,45, 9) = (/ &
        & 0.94655e+03_rb,0.94858e+03_rb,0.95004e+03_rb,0.95095e+03_rb,0.95127e+03_rb /)
      kbo(:,46, 9) = (/ &
        & 0.94568e+03_rb,0.94792e+03_rb,0.94958e+03_rb,0.95070e+03_rb,0.95122e+03_rb /)
      kbo(:,47, 9) = (/ &
        & 0.94463e+03_rb,0.94708e+03_rb,0.94898e+03_rb,0.95031e+03_rb,0.95108e+03_rb /)
      kbo(:,48, 9) = (/ &
        & 0.94348e+03_rb,0.94616e+03_rb,0.94829e+03_rb,0.94984e+03_rb,0.95084e+03_rb /)
      kbo(:,49, 9) = (/ &
        & 0.94226e+03_rb,0.94515e+03_rb,0.94748e+03_rb,0.94928e+03_rb,0.95051e+03_rb /)
      kbo(:,50, 9) = (/ &
        & 0.94103e+03_rb,0.94413e+03_rb,0.94669e+03_rb,0.94868e+03_rb,0.95011e+03_rb /)
      kbo(:,51, 9) = (/ &
        & 0.93978e+03_rb,0.94307e+03_rb,0.94580e+03_rb,0.94802e+03_rb,0.94965e+03_rb /)
      kbo(:,52, 9) = (/ &
        & 0.93828e+03_rb,0.94192e+03_rb,0.94488e+03_rb,0.94728e+03_rb,0.94912e+03_rb /)
      kbo(:,53, 9) = (/ &
        & 0.93678e+03_rb,0.94072e+03_rb,0.94388e+03_rb,0.94646e+03_rb,0.94851e+03_rb /)
      kbo(:,54, 9) = (/ &
        & 0.93537e+03_rb,0.93955e+03_rb,0.94287e+03_rb,0.94565e+03_rb,0.94789e+03_rb /)
      kbo(:,55, 9) = (/ &
        & 0.93380e+03_rb,0.93820e+03_rb,0.94186e+03_rb,0.94483e+03_rb,0.94722e+03_rb /)
      kbo(:,56, 9) = (/ &
        & 0.93205e+03_rb,0.93688e+03_rb,0.94080e+03_rb,0.94394e+03_rb,0.94652e+03_rb /)
      kbo(:,57, 9) = (/ &
        & 0.92997e+03_rb,0.93554e+03_rb,0.93969e+03_rb,0.94299e+03_rb,0.94574e+03_rb /)
      kbo(:,58, 9) = (/ &
        & 0.92808e+03_rb,0.93414e+03_rb,0.93844e+03_rb,0.94204e+03_rb,0.94498e+03_rb /)
      kbo(:,59, 9) = (/ &
        & 0.92793e+03_rb,0.93341e+03_rb,0.93792e+03_rb,0.94165e+03_rb,0.94465e+03_rb /)
      kbo(:,13,10) = (/ &
        & 0.98036e+03_rb,0.98526e+03_rb,0.98973e+03_rb,0.99279e+03_rb,0.99468e+03_rb /)
      kbo(:,14,10) = (/ &
        & 0.98067e+03_rb,0.98568e+03_rb,0.99005e+03_rb,0.99301e+03_rb,0.99477e+03_rb /)
      kbo(:,15,10) = (/ &
        & 0.98125e+03_rb,0.98614e+03_rb,0.99040e+03_rb,0.99324e+03_rb,0.99485e+03_rb /)
      kbo(:,16,10) = (/ &
        & 0.98185e+03_rb,0.98662e+03_rb,0.99076e+03_rb,0.99348e+03_rb,0.99493e+03_rb /)
      kbo(:,17,10) = (/ &
        & 0.98242e+03_rb,0.98721e+03_rb,0.99109e+03_rb,0.99369e+03_rb,0.99498e+03_rb /)
      kbo(:,18,10) = (/ &
        & 0.98296e+03_rb,0.98755e+03_rb,0.99141e+03_rb,0.99390e+03_rb,0.99502e+03_rb /)
      kbo(:,19,10) = (/ &
        & 0.98350e+03_rb,0.98800e+03_rb,0.99172e+03_rb,0.99408e+03_rb,0.99505e+03_rb /)
      kbo(:,20,10) = (/ &
        & 0.98411e+03_rb,0.98852e+03_rb,0.99205e+03_rb,0.99428e+03_rb,0.99507e+03_rb /)
      kbo(:,21,10) = (/ &
        & 0.98464e+03_rb,0.98894e+03_rb,0.99238e+03_rb,0.99447e+03_rb,0.99496e+03_rb /)
      kbo(:,22,10) = (/ &
        & 0.98547e+03_rb,0.98983e+03_rb,0.99286e+03_rb,0.99470e+03_rb,0.99509e+03_rb /)
      kbo(:,23,10) = (/ &
        & 0.98617e+03_rb,0.99062e+03_rb,0.99331e+03_rb,0.99487e+03_rb,0.99508e+03_rb /)
      kbo(:,24,10) = (/ &
        & 0.98718e+03_rb,0.99115e+03_rb,0.99373e+03_rb,0.99499e+03_rb,0.99503e+03_rb /)
      kbo(:,25,10) = (/ &
        & 0.98813e+03_rb,0.99180e+03_rb,0.99413e+03_rb,0.99494e+03_rb,0.99492e+03_rb /)
      kbo(:,26,10) = (/ &
        & 0.98918e+03_rb,0.99244e+03_rb,0.99450e+03_rb,0.99508e+03_rb,0.99475e+03_rb /)
      kbo(:,27,10) = (/ &
        & 0.99010e+03_rb,0.99304e+03_rb,0.99478e+03_rb,0.99509e+03_rb,0.99499e+03_rb /)
      kbo(:,28,10) = (/ &
        & 0.99090e+03_rb,0.99357e+03_rb,0.99495e+03_rb,0.99505e+03_rb,0.99433e+03_rb /)
      kbo(:,29,10) = (/ &
        & 0.99167e+03_rb,0.99393e+03_rb,0.99505e+03_rb,0.99495e+03_rb,0.99385e+03_rb /)
      kbo(:,30,10) = (/ &
        & 0.99249e+03_rb,0.99447e+03_rb,0.99508e+03_rb,0.99477e+03_rb,0.99348e+03_rb /)
      kbo(:,31,10) = (/ &
        & 0.99290e+03_rb,0.99477e+03_rb,0.99509e+03_rb,0.99452e+03_rb,0.99326e+03_rb /)
      kbo(:,32,10) = (/ &
        & 0.99360e+03_rb,0.99496e+03_rb,0.99505e+03_rb,0.99419e+03_rb,0.99287e+03_rb /)
      kbo(:,33,10) = (/ &
        & 0.99410e+03_rb,0.99506e+03_rb,0.99493e+03_rb,0.99381e+03_rb,0.99210e+03_rb /)
      kbo(:,34,10) = (/ &
        & 0.99447e+03_rb,0.99508e+03_rb,0.99465e+03_rb,0.99347e+03_rb,0.99121e+03_rb /)
      kbo(:,35,10) = (/ &
        & 0.99453e+03_rb,0.99509e+03_rb,0.99465e+03_rb,0.99330e+03_rb,0.99083e+03_rb /)
      kbo(:,36,10) = (/ &
        & 0.99467e+03_rb,0.99509e+03_rb,0.99463e+03_rb,0.99327e+03_rb,0.98992e+03_rb /)
      kbo(:,37,10) = (/ &
        & 0.99451e+03_rb,0.99508e+03_rb,0.99475e+03_rb,0.99344e+03_rb,0.99124e+03_rb /)
      kbo(:,38,10) = (/ &
        & 0.99429e+03_rb,0.99507e+03_rb,0.99486e+03_rb,0.99365e+03_rb,0.99174e+03_rb /)
      kbo(:,39,10) = (/ &
        & 0.99405e+03_rb,0.99505e+03_rb,0.99495e+03_rb,0.99373e+03_rb,0.99219e+03_rb /)
      kbo(:,40,10) = (/ &
        & 0.99347e+03_rb,0.99496e+03_rb,0.99505e+03_rb,0.99420e+03_rb,0.99289e+03_rb /)
      kbo(:,41,10) = (/ &
        & 0.99302e+03_rb,0.99477e+03_rb,0.99509e+03_rb,0.99452e+03_rb,0.99315e+03_rb /)
      kbo(:,42,10) = (/ &
        & 0.99237e+03_rb,0.99435e+03_rb,0.99508e+03_rb,0.99477e+03_rb,0.99348e+03_rb /)
      kbo(:,43,10) = (/ &
        & 0.99148e+03_rb,0.99394e+03_rb,0.99491e+03_rb,0.99498e+03_rb,0.99395e+03_rb /)
      kbo(:,44,10) = (/ &
        & 0.99040e+03_rb,0.99324e+03_rb,0.99485e+03_rb,0.99508e+03_rb,0.99453e+03_rb /)
      kbo(:,45,10) = (/ &
        & 0.98910e+03_rb,0.99241e+03_rb,0.99449e+03_rb,0.99508e+03_rb,0.99476e+03_rb /)
      kbo(:,46,10) = (/ &
        & 0.98735e+03_rb,0.99138e+03_rb,0.99388e+03_rb,0.99490e+03_rb,0.99500e+03_rb /)
      kbo(:,47,10) = (/ &
        & 0.98565e+03_rb,0.99006e+03_rb,0.99301e+03_rb,0.99477e+03_rb,0.99509e+03_rb /)
      kbo(:,48,10) = (/ &
        & 0.98383e+03_rb,0.98834e+03_rb,0.99196e+03_rb,0.99423e+03_rb,0.99507e+03_rb /)
      kbo(:,49,10) = (/ &
        & 0.98174e+03_rb,0.98651e+03_rb,0.99073e+03_rb,0.99346e+03_rb,0.99492e+03_rb /)
      kbo(:,50,10) = (/ &
        & 0.97955e+03_rb,0.98480e+03_rb,0.98924e+03_rb,0.99257e+03_rb,0.99470e+03_rb /)
      kbo(:,51,10) = (/ &
        & 0.97755e+03_rb,0.98298e+03_rb,0.98767e+03_rb,0.99154e+03_rb,0.99399e+03_rb /)
      kbo(:,52,10) = (/ &
        & 0.97554e+03_rb,0.98111e+03_rb,0.98597e+03_rb,0.99036e+03_rb,0.99323e+03_rb /)
      kbo(:,53,10) = (/ &
        & 0.97320e+03_rb,0.97892e+03_rb,0.98413e+03_rb,0.98888e+03_rb,0.99232e+03_rb /)
      kbo(:,54,10) = (/ &
        & 0.97086e+03_rb,0.97720e+03_rb,0.98263e+03_rb,0.98733e+03_rb,0.99135e+03_rb /)
      kbo(:,55,10) = (/ &
        & 0.96881e+03_rb,0.97539e+03_rb,0.98096e+03_rb,0.98583e+03_rb,0.99029e+03_rb /)
      kbo(:,56,10) = (/ &
        & 0.96735e+03_rb,0.97336e+03_rb,0.97906e+03_rb,0.98435e+03_rb,0.98900e+03_rb /)
      kbo(:,57,10) = (/ &
        & 0.96407e+03_rb,0.97115e+03_rb,0.97740e+03_rb,0.98282e+03_rb,0.98752e+03_rb /)
      kbo(:,58,10) = (/ &
        & 0.96154e+03_rb,0.96893e+03_rb,0.97574e+03_rb,0.98129e+03_rb,0.98611e+03_rb /)
      kbo(:,59,10) = (/ &
        & 0.95571e+03_rb,0.96861e+03_rb,0.97503e+03_rb,0.98063e+03_rb,0.98557e+03_rb /)
      kbo(:,13,11) = (/ &
        & 0.98771e+03_rb,0.99296e+03_rb,0.99655e+03_rb,0.99859e+03_rb,0.99936e+03_rb /)
      kbo(:,14,11) = (/ &
        & 0.98850e+03_rb,0.99335e+03_rb,0.99679e+03_rb,0.99871e+03_rb,0.99939e+03_rb /)
      kbo(:,15,11) = (/ &
        & 0.98905e+03_rb,0.99377e+03_rb,0.99705e+03_rb,0.99882e+03_rb,0.99942e+03_rb /)
      kbo(:,16,11) = (/ &
        & 0.98962e+03_rb,0.99419e+03_rb,0.99730e+03_rb,0.99893e+03_rb,0.99945e+03_rb /)
      kbo(:,17,11) = (/ &
        & 0.99016e+03_rb,0.99444e+03_rb,0.99754e+03_rb,0.99903e+03_rb,0.99947e+03_rb /)
      kbo(:,18,11) = (/ &
        & 0.99067e+03_rb,0.99497e+03_rb,0.99775e+03_rb,0.99911e+03_rb,0.99948e+03_rb /)
      kbo(:,19,11) = (/ &
        & 0.99117e+03_rb,0.99532e+03_rb,0.99795e+03_rb,0.99918e+03_rb,0.99949e+03_rb /)
      kbo(:,20,11) = (/ &
        & 0.99173e+03_rb,0.99572e+03_rb,0.99816e+03_rb,0.99924e+03_rb,0.99949e+03_rb /)
      kbo(:,21,11) = (/ &
        & 0.99228e+03_rb,0.99610e+03_rb,0.99835e+03_rb,0.99929e+03_rb,0.99946e+03_rb /)
      kbo(:,22,11) = (/ &
        & 0.99309e+03_rb,0.99663e+03_rb,0.99862e+03_rb,0.99936e+03_rb,0.99953e+03_rb /)
      kbo(:,23,11) = (/ &
        & 0.99388e+03_rb,0.99698e+03_rb,0.99900e+03_rb,0.99942e+03_rb,0.99925e+03_rb /)
      kbo(:,24,11) = (/ &
        & 0.99466e+03_rb,0.99758e+03_rb,0.99904e+03_rb,0.99947e+03_rb,0.99905e+03_rb /)
      kbo(:,25,11) = (/ &
        & 0.99542e+03_rb,0.99800e+03_rb,0.99919e+03_rb,0.99963e+03_rb,0.99878e+03_rb /)
      kbo(:,26,11) = (/ &
        & 0.99618e+03_rb,0.99839e+03_rb,0.99930e+03_rb,0.99945e+03_rb,0.99847e+03_rb /)
      kbo(:,27,11) = (/ &
        & 0.99683e+03_rb,0.99872e+03_rb,0.99939e+03_rb,0.99934e+03_rb,0.99751e+03_rb /)
      kbo(:,28,11) = (/ &
        & 0.99741e+03_rb,0.99897e+03_rb,0.99946e+03_rb,0.99913e+03_rb,0.99749e+03_rb /)
      kbo(:,29,11) = (/ &
        & 0.99792e+03_rb,0.99917e+03_rb,0.99949e+03_rb,0.99884e+03_rb,0.99707e+03_rb /)
      kbo(:,30,11) = (/ &
        & 0.99821e+03_rb,0.99929e+03_rb,0.99946e+03_rb,0.99850e+03_rb,0.99637e+03_rb /)
      kbo(:,31,11) = (/ &
        & 0.99871e+03_rb,0.99938e+03_rb,0.99949e+03_rb,0.99810e+03_rb,0.99532e+03_rb /)
      kbo(:,32,11) = (/ &
        & 0.99899e+03_rb,0.99961e+03_rb,0.99926e+03_rb,0.99761e+03_rb,0.99460e+03_rb /)
      kbo(:,33,11) = (/ &
        & 0.99918e+03_rb,0.99949e+03_rb,0.99880e+03_rb,0.99700e+03_rb,0.99413e+03_rb /)
      kbo(:,34,11) = (/ &
        & 0.99929e+03_rb,0.99946e+03_rb,0.99850e+03_rb,0.99635e+03_rb,0.99329e+03_rb /)
      kbo(:,35,11) = (/ &
        & 0.99934e+03_rb,0.99941e+03_rb,0.99830e+03_rb,0.99590e+03_rb,0.99292e+03_rb /)
      kbo(:,36,11) = (/ &
        & 0.99935e+03_rb,0.99940e+03_rb,0.99826e+03_rb,0.99583e+03_rb,0.99287e+03_rb /)
      kbo(:,37,11) = (/ &
        & 0.99930e+03_rb,0.99946e+03_rb,0.99846e+03_rb,0.99628e+03_rb,0.99327e+03_rb /)
      kbo(:,38,11) = (/ &
        & 0.99924e+03_rb,0.99949e+03_rb,0.99866e+03_rb,0.99670e+03_rb,0.99350e+03_rb /)
      kbo(:,39,11) = (/ &
        & 0.99917e+03_rb,0.99949e+03_rb,0.99899e+03_rb,0.99707e+03_rb,0.99451e+03_rb /)
      kbo(:,40,11) = (/ &
        & 0.99898e+03_rb,0.99946e+03_rb,0.99913e+03_rb,0.99777e+03_rb,0.99505e+03_rb /)
      kbo(:,41,11) = (/ &
        & 0.99871e+03_rb,0.99939e+03_rb,0.99935e+03_rb,0.99810e+03_rb,0.99546e+03_rb /)
      kbo(:,42,11) = (/ &
        & 0.99835e+03_rb,0.99929e+03_rb,0.99947e+03_rb,0.99851e+03_rb,0.99637e+03_rb /)
      kbo(:,43,11) = (/ &
        & 0.99780e+03_rb,0.99913e+03_rb,0.99949e+03_rb,0.99893e+03_rb,0.99722e+03_rb /)
      kbo(:,44,11) = (/ &
        & 0.99704e+03_rb,0.99882e+03_rb,0.99942e+03_rb,0.99928e+03_rb,0.99778e+03_rb /)
      kbo(:,45,11) = (/ &
        & 0.99613e+03_rb,0.99837e+03_rb,0.99930e+03_rb,0.99947e+03_rb,0.99849e+03_rb /)
      kbo(:,46,11) = (/ &
        & 0.99492e+03_rb,0.99787e+03_rb,0.99911e+03_rb,0.99949e+03_rb,0.99897e+03_rb /)
      kbo(:,47,11) = (/ &
        & 0.99335e+03_rb,0.99679e+03_rb,0.99871e+03_rb,0.99939e+03_rb,0.99936e+03_rb /)
      kbo(:,48,11) = (/ &
        & 0.99156e+03_rb,0.99560e+03_rb,0.99810e+03_rb,0.99923e+03_rb,0.99951e+03_rb /)
      kbo(:,49,11) = (/ &
        & 0.98955e+03_rb,0.99414e+03_rb,0.99727e+03_rb,0.99894e+03_rb,0.99946e+03_rb /)
      kbo(:,50,11) = (/ &
        & 0.98749e+03_rb,0.99256e+03_rb,0.99630e+03_rb,0.99847e+03_rb,0.99918e+03_rb /)
      kbo(:,51,11) = (/ &
        & 0.98496e+03_rb,0.99085e+03_rb,0.99509e+03_rb,0.99784e+03_rb,0.99916e+03_rb /)
      kbo(:,52,11) = (/ &
        & 0.98304e+03_rb,0.98897e+03_rb,0.99370e+03_rb,0.99701e+03_rb,0.99883e+03_rb /)
      kbo(:,53,11) = (/ &
        & 0.98104e+03_rb,0.98693e+03_rb,0.99213e+03_rb,0.99600e+03_rb,0.99833e+03_rb /)
      kbo(:,54,11) = (/ &
        & 0.97906e+03_rb,0.98440e+03_rb,0.99053e+03_rb,0.99485e+03_rb,0.99771e+03_rb /)
      kbo(:,55,11) = (/ &
        & 0.97789e+03_rb,0.98286e+03_rb,0.98886e+03_rb,0.99361e+03_rb,0.99696e+03_rb /)
      kbo(:,56,11) = (/ &
        & 0.97514e+03_rb,0.98117e+03_rb,0.98708e+03_rb,0.99224e+03_rb,0.99622e+03_rb /)
      kbo(:,57,11) = (/ &
        & 0.97242e+03_rb,0.97945e+03_rb,0.98472e+03_rb,0.99072e+03_rb,0.99500e+03_rb /)
      kbo(:,58,11) = (/ &
        & 0.96969e+03_rb,0.97814e+03_rb,0.98316e+03_rb,0.98916e+03_rb,0.99384e+03_rb /)
      kbo(:,59,11) = (/ &
        & 0.96859e+03_rb,0.97725e+03_rb,0.98261e+03_rb,0.98851e+03_rb,0.99335e+03_rb /)
      kbo(:,13,12) = (/ &
        & 0.99768e+03_rb,0.99984e+03_rb,0.10010e+04_rb,0.10024e+04_rb,0.10029e+04_rb /)
      kbo(:,14,12) = (/ &
        & 0.99795e+03_rb,0.99992e+03_rb,0.10011e+04_rb,0.10025e+04_rb,0.10029e+04_rb /)
      kbo(:,15,12) = (/ &
        & 0.99880e+03_rb,0.99996e+03_rb,0.10013e+04_rb,0.10026e+04_rb,0.10031e+04_rb /)
      kbo(:,16,12) = (/ &
        & 0.99852e+03_rb,0.10003e+04_rb,0.10015e+04_rb,0.10028e+04_rb,0.10029e+04_rb /)
      kbo(:,17,12) = (/ &
        & 0.99876e+03_rb,0.10000e+04_rb,0.10016e+04_rb,0.10027e+04_rb,0.10029e+04_rb /)
      kbo(:,18,12) = (/ &
        & 0.99898e+03_rb,0.10001e+04_rb,0.10018e+04_rb,0.10028e+04_rb,0.10028e+04_rb /)
      kbo(:,19,12) = (/ &
        & 0.99918e+03_rb,0.10003e+04_rb,0.10021e+04_rb,0.10028e+04_rb,0.10028e+04_rb /)
      kbo(:,20,12) = (/ &
        & 0.99940e+03_rb,0.10005e+04_rb,0.10026e+04_rb,0.10029e+04_rb,0.10031e+04_rb /)
      kbo(:,21,12) = (/ &
        & 0.99959e+03_rb,0.10007e+04_rb,0.10022e+04_rb,0.10029e+04_rb,0.10026e+04_rb /)
      kbo(:,22,12) = (/ &
        & 0.99984e+03_rb,0.10010e+04_rb,0.10024e+04_rb,0.10030e+04_rb,0.10023e+04_rb /)
      kbo(:,23,12) = (/ &
        & 0.99996e+03_rb,0.10013e+04_rb,0.10024e+04_rb,0.10029e+04_rb,0.10025e+04_rb /)
      kbo(:,24,12) = (/ &
        & 0.10000e+04_rb,0.10016e+04_rb,0.10029e+04_rb,0.10029e+04_rb,0.10023e+04_rb /)
      kbo(:,25,12) = (/ &
        & 0.10003e+04_rb,0.10023e+04_rb,0.10029e+04_rb,0.10026e+04_rb,0.10019e+04_rb /)
      kbo(:,26,12) = (/ &
        & 0.10007e+04_rb,0.10022e+04_rb,0.10029e+04_rb,0.10026e+04_rb,0.10015e+04_rb /)
      kbo(:,27,12) = (/ &
        & 0.10012e+04_rb,0.10025e+04_rb,0.10031e+04_rb,0.10024e+04_rb,0.10010e+04_rb /)
      kbo(:,28,12) = (/ &
        & 0.10015e+04_rb,0.10027e+04_rb,0.10029e+04_rb,0.10022e+04_rb,0.10005e+04_rb /)
      kbo(:,29,12) = (/ &
        & 0.10019e+04_rb,0.10028e+04_rb,0.10028e+04_rb,0.10019e+04_rb,0.99992e+03_rb /)
      kbo(:,30,12) = (/ &
        & 0.10022e+04_rb,0.10029e+04_rb,0.10026e+04_rb,0.10017e+04_rb,0.99965e+03_rb /)
      kbo(:,31,12) = (/ &
        & 0.10025e+04_rb,0.10030e+04_rb,0.10022e+04_rb,0.10011e+04_rb,0.99843e+03_rb /)
      kbo(:,32,12) = (/ &
        & 0.10027e+04_rb,0.10029e+04_rb,0.10020e+04_rb,0.10007e+04_rb,0.99748e+03_rb /)
      kbo(:,33,12) = (/ &
        & 0.10029e+04_rb,0.10028e+04_rb,0.10021e+04_rb,0.10000e+04_rb,0.99658e+03_rb /)
      kbo(:,34,12) = (/ &
        & 0.10031e+04_rb,0.10026e+04_rb,0.10017e+04_rb,0.99945e+03_rb,0.99563e+03_rb /)
      kbo(:,35,12) = (/ &
        & 0.10030e+04_rb,0.10025e+04_rb,0.10031e+04_rb,0.99888e+03_rb,0.99496e+03_rb /)
      kbo(:,36,12) = (/ &
        & 0.10030e+04_rb,0.10025e+04_rb,0.10020e+04_rb,0.99880e+03_rb,0.99482e+03_rb /)
      kbo(:,37,12) = (/ &
        & 0.10029e+04_rb,0.10026e+04_rb,0.10015e+04_rb,0.99939e+03_rb,0.99547e+03_rb /)
      kbo(:,38,12) = (/ &
        & 0.10029e+04_rb,0.10027e+04_rb,0.10017e+04_rb,0.99958e+03_rb,0.99634e+03_rb /)
      kbo(:,39,12) = (/ &
        & 0.10028e+04_rb,0.10028e+04_rb,0.10017e+04_rb,0.99993e+03_rb,0.99613e+03_rb /)
      kbo(:,40,12) = (/ &
        & 0.10027e+04_rb,0.10029e+04_rb,0.10022e+04_rb,0.10003e+04_rb,0.99696e+03_rb /)
      kbo(:,41,12) = (/ &
        & 0.10027e+04_rb,0.10029e+04_rb,0.10024e+04_rb,0.10013e+04_rb,0.99857e+03_rb /)
      kbo(:,42,12) = (/ &
        & 0.10022e+04_rb,0.10029e+04_rb,0.10028e+04_rb,0.10021e+04_rb,0.99929e+03_rb /)
      kbo(:,43,12) = (/ &
        & 0.10018e+04_rb,0.10028e+04_rb,0.10028e+04_rb,0.10020e+04_rb,0.10001e+04_rb /)
      kbo(:,44,12) = (/ &
        & 0.10013e+04_rb,0.10026e+04_rb,0.10029e+04_rb,0.10023e+04_rb,0.10016e+04_rb /)
      kbo(:,45,12) = (/ &
        & 0.10007e+04_rb,0.10022e+04_rb,0.10029e+04_rb,0.10026e+04_rb,0.10015e+04_rb /)
      kbo(:,46,12) = (/ &
        & 0.10014e+04_rb,0.10016e+04_rb,0.10028e+04_rb,0.10028e+04_rb,0.10020e+04_rb /)
      kbo(:,47,12) = (/ &
        & 0.99993e+03_rb,0.10012e+04_rb,0.10025e+04_rb,0.10029e+04_rb,0.10024e+04_rb /)
      kbo(:,48,12) = (/ &
        & 0.99937e+03_rb,0.10005e+04_rb,0.10022e+04_rb,0.10029e+04_rb,0.10027e+04_rb /)
      kbo(:,49,12) = (/ &
        & 0.99853e+03_rb,0.10002e+04_rb,0.10015e+04_rb,0.10026e+04_rb,0.10029e+04_rb /)
      kbo(:,50,12) = (/ &
        & 0.99741e+03_rb,0.99993e+03_rb,0.10008e+04_rb,0.10023e+04_rb,0.10029e+04_rb /)
      kbo(:,51,12) = (/ &
        & 0.99665e+03_rb,0.99912e+03_rb,0.10002e+04_rb,0.10018e+04_rb,0.10028e+04_rb /)
      kbo(:,52,12) = (/ &
        & 0.99385e+03_rb,0.99826e+03_rb,0.10000e+04_rb,0.10013e+04_rb,0.10025e+04_rb /)
      kbo(:,53,12) = (/ &
        & 0.99099e+03_rb,0.99714e+03_rb,0.99965e+03_rb,0.10007e+04_rb,0.10022e+04_rb /)
      kbo(:,54,12) = (/ &
        & 0.98826e+03_rb,0.99612e+03_rb,0.99903e+03_rb,0.10002e+04_rb,0.10018e+04_rb /)
      kbo(:,55,12) = (/ &
        & 0.98397e+03_rb,0.99379e+03_rb,0.99822e+03_rb,0.10000e+04_rb,0.10013e+04_rb /)
      kbo(:,56,12) = (/ &
        & 0.98006e+03_rb,0.99120e+03_rb,0.99722e+03_rb,0.99969e+03_rb,0.10006e+04_rb /)
      kbo(:,57,12) = (/ &
        & 0.97631e+03_rb,0.98840e+03_rb,0.99624e+03_rb,0.99911e+03_rb,0.10002e+04_rb /)
      kbo(:,58,12) = (/ &
        & 0.97279e+03_rb,0.98508e+03_rb,0.99422e+03_rb,0.99838e+03_rb,0.10001e+04_rb /)
      kbo(:,59,12) = (/ &
        & 0.97171e+03_rb,0.98327e+03_rb,0.99315e+03_rb,0.99818e+03_rb,0.99997e+03_rb /)
      kbo(:,13,13) = (/ &
        & 0.99623e+03_rb,0.99978e+03_rb,0.10034e+04_rb,0.10050e+04_rb,0.10054e+04_rb /)
      kbo(:,14,13) = (/ &
        & 0.99674e+03_rb,0.10002e+04_rb,0.10034e+04_rb,0.10050e+04_rb,0.10053e+04_rb /)
      kbo(:,15,13) = (/ &
        & 0.99652e+03_rb,0.10007e+04_rb,0.10036e+04_rb,0.10051e+04_rb,0.10051e+04_rb /)
      kbo(:,16,13) = (/ &
        & 0.99785e+03_rb,0.10007e+04_rb,0.10039e+04_rb,0.10049e+04_rb,0.10053e+04_rb /)
      kbo(:,17,13) = (/ &
        & 0.99816e+03_rb,0.10017e+04_rb,0.10043e+04_rb,0.10052e+04_rb,0.10052e+04_rb /)
      kbo(:,18,13) = (/ &
        & 0.99792e+03_rb,0.10021e+04_rb,0.10042e+04_rb,0.10053e+04_rb,0.10051e+04_rb /)
      kbo(:,19,13) = (/ &
        & 0.99826e+03_rb,0.10023e+04_rb,0.10042e+04_rb,0.10053e+04_rb,0.10051e+04_rb /)
      kbo(:,20,13) = (/ &
        & 0.99869e+03_rb,0.10028e+04_rb,0.10038e+04_rb,0.10053e+04_rb,0.10044e+04_rb /)
      kbo(:,21,13) = (/ &
        & 0.99914e+03_rb,0.10028e+04_rb,0.10048e+04_rb,0.10054e+04_rb,0.10048e+04_rb /)
      kbo(:,22,13) = (/ &
        & 0.99989e+03_rb,0.10035e+04_rb,0.10050e+04_rb,0.10053e+04_rb,0.10046e+04_rb /)
      kbo(:,23,13) = (/ &
        & 0.10011e+04_rb,0.10037e+04_rb,0.10051e+04_rb,0.10053e+04_rb,0.10041e+04_rb /)
      kbo(:,24,13) = (/ &
        & 0.10018e+04_rb,0.10041e+04_rb,0.10050e+04_rb,0.10052e+04_rb,0.10037e+04_rb /)
      kbo(:,25,13) = (/ &
        & 0.10026e+04_rb,0.10040e+04_rb,0.10056e+04_rb,0.10050e+04_rb,0.10036e+04_rb /)
      kbo(:,26,13) = (/ &
        & 0.10031e+04_rb,0.10048e+04_rb,0.10054e+04_rb,0.10050e+04_rb,0.10030e+04_rb /)
      kbo(:,27,13) = (/ &
        & 0.10047e+04_rb,0.10050e+04_rb,0.10051e+04_rb,0.10045e+04_rb,0.10025e+04_rb /)
      kbo(:,28,13) = (/ &
        & 0.10039e+04_rb,0.10052e+04_rb,0.10052e+04_rb,0.10041e+04_rb,0.10018e+04_rb /)
      kbo(:,29,13) = (/ &
        & 0.10044e+04_rb,0.10053e+04_rb,0.10051e+04_rb,0.10037e+04_rb,0.10011e+04_rb /)
      kbo(:,30,13) = (/ &
        & 0.10048e+04_rb,0.10054e+04_rb,0.10048e+04_rb,0.10029e+04_rb,0.99977e+03_rb /)
      kbo(:,31,13) = (/ &
        & 0.10050e+04_rb,0.10053e+04_rb,0.10045e+04_rb,0.10025e+04_rb,0.99968e+03_rb /)
      kbo(:,32,13) = (/ &
        & 0.10052e+04_rb,0.10050e+04_rb,0.10041e+04_rb,0.10015e+04_rb,0.99880e+03_rb /)
      kbo(:,33,13) = (/ &
        & 0.10053e+04_rb,0.10050e+04_rb,0.10033e+04_rb,0.10008e+04_rb,0.99675e+03_rb /)
      kbo(:,34,13) = (/ &
        & 0.10051e+04_rb,0.10048e+04_rb,0.10028e+04_rb,0.10000e+04_rb,0.99529e+03_rb /)
      kbo(:,35,13) = (/ &
        & 0.10054e+04_rb,0.10047e+04_rb,0.10003e+04_rb,0.99992e+03_rb,0.99481e+03_rb /)
      kbo(:,36,13) = (/ &
        & 0.10054e+04_rb,0.10046e+04_rb,0.10017e+04_rb,0.99987e+03_rb,0.99445e+03_rb /)
      kbo(:,37,13) = (/ &
        & 0.10054e+04_rb,0.10048e+04_rb,0.10030e+04_rb,0.99995e+03_rb,0.99517e+03_rb /)
      kbo(:,38,13) = (/ &
        & 0.10054e+04_rb,0.10049e+04_rb,0.10034e+04_rb,0.10006e+04_rb,0.99603e+03_rb /)
      kbo(:,39,13) = (/ &
        & 0.10053e+04_rb,0.10051e+04_rb,0.10037e+04_rb,0.10011e+04_rb,0.99693e+03_rb /)
      kbo(:,40,13) = (/ &
        & 0.10052e+04_rb,0.10052e+04_rb,0.10041e+04_rb,0.10018e+04_rb,0.99860e+03_rb /)
      kbo(:,41,13) = (/ &
        & 0.10048e+04_rb,0.10053e+04_rb,0.10045e+04_rb,0.10023e+04_rb,0.99951e+03_rb /)
      kbo(:,42,13) = (/ &
        & 0.10048e+04_rb,0.10054e+04_rb,0.10046e+04_rb,0.10024e+04_rb,0.10003e+04_rb /)
      kbo(:,43,13) = (/ &
        & 0.10043e+04_rb,0.10053e+04_rb,0.10051e+04_rb,0.10038e+04_rb,0.10013e+04_rb /)
      kbo(:,44,13) = (/ &
        & 0.10036e+04_rb,0.10051e+04_rb,0.10053e+04_rb,0.10044e+04_rb,0.10012e+04_rb /)
      kbo(:,45,13) = (/ &
        & 0.10028e+04_rb,0.10048e+04_rb,0.10054e+04_rb,0.10048e+04_rb,0.10031e+04_rb /)
      kbo(:,46,13) = (/ &
        & 0.10008e+04_rb,0.10042e+04_rb,0.10053e+04_rb,0.10052e+04_rb,0.10038e+04_rb /)
      kbo(:,47,13) = (/ &
        & 0.10002e+04_rb,0.10039e+04_rb,0.10050e+04_rb,0.10053e+04_rb,0.10045e+04_rb /)
      kbo(:,48,13) = (/ &
        & 0.99859e+03_rb,0.10025e+04_rb,0.10043e+04_rb,0.10054e+04_rb,0.10050e+04_rb /)
      kbo(:,49,13) = (/ &
        & 0.99776e+03_rb,0.10009e+04_rb,0.10041e+04_rb,0.10052e+04_rb,0.10053e+04_rb /)
      kbo(:,50,13) = (/ &
        & 0.99565e+03_rb,0.99944e+03_rb,0.10034e+04_rb,0.10049e+04_rb,0.10054e+04_rb /)
      kbo(:,51,13) = (/ &
        & 0.99271e+03_rb,0.99811e+03_rb,0.10022e+04_rb,0.10043e+04_rb,0.10053e+04_rb /)
      kbo(:,52,13) = (/ &
        & 0.99056e+03_rb,0.99717e+03_rb,0.10006e+04_rb,0.10036e+04_rb,0.10051e+04_rb /)
      kbo(:,53,13) = (/ &
        & 0.98830e+03_rb,0.99502e+03_rb,0.99909e+03_rb,0.10028e+04_rb,0.10048e+04_rb /)
      kbo(:,54,13) = (/ &
        & 0.98610e+03_rb,0.99275e+03_rb,0.99821e+03_rb,0.10021e+04_rb,0.10042e+04_rb /)
      kbo(:,55,13) = (/ &
        & 0.98386e+03_rb,0.99045e+03_rb,0.99729e+03_rb,0.10008e+04_rb,0.10038e+04_rb /)
      kbo(:,56,13) = (/ &
        & 0.98150e+03_rb,0.98846e+03_rb,0.99517e+03_rb,0.99919e+03_rb,0.10028e+04_rb /)
      kbo(:,57,13) = (/ &
        & 0.97902e+03_rb,0.98636e+03_rb,0.99303e+03_rb,0.99832e+03_rb,0.10022e+04_rb /)
      kbo(:,58,13) = (/ &
        & 0.97656e+03_rb,0.98426e+03_rb,0.99079e+03_rb,0.99735e+03_rb,0.10008e+04_rb /)
      kbo(:,59,13) = (/ &
        & 0.97503e+03_rb,0.98337e+03_rb,0.99004e+03_rb,0.99647e+03_rb,0.10002e+04_rb /)
      kbo(:,13,14) = (/ &
        & 0.99596e+03_rb,0.10023e+04_rb,0.10047e+04_rb,0.10067e+04_rb,0.10072e+04_rb /)
      kbo(:,14,14) = (/ &
        & 0.99625e+03_rb,0.10024e+04_rb,0.10052e+04_rb,0.10068e+04_rb,0.10071e+04_rb /)
      kbo(:,15,14) = (/ &
        & 0.99662e+03_rb,0.10025e+04_rb,0.10054e+04_rb,0.10069e+04_rb,0.10071e+04_rb /)
      kbo(:,16,14) = (/ &
        & 0.99703e+03_rb,0.10026e+04_rb,0.10056e+04_rb,0.10070e+04_rb,0.10070e+04_rb /)
      kbo(:,17,14) = (/ &
        & 0.99784e+03_rb,0.10032e+04_rb,0.10054e+04_rb,0.10071e+04_rb,0.10070e+04_rb /)
      kbo(:,18,14) = (/ &
        & 0.99947e+03_rb,0.10032e+04_rb,0.10060e+04_rb,0.10071e+04_rb,0.10069e+04_rb /)
      kbo(:,19,14) = (/ &
        & 0.10001e+04_rb,0.10036e+04_rb,0.10062e+04_rb,0.10071e+04_rb,0.10068e+04_rb /)
      kbo(:,20,14) = (/ &
        & 0.10009e+04_rb,0.10037e+04_rb,0.10064e+04_rb,0.10076e+04_rb,0.10067e+04_rb /)
      kbo(:,21,14) = (/ &
        & 0.10015e+04_rb,0.10046e+04_rb,0.10065e+04_rb,0.10072e+04_rb,0.10065e+04_rb /)
      kbo(:,22,14) = (/ &
        & 0.10023e+04_rb,0.10047e+04_rb,0.10068e+04_rb,0.10071e+04_rb,0.10063e+04_rb /)
      kbo(:,23,14) = (/ &
        & 0.10021e+04_rb,0.10055e+04_rb,0.10069e+04_rb,0.10071e+04_rb,0.10060e+04_rb /)
      kbo(:,24,14) = (/ &
        & 0.10028e+04_rb,0.10059e+04_rb,0.10071e+04_rb,0.10069e+04_rb,0.10057e+04_rb /)
      kbo(:,25,14) = (/ &
        & 0.10033e+04_rb,0.10062e+04_rb,0.10068e+04_rb,0.10068e+04_rb,0.10052e+04_rb /)
      kbo(:,26,14) = (/ &
        & 0.10043e+04_rb,0.10066e+04_rb,0.10072e+04_rb,0.10061e+04_rb,0.10047e+04_rb /)
      kbo(:,27,14) = (/ &
        & 0.10033e+04_rb,0.10068e+04_rb,0.10071e+04_rb,0.10062e+04_rb,0.10042e+04_rb /)
      kbo(:,28,14) = (/ &
        & 0.10057e+04_rb,0.10070e+04_rb,0.10070e+04_rb,0.10058e+04_rb,0.10036e+04_rb /)
      kbo(:,29,14) = (/ &
        & 0.10062e+04_rb,0.10075e+04_rb,0.10068e+04_rb,0.10053e+04_rb,0.10030e+04_rb /)
      kbo(:,30,14) = (/ &
        & 0.10065e+04_rb,0.10072e+04_rb,0.10069e+04_rb,0.10048e+04_rb,0.10022e+04_rb /)
      kbo(:,31,14) = (/ &
        & 0.10068e+04_rb,0.10071e+04_rb,0.10062e+04_rb,0.10042e+04_rb,0.10014e+04_rb /)
      kbo(:,32,14) = (/ &
        & 0.10070e+04_rb,0.10070e+04_rb,0.10058e+04_rb,0.10035e+04_rb,0.99925e+03_rb /)
      kbo(:,33,14) = (/ &
        & 0.10071e+04_rb,0.10068e+04_rb,0.10053e+04_rb,0.10029e+04_rb,0.99808e+03_rb /)
      kbo(:,34,14) = (/ &
        & 0.10072e+04_rb,0.10065e+04_rb,0.10048e+04_rb,0.10021e+04_rb,0.99788e+03_rb /)
      kbo(:,35,14) = (/ &
        & 0.10072e+04_rb,0.10064e+04_rb,0.10045e+04_rb,0.10016e+04_rb,0.99700e+03_rb /)
      kbo(:,36,14) = (/ &
        & 0.10072e+04_rb,0.10063e+04_rb,0.10044e+04_rb,0.10015e+04_rb,0.99731e+03_rb /)
      kbo(:,37,14) = (/ &
        & 0.10072e+04_rb,0.10065e+04_rb,0.10047e+04_rb,0.10021e+04_rb,0.99778e+03_rb /)
      kbo(:,38,14) = (/ &
        & 0.10072e+04_rb,0.10067e+04_rb,0.10050e+04_rb,0.10026e+04_rb,0.99820e+03_rb /)
      kbo(:,39,14) = (/ &
        & 0.10071e+04_rb,0.10068e+04_rb,0.10053e+04_rb,0.10030e+04_rb,0.99855e+03_rb /)
      kbo(:,40,14) = (/ &
        & 0.10070e+04_rb,0.10070e+04_rb,0.10058e+04_rb,0.10036e+04_rb,0.99965e+03_rb /)
      kbo(:,41,14) = (/ &
        & 0.10068e+04_rb,0.10071e+04_rb,0.10062e+04_rb,0.10042e+04_rb,0.10010e+04_rb /)
      kbo(:,42,14) = (/ &
        & 0.10065e+04_rb,0.10072e+04_rb,0.10065e+04_rb,0.10048e+04_rb,0.10022e+04_rb /)
      kbo(:,43,14) = (/ &
        & 0.10061e+04_rb,0.10071e+04_rb,0.10069e+04_rb,0.10055e+04_rb,0.10031e+04_rb /)
      kbo(:,44,14) = (/ &
        & 0.10054e+04_rb,0.10069e+04_rb,0.10071e+04_rb,0.10061e+04_rb,0.10039e+04_rb /)
      kbo(:,45,14) = (/ &
        & 0.10046e+04_rb,0.10066e+04_rb,0.10072e+04_rb,0.10065e+04_rb,0.10048e+04_rb /)
      kbo(:,46,14) = (/ &
        & 0.10023e+04_rb,0.10060e+04_rb,0.10075e+04_rb,0.10069e+04_rb,0.10055e+04_rb /)
      kbo(:,47,14) = (/ &
        & 0.10024e+04_rb,0.10044e+04_rb,0.10068e+04_rb,0.10071e+04_rb,0.10062e+04_rb /)
      kbo(:,48,14) = (/ &
        & 0.10006e+04_rb,0.10039e+04_rb,0.10063e+04_rb,0.10072e+04_rb,0.10067e+04_rb /)
      kbo(:,49,14) = (/ &
        & 0.99705e+03_rb,0.10026e+04_rb,0.10052e+04_rb,0.10070e+04_rb,0.10070e+04_rb /)
      kbo(:,50,14) = (/ &
        & 0.99574e+03_rb,0.10014e+04_rb,0.10040e+04_rb,0.10066e+04_rb,0.10072e+04_rb /)
      kbo(:,51,14) = (/ &
        & 0.99447e+03_rb,0.99966e+03_rb,0.10033e+04_rb,0.10065e+04_rb,0.10071e+04_rb /)
      kbo(:,52,14) = (/ &
        & 0.99299e+03_rb,0.99670e+03_rb,0.10025e+04_rb,0.10054e+04_rb,0.10069e+04_rb /)
      kbo(:,53,14) = (/ &
        & 0.99138e+03_rb,0.99552e+03_rb,0.10013e+04_rb,0.10044e+04_rb,0.10065e+04_rb /)
      kbo(:,54,14) = (/ &
        & 0.98979e+03_rb,0.99462e+03_rb,0.99874e+03_rb,0.10030e+04_rb,0.10060e+04_rb /)
      kbo(:,55,14) = (/ &
        & 0.98814e+03_rb,0.99291e+03_rb,0.99630e+03_rb,0.10029e+04_rb,0.10050e+04_rb /)
      kbo(:,56,14) = (/ &
        & 0.98639e+03_rb,0.99150e+03_rb,0.99562e+03_rb,0.10014e+04_rb,0.10045e+04_rb /)
      kbo(:,57,14) = (/ &
        & 0.98454e+03_rb,0.98998e+03_rb,0.99438e+03_rb,0.99904e+03_rb,0.10032e+04_rb /)
      kbo(:,58,14) = (/ &
        & 0.98268e+03_rb,0.98844e+03_rb,0.99315e+03_rb,0.99688e+03_rb,0.10025e+04_rb /)
      kbo(:,59,14) = (/ &
        & 0.98190e+03_rb,0.98779e+03_rb,0.99262e+03_rb,0.99638e+03_rb,0.10024e+04_rb /)
      kbo(:,13,15) = (/ &
        & 0.99739e+03_rb,0.10017e+04_rb,0.10064e+04_rb,0.10076e+04_rb,0.10080e+04_rb /)
      kbo(:,14,15) = (/ &
        & 0.99781e+03_rb,0.10021e+04_rb,0.10065e+04_rb,0.10077e+04_rb,0.10090e+04_rb /)
      kbo(:,15,15) = (/ &
        & 0.99827e+03_rb,0.10029e+04_rb,0.10067e+04_rb,0.10077e+04_rb,0.10079e+04_rb /)
      kbo(:,16,15) = (/ &
        & 0.99875e+03_rb,0.10040e+04_rb,0.10068e+04_rb,0.10078e+04_rb,0.10089e+04_rb /)
      kbo(:,17,15) = (/ &
        & 0.99816e+03_rb,0.10041e+04_rb,0.10080e+04_rb,0.10079e+04_rb,0.10078e+04_rb /)
      kbo(:,18,15) = (/ &
        & 0.99861e+03_rb,0.10065e+04_rb,0.10071e+04_rb,0.10090e+04_rb,0.10078e+04_rb /)
      kbo(:,19,15) = (/ &
        & 0.10001e+04_rb,0.10056e+04_rb,0.10072e+04_rb,0.10079e+04_rb,0.10077e+04_rb /)
      kbo(:,20,15) = (/ &
        & 0.99952e+03_rb,0.10059e+04_rb,0.10074e+04_rb,0.10090e+04_rb,0.10087e+04_rb /)
      kbo(:,21,15) = (/ &
        & 0.10000e+04_rb,0.10061e+04_rb,0.10085e+04_rb,0.10080e+04_rb,0.10085e+04_rb /)
      kbo(:,22,15) = (/ &
        & 0.10012e+04_rb,0.10064e+04_rb,0.10076e+04_rb,0.10090e+04_rb,0.10073e+04_rb /)
      kbo(:,23,15) = (/ &
        & 0.10032e+04_rb,0.10067e+04_rb,0.10088e+04_rb,0.10079e+04_rb,0.10070e+04_rb /)
      kbo(:,24,15) = (/ &
        & 0.10052e+04_rb,0.10070e+04_rb,0.10079e+04_rb,0.10089e+04_rb,0.10067e+04_rb /)
      kbo(:,25,15) = (/ &
        & 0.10057e+04_rb,0.10083e+04_rb,0.10079e+04_rb,0.10098e+04_rb,0.10062e+04_rb /)
      kbo(:,26,15) = (/ &
        & 0.10061e+04_rb,0.10085e+04_rb,0.10080e+04_rb,0.10075e+04_rb,0.10068e+04_rb /)
      kbo(:,27,15) = (/ &
        & 0.10066e+04_rb,0.10087e+04_rb,0.10111e+04_rb,0.10072e+04_rb,0.10051e+04_rb /)
      kbo(:,28,15) = (/ &
        & 0.10069e+04_rb,0.10089e+04_rb,0.10079e+04_rb,0.10068e+04_rb,0.10054e+04_rb /)
      kbo(:,29,15) = (/ &
        & 0.10072e+04_rb,0.10069e+04_rb,0.10077e+04_rb,0.10063e+04_rb,0.10034e+04_rb /)
      kbo(:,30,15) = (/ &
        & 0.10075e+04_rb,0.10111e+04_rb,0.10075e+04_rb,0.10068e+04_rb,0.10088e+04_rb /)
      kbo(:,31,15) = (/ &
        & 0.10077e+04_rb,0.10080e+04_rb,0.10072e+04_rb,0.10061e+04_rb,0.10007e+04_rb /)
      kbo(:,32,15) = (/ &
        & 0.10078e+04_rb,0.10089e+04_rb,0.10068e+04_rb,0.10043e+04_rb,0.10013e+04_rb /)
      kbo(:,33,15) = (/ &
        & 0.10079e+04_rb,0.10077e+04_rb,0.10063e+04_rb,0.10044e+04_rb,0.10009e+04_rb /)
      kbo(:,34,15) = (/ &
        & 0.10090e+04_rb,0.10075e+04_rb,0.10058e+04_rb,0.10025e+04_rb,0.10002e+04_rb /)
      kbo(:,35,15) = (/ &
        & 0.10080e+04_rb,0.10073e+04_rb,0.10054e+04_rb,0.10021e+04_rb,0.99972e+03_rb /)
      kbo(:,36,15) = (/ &
        & 0.10090e+04_rb,0.10073e+04_rb,0.10054e+04_rb,0.10020e+04_rb,0.99964e+03_rb /)
      kbo(:,37,15) = (/ &
        & 0.10080e+04_rb,0.10075e+04_rb,0.10057e+04_rb,0.10024e+04_rb,0.10001e+04_rb /)
      kbo(:,38,15) = (/ &
        & 0.10090e+04_rb,0.10076e+04_rb,0.10060e+04_rb,0.10040e+04_rb,0.10006e+04_rb /)
      kbo(:,39,15) = (/ &
        & 0.10079e+04_rb,0.10077e+04_rb,0.10063e+04_rb,0.10034e+04_rb,0.10010e+04_rb /)
      kbo(:,40,15) = (/ &
        & 0.10089e+04_rb,0.10089e+04_rb,0.10068e+04_rb,0.10054e+04_rb,0.10014e+04_rb /)
      kbo(:,41,15) = (/ &
        & 0.10087e+04_rb,0.10090e+04_rb,0.10082e+04_rb,0.10051e+04_rb,0.10018e+04_rb /)
      kbo(:,42,15) = (/ &
        & 0.10085e+04_rb,0.10090e+04_rb,0.10075e+04_rb,0.10058e+04_rb,0.10025e+04_rb /)
      kbo(:,43,15) = (/ &
        & 0.10071e+04_rb,0.10079e+04_rb,0.10078e+04_rb,0.10086e+04_rb,0.10037e+04_rb /)
      kbo(:,44,15) = (/ &
        & 0.10067e+04_rb,0.10088e+04_rb,0.10079e+04_rb,0.10071e+04_rb,0.10048e+04_rb /)
      kbo(:,45,15) = (/ &
        & 0.10061e+04_rb,0.10075e+04_rb,0.10090e+04_rb,0.10096e+04_rb,0.10057e+04_rb /)
      kbo(:,46,15) = (/ &
        & 0.10054e+04_rb,0.10071e+04_rb,0.10068e+04_rb,0.10078e+04_rb,0.10065e+04_rb /)
      kbo(:,47,15) = (/ &
        & 0.10020e+04_rb,0.10065e+04_rb,0.10087e+04_rb,0.10079e+04_rb,0.10093e+04_rb /)
      kbo(:,48,15) = (/ &
        & 0.99938e+03_rb,0.10058e+04_rb,0.10073e+04_rb,0.10079e+04_rb,0.10087e+04_rb /)
      kbo(:,49,15) = (/ &
        & 0.99869e+03_rb,0.10038e+04_rb,0.10079e+04_rb,0.10099e+04_rb,0.10089e+04_rb /)
      kbo(:,50,15) = (/ &
        & 0.99593e+03_rb,0.10003e+04_rb,0.10062e+04_rb,0.10075e+04_rb,0.10090e+04_rb /)
      kbo(:,51,15) = (/ &
        & 0.99417e+03_rb,0.99877e+03_rb,0.10055e+04_rb,0.10061e+04_rb,0.10090e+04_rb /)
      kbo(:,52,15) = (/ &
        & 0.99334e+03_rb,0.99717e+03_rb,0.10030e+04_rb,0.10077e+04_rb,0.10077e+04_rb /)
      kbo(:,53,15) = (/ &
        & 0.99034e+03_rb,0.99548e+03_rb,0.99989e+03_rb,0.10071e+04_rb,0.10085e+04_rb /)
      kbo(:,54,15) = (/ &
        & 0.98844e+03_rb,0.99385e+03_rb,0.99849e+03_rb,0.10064e+04_rb,0.10081e+04_rb /)
      kbo(:,55,15) = (/ &
        & 0.98654e+03_rb,0.99323e+03_rb,0.99708e+03_rb,0.10005e+04_rb,0.10066e+04_rb /)
      kbo(:,56,15) = (/ &
        & 0.98558e+03_rb,0.99048e+03_rb,0.99560e+03_rb,0.99999e+03_rb,0.10071e+04_rb /)
      kbo(:,57,15) = (/ &
        & 0.98349e+03_rb,0.98867e+03_rb,0.99404e+03_rb,0.99866e+03_rb,0.10054e+04_rb /)
      kbo(:,58,15) = (/ &
        & 0.98039e+03_rb,0.98688e+03_rb,0.99250e+03_rb,0.99837e+03_rb,0.10028e+04_rb /)
      kbo(:,59,15) = (/ &
        & 0.97954e+03_rb,0.98614e+03_rb,0.99186e+03_rb,0.99679e+03_rb,0.10019e+04_rb /)
      kbo(:,13,16) = (/ &
        & 0.98905e+03_rb,0.99319e+03_rb,0.10065e+04_rb,0.10075e+04_rb,0.10080e+04_rb /)
      kbo(:,14,16) = (/ &
        & 0.98947e+03_rb,0.10009e+04_rb,0.10067e+04_rb,0.10076e+04_rb,0.10005e+04_rb /)
      kbo(:,15,16) = (/ &
        & 0.98993e+03_rb,0.10052e+04_rb,0.10068e+04_rb,0.10076e+04_rb,0.10080e+04_rb /)
      kbo(:,16,16) = (/ &
        & 0.99040e+03_rb,0.10054e+04_rb,0.10069e+04_rb,0.10077e+04_rb,0.10005e+04_rb /)
      kbo(:,17,16) = (/ &
        & 0.99823e+03_rb,0.10056e+04_rb,0.99955e+03_rb,0.10078e+04_rb,0.10080e+04_rb /)
      kbo(:,18,16) = (/ &
        & 0.99867e+03_rb,0.99832e+03_rb,0.10071e+04_rb,0.10003e+04_rb,0.10079e+04_rb /)
      kbo(:,19,16) = (/ &
        & 0.99168e+03_rb,0.10059e+04_rb,0.10072e+04_rb,0.10078e+04_rb,0.10079e+04_rb /)
      kbo(:,20,16) = (/ &
        & 0.99955e+03_rb,0.10061e+04_rb,0.10073e+04_rb,0.99297e+03_rb,0.10003e+04_rb /)
      kbo(:,21,16) = (/ &
        & 0.10000e+04_rb,0.10063e+04_rb,0.99994e+03_rb,0.10079e+04_rb,0.10002e+04_rb /)
      kbo(:,22,16) = (/ &
        & 0.10007e+04_rb,0.10066e+04_rb,0.10075e+04_rb,0.10005e+04_rb,0.10075e+04_rb /)
      kbo(:,23,16) = (/ &
        & 0.10052e+04_rb,0.10068e+04_rb,0.10002e+04_rb,0.10080e+04_rb,0.10072e+04_rb /)
      kbo(:,24,16) = (/ &
        & 0.10056e+04_rb,0.10070e+04_rb,0.10078e+04_rb,0.10005e+04_rb,0.10068e+04_rb /)
      kbo(:,25,16) = (/ &
        & 0.10060e+04_rb,0.99977e+03_rb,0.10079e+04_rb,0.99294e+03_rb,0.10064e+04_rb /)
      kbo(:,26,16) = (/ &
        & 0.10064e+04_rb,0.99996e+03_rb,0.10080e+04_rb,0.10077e+04_rb,0.99839e+03_rb /)
      kbo(:,27,16) = (/ &
        & 0.10067e+04_rb,0.10001e+04_rb,0.98561e+03_rb,0.10074e+04_rb,0.10052e+04_rb /)
      kbo(:,28,16) = (/ &
        & 0.10069e+04_rb,0.10003e+04_rb,0.10080e+04_rb,0.10070e+04_rb,0.99700e+03_rb /)
      kbo(:,29,16) = (/ &
        & 0.10072e+04_rb,0.10078e+04_rb,0.10079e+04_rb,0.10065e+04_rb,0.10036e+04_rb /)
      kbo(:,30,16) = (/ &
        & 0.10074e+04_rb,0.98555e+03_rb,0.10002e+04_rb,0.99845e+03_rb,0.95900e+03_rb /)
      kbo(:,31,16) = (/ &
        & 0.10076e+04_rb,0.10080e+04_rb,0.10074e+04_rb,0.99776e+03_rb,0.10037e+04_rb /)
      kbo(:,32,16) = (/ &
        & 0.10077e+04_rb,0.10005e+04_rb,0.10070e+04_rb,0.10044e+04_rb,0.10038e+04_rb /)
      kbo(:,33,16) = (/ &
        & 0.10079e+04_rb,0.10079e+04_rb,0.10064e+04_rb,0.99602e+03_rb,0.10038e+04_rb /)
      kbo(:,34,16) = (/ &
        & 0.10005e+04_rb,0.10077e+04_rb,0.10059e+04_rb,0.10036e+04_rb,0.10037e+04_rb /)
      kbo(:,35,16) = (/ &
        & 0.10080e+04_rb,0.10075e+04_rb,0.10055e+04_rb,0.10037e+04_rb,0.10036e+04_rb /)
      kbo(:,36,16) = (/ &
        & 0.10005e+04_rb,0.10075e+04_rb,0.10055e+04_rb,0.10037e+04_rb,0.10036e+04_rb /)
      kbo(:,37,16) = (/ &
        & 0.10080e+04_rb,0.10077e+04_rb,0.10058e+04_rb,0.10037e+04_rb,0.10037e+04_rb /)
      kbo(:,38,16) = (/ &
        & 0.10004e+04_rb,0.10078e+04_rb,0.10062e+04_rb,0.99569e+03_rb,0.10037e+04_rb /)
      kbo(:,39,16) = (/ &
        & 0.10078e+04_rb,0.10079e+04_rb,0.10065e+04_rb,0.10036e+04_rb,0.10038e+04_rb /)
      kbo(:,40,16) = (/ &
        & 0.10003e+04_rb,0.10005e+04_rb,0.10070e+04_rb,0.99698e+03_rb,0.10038e+04_rb /)
      kbo(:,41,16) = (/ &
        & 0.10001e+04_rb,0.10005e+04_rb,0.99992e+03_rb,0.10052e+04_rb,0.10037e+04_rb /)
      kbo(:,42,16) = (/ &
        & 0.99994e+03_rb,0.10005e+04_rb,0.10077e+04_rb,0.10059e+04_rb,0.10036e+04_rb /)
      kbo(:,43,16) = (/ &
        & 0.10071e+04_rb,0.10078e+04_rb,0.10079e+04_rb,0.99172e+03_rb,0.10038e+04_rb /)
      kbo(:,44,16) = (/ &
        & 0.10068e+04_rb,0.10002e+04_rb,0.10080e+04_rb,0.10072e+04_rb,0.10049e+04_rb /)
      kbo(:,45,16) = (/ &
        & 0.10063e+04_rb,0.10074e+04_rb,0.10005e+04_rb,0.99276e+03_rb,0.10059e+04_rb /)
      kbo(:,46,16) = (/ &
        & 0.10057e+04_rb,0.10071e+04_rb,0.10078e+04_rb,0.10079e+04_rb,0.10067e+04_rb /)
      kbo(:,47,16) = (/ &
        & 0.10009e+04_rb,0.10067e+04_rb,0.10001e+04_rb,0.10080e+04_rb,0.99247e+03_rb /)
      kbo(:,48,16) = (/ &
        & 0.99941e+03_rb,0.10061e+04_rb,0.10073e+04_rb,0.10079e+04_rb,0.10004e+04_rb /)
      kbo(:,49,16) = (/ &
        & 0.99034e+03_rb,0.10054e+04_rb,0.99943e+03_rb,0.99276e+03_rb,0.10005e+04_rb /)
      kbo(:,50,16) = (/ &
        & 0.99601e+03_rb,0.10003e+04_rb,0.10064e+04_rb,0.10074e+04_rb,0.10005e+04_rb /)
      kbo(:,51,16) = (/ &
        & 0.99422e+03_rb,0.99882e+03_rb,0.10058e+04_rb,0.10071e+04_rb,0.10003e+04_rb /)
      kbo(:,52,16) = (/ &
        & 0.98496e+03_rb,0.99726e+03_rb,0.10013e+04_rb,0.99930e+03_rb,0.10076e+04_rb /)
      kbo(:,53,16) = (/ &
        & 0.99030e+03_rb,0.99555e+03_rb,0.99990e+03_rb,0.99882e+03_rb,0.99991e+03_rb /)
      kbo(:,54,16) = (/ &
        & 0.98836e+03_rb,0.98653e+03_rb,0.99856e+03_rb,0.99827e+03_rb,0.99962e+03_rb /)
      kbo(:,55,16) = (/ &
        & 0.98643e+03_rb,0.98485e+03_rb,0.99716e+03_rb,0.10012e+04_rb,0.10067e+04_rb /)
      kbo(:,56,16) = (/ &
        & 0.97714e+03_rb,0.99044e+03_rb,0.99567e+03_rb,0.99999e+03_rb,0.99885e+03_rb /)
      kbo(:,57,16) = (/ &
        & 0.97511e+03_rb,0.98860e+03_rb,0.99409e+03_rb,0.99872e+03_rb,0.10058e+04_rb /)
      kbo(:,58,16) = (/ &
        & 0.98036e+03_rb,0.98678e+03_rb,0.99251e+03_rb,0.99003e+03_rb,0.10052e+04_rb /)
      kbo(:,59,16) = (/ &
        & 0.97953e+03_rb,0.98603e+03_rb,0.99185e+03_rb,0.99687e+03_rb,0.10009e+04_rb /)
     
      end subroutine sw_kgb27

! **************************************************************************
      subroutine sw_kgb28
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg28, only : kao, kbo, sfluxrefo, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      save

! Kurucz solar source function
      sfluxrefo(:,1) = (/ &
        & 1.06156_rb    , 0.599910_rb   , 0.422462_rb   , 0.400077_rb   , &
        & 0.282221_rb   , 0.187893_rb   , 6.77357e-02_rb, 3.04572e-02_rb, &
        & 2.00442e-02_rb, 2.30786e-03_rb, 2.08824e-03_rb, 1.42604e-03_rb, &
        & 9.67384e-04_rb, 6.35362e-04_rb, 1.47727e-04_rb, 6.87639e-06_rb /)
      sfluxrefo(:,2) = (/ &
        & 1.07598_rb    , 0.585099_rb   , 0.422852_rb   , 0.400077_rb   , &
        & 0.282221_rb   , 0.187893_rb   , 6.69686e-02_rb, 3.09070e-02_rb, &
        & 2.02400e-02_rb, 2.47760e-03_rb, 1.89411e-03_rb, 1.41122e-03_rb, &
        & 1.12449e-03_rb, 5.73505e-04_rb, 2.04160e-04_rb, 1.58371e-05_rb /)
      sfluxrefo(:,3) = (/ &
        & 0.461647_rb   , 0.406113_rb   , 0.332506_rb   , 0.307508_rb   , &
        & 0.211167_rb   , 0.235457_rb   , 0.495886_rb   , 0.363921_rb   , &
        & 0.192700_rb   , 2.04678e-02_rb, 1.55407e-02_rb, 1.03882e-02_rb, &
        & 1.10778e-02_rb, 1.00504e-02_rb, 4.93497e-03_rb, 5.73410e-04_rb /)
      sfluxrefo(:,4) = (/ &
        & 0.132669_rb   , 0.175058_rb   , 0.359263_rb   , 0.388142_rb   , &
        & 0.350359_rb   , 0.475892_rb   , 0.489593_rb   , 0.408437_rb   , &
        & 0.221049_rb   , 1.94514e-02_rb, 1.54848e-02_rb, 1.44999e-02_rb, &
        & 1.44568e-02_rb, 1.00527e-02_rb, 4.95897e-03_rb, 5.73327e-04_rb /)
      sfluxrefo(:,5) = (/ &
        & 7.54800e-02_rb, 0.232246_rb   , 0.359263_rb   , 0.388142_rb   , &
        & 0.350359_rb   , 0.426317_rb   , 0.493485_rb   , 0.432016_rb   , &
        & 0.239203_rb   , 1.74951e-02_rb, 1.74477e-02_rb, 1.83566e-02_rb, &
        & 1.44818e-02_rb, 1.01048e-02_rb, 4.97487e-03_rb, 5.66831e-04_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:,1) = (/ &
        & 9.23242E-01_rb, 5.50262E-01_rb, 3.94471E-01_rb, 3.75035E-01_rb,&
        & 2.58536E-01_rb, 1.76381E-01_rb, 6.41715E-02_rb, 2.89114E-02_rb,&
        & 1.48259E-02_rb, 1.81210E-03_rb, 1.80010E-03_rb, 1.19695E-03_rb,&
        & 9.97711E-04_rb, 5.51536E-04_rb, 1.38032E-04_rb, 3.07915E-05_rb/)
      irradnceo(:,2) = (/ &
        & 9.35244E-01_rb, 5.33447E-01_rb, 3.99285E-01_rb, 3.75035E-01_rb,&
        & 2.58536E-01_rb, 1.76381E-01_rb, 6.41715E-02_rb, 2.86312E-02_rb,&
        & 1.51901E-02_rb, 1.76000E-03_rb, 1.76488E-03_rb, 1.21655E-03_rb,&
        & 1.02671E-03_rb, 5.17487E-04_rb, 1.44549E-04_rb, 1.30341E-05_rb/)
      irradnceo(:,3) = (/ &
        & 4.34804E-01_rb, 3.89880E-01_rb, 3.44099E-01_rb, 5.12283E-01_rb,&
        & 5.02214E-01_rb, 3.33402E-01_rb, 2.11876E-01_rb, 4.18951E-02_rb,&
        & 1.55592E-02_rb, 1.71751E-03_rb, 1.52230E-03_rb, 1.28360E-03_rb,&
        & 1.09833E-03_rb, 5.07196E-04_rb, 1.96714E-04_rb, 2.62763E-05_rb/)
      irradnceo(:,4) = (/ &
        & 2.54821E-01_rb, 1.32937E-01_rb, 2.28677E-01_rb, 3.66257E-01_rb,&
        & 3.24096E-01_rb, 4.39505E-01_rb, 4.16793E-01_rb, 3.78321E-01_rb,&
        & 1.72447E-01_rb, 2.28376E-02_rb, 1.16041E-02_rb, 1.53595E-02_rb,&
        & 1.70854E-02_rb, 6.64788E-03_rb, 4.52287E-03_rb, 4.51557E-04_rb/)
      irradnceo(:,5) = (/ &
        & 6.77685E-02_rb, 2.13208E-01_rb, 3.35458E-01_rb, 3.66257E-01_rb,&
        & 3.24096E-01_rb, 4.07488E-01_rb, 3.97365E-01_rb, 3.94103E-01_rb,&
        & 2.04588E-01_rb, 1.39082E-02_rb, 1.84993E-02_rb, 2.08802E-02_rb,&
        & 1.69242E-02_rb, 6.75691E-03_rb, 4.41066E-03_rb, 6.51614E-04_rb/)
! Solar variability components: facular brightening
      facbrghto(:,1) = (/ &
        & 8.94260E-03_rb, 7.05274E-03_rb, 5.62959E-03_rb, 5.32177E-03_rb,&
        & 4.05892E-03_rb, 2.70338E-03_rb, 1.67619E-03_rb, 9.37834E-04_rb,&
        & 4.89025E-04_rb, 5.54583E-05_rb, 4.65152E-05_rb, 3.72440E-05_rb,&
        & 2.73222E-05_rb, 1.79851E-05_rb, 5.86530E-06_rb, 9.14859E-07_rb/)
      facbrghto(:,2) = (/ &
        & 9.03133E-03_rb, 6.91971E-03_rb, 5.67388E-03_rb, 5.32177E-03_rb,&
        & 4.05892E-03_rb, 2.70338E-03_rb, 1.67619E-03_rb, 9.35156E-04_rb,&
        & 4.92581E-04_rb, 5.51092E-05_rb, 4.62811E-05_rb, 3.70568E-05_rb,&
        & 2.72642E-05_rb, 1.76984E-05_rb, 6.52825E-06_rb, 4.88668E-07_rb/)
      facbrghto(:,3) = (/ &
        & 6.17725E-03_rb, 5.71948E-03_rb, 5.17639E-03_rb, 5.97871E-03_rb,&
        & 5.01536E-03_rb, 4.26095E-03_rb, 2.90785E-03_rb, 1.07749E-03_rb,&
        & 5.01063E-04_rb, 5.44961E-05_rb, 4.58037E-05_rb, 3.66346E-05_rb,&
        & 2.75308E-05_rb, 1.74820E-05_rb, 5.95745E-06_rb, 8.96580E-07_rb/)
      facbrghto(:,4) = (/ &
        & 4.01090E-03_rb, 3.19646E-03_rb, 3.74391E-03_rb, 5.23833E-03_rb,&
        & 4.66285E-03_rb, 5.19210E-03_rb, 4.69477E-03_rb, 3.33268E-03_rb,&
        & 2.10668E-03_rb, 2.23501E-04_rb, 1.94378E-04_rb, 1.60047E-04_rb,&
        & 1.29611E-04_rb, 8.24397E-05_rb, 3.04256E-05_rb, 4.27101E-06_rb/)
      facbrghto(:,5) = (/ &
        & 2.21756E-03_rb, 3.66178E-03_rb, 5.07193E-03_rb, 5.23833E-03_rb,&
        & 4.66285E-03_rb, 4.75028E-03_rb, 5.01144E-03_rb, 3.39362E-03_rb,&
        & 2.13143E-03_rb, 2.37271E-04_rb, 2.01780E-04_rb, 1.78243E-04_rb,&
        & 1.29586E-04_rb, 8.25391E-05_rb, 3.04341E-05_rb, 4.27232E-06_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:,1) = (/ &
        &-5.42001E-04_rb,-2.88612E-04_rb,-2.19042E-04_rb,-2.63618E-04_rb,&
        &-1.94681E-04_rb,-1.30544E-04_rb,-3.56374E-05_rb,-1.63358E-05_rb,&
        &-8.70345E-06_rb,-1.13907E-06_rb,-9.53870E-07_rb,-7.68985E-07_rb,&
        &-5.66993E-07_rb,-3.73814E-07_rb,-1.22016E-07_rb,-1.90312E-08_rb/)
      snsptdrko(:,2) = (/ &
        &-5.45408E-04_rb,-2.82160E-04_rb,-2.22086E-04_rb,-2.63618E-04_rb,&
        &-1.94681E-04_rb,-1.30544E-04_rb,-3.56374E-05_rb,-1.62791E-05_rb,&
        &-8.78087E-06_rb,-1.13455E-06_rb,-9.48283E-07_rb,-7.62481E-07_rb,&
        &-5.64628E-07_rb,-3.67365E-07_rb,-1.35594E-07_rb,-1.01520E-08_rb/)
      snsptdrko(:,3) = (/ &
        &-3.03939E-04_rb,-2.52811E-04_rb,-2.12612E-04_rb,-2.95530E-04_rb,&
        &-2.73655E-04_rb,-2.05534E-04_rb,-1.20984E-04_rb,-2.52159E-05_rb,&
        &-8.95330E-06_rb,-1.12452E-06_rb,-9.39286E-07_rb,-7.48303E-07_rb,&
        &-5.67632E-07_rb,-3.61776E-07_rb,-1.23360E-07_rb,-1.85684E-08_rb/)
      snsptdrko(:,4) = (/ &
        &-1.88122E-04_rb,-9.08383E-05_rb,-1.59058E-04_rb,-2.55097E-04_rb,&
        &-1.74596E-04_rb,-2.40621E-04_rb,-2.19367E-04_rb,-2.01433E-04_rb,&
        &-1.24533E-04_rb,-1.14447E-05_rb,-1.01739E-05_rb,-9.65448E-06_rb,&
        &-9.59121E-06_rb,-6.08667E-06_rb,-2.19334E-06_rb,-3.08202E-07_rb/)
      snsptdrko(:,5) = (/ &
        &-3.77908E-05_rb,-1.52621E-04_rb,-2.47606E-04_rb,-2.55097E-04_rb,&
        &-1.74596E-04_rb,-2.19374E-04_rb,-2.01739E-04_rb,-2.29888E-04_rb,&
        &-1.26606E-04_rb,-1.37927E-05_rb,-1.23743E-05_rb,-1.34401E-05_rb,&
        &-9.58716E-06_rb,-6.10238E-06_rb,-2.19460E-06_rb,-3.08396E-07_rb/)

! Rayleigh extinction coefficient at v = ????? cm-1.
      rayl = 2.02e-05_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1, 1) = (/ &
        & 0.46447e+02_rb,0.18001e+03_rb,0.31338e+03_rb,0.44652e+03_rb,0.54636e+03_rb, &
        & 0.54173e+03_rb,0.45707e+03_rb,0.27363e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 1, 1) = (/ &
        & 0.45270e+02_rb,0.17886e+03_rb,0.31225e+03_rb,0.44541e+03_rb,0.54177e+03_rb, &
        & 0.53436e+03_rb,0.44932e+03_rb,0.26748e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 1, 1) = (/ &
        & 0.44214e+02_rb,0.17776e+03_rb,0.31115e+03_rb,0.44427e+03_rb,0.53733e+03_rb, &
        & 0.52756e+03_rb,0.44224e+03_rb,0.26195e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 1, 1) = (/ &
        & 0.43259e+02_rb,0.17673e+03_rb,0.31005e+03_rb,0.44305e+03_rb,0.53298e+03_rb, &
        & 0.52129e+03_rb,0.43574e+03_rb,0.25696e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 1, 1) = (/ &
        & 0.42393e+02_rb,0.17575e+03_rb,0.30896e+03_rb,0.44173e+03_rb,0.52883e+03_rb, &
        & 0.51548e+03_rb,0.42976e+03_rb,0.25242e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 2, 1) = (/ &
        & 0.42908e+02_rb,0.17695e+03_rb,0.31077e+03_rb,0.44434e+03_rb,0.53161e+03_rb, &
        & 0.51895e+03_rb,0.43332e+03_rb,0.25512e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 2, 1) = (/ &
        & 0.41899e+02_rb,0.17596e+03_rb,0.30980e+03_rb,0.44336e+03_rb,0.52686e+03_rb, &
        & 0.51213e+03_rb,0.42632e+03_rb,0.24984e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 2, 1) = (/ &
        & 0.40995e+02_rb,0.17502e+03_rb,0.30884e+03_rb,0.44232e+03_rb,0.52230e+03_rb, &
        & 0.50592e+03_rb,0.41995e+03_rb,0.24511e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 2, 1) = (/ &
        & 0.40181e+02_rb,0.17412e+03_rb,0.30788e+03_rb,0.44119e+03_rb,0.51785e+03_rb, &
        & 0.50025e+03_rb,0.41414e+03_rb,0.24085e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 2, 1) = (/ &
        & 0.39443e+02_rb,0.17326e+03_rb,0.30691e+03_rb,0.43992e+03_rb,0.51364e+03_rb, &
        & 0.49503e+03_rb,0.40882e+03_rb,0.23699e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 3, 1) = (/ &
        & 0.40126e+02_rb,0.17457e+03_rb,0.30876e+03_rb,0.44248e+03_rb,0.51755e+03_rb, &
        & 0.49986e+03_rb,0.41375e+03_rb,0.24056e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 3, 1) = (/ &
        & 0.39240e+02_rb,0.17370e+03_rb,0.30792e+03_rb,0.44146e+03_rb,0.51254e+03_rb, &
        & 0.49358e+03_rb,0.40735e+03_rb,0.23593e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 3, 1) = (/ &
        & 0.38450e+02_rb,0.17288e+03_rb,0.30708e+03_rb,0.44031e+03_rb,0.50793e+03_rb, &
        & 0.48786e+03_rb,0.40155e+03_rb,0.23179e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 3, 1) = (/ &
        & 0.37740e+02_rb,0.17210e+03_rb,0.30624e+03_rb,0.43908e+03_rb,0.50368e+03_rb, &
        & 0.48264e+03_rb,0.39629e+03_rb,0.22808e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 3, 1) = (/ &
        & 0.37099e+02_rb,0.17134e+03_rb,0.30539e+03_rb,0.43781e+03_rb,0.49975e+03_rb, &
        & 0.47787e+03_rb,0.39149e+03_rb,0.22473e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 4, 1) = (/ &
        & 0.37818e+02_rb,0.17258e+03_rb,0.30708e+03_rb,0.43967e+03_rb,0.50412e+03_rb, &
        & 0.48321e+03_rb,0.39687e+03_rb,0.22848e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 4, 1) = (/ &
        & 0.37034e+02_rb,0.17183e+03_rb,0.30636e+03_rb,0.43833e+03_rb,0.49935e+03_rb, &
        & 0.47738e+03_rb,0.39100e+03_rb,0.22439e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 4, 1) = (/ &
        & 0.36338e+02_rb,0.17111e+03_rb,0.30564e+03_rb,0.43700e+03_rb,0.49496e+03_rb, &
        & 0.47212e+03_rb,0.38572e+03_rb,0.22075e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 4, 1) = (/ &
        & 0.35716e+02_rb,0.17043e+03_rb,0.30491e+03_rb,0.43569e+03_rb,0.49094e+03_rb, &
        & 0.46732e+03_rb,0.38094e+03_rb,0.21749e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 4, 1) = (/ &
        & 0.35156e+02_rb,0.16977e+03_rb,0.30416e+03_rb,0.43437e+03_rb,0.48726e+03_rb, &
        & 0.46294e+03_rb,0.37660e+03_rb,0.21456e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 5, 1) = (/ &
        & 0.35857e+02_rb,0.17089e+03_rb,0.30563e+03_rb,0.43619e+03_rb,0.49186e+03_rb, &
        & 0.46841e+03_rb,0.38202e+03_rb,0.21822e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 5, 1) = (/ &
        & 0.35163e+02_rb,0.17023e+03_rb,0.30502e+03_rb,0.43478e+03_rb,0.48731e+03_rb, &
        & 0.46300e+03_rb,0.37666e+03_rb,0.21460e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 5, 1) = (/ &
        & 0.34550e+02_rb,0.16961e+03_rb,0.30441e+03_rb,0.43332e+03_rb,0.48321e+03_rb, &
        & 0.45813e+03_rb,0.37186e+03_rb,0.21139e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 5, 1) = (/ &
        & 0.34004e+02_rb,0.16902e+03_rb,0.30378e+03_rb,0.43194e+03_rb,0.47950e+03_rb, &
        & 0.45375e+03_rb,0.36755e+03_rb,0.20853e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 5, 1) = (/ &
        & 0.33514e+02_rb,0.16844e+03_rb,0.30312e+03_rb,0.43058e+03_rb,0.47611e+03_rb, &
        & 0.44977e+03_rb,0.36365e+03_rb,0.20597e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 6, 1) = (/ &
        & 0.34212e+02_rb,0.16946e+03_rb,0.30440e+03_rb,0.43247e+03_rb,0.48092e+03_rb, &
        & 0.45542e+03_rb,0.36920e+03_rb,0.20962e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 6, 1) = (/ &
        & 0.33595e+02_rb,0.16889e+03_rb,0.30389e+03_rb,0.43093e+03_rb,0.47667e+03_rb, &
        & 0.45043e+03_rb,0.36430e+03_rb,0.20639e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 6, 1) = (/ &
        & 0.33052e+02_rb,0.16835e+03_rb,0.30337e+03_rb,0.42946e+03_rb,0.47287e+03_rb, &
        & 0.44599e+03_rb,0.35993e+03_rb,0.20355e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 6, 1) = (/ &
        & 0.32571e+02_rb,0.16784e+03_rb,0.30283e+03_rb,0.42807e+03_rb,0.46947e+03_rb, &
        & 0.44201e+03_rb,0.35602e+03_rb,0.20103e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 6, 1) = (/ &
        & 0.32141e+02_rb,0.16733e+03_rb,0.30227e+03_rb,0.42674e+03_rb,0.46640e+03_rb, &
        & 0.43842e+03_rb,0.35250e+03_rb,0.19879e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 7, 1) = (/ &
        & 0.32812e+02_rb,0.16823e+03_rb,0.30334e+03_rb,0.42859e+03_rb,0.47118e+03_rb, &
        & 0.44401e+03_rb,0.35799e+03_rb,0.20230e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 7, 1) = (/ &
        & 0.32263e+02_rb,0.16774e+03_rb,0.30291e+03_rb,0.42708e+03_rb,0.46727e+03_rb, &
        & 0.43944e+03_rb,0.35350e+03_rb,0.19942e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 7, 1) = (/ &
        & 0.31782e+02_rb,0.16728e+03_rb,0.30248e+03_rb,0.42558e+03_rb,0.46381e+03_rb, &
        & 0.43539e+03_rb,0.34954e+03_rb,0.19691e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 7, 1) = (/ &
        & 0.31357e+02_rb,0.16683e+03_rb,0.30203e+03_rb,0.42413e+03_rb,0.46069e+03_rb, &
        & 0.43177e+03_rb,0.34599e+03_rb,0.19468e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 7, 1) = (/ &
        & 0.30980e+02_rb,0.16640e+03_rb,0.30154e+03_rb,0.42276e+03_rb,0.45787e+03_rb, &
        & 0.42852e+03_rb,0.34280e+03_rb,0.19271e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 8, 1) = (/ &
        & 0.31614e+02_rb,0.16716e+03_rb,0.30239e+03_rb,0.42473e+03_rb,0.46259e+03_rb, &
        & 0.43397e+03_rb,0.34814e+03_rb,0.19603e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 8, 1) = (/ &
        & 0.31125e+02_rb,0.16674e+03_rb,0.30206e+03_rb,0.42311e+03_rb,0.45896e+03_rb, &
        & 0.42978e+03_rb,0.34403e+03_rb,0.19347e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 8, 1) = (/ &
        & 0.30699e+02_rb,0.16635e+03_rb,0.30170e+03_rb,0.42163e+03_rb,0.45573e+03_rb, &
        & 0.42608e+03_rb,0.34039e+03_rb,0.19124e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 8, 1) = (/ &
        & 0.30325e+02_rb,0.16597e+03_rb,0.30133e+03_rb,0.42027e+03_rb,0.45286e+03_rb, &
        & 0.42281e+03_rb,0.33716e+03_rb,0.18928e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 8, 1) = (/ &
        & 0.29993e+02_rb,0.16559e+03_rb,0.30092e+03_rb,0.41901e+03_rb,0.45030e+03_rb, &
        & 0.41988e+03_rb,0.33427e+03_rb,0.18755e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 9, 1) = (/ &
        & 0.30594e+02_rb,0.16624e+03_rb,0.30156e+03_rb,0.42087e+03_rb,0.45493e+03_rb, &
        & 0.42517e+03_rb,0.33949e+03_rb,0.19069e+03_rb,0.35000e+02_rb /)
      kao(:, 2, 9, 1) = (/ &
        & 0.30158e+02_rb,0.16589e+03_rb,0.30131e+03_rb,0.41937e+03_rb,0.45157e+03_rb, &
        & 0.42134e+03_rb,0.33570e+03_rb,0.18841e+03_rb,0.35000e+02_rb /)
      kao(:, 3, 9, 1) = (/ &
        & 0.29780e+02_rb,0.16556e+03_rb,0.30103e+03_rb,0.41800e+03_rb,0.44864e+03_rb, &
        & 0.41799e+03_rb,0.33239e+03_rb,0.18643e+03_rb,0.35000e+02_rb /)
      kao(:, 4, 9, 1) = (/ &
        & 0.29450e+02_rb,0.16523e+03_rb,0.30072e+03_rb,0.41674e+03_rb,0.44606e+03_rb, &
        & 0.41504e+03_rb,0.32946e+03_rb,0.18471e+03_rb,0.35000e+02_rb /)
      kao(:, 5, 9, 1) = (/ &
        & 0.29159e+02_rb,0.16491e+03_rb,0.30039e+03_rb,0.41557e+03_rb,0.44377e+03_rb, &
        & 0.41242e+03_rb,0.32686e+03_rb,0.18318e+03_rb,0.35000e+02_rb /)
      kao(:, 1,10, 1) = (/ &
        & 0.29691e+02_rb,0.16541e+03_rb,0.30081e+03_rb,0.41722e+03_rb,0.44795e+03_rb, &
        & 0.41720e+03_rb,0.33160e+03_rb,0.18597e+03_rb,0.35000e+02_rb /)
      kao(:, 2,10, 1) = (/ &
        & 0.29307e+02_rb,0.16512e+03_rb,0.30063e+03_rb,0.41581e+03_rb,0.44493e+03_rb, &
        & 0.41375e+03_rb,0.32818e+03_rb,0.18396e+03_rb,0.35000e+02_rb /)
      kao(:, 3,10, 1) = (/ &
        & 0.28975e+02_rb,0.16485e+03_rb,0.30042e+03_rb,0.41454e+03_rb,0.44231e+03_rb, &
        & 0.41076e+03_rb,0.32520e+03_rb,0.18222e+03_rb,0.35000e+02_rb /)
      kao(:, 4,10, 1) = (/ &
        & 0.28687e+02_rb,0.16458e+03_rb,0.30018e+03_rb,0.41340e+03_rb,0.44002e+03_rb, &
        & 0.40814e+03_rb,0.32259e+03_rb,0.18071e+03_rb,0.35000e+02_rb /)
      kao(:, 5,10, 1) = (/ &
        & 0.28434e+02_rb,0.16431e+03_rb,0.29989e+03_rb,0.41235e+03_rb,0.43799e+03_rb, &
        & 0.40583e+03_rb,0.32028e+03_rb,0.17939e+03_rb,0.35000e+02_rb /)
      kao(:, 1,11, 1) = (/ &
        & 0.28760e+02_rb,0.16459e+03_rb,0.30010e+03_rb,0.41321e+03_rb,0.44060e+03_rb, &
        & 0.40880e+03_rb,0.32325e+03_rb,0.18109e+03_rb,0.35000e+02_rb /)
      kao(:, 2,11, 1) = (/ &
        & 0.28444e+02_rb,0.16436e+03_rb,0.29997e+03_rb,0.41200e+03_rb,0.43807e+03_rb, &
        & 0.40592e+03_rb,0.32038e+03_rb,0.17944e+03_rb,0.35000e+02_rb /)
      kao(:, 3,11, 1) = (/ &
        & 0.28173e+02_rb,0.16414e+03_rb,0.29981e+03_rb,0.41092e+03_rb,0.43587e+03_rb, &
        & 0.40342e+03_rb,0.31788e+03_rb,0.17802e+03_rb,0.35000e+02_rb /)
      kao(:, 4,11, 1) = (/ &
        & 0.27937e+02_rb,0.16392e+03_rb,0.29960e+03_rb,0.40994e+03_rb,0.43395e+03_rb, &
        & 0.40124e+03_rb,0.31570e+03_rb,0.17679e+03_rb,0.35000e+02_rb /)
      kao(:, 5,11, 1) = (/ &
        & 0.27730e+02_rb,0.16369e+03_rb,0.29936e+03_rb,0.40905e+03_rb,0.43224e+03_rb, &
        & 0.39931e+03_rb,0.31377e+03_rb,0.17571e+03_rb,0.35000e+02_rb /)
      kao(:, 1,12, 1) = (/ &
        & 0.27995e+02_rb,0.16391e+03_rb,0.29951e+03_rb,0.40970e+03_rb,0.43443e+03_rb, &
        & 0.40178e+03_rb,0.31624e+03_rb,0.17709e+03_rb,0.35000e+02_rb /)
      kao(:, 2,12, 1) = (/ &
        & 0.27737e+02_rb,0.16374e+03_rb,0.29943e+03_rb,0.40869e+03_rb,0.43230e+03_rb, &
        & 0.39938e+03_rb,0.31384e+03_rb,0.17574e+03_rb,0.35000e+02_rb /)
      kao(:, 3,12, 1) = (/ &
        & 0.27515e+02_rb,0.16356e+03_rb,0.29930e+03_rb,0.40779e+03_rb,0.43046e+03_rb, &
        & 0.39730e+03_rb,0.31176e+03_rb,0.17458e+03_rb,0.35000e+02_rb /)
      kao(:, 4,12, 1) = (/ &
        & 0.27322e+02_rb,0.16338e+03_rb,0.29913e+03_rb,0.40697e+03_rb,0.42883e+03_rb, &
        & 0.39548e+03_rb,0.30994e+03_rb,0.17357e+03_rb,0.35000e+02_rb /)
      kao(:, 5,12, 1) = (/ &
        & 0.27152e+02_rb,0.16318e+03_rb,0.29892e+03_rb,0.40622e+03_rb,0.42740e+03_rb, &
        & 0.39387e+03_rb,0.30834e+03_rb,0.17268e+03_rb,0.35000e+02_rb /)
      kao(:, 1,13, 1) = (/ &
        & 0.27369e+02_rb,0.16336e+03_rb,0.29903e+03_rb,0.40671e+03_rb,0.42924e+03_rb, &
        & 0.39593e+03_rb,0.31039e+03_rb,0.17382e+03_rb,0.35000e+02_rb /)
      kao(:, 2,13, 1) = (/ &
        & 0.27158e+02_rb,0.16323e+03_rb,0.29898e+03_rb,0.40587e+03_rb,0.42745e+03_rb, &
        & 0.39393e+03_rb,0.30840e+03_rb,0.17271e+03_rb,0.35000e+02_rb /)
      kao(:, 3,13, 1) = (/ &
        & 0.26976e+02_rb,0.16309e+03_rb,0.29889e+03_rb,0.40513e+03_rb,0.42590e+03_rb, &
        & 0.39221e+03_rb,0.30667e+03_rb,0.17176e+03_rb,0.35000e+02_rb /)
      kao(:, 4,13, 1) = (/ &
        & 0.26818e+02_rb,0.16293e+03_rb,0.29875e+03_rb,0.40445e+03_rb,0.42455e+03_rb, &
        & 0.39070e+03_rb,0.30516e+03_rb,0.17093e+03_rb,0.35000e+02_rb /)
      kao(:, 5,13, 1) = (/ &
        & 0.26679e+02_rb,0.16276e+03_rb,0.29855e+03_rb,0.40381e+03_rb,0.42335e+03_rb, &
        & 0.38937e+03_rb,0.30384e+03_rb,0.17021e+03_rb,0.35000e+02_rb /)
      kao(:, 1, 1, 2) = (/ &
        & 0.17271e+03_rb,0.27646e+03_rb,0.38039e+03_rb,0.48420e+03_rb,0.56932e+03_rb, &
        & 0.57534e+03_rb,0.47873e+03_rb,0.28147e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 1, 2) = (/ &
        & 0.16834e+03_rb,0.27246e+03_rb,0.37678e+03_rb,0.48091e+03_rb,0.56471e+03_rb, &
        & 0.56734e+03_rb,0.47005e+03_rb,0.27593e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 1, 2) = (/ &
        & 0.16441e+03_rb,0.26883e+03_rb,0.37341e+03_rb,0.47777e+03_rb,0.56037e+03_rb, &
        & 0.56003e+03_rb,0.46210e+03_rb,0.27096e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 1, 2) = (/ &
        & 0.16086e+03_rb,0.26548e+03_rb,0.37025e+03_rb,0.47479e+03_rb,0.55629e+03_rb, &
        & 0.55327e+03_rb,0.45481e+03_rb,0.26647e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 1, 2) = (/ &
        & 0.15764e+03_rb,0.26237e+03_rb,0.36725e+03_rb,0.47195e+03_rb,0.55235e+03_rb, &
        & 0.54701e+03_rb,0.44810e+03_rb,0.26239e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 2, 2) = (/ &
        & 0.15955e+03_rb,0.26500e+03_rb,0.37067e+03_rb,0.47595e+03_rb,0.55480e+03_rb, &
        & 0.55074e+03_rb,0.45210e+03_rb,0.26482e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 2, 2) = (/ &
        & 0.15580e+03_rb,0.26157e+03_rb,0.36756e+03_rb,0.47303e+03_rb,0.55028e+03_rb, &
        & 0.54339e+03_rb,0.44423e+03_rb,0.26007e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 2, 2) = (/ &
        & 0.15244e+03_rb,0.25844e+03_rb,0.36465e+03_rb,0.47023e+03_rb,0.54619e+03_rb, &
        & 0.53664e+03_rb,0.43708e+03_rb,0.25581e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 2, 2) = (/ &
        & 0.14941e+03_rb,0.25556e+03_rb,0.36191e+03_rb,0.46753e+03_rb,0.54253e+03_rb, &
        & 0.53041e+03_rb,0.43052e+03_rb,0.25198e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 2, 2) = (/ &
        & 0.14667e+03_rb,0.25289e+03_rb,0.35930e+03_rb,0.46494e+03_rb,0.53908e+03_rb, &
        & 0.52470e+03_rb,0.42448e+03_rb,0.24851e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 3, 2) = (/ &
        & 0.14921e+03_rb,0.25601e+03_rb,0.36308e+03_rb,0.46872e+03_rb,0.54234e+03_rb, &
        & 0.52999e+03_rb,0.43007e+03_rb,0.25172e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 3, 2) = (/ &
        & 0.14591e+03_rb,0.25301e+03_rb,0.36036e+03_rb,0.46599e+03_rb,0.53834e+03_rb, &
        & 0.52313e+03_rb,0.42280e+03_rb,0.24755e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 3, 2) = (/ &
        & 0.14297e+03_rb,0.25028e+03_rb,0.35782e+03_rb,0.46350e+03_rb,0.53459e+03_rb, &
        & 0.51696e+03_rb,0.41619e+03_rb,0.24383e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 3, 2) = (/ &
        & 0.14033e+03_rb,0.24777e+03_rb,0.35542e+03_rb,0.46116e+03_rb,0.53100e+03_rb, &
        & 0.51122e+03_rb,0.41016e+03_rb,0.24049e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 3, 2) = (/ &
        & 0.13795e+03_rb,0.24544e+03_rb,0.35313e+03_rb,0.45882e+03_rb,0.52751e+03_rb, &
        & 0.50591e+03_rb,0.40463e+03_rb,0.23747e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 4, 2) = (/ &
        & 0.14062e+03_rb,0.24856e+03_rb,0.35678e+03_rb,0.46260e+03_rb,0.53137e+03_rb, &
        & 0.51186e+03_rb,0.41082e+03_rb,0.24086e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 4, 2) = (/ &
        & 0.13771e+03_rb,0.24592e+03_rb,0.35440e+03_rb,0.46020e+03_rb,0.52732e+03_rb, &
        & 0.50537e+03_rb,0.40407e+03_rb,0.23717e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 4, 2) = (/ &
        & 0.13512e+03_rb,0.24353e+03_rb,0.35219e+03_rb,0.45796e+03_rb,0.52348e+03_rb, &
        & 0.49954e+03_rb,0.39798e+03_rb,0.23389e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 4, 2) = (/ &
        & 0.13281e+03_rb,0.24133e+03_rb,0.35008e+03_rb,0.45583e+03_rb,0.51985e+03_rb, &
        & 0.49431e+03_rb,0.39246e+03_rb,0.23096e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 4, 2) = (/ &
        & 0.13073e+03_rb,0.23928e+03_rb,0.34807e+03_rb,0.45378e+03_rb,0.51643e+03_rb, &
        & 0.48957e+03_rb,0.38745e+03_rb,0.22833e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 5, 2) = (/ &
        & 0.13333e+03_rb,0.24222e+03_rb,0.35141e+03_rb,0.45702e+03_rb,0.52057e+03_rb, &
        & 0.49550e+03_rb,0.39372e+03_rb,0.23163e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 5, 2) = (/ &
        & 0.13075e+03_rb,0.23990e+03_rb,0.34934e+03_rb,0.45476e+03_rb,0.51651e+03_rb, &
        & 0.48963e+03_rb,0.38752e+03_rb,0.22836e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 5, 2) = (/ &
        & 0.12847e+03_rb,0.23780e+03_rb,0.34741e+03_rb,0.45266e+03_rb,0.51271e+03_rb, &
        & 0.48438e+03_rb,0.38196e+03_rb,0.22548e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 5, 2) = (/ &
        & 0.12644e+03_rb,0.23588e+03_rb,0.34557e+03_rb,0.45066e+03_rb,0.50911e+03_rb, &
        & 0.47962e+03_rb,0.37696e+03_rb,0.22291e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 5, 2) = (/ &
        & 0.12462e+03_rb,0.23409e+03_rb,0.34381e+03_rb,0.44874e+03_rb,0.50570e+03_rb, &
        & 0.47526e+03_rb,0.37245e+03_rb,0.22060e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 6, 2) = (/ &
        & 0.12722e+03_rb,0.23690e+03_rb,0.34688e+03_rb,0.45177e+03_rb,0.51032e+03_rb, &
        & 0.48144e+03_rb,0.37887e+03_rb,0.22389e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 6, 2) = (/ &
        & 0.12492e+03_rb,0.23485e+03_rb,0.34508e+03_rb,0.44967e+03_rb,0.50620e+03_rb, &
        & 0.47598e+03_rb,0.37319e+03_rb,0.22098e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 6, 2) = (/ &
        & 0.12290e+03_rb,0.23301e+03_rb,0.34340e+03_rb,0.44769e+03_rb,0.50242e+03_rb, &
        & 0.47108e+03_rb,0.36815e+03_rb,0.21843e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 6, 2) = (/ &
        & 0.12111e+03_rb,0.23132e+03_rb,0.34180e+03_rb,0.44582e+03_rb,0.49894e+03_rb, &
        & 0.46668e+03_rb,0.36363e+03_rb,0.21616e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 6, 2) = (/ &
        & 0.11952e+03_rb,0.22976e+03_rb,0.34026e+03_rb,0.44407e+03_rb,0.49574e+03_rb, &
        & 0.46270e+03_rb,0.35956e+03_rb,0.21414e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 7, 2) = (/ &
        & 0.12201e+03_rb,0.23235e+03_rb,0.34300e+03_rb,0.44700e+03_rb,0.50054e+03_rb, &
        & 0.46889e+03_rb,0.36590e+03_rb,0.21730e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 7, 2) = (/ &
        & 0.11997e+03_rb,0.23055e+03_rb,0.34145e+03_rb,0.44492e+03_rb,0.49657e+03_rb, &
        & 0.46383e+03_rb,0.36071e+03_rb,0.21471e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 7, 2) = (/ &
        & 0.11818e+03_rb,0.22894e+03_rb,0.33999e+03_rb,0.44309e+03_rb,0.49292e+03_rb, &
        & 0.45934e+03_rb,0.35613e+03_rb,0.21245e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 7, 2) = (/ &
        & 0.11660e+03_rb,0.22746e+03_rb,0.33859e+03_rb,0.44146e+03_rb,0.48959e+03_rb, &
        & 0.45533e+03_rb,0.35208e+03_rb,0.21045e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 7, 2) = (/ &
        & 0.11520e+03_rb,0.22609e+03_rb,0.33724e+03_rb,0.43996e+03_rb,0.48657e+03_rb, &
        & 0.45175e+03_rb,0.34847e+03_rb,0.20867e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 8, 2) = (/ &
        & 0.11756e+03_rb,0.22844e+03_rb,0.33965e+03_rb,0.44261e+03_rb,0.49147e+03_rb, &
        & 0.45776e+03_rb,0.35454e+03_rb,0.21166e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 8, 2) = (/ &
        & 0.11574e+03_rb,0.22687e+03_rb,0.33830e+03_rb,0.44082e+03_rb,0.48767e+03_rb, &
        & 0.45313e+03_rb,0.34986e+03_rb,0.20936e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 8, 2) = (/ &
        & 0.11415e+03_rb,0.22546e+03_rb,0.33704e+03_rb,0.43915e+03_rb,0.48429e+03_rb, &
        & 0.44908e+03_rb,0.34579e+03_rb,0.20735e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 8, 2) = (/ &
        & 0.11276e+03_rb,0.22417e+03_rb,0.33584e+03_rb,0.43758e+03_rb,0.48126e+03_rb, &
        & 0.44549e+03_rb,0.34220e+03_rb,0.20559e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 8, 2) = (/ &
        & 0.11153e+03_rb,0.22298e+03_rb,0.33466e+03_rb,0.43617e+03_rb,0.47855e+03_rb, &
        & 0.44228e+03_rb,0.33902e+03_rb,0.20403e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 9, 2) = (/ &
        & 0.11376e+03_rb,0.22510e+03_rb,0.33674e+03_rb,0.43882e+03_rb,0.48336e+03_rb, &
        & 0.44807e+03_rb,0.34479e+03_rb,0.20686e+03_rb,0.71840e+02_rb /)
      kao(:, 2, 9, 2) = (/ &
        & 0.11214e+03_rb,0.22372e+03_rb,0.33559e+03_rb,0.43706e+03_rb,0.47989e+03_rb, &
        & 0.44388e+03_rb,0.34060e+03_rb,0.20480e+03_rb,0.71840e+02_rb /)
      kao(:, 3, 9, 2) = (/ &
        & 0.11074e+03_rb,0.22250e+03_rb,0.33451e+03_rb,0.43547e+03_rb,0.47679e+03_rb, &
        & 0.44020e+03_rb,0.33697e+03_rb,0.20303e+03_rb,0.71840e+02_rb /)
      kao(:, 4, 9, 2) = (/ &
        & 0.10951e+03_rb,0.22138e+03_rb,0.33348e+03_rb,0.43404e+03_rb,0.47404e+03_rb, &
        & 0.43694e+03_rb,0.33379e+03_rb,0.20147e+03_rb,0.71840e+02_rb /)
      kao(:, 5, 9, 2) = (/ &
        & 0.10843e+03_rb,0.22034e+03_rb,0.33246e+03_rb,0.43277e+03_rb,0.47159e+03_rb, &
        & 0.43405e+03_rb,0.33096e+03_rb,0.20010e+03_rb,0.71840e+02_rb /)
      kao(:, 1,10, 2) = (/ &
        & 0.11041e+03_rb,0.22214e+03_rb,0.33414e+03_rb,0.43517e+03_rb,0.47604e+03_rb, &
        & 0.43932e+03_rb,0.33611e+03_rb,0.20261e+03_rb,0.71840e+02_rb /)
      kao(:, 2,10, 2) = (/ &
        & 0.10898e+03_rb,0.22095e+03_rb,0.33317e+03_rb,0.43358e+03_rb,0.47283e+03_rb, &
        & 0.43552e+03_rb,0.33239e+03_rb,0.20080e+03_rb,0.71840e+02_rb /)
      kao(:, 3,10, 2) = (/ &
        & 0.10774e+03_rb,0.21989e+03_rb,0.33226e+03_rb,0.43213e+03_rb,0.47002e+03_rb, &
        & 0.43222e+03_rb,0.32917e+03_rb,0.19924e+03_rb,0.71840e+02_rb /)
      kao(:, 4,10, 2) = (/ &
        & 0.10667e+03_rb,0.21893e+03_rb,0.33139e+03_rb,0.43083e+03_rb,0.46753e+03_rb, &
        & 0.42932e+03_rb,0.32634e+03_rb,0.19788e+03_rb,0.71840e+02_rb /)
      kao(:, 5,10, 2) = (/ &
        & 0.10573e+03_rb,0.21803e+03_rb,0.33053e+03_rb,0.42965e+03_rb,0.46532e+03_rb, &
        & 0.42677e+03_rb,0.32386e+03_rb,0.19669e+03_rb,0.71840e+02_rb /)
      kao(:, 1,11, 2) = (/ &
        & 0.10694e+03_rb,0.21911e+03_rb,0.33152e+03_rb,0.43127e+03_rb,0.46816e+03_rb, &
        & 0.43005e+03_rb,0.32706e+03_rb,0.19822e+03_rb,0.71840e+02_rb /)
      kao(:, 2,11, 2) = (/ &
        & 0.10577e+03_rb,0.21815e+03_rb,0.33074e+03_rb,0.42984e+03_rb,0.46541e+03_rb, &
        & 0.42687e+03_rb,0.32396e+03_rb,0.19674e+03_rb,0.71840e+02_rb /)
      kao(:, 3,11, 2) = (/ &
        & 0.10476e+03_rb,0.21728e+03_rb,0.33000e+03_rb,0.42855e+03_rb,0.46301e+03_rb, &
        & 0.42411e+03_rb,0.32130e+03_rb,0.19546e+03_rb,0.71840e+02_rb /)
      kao(:, 4,11, 2) = (/ &
        & 0.10388e+03_rb,0.21649e+03_rb,0.32928e+03_rb,0.42741e+03_rb,0.46089e+03_rb, &
        & 0.42169e+03_rb,0.31897e+03_rb,0.19435e+03_rb,0.71840e+02_rb /)
      kao(:, 5,11, 2) = (/ &
        & 0.10311e+03_rb,0.21575e+03_rb,0.32855e+03_rb,0.42639e+03_rb,0.45902e+03_rb, &
        & 0.41954e+03_rb,0.31692e+03_rb,0.19338e+03_rb,0.71840e+02_rb /)
      kao(:, 1,12, 2) = (/ &
        & 0.10410e+03_rb,0.21663e+03_rb,0.32937e+03_rb,0.42778e+03_rb,0.46141e+03_rb, &
        & 0.42229e+03_rb,0.31955e+03_rb,0.19463e+03_rb,0.71840e+02_rb /)
      kao(:, 2,12, 2) = (/ &
        & 0.10314e+03_rb,0.21585e+03_rb,0.32875e+03_rb,0.42654e+03_rb,0.45909e+03_rb, &
        & 0.41961e+03_rb,0.31700e+03_rb,0.19341e+03_rb,0.71840e+02_rb /)
      kao(:, 3,12, 2) = (/ &
        & 0.10231e+03_rb,0.21515e+03_rb,0.32815e+03_rb,0.42544e+03_rb,0.45707e+03_rb, &
        & 0.41729e+03_rb,0.31480e+03_rb,0.19236e+03_rb,0.71840e+02_rb /)
      kao(:, 4,12, 2) = (/ &
        & 0.10159e+03_rb,0.21449e+03_rb,0.32754e+03_rb,0.42445e+03_rb,0.45532e+03_rb, &
        & 0.41526e+03_rb,0.31288e+03_rb,0.19145e+03_rb,0.71840e+02_rb /)
      kao(:, 5,12, 2) = (/ &
        & 0.10096e+03_rb,0.21387e+03_rb,0.32692e+03_rb,0.42356e+03_rb,0.45377e+03_rb, &
        & 0.41347e+03_rb,0.31119e+03_rb,0.19066e+03_rb,0.71840e+02_rb /)
      kao(:, 1,13, 2) = (/ &
        & 0.10177e+03_rb,0.21460e+03_rb,0.32761e+03_rb,0.42475e+03_rb,0.45575e+03_rb, &
        & 0.41576e+03_rb,0.31335e+03_rb,0.19168e+03_rb,0.71840e+02_rb /)
      kao(:, 2,13, 2) = (/ &
        & 0.10099e+03_rb,0.21397e+03_rb,0.32712e+03_rb,0.42368e+03_rb,0.45383e+03_rb, &
        & 0.41353e+03_rb,0.31125e+03_rb,0.19068e+03_rb,0.71840e+02_rb /)
      kao(:, 3,13, 2) = (/ &
        & 0.10031e+03_rb,0.21340e+03_rb,0.32663e+03_rb,0.42272e+03_rb,0.45216e+03_rb, &
        & 0.41160e+03_rb,0.30943e+03_rb,0.18983e+03_rb,0.71840e+02_rb /)
      kao(:, 4,13, 2) = (/ &
        & 0.99720e+02_rb,0.21286e+03_rb,0.32612e+03_rb,0.42187e+03_rb,0.45071e+03_rb, &
        & 0.40991e+03_rb,0.30785e+03_rb,0.18908e+03_rb,0.71840e+02_rb /)
      kao(:, 5,13, 2) = (/ &
        & 0.99204e+02_rb,0.21233e+03_rb,0.32558e+03_rb,0.42112e+03_rb,0.44942e+03_rb, &
        & 0.40842e+03_rb,0.30646e+03_rb,0.18843e+03_rb,0.71840e+02_rb /)
      kao(:, 1, 1, 3) = (/ &
        & 0.44909e+03_rb,0.47445e+03_rb,0.49981e+03_rb,0.52556e+03_rb,0.59352e+03_rb, &
        & 0.63011e+03_rb,0.50440e+03_rb,0.31793e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 1, 3) = (/ &
        & 0.43771e+03_rb,0.46449e+03_rb,0.49128e+03_rb,0.51852e+03_rb,0.58982e+03_rb, &
        & 0.62155e+03_rb,0.49364e+03_rb,0.31383e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 1, 3) = (/ &
        & 0.42749e+03_rb,0.45555e+03_rb,0.48362e+03_rb,0.51220e+03_rb,0.58623e+03_rb, &
        & 0.61324e+03_rb,0.48379e+03_rb,0.31014e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 1, 3) = (/ &
        & 0.41826e+03_rb,0.44748e+03_rb,0.47670e+03_rb,0.50650e+03_rb,0.58281e+03_rb, &
        & 0.60555e+03_rb,0.47483e+03_rb,0.30681e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 1, 3) = (/ &
        & 0.40989e+03_rb,0.44015e+03_rb,0.47042e+03_rb,0.50132e+03_rb,0.57944e+03_rb, &
        & 0.59846e+03_rb,0.46657e+03_rb,0.30379e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 2, 3) = (/ &
        & 0.41487e+03_rb,0.44451e+03_rb,0.47415e+03_rb,0.50447e+03_rb,0.58410e+03_rb, &
        & 0.60269e+03_rb,0.47150e+03_rb,0.30559e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 2, 3) = (/ &
        & 0.40511e+03_rb,0.43597e+03_rb,0.46683e+03_rb,0.49854e+03_rb,0.58056e+03_rb, &
        & 0.59434e+03_rb,0.46181e+03_rb,0.30207e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 2, 3) = (/ &
        & 0.39637e+03_rb,0.42832e+03_rb,0.46028e+03_rb,0.49326e+03_rb,0.57701e+03_rb, &
        & 0.58648e+03_rb,0.45298e+03_rb,0.29892e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 2, 3) = (/ &
        & 0.38850e+03_rb,0.42143e+03_rb,0.45437e+03_rb,0.48855e+03_rb,0.57350e+03_rb, &
        & 0.57914e+03_rb,0.44500e+03_rb,0.29608e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 2, 3) = (/ &
        & 0.38136e+03_rb,0.41519e+03_rb,0.44902e+03_rb,0.48433e+03_rb,0.57004e+03_rb, &
        & 0.57245e+03_rb,0.43774e+03_rb,0.29351e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 3, 3) = (/ &
        & 0.38797e+03_rb,0.42097e+03_rb,0.45397e+03_rb,0.48897e+03_rb,0.57508e+03_rb, &
        & 0.57865e+03_rb,0.44446e+03_rb,0.29589e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 3, 3) = (/ &
        & 0.37940e+03_rb,0.41348e+03_rb,0.44755e+03_rb,0.48416e+03_rb,0.57138e+03_rb, &
        & 0.57059e+03_rb,0.43573e+03_rb,0.29280e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 3, 3) = (/ &
        & 0.37176e+03_rb,0.40679e+03_rb,0.44182e+03_rb,0.47985e+03_rb,0.56781e+03_rb, &
        & 0.56324e+03_rb,0.42791e+03_rb,0.29005e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 3, 3) = (/ &
        & 0.36490e+03_rb,0.40078e+03_rb,0.43667e+03_rb,0.47599e+03_rb,0.56436e+03_rb, &
        & 0.55656e+03_rb,0.42088e+03_rb,0.28757e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 3, 3) = (/ &
        & 0.35870e+03_rb,0.39536e+03_rb,0.43202e+03_rb,0.47253e+03_rb,0.56109e+03_rb, &
        & 0.55041e+03_rb,0.41448e+03_rb,0.28534e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 4, 3) = (/ &
        & 0.36565e+03_rb,0.40144e+03_rb,0.43723e+03_rb,0.47759e+03_rb,0.56581e+03_rb, &
        & 0.55729e+03_rb,0.42164e+03_rb,0.28784e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 4, 3) = (/ &
        & 0.35808e+03_rb,0.39482e+03_rb,0.43155e+03_rb,0.47386e+03_rb,0.56215e+03_rb, &
        & 0.54978e+03_rb,0.41381e+03_rb,0.28511e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 4, 3) = (/ &
        & 0.35135e+03_rb,0.38893e+03_rb,0.42651e+03_rb,0.47052e+03_rb,0.55875e+03_rb, &
        & 0.54295e+03_rb,0.40683e+03_rb,0.28269e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 4, 3) = (/ &
        & 0.34533e+03_rb,0.38366e+03_rb,0.42199e+03_rb,0.46746e+03_rb,0.55555e+03_rb, &
        & 0.53669e+03_rb,0.40055e+03_rb,0.28052e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 4, 3) = (/ &
        & 0.33991e+03_rb,0.37892e+03_rb,0.41793e+03_rb,0.46467e+03_rb,0.55244e+03_rb, &
        & 0.53093e+03_rb,0.39488e+03_rb,0.27856e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 5, 3) = (/ &
        & 0.34669e+03_rb,0.38485e+03_rb,0.42301e+03_rb,0.46943e+03_rb,0.55667e+03_rb, &
        & 0.53812e+03_rb,0.40197e+03_rb,0.28101e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 5, 3) = (/ &
        & 0.33999e+03_rb,0.37899e+03_rb,0.41799e+03_rb,0.46662e+03_rb,0.55311e+03_rb, &
        & 0.53101e+03_rb,0.39496e+03_rb,0.27859e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 5, 3) = (/ &
        & 0.33406e+03_rb,0.37380e+03_rb,0.41354e+03_rb,0.46416e+03_rb,0.54976e+03_rb, &
        & 0.52459e+03_rb,0.38873e+03_rb,0.27645e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 5, 3) = (/ &
        & 0.32878e+03_rb,0.36918e+03_rb,0.40958e+03_rb,0.46185e+03_rb,0.54671e+03_rb, &
        & 0.51881e+03_rb,0.38326e+03_rb,0.27455e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 5, 3) = (/ &
        & 0.32404e+03_rb,0.36503e+03_rb,0.40603e+03_rb,0.45969e+03_rb,0.54395e+03_rb, &
        & 0.51356e+03_rb,0.37844e+03_rb,0.27284e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 6, 3) = (/ &
        & 0.33079e+03_rb,0.37094e+03_rb,0.41109e+03_rb,0.46395e+03_rb,0.54790e+03_rb, &
        & 0.52101e+03_rb,0.38534e+03_rb,0.27527e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 6, 3) = (/ &
        & 0.32482e+03_rb,0.36572e+03_rb,0.40662e+03_rb,0.46188e+03_rb,0.54448e+03_rb, &
        & 0.51443e+03_rb,0.37923e+03_rb,0.27312e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 6, 3) = (/ &
        & 0.31958e+03_rb,0.36113e+03_rb,0.40268e+03_rb,0.45998e+03_rb,0.54143e+03_rb, &
        & 0.50857e+03_rb,0.37397e+03_rb,0.27123e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 6, 3) = (/ &
        & 0.31492e+03_rb,0.35705e+03_rb,0.39919e+03_rb,0.45816e+03_rb,0.53866e+03_rb, &
        & 0.50335e+03_rb,0.36939e+03_rb,0.26955e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 6, 3) = (/ &
        & 0.31076e+03_rb,0.35342e+03_rb,0.39607e+03_rb,0.45638e+03_rb,0.53611e+03_rb, &
        & 0.49868e+03_rb,0.36537e+03_rb,0.26805e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 7, 3) = (/ &
        & 0.31725e+03_rb,0.35910e+03_rb,0.40094e+03_rb,0.45992e+03_rb,0.53978e+03_rb, &
        & 0.50597e+03_rb,0.37168e+03_rb,0.27039e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 7, 3) = (/ &
        & 0.31194e+03_rb,0.35445e+03_rb,0.39695e+03_rb,0.45835e+03_rb,0.53664e+03_rb, &
        & 0.50000e+03_rb,0.36650e+03_rb,0.26848e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 7, 3) = (/ &
        & 0.30729e+03_rb,0.35038e+03_rb,0.39347e+03_rb,0.45686e+03_rb,0.53383e+03_rb, &
        & 0.49465e+03_rb,0.36205e+03_rb,0.26680e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 7, 3) = (/ &
        & 0.30318e+03_rb,0.34679e+03_rb,0.39040e+03_rb,0.45536e+03_rb,0.53125e+03_rb, &
        & 0.48979e+03_rb,0.35818e+03_rb,0.26532e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 7, 3) = (/ &
        & 0.29953e+03_rb,0.34359e+03_rb,0.38767e+03_rb,0.45383e+03_rb,0.52882e+03_rb, &
        & 0.48543e+03_rb,0.35479e+03_rb,0.26401e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 8, 3) = (/ &
        & 0.30567e+03_rb,0.34896e+03_rb,0.39226e+03_rb,0.45673e+03_rb,0.53243e+03_rb, &
        & 0.49274e+03_rb,0.36052e+03_rb,0.26622e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 8, 3) = (/ &
        & 0.30094e+03_rb,0.34482e+03_rb,0.38872e+03_rb,0.45541e+03_rb,0.52945e+03_rb, &
        & 0.48711e+03_rb,0.35609e+03_rb,0.26451e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 8, 3) = (/ &
        & 0.29682e+03_rb,0.34122e+03_rb,0.38564e+03_rb,0.45413e+03_rb,0.52674e+03_rb, &
        & 0.48217e+03_rb,0.35231e+03_rb,0.26303e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 8, 3) = (/ &
        & 0.29320e+03_rb,0.33805e+03_rb,0.38294e+03_rb,0.45285e+03_rb,0.52426e+03_rb, &
        & 0.47779e+03_rb,0.34904e+03_rb,0.26172e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 8, 3) = (/ &
        & 0.29000e+03_rb,0.33525e+03_rb,0.38055e+03_rb,0.45152e+03_rb,0.52191e+03_rb, &
        & 0.47388e+03_rb,0.34618e+03_rb,0.26057e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 9, 3) = (/ &
        & 0.29581e+03_rb,0.34033e+03_rb,0.38489e+03_rb,0.45387e+03_rb,0.52555e+03_rb, &
        & 0.48095e+03_rb,0.35139e+03_rb,0.26266e+03_rb,0.17829e+03_rb /)
      kao(:, 2, 9, 3) = (/ &
        & 0.29159e+03_rb,0.33664e+03_rb,0.38173e+03_rb,0.45277e+03_rb,0.52268e+03_rb, &
        & 0.47582e+03_rb,0.34759e+03_rb,0.26114e+03_rb,0.17829e+03_rb /)
      kao(:, 3, 9, 3) = (/ &
        & 0.28794e+03_rb,0.33344e+03_rb,0.37901e+03_rb,0.45167e+03_rb,0.52008e+03_rb, &
        & 0.47135e+03_rb,0.34436e+03_rb,0.25982e+03_rb,0.17829e+03_rb /)
      kao(:, 4, 9, 3) = (/ &
        & 0.28475e+03_rb,0.33065e+03_rb,0.37663e+03_rb,0.45055e+03_rb,0.51765e+03_rb, &
        & 0.46738e+03_rb,0.34157e+03_rb,0.25867e+03_rb,0.17829e+03_rb /)
      kao(:, 5, 9, 3) = (/ &
        & 0.28193e+03_rb,0.32819e+03_rb,0.37453e+03_rb,0.44937e+03_rb,0.51538e+03_rb, &
        & 0.46382e+03_rb,0.33916e+03_rb,0.25766e+03_rb,0.17829e+03_rb /)
      kao(:, 1,10, 3) = (/ &
        & 0.28708e+03_rb,0.33269e+03_rb,0.37836e+03_rb,0.45121e+03_rb,0.51885e+03_rb, &
        & 0.47029e+03_rb,0.34361e+03_rb,0.25951e+03_rb,0.17829e+03_rb /)
      kao(:, 2,10, 3) = (/ &
        & 0.28336e+03_rb,0.32944e+03_rb,0.37559e+03_rb,0.45027e+03_rb,0.51611e+03_rb, &
        & 0.46563e+03_rb,0.34037e+03_rb,0.25817e+03_rb,0.17829e+03_rb /)
      kao(:, 3,10, 3) = (/ &
        & 0.28016e+03_rb,0.32664e+03_rb,0.37320e+03_rb,0.44932e+03_rb,0.51361e+03_rb, &
        & 0.46153e+03_rb,0.33765e+03_rb,0.25702e+03_rb,0.17829e+03_rb /)
      kao(:, 4,10, 3) = (/ &
        & 0.27737e+03_rb,0.32420e+03_rb,0.37112e+03_rb,0.44835e+03_rb,0.51133e+03_rb, &
        & 0.45793e+03_rb,0.33532e+03_rb,0.25601e+03_rb,0.17829e+03_rb /)
      kao(:, 5,10, 3) = (/ &
        & 0.27492e+03_rb,0.32206e+03_rb,0.36930e+03_rb,0.44735e+03_rb,0.50922e+03_rb, &
        & 0.45473e+03_rb,0.33330e+03_rb,0.25513e+03_rb,0.17829e+03_rb /)
      kao(:, 1,11, 3) = (/ &
        & 0.27807e+03_rb,0.32481e+03_rb,0.37164e+03_rb,0.44842e+03_rb,0.51141e+03_rb, &
        & 0.45884e+03_rb,0.33590e+03_rb,0.25627e+03_rb,0.17829e+03_rb /)
      kao(:, 2,11, 3) = (/ &
        & 0.27502e+03_rb,0.32214e+03_rb,0.36937e+03_rb,0.44764e+03_rb,0.50894e+03_rb, &
        & 0.45486e+03_rb,0.33338e+03_rb,0.25517e+03_rb,0.17829e+03_rb /)
      kao(:, 3,11, 3) = (/ &
        & 0.27240e+03_rb,0.31985e+03_rb,0.36742e+03_rb,0.44685e+03_rb,0.50671e+03_rb, &
        & 0.45139e+03_rb,0.33123e+03_rb,0.25422e+03_rb,0.17829e+03_rb /)
      kao(:, 4,11, 3) = (/ &
        & 0.27012e+03_rb,0.31785e+03_rb,0.36572e+03_rb,0.44602e+03_rb,0.50464e+03_rb, &
        & 0.44836e+03_rb,0.32939e+03_rb,0.25340e+03_rb,0.17829e+03_rb /)
      kao(:, 5,11, 3) = (/ &
        & 0.26811e+03_rb,0.31610e+03_rb,0.36422e+03_rb,0.44514e+03_rb,0.50273e+03_rb, &
        & 0.44569e+03_rb,0.32779e+03_rb,0.25268e+03_rb,0.17829e+03_rb /)
      kao(:, 1,12, 3) = (/ &
        & 0.27068e+03_rb,0.31834e+03_rb,0.36613e+03_rb,0.44605e+03_rb,0.50476e+03_rb, &
        & 0.44911e+03_rb,0.32983e+03_rb,0.25360e+03_rb,0.17829e+03_rb /)
      kao(:, 2,12, 3) = (/ &
        & 0.26818e+03_rb,0.31616e+03_rb,0.36428e+03_rb,0.44539e+03_rb,0.50255e+03_rb, &
        & 0.44579e+03_rb,0.32784e+03_rb,0.25270e+03_rb,0.17829e+03_rb /)
      kao(:, 3,12, 3) = (/ &
        & 0.26603e+03_rb,0.31428e+03_rb,0.36268e+03_rb,0.44471e+03_rb,0.50054e+03_rb, &
        & 0.44293e+03_rb,0.32614e+03_rb,0.25193e+03_rb,0.17829e+03_rb /)
      kao(:, 4,12, 3) = (/ &
        & 0.26417e+03_rb,0.31264e+03_rb,0.36129e+03_rb,0.44399e+03_rb,0.49874e+03_rb, &
        & 0.44043e+03_rb,0.32468e+03_rb,0.25125e+03_rb,0.17829e+03_rb /)
      kao(:, 5,12, 3) = (/ &
        & 0.26252e+03_rb,0.31121e+03_rb,0.36005e+03_rb,0.44322e+03_rb,0.49710e+03_rb, &
        & 0.43824e+03_rb,0.32341e+03_rb,0.25066e+03_rb,0.17829e+03_rb /)
      kao(:, 1,13, 3) = (/ &
        & 0.26463e+03_rb,0.31305e+03_rb,0.36163e+03_rb,0.44400e+03_rb,0.49889e+03_rb, &
        & 0.44105e+03_rb,0.32504e+03_rb,0.25142e+03_rb,0.17829e+03_rb /)
      kao(:, 2,13, 3) = (/ &
        & 0.26258e+03_rb,0.31126e+03_rb,0.36010e+03_rb,0.44348e+03_rb,0.49698e+03_rb, &
        & 0.43831e+03_rb,0.32345e+03_rb,0.25068e+03_rb,0.17829e+03_rb /)
      kao(:, 3,13, 3) = (/ &
        & 0.26082e+03_rb,0.30972e+03_rb,0.35880e+03_rb,0.44291e+03_rb,0.49519e+03_rb, &
        & 0.43595e+03_rb,0.32210e+03_rb,0.25005e+03_rb,0.17829e+03_rb /)
      kao(:, 4,13, 3) = (/ &
        & 0.25929e+03_rb,0.30838e+03_rb,0.35766e+03_rb,0.44229e+03_rb,0.49360e+03_rb, &
        & 0.43388e+03_rb,0.32093e+03_rb,0.24950e+03_rb,0.17829e+03_rb /)
      kao(:, 5,13, 3) = (/ &
        & 0.25795e+03_rb,0.30721e+03_rb,0.35666e+03_rb,0.44160e+03_rb,0.49217e+03_rb, &
        & 0.43204e+03_rb,0.31991e+03_rb,0.24901e+03_rb,0.17829e+03_rb /)
      kao(:, 1, 1, 4) = (/ &
        & 0.86913e+03_rb,0.80389e+03_rb,0.73864e+03_rb,0.67339e+03_rb,0.62239e+03_rb, &
        & 0.67741e+03_rb,0.51596e+03_rb,0.43641e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 1, 4) = (/ &
        & 0.84711e+03_rb,0.78462e+03_rb,0.72212e+03_rb,0.65963e+03_rb,0.61530e+03_rb, &
        & 0.66963e+03_rb,0.50564e+03_rb,0.43387e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 1, 4) = (/ &
        & 0.82734e+03_rb,0.76731e+03_rb,0.70729e+03_rb,0.64727e+03_rb,0.60932e+03_rb, &
        & 0.66251e+03_rb,0.49688e+03_rb,0.43160e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 1, 4) = (/ &
        & 0.80948e+03_rb,0.75169e+03_rb,0.69390e+03_rb,0.63611e+03_rb,0.60420e+03_rb, &
        & 0.65531e+03_rb,0.48930e+03_rb,0.42954e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 1, 4) = (/ &
        & 0.79327e+03_rb,0.73751e+03_rb,0.68174e+03_rb,0.62598e+03_rb,0.59980e+03_rb, &
        & 0.64776e+03_rb,0.48275e+03_rb,0.42767e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 2, 4) = (/ &
        & 0.80291e+03_rb,0.74594e+03_rb,0.68897e+03_rb,0.63200e+03_rb,0.60547e+03_rb, &
        & 0.65273e+03_rb,0.48660e+03_rb,0.42878e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 2, 4) = (/ &
        & 0.78403e+03_rb,0.72942e+03_rb,0.67481e+03_rb,0.62020e+03_rb,0.60133e+03_rb, &
        & 0.64384e+03_rb,0.47915e+03_rb,0.42661e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 2, 4) = (/ &
        & 0.76711e+03_rb,0.71462e+03_rb,0.66212e+03_rb,0.60962e+03_rb,0.59788e+03_rb, &
        & 0.63506e+03_rb,0.47284e+03_rb,0.42466e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 2, 4) = (/ &
        & 0.75187e+03_rb,0.70128e+03_rb,0.65069e+03_rb,0.60010e+03_rb,0.59472e+03_rb, &
        & 0.62679e+03_rb,0.46742e+03_rb,0.42291e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 2, 4) = (/ &
        & 0.73807e+03_rb,0.68920e+03_rb,0.64034e+03_rb,0.59147e+03_rb,0.59165e+03_rb, &
        & 0.61864e+03_rb,0.46272e+03_rb,0.42132e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 3, 4) = (/ &
        & 0.75084e+03_rb,0.70038e+03_rb,0.64992e+03_rb,0.59946e+03_rb,0.59732e+03_rb, &
        & 0.62621e+03_rb,0.46707e+03_rb,0.42279e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 3, 4) = (/ &
        & 0.73427e+03_rb,0.68588e+03_rb,0.63749e+03_rb,0.58910e+03_rb,0.59435e+03_rb, &
        & 0.61627e+03_rb,0.46147e+03_rb,0.42088e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 3, 4) = (/ &
        & 0.71948e+03_rb,0.67294e+03_rb,0.62640e+03_rb,0.57986e+03_rb,0.59145e+03_rb, &
        & 0.60690e+03_rb,0.45675e+03_rb,0.41918e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 3, 4) = (/ &
        & 0.70620e+03_rb,0.66132e+03_rb,0.61644e+03_rb,0.57156e+03_rb,0.58863e+03_rb, &
        & 0.59842e+03_rb,0.45269e+03_rb,0.41765e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 3, 4) = (/ &
        & 0.69421e+03_rb,0.65083e+03_rb,0.60744e+03_rb,0.56406e+03_rb,0.58580e+03_rb, &
        & 0.59032e+03_rb,0.44923e+03_rb,0.41627e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 4, 4) = (/ &
        & 0.70765e+03_rb,0.66259e+03_rb,0.61753e+03_rb,0.57246e+03_rb,0.59114e+03_rb, &
        & 0.59936e+03_rb,0.45313e+03_rb,0.41781e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 4, 4) = (/ &
        & 0.69300e+03_rb,0.64977e+03_rb,0.60654e+03_rb,0.56330e+03_rb,0.58834e+03_rb, &
        & 0.58949e+03_rb,0.44892e+03_rb,0.41613e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 4, 4) = (/ &
        & 0.67997e+03_rb,0.63837e+03_rb,0.59677e+03_rb,0.55516e+03_rb,0.58563e+03_rb, &
        & 0.58057e+03_rb,0.44538e+03_rb,0.41463e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 4, 4) = (/ &
        & 0.66833e+03_rb,0.62818e+03_rb,0.58803e+03_rb,0.54788e+03_rb,0.58292e+03_rb, &
        & 0.57257e+03_rb,0.44238e+03_rb,0.41329e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 4, 4) = (/ &
        & 0.65784e+03_rb,0.61901e+03_rb,0.58017e+03_rb,0.54133e+03_rb,0.58031e+03_rb, &
        & 0.56522e+03_rb,0.43982e+03_rb,0.41208e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 5, 4) = (/ &
        & 0.67096e+03_rb,0.63048e+03_rb,0.59001e+03_rb,0.54953e+03_rb,0.58521e+03_rb, &
        & 0.57438e+03_rb,0.44304e+03_rb,0.41359e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 5, 4) = (/ &
        & 0.65799e+03_rb,0.61913e+03_rb,0.58028e+03_rb,0.54142e+03_rb,0.58256e+03_rb, &
        & 0.56532e+03_rb,0.43984e+03_rb,0.41210e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 5, 4) = (/ &
        & 0.64651e+03_rb,0.60909e+03_rb,0.57167e+03_rb,0.53427e+03_rb,0.57997e+03_rb, &
        & 0.55705e+03_rb,0.43718e+03_rb,0.41077e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 5, 4) = (/ &
        & 0.63629e+03_rb,0.60015e+03_rb,0.56401e+03_rb,0.52793e+03_rb,0.57742e+03_rb, &
        & 0.54959e+03_rb,0.43482e+03_rb,0.40960e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 5, 4) = (/ &
        & 0.62713e+03_rb,0.59213e+03_rb,0.55713e+03_rb,0.52227e+03_rb,0.57480e+03_rb, &
        & 0.54282e+03_rb,0.43271e+03_rb,0.40854e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 6, 4) = (/ &
        & 0.64019e+03_rb,0.60356e+03_rb,0.56693e+03_rb,0.53034e+03_rb,0.57955e+03_rb, &
        & 0.55244e+03_rb,0.43572e+03_rb,0.41005e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 6, 4) = (/ &
        & 0.62864e+03_rb,0.59346e+03_rb,0.55827e+03_rb,0.52321e+03_rb,0.57700e+03_rb, &
        & 0.54395e+03_rb,0.43306e+03_rb,0.40872e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 6, 4) = (/ &
        & 0.61849e+03_rb,0.58457e+03_rb,0.55065e+03_rb,0.51703e+03_rb,0.57439e+03_rb, &
        & 0.53634e+03_rb,0.43072e+03_rb,0.40755e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 6, 4) = (/ &
        & 0.60948e+03_rb,0.57669e+03_rb,0.54390e+03_rb,0.51158e+03_rb,0.57170e+03_rb, &
        & 0.52946e+03_rb,0.42865e+03_rb,0.40651e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 6, 4) = (/ &
        & 0.60143e+03_rb,0.56965e+03_rb,0.53786e+03_rb,0.50675e+03_rb,0.56897e+03_rb, &
        & 0.52321e+03_rb,0.42679e+03_rb,0.40558e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 7, 4) = (/ &
        & 0.61399e+03_rb,0.58064e+03_rb,0.54728e+03_rb,0.51440e+03_rb,0.57380e+03_rb, &
        & 0.53292e+03_rb,0.42969e+03_rb,0.40703e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 7, 4) = (/ &
        & 0.60371e+03_rb,0.57164e+03_rb,0.53957e+03_rb,0.50828e+03_rb,0.57103e+03_rb, &
        & 0.52498e+03_rb,0.42732e+03_rb,0.40585e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 7, 4) = (/ &
        & 0.59471e+03_rb,0.56376e+03_rb,0.53282e+03_rb,0.50302e+03_rb,0.56836e+03_rb, &
        & 0.51802e+03_rb,0.42525e+03_rb,0.40481e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 7, 4) = (/ &
        & 0.58676e+03_rb,0.55681e+03_rb,0.52686e+03_rb,0.49842e+03_rb,0.56580e+03_rb, &
        & 0.51197e+03_rb,0.42342e+03_rb,0.40389e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 7, 4) = (/ &
        & 0.57970e+03_rb,0.55063e+03_rb,0.52156e+03_rb,0.49437e+03_rb,0.56333e+03_rb, &
        & 0.50656e+03_rb,0.42179e+03_rb,0.40308e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 8, 4) = (/ &
        & 0.59158e+03_rb,0.56102e+03_rb,0.53047e+03_rb,0.50136e+03_rb,0.56762e+03_rb, &
        & 0.51564e+03_rb,0.42452e+03_rb,0.40445e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 8, 4) = (/ &
        & 0.58242e+03_rb,0.55301e+03_rb,0.52360e+03_rb,0.49623e+03_rb,0.56499e+03_rb, &
        & 0.50866e+03_rb,0.42242e+03_rb,0.40339e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 8, 4) = (/ &
        & 0.57445e+03_rb,0.54604e+03_rb,0.51762e+03_rb,0.49188e+03_rb,0.56245e+03_rb, &
        & 0.50238e+03_rb,0.42058e+03_rb,0.40248e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 8, 4) = (/ &
        & 0.56745e+03_rb,0.53991e+03_rb,0.51237e+03_rb,0.48811e+03_rb,0.55998e+03_rb, &
        & 0.49675e+03_rb,0.41897e+03_rb,0.40167e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 8, 4) = (/ &
        & 0.56125e+03_rb,0.53448e+03_rb,0.50772e+03_rb,0.48477e+03_rb,0.55757e+03_rb, &
        & 0.49178e+03_rb,0.41754e+03_rb,0.40096e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 9, 4) = (/ &
        & 0.57249e+03_rb,0.54432e+03_rb,0.51615e+03_rb,0.49103e+03_rb,0.56159e+03_rb, &
        & 0.50080e+03_rb,0.42013e+03_rb,0.40225e+03_rb,0.38437e+03_rb /)
      kao(:, 2, 9, 4) = (/ &
        & 0.56432e+03_rb,0.53718e+03_rb,0.51003e+03_rb,0.48678e+03_rb,0.55899e+03_rb, &
        & 0.49424e+03_rb,0.41825e+03_rb,0.40131e+03_rb,0.38437e+03_rb /)
      kao(:, 3, 9, 4) = (/ &
        & 0.55726e+03_rb,0.53099e+03_rb,0.50473e+03_rb,0.48321e+03_rb,0.55650e+03_rb, &
        & 0.48858e+03_rb,0.41662e+03_rb,0.40050e+03_rb,0.38437e+03_rb /)
      kao(:, 4, 9, 4) = (/ &
        & 0.55108e+03_rb,0.52559e+03_rb,0.50010e+03_rb,0.48011e+03_rb,0.55416e+03_rb, &
        & 0.48367e+03_rb,0.41520e+03_rb,0.39978e+03_rb,0.38437e+03_rb /)
      kao(:, 5, 9, 4) = (/ &
        & 0.54564e+03_rb,0.52083e+03_rb,0.49601e+03_rb,0.47737e+03_rb,0.55194e+03_rb, &
        & 0.47938e+03_rb,0.41394e+03_rb,0.39916e+03_rb,0.38437e+03_rb /)
      kao(:, 1,10, 4) = (/ &
        & 0.55559e+03_rb,0.52954e+03_rb,0.50348e+03_rb,0.48256e+03_rb,0.55544e+03_rb, &
        & 0.48725e+03_rb,0.41624e+03_rb,0.40030e+03_rb,0.38437e+03_rb /)
      kao(:, 2,10, 4) = (/ &
        & 0.54839e+03_rb,0.52324e+03_rb,0.49808e+03_rb,0.47910e+03_rb,0.55295e+03_rb, &
        & 0.48155e+03_rb,0.41458e+03_rb,0.39948e+03_rb,0.38437e+03_rb /)
      kao(:, 3,10, 4) = (/ &
        & 0.54220e+03_rb,0.51781e+03_rb,0.49343e+03_rb,0.47621e+03_rb,0.55067e+03_rb, &
        & 0.47667e+03_rb,0.41315e+03_rb,0.39876e+03_rb,0.38437e+03_rb /)
      kao(:, 4,10, 4) = (/ &
        & 0.53680e+03_rb,0.51310e+03_rb,0.48939e+03_rb,0.47371e+03_rb,0.54855e+03_rb, &
        & 0.47238e+03_rb,0.41191e+03_rb,0.39814e+03_rb,0.38437e+03_rb /)
      kao(:, 5,10, 4) = (/ &
        & 0.53207e+03_rb,0.50895e+03_rb,0.48584e+03_rb,0.47148e+03_rb,0.54655e+03_rb, &
        & 0.46857e+03_rb,0.41082e+03_rb,0.39760e+03_rb,0.38437e+03_rb /)
      kao(:, 1,11, 4) = (/ &
        & 0.53816e+03_rb,0.51428e+03_rb,0.49041e+03_rb,0.47455e+03_rb,0.54857e+03_rb, &
        & 0.47346e+03_rb,0.41222e+03_rb,0.39830e+03_rb,0.38437e+03_rb /)
      kao(:, 2,11, 4) = (/ &
        & 0.53226e+03_rb,0.50912e+03_rb,0.48598e+03_rb,0.47200e+03_rb,0.54640e+03_rb, &
        & 0.46874e+03_rb,0.41086e+03_rb,0.39762e+03_rb,0.38437e+03_rb /)
      kao(:, 3,11, 4) = (/ &
        & 0.52718e+03_rb,0.50468e+03_rb,0.48217e+03_rb,0.46982e+03_rb,0.54436e+03_rb, &
        & 0.46486e+03_rb,0.40969e+03_rb,0.39703e+03_rb,0.38437e+03_rb /)
      kao(:, 4,11, 4) = (/ &
        & 0.52277e+03_rb,0.50081e+03_rb,0.47886e+03_rb,0.46790e+03_rb,0.54248e+03_rb, &
        & 0.46109e+03_rb,0.40868e+03_rb,0.39652e+03_rb,0.38437e+03_rb /)
      kao(:, 5,11, 4) = (/ &
        & 0.51889e+03_rb,0.49742e+03_rb,0.47595e+03_rb,0.46618e+03_rb,0.54076e+03_rb, &
        & 0.45793e+03_rb,0.40778e+03_rb,0.39608e+03_rb,0.38437e+03_rb /)
      kao(:, 1,12, 4) = (/ &
        & 0.52385e+03_rb,0.50177e+03_rb,0.47968e+03_rb,0.46863e+03_rb,0.54244e+03_rb, &
        & 0.46202e+03_rb,0.40893e+03_rb,0.39665e+03_rb,0.38437e+03_rb /)
      kao(:, 2,12, 4) = (/ &
        & 0.51902e+03_rb,0.49754e+03_rb,0.47606e+03_rb,0.46673e+03_rb,0.54050e+03_rb, &
        & 0.45805e+03_rb,0.40782e+03_rb,0.39609e+03_rb,0.38437e+03_rb /)
      kao(:, 3,12, 4) = (/ &
        & 0.51487e+03_rb,0.49390e+03_rb,0.47294e+03_rb,0.46509e+03_rb,0.53868e+03_rb, &
        & 0.45465e+03_rb,0.40686e+03_rb,0.39561e+03_rb,0.38437e+03_rb /)
      kao(:, 4,12, 4) = (/ &
        & 0.51125e+03_rb,0.49074e+03_rb,0.47022e+03_rb,0.46362e+03_rb,0.53703e+03_rb, &
        & 0.45171e+03_rb,0.40602e+03_rb,0.39520e+03_rb,0.38437e+03_rb /)
      kao(:, 5,12, 4) = (/ &
        & 0.50807e+03_rb,0.48796e+03_rb,0.46784e+03_rb,0.46227e+03_rb,0.53558e+03_rb, &
        & 0.44916e+03_rb,0.40529e+03_rb,0.39483e+03_rb,0.38437e+03_rb /)
      kao(:, 1,13, 4) = (/ &
        & 0.51214e+03_rb,0.49152e+03_rb,0.47089e+03_rb,0.46425e+03_rb,0.53698e+03_rb, &
        & 0.45243e+03_rb,0.40623e+03_rb,0.39530e+03_rb,0.38437e+03_rb /)
      kao(:, 2,13, 4) = (/ &
        & 0.50819e+03_rb,0.48806e+03_rb,0.46793e+03_rb,0.46281e+03_rb,0.53522e+03_rb, &
        & 0.44925e+03_rb,0.40532e+03_rb,0.39485e+03_rb,0.38437e+03_rb /)
      kao(:, 3,13, 4) = (/ &
        & 0.50478e+03_rb,0.48508e+03_rb,0.46537e+03_rb,0.46157e+03_rb,0.53378e+03_rb, &
        & 0.44654e+03_rb,0.40453e+03_rb,0.39445e+03_rb,0.38437e+03_rb /)
      kao(:, 4,13, 4) = (/ &
        & 0.50182e+03_rb,0.48249e+03_rb,0.46315e+03_rb,0.46044e+03_rb,0.53251e+03_rb, &
        & 0.44422e+03_rb,0.40385e+03_rb,0.39411e+03_rb,0.38437e+03_rb /)
      kao(:, 5,13, 4) = (/ &
        & 0.49922e+03_rb,0.48021e+03_rb,0.46120e+03_rb,0.45936e+03_rb,0.53137e+03_rb, &
        & 0.44221e+03_rb,0.40325e+03_rb,0.39381e+03_rb,0.38437e+03_rb /)
      kao(:, 1, 1, 5) = (/ &
        & 0.13243e+04_rb,0.11799e+04_rb,0.10354e+04_rb,0.89093e+03_rb,0.74647e+03_rb, &
        & 0.71044e+03_rb,0.61021e+03_rb,0.64069e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 1, 5) = (/ &
        & 0.12908e+04_rb,0.11505e+04_rb,0.10102e+04_rb,0.86996e+03_rb,0.72969e+03_rb, &
        & 0.70292e+03_rb,0.60750e+03_rb,0.63934e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 1, 5) = (/ &
        & 0.12606e+04_rb,0.11241e+04_rb,0.98763e+03_rb,0.85113e+03_rb,0.71463e+03_rb, &
        & 0.69451e+03_rb,0.60507e+03_rb,0.63812e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 1, 5) = (/ &
        & 0.12334e+04_rb,0.11003e+04_rb,0.96722e+03_rb,0.83412e+03_rb,0.70102e+03_rb, &
        & 0.68564e+03_rb,0.60287e+03_rb,0.63703e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 1, 5) = (/ &
        & 0.12087e+04_rb,0.10787e+04_rb,0.94870e+03_rb,0.81869e+03_rb,0.68867e+03_rb, &
        & 0.67724e+03_rb,0.60088e+03_rb,0.63603e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 2, 5) = (/ &
        & 0.12234e+04_rb,0.10916e+04_rb,0.95972e+03_rb,0.82787e+03_rb,0.69602e+03_rb, &
        & 0.68345e+03_rb,0.60207e+03_rb,0.63662e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 2, 5) = (/ &
        & 0.11946e+04_rb,0.10664e+04_rb,0.93814e+03_rb,0.80988e+03_rb,0.68163e+03_rb, &
        & 0.67225e+03_rb,0.59975e+03_rb,0.63546e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 2, 5) = (/ &
        & 0.11689e+04_rb,0.10438e+04_rb,0.91881e+03_rb,0.79377e+03_rb,0.66882e+03_rb, &
        & 0.66165e+03_rb,0.59767e+03_rb,0.63442e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 2, 5) = (/ &
        & 0.11456e+04_rb,0.10235e+04_rb,0.90139e+03_rb,0.77926e+03_rb,0.65759e+03_rb, &
        & 0.65058e+03_rb,0.59579e+03_rb,0.63348e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 2, 5) = (/ &
        & 0.11246e+04_rb,0.10051e+04_rb,0.88561e+03_rb,0.76611e+03_rb,0.64778e+03_rb, &
        & 0.64016e+03_rb,0.59410e+03_rb,0.63264e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 3, 5) = (/ &
        & 0.11441e+04_rb,0.10221e+04_rb,0.90021e+03_rb,0.77828e+03_rb,0.65728e+03_rb, &
        & 0.64981e+03_rb,0.59567e+03_rb,0.63342e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 3, 5) = (/ &
        & 0.11188e+04_rb,0.10001e+04_rb,0.88128e+03_rb,0.76250e+03_rb,0.64582e+03_rb, &
        & 0.63739e+03_rb,0.59363e+03_rb,0.63240e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 3, 5) = (/ &
        & 0.10963e+04_rb,0.98033e+03_rb,0.86437e+03_rb,0.74841e+03_rb,0.63611e+03_rb, &
        & 0.62581e+03_rb,0.59181e+03_rb,0.63149e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 3, 5) = (/ &
        & 0.10761e+04_rb,0.96263e+03_rb,0.84920e+03_rb,0.73577e+03_rb,0.62780e+03_rb, &
        & 0.61513e+03_rb,0.59018e+03_rb,0.63068e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 3, 5) = (/ &
        & 0.10578e+04_rb,0.94664e+03_rb,0.83549e+03_rb,0.72435e+03_rb,0.62066e+03_rb, &
        & 0.60588e+03_rb,0.58871e+03_rb,0.62994e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 4, 5) = (/ &
        & 0.10783e+04_rb,0.96456e+03_rb,0.85086e+03_rb,0.73715e+03_rb,0.62971e+03_rb, &
        & 0.61628e+03_rb,0.59036e+03_rb,0.63077e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 4, 5) = (/ &
        & 0.10559e+04_rb,0.94502e+03_rb,0.83411e+03_rb,0.72319e+03_rb,0.62145e+03_rb, &
        & 0.60490e+03_rb,0.58856e+03_rb,0.62987e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 4, 5) = (/ &
        & 0.10361e+04_rb,0.92766e+03_rb,0.81923e+03_rb,0.71079e+03_rb,0.61457e+03_rb, &
        & 0.59452e+03_rb,0.58696e+03_rb,0.62907e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 4, 5) = (/ &
        & 0.10183e+04_rb,0.91213e+03_rb,0.80591e+03_rb,0.69970e+03_rb,0.60876e+03_rb, &
        & 0.58559e+03_rb,0.58552e+03_rb,0.62835e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 4, 5) = (/ &
        & 0.10024e+04_rb,0.89816e+03_rb,0.79394e+03_rb,0.68972e+03_rb,0.60374e+03_rb, &
        & 0.57800e+03_rb,0.58424e+03_rb,0.62771e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 5, 5) = (/ &
        & 0.10224e+04_rb,0.91564e+03_rb,0.80892e+03_rb,0.70221e+03_rb,0.61144e+03_rb, &
        & 0.58757e+03_rb,0.58585e+03_rb,0.62851e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 5, 5) = (/ &
        & 0.10026e+04_rb,0.89835e+03_rb,0.79410e+03_rb,0.68985e+03_rb,0.60588e+03_rb, &
        & 0.57810e+03_rb,0.58425e+03_rb,0.62772e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 5, 5) = (/ &
        & 0.98511e+03_rb,0.88305e+03_rb,0.78099e+03_rb,0.67893e+03_rb,0.60138e+03_rb, &
        & 0.57034e+03_rb,0.58284e+03_rb,0.62701e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 5, 5) = (/ &
        & 0.96954e+03_rb,0.86942e+03_rb,0.76931e+03_rb,0.66919e+03_rb,0.59759e+03_rb, &
        & 0.56378e+03_rb,0.58159e+03_rb,0.62638e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 5, 5) = (/ &
        & 0.95557e+03_rb,0.85720e+03_rb,0.75883e+03_rb,0.66046e+03_rb,0.59436e+03_rb, &
        & 0.55823e+03_rb,0.58046e+03_rb,0.62582e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 6, 5) = (/ &
        & 0.97547e+03_rb,0.87461e+03_rb,0.77376e+03_rb,0.67290e+03_rb,0.60045e+03_rb, &
        & 0.56624e+03_rb,0.58207e+03_rb,0.62662e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 6, 5) = (/ &
        & 0.95788e+03_rb,0.85922e+03_rb,0.76057e+03_rb,0.66191e+03_rb,0.59710e+03_rb, &
        & 0.55912e+03_rb,0.58065e+03_rb,0.62591e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 6, 5) = (/ &
        & 0.94241e+03_rb,0.84568e+03_rb,0.74896e+03_rb,0.65223e+03_rb,0.59434e+03_rb, &
        & 0.55328e+03_rb,0.57940e+03_rb,0.62529e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 6, 5) = (/ &
        & 0.92868e+03_rb,0.83367e+03_rb,0.73866e+03_rb,0.64365e+03_rb,0.59182e+03_rb, &
        & 0.54839e+03_rb,0.57829e+03_rb,0.62473e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 6, 5) = (/ &
        & 0.91642e+03_rb,0.82295e+03_rb,0.72947e+03_rb,0.63599e+03_rb,0.58948e+03_rb, &
        & 0.54428e+03_rb,0.57730e+03_rb,0.62424e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 7, 5) = (/ &
        & 0.93556e+03_rb,0.83969e+03_rb,0.74382e+03_rb,0.64795e+03_rb,0.59420e+03_rb, &
        & 0.55081e+03_rb,0.57885e+03_rb,0.62501e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 7, 5) = (/ &
        & 0.91989e+03_rb,0.82598e+03_rb,0.73207e+03_rb,0.63816e+03_rb,0.59179e+03_rb, &
        & 0.54542e+03_rb,0.57758e+03_rb,0.62438e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 7, 5) = (/ &
        & 0.90618e+03_rb,0.81398e+03_rb,0.72179e+03_rb,0.62959e+03_rb,0.58955e+03_rb, &
        & 0.54102e+03_rb,0.57648e+03_rb,0.62383e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 7, 5) = (/ &
        & 0.89407e+03_rb,0.80339e+03_rb,0.71271e+03_rb,0.62202e+03_rb,0.58737e+03_rb, &
        & 0.53729e+03_rb,0.57550e+03_rb,0.62334e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 7, 5) = (/ &
        & 0.88331e+03_rb,0.79397e+03_rb,0.70463e+03_rb,0.61530e+03_rb,0.58524e+03_rb, &
        & 0.53413e+03_rb,0.57463e+03_rb,0.62290e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 8, 5) = (/ &
        & 0.90140e+03_rb,0.80981e+03_rb,0.71821e+03_rb,0.62661e+03_rb,0.58941e+03_rb, &
        & 0.53953e+03_rb,0.57609e+03_rb,0.62363e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 8, 5) = (/ &
        & 0.88745e+03_rb,0.79760e+03_rb,0.70774e+03_rb,0.61789e+03_rb,0.58727e+03_rb, &
        & 0.53533e+03_rb,0.57497e+03_rb,0.62307e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 8, 5) = (/ &
        & 0.87531e+03_rb,0.78697e+03_rb,0.69863e+03_rb,0.61030e+03_rb,0.58522e+03_rb, &
        & 0.53203e+03_rb,0.57399e+03_rb,0.62258e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 8, 5) = (/ &
        & 0.86464e+03_rb,0.77763e+03_rb,0.69063e+03_rb,0.60363e+03_rb,0.58317e+03_rb, &
        & 0.52938e+03_rb,0.57313e+03_rb,0.62215e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 8, 5) = (/ &
        & 0.85519e+03_rb,0.76937e+03_rb,0.68354e+03_rb,0.59772e+03_rb,0.58120e+03_rb, &
        & 0.52712e+03_rb,0.57236e+03_rb,0.62177e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 9, 5) = (/ &
        & 0.87232e+03_rb,0.78435e+03_rb,0.69639e+03_rb,0.60843e+03_rb,0.58508e+03_rb, &
        & 0.53128e+03_rb,0.57375e+03_rb,0.62246e+03_rb,0.67118e+03_rb /)
      kao(:, 2, 9, 5) = (/ &
        & 0.85988e+03_rb,0.77347e+03_rb,0.68706e+03_rb,0.60065e+03_rb,0.58302e+03_rb, &
        & 0.52823e+03_rb,0.57274e+03_rb,0.62196e+03_rb,0.67118e+03_rb /)
      kao(:, 3, 9, 5) = (/ &
        & 0.84911e+03_rb,0.76405e+03_rb,0.67898e+03_rb,0.59392e+03_rb,0.58102e+03_rb, &
        & 0.52572e+03_rb,0.57187e+03_rb,0.62153e+03_rb,0.67118e+03_rb /)
      kao(:, 4, 9, 5) = (/ &
        & 0.83970e+03_rb,0.75581e+03_rb,0.67193e+03_rb,0.58804e+03_rb,0.57906e+03_rb, &
        & 0.52364e+03_rb,0.57111e+03_rb,0.62115e+03_rb,0.67118e+03_rb /)
      kao(:, 5, 9, 5) = (/ &
        & 0.83140e+03_rb,0.74855e+03_rb,0.66571e+03_rb,0.58286e+03_rb,0.57713e+03_rb, &
        & 0.52190e+03_rb,0.57045e+03_rb,0.62081e+03_rb,0.67118e+03_rb /)
      kao(:, 1,10, 5) = (/ &
        & 0.84657e+03_rb,0.76183e+03_rb,0.67708e+03_rb,0.59234e+03_rb,0.58061e+03_rb, &
        & 0.52515e+03_rb,0.57167e+03_rb,0.62142e+03_rb,0.67118e+03_rb /)
      kao(:, 2,10, 5) = (/ &
        & 0.83560e+03_rb,0.75223e+03_rb,0.66886e+03_rb,0.58548e+03_rb,0.57865e+03_rb, &
        & 0.52276e+03_rb,0.57078e+03_rb,0.62098e+03_rb,0.67118e+03_rb /)
      kao(:, 3,10, 5) = (/ &
        & 0.82616e+03_rb,0.74397e+03_rb,0.66177e+03_rb,0.57958e+03_rb,0.57663e+03_rb, &
        & 0.52085e+03_rb,0.57002e+03_rb,0.62060e+03_rb,0.67118e+03_rb /)
      kao(:, 4,10, 5) = (/ &
        & 0.81794e+03_rb,0.73678e+03_rb,0.65561e+03_rb,0.57444e+03_rb,0.57466e+03_rb, &
        & 0.51933e+03_rb,0.56936e+03_rb,0.62027e+03_rb,0.67118e+03_rb /)
      kao(:, 5,10, 5) = (/ &
        & 0.81073e+03_rb,0.73047e+03_rb,0.65020e+03_rb,0.56994e+03_rb,0.57276e+03_rb, &
        & 0.51811e+03_rb,0.56878e+03_rb,0.61998e+03_rb,0.67118e+03_rb /)
      kao(:, 1,11, 5) = (/ &
        & 0.82001e+03_rb,0.73859e+03_rb,0.65716e+03_rb,0.57574e+03_rb,0.57531e+03_rb, &
        & 0.51970e+03_rb,0.56953e+03_rb,0.62035e+03_rb,0.67118e+03_rb /)
      kao(:, 2,11, 5) = (/ &
        & 0.81102e+03_rb,0.73072e+03_rb,0.65042e+03_rb,0.57012e+03_rb,0.57326e+03_rb, &
        & 0.51814e+03_rb,0.56880e+03_rb,0.61999e+03_rb,0.67118e+03_rb /)
      kao(:, 3,11, 5) = (/ &
        & 0.80328e+03_rb,0.72395e+03_rb,0.64462e+03_rb,0.56528e+03_rb,0.57135e+03_rb, &
        & 0.51670e+03_rb,0.56818e+03_rb,0.61968e+03_rb,0.67118e+03_rb /)
      kao(:, 4,11, 5) = (/ &
        & 0.79655e+03_rb,0.71806e+03_rb,0.63957e+03_rb,0.56108e+03_rb,0.56956e+03_rb, &
        & 0.51595e+03_rb,0.56763e+03_rb,0.61941e+03_rb,0.67118e+03_rb /)
      kao(:, 5,11, 5) = (/ &
        & 0.79064e+03_rb,0.71289e+03_rb,0.63514e+03_rb,0.55738e+03_rb,0.56781e+03_rb, &
        & 0.51517e+03_rb,0.56716e+03_rb,0.61917e+03_rb,0.67118e+03_rb /)
      kao(:, 1,12, 5) = (/ &
        & 0.79821e+03_rb,0.71951e+03_rb,0.64081e+03_rb,0.56211e+03_rb,0.57002e+03_rb, &
        & 0.51613e+03_rb,0.56777e+03_rb,0.61947e+03_rb,0.67118e+03_rb /)
      kao(:, 2,12, 5) = (/ &
        & 0.79085e+03_rb,0.71307e+03_rb,0.63529e+03_rb,0.55751e+03_rb,0.56813e+03_rb, &
        & 0.51519e+03_rb,0.56717e+03_rb,0.61918e+03_rb,0.67118e+03_rb /)
      kao(:, 3,12, 5) = (/ &
        & 0.78452e+03_rb,0.70753e+03_rb,0.63054e+03_rb,0.55355e+03_rb,0.56648e+03_rb, &
        & 0.51441e+03_rb,0.56666e+03_rb,0.61892e+03_rb,0.67118e+03_rb /)
      kao(:, 4,12, 5) = (/ &
        & 0.77901e+03_rb,0.70271e+03_rb,0.62641e+03_rb,0.55011e+03_rb,0.56489e+03_rb, &
        & 0.51374e+03_rb,0.56622e+03_rb,0.61870e+03_rb,0.67118e+03_rb /)
      kao(:, 5,12, 5) = (/ &
        & 0.77417e+03_rb,0.69847e+03_rb,0.62278e+03_rb,0.54708e+03_rb,0.56325e+03_rb, &
        & 0.51315e+03_rb,0.56583e+03_rb,0.61850e+03_rb,0.67118e+03_rb /)
      kao(:, 1,13, 5) = (/ &
        & 0.78036e+03_rb,0.70389e+03_rb,0.62743e+03_rb,0.55096e+03_rb,0.56495e+03_rb, &
        & 0.51390e+03_rb,0.56633e+03_rb,0.61875e+03_rb,0.67118e+03_rb /)
      kao(:, 2,13, 5) = (/ &
        & 0.77434e+03_rb,0.69862e+03_rb,0.62291e+03_rb,0.54719e+03_rb,0.56343e+03_rb, &
        & 0.51318e+03_rb,0.56584e+03_rb,0.61851e+03_rb,0.67118e+03_rb /)
      kao(:, 3,13, 5) = (/ &
        & 0.76915e+03_rb,0.69408e+03_rb,0.61902e+03_rb,0.54395e+03_rb,0.56189e+03_rb, &
        & 0.51255e+03_rb,0.56542e+03_rb,0.61830e+03_rb,0.67118e+03_rb /)
      kao(:, 4,13, 5) = (/ &
        & 0.76464e+03_rb,0.69014e+03_rb,0.61563e+03_rb,0.54113e+03_rb,0.56027e+03_rb, &
        & 0.51200e+03_rb,0.56506e+03_rb,0.61812e+03_rb,0.67118e+03_rb /)
      kao(:, 5,13, 5) = (/ &
        & 0.76068e+03_rb,0.68667e+03_rb,0.61266e+03_rb,0.53865e+03_rb,0.55865e+03_rb, &
        & 0.51152e+03_rb,0.56474e+03_rb,0.61796e+03_rb,0.67118e+03_rb /)
      kao(:, 1, 1, 6) = (/ &
        & 0.16832e+04_rb,0.14832e+04_rb,0.12833e+04_rb,0.10834e+04_rb,0.88343e+03_rb, &
        & 0.73047e+03_rb,0.75923e+03_rb,0.85568e+03_rb,0.94989e+03_rb /)
      kao(:, 2, 1, 6) = (/ &
        & 0.16405e+04_rb,0.14459e+04_rb,0.12513e+04_rb,0.10567e+04_rb,0.86211e+03_rb, &
        & 0.72257e+03_rb,0.75791e+03_rb,0.85457e+03_rb,0.94915e+03_rb /)
      kao(:, 3, 1, 6) = (/ &
        & 0.16022e+04_rb,0.14124e+04_rb,0.12226e+04_rb,0.10328e+04_rb,0.84296e+03_rb, &
        & 0.71182e+03_rb,0.75643e+03_rb,0.85315e+03_rb,0.94805e+03_rb /)
      kao(:, 4, 1, 6) = (/ &
        & 0.15677e+04_rb,0.13822e+04_rb,0.11967e+04_rb,0.10112e+04_rb,0.82566e+03_rb, &
        & 0.70003e+03_rb,0.75475e+03_rb,0.85145e+03_rb,0.94659e+03_rb /)
      kao(:, 5, 1, 6) = (/ &
        & 0.15363e+04_rb,0.13547e+04_rb,0.11731e+04_rb,0.99155e+03_rb,0.80997e+03_rb, &
        & 0.68802e+03_rb,0.75277e+03_rb,0.84941e+03_rb,0.94478e+03_rb /)
      kao(:, 1, 2, 6) = (/ &
        & 0.15549e+04_rb,0.13710e+04_rb,0.11871e+04_rb,0.10032e+04_rb,0.81930e+03_rb, &
        & 0.69644e+03_rb,0.75599e+03_rb,0.85402e+03_rb,0.95009e+03_rb /)
      kao(:, 2, 2, 6) = (/ &
        & 0.15184e+04_rb,0.13390e+04_rb,0.11597e+04_rb,0.98036e+03_rb,0.80102e+03_rb, &
        & 0.68214e+03_rb,0.75486e+03_rb,0.85308e+03_rb,0.94950e+03_rb /)
      kao(:, 3, 2, 6) = (/ &
        & 0.14856e+04_rb,0.13104e+04_rb,0.11351e+04_rb,0.95988e+03_rb,0.78464e+03_rb, &
        & 0.67032e+03_rb,0.75359e+03_rb,0.85186e+03_rb,0.94855e+03_rb /)
      kao(:, 4, 2, 6) = (/ &
        & 0.14561e+04_rb,0.12845e+04_rb,0.11130e+04_rb,0.94143e+03_rb,0.76988e+03_rb, &
        & 0.66240e+03_rb,0.75215e+03_rb,0.85037e+03_rb,0.94724e+03_rb /)
      kao(:, 5, 2, 6) = (/ &
        & 0.14294e+04_rb,0.12611e+04_rb,0.10929e+04_rb,0.92472e+03_rb,0.75651e+03_rb, &
        & 0.65660e+03_rb,0.75042e+03_rb,0.84857e+03_rb,0.94557e+03_rb /)
      kao(:, 1, 3, 6) = (/ &
        & 0.14541e+04_rb,0.12828e+04_rb,0.11115e+04_rb,0.94019e+03_rb,0.76889e+03_rb, &
        & 0.66226e+03_rb,0.75340e+03_rb,0.85268e+03_rb,0.95029e+03_rb /)
      kao(:, 2, 3, 6) = (/ &
        & 0.14220e+04_rb,0.12547e+04_rb,0.10874e+04_rb,0.92013e+03_rb,0.75284e+03_rb, &
        & 0.65611e+03_rb,0.75249e+03_rb,0.85200e+03_rb,0.94990e+03_rb /)
      kao(:, 3, 3, 6) = (/ &
        & 0.13934e+04_rb,0.12297e+04_rb,0.10659e+04_rb,0.90223e+03_rb,0.73852e+03_rb, &
        & 0.65222e+03_rb,0.75143e+03_rb,0.85101e+03_rb,0.94916e+03_rb /)
      kao(:, 4, 3, 6) = (/ &
        & 0.13676e+04_rb,0.12071e+04_rb,0.10467e+04_rb,0.88615e+03_rb,0.72566e+03_rb, &
        & 0.64945e+03_rb,0.75024e+03_rb,0.84977e+03_rb,0.94807e+03_rb /)
      kao(:, 5, 3, 6) = (/ &
        & 0.13444e+04_rb,0.11868e+04_rb,0.10292e+04_rb,0.87164e+03_rb,0.71405e+03_rb, &
        & 0.64733e+03_rb,0.74880e+03_rb,0.84822e+03_rb,0.94662e+03_rb /)
      kao(:, 1, 4, 6) = (/ &
        & 0.13705e+04_rb,0.12096e+04_rb,0.10488e+04_rb,0.88791e+03_rb,0.72707e+03_rb, &
        & 0.64976e+03_rb,0.75119e+03_rb,0.85151e+03_rb,0.95037e+03_rb /)
      kao(:, 2, 4, 6) = (/ &
        & 0.13421e+04_rb,0.11848e+04_rb,0.10275e+04_rb,0.87017e+03_rb,0.71287e+03_rb, &
        & 0.64785e+03_rb,0.75047e+03_rb,0.85104e+03_rb,0.95019e+03_rb /)
      kao(:, 3, 4, 6) = (/ &
        & 0.13169e+04_rb,0.11627e+04_rb,0.10086e+04_rb,0.85441e+03_rb,0.70026e+03_rb, &
        & 0.64696e+03_rb,0.74964e+03_rb,0.85031e+03_rb,0.94968e+03_rb /)
      kao(:, 4, 4, 6) = (/ &
        & 0.12943e+04_rb,0.11430e+04_rb,0.99164e+03_rb,0.84031e+03_rb,0.68898e+03_rb, &
        & 0.64601e+03_rb,0.74865e+03_rb,0.84930e+03_rb,0.94882e+03_rb /)
      kao(:, 5, 4, 6) = (/ &
        & 0.12740e+04_rb,0.11252e+04_rb,0.97642e+03_rb,0.82763e+03_rb,0.67884e+03_rb, &
        & 0.64496e+03_rb,0.74747e+03_rb,0.84802e+03_rb,0.94760e+03_rb /)
      kao(:, 1, 5, 6) = (/ &
        & 0.12994e+04_rb,0.11474e+04_rb,0.99546e+03_rb,0.84350e+03_rb,0.69153e+03_rb, &
        & 0.64599e+03_rb,0.74922e+03_rb,0.85040e+03_rb,0.95032e+03_rb /)
      kao(:, 2, 5, 6) = (/ &
        & 0.12743e+04_rb,0.11254e+04_rb,0.97662e+03_rb,0.82780e+03_rb,0.67897e+03_rb, &
        & 0.64529e+03_rb,0.74869e+03_rb,0.85015e+03_rb,0.95034e+03_rb /)
      kao(:, 3, 5, 6) = (/ &
        & 0.12521e+04_rb,0.11060e+04_rb,0.95996e+03_rb,0.81391e+03_rb,0.66786e+03_rb, &
        & 0.64458e+03_rb,0.74805e+03_rb,0.84965e+03_rb,0.95004e+03_rb /)
      kao(:, 4, 5, 6) = (/ &
        & 0.12323e+04_rb,0.10887e+04_rb,0.94511e+03_rb,0.80154e+03_rb,0.65797e+03_rb, &
        & 0.64380e+03_rb,0.74727e+03_rb,0.84886e+03_rb,0.94940e+03_rb /)
      kao(:, 5, 5, 6) = (/ &
        & 0.12145e+04_rb,0.10732e+04_rb,0.93180e+03_rb,0.79045e+03_rb,0.64909e+03_rb, &
        & 0.64294e+03_rb,0.74629e+03_rb,0.84781e+03_rb,0.94841e+03_rb /)
      kao(:, 1, 6, 6) = (/ &
        & 0.12398e+04_rb,0.10953e+04_rb,0.95077e+03_rb,0.80625e+03_rb,0.66174e+03_rb, &
        & 0.64362e+03_rb,0.74747e+03_rb,0.84935e+03_rb,0.95014e+03_rb /)
      kao(:, 2, 6, 6) = (/ &
        & 0.12174e+04_rb,0.10757e+04_rb,0.93400e+03_rb,0.79228e+03_rb,0.65056e+03_rb, &
        & 0.64308e+03_rb,0.74712e+03_rb,0.84932e+03_rb,0.95036e+03_rb /)
      kao(:, 3, 6, 6) = (/ &
        & 0.11978e+04_rb,0.10585e+04_rb,0.91925e+03_rb,0.77999e+03_rb,0.64087e+03_rb, &
        & 0.64253e+03_rb,0.74666e+03_rb,0.84902e+03_rb,0.95027e+03_rb /)
      kao(:, 4, 6, 6) = (/ &
        & 0.11803e+04_rb,0.10432e+04_rb,0.90616e+03_rb,0.76908e+03_rb,0.63262e+03_rb, &
        & 0.64192e+03_rb,0.74606e+03_rb,0.84847e+03_rb,0.94985e+03_rb /)
      kao(:, 5, 6, 6) = (/ &
        & 0.11647e+04_rb,0.10296e+04_rb,0.89448e+03_rb,0.75935e+03_rb,0.62549e+03_rb, &
        & 0.64122e+03_rb,0.74529e+03_rb,0.84763e+03_rb,0.94909e+03_rb /)
      kao(:, 1, 7, 6) = (/ &
        & 0.11891e+04_rb,0.10509e+04_rb,0.91272e+03_rb,0.77455e+03_rb,0.63693e+03_rb, &
        & 0.64154e+03_rb,0.74584e+03_rb,0.84830e+03_rb,0.94975e+03_rb /)
      kao(:, 2, 7, 6) = (/ &
        & 0.11692e+04_rb,0.10335e+04_rb,0.89779e+03_rb,0.76210e+03_rb,0.62795e+03_rb, &
        & 0.64115e+03_rb,0.74575e+03_rb,0.84848e+03_rb,0.95025e+03_rb /)
      kao(:, 3, 7, 6) = (/ &
        & 0.11517e+04_rb,0.10182e+04_rb,0.88471e+03_rb,0.75121e+03_rb,0.62051e+03_rb, &
        & 0.64074e+03_rb,0.74542e+03_rb,0.84840e+03_rb,0.95037e+03_rb /)
      kao(:, 4, 7, 6) = (/ &
        & 0.11363e+04_rb,0.10048e+04_rb,0.87317e+03_rb,0.74159e+03_rb,0.61432e+03_rb, &
        & 0.64028e+03_rb,0.74499e+03_rb,0.84806e+03_rb,0.95015e+03_rb /)
      kao(:, 5, 7, 6) = (/ &
        & 0.11227e+04_rb,0.99279e+03_rb,0.86291e+03_rb,0.73304e+03_rb,0.60908e+03_rb, &
        & 0.63974e+03_rb,0.74440e+03_rb,0.84744e+03_rb,0.94961e+03_rb /)
      kao(:, 1, 8, 6) = (/ &
        & 0.11457e+04_rb,0.10129e+04_rb,0.88017e+03_rb,0.74742e+03_rb,0.61848e+03_rb, &
        & 0.63968e+03_rb,0.74433e+03_rb,0.84725e+03_rb,0.94927e+03_rb /)
      kao(:, 2, 8, 6) = (/ &
        & 0.11279e+04_rb,0.99740e+03_rb,0.86687e+03_rb,0.73633e+03_rb,0.61183e+03_rb, &
        & 0.63944e+03_rb,0.74439e+03_rb,0.84765e+03_rb,0.94999e+03_rb /)
      kao(:, 3, 8, 6) = (/ &
        & 0.11125e+04_rb,0.98389e+03_rb,0.85529e+03_rb,0.72668e+03_rb,0.60652e+03_rb, &
        & 0.63917e+03_rb,0.74428e+03_rb,0.84777e+03_rb,0.95034e+03_rb /)
      kao(:, 4, 8, 6) = (/ &
        & 0.10989e+04_rb,0.97202e+03_rb,0.84512e+03_rb,0.71821e+03_rb,0.60224e+03_rb, &
        & 0.63885e+03_rb,0.74401e+03_rb,0.84763e+03_rb,0.95032e+03_rb /)
      kao(:, 5, 8, 6) = (/ &
        & 0.10869e+04_rb,0.96152e+03_rb,0.83611e+03_rb,0.71070e+03_rb,0.59867e+03_rb, &
        & 0.63844e+03_rb,0.74359e+03_rb,0.84721e+03_rb,0.94997e+03_rb /)
      kao(:, 1, 9, 6) = (/ &
        & 0.11087e+04_rb,0.98057e+03_rb,0.85244e+03_rb,0.72431e+03_rb,0.60583e+03_rb, &
        & 0.63802e+03_rb,0.74290e+03_rb,0.84619e+03_rb,0.94868e+03_rb /)
      kao(:, 2, 9, 6) = (/ &
        & 0.10929e+04_rb,0.96673e+03_rb,0.84058e+03_rb,0.71443e+03_rb,0.60128e+03_rb, &
        & 0.63792e+03_rb,0.74318e+03_rb,0.84681e+03_rb,0.94962e+03_rb /)
      kao(:, 3, 9, 6) = (/ &
        & 0.10792e+04_rb,0.95475e+03_rb,0.83031e+03_rb,0.70587e+03_rb,0.59784e+03_rb, &
        & 0.63778e+03_rb,0.74323e+03_rb,0.84713e+03_rb,0.95019e+03_rb /)
      kao(:, 4, 9, 6) = (/ &
        & 0.10672e+04_rb,0.94429e+03_rb,0.82134e+03_rb,0.69840e+03_rb,0.59517e+03_rb, &
        & 0.63759e+03_rb,0.74313e+03_rb,0.84718e+03_rb,0.95037e+03_rb /)
      kao(:, 5, 9, 6) = (/ &
        & 0.10567e+04_rb,0.93506e+03_rb,0.81344e+03_rb,0.69181e+03_rb,0.59278e+03_rb, &
        & 0.63730e+03_rb,0.74285e+03_rb,0.84695e+03_rb,0.95021e+03_rb /)
      kao(:, 1,10, 6) = (/ &
        & 0.10760e+04_rb,0.95193e+03_rb,0.82790e+03_rb,0.70386e+03_rb,0.59768e+03_rb, &
        & 0.63651e+03_rb,0.74157e+03_rb,0.84519e+03_rb,0.94809e+03_rb /)
      kao(:, 2,10, 6) = (/ &
        & 0.10620e+04_rb,0.93974e+03_rb,0.81744e+03_rb,0.69515e+03_rb,0.59477e+03_rb, &
        & 0.63656e+03_rb,0.74203e+03_rb,0.84600e+03_rb,0.94921e+03_rb /)
      kao(:, 3,10, 6) = (/ &
        & 0.10500e+04_rb,0.92923e+03_rb,0.80844e+03_rb,0.68764e+03_rb,0.59242e+03_rb, &
        & 0.63654e+03_rb,0.74226e+03_rb,0.84650e+03_rb,0.94996e+03_rb /)
      kao(:, 4,10, 6) = (/ &
        & 0.10396e+04_rb,0.92010e+03_rb,0.80061e+03_rb,0.68112e+03_rb,0.59042e+03_rb, &
        & 0.63645e+03_rb,0.74230e+03_rb,0.84672e+03_rb,0.95033e+03_rb /)
      kao(:, 5,10, 6) = (/ &
        & 0.10304e+04_rb,0.91207e+03_rb,0.79373e+03_rb,0.67539e+03_rb,0.58849e+03_rb, &
        & 0.63628e+03_rb,0.74216e+03_rb,0.84665e+03_rb,0.95033e+03_rb /)
      kao(:, 1,11, 6) = (/ &
        & 0.10422e+04_rb,0.92239e+03_rb,0.80258e+03_rb,0.68276e+03_rb,0.59118e+03_rb, &
        & 0.63525e+03_rb,0.74065e+03_rb,0.84470e+03_rb,0.94808e+03_rb /)
      kao(:, 2,11, 6) = (/ &
        & 0.10308e+04_rb,0.91240e+03_rb,0.79401e+03_rb,0.67562e+03_rb,0.58920e+03_rb, &
        & 0.63540e+03_rb,0.74118e+03_rb,0.84554e+03_rb,0.94920e+03_rb /)
      kao(:, 3,11, 6) = (/ &
        & 0.10210e+04_rb,0.90379e+03_rb,0.78663e+03_rb,0.66947e+03_rb,0.58737e+03_rb, &
        & 0.63545e+03_rb,0.74147e+03_rb,0.84609e+03_rb,0.94995e+03_rb /)
      kao(:, 4,11, 6) = (/ &
        & 0.10124e+04_rb,0.89631e+03_rb,0.78022e+03_rb,0.66413e+03_rb,0.58569e+03_rb, &
        & 0.63543e+03_rb,0.74156e+03_rb,0.84634e+03_rb,0.95033e+03_rb /)
      kao(:, 5,11, 6) = (/ &
        & 0.10049e+04_rb,0.88974e+03_rb,0.77458e+03_rb,0.65943e+03_rb,0.58398e+03_rb, &
        & 0.63532e+03_rb,0.74146e+03_rb,0.84628e+03_rb,0.95033e+03_rb /)
      kao(:, 1,12, 6) = (/ &
        & 0.10145e+04_rb,0.89815e+03_rb,0.78180e+03_rb,0.66544e+03_rb,0.58616e+03_rb, &
        & 0.63421e+03_rb,0.73990e+03_rb,0.84430e+03_rb,0.94808e+03_rb /)
      kao(:, 2,12, 6) = (/ &
        & 0.10052e+04_rb,0.88997e+03_rb,0.77478e+03_rb,0.65960e+03_rb,0.58451e+03_rb, &
        & 0.63444e+03_rb,0.74048e+03_rb,0.84518e+03_rb,0.94921e+03_rb /)
      kao(:, 3,12, 6) = (/ &
        & 0.99710e+03_rb,0.88292e+03_rb,0.76874e+03_rb,0.65457e+03_rb,0.58265e+03_rb, &
        & 0.63457e+03_rb,0.74082e+03_rb,0.84574e+03_rb,0.94995e+03_rb /)
      kao(:, 4,12, 6) = (/ &
        & 0.99010e+03_rb,0.87679e+03_rb,0.76349e+03_rb,0.65019e+03_rb,0.58054e+03_rb, &
        & 0.63460e+03_rb,0.74095e+03_rb,0.84600e+03_rb,0.95033e+03_rb /)
      kao(:, 5,12, 6) = (/ &
        & 0.98395e+03_rb,0.87141e+03_rb,0.75888e+03_rb,0.64634e+03_rb,0.57844e+03_rb, &
        & 0.63453e+03_rb,0.74089e+03_rb,0.84598e+03_rb,0.95033e+03_rb /)
      kao(:, 1,13, 6) = (/ &
        & 0.99182e+03_rb,0.87830e+03_rb,0.76478e+03_rb,0.65127e+03_rb,0.58122e+03_rb, &
        & 0.63336e+03_rb,0.73928e+03_rb,0.84397e+03_rb,0.94808e+03_rb /)
      kao(:, 2,13, 6) = (/ &
        & 0.98416e+03_rb,0.87160e+03_rb,0.75904e+03_rb,0.64648e+03_rb,0.57888e+03_rb, &
        & 0.63366e+03_rb,0.73991e+03_rb,0.84487e+03_rb,0.94921e+03_rb /)
      kao(:, 3,13, 6) = (/ &
        & 0.97757e+03_rb,0.86583e+03_rb,0.75410e+03_rb,0.64236e+03_rb,0.57650e+03_rb, &
        & 0.63384e+03_rb,0.74029e+03_rb,0.84546e+03_rb,0.94996e+03_rb /)
      kao(:, 4,13, 6) = (/ &
        & 0.97184e+03_rb,0.86082e+03_rb,0.74980e+03_rb,0.63878e+03_rb,0.57428e+03_rb, &
        & 0.63391e+03_rb,0.74045e+03_rb,0.84574e+03_rb,0.95033e+03_rb /)
      kao(:, 5,13, 6) = (/ &
        & 0.96680e+03_rb,0.85641e+03_rb,0.74602e+03_rb,0.63563e+03_rb,0.57218e+03_rb, &
        & 0.63388e+03_rb,0.74043e+03_rb,0.84574e+03_rb,0.95033e+03_rb /)
      kao(:, 1, 1, 7) = (/ &
        & 0.18845e+04_rb,0.16548e+04_rb,0.14251e+04_rb,0.11953e+04_rb,0.96557e+03_rb, &
        & 0.74301e+03_rb,0.82697e+03_rb,0.94964e+03_rb,0.10740e+04_rb /)
      kao(:, 2, 1, 7) = (/ &
        & 0.18367e+04_rb,0.16130e+04_rb,0.13892e+04_rb,0.11655e+04_rb,0.94169e+03_rb, &
        & 0.73101e+03_rb,0.82533e+03_rb,0.94833e+03_rb,0.10730e+04_rb /)
      kao(:, 3, 1, 7) = (/ &
        & 0.17938e+04_rb,0.15755e+04_rb,0.13571e+04_rb,0.11387e+04_rb,0.92025e+03_rb, &
        & 0.71894e+03_rb,0.82353e+03_rb,0.94679e+03_rb,0.10715e+04_rb /)
      kao(:, 4, 1, 7) = (/ &
        & 0.17551e+04_rb,0.15416e+04_rb,0.13280e+04_rb,0.11145e+04_rb,0.90089e+03_rb, &
        & 0.70691e+03_rb,0.82149e+03_rb,0.94487e+03_rb,0.10696e+04_rb /)
      kao(:, 5, 1, 7) = (/ &
        & 0.17200e+04_rb,0.15108e+04_rb,0.13017e+04_rb,0.10925e+04_rb,0.88332e+03_rb, &
        & 0.69946e+03_rb,0.81920e+03_rb,0.94260e+03_rb,0.10672e+04_rb /)
      kao(:, 1, 2, 7) = (/ &
        & 0.17409e+04_rb,0.15291e+04_rb,0.13174e+04_rb,0.11056e+04_rb,0.89377e+03_rb, &
        & 0.70556e+03_rb,0.82545e+03_rb,0.94918e+03_rb,0.10743e+04_rb /)
      kao(:, 2, 2, 7) = (/ &
        & 0.16999e+04_rb,0.14933e+04_rb,0.12867e+04_rb,0.10800e+04_rb,0.87329e+03_rb, &
        & 0.70177e+03_rb,0.82404e+03_rb,0.94806e+03_rb,0.10734e+04_rb /)
      kao(:, 3, 2, 7) = (/ &
        & 0.16632e+04_rb,0.14612e+04_rb,0.12591e+04_rb,0.10571e+04_rb,0.85495e+03_rb, &
        & 0.69985e+03_rb,0.82245e+03_rb,0.94668e+03_rb,0.10721e+04_rb /)
      kao(:, 4, 2, 7) = (/ &
        & 0.16302e+04_rb,0.14323e+04_rb,0.12344e+04_rb,0.10364e+04_rb,0.83842e+03_rb, &
        & 0.69778e+03_rb,0.82061e+03_rb,0.94495e+03_rb,0.10705e+04_rb /)
      kao(:, 5, 2, 7) = (/ &
        & 0.16003e+04_rb,0.14061e+04_rb,0.12119e+04_rb,0.10177e+04_rb,0.82345e+03_rb, &
        & 0.69549e+03_rb,0.81853e+03_rb,0.94286e+03_rb,0.10683e+04_rb /)
      kao(:, 1, 3, 7) = (/ &
        & 0.16280e+04_rb,0.14303e+04_rb,0.12327e+04_rb,0.10350e+04_rb,0.83731e+03_rb, &
        & 0.70164e+03_rb,0.82440e+03_rb,0.94900e+03_rb,0.10748e+04_rb /)
      kao(:, 2, 3, 7) = (/ &
        & 0.15920e+04_rb,0.13989e+04_rb,0.12057e+04_rb,0.10126e+04_rb,0.81934e+03_rb, &
        & 0.70016e+03_rb,0.82323e+03_rb,0.94804e+03_rb,0.10740e+04_rb /)
      kao(:, 3, 3, 7) = (/ &
        & 0.15600e+04_rb,0.13708e+04_rb,0.11817e+04_rb,0.99252e+03_rb,0.80330e+03_rb, &
        & 0.69850e+03_rb,0.82187e+03_rb,0.94688e+03_rb,0.10730e+04_rb /)
      kao(:, 4, 3, 7) = (/ &
        & 0.15312e+04_rb,0.13457e+04_rb,0.11601e+04_rb,0.97452e+03_rb,0.78890e+03_rb, &
        & 0.69669e+03_rb,0.82029e+03_rb,0.94542e+03_rb,0.10716e+04_rb /)
      kao(:, 5, 3, 7) = (/ &
        & 0.15052e+04_rb,0.13229e+04_rb,0.11406e+04_rb,0.95827e+03_rb,0.77590e+03_rb, &
        & 0.69468e+03_rb,0.81844e+03_rb,0.94358e+03_rb,0.10697e+04_rb /)
      kao(:, 1, 4, 7) = (/ &
        & 0.15343e+04_rb,0.13484e+04_rb,0.11625e+04_rb,0.97650e+03_rb,0.79047e+03_rb, &
        & 0.70005e+03_rb,0.82352e+03_rb,0.94883e+03_rb,0.10751e+04_rb /)
      kao(:, 2, 4, 7) = (/ &
        & 0.15026e+04_rb,0.13206e+04_rb,0.11386e+04_rb,0.95663e+03_rb,0.77458e+03_rb, &
        & 0.69884e+03_rb,0.82260e+03_rb,0.94808e+03_rb,0.10745e+04_rb /)
      kao(:, 3, 4, 7) = (/ &
        & 0.14743e+04_rb,0.12959e+04_rb,0.11175e+04_rb,0.93898e+03_rb,0.76046e+03_rb, &
        & 0.69743e+03_rb,0.82145e+03_rb,0.94711e+03_rb,0.10737e+04_rb /)
      kao(:, 4, 4, 7) = (/ &
        & 0.14491e+04_rb,0.12738e+04_rb,0.10985e+04_rb,0.92319e+03_rb,0.74783e+03_rb, &
        & 0.69586e+03_rb,0.82010e+03_rb,0.94588e+03_rb,0.10725e+04_rb /)
      kao(:, 5, 4, 7) = (/ &
        & 0.14263e+04_rb,0.12539e+04_rb,0.10815e+04_rb,0.90900e+03_rb,0.73646e+03_rb, &
        & 0.69412e+03_rb,0.81852e+03_rb,0.94430e+03_rb,0.10710e+04_rb /)
      kao(:, 1, 5, 7) = (/ &
        & 0.14548e+04_rb,0.12788e+04_rb,0.11028e+04_rb,0.92676e+03_rb,0.75068e+03_rb, &
        & 0.69862e+03_rb,0.82271e+03_rb,0.94860e+03_rb,0.10753e+04_rb /)
      kao(:, 2, 5, 7) = (/ &
        & 0.14266e+04_rb,0.12542e+04_rb,0.10817e+04_rb,0.90918e+03_rb,0.73662e+03_rb, &
        & 0.69765e+03_rb,0.82202e+03_rb,0.94804e+03_rb,0.10749e+04_rb /)
      kao(:, 3, 5, 7) = (/ &
        & 0.14018e+04_rb,0.12324e+04_rb,0.10630e+04_rb,0.89363e+03_rb,0.72418e+03_rb, &
        & 0.69649e+03_rb,0.82110e+03_rb,0.94729e+03_rb,0.10743e+04_rb /)
      kao(:, 4, 5, 7) = (/ &
        & 0.13796e+04_rb,0.12130e+04_rb,0.10464e+04_rb,0.87978e+03_rb,0.71309e+03_rb, &
        & 0.69515e+03_rb,0.81992e+03_rb,0.94623e+03_rb,0.10733e+04_rb /)
      kao(:, 5, 5, 7) = (/ &
        & 0.13597e+04_rb,0.11956e+04_rb,0.10315e+04_rb,0.86737e+03_rb,0.70315e+03_rb, &
        & 0.69363e+03_rb,0.81858e+03_rb,0.94492e+03_rb,0.10720e+04_rb /)
      kao(:, 1, 6, 7) = (/ &
        & 0.13881e+04_rb,0.12204e+04_rb,0.10528e+04_rb,0.88506e+03_rb,0.71732e+03_rb, &
        & 0.69732e+03_rb,0.82195e+03_rb,0.94833e+03_rb,0.10753e+04_rb /)
      kao(:, 2, 6, 7) = (/ &
        & 0.13630e+04_rb,0.11985e+04_rb,0.10340e+04_rb,0.86942e+03_rb,0.70480e+03_rb, &
        & 0.69661e+03_rb,0.82150e+03_rb,0.94796e+03_rb,0.10752e+04_rb /)
      kao(:, 3, 6, 7) = (/ &
        & 0.13410e+04_rb,0.11792e+04_rb,0.10175e+04_rb,0.85565e+03_rb,0.69378e+03_rb, &
        & 0.69568e+03_rb,0.82079e+03_rb,0.94739e+03_rb,0.10747e+04_rb /)
      kao(:, 4, 6, 7) = (/ &
        & 0.13215e+04_rb,0.11622e+04_rb,0.10028e+04_rb,0.84344e+03_rb,0.68401e+03_rb, &
        & 0.69456e+03_rb,0.81984e+03_rb,0.94655e+03_rb,0.10740e+04_rb /)
      kao(:, 5, 6, 7) = (/ &
        & 0.13040e+04_rb,0.11469e+04_rb,0.98973e+03_rb,0.83254e+03_rb,0.67529e+03_rb, &
        & 0.69326e+03_rb,0.81868e+03_rb,0.94544e+03_rb,0.10729e+04_rb /)
      kao(:, 1, 7, 7) = (/ &
        & 0.13313e+04_rb,0.11707e+04_rb,0.10101e+04_rb,0.84956e+03_rb,0.68891e+03_rb, &
        & 0.69611e+03_rb,0.82122e+03_rb,0.94800e+03_rb,0.10754e+04_rb /)
      kao(:, 2, 7, 7) = (/ &
        & 0.13090e+04_rb,0.11512e+04_rb,0.99343e+03_rb,0.83562e+03_rb,0.67776e+03_rb, &
        & 0.69564e+03_rb,0.82091e+03_rb,0.94785e+03_rb,0.10753e+04_rb /)
      kao(:, 3, 7, 7) = (/ &
        & 0.12894e+04_rb,0.11341e+04_rb,0.97879e+03_rb,0.82342e+03_rb,0.66800e+03_rb, &
        & 0.69494e+03_rb,0.82048e+03_rb,0.94742e+03_rb,0.10750e+04_rb /)
      kao(:, 4, 7, 7) = (/ &
        & 0.12722e+04_rb,0.11191e+04_rb,0.96587e+03_rb,0.81266e+03_rb,0.65938e+03_rb, &
        & 0.69403e+03_rb,0.81973e+03_rb,0.94677e+03_rb,0.10745e+04_rb /)
      kao(:, 5, 7, 7) = (/ &
        & 0.12569e+04_rb,0.11057e+04_rb,0.95438e+03_rb,0.80308e+03_rb,0.65172e+03_rb, &
        & 0.69293e+03_rb,0.81874e+03_rb,0.94587e+03_rb,0.10736e+04_rb /)
      kao(:, 1, 8, 7) = (/ &
        & 0.12827e+04_rb,0.11282e+04_rb,0.97371e+03_rb,0.81918e+03_rb,0.66460e+03_rb, &
        & 0.69497e+03_rb,0.82050e+03_rb,0.94762e+03_rb,0.10753e+04_rb /)
      kao(:, 2, 8, 7) = (/ &
        & 0.12628e+04_rb,0.11108e+04_rb,0.95881e+03_rb,0.80678e+03_rb,0.65467e+03_rb, &
        & 0.69473e+03_rb,0.82044e+03_rb,0.94765e+03_rb,0.10754e+04_rb /)
      kao(:, 3, 8, 7) = (/ &
        & 0.12455e+04_rb,0.10957e+04_rb,0.94585e+03_rb,0.79597e+03_rb,0.64602e+03_rb, &
        & 0.69425e+03_rb,0.82014e+03_rb,0.94746e+03_rb,0.10752e+04_rb /)
      kao(:, 4, 8, 7) = (/ &
        & 0.12303e+04_rb,0.10824e+04_rb,0.93446e+03_rb,0.78648e+03_rb,0.63843e+03_rb, &
        & 0.69354e+03_rb,0.81959e+03_rb,0.94691e+03_rb,0.10748e+04_rb /)
      kao(:, 5, 8, 7) = (/ &
        & 0.12169e+04_rb,0.10706e+04_rb,0.92437e+03_rb,0.77807e+03_rb,0.63171e+03_rb, &
        & 0.69262e+03_rb,0.81878e+03_rb,0.94618e+03_rb,0.10741e+04_rb /)
      kao(:, 1, 9, 7) = (/ &
        & 0.12413e+04_rb,0.10920e+04_rb,0.94266e+03_rb,0.79331e+03_rb,0.64390e+03_rb, &
        & 0.69387e+03_rb,0.81977e+03_rb,0.94713e+03_rb,0.10750e+04_rb /)
      kao(:, 2, 9, 7) = (/ &
        & 0.12236e+04_rb,0.10765e+04_rb,0.92938e+03_rb,0.78225e+03_rb,0.63504e+03_rb, &
        & 0.69385e+03_rb,0.81989e+03_rb,0.94741e+03_rb,0.10753e+04_rb /)
      kao(:, 3, 9, 7) = (/ &
        & 0.12082e+04_rb,0.10631e+04_rb,0.91789e+03_rb,0.77267e+03_rb,0.62738e+03_rb, &
        & 0.69359e+03_rb,0.81979e+03_rb,0.94732e+03_rb,0.10754e+04_rb /)
      kao(:, 4, 9, 7) = (/ &
        & 0.11949e+04_rb,0.10514e+04_rb,0.90785e+03_rb,0.76430e+03_rb,0.62068e+03_rb, &
        & 0.69307e+03_rb,0.81942e+03_rb,0.94698e+03_rb,0.10751e+04_rb /)
      kao(:, 5, 9, 7) = (/ &
        & 0.11831e+04_rb,0.10410e+04_rb,0.89899e+03_rb,0.75692e+03_rb,0.61500e+03_rb, &
        & 0.69235e+03_rb,0.81879e+03_rb,0.94641e+03_rb,0.10746e+04_rb /)
      kao(:, 1,10, 7) = (/ &
        & 0.12046e+04_rb,0.10599e+04_rb,0.91518e+03_rb,0.77041e+03_rb,0.62557e+03_rb, &
        & 0.69285e+03_rb,0.81909e+03_rb,0.94663e+03_rb,0.10746e+04_rb /)
      kao(:, 2,10, 7) = (/ &
        & 0.11890e+04_rb,0.10463e+04_rb,0.90347e+03_rb,0.76066e+03_rb,0.61799e+03_rb, &
        & 0.69301e+03_rb,0.81937e+03_rb,0.94709e+03_rb,0.10752e+04_rb /)
      kao(:, 3,10, 7) = (/ &
        & 0.11756e+04_rb,0.10345e+04_rb,0.89339e+03_rb,0.75225e+03_rb,0.61189e+03_rb, &
        & 0.69293e+03_rb,0.81942e+03_rb,0.94719e+03_rb,0.10754e+04_rb /)
      kao(:, 4,10, 7) = (/ &
        & 0.11639e+04_rb,0.10243e+04_rb,0.88463e+03_rb,0.74495e+03_rb,0.60683e+03_rb, &
        & 0.69260e+03_rb,0.81922e+03_rb,0.94698e+03_rb,0.10753e+04_rb /)
      kao(:, 5,10, 7) = (/ &
        & 0.11536e+04_rb,0.10153e+04_rb,0.87693e+03_rb,0.73853e+03_rb,0.60268e+03_rb, &
        & 0.69203e+03_rb,0.81871e+03_rb,0.94653e+03_rb,0.10749e+04_rb /)
      kao(:, 1,11, 7) = (/ &
        & 0.11668e+04_rb,0.10268e+04_rb,0.88684e+03_rb,0.74678e+03_rb,0.60837e+03_rb, &
        & 0.69208e+03_rb,0.81864e+03_rb,0.94642e+03_rb,0.10746e+04_rb /)
      kao(:, 2,11, 7) = (/ &
        & 0.11540e+04_rb,0.10157e+04_rb,0.87724e+03_rb,0.73879e+03_rb,0.60335e+03_rb, &
        & 0.69229e+03_rb,0.81896e+03_rb,0.94691e+03_rb,0.10752e+04_rb /)
      kao(:, 3,11, 7) = (/ &
        & 0.11430e+04_rb,0.10060e+04_rb,0.86898e+03_rb,0.73191e+03_rb,0.59939e+03_rb, &
        & 0.69226e+03_rb,0.81904e+03_rb,0.94700e+03_rb,0.10754e+04_rb /)
      kao(:, 4,11, 7) = (/ &
        & 0.11335e+04_rb,0.99764e+03_rb,0.86180e+03_rb,0.72592e+03_rb,0.59585e+03_rb, &
        & 0.69198e+03_rb,0.81884e+03_rb,0.94679e+03_rb,0.10753e+04_rb /)
      kao(:, 5,11, 7) = (/ &
        & 0.11251e+04_rb,0.99028e+03_rb,0.85549e+03_rb,0.72067e+03_rb,0.59286e+03_rb, &
        & 0.69146e+03_rb,0.81839e+03_rb,0.94639e+03_rb,0.10749e+04_rb /)
      kao(:, 1,12, 7) = (/ &
        & 0.11358e+04_rb,0.99970e+03_rb,0.86357e+03_rb,0.72739e+03_rb,0.59707e+03_rb, &
        & 0.69145e+03_rb,0.81829e+03_rb,0.94626e+03_rb,0.10746e+04_rb /)
      kao(:, 2,12, 7) = (/ &
        & 0.11253e+04_rb,0.99054e+03_rb,0.85571e+03_rb,0.72085e+03_rb,0.59339e+03_rb, &
        & 0.69172e+03_rb,0.81863e+03_rb,0.94675e+03_rb,0.10752e+04_rb /)
      kao(:, 3,12, 7) = (/ &
        & 0.11163e+04_rb,0.98265e+03_rb,0.84895e+03_rb,0.71521e+03_rb,0.59030e+03_rb, &
        & 0.69172e+03_rb,0.81873e+03_rb,0.94686e+03_rb,0.10754e+04_rb /)
      kao(:, 4,12, 7) = (/ &
        & 0.11085e+04_rb,0.97579e+03_rb,0.84307e+03_rb,0.71031e+03_rb,0.58782e+03_rb, &
        & 0.69148e+03_rb,0.81855e+03_rb,0.94672e+03_rb,0.10753e+04_rb /)
      kao(:, 5,12, 7) = (/ &
        & 0.11016e+04_rb,0.96977e+03_rb,0.83790e+03_rb,0.70601e+03_rb,0.58538e+03_rb, &
        & 0.69099e+03_rb,0.81812e+03_rb,0.94628e+03_rb,0.10749e+04_rb /)
      kao(:, 1,13, 7) = (/ &
        & 0.11104e+04_rb,0.97748e+03_rb,0.84452e+03_rb,0.71152e+03_rb,0.58852e+03_rb, &
        & 0.69093e+03_rb,0.81799e+03_rb,0.94612e+03_rb,0.10746e+04_rb /)
      kao(:, 2,13, 7) = (/ &
        & 0.11018e+04_rb,0.96998e+03_rb,0.83809e+03_rb,0.70616e+03_rb,0.58570e+03_rb, &
        & 0.69123e+03_rb,0.81841e+03_rb,0.94664e+03_rb,0.10752e+04_rb /)
      kao(:, 3,13, 7) = (/ &
        & 0.10945e+04_rb,0.96352e+03_rb,0.83255e+03_rb,0.70155e+03_rb,0.58307e+03_rb, &
        & 0.69128e+03_rb,0.81849e+03_rb,0.94675e+03_rb,0.10754e+04_rb /)
      kao(:, 4,13, 7) = (/ &
        & 0.10880e+04_rb,0.95790e+03_rb,0.82774e+03_rb,0.69753e+03_rb,0.58045e+03_rb, &
        & 0.69107e+03_rb,0.81832e+03_rb,0.94657e+03_rb,0.10753e+04_rb /)
      kao(:, 5,13, 7) = (/ &
        & 0.10824e+04_rb,0.95297e+03_rb,0.82351e+03_rb,0.69401e+03_rb,0.57806e+03_rb, &
        & 0.69061e+03_rb,0.81788e+03_rb,0.94614e+03_rb,0.10749e+04_rb /)
      kao(:, 1, 1, 8) = (/ &
        & 0.19433e+04_rb,0.17047e+04_rb,0.14661e+04_rb,0.12275e+04_rb,0.98902e+03_rb, &
        & 0.75120e+03_rb,0.85754e+03_rb,0.98845e+03_rb,0.11206e+04_rb /)
      kao(:, 2, 1, 8) = (/ &
        & 0.18941e+04_rb,0.16616e+04_rb,0.14292e+04_rb,0.11968e+04_rb,0.96439e+03_rb, &
        & 0.73548e+03_rb,0.85594e+03_rb,0.98707e+03_rb,0.11193e+04_rb /)
      kao(:, 3, 1, 8) = (/ &
        & 0.18499e+04_rb,0.16229e+04_rb,0.13960e+04_rb,0.11691e+04_rb,0.94229e+03_rb, &
        & 0.72448e+03_rb,0.85413e+03_rb,0.98549e+03_rb,0.11177e+04_rb /)
      kao(:, 4, 1, 8) = (/ &
        & 0.18099e+04_rb,0.15880e+04_rb,0.13661e+04_rb,0.11442e+04_rb,0.92233e+03_rb, &
        & 0.72225e+03_rb,0.85214e+03_rb,0.98373e+03_rb,0.11158e+04_rb /)
      kao(:, 5, 1, 8) = (/ &
        & 0.17737e+04_rb,0.15563e+04_rb,0.13389e+04_rb,0.11215e+04_rb,0.90421e+03_rb, &
        & 0.72001e+03_rb,0.85005e+03_rb,0.98175e+03_rb,0.11138e+04_rb /)
      kao(:, 1, 2, 8) = (/ &
        & 0.17953e+04_rb,0.15751e+04_rb,0.13551e+04_rb,0.11350e+04_rb,0.91499e+03_rb, &
        & 0.72569e+03_rb,0.85649e+03_rb,0.98821e+03_rb,0.11210e+04_rb /)
      kao(:, 2, 2, 8) = (/ &
        & 0.17530e+04_rb,0.15382e+04_rb,0.13234e+04_rb,0.11086e+04_rb,0.89388e+03_rb, &
        & 0.72424e+03_rb,0.85509e+03_rb,0.98698e+03_rb,0.11198e+04_rb /)
      kao(:, 3, 2, 8) = (/ &
        & 0.17152e+04_rb,0.15051e+04_rb,0.12950e+04_rb,0.10850e+04_rb,0.87497e+03_rb, &
        & 0.72267e+03_rb,0.85348e+03_rb,0.98553e+03_rb,0.11184e+04_rb /)
      kao(:, 4, 2, 8) = (/ &
        & 0.16811e+04_rb,0.14753e+04_rb,0.12695e+04_rb,0.10637e+04_rb,0.85793e+03_rb, &
        & 0.72084e+03_rb,0.85163e+03_rb,0.98389e+03_rb,0.11166e+04_rb /)
      kao(:, 5, 2, 8) = (/ &
        & 0.16502e+04_rb,0.14483e+04_rb,0.12463e+04_rb,0.10444e+04_rb,0.84250e+03_rb, &
        & 0.71877e+03_rb,0.84969e+03_rb,0.98204e+03_rb,0.11147e+04_rb /)
      kao(:, 1, 3, 8) = (/ &
        & 0.16788e+04_rb,0.14733e+04_rb,0.12678e+04_rb,0.10623e+04_rb,0.85679e+03_rb, &
        & 0.72434e+03_rb,0.85588e+03_rb,0.98822e+03_rb,0.11216e+04_rb /)
      kao(:, 2, 3, 8) = (/ &
        & 0.16418e+04_rb,0.14409e+04_rb,0.12400e+04_rb,0.10391e+04_rb,0.83827e+03_rb, &
        & 0.72306e+03_rb,0.85466e+03_rb,0.98716e+03_rb,0.11206e+04_rb /)
      kao(:, 3, 3, 8) = (/ &
        & 0.16087e+04_rb,0.14119e+04_rb,0.12152e+04_rb,0.10184e+04_rb,0.82172e+03_rb, &
        & 0.72165e+03_rb,0.85327e+03_rb,0.98590e+03_rb,0.11193e+04_rb /)
      kao(:, 4, 3, 8) = (/ &
        & 0.15790e+04_rb,0.13859e+04_rb,0.11929e+04_rb,0.99988e+03_rb,0.80688e+03_rb, &
        & 0.72007e+03_rb,0.85161e+03_rb,0.98438e+03_rb,0.11177e+04_rb /)
      kao(:, 5, 3, 8) = (/ &
        & 0.15522e+04_rb,0.13625e+04_rb,0.11728e+04_rb,0.98311e+03_rb,0.79349e+03_rb, &
        & 0.71824e+03_rb,0.84982e+03_rb,0.98270e+03_rb,0.11159e+04_rb /)
      kao(:, 1, 4, 8) = (/ &
        & 0.15823e+04_rb,0.13888e+04_rb,0.11953e+04_rb,0.10019e+04_rb,0.80851e+03_rb, &
        & 0.72326e+03_rb,0.85540e+03_rb,0.98823e+03_rb,0.11221e+04_rb /)
      kao(:, 2, 4, 8) = (/ &
        & 0.15495e+04_rb,0.13601e+04_rb,0.11708e+04_rb,0.98142e+03_rb,0.79213e+03_rb, &
        & 0.72214e+03_rb,0.85436e+03_rb,0.98737e+03_rb,0.11213e+04_rb /)
      kao(:, 3, 4, 8) = (/ &
        & 0.15204e+04_rb,0.13346e+04_rb,0.11489e+04_rb,0.96322e+03_rb,0.77758e+03_rb, &
        & 0.72091e+03_rb,0.85317e+03_rb,0.98628e+03_rb,0.11202e+04_rb /)
      kao(:, 4, 4, 8) = (/ &
        & 0.14943e+04_rb,0.13118e+04_rb,0.11294e+04_rb,0.94694e+03_rb,0.76455e+03_rb, &
        & 0.71952e+03_rb,0.85175e+03_rb,0.98496e+03_rb,0.11188e+04_rb /)
      kao(:, 5, 4, 8) = (/ &
        & 0.14709e+04_rb,0.12913e+04_rb,0.11118e+04_rb,0.93229e+03_rb,0.75284e+03_rb, &
        & 0.71792e+03_rb,0.85010e+03_rb,0.98342e+03_rb,0.11171e+04_rb /)
      kao(:, 1, 5, 8) = (/ &
        & 0.15002e+04_rb,0.13170e+04_rb,0.11338e+04_rb,0.95063e+03_rb,0.76749e+03_rb, &
        & 0.72234e+03_rb,0.85495e+03_rb,0.98821e+03_rb,0.11224e+04_rb /)
      kao(:, 2, 5, 8) = (/ &
        & 0.14712e+04_rb,0.12916e+04_rb,0.11120e+04_rb,0.93250e+03_rb,0.75300e+03_rb, &
        & 0.72139e+03_rb,0.85409e+03_rb,0.98753e+03_rb,0.11218e+04_rb /)
      kao(:, 3, 5, 8) = (/ &
        & 0.14456e+04_rb,0.12692e+04_rb,0.10928e+04_rb,0.91647e+03_rb,0.74017e+03_rb, &
        & 0.72031e+03_rb,0.85307e+03_rb,0.98657e+03_rb,0.11209e+04_rb /)
      kao(:, 4, 5, 8) = (/ &
        & 0.14227e+04_rb,0.12492e+04_rb,0.10757e+04_rb,0.90219e+03_rb,0.72875e+03_rb, &
        & 0.71909e+03_rb,0.85186e+03_rb,0.98546e+03_rb,0.11197e+04_rb /)
      kao(:, 5, 5, 8) = (/ &
        & 0.14022e+04_rb,0.12312e+04_rb,0.10603e+04_rb,0.88937e+03_rb,0.71851e+03_rb, &
        & 0.71769e+03_rb,0.85037e+03_rb,0.98406e+03_rb,0.11182e+04_rb /)
      kao(:, 1, 6, 8) = (/ &
        & 0.14314e+04_rb,0.12568e+04_rb,0.10822e+04_rb,0.90763e+03_rb,0.73310e+03_rb, &
        & 0.72155e+03_rb,0.85452e+03_rb,0.98813e+03_rb,0.11226e+04_rb /)
      kao(:, 2, 6, 8) = (/ &
        & 0.14056e+04_rb,0.12342e+04_rb,0.10628e+04_rb,0.89150e+03_rb,0.72020e+03_rb, &
        & 0.72077e+03_rb,0.85386e+03_rb,0.98765e+03_rb,0.11222e+04_rb /)
      kao(:, 3, 6, 8) = (/ &
        & 0.13829e+04_rb,0.12143e+04_rb,0.10458e+04_rb,0.87730e+03_rb,0.70887e+03_rb, &
        & 0.71986e+03_rb,0.85300e+03_rb,0.98688e+03_rb,0.11215e+04_rb /)
      kao(:, 4, 6, 8) = (/ &
        & 0.13627e+04_rb,0.11967e+04_rb,0.10307e+04_rb,0.86473e+03_rb,0.69880e+03_rb, &
        & 0.71879e+03_rb,0.85197e+03_rb,0.98590e+03_rb,0.11205e+04_rb /)
      kao(:, 5, 6, 8) = (/ &
        & 0.13448e+04_rb,0.11810e+04_rb,0.10172e+04_rb,0.85348e+03_rb,0.68979e+03_rb, &
        & 0.71757e+03_rb,0.85073e+03_rb,0.98469e+03_rb,0.11192e+04_rb /)
      kao(:, 1, 7, 8) = (/ &
        & 0.13728e+04_rb,0.12055e+04_rb,0.10383e+04_rb,0.87102e+03_rb,0.70382e+03_rb, &
        & 0.72081e+03_rb,0.85410e+03_rb,0.98799e+03_rb,0.11227e+04_rb /)
      kao(:, 2, 7, 8) = (/ &
        & 0.13499e+04_rb,0.11854e+04_rb,0.10210e+04_rb,0.85666e+03_rb,0.69233e+03_rb, &
        & 0.72022e+03_rb,0.85362e+03_rb,0.98768e+03_rb,0.11225e+04_rb /)
      kao(:, 3, 7, 8) = (/ &
        & 0.13297e+04_rb,0.11678e+04_rb,0.10059e+04_rb,0.84408e+03_rb,0.68228e+03_rb, &
        & 0.71945e+03_rb,0.85293e+03_rb,0.98711e+03_rb,0.11220e+04_rb /)
      kao(:, 4, 7, 8) = (/ &
        & 0.13120e+04_rb,0.11523e+04_rb,0.99261e+03_rb,0.83298e+03_rb,0.67339e+03_rb, &
        & 0.71854e+03_rb,0.85205e+03_rb,0.98632e+03_rb,0.11212e+04_rb /)
      kao(:, 5, 7, 8) = (/ &
        & 0.12962e+04_rb,0.11384e+04_rb,0.98077e+03_rb,0.82312e+03_rb,0.66550e+03_rb, &
        & 0.71746e+03_rb,0.85098e+03_rb,0.98523e+03_rb,0.11200e+04_rb /)
      kao(:, 1, 8, 8) = (/ &
        & 0.13227e+04_rb,0.11617e+04_rb,0.10007e+04_rb,0.83971e+03_rb,0.67877e+03_rb, &
        & 0.72007e+03_rb,0.85366e+03_rb,0.98779e+03_rb,0.11227e+04_rb /)
      kao(:, 2, 8, 8) = (/ &
        & 0.13022e+04_rb,0.11438e+04_rb,0.98533e+03_rb,0.82690e+03_rb,0.66854e+03_rb, &
        & 0.71969e+03_rb,0.85334e+03_rb,0.98763e+03_rb,0.11227e+04_rb /)
      kao(:, 3, 8, 8) = (/ &
        & 0.12844e+04_rb,0.11282e+04_rb,0.97196e+03_rb,0.81577e+03_rb,0.65963e+03_rb, &
        & 0.71908e+03_rb,0.85283e+03_rb,0.98717e+03_rb,0.11223e+04_rb /)
      kao(:, 4, 8, 8) = (/ &
        & 0.12688e+04_rb,0.11145e+04_rb,0.96023e+03_rb,0.80599e+03_rb,0.65182e+03_rb, &
        & 0.71830e+03_rb,0.85210e+03_rb,0.98659e+03_rb,0.11217e+04_rb /)
      kao(:, 5, 8, 8) = (/ &
        & 0.12549e+04_rb,0.11024e+04_rb,0.94982e+03_rb,0.79733e+03_rb,0.64488e+03_rb, &
        & 0.71736e+03_rb,0.85118e+03_rb,0.98569e+03_rb,0.11208e+04_rb /)
      kao(:, 1, 9, 8) = (/ &
        & 0.12800e+04_rb,0.11244e+04_rb,0.96868e+03_rb,0.81302e+03_rb,0.65744e+03_rb, &
        & 0.71936e+03_rb,0.85323e+03_rb,0.98761e+03_rb,0.11226e+04_rb /)
      kao(:, 2, 9, 8) = (/ &
        & 0.12618e+04_rb,0.11084e+04_rb,0.95498e+03_rb,0.80161e+03_rb,0.64832e+03_rb, &
        & 0.71916e+03_rb,0.85306e+03_rb,0.98751e+03_rb,0.11227e+04_rb /)
      kao(:, 3, 9, 8) = (/ &
        & 0.12460e+04_rb,0.10945e+04_rb,0.94313e+03_rb,0.79175e+03_rb,0.64042e+03_rb, &
        & 0.71874e+03_rb,0.85269e+03_rb,0.98731e+03_rb,0.11226e+04_rb /)
      kao(:, 4, 9, 8) = (/ &
        & 0.12322e+04_rb,0.10825e+04_rb,0.93277e+03_rb,0.78311e+03_rb,0.63351e+03_rb, &
        & 0.71809e+03_rb,0.85213e+03_rb,0.98681e+03_rb,0.11221e+04_rb /)
      kao(:, 5, 9, 8) = (/ &
        & 0.12200e+04_rb,0.10718e+04_rb,0.92365e+03_rb,0.77552e+03_rb,0.62743e+03_rb, &
        & 0.71728e+03_rb,0.85134e+03_rb,0.98607e+03_rb,0.11214e+04_rb /)
      kao(:, 1,10, 8) = (/ &
        & 0.12423e+04_rb,0.10913e+04_rb,0.94034e+03_rb,0.78942e+03_rb,0.63856e+03_rb, &
        & 0.71867e+03_rb,0.85279e+03_rb,0.98738e+03_rb,0.11225e+04_rb /)
      kao(:, 2,10, 8) = (/ &
        & 0.12262e+04_rb,0.10772e+04_rb,0.92827e+03_rb,0.77936e+03_rb,0.63052e+03_rb, &
        & 0.71863e+03_rb,0.85274e+03_rb,0.98738e+03_rb,0.11227e+04_rb /)
      kao(:, 3,10, 8) = (/ &
        & 0.12123e+04_rb,0.10651e+04_rb,0.91788e+03_rb,0.77071e+03_rb,0.62359e+03_rb, &
        & 0.71835e+03_rb,0.85251e+03_rb,0.98725e+03_rb,0.11227e+04_rb /)
      kao(:, 4,10, 8) = (/ &
        & 0.12002e+04_rb,0.10545e+04_rb,0.90884e+03_rb,0.76317e+03_rb,0.61756e+03_rb, &
        & 0.71785e+03_rb,0.85203e+03_rb,0.98691e+03_rb,0.11224e+04_rb /)
      kao(:, 5,10, 8) = (/ &
        & 0.11897e+04_rb,0.10453e+04_rb,0.90090e+03_rb,0.75655e+03_rb,0.61227e+03_rb, &
        & 0.71716e+03_rb,0.85141e+03_rb,0.98631e+03_rb,0.11218e+04_rb /)
      kao(:, 1,11, 8) = (/ &
        & 0.12033e+04_rb,0.10572e+04_rb,0.91110e+03_rb,0.76506e+03_rb,0.61908e+03_rb, &
        & 0.71809e+03_rb,0.85242e+03_rb,0.98721e+03_rb,0.11225e+04_rb /)
      kao(:, 2,11, 8) = (/ &
        & 0.11901e+04_rb,0.10456e+04_rb,0.90123e+03_rb,0.75682e+03_rb,0.61248e+03_rb, &
        & 0.71809e+03_rb,0.85240e+03_rb,0.98723e+03_rb,0.11226e+04_rb /)
      kao(:, 3,11, 8) = (/ &
        & 0.11788e+04_rb,0.10357e+04_rb,0.89271e+03_rb,0.74972e+03_rb,0.60692e+03_rb, &
        & 0.71785e+03_rb,0.85217e+03_rb,0.98712e+03_rb,0.11227e+04_rb /)
      kao(:, 4,11, 8) = (/ &
        & 0.11689e+04_rb,0.10271e+04_rb,0.88529e+03_rb,0.74355e+03_rb,0.60253e+03_rb, &
        & 0.71738e+03_rb,0.85177e+03_rb,0.98681e+03_rb,0.11224e+04_rb /)
      kao(:, 5,11, 8) = (/ &
        & 0.11602e+04_rb,0.10195e+04_rb,0.87880e+03_rb,0.73813e+03_rb,0.59855e+03_rb, &
        & 0.71672e+03_rb,0.85114e+03_rb,0.98618e+03_rb,0.11218e+04_rb /)
      kao(:, 1,12, 8) = (/ &
        & 0.11713e+04_rb,0.10292e+04_rb,0.88712e+03_rb,0.74507e+03_rb,0.60379e+03_rb, &
        & 0.71766e+03_rb,0.85212e+03_rb,0.98706e+03_rb,0.11225e+04_rb /)
      kao(:, 2,12, 8) = (/ &
        & 0.11605e+04_rb,0.10197e+04_rb,0.87902e+03_rb,0.73833e+03_rb,0.59891e+03_rb, &
        & 0.71764e+03_rb,0.85212e+03_rb,0.98710e+03_rb,0.11227e+04_rb /)
      kao(:, 3,12, 8) = (/ &
        & 0.11512e+04_rb,0.10116e+04_rb,0.87205e+03_rb,0.73251e+03_rb,0.59502e+03_rb, &
        & 0.71744e+03_rb,0.85192e+03_rb,0.98700e+03_rb,0.11227e+04_rb /)
      kao(:, 4,12, 8) = (/ &
        & 0.11431e+04_rb,0.10045e+04_rb,0.86598e+03_rb,0.72746e+03_rb,0.59151e+03_rb, &
        & 0.71699e+03_rb,0.85153e+03_rb,0.98663e+03_rb,0.11224e+04_rb /)
      kao(:, 5,12, 8) = (/ &
        & 0.11360e+04_rb,0.99832e+03_rb,0.86067e+03_rb,0.72302e+03_rb,0.58868e+03_rb, &
        & 0.71636e+03_rb,0.85092e+03_rb,0.98606e+03_rb,0.11218e+04_rb /)
      kao(:, 1,13, 8) = (/ &
        & 0.11451e+04_rb,0.10063e+04_rb,0.86748e+03_rb,0.72871e+03_rb,0.59258e+03_rb, &
        & 0.71726e+03_rb,0.85187e+03_rb,0.98696e+03_rb,0.11225e+04_rb /)
      kao(:, 2,13, 8) = (/ &
        & 0.11363e+04_rb,0.99855e+03_rb,0.86085e+03_rb,0.72320e+03_rb,0.58896e+03_rb, &
        & 0.71729e+03_rb,0.85183e+03_rb,0.98699e+03_rb,0.11227e+04_rb /)
      kao(:, 3,13, 8) = (/ &
        & 0.11287e+04_rb,0.99189e+03_rb,0.85515e+03_rb,0.71842e+03_rb,0.58585e+03_rb, &
        & 0.71710e+03_rb,0.85169e+03_rb,0.98691e+03_rb,0.11227e+04_rb /)
      kao(:, 4,13, 8) = (/ &
        & 0.11220e+04_rb,0.98609e+03_rb,0.85017e+03_rb,0.71429e+03_rb,0.58380e+03_rb, &
        & 0.71668e+03_rb,0.85133e+03_rb,0.98660e+03_rb,0.11224e+04_rb /)
      kao(:, 5,13, 8) = (/ &
        & 0.11162e+04_rb,0.98101e+03_rb,0.84582e+03_rb,0.71065e+03_rb,0.58234e+03_rb, &
        & 0.71606e+03_rb,0.85074e+03_rb,0.98601e+03_rb,0.11218e+04_rb /)
      kao(:, 1, 1, 9) = (/ &
        & 0.19571e+04_rb,0.17167e+04_rb,0.14761e+04_rb,0.12357e+04_rb,0.99528e+03_rb, &
        & 0.75488e+03_rb,0.87315e+03_rb,0.10086e+04_rb,0.11441e+04_rb /)
      kao(:, 2, 1, 9) = (/ &
        & 0.19076e+04_rb,0.16733e+04_rb,0.14390e+04_rb,0.12047e+04_rb,0.97044e+03_rb, &
        & 0.73841e+03_rb,0.87146e+03_rb,0.10070e+04_rb,0.11425e+04_rb /)
      kao(:, 3, 1, 9) = (/ &
        & 0.18630e+04_rb,0.16343e+04_rb,0.14056e+04_rb,0.11769e+04_rb,0.94822e+03_rb, &
        & 0.73460e+03_rb,0.86967e+03_rb,0.10052e+04_rb,0.11407e+04_rb /)
      kao(:, 4, 1, 9) = (/ &
        & 0.18229e+04_rb,0.15991e+04_rb,0.13754e+04_rb,0.11517e+04_rb,0.92811e+03_rb, &
        & 0.73269e+03_rb,0.86788e+03_rb,0.10033e+04_rb,0.11387e+04_rb /)
      kao(:, 5, 1, 9) = (/ &
        & 0.17864e+04_rb,0.15672e+04_rb,0.13481e+04_rb,0.11289e+04_rb,0.90984e+03_rb, &
        & 0.73091e+03_rb,0.86616e+03_rb,0.10016e+04_rb,0.11368e+04_rb /)
      kao(:, 1, 2, 9) = (/ &
        & 0.18080e+04_rb,0.15862e+04_rb,0.13643e+04_rb,0.11425e+04_rb,0.92072e+03_rb, &
        & 0.73694e+03_rb,0.87246e+03_rb,0.10086e+04_rb,0.11448e+04_rb /)
      kao(:, 2, 2, 9) = (/ &
        & 0.17655e+04_rb,0.15491e+04_rb,0.13324e+04_rb,0.11159e+04_rb,0.89943e+03_rb, &
        & 0.73525e+03_rb,0.87092e+03_rb,0.10070e+04_rb,0.11432e+04_rb /)
      kao(:, 3, 2, 9) = (/ &
        & 0.17274e+04_rb,0.15157e+04_rb,0.13039e+04_rb,0.10921e+04_rb,0.88039e+03_rb, &
        & 0.73343e+03_rb,0.86922e+03_rb,0.10053e+04_rb,0.11415e+04_rb /)
      kao(:, 4, 2, 9) = (/ &
        & 0.16931e+04_rb,0.14856e+04_rb,0.12781e+04_rb,0.10706e+04_rb,0.86326e+03_rb, &
        & 0.73166e+03_rb,0.86752e+03_rb,0.10036e+04_rb,0.11395e+04_rb /)
      kao(:, 5, 2, 9) = (/ &
        & 0.16621e+04_rb,0.14584e+04_rb,0.12549e+04_rb,0.10512e+04_rb,0.84768e+03_rb, &
        & 0.73000e+03_rb,0.86583e+03_rb,0.10018e+04_rb,0.11376e+04_rb /)
      kao(:, 1, 3, 9) = (/ &
        & 0.16908e+04_rb,0.14837e+04_rb,0.12764e+04_rb,0.10692e+04_rb,0.86209e+03_rb, &
        & 0.73591e+03_rb,0.87213e+03_rb,0.10088e+04_rb,0.11456e+04_rb /)
      kao(:, 2, 3, 9) = (/ &
        & 0.16535e+04_rb,0.14509e+04_rb,0.12484e+04_rb,0.10459e+04_rb,0.84339e+03_rb, &
        & 0.73450e+03_rb,0.87080e+03_rb,0.10075e+04_rb,0.11442e+04_rb /)
      kao(:, 3, 3, 9) = (/ &
        & 0.16202e+04_rb,0.14218e+04_rb,0.12234e+04_rb,0.10251e+04_rb,0.82680e+03_rb, &
        & 0.73291e+03_rb,0.86926e+03_rb,0.10059e+04_rb,0.11426e+04_rb /)
      kao(:, 4, 3, 9) = (/ &
        & 0.15902e+04_rb,0.13956e+04_rb,0.12011e+04_rb,0.10064e+04_rb,0.81183e+03_rb, &
        & 0.73120e+03_rb,0.86761e+03_rb,0.10042e+04_rb,0.11407e+04_rb /)
      kao(:, 5, 3, 9) = (/ &
        & 0.15632e+04_rb,0.13720e+04_rb,0.11808e+04_rb,0.98952e+03_rb,0.79832e+03_rb, &
        & 0.72957e+03_rb,0.86594e+03_rb,0.10024e+04_rb,0.11387e+04_rb /)
      kao(:, 1, 4, 9) = (/ &
        & 0.15935e+04_rb,0.13985e+04_rb,0.12035e+04_rb,0.10084e+04_rb,0.81344e+03_rb, &
        & 0.73513e+03_rb,0.87195e+03_rb,0.10092e+04_rb,0.11463e+04_rb /)
      kao(:, 2, 4, 9) = (/ &
        & 0.15605e+04_rb,0.13697e+04_rb,0.11787e+04_rb,0.98782e+03_rb,0.79694e+03_rb, &
        & 0.73394e+03_rb,0.87080e+03_rb,0.10079e+04_rb,0.11451e+04_rb /)
      kao(:, 3, 4, 9) = (/ &
        & 0.15313e+04_rb,0.13439e+04_rb,0.11567e+04_rb,0.96950e+03_rb,0.78229e+03_rb, &
        & 0.73257e+03_rb,0.86942e+03_rb,0.10065e+04_rb,0.11436e+04_rb /)
      kao(:, 4, 4, 9) = (/ &
        & 0.15050e+04_rb,0.13210e+04_rb,0.11371e+04_rb,0.95308e+03_rb,0.76922e+03_rb, &
        & 0.73104e+03_rb,0.86787e+03_rb,0.10049e+04_rb,0.11419e+04_rb /)
      kao(:, 5, 4, 9) = (/ &
        & 0.14814e+04_rb,0.13003e+04_rb,0.11193e+04_rb,0.93833e+03_rb,0.75745e+03_rb, &
        & 0.72946e+03_rb,0.86622e+03_rb,0.10032e+04_rb,0.11400e+04_rb /)
      kao(:, 1, 5, 9) = (/ &
        & 0.15109e+04_rb,0.13262e+04_rb,0.11415e+04_rb,0.95685e+03_rb,0.77217e+03_rb, &
        & 0.73443e+03_rb,0.87177e+03_rb,0.10094e+04_rb,0.11470e+04_rb /)
      kao(:, 2, 5, 9) = (/ &
        & 0.14817e+04_rb,0.13007e+04_rb,0.11196e+04_rb,0.93852e+03_rb,0.75755e+03_rb, &
        & 0.73343e+03_rb,0.87078e+03_rb,0.10083e+04_rb,0.11459e+04_rb /)
      kao(:, 3, 5, 9) = (/ &
        & 0.14559e+04_rb,0.12780e+04_rb,0.11003e+04_rb,0.92240e+03_rb,0.74463e+03_rb, &
        & 0.73229e+03_rb,0.86958e+03_rb,0.10071e+04_rb,0.11446e+04_rb /)
      kao(:, 4, 5, 9) = (/ &
        & 0.14329e+04_rb,0.12579e+04_rb,0.10829e+04_rb,0.90802e+03_rb,0.73310e+03_rb, &
        & 0.73095e+03_rb,0.86817e+03_rb,0.10056e+04_rb,0.11430e+04_rb /)
      kao(:, 5, 5, 9) = (/ &
        & 0.14122e+04_rb,0.12398e+04_rb,0.10675e+04_rb,0.89511e+03_rb,0.72281e+03_rb, &
        & 0.72946e+03_rb,0.86664e+03_rb,0.10039e+04_rb,0.11412e+04_rb /)
      kao(:, 1, 6, 9) = (/ &
        & 0.14417e+04_rb,0.12655e+04_rb,0.10896e+04_rb,0.91348e+03_rb,0.73747e+03_rb, &
        & 0.73386e+03_rb,0.87165e+03_rb,0.10096e+04_rb,0.11475e+04_rb /)
      kao(:, 2, 6, 9) = (/ &
        & 0.14156e+04_rb,0.12429e+04_rb,0.10700e+04_rb,0.89725e+03_rb,0.72457e+03_rb, &
        & 0.73304e+03_rb,0.87078e+03_rb,0.10087e+04_rb,0.11466e+04_rb /)
      kao(:, 3, 6, 9) = (/ &
        & 0.13928e+04_rb,0.12228e+04_rb,0.10529e+04_rb,0.88300e+03_rb,0.71308e+03_rb, &
        & 0.73207e+03_rb,0.86977e+03_rb,0.10076e+04_rb,0.11455e+04_rb /)
      kao(:, 4, 6, 9) = (/ &
        & 0.13725e+04_rb,0.12051e+04_rb,0.10377e+04_rb,0.87029e+03_rb,0.70290e+03_rb, &
        & 0.73093e+03_rb,0.86853e+03_rb,0.10063e+04_rb,0.11441e+04_rb /)
      kao(:, 5, 6, 9) = (/ &
        & 0.13544e+04_rb,0.11892e+04_rb,0.10241e+04_rb,0.85897e+03_rb,0.69390e+03_rb, &
        & 0.72961e+03_rb,0.86706e+03_rb,0.10048e+04_rb,0.11424e+04_rb /)
      kao(:, 1, 7, 9) = (/ &
        & 0.13826e+04_rb,0.12140e+04_rb,0.10453e+04_rb,0.87664e+03_rb,0.70803e+03_rb, &
        & 0.73339e+03_rb,0.87155e+03_rb,0.10098e+04_rb,0.11480e+04_rb /)
      kao(:, 2, 7, 9) = (/ &
        & 0.13594e+04_rb,0.11937e+04_rb,0.10279e+04_rb,0.86216e+03_rb,0.69645e+03_rb, &
        & 0.73269e+03_rb,0.87079e+03_rb,0.10090e+04_rb,0.11472e+04_rb /)
      kao(:, 3, 7, 9) = (/ &
        & 0.13392e+04_rb,0.11759e+04_rb,0.10127e+04_rb,0.84953e+03_rb,0.68632e+03_rb, &
        & 0.73188e+03_rb,0.86990e+03_rb,0.10081e+04_rb,0.11462e+04_rb /)
      kao(:, 4, 7, 9) = (/ &
        & 0.13213e+04_rb,0.11603e+04_rb,0.99928e+03_rb,0.83829e+03_rb,0.67737e+03_rb, &
        & 0.73090e+03_rb,0.86884e+03_rb,0.10069e+04_rb,0.11450e+04_rb /)
      kao(:, 5, 7, 9) = (/ &
        & 0.13054e+04_rb,0.11464e+04_rb,0.98739e+03_rb,0.82837e+03_rb,0.66946e+03_rb, &
        & 0.72975e+03_rb,0.86756e+03_rb,0.10055e+04_rb,0.11435e+04_rb /)
      kao(:, 1, 8, 9) = (/ &
        & 0.13322e+04_rb,0.11698e+04_rb,0.10074e+04_rb,0.84511e+03_rb,0.68280e+03_rb, &
        & 0.73302e+03_rb,0.87145e+03_rb,0.10099e+04_rb,0.11483e+04_rb /)
      kao(:, 2, 8, 9) = (/ &
        & 0.13116e+04_rb,0.11517e+04_rb,0.99199e+03_rb,0.83221e+03_rb,0.67250e+03_rb, &
        & 0.73244e+03_rb,0.87079e+03_rb,0.10093e+04_rb,0.11477e+04_rb /)
      kao(:, 3, 8, 9) = (/ &
        & 0.12937e+04_rb,0.11360e+04_rb,0.97848e+03_rb,0.82097e+03_rb,0.66349e+03_rb, &
        & 0.73170e+03_rb,0.87002e+03_rb,0.10085e+04_rb,0.11469e+04_rb /)
      kao(:, 4, 8, 9) = (/ &
        & 0.12778e+04_rb,0.11223e+04_rb,0.96666e+03_rb,0.81113e+03_rb,0.65562e+03_rb, &
        & 0.73087e+03_rb,0.86909e+03_rb,0.10075e+04_rb,0.11458e+04_rb /)
      kao(:, 5, 8, 9) = (/ &
        & 0.12639e+04_rb,0.11101e+04_rb,0.95620e+03_rb,0.80244e+03_rb,0.64865e+03_rb, &
        & 0.72985e+03_rb,0.86796e+03_rb,0.10062e+04_rb,0.11444e+04_rb /)
      kao(:, 1, 9, 9) = (/ &
        & 0.12892e+04_rb,0.11322e+04_rb,0.97520e+03_rb,0.81821e+03_rb,0.66134e+03_rb, &
        & 0.73274e+03_rb,0.87135e+03_rb,0.10099e+04_rb,0.11485e+04_rb /)
      kao(:, 2, 9, 9) = (/ &
        & 0.12708e+04_rb,0.11161e+04_rb,0.96140e+03_rb,0.80674e+03_rb,0.65213e+03_rb, &
        & 0.73213e+03_rb,0.87079e+03_rb,0.10095e+04_rb,0.11481e+04_rb /)
      kao(:, 3, 9, 9) = (/ &
        & 0.12548e+04_rb,0.11022e+04_rb,0.94949e+03_rb,0.79681e+03_rb,0.64421e+03_rb, &
        & 0.73152e+03_rb,0.87011e+03_rb,0.10088e+04_rb,0.11474e+04_rb /)
      kao(:, 4, 9, 9) = (/ &
        & 0.12410e+04_rb,0.10900e+04_rb,0.93905e+03_rb,0.78808e+03_rb,0.63726e+03_rb, &
        & 0.73083e+03_rb,0.86928e+03_rb,0.10079e+04_rb,0.11464e+04_rb /)
      kao(:, 5, 9, 9) = (/ &
        & 0.12287e+04_rb,0.10793e+04_rb,0.92981e+03_rb,0.78042e+03_rb,0.63106e+03_rb, &
        & 0.72995e+03_rb,0.86831e+03_rb,0.10068e+04_rb,0.11452e+04_rb /)
      kao(:, 1,10, 9) = (/ &
        & 0.12511e+04_rb,0.10989e+04_rb,0.94665e+03_rb,0.79449e+03_rb,0.64227e+03_rb, &
        & 0.73248e+03_rb,0.87121e+03_rb,0.10099e+04_rb,0.11487e+04_rb /)
      kao(:, 2,10, 9) = (/ &
        & 0.12350e+04_rb,0.10848e+04_rb,0.93449e+03_rb,0.78435e+03_rb,0.63416e+03_rb, &
        & 0.73188e+03_rb,0.87073e+03_rb,0.10095e+04_rb,0.11483e+04_rb /)
      kao(:, 3,10, 9) = (/ &
        & 0.12209e+04_rb,0.10725e+04_rb,0.92407e+03_rb,0.77565e+03_rb,0.62724e+03_rb, &
        & 0.73132e+03_rb,0.87012e+03_rb,0.10091e+04_rb,0.11477e+04_rb /)
      kao(:, 4,10, 9) = (/ &
        & 0.12088e+04_rb,0.10618e+04_rb,0.91493e+03_rb,0.76801e+03_rb,0.62113e+03_rb, &
        & 0.73072e+03_rb,0.86943e+03_rb,0.10082e+04_rb,0.11469e+04_rb /)
      kao(:, 5,10, 9) = (/ &
        & 0.11981e+04_rb,0.10526e+04_rb,0.90691e+03_rb,0.76134e+03_rb,0.61583e+03_rb, &
        & 0.72997e+03_rb,0.86853e+03_rb,0.10072e+04_rb,0.11458e+04_rb /)
      kao(:, 1,11, 9) = (/ &
        & 0.12119e+04_rb,0.10646e+04_rb,0.91723e+03_rb,0.76995e+03_rb,0.62265e+03_rb, &
        & 0.73201e+03_rb,0.87093e+03_rb,0.10098e+04_rb,0.11487e+04_rb /)
      kao(:, 2,11, 9) = (/ &
        & 0.11986e+04_rb,0.10529e+04_rb,0.90727e+03_rb,0.76161e+03_rb,0.61603e+03_rb, &
        & 0.73147e+03_rb,0.87046e+03_rb,0.10094e+04_rb,0.11483e+04_rb /)
      kao(:, 3,11, 9) = (/ &
        & 0.11871e+04_rb,0.10429e+04_rb,0.89865e+03_rb,0.75449e+03_rb,0.61031e+03_rb, &
        & 0.73094e+03_rb,0.86988e+03_rb,0.10088e+04_rb,0.11477e+04_rb /)
      kao(:, 4,11, 9) = (/ &
        & 0.11772e+04_rb,0.10342e+04_rb,0.89122e+03_rb,0.74824e+03_rb,0.60537e+03_rb, &
        & 0.73035e+03_rb,0.86918e+03_rb,0.10081e+04_rb,0.11469e+04_rb /)
      kao(:, 5,11, 9) = (/ &
        & 0.11684e+04_rb,0.10265e+04_rb,0.88467e+03_rb,0.74278e+03_rb,0.60127e+03_rb, &
        & 0.72963e+03_rb,0.86831e+03_rb,0.10071e+04_rb,0.11458e+04_rb /)
      kao(:, 1,12, 9) = (/ &
        & 0.11796e+04_rb,0.10363e+04_rb,0.89308e+03_rb,0.74980e+03_rb,0.60664e+03_rb, &
        & 0.73158e+03_rb,0.87066e+03_rb,0.10097e+04_rb,0.11487e+04_rb /)
      kao(:, 2,12, 9) = (/ &
        & 0.11688e+04_rb,0.10268e+04_rb,0.88492e+03_rb,0.74297e+03_rb,0.60173e+03_rb, &
        & 0.73111e+03_rb,0.87023e+03_rb,0.10093e+04_rb,0.11483e+04_rb /)
      kao(:, 3,12, 9) = (/ &
        & 0.11594e+04_rb,0.10186e+04_rb,0.87787e+03_rb,0.73715e+03_rb,0.59711e+03_rb, &
        & 0.73061e+03_rb,0.86968e+03_rb,0.10088e+04_rb,0.11478e+04_rb /)
      kao(:, 4,12, 9) = (/ &
        & 0.11512e+04_rb,0.10115e+04_rb,0.87180e+03_rb,0.73203e+03_rb,0.59373e+03_rb, &
        & 0.73004e+03_rb,0.86900e+03_rb,0.10080e+04_rb,0.11469e+04_rb /)
      kao(:, 5,12, 9) = (/ &
        & 0.11441e+04_rb,0.10053e+04_rb,0.86640e+03_rb,0.72756e+03_rb,0.59149e+03_rb, &
        & 0.72934e+03_rb,0.86813e+03_rb,0.10070e+04_rb,0.11458e+04_rb /)
      kao(:, 1,13, 9) = (/ &
        & 0.11533e+04_rb,0.10132e+04_rb,0.87330e+03_rb,0.73334e+03_rb,0.59476e+03_rb, &
        & 0.73130e+03_rb,0.87046e+03_rb,0.10096e+04_rb,0.11487e+04_rb /)
      kao(:, 2,13, 9) = (/ &
        & 0.11444e+04_rb,0.10055e+04_rb,0.86659e+03_rb,0.72770e+03_rb,0.59240e+03_rb, &
        & 0.73084e+03_rb,0.87005e+03_rb,0.10092e+04_rb,0.11483e+04_rb /)
      kao(:, 3,13, 9) = (/ &
        & 0.11367e+04_rb,0.99882e+03_rb,0.86084e+03_rb,0.72297e+03_rb,0.59186e+03_rb, &
        & 0.73036e+03_rb,0.86950e+03_rb,0.10087e+04_rb,0.11478e+04_rb /)
      kao(:, 4,13, 9) = (/ &
        & 0.11300e+04_rb,0.99294e+03_rb,0.85585e+03_rb,0.71880e+03_rb,0.59143e+03_rb, &
        & 0.72979e+03_rb,0.86883e+03_rb,0.10079e+04_rb,0.11469e+04_rb /)
      kao(:, 5,13, 9) = (/ &
        & 0.11242e+04_rb,0.98788e+03_rb,0.85146e+03_rb,0.71512e+03_rb,0.59082e+03_rb, &
        & 0.72910e+03_rb,0.86797e+03_rb,0.10069e+04_rb,0.11459e+04_rb /)
      kao(:, 1, 1,10) = (/ &
        & 0.19609e+04_rb,0.17196e+04_rb,0.14787e+04_rb,0.12373e+04_rb,0.99636e+03_rb, &
        & 0.75577e+03_rb,0.88158e+03_rb,0.10175e+04_rb,0.11538e+04_rb /)
      kao(:, 2, 1,10) = (/ &
        & 0.19102e+04_rb,0.16765e+04_rb,0.14403e+04_rb,0.12063e+04_rb,0.97232e+03_rb, &
        & 0.74406e+03_rb,0.87986e+03_rb,0.10152e+04_rb,0.11513e+04_rb /)
      kao(:, 3, 1,10) = (/ &
        & 0.18658e+04_rb,0.16364e+04_rb,0.14075e+04_rb,0.11783e+04_rb,0.94916e+03_rb, &
        & 0.74249e+03_rb,0.87815e+03_rb,0.10135e+04_rb,0.11491e+04_rb /)
      kao(:, 4, 1,10) = (/ &
        & 0.18250e+04_rb,0.16014e+04_rb,0.13778e+04_rb,0.11533e+04_rb,0.92945e+03_rb, &
        & 0.74050e+03_rb,0.87640e+03_rb,0.10114e+04_rb,0.11477e+04_rb /)
      kao(:, 5, 1,10) = (/ &
        & 0.17888e+04_rb,0.15692e+04_rb,0.13504e+04_rb,0.11313e+04_rb,0.91134e+03_rb, &
        & 0.73837e+03_rb,0.87432e+03_rb,0.10096e+04_rb,0.11467e+04_rb /)
      kao(:, 1, 2,10) = (/ &
        & 0.18108e+04_rb,0.15879e+04_rb,0.13673e+04_rb,0.11437e+04_rb,0.92213e+03_rb, &
        & 0.74428e+03_rb,0.88077e+03_rb,0.10175e+04_rb,0.11547e+04_rb /)
      kao(:, 2, 2,10) = (/ &
        & 0.17684e+04_rb,0.15508e+04_rb,0.13352e+04_rb,0.11178e+04_rb,0.90099e+03_rb, &
        & 0.74265e+03_rb,0.87920e+03_rb,0.10158e+04_rb,0.11521e+04_rb /)
      kao(:, 3, 2,10) = (/ &
        & 0.17293e+04_rb,0.15183e+04_rb,0.13063e+04_rb,0.10931e+04_rb,0.88199e+03_rb, &
        & 0.74117e+03_rb,0.87755e+03_rb,0.10136e+04_rb,0.11500e+04_rb /)
      kao(:, 4, 2,10) = (/ &
        & 0.16960e+04_rb,0.14887e+04_rb,0.12799e+04_rb,0.10727e+04_rb,0.86433e+03_rb, &
        & 0.73957e+03_rb,0.87577e+03_rb,0.10116e+04_rb,0.11482e+04_rb /)
      kao(:, 5, 2,10) = (/ &
        & 0.16637e+04_rb,0.14607e+04_rb,0.12562e+04_rb,0.10527e+04_rb,0.84877e+03_rb, &
        & 0.73748e+03_rb,0.87409e+03_rb,0.10096e+04_rb,0.11470e+04_rb /)
      kao(:, 1, 3,10) = (/ &
        & 0.16939e+04_rb,0.14847e+04_rb,0.12786e+04_rb,0.10705e+04_rb,0.86372e+03_rb, &
        & 0.74343e+03_rb,0.88064e+03_rb,0.10178e+04_rb,0.11559e+04_rb /)
      kao(:, 2, 3,10) = (/ &
        & 0.16557e+04_rb,0.14542e+04_rb,0.12503e+04_rb,0.10478e+04_rb,0.84533e+03_rb, &
        & 0.74182e+03_rb,0.87906e+03_rb,0.10163e+04_rb,0.11537e+04_rb /)
      kao(:, 3, 3,10) = (/ &
        & 0.16218e+04_rb,0.14239e+04_rb,0.12249e+04_rb,0.10266e+04_rb,0.82819e+03_rb, &
        & 0.74031e+03_rb,0.87730e+03_rb,0.10137e+04_rb,0.11513e+04_rb /)
      kao(:, 4, 3,10) = (/ &
        & 0.15929e+04_rb,0.13992e+04_rb,0.12027e+04_rb,0.10077e+04_rb,0.81299e+03_rb, &
        & 0.73896e+03_rb,0.87590e+03_rb,0.10123e+04_rb,0.11493e+04_rb /)
      kao(:, 5, 3,10) = (/ &
        & 0.15654e+04_rb,0.13740e+04_rb,0.11824e+04_rb,0.99136e+03_rb,0.79933e+03_rb, &
        & 0.73740e+03_rb,0.87406e+03_rb,0.10104e+04_rb,0.11477e+04_rb /)
      kao(:, 1, 4,10) = (/ &
        & 0.15959e+04_rb,0.14010e+04_rb,0.12054e+04_rb,0.10100e+04_rb,0.81522e+03_rb, &
        & 0.74271e+03_rb,0.88040e+03_rb,0.10180e+04_rb,0.11570e+04_rb /)
      kao(:, 2, 4,10) = (/ &
        & 0.15637e+04_rb,0.13710e+04_rb,0.11804e+04_rb,0.98920e+03_rb,0.79803e+03_rb, &
        & 0.74139e+03_rb,0.87903e+03_rb,0.10168e+04_rb,0.11552e+04_rb /)
      kao(:, 3, 4,10) = (/ &
        & 0.15330e+04_rb,0.13459e+04_rb,0.11594e+04_rb,0.97120e+03_rb,0.78314e+03_rb, &
        & 0.74000e+03_rb,0.87753e+03_rb,0.10151e+04_rb,0.11528e+04_rb /)
      kao(:, 4, 4,10) = (/ &
        & 0.15077e+04_rb,0.13229e+04_rb,0.11386e+04_rb,0.95486e+03_rb,0.77029e+03_rb, &
        & 0.73854e+03_rb,0.87600e+03_rb,0.10131e+04_rb,0.11506e+04_rb /)
      kao(:, 5, 4,10) = (/ &
        & 0.14835e+04_rb,0.13025e+04_rb,0.11209e+04_rb,0.94000e+03_rb,0.75786e+03_rb, &
        & 0.73717e+03_rb,0.87438e+03_rb,0.10111e+04_rb,0.11486e+04_rb /)
      kao(:, 1, 5,10) = (/ &
        & 0.15126e+04_rb,0.13285e+04_rb,0.11431e+04_rb,0.95745e+03_rb,0.77284e+03_rb, &
        & 0.74220e+03_rb,0.88024e+03_rb,0.10187e+04_rb,0.11578e+04_rb /)
      kao(:, 2, 5,10) = (/ &
        & 0.14829e+04_rb,0.13024e+04_rb,0.11209e+04_rb,0.94020e+03_rb,0.75863e+03_rb, &
        & 0.74099e+03_rb,0.87909e+03_rb,0.10174e+04_rb,0.11564e+04_rb /)
      kao(:, 3, 5,10) = (/ &
        & 0.14582e+04_rb,0.12806e+04_rb,0.11014e+04_rb,0.92377e+03_rb,0.74596e+03_rb, &
        & 0.73960e+03_rb,0.87772e+03_rb,0.10159e+04_rb,0.11544e+04_rb /)
      kao(:, 4, 5,10) = (/ &
        & 0.14345e+04_rb,0.12594e+04_rb,0.10847e+04_rb,0.90887e+03_rb,0.73410e+03_rb, &
        & 0.73828e+03_rb,0.87622e+03_rb,0.10139e+04_rb,0.11518e+04_rb /)
      kao(:, 5, 5,10) = (/ &
        & 0.14140e+04_rb,0.12418e+04_rb,0.10683e+04_rb,0.89661e+03_rb,0.72364e+03_rb, &
        & 0.73696e+03_rb,0.87469e+03_rb,0.10119e+04_rb,0.11501e+04_rb /)
      kao(:, 1, 6,10) = (/ &
        & 0.14435e+04_rb,0.12676e+04_rb,0.10914e+04_rb,0.91493e+03_rb,0.73889e+03_rb, &
        & 0.74179e+03_rb,0.88024e+03_rb,0.10190e+04_rb,0.11586e+04_rb /)
      kao(:, 2, 6,10) = (/ &
        & 0.14184e+04_rb,0.12443e+04_rb,0.10722e+04_rb,0.89834e+03_rb,0.72535e+03_rb, &
        & 0.74069e+03_rb,0.87922e+03_rb,0.10178e+04_rb,0.11574e+04_rb /)
      kao(:, 3, 6,10) = (/ &
        & 0.13949e+04_rb,0.12245e+04_rb,0.10548e+04_rb,0.88411e+03_rb,0.71441e+03_rb, &
        & 0.73951e+03_rb,0.87792e+03_rb,0.10164e+04_rb,0.11554e+04_rb /)
      kao(:, 4, 6,10) = (/ &
        & 0.13745e+04_rb,0.12070e+04_rb,0.10388e+04_rb,0.87157e+03_rb,0.70381e+03_rb, &
        & 0.73817e+03_rb,0.87654e+03_rb,0.10150e+04_rb,0.11533e+04_rb /)
      kao(:, 5, 6,10) = (/ &
        & 0.13558e+04_rb,0.11909e+04_rb,0.10253e+04_rb,0.86039e+03_rb,0.69505e+03_rb, &
        & 0.73669e+03_rb,0.87508e+03_rb,0.10128e+04_rb,0.11510e+04_rb /)
      kao(:, 1, 7,10) = (/ &
        & 0.13849e+04_rb,0.12154e+04_rb,0.10465e+04_rb,0.87809e+03_rb,0.70922e+03_rb, &
        & 0.74150e+03_rb,0.88017e+03_rb,0.10197e+04_rb,0.11599e+04_rb /)
      kao(:, 2, 7,10) = (/ &
        & 0.13622e+04_rb,0.11954e+04_rb,0.10296e+04_rb,0.86328e+03_rb,0.69750e+03_rb, &
        & 0.74031e+03_rb,0.87926e+03_rb,0.10184e+04_rb,0.11581e+04_rb /)
      kao(:, 3, 7,10) = (/ &
        & 0.13407e+04_rb,0.11775e+04_rb,0.10144e+04_rb,0.85098e+03_rb,0.68710e+03_rb, &
        & 0.73933e+03_rb,0.87824e+03_rb,0.10171e+04_rb,0.11568e+04_rb /)
      kao(:, 4, 7,10) = (/ &
        & 0.13234e+04_rb,0.11620e+04_rb,0.10006e+04_rb,0.83940e+03_rb,0.67808e+03_rb, &
        & 0.73818e+03_rb,0.87693e+03_rb,0.10157e+04_rb,0.11549e+04_rb /)
      kao(:, 5, 7,10) = (/ &
        & 0.13077e+04_rb,0.11481e+04_rb,0.98850e+03_rb,0.82952e+03_rb,0.66976e+03_rb, &
        & 0.73685e+03_rb,0.87533e+03_rb,0.10140e+04_rb,0.11525e+04_rb /)
      kao(:, 1, 8,10) = (/ &
        & 0.13336e+04_rb,0.11711e+04_rb,0.10088e+04_rb,0.84626e+03_rb,0.68382e+03_rb, &
        & 0.74131e+03_rb,0.88025e+03_rb,0.10205e+04_rb,0.11609e+04_rb /)
      kao(:, 2, 8,10) = (/ &
        & 0.13133e+04_rb,0.11535e+04_rb,0.99382e+03_rb,0.83328e+03_rb,0.67355e+03_rb, &
        & 0.73954e+03_rb,0.87931e+03_rb,0.10187e+04_rb,0.11591e+04_rb /)
      kao(:, 3, 8,10) = (/ &
        & 0.12952e+04_rb,0.11377e+04_rb,0.97980e+03_rb,0.82257e+03_rb,0.66450e+03_rb, &
        & 0.73910e+03_rb,0.87831e+03_rb,0.10177e+04_rb,0.11576e+04_rb /)
      kao(:, 4, 8,10) = (/ &
        & 0.12798e+04_rb,0.11238e+04_rb,0.96848e+03_rb,0.81189e+03_rb,0.65672e+03_rb, &
        & 0.73820e+03_rb,0.87729e+03_rb,0.10162e+04_rb,0.11560e+04_rb /)
      kao(:, 5, 8,10) = (/ &
        & 0.12652e+04_rb,0.11115e+04_rb,0.95739e+03_rb,0.80373e+03_rb,0.64938e+03_rb, &
        & 0.73712e+03_rb,0.87584e+03_rb,0.10149e+04_rb,0.11541e+04_rb /)
      kao(:, 1, 9,10) = (/ &
        & 0.12921e+04_rb,0.11338e+04_rb,0.97649e+03_rb,0.81965e+03_rb,0.66193e+03_rb, &
        & 0.74117e+03_rb,0.88056e+03_rb,0.10212e+04_rb,0.11621e+04_rb /)
      kao(:, 2, 9,10) = (/ &
        & 0.12725e+04_rb,0.11175e+04_rb,0.96294e+03_rb,0.80792e+03_rb,0.65314e+03_rb, &
        & 0.74018e+03_rb,0.87944e+03_rb,0.10196e+04_rb,0.11602e+04_rb /)
      kao(:, 3, 9,10) = (/ &
        & 0.12569e+04_rb,0.11032e+04_rb,0.95038e+03_rb,0.79825e+03_rb,0.64471e+03_rb, &
        & 0.73910e+03_rb,0.87852e+03_rb,0.10181e+04_rb,0.11585e+04_rb /)
      kao(:, 4, 9,10) = (/ &
        & 0.12428e+04_rb,0.10914e+04_rb,0.94028e+03_rb,0.78940e+03_rb,0.63769e+03_rb, &
        & 0.73816e+03_rb,0.87758e+03_rb,0.10170e+04_rb,0.11569e+04_rb /)
      kao(:, 5, 9,10) = (/ &
        & 0.12310e+04_rb,0.10815e+04_rb,0.93149e+03_rb,0.78151e+03_rb,0.63218e+03_rb, &
        & 0.73726e+03_rb,0.87637e+03_rb,0.10156e+04_rb,0.11553e+04_rb /)
      kao(:, 1,10,10) = (/ &
        & 0.12531e+04_rb,0.11009e+04_rb,0.94808e+03_rb,0.79569e+03_rb,0.64353e+03_rb, &
        & 0.74092e+03_rb,0.88101e+03_rb,0.10222e+04_rb,0.11631e+04_rb /)
      kao(:, 2,10,10) = (/ &
        & 0.12357e+04_rb,0.10861e+04_rb,0.93599e+03_rb,0.78490e+03_rb,0.63564e+03_rb, &
        & 0.74008e+03_rb,0.87961e+03_rb,0.10203e+04_rb,0.11613e+04_rb /)
      kao(:, 3,10,10) = (/ &
        & 0.12227e+04_rb,0.10738e+04_rb,0.92501e+03_rb,0.77680e+03_rb,0.62842e+03_rb, &
        & 0.73914e+03_rb,0.87860e+03_rb,0.10179e+04_rb,0.11592e+04_rb /)
      kao(:, 4,10,10) = (/ &
        & 0.12109e+04_rb,0.10632e+04_rb,0.91605e+03_rb,0.76920e+03_rb,0.62177e+03_rb, &
        & 0.73819e+03_rb,0.87758e+03_rb,0.10174e+04_rb,0.11577e+04_rb /)
      kao(:, 5,10,10) = (/ &
        & 0.11998e+04_rb,0.10537e+04_rb,0.90813e+03_rb,0.76232e+03_rb,0.61616e+03_rb, &
        & 0.73732e+03_rb,0.87676e+03_rb,0.10162e+04_rb,0.11563e+04_rb /)
      kao(:, 1,11,10) = (/ &
        & 0.12137e+04_rb,0.10654e+04_rb,0.91884e+03_rb,0.77105e+03_rb,0.62358e+03_rb, &
        & 0.74042e+03_rb,0.88057e+03_rb,0.10221e+04_rb,0.11630e+04_rb /)
      kao(:, 2,11,10) = (/ &
        & 0.11999e+04_rb,0.10545e+04_rb,0.90854e+03_rb,0.76253e+03_rb,0.61715e+03_rb, &
        & 0.73961e+03_rb,0.87932e+03_rb,0.10201e+04_rb,0.11611e+04_rb /)
      kao(:, 3,11,10) = (/ &
        & 0.11890e+04_rb,0.10445e+04_rb,0.90019e+03_rb,0.75537e+03_rb,0.61109e+03_rb, &
        & 0.73864e+03_rb,0.87832e+03_rb,0.10186e+04_rb,0.11592e+04_rb /)
      kao(:, 4,11,10) = (/ &
        & 0.11789e+04_rb,0.10359e+04_rb,0.89280e+03_rb,0.74928e+03_rb,0.60622e+03_rb, &
        & 0.73779e+03_rb,0.87747e+03_rb,0.10172e+04_rb,0.11577e+04_rb /)
      kao(:, 5,11,10) = (/ &
        & 0.11709e+04_rb,0.10287e+04_rb,0.88607e+03_rb,0.74401e+03_rb,0.60247e+03_rb, &
        & 0.73688e+03_rb,0.87644e+03_rb,0.10160e+04_rb,0.11563e+04_rb /)
      kao(:, 1,12,10) = (/ &
        & 0.11820e+04_rb,0.10383e+04_rb,0.89456e+03_rb,0.75081e+03_rb,0.60714e+03_rb, &
        & 0.74001e+03_rb,0.88049e+03_rb,0.10220e+04_rb,0.11631e+04_rb /)
      kao(:, 2,12,10) = (/ &
        & 0.11713e+04_rb,0.10290e+04_rb,0.88623e+03_rb,0.74407e+03_rb,0.60058e+03_rb, &
        & 0.73921e+03_rb,0.87901e+03_rb,0.10201e+04_rb,0.11611e+04_rb /)
      kao(:, 3,12,10) = (/ &
        & 0.11605e+04_rb,0.10200e+04_rb,0.87951e+03_rb,0.73788e+03_rb,0.59769e+03_rb, &
        & 0.73835e+03_rb,0.87808e+03_rb,0.10182e+04_rb,0.11591e+04_rb /)
      kao(:, 4,12,10) = (/ &
        & 0.11533e+04_rb,0.10132e+04_rb,0.87284e+03_rb,0.73305e+03_rb,0.59795e+03_rb, &
        & 0.73753e+03_rb,0.87718e+03_rb,0.10171e+04_rb,0.11577e+04_rb /)
      kao(:, 5,12,10) = (/ &
        & 0.11467e+04_rb,0.10071e+04_rb,0.86739e+03_rb,0.72890e+03_rb,0.59711e+03_rb, &
        & 0.73664e+03_rb,0.87623e+03_rb,0.10159e+04_rb,0.11563e+04_rb /)
      kao(:, 1,13,10) = (/ &
        & 0.11553e+04_rb,0.10149e+04_rb,0.87419e+03_rb,0.73470e+03_rb,0.59975e+03_rb, &
        & 0.73968e+03_rb,0.88029e+03_rb,0.10219e+04_rb,0.11630e+04_rb /)
      kao(:, 2,13,10) = (/ &
        & 0.11455e+04_rb,0.10072e+04_rb,0.86782e+03_rb,0.72873e+03_rb,0.59921e+03_rb, &
        & 0.73889e+03_rb,0.87889e+03_rb,0.10200e+04_rb,0.11611e+04_rb /)
      kao(:, 3,13,10) = (/ &
        & 0.11380e+04_rb,0.10000e+04_rb,0.86190e+03_rb,0.72452e+03_rb,0.59831e+03_rb, &
        & 0.73799e+03_rb,0.87789e+03_rb,0.10182e+04_rb,0.11592e+04_rb /)
      kao(:, 4,13,10) = (/ &
        & 0.11318e+04_rb,0.99472e+03_rb,0.85697e+03_rb,0.71950e+03_rb,0.59744e+03_rb, &
        & 0.73726e+03_rb,0.87708e+03_rb,0.10170e+04_rb,0.11577e+04_rb /)
      kao(:, 5,13,10) = (/ &
        & 0.11253e+04_rb,0.98825e+03_rb,0.85273e+03_rb,0.71635e+03_rb,0.59683e+03_rb, &
        & 0.73639e+03_rb,0.87606e+03_rb,0.10160e+04_rb,0.11562e+04_rb /)
      kao(:, 1, 1,11) = (/ &
        & 0.19598e+04_rb,0.17186e+04_rb,0.14777e+04_rb,0.12380e+04_rb,0.99729e+03_rb, &
        & 0.75567e+03_rb,0.88469e+03_rb,0.10202e+04_rb,0.11560e+04_rb /)
      kao(:, 2, 1,11) = (/ &
        & 0.19102e+04_rb,0.16743e+04_rb,0.14414e+04_rb,0.12066e+04_rb,0.97117e+03_rb, &
        & 0.74679e+03_rb,0.88252e+03_rb,0.10178e+04_rb,0.11544e+04_rb /)
      kao(:, 3, 1,11) = (/ &
        & 0.18656e+04_rb,0.16383e+04_rb,0.14073e+04_rb,0.11791e+04_rb,0.94951e+03_rb, &
        & 0.74505e+03_rb,0.88029e+03_rb,0.10159e+04_rb,0.11530e+04_rb /)
      kao(:, 4, 1,11) = (/ &
        & 0.18261e+04_rb,0.16025e+04_rb,0.13782e+04_rb,0.11540e+04_rb,0.92917e+03_rb, &
        & 0.74299e+03_rb,0.87816e+03_rb,0.10141e+04_rb,0.11522e+04_rb /)
      kao(:, 5, 1,11) = (/ &
        & 0.17888e+04_rb,0.15710e+04_rb,0.13500e+04_rb,0.11298e+04_rb,0.91140e+03_rb, &
        & 0.74109e+03_rb,0.87588e+03_rb,0.10122e+04_rb,0.11510e+04_rb /)
      kao(:, 1, 2,11) = (/ &
        & 0.18117e+04_rb,0.15887e+04_rb,0.13651e+04_rb,0.11459e+04_rb,0.92177e+03_rb, &
        & 0.74731e+03_rb,0.88412e+03_rb,0.10203e+04_rb,0.11568e+04_rb /)
      kao(:, 2, 2,11) = (/ &
        & 0.17673e+04_rb,0.15512e+04_rb,0.13335e+04_rb,0.11174e+04_rb,0.90028e+03_rb, &
        & 0.74546e+03_rb,0.88203e+03_rb,0.10179e+04_rb,0.11551e+04_rb /)
      kao(:, 3, 2,11) = (/ &
        & 0.17317e+04_rb,0.15193e+04_rb,0.13050e+04_rb,0.10955e+04_rb,0.88112e+03_rb, &
        & 0.74378e+03_rb,0.87978e+03_rb,0.10161e+04_rb,0.11535e+04_rb /)
      kao(:, 4, 2,11) = (/ &
        & 0.16956e+04_rb,0.14873e+04_rb,0.12805e+04_rb,0.10729e+04_rb,0.86470e+03_rb, &
        & 0.74189e+03_rb,0.87768e+03_rb,0.10142e+04_rb,0.11524e+04_rb /)
      kao(:, 5, 2,11) = (/ &
        & 0.16657e+04_rb,0.14596e+04_rb,0.12568e+04_rb,0.10531e+04_rb,0.84904e+03_rb, &
        & 0.73979e+03_rb,0.87528e+03_rb,0.10122e+04_rb,0.11511e+04_rb /)
      kao(:, 1, 3,11) = (/ &
        & 0.16927e+04_rb,0.14859e+04_rb,0.12782e+04_rb,0.10724e+04_rb,0.86335e+03_rb, &
        & 0.74646e+03_rb,0.88392e+03_rb,0.10211e+04_rb,0.11579e+04_rb /)
      kao(:, 2, 3,11) = (/ &
        & 0.16581e+04_rb,0.14524e+04_rb,0.12540e+04_rb,0.10466e+04_rb,0.84448e+03_rb, &
        & 0.74471e+03_rb,0.88205e+03_rb,0.10189e+04_rb,0.11562e+04_rb /)
      kao(:, 3, 3,11) = (/ &
        & 0.16261e+04_rb,0.14229e+04_rb,0.12269e+04_rb,0.10264e+04_rb,0.82790e+03_rb, &
        & 0.74305e+03_rb,0.87988e+03_rb,0.10167e+04_rb,0.11544e+04_rb /)
      kao(:, 4, 3,11) = (/ &
        & 0.15929e+04_rb,0.13986e+04_rb,0.12038e+04_rb,0.10084e+04_rb,0.81266e+03_rb, &
        & 0.74160e+03_rb,0.87767e+03_rb,0.10148e+04_rb,0.11530e+04_rb /)
      kao(:, 5, 3,11) = (/ &
        & 0.15661e+04_rb,0.13746e+04_rb,0.11837e+04_rb,0.99103e+03_rb,0.79967e+03_rb, &
        & 0.73929e+03_rb,0.87586e+03_rb,0.10132e+04_rb,0.11521e+04_rb /)
      kao(:, 1, 4,11) = (/ &
        & 0.15973e+04_rb,0.14004e+04_rb,0.12065e+04_rb,0.10106e+04_rb,0.81385e+03_rb, &
        & 0.74599e+03_rb,0.88404e+03_rb,0.10218e+04_rb,0.11590e+04_rb /)
      kao(:, 2, 4,11) = (/ &
        & 0.15637e+04_rb,0.13722e+04_rb,0.11812e+04_rb,0.98900e+03_rb,0.79856e+03_rb, &
        & 0.74428e+03_rb,0.88217e+03_rb,0.10197e+04_rb,0.11572e+04_rb /)
      kao(:, 3, 4,11) = (/ &
        & 0.15340e+04_rb,0.13469e+04_rb,0.11576e+04_rb,0.97012e+03_rb,0.78412e+03_rb, &
        & 0.74257e+03_rb,0.88022e+03_rb,0.10176e+04_rb,0.11556e+04_rb /)
      kao(:, 4, 4,11) = (/ &
        & 0.15085e+04_rb,0.13226e+04_rb,0.11393e+04_rb,0.95486e+03_rb,0.76980e+03_rb, &
        & 0.74114e+03_rb,0.87815e+03_rb,0.10155e+04_rb,0.11538e+04_rb /)
      kao(:, 5, 4,11) = (/ &
        & 0.14836e+04_rb,0.13020e+04_rb,0.11212e+04_rb,0.93957e+03_rb,0.75824e+03_rb, &
        & 0.73939e+03_rb,0.87630e+03_rb,0.10138e+04_rb,0.11526e+04_rb /)
      kao(:, 1, 5,11) = (/ &
        & 0.15143e+04_rb,0.13286e+04_rb,0.11431e+04_rb,0.95847e+03_rb,0.77403e+03_rb, &
        & 0.74572e+03_rb,0.88418e+03_rb,0.10223e+04_rb,0.11603e+04_rb /)
      kao(:, 2, 5,11) = (/ &
        & 0.14842e+04_rb,0.13025e+04_rb,0.11230e+04_rb,0.94028e+03_rb,0.75881e+03_rb, &
        & 0.74405e+03_rb,0.88254e+03_rb,0.10205e+04_rb,0.11584e+04_rb /)
      kao(:, 3, 5,11) = (/ &
        & 0.14577e+04_rb,0.12799e+04_rb,0.11020e+04_rb,0.92340e+03_rb,0.74531e+03_rb, &
        & 0.74244e+03_rb,0.88060e+03_rb,0.10185e+04_rb,0.11566e+04_rb /)
      kao(:, 4, 5,11) = (/ &
        & 0.14365e+04_rb,0.12603e+04_rb,0.10859e+04_rb,0.90939e+03_rb,0.73467e+03_rb, &
        & 0.74090e+03_rb,0.87865e+03_rb,0.10164e+04_rb,0.11549e+04_rb /)
      kao(:, 5, 5,11) = (/ &
        & 0.14147e+04_rb,0.12422e+04_rb,0.10695e+04_rb,0.89657e+03_rb,0.72409e+03_rb, &
        & 0.73938e+03_rb,0.87659e+03_rb,0.10145e+04_rb,0.11530e+04_rb /)
      kao(:, 1, 6,11) = (/ &
        & 0.14426e+04_rb,0.12686e+04_rb,0.10911e+04_rb,0.91528e+03_rb,0.73914e+03_rb, &
        & 0.74563e+03_rb,0.88436e+03_rb,0.10228e+04_rb,0.11615e+04_rb /)
      kao(:, 2, 6,11) = (/ &
        & 0.14171e+04_rb,0.12457e+04_rb,0.10728e+04_rb,0.89861e+03_rb,0.72517e+03_rb, &
        & 0.74402e+03_rb,0.88280e+03_rb,0.10215e+04_rb,0.11596e+04_rb /)
      kao(:, 3, 6,11) = (/ &
        & 0.13943e+04_rb,0.12252e+04_rb,0.10539e+04_rb,0.88362e+03_rb,0.71333e+03_rb, &
        & 0.74243e+03_rb,0.88117e+03_rb,0.10194e+04_rb,0.11577e+04_rb /)
      kao(:, 4, 6,11) = (/ &
        & 0.13749e+04_rb,0.12056e+04_rb,0.10405e+04_rb,0.87188e+03_rb,0.70449e+03_rb, &
        & 0.74089e+03_rb,0.87923e+03_rb,0.10173e+04_rb,0.11560e+04_rb /)
      kao(:, 5, 6,11) = (/ &
        & 0.13580e+04_rb,0.11907e+04_rb,0.10258e+04_rb,0.85998e+03_rb,0.69473e+03_rb, &
        & 0.73937e+03_rb,0.87727e+03_rb,0.10154e+04_rb,0.11543e+04_rb /)
      kao(:, 1, 7,11) = (/ &
        & 0.13844e+04_rb,0.12166e+04_rb,0.10469e+04_rb,0.87815e+03_rb,0.70873e+03_rb, &
        & 0.74560e+03_rb,0.88449e+03_rb,0.10233e+04_rb,0.11626e+04_rb /)
      kao(:, 2, 7,11) = (/ &
        & 0.13610e+04_rb,0.11962e+04_rb,0.10291e+04_rb,0.86318e+03_rb,0.69730e+03_rb, &
        & 0.74409e+03_rb,0.88322e+03_rb,0.10221e+04_rb,0.11608e+04_rb /)
      kao(:, 3, 7,11) = (/ &
        & 0.13413e+04_rb,0.11786e+04_rb,0.10145e+04_rb,0.85031e+03_rb,0.68760e+03_rb, &
        & 0.74253e+03_rb,0.88154e+03_rb,0.10203e+04_rb,0.11589e+04_rb /)
      kao(:, 4, 7,11) = (/ &
        & 0.13235e+04_rb,0.11630e+04_rb,0.10007e+04_rb,0.84004e+03_rb,0.67864e+03_rb, &
        & 0.74101e+03_rb,0.87982e+03_rb,0.10185e+04_rb,0.11572e+04_rb /)
      kao(:, 5, 7,11) = (/ &
        & 0.13063e+04_rb,0.11482e+04_rb,0.98932e+03_rb,0.82998e+03_rb,0.67037e+03_rb, &
        & 0.73955e+03_rb,0.87796e+03_rb,0.10167e+04_rb,0.11554e+04_rb /)
      kao(:, 1, 8,11) = (/ &
        & 0.13361e+04_rb,0.11714e+04_rb,0.10092e+04_rb,0.84627e+03_rb,0.68340e+03_rb, &
        & 0.74540e+03_rb,0.88457e+03_rb,0.10236e+04_rb,0.11638e+04_rb /)
      kao(:, 2, 8,11) = (/ &
        & 0.13143e+04_rb,0.11537e+04_rb,0.99295e+03_rb,0.83355e+03_rb,0.67330e+03_rb, &
        & 0.74420e+03_rb,0.88339e+03_rb,0.10225e+04_rb,0.11619e+04_rb /)
      kao(:, 3, 8,11) = (/ &
        & 0.12952e+04_rb,0.11377e+04_rb,0.98075e+03_rb,0.82213e+03_rb,0.66496e+03_rb, &
        & 0.74271e+03_rb,0.88199e+03_rb,0.10211e+04_rb,0.11601e+04_rb /)
      kao(:, 4, 8,11) = (/ &
        & 0.12808e+04_rb,0.11242e+04_rb,0.96858e+03_rb,0.81350e+03_rb,0.65663e+03_rb, &
        & 0.74121e+03_rb,0.88037e+03_rb,0.10192e+04_rb,0.11582e+04_rb /)
      kao(:, 5, 8,11) = (/ &
        & 0.12664e+04_rb,0.11113e+04_rb,0.95757e+03_rb,0.80317e+03_rb,0.64978e+03_rb, &
        & 0.73975e+03_rb,0.87862e+03_rb,0.10173e+04_rb,0.11564e+04_rb /)
      kao(:, 1, 9,11) = (/ &
        & 0.12911e+04_rb,0.11331e+04_rb,0.97754e+03_rb,0.81972e+03_rb,0.66210e+03_rb, &
        & 0.74501e+03_rb,0.88462e+03_rb,0.10244e+04_rb,0.11652e+04_rb /)
      kao(:, 2, 9,11) = (/ &
        & 0.12743e+04_rb,0.11193e+04_rb,0.96288e+03_rb,0.80865e+03_rb,0.65325e+03_rb, &
        & 0.74437e+03_rb,0.88360e+03_rb,0.10230e+04_rb,0.11629e+04_rb /)
      kao(:, 3, 9,11) = (/ &
        & 0.12573e+04_rb,0.11051e+04_rb,0.95155e+03_rb,0.79729e+03_rb,0.64451e+03_rb, &
        & 0.74294e+03_rb,0.88237e+03_rb,0.10218e+04_rb,0.11610e+04_rb /)
      kao(:, 4, 9,11) = (/ &
        & 0.12441e+04_rb,0.10920e+04_rb,0.94089e+03_rb,0.78911e+03_rb,0.63797e+03_rb, &
        & 0.74148e+03_rb,0.88091e+03_rb,0.10202e+04_rb,0.11593e+04_rb /)
      kao(:, 5, 9,11) = (/ &
        & 0.12297e+04_rb,0.10813e+04_rb,0.93127e+03_rb,0.78127e+03_rb,0.63265e+03_rb, &
        & 0.74003e+03_rb,0.87926e+03_rb,0.10184e+04_rb,0.11573e+04_rb /)
      kao(:, 1,10,11) = (/ &
        & 0.12530e+04_rb,0.11001e+04_rb,0.94821e+03_rb,0.79486e+03_rb,0.64277e+03_rb, &
        & 0.74454e+03_rb,0.88447e+03_rb,0.10249e+04_rb,0.11664e+04_rb /)
      kao(:, 2,10,11) = (/ &
        & 0.12387e+04_rb,0.10865e+04_rb,0.93685e+03_rb,0.78643e+03_rb,0.63425e+03_rb, &
        & 0.74423e+03_rb,0.88371e+03_rb,0.10234e+04_rb,0.11637e+04_rb /)
      kao(:, 3,10,11) = (/ &
        & 0.12243e+04_rb,0.10744e+04_rb,0.92585e+03_rb,0.77676e+03_rb,0.62740e+03_rb, &
        & 0.74307e+03_rb,0.88260e+03_rb,0.10221e+04_rb,0.11620e+04_rb /)
      kao(:, 4,10,11) = (/ &
        & 0.12107e+04_rb,0.10640e+04_rb,0.91623e+03_rb,0.76957e+03_rb,0.62235e+03_rb, &
        & 0.74168e+03_rb,0.88129e+03_rb,0.10208e+04_rb,0.11602e+04_rb /)
      kao(:, 5,10,11) = (/ &
        & 0.12010e+04_rb,0.10536e+04_rb,0.90891e+03_rb,0.76248e+03_rb,0.61761e+03_rb, &
        & 0.74025e+03_rb,0.87965e+03_rb,0.10191e+04_rb,0.11583e+04_rb /)
      kao(:, 1,11,11) = (/ &
        & 0.12140e+04_rb,0.10672e+04_rb,0.91852e+03_rb,0.77048e+03_rb,0.62320e+03_rb, &
        & 0.74401e+03_rb,0.88413e+03_rb,0.10249e+04_rb,0.11664e+04_rb /)
      kao(:, 2,11,11) = (/ &
        & 0.12007e+04_rb,0.10549e+04_rb,0.90904e+03_rb,0.76333e+03_rb,0.61635e+03_rb, &
        & 0.74367e+03_rb,0.88337e+03_rb,0.10233e+04_rb,0.11639e+04_rb /)
      kao(:, 3,11,11) = (/ &
        & 0.11891e+04_rb,0.10458e+04_rb,0.90055e+03_rb,0.75543e+03_rb,0.61123e+03_rb, &
        & 0.74262e+03_rb,0.88229e+03_rb,0.10219e+04_rb,0.11620e+04_rb /)
      kao(:, 4,11,11) = (/ &
        & 0.11794e+04_rb,0.10371e+04_rb,0.89190e+03_rb,0.74912e+03_rb,0.60572e+03_rb, &
        & 0.74128e+03_rb,0.88110e+03_rb,0.10207e+04_rb,0.11602e+04_rb /)
      kao(:, 5,11,11) = (/ &
        & 0.11697e+04_rb,0.10285e+04_rb,0.88631e+03_rb,0.74374e+03_rb,0.60062e+03_rb, &
        & 0.73987e+03_rb,0.87958e+03_rb,0.10191e+04_rb,0.11583e+04_rb /)
      kao(:, 1,12,11) = (/ &
        & 0.11829e+04_rb,0.10373e+04_rb,0.89433e+03_rb,0.75107e+03_rb,0.60775e+03_rb, &
        & 0.74357e+03_rb,0.88385e+03_rb,0.10247e+04_rb,0.11664e+04_rb /)
      kao(:, 2,12,11) = (/ &
        & 0.11707e+04_rb,0.10276e+04_rb,0.88712e+03_rb,0.74426e+03_rb,0.60163e+03_rb, &
        & 0.74327e+03_rb,0.88309e+03_rb,0.10231e+04_rb,0.11639e+04_rb /)
      kao(:, 3,12,11) = (/ &
        & 0.11629e+04_rb,0.10211e+04_rb,0.88018e+03_rb,0.73831e+03_rb,0.60184e+03_rb, &
        & 0.74224e+03_rb,0.88203e+03_rb,0.10219e+04_rb,0.11620e+04_rb /)
      kao(:, 4,12,11) = (/ &
        & 0.11538e+04_rb,0.10126e+04_rb,0.87253e+03_rb,0.73377e+03_rb,0.60080e+03_rb, &
        & 0.74101e+03_rb,0.88067e+03_rb,0.10205e+04_rb,0.11602e+04_rb /)
      kao(:, 5,12,11) = (/ &
        & 0.11449e+04_rb,0.10062e+04_rb,0.86764e+03_rb,0.72842e+03_rb,0.59975e+03_rb, &
        & 0.73955e+03_rb,0.87926e+03_rb,0.10190e+04_rb,0.11583e+04_rb /)
      kao(:, 1,13,11) = (/ &
        & 0.11561e+04_rb,0.10152e+04_rb,0.87509e+03_rb,0.73383e+03_rb,0.60253e+03_rb, &
        & 0.74321e+03_rb,0.88362e+03_rb,0.10246e+04_rb,0.11664e+04_rb /)
      kao(:, 2,13,11) = (/ &
        & 0.11468e+04_rb,0.10069e+04_rb,0.86783e+03_rb,0.72921e+03_rb,0.60211e+03_rb, &
        & 0.74294e+03_rb,0.88286e+03_rb,0.10230e+04_rb,0.11639e+04_rb /)
      kao(:, 3,13,11) = (/ &
        & 0.11390e+04_rb,0.10001e+04_rb,0.86301e+03_rb,0.72323e+03_rb,0.60140e+03_rb, &
        & 0.74194e+03_rb,0.88181e+03_rb,0.10218e+04_rb,0.11621e+04_rb /)
      kao(:, 4,13,11) = (/ &
        & 0.11313e+04_rb,0.99361e+03_rb,0.85706e+03_rb,0.72032e+03_rb,0.60042e+03_rb, &
        & 0.74065e+03_rb,0.88048e+03_rb,0.10204e+04_rb,0.11602e+04_rb /)
      kao(:, 5,13,11) = (/ &
        & 0.11266e+04_rb,0.99042e+03_rb,0.85257e+03_rb,0.71548e+03_rb,0.59931e+03_rb, &
        & 0.73930e+03_rb,0.87908e+03_rb,0.10187e+04_rb,0.11583e+04_rb /)
      kao(:, 1, 1,12) = (/ &
        & 0.19596e+04_rb,0.17200e+04_rb,0.14802e+04_rb,0.12370e+04_rb,0.99641e+03_rb, &
        & 0.75556e+03_rb,0.88658e+03_rb,0.10236e+04_rb,0.11622e+04_rb /)
      kao(:, 2, 1,12) = (/ &
        & 0.19107e+04_rb,0.16797e+04_rb,0.14463e+04_rb,0.12083e+04_rb,0.97440e+03_rb, &
        & 0.74935e+03_rb,0.88455e+03_rb,0.10223e+04_rb,0.11611e+04_rb /)
      kao(:, 3, 1,12) = (/ &
        & 0.18687e+04_rb,0.16368e+04_rb,0.14091e+04_rb,0.11809e+04_rb,0.95100e+03_rb, &
        & 0.74626e+03_rb,0.88278e+03_rb,0.10214e+04_rb,0.11600e+04_rb /)
      kao(:, 4, 1,12) = (/ &
        & 0.18274e+04_rb,0.15998e+04_rb,0.13805e+04_rb,0.11544e+04_rb,0.92955e+03_rb, &
        & 0.74432e+03_rb,0.88029e+03_rb,0.10193e+04_rb,0.11572e+04_rb /)
      kao(:, 5, 1,12) = (/ &
        & 0.17908e+04_rb,0.15685e+04_rb,0.13492e+04_rb,0.11303e+04_rb,0.91228e+03_rb, &
        & 0.74183e+03_rb,0.87814e+03_rb,0.10165e+04_rb,0.11551e+04_rb /)
      kao(:, 1, 2,12) = (/ &
        & 0.18091e+04_rb,0.15890e+04_rb,0.13701e+04_rb,0.11447e+04_rb,0.92240e+03_rb, &
        & 0.75031e+03_rb,0.88580e+03_rb,0.10236e+04_rb,0.11628e+04_rb /)
      kao(:, 2, 2,12) = (/ &
        & 0.17702e+04_rb,0.15507e+04_rb,0.13346e+04_rb,0.11179e+04_rb,0.90072e+03_rb, &
        & 0.74790e+03_rb,0.88384e+03_rb,0.10222e+04_rb,0.11616e+04_rb /)
      kao(:, 3, 2,12) = (/ &
        & 0.17282e+04_rb,0.15159e+04_rb,0.13066e+04_rb,0.10967e+04_rb,0.88274e+03_rb, &
        & 0.74530e+03_rb,0.88241e+03_rb,0.10213e+04_rb,0.11606e+04_rb /)
      kao(:, 4, 2,12) = (/ &
        & 0.16964e+04_rb,0.14887e+04_rb,0.12816e+04_rb,0.10711e+04_rb,0.86387e+03_rb, &
        & 0.74308e+03_rb,0.88064e+03_rb,0.10199e+04_rb,0.11593e+04_rb /)
      kao(:, 5, 2,12) = (/ &
        & 0.16664e+04_rb,0.14634e+04_rb,0.12570e+04_rb,0.10541e+04_rb,0.84878e+03_rb, &
        & 0.74105e+03_rb,0.87802e+03_rb,0.10170e+04_rb,0.11559e+04_rb /)
      kao(:, 1, 3,12) = (/ &
        & 0.16969e+04_rb,0.14893e+04_rb,0.12806e+04_rb,0.10686e+04_rb,0.86200e+03_rb, &
        & 0.74953e+03_rb,0.88586e+03_rb,0.10239e+04_rb,0.11637e+04_rb /)
      kao(:, 2, 3,12) = (/ &
        & 0.16530e+04_rb,0.14535e+04_rb,0.12505e+04_rb,0.10502e+04_rb,0.84485e+03_rb, &
        & 0.74734e+03_rb,0.88380e+03_rb,0.10224e+04_rb,0.11623e+04_rb /)
      kao(:, 3, 3,12) = (/ &
        & 0.16258e+04_rb,0.14316e+04_rb,0.12308e+04_rb,0.10274e+04_rb,0.82737e+03_rb, &
        & 0.74498e+03_rb,0.88220e+03_rb,0.10214e+04_rb,0.11612e+04_rb /)
      kao(:, 4, 3,12) = (/ &
        & 0.15947e+04_rb,0.13987e+04_rb,0.12036e+04_rb,0.10100e+04_rb,0.81398e+03_rb, &
        & 0.74250e+03_rb,0.88073e+03_rb,0.10203e+04_rb,0.11601e+04_rb /)
      kao(:, 5, 3,12) = (/ &
        & 0.15659e+04_rb,0.13745e+04_rb,0.11827e+04_rb,0.99357e+03_rb,0.79945e+03_rb, &
        & 0.74072e+03_rb,0.87844e+03_rb,0.10192e+04_rb,0.11574e+04_rb /)
      kao(:, 1, 4,12) = (/ &
        & 0.15993e+04_rb,0.14029e+04_rb,0.12034e+04_rb,0.10105e+04_rb,0.81494e+03_rb, &
        & 0.74898e+03_rb,0.88604e+03_rb,0.10246e+04_rb,0.11648e+04_rb /)
      kao(:, 2, 4,12) = (/ &
        & 0.15603e+04_rb,0.13717e+04_rb,0.11803e+04_rb,0.99158e+03_rb,0.79797e+03_rb, &
        & 0.74702e+03_rb,0.88413e+03_rb,0.10230e+04_rb,0.11631e+04_rb /)
      kao(:, 3, 4,12) = (/ &
        & 0.15363e+04_rb,0.13460e+04_rb,0.11581e+04_rb,0.97158e+03_rb,0.78286e+03_rb, &
        & 0.74488e+03_rb,0.88249e+03_rb,0.10215e+04_rb,0.11618e+04_rb /)
      kao(:, 4, 4,12) = (/ &
        & 0.15069e+04_rb,0.13248e+04_rb,0.11397e+04_rb,0.95515e+03_rb,0.77185e+03_rb, &
        & 0.74259e+03_rb,0.88086e+03_rb,0.10207e+04_rb,0.11608e+04_rb /)
      kao(:, 5, 4,12) = (/ &
        & 0.14842e+04_rb,0.13030e+04_rb,0.11201e+04_rb,0.94205e+03_rb,0.75928e+03_rb, &
        & 0.74072e+03_rb,0.87912e+03_rb,0.10193e+04_rb,0.11593e+04_rb /)
      kao(:, 1, 5,12) = (/ &
        & 0.15161e+04_rb,0.13293e+04_rb,0.11434e+04_rb,0.96071e+03_rb,0.77366e+03_rb, &
        & 0.74856e+03_rb,0.88634e+03_rb,0.10256e+04_rb,0.11660e+04_rb /)
      kao(:, 2, 5,12) = (/ &
        & 0.14862e+04_rb,0.13039e+04_rb,0.11202e+04_rb,0.93927e+03_rb,0.75808e+03_rb, &
        & 0.74678e+03_rb,0.88432e+03_rb,0.10237e+04_rb,0.11641e+04_rb /)
      kao(:, 3, 5,12) = (/ &
        & 0.14569e+04_rb,0.12812e+04_rb,0.11040e+04_rb,0.92477e+03_rb,0.74608e+03_rb, &
        & 0.74483e+03_rb,0.88267e+03_rb,0.10221e+04_rb,0.11626e+04_rb /)
      kao(:, 4, 5,12) = (/ &
        & 0.14370e+04_rb,0.12601e+04_rb,0.10866e+04_rb,0.91322e+03_rb,0.73546e+03_rb, &
        & 0.74282e+03_rb,0.88106e+03_rb,0.10210e+04_rb,0.11614e+04_rb /)
      kao(:, 5, 5,12) = (/ &
        & 0.14157e+04_rb,0.12432e+04_rb,0.10692e+04_rb,0.89766e+03_rb,0.72371e+03_rb, &
        & 0.74075e+03_rb,0.87977e+03_rb,0.10201e+04_rb,0.11604e+04_rb /)
      kao(:, 1, 6,12) = (/ &
        & 0.14446e+04_rb,0.12668e+04_rb,0.10902e+04_rb,0.91458e+03_rb,0.73853e+03_rb, &
        & 0.74800e+03_rb,0.88683e+03_rb,0.10267e+04_rb,0.11674e+04_rb /)
      kao(:, 2, 6,12) = (/ &
        & 0.14180e+04_rb,0.12442e+04_rb,0.10729e+04_rb,0.90054e+03_rb,0.72712e+03_rb, &
        & 0.74664e+03_rb,0.88490e+03_rb,0.10243e+04_rb,0.11653e+04_rb /)
      kao(:, 3, 6,12) = (/ &
        & 0.13970e+04_rb,0.12338e+04_rb,0.10562e+04_rb,0.88644e+03_rb,0.71466e+03_rb, &
        & 0.74490e+03_rb,0.88299e+03_rb,0.10228e+04_rb,0.11635e+04_rb /)
      kao(:, 4, 6,12) = (/ &
        & 0.13772e+04_rb,0.12092e+04_rb,0.10403e+04_rb,0.87191e+03_rb,0.70566e+03_rb, &
        & 0.74297e+03_rb,0.88145e+03_rb,0.10214e+04_rb,0.11622e+04_rb /)
      kao(:, 5, 6,12) = (/ &
        & 0.13536e+04_rb,0.11933e+04_rb,0.10279e+04_rb,0.85985e+03_rb,0.69532e+03_rb, &
        & 0.74091e+03_rb,0.88002e+03_rb,0.10207e+04_rb,0.11611e+04_rb /)
      kao(:, 1, 7,12) = (/ &
        & 0.13872e+04_rb,0.12173e+04_rb,0.10473e+04_rb,0.87779e+03_rb,0.70929e+03_rb, &
        & 0.74753e+03_rb,0.88739e+03_rb,0.10279e+04_rb,0.11686e+04_rb /)
      kao(:, 2, 7,12) = (/ &
        & 0.13614e+04_rb,0.11960e+04_rb,0.10302e+04_rb,0.86342e+03_rb,0.69779e+03_rb, &
        & 0.74649e+03_rb,0.88535e+03_rb,0.10254e+04_rb,0.11666e+04_rb /)
      kao(:, 3, 7,12) = (/ &
        & 0.13423e+04_rb,0.11779e+04_rb,0.10152e+04_rb,0.85120e+03_rb,0.68669e+03_rb, &
        & 0.74500e+03_rb,0.88364e+03_rb,0.10236e+04_rb,0.11646e+04_rb /)
      kao(:, 4, 7,12) = (/ &
        & 0.13222e+04_rb,0.11614e+04_rb,0.10027e+04_rb,0.84138e+03_rb,0.67980e+03_rb, &
        & 0.74324e+03_rb,0.88194e+03_rb,0.10219e+04_rb,0.11628e+04_rb /)
      kao(:, 5, 7,12) = (/ &
        & 0.13077e+04_rb,0.11487e+04_rb,0.98870e+03_rb,0.82923e+03_rb,0.67117e+03_rb, &
        & 0.74131e+03_rb,0.88052e+03_rb,0.10200e+04_rb,0.11617e+04_rb /)
      kao(:, 1, 8,12) = (/ &
        & 0.13318e+04_rb,0.11751e+04_rb,0.10116e+04_rb,0.84760e+03_rb,0.68552e+03_rb, &
        & 0.74755e+03_rb,0.88764e+03_rb,0.10284e+04_rb,0.11697e+04_rb /)
      kao(:, 2, 8,12) = (/ &
        & 0.13123e+04_rb,0.11533e+04_rb,0.99316e+03_rb,0.83364e+03_rb,0.67452e+03_rb, &
        & 0.74634e+03_rb,0.88623e+03_rb,0.10267e+04_rb,0.11679e+04_rb /)
      kao(:, 3, 8,12) = (/ &
        & 0.12963e+04_rb,0.11395e+04_rb,0.98036e+03_rb,0.82206e+03_rb,0.66378e+03_rb, &
        & 0.74528e+03_rb,0.88422e+03_rb,0.10245e+04_rb,0.11657e+04_rb /)
      kao(:, 4, 8,12) = (/ &
        & 0.12799e+04_rb,0.11235e+04_rb,0.96838e+03_rb,0.81175e+03_rb,0.65688e+03_rb, &
        & 0.74350e+03_rb,0.88249e+03_rb,0.10227e+04_rb,0.11639e+04_rb /)
      kao(:, 5, 8,12) = (/ &
        & 0.12649e+04_rb,0.11128e+04_rb,0.95762e+03_rb,0.80323e+03_rb,0.64917e+03_rb, &
        & 0.74172e+03_rb,0.88089e+03_rb,0.10213e+04_rb,0.11624e+04_rb /)
      kao(:, 1, 9,12) = (/ &
        & 0.12892e+04_rb,0.11353e+04_rb,0.97529e+03_rb,0.81997e+03_rb,0.66439e+03_rb, &
        & 0.74689e+03_rb,0.88774e+03_rb,0.10288e+04_rb,0.11703e+04_rb /)
      kao(:, 2, 9,12) = (/ &
        & 0.12710e+04_rb,0.11179e+04_rb,0.96230e+03_rb,0.80757e+03_rb,0.65174e+03_rb, &
        & 0.74606e+03_rb,0.88671e+03_rb,0.10277e+04_rb,0.11691e+04_rb /)
      kao(:, 3, 9,12) = (/ &
        & 0.12556e+04_rb,0.11026e+04_rb,0.95506e+03_rb,0.79904e+03_rb,0.64512e+03_rb, &
        & 0.74525e+03_rb,0.88489e+03_rb,0.10254e+04_rb,0.11669e+04_rb /)
      kao(:, 4, 9,12) = (/ &
        & 0.12434e+04_rb,0.10930e+04_rb,0.94625e+03_rb,0.79326e+03_rb,0.63956e+03_rb, &
        & 0.74375e+03_rb,0.88308e+03_rb,0.10236e+04_rb,0.11649e+04_rb /)
      kao(:, 5, 9,12) = (/ &
        & 0.12362e+04_rb,0.10807e+04_rb,0.93519e+03_rb,0.78318e+03_rb,0.63176e+03_rb, &
        & 0.74222e+03_rb,0.88144e+03_rb,0.10218e+04_rb,0.11632e+04_rb /)
      kao(:, 1,10,12) = (/ &
        & 0.12560e+04_rb,0.11008e+04_rb,0.94819e+03_rb,0.79713e+03_rb,0.64289e+03_rb, &
        & 0.74662e+03_rb,0.88790e+03_rb,0.10293e+04_rb,0.11710e+04_rb /)
      kao(:, 2,10,12) = (/ &
        & 0.12370e+04_rb,0.10868e+04_rb,0.93614e+03_rb,0.78750e+03_rb,0.63591e+03_rb, &
        & 0.74589e+03_rb,0.88689e+03_rb,0.10281e+04_rb,0.11697e+04_rb /)
      kao(:, 3,10,12) = (/ &
        & 0.12228e+04_rb,0.10745e+04_rb,0.92565e+03_rb,0.77586e+03_rb,0.63013e+03_rb, &
        & 0.74525e+03_rb,0.88546e+03_rb,0.10265e+04_rb,0.11680e+04_rb /)
      kao(:, 4,10,12) = (/ &
        & 0.12130e+04_rb,0.10629e+04_rb,0.91805e+03_rb,0.77218e+03_rb,0.62214e+03_rb, &
        & 0.74398e+03_rb,0.88360e+03_rb,0.10243e+04_rb,0.11659e+04_rb /)
      kao(:, 5,10,12) = (/ &
        & 0.11990e+04_rb,0.10561e+04_rb,0.90990e+03_rb,0.76234e+03_rb,0.61782e+03_rb, &
        & 0.74238e+03_rb,0.88192e+03_rb,0.10225e+04_rb,0.11640e+04_rb /)
      kao(:, 1,11,12) = (/ &
        & 0.12160e+04_rb,0.10660e+04_rb,0.91756e+03_rb,0.77092e+03_rb,0.62410e+03_rb, &
        & 0.74614e+03_rb,0.88758e+03_rb,0.10288e+04_rb,0.11712e+04_rb /)
      kao(:, 2,11,12) = (/ &
        & 0.12001e+04_rb,0.10573e+04_rb,0.90897e+03_rb,0.76187e+03_rb,0.61778e+03_rb, &
        & 0.74563e+03_rb,0.88659e+03_rb,0.10280e+04_rb,0.11697e+04_rb /)
      kao(:, 3,11,12) = (/ &
        & 0.11883e+04_rb,0.10436e+04_rb,0.89901e+03_rb,0.75711e+03_rb,0.61087e+03_rb, &
        & 0.74468e+03_rb,0.88519e+03_rb,0.10264e+04_rb,0.11680e+04_rb /)
      kao(:, 4,11,12) = (/ &
        & 0.11791e+04_rb,0.10338e+04_rb,0.89462e+03_rb,0.74974e+03_rb,0.60740e+03_rb, &
        & 0.74372e+03_rb,0.88322e+03_rb,0.10242e+04_rb,0.11659e+04_rb /)
      kao(:, 5,11,12) = (/ &
        & 0.11736e+04_rb,0.10284e+04_rb,0.88610e+03_rb,0.74386e+03_rb,0.60256e+03_rb, &
        & 0.74204e+03_rb,0.88156e+03_rb,0.10223e+04_rb,0.11640e+04_rb /)
      kao(:, 1,12,12) = (/ &
        & 0.11783e+04_rb,0.10408e+04_rb,0.89462e+03_rb,0.75034e+03_rb,0.60838e+03_rb, &
        & 0.74575e+03_rb,0.88732e+03_rb,0.10288e+04_rb,0.11710e+04_rb /)
      kao(:, 2,12,12) = (/ &
        & 0.11693e+04_rb,0.10333e+04_rb,0.88486e+03_rb,0.74467e+03_rb,0.60420e+03_rb, &
        & 0.74514e+03_rb,0.88635e+03_rb,0.10279e+04_rb,0.11697e+04_rb /)
      kao(:, 3,12,12) = (/ &
        & 0.11597e+04_rb,0.10197e+04_rb,0.87845e+03_rb,0.73901e+03_rb,0.60371e+03_rb, &
        & 0.74440e+03_rb,0.88495e+03_rb,0.10263e+04_rb,0.11680e+04_rb /)
      kao(:, 4,12,12) = (/ &
        & 0.11554e+04_rb,0.10154e+04_rb,0.87390e+03_rb,0.73209e+03_rb,0.60310e+03_rb, &
        & 0.74343e+03_rb,0.88312e+03_rb,0.10241e+04_rb,0.11659e+04_rb /)
      kao(:, 5,12,12) = (/ &
        & 0.11463e+04_rb,0.10081e+04_rb,0.86768e+03_rb,0.73113e+03_rb,0.60226e+03_rb, &
        & 0.74178e+03_rb,0.88148e+03_rb,0.10222e+04_rb,0.11640e+04_rb /)
      kao(:, 1,13,12) = (/ &
        & 0.11576e+04_rb,0.10172e+04_rb,0.87697e+03_rb,0.73496e+03_rb,0.60442e+03_rb, &
        & 0.74542e+03_rb,0.88710e+03_rb,0.10287e+04_rb,0.11710e+04_rb /)
      kao(:, 2,13,12) = (/ &
        & 0.11489e+04_rb,0.10121e+04_rb,0.86787e+03_rb,0.73038e+03_rb,0.60369e+03_rb, &
        & 0.74483e+03_rb,0.88615e+03_rb,0.10278e+04_rb,0.11697e+04_rb /)
      kao(:, 3,13,12) = (/ &
        & 0.11408e+04_rb,0.10003e+04_rb,0.86069e+03_rb,0.72468e+03_rb,0.60332e+03_rb, &
        & 0.74398e+03_rb,0.88477e+03_rb,0.10262e+04_rb,0.11679e+04_rb /)
      kao(:, 4,13,12) = (/ &
        & 0.11353e+04_rb,0.99812e+03_rb,0.85805e+03_rb,0.72010e+03_rb,0.60276e+03_rb, &
        & 0.74301e+03_rb,0.88294e+03_rb,0.10240e+04_rb,0.11659e+04_rb /)
      kao(:, 5,13,12) = (/ &
        & 0.11276e+04_rb,0.99093e+03_rb,0.85424e+03_rb,0.71851e+03_rb,0.60186e+03_rb, &
        & 0.74137e+03_rb,0.88131e+03_rb,0.10223e+04_rb,0.11640e+04_rb /)
      kao(:, 1, 1,13) = (/ &
        & 0.19698e+04_rb,0.17183e+04_rb,0.14812e+04_rb,0.12476e+04_rb,0.10057e+04_rb, &
        & 0.76142e+03_rb,0.89076e+03_rb,0.10305e+04_rb,0.11701e+04_rb /)
      kao(:, 2, 1,13) = (/ &
        & 0.19138e+04_rb,0.16790e+04_rb,0.14471e+04_rb,0.12079e+04_rb,0.97024e+03_rb, &
        & 0.74958e+03_rb,0.88854e+03_rb,0.10281e+04_rb,0.11676e+04_rb /)
      kao(:, 3, 1,13) = (/ &
        & 0.18656e+04_rb,0.16362e+04_rb,0.14072e+04_rb,0.11764e+04_rb,0.95368e+03_rb, &
        & 0.74756e+03_rb,0.88578e+03_rb,0.10247e+04_rb,0.11642e+04_rb /)
      kao(:, 4, 1,13) = (/ &
        & 0.18274e+04_rb,0.16076e+04_rb,0.13744e+04_rb,0.11522e+04_rb,0.93101e+03_rb, &
        & 0.74489e+03_rb,0.88368e+03_rb,0.10224e+04_rb,0.11624e+04_rb /)
      kao(:, 5, 1,13) = (/ &
        & 0.18123e+04_rb,0.15711e+04_rb,0.13539e+04_rb,0.11391e+04_rb,0.91271e+03_rb, &
        & 0.74309e+03_rb,0.88194e+03_rb,0.10210e+04_rb,0.11601e+04_rb /)
      kao(:, 1, 2,13) = (/ &
        & 0.18136e+04_rb,0.16074e+04_rb,0.13682e+04_rb,0.11455e+04_rb,0.92152e+03_rb, &
        & 0.75090e+03_rb,0.89044e+03_rb,0.10309e+04_rb,0.11711e+04_rb /)
      kao(:, 2, 2,13) = (/ &
        & 0.17663e+04_rb,0.15639e+04_rb,0.13412e+04_rb,0.11338e+04_rb,0.90565e+03_rb, &
        & 0.74892e+03_rb,0.88842e+03_rb,0.10286e+04_rb,0.11690e+04_rb /)
      kao(:, 3, 2,13) = (/ &
        & 0.17473e+04_rb,0.15168e+04_rb,0.13179e+04_rb,0.10933e+04_rb,0.88564e+03_rb, &
        & 0.74657e+03_rb,0.88587e+03_rb,0.10277e+04_rb,0.11659e+04_rb /)
      kao(:, 4, 2,13) = (/ &
        & 0.16935e+04_rb,0.14940e+04_rb,0.12866e+04_rb,0.10771e+04_rb,0.86513e+03_rb, &
        & 0.74388e+03_rb,0.88292e+03_rb,0.10226e+04_rb,0.11621e+04_rb /)
      kao(:, 5, 2,13) = (/ &
        & 0.16796e+04_rb,0.14587e+04_rb,0.12610e+04_rb,0.10625e+04_rb,0.85051e+03_rb, &
        & 0.74222e+03_rb,0.88187e+03_rb,0.10215e+04_rb,0.11614e+04_rb /)
      kao(:, 1, 3,13) = (/ &
        & 0.16915e+04_rb,0.14853e+04_rb,0.12759e+04_rb,0.10742e+04_rb,0.87078e+03_rb, &
        & 0.75040e+03_rb,0.89056e+03_rb,0.10314e+04_rb,0.11723e+04_rb /)
      kao(:, 2, 3,13) = (/ &
        & 0.16685e+04_rb,0.14695e+04_rb,0.12569e+04_rb,0.10522e+04_rb,0.84369e+03_rb, &
        & 0.74849e+03_rb,0.88875e+03_rb,0.10295e+04_rb,0.11702e+04_rb /)
      kao(:, 3, 3,13) = (/ &
        & 0.16315e+04_rb,0.14234e+04_rb,0.12207e+04_rb,0.10278e+04_rb,0.83542e+03_rb, &
        & 0.74665e+03_rb,0.88665e+03_rb,0.10272e+04_rb,0.11677e+04_rb /)
      kao(:, 4, 3,13) = (/ &
        & 0.15936e+04_rb,0.13973e+04_rb,0.11983e+04_rb,0.10068e+04_rb,0.81253e+03_rb, &
        & 0.74392e+03_rb,0.88383e+03_rb,0.10241e+04_rb,0.11643e+04_rb /)
      kao(:, 5, 3,13) = (/ &
        & 0.15704e+04_rb,0.13749e+04_rb,0.11794e+04_rb,0.99081e+03_rb,0.79887e+03_rb, &
        & 0.74214e+03_rb,0.88216e+03_rb,0.10204e+04_rb,0.11625e+04_rb /)
      kao(:, 1, 4,13) = (/ &
        & 0.16005e+04_rb,0.14146e+04_rb,0.12141e+04_rb,0.10096e+04_rb,0.81830e+03_rb, &
        & 0.74981e+03_rb,0.89073e+03_rb,0.10323e+04_rb,0.11736e+04_rb /)
      kao(:, 2, 4,13) = (/ &
        & 0.15697e+04_rb,0.13709e+04_rb,0.11792e+04_rb,0.98706e+03_rb,0.80274e+03_rb, &
        & 0.74843e+03_rb,0.88917e+03_rb,0.10304e+04_rb,0.11716e+04_rb /)
      kao(:, 3, 4,13) = (/ &
        & 0.15342e+04_rb,0.13528e+04_rb,0.11633e+04_rb,0.97494e+03_rb,0.78298e+03_rb, &
        & 0.74655e+03_rb,0.88714e+03_rb,0.10283e+04_rb,0.11694e+04_rb /)
      kao(:, 4, 4,13) = (/ &
        & 0.15079e+04_rb,0.13273e+04_rb,0.11368e+04_rb,0.95326e+03_rb,0.76898e+03_rb, &
        & 0.74477e+03_rb,0.88522e+03_rb,0.10258e+04_rb,0.11666e+04_rb /)
      kao(:, 5, 4,13) = (/ &
        & 0.14904e+04_rb,0.13050e+04_rb,0.11274e+04_rb,0.94133e+03_rb,0.75918e+03_rb, &
        & 0.74185e+03_rb,0.88236e+03_rb,0.10229e+04_rb,0.11634e+04_rb /)
      kao(:, 1, 5,13) = (/ &
        & 0.15223e+04_rb,0.13292e+04_rb,0.11458e+04_rb,0.96632e+03_rb,0.77253e+03_rb, &
        & 0.74931e+03_rb,0.89080e+03_rb,0.10327e+04_rb,0.11744e+04_rb /)
      kao(:, 2, 5,13) = (/ &
        & 0.14887e+04_rb,0.13100e+04_rb,0.11308e+04_rb,0.95564e+03_rb,0.76318e+03_rb, &
        & 0.74814e+03_rb,0.88953e+03_rb,0.10311e+04_rb,0.11728e+04_rb /)
      kao(:, 3, 5,13) = (/ &
        & 0.14666e+04_rb,0.12805e+04_rb,0.10990e+04_rb,0.92585e+03_rb,0.74527e+03_rb, &
        & 0.74662e+03_rb,0.88788e+03_rb,0.10293e+04_rb,0.11708e+04_rb /)
      kao(:, 4, 5,13) = (/ &
        & 0.14380e+04_rb,0.12615e+04_rb,0.10805e+04_rb,0.91192e+03_rb,0.73306e+03_rb, &
        & 0.74471e+03_rb,0.88594e+03_rb,0.10272e+04_rb,0.11684e+04_rb /)
      kao(:, 5, 5,13) = (/ &
        & 0.14117e+04_rb,0.12389e+04_rb,0.10707e+04_rb,0.89863e+03_rb,0.72872e+03_rb, &
        & 0.74254e+03_rb,0.88356e+03_rb,0.10243e+04_rb,0.11652e+04_rb /)
      kao(:, 1, 6,13) = (/ &
        & 0.14428e+04_rb,0.12661e+04_rb,0.10990e+04_rb,0.91605e+03_rb,0.74645e+03_rb, &
        & 0.74914e+03_rb,0.89068e+03_rb,0.10327e+04_rb,0.11747e+04_rb /)
      kao(:, 2, 6,13) = (/ &
        & 0.14277e+04_rb,0.12454e+04_rb,0.10670e+04_rb,0.90364e+03_rb,0.72758e+03_rb, &
        & 0.74802e+03_rb,0.88983e+03_rb,0.10318e+04_rb,0.11738e+04_rb /)
      kao(:, 3, 6,13) = (/ &
        & 0.14072e+04_rb,0.12244e+04_rb,0.10539e+04_rb,0.88576e+03_rb,0.71342e+03_rb, &
        & 0.74674e+03_rb,0.88844e+03_rb,0.10303e+04_rb,0.11723e+04_rb /)
      kao(:, 4, 6,13) = (/ &
        & 0.13764e+04_rb,0.12101e+04_rb,0.10371e+04_rb,0.87173e+03_rb,0.70565e+03_rb, &
        & 0.74519e+03_rb,0.88672e+03_rb,0.10284e+04_rb,0.11700e+04_rb /)
      kao(:, 5, 6,13) = (/ &
        & 0.13559e+04_rb,0.11928e+04_rb,0.10275e+04_rb,0.87202e+03_rb,0.69916e+03_rb, &
        & 0.74337e+03_rb,0.88468e+03_rb,0.10257e+04_rb,0.11677e+04_rb /)
      kao(:, 1, 7,13) = (/ &
        & 0.13816e+04_rb,0.12181e+04_rb,0.10498e+04_rb,0.87727e+03_rb,0.70862e+03_rb, &
        & 0.74879e+03_rb,0.89032e+03_rb,0.10324e+04_rb,0.11748e+04_rb /)
      kao(:, 2, 7,13) = (/ &
        & 0.13746e+04_rb,0.12007e+04_rb,0.10368e+04_rb,0.86700e+03_rb,0.69742e+03_rb, &
        & 0.74791e+03_rb,0.88998e+03_rb,0.10322e+04_rb,0.11745e+04_rb /)
      kao(:, 3, 7,13) = (/ &
        & 0.13471e+04_rb,0.11822e+04_rb,0.10118e+04_rb,0.85187e+03_rb,0.69076e+03_rb, &
        & 0.74684e+03_rb,0.88891e+03_rb,0.10311e+04_rb,0.11732e+04_rb /)
      kao(:, 4, 7,13) = (/ &
        & 0.13306e+04_rb,0.11625e+04_rb,0.10055e+04_rb,0.83687e+03_rb,0.67946e+03_rb, &
        & 0.74564e+03_rb,0.88741e+03_rb,0.10294e+04_rb,0.11714e+04_rb /)
      kao(:, 5, 7,13) = (/ &
        & 0.13072e+04_rb,0.11462e+04_rb,0.98997e+03_rb,0.82975e+03_rb,0.67148e+03_rb, &
        & 0.74390e+03_rb,0.88544e+03_rb,0.10275e+04_rb,0.11691e+04_rb /)
      kao(:, 1, 8,13) = (/ &
        & 0.13342e+04_rb,0.11738e+04_rb,0.10072e+04_rb,0.84777e+03_rb,0.68336e+03_rb, &
        & 0.74810e+03_rb,0.89032e+03_rb,0.10328e+04_rb,0.11753e+04_rb /)
      kao(:, 2, 8,13) = (/ &
        & 0.13170e+04_rb,0.11565e+04_rb,0.99551e+03_rb,0.83812e+03_rb,0.67873e+03_rb, &
        & 0.74767e+03_rb,0.88984e+03_rb,0.10325e+04_rb,0.11747e+04_rb /)
      kao(:, 3, 8,13) = (/ &
        & 0.12961e+04_rb,0.11348e+04_rb,0.98531e+03_rb,0.82142e+03_rb,0.66767e+03_rb, &
        & 0.74660e+03_rb,0.88923e+03_rb,0.10317e+04_rb,0.11741e+04_rb /)
      kao(:, 4, 8,13) = (/ &
        & 0.12863e+04_rb,0.11324e+04_rb,0.96704e+03_rb,0.81553e+03_rb,0.65703e+03_rb, &
        & 0.74590e+03_rb,0.88815e+03_rb,0.10305e+04_rb,0.11725e+04_rb /)
      kao(:, 5, 8,13) = (/ &
        & 0.12689e+04_rb,0.11113e+04_rb,0.96142e+03_rb,0.81188e+03_rb,0.65150e+03_rb, &
        & 0.74435e+03_rb,0.88637e+03_rb,0.10284e+04_rb,0.11705e+04_rb /)
      kao(:, 1, 9,13) = (/ &
        & 0.12929e+04_rb,0.11420e+04_rb,0.97941e+03_rb,0.82358e+03_rb,0.66259e+03_rb, &
        & 0.74862e+03_rb,0.89046e+03_rb,0.10332e+04_rb,0.11761e+04_rb /)
      kao(:, 2, 9,13) = (/ &
        & 0.12739e+04_rb,0.11222e+04_rb,0.96918e+03_rb,0.81141e+03_rb,0.65436e+03_rb, &
        & 0.74747e+03_rb,0.88962e+03_rb,0.10323e+04_rb,0.11749e+04_rb /)
      kao(:, 3, 9,13) = (/ &
        & 0.12674e+04_rb,0.11070e+04_rb,0.94554e+03_rb,0.79811e+03_rb,0.64989e+03_rb, &
        & 0.74671e+03_rb,0.88937e+03_rb,0.10320e+04_rb,0.11746e+04_rb /)
      kao(:, 4, 9,13) = (/ &
        & 0.12536e+04_rb,0.10974e+04_rb,0.93180e+03_rb,0.78881e+03_rb,0.64048e+03_rb, &
        & 0.74596e+03_rb,0.88846e+03_rb,0.10310e+04_rb,0.11735e+04_rb /)
      kao(:, 5, 9,13) = (/ &
        & 0.12304e+04_rb,0.10931e+04_rb,0.92897e+03_rb,0.78158e+03_rb,0.63657e+03_rb, &
        & 0.74461e+03_rb,0.88706e+03_rb,0.10294e+04_rb,0.11717e+04_rb /)
      kao(:, 1,10,13) = (/ &
        & 0.12516e+04_rb,0.11039e+04_rb,0.96641e+03_rb,0.80204e+03_rb,0.64728e+03_rb, &
        & 0.74871e+03_rb,0.89043e+03_rb,0.10331e+04_rb,0.11761e+04_rb /)
      kao(:, 2,10,13) = (/ &
        & 0.12445e+04_rb,0.10951e+04_rb,0.94205e+03_rb,0.78296e+03_rb,0.63684e+03_rb, &
        & 0.74738e+03_rb,0.88974e+03_rb,0.10326e+04_rb,0.11754e+04_rb /)
      kao(:, 3,10,13) = (/ &
        & 0.12212e+04_rb,0.10830e+04_rb,0.92907e+03_rb,0.78344e+03_rb,0.62583e+03_rb, &
        & 0.74649e+03_rb,0.88924e+03_rb,0.10320e+04_rb,0.11747e+04_rb /)
      kao(:, 4,10,13) = (/ &
        & 0.12116e+04_rb,0.10675e+04_rb,0.91424e+03_rb,0.76668e+03_rb,0.62374e+03_rb, &
        & 0.74589e+03_rb,0.88889e+03_rb,0.10314e+04_rb,0.11741e+04_rb /)
      kao(:, 5,10,13) = (/ &
        & 0.12052e+04_rb,0.10532e+04_rb,0.90488e+03_rb,0.76246e+03_rb,0.61399e+03_rb, &
        & 0.74500e+03_rb,0.88754e+03_rb,0.10301e+04_rb,0.11727e+04_rb /)
      kao(:, 1,11,13) = (/ &
        & 0.12119e+04_rb,0.10670e+04_rb,0.92549e+03_rb,0.77292e+03_rb,0.62469e+03_rb, &
        & 0.74822e+03_rb,0.89015e+03_rb,0.10331e+04_rb,0.11759e+04_rb /)
      kao(:, 2,11,13) = (/ &
        & 0.12040e+04_rb,0.10529e+04_rb,0.90947e+03_rb,0.76631e+03_rb,0.61756e+03_rb, &
        & 0.74680e+03_rb,0.88950e+03_rb,0.10325e+04_rb,0.11754e+04_rb /)
      kao(:, 3,11,13) = (/ &
        & 0.11932e+04_rb,0.10470e+04_rb,0.90374e+03_rb,0.75316e+03_rb,0.61137e+03_rb, &
        & 0.74625e+03_rb,0.88901e+03_rb,0.10319e+04_rb,0.11747e+04_rb /)
      kao(:, 4,11,13) = (/ &
        & 0.11901e+04_rb,0.10395e+04_rb,0.89205e+03_rb,0.75483e+03_rb,0.60707e+03_rb, &
        & 0.74529e+03_rb,0.88868e+03_rb,0.10313e+04_rb,0.11742e+04_rb /)
      kao(:, 5,11,13) = (/ &
        & 0.11740e+04_rb,0.10262e+04_rb,0.88782e+03_rb,0.74555e+03_rb,0.60321e+03_rb, &
        & 0.74456e+03_rb,0.88735e+03_rb,0.10302e+04_rb,0.11727e+04_rb /)
      kao(:, 1,12,13) = (/ &
        & 0.11952e+04_rb,0.10387e+04_rb,0.90464e+03_rb,0.75031e+03_rb,0.60579e+03_rb, &
        & 0.74809e+03_rb,0.88993e+03_rb,0.10330e+04_rb,0.11761e+04_rb /)
      kao(:, 2,12,13) = (/ &
        & 0.11767e+04_rb,0.10280e+04_rb,0.89305e+03_rb,0.74295e+03_rb,0.60520e+03_rb, &
        & 0.74658e+03_rb,0.88930e+03_rb,0.10324e+04_rb,0.11754e+04_rb /)
      kao(:, 3,12,13) = (/ &
        & 0.11638e+04_rb,0.10252e+04_rb,0.88200e+03_rb,0.74209e+03_rb,0.60448e+03_rb, &
        & 0.74580e+03_rb,0.88883e+03_rb,0.10318e+04_rb,0.11747e+04_rb /)
      kao(:, 4,12,13) = (/ &
        & 0.11608e+04_rb,0.10182e+04_rb,0.87717e+03_rb,0.73339e+03_rb,0.60384e+03_rb, &
        & 0.74474e+03_rb,0.88834e+03_rb,0.10314e+04_rb,0.11742e+04_rb /)
      kao(:, 5,12,13) = (/ &
        & 0.11455e+04_rb,0.10085e+04_rb,0.86868e+03_rb,0.72782e+03_rb,0.60261e+03_rb, &
        & 0.74417e+03_rb,0.88752e+03_rb,0.10299e+04_rb,0.11727e+04_rb /)
      kao(:, 1,13,13) = (/ &
        & 0.11538e+04_rb,0.10104e+04_rb,0.87836e+03_rb,0.74181e+03_rb,0.60545e+03_rb, &
        & 0.74747e+03_rb,0.88975e+03_rb,0.10329e+04_rb,0.11763e+04_rb /)
      kao(:, 2,13,13) = (/ &
        & 0.11412e+04_rb,0.10013e+04_rb,0.88484e+03_rb,0.73153e+03_rb,0.60489e+03_rb, &
        & 0.74629e+03_rb,0.88914e+03_rb,0.10323e+04_rb,0.11754e+04_rb /)
      kao(:, 3,13,13) = (/ &
        & 0.11370e+04_rb,0.10022e+04_rb,0.86246e+03_rb,0.73215e+03_rb,0.60419e+03_rb, &
        & 0.74569e+03_rb,0.88867e+03_rb,0.10317e+04_rb,0.11747e+04_rb /)
      kao(:, 4,13,13) = (/ &
        & 0.11288e+04_rb,0.99109e+03_rb,0.86549e+03_rb,0.72280e+03_rb,0.60356e+03_rb, &
        & 0.74480e+03_rb,0.88820e+03_rb,0.10312e+04_rb,0.11742e+04_rb /)
      kao(:, 5,13,13) = (/ &
        & 0.11445e+04_rb,0.99099e+03_rb,0.85129e+03_rb,0.71952e+03_rb,0.60234e+03_rb, &
        & 0.74410e+03_rb,0.88705e+03_rb,0.10299e+04_rb,0.11727e+04_rb /)
      kao(:, 1, 1,14) = (/ &
        & 0.20185e+04_rb,0.18044e+04_rb,0.14758e+04_rb,0.13047e+04_rb,0.10029e+04_rb, &
        & 0.77460e+03_rb,0.89226e+03_rb,0.10322e+04_rb,0.11721e+04_rb /)
      kao(:, 2, 1,14) = (/ &
        & 0.19173e+04_rb,0.17154e+04_rb,0.14467e+04_rb,0.12101e+04_rb,0.99594e+03_rb, &
        & 0.75026e+03_rb,0.88974e+03_rb,0.10295e+04_rb,0.11713e+04_rb /)
      kao(:, 3, 1,14) = (/ &
        & 0.19468e+04_rb,0.16851e+04_rb,0.14136e+04_rb,0.12474e+04_rb,0.94959e+03_rb, &
        & 0.74773e+03_rb,0.88696e+03_rb,0.10271e+04_rb,0.11663e+04_rb /)
      kao(:, 4, 1,14) = (/ &
        & 0.18423e+04_rb,0.16214e+04_rb,0.13991e+04_rb,0.11633e+04_rb,0.94698e+03_rb, &
        & 0.74507e+03_rb,0.88457e+03_rb,0.10239e+04_rb,0.11632e+04_rb /)
      kao(:, 5, 1,14) = (/ &
        & 0.17538e+04_rb,0.16376e+04_rb,0.13645e+04_rb,0.11303e+04_rb,0.98877e+03_rb, &
        & 0.74430e+03_rb,0.88372e+03_rb,0.10232e+04_rb,0.11626e+04_rb /)
      kao(:, 1, 2,14) = (/ &
        & 0.18551e+04_rb,0.16163e+04_rb,0.13640e+04_rb,0.12440e+04_rb,0.97095e+03_rb, &
        & 0.75153e+03_rb,0.89202e+03_rb,0.10323e+04_rb,0.11731e+04_rb /)
      kao(:, 2, 2,14) = (/ &
        & 0.18136e+04_rb,0.16209e+04_rb,0.13887e+04_rb,0.11127e+04_rb,0.90845e+03_rb, &
        & 0.74936e+03_rb,0.88974e+03_rb,0.10301e+04_rb,0.11702e+04_rb /)
      kao(:, 3, 2,14) = (/ &
        & 0.17918e+04_rb,0.15931e+04_rb,0.13282e+04_rb,0.11266e+04_rb,0.93052e+03_rb, &
        & 0.74732e+03_rb,0.88725e+03_rb,0.10243e+04_rb,0.11672e+04_rb /)
      kao(:, 4, 2,14) = (/ &
        & 0.17507e+04_rb,0.14791e+04_rb,0.12865e+04_rb,0.10818e+04_rb,0.87901e+03_rb, &
        & 0.74478e+03_rb,0.88467e+03_rb,0.10246e+04_rb,0.11645e+04_rb /)
      kao(:, 5, 2,14) = (/ &
        & 0.16872e+04_rb,0.14818e+04_rb,0.12871e+04_rb,0.10513e+04_rb,0.88744e+03_rb, &
        & 0.74325e+03_rb,0.88337e+03_rb,0.10230e+04_rb,0.11625e+04_rb /)
      kao(:, 1, 3,14) = (/ &
        & 0.16919e+04_rb,0.15533e+04_rb,0.13337e+04_rb,0.10831e+04_rb,0.86308e+03_rb, &
        & 0.75085e+03_rb,0.89217e+03_rb,0.10333e+04_rb,0.11744e+04_rb /)
      kao(:, 2, 3,14) = (/ &
        & 0.17034e+04_rb,0.15495e+04_rb,0.12643e+04_rb,0.11407e+04_rb,0.84365e+03_rb, &
        & 0.74929e+03_rb,0.89025e+03_rb,0.10312e+04_rb,0.11722e+04_rb /)
      kao(:, 3, 3,14) = (/ &
        & 0.16844e+04_rb,0.14282e+04_rb,0.12585e+04_rb,0.10790e+04_rb,0.84114e+03_rb, &
        & 0.74691e+03_rb,0.88785e+03_rb,0.10286e+04_rb,0.11700e+04_rb /)
      kao(:, 4, 3,14) = (/ &
        & 0.16791e+04_rb,0.14180e+04_rb,0.12209e+04_rb,0.10441e+04_rb,0.85899e+03_rb, &
        & 0.74519e+03_rb,0.88544e+03_rb,0.10259e+04_rb,0.11667e+04_rb /)
      kao(:, 5, 3,14) = (/ &
        & 0.16063e+04_rb,0.13774e+04_rb,0.12181e+04_rb,0.10006e+04_rb,0.82020e+03_rb, &
        & 0.74245e+03_rb,0.88287e+03_rb,0.10231e+04_rb,0.11633e+04_rb /)
      kao(:, 1, 4,14) = (/ &
        & 0.15960e+04_rb,0.13846e+04_rb,0.12078e+04_rb,0.10284e+04_rb,0.81079e+03_rb, &
        & 0.75078e+03_rb,0.89233e+03_rb,0.10336e+04_rb,0.11751e+04_rb /)
      kao(:, 2, 4,14) = (/ &
        & 0.16237e+04_rb,0.13919e+04_rb,0.11906e+04_rb,0.10115e+04_rb,0.83009e+03_rb, &
        & 0.74909e+03_rb,0.89076e+03_rb,0.10322e+04_rb,0.11737e+04_rb /)
      kao(:, 3, 4,14) = (/ &
        & 0.16136e+04_rb,0.13698e+04_rb,0.12046e+04_rb,0.98839e+03_rb,0.80997e+03_rb, &
        & 0.74745e+03_rb,0.88871e+03_rb,0.10300e+04_rb,0.11712e+04_rb /)
      kao(:, 4, 4,14) = (/ &
        & 0.15739e+04_rb,0.13753e+04_rb,0.11394e+04_rb,0.97871e+03_rb,0.77029e+03_rb, &
        & 0.74770e+03_rb,0.88627e+03_rb,0.10279e+04_rb,0.11682e+04_rb /)
      kao(:, 5, 4,14) = (/ &
        & 0.14863e+04_rb,0.13334e+04_rb,0.11445e+04_rb,0.95775e+03_rb,0.78998e+03_rb, &
        & 0.74317e+03_rb,0.88387e+03_rb,0.10246e+04_rb,0.11653e+04_rb /)
      kao(:, 1, 5,14) = (/ &
        & 0.15101e+04_rb,0.14006e+04_rb,0.11513e+04_rb,0.96783e+03_rb,0.79581e+03_rb, &
        & 0.75045e+03_rb,0.89239e+03_rb,0.10340e+04_rb,0.11759e+04_rb /)
      kao(:, 2, 5,14) = (/ &
        & 0.15461e+04_rb,0.13003e+04_rb,0.11685e+04_rb,0.92112e+03_rb,0.77517e+03_rb, &
        & 0.74927e+03_rb,0.89114e+03_rb,0.10330e+04_rb,0.11749e+04_rb /)
      kao(:, 3, 5,14) = (/ &
        & 0.14676e+04_rb,0.12966e+04_rb,0.11042e+04_rb,0.93875e+03_rb,0.79645e+03_rb, &
        & 0.74772e+03_rb,0.88943e+03_rb,0.10311e+04_rb,0.11729e+04_rb /)
      kao(:, 4, 5,14) = (/ &
        & 0.14571e+04_rb,0.12955e+04_rb,0.11734e+04_rb,0.89731e+03_rb,0.76986e+03_rb, &
        & 0.74578e+03_rb,0.88723e+03_rb,0.10287e+04_rb,0.11702e+04_rb /)
      kao(:, 5, 5,14) = (/ &
        & 0.14311e+04_rb,0.13054e+04_rb,0.10738e+04_rb,0.10079e+04_rb,0.72704e+03_rb, &
        & 0.74437e+03_rb,0.88461e+03_rb,0.10260e+04_rb,0.11675e+04_rb /)
      kao(:, 1, 6,14) = (/ &
        & 0.14715e+04_rb,0.12798e+04_rb,0.11182e+04_rb,0.92160e+03_rb,0.75379e+03_rb, &
        & 0.75023e+03_rb,0.89251e+03_rb,0.10348e+04_rb,0.11771e+04_rb /)
      kao(:, 2, 6,14) = (/ &
        & 0.14479e+04_rb,0.12414e+04_rb,0.11222e+04_rb,0.91289e+03_rb,0.75013e+03_rb, &
        & 0.74921e+03_rb,0.89141e+03_rb,0.10336e+04_rb,0.11758e+04_rb /)
      kao(:, 3, 6,14) = (/ &
        & 0.14270e+04_rb,0.12651e+04_rb,0.10793e+04_rb,0.91215e+03_rb,0.74649e+03_rb, &
        & 0.74796e+03_rb,0.89005e+03_rb,0.10321e+04_rb,0.11739e+04_rb /)
      kao(:, 4, 6,14) = (/ &
        & 0.14058e+04_rb,0.12702e+04_rb,0.11005e+04_rb,0.93809e+03_rb,0.69854e+03_rb, &
        & 0.74632e+03_rb,0.88820e+03_rb,0.10301e+04_rb,0.11720e+04_rb /)
      kao(:, 5, 6,14) = (/ &
        & 0.14048e+04_rb,0.12100e+04_rb,0.10413e+04_rb,0.84934e+03_rb,0.69794e+03_rb, &
        & 0.74429e+03_rb,0.88588e+03_rb,0.10284e+04_rb,0.11701e+04_rb /)
      kao(:, 1, 7,14) = (/ &
        & 0.14325e+04_rb,0.12413e+04_rb,0.10595e+04_rb,0.88906e+03_rb,0.71717e+03_rb, &
        & 0.75037e+03_rb,0.89273e+03_rb,0.10353e+04_rb,0.11779e+04_rb /)
      kao(:, 2, 7,14) = (/ &
        & 0.13620e+04_rb,0.12089e+04_rb,0.10598e+04_rb,0.89550e+03_rb,0.73134e+03_rb, &
        & 0.74911e+03_rb,0.89161e+03_rb,0.10341e+04_rb,0.11766e+04_rb /)
      kao(:, 3, 7,14) = (/ &
        & 0.13346e+04_rb,0.12111e+04_rb,0.10262e+04_rb,0.84775e+03_rb,0.68161e+03_rb, &
        & 0.74811e+03_rb,0.89050e+03_rb,0.10329e+04_rb,0.11753e+04_rb /)
      kao(:, 4, 7,14) = (/ &
        & 0.13132e+04_rb,0.11998e+04_rb,0.99757e+03_rb,0.86083e+03_rb,0.70251e+03_rb, &
        & 0.74653e+03_rb,0.88899e+03_rb,0.10312e+04_rb,0.11735e+04_rb /)
      kao(:, 5, 7,14) = (/ &
        & 0.13232e+04_rb,0.11634e+04_rb,0.10230e+04_rb,0.84353e+03_rb,0.68339e+03_rb, &
        & 0.74500e+03_rb,0.88698e+03_rb,0.10287e+04_rb,0.11709e+04_rb /)
      kao(:, 1, 8,14) = (/ &
        & 0.13366e+04_rb,0.11630e+04_rb,0.10188e+04_rb,0.86405e+03_rb,0.71800e+03_rb, &
        & 0.75008e+03_rb,0.89285e+03_rb,0.10357e+04_rb,0.11785e+04_rb /)
      kao(:, 2, 8,14) = (/ &
        & 0.13647e+04_rb,0.11518e+04_rb,0.10110e+04_rb,0.84719e+03_rb,0.67819e+03_rb, &
        & 0.74911e+03_rb,0.89187e+03_rb,0.10343e+04_rb,0.11777e+04_rb /)
      kao(:, 3, 8,14) = (/ &
        & 0.13128e+04_rb,0.11472e+04_rb,0.99038e+03_rb,0.82656e+03_rb,0.66483e+03_rb, &
        & 0.74815e+03_rb,0.89080e+03_rb,0.10335e+04_rb,0.11761e+04_rb /)
      kao(:, 4, 8,14) = (/ &
        & 0.12817e+04_rb,0.11127e+04_rb,0.10092e+04_rb,0.82047e+03_rb,0.65442e+03_rb, &
        & 0.74685e+03_rb,0.88933e+03_rb,0.10318e+04_rb,0.11746e+04_rb /)
      kao(:, 5, 8,14) = (/ &
        & 0.13162e+04_rb,0.11356e+04_rb,0.98305e+03_rb,0.84668e+03_rb,0.66544e+03_rb, &
        & 0.74558e+03_rb,0.88790e+03_rb,0.10302e+04_rb,0.11725e+04_rb /)
      kao(:, 1, 9,14) = (/ &
        & 0.13174e+04_rb,0.11399e+04_rb,0.97696e+03_rb,0.84117e+03_rb,0.66145e+03_rb, &
        & 0.75011e+03_rb,0.89307e+03_rb,0.10362e+04_rb,0.11789e+04_rb /)
      kao(:, 2, 9,14) = (/ &
        & 0.13196e+04_rb,0.11310e+04_rb,0.99094e+03_rb,0.80674e+03_rb,0.67265e+03_rb, &
        & 0.74915e+03_rb,0.89213e+03_rb,0.10351e+04_rb,0.11781e+04_rb /)
      kao(:, 3, 9,14) = (/ &
        & 0.12950e+04_rb,0.10983e+04_rb,0.10229e+04_rb,0.86051e+03_rb,0.66007e+03_rb, &
        & 0.74819e+03_rb,0.89107e+03_rb,0.10339e+04_rb,0.11768e+04_rb /)
      kao(:, 4, 9,14) = (/ &
        & 0.12605e+04_rb,0.10785e+04_rb,0.10061e+04_rb,0.79531e+03_rb,0.63414e+03_rb, &
        & 0.74728e+03_rb,0.89004e+03_rb,0.10328e+04_rb,0.11756e+04_rb /)
      kao(:, 5, 9,14) = (/ &
        & 0.12553e+04_rb,0.10628e+04_rb,0.94908e+03_rb,0.82074e+03_rb,0.64983e+03_rb, &
        & 0.74606e+03_rb,0.88866e+03_rb,0.10313e+04_rb,0.11739e+04_rb /)
      kao(:, 1,10,14) = (/ &
        & 0.13413e+04_rb,0.11632e+04_rb,0.10048e+04_rb,0.80476e+03_rb,0.65189e+03_rb, &
        & 0.75022e+03_rb,0.89354e+03_rb,0.10369e+04_rb,0.11806e+04_rb /)
      kao(:, 2,10,14) = (/ &
        & 0.12838e+04_rb,0.11017e+04_rb,0.10173e+04_rb,0.85213e+03_rb,0.64367e+03_rb, &
        & 0.74909e+03_rb,0.89223e+03_rb,0.10354e+04_rb,0.11786e+04_rb /)
      kao(:, 3,10,14) = (/ &
        & 0.12657e+04_rb,0.11229e+04_rb,0.92134e+03_rb,0.80214e+03_rb,0.64195e+03_rb, &
        & 0.74870e+03_rb,0.89159e+03_rb,0.10344e+04_rb,0.11778e+04_rb /)
      kao(:, 4,10,14) = (/ &
        & 0.12310e+04_rb,0.10594e+04_rb,0.92792e+03_rb,0.78251e+03_rb,0.62401e+03_rb, &
        & 0.74734e+03_rb,0.89003e+03_rb,0.10332e+04_rb,0.11762e+04_rb /)
      kao(:, 5,10,14) = (/ &
        & 0.12088e+04_rb,0.10663e+04_rb,0.92275e+03_rb,0.77707e+03_rb,0.64354e+03_rb, &
        & 0.74634e+03_rb,0.88914e+03_rb,0.10320e+04_rb,0.11748e+04_rb /)
      kao(:, 1,11,14) = (/ &
        & 0.12205e+04_rb,0.10982e+04_rb,0.93349e+03_rb,0.79899e+03_rb,0.64961e+03_rb, &
        & 0.74981e+03_rb,0.89329e+03_rb,0.10368e+04_rb,0.11803e+04_rb /)
      kao(:, 2,11,14) = (/ &
        & 0.12266e+04_rb,0.10977e+04_rb,0.90836e+03_rb,0.76723e+03_rb,0.61851e+03_rb, &
        & 0.74871e+03_rb,0.89199e+03_rb,0.10353e+04_rb,0.11786e+04_rb /)
      kao(:, 3,11,14) = (/ &
        & 0.11876e+04_rb,0.10750e+04_rb,0.92258e+03_rb,0.76849e+03_rb,0.61349e+03_rb, &
        & 0.74902e+03_rb,0.89163e+03_rb,0.10343e+04_rb,0.11775e+04_rb /)
      kao(:, 4,11,14) = (/ &
        & 0.12363e+04_rb,0.10327e+04_rb,0.91965e+03_rb,0.74736e+03_rb,0.60206e+03_rb, &
        & 0.74703e+03_rb,0.88982e+03_rb,0.10331e+04_rb,0.11762e+04_rb /)
      kao(:, 5,11,14) = (/ &
        & 0.11827e+04_rb,0.10266e+04_rb,0.89018e+03_rb,0.76668e+03_rb,0.60360e+03_rb, &
        & 0.74604e+03_rb,0.88895e+03_rb,0.10316e+04_rb,0.11748e+04_rb /)
      kao(:, 1,12,14) = (/ &
        & 0.11808e+04_rb,0.10792e+04_rb,0.88334e+03_rb,0.76947e+03_rb,0.60663e+03_rb, &
        & 0.74904e+03_rb,0.89307e+03_rb,0.10367e+04_rb,0.11803e+04_rb /)
      kao(:, 2,12,14) = (/ &
        & 0.12120e+04_rb,0.10402e+04_rb,0.92096e+03_rb,0.78880e+03_rb,0.60609e+03_rb, &
        & 0.74841e+03_rb,0.89179e+03_rb,0.10352e+04_rb,0.11786e+04_rb /)
      kao(:, 3,12,14) = (/ &
        & 0.12016e+04_rb,0.10899e+04_rb,0.90018e+03_rb,0.73017e+03_rb,0.60527e+03_rb, &
        & 0.75028e+03_rb,0.89119e+03_rb,0.10342e+04_rb,0.11778e+04_rb /)
      kao(:, 4,12,14) = (/ &
        & 0.11901e+04_rb,0.10101e+04_rb,0.88929e+03_rb,0.75342e+03_rb,0.60426e+03_rb, &
        & 0.74677e+03_rb,0.88991e+03_rb,0.10327e+04_rb,0.11762e+04_rb /)
      kao(:, 5,12,14) = (/ &
        & 0.12404e+04_rb,0.10285e+04_rb,0.87283e+03_rb,0.77029e+03_rb,0.60316e+03_rb, &
        & 0.74579e+03_rb,0.88852e+03_rb,0.10318e+04_rb,0.11748e+04_rb /)
      kao(:, 1,13,14) = (/ &
        & 0.11819e+04_rb,0.10465e+04_rb,0.87588e+03_rb,0.72550e+03_rb,0.60649e+03_rb, &
        & 0.74922e+03_rb,0.89289e+03_rb,0.10366e+04_rb,0.11799e+04_rb /)
      kao(:, 2,13,14) = (/ &
        & 0.11759e+04_rb,0.10245e+04_rb,0.86816e+03_rb,0.77411e+03_rb,0.60565e+03_rb, &
        & 0.74816e+03_rb,0.89163e+03_rb,0.10351e+04_rb,0.11786e+04_rb /)
      kao(:, 3,13,14) = (/ &
        & 0.11354e+04_rb,0.10124e+04_rb,0.96299e+03_rb,0.72358e+03_rb,0.60486e+03_rb, &
        & 0.74851e+03_rb,0.89156e+03_rb,0.10341e+04_rb,0.11778e+04_rb /)
      kao(:, 4,13,14) = (/ &
        & 0.11811e+04_rb,0.10736e+04_rb,0.85638e+03_rb,0.74549e+03_rb,0.60366e+03_rb, &
        & 0.74655e+03_rb,0.88977e+03_rb,0.10330e+04_rb,0.11762e+04_rb /)
      kao(:, 5,13,14) = (/ &
        & 0.11332e+04_rb,0.99282e+03_rb,0.87259e+03_rb,0.77082e+03_rb,0.60280e+03_rb, &
        & 0.74559e+03_rb,0.88865e+03_rb,0.10317e+04_rb,0.11748e+04_rb /)
      kao(:, 1, 1,15) = (/ &
        & 0.19847e+04_rb,0.16615e+04_rb,0.16299e+04_rb,0.11466e+04_rb,0.10123e+04_rb, &
        & 0.72871e+03_rb,0.89405e+03_rb,0.10334e+04_rb,0.11734e+04_rb /)
      kao(:, 2, 1,15) = (/ &
        & 0.18867e+04_rb,0.16470e+04_rb,0.14051e+04_rb,0.12848e+04_rb,0.91870e+03_rb, &
        & 0.75104e+03_rb,0.89013e+03_rb,0.10307e+04_rb,0.11650e+04_rb /)
      kao(:, 3, 1,15) = (/ &
        & 0.17405e+04_rb,0.16240e+04_rb,0.14364e+04_rb,0.10773e+04_rb,0.10008e+04_rb, &
        & 0.74790e+03_rb,0.89023e+03_rb,0.10382e+04_rb,0.11720e+04_rb /)
      kao(:, 4, 1,15) = (/ &
        & 0.19099e+04_rb,0.16930e+04_rb,0.14454e+04_rb,0.12410e+04_rb,0.95925e+03_rb, &
        & 0.74605e+03_rb,0.88544e+03_rb,0.10248e+04_rb,0.11651e+04_rb /)
      kao(:, 5, 1,15) = (/ &
        & 0.19738e+04_rb,0.13892e+04_rb,0.14377e+04_rb,0.12129e+04_rb,0.77083e+03_rb, &
        & 0.74496e+03_rb,0.88458e+03_rb,0.10242e+04_rb,0.11638e+04_rb /)
      kao(:, 1, 2,15) = (/ &
        & 0.18788e+04_rb,0.15414e+04_rb,0.14391e+04_rb,0.94831e+03_rb,0.88384e+03_rb, &
        & 0.75320e+03_rb,0.89399e+03_rb,0.10340e+04_rb,0.11746e+04_rb /)
      kao(:, 2, 2,15) = (/ &
        & 0.16660e+04_rb,0.14507e+04_rb,0.12145e+04_rb,0.11753e+04_rb,0.91977e+03_rb, &
        & 0.75003e+03_rb,0.89113e+03_rb,0.10308e+04_rb,0.11712e+04_rb /)
      kao(:, 3, 2,15) = (/ &
        & 0.15746e+04_rb,0.14551e+04_rb,0.13375e+04_rb,0.10698e+04_rb,0.76931e+03_rb, &
        & 0.74657e+03_rb,0.88729e+03_rb,0.10267e+04_rb,0.11695e+04_rb /)
      kao(:, 4, 2,15) = (/ &
        & 0.16772e+04_rb,0.16415e+04_rb,0.13391e+04_rb,0.10287e+04_rb,0.88207e+03_rb, &
        & 0.74593e+03_rb,0.88530e+03_rb,0.10252e+04_rb,0.11661e+04_rb /)
      kao(:, 5, 2,15) = (/ &
        & 0.16581e+04_rb,0.15452e+04_rb,0.12479e+04_rb,0.10067e+04_rb,0.83698e+03_rb, &
        & 0.74419e+03_rb,0.88279e+03_rb,0.10234e+04_rb,0.11634e+04_rb /)
      kao(:, 1, 3,15) = (/ &
        & 0.17888e+04_rb,0.13395e+04_rb,0.12224e+04_rb,0.11479e+04_rb,0.91734e+03_rb, &
        & 0.75291e+03_rb,0.89358e+03_rb,0.10348e+04_rb,0.11761e+04_rb /)
      kao(:, 2, 3,15) = (/ &
        & 0.16146e+04_rb,0.11805e+04_rb,0.12709e+04_rb,0.85911e+03_rb,0.90210e+03_rb, &
        & 0.75028e+03_rb,0.89203e+03_rb,0.10324e+04_rb,0.11734e+04_rb /)
      kao(:, 3, 3,15) = (/ &
        & 0.14717e+04_rb,0.15353e+04_rb,0.12224e+04_rb,0.96001e+03_rb,0.77554e+03_rb, &
        & 0.74753e+03_rb,0.88823e+03_rb,0.10297e+04_rb,0.11678e+04_rb /)
      kao(:, 4, 3,15) = (/ &
        & 0.15479e+04_rb,0.14535e+04_rb,0.11756e+04_rb,0.99609e+03_rb,0.71892e+03_rb, &
        & 0.74634e+03_rb,0.88703e+03_rb,0.10366e+04_rb,0.11684e+04_rb /)
      kao(:, 5, 3,15) = (/ &
        & 0.15155e+04_rb,0.14515e+04_rb,0.11733e+04_rb,0.10638e+04_rb,0.80510e+03_rb, &
        & 0.74461e+03_rb,0.88580e+03_rb,0.10240e+04_rb,0.11652e+04_rb /)
      kao(:, 1, 4,15) = (/ &
        & 0.15848e+04_rb,0.14083e+04_rb,0.12006e+04_rb,0.10439e+04_rb,0.81336e+03_rb, &
        & 0.75201e+03_rb,0.89444e+03_rb,0.10363e+04_rb,0.11781e+04_rb /)
      kao(:, 2, 4,15) = (/ &
        & 0.13736e+04_rb,0.13760e+04_rb,0.12165e+04_rb,0.94289e+03_rb,0.75006e+03_rb, &
        & 0.75167e+03_rb,0.89210e+03_rb,0.10337e+04_rb,0.11753e+04_rb /)
      kao(:, 3, 4,15) = (/ &
        & 0.14353e+04_rb,0.13702e+04_rb,0.10217e+04_rb,0.10119e+04_rb,0.72585e+03_rb, &
        & 0.74883e+03_rb,0.88956e+03_rb,0.10325e+04_rb,0.11731e+04_rb /)
      kao(:, 4, 4,15) = (/ &
        & 0.14295e+04_rb,0.12548e+04_rb,0.12198e+04_rb,0.96945e+03_rb,0.84557e+03_rb, &
        & 0.73846e+03_rb,0.88641e+03_rb,0.10258e+04_rb,0.11703e+04_rb /)
      kao(:, 5, 4,15) = (/ &
        & 0.15742e+04_rb,0.13268e+04_rb,0.10395e+04_rb,0.90301e+03_rb,0.67604e+03_rb, &
        & 0.74473e+03_rb,0.88569e+03_rb,0.10266e+04_rb,0.11676e+04_rb /)
      kao(:, 1, 5,15) = (/ &
        & 0.15038e+04_rb,0.11787e+04_rb,0.11191e+04_rb,0.96573e+03_rb,0.74364e+03_rb, &
        & 0.75161e+03_rb,0.89373e+03_rb,0.10358e+04_rb,0.11780e+04_rb /)
      kao(:, 2, 5,15) = (/ &
        & 0.12972e+04_rb,0.14157e+04_rb,0.10073e+04_rb,0.10120e+04_rb,0.78829e+03_rb, &
        & 0.75051e+03_rb,0.89254e+03_rb,0.10346e+04_rb,0.11785e+04_rb /)
      kao(:, 3, 5,15) = (/ &
        & 0.15562e+04_rb,0.12980e+04_rb,0.11053e+04_rb,0.96784e+03_rb,0.68156e+03_rb, &
        & 0.74938e+03_rb,0.89063e+03_rb,0.10325e+04_rb,0.11743e+04_rb /)
      kao(:, 4, 5,15) = (/ &
        & 0.15203e+04_rb,0.12953e+04_rb,0.96757e+03_rb,0.95061e+03_rb,0.68480e+03_rb, &
        & 0.74750e+03_rb,0.88780e+03_rb,0.10293e+04_rb,0.11708e+04_rb /)
      kao(:, 5, 5,15) = (/ &
        & 0.14730e+04_rb,0.11456e+04_rb,0.11151e+04_rb,0.68408e+03_rb,0.76176e+03_rb, &
        & 0.75307e+03_rb,0.89322e+03_rb,0.10317e+04_rb,0.11682e+04_rb /)
      kao(:, 1, 6,15) = (/ &
        & 0.15240e+04_rb,0.12794e+04_rb,0.10826e+04_rb,0.93131e+03_rb,0.69525e+03_rb, &
        & 0.75122e+03_rb,0.89364e+03_rb,0.10361e+04_rb,0.11785e+04_rb /)
      kao(:, 2, 6,15) = (/ &
        & 0.13735e+04_rb,0.12837e+04_rb,0.10725e+04_rb,0.91616e+03_rb,0.69548e+03_rb, &
        & 0.75041e+03_rb,0.89279e+03_rb,0.10360e+04_rb,0.11776e+04_rb /)
      kao(:, 3, 6,15) = (/ &
        & 0.14287e+04_rb,0.11366e+04_rb,0.10240e+04_rb,0.87529e+03_rb,0.70795e+03_rb, &
        & 0.75034e+03_rb,0.89142e+03_rb,0.10337e+04_rb,0.11759e+04_rb /)
      kao(:, 4, 6,15) = (/ &
        & 0.12768e+04_rb,0.11078e+04_rb,0.98805e+03_rb,0.76254e+03_rb,0.77383e+03_rb, &
        & 0.74782e+03_rb,0.88921e+03_rb,0.10312e+04_rb,0.11732e+04_rb /)
      kao(:, 5, 6,15) = (/ &
        & 0.13045e+04_rb,0.12087e+04_rb,0.10585e+04_rb,0.89185e+03_rb,0.69378e+03_rb, &
        & 0.74461e+03_rb,0.88618e+03_rb,0.10253e+04_rb,0.11684e+04_rb /)
      kao(:, 1, 7,15) = (/ &
        & 0.13068e+04_rb,0.11778e+04_rb,0.10485e+04_rb,0.87245e+03_rb,0.70237e+03_rb, &
        & 0.75029e+03_rb,0.89358e+03_rb,0.10363e+04_rb,0.11790e+04_rb /)
      kao(:, 2, 7,15) = (/ &
        & 0.13988e+04_rb,0.11525e+04_rb,0.98924e+03_rb,0.83779e+03_rb,0.64617e+03_rb, &
        & 0.75022e+03_rb,0.89357e+03_rb,0.10355e+04_rb,0.11782e+04_rb /)
      kao(:, 3, 7,15) = (/ &
        & 0.14294e+04_rb,0.11695e+04_rb,0.10182e+04_rb,0.91196e+03_rb,0.69696e+03_rb, &
        & 0.74932e+03_rb,0.89330e+03_rb,0.10353e+04_rb,0.11780e+04_rb /)
      kao(:, 4, 7,15) = (/ &
        & 0.14082e+04_rb,0.10640e+04_rb,0.10540e+04_rb,0.79295e+03_rb,0.62688e+03_rb, &
        & 0.74789e+03_rb,0.89097e+03_rb,0.10327e+04_rb,0.11750e+04_rb /)
      kao(:, 5, 7,15) = (/ &
        & 0.13514e+04_rb,0.12038e+04_rb,0.10012e+04_rb,0.83544e+03_rb,0.67927e+03_rb, &
        & 0.74629e+03_rb,0.88843e+03_rb,0.10298e+04_rb,0.11718e+04_rb /)
      kao(:, 1, 8,15) = (/ &
        & 0.14761e+04_rb,0.12412e+04_rb,0.10429e+04_rb,0.84370e+03_rb,0.62388e+03_rb, &
        & 0.75133e+03_rb,0.89371e+03_rb,0.10367e+04_rb,0.11796e+04_rb /)
      kao(:, 2, 8,15) = (/ &
        & 0.12119e+04_rb,0.11409e+04_rb,0.10121e+04_rb,0.85755e+03_rb,0.70976e+03_rb, &
        & 0.74998e+03_rb,0.89216e+03_rb,0.10357e+04_rb,0.11786e+04_rb /)
      kao(:, 3, 8,15) = (/ &
        & 0.12581e+04_rb,0.11663e+04_rb,0.10198e+04_rb,0.88649e+03_rb,0.69509e+03_rb, &
        & 0.74932e+03_rb,0.89355e+03_rb,0.10350e+04_rb,0.11787e+04_rb /)
      kao(:, 4, 8,15) = (/ &
        & 0.12478e+04_rb,0.12066e+04_rb,0.87201e+03_rb,0.84155e+03_rb,0.70223e+03_rb, &
        & 0.74886e+03_rb,0.89098e+03_rb,0.10337e+04_rb,0.11764e+04_rb /)
      kao(:, 5, 8,15) = (/ &
        & 0.12322e+04_rb,0.11048e+04_rb,0.93070e+03_rb,0.68296e+03_rb,0.66022e+03_rb, &
        & 0.74776e+03_rb,0.88972e+03_rb,0.10315e+04_rb,0.11748e+04_rb /)
      kao(:, 1, 9,15) = (/ &
        & 0.13164e+04_rb,0.12093e+04_rb,0.10729e+04_rb,0.80022e+03_rb,0.68605e+03_rb, &
        & 0.75075e+03_rb,0.89396e+03_rb,0.10372e+04_rb,0.11804e+04_rb /)
      kao(:, 2, 9,15) = (/ &
        & 0.12738e+04_rb,0.11851e+04_rb,0.88381e+03_rb,0.85846e+03_rb,0.65813e+03_rb, &
        & 0.74985e+03_rb,0.89295e+03_rb,0.10369e+04_rb,0.11792e+04_rb /)
      kao(:, 3, 9,15) = (/ &
        & 0.12657e+04_rb,0.12031e+04_rb,0.79849e+03_rb,0.64636e+03_rb,0.60824e+03_rb, &
        & 0.74924e+03_rb,0.89227e+03_rb,0.10353e+04_rb,0.11783e+04_rb /)
      kao(:, 4, 9,15) = (/ &
        & 0.12866e+04_rb,0.11531e+04_rb,0.85958e+03_rb,0.76890e+03_rb,0.66204e+03_rb, &
        & 0.74965e+03_rb,0.89143e+03_rb,0.10344e+04_rb,0.11773e+04_rb /)
      kao(:, 5, 9,15) = (/ &
        & 0.11604e+04_rb,0.11730e+04_rb,0.87009e+03_rb,0.73148e+03_rb,0.58106e+03_rb, &
        & 0.74723e+03_rb,0.88998e+03_rb,0.10327e+04_rb,0.11755e+04_rb /)
      kao(:, 1,10,15) = (/ &
        & 0.11188e+04_rb,0.99215e+03_rb,0.73460e+03_rb,0.83046e+03_rb,0.68844e+03_rb, &
        & 0.75085e+03_rb,0.89429e+03_rb,0.10377e+04_rb,0.11803e+04_rb /)
      kao(:, 2,10,15) = (/ &
        & 0.12059e+04_rb,0.10804e+04_rb,0.70761e+03_rb,0.65712e+03_rb,0.66535e+03_rb, &
        & 0.74980e+03_rb,0.89311e+03_rb,0.10364e+04_rb,0.11807e+04_rb /)
      kao(:, 3,10,15) = (/ &
        & 0.11817e+04_rb,0.10195e+04_rb,0.95365e+03_rb,0.71601e+03_rb,0.59959e+03_rb, &
        & 0.74791e+03_rb,0.89228e+03_rb,0.10355e+04_rb,0.11778e+04_rb /)
      kao(:, 4,10,15) = (/ &
        & 0.11350e+04_rb,0.11533e+04_rb,0.91058e+03_rb,0.81845e+03_rb,0.64874e+03_rb, &
        & 0.74908e+03_rb,0.89233e+03_rb,0.10356e+04_rb,0.11779e+04_rb /)
      kao(:, 5,10,15) = (/ &
        & 0.12000e+04_rb,0.10896e+04_rb,0.93671e+03_rb,0.81314e+03_rb,0.60750e+03_rb, &
        & 0.74754e+03_rb,0.89053e+03_rb,0.10335e+04_rb,0.11765e+04_rb /)
      kao(:, 1,11,15) = (/ &
        & 0.12887e+04_rb,0.10795e+04_rb,0.92507e+03_rb,0.76020e+03_rb,0.61621e+03_rb, &
        & 0.75046e+03_rb,0.89404e+03_rb,0.10376e+04_rb,0.11812e+04_rb /)
      kao(:, 2,11,15) = (/ &
        & 0.11864e+04_rb,0.99352e+03_rb,0.96688e+03_rb,0.81999e+03_rb,0.65449e+03_rb, &
        & 0.74944e+03_rb,0.89287e+03_rb,0.10363e+04_rb,0.11797e+04_rb /)
      kao(:, 3,11,15) = (/ &
        & 0.11946e+04_rb,0.10069e+04_rb,0.91904e+03_rb,0.81170e+03_rb,0.65606e+03_rb, &
        & 0.74582e+03_rb,0.89066e+03_rb,0.10354e+04_rb,0.11787e+04_rb /)
      kao(:, 4,11,15) = (/ &
        & 0.10269e+04_rb,0.10463e+04_rb,0.82697e+03_rb,0.81731e+03_rb,0.61185e+03_rb, &
        & 0.74876e+03_rb,0.89142e+03_rb,0.10347e+04_rb,0.11779e+04_rb /)
      kao(:, 5,11,15) = (/ &
        & 0.11483e+04_rb,0.10490e+04_rb,0.91718e+03_rb,0.72295e+03_rb,0.60510e+03_rb, &
        & 0.74783e+03_rb,0.89173e+03_rb,0.10334e+04_rb,0.11765e+04_rb /)
      kao(:, 1,12,15) = (/ &
        & 0.12157e+04_rb,0.94948e+03_rb,0.89441e+03_rb,0.76380e+03_rb,0.60866e+03_rb, &
        & 0.75013e+03_rb,0.89382e+03_rb,0.10375e+04_rb,0.11812e+04_rb /)
      kao(:, 2,12,15) = (/ &
        & 0.10470e+04_rb,0.10059e+04_rb,0.83704e+03_rb,0.70161e+03_rb,0.60721e+03_rb, &
        & 0.74914e+03_rb,0.89267e+03_rb,0.10370e+04_rb,0.11807e+04_rb /)
      kao(:, 3,12,15) = (/ &
        & 0.10541e+04_rb,0.86865e+03_rb,0.86614e+03_rb,0.79420e+03_rb,0.60576e+03_rb, &
        & 0.74144e+03_rb,0.89117e+03_rb,0.10353e+04_rb,0.11778e+04_rb /)
      kao(:, 4,12,15) = (/ &
        & 0.11152e+04_rb,0.10946e+04_rb,0.91200e+03_rb,0.75719e+03_rb,0.60459e+03_rb, &
        & 0.74791e+03_rb,0.89124e+03_rb,0.10346e+04_rb,0.11788e+04_rb /)
      kao(:, 5,12,15) = (/ &
        & 0.10232e+04_rb,0.97767e+03_rb,0.95862e+03_rb,0.62268e+03_rb,0.60382e+03_rb, &
        & 0.74758e+03_rb,0.88947e+03_rb,0.10333e+04_rb,0.11765e+04_rb /)
      kao(:, 1,13,15) = (/ &
        & 0.11027e+04_rb,0.93786e+03_rb,0.84040e+03_rb,0.77686e+03_rb,0.60858e+03_rb, &
        & 0.74987e+03_rb,0.89364e+03_rb,0.10374e+04_rb,0.11812e+04_rb /)
      kao(:, 2,13,15) = (/ &
        & 0.10816e+04_rb,0.10747e+04_rb,0.88690e+03_rb,0.66213e+03_rb,0.60670e+03_rb, &
        & 0.74889e+03_rb,0.89251e+03_rb,0.10369e+04_rb,0.11797e+04_rb /)
      kao(:, 3,13,15) = (/ &
        & 0.11963e+04_rb,0.98701e+03_rb,0.69767e+03_rb,0.73646e+03_rb,0.60528e+03_rb, &
        & 0.74531e+03_rb,0.88962e+03_rb,0.10352e+04_rb,0.11778e+04_rb /)
      kao(:, 4,13,15) = (/ &
        & 0.10530e+04_rb,0.78530e+03_rb,0.91477e+03_rb,0.71026e+03_rb,0.60430e+03_rb, &
        & 0.74770e+03_rb,0.89110e+03_rb,0.10345e+04_rb,0.11779e+04_rb /)
      kao(:, 5,13,15) = (/ &
        & 0.11409e+04_rb,0.10574e+04_rb,0.83207e+03_rb,0.61034e+03_rb,0.60355e+03_rb, &
        & 0.74679e+03_rb,0.89073e+03_rb,0.10333e+04_rb,0.11765e+04_rb /)
      kao(:, 1, 1,16) = (/ &
        & 0.34848e+03_rb,0.49670e+03_rb,0.29569e+03_rb,0.31627e+03_rb,0.46514e+03_rb, &
        & 0.42834e+03_rb,0.88848e+03_rb,0.10335e+04_rb,0.11735e+04_rb /)
      kao(:, 2, 1,16) = (/ &
        & 0.18044e+04_rb,0.87518e+03_rb,0.11930e+04_rb,0.45580e+03_rb,0.83690e+03_rb, &
        & 0.74631e+03_rb,0.89022e+03_rb,0.10242e+04_rb,0.11632e+04_rb /)
      kao(:, 3, 1,16) = (/ &
        & 0.11196e+04_rb,0.73658e+03_rb,0.10480e+04_rb,0.53695e+03_rb,0.39566e+03_rb, &
        & 0.74377e+03_rb,0.86775e+03_rb,0.94144e+03_rb,0.11277e+04_rb /)
      kao(:, 4, 1,16) = (/ &
        & 0.72013e+03_rb,0.40040e+03_rb,0.39030e+03_rb,0.31401e+03_rb,0.31498e+03_rb, &
        & 0.74613e+03_rb,0.88553e+03_rb,0.10249e+04_rb,0.11579e+04_rb /)
      kao(:, 5, 1,16) = (/ &
        & 0.34789e+03_rb,0.14911e+04_rb,0.33003e+03_rb,0.29522e+03_rb,0.31385e+03_rb, &
        & 0.74516e+03_rb,0.88480e+03_rb,0.10244e+04_rb,0.11641e+04_rb /)
      kao(:, 1, 2,16) = (/ &
        & 0.41247e+03_rb,0.81197e+03_rb,0.68322e+03_rb,0.50855e+03_rb,0.24076e+03_rb, &
        & 0.74851e+03_rb,0.88842e+03_rb,0.10341e+04_rb,0.11748e+04_rb /)
      kao(:, 2, 2,16) = (/ &
        & 0.16209e+04_rb,0.56884e+03_rb,0.94156e+03_rb,0.29182e+03_rb,0.46039e+03_rb, &
        & 0.75011e+03_rb,0.88557e+03_rb,0.10309e+04_rb,0.11714e+04_rb /)
      kao(:, 3, 2,16) = (/ &
        & 0.11534e+04_rb,0.51443e+03_rb,0.23941e+03_rb,0.44358e+03_rb,0.58779e+03_rb, &
        & 0.74720e+03_rb,0.88237e+03_rb,0.10275e+04_rb,0.11547e+04_rb /)
      kao(:, 4, 2,16) = (/ &
        & 0.78193e+03_rb,0.30585e+03_rb,0.42670e+03_rb,0.10365e+04_rb,0.43225e+03_rb, &
        & 0.74126e+03_rb,0.88535e+03_rb,0.10253e+04_rb,0.11588e+04_rb /)
      kao(:, 5, 2,16) = (/ &
        & 0.67509e+03_rb,0.40574e+03_rb,0.57252e+03_rb,0.10353e+04_rb,0.15564e+03_rb, &
        & 0.73954e+03_rb,0.88360e+03_rb,0.10236e+04_rb,0.11636e+04_rb /)
      kao(:, 1, 3,16) = (/ &
        & 0.91257e+03_rb,0.11144e+04_rb,0.54685e+03_rb,0.19037e+03_rb,0.25900e+03_rb, &
        & 0.74823e+03_rb,0.89369e+03_rb,0.10350e+04_rb,0.11762e+04_rb /)
      kao(:, 2, 3,16) = (/ &
        & 0.67162e+03_rb,0.10092e+04_rb,0.40983e+03_rb,0.37826e+03_rb,0.42702e+03_rb, &
        & 0.75035e+03_rb,0.88647e+03_rb,0.10325e+04_rb,0.11736e+04_rb /)
      kao(:, 3, 3,16) = (/ &
        & 0.94657e+03_rb,0.26107e+03_rb,0.43567e+03_rb,0.41065e+03_rb,0.70838e+03_rb, &
        & 0.74760e+03_rb,0.88832e+03_rb,0.10233e+04_rb,0.11698e+04_rb /)
      kao(:, 4, 3,16) = (/ &
        & 0.10618e+03_rb,0.45040e+03_rb,0.10291e+04_rb,0.30237e+03_rb,0.57813e+03_rb, &
        & 0.73277e+03_rb,0.87582e+03_rb,0.95203e+03_rb,0.11472e+04_rb /)
      kao(:, 5, 3,16) = (/ &
        & 0.96551e+03_rb,0.68706e+03_rb,0.56500e+03_rb,0.14865e+03_rb,0.35532e+03_rb, &
        & 0.73527e+03_rb,0.86908e+03_rb,0.10241e+04_rb,0.11579e+04_rb /)
      kao(:, 1, 4,16) = (/ &
        & 0.13211e+04_rb,0.11050e+04_rb,0.91067e+03_rb,0.34788e+03_rb,0.76485e+03_rb, &
        & 0.75213e+03_rb,0.88892e+03_rb,0.10299e+04_rb,0.11709e+04_rb /)
      kao(:, 2, 4,16) = (/ &
        & 0.15543e+04_rb,0.92211e+03_rb,0.72798e+03_rb,0.86839e+03_rb,0.36802e+03_rb, &
        & 0.74223e+03_rb,0.89219e+03_rb,0.10338e+04_rb,0.11754e+04_rb /)
      kao(:, 3, 4,16) = (/ &
        & 0.53676e+03_rb,0.46363e+03_rb,0.10941e+04_rb,0.19958e+03_rb,0.68767e+03_rb, &
        & 0.74416e+03_rb,0.88965e+03_rb,0.10195e+04_rb,0.11658e+04_rb /)
      kao(:, 4, 4,16) = (/ &
        & 0.66992e+03_rb,0.61007e+03_rb,0.50612e+03_rb,0.34999e+03_rb,0.19685e+03_rb, &
        & 0.74139e+03_rb,0.88155e+03_rb,0.10274e+04_rb,0.11555e+04_rb /)
      kao(:, 5, 4,16) = (/ &
        & 0.55226e+03_rb,0.42689e+03_rb,0.10712e+04_rb,0.69959e+03_rb,0.69105e+03_rb, &
        & 0.73534e+03_rb,0.87452e+03_rb,0.10137e+04_rb,0.11529e+04_rb /)
      kao(:, 1, 5,16) = (/ &
        & 0.11854e+04_rb,0.90025e+03_rb,0.10734e+04_rb,0.36732e+03_rb,0.51551e+03_rb, &
        & 0.75181e+03_rb,0.89397e+03_rb,0.10361e+04_rb,0.11783e+04_rb /)
      kao(:, 2, 5,16) = (/ &
        & 0.13933e+04_rb,0.23882e+03_rb,0.71015e+03_rb,0.29765e+03_rb,0.88507e+02_rb, &
        & 0.75060e+03_rb,0.89266e+03_rb,0.10347e+04_rb,0.11637e+04_rb /)
      kao(:, 3, 5,16) = (/ &
        & 0.30783e+03_rb,0.72533e+03_rb,0.99782e+03_rb,0.22581e+03_rb,0.21957e+03_rb, &
        & 0.74471e+03_rb,0.89072e+03_rb,0.10326e+04_rb,0.11744e+04_rb /)
      kao(:, 4, 5,16) = (/ &
        & 0.12756e+03_rb,0.23797e+03_rb,0.16871e+03_rb,0.60625e+03_rb,0.35893e+03_rb, &
        & 0.73811e+03_rb,0.88789e+03_rb,0.10294e+04_rb,0.11709e+04_rb /)
      kao(:, 5, 5,16) = (/ &
        & 0.67577e+03_rb,0.67606e+03_rb,0.57612e+03_rb,0.15439e+03_rb,0.22922e+03_rb, &
        & 0.66526e+03_rb,0.82593e+03_rb,0.98621e+03_rb,0.11543e+04_rb /)
      kao(:, 1, 6,16) = (/ &
        & 0.35294e+03_rb,0.92951e+03_rb,0.39404e+03_rb,0.59466e+03_rb,0.46365e+03_rb, &
        & 0.75155e+03_rb,0.89403e+03_rb,0.10365e+04_rb,0.11790e+04_rb /)
      kao(:, 2, 6,16) = (/ &
        & 0.83494e+03_rb,0.91972e+03_rb,0.83351e+02_rb,0.26459e+03_rb,0.33053e+03_rb, &
        & 0.75056e+03_rb,0.89298e+03_rb,0.10296e+04_rb,0.11778e+04_rb /)
      kao(:, 3, 6,16) = (/ &
        & 0.77495e+02_rb,0.67358e+03_rb,0.71470e+03_rb,0.28985e+03_rb,0.12695e+03_rb, &
        & 0.74093e+03_rb,0.89153e+03_rb,0.10338e+04_rb,0.11761e+04_rb /)
      kao(:, 4, 6,16) = (/ &
        & 0.12524e+04_rb,0.52300e+03_rb,0.17321e+03_rb,0.31958e+03_rb,0.15643e+03_rb, &
        & 0.74316e+03_rb,0.88930e+03_rb,0.10313e+04_rb,0.11733e+04_rb /)
      kao(:, 5, 6,16) = (/ &
        & 0.79880e+03_rb,0.54259e+03_rb,0.31909e+03_rb,0.47791e+03_rb,0.45938e+03_rb, &
        & 0.74468e+03_rb,0.88626e+03_rb,0.10278e+04_rb,0.11499e+04_rb /)
      kao(:, 1, 7,16) = (/ &
        & 0.10078e+04_rb,0.83081e+03_rb,0.68047e+03_rb,0.68284e+03_rb,0.57904e+03_rb, &
        & 0.75128e+03_rb,0.89403e+03_rb,0.10368e+04_rb,0.11795e+04_rb /)
      kao(:, 2, 7,16) = (/ &
        & 0.67323e+03_rb,0.10228e+04_rb,0.49758e+03_rb,0.33578e+03_rb,0.38746e+03_rb, &
        & 0.75047e+03_rb,0.88822e+03_rb,0.10359e+04_rb,0.11786e+04_rb /)
      kao(:, 3, 7,16) = (/ &
        & 0.60358e+03_rb,0.41876e+03_rb,0.74942e+03_rb,0.38757e+03_rb,0.61092e+03_rb, &
        & 0.74943e+03_rb,0.88213e+03_rb,0.10289e+04_rb,0.11707e+04_rb /)
      kao(:, 4, 7,16) = (/ &
        & 0.69109e+03_rb,0.11040e+04_rb,0.46705e+03_rb,0.74628e+03_rb,0.47483e+03_rb, &
        & 0.74797e+03_rb,0.88542e+03_rb,0.10328e+04_rb,0.11752e+04_rb /)
      kao(:, 5, 7,16) = (/ &
        & 0.68277e+03_rb,0.46564e+03_rb,0.19228e+03_rb,0.48854e+03_rb,0.28304e+03_rb, &
        & 0.74163e+03_rb,0.88289e+03_rb,0.10299e+04_rb,0.11719e+04_rb /)
      kao(:, 1, 8,16) = (/ &
        & 0.26685e+03_rb,0.60532e+03_rb,0.51009e+03_rb,0.40907e+03_rb,0.39126e+03_rb, &
        & 0.74682e+03_rb,0.89396e+03_rb,0.10369e+04_rb,0.11799e+04_rb /)
      kao(:, 2, 8,16) = (/ &
        & 0.94869e+03_rb,0.11471e+04_rb,0.44706e+03_rb,0.23151e+03_rb,0.93532e+02_rb, &
        & 0.75035e+03_rb,0.89331e+03_rb,0.10363e+04_rb,0.11727e+04_rb /)
      kao(:, 3, 8,16) = (/ &
        & 0.11588e+04_rb,0.74588e+03_rb,0.28311e+03_rb,0.27864e+03_rb,0.32853e+03_rb, &
        & 0.74950e+03_rb,0.88246e+03_rb,0.10352e+04_rb,0.11716e+04_rb /)
      kao(:, 4, 8,16) = (/ &
        & 0.11874e+04_rb,0.47456e+03_rb,0.83368e+03_rb,0.31591e+03_rb,0.30274e+03_rb, &
        & 0.74421e+03_rb,0.89109e+03_rb,0.10338e+04_rb,0.11765e+04_rb /)
      kao(:, 5, 8,16) = (/ &
        & 0.45710e+03_rb,0.66086e+03_rb,0.55863e+03_rb,0.58037e+03_rb,0.20569e+03_rb, &
        & 0.73836e+03_rb,0.88417e+03_rb,0.10316e+04_rb,0.11675e+04_rb /)
      kao(:, 1, 9,16) = (/ &
        & 0.55233e+03_rb,0.18269e+03_rb,0.18991e+03_rb,0.35964e+03_rb,0.38998e+03_rb, &
        & 0.75089e+03_rb,0.89415e+03_rb,0.10374e+04_rb,0.11807e+04_rb /)
      kao(:, 2, 9,16) = (/ &
        & 0.31819e+03_rb,0.14904e+03_rb,0.79703e+03_rb,0.32317e+03_rb,0.20679e+03_rb, &
        & 0.75021e+03_rb,0.89337e+03_rb,0.10308e+04_rb,0.11797e+04_rb /)
      kao(:, 3, 9,16) = (/ &
        & 0.16059e+03_rb,0.37409e+03_rb,0.62337e+03_rb,0.62951e+03_rb,0.47662e+03_rb, &
        & 0.74953e+03_rb,0.89262e+03_rb,0.10357e+04_rb,0.11788e+04_rb /)
      kao(:, 4, 9,16) = (/ &
        & 0.15191e+03_rb,0.63681e+03_rb,0.25601e+03_rb,0.65772e+03_rb,0.38994e+03_rb, &
        & 0.74029e+03_rb,0.89158e+03_rb,0.10346e+04_rb,0.11775e+04_rb /)
      kao(:, 5, 9,16) = (/ &
        & 0.10392e+04_rb,0.33629e+03_rb,0.90538e+03_rb,0.33435e+03_rb,0.46694e+03_rb, &
        & 0.74731e+03_rb,0.89008e+03_rb,0.10329e+04_rb,0.11756e+04_rb /)
      kao(:, 1,10,16) = (/ &
        & 0.43161e+03_rb,0.53194e+03_rb,0.82168e+03_rb,0.11050e+03_rb,0.32159e+02_rb, &
        & 0.75101e+03_rb,0.89450e+03_rb,0.10380e+04_rb,0.11815e+04_rb /)
      kao(:, 2,10,16) = (/ &
        & 0.23362e+03_rb,0.50697e+03_rb,0.75913e+03_rb,0.37965e+03_rb,0.15877e+03_rb, &
        & 0.75002e+03_rb,0.89333e+03_rb,0.10366e+04_rb,0.11734e+04_rb /)
      kao(:, 3,10,16) = (/ &
        & 0.66577e+03_rb,0.22676e+03_rb,0.63752e+03_rb,0.50488e+03_rb,0.52339e+03_rb, &
        & 0.74947e+03_rb,0.88778e+03_rb,0.10360e+04_rb,0.11793e+04_rb /)
      kao(:, 4,10,16) = (/ &
        & 0.12040e+04_rb,0.37222e+03_rb,0.71269e+03_rb,0.68368e+02_rb,0.31445e+03_rb, &
        & 0.74453e+03_rb,0.88691e+03_rb,0.10293e+04_rb,0.11782e+04_rb /)
      kao(:, 5,10,16) = (/ &
        & 0.85334e+03_rb,0.50951e+03_rb,0.43397e+03_rb,0.11861e+03_rb,0.15074e+03_rb, &
        & 0.74764e+03_rb,0.89065e+03_rb,0.10337e+04_rb,0.11767e+04_rb /)
      kao(:, 1,11,16) = (/ &
        & 0.45853e+03_rb,0.29024e+03_rb,0.38275e+03_rb,0.26130e+03_rb,0.12124e+03_rb, &
        & 0.75062e+03_rb,0.89424e+03_rb,0.10379e+04_rb,0.11815e+04_rb /)
      kao(:, 2,11,16) = (/ &
        & 0.66691e+03_rb,0.55656e+03_rb,0.39879e+03_rb,0.16526e+03_rb,0.24675e+03_rb, &
        & 0.74965e+03_rb,0.89309e+03_rb,0.10365e+04_rb,0.11800e+04_rb /)
      kao(:, 3,11,16) = (/ &
        & 0.10239e+04_rb,0.60930e+03_rb,0.22001e+03_rb,0.10914e+03_rb,0.22748e+03_rb, &
        & 0.74913e+03_rb,0.89251e+03_rb,0.10359e+04_rb,0.11793e+04_rb /)
      kao(:, 4,11,16) = (/ &
        & 0.80567e+03_rb,0.90344e+03_rb,0.77356e+03_rb,0.12489e+03_rb,0.55902e+03_rb, &
        & 0.74421e+03_rb,0.89165e+03_rb,0.10349e+04_rb,0.11782e+04_rb /)
      kao(:, 5,11,16) = (/ &
        & 0.80619e+03_rb,0.87963e+03_rb,0.47254e+03_rb,0.39261e+03_rb,0.59752e+03_rb, &
        & 0.74319e+03_rb,0.88056e+03_rb,0.10336e+04_rb,0.11767e+04_rb /)
      kao(:, 1,12,16) = (/ &
        & 0.54479e+03_rb,0.73809e+03_rb,0.74534e+03_rb,0.30870e+03_rb,0.60893e+03_rb, &
        & 0.75030e+03_rb,0.89402e+03_rb,0.10378e+04_rb,0.11815e+04_rb /)
      kao(:, 2,12,16) = (/ &
        & 0.10600e+04_rb,0.76559e+03_rb,0.34957e+03_rb,0.18602e+03_rb,0.60736e+03_rb, &
        & 0.74935e+03_rb,0.89289e+03_rb,0.10307e+04_rb,0.11734e+04_rb /)
      kao(:, 3,12,16) = (/ &
        & 0.10515e+04_rb,0.60079e+03_rb,0.43472e+03_rb,0.33220e+03_rb,0.60582e+03_rb, &
        & 0.74885e+03_rb,0.89233e+03_rb,0.10358e+04_rb,0.11793e+04_rb /)
      kao(:, 4,12,16) = (/ &
        & 0.32670e+03_rb,0.23075e+03_rb,0.13581e+03_rb,0.17106e+03_rb,0.60474e+03_rb, &
        & 0.74811e+03_rb,0.89148e+03_rb,0.10348e+04_rb,0.11717e+04_rb /)
      kao(:, 5,12,16) = (/ &
        & 0.20371e+03_rb,0.68798e+03_rb,0.91592e+02_rb,0.60316e+03_rb,0.60390e+03_rb, &
        & 0.74294e+03_rb,0.89029e+03_rb,0.10335e+04_rb,0.11767e+04_rb /)
      kao(:, 1,13,16) = (/ &
        & 0.85988e+03_rb,0.94717e+03_rb,0.84544e+03_rb,0.31415e+03_rb,0.60500e+03_rb, &
        & 0.75003e+03_rb,0.89385e+03_rb,0.10377e+04_rb,0.11815e+04_rb /)
      kao(:, 2,13,16) = (/ &
        & 0.10379e+04_rb,0.11189e+03_rb,0.20733e+03_rb,0.17410e+03_rb,0.60685e+03_rb, &
        & 0.74910e+03_rb,0.89272e+03_rb,0.10306e+04_rb,0.11800e+04_rb /)
      kao(:, 3,13,16) = (/ &
        & 0.70206e+03_rb,0.77801e+03_rb,0.11974e+03_rb,0.34591e+03_rb,0.60534e+03_rb, &
        & 0.74861e+03_rb,0.89217e+03_rb,0.10357e+04_rb,0.11793e+04_rb /)
      kao(:, 4,13,16) = (/ &
        & 0.69165e+03_rb,0.93369e+03_rb,0.16190e+03_rb,0.16396e+03_rb,0.60445e+03_rb, &
        & 0.74789e+03_rb,0.89133e+03_rb,0.10348e+04_rb,0.11782e+04_rb /)
      kao(:, 5,13,16) = (/ &
        & 0.20016e+03_rb,0.25832e+03_rb,0.57797e+03_rb,0.24270e+03_rb,0.60363e+03_rb, &
        & 0.74689e+03_rb,0.88521e+03_rb,0.10334e+04_rb,0.11767e+04_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:, 1,13, 1) = (/ &
        & 0.27369e+02_rb,0.29903e+03_rb,0.42924e+03_rb,0.31039e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,13, 1) = (/ &
        & 0.27158e+02_rb,0.29898e+03_rb,0.42745e+03_rb,0.30840e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,13, 1) = (/ &
        & 0.26976e+02_rb,0.29889e+03_rb,0.42590e+03_rb,0.30668e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,13, 1) = (/ &
        & 0.26818e+02_rb,0.29875e+03_rb,0.42455e+03_rb,0.30517e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,13, 1) = (/ &
        & 0.26679e+02_rb,0.29855e+03_rb,0.42335e+03_rb,0.30384e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,14, 1) = (/ &
        & 0.26840e+02_rb,0.29863e+03_rb,0.42474e+03_rb,0.30538e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,14, 1) = (/ &
        & 0.26669e+02_rb,0.29862e+03_rb,0.42327e+03_rb,0.30375e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,14, 1) = (/ &
        & 0.26522e+02_rb,0.29854e+03_rb,0.42199e+03_rb,0.30234e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,14, 1) = (/ &
        & 0.26394e+02_rb,0.29842e+03_rb,0.42088e+03_rb,0.30111e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,14, 1) = (/ &
        & 0.26282e+02_rb,0.29824e+03_rb,0.41990e+03_rb,0.30002e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,15, 1) = (/ &
        & 0.26408e+02_rb,0.29831e+03_rb,0.42100e+03_rb,0.30124e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,15, 1) = (/ &
        & 0.26271e+02_rb,0.29832e+03_rb,0.41980e+03_rb,0.29991e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,15, 1) = (/ &
        & 0.26152e+02_rb,0.29826e+03_rb,0.41876e+03_rb,0.29877e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,15, 1) = (/ &
        & 0.26049e+02_rb,0.29814e+03_rb,0.41785e+03_rb,0.29776e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,15, 1) = (/ &
        & 0.25958e+02_rb,0.29797e+03_rb,0.41705e+03_rb,0.29688e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,16, 1) = (/ &
        & 0.26057e+02_rb,0.29806e+03_rb,0.41793e+03_rb,0.29784e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,16, 1) = (/ &
        & 0.25946e+02_rb,0.29807e+03_rb,0.41695e+03_rb,0.29676e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,16, 1) = (/ &
        & 0.25851e+02_rb,0.29803e+03_rb,0.41610e+03_rb,0.29583e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,16, 1) = (/ &
        & 0.25767e+02_rb,0.29792e+03_rb,0.41536e+03_rb,0.29502e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,16, 1) = (/ &
        & 0.25694e+02_rb,0.29776e+03_rb,0.41470e+03_rb,0.29430e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,17, 1) = (/ &
        & 0.25772e+02_rb,0.29785e+03_rb,0.41540e+03_rb,0.29506e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,17, 1) = (/ &
        & 0.25683e+02_rb,0.29787e+03_rb,0.41460e+03_rb,0.29419e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,17, 1) = (/ &
        & 0.25606e+02_rb,0.29784e+03_rb,0.41391e+03_rb,0.29343e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,17, 1) = (/ &
        & 0.25538e+02_rb,0.29774e+03_rb,0.41331e+03_rb,0.29277e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,17, 1) = (/ &
        & 0.25479e+02_rb,0.29758e+03_rb,0.41278e+03_rb,0.29219e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,18, 1) = (/ &
        & 0.25541e+02_rb,0.29768e+03_rb,0.41334e+03_rb,0.29280e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,18, 1) = (/ &
        & 0.25469e+02_rb,0.29772e+03_rb,0.41269e+03_rb,0.29209e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,18, 1) = (/ &
        & 0.25407e+02_rb,0.29768e+03_rb,0.41213e+03_rb,0.29147e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,18, 1) = (/ &
        & 0.25353e+02_rb,0.29758e+03_rb,0.41163e+03_rb,0.29094e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,18, 1) = (/ &
        & 0.25305e+02_rb,0.29743e+03_rb,0.41120e+03_rb,0.29046e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,19, 1) = (/ &
        & 0.25354e+02_rb,0.29755e+03_rb,0.41165e+03_rb,0.29095e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,19, 1) = (/ &
        & 0.25296e+02_rb,0.29758e+03_rb,0.41112e+03_rb,0.29037e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,19, 1) = (/ &
        & 0.25245e+02_rb,0.29755e+03_rb,0.41067e+03_rb,0.28988e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,19, 1) = (/ &
        & 0.25201e+02_rb,0.29746e+03_rb,0.41027e+03_rb,0.28944e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,19, 1) = (/ &
        & 0.25163e+02_rb,0.29730e+03_rb,0.40991e+03_rb,0.28905e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,20, 1) = (/ &
        & 0.25201e+02_rb,0.29744e+03_rb,0.41026e+03_rb,0.28943e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,20, 1) = (/ &
        & 0.25154e+02_rb,0.29748e+03_rb,0.40984e+03_rb,0.28897e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,20, 1) = (/ &
        & 0.25113e+02_rb,0.29745e+03_rb,0.40947e+03_rb,0.28856e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,20, 1) = (/ &
        & 0.25078e+02_rb,0.29735e+03_rb,0.40914e+03_rb,0.28821e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,20, 1) = (/ &
        & 0.25047e+02_rb,0.29719e+03_rb,0.40886e+03_rb,0.28790e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,21, 1) = (/ &
        & 0.25076e+02_rb,0.29736e+03_rb,0.40913e+03_rb,0.28819e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,21, 1) = (/ &
        & 0.25039e+02_rb,0.29739e+03_rb,0.40879e+03_rb,0.28782e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,21, 1) = (/ &
        & 0.25006e+02_rb,0.29736e+03_rb,0.40849e+03_rb,0.28749e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,21, 1) = (/ &
        & 0.24978e+02_rb,0.29726e+03_rb,0.40823e+03_rb,0.28722e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,21, 1) = (/ &
        & 0.24952e+02_rb,0.29710e+03_rb,0.40800e+03_rb,0.28695e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,22, 1) = (/ &
        & 0.24973e+02_rb,0.29729e+03_rb,0.40819e+03_rb,0.28717e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,22, 1) = (/ &
        & 0.24944e+02_rb,0.29732e+03_rb,0.40792e+03_rb,0.28687e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,22, 1) = (/ &
        & 0.24917e+02_rb,0.29728e+03_rb,0.40768e+03_rb,0.28661e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,22, 1) = (/ &
        & 0.24895e+02_rb,0.29717e+03_rb,0.40747e+03_rb,0.28638e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,22, 1) = (/ &
        & 0.24874e+02_rb,0.29700e+03_rb,0.40728e+03_rb,0.28618e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,23, 1) = (/ &
        & 0.24890e+02_rb,0.29724e+03_rb,0.40743e+03_rb,0.28634e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,23, 1) = (/ &
        & 0.24866e+02_rb,0.29726e+03_rb,0.40721e+03_rb,0.28610e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,23, 1) = (/ &
        & 0.24846e+02_rb,0.29721e+03_rb,0.40702e+03_rb,0.28589e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,23, 1) = (/ &
        & 0.24827e+02_rb,0.29710e+03_rb,0.40685e+03_rb,0.28571e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,23, 1) = (/ &
        & 0.24811e+02_rb,0.29692e+03_rb,0.40671e+03_rb,0.28554e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,24, 1) = (/ &
        & 0.24823e+02_rb,0.29721e+03_rb,0.40681e+03_rb,0.28566e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,24, 1) = (/ &
        & 0.24804e+02_rb,0.29721e+03_rb,0.40664e+03_rb,0.28547e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,24, 1) = (/ &
        & 0.24787e+02_rb,0.29715e+03_rb,0.40649e+03_rb,0.28530e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,24, 1) = (/ &
        & 0.24773e+02_rb,0.29703e+03_rb,0.40635e+03_rb,0.28516e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,24, 1) = (/ &
        & 0.24760e+02_rb,0.29684e+03_rb,0.40623e+03_rb,0.28503e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,25, 1) = (/ &
        & 0.24769e+02_rb,0.29717e+03_rb,0.40631e+03_rb,0.28511e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,25, 1) = (/ &
        & 0.24753e+02_rb,0.29717e+03_rb,0.40618e+03_rb,0.28496e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,25, 1) = (/ &
        & 0.24740e+02_rb,0.29710e+03_rb,0.40606e+03_rb,0.28483e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,25, 1) = (/ &
        & 0.24729e+02_rb,0.29696e+03_rb,0.40595e+03_rb,0.28472e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,25, 1) = (/ &
        & 0.24718e+02_rb,0.29676e+03_rb,0.40585e+03_rb,0.28462e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,26, 1) = (/ &
        & 0.24724e+02_rb,0.29715e+03_rb,0.40591e+03_rb,0.28467e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,26, 1) = (/ &
        & 0.24712e+02_rb,0.29713e+03_rb,0.40580e+03_rb,0.28456e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,26, 1) = (/ &
        & 0.24702e+02_rb,0.29704e+03_rb,0.40570e+03_rb,0.28445e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,26, 1) = (/ &
        & 0.24693e+02_rb,0.29689e+03_rb,0.40562e+03_rb,0.28435e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,26, 1) = (/ &
        & 0.24684e+02_rb,0.29668e+03_rb,0.40554e+03_rb,0.28427e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,27, 1) = (/ &
        & 0.24689e+02_rb,0.29713e+03_rb,0.40558e+03_rb,0.28431e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,27, 1) = (/ &
        & 0.24679e+02_rb,0.29709e+03_rb,0.40550e+03_rb,0.28422e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,27, 1) = (/ &
        & 0.24671e+02_rb,0.29699e+03_rb,0.40542e+03_rb,0.28423e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,27, 1) = (/ &
        & 0.24664e+02_rb,0.29683e+03_rb,0.40535e+03_rb,0.28405e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,27, 1) = (/ &
        & 0.24657e+02_rb,0.29660e+03_rb,0.40529e+03_rb,0.28399e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,28, 1) = (/ &
        & 0.24660e+02_rb,0.29710e+03_rb,0.40532e+03_rb,0.28404e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,28, 1) = (/ &
        & 0.24653e+02_rb,0.29706e+03_rb,0.40525e+03_rb,0.28395e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,28, 1) = (/ &
        & 0.24646e+02_rb,0.29694e+03_rb,0.40519e+03_rb,0.28389e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,28, 1) = (/ &
        & 0.24640e+02_rb,0.29676e+03_rb,0.40514e+03_rb,0.28383e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,28, 1) = (/ &
        & 0.24635e+02_rb,0.29651e+03_rb,0.40509e+03_rb,0.28376e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,29, 1) = (/ &
        & 0.24637e+02_rb,0.29708e+03_rb,0.40511e+03_rb,0.28380e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,29, 1) = (/ &
        & 0.24631e+02_rb,0.29702e+03_rb,0.40505e+03_rb,0.28374e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,29, 1) = (/ &
        & 0.24626e+02_rb,0.29689e+03_rb,0.40500e+03_rb,0.28368e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,29, 1) = (/ &
        & 0.24621e+02_rb,0.29669e+03_rb,0.40496e+03_rb,0.28364e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,29, 1) = (/ &
        & 0.24617e+02_rb,0.29643e+03_rb,0.40492e+03_rb,0.28359e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,30, 1) = (/ &
        & 0.24619e+02_rb,0.29706e+03_rb,0.40494e+03_rb,0.28360e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,30, 1) = (/ &
        & 0.24614e+02_rb,0.29698e+03_rb,0.40489e+03_rb,0.28355e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,30, 1) = (/ &
        & 0.24610e+02_rb,0.29683e+03_rb,0.40485e+03_rb,0.28351e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,30, 1) = (/ &
        & 0.24606e+02_rb,0.29662e+03_rb,0.40482e+03_rb,0.28347e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,30, 1) = (/ &
        & 0.24603e+02_rb,0.29634e+03_rb,0.40479e+03_rb,0.28344e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,31, 1) = (/ &
        & 0.24604e+02_rb,0.29704e+03_rb,0.40480e+03_rb,0.28345e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,31, 1) = (/ &
        & 0.24600e+02_rb,0.29694e+03_rb,0.40477e+03_rb,0.28341e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,31, 1) = (/ &
        & 0.24597e+02_rb,0.29678e+03_rb,0.40473e+03_rb,0.28338e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,31, 1) = (/ &
        & 0.24594e+02_rb,0.29655e+03_rb,0.40471e+03_rb,0.28336e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,31, 1) = (/ &
        & 0.24591e+02_rb,0.29625e+03_rb,0.40468e+03_rb,0.28332e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,32, 1) = (/ &
        & 0.24592e+02_rb,0.29701e+03_rb,0.40469e+03_rb,0.28333e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,32, 1) = (/ &
        & 0.24589e+02_rb,0.29689e+03_rb,0.40466e+03_rb,0.28330e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,32, 1) = (/ &
        & 0.24586e+02_rb,0.29672e+03_rb,0.40464e+03_rb,0.28328e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,32, 1) = (/ &
        & 0.24584e+02_rb,0.29647e+03_rb,0.40461e+03_rb,0.28324e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,32, 1) = (/ &
        & 0.24582e+02_rb,0.29615e+03_rb,0.40459e+03_rb,0.28325e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,33, 1) = (/ &
        & 0.24582e+02_rb,0.29698e+03_rb,0.40460e+03_rb,0.28323e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,33, 1) = (/ &
        & 0.24580e+02_rb,0.29685e+03_rb,0.40458e+03_rb,0.28321e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,33, 1) = (/ &
        & 0.24578e+02_rb,0.29665e+03_rb,0.40456e+03_rb,0.28318e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,33, 1) = (/ &
        & 0.24576e+02_rb,0.29639e+03_rb,0.40454e+03_rb,0.28317e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,33, 1) = (/ &
        & 0.24574e+02_rb,0.29605e+03_rb,0.40452e+03_rb,0.28315e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,34, 1) = (/ &
        & 0.24574e+02_rb,0.29695e+03_rb,0.40453e+03_rb,0.28315e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,34, 1) = (/ &
        & 0.24572e+02_rb,0.29680e+03_rb,0.40451e+03_rb,0.28313e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,34, 1) = (/ &
        & 0.24571e+02_rb,0.29659e+03_rb,0.40449e+03_rb,0.28312e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,34, 1) = (/ &
        & 0.24569e+02_rb,0.29631e+03_rb,0.40448e+03_rb,0.28310e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,34, 1) = (/ &
        & 0.24568e+02_rb,0.29596e+03_rb,0.40447e+03_rb,0.28308e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,35, 1) = (/ &
        & 0.24568e+02_rb,0.29693e+03_rb,0.40447e+03_rb,0.28309e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,35, 1) = (/ &
        & 0.24567e+02_rb,0.29678e+03_rb,0.40446e+03_rb,0.28308e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,35, 1) = (/ &
        & 0.24565e+02_rb,0.29656e+03_rb,0.40444e+03_rb,0.28306e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,35, 1) = (/ &
        & 0.24564e+02_rb,0.29627e+03_rb,0.40443e+03_rb,0.28305e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,35, 1) = (/ &
        & 0.24563e+02_rb,0.29590e+03_rb,0.40442e+03_rb,0.28304e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,36, 1) = (/ &
        & 0.24563e+02_rb,0.29692e+03_rb,0.40443e+03_rb,0.28305e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,36, 1) = (/ &
        & 0.24562e+02_rb,0.29677e+03_rb,0.40442e+03_rb,0.28304e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,36, 1) = (/ &
        & 0.24561e+02_rb,0.29655e+03_rb,0.40440e+03_rb,0.28301e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,36, 1) = (/ &
        & 0.24560e+02_rb,0.29626e+03_rb,0.40440e+03_rb,0.28302e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,36, 1) = (/ &
        & 0.24559e+02_rb,0.29589e+03_rb,0.40439e+03_rb,0.28301e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,37, 1) = (/ &
        & 0.24560e+02_rb,0.29693e+03_rb,0.40439e+03_rb,0.28301e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,37, 1) = (/ &
        & 0.24559e+02_rb,0.29679e+03_rb,0.40438e+03_rb,0.28299e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,37, 1) = (/ &
        & 0.24558e+02_rb,0.29658e+03_rb,0.40437e+03_rb,0.28299e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,37, 1) = (/ &
        & 0.24557e+02_rb,0.29630e+03_rb,0.40437e+03_rb,0.28297e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,37, 1) = (/ &
        & 0.24556e+02_rb,0.29594e+03_rb,0.40436e+03_rb,0.28297e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,38, 1) = (/ &
        & 0.24557e+02_rb,0.29695e+03_rb,0.40436e+03_rb,0.28297e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,38, 1) = (/ &
        & 0.24556e+02_rb,0.29681e+03_rb,0.40436e+03_rb,0.28296e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,38, 1) = (/ &
        & 0.24555e+02_rb,0.29661e+03_rb,0.40435e+03_rb,0.28297e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,38, 1) = (/ &
        & 0.24554e+02_rb,0.29634e+03_rb,0.40434e+03_rb,0.28296e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,38, 1) = (/ &
        & 0.24554e+02_rb,0.29599e+03_rb,0.40434e+03_rb,0.28294e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,39, 1) = (/ &
        & 0.24554e+02_rb,0.29696e+03_rb,0.40434e+03_rb,0.28295e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,39, 1) = (/ &
        & 0.24553e+02_rb,0.29683e+03_rb,0.40433e+03_rb,0.28294e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,39, 1) = (/ &
        & 0.24553e+02_rb,0.29664e+03_rb,0.40433e+03_rb,0.28293e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,39, 1) = (/ &
        & 0.24552e+02_rb,0.29638e+03_rb,0.40432e+03_rb,0.28292e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,39, 1) = (/ &
        & 0.24552e+02_rb,0.29604e+03_rb,0.40432e+03_rb,0.28292e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,40, 1) = (/ &
        & 0.24552e+02_rb,0.29698e+03_rb,0.40432e+03_rb,0.28292e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,40, 1) = (/ &
        & 0.24551e+02_rb,0.29687e+03_rb,0.40432e+03_rb,0.28293e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,40, 1) = (/ &
        & 0.24551e+02_rb,0.29669e+03_rb,0.40431e+03_rb,0.28292e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,40, 1) = (/ &
        & 0.24550e+02_rb,0.29645e+03_rb,0.40431e+03_rb,0.28291e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,40, 1) = (/ &
        & 0.24550e+02_rb,0.29613e+03_rb,0.40430e+03_rb,0.28291e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,41, 1) = (/ &
        & 0.24550e+02_rb,0.29699e+03_rb,0.40431e+03_rb,0.28290e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,41, 1) = (/ &
        & 0.24550e+02_rb,0.29690e+03_rb,0.40430e+03_rb,0.28291e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,41, 1) = (/ &
        & 0.24549e+02_rb,0.29674e+03_rb,0.40430e+03_rb,0.28289e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,41, 1) = (/ &
        & 0.24549e+02_rb,0.29651e+03_rb,0.40429e+03_rb,0.28290e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,41, 1) = (/ &
        & 0.24549e+02_rb,0.29622e+03_rb,0.40429e+03_rb,0.28289e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,42, 1) = (/ &
        & 0.24549e+02_rb,0.29700e+03_rb,0.40429e+03_rb,0.28289e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,42, 1) = (/ &
        & 0.24549e+02_rb,0.29693e+03_rb,0.40429e+03_rb,0.28289e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,42, 1) = (/ &
        & 0.24548e+02_rb,0.29679e+03_rb,0.40429e+03_rb,0.28289e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,42, 1) = (/ &
        & 0.24548e+02_rb,0.29658e+03_rb,0.40428e+03_rb,0.28289e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,42, 1) = (/ &
        & 0.24547e+02_rb,0.29630e+03_rb,0.40428e+03_rb,0.28288e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,43, 1) = (/ &
        & 0.24548e+02_rb,0.29701e+03_rb,0.40428e+03_rb,0.28288e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,43, 1) = (/ &
        & 0.24547e+02_rb,0.29696e+03_rb,0.40428e+03_rb,0.28288e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,43, 1) = (/ &
        & 0.24547e+02_rb,0.29684e+03_rb,0.40428e+03_rb,0.28288e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,43, 1) = (/ &
        & 0.24547e+02_rb,0.29665e+03_rb,0.40427e+03_rb,0.28288e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,43, 1) = (/ &
        & 0.24547e+02_rb,0.29639e+03_rb,0.40427e+03_rb,0.28288e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,44, 1) = (/ &
        & 0.24547e+02_rb,0.29702e+03_rb,0.40427e+03_rb,0.28287e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,44, 1) = (/ &
        & 0.24547e+02_rb,0.29698e+03_rb,0.40427e+03_rb,0.28287e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,44, 1) = (/ &
        & 0.24546e+02_rb,0.29688e+03_rb,0.40427e+03_rb,0.28287e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,44, 1) = (/ &
        & 0.24546e+02_rb,0.29672e+03_rb,0.40427e+03_rb,0.28287e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,44, 1) = (/ &
        & 0.24546e+02_rb,0.29649e+03_rb,0.40426e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,45, 1) = (/ &
        & 0.24546e+02_rb,0.29701e+03_rb,0.40427e+03_rb,0.28287e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,45, 1) = (/ &
        & 0.24546e+02_rb,0.29700e+03_rb,0.40426e+03_rb,0.28287e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,45, 1) = (/ &
        & 0.24546e+02_rb,0.29692e+03_rb,0.40426e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,45, 1) = (/ &
        & 0.24545e+02_rb,0.29678e+03_rb,0.40426e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,45, 1) = (/ &
        & 0.24545e+02_rb,0.29657e+03_rb,0.40426e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,46, 1) = (/ &
        & 0.24545e+02_rb,0.29699e+03_rb,0.40426e+03_rb,0.28287e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,46, 1) = (/ &
        & 0.24545e+02_rb,0.29701e+03_rb,0.40426e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,46, 1) = (/ &
        & 0.24545e+02_rb,0.29696e+03_rb,0.40426e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,46, 1) = (/ &
        & 0.24545e+02_rb,0.29684e+03_rb,0.40425e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,46, 1) = (/ &
        & 0.24545e+02_rb,0.29665e+03_rb,0.40425e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,47, 1) = (/ &
        & 0.24545e+02_rb,0.29696e+03_rb,0.40425e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,47, 1) = (/ &
        & 0.24545e+02_rb,0.29701e+03_rb,0.40425e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,47, 1) = (/ &
        & 0.24544e+02_rb,0.29699e+03_rb,0.40425e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,47, 1) = (/ &
        & 0.24544e+02_rb,0.29690e+03_rb,0.40425e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,47, 1) = (/ &
        & 0.24544e+02_rb,0.29674e+03_rb,0.40425e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,48, 1) = (/ &
        & 0.24544e+02_rb,0.29691e+03_rb,0.40425e+03_rb,0.28286e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,48, 1) = (/ &
        & 0.24544e+02_rb,0.29700e+03_rb,0.40425e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,48, 1) = (/ &
        & 0.24544e+02_rb,0.29701e+03_rb,0.40425e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,48, 1) = (/ &
        & 0.24544e+02_rb,0.29694e+03_rb,0.40425e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,48, 1) = (/ &
        & 0.24544e+02_rb,0.29681e+03_rb,0.40425e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,49, 1) = (/ &
        & 0.24544e+02_rb,0.29685e+03_rb,0.40425e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,49, 1) = (/ &
        & 0.24544e+02_rb,0.29698e+03_rb,0.40425e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,49, 1) = (/ &
        & 0.24544e+02_rb,0.29701e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,49, 1) = (/ &
        & 0.24544e+02_rb,0.29697e+03_rb,0.40424e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,49, 1) = (/ &
        & 0.24544e+02_rb,0.29687e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,50, 1) = (/ &
        & 0.24544e+02_rb,0.29678e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,50, 1) = (/ &
        & 0.24544e+02_rb,0.29694e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,50, 1) = (/ &
        & 0.24543e+02_rb,0.29701e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,50, 1) = (/ &
        & 0.24543e+02_rb,0.29700e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,50, 1) = (/ &
        & 0.24543e+02_rb,0.29692e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,51, 1) = (/ &
        & 0.24543e+02_rb,0.29670e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,51, 1) = (/ &
        & 0.24543e+02_rb,0.29689e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,51, 1) = (/ &
        & 0.24543e+02_rb,0.29699e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,51, 1) = (/ &
        & 0.24543e+02_rb,0.29701e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,51, 1) = (/ &
        & 0.24543e+02_rb,0.29695e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,52, 1) = (/ &
        & 0.24543e+02_rb,0.29660e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,52, 1) = (/ &
        & 0.24543e+02_rb,0.29684e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,52, 1) = (/ &
        & 0.24543e+02_rb,0.29697e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,52, 1) = (/ &
        & 0.24543e+02_rb,0.29701e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,52, 1) = (/ &
        & 0.24543e+02_rb,0.29698e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,53, 1) = (/ &
        & 0.24543e+02_rb,0.29648e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,53, 1) = (/ &
        & 0.24543e+02_rb,0.29676e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,53, 1) = (/ &
        & 0.24543e+02_rb,0.29693e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,53, 1) = (/ &
        & 0.24543e+02_rb,0.29701e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,53, 1) = (/ &
        & 0.24543e+02_rb,0.29700e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,54, 1) = (/ &
        & 0.24543e+02_rb,0.29635e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,54, 1) = (/ &
        & 0.24543e+02_rb,0.29668e+03_rb,0.40424e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,54, 1) = (/ &
        & 0.24543e+02_rb,0.29688e+03_rb,0.40424e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,54, 1) = (/ &
        & 0.24543e+02_rb,0.29699e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,54, 1) = (/ &
        & 0.24543e+02_rb,0.29701e+03_rb,0.40423e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,55, 1) = (/ &
        & 0.24543e+02_rb,0.29622e+03_rb,0.40424e+03_rb,0.28285e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,55, 1) = (/ &
        & 0.24543e+02_rb,0.29659e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,55, 1) = (/ &
        & 0.24543e+02_rb,0.29683e+03_rb,0.40423e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,55, 1) = (/ &
        & 0.24543e+02_rb,0.29696e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,55, 1) = (/ &
        & 0.24543e+02_rb,0.29701e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,56, 1) = (/ &
        & 0.24543e+02_rb,0.29607e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,56, 1) = (/ &
        & 0.24543e+02_rb,0.29649e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,56, 1) = (/ &
        & 0.24543e+02_rb,0.29676e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,56, 1) = (/ &
        & 0.24542e+02_rb,0.29693e+03_rb,0.40423e+03_rb,0.28282e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,56, 1) = (/ &
        & 0.24542e+02_rb,0.29700e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,57, 1) = (/ &
        & 0.24543e+02_rb,0.29591e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,57, 1) = (/ &
        & 0.24542e+02_rb,0.29637e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,57, 1) = (/ &
        & 0.24542e+02_rb,0.29669e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,57, 1) = (/ &
        & 0.24542e+02_rb,0.29689e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,57, 1) = (/ &
        & 0.24542e+02_rb,0.29699e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,58, 1) = (/ &
        & 0.24542e+02_rb,0.29574e+03_rb,0.40423e+03_rb,0.28282e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,58, 1) = (/ &
        & 0.24542e+02_rb,0.29624e+03_rb,0.40423e+03_rb,0.28282e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,58, 1) = (/ &
        & 0.24542e+02_rb,0.29660e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,58, 1) = (/ &
        & 0.24542e+02_rb,0.29684e+03_rb,0.40423e+03_rb,0.28284e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,58, 1) = (/ &
        & 0.24542e+02_rb,0.29697e+03_rb,0.40423e+03_rb,0.28282e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,59, 1) = (/ &
        & 0.24542e+02_rb,0.29566e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 2,59, 1) = (/ &
        & 0.24542e+02_rb,0.29619e+03_rb,0.40423e+03_rb,0.28282e+03_rb,0.35000e+02_rb /)
      kbo(:, 3,59, 1) = (/ &
        & 0.24542e+02_rb,0.29657e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 4,59, 1) = (/ &
        & 0.24542e+02_rb,0.29682e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 5,59, 1) = (/ &
        & 0.24542e+02_rb,0.29696e+03_rb,0.40423e+03_rb,0.28283e+03_rb,0.35000e+02_rb /)
      kbo(:, 1,13, 2) = (/ &
        & 0.10177e+03_rb,0.32761e+03_rb,0.45575e+03_rb,0.31335e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,13, 2) = (/ &
        & 0.10099e+03_rb,0.32713e+03_rb,0.45383e+03_rb,0.31125e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,13, 2) = (/ &
        & 0.10031e+03_rb,0.32663e+03_rb,0.45216e+03_rb,0.30943e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,13, 2) = (/ &
        & 0.99721e+02_rb,0.32612e+03_rb,0.45071e+03_rb,0.30785e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,13, 2) = (/ &
        & 0.99204e+02_rb,0.32558e+03_rb,0.44942e+03_rb,0.30646e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,14, 2) = (/ &
        & 0.99803e+02_rb,0.32613e+03_rb,0.45091e+03_rb,0.30807e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,14, 2) = (/ &
        & 0.99169e+02_rb,0.32574e+03_rb,0.44934e+03_rb,0.30636e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,14, 2) = (/ &
        & 0.98622e+02_rb,0.32534e+03_rb,0.44797e+03_rb,0.30489e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,14, 2) = (/ &
        & 0.98146e+02_rb,0.32491e+03_rb,0.44677e+03_rb,0.30362e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,14, 2) = (/ &
        & 0.97728e+02_rb,0.32444e+03_rb,0.44571e+03_rb,0.30248e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,15, 2) = (/ &
        & 0.98198e+02_rb,0.32493e+03_rb,0.44690e+03_rb,0.30375e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,15, 2) = (/ &
        & 0.97687e+02_rb,0.32462e+03_rb,0.44560e+03_rb,0.30237e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,15, 2) = (/ &
        & 0.97245e+02_rb,0.32429e+03_rb,0.44448e+03_rb,0.30119e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,15, 2) = (/ &
        & 0.96861e+02_rb,0.32392e+03_rb,0.44350e+03_rb,0.30014e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,15, 2) = (/ &
        & 0.96523e+02_rb,0.32350e+03_rb,0.44263e+03_rb,0.29923e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,16, 2) = (/ &
        & 0.96892e+02_rb,0.32395e+03_rb,0.44358e+03_rb,0.30023e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,16, 2) = (/ &
        & 0.96480e+02_rb,0.32371e+03_rb,0.44252e+03_rb,0.29912e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,16, 2) = (/ &
        & 0.96125e+02_rb,0.32343e+03_rb,0.44161e+03_rb,0.29817e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,16, 2) = (/ &
        & 0.95814e+02_rb,0.32311e+03_rb,0.44081e+03_rb,0.29733e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,16, 2) = (/ &
        & 0.95541e+02_rb,0.32273e+03_rb,0.44010e+03_rb,0.29661e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,17, 2) = (/ &
        & 0.95833e+02_rb,0.32316e+03_rb,0.44086e+03_rb,0.29737e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,17, 2) = (/ &
        & 0.95501e+02_rb,0.32297e+03_rb,0.44000e+03_rb,0.29648e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,17, 2) = (/ &
        & 0.95214e+02_rb,0.32274e+03_rb,0.43925e+03_rb,0.29569e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,17, 2) = (/ &
        & 0.94964e+02_rb,0.32245e+03_rb,0.43860e+03_rb,0.29502e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,17, 2) = (/ &
        & 0.94743e+02_rb,0.32210e+03_rb,0.43803e+03_rb,0.29442e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,18, 2) = (/ &
        & 0.94974e+02_rb,0.32251e+03_rb,0.43863e+03_rb,0.29504e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,18, 2) = (/ &
        & 0.94707e+02_rb,0.32236e+03_rb,0.43793e+03_rb,0.29432e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,18, 2) = (/ &
        & 0.94475e+02_rb,0.32217e+03_rb,0.43733e+03_rb,0.29369e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,18, 2) = (/ &
        & 0.94272e+02_rb,0.32191e+03_rb,0.43680e+03_rb,0.29315e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,18, 2) = (/ &
        & 0.94094e+02_rb,0.32158e+03_rb,0.43633e+03_rb,0.29267e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,19, 2) = (/ &
        & 0.94277e+02_rb,0.32199e+03_rb,0.43681e+03_rb,0.29316e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,19, 2) = (/ &
        & 0.94061e+02_rb,0.32187e+03_rb,0.43624e+03_rb,0.29258e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,19, 2) = (/ &
        & 0.93874e+02_rb,0.32170e+03_rb,0.43575e+03_rb,0.29207e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,19, 2) = (/ &
        & 0.93711e+02_rb,0.32146e+03_rb,0.43532e+03_rb,0.29164e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,19, 2) = (/ &
        & 0.93566e+02_rb,0.32115e+03_rb,0.43494e+03_rb,0.29125e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,20, 2) = (/ &
        & 0.93708e+02_rb,0.32157e+03_rb,0.43531e+03_rb,0.29163e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,20, 2) = (/ &
        & 0.93534e+02_rb,0.32147e+03_rb,0.43485e+03_rb,0.29117e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,20, 2) = (/ &
        & 0.93383e+02_rb,0.32132e+03_rb,0.43445e+03_rb,0.29076e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,20, 2) = (/ &
        & 0.93251e+02_rb,0.32109e+03_rb,0.43411e+03_rb,0.29041e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,20, 2) = (/ &
        & 0.93135e+02_rb,0.32079e+03_rb,0.43380e+03_rb,0.29010e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,21, 2) = (/ &
        & 0.93246e+02_rb,0.32122e+03_rb,0.43409e+03_rb,0.29040e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,21, 2) = (/ &
        & 0.93106e+02_rb,0.32114e+03_rb,0.43372e+03_rb,0.29003e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,21, 2) = (/ &
        & 0.92985e+02_rb,0.32100e+03_rb,0.43340e+03_rb,0.28970e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,21, 2) = (/ &
        & 0.92878e+02_rb,0.32079e+03_rb,0.43311e+03_rb,0.28941e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,21, 2) = (/ &
        & 0.92784e+02_rb,0.32049e+03_rb,0.43286e+03_rb,0.28917e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,22, 2) = (/ &
        & 0.92863e+02_rb,0.32094e+03_rb,0.43307e+03_rb,0.28938e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,22, 2) = (/ &
        & 0.92751e+02_rb,0.32087e+03_rb,0.43278e+03_rb,0.28908e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,22, 2) = (/ &
        & 0.92655e+02_rb,0.32073e+03_rb,0.43252e+03_rb,0.28882e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,22, 2) = (/ &
        & 0.92569e+02_rb,0.32051e+03_rb,0.43229e+03_rb,0.28860e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,22, 2) = (/ &
        & 0.92494e+02_rb,0.32022e+03_rb,0.43209e+03_rb,0.28840e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,23, 2) = (/ &
        & 0.92553e+02_rb,0.32072e+03_rb,0.43225e+03_rb,0.28855e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,23, 2) = (/ &
        & 0.92465e+02_rb,0.32064e+03_rb,0.43201e+03_rb,0.28832e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,23, 2) = (/ &
        & 0.92387e+02_rb,0.32050e+03_rb,0.43180e+03_rb,0.28811e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,23, 2) = (/ &
        & 0.92319e+02_rb,0.32029e+03_rb,0.43162e+03_rb,0.28793e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,23, 2) = (/ &
        & 0.92259e+02_rb,0.31998e+03_rb,0.43146e+03_rb,0.28778e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,24, 2) = (/ &
        & 0.92303e+02_rb,0.32053e+03_rb,0.43158e+03_rb,0.28789e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,24, 2) = (/ &
        & 0.92233e+02_rb,0.32046e+03_rb,0.43139e+03_rb,0.28770e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,24, 2) = (/ &
        & 0.92171e+02_rb,0.32031e+03_rb,0.43122e+03_rb,0.28754e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,24, 2) = (/ &
        & 0.92117e+02_rb,0.32009e+03_rb,0.43108e+03_rb,0.28740e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,24, 2) = (/ &
        & 0.92068e+02_rb,0.31978e+03_rb,0.43095e+03_rb,0.28727e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,25, 2) = (/ &
        & 0.92101e+02_rb,0.32038e+03_rb,0.43103e+03_rb,0.28736e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,25, 2) = (/ &
        & 0.92045e+02_rb,0.32030e+03_rb,0.43088e+03_rb,0.28721e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,25, 2) = (/ &
        & 0.91996e+02_rb,0.32015e+03_rb,0.43075e+03_rb,0.28708e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,25, 2) = (/ &
        & 0.91952e+02_rb,0.31991e+03_rb,0.43064e+03_rb,0.28696e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,25, 2) = (/ &
        & 0.91914e+02_rb,0.31959e+03_rb,0.43053e+03_rb,0.28684e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,26, 2) = (/ &
        & 0.91937e+02_rb,0.32025e+03_rb,0.43059e+03_rb,0.28693e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,26, 2) = (/ &
        & 0.91892e+02_rb,0.32016e+03_rb,0.43047e+03_rb,0.28680e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,26, 2) = (/ &
        & 0.91853e+02_rb,0.32000e+03_rb,0.43037e+03_rb,0.28670e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,26, 2) = (/ &
        & 0.91819e+02_rb,0.31975e+03_rb,0.43028e+03_rb,0.28661e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,26, 2) = (/ &
        & 0.91788e+02_rb,0.31942e+03_rb,0.43019e+03_rb,0.28653e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,27, 2) = (/ &
        & 0.91805e+02_rb,0.32015e+03_rb,0.43024e+03_rb,0.28658e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,27, 2) = (/ &
        & 0.91770e+02_rb,0.32005e+03_rb,0.43014e+03_rb,0.28648e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,27, 2) = (/ &
        & 0.91738e+02_rb,0.31986e+03_rb,0.43006e+03_rb,0.28630e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,27, 2) = (/ &
        & 0.91711e+02_rb,0.31960e+03_rb,0.42998e+03_rb,0.28633e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,27, 2) = (/ &
        & 0.91687e+02_rb,0.31925e+03_rb,0.42992e+03_rb,0.28626e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,28, 2) = (/ &
        & 0.91698e+02_rb,0.32005e+03_rb,0.42995e+03_rb,0.28628e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,28, 2) = (/ &
        & 0.91670e+02_rb,0.31994e+03_rb,0.42987e+03_rb,0.28622e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,28, 2) = (/ &
        & 0.91646e+02_rb,0.31974e+03_rb,0.42981e+03_rb,0.28615e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,28, 2) = (/ &
        & 0.91624e+02_rb,0.31946e+03_rb,0.42975e+03_rb,0.28609e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,28, 2) = (/ &
        & 0.91604e+02_rb,0.31910e+03_rb,0.42970e+03_rb,0.28605e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,29, 2) = (/ &
        & 0.91613e+02_rb,0.31997e+03_rb,0.42972e+03_rb,0.28605e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,29, 2) = (/ &
        & 0.91591e+02_rb,0.31984e+03_rb,0.42966e+03_rb,0.28600e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,29, 2) = (/ &
        & 0.91571e+02_rb,0.31963e+03_rb,0.42960e+03_rb,0.28595e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,29, 2) = (/ &
        & 0.91554e+02_rb,0.31933e+03_rb,0.42956e+03_rb,0.28590e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,29, 2) = (/ &
        & 0.91538e+02_rb,0.31895e+03_rb,0.42952e+03_rb,0.28586e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,30, 2) = (/ &
        & 0.91544e+02_rb,0.31990e+03_rb,0.42953e+03_rb,0.28589e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,30, 2) = (/ &
        & 0.91526e+02_rb,0.31975e+03_rb,0.42948e+03_rb,0.28584e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,30, 2) = (/ &
        & 0.91511e+02_rb,0.31952e+03_rb,0.42944e+03_rb,0.28580e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,30, 2) = (/ &
        & 0.91497e+02_rb,0.31920e+03_rb,0.42940e+03_rb,0.28577e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,30, 2) = (/ &
        & 0.91484e+02_rb,0.31880e+03_rb,0.42937e+03_rb,0.28573e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,31, 2) = (/ &
        & 0.91488e+02_rb,0.31983e+03_rb,0.42938e+03_rb,0.28574e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,31, 2) = (/ &
        & 0.91474e+02_rb,0.31966e+03_rb,0.42934e+03_rb,0.28571e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,31, 2) = (/ &
        & 0.91462e+02_rb,0.31941e+03_rb,0.42931e+03_rb,0.28567e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,31, 2) = (/ &
        & 0.91451e+02_rb,0.31908e+03_rb,0.42928e+03_rb,0.28564e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,31, 2) = (/ &
        & 0.91441e+02_rb,0.31866e+03_rb,0.42925e+03_rb,0.28562e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,32, 2) = (/ &
        & 0.91443e+02_rb,0.31976e+03_rb,0.42926e+03_rb,0.28562e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,32, 2) = (/ &
        & 0.91432e+02_rb,0.31958e+03_rb,0.42923e+03_rb,0.28559e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,32, 2) = (/ &
        & 0.91423e+02_rb,0.31930e+03_rb,0.42920e+03_rb,0.28557e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,32, 2) = (/ &
        & 0.91414e+02_rb,0.31895e+03_rb,0.42918e+03_rb,0.28555e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,32, 2) = (/ &
        & 0.91406e+02_rb,0.31851e+03_rb,0.42916e+03_rb,0.28551e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,33, 2) = (/ &
        & 0.91407e+02_rb,0.31970e+03_rb,0.42916e+03_rb,0.28553e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,33, 2) = (/ &
        & 0.91399e+02_rb,0.31949e+03_rb,0.42914e+03_rb,0.28551e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,33, 2) = (/ &
        & 0.91391e+02_rb,0.31920e+03_rb,0.42912e+03_rb,0.28549e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,33, 2) = (/ &
        & 0.91384e+02_rb,0.31882e+03_rb,0.42910e+03_rb,0.28547e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,33, 2) = (/ &
        & 0.91378e+02_rb,0.31837e+03_rb,0.42908e+03_rb,0.28546e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,34, 2) = (/ &
        & 0.91379e+02_rb,0.31964e+03_rb,0.42908e+03_rb,0.28546e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,34, 2) = (/ &
        & 0.91372e+02_rb,0.31941e+03_rb,0.42906e+03_rb,0.28544e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,34, 2) = (/ &
        & 0.91365e+02_rb,0.31910e+03_rb,0.42905e+03_rb,0.28542e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,34, 2) = (/ &
        & 0.91360e+02_rb,0.31871e+03_rb,0.42903e+03_rb,0.28541e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,34, 2) = (/ &
        & 0.91355e+02_rb,0.31824e+03_rb,0.42902e+03_rb,0.28540e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,35, 2) = (/ &
        & 0.91356e+02_rb,0.31960e+03_rb,0.42902e+03_rb,0.28539e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,35, 2) = (/ &
        & 0.91350e+02_rb,0.31936e+03_rb,0.42901e+03_rb,0.28538e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,35, 2) = (/ &
        & 0.91345e+02_rb,0.31904e+03_rb,0.42899e+03_rb,0.28537e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,35, 2) = (/ &
        & 0.91341e+02_rb,0.31863e+03_rb,0.42898e+03_rb,0.28535e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,35, 2) = (/ &
        & 0.91337e+02_rb,0.31816e+03_rb,0.42897e+03_rb,0.28535e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,36, 2) = (/ &
        & 0.91338e+02_rb,0.31958e+03_rb,0.42897e+03_rb,0.28535e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,36, 2) = (/ &
        & 0.91334e+02_rb,0.31934e+03_rb,0.42896e+03_rb,0.28533e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,36, 2) = (/ &
        & 0.91330e+02_rb,0.31902e+03_rb,0.42895e+03_rb,0.28533e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,36, 2) = (/ &
        & 0.91326e+02_rb,0.31862e+03_rb,0.42894e+03_rb,0.28530e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,36, 2) = (/ &
        & 0.91323e+02_rb,0.31814e+03_rb,0.42893e+03_rb,0.28530e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,37, 2) = (/ &
        & 0.91324e+02_rb,0.31959e+03_rb,0.42894e+03_rb,0.28531e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,37, 2) = (/ &
        & 0.91320e+02_rb,0.31937e+03_rb,0.42893e+03_rb,0.28530e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,37, 2) = (/ &
        & 0.91317e+02_rb,0.31906e+03_rb,0.42892e+03_rb,0.28529e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,37, 2) = (/ &
        & 0.91314e+02_rb,0.31866e+03_rb,0.42891e+03_rb,0.28529e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,37, 2) = (/ &
        & 0.91311e+02_rb,0.31819e+03_rb,0.42890e+03_rb,0.28527e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,38, 2) = (/ &
        & 0.91313e+02_rb,0.31961e+03_rb,0.42890e+03_rb,0.28528e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,38, 2) = (/ &
        & 0.91310e+02_rb,0.31939e+03_rb,0.42890e+03_rb,0.28528e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,38, 2) = (/ &
        & 0.91307e+02_rb,0.31910e+03_rb,0.42889e+03_rb,0.28526e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,38, 2) = (/ &
        & 0.91304e+02_rb,0.31871e+03_rb,0.42888e+03_rb,0.28525e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,38, 2) = (/ &
        & 0.91302e+02_rb,0.31826e+03_rb,0.42888e+03_rb,0.28525e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,39, 2) = (/ &
        & 0.91303e+02_rb,0.31962e+03_rb,0.42888e+03_rb,0.28525e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,39, 2) = (/ &
        & 0.91301e+02_rb,0.31942e+03_rb,0.42887e+03_rb,0.28525e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,39, 2) = (/ &
        & 0.91298e+02_rb,0.31914e+03_rb,0.42886e+03_rb,0.28525e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,39, 2) = (/ &
        & 0.91296e+02_rb,0.31877e+03_rb,0.42886e+03_rb,0.28524e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,39, 2) = (/ &
        & 0.91294e+02_rb,0.31832e+03_rb,0.42885e+03_rb,0.28523e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,40, 2) = (/ &
        & 0.91296e+02_rb,0.31965e+03_rb,0.42886e+03_rb,0.28524e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,40, 2) = (/ &
        & 0.91293e+02_rb,0.31947e+03_rb,0.42885e+03_rb,0.28522e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,40, 2) = (/ &
        & 0.91292e+02_rb,0.31921e+03_rb,0.42885e+03_rb,0.28522e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,40, 2) = (/ &
        & 0.91290e+02_rb,0.31886e+03_rb,0.42884e+03_rb,0.28522e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,40, 2) = (/ &
        & 0.91288e+02_rb,0.31843e+03_rb,0.42884e+03_rb,0.28521e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,41, 2) = (/ &
        & 0.91289e+02_rb,0.31968e+03_rb,0.42884e+03_rb,0.28523e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,41, 2) = (/ &
        & 0.91288e+02_rb,0.31952e+03_rb,0.42884e+03_rb,0.28521e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,41, 2) = (/ &
        & 0.91286e+02_rb,0.31928e+03_rb,0.42883e+03_rb,0.28522e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,41, 2) = (/ &
        & 0.91284e+02_rb,0.31895e+03_rb,0.42883e+03_rb,0.28521e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,41, 2) = (/ &
        & 0.91283e+02_rb,0.31854e+03_rb,0.42882e+03_rb,0.28521e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,42, 2) = (/ &
        & 0.91284e+02_rb,0.31970e+03_rb,0.42883e+03_rb,0.28521e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,42, 2) = (/ &
        & 0.91283e+02_rb,0.31957e+03_rb,0.42882e+03_rb,0.28521e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,42, 2) = (/ &
        & 0.91281e+02_rb,0.31934e+03_rb,0.42882e+03_rb,0.28520e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,42, 2) = (/ &
        & 0.91280e+02_rb,0.31904e+03_rb,0.42882e+03_rb,0.28519e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,42, 2) = (/ &
        & 0.91279e+02_rb,0.31865e+03_rb,0.42881e+03_rb,0.28520e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,43, 2) = (/ &
        & 0.91280e+02_rb,0.31972e+03_rb,0.42882e+03_rb,0.28520e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,43, 2) = (/ &
        & 0.91279e+02_rb,0.31961e+03_rb,0.42881e+03_rb,0.28520e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,43, 2) = (/ &
        & 0.91277e+02_rb,0.31942e+03_rb,0.42881e+03_rb,0.28519e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,43, 2) = (/ &
        & 0.91276e+02_rb,0.31914e+03_rb,0.42881e+03_rb,0.28518e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,43, 2) = (/ &
        & 0.91275e+02_rb,0.31878e+03_rb,0.42880e+03_rb,0.28518e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,44, 2) = (/ &
        & 0.91276e+02_rb,0.31974e+03_rb,0.42881e+03_rb,0.28519e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,44, 2) = (/ &
        & 0.91275e+02_rb,0.31966e+03_rb,0.42880e+03_rb,0.28519e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,44, 2) = (/ &
        & 0.91274e+02_rb,0.31949e+03_rb,0.42880e+03_rb,0.28518e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,44, 2) = (/ &
        & 0.91273e+02_rb,0.31924e+03_rb,0.42880e+03_rb,0.28518e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,44, 2) = (/ &
        & 0.91273e+02_rb,0.31890e+03_rb,0.42880e+03_rb,0.28518e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,45, 2) = (/ &
        & 0.91274e+02_rb,0.31974e+03_rb,0.42880e+03_rb,0.28518e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,45, 2) = (/ &
        & 0.91273e+02_rb,0.31969e+03_rb,0.42880e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,45, 2) = (/ &
        & 0.91272e+02_rb,0.31955e+03_rb,0.42879e+03_rb,0.28518e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,45, 2) = (/ &
        & 0.91271e+02_rb,0.31933e+03_rb,0.42879e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,45, 2) = (/ &
        & 0.91270e+02_rb,0.31903e+03_rb,0.42879e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,46, 2) = (/ &
        & 0.91271e+02_rb,0.31974e+03_rb,0.42879e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,46, 2) = (/ &
        & 0.91270e+02_rb,0.31972e+03_rb,0.42879e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,46, 2) = (/ &
        & 0.91270e+02_rb,0.31961e+03_rb,0.42879e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,46, 2) = (/ &
        & 0.91269e+02_rb,0.31942e+03_rb,0.42879e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,46, 2) = (/ &
        & 0.91268e+02_rb,0.31914e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,47, 2) = (/ &
        & 0.91269e+02_rb,0.31973e+03_rb,0.42879e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,47, 2) = (/ &
        & 0.91268e+02_rb,0.31974e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,47, 2) = (/ &
        & 0.91268e+02_rb,0.31966e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,47, 2) = (/ &
        & 0.91267e+02_rb,0.31951e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,47, 2) = (/ &
        & 0.91267e+02_rb,0.31926e+03_rb,0.42878e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,48, 2) = (/ &
        & 0.91267e+02_rb,0.31970e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,48, 2) = (/ &
        & 0.91267e+02_rb,0.31974e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,48, 2) = (/ &
        & 0.91266e+02_rb,0.31970e+03_rb,0.42878e+03_rb,0.28517e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,48, 2) = (/ &
        & 0.91266e+02_rb,0.31958e+03_rb,0.42878e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,48, 2) = (/ &
        & 0.91265e+02_rb,0.31937e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,49, 2) = (/ &
        & 0.91266e+02_rb,0.31966e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,49, 2) = (/ &
        & 0.91265e+02_rb,0.31973e+03_rb,0.42878e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,49, 2) = (/ &
        & 0.91265e+02_rb,0.31973e+03_rb,0.42877e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,49, 2) = (/ &
        & 0.91265e+02_rb,0.31964e+03_rb,0.42878e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,49, 2) = (/ &
        & 0.91264e+02_rb,0.31946e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,50, 2) = (/ &
        & 0.91265e+02_rb,0.31962e+03_rb,0.42877e+03_rb,0.28516e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,50, 2) = (/ &
        & 0.91264e+02_rb,0.31972e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,50, 2) = (/ &
        & 0.91264e+02_rb,0.31974e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,50, 2) = (/ &
        & 0.91264e+02_rb,0.31968e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,50, 2) = (/ &
        & 0.91263e+02_rb,0.31954e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,51, 2) = (/ &
        & 0.91264e+02_rb,0.31958e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,51, 2) = (/ &
        & 0.91263e+02_rb,0.31969e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,51, 2) = (/ &
        & 0.91263e+02_rb,0.31974e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,51, 2) = (/ &
        & 0.91263e+02_rb,0.31971e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,51, 2) = (/ &
        & 0.91262e+02_rb,0.31960e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,52, 2) = (/ &
        & 0.91263e+02_rb,0.31953e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,52, 2) = (/ &
        & 0.91263e+02_rb,0.31965e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,52, 2) = (/ &
        & 0.91262e+02_rb,0.31973e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,52, 2) = (/ &
        & 0.91262e+02_rb,0.31973e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,52, 2) = (/ &
        & 0.91262e+02_rb,0.31965e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,53, 2) = (/ &
        & 0.91262e+02_rb,0.31948e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,53, 2) = (/ &
        & 0.91262e+02_rb,0.31961e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,53, 2) = (/ &
        & 0.91262e+02_rb,0.31971e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,53, 2) = (/ &
        & 0.91261e+02_rb,0.31974e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,53, 2) = (/ &
        & 0.91261e+02_rb,0.31969e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,54, 2) = (/ &
        & 0.91262e+02_rb,0.31942e+03_rb,0.42877e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,54, 2) = (/ &
        & 0.91261e+02_rb,0.31957e+03_rb,0.42877e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,54, 2) = (/ &
        & 0.91261e+02_rb,0.31968e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,54, 2) = (/ &
        & 0.91261e+02_rb,0.31973e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,54, 2) = (/ &
        & 0.91261e+02_rb,0.31971e+03_rb,0.42876e+03_rb,0.28513e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,55, 2) = (/ &
        & 0.91261e+02_rb,0.31937e+03_rb,0.42876e+03_rb,0.28513e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,55, 2) = (/ &
        & 0.91261e+02_rb,0.31953e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,55, 2) = (/ &
        & 0.91261e+02_rb,0.31965e+03_rb,0.42876e+03_rb,0.28513e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,55, 2) = (/ &
        & 0.91261e+02_rb,0.31973e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,55, 2) = (/ &
        & 0.91260e+02_rb,0.31973e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,56, 2) = (/ &
        & 0.91261e+02_rb,0.31932e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,56, 2) = (/ &
        & 0.91260e+02_rb,0.31947e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,56, 2) = (/ &
        & 0.91260e+02_rb,0.31961e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,56, 2) = (/ &
        & 0.91260e+02_rb,0.31971e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,56, 2) = (/ &
        & 0.91260e+02_rb,0.31974e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,57, 2) = (/ &
        & 0.91260e+02_rb,0.31927e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,57, 2) = (/ &
        & 0.91260e+02_rb,0.31943e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,57, 2) = (/ &
        & 0.91260e+02_rb,0.31957e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,57, 2) = (/ &
        & 0.91260e+02_rb,0.31968e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,57, 2) = (/ &
        & 0.91260e+02_rb,0.31974e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,58, 2) = (/ &
        & 0.91260e+02_rb,0.31922e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,58, 2) = (/ &
        & 0.91260e+02_rb,0.31938e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,58, 2) = (/ &
        & 0.91260e+02_rb,0.31953e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,58, 2) = (/ &
        & 0.91260e+02_rb,0.31965e+03_rb,0.42876e+03_rb,0.28513e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,58, 2) = (/ &
        & 0.91260e+02_rb,0.31973e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,59, 2) = (/ &
        & 0.91260e+02_rb,0.31920e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 2,59, 2) = (/ &
        & 0.91260e+02_rb,0.31936e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 3,59, 2) = (/ &
        & 0.91259e+02_rb,0.31952e+03_rb,0.42876e+03_rb,0.28515e+03_rb,0.71839e+02_rb /)
      kbo(:, 4,59, 2) = (/ &
        & 0.91259e+02_rb,0.31964e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 5,59, 2) = (/ &
        & 0.91259e+02_rb,0.31972e+03_rb,0.42876e+03_rb,0.28514e+03_rb,0.71839e+02_rb /)
      kbo(:, 1,13, 3) = (/ &
        & 0.26463e+03_rb,0.36163e+03_rb,0.49889e+03_rb,0.32504e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,13, 3) = (/ &
        & 0.26258e+03_rb,0.36010e+03_rb,0.49695e+03_rb,0.32345e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,13, 3) = (/ &
        & 0.26083e+03_rb,0.35879e+03_rb,0.49519e+03_rb,0.32209e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,13, 3) = (/ &
        & 0.25930e+03_rb,0.35766e+03_rb,0.49360e+03_rb,0.32093e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,13, 3) = (/ &
        & 0.25795e+03_rb,0.35666e+03_rb,0.49217e+03_rb,0.31991e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,14, 3) = (/ &
        & 0.25951e+03_rb,0.35782e+03_rb,0.49358e+03_rb,0.32109e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,14, 3) = (/ &
        & 0.25786e+03_rb,0.35659e+03_rb,0.49191e+03_rb,0.31984e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,14, 3) = (/ &
        & 0.25644e+03_rb,0.35554e+03_rb,0.49043e+03_rb,0.31877e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,14, 3) = (/ &
        & 0.25520e+03_rb,0.35461e+03_rb,0.48912e+03_rb,0.31784e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,14, 3) = (/ &
        & 0.25411e+03_rb,0.35381e+03_rb,0.48795e+03_rb,0.31704e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,15, 3) = (/ &
        & 0.25534e+03_rb,0.35471e+03_rb,0.48908e+03_rb,0.31794e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,15, 3) = (/ &
        & 0.25401e+03_rb,0.35373e+03_rb,0.48771e+03_rb,0.31695e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,15, 3) = (/ &
        & 0.25286e+03_rb,0.35287e+03_rb,0.48651e+03_rb,0.31610e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,15, 3) = (/ &
        & 0.25186e+03_rb,0.35213e+03_rb,0.48543e+03_rb,0.31537e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,15, 3) = (/ &
        & 0.25098e+03_rb,0.35147e+03_rb,0.48447e+03_rb,0.31472e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,16, 3) = (/ &
        & 0.25194e+03_rb,0.35219e+03_rb,0.48538e+03_rb,0.31543e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,16, 3) = (/ &
        & 0.25087e+03_rb,0.35140e+03_rb,0.48427e+03_rb,0.31463e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,16, 3) = (/ &
        & 0.24994e+03_rb,0.35071e+03_rb,0.48328e+03_rb,0.31395e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,16, 3) = (/ &
        & 0.24914e+03_rb,0.35011e+03_rb,0.48239e+03_rb,0.31337e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,16, 3) = (/ &
        & 0.24843e+03_rb,0.34958e+03_rb,0.48159e+03_rb,0.31282e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,17, 3) = (/ &
        & 0.24919e+03_rb,0.35014e+03_rb,0.48236e+03_rb,0.31341e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,17, 3) = (/ &
        & 0.24832e+03_rb,0.34950e+03_rb,0.48145e+03_rb,0.31278e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,17, 3) = (/ &
        & 0.24758e+03_rb,0.34895e+03_rb,0.48063e+03_rb,0.31225e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,17, 3) = (/ &
        & 0.24693e+03_rb,0.34847e+03_rb,0.47989e+03_rb,0.31178e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,17, 3) = (/ &
        & 0.24635e+03_rb,0.34804e+03_rb,0.47924e+03_rb,0.31137e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,18, 3) = (/ &
        & 0.24695e+03_rb,0.34848e+03_rb,0.47988e+03_rb,0.31180e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,18, 3) = (/ &
        & 0.24626e+03_rb,0.34797e+03_rb,0.47913e+03_rb,0.31130e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,18, 3) = (/ &
        & 0.24565e+03_rb,0.34752e+03_rb,0.47845e+03_rb,0.31086e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,18, 3) = (/ &
        & 0.24513e+03_rb,0.34713e+03_rb,0.47784e+03_rb,0.31048e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,18, 3) = (/ &
        & 0.24466e+03_rb,0.34679e+03_rb,0.47731e+03_rb,0.31015e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,19, 3) = (/ &
        & 0.24514e+03_rb,0.34714e+03_rb,0.47784e+03_rb,0.31049e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,19, 3) = (/ &
        & 0.24458e+03_rb,0.34672e+03_rb,0.47721e+03_rb,0.31009e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,19, 3) = (/ &
        & 0.24409e+03_rb,0.34636e+03_rb,0.47665e+03_rb,0.30974e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,19, 3) = (/ &
        & 0.24367e+03_rb,0.34605e+03_rb,0.47616e+03_rb,0.30943e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,19, 3) = (/ &
        & 0.24329e+03_rb,0.34577e+03_rb,0.47573e+03_rb,0.30916e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,20, 3) = (/ &
        & 0.24366e+03_rb,0.34604e+03_rb,0.47615e+03_rb,0.30943e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,20, 3) = (/ &
        & 0.24321e+03_rb,0.34571e+03_rb,0.47563e+03_rb,0.30910e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,20, 3) = (/ &
        & 0.24282e+03_rb,0.34542e+03_rb,0.47518e+03_rb,0.30882e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,20, 3) = (/ &
        & 0.24247e+03_rb,0.34516e+03_rb,0.47478e+03_rb,0.30857e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,20, 3) = (/ &
        & 0.24217e+03_rb,0.34494e+03_rb,0.47443e+03_rb,0.30835e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,21, 3) = (/ &
        & 0.24246e+03_rb,0.34515e+03_rb,0.47476e+03_rb,0.30856e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,21, 3) = (/ &
        & 0.24210e+03_rb,0.34488e+03_rb,0.47434e+03_rb,0.30830e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,21, 3) = (/ &
        & 0.24178e+03_rb,0.34465e+03_rb,0.47397e+03_rb,0.30807e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,21, 3) = (/ &
        & 0.24150e+03_rb,0.34444e+03_rb,0.47364e+03_rb,0.30787e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,21, 3) = (/ &
        & 0.24126e+03_rb,0.34426e+03_rb,0.47341e+03_rb,0.30769e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,22, 3) = (/ &
        & 0.24146e+03_rb,0.34441e+03_rb,0.47360e+03_rb,0.30784e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,22, 3) = (/ &
        & 0.24117e+03_rb,0.34420e+03_rb,0.47327e+03_rb,0.30763e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,22, 3) = (/ &
        & 0.24092e+03_rb,0.34401e+03_rb,0.47296e+03_rb,0.30745e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,22, 3) = (/ &
        & 0.24070e+03_rb,0.34385e+03_rb,0.47270e+03_rb,0.30729e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,22, 3) = (/ &
        & 0.24050e+03_rb,0.34370e+03_rb,0.47246e+03_rb,0.30715e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,23, 3) = (/ &
        & 0.24066e+03_rb,0.34382e+03_rb,0.47265e+03_rb,0.30726e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,23, 3) = (/ &
        & 0.24043e+03_rb,0.34365e+03_rb,0.47237e+03_rb,0.30709e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,23, 3) = (/ &
        & 0.24023e+03_rb,0.34350e+03_rb,0.47213e+03_rb,0.30695e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,23, 3) = (/ &
        & 0.24005e+03_rb,0.34336e+03_rb,0.47191e+03_rb,0.30682e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,23, 3) = (/ &
        & 0.23989e+03_rb,0.34325e+03_rb,0.47173e+03_rb,0.30671e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,24, 3) = (/ &
        & 0.24001e+03_rb,0.34333e+03_rb,0.47187e+03_rb,0.30679e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,24, 3) = (/ &
        & 0.23982e+03_rb,0.34320e+03_rb,0.47164e+03_rb,0.30666e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,24, 3) = (/ &
        & 0.23966e+03_rb,0.34308e+03_rb,0.47145e+03_rb,0.30654e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,24, 3) = (/ &
        & 0.23952e+03_rb,0.34297e+03_rb,0.47128e+03_rb,0.30644e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,24, 3) = (/ &
        & 0.23940e+03_rb,0.34288e+03_rb,0.47113e+03_rb,0.30635e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,25, 3) = (/ &
        & 0.23948e+03_rb,0.34294e+03_rb,0.47124e+03_rb,0.30641e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,25, 3) = (/ &
        & 0.23934e+03_rb,0.34284e+03_rb,0.47105e+03_rb,0.30631e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,25, 3) = (/ &
        & 0.23921e+03_rb,0.34274e+03_rb,0.47089e+03_rb,0.30622e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,25, 3) = (/ &
        & 0.23910e+03_rb,0.34266e+03_rb,0.47075e+03_rb,0.30613e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,25, 3) = (/ &
        & 0.23900e+03_rb,0.34258e+03_rb,0.47063e+03_rb,0.30606e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,26, 3) = (/ &
        & 0.23906e+03_rb,0.34263e+03_rb,0.47070e+03_rb,0.30610e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,26, 3) = (/ &
        & 0.23894e+03_rb,0.34254e+03_rb,0.47056e+03_rb,0.30602e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,26, 3) = (/ &
        & 0.23884e+03_rb,0.34247e+03_rb,0.47044e+03_rb,0.30595e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,26, 3) = (/ &
        & 0.23875e+03_rb,0.34240e+03_rb,0.47033e+03_rb,0.30588e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,26, 3) = (/ &
        & 0.23867e+03_rb,0.34234e+03_rb,0.47023e+03_rb,0.30583e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,27, 3) = (/ &
        & 0.23871e+03_rb,0.34237e+03_rb,0.47028e+03_rb,0.30586e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,27, 3) = (/ &
        & 0.23862e+03_rb,0.34230e+03_rb,0.47017e+03_rb,0.30579e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,27, 3) = (/ &
        & 0.23854e+03_rb,0.34225e+03_rb,0.47007e+03_rb,0.30573e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,27, 3) = (/ &
        & 0.23847e+03_rb,0.34219e+03_rb,0.46998e+03_rb,0.30568e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,27, 3) = (/ &
        & 0.23840e+03_rb,0.34214e+03_rb,0.46990e+03_rb,0.30564e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,28, 3) = (/ &
        & 0.23844e+03_rb,0.34217e+03_rb,0.46994e+03_rb,0.30566e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,28, 3) = (/ &
        & 0.23836e+03_rb,0.34211e+03_rb,0.46985e+03_rb,0.30561e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,28, 3) = (/ &
        & 0.23830e+03_rb,0.34207e+03_rb,0.46977e+03_rb,0.30556e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,28, 3) = (/ &
        & 0.23824e+03_rb,0.34202e+03_rb,0.46970e+03_rb,0.30552e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,28, 3) = (/ &
        & 0.23819e+03_rb,0.34198e+03_rb,0.46964e+03_rb,0.30548e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,29, 3) = (/ &
        & 0.23821e+03_rb,0.34200e+03_rb,0.46966e+03_rb,0.30550e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,29, 3) = (/ &
        & 0.23815e+03_rb,0.34196e+03_rb,0.46959e+03_rb,0.30546e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,29, 3) = (/ &
        & 0.23810e+03_rb,0.34192e+03_rb,0.46953e+03_rb,0.30542e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,29, 3) = (/ &
        & 0.23806e+03_rb,0.34189e+03_rb,0.46948e+03_rb,0.30539e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,29, 3) = (/ &
        & 0.23802e+03_rb,0.34185e+03_rb,0.46942e+03_rb,0.30536e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,30, 3) = (/ &
        & 0.23803e+03_rb,0.34187e+03_rb,0.46944e+03_rb,0.30537e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,30, 3) = (/ &
        & 0.23799e+03_rb,0.34184e+03_rb,0.46939e+03_rb,0.30533e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,30, 3) = (/ &
        & 0.23795e+03_rb,0.34180e+03_rb,0.46934e+03_rb,0.30531e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,30, 3) = (/ &
        & 0.23791e+03_rb,0.34178e+03_rb,0.46929e+03_rb,0.30528e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,30, 3) = (/ &
        & 0.23788e+03_rb,0.34175e+03_rb,0.46925e+03_rb,0.30526e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,31, 3) = (/ &
        & 0.23789e+03_rb,0.34176e+03_rb,0.46926e+03_rb,0.30526e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,31, 3) = (/ &
        & 0.23785e+03_rb,0.34173e+03_rb,0.46922e+03_rb,0.30524e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,31, 3) = (/ &
        & 0.23782e+03_rb,0.34171e+03_rb,0.46918e+03_rb,0.30521e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,31, 3) = (/ &
        & 0.23779e+03_rb,0.34169e+03_rb,0.46914e+03_rb,0.30519e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,31, 3) = (/ &
        & 0.23777e+03_rb,0.34167e+03_rb,0.46911e+03_rb,0.30518e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,32, 3) = (/ &
        & 0.23777e+03_rb,0.34168e+03_rb,0.46912e+03_rb,0.30518e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,32, 3) = (/ &
        & 0.23774e+03_rb,0.34166e+03_rb,0.46908e+03_rb,0.30516e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,32, 3) = (/ &
        & 0.23772e+03_rb,0.34163e+03_rb,0.46905e+03_rb,0.30514e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,32, 3) = (/ &
        & 0.23770e+03_rb,0.34162e+03_rb,0.46902e+03_rb,0.30512e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,32, 3) = (/ &
        & 0.23767e+03_rb,0.34160e+03_rb,0.46900e+03_rb,0.30511e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,33, 3) = (/ &
        & 0.23768e+03_rb,0.34161e+03_rb,0.46900e+03_rb,0.30511e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,33, 3) = (/ &
        & 0.23766e+03_rb,0.34159e+03_rb,0.46897e+03_rb,0.30510e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,33, 3) = (/ &
        & 0.23764e+03_rb,0.34157e+03_rb,0.46895e+03_rb,0.30508e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,33, 3) = (/ &
        & 0.23762e+03_rb,0.34156e+03_rb,0.46893e+03_rb,0.30507e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,33, 3) = (/ &
        & 0.23760e+03_rb,0.34154e+03_rb,0.46891e+03_rb,0.30506e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,34, 3) = (/ &
        & 0.23760e+03_rb,0.34155e+03_rb,0.46891e+03_rb,0.30506e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,34, 3) = (/ &
        & 0.23759e+03_rb,0.34154e+03_rb,0.46889e+03_rb,0.30505e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,34, 3) = (/ &
        & 0.23757e+03_rb,0.34152e+03_rb,0.46887e+03_rb,0.30503e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,34, 3) = (/ &
        & 0.23756e+03_rb,0.34151e+03_rb,0.46885e+03_rb,0.30502e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,34, 3) = (/ &
        & 0.23754e+03_rb,0.34150e+03_rb,0.46883e+03_rb,0.30501e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,35, 3) = (/ &
        & 0.23755e+03_rb,0.34151e+03_rb,0.46884e+03_rb,0.30502e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,35, 3) = (/ &
        & 0.23753e+03_rb,0.34150e+03_rb,0.46882e+03_rb,0.30501e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,35, 3) = (/ &
        & 0.23752e+03_rb,0.34148e+03_rb,0.46880e+03_rb,0.30500e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,35, 3) = (/ &
        & 0.23751e+03_rb,0.34147e+03_rb,0.46879e+03_rb,0.30499e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,35, 3) = (/ &
        & 0.23750e+03_rb,0.34146e+03_rb,0.46878e+03_rb,0.30498e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,36, 3) = (/ &
        & 0.23750e+03_rb,0.34147e+03_rb,0.46878e+03_rb,0.30498e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,36, 3) = (/ &
        & 0.23749e+03_rb,0.34146e+03_rb,0.46877e+03_rb,0.30497e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,36, 3) = (/ &
        & 0.23748e+03_rb,0.34145e+03_rb,0.46875e+03_rb,0.30497e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,36, 3) = (/ &
        & 0.23747e+03_rb,0.34144e+03_rb,0.46874e+03_rb,0.30496e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,36, 3) = (/ &
        & 0.23746e+03_rb,0.34144e+03_rb,0.46873e+03_rb,0.30495e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,37, 3) = (/ &
        & 0.23746e+03_rb,0.34145e+03_rb,0.46874e+03_rb,0.30496e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,37, 3) = (/ &
        & 0.23745e+03_rb,0.34144e+03_rb,0.46872e+03_rb,0.30495e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,37, 3) = (/ &
        & 0.23744e+03_rb,0.34143e+03_rb,0.46871e+03_rb,0.30494e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,37, 3) = (/ &
        & 0.23744e+03_rb,0.34142e+03_rb,0.46870e+03_rb,0.30494e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,37, 3) = (/ &
        & 0.23743e+03_rb,0.34141e+03_rb,0.46869e+03_rb,0.30493e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,38, 3) = (/ &
        & 0.23743e+03_rb,0.34143e+03_rb,0.46870e+03_rb,0.30493e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,38, 3) = (/ &
        & 0.23742e+03_rb,0.34142e+03_rb,0.46869e+03_rb,0.30493e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,38, 3) = (/ &
        & 0.23742e+03_rb,0.34141e+03_rb,0.46868e+03_rb,0.30492e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,38, 3) = (/ &
        & 0.23741e+03_rb,0.34140e+03_rb,0.46867e+03_rb,0.30492e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,38, 3) = (/ &
        & 0.23740e+03_rb,0.34140e+03_rb,0.46866e+03_rb,0.30491e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,39, 3) = (/ &
        & 0.23741e+03_rb,0.34141e+03_rb,0.46867e+03_rb,0.30492e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,39, 3) = (/ &
        & 0.23740e+03_rb,0.34140e+03_rb,0.46866e+03_rb,0.30491e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,39, 3) = (/ &
        & 0.23739e+03_rb,0.34139e+03_rb,0.46865e+03_rb,0.30491e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,39, 3) = (/ &
        & 0.23739e+03_rb,0.34139e+03_rb,0.46865e+03_rb,0.30490e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,39, 3) = (/ &
        & 0.23738e+03_rb,0.34138e+03_rb,0.46864e+03_rb,0.30490e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,40, 3) = (/ &
        & 0.23739e+03_rb,0.34139e+03_rb,0.46864e+03_rb,0.30490e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,40, 3) = (/ &
        & 0.23738e+03_rb,0.34139e+03_rb,0.46864e+03_rb,0.30490e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,40, 3) = (/ &
        & 0.23738e+03_rb,0.34138e+03_rb,0.46863e+03_rb,0.30489e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,40, 3) = (/ &
        & 0.23737e+03_rb,0.34138e+03_rb,0.46862e+03_rb,0.30489e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,40, 3) = (/ &
        & 0.23737e+03_rb,0.34137e+03_rb,0.46862e+03_rb,0.30489e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,41, 3) = (/ &
        & 0.23737e+03_rb,0.34138e+03_rb,0.46862e+03_rb,0.30489e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,41, 3) = (/ &
        & 0.23737e+03_rb,0.34138e+03_rb,0.46862e+03_rb,0.30489e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,41, 3) = (/ &
        & 0.23736e+03_rb,0.34137e+03_rb,0.46861e+03_rb,0.30488e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,41, 3) = (/ &
        & 0.23736e+03_rb,0.34137e+03_rb,0.46861e+03_rb,0.30488e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,41, 3) = (/ &
        & 0.23736e+03_rb,0.34136e+03_rb,0.46860e+03_rb,0.30488e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,42, 3) = (/ &
        & 0.23736e+03_rb,0.34137e+03_rb,0.46861e+03_rb,0.30488e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,42, 3) = (/ &
        & 0.23735e+03_rb,0.34137e+03_rb,0.46860e+03_rb,0.30488e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,42, 3) = (/ &
        & 0.23735e+03_rb,0.34136e+03_rb,0.46860e+03_rb,0.30488e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,42, 3) = (/ &
        & 0.23735e+03_rb,0.34136e+03_rb,0.46859e+03_rb,0.30487e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,42, 3) = (/ &
        & 0.23734e+03_rb,0.34135e+03_rb,0.46859e+03_rb,0.30487e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,43, 3) = (/ &
        & 0.23735e+03_rb,0.34136e+03_rb,0.46859e+03_rb,0.30487e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,43, 3) = (/ &
        & 0.23734e+03_rb,0.34136e+03_rb,0.46859e+03_rb,0.30487e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,43, 3) = (/ &
        & 0.23734e+03_rb,0.34136e+03_rb,0.46858e+03_rb,0.30487e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,43, 3) = (/ &
        & 0.23734e+03_rb,0.34135e+03_rb,0.46858e+03_rb,0.30487e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,43, 3) = (/ &
        & 0.23734e+03_rb,0.34135e+03_rb,0.46858e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,44, 3) = (/ &
        & 0.23734e+03_rb,0.34136e+03_rb,0.46858e+03_rb,0.30487e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,44, 3) = (/ &
        & 0.23734e+03_rb,0.34135e+03_rb,0.46858e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,44, 3) = (/ &
        & 0.23733e+03_rb,0.34135e+03_rb,0.46857e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,44, 3) = (/ &
        & 0.23733e+03_rb,0.34135e+03_rb,0.46857e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,44, 3) = (/ &
        & 0.23733e+03_rb,0.34134e+03_rb,0.46857e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,45, 3) = (/ &
        & 0.23733e+03_rb,0.34135e+03_rb,0.46857e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,45, 3) = (/ &
        & 0.23733e+03_rb,0.34135e+03_rb,0.46857e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,45, 3) = (/ &
        & 0.23733e+03_rb,0.34135e+03_rb,0.46857e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,45, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46856e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,45, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46856e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,46, 3) = (/ &
        & 0.23732e+03_rb,0.34135e+03_rb,0.46856e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,46, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46856e+03_rb,0.30486e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,46, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46856e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,46, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46856e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,46, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46856e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,47, 3) = (/ &
        & 0.23732e+03_rb,0.34135e+03_rb,0.46856e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,47, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46856e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,47, 3) = (/ &
        & 0.23732e+03_rb,0.34134e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,47, 3) = (/ &
        & 0.23731e+03_rb,0.34134e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,47, 3) = (/ &
        & 0.23731e+03_rb,0.34133e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,48, 3) = (/ &
        & 0.23731e+03_rb,0.34135e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,48, 3) = (/ &
        & 0.23731e+03_rb,0.34134e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,48, 3) = (/ &
        & 0.23731e+03_rb,0.34134e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,48, 3) = (/ &
        & 0.23731e+03_rb,0.34134e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,48, 3) = (/ &
        & 0.23731e+03_rb,0.34133e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,49, 3) = (/ &
        & 0.23731e+03_rb,0.34135e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,49, 3) = (/ &
        & 0.23731e+03_rb,0.34134e+03_rb,0.46855e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,49, 3) = (/ &
        & 0.23731e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30485e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,49, 3) = (/ &
        & 0.23731e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,49, 3) = (/ &
        & 0.23731e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,50, 3) = (/ &
        & 0.23731e+03_rb,0.34136e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,50, 3) = (/ &
        & 0.23731e+03_rb,0.34134e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,50, 3) = (/ &
        & 0.23731e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,50, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,50, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,51, 3) = (/ &
        & 0.23731e+03_rb,0.34136e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,51, 3) = (/ &
        & 0.23730e+03_rb,0.34134e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,51, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,51, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,51, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,52, 3) = (/ &
        & 0.23730e+03_rb,0.34136e+03_rb,0.46852e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,52, 3) = (/ &
        & 0.23730e+03_rb,0.34135e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,52, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,52, 3) = (/ &
        & 0.23730e+03_rb,0.34132e+03_rb,0.46854e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,52, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,53, 3) = (/ &
        & 0.23730e+03_rb,0.34137e+03_rb,0.46851e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,53, 3) = (/ &
        & 0.23730e+03_rb,0.34135e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,53, 3) = (/ &
        & 0.23730e+03_rb,0.34134e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,53, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,53, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,54, 3) = (/ &
        & 0.23730e+03_rb,0.34137e+03_rb,0.46849e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,54, 3) = (/ &
        & 0.23730e+03_rb,0.34136e+03_rb,0.46852e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,54, 3) = (/ &
        & 0.23730e+03_rb,0.34134e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,54, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,54, 3) = (/ &
        & 0.23730e+03_rb,0.34132e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,55, 3) = (/ &
        & 0.23730e+03_rb,0.34138e+03_rb,0.46847e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,55, 3) = (/ &
        & 0.23730e+03_rb,0.34136e+03_rb,0.46852e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,55, 3) = (/ &
        & 0.23730e+03_rb,0.34134e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,55, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,55, 3) = (/ &
        & 0.23730e+03_rb,0.34132e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,56, 3) = (/ &
        & 0.23730e+03_rb,0.34139e+03_rb,0.46844e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,56, 3) = (/ &
        & 0.23730e+03_rb,0.34137e+03_rb,0.46850e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,56, 3) = (/ &
        & 0.23730e+03_rb,0.34135e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,56, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,56, 3) = (/ &
        & 0.23730e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,57, 3) = (/ &
        & 0.23730e+03_rb,0.34140e+03_rb,0.46840e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,57, 3) = (/ &
        & 0.23730e+03_rb,0.34137e+03_rb,0.46849e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,57, 3) = (/ &
        & 0.23730e+03_rb,0.34135e+03_rb,0.46852e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,57, 3) = (/ &
        & 0.23730e+03_rb,0.34134e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,57, 3) = (/ &
        & 0.23729e+03_rb,0.34132e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,58, 3) = (/ &
        & 0.23730e+03_rb,0.34140e+03_rb,0.46837e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,58, 3) = (/ &
        & 0.23729e+03_rb,0.34138e+03_rb,0.46847e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,58, 3) = (/ &
        & 0.23729e+03_rb,0.34136e+03_rb,0.46851e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,58, 3) = (/ &
        & 0.23729e+03_rb,0.34134e+03_rb,0.46853e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,58, 3) = (/ &
        & 0.23729e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30483e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,59, 3) = (/ &
        & 0.23729e+03_rb,0.34141e+03_rb,0.46835e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 2,59, 3) = (/ &
        & 0.23729e+03_rb,0.34138e+03_rb,0.46846e+03_rb,0.30484e+03_rb,0.17829e+03_rb /)
      kbo(:, 3,59, 3) = (/ &
        & 0.23729e+03_rb,0.34136e+03_rb,0.46851e+03_rb,0.30483e+03_rb,0.17829e+03_rb /)
      kbo(:, 4,59, 3) = (/ &
        & 0.23729e+03_rb,0.34134e+03_rb,0.46853e+03_rb,0.30483e+03_rb,0.17829e+03_rb /)
      kbo(:, 5,59, 3) = (/ &
        & 0.23729e+03_rb,0.34133e+03_rb,0.46853e+03_rb,0.30483e+03_rb,0.17829e+03_rb /)
      kbo(:, 1,13, 4) = (/ &
        & 0.51214e+03_rb,0.47089e+03_rb,0.53698e+03_rb,0.40623e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,13, 4) = (/ &
        & 0.50819e+03_rb,0.46793e+03_rb,0.53525e+03_rb,0.40532e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,13, 4) = (/ &
        & 0.50478e+03_rb,0.46537e+03_rb,0.53379e+03_rb,0.40453e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,13, 4) = (/ &
        & 0.50182e+03_rb,0.46315e+03_rb,0.53251e+03_rb,0.40385e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,13, 4) = (/ &
        & 0.49922e+03_rb,0.46120e+03_rb,0.53137e+03_rb,0.40325e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,14, 4) = (/ &
        & 0.50224e+03_rb,0.46346e+03_rb,0.53224e+03_rb,0.40395e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,14, 4) = (/ &
        & 0.49905e+03_rb,0.46107e+03_rb,0.53090e+03_rb,0.40321e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,14, 4) = (/ &
        & 0.49629e+03_rb,0.45901e+03_rb,0.52974e+03_rb,0.40258e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,14, 4) = (/ &
        & 0.49390e+03_rb,0.45721e+03_rb,0.52871e+03_rb,0.40203e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,14, 4) = (/ &
        & 0.49179e+03_rb,0.45563e+03_rb,0.52774e+03_rb,0.40154e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,15, 4) = (/ &
        & 0.49416e+03_rb,0.45741e+03_rb,0.52835e+03_rb,0.40209e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,15, 4) = (/ &
        & 0.49159e+03_rb,0.45548e+03_rb,0.52725e+03_rb,0.40149e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,15, 4) = (/ &
        & 0.48937e+03_rb,0.45381e+03_rb,0.52626e+03_rb,0.40098e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,15, 4) = (/ &
        & 0.48743e+03_rb,0.45236e+03_rb,0.52534e+03_rb,0.40054e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,15, 4) = (/ &
        & 0.48573e+03_rb,0.45108e+03_rb,0.52448e+03_rb,0.40015e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,16, 4) = (/ &
        & 0.48759e+03_rb,0.45248e+03_rb,0.52492e+03_rb,0.40057e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,16, 4) = (/ &
        & 0.48552e+03_rb,0.45092e+03_rb,0.52397e+03_rb,0.40010e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,16, 4) = (/ &
        & 0.48373e+03_rb,0.44958e+03_rb,0.52310e+03_rb,0.39968e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,16, 4) = (/ &
        & 0.48216e+03_rb,0.44841e+03_rb,0.52232e+03_rb,0.39932e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,16, 4) = (/ &
        & 0.48079e+03_rb,0.44738e+03_rb,0.52159e+03_rb,0.39901e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,17, 4) = (/ &
        & 0.48226e+03_rb,0.44848e+03_rb,0.52188e+03_rb,0.39935e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,17, 4) = (/ &
        & 0.48059e+03_rb,0.44723e+03_rb,0.52110e+03_rb,0.39896e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,17, 4) = (/ &
        & 0.47915e+03_rb,0.44615e+03_rb,0.52038e+03_rb,0.39863e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,17, 4) = (/ &
        & 0.47788e+03_rb,0.44520e+03_rb,0.51974e+03_rb,0.39834e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,17, 4) = (/ &
        & 0.47677e+03_rb,0.44437e+03_rb,0.51913e+03_rb,0.39808e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,18, 4) = (/ &
        & 0.47794e+03_rb,0.44524e+03_rb,0.51928e+03_rb,0.39835e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,18, 4) = (/ &
        & 0.47659e+03_rb,0.44423e+03_rb,0.51865e+03_rb,0.39804e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,18, 4) = (/ &
        & 0.47542e+03_rb,0.44336e+03_rb,0.51808e+03_rb,0.39777e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,18, 4) = (/ &
        & 0.47441e+03_rb,0.44259e+03_rb,0.51755e+03_rb,0.39754e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,18, 4) = (/ &
        & 0.47351e+03_rb,0.44192e+03_rb,0.51705e+03_rb,0.39733e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,19, 4) = (/ &
        & 0.47443e+03_rb,0.44261e+03_rb,0.51710e+03_rb,0.39754e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,19, 4) = (/ &
        & 0.47334e+03_rb,0.44179e+03_rb,0.51660e+03_rb,0.39729e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,19, 4) = (/ &
        & 0.47240e+03_rb,0.44109e+03_rb,0.51615e+03_rb,0.39708e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,19, 4) = (/ &
        & 0.47158e+03_rb,0.44047e+03_rb,0.51573e+03_rb,0.39689e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,19, 4) = (/ &
        & 0.47085e+03_rb,0.43993e+03_rb,0.51533e+03_rb,0.39672e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,20, 4) = (/ &
        & 0.47156e+03_rb,0.44046e+03_rb,0.51527e+03_rb,0.39688e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,20, 4) = (/ &
        & 0.47069e+03_rb,0.43980e+03_rb,0.51490e+03_rb,0.39668e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,20, 4) = (/ &
        & 0.46993e+03_rb,0.43924e+03_rb,0.51455e+03_rb,0.39651e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,20, 4) = (/ &
        & 0.46927e+03_rb,0.43874e+03_rb,0.51422e+03_rb,0.39635e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,20, 4) = (/ &
        & 0.46868e+03_rb,0.43830e+03_rb,0.51389e+03_rb,0.39622e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,21, 4) = (/ &
        & 0.46924e+03_rb,0.43872e+03_rb,0.51377e+03_rb,0.39635e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,21, 4) = (/ &
        & 0.46853e+03_rb,0.43819e+03_rb,0.51350e+03_rb,0.39619e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,21, 4) = (/ &
        & 0.46792e+03_rb,0.43773e+03_rb,0.51323e+03_rb,0.39604e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,21, 4) = (/ &
        & 0.46739e+03_rb,0.43733e+03_rb,0.51296e+03_rb,0.39592e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,21, 4) = (/ &
        & 0.46692e+03_rb,0.43697e+03_rb,0.51263e+03_rb,0.39581e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,22, 4) = (/ &
        & 0.46731e+03_rb,0.43727e+03_rb,0.51254e+03_rb,0.39590e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,22, 4) = (/ &
        & 0.46675e+03_rb,0.43685e+03_rb,0.51231e+03_rb,0.39577e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,22, 4) = (/ &
        & 0.46626e+03_rb,0.43648e+03_rb,0.51212e+03_rb,0.39566e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,22, 4) = (/ &
        & 0.46584e+03_rb,0.43616e+03_rb,0.51190e+03_rb,0.39556e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,22, 4) = (/ &
        & 0.46546e+03_rb,0.43588e+03_rb,0.51168e+03_rb,0.39548e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,23, 4) = (/ &
        & 0.46575e+03_rb,0.43610e+03_rb,0.51153e+03_rb,0.39554e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,23, 4) = (/ &
        & 0.46531e+03_rb,0.43577e+03_rb,0.51138e+03_rb,0.39544e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,23, 4) = (/ &
        & 0.46492e+03_rb,0.43548e+03_rb,0.51122e+03_rb,0.39535e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,23, 4) = (/ &
        & 0.46458e+03_rb,0.43522e+03_rb,0.51104e+03_rb,0.39527e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,23, 4) = (/ &
        & 0.46427e+03_rb,0.43499e+03_rb,0.51086e+03_rb,0.39520e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,24, 4) = (/ &
        & 0.46450e+03_rb,0.43516e+03_rb,0.51071e+03_rb,0.39525e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,24, 4) = (/ &
        & 0.46414e+03_rb,0.43489e+03_rb,0.51060e+03_rb,0.39517e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,24, 4) = (/ &
        & 0.46383e+03_rb,0.43466e+03_rb,0.51047e+03_rb,0.39510e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,24, 4) = (/ &
        & 0.46356e+03_rb,0.43445e+03_rb,0.51033e+03_rb,0.39504e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,24, 4) = (/ &
        & 0.46331e+03_rb,0.43427e+03_rb,0.51019e+03_rb,0.39498e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,25, 4) = (/ &
        & 0.46348e+03_rb,0.43439e+03_rb,0.51003e+03_rb,0.39502e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,25, 4) = (/ &
        & 0.46319e+03_rb,0.43418e+03_rb,0.50997e+03_rb,0.39496e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,25, 4) = (/ &
        & 0.46295e+03_rb,0.43400e+03_rb,0.50987e+03_rb,0.39490e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,25, 4) = (/ &
        & 0.46273e+03_rb,0.43383e+03_rb,0.50976e+03_rb,0.39485e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,25, 4) = (/ &
        & 0.46254e+03_rb,0.43369e+03_rb,0.50965e+03_rb,0.39480e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,26, 4) = (/ &
        & 0.46265e+03_rb,0.43378e+03_rb,0.50951e+03_rb,0.39483e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,26, 4) = (/ &
        & 0.46243e+03_rb,0.43361e+03_rb,0.50946e+03_rb,0.39478e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,26, 4) = (/ &
        & 0.46223e+03_rb,0.43346e+03_rb,0.50938e+03_rb,0.39473e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,26, 4) = (/ &
        & 0.46206e+03_rb,0.43333e+03_rb,0.50930e+03_rb,0.39469e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,26, 4) = (/ &
        & 0.46190e+03_rb,0.43321e+03_rb,0.50921e+03_rb,0.39466e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,27, 4) = (/ &
        & 0.46199e+03_rb,0.43328e+03_rb,0.50908e+03_rb,0.39468e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,27, 4) = (/ &
        & 0.46181e+03_rb,0.43314e+03_rb,0.50904e+03_rb,0.39464e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,27, 4) = (/ &
        & 0.46165e+03_rb,0.43303e+03_rb,0.50899e+03_rb,0.39460e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,27, 4) = (/ &
        & 0.46152e+03_rb,0.43292e+03_rb,0.50892e+03_rb,0.39457e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,27, 4) = (/ &
        & 0.46139e+03_rb,0.43283e+03_rb,0.50885e+03_rb,0.39454e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,28, 4) = (/ &
        & 0.46145e+03_rb,0.43288e+03_rb,0.50873e+03_rb,0.39455e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,28, 4) = (/ &
        & 0.46131e+03_rb,0.43277e+03_rb,0.50871e+03_rb,0.39452e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,28, 4) = (/ &
        & 0.46119e+03_rb,0.43268e+03_rb,0.50867e+03_rb,0.39449e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,28, 4) = (/ &
        & 0.46108e+03_rb,0.43259e+03_rb,0.50862e+03_rb,0.39447e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,28, 4) = (/ &
        & 0.46098e+03_rb,0.43252e+03_rb,0.50856e+03_rb,0.39445e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,29, 4) = (/ &
        & 0.46102e+03_rb,0.43255e+03_rb,0.50845e+03_rb,0.39445e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,29, 4) = (/ &
        & 0.46091e+03_rb,0.43247e+03_rb,0.50844e+03_rb,0.39443e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,29, 4) = (/ &
        & 0.46081e+03_rb,0.43239e+03_rb,0.50842e+03_rb,0.39441e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,29, 4) = (/ &
        & 0.46072e+03_rb,0.43233e+03_rb,0.50837e+03_rb,0.39439e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,29, 4) = (/ &
        & 0.46064e+03_rb,0.43227e+03_rb,0.50832e+03_rb,0.39437e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,30, 4) = (/ &
        & 0.46067e+03_rb,0.43229e+03_rb,0.50823e+03_rb,0.39437e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,30, 4) = (/ &
        & 0.46058e+03_rb,0.43223e+03_rb,0.50823e+03_rb,0.39435e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,30, 4) = (/ &
        & 0.46051e+03_rb,0.43217e+03_rb,0.50821e+03_rb,0.39434e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,30, 4) = (/ &
        & 0.46044e+03_rb,0.43211e+03_rb,0.50818e+03_rb,0.39432e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,30, 4) = (/ &
        & 0.46037e+03_rb,0.43207e+03_rb,0.50813e+03_rb,0.39431e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,31, 4) = (/ &
        & 0.46039e+03_rb,0.43208e+03_rb,0.50805e+03_rb,0.39431e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,31, 4) = (/ &
        & 0.46032e+03_rb,0.43203e+03_rb,0.50805e+03_rb,0.39429e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,31, 4) = (/ &
        & 0.46026e+03_rb,0.43198e+03_rb,0.50804e+03_rb,0.39428e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,31, 4) = (/ &
        & 0.46021e+03_rb,0.43194e+03_rb,0.50801e+03_rb,0.39427e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,31, 4) = (/ &
        & 0.46016e+03_rb,0.43190e+03_rb,0.50797e+03_rb,0.39426e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,32, 4) = (/ &
        & 0.46017e+03_rb,0.43191e+03_rb,0.50790e+03_rb,0.39426e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,32, 4) = (/ &
        & 0.46011e+03_rb,0.43187e+03_rb,0.50792e+03_rb,0.39425e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,32, 4) = (/ &
        & 0.46006e+03_rb,0.43183e+03_rb,0.50791e+03_rb,0.39423e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,32, 4) = (/ &
        & 0.46002e+03_rb,0.43180e+03_rb,0.50788e+03_rb,0.39422e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,32, 4) = (/ &
        & 0.45998e+03_rb,0.43177e+03_rb,0.50784e+03_rb,0.39421e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,33, 4) = (/ &
        & 0.45999e+03_rb,0.43178e+03_rb,0.50779e+03_rb,0.39422e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,33, 4) = (/ &
        & 0.45994e+03_rb,0.43174e+03_rb,0.50781e+03_rb,0.39421e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,33, 4) = (/ &
        & 0.45990e+03_rb,0.43171e+03_rb,0.50780e+03_rb,0.39420e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,33, 4) = (/ &
        & 0.45987e+03_rb,0.43169e+03_rb,0.50777e+03_rb,0.39419e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,33, 4) = (/ &
        & 0.45984e+03_rb,0.43166e+03_rb,0.50773e+03_rb,0.39418e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,34, 4) = (/ &
        & 0.45984e+03_rb,0.43167e+03_rb,0.50770e+03_rb,0.39418e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,34, 4) = (/ &
        & 0.45981e+03_rb,0.43164e+03_rb,0.50772e+03_rb,0.39418e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,34, 4) = (/ &
        & 0.45978e+03_rb,0.43162e+03_rb,0.50771e+03_rb,0.39417e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,34, 4) = (/ &
        & 0.45975e+03_rb,0.43160e+03_rb,0.50768e+03_rb,0.39416e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,34, 4) = (/ &
        & 0.45972e+03_rb,0.43158e+03_rb,0.50764e+03_rb,0.39416e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,35, 4) = (/ &
        & 0.45973e+03_rb,0.43158e+03_rb,0.50763e+03_rb,0.39416e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,35, 4) = (/ &
        & 0.45970e+03_rb,0.43156e+03_rb,0.50764e+03_rb,0.39415e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,35, 4) = (/ &
        & 0.45968e+03_rb,0.43154e+03_rb,0.50764e+03_rb,0.39414e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,35, 4) = (/ &
        & 0.45965e+03_rb,0.43153e+03_rb,0.50761e+03_rb,0.39414e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,35, 4) = (/ &
        & 0.45963e+03_rb,0.43151e+03_rb,0.50757e+03_rb,0.39413e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,36, 4) = (/ &
        & 0.45964e+03_rb,0.43152e+03_rb,0.50757e+03_rb,0.39414e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,36, 4) = (/ &
        & 0.45962e+03_rb,0.43150e+03_rb,0.50758e+03_rb,0.39413e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,36, 4) = (/ &
        & 0.45960e+03_rb,0.43148e+03_rb,0.50758e+03_rb,0.39413e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,36, 4) = (/ &
        & 0.45958e+03_rb,0.43147e+03_rb,0.50756e+03_rb,0.39412e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,36, 4) = (/ &
        & 0.45956e+03_rb,0.43146e+03_rb,0.50752e+03_rb,0.39412e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,37, 4) = (/ &
        & 0.45957e+03_rb,0.43146e+03_rb,0.50751e+03_rb,0.39412e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,37, 4) = (/ &
        & 0.45955e+03_rb,0.43145e+03_rb,0.50753e+03_rb,0.39412e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,37, 4) = (/ &
        & 0.45953e+03_rb,0.43144e+03_rb,0.50754e+03_rb,0.39411e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,37, 4) = (/ &
        & 0.45952e+03_rb,0.43142e+03_rb,0.50752e+03_rb,0.39411e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,37, 4) = (/ &
        & 0.45950e+03_rb,0.43141e+03_rb,0.50748e+03_rb,0.39411e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,38, 4) = (/ &
        & 0.45951e+03_rb,0.43142e+03_rb,0.50746e+03_rb,0.39411e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,38, 4) = (/ &
        & 0.45950e+03_rb,0.43141e+03_rb,0.50749e+03_rb,0.39410e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,38, 4) = (/ &
        & 0.45948e+03_rb,0.43140e+03_rb,0.50750e+03_rb,0.39410e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,38, 4) = (/ &
        & 0.45947e+03_rb,0.43139e+03_rb,0.50749e+03_rb,0.39410e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,38, 4) = (/ &
        & 0.45946e+03_rb,0.43138e+03_rb,0.50745e+03_rb,0.39409e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,39, 4) = (/ &
        & 0.45946e+03_rb,0.43138e+03_rb,0.50742e+03_rb,0.39410e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,39, 4) = (/ &
        & 0.45945e+03_rb,0.43137e+03_rb,0.50746e+03_rb,0.39409e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,39, 4) = (/ &
        & 0.45944e+03_rb,0.43137e+03_rb,0.50747e+03_rb,0.39409e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,39, 4) = (/ &
        & 0.45943e+03_rb,0.43136e+03_rb,0.50746e+03_rb,0.39409e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,39, 4) = (/ &
        & 0.45942e+03_rb,0.43135e+03_rb,0.50743e+03_rb,0.39409e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,40, 4) = (/ &
        & 0.45943e+03_rb,0.43136e+03_rb,0.50738e+03_rb,0.39409e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,40, 4) = (/ &
        & 0.45941e+03_rb,0.43135e+03_rb,0.50742e+03_rb,0.39408e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,40, 4) = (/ &
        & 0.45940e+03_rb,0.43134e+03_rb,0.50744e+03_rb,0.39408e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,40, 4) = (/ &
        & 0.45940e+03_rb,0.43133e+03_rb,0.50744e+03_rb,0.39408e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,40, 4) = (/ &
        & 0.45939e+03_rb,0.43133e+03_rb,0.50742e+03_rb,0.39408e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,41, 4) = (/ &
        & 0.45939e+03_rb,0.43133e+03_rb,0.50734e+03_rb,0.39408e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,41, 4) = (/ &
        & 0.45938e+03_rb,0.43133e+03_rb,0.50739e+03_rb,0.39408e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,41, 4) = (/ &
        & 0.45938e+03_rb,0.43132e+03_rb,0.50742e+03_rb,0.39408e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,41, 4) = (/ &
        & 0.45937e+03_rb,0.43131e+03_rb,0.50742e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,41, 4) = (/ &
        & 0.45936e+03_rb,0.43131e+03_rb,0.50741e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,42, 4) = (/ &
        & 0.45937e+03_rb,0.43131e+03_rb,0.50731e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,42, 4) = (/ &
        & 0.45936e+03_rb,0.43131e+03_rb,0.50736e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,42, 4) = (/ &
        & 0.45935e+03_rb,0.43130e+03_rb,0.50739e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,42, 4) = (/ &
        & 0.45935e+03_rb,0.43130e+03_rb,0.50741e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,42, 4) = (/ &
        & 0.45934e+03_rb,0.43129e+03_rb,0.50740e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,43, 4) = (/ &
        & 0.45935e+03_rb,0.43130e+03_rb,0.50727e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,43, 4) = (/ &
        & 0.45934e+03_rb,0.43129e+03_rb,0.50733e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,43, 4) = (/ &
        & 0.45933e+03_rb,0.43129e+03_rb,0.50737e+03_rb,0.39407e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,43, 4) = (/ &
        & 0.45933e+03_rb,0.43128e+03_rb,0.50739e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,43, 4) = (/ &
        & 0.45932e+03_rb,0.43128e+03_rb,0.50739e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,44, 4) = (/ &
        & 0.45933e+03_rb,0.43128e+03_rb,0.50723e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,44, 4) = (/ &
        & 0.45932e+03_rb,0.43128e+03_rb,0.50730e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,44, 4) = (/ &
        & 0.45932e+03_rb,0.43128e+03_rb,0.50735e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,44, 4) = (/ &
        & 0.45931e+03_rb,0.43127e+03_rb,0.50737e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,44, 4) = (/ &
        & 0.45931e+03_rb,0.43127e+03_rb,0.50738e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,45, 4) = (/ &
        & 0.45931e+03_rb,0.43127e+03_rb,0.50718e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,45, 4) = (/ &
        & 0.45931e+03_rb,0.43127e+03_rb,0.50727e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,45, 4) = (/ &
        & 0.45930e+03_rb,0.43127e+03_rb,0.50732e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,45, 4) = (/ &
        & 0.45930e+03_rb,0.43126e+03_rb,0.50736e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,45, 4) = (/ &
        & 0.45930e+03_rb,0.43126e+03_rb,0.50737e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,46, 4) = (/ &
        & 0.45930e+03_rb,0.43126e+03_rb,0.50714e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,46, 4) = (/ &
        & 0.45930e+03_rb,0.43126e+03_rb,0.50723e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,46, 4) = (/ &
        & 0.45929e+03_rb,0.43126e+03_rb,0.50730e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,46, 4) = (/ &
        & 0.45929e+03_rb,0.43125e+03_rb,0.50734e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,46, 4) = (/ &
        & 0.45929e+03_rb,0.43125e+03_rb,0.50736e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,47, 4) = (/ &
        & 0.45929e+03_rb,0.43126e+03_rb,0.50708e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,47, 4) = (/ &
        & 0.45929e+03_rb,0.43125e+03_rb,0.50719e+03_rb,0.39406e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,47, 4) = (/ &
        & 0.45928e+03_rb,0.43125e+03_rb,0.50727e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,47, 4) = (/ &
        & 0.45928e+03_rb,0.43125e+03_rb,0.50732e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,47, 4) = (/ &
        & 0.45928e+03_rb,0.43125e+03_rb,0.50735e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,48, 4) = (/ &
        & 0.45928e+03_rb,0.43125e+03_rb,0.50701e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,48, 4) = (/ &
        & 0.45928e+03_rb,0.43125e+03_rb,0.50714e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,48, 4) = (/ &
        & 0.45928e+03_rb,0.43124e+03_rb,0.50723e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,48, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50729e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,48, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50733e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,49, 4) = (/ &
        & 0.45928e+03_rb,0.43124e+03_rb,0.50694e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,49, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50709e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,49, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50720e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,49, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50727e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,49, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50732e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,50, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50686e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,50, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50704e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,50, 4) = (/ &
        & 0.45927e+03_rb,0.43124e+03_rb,0.50716e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,50, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50724e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,50, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50730e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,51, 4) = (/ &
        & 0.45926e+03_rb,0.43124e+03_rb,0.50679e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,51, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50697e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,51, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50711e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,51, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50721e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,51, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50728e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,52, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50671e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,52, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50691e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,52, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50707e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,52, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50718e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,52, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50725e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,53, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50663e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,53, 4) = (/ &
        & 0.45926e+03_rb,0.43123e+03_rb,0.50684e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,53, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50701e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,53, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50714e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,53, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50722e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,54, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50655e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,54, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50677e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,54, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50696e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,54, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50710e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,54, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50720e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,55, 4) = (/ &
        & 0.45925e+03_rb,0.43123e+03_rb,0.50647e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,55, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50670e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,55, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50690e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,55, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50706e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,55, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50717e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,56, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50640e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,56, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50663e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,56, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50683e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,56, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50701e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,56, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50714e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,57, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50632e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,57, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50656e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,57, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50677e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,57, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50696e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,57, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50710e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,58, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50624e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,58, 4) = (/ &
        & 0.45925e+03_rb,0.43122e+03_rb,0.50648e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,58, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50671e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,58, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50690e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,58, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50706e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,59, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50622e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 2,59, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50645e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 3,59, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50668e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 4,59, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50688e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 5,59, 4) = (/ &
        & 0.45924e+03_rb,0.43122e+03_rb,0.50704e+03_rb,0.39405e+03_rb,0.38437e+03_rb /)
      kbo(:, 1,13, 5) = (/ &
        & 0.78037e+03_rb,0.62743e+03_rb,0.56495e+03_rb,0.56633e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,13, 5) = (/ &
        & 0.77434e+03_rb,0.62291e+03_rb,0.56343e+03_rb,0.56584e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,13, 5) = (/ &
        & 0.76915e+03_rb,0.61902e+03_rb,0.56189e+03_rb,0.56542e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,13, 5) = (/ &
        & 0.76464e+03_rb,0.61563e+03_rb,0.56027e+03_rb,0.56506e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,13, 5) = (/ &
        & 0.76068e+03_rb,0.61266e+03_rb,0.55865e+03_rb,0.56474e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,14, 5) = (/ &
        & 0.76527e+03_rb,0.61611e+03_rb,0.56011e+03_rb,0.56511e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,14, 5) = (/ &
        & 0.76041e+03_rb,0.61246e+03_rb,0.55859e+03_rb,0.56472e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,14, 5) = (/ &
        & 0.75622e+03_rb,0.60932e+03_rb,0.55696e+03_rb,0.56438e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,14, 5) = (/ &
        & 0.75257e+03_rb,0.60658e+03_rb,0.55539e+03_rb,0.56408e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,14, 5) = (/ &
        & 0.74936e+03_rb,0.60417e+03_rb,0.55391e+03_rb,0.56383e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,15, 5) = (/ &
        & 0.75296e+03_rb,0.60688e+03_rb,0.55521e+03_rb,0.56412e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,15, 5) = (/ &
        & 0.74904e+03_rb,0.60394e+03_rb,0.55372e+03_rb,0.56380e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,15, 5) = (/ &
        & 0.74566e+03_rb,0.60140e+03_rb,0.55235e+03_rb,0.56353e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,15, 5) = (/ &
        & 0.74271e+03_rb,0.59919e+03_rb,0.55103e+03_rb,0.56329e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,15, 5) = (/ &
        & 0.74012e+03_rb,0.59724e+03_rb,0.54977e+03_rb,0.56308e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,16, 5) = (/ &
        & 0.74295e+03_rb,0.59937e+03_rb,0.55079e+03_rb,0.56331e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,16, 5) = (/ &
        & 0.73979e+03_rb,0.59700e+03_rb,0.54953e+03_rb,0.56305e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,16, 5) = (/ &
        & 0.73707e+03_rb,0.59495e+03_rb,0.54828e+03_rb,0.56283e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,16, 5) = (/ &
        & 0.73469e+03_rb,0.59317e+03_rb,0.54708e+03_rb,0.56264e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,16, 5) = (/ &
        & 0.73259e+03_rb,0.59160e+03_rb,0.54600e+03_rb,0.56247e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,17, 5) = (/ &
        & 0.73483e+03_rb,0.59328e+03_rb,0.54695e+03_rb,0.56265e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,17, 5) = (/ &
        & 0.73229e+03_rb,0.59137e+03_rb,0.54572e+03_rb,0.56245e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,17, 5) = (/ &
        & 0.73009e+03_rb,0.58972e+03_rb,0.54462e+03_rb,0.56227e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,17, 5) = (/ &
        & 0.72817e+03_rb,0.58828e+03_rb,0.54366e+03_rb,0.56212e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,17, 5) = (/ &
        & 0.72647e+03_rb,0.58701e+03_rb,0.54281e+03_rb,0.56198e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,18, 5) = (/ &
        & 0.72825e+03_rb,0.58834e+03_rb,0.54354e+03_rb,0.56212e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,18, 5) = (/ &
        & 0.72619e+03_rb,0.58680e+03_rb,0.54253e+03_rb,0.56196e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,18, 5) = (/ &
        & 0.72442e+03_rb,0.58547e+03_rb,0.54166e+03_rb,0.56181e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,18, 5) = (/ &
        & 0.72286e+03_rb,0.58430e+03_rb,0.54091e+03_rb,0.56169e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,18, 5) = (/ &
        & 0.72150e+03_rb,0.58328e+03_rb,0.54024e+03_rb,0.56158e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,19, 5) = (/ &
        & 0.72290e+03_rb,0.58433e+03_rb,0.54075e+03_rb,0.56169e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,19, 5) = (/ &
        & 0.72124e+03_rb,0.58309e+03_rb,0.53996e+03_rb,0.56156e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,19, 5) = (/ &
        & 0.71981e+03_rb,0.58201e+03_rb,0.53928e+03_rb,0.56144e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,19, 5) = (/ &
        & 0.71856e+03_rb,0.58107e+03_rb,0.53868e+03_rb,0.56134e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,19, 5) = (/ &
        & 0.71745e+03_rb,0.58024e+03_rb,0.53816e+03_rb,0.56125e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,20, 5) = (/ &
        & 0.71853e+03_rb,0.58105e+03_rb,0.53849e+03_rb,0.56134e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,20, 5) = (/ &
        & 0.71720e+03_rb,0.58006e+03_rb,0.53788e+03_rb,0.56123e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,20, 5) = (/ &
        & 0.71605e+03_rb,0.57919e+03_rb,0.53734e+03_rb,0.56114e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,20, 5) = (/ &
        & 0.71504e+03_rb,0.57843e+03_rb,0.53687e+03_rb,0.56106e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,20, 5) = (/ &
        & 0.71414e+03_rb,0.57776e+03_rb,0.53643e+03_rb,0.56098e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,21, 5) = (/ &
        & 0.71499e+03_rb,0.57840e+03_rb,0.53667e+03_rb,0.56105e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,21, 5) = (/ &
        & 0.71392e+03_rb,0.57759e+03_rb,0.53620e+03_rb,0.56097e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,21, 5) = (/ &
        & 0.71299e+03_rb,0.57690e+03_rb,0.53578e+03_rb,0.56089e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,21, 5) = (/ &
        & 0.71217e+03_rb,0.57628e+03_rb,0.53541e+03_rb,0.56083e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,21, 5) = (/ &
        & 0.71145e+03_rb,0.57574e+03_rb,0.53506e+03_rb,0.56077e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,22, 5) = (/ &
        & 0.71206e+03_rb,0.57620e+03_rb,0.53519e+03_rb,0.56082e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,22, 5) = (/ &
        & 0.71120e+03_rb,0.57555e+03_rb,0.53482e+03_rb,0.56075e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,22, 5) = (/ &
        & 0.71046e+03_rb,0.57500e+03_rb,0.53450e+03_rb,0.56069e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,22, 5) = (/ &
        & 0.70981e+03_rb,0.57451e+03_rb,0.53420e+03_rb,0.56064e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,22, 5) = (/ &
        & 0.70923e+03_rb,0.57408e+03_rb,0.53390e+03_rb,0.56059e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,23, 5) = (/ &
        & 0.70968e+03_rb,0.57442e+03_rb,0.53396e+03_rb,0.56063e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,23, 5) = (/ &
        & 0.70900e+03_rb,0.57391e+03_rb,0.53367e+03_rb,0.56057e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,23, 5) = (/ &
        & 0.70841e+03_rb,0.57346e+03_rb,0.53341e+03_rb,0.56052e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,23, 5) = (/ &
        & 0.70789e+03_rb,0.57307e+03_rb,0.53316e+03_rb,0.56048e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,23, 5) = (/ &
        & 0.70743e+03_rb,0.57272e+03_rb,0.53290e+03_rb,0.56044e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,24, 5) = (/ &
        & 0.70776e+03_rb,0.57298e+03_rb,0.53293e+03_rb,0.56047e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,24, 5) = (/ &
        & 0.70722e+03_rb,0.57257e+03_rb,0.53269e+03_rb,0.56043e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,24, 5) = (/ &
        & 0.70675e+03_rb,0.57222e+03_rb,0.53248e+03_rb,0.56039e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,24, 5) = (/ &
        & 0.70633e+03_rb,0.57190e+03_rb,0.53228e+03_rb,0.56036e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,24, 5) = (/ &
        & 0.70596e+03_rb,0.57163e+03_rb,0.53207e+03_rb,0.56033e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,25, 5) = (/ &
        & 0.70621e+03_rb,0.57181e+03_rb,0.53206e+03_rb,0.56035e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,25, 5) = (/ &
        & 0.70578e+03_rb,0.57149e+03_rb,0.53188e+03_rb,0.56031e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,25, 5) = (/ &
        & 0.70541e+03_rb,0.57121e+03_rb,0.53172e+03_rb,0.56028e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,25, 5) = (/ &
        & 0.70507e+03_rb,0.57096e+03_rb,0.53156e+03_rb,0.56025e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,25, 5) = (/ &
        & 0.70478e+03_rb,0.57074e+03_rb,0.53138e+03_rb,0.56023e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,26, 5) = (/ &
        & 0.70496e+03_rb,0.57087e+03_rb,0.53135e+03_rb,0.56024e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,26, 5) = (/ &
        & 0.70461e+03_rb,0.57061e+03_rb,0.53122e+03_rb,0.56022e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,26, 5) = (/ &
        & 0.70432e+03_rb,0.57039e+03_rb,0.53110e+03_rb,0.56019e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,26, 5) = (/ &
        & 0.70405e+03_rb,0.57019e+03_rb,0.53097e+03_rb,0.56017e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,26, 5) = (/ &
        & 0.70382e+03_rb,0.57001e+03_rb,0.53082e+03_rb,0.56015e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,27, 5) = (/ &
        & 0.70394e+03_rb,0.57011e+03_rb,0.53077e+03_rb,0.56016e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,27, 5) = (/ &
        & 0.70367e+03_rb,0.56991e+03_rb,0.53069e+03_rb,0.56014e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,27, 5) = (/ &
        & 0.70343e+03_rb,0.56973e+03_rb,0.53060e+03_rb,0.56012e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,27, 5) = (/ &
        & 0.70322e+03_rb,0.56957e+03_rb,0.53049e+03_rb,0.56010e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,27, 5) = (/ &
        & 0.70304e+03_rb,0.56943e+03_rb,0.53036e+03_rb,0.56009e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,28, 5) = (/ &
        & 0.70313e+03_rb,0.56950e+03_rb,0.53032e+03_rb,0.56010e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,28, 5) = (/ &
        & 0.70291e+03_rb,0.56934e+03_rb,0.53026e+03_rb,0.56008e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,28, 5) = (/ &
        & 0.70272e+03_rb,0.56920e+03_rb,0.53019e+03_rb,0.56006e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,28, 5) = (/ &
        & 0.70256e+03_rb,0.56907e+03_rb,0.53010e+03_rb,0.56005e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,28, 5) = (/ &
        & 0.70241e+03_rb,0.56896e+03_rb,0.52998e+03_rb,0.56004e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,29, 5) = (/ &
        & 0.70247e+03_rb,0.56901e+03_rb,0.52995e+03_rb,0.56004e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,29, 5) = (/ &
        & 0.70230e+03_rb,0.56888e+03_rb,0.52991e+03_rb,0.56003e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,29, 5) = (/ &
        & 0.70215e+03_rb,0.56877e+03_rb,0.52985e+03_rb,0.56002e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,29, 5) = (/ &
        & 0.70202e+03_rb,0.56867e+03_rb,0.52978e+03_rb,0.56001e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,29, 5) = (/ &
        & 0.70190e+03_rb,0.56858e+03_rb,0.52967e+03_rb,0.56000e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,30, 5) = (/ &
        & 0.70194e+03_rb,0.56861e+03_rb,0.52966e+03_rb,0.56000e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,30, 5) = (/ &
        & 0.70181e+03_rb,0.56851e+03_rb,0.52963e+03_rb,0.55999e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,30, 5) = (/ &
        & 0.70169e+03_rb,0.56842e+03_rb,0.52958e+03_rb,0.55998e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,30, 5) = (/ &
        & 0.70158e+03_rb,0.56834e+03_rb,0.52951e+03_rb,0.55997e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,30, 5) = (/ &
        & 0.70149e+03_rb,0.56827e+03_rb,0.52941e+03_rb,0.55996e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,31, 5) = (/ &
        & 0.70151e+03_rb,0.56829e+03_rb,0.52942e+03_rb,0.55997e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,31, 5) = (/ &
        & 0.70141e+03_rb,0.56821e+03_rb,0.52941e+03_rb,0.55996e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,31, 5) = (/ &
        & 0.70131e+03_rb,0.56814e+03_rb,0.52936e+03_rb,0.55995e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,31, 5) = (/ &
        & 0.70123e+03_rb,0.56807e+03_rb,0.52930e+03_rb,0.55994e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,31, 5) = (/ &
        & 0.70115e+03_rb,0.56802e+03_rb,0.52920e+03_rb,0.55994e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,32, 5) = (/ &
        & 0.70117e+03_rb,0.56803e+03_rb,0.52924e+03_rb,0.55994e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,32, 5) = (/ &
        & 0.70109e+03_rb,0.56797e+03_rb,0.52922e+03_rb,0.55993e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,32, 5) = (/ &
        & 0.70101e+03_rb,0.56791e+03_rb,0.52919e+03_rb,0.55993e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,32, 5) = (/ &
        & 0.70094e+03_rb,0.56786e+03_rb,0.52912e+03_rb,0.55992e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,32, 5) = (/ &
        & 0.70088e+03_rb,0.56782e+03_rb,0.52902e+03_rb,0.55992e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,33, 5) = (/ &
        & 0.70090e+03_rb,0.56782e+03_rb,0.52908e+03_rb,0.55992e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,33, 5) = (/ &
        & 0.70083e+03_rb,0.56777e+03_rb,0.52907e+03_rb,0.55991e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,33, 5) = (/ &
        & 0.70077e+03_rb,0.56773e+03_rb,0.52904e+03_rb,0.55991e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,33, 5) = (/ &
        & 0.70072e+03_rb,0.56769e+03_rb,0.52897e+03_rb,0.55990e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,33, 5) = (/ &
        & 0.70067e+03_rb,0.56765e+03_rb,0.52887e+03_rb,0.55990e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,34, 5) = (/ &
        & 0.70068e+03_rb,0.56766e+03_rb,0.52896e+03_rb,0.55990e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,34, 5) = (/ &
        & 0.70062e+03_rb,0.56762e+03_rb,0.52895e+03_rb,0.55989e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,34, 5) = (/ &
        & 0.70057e+03_rb,0.56758e+03_rb,0.52892e+03_rb,0.55989e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,34, 5) = (/ &
        & 0.70053e+03_rb,0.56755e+03_rb,0.52885e+03_rb,0.55989e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,34, 5) = (/ &
        & 0.70049e+03_rb,0.56752e+03_rb,0.52874e+03_rb,0.55988e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,35, 5) = (/ &
        & 0.70050e+03_rb,0.56753e+03_rb,0.52886e+03_rb,0.55988e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,35, 5) = (/ &
        & 0.70046e+03_rb,0.56750e+03_rb,0.52885e+03_rb,0.55988e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,35, 5) = (/ &
        & 0.70042e+03_rb,0.56747e+03_rb,0.52883e+03_rb,0.55988e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,35, 5) = (/ &
        & 0.70039e+03_rb,0.56744e+03_rb,0.52876e+03_rb,0.55988e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,35, 5) = (/ &
        & 0.70035e+03_rb,0.56742e+03_rb,0.52864e+03_rb,0.55987e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,36, 5) = (/ &
        & 0.70037e+03_rb,0.56743e+03_rb,0.52877e+03_rb,0.55987e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,36, 5) = (/ &
        & 0.70033e+03_rb,0.56740e+03_rb,0.52878e+03_rb,0.55987e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,36, 5) = (/ &
        & 0.70030e+03_rb,0.56738e+03_rb,0.52875e+03_rb,0.55987e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,36, 5) = (/ &
        & 0.70027e+03_rb,0.56736e+03_rb,0.52869e+03_rb,0.55987e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,36, 5) = (/ &
        & 0.70025e+03_rb,0.56734e+03_rb,0.52857e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,37, 5) = (/ &
        & 0.70026e+03_rb,0.56735e+03_rb,0.52870e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,37, 5) = (/ &
        & 0.70023e+03_rb,0.56733e+03_rb,0.52872e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,37, 5) = (/ &
        & 0.70020e+03_rb,0.56731e+03_rb,0.52870e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,37, 5) = (/ &
        & 0.70018e+03_rb,0.56729e+03_rb,0.52864e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,37, 5) = (/ &
        & 0.70016e+03_rb,0.56727e+03_rb,0.52854e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,38, 5) = (/ &
        & 0.70017e+03_rb,0.56728e+03_rb,0.52864e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,38, 5) = (/ &
        & 0.70015e+03_rb,0.56726e+03_rb,0.52866e+03_rb,0.55986e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,38, 5) = (/ &
        & 0.70012e+03_rb,0.56725e+03_rb,0.52866e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,38, 5) = (/ &
        & 0.70010e+03_rb,0.56723e+03_rb,0.52860e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,38, 5) = (/ &
        & 0.70009e+03_rb,0.56722e+03_rb,0.52852e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,39, 5) = (/ &
        & 0.70010e+03_rb,0.56723e+03_rb,0.52860e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,39, 5) = (/ &
        & 0.70008e+03_rb,0.56721e+03_rb,0.52862e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,39, 5) = (/ &
        & 0.70006e+03_rb,0.56720e+03_rb,0.52861e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,39, 5) = (/ &
        & 0.70004e+03_rb,0.56719e+03_rb,0.52858e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,39, 5) = (/ &
        & 0.70003e+03_rb,0.56717e+03_rb,0.52850e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,40, 5) = (/ &
        & 0.70004e+03_rb,0.56718e+03_rb,0.52855e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,40, 5) = (/ &
        & 0.70002e+03_rb,0.56717e+03_rb,0.52858e+03_rb,0.55985e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,40, 5) = (/ &
        & 0.70001e+03_rb,0.56716e+03_rb,0.52859e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,40, 5) = (/ &
        & 0.69999e+03_rb,0.56715e+03_rb,0.52856e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,40, 5) = (/ &
        & 0.69998e+03_rb,0.56714e+03_rb,0.52849e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,41, 5) = (/ &
        & 0.69999e+03_rb,0.56715e+03_rb,0.52850e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,41, 5) = (/ &
        & 0.69998e+03_rb,0.56714e+03_rb,0.52854e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,41, 5) = (/ &
        & 0.69996e+03_rb,0.56713e+03_rb,0.52856e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,41, 5) = (/ &
        & 0.69995e+03_rb,0.56712e+03_rb,0.52854e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,41, 5) = (/ &
        & 0.69994e+03_rb,0.56711e+03_rb,0.52848e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,42, 5) = (/ &
        & 0.69995e+03_rb,0.56712e+03_rb,0.52846e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,42, 5) = (/ &
        & 0.69994e+03_rb,0.56711e+03_rb,0.52851e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,42, 5) = (/ &
        & 0.69993e+03_rb,0.56710e+03_rb,0.52853e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,42, 5) = (/ &
        & 0.69992e+03_rb,0.56709e+03_rb,0.52853e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,42, 5) = (/ &
        & 0.69991e+03_rb,0.56709e+03_rb,0.52848e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,43, 5) = (/ &
        & 0.69992e+03_rb,0.56709e+03_rb,0.52843e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,43, 5) = (/ &
        & 0.69991e+03_rb,0.56708e+03_rb,0.52848e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,43, 5) = (/ &
        & 0.69990e+03_rb,0.56708e+03_rb,0.52851e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,43, 5) = (/ &
        & 0.69989e+03_rb,0.56707e+03_rb,0.52851e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,43, 5) = (/ &
        & 0.69988e+03_rb,0.56707e+03_rb,0.52848e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,44, 5) = (/ &
        & 0.69989e+03_rb,0.56707e+03_rb,0.52838e+03_rb,0.55984e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,44, 5) = (/ &
        & 0.69988e+03_rb,0.56707e+03_rb,0.52844e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,44, 5) = (/ &
        & 0.69988e+03_rb,0.56706e+03_rb,0.52848e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,44, 5) = (/ &
        & 0.69987e+03_rb,0.56705e+03_rb,0.52850e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,44, 5) = (/ &
        & 0.69986e+03_rb,0.56705e+03_rb,0.52849e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,45, 5) = (/ &
        & 0.69987e+03_rb,0.56706e+03_rb,0.52834e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,45, 5) = (/ &
        & 0.69986e+03_rb,0.56705e+03_rb,0.52841e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,45, 5) = (/ &
        & 0.69986e+03_rb,0.56704e+03_rb,0.52846e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,45, 5) = (/ &
        & 0.69985e+03_rb,0.56704e+03_rb,0.52848e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,45, 5) = (/ &
        & 0.69984e+03_rb,0.56704e+03_rb,0.52848e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,46, 5) = (/ &
        & 0.69985e+03_rb,0.56704e+03_rb,0.52830e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,46, 5) = (/ &
        & 0.69984e+03_rb,0.56704e+03_rb,0.52837e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,46, 5) = (/ &
        & 0.69984e+03_rb,0.56703e+03_rb,0.52843e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,46, 5) = (/ &
        & 0.69983e+03_rb,0.56703e+03_rb,0.52847e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,46, 5) = (/ &
        & 0.69983e+03_rb,0.56703e+03_rb,0.52848e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,47, 5) = (/ &
        & 0.69984e+03_rb,0.56703e+03_rb,0.52826e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,47, 5) = (/ &
        & 0.69983e+03_rb,0.56703e+03_rb,0.52834e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,47, 5) = (/ &
        & 0.69982e+03_rb,0.56702e+03_rb,0.52840e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,47, 5) = (/ &
        & 0.69982e+03_rb,0.56702e+03_rb,0.52845e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,47, 5) = (/ &
        & 0.69982e+03_rb,0.56702e+03_rb,0.52847e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,48, 5) = (/ &
        & 0.69982e+03_rb,0.56702e+03_rb,0.52821e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,48, 5) = (/ &
        & 0.69982e+03_rb,0.56702e+03_rb,0.52830e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,48, 5) = (/ &
        & 0.69981e+03_rb,0.56701e+03_rb,0.52837e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,48, 5) = (/ &
        & 0.69981e+03_rb,0.56701e+03_rb,0.52842e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,48, 5) = (/ &
        & 0.69981e+03_rb,0.56701e+03_rb,0.52845e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,49, 5) = (/ &
        & 0.69981e+03_rb,0.56701e+03_rb,0.52817e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,49, 5) = (/ &
        & 0.69981e+03_rb,0.56701e+03_rb,0.52825e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,49, 5) = (/ &
        & 0.69980e+03_rb,0.56701e+03_rb,0.52833e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,49, 5) = (/ &
        & 0.69980e+03_rb,0.56700e+03_rb,0.52840e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,49, 5) = (/ &
        & 0.69980e+03_rb,0.56700e+03_rb,0.52844e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,50, 5) = (/ &
        & 0.69980e+03_rb,0.56701e+03_rb,0.52812e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,50, 5) = (/ &
        & 0.69980e+03_rb,0.56700e+03_rb,0.52821e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,50, 5) = (/ &
        & 0.69980e+03_rb,0.56700e+03_rb,0.52830e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,50, 5) = (/ &
        & 0.69979e+03_rb,0.56700e+03_rb,0.52837e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,50, 5) = (/ &
        & 0.69979e+03_rb,0.56700e+03_rb,0.52842e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,51, 5) = (/ &
        & 0.69979e+03_rb,0.56700e+03_rb,0.52807e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,51, 5) = (/ &
        & 0.69979e+03_rb,0.56700e+03_rb,0.52818e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,51, 5) = (/ &
        & 0.69979e+03_rb,0.56699e+03_rb,0.52827e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,51, 5) = (/ &
        & 0.69979e+03_rb,0.56699e+03_rb,0.52834e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,51, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52841e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,52, 5) = (/ &
        & 0.69979e+03_rb,0.56699e+03_rb,0.52802e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,52, 5) = (/ &
        & 0.69979e+03_rb,0.56699e+03_rb,0.52814e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,52, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52823e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,52, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52831e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,52, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52838e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,53, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52798e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,53, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52809e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,53, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52820e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,53, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52828e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,53, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52836e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,54, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52794e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,54, 5) = (/ &
        & 0.69978e+03_rb,0.56699e+03_rb,0.52804e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,54, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52816e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,54, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52825e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,54, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52833e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,55, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52790e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,55, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52801e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,55, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52812e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,55, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52822e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,55, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52830e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,56, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52786e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,56, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52797e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,56, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52808e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,56, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52819e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,56, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52827e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,57, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52781e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,57, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52794e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,57, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52805e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,57, 5) = (/ &
        & 0.69976e+03_rb,0.56698e+03_rb,0.52815e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,57, 5) = (/ &
        & 0.69976e+03_rb,0.56698e+03_rb,0.52825e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,58, 5) = (/ &
        & 0.69977e+03_rb,0.56698e+03_rb,0.52777e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,58, 5) = (/ &
        & 0.69976e+03_rb,0.56698e+03_rb,0.52790e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,58, 5) = (/ &
        & 0.69976e+03_rb,0.56698e+03_rb,0.52801e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,58, 5) = (/ &
        & 0.69976e+03_rb,0.56698e+03_rb,0.52812e+03_rb,0.55982e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,58, 5) = (/ &
        & 0.69976e+03_rb,0.56697e+03_rb,0.52822e+03_rb,0.55982e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,59, 5) = (/ &
        & 0.69976e+03_rb,0.56698e+03_rb,0.52775e+03_rb,0.55983e+03_rb,0.67117e+03_rb /)
      kbo(:, 2,59, 5) = (/ &
        & 0.69976e+03_rb,0.56698e+03_rb,0.52788e+03_rb,0.55982e+03_rb,0.67117e+03_rb /)
      kbo(:, 3,59, 5) = (/ &
        & 0.69976e+03_rb,0.56697e+03_rb,0.52801e+03_rb,0.55982e+03_rb,0.67117e+03_rb /)
      kbo(:, 4,59, 5) = (/ &
        & 0.69976e+03_rb,0.56697e+03_rb,0.52811e+03_rb,0.55982e+03_rb,0.67117e+03_rb /)
      kbo(:, 5,59, 5) = (/ &
        & 0.69976e+03_rb,0.56697e+03_rb,0.52821e+03_rb,0.55982e+03_rb,0.67117e+03_rb /)
      kbo(:, 1,13, 6) = (/ &
        & 0.99182e+03_rb,0.76479e+03_rb,0.58123e+03_rb,0.73928e+03_rb,0.94808e+03_rb /)
      kbo(:, 2,13, 6) = (/ &
        & 0.98417e+03_rb,0.75904e+03_rb,0.57888e+03_rb,0.73991e+03_rb,0.94920e+03_rb /)
      kbo(:, 3,13, 6) = (/ &
        & 0.97757e+03_rb,0.75410e+03_rb,0.57650e+03_rb,0.74029e+03_rb,0.94995e+03_rb /)
      kbo(:, 4,13, 6) = (/ &
        & 0.97184e+03_rb,0.74980e+03_rb,0.57428e+03_rb,0.74046e+03_rb,0.95033e+03_rb /)
      kbo(:, 5,13, 6) = (/ &
        & 0.96680e+03_rb,0.74602e+03_rb,0.57217e+03_rb,0.74043e+03_rb,0.95033e+03_rb /)
      kbo(:, 1,14, 6) = (/ &
        & 0.97264e+03_rb,0.75040e+03_rb,0.57482e+03_rb,0.73884e+03_rb,0.94820e+03_rb /)
      kbo(:, 2,14, 6) = (/ &
        & 0.96646e+03_rb,0.74576e+03_rb,0.57229e+03_rb,0.73949e+03_rb,0.94928e+03_rb /)
      kbo(:, 3,14, 6) = (/ &
        & 0.96113e+03_rb,0.74177e+03_rb,0.57016e+03_rb,0.73988e+03_rb,0.95000e+03_rb /)
      kbo(:, 4,14, 6) = (/ &
        & 0.95649e+03_rb,0.73829e+03_rb,0.56822e+03_rb,0.74006e+03_rb,0.95035e+03_rb /)
      kbo(:, 5,14, 6) = (/ &
        & 0.95242e+03_rb,0.73523e+03_rb,0.56635e+03_rb,0.74003e+03_rb,0.95032e+03_rb /)
      kbo(:, 1,15, 6) = (/ &
        & 0.95700e+03_rb,0.73867e+03_rb,0.56844e+03_rb,0.73851e+03_rb,0.94832e+03_rb /)
      kbo(:, 2,15, 6) = (/ &
        & 0.95201e+03_rb,0.73493e+03_rb,0.56632e+03_rb,0.73916e+03_rb,0.94937e+03_rb /)
      kbo(:, 3,15, 6) = (/ &
        & 0.94772e+03_rb,0.73170e+03_rb,0.56426e+03_rb,0.73956e+03_rb,0.95005e+03_rb /)
      kbo(:, 4,15, 6) = (/ &
        & 0.94397e+03_rb,0.72889e+03_rb,0.56242e+03_rb,0.73973e+03_rb,0.95035e+03_rb /)
      kbo(:, 5,15, 6) = (/ &
        & 0.94067e+03_rb,0.72642e+03_rb,0.56076e+03_rb,0.73970e+03_rb,0.95029e+03_rb /)
      kbo(:, 1,16, 6) = (/ &
        & 0.94427e+03_rb,0.72912e+03_rb,0.56257e+03_rb,0.73826e+03_rb,0.94846e+03_rb /)
      kbo(:, 2,16, 6) = (/ &
        & 0.94026e+03_rb,0.72611e+03_rb,0.56064e+03_rb,0.73891e+03_rb,0.94946e+03_rb /)
      kbo(:, 3,16, 6) = (/ &
        & 0.93679e+03_rb,0.72351e+03_rb,0.55906e+03_rb,0.73929e+03_rb,0.95011e+03_rb /)
      kbo(:, 4,16, 6) = (/ &
        & 0.93377e+03_rb,0.72124e+03_rb,0.55770e+03_rb,0.73948e+03_rb,0.95036e+03_rb /)
      kbo(:, 5,16, 6) = (/ &
        & 0.93110e+03_rb,0.71925e+03_rb,0.55641e+03_rb,0.73943e+03_rb,0.95027e+03_rb /)
      kbo(:, 1,17, 6) = (/ &
        & 0.93395e+03_rb,0.72138e+03_rb,0.55755e+03_rb,0.73807e+03_rb,0.94857e+03_rb /)
      kbo(:, 2,17, 6) = (/ &
        & 0.93072e+03_rb,0.71896e+03_rb,0.55622e+03_rb,0.73871e+03_rb,0.94955e+03_rb /)
      kbo(:, 3,17, 6) = (/ &
        & 0.92792e+03_rb,0.71686e+03_rb,0.55497e+03_rb,0.73909e+03_rb,0.95015e+03_rb /)
      kbo(:, 4,17, 6) = (/ &
        & 0.92548e+03_rb,0.71503e+03_rb,0.55372e+03_rb,0.73925e+03_rb,0.95038e+03_rb /)
      kbo(:, 5,17, 6) = (/ &
        & 0.92333e+03_rb,0.71341e+03_rb,0.55252e+03_rb,0.73921e+03_rb,0.95024e+03_rb /)
      kbo(:, 1,18, 6) = (/ &
        & 0.92558e+03_rb,0.71510e+03_rb,0.55360e+03_rb,0.73792e+03_rb,0.94869e+03_rb /)
      kbo(:, 2,18, 6) = (/ &
        & 0.92297e+03_rb,0.71315e+03_rb,0.55237e+03_rb,0.73856e+03_rb,0.94963e+03_rb /)
      kbo(:, 3,18, 6) = (/ &
        & 0.92072e+03_rb,0.71146e+03_rb,0.55114e+03_rb,0.73892e+03_rb,0.95019e+03_rb /)
      kbo(:, 4,18, 6) = (/ &
        & 0.91874e+03_rb,0.70998e+03_rb,0.55001e+03_rb,0.73908e+03_rb,0.95037e+03_rb /)
      kbo(:, 5,18, 6) = (/ &
        & 0.91700e+03_rb,0.70867e+03_rb,0.54903e+03_rb,0.73903e+03_rb,0.95021e+03_rb /)
      kbo(:, 1,19, 6) = (/ &
        & 0.91879e+03_rb,0.71001e+03_rb,0.55004e+03_rb,0.73782e+03_rb,0.94880e+03_rb /)
      kbo(:, 2,19, 6) = (/ &
        & 0.91668e+03_rb,0.70843e+03_rb,0.54889e+03_rb,0.73844e+03_rb,0.94970e+03_rb /)
      kbo(:, 3,19, 6) = (/ &
        & 0.91486e+03_rb,0.70706e+03_rb,0.54830e+03_rb,0.73880e+03_rb,0.95023e+03_rb /)
      kbo(:, 4,19, 6) = (/ &
        & 0.91327e+03_rb,0.70587e+03_rb,0.54705e+03_rb,0.73894e+03_rb,0.95037e+03_rb /)
      kbo(:, 5,19, 6) = (/ &
        & 0.91186e+03_rb,0.70481e+03_rb,0.54615e+03_rb,0.73887e+03_rb,0.95019e+03_rb /)
      kbo(:, 1,20, 6) = (/ &
        & 0.91324e+03_rb,0.70585e+03_rb,0.54718e+03_rb,0.73776e+03_rb,0.94893e+03_rb /)
      kbo(:, 2,20, 6) = (/ &
        & 0.91155e+03_rb,0.70458e+03_rb,0.54599e+03_rb,0.73835e+03_rb,0.94979e+03_rb /)
      kbo(:, 3,20, 6) = (/ &
        & 0.91008e+03_rb,0.70348e+03_rb,0.54519e+03_rb,0.73870e+03_rb,0.95026e+03_rb /)
      kbo(:, 4,20, 6) = (/ &
        & 0.90879e+03_rb,0.70251e+03_rb,0.54450e+03_rb,0.73882e+03_rb,0.95036e+03_rb /)
      kbo(:, 5,20, 6) = (/ &
        & 0.90766e+03_rb,0.70166e+03_rb,0.54390e+03_rb,0.73874e+03_rb,0.95013e+03_rb /)
      kbo(:, 1,21, 6) = (/ &
        & 0.90874e+03_rb,0.70247e+03_rb,0.54444e+03_rb,0.73773e+03_rb,0.94905e+03_rb /)
      kbo(:, 2,21, 6) = (/ &
        & 0.90737e+03_rb,0.70145e+03_rb,0.54370e+03_rb,0.73831e+03_rb,0.94986e+03_rb /)
      kbo(:, 3,21, 6) = (/ &
        & 0.90619e+03_rb,0.70056e+03_rb,0.54308e+03_rb,0.73862e+03_rb,0.95029e+03_rb /)
      kbo(:, 4,21, 6) = (/ &
        & 0.90515e+03_rb,0.69978e+03_rb,0.54255e+03_rb,0.73872e+03_rb,0.95035e+03_rb /)
      kbo(:, 5,21, 6) = (/ &
        & 0.90424e+03_rb,0.69910e+03_rb,0.54208e+03_rb,0.73862e+03_rb,0.95008e+03_rb /)
      kbo(:, 1,22, 6) = (/ &
        & 0.90500e+03_rb,0.69967e+03_rb,0.54240e+03_rb,0.73776e+03_rb,0.94923e+03_rb /)
      kbo(:, 2,22, 6) = (/ &
        & 0.90392e+03_rb,0.69886e+03_rb,0.54185e+03_rb,0.73829e+03_rb,0.94997e+03_rb /)
      kbo(:, 3,22, 6) = (/ &
        & 0.90297e+03_rb,0.69815e+03_rb,0.54138e+03_rb,0.73857e+03_rb,0.95033e+03_rb /)
      kbo(:, 4,22, 6) = (/ &
        & 0.90215e+03_rb,0.69753e+03_rb,0.54099e+03_rb,0.73864e+03_rb,0.95033e+03_rb /)
      kbo(:, 5,22, 6) = (/ &
        & 0.90141e+03_rb,0.69698e+03_rb,0.54066e+03_rb,0.73851e+03_rb,0.94999e+03_rb /)
      kbo(:, 1,23, 6) = (/ &
        & 0.90199e+03_rb,0.69741e+03_rb,0.54083e+03_rb,0.73780e+03_rb,0.94940e+03_rb /)
      kbo(:, 2,23, 6) = (/ &
        & 0.90112e+03_rb,0.69676e+03_rb,0.54044e+03_rb,0.73828e+03_rb,0.95007e+03_rb /)
      kbo(:, 3,23, 6) = (/ &
        & 0.90037e+03_rb,0.69620e+03_rb,0.54010e+03_rb,0.73854e+03_rb,0.95036e+03_rb /)
      kbo(:, 4,23, 6) = (/ &
        & 0.89971e+03_rb,0.69570e+03_rb,0.53983e+03_rb,0.73858e+03_rb,0.95029e+03_rb /)
      kbo(:, 5,23, 6) = (/ &
        & 0.89912e+03_rb,0.69526e+03_rb,0.53959e+03_rb,0.73840e+03_rb,0.94989e+03_rb /)
      kbo(:, 1,24, 6) = (/ &
        & 0.89955e+03_rb,0.69558e+03_rb,0.53965e+03_rb,0.73787e+03_rb,0.94957e+03_rb /)
      kbo(:, 2,24, 6) = (/ &
        & 0.89886e+03_rb,0.69507e+03_rb,0.53937e+03_rb,0.73829e+03_rb,0.95016e+03_rb /)
      kbo(:, 3,24, 6) = (/ &
        & 0.89826e+03_rb,0.69461e+03_rb,0.53914e+03_rb,0.73851e+03_rb,0.95038e+03_rb /)
      kbo(:, 4,24, 6) = (/ &
        & 0.89773e+03_rb,0.69422e+03_rb,0.53895e+03_rb,0.73850e+03_rb,0.95023e+03_rb /)
      kbo(:, 5,24, 6) = (/ &
        & 0.89726e+03_rb,0.69387e+03_rb,0.53876e+03_rb,0.73829e+03_rb,0.94977e+03_rb /)
      kbo(:, 1,25, 6) = (/ &
        & 0.89758e+03_rb,0.69410e+03_rb,0.53876e+03_rb,0.73792e+03_rb,0.94973e+03_rb /)
      kbo(:, 2,25, 6) = (/ &
        & 0.89703e+03_rb,0.69369e+03_rb,0.53857e+03_rb,0.73831e+03_rb,0.95024e+03_rb /)
      kbo(:, 3,25, 6) = (/ &
        & 0.89655e+03_rb,0.69333e+03_rb,0.53841e+03_rb,0.73848e+03_rb,0.95037e+03_rb /)
      kbo(:, 4,25, 6) = (/ &
        & 0.89613e+03_rb,0.69302e+03_rb,0.53826e+03_rb,0.73843e+03_rb,0.95016e+03_rb /)
      kbo(:, 5,25, 6) = (/ &
        & 0.89576e+03_rb,0.69273e+03_rb,0.53811e+03_rb,0.73818e+03_rb,0.94963e+03_rb /)
      kbo(:, 1,26, 6) = (/ &
        & 0.89598e+03_rb,0.69290e+03_rb,0.53808e+03_rb,0.73799e+03_rb,0.94987e+03_rb /)
      kbo(:, 2,26, 6) = (/ &
        & 0.89555e+03_rb,0.69258e+03_rb,0.53794e+03_rb,0.73834e+03_rb,0.95030e+03_rb /)
      kbo(:, 3,26, 6) = (/ &
        & 0.89517e+03_rb,0.69229e+03_rb,0.53782e+03_rb,0.73845e+03_rb,0.95035e+03_rb /)
      kbo(:, 4,26, 6) = (/ &
        & 0.89483e+03_rb,0.69204e+03_rb,0.53770e+03_rb,0.73836e+03_rb,0.95007e+03_rb /)
      kbo(:, 5,26, 6) = (/ &
        & 0.89453e+03_rb,0.69182e+03_rb,0.53759e+03_rb,0.73806e+03_rb,0.94946e+03_rb /)
      kbo(:, 1,27, 6) = (/ &
        & 0.89469e+03_rb,0.69194e+03_rb,0.53754e+03_rb,0.73806e+03_rb,0.95001e+03_rb /)
      kbo(:, 2,27, 6) = (/ &
        & 0.89435e+03_rb,0.69168e+03_rb,0.53744e+03_rb,0.73835e+03_rb,0.95034e+03_rb /)
      kbo(:, 3,27, 6) = (/ &
        & 0.89405e+03_rb,0.69145e+03_rb,0.53736e+03_rb,0.73843e+03_rb,0.95031e+03_rb /)
      kbo(:, 4,27, 6) = (/ &
        & 0.89378e+03_rb,0.69125e+03_rb,0.53727e+03_rb,0.73828e+03_rb,0.94995e+03_rb /)
      kbo(:, 5,27, 6) = (/ &
        & 0.89354e+03_rb,0.69107e+03_rb,0.53717e+03_rb,0.73792e+03_rb,0.94926e+03_rb /)
      kbo(:, 1,28, 6) = (/ &
        & 0.89366e+03_rb,0.69116e+03_rb,0.53711e+03_rb,0.73814e+03_rb,0.95012e+03_rb /)
      kbo(:, 2,28, 6) = (/ &
        & 0.89338e+03_rb,0.69096e+03_rb,0.53705e+03_rb,0.73836e+03_rb,0.95037e+03_rb /)
      kbo(:, 3,28, 6) = (/ &
        & 0.89314e+03_rb,0.69078e+03_rb,0.53699e+03_rb,0.73838e+03_rb,0.95026e+03_rb /)
      kbo(:, 4,28, 6) = (/ &
        & 0.89293e+03_rb,0.69062e+03_rb,0.53692e+03_rb,0.73819e+03_rb,0.94982e+03_rb /)
      kbo(:, 5,28, 6) = (/ &
        & 0.89274e+03_rb,0.69047e+03_rb,0.53684e+03_rb,0.73778e+03_rb,0.94904e+03_rb /)
      kbo(:, 1,29, 6) = (/ &
        & 0.89282e+03_rb,0.69053e+03_rb,0.53678e+03_rb,0.73818e+03_rb,0.95022e+03_rb /)
      kbo(:, 2,29, 6) = (/ &
        & 0.89261e+03_rb,0.69037e+03_rb,0.53674e+03_rb,0.73837e+03_rb,0.95037e+03_rb /)
      kbo(:, 3,29, 6) = (/ &
        & 0.89241e+03_rb,0.69023e+03_rb,0.53670e+03_rb,0.73834e+03_rb,0.95018e+03_rb /)
      kbo(:, 4,29, 6) = (/ &
        & 0.89224e+03_rb,0.69010e+03_rb,0.53663e+03_rb,0.73810e+03_rb,0.94966e+03_rb /)
      kbo(:, 5,29, 6) = (/ &
        & 0.89209e+03_rb,0.68999e+03_rb,0.53656e+03_rb,0.73762e+03_rb,0.94880e+03_rb /)
      kbo(:, 1,30, 6) = (/ &
        & 0.89215e+03_rb,0.69003e+03_rb,0.53652e+03_rb,0.73823e+03_rb,0.95029e+03_rb /)
      kbo(:, 2,30, 6) = (/ &
        & 0.89198e+03_rb,0.68990e+03_rb,0.53650e+03_rb,0.73836e+03_rb,0.95035e+03_rb /)
      kbo(:, 3,30, 6) = (/ &
        & 0.89183e+03_rb,0.68979e+03_rb,0.53646e+03_rb,0.73828e+03_rb,0.95008e+03_rb /)
      kbo(:, 4,30, 6) = (/ &
        & 0.89169e+03_rb,0.68969e+03_rb,0.53641e+03_rb,0.73798e+03_rb,0.94948e+03_rb /)
      kbo(:, 5,30, 6) = (/ &
        & 0.89157e+03_rb,0.68960e+03_rb,0.53634e+03_rb,0.73744e+03_rb,0.94852e+03_rb /)
      kbo(:, 1,31, 6) = (/ &
        & 0.89161e+03_rb,0.68962e+03_rb,0.53631e+03_rb,0.73827e+03_rb,0.95034e+03_rb /)
      kbo(:, 2,31, 6) = (/ &
        & 0.89147e+03_rb,0.68952e+03_rb,0.53629e+03_rb,0.73835e+03_rb,0.95031e+03_rb /)
      kbo(:, 3,31, 6) = (/ &
        & 0.89135e+03_rb,0.68943e+03_rb,0.53627e+03_rb,0.73821e+03_rb,0.94996e+03_rb /)
      kbo(:, 4,31, 6) = (/ &
        & 0.89124e+03_rb,0.68935e+03_rb,0.53622e+03_rb,0.73786e+03_rb,0.94927e+03_rb /)
      kbo(:, 5,31, 6) = (/ &
        & 0.89115e+03_rb,0.68928e+03_rb,0.53615e+03_rb,0.73725e+03_rb,0.94822e+03_rb /)
      kbo(:, 1,32, 6) = (/ &
        & 0.89117e+03_rb,0.68930e+03_rb,0.53614e+03_rb,0.73830e+03_rb,0.95037e+03_rb /)
      kbo(:, 2,32, 6) = (/ &
        & 0.89106e+03_rb,0.68922e+03_rb,0.53614e+03_rb,0.73833e+03_rb,0.95025e+03_rb /)
      kbo(:, 3,32, 6) = (/ &
        & 0.89097e+03_rb,0.68914e+03_rb,0.53611e+03_rb,0.73813e+03_rb,0.94981e+03_rb /)
      kbo(:, 4,32, 6) = (/ &
        & 0.89088e+03_rb,0.68908e+03_rb,0.53608e+03_rb,0.73772e+03_rb,0.94903e+03_rb /)
      kbo(:, 5,32, 6) = (/ &
        & 0.89081e+03_rb,0.68902e+03_rb,0.53600e+03_rb,0.73703e+03_rb,0.94789e+03_rb /)
      kbo(:, 1,33, 6) = (/ &
        & 0.89082e+03_rb,0.68903e+03_rb,0.53602e+03_rb,0.73832e+03_rb,0.95037e+03_rb /)
      kbo(:, 2,33, 6) = (/ &
        & 0.89073e+03_rb,0.68897e+03_rb,0.53601e+03_rb,0.73829e+03_rb,0.95017e+03_rb /)
      kbo(:, 3,33, 6) = (/ &
        & 0.89066e+03_rb,0.68891e+03_rb,0.53599e+03_rb,0.73804e+03_rb,0.94964e+03_rb /)
      kbo(:, 4,33, 6) = (/ &
        & 0.89059e+03_rb,0.68886e+03_rb,0.53595e+03_rb,0.73756e+03_rb,0.94877e+03_rb /)
      kbo(:, 5,33, 6) = (/ &
        & 0.89053e+03_rb,0.68882e+03_rb,0.53586e+03_rb,0.73678e+03_rb,0.94753e+03_rb /)
      kbo(:, 1,34, 6) = (/ &
        & 0.89054e+03_rb,0.68882e+03_rb,0.53592e+03_rb,0.73832e+03_rb,0.95035e+03_rb /)
      kbo(:, 2,34, 6) = (/ &
        & 0.89047e+03_rb,0.68877e+03_rb,0.53592e+03_rb,0.73824e+03_rb,0.95008e+03_rb /)
      kbo(:, 3,34, 6) = (/ &
        & 0.89041e+03_rb,0.68873e+03_rb,0.53589e+03_rb,0.73795e+03_rb,0.94947e+03_rb /)
      kbo(:, 4,34, 6) = (/ &
        & 0.89036e+03_rb,0.68869e+03_rb,0.53584e+03_rb,0.73741e+03_rb,0.94852e+03_rb /)
      kbo(:, 5,34, 6) = (/ &
        & 0.89031e+03_rb,0.68865e+03_rb,0.53575e+03_rb,0.73655e+03_rb,0.94719e+03_rb /)
      kbo(:, 1,35, 6) = (/ &
        & 0.89032e+03_rb,0.68866e+03_rb,0.53584e+03_rb,0.73832e+03_rb,0.95034e+03_rb /)
      kbo(:, 2,35, 6) = (/ &
        & 0.89027e+03_rb,0.68862e+03_rb,0.53584e+03_rb,0.73821e+03_rb,0.95002e+03_rb /)
      kbo(:, 3,35, 6) = (/ &
        & 0.89022e+03_rb,0.68858e+03_rb,0.53581e+03_rb,0.73788e+03_rb,0.94937e+03_rb /)
      kbo(:, 4,35, 6) = (/ &
        & 0.89017e+03_rb,0.68855e+03_rb,0.53576e+03_rb,0.73731e+03_rb,0.94836e+03_rb /)
      kbo(:, 5,35, 6) = (/ &
        & 0.89013e+03_rb,0.68852e+03_rb,0.53567e+03_rb,0.73640e+03_rb,0.94700e+03_rb /)
      kbo(:, 1,36, 6) = (/ &
        & 0.89015e+03_rb,0.68853e+03_rb,0.53577e+03_rb,0.73831e+03_rb,0.95033e+03_rb /)
      kbo(:, 2,36, 6) = (/ &
        & 0.89010e+03_rb,0.68849e+03_rb,0.53577e+03_rb,0.73820e+03_rb,0.95001e+03_rb /)
      kbo(:, 3,36, 6) = (/ &
        & 0.89006e+03_rb,0.68846e+03_rb,0.53575e+03_rb,0.73787e+03_rb,0.94935e+03_rb /)
      kbo(:, 4,36, 6) = (/ &
        & 0.89003e+03_rb,0.68844e+03_rb,0.53570e+03_rb,0.73729e+03_rb,0.94834e+03_rb /)
      kbo(:, 5,36, 6) = (/ &
        & 0.88999e+03_rb,0.68841e+03_rb,0.53561e+03_rb,0.73637e+03_rb,0.94696e+03_rb /)
      kbo(:, 1,37, 6) = (/ &
        & 0.89001e+03_rb,0.68843e+03_rb,0.53571e+03_rb,0.73831e+03_rb,0.95036e+03_rb /)
      kbo(:, 2,37, 6) = (/ &
        & 0.88997e+03_rb,0.68840e+03_rb,0.53572e+03_rb,0.73823e+03_rb,0.95007e+03_rb /)
      kbo(:, 3,37, 6) = (/ &
        & 0.88994e+03_rb,0.68837e+03_rb,0.53570e+03_rb,0.73792e+03_rb,0.94946e+03_rb /)
      kbo(:, 4,37, 6) = (/ &
        & 0.88991e+03_rb,0.68835e+03_rb,0.53566e+03_rb,0.73738e+03_rb,0.94849e+03_rb /)
      kbo(:, 5,37, 6) = (/ &
        & 0.88988e+03_rb,0.68833e+03_rb,0.53557e+03_rb,0.73651e+03_rb,0.94716e+03_rb /)
      kbo(:, 1,38, 6) = (/ &
        & 0.88990e+03_rb,0.68834e+03_rb,0.53566e+03_rb,0.73830e+03_rb,0.95036e+03_rb /)
      kbo(:, 2,38, 6) = (/ &
        & 0.88987e+03_rb,0.68832e+03_rb,0.53568e+03_rb,0.73825e+03_rb,0.95013e+03_rb /)
      kbo(:, 3,38, 6) = (/ &
        & 0.88984e+03_rb,0.68830e+03_rb,0.53566e+03_rb,0.73798e+03_rb,0.94957e+03_rb /)
      kbo(:, 4,38, 6) = (/ &
        & 0.88982e+03_rb,0.68828e+03_rb,0.53563e+03_rb,0.73747e+03_rb,0.94865e+03_rb /)
      kbo(:, 5,38, 6) = (/ &
        & 0.88979e+03_rb,0.68826e+03_rb,0.53555e+03_rb,0.73665e+03_rb,0.94737e+03_rb /)
      kbo(:, 1,39, 6) = (/ &
        & 0.88981e+03_rb,0.68827e+03_rb,0.53562e+03_rb,0.73829e+03_rb,0.95037e+03_rb /)
      kbo(:, 2,39, 6) = (/ &
        & 0.88978e+03_rb,0.68825e+03_rb,0.53564e+03_rb,0.73826e+03_rb,0.95018e+03_rb /)
      kbo(:, 3,39, 6) = (/ &
        & 0.88976e+03_rb,0.68824e+03_rb,0.53563e+03_rb,0.73803e+03_rb,0.94966e+03_rb /)
      kbo(:, 4,39, 6) = (/ &
        & 0.88974e+03_rb,0.68822e+03_rb,0.53560e+03_rb,0.73755e+03_rb,0.94880e+03_rb /)
      kbo(:, 5,39, 6) = (/ &
        & 0.88972e+03_rb,0.68821e+03_rb,0.53553e+03_rb,0.73679e+03_rb,0.94757e+03_rb /)
      kbo(:, 1,40, 6) = (/ &
        & 0.88973e+03_rb,0.68822e+03_rb,0.53558e+03_rb,0.73826e+03_rb,0.95037e+03_rb /)
      kbo(:, 2,40, 6) = (/ &
        & 0.88971e+03_rb,0.68820e+03_rb,0.53561e+03_rb,0.73829e+03_rb,0.95026e+03_rb /)
      kbo(:, 3,40, 6) = (/ &
        & 0.88969e+03_rb,0.68819e+03_rb,0.53561e+03_rb,0.73810e+03_rb,0.94982e+03_rb /)
      kbo(:, 4,40, 6) = (/ &
        & 0.88967e+03_rb,0.68817e+03_rb,0.53559e+03_rb,0.73769e+03_rb,0.94904e+03_rb /)
      kbo(:, 5,40, 6) = (/ &
        & 0.88966e+03_rb,0.68816e+03_rb,0.53553e+03_rb,0.73700e+03_rb,0.94790e+03_rb /)
      kbo(:, 1,41, 6) = (/ &
        & 0.88967e+03_rb,0.68817e+03_rb,0.53555e+03_rb,0.73822e+03_rb,0.95035e+03_rb /)
      kbo(:, 2,41, 6) = (/ &
        & 0.88965e+03_rb,0.68816e+03_rb,0.53558e+03_rb,0.73830e+03_rb,0.95031e+03_rb /)
      kbo(:, 3,41, 6) = (/ &
        & 0.88964e+03_rb,0.68815e+03_rb,0.53559e+03_rb,0.73817e+03_rb,0.94996e+03_rb /)
      kbo(:, 4,41, 6) = (/ &
        & 0.88962e+03_rb,0.68813e+03_rb,0.53558e+03_rb,0.73782e+03_rb,0.94927e+03_rb /)
      kbo(:, 5,41, 6) = (/ &
        & 0.88961e+03_rb,0.68812e+03_rb,0.53553e+03_rb,0.73722e+03_rb,0.94822e+03_rb /)
      kbo(:, 1,42, 6) = (/ &
        & 0.88962e+03_rb,0.68813e+03_rb,0.53551e+03_rb,0.73816e+03_rb,0.95029e+03_rb /)
      kbo(:, 2,42, 6) = (/ &
        & 0.88960e+03_rb,0.68812e+03_rb,0.53555e+03_rb,0.73830e+03_rb,0.95035e+03_rb /)
      kbo(:, 3,42, 6) = (/ &
        & 0.88959e+03_rb,0.68811e+03_rb,0.53557e+03_rb,0.73822e+03_rb,0.95008e+03_rb /)
      kbo(:, 4,42, 6) = (/ &
        & 0.88958e+03_rb,0.68810e+03_rb,0.53556e+03_rb,0.73793e+03_rb,0.94948e+03_rb /)
      kbo(:, 5,42, 6) = (/ &
        & 0.88957e+03_rb,0.68809e+03_rb,0.53553e+03_rb,0.73739e+03_rb,0.94853e+03_rb /)
      kbo(:, 1,43, 6) = (/ &
        & 0.88958e+03_rb,0.68810e+03_rb,0.53547e+03_rb,0.73807e+03_rb,0.95020e+03_rb /)
      kbo(:, 2,43, 6) = (/ &
        & 0.88957e+03_rb,0.68809e+03_rb,0.53552e+03_rb,0.73828e+03_rb,0.95037e+03_rb /)
      kbo(:, 3,43, 6) = (/ &
        & 0.88955e+03_rb,0.68808e+03_rb,0.53555e+03_rb,0.73826e+03_rb,0.95020e+03_rb /)
      kbo(:, 4,43, 6) = (/ &
        & 0.88954e+03_rb,0.68808e+03_rb,0.53555e+03_rb,0.73804e+03_rb,0.94970e+03_rb /)
      kbo(:, 5,43, 6) = (/ &
        & 0.88953e+03_rb,0.68807e+03_rb,0.53553e+03_rb,0.73758e+03_rb,0.94886e+03_rb /)
      kbo(:, 1,44, 6) = (/ &
        & 0.88954e+03_rb,0.68808e+03_rb,0.53544e+03_rb,0.73795e+03_rb,0.95005e+03_rb /)
      kbo(:, 2,44, 6) = (/ &
        & 0.88953e+03_rb,0.68807e+03_rb,0.53549e+03_rb,0.73823e+03_rb,0.95035e+03_rb /)
      kbo(:, 3,44, 6) = (/ &
        & 0.88952e+03_rb,0.68806e+03_rb,0.53553e+03_rb,0.73829e+03_rb,0.95030e+03_rb /)
      kbo(:, 4,44, 6) = (/ &
        & 0.88951e+03_rb,0.68805e+03_rb,0.53554e+03_rb,0.73815e+03_rb,0.94991e+03_rb /)
      kbo(:, 5,44, 6) = (/ &
        & 0.88951e+03_rb,0.68805e+03_rb,0.53553e+03_rb,0.73776e+03_rb,0.94919e+03_rb /)
      kbo(:, 1,45, 6) = (/ &
        & 0.88952e+03_rb,0.68806e+03_rb,0.53540e+03_rb,0.73782e+03_rb,0.94987e+03_rb /)
      kbo(:, 2,45, 6) = (/ &
        & 0.88951e+03_rb,0.68805e+03_rb,0.53547e+03_rb,0.73816e+03_rb,0.95030e+03_rb /)
      kbo(:, 3,45, 6) = (/ &
        & 0.88950e+03_rb,0.68804e+03_rb,0.53551e+03_rb,0.73830e+03_rb,0.95035e+03_rb /)
      kbo(:, 4,45, 6) = (/ &
        & 0.88949e+03_rb,0.68804e+03_rb,0.53553e+03_rb,0.73821e+03_rb,0.95008e+03_rb /)
      kbo(:, 5,45, 6) = (/ &
        & 0.88948e+03_rb,0.68803e+03_rb,0.53553e+03_rb,0.73792e+03_rb,0.94947e+03_rb /)
      kbo(:, 1,46, 6) = (/ &
        & 0.88949e+03_rb,0.68804e+03_rb,0.53537e+03_rb,0.73763e+03_rb,0.94962e+03_rb /)
      kbo(:, 2,46, 6) = (/ &
        & 0.88948e+03_rb,0.68803e+03_rb,0.53544e+03_rb,0.73806e+03_rb,0.95019e+03_rb /)
      kbo(:, 3,46, 6) = (/ &
        & 0.88948e+03_rb,0.68803e+03_rb,0.53549e+03_rb,0.73827e+03_rb,0.95037e+03_rb /)
      kbo(:, 4,46, 6) = (/ &
        & 0.88947e+03_rb,0.68802e+03_rb,0.53552e+03_rb,0.73827e+03_rb,0.95021e+03_rb /)
      kbo(:, 5,46, 6) = (/ &
        & 0.88946e+03_rb,0.68802e+03_rb,0.53552e+03_rb,0.73805e+03_rb,0.94973e+03_rb /)
      kbo(:, 1,47, 6) = (/ &
        & 0.88947e+03_rb,0.68802e+03_rb,0.53531e+03_rb,0.73737e+03_rb,0.94929e+03_rb /)
      kbo(:, 2,47, 6) = (/ &
        & 0.88947e+03_rb,0.68802e+03_rb,0.53539e+03_rb,0.73792e+03_rb,0.95000e+03_rb /)
      kbo(:, 3,47, 6) = (/ &
        & 0.88946e+03_rb,0.68801e+03_rb,0.53546e+03_rb,0.73821e+03_rb,0.95034e+03_rb /)
      kbo(:, 4,47, 6) = (/ &
        & 0.88945e+03_rb,0.68801e+03_rb,0.53550e+03_rb,0.73829e+03_rb,0.95032e+03_rb /)
      kbo(:, 5,47, 6) = (/ &
        & 0.88945e+03_rb,0.68800e+03_rb,0.53551e+03_rb,0.73816e+03_rb,0.94996e+03_rb /)
      kbo(:, 1,48, 6) = (/ &
        & 0.88946e+03_rb,0.68801e+03_rb,0.53528e+03_rb,0.73708e+03_rb,0.94889e+03_rb /)
      kbo(:, 2,48, 6) = (/ &
        & 0.88945e+03_rb,0.68801e+03_rb,0.53536e+03_rb,0.73773e+03_rb,0.94976e+03_rb /)
      kbo(:, 3,48, 6) = (/ &
        & 0.88944e+03_rb,0.68800e+03_rb,0.53543e+03_rb,0.73812e+03_rb,0.95025e+03_rb /)
      kbo(:, 4,48, 6) = (/ &
        & 0.88944e+03_rb,0.68800e+03_rb,0.53548e+03_rb,0.73829e+03_rb,0.95037e+03_rb /)
      kbo(:, 5,48, 6) = (/ &
        & 0.88944e+03_rb,0.68800e+03_rb,0.53551e+03_rb,0.73824e+03_rb,0.95014e+03_rb /)
      kbo(:, 1,49, 6) = (/ &
        & 0.88944e+03_rb,0.68800e+03_rb,0.53524e+03_rb,0.73674e+03_rb,0.94844e+03_rb /)
      kbo(:, 2,49, 6) = (/ &
        & 0.88944e+03_rb,0.68800e+03_rb,0.53533e+03_rb,0.73750e+03_rb,0.94945e+03_rb /)
      kbo(:, 3,49, 6) = (/ &
        & 0.88943e+03_rb,0.68799e+03_rb,0.53540e+03_rb,0.73799e+03_rb,0.95016e+03_rb /)
      kbo(:, 4,49, 6) = (/ &
        & 0.88943e+03_rb,0.68799e+03_rb,0.53546e+03_rb,0.73825e+03_rb,0.95036e+03_rb /)
      kbo(:, 5,49, 6) = (/ &
        & 0.88942e+03_rb,0.68799e+03_rb,0.53549e+03_rb,0.73828e+03_rb,0.95027e+03_rb /)
      kbo(:, 1,50, 6) = (/ &
        & 0.88943e+03_rb,0.68799e+03_rb,0.53521e+03_rb,0.73638e+03_rb,0.94796e+03_rb /)
      kbo(:, 2,50, 6) = (/ &
        & 0.88943e+03_rb,0.68799e+03_rb,0.53529e+03_rb,0.73724e+03_rb,0.94911e+03_rb /)
      kbo(:, 3,50, 6) = (/ &
        & 0.88942e+03_rb,0.68798e+03_rb,0.53537e+03_rb,0.73783e+03_rb,0.94990e+03_rb /)
      kbo(:, 4,50, 6) = (/ &
        & 0.88942e+03_rb,0.68798e+03_rb,0.53544e+03_rb,0.73817e+03_rb,0.95031e+03_rb /)
      kbo(:, 5,50, 6) = (/ &
        & 0.88942e+03_rb,0.68798e+03_rb,0.53548e+03_rb,0.73829e+03_rb,0.95034e+03_rb /)
      kbo(:, 1,51, 6) = (/ &
        & 0.88942e+03_rb,0.68798e+03_rb,0.53518e+03_rb,0.73600e+03_rb,0.94745e+03_rb /)
      kbo(:, 2,51, 6) = (/ &
        & 0.88942e+03_rb,0.68798e+03_rb,0.53525e+03_rb,0.73696e+03_rb,0.94873e+03_rb /)
      kbo(:, 3,51, 6) = (/ &
        & 0.88941e+03_rb,0.68798e+03_rb,0.53533e+03_rb,0.73765e+03_rb,0.94966e+03_rb /)
      kbo(:, 4,51, 6) = (/ &
        & 0.88941e+03_rb,0.68798e+03_rb,0.53541e+03_rb,0.73807e+03_rb,0.95020e+03_rb /)
      kbo(:, 5,51, 6) = (/ &
        & 0.88941e+03_rb,0.68797e+03_rb,0.53546e+03_rb,0.73827e+03_rb,0.95037e+03_rb /)
      kbo(:, 1,52, 6) = (/ &
        & 0.88941e+03_rb,0.68798e+03_rb,0.53513e+03_rb,0.73556e+03_rb,0.94688e+03_rb /)
      kbo(:, 2,52, 6) = (/ &
        & 0.88941e+03_rb,0.68798e+03_rb,0.53522e+03_rb,0.73664e+03_rb,0.94831e+03_rb /)
      kbo(:, 3,52, 6) = (/ &
        & 0.88941e+03_rb,0.68797e+03_rb,0.53531e+03_rb,0.73743e+03_rb,0.94936e+03_rb /)
      kbo(:, 4,52, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53538e+03_rb,0.73795e+03_rb,0.95005e+03_rb /)
      kbo(:, 5,52, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53545e+03_rb,0.73822e+03_rb,0.95036e+03_rb /)
      kbo(:, 1,53, 6) = (/ &
        & 0.88941e+03_rb,0.68797e+03_rb,0.53507e+03_rb,0.73509e+03_rb,0.94627e+03_rb /)
      kbo(:, 2,53, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53519e+03_rb,0.73629e+03_rb,0.94784e+03_rb /)
      kbo(:, 3,53, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53527e+03_rb,0.73717e+03_rb,0.94902e+03_rb /)
      kbo(:, 4,53, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53535e+03_rb,0.73779e+03_rb,0.94984e+03_rb /)
      kbo(:, 5,53, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53542e+03_rb,0.73815e+03_rb,0.95029e+03_rb /)
      kbo(:, 1,54, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53502e+03_rb,0.73463e+03_rb,0.94566e+03_rb /)
      kbo(:, 2,54, 6) = (/ &
        & 0.88940e+03_rb,0.68797e+03_rb,0.53516e+03_rb,0.73592e+03_rb,0.94735e+03_rb /)
      kbo(:, 3,54, 6) = (/ &
        & 0.88940e+03_rb,0.68796e+03_rb,0.53524e+03_rb,0.73691e+03_rb,0.94866e+03_rb /)
      kbo(:, 4,54, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53533e+03_rb,0.73761e+03_rb,0.94961e+03_rb /)
      kbo(:, 5,54, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53539e+03_rb,0.73805e+03_rb,0.95018e+03_rb /)
      kbo(:, 1,55, 6) = (/ &
        & 0.88940e+03_rb,0.68796e+03_rb,0.53495e+03_rb,0.73415e+03_rb,0.94503e+03_rb /)
      kbo(:, 2,55, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53513e+03_rb,0.73554e+03_rb,0.94685e+03_rb /)
      kbo(:, 3,55, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53521e+03_rb,0.73662e+03_rb,0.94828e+03_rb /)
      kbo(:, 4,55, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53530e+03_rb,0.73742e+03_rb,0.94934e+03_rb /)
      kbo(:, 5,55, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53537e+03_rb,0.73794e+03_rb,0.95004e+03_rb /)
      kbo(:, 1,56, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53489e+03_rb,0.73364e+03_rb,0.94437e+03_rb /)
      kbo(:, 2,56, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53507e+03_rb,0.73512e+03_rb,0.94631e+03_rb /)
      kbo(:, 3,56, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53519e+03_rb,0.73631e+03_rb,0.94787e+03_rb /)
      kbo(:, 4,56, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53527e+03_rb,0.73719e+03_rb,0.94904e+03_rb /)
      kbo(:, 5,56, 6) = (/ &
        & 0.88938e+03_rb,0.68796e+03_rb,0.53535e+03_rb,0.73780e+03_rb,0.94987e+03_rb /)
      kbo(:, 1,57, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53481e+03_rb,0.73310e+03_rb,0.94366e+03_rb /)
      kbo(:, 2,57, 6) = (/ &
        & 0.88939e+03_rb,0.68796e+03_rb,0.53502e+03_rb,0.73469e+03_rb,0.94573e+03_rb /)
      kbo(:, 3,57, 6) = (/ &
        & 0.88938e+03_rb,0.68796e+03_rb,0.53516e+03_rb,0.73597e+03_rb,0.94741e+03_rb /)
      kbo(:, 4,57, 6) = (/ &
        & 0.88938e+03_rb,0.68796e+03_rb,0.53524e+03_rb,0.73694e+03_rb,0.94871e+03_rb /)
      kbo(:, 5,57, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53532e+03_rb,0.73764e+03_rb,0.94964e+03_rb /)
      kbo(:, 1,58, 6) = (/ &
        & 0.88938e+03_rb,0.68796e+03_rb,0.53474e+03_rb,0.73255e+03_rb,0.94296e+03_rb /)
      kbo(:, 2,58, 6) = (/ &
        & 0.88938e+03_rb,0.68796e+03_rb,0.53496e+03_rb,0.73423e+03_rb,0.94515e+03_rb /)
      kbo(:, 3,58, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53512e+03_rb,0.73561e+03_rb,0.94695e+03_rb /)
      kbo(:, 4,58, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53522e+03_rb,0.73667e+03_rb,0.94836e+03_rb /)
      kbo(:, 5,58, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53530e+03_rb,0.73745e+03_rb,0.94939e+03_rb /)
      kbo(:, 1,59, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53470e+03_rb,0.73233e+03_rb,0.94266e+03_rb /)
      kbo(:, 2,59, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53494e+03_rb,0.73405e+03_rb,0.94490e+03_rb /)
      kbo(:, 3,59, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53510e+03_rb,0.73546e+03_rb,0.94675e+03_rb /)
      kbo(:, 4,59, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53521e+03_rb,0.73657e+03_rb,0.94820e+03_rb /)
      kbo(:, 5,59, 6) = (/ &
        & 0.88938e+03_rb,0.68795e+03_rb,0.53529e+03_rb,0.73737e+03_rb,0.94929e+03_rb /)
      kbo(:, 1,13, 7) = (/ &
        & 0.11104e+04_rb,0.84453e+03_rb,0.58852e+03_rb,0.81799e+03_rb,0.10746e+04_rb /)
      kbo(:, 2,13, 7) = (/ &
        & 0.11019e+04_rb,0.83810e+03_rb,0.58572e+03_rb,0.81836e+03_rb,0.10752e+04_rb /)
      kbo(:, 3,13, 7) = (/ &
        & 0.10945e+04_rb,0.83257e+03_rb,0.58306e+03_rb,0.81848e+03_rb,0.10754e+04_rb /)
      kbo(:, 4,13, 7) = (/ &
        & 0.10881e+04_rb,0.82775e+03_rb,0.58047e+03_rb,0.81831e+03_rb,0.10753e+04_rb /)
      kbo(:, 5,13, 7) = (/ &
        & 0.10824e+04_rb,0.82352e+03_rb,0.57809e+03_rb,0.81788e+03_rb,0.10749e+04_rb /)
      kbo(:, 1,14, 7) = (/ &
        & 0.10890e+04_rb,0.82842e+03_rb,0.58087e+03_rb,0.81780e+03_rb,0.10747e+04_rb /)
      kbo(:, 2,14, 7) = (/ &
        & 0.10820e+04_rb,0.82323e+03_rb,0.57803e+03_rb,0.81816e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,14, 7) = (/ &
        & 0.10761e+04_rb,0.81875e+03_rb,0.57524e+03_rb,0.81827e+03_rb,0.10754e+04_rb /)
      kbo(:, 4,14, 7) = (/ &
        & 0.10709e+04_rb,0.81486e+03_rb,0.57281e+03_rb,0.81809e+03_rb,0.10752e+04_rb /)
      kbo(:, 5,14, 7) = (/ &
        & 0.10663e+04_rb,0.81144e+03_rb,0.57087e+03_rb,0.81766e+03_rb,0.10748e+04_rb /)
      kbo(:, 1,15, 7) = (/ &
        & 0.10714e+04_rb,0.81529e+03_rb,0.57324e+03_rb,0.81765e+03_rb,0.10748e+04_rb /)
      kbo(:, 2,15, 7) = (/ &
        & 0.10659e+04_rb,0.81110e+03_rb,0.57094e+03_rb,0.81800e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,15, 7) = (/ &
        & 0.10611e+04_rb,0.80749e+03_rb,0.56926e+03_rb,0.81809e+03_rb,0.10754e+04_rb /)
      kbo(:, 4,15, 7) = (/ &
        & 0.10569e+04_rb,0.80434e+03_rb,0.56791e+03_rb,0.81790e+03_rb,0.10752e+04_rb /)
      kbo(:, 5,15, 7) = (/ &
        & 0.10532e+04_rb,0.80157e+03_rb,0.56672e+03_rb,0.81746e+03_rb,0.10748e+04_rb /)
      kbo(:, 1,16, 7) = (/ &
        & 0.10572e+04_rb,0.80460e+03_rb,0.56822e+03_rb,0.81755e+03_rb,0.10748e+04_rb /)
      kbo(:, 2,16, 7) = (/ &
        & 0.10527e+04_rb,0.80122e+03_rb,0.56701e+03_rb,0.81788e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,16, 7) = (/ &
        & 0.10488e+04_rb,0.79831e+03_rb,0.56599e+03_rb,0.81796e+03_rb,0.10754e+04_rb /)
      kbo(:, 4,16, 7) = (/ &
        & 0.10454e+04_rb,0.79577e+03_rb,0.56505e+03_rb,0.81773e+03_rb,0.10752e+04_rb /)
      kbo(:, 5,16, 7) = (/ &
        & 0.10425e+04_rb,0.79354e+03_rb,0.56418e+03_rb,0.81728e+03_rb,0.10747e+04_rb /)
      kbo(:, 1,17, 7) = (/ &
        & 0.10456e+04_rb,0.79593e+03_rb,0.56540e+03_rb,0.81747e+03_rb,0.10749e+04_rb /)
      kbo(:, 2,17, 7) = (/ &
        & 0.10420e+04_rb,0.79321e+03_rb,0.56464e+03_rb,0.81778e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,17, 7) = (/ &
        & 0.10389e+04_rb,0.79086e+03_rb,0.56406e+03_rb,0.81783e+03_rb,0.10754e+04_rb /)
      kbo(:, 4,17, 7) = (/ &
        & 0.10362e+04_rb,0.78881e+03_rb,0.56355e+03_rb,0.81761e+03_rb,0.10751e+04_rb /)
      kbo(:, 5,17, 7) = (/ &
        & 0.10337e+04_rb,0.78701e+03_rb,0.56298e+03_rb,0.81712e+03_rb,0.10746e+04_rb /)
      kbo(:, 1,18, 7) = (/ &
        & 0.10363e+04_rb,0.78890e+03_rb,0.56372e+03_rb,0.81741e+03_rb,0.10750e+04_rb /)
      kbo(:, 2,18, 7) = (/ &
        & 0.10334e+04_rb,0.78671e+03_rb,0.56351e+03_rb,0.81771e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,18, 7) = (/ &
        & 0.10308e+04_rb,0.78481e+03_rb,0.56335e+03_rb,0.81775e+03_rb,0.10754e+04_rb /)
      kbo(:, 4,18, 7) = (/ &
        & 0.10286e+04_rb,0.78316e+03_rb,0.56306e+03_rb,0.81749e+03_rb,0.10751e+04_rb /)
      kbo(:, 5,18, 7) = (/ &
        & 0.10267e+04_rb,0.78170e+03_rb,0.56253e+03_rb,0.81698e+03_rb,0.10746e+04_rb /)
      kbo(:, 1,19, 7) = (/ &
        & 0.10287e+04_rb,0.78319e+03_rb,0.56306e+03_rb,0.81737e+03_rb,0.10751e+04_rb /)
      kbo(:, 2,19, 7) = (/ &
        & 0.10263e+04_rb,0.78143e+03_rb,0.56317e+03_rb,0.81766e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,19, 7) = (/ &
        & 0.10243e+04_rb,0.77990e+03_rb,0.56256e+03_rb,0.81764e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,19, 7) = (/ &
        & 0.10225e+04_rb,0.77856e+03_rb,0.56270e+03_rb,0.81737e+03_rb,0.10751e+04_rb /)
      kbo(:, 5,19, 7) = (/ &
        & 0.10209e+04_rb,0.77738e+03_rb,0.56230e+03_rb,0.81685e+03_rb,0.10745e+04_rb /)
      kbo(:, 1,20, 7) = (/ &
        & 0.10225e+04_rb,0.77853e+03_rb,0.56258e+03_rb,0.81735e+03_rb,0.10751e+04_rb /)
      kbo(:, 2,20, 7) = (/ &
        & 0.10206e+04_rb,0.77711e+03_rb,0.56304e+03_rb,0.81761e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,20, 7) = (/ &
        & 0.10189e+04_rb,0.77589e+03_rb,0.56295e+03_rb,0.81757e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,20, 7) = (/ &
        & 0.10175e+04_rb,0.77480e+03_rb,0.56264e+03_rb,0.81728e+03_rb,0.10750e+04_rb /)
      kbo(:, 5,20, 7) = (/ &
        & 0.10162e+04_rb,0.77385e+03_rb,0.56211e+03_rb,0.81672e+03_rb,0.10744e+04_rb /)
      kbo(:, 1,21, 7) = (/ &
        & 0.10174e+04_rb,0.77475e+03_rb,0.56279e+03_rb,0.81734e+03_rb,0.10752e+04_rb /)
      kbo(:, 2,21, 7) = (/ &
        & 0.10159e+04_rb,0.77361e+03_rb,0.56292e+03_rb,0.81756e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,21, 7) = (/ &
        & 0.10146e+04_rb,0.77262e+03_rb,0.56281e+03_rb,0.81751e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,21, 7) = (/ &
        & 0.10134e+04_rb,0.77175e+03_rb,0.56248e+03_rb,0.81718e+03_rb,0.10750e+04_rb /)
      kbo(:, 5,21, 7) = (/ &
        & 0.10124e+04_rb,0.77098e+03_rb,0.56193e+03_rb,0.81661e+03_rb,0.10743e+04_rb /)
      kbo(:, 1,22, 7) = (/ &
        & 0.10132e+04_rb,0.77163e+03_rb,0.56273e+03_rb,0.81736e+03_rb,0.10752e+04_rb /)
      kbo(:, 2,22, 7) = (/ &
        & 0.10120e+04_rb,0.77071e+03_rb,0.56281e+03_rb,0.81753e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,22, 7) = (/ &
        & 0.10110e+04_rb,0.76992e+03_rb,0.56267e+03_rb,0.81743e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,22, 7) = (/ &
        & 0.10100e+04_rb,0.76922e+03_rb,0.56230e+03_rb,0.81706e+03_rb,0.10749e+04_rb /)
      kbo(:, 5,22, 7) = (/ &
        & 0.10092e+04_rb,0.76860e+03_rb,0.56172e+03_rb,0.81643e+03_rb,0.10742e+04_rb /)
      kbo(:, 1,23, 7) = (/ &
        & 0.10099e+04_rb,0.76909e+03_rb,0.56267e+03_rb,0.81738e+03_rb,0.10753e+04_rb /)
      kbo(:, 2,23, 7) = (/ &
        & 0.10089e+04_rb,0.76836e+03_rb,0.56273e+03_rb,0.81751e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,23, 7) = (/ &
        & 0.10080e+04_rb,0.76773e+03_rb,0.56254e+03_rb,0.81734e+03_rb,0.10752e+04_rb /)
      kbo(:, 4,23, 7) = (/ &
        & 0.10073e+04_rb,0.76717e+03_rb,0.56214e+03_rb,0.81691e+03_rb,0.10748e+04_rb /)
      kbo(:, 5,23, 7) = (/ &
        & 0.10067e+04_rb,0.76668e+03_rb,0.56152e+03_rb,0.81625e+03_rb,0.10740e+04_rb /)
      kbo(:, 1,24, 7) = (/ &
        & 0.10071e+04_rb,0.76704e+03_rb,0.56263e+03_rb,0.81739e+03_rb,0.10753e+04_rb /)
      kbo(:, 2,24, 7) = (/ &
        & 0.10064e+04_rb,0.76647e+03_rb,0.56264e+03_rb,0.81747e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,24, 7) = (/ &
        & 0.10057e+04_rb,0.76596e+03_rb,0.56242e+03_rb,0.81725e+03_rb,0.10751e+04_rb /)
      kbo(:, 4,24, 7) = (/ &
        & 0.10051e+04_rb,0.76551e+03_rb,0.56198e+03_rb,0.81678e+03_rb,0.10746e+04_rb /)
      kbo(:, 5,24, 7) = (/ &
        & 0.10046e+04_rb,0.76512e+03_rb,0.56132e+03_rb,0.81606e+03_rb,0.10738e+04_rb /)
      kbo(:, 1,25, 7) = (/ &
        & 0.10049e+04_rb,0.76538e+03_rb,0.56260e+03_rb,0.81741e+03_rb,0.10754e+04_rb /)
      kbo(:, 2,25, 7) = (/ &
        & 0.10043e+04_rb,0.76492e+03_rb,0.56256e+03_rb,0.81742e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,25, 7) = (/ &
        & 0.10038e+04_rb,0.76453e+03_rb,0.56229e+03_rb,0.81717e+03_rb,0.10750e+04_rb /)
      kbo(:, 4,25, 7) = (/ &
        & 0.10033e+04_rb,0.76417e+03_rb,0.56181e+03_rb,0.81663e+03_rb,0.10745e+04_rb /)
      kbo(:, 5,25, 7) = (/ &
        & 0.10029e+04_rb,0.76385e+03_rb,0.56111e+03_rb,0.81585e+03_rb,0.10736e+04_rb /)
      kbo(:, 1,26, 7) = (/ &
        & 0.10031e+04_rb,0.76404e+03_rb,0.56257e+03_rb,0.81743e+03_rb,0.10754e+04_rb /)
      kbo(:, 2,26, 7) = (/ &
        & 0.10026e+04_rb,0.76368e+03_rb,0.56248e+03_rb,0.81736e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,26, 7) = (/ &
        & 0.10022e+04_rb,0.76336e+03_rb,0.56216e+03_rb,0.81707e+03_rb,0.10749e+04_rb /)
      kbo(:, 4,26, 7) = (/ &
        & 0.10018e+04_rb,0.76308e+03_rb,0.56163e+03_rb,0.81645e+03_rb,0.10743e+04_rb /)
      kbo(:, 5,26, 7) = (/ &
        & 0.10015e+04_rb,0.76282e+03_rb,0.56088e+03_rb,0.81561e+03_rb,0.10734e+04_rb /)
      kbo(:, 1,27, 7) = (/ &
        & 0.10017e+04_rb,0.76296e+03_rb,0.56253e+03_rb,0.81743e+03_rb,0.10754e+04_rb /)
      kbo(:, 2,27, 7) = (/ &
        & 0.10013e+04_rb,0.76268e+03_rb,0.56239e+03_rb,0.81730e+03_rb,0.10752e+04_rb /)
      kbo(:, 3,27, 7) = (/ &
        & 0.10010e+04_rb,0.76242e+03_rb,0.56202e+03_rb,0.81696e+03_rb,0.10748e+04_rb /)
      kbo(:, 4,27, 7) = (/ &
        & 0.10007e+04_rb,0.76220e+03_rb,0.56144e+03_rb,0.81630e+03_rb,0.10741e+04_rb /)
      kbo(:, 5,27, 7) = (/ &
        & 0.10004e+04_rb,0.76200e+03_rb,0.56064e+03_rb,0.81536e+03_rb,0.10731e+04_rb /)
      kbo(:, 1,28, 7) = (/ &
        & 0.10005e+04_rb,0.76209e+03_rb,0.56249e+03_rb,0.81738e+03_rb,0.10754e+04_rb /)
      kbo(:, 2,28, 7) = (/ &
        & 0.10002e+04_rb,0.76186e+03_rb,0.56229e+03_rb,0.81724e+03_rb,0.10752e+04_rb /)
      kbo(:, 3,28, 7) = (/ &
        & 0.99995e+03_rb,0.76166e+03_rb,0.56188e+03_rb,0.81687e+03_rb,0.10747e+04_rb /)
      kbo(:, 4,28, 7) = (/ &
        & 0.99971e+03_rb,0.76148e+03_rb,0.56124e+03_rb,0.81607e+03_rb,0.10739e+04_rb /)
      kbo(:, 5,28, 7) = (/ &
        & 0.99950e+03_rb,0.76132e+03_rb,0.56040e+03_rb,0.81510e+03_rb,0.10728e+04_rb /)
      kbo(:, 1,29, 7) = (/ &
        & 0.99960e+03_rb,0.76139e+03_rb,0.56244e+03_rb,0.81739e+03_rb,0.10753e+04_rb /)
      kbo(:, 2,29, 7) = (/ &
        & 0.99935e+03_rb,0.76121e+03_rb,0.56219e+03_rb,0.81712e+03_rb,0.10751e+04_rb /)
      kbo(:, 3,29, 7) = (/ &
        & 0.99914e+03_rb,0.76105e+03_rb,0.56172e+03_rb,0.81663e+03_rb,0.10745e+04_rb /)
      kbo(:, 4,29, 7) = (/ &
        & 0.99895e+03_rb,0.76091e+03_rb,0.56103e+03_rb,0.81584e+03_rb,0.10737e+04_rb /)
      kbo(:, 5,29, 7) = (/ &
        & 0.99878e+03_rb,0.76078e+03_rb,0.56014e+03_rb,0.81482e+03_rb,0.10725e+04_rb /)
      kbo(:, 1,30, 7) = (/ &
        & 0.99885e+03_rb,0.76083e+03_rb,0.56238e+03_rb,0.81733e+03_rb,0.10753e+04_rb /)
      kbo(:, 2,30, 7) = (/ &
        & 0.99865e+03_rb,0.76069e+03_rb,0.56207e+03_rb,0.81703e+03_rb,0.10750e+04_rb /)
      kbo(:, 3,30, 7) = (/ &
        & 0.99848e+03_rb,0.76055e+03_rb,0.56155e+03_rb,0.81643e+03_rb,0.10743e+04_rb /)
      kbo(:, 4,30, 7) = (/ &
        & 0.99833e+03_rb,0.76044e+03_rb,0.56081e+03_rb,0.81561e+03_rb,0.10734e+04_rb /)
      kbo(:, 5,30, 7) = (/ &
        & 0.99819e+03_rb,0.76034e+03_rb,0.55986e+03_rb,0.81451e+03_rb,0.10721e+04_rb /)
      kbo(:, 1,31, 7) = (/ &
        & 0.99824e+03_rb,0.76037e+03_rb,0.56231e+03_rb,0.81727e+03_rb,0.10752e+04_rb /)
      kbo(:, 2,31, 7) = (/ &
        & 0.99808e+03_rb,0.76026e+03_rb,0.56195e+03_rb,0.81690e+03_rb,0.10748e+04_rb /)
      kbo(:, 3,31, 7) = (/ &
        & 0.99794e+03_rb,0.76015e+03_rb,0.56137e+03_rb,0.81625e+03_rb,0.10741e+04_rb /)
      kbo(:, 4,31, 7) = (/ &
        & 0.99782e+03_rb,0.76006e+03_rb,0.56058e+03_rb,0.81535e+03_rb,0.10731e+04_rb /)
      kbo(:, 5,31, 7) = (/ &
        & 0.99772e+03_rb,0.75998e+03_rb,0.55958e+03_rb,0.81419e+03_rb,0.10717e+04_rb /)
      kbo(:, 1,32, 7) = (/ &
        & 0.99774e+03_rb,0.76000e+03_rb,0.56222e+03_rb,0.81719e+03_rb,0.10752e+04_rb /)
      kbo(:, 2,32, 7) = (/ &
        & 0.99763e+03_rb,0.75991e+03_rb,0.56181e+03_rb,0.81675e+03_rb,0.10747e+04_rb /)
      kbo(:, 3,32, 7) = (/ &
        & 0.99752e+03_rb,0.75984e+03_rb,0.56117e+03_rb,0.81604e+03_rb,0.10739e+04_rb /)
      kbo(:, 4,32, 7) = (/ &
        & 0.99742e+03_rb,0.75976e+03_rb,0.56033e+03_rb,0.81507e+03_rb,0.10728e+04_rb /)
      kbo(:, 5,32, 7) = (/ &
        & 0.99733e+03_rb,0.75970e+03_rb,0.55927e+03_rb,0.81384e+03_rb,0.10713e+04_rb /)
      kbo(:, 1,33, 7) = (/ &
        & 0.99735e+03_rb,0.75971e+03_rb,0.56212e+03_rb,0.81709e+03_rb,0.10751e+04_rb /)
      kbo(:, 2,33, 7) = (/ &
        & 0.99725e+03_rb,0.75964e+03_rb,0.56165e+03_rb,0.81658e+03_rb,0.10745e+04_rb /)
      kbo(:, 3,33, 7) = (/ &
        & 0.99717e+03_rb,0.75957e+03_rb,0.56096e+03_rb,0.81581e+03_rb,0.10736e+04_rb /)
      kbo(:, 4,33, 7) = (/ &
        & 0.99709e+03_rb,0.75952e+03_rb,0.56006e+03_rb,0.81477e+03_rb,0.10724e+04_rb /)
      kbo(:, 5,33, 7) = (/ &
        & 0.99703e+03_rb,0.75946e+03_rb,0.55896e+03_rb,0.81347e+03_rb,0.10709e+04_rb /)
      kbo(:, 1,34, 7) = (/ &
        & 0.99704e+03_rb,0.75947e+03_rb,0.56203e+03_rb,0.81702e+03_rb,0.10750e+04_rb /)
      kbo(:, 2,34, 7) = (/ &
        & 0.99696e+03_rb,0.75942e+03_rb,0.56151e+03_rb,0.81642e+03_rb,0.10743e+04_rb /)
      kbo(:, 3,34, 7) = (/ &
        & 0.99689e+03_rb,0.75937e+03_rb,0.56077e+03_rb,0.81558e+03_rb,0.10734e+04_rb /)
      kbo(:, 4,34, 7) = (/ &
        & 0.99683e+03_rb,0.75932e+03_rb,0.55982e+03_rb,0.81449e+03_rb,0.10721e+04_rb /)
      kbo(:, 5,34, 7) = (/ &
        & 0.99678e+03_rb,0.75928e+03_rb,0.55868e+03_rb,0.81314e+03_rb,0.10704e+04_rb /)
      kbo(:, 1,35, 7) = (/ &
        & 0.99679e+03_rb,0.75929e+03_rb,0.56197e+03_rb,0.81693e+03_rb,0.10749e+04_rb /)
      kbo(:, 2,35, 7) = (/ &
        & 0.99673e+03_rb,0.75925e+03_rb,0.56142e+03_rb,0.81633e+03_rb,0.10742e+04_rb /)
      kbo(:, 3,35, 7) = (/ &
        & 0.99667e+03_rb,0.75920e+03_rb,0.56065e+03_rb,0.81545e+03_rb,0.10733e+04_rb /)
      kbo(:, 4,35, 7) = (/ &
        & 0.99663e+03_rb,0.75916e+03_rb,0.55968e+03_rb,0.81434e+03_rb,0.10719e+04_rb /)
      kbo(:, 5,35, 7) = (/ &
        & 0.99658e+03_rb,0.75913e+03_rb,0.55851e+03_rb,0.81294e+03_rb,0.10702e+04_rb /)
      kbo(:, 1,36, 7) = (/ &
        & 0.99660e+03_rb,0.75914e+03_rb,0.56195e+03_rb,0.81692e+03_rb,0.10749e+04_rb /)
      kbo(:, 2,36, 7) = (/ &
        & 0.99655e+03_rb,0.75911e+03_rb,0.56140e+03_rb,0.81631e+03_rb,0.10742e+04_rb /)
      kbo(:, 3,36, 7) = (/ &
        & 0.99650e+03_rb,0.75907e+03_rb,0.56063e+03_rb,0.81543e+03_rb,0.10732e+04_rb /)
      kbo(:, 4,36, 7) = (/ &
        & 0.99646e+03_rb,0.75904e+03_rb,0.55965e+03_rb,0.81430e+03_rb,0.10719e+04_rb /)
      kbo(:, 5,36, 7) = (/ &
        & 0.99642e+03_rb,0.75901e+03_rb,0.55848e+03_rb,0.81290e+03_rb,0.10702e+04_rb /)
      kbo(:, 1,37, 7) = (/ &
        & 0.99645e+03_rb,0.75903e+03_rb,0.56200e+03_rb,0.81697e+03_rb,0.10749e+04_rb /)
      kbo(:, 2,37, 7) = (/ &
        & 0.99640e+03_rb,0.75900e+03_rb,0.56148e+03_rb,0.81640e+03_rb,0.10743e+04_rb /)
      kbo(:, 3,37, 7) = (/ &
        & 0.99636e+03_rb,0.75897e+03_rb,0.56074e+03_rb,0.81556e+03_rb,0.10734e+04_rb /)
      kbo(:, 4,37, 7) = (/ &
        & 0.99633e+03_rb,0.75894e+03_rb,0.55979e+03_rb,0.81446e+03_rb,0.10721e+04_rb /)
      kbo(:, 5,37, 7) = (/ &
        & 0.99630e+03_rb,0.75892e+03_rb,0.55864e+03_rb,0.81313e+03_rb,0.10704e+04_rb /)
      kbo(:, 1,38, 7) = (/ &
        & 0.99632e+03_rb,0.75894e+03_rb,0.56206e+03_rb,0.81703e+03_rb,0.10750e+04_rb /)
      kbo(:, 2,38, 7) = (/ &
        & 0.99628e+03_rb,0.75891e+03_rb,0.56157e+03_rb,0.81650e+03_rb,0.10744e+04_rb /)
      kbo(:, 3,38, 7) = (/ &
        & 0.99625e+03_rb,0.75889e+03_rb,0.56085e+03_rb,0.81569e+03_rb,0.10735e+04_rb /)
      kbo(:, 4,38, 7) = (/ &
        & 0.99623e+03_rb,0.75887e+03_rb,0.55993e+03_rb,0.81464e+03_rb,0.10723e+04_rb /)
      kbo(:, 5,38, 7) = (/ &
        & 0.99620e+03_rb,0.75885e+03_rb,0.55881e+03_rb,0.81331e+03_rb,0.10707e+04_rb /)
      kbo(:, 1,39, 7) = (/ &
        & 0.99622e+03_rb,0.75886e+03_rb,0.56210e+03_rb,0.81708e+03_rb,0.10751e+04_rb /)
      kbo(:, 2,39, 7) = (/ &
        & 0.99619e+03_rb,0.75884e+03_rb,0.56164e+03_rb,0.81666e+03_rb,0.10745e+04_rb /)
      kbo(:, 3,39, 7) = (/ &
        & 0.99617e+03_rb,0.75882e+03_rb,0.56096e+03_rb,0.81582e+03_rb,0.10737e+04_rb /)
      kbo(:, 4,39, 7) = (/ &
        & 0.99614e+03_rb,0.75880e+03_rb,0.56007e+03_rb,0.81479e+03_rb,0.10725e+04_rb /)
      kbo(:, 5,39, 7) = (/ &
        & 0.99612e+03_rb,0.75878e+03_rb,0.55897e+03_rb,0.81351e+03_rb,0.10709e+04_rb /)
      kbo(:, 1,40, 7) = (/ &
        & 0.99613e+03_rb,0.75880e+03_rb,0.56218e+03_rb,0.81716e+03_rb,0.10752e+04_rb /)
      kbo(:, 2,40, 7) = (/ &
        & 0.99611e+03_rb,0.75878e+03_rb,0.56177e+03_rb,0.81672e+03_rb,0.10747e+04_rb /)
      kbo(:, 3,40, 7) = (/ &
        & 0.99609e+03_rb,0.75876e+03_rb,0.56114e+03_rb,0.81603e+03_rb,0.10739e+04_rb /)
      kbo(:, 4,40, 7) = (/ &
        & 0.99607e+03_rb,0.75875e+03_rb,0.56030e+03_rb,0.81506e+03_rb,0.10728e+04_rb /)
      kbo(:, 5,40, 7) = (/ &
        & 0.99605e+03_rb,0.75874e+03_rb,0.55925e+03_rb,0.81383e+03_rb,0.10713e+04_rb /)
      kbo(:, 1,41, 7) = (/ &
        & 0.99606e+03_rb,0.75874e+03_rb,0.56225e+03_rb,0.81724e+03_rb,0.10752e+04_rb /)
      kbo(:, 2,41, 7) = (/ &
        & 0.99604e+03_rb,0.75873e+03_rb,0.56189e+03_rb,0.81688e+03_rb,0.10748e+04_rb /)
      kbo(:, 3,41, 7) = (/ &
        & 0.99602e+03_rb,0.75871e+03_rb,0.56132e+03_rb,0.81623e+03_rb,0.10741e+04_rb /)
      kbo(:, 4,41, 7) = (/ &
        & 0.99601e+03_rb,0.75870e+03_rb,0.56053e+03_rb,0.81532e+03_rb,0.10731e+04_rb /)
      kbo(:, 5,41, 7) = (/ &
        & 0.99600e+03_rb,0.75869e+03_rb,0.55954e+03_rb,0.81416e+03_rb,0.10717e+04_rb /)
      kbo(:, 1,42, 7) = (/ &
        & 0.99601e+03_rb,0.75870e+03_rb,0.56231e+03_rb,0.81730e+03_rb,0.10753e+04_rb /)
      kbo(:, 2,42, 7) = (/ &
        & 0.99599e+03_rb,0.75869e+03_rb,0.56200e+03_rb,0.81698e+03_rb,0.10750e+04_rb /)
      kbo(:, 3,42, 7) = (/ &
        & 0.99597e+03_rb,0.75868e+03_rb,0.56149e+03_rb,0.81641e+03_rb,0.10743e+04_rb /)
      kbo(:, 4,42, 7) = (/ &
        & 0.99596e+03_rb,0.75867e+03_rb,0.56075e+03_rb,0.81559e+03_rb,0.10734e+04_rb /)
      kbo(:, 5,42, 7) = (/ &
        & 0.99595e+03_rb,0.75866e+03_rb,0.55981e+03_rb,0.81449e+03_rb,0.10721e+04_rb /)
      kbo(:, 1,43, 7) = (/ &
        & 0.99596e+03_rb,0.75867e+03_rb,0.56235e+03_rb,0.81734e+03_rb,0.10754e+04_rb /)
      kbo(:, 2,43, 7) = (/ &
        & 0.99595e+03_rb,0.75866e+03_rb,0.56212e+03_rb,0.81712e+03_rb,0.10751e+04_rb /)
      kbo(:, 3,43, 7) = (/ &
        & 0.99594e+03_rb,0.75865e+03_rb,0.56167e+03_rb,0.81662e+03_rb,0.10746e+04_rb /)
      kbo(:, 4,43, 7) = (/ &
        & 0.99592e+03_rb,0.75864e+03_rb,0.56100e+03_rb,0.81588e+03_rb,0.10737e+04_rb /)
      kbo(:, 5,43, 7) = (/ &
        & 0.99591e+03_rb,0.75863e+03_rb,0.56012e+03_rb,0.81487e+03_rb,0.10726e+04_rb /)
      kbo(:, 1,44, 7) = (/ &
        & 0.99592e+03_rb,0.75864e+03_rb,0.56237e+03_rb,0.81736e+03_rb,0.10754e+04_rb /)
      kbo(:, 2,44, 7) = (/ &
        & 0.99591e+03_rb,0.75863e+03_rb,0.56222e+03_rb,0.81722e+03_rb,0.10752e+04_rb /)
      kbo(:, 3,44, 7) = (/ &
        & 0.99591e+03_rb,0.75862e+03_rb,0.56184e+03_rb,0.81680e+03_rb,0.10748e+04_rb /)
      kbo(:, 4,44, 7) = (/ &
        & 0.99589e+03_rb,0.75861e+03_rb,0.56125e+03_rb,0.81614e+03_rb,0.10741e+04_rb /)
      kbo(:, 5,44, 7) = (/ &
        & 0.99588e+03_rb,0.75861e+03_rb,0.56044e+03_rb,0.81523e+03_rb,0.10730e+04_rb /)
      kbo(:, 1,45, 7) = (/ &
        & 0.99589e+03_rb,0.75861e+03_rb,0.56237e+03_rb,0.81733e+03_rb,0.10754e+04_rb /)
      kbo(:, 2,45, 7) = (/ &
        & 0.99588e+03_rb,0.75861e+03_rb,0.56230e+03_rb,0.81729e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,45, 7) = (/ &
        & 0.99587e+03_rb,0.75860e+03_rb,0.56199e+03_rb,0.81699e+03_rb,0.10750e+04_rb /)
      kbo(:, 4,45, 7) = (/ &
        & 0.99586e+03_rb,0.75859e+03_rb,0.56147e+03_rb,0.81641e+03_rb,0.10743e+04_rb /)
      kbo(:, 5,45, 7) = (/ &
        & 0.99585e+03_rb,0.75859e+03_rb,0.56074e+03_rb,0.81557e+03_rb,0.10734e+04_rb /)
      kbo(:, 1,46, 7) = (/ &
        & 0.99586e+03_rb,0.75859e+03_rb,0.56233e+03_rb,0.81728e+03_rb,0.10753e+04_rb /)
      kbo(:, 2,46, 7) = (/ &
        & 0.99585e+03_rb,0.75859e+03_rb,0.56235e+03_rb,0.81734e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,46, 7) = (/ &
        & 0.99585e+03_rb,0.75858e+03_rb,0.56213e+03_rb,0.81712e+03_rb,0.10751e+04_rb /)
      kbo(:, 4,46, 7) = (/ &
        & 0.99584e+03_rb,0.75858e+03_rb,0.56169e+03_rb,0.81663e+03_rb,0.10746e+04_rb /)
      kbo(:, 5,46, 7) = (/ &
        & 0.99583e+03_rb,0.75858e+03_rb,0.56103e+03_rb,0.81590e+03_rb,0.10738e+04_rb /)
      kbo(:, 1,47, 7) = (/ &
        & 0.99585e+03_rb,0.75858e+03_rb,0.56226e+03_rb,0.81718e+03_rb,0.10753e+04_rb /)
      kbo(:, 2,47, 7) = (/ &
        & 0.99583e+03_rb,0.75857e+03_rb,0.56237e+03_rb,0.81736e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,47, 7) = (/ &
        & 0.99583e+03_rb,0.75857e+03_rb,0.56224e+03_rb,0.81723e+03_rb,0.10752e+04_rb /)
      kbo(:, 4,47, 7) = (/ &
        & 0.99582e+03_rb,0.75856e+03_rb,0.56189e+03_rb,0.81686e+03_rb,0.10748e+04_rb /)
      kbo(:, 5,47, 7) = (/ &
        & 0.99581e+03_rb,0.75856e+03_rb,0.56132e+03_rb,0.81626e+03_rb,0.10741e+04_rb /)
      kbo(:, 1,48, 7) = (/ &
        & 0.99582e+03_rb,0.75856e+03_rb,0.56215e+03_rb,0.81703e+03_rb,0.10751e+04_rb /)
      kbo(:, 2,48, 7) = (/ &
        & 0.99582e+03_rb,0.75856e+03_rb,0.56235e+03_rb,0.81732e+03_rb,0.10754e+04_rb /)
      kbo(:, 3,48, 7) = (/ &
        & 0.99581e+03_rb,0.75855e+03_rb,0.56232e+03_rb,0.81733e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,48, 7) = (/ &
        & 0.99580e+03_rb,0.75855e+03_rb,0.56206e+03_rb,0.81704e+03_rb,0.10750e+04_rb /)
      kbo(:, 5,48, 7) = (/ &
        & 0.99580e+03_rb,0.75855e+03_rb,0.56157e+03_rb,0.81651e+03_rb,0.10744e+04_rb /)
      kbo(:, 1,49, 7) = (/ &
        & 0.99581e+03_rb,0.75855e+03_rb,0.56199e+03_rb,0.81684e+03_rb,0.10748e+04_rb /)
      kbo(:, 2,49, 7) = (/ &
        & 0.99580e+03_rb,0.75855e+03_rb,0.56230e+03_rb,0.81724e+03_rb,0.10753e+04_rb /)
      kbo(:, 3,49, 7) = (/ &
        & 0.99580e+03_rb,0.75854e+03_rb,0.56236e+03_rb,0.81736e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,49, 7) = (/ &
        & 0.99579e+03_rb,0.75854e+03_rb,0.56219e+03_rb,0.81718e+03_rb,0.10752e+04_rb /)
      kbo(:, 5,49, 7) = (/ &
        & 0.99579e+03_rb,0.75854e+03_rb,0.56179e+03_rb,0.81678e+03_rb,0.10747e+04_rb /)
      kbo(:, 1,50, 7) = (/ &
        & 0.99579e+03_rb,0.75854e+03_rb,0.56180e+03_rb,0.81662e+03_rb,0.10745e+04_rb /)
      kbo(:, 2,50, 7) = (/ &
        & 0.99579e+03_rb,0.75854e+03_rb,0.56221e+03_rb,0.81713e+03_rb,0.10752e+04_rb /)
      kbo(:, 3,50, 7) = (/ &
        & 0.99579e+03_rb,0.75854e+03_rb,0.56237e+03_rb,0.81735e+03_rb,0.10754e+04_rb /)
      kbo(:, 4,50, 7) = (/ &
        & 0.99579e+03_rb,0.75854e+03_rb,0.56228e+03_rb,0.81728e+03_rb,0.10753e+04_rb /)
      kbo(:, 5,50, 7) = (/ &
        & 0.99578e+03_rb,0.75853e+03_rb,0.56197e+03_rb,0.81695e+03_rb,0.10749e+04_rb /)
      kbo(:, 1,51, 7) = (/ &
        & 0.99578e+03_rb,0.75853e+03_rb,0.56158e+03_rb,0.81637e+03_rb,0.10741e+04_rb /)
      kbo(:, 2,51, 7) = (/ &
        & 0.99578e+03_rb,0.75853e+03_rb,0.56209e+03_rb,0.81697e+03_rb,0.10750e+04_rb /)
      kbo(:, 3,51, 7) = (/ &
        & 0.99578e+03_rb,0.75853e+03_rb,0.56234e+03_rb,0.81729e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,51, 7) = (/ &
        & 0.99578e+03_rb,0.75853e+03_rb,0.56234e+03_rb,0.81734e+03_rb,0.10754e+04_rb /)
      kbo(:, 5,51, 7) = (/ &
        & 0.99577e+03_rb,0.75852e+03_rb,0.56211e+03_rb,0.81712e+03_rb,0.10751e+04_rb /)
      kbo(:, 1,52, 7) = (/ &
        & 0.99577e+03_rb,0.75853e+03_rb,0.56131e+03_rb,0.81608e+03_rb,0.10737e+04_rb /)
      kbo(:, 2,52, 7) = (/ &
        & 0.99577e+03_rb,0.75852e+03_rb,0.56194e+03_rb,0.81678e+03_rb,0.10748e+04_rb /)
      kbo(:, 3,52, 7) = (/ &
        & 0.99577e+03_rb,0.75852e+03_rb,0.56228e+03_rb,0.81721e+03_rb,0.10753e+04_rb /)
      kbo(:, 4,52, 7) = (/ &
        & 0.99577e+03_rb,0.75852e+03_rb,0.56237e+03_rb,0.81738e+03_rb,0.10754e+04_rb /)
      kbo(:, 5,52, 7) = (/ &
        & 0.99576e+03_rb,0.75852e+03_rb,0.56222e+03_rb,0.81722e+03_rb,0.10752e+04_rb /)
      kbo(:, 1,53, 7) = (/ &
        & 0.99577e+03_rb,0.75852e+03_rb,0.56102e+03_rb,0.81574e+03_rb,0.10731e+04_rb /)
      kbo(:, 2,53, 7) = (/ &
        & 0.99576e+03_rb,0.75852e+03_rb,0.56174e+03_rb,0.81656e+03_rb,0.10744e+04_rb /)
      kbo(:, 3,53, 7) = (/ &
        & 0.99576e+03_rb,0.75852e+03_rb,0.56218e+03_rb,0.81709e+03_rb,0.10752e+04_rb /)
      kbo(:, 4,53, 7) = (/ &
        & 0.99576e+03_rb,0.75851e+03_rb,0.56236e+03_rb,0.81734e+03_rb,0.10754e+04_rb /)
      kbo(:, 5,53, 7) = (/ &
        & 0.99576e+03_rb,0.75852e+03_rb,0.56230e+03_rb,0.81736e+03_rb,0.10753e+04_rb /)
      kbo(:, 1,54, 7) = (/ &
        & 0.99576e+03_rb,0.75852e+03_rb,0.56072e+03_rb,0.81536e+03_rb,0.10726e+04_rb /)
      kbo(:, 2,54, 7) = (/ &
        & 0.99576e+03_rb,0.75851e+03_rb,0.56153e+03_rb,0.81633e+03_rb,0.10740e+04_rb /)
      kbo(:, 3,54, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56207e+03_rb,0.81694e+03_rb,0.10750e+04_rb /)
      kbo(:, 4,54, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56233e+03_rb,0.81728e+03_rb,0.10753e+04_rb /)
      kbo(:, 5,54, 7) = (/ &
        & 0.99576e+03_rb,0.75851e+03_rb,0.56235e+03_rb,0.81734e+03_rb,0.10754e+04_rb /)
      kbo(:, 1,55, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56039e+03_rb,0.81495e+03_rb,0.10720e+04_rb /)
      kbo(:, 2,55, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56128e+03_rb,0.81606e+03_rb,0.10736e+04_rb /)
      kbo(:, 3,55, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56193e+03_rb,0.81677e+03_rb,0.10747e+04_rb /)
      kbo(:, 4,55, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56227e+03_rb,0.81720e+03_rb,0.10753e+04_rb /)
      kbo(:, 5,55, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56237e+03_rb,0.81736e+03_rb,0.10754e+04_rb /)
      kbo(:, 1,56, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56002e+03_rb,0.81448e+03_rb,0.10713e+04_rb /)
      kbo(:, 2,56, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56104e+03_rb,0.81576e+03_rb,0.10732e+04_rb /)
      kbo(:, 3,56, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.56175e+03_rb,0.81659e+03_rb,0.10744e+04_rb /)
      kbo(:, 4,56, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56219e+03_rb,0.81709e+03_rb,0.10752e+04_rb /)
      kbo(:, 5,56, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56236e+03_rb,0.81734e+03_rb,0.10754e+04_rb /)
      kbo(:, 1,57, 7) = (/ &
        & 0.99575e+03_rb,0.75851e+03_rb,0.55963e+03_rb,0.81395e+03_rb,0.10705e+04_rb /)
      kbo(:, 2,57, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56076e+03_rb,0.81540e+03_rb,0.10727e+04_rb /)
      kbo(:, 3,57, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56157e+03_rb,0.81635e+03_rb,0.10741e+04_rb /)
      kbo(:, 4,57, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56208e+03_rb,0.81696e+03_rb,0.10750e+04_rb /)
      kbo(:, 5,57, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56234e+03_rb,0.81729e+03_rb,0.10754e+04_rb /)
      kbo(:, 1,58, 7) = (/ &
        & 0.99575e+03_rb,0.75850e+03_rb,0.55922e+03_rb,0.81340e+03_rb,0.10697e+04_rb /)
      kbo(:, 2,58, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56044e+03_rb,0.81503e+03_rb,0.10721e+04_rb /)
      kbo(:, 3,58, 7) = (/ &
        & 0.99574e+03_rb,0.75851e+03_rb,0.56134e+03_rb,0.81611e+03_rb,0.10737e+04_rb /)
      kbo(:, 4,58, 7) = (/ &
        & 0.99574e+03_rb,0.75851e+03_rb,0.56195e+03_rb,0.81681e+03_rb,0.10748e+04_rb /)
      kbo(:, 5,58, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56228e+03_rb,0.81722e+03_rb,0.10753e+04_rb /)
      kbo(:, 1,59, 7) = (/ &
        & 0.99575e+03_rb,0.75850e+03_rb,0.55905e+03_rb,0.81317e+03_rb,0.10694e+04_rb /)
      kbo(:, 2,59, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56032e+03_rb,0.81485e+03_rb,0.10718e+04_rb /)
      kbo(:, 3,59, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56125e+03_rb,0.81601e+03_rb,0.10735e+04_rb /)
      kbo(:, 4,59, 7) = (/ &
        & 0.99574e+03_rb,0.75850e+03_rb,0.56189e+03_rb,0.81673e+03_rb,0.10747e+04_rb /)
      kbo(:, 5,59, 7) = (/ &
        & 0.99573e+03_rb,0.75850e+03_rb,0.56225e+03_rb,0.81718e+03_rb,0.10753e+04_rb /)
      kbo(:, 1,13, 8) = (/ &
        & 0.11451e+04_rb,0.86748e+03_rb,0.59259e+03_rb,0.85187e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,13, 8) = (/ &
        & 0.11363e+04_rb,0.86085e+03_rb,0.58896e+03_rb,0.85188e+03_rb,0.11226e+04_rb /)
      kbo(:, 3,13, 8) = (/ &
        & 0.11286e+04_rb,0.85515e+03_rb,0.58586e+03_rb,0.85170e+03_rb,0.11227e+04_rb /)
      kbo(:, 4,13, 8) = (/ &
        & 0.11220e+04_rb,0.85016e+03_rb,0.58378e+03_rb,0.85132e+03_rb,0.11224e+04_rb /)
      kbo(:, 5,13, 8) = (/ &
        & 0.11162e+04_rb,0.84582e+03_rb,0.58229e+03_rb,0.85074e+03_rb,0.11218e+04_rb /)
      kbo(:, 1,14, 8) = (/ &
        & 0.11230e+04_rb,0.85087e+03_rb,0.58392e+03_rb,0.85168e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,14, 8) = (/ &
        & 0.11158e+04_rb,0.84552e+03_rb,0.58280e+03_rb,0.85169e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,14, 8) = (/ &
        & 0.11097e+04_rb,0.84090e+03_rb,0.58233e+03_rb,0.85151e+03_rb,0.11227e+04_rb /)
      kbo(:, 4,14, 8) = (/ &
        & 0.11043e+04_rb,0.83689e+03_rb,0.58194e+03_rb,0.85113e+03_rb,0.11223e+04_rb /)
      kbo(:, 5,14, 8) = (/ &
        & 0.10996e+04_rb,0.83336e+03_rb,0.58140e+03_rb,0.85052e+03_rb,0.11217e+04_rb /)
      kbo(:, 1,15, 8) = (/ &
        & 0.11049e+04_rb,0.83731e+03_rb,0.58207e+03_rb,0.85153e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,15, 8) = (/ &
        & 0.10991e+04_rb,0.83301e+03_rb,0.58217e+03_rb,0.85153e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,15, 8) = (/ &
        & 0.10942e+04_rb,0.82928e+03_rb,0.58200e+03_rb,0.85134e+03_rb,0.11226e+04_rb /)
      kbo(:, 4,15, 8) = (/ &
        & 0.10899e+04_rb,0.82605e+03_rb,0.58161e+03_rb,0.85094e+03_rb,0.11223e+04_rb /)
      kbo(:, 5,15, 8) = (/ &
        & 0.10860e+04_rb,0.82320e+03_rb,0.58108e+03_rb,0.85034e+03_rb,0.11216e+04_rb /)
      kbo(:, 1,16, 8) = (/ &
        & 0.10902e+04_rb,0.82630e+03_rb,0.58181e+03_rb,0.85141e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,16, 8) = (/ &
        & 0.10856e+04_rb,0.82283e+03_rb,0.58190e+03_rb,0.85139e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,16, 8) = (/ &
        & 0.10816e+04_rb,0.81983e+03_rb,0.58172e+03_rb,0.85119e+03_rb,0.11226e+04_rb /)
      kbo(:, 4,16, 8) = (/ &
        & 0.10781e+04_rb,0.81721e+03_rb,0.58133e+03_rb,0.85078e+03_rb,0.11222e+04_rb /)
      kbo(:, 5,16, 8) = (/ &
        & 0.10750e+04_rb,0.81491e+03_rb,0.58080e+03_rb,0.85015e+03_rb,0.11215e+04_rb /)
      kbo(:, 1,17, 8) = (/ &
        & 0.10783e+04_rb,0.81738e+03_rb,0.58160e+03_rb,0.85131e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,17, 8) = (/ &
        & 0.10746e+04_rb,0.81457e+03_rb,0.58168e+03_rb,0.85129e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,17, 8) = (/ &
        & 0.10713e+04_rb,0.81215e+03_rb,0.58149e+03_rb,0.85108e+03_rb,0.11226e+04_rb /)
      kbo(:, 4,17, 8) = (/ &
        & 0.10685e+04_rb,0.81004e+03_rb,0.58109e+03_rb,0.85064e+03_rb,0.11222e+04_rb /)
      kbo(:, 5,17, 8) = (/ &
        & 0.10660e+04_rb,0.80818e+03_rb,0.58056e+03_rb,0.85000e+03_rb,0.11214e+04_rb /)
      kbo(:, 1,18, 8) = (/ &
        & 0.10686e+04_rb,0.81013e+03_rb,0.58144e+03_rb,0.85124e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,18, 8) = (/ &
        & 0.10656e+04_rb,0.80786e+03_rb,0.58150e+03_rb,0.85119e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,18, 8) = (/ &
        & 0.10630e+04_rb,0.80591e+03_rb,0.58129e+03_rb,0.85095e+03_rb,0.11226e+04_rb /)
      kbo(:, 4,18, 8) = (/ &
        & 0.10607e+04_rb,0.80419e+03_rb,0.58089e+03_rb,0.85052e+03_rb,0.11221e+04_rb /)
      kbo(:, 5,18, 8) = (/ &
        & 0.10587e+04_rb,0.80269e+03_rb,0.58036e+03_rb,0.84984e+03_rb,0.11213e+04_rb /)
      kbo(:, 1,19, 8) = (/ &
        & 0.10608e+04_rb,0.80424e+03_rb,0.58130e+03_rb,0.85117e+03_rb,0.11226e+04_rb /)
      kbo(:, 2,19, 8) = (/ &
        & 0.10584e+04_rb,0.80242e+03_rb,0.58135e+03_rb,0.85109e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,19, 8) = (/ &
        & 0.10562e+04_rb,0.80084e+03_rb,0.58113e+03_rb,0.85087e+03_rb,0.11225e+04_rb /)
      kbo(:, 4,19, 8) = (/ &
        & 0.10544e+04_rb,0.79946e+03_rb,0.58072e+03_rb,0.85040e+03_rb,0.11220e+04_rb /)
      kbo(:, 5,19, 8) = (/ &
        & 0.10528e+04_rb,0.79825e+03_rb,0.58019e+03_rb,0.84971e+03_rb,0.11212e+04_rb /)
      kbo(:, 1,20, 8) = (/ &
        & 0.10544e+04_rb,0.79943e+03_rb,0.58120e+03_rb,0.85111e+03_rb,0.11226e+04_rb /)
      kbo(:, 2,20, 8) = (/ &
        & 0.10524e+04_rb,0.79797e+03_rb,0.58122e+03_rb,0.85103e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,20, 8) = (/ &
        & 0.10507e+04_rb,0.79669e+03_rb,0.58098e+03_rb,0.85078e+03_rb,0.11225e+04_rb /)
      kbo(:, 4,20, 8) = (/ &
        & 0.10492e+04_rb,0.79558e+03_rb,0.58056e+03_rb,0.85028e+03_rb,0.11220e+04_rb /)
      kbo(:, 5,20, 8) = (/ &
        & 0.10479e+04_rb,0.79460e+03_rb,0.58002e+03_rb,0.84958e+03_rb,0.11211e+04_rb /)
      kbo(:, 1,21, 8) = (/ &
        & 0.10492e+04_rb,0.79553e+03_rb,0.58112e+03_rb,0.85107e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,21, 8) = (/ &
        & 0.10476e+04_rb,0.79436e+03_rb,0.58112e+03_rb,0.85096e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,21, 8) = (/ &
        & 0.10462e+04_rb,0.79334e+03_rb,0.58085e+03_rb,0.85069e+03_rb,0.11224e+04_rb /)
      kbo(:, 4,21, 8) = (/ &
        & 0.10450e+04_rb,0.79243e+03_rb,0.58042e+03_rb,0.85021e+03_rb,0.11219e+04_rb /)
      kbo(:, 5,21, 8) = (/ &
        & 0.10440e+04_rb,0.79165e+03_rb,0.57987e+03_rb,0.84945e+03_rb,0.11210e+04_rb /)
      kbo(:, 1,22, 8) = (/ &
        & 0.10449e+04_rb,0.79230e+03_rb,0.58106e+03_rb,0.85103e+03_rb,0.11226e+04_rb /)
      kbo(:, 2,22, 8) = (/ &
        & 0.10436e+04_rb,0.79136e+03_rb,0.58100e+03_rb,0.85090e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,22, 8) = (/ &
        & 0.10425e+04_rb,0.79054e+03_rb,0.58071e+03_rb,0.85058e+03_rb,0.11224e+04_rb /)
      kbo(:, 4,22, 8) = (/ &
        & 0.10416e+04_rb,0.78983e+03_rb,0.58026e+03_rb,0.85002e+03_rb,0.11217e+04_rb /)
      kbo(:, 5,22, 8) = (/ &
        & 0.10407e+04_rb,0.78921e+03_rb,0.57969e+03_rb,0.84925e+03_rb,0.11208e+04_rb /)
      kbo(:, 1,23, 8) = (/ &
        & 0.10414e+04_rb,0.78969e+03_rb,0.58100e+03_rb,0.85099e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,23, 8) = (/ &
        & 0.10404e+04_rb,0.78894e+03_rb,0.58090e+03_rb,0.85083e+03_rb,0.11226e+04_rb /)
      kbo(:, 3,23, 8) = (/ &
        & 0.10395e+04_rb,0.78829e+03_rb,0.58058e+03_rb,0.85047e+03_rb,0.11223e+04_rb /)
      kbo(:, 4,23, 8) = (/ &
        & 0.10387e+04_rb,0.78772e+03_rb,0.58011e+03_rb,0.84989e+03_rb,0.11216e+04_rb /)
      kbo(:, 5,23, 8) = (/ &
        & 0.10381e+04_rb,0.78721e+03_rb,0.57952e+03_rb,0.84905e+03_rb,0.11206e+04_rb /)
      kbo(:, 1,24, 8) = (/ &
        & 0.10386e+04_rb,0.78759e+03_rb,0.58095e+03_rb,0.85095e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,24, 8) = (/ &
        & 0.10378e+04_rb,0.78698e+03_rb,0.58081e+03_rb,0.85075e+03_rb,0.11226e+04_rb /)
      kbo(:, 3,24, 8) = (/ &
        & 0.10371e+04_rb,0.78646e+03_rb,0.58046e+03_rb,0.85035e+03_rb,0.11222e+04_rb /)
      kbo(:, 4,24, 8) = (/ &
        & 0.10365e+04_rb,0.78601e+03_rb,0.57995e+03_rb,0.84971e+03_rb,0.11214e+04_rb /)
      kbo(:, 5,24, 8) = (/ &
        & 0.10359e+04_rb,0.78561e+03_rb,0.57934e+03_rb,0.84885e+03_rb,0.11203e+04_rb /)
      kbo(:, 1,25, 8) = (/ &
        & 0.10363e+04_rb,0.78588e+03_rb,0.58090e+03_rb,0.85090e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,25, 8) = (/ &
        & 0.10357e+04_rb,0.78541e+03_rb,0.58070e+03_rb,0.85067e+03_rb,0.11225e+04_rb /)
      kbo(:, 3,25, 8) = (/ &
        & 0.10351e+04_rb,0.78498e+03_rb,0.58032e+03_rb,0.85020e+03_rb,0.11220e+04_rb /)
      kbo(:, 4,25, 8) = (/ &
        & 0.10346e+04_rb,0.78461e+03_rb,0.57980e+03_rb,0.84953e+03_rb,0.11212e+04_rb /)
      kbo(:, 5,25, 8) = (/ &
        & 0.10342e+04_rb,0.78430e+03_rb,0.57916e+03_rb,0.84863e+03_rb,0.11201e+04_rb /)
      kbo(:, 1,26, 8) = (/ &
        & 0.10345e+04_rb,0.78450e+03_rb,0.58084e+03_rb,0.85084e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,26, 8) = (/ &
        & 0.10339e+04_rb,0.78411e+03_rb,0.58059e+03_rb,0.85057e+03_rb,0.11224e+04_rb /)
      kbo(:, 3,26, 8) = (/ &
        & 0.10335e+04_rb,0.78378e+03_rb,0.58018e+03_rb,0.85004e+03_rb,0.11219e+04_rb /)
      kbo(:, 4,26, 8) = (/ &
        & 0.10331e+04_rb,0.78349e+03_rb,0.57964e+03_rb,0.84934e+03_rb,0.11210e+04_rb /)
      kbo(:, 5,26, 8) = (/ &
        & 0.10328e+04_rb,0.78323e+03_rb,0.57897e+03_rb,0.84838e+03_rb,0.11198e+04_rb /)
      kbo(:, 1,27, 8) = (/ &
        & 0.10329e+04_rb,0.78338e+03_rb,0.58078e+03_rb,0.85078e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,27, 8) = (/ &
        & 0.10326e+04_rb,0.78307e+03_rb,0.58048e+03_rb,0.85045e+03_rb,0.11223e+04_rb /)
      kbo(:, 3,27, 8) = (/ &
        & 0.10322e+04_rb,0.78282e+03_rb,0.58003e+03_rb,0.84982e+03_rb,0.11217e+04_rb /)
      kbo(:, 4,27, 8) = (/ &
        & 0.10319e+04_rb,0.78258e+03_rb,0.57947e+03_rb,0.84907e+03_rb,0.11207e+04_rb /)
      kbo(:, 5,27, 8) = (/ &
        & 0.10316e+04_rb,0.78237e+03_rb,0.57877e+03_rb,0.84811e+03_rb,0.11194e+04_rb /)
      kbo(:, 1,28, 8) = (/ &
        & 0.10318e+04_rb,0.78248e+03_rb,0.58069e+03_rb,0.85072e+03_rb,0.11226e+04_rb /)
      kbo(:, 2,28, 8) = (/ &
        & 0.10314e+04_rb,0.78225e+03_rb,0.58037e+03_rb,0.85030e+03_rb,0.11222e+04_rb /)
      kbo(:, 3,28, 8) = (/ &
        & 0.10312e+04_rb,0.78205e+03_rb,0.57989e+03_rb,0.84960e+03_rb,0.11215e+04_rb /)
      kbo(:, 4,28, 8) = (/ &
        & 0.10309e+04_rb,0.78185e+03_rb,0.57929e+03_rb,0.84888e+03_rb,0.11204e+04_rb /)
      kbo(:, 5,28, 8) = (/ &
        & 0.10307e+04_rb,0.78168e+03_rb,0.57856e+03_rb,0.84784e+03_rb,0.11191e+04_rb /)
      kbo(:, 1,29, 8) = (/ &
        & 0.10308e+04_rb,0.78175e+03_rb,0.58061e+03_rb,0.85062e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,29, 8) = (/ &
        & 0.10305e+04_rb,0.78157e+03_rb,0.58024e+03_rb,0.85019e+03_rb,0.11221e+04_rb /)
      kbo(:, 3,29, 8) = (/ &
        & 0.10303e+04_rb,0.78140e+03_rb,0.57974e+03_rb,0.84950e+03_rb,0.11213e+04_rb /)
      kbo(:, 4,29, 8) = (/ &
        & 0.10301e+04_rb,0.78126e+03_rb,0.57911e+03_rb,0.84863e+03_rb,0.11201e+04_rb /)
      kbo(:, 5,29, 8) = (/ &
        & 0.10300e+04_rb,0.78112e+03_rb,0.57834e+03_rb,0.84752e+03_rb,0.11188e+04_rb /)
      kbo(:, 1,30, 8) = (/ &
        & 0.10300e+04_rb,0.78117e+03_rb,0.58052e+03_rb,0.85055e+03_rb,0.11224e+04_rb /)
      kbo(:, 2,30, 8) = (/ &
        & 0.10298e+04_rb,0.78102e+03_rb,0.58011e+03_rb,0.85001e+03_rb,0.11219e+04_rb /)
      kbo(:, 3,30, 8) = (/ &
        & 0.10296e+04_rb,0.78089e+03_rb,0.57958e+03_rb,0.84932e+03_rb,0.11210e+04_rb /)
      kbo(:, 4,30, 8) = (/ &
        & 0.10295e+04_rb,0.78077e+03_rb,0.57892e+03_rb,0.84838e+03_rb,0.11198e+04_rb /)
      kbo(:, 5,30, 8) = (/ &
        & 0.10294e+04_rb,0.78066e+03_rb,0.57811e+03_rb,0.84720e+03_rb,0.11184e+04_rb /)
      kbo(:, 1,31, 8) = (/ &
        & 0.10294e+04_rb,0.78070e+03_rb,0.58042e+03_rb,0.85041e+03_rb,0.11223e+04_rb /)
      kbo(:, 2,31, 8) = (/ &
        & 0.10292e+04_rb,0.78058e+03_rb,0.57998e+03_rb,0.84985e+03_rb,0.11217e+04_rb /)
      kbo(:, 3,31, 8) = (/ &
        & 0.10291e+04_rb,0.78049e+03_rb,0.57941e+03_rb,0.84908e+03_rb,0.11207e+04_rb /)
      kbo(:, 4,31, 8) = (/ &
        & 0.10290e+04_rb,0.78039e+03_rb,0.57872e+03_rb,0.84808e+03_rb,0.11195e+04_rb /)
      kbo(:, 5,31, 8) = (/ &
        & 0.10289e+04_rb,0.78030e+03_rb,0.57786e+03_rb,0.84686e+03_rb,0.11179e+04_rb /)
      kbo(:, 1,32, 8) = (/ &
        & 0.10289e+04_rb,0.78033e+03_rb,0.58031e+03_rb,0.85030e+03_rb,0.11222e+04_rb /)
      kbo(:, 2,32, 8) = (/ &
        & 0.10288e+04_rb,0.78024e+03_rb,0.57984e+03_rb,0.84967e+03_rb,0.11215e+04_rb /)
      kbo(:, 3,32, 8) = (/ &
        & 0.10287e+04_rb,0.78016e+03_rb,0.57924e+03_rb,0.84884e+03_rb,0.11204e+04_rb /)
      kbo(:, 4,32, 8) = (/ &
        & 0.10286e+04_rb,0.78009e+03_rb,0.57851e+03_rb,0.84778e+03_rb,0.11191e+04_rb /)
      kbo(:, 5,32, 8) = (/ &
        & 0.10285e+04_rb,0.78000e+03_rb,0.57759e+03_rb,0.84651e+03_rb,0.11175e+04_rb /)
      kbo(:, 1,33, 8) = (/ &
        & 0.10285e+04_rb,0.78003e+03_rb,0.58019e+03_rb,0.85016e+03_rb,0.11220e+04_rb /)
      kbo(:, 2,33, 8) = (/ &
        & 0.10284e+04_rb,0.77995e+03_rb,0.57968e+03_rb,0.84947e+03_rb,0.11212e+04_rb /)
      kbo(:, 3,33, 8) = (/ &
        & 0.10283e+04_rb,0.77988e+03_rb,0.57905e+03_rb,0.84858e+03_rb,0.11201e+04_rb /)
      kbo(:, 4,33, 8) = (/ &
        & 0.10282e+04_rb,0.77982e+03_rb,0.57829e+03_rb,0.84747e+03_rb,0.11187e+04_rb /)
      kbo(:, 5,33, 8) = (/ &
        & 0.10282e+04_rb,0.77978e+03_rb,0.57730e+03_rb,0.84615e+03_rb,0.11170e+04_rb /)
      kbo(:, 1,34, 8) = (/ &
        & 0.10282e+04_rb,0.77978e+03_rb,0.58008e+03_rb,0.84998e+03_rb,0.11219e+04_rb /)
      kbo(:, 2,34, 8) = (/ &
        & 0.10281e+04_rb,0.77972e+03_rb,0.57954e+03_rb,0.84930e+03_rb,0.11210e+04_rb /)
      kbo(:, 3,34, 8) = (/ &
        & 0.10280e+04_rb,0.77967e+03_rb,0.57889e+03_rb,0.84836e+03_rb,0.11198e+04_rb /)
      kbo(:, 4,34, 8) = (/ &
        & 0.10280e+04_rb,0.77962e+03_rb,0.57808e+03_rb,0.84718e+03_rb,0.11184e+04_rb /)
      kbo(:, 5,34, 8) = (/ &
        & 0.10279e+04_rb,0.77958e+03_rb,0.57705e+03_rb,0.84582e+03_rb,0.11166e+04_rb /)
      kbo(:, 1,35, 8) = (/ &
        & 0.10279e+04_rb,0.77958e+03_rb,0.58001e+03_rb,0.84995e+03_rb,0.11218e+04_rb /)
      kbo(:, 2,35, 8) = (/ &
        & 0.10278e+04_rb,0.77953e+03_rb,0.57946e+03_rb,0.84916e+03_rb,0.11209e+04_rb /)
      kbo(:, 3,35, 8) = (/ &
        & 0.10278e+04_rb,0.77951e+03_rb,0.57880e+03_rb,0.84823e+03_rb,0.11196e+04_rb /)
      kbo(:, 4,35, 8) = (/ &
        & 0.10277e+04_rb,0.77946e+03_rb,0.57796e+03_rb,0.84701e+03_rb,0.11181e+04_rb /)
      kbo(:, 5,35, 8) = (/ &
        & 0.10277e+04_rb,0.77943e+03_rb,0.57689e+03_rb,0.84563e+03_rb,0.11163e+04_rb /)
      kbo(:, 1,36, 8) = (/ &
        & 0.10277e+04_rb,0.77944e+03_rb,0.58000e+03_rb,0.84992e+03_rb,0.11218e+04_rb /)
      kbo(:, 2,36, 8) = (/ &
        & 0.10277e+04_rb,0.77940e+03_rb,0.57944e+03_rb,0.84917e+03_rb,0.11208e+04_rb /)
      kbo(:, 3,36, 8) = (/ &
        & 0.10276e+04_rb,0.77936e+03_rb,0.57877e+03_rb,0.84819e+03_rb,0.11196e+04_rb /)
      kbo(:, 4,36, 8) = (/ &
        & 0.10276e+04_rb,0.77933e+03_rb,0.57793e+03_rb,0.84697e+03_rb,0.11181e+04_rb /)
      kbo(:, 5,36, 8) = (/ &
        & 0.10275e+04_rb,0.77931e+03_rb,0.57686e+03_rb,0.84561e+03_rb,0.11162e+04_rb /)
      kbo(:, 1,37, 8) = (/ &
        & 0.10275e+04_rb,0.77932e+03_rb,0.58006e+03_rb,0.85002e+03_rb,0.11219e+04_rb /)
      kbo(:, 2,37, 8) = (/ &
        & 0.10275e+04_rb,0.77929e+03_rb,0.57952e+03_rb,0.84927e+03_rb,0.11210e+04_rb /)
      kbo(:, 3,37, 8) = (/ &
        & 0.10275e+04_rb,0.77926e+03_rb,0.57886e+03_rb,0.84832e+03_rb,0.11198e+04_rb /)
      kbo(:, 4,37, 8) = (/ &
        & 0.10274e+04_rb,0.77924e+03_rb,0.57805e+03_rb,0.84715e+03_rb,0.11183e+04_rb /)
      kbo(:, 5,37, 8) = (/ &
        & 0.10274e+04_rb,0.77921e+03_rb,0.57701e+03_rb,0.84575e+03_rb,0.11165e+04_rb /)
      kbo(:, 1,38, 8) = (/ &
        & 0.10274e+04_rb,0.77922e+03_rb,0.58012e+03_rb,0.85009e+03_rb,0.11220e+04_rb /)
      kbo(:, 2,38, 8) = (/ &
        & 0.10274e+04_rb,0.77920e+03_rb,0.57959e+03_rb,0.84938e+03_rb,0.11211e+04_rb /)
      kbo(:, 3,38, 8) = (/ &
        & 0.10274e+04_rb,0.77918e+03_rb,0.57896e+03_rb,0.84848e+03_rb,0.11200e+04_rb /)
      kbo(:, 4,38, 8) = (/ &
        & 0.10273e+04_rb,0.77915e+03_rb,0.57818e+03_rb,0.84731e+03_rb,0.11185e+04_rb /)
      kbo(:, 5,38, 8) = (/ &
        & 0.10273e+04_rb,0.77912e+03_rb,0.57716e+03_rb,0.84599e+03_rb,0.11168e+04_rb /)
      kbo(:, 1,39, 8) = (/ &
        & 0.10273e+04_rb,0.77914e+03_rb,0.58018e+03_rb,0.85016e+03_rb,0.11221e+04_rb /)
      kbo(:, 2,39, 8) = (/ &
        & 0.10273e+04_rb,0.77912e+03_rb,0.57968e+03_rb,0.84939e+03_rb,0.11213e+04_rb /)
      kbo(:, 3,39, 8) = (/ &
        & 0.10273e+04_rb,0.77911e+03_rb,0.57905e+03_rb,0.84861e+03_rb,0.11201e+04_rb /)
      kbo(:, 4,39, 8) = (/ &
        & 0.10272e+04_rb,0.77910e+03_rb,0.57829e+03_rb,0.84750e+03_rb,0.11187e+04_rb /)
      kbo(:, 5,39, 8) = (/ &
        & 0.10272e+04_rb,0.77907e+03_rb,0.57732e+03_rb,0.84617e+03_rb,0.11171e+04_rb /)
      kbo(:, 1,40, 8) = (/ &
        & 0.10272e+04_rb,0.77908e+03_rb,0.58028e+03_rb,0.85029e+03_rb,0.11222e+04_rb /)
      kbo(:, 2,40, 8) = (/ &
        & 0.10272e+04_rb,0.77906e+03_rb,0.57981e+03_rb,0.84968e+03_rb,0.11215e+04_rb /)
      kbo(:, 3,40, 8) = (/ &
        & 0.10272e+04_rb,0.77904e+03_rb,0.57922e+03_rb,0.84885e+03_rb,0.11204e+04_rb /)
      kbo(:, 4,40, 8) = (/ &
        & 0.10272e+04_rb,0.77904e+03_rb,0.57849e+03_rb,0.84779e+03_rb,0.11191e+04_rb /)
      kbo(:, 5,40, 8) = (/ &
        & 0.10272e+04_rb,0.77901e+03_rb,0.57757e+03_rb,0.84651e+03_rb,0.11176e+04_rb /)
      kbo(:, 1,41, 8) = (/ &
        & 0.10272e+04_rb,0.77903e+03_rb,0.58037e+03_rb,0.85041e+03_rb,0.11224e+04_rb /)
      kbo(:, 2,41, 8) = (/ &
        & 0.10272e+04_rb,0.77902e+03_rb,0.57994e+03_rb,0.84983e+03_rb,0.11217e+04_rb /)
      kbo(:, 3,41, 8) = (/ &
        & 0.10271e+04_rb,0.77900e+03_rb,0.57937e+03_rb,0.84906e+03_rb,0.11207e+04_rb /)
      kbo(:, 4,41, 8) = (/ &
        & 0.10271e+04_rb,0.77898e+03_rb,0.57870e+03_rb,0.84808e+03_rb,0.11195e+04_rb /)
      kbo(:, 5,41, 8) = (/ &
        & 0.10271e+04_rb,0.77898e+03_rb,0.57782e+03_rb,0.84685e+03_rb,0.11180e+04_rb /)
      kbo(:, 1,42, 8) = (/ &
        & 0.10271e+04_rb,0.77900e+03_rb,0.58046e+03_rb,0.85051e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,42, 8) = (/ &
        & 0.10271e+04_rb,0.77898e+03_rb,0.58006e+03_rb,0.85002e+03_rb,0.11219e+04_rb /)
      kbo(:, 3,42, 8) = (/ &
        & 0.10271e+04_rb,0.77896e+03_rb,0.57953e+03_rb,0.84930e+03_rb,0.11210e+04_rb /)
      kbo(:, 4,42, 8) = (/ &
        & 0.10271e+04_rb,0.77895e+03_rb,0.57888e+03_rb,0.84835e+03_rb,0.11198e+04_rb /)
      kbo(:, 5,42, 8) = (/ &
        & 0.10270e+04_rb,0.77894e+03_rb,0.57806e+03_rb,0.84718e+03_rb,0.11184e+04_rb /)
      kbo(:, 1,43, 8) = (/ &
        & 0.10271e+04_rb,0.77895e+03_rb,0.58056e+03_rb,0.85062e+03_rb,0.11226e+04_rb /)
      kbo(:, 2,43, 8) = (/ &
        & 0.10270e+04_rb,0.77895e+03_rb,0.58020e+03_rb,0.85017e+03_rb,0.11221e+04_rb /)
      kbo(:, 3,43, 8) = (/ &
        & 0.10270e+04_rb,0.77892e+03_rb,0.57970e+03_rb,0.84954e+03_rb,0.11213e+04_rb /)
      kbo(:, 4,43, 8) = (/ &
        & 0.10270e+04_rb,0.77892e+03_rb,0.57909e+03_rb,0.84866e+03_rb,0.11202e+04_rb /)
      kbo(:, 5,43, 8) = (/ &
        & 0.10270e+04_rb,0.77891e+03_rb,0.57834e+03_rb,0.84756e+03_rb,0.11188e+04_rb /)
      kbo(:, 1,44, 8) = (/ &
        & 0.10270e+04_rb,0.77892e+03_rb,0.58064e+03_rb,0.85071e+03_rb,0.11226e+04_rb /)
      kbo(:, 2,44, 8) = (/ &
        & 0.10270e+04_rb,0.77891e+03_rb,0.58034e+03_rb,0.85036e+03_rb,0.11223e+04_rb /)
      kbo(:, 3,44, 8) = (/ &
        & 0.10270e+04_rb,0.77891e+03_rb,0.57988e+03_rb,0.84979e+03_rb,0.11216e+04_rb /)
      kbo(:, 4,44, 8) = (/ &
        & 0.10270e+04_rb,0.77890e+03_rb,0.57931e+03_rb,0.84899e+03_rb,0.11206e+04_rb /)
      kbo(:, 5,44, 8) = (/ &
        & 0.10270e+04_rb,0.77888e+03_rb,0.57861e+03_rb,0.84797e+03_rb,0.11193e+04_rb /)
      kbo(:, 1,45, 8) = (/ &
        & 0.10270e+04_rb,0.77889e+03_rb,0.58069e+03_rb,0.85078e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,45, 8) = (/ &
        & 0.10270e+04_rb,0.77888e+03_rb,0.58047e+03_rb,0.85051e+03_rb,0.11224e+04_rb /)
      kbo(:, 3,45, 8) = (/ &
        & 0.10270e+04_rb,0.77888e+03_rb,0.58005e+03_rb,0.84999e+03_rb,0.11219e+04_rb /)
      kbo(:, 4,45, 8) = (/ &
        & 0.10270e+04_rb,0.77888e+03_rb,0.57952e+03_rb,0.84927e+03_rb,0.11210e+04_rb /)
      kbo(:, 5,45, 8) = (/ &
        & 0.10269e+04_rb,0.77886e+03_rb,0.57890e+03_rb,0.84837e+03_rb,0.11198e+04_rb /)
      kbo(:, 1,46, 8) = (/ &
        & 0.10270e+04_rb,0.77887e+03_rb,0.58072e+03_rb,0.85082e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,46, 8) = (/ &
        & 0.10270e+04_rb,0.77886e+03_rb,0.58056e+03_rb,0.85063e+03_rb,0.11226e+04_rb /)
      kbo(:, 3,46, 8) = (/ &
        & 0.10269e+04_rb,0.77886e+03_rb,0.58022e+03_rb,0.85021e+03_rb,0.11221e+04_rb /)
      kbo(:, 4,46, 8) = (/ &
        & 0.10270e+04_rb,0.77885e+03_rb,0.57972e+03_rb,0.84957e+03_rb,0.11213e+04_rb /)
      kbo(:, 5,46, 8) = (/ &
        & 0.10269e+04_rb,0.77884e+03_rb,0.57911e+03_rb,0.84871e+03_rb,0.11203e+04_rb /)
      kbo(:, 1,47, 8) = (/ &
        & 0.10269e+04_rb,0.77885e+03_rb,0.58070e+03_rb,0.85086e+03_rb,0.11227e+04_rb /)
      kbo(:, 2,47, 8) = (/ &
        & 0.10269e+04_rb,0.77886e+03_rb,0.58066e+03_rb,0.85073e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,47, 8) = (/ &
        & 0.10269e+04_rb,0.77885e+03_rb,0.58037e+03_rb,0.85041e+03_rb,0.11223e+04_rb /)
      kbo(:, 4,47, 8) = (/ &
        & 0.10269e+04_rb,0.77884e+03_rb,0.57993e+03_rb,0.84985e+03_rb,0.11217e+04_rb /)
      kbo(:, 5,47, 8) = (/ &
        & 0.10269e+04_rb,0.77883e+03_rb,0.57937e+03_rb,0.84903e+03_rb,0.11207e+04_rb /)
      kbo(:, 1,48, 8) = (/ &
        & 0.10269e+04_rb,0.77884e+03_rb,0.58064e+03_rb,0.85086e+03_rb,0.11226e+04_rb /)
      kbo(:, 2,48, 8) = (/ &
        & 0.10269e+04_rb,0.77883e+03_rb,0.58071e+03_rb,0.85080e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,48, 8) = (/ &
        & 0.10269e+04_rb,0.77883e+03_rb,0.58051e+03_rb,0.85055e+03_rb,0.11225e+04_rb /)
      kbo(:, 4,48, 8) = (/ &
        & 0.10269e+04_rb,0.77883e+03_rb,0.58013e+03_rb,0.85010e+03_rb,0.11220e+04_rb /)
      kbo(:, 5,48, 8) = (/ &
        & 0.10269e+04_rb,0.77883e+03_rb,0.57961e+03_rb,0.84941e+03_rb,0.11212e+04_rb /)
      kbo(:, 1,49, 8) = (/ &
        & 0.10269e+04_rb,0.77884e+03_rb,0.58054e+03_rb,0.85082e+03_rb,0.11225e+04_rb /)
      kbo(:, 2,49, 8) = (/ &
        & 0.10269e+04_rb,0.77882e+03_rb,0.58072e+03_rb,0.85085e+03_rb,0.11227e+04_rb /)
      kbo(:, 3,49, 8) = (/ &
        & 0.10269e+04_rb,0.77883e+03_rb,0.58061e+03_rb,0.85068e+03_rb,0.11226e+04_rb /)
      kbo(:, 4,49, 8) = (/ &
        & 0.10269e+04_rb,0.77882e+03_rb,0.58029e+03_rb,0.85032e+03_rb,0.11222e+04_rb /)
      kbo(:, 5,49, 8) = (/ &
        & 0.10269e+04_rb,0.77881e+03_rb,0.57983e+03_rb,0.84969e+03_rb,0.11215e+04_rb /)
      kbo(:, 1,50, 8) = (/ &
        & 0.10269e+04_rb,0.77883e+03_rb,0.58042e+03_rb,0.85074e+03_rb,0.11224e+04_rb /)
      kbo(:, 2,50, 8) = (/ &
        & 0.10269e+04_rb,0.77881e+03_rb,0.58068e+03_rb,0.85085e+03_rb,0.11226e+04_rb /)
      kbo(:, 3,50, 8) = (/ &
        & 0.10269e+04_rb,0.77882e+03_rb,0.58069e+03_rb,0.85077e+03_rb,0.11227e+04_rb /)
      kbo(:, 4,50, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58043e+03_rb,0.85048e+03_rb,0.11224e+04_rb /)
      kbo(:, 5,50, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58002e+03_rb,0.84997e+03_rb,0.11218e+04_rb /)
      kbo(:, 1,51, 8) = (/ &
        & 0.10269e+04_rb,0.77881e+03_rb,0.58027e+03_rb,0.85064e+03_rb,0.11224e+04_rb /)
      kbo(:, 2,51, 8) = (/ &
        & 0.10269e+04_rb,0.77882e+03_rb,0.58061e+03_rb,0.85085e+03_rb,0.11226e+04_rb /)
      kbo(:, 3,51, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58071e+03_rb,0.85082e+03_rb,0.11227e+04_rb /)
      kbo(:, 4,51, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58055e+03_rb,0.85061e+03_rb,0.11226e+04_rb /)
      kbo(:, 5,51, 8) = (/ &
        & 0.10269e+04_rb,0.77881e+03_rb,0.58020e+03_rb,0.85015e+03_rb,0.11221e+04_rb /)
      kbo(:, 1,52, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58011e+03_rb,0.85051e+03_rb,0.11222e+04_rb /)
      kbo(:, 2,52, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58051e+03_rb,0.85080e+03_rb,0.11225e+04_rb /)
      kbo(:, 3,52, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58071e+03_rb,0.85085e+03_rb,0.11227e+04_rb /)
      kbo(:, 4,52, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.58064e+03_rb,0.85068e+03_rb,0.11226e+04_rb /)
      kbo(:, 5,52, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.58034e+03_rb,0.85037e+03_rb,0.11223e+04_rb /)
      kbo(:, 1,53, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.57994e+03_rb,0.85036e+03_rb,0.11220e+04_rb /)
      kbo(:, 2,53, 8) = (/ &
        & 0.10269e+04_rb,0.77879e+03_rb,0.58038e+03_rb,0.85072e+03_rb,0.11224e+04_rb /)
      kbo(:, 3,53, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.58066e+03_rb,0.85087e+03_rb,0.11226e+04_rb /)
      kbo(:, 4,53, 8) = (/ &
        & 0.10269e+04_rb,0.77880e+03_rb,0.58069e+03_rb,0.85078e+03_rb,0.11227e+04_rb /)
      kbo(:, 5,53, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58047e+03_rb,0.85044e+03_rb,0.11225e+04_rb /)
      kbo(:, 1,54, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.57977e+03_rb,0.85022e+03_rb,0.11219e+04_rb /)
      kbo(:, 2,54, 8) = (/ &
        & 0.10269e+04_rb,0.77879e+03_rb,0.58026e+03_rb,0.85061e+03_rb,0.11223e+04_rb /)
      kbo(:, 3,54, 8) = (/ &
        & 0.10268e+04_rb,0.77881e+03_rb,0.58060e+03_rb,0.85084e+03_rb,0.11226e+04_rb /)
      kbo(:, 4,54, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.58072e+03_rb,0.85083e+03_rb,0.11227e+04_rb /)
      kbo(:, 5,54, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58058e+03_rb,0.85063e+03_rb,0.11226e+04_rb /)
      kbo(:, 1,55, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.57962e+03_rb,0.85008e+03_rb,0.11217e+04_rb /)
      kbo(:, 2,55, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58011e+03_rb,0.85050e+03_rb,0.11222e+04_rb /)
      kbo(:, 3,55, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.58050e+03_rb,0.85080e+03_rb,0.11225e+04_rb /)
      kbo(:, 4,55, 8) = (/ &
        & 0.10268e+04_rb,0.77880e+03_rb,0.58071e+03_rb,0.85085e+03_rb,0.11226e+04_rb /)
      kbo(:, 5,55, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58064e+03_rb,0.85072e+03_rb,0.11226e+04_rb /)
      kbo(:, 1,56, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.57945e+03_rb,0.84993e+03_rb,0.11216e+04_rb /)
      kbo(:, 2,56, 8) = (/ &
        & 0.10268e+04_rb,0.77877e+03_rb,0.57995e+03_rb,0.85037e+03_rb,0.11220e+04_rb /)
      kbo(:, 3,56, 8) = (/ &
        & 0.10268e+04_rb,0.77877e+03_rb,0.58039e+03_rb,0.85071e+03_rb,0.11224e+04_rb /)
      kbo(:, 4,56, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58067e+03_rb,0.85086e+03_rb,0.11226e+04_rb /)
      kbo(:, 5,56, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58069e+03_rb,0.85078e+03_rb,0.11227e+04_rb /)
      kbo(:, 1,57, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.57927e+03_rb,0.84978e+03_rb,0.11215e+04_rb /)
      kbo(:, 2,57, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.57979e+03_rb,0.85024e+03_rb,0.11219e+04_rb /)
      kbo(:, 3,57, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58025e+03_rb,0.85063e+03_rb,0.11223e+04_rb /)
      kbo(:, 4,57, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58060e+03_rb,0.85084e+03_rb,0.11226e+04_rb /)
      kbo(:, 5,57, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58071e+03_rb,0.85083e+03_rb,0.11227e+04_rb /)
      kbo(:, 1,58, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.57911e+03_rb,0.84962e+03_rb,0.11212e+04_rb /)
      kbo(:, 2,58, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.57964e+03_rb,0.85010e+03_rb,0.11218e+04_rb /)
      kbo(:, 3,58, 8) = (/ &
        & 0.10268e+04_rb,0.77877e+03_rb,0.58013e+03_rb,0.85053e+03_rb,0.11222e+04_rb /)
      kbo(:, 4,58, 8) = (/ &
        & 0.10268e+04_rb,0.77877e+03_rb,0.58052e+03_rb,0.85080e+03_rb,0.11225e+04_rb /)
      kbo(:, 5,58, 8) = (/ &
        & 0.10268e+04_rb,0.77879e+03_rb,0.58071e+03_rb,0.85085e+03_rb,0.11227e+04_rb /)
      kbo(:, 1,59, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.57904e+03_rb,0.84955e+03_rb,0.11212e+04_rb /)
      kbo(:, 2,59, 8) = (/ &
        & 0.10268e+04_rb,0.77877e+03_rb,0.57958e+03_rb,0.85006e+03_rb,0.11217e+04_rb /)
      kbo(:, 3,59, 8) = (/ &
        & 0.10268e+04_rb,0.77877e+03_rb,0.58007e+03_rb,0.85048e+03_rb,0.11222e+04_rb /)
      kbo(:, 4,59, 8) = (/ &
        & 0.10268e+04_rb,0.77878e+03_rb,0.58048e+03_rb,0.85078e+03_rb,0.11225e+04_rb /)
      kbo(:, 5,59, 8) = (/ &
        & 0.10268e+04_rb,0.77877e+03_rb,0.58070e+03_rb,0.85086e+03_rb,0.11227e+04_rb /)
      kbo(:, 1,13, 9) = (/ &
        & 0.11533e+04_rb,0.87327e+03_rb,0.59473e+03_rb,0.87046e+03_rb,0.11487e+04_rb /)
      kbo(:, 2,13, 9) = (/ &
        & 0.11444e+04_rb,0.86660e+03_rb,0.59240e+03_rb,0.87005e+03_rb,0.11483e+04_rb /)
      kbo(:, 3,13, 9) = (/ &
        & 0.11367e+04_rb,0.86090e+03_rb,0.59185e+03_rb,0.86950e+03_rb,0.11478e+04_rb /)
      kbo(:, 4,13, 9) = (/ &
        & 0.11301e+04_rb,0.85587e+03_rb,0.59143e+03_rb,0.86883e+03_rb,0.11469e+04_rb /)
      kbo(:, 5,13, 9) = (/ &
        & 0.11242e+04_rb,0.85148e+03_rb,0.59084e+03_rb,0.86797e+03_rb,0.11458e+04_rb /)
      kbo(:, 1,14, 9) = (/ &
        & 0.11310e+04_rb,0.85656e+03_rb,0.59238e+03_rb,0.87026e+03_rb,0.11487e+04_rb /)
      kbo(:, 2,14, 9) = (/ &
        & 0.11238e+04_rb,0.85118e+03_rb,0.59190e+03_rb,0.86988e+03_rb,0.11483e+04_rb /)
      kbo(:, 3,14, 9) = (/ &
        & 0.11176e+04_rb,0.84654e+03_rb,0.59149e+03_rb,0.86931e+03_rb,0.11477e+04_rb /)
      kbo(:, 4,14, 9) = (/ &
        & 0.11122e+04_rb,0.84248e+03_rb,0.59105e+03_rb,0.86863e+03_rb,0.11468e+04_rb /)
      kbo(:, 5,14, 9) = (/ &
        & 0.11074e+04_rb,0.83890e+03_rb,0.59046e+03_rb,0.86777e+03_rb,0.11457e+04_rb /)
      kbo(:, 1,15, 9) = (/ &
        & 0.11128e+04_rb,0.84292e+03_rb,0.59202e+03_rb,0.87009e+03_rb,0.11486e+04_rb /)
      kbo(:, 2,15, 9) = (/ &
        & 0.11070e+04_rb,0.83857e+03_rb,0.59157e+03_rb,0.86969e+03_rb,0.11482e+04_rb /)
      kbo(:, 3,15, 9) = (/ &
        & 0.11020e+04_rb,0.83480e+03_rb,0.59118e+03_rb,0.86913e+03_rb,0.11476e+04_rb /)
      kbo(:, 4,15, 9) = (/ &
        & 0.10976e+04_rb,0.83151e+03_rb,0.59076e+03_rb,0.86843e+03_rb,0.11467e+04_rb /)
      kbo(:, 5,15, 9) = (/ &
        & 0.10938e+04_rb,0.82865e+03_rb,0.59015e+03_rb,0.86756e+03_rb,0.11457e+04_rb /)
      kbo(:, 1,16, 9) = (/ &
        & 0.10980e+04_rb,0.83182e+03_rb,0.59172e+03_rb,0.86994e+03_rb,0.11486e+04_rb /)
      kbo(:, 2,16, 9) = (/ &
        & 0.10933e+04_rb,0.82831e+03_rb,0.59129e+03_rb,0.86966e+03_rb,0.11482e+04_rb /)
      kbo(:, 3,16, 9) = (/ &
        & 0.10893e+04_rb,0.82527e+03_rb,0.59092e+03_rb,0.86898e+03_rb,0.11475e+04_rb /)
      kbo(:, 4,16, 9) = (/ &
        & 0.10858e+04_rb,0.82261e+03_rb,0.59049e+03_rb,0.86826e+03_rb,0.11466e+04_rb /)
      kbo(:, 5,16, 9) = (/ &
        & 0.10827e+04_rb,0.82037e+03_rb,0.58989e+03_rb,0.86738e+03_rb,0.11455e+04_rb /)
      kbo(:, 1,17, 9) = (/ &
        & 0.10860e+04_rb,0.82280e+03_rb,0.59147e+03_rb,0.86983e+03_rb,0.11485e+04_rb /)
      kbo(:, 2,17, 9) = (/ &
        & 0.10822e+04_rb,0.81997e+03_rb,0.59107e+03_rb,0.86941e+03_rb,0.11481e+04_rb /)
      kbo(:, 3,17, 9) = (/ &
        & 0.10790e+04_rb,0.81753e+03_rb,0.59070e+03_rb,0.86883e+03_rb,0.11474e+04_rb /)
      kbo(:, 4,17, 9) = (/ &
        & 0.10761e+04_rb,0.81539e+03_rb,0.59027e+03_rb,0.86811e+03_rb,0.11465e+04_rb /)
      kbo(:, 5,17, 9) = (/ &
        & 0.10736e+04_rb,0.81353e+03_rb,0.58965e+03_rb,0.86721e+03_rb,0.11454e+04_rb /)
      kbo(:, 1,18, 9) = (/ &
        & 0.10762e+04_rb,0.81555e+03_rb,0.59126e+03_rb,0.86969e+03_rb,0.11485e+04_rb /)
      kbo(:, 2,18, 9) = (/ &
        & 0.10732e+04_rb,0.81322e+03_rb,0.59088e+03_rb,0.86928e+03_rb,0.11481e+04_rb /)
      kbo(:, 3,18, 9) = (/ &
        & 0.10706e+04_rb,0.81128e+03_rb,0.59052e+03_rb,0.86870e+03_rb,0.11474e+04_rb /)
      kbo(:, 4,18, 9) = (/ &
        & 0.10683e+04_rb,0.80956e+03_rb,0.59008e+03_rb,0.86800e+03_rb,0.11464e+04_rb /)
      kbo(:, 5,18, 9) = (/ &
        & 0.10663e+04_rb,0.80808e+03_rb,0.58945e+03_rb,0.86705e+03_rb,0.11452e+04_rb /)
      kbo(:, 1,19, 9) = (/ &
        & 0.10684e+04_rb,0.80963e+03_rb,0.59108e+03_rb,0.86962e+03_rb,0.11485e+04_rb /)
      kbo(:, 2,19, 9) = (/ &
        & 0.10659e+04_rb,0.80774e+03_rb,0.59072e+03_rb,0.86917e+03_rb,0.11480e+04_rb /)
      kbo(:, 3,19, 9) = (/ &
        & 0.10638e+04_rb,0.80613e+03_rb,0.59038e+03_rb,0.86858e+03_rb,0.11473e+04_rb /)
      kbo(:, 4,19, 9) = (/ &
        & 0.10619e+04_rb,0.80481e+03_rb,0.58991e+03_rb,0.86784e+03_rb,0.11463e+04_rb /)
      kbo(:, 5,19, 9) = (/ &
        & 0.10603e+04_rb,0.80353e+03_rb,0.58926e+03_rb,0.86690e+03_rb,0.11451e+04_rb /)
      kbo(:, 1,20, 9) = (/ &
        & 0.10619e+04_rb,0.80474e+03_rb,0.59092e+03_rb,0.86952e+03_rb,0.11484e+04_rb /)
      kbo(:, 2,20, 9) = (/ &
        & 0.10599e+04_rb,0.80326e+03_rb,0.59057e+03_rb,0.86907e+03_rb,0.11479e+04_rb /)
      kbo(:, 3,20, 9) = (/ &
        & 0.10582e+04_rb,0.80198e+03_rb,0.59022e+03_rb,0.86845e+03_rb,0.11472e+04_rb /)
      kbo(:, 4,20, 9) = (/ &
        & 0.10567e+04_rb,0.80089e+03_rb,0.58976e+03_rb,0.86771e+03_rb,0.11462e+04_rb /)
      kbo(:, 5,20, 9) = (/ &
        & 0.10554e+04_rb,0.79986e+03_rb,0.58910e+03_rb,0.86674e+03_rb,0.11449e+04_rb /)
      kbo(:, 1,21, 9) = (/ &
        & 0.10567e+04_rb,0.80080e+03_rb,0.59079e+03_rb,0.86945e+03_rb,0.11483e+04_rb /)
      kbo(:, 2,21, 9) = (/ &
        & 0.10550e+04_rb,0.79960e+03_rb,0.59045e+03_rb,0.86897e+03_rb,0.11478e+04_rb /)
      kbo(:, 3,21, 9) = (/ &
        & 0.10537e+04_rb,0.79858e+03_rb,0.59010e+03_rb,0.86836e+03_rb,0.11471e+04_rb /)
      kbo(:, 4,21, 9) = (/ &
        & 0.10525e+04_rb,0.79770e+03_rb,0.58962e+03_rb,0.86752e+03_rb,0.11460e+04_rb /)
      kbo(:, 5,21, 9) = (/ &
        & 0.10514e+04_rb,0.79685e+03_rb,0.58893e+03_rb,0.86654e+03_rb,0.11447e+04_rb /)
      kbo(:, 1,22, 9) = (/ &
        & 0.10523e+04_rb,0.79754e+03_rb,0.59066e+03_rb,0.86933e+03_rb,0.11483e+04_rb /)
      kbo(:, 2,22, 9) = (/ &
        & 0.10510e+04_rb,0.79661e+03_rb,0.59033e+03_rb,0.86884e+03_rb,0.11477e+04_rb /)
      kbo(:, 3,22, 9) = (/ &
        & 0.10500e+04_rb,0.79577e+03_rb,0.58997e+03_rb,0.86819e+03_rb,0.11469e+04_rb /)
      kbo(:, 4,22, 9) = (/ &
        & 0.10490e+04_rb,0.79506e+03_rb,0.58945e+03_rb,0.86737e+03_rb,0.11458e+04_rb /)
      kbo(:, 5,22, 9) = (/ &
        & 0.10482e+04_rb,0.79443e+03_rb,0.58872e+03_rb,0.86635e+03_rb,0.11444e+04_rb /)
      kbo(:, 1,23, 9) = (/ &
        & 0.10488e+04_rb,0.79491e+03_rb,0.59053e+03_rb,0.86922e+03_rb,0.11482e+04_rb /)
      kbo(:, 2,23, 9) = (/ &
        & 0.10478e+04_rb,0.79417e+03_rb,0.59023e+03_rb,0.86872e+03_rb,0.11476e+04_rb /)
      kbo(:, 3,23, 9) = (/ &
        & 0.10469e+04_rb,0.79351e+03_rb,0.58983e+03_rb,0.86802e+03_rb,0.11467e+04_rb /)
      kbo(:, 4,23, 9) = (/ &
        & 0.10461e+04_rb,0.79293e+03_rb,0.58929e+03_rb,0.86717e+03_rb,0.11456e+04_rb /)
      kbo(:, 5,23, 9) = (/ &
        & 0.10455e+04_rb,0.79241e+03_rb,0.58851e+03_rb,0.86610e+03_rb,0.11442e+04_rb /)
      kbo(:, 1,24, 9) = (/ &
        & 0.10460e+04_rb,0.79285e+03_rb,0.59042e+03_rb,0.86910e+03_rb,0.11481e+04_rb /)
      kbo(:, 2,24, 9) = (/ &
        & 0.10452e+04_rb,0.79220e+03_rb,0.59009e+03_rb,0.86856e+03_rb,0.11474e+04_rb /)
      kbo(:, 3,24, 9) = (/ &
        & 0.10445e+04_rb,0.79170e+03_rb,0.58968e+03_rb,0.86785e+03_rb,0.11465e+04_rb /)
      kbo(:, 4,24, 9) = (/ &
        & 0.10439e+04_rb,0.79123e+03_rb,0.58911e+03_rb,0.86698e+03_rb,0.11453e+04_rb /)
      kbo(:, 5,24, 9) = (/ &
        & 0.10433e+04_rb,0.79086e+03_rb,0.58830e+03_rb,0.86584e+03_rb,0.11439e+04_rb /)
      kbo(:, 1,25, 9) = (/ &
        & 0.10438e+04_rb,0.79109e+03_rb,0.59032e+03_rb,0.86899e+03_rb,0.11480e+04_rb /)
      kbo(:, 2,25, 9) = (/ &
        & 0.10430e+04_rb,0.79059e+03_rb,0.58999e+03_rb,0.86841e+03_rb,0.11473e+04_rb /)
      kbo(:, 3,25, 9) = (/ &
        & 0.10425e+04_rb,0.79018e+03_rb,0.58955e+03_rb,0.86768e+03_rb,0.11463e+04_rb /)
      kbo(:, 4,25, 9) = (/ &
        & 0.10420e+04_rb,0.78985e+03_rb,0.58893e+03_rb,0.86674e+03_rb,0.11450e+04_rb /)
      kbo(:, 5,25, 9) = (/ &
        & 0.10416e+04_rb,0.78947e+03_rb,0.58810e+03_rb,0.86557e+03_rb,0.11436e+04_rb /)
      kbo(:, 1,26, 9) = (/ &
        & 0.10418e+04_rb,0.78966e+03_rb,0.59022e+03_rb,0.86886e+03_rb,0.11478e+04_rb /)
      kbo(:, 2,26, 9) = (/ &
        & 0.10414e+04_rb,0.78932e+03_rb,0.58989e+03_rb,0.86824e+03_rb,0.11471e+04_rb /)
      kbo(:, 3,26, 9) = (/ &
        & 0.10409e+04_rb,0.78898e+03_rb,0.58940e+03_rb,0.86743e+03_rb,0.11460e+04_rb /)
      kbo(:, 4,26, 9) = (/ &
        & 0.10405e+04_rb,0.78870e+03_rb,0.58873e+03_rb,0.86647e+03_rb,0.11447e+04_rb /)
      kbo(:, 5,26, 9) = (/ &
        & 0.10401e+04_rb,0.78842e+03_rb,0.58782e+03_rb,0.86527e+03_rb,0.11432e+04_rb /)
      kbo(:, 1,27, 9) = (/ &
        & 0.10403e+04_rb,0.78855e+03_rb,0.59012e+03_rb,0.86872e+03_rb,0.11477e+04_rb /)
      kbo(:, 2,27, 9) = (/ &
        & 0.10399e+04_rb,0.78827e+03_rb,0.58977e+03_rb,0.86806e+03_rb,0.11468e+04_rb /)
      kbo(:, 3,27, 9) = (/ &
        & 0.10396e+04_rb,0.78799e+03_rb,0.58924e+03_rb,0.86723e+03_rb,0.11458e+04_rb /)
      kbo(:, 4,27, 9) = (/ &
        & 0.10392e+04_rb,0.78776e+03_rb,0.58850e+03_rb,0.86619e+03_rb,0.11444e+04_rb /)
      kbo(:, 5,27, 9) = (/ &
        & 0.10390e+04_rb,0.78757e+03_rb,0.58756e+03_rb,0.86495e+03_rb,0.11428e+04_rb /)
      kbo(:, 1,28, 9) = (/ &
        & 0.10391e+04_rb,0.78770e+03_rb,0.59002e+03_rb,0.86856e+03_rb,0.11475e+04_rb /)
      kbo(:, 2,28, 9) = (/ &
        & 0.10388e+04_rb,0.78747e+03_rb,0.58964e+03_rb,0.86790e+03_rb,0.11466e+04_rb /)
      kbo(:, 3,28, 9) = (/ &
        & 0.10385e+04_rb,0.78719e+03_rb,0.58908e+03_rb,0.86701e+03_rb,0.11455e+04_rb /)
      kbo(:, 4,28, 9) = (/ &
        & 0.10383e+04_rb,0.78704e+03_rb,0.58829e+03_rb,0.86590e+03_rb,0.11440e+04_rb /)
      kbo(:, 5,28, 9) = (/ &
        & 0.10381e+04_rb,0.78687e+03_rb,0.58729e+03_rb,0.86461e+03_rb,0.11423e+04_rb /)
      kbo(:, 1,29, 9) = (/ &
        & 0.10382e+04_rb,0.78694e+03_rb,0.58994e+03_rb,0.86841e+03_rb,0.11473e+04_rb /)
      kbo(:, 2,29, 9) = (/ &
        & 0.10379e+04_rb,0.78677e+03_rb,0.58950e+03_rb,0.86767e+03_rb,0.11463e+04_rb /)
      kbo(:, 3,29, 9) = (/ &
        & 0.10377e+04_rb,0.78658e+03_rb,0.58888e+03_rb,0.86674e+03_rb,0.11451e+04_rb /)
      kbo(:, 4,29, 9) = (/ &
        & 0.10375e+04_rb,0.78645e+03_rb,0.58805e+03_rb,0.86559e+03_rb,0.11436e+04_rb /)
      kbo(:, 5,29, 9) = (/ &
        & 0.10374e+04_rb,0.78628e+03_rb,0.58702e+03_rb,0.86429e+03_rb,0.11419e+04_rb /)
      kbo(:, 1,30, 9) = (/ &
        & 0.10373e+04_rb,0.78632e+03_rb,0.58983e+03_rb,0.86822e+03_rb,0.11471e+04_rb /)
      kbo(:, 2,30, 9) = (/ &
        & 0.10371e+04_rb,0.78619e+03_rb,0.58935e+03_rb,0.86746e+03_rb,0.11460e+04_rb /)
      kbo(:, 3,30, 9) = (/ &
        & 0.10370e+04_rb,0.78613e+03_rb,0.58869e+03_rb,0.86647e+03_rb,0.11448e+04_rb /)
      kbo(:, 4,30, 9) = (/ &
        & 0.10369e+04_rb,0.78594e+03_rb,0.58779e+03_rb,0.86529e+03_rb,0.11432e+04_rb /)
      kbo(:, 5,30, 9) = (/ &
        & 0.10367e+04_rb,0.78585e+03_rb,0.58673e+03_rb,0.86393e+03_rb,0.11414e+04_rb /)
      kbo(:, 1,31, 9) = (/ &
        & 0.10367e+04_rb,0.78588e+03_rb,0.58971e+03_rb,0.86804e+03_rb,0.11468e+04_rb /)
      kbo(:, 2,31, 9) = (/ &
        & 0.10365e+04_rb,0.78574e+03_rb,0.58919e+03_rb,0.86720e+03_rb,0.11457e+04_rb /)
      kbo(:, 3,31, 9) = (/ &
        & 0.10364e+04_rb,0.78562e+03_rb,0.58847e+03_rb,0.86618e+03_rb,0.11443e+04_rb /)
      kbo(:, 4,31, 9) = (/ &
        & 0.10363e+04_rb,0.78557e+03_rb,0.58752e+03_rb,0.86495e+03_rb,0.11427e+04_rb /)
      kbo(:, 5,31, 9) = (/ &
        & 0.10362e+04_rb,0.78546e+03_rb,0.58644e+03_rb,0.86356e+03_rb,0.11409e+04_rb /)
      kbo(:, 1,32, 9) = (/ &
        & 0.10362e+04_rb,0.78556e+03_rb,0.58958e+03_rb,0.86784e+03_rb,0.11466e+04_rb /)
      kbo(:, 2,32, 9) = (/ &
        & 0.10361e+04_rb,0.78539e+03_rb,0.58901e+03_rb,0.86697e+03_rb,0.11454e+04_rb /)
      kbo(:, 3,32, 9) = (/ &
        & 0.10360e+04_rb,0.78539e+03_rb,0.58824e+03_rb,0.86587e+03_rb,0.11440e+04_rb /)
      kbo(:, 4,32, 9) = (/ &
        & 0.10359e+04_rb,0.78525e+03_rb,0.58724e+03_rb,0.86459e+03_rb,0.11423e+04_rb /)
      kbo(:, 5,32, 9) = (/ &
        & 0.10358e+04_rb,0.78516e+03_rb,0.58615e+03_rb,0.86318e+03_rb,0.11404e+04_rb /)
      kbo(:, 1,33, 9) = (/ &
        & 0.10358e+04_rb,0.78520e+03_rb,0.58945e+03_rb,0.86763e+03_rb,0.11463e+04_rb /)
      kbo(:, 2,33, 9) = (/ &
        & 0.10357e+04_rb,0.78510e+03_rb,0.58883e+03_rb,0.86670e+03_rb,0.11450e+04_rb /)
      kbo(:, 3,33, 9) = (/ &
        & 0.10356e+04_rb,0.78504e+03_rb,0.58799e+03_rb,0.86554e+03_rb,0.11435e+04_rb /)
      kbo(:, 4,33, 9) = (/ &
        & 0.10356e+04_rb,0.78500e+03_rb,0.58695e+03_rb,0.86422e+03_rb,0.11418e+04_rb /)
      kbo(:, 5,33, 9) = (/ &
        & 0.10355e+04_rb,0.78493e+03_rb,0.58586e+03_rb,0.86281e+03_rb,0.11399e+04_rb /)
      kbo(:, 1,34, 9) = (/ &
        & 0.10355e+04_rb,0.78495e+03_rb,0.58932e+03_rb,0.86743e+03_rb,0.11460e+04_rb /)
      kbo(:, 2,34, 9) = (/ &
        & 0.10354e+04_rb,0.78487e+03_rb,0.58865e+03_rb,0.86644e+03_rb,0.11447e+04_rb /)
      kbo(:, 3,34, 9) = (/ &
        & 0.10354e+04_rb,0.78485e+03_rb,0.58776e+03_rb,0.86524e+03_rb,0.11432e+04_rb /)
      kbo(:, 4,34, 9) = (/ &
        & 0.10353e+04_rb,0.78479e+03_rb,0.58670e+03_rb,0.86390e+03_rb,0.11414e+04_rb /)
      kbo(:, 5,34, 9) = (/ &
        & 0.10352e+04_rb,0.78474e+03_rb,0.58560e+03_rb,0.86249e+03_rb,0.11395e+04_rb /)
      kbo(:, 1,35, 9) = (/ &
        & 0.10352e+04_rb,0.78478e+03_rb,0.58924e+03_rb,0.86731e+03_rb,0.11459e+04_rb /)
      kbo(:, 2,35, 9) = (/ &
        & 0.10352e+04_rb,0.78472e+03_rb,0.58855e+03_rb,0.86632e+03_rb,0.11445e+04_rb /)
      kbo(:, 3,35, 9) = (/ &
        & 0.10351e+04_rb,0.78465e+03_rb,0.58761e+03_rb,0.86508e+03_rb,0.11430e+04_rb /)
      kbo(:, 4,35, 9) = (/ &
        & 0.10350e+04_rb,0.78465e+03_rb,0.58655e+03_rb,0.86369e+03_rb,0.11412e+04_rb /)
      kbo(:, 5,35, 9) = (/ &
        & 0.10350e+04_rb,0.78456e+03_rb,0.58545e+03_rb,0.86237e+03_rb,0.11393e+04_rb /)
      kbo(:, 1,36, 9) = (/ &
        & 0.10350e+04_rb,0.78465e+03_rb,0.58922e+03_rb,0.86729e+03_rb,0.11458e+04_rb /)
      kbo(:, 2,36, 9) = (/ &
        & 0.10350e+04_rb,0.78456e+03_rb,0.58852e+03_rb,0.86627e+03_rb,0.11445e+04_rb /)
      kbo(:, 3,36, 9) = (/ &
        & 0.10350e+04_rb,0.78452e+03_rb,0.58759e+03_rb,0.86505e+03_rb,0.11429e+04_rb /)
      kbo(:, 4,36, 9) = (/ &
        & 0.10349e+04_rb,0.78450e+03_rb,0.58651e+03_rb,0.86369e+03_rb,0.11411e+04_rb /)
      kbo(:, 5,36, 9) = (/ &
        & 0.10348e+04_rb,0.78444e+03_rb,0.58544e+03_rb,0.86227e+03_rb,0.11392e+04_rb /)
      kbo(:, 1,37, 9) = (/ &
        & 0.10349e+04_rb,0.78446e+03_rb,0.58930e+03_rb,0.86738e+03_rb,0.11460e+04_rb /)
      kbo(:, 2,37, 9) = (/ &
        & 0.10348e+04_rb,0.78444e+03_rb,0.58863e+03_rb,0.86642e+03_rb,0.11447e+04_rb /)
      kbo(:, 3,37, 9) = (/ &
        & 0.10348e+04_rb,0.78441e+03_rb,0.58773e+03_rb,0.86522e+03_rb,0.11432e+04_rb /)
      kbo(:, 4,37, 9) = (/ &
        & 0.10348e+04_rb,0.78439e+03_rb,0.58667e+03_rb,0.86387e+03_rb,0.11414e+04_rb /)
      kbo(:, 5,37, 9) = (/ &
        & 0.10347e+04_rb,0.78436e+03_rb,0.58559e+03_rb,0.86245e+03_rb,0.11394e+04_rb /)
      kbo(:, 1,38, 9) = (/ &
        & 0.10348e+04_rb,0.78440e+03_rb,0.58936e+03_rb,0.86753e+03_rb,0.11462e+04_rb /)
      kbo(:, 2,38, 9) = (/ &
        & 0.10347e+04_rb,0.78434e+03_rb,0.58873e+03_rb,0.86658e+03_rb,0.11449e+04_rb /)
      kbo(:, 3,38, 9) = (/ &
        & 0.10347e+04_rb,0.78432e+03_rb,0.58786e+03_rb,0.86537e+03_rb,0.11434e+04_rb /)
      kbo(:, 4,38, 9) = (/ &
        & 0.10346e+04_rb,0.78431e+03_rb,0.58681e+03_rb,0.86407e+03_rb,0.11416e+04_rb /)
      kbo(:, 5,38, 9) = (/ &
        & 0.10347e+04_rb,0.78428e+03_rb,0.58573e+03_rb,0.86265e+03_rb,0.11397e+04_rb /)
      kbo(:, 1,39, 9) = (/ &
        & 0.10347e+04_rb,0.78431e+03_rb,0.58943e+03_rb,0.86764e+03_rb,0.11463e+04_rb /)
      kbo(:, 2,39, 9) = (/ &
        & 0.10346e+04_rb,0.78431e+03_rb,0.58882e+03_rb,0.86672e+03_rb,0.11451e+04_rb /)
      kbo(:, 3,39, 9) = (/ &
        & 0.10346e+04_rb,0.78424e+03_rb,0.58800e+03_rb,0.86557e+03_rb,0.11436e+04_rb /)
      kbo(:, 4,39, 9) = (/ &
        & 0.10346e+04_rb,0.78421e+03_rb,0.58697e+03_rb,0.86426e+03_rb,0.11419e+04_rb /)
      kbo(:, 5,39, 9) = (/ &
        & 0.10346e+04_rb,0.78429e+03_rb,0.58587e+03_rb,0.86284e+03_rb,0.11400e+04_rb /)
      kbo(:, 1,40, 9) = (/ &
        & 0.10346e+04_rb,0.78426e+03_rb,0.58957e+03_rb,0.86785e+03_rb,0.11466e+04_rb /)
      kbo(:, 2,40, 9) = (/ &
        & 0.10345e+04_rb,0.78427e+03_rb,0.58899e+03_rb,0.86697e+03_rb,0.11454e+04_rb /)
      kbo(:, 3,40, 9) = (/ &
        & 0.10345e+04_rb,0.78422e+03_rb,0.58822e+03_rb,0.86585e+03_rb,0.11440e+04_rb /)
      kbo(:, 4,40, 9) = (/ &
        & 0.10345e+04_rb,0.78421e+03_rb,0.58723e+03_rb,0.86460e+03_rb,0.11423e+04_rb /)
      kbo(:, 5,40, 9) = (/ &
        & 0.10344e+04_rb,0.78416e+03_rb,0.58614e+03_rb,0.86319e+03_rb,0.11403e+04_rb /)
      kbo(:, 1,41, 9) = (/ &
        & 0.10345e+04_rb,0.78421e+03_rb,0.58967e+03_rb,0.86804e+03_rb,0.11468e+04_rb /)
      kbo(:, 2,41, 9) = (/ &
        & 0.10345e+04_rb,0.78418e+03_rb,0.58916e+03_rb,0.86720e+03_rb,0.11457e+04_rb /)
      kbo(:, 3,41, 9) = (/ &
        & 0.10344e+04_rb,0.78415e+03_rb,0.58844e+03_rb,0.86618e+03_rb,0.11444e+04_rb /)
      kbo(:, 4,41, 9) = (/ &
        & 0.10344e+04_rb,0.78413e+03_rb,0.58748e+03_rb,0.86492e+03_rb,0.11428e+04_rb /)
      kbo(:, 5,41, 9) = (/ &
        & 0.10344e+04_rb,0.78411e+03_rb,0.58642e+03_rb,0.86355e+03_rb,0.11409e+04_rb /)
      kbo(:, 1,42, 9) = (/ &
        & 0.10345e+04_rb,0.78414e+03_rb,0.58976e+03_rb,0.86830e+03_rb,0.11471e+04_rb /)
      kbo(:, 2,42, 9) = (/ &
        & 0.10344e+04_rb,0.78417e+03_rb,0.58930e+03_rb,0.86744e+03_rb,0.11460e+04_rb /)
      kbo(:, 3,42, 9) = (/ &
        & 0.10344e+04_rb,0.78411e+03_rb,0.58863e+03_rb,0.86645e+03_rb,0.11448e+04_rb /)
      kbo(:, 4,42, 9) = (/ &
        & 0.10344e+04_rb,0.78409e+03_rb,0.58775e+03_rb,0.86525e+03_rb,0.11432e+04_rb /)
      kbo(:, 5,42, 9) = (/ &
        & 0.10344e+04_rb,0.78408e+03_rb,0.58670e+03_rb,0.86391e+03_rb,0.11414e+04_rb /)
      kbo(:, 1,43, 9) = (/ &
        & 0.10344e+04_rb,0.78410e+03_rb,0.58987e+03_rb,0.86842e+03_rb,0.11473e+04_rb /)
      kbo(:, 2,43, 9) = (/ &
        & 0.10344e+04_rb,0.78408e+03_rb,0.58947e+03_rb,0.86769e+03_rb,0.11464e+04_rb /)
      kbo(:, 3,43, 9) = (/ &
        & 0.10344e+04_rb,0.78409e+03_rb,0.58887e+03_rb,0.86678e+03_rb,0.11452e+04_rb /)
      kbo(:, 4,43, 9) = (/ &
        & 0.10344e+04_rb,0.78410e+03_rb,0.58806e+03_rb,0.86567e+03_rb,0.11437e+04_rb /)
      kbo(:, 5,43, 9) = (/ &
        & 0.10343e+04_rb,0.78406e+03_rb,0.58704e+03_rb,0.86435e+03_rb,0.11420e+04_rb /)
      kbo(:, 1,44, 9) = (/ &
        & 0.10344e+04_rb,0.78409e+03_rb,0.58998e+03_rb,0.86862e+03_rb,0.11476e+04_rb /)
      kbo(:, 2,44, 9) = (/ &
        & 0.10343e+04_rb,0.78407e+03_rb,0.58962e+03_rb,0.86796e+03_rb,0.11467e+04_rb /)
      kbo(:, 3,44, 9) = (/ &
        & 0.10343e+04_rb,0.78406e+03_rb,0.58910e+03_rb,0.86711e+03_rb,0.11456e+04_rb /)
      kbo(:, 4,44, 9) = (/ &
        & 0.10343e+04_rb,0.78403e+03_rb,0.58835e+03_rb,0.86607e+03_rb,0.11442e+04_rb /)
      kbo(:, 5,44, 9) = (/ &
        & 0.10343e+04_rb,0.78403e+03_rb,0.58738e+03_rb,0.86483e+03_rb,0.11426e+04_rb /)
      kbo(:, 1,45, 9) = (/ &
        & 0.10343e+04_rb,0.78404e+03_rb,0.59009e+03_rb,0.86881e+03_rb,0.11478e+04_rb /)
      kbo(:, 2,45, 9) = (/ &
        & 0.10343e+04_rb,0.78405e+03_rb,0.58975e+03_rb,0.86821e+03_rb,0.11471e+04_rb /)
      kbo(:, 3,45, 9) = (/ &
        & 0.10343e+04_rb,0.78404e+03_rb,0.58930e+03_rb,0.86741e+03_rb,0.11460e+04_rb /)
      kbo(:, 4,45, 9) = (/ &
        & 0.10342e+04_rb,0.78400e+03_rb,0.58862e+03_rb,0.86644e+03_rb,0.11447e+04_rb /)
      kbo(:, 5,45, 9) = (/ &
        & 0.10343e+04_rb,0.78402e+03_rb,0.58769e+03_rb,0.86520e+03_rb,0.11432e+04_rb /)
      kbo(:, 1,46, 9) = (/ &
        & 0.10343e+04_rb,0.78403e+03_rb,0.59019e+03_rb,0.86899e+03_rb,0.11481e+04_rb /)
      kbo(:, 2,46, 9) = (/ &
        & 0.10343e+04_rb,0.78403e+03_rb,0.58989e+03_rb,0.86845e+03_rb,0.11474e+04_rb /)
      kbo(:, 3,46, 9) = (/ &
        & 0.10343e+04_rb,0.78403e+03_rb,0.58947e+03_rb,0.86773e+03_rb,0.11464e+04_rb /)
      kbo(:, 4,46, 9) = (/ &
        & 0.10342e+04_rb,0.78401e+03_rb,0.58890e+03_rb,0.86681e+03_rb,0.11452e+04_rb /)
      kbo(:, 5,46, 9) = (/ &
        & 0.10343e+04_rb,0.78405e+03_rb,0.58808e+03_rb,0.86568e+03_rb,0.11438e+04_rb /)
      kbo(:, 1,47, 9) = (/ &
        & 0.10343e+04_rb,0.78401e+03_rb,0.59032e+03_rb,0.86917e+03_rb,0.11483e+04_rb /)
      kbo(:, 2,47, 9) = (/ &
        & 0.10342e+04_rb,0.78398e+03_rb,0.59001e+03_rb,0.86868e+03_rb,0.11477e+04_rb /)
      kbo(:, 3,47, 9) = (/ &
        & 0.10342e+04_rb,0.78398e+03_rb,0.58967e+03_rb,0.86807e+03_rb,0.11468e+04_rb /)
      kbo(:, 4,47, 9) = (/ &
        & 0.10343e+04_rb,0.78397e+03_rb,0.58916e+03_rb,0.86720e+03_rb,0.11457e+04_rb /)
      kbo(:, 5,47, 9) = (/ &
        & 0.10342e+04_rb,0.78399e+03_rb,0.58844e+03_rb,0.86618e+03_rb,0.11444e+04_rb /)
      kbo(:, 1,48, 9) = (/ &
        & 0.10342e+04_rb,0.78399e+03_rb,0.59046e+03_rb,0.86932e+03_rb,0.11485e+04_rb /)
      kbo(:, 2,48, 9) = (/ &
        & 0.10343e+04_rb,0.78398e+03_rb,0.59013e+03_rb,0.86890e+03_rb,0.11480e+04_rb /)
      kbo(:, 3,48, 9) = (/ &
        & 0.10342e+04_rb,0.78402e+03_rb,0.58982e+03_rb,0.86829e+03_rb,0.11472e+04_rb /)
      kbo(:, 4,48, 9) = (/ &
        & 0.10342e+04_rb,0.78396e+03_rb,0.58938e+03_rb,0.86756e+03_rb,0.11462e+04_rb /)
      kbo(:, 5,48, 9) = (/ &
        & 0.10342e+04_rb,0.78400e+03_rb,0.58875e+03_rb,0.86661e+03_rb,0.11449e+04_rb /)
      kbo(:, 1,49, 9) = (/ &
        & 0.10343e+04_rb,0.78402e+03_rb,0.59060e+03_rb,0.86945e+03_rb,0.11486e+04_rb /)
      kbo(:, 2,49, 9) = (/ &
        & 0.10342e+04_rb,0.78398e+03_rb,0.59025e+03_rb,0.86908e+03_rb,0.11482e+04_rb /)
      kbo(:, 3,49, 9) = (/ &
        & 0.10342e+04_rb,0.78399e+03_rb,0.58995e+03_rb,0.86856e+03_rb,0.11475e+04_rb /)
      kbo(:, 4,49, 9) = (/ &
        & 0.10342e+04_rb,0.78399e+03_rb,0.58958e+03_rb,0.86790e+03_rb,0.11466e+04_rb /)
      kbo(:, 5,49, 9) = (/ &
        & 0.10342e+04_rb,0.78398e+03_rb,0.58903e+03_rb,0.86700e+03_rb,0.11455e+04_rb /)
      kbo(:, 1,50, 9) = (/ &
        & 0.10343e+04_rb,0.78395e+03_rb,0.59074e+03_rb,0.86956e+03_rb,0.11487e+04_rb /)
      kbo(:, 2,50, 9) = (/ &
        & 0.10342e+04_rb,0.78399e+03_rb,0.59038e+03_rb,0.86924e+03_rb,0.11483e+04_rb /)
      kbo(:, 3,50, 9) = (/ &
        & 0.10342e+04_rb,0.78394e+03_rb,0.59006e+03_rb,0.86878e+03_rb,0.11478e+04_rb /)
      kbo(:, 4,50, 9) = (/ &
        & 0.10342e+04_rb,0.78399e+03_rb,0.58974e+03_rb,0.86817e+03_rb,0.11470e+04_rb /)
      kbo(:, 5,50, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.58927e+03_rb,0.86736e+03_rb,0.11460e+04_rb /)
      kbo(:, 1,51, 9) = (/ &
        & 0.10342e+04_rb,0.78399e+03_rb,0.59088e+03_rb,0.86965e+03_rb,0.11488e+04_rb /)
      kbo(:, 2,51, 9) = (/ &
        & 0.10342e+04_rb,0.78393e+03_rb,0.59051e+03_rb,0.86937e+03_rb,0.11485e+04_rb /)
      kbo(:, 3,51, 9) = (/ &
        & 0.10342e+04_rb,0.78400e+03_rb,0.59018e+03_rb,0.86897e+03_rb,0.11480e+04_rb /)
      kbo(:, 4,51, 9) = (/ &
        & 0.10342e+04_rb,0.78396e+03_rb,0.58987e+03_rb,0.86840e+03_rb,0.11473e+04_rb /)
      kbo(:, 5,51, 9) = (/ &
        & 0.10342e+04_rb,0.78396e+03_rb,0.58946e+03_rb,0.86768e+03_rb,0.11464e+04_rb /)
      kbo(:, 1,52, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.59100e+03_rb,0.86971e+03_rb,0.11490e+04_rb /)
      kbo(:, 2,52, 9) = (/ &
        & 0.10342e+04_rb,0.78394e+03_rb,0.59064e+03_rb,0.86949e+03_rb,0.11486e+04_rb /)
      kbo(:, 3,52, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.59029e+03_rb,0.86915e+03_rb,0.11482e+04_rb /)
      kbo(:, 4,52, 9) = (/ &
        & 0.10342e+04_rb,0.78394e+03_rb,0.58998e+03_rb,0.86864e+03_rb,0.11476e+04_rb /)
      kbo(:, 5,52, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.58963e+03_rb,0.86796e+03_rb,0.11468e+04_rb /)
      kbo(:, 1,53, 9) = (/ &
        & 0.10342e+04_rb,0.78397e+03_rb,0.59111e+03_rb,0.86975e+03_rb,0.11490e+04_rb /)
      kbo(:, 2,53, 9) = (/ &
        & 0.10342e+04_rb,0.78394e+03_rb,0.59078e+03_rb,0.86960e+03_rb,0.11488e+04_rb /)
      kbo(:, 3,53, 9) = (/ &
        & 0.10342e+04_rb,0.78396e+03_rb,0.59041e+03_rb,0.86926e+03_rb,0.11484e+04_rb /)
      kbo(:, 4,53, 9) = (/ &
        & 0.10341e+04_rb,0.78393e+03_rb,0.59010e+03_rb,0.86884e+03_rb,0.11479e+04_rb /)
      kbo(:, 5,53, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.58977e+03_rb,0.86822e+03_rb,0.11471e+04_rb /)
      kbo(:, 1,54, 9) = (/ &
        & 0.10343e+04_rb,0.78394e+03_rb,0.59121e+03_rb,0.86979e+03_rb,0.11491e+04_rb /)
      kbo(:, 2,54, 9) = (/ &
        & 0.10341e+04_rb,0.78399e+03_rb,0.59088e+03_rb,0.86966e+03_rb,0.11489e+04_rb /)
      kbo(:, 3,54, 9) = (/ &
        & 0.10341e+04_rb,0.78394e+03_rb,0.59052e+03_rb,0.86939e+03_rb,0.11485e+04_rb /)
      kbo(:, 4,54, 9) = (/ &
        & 0.10341e+04_rb,0.78394e+03_rb,0.59020e+03_rb,0.86900e+03_rb,0.11481e+04_rb /)
      kbo(:, 5,54, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.58987e+03_rb,0.86845e+03_rb,0.11474e+04_rb /)
      kbo(:, 1,55, 9) = (/ &
        & 0.10342e+04_rb,0.78396e+03_rb,0.59130e+03_rb,0.86983e+03_rb,0.11491e+04_rb /)
      kbo(:, 2,55, 9) = (/ &
        & 0.10341e+04_rb,0.78393e+03_rb,0.59100e+03_rb,0.86971e+03_rb,0.11490e+04_rb /)
      kbo(:, 3,55, 9) = (/ &
        & 0.10342e+04_rb,0.78393e+03_rb,0.59065e+03_rb,0.86949e+03_rb,0.11486e+04_rb /)
      kbo(:, 4,55, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.59029e+03_rb,0.86914e+03_rb,0.11483e+04_rb /)
      kbo(:, 5,55, 9) = (/ &
        & 0.10341e+04_rb,0.78393e+03_rb,0.59000e+03_rb,0.86864e+03_rb,0.11477e+04_rb /)
      kbo(:, 1,56, 9) = (/ &
        & 0.10342e+04_rb,0.78393e+03_rb,0.59137e+03_rb,0.86987e+03_rb,0.11492e+04_rb /)
      kbo(:, 2,56, 9) = (/ &
        & 0.10342e+04_rb,0.78395e+03_rb,0.59111e+03_rb,0.86975e+03_rb,0.11490e+04_rb /)
      kbo(:, 3,56, 9) = (/ &
        & 0.10342e+04_rb,0.78392e+03_rb,0.59077e+03_rb,0.86958e+03_rb,0.11488e+04_rb /)
      kbo(:, 4,56, 9) = (/ &
        & 0.10342e+04_rb,0.78393e+03_rb,0.59040e+03_rb,0.86926e+03_rb,0.11484e+04_rb /)
      kbo(:, 5,56, 9) = (/ &
        & 0.10341e+04_rb,0.78393e+03_rb,0.59009e+03_rb,0.86883e+03_rb,0.11479e+04_rb /)
      kbo(:, 1,57, 9) = (/ &
        & 0.10342e+04_rb,0.78396e+03_rb,0.59143e+03_rb,0.86993e+03_rb,0.11492e+04_rb /)
      kbo(:, 2,57, 9) = (/ &
        & 0.10342e+04_rb,0.78392e+03_rb,0.59122e+03_rb,0.86979e+03_rb,0.11491e+04_rb /)
      kbo(:, 3,57, 9) = (/ &
        & 0.10342e+04_rb,0.78392e+03_rb,0.59089e+03_rb,0.86966e+03_rb,0.11489e+04_rb /)
      kbo(:, 4,57, 9) = (/ &
        & 0.10341e+04_rb,0.78395e+03_rb,0.59052e+03_rb,0.86938e+03_rb,0.11485e+04_rb /)
      kbo(:, 5,57, 9) = (/ &
        & 0.10341e+04_rb,0.78393e+03_rb,0.59018e+03_rb,0.86897e+03_rb,0.11481e+04_rb /)
      kbo(:, 1,58, 9) = (/ &
        & 0.10341e+04_rb,0.78395e+03_rb,0.59147e+03_rb,0.86996e+03_rb,0.11493e+04_rb /)
      kbo(:, 2,58, 9) = (/ &
        & 0.10341e+04_rb,0.78397e+03_rb,0.59129e+03_rb,0.86982e+03_rb,0.11491e+04_rb /)
      kbo(:, 3,58, 9) = (/ &
        & 0.10341e+04_rb,0.78393e+03_rb,0.59099e+03_rb,0.86970e+03_rb,0.11489e+04_rb /)
      kbo(:, 4,58, 9) = (/ &
        & 0.10342e+04_rb,0.78397e+03_rb,0.59063e+03_rb,0.86948e+03_rb,0.11486e+04_rb /)
      kbo(:, 5,58, 9) = (/ &
        & 0.10342e+04_rb,0.78390e+03_rb,0.59028e+03_rb,0.86913e+03_rb,0.11482e+04_rb /)
      kbo(:, 1,59, 9) = (/ &
        & 0.10342e+04_rb,0.78393e+03_rb,0.59148e+03_rb,0.86998e+03_rb,0.11494e+04_rb /)
      kbo(:, 2,59, 9) = (/ &
        & 0.10342e+04_rb,0.78396e+03_rb,0.59132e+03_rb,0.86982e+03_rb,0.11492e+04_rb /)
      kbo(:, 3,59, 9) = (/ &
        & 0.10342e+04_rb,0.78393e+03_rb,0.59103e+03_rb,0.86972e+03_rb,0.11490e+04_rb /)
      kbo(:, 4,59, 9) = (/ &
        & 0.10341e+04_rb,0.78394e+03_rb,0.59067e+03_rb,0.86951e+03_rb,0.11487e+04_rb /)
      kbo(:, 5,59, 9) = (/ &
        & 0.10341e+04_rb,0.78397e+03_rb,0.59032e+03_rb,0.86918e+03_rb,0.11483e+04_rb /)
      kbo(:, 1,13,10) = (/ &
        & 0.11544e+04_rb,0.87486e+03_rb,0.59975e+03_rb,0.88029e+03_rb,0.11631e+04_rb /)
      kbo(:, 2,13,10) = (/ &
        & 0.11458e+04_rb,0.86752e+03_rb,0.59921e+03_rb,0.87889e+03_rb,0.11611e+04_rb /)
      kbo(:, 3,13,10) = (/ &
        & 0.11380e+04_rb,0.86177e+03_rb,0.59836e+03_rb,0.87789e+03_rb,0.11592e+04_rb /)
      kbo(:, 4,13,10) = (/ &
        & 0.11310e+04_rb,0.85697e+03_rb,0.59745e+03_rb,0.87697e+03_rb,0.11577e+04_rb /)
      kbo(:, 5,13,10) = (/ &
        & 0.11259e+04_rb,0.85279e+03_rb,0.59663e+03_rb,0.87606e+03_rb,0.11564e+04_rb /)
      kbo(:, 1,14,10) = (/ &
        & 0.11329e+04_rb,0.85757e+03_rb,0.59935e+03_rb,0.88000e+03_rb,0.11630e+04_rb /)
      kbo(:, 2,14,10) = (/ &
        & 0.11257e+04_rb,0.85228e+03_rb,0.59878e+03_rb,0.87841e+03_rb,0.11609e+04_rb /)
      kbo(:, 3,14,10) = (/ &
        & 0.11201e+04_rb,0.84731e+03_rb,0.59797e+03_rb,0.87766e+03_rb,0.11591e+04_rb /)
      kbo(:, 4,14,10) = (/ &
        & 0.11140e+04_rb,0.84389e+03_rb,0.59731e+03_rb,0.87707e+03_rb,0.11577e+04_rb /)
      kbo(:, 5,14,10) = (/ &
        & 0.11094e+04_rb,0.84001e+03_rb,0.59642e+03_rb,0.87571e+03_rb,0.11561e+04_rb /)
      kbo(:, 1,15,10) = (/ &
        & 0.11142e+04_rb,0.84370e+03_rb,0.59900e+03_rb,0.87973e+03_rb,0.11627e+04_rb /)
      kbo(:, 2,15,10) = (/ &
        & 0.11097e+04_rb,0.84029e+03_rb,0.59841e+03_rb,0.87827e+03_rb,0.11607e+04_rb /)
      kbo(:, 3,15,10) = (/ &
        & 0.11039e+04_rb,0.83570e+03_rb,0.59763e+03_rb,0.87745e+03_rb,0.11587e+04_rb /)
      kbo(:, 4,15,10) = (/ &
        & 0.10995e+04_rb,0.83295e+03_rb,0.59685e+03_rb,0.87664e+03_rb,0.11574e+04_rb /)
      kbo(:, 5,15,10) = (/ &
        & 0.10956e+04_rb,0.82966e+03_rb,0.59611e+03_rb,0.87548e+03_rb,0.11555e+04_rb /)
      kbo(:, 1,16,10) = (/ &
        & 0.11006e+04_rb,0.83290e+03_rb,0.59871e+03_rb,0.87947e+03_rb,0.11625e+04_rb /)
      kbo(:, 2,16,10) = (/ &
        & 0.10967e+04_rb,0.82960e+03_rb,0.59809e+03_rb,0.87696e+03_rb,0.11605e+04_rb /)
      kbo(:, 3,16,10) = (/ &
        & 0.10908e+04_rb,0.82627e+03_rb,0.59735e+03_rb,0.87716e+03_rb,0.11587e+04_rb /)
      kbo(:, 4,16,10) = (/ &
        & 0.10878e+04_rb,0.82496e+03_rb,0.59665e+03_rb,0.87645e+03_rb,0.11573e+04_rb /)
      kbo(:, 5,16,10) = (/ &
        & 0.10846e+04_rb,0.82140e+03_rb,0.59584e+03_rb,0.87537e+03_rb,0.11556e+04_rb /)
      kbo(:, 1,17,10) = (/ &
        & 0.10872e+04_rb,0.82399e+03_rb,0.59847e+03_rb,0.87912e+03_rb,0.11623e+04_rb /)
      kbo(:, 2,17,10) = (/ &
        & 0.10836e+04_rb,0.82137e+03_rb,0.59782e+03_rb,0.87785e+03_rb,0.11604e+04_rb /)
      kbo(:, 3,17,10) = (/ &
        & 0.10806e+04_rb,0.81903e+03_rb,0.59710e+03_rb,0.87700e+03_rb,0.11586e+04_rb /)
      kbo(:, 4,17,10) = (/ &
        & 0.10781e+04_rb,0.81649e+03_rb,0.59641e+03_rb,0.87627e+03_rb,0.11571e+04_rb /)
      kbo(:, 5,17,10) = (/ &
        & 0.10750e+04_rb,0.81461e+03_rb,0.59560e+03_rb,0.87517e+03_rb,0.11554e+04_rb /)
      kbo(:, 1,18,10) = (/ &
        & 0.10783e+04_rb,0.81627e+03_rb,0.59826e+03_rb,0.87902e+03_rb,0.11620e+04_rb /)
      kbo(:, 2,18,10) = (/ &
        & 0.10752e+04_rb,0.81450e+03_rb,0.59759e+03_rb,0.87778e+03_rb,0.11602e+04_rb /)
      kbo(:, 3,18,10) = (/ &
        & 0.10718e+04_rb,0.81284e+03_rb,0.59689e+03_rb,0.87707e+03_rb,0.11584e+04_rb /)
      kbo(:, 4,18,10) = (/ &
        & 0.10695e+04_rb,0.81051e+03_rb,0.59620e+03_rb,0.87578e+03_rb,0.11570e+04_rb /)
      kbo(:, 5,18,10) = (/ &
        & 0.10675e+04_rb,0.80859e+03_rb,0.59539e+03_rb,0.87498e+03_rb,0.11554e+04_rb /)
      kbo(:, 1,19,10) = (/ &
        & 0.10693e+04_rb,0.81014e+03_rb,0.59808e+03_rb,0.87871e+03_rb,0.11618e+04_rb /)
      kbo(:, 2,19,10) = (/ &
        & 0.10674e+04_rb,0.80961e+03_rb,0.59739e+03_rb,0.87763e+03_rb,0.11601e+04_rb /)
      kbo(:, 3,19,10) = (/ &
        & 0.10651e+04_rb,0.80782e+03_rb,0.59664e+03_rb,0.87683e+03_rb,0.11582e+04_rb /)
      kbo(:, 4,19,10) = (/ &
        & 0.10639e+04_rb,0.80562e+03_rb,0.59610e+03_rb,0.87595e+03_rb,0.11569e+04_rb /)
      kbo(:, 5,19,10) = (/ &
        & 0.10620e+04_rb,0.80440e+03_rb,0.59520e+03_rb,0.87481e+03_rb,0.11551e+04_rb /)
      kbo(:, 1,20,10) = (/ &
        & 0.10633e+04_rb,0.80580e+03_rb,0.59791e+03_rb,0.87862e+03_rb,0.11616e+04_rb /)
      kbo(:, 2,20,10) = (/ &
        & 0.10619e+04_rb,0.80442e+03_rb,0.59720e+03_rb,0.87747e+03_rb,0.11598e+04_rb /)
      kbo(:, 3,20,10) = (/ &
        & 0.10607e+04_rb,0.80344e+03_rb,0.59654e+03_rb,0.87680e+03_rb,0.11581e+04_rb /)
      kbo(:, 4,20,10) = (/ &
        & 0.10584e+04_rb,0.80182e+03_rb,0.59585e+03_rb,0.87568e+03_rb,0.11568e+04_rb /)
      kbo(:, 5,20,10) = (/ &
        & 0.10568e+04_rb,0.80142e+03_rb,0.59502e+03_rb,0.87461e+03_rb,0.11549e+04_rb /)
      kbo(:, 1,21,10) = (/ &
        & 0.10577e+04_rb,0.80247e+03_rb,0.59775e+03_rb,0.87831e+03_rb,0.11614e+04_rb /)
      kbo(:, 2,21,10) = (/ &
        & 0.10572e+04_rb,0.80108e+03_rb,0.59703e+03_rb,0.87733e+03_rb,0.11595e+04_rb /)
      kbo(:, 3,21,10) = (/ &
        & 0.10551e+04_rb,0.79955e+03_rb,0.59639e+03_rb,0.87689e+03_rb,0.11579e+04_rb /)
      kbo(:, 4,21,10) = (/ &
        & 0.10548e+04_rb,0.79914e+03_rb,0.59569e+03_rb,0.87563e+03_rb,0.11565e+04_rb /)
      kbo(:, 5,21,10) = (/ &
        & 0.10533e+04_rb,0.79794e+03_rb,0.59484e+03_rb,0.87454e+03_rb,0.11546e+04_rb /)
      kbo(:, 1,22,10) = (/ &
        & 0.10544e+04_rb,0.79901e+03_rb,0.59755e+03_rb,0.87816e+03_rb,0.11610e+04_rb /)
      kbo(:, 2,22,10) = (/ &
        & 0.10524e+04_rb,0.79865e+03_rb,0.59692e+03_rb,0.87715e+03_rb,0.11592e+04_rb /)
      kbo(:, 3,22,10) = (/ &
        & 0.10518e+04_rb,0.79733e+03_rb,0.59621e+03_rb,0.87638e+03_rb,0.11576e+04_rb /)
      kbo(:, 4,22,10) = (/ &
        & 0.10503e+04_rb,0.79602e+03_rb,0.59549e+03_rb,0.87540e+03_rb,0.11562e+04_rb /)
      kbo(:, 5,22,10) = (/ &
        & 0.10497e+04_rb,0.79577e+03_rb,0.59455e+03_rb,0.87415e+03_rb,0.11542e+04_rb /)
      kbo(:, 1,23,10) = (/ &
        & 0.10504e+04_rb,0.79618e+03_rb,0.59736e+03_rb,0.87790e+03_rb,0.11608e+04_rb /)
      kbo(:, 2,23,10) = (/ &
        & 0.10488e+04_rb,0.79542e+03_rb,0.59652e+03_rb,0.87687e+03_rb,0.11588e+04_rb /)
      kbo(:, 3,23,10) = (/ &
        & 0.10489e+04_rb,0.79466e+03_rb,0.59603e+03_rb,0.87619e+03_rb,0.11574e+04_rb /)
      kbo(:, 4,23,10) = (/ &
        & 0.10477e+04_rb,0.79399e+03_rb,0.59528e+03_rb,0.87505e+03_rb,0.11559e+04_rb /)
      kbo(:, 5,23,10) = (/ &
        & 0.10468e+04_rb,0.79357e+03_rb,0.59440e+03_rb,0.87388e+03_rb,0.11537e+04_rb /)
      kbo(:, 1,24,10) = (/ &
        & 0.10475e+04_rb,0.79355e+03_rb,0.59716e+03_rb,0.87766e+03_rb,0.11603e+04_rb /)
      kbo(:, 2,24,10) = (/ &
        & 0.10471e+04_rb,0.79325e+03_rb,0.59650e+03_rb,0.87681e+03_rb,0.11585e+04_rb /)
      kbo(:, 3,24,10) = (/ &
        & 0.10463e+04_rb,0.79243e+03_rb,0.59586e+03_rb,0.87587e+03_rb,0.11571e+04_rb /)
      kbo(:, 4,24,10) = (/ &
        & 0.10454e+04_rb,0.79206e+03_rb,0.59508e+03_rb,0.87468e+03_rb,0.11555e+04_rb /)
      kbo(:, 5,24,10) = (/ &
        & 0.10455e+04_rb,0.79146e+03_rb,0.59426e+03_rb,0.87360e+03_rb,0.11532e+04_rb /)
      kbo(:, 1,25,10) = (/ &
        & 0.10443e+04_rb,0.79222e+03_rb,0.59698e+03_rb,0.87742e+03_rb,0.11598e+04_rb /)
      kbo(:, 2,25,10) = (/ &
        & 0.10446e+04_rb,0.79165e+03_rb,0.59633e+03_rb,0.87696e+03_rb,0.11582e+04_rb /)
      kbo(:, 3,25,10) = (/ &
        & 0.10440e+04_rb,0.79123e+03_rb,0.59567e+03_rb,0.87565e+03_rb,0.11568e+04_rb /)
      kbo(:, 4,25,10) = (/ &
        & 0.10433e+04_rb,0.79106e+03_rb,0.59487e+03_rb,0.87451e+03_rb,0.11551e+04_rb /)
      kbo(:, 5,25,10) = (/ &
        & 0.10423e+04_rb,0.79063e+03_rb,0.59375e+03_rb,0.87329e+03_rb,0.11525e+04_rb /)
      kbo(:, 1,26,10) = (/ &
        & 0.10431e+04_rb,0.79112e+03_rb,0.59671e+03_rb,0.87720e+03_rb,0.11595e+04_rb /)
      kbo(:, 2,26,10) = (/ &
        & 0.10430e+04_rb,0.79016e+03_rb,0.59608e+03_rb,0.87633e+03_rb,0.11579e+04_rb /)
      kbo(:, 3,26,10) = (/ &
        & 0.10422e+04_rb,0.78992e+03_rb,0.59547e+03_rb,0.87551e+03_rb,0.11565e+04_rb /)
      kbo(:, 4,26,10) = (/ &
        & 0.10418e+04_rb,0.78943e+03_rb,0.59456e+03_rb,0.87419e+03_rb,0.11545e+04_rb /)
      kbo(:, 5,26,10) = (/ &
        & 0.10417e+04_rb,0.78966e+03_rb,0.59375e+03_rb,0.87285e+03_rb,0.11523e+04_rb /)
      kbo(:, 1,27,10) = (/ &
        & 0.10421e+04_rb,0.78970e+03_rb,0.59653e+03_rb,0.87700e+03_rb,0.11589e+04_rb /)
      kbo(:, 2,27,10) = (/ &
        & 0.10419e+04_rb,0.78911e+03_rb,0.59591e+03_rb,0.87623e+03_rb,0.11575e+04_rb /)
      kbo(:, 3,27,10) = (/ &
        & 0.10414e+04_rb,0.78914e+03_rb,0.59527e+03_rb,0.87524e+03_rb,0.11555e+04_rb /)
      kbo(:, 4,27,10) = (/ &
        & 0.10412e+04_rb,0.78891e+03_rb,0.59440e+03_rb,0.87398e+03_rb,0.11540e+04_rb /)
      kbo(:, 5,27,10) = (/ &
        & 0.10400e+04_rb,0.78850e+03_rb,0.59354e+03_rb,0.87262e+03_rb,0.11516e+04_rb /)
      kbo(:, 1,28,10) = (/ &
        & 0.10402e+04_rb,0.78860e+03_rb,0.59644e+03_rb,0.87682e+03_rb,0.11586e+04_rb /)
      kbo(:, 2,28,10) = (/ &
        & 0.10405e+04_rb,0.78788e+03_rb,0.59574e+03_rb,0.87579e+03_rb,0.11571e+04_rb /)
      kbo(:, 3,28,10) = (/ &
        & 0.10403e+04_rb,0.78855e+03_rb,0.59498e+03_rb,0.87473e+03_rb,0.11552e+04_rb /)
      kbo(:, 4,28,10) = (/ &
        & 0.10397e+04_rb,0.78826e+03_rb,0.59417e+03_rb,0.87377e+03_rb,0.11534e+04_rb /)
      kbo(:, 5,28,10) = (/ &
        & 0.10396e+04_rb,0.78810e+03_rb,0.59333e+03_rb,0.87227e+03_rb,0.11511e+04_rb /)
      kbo(:, 1,29,10) = (/ &
        & 0.10391e+04_rb,0.78797e+03_rb,0.59612e+03_rb,0.87685e+03_rb,0.11584e+04_rb /)
      kbo(:, 2,29,10) = (/ &
        & 0.10393e+04_rb,0.78808e+03_rb,0.59562e+03_rb,0.87577e+03_rb,0.11569e+04_rb /)
      kbo(:, 3,29,10) = (/ &
        & 0.10401e+04_rb,0.78791e+03_rb,0.59476e+03_rb,0.87463e+03_rb,0.11550e+04_rb /)
      kbo(:, 4,29,10) = (/ &
        & 0.10388e+04_rb,0.78727e+03_rb,0.59387e+03_rb,0.87332e+03_rb,0.11526e+04_rb /)
      kbo(:, 5,29,10) = (/ &
        & 0.10383e+04_rb,0.78753e+03_rb,0.59306e+03_rb,0.87189e+03_rb,0.11504e+04_rb /)
      kbo(:, 1,30,10) = (/ &
        & 0.10398e+04_rb,0.78748e+03_rb,0.59603e+03_rb,0.87643e+03_rb,0.11579e+04_rb /)
      kbo(:, 2,30,10) = (/ &
        & 0.10387e+04_rb,0.78733e+03_rb,0.59550e+03_rb,0.87550e+03_rb,0.11567e+04_rb /)
      kbo(:, 3,30,10) = (/ &
        & 0.10384e+04_rb,0.78700e+03_rb,0.59460e+03_rb,0.87431e+03_rb,0.11543e+04_rb /)
      kbo(:, 4,30,10) = (/ &
        & 0.10381e+04_rb,0.78738e+03_rb,0.59372e+03_rb,0.87275e+03_rb,0.11522e+04_rb /)
      kbo(:, 5,30,10) = (/ &
        & 0.10382e+04_rb,0.78707e+03_rb,0.59292e+03_rb,0.87137e+03_rb,0.11500e+04_rb /)
      kbo(:, 1,31,10) = (/ &
        & 0.10388e+04_rb,0.78740e+03_rb,0.59593e+03_rb,0.87621e+03_rb,0.11575e+04_rb /)
      kbo(:, 2,31,10) = (/ &
        & 0.10385e+04_rb,0.78757e+03_rb,0.59522e+03_rb,0.87522e+03_rb,0.11561e+04_rb /)
      kbo(:, 3,31,10) = (/ &
        & 0.10386e+04_rb,0.78718e+03_rb,0.59436e+03_rb,0.87397e+03_rb,0.11540e+04_rb /)
      kbo(:, 4,31,10) = (/ &
        & 0.10377e+04_rb,0.78650e+03_rb,0.59350e+03_rb,0.87250e+03_rb,0.11516e+04_rb /)
      kbo(:, 5,31,10) = (/ &
        & 0.10376e+04_rb,0.78651e+03_rb,0.59269e+03_rb,0.87108e+03_rb,0.11495e+04_rb /)
      kbo(:, 1,32,10) = (/ &
        & 0.10383e+04_rb,0.78682e+03_rb,0.59576e+03_rb,0.87597e+03_rb,0.11572e+04_rb /)
      kbo(:, 2,32,10) = (/ &
        & 0.10383e+04_rb,0.78712e+03_rb,0.59500e+03_rb,0.87480e+03_rb,0.11556e+04_rb /)
      kbo(:, 3,32,10) = (/ &
        & 0.10381e+04_rb,0.78606e+03_rb,0.59405e+03_rb,0.87362e+03_rb,0.11534e+04_rb /)
      kbo(:, 4,32,10) = (/ &
        & 0.10372e+04_rb,0.78628e+03_rb,0.59329e+03_rb,0.87223e+03_rb,0.11510e+04_rb /)
      kbo(:, 5,32,10) = (/ &
        & 0.10377e+04_rb,0.78690e+03_rb,0.59245e+03_rb,0.87068e+03_rb,0.11490e+04_rb /)
      kbo(:, 1,33,10) = (/ &
        & 0.10380e+04_rb,0.78593e+03_rb,0.59549e+03_rb,0.87572e+03_rb,0.11567e+04_rb /)
      kbo(:, 2,33,10) = (/ &
        & 0.10375e+04_rb,0.78654e+03_rb,0.59477e+03_rb,0.87458e+03_rb,0.11551e+04_rb /)
      kbo(:, 3,33,10) = (/ &
        & 0.10374e+04_rb,0.78686e+03_rb,0.59389e+03_rb,0.87337e+03_rb,0.11527e+04_rb /)
      kbo(:, 4,33,10) = (/ &
        & 0.10366e+04_rb,0.78593e+03_rb,0.59308e+03_rb,0.87194e+03_rb,0.11506e+04_rb /)
      kbo(:, 5,33,10) = (/ &
        & 0.10369e+04_rb,0.78626e+03_rb,0.59219e+03_rb,0.87038e+03_rb,0.11486e+04_rb /)
      kbo(:, 1,34,10) = (/ &
        & 0.10373e+04_rb,0.78657e+03_rb,0.59539e+03_rb,0.87559e+03_rb,0.11565e+04_rb /)
      kbo(:, 2,34,10) = (/ &
        & 0.10370e+04_rb,0.78641e+03_rb,0.59464e+03_rb,0.87429e+03_rb,0.11546e+04_rb /)
      kbo(:, 3,34,10) = (/ &
        & 0.10370e+04_rb,0.78587e+03_rb,0.59369e+03_rb,0.87295e+03_rb,0.11522e+04_rb /)
      kbo(:, 4,34,10) = (/ &
        & 0.10371e+04_rb,0.78562e+03_rb,0.59289e+03_rb,0.87146e+03_rb,0.11500e+04_rb /)
      kbo(:, 5,34,10) = (/ &
        & 0.10372e+04_rb,0.78597e+03_rb,0.59202e+03_rb,0.86991e+03_rb,0.11482e+04_rb /)
      kbo(:, 1,35,10) = (/ &
        & 0.10364e+04_rb,0.78559e+03_rb,0.59529e+03_rb,0.87523e+03_rb,0.11563e+04_rb /)
      kbo(:, 2,35,10) = (/ &
        & 0.10362e+04_rb,0.78545e+03_rb,0.59445e+03_rb,0.87400e+03_rb,0.11543e+04_rb /)
      kbo(:, 3,35,10) = (/ &
        & 0.10372e+04_rb,0.78599e+03_rb,0.59365e+03_rb,0.87265e+03_rb,0.11519e+04_rb /)
      kbo(:, 4,35,10) = (/ &
        & 0.10370e+04_rb,0.78527e+03_rb,0.59278e+03_rb,0.87136e+03_rb,0.11496e+04_rb /)
      kbo(:, 5,35,10) = (/ &
        & 0.10363e+04_rb,0.78582e+03_rb,0.59179e+03_rb,0.86905e+03_rb,0.11477e+04_rb /)
      kbo(:, 1,36,10) = (/ &
        & 0.10368e+04_rb,0.78515e+03_rb,0.59520e+03_rb,0.87532e+03_rb,0.11563e+04_rb /)
      kbo(:, 2,36,10) = (/ &
        & 0.10358e+04_rb,0.78589e+03_rb,0.59443e+03_rb,0.87419e+03_rb,0.11542e+04_rb /)
      kbo(:, 3,36,10) = (/ &
        & 0.10361e+04_rb,0.78556e+03_rb,0.59355e+03_rb,0.87273e+03_rb,0.11517e+04_rb /)
      kbo(:, 4,36,10) = (/ &
        & 0.10361e+04_rb,0.78543e+03_rb,0.59275e+03_rb,0.87122e+03_rb,0.11497e+04_rb /)
      kbo(:, 5,36,10) = (/ &
        & 0.10368e+04_rb,0.78570e+03_rb,0.59176e+03_rb,0.86999e+03_rb,0.11477e+04_rb /)
      kbo(:, 1,37,10) = (/ &
        & 0.10363e+04_rb,0.78591e+03_rb,0.59537e+03_rb,0.87546e+03_rb,0.11565e+04_rb /)
      kbo(:, 2,37,10) = (/ &
        & 0.10360e+04_rb,0.78548e+03_rb,0.59454e+03_rb,0.87425e+03_rb,0.11545e+04_rb /)
      kbo(:, 3,37,10) = (/ &
        & 0.10362e+04_rb,0.78546e+03_rb,0.59366e+03_rb,0.87291e+03_rb,0.11521e+04_rb /)
      kbo(:, 4,37,10) = (/ &
        & 0.10364e+04_rb,0.78543e+03_rb,0.59286e+03_rb,0.87142e+03_rb,0.11500e+04_rb /)
      kbo(:, 5,37,10) = (/ &
        & 0.10360e+04_rb,0.78560e+03_rb,0.59177e+03_rb,0.86987e+03_rb,0.11482e+04_rb /)
      kbo(:, 1,38,10) = (/ &
        & 0.10362e+04_rb,0.78542e+03_rb,0.59547e+03_rb,0.87560e+03_rb,0.11567e+04_rb /)
      kbo(:, 2,38,10) = (/ &
        & 0.10357e+04_rb,0.78588e+03_rb,0.59466e+03_rb,0.87433e+03_rb,0.11549e+04_rb /)
      kbo(:, 3,38,10) = (/ &
        & 0.10370e+04_rb,0.78605e+03_rb,0.59385e+03_rb,0.87310e+03_rb,0.11524e+04_rb /)
      kbo(:, 4,38,10) = (/ &
        & 0.10362e+04_rb,0.78574e+03_rb,0.59297e+03_rb,0.87186e+03_rb,0.11501e+04_rb /)
      kbo(:, 5,38,10) = (/ &
        & 0.10364e+04_rb,0.78533e+03_rb,0.59199e+03_rb,0.87009e+03_rb,0.11482e+04_rb /)
      kbo(:, 1,39,10) = (/ &
        & 0.10359e+04_rb,0.78514e+03_rb,0.59557e+03_rb,0.87574e+03_rb,0.11567e+04_rb /)
      kbo(:, 2,39,10) = (/ &
        & 0.10371e+04_rb,0.78541e+03_rb,0.59478e+03_rb,0.87461e+03_rb,0.11552e+04_rb /)
      kbo(:, 3,39,10) = (/ &
        & 0.10366e+04_rb,0.78569e+03_rb,0.59382e+03_rb,0.87329e+03_rb,0.11528e+04_rb /)
      kbo(:, 4,39,10) = (/ &
        & 0.10359e+04_rb,0.78528e+03_rb,0.59308e+03_rb,0.87198e+03_rb,0.11506e+04_rb /)
      kbo(:, 5,39,10) = (/ &
        & 0.10356e+04_rb,0.78487e+03_rb,0.59220e+03_rb,0.87042e+03_rb,0.11486e+04_rb /)
      kbo(:, 1,40,10) = (/ &
        & 0.10357e+04_rb,0.78518e+03_rb,0.59566e+03_rb,0.87586e+03_rb,0.11569e+04_rb /)
      kbo(:, 2,40,10) = (/ &
        & 0.10357e+04_rb,0.78506e+03_rb,0.59498e+03_rb,0.87480e+03_rb,0.11555e+04_rb /)
      kbo(:, 3,40,10) = (/ &
        & 0.10366e+04_rb,0.78504e+03_rb,0.59396e+03_rb,0.87362e+03_rb,0.11534e+04_rb /)
      kbo(:, 4,40,10) = (/ &
        & 0.10363e+04_rb,0.78503e+03_rb,0.59319e+03_rb,0.87212e+03_rb,0.11511e+04_rb /)
      kbo(:, 5,40,10) = (/ &
        & 0.10363e+04_rb,0.78540e+03_rb,0.59243e+03_rb,0.87078e+03_rb,0.11490e+04_rb /)
      kbo(:, 1,41,10) = (/ &
        & 0.10363e+04_rb,0.78503e+03_rb,0.59590e+03_rb,0.87608e+03_rb,0.11576e+04_rb /)
      kbo(:, 2,41,10) = (/ &
        & 0.10356e+04_rb,0.78491e+03_rb,0.59519e+03_rb,0.87520e+03_rb,0.11561e+04_rb /)
      kbo(:, 3,41,10) = (/ &
        & 0.10364e+04_rb,0.78558e+03_rb,0.59433e+03_rb,0.87384e+03_rb,0.11540e+04_rb /)
      kbo(:, 4,41,10) = (/ &
        & 0.10357e+04_rb,0.78518e+03_rb,0.59354e+03_rb,0.87259e+03_rb,0.11516e+04_rb /)
      kbo(:, 5,41,10) = (/ &
        & 0.10360e+04_rb,0.78526e+03_rb,0.59259e+03_rb,0.87107e+03_rb,0.11495e+04_rb /)
      kbo(:, 1,42,10) = (/ &
        & 0.10360e+04_rb,0.78488e+03_rb,0.59605e+03_rb,0.87553e+03_rb,0.11576e+04_rb /)
      kbo(:, 2,42,10) = (/ &
        & 0.10363e+04_rb,0.78507e+03_rb,0.59538e+03_rb,0.87537e+03_rb,0.11565e+04_rb /)
      kbo(:, 3,42,10) = (/ &
        & 0.10362e+04_rb,0.78535e+03_rb,0.59456e+03_rb,0.87429e+03_rb,0.11545e+04_rb /)
      kbo(:, 4,42,10) = (/ &
        & 0.10361e+04_rb,0.78514e+03_rb,0.59368e+03_rb,0.87295e+03_rb,0.11522e+04_rb /)
      kbo(:, 5,42,10) = (/ &
        & 0.10360e+04_rb,0.78523e+03_rb,0.59281e+03_rb,0.87146e+03_rb,0.11502e+04_rb /)
      kbo(:, 1,43,10) = (/ &
        & 0.10359e+04_rb,0.78504e+03_rb,0.59624e+03_rb,0.87675e+03_rb,0.11584e+04_rb /)
      kbo(:, 2,43,10) = (/ &
        & 0.10355e+04_rb,0.78552e+03_rb,0.59568e+03_rb,0.87580e+03_rb,0.11570e+04_rb /)
      kbo(:, 3,43,10) = (/ &
        & 0.10354e+04_rb,0.78512e+03_rb,0.59483e+03_rb,0.87469e+03_rb,0.11553e+04_rb /)
      kbo(:, 4,43,10) = (/ &
        & 0.10354e+04_rb,0.78569e+03_rb,0.59395e+03_rb,0.87316e+03_rb,0.11531e+04_rb /)
      kbo(:, 5,43,10) = (/ &
        & 0.10359e+04_rb,0.78598e+03_rb,0.59313e+03_rb,0.87186e+03_rb,0.11507e+04_rb /)
      kbo(:, 1,44,10) = (/ &
        & 0.10359e+04_rb,0.78521e+03_rb,0.59645e+03_rb,0.87700e+03_rb,0.11590e+04_rb /)
      kbo(:, 2,44,10) = (/ &
        & 0.10361e+04_rb,0.78510e+03_rb,0.59583e+03_rb,0.87611e+03_rb,0.11574e+04_rb /)
      kbo(:, 3,44,10) = (/ &
        & 0.10368e+04_rb,0.78519e+03_rb,0.59511e+03_rb,0.87509e+03_rb,0.11561e+04_rb /)
      kbo(:, 4,44,10) = (/ &
        & 0.10360e+04_rb,0.78508e+03_rb,0.59424e+03_rb,0.87372e+03_rb,0.11538e+04_rb /)
      kbo(:, 5,44,10) = (/ &
        & 0.10356e+04_rb,0.78537e+03_rb,0.59339e+03_rb,0.87213e+03_rb,0.11514e+04_rb /)
      kbo(:, 1,45,10) = (/ &
        & 0.10357e+04_rb,0.78557e+03_rb,0.59666e+03_rb,0.87715e+03_rb,0.11597e+04_rb /)
      kbo(:, 2,45,10) = (/ &
        & 0.10356e+04_rb,0.78527e+03_rb,0.59604e+03_rb,0.87639e+03_rb,0.11579e+04_rb /)
      kbo(:, 3,45,10) = (/ &
        & 0.10360e+04_rb,0.78546e+03_rb,0.59537e+03_rb,0.87547e+03_rb,0.11565e+04_rb /)
      kbo(:, 4,45,10) = (/ &
        & 0.10361e+04_rb,0.78506e+03_rb,0.59454e+03_rb,0.87427e+03_rb,0.11544e+04_rb /)
      kbo(:, 5,45,10) = (/ &
        & 0.10353e+04_rb,0.78535e+03_rb,0.59366e+03_rb,0.87293e+03_rb,0.11520e+04_rb /)
      kbo(:, 1,46,10) = (/ &
        & 0.10358e+04_rb,0.78506e+03_rb,0.59691e+03_rb,0.87749e+03_rb,0.11602e+04_rb /)
      kbo(:, 2,46,10) = (/ &
        & 0.10353e+04_rb,0.78486e+03_rb,0.59626e+03_rb,0.87678e+03_rb,0.11584e+04_rb /)
      kbo(:, 3,46,10) = (/ &
        & 0.10353e+04_rb,0.78495e+03_rb,0.59563e+03_rb,0.87572e+03_rb,0.11570e+04_rb /)
      kbo(:, 4,46,10) = (/ &
        & 0.10363e+04_rb,0.78534e+03_rb,0.59478e+03_rb,0.87473e+03_rb,0.11554e+04_rb /)
      kbo(:, 5,46,10) = (/ &
        & 0.10355e+04_rb,0.78455e+03_rb,0.59398e+03_rb,0.87342e+03_rb,0.11530e+04_rb /)
      kbo(:, 1,47,10) = (/ &
        & 0.10357e+04_rb,0.78544e+03_rb,0.59719e+03_rb,0.87795e+03_rb,0.11609e+04_rb /)
      kbo(:, 2,47,10) = (/ &
        & 0.10355e+04_rb,0.78533e+03_rb,0.59651e+03_rb,0.87696e+03_rb,0.11589e+04_rb /)
      kbo(:, 3,47,10) = (/ &
        & 0.10358e+04_rb,0.78543e+03_rb,0.59590e+03_rb,0.87576e+03_rb,0.11576e+04_rb /)
      kbo(:, 4,47,10) = (/ &
        & 0.10351e+04_rb,0.78562e+03_rb,0.59519e+03_rb,0.87521e+03_rb,0.11561e+04_rb /)
      kbo(:, 5,47,10) = (/ &
        & 0.10359e+04_rb,0.78571e+03_rb,0.59433e+03_rb,0.87385e+03_rb,0.11540e+04_rb /)
      kbo(:, 1,48,10) = (/ &
        & 0.10363e+04_rb,0.78513e+03_rb,0.59744e+03_rb,0.87846e+03_rb,0.11618e+04_rb /)
      kbo(:, 2,48,10) = (/ &
        & 0.10360e+04_rb,0.78532e+03_rb,0.59677e+03_rb,0.87730e+03_rb,0.11598e+04_rb /)
      kbo(:, 3,48,10) = (/ &
        & 0.10365e+04_rb,0.78483e+03_rb,0.59614e+03_rb,0.87707e+03_rb,0.11581e+04_rb /)
      kbo(:, 4,48,10) = (/ &
        & 0.10355e+04_rb,0.78521e+03_rb,0.59549e+03_rb,0.87564e+03_rb,0.11567e+04_rb /)
      kbo(:, 5,48,10) = (/ &
        & 0.10355e+04_rb,0.78531e+03_rb,0.59469e+03_rb,0.87448e+03_rb,0.11550e+04_rb /)
      kbo(:, 1,49,10) = (/ &
        & 0.10351e+04_rb,0.78492e+03_rb,0.59768e+03_rb,0.87902e+03_rb,0.11625e+04_rb /)
      kbo(:, 2,49,10) = (/ &
        & 0.10355e+04_rb,0.78541e+03_rb,0.59705e+03_rb,0.87772e+03_rb,0.11606e+04_rb /)
      kbo(:, 3,49,10) = (/ &
        & 0.10352e+04_rb,0.78472e+03_rb,0.59639e+03_rb,0.87682e+03_rb,0.11587e+04_rb /)
      kbo(:, 4,49,10) = (/ &
        & 0.10356e+04_rb,0.78511e+03_rb,0.59577e+03_rb,0.87591e+03_rb,0.11574e+04_rb /)
      kbo(:, 5,49,10) = (/ &
        & 0.10352e+04_rb,0.78501e+03_rb,0.59510e+03_rb,0.87498e+03_rb,0.11558e+04_rb /)
      kbo(:, 1,50,10) = (/ &
        & 0.10361e+04_rb,0.78540e+03_rb,0.59770e+03_rb,0.87956e+03_rb,0.11633e+04_rb /)
      kbo(:, 2,50,10) = (/ &
        & 0.10357e+04_rb,0.78530e+03_rb,0.59731e+03_rb,0.87817e+03_rb,0.11612e+04_rb /)
      kbo(:, 3,50,10) = (/ &
        & 0.10365e+04_rb,0.78510e+03_rb,0.59662e+03_rb,0.87710e+03_rb,0.11596e+04_rb /)
      kbo(:, 4,50,10) = (/ &
        & 0.10365e+04_rb,0.78510e+03_rb,0.59601e+03_rb,0.87645e+03_rb,0.11578e+04_rb /)
      kbo(:, 5,50,10) = (/ &
        & 0.10352e+04_rb,0.78588e+03_rb,0.59525e+03_rb,0.87541e+03_rb,0.11564e+04_rb /)
      kbo(:, 1,51,10) = (/ &
        & 0.10356e+04_rb,0.78490e+03_rb,0.59781e+03_rb,0.88010e+03_rb,0.11639e+04_rb /)
      kbo(:, 2,51,10) = (/ &
        & 0.10356e+04_rb,0.78539e+03_rb,0.59751e+03_rb,0.87866e+03_rb,0.11620e+04_rb /)
      kbo(:, 3,51,10) = (/ &
        & 0.10359e+04_rb,0.78461e+03_rb,0.59687e+03_rb,0.87744e+03_rb,0.11601e+04_rb /)
      kbo(:, 4,51,10) = (/ &
        & 0.10352e+04_rb,0.78519e+03_rb,0.59623e+03_rb,0.87674e+03_rb,0.11583e+04_rb /)
      kbo(:, 5,51,10) = (/ &
        & 0.10356e+04_rb,0.78470e+03_rb,0.59552e+03_rb,0.87578e+03_rb,0.11569e+04_rb /)
      kbo(:, 1,52,10) = (/ &
        & 0.10352e+04_rb,0.78509e+03_rb,0.59800e+03_rb,0.88094e+03_rb,0.11643e+04_rb /)
      kbo(:, 2,52,10) = (/ &
        & 0.10355e+04_rb,0.78528e+03_rb,0.59763e+03_rb,0.87918e+03_rb,0.11627e+04_rb /)
      kbo(:, 3,52,10) = (/ &
        & 0.10361e+04_rb,0.78479e+03_rb,0.59712e+03_rb,0.87762e+03_rb,0.11608e+04_rb /)
      kbo(:, 4,52,10) = (/ &
        & 0.10358e+04_rb,0.78528e+03_rb,0.59645e+03_rb,0.87678e+03_rb,0.11589e+04_rb /)
      kbo(:, 5,52,10) = (/ &
        & 0.10355e+04_rb,0.78528e+03_rb,0.59584e+03_rb,0.87612e+03_rb,0.11573e+04_rb /)
      kbo(:, 1,53,10) = (/ &
        & 0.10357e+04_rb,0.78528e+03_rb,0.59832e+03_rb,0.88148e+03_rb,0.11650e+04_rb /)
      kbo(:, 2,53,10) = (/ &
        & 0.10357e+04_rb,0.78596e+03_rb,0.59772e+03_rb,0.87969e+03_rb,0.11634e+04_rb /)
      kbo(:, 3,53,10) = (/ &
        & 0.10361e+04_rb,0.78547e+03_rb,0.59737e+03_rb,0.87830e+03_rb,0.11614e+04_rb /)
      kbo(:, 4,53,10) = (/ &
        & 0.10362e+04_rb,0.78537e+03_rb,0.59668e+03_rb,0.87707e+03_rb,0.11596e+04_rb /)
      kbo(:, 5,53,10) = (/ &
        & 0.10356e+04_rb,0.78498e+03_rb,0.59606e+03_rb,0.87642e+03_rb,0.11578e+04_rb /)
      kbo(:, 1,54,10) = (/ &
        & 0.10351e+04_rb,0.78567e+03_rb,0.59851e+03_rb,0.88199e+03_rb,0.11656e+04_rb /)
      kbo(:, 2,54,10) = (/ &
        & 0.10366e+04_rb,0.78518e+03_rb,0.59776e+03_rb,0.88021e+03_rb,0.11638e+04_rb /)
      kbo(:, 3,54,10) = (/ &
        & 0.10364e+04_rb,0.78478e+03_rb,0.59753e+03_rb,0.87875e+03_rb,0.11621e+04_rb /)
      kbo(:, 4,54,10) = (/ &
        & 0.10356e+04_rb,0.78498e+03_rb,0.59692e+03_rb,0.87751e+03_rb,0.11602e+04_rb /)
      kbo(:, 5,54,10) = (/ &
        & 0.10362e+04_rb,0.78527e+03_rb,0.59627e+03_rb,0.87657e+03_rb,0.11584e+04_rb /)
      kbo(:, 1,55,10) = (/ &
        & 0.10355e+04_rb,0.78488e+03_rb,0.59876e+03_rb,0.88250e+03_rb,0.11663e+04_rb /)
      kbo(:, 2,55,10) = (/ &
        & 0.10362e+04_rb,0.78527e+03_rb,0.59801e+03_rb,0.88087e+03_rb,0.11643e+04_rb /)
      kbo(:, 3,55,10) = (/ &
        & 0.10356e+04_rb,0.78507e+03_rb,0.59764e+03_rb,0.87920e+03_rb,0.11628e+04_rb /)
      kbo(:, 4,55,10) = (/ &
        & 0.10353e+04_rb,0.78488e+03_rb,0.59714e+03_rb,0.87787e+03_rb,0.11606e+04_rb /)
      kbo(:, 5,55,10) = (/ &
        & 0.10356e+04_rb,0.78488e+03_rb,0.59639e+03_rb,0.87691e+03_rb,0.11585e+04_rb /)
      kbo(:, 1,56,10) = (/ &
        & 0.10353e+04_rb,0.78537e+03_rb,0.59904e+03_rb,0.88318e+03_rb,0.11672e+04_rb /)
      kbo(:, 2,56,10) = (/ &
        & 0.10354e+04_rb,0.78497e+03_rb,0.59830e+03_rb,0.88144e+03_rb,0.11649e+04_rb /)
      kbo(:, 3,56,10) = (/ &
        & 0.10361e+04_rb,0.78536e+03_rb,0.59771e+03_rb,0.87966e+03_rb,0.11634e+04_rb /)
      kbo(:, 4,56,10) = (/ &
        & 0.10357e+04_rb,0.78497e+03_rb,0.59736e+03_rb,0.87826e+03_rb,0.11615e+04_rb /)
      kbo(:, 5,56,10) = (/ &
        & 0.10353e+04_rb,0.78478e+03_rb,0.59667e+03_rb,0.87705e+03_rb,0.11595e+04_rb /)
      kbo(:, 1,57,10) = (/ &
        & 0.10358e+04_rb,0.78458e+03_rb,0.59927e+03_rb,0.88350e+03_rb,0.11679e+04_rb /)
      kbo(:, 2,57,10) = (/ &
        & 0.10352e+04_rb,0.78536e+03_rb,0.59840e+03_rb,0.88193e+03_rb,0.11656e+04_rb /)
      kbo(:, 3,57,10) = (/ &
        & 0.10352e+04_rb,0.78526e+03_rb,0.59782e+03_rb,0.88003e+03_rb,0.11638e+04_rb /)
      kbo(:, 4,57,10) = (/ &
        & 0.10358e+04_rb,0.78497e+03_rb,0.59751e+03_rb,0.87869e+03_rb,0.11622e+04_rb /)
      kbo(:, 5,57,10) = (/ &
        & 0.10360e+04_rb,0.78497e+03_rb,0.59689e+03_rb,0.87746e+03_rb,0.11601e+04_rb /)
      kbo(:, 1,58,10) = (/ &
        & 0.10358e+04_rb,0.78468e+03_rb,0.59969e+03_rb,0.88402e+03_rb,0.11685e+04_rb /)
      kbo(:, 2,58,10) = (/ &
        & 0.10357e+04_rb,0.78458e+03_rb,0.59873e+03_rb,0.88241e+03_rb,0.11662e+04_rb /)
      kbo(:, 3,58,10) = (/ &
        & 0.10358e+04_rb,0.78526e+03_rb,0.59797e+03_rb,0.88085e+03_rb,0.11642e+04_rb /)
      kbo(:, 4,58,10) = (/ &
        & 0.10356e+04_rb,0.78487e+03_rb,0.59762e+03_rb,0.87912e+03_rb,0.11626e+04_rb /)
      kbo(:, 5,58,10) = (/ &
        & 0.10357e+04_rb,0.78565e+03_rb,0.59710e+03_rb,0.87769e+03_rb,0.11608e+04_rb /)
      kbo(:, 1,59,10) = (/ &
        & 0.10360e+04_rb,0.78507e+03_rb,0.59979e+03_rb,0.88416e+03_rb,0.11687e+04_rb /)
      kbo(:, 2,59,10) = (/ &
        & 0.10357e+04_rb,0.78477e+03_rb,0.59879e+03_rb,0.88260e+03_rb,0.11665e+04_rb /)
      kbo(:, 3,59,10) = (/ &
        & 0.10350e+04_rb,0.78526e+03_rb,0.59804e+03_rb,0.88100e+03_rb,0.11644e+04_rb /)
      kbo(:, 4,59,10) = (/ &
        & 0.10357e+04_rb,0.78467e+03_rb,0.59773e+03_rb,0.87930e+03_rb,0.11629e+04_rb /)
      kbo(:, 5,59,10) = (/ &
        & 0.10358e+04_rb,0.78457e+03_rb,0.59718e+03_rb,0.87783e+03_rb,0.11609e+04_rb /)
      kbo(:, 1,13,11) = (/ &
        & 0.11563e+04_rb,0.87491e+03_rb,0.60253e+03_rb,0.88362e+03_rb,0.11664e+04_rb /)
      kbo(:, 2,13,11) = (/ &
        & 0.11454e+04_rb,0.86861e+03_rb,0.60211e+03_rb,0.88300e+03_rb,0.11639e+04_rb /)
      kbo(:, 3,13,11) = (/ &
        & 0.11392e+04_rb,0.86181e+03_rb,0.60140e+03_rb,0.88181e+03_rb,0.11620e+04_rb /)
      kbo(:, 4,13,11) = (/ &
        & 0.11315e+04_rb,0.85680e+03_rb,0.60042e+03_rb,0.88057e+03_rb,0.11602e+04_rb /)
      kbo(:, 5,13,11) = (/ &
        & 0.11273e+04_rb,0.85330e+03_rb,0.59929e+03_rb,0.87908e+03_rb,0.11581e+04_rb /)
      kbo(:, 1,14,11) = (/ &
        & 0.11333e+04_rb,0.85776e+03_rb,0.60215e+03_rb,0.88338e+03_rb,0.11660e+04_rb /)
      kbo(:, 2,14,11) = (/ &
        & 0.11256e+04_rb,0.85211e+03_rb,0.60176e+03_rb,0.88258e+03_rb,0.11637e+04_rb /)
      kbo(:, 3,14,11) = (/ &
        & 0.11185e+04_rb,0.84809e+03_rb,0.60101e+03_rb,0.88154e+03_rb,0.11618e+04_rb /)
      kbo(:, 4,14,11) = (/ &
        & 0.11137e+04_rb,0.84328e+03_rb,0.59992e+03_rb,0.87989e+03_rb,0.11599e+04_rb /)
      kbo(:, 5,14,11) = (/ &
        & 0.11092e+04_rb,0.83997e+03_rb,0.59898e+03_rb,0.87879e+03_rb,0.11581e+04_rb /)
      kbo(:, 1,15,11) = (/ &
        & 0.11149e+04_rb,0.84435e+03_rb,0.60192e+03_rb,0.88317e+03_rb,0.11660e+04_rb /)
      kbo(:, 2,15,11) = (/ &
        & 0.11071e+04_rb,0.83899e+03_rb,0.60145e+03_rb,0.88233e+03_rb,0.11635e+04_rb /)
      kbo(:, 3,15,11) = (/ &
        & 0.11058e+04_rb,0.83612e+03_rb,0.60066e+03_rb,0.88128e+03_rb,0.11617e+04_rb /)
      kbo(:, 4,15,11) = (/ &
        & 0.10996e+04_rb,0.83410e+03_rb,0.59965e+03_rb,0.88002e+03_rb,0.11599e+04_rb /)
      kbo(:, 5,15,11) = (/ &
        & 0.10957e+04_rb,0.83022e+03_rb,0.59863e+03_rb,0.87850e+03_rb,0.11578e+04_rb /)
      kbo(:, 1,16,11) = (/ &
        & 0.10993e+04_rb,0.83361e+03_rb,0.60157e+03_rb,0.88298e+03_rb,0.11657e+04_rb /)
      kbo(:, 2,16,11) = (/ &
        & 0.10933e+04_rb,0.82961e+03_rb,0.60127e+03_rb,0.88210e+03_rb,0.11633e+04_rb /)
      kbo(:, 3,16,11) = (/ &
        & 0.10924e+04_rb,0.82745e+03_rb,0.60035e+03_rb,0.88104e+03_rb,0.11615e+04_rb /)
      kbo(:, 4,16,11) = (/ &
        & 0.10885e+04_rb,0.82319e+03_rb,0.59934e+03_rb,0.87977e+03_rb,0.11597e+04_rb /)
      kbo(:, 5,16,11) = (/ &
        & 0.10838e+04_rb,0.82062e+03_rb,0.59823e+03_rb,0.87823e+03_rb,0.11577e+04_rb /)
      kbo(:, 1,17,11) = (/ &
        & 0.10881e+04_rb,0.82385e+03_rb,0.60135e+03_rb,0.88282e+03_rb,0.11656e+04_rb /)
      kbo(:, 2,17,11) = (/ &
        & 0.10840e+04_rb,0.82201e+03_rb,0.60094e+03_rb,0.88190e+03_rb,0.11631e+04_rb /)
      kbo(:, 3,17,11) = (/ &
        & 0.10809e+04_rb,0.81908e+03_rb,0.60007e+03_rb,0.88083e+03_rb,0.11613e+04_rb /)
      kbo(:, 4,17,11) = (/ &
        & 0.10772e+04_rb,0.81792e+03_rb,0.59907e+03_rb,0.87953e+03_rb,0.11595e+04_rb /)
      kbo(:, 5,17,11) = (/ &
        & 0.10760e+04_rb,0.81580e+03_rb,0.59806e+03_rb,0.87798e+03_rb,0.11575e+04_rb /)
      kbo(:, 1,18,11) = (/ &
        & 0.10780e+04_rb,0.81679e+03_rb,0.60116e+03_rb,0.88267e+03_rb,0.11653e+04_rb /)
      kbo(:, 2,18,11) = (/ &
        & 0.10749e+04_rb,0.81512e+03_rb,0.60072e+03_rb,0.88171e+03_rb,0.11629e+04_rb /)
      kbo(:, 3,18,11) = (/ &
        & 0.10728e+04_rb,0.81180e+03_rb,0.59983e+03_rb,0.88050e+03_rb,0.11611e+04_rb /)
      kbo(:, 4,18,11) = (/ &
        & 0.10707e+04_rb,0.81081e+03_rb,0.59883e+03_rb,0.87944e+03_rb,0.11593e+04_rb /)
      kbo(:, 5,18,11) = (/ &
        & 0.10678e+04_rb,0.80905e+03_rb,0.59782e+03_rb,0.87775e+03_rb,0.11574e+04_rb /)
      kbo(:, 1,19,11) = (/ &
        & 0.10696e+04_rb,0.81122e+03_rb,0.60101e+03_rb,0.88252e+03_rb,0.11649e+04_rb /)
      kbo(:, 2,19,11) = (/ &
        & 0.10692e+04_rb,0.80792e+03_rb,0.60053e+03_rb,0.88155e+03_rb,0.11625e+04_rb /)
      kbo(:, 3,19,11) = (/ &
        & 0.10653e+04_rb,0.80791e+03_rb,0.59962e+03_rb,0.88058e+03_rb,0.11609e+04_rb /)
      kbo(:, 4,19,11) = (/ &
        & 0.10635e+04_rb,0.80651e+03_rb,0.59853e+03_rb,0.87910e+03_rb,0.11591e+04_rb /)
      kbo(:, 5,19,11) = (/ &
        & 0.10626e+04_rb,0.80552e+03_rb,0.59761e+03_rb,0.87753e+03_rb,0.11572e+04_rb /)
      kbo(:, 1,20,11) = (/ &
        & 0.10638e+04_rb,0.80613e+03_rb,0.60096e+03_rb,0.88236e+03_rb,0.11646e+04_rb /)
      kbo(:, 2,20,11) = (/ &
        & 0.10623e+04_rb,0.80416e+03_rb,0.60035e+03_rb,0.88137e+03_rb,0.11625e+04_rb /)
      kbo(:, 3,20,11) = (/ &
        & 0.10590e+04_rb,0.80264e+03_rb,0.59950e+03_rb,0.88013e+03_rb,0.11607e+04_rb /)
      kbo(:, 4,20,11) = (/ &
        & 0.10603e+04_rb,0.80308e+03_rb,0.59841e+03_rb,0.87915e+03_rb,0.11587e+04_rb /)
      kbo(:, 5,20,11) = (/ &
        & 0.10573e+04_rb,0.80101e+03_rb,0.59740e+03_rb,0.87729e+03_rb,0.11568e+04_rb /)
      kbo(:, 1,21,11) = (/ &
        & 0.10580e+04_rb,0.80255e+03_rb,0.60075e+03_rb,0.88233e+03_rb,0.11643e+04_rb /)
      kbo(:, 2,21,11) = (/ &
        & 0.10580e+04_rb,0.80172e+03_rb,0.60017e+03_rb,0.88121e+03_rb,0.11623e+04_rb /)
      kbo(:, 3,21,11) = (/ &
        & 0.10551e+04_rb,0.80129e+03_rb,0.59922e+03_rb,0.87954e+03_rb,0.11605e+04_rb /)
      kbo(:, 4,21,11) = (/ &
        & 0.10544e+04_rb,0.79894e+03_rb,0.59822e+03_rb,0.87866e+03_rb,0.11586e+04_rb /)
      kbo(:, 5,21,11) = (/ &
        & 0.10531e+04_rb,0.79850e+03_rb,0.59721e+03_rb,0.87693e+03_rb,0.11568e+04_rb /)
      kbo(:, 1,22,11) = (/ &
        & 0.10535e+04_rb,0.79809e+03_rb,0.60064e+03_rb,0.88198e+03_rb,0.11639e+04_rb /)
      kbo(:, 2,22,11) = (/ &
        & 0.10534e+04_rb,0.79679e+03_rb,0.59986e+03_rb,0.88098e+03_rb,0.11619e+04_rb /)
      kbo(:, 3,22,11) = (/ &
        & 0.10517e+04_rb,0.79764e+03_rb,0.59897e+03_rb,0.87993e+03_rb,0.11602e+04_rb /)
      kbo(:, 4,22,11) = (/ &
        & 0.10510e+04_rb,0.79739e+03_rb,0.59797e+03_rb,0.87835e+03_rb,0.11582e+04_rb /)
      kbo(:, 5,22,11) = (/ &
        & 0.10492e+04_rb,0.79579e+03_rb,0.59697e+03_rb,0.87672e+03_rb,0.11565e+04_rb /)
      kbo(:, 1,23,11) = (/ &
        & 0.10516e+04_rb,0.79606e+03_rb,0.60095e+03_rb,0.88176e+03_rb,0.11633e+04_rb /)
      kbo(:, 2,23,11) = (/ &
        & 0.10497e+04_rb,0.79542e+03_rb,0.59972e+03_rb,0.88074e+03_rb,0.11616e+04_rb /)
      kbo(:, 3,23,11) = (/ &
        & 0.10482e+04_rb,0.79453e+03_rb,0.59873e+03_rb,0.87952e+03_rb,0.11598e+04_rb /)
      kbo(:, 4,23,11) = (/ &
        & 0.10486e+04_rb,0.79431e+03_rb,0.59782e+03_rb,0.87802e+03_rb,0.11579e+04_rb /)
      kbo(:, 5,23,11) = (/ &
        & 0.10473e+04_rb,0.79522e+03_rb,0.59672e+03_rb,0.87637e+03_rb,0.11562e+04_rb /)
      kbo(:, 1,24,11) = (/ &
        & 0.10467e+04_rb,0.79476e+03_rb,0.60034e+03_rb,0.88154e+03_rb,0.11630e+04_rb /)
      kbo(:, 2,24,11) = (/ &
        & 0.10461e+04_rb,0.79357e+03_rb,0.59948e+03_rb,0.88049e+03_rb,0.11613e+04_rb /)
      kbo(:, 3,24,11) = (/ &
        & 0.10460e+04_rb,0.79280e+03_rb,0.59849e+03_rb,0.87921e+03_rb,0.11596e+04_rb /)
      kbo(:, 4,24,11) = (/ &
        & 0.10457e+04_rb,0.79222e+03_rb,0.59749e+03_rb,0.87768e+03_rb,0.11575e+04_rb /)
      kbo(:, 5,24,11) = (/ &
        & 0.10450e+04_rb,0.79193e+03_rb,0.59639e+03_rb,0.87600e+03_rb,0.11558e+04_rb /)
      kbo(:, 1,25,11) = (/ &
        & 0.10463e+04_rb,0.79209e+03_rb,0.60015e+03_rb,0.88131e+03_rb,0.11626e+04_rb /)
      kbo(:, 2,25,11) = (/ &
        & 0.10455e+04_rb,0.79185e+03_rb,0.59924e+03_rb,0.87983e+03_rb,0.11609e+04_rb /)
      kbo(:, 3,25,11) = (/ &
        & 0.10450e+04_rb,0.79214e+03_rb,0.59824e+03_rb,0.87889e+03_rb,0.11590e+04_rb /)
      kbo(:, 4,25,11) = (/ &
        & 0.10445e+04_rb,0.79118e+03_rb,0.59725e+03_rb,0.87732e+03_rb,0.11571e+04_rb /)
      kbo(:, 5,25,11) = (/ &
        & 0.10428e+04_rb,0.79168e+03_rb,0.59624e+03_rb,0.87561e+03_rb,0.11555e+04_rb /)
      kbo(:, 1,26,11) = (/ &
        & 0.10437e+04_rb,0.79093e+03_rb,0.59993e+03_rb,0.88106e+03_rb,0.11624e+04_rb /)
      kbo(:, 2,26,11) = (/ &
        & 0.10430e+04_rb,0.79091e+03_rb,0.59898e+03_rb,0.87993e+03_rb,0.11605e+04_rb /)
      kbo(:, 3,26,11) = (/ &
        & 0.10430e+04_rb,0.79069e+03_rb,0.59808e+03_rb,0.87852e+03_rb,0.11586e+04_rb /)
      kbo(:, 4,26,11) = (/ &
        & 0.10425e+04_rb,0.79087e+03_rb,0.59699e+03_rb,0.87692e+03_rb,0.11569e+04_rb /)
      kbo(:, 5,26,11) = (/ &
        & 0.10414e+04_rb,0.78954e+03_rb,0.59598e+03_rb,0.87517e+03_rb,0.11547e+04_rb /)
      kbo(:, 1,27,11) = (/ &
        & 0.10423e+04_rb,0.79028e+03_rb,0.59970e+03_rb,0.88080e+03_rb,0.11616e+04_rb /)
      kbo(:, 2,27,11) = (/ &
        & 0.10415e+04_rb,0.78939e+03_rb,0.59872e+03_rb,0.87961e+03_rb,0.11600e+04_rb /)
      kbo(:, 3,27,11) = (/ &
        & 0.10419e+04_rb,0.78972e+03_rb,0.59773e+03_rb,0.87814e+03_rb,0.11581e+04_rb /)
      kbo(:, 4,27,11) = (/ &
        & 0.10405e+04_rb,0.78901e+03_rb,0.59673e+03_rb,0.87651e+03_rb,0.11564e+04_rb /)
      kbo(:, 5,27,11) = (/ &
        & 0.10410e+04_rb,0.78880e+03_rb,0.59572e+03_rb,0.87471e+03_rb,0.11546e+04_rb /)
      kbo(:, 1,28,11) = (/ &
        & 0.10411e+04_rb,0.78831e+03_rb,0.59945e+03_rb,0.88052e+03_rb,0.11614e+04_rb /)
      kbo(:, 2,28,11) = (/ &
        & 0.10402e+04_rb,0.78866e+03_rb,0.59846e+03_rb,0.87954e+03_rb,0.11596e+04_rb /)
      kbo(:, 3,28,11) = (/ &
        & 0.10401e+04_rb,0.78798e+03_rb,0.59747e+03_rb,0.87776e+03_rb,0.11577e+04_rb /)
      kbo(:, 4,28,11) = (/ &
        & 0.10407e+04_rb,0.78779e+03_rb,0.59647e+03_rb,0.87596e+03_rb,0.11560e+04_rb /)
      kbo(:, 5,28,11) = (/ &
        & 0.10407e+04_rb,0.78751e+03_rb,0.59548e+03_rb,0.87425e+03_rb,0.11542e+04_rb /)
      kbo(:, 1,29,11) = (/ &
        & 0.10408e+04_rb,0.78876e+03_rb,0.59920e+03_rb,0.87983e+03_rb,0.11608e+04_rb /)
      kbo(:, 2,29,11) = (/ &
        & 0.10408e+04_rb,0.78834e+03_rb,0.59820e+03_rb,0.87891e+03_rb,0.11591e+04_rb /)
      kbo(:, 3,29,11) = (/ &
        & 0.10387e+04_rb,0.78864e+03_rb,0.59721e+03_rb,0.87735e+03_rb,0.11572e+04_rb /)
      kbo(:, 4,29,11) = (/ &
        & 0.10384e+04_rb,0.78778e+03_rb,0.59621e+03_rb,0.87578e+03_rb,0.11555e+04_rb /)
      kbo(:, 5,29,11) = (/ &
        & 0.10412e+04_rb,0.78824e+03_rb,0.59524e+03_rb,0.87368e+03_rb,0.11538e+04_rb /)
      kbo(:, 1,30,11) = (/ &
        & 0.10383e+04_rb,0.78758e+03_rb,0.59894e+03_rb,0.87992e+03_rb,0.11605e+04_rb /)
      kbo(:, 2,30,11) = (/ &
        & 0.10390e+04_rb,0.78755e+03_rb,0.59785e+03_rb,0.87840e+03_rb,0.11584e+04_rb /)
      kbo(:, 3,30,11) = (/ &
        & 0.10391e+04_rb,0.78813e+03_rb,0.59695e+03_rb,0.87693e+03_rb,0.11568e+04_rb /)
      kbo(:, 4,30,11) = (/ &
        & 0.10384e+04_rb,0.78683e+03_rb,0.59594e+03_rb,0.87519e+03_rb,0.11551e+04_rb /)
      kbo(:, 5,30,11) = (/ &
        & 0.10388e+04_rb,0.78660e+03_rb,0.59501e+03_rb,0.87341e+03_rb,0.11534e+04_rb /)
      kbo(:, 1,31,11) = (/ &
        & 0.10376e+04_rb,0.78640e+03_rb,0.59867e+03_rb,0.87959e+03_rb,0.11601e+04_rb /)
      kbo(:, 2,31,11) = (/ &
        & 0.10381e+04_rb,0.78687e+03_rb,0.59768e+03_rb,0.87813e+03_rb,0.11581e+04_rb /)
      kbo(:, 3,31,11) = (/ &
        & 0.10369e+04_rb,0.78736e+03_rb,0.59668e+03_rb,0.87650e+03_rb,0.11564e+04_rb /)
      kbo(:, 4,31,11) = (/ &
        & 0.10380e+04_rb,0.78738e+03_rb,0.59577e+03_rb,0.87483e+03_rb,0.11546e+04_rb /)
      kbo(:, 5,31,11) = (/ &
        & 0.10391e+04_rb,0.78671e+03_rb,0.59479e+03_rb,0.87299e+03_rb,0.11532e+04_rb /)
      kbo(:, 1,32,11) = (/ &
        & 0.10371e+04_rb,0.78531e+03_rb,0.59840e+03_rb,0.87936e+03_rb,0.11596e+04_rb /)
      kbo(:, 2,32,11) = (/ &
        & 0.10383e+04_rb,0.78734e+03_rb,0.59742e+03_rb,0.87771e+03_rb,0.11576e+04_rb /)
      kbo(:, 3,32,11) = (/ &
        & 0.10363e+04_rb,0.78691e+03_rb,0.59641e+03_rb,0.87605e+03_rb,0.11559e+04_rb /)
      kbo(:, 4,32,11) = (/ &
        & 0.10373e+04_rb,0.78648e+03_rb,0.59543e+03_rb,0.87420e+03_rb,0.11542e+04_rb /)
      kbo(:, 5,32,11) = (/ &
        & 0.10376e+04_rb,0.78617e+03_rb,0.59455e+03_rb,0.87256e+03_rb,0.11529e+04_rb /)
      kbo(:, 1,33,11) = (/ &
        & 0.10366e+04_rb,0.78725e+03_rb,0.59814e+03_rb,0.87885e+03_rb,0.11591e+04_rb /)
      kbo(:, 2,33,11) = (/ &
        & 0.10366e+04_rb,0.78694e+03_rb,0.59715e+03_rb,0.87729e+03_rb,0.11572e+04_rb /)
      kbo(:, 3,33,11) = (/ &
        & 0.10365e+04_rb,0.78534e+03_rb,0.59614e+03_rb,0.87545e+03_rb,0.11555e+04_rb /)
      kbo(:, 4,33,11) = (/ &
        & 0.10377e+04_rb,0.78705e+03_rb,0.59518e+03_rb,0.87362e+03_rb,0.11536e+04_rb /)
      kbo(:, 5,33,11) = (/ &
        & 0.10381e+04_rb,0.78546e+03_rb,0.59428e+03_rb,0.87201e+03_rb,0.11526e+04_rb /)
      kbo(:, 1,34,11) = (/ &
        & 0.10370e+04_rb,0.78547e+03_rb,0.59791e+03_rb,0.87837e+03_rb,0.11586e+04_rb /)
      kbo(:, 2,34,11) = (/ &
        & 0.10369e+04_rb,0.78635e+03_rb,0.59683e+03_rb,0.87691e+03_rb,0.11568e+04_rb /)
      kbo(:, 3,34,11) = (/ &
        & 0.10373e+04_rb,0.78607e+03_rb,0.59591e+03_rb,0.87516e+03_rb,0.11551e+04_rb /)
      kbo(:, 4,34,11) = (/ &
        & 0.10371e+04_rb,0.78696e+03_rb,0.59498e+03_rb,0.87338e+03_rb,0.11534e+04_rb /)
      kbo(:, 5,34,11) = (/ &
        & 0.10366e+04_rb,0.78538e+03_rb,0.59374e+03_rb,0.87179e+03_rb,0.11524e+04_rb /)
      kbo(:, 1,35,11) = (/ &
        & 0.10371e+04_rb,0.78681e+03_rb,0.59778e+03_rb,0.87830e+03_rb,0.11583e+04_rb /)
      kbo(:, 2,35,11) = (/ &
        & 0.10367e+04_rb,0.78665e+03_rb,0.59678e+03_rb,0.87669e+03_rb,0.11566e+04_rb /)
      kbo(:, 3,35,11) = (/ &
        & 0.10363e+04_rb,0.78613e+03_rb,0.59569e+03_rb,0.87491e+03_rb,0.11549e+04_rb /)
      kbo(:, 4,35,11) = (/ &
        & 0.10364e+04_rb,0.78668e+03_rb,0.59487e+03_rb,0.87304e+03_rb,0.11533e+04_rb /)
      kbo(:, 5,35,11) = (/ &
        & 0.10370e+04_rb,0.78641e+03_rb,0.59363e+03_rb,0.87159e+03_rb,0.11523e+04_rb /)
      kbo(:, 1,36,11) = (/ &
        & 0.10366e+04_rb,0.78595e+03_rb,0.59775e+03_rb,0.87827e+03_rb,0.11583e+04_rb /)
      kbo(:, 2,36,11) = (/ &
        & 0.10367e+04_rb,0.78568e+03_rb,0.59676e+03_rb,0.87652e+03_rb,0.11565e+04_rb /)
      kbo(:, 3,36,11) = (/ &
        & 0.10363e+04_rb,0.78600e+03_rb,0.59575e+03_rb,0.87487e+03_rb,0.11548e+04_rb /)
      kbo(:, 4,36,11) = (/ &
        & 0.10370e+04_rb,0.78596e+03_rb,0.59503e+03_rb,0.87314e+03_rb,0.11533e+04_rb /)
      kbo(:, 5,36,11) = (/ &
        & 0.10356e+04_rb,0.78546e+03_rb,0.59359e+03_rb,0.87116e+03_rb,0.11523e+04_rb /)
      kbo(:, 1,37,11) = (/ &
        & 0.10367e+04_rb,0.78524e+03_rb,0.59788e+03_rb,0.87847e+03_rb,0.11586e+04_rb /)
      kbo(:, 2,37,11) = (/ &
        & 0.10373e+04_rb,0.78580e+03_rb,0.59689e+03_rb,0.87687e+03_rb,0.11567e+04_rb /)
      kbo(:, 3,37,11) = (/ &
        & 0.10369e+04_rb,0.78554e+03_rb,0.59588e+03_rb,0.87512e+03_rb,0.11550e+04_rb /)
      kbo(:, 4,37,11) = (/ &
        & 0.10372e+04_rb,0.78551e+03_rb,0.59495e+03_rb,0.87335e+03_rb,0.11534e+04_rb /)
      kbo(:, 5,37,11) = (/ &
        & 0.10364e+04_rb,0.78631e+03_rb,0.59378e+03_rb,0.87175e+03_rb,0.11524e+04_rb /)
      kbo(:, 1,38,11) = (/ &
        & 0.10372e+04_rb,0.78574e+03_rb,0.59801e+03_rb,0.87867e+03_rb,0.11588e+04_rb /)
      kbo(:, 2,38,11) = (/ &
        & 0.10364e+04_rb,0.78476e+03_rb,0.59702e+03_rb,0.87710e+03_rb,0.11570e+04_rb /)
      kbo(:, 3,38,11) = (/ &
        & 0.10351e+04_rb,0.78521e+03_rb,0.59593e+03_rb,0.87537e+03_rb,0.11553e+04_rb /)
      kbo(:, 4,38,11) = (/ &
        & 0.10362e+04_rb,0.78507e+03_rb,0.59507e+03_rb,0.87344e+03_rb,0.11536e+04_rb /)
      kbo(:, 5,38,11) = (/ &
        & 0.10371e+04_rb,0.78635e+03_rb,0.59400e+03_rb,0.87197e+03_rb,0.11525e+04_rb /)
      kbo(:, 1,39,11) = (/ &
        & 0.10359e+04_rb,0.78601e+03_rb,0.59815e+03_rb,0.87888e+03_rb,0.11591e+04_rb /)
      kbo(:, 2,39,11) = (/ &
        & 0.10375e+04_rb,0.78669e+03_rb,0.59716e+03_rb,0.87732e+03_rb,0.11572e+04_rb /)
      kbo(:, 3,39,11) = (/ &
        & 0.10350e+04_rb,0.78597e+03_rb,0.59615e+03_rb,0.87562e+03_rb,0.11555e+04_rb /)
      kbo(:, 4,39,11) = (/ &
        & 0.10366e+04_rb,0.78524e+03_rb,0.59519e+03_rb,0.87365e+03_rb,0.11538e+04_rb /)
      kbo(:, 5,39,11) = (/ &
        & 0.10373e+04_rb,0.78605e+03_rb,0.59422e+03_rb,0.87205e+03_rb,0.11526e+04_rb /)
      kbo(:, 1,40,11) = (/ &
        & 0.10361e+04_rb,0.78559e+03_rb,0.59838e+03_rb,0.87949e+03_rb,0.11596e+04_rb /)
      kbo(:, 2,40,11) = (/ &
        & 0.10364e+04_rb,0.78533e+03_rb,0.59740e+03_rb,0.87771e+03_rb,0.11576e+04_rb /)
      kbo(:, 3,40,11) = (/ &
        & 0.10357e+04_rb,0.78567e+03_rb,0.59640e+03_rb,0.87605e+03_rb,0.11559e+04_rb /)
      kbo(:, 4,40,11) = (/ &
        & 0.10354e+04_rb,0.78518e+03_rb,0.59541e+03_rb,0.87434e+03_rb,0.11542e+04_rb /)
      kbo(:, 5,40,11) = (/ &
        & 0.10361e+04_rb,0.78553e+03_rb,0.59453e+03_rb,0.87244e+03_rb,0.11529e+04_rb /)
      kbo(:, 1,41,11) = (/ &
        & 0.10352e+04_rb,0.78554e+03_rb,0.59863e+03_rb,0.87958e+03_rb,0.11601e+04_rb /)
      kbo(:, 2,41,11) = (/ &
        & 0.10361e+04_rb,0.78611e+03_rb,0.59765e+03_rb,0.87811e+03_rb,0.11581e+04_rb /)
      kbo(:, 3,41,11) = (/ &
        & 0.10357e+04_rb,0.78586e+03_rb,0.59665e+03_rb,0.87648e+03_rb,0.11564e+04_rb /)
      kbo(:, 4,41,11) = (/ &
        & 0.10369e+04_rb,0.78632e+03_rb,0.59556e+03_rb,0.87469e+03_rb,0.11547e+04_rb /)
      kbo(:, 5,41,11) = (/ &
        & 0.10372e+04_rb,0.78631e+03_rb,0.59476e+03_rb,0.87298e+03_rb,0.11532e+04_rb /)
      kbo(:, 1,42,11) = (/ &
        & 0.10352e+04_rb,0.78620e+03_rb,0.59889e+03_rb,0.87990e+03_rb,0.11605e+04_rb /)
      kbo(:, 2,42,11) = (/ &
        & 0.10362e+04_rb,0.78666e+03_rb,0.59790e+03_rb,0.87851e+03_rb,0.11586e+04_rb /)
      kbo(:, 3,42,11) = (/ &
        & 0.10358e+04_rb,0.78476e+03_rb,0.59691e+03_rb,0.87691e+03_rb,0.11568e+04_rb /)
      kbo(:, 4,42,11) = (/ &
        & 0.10367e+04_rb,0.78534e+03_rb,0.59590e+03_rb,0.87517e+03_rb,0.11551e+04_rb /)
      kbo(:, 5,42,11) = (/ &
        & 0.10374e+04_rb,0.78497e+03_rb,0.59497e+03_rb,0.87339e+03_rb,0.11533e+04_rb /)
      kbo(:, 1,43,11) = (/ &
        & 0.10357e+04_rb,0.78569e+03_rb,0.59919e+03_rb,0.88014e+03_rb,0.11611e+04_rb /)
      kbo(:, 2,43,11) = (/ &
        & 0.10384e+04_rb,0.78603e+03_rb,0.59811e+03_rb,0.87910e+03_rb,0.11592e+04_rb /)
      kbo(:, 3,43,11) = (/ &
        & 0.10376e+04_rb,0.78685e+03_rb,0.59731e+03_rb,0.87742e+03_rb,0.11573e+04_rb /)
      kbo(:, 4,43,11) = (/ &
        & 0.10368e+04_rb,0.78484e+03_rb,0.59612e+03_rb,0.87573e+03_rb,0.11555e+04_rb /)
      kbo(:, 5,43,11) = (/ &
        & 0.10371e+04_rb,0.78612e+03_rb,0.59525e+03_rb,0.87389e+03_rb,0.11539e+04_rb /)
      kbo(:, 1,44,11) = (/ &
        & 0.10362e+04_rb,0.78531e+03_rb,0.59952e+03_rb,0.88052e+03_rb,0.11615e+04_rb /)
      kbo(:, 2,44,11) = (/ &
        & 0.10357e+04_rb,0.78506e+03_rb,0.59854e+03_rb,0.87945e+03_rb,0.11599e+04_rb /)
      kbo(:, 3,44,11) = (/ &
        & 0.10347e+04_rb,0.78494e+03_rb,0.59755e+03_rb,0.87796e+03_rb,0.11576e+04_rb /)
      kbo(:, 4,44,11) = (/ &
        & 0.10360e+04_rb,0.78599e+03_rb,0.59655e+03_rb,0.87632e+03_rb,0.11562e+04_rb /)
      kbo(:, 5,44,11) = (/ &
        & 0.10364e+04_rb,0.78610e+03_rb,0.59556e+03_rb,0.87451e+03_rb,0.11545e+04_rb /)
      kbo(:, 1,45,11) = (/ &
        & 0.10355e+04_rb,0.78552e+03_rb,0.59982e+03_rb,0.88100e+03_rb,0.11621e+04_rb /)
      kbo(:, 2,45,11) = (/ &
        & 0.10357e+04_rb,0.78527e+03_rb,0.59887e+03_rb,0.87975e+03_rb,0.11605e+04_rb /)
      kbo(:, 3,45,11) = (/ &
        & 0.10358e+04_rb,0.78432e+03_rb,0.59788e+03_rb,0.87848e+03_rb,0.11586e+04_rb /)
      kbo(:, 4,45,11) = (/ &
        & 0.10358e+04_rb,0.78597e+03_rb,0.59689e+03_rb,0.87689e+03_rb,0.11568e+04_rb /)
      kbo(:, 5,45,11) = (/ &
        & 0.10353e+04_rb,0.78525e+03_rb,0.59589e+03_rb,0.87514e+03_rb,0.11551e+04_rb /)
      kbo(:, 1,46,11) = (/ &
        & 0.10358e+04_rb,0.78503e+03_rb,0.60010e+03_rb,0.88136e+03_rb,0.11629e+04_rb /)
      kbo(:, 2,46,11) = (/ &
        & 0.10366e+04_rb,0.78667e+03_rb,0.59923e+03_rb,0.88004e+03_rb,0.11611e+04_rb /)
      kbo(:, 3,46,11) = (/ &
        & 0.10372e+04_rb,0.78560e+03_rb,0.59824e+03_rb,0.87915e+03_rb,0.11593e+04_rb /)
      kbo(:, 4,46,11) = (/ &
        & 0.10348e+04_rb,0.78513e+03_rb,0.59725e+03_rb,0.87748e+03_rb,0.11574e+04_rb /)
      kbo(:, 5,46,11) = (/ &
        & 0.10372e+04_rb,0.78595e+03_rb,0.59625e+03_rb,0.87592e+03_rb,0.11557e+04_rb /)
      kbo(:, 1,47,11) = (/ &
        & 0.10352e+04_rb,0.78466e+03_rb,0.60059e+03_rb,0.88175e+03_rb,0.11637e+04_rb /)
      kbo(:, 2,47,11) = (/ &
        & 0.10362e+04_rb,0.78477e+03_rb,0.59970e+03_rb,0.88075e+03_rb,0.11618e+04_rb /)
      kbo(:, 3,47,11) = (/ &
        & 0.10362e+04_rb,0.78559e+03_rb,0.59863e+03_rb,0.87958e+03_rb,0.11601e+04_rb /)
      kbo(:, 4,47,11) = (/ &
        & 0.10356e+04_rb,0.78476e+03_rb,0.59765e+03_rb,0.87812e+03_rb,0.11581e+04_rb /)
      kbo(:, 5,47,11) = (/ &
        & 0.10361e+04_rb,0.78428e+03_rb,0.59665e+03_rb,0.87649e+03_rb,0.11564e+04_rb /)
      kbo(:, 1,48,11) = (/ &
        & 0.10354e+04_rb,0.78500e+03_rb,0.60042e+03_rb,0.88214e+03_rb,0.11645e+04_rb /)
      kbo(:, 2,48,11) = (/ &
        & 0.10348e+04_rb,0.78582e+03_rb,0.59995e+03_rb,0.88117e+03_rb,0.11625e+04_rb /)
      kbo(:, 3,48,11) = (/ &
        & 0.10353e+04_rb,0.78569e+03_rb,0.59903e+03_rb,0.87942e+03_rb,0.11608e+04_rb /)
      kbo(:, 4,48,11) = (/ &
        & 0.10359e+04_rb,0.78569e+03_rb,0.59804e+03_rb,0.87873e+03_rb,0.11589e+04_rb /)
      kbo(:, 5,48,11) = (/ &
        & 0.10354e+04_rb,0.78509e+03_rb,0.59705e+03_rb,0.87716e+03_rb,0.11570e+04_rb /)
      kbo(:, 1,49,11) = (/ &
        & 0.10362e+04_rb,0.78463e+03_rb,0.60042e+03_rb,0.88242e+03_rb,0.11657e+04_rb /)
      kbo(:, 2,49,11) = (/ &
        & 0.10367e+04_rb,0.78580e+03_rb,0.60023e+03_rb,0.88156e+03_rb,0.11633e+04_rb /)
      kbo(:, 3,49,11) = (/ &
        & 0.10373e+04_rb,0.78533e+03_rb,0.59942e+03_rb,0.88054e+03_rb,0.11615e+04_rb /)
      kbo(:, 4,49,11) = (/ &
        & 0.10373e+04_rb,0.78462e+03_rb,0.59844e+03_rb,0.87918e+03_rb,0.11595e+04_rb /)
      kbo(:, 5,49,11) = (/ &
        & 0.10370e+04_rb,0.78532e+03_rb,0.59736e+03_rb,0.87781e+03_rb,0.11578e+04_rb /)
      kbo(:, 1,50,11) = (/ &
        & 0.10356e+04_rb,0.78474e+03_rb,0.60056e+03_rb,0.88267e+03_rb,0.11666e+04_rb /)
      kbo(:, 2,50,11) = (/ &
        & 0.10370e+04_rb,0.78497e+03_rb,0.60046e+03_rb,0.88192e+03_rb,0.11642e+04_rb /)
      kbo(:, 3,50,11) = (/ &
        & 0.10348e+04_rb,0.78508e+03_rb,0.59977e+03_rb,0.88094e+03_rb,0.11634e+04_rb /)
      kbo(:, 4,50,11) = (/ &
        & 0.10352e+04_rb,0.78473e+03_rb,0.59881e+03_rb,0.87954e+03_rb,0.11604e+04_rb /)
      kbo(:, 5,50,11) = (/ &
        & 0.10360e+04_rb,0.78437e+03_rb,0.59782e+03_rb,0.87840e+03_rb,0.11585e+04_rb /)
      kbo(:, 1,51,11) = (/ &
        & 0.10365e+04_rb,0.78532e+03_rb,0.60054e+03_rb,0.88306e+03_rb,0.11674e+04_rb /)
      kbo(:, 2,51,11) = (/ &
        & 0.10360e+04_rb,0.78555e+03_rb,0.60046e+03_rb,0.88226e+03_rb,0.11651e+04_rb /)
      kbo(:, 3,51,11) = (/ &
        & 0.10359e+04_rb,0.78508e+03_rb,0.60006e+03_rb,0.88131e+03_rb,0.11628e+04_rb /)
      kbo(:, 4,51,11) = (/ &
        & 0.10359e+04_rb,0.78554e+03_rb,0.59917e+03_rb,0.88011e+03_rb,0.11610e+04_rb /)
      kbo(:, 5,51,11) = (/ &
        & 0.10363e+04_rb,0.78507e+03_rb,0.59818e+03_rb,0.87894e+03_rb,0.11592e+04_rb /)
      kbo(:, 1,52,11) = (/ &
        & 0.10357e+04_rb,0.78531e+03_rb,0.60048e+03_rb,0.88333e+03_rb,0.11687e+04_rb /)
      kbo(:, 2,52,11) = (/ &
        & 0.10368e+04_rb,0.78554e+03_rb,0.60053e+03_rb,0.88249e+03_rb,0.11660e+04_rb /)
      kbo(:, 3,52,11) = (/ &
        & 0.10362e+04_rb,0.78601e+03_rb,0.60037e+03_rb,0.88166e+03_rb,0.11635e+04_rb /)
      kbo(:, 4,52,11) = (/ &
        & 0.10354e+04_rb,0.78507e+03_rb,0.59952e+03_rb,0.88066e+03_rb,0.11617e+04_rb /)
      kbo(:, 5,52,11) = (/ &
        & 0.10363e+04_rb,0.78565e+03_rb,0.59854e+03_rb,0.87959e+03_rb,0.11599e+04_rb /)
      kbo(:, 1,53,11) = (/ &
        & 0.10366e+04_rb,0.78460e+03_rb,0.60030e+03_rb,0.88415e+03_rb,0.11699e+04_rb /)
      kbo(:, 2,53,11) = (/ &
        & 0.10366e+04_rb,0.78400e+03_rb,0.60056e+03_rb,0.88262e+03_rb,0.11668e+04_rb /)
      kbo(:, 3,53,11) = (/ &
        & 0.10358e+04_rb,0.78459e+03_rb,0.60039e+03_rb,0.88202e+03_rb,0.11644e+04_rb /)
      kbo(:, 4,53,11) = (/ &
        & 0.10348e+04_rb,0.78471e+03_rb,0.59985e+03_rb,0.88104e+03_rb,0.11623e+04_rb /)
      kbo(:, 5,53,11) = (/ &
        & 0.10366e+04_rb,0.78577e+03_rb,0.59891e+03_rb,0.87992e+03_rb,0.11606e+04_rb /)
      kbo(:, 1,54,11) = (/ &
        & 0.10355e+04_rb,0.78471e+03_rb,0.60033e+03_rb,0.88501e+03_rb,0.11708e+04_rb /)
      kbo(:, 2,54,11) = (/ &
        & 0.10360e+04_rb,0.78494e+03_rb,0.60053e+03_rb,0.88313e+03_rb,0.11678e+04_rb /)
      kbo(:, 3,54,11) = (/ &
        & 0.10360e+04_rb,0.78671e+03_rb,0.60047e+03_rb,0.88230e+03_rb,0.11652e+04_rb /)
      kbo(:, 4,54,11) = (/ &
        & 0.10355e+04_rb,0.78588e+03_rb,0.60010e+03_rb,0.88137e+03_rb,0.11629e+04_rb /)
      kbo(:, 5,54,11) = (/ &
        & 0.10357e+04_rb,0.78517e+03_rb,0.59924e+03_rb,0.88032e+03_rb,0.11612e+04_rb /)
      kbo(:, 1,55,11) = (/ &
        & 0.10357e+04_rb,0.78506e+03_rb,0.60035e+03_rb,0.88567e+03_rb,0.11714e+04_rb /)
      kbo(:, 2,55,11) = (/ &
        & 0.10357e+04_rb,0.78518e+03_rb,0.60047e+03_rb,0.88349e+03_rb,0.11688e+04_rb /)
      kbo(:, 3,55,11) = (/ &
        & 0.10361e+04_rb,0.78517e+03_rb,0.60053e+03_rb,0.88250e+03_rb,0.11660e+04_rb /)
      kbo(:, 4,55,11) = (/ &
        & 0.10357e+04_rb,0.78529e+03_rb,0.60029e+03_rb,0.88168e+03_rb,0.11636e+04_rb /)
      kbo(:, 5,55,11) = (/ &
        & 0.10360e+04_rb,0.78529e+03_rb,0.59954e+03_rb,0.88068e+03_rb,0.11617e+04_rb /)
      kbo(:, 1,56,11) = (/ &
        & 0.10361e+04_rb,0.78494e+03_rb,0.60049e+03_rb,0.88607e+03_rb,0.11718e+04_rb /)
      kbo(:, 2,56,11) = (/ &
        & 0.10360e+04_rb,0.78482e+03_rb,0.60030e+03_rb,0.88410e+03_rb,0.11698e+04_rb /)
      kbo(:, 3,56,11) = (/ &
        & 0.10354e+04_rb,0.78493e+03_rb,0.60056e+03_rb,0.88273e+03_rb,0.11667e+04_rb /)
      kbo(:, 4,56,11) = (/ &
        & 0.10357e+04_rb,0.78552e+03_rb,0.60038e+03_rb,0.88199e+03_rb,0.11645e+04_rb /)
      kbo(:, 5,56,11) = (/ &
        & 0.10357e+04_rb,0.78517e+03_rb,0.59983e+03_rb,0.88101e+03_rb,0.11624e+04_rb /)
      kbo(:, 1,57,11) = (/ &
        & 0.10357e+04_rb,0.78588e+03_rb,0.60099e+03_rb,0.88686e+03_rb,0.11725e+04_rb /)
      kbo(:, 2,57,11) = (/ &
        & 0.10366e+04_rb,0.78458e+03_rb,0.60033e+03_rb,0.88491e+03_rb,0.11707e+04_rb /)
      kbo(:, 3,57,11) = (/ &
        & 0.10366e+04_rb,0.78481e+03_rb,0.60053e+03_rb,0.88335e+03_rb,0.11676e+04_rb /)
      kbo(:, 4,57,11) = (/ &
        & 0.10358e+04_rb,0.78611e+03_rb,0.60046e+03_rb,0.88227e+03_rb,0.11650e+04_rb /)
      kbo(:, 5,57,11) = (/ &
        & 0.10355e+04_rb,0.78599e+03_rb,0.60008e+03_rb,0.88133e+03_rb,0.11630e+04_rb /)
      kbo(:, 1,58,11) = (/ &
        & 0.10363e+04_rb,0.78576e+03_rb,0.60143e+03_rb,0.88764e+03_rb,0.11733e+04_rb /)
      kbo(:, 2,58,11) = (/ &
        & 0.10354e+04_rb,0.78611e+03_rb,0.60042e+03_rb,0.88557e+03_rb,0.11713e+04_rb /)
      kbo(:, 3,58,11) = (/ &
        & 0.10363e+04_rb,0.78576e+03_rb,0.60048e+03_rb,0.88329e+03_rb,0.11686e+04_rb /)
      kbo(:, 4,58,11) = (/ &
        & 0.10364e+04_rb,0.78505e+03_rb,0.60052e+03_rb,0.88247e+03_rb,0.11659e+04_rb /)
      kbo(:, 5,58,11) = (/ &
        & 0.10357e+04_rb,0.78552e+03_rb,0.60026e+03_rb,0.88163e+03_rb,0.11633e+04_rb /)
      kbo(:, 1,59,11) = (/ &
        & 0.10354e+04_rb,0.78634e+03_rb,0.60176e+03_rb,0.88797e+03_rb,0.11736e+04_rb /)
      kbo(:, 2,59,11) = (/ &
        & 0.10350e+04_rb,0.78505e+03_rb,0.60038e+03_rb,0.88577e+03_rb,0.11715e+04_rb /)
      kbo(:, 3,59,11) = (/ &
        & 0.10360e+04_rb,0.78469e+03_rb,0.60046e+03_rb,0.88358e+03_rb,0.11703e+04_rb /)
      kbo(:, 4,59,11) = (/ &
        & 0.10357e+04_rb,0.78493e+03_rb,0.60045e+03_rb,0.88254e+03_rb,0.11662e+04_rb /)
      kbo(:, 5,59,11) = (/ &
        & 0.10366e+04_rb,0.78516e+03_rb,0.60041e+03_rb,0.88174e+03_rb,0.11637e+04_rb /)
      kbo(:, 1,13,12) = (/ &
        & 0.11535e+04_rb,0.87955e+03_rb,0.60434e+03_rb,0.88744e+03_rb,0.11710e+04_rb /)
      kbo(:, 2,13,12) = (/ &
        & 0.11485e+04_rb,0.86886e+03_rb,0.60369e+03_rb,0.88598e+03_rb,0.11697e+04_rb /)
      kbo(:, 3,13,12) = (/ &
        & 0.11395e+04_rb,0.86441e+03_rb,0.60332e+03_rb,0.88477e+03_rb,0.11680e+04_rb /)
      kbo(:, 4,13,12) = (/ &
        & 0.11350e+04_rb,0.86216e+03_rb,0.60276e+03_rb,0.88294e+03_rb,0.11659e+04_rb /)
      kbo(:, 5,13,12) = (/ &
        & 0.11237e+04_rb,0.85514e+03_rb,0.60186e+03_rb,0.88131e+03_rb,0.11640e+04_rb /)
      kbo(:, 1,14,12) = (/ &
        & 0.11312e+04_rb,0.85960e+03_rb,0.60391e+03_rb,0.88683e+03_rb,0.11708e+04_rb /)
      kbo(:, 2,14,12) = (/ &
        & 0.11248e+04_rb,0.85192e+03_rb,0.60329e+03_rb,0.88591e+03_rb,0.11696e+04_rb /)
      kbo(:, 3,14,12) = (/ &
        & 0.11210e+04_rb,0.84824e+03_rb,0.60297e+03_rb,0.88444e+03_rb,0.11678e+04_rb /)
      kbo(:, 4,14,12) = (/ &
        & 0.11139e+04_rb,0.84322e+03_rb,0.60241e+03_rb,0.88264e+03_rb,0.11657e+04_rb /)
      kbo(:, 5,14,12) = (/ &
        & 0.11123e+04_rb,0.84158e+03_rb,0.60150e+03_rb,0.88104e+03_rb,0.11639e+04_rb /)
      kbo(:, 1,15,12) = (/ &
        & 0.11159e+04_rb,0.84414e+03_rb,0.60344e+03_rb,0.88657e+03_rb,0.11715e+04_rb /)
      kbo(:, 2,15,12) = (/ &
        & 0.11106e+04_rb,0.84043e+03_rb,0.60296e+03_rb,0.88568e+03_rb,0.11695e+04_rb /)
      kbo(:, 3,15,12) = (/ &
        & 0.11018e+04_rb,0.83588e+03_rb,0.60268e+03_rb,0.88413e+03_rb,0.11676e+04_rb /)
      kbo(:, 4,15,12) = (/ &
        & 0.10976e+04_rb,0.83213e+03_rb,0.60210e+03_rb,0.88235e+03_rb,0.11655e+04_rb /)
      kbo(:, 5,15,12) = (/ &
        & 0.10965e+04_rb,0.83273e+03_rb,0.60128e+03_rb,0.88077e+03_rb,0.11639e+04_rb /)
      kbo(:, 1,16,12) = (/ &
        & 0.11003e+04_rb,0.83414e+03_rb,0.60325e+03_rb,0.88634e+03_rb,0.11705e+04_rb /)
      kbo(:, 2,16,12) = (/ &
        & 0.10991e+04_rb,0.82921e+03_rb,0.60259e+03_rb,0.88546e+03_rb,0.11694e+04_rb /)
      kbo(:, 3,16,12) = (/ &
        & 0.10955e+04_rb,0.82665e+03_rb,0.60243e+03_rb,0.88384e+03_rb,0.11674e+04_rb /)
      kbo(:, 4,16,12) = (/ &
        & 0.10851e+04_rb,0.82401e+03_rb,0.60183e+03_rb,0.88207e+03_rb,0.11653e+04_rb /)
      kbo(:, 5,16,12) = (/ &
        & 0.10849e+04_rb,0.82356e+03_rb,0.60085e+03_rb,0.88053e+03_rb,0.11635e+04_rb /)
      kbo(:, 1,17,12) = (/ &
        & 0.10876e+04_rb,0.82527e+03_rb,0.60299e+03_rb,0.88614e+03_rb,0.11702e+04_rb /)
      kbo(:, 2,17,12) = (/ &
        & 0.10859e+04_rb,0.82135e+03_rb,0.60250e+03_rb,0.88526e+03_rb,0.11692e+04_rb /)
      kbo(:, 3,17,12) = (/ &
        & 0.10841e+04_rb,0.81891e+03_rb,0.60222e+03_rb,0.88356e+03_rb,0.11671e+04_rb /)
      kbo(:, 4,17,12) = (/ &
        & 0.10784e+04_rb,0.81708e+03_rb,0.60159e+03_rb,0.88183e+03_rb,0.11651e+04_rb /)
      kbo(:, 5,17,12) = (/ &
        & 0.10763e+04_rb,0.81350e+03_rb,0.60170e+03_rb,0.88030e+03_rb,0.11634e+04_rb /)
      kbo(:, 1,18,12) = (/ &
        & 0.10781e+04_rb,0.81748e+03_rb,0.60276e+03_rb,0.88597e+03_rb,0.11700e+04_rb /)
      kbo(:, 2,18,12) = (/ &
        & 0.10771e+04_rb,0.81396e+03_rb,0.60232e+03_rb,0.88573e+03_rb,0.11691e+04_rb /)
      kbo(:, 3,18,12) = (/ &
        & 0.10718e+04_rb,0.81415e+03_rb,0.60204e+03_rb,0.88331e+03_rb,0.11669e+04_rb /)
      kbo(:, 4,18,12) = (/ &
        & 0.10689e+04_rb,0.81027e+03_rb,0.60137e+03_rb,0.88144e+03_rb,0.11649e+04_rb /)
      kbo(:, 5,18,12) = (/ &
        & 0.10691e+04_rb,0.80937e+03_rb,0.60029e+03_rb,0.88009e+03_rb,0.11632e+04_rb /)
      kbo(:, 1,19,12) = (/ &
        & 0.10732e+04_rb,0.81093e+03_rb,0.60257e+03_rb,0.88582e+03_rb,0.11702e+04_rb /)
      kbo(:, 2,19,12) = (/ &
        & 0.10665e+04_rb,0.81093e+03_rb,0.60218e+03_rb,0.88486e+03_rb,0.11689e+04_rb /)
      kbo(:, 3,19,12) = (/ &
        & 0.10668e+04_rb,0.80596e+03_rb,0.60188e+03_rb,0.88291e+03_rb,0.11667e+04_rb /)
      kbo(:, 4,19,12) = (/ &
        & 0.10625e+04_rb,0.80871e+03_rb,0.60129e+03_rb,0.88139e+03_rb,0.11647e+04_rb /)
      kbo(:, 5,19,12) = (/ &
        & 0.10641e+04_rb,0.80549e+03_rb,0.60005e+03_rb,0.87990e+03_rb,0.11631e+04_rb /)
      kbo(:, 1,20,12) = (/ &
        & 0.10655e+04_rb,0.80608e+03_rb,0.60228e+03_rb,0.88568e+03_rb,0.11700e+04_rb /)
      kbo(:, 2,20,12) = (/ &
        & 0.10621e+04_rb,0.80582e+03_rb,0.60205e+03_rb,0.88462e+03_rb,0.11686e+04_rb /)
      kbo(:, 3,20,12) = (/ &
        & 0.10596e+04_rb,0.80439e+03_rb,0.60161e+03_rb,0.88282e+03_rb,0.11664e+04_rb /)
      kbo(:, 4,20,12) = (/ &
        & 0.10569e+04_rb,0.80052e+03_rb,0.60133e+03_rb,0.88084e+03_rb,0.11645e+04_rb /)
      kbo(:, 5,20,12) = (/ &
        & 0.10576e+04_rb,0.80029e+03_rb,0.59980e+03_rb,0.87969e+03_rb,0.11629e+04_rb /)
      kbo(:, 1,21,12) = (/ &
        & 0.10601e+04_rb,0.80047e+03_rb,0.60235e+03_rb,0.88539e+03_rb,0.11699e+04_rb /)
      kbo(:, 2,21,12) = (/ &
        & 0.10553e+04_rb,0.80126e+03_rb,0.60195e+03_rb,0.88437e+03_rb,0.11684e+04_rb /)
      kbo(:, 3,21,12) = (/ &
        & 0.10581e+04_rb,0.79916e+03_rb,0.60159e+03_rb,0.88257e+03_rb,0.11662e+04_rb /)
      kbo(:, 4,21,12) = (/ &
        & 0.10523e+04_rb,0.79871e+03_rb,0.60090e+03_rb,0.88096e+03_rb,0.11643e+04_rb /)
      kbo(:, 5,21,12) = (/ &
        & 0.10536e+04_rb,0.79852e+03_rb,0.59957e+03_rb,0.87950e+03_rb,0.11627e+04_rb /)
      kbo(:, 1,22,12) = (/ &
        & 0.10543e+04_rb,0.79995e+03_rb,0.60207e+03_rb,0.88538e+03_rb,0.11697e+04_rb /)
      kbo(:, 2,22,12) = (/ &
        & 0.10531e+04_rb,0.79794e+03_rb,0.60195e+03_rb,0.88401e+03_rb,0.11680e+04_rb /)
      kbo(:, 3,22,12) = (/ &
        & 0.10510e+04_rb,0.79590e+03_rb,0.60141e+03_rb,0.88205e+03_rb,0.11658e+04_rb /)
      kbo(:, 4,22,12) = (/ &
        & 0.10516e+04_rb,0.79518e+03_rb,0.60121e+03_rb,0.88065e+03_rb,0.11640e+04_rb /)
      kbo(:, 5,22,12) = (/ &
        & 0.10507e+04_rb,0.79514e+03_rb,0.59925e+03_rb,0.87922e+03_rb,0.11625e+04_rb /)
      kbo(:, 1,23,12) = (/ &
        & 0.10496e+04_rb,0.79686e+03_rb,0.60138e+03_rb,0.88518e+03_rb,0.11695e+04_rb /)
      kbo(:, 2,23,12) = (/ &
        & 0.10500e+04_rb,0.79504e+03_rb,0.60174e+03_rb,0.88363e+03_rb,0.11675e+04_rb /)
      kbo(:, 3,23,12) = (/ &
        & 0.10497e+04_rb,0.79559e+03_rb,0.60122e+03_rb,0.88187e+03_rb,0.11654e+04_rb /)
      kbo(:, 4,23,12) = (/ &
        & 0.10478e+04_rb,0.79456e+03_rb,0.60013e+03_rb,0.88034e+03_rb,0.11637e+04_rb /)
      kbo(:, 5,23,12) = (/ &
        & 0.10481e+04_rb,0.79269e+03_rb,0.59893e+03_rb,0.87894e+03_rb,0.11623e+04_rb /)
      kbo(:, 1,24,12) = (/ &
        & 0.10504e+04_rb,0.79322e+03_rb,0.60186e+03_rb,0.88494e+03_rb,0.11692e+04_rb /)
      kbo(:, 2,24,12) = (/ &
        & 0.10474e+04_rb,0.79292e+03_rb,0.60174e+03_rb,0.88325e+03_rb,0.11671e+04_rb /)
      kbo(:, 3,24,12) = (/ &
        & 0.10457e+04_rb,0.79315e+03_rb,0.60103e+03_rb,0.88153e+03_rb,0.11648e+04_rb /)
      kbo(:, 4,24,12) = (/ &
        & 0.10462e+04_rb,0.79329e+03_rb,0.59996e+03_rb,0.88003e+03_rb,0.11633e+04_rb /)
      kbo(:, 5,24,12) = (/ &
        & 0.10477e+04_rb,0.79348e+03_rb,0.59860e+03_rb,0.87867e+03_rb,0.11620e+04_rb /)
      kbo(:, 1,25,12) = (/ &
        & 0.10437e+04_rb,0.79315e+03_rb,0.60179e+03_rb,0.88497e+03_rb,0.11688e+04_rb /)
      kbo(:, 2,25,12) = (/ &
        & 0.10446e+04_rb,0.79177e+03_rb,0.60151e+03_rb,0.88285e+03_rb,0.11666e+04_rb /)
      kbo(:, 3,25,12) = (/ &
        & 0.10443e+04_rb,0.79105e+03_rb,0.60081e+03_rb,0.88119e+03_rb,0.11647e+04_rb /)
      kbo(:, 4,25,12) = (/ &
        & 0.10440e+04_rb,0.79204e+03_rb,0.59965e+03_rb,0.87971e+03_rb,0.11630e+04_rb /)
      kbo(:, 5,25,12) = (/ &
        & 0.10441e+04_rb,0.79141e+03_rb,0.59825e+03_rb,0.87840e+03_rb,0.11618e+04_rb /)
      kbo(:, 1,26,12) = (/ &
        & 0.10444e+04_rb,0.79040e+03_rb,0.60171e+03_rb,0.88422e+03_rb,0.11681e+04_rb /)
      kbo(:, 2,26,12) = (/ &
        & 0.10415e+04_rb,0.78987e+03_rb,0.60136e+03_rb,0.88243e+03_rb,0.11661e+04_rb /)
      kbo(:, 3,26,12) = (/ &
        & 0.10417e+04_rb,0.79014e+03_rb,0.60043e+03_rb,0.88082e+03_rb,0.11642e+04_rb /)
      kbo(:, 4,26,12) = (/ &
        & 0.10442e+04_rb,0.78985e+03_rb,0.59932e+03_rb,0.87937e+03_rb,0.11625e+04_rb /)
      kbo(:, 5,26,12) = (/ &
        & 0.10421e+04_rb,0.79004e+03_rb,0.59787e+03_rb,0.87814e+03_rb,0.11615e+04_rb /)
      kbo(:, 1,27,12) = (/ &
        & 0.10431e+04_rb,0.78868e+03_rb,0.60163e+03_rb,0.88378e+03_rb,0.11678e+04_rb /)
      kbo(:, 2,27,12) = (/ &
        & 0.10423e+04_rb,0.79138e+03_rb,0.60118e+03_rb,0.88201e+03_rb,0.11657e+04_rb /)
      kbo(:, 3,27,12) = (/ &
        & 0.10404e+04_rb,0.78916e+03_rb,0.60037e+03_rb,0.88046e+03_rb,0.11638e+04_rb /)
      kbo(:, 4,27,12) = (/ &
        & 0.10405e+04_rb,0.78893e+03_rb,0.59897e+03_rb,0.87905e+03_rb,0.11624e+04_rb /)
      kbo(:, 5,27,12) = (/ &
        & 0.10396e+04_rb,0.78917e+03_rb,0.59748e+03_rb,0.87790e+03_rb,0.11613e+04_rb /)
      kbo(:, 1,28,12) = (/ &
        & 0.10409e+04_rb,0.79152e+03_rb,0.60155e+03_rb,0.88334e+03_rb,0.11673e+04_rb /)
      kbo(:, 2,28,12) = (/ &
        & 0.10439e+04_rb,0.78963e+03_rb,0.60099e+03_rb,0.88127e+03_rb,0.11652e+04_rb /)
      kbo(:, 3,28,12) = (/ &
        & 0.10423e+04_rb,0.78823e+03_rb,0.60006e+03_rb,0.88010e+03_rb,0.11635e+04_rb /)
      kbo(:, 4,28,12) = (/ &
        & 0.10442e+04_rb,0.78834e+03_rb,0.59861e+03_rb,0.87873e+03_rb,0.11621e+04_rb /)
      kbo(:, 5,28,12) = (/ &
        & 0.10392e+04_rb,0.79042e+03_rb,0.59708e+03_rb,0.87768e+03_rb,0.11610e+04_rb /)
      kbo(:, 1,29,12) = (/ &
        & 0.10405e+04_rb,0.79019e+03_rb,0.60167e+03_rb,0.88289e+03_rb,0.11667e+04_rb /)
      kbo(:, 2,29,12) = (/ &
        & 0.10432e+04_rb,0.78865e+03_rb,0.60111e+03_rb,0.88122e+03_rb,0.11647e+04_rb /)
      kbo(:, 3,29,12) = (/ &
        & 0.10406e+04_rb,0.78834e+03_rb,0.59962e+03_rb,0.87974e+03_rb,0.11631e+04_rb /)
      kbo(:, 4,29,12) = (/ &
        & 0.10397e+04_rb,0.78909e+03_rb,0.59824e+03_rb,0.87826e+03_rb,0.11618e+04_rb /)
      kbo(:, 5,29,12) = (/ &
        & 0.10387e+04_rb,0.78925e+03_rb,0.59665e+03_rb,0.87746e+03_rb,0.11608e+04_rb /)
      kbo(:, 1,30,12) = (/ &
        & 0.10411e+04_rb,0.78840e+03_rb,0.60130e+03_rb,0.88244e+03_rb,0.11662e+04_rb /)
      kbo(:, 2,30,12) = (/ &
        & 0.10395e+04_rb,0.79020e+03_rb,0.60051e+03_rb,0.88083e+03_rb,0.11643e+04_rb /)
      kbo(:, 3,30,12) = (/ &
        & 0.10384e+04_rb,0.78543e+03_rb,0.59928e+03_rb,0.87938e+03_rb,0.11627e+04_rb /)
      kbo(:, 4,30,12) = (/ &
        & 0.10394e+04_rb,0.78756e+03_rb,0.59785e+03_rb,0.87814e+03_rb,0.11615e+04_rb /)
      kbo(:, 5,30,12) = (/ &
        & 0.10383e+04_rb,0.78820e+03_rb,0.59620e+03_rb,0.87743e+03_rb,0.11605e+04_rb /)
      kbo(:, 1,31,12) = (/ &
        & 0.10385e+04_rb,0.78703e+03_rb,0.60113e+03_rb,0.88199e+03_rb,0.11657e+04_rb /)
      kbo(:, 2,31,12) = (/ &
        & 0.10401e+04_rb,0.79050e+03_rb,0.60021e+03_rb,0.88045e+03_rb,0.11639e+04_rb /)
      kbo(:, 3,31,12) = (/ &
        & 0.10412e+04_rb,0.78681e+03_rb,0.59892e+03_rb,0.87903e+03_rb,0.11624e+04_rb /)
      kbo(:, 4,31,12) = (/ &
        & 0.10385e+04_rb,0.78582e+03_rb,0.59732e+03_rb,0.87772e+03_rb,0.11613e+04_rb /)
      kbo(:, 5,31,12) = (/ &
        & 0.10372e+04_rb,0.78947e+03_rb,0.59573e+03_rb,0.87705e+03_rb,0.11602e+04_rb /)
      kbo(:, 1,32,12) = (/ &
        & 0.10378e+04_rb,0.78710e+03_rb,0.60105e+03_rb,0.88140e+03_rb,0.11652e+04_rb /)
      kbo(:, 2,32,12) = (/ &
        & 0.10375e+04_rb,0.78506e+03_rb,0.59989e+03_rb,0.88006e+03_rb,0.11634e+04_rb /)
      kbo(:, 3,32,12) = (/ &
        & 0.10409e+04_rb,0.78648e+03_rb,0.59855e+03_rb,0.87869e+03_rb,0.11621e+04_rb /)
      kbo(:, 4,32,12) = (/ &
        & 0.10389e+04_rb,0.78640e+03_rb,0.59701e+03_rb,0.87765e+03_rb,0.11610e+04_rb /)
      kbo(:, 5,32,12) = (/ &
        & 0.10378e+04_rb,0.78589e+03_rb,0.59527e+03_rb,0.87675e+03_rb,0.11598e+04_rb /)
      kbo(:, 1,33,12) = (/ &
        & 0.10380e+04_rb,0.78620e+03_rb,0.60093e+03_rb,0.88116e+03_rb,0.11647e+04_rb /)
      kbo(:, 2,33,12) = (/ &
        & 0.10383e+04_rb,0.78582e+03_rb,0.59966e+03_rb,0.87968e+03_rb,0.11631e+04_rb /)
      kbo(:, 3,33,12) = (/ &
        & 0.10388e+04_rb,0.78740e+03_rb,0.59816e+03_rb,0.87838e+03_rb,0.11618e+04_rb /)
      kbo(:, 4,33,12) = (/ &
        & 0.10367e+04_rb,0.78734e+03_rb,0.59657e+03_rb,0.87743e+03_rb,0.11608e+04_rb /)
      kbo(:, 5,33,12) = (/ &
        & 0.10382e+04_rb,0.78803e+03_rb,0.59484e+03_rb,0.87630e+03_rb,0.11592e+04_rb /)
      kbo(:, 1,34,12) = (/ &
        & 0.10390e+04_rb,0.78774e+03_rb,0.60058e+03_rb,0.88098e+03_rb,0.11643e+04_rb /)
      kbo(:, 2,34,12) = (/ &
        & 0.10374e+04_rb,0.78664e+03_rb,0.59936e+03_rb,0.87936e+03_rb,0.11627e+04_rb /)
      kbo(:, 3,34,12) = (/ &
        & 0.10352e+04_rb,0.78733e+03_rb,0.59781e+03_rb,0.87813e+03_rb,0.11615e+04_rb /)
      kbo(:, 4,34,12) = (/ &
        & 0.10386e+04_rb,0.78609e+03_rb,0.59617e+03_rb,0.87742e+03_rb,0.11605e+04_rb /)
      kbo(:, 5,34,12) = (/ &
        & 0.10400e+04_rb,0.78709e+03_rb,0.59476e+03_rb,0.87581e+03_rb,0.11592e+04_rb /)
      kbo(:, 1,35,12) = (/ &
        & 0.10374e+04_rb,0.78486e+03_rb,0.60032e+03_rb,0.88061e+03_rb,0.11641e+04_rb /)
      kbo(:, 2,35,12) = (/ &
        & 0.10385e+04_rb,0.78482e+03_rb,0.59906e+03_rb,0.87918e+03_rb,0.11626e+04_rb /)
      kbo(:, 3,35,12) = (/ &
        & 0.10371e+04_rb,0.78627e+03_rb,0.59760e+03_rb,0.87799e+03_rb,0.11614e+04_rb /)
      kbo(:, 4,35,12) = (/ &
        & 0.10378e+04_rb,0.78697e+03_rb,0.59593e+03_rb,0.87715e+03_rb,0.11604e+04_rb /)
      kbo(:, 5,35,12) = (/ &
        & 0.10370e+04_rb,0.78545e+03_rb,0.59500e+03_rb,0.87549e+03_rb,0.11592e+04_rb /)
      kbo(:, 1,36,12) = (/ &
        & 0.10362e+04_rb,0.78845e+03_rb,0.60029e+03_rb,0.88058e+03_rb,0.11640e+04_rb /)
      kbo(:, 2,36,12) = (/ &
        & 0.10379e+04_rb,0.78676e+03_rb,0.59903e+03_rb,0.87915e+03_rb,0.11625e+04_rb /)
      kbo(:, 3,36,12) = (/ &
        & 0.10369e+04_rb,0.78598e+03_rb,0.59768e+03_rb,0.87797e+03_rb,0.11614e+04_rb /)
      kbo(:, 4,36,12) = (/ &
        & 0.10369e+04_rb,0.78595e+03_rb,0.59566e+03_rb,0.87713e+03_rb,0.11604e+04_rb /)
      kbo(:, 5,36,12) = (/ &
        & 0.10368e+04_rb,0.78548e+03_rb,0.59464e+03_rb,0.87594e+03_rb,0.11600e+04_rb /)
      kbo(:, 1,37,12) = (/ &
        & 0.10374e+04_rb,0.78579e+03_rb,0.60055e+03_rb,0.88077e+03_rb,0.11642e+04_rb /)
      kbo(:, 2,37,12) = (/ &
        & 0.10362e+04_rb,0.78844e+03_rb,0.59921e+03_rb,0.87933e+03_rb,0.11627e+04_rb /)
      kbo(:, 3,37,12) = (/ &
        & 0.10360e+04_rb,0.78603e+03_rb,0.59777e+03_rb,0.87810e+03_rb,0.11615e+04_rb /)
      kbo(:, 4,37,12) = (/ &
        & 0.10367e+04_rb,0.78734e+03_rb,0.59623e+03_rb,0.87723e+03_rb,0.11605e+04_rb /)
      kbo(:, 5,37,12) = (/ &
        & 0.10361e+04_rb,0.78702e+03_rb,0.59474e+03_rb,0.87592e+03_rb,0.11587e+04_rb /)
      kbo(:, 1,38,12) = (/ &
        & 0.10367e+04_rb,0.78778e+03_rb,0.60058e+03_rb,0.88098e+03_rb,0.11645e+04_rb /)
      kbo(:, 2,38,12) = (/ &
        & 0.10387e+04_rb,0.78865e+03_rb,0.59939e+03_rb,0.87952e+03_rb,0.11629e+04_rb /)
      kbo(:, 3,38,12) = (/ &
        & 0.10363e+04_rb,0.78758e+03_rb,0.59820e+03_rb,0.87825e+03_rb,0.11617e+04_rb /)
      kbo(:, 4,38,12) = (/ &
        & 0.10372e+04_rb,0.78786e+03_rb,0.59636e+03_rb,0.87717e+03_rb,0.11609e+04_rb /)
      kbo(:, 5,38,12) = (/ &
        & 0.10362e+04_rb,0.78470e+03_rb,0.59485e+03_rb,0.87657e+03_rb,0.11589e+04_rb /)
      kbo(:, 1,39,12) = (/ &
        & 0.10374e+04_rb,0.78680e+03_rb,0.60094e+03_rb,0.88119e+03_rb,0.11647e+04_rb /)
      kbo(:, 2,39,12) = (/ &
        & 0.10352e+04_rb,0.78618e+03_rb,0.60014e+03_rb,0.87971e+03_rb,0.11631e+04_rb /)
      kbo(:, 3,39,12) = (/ &
        & 0.10401e+04_rb,0.78497e+03_rb,0.59818e+03_rb,0.87840e+03_rb,0.11618e+04_rb /)
      kbo(:, 4,39,12) = (/ &
        & 0.10369e+04_rb,0.78868e+03_rb,0.59660e+03_rb,0.87744e+03_rb,0.11608e+04_rb /)
      kbo(:, 5,39,12) = (/ &
        & 0.10353e+04_rb,0.78494e+03_rb,0.59497e+03_rb,0.87635e+03_rb,0.11593e+04_rb /)
      kbo(:, 1,40,12) = (/ &
        & 0.10385e+04_rb,0.78584e+03_rb,0.60091e+03_rb,0.88123e+03_rb,0.11652e+04_rb /)
      kbo(:, 2,40,12) = (/ &
        & 0.10387e+04_rb,0.78568e+03_rb,0.59987e+03_rb,0.88006e+03_rb,0.11635e+04_rb /)
      kbo(:, 3,40,12) = (/ &
        & 0.10371e+04_rb,0.78626e+03_rb,0.59853e+03_rb,0.87869e+03_rb,0.11621e+04_rb /)
      kbo(:, 4,40,12) = (/ &
        & 0.10386e+04_rb,0.78684e+03_rb,0.59700e+03_rb,0.87748e+03_rb,0.11610e+04_rb /)
      kbo(:, 5,40,12) = (/ &
        & 0.10362e+04_rb,0.78503e+03_rb,0.59526e+03_rb,0.87692e+03_rb,0.11598e+04_rb /)
      kbo(:, 1,41,12) = (/ &
        & 0.10384e+04_rb,0.78504e+03_rb,0.60109e+03_rb,0.88198e+03_rb,0.11657e+04_rb /)
      kbo(:, 2,41,12) = (/ &
        & 0.10360e+04_rb,0.78533e+03_rb,0.60017e+03_rb,0.88043e+03_rb,0.11639e+04_rb /)
      kbo(:, 3,41,12) = (/ &
        & 0.10374e+04_rb,0.78412e+03_rb,0.59889e+03_rb,0.87902e+03_rb,0.11624e+04_rb /)
      kbo(:, 4,41,12) = (/ &
        & 0.10350e+04_rb,0.78545e+03_rb,0.59741e+03_rb,0.87788e+03_rb,0.11613e+04_rb /)
      kbo(:, 5,41,12) = (/ &
        & 0.10385e+04_rb,0.78529e+03_rb,0.59570e+03_rb,0.87704e+03_rb,0.11607e+04_rb /)
      kbo(:, 1,42,12) = (/ &
        & 0.10370e+04_rb,0.78560e+03_rb,0.60125e+03_rb,0.88242e+03_rb,0.11662e+04_rb /)
      kbo(:, 2,42,12) = (/ &
        & 0.10383e+04_rb,0.78469e+03_rb,0.60057e+03_rb,0.88081e+03_rb,0.11643e+04_rb /)
      kbo(:, 3,42,12) = (/ &
        & 0.10368e+04_rb,0.78617e+03_rb,0.59924e+03_rb,0.87937e+03_rb,0.11627e+04_rb /)
      kbo(:, 4,42,12) = (/ &
        & 0.10362e+04_rb,0.78586e+03_rb,0.59780e+03_rb,0.87813e+03_rb,0.11615e+04_rb /)
      kbo(:, 5,42,12) = (/ &
        & 0.10350e+04_rb,0.78958e+03_rb,0.59616e+03_rb,0.87725e+03_rb,0.11605e+04_rb /)
      kbo(:, 1,43,12) = (/ &
        & 0.10375e+04_rb,0.78586e+03_rb,0.60140e+03_rb,0.88280e+03_rb,0.11669e+04_rb /)
      kbo(:, 2,43,12) = (/ &
        & 0.10344e+04_rb,0.78615e+03_rb,0.60088e+03_rb,0.88112e+03_rb,0.11651e+04_rb /)
      kbo(:, 3,43,12) = (/ &
        & 0.10344e+04_rb,0.78345e+03_rb,0.59953e+03_rb,0.87980e+03_rb,0.11632e+04_rb /)
      kbo(:, 4,43,12) = (/ &
        & 0.10347e+04_rb,0.78777e+03_rb,0.59827e+03_rb,0.87847e+03_rb,0.11619e+04_rb /)
      kbo(:, 5,43,12) = (/ &
        & 0.10355e+04_rb,0.78627e+03_rb,0.59681e+03_rb,0.87766e+03_rb,0.11609e+04_rb /)
      kbo(:, 1,44,12) = (/ &
        & 0.10355e+04_rb,0.78449e+03_rb,0.60162e+03_rb,0.88358e+03_rb,0.11676e+04_rb /)
      kbo(:, 2,44,12) = (/ &
        & 0.10359e+04_rb,0.78627e+03_rb,0.60103e+03_rb,0.88182e+03_rb,0.11655e+04_rb /)
      kbo(:, 3,44,12) = (/ &
        & 0.10377e+04_rb,0.78715e+03_rb,0.60005e+03_rb,0.88029e+03_rb,0.11637e+04_rb /)
      kbo(:, 4,44,12) = (/ &
        & 0.10378e+04_rb,0.78625e+03_rb,0.59875e+03_rb,0.87890e+03_rb,0.11623e+04_rb /)
      kbo(:, 5,44,12) = (/ &
        & 0.10374e+04_rb,0.78401e+03_rb,0.59725e+03_rb,0.87779e+03_rb,0.11612e+04_rb /)
      kbo(:, 1,45,12) = (/ &
        & 0.10390e+04_rb,0.78655e+03_rb,0.60158e+03_rb,0.88419e+03_rb,0.11684e+04_rb /)
      kbo(:, 2,45,12) = (/ &
        & 0.10396e+04_rb,0.78684e+03_rb,0.60124e+03_rb,0.88239e+03_rb,0.11662e+04_rb /)
      kbo(:, 3,45,12) = (/ &
        & 0.10355e+04_rb,0.78758e+03_rb,0.60044e+03_rb,0.88079e+03_rb,0.11643e+04_rb /)
      kbo(:, 4,45,12) = (/ &
        & 0.10368e+04_rb,0.78548e+03_rb,0.59922e+03_rb,0.87935e+03_rb,0.11627e+04_rb /)
      kbo(:, 5,45,12) = (/ &
        & 0.10360e+04_rb,0.78578e+03_rb,0.59778e+03_rb,0.87811e+03_rb,0.11615e+04_rb /)
      kbo(:, 1,46,12) = (/ &
        & 0.10390e+04_rb,0.78534e+03_rb,0.60165e+03_rb,0.88476e+03_rb,0.11691e+04_rb /)
      kbo(:, 2,46,12) = (/ &
        & 0.10368e+04_rb,0.78339e+03_rb,0.60141e+03_rb,0.88303e+03_rb,0.11669e+04_rb /)
      kbo(:, 3,46,12) = (/ &
        & 0.10364e+04_rb,0.78696e+03_rb,0.60079e+03_rb,0.88117e+03_rb,0.11649e+04_rb /)
      kbo(:, 4,46,12) = (/ &
        & 0.10407e+04_rb,0.78666e+03_rb,0.59968e+03_rb,0.87985e+03_rb,0.11632e+04_rb /)
      kbo(:, 5,46,12) = (/ &
        & 0.10344e+04_rb,0.78501e+03_rb,0.59832e+03_rb,0.87835e+03_rb,0.11619e+04_rb /)
      kbo(:, 1,47,12) = (/ &
        & 0.10372e+04_rb,0.78547e+03_rb,0.60140e+03_rb,0.88518e+03_rb,0.11696e+04_rb /)
      kbo(:, 2,47,12) = (/ &
        & 0.10368e+04_rb,0.78740e+03_rb,0.60142e+03_rb,0.88376e+03_rb,0.11678e+04_rb /)
      kbo(:, 3,47,12) = (/ &
        & 0.10360e+04_rb,0.78411e+03_rb,0.60109e+03_rb,0.88198e+03_rb,0.11657e+04_rb /)
      kbo(:, 4,47,12) = (/ &
        & 0.10358e+04_rb,0.78709e+03_rb,0.60017e+03_rb,0.88044e+03_rb,0.11639e+04_rb /)
      kbo(:, 5,47,12) = (/ &
        & 0.10354e+04_rb,0.78664e+03_rb,0.59889e+03_rb,0.87902e+03_rb,0.11624e+04_rb /)
      kbo(:, 1,48,12) = (/ &
        & 0.10362e+04_rb,0.78680e+03_rb,0.60195e+03_rb,0.88548e+03_rb,0.11701e+04_rb /)
      kbo(:, 2,48,12) = (/ &
        & 0.10362e+04_rb,0.78545e+03_rb,0.60161e+03_rb,0.88447e+03_rb,0.11687e+04_rb /)
      kbo(:, 3,48,12) = (/ &
        & 0.10378e+04_rb,0.78604e+03_rb,0.60144e+03_rb,0.88268e+03_rb,0.11665e+04_rb /)
      kbo(:, 4,48,12) = (/ &
        & 0.10382e+04_rb,0.78544e+03_rb,0.60061e+03_rb,0.88104e+03_rb,0.11646e+04_rb /)
      kbo(:, 5,48,12) = (/ &
        & 0.10364e+04_rb,0.78424e+03_rb,0.59943e+03_rb,0.87957e+03_rb,0.11630e+04_rb /)
      kbo(:, 1,49,12) = (/ &
        & 0.10370e+04_rb,0.78693e+03_rb,0.60225e+03_rb,0.88600e+03_rb,0.11705e+04_rb /)
      kbo(:, 2,49,12) = (/ &
        & 0.10356e+04_rb,0.78469e+03_rb,0.60169e+03_rb,0.88499e+03_rb,0.11694e+04_rb /)
      kbo(:, 3,49,12) = (/ &
        & 0.10350e+04_rb,0.78588e+03_rb,0.60148e+03_rb,0.88340e+03_rb,0.11674e+04_rb /)
      kbo(:, 4,49,12) = (/ &
        & 0.10352e+04_rb,0.78677e+03_rb,0.60095e+03_rb,0.88166e+03_rb,0.11653e+04_rb /)
      kbo(:, 5,49,12) = (/ &
        & 0.10356e+04_rb,0.78647e+03_rb,0.59993e+03_rb,0.88015e+03_rb,0.11636e+04_rb /)
      kbo(:, 1,50,12) = (/ &
        & 0.10356e+04_rb,0.78558e+03_rb,0.60226e+03_rb,0.88620e+03_rb,0.11712e+04_rb /)
      kbo(:, 2,50,12) = (/ &
        & 0.10346e+04_rb,0.78796e+03_rb,0.60171e+03_rb,0.88533e+03_rb,0.11698e+04_rb /)
      kbo(:, 3,50,12) = (/ &
        & 0.10366e+04_rb,0.78751e+03_rb,0.60168e+03_rb,0.88408e+03_rb,0.11667e+04_rb /)
      kbo(:, 4,50,12) = (/ &
        & 0.10366e+04_rb,0.78646e+03_rb,0.60132e+03_rb,0.88229e+03_rb,0.11661e+04_rb /)
      kbo(:, 5,50,12) = (/ &
        & 0.10393e+04_rb,0.78616e+03_rb,0.60083e+03_rb,0.88070e+03_rb,0.11642e+04_rb /)
      kbo(:, 1,51,12) = (/ &
        & 0.10356e+04_rb,0.78468e+03_rb,0.60241e+03_rb,0.88644e+03_rb,0.11719e+04_rb /)
      kbo(:, 2,51,12) = (/ &
        & 0.10368e+04_rb,0.78423e+03_rb,0.60203e+03_rb,0.88559e+03_rb,0.11702e+04_rb /)
      kbo(:, 3,51,12) = (/ &
        & 0.10356e+04_rb,0.78586e+03_rb,0.60163e+03_rb,0.88469e+03_rb,0.11690e+04_rb /)
      kbo(:, 4,51,12) = (/ &
        & 0.10358e+04_rb,0.78601e+03_rb,0.60139e+03_rb,0.88294e+03_rb,0.11668e+04_rb /)
      kbo(:, 5,51,12) = (/ &
        & 0.10379e+04_rb,0.78646e+03_rb,0.60074e+03_rb,0.88126e+03_rb,0.11648e+04_rb /)
      kbo(:, 1,52,12) = (/ &
        & 0.10383e+04_rb,0.78512e+03_rb,0.60257e+03_rb,0.88672e+03_rb,0.11727e+04_rb /)
      kbo(:, 2,52,12) = (/ &
        & 0.10350e+04_rb,0.78646e+03_rb,0.60217e+03_rb,0.88594e+03_rb,0.11707e+04_rb /)
      kbo(:, 3,52,12) = (/ &
        & 0.10358e+04_rb,0.78466e+03_rb,0.60160e+03_rb,0.88510e+03_rb,0.11695e+04_rb /)
      kbo(:, 4,52,12) = (/ &
        & 0.10356e+04_rb,0.78511e+03_rb,0.60151e+03_rb,0.88359e+03_rb,0.11676e+04_rb /)
      kbo(:, 5,52,12) = (/ &
        & 0.10365e+04_rb,0.78541e+03_rb,0.60103e+03_rb,0.88167e+03_rb,0.11655e+04_rb /)
      kbo(:, 1,53,12) = (/ &
        & 0.10352e+04_rb,0.78496e+03_rb,0.60273e+03_rb,0.88710e+03_rb,0.11735e+04_rb /)
      kbo(:, 2,53,12) = (/ &
        & 0.10359e+04_rb,0.78600e+03_rb,0.60229e+03_rb,0.88628e+03_rb,0.11714e+04_rb /)
      kbo(:, 3,53,12) = (/ &
        & 0.10369e+04_rb,0.78526e+03_rb,0.60188e+03_rb,0.88540e+03_rb,0.11699e+04_rb /)
      kbo(:, 4,53,12) = (/ &
        & 0.10383e+04_rb,0.78660e+03_rb,0.60159e+03_rb,0.88425e+03_rb,0.11685e+04_rb /)
      kbo(:, 5,53,12) = (/ &
        & 0.10373e+04_rb,0.78555e+03_rb,0.60126e+03_rb,0.88246e+03_rb,0.11663e+04_rb /)
      kbo(:, 1,54,12) = (/ &
        & 0.10367e+04_rb,0.78451e+03_rb,0.60297e+03_rb,0.88752e+03_rb,0.11746e+04_rb /)
      kbo(:, 2,54,12) = (/ &
        & 0.10355e+04_rb,0.78600e+03_rb,0.60244e+03_rb,0.88648e+03_rb,0.11721e+04_rb /)
      kbo(:, 3,54,12) = (/ &
        & 0.10348e+04_rb,0.78301e+03_rb,0.60206e+03_rb,0.88564e+03_rb,0.11703e+04_rb /)
      kbo(:, 4,54,12) = (/ &
        & 0.10395e+04_rb,0.78629e+03_rb,0.60165e+03_rb,0.88477e+03_rb,0.11691e+04_rb /)
      kbo(:, 5,54,12) = (/ &
        & 0.10361e+04_rb,0.78420e+03_rb,0.60141e+03_rb,0.88305e+03_rb,0.11670e+04_rb /)
      kbo(:, 1,55,12) = (/ &
        & 0.10361e+04_rb,0.78555e+03_rb,0.60296e+03_rb,0.88822e+03_rb,0.11760e+04_rb /)
      kbo(:, 2,55,12) = (/ &
        & 0.10355e+04_rb,0.78495e+03_rb,0.60258e+03_rb,0.88674e+03_rb,0.11727e+04_rb /)
      kbo(:, 3,55,12) = (/ &
        & 0.10349e+04_rb,0.78525e+03_rb,0.60217e+03_rb,0.88596e+03_rb,0.11709e+04_rb /)
      kbo(:, 4,55,12) = (/ &
        & 0.10361e+04_rb,0.78465e+03_rb,0.60184e+03_rb,0.88512e+03_rb,0.11696e+04_rb /)
      kbo(:, 5,55,12) = (/ &
        & 0.10363e+04_rb,0.78599e+03_rb,0.60163e+03_rb,0.88363e+03_rb,0.11677e+04_rb /)
      kbo(:, 1,56,12) = (/ &
        & 0.10361e+04_rb,0.78599e+03_rb,0.60309e+03_rb,0.88918e+03_rb,0.11774e+04_rb /)
      kbo(:, 2,56,12) = (/ &
        & 0.10391e+04_rb,0.78674e+03_rb,0.60272e+03_rb,0.88707e+03_rb,0.11734e+04_rb /)
      kbo(:, 3,56,12) = (/ &
        & 0.10363e+04_rb,0.78584e+03_rb,0.60228e+03_rb,0.88626e+03_rb,0.11716e+04_rb /)
      kbo(:, 4,56,12) = (/ &
        & 0.10369e+04_rb,0.78509e+03_rb,0.60186e+03_rb,0.88538e+03_rb,0.11695e+04_rb /)
      kbo(:, 5,56,12) = (/ &
        & 0.10363e+04_rb,0.78554e+03_rb,0.60158e+03_rb,0.88421e+03_rb,0.11682e+04_rb /)
      kbo(:, 1,57,12) = (/ &
        & 0.10381e+04_rb,0.78509e+03_rb,0.60327e+03_rb,0.89026e+03_rb,0.11789e+04_rb /)
      kbo(:, 2,57,12) = (/ &
        & 0.10344e+04_rb,0.78852e+03_rb,0.60284e+03_rb,0.88747e+03_rb,0.11744e+04_rb /)
      kbo(:, 3,57,12) = (/ &
        & 0.10369e+04_rb,0.78509e+03_rb,0.60242e+03_rb,0.88612e+03_rb,0.11720e+04_rb /)
      kbo(:, 4,57,12) = (/ &
        & 0.10393e+04_rb,0.78330e+03_rb,0.60216e+03_rb,0.88561e+03_rb,0.11703e+04_rb /)
      kbo(:, 5,57,12) = (/ &
        & 0.10365e+04_rb,0.78494e+03_rb,0.60164e+03_rb,0.88472e+03_rb,0.11688e+04_rb /)
      kbo(:, 1,58,12) = (/ &
        & 0.10353e+04_rb,0.78509e+03_rb,0.60357e+03_rb,0.89125e+03_rb,0.11802e+04_rb /)
      kbo(:, 2,58,12) = (/ &
        & 0.10371e+04_rb,0.78524e+03_rb,0.60283e+03_rb,0.88807e+03_rb,0.11757e+04_rb /)
      kbo(:, 3,58,12) = (/ &
        & 0.10365e+04_rb,0.78718e+03_rb,0.60278e+03_rb,0.88669e+03_rb,0.11726e+04_rb /)
      kbo(:, 4,58,12) = (/ &
        & 0.10349e+04_rb,0.78673e+03_rb,0.60216e+03_rb,0.88590e+03_rb,0.11708e+04_rb /)
      kbo(:, 5,58,12) = (/ &
        & 0.10365e+04_rb,0.78389e+03_rb,0.60170e+03_rb,0.88506e+03_rb,0.11695e+04_rb /)
      kbo(:, 1,59,12) = (/ &
        & 0.10365e+04_rb,0.78390e+03_rb,0.60373e+03_rb,0.89165e+03_rb,0.11808e+04_rb /)
      kbo(:, 2,59,12) = (/ &
        & 0.10369e+04_rb,0.78479e+03_rb,0.60298e+03_rb,0.88840e+03_rb,0.11763e+04_rb /)
      kbo(:, 3,59,12) = (/ &
        & 0.10357e+04_rb,0.78703e+03_rb,0.60261e+03_rb,0.88680e+03_rb,0.11713e+04_rb /)
      kbo(:, 4,59,12) = (/ &
        & 0.10363e+04_rb,0.78509e+03_rb,0.60219e+03_rb,0.88602e+03_rb,0.11708e+04_rb /)
      kbo(:, 5,59,12) = (/ &
        & 0.10365e+04_rb,0.78479e+03_rb,0.60174e+03_rb,0.88517e+03_rb,0.11696e+04_rb /)
      kbo(:, 1,13,13) = (/ &
        & 0.11661e+04_rb,0.87073e+03_rb,0.60556e+03_rb,0.88929e+03_rb,0.11761e+04_rb /)
      kbo(:, 2,13,13) = (/ &
        & 0.11527e+04_rb,0.87014e+03_rb,0.60489e+03_rb,0.88914e+03_rb,0.11754e+04_rb /)
      kbo(:, 3,13,13) = (/ &
        & 0.11423e+04_rb,0.85968e+03_rb,0.60419e+03_rb,0.88867e+03_rb,0.11747e+04_rb /)
      kbo(:, 4,13,13) = (/ &
        & 0.11371e+04_rb,0.85890e+03_rb,0.60344e+03_rb,0.88820e+03_rb,0.11742e+04_rb /)
      kbo(:, 5,13,13) = (/ &
        & 0.11390e+04_rb,0.85008e+03_rb,0.60234e+03_rb,0.88705e+03_rb,0.11727e+04_rb /)
      kbo(:, 1,14,13) = (/ &
        & 0.11362e+04_rb,0.85805e+03_rb,0.60511e+03_rb,0.88957e+03_rb,0.11761e+04_rb /)
      kbo(:, 2,14,13) = (/ &
        & 0.11276e+04_rb,0.85398e+03_rb,0.60448e+03_rb,0.88893e+03_rb,0.11753e+04_rb /)
      kbo(:, 3,14,13) = (/ &
        & 0.11208e+04_rb,0.84888e+03_rb,0.60397e+03_rb,0.88854e+03_rb,0.11747e+04_rb /)
      kbo(:, 4,14,13) = (/ &
        & 0.11194e+04_rb,0.85006e+03_rb,0.60305e+03_rb,0.88823e+03_rb,0.11740e+04_rb /)
      kbo(:, 5,14,13) = (/ &
        & 0.11083e+04_rb,0.84212e+03_rb,0.60191e+03_rb,0.88682e+03_rb,0.11725e+04_rb /)
      kbo(:, 1,15,13) = (/ &
        & 0.11142e+04_rb,0.84766e+03_rb,0.60488e+03_rb,0.88941e+03_rb,0.11748e+04_rb /)
      kbo(:, 2,15,13) = (/ &
        & 0.11176e+04_rb,0.84112e+03_rb,0.60414e+03_rb,0.88873e+03_rb,0.11752e+04_rb /)
      kbo(:, 3,15,13) = (/ &
        & 0.11051e+04_rb,0.84191e+03_rb,0.60348e+03_rb,0.88843e+03_rb,0.11747e+04_rb /)
      kbo(:, 4,15,13) = (/ &
        & 0.11036e+04_rb,0.83753e+03_rb,0.60270e+03_rb,0.88781e+03_rb,0.11739e+04_rb /)
      kbo(:, 5,15,13) = (/ &
        & 0.10935e+04_rb,0.82883e+03_rb,0.60139e+03_rb,0.88658e+03_rb,0.11723e+04_rb /)
      kbo(:, 1,16,13) = (/ &
        & 0.11074e+04_rb,0.83456e+03_rb,0.60440e+03_rb,0.88927e+03_rb,0.11760e+04_rb /)
      kbo(:, 2,16,13) = (/ &
        & 0.11018e+04_rb,0.83298e+03_rb,0.60385e+03_rb,0.88856e+03_rb,0.11751e+04_rb /)
      kbo(:, 3,16,13) = (/ &
        & 0.10859e+04_rb,0.83101e+03_rb,0.60319e+03_rb,0.88832e+03_rb,0.11747e+04_rb /)
      kbo(:, 4,16,13) = (/ &
        & 0.10925e+04_rb,0.82836e+03_rb,0.60254e+03_rb,0.88762e+03_rb,0.11738e+04_rb /)
      kbo(:, 5,16,13) = (/ &
        & 0.10900e+04_rb,0.82283e+03_rb,0.60124e+03_rb,0.88636e+03_rb,0.11721e+04_rb /)
      kbo(:, 1,17,13) = (/ &
        & 0.10953e+04_rb,0.82596e+03_rb,0.60413e+03_rb,0.88915e+03_rb,0.11759e+04_rb /)
      kbo(:, 2,17,13) = (/ &
        & 0.10864e+04_rb,0.82504e+03_rb,0.60360e+03_rb,0.88842e+03_rb,0.11750e+04_rb /)
      kbo(:, 3,17,13) = (/ &
        & 0.10812e+04_rb,0.81898e+03_rb,0.60295e+03_rb,0.88822e+03_rb,0.11746e+04_rb /)
      kbo(:, 4,17,13) = (/ &
        & 0.10806e+04_rb,0.81706e+03_rb,0.60242e+03_rb,0.88745e+03_rb,0.11736e+04_rb /)
      kbo(:, 5,17,13) = (/ &
        & 0.10764e+04_rb,0.81771e+03_rb,0.59943e+03_rb,0.88638e+03_rb,0.11722e+04_rb /)
      kbo(:, 1,18,13) = (/ &
        & 0.10787e+04_rb,0.82265e+03_rb,0.60390e+03_rb,0.88903e+03_rb,0.11759e+04_rb /)
      kbo(:, 2,18,13) = (/ &
        & 0.10743e+04_rb,0.81466e+03_rb,0.60339e+03_rb,0.88738e+03_rb,0.11749e+04_rb /)
      kbo(:, 3,18,13) = (/ &
        & 0.10792e+04_rb,0.81648e+03_rb,0.60273e+03_rb,0.88812e+03_rb,0.11749e+04_rb /)
      kbo(:, 4,18,13) = (/ &
        & 0.10724e+04_rb,0.81516e+03_rb,0.60185e+03_rb,0.88728e+03_rb,0.11735e+04_rb /)
      kbo(:, 5,18,13) = (/ &
        & 0.10737e+04_rb,0.81301e+03_rb,0.60092e+03_rb,0.88618e+03_rb,0.11717e+04_rb /)
      kbo(:, 1,19,13) = (/ &
        & 0.10778e+04_rb,0.81646e+03_rb,0.60388e+03_rb,0.88893e+03_rb,0.11758e+04_rb /)
      kbo(:, 2,19,13) = (/ &
        & 0.10709e+04_rb,0.81021e+03_rb,0.60321e+03_rb,0.88822e+03_rb,0.11748e+04_rb /)
      kbo(:, 3,19,13) = (/ &
        & 0.10651e+04_rb,0.80945e+03_rb,0.60254e+03_rb,0.88803e+03_rb,0.11745e+04_rb /)
      kbo(:, 4,19,13) = (/ &
        & 0.10677e+04_rb,0.80743e+03_rb,0.60147e+03_rb,0.88735e+03_rb,0.11733e+04_rb /)
      kbo(:, 5,19,13) = (/ &
        & 0.10575e+04_rb,0.80245e+03_rb,0.60072e+03_rb,0.88575e+03_rb,0.11721e+04_rb /)
      kbo(:, 1,20,13) = (/ &
        & 0.10622e+04_rb,0.81096e+03_rb,0.60356e+03_rb,0.88881e+03_rb,0.11757e+04_rb /)
      kbo(:, 2,20,13) = (/ &
        & 0.10607e+04_rb,0.80739e+03_rb,0.60305e+03_rb,0.88816e+03_rb,0.11748e+04_rb /)
      kbo(:, 3,20,13) = (/ &
        & 0.10623e+04_rb,0.80381e+03_rb,0.60252e+03_rb,0.88792e+03_rb,0.11744e+04_rb /)
      kbo(:, 4,20,13) = (/ &
        & 0.10570e+04_rb,0.80456e+03_rb,0.60093e+03_rb,0.88695e+03_rb,0.11731e+04_rb /)
      kbo(:, 5,20,13) = (/ &
        & 0.10628e+04_rb,0.80481e+03_rb,0.60051e+03_rb,0.88554e+03_rb,0.11713e+04_rb /)
      kbo(:, 1,21,13) = (/ &
        & 0.10564e+04_rb,0.80721e+03_rb,0.60326e+03_rb,0.88869e+03_rb,0.11756e+04_rb /)
      kbo(:, 2,21,13) = (/ &
        & 0.10630e+04_rb,0.79813e+03_rb,0.60290e+03_rb,0.88813e+03_rb,0.11748e+04_rb /)
      kbo(:, 3,21,13) = (/ &
        & 0.10553e+04_rb,0.80228e+03_rb,0.60219e+03_rb,0.88781e+03_rb,0.11743e+04_rb /)
      kbo(:, 4,21,13) = (/ &
        & 0.10544e+04_rb,0.79888e+03_rb,0.60136e+03_rb,0.88677e+03_rb,0.11729e+04_rb /)
      kbo(:, 5,21,13) = (/ &
        & 0.10596e+04_rb,0.81049e+03_rb,0.60032e+03_rb,0.88532e+03_rb,0.11710e+04_rb /)
      kbo(:, 1,22,13) = (/ &
        & 0.10550e+04_rb,0.80310e+03_rb,0.60326e+03_rb,0.88850e+03_rb,0.11754e+04_rb /)
      kbo(:, 2,22,13) = (/ &
        & 0.10516e+04_rb,0.80380e+03_rb,0.60256e+03_rb,0.88810e+03_rb,0.11747e+04_rb /)
      kbo(:, 3,22,13) = (/ &
        & 0.10562e+04_rb,0.80152e+03_rb,0.60229e+03_rb,0.88764e+03_rb,0.11741e+04_rb /)
      kbo(:, 4,22,13) = (/ &
        & 0.10509e+04_rb,0.80224e+03_rb,0.60002e+03_rb,0.88651e+03_rb,0.11726e+04_rb /)
      kbo(:, 5,22,13) = (/ &
        & 0.10563e+04_rb,0.79871e+03_rb,0.59990e+03_rb,0.88500e+03_rb,0.11706e+04_rb /)
      kbo(:, 1,23,13) = (/ &
        & 0.10567e+04_rb,0.79860e+03_rb,0.60326e+03_rb,0.88832e+03_rb,0.11752e+04_rb /)
      kbo(:, 2,23,13) = (/ &
        & 0.10532e+04_rb,0.79557e+03_rb,0.60254e+03_rb,0.88806e+03_rb,0.11747e+04_rb /)
      kbo(:, 3,23,13) = (/ &
        & 0.10534e+04_rb,0.79512e+03_rb,0.60237e+03_rb,0.88744e+03_rb,0.11739e+04_rb /)
      kbo(:, 4,23,13) = (/ &
        & 0.10472e+04_rb,0.79721e+03_rb,0.60072e+03_rb,0.88622e+03_rb,0.11723e+04_rb /)
      kbo(:, 5,23,13) = (/ &
        & 0.10509e+04_rb,0.79423e+03_rb,0.59964e+03_rb,0.88466e+03_rb,0.11702e+04_rb /)
      kbo(:, 1,24,13) = (/ &
        & 0.10549e+04_rb,0.79502e+03_rb,0.60296e+03_rb,0.88817e+03_rb,0.11750e+04_rb /)
      kbo(:, 2,24,13) = (/ &
        & 0.10498e+04_rb,0.80428e+03_rb,0.60221e+03_rb,0.88799e+03_rb,0.11746e+04_rb /)
      kbo(:, 3,24,13) = (/ &
        & 0.10553e+04_rb,0.79369e+03_rb,0.60149e+03_rb,0.88721e+03_rb,0.11736e+04_rb /)
      kbo(:, 4,24,13) = (/ &
        & 0.10517e+04_rb,0.79240e+03_rb,0.60048e+03_rb,0.88592e+03_rb,0.11719e+04_rb /)
      kbo(:, 5,24,13) = (/ &
        & 0.10493e+04_rb,0.79281e+03_rb,0.59936e+03_rb,0.88429e+03_rb,0.11697e+04_rb /)
      kbo(:, 1,25,13) = (/ &
        & 0.10472e+04_rb,0.79884e+03_rb,0.60281e+03_rb,0.88760e+03_rb,0.11748e+04_rb /)
      kbo(:, 2,25,13) = (/ &
        & 0.10433e+04_rb,0.79261e+03_rb,0.60217e+03_rb,0.88787e+03_rb,0.11745e+04_rb /)
      kbo(:, 3,25,13) = (/ &
        & 0.10495e+04_rb,0.79568e+03_rb,0.60138e+03_rb,0.88696e+03_rb,0.11733e+04_rb /)
      kbo(:, 4,25,13) = (/ &
        & 0.10412e+04_rb,0.79264e+03_rb,0.60023e+03_rb,0.88559e+03_rb,0.11715e+04_rb /)
      kbo(:, 5,25,13) = (/ &
        & 0.10432e+04_rb,0.79682e+03_rb,0.59908e+03_rb,0.88390e+03_rb,0.11692e+04_rb /)
      kbo(:, 1,26,13) = (/ &
        & 0.10426e+04_rb,0.79538e+03_rb,0.60266e+03_rb,0.88804e+03_rb,0.11748e+04_rb /)
      kbo(:, 2,26,13) = (/ &
        & 0.10459e+04_rb,0.79909e+03_rb,0.60196e+03_rb,0.88771e+03_rb,0.11743e+04_rb /)
      kbo(:, 3,26,13) = (/ &
        & 0.10455e+04_rb,0.79078e+03_rb,0.60113e+03_rb,0.88666e+03_rb,0.11729e+04_rb /)
      kbo(:, 4,26,13) = (/ &
        & 0.10380e+04_rb,0.79069e+03_rb,0.60011e+03_rb,0.88521e+03_rb,0.11710e+04_rb /)
      kbo(:, 5,26,13) = (/ &
        & 0.10428e+04_rb,0.79124e+03_rb,0.59892e+03_rb,0.88345e+03_rb,0.11686e+04_rb /)
      kbo(:, 1,27,13) = (/ &
        & 0.10409e+04_rb,0.79057e+03_rb,0.60249e+03_rb,0.88802e+03_rb,0.11747e+04_rb /)
      kbo(:, 2,27,13) = (/ &
        & 0.10432e+04_rb,0.78863e+03_rb,0.60172e+03_rb,0.88750e+03_rb,0.11740e+04_rb /)
      kbo(:, 3,27,13) = (/ &
        & 0.10404e+04_rb,0.79041e+03_rb,0.60057e+03_rb,0.88634e+03_rb,0.11728e+04_rb /)
      kbo(:, 4,27,13) = (/ &
        & 0.10425e+04_rb,0.78854e+03_rb,0.59967e+03_rb,0.88481e+03_rb,0.11704e+04_rb /)
      kbo(:, 5,27,13) = (/ &
        & 0.10492e+04_rb,0.78874e+03_rb,0.59843e+03_rb,0.88297e+03_rb,0.11680e+04_rb /)
      kbo(:, 1,28,13) = (/ &
        & 0.10434e+04_rb,0.79273e+03_rb,0.60231e+03_rb,0.88820e+03_rb,0.11747e+04_rb /)
      kbo(:, 2,28,13) = (/ &
        & 0.10472e+04_rb,0.78799e+03_rb,0.60162e+03_rb,0.88726e+03_rb,0.11737e+04_rb /)
      kbo(:, 3,28,13) = (/ &
        & 0.10366e+04_rb,0.79125e+03_rb,0.60032e+03_rb,0.88600e+03_rb,0.11720e+04_rb /)
      kbo(:, 4,28,13) = (/ &
        & 0.10345e+04_rb,0.78841e+03_rb,0.59938e+03_rb,0.88440e+03_rb,0.11699e+04_rb /)
      kbo(:, 5,28,13) = (/ &
        & 0.10534e+04_rb,0.78866e+03_rb,0.59808e+03_rb,0.88245e+03_rb,0.11673e+04_rb /)
      kbo(:, 1,29,13) = (/ &
        & 0.10524e+04_rb,0.78934e+03_rb,0.60181e+03_rb,0.88786e+03_rb,0.11745e+04_rb /)
      kbo(:, 2,29,13) = (/ &
        & 0.10481e+04_rb,0.78588e+03_rb,0.60321e+03_rb,0.88698e+03_rb,0.11734e+04_rb /)
      kbo(:, 3,29,13) = (/ &
        & 0.10466e+04_rb,0.79266e+03_rb,0.60021e+03_rb,0.88562e+03_rb,0.11715e+04_rb /)
      kbo(:, 4,29,13) = (/ &
        & 0.10466e+04_rb,0.79965e+03_rb,0.59906e+03_rb,0.88395e+03_rb,0.11693e+04_rb /)
      kbo(:, 5,29,13) = (/ &
        & 0.10459e+04_rb,0.78993e+03_rb,0.59799e+03_rb,0.88185e+03_rb,0.11668e+04_rb /)
      kbo(:, 1,30,13) = (/ &
        & 0.10360e+04_rb,0.78916e+03_rb,0.60191e+03_rb,0.88770e+03_rb,0.11743e+04_rb /)
      kbo(:, 2,30,13) = (/ &
        & 0.10409e+04_rb,0.78411e+03_rb,0.60093e+03_rb,0.88667e+03_rb,0.11729e+04_rb /)
      kbo(:, 3,30,13) = (/ &
        & 0.10391e+04_rb,0.79193e+03_rb,0.59993e+03_rb,0.88546e+03_rb,0.11710e+04_rb /)
      kbo(:, 4,30,13) = (/ &
        & 0.10409e+04_rb,0.78876e+03_rb,0.59874e+03_rb,0.88348e+03_rb,0.11687e+04_rb /)
      kbo(:, 5,30,13) = (/ &
        & 0.10396e+04_rb,0.78722e+03_rb,0.59722e+03_rb,0.88117e+03_rb,0.11665e+04_rb /)
      kbo(:, 1,31,13) = (/ &
        & 0.10394e+04_rb,0.79419e+03_rb,0.60167e+03_rb,0.88749e+03_rb,0.11740e+04_rb /)
      kbo(:, 2,31,13) = (/ &
        & 0.10363e+04_rb,0.79203e+03_rb,0.60068e+03_rb,0.88633e+03_rb,0.11725e+04_rb /)
      kbo(:, 3,31,13) = (/ &
        & 0.10437e+04_rb,0.78663e+03_rb,0.59963e+03_rb,0.88481e+03_rb,0.11705e+04_rb /)
      kbo(:, 4,31,13) = (/ &
        & 0.10406e+04_rb,0.79000e+03_rb,0.59839e+03_rb,0.88297e+03_rb,0.11680e+04_rb /)
      kbo(:, 5,31,13) = (/ &
        & 0.10413e+04_rb,0.79093e+03_rb,0.59674e+03_rb,0.88044e+03_rb,0.11650e+04_rb /)
      kbo(:, 1,32,13) = (/ &
        & 0.10440e+04_rb,0.79014e+03_rb,0.60327e+03_rb,0.88723e+03_rb,0.11737e+04_rb /)
      kbo(:, 2,32,13) = (/ &
        & 0.10391e+04_rb,0.79106e+03_rb,0.60042e+03_rb,0.88596e+03_rb,0.11720e+04_rb /)
      kbo(:, 3,32,13) = (/ &
        & 0.10467e+04_rb,0.78955e+03_rb,0.59932e+03_rb,0.88436e+03_rb,0.11699e+04_rb /)
      kbo(:, 4,32,13) = (/ &
        & 0.10407e+04_rb,0.79722e+03_rb,0.59818e+03_rb,0.88241e+03_rb,0.11673e+04_rb /)
      kbo(:, 5,32,13) = (/ &
        & 0.10382e+04_rb,0.78961e+03_rb,0.59626e+03_rb,0.87980e+03_rb,0.11639e+04_rb /)
      kbo(:, 1,33,13) = (/ &
        & 0.10522e+04_rb,0.78494e+03_rb,0.60081e+03_rb,0.88693e+03_rb,0.11733e+04_rb /)
      kbo(:, 2,33,13) = (/ &
        & 0.10373e+04_rb,0.79220e+03_rb,0.59999e+03_rb,0.88557e+03_rb,0.11715e+04_rb /)
      kbo(:, 3,33,13) = (/ &
        & 0.10359e+04_rb,0.78622e+03_rb,0.59900e+03_rb,0.88389e+03_rb,0.11695e+04_rb /)
      kbo(:, 4,33,13) = (/ &
        & 0.10377e+04_rb,0.78474e+03_rb,0.59760e+03_rb,0.88177e+03_rb,0.11667e+04_rb /)
      kbo(:, 5,33,13) = (/ &
        & 0.10406e+04_rb,0.78876e+03_rb,0.59580e+03_rb,0.87933e+03_rb,0.11633e+04_rb /)
      kbo(:, 1,34,13) = (/ &
        & 0.10360e+04_rb,0.79366e+03_rb,0.60074e+03_rb,0.88666e+03_rb,0.11729e+04_rb /)
      kbo(:, 2,34,13) = (/ &
        & 0.10445e+04_rb,0.78362e+03_rb,0.59974e+03_rb,0.88521e+03_rb,0.11710e+04_rb /)
      kbo(:, 3,34,13) = (/ &
        & 0.10445e+04_rb,0.78784e+03_rb,0.59871e+03_rb,0.88346e+03_rb,0.11687e+04_rb /)
      kbo(:, 4,34,13) = (/ &
        & 0.10420e+04_rb,0.78840e+03_rb,0.59718e+03_rb,0.88091e+03_rb,0.11665e+04_rb /)
      kbo(:, 5,34,13) = (/ &
        & 0.10384e+04_rb,0.79182e+03_rb,0.59540e+03_rb,0.87927e+03_rb,0.11620e+04_rb /)
      kbo(:, 1,35,13) = (/ &
        & 0.10422e+04_rb,0.79326e+03_rb,0.60077e+03_rb,0.88648e+03_rb,0.11727e+04_rb /)
      kbo(:, 2,35,13) = (/ &
        & 0.10360e+04_rb,0.79158e+03_rb,0.59975e+03_rb,0.88500e+03_rb,0.11707e+04_rb /)
      kbo(:, 3,35,13) = (/ &
        & 0.10370e+04_rb,0.78869e+03_rb,0.59854e+03_rb,0.88320e+03_rb,0.11683e+04_rb /)
      kbo(:, 4,35,13) = (/ &
        & 0.10364e+04_rb,0.79354e+03_rb,0.59694e+03_rb,0.88077e+03_rb,0.11651e+04_rb /)
      kbo(:, 5,35,13) = (/ &
        & 0.10369e+04_rb,0.78943e+03_rb,0.59473e+03_rb,0.87892e+03_rb,0.11616e+04_rb /)
      kbo(:, 1,36,13) = (/ &
        & 0.10401e+04_rb,0.78191e+03_rb,0.60075e+03_rb,0.88645e+03_rb,0.11727e+04_rb /)
      kbo(:, 2,36,13) = (/ &
        & 0.10371e+04_rb,0.78614e+03_rb,0.59972e+03_rb,0.88519e+03_rb,0.11707e+04_rb /)
      kbo(:, 3,36,13) = (/ &
        & 0.10392e+04_rb,0.78550e+03_rb,0.59835e+03_rb,0.88339e+03_rb,0.11683e+04_rb /)
      kbo(:, 4,36,13) = (/ &
        & 0.10378e+04_rb,0.78649e+03_rb,0.59690e+03_rb,0.88071e+03_rb,0.11650e+04_rb /)
      kbo(:, 5,36,13) = (/ &
        & 0.10389e+04_rb,0.79012e+03_rb,0.59516e+03_rb,0.87821e+03_rb,0.11604e+04_rb /)
      kbo(:, 1,37,13) = (/ &
        & 0.10397e+04_rb,0.78790e+03_rb,0.60071e+03_rb,0.88662e+03_rb,0.11729e+04_rb /)
      kbo(:, 2,37,13) = (/ &
        & 0.10413e+04_rb,0.78135e+03_rb,0.59986e+03_rb,0.88540e+03_rb,0.11710e+04_rb /)
      kbo(:, 3,37,13) = (/ &
        & 0.10434e+04_rb,0.78885e+03_rb,0.59867e+03_rb,0.88341e+03_rb,0.11686e+04_rb /)
      kbo(:, 4,37,13) = (/ &
        & 0.10382e+04_rb,0.79636e+03_rb,0.59698e+03_rb,0.88108e+03_rb,0.11655e+04_rb /)
      kbo(:, 5,37,13) = (/ &
        & 0.10369e+04_rb,0.78656e+03_rb,0.59536e+03_rb,0.87879e+03_rb,0.11626e+04_rb /)
      kbo(:, 1,38,13) = (/ &
        & 0.10433e+04_rb,0.78475e+03_rb,0.60099e+03_rb,0.88680e+03_rb,0.11731e+04_rb /)
      kbo(:, 2,38,13) = (/ &
        & 0.10422e+04_rb,0.78187e+03_rb,0.60001e+03_rb,0.88539e+03_rb,0.11713e+04_rb /)
      kbo(:, 3,38,13) = (/ &
        & 0.10376e+04_rb,0.78470e+03_rb,0.59854e+03_rb,0.88368e+03_rb,0.11690e+04_rb /)
      kbo(:, 4,38,13) = (/ &
        & 0.10379e+04_rb,0.78467e+03_rb,0.59739e+03_rb,0.88147e+03_rb,0.11660e+04_rb /)
      kbo(:, 5,38,13) = (/ &
        & 0.10346e+04_rb,0.79279e+03_rb,0.59559e+03_rb,0.87849e+03_rb,0.11631e+04_rb /)
      kbo(:, 1,39,13) = (/ &
        & 0.10564e+04_rb,0.79016e+03_rb,0.60113e+03_rb,0.88696e+03_rb,0.11734e+04_rb /)
      kbo(:, 2,39,13) = (/ &
        & 0.10405e+04_rb,0.78139e+03_rb,0.59938e+03_rb,0.88561e+03_rb,0.11715e+04_rb /)
      kbo(:, 3,39,13) = (/ &
        & 0.10386e+04_rb,0.79256e+03_rb,0.59902e+03_rb,0.88393e+03_rb,0.11693e+04_rb /)
      kbo(:, 4,39,13) = (/ &
        & 0.10351e+04_rb,0.78501e+03_rb,0.59779e+03_rb,0.88184e+03_rb,0.11668e+04_rb /)
      kbo(:, 5,39,13) = (/ &
        & 0.10353e+04_rb,0.78561e+03_rb,0.59583e+03_rb,0.87937e+03_rb,0.11634e+04_rb /)
      kbo(:, 1,40,13) = (/ &
        & 0.10351e+04_rb,0.78501e+03_rb,0.60325e+03_rb,0.88723e+03_rb,0.11737e+04_rb /)
      kbo(:, 2,40,13) = (/ &
        & 0.10332e+04_rb,0.78519e+03_rb,0.60040e+03_rb,0.88596e+03_rb,0.11720e+04_rb /)
      kbo(:, 3,40,13) = (/ &
        & 0.10393e+04_rb,0.78843e+03_rb,0.59931e+03_rb,0.88436e+03_rb,0.11699e+04_rb /)
      kbo(:, 4,40,13) = (/ &
        & 0.10339e+04_rb,0.78821e+03_rb,0.59801e+03_rb,0.88242e+03_rb,0.11676e+04_rb /)
      kbo(:, 5,40,13) = (/ &
        & 0.10476e+04_rb,0.78616e+03_rb,0.59626e+03_rb,0.87958e+03_rb,0.11639e+04_rb /)
      kbo(:, 1,41,13) = (/ &
        & 0.10420e+04_rb,0.78658e+03_rb,0.60226e+03_rb,0.88748e+03_rb,0.11740e+04_rb /)
      kbo(:, 2,41,13) = (/ &
        & 0.10471e+04_rb,0.78433e+03_rb,0.60065e+03_rb,0.88632e+03_rb,0.11725e+04_rb /)
      kbo(:, 3,41,13) = (/ &
        & 0.10385e+04_rb,0.78777e+03_rb,0.59960e+03_rb,0.88480e+03_rb,0.11705e+04_rb /)
      kbo(:, 4,41,13) = (/ &
        & 0.10374e+04_rb,0.79305e+03_rb,0.59837e+03_rb,0.88319e+03_rb,0.11680e+04_rb /)
      kbo(:, 5,41,13) = (/ &
        & 0.10341e+04_rb,0.79121e+03_rb,0.59671e+03_rb,0.88067e+03_rb,0.11641e+04_rb /)
      kbo(:, 1,42,13) = (/ &
        & 0.10427e+04_rb,0.78654e+03_rb,0.60202e+03_rb,0.88768e+03_rb,0.11743e+04_rb /)
      kbo(:, 2,42,13) = (/ &
        & 0.10376e+04_rb,0.78774e+03_rb,0.60073e+03_rb,0.88666e+03_rb,0.11730e+04_rb /)
      kbo(:, 3,42,13) = (/ &
        & 0.10387e+04_rb,0.78631e+03_rb,0.59989e+03_rb,0.88522e+03_rb,0.11710e+04_rb /)
      kbo(:, 4,42,13) = (/ &
        & 0.10432e+04_rb,0.79240e+03_rb,0.59870e+03_rb,0.88347e+03_rb,0.11690e+04_rb /)
      kbo(:, 5,42,13) = (/ &
        & 0.10408e+04_rb,0.78283e+03_rb,0.59718e+03_rb,0.88116e+03_rb,0.11677e+04_rb /)
      kbo(:, 1,43,13) = (/ &
        & 0.10384e+04_rb,0.78732e+03_rb,0.60211e+03_rb,0.88787e+03_rb,0.11746e+04_rb /)
      kbo(:, 2,43,13) = (/ &
        & 0.10392e+04_rb,0.79483e+03_rb,0.60103e+03_rb,0.88703e+03_rb,0.11735e+04_rb /)
      kbo(:, 3,43,13) = (/ &
        & 0.10435e+04_rb,0.79279e+03_rb,0.60022e+03_rb,0.88570e+03_rb,0.11717e+04_rb /)
      kbo(:, 4,43,13) = (/ &
        & 0.10453e+04_rb,0.78261e+03_rb,0.59909e+03_rb,0.88405e+03_rb,0.11698e+04_rb /)
      kbo(:, 5,43,13) = (/ &
        & 0.10349e+04_rb,0.78239e+03_rb,0.59773e+03_rb,0.88200e+03_rb,0.11670e+04_rb /)
      kbo(:, 1,44,13) = (/ &
        & 0.10376e+04_rb,0.78973e+03_rb,0.60218e+03_rb,0.88797e+03_rb,0.11747e+04_rb /)
      kbo(:, 2,44,13) = (/ &
        & 0.10357e+04_rb,0.79073e+03_rb,0.60154e+03_rb,0.88738e+03_rb,0.11742e+04_rb /)
      kbo(:, 3,44,13) = (/ &
        & 0.10381e+04_rb,0.78584e+03_rb,0.60056e+03_rb,0.88619e+03_rb,0.11723e+04_rb /)
      kbo(:, 4,44,13) = (/ &
        & 0.10410e+04_rb,0.78726e+03_rb,0.59965e+03_rb,0.88464e+03_rb,0.11703e+04_rb /)
      kbo(:, 5,44,13) = (/ &
        & 0.10370e+04_rb,0.78461e+03_rb,0.59824e+03_rb,0.88299e+03_rb,0.11677e+04_rb /)
      kbo(:, 1,45,13) = (/ &
        & 0.10383e+04_rb,0.78177e+03_rb,0.60254e+03_rb,0.88799e+03_rb,0.11748e+04_rb /)
      kbo(:, 2,45,13) = (/ &
        & 0.10337e+04_rb,0.78400e+03_rb,0.60200e+03_rb,0.88790e+03_rb,0.11743e+04_rb /)
      kbo(:, 3,45,13) = (/ &
        & 0.10407e+04_rb,0.78928e+03_rb,0.60087e+03_rb,0.88664e+03_rb,0.11732e+04_rb /)
      kbo(:, 4,45,13) = (/ &
        & 0.10410e+04_rb,0.79578e+03_rb,0.59987e+03_rb,0.88519e+03_rb,0.11713e+04_rb /)
      kbo(:, 5,45,13) = (/ &
        & 0.10369e+04_rb,0.78337e+03_rb,0.59884e+03_rb,0.88344e+03_rb,0.11687e+04_rb /)
      kbo(:, 1,46,13) = (/ &
        & 0.10415e+04_rb,0.79090e+03_rb,0.60272e+03_rb,0.88805e+03_rb,0.11749e+04_rb /)
      kbo(:, 2,46,13) = (/ &
        & 0.10332e+04_rb,0.79374e+03_rb,0.60213e+03_rb,0.88788e+03_rb,0.11746e+04_rb /)
      kbo(:, 3,46,13) = (/ &
        & 0.10495e+04_rb,0.78743e+03_rb,0.60122e+03_rb,0.88707e+03_rb,0.11738e+04_rb /)
      kbo(:, 4,46,13) = (/ &
        & 0.10359e+04_rb,0.78275e+03_rb,0.60025e+03_rb,0.88575e+03_rb,0.11717e+04_rb /)
      kbo(:, 5,46,13) = (/ &
        & 0.10407e+04_rb,0.79210e+03_rb,0.59913e+03_rb,0.88411e+03_rb,0.11695e+04_rb /)
      kbo(:, 1,47,13) = (/ &
        & 0.10396e+04_rb,0.78478e+03_rb,0.60291e+03_rb,0.88833e+03_rb,0.11753e+04_rb /)
      kbo(:, 2,47,13) = (/ &
        & 0.10391e+04_rb,0.78254e+03_rb,0.60239e+03_rb,0.88798e+03_rb,0.11747e+04_rb /)
      kbo(:, 3,47,13) = (/ &
        & 0.10358e+04_rb,0.79087e+03_rb,0.60194e+03_rb,0.88748e+03_rb,0.11740e+04_rb /)
      kbo(:, 4,47,13) = (/ &
        & 0.10382e+04_rb,0.78212e+03_rb,0.60065e+03_rb,0.88632e+03_rb,0.11725e+04_rb /)
      kbo(:, 5,47,13) = (/ &
        & 0.10417e+04_rb,0.78639e+03_rb,0.59976e+03_rb,0.88480e+03_rb,0.11705e+04_rb /)
      kbo(:, 1,48,13) = (/ &
        & 0.10401e+04_rb,0.78456e+03_rb,0.60307e+03_rb,0.88865e+03_rb,0.11757e+04_rb /)
      kbo(:, 2,48,13) = (/ &
        & 0.10385e+04_rb,0.78639e+03_rb,0.60262e+03_rb,0.88799e+03_rb,0.11748e+04_rb /)
      kbo(:, 3,48,13) = (/ &
        & 0.10377e+04_rb,0.78374e+03_rb,0.60183e+03_rb,0.88778e+03_rb,0.11745e+04_rb /)
      kbo(:, 4,48,13) = (/ &
        & 0.10347e+04_rb,0.78455e+03_rb,0.60102e+03_rb,0.88707e+03_rb,0.11732e+04_rb /)
      kbo(:, 5,48,13) = (/ &
        & 0.10420e+04_rb,0.79085e+03_rb,0.60005e+03_rb,0.88545e+03_rb,0.11713e+04_rb /)
      kbo(:, 1,49,13) = (/ &
        & 0.10372e+04_rb,0.78740e+03_rb,0.60317e+03_rb,0.88862e+03_rb,0.11760e+04_rb /)
      kbo(:, 2,49,13) = (/ &
        & 0.10417e+04_rb,0.78862e+03_rb,0.60282e+03_rb,0.88818e+03_rb,0.11751e+04_rb /)
      kbo(:, 3,49,13) = (/ &
        & 0.10345e+04_rb,0.78780e+03_rb,0.60227e+03_rb,0.88818e+03_rb,0.11747e+04_rb /)
      kbo(:, 4,49,13) = (/ &
        & 0.10406e+04_rb,0.78617e+03_rb,0.60221e+03_rb,0.88729e+03_rb,0.11738e+04_rb /)
      kbo(:, 5,49,13) = (/ &
        & 0.10382e+04_rb,0.79023e+03_rb,0.60061e+03_rb,0.88605e+03_rb,0.11721e+04_rb /)
      kbo(:, 1,50,13) = (/ &
        & 0.10347e+04_rb,0.79207e+03_rb,0.60367e+03_rb,0.88900e+03_rb,0.11761e+04_rb /)
      kbo(:, 2,50,13) = (/ &
        & 0.10350e+04_rb,0.78739e+03_rb,0.60298e+03_rb,0.88872e+03_rb,0.11755e+04_rb /)
      kbo(:, 3,50,13) = (/ &
        & 0.10353e+04_rb,0.78433e+03_rb,0.60234e+03_rb,0.88799e+03_rb,0.11744e+04_rb /)
      kbo(:, 4,50,13) = (/ &
        & 0.10489e+04_rb,0.79267e+03_rb,0.60164e+03_rb,0.88763e+03_rb,0.11743e+04_rb /)
      kbo(:, 5,50,13) = (/ &
        & 0.10406e+04_rb,0.78514e+03_rb,0.60019e+03_rb,0.88656e+03_rb,0.11728e+04_rb /)
      kbo(:, 1,51,13) = (/ &
        & 0.10345e+04_rb,0.78799e+03_rb,0.60398e+03_rb,0.88932e+03_rb,0.11765e+04_rb /)
      kbo(:, 2,51,13) = (/ &
        & 0.10401e+04_rb,0.78697e+03_rb,0.60314e+03_rb,0.88874e+03_rb,0.11758e+04_rb /)
      kbo(:, 3,51,13) = (/ &
        & 0.10366e+04_rb,0.78656e+03_rb,0.60285e+03_rb,0.88803e+03_rb,0.11749e+04_rb /)
      kbo(:, 4,51,13) = (/ &
        & 0.10374e+04_rb,0.78473e+03_rb,0.60240e+03_rb,0.88809e+03_rb,0.11746e+04_rb /)
      kbo(:, 5,51,13) = (/ &
        & 0.10390e+04_rb,0.78920e+03_rb,0.60116e+03_rb,0.88701e+03_rb,0.11734e+04_rb /)
      kbo(:, 1,52,13) = (/ &
        & 0.10369e+04_rb,0.78758e+03_rb,0.60425e+03_rb,0.88974e+03_rb,0.11770e+04_rb /)
      kbo(:, 2,52,13) = (/ &
        & 0.10369e+04_rb,0.78229e+03_rb,0.60357e+03_rb,0.88889e+03_rb,0.11760e+04_rb /)
      kbo(:, 3,52,13) = (/ &
        & 0.10371e+04_rb,0.78737e+03_rb,0.60287e+03_rb,0.88826e+03_rb,0.11752e+04_rb /)
      kbo(:, 4,52,13) = (/ &
        & 0.10414e+04_rb,0.78574e+03_rb,0.60234e+03_rb,0.88797e+03_rb,0.11747e+04_rb /)
      kbo(:, 5,52,13) = (/ &
        & 0.10377e+04_rb,0.78696e+03_rb,0.60170e+03_rb,0.88739e+03_rb,0.11739e+04_rb /)
      kbo(:, 1,53,13) = (/ &
        & 0.10446e+04_rb,0.78533e+03_rb,0.60451e+03_rb,0.89022e+03_rb,0.11778e+04_rb /)
      kbo(:, 2,53,13) = (/ &
        & 0.10366e+04_rb,0.78675e+03_rb,0.60391e+03_rb,0.88907e+03_rb,0.11762e+04_rb /)
      kbo(:, 3,53,13) = (/ &
        & 0.10339e+04_rb,0.78472e+03_rb,0.60302e+03_rb,0.88857e+03_rb,0.11756e+04_rb /)
      kbo(:, 4,53,13) = (/ &
        & 0.10347e+04_rb,0.78858e+03_rb,0.60271e+03_rb,0.88799e+03_rb,0.11748e+04_rb /)
      kbo(:, 5,53,13) = (/ &
        & 0.10414e+04_rb,0.78370e+03_rb,0.60188e+03_rb,0.88770e+03_rb,0.11743e+04_rb /)
      kbo(:, 1,54,13) = (/ &
        & 0.10366e+04_rb,0.78980e+03_rb,0.60463e+03_rb,0.89072e+03_rb,0.11788e+04_rb /)
      kbo(:, 2,54,13) = (/ &
        & 0.10376e+04_rb,0.78289e+03_rb,0.60402e+03_rb,0.88939e+03_rb,0.11765e+04_rb /)
      kbo(:, 3,54,13) = (/ &
        & 0.10387e+04_rb,0.78695e+03_rb,0.60318e+03_rb,0.88877e+03_rb,0.11759e+04_rb /)
      kbo(:, 4,54,13) = (/ &
        & 0.10435e+04_rb,0.78370e+03_rb,0.60273e+03_rb,0.88806e+03_rb,0.11749e+04_rb /)
      kbo(:, 5,54,13) = (/ &
        & 0.10492e+04_rb,0.79000e+03_rb,0.60214e+03_rb,0.88789e+03_rb,0.11746e+04_rb /)
      kbo(:, 1,55,13) = (/ &
        & 0.10395e+04_rb,0.78654e+03_rb,0.60524e+03_rb,0.89136e+03_rb,0.11799e+04_rb /)
      kbo(:, 2,55,13) = (/ &
        & 0.10379e+04_rb,0.78553e+03_rb,0.60426e+03_rb,0.88977e+03_rb,0.11770e+04_rb /)
      kbo(:, 3,55,13) = (/ &
        & 0.10376e+04_rb,0.78959e+03_rb,0.60343e+03_rb,0.88890e+03_rb,0.11757e+04_rb /)
      kbo(:, 4,55,13) = (/ &
        & 0.10360e+04_rb,0.78634e+03_rb,0.60272e+03_rb,0.88828e+03_rb,0.11752e+04_rb /)
      kbo(:, 5,55,13) = (/ &
        & 0.10355e+04_rb,0.78796e+03_rb,0.60219e+03_rb,0.88821e+03_rb,0.11747e+04_rb /)
      kbo(:, 1,56,13) = (/ &
        & 0.10500e+04_rb,0.78471e+03_rb,0.60540e+03_rb,0.89210e+03_rb,0.11812e+04_rb /)
      kbo(:, 2,56,13) = (/ &
        & 0.10441e+04_rb,0.78735e+03_rb,0.60449e+03_rb,0.89019e+03_rb,0.11777e+04_rb /)
      kbo(:, 3,56,13) = (/ &
        & 0.10382e+04_rb,0.78755e+03_rb,0.60374e+03_rb,0.88905e+03_rb,0.11759e+04_rb /)
      kbo(:, 4,56,13) = (/ &
        & 0.10470e+04_rb,0.78715e+03_rb,0.60301e+03_rb,0.88855e+03_rb,0.11756e+04_rb /)
      kbo(:, 5,56,13) = (/ &
        & 0.10358e+04_rb,0.78674e+03_rb,0.60270e+03_rb,0.88799e+03_rb,0.11748e+04_rb /)
      kbo(:, 1,57,13) = (/ &
        & 0.10339e+04_rb,0.78715e+03_rb,0.60588e+03_rb,0.89295e+03_rb,0.11828e+04_rb /)
      kbo(:, 2,57,13) = (/ &
        & 0.10441e+04_rb,0.78267e+03_rb,0.60475e+03_rb,0.89065e+03_rb,0.11787e+04_rb /)
      kbo(:, 3,57,13) = (/ &
        & 0.10435e+04_rb,0.78938e+03_rb,0.60399e+03_rb,0.88934e+03_rb,0.11765e+04_rb /)
      kbo(:, 4,57,13) = (/ &
        & 0.10392e+04_rb,0.79203e+03_rb,0.60300e+03_rb,0.88875e+03_rb,0.11758e+04_rb /)
      kbo(:, 5,57,13) = (/ &
        & 0.10360e+04_rb,0.78491e+03_rb,0.60271e+03_rb,0.88827e+03_rb,0.11749e+04_rb /)
      kbo(:, 1,58,13) = (/ &
        & 0.10392e+04_rb,0.78430e+03_rb,0.60601e+03_rb,0.89398e+03_rb,0.11847e+04_rb /)
      kbo(:, 2,58,13) = (/ &
        & 0.10363e+04_rb,0.78694e+03_rb,0.60503e+03_rb,0.89124e+03_rb,0.11797e+04_rb /)
      kbo(:, 3,58,13) = (/ &
        & 0.10360e+04_rb,0.78084e+03_rb,0.60391e+03_rb,0.88970e+03_rb,0.11769e+04_rb /)
      kbo(:, 4,58,13) = (/ &
        & 0.10459e+04_rb,0.78267e+03_rb,0.60338e+03_rb,0.88888e+03_rb,0.11757e+04_rb /)
      kbo(:, 5,58,13) = (/ &
        & 0.10368e+04_rb,0.79101e+03_rb,0.60285e+03_rb,0.88823e+03_rb,0.11752e+04_rb /)
      kbo(:, 1,59,13) = (/ &
        & 0.10403e+04_rb,0.78613e+03_rb,0.60612e+03_rb,0.89444e+03_rb,0.11849e+04_rb /)
      kbo(:, 2,59,13) = (/ &
        & 0.10349e+04_rb,0.78531e+03_rb,0.60514e+03_rb,0.89150e+03_rb,0.11802e+04_rb /)
      kbo(:, 3,59,13) = (/ &
        & 0.10360e+04_rb,0.78470e+03_rb,0.60447e+03_rb,0.88985e+03_rb,0.11771e+04_rb /)
      kbo(:, 4,59,13) = (/ &
        & 0.10411e+04_rb,0.78531e+03_rb,0.60349e+03_rb,0.88892e+03_rb,0.11761e+04_rb /)
      kbo(:, 5,59,13) = (/ &
        & 0.10355e+04_rb,0.78795e+03_rb,0.60275e+03_rb,0.88833e+03_rb,0.11753e+04_rb /)
      kbo(:, 1,13,14) = (/ &
        & 0.12194e+04_rb,0.91192e+03_rb,0.60649e+03_rb,0.89289e+03_rb,0.11803e+04_rb /)
      kbo(:, 2,13,14) = (/ &
        & 0.11340e+04_rb,0.93388e+03_rb,0.60565e+03_rb,0.89163e+03_rb,0.11786e+04_rb /)
      kbo(:, 3,13,14) = (/ &
        & 0.11287e+04_rb,0.93154e+03_rb,0.60510e+03_rb,0.89077e+03_rb,0.11775e+04_rb /)
      kbo(:, 4,13,14) = (/ &
        & 0.11571e+04_rb,0.89198e+03_rb,0.60383e+03_rb,0.88977e+03_rb,0.11762e+04_rb /)
      kbo(:, 5,13,14) = (/ &
        & 0.11232e+04_rb,0.85932e+03_rb,0.60280e+03_rb,0.88865e+03_rb,0.11748e+04_rb /)
      kbo(:, 1,14,14) = (/ &
        & 0.11313e+04_rb,0.85574e+03_rb,0.60604e+03_rb,0.89259e+03_rb,0.11800e+04_rb /)
      kbo(:, 2,14,14) = (/ &
        & 0.11392e+04_rb,0.85278e+03_rb,0.60523e+03_rb,0.89143e+03_rb,0.11785e+04_rb /)
      kbo(:, 3,14,14) = (/ &
        & 0.11402e+04_rb,0.86846e+03_rb,0.60420e+03_rb,0.89055e+03_rb,0.11774e+04_rb /)
      kbo(:, 4,14,14) = (/ &
        & 0.11058e+04_rb,0.85128e+03_rb,0.60339e+03_rb,0.88920e+03_rb,0.11761e+04_rb /)
      kbo(:, 5,14,14) = (/ &
        & 0.11286e+04_rb,0.84325e+03_rb,0.60241e+03_rb,0.88842e+03_rb,0.11746e+04_rb /)
      kbo(:, 1,15,14) = (/ &
        & 0.11349e+04_rb,0.84417e+03_rb,0.60541e+03_rb,0.89231e+03_rb,0.11798e+04_rb /)
      kbo(:, 2,15,14) = (/ &
        & 0.11015e+04_rb,0.88045e+03_rb,0.60487e+03_rb,0.89125e+03_rb,0.11784e+04_rb /)
      kbo(:, 3,15,14) = (/ &
        & 0.11261e+04_rb,0.83776e+03_rb,0.60408e+03_rb,0.89034e+03_rb,0.11772e+04_rb /)
      kbo(:, 4,15,14) = (/ &
        & 0.10962e+04_rb,0.86597e+03_rb,0.60302e+03_rb,0.88937e+03_rb,0.11760e+04_rb /)
      kbo(:, 5,15,14) = (/ &
        & 0.10995e+04_rb,0.86095e+03_rb,0.60207e+03_rb,0.88855e+03_rb,0.11740e+04_rb /)
      kbo(:, 1,16,14) = (/ &
        & 0.11556e+04_rb,0.89943e+03_rb,0.60533e+03_rb,0.89241e+03_rb,0.11796e+04_rb /)
      kbo(:, 2,16,14) = (/ &
        & 0.10928e+04_rb,0.87845e+03_rb,0.60456e+03_rb,0.89108e+03_rb,0.11783e+04_rb /)
      kbo(:, 3,16,14) = (/ &
        & 0.11526e+04_rb,0.84568e+03_rb,0.60375e+03_rb,0.89014e+03_rb,0.11771e+04_rb /)
      kbo(:, 4,16,14) = (/ &
        & 0.11224e+04_rb,0.82958e+03_rb,0.60245e+03_rb,0.88919e+03_rb,0.11758e+04_rb /)
      kbo(:, 5,16,14) = (/ &
        & 0.11104e+04_rb,0.83492e+03_rb,0.60177e+03_rb,0.88796e+03_rb,0.11747e+04_rb /)
      kbo(:, 1,17,14) = (/ &
        & 0.11093e+04_rb,0.86391e+03_rb,0.60505e+03_rb,0.89183e+03_rb,0.11794e+04_rb /)
      kbo(:, 2,17,14) = (/ &
        & 0.10879e+04_rb,0.83992e+03_rb,0.60455e+03_rb,0.89093e+03_rb,0.11782e+04_rb /)
      kbo(:, 3,17,14) = (/ &
        & 0.11180e+04_rb,0.83675e+03_rb,0.60345e+03_rb,0.88997e+03_rb,0.11769e+04_rb /)
      kbo(:, 4,17,14) = (/ &
        & 0.11203e+04_rb,0.81299e+03_rb,0.60195e+03_rb,0.88902e+03_rb,0.11757e+04_rb /)
      kbo(:, 5,17,14) = (/ &
        & 0.10902e+04_rb,0.84755e+03_rb,0.60151e+03_rb,0.88739e+03_rb,0.11736e+04_rb /)
      kbo(:, 1,18,14) = (/ &
        & 0.10889e+04_rb,0.82171e+03_rb,0.60482e+03_rb,0.89199e+03_rb,0.11792e+04_rb /)
      kbo(:, 2,18,14) = (/ &
        & 0.10836e+04_rb,0.85518e+03_rb,0.60408e+03_rb,0.89079e+03_rb,0.11781e+04_rb /)
      kbo(:, 3,18,14) = (/ &
        & 0.10919e+04_rb,0.82437e+03_rb,0.60318e+03_rb,0.88981e+03_rb,0.11763e+04_rb /)
      kbo(:, 4,18,14) = (/ &
        & 0.11143e+04_rb,0.86811e+03_rb,0.60223e+03_rb,0.88886e+03_rb,0.11755e+04_rb /)
      kbo(:, 5,18,14) = (/ &
        & 0.10618e+04_rb,0.80694e+03_rb,0.60104e+03_rb,0.88718e+03_rb,0.11738e+04_rb /)
      kbo(:, 1,19,14) = (/ &
        & 0.10700e+04_rb,0.81179e+03_rb,0.60437e+03_rb,0.89145e+03_rb,0.11790e+04_rb /)
      kbo(:, 2,19,14) = (/ &
        & 0.11096e+04_rb,0.84185e+03_rb,0.60389e+03_rb,0.89066e+03_rb,0.11785e+04_rb /)
      kbo(:, 3,19,14) = (/ &
        & 0.10919e+04_rb,0.82575e+03_rb,0.60295e+03_rb,0.89004e+03_rb,0.11767e+04_rb /)
      kbo(:, 4,19,14) = (/ &
        & 0.10874e+04_rb,0.81974e+03_rb,0.60205e+03_rb,0.88835e+03_rb,0.11754e+04_rb /)
      kbo(:, 5,19,14) = (/ &
        & 0.11077e+04_rb,0.86723e+03_rb,0.60084e+03_rb,0.88734e+03_rb,0.11727e+04_rb /)
      kbo(:, 1,20,14) = (/ &
        & 0.10813e+04_rb,0.82037e+03_rb,0.60443e+03_rb,0.89128e+03_rb,0.11789e+04_rb /)
      kbo(:, 2,20,14) = (/ &
        & 0.10944e+04_rb,0.83718e+03_rb,0.60369e+03_rb,0.89052e+03_rb,0.11778e+04_rb /)
      kbo(:, 3,20,14) = (/ &
        & 0.10922e+04_rb,0.81527e+03_rb,0.60248e+03_rb,0.88953e+03_rb,0.11765e+04_rb /)
      kbo(:, 4,20,14) = (/ &
        & 0.10962e+04_rb,0.82815e+03_rb,0.60188e+03_rb,0.88854e+03_rb,0.11752e+04_rb /)
      kbo(:, 5,20,14) = (/ &
        & 0.10768e+04_rb,0.81150e+03_rb,0.60064e+03_rb,0.88711e+03_rb,0.11734e+04_rb /)
      kbo(:, 1,21,14) = (/ &
        & 0.10919e+04_rb,0.79941e+03_rb,0.60426e+03_rb,0.89114e+03_rb,0.11787e+04_rb /)
      kbo(:, 2,21,14) = (/ &
        & 0.10481e+04_rb,0.83111e+03_rb,0.60351e+03_rb,0.89037e+03_rb,0.11777e+04_rb /)
      kbo(:, 3,21,14) = (/ &
        & 0.10716e+04_rb,0.81475e+03_rb,0.60253e+03_rb,0.88975e+03_rb,0.11764e+04_rb /)
      kbo(:, 4,21,14) = (/ &
        & 0.11197e+04_rb,0.80344e+03_rb,0.60123e+03_rb,0.88837e+03_rb,0.11750e+04_rb /)
      kbo(:, 5,21,14) = (/ &
        & 0.10436e+04_rb,0.80977e+03_rb,0.60045e+03_rb,0.88688e+03_rb,0.11731e+04_rb /)
      kbo(:, 1,22,14) = (/ &
        & 0.11028e+04_rb,0.82019e+03_rb,0.60406e+03_rb,0.89099e+03_rb,0.11785e+04_rb /)
      kbo(:, 2,22,14) = (/ &
        & 0.10664e+04_rb,0.84127e+03_rb,0.60327e+03_rb,0.89089e+03_rb,0.11775e+04_rb /)
      kbo(:, 3,22,14) = (/ &
        & 0.10764e+04_rb,0.83101e+03_rb,0.60182e+03_rb,0.88920e+03_rb,0.11762e+04_rb /)
      kbo(:, 4,22,14) = (/ &
        & 0.10955e+04_rb,0.82928e+03_rb,0.60175e+03_rb,0.88811e+03_rb,0.11747e+04_rb /)
      kbo(:, 5,22,14) = (/ &
        & 0.10429e+04_rb,0.82150e+03_rb,0.60042e+03_rb,0.88653e+03_rb,0.11726e+04_rb /)
      kbo(:, 1,23,14) = (/ &
        & 0.10449e+04_rb,0.79677e+03_rb,0.60363e+03_rb,0.89084e+03_rb,0.11784e+04_rb /)
      kbo(:, 2,23,14) = (/ &
        & 0.10443e+04_rb,0.81477e+03_rb,0.60352e+03_rb,0.88994e+03_rb,0.11772e+04_rb /)
      kbo(:, 3,23,14) = (/ &
        & 0.10490e+04_rb,0.84026e+03_rb,0.60112e+03_rb,0.88900e+03_rb,0.11759e+04_rb /)
      kbo(:, 4,23,14) = (/ &
        & 0.10652e+04_rb,0.80737e+03_rb,0.60128e+03_rb,0.88783e+03_rb,0.11744e+04_rb /)
      kbo(:, 5,23,14) = (/ &
        & 0.10777e+04_rb,0.80975e+03_rb,0.60014e+03_rb,0.88615e+03_rb,0.11722e+04_rb /)
      kbo(:, 1,24,14) = (/ &
        & 0.10587e+04_rb,0.83370e+03_rb,0.60369e+03_rb,0.89067e+03_rb,0.11782e+04_rb /)
      kbo(:, 2,24,14) = (/ &
        & 0.10413e+04_rb,0.79049e+03_rb,0.60278e+03_rb,0.88972e+03_rb,0.11769e+04_rb /)
      kbo(:, 3,24,14) = (/ &
        & 0.10508e+04_rb,0.83928e+03_rb,0.60192e+03_rb,0.88879e+03_rb,0.11757e+04_rb /)
      kbo(:, 4,24,14) = (/ &
        & 0.10667e+04_rb,0.80304e+03_rb,0.60105e+03_rb,0.88752e+03_rb,0.11740e+04_rb /)
      kbo(:, 5,24,14) = (/ &
        & 0.10776e+04_rb,0.78685e+03_rb,0.59985e+03_rb,0.88574e+03_rb,0.11716e+04_rb /)
      kbo(:, 1,25,14) = (/ &
        & 0.10559e+04_rb,0.78648e+03_rb,0.60348e+03_rb,0.89048e+03_rb,0.11779e+04_rb /)
      kbo(:, 2,25,14) = (/ &
        & 0.10600e+04_rb,0.81980e+03_rb,0.60253e+03_rb,0.88951e+03_rb,0.11766e+04_rb /)
      kbo(:, 3,25,14) = (/ &
        & 0.10484e+04_rb,0.79331e+03_rb,0.60149e+03_rb,0.88854e+03_rb,0.11754e+04_rb /)
      kbo(:, 4,25,14) = (/ &
        & 0.11000e+04_rb,0.80548e+03_rb,0.60080e+03_rb,0.88717e+03_rb,0.11735e+04_rb /)
      kbo(:, 5,25,14) = (/ &
        & 0.11165e+04_rb,0.79968e+03_rb,0.59952e+03_rb,0.88528e+03_rb,0.11711e+04_rb /)
      kbo(:, 1,26,14) = (/ &
        & 0.10604e+04_rb,0.79570e+03_rb,0.60325e+03_rb,0.89026e+03_rb,0.11777e+04_rb /)
      kbo(:, 2,26,14) = (/ &
        & 0.10870e+04_rb,0.78311e+03_rb,0.60230e+03_rb,0.88929e+03_rb,0.11764e+04_rb /)
      kbo(:, 3,26,14) = (/ &
        & 0.10476e+04_rb,0.79338e+03_rb,0.60128e+03_rb,0.88826e+03_rb,0.11750e+04_rb /)
      kbo(:, 4,26,14) = (/ &
        & 0.10459e+04_rb,0.80175e+03_rb,0.60027e+03_rb,0.88677e+03_rb,0.11730e+04_rb /)
      kbo(:, 5,26,14) = (/ &
        & 0.10485e+04_rb,0.81112e+03_rb,0.59892e+03_rb,0.88476e+03_rb,0.11704e+04_rb /)
      kbo(:, 1,27,14) = (/ &
        & 0.10458e+04_rb,0.79457e+03_rb,0.60301e+03_rb,0.89001e+03_rb,0.11773e+04_rb /)
      kbo(:, 2,27,14) = (/ &
        & 0.10576e+04_rb,0.79170e+03_rb,0.60208e+03_rb,0.88906e+03_rb,0.11761e+04_rb /)
      kbo(:, 3,27,14) = (/ &
        & 0.10632e+04_rb,0.79464e+03_rb,0.60130e+03_rb,0.88794e+03_rb,0.11741e+04_rb /)
      kbo(:, 4,27,14) = (/ &
        & 0.10506e+04_rb,0.80916e+03_rb,0.60021e+03_rb,0.88633e+03_rb,0.11725e+04_rb /)
      kbo(:, 5,27,14) = (/ &
        & 0.10406e+04_rb,0.79035e+03_rb,0.59878e+03_rb,0.88422e+03_rb,0.11697e+04_rb /)
      kbo(:, 1,28,14) = (/ &
        & 0.10390e+04_rb,0.79142e+03_rb,0.60274e+03_rb,0.88940e+03_rb,0.11770e+04_rb /)
      kbo(:, 2,28,14) = (/ &
        & 0.10793e+04_rb,0.84664e+03_rb,0.60165e+03_rb,0.88883e+03_rb,0.11758e+04_rb /)
      kbo(:, 3,28,14) = (/ &
        & 0.10389e+04_rb,0.81244e+03_rb,0.60106e+03_rb,0.88760e+03_rb,0.11741e+04_rb /)
      kbo(:, 4,28,14) = (/ &
        & 0.10910e+04_rb,0.82250e+03_rb,0.59988e+03_rb,0.88586e+03_rb,0.11718e+04_rb /)
      kbo(:, 5,28,14) = (/ &
        & 0.10984e+04_rb,0.79766e+03_rb,0.59863e+03_rb,0.88510e+03_rb,0.11689e+04_rb /)
      kbo(:, 1,29,14) = (/ &
        & 0.10453e+04_rb,0.79421e+03_rb,0.60249e+03_rb,0.88988e+03_rb,0.11767e+04_rb /)
      kbo(:, 2,29,14) = (/ &
        & 0.10759e+04_rb,0.85231e+03_rb,0.59780e+03_rb,0.88856e+03_rb,0.11754e+04_rb /)
      kbo(:, 3,29,14) = (/ &
        & 0.10355e+04_rb,0.83260e+03_rb,0.60079e+03_rb,0.88721e+03_rb,0.11736e+04_rb /)
      kbo(:, 4,29,14) = (/ &
        & 0.10260e+04_rb,0.78313e+03_rb,0.59953e+03_rb,0.88535e+03_rb,0.11712e+04_rb /)
      kbo(:, 5,29,14) = (/ &
        & 0.10495e+04_rb,0.79101e+03_rb,0.59751e+03_rb,0.88309e+03_rb,0.11682e+04_rb /)
      kbo(:, 1,30,14) = (/ &
        & 0.10479e+04_rb,0.78753e+03_rb,0.60225e+03_rb,0.88928e+03_rb,0.11764e+04_rb /)
      kbo(:, 2,30,14) = (/ &
        & 0.10451e+04_rb,0.80563e+03_rb,0.60150e+03_rb,0.88827e+03_rb,0.11750e+04_rb /)
      kbo(:, 3,30,14) = (/ &
        & 0.10758e+04_rb,0.79525e+03_rb,0.60050e+03_rb,0.88643e+03_rb,0.11731e+04_rb /)
      kbo(:, 4,30,14) = (/ &
        & 0.10781e+04_rb,0.79929e+03_rb,0.59916e+03_rb,0.88480e+03_rb,0.11705e+04_rb /)
      kbo(:, 5,30,14) = (/ &
        & 0.10459e+04_rb,0.81358e+03_rb,0.59790e+03_rb,0.88220e+03_rb,0.11665e+04_rb /)
      kbo(:, 1,31,14) = (/ &
        & 0.10607e+04_rb,0.80146e+03_rb,0.60204e+03_rb,0.88905e+03_rb,0.11761e+04_rb /)
      kbo(:, 2,31,14) = (/ &
        & 0.10395e+04_rb,0.78822e+03_rb,0.60127e+03_rb,0.88793e+03_rb,0.11746e+04_rb /)
      kbo(:, 3,31,14) = (/ &
        & 0.10321e+04_rb,0.80923e+03_rb,0.60018e+03_rb,0.88632e+03_rb,0.11725e+04_rb /)
      kbo(:, 4,31,14) = (/ &
        & 0.11042e+04_rb,0.80049e+03_rb,0.59875e+03_rb,0.88457e+03_rb,0.11697e+04_rb /)
      kbo(:, 5,31,14) = (/ &
        & 0.10581e+04_rb,0.80105e+03_rb,0.59750e+03_rb,0.88200e+03_rb,0.11672e+04_rb /)
      kbo(:, 1,32,14) = (/ &
        & 0.10889e+04_rb,0.84488e+03_rb,0.59867e+03_rb,0.88880e+03_rb,0.11757e+04_rb /)
      kbo(:, 2,32,14) = (/ &
        & 0.10428e+04_rb,0.79586e+03_rb,0.60101e+03_rb,0.88756e+03_rb,0.11741e+04_rb /)
      kbo(:, 3,32,14) = (/ &
        & 0.10671e+04_rb,0.82487e+03_rb,0.59984e+03_rb,0.88582e+03_rb,0.11718e+04_rb /)
      kbo(:, 4,32,14) = (/ &
        & 0.10426e+04_rb,0.83726e+03_rb,0.59809e+03_rb,0.88469e+03_rb,0.11694e+04_rb /)
      kbo(:, 5,32,14) = (/ &
        & 0.10522e+04_rb,0.80937e+03_rb,0.59684e+03_rb,0.88175e+03_rb,0.11659e+04_rb /)
      kbo(:, 1,33,14) = (/ &
        & 0.10391e+04_rb,0.79980e+03_rb,0.60166e+03_rb,0.88852e+03_rb,0.11754e+04_rb /)
      kbo(:, 2,33,14) = (/ &
        & 0.10719e+04_rb,0.78917e+03_rb,0.60073e+03_rb,0.88751e+03_rb,0.11736e+04_rb /)
      kbo(:, 3,33,14) = (/ &
        & 0.10389e+04_rb,0.82235e+03_rb,0.59947e+03_rb,0.88564e+03_rb,0.11706e+04_rb /)
      kbo(:, 4,33,14) = (/ &
        & 0.10819e+04_rb,0.81877e+03_rb,0.59794e+03_rb,0.88337e+03_rb,0.11676e+04_rb /)
      kbo(:, 5,33,14) = (/ &
        & 0.10266e+04_rb,0.78772e+03_rb,0.59642e+03_rb,0.88112e+03_rb,0.11651e+04_rb /)
      kbo(:, 1,34,14) = (/ &
        & 0.10675e+04_rb,0.78932e+03_rb,0.60147e+03_rb,0.88789e+03_rb,0.11750e+04_rb /)
      kbo(:, 2,34,14) = (/ &
        & 0.10375e+04_rb,0.79310e+03_rb,0.60047e+03_rb,0.88677e+03_rb,0.11731e+04_rb /)
      kbo(:, 3,34,14) = (/ &
        & 0.10378e+04_rb,0.82372e+03_rb,0.59913e+03_rb,0.88478e+03_rb,0.11704e+04_rb /)
      kbo(:, 4,34,14) = (/ &
        & 0.10677e+04_rb,0.78757e+03_rb,0.59836e+03_rb,0.88254e+03_rb,0.11660e+04_rb /)
      kbo(:, 5,34,14) = (/ &
        & 0.10419e+04_rb,0.77666e+03_rb,0.59604e+03_rb,0.87984e+03_rb,0.11644e+04_rb /)
      kbo(:, 1,35,14) = (/ &
        & 0.10613e+04_rb,0.80031e+03_rb,0.60136e+03_rb,0.88808e+03_rb,0.11748e+04_rb /)
      kbo(:, 2,35,14) = (/ &
        & 0.11118e+04_rb,0.80250e+03_rb,0.60031e+03_rb,0.88654e+03_rb,0.11732e+04_rb /)
      kbo(:, 3,35,14) = (/ &
        & 0.10759e+04_rb,0.83185e+03_rb,0.59892e+03_rb,0.88448e+03_rb,0.11701e+04_rb /)
      kbo(:, 4,35,14) = (/ &
        & 0.11049e+04_rb,0.77622e+03_rb,0.59743e+03_rb,0.88477e+03_rb,0.11680e+04_rb /)
      kbo(:, 5,35,14) = (/ &
        & 0.10459e+04_rb,0.78322e+03_rb,0.59581e+03_rb,0.87986e+03_rb,0.11634e+04_rb /)
      kbo(:, 1,36,14) = (/ &
        & 0.10396e+04_rb,0.79026e+03_rb,0.60134e+03_rb,0.88805e+03_rb,0.11748e+04_rb /)
      kbo(:, 2,36,14) = (/ &
        & 0.10471e+04_rb,0.80778e+03_rb,0.60028e+03_rb,0.88614e+03_rb,0.11727e+04_rb /)
      kbo(:, 3,36,14) = (/ &
        & 0.10450e+04_rb,0.79912e+03_rb,0.59889e+03_rb,0.88407e+03_rb,0.11700e+04_rb /)
      kbo(:, 4,36,14) = (/ &
        & 0.10626e+04_rb,0.80803e+03_rb,0.59740e+03_rb,0.88257e+03_rb,0.11756e+04_rb /)
      kbo(:, 5,36,14) = (/ &
        & 0.10609e+04_rb,0.81056e+03_rb,0.59577e+03_rb,0.87981e+03_rb,0.11634e+04_rb /)
      kbo(:, 1,37,14) = (/ &
        & 0.10479e+04_rb,0.79014e+03_rb,0.60145e+03_rb,0.88822e+03_rb,0.11750e+04_rb /)
      kbo(:, 2,37,14) = (/ &
        & 0.10744e+04_rb,0.81437e+03_rb,0.60044e+03_rb,0.88637e+03_rb,0.11730e+04_rb /)
      kbo(:, 3,37,14) = (/ &
        & 0.10335e+04_rb,0.79550e+03_rb,0.59909e+03_rb,0.88473e+03_rb,0.11704e+04_rb /)
      kbo(:, 4,37,14) = (/ &
        & 0.10410e+04_rb,0.77504e+03_rb,0.59808e+03_rb,0.88321e+03_rb,0.11674e+04_rb /)
      kbo(:, 5,37,14) = (/ &
        & 0.10557e+04_rb,0.79226e+03_rb,0.59599e+03_rb,0.88014e+03_rb,0.11643e+04_rb /)
      kbo(:, 1,38,14) = (/ &
        & 0.10418e+04_rb,0.80249e+03_rb,0.60156e+03_rb,0.88838e+03_rb,0.11752e+04_rb /)
      kbo(:, 2,38,14) = (/ &
        & 0.10220e+04_rb,0.82577e+03_rb,0.60060e+03_rb,0.88697e+03_rb,0.11733e+04_rb /)
      kbo(:, 3,38,14) = (/ &
        & 0.10502e+04_rb,0.80915e+03_rb,0.59929e+03_rb,0.88503e+03_rb,0.11708e+04_rb /)
      kbo(:, 4,38,14) = (/ &
        & 0.10497e+04_rb,0.81774e+03_rb,0.59826e+03_rb,0.88277e+03_rb,0.11673e+04_rb /)
      kbo(:, 5,38,14) = (/ &
        & 0.10771e+04_rb,0.81134e+03_rb,0.59623e+03_rb,0.88049e+03_rb,0.11648e+04_rb /)
      kbo(:, 1,39,14) = (/ &
        & 0.10165e+04_rb,0.77782e+03_rb,0.60118e+03_rb,0.88854e+03_rb,0.11754e+04_rb /)
      kbo(:, 2,39,14) = (/ &
        & 0.10531e+04_rb,0.80302e+03_rb,0.60075e+03_rb,0.88719e+03_rb,0.11736e+04_rb /)
      kbo(:, 3,39,14) = (/ &
        & 0.10758e+04_rb,0.78257e+03_rb,0.59949e+03_rb,0.88533e+03_rb,0.11712e+04_rb /)
      kbo(:, 4,39,14) = (/ &
        & 0.10568e+04_rb,0.79852e+03_rb,0.59772e+03_rb,0.88307e+03_rb,0.11677e+04_rb /)
      kbo(:, 5,39,14) = (/ &
        & 0.11120e+04_rb,0.79275e+03_rb,0.59645e+03_rb,0.88083e+03_rb,0.11652e+04_rb /)
      kbo(:, 1,40,14) = (/ &
        & 0.10416e+04_rb,0.79660e+03_rb,0.59890e+03_rb,0.88879e+03_rb,0.11758e+04_rb /)
      kbo(:, 2,40,14) = (/ &
        & 0.10391e+04_rb,0.79051e+03_rb,0.60100e+03_rb,0.88756e+03_rb,0.11741e+04_rb /)
      kbo(:, 3,40,14) = (/ &
        & 0.10420e+04_rb,0.78155e+03_rb,0.59983e+03_rb,0.88583e+03_rb,0.11718e+04_rb /)
      kbo(:, 4,40,14) = (/ &
        & 0.10496e+04_rb,0.79527e+03_rb,0.59833e+03_rb,0.88434e+03_rb,0.11689e+04_rb /)
      kbo(:, 5,40,14) = (/ &
        & 0.10264e+04_rb,0.78919e+03_rb,0.59684e+03_rb,0.88140e+03_rb,0.11660e+04_rb /)
      kbo(:, 1,41,14) = (/ &
        & 0.10689e+04_rb,0.80516e+03_rb,0.60103e+03_rb,0.88904e+03_rb,0.11761e+04_rb /)
      kbo(:, 2,41,14) = (/ &
        & 0.10416e+04_rb,0.82239e+03_rb,0.60124e+03_rb,0.88792e+03_rb,0.11746e+04_rb /)
      kbo(:, 3,41,14) = (/ &
        & 0.10563e+04_rb,0.81535e+03_rb,0.60016e+03_rb,0.88632e+03_rb,0.11725e+04_rb /)
      kbo(:, 4,41,14) = (/ &
        & 0.10676e+04_rb,0.82044e+03_rb,0.59873e+03_rb,0.88421e+03_rb,0.11697e+04_rb /)
      kbo(:, 5,41,14) = (/ &
        & 0.10596e+04_rb,0.78372e+03_rb,0.59724e+03_rb,0.88164e+03_rb,0.11672e+04_rb /)
      kbo(:, 1,42,14) = (/ &
        & 0.10655e+04_rb,0.80320e+03_rb,0.60196e+03_rb,0.88927e+03_rb,0.11764e+04_rb /)
      kbo(:, 2,42,14) = (/ &
        & 0.10714e+04_rb,0.78435e+03_rb,0.60147e+03_rb,0.88825e+03_rb,0.11750e+04_rb /)
      kbo(:, 3,42,14) = (/ &
        & 0.10516e+04_rb,0.79392e+03_rb,0.60071e+03_rb,0.88678e+03_rb,0.11731e+04_rb /)
      kbo(:, 4,42,14) = (/ &
        & 0.10516e+04_rb,0.77763e+03_rb,0.59913e+03_rb,0.88479e+03_rb,0.11700e+04_rb /)
      kbo(:, 5,42,14) = (/ &
        & 0.10486e+04_rb,0.79390e+03_rb,0.59811e+03_rb,0.88255e+03_rb,0.11641e+04_rb /)
      kbo(:, 1,43,14) = (/ &
        & 0.10979e+04_rb,0.83796e+03_rb,0.60248e+03_rb,0.88955e+03_rb,0.11768e+04_rb /)
      kbo(:, 2,43,14) = (/ &
        & 0.11147e+04_rb,0.76548e+03_rb,0.60171e+03_rb,0.88861e+03_rb,0.11750e+04_rb /)
      kbo(:, 3,43,14) = (/ &
        & 0.10326e+04_rb,0.79357e+03_rb,0.60081e+03_rb,0.88729e+03_rb,0.11738e+04_rb /)
      kbo(:, 4,43,14) = (/ &
        & 0.10439e+04_rb,0.86155e+03_rb,0.59958e+03_rb,0.88546e+03_rb,0.11709e+04_rb /)
      kbo(:, 5,43,14) = (/ &
        & 0.10945e+04_rb,0.81749e+03_rb,0.59781e+03_rb,0.88321e+03_rb,0.11679e+04_rb /)
      kbo(:, 1,44,14) = (/ &
        & 0.10684e+04_rb,0.83027e+03_rb,0.60306e+03_rb,0.88988e+03_rb,0.11772e+04_rb /)
      kbo(:, 2,44,14) = (/ &
        & 0.10549e+04_rb,0.78589e+03_rb,0.60194e+03_rb,0.88895e+03_rb,0.11755e+04_rb /)
      kbo(:, 3,44,14) = (/ &
        & 0.10654e+04_rb,0.79705e+03_rb,0.60140e+03_rb,0.88779e+03_rb,0.11744e+04_rb /)
      kbo(:, 4,44,14) = (/ &
        & 0.10595e+04_rb,0.79130e+03_rb,0.59979e+03_rb,0.88614e+03_rb,0.11722e+04_rb /)
      kbo(:, 5,44,14) = (/ &
        & 0.10477e+04_rb,0.80597e+03_rb,0.59858e+03_rb,0.88363e+03_rb,0.11704e+04_rb /)
      kbo(:, 1,45,14) = (/ &
        & 0.10873e+04_rb,0.79417e+03_rb,0.60313e+03_rb,0.89023e+03_rb,0.11777e+04_rb /)
      kbo(:, 2,45,14) = (/ &
        & 0.10616e+04_rb,0.78299e+03_rb,0.60195e+03_rb,0.88889e+03_rb,0.11764e+04_rb /)
      kbo(:, 3,45,14) = (/ &
        & 0.10452e+04_rb,0.80979e+03_rb,0.60145e+03_rb,0.88823e+03_rb,0.11745e+04_rb /)
      kbo(:, 4,45,14) = (/ &
        & 0.10957e+04_rb,0.80372e+03_rb,0.60045e+03_rb,0.88675e+03_rb,0.11726e+04_rb /)
      kbo(:, 5,45,14) = (/ &
        & 0.10725e+04_rb,0.79223e+03_rb,0.59886e+03_rb,0.88476e+03_rb,0.11704e+04_rb /)
      kbo(:, 1,46,14) = (/ &
        & 0.10477e+04_rb,0.79958e+03_rb,0.60344e+03_rb,0.89054e+03_rb,0.11781e+04_rb /)
      kbo(:, 2,46,14) = (/ &
        & 0.10873e+04_rb,0.80021e+03_rb,0.60251e+03_rb,0.88958e+03_rb,0.11768e+04_rb /)
      kbo(:, 3,46,14) = (/ &
        & 0.10228e+04_rb,0.78903e+03_rb,0.60173e+03_rb,0.88864e+03_rb,0.11751e+04_rb /)
      kbo(:, 4,46,14) = (/ &
        & 0.10295e+04_rb,0.83052e+03_rb,0.60085e+03_rb,0.88734e+03_rb,0.11738e+04_rb /)
      kbo(:, 5,46,14) = (/ &
        & 0.10527e+04_rb,0.80434e+03_rb,0.59963e+03_rb,0.88553e+03_rb,0.11714e+04_rb /)
      kbo(:, 1,47,14) = (/ &
        & 0.10317e+04_rb,0.78934e+03_rb,0.60373e+03_rb,0.89119e+03_rb,0.11785e+04_rb /)
      kbo(:, 2,47,14) = (/ &
        & 0.10325e+04_rb,0.79125e+03_rb,0.60291e+03_rb,0.88999e+03_rb,0.11778e+04_rb /)
      kbo(:, 3,47,14) = (/ &
        & 0.10417e+04_rb,0.79891e+03_rb,0.60152e+03_rb,0.88904e+03_rb,0.11761e+04_rb /)
      kbo(:, 4,47,14) = (/ &
        & 0.10561e+04_rb,0.78901e+03_rb,0.60124e+03_rb,0.88793e+03_rb,0.11746e+04_rb /)
      kbo(:, 5,47,14) = (/ &
        & 0.10468e+04_rb,0.79347e+03_rb,0.59991e+03_rb,0.88632e+03_rb,0.11725e+04_rb /)
      kbo(:, 1,48,14) = (/ &
        & 0.10540e+04_rb,0.78901e+03_rb,0.60401e+03_rb,0.89113e+03_rb,0.11789e+04_rb /)
      kbo(:, 2,48,14) = (/ &
        & 0.10320e+04_rb,0.80975e+03_rb,0.60327e+03_rb,0.89037e+03_rb,0.11779e+04_rb /)
      kbo(:, 3,48,14) = (/ &
        & 0.10535e+04_rb,0.79283e+03_rb,0.60233e+03_rb,0.88940e+03_rb,0.11766e+04_rb /)
      kbo(:, 4,48,14) = (/ &
        & 0.10485e+04_rb,0.81102e+03_rb,0.60158e+03_rb,0.88807e+03_rb,0.11753e+04_rb /)
      kbo(:, 5,48,14) = (/ &
        & 0.10337e+04_rb,0.82793e+03_rb,0.60064e+03_rb,0.88703e+03_rb,0.11734e+04_rb /)
      kbo(:, 1,49,14) = (/ &
        & 0.10304e+04_rb,0.83177e+03_rb,0.60454e+03_rb,0.89164e+03_rb,0.11796e+04_rb /)
      kbo(:, 2,49,14) = (/ &
        & 0.10417e+04_rb,0.78580e+03_rb,0.60359e+03_rb,0.89069e+03_rb,0.11783e+04_rb /)
      kbo(:, 3,49,14) = (/ &
        & 0.10725e+04_rb,0.82186e+03_rb,0.60271e+03_rb,0.88942e+03_rb,0.11771e+04_rb /)
      kbo(:, 4,49,14) = (/ &
        & 0.10362e+04_rb,0.80207e+03_rb,0.60065e+03_rb,0.88885e+03_rb,0.11758e+04_rb /)
      kbo(:, 5,49,14) = (/ &
        & 0.10383e+04_rb,0.79856e+03_rb,0.60081e+03_rb,0.88765e+03_rb,0.11743e+04_rb /)
      kbo(:, 1,50,14) = (/ &
        & 0.10581e+04_rb,0.78356e+03_rb,0.60457e+03_rb,0.89226e+03_rb,0.11805e+04_rb /)
      kbo(:, 2,50,14) = (/ &
        & 0.10447e+04_rb,0.80207e+03_rb,0.60385e+03_rb,0.89058e+03_rb,0.11786e+04_rb /)
      kbo(:, 3,50,14) = (/ &
        & 0.10788e+04_rb,0.82154e+03_rb,0.60308e+03_rb,0.89017e+03_rb,0.11776e+04_rb /)
      kbo(:, 4,50,14) = (/ &
        & 0.10695e+04_rb,0.79696e+03_rb,0.60214e+03_rb,0.88920e+03_rb,0.11763e+04_rb /)
      kbo(:, 5,50,14) = (/ &
        & 0.10859e+04_rb,0.79983e+03_rb,0.60140e+03_rb,0.88816e+03_rb,0.11749e+04_rb /)
      kbo(:, 1,51,14) = (/ &
        & 0.10459e+04_rb,0.79632e+03_rb,0.60498e+03_rb,0.89284e+03_rb,0.11812e+04_rb /)
      kbo(:, 2,51,14) = (/ &
        & 0.10371e+04_rb,0.78419e+03_rb,0.60411e+03_rb,0.89166e+03_rb,0.11796e+04_rb /)
      kbo(:, 3,51,14) = (/ &
        & 0.10657e+04_rb,0.82185e+03_rb,0.60315e+03_rb,0.89049e+03_rb,0.11780e+04_rb /)
      kbo(:, 4,51,14) = (/ &
        & 0.10396e+04_rb,0.78418e+03_rb,0.60197e+03_rb,0.88917e+03_rb,0.11768e+04_rb /)
      kbo(:, 5,51,14) = (/ &
        & 0.10354e+04_rb,0.78290e+03_rb,0.60169e+03_rb,0.88859e+03_rb,0.11755e+04_rb /)
      kbo(:, 1,52,14) = (/ &
        & 0.10434e+04_rb,0.78546e+03_rb,0.60574e+03_rb,0.89370e+03_rb,0.11824e+04_rb /)
      kbo(:, 2,52,14) = (/ &
        & 0.10665e+04_rb,0.78610e+03_rb,0.60413e+03_rb,0.89180e+03_rb,0.11798e+04_rb /)
      kbo(:, 3,52,14) = (/ &
        & 0.10821e+04_rb,0.78641e+03_rb,0.60367e+03_rb,0.89113e+03_rb,0.11784e+04_rb /)
      kbo(:, 4,52,14) = (/ &
        & 0.10514e+04_rb,0.79279e+03_rb,0.60282e+03_rb,0.88989e+03_rb,0.11772e+04_rb /)
      kbo(:, 5,52,14) = (/ &
        & 0.10408e+04_rb,0.77939e+03_rb,0.60170e+03_rb,0.88895e+03_rb,0.11760e+04_rb /)
      kbo(:, 1,53,14) = (/ &
        & 0.10497e+04_rb,0.80875e+03_rb,0.60601e+03_rb,0.89382e+03_rb,0.11825e+04_rb /)
      kbo(:, 2,53,14) = (/ &
        & 0.10514e+04_rb,0.81226e+03_rb,0.60441e+03_rb,0.89243e+03_rb,0.11807e+04_rb /)
      kbo(:, 3,53,14) = (/ &
        & 0.11137e+04_rb,0.80237e+03_rb,0.60392e+03_rb,0.89101e+03_rb,0.11787e+04_rb /)
      kbo(:, 4,53,14) = (/ &
        & 0.10661e+04_rb,0.79885e+03_rb,0.60292e+03_rb,0.89026e+03_rb,0.11777e+04_rb /)
      kbo(:, 5,53,14) = (/ &
        & 0.10324e+04_rb,0.82343e+03_rb,0.60222e+03_rb,0.88929e+03_rb,0.11764e+04_rb /)
      kbo(:, 1,54,14) = (/ &
        & 0.10594e+04_rb,0.86492e+03_rb,0.60646e+03_rb,0.89428e+03_rb,0.11831e+04_rb /)
      kbo(:, 2,54,14) = (/ &
        & 0.10303e+04_rb,0.85949e+03_rb,0.60506e+03_rb,0.89294e+03_rb,0.11813e+04_rb /)
      kbo(:, 3,54,14) = (/ &
        & 0.10905e+04_rb,0.84800e+03_rb,0.60416e+03_rb,0.89138e+03_rb,0.11793e+04_rb /)
      kbo(:, 4,54,14) = (/ &
        & 0.10438e+04_rb,0.78736e+03_rb,0.60344e+03_rb,0.89055e+03_rb,0.11781e+04_rb /)
      kbo(:, 5,54,14) = (/ &
        & 0.10324e+04_rb,0.83619e+03_rb,0.60252e+03_rb,0.88959e+03_rb,0.11768e+04_rb /)
      kbo(:, 1,55,14) = (/ &
        & 0.10463e+04_rb,0.78385e+03_rb,0.60663e+03_rb,0.89475e+03_rb,0.11838e+04_rb /)
      kbo(:, 2,55,14) = (/ &
        & 0.10543e+04_rb,0.79119e+03_rb,0.60552e+03_rb,0.89337e+03_rb,0.11824e+04_rb /)
      kbo(:, 3,55,14) = (/ &
        & 0.10391e+04_rb,0.80395e+03_rb,0.60439e+03_rb,0.89183e+03_rb,0.11799e+04_rb /)
      kbo(:, 4,55,14) = (/ &
        & 0.10421e+04_rb,0.79916e+03_rb,0.60368e+03_rb,0.89078e+03_rb,0.11784e+04_rb /)
      kbo(:, 5,55,14) = (/ &
        & 0.10909e+04_rb,0.78448e+03_rb,0.60284e+03_rb,0.88955e+03_rb,0.11777e+04_rb /)
      kbo(:, 1,56,14) = (/ &
        & 0.10143e+04_rb,0.81481e+03_rb,0.60727e+03_rb,0.89531e+03_rb,0.11848e+04_rb /)
      kbo(:, 2,56,14) = (/ &
        & 0.10240e+04_rb,0.81034e+03_rb,0.60598e+03_rb,0.89378e+03_rb,0.11825e+04_rb /)
      kbo(:, 3,56,14) = (/ &
        & 0.10324e+04_rb,0.78927e+03_rb,0.60463e+03_rb,0.89238e+03_rb,0.11806e+04_rb /)
      kbo(:, 4,56,14) = (/ &
        & 0.10273e+04_rb,0.78959e+03_rb,0.60390e+03_rb,0.89099e+03_rb,0.11787e+04_rb /)
      kbo(:, 5,56,14) = (/ &
        & 0.10404e+04_rb,0.80937e+03_rb,0.60290e+03_rb,0.89024e+03_rb,0.11777e+04_rb /)
      kbo(:, 1,57,14) = (/ &
        & 0.10408e+04_rb,0.79150e+03_rb,0.60742e+03_rb,0.89593e+03_rb,0.11860e+04_rb /)
      kbo(:, 2,57,14) = (/ &
        & 0.10640e+04_rb,0.82150e+03_rb,0.60641e+03_rb,0.89422e+03_rb,0.11831e+04_rb /)
      kbo(:, 3,57,14) = (/ &
        & 0.10551e+04_rb,0.79661e+03_rb,0.60501e+03_rb,0.89288e+03_rb,0.11813e+04_rb /)
      kbo(:, 4,57,14) = (/ &
        & 0.10817e+04_rb,0.79118e+03_rb,0.60413e+03_rb,0.89133e+03_rb,0.11792e+04_rb /)
      kbo(:, 5,57,14) = (/ &
        & 0.10370e+04_rb,0.79629e+03_rb,0.60341e+03_rb,0.89015e+03_rb,0.11781e+04_rb /)
      kbo(:, 1,58,14) = (/ &
        & 0.10442e+04_rb,0.84033e+03_rb,0.60809e+03_rb,0.89672e+03_rb,0.11869e+04_rb /)
      kbo(:, 2,58,14) = (/ &
        & 0.10934e+04_rb,0.79820e+03_rb,0.60680e+03_rb,0.89466e+03_rb,0.11837e+04_rb /)
      kbo(:, 3,58,14) = (/ &
        & 0.10657e+04_rb,0.80043e+03_rb,0.60544e+03_rb,0.89329e+03_rb,0.11818e+04_rb /)
      kbo(:, 4,58,14) = (/ &
        & 0.10223e+04_rb,0.78480e+03_rb,0.60435e+03_rb,0.89175e+03_rb,0.11798e+04_rb /)
      kbo(:, 5,58,14) = (/ &
        & 0.10438e+04_rb,0.81735e+03_rb,0.60364e+03_rb,0.89074e+03_rb,0.11789e+04_rb /)
      kbo(:, 1,59,14) = (/ &
        & 0.10513e+04_rb,0.80331e+03_rb,0.60826e+03_rb,0.89709e+03_rb,0.11883e+04_rb /)
      kbo(:, 2,59,14) = (/ &
        & 0.10429e+04_rb,0.81799e+03_rb,0.60696e+03_rb,0.89486e+03_rb,0.11840e+04_rb /)
      kbo(:, 3,59,14) = (/ &
        & 0.10400e+04_rb,0.80905e+03_rb,0.60537e+03_rb,0.89345e+03_rb,0.11830e+04_rb /)
      kbo(:, 4,59,14) = (/ &
        & 0.10488e+04_rb,0.79628e+03_rb,0.60443e+03_rb,0.89194e+03_rb,0.11800e+04_rb /)
      kbo(:, 5,59,14) = (/ &
        & 0.10795e+04_rb,0.78415e+03_rb,0.60372e+03_rb,0.89082e+03_rb,0.11785e+04_rb /)
      kbo(:, 1,13,15) = (/ &
        & 0.98934e+03_rb,0.75572e+03_rb,0.60810e+03_rb,0.89364e+03_rb,0.11812e+04_rb /)
      kbo(:, 2,13,15) = (/ &
        & 0.12809e+04_rb,0.72091e+03_rb,0.60670e+03_rb,0.89251e+03_rb,0.11797e+04_rb /)
      kbo(:, 3,13,15) = (/ &
        & 0.11704e+04_rb,0.77844e+03_rb,0.60529e+03_rb,0.89171e+03_rb,0.11787e+04_rb /)
      kbo(:, 4,13,15) = (/ &
        & 0.11159e+04_rb,0.83495e+03_rb,0.60430e+03_rb,0.89110e+03_rb,0.11779e+04_rb /)
      kbo(:, 5,13,15) = (/ &
        & 0.11939e+04_rb,0.93553e+03_rb,0.60355e+03_rb,0.89003e+03_rb,0.11765e+04_rb /)
      kbo(:, 1,14,15) = (/ &
        & 0.11963e+04_rb,0.95315e+03_rb,0.60753e+03_rb,0.89338e+03_rb,0.11810e+04_rb /)
      kbo(:, 2,14,15) = (/ &
        & 0.11729e+04_rb,0.92141e+03_rb,0.60616e+03_rb,0.89229e+03_rb,0.11796e+04_rb /)
      kbo(:, 3,14,15) = (/ &
        & 0.11471e+04_rb,0.85602e+03_rb,0.60478e+03_rb,0.89154e+03_rb,0.11786e+04_rb /)
      kbo(:, 4,14,15) = (/ &
        & 0.11824e+04_rb,0.88196e+03_rb,0.60401e+03_rb,0.89091e+03_rb,0.11778e+04_rb /)
      kbo(:, 5,14,15) = (/ &
        & 0.11271e+04_rb,0.87462e+03_rb,0.60324e+03_rb,0.89075e+03_rb,0.11763e+04_rb /)
      kbo(:, 1,15,15) = (/ &
        & 0.10604e+04_rb,0.90519e+03_rb,0.60703e+03_rb,0.89409e+03_rb,0.11809e+04_rb /)
      kbo(:, 2,15,15) = (/ &
        & 0.11577e+04_rb,0.75557e+03_rb,0.60568e+03_rb,0.89209e+03_rb,0.11795e+04_rb /)
      kbo(:, 3,15,15) = (/ &
        & 0.11406e+04_rb,0.81803e+03_rb,0.60438e+03_rb,0.89138e+03_rb,0.11785e+04_rb /)
      kbo(:, 4,15,15) = (/ &
        & 0.11847e+04_rb,0.71960e+03_rb,0.60376e+03_rb,0.89073e+03_rb,0.11790e+04_rb /)
      kbo(:, 5,15,15) = (/ &
        & 0.11179e+04_rb,0.81112e+03_rb,0.60296e+03_rb,0.88859e+03_rb,0.11787e+04_rb /)
      kbo(:, 1,16,15) = (/ &
        & 0.10356e+04_rb,0.71983e+03_rb,0.60659e+03_rb,0.89194e+03_rb,0.11820e+04_rb /)
      kbo(:, 2,16,15) = (/ &
        & 0.11364e+04_rb,0.76870e+03_rb,0.60526e+03_rb,0.89190e+03_rb,0.11794e+04_rb /)
      kbo(:, 3,16,15) = (/ &
        & 0.96034e+03_rb,0.83277e+03_rb,0.60407e+03_rb,0.89124e+03_rb,0.11785e+04_rb /)
      kbo(:, 4,16,15) = (/ &
        & 0.10077e+04_rb,0.87988e+03_rb,0.60354e+03_rb,0.89055e+03_rb,0.11776e+04_rb /)
      kbo(:, 5,16,15) = (/ &
        & 0.10422e+04_rb,0.83397e+03_rb,0.60335e+03_rb,0.88931e+03_rb,0.11759e+04_rb /)
      kbo(:, 1,17,15) = (/ &
        & 0.10091e+04_rb,0.77782e+03_rb,0.60621e+03_rb,0.89269e+03_rb,0.11805e+04_rb /)
      kbo(:, 2,17,15) = (/ &
        & 0.11716e+04_rb,0.80971e+03_rb,0.60425e+03_rb,0.89175e+03_rb,0.11793e+04_rb /)
      kbo(:, 3,17,15) = (/ &
        & 0.10550e+04_rb,0.78786e+03_rb,0.60385e+03_rb,0.89112e+03_rb,0.11784e+04_rb /)
      kbo(:, 4,17,15) = (/ &
        & 0.99990e+03_rb,0.88096e+03_rb,0.60335e+03_rb,0.89039e+03_rb,0.11774e+04_rb /)
      kbo(:, 5,17,15) = (/ &
        & 0.11263e+04_rb,0.73127e+03_rb,0.60311e+03_rb,0.88908e+03_rb,0.11770e+04_rb /)
      kbo(:, 1,18,15) = (/ &
        & 0.11151e+04_rb,0.85991e+03_rb,0.60588e+03_rb,0.89155e+03_rb,0.11804e+04_rb /)
      kbo(:, 2,18,15) = (/ &
        & 0.10413e+04_rb,0.70815e+03_rb,0.60458e+03_rb,0.89161e+03_rb,0.11791e+04_rb /)
      kbo(:, 3,18,15) = (/ &
        & 0.11058e+04_rb,0.82037e+03_rb,0.60370e+03_rb,0.89101e+03_rb,0.11783e+04_rb /)
      kbo(:, 4,18,15) = (/ &
        & 0.10134e+04_rb,0.64109e+03_rb,0.60383e+03_rb,0.89024e+03_rb,0.11786e+04_rb /)
      kbo(:, 5,18,15) = (/ &
        & 0.11497e+04_rb,0.88170e+03_rb,0.60224e+03_rb,0.88981e+03_rb,0.11767e+04_rb /)
      kbo(:, 1,19,15) = (/ &
        & 0.10261e+04_rb,0.82130e+03_rb,0.60625e+03_rb,0.89236e+03_rb,0.11802e+04_rb /)
      kbo(:, 2,19,15) = (/ &
        & 0.10077e+04_rb,0.78017e+03_rb,0.60431e+03_rb,0.89148e+03_rb,0.11778e+04_rb /)
      kbo(:, 3,19,15) = (/ &
        & 0.10505e+04_rb,0.75163e+03_rb,0.60422e+03_rb,0.88995e+03_rb,0.11782e+04_rb /)
      kbo(:, 4,19,15) = (/ &
        & 0.10498e+04_rb,0.84163e+03_rb,0.60302e+03_rb,0.89009e+03_rb,0.11772e+04_rb /)
      kbo(:, 5,19,15) = (/ &
        & 0.94853e+03_rb,0.71012e+03_rb,0.60269e+03_rb,0.88958e+03_rb,0.11765e+04_rb /)
      kbo(:, 1,20,15) = (/ &
        & 0.11198e+04_rb,0.83291e+03_rb,0.60532e+03_rb,0.89221e+03_rb,0.11814e+04_rb /)
      kbo(:, 2,20,15) = (/ &
        & 0.10146e+04_rb,0.70988e+03_rb,0.60473e+03_rb,0.89136e+03_rb,0.11789e+04_rb /)
      kbo(:, 3,20,15) = (/ &
        & 0.98440e+03_rb,0.83872e+03_rb,0.60345e+03_rb,0.89080e+03_rb,0.11781e+04_rb /)
      kbo(:, 4,20,15) = (/ &
        & 0.10595e+04_rb,0.75621e+03_rb,0.60286e+03_rb,0.88992e+03_rb,0.11770e+04_rb /)
      kbo(:, 5,20,15) = (/ &
        & 0.10274e+04_rb,0.81836e+03_rb,0.60247e+03_rb,0.88836e+03_rb,0.11749e+04_rb /)
      kbo(:, 1,21,15) = (/ &
        & 0.10788e+04_rb,0.78645e+03_rb,0.60506e+03_rb,0.89207e+03_rb,0.11799e+04_rb /)
      kbo(:, 2,21,15) = (/ &
        & 0.10852e+04_rb,0.74295e+03_rb,0.60386e+03_rb,0.89125e+03_rb,0.11788e+04_rb /)
      kbo(:, 3,21,15) = (/ &
        & 0.10428e+04_rb,0.82396e+03_rb,0.60333e+03_rb,0.88973e+03_rb,0.11780e+04_rb /)
      kbo(:, 4,21,15) = (/ &
        & 0.10018e+04_rb,0.87387e+03_rb,0.60336e+03_rb,0.89071e+03_rb,0.11768e+04_rb /)
      kbo(:, 5,21,15) = (/ &
        & 0.11155e+04_rb,0.79293e+03_rb,0.60225e+03_rb,0.88809e+03_rb,0.11746e+04_rb /)
      kbo(:, 1,22,15) = (/ &
        & 0.10301e+04_rb,0.79445e+03_rb,0.60539e+03_rb,0.89186e+03_rb,0.11797e+04_rb /)
      kbo(:, 2,22,15) = (/ &
        & 0.11174e+04_rb,0.68851e+03_rb,0.60364e+03_rb,0.88919e+03_rb,0.11787e+04_rb /)
      kbo(:, 3,22,15) = (/ &
        & 0.10845e+04_rb,0.72821e+03_rb,0.60320e+03_rb,0.89053e+03_rb,0.11779e+04_rb /)
      kbo(:, 4,22,15) = (/ &
        & 0.94182e+03_rb,0.74129e+03_rb,0.60185e+03_rb,0.88948e+03_rb,0.11777e+04_rb /)
      kbo(:, 5,22,15) = (/ &
        & 0.11256e+04_rb,0.74069e+03_rb,0.60323e+03_rb,0.88766e+03_rb,0.11740e+04_rb /)
      kbo(:, 1,23,15) = (/ &
        & 0.11218e+04_rb,0.85968e+03_rb,0.60443e+03_rb,0.89167e+03_rb,0.11795e+04_rb /)
      kbo(:, 2,23,15) = (/ &
        & 0.10449e+04_rb,0.80739e+03_rb,0.60218e+03_rb,0.89100e+03_rb,0.11785e+04_rb /)
      kbo(:, 3,23,15) = (/ &
        & 0.11277e+04_rb,0.69098e+03_rb,0.60306e+03_rb,0.89036e+03_rb,0.11777e+04_rb /)
      kbo(:, 4,23,15) = (/ &
        & 0.10580e+04_rb,0.81384e+03_rb,0.60228e+03_rb,0.88919e+03_rb,0.11761e+04_rb /)
      kbo(:, 5,23,15) = (/ &
        & 0.10279e+04_rb,0.82530e+03_rb,0.60094e+03_rb,0.88718e+03_rb,0.11734e+04_rb /)
      kbo(:, 1,24,15) = (/ &
        & 0.10227e+04_rb,0.68179e+03_rb,0.60412e+03_rb,0.89149e+03_rb,0.11792e+04_rb /)
      kbo(:, 2,24,15) = (/ &
        & 0.11518e+04_rb,0.80624e+03_rb,0.60338e+03_rb,0.89088e+03_rb,0.11784e+04_rb /)
      kbo(:, 3,24,15) = (/ &
        & 0.10788e+04_rb,0.70820e+03_rb,0.60290e+03_rb,0.89016e+03_rb,0.11787e+04_rb /)
      kbo(:, 4,24,15) = (/ &
        & 0.10466e+04_rb,0.82233e+03_rb,0.60268e+03_rb,0.88884e+03_rb,0.11757e+04_rb /)
      kbo(:, 5,24,15) = (/ &
        & 0.10066e+04_rb,0.82618e+03_rb,0.60057e+03_rb,0.88665e+03_rb,0.11740e+04_rb /)
      kbo(:, 1,25,15) = (/ &
        & 0.10780e+04_rb,0.80937e+03_rb,0.60386e+03_rb,0.89131e+03_rb,0.11790e+04_rb /)
      kbo(:, 2,25,15) = (/ &
        & 0.10199e+04_rb,0.81059e+03_rb,0.60327e+03_rb,0.89075e+03_rb,0.11782e+04_rb /)
      kbo(:, 3,25,15) = (/ &
        & 0.11207e+04_rb,0.80162e+03_rb,0.60338e+03_rb,0.88992e+03_rb,0.11771e+04_rb /)
      kbo(:, 4,25,15) = (/ &
        & 0.97495e+03_rb,0.78589e+03_rb,0.60305e+03_rb,0.88845e+03_rb,0.11751e+04_rb /)
      kbo(:, 5,25,15) = (/ &
        & 0.95428e+03_rb,0.80945e+03_rb,0.60015e+03_rb,0.88605e+03_rb,0.11720e+04_rb /)
      kbo(:, 1,26,15) = (/ &
        & 0.10862e+04_rb,0.78235e+03_rb,0.60361e+03_rb,0.89115e+03_rb,0.11801e+04_rb /)
      kbo(:, 2,26,15) = (/ &
        & 0.91581e+03_rb,0.82546e+03_rb,0.60381e+03_rb,0.89059e+03_rb,0.11780e+04_rb /)
      kbo(:, 3,26,15) = (/ &
        & 0.10920e+04_rb,0.84813e+03_rb,0.60253e+03_rb,0.88964e+03_rb,0.11767e+04_rb /)
      kbo(:, 4,26,15) = (/ &
        & 0.11174e+04_rb,0.83078e+03_rb,0.60207e+03_rb,0.88892e+03_rb,0.11745e+04_rb /)
      kbo(:, 5,26,15) = (/ &
        & 0.11159e+04_rb,0.75128e+03_rb,0.59967e+03_rb,0.88536e+03_rb,0.11723e+04_rb /)
      kbo(:, 1,27,15) = (/ &
        & 0.10835e+04_rb,0.80424e+03_rb,0.60343e+03_rb,0.89101e+03_rb,0.11786e+04_rb /)
      kbo(:, 2,27,15) = (/ &
        & 0.10910e+04_rb,0.86525e+03_rb,0.60302e+03_rb,0.89041e+03_rb,0.11778e+04_rb /)
      kbo(:, 3,27,15) = (/ &
        & 0.10210e+04_rb,0.77642e+03_rb,0.60231e+03_rb,0.89027e+03_rb,0.11763e+04_rb /)
      kbo(:, 4,27,15) = (/ &
        & 0.10645e+04_rb,0.80598e+03_rb,0.60105e+03_rb,0.88742e+03_rb,0.11751e+04_rb /)
      kbo(:, 5,27,15) = (/ &
        & 0.10451e+04_rb,0.82023e+03_rb,0.59981e+03_rb,0.88559e+03_rb,0.11701e+04_rb /)
      kbo(:, 1,28,15) = (/ &
        & 0.10856e+04_rb,0.83991e+03_rb,0.60332e+03_rb,0.89088e+03_rb,0.11784e+04_rb /)
      kbo(:, 2,28,15) = (/ &
        & 0.93268e+03_rb,0.70779e+03_rb,0.60288e+03_rb,0.89019e+03_rb,0.11788e+04_rb /)
      kbo(:, 3,28,15) = (/ &
        & 0.11243e+04_rb,0.76544e+03_rb,0.60205e+03_rb,0.88894e+03_rb,0.11758e+04_rb /)
      kbo(:, 4,28,15) = (/ &
        & 0.10096e+04_rb,0.72189e+03_rb,0.60064e+03_rb,0.88682e+03_rb,0.11730e+04_rb /)
      kbo(:, 5,28,15) = (/ &
        & 0.88490e+03_rb,0.83821e+03_rb,0.59801e+03_rb,0.88198e+03_rb,0.11692e+04_rb /)
      kbo(:, 1,29,15) = (/ &
        & 0.10218e+04_rb,0.78897e+03_rb,0.60322e+03_rb,0.88978e+03_rb,0.11783e+04_rb /)
      kbo(:, 2,29,15) = (/ &
        & 0.97223e+03_rb,0.70803e+03_rb,0.60270e+03_rb,0.88994e+03_rb,0.11772e+04_rb /)
      kbo(:, 3,29,15) = (/ &
        & 0.10729e+04_rb,0.71638e+03_rb,0.60175e+03_rb,0.88850e+03_rb,0.11752e+04_rb /)
      kbo(:, 4,29,15) = (/ &
        & 0.10369e+04_rb,0.79357e+03_rb,0.60017e+03_rb,0.88614e+03_rb,0.11734e+04_rb /)
      kbo(:, 5,29,15) = (/ &
        & 0.10748e+04_rb,0.83591e+03_rb,0.59814e+03_rb,0.88321e+03_rb,0.11671e+04_rb /)
      kbo(:, 1,30,15) = (/ &
        & 0.10681e+04_rb,0.78838e+03_rb,0.60311e+03_rb,0.89058e+03_rb,0.11780e+04_rb /)
      kbo(:, 2,30,15) = (/ &
        & 0.10690e+04_rb,0.81372e+03_rb,0.60250e+03_rb,0.88965e+03_rb,0.11768e+04_rb /)
      kbo(:, 3,30,15) = (/ &
        & 0.10330e+04_rb,0.79065e+03_rb,0.60271e+03_rb,0.88800e+03_rb,0.11759e+04_rb /)
      kbo(:, 4,30,15) = (/ &
        & 0.99812e+03_rb,0.80751e+03_rb,0.60032e+03_rb,0.88541e+03_rb,0.11712e+04_rb /)
      kbo(:, 5,30,15) = (/ &
        & 0.10394e+04_rb,0.71741e+03_rb,0.59705e+03_rb,0.88643e+03_rb,0.11676e+04_rb /)
      kbo(:, 1,31,15) = (/ &
        & 0.10025e+04_rb,0.73187e+03_rb,0.60298e+03_rb,0.89039e+03_rb,0.11778e+04_rb /)
      kbo(:, 2,31,15) = (/ &
        & 0.11099e+04_rb,0.74280e+03_rb,0.60227e+03_rb,0.88930e+03_rb,0.11763e+04_rb /)
      kbo(:, 3,31,15) = (/ &
        & 0.10884e+04_rb,0.79617e+03_rb,0.60102e+03_rb,0.88838e+03_rb,0.11738e+04_rb /)
      kbo(:, 4,31,15) = (/ &
        & 0.90245e+03_rb,0.76637e+03_rb,0.59915e+03_rb,0.88368e+03_rb,0.11701e+04_rb /)
      kbo(:, 5,31,15) = (/ &
        & 0.10031e+04_rb,0.78751e+03_rb,0.59673e+03_rb,0.89640e+03_rb,0.11644e+04_rb /)
      kbo(:, 1,32,15) = (/ &
        & 0.87663e+03_rb,0.68400e+03_rb,0.60283e+03_rb,0.89016e+03_rb,0.11775e+04_rb /)
      kbo(:, 2,32,15) = (/ &
        & 0.10467e+04_rb,0.83071e+03_rb,0.60330e+03_rb,0.88890e+03_rb,0.11758e+04_rb /)
      kbo(:, 3,32,15) = (/ &
        & 0.97045e+03_rb,0.72881e+03_rb,0.60123e+03_rb,0.88677e+03_rb,0.11730e+04_rb /)
      kbo(:, 4,32,15) = (/ &
        & 0.11080e+04_rb,0.69141e+03_rb,0.59860e+03_rb,0.88193e+03_rb,0.11679e+04_rb /)
      kbo(:, 5,32,15) = (/ &
        & 0.10206e+04_rb,0.73037e+03_rb,0.59837e+03_rb,0.88358e+03_rb,0.11726e+04_rb /)
      kbo(:, 1,33,15) = (/ &
        & 0.10520e+04_rb,0.78807e+03_rb,0.60265e+03_rb,0.89086e+03_rb,0.11771e+04_rb /)
      kbo(:, 2,33,15) = (/ &
        & 0.98139e+03_rb,0.81175e+03_rb,0.60169e+03_rb,0.88843e+03_rb,0.11752e+04_rb /)
      kbo(:, 3,33,15) = (/ &
        & 0.11223e+04_rb,0.75994e+03_rb,0.60074e+03_rb,0.88510e+03_rb,0.11720e+04_rb /)
      kbo(:, 4,33,15) = (/ &
        & 0.10271e+04_rb,0.75564e+03_rb,0.59806e+03_rb,0.88218e+03_rb,0.11708e+04_rb /)
      kbo(:, 5,33,15) = (/ &
        & 0.10438e+04_rb,0.78527e+03_rb,0.59677e+03_rb,0.88217e+03_rb,0.11682e+04_rb /)
      kbo(:, 1,34,15) = (/ &
        & 0.10293e+04_rb,0.82175e+03_rb,0.60313e+03_rb,0.88963e+03_rb,0.11768e+04_rb /)
      kbo(:, 2,34,15) = (/ &
        & 0.11086e+04_rb,0.80472e+03_rb,0.60138e+03_rb,0.88797e+03_rb,0.11746e+04_rb /)
      kbo(:, 3,34,15) = (/ &
        & 0.10235e+04_rb,0.72497e+03_rb,0.60029e+03_rb,0.88634e+03_rb,0.11724e+04_rb /)
      kbo(:, 4,34,15) = (/ &
        & 0.10022e+04_rb,0.79360e+03_rb,0.59703e+03_rb,0.88927e+03_rb,0.11777e+04_rb /)
      kbo(:, 5,34,15) = (/ &
        & 0.11107e+04_rb,0.84019e+03_rb,0.59650e+03_rb,0.88082e+03_rb,0.11651e+04_rb /)
      kbo(:, 1,35,15) = (/ &
        & 0.10782e+04_rb,0.81052e+03_rb,0.60236e+03_rb,0.89138e+03_rb,0.11766e+04_rb /)
      kbo(:, 2,35,15) = (/ &
        & 0.89922e+03_rb,0.74859e+03_rb,0.60119e+03_rb,0.88769e+03_rb,0.11729e+04_rb /)
      kbo(:, 3,35,15) = (/ &
        & 0.10088e+04_rb,0.71209e+03_rb,0.59938e+03_rb,0.88499e+03_rb,0.11706e+04_rb /)
      kbo(:, 4,35,15) = (/ &
        & 0.96175e+03_rb,0.77055e+03_rb,0.60072e+03_rb,0.87852e+03_rb,0.11710e+04_rb /)
      kbo(:, 5,35,15) = (/ &
        & 0.10310e+04_rb,0.84426e+03_rb,0.59635e+03_rb,0.88059e+03_rb,0.11648e+04_rb /)
      kbo(:, 1,36,15) = (/ &
        & 0.10188e+04_rb,0.85190e+03_rb,0.60234e+03_rb,0.88943e+03_rb,0.11765e+04_rb /)
      kbo(:, 2,36,15) = (/ &
        & 0.10154e+04_rb,0.76964e+03_rb,0.60245e+03_rb,0.88859e+03_rb,0.11741e+04_rb /)
      kbo(:, 3,36,15) = (/ &
        & 0.10902e+04_rb,0.77130e+03_rb,0.59998e+03_rb,0.88588e+03_rb,0.11705e+04_rb /)
      kbo(:, 4,36,15) = (/ &
        & 0.98172e+03_rb,0.79246e+03_rb,0.59876e+03_rb,0.89371e+03_rb,0.11495e+04_rb /)
      kbo(:, 5,36,15) = (/ &
        & 0.10409e+04_rb,0.75260e+03_rb,0.59632e+03_rb,0.88055e+03_rb,0.11660e+04_rb /)
      kbo(:, 1,37,15) = (/ &
        & 0.10398e+04_rb,0.81024e+03_rb,0.60310e+03_rb,0.88960e+03_rb,0.11767e+04_rb /)
      kbo(:, 2,37,15) = (/ &
        & 0.94253e+03_rb,0.72207e+03_rb,0.60199e+03_rb,0.88792e+03_rb,0.11745e+04_rb /)
      kbo(:, 3,37,15) = (/ &
        & 0.11326e+04_rb,0.78984e+03_rb,0.60024e+03_rb,0.88532e+03_rb,0.11710e+04_rb /)
      kbo(:, 4,37,15) = (/ &
        & 0.10789e+04_rb,0.84320e+03_rb,0.59634e+03_rb,0.88350e+03_rb,0.11688e+04_rb /)
      kbo(:, 5,37,15) = (/ &
        & 0.10084e+04_rb,0.82623e+03_rb,0.59647e+03_rb,0.88078e+03_rb,0.11651e+04_rb /)
      kbo(:, 1,38,15) = (/ &
        & 0.99835e+03_rb,0.77031e+03_rb,0.60321e+03_rb,0.88977e+03_rb,0.11782e+04_rb /)
      kbo(:, 2,38,15) = (/ &
        & 0.11135e+04_rb,0.73046e+03_rb,0.60283e+03_rb,0.88917e+03_rb,0.11749e+04_rb /)
      kbo(:, 3,38,15) = (/ &
        & 0.10419e+04_rb,0.72620e+03_rb,0.59987e+03_rb,0.88669e+03_rb,0.11716e+04_rb /)
      kbo(:, 4,38,15) = (/ &
        & 0.10631e+04_rb,0.75245e+03_rb,0.59721e+03_rb,0.88572e+03_rb,0.11692e+04_rb /)
      kbo(:, 5,38,15) = (/ &
        & 0.10228e+04_rb,0.73209e+03_rb,0.59792e+03_rb,0.88102e+03_rb,0.11654e+04_rb /)
      kbo(:, 1,39,15) = (/ &
        & 0.10910e+04_rb,0.86853e+03_rb,0.60266e+03_rb,0.88992e+03_rb,0.11772e+04_rb /)
      kbo(:, 2,39,15) = (/ &
        & 0.10519e+04_rb,0.73802e+03_rb,0.60236e+03_rb,0.88944e+03_rb,0.11753e+04_rb /)
      kbo(:, 3,39,15) = (/ &
        & 0.90206e+03_rb,0.82442e+03_rb,0.60079e+03_rb,0.88708e+03_rb,0.11721e+04_rb /)
      kbo(:, 4,39,15) = (/ &
        & 0.10954e+04_rb,0.83033e+03_rb,0.59811e+03_rb,0.88319e+03_rb,0.11696e+04_rb /)
      kbo(:, 5,39,15) = (/ &
        & 0.92773e+03_rb,0.79134e+03_rb,0.59808e+03_rb,0.88316e+03_rb,0.11657e+04_rb /)
      kbo(:, 1,40,15) = (/ &
        & 0.10887e+04_rb,0.78119e+03_rb,0.60281e+03_rb,0.89016e+03_rb,0.11775e+04_rb /)
      kbo(:, 2,40,15) = (/ &
        & 0.10361e+04_rb,0.78964e+03_rb,0.60199e+03_rb,0.88890e+03_rb,0.11758e+04_rb /)
      kbo(:, 3,40,15) = (/ &
        & 0.10674e+04_rb,0.81250e+03_rb,0.60058e+03_rb,0.88678e+03_rb,0.11730e+04_rb /)
      kbo(:, 4,40,15) = (/ &
        & 0.11076e+04_rb,0.76250e+03_rb,0.59989e+03_rb,0.88480e+03_rb,0.11679e+04_rb /)
      kbo(:, 5,40,15) = (/ &
        & 0.11154e+04_rb,0.83280e+03_rb,0.59900e+03_rb,0.88168e+03_rb,0.11814e+04_rb /)
      kbo(:, 1,41,15) = (/ &
        & 0.10350e+04_rb,0.79893e+03_rb,0.60296e+03_rb,0.89038e+03_rb,0.11778e+04_rb /)
      kbo(:, 2,41,15) = (/ &
        & 0.10104e+04_rb,0.69640e+03_rb,0.60225e+03_rb,0.88929e+03_rb,0.11763e+04_rb /)
      kbo(:, 3,41,15) = (/ &
        & 0.10193e+04_rb,0.78110e+03_rb,0.60100e+03_rb,0.88741e+03_rb,0.11751e+04_rb /)
      kbo(:, 4,41,15) = (/ &
        & 0.10785e+04_rb,0.73450e+03_rb,0.59977e+03_rb,0.88368e+03_rb,0.11701e+04_rb /)
      kbo(:, 5,41,15) = (/ &
        & 0.96003e+03_rb,0.80480e+03_rb,0.59865e+03_rb,0.90400e+03_rb,0.11833e+04_rb /)
      kbo(:, 1,42,15) = (/ &
        & 0.97569e+03_rb,0.77685e+03_rb,0.60307e+03_rb,0.89056e+03_rb,0.11793e+04_rb /)
      kbo(:, 2,42,15) = (/ &
        & 0.98796e+03_rb,0.83953e+03_rb,0.60247e+03_rb,0.88963e+03_rb,0.11768e+04_rb /)
      kbo(:, 3,42,15) = (/ &
        & 0.10852e+04_rb,0.82427e+03_rb,0.60332e+03_rb,0.88798e+03_rb,0.11746e+04_rb /)
      kbo(:, 4,42,15) = (/ &
        & 0.10360e+04_rb,0.83188e+03_rb,0.59965e+03_rb,0.88636e+03_rb,0.11712e+04_rb /)
      kbo(:, 5,42,15) = (/ &
        & 0.10125e+04_rb,0.74885e+03_rb,0.59702e+03_rb,0.88356e+03_rb,0.11676e+04_rb /)
      kbo(:, 1,43,15) = (/ &
        & 0.91082e+03_rb,0.72514e+03_rb,0.60320e+03_rb,0.89075e+03_rb,0.11796e+04_rb /)
      kbo(:, 2,43,15) = (/ &
        & 0.82029e+03_rb,0.80137e+03_rb,0.60335e+03_rb,0.88998e+03_rb,0.11773e+04_rb /)
      kbo(:, 3,43,15) = (/ &
        & 0.11298e+04_rb,0.78950e+03_rb,0.60243e+03_rb,0.88859e+03_rb,0.11754e+04_rb /)
      kbo(:, 4,43,15) = (/ &
        & 0.10717e+04_rb,0.65904e+03_rb,0.60090e+03_rb,0.88726e+03_rb,0.11724e+04_rb /)
      kbo(:, 5,43,15) = (/ &
        & 0.89290e+03_rb,0.78779e+03_rb,0.59823e+03_rb,0.88240e+03_rb,0.11774e+04_rb /)
      kbo(:, 1,44,15) = (/ &
        & 0.10292e+04_rb,0.72935e+03_rb,0.60266e+03_rb,0.89093e+03_rb,0.11785e+04_rb /)
      kbo(:, 2,44,15) = (/ &
        & 0.10616e+04_rb,0.76323e+03_rb,0.60355e+03_rb,0.89030e+03_rb,0.11790e+04_rb /)
      kbo(:, 3,44,15) = (/ &
        & 0.10058e+04_rb,0.82082e+03_rb,0.60215e+03_rb,0.88915e+03_rb,0.11762e+04_rb /)
      kbo(:, 4,44,15) = (/ &
        & 0.96216e+03_rb,0.76914e+03_rb,0.60084e+03_rb,0.88814e+03_rb,0.11735e+04_rb /)
      kbo(:, 5,44,15) = (/ &
        & 0.10069e+04_rb,0.76998e+03_rb,0.59893e+03_rb,0.88624e+03_rb,0.11672e+04_rb /)
      kbo(:, 1,45,15) = (/ &
        & 0.92082e+03_rb,0.79879e+03_rb,0.60350e+03_rb,0.89110e+03_rb,0.11788e+04_rb /)
      kbo(:, 2,45,15) = (/ &
        & 0.96551e+03_rb,0.79963e+03_rb,0.60307e+03_rb,0.89055e+03_rb,0.11780e+04_rb /)
      kbo(:, 3,45,15) = (/ &
        & 0.10337e+04_rb,0.76320e+03_rb,0.60246e+03_rb,0.88962e+03_rb,0.11768e+04_rb /)
      kbo(:, 4,45,15) = (/ &
        & 0.97219e+03_rb,0.72677e+03_rb,0.60201e+03_rb,0.88795e+03_rb,0.11746e+04_rb /)
      kbo(:, 5,45,15) = (/ &
        & 0.10146e+04_rb,0.80892e+03_rb,0.59962e+03_rb,0.88632e+03_rb,0.11711e+04_rb /)
      kbo(:, 1,46,15) = (/ &
        & 0.10772e+04_rb,0.81655e+03_rb,0.60383e+03_rb,0.89135e+03_rb,0.11792e+04_rb /)
      kbo(:, 2,46,15) = (/ &
        & 0.98671e+03_rb,0.73523e+03_rb,0.60321e+03_rb,0.89077e+03_rb,0.11796e+04_rb /)
      kbo(:, 3,46,15) = (/ &
        & 0.10348e+04_rb,0.84026e+03_rb,0.60272e+03_rb,0.89002e+03_rb,0.11773e+04_rb /)
      kbo(:, 4,46,15) = (/ &
        & 0.11018e+04_rb,0.66661e+03_rb,0.60312e+03_rb,0.88865e+03_rb,0.11755e+04_rb /)
      kbo(:, 5,46,15) = (/ &
        & 0.10012e+04_rb,0.71404e+03_rb,0.60161e+03_rb,0.88639e+03_rb,0.11725e+04_rb /)
      kbo(:, 1,47,15) = (/ &
        & 0.10560e+04_rb,0.78520e+03_rb,0.60429e+03_rb,0.89072e+03_rb,0.11796e+04_rb /)
      kbo(:, 2,47,15) = (/ &
        & 0.10627e+04_rb,0.82754e+03_rb,0.60335e+03_rb,0.89097e+03_rb,0.11773e+04_rb /)
      kbo(:, 3,47,15) = (/ &
        & 0.10705e+04_rb,0.76570e+03_rb,0.60295e+03_rb,0.89038e+03_rb,0.11791e+04_rb /)
      kbo(:, 4,47,15) = (/ &
        & 0.11051e+04_rb,0.83346e+03_rb,0.60225e+03_rb,0.89026e+03_rb,0.11789e+04_rb /)
      kbo(:, 5,47,15) = (/ &
        & 0.10627e+04_rb,0.79957e+03_rb,0.60100e+03_rb,0.88742e+03_rb,0.11738e+04_rb /)
      kbo(:, 1,48,15) = (/ &
        & 0.10526e+04_rb,0.83431e+03_rb,0.60478e+03_rb,0.89205e+03_rb,0.11801e+04_rb /)
      kbo(:, 2,48,15) = (/ &
        & 0.11375e+04_rb,0.74537e+03_rb,0.60363e+03_rb,0.89121e+03_rb,0.11790e+04_rb /)
      kbo(:, 3,48,15) = (/ &
        & 0.10571e+04_rb,0.77077e+03_rb,0.60313e+03_rb,0.89066e+03_rb,0.11782e+04_rb /)
      kbo(:, 4,48,15) = (/ &
        & 0.10694e+04_rb,0.73604e+03_rb,0.60258e+03_rb,0.88981e+03_rb,0.11770e+04_rb /)
      kbo(:, 5,48,15) = (/ &
        & 0.11062e+04_rb,0.66828e+03_rb,0.60158e+03_rb,0.88925e+03_rb,0.11750e+04_rb /)
      kbo(:, 1,49,15) = (/ &
        & 0.10727e+04_rb,0.73859e+03_rb,0.60462e+03_rb,0.89249e+03_rb,0.11807e+04_rb /)
      kbo(:, 2,49,15) = (/ &
        & 0.11006e+04_rb,0.84446e+03_rb,0.60404e+03_rb,0.89152e+03_rb,0.11794e+04_rb /)
      kbo(:, 3,49,15) = (/ &
        & 0.96096e+03_rb,0.73858e+03_rb,0.60393e+03_rb,0.89088e+03_rb,0.11785e+04_rb /)
      kbo(:, 4,49,15) = (/ &
        & 0.11140e+04_rb,0.80464e+03_rb,0.60285e+03_rb,0.89021e+03_rb,0.11776e+04_rb /)
      kbo(:, 5,49,15) = (/ &
        & 0.10928e+04_rb,0.75297e+03_rb,0.60205e+03_rb,0.88996e+03_rb,0.11760e+04_rb /)
      kbo(:, 1,50,15) = (/ &
        & 0.10626e+04_rb,0.84953e+03_rb,0.60572e+03_rb,0.89295e+03_rb,0.11814e+04_rb /)
      kbo(:, 2,50,15) = (/ &
        & 0.11017e+04_rb,0.71824e+03_rb,0.60451e+03_rb,0.89185e+03_rb,0.11811e+04_rb /)
      kbo(:, 3,50,15) = (/ &
        & 0.10336e+04_rb,0.71570e+03_rb,0.60345e+03_rb,0.89107e+03_rb,0.11800e+04_rb /)
      kbo(:, 4,50,15) = (/ &
        & 0.95200e+03_rb,0.76482e+03_rb,0.60304e+03_rb,0.89051e+03_rb,0.11793e+04_rb /)
      kbo(:, 5,50,15) = (/ &
        & 0.96988e+03_rb,0.83512e+03_rb,0.60306e+03_rb,0.88954e+03_rb,0.11767e+04_rb /)
      kbo(:, 1,51,15) = (/ &
        & 0.11017e+04_rb,0.77329e+03_rb,0.60615e+03_rb,0.89371e+03_rb,0.11825e+04_rb /)
      kbo(:, 2,51,15) = (/ &
        & 0.10950e+04_rb,0.81479e+03_rb,0.60496e+03_rb,0.89123e+03_rb,0.11790e+04_rb /)
      kbo(:, 3,51,15) = (/ &
        & 0.99893e+03_rb,0.69452e+03_rb,0.60377e+03_rb,0.89131e+03_rb,0.11791e+04_rb /)
      kbo(:, 4,51,15) = (/ &
        & 0.10704e+04_rb,0.82834e+03_rb,0.60319e+03_rb,0.89074e+03_rb,0.11783e+04_rb /)
      kbo(:, 5,51,15) = (/ &
        & 0.10391e+04_rb,0.80716e+03_rb,0.60333e+03_rb,0.89092e+03_rb,0.11772e+04_rb /)
      kbo(:, 1,52,15) = (/ &
        & 0.10358e+04_rb,0.83681e+03_rb,0.60593e+03_rb,0.89390e+03_rb,0.11828e+04_rb /)
      kbo(:, 2,52,15) = (/ &
        & 0.95423e+03_rb,0.78599e+03_rb,0.60540e+03_rb,0.89262e+03_rb,0.11809e+04_rb /)
      kbo(:, 3,52,15) = (/ &
        & 0.98439e+03_rb,0.83257e+03_rb,0.60417e+03_rb,0.89064e+03_rb,0.11795e+04_rb /)
      kbo(:, 4,52,15) = (/ &
        & 0.10548e+04_rb,0.77074e+03_rb,0.60331e+03_rb,0.89093e+03_rb,0.11786e+04_rb /)
      kbo(:, 5,52,15) = (/ &
        & 0.10358e+04_rb,0.81478e+03_rb,0.60291e+03_rb,0.89030e+03_rb,0.11777e+04_rb /)
      kbo(:, 1,53,15) = (/ &
        & 0.96204e+03_rb,0.74448e+03_rb,0.60716e+03_rb,0.89629e+03_rb,0.11860e+04_rb /)
      kbo(:, 2,53,15) = (/ &
        & 0.10872e+04_rb,0.75126e+03_rb,0.60583e+03_rb,0.89310e+03_rb,0.11816e+04_rb /)
      kbo(:, 3,53,15) = (/ &
        & 0.90617e+03_rb,0.75210e+03_rb,0.60463e+03_rb,0.89194e+03_rb,0.11800e+04_rb /)
      kbo(:, 4,53,15) = (/ &
        & 0.97991e+03_rb,0.76565e+03_rb,0.60352e+03_rb,0.89113e+03_rb,0.11788e+04_rb /)
      kbo(:, 5,53,15) = (/ &
        & 0.10425e+04_rb,0.76988e+03_rb,0.60308e+03_rb,0.89058e+03_rb,0.11793e+04_rb /)
      kbo(:, 1,54,15) = (/ &
        & 0.10112e+04_rb,0.55645e+03_rb,0.60785e+03_rb,0.89766e+03_rb,0.11878e+04_rb /)
      kbo(:, 2,54,15) = (/ &
        & 0.10637e+04_rb,0.66232e+03_rb,0.60623e+03_rb,0.89389e+03_rb,0.11827e+04_rb /)
      kbo(:, 3,54,15) = (/ &
        & 0.88717e+03_rb,0.63014e+03_rb,0.60503e+03_rb,0.89226e+03_rb,0.11804e+04_rb /)
      kbo(:, 4,54,15) = (/ &
        & 0.10369e+04_rb,0.84103e+03_rb,0.60384e+03_rb,0.89232e+03_rb,0.11792e+04_rb /)
      kbo(:, 5,54,15) = (/ &
        & 0.10000e+04_rb,0.70043e+03_rb,0.60321e+03_rb,0.89174e+03_rb,0.11783e+04_rb /)
      kbo(:, 1,55,15) = (/ &
        & 0.10615e+04_rb,0.77920e+03_rb,0.60858e+03_rb,0.89903e+03_rb,0.11896e+04_rb /)
      kbo(:, 2,55,15) = (/ &
        & 0.10425e+04_rb,0.78513e+03_rb,0.60661e+03_rb,0.89494e+03_rb,0.11829e+04_rb /)
      kbo(:, 3,55,15) = (/ &
        & 0.10916e+04_rb,0.73854e+03_rb,0.60542e+03_rb,0.89361e+03_rb,0.11809e+04_rb /)
      kbo(:, 4,55,15) = (/ &
        & 0.10715e+04_rb,0.78936e+03_rb,0.60420e+03_rb,0.89162e+03_rb,0.11795e+04_rb /)
      kbo(:, 5,55,15) = (/ &
        & 0.96091e+03_rb,0.82578e+03_rb,0.60397e+03_rb,0.89094e+03_rb,0.11773e+04_rb /)
      kbo(:, 1,56,15) = (/ &
        & 0.10458e+04_rb,0.77242e+03_rb,0.60942e+03_rb,0.90044e+03_rb,0.11915e+04_rb /)
      kbo(:, 2,56,15) = (/ &
        & 0.11062e+04_rb,0.77581e+03_rb,0.60712e+03_rb,0.89619e+03_rb,0.11859e+04_rb /)
      kbo(:, 3,56,15) = (/ &
        & 0.10883e+04_rb,0.80121e+03_rb,0.60580e+03_rb,0.89306e+03_rb,0.11815e+04_rb /)
      kbo(:, 4,56,15) = (/ &
        & 0.10726e+04_rb,0.82069e+03_rb,0.60459e+03_rb,0.89192e+03_rb,0.11799e+04_rb /)
      kbo(:, 5,56,15) = (/ &
        & 0.11263e+04_rb,0.78597e+03_rb,0.60416e+03_rb,0.89111e+03_rb,0.11788e+04_rb /)
      kbo(:, 1,57,15) = (/ &
        & 0.11095e+04_rb,0.76988e+03_rb,0.61034e+03_rb,0.90190e+03_rb,0.11935e+04_rb /)
      kbo(:, 2,57,15) = (/ &
        & 0.97543e+03_rb,0.72583e+03_rb,0.60777e+03_rb,0.89749e+03_rb,0.11876e+04_rb /)
      kbo(:, 3,57,15) = (/ &
        & 0.98548e+03_rb,0.78597e+03_rb,0.60618e+03_rb,0.89378e+03_rb,0.11826e+04_rb /)
      kbo(:, 4,57,15) = (/ &
        & 0.98213e+03_rb,0.81053e+03_rb,0.60499e+03_rb,0.89222e+03_rb,0.11804e+04_rb /)
      kbo(:, 5,57,15) = (/ &
        & 0.10760e+04_rb,0.81137e+03_rb,0.60380e+03_rb,0.89133e+03_rb,0.11791e+04_rb /)
      kbo(:, 1,58,15) = (/ &
        & 0.10034e+04_rb,0.71059e+03_rb,0.61128e+03_rb,0.90333e+03_rb,0.11954e+04_rb /)
      kbo(:, 2,58,15) = (/ &
        & 0.97431e+03_rb,0.80290e+03_rb,0.60845e+03_rb,0.89878e+03_rb,0.11893e+04_rb /)
      kbo(:, 3,58,15) = (/ &
        & 0.95308e+03_rb,0.77157e+03_rb,0.60654e+03_rb,0.89474e+03_rb,0.11839e+04_rb /)
      kbo(:, 4,58,15) = (/ &
        & 0.10737e+04_rb,0.87404e+03_rb,0.60535e+03_rb,0.89257e+03_rb,0.11808e+04_rb /)
      kbo(:, 5,58,15) = (/ &
        & 0.10626e+04_rb,0.67671e+03_rb,0.60413e+03_rb,0.89157e+03_rb,0.11782e+04_rb /)
      kbo(:, 1,59,15) = (/ &
        & 0.98325e+03_rb,0.78258e+03_rb,0.61167e+03_rb,0.90392e+03_rb,0.11962e+04_rb /)
      kbo(:, 2,59,15) = (/ &
        & 0.10481e+04_rb,0.74785e+03_rb,0.60874e+03_rb,0.89931e+03_rb,0.11900e+04_rb /)
      kbo(:, 3,59,15) = (/ &
        & 0.11151e+04_rb,0.73006e+03_rb,0.60669e+03_rb,0.89518e+03_rb,0.11820e+04_rb /)
      kbo(:, 4,59,15) = (/ &
        & 0.10425e+04_rb,0.79358e+03_rb,0.60550e+03_rb,0.89369e+03_rb,0.11810e+04_rb /)
      kbo(:, 5,59,15) = (/ &
        & 0.97877e+03_rb,0.79866e+03_rb,0.60428e+03_rb,0.89168e+03_rb,0.11796e+04_rb /)
      kbo(:, 1,13,16) = (/ &
        & 0.75523e+03_rb,0.86786e+03_rb,0.60838e+03_rb,0.89385e+03_rb,0.11815e+04_rb /)
      kbo(:, 2,13,16) = (/ &
        & 0.10580e+03_rb,0.52074e+03_rb,0.60685e+03_rb,0.89272e+03_rb,0.11800e+04_rb /)
      kbo(:, 3,13,16) = (/ &
        & 0.88450e+03_rb,0.99478e+02_rb,0.60069e+03_rb,0.89217e+03_rb,0.11793e+04_rb /)
      kbo(:, 4,13,16) = (/ &
        & 0.47883e+03_rb,0.85714e+02_rb,0.60445e+03_rb,0.89133e+03_rb,0.11782e+04_rb /)
      kbo(:, 5,13,16) = (/ &
        & 0.30313e+03_rb,0.39357e+02_rb,0.60363e+03_rb,0.89015e+03_rb,0.11767e+04_rb /)
      kbo(:, 1,14,16) = (/ &
        & 0.59250e+03_rb,0.12538e+03_rb,0.60779e+03_rb,0.89359e+03_rb,0.11813e+04_rb /)
      kbo(:, 2,14,16) = (/ &
        & 0.45021e+03_rb,0.30819e+03_rb,0.60630e+03_rb,0.89255e+03_rb,0.11799e+04_rb /)
      kbo(:, 3,14,16) = (/ &
        & 0.38746e+03_rb,0.31955e+03_rb,0.60482e+03_rb,0.89198e+03_rb,0.11792e+04_rb /)
      kbo(:, 4,14,16) = (/ &
        & 0.59123e+03_rb,0.24014e+03_rb,0.60415e+03_rb,0.89112e+03_rb,0.11781e+04_rb /)
      kbo(:, 5,14,16) = (/ &
        & 0.47780e+03_rb,0.41362e+03_rb,0.60332e+03_rb,0.88306e+03_rb,0.11765e+04_rb /)
      kbo(:, 1,15,16) = (/ &
        & 0.10202e+04_rb,0.29221e+03_rb,0.60727e+03_rb,0.88647e+03_rb,0.11812e+04_rb /)
      kbo(:, 2,15,16) = (/ &
        & 0.55435e+03_rb,0.58788e+03_rb,0.60581e+03_rb,0.89239e+03_rb,0.11799e+04_rb /)
      kbo(:, 3,15,16) = (/ &
        & 0.31413e+03_rb,0.75889e+03_rb,0.60450e+03_rb,0.89181e+03_rb,0.11791e+04_rb /)
      kbo(:, 4,15,16) = (/ &
        & 0.41436e+03_rb,0.80717e+03_rb,0.60389e+03_rb,0.89092e+03_rb,0.11689e+04_rb /)
      kbo(:, 5,15,16) = (/ &
        & 0.84269e+03_rb,0.27451e+03_rb,0.60303e+03_rb,0.88966e+03_rb,0.11582e+04_rb /)
      kbo(:, 1,16,16) = (/ &
        & 0.19456e+03_rb,0.24351e+03_rb,0.60682e+03_rb,0.89311e+03_rb,0.11719e+04_rb /)
      kbo(:, 2,16,16) = (/ &
        & 0.43800e+03_rb,0.21696e+03_rb,0.60538e+03_rb,0.89225e+03_rb,0.11798e+04_rb /)
      kbo(:, 3,16,16) = (/ &
        & 0.73851e+03_rb,0.22888e+03_rb,0.60428e+03_rb,0.89164e+03_rb,0.11790e+04_rb /)
      kbo(:, 4,16,16) = (/ &
        & 0.83650e+03_rb,0.13309e+03_rb,0.60366e+03_rb,0.89073e+03_rb,0.11778e+04_rb /)
      kbo(:, 5,16,16) = (/ &
        & 0.65895e+03_rb,0.36022e+03_rb,0.59813e+03_rb,0.88942e+03_rb,0.11670e+04_rb /)
      kbo(:, 1,17,16) = (/ &
        & 0.97890e+03_rb,0.27257e+03_rb,0.60643e+03_rb,0.89290e+03_rb,0.11808e+04_rb /)
      kbo(:, 2,17,16) = (/ &
        & 0.21678e+03_rb,0.36639e+03_rb,0.60501e+03_rb,0.89213e+03_rb,0.11797e+04_rb /)
      kbo(:, 3,17,16) = (/ &
        & 0.36576e+03_rb,0.64243e+03_rb,0.60409e+03_rb,0.89150e+03_rb,0.11789e+04_rb /)
      kbo(:, 4,17,16) = (/ &
        & 0.71301e+03_rb,0.32666e+03_rb,0.60345e+03_rb,0.89056e+03_rb,0.11777e+04_rb /)
      kbo(:, 5,17,16) = (/ &
        & 0.30605e+03_rb,0.68315e+03_rb,0.59789e+03_rb,0.88918e+03_rb,0.11668e+04_rb /)
      kbo(:, 1,18,16) = (/ &
        & 0.52238e+03_rb,0.17591e+03_rb,0.60609e+03_rb,0.89271e+03_rb,0.11807e+04_rb /)
      kbo(:, 2,18,16) = (/ &
        & 0.10253e+04_rb,0.75807e+03_rb,0.60468e+03_rb,0.89202e+03_rb,0.11797e+04_rb /)
      kbo(:, 3,18,16) = (/ &
        & 0.23095e+03_rb,0.31874e+03_rb,0.60392e+03_rb,0.89136e+03_rb,0.11788e+04_rb /)
      kbo(:, 4,18,16) = (/ &
        & 0.55144e+03_rb,0.80452e+03_rb,0.59863e+03_rb,0.89039e+03_rb,0.11685e+04_rb /)
      kbo(:, 5,18,16) = (/ &
        & 0.36145e+03_rb,0.21165e+03_rb,0.60231e+03_rb,0.88211e+03_rb,0.11666e+04_rb /)
      kbo(:, 1,19,16) = (/ &
        & 0.10124e+04_rb,0.51143e+03_rb,0.60113e+03_rb,0.89253e+03_rb,0.11805e+04_rb /)
      kbo(:, 2,19,16) = (/ &
        & 0.54199e+03_rb,0.26758e+03_rb,0.60439e+03_rb,0.89192e+03_rb,0.11796e+04_rb /)
      kbo(:, 3,19,16) = (/ &
        & 0.61467e+03_rb,0.78253e+03_rb,0.59913e+03_rb,0.89123e+03_rb,0.11787e+04_rb /)
      kbo(:, 4,19,16) = (/ &
        & 0.58906e+03_rb,0.74398e+02_rb,0.60311e+03_rb,0.89023e+03_rb,0.11774e+04_rb /)
      kbo(:, 5,19,16) = (/ &
        & 0.98025e+03_rb,0.28476e+03_rb,0.59747e+03_rb,0.88189e+03_rb,0.11663e+04_rb /)
      kbo(:, 1,20,16) = (/ &
        & 0.25361e+03_rb,0.16739e+03_rb,0.60550e+03_rb,0.89234e+03_rb,0.11712e+04_rb /)
      kbo(:, 2,20,16) = (/ &
        & 0.70227e+03_rb,0.71784e+03_rb,0.59948e+03_rb,0.89182e+03_rb,0.11795e+04_rb /)
      kbo(:, 3,20,16) = (/ &
        & 0.89681e+03_rb,0.24714e+03_rb,0.60364e+03_rb,0.89109e+03_rb,0.11786e+04_rb /)
      kbo(:, 4,20,16) = (/ &
        & 0.30123e+03_rb,0.54911e+03_rb,0.60295e+03_rb,0.89006e+03_rb,0.11772e+04_rb /)
      kbo(:, 5,20,16) = (/ &
        & 0.68301e+03_rb,0.36973e+03_rb,0.59725e+03_rb,0.88846e+03_rb,0.11750e+04_rb /)
      kbo(:, 1,21,16) = (/ &
        & 0.25236e+03_rb,0.80202e+03_rb,0.60523e+03_rb,0.89218e+03_rb,0.11801e+04_rb /)
      kbo(:, 2,21,16) = (/ &
        & 0.77221e+03_rb,0.62834e+03_rb,0.60401e+03_rb,0.89171e+03_rb,0.11794e+04_rb /)
      kbo(:, 3,21,16) = (/ &
        & 0.70626e+03_rb,0.20918e+03_rb,0.60351e+03_rb,0.89096e+03_rb,0.11784e+04_rb /)
      kbo(:, 4,21,16) = (/ &
        & 0.16217e+03_rb,0.14134e+03_rb,0.59816e+03_rb,0.88303e+03_rb,0.11770e+04_rb /)
      kbo(:, 5,21,16) = (/ &
        & 0.51843e+03_rb,0.20259e+03_rb,0.59703e+03_rb,0.88818e+03_rb,0.11747e+04_rb /)
      kbo(:, 1,22,16) = (/ &
        & 0.21890e+03_rb,0.24577e+03_rb,0.60023e+03_rb,0.89209e+03_rb,0.11799e+04_rb /)
      kbo(:, 2,22,16) = (/ &
        & 0.30771e+03_rb,0.55233e+03_rb,0.60388e+03_rb,0.89157e+03_rb,0.11793e+04_rb /)
      kbo(:, 3,22,16) = (/ &
        & 0.17796e+03_rb,0.48432e+03_rb,0.60334e+03_rb,0.89076e+03_rb,0.11782e+04_rb /)
      kbo(:, 4,22,16) = (/ &
        & 0.88899e+03_rb,0.36751e+03_rb,0.60258e+03_rb,0.88960e+03_rb,0.11676e+04_rb /)
      kbo(:, 5,22,16) = (/ &
        & 0.37953e+03_rb,0.56918e+03_rb,0.58746e+03_rb,0.88775e+03_rb,0.11742e+04_rb /)
      kbo(:, 1,23,16) = (/ &
        & 0.42018e+03_rb,0.21434e+03_rb,0.60456e+03_rb,0.89198e+03_rb,0.11798e+04_rb /)
      kbo(:, 2,23,16) = (/ &
        & 0.10252e+04_rb,0.32427e+03_rb,0.60375e+03_rb,0.89142e+03_rb,0.11791e+04_rb /)
      kbo(:, 3,23,16) = (/ &
        & 0.25811e+03_rb,0.61743e+03_rb,0.60318e+03_rb,0.89055e+03_rb,0.11779e+04_rb /)
      kbo(:, 4,23,16) = (/ &
        & 0.60449e+03_rb,0.28100e+03_rb,0.60235e+03_rb,0.88930e+03_rb,0.11762e+04_rb /)
      kbo(:, 5,23,16) = (/ &
        & 0.43495e+03_rb,0.18925e+03_rb,0.60100e+03_rb,0.88727e+03_rb,0.11735e+04_rb /)
      kbo(:, 1,24,16) = (/ &
        & 0.68497e+03_rb,0.77568e+03_rb,0.60423e+03_rb,0.89188e+03_rb,0.11797e+04_rb /)
      kbo(:, 2,24,16) = (/ &
        & 0.27378e+03_rb,0.43332e+03_rb,0.60362e+03_rb,0.89125e+03_rb,0.11789e+04_rb /)
      kbo(:, 3,24,16) = (/ &
        & 0.44258e+03_rb,0.48182e+03_rb,0.60301e+03_rb,0.89032e+03_rb,0.11686e+04_rb /)
      kbo(:, 4,24,16) = (/ &
        & 0.38603e+03_rb,0.34135e+03_rb,0.59747e+03_rb,0.88895e+03_rb,0.11758e+04_rb /)
      kbo(:, 5,24,16) = (/ &
        & 0.39386e+03_rb,0.54222e+03_rb,0.60062e+03_rb,0.88674e+03_rb,0.11638e+04_rb /)
      kbo(:, 1,25,16) = (/ &
        & 0.55482e+03_rb,0.53022e+03_rb,0.60394e+03_rb,0.89175e+03_rb,0.11796e+04_rb /)
      kbo(:, 2,25,16) = (/ &
        & 0.90002e+03_rb,0.91361e+02_rb,0.60348e+03_rb,0.89107e+03_rb,0.11787e+04_rb /)
      kbo(:, 3,25,16) = (/ &
        & 0.17670e+03_rb,0.51744e+03_rb,0.59819e+03_rb,0.89007e+03_rb,0.11773e+04_rb /)
      kbo(:, 4,25,16) = (/ &
        & 0.44153e+03_rb,0.41376e+03_rb,0.59256e+03_rb,0.88854e+03_rb,0.11753e+04_rb /)
      kbo(:, 5,25,16) = (/ &
        & 0.24073e+03_rb,0.21288e+03_rb,0.60020e+03_rb,0.88614e+03_rb,0.11721e+04_rb /)
      kbo(:, 1,26,16) = (/ &
        & 0.37725e+03_rb,0.61445e+03_rb,0.60383e+03_rb,0.89161e+03_rb,0.11703e+04_rb /)
      kbo(:, 2,26,16) = (/ &
        & 0.10269e+04_rb,0.42565e+03_rb,0.59868e+03_rb,0.89085e+03_rb,0.11784e+04_rb /)
      kbo(:, 3,26,16) = (/ &
        & 0.49719e+03_rb,0.26744e+03_rb,0.60261e+03_rb,0.88977e+03_rb,0.11769e+04_rb /)
      kbo(:, 4,26,16) = (/ &
        & 0.41684e+03_rb,0.19443e+03_rb,0.59685e+03_rb,0.88123e+03_rb,0.11746e+04_rb /)
      kbo(:, 5,26,16) = (/ &
        & 0.32855e+03_rb,0.57095e+03_rb,0.59973e+03_rb,0.88545e+03_rb,0.11622e+04_rb /)
      kbo(:, 1,27,16) = (/ &
        & 0.60914e+03_rb,0.57105e+03_rb,0.60371e+03_rb,0.89145e+03_rb,0.11792e+04_rb /)
      kbo(:, 2,27,16) = (/ &
        & 0.26439e+03_rb,0.15182e+03_rb,0.60316e+03_rb,0.89062e+03_rb,0.11781e+04_rb /)
      kbo(:, 3,27,16) = (/ &
        & 0.73685e+03_rb,0.69813e+03_rb,0.60238e+03_rb,0.88259e+03_rb,0.11765e+04_rb /)
      kbo(:, 4,27,16) = (/ &
        & 0.64054e+03_rb,0.28524e+03_rb,0.60110e+03_rb,0.88751e+03_rb,0.11649e+04_rb /)
      kbo(:, 5,27,16) = (/ &
        & 0.75244e+03_rb,0.50359e+03_rb,0.59461e+03_rb,0.87791e+03_rb,0.11702e+04_rb /)
      kbo(:, 1,28,16) = (/ &
        & 0.64046e+03_rb,0.15170e+03_rb,0.60358e+03_rb,0.89127e+03_rb,0.11790e+04_rb /)
      kbo(:, 2,28,16) = (/ &
        & 0.70429e+03_rb,0.23052e+03_rb,0.60299e+03_rb,0.89037e+03_rb,0.11687e+04_rb /)
      kbo(:, 3,28,16) = (/ &
        & 0.44006e+03_rb,0.41240e+03_rb,0.60212e+03_rb,0.88904e+03_rb,0.11760e+04_rb /)
      kbo(:, 4,28,16) = (/ &
        & 0.22398e+03_rb,0.59419e+03_rb,0.60069e+03_rb,0.88691e+03_rb,0.11731e+04_rb /)
      kbo(:, 5,28,16) = (/ &
        & 0.55183e+03_rb,0.13336e+03_rb,0.59871e+03_rb,0.87038e+03_rb,0.11692e+04_rb /)
      kbo(:, 1,29,16) = (/ &
        & 0.59987e+03_rb,0.49106e+03_rb,0.60344e+03_rb,0.89107e+03_rb,0.11787e+04_rb /)
      kbo(:, 2,29,16) = (/ &
        & 0.34384e+03_rb,0.96976e+02_rb,0.60280e+03_rb,0.89009e+03_rb,0.11774e+04_rb /)
      kbo(:, 3,29,16) = (/ &
        & 0.54363e+03_rb,0.20603e+03_rb,0.60181e+03_rb,0.88860e+03_rb,0.11754e+04_rb /)
      kbo(:, 4,29,16) = (/ &
        & 0.10391e+04_rb,0.38775e+03_rb,0.60023e+03_rb,0.88623e+03_rb,0.11632e+04_rb /)
      kbo(:, 5,29,16) = (/ &
        & 0.22377e+03_rb,0.17567e+03_rb,0.59820e+03_rb,0.88324e+03_rb,0.11684e+04_rb /)
      kbo(:, 1,30,16) = (/ &
        & 0.63138e+03_rb,0.68455e+03_rb,0.60329e+03_rb,0.89085e+03_rb,0.11784e+04_rb /)
      kbo(:, 2,30,16) = (/ &
        & 0.58332e+03_rb,0.21199e+03_rb,0.60259e+03_rb,0.88978e+03_rb,0.11770e+04_rb /)
      kbo(:, 3,30,16) = (/ &
        & 0.33555e+03_rb,0.46024e+03_rb,0.59221e+03_rb,0.88809e+03_rb,0.11657e+04_rb /)
      kbo(:, 4,30,16) = (/ &
        & 0.44733e+03_rb,0.32696e+03_rb,0.59512e+03_rb,0.88550e+03_rb,0.11713e+04_rb /)
      kbo(:, 5,30,16) = (/ &
        & 0.79071e+03_rb,0.72043e+03_rb,0.59773e+03_rb,0.85548e+03_rb,0.11586e+04_rb /)
      kbo(:, 1,31,16) = (/ &
        & 0.81471e+03_rb,0.68414e+03_rb,0.60312e+03_rb,0.89060e+03_rb,0.11781e+04_rb /)
      kbo(:, 2,31,16) = (/ &
        & 0.44722e+03_rb,0.73246e+03_rb,0.60235e+03_rb,0.88942e+03_rb,0.11765e+04_rb /)
      kbo(:, 3,31,16) = (/ &
        & 0.48709e+03_rb,0.24210e+03_rb,0.60108e+03_rb,0.88068e+03_rb,0.11739e+04_rb /)
      kbo(:, 4,31,16) = (/ &
        & 0.63873e+03_rb,0.56888e+03_rb,0.59920e+03_rb,0.88472e+03_rb,0.11702e+04_rb /)
      kbo(:, 5,31,16) = (/ &
        & 0.72647e+03_rb,0.26020e+03_rb,0.59739e+03_rb,0.78035e+03_rb,0.11669e+04_rb /)
      kbo(:, 1,32,16) = (/ &
        & 0.10219e+04_rb,0.29652e+03_rb,0.60294e+03_rb,0.89034e+03_rb,0.11777e+04_rb /)
      kbo(:, 2,32,16) = (/ &
        & 0.75833e+03_rb,0.13311e+03_rb,0.59281e+03_rb,0.88900e+03_rb,0.11759e+04_rb /)
      kbo(:, 3,32,16) = (/ &
        & 0.54275e+03_rb,0.31460e+03_rb,0.59602e+03_rb,0.88686e+03_rb,0.11731e+04_rb /)
      kbo(:, 4,32,16) = (/ &
        & 0.28731e+03_rb,0.15124e+03_rb,0.59865e+03_rb,0.87712e+03_rb,0.11692e+04_rb /)
      kbo(:, 5,32,16) = (/ &
        & 0.79004e+03_rb,0.63514e+03_rb,0.58791e+03_rb,0.86135e+03_rb,0.11214e+04_rb /)
      kbo(:, 1,33,16) = (/ &
        & 0.42295e+03_rb,0.52022e+03_rb,0.60275e+03_rb,0.88319e+03_rb,0.11773e+04_rb /)
      kbo(:, 2,33,16) = (/ &
        & 0.72614e+03_rb,0.33266e+03_rb,0.60175e+03_rb,0.88169e+03_rb,0.11753e+04_rb /)
      kbo(:, 3,33,16) = (/ &
        & 0.36703e+03_rb,0.22982e+03_rb,0.59554e+03_rb,0.88614e+03_rb,0.11721e+04_rb /)
      kbo(:, 4,33,16) = (/ &
        & 0.23137e+03_rb,0.32656e+03_rb,0.59813e+03_rb,0.88315e+03_rb,0.11503e+04_rb /)
      kbo(:, 5,33,16) = (/ &
        & 0.98125e+03_rb,0.60471e+03_rb,0.59681e+03_rb,0.86770e+03_rb,0.11478e+04_rb /)
      kbo(:, 1,34,16) = (/ &
        & 0.44676e+03_rb,0.16327e+03_rb,0.59792e+03_rb,0.88976e+03_rb,0.11770e+04_rb /)
      kbo(:, 2,34,16) = (/ &
        & 0.27122e+03_rb,0.52002e+03_rb,0.60144e+03_rb,0.88807e+03_rb,0.11747e+04_rb /)
      kbo(:, 3,34,16) = (/ &
        & 0.89338e+03_rb,0.37487e+03_rb,0.59509e+03_rb,0.87866e+03_rb,0.11622e+04_rb /)
      kbo(:, 4,34,16) = (/ &
        & 0.43071e+03_rb,0.56832e+03_rb,0.58850e+03_rb,0.83509e+03_rb,0.10958e+04_rb /)
      kbo(:, 5,34,16) = (/ &
        & 0.20737e+03_rb,0.35064e+03_rb,0.59655e+03_rb,0.88088e+03_rb,0.11652e+04_rb /)
      kbo(:, 1,35,16) = (/ &
        & 0.79758e+02_rb,0.10278e+03_rb,0.60244e+03_rb,0.87589e+03_rb,0.11767e+04_rb /)
      kbo(:, 2,35,16) = (/ &
        & 0.52637e+03_rb,0.56826e+03_rb,0.60124e+03_rb,0.88778e+03_rb,0.11743e+04_rb /)
      kbo(:, 3,35,16) = (/ &
        & 0.43064e+03_rb,0.29620e+03_rb,0.59943e+03_rb,0.88508e+03_rb,0.11707e+04_rb /)
      kbo(:, 4,35,16) = (/ &
        & 0.21531e+03_rb,0.74954e+03_rb,0.57453e+03_rb,0.86197e+03_rb,0.11223e+04_rb /)
      kbo(:, 5,35,16) = (/ &
        & 0.85323e+03_rb,0.27200e+03_rb,0.59640e+03_rb,0.88066e+03_rb,0.11649e+04_rb /)
      kbo(:, 1,36,16) = (/ &
        & 0.99678e+03_rb,0.21760e+03_rb,0.60242e+03_rb,0.88955e+03_rb,0.11767e+04_rb /)
      kbo(:, 2,36,16) = (/ &
        & 0.90902e+03_rb,0.40496e+03_rb,0.59196e+03_rb,0.88090e+03_rb,0.11742e+04_rb /)
      kbo(:, 3,36,16) = (/ &
        & 0.38273e+03_rb,0.60439e+03_rb,0.59478e+03_rb,0.87820e+03_rb,0.11706e+04_rb /)
      kbo(:, 4,36,16) = (/ &
        & 0.84516e+03_rb,0.25383e+03_rb,0.58829e+03_rb,0.79406e+03_rb,0.11312e+04_rb /)
      kbo(:, 5,36,16) = (/ &
        & 0.45445e+03_rb,0.41700e+03_rb,0.59637e+03_rb,0.88062e+03_rb,0.11559e+04_rb /)
      kbo(:, 1,37,16) = (/ &
        & 0.62987e+03_rb,0.44723e+03_rb,0.59790e+03_rb,0.88973e+03_rb,0.11769e+04_rb /)
      kbo(:, 2,37,16) = (/ &
        & 0.80524e+03_rb,0.68894e+03_rb,0.59677e+03_rb,0.88802e+03_rb,0.11746e+04_rb /)
      kbo(:, 3,37,16) = (/ &
        & 0.18337e+03_rb,0.45323e+03_rb,0.59504e+03_rb,0.88541e+03_rb,0.11712e+04_rb /)
      kbo(:, 4,37,16) = (/ &
        & 0.50224e+03_rb,0.16920e+03_rb,0.59765e+03_rb,0.86220e+03_rb,0.11586e+04_rb /)
      kbo(:, 5,37,16) = (/ &
        & 0.85299e+03_rb,0.22358e+03_rb,0.59652e+03_rb,0.88084e+03_rb,0.11652e+04_rb /)
      kbo(:, 1,38,16) = (/ &
        & 0.91678e+03_rb,0.47738e+03_rb,0.59801e+03_rb,0.88990e+03_rb,0.11681e+04_rb /)
      kbo(:, 2,38,16) = (/ &
        & 0.47033e+03_rb,0.39881e+03_rb,0.59234e+03_rb,0.88147e+03_rb,0.11750e+04_rb /)
      kbo(:, 3,38,16) = (/ &
        & 0.69352e+03_rb,0.66467e+03_rb,0.59993e+03_rb,0.87901e+03_rb,0.11717e+04_rb /)
      kbo(:, 4,38,16) = (/ &
        & 0.48625e+03_rb,0.32628e+03_rb,0.59330e+03_rb,0.86251e+03_rb,0.11590e+04_rb /)
      kbo(:, 5,38,16) = (/ &
        & 0.31087e+03_rb,0.44108e+03_rb,0.58749e+03_rb,0.88107e+03_rb,0.11655e+04_rb /)
      kbo(:, 1,39,16) = (/ &
        & 0.35870e+03_rb,0.11480e+03_rb,0.60275e+03_rb,0.89007e+03_rb,0.11774e+04_rb /)
      kbo(:, 2,39,16) = (/ &
        & 0.37464e+03_rb,0.78547e+03_rb,0.59714e+03_rb,0.88174e+03_rb,0.11754e+04_rb /)
      kbo(:, 3,39,16) = (/ &
        & 0.10123e+04_rb,0.30814e+03_rb,0.59558e+03_rb,0.87940e+03_rb,0.11722e+04_rb /)
      kbo(:, 4,39,16) = (/ &
        & 0.17535e+03_rb,0.96668e+02_rb,0.59817e+03_rb,0.88323e+03_rb,0.11594e+04_rb /)
      kbo(:, 5,39,16) = (/ &
        & 0.36664e+03_rb,0.56187e+03_rb,0.58764e+03_rb,0.86774e+03_rb,0.11658e+04_rb /)
      kbo(:, 1,40,16) = (/ &
        & 0.48620e+03_rb,0.57396e+03_rb,0.60293e+03_rb,0.89033e+03_rb,0.11777e+04_rb /)
      kbo(:, 2,40,16) = (/ &
        & 0.94847e+03_rb,0.62832e+03_rb,0.60206e+03_rb,0.88901e+03_rb,0.11760e+04_rb /)
      kbo(:, 3,40,16) = (/ &
        & 0.54197e+03_rb,0.51352e+03_rb,0.60063e+03_rb,0.88687e+03_rb,0.11731e+04_rb /)
      kbo(:, 4,40,16) = (/ &
        & 0.21519e+03_rb,0.61017e+03_rb,0.58944e+03_rb,0.86354e+03_rb,0.11692e+04_rb /)
      kbo(:, 5,40,16) = (/ &
        & 0.27098e+03_rb,0.33831e+03_rb,0.58331e+03_rb,0.88170e+03_rb,0.10587e+04_rb /)
      kbo(:, 1,41,16) = (/ &
        & 0.14346e+03_rb,0.26582e+03_rb,0.60309e+03_rb,0.89059e+03_rb,0.11781e+04_rb /)
      kbo(:, 2,41,16) = (/ &
        & 0.76510e+03_rb,0.70682e+03_rb,0.60232e+03_rb,0.88941e+03_rb,0.11765e+04_rb /)
      kbo(:, 3,41,16) = (/ &
        & 0.62163e+03_rb,0.16311e+03_rb,0.60105e+03_rb,0.88750e+03_rb,0.11649e+04_rb /)
      kbo(:, 4,41,16) = (/ &
        & 0.87665e+02_rb,0.19331e+03_rb,0.59457e+03_rb,0.88472e+03_rb,0.11703e+04_rb /)
      kbo(:, 5,41,16) = (/ &
        & 0.10041e+04_rb,0.44098e+03_rb,0.58818e+03_rb,0.72606e+03_rb,0.10412e+04_rb /)
      kbo(:, 1,42,16) = (/ &
        & 0.66944e+03_rb,0.41079e+03_rb,0.60325e+03_rb,0.89083e+03_rb,0.11693e+04_rb /)
      kbo(:, 2,42,16) = (/ &
        & 0.51801e+03_rb,0.25976e+03_rb,0.60255e+03_rb,0.88976e+03_rb,0.11770e+04_rb /)
      kbo(:, 3,42,16) = (/ &
        & 0.23111e+03_rb,0.27788e+03_rb,0.58293e+03_rb,0.88808e+03_rb,0.11747e+04_rb /)
      kbo(:, 4,42,16) = (/ &
        & 0.44627e+03_rb,0.35036e+03_rb,0.59971e+03_rb,0.87868e+03_rb,0.11713e+04_rb /)
      kbo(:, 5,42,16) = (/ &
        & 0.75706e+03_rb,0.77924e+03_rb,0.59310e+03_rb,0.87584e+03_rb,0.11676e+04_rb /)
      kbo(:, 1,43,16) = (/ &
        & 0.60566e+03_rb,0.10873e+03_rb,0.60342e+03_rb,0.89110e+03_rb,0.11697e+04_rb /)
      kbo(:, 2,43,16) = (/ &
        & 0.92441e+03_rb,0.64634e+03_rb,0.59816e+03_rb,0.89014e+03_rb,0.11775e+04_rb /)
      kbo(:, 3,43,16) = (/ &
        & 0.18329e+03_rb,0.35035e+03_rb,0.59722e+03_rb,0.88869e+03_rb,0.11755e+04_rb /)
      kbo(:, 4,43,16) = (/ &
        & 0.34266e+03_rb,0.17517e+03_rb,0.59569e+03_rb,0.87957e+03_rb,0.11725e+04_rb /)
      kbo(:, 5,43,16) = (/ &
        & 0.92438e+03_rb,0.90606e+02_rb,0.59829e+03_rb,0.88341e+03_rb,0.11057e+04_rb /)
      kbo(:, 1,44,16) = (/ &
        & 0.38251e+03_rb,0.19934e+03_rb,0.60359e+03_rb,0.89135e+03_rb,0.11791e+04_rb /)
      kbo(:, 2,44,16) = (/ &
        & 0.48609e+03_rb,0.71881e+03_rb,0.59839e+03_rb,0.89050e+03_rb,0.11689e+04_rb /)
      kbo(:, 3,44,16) = (/ &
        & 0.54984e+03_rb,0.19933e+03_rb,0.59759e+03_rb,0.88926e+03_rb,0.11763e+04_rb /)
      kbo(:, 4,44,16) = (/ &
        & 0.85264e+03_rb,0.64027e+03_rb,0.60090e+03_rb,0.88045e+03_rb,0.11736e+04_rb /)
      kbo(:, 5,44,16) = (/ &
        & 0.88451e+03_rb,0.49530e+03_rb,0.59898e+03_rb,0.87082e+03_rb,0.11699e+04_rb /)
      kbo(:, 1,45,16) = (/ &
        & 0.66936e+03_rb,0.51946e+03_rb,0.60373e+03_rb,0.89157e+03_rb,0.11794e+04_rb /)
      kbo(:, 2,45,16) = (/ &
        & 0.95622e+03_rb,0.66442e+03_rb,0.60324e+03_rb,0.89082e+03_rb,0.11784e+04_rb /)
      kbo(:, 3,45,16) = (/ &
        & 0.70122e+03_rb,0.25973e+03_rb,0.60254e+03_rb,0.88974e+03_rb,0.11769e+04_rb /)
      kbo(:, 4,45,16) = (/ &
        & 0.13546e+03_rb,0.47717e+03_rb,0.59679e+03_rb,0.88805e+03_rb,0.11747e+04_rb /)
      kbo(:, 5,45,16) = (/ &
        & 0.44623e+03_rb,0.47716e+03_rb,0.59968e+03_rb,0.87864e+03_rb,0.11712e+04_rb /)
      kbo(:, 1,46,16) = (/ &
        & 0.16734e+03_rb,0.11476e+03_rb,0.60389e+03_rb,0.89176e+03_rb,0.11797e+04_rb /)
      kbo(:, 2,46,16) = (/ &
        & 0.41435e+03_rb,0.59796e+03_rb,0.60344e+03_rb,0.89112e+03_rb,0.11697e+04_rb /)
      kbo(:, 3,46,16) = (/ &
        & 0.78886e+03_rb,0.13892e+03_rb,0.60282e+03_rb,0.89018e+03_rb,0.11775e+04_rb /)
      kbo(:, 4,46,16) = (/ &
        & 0.48606e+03_rb,0.76103e+03_rb,0.59263e+03_rb,0.88876e+03_rb,0.11756e+04_rb /)
      kbo(:, 5,46,16) = (/ &
        & 0.77291e+03_rb,0.68855e+03_rb,0.59113e+03_rb,0.88648e+03_rb,0.11726e+04_rb /)
      kbo(:, 1,47,16) = (/ &
        & 0.80479e+03_rb,0.70064e+03_rb,0.60443e+03_rb,0.89194e+03_rb,0.11799e+04_rb /)
      kbo(:, 2,47,16) = (/ &
        & 0.74901e+03_rb,0.35032e+03_rb,0.60363e+03_rb,0.89142e+03_rb,0.11792e+04_rb /)
      kbo(:, 3,47,16) = (/ &
        & 0.62948e+03_rb,0.48923e+03_rb,0.60309e+03_rb,0.89059e+03_rb,0.11690e+04_rb /)
      kbo(:, 4,47,16) = (/ &
        & 0.87648e+02_rb,0.36239e+03_rb,0.60232e+03_rb,0.88257e+03_rb,0.11584e+04_rb /)
      kbo(:, 5,47,16) = (/ &
        & 0.44621e+03_rb,0.42279e+03_rb,0.60106e+03_rb,0.88751e+03_rb,0.11740e+04_rb /)
      kbo(:, 1,48,16) = (/ &
        & 0.39044e+03_rb,0.31407e+03_rb,0.60497e+03_rb,0.89220e+03_rb,0.11803e+04_rb /)
      kbo(:, 2,48,16) = (/ &
        & 0.28685e+03_rb,0.50131e+03_rb,0.60379e+03_rb,0.89166e+03_rb,0.11795e+04_rb /)
      kbo(:, 3,48,16) = (/ &
        & 0.38246e+03_rb,0.70665e+03_rb,0.60333e+03_rb,0.89096e+03_rb,0.11786e+04_rb /)
      kbo(:, 4,48,16) = (/ &
        & 0.47011e+03_rb,0.61002e+03_rb,0.60267e+03_rb,0.88995e+03_rb,0.11772e+04_rb /)
      kbo(:, 5,48,16) = (/ &
        & 0.36652e+03_rb,0.63417e+03_rb,0.60164e+03_rb,0.88155e+03_rb,0.11751e+04_rb /)
      kbo(:, 1,49,16) = (/ &
        & 0.76492e+03_rb,0.96636e+02_rb,0.60550e+03_rb,0.89270e+03_rb,0.11810e+04_rb /)
      kbo(:, 2,49,16) = (/ &
        & 0.23107e+03_rb,0.19931e+03_rb,0.60416e+03_rb,0.89186e+03_rb,0.11798e+04_rb /)
      kbo(:, 3,49,16) = (/ &
        & 0.86850e+03_rb,0.30199e+03_rb,0.59890e+03_rb,0.89128e+03_rb,0.11790e+04_rb /)
      kbo(:, 4,49,16) = (/ &
        & 0.27091e+03_rb,0.22951e+03_rb,0.60296e+03_rb,0.89040e+03_rb,0.11778e+04_rb /)
      kbo(:, 5,49,16) = (/ &
        & 0.45416e+03_rb,0.53149e+03_rb,0.60212e+03_rb,0.88227e+03_rb,0.11761e+04_rb /)
      kbo(:, 1,50,16) = (/ &
        & 0.39042e+03_rb,0.12079e+03_rb,0.60601e+03_rb,0.89316e+03_rb,0.11816e+04_rb /)
      kbo(:, 2,50,16) = (/ &
        & 0.38245e+03_rb,0.73080e+03_rb,0.60467e+03_rb,0.89201e+03_rb,0.11709e+04_rb /)
      kbo(:, 3,50,16) = (/ &
        & 0.19919e+03_rb,0.50733e+03_rb,0.60370e+03_rb,0.89153e+03_rb,0.11703e+04_rb /)
      kbo(:, 4,50,16) = (/ &
        & 0.52587e+03_rb,0.41673e+03_rb,0.60320e+03_rb,0.89076e+03_rb,0.11693e+04_rb /)
      kbo(:, 5,50,16) = (/ &
        & 0.25497e+03_rb,0.90594e+02_rb,0.59785e+03_rb,0.88966e+03_rb,0.11768e+04_rb /)
      kbo(:, 1,51,16) = (/ &
        & 0.35058e+03_rb,0.56772e+03_rb,0.60650e+03_rb,0.89405e+03_rb,0.11830e+04_rb /)
      kbo(:, 2,51,16) = (/ &
        & 0.39839e+03_rb,0.52545e+03_rb,0.60516e+03_rb,0.89238e+03_rb,0.11806e+04_rb /)
      kbo(:, 3,51,16) = (/ &
        & 0.69319e+03_rb,0.65831e+03_rb,0.60384e+03_rb,0.89174e+03_rb,0.11796e+04_rb /)
      kbo(:, 4,51,16) = (/ &
        & 0.66132e+03_rb,0.42881e+03_rb,0.60341e+03_rb,0.89108e+03_rb,0.11788e+04_rb /)
      kbo(:, 5,51,16) = (/ &
        & 0.80473e+03_rb,0.49524e+03_rb,0.59814e+03_rb,0.88327e+03_rb,0.11775e+04_rb /)
      kbo(:, 1,52,16) = (/ &
        & 0.75693e+03_rb,0.32010e+03_rb,0.60698e+03_rb,0.89551e+03_rb,0.11849e+04_rb /)
      kbo(:, 2,52,16) = (/ &
        & 0.98002e+03_rb,0.74890e+03_rb,0.60565e+03_rb,0.89283e+03_rb,0.11812e+04_rb /)
      kbo(:, 3,52,16) = (/ &
        & 0.42228e+03_rb,0.32613e+03_rb,0.60430e+03_rb,0.89190e+03_rb,0.11799e+04_rb /)
      kbo(:, 4,52,16) = (/ &
        & 0.43025e+03_rb,0.70662e+03_rb,0.60359e+03_rb,0.89136e+03_rb,0.11791e+04_rb /)
      kbo(:, 5,52,16) = (/ &
        & 0.81269e+03_rb,0.56167e+03_rb,0.60303e+03_rb,0.89051e+03_rb,0.11780e+04_rb /)
      kbo(:, 1,53,16) = (/ &
        & 0.98002e+03_rb,0.61603e+03_rb,0.60747e+03_rb,0.89705e+03_rb,0.11870e+04_rb /)
      kbo(:, 2,53,16) = (/ &
        & 0.28683e+03_rb,0.42277e+03_rb,0.60613e+03_rb,0.89328e+03_rb,0.11818e+04_rb /)
      kbo(:, 3,53,16) = (/ &
        & 0.45415e+03_rb,0.68246e+03_rb,0.60479e+03_rb,0.89204e+03_rb,0.11801e+04_rb /)
      kbo(:, 4,53,16) = (/ &
        & 0.78879e+03_rb,0.48920e+03_rb,0.60374e+03_rb,0.89159e+03_rb,0.11794e+04_rb /)
      kbo(:, 5,53,16) = (/ &
        & 0.76488e+03_rb,0.14495e+03_rb,0.60326e+03_rb,0.89085e+03_rb,0.11694e+04_rb /)
      kbo(:, 1,54,16) = (/ &
        & 0.67724e+03_rb,0.76701e+03_rb,0.60808e+03_rb,0.89852e+03_rb,0.11890e+04_rb /)
      kbo(:, 2,54,16) = (/ &
        & 0.81269e+03_rb,0.24762e+03_rb,0.60658e+03_rb,0.89430e+03_rb,0.11833e+04_rb /)
      kbo(:, 3,54,16) = (/ &
        & 0.94813e+03_rb,0.63414e+03_rb,0.60524e+03_rb,0.89246e+03_rb,0.11807e+04_rb /)
      kbo(:, 4,54,16) = (/ &
        & 0.49398e+03_rb,0.28989e+03_rb,0.60390e+03_rb,0.88491e+03_rb,0.11797e+04_rb /)
      kbo(:, 5,54,16) = (/ &
        & 0.89236e+03_rb,0.27781e+03_rb,0.60344e+03_rb,0.88428e+03_rb,0.11788e+04_rb /)
      kbo(:, 1,55,16) = (/ &
        & 0.52586e+03_rb,0.77305e+03_rb,0.60906e+03_rb,0.89999e+03_rb,0.11909e+04_rb /)
      kbo(:, 2,55,16) = (/ &
        & 0.54179e+03_rb,0.64018e+03_rb,0.60701e+03_rb,0.89560e+03_rb,0.11851e+04_rb /)
      kbo(:, 3,55,16) = (/ &
        & 0.50992e+03_rb,0.60394e+03_rb,0.60567e+03_rb,0.88598e+03_rb,0.11812e+04_rb /)
      kbo(:, 4,55,16) = (/ &
        & 0.63740e+03_rb,0.44088e+03_rb,0.60433e+03_rb,0.89191e+03_rb,0.11799e+04_rb /)
      kbo(:, 5,55,16) = (/ &
        & 0.49398e+03_rb,0.37444e+03_rb,0.59895e+03_rb,0.89137e+03_rb,0.11791e+04_rb /)
      kbo(:, 1,56,16) = (/ &
        & 0.90829e+03_rb,0.27177e+03_rb,0.61007e+03_rb,0.90151e+03_rb,0.11929e+04_rb /)
      kbo(:, 2,56,16) = (/ &
        & 0.35854e+03_rb,0.24158e+03_rb,0.60744e+03_rb,0.89694e+03_rb,0.11869e+04_rb /)
      kbo(:, 3,56,16) = (/ &
        & 0.59756e+03_rb,0.47107e+03_rb,0.60610e+03_rb,0.89325e+03_rb,0.11817e+04_rb /)
      kbo(:, 4,56,16) = (/ &
        & 0.52585e+03_rb,0.35632e+03_rb,0.60476e+03_rb,0.89203e+03_rb,0.11801e+04_rb /)
      kbo(:, 5,56,16) = (/ &
        & 0.27886e+03_rb,0.24761e+03_rb,0.59909e+03_rb,0.89157e+03_rb,0.11794e+04_rb /)
      kbo(:, 1,57,16) = (/ &
        & 0.34260e+03_rb,0.66433e+03_rb,0.61112e+03_rb,0.90308e+03_rb,0.11950e+04_rb /)
      kbo(:, 2,57,16) = (/ &
        & 0.66130e+03_rb,0.45295e+03_rb,0.60796e+03_rb,0.89834e+03_rb,0.11887e+04_rb /)
      kbo(:, 3,57,16) = (/ &
        & 0.69317e+03_rb,0.42880e+03_rb,0.60653e+03_rb,0.89415e+03_rb,0.11831e+04_rb /)
      kbo(:, 4,57,16) = (/ &
        & 0.27089e+03_rb,0.28385e+03_rb,0.60519e+03_rb,0.89241e+03_rb,0.11806e+04_rb /)
      kbo(:, 5,57,16) = (/ &
        & 0.66926e+03_rb,0.34424e+03_rb,0.60385e+03_rb,0.89175e+03_rb,0.11797e+04_rb /)
      kbo(:, 1,58,16) = (/ &
        & 0.96406e+03_rb,0.25365e+03_rb,0.61214e+03_rb,0.90462e+03_rb,0.11971e+04_rb /)
      kbo(:, 2,58,16) = (/ &
        & 0.31073e+03_rb,0.28385e+03_rb,0.60888e+03_rb,0.89972e+03_rb,0.11906e+04_rb /)
      kbo(:, 3,58,16) = (/ &
        & 0.96405e+03_rb,0.57374e+03_rb,0.60693e+03_rb,0.89536e+03_rb,0.11848e+04_rb /)
      kbo(:, 4,58,16) = (/ &
        & 0.69316e+03_rb,0.13891e+03_rb,0.60560e+03_rb,0.89278e+03_rb,0.11811e+04_rb /)
      kbo(:, 5,58,16) = (/ &
        & 0.59755e+03_rb,0.75492e+03_rb,0.60426e+03_rb,0.89189e+03_rb,0.11798e+04_rb /)
      kbo(:, 1,59,16) = (/ &
        & 0.92422e+03_rb,0.39860e+03_rb,0.61256e+03_rb,0.90525e+03_rb,0.11979e+04_rb /)
      kbo(:, 2,59,16) = (/ &
        & 0.78877e+03_rb,0.42275e+03_rb,0.60926e+03_rb,0.90029e+03_rb,0.11913e+04_rb /)
      kbo(:, 3,59,16) = (/ &
        & 0.35853e+03_rb,0.65225e+03_rb,0.60710e+03_rb,0.89586e+03_rb,0.11854e+04_rb /)
      kbo(:, 4,59,16) = (/ &
        & 0.54975e+03_rb,0.50730e+03_rb,0.60576e+03_rb,0.88606e+03_rb,0.11813e+04_rb /)
      kbo(:, 5,59,16) = (/ &
        & 0.52585e+03_rb,0.62205e+03_rb,0.60442e+03_rb,0.89194e+03_rb,0.11799e+04_rb /)
     
      end subroutine sw_kgb28

! **************************************************************************
      subroutine sw_kgb29
! **************************************************************************

      use parkind, only : im => kind_im, rb => kind_rb 
      use rrsw_kg29, only : kao, kbo, selfrefo, forrefo, sfluxrefo, &
                            absh2oo, absco2o, rayl, &
                            irradnceo, facbrghto, snsptdrko

      implicit none
      real :: irradscl
      save

! Kurucz solar source function
      sfluxrefo(:) = (/ &
        & 1.32880_rb    , 2.14018_rb    , 1.97612_rb    , 1.79000_rb    , &
        & 1.51242_rb    , 1.22977_rb    , 1.06052_rb    , 0.800996_rb   , &
        & 0.748053_rb   , 8.64369e-02_rb, 7.10675e-02_rb, 5.62425e-02_rb, &
        & 4.46988e-02_rb, 3.07441e-02_rb, 1.16728e-02_rb, 1.65573e-03_rb /)

! Solar variability components: time-invariant baseline quiet sun irradiance
      irradnceo(:) = (/ &
        & 3.07026E+00_rb, 1.17711E+00_rb, 1.40732E+00_rb, 1.72794E+00_rb,&
        & 1.50898E+00_rb, 1.22020E+00_rb, 1.05592E+00_rb, 8.60695E-01_rb,&
        & 5.41761E-01_rb, 5.89856E-02_rb, 4.94552E-02_rb, 3.89977E-02_rb,&
        & 2.42702E-02_rb, 1.50606E-02_rb, 5.90834E-03_rb, 8.87187E-04_rb/)
! Apply scaling to irradnceo in this band to account for solar function 
! below 820 cm-1 (scaling quantities were derived from high-resolution data)
      irradscl = 13.221_rb / (13.221_rb - 0.455_rb)
      irradnceo(:) = irradscl * irradnceo(:)
! Solar variability components: facular brightening
      facbrghto(:) = (/ &
        & 1.53283E-03_rb, 9.63733E-04_rb, 1.36406E-03_rb, 1.60915E-03_rb,&
        & 1.24673E-03_rb, 8.97788E-04_rb, 7.17956E-04_rb, 5.39162E-04_rb,&
        & 3.47312E-04_rb, 3.76054E-05_rb, 3.11846E-05_rb, 2.41484E-05_rb,&
        & 1.66126E-05_rb, 1.03199E-05_rb, 4.02661E-06_rb, 6.15065E-07_rb/)
! Solar variability components: sunspot darkening
      snsptdrko(:) = (/ &
        &-3.53619E-04_rb,-1.42336E-04_rb,-1.93668E-04_rb,-2.39345E-04_rb,&
        &-1.94758E-04_rb,-1.46675E-04_rb,-1.24160E-04_rb,-9.96834E-05_rb,&
        &-6.22416E-05_rb,-6.75856E-06_rb,-5.65172E-06_rb,-4.42212E-06_rb,&
        &-2.75547E-06_rb,-1.69966E-06_rb,-6.69789E-07_rb,-1.01641E-07_rb/)

      absco2o(:) = (/ &
        & 2.90073e-06_rb, 2.12382e-05_rb, 1.03032e-04_rb, 1.86481e-04_rb, &
        & 4.31997e-04_rb, 6.08238e-04_rb, 2.17603e-03_rb, 4.64479e-02_rb, &
        & 2.96956_rb    , 14.9569_rb    , 28.4831_rb    , 61.3998_rb    , &
        & 164.129_rb    , 832.282_rb    , 4995.02_rb    , 12678.1_rb     /)

      absh2oo(:) = (/ &
        & 2.99508e-04_rb, 3.95012e-03_rb, 1.49316e-02_rb, 3.24384e-02_rb, &
        & 6.92879e-02_rb, 0.123523_rb   , 0.360985_rb   , 1.86434_rb    , &
        & 10.38157_rb   , 0.214129_rb   , 0.213914_rb   , 0.212781_rb   , &
        & 0.215562_rb   , 0.218087_rb   , 0.220918_rb   , 0.218546_rb    /)
           
! Rayleigh extinction coefficient at v = 2200 cm-1.
      rayl = 9.30e-11_rb

!     ------------------------------------------------------------------
!     The array KAO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels> ~100mb, temperatures, and binary
!     species parameters (see taumol.f for definition).  The first 
!     index in the array, JS, runs from 1 to 9, and corresponds to 
!     different values of the binary species parameter.  For instance, 
!     JS=1 refers to dry air, JS = 2 corresponds to the paramter value 1/8, 
!     JS = 3 corresponds to the parameter value 2/8, etc.  The second index
!     in the array, JT, which runs from 1 to 5, corresponds to different
!     temperatures.  More specifically, JT = 3 means that the data are for
!     the reference temperature TREF for this  pressure level, JT = 2 refers
!     to TREF-15, JT = 1 is for TREF-30, JT = 4 is for TREF+15, and JT = 5
!     is for TREF+30.  The third index, JP, runs from 1 to 13 and refers
!     to the JPth reference pressure level (see taumol.f for these levels
!     in mb).  The fourth index, IG, goes from 1 to 16, and indicates
!     which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kao(:, 1, 1) = (/ &
        & 0.11565e-03_rb,0.10123e-03_rb,0.90804e-04_rb,0.82282e-04_rb,0.71083e-04_rb /)
      kao(:, 2, 1) = (/ &
        & 0.96434e-04_rb,0.82830e-04_rb,0.72366e-04_rb,0.61803e-04_rb,0.52497e-04_rb /)
      kao(:, 3, 1) = (/ &
        & 0.64539e-04_rb,0.56659e-04_rb,0.46605e-04_rb,0.39815e-04_rb,0.37118e-04_rb /)
      kao(:, 4, 1) = (/ &
        & 0.34417e-04_rb,0.27113e-04_rb,0.25362e-04_rb,0.30345e-04_rb,0.36275e-04_rb /)
      kao(:, 5, 1) = (/ &
        & 0.12260e-04_rb,0.15856e-04_rb,0.20834e-04_rb,0.27363e-04_rb,0.36114e-04_rb /)
      kao(:, 6, 1) = (/ &
        & 0.10221e-04_rb,0.14598e-04_rb,0.19695e-04_rb,0.25976e-04_rb,0.33702e-04_rb /)
      kao(:, 7, 1) = (/ &
        & 0.97563e-05_rb,0.13809e-04_rb,0.20231e-04_rb,0.27238e-04_rb,0.36110e-04_rb /)
      kao(:, 8, 1) = (/ &
        & 0.14062e-04_rb,0.19587e-04_rb,0.27069e-04_rb,0.36937e-04_rb,0.49415e-04_rb /)
      kao(:, 9, 1) = (/ &
        & 0.36371e-04_rb,0.48122e-04_rb,0.61586e-04_rb,0.77647e-04_rb,0.99897e-04_rb /)
      kao(:,10, 1) = (/ &
        & 0.99203e-04_rb,0.12842e-03_rb,0.16588e-03_rb,0.20834e-03_rb,0.26000e-03_rb /)
      kao(:,11, 1) = (/ &
        & 0.13233e-03_rb,0.17318e-03_rb,0.22059e-03_rb,0.28904e-03_rb,0.36062e-03_rb /)
      kao(:,12, 1) = (/ &
        & 0.13379e-03_rb,0.17484e-03_rb,0.23687e-03_rb,0.30286e-03_rb,0.37504e-03_rb /)
      kao(:,13, 1) = (/ &
        & 0.11740e-03_rb,0.15667e-03_rb,0.20962e-03_rb,0.26768e-03_rb,0.33485e-03_rb /)
      kao(:, 1, 2) = (/ &
        & 0.10246e-03_rb,0.10450e-03_rb,0.97383e-04_rb,0.96398e-04_rb,0.10549e-03_rb /)
      kao(:, 2, 2) = (/ &
        & 0.10589e-03_rb,0.10240e-03_rb,0.95801e-04_rb,0.98509e-04_rb,0.11993e-03_rb /)
      kao(:, 3, 2) = (/ &
        & 0.94054e-04_rb,0.87009e-04_rb,0.10941e-03_rb,0.13486e-03_rb,0.15918e-03_rb /)
      kao(:, 4, 2) = (/ &
        & 0.11883e-03_rb,0.14236e-03_rb,0.16636e-03_rb,0.18235e-03_rb,0.19785e-03_rb /)
      kao(:, 5, 2) = (/ &
        & 0.17800e-03_rb,0.19347e-03_rb,0.20977e-03_rb,0.22730e-03_rb,0.25111e-03_rb /)
      kao(:, 6, 2) = (/ &
        & 0.22243e-03_rb,0.24157e-03_rb,0.26567e-03_rb,0.28549e-03_rb,0.30723e-03_rb /)
      kao(:, 7, 2) = (/ &
        & 0.29248e-03_rb,0.32242e-03_rb,0.34635e-03_rb,0.37915e-03_rb,0.40803e-03_rb /)
      kao(:, 8, 2) = (/ &
        & 0.43386e-03_rb,0.48611e-03_rb,0.52681e-03_rb,0.56812e-03_rb,0.60642e-03_rb /)
      kao(:, 9, 2) = (/ &
        & 0.89109e-03_rb,0.10345e-02_rb,0.11794e-02_rb,0.13045e-02_rb,0.14303e-02_rb /)
      kao(:,10, 2) = (/ &
        & 0.21538e-02_rb,0.24459e-02_rb,0.27329e-02_rb,0.30932e-02_rb,0.35253e-02_rb /)
      kao(:,11, 2) = (/ &
        & 0.29272e-02_rb,0.32676e-02_rb,0.36353e-02_rb,0.40062e-02_rb,0.45441e-02_rb /)
      kao(:,12, 2) = (/ &
        & 0.30762e-02_rb,0.34365e-02_rb,0.38146e-02_rb,0.41556e-02_rb,0.47164e-02_rb /)
      kao(:,13, 2) = (/ &
        & 0.27808e-02_rb,0.31114e-02_rb,0.34383e-02_rb,0.37954e-02_rb,0.42576e-02_rb /)
      kao(:, 1, 3) = (/ &
        & 0.24042e-03_rb,0.32719e-03_rb,0.44370e-03_rb,0.56836e-03_rb,0.70357e-03_rb /)
      kao(:, 2, 3) = (/ &
        & 0.23303e-03_rb,0.31901e-03_rb,0.41926e-03_rb,0.52400e-03_rb,0.62365e-03_rb /)
      kao(:, 3, 3) = (/ &
        & 0.35050e-03_rb,0.42158e-03_rb,0.47490e-03_rb,0.53390e-03_rb,0.60411e-03_rb /)
      kao(:, 4, 3) = (/ &
        & 0.49954e-03_rb,0.53067e-03_rb,0.56338e-03_rb,0.60859e-03_rb,0.66569e-03_rb /)
      kao(:, 5, 3) = (/ &
        & 0.66908e-03_rb,0.69727e-03_rb,0.72898e-03_rb,0.76756e-03_rb,0.80358e-03_rb /)
      kao(:, 6, 3) = (/ &
        & 0.88634e-03_rb,0.92853e-03_rb,0.96856e-03_rb,0.10101e-02_rb,0.10556e-02_rb /)
      kao(:, 7, 3) = (/ &
        & 0.11659e-02_rb,0.12355e-02_rb,0.13238e-02_rb,0.13889e-02_rb,0.14491e-02_rb /)
      kao(:, 8, 3) = (/ &
        & 0.17485e-02_rb,0.18223e-02_rb,0.19411e-02_rb,0.20705e-02_rb,0.22025e-02_rb /)
      kao(:, 9, 3) = (/ &
        & 0.42442e-02_rb,0.43561e-02_rb,0.45061e-02_rb,0.47505e-02_rb,0.50358e-02_rb /)
      kao(:,10, 3) = (/ &
        & 0.10940e-01_rb,0.11499e-01_rb,0.11906e-01_rb,0.12570e-01_rb,0.12803e-01_rb /)
      kao(:,11, 3) = (/ &
        & 0.14287e-01_rb,0.15010e-01_rb,0.15581e-01_rb,0.16054e-01_rb,0.16609e-01_rb /)
      kao(:,12, 3) = (/ &
        & 0.14856e-01_rb,0.15546e-01_rb,0.16074e-01_rb,0.16478e-01_rb,0.17057e-01_rb /)
      kao(:,13, 3) = (/ &
        & 0.13257e-01_rb,0.13834e-01_rb,0.14155e-01_rb,0.14723e-01_rb,0.15095e-01_rb /)
      kao(:, 1, 4) = (/ &
        & 0.24391e-02_rb,0.28720e-02_rb,0.33497e-02_rb,0.38926e-02_rb,0.45054e-02_rb /)
      kao(:, 2, 4) = (/ &
        & 0.22506e-02_rb,0.26004e-02_rb,0.30002e-02_rb,0.34490e-02_rb,0.39579e-02_rb /)
      kao(:, 3, 4) = (/ &
        & 0.22153e-02_rb,0.25711e-02_rb,0.29320e-02_rb,0.33330e-02_rb,0.37728e-02_rb /)
      kao(:, 4, 4) = (/ &
        & 0.22483e-02_rb,0.25445e-02_rb,0.28745e-02_rb,0.32484e-02_rb,0.36554e-02_rb /)
      kao(:, 5, 4) = (/ &
        & 0.23388e-02_rb,0.25985e-02_rb,0.28914e-02_rb,0.32377e-02_rb,0.36113e-02_rb /)
      kao(:, 6, 4) = (/ &
        & 0.24669e-02_rb,0.26579e-02_rb,0.28834e-02_rb,0.31538e-02_rb,0.34593e-02_rb /)
      kao(:, 7, 4) = (/ &
        & 0.32536e-02_rb,0.33413e-02_rb,0.34375e-02_rb,0.35905e-02_rb,0.38010e-02_rb /)
      kao(:, 8, 4) = (/ &
        & 0.51228e-02_rb,0.51967e-02_rb,0.52820e-02_rb,0.53446e-02_rb,0.53797e-02_rb /)
      kao(:, 9, 4) = (/ &
        & 0.13029e-01_rb,0.13065e-01_rb,0.12891e-01_rb,0.12848e-01_rb,0.12777e-01_rb /)
      kao(:,10, 4) = (/ &
        & 0.29911e-01_rb,0.30117e-01_rb,0.28340e-01_rb,0.27321e-01_rb,0.27485e-01_rb /)
      kao(:,11, 4) = (/ &
        & 0.36663e-01_rb,0.36877e-01_rb,0.34688e-01_rb,0.33004e-01_rb,0.32437e-01_rb /)
      kao(:,12, 4) = (/ &
        & 0.37282e-01_rb,0.37775e-01_rb,0.34910e-01_rb,0.33463e-01_rb,0.33074e-01_rb /)
      kao(:,13, 4) = (/ &
        & 0.33412e-01_rb,0.32961e-01_rb,0.30323e-01_rb,0.29437e-01_rb,0.29572e-01_rb /)
      kao(:, 1, 5) = (/ &
        & 0.20792e-01_rb,0.22727e-01_rb,0.25207e-01_rb,0.27650e-01_rb,0.29866e-01_rb /)
      kao(:, 2, 5) = (/ &
        & 0.17515e-01_rb,0.19421e-01_rb,0.21509e-01_rb,0.23389e-01_rb,0.25418e-01_rb /)
      kao(:, 3, 5) = (/ &
        & 0.14912e-01_rb,0.16535e-01_rb,0.18159e-01_rb,0.19873e-01_rb,0.21718e-01_rb /)
      kao(:, 4, 5) = (/ &
        & 0.13498e-01_rb,0.14786e-01_rb,0.16253e-01_rb,0.17812e-01_rb,0.19494e-01_rb /)
      kao(:, 5, 5) = (/ &
        & 0.12840e-01_rb,0.13983e-01_rb,0.15257e-01_rb,0.16617e-01_rb,0.18015e-01_rb /)
      kao(:, 6, 5) = (/ &
        & 0.12557e-01_rb,0.13714e-01_rb,0.14939e-01_rb,0.16145e-01_rb,0.17356e-01_rb /)
      kao(:, 7, 5) = (/ &
        & 0.11839e-01_rb,0.12951e-01_rb,0.14090e-01_rb,0.15237e-01_rb,0.16396e-01_rb /)
      kao(:, 8, 5) = (/ &
        & 0.13190e-01_rb,0.13998e-01_rb,0.14783e-01_rb,0.15641e-01_rb,0.16632e-01_rb /)
      kao(:, 9, 5) = (/ &
        & 0.24479e-01_rb,0.24660e-01_rb,0.25206e-01_rb,0.25571e-01_rb,0.26098e-01_rb /)
      kao(:,10, 5) = (/ &
        & 0.62373e-01_rb,0.60241e-01_rb,0.61372e-01_rb,0.61530e-01_rb,0.60978e-01_rb /)
      kao(:,11, 5) = (/ &
        & 0.76394e-01_rb,0.73727e-01_rb,0.74701e-01_rb,0.75649e-01_rb,0.75608e-01_rb /)
      kao(:,12, 5) = (/ &
        & 0.76656e-01_rb,0.73608e-01_rb,0.75610e-01_rb,0.76362e-01_rb,0.76033e-01_rb /)
      kao(:,13, 5) = (/ &
        & 0.66025e-01_rb,0.64890e-01_rb,0.66873e-01_rb,0.66992e-01_rb,0.66685e-01_rb /)
      kao(:, 1, 6) = (/ &
        & 0.99799e-01_rb,0.10479e+00_rb,0.10918e+00_rb,0.11347e+00_rb,0.11767e+00_rb /)
      kao(:, 2, 6) = (/ &
        & 0.87946e-01_rb,0.91820e-01_rb,0.95208e-01_rb,0.99071e-01_rb,0.10286e+00_rb /)
      kao(:, 3, 6) = (/ &
        & 0.76753e-01_rb,0.80377e-01_rb,0.84204e-01_rb,0.87924e-01_rb,0.91370e-01_rb /)
      kao(:, 4, 6) = (/ &
        & 0.67002e-01_rb,0.70630e-01_rb,0.73736e-01_rb,0.76764e-01_rb,0.79671e-01_rb /)
      kao(:, 5, 6) = (/ &
        & 0.58933e-01_rb,0.61857e-01_rb,0.64756e-01_rb,0.67323e-01_rb,0.69871e-01_rb /)
      kao(:, 6, 6) = (/ &
        & 0.53143e-01_rb,0.55386e-01_rb,0.57507e-01_rb,0.59715e-01_rb,0.62263e-01_rb /)
      kao(:, 7, 6) = (/ &
        & 0.50856e-01_rb,0.52603e-01_rb,0.54414e-01_rb,0.56230e-01_rb,0.58483e-01_rb /)
      kao(:, 8, 6) = (/ &
        & 0.50528e-01_rb,0.52598e-01_rb,0.54634e-01_rb,0.56710e-01_rb,0.58872e-01_rb /)
      kao(:, 9, 6) = (/ &
        & 0.64616e-01_rb,0.65915e-01_rb,0.67210e-01_rb,0.68742e-01_rb,0.70092e-01_rb /)
      kao(:,10, 6) = (/ &
        & 0.11509e+00_rb,0.11448e+00_rb,0.11368e+00_rb,0.11348e+00_rb,0.11401e+00_rb /)
      kao(:,11, 6) = (/ &
        & 0.14266e+00_rb,0.13997e+00_rb,0.13810e+00_rb,0.13688e+00_rb,0.13622e+00_rb /)
      kao(:,12, 6) = (/ &
        & 0.14464e+00_rb,0.14198e+00_rb,0.13915e+00_rb,0.13789e+00_rb,0.13781e+00_rb /)
      kao(:,13, 6) = (/ &
        & 0.12550e+00_rb,0.12328e+00_rb,0.12173e+00_rb,0.12127e+00_rb,0.12078e+00_rb /)
      kao(:, 1, 7) = (/ &
        & 0.30825e+00_rb,0.31119e+00_rb,0.31335e+00_rb,0.31641e+00_rb,0.32024e+00_rb /)
      kao(:, 2, 7) = (/ &
        & 0.28100e+00_rb,0.28660e+00_rb,0.29192e+00_rb,0.29680e+00_rb,0.30014e+00_rb /)
      kao(:, 3, 7) = (/ &
        & 0.26374e+00_rb,0.27129e+00_rb,0.27887e+00_rb,0.28626e+00_rb,0.29478e+00_rb /)
      kao(:, 4, 7) = (/ &
        & 0.24539e+00_rb,0.25325e+00_rb,0.26070e+00_rb,0.26955e+00_rb,0.27794e+00_rb /)
      kao(:, 5, 7) = (/ &
        & 0.21737e+00_rb,0.22642e+00_rb,0.23531e+00_rb,0.24408e+00_rb,0.25322e+00_rb /)
      kao(:, 6, 7) = (/ &
        & 0.19066e+00_rb,0.19913e+00_rb,0.20834e+00_rb,0.21782e+00_rb,0.22698e+00_rb /)
      kao(:, 7, 7) = (/ &
        & 0.17386e+00_rb,0.18325e+00_rb,0.19211e+00_rb,0.20024e+00_rb,0.20881e+00_rb /)
      kao(:, 8, 7) = (/ &
        & 0.17625e+00_rb,0.18236e+00_rb,0.18905e+00_rb,0.19574e+00_rb,0.20193e+00_rb /)
      kao(:, 9, 7) = (/ &
        & 0.21820e+00_rb,0.22416e+00_rb,0.22986e+00_rb,0.23737e+00_rb,0.24518e+00_rb /)
      kao(:,10, 7) = (/ &
        & 0.33529e+00_rb,0.34302e+00_rb,0.35397e+00_rb,0.36397e+00_rb,0.37339e+00_rb /)
      kao(:,11, 7) = (/ &
        & 0.38474e+00_rb,0.39863e+00_rb,0.41168e+00_rb,0.42115e+00_rb,0.42917e+00_rb /)
      kao(:,12, 7) = (/ &
        & 0.38499e+00_rb,0.39580e+00_rb,0.40497e+00_rb,0.41279e+00_rb,0.42045e+00_rb /)
      kao(:,13, 7) = (/ &
        & 0.34291e+00_rb,0.35180e+00_rb,0.35970e+00_rb,0.36697e+00_rb,0.37460e+00_rb /)
      kao(:, 1, 8) = (/ &
        & 0.87131e+00_rb,0.87876e+00_rb,0.88509e+00_rb,0.89061e+00_rb,0.89432e+00_rb /)
      kao(:, 2, 8) = (/ &
        & 0.78963e+00_rb,0.79236e+00_rb,0.79978e+00_rb,0.80959e+00_rb,0.82163e+00_rb /)
      kao(:, 3, 8) = (/ &
        & 0.82479e+00_rb,0.82842e+00_rb,0.83069e+00_rb,0.83679e+00_rb,0.84334e+00_rb /)
      kao(:, 4, 8) = (/ &
        & 0.90877e+00_rb,0.91962e+00_rb,0.92466e+00_rb,0.93139e+00_rb,0.93904e+00_rb /)
      kao(:, 5, 8) = (/ &
        & 0.97131e+00_rb,0.99301e+00_rb,0.10095e+01_rb,0.10270e+01_rb,0.10371e+01_rb /)
      kao(:, 6, 8) = (/ &
        & 0.96984e+00_rb,0.10040e+01_rb,0.10393e+01_rb,0.10670e+01_rb,0.10990e+01_rb /)
      kao(:, 7, 8) = (/ &
        & 0.95686e+00_rb,0.10029e+01_rb,0.10449e+01_rb,0.10923e+01_rb,0.11334e+01_rb /)
      kao(:, 8, 8) = (/ &
        & 0.97227e+00_rb,0.10380e+01_rb,0.11014e+01_rb,0.11624e+01_rb,0.12218e+01_rb /)
      kao(:, 9, 8) = (/ &
        & 0.12290e+01_rb,0.13071e+01_rb,0.14005e+01_rb,0.15005e+01_rb,0.15980e+01_rb /)
      kao(:,10, 8) = (/ &
        & 0.17996e+01_rb,0.19304e+01_rb,0.20752e+01_rb,0.22219e+01_rb,0.23814e+01_rb /)
      kao(:,11, 8) = (/ &
        & 0.19289e+01_rb,0.20656e+01_rb,0.22166e+01_rb,0.23715e+01_rb,0.25386e+01_rb /)
      kao(:,12, 8) = (/ &
        & 0.18429e+01_rb,0.19765e+01_rb,0.21175e+01_rb,0.22722e+01_rb,0.24239e+01_rb /)
      kao(:,13, 8) = (/ &
        & 0.16420e+01_rb,0.17464e+01_rb,0.18635e+01_rb,0.19773e+01_rb,0.21012e+01_rb /)
      kao(:, 1, 9) = (/ &
        & 0.39892e+01_rb,0.40153e+01_rb,0.40324e+01_rb,0.40465e+01_rb,0.40524e+01_rb /)
      kao(:, 2, 9) = (/ &
        & 0.38871e+01_rb,0.39035e+01_rb,0.39123e+01_rb,0.39056e+01_rb,0.38966e+01_rb /)
      kao(:, 3, 9) = (/ &
        & 0.37883e+01_rb,0.37950e+01_rb,0.38080e+01_rb,0.37991e+01_rb,0.37826e+01_rb /)
      kao(:, 4, 9) = (/ &
        & 0.37778e+01_rb,0.37663e+01_rb,0.37745e+01_rb,0.37709e+01_rb,0.37556e+01_rb /)
      kao(:, 5, 9) = (/ &
        & 0.38757e+01_rb,0.38404e+01_rb,0.38539e+01_rb,0.39228e+01_rb,0.39777e+01_rb /)
      kao(:, 6, 9) = (/ &
        & 0.40990e+01_rb,0.41407e+01_rb,0.41745e+01_rb,0.42258e+01_rb,0.42574e+01_rb /)
      kao(:, 7, 9) = (/ &
        & 0.46288e+01_rb,0.46726e+01_rb,0.47096e+01_rb,0.47165e+01_rb,0.47374e+01_rb /)
      kao(:, 8, 9) = (/ &
        & 0.58432e+01_rb,0.58474e+01_rb,0.58608e+01_rb,0.58935e+01_rb,0.59261e+01_rb /)
      kao(:, 9, 9) = (/ &
        & 0.88685e+01_rb,0.88567e+01_rb,0.88137e+01_rb,0.87653e+01_rb,0.87262e+01_rb /)
      kao(:,10, 9) = (/ &
        & 0.10455e+02_rb,0.10242e+02_rb,0.10004e+02_rb,0.97643e+01_rb,0.95067e+01_rb /)
      kao(:,11, 9) = (/ &
        & 0.10240e+02_rb,0.99968e+01_rb,0.97347e+01_rb,0.94750e+01_rb,0.92006e+01_rb /)
      kao(:,12, 9) = (/ &
        & 0.10401e+02_rb,0.10165e+02_rb,0.99240e+01_rb,0.96656e+01_rb,0.94112e+01_rb /)
      kao(:,13, 9) = (/ &
        & 0.10764e+02_rb,0.10580e+02_rb,0.10381e+02_rb,0.10188e+02_rb,0.99802e+01_rb /)
      kao(:, 1,10) = (/ &
        & 0.10712e+02_rb,0.10575e+02_rb,0.10581e+02_rb,0.10556e+02_rb,0.10575e+02_rb /)
      kao(:, 2,10) = (/ &
        & 0.11144e+02_rb,0.11109e+02_rb,0.11034e+02_rb,0.11107e+02_rb,0.11138e+02_rb /)
      kao(:, 3,10) = (/ &
        & 0.12257e+02_rb,0.12112e+02_rb,0.11992e+02_rb,0.11951e+02_rb,0.12031e+02_rb /)
      kao(:, 4,10) = (/ &
        & 0.13040e+02_rb,0.13085e+02_rb,0.12882e+02_rb,0.12756e+02_rb,0.12737e+02_rb /)
      kao(:, 5,10) = (/ &
        & 0.13809e+02_rb,0.13771e+02_rb,0.13452e+02_rb,0.12567e+02_rb,0.12086e+02_rb /)
      kao(:, 6,10) = (/ &
        & 0.14608e+02_rb,0.13942e+02_rb,0.13365e+02_rb,0.13494e+02_rb,0.13762e+02_rb /)
      kao(:, 7,10) = (/ &
        & 0.14665e+02_rb,0.14685e+02_rb,0.15121e+02_rb,0.15602e+02_rb,0.15768e+02_rb /)
      kao(:, 8,10) = (/ &
        & 0.15612e+02_rb,0.16507e+02_rb,0.16644e+02_rb,0.16706e+02_rb,0.16696e+02_rb /)
      kao(:, 9,10) = (/ &
        & 0.15405e+02_rb,0.15874e+02_rb,0.16716e+02_rb,0.17957e+02_rb,0.17859e+02_rb /)
      kao(:,10,10) = (/ &
        & 0.19406e+00_rb,0.17932e+00_rb,0.16660e+00_rb,0.15558e+00_rb,0.14589e+00_rb /)
      kao(:,11,10) = (/ &
        & 0.27650e+00_rb,0.25556e+00_rb,0.23727e+00_rb,0.22182e+00_rb,0.20805e+00_rb /)
      kao(:,12,10) = (/ &
        & 0.29298e+00_rb,0.27044e+00_rb,0.25162e+00_rb,0.23497e+00_rb,0.22076e+00_rb /)
      kao(:,13,10) = (/ &
        & 0.24883e+00_rb,0.22985e+00_rb,0.21413e+00_rb,0.20012e+00_rb,0.18795e+00_rb /)
      kao(:, 1,11) = (/ &
        & 0.13650e+02_rb,0.13767e+02_rb,0.13771e+02_rb,0.13650e+02_rb,0.13663e+02_rb /)
      kao(:, 2,11) = (/ &
        & 0.15146e+02_rb,0.15253e+02_rb,0.15272e+02_rb,0.15152e+02_rb,0.15138e+02_rb /)
      kao(:, 3,11) = (/ &
        & 0.16834e+02_rb,0.17107e+02_rb,0.17158e+02_rb,0.17187e+02_rb,0.17057e+02_rb /)
      kao(:, 4,11) = (/ &
        & 0.19191e+02_rb,0.19144e+02_rb,0.19155e+02_rb,0.19247e+02_rb,0.19292e+02_rb /)
      kao(:, 5,11) = (/ &
        & 0.20953e+02_rb,0.21248e+02_rb,0.21186e+02_rb,0.21027e+02_rb,0.20703e+02_rb /)
      kao(:, 6,11) = (/ &
        & 0.22592e+02_rb,0.22192e+02_rb,0.22056e+02_rb,0.20972e+02_rb,0.19910e+02_rb /)
      kao(:, 7,11) = (/ &
        & 0.24652e+02_rb,0.23679e+02_rb,0.22339e+02_rb,0.21302e+02_rb,0.21697e+02_rb /)
      kao(:, 8,11) = (/ &
        & 0.22508e+02_rb,0.20438e+02_rb,0.20163e+02_rb,0.20628e+02_rb,0.20954e+02_rb /)
      kao(:, 9,11) = (/ &
        & 0.10264e+02_rb,0.85820e+01_rb,0.65270e+01_rb,0.38883e+01_rb,0.27974e+01_rb /)
      kao(:,10,11) = (/ &
        & 0.19324e+00_rb,0.17913e+00_rb,0.16619e+00_rb,0.15516e+00_rb,0.14524e+00_rb /)
      kao(:,11,11) = (/ &
        & 0.27573e+00_rb,0.25425e+00_rb,0.23627e+00_rb,0.22036e+00_rb,0.20639e+00_rb /)
      kao(:,12,11) = (/ &
        & 0.29224e+00_rb,0.26998e+00_rb,0.25055e+00_rb,0.23380e+00_rb,0.21908e+00_rb /)
      kao(:,13,11) = (/ &
        & 0.24964e+00_rb,0.23056e+00_rb,0.21391e+00_rb,0.19980e+00_rb,0.18698e+00_rb /)
      kao(:, 1,12) = (/ &
        & 0.17721e+02_rb,0.17805e+02_rb,0.17752e+02_rb,0.17857e+02_rb,0.17872e+02_rb /)
      kao(:, 2,12) = (/ &
        & 0.20222e+02_rb,0.20161e+02_rb,0.20267e+02_rb,0.20335e+02_rb,0.20254e+02_rb /)
      kao(:, 3,12) = (/ &
        & 0.23717e+02_rb,0.23414e+02_rb,0.23571e+02_rb,0.23552e+02_rb,0.23687e+02_rb /)
      kao(:, 4,12) = (/ &
        & 0.26876e+02_rb,0.26972e+02_rb,0.27155e+02_rb,0.26947e+02_rb,0.27025e+02_rb /)
      kao(:, 5,12) = (/ &
        & 0.30213e+02_rb,0.29978e+02_rb,0.30226e+02_rb,0.30477e+02_rb,0.30329e+02_rb /)
      kao(:, 6,12) = (/ &
        & 0.33660e+02_rb,0.33482e+02_rb,0.33159e+02_rb,0.32959e+02_rb,0.32299e+02_rb /)
      kao(:, 7,12) = (/ &
        & 0.35893e+02_rb,0.35649e+02_rb,0.35067e+02_rb,0.34565e+02_rb,0.32437e+02_rb /)
      kao(:, 8,12) = (/ &
        & 0.33426e+02_rb,0.33323e+02_rb,0.31984e+02_rb,0.29587e+02_rb,0.27505e+02_rb /)
      kao(:, 9,12) = (/ &
        & 0.46633e-01_rb,0.43205e-01_rb,0.40562e-01_rb,0.38133e-01_rb,0.35900e-01_rb /)
      kao(:,10,12) = (/ &
        & 0.19421e+00_rb,0.18024e+00_rb,0.16872e+00_rb,0.15815e+00_rb,0.14837e+00_rb /)
      kao(:,11,12) = (/ &
        & 0.27315e+00_rb,0.25480e+00_rb,0.23771e+00_rb,0.22191e+00_rb,0.20780e+00_rb /)
      kao(:,12,12) = (/ &
        & 0.29027e+00_rb,0.26969e+00_rb,0.25017e+00_rb,0.23314e+00_rb,0.21774e+00_rb /)
      kao(:,13,12) = (/ &
        & 0.24833e+00_rb,0.22943e+00_rb,0.21278e+00_rb,0.19800e+00_rb,0.18543e+00_rb /)
      kao(:, 1,13) = (/ &
        & 0.29672e+02_rb,0.29291e+02_rb,0.29191e+02_rb,0.29170e+02_rb,0.29116e+02_rb /)
      kao(:, 2,13) = (/ &
        & 0.24713e+02_rb,0.24965e+02_rb,0.25039e+02_rb,0.25355e+02_rb,0.25650e+02_rb /)
      kao(:, 3,13) = (/ &
        & 0.30510e+02_rb,0.31166e+02_rb,0.30663e+02_rb,0.31021e+02_rb,0.30901e+02_rb /)
      kao(:, 4,13) = (/ &
        & 0.38695e+02_rb,0.38922e+02_rb,0.38741e+02_rb,0.39204e+02_rb,0.38826e+02_rb /)
      kao(:, 5,13) = (/ &
        & 0.45118e+02_rb,0.44337e+02_rb,0.43664e+02_rb,0.43267e+02_rb,0.43524e+02_rb /)
      kao(:, 6,13) = (/ &
        & 0.34652e+02_rb,0.35182e+02_rb,0.35339e+02_rb,0.35340e+02_rb,0.35899e+02_rb /)
      kao(:, 7,13) = (/ &
        & 0.18801e+02_rb,0.18598e+02_rb,0.18855e+02_rb,0.18819e+02_rb,0.19052e+02_rb /)
      kao(:, 8,13) = (/ &
        & 0.12297e-01_rb,0.11425e-01_rb,0.10670e-01_rb,0.99994e-02_rb,0.94139e-02_rb /)
      kao(:, 9,13) = (/ &
        & 0.46821e-01_rb,0.43361e-01_rb,0.40368e-01_rb,0.37752e-01_rb,0.35493e-01_rb /)
      kao(:,10,13) = (/ &
        & 0.19817e+00_rb,0.18303e+00_rb,0.17002e+00_rb,0.15883e+00_rb,0.14933e+00_rb /)
      kao(:,11,13) = (/ &
        & 0.28145e+00_rb,0.25995e+00_rb,0.24181e+00_rb,0.22660e+00_rb,0.21326e+00_rb /)
      kao(:,12,13) = (/ &
        & 0.29510e+00_rb,0.27294e+00_rb,0.25519e+00_rb,0.23947e+00_rb,0.22551e+00_rb /)
      kao(:,13,13) = (/ &
        & 0.24830e+00_rb,0.23074e+00_rb,0.21556e+00_rb,0.20226e+00_rb,0.19029e+00_rb /)
      kao(:, 1,14) = (/ &
        & 0.47168e+02_rb,0.46890e+02_rb,0.46612e+02_rb,0.46353e+02_rb,0.46088e+02_rb /)
      kao(:, 2,14) = (/ &
        & 0.47771e+02_rb,0.46980e+02_rb,0.46445e+02_rb,0.45828e+02_rb,0.45437e+02_rb /)
      kao(:, 3,14) = (/ &
        & 0.42946e+02_rb,0.42652e+02_rb,0.43791e+02_rb,0.43237e+02_rb,0.44104e+02_rb /)
      kao(:, 4,14) = (/ &
        & 0.30957e+02_rb,0.30098e+02_rb,0.30055e+02_rb,0.29555e+02_rb,0.29902e+02_rb /)
      kao(:, 5,14) = (/ &
        & 0.28397e+01_rb,0.38572e+01_rb,0.42906e+01_rb,0.47091e+01_rb,0.48135e+01_rb /)
      kao(:, 6,14) = (/ &
        & 0.36978e-02_rb,0.34591e-02_rb,0.32524e-02_rb,0.30649e-02_rb,0.29018e-02_rb /)
      kao(:, 7,14) = (/ &
        & 0.61733e-02_rb,0.57589e-02_rb,0.53925e-02_rb,0.50742e-02_rb,0.47835e-02_rb /)
      kao(:, 8,14) = (/ &
        & 0.12390e-01_rb,0.11526e-01_rb,0.10766e-01_rb,0.10096e-01_rb,0.95010e-02_rb /)
      kao(:, 9,14) = (/ &
        & 0.47105e-01_rb,0.43648e-01_rb,0.40665e-01_rb,0.38054e-01_rb,0.35722e-01_rb /)
      kao(:,10,14) = (/ &
        & 0.19915e+00_rb,0.18412e+00_rb,0.17099e+00_rb,0.15957e+00_rb,0.14948e+00_rb /)
      kao(:,11,14) = (/ &
        & 0.28280e+00_rb,0.26124e+00_rb,0.24266e+00_rb,0.22645e+00_rb,0.21214e+00_rb /)
      kao(:,12,14) = (/ &
        & 0.29891e+00_rb,0.27613e+00_rb,0.25648e+00_rb,0.23927e+00_rb,0.22423e+00_rb /)
      kao(:,13,14) = (/ &
        & 0.25421e+00_rb,0.23484e+00_rb,0.21809e+00_rb,0.20347e+00_rb,0.19057e+00_rb /)
      kao(:, 1,15) = (/ &
        & 0.64994e+02_rb,0.64283e+02_rb,0.63755e+02_rb,0.63407e+02_rb,0.63287e+02_rb /)
      kao(:, 2,15) = (/ &
        & 0.78266e+02_rb,0.77364e+02_rb,0.76722e+02_rb,0.76285e+02_rb,0.75992e+02_rb /)
      kao(:, 3,15) = (/ &
        & 0.41710e+02_rb,0.38379e+02_rb,0.35076e+02_rb,0.35418e+02_rb,0.33306e+02_rb /)
      kao(:, 4,15) = (/ &
        & 0.14170e-02_rb,0.13401e-02_rb,0.12716e-02_rb,0.12073e-02_rb,0.11535e-02_rb /)
      kao(:, 5,15) = (/ &
        & 0.23659e-02_rb,0.22261e-02_rb,0.21042e-02_rb,0.19950e-02_rb,0.18970e-02_rb /)
      kao(:, 6,15) = (/ &
        & 0.37432e-02_rb,0.35105e-02_rb,0.33006e-02_rb,0.31222e-02_rb,0.29521e-02_rb /)
      kao(:, 7,15) = (/ &
        & 0.62408e-02_rb,0.58258e-02_rb,0.54782e-02_rb,0.51594e-02_rb,0.48767e-02_rb /)
      kao(:, 8,15) = (/ &
        & 0.12522e-01_rb,0.11652e-01_rb,0.10922e-01_rb,0.10265e-01_rb,0.96672e-02_rb /)
      kao(:, 9,15) = (/ &
        & 0.47545e-01_rb,0.44152e-01_rb,0.41251e-01_rb,0.38666e-01_rb,0.36358e-01_rb /)
      kao(:,10,15) = (/ &
        & 0.20097e+00_rb,0.18592e+00_rb,0.17341e+00_rb,0.16203e+00_rb,0.15235e+00_rb /)
      kao(:,11,15) = (/ &
        & 0.28499e+00_rb,0.26428e+00_rb,0.24585e+00_rb,0.22980e+00_rb,0.21575e+00_rb /)
      kao(:,12,15) = (/ &
        & 0.30172e+00_rb,0.27929e+00_rb,0.25980e+00_rb,0.24306e+00_rb,0.22789e+00_rb /)
      kao(:,13,15) = (/ &
        & 0.25655e+00_rb,0.23731e+00_rb,0.22092e+00_rb,0.20639e+00_rb,0.19367e+00_rb /)
      kao(:, 1,16) = (/ &
        & 0.80810e+02_rb,0.81099e+02_rb,0.81190e+02_rb,0.81107e+02_rb,0.80989e+02_rb /)
      kao(:, 2,16) = (/ &
        & 0.99319e+02_rb,0.99708e+02_rb,0.99822e+02_rb,0.99871e+02_rb,0.99993e+02_rb /)
      kao(:, 3,16) = (/ &
        & 0.46927e+02_rb,0.54316e+02_rb,0.57355e+02_rb,0.53715e+02_rb,0.52802e+02_rb /)
      kao(:, 4,16) = (/ &
        & 0.12802e-02_rb,0.12027e-02_rb,0.11386e-02_rb,0.10866e-02_rb,0.10525e-02_rb /)
      kao(:, 5,16) = (/ &
        & 0.21675e-02_rb,0.20715e-02_rb,0.19422e-02_rb,0.18402e-02_rb,0.17681e-02_rb /)
      kao(:, 6,16) = (/ &
        & 0.34707e-02_rb,0.32796e-02_rb,0.30986e-02_rb,0.29246e-02_rb,0.28034e-02_rb /)
      kao(:, 7,16) = (/ &
        & 0.58659e-02_rb,0.55310e-02_rb,0.51820e-02_rb,0.48829e-02_rb,0.46784e-02_rb /)
      kao(:, 8,16) = (/ &
        & 0.11875e-01_rb,0.11183e-01_rb,0.10438e-01_rb,0.98038e-02_rb,0.93665e-02_rb /)
      kao(:, 9,16) = (/ &
        & 0.45519e-01_rb,0.42850e-01_rb,0.39715e-01_rb,0.37447e-01_rb,0.35369e-01_rb /)
      kao(:,10,16) = (/ &
        & 0.19392e+00_rb,0.18108e+00_rb,0.16817e+00_rb,0.15883e+00_rb,0.14864e+00_rb /)
      kao(:,11,16) = (/ &
        & 0.27830e+00_rb,0.25715e+00_rb,0.24100e+00_rb,0.22595e+00_rb,0.21321e+00_rb /)
      kao(:,12,16) = (/ &
        & 0.29533e+00_rb,0.27422e+00_rb,0.25636e+00_rb,0.23936e+00_rb,0.22658e+00_rb /)
      kao(:,13,16) = (/ &
        & 0.25310e+00_rb,0.23535e+00_rb,0.21855e+00_rb,0.20552e+00_rb,0.19293e+00_rb /)
  
!     -----------------------------------------------------------------
!     The array KBO contains absorption coefs at the 16 chosen g-values 
!     for a range of pressure levels < ~100mb and temperatures. The first 
!     index in the array, JT, which runs from 1 to 5, corresponds to 
!     different temperatures.  More specifically, JT = 3 means that the 
!     data are for the reference temperature TREF for this pressure 
!     level, JT = 2 refers to the temperature TREF-15, JT = 1 is for
!     TREF-30, JT = 4 is for TREF+15, and JT = 5 is for TREF+30.  
!     The second index, JP, runs from 13 to 59 and refers to the JPth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  The third index, IG, goes from 1 to 16,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      kbo(:,13, 1) = (/ &
        & 0.18379e-05_rb,0.23296e-05_rb,0.29007e-05_rb,0.35902e-05_rb,0.43437e-05_rb /)
      kbo(:,14, 1) = (/ &
        & 0.15919e-05_rb,0.19832e-05_rb,0.24720e-05_rb,0.30683e-05_rb,0.37253e-05_rb /)
      kbo(:,15, 1) = (/ &
        & 0.13850e-05_rb,0.17115e-05_rb,0.21225e-05_rb,0.26292e-05_rb,0.31945e-05_rb /)
      kbo(:,16, 1) = (/ &
        & 0.11896e-05_rb,0.14680e-05_rb,0.18142e-05_rb,0.22537e-05_rb,0.27284e-05_rb /)
      kbo(:,17, 1) = (/ &
        & 0.10228e-05_rb,0.12597e-05_rb,0.15532e-05_rb,0.19272e-05_rb,0.23163e-05_rb /)
      kbo(:,18, 1) = (/ &
        & 0.88243e-06_rb,0.10835e-05_rb,0.13316e-05_rb,0.16603e-05_rb,0.19808e-05_rb /)
      kbo(:,19, 1) = (/ &
        & 0.75677e-06_rb,0.92925e-06_rb,0.11401e-05_rb,0.14107e-05_rb,0.16901e-05_rb /)
      kbo(:,20, 1) = (/ &
        & 0.64614e-06_rb,0.79361e-06_rb,0.97353e-06_rb,0.11964e-05_rb,0.14414e-05_rb /)
      kbo(:,21, 1) = (/ &
        & 0.55323e-06_rb,0.67580e-06_rb,0.82647e-06_rb,0.10137e-05_rb,0.12318e-05_rb /)
      kbo(:,22, 1) = (/ &
        & 0.47719e-06_rb,0.58149e-06_rb,0.71060e-06_rb,0.87005e-06_rb,0.10605e-05_rb /)
      kbo(:,23, 1) = (/ &
        & 0.41128e-06_rb,0.50080e-06_rb,0.61104e-06_rb,0.75347e-06_rb,0.91200e-06_rb /)
      kbo(:,24, 1) = (/ &
        & 0.35306e-06_rb,0.42860e-06_rb,0.52320e-06_rb,0.65156e-06_rb,0.78122e-06_rb /)
      kbo(:,25, 1) = (/ &
        & 0.30363e-06_rb,0.36731e-06_rb,0.44673e-06_rb,0.56097e-06_rb,0.66791e-06_rb /)
      kbo(:,26, 1) = (/ &
        & 0.26073e-06_rb,0.31537e-06_rb,0.38738e-06_rb,0.48210e-06_rb,0.57382e-06_rb /)
      kbo(:,27, 1) = (/ &
        & 0.22303e-06_rb,0.26986e-06_rb,0.33628e-06_rb,0.41161e-06_rb,0.48722e-06_rb /)
      kbo(:,28, 1) = (/ &
        & 0.18954e-06_rb,0.22952e-06_rb,0.28797e-06_rb,0.35166e-06_rb,0.41440e-06_rb /)
      kbo(:,29, 1) = (/ &
        & 0.16106e-06_rb,0.19430e-06_rb,0.24661e-06_rb,0.29815e-06_rb,0.35151e-06_rb /)
      kbo(:,30, 1) = (/ &
        & 0.13630e-06_rb,0.16528e-06_rb,0.20922e-06_rb,0.25193e-06_rb,0.29913e-06_rb /)
      kbo(:,31, 1) = (/ &
        & 0.11504e-06_rb,0.14276e-06_rb,0.17807e-06_rb,0.21482e-06_rb,0.25451e-06_rb /)
      kbo(:,32, 1) = (/ &
        & 0.96980e-07_rb,0.12215e-06_rb,0.15159e-06_rb,0.18179e-06_rb,0.21515e-06_rb /)
      kbo(:,33, 1) = (/ &
        & 0.81598e-07_rb,0.10315e-06_rb,0.12730e-06_rb,0.15190e-06_rb,0.18093e-06_rb /)
      kbo(:,34, 1) = (/ &
        & 0.67700e-07_rb,0.85970e-07_rb,0.10559e-06_rb,0.12608e-06_rb,0.15006e-06_rb /)
      kbo(:,35, 1) = (/ &
        & 0.55177e-07_rb,0.69519e-07_rb,0.85967e-07_rb,0.10223e-06_rb,0.12147e-06_rb /)
      kbo(:,36, 1) = (/ &
        & 0.43924e-07_rb,0.54395e-07_rb,0.67998e-07_rb,0.81142e-07_rb,0.95717e-07_rb /)
      kbo(:,37, 1) = (/ &
        & 0.34469e-07_rb,0.41036e-07_rb,0.52575e-07_rb,0.63117e-07_rb,0.74301e-07_rb /)
      kbo(:,38, 1) = (/ &
        & 0.26777e-07_rb,0.31734e-07_rb,0.40555e-07_rb,0.49032e-07_rb,0.57653e-07_rb /)
      kbo(:,39, 1) = (/ &
        & 0.20670e-07_rb,0.24721e-07_rb,0.30685e-07_rb,0.37873e-07_rb,0.44555e-07_rb /)
      kbo(:,40, 1) = (/ &
        & 0.15860e-07_rb,0.19032e-07_rb,0.22612e-07_rb,0.28845e-07_rb,0.34246e-07_rb /)
      kbo(:,41, 1) = (/ &
        & 0.12129e-07_rb,0.14603e-07_rb,0.17357e-07_rb,0.21851e-07_rb,0.26346e-07_rb /)
      kbo(:,42, 1) = (/ &
        & 0.92319e-08_rb,0.11199e-07_rb,0.13365e-07_rb,0.16174e-07_rb,0.20019e-07_rb /)
      kbo(:,43, 1) = (/ &
        & 0.69289e-08_rb,0.84537e-08_rb,0.10159e-07_rb,0.11992e-07_rb,0.14940e-07_rb /)
      kbo(:,44, 1) = (/ &
        & 0.51546e-08_rb,0.63380e-08_rb,0.76593e-08_rb,0.90562e-08_rb,0.10716e-07_rb /)
      kbo(:,45, 1) = (/ &
        & 0.38335e-08_rb,0.47085e-08_rb,0.57219e-08_rb,0.68148e-08_rb,0.79518e-08_rb /)
      kbo(:,46, 1) = (/ &
        & 0.28583e-08_rb,0.34724e-08_rb,0.42447e-08_rb,0.50888e-08_rb,0.59494e-08_rb /)
      kbo(:,47, 1) = (/ &
        & 0.20945e-08_rb,0.25418e-08_rb,0.31058e-08_rb,0.37494e-08_rb,0.44274e-08_rb /)
      kbo(:,48, 1) = (/ &
        & 0.15164e-08_rb,0.18560e-08_rb,0.22628e-08_rb,0.27371e-08_rb,0.32773e-08_rb /)
      kbo(:,49, 1) = (/ &
        & 0.10916e-08_rb,0.13465e-08_rb,0.16387e-08_rb,0.19740e-08_rb,0.23865e-08_rb /)
      kbo(:,50, 1) = (/ &
        & 0.78324e-09_rb,0.98361e-09_rb,0.11977e-08_rb,0.14517e-08_rb,0.17457e-08_rb /)
      kbo(:,51, 1) = (/ &
        & 0.57263e-09_rb,0.71786e-09_rb,0.87689e-09_rb,0.10705e-08_rb,0.12881e-08_rb /)
      kbo(:,52, 1) = (/ &
        & 0.41574e-09_rb,0.51264e-09_rb,0.64152e-09_rb,0.78315e-09_rb,0.94705e-09_rb /)
      kbo(:,53, 1) = (/ &
        & 0.29940e-09_rb,0.36836e-09_rb,0.45886e-09_rb,0.56405e-09_rb,0.69166e-09_rb /)
      kbo(:,54, 1) = (/ &
        & 0.22617e-09_rb,0.27395e-09_rb,0.33754e-09_rb,0.42095e-09_rb,0.51544e-09_rb /)
      kbo(:,55, 1) = (/ &
        & 0.17483e-09_rb,0.20845e-09_rb,0.25546e-09_rb,0.31817e-09_rb,0.39119e-09_rb /)
      kbo(:,56, 1) = (/ &
        & 0.13496e-09_rb,0.15930e-09_rb,0.19380e-09_rb,0.23861e-09_rb,0.29770e-09_rb /)
      kbo(:,57, 1) = (/ &
        & 0.10406e-09_rb,0.12247e-09_rb,0.14749e-09_rb,0.18078e-09_rb,0.22597e-09_rb /)
      kbo(:,58, 1) = (/ &
        & 0.81345e-10_rb,0.94816e-10_rb,0.11329e-09_rb,0.13832e-09_rb,0.17161e-09_rb /)
      kbo(:,59, 1) = (/ &
        & 0.67159e-10_rb,0.78312e-10_rb,0.94745e-10_rb,0.11544e-09_rb,0.14442e-09_rb /)
      kbo(:,13, 2) = (/ &
        & 0.16654e-04_rb,0.18728e-04_rb,0.21238e-04_rb,0.24491e-04_rb,0.29038e-04_rb /)
      kbo(:,14, 2) = (/ &
        & 0.15636e-04_rb,0.17574e-04_rb,0.19918e-04_rb,0.23671e-04_rb,0.27237e-04_rb /)
      kbo(:,15, 2) = (/ &
        & 0.14853e-04_rb,0.16629e-04_rb,0.19700e-04_rb,0.22604e-04_rb,0.25586e-04_rb /)
      kbo(:,16, 2) = (/ &
        & 0.13519e-04_rb,0.15923e-04_rb,0.18289e-04_rb,0.20530e-04_rb,0.22799e-04_rb /)
      kbo(:,17, 2) = (/ &
        & 0.12411e-04_rb,0.14439e-04_rb,0.16431e-04_rb,0.18235e-04_rb,0.20109e-04_rb /)
      kbo(:,18, 2) = (/ &
        & 0.11267e-04_rb,0.12880e-04_rb,0.14429e-04_rb,0.16005e-04_rb,0.17538e-04_rb /)
      kbo(:,19, 2) = (/ &
        & 0.10025e-04_rb,0.11332e-04_rb,0.12550e-04_rb,0.13853e-04_rb,0.15103e-04_rb /)
      kbo(:,20, 2) = (/ &
        & 0.86342e-05_rb,0.97455e-05_rb,0.10778e-04_rb,0.11898e-04_rb,0.12789e-04_rb /)
      kbo(:,21, 2) = (/ &
        & 0.74004e-05_rb,0.83107e-05_rb,0.91633e-05_rb,0.10071e-04_rb,0.10674e-04_rb /)
      kbo(:,22, 2) = (/ &
        & 0.63700e-05_rb,0.70549e-05_rb,0.78271e-05_rb,0.85170e-05_rb,0.90022e-05_rb /)
      kbo(:,23, 2) = (/ &
        & 0.54404e-05_rb,0.60156e-05_rb,0.66393e-05_rb,0.71320e-05_rb,0.75007e-05_rb /)
      kbo(:,24, 2) = (/ &
        & 0.45997e-05_rb,0.50718e-05_rb,0.55713e-05_rb,0.59455e-05_rb,0.62118e-05_rb /)
      kbo(:,25, 2) = (/ &
        & 0.38745e-05_rb,0.42617e-05_rb,0.46479e-05_rb,0.49297e-05_rb,0.51638e-05_rb /)
      kbo(:,26, 2) = (/ &
        & 0.32488e-05_rb,0.35592e-05_rb,0.38685e-05_rb,0.40759e-05_rb,0.43092e-05_rb /)
      kbo(:,27, 2) = (/ &
        & 0.26995e-05_rb,0.29601e-05_rb,0.32065e-05_rb,0.33666e-05_rb,0.35609e-05_rb /)
      kbo(:,28, 2) = (/ &
        & 0.22359e-05_rb,0.24429e-05_rb,0.26408e-05_rb,0.27752e-05_rb,0.29293e-05_rb /)
      kbo(:,29, 2) = (/ &
        & 0.18487e-05_rb,0.20089e-05_rb,0.21668e-05_rb,0.22793e-05_rb,0.24029e-05_rb /)
      kbo(:,30, 2) = (/ &
        & 0.15291e-05_rb,0.16589e-05_rb,0.17733e-05_rb,0.18706e-05_rb,0.19584e-05_rb /)
      kbo(:,31, 2) = (/ &
        & 0.12554e-05_rb,0.13589e-05_rb,0.14436e-05_rb,0.15195e-05_rb,0.15928e-05_rb /)
      kbo(:,32, 2) = (/ &
        & 0.10248e-05_rb,0.11046e-05_rb,0.11716e-05_rb,0.12288e-05_rb,0.12916e-05_rb /)
      kbo(:,33, 2) = (/ &
        & 0.83272e-06_rb,0.89120e-06_rb,0.95036e-06_rb,0.99040e-06_rb,0.10409e-05_rb /)
      kbo(:,34, 2) = (/ &
        & 0.67518e-06_rb,0.72097e-06_rb,0.76854e-06_rb,0.80246e-06_rb,0.83912e-06_rb /)
      kbo(:,35, 2) = (/ &
        & 0.53577e-06_rb,0.57870e-06_rb,0.61715e-06_rb,0.64987e-06_rb,0.67220e-06_rb /)
      kbo(:,36, 2) = (/ &
        & 0.42402e-06_rb,0.45825e-06_rb,0.49307e-06_rb,0.52005e-06_rb,0.54029e-06_rb /)
      kbo(:,37, 2) = (/ &
        & 0.33419e-06_rb,0.36213e-06_rb,0.38911e-06_rb,0.41359e-06_rb,0.43177e-06_rb /)
      kbo(:,38, 2) = (/ &
        & 0.26240e-06_rb,0.28355e-06_rb,0.30530e-06_rb,0.32881e-06_rb,0.34617e-06_rb /)
      kbo(:,39, 2) = (/ &
        & 0.20468e-06_rb,0.22245e-06_rb,0.23980e-06_rb,0.25776e-06_rb,0.27424e-06_rb /)
      kbo(:,40, 2) = (/ &
        & 0.16070e-06_rb,0.17478e-06_rb,0.18856e-06_rb,0.20230e-06_rb,0.21801e-06_rb /)
      kbo(:,41, 2) = (/ &
        & 0.12583e-06_rb,0.13680e-06_rb,0.14851e-06_rb,0.15917e-06_rb,0.17184e-06_rb /)
      kbo(:,42, 2) = (/ &
        & 0.98494e-07_rb,0.10639e-06_rb,0.11634e-06_rb,0.12500e-06_rb,0.13490e-06_rb /)
      kbo(:,43, 2) = (/ &
        & 0.75196e-07_rb,0.82809e-07_rb,0.90095e-07_rb,0.97952e-07_rb,0.10509e-06_rb /)
      kbo(:,44, 2) = (/ &
        & 0.57305e-07_rb,0.63787e-07_rb,0.69227e-07_rb,0.75635e-07_rb,0.81687e-07_rb /)
      kbo(:,45, 2) = (/ &
        & 0.42965e-07_rb,0.48245e-07_rb,0.53346e-07_rb,0.58061e-07_rb,0.63120e-07_rb /)
      kbo(:,46, 2) = (/ &
        & 0.31048e-07_rb,0.36457e-07_rb,0.40514e-07_rb,0.44557e-07_rb,0.48550e-07_rb /)
      kbo(:,47, 2) = (/ &
        & 0.23094e-07_rb,0.26356e-07_rb,0.30443e-07_rb,0.33954e-07_rb,0.36789e-07_rb /)
      kbo(:,48, 2) = (/ &
        & 0.16858e-07_rb,0.19402e-07_rb,0.22449e-07_rb,0.25401e-07_rb,0.28254e-07_rb /)
      kbo(:,49, 2) = (/ &
        & 0.11413e-07_rb,0.14055e-07_rb,0.16046e-07_rb,0.18927e-07_rb,0.21080e-07_rb /)
      kbo(:,50, 2) = (/ &
        & 0.78650e-08_rb,0.99879e-08_rb,0.11846e-07_rb,0.13654e-07_rb,0.15871e-07_rb /)
      kbo(:,51, 2) = (/ &
        & 0.55417e-08_rb,0.68977e-08_rb,0.87006e-08_rb,0.10108e-07_rb,0.11829e-07_rb /)
      kbo(:,52, 2) = (/ &
        & 0.39143e-08_rb,0.48025e-08_rb,0.60180e-08_rb,0.73762e-08_rb,0.85851e-08_rb /)
      kbo(:,53, 2) = (/ &
        & 0.27637e-08_rb,0.33397e-08_rb,0.41996e-08_rb,0.52787e-08_rb,0.62923e-08_rb /)
      kbo(:,54, 2) = (/ &
        & 0.19691e-08_rb,0.24529e-08_rb,0.30071e-08_rb,0.38041e-08_rb,0.46981e-08_rb /)
      kbo(:,55, 2) = (/ &
        & 0.13034e-08_rb,0.18257e-08_rb,0.22359e-08_rb,0.28210e-08_rb,0.35386e-08_rb /)
      kbo(:,56, 2) = (/ &
        & 0.10026e-08_rb,0.13422e-08_rb,0.16687e-08_rb,0.20793e-08_rb,0.26054e-08_rb /)
      kbo(:,57, 2) = (/ &
        & 0.77730e-09_rb,0.91190e-09_rb,0.12468e-08_rb,0.15302e-08_rb,0.19418e-08_rb /)
      kbo(:,58, 2) = (/ &
        & 0.60359e-09_rb,0.67848e-09_rb,0.92414e-09_rb,0.11511e-08_rb,0.14401e-08_rb /)
      kbo(:,59, 2) = (/ &
        & 0.50441e-09_rb,0.56797e-09_rb,0.77369e-09_rb,0.95612e-09_rb,0.11957e-08_rb /)
      kbo(:,13, 3) = (/ &
        & 0.98124e-04_rb,0.10149e-03_rb,0.10303e-03_rb,0.10669e-03_rb,0.10870e-03_rb /)
      kbo(:,14, 3) = (/ &
        & 0.88110e-04_rb,0.89730e-04_rb,0.90679e-04_rb,0.91892e-04_rb,0.91413e-04_rb /)
      kbo(:,15, 3) = (/ &
        & 0.78555e-04_rb,0.78633e-04_rb,0.79145e-04_rb,0.77689e-04_rb,0.77791e-04_rb /)
      kbo(:,16, 3) = (/ &
        & 0.67245e-04_rb,0.67743e-04_rb,0.66690e-04_rb,0.65765e-04_rb,0.65950e-04_rb /)
      kbo(:,17, 3) = (/ &
        & 0.57553e-04_rb,0.57490e-04_rb,0.56012e-04_rb,0.55634e-04_rb,0.56016e-04_rb /)
      kbo(:,18, 3) = (/ &
        & 0.49729e-04_rb,0.48302e-04_rb,0.47418e-04_rb,0.47262e-04_rb,0.47732e-04_rb /)
      kbo(:,19, 3) = (/ &
        & 0.42101e-04_rb,0.40906e-04_rb,0.40541e-04_rb,0.40229e-04_rb,0.40671e-04_rb /)
      kbo(:,20, 3) = (/ &
        & 0.35046e-04_rb,0.34193e-04_rb,0.34032e-04_rb,0.33881e-04_rb,0.34691e-04_rb /)
      kbo(:,21, 3) = (/ &
        & 0.29192e-04_rb,0.28813e-04_rb,0.28525e-04_rb,0.28485e-04_rb,0.29764e-04_rb /)
      kbo(:,22, 3) = (/ &
        & 0.24212e-04_rb,0.23970e-04_rb,0.23804e-04_rb,0.23947e-04_rb,0.24310e-04_rb /)
      kbo(:,23, 3) = (/ &
        & 0.20167e-04_rb,0.19961e-04_rb,0.19837e-04_rb,0.20528e-04_rb,0.19887e-04_rb /)
      kbo(:,24, 3) = (/ &
        & 0.16817e-04_rb,0.16624e-04_rb,0.16525e-04_rb,0.17263e-04_rb,0.16481e-04_rb /)
      kbo(:,25, 3) = (/ &
        & 0.13896e-04_rb,0.13806e-04_rb,0.13810e-04_rb,0.14348e-04_rb,0.13634e-04_rb /)
      kbo(:,26, 3) = (/ &
        & 0.11515e-04_rb,0.11474e-04_rb,0.11553e-04_rb,0.11705e-04_rb,0.11167e-04_rb /)
      kbo(:,27, 3) = (/ &
        & 0.95004e-05_rb,0.94543e-05_rb,0.95650e-05_rb,0.95316e-05_rb,0.91651e-05_rb /)
      kbo(:,28, 3) = (/ &
        & 0.78135e-05_rb,0.77862e-05_rb,0.78715e-05_rb,0.77825e-05_rb,0.74877e-05_rb /)
      kbo(:,29, 3) = (/ &
        & 0.63615e-05_rb,0.63601e-05_rb,0.64198e-05_rb,0.63507e-05_rb,0.61200e-05_rb /)
      kbo(:,30, 3) = (/ &
        & 0.51738e-05_rb,0.51781e-05_rb,0.52250e-05_rb,0.51709e-05_rb,0.50243e-05_rb /)
      kbo(:,31, 3) = (/ &
        & 0.41825e-05_rb,0.42240e-05_rb,0.42510e-05_rb,0.42241e-05_rb,0.41058e-05_rb /)
      kbo(:,32, 3) = (/ &
        & 0.34024e-05_rb,0.34302e-05_rb,0.34514e-05_rb,0.34436e-05_rb,0.33474e-05_rb /)
      kbo(:,33, 3) = (/ &
        & 0.27510e-05_rb,0.27828e-05_rb,0.28059e-05_rb,0.28125e-05_rb,0.27394e-05_rb /)
      kbo(:,34, 3) = (/ &
        & 0.22195e-05_rb,0.22517e-05_rb,0.22743e-05_rb,0.22728e-05_rb,0.22254e-05_rb /)
      kbo(:,35, 3) = (/ &
        & 0.18029e-05_rb,0.18238e-05_rb,0.18432e-05_rb,0.18461e-05_rb,0.18241e-05_rb /)
      kbo(:,36, 3) = (/ &
        & 0.14477e-05_rb,0.14734e-05_rb,0.14936e-05_rb,0.15116e-05_rb,0.14874e-05_rb /)
      kbo(:,37, 3) = (/ &
        & 0.11637e-05_rb,0.11929e-05_rb,0.12098e-05_rb,0.12265e-05_rb,0.12076e-05_rb /)
      kbo(:,38, 3) = (/ &
        & 0.93292e-06_rb,0.96246e-06_rb,0.98169e-06_rb,0.99435e-06_rb,0.98749e-06_rb /)
      kbo(:,39, 3) = (/ &
        & 0.74927e-06_rb,0.76742e-06_rb,0.79020e-06_rb,0.80604e-06_rb,0.81415e-06_rb /)
      kbo(:,40, 3) = (/ &
        & 0.60479e-06_rb,0.61743e-06_rb,0.63973e-06_rb,0.65295e-06_rb,0.66261e-06_rb /)
      kbo(:,41, 3) = (/ &
        & 0.48450e-06_rb,0.49742e-06_rb,0.51366e-06_rb,0.52876e-06_rb,0.53813e-06_rb /)
      kbo(:,42, 3) = (/ &
        & 0.38298e-06_rb,0.40088e-06_rb,0.41192e-06_rb,0.42651e-06_rb,0.43595e-06_rb /)
      kbo(:,43, 3) = (/ &
        & 0.30310e-06_rb,0.31905e-06_rb,0.32907e-06_rb,0.34154e-06_rb,0.35007e-06_rb /)
      kbo(:,44, 3) = (/ &
        & 0.23745e-06_rb,0.25035e-06_rb,0.26224e-06_rb,0.27145e-06_rb,0.28146e-06_rb /)
      kbo(:,45, 3) = (/ &
        & 0.18653e-06_rb,0.19510e-06_rb,0.20644e-06_rb,0.21510e-06_rb,0.22354e-06_rb /)
      kbo(:,46, 3) = (/ &
        & 0.14759e-06_rb,0.15322e-06_rb,0.16145e-06_rb,0.17034e-06_rb,0.17744e-06_rb /)
      kbo(:,47, 3) = (/ &
        & 0.11410e-06_rb,0.12007e-06_rb,0.12507e-06_rb,0.13237e-06_rb,0.13995e-06_rb /)
      kbo(:,48, 3) = (/ &
        & 0.88327e-07_rb,0.93967e-07_rb,0.97683e-07_rb,0.10257e-06_rb,0.10921e-06_rb /)
      kbo(:,49, 3) = (/ &
        & 0.66715e-07_rb,0.72172e-07_rb,0.76442e-07_rb,0.79214e-07_rb,0.84006e-07_rb /)
      kbo(:,50, 3) = (/ &
        & 0.51458e-07_rb,0.55761e-07_rb,0.59239e-07_rb,0.62292e-07_rb,0.64470e-07_rb /)
      kbo(:,51, 3) = (/ &
        & 0.39957e-07_rb,0.42400e-07_rb,0.45870e-07_rb,0.48592e-07_rb,0.50287e-07_rb /)
      kbo(:,52, 3) = (/ &
        & 0.31150e-07_rb,0.32725e-07_rb,0.35598e-07_rb,0.37633e-07_rb,0.39607e-07_rb /)
      kbo(:,53, 3) = (/ &
        & 0.23783e-07_rb,0.25415e-07_rb,0.26634e-07_rb,0.28885e-07_rb,0.30620e-07_rb /)
      kbo(:,54, 3) = (/ &
        & 0.18531e-07_rb,0.19938e-07_rb,0.20988e-07_rb,0.22678e-07_rb,0.23867e-07_rb /)
      kbo(:,55, 3) = (/ &
        & 0.14810e-07_rb,0.15772e-07_rb,0.16651e-07_rb,0.17727e-07_rb,0.18861e-07_rb /)
      kbo(:,56, 3) = (/ &
        & 0.11652e-07_rb,0.12557e-07_rb,0.13319e-07_rb,0.13930e-07_rb,0.15077e-07_rb /)
      kbo(:,57, 3) = (/ &
        & 0.91523e-08_rb,0.99835e-08_rb,0.10603e-07_rb,0.11081e-07_rb,0.11999e-07_rb /)
      kbo(:,58, 3) = (/ &
        & 0.72383e-08_rb,0.79060e-08_rb,0.84489e-08_rb,0.88615e-08_rb,0.94456e-08_rb /)
      kbo(:,59, 3) = (/ &
        & 0.60315e-08_rb,0.65931e-08_rb,0.71001e-08_rb,0.74012e-08_rb,0.78761e-08_rb /)
      kbo(:,13, 4) = (/ &
        & 0.21882e-03_rb,0.20968e-03_rb,0.18648e-03_rb,0.17505e-03_rb,0.17320e-03_rb /)
      kbo(:,14, 4) = (/ &
        & 0.19140e-03_rb,0.17026e-03_rb,0.15903e-03_rb,0.15552e-03_rb,0.16009e-03_rb /)
      kbo(:,15, 4) = (/ &
        & 0.16003e-03_rb,0.14531e-03_rb,0.14007e-03_rb,0.14511e-03_rb,0.14619e-03_rb /)
      kbo(:,16, 4) = (/ &
        & 0.13387e-03_rb,0.12423e-03_rb,0.12444e-03_rb,0.12743e-03_rb,0.12749e-03_rb /)
      kbo(:,17, 4) = (/ &
        & 0.11266e-03_rb,0.10780e-03_rb,0.11110e-03_rb,0.11051e-03_rb,0.11075e-03_rb /)
      kbo(:,18, 4) = (/ &
        & 0.96156e-04_rb,0.95617e-04_rb,0.96775e-04_rb,0.96220e-04_rb,0.96078e-04_rb /)
      kbo(:,19, 4) = (/ &
        & 0.82712e-04_rb,0.82946e-04_rb,0.83265e-04_rb,0.82827e-04_rb,0.82044e-04_rb /)
      kbo(:,20, 4) = (/ &
        & 0.71080e-04_rb,0.71167e-04_rb,0.71040e-04_rb,0.70443e-04_rb,0.69069e-04_rb /)
      kbo(:,21, 4) = (/ &
        & 0.60110e-04_rb,0.60836e-04_rb,0.59994e-04_rb,0.58992e-04_rb,0.57237e-04_rb /)
      kbo(:,22, 4) = (/ &
        & 0.50688e-04_rb,0.51296e-04_rb,0.50271e-04_rb,0.48889e-04_rb,0.48471e-04_rb /)
      kbo(:,23, 4) = (/ &
        & 0.42706e-04_rb,0.42628e-04_rb,0.41764e-04_rb,0.39999e-04_rb,0.40893e-04_rb /)
      kbo(:,24, 4) = (/ &
        & 0.35803e-04_rb,0.35316e-04_rb,0.34535e-04_rb,0.33229e-04_rb,0.34127e-04_rb /)
      kbo(:,25, 4) = (/ &
        & 0.29906e-04_rb,0.29201e-04_rb,0.28336e-04_rb,0.27455e-04_rb,0.28044e-04_rb /)
      kbo(:,26, 4) = (/ &
        & 0.24659e-04_rb,0.23926e-04_rb,0.23159e-04_rb,0.22827e-04_rb,0.23266e-04_rb /)
      kbo(:,27, 4) = (/ &
        & 0.20221e-04_rb,0.19578e-04_rb,0.18838e-04_rb,0.18853e-04_rb,0.19067e-04_rb /)
      kbo(:,28, 4) = (/ &
        & 0.16345e-04_rb,0.15913e-04_rb,0.15327e-04_rb,0.15412e-04_rb,0.15560e-04_rb /)
      kbo(:,29, 4) = (/ &
        & 0.13195e-04_rb,0.12870e-04_rb,0.12430e-04_rb,0.12518e-04_rb,0.12639e-04_rb /)
      kbo(:,30, 4) = (/ &
        & 0.10622e-04_rb,0.10384e-04_rb,0.10068e-04_rb,0.10151e-04_rb,0.10256e-04_rb /)
      kbo(:,31, 4) = (/ &
        & 0.85785e-05_rb,0.83523e-05_rb,0.81227e-05_rb,0.81998e-05_rb,0.83313e-05_rb /)
      kbo(:,32, 4) = (/ &
        & 0.69114e-05_rb,0.67093e-05_rb,0.65758e-05_rb,0.66222e-05_rb,0.67679e-05_rb /)
      kbo(:,33, 4) = (/ &
        & 0.55456e-05_rb,0.54342e-05_rb,0.53229e-05_rb,0.53610e-05_rb,0.54819e-05_rb /)
      kbo(:,34, 4) = (/ &
        & 0.44461e-05_rb,0.43891e-05_rb,0.43112e-05_rb,0.43632e-05_rb,0.44560e-05_rb /)
      kbo(:,35, 4) = (/ &
        & 0.35725e-05_rb,0.35192e-05_rb,0.34703e-05_rb,0.35075e-05_rb,0.35919e-05_rb /)
      kbo(:,36, 4) = (/ &
        & 0.28842e-05_rb,0.28362e-05_rb,0.28143e-05_rb,0.28097e-05_rb,0.29057e-05_rb /)
      kbo(:,37, 4) = (/ &
        & 0.23374e-05_rb,0.22987e-05_rb,0.22934e-05_rb,0.22812e-05_rb,0.23451e-05_rb /)
      kbo(:,38, 4) = (/ &
        & 0.18966e-05_rb,0.18679e-05_rb,0.18613e-05_rb,0.18437e-05_rb,0.18894e-05_rb /)
      kbo(:,39, 4) = (/ &
        & 0.15300e-05_rb,0.15180e-05_rb,0.15096e-05_rb,0.14915e-05_rb,0.15107e-05_rb /)
      kbo(:,40, 4) = (/ &
        & 0.12426e-05_rb,0.12357e-05_rb,0.12171e-05_rb,0.12170e-05_rb,0.12227e-05_rb /)
      kbo(:,41, 4) = (/ &
        & 0.10102e-05_rb,0.10077e-05_rb,0.98593e-06_rb,0.98478e-06_rb,0.99213e-06_rb /)
      kbo(:,42, 4) = (/ &
        & 0.82201e-06_rb,0.81050e-06_rb,0.80313e-06_rb,0.80181e-06_rb,0.80361e-06_rb /)
      kbo(:,43, 4) = (/ &
        & 0.67318e-06_rb,0.65347e-06_rb,0.65478e-06_rb,0.64360e-06_rb,0.64722e-06_rb /)
      kbo(:,44, 4) = (/ &
        & 0.55598e-06_rb,0.53355e-06_rb,0.52778e-06_rb,0.52097e-06_rb,0.51997e-06_rb /)
      kbo(:,45, 4) = (/ &
        & 0.46157e-06_rb,0.43457e-06_rb,0.42412e-06_rb,0.42137e-06_rb,0.41750e-06_rb /)
      kbo(:,46, 4) = (/ &
        & 0.39156e-06_rb,0.35690e-06_rb,0.34498e-06_rb,0.33995e-06_rb,0.33614e-06_rb /)
      kbo(:,47, 4) = (/ &
        & 0.33392e-06_rb,0.29746e-06_rb,0.28074e-06_rb,0.27344e-06_rb,0.27079e-06_rb /)
      kbo(:,48, 4) = (/ &
        & 0.27066e-06_rb,0.25369e-06_rb,0.23115e-06_rb,0.22213e-06_rb,0.21766e-06_rb /)
      kbo(:,49, 4) = (/ &
        & 0.22198e-06_rb,0.21395e-06_rb,0.19274e-06_rb,0.18040e-06_rb,0.17542e-06_rb /)
      kbo(:,50, 4) = (/ &
        & 0.18070e-06_rb,0.17367e-06_rb,0.16473e-06_rb,0.14916e-06_rb,0.14306e-06_rb /)
      kbo(:,51, 4) = (/ &
        & 0.14595e-06_rb,0.14287e-06_rb,0.13847e-06_rb,0.12418e-06_rb,0.11697e-06_rb /)
      kbo(:,52, 4) = (/ &
        & 0.11819e-06_rb,0.11588e-06_rb,0.11225e-06_rb,0.10604e-06_rb,0.96269e-07_rb /)
      kbo(:,53, 4) = (/ &
        & 0.95028e-07_rb,0.93094e-07_rb,0.91303e-07_rb,0.88900e-07_rb,0.80262e-07_rb /)
      kbo(:,54, 4) = (/ &
        & 0.77005e-07_rb,0.76238e-07_rb,0.74228e-07_rb,0.72526e-07_rb,0.67965e-07_rb /)
      kbo(:,55, 4) = (/ &
        & 0.62979e-07_rb,0.62692e-07_rb,0.60930e-07_rb,0.59579e-07_rb,0.57700e-07_rb /)
      kbo(:,56, 4) = (/ &
        & 0.51571e-07_rb,0.51196e-07_rb,0.50031e-07_rb,0.48916e-07_rb,0.48127e-07_rb /)
      kbo(:,57, 4) = (/ &
        & 0.42602e-07_rb,0.41678e-07_rb,0.41006e-07_rb,0.39709e-07_rb,0.39283e-07_rb /)
      kbo(:,58, 4) = (/ &
        & 0.35155e-07_rb,0.34052e-07_rb,0.33561e-07_rb,0.32395e-07_rb,0.32057e-07_rb /)
      kbo(:,59, 4) = (/ &
        & 0.29318e-07_rb,0.28516e-07_rb,0.28097e-07_rb,0.26888e-07_rb,0.26840e-07_rb /)
      kbo(:,13, 5) = (/ &
        & 0.43718e-03_rb,0.42279e-03_rb,0.43200e-03_rb,0.42910e-03_rb,0.42775e-03_rb /)
      kbo(:,14, 5) = (/ &
        & 0.37902e-03_rb,0.38090e-03_rb,0.38058e-03_rb,0.37637e-03_rb,0.36798e-03_rb /)
      kbo(:,15, 5) = (/ &
        & 0.33222e-03_rb,0.33597e-03_rb,0.33205e-03_rb,0.32000e-03_rb,0.31310e-03_rb /)
      kbo(:,16, 5) = (/ &
        & 0.28736e-03_rb,0.28700e-03_rb,0.27853e-03_rb,0.26891e-03_rb,0.26391e-03_rb /)
      kbo(:,17, 5) = (/ &
        & 0.24534e-03_rb,0.24098e-03_rb,0.23072e-03_rb,0.22519e-03_rb,0.22013e-03_rb /)
      kbo(:,18, 5) = (/ &
        & 0.20645e-03_rb,0.20019e-03_rb,0.19266e-03_rb,0.18716e-03_rb,0.18139e-03_rb /)
      kbo(:,19, 5) = (/ &
        & 0.17271e-03_rb,0.16642e-03_rb,0.16064e-03_rb,0.15556e-03_rb,0.14951e-03_rb /)
      kbo(:,20, 5) = (/ &
        & 0.14266e-03_rb,0.13766e-03_rb,0.13239e-03_rb,0.12767e-03_rb,0.12305e-03_rb /)
      kbo(:,21, 5) = (/ &
        & 0.11797e-03_rb,0.11256e-03_rb,0.10880e-03_rb,0.10424e-03_rb,0.10062e-03_rb /)
      kbo(:,22, 5) = (/ &
        & 0.97085e-04_rb,0.92101e-04_rb,0.88420e-04_rb,0.85535e-04_rb,0.82201e-04_rb /)
      kbo(:,23, 5) = (/ &
        & 0.79380e-04_rb,0.75312e-04_rb,0.72177e-04_rb,0.69710e-04_rb,0.66966e-04_rb /)
      kbo(:,24, 5) = (/ &
        & 0.64591e-04_rb,0.61520e-04_rb,0.58891e-04_rb,0.56626e-04_rb,0.54603e-04_rb /)
      kbo(:,25, 5) = (/ &
        & 0.52423e-04_rb,0.49934e-04_rb,0.48122e-04_rb,0.46331e-04_rb,0.44875e-04_rb /)
      kbo(:,26, 5) = (/ &
        & 0.42528e-04_rb,0.40739e-04_rb,0.39218e-04_rb,0.37806e-04_rb,0.36631e-04_rb /)
      kbo(:,27, 5) = (/ &
        & 0.34453e-04_rb,0.33104e-04_rb,0.32009e-04_rb,0.30764e-04_rb,0.30000e-04_rb /)
      kbo(:,28, 5) = (/ &
        & 0.28076e-04_rb,0.26948e-04_rb,0.26097e-04_rb,0.25065e-04_rb,0.24596e-04_rb /)
      kbo(:,29, 5) = (/ &
        & 0.22814e-04_rb,0.21902e-04_rb,0.21222e-04_rb,0.20350e-04_rb,0.20152e-04_rb /)
      kbo(:,30, 5) = (/ &
        & 0.18551e-04_rb,0.17731e-04_rb,0.17162e-04_rb,0.16614e-04_rb,0.16461e-04_rb /)
      kbo(:,31, 5) = (/ &
        & 0.15020e-04_rb,0.14383e-04_rb,0.13922e-04_rb,0.13548e-04_rb,0.13388e-04_rb /)
      kbo(:,32, 5) = (/ &
        & 0.12091e-04_rb,0.11653e-04_rb,0.11308e-04_rb,0.11033e-04_rb,0.10954e-04_rb /)
      kbo(:,33, 5) = (/ &
        & 0.97901e-05_rb,0.93570e-05_rb,0.91428e-05_rb,0.89594e-05_rb,0.89614e-05_rb /)
      kbo(:,34, 5) = (/ &
        & 0.79439e-05_rb,0.76095e-05_rb,0.74365e-05_rb,0.73099e-05_rb,0.73269e-05_rb /)
      kbo(:,35, 5) = (/ &
        & 0.64505e-05_rb,0.62326e-05_rb,0.60921e-05_rb,0.59939e-05_rb,0.59799e-05_rb /)
      kbo(:,36, 5) = (/ &
        & 0.52745e-05_rb,0.51017e-05_rb,0.49658e-05_rb,0.48988e-05_rb,0.48773e-05_rb /)
      kbo(:,37, 5) = (/ &
        & 0.43522e-05_rb,0.42053e-05_rb,0.40681e-05_rb,0.40176e-05_rb,0.40058e-05_rb /)
      kbo(:,38, 5) = (/ &
        & 0.35676e-05_rb,0.34559e-05_rb,0.33446e-05_rb,0.33017e-05_rb,0.32771e-05_rb /)
      kbo(:,39, 5) = (/ &
        & 0.29238e-05_rb,0.28445e-05_rb,0.27512e-05_rb,0.27095e-05_rb,0.26825e-05_rb /)
      kbo(:,40, 5) = (/ &
        & 0.23985e-05_rb,0.23437e-05_rb,0.22756e-05_rb,0.22251e-05_rb,0.22009e-05_rb /)
      kbo(:,41, 5) = (/ &
        & 0.19763e-05_rb,0.19311e-05_rb,0.18886e-05_rb,0.18370e-05_rb,0.18072e-05_rb /)
      kbo(:,42, 5) = (/ &
        & 0.16256e-05_rb,0.15918e-05_rb,0.15558e-05_rb,0.15108e-05_rb,0.14861e-05_rb /)
      kbo(:,43, 5) = (/ &
        & 0.13290e-05_rb,0.13116e-05_rb,0.12833e-05_rb,0.12550e-05_rb,0.12262e-05_rb /)
      kbo(:,44, 5) = (/ &
        & 0.10777e-05_rb,0.10760e-05_rb,0.10566e-05_rb,0.10365e-05_rb,0.10136e-05_rb /)
      kbo(:,45, 5) = (/ &
        & 0.86690e-06_rb,0.87981e-06_rb,0.87044e-06_rb,0.85428e-06_rb,0.83627e-06_rb /)
      kbo(:,46, 5) = (/ &
        & 0.68706e-06_rb,0.71088e-06_rb,0.70939e-06_rb,0.70110e-06_rb,0.69032e-06_rb /)
      kbo(:,47, 5) = (/ &
        & 0.54665e-06_rb,0.56934e-06_rb,0.58033e-06_rb,0.57830e-06_rb,0.57105e-06_rb /)
      kbo(:,48, 5) = (/ &
        & 0.44874e-06_rb,0.44653e-06_rb,0.46650e-06_rb,0.47090e-06_rb,0.46732e-06_rb /)
      kbo(:,49, 5) = (/ &
        & 0.36869e-06_rb,0.35456e-06_rb,0.36842e-06_rb,0.37978e-06_rb,0.38217e-06_rb /)
      kbo(:,50, 5) = (/ &
        & 0.30489e-06_rb,0.29229e-06_rb,0.29104e-06_rb,0.30430e-06_rb,0.31068e-06_rb /)
      kbo(:,51, 5) = (/ &
        & 0.25290e-06_rb,0.24012e-06_rb,0.23271e-06_rb,0.24366e-06_rb,0.25149e-06_rb /)
      kbo(:,52, 5) = (/ &
        & 0.21148e-06_rb,0.19715e-06_rb,0.19121e-06_rb,0.19177e-06_rb,0.20148e-06_rb /)
      kbo(:,53, 5) = (/ &
        & 0.17334e-06_rb,0.16335e-06_rb,0.15574e-06_rb,0.15266e-06_rb,0.15921e-06_rb /)
      kbo(:,54, 5) = (/ &
        & 0.14189e-06_rb,0.13650e-06_rb,0.12781e-06_rb,0.12535e-06_rb,0.12749e-06_rb /)
      kbo(:,55, 5) = (/ &
        & 0.11763e-06_rb,0.11398e-06_rb,0.10627e-06_rb,0.10373e-06_rb,0.10268e-06_rb /)
      kbo(:,56, 5) = (/ &
        & 0.98316e-07_rb,0.94892e-07_rb,0.88799e-07_rb,0.85036e-07_rb,0.83393e-07_rb /)
      kbo(:,57, 5) = (/ &
        & 0.83049e-07_rb,0.78499e-07_rb,0.74140e-07_rb,0.70137e-07_rb,0.68774e-07_rb /)
      kbo(:,58, 5) = (/ &
        & 0.71768e-07_rb,0.64682e-07_rb,0.62012e-07_rb,0.58149e-07_rb,0.57037e-07_rb /)
      kbo(:,59, 5) = (/ &
        & 0.60166e-07_rb,0.54824e-07_rb,0.52042e-07_rb,0.49198e-07_rb,0.47862e-07_rb /)
      kbo(:,13, 6) = (/ &
        & 0.71254e-03_rb,0.65708e-03_rb,0.60824e-03_rb,0.57965e-03_rb,0.55389e-03_rb /)
      kbo(:,14, 6) = (/ &
        & 0.58852e-03_rb,0.54811e-03_rb,0.51125e-03_rb,0.48557e-03_rb,0.47730e-03_rb /)
      kbo(:,15, 6) = (/ &
        & 0.50599e-03_rb,0.46869e-03_rb,0.44362e-03_rb,0.43503e-03_rb,0.43705e-03_rb /)
      kbo(:,16, 6) = (/ &
        & 0.43493e-03_rb,0.40996e-03_rb,0.39680e-03_rb,0.39623e-03_rb,0.40784e-03_rb /)
      kbo(:,17, 6) = (/ &
        & 0.38028e-03_rb,0.36175e-03_rb,0.35696e-03_rb,0.36547e-03_rb,0.38127e-03_rb /)
      kbo(:,18, 6) = (/ &
        & 0.33113e-03_rb,0.32205e-03_rb,0.32458e-03_rb,0.33502e-03_rb,0.35577e-03_rb /)
      kbo(:,19, 6) = (/ &
        & 0.28886e-03_rb,0.29134e-03_rb,0.29799e-03_rb,0.31134e-03_rb,0.33676e-03_rb /)
      kbo(:,20, 6) = (/ &
        & 0.25113e-03_rb,0.25555e-03_rb,0.26535e-03_rb,0.28470e-03_rb,0.30677e-03_rb /)
      kbo(:,21, 6) = (/ &
        & 0.21640e-03_rb,0.22152e-03_rb,0.23323e-03_rb,0.25467e-03_rb,0.27764e-03_rb /)
      kbo(:,22, 6) = (/ &
        & 0.18601e-03_rb,0.19217e-03_rb,0.20696e-03_rb,0.22593e-03_rb,0.24729e-03_rb /)
      kbo(:,23, 6) = (/ &
        & 0.15978e-03_rb,0.16833e-03_rb,0.18229e-03_rb,0.19994e-03_rb,0.21913e-03_rb /)
      kbo(:,24, 6) = (/ &
        & 0.13748e-03_rb,0.14696e-03_rb,0.16025e-03_rb,0.17620e-03_rb,0.19459e-03_rb /)
      kbo(:,25, 6) = (/ &
        & 0.11841e-03_rb,0.12828e-03_rb,0.13998e-03_rb,0.15487e-03_rb,0.17174e-03_rb /)
      kbo(:,26, 6) = (/ &
        & 0.10209e-03_rb,0.11175e-03_rb,0.12193e-03_rb,0.13590e-03_rb,0.15125e-03_rb /)
      kbo(:,27, 6) = (/ &
        & 0.87003e-04_rb,0.95681e-04_rb,0.10499e-03_rb,0.11744e-03_rb,0.13086e-03_rb /)
      kbo(:,28, 6) = (/ &
        & 0.73110e-04_rb,0.80653e-04_rb,0.89103e-04_rb,0.99988e-04_rb,0.11120e-03_rb /)
      kbo(:,29, 6) = (/ &
        & 0.60620e-04_rb,0.66736e-04_rb,0.74222e-04_rb,0.82863e-04_rb,0.92471e-04_rb /)
      kbo(:,30, 6) = (/ &
        & 0.49831e-04_rb,0.54602e-04_rb,0.61288e-04_rb,0.68281e-04_rb,0.75599e-04_rb /)
      kbo(:,31, 6) = (/ &
        & 0.40315e-04_rb,0.44435e-04_rb,0.49703e-04_rb,0.55140e-04_rb,0.60140e-04_rb /)
      kbo(:,32, 6) = (/ &
        & 0.32490e-04_rb,0.36020e-04_rb,0.39894e-04_rb,0.44068e-04_rb,0.48080e-04_rb /)
      kbo(:,33, 6) = (/ &
        & 0.26085e-04_rb,0.29063e-04_rb,0.32003e-04_rb,0.35074e-04_rb,0.38662e-04_rb /)
      kbo(:,34, 6) = (/ &
        & 0.21201e-04_rb,0.23256e-04_rb,0.25636e-04_rb,0.28140e-04_rb,0.31273e-04_rb /)
      kbo(:,35, 6) = (/ &
        & 0.17022e-04_rb,0.18534e-04_rb,0.20322e-04_rb,0.22464e-04_rb,0.25109e-04_rb /)
      kbo(:,36, 6) = (/ &
        & 0.13427e-04_rb,0.14606e-04_rb,0.16055e-04_rb,0.17783e-04_rb,0.19877e-04_rb /)
      kbo(:,37, 6) = (/ &
        & 0.10630e-04_rb,0.11559e-04_rb,0.12731e-04_rb,0.14118e-04_rb,0.15831e-04_rb /)
      kbo(:,38, 6) = (/ &
        & 0.84365e-05_rb,0.91300e-05_rb,0.10077e-04_rb,0.11171e-04_rb,0.12568e-04_rb /)
      kbo(:,39, 6) = (/ &
        & 0.66762e-05_rb,0.71770e-05_rb,0.79359e-05_rb,0.88044e-05_rb,0.99043e-05_rb /)
      kbo(:,40, 6) = (/ &
        & 0.53561e-05_rb,0.57216e-05_rb,0.63210e-05_rb,0.70080e-05_rb,0.79213e-05_rb /)
      kbo(:,41, 6) = (/ &
        & 0.42954e-05_rb,0.45692e-05_rb,0.50356e-05_rb,0.56102e-05_rb,0.63394e-05_rb /)
      kbo(:,42, 6) = (/ &
        & 0.34409e-05_rb,0.36788e-05_rb,0.40124e-05_rb,0.44898e-05_rb,0.50645e-05_rb /)
      kbo(:,43, 6) = (/ &
        & 0.27453e-05_rb,0.29361e-05_rb,0.31651e-05_rb,0.35498e-05_rb,0.40058e-05_rb /)
      kbo(:,44, 6) = (/ &
        & 0.21863e-05_rb,0.23320e-05_rb,0.24965e-05_rb,0.27915e-05_rb,0.31593e-05_rb /)
      kbo(:,45, 6) = (/ &
        & 0.17409e-05_rb,0.18472e-05_rb,0.19801e-05_rb,0.21812e-05_rb,0.24741e-05_rb /)
      kbo(:,46, 6) = (/ &
        & 0.13903e-05_rb,0.14639e-05_rb,0.15660e-05_rb,0.17042e-05_rb,0.19243e-05_rb /)
      kbo(:,47, 6) = (/ &
        & 0.11229e-05_rb,0.11661e-05_rb,0.12377e-05_rb,0.13335e-05_rb,0.14898e-05_rb /)
      kbo(:,48, 6) = (/ &
        & 0.90528e-06_rb,0.92962e-06_rb,0.97799e-06_rb,0.10483e-05_rb,0.11575e-05_rb /)
      kbo(:,49, 6) = (/ &
        & 0.72877e-06_rb,0.74278e-06_rb,0.77380e-06_rb,0.82564e-06_rb,0.89736e-06_rb /)
      kbo(:,50, 6) = (/ &
        & 0.59358e-06_rb,0.59955e-06_rb,0.62170e-06_rb,0.65720e-06_rb,0.70771e-06_rb /)
      kbo(:,51, 6) = (/ &
        & 0.48783e-06_rb,0.48769e-06_rb,0.50300e-06_rb,0.52655e-06_rb,0.56377e-06_rb /)
      kbo(:,52, 6) = (/ &
        & 0.39834e-06_rb,0.39922e-06_rb,0.40604e-06_rb,0.42316e-06_rb,0.45057e-06_rb /)
      kbo(:,53, 6) = (/ &
        & 0.33019e-06_rb,0.32505e-06_rb,0.33011e-06_rb,0.33917e-06_rb,0.35858e-06_rb /)
      kbo(:,54, 6) = (/ &
        & 0.27829e-06_rb,0.26763e-06_rb,0.27219e-06_rb,0.27714e-06_rb,0.29159e-06_rb /)
      kbo(:,55, 6) = (/ &
        & 0.23259e-06_rb,0.22250e-06_rb,0.22510e-06_rb,0.22855e-06_rb,0.23937e-06_rb /)
      kbo(:,56, 6) = (/ &
        & 0.19401e-06_rb,0.18605e-06_rb,0.18566e-06_rb,0.18937e-06_rb,0.19605e-06_rb /)
      kbo(:,57, 6) = (/ &
        & 0.15908e-06_rb,0.15649e-06_rb,0.15396e-06_rb,0.15754e-06_rb,0.16039e-06_rb /)
      kbo(:,58, 6) = (/ &
        & 0.12892e-06_rb,0.13132e-06_rb,0.12793e-06_rb,0.13093e-06_rb,0.13273e-06_rb /)
      kbo(:,59, 6) = (/ &
        & 0.11177e-06_rb,0.11165e-06_rb,0.10875e-06_rb,0.11132e-06_rb,0.11431e-06_rb /)
      kbo(:,13, 7) = (/ &
        & 0.19357e-02_rb,0.20341e-02_rb,0.21760e-02_rb,0.22982e-02_rb,0.24550e-02_rb /)
      kbo(:,14, 7) = (/ &
        & 0.20343e-02_rb,0.21709e-02_rb,0.23192e-02_rb,0.24854e-02_rb,0.27026e-02_rb /)
      kbo(:,15, 7) = (/ &
        & 0.21108e-02_rb,0.23009e-02_rb,0.25488e-02_rb,0.28048e-02_rb,0.30159e-02_rb /)
      kbo(:,16, 7) = (/ &
        & 0.21016e-02_rb,0.23581e-02_rb,0.25958e-02_rb,0.28093e-02_rb,0.29839e-02_rb /)
      kbo(:,17, 7) = (/ &
        & 0.21350e-02_rb,0.23722e-02_rb,0.25718e-02_rb,0.27703e-02_rb,0.30036e-02_rb /)
      kbo(:,18, 7) = (/ &
        & 0.21493e-02_rb,0.23550e-02_rb,0.25593e-02_rb,0.27827e-02_rb,0.29722e-02_rb /)
      kbo(:,19, 7) = (/ &
        & 0.21677e-02_rb,0.23602e-02_rb,0.25446e-02_rb,0.27267e-02_rb,0.29542e-02_rb /)
      kbo(:,20, 7) = (/ &
        & 0.21009e-02_rb,0.22669e-02_rb,0.24261e-02_rb,0.26095e-02_rb,0.28436e-02_rb /)
      kbo(:,21, 7) = (/ &
        & 0.19811e-02_rb,0.21249e-02_rb,0.22790e-02_rb,0.24716e-02_rb,0.26844e-02_rb /)
      kbo(:,22, 7) = (/ &
        & 0.18393e-02_rb,0.19585e-02_rb,0.21127e-02_rb,0.22962e-02_rb,0.25211e-02_rb /)
      kbo(:,23, 7) = (/ &
        & 0.16905e-02_rb,0.18032e-02_rb,0.19594e-02_rb,0.21280e-02_rb,0.23466e-02_rb /)
      kbo(:,24, 7) = (/ &
        & 0.15327e-02_rb,0.16542e-02_rb,0.18047e-02_rb,0.19642e-02_rb,0.21591e-02_rb /)
      kbo(:,25, 7) = (/ &
        & 0.13831e-02_rb,0.14932e-02_rb,0.16391e-02_rb,0.17921e-02_rb,0.19606e-02_rb /)
      kbo(:,26, 7) = (/ &
        & 0.12298e-02_rb,0.13325e-02_rb,0.14676e-02_rb,0.16086e-02_rb,0.17628e-02_rb /)
      kbo(:,27, 7) = (/ &
        & 0.10692e-02_rb,0.11652e-02_rb,0.12858e-02_rb,0.14124e-02_rb,0.15514e-02_rb /)
      kbo(:,28, 7) = (/ &
        & 0.91706e-03_rb,0.10034e-02_rb,0.11100e-02_rb,0.12231e-02_rb,0.13481e-02_rb /)
      kbo(:,29, 7) = (/ &
        & 0.76824e-03_rb,0.84492e-03_rb,0.93654e-03_rb,0.10377e-02_rb,0.11448e-02_rb /)
      kbo(:,30, 7) = (/ &
        & 0.63841e-03_rb,0.70573e-03_rb,0.78553e-03_rb,0.87197e-03_rb,0.96607e-03_rb /)
      kbo(:,31, 7) = (/ &
        & 0.51979e-03_rb,0.58056e-03_rb,0.64728e-03_rb,0.71891e-03_rb,0.80875e-03_rb /)
      kbo(:,32, 7) = (/ &
        & 0.42659e-03_rb,0.47686e-03_rb,0.53347e-03_rb,0.60016e-03_rb,0.67592e-03_rb /)
      kbo(:,33, 7) = (/ &
        & 0.34820e-03_rb,0.39030e-03_rb,0.44004e-03_rb,0.49875e-03_rb,0.56492e-03_rb /)
      kbo(:,34, 7) = (/ &
        & 0.28753e-03_rb,0.32477e-03_rb,0.36831e-03_rb,0.41989e-03_rb,0.47708e-03_rb /)
      kbo(:,35, 7) = (/ &
        & 0.23568e-03_rb,0.26716e-03_rb,0.30618e-03_rb,0.35062e-03_rb,0.39934e-03_rb /)
      kbo(:,36, 7) = (/ &
        & 0.19116e-03_rb,0.21863e-03_rb,0.25180e-03_rb,0.28983e-03_rb,0.33082e-03_rb /)
      kbo(:,37, 7) = (/ &
        & 0.15437e-03_rb,0.17829e-03_rb,0.20646e-03_rb,0.23900e-03_rb,0.27320e-03_rb /)
      kbo(:,38, 7) = (/ &
        & 0.12397e-03_rb,0.14462e-03_rb,0.16847e-03_rb,0.19621e-03_rb,0.22528e-03_rb /)
      kbo(:,39, 7) = (/ &
        & 0.98930e-04_rb,0.11647e-03_rb,0.13711e-03_rb,0.16057e-03_rb,0.18549e-03_rb /)
      kbo(:,40, 7) = (/ &
        & 0.79845e-04_rb,0.94437e-04_rb,0.11202e-03_rb,0.13217e-03_rb,0.15346e-03_rb /)
      kbo(:,41, 7) = (/ &
        & 0.64260e-04_rb,0.76439e-04_rb,0.91253e-04_rb,0.10837e-03_rb,0.12691e-03_rb /)
      kbo(:,42, 7) = (/ &
        & 0.51561e-04_rb,0.61686e-04_rb,0.74044e-04_rb,0.88504e-04_rb,0.10431e-03_rb /)
      kbo(:,43, 7) = (/ &
        & 0.40906e-04_rb,0.49267e-04_rb,0.59361e-04_rb,0.71417e-04_rb,0.84751e-04_rb /)
      kbo(:,44, 7) = (/ &
        & 0.32247e-04_rb,0.38955e-04_rb,0.47246e-04_rb,0.57199e-04_rb,0.68540e-04_rb /)
      kbo(:,45, 7) = (/ &
        & 0.25243e-04_rb,0.30790e-04_rb,0.37490e-04_rb,0.45754e-04_rb,0.55394e-04_rb /)
      kbo(:,46, 7) = (/ &
        & 0.19604e-04_rb,0.24131e-04_rb,0.29598e-04_rb,0.36431e-04_rb,0.44419e-04_rb /)
      kbo(:,47, 7) = (/ &
        & 0.15116e-04_rb,0.18785e-04_rb,0.23180e-04_rb,0.28780e-04_rb,0.35381e-04_rb /)
      kbo(:,48, 7) = (/ &
        & 0.11550e-04_rb,0.14451e-04_rb,0.18039e-04_rb,0.22545e-04_rb,0.28005e-04_rb /)
      kbo(:,49, 7) = (/ &
        & 0.87452e-05_rb,0.10992e-04_rb,0.13863e-04_rb,0.17498e-04_rb,0.21949e-04_rb /)
      kbo(:,50, 7) = (/ &
        & 0.66768e-05_rb,0.84974e-05_rb,0.10835e-04_rb,0.13777e-04_rb,0.17448e-04_rb /)
      kbo(:,51, 7) = (/ &
        & 0.51359e-05_rb,0.66277e-05_rb,0.85196e-05_rb,0.10905e-04_rb,0.13933e-04_rb /)
      kbo(:,52, 7) = (/ &
        & 0.38737e-05_rb,0.51001e-05_rb,0.66347e-05_rb,0.85842e-05_rb,0.11033e-04_rb /)
      kbo(:,53, 7) = (/ &
        & 0.28793e-05_rb,0.38820e-05_rb,0.50987e-05_rb,0.66800e-05_rb,0.86846e-05_rb /)
      kbo(:,54, 7) = (/ &
        & 0.22250e-05_rb,0.30528e-05_rb,0.40721e-05_rb,0.53813e-05_rb,0.70529e-05_rb /)
      kbo(:,55, 7) = (/ &
        & 0.17548e-05_rb,0.24455e-05_rb,0.32962e-05_rb,0.43877e-05_rb,0.58071e-05_rb /)
      kbo(:,56, 7) = (/ &
        & 0.13740e-05_rb,0.19413e-05_rb,0.26523e-05_rb,0.35566e-05_rb,0.47512e-05_rb /)
      kbo(:,57, 7) = (/ &
        & 0.10745e-05_rb,0.15242e-05_rb,0.21190e-05_rb,0.28679e-05_rb,0.38612e-05_rb /)
      kbo(:,58, 7) = (/ &
        & 0.84271e-06_rb,0.12117e-05_rb,0.17039e-05_rb,0.23246e-05_rb,0.31564e-05_rb /)
      kbo(:,59, 7) = (/ &
        & 0.75625e-06_rb,0.11002e-05_rb,0.15376e-05_rb,0.20855e-05_rb,0.28548e-05_rb /)
      kbo(:,13, 8) = (/ &
        & 0.41133e-01_rb,0.43585e-01_rb,0.46448e-01_rb,0.50156e-01_rb,0.54607e-01_rb /)
      kbo(:,14, 8) = (/ &
        & 0.39162e-01_rb,0.41636e-01_rb,0.44649e-01_rb,0.48118e-01_rb,0.52078e-01_rb /)
      kbo(:,15, 8) = (/ &
        & 0.37770e-01_rb,0.40048e-01_rb,0.43090e-01_rb,0.46524e-01_rb,0.50698e-01_rb /)
      kbo(:,16, 8) = (/ &
        & 0.35235e-01_rb,0.37712e-01_rb,0.40429e-01_rb,0.44092e-01_rb,0.48470e-01_rb /)
      kbo(:,17, 8) = (/ &
        & 0.32748e-01_rb,0.35232e-01_rb,0.38049e-01_rb,0.41536e-01_rb,0.45596e-01_rb /)
      kbo(:,18, 8) = (/ &
        & 0.30656e-01_rb,0.33078e-01_rb,0.36084e-01_rb,0.39403e-01_rb,0.43319e-01_rb /)
      kbo(:,19, 8) = (/ &
        & 0.28887e-01_rb,0.31462e-01_rb,0.34348e-01_rb,0.37660e-01_rb,0.41532e-01_rb /)
      kbo(:,20, 8) = (/ &
        & 0.26521e-01_rb,0.29153e-01_rb,0.32006e-01_rb,0.35368e-01_rb,0.39083e-01_rb /)
      kbo(:,21, 8) = (/ &
        & 0.23988e-01_rb,0.26537e-01_rb,0.29508e-01_rb,0.32666e-01_rb,0.36356e-01_rb /)
      kbo(:,22, 8) = (/ &
        & 0.21655e-01_rb,0.24205e-01_rb,0.27193e-01_rb,0.30346e-01_rb,0.33820e-01_rb /)
      kbo(:,23, 8) = (/ &
        & 0.19724e-01_rb,0.22257e-01_rb,0.25021e-01_rb,0.27997e-01_rb,0.31309e-01_rb /)
      kbo(:,24, 8) = (/ &
        & 0.18094e-01_rb,0.20464e-01_rb,0.23167e-01_rb,0.26069e-01_rb,0.29357e-01_rb /)
      kbo(:,25, 8) = (/ &
        & 0.16666e-01_rb,0.18896e-01_rb,0.21422e-01_rb,0.24282e-01_rb,0.27401e-01_rb /)
      kbo(:,26, 8) = (/ &
        & 0.15252e-01_rb,0.17451e-01_rb,0.19828e-01_rb,0.22565e-01_rb,0.25604e-01_rb /)
      kbo(:,27, 8) = (/ &
        & 0.13902e-01_rb,0.15940e-01_rb,0.18143e-01_rb,0.20720e-01_rb,0.23727e-01_rb /)
      kbo(:,28, 8) = (/ &
        & 0.12642e-01_rb,0.14401e-01_rb,0.16467e-01_rb,0.18975e-01_rb,0.21740e-01_rb /)
      kbo(:,29, 8) = (/ &
        & 0.11177e-01_rb,0.12765e-01_rb,0.14750e-01_rb,0.16957e-01_rb,0.19575e-01_rb /)
      kbo(:,30, 8) = (/ &
        & 0.97810e-02_rb,0.11281e-01_rb,0.13066e-01_rb,0.15112e-01_rb,0.17581e-01_rb /)
      kbo(:,31, 8) = (/ &
        & 0.84562e-02_rb,0.98553e-02_rb,0.11427e-01_rb,0.13312e-01_rb,0.15536e-01_rb /)
      kbo(:,32, 8) = (/ &
        & 0.73434e-02_rb,0.85939e-02_rb,0.10020e-01_rb,0.11770e-01_rb,0.13758e-01_rb /)
      kbo(:,33, 8) = (/ &
        & 0.63801e-02_rb,0.74957e-02_rb,0.87874e-02_rb,0.10375e-01_rb,0.12145e-01_rb /)
      kbo(:,34, 8) = (/ &
        & 0.56002e-02_rb,0.65926e-02_rb,0.77964e-02_rb,0.92186e-02_rb,0.10768e-01_rb /)
      kbo(:,35, 8) = (/ &
        & 0.48489e-02_rb,0.57164e-02_rb,0.68333e-02_rb,0.80886e-02_rb,0.94500e-02_rb /)
      kbo(:,36, 8) = (/ &
        & 0.41428e-02_rb,0.49220e-02_rb,0.58884e-02_rb,0.69905e-02_rb,0.81862e-02_rb /)
      kbo(:,37, 8) = (/ &
        & 0.34967e-02_rb,0.41841e-02_rb,0.50244e-02_rb,0.59657e-02_rb,0.70167e-02_rb /)
      kbo(:,38, 8) = (/ &
        & 0.29333e-02_rb,0.35364e-02_rb,0.42644e-02_rb,0.50719e-02_rb,0.59938e-02_rb /)
      kbo(:,39, 8) = (/ &
        & 0.24443e-02_rb,0.29647e-02_rb,0.35953e-02_rb,0.42898e-02_rb,0.50972e-02_rb /)
      kbo(:,40, 8) = (/ &
        & 0.20439e-02_rb,0.25046e-02_rb,0.30512e-02_rb,0.36459e-02_rb,0.43466e-02_rb /)
      kbo(:,41, 8) = (/ &
        & 0.17068e-02_rb,0.21107e-02_rb,0.25838e-02_rb,0.30959e-02_rb,0.36971e-02_rb /)
      kbo(:,42, 8) = (/ &
        & 0.14199e-02_rb,0.17705e-02_rb,0.21755e-02_rb,0.26204e-02_rb,0.31356e-02_rb /)
      kbo(:,43, 8) = (/ &
        & 0.11616e-02_rb,0.14584e-02_rb,0.18069e-02_rb,0.21876e-02_rb,0.26286e-02_rb /)
      kbo(:,44, 8) = (/ &
        & 0.93807e-03_rb,0.11864e-02_rb,0.14851e-02_rb,0.18080e-02_rb,0.21849e-02_rb /)
      kbo(:,45, 8) = (/ &
        & 0.75026e-03_rb,0.95460e-03_rb,0.12092e-02_rb,0.14831e-02_rb,0.18037e-02_rb /)
      kbo(:,46, 8) = (/ &
        & 0.59319e-03_rb,0.76134e-03_rb,0.97292e-03_rb,0.12059e-02_rb,0.14787e-02_rb /)
      kbo(:,47, 8) = (/ &
        & 0.46546e-03_rb,0.60113e-03_rb,0.77762e-03_rb,0.97379e-03_rb,0.12030e-02_rb /)
      kbo(:,48, 8) = (/ &
        & 0.36131e-03_rb,0.46999e-03_rb,0.61292e-03_rb,0.77778e-03_rb,0.96667e-03_rb /)
      kbo(:,49, 8) = (/ &
        & 0.27490e-03_rb,0.36153e-03_rb,0.47561e-03_rb,0.61173e-03_rb,0.76634e-03_rb /)
      kbo(:,50, 8) = (/ &
        & 0.21403e-03_rb,0.28416e-03_rb,0.37592e-03_rb,0.48994e-03_rb,0.61812e-03_rb /)
      kbo(:,51, 8) = (/ &
        & 0.16745e-03_rb,0.22425e-03_rb,0.29943e-03_rb,0.39474e-03_rb,0.50310e-03_rb /)
      kbo(:,52, 8) = (/ &
        & 0.12927e-03_rb,0.17496e-03_rb,0.23541e-03_rb,0.31434e-03_rb,0.40442e-03_rb /)
      kbo(:,53, 8) = (/ &
        & 0.98512e-04_rb,0.13503e-03_rb,0.18276e-03_rb,0.24681e-03_rb,0.32086e-03_rb /)
      kbo(:,54, 8) = (/ &
        & 0.78150e-04_rb,0.10819e-03_rb,0.14843e-03_rb,0.20243e-03_rb,0.26646e-03_rb /)
      kbo(:,55, 8) = (/ &
        & 0.63455e-04_rb,0.88570e-04_rb,0.12297e-03_rb,0.17015e-03_rb,0.22626e-03_rb /)
      kbo(:,56, 8) = (/ &
        & 0.51095e-04_rb,0.72223e-04_rb,0.10144e-03_rb,0.14188e-03_rb,0.19132e-03_rb /)
      kbo(:,57, 8) = (/ &
        & 0.40866e-04_rb,0.58631e-04_rb,0.83096e-04_rb,0.11753e-03_rb,0.16055e-03_rb /)
      kbo(:,58, 8) = (/ &
        & 0.32890e-04_rb,0.47893e-04_rb,0.68644e-04_rb,0.98156e-04_rb,0.13564e-03_rb /)
      kbo(:,59, 8) = (/ &
        & 0.29702e-04_rb,0.43928e-04_rb,0.64217e-04_rb,0.92621e-04_rb,0.12965e-03_rb /)
      kbo(:,13, 9) = (/ &
        & 0.27690e+01_rb,0.28644e+01_rb,0.29696e+01_rb,0.30850e+01_rb,0.32162e+01_rb /)
      kbo(:,14, 9) = (/ &
        & 0.23162e+01_rb,0.23973e+01_rb,0.24876e+01_rb,0.25913e+01_rb,0.27107e+01_rb /)
      kbo(:,15, 9) = (/ &
        & 0.19377e+01_rb,0.20096e+01_rb,0.20909e+01_rb,0.21846e+01_rb,0.22904e+01_rb /)
      kbo(:,16, 9) = (/ &
        & 0.16221e+01_rb,0.16854e+01_rb,0.17596e+01_rb,0.18440e+01_rb,0.19374e+01_rb /)
      kbo(:,17, 9) = (/ &
        & 0.13581e+01_rb,0.14147e+01_rb,0.14813e+01_rb,0.15562e+01_rb,0.16364e+01_rb /)
      kbo(:,18, 9) = (/ &
        & 0.11494e+01_rb,0.11995e+01_rb,0.12558e+01_rb,0.13168e+01_rb,0.13828e+01_rb /)
      kbo(:,19, 9) = (/ &
        & 0.99857e+00_rb,0.10422e+01_rb,0.10890e+01_rb,0.11391e+01_rb,0.11917e+01_rb /)
      kbo(:,20, 9) = (/ &
        & 0.86585e+00_rb,0.90292e+00_rb,0.94341e+00_rb,0.98616e+00_rb,0.10332e+01_rb /)
      kbo(:,21, 9) = (/ &
        & 0.74752e+00_rb,0.77983e+00_rb,0.81433e+00_rb,0.85283e+00_rb,0.89462e+00_rb /)
      kbo(:,22, 9) = (/ &
        & 0.64494e+00_rb,0.67286e+00_rb,0.70277e+00_rb,0.73658e+00_rb,0.77352e+00_rb /)
      kbo(:,23, 9) = (/ &
        & 0.55485e+00_rb,0.57897e+00_rb,0.60654e+00_rb,0.63699e+00_rb,0.66979e+00_rb /)
      kbo(:,24, 9) = (/ &
        & 0.47664e+00_rb,0.49869e+00_rb,0.52329e+00_rb,0.54973e+00_rb,0.57955e+00_rb /)
      kbo(:,25, 9) = (/ &
        & 0.41114e+00_rb,0.42988e+00_rb,0.45044e+00_rb,0.47447e+00_rb,0.50163e+00_rb /)
      kbo(:,26, 9) = (/ &
        & 0.35513e+00_rb,0.37142e+00_rb,0.38942e+00_rb,0.41045e+00_rb,0.43493e+00_rb /)
      kbo(:,27, 9) = (/ &
        & 0.30546e+00_rb,0.31995e+00_rb,0.33673e+00_rb,0.35597e+00_rb,0.37742e+00_rb /)
      kbo(:,28, 9) = (/ &
        & 0.26188e+00_rb,0.27552e+00_rb,0.29117e+00_rb,0.30879e+00_rb,0.32839e+00_rb /)
      kbo(:,29, 9) = (/ &
        & 0.22377e+00_rb,0.23654e+00_rb,0.25102e+00_rb,0.26735e+00_rb,0.28587e+00_rb /)
      kbo(:,30, 9) = (/ &
        & 0.19129e+00_rb,0.20332e+00_rb,0.21691e+00_rb,0.23233e+00_rb,0.24924e+00_rb /)
      kbo(:,31, 9) = (/ &
        & 0.16347e+00_rb,0.17463e+00_rb,0.18745e+00_rb,0.20165e+00_rb,0.21712e+00_rb /)
      kbo(:,32, 9) = (/ &
        & 0.14037e+00_rb,0.15075e+00_rb,0.16282e+00_rb,0.17567e+00_rb,0.18973e+00_rb /)
      kbo(:,33, 9) = (/ &
        & 0.12095e+00_rb,0.13081e+00_rb,0.14161e+00_rb,0.15337e+00_rb,0.16632e+00_rb /)
      kbo(:,34, 9) = (/ &
        & 0.10468e+00_rb,0.11371e+00_rb,0.12351e+00_rb,0.13423e+00_rb,0.14632e+00_rb /)
      kbo(:,35, 9) = (/ &
        & 0.90259e-01_rb,0.98421e-01_rb,0.10717e+00_rb,0.11695e+00_rb,0.12831e+00_rb /)
      kbo(:,36, 9) = (/ &
        & 0.77272e-01_rb,0.84599e-01_rb,0.92464e-01_rb,0.10140e+00_rb,0.11193e+00_rb /)
      kbo(:,37, 9) = (/ &
        & 0.65606e-01_rb,0.72029e-01_rb,0.79063e-01_rb,0.87174e-01_rb,0.96643e-01_rb /)
      kbo(:,38, 9) = (/ &
        & 0.55528e-01_rb,0.61268e-01_rb,0.67613e-01_rb,0.74898e-01_rb,0.83456e-01_rb /)
      kbo(:,39, 9) = (/ &
        & 0.46985e-01_rb,0.52085e-01_rb,0.57765e-01_rb,0.64303e-01_rb,0.72021e-01_rb /)
      kbo(:,40, 9) = (/ &
        & 0.39626e-01_rb,0.44094e-01_rb,0.49150e-01_rb,0.55005e-01_rb,0.61884e-01_rb /)
      kbo(:,41, 9) = (/ &
        & 0.33376e-01_rb,0.37326e-01_rb,0.41783e-01_rb,0.46997e-01_rb,0.53158e-01_rb /)
      kbo(:,42, 9) = (/ &
        & 0.28074e-01_rb,0.31535e-01_rb,0.35493e-01_rb,0.40112e-01_rb,0.45645e-01_rb /)
      kbo(:,43, 9) = (/ &
        & 0.23419e-01_rb,0.26419e-01_rb,0.29889e-01_rb,0.33982e-01_rb,0.38867e-01_rb /)
      kbo(:,44, 9) = (/ &
        & 0.19437e-01_rb,0.22035e-01_rb,0.25042e-01_rb,0.28622e-01_rb,0.32930e-01_rb /)
      kbo(:,45, 9) = (/ &
        & 0.16083e-01_rb,0.18329e-01_rb,0.20937e-01_rb,0.24027e-01_rb,0.27786e-01_rb /)
      kbo(:,46, 9) = (/ &
        & 0.13236e-01_rb,0.15146e-01_rb,0.17420e-01_rb,0.20094e-01_rb,0.23328e-01_rb /)
      kbo(:,47, 9) = (/ &
        & 0.10820e-01_rb,0.12455e-01_rb,0.14377e-01_rb,0.16656e-01_rb,0.19464e-01_rb /)
      kbo(:,48, 9) = (/ &
        & 0.88043e-02_rb,0.10190e-01_rb,0.11813e-01_rb,0.13745e-01_rb,0.16141e-01_rb /)
      kbo(:,49, 9) = (/ &
        & 0.71234e-02_rb,0.82775e-02_rb,0.96453e-02_rb,0.11284e-01_rb,0.13310e-01_rb /)
      kbo(:,50, 9) = (/ &
        & 0.58127e-02_rb,0.67802e-02_rb,0.79379e-02_rb,0.93254e-02_rb,0.11078e-01_rb /)
      kbo(:,51, 9) = (/ &
        & 0.47494e-02_rb,0.55659e-02_rb,0.65602e-02_rb,0.77407e-02_rb,0.92288e-02_rb /)
      kbo(:,52, 9) = (/ &
        & 0.38618e-02_rb,0.45424e-02_rb,0.53925e-02_rb,0.64011e-02_rb,0.76645e-02_rb /)
      kbo(:,53, 9) = (/ &
        & 0.31264e-02_rb,0.36823e-02_rb,0.43952e-02_rb,0.52615e-02_rb,0.63311e-02_rb /)
      kbo(:,54, 9) = (/ &
        & 0.25794e-02_rb,0.30481e-02_rb,0.36548e-02_rb,0.44037e-02_rb,0.53380e-02_rb /)
      kbo(:,55, 9) = (/ &
        & 0.21508e-02_rb,0.25472e-02_rb,0.30665e-02_rb,0.37188e-02_rb,0.45472e-02_rb /)
      kbo(:,56, 9) = (/ &
        & 0.17938e-02_rb,0.21263e-02_rb,0.25713e-02_rb,0.31411e-02_rb,0.38667e-02_rb /)
      kbo(:,57, 9) = (/ &
        & 0.14949e-02_rb,0.17755e-02_rb,0.21510e-02_rb,0.26439e-02_rb,0.32830e-02_rb /)
      kbo(:,58, 9) = (/ &
        & 0.12521e-02_rb,0.14938e-02_rb,0.18137e-02_rb,0.22426e-02_rb,0.28028e-02_rb /)
      kbo(:,59, 9) = (/ &
        & 0.11302e-02_rb,0.13552e-02_rb,0.16582e-02_rb,0.20691e-02_rb,0.26051e-02_rb /)
      kbo(:,13,10) = (/ &
        & 0.14370e+02_rb,0.14649e+02_rb,0.14957e+02_rb,0.15296e+02_rb,0.15663e+02_rb /)
      kbo(:,14,10) = (/ &
        & 0.12178e+02_rb,0.12411e+02_rb,0.12682e+02_rb,0.12978e+02_rb,0.13316e+02_rb /)
      kbo(:,15,10) = (/ &
        & 0.10211e+02_rb,0.10402e+02_rb,0.10625e+02_rb,0.10894e+02_rb,0.11239e+02_rb /)
      kbo(:,16,10) = (/ &
        & 0.85974e+01_rb,0.87539e+01_rb,0.89410e+01_rb,0.91849e+01_rb,0.95108e+01_rb /)
      kbo(:,17,10) = (/ &
        & 0.72440e+01_rb,0.73858e+01_rb,0.75683e+01_rb,0.78199e+01_rb,0.81516e+01_rb /)
      kbo(:,18,10) = (/ &
        & 0.59680e+01_rb,0.61054e+01_rb,0.63108e+01_rb,0.66074e+01_rb,0.69367e+01_rb /)
      kbo(:,19,10) = (/ &
        & 0.47759e+01_rb,0.49000e+01_rb,0.51125e+01_rb,0.53858e+01_rb,0.56972e+01_rb /)
      kbo(:,20,10) = (/ &
        & 0.40682e+01_rb,0.42060e+01_rb,0.43988e+01_rb,0.46035e+01_rb,0.48052e+01_rb /)
      kbo(:,21,10) = (/ &
        & 0.35226e+01_rb,0.36715e+01_rb,0.38357e+01_rb,0.40017e+01_rb,0.41636e+01_rb /)
      kbo(:,22,10) = (/ &
        & 0.30730e+01_rb,0.32059e+01_rb,0.33607e+01_rb,0.35055e+01_rb,0.36621e+01_rb /)
      kbo(:,23,10) = (/ &
        & 0.26688e+01_rb,0.27999e+01_rb,0.29296e+01_rb,0.30668e+01_rb,0.32115e+01_rb /)
      kbo(:,24,10) = (/ &
        & 0.23228e+01_rb,0.24338e+01_rb,0.25510e+01_rb,0.26866e+01_rb,0.28223e+01_rb /)
      kbo(:,25,10) = (/ &
        & 0.19964e+01_rb,0.21083e+01_rb,0.22332e+01_rb,0.23566e+01_rb,0.24845e+01_rb /)
      kbo(:,26,10) = (/ &
        & 0.17097e+01_rb,0.18190e+01_rb,0.19420e+01_rb,0.20663e+01_rb,0.21952e+01_rb /)
      kbo(:,27,10) = (/ &
        & 0.14729e+01_rb,0.15741e+01_rb,0.16825e+01_rb,0.17963e+01_rb,0.19295e+01_rb /)
      kbo(:,28,10) = (/ &
        & 0.12788e+01_rb,0.13609e+01_rb,0.14562e+01_rb,0.15668e+01_rb,0.17032e+01_rb /)
      kbo(:,29,10) = (/ &
        & 0.11113e+01_rb,0.11813e+01_rb,0.12669e+01_rb,0.13763e+01_rb,0.15012e+01_rb /)
      kbo(:,30,10) = (/ &
        & 0.96661e+00_rb,0.10287e+01_rb,0.11081e+01_rb,0.12093e+01_rb,0.13273e+01_rb /)
      kbo(:,31,10) = (/ &
        & 0.83795e+00_rb,0.89792e+00_rb,0.97327e+00_rb,0.10727e+01_rb,0.11828e+01_rb /)
      kbo(:,32,10) = (/ &
        & 0.72634e+00_rb,0.78480e+00_rb,0.86071e+00_rb,0.95503e+00_rb,0.10615e+01_rb /)
      kbo(:,33,10) = (/ &
        & 0.63134e+00_rb,0.68789e+00_rb,0.76628e+00_rb,0.85854e+00_rb,0.96370e+00_rb /)
      kbo(:,34,10) = (/ &
        & 0.55172e+00_rb,0.60863e+00_rb,0.68672e+00_rb,0.77637e+00_rb,0.87664e+00_rb /)
      kbo(:,35,10) = (/ &
        & 0.48039e+00_rb,0.53701e+00_rb,0.61125e+00_rb,0.69795e+00_rb,0.79084e+00_rb /)
      kbo(:,36,10) = (/ &
        & 0.41819e+00_rb,0.47188e+00_rb,0.54096e+00_rb,0.62096e+00_rb,0.70693e+00_rb /)
      kbo(:,37,10) = (/ &
        & 0.36050e+00_rb,0.40906e+00_rb,0.47069e+00_rb,0.54184e+00_rb,0.62144e+00_rb /)
      kbo(:,38,10) = (/ &
        & 0.31156e+00_rb,0.35409e+00_rb,0.40958e+00_rb,0.47371e+00_rb,0.54611e+00_rb /)
      kbo(:,39,10) = (/ &
        & 0.26782e+00_rb,0.30677e+00_rb,0.35645e+00_rb,0.41535e+00_rb,0.48073e+00_rb /)
      kbo(:,40,10) = (/ &
        & 0.23006e+00_rb,0.26273e+00_rb,0.30670e+00_rb,0.35934e+00_rb,0.41871e+00_rb /)
      kbo(:,41,10) = (/ &
        & 0.19681e+00_rb,0.22601e+00_rb,0.26383e+00_rb,0.31041e+00_rb,0.36443e+00_rb /)
      kbo(:,42,10) = (/ &
        & 0.16794e+00_rb,0.19457e+00_rb,0.22757e+00_rb,0.26875e+00_rb,0.31708e+00_rb /)
      kbo(:,43,10) = (/ &
        & 0.14222e+00_rb,0.16606e+00_rb,0.19480e+00_rb,0.23049e+00_rb,0.27422e+00_rb /)
      kbo(:,44,10) = (/ &
        & 0.11907e+00_rb,0.14059e+00_rb,0.16567e+00_rb,0.19741e+00_rb,0.23557e+00_rb /)
      kbo(:,45,10) = (/ &
        & 0.99508e-01_rb,0.11792e+00_rb,0.14029e+00_rb,0.16912e+00_rb,0.20294e+00_rb /)
      kbo(:,46,10) = (/ &
        & 0.82787e-01_rb,0.98729e-01_rb,0.11783e+00_rb,0.14344e+00_rb,0.17433e+00_rb /)
      kbo(:,47,10) = (/ &
        & 0.68205e-01_rb,0.81739e-01_rb,0.98558e-01_rb,0.12017e+00_rb,0.14798e+00_rb /)
      kbo(:,48,10) = (/ &
        & 0.55907e-01_rb,0.67050e-01_rb,0.81465e-01_rb,0.10030e+00_rb,0.12506e+00_rb /)
      kbo(:,49,10) = (/ &
        & 0.45682e-01_rb,0.54871e-01_rb,0.66950e-01_rb,0.83270e-01_rb,0.10500e+00_rb /)
      kbo(:,50,10) = (/ &
        & 0.37515e-01_rb,0.45259e-01_rb,0.55584e-01_rb,0.69475e-01_rb,0.88432e-01_rb /)
      kbo(:,51,10) = (/ &
        & 0.31052e-01_rb,0.37347e-01_rb,0.46136e-01_rb,0.58029e-01_rb,0.74379e-01_rb /)
      kbo(:,52,10) = (/ &
        & 0.25697e-01_rb,0.30758e-01_rb,0.38027e-01_rb,0.48169e-01_rb,0.62340e-01_rb /)
      kbo(:,53,10) = (/ &
        & 0.21053e-01_rb,0.25282e-01_rb,0.31221e-01_rb,0.39765e-01_rb,0.52061e-01_rb /)
      kbo(:,54,10) = (/ &
        & 0.17506e-01_rb,0.21171e-01_rb,0.26181e-01_rb,0.33450e-01_rb,0.43837e-01_rb /)
      kbo(:,55,10) = (/ &
        & 0.14733e-01_rb,0.17912e-01_rb,0.22352e-01_rb,0.28517e-01_rb,0.37310e-01_rb /)
      kbo(:,56,10) = (/ &
        & 0.12382e-01_rb,0.15138e-01_rb,0.19031e-01_rb,0.24337e-01_rb,0.31780e-01_rb /)
      kbo(:,57,10) = (/ &
        & 0.10363e-01_rb,0.12802e-01_rb,0.16188e-01_rb,0.20731e-01_rb,0.27252e-01_rb /)
      kbo(:,58,10) = (/ &
        & 0.87632e-02_rb,0.10863e-01_rb,0.13823e-01_rb,0.17907e-01_rb,0.23570e-01_rb /)
      kbo(:,59,10) = (/ &
        & 0.79477e-02_rb,0.99708e-02_rb,0.12797e-01_rb,0.16832e-01_rb,0.22384e-01_rb /)
      kbo(:,13,11) = (/ &
        & 0.27028e+02_rb,0.27753e+02_rb,0.28483e+02_rb,0.29219e+02_rb,0.29909e+02_rb /)
      kbo(:,14,11) = (/ &
        & 0.23419e+02_rb,0.24025e+02_rb,0.24638e+02_rb,0.25228e+02_rb,0.25789e+02_rb /)
      kbo(:,15,11) = (/ &
        & 0.20150e+02_rb,0.20660e+02_rb,0.21151e+02_rb,0.21642e+02_rb,0.22118e+02_rb /)
      kbo(:,16,11) = (/ &
        & 0.17107e+02_rb,0.17510e+02_rb,0.17925e+02_rb,0.18355e+02_rb,0.18823e+02_rb /)
      kbo(:,17,11) = (/ &
        & 0.14361e+02_rb,0.14686e+02_rb,0.15035e+02_rb,0.15422e+02_rb,0.15903e+02_rb /)
      kbo(:,18,11) = (/ &
        & 0.12138e+02_rb,0.12397e+02_rb,0.12696e+02_rb,0.13058e+02_rb,0.13553e+02_rb /)
      kbo(:,19,11) = (/ &
        & 0.10135e+02_rb,0.10395e+02_rb,0.10708e+02_rb,0.11115e+02_rb,0.11638e+02_rb /)
      kbo(:,20,11) = (/ &
        & 0.81786e+01_rb,0.84190e+01_rb,0.87492e+01_rb,0.92221e+01_rb,0.97673e+01_rb /)
      kbo(:,21,11) = (/ &
        & 0.68203e+01_rb,0.70074e+01_rb,0.73249e+01_rb,0.77062e+01_rb,0.81227e+01_rb /)
      kbo(:,22,11) = (/ &
        & 0.58880e+01_rb,0.61142e+01_rb,0.64089e+01_rb,0.67273e+01_rb,0.70540e+01_rb /)
      kbo(:,23,11) = (/ &
        & 0.51482e+01_rb,0.53972e+01_rb,0.56613e+01_rb,0.59190e+01_rb,0.62039e+01_rb /)
      kbo(:,24,11) = (/ &
        & 0.45460e+01_rb,0.47798e+01_rb,0.50055e+01_rb,0.52340e+01_rb,0.54793e+01_rb /)
      kbo(:,25,11) = (/ &
        & 0.40038e+01_rb,0.42162e+01_rb,0.44356e+01_rb,0.46519e+01_rb,0.48861e+01_rb /)
      kbo(:,26,11) = (/ &
        & 0.35330e+01_rb,0.37282e+01_rb,0.39261e+01_rb,0.41342e+01_rb,0.43614e+01_rb /)
      kbo(:,27,11) = (/ &
        & 0.30918e+01_rb,0.32790e+01_rb,0.34783e+01_rb,0.36774e+01_rb,0.38933e+01_rb /)
      kbo(:,28,11) = (/ &
        & 0.27019e+01_rb,0.28880e+01_rb,0.30842e+01_rb,0.32755e+01_rb,0.34826e+01_rb /)
      kbo(:,29,11) = (/ &
        & 0.23567e+01_rb,0.25399e+01_rb,0.27237e+01_rb,0.29118e+01_rb,0.31184e+01_rb /)
      kbo(:,30,11) = (/ &
        & 0.20673e+01_rb,0.22421e+01_rb,0.24146e+01_rb,0.26001e+01_rb,0.28136e+01_rb /)
      kbo(:,31,11) = (/ &
        & 0.18233e+01_rb,0.19796e+01_rb,0.21491e+01_rb,0.23275e+01_rb,0.25391e+01_rb /)
      kbo(:,32,11) = (/ &
        & 0.16164e+01_rb,0.17629e+01_rb,0.19137e+01_rb,0.20921e+01_rb,0.23020e+01_rb /)
      kbo(:,33,11) = (/ &
        & 0.14325e+01_rb,0.15691e+01_rb,0.17171e+01_rb,0.18965e+01_rb,0.20991e+01_rb /)
      kbo(:,34,11) = (/ &
        & 0.12711e+01_rb,0.14025e+01_rb,0.15514e+01_rb,0.17231e+01_rb,0.19204e+01_rb /)
      kbo(:,35,11) = (/ &
        & 0.11251e+01_rb,0.12478e+01_rb,0.13948e+01_rb,0.15591e+01_rb,0.17428e+01_rb /)
      kbo(:,36,11) = (/ &
        & 0.98734e+00_rb,0.11026e+01_rb,0.12425e+01_rb,0.13991e+01_rb,0.15730e+01_rb /)
      kbo(:,37,11) = (/ &
        & 0.85152e+00_rb,0.96327e+00_rb,0.10911e+01_rb,0.12398e+01_rb,0.14045e+01_rb /)
      kbo(:,38,11) = (/ &
        & 0.73399e+00_rb,0.83942e+00_rb,0.95984e+00_rb,0.10967e+01_rb,0.12526e+01_rb /)
      kbo(:,39,11) = (/ &
        & 0.63468e+00_rb,0.73175e+00_rb,0.84488e+00_rb,0.97279e+00_rb,0.11163e+01_rb /)
      kbo(:,40,11) = (/ &
        & 0.54166e+00_rb,0.63099e+00_rb,0.73459e+00_rb,0.85525e+00_rb,0.98986e+00_rb /)
      kbo(:,41,11) = (/ &
        & 0.46341e+00_rb,0.54185e+00_rb,0.63786e+00_rb,0.75041e+00_rb,0.87557e+00_rb /)
      kbo(:,42,11) = (/ &
        & 0.39710e+00_rb,0.46604e+00_rb,0.55335e+00_rb,0.65788e+00_rb,0.77518e+00_rb /)
      kbo(:,43,11) = (/ &
        & 0.33809e+00_rb,0.39845e+00_rb,0.47656e+00_rb,0.56991e+00_rb,0.67763e+00_rb /)
      kbo(:,44,11) = (/ &
        & 0.28738e+00_rb,0.33811e+00_rb,0.40783e+00_rb,0.49238e+00_rb,0.58805e+00_rb /)
      kbo(:,45,11) = (/ &
        & 0.24372e+00_rb,0.28787e+00_rb,0.34858e+00_rb,0.42338e+00_rb,0.51131e+00_rb /)
      kbo(:,46,11) = (/ &
        & 0.20655e+00_rb,0.24407e+00_rb,0.29751e+00_rb,0.36239e+00_rb,0.44206e+00_rb /)
      kbo(:,47,11) = (/ &
        & 0.17359e+00_rb,0.20653e+00_rb,0.25158e+00_rb,0.30900e+00_rb,0.37888e+00_rb /)
      kbo(:,48,11) = (/ &
        & 0.14508e+00_rb,0.17395e+00_rb,0.21335e+00_rb,0.26232e+00_rb,0.32328e+00_rb /)
      kbo(:,49,11) = (/ &
        & 0.12016e+00_rb,0.14523e+00_rb,0.17928e+00_rb,0.22128e+00_rb,0.27412e+00_rb /)
      kbo(:,50,11) = (/ &
        & 0.99920e-01_rb,0.12218e+00_rb,0.15097e+00_rb,0.18907e+00_rb,0.23575e+00_rb /)
      kbo(:,51,11) = (/ &
        & 0.82909e-01_rb,0.10226e+00_rb,0.12737e+00_rb,0.16132e+00_rb,0.20378e+00_rb /)
      kbo(:,52,11) = (/ &
        & 0.68577e-01_rb,0.85131e-01_rb,0.10673e+00_rb,0.13644e+00_rb,0.17453e+00_rb /)
      kbo(:,53,11) = (/ &
        & 0.56302e-01_rb,0.70491e-01_rb,0.88684e-01_rb,0.11469e+00_rb,0.14786e+00_rb /)
      kbo(:,54,11) = (/ &
        & 0.47171e-01_rb,0.59185e-01_rb,0.74877e-01_rb,0.97387e-01_rb,0.12762e+00_rb /)
      kbo(:,55,11) = (/ &
        & 0.40028e-01_rb,0.50199e-01_rb,0.63960e-01_rb,0.83211e-01_rb,0.11104e+00_rb /)
      kbo(:,56,11) = (/ &
        & 0.33854e-01_rb,0.42508e-01_rb,0.54649e-01_rb,0.71262e-01_rb,0.95596e-01_rb /)
      kbo(:,57,11) = (/ &
        & 0.28590e-01_rb,0.36142e-01_rb,0.46409e-01_rb,0.61269e-01_rb,0.82012e-01_rb /)
      kbo(:,58,11) = (/ &
        & 0.24352e-01_rb,0.30934e-01_rb,0.39697e-01_rb,0.52483e-01_rb,0.71167e-01_rb /)
      kbo(:,59,11) = (/ &
        & 0.22598e-01_rb,0.29152e-01_rb,0.37468e-01_rb,0.48573e-01_rb,0.66434e-01_rb /)
      kbo(:,13,12) = (/ &
        & 0.57172e+02_rb,0.59297e+02_rb,0.61400e+02_rb,0.63512e+02_rb,0.65627e+02_rb /)
      kbo(:,14,12) = (/ &
        & 0.50361e+02_rb,0.52086e+02_rb,0.53828e+02_rb,0.55595e+02_rb,0.57346e+02_rb /)
      kbo(:,15,12) = (/ &
        & 0.44078e+02_rb,0.45453e+02_rb,0.46902e+02_rb,0.48347e+02_rb,0.49789e+02_rb /)
      kbo(:,16,12) = (/ &
        & 0.38395e+02_rb,0.39554e+02_rb,0.40754e+02_rb,0.41955e+02_rb,0.43150e+02_rb /)
      kbo(:,17,12) = (/ &
        & 0.33363e+02_rb,0.34355e+02_rb,0.35352e+02_rb,0.36337e+02_rb,0.37312e+02_rb /)
      kbo(:,18,12) = (/ &
        & 0.28599e+02_rb,0.29463e+02_rb,0.30307e+02_rb,0.31155e+02_rb,0.32043e+02_rb /)
      kbo(:,19,12) = (/ &
        & 0.24284e+02_rb,0.24982e+02_rb,0.25685e+02_rb,0.26424e+02_rb,0.27289e+02_rb /)
      kbo(:,20,12) = (/ &
        & 0.20576e+02_rb,0.21117e+02_rb,0.21693e+02_rb,0.22370e+02_rb,0.23304e+02_rb /)
      kbo(:,21,12) = (/ &
        & 0.17126e+02_rb,0.17629e+02_rb,0.18193e+02_rb,0.18968e+02_rb,0.20012e+02_rb /)
      kbo(:,22,12) = (/ &
        & 0.14084e+02_rb,0.14526e+02_rb,0.15112e+02_rb,0.15986e+02_rb,0.17086e+02_rb /)
      kbo(:,23,12) = (/ &
        & 0.11727e+02_rb,0.12073e+02_rb,0.12663e+02_rb,0.13546e+02_rb,0.14594e+02_rb /)
      kbo(:,24,12) = (/ &
        & 0.10116e+02_rb,0.10483e+02_rb,0.11087e+02_rb,0.11839e+02_rb,0.12630e+02_rb /)
      kbo(:,25,12) = (/ &
        & 0.89362e+01_rb,0.93748e+01_rb,0.99578e+01_rb,0.10576e+02_rb,0.11263e+02_rb /)
      kbo(:,26,12) = (/ &
        & 0.79574e+01_rb,0.84325e+01_rb,0.89960e+01_rb,0.95862e+01_rb,0.10135e+02_rb /)
      kbo(:,27,12) = (/ &
        & 0.71290e+01_rb,0.76343e+01_rb,0.81343e+01_rb,0.86609e+01_rb,0.91441e+01_rb /)
      kbo(:,28,12) = (/ &
        & 0.64441e+01_rb,0.69081e+01_rb,0.73340e+01_rb,0.77892e+01_rb,0.82579e+01_rb /)
      kbo(:,29,12) = (/ &
        & 0.57859e+01_rb,0.61904e+01_rb,0.65762e+01_rb,0.69937e+01_rb,0.74255e+01_rb /)
      kbo(:,30,12) = (/ &
        & 0.51602e+01_rb,0.55338e+01_rb,0.58953e+01_rb,0.62766e+01_rb,0.66823e+01_rb /)
      kbo(:,31,12) = (/ &
        & 0.45943e+01_rb,0.49270e+01_rb,0.52739e+01_rb,0.56298e+01_rb,0.60118e+01_rb /)
      kbo(:,32,12) = (/ &
        & 0.40881e+01_rb,0.44095e+01_rb,0.47450e+01_rb,0.50839e+01_rb,0.54488e+01_rb /)
      kbo(:,33,12) = (/ &
        & 0.36534e+01_rb,0.39549e+01_rb,0.42670e+01_rb,0.46043e+01_rb,0.49743e+01_rb /)
      kbo(:,34,12) = (/ &
        & 0.32874e+01_rb,0.35667e+01_rb,0.38571e+01_rb,0.41872e+01_rb,0.45862e+01_rb /)
      kbo(:,35,12) = (/ &
        & 0.29379e+01_rb,0.31963e+01_rb,0.34845e+01_rb,0.38151e+01_rb,0.42290e+01_rb /)
      kbo(:,36,12) = (/ &
        & 0.26101e+01_rb,0.28628e+01_rb,0.31405e+01_rb,0.34760e+01_rb,0.38927e+01_rb /)
      kbo(:,37,12) = (/ &
        & 0.23047e+01_rb,0.25368e+01_rb,0.28124e+01_rb,0.31292e+01_rb,0.35310e+01_rb /)
      kbo(:,38,12) = (/ &
        & 0.20413e+01_rb,0.22603e+01_rb,0.25129e+01_rb,0.28252e+01_rb,0.32149e+01_rb /)
      kbo(:,39,12) = (/ &
        & 0.18040e+01_rb,0.20133e+01_rb,0.22565e+01_rb,0.25571e+01_rb,0.29423e+01_rb /)
      kbo(:,40,12) = (/ &
        & 0.15898e+01_rb,0.17862e+01_rb,0.20189e+01_rb,0.22992e+01_rb,0.26576e+01_rb /)
      kbo(:,41,12) = (/ &
        & 0.14002e+01_rb,0.15861e+01_rb,0.18008e+01_rb,0.20620e+01_rb,0.24028e+01_rb /)
      kbo(:,42,12) = (/ &
        & 0.12302e+01_rb,0.14049e+01_rb,0.16015e+01_rb,0.18444e+01_rb,0.21658e+01_rb /)
      kbo(:,43,12) = (/ &
        & 0.10668e+01_rb,0.12258e+01_rb,0.14101e+01_rb,0.16347e+01_rb,0.19297e+01_rb /)
      kbo(:,44,12) = (/ &
        & 0.91800e+00_rb,0.10642e+01_rb,0.12324e+01_rb,0.14347e+01_rb,0.17074e+01_rb /)
      kbo(:,45,12) = (/ &
        & 0.78687e+00_rb,0.91967e+00_rb,0.10737e+01_rb,0.12598e+01_rb,0.15049e+01_rb /)
      kbo(:,46,12) = (/ &
        & 0.67148e+00_rb,0.79041e+00_rb,0.92761e+00_rb,0.10973e+01_rb,0.13177e+01_rb /)
      kbo(:,47,12) = (/ &
        & 0.57045e+00_rb,0.67640e+00_rb,0.80034e+00_rb,0.94779e+00_rb,0.11432e+01_rb /)
      kbo(:,48,12) = (/ &
        & 0.48142e+00_rb,0.57646e+00_rb,0.68676e+00_rb,0.82116e+00_rb,0.98898e+00_rb /)
      kbo(:,49,12) = (/ &
        & 0.40194e+00_rb,0.48757e+00_rb,0.58498e+00_rb,0.70838e+00_rb,0.85881e+00_rb /)
      kbo(:,50,12) = (/ &
        & 0.34095e+00_rb,0.41499e+00_rb,0.50425e+00_rb,0.61464e+00_rb,0.75451e+00_rb /)
      kbo(:,51,12) = (/ &
        & 0.29038e+00_rb,0.35608e+00_rb,0.43553e+00_rb,0.53422e+00_rb,0.66315e+00_rb /)
      kbo(:,52,12) = (/ &
        & 0.24615e+00_rb,0.30360e+00_rb,0.37359e+00_rb,0.46378e+00_rb,0.57808e+00_rb /)
      kbo(:,53,12) = (/ &
        & 0.20660e+00_rb,0.25764e+00_rb,0.31993e+00_rb,0.39869e+00_rb,0.50232e+00_rb /)
      kbo(:,54,12) = (/ &
        & 0.17518e+00_rb,0.22280e+00_rb,0.27852e+00_rb,0.34881e+00_rb,0.44243e+00_rb /)
      kbo(:,55,12) = (/ &
        & 0.14906e+00_rb,0.19392e+00_rb,0.24538e+00_rb,0.31045e+00_rb,0.39204e+00_rb /)
      kbo(:,56,12) = (/ &
        & 0.12564e+00_rb,0.16748e+00_rb,0.21535e+00_rb,0.27386e+00_rb,0.34816e+00_rb /)
      kbo(:,57,12) = (/ &
        & 0.10489e+00_rb,0.14334e+00_rb,0.18902e+00_rb,0.24147e+00_rb,0.30976e+00_rb /)
      kbo(:,58,12) = (/ &
        & 0.88415e-01_rb,0.12250e+00_rb,0.16566e+00_rb,0.21499e+00_rb,0.27515e+00_rb /)
      kbo(:,59,12) = (/ &
        & 0.80710e-01_rb,0.11375e+00_rb,0.15629e+00_rb,0.20737e+00_rb,0.26838e+00_rb /)
      kbo(:,13,13) = (/ &
        & 0.16465e+03_rb,0.16298e+03_rb,0.16413e+03_rb,0.16796e+03_rb,0.17425e+03_rb /)
      kbo(:,14,13) = (/ &
        & 0.14134e+03_rb,0.14223e+03_rb,0.14584e+03_rb,0.15183e+03_rb,0.15823e+03_rb /)
      kbo(:,15,13) = (/ &
        & 0.12334e+03_rb,0.12642e+03_rb,0.13170e+03_rb,0.13719e+03_rb,0.14248e+03_rb /)
      kbo(:,16,13) = (/ &
        & 0.10903e+03_rb,0.11350e+03_rb,0.11825e+03_rb,0.12284e+03_rb,0.12720e+03_rb /)
      kbo(:,17,13) = (/ &
        & 0.96744e+02_rb,0.10077e+03_rb,0.10473e+03_rb,0.10856e+03_rb,0.11225e+03_rb /)
      kbo(:,18,13) = (/ &
        & 0.85368e+02_rb,0.88669e+02_rb,0.91934e+02_rb,0.95127e+02_rb,0.98216e+02_rb /)
      kbo(:,19,13) = (/ &
        & 0.74865e+02_rb,0.77560e+02_rb,0.80263e+02_rb,0.82912e+02_rb,0.85499e+02_rb /)
      kbo(:,20,13) = (/ &
        & 0.65304e+02_rb,0.67588e+02_rb,0.69849e+02_rb,0.72099e+02_rb,0.74316e+02_rb /)
      kbo(:,21,13) = (/ &
        & 0.56704e+02_rb,0.58616e+02_rb,0.60541e+02_rb,0.62457e+02_rb,0.64444e+02_rb /)
      kbo(:,22,13) = (/ &
        & 0.49099e+02_rb,0.50788e+02_rb,0.52451e+02_rb,0.54119e+02_rb,0.55970e+02_rb /)
      kbo(:,23,13) = (/ &
        & 0.42076e+02_rb,0.43646e+02_rb,0.45230e+02_rb,0.46888e+02_rb,0.48798e+02_rb /)
      kbo(:,24,13) = (/ &
        & 0.35457e+02_rb,0.36904e+02_rb,0.38447e+02_rb,0.40262e+02_rb,0.42565e+02_rb /)
      kbo(:,25,13) = (/ &
        & 0.29832e+02_rb,0.31036e+02_rb,0.32508e+02_rb,0.34502e+02_rb,0.36979e+02_rb /)
      kbo(:,26,13) = (/ &
        & 0.25205e+02_rb,0.26348e+02_rb,0.27792e+02_rb,0.29788e+02_rb,0.32459e+02_rb /)
      kbo(:,27,13) = (/ &
        & 0.21486e+02_rb,0.22586e+02_rb,0.24162e+02_rb,0.26260e+02_rb,0.28944e+02_rb /)
      kbo(:,28,13) = (/ &
        & 0.18712e+02_rb,0.19693e+02_rb,0.21370e+02_rb,0.23591e+02_rb,0.26158e+02_rb /)
      kbo(:,29,13) = (/ &
        & 0.16630e+02_rb,0.17716e+02_rb,0.19369e+02_rb,0.21565e+02_rb,0.24102e+02_rb /)
      kbo(:,30,13) = (/ &
        & 0.14923e+02_rb,0.16119e+02_rb,0.17783e+02_rb,0.19989e+02_rb,0.22379e+02_rb /)
      kbo(:,31,13) = (/ &
        & 0.13532e+02_rb,0.14853e+02_rb,0.16576e+02_rb,0.18745e+02_rb,0.21084e+02_rb /)
      kbo(:,32,13) = (/ &
        & 0.12453e+02_rb,0.13784e+02_rb,0.15587e+02_rb,0.17718e+02_rb,0.20022e+02_rb /)
      kbo(:,33,13) = (/ &
        & 0.11580e+02_rb,0.13038e+02_rb,0.14851e+02_rb,0.16892e+02_rb,0.19192e+02_rb /)
      kbo(:,34,13) = (/ &
        & 0.10820e+02_rb,0.12376e+02_rb,0.14134e+02_rb,0.16152e+02_rb,0.18402e+02_rb /)
      kbo(:,35,13) = (/ &
        & 0.10116e+02_rb,0.11686e+02_rb,0.13402e+02_rb,0.15346e+02_rb,0.17556e+02_rb /)
      kbo(:,36,13) = (/ &
        & 0.93933e+01_rb,0.10910e+02_rb,0.12572e+02_rb,0.14432e+02_rb,0.16536e+02_rb /)
      kbo(:,37,13) = (/ &
        & 0.85160e+01_rb,0.99755e+01_rb,0.11554e+02_rb,0.13358e+02_rb,0.15350e+02_rb /)
      kbo(:,38,13) = (/ &
        & 0.77505e+01_rb,0.91225e+01_rb,0.10670e+02_rb,0.12380e+02_rb,0.14273e+02_rb /)
      kbo(:,39,13) = (/ &
        & 0.70897e+01_rb,0.83885e+01_rb,0.98694e+01_rb,0.11502e+02_rb,0.13343e+02_rb /)
      kbo(:,40,13) = (/ &
        & 0.63295e+01_rb,0.75506e+01_rb,0.89466e+01_rb,0.10495e+02_rb,0.12287e+02_rb /)
      kbo(:,41,13) = (/ &
        & 0.56302e+01_rb,0.67988e+01_rb,0.80957e+01_rb,0.96013e+01_rb,0.11280e+02_rb /)
      kbo(:,42,13) = (/ &
        & 0.50234e+01_rb,0.61071e+01_rb,0.73389e+01_rb,0.87816e+01_rb,0.10380e+02_rb /)
      kbo(:,43,13) = (/ &
        & 0.44225e+01_rb,0.54282e+01_rb,0.65963e+01_rb,0.79433e+01_rb,0.94666e+01_rb /)
      kbo(:,44,13) = (/ &
        & 0.38558e+01_rb,0.47912e+01_rb,0.58905e+01_rb,0.71478e+01_rb,0.86145e+01_rb /)
      kbo(:,45,13) = (/ &
        & 0.33575e+01_rb,0.42351e+01_rb,0.52475e+01_rb,0.64292e+01_rb,0.78296e+01_rb /)
      kbo(:,46,13) = (/ &
        & 0.29053e+01_rb,0.37074e+01_rb,0.46432e+01_rb,0.57484e+01_rb,0.70674e+01_rb /)
      kbo(:,47,13) = (/ &
        & 0.24706e+01_rb,0.31851e+01_rb,0.40524e+01_rb,0.50837e+01_rb,0.62990e+01_rb /)
      kbo(:,48,13) = (/ &
        & 0.20907e+01_rb,0.27320e+01_rb,0.35232e+01_rb,0.44689e+01_rb,0.55998e+01_rb /)
      kbo(:,49,13) = (/ &
        & 0.17652e+01_rb,0.23397e+01_rb,0.30525e+01_rb,0.39130e+01_rb,0.49641e+01_rb /)
      kbo(:,50,13) = (/ &
        & 0.14980e+01_rb,0.19995e+01_rb,0.26386e+01_rb,0.34309e+01_rb,0.43930e+01_rb /)
      kbo(:,51,13) = (/ &
        & 0.12883e+01_rb,0.17229e+01_rb,0.22808e+01_rb,0.30058e+01_rb,0.38921e+01_rb /)
      kbo(:,52,13) = (/ &
        & 0.11032e+01_rb,0.14878e+01_rb,0.19945e+01_rb,0.26225e+01_rb,0.34327e+01_rb /)
      kbo(:,53,13) = (/ &
        & 0.94256e+00_rb,0.12803e+01_rb,0.17436e+01_rb,0.23095e+01_rb,0.30258e+01_rb /)
      kbo(:,54,13) = (/ &
        & 0.80847e+00_rb,0.11045e+01_rb,0.15250e+01_rb,0.20573e+01_rb,0.27048e+01_rb /)
      kbo(:,55,13) = (/ &
        & 0.69972e+00_rb,0.95008e+00_rb,0.13215e+01_rb,0.18178e+01_rb,0.24326e+01_rb /)
      kbo(:,56,13) = (/ &
        & 0.60659e+00_rb,0.81298e+00_rb,0.11430e+01_rb,0.15925e+01_rb,0.21783e+01_rb /)
      kbo(:,57,13) = (/ &
        & 0.52664e+00_rb,0.69670e+00_rb,0.97998e+00_rb,0.13891e+01_rb,0.19333e+01_rb /)
      kbo(:,58,13) = (/ &
        & 0.45985e+00_rb,0.60682e+00_rb,0.84337e+00_rb,0.12113e+01_rb,0.17188e+01_rb /)
      kbo(:,59,13) = (/ &
        & 0.43853e+00_rb,0.57879e+00_rb,0.79018e+00_rb,0.11130e+01_rb,0.16039e+01_rb /)
      kbo(:,13,14) = (/ &
        & 0.86016e+03_rb,0.84626e+03_rb,0.83228e+03_rb,0.81817e+03_rb,0.80450e+03_rb /)
      kbo(:,14,14) = (/ &
        & 0.75981e+03_rb,0.74663e+03_rb,0.73345e+03_rb,0.72104e+03_rb,0.71224e+03_rb /)
      kbo(:,15,14) = (/ &
        & 0.66201e+03_rb,0.64993e+03_rb,0.63891e+03_rb,0.63210e+03_rb,0.62978e+03_rb /)
      kbo(:,16,14) = (/ &
        & 0.57014e+03_rb,0.56027e+03_rb,0.55466e+03_rb,0.55378e+03_rb,0.55721e+03_rb /)
      kbo(:,17,14) = (/ &
        & 0.48861e+03_rb,0.48354e+03_rb,0.48324e+03_rb,0.48760e+03_rb,0.49602e+03_rb /)
      kbo(:,18,14) = (/ &
        & 0.41987e+03_rb,0.41964e+03_rb,0.42411e+03_rb,0.43291e+03_rb,0.44575e+03_rb /)
      kbo(:,19,14) = (/ &
        & 0.36303e+03_rb,0.36702e+03_rb,0.37556e+03_rb,0.38834e+03_rb,0.40471e+03_rb /)
      kbo(:,20,14) = (/ &
        & 0.31651e+03_rb,0.32421e+03_rb,0.33627e+03_rb,0.35219e+03_rb,0.37072e+03_rb /)
      kbo(:,21,14) = (/ &
        & 0.27867e+03_rb,0.28947e+03_rb,0.30434e+03_rb,0.32131e+03_rb,0.33823e+03_rb /)
      kbo(:,22,14) = (/ &
        & 0.24916e+03_rb,0.26251e+03_rb,0.27768e+03_rb,0.29296e+03_rb,0.30844e+03_rb /)
      kbo(:,23,14) = (/ &
        & 0.22571e+03_rb,0.23932e+03_rb,0.25291e+03_rb,0.26670e+03_rb,0.28069e+03_rb /)
      kbo(:,24,14) = (/ &
        & 0.20503e+03_rb,0.21765e+03_rb,0.23029e+03_rb,0.24291e+03_rb,0.25575e+03_rb /)
      kbo(:,25,14) = (/ &
        & 0.18574e+03_rb,0.19763e+03_rb,0.20960e+03_rb,0.22151e+03_rb,0.23356e+03_rb /)
      kbo(:,26,14) = (/ &
        & 0.16825e+03_rb,0.17945e+03_rb,0.19085e+03_rb,0.20242e+03_rb,0.21406e+03_rb /)
      kbo(:,27,14) = (/ &
        & 0.15254e+03_rb,0.16311e+03_rb,0.17402e+03_rb,0.18521e+03_rb,0.19696e+03_rb /)
      kbo(:,28,14) = (/ &
        & 0.13815e+03_rb,0.14857e+03_rb,0.15913e+03_rb,0.17022e+03_rb,0.18238e+03_rb /)
      kbo(:,29,14) = (/ &
        & 0.12560e+03_rb,0.13571e+03_rb,0.14630e+03_rb,0.15763e+03_rb,0.17019e+03_rb /)
      kbo(:,30,14) = (/ &
        & 0.11497e+03_rb,0.12476e+03_rb,0.13550e+03_rb,0.14719e+03_rb,0.16044e+03_rb /)
      kbo(:,31,14) = (/ &
        & 0.10602e+03_rb,0.11577e+03_rb,0.12671e+03_rb,0.13889e+03_rb,0.15289e+03_rb /)
      kbo(:,32,14) = (/ &
        & 0.98519e+02_rb,0.10855e+03_rb,0.11978e+03_rb,0.13265e+03_rb,0.14744e+03_rb /)
      kbo(:,33,14) = (/ &
        & 0.92517e+02_rb,0.10274e+03_rb,0.11452e+03_rb,0.12822e+03_rb,0.14389e+03_rb /)
      kbo(:,34,14) = (/ &
        & 0.87441e+02_rb,0.97936e+02_rb,0.11044e+03_rb,0.12495e+03_rb,0.14148e+03_rb /)
      kbo(:,35,14) = (/ &
        & 0.82499e+02_rb,0.93333e+02_rb,0.10635e+03_rb,0.12151e+03_rb,0.13874e+03_rb /)
      kbo(:,36,14) = (/ &
        & 0.77442e+02_rb,0.88493e+02_rb,0.10184e+03_rb,0.11740e+03_rb,0.13513e+03_rb /)
      kbo(:,37,14) = (/ &
        & 0.71771e+02_rb,0.82731e+02_rb,0.96032e+02_rb,0.11158e+03_rb,0.12941e+03_rb /)
      kbo(:,38,14) = (/ &
        & 0.66727e+02_rb,0.77610e+02_rb,0.90795e+02_rb,0.10632e+03_rb,0.12415e+03_rb /)
      kbo(:,39,14) = (/ &
        & 0.62339e+02_rb,0.73113e+02_rb,0.86200e+02_rb,0.10167e+03_rb,0.11942e+03_rb /)
      kbo(:,40,14) = (/ &
        & 0.57601e+02_rb,0.67999e+02_rb,0.80724e+02_rb,0.95858e+02_rb,0.11323e+03_rb /)
      kbo(:,41,14) = (/ &
        & 0.53285e+02_rb,0.63203e+02_rb,0.75577e+02_rb,0.90242e+02_rb,0.10726e+03_rb /)
      kbo(:,42,14) = (/ &
        & 0.49392e+02_rb,0.58908e+02_rb,0.70834e+02_rb,0.85045e+02_rb,0.10163e+03_rb /)
      kbo(:,43,14) = (/ &
        & 0.45446e+02_rb,0.54452e+02_rb,0.65751e+02_rb,0.79402e+02_rb,0.95397e+02_rb /)
      kbo(:,44,14) = (/ &
        & 0.41620e+02_rb,0.50184e+02_rb,0.60777e+02_rb,0.73786e+02_rb,0.89076e+02_rb /)
      kbo(:,45,14) = (/ &
        & 0.38059e+02_rb,0.46282e+02_rb,0.56226e+02_rb,0.68539e+02_rb,0.83136e+02_rb /)
      kbo(:,46,14) = (/ &
        & 0.34554e+02_rb,0.42501e+02_rb,0.51829e+02_rb,0.63383e+02_rb,0.77220e+02_rb /)
      kbo(:,47,14) = (/ &
        & 0.31009e+02_rb,0.38610e+02_rb,0.47385e+02_rb,0.58058e+02_rb,0.71080e+02_rb /)
      kbo(:,48,14) = (/ &
        & 0.27715e+02_rb,0.34934e+02_rb,0.43319e+02_rb,0.53190e+02_rb,0.65347e+02_rb /)
      kbo(:,49,14) = (/ &
        & 0.24658e+02_rb,0.31478e+02_rb,0.39500e+02_rb,0.48743e+02_rb,0.59997e+02_rb /)
      kbo(:,50,14) = (/ &
        & 0.22034e+02_rb,0.28437e+02_rb,0.36093e+02_rb,0.44894e+02_rb,0.55346e+02_rb /)
      kbo(:,51,14) = (/ &
        & 0.19684e+02_rb,0.25626e+02_rb,0.32937e+02_rb,0.41385e+02_rb,0.51154e+02_rb /)
      kbo(:,52,14) = (/ &
        & 0.17504e+02_rb,0.22995e+02_rb,0.29925e+02_rb,0.38057e+02_rb,0.47307e+02_rb /)
      kbo(:,53,14) = (/ &
        & 0.15552e+02_rb,0.20544e+02_rb,0.27079e+02_rb,0.34832e+02_rb,0.43722e+02_rb /)
      kbo(:,54,14) = (/ &
        & 0.13886e+02_rb,0.18476e+02_rb,0.24607e+02_rb,0.31966e+02_rb,0.40540e+02_rb /)
      kbo(:,55,14) = (/ &
        & 0.12396e+02_rb,0.16681e+02_rb,0.22385e+02_rb,0.29384e+02_rb,0.37592e+02_rb /)
      kbo(:,56,14) = (/ &
        & 0.11028e+02_rb,0.15006e+02_rb,0.20297e+02_rb,0.26955e+02_rb,0.34780e+02_rb /)
      kbo(:,57,14) = (/ &
        & 0.97493e+01_rb,0.13452e+02_rb,0.18354e+02_rb,0.24644e+02_rb,0.32099e+02_rb /)
      kbo(:,58,14) = (/ &
        & 0.86411e+01_rb,0.12068e+02_rb,0.16627e+02_rb,0.22569e+02_rb,0.29684e+02_rb /)
      kbo(:,59,14) = (/ &
        & 0.80894e+01_rb,0.11449e+02_rb,0.15905e+02_rb,0.21753e+02_rb,0.28728e+02_rb /)
      kbo(:,13,15) = (/ &
        & 0.50046e+04_rb,0.50034e+04_rb,0.49950e+04_rb,0.49781e+04_rb,0.49542e+04_rb /)
      kbo(:,14,15) = (/ &
        & 0.51812e+04_rb,0.51738e+04_rb,0.51582e+04_rb,0.51357e+04_rb,0.51054e+04_rb /)
      kbo(:,15,15) = (/ &
        & 0.52793e+04_rb,0.52645e+04_rb,0.52450e+04_rb,0.52172e+04_rb,0.51836e+04_rb /)
      kbo(:,16,15) = (/ &
        & 0.53001e+04_rb,0.52818e+04_rb,0.52568e+04_rb,0.52274e+04_rb,0.51924e+04_rb /)
      kbo(:,17,15) = (/ &
        & 0.52448e+04_rb,0.52242e+04_rb,0.52004e+04_rb,0.51715e+04_rb,0.51377e+04_rb /)
      kbo(:,18,15) = (/ &
        & 0.51200e+04_rb,0.51031e+04_rb,0.50817e+04_rb,0.50571e+04_rb,0.50308e+04_rb /)
      kbo(:,19,15) = (/ &
        & 0.49389e+04_rb,0.49284e+04_rb,0.49153e+04_rb,0.49007e+04_rb,0.48819e+04_rb /)
      kbo(:,20,15) = (/ &
        & 0.47181e+04_rb,0.47168e+04_rb,0.47163e+04_rb,0.47129e+04_rb,0.47111e+04_rb /)
      kbo(:,21,15) = (/ &
        & 0.44718e+04_rb,0.44865e+04_rb,0.44990e+04_rb,0.45153e+04_rb,0.45403e+04_rb /)
      kbo(:,22,15) = (/ &
        & 0.42192e+04_rb,0.42497e+04_rb,0.42861e+04_rb,0.43316e+04_rb,0.43809e+04_rb /)
      kbo(:,23,15) = (/ &
        & 0.39727e+04_rb,0.40275e+04_rb,0.40936e+04_rb,0.41646e+04_rb,0.42404e+04_rb /)
      kbo(:,24,15) = (/ &
        & 0.37507e+04_rb,0.38336e+04_rb,0.39239e+04_rb,0.40206e+04_rb,0.41209e+04_rb /)
      kbo(:,25,15) = (/ &
        & 0.35606e+04_rb,0.36694e+04_rb,0.37826e+04_rb,0.39026e+04_rb,0.40244e+04_rb /)
      kbo(:,26,15) = (/ &
        & 0.34043e+04_rb,0.35364e+04_rb,0.36731e+04_rb,0.38130e+04_rb,0.39519e+04_rb /)
      kbo(:,27,15) = (/ &
        & 0.32803e+04_rb,0.34346e+04_rb,0.35912e+04_rb,0.37476e+04_rb,0.39014e+04_rb /)
      kbo(:,28,15) = (/ &
        & 0.31876e+04_rb,0.33610e+04_rb,0.35344e+04_rb,0.37042e+04_rb,0.38699e+04_rb /)
      kbo(:,29,15) = (/ &
        & 0.31239e+04_rb,0.33137e+04_rb,0.34995e+04_rb,0.36813e+04_rb,0.38573e+04_rb /)
      kbo(:,30,15) = (/ &
        & 0.30850e+04_rb,0.32872e+04_rb,0.34837e+04_rb,0.36745e+04_rb,0.38591e+04_rb /)
      kbo(:,31,15) = (/ &
        & 0.30674e+04_rb,0.32795e+04_rb,0.34848e+04_rb,0.36826e+04_rb,0.38724e+04_rb /)
      kbo(:,32,15) = (/ &
        & 0.30683e+04_rb,0.32879e+04_rb,0.34990e+04_rb,0.37022e+04_rb,0.38953e+04_rb /)
      kbo(:,33,15) = (/ &
        & 0.30831e+04_rb,0.33087e+04_rb,0.35243e+04_rb,0.37303e+04_rb,0.39250e+04_rb /)
      kbo(:,34,15) = (/ &
        & 0.31017e+04_rb,0.33308e+04_rb,0.35496e+04_rb,0.37580e+04_rb,0.39527e+04_rb /)
      kbo(:,35,15) = (/ &
        & 0.31057e+04_rb,0.33387e+04_rb,0.35601e+04_rb,0.37700e+04_rb,0.39654e+04_rb /)
      kbo(:,36,15) = (/ &
        & 0.30895e+04_rb,0.33260e+04_rb,0.35504e+04_rb,0.37630e+04_rb,0.39606e+04_rb /)
      kbo(:,37,15) = (/ &
        & 0.30400e+04_rb,0.32811e+04_rb,0.35096e+04_rb,0.37263e+04_rb,0.39277e+04_rb /)
      kbo(:,38,15) = (/ &
        & 0.29913e+04_rb,0.32361e+04_rb,0.34690e+04_rb,0.36890e+04_rb,0.38942e+04_rb /)
      kbo(:,39,15) = (/ &
        & 0.29441e+04_rb,0.31926e+04_rb,0.34294e+04_rb,0.36526e+04_rb,0.38612e+04_rb /)
      kbo(:,40,15) = (/ &
        & 0.28722e+04_rb,0.31254e+04_rb,0.33667e+04_rb,0.35943e+04_rb,0.38073e+04_rb /)
      kbo(:,41,15) = (/ &
        & 0.27971e+04_rb,0.30548e+04_rb,0.33004e+04_rb,0.35326e+04_rb,0.37505e+04_rb /)
      kbo(:,42,15) = (/ &
        & 0.27226e+04_rb,0.29837e+04_rb,0.32336e+04_rb,0.34701e+04_rb,0.36925e+04_rb /)
      kbo(:,43,15) = (/ &
        & 0.26332e+04_rb,0.28981e+04_rb,0.31525e+04_rb,0.33944e+04_rb,0.36223e+04_rb /)
      kbo(:,44,15) = (/ &
        & 0.25372e+04_rb,0.28050e+04_rb,0.30643e+04_rb,0.33114e+04_rb,0.35441e+04_rb /)
      kbo(:,45,15) = (/ &
        & 0.24417e+04_rb,0.27113e+04_rb,0.29748e+04_rb,0.32268e+04_rb,0.34651e+04_rb /)
      kbo(:,46,15) = (/ &
        & 0.23395e+04_rb,0.26115e+04_rb,0.28783e+04_rb,0.31351e+04_rb,0.33792e+04_rb /)
      kbo(:,47,15) = (/ &
        & 0.22243e+04_rb,0.24998e+04_rb,0.27694e+04_rb,0.30313e+04_rb,0.32810e+04_rb /)
      kbo(:,48,15) = (/ &
        & 0.21083e+04_rb,0.23877e+04_rb,0.26592e+04_rb,0.29257e+04_rb,0.31803e+04_rb /)
      kbo(:,49,15) = (/ &
        & 0.19925e+04_rb,0.22741e+04_rb,0.25488e+04_rb,0.28180e+04_rb,0.30783e+04_rb /)
      kbo(:,50,15) = (/ &
        & 0.18841e+04_rb,0.21666e+04_rb,0.24447e+04_rb,0.27159e+04_rb,0.29806e+04_rb /)
      kbo(:,51,15) = (/ &
        & 0.17798e+04_rb,0.20618e+04_rb,0.23428e+04_rb,0.26160e+04_rb,0.28839e+04_rb /)
      kbo(:,52,15) = (/ &
        & 0.16762e+04_rb,0.19574e+04_rb,0.22398e+04_rb,0.25156e+04_rb,0.27862e+04_rb /)
      kbo(:,53,15) = (/ &
        & 0.15731e+04_rb,0.18530e+04_rb,0.21358e+04_rb,0.24152e+04_rb,0.26872e+04_rb /)
      kbo(:,54,15) = (/ &
        & 0.14802e+04_rb,0.17581e+04_rb,0.20401e+04_rb,0.23219e+04_rb,0.25956e+04_rb /)
      kbo(:,55,15) = (/ &
        & 0.13915e+04_rb,0.16675e+04_rb,0.19488e+04_rb,0.22318e+04_rb,0.25080e+04_rb /)
      kbo(:,56,15) = (/ &
        & 0.13039e+04_rb,0.15778e+04_rb,0.18577e+04_rb,0.21406e+04_rb,0.24200e+04_rb /)
      kbo(:,57,15) = (/ &
        & 0.12167e+04_rb,0.14890e+04_rb,0.17672e+04_rb,0.20493e+04_rb,0.23313e+04_rb /)
      kbo(:,58,15) = (/ &
        & 0.11349e+04_rb,0.14057e+04_rb,0.16818e+04_rb,0.19631e+04_rb,0.22464e+04_rb /)
      kbo(:,59,15) = (/ &
        & 0.11025e+04_rb,0.13721e+04_rb,0.16476e+04_rb,0.19283e+04_rb,0.22117e+04_rb /)
      kbo(:,13,16) = (/ &
        & 0.12492e+05_rb,0.12602e+05_rb,0.12678e+05_rb,0.12730e+05_rb,0.12749e+05_rb /)
      kbo(:,14,16) = (/ &
        & 0.14696e+05_rb,0.14792e+05_rb,0.14862e+05_rb,0.14890e+05_rb,0.14889e+05_rb /)
      kbo(:,15,16) = (/ &
        & 0.17153e+05_rb,0.17242e+05_rb,0.17275e+05_rb,0.17274e+05_rb,0.17231e+05_rb /)
      kbo(:,16,16) = (/ &
        & 0.19843e+05_rb,0.19894e+05_rb,0.19898e+05_rb,0.19845e+05_rb,0.19747e+05_rb /)
      kbo(:,17,16) = (/ &
        & 0.22727e+05_rb,0.22731e+05_rb,0.22667e+05_rb,0.22551e+05_rb,0.22386e+05_rb /)
      kbo(:,18,16) = (/ &
        & 0.25761e+05_rb,0.25684e+05_rb,0.25544e+05_rb,0.25342e+05_rb,0.25070e+05_rb /)
      kbo(:,19,16) = (/ &
        & 0.28878e+05_rb,0.28701e+05_rb,0.28454e+05_rb,0.28131e+05_rb,0.27759e+05_rb /)
      kbo(:,20,16) = (/ &
        & 0.31981e+05_rb,0.31695e+05_rb,0.31314e+05_rb,0.30871e+05_rb,0.30360e+05_rb /)
      kbo(:,21,16) = (/ &
        & 0.35017e+05_rb,0.34575e+05_rb,0.34064e+05_rb,0.33480e+05_rb,0.32821e+05_rb /)
      kbo(:,22,16) = (/ &
        & 0.37854e+05_rb,0.37264e+05_rb,0.36589e+05_rb,0.35840e+05_rb,0.35046e+05_rb /)
      kbo(:,23,16) = (/ &
        & 0.40473e+05_rb,0.39710e+05_rb,0.38862e+05_rb,0.37967e+05_rb,0.37023e+05_rb /)
      kbo(:,24,16) = (/ &
        & 0.42796e+05_rb,0.41864e+05_rb,0.40869e+05_rb,0.39817e+05_rb,0.38718e+05_rb /)
      kbo(:,25,16) = (/ &
        & 0.44813e+05_rb,0.43719e+05_rb,0.42578e+05_rb,0.41380e+05_rb,0.40143e+05_rb /)
      kbo(:,26,16) = (/ &
        & 0.46504e+05_rb,0.45263e+05_rb,0.43973e+05_rb,0.42641e+05_rb,0.41294e+05_rb /)
      kbo(:,27,16) = (/ &
        & 0.47893e+05_rb,0.46507e+05_rb,0.45087e+05_rb,0.43647e+05_rb,0.42197e+05_rb /)
      kbo(:,28,16) = (/ &
        & 0.48993e+05_rb,0.47480e+05_rb,0.45952e+05_rb,0.44417e+05_rb,0.42878e+05_rb /)
      kbo(:,29,16) = (/ &
        & 0.49823e+05_rb,0.48206e+05_rb,0.46588e+05_rb,0.44969e+05_rb,0.43353e+05_rb /)
      kbo(:,30,16) = (/ &
        & 0.50415e+05_rb,0.48716e+05_rb,0.47025e+05_rb,0.45339e+05_rb,0.43659e+05_rb /)
      kbo(:,31,16) = (/ &
        & 0.50811e+05_rb,0.49042e+05_rb,0.47288e+05_rb,0.45543e+05_rb,0.43813e+05_rb /)
      kbo(:,32,16) = (/ &
        & 0.51032e+05_rb,0.49203e+05_rb,0.47400e+05_rb,0.45610e+05_rb,0.43841e+05_rb /)
      kbo(:,33,16) = (/ &
        & 0.51107e+05_rb,0.49234e+05_rb,0.47393e+05_rb,0.45565e+05_rb,0.43775e+05_rb /)
      kbo(:,34,16) = (/ &
        & 0.51130e+05_rb,0.49228e+05_rb,0.47350e+05_rb,0.45497e+05_rb,0.43688e+05_rb /)
      kbo(:,35,16) = (/ &
        & 0.51256e+05_rb,0.49316e+05_rb,0.47413e+05_rb,0.45540e+05_rb,0.43711e+05_rb /)
      kbo(:,36,16) = (/ &
        & 0.51519e+05_rb,0.49555e+05_rb,0.47626e+05_rb,0.45731e+05_rb,0.43885e+05_rb /)
      kbo(:,37,16) = (/ &
        & 0.52034e+05_rb,0.50041e+05_rb,0.48090e+05_rb,0.46168e+05_rb,0.44302e+05_rb /)
      kbo(:,38,16) = (/ &
        & 0.52529e+05_rb,0.50511e+05_rb,0.48535e+05_rb,0.46595e+05_rb,0.44706e+05_rb /)
      kbo(:,39,16) = (/ &
        & 0.52985e+05_rb,0.50952e+05_rb,0.48954e+05_rb,0.46996e+05_rb,0.45085e+05_rb /)
      kbo(:,40,16) = (/ &
        & 0.53633e+05_rb,0.51575e+05_rb,0.49556e+05_rb,0.47578e+05_rb,0.45648e+05_rb /)
      kbo(:,41,16) = (/ &
        & 0.54289e+05_rb,0.52211e+05_rb,0.50171e+05_rb,0.48171e+05_rb,0.46220e+05_rb /)
      kbo(:,42,16) = (/ &
        & 0.54932e+05_rb,0.52836e+05_rb,0.50778e+05_rb,0.48759e+05_rb,0.46785e+05_rb /)
      kbo(:,43,16) = (/ &
        & 0.55673e+05_rb,0.53569e+05_rb,0.51488e+05_rb,0.49449e+05_rb,0.47456e+05_rb /)
      kbo(:,44,16) = (/ &
        & 0.56461e+05_rb,0.54346e+05_rb,0.52247e+05_rb,0.50190e+05_rb,0.48178e+05_rb /)
      kbo(:,45,16) = (/ &
        & 0.57241e+05_rb,0.55121e+05_rb,0.53007e+05_rb,0.50929e+05_rb,0.48897e+05_rb /)
      kbo(:,46,16) = (/ &
        & 0.58063e+05_rb,0.55933e+05_rb,0.53806e+05_rb,0.51710e+05_rb,0.49656e+05_rb /)
      kbo(:,47,16) = (/ &
        & 0.58978e+05_rb,0.56828e+05_rb,0.54697e+05_rb,0.52582e+05_rb,0.50505e+05_rb /)
      kbo(:,48,16) = (/ &
        & 0.59891e+05_rb,0.57720e+05_rb,0.55586e+05_rb,0.53459e+05_rb,0.51367e+05_rb /)
      kbo(:,49,16) = (/ &
        & 0.60795e+05_rb,0.58621e+05_rb,0.56472e+05_rb,0.54336e+05_rb,0.52224e+05_rb /)
      kbo(:,50,16) = (/ &
        & 0.61639e+05_rb,0.59466e+05_rb,0.57298e+05_rb,0.55162e+05_rb,0.53034e+05_rb /)
      kbo(:,51,16) = (/ &
        & 0.62447e+05_rb,0.60284e+05_rb,0.58110e+05_rb,0.55961e+05_rb,0.53824e+05_rb /)
      kbo(:,52,16) = (/ &
        & 0.63240e+05_rb,0.61098e+05_rb,0.58916e+05_rb,0.56760e+05_rb,0.54612e+05_rb /)
      kbo(:,53,16) = (/ &
        & 0.64030e+05_rb,0.61903e+05_rb,0.59727e+05_rb,0.57558e+05_rb,0.55411e+05_rb /)
      kbo(:,54,16) = (/ &
        & 0.64735e+05_rb,0.62637e+05_rb,0.60471e+05_rb,0.58287e+05_rb,0.56138e+05_rb /)
      kbo(:,55,16) = (/ &
        & 0.65407e+05_rb,0.63332e+05_rb,0.61181e+05_rb,0.59003e+05_rb,0.56834e+05_rb /)
      kbo(:,56,16) = (/ &
        & 0.66069e+05_rb,0.64015e+05_rb,0.61881e+05_rb,0.59702e+05_rb,0.57529e+05_rb /)
      kbo(:,57,16) = (/ &
        & 0.66725e+05_rb,0.64683e+05_rb,0.62576e+05_rb,0.60411e+05_rb,0.58225e+05_rb /)
      kbo(:,58,16) = (/ &
        & 0.67327e+05_rb,0.65315e+05_rb,0.63232e+05_rb,0.61078e+05_rb,0.58891e+05_rb /)
      kbo(:,59,16) = (/ &
        & 0.67577e+05_rb,0.65572e+05_rb,0.63496e+05_rb,0.61349e+05_rb,0.59161e+05_rb /)

!     -----------------------------------------------------------------

      forrefo(:, 1) = (/ 0.299818e-05_rb, 0.209282e-05_rb, 0.988353e-04_rb, 0.632178e-03_rb /)
      forrefo(:, 2) = (/ 0.633648e-05_rb, 0.509214e-04_rb, 0.650535e-03_rb, 0.264019e-02_rb /)
      forrefo(:, 3) = (/ 0.636782e-04_rb, 0.136577e-03_rb, 0.166500e-02_rb, 0.750821e-02_rb /)
      forrefo(:, 4) = (/ 0.472314e-03_rb, 0.988296e-03_rb, 0.585751e-02_rb, 0.187352e-01_rb /)
      forrefo(:, 5) = (/ 0.558635e-02_rb, 0.856489e-02_rb, 0.157438e-01_rb, 0.181471e-01_rb /)
      forrefo(:, 6) = (/ 0.217395e-01_rb, 0.229156e-01_rb, 0.230125e-01_rb, 0.143821e-01_rb /)
      forrefo(:, 7) = (/ 0.277222e-01_rb, 0.299252e-01_rb, 0.208929e-01_rb, 0.826748e-02_rb /)
      forrefo(:, 8) = (/ 0.252119e-01_rb, 0.262911e-01_rb, 0.187663e-01_rb, 0.417110e-02_rb /)
      forrefo(:, 9) = (/ 0.304941e-01_rb, 0.175545e-01_rb, 0.971224e-02_rb, 0.142023e-02_rb /)
      forrefo(:,10) = (/ 0.327200e-01_rb, 0.215788e-01_rb, 0.346831e-02_rb, 0.157989e-02_rb /)
      forrefo(:,11) = (/ 0.324955e-01_rb, 0.228571e-01_rb, 0.171749e-02_rb, 0.226853e-02_rb /)
      forrefo(:,12) = (/ 0.326588e-01_rb, 0.198544e-01_rb, 0.532339e-06_rb, 0.279086e-02_rb /)
      forrefo(:,13) = (/ 0.345157e-01_rb, 0.168679e-01_rb, 0.505361e-06_rb, 0.276647e-02_rb /)
      forrefo(:,14) = (/ 0.448765e-01_rb, 0.123791e-02_rb, 0.488367e-06_rb, 0.122245e-02_rb /)
      forrefo(:,15) = (/ 0.486925e-01_rb, 0.464371e-06_rb, 0.464241e-06_rb, 0.753846e-06_rb /)
      forrefo(:,16) = (/ 0.530511e-01_rb, 0.376234e-06_rb, 0.409824e-06_rb, 0.470650e-06_rb /)

!     -----------------------------------------------------------------
!     The array SELFREFO contains the coefficient of the water vapor
!     self-continuum (including the energy term).  The first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     JT = 1 refers to a temperature of 245.6, JT = 2 refers to 252.8,
!     etc.  The second index runs over the g-channel (1 to 16).

      selfrefo(:, 1) = (/ &
        &  0.118069e+00_rb, 0.713523e-01_rb, 0.431199e-01_rb, 0.260584e-01_rb, 0.157477e-01_rb, &
        &  0.951675e-02_rb, 0.575121e-02_rb, 0.347560e-02_rb, 0.210039e-02_rb, 0.126932e-02_rb /)
      selfrefo(:, 2) = (/ &
        &  0.137081e-01_rb, 0.139046e-01_rb, 0.141040e-01_rb, 0.143061e-01_rb, 0.145112e-01_rb, &
        &  0.147193e-01_rb, 0.149303e-01_rb, 0.151443e-01_rb, 0.153614e-01_rb, 0.155816e-01_rb /)
      selfrefo(:, 3) = (/ &
        &  0.166575e-01_rb, 0.164916e-01_rb, 0.163273e-01_rb, 0.161647e-01_rb, 0.160037e-01_rb, &
        &  0.158443e-01_rb, 0.156864e-01_rb, 0.155302e-01_rb, 0.153755e-01_rb, 0.152224e-01_rb /)
      selfrefo(:, 4) = (/ &
        &  0.597379e-01_rb, 0.509517e-01_rb, 0.434579e-01_rb, 0.370662e-01_rb, 0.316145e-01_rb, &
        &  0.269647e-01_rb, 0.229988e-01_rb, 0.196162e-01_rb, 0.167311e-01_rb, 0.142703e-01_rb /)
      selfrefo(:, 5) = (/ &
        &  0.227517e+00_rb, 0.198401e+00_rb, 0.173011e+00_rb, 0.150870e+00_rb, 0.131563e+00_rb, &
        &  0.114726e+00_rb, 0.100044e+00_rb, 0.872415e-01_rb, 0.760769e-01_rb, 0.663411e-01_rb /)
      selfrefo(:, 6) = (/ &
        &  0.453235e+00_rb, 0.414848e+00_rb, 0.379712e+00_rb, 0.347552e+00_rb, 0.318116e+00_rb, &
        &  0.291173e+00_rb, 0.266512e+00_rb, 0.243940e+00_rb, 0.223279e+00_rb, 0.204368e+00_rb /)
      selfrefo(:, 7) = (/ &
        &  0.569263e+00_rb, 0.516415e+00_rb, 0.468473e+00_rb, 0.424982e+00_rb, 0.385528e+00_rb, &
        &  0.349737e+00_rb, 0.317269e+00_rb, 0.287815e+00_rb, 0.261095e+00_rb, 0.236856e+00_rb /)
      selfrefo(:, 8) = (/ &
        &  0.490314e+00_rb, 0.448042e+00_rb, 0.409413e+00_rb, 0.374116e+00_rb, 0.341861e+00_rb, &
        &  0.312387e+00_rb, 0.285455e+00_rb, 0.260844e+00_rb, 0.238355e+00_rb, 0.217805e+00_rb /)
      selfrefo(:, 9) = (/ &
        &  0.258162e+00_rb, 0.265085e+00_rb, 0.272193e+00_rb, 0.279493e+00_rb, 0.286988e+00_rb, &
        &  0.294684e+00_rb, 0.302586e+00_rb, 0.310701e+00_rb, 0.319033e+00_rb, 0.327588e+00_rb /)
      selfrefo(:,10) = (/ &
        &  0.332019e+00_rb, 0.331902e+00_rb, 0.331784e+00_rb, 0.331666e+00_rb, 0.331549e+00_rb, &
        &  0.331431e+00_rb, 0.331314e+00_rb, 0.331197e+00_rb, 0.331079e+00_rb, 0.330962e+00_rb /)
      selfrefo(:,11) = (/ &
        &  0.357523e+00_rb, 0.353154e+00_rb, 0.348839e+00_rb, 0.344576e+00_rb, 0.340366e+00_rb, &
        &  0.336207e+00_rb, 0.332099e+00_rb, 0.328041e+00_rb, 0.324032e+00_rb, 0.320073e+00_rb /)
      selfrefo(:,12) = (/ &
        &  0.294662e+00_rb, 0.299043e+00_rb, 0.303488e+00_rb, 0.308000e+00_rb, 0.312579e+00_rb, &
        &  0.317226e+00_rb, 0.321941e+00_rb, 0.326727e+00_rb, 0.331585e+00_rb, 0.336514e+00_rb /)
      selfrefo(:,13) = (/ &
        &  0.227445e+00_rb, 0.241545e+00_rb, 0.256519e+00_rb, 0.272422e+00_rb, 0.289311e+00_rb, &
        &  0.307247e+00_rb, 0.326294e+00_rb, 0.346523e+00_rb, 0.368005e+00_rb, 0.390820e+00_rb /)
      selfrefo(:,14) = (/ &
        &  0.616203e-02_rb, 0.113523e-01_rb, 0.209144e-01_rb, 0.385307e-01_rb, 0.709852e-01_rb, &
        &  0.130776e+00_rb, 0.240929e+00_rb, 0.443865e+00_rb, 0.817733e+00_rb, 0.150651e+01_rb /)
      selfrefo(:,15) = (/ &
        &  0.279552e-03_rb, 0.808472e-03_rb, 0.233812e-02_rb, 0.676192e-02_rb, 0.195557e-01_rb, &
        &  0.565555e-01_rb, 0.163560e+00_rb, 0.473020e+00_rb, 0.136799e+01_rb, 0.395626e+01_rb /)
      selfrefo(:,16) = (/ &
        &  0.261006e-03_rb, 0.771043e-03_rb, 0.227776e-02_rb, 0.672879e-02_rb, 0.198777e-01_rb, &
        &  0.587212e-01_rb, 0.173470e+00_rb, 0.512452e+00_rb, 0.151385e+01_rb, 0.447209e+01_rb /)
     
      end subroutine sw_kgb29

